def hurricanewavecontoury88(xgrid, ygrid, xCenter, yCenter, Hmax, Vmax, RVmax, Vt=0, VtAzmdir=0, distCalcMethod='gc', dispout='no'):
    """
    .. ++++++++++++++++++++++++++++++++YA LATIF++++++++++++++++++++++++++++++++++
    .. +                                                                        +
    .. + ScientiMate                                                            +
    .. + Earth-Science Data Analysis Library                                    +
    .. +                                                                        +
    .. + Developed by: Arash Karimpour                                          +
    .. + Contact     : www.arashkarimpour.com                                   +
    .. + Developed/Updated (yyyy-mm-dd): 2017-11-01                             +
    .. +                                                                        +
    .. ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    scientimate.hurricanewavecontoury88
    ===================================

    .. code:: python

        Hgrid = scientimate.hurricanewavecontoury88(xgrid, ygrid, xCenter, yCenter, Hmax, Vmax, RVmax, Vt=0, VtAzmdir=0, distCalcMethod='gc', dispout='no')

    Description
    -----------

    | Calculate hurricane wave height field (contours) on given mesh using method from Young (1988)
    | For method from U.S. Army Corps of Engineers use hurricanewavecontourhcem
    | For method from Hwang (2016) and Hwang & Walsh (2016) use hurricanewavecontourh16

    Inputs
    ------

    xgrid
        | x (longitude) of points which outputs are calculated at
        | xgrid can be a single point or 1d or 2d array 
    ygrid
        | y (latitude) of points which outputs are calculated at
        | ygrid can be a single point or 1d or 2d array 
    xCenter
        x (longitude) of hurricane center (track)
    yCenter
        y (latitude) of hurricane center (track)
    Hmax
        Hurricane maximum wave height
    Vmax
        | Maximum hurricane 1-min wind velocity at the surface level (at 10-m height) in (m/s)
        | Available Vmax are Vmax=15,20,25,30,40,50,60 (m/s)
        | If Vmax is not equal to above values but is within 20-40 (m/s) range, wave distribution is interpolated 
        | If Vmax is not equal to above values and is outside 20-40 (m/s) range, wave distribution is set equal to nearest one 
        | Vmax=15 (m/s) is only avilable for Vt=5 (m/s)
    RVmax
        Distance (radius) from hurricane center to a location of maximum hurricane wind velocity (m)
    Vt=0
        Hurricane central translational velocity in (m/s)
    VtAzmdir=0
        | Hurricane center velocity azimuth (bearing) direction in (Degree)
        | azimuth (bearing) direction which is measured clockwise from the north:
        | 0 (degree): toward North, 90 (degree): toward East, 180 (degree): toward South, 270 (degree): toward West 
        | Available Vt are Vt=0,2.5,5,7.5,10 (m/s)
        | If Vt is not equal to above values but is within 0-10 (m/s) range, wave distribution is interpolated 
        | If Vt is not equal to above values and is outside 0-10 (m/s) range, wave distribution is set equal to nearest one 
    distCalcMethod='gc'
        | Distance calculation method 
        | 'cart': Distances are calculated on cartesian coordinate
        | 'gc': Distances are calculated on Great Circle based on Vincenty formula, Vincenty (1975)
        | Earth radius coonsidered as mean earth radius=6371000 m
    dispout='no'
        | Define to display outputs or not
        | 'imagesc': 2 dimensional plot using imagesc or imshow
        | 'pcolor': 2 dimensional plot using pcolor
        | 'contour': 2 dimensional contour plot, number of contour=ncolor
        | 'no': not display 
        | Use dispout='no' if calculation mesh is not 2d array
        | if there is more than one time step, only the last one is plotted

    Outputs
    -------

    Hgrid
        Hurricane wave height on grid mesh

    Examples
    --------

    .. code:: python

        import scientimate as sm
        import numpy as np


        #EXAMPLE 1

        #Creating calculation mesh
        xgrid,ygrid=np.meshgrid(np.linspace(-98,-68,100),np.linspace(16,44,100))

        #Longitude of Hurricane Katrine center at max velocity
        longCenter=-88.6

        #Latitude of Hurricane Katrine center at max velocity
        latCenter=26.3

        #Hurricane Katrina maximum significant wave height (m) at max velocity
        Hmax=24.9821

        #Hurricane Katrina translational velocity (m/s) at max velocity
        Vt=5.18467

        #Hurricane Katrina velocity azimuth (bearing) in (Degree) at max velocity
        VtAzmdir=306.76219

        #Hurricane Katrina 1-min sustained maximum velocity (m/s) at max velocity
        Vmax=76.5
        Vmax=Vmax-Vt #Removing hurricane translation velocity from Vgmax

        #Hurricane Katrina radius from hurricane center to a location of maximum hurricane wind velocity (m) at max velocity
        RVmax=6.2750e+004

        Hgrid=sm.hurricanewavecontoury88(xgrid,ygrid,longCenter,latCenter,Hmax,Vmax,RVmax,Vt,VtAzmdir,'gc','contour')


        #EXAMPLE 2

        #Creating calculation mesh
        xgrid,ygrid=np.meshgrid(np.linspace(-98,-68,100),np.linspace(16,44,100))

        #Longitude of Hurricane Katrine best track
        longtrack=[-75.1,-75.7,-76.2,-76.5,-76.9,-77.7,-78.4,-79.0,-79.6,-80.1,-80.3,-81.3,\
            -82.0,-82.6,-83.3,-84.0,-84.7,-85.3,-85.9,-86.7,-87.7,-88.6,-89.2,-89.6,\
            -89.6,-89.6,-89.6,-89.6,-89.1,-88.6,-88.0,-87.0,-85.3,-82.9]

        #Latitude of Hurricane Katrine best track
        lattrack=[23.1,23.4,23.8,24.5,25.4,26.0,26.1,26.2,26.2,26.0,25.9,25.4,\
            25.1,24.9,24.6,24.4,24.4,24.5,24.8,25.2,25.7,26.3,27.2,28.2,\
            29.3,29.5,30.2,31.1,32.6,34.1,35.6,37.0,38.6,40.1]

        #Hurricane Katrina maximum significant wave height
        Hmax=[0,0,0,4.3788,4.9295,5.5527,6.2110,6.8516,7.5428,9.1513,8.5021,8.6332,10.1511,11.3434,\
            12.3171,13.5606,14.1226,14.4931,14.1972,19.9683,24.0121,24.9821,23.0419,19.9342,16.5366,\
            14.5246,14.8050,0,0,0,0,0,0,0]

        #Hurricane Katrina translational velocity (m/s)
        Vt=np.array([0.00000,3.23091,3.13105,3.86928,4.99513,4.82816,3.27813,2.81998,2.77140,2.53041,\
            1.05928,5.30662,3.60661,2.98269,3.61863,3.43691,3.28168,2.85849,3.20404,4.26279,\
            5.31340,5.18467,5.39195,5.46121,5.66270,1.02958,3.60354,4.63312,8.02540,8.01558,\
            8.12721,8.31580,10.75406,12.28350])
            
        #Hurricane Katrina velocity azimuth (bearing) in (Degree)
        VtAzmdir=[0.00000,298.67291,311.22135,338.70264,338.13626,309.94476,279.18860,280.65053,270.13245,\
            246.10095,240.96690,241.20181,244.79591,249.93382,244.88325,252.71384,270.14459,280.49918,\
            298.94148,299.05364,299.18896,306.76219,329.36839,340.59069,0.00000,0.00000,0.00000,\
            0.00000,15.67775,15.42254,18.00215,29.63266,39.49673,50.29744]

        #Hurricane Katrina 1-min sustained maximum velocity (m/s)
        Vmax=np.array([15.3,15.3,15.3,17.850,20.4,22.950,25.5,28.050,30.6,35.7,35.7,33.150,\
            38.250,43.350,45.9,48.450,51.0,51.0,51.0,63.750,73.950,76.5,71.4,63.750,\
            56.1,56.1,53.550,40.8,25.5,20.4,15.3,15.3,15.3,12.750])

        Vmax=Vmax-Vt #Removing hurricane translation velocity from Vmax

        #Hurricane Katrina radius from hurricane center to a location of maximum hurricane wind velocity (m)
        RVmax=[0,0,0,8.0290e+004,5.6029e+004,4.2063e+004,3.6769e+004,3.3849e+004,3.1352e+004,3.3405e+004,3.3773e+004,\
            3.2657e+004,3.1122e+004,2.7037e+004,2.6512e+004,3.3476e+004,3.0881e+004,4.0266e+004,3.2433e+004,\
            5.1747e+004,5.7297e+004,6.2750e+004,5.3376e+004,4.3074e+004,3.1790e+004,4.3114e+004,2.7800e+004,\
            0,0,0,0,0,0,0]

        Hgrid=sm.hurricanewavecontoury88(xgrid,ygrid,longtrack[3:27],lattrack[3:27],Hmax[3:27],Vmax[3:27],RVmax[3:27],Vt[3:27],VtAzmdir[3:27],'gc','contour')


        #EXAMPLE 3
        xgrid,ygrid=np.meshgrid(np.linspace(-4.5,7.5,121),np.linspace(-5,6,111))
        longCenter=0
        latCenter=0
        Hmax=1
        Vt=10
        VtAzmdir=0
        Vmax=40
        RVmax=1
        Hgrid=sm.hurricanewavecontoury88(xgrid,ygrid,longCenter,latCenter,Hmax,Vmax,RVmax,Vt,VtAzmdir,'cart','contour')

    References
    ----------

    Data

    * www.nhc.noaa.gov/data/
    * www.nhc.noaa.gov/data/hurdat/hurdat2-format-nencpac.pdf
    * coast.noaa.gov/hurricanes
    * www.aoml.noaa.gov/hrd/data_sub/re_anal.html

    Young, I. R. (1988). 
    Parametric hurricane wave prediction model. 
    Journal of Waterway, Port, Coastal, and Ocean Engineering, 114(5), 637-652.

    Young, I.R. (2017)
    A Review of Parametric Descriptions of Tropical Cyclone Wind-Wave Generation.
    Atmosphere 2017, 8, 194.

    .. License & Disclaimer
    .. --------------------
    ..
    .. Copyright (c) 2020 Arash Karimpour
    ..
    .. http://www.arashkarimpour.com
    ..
    .. THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    .. IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    .. FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    .. AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    .. LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    .. OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    .. SOFTWARE.
    """

    #--------------------------------------------------------------------------
    #CODE
    #--------------------------------------------------------------------------
    #Import required packages

    import numpy as np
    import scipy as sp
    from scipy import interpolate
    if dispout!='no':
        import matplotlib.pyplot as plt 

    #--------------------------------------------------------------------------
    #Convert inputs to numpy array

    #Changing type to numpy array
    def type2numpy(variable):
        if type(variable) is not str:
            if np.size(variable)==1:
                if ((type(variable) is list) or (type(variable) is np.ndarray)):
                    variable=np.array(variable)
                else:
                    variable=np.array([variable])
            elif np.size(variable)>1:
                if (type(variable).__module__)!='numpy':
                    variable=np.array(variable) 
        return variable
    
    xgrid=type2numpy(xgrid)
    ygrid=type2numpy(ygrid)
    xCenter=type2numpy(xCenter)
    yCenter=type2numpy(yCenter)
    Hmax=type2numpy(Hmax)
    Vmax=type2numpy(Vmax)
    RVmax=type2numpy(RVmax)
    Vt=type2numpy(Vt)
    VtAzmdir=type2numpy(VtAzmdir)

    #--------------------------------------------------------------------------
    #Defining required functions

    def cart2pol(x,y):
        rho=np.sqrt(x**2+y**2)
        theta=np.arctan2(y,x)
        return theta,rho
    
    def pol2cart(theta,rho):
        x=rho*np.cos(theta)
        y=rho*np.sin(theta)
        return x,y

    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young

    #x grid values in form of x/RVmax ratio and y grid values in form of y/RVmax ratio
    #x grid has a range of -4.5<=(x/RVmax)<=7.5 with 0.1 interval, i.e. x/RVmax=[-4.5:0.1:7.5]
    #y grid has a range of -5.0<=(y/RVmax)<=6.0 with 0.1 interval, i.e. y/RVmax=[-5.0:0.1:6.0]
    #x grid and y grid have a zise of [111*121]
    xgrid_x_RVmax,ygrid_y_RVmax=np.meshgrid(np.linspace(-4.5,7.5,121),np.linspace(-5,6,111))

    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=15 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax15vt5
    zgrid_H_Hmax_vmax15vt5=[[0.3947166, 0.3943584, 0.3933525, 0.3922124, 0.3912047, 0.3904841, 0.3901129, 0.3900488, 0.3901378,\
        0.3901299, 0.3897612, 0.3888962, 0.3875914, 0.3859950, 0.3842284, 0.3823544, 0.3803961, 0.3783603,\
        0.3762430, 0.3740275, 0.3716911, 0.3692184, 0.3666105, 0.3638835, 0.3610626, 0.3581774, 0.3552593,\
        0.3523415, 0.3494590, 0.3466493, 0.3439520, 0.3414080, 0.3390573, 0.3369376, 0.3350818, 0.3335178,\
        0.3322674, 0.3313463, 0.3307649, 0.3305289, 0.3306398, 0.3310959, 0.3318927, 0.3330238, 0.3344810,\
        0.3362544, 0.3383333, 0.3407055, 0.3433577, 0.3462757, 0.3494442, 0.3528469, 0.3564675, 0.3602907,\
        0.3643047, 0.3685027, 0.3728800, 0.3774214, 0.3820806, 0.3867554, 0.3912851, 0.3956702, 0.4002921,\
        0.4050688, 0.4098092, 0.4144091, 0.4187868, 0.4228694, 0.4265920, 0.4299011, 0.4327614, 0.4351627,\
        0.4371233, 0.4386856, 0.4399056, 0.4408391, 0.4415313, 0.4420134, 0.4423035, 0.4424125, 0.4423487,\
        0.4421215, 0.4417429, 0.4412268, 0.4405875, 0.4398381, 0.4389890, 0.4380480, 0.4370199, 0.4359078,\
        0.4347130, 0.4334358, 0.4320757, 0.4306312, 0.4291001, 0.4274792, 0.4257639, 0.4239487, 0.4220266,\
        0.4199890, 0.4178263, 0.4155270, 0.4130787, 0.4104676, 0.4076785, 0.4046958, 0.4015044, 0.3981020,\
        0.3945260, 0.3904196, 0.3857070, 0.3806249, 0.3752452, 0.3695894, 0.3636582, 0.3574353, 0.3508994,\
        0.3440343, 0.3368327, 0.3292943, 0.3214221],\
        [0.3987386, 0.3977186, 0.3962991, 0.3949021, 0.3937472, 0.3930045, 0.3927606, 0.3929725, 0.3934606,\
        0.3938755, 0.3937871, 0.3929734, 0.3916091, 0.3899676, 0.3882045, 0.3863651, 0.3844497, 0.3824612,\
        0.3803983, 0.3782334, 0.3759212, 0.3734331, 0.3707720, 0.3679628, 0.3650384, 0.3620337, 0.3589833,\
        0.3559221, 0.3528873, 0.3499197, 0.3470641, 0.3443673, 0.3418758, 0.3396326, 0.3376752, 0.3360338,\
        0.3347315, 0.3337838, 0.3332004, 0.3329853, 0.3331385, 0.3336566, 0.3345336, 0.3357618, 0.3373318,\
        0.3392330, 0.3414538, 0.3439816, 0.3468023, 0.3499012, 0.3532619, 0.3568667, 0.3606966, 0.3647327,\
        0.3689605, 0.3733766, 0.3779915, 0.3828132, 0.3878153, 0.3929066, 0.3978486, 0.4024117, 0.4072236,\
        0.4122110, 0.4171650, 0.4219738, 0.4265480, 0.4308052, 0.4346672, 0.4380654, 0.4409515, 0.4433106,\
        0.4451693, 0.4465909, 0.4476568, 0.4484434, 0.4490063, 0.4493763, 0.4495657, 0.4495777, 0.4494150,\
        0.4490851, 0.4486011, 0.4479800, 0.4472396, 0.4463956, 0.4454599, 0.4444401, 0.4433409, 0.4421644,\
        0.4409113, 0.4395819, 0.4381753, 0.4366906, 0.4351256, 0.4334774, 0.4317421, 0.4299141, 0.4279865,\
        0.4259509, 0.4237974, 0.4215144, 0.4190896, 0.4165090, 0.4137586, 0.4108244, 0.4076957, 0.4043813,\
        0.4009815, 0.3967432, 0.3919665, 0.3869108, 0.3815806, 0.3759977, 0.3701594, 0.3640324, 0.3575805,\
        0.3507831, 0.3436360, 0.3361440, 0.3283148],\
        [0.4013466, 0.4001609, 0.3986350, 0.3971339, 0.3958937, 0.3951546, 0.3950666, 0.3955603, 0.3964920,\
        0.3974236, 0.3976907, 0.3968634, 0.3954233, 0.3937748, 0.3920820, 0.3903306, 0.3884815, 0.3865532,\
        0.3845733, 0.3824984, 0.3802340, 0.3777316, 0.3750084, 0.3721070, 0.3690708, 0.3659397, 0.3627501,\
        0.3595370, 0.3563383, 0.3531980, 0.3501671, 0.3473006, 0.3446532, 0.3422751, 0.3402083, 0.3384859,\
        0.3371312, 0.3361594, 0.3355781, 0.3353893, 0.3355908, 0.3361771, 0.3371405, 0.3384718, 0.3401607,\
        0.3421958, 0.3445650, 0.3472551, 0.3502520, 0.3535405, 0.3571039, 0.3609233, 0.3649771, 0.3692402,\
        0.3736888, 0.3783135, 0.3831399, 0.3882146, 0.3935256, 0.3989839, 0.4043357, 0.4093682, 0.4144249,\
        0.4195855, 0.4247242, 0.4297246, 0.4344886, 0.4389224, 0.4429310, 0.4464233, 0.4493279, 0.4516176,\
        0.4533290, 0.4545575, 0.4554251, 0.4560375, 0.4564604, 0.4567182, 0.4568095, 0.4567246, 0.4564581,\
        0.4560155, 0.4554125, 0.4546720, 0.4538172, 0.4528676, 0.4518360, 0.4507297, 0.4495516, 0.4483027,\
        0.4469830, 0.4455921, 0.4441295, 0.4425944, 0.4409852, 0.4392996, 0.4375338, 0.4356827, 0.4337395,\
        0.4316955, 0.4295404, 0.4272626, 0.4248490, 0.4222856, 0.4195583, 0.4166533, 0.4135597, 0.4102647,\
        0.4066566, 0.4024212, 0.3977635, 0.3927996, 0.3875403, 0.3820605, 0.3763528, 0.3703475, 0.3639887,\
        0.3572571, 0.3501585, 0.3427082, 0.3349200],\
        [0.4032402, 0.4023064, 0.4008105, 0.3993034, 0.3979662, 0.3972159, 0.3973396, 0.3980078, 0.3992751,\
        0.4006097, 0.4011505, 0.4004068, 0.3990586, 0.3974882, 0.3959730, 0.3943566, 0.3925661, 0.3906751,\
        0.3888060, 0.3868859, 0.3846891, 0.3821533, 0.3793477, 0.3763410, 0.3731843, 0.3699210, 0.3665874,\
        0.3632150, 0.3598397, 0.3565084, 0.3532809, 0.3502237, 0.3474027, 0.3448765, 0.3426927, 0.3408864,\
        0.3394807, 0.3384885, 0.3379149, 0.3377589, 0.3380153, 0.3386761, 0.3397318, 0.3411718, 0.3429848,\
        0.3451589, 0.3476817, 0.3505398, 0.3537193, 0.3572051, 0.3609810, 0.3650288, 0.3693256, 0.3738412,\
        0.3785357, 0.3833722, 0.3883746, 0.3936704, 0.3992836, 0.4050345, 0.4107399, 0.4162574, 0.4216835,\
        0.4270870, 0.4324258, 0.4376173, 0.4425720, 0.4471917, 0.4513649, 0.4549702, 0.4578973, 0.4600910,\
        0.4615971, 0.4625647, 0.4631822, 0.4635975, 0.4638814, 0.4640394, 0.4640448, 0.4638680, 0.4634935,\
        0.4629257, 0.4621870, 0.4613100, 0.4603267, 0.4592607, 0.4581255, 0.4569261, 0.4556626, 0.4543338,\
        0.4529387, 0.4514769, 0.4499481, 0.4483520, 0.4466878, 0.4449537, 0.4431467, 0.4412619, 0.4392924,\
        0.4372293, 0.4350617, 0.4327770, 0.4303613, 0.4277993, 0.4250745, 0.4221685, 0.4190569, 0.4156925,\
        0.4119726, 0.4077997, 0.4032370, 0.3983505, 0.3931392, 0.3878058, 0.3822770, 0.3764125, 0.3701409,\
        0.3634591, 0.3563952, 0.3489790, 0.3412302],\
        [0.4051597, 0.4044841, 0.4032348, 0.4018010, 0.4004185, 0.3996511, 0.4001154, 0.4009194, 0.4022052,\
        0.4035675, 0.4041848, 0.4038229, 0.4027619, 0.4014059, 0.4000823, 0.3985657, 0.3967968, 0.3948503,\
        0.3931260, 0.3914938, 0.3893444, 0.3867159, 0.3838067, 0.3806860, 0.3774017, 0.3740033, 0.3705252,\
        0.3669889, 0.3634226, 0.3598770, 0.3564252, 0.3531509, 0.3501353, 0.3474473, 0.3451398, 0.3432489,\
        0.3417955, 0.3407892, 0.3402306, 0.3401149, 0.3404331, 0.3411748, 0.3423282, 0.3438813, 0.3458222,\
        0.3481388, 0.3508188, 0.3538492, 0.3572160, 0.3609045, 0.3649002, 0.3691873, 0.3737467, 0.3785495,\
        0.3835470, 0.3886523, 0.3938164, 0.3992852, 0.4052292, 0.4112955, 0.4173216, 0.4232193, 0.4289930,\
        0.4346648, 0.4402175, 0.4456013, 0.4507471, 0.4555640, 0.4599293, 0.4636856, 0.4666610, 0.4687378,\
        0.4699618, 0.4705762, 0.4708863, 0.4710950, 0.4712615, 0.4713494, 0.4712905, 0.4710294, 0.4705409,\
        0.4698311, 0.4689343, 0.4679000, 0.4667733, 0.4655823, 0.4643383, 0.4630413, 0.4616869, 0.4602709,\
        0.4587915, 0.4572486, 0.4556427, 0.4539743, 0.4522435, 0.4504494, 0.4485898, 0.4466603, 0.4446540,\
        0.4425612, 0.4403701, 0.4380670, 0.4356361, 0.4330601, 0.4303193, 0.4273897, 0.4242383, 0.4208159,\
        0.4170578, 0.4129210, 0.4084160, 0.4035541, 0.3984396, 0.3933172, 0.3880039, 0.3822666, 0.3760456,\
        0.3693795, 0.3623300, 0.3549418, 0.3472353],\
        [0.4072818, 0.4069084, 0.4059990, 0.4048544, 0.4037922, 0.4033184, 0.4037433, 0.4046489, 0.4058305,\
        0.4069760, 0.4076069, 0.4075169, 0.4068142, 0.4057536, 0.4045093, 0.4030264, 0.4012439, 0.3991231,\
        0.3976022, 0.3964846, 0.3941690, 0.3913910, 0.3884005, 0.3851660, 0.3817448, 0.3782119, 0.3745983,\
        0.3708987, 0.3671237, 0.3633310, 0.3596176, 0.3560932, 0.3528592, 0.3499963, 0.3475612, 0.3455884,\
        0.3440944, 0.3430828, 0.3425485, 0.3424815, 0.3428690, 0.3436972, 0.3449524, 0.3466214, 0.3486921,\
        0.3511527, 0.3539915, 0.3571962, 0.3607530, 0.3646470, 0.3688648, 0.3733956, 0.3782272, 0.3833398,\
        0.3887049, 0.3942294, 0.3996840, 0.4053786, 0.4115289, 0.4178426, 0.4241420, 0.4303403, 0.4364039,\
        0.4423210, 0.4480740, 0.4536336, 0.4589566, 0.4639710, 0.4685541, 0.4725174, 0.4756044, 0.4775640,\
        0.4783987, 0.4785327, 0.4784852, 0.4785091, 0.4786095, 0.4786750, 0.4785773, 0.4782364, 0.4776232,\
        0.4767473, 0.4756616, 0.4744451, 0.4731618, 0.4718409, 0.4704869, 0.4690910, 0.4676411, 0.4661299,\
        0.4645560, 0.4629210, 0.4612264, 0.4594735, 0.4576635, 0.4557972, 0.4538734, 0.4518884, 0.4498349,\
        0.4477024, 0.4454778, 0.4431455, 0.4406882, 0.4380863, 0.4353173, 0.4323546, 0.4291648, 0.4257074,\
        0.4219408, 0.4178391, 0.4134019, 0.4086565, 0.4037108, 0.3987586, 0.3936020, 0.3879178, 0.3816772,\
        0.3749812, 0.3679313, 0.3605761, 0.3529247],\
        [0.4096782, 0.4096589, 0.4091722, 0.4084801, 0.4078889, 0.4077291, 0.4081507, 0.4090011, 0.4100495,\
        0.4110241, 0.4116332, 0.4117138, 0.4112815, 0.4104507, 0.4093099, 0.4078610, 0.4060804, 0.4040899,\
        0.4026999, 0.4015304, 0.3989208, 0.3961944, 0.3931889, 0.3898153, 0.3862322, 0.3825719, 0.3788512,\
        0.3749968, 0.3709849, 0.3668955, 0.3628694, 0.3590553, 0.3555789, 0.3525314, 0.3499700, 0.3479236,\
        0.3464006, 0.3453959, 0.3448971, 0.3448879, 0.3453513, 0.3462705, 0.3476298, 0.3494154, 0.3516150,\
        0.3542180, 0.3572145, 0.3605934, 0.3643415, 0.3684418, 0.3728789, 0.3776476, 0.3827486, 0.3881612,\
        0.3938634, 0.3998492, 0.4057861, 0.4118190, 0.4181605, 0.4246744, 0.4312036, 0.4376482, 0.4439511,\
        0.4500758, 0.4559941, 0.4616886, 0.4671476, 0.4723324, 0.4771328, 0.4813501, 0.4846590, 0.4865639,\
        0.4868541, 0.4863437, 0.4859452, 0.4858607, 0.4859718, 0.4860695, 0.4859439, 0.4855135, 0.4847604,\
        0.4836867, 0.4823691, 0.4809437, 0.4794982, 0.4780493, 0.4765893, 0.4750962, 0.4735460, 0.4719290,\
        0.4702483, 0.4685089, 0.4667133, 0.4648629, 0.4629603, 0.4610087, 0.4590089, 0.4569578, 0.4548475,\
        0.4526663, 0.4503992, 0.4480290, 0.4455362, 0.4428999, 0.4400962, 0.4370985, 0.4338770, 0.4304002,\
        0.4266411, 0.4225860, 0.4182439, 0.4136574, 0.4089144, 0.4040816, 0.3989785, 0.3932764, 0.3869429,\
        0.3801904, 0.3731554, 0.3658627, 0.3582939],\
        [0.4123534, 0.4127281, 0.4127040, 0.4125017, 0.4123637, 0.4125044, 0.4130169, 0.4138319, 0.4147814,\
        0.4156565, 0.4162561, 0.4164508, 0.4162102, 0.4155690, 0.4145691, 0.4132347, 0.4116136, 0.4098829,\
        0.4082936, 0.4065122, 0.4040787, 0.4013597, 0.3982365, 0.3946613, 0.3908711, 0.3871078, 0.3833532,\
        0.3793556, 0.3750491, 0.3705846, 0.3661793, 0.3620325, 0.3582941, 0.3550609, 0.3523838, 0.3502800,\
        0.3487451, 0.3477626, 0.3473112, 0.3473685, 0.3479131, 0.3489255, 0.3503888, 0.3522884, 0.3546128,\
        0.3573528, 0.3605016, 0.3640517, 0.3679919, 0.3723014, 0.3769534, 0.3819405, 0.3873026, 0.3930274,\
        0.3990119, 0.4053271, 0.4117788, 0.4182979, 0.4249617, 0.4317219, 0.4384771, 0.4451418, 0.4516532,\
        0.4579564, 0.4640015, 0.4697696, 0.4752912, 0.4805859, 0.4855498, 0.4899748, 0.4935800, 0.4956216,\
        0.4951758, 0.4939403, 0.4933675, 0.4932854, 0.4934478, 0.4936130, 0.4934138, 0.4928596, 0.4919564,\
        0.4906542, 0.4890433, 0.4873920, 0.4858002, 0.4842288, 0.4826709, 0.4810860, 0.4794267, 0.4776872,\
        0.4758843, 0.4740283, 0.4721195, 0.4701575, 0.4681473, 0.4660963, 0.4640084, 0.4618813, 0.4597059,\
        0.4574683, 0.4551513, 0.4527356, 0.4502005, 0.4475238, 0.4446816, 0.4416489, 0.4383998, 0.4349101,\
        0.4311618, 0.4271495, 0.4228860, 0.4184063, 0.4137562, 0.4089379, 0.4038265, 0.3981216, 0.3916762,\
        0.3849194, 0.3779737, 0.3708026, 0.3633545],\
        [0.4152722, 0.4160593, 0.4164999, 0.4167710, 0.4170541, 0.4174951, 0.4181586, 0.4190083, 0.4199312,\
        0.4207775, 0.4214009, 0.4216957, 0.4216124, 0.4211464, 0.4203143, 0.4191446, 0.4176949, 0.4160627,\
        0.4142951, 0.4122424, 0.4097524, 0.4068588, 0.4035151, 0.3997334, 0.3956638, 0.3918640, 0.3882381,\
        0.3840616, 0.3793346, 0.3743829, 0.3695246, 0.3650096, 0.3610041, 0.3575991, 0.3548297, 0.3526940,\
        0.3511696, 0.3502266, 0.3498339, 0.3499638, 0.3505916, 0.3516964, 0.3532601, 0.3552678, 0.3577086,\
        0.3605754, 0.3638653, 0.3675778, 0.3717092, 0.3762402, 0.3811190, 0.3862875, 0.3918402, 0.3979798,\
        0.4043189, 0.4109739, 0.4178521, 0.4248358, 0.4318829, 0.4389389, 0.4459332, 0.4528079, 0.4595172,\
        0.4659968, 0.4721541, 0.4779314, 0.4834101, 0.4887382, 0.4937954, 0.4982363, 0.5018560, 0.5038411,\
        0.5031715, 0.5018032, 0.5012109, 0.5011109, 0.5011938, 0.5013285, 0.5009465, 0.5002295, 0.4991629,\
        0.4976348, 0.4956480, 0.4938123, 0.4921238, 0.4904075, 0.4887663, 0.4871000, 0.4853094, 0.4834193,\
        0.4814771, 0.4794954, 0.4774629, 0.4753736, 0.4732380, 0.4710721, 0.4688846, 0.4666731, 0.4644258,\
        0.4621256, 0.4597524, 0.4572851, 0.4547017, 0.4519797, 0.4490958, 0.4460265, 0.4427493, 0.4392446,\
        0.4354990, 0.4315086, 0.4272807, 0.4228308, 0.4181661, 0.4132527, 0.4079797, 0.4021503, 0.3957486,\
        0.3891631, 0.3824254, 0.3754394, 0.3681416],\
        [0.4183816, 0.4195830, 0.4204730, 0.4211904, 0.4218701, 0.4226158, 0.4234750, 0.4244283, 0.4254008,\
        0.4262870, 0.4269782, 0.4273868, 0.4274579, 0.4271678, 0.4265157, 0.4255184, 0.4242098, 0.4226270,\
        0.4207656, 0.4185524, 0.4159087, 0.4127995, 0.4092009, 0.4051310, 0.4007149, 0.3970717, 0.3937434,\
        0.3891189, 0.3837782, 0.3782264, 0.3728596, 0.3679680, 0.3637162, 0.3601753, 0.3573529, 0.3552198,\
        0.3537318, 0.3528438, 0.3525170, 0.3527202, 0.3534282, 0.3546196, 0.3562762, 0.3583827, 0.3609280,\
        0.3639063, 0.3673186, 0.3711728, 0.3754819, 0.3802503, 0.3854255, 0.3908168, 0.3963274, 0.4027109,\
        0.4096175, 0.4167387, 0.4240637, 0.4314976, 0.4389476, 0.4463257, 0.4535614, 0.4606281, 0.4675294,\
        0.4742201, 0.4805428, 0.4863126, 0.4916211, 0.4968999, 0.5020488, 0.5064093, 0.5097321, 0.5114717,\
        0.5114136, 0.5105627, 0.5099040, 0.5095581, 0.5093680, 0.5091168, 0.5085277, 0.5075766, 0.5062565,\
        0.5044823, 0.5022838, 0.5003774, 0.4985456, 0.4966060, 0.4949300, 0.4931860, 0.4912072, 0.4891283,\
        0.4870363, 0.4849302, 0.4827664, 0.4805285, 0.4782446, 0.4759475, 0.4736508, 0.4713490, 0.4690252,\
        0.4666574, 0.4642225, 0.4616975, 0.4590602, 0.4562879, 0.4533580, 0.4502484, 0.4469385, 0.4434112,\
        0.4396538, 0.4356597, 0.4314270, 0.4269524, 0.4222180, 0.4171729, 0.4117229, 0.4057762, 0.3994572,\
        0.3931211, 0.3866517, 0.3798589, 0.3727057],\
        [0.4216243, 0.4232331, 0.4245529, 0.4256919, 0.4267514, 0.4278080, 0.4288982, 0.4300127, 0.4311020,\
        0.4320912, 0.4328981, 0.4334492, 0.4336897, 0.4335855, 0.4331213, 0.4322968, 0.4311200, 0.4295946,\
        0.4277010, 0.4253913, 0.4226132, 0.4193326, 0.4155465, 0.4113246, 0.4069729, 0.4033186, 0.3995935,\
        0.3941959, 0.3882208, 0.3820112, 0.3761362, 0.3709078, 0.3664660, 0.3628491, 0.3600273, 0.3579370,\
        0.3565089, 0.3556845, 0.3554210, 0.3556887, 0.3564655, 0.3577320, 0.3594700, 0.3616628, 0.3642982,\
        0.3673700, 0.3708801, 0.3748410, 0.3792819, 0.3842519, 0.3897938, 0.3957393, 0.4014645, 0.4078663,\
        0.4150745, 0.4226367, 0.4304197, 0.4383138, 0.4461902, 0.4539133, 0.4613789, 0.4685828, 0.4756345,\
        0.4825843, 0.4892133, 0.4951289, 0.5002796, 0.5054961, 0.5105514, 0.5148061, 0.5179204, 0.5196451,\
        0.5200300, 0.5196130, 0.5189864, 0.5183988, 0.5178344, 0.5171616, 0.5162337, 0.5149825, 0.5133812,\
        0.5114213, 0.5092382, 0.5071392, 0.5050617, 0.5030014, 0.5012227, 0.4993245, 0.4970931, 0.4948109,\
        0.4925757, 0.4903638, 0.4880585, 0.4856357, 0.4831748, 0.4807330, 0.4783222, 0.4759282, 0.4735254,\
        0.4710856, 0.4685827, 0.4659934, 0.4632956, 0.4604672, 0.4574858, 0.4543302, 0.4509807, 0.4474207,\
        0.4436368, 0.4396187, 0.4353566, 0.4308361, 0.4260310, 0.4208968, 0.4153805, 0.4094688, 0.4032848,\
        0.3971392, 0.3908197, 0.3841365, 0.3770850],\
        [0.4249458, 0.4269516, 0.4286826, 0.4302234, 0.4316514, 0.4330250, 0.4343739, 0.4356952, 0.4369565,\
        0.4381039, 0.4390737, 0.4398031, 0.4402380, 0.4403375, 0.4400743, 0.4394320, 0.4383994, 0.4369612,\
        0.4350902, 0.4327475, 0.4298950, 0.4265131, 0.4226253, 0.4183439, 0.4139261, 0.4095819, 0.4047896,\
        0.3991654, 0.3925143, 0.3856427, 0.3793642, 0.3738920, 0.3693441, 0.3657285, 0.3629683, 0.3609575,\
        0.3596014, 0.3588327, 0.3586125, 0.3589207, 0.3597440, 0.3610673, 0.3628714, 0.3651365, 0.3678472,\
        0.3709958, 0.3745819, 0.3786133, 0.3831153, 0.3881527, 0.3938377, 0.4002370, 0.4066381, 0.4133468,\
        0.4207663, 0.4286854, 0.4369096, 0.4452820, 0.4536304, 0.4617519, 0.4694513, 0.4766853, 0.4837372,\
        0.4909403, 0.4979434, 0.5041900, 0.5095887, 0.5147645, 0.5196069, 0.5236824, 0.5266703, 0.5284261,\
        0.5290426, 0.5288505, 0.5282318, 0.5274261, 0.5265024, 0.5254233, 0.5241200, 0.5225453, 0.5206853,\
        0.5185711, 0.5163097, 0.5140360, 0.5117893, 0.5096136, 0.5075413, 0.5053673, 0.5029754, 0.5005176,\
        0.4981351, 0.4958443, 0.4933630, 0.4906909, 0.4880301, 0.4854422, 0.4829200, 0.4804355, 0.4779522,\
        0.4754346, 0.4728553, 0.4701928, 0.4674265, 0.4645349, 0.4614956, 0.4582870, 0.4548897, 0.4512871,\
        0.4474649, 0.4434101, 0.4391090, 0.4345441, 0.4296921, 0.4245267, 0.4190336, 0.4132457, 0.4072778,\
        0.4012338, 0.3949136, 0.3882612, 0.3812774],\
        [0.4282961, 0.4306877, 0.4328139, 0.4347411, 0.4365296, 0.4382251, 0.4398533, 0.4414161, 0.4428930,\
        0.4442453, 0.4454221, 0.4463678, 0.4470282, 0.4473562, 0.4473147, 0.4468753, 0.4460138, 0.4447033,\
        0.4429109, 0.4405997, 0.4377379, 0.4343112, 0.4303420, 0.4259056, 0.4211097, 0.4159598, 0.4102495,\
        0.4037585, 0.3963272, 0.3892316, 0.3827329, 0.3770929, 0.3725202, 0.3689871, 0.3663431, 0.3644305,\
        0.3631322, 0.3623822, 0.3621581, 0.3624617, 0.3632958, 0.3646509, 0.3665038, 0.3688267, 0.3715989,\
        0.3748122, 0.3784660, 0.3825587, 0.3870918, 0.3921079, 0.3977596, 0.4042546, 0.4113052, 0.4186570,\
        0.4264963, 0.4348124, 0.4434830, 0.4523563, 0.4612421, 0.4698768, 0.4779145, 0.4850959, 0.4918096,\
        0.4991521, 0.5064755, 0.5130720, 0.5189600, 0.5243209, 0.5291135, 0.5330812, 0.5359871, 0.5377377,\
        0.5384223, 0.5382788, 0.5375867, 0.5365593, 0.5353047, 0.5338492, 0.5321812, 0.5302878, 0.5281759,\
        0.5258835, 0.5234830, 0.5210520, 0.5186385, 0.5162671, 0.5139160, 0.5114938, 0.5089535, 0.5063618,\
        0.5038242, 0.5013611, 0.4986086, 0.4956638, 0.4928249, 0.4901057, 0.4874773, 0.4849046, 0.4823372,\
        0.4797312, 0.4770617, 0.4743136, 0.4714695, 0.4685072, 0.4654025, 0.4621331, 0.4586794, 0.4550251,\
        0.4511558, 0.4470575, 0.4427158, 0.4381147, 0.4332380, 0.4280742, 0.4226283, 0.4169375, 0.4110659,\
        0.4050361, 0.3987701, 0.3921441, 0.3852518],\
        [0.4316299, 0.4343971, 0.4369051, 0.4392065, 0.4413487, 0.4433691, 0.4452906, 0.4471194, 0.4488447,\
        0.4504395, 0.4518628, 0.4530630, 0.4539836, 0.4545704, 0.4547777, 0.4545699, 0.4539151, 0.4527792,\
        0.4511235, 0.4489087, 0.4461009, 0.4426787, 0.4386389, 0.4339967, 0.4287621, 0.4228868, 0.4162480,\
        0.4087446, 0.4008619, 0.3935527, 0.3866460, 0.3807831, 0.3762542, 0.3728783, 0.3703758, 0.3685391,\
        0.3672400, 0.3664270, 0.3661134, 0.3663400, 0.3671353, 0.3684939, 0.3703801, 0.3727482, 0.3755668,\
        0.3788326, 0.3825581, 0.3867432, 0.3913669, 0.3964277, 0.4021710, 0.4088342, 0.4161550, 0.4239562,\
        0.4322295, 0.4409619, 0.4500755, 0.4594474, 0.4689087, 0.4781951, 0.4868476, 0.4942127, 0.5004944,\
        0.5076565, 0.5151526, 0.5220944, 0.5283851, 0.5340312, 0.5389416, 0.5429204, 0.5457873, 0.5474855,\
        0.5481133, 0.5478825, 0.5470343, 0.5457651, 0.5441983, 0.5423964, 0.5403873, 0.5381887, 0.5358241,\
        0.5333314, 0.5307612, 0.5281628, 0.5255689, 0.5229881, 0.5203983, 0.5177568, 0.5150452, 0.5122936,\
        0.5095427, 0.5067487, 0.5037464, 0.5006204, 0.4976419, 0.4947852, 0.4920418, 0.4893785, 0.4867173,\
        0.4840012, 0.4812192, 0.4783701, 0.4754387, 0.4723984, 0.4692207, 0.4658817, 0.4623623, 0.4586475,\
        0.4547237, 0.4505776, 0.4461957, 0.4415647, 0.4366738, 0.4315186, 0.4261072, 0.4204614, 0.4146053,\
        0.4085404, 0.4022408, 0.3956387, 0.3890327],\
        [0.4349060, 0.4380393, 0.4409182, 0.4435838, 0.4460739, 0.4484193, 0.4506415, 0.4527513, 0.4547469,\
        0.4566124, 0.4583159, 0.4598081, 0.4610263, 0.4619054, 0.4623916, 0.4624471, 0.4620396, 0.4611300,\
        0.4596716, 0.4576193, 0.4549341, 0.4515812, 0.4475261, 0.4427331, 0.4371614, 0.4307616, 0.4235045,\
        0.4155084, 0.4072401, 0.3991170, 0.3914301, 0.3852766, 0.3809274, 0.3777498, 0.3753375, 0.3734846,\
        0.3720626, 0.3710443, 0.3705058, 0.3705532, 0.3712494, 0.3725867, 0.3745002, 0.3769067, 0.3797517,\
        0.3830430, 0.3868343, 0.3911497, 0.3959488, 0.4012172, 0.4070911, 0.4138283, 0.4213081, 0.4293970,\
        0.4380250, 0.4471382, 0.4566564, 0.4664684, 0.4764350, 0.4863570, 0.4957563, 0.5036878, 0.5102556,\
        0.5171345, 0.5243577, 0.5313816, 0.5379372, 0.5438762, 0.5490111, 0.5531053, 0.5559707, 0.5575703,\
        0.5580395, 0.5576170, 0.5565479, 0.5550202, 0.5531560, 0.5510337, 0.5487078, 0.5462191, 0.5436038,\
        0.5408994, 0.5381434, 0.5353672, 0.5325880, 0.5298048, 0.5269997, 0.5241489, 0.5212414, 0.5182850,\
        0.5152838, 0.5122050, 0.5090088, 0.5057635, 0.5026005, 0.4995558, 0.4966690, 0.4939070, 0.4911292,\
        0.4882605, 0.4853346, 0.4823705, 0.4793459, 0.4762221, 0.4729632, 0.4695440, 0.4659480, 0.4621632,\
        0.4581780, 0.4539801, 0.4495570, 0.4448973, 0.4399932, 0.4348423, 0.4294495, 0.4238239, 0.4179682,\
        0.4118608, 0.4054473, 0.3990083, 0.3929196],\
        [0.4380860, 0.4415776, 0.4448183, 0.4478402, 0.4506723, 0.4533403, 0.4558646, 0.4582605, 0.4605363,\
        0.4626895, 0.4646995, 0.4665199, 0.4680764, 0.4692837, 0.4700764, 0.4704243, 0.4703065, 0.4696791,\
        0.4684814, 0.4666599, 0.4641742, 0.4609812, 0.4570187, 0.4522091, 0.4464833, 0.4398086, 0.4322156,\
        0.4238471, 0.4149844, 0.4059651, 0.3973024, 0.3911210, 0.3871638, 0.3840259, 0.3814941, 0.3794474,\
        0.3777089, 0.3762676, 0.3753120, 0.3750510, 0.3755869, 0.3768931, 0.3788493, 0.3813038, 0.3841512,\
        0.3874067, 0.3912136, 0.3956663, 0.4006406, 0.4060920, 0.4121861, 0.4190571, 0.4266916, 0.4350114,\
        0.4439394, 0.4533943, 0.4632726, 0.4734424, 0.4837433, 0.4940060, 0.5038435, 0.5122829, 0.5196134,\
        0.5266694, 0.5337734, 0.5407951, 0.5475344, 0.5537765, 0.5592294, 0.5635406, 0.5664367, 0.5678851,\
        0.5681015, 0.5674078, 0.5660842, 0.5643033, 0.5621621, 0.5597431, 0.5571230, 0.5543606, 0.5515011,\
        0.5485825, 0.5456350, 0.5426773, 0.5397146, 0.5367380, 0.5337294, 0.5306702, 0.5275502, 0.5243684,\
        0.5211240, 0.5178096, 0.5144320, 0.5110424, 0.5077137, 0.5044956, 0.5014305, 0.4985368, 0.4955872,\
        0.4924956, 0.4893999, 0.4863196, 0.4832040, 0.4799928, 0.4766416, 0.4731278, 0.4694417, 0.4655765,\
        0.4615227, 0.4572684, 0.4528011, 0.4481099, 0.4431880, 0.4380332, 0.4326488, 0.4270415, 0.4212197,\
        0.4152033, 0.4090905, 0.4031981, 0.3968725],\
        [0.4411336, 0.4449769, 0.4485728, 0.4519447, 0.4551140, 0.4581005, 0.4609228, 0.4636004, 0.4661529,\
        0.4685955, 0.4709266, 0.4731085, 0.4750508, 0.4766264, 0.4777438, 0.4784011, 0.4786159, 0.4783342,\
        0.4774621, 0.4759368, 0.4737311, 0.4708088, 0.4670862, 0.4624383, 0.4567657, 0.4500603, 0.4423928,\
        0.4338781, 0.4246891, 0.4151135, 0.4059361, 0.3999801, 0.3957093, 0.3920282, 0.3889913, 0.3865362,\
        0.3842202, 0.3820470, 0.3804321, 0.3797268, 0.3800560, 0.3813464, 0.3833925, 0.3859369, 0.3887800,\
        0.3919069, 0.3955936, 0.4001515, 0.4052954, 0.4109891, 0.4173378, 0.4244185, 0.4322501, 0.4408051,\
        0.4500277, 0.4598254, 0.4700647, 0.4805821, 0.4911724, 0.5016083, 0.5116739, 0.5206621, 0.5285874,\
        0.5359298, 0.5430537, 0.5500808, 0.5569735, 0.5635515, 0.5694420, 0.5741049, 0.5770691, 0.5782970,\
        0.5781687, 0.5771546, 0.5755941, 0.5736067, 0.5712183, 0.5685216, 0.5656304, 0.5626126, 0.5595182,\
        0.5563890, 0.5532508, 0.5501136, 0.5469725, 0.5438097, 0.5406027, 0.5373324, 0.5339892, 0.5305733,\
        0.5270909, 0.5235536, 0.5199858, 0.5164304, 0.5129390, 0.5095561, 0.5063137, 0.5031927, 0.4999935,\
        0.4966543, 0.4934096, 0.4902317, 0.4870331, 0.4837270, 0.4802646, 0.4766349, 0.4728419, 0.4688855,\
        0.4647568, 0.4604417, 0.4559257, 0.4511972, 0.4462492, 0.4410797, 0.4356916, 0.4300934, 0.4243023,\
        0.4183560, 0.4123235, 0.4061806, 0.3994006],\
        [0.4440139, 0.4482037, 0.4521500, 0.4558682, 0.4593718, 0.4626731, 0.4657867, 0.4687335, 0.4715443,\
        0.4742572, 0.4769023, 0.4794692, 0.4818577, 0.4838581, 0.4852979, 0.4862527, 0.4868477, 0.4869895,\
        0.4865054, 0.4853298, 0.4834767, 0.4809420, 0.4776300, 0.4733399, 0.4679107, 0.4613936, 0.4539451,\
        0.4456598, 0.4366743, 0.4273857, 0.4187390, 0.4119259, 0.4065062, 0.4017505, 0.3977974, 0.3947402,\
        0.3914861, 0.3881885, 0.3856836, 0.3844354, 0.3845425, 0.3858519, 0.3880559, 0.3907678, 0.3936501,\
        0.3966377, 0.4000231, 0.4046388, 0.4099943, 0.4159649, 0.4225926, 0.4299231, 0.4379875, 0.4468023,\
        0.4563533, 0.4665534, 0.4772262, 0.4881809, 0.4992232, 0.5100334, 0.5202262, 0.5294240, 0.5375964,\
        0.5450390, 0.5521018, 0.5590477, 0.5660120, 0.5729250, 0.5793833, 0.5846019, 0.5876883, 0.5886099,\
        0.5880834, 0.5867505, 0.5850505, 0.5829666, 0.5803474, 0.5773778, 0.5742519, 0.5709984, 0.5676776,\
        0.5643435, 0.5610178, 0.5577049, 0.5543910, 0.5510467, 0.5476410, 0.5441526, 0.5405739, 0.5369116,\
        0.5331830, 0.5294142, 0.5256383, 0.5218944, 0.5182207, 0.5146461, 0.5111759, 0.5077628, 0.5042981,\
        0.5007962, 0.4974130, 0.4941469, 0.4908627, 0.4874399, 0.4838320, 0.4800558, 0.4761382, 0.4720833,\
        0.4678766, 0.4634974, 0.4589275, 0.4541535, 0.4491681, 0.4439689, 0.4385580, 0.4329422, 0.4271328,\
        0.4211413, 0.4149516, 0.4084433, 0.4014689],\
        [0.4466922, 0.4512240, 0.4555179, 0.4595819, 0.4634205, 0.4670371, 0.4704372, 0.4736376, 0.4766758,\
        0.4796151, 0.4825304, 0.4854692, 0.4883693, 0.4909082, 0.4926409, 0.4938313, 0.4948714, 0.4955241,\
        0.4954744, 0.4946914, 0.4932536, 0.4912050, 0.4884739, 0.4847415, 0.4796790, 0.4735009, 0.4665813,\
        0.4589003, 0.4504841, 0.4417129, 0.4332551, 0.4256781, 0.4189843, 0.4129929, 0.4078560, 0.4035293,\
        0.3989898, 0.3943351, 0.3908922, 0.3890748, 0.3889705, 0.3903115, 0.3926914, 0.3956393, 0.3985681,\
        0.4015307, 0.4049627, 0.4094456, 0.4148968, 0.4211146, 0.4280338, 0.4356405, 0.4439566, 0.4530443,\
        0.4629699, 0.4736956, 0.4849659, 0.4964916, 0.5081465, 0.5193631, 0.5296719, 0.5388513, 0.5469442,\
        0.5542086, 0.5609880, 0.5676319, 0.5744517, 0.5815691, 0.5886092, 0.5945943, 0.5978963, 0.5985107,\
        0.5977366, 0.5961795, 0.5945043, 0.5925006, 0.5895572, 0.5863249, 0.5830604, 0.5795673, 0.5760240,\
        0.5724926, 0.5689770, 0.5654891, 0.5620065, 0.5584813, 0.5548698, 0.5511483, 0.5473139, 0.5433829,\
        0.5393867, 0.5353649, 0.5313580, 0.5274021, 0.5235238, 0.5197357, 0.5160283, 0.5123639, 0.5087004,\
        0.5050577, 0.5015144, 0.4981203, 0.4947132, 0.4911306, 0.4873195, 0.4833611, 0.4793102, 0.4751600,\
        0.4708786, 0.4664342, 0.4618042, 0.4569745, 0.4519378, 0.4466912, 0.4412349, 0.4355710, 0.4296994,\
        0.4236106, 0.4172683, 0.4106061, 0.4035918],\
        [0.4491332, 0.4540028, 0.4586427, 0.4630543, 0.4672341, 0.4711741, 0.4748657, 0.4783111, 0.4815444,\
        0.4846495, 0.4877501, 0.4909668, 0.4943480, 0.4976337, 0.4996627, 0.5010945, 0.5025991, 0.5037317,\
        0.5041603, 0.5038662, 0.5029353, 0.5014301, 0.4993528, 0.4963401, 0.4916456, 0.4858789, 0.4798695,\
        0.4730771, 0.4653335, 0.4570072, 0.4485546, 0.4403038, 0.4324158, 0.4250303, 0.4183117, 0.4121618,\
        0.4061467, 0.4004807, 0.3961645, 0.3937175, 0.3934132, 0.3947124, 0.3970611, 0.4001150, 0.4030194,\
        0.4060473, 0.4097101, 0.4143333, 0.4199608, 0.4264708, 0.4337378, 0.4416710, 0.4502521, 0.4595874,\
        0.4698861, 0.4812763, 0.4934911, 0.5058420, 0.5181557, 0.5297389, 0.5400826, 0.5490510, 0.5568040,\
        0.5636439, 0.5699066, 0.5759640, 0.5822816, 0.5892893, 0.5966897, 0.6029840, 0.6066776, 0.6077233,\
        0.6072112, 0.6058346, 0.6041859, 0.6021965, 0.5987357, 0.5954949, 0.5922198, 0.5883627, 0.5846410,\
        0.5809151, 0.5771827, 0.5735132, 0.5698643, 0.5661509, 0.5623142, 0.5583333, 0.5542115, 0.5499770,\
        0.5456794, 0.5413758, 0.5371144, 0.5329278, 0.5288316, 0.5248257, 0.5208953, 0.5170165, 0.5131749,\
        0.5093890, 0.5057022, 0.5021212, 0.4985504, 0.4947502, 0.4906517, 0.4864981, 0.4823356, 0.4781118,\
        0.4737656, 0.4692548, 0.4645561, 0.4596577, 0.4545539, 0.4492417, 0.4437194, 0.4379842, 0.4320293,\
        0.4258390, 0.4193846, 0.4126331, 0.4055683],\
        [0.4513007, 0.4565025, 0.4614863, 0.4662487, 0.4707797, 0.4750612, 0.4790670, 0.4827718, 0.4861858,\
        0.4894068, 0.4926141, 0.4959772, 0.4996295, 0.5033180, 0.5060865, 0.5081553, 0.5100237, 0.5115377,\
        0.5124683, 0.5127488, 0.5124102, 0.5114861, 0.5099155, 0.5073760, 0.5031766, 0.4981165, 0.4934542,\
        0.4875786, 0.4804184, 0.4724664, 0.4640020, 0.4551364, 0.4460660, 0.4371807, 0.4288166, 0.4210425,\
        0.4137870, 0.4072696, 0.4020376, 0.3987367, 0.3982127, 0.3992857, 0.4011768, 0.4038809, 0.4068402,\
        0.4101352, 0.4141298, 0.4190791, 0.4250611, 0.4320053, 0.4397591, 0.4481418, 0.4570349, 0.4665487,\
        0.4771049, 0.4891189, 0.5024536, 0.5160632, 0.5292125, 0.5412042, 0.5514791, 0.5600258, 0.5672349,\
        0.5735165, 0.5791669, 0.5844777, 0.5899336, 0.5964281, 0.6039020, 0.6103429, 0.6145657, 0.6164802,\
        0.6166647, 0.6157421, 0.6141252, 0.6117723, 0.6085201, 0.6053125, 0.6016449, 0.5974440, 0.5937367,\
        0.5897100, 0.5856935, 0.5818374, 0.5780217, 0.5740941, 0.5699920, 0.5657125, 0.5612597, 0.5566723,\
        0.5520286, 0.5474108, 0.5428760, 0.5384496, 0.5341333, 0.5299142, 0.5257727, 0.5216905, 0.5176603,\
        0.5136924, 0.5098052, 0.5059957, 0.5022074, 0.4980829, 0.4936885, 0.4894215, 0.4852171, 0.4809557,\
        0.4765525, 0.4719671, 0.4671852, 0.4622016, 0.4570138, 0.4516190, 0.4460139, 0.4401928, 0.4341466,\
        0.4278602, 0.4213136, 0.4144871, 0.4073674],\
        [0.4531578, 0.4586835, 0.4640058, 0.4691189, 0.4740098, 0.4786561, 0.4830191, 0.4870371, 0.4906600,\
        0.4939867, 0.4973040, 0.5008688, 0.5047423, 0.5086155, 0.5119517, 0.5147046, 0.5170563, 0.5189992,\
        0.5204254, 0.5212669, 0.5215038, 0.5211084, 0.5199699, 0.5178405, 0.5145484, 0.5106751, 0.5066788,\
        0.5014617, 0.4948882, 0.4873476, 0.4790147, 0.4696136, 0.4593217, 0.4489688, 0.4391873, 0.4301875,\
        0.4220174, 0.4148447, 0.4090373, 0.4051336, 0.4038048, 0.4041908, 0.4055025, 0.4076926, 0.4105193,\
        0.4139885, 0.4183001, 0.4236458, 0.4301109, 0.4376538, 0.4461083, 0.4551890, 0.4645678, 0.4742074,\
        0.4848144, 0.4974346, 0.5118799, 0.5268765, 0.5411618, 0.5537129, 0.5637917, 0.5716250, 0.5781340,\
        0.5838897, 0.5890693, 0.5938737, 0.5985676, 0.6046637, 0.6118638, 0.6182663, 0.6228523, 0.6254237,\
        0.6262602, 0.6257829, 0.6243134, 0.6219869, 0.6189570, 0.6155246, 0.6116171, 0.6074553, 0.6034165,\
        0.5989151, 0.5946099, 0.5905494, 0.5865518, 0.5823336, 0.5778936, 0.5732762, 0.5684393, 0.5634305,\
        0.5583852, 0.5534246, 0.5486095, 0.5439481, 0.5394201, 0.5349962, 0.5306490, 0.5263589, 0.5221163,\
        0.5179196, 0.5137637, 0.5096167, 0.5053773, 0.5008871, 0.4964310, 0.4921917, 0.4880223, 0.4837404,\
        0.4792643, 0.4745795, 0.4696910, 0.4646026, 0.4593140, 0.4538219, 0.4481211, 0.4422047, 0.4360633,\
        0.4296840, 0.4230524, 0.4161541, 0.4089776],\
        [0.4546678, 0.4605045, 0.4661545, 0.4716104, 0.4768590, 0.4818807, 0.4866444, 0.4910748, 0.4950051,\
        0.4984746, 0.5019960, 0.5058205, 0.5098519, 0.5138584, 0.5175533, 0.5208162, 0.5236582, 0.5260681,\
        0.5279890, 0.5293622, 0.5301389, 0.5302564, 0.5296088, 0.5280689, 0.5256325, 0.5225246, 0.5187836,\
        0.5139973, 0.5079982, 0.5009607, 0.4930164, 0.4831290, 0.4714840, 0.4598726, 0.4491253, 0.4393855,\
        0.4306669, 0.4230891, 0.4169271, 0.4125647, 0.4102370, 0.4095982, 0.4101822, 0.4117960, 0.4143343,\
        0.4178060, 0.4223290, 0.4280426, 0.4350321, 0.4432875, 0.4526686, 0.4628386, 0.4731723, 0.4830907,\
        0.4935882, 0.5067787, 0.5223380, 0.5384021, 0.5538075, 0.5671293, 0.5768482, 0.5834370, 0.5891891,\
        0.5946174, 0.5996206, 0.6043406, 0.6092243, 0.6149947, 0.6214850, 0.6274835, 0.6321004, 0.6350017,\
        0.6362569, 0.6361202, 0.6348557, 0.6326692, 0.6297475, 0.6262592, 0.6223015, 0.6180357, 0.6135554,\
        0.6088366, 0.6041877, 0.5997552, 0.5955366, 0.5908059, 0.5859570, 0.5810073, 0.5757149, 0.5701823,\
        0.5646753, 0.5593614, 0.5542835, 0.5494112, 0.5446905, 0.5400718, 0.5355195, 0.5310116, 0.5265357,\
        0.5220815, 0.5176301, 0.5131385, 0.5085383, 0.5038169, 0.4992016, 0.4950255, 0.4908818, 0.4865269,\
        0.4819172, 0.4770879, 0.4720642, 0.4668520, 0.4614483, 0.4558469, 0.4500410, 0.4440227, 0.4377831,\
        0.4313116, 0.4245967, 0.4176273, 0.4103929],\
        [0.4557962, 0.4619260, 0.4678860, 0.4736673, 0.4792541, 0.4846262, 0.4897699, 0.4946759, 0.4991192,\
        0.5028253, 0.5065841, 0.5105831, 0.5147503, 0.5189154, 0.5228897, 0.5265481, 0.5298332, 0.5327041,\
        0.5351097, 0.5369922, 0.5382895, 0.5389270, 0.5388135, 0.5378668, 0.5360651, 0.5334350, 0.5299115,\
        0.5253117, 0.5195538, 0.5127560, 0.5049646, 0.4944469, 0.4815787, 0.4694213, 0.4584398, 0.4485056,\
        0.4395847, 0.4317827, 0.4253059, 0.4203960, 0.4171785, 0.4155421, 0.4152756, 0.4162379, 0.4183711,\
        0.4216947, 0.4262933, 0.4322767, 0.4397338, 0.4486933, 0.4590878, 0.4706856, 0.4828340, 0.4937971,\
        0.5044498, 0.5180966, 0.5337201, 0.5501501, 0.5663509, 0.5808375, 0.5903958, 0.5946606, 0.6000743,\
        0.6053934, 0.6104742, 0.6154237, 0.6206136, 0.6263195, 0.6323298, 0.6379191, 0.6423863, 0.6453595,\
        0.6467960, 0.6468508, 0.6457443, 0.6436878, 0.6408594, 0.6373967, 0.6334095, 0.6290093, 0.6242872,\
        0.6193461, 0.6143696, 0.6095028, 0.6046382, 0.5993102, 0.5941198, 0.5889177, 0.5829991, 0.5767959,\
        0.5707960, 0.5651671, 0.5598833, 0.5548459, 0.5499577, 0.5451513, 0.5403886, 0.5356507, 0.5309279,\
        0.5262106, 0.5214824, 0.5167186, 0.5119022, 0.5070789, 0.5024374, 0.4981363, 0.4938924, 0.4893203,\
        0.4844795, 0.4794611, 0.4742823, 0.4689350, 0.4634072, 0.4576886, 0.4517706, 0.4456453, 0.4393047,\
        0.4327401, 0.4259421, 0.4189014, 0.4116087],\
        [0.4565129, 0.4629134, 0.4691601, 0.4752425, 0.4811396, 0.4868148, 0.4922233, 0.4973755, 0.5022331,\
        0.5065019, 0.5106333, 0.5148853, 0.5192503, 0.5236218, 0.5278667, 0.5318732, 0.5355623, 0.5388729,\
        0.5417468, 0.5441219, 0.5459295, 0.5470902, 0.5475142, 0.5471133, 0.5458129, 0.5435370, 0.5401703,\
        0.5355698, 0.5296349, 0.5222996, 0.5132696, 0.5017495, 0.4891832, 0.4777910, 0.4673792, 0.4576676,\
        0.4487684, 0.4408356, 0.4340319, 0.4285358, 0.4244808, 0.4219005, 0.4207587, 0.4210145, 0.4226563,\
        0.4257131, 0.4302538, 0.4363722, 0.4441601, 0.4536727, 0.4648898, 0.4776784, 0.4916900, 0.5050655,\
        0.5167004, 0.5302213, 0.5453114, 0.5612283, 0.5772703, 0.5921983, 0.6027339, 0.6059783, 0.6108007,\
        0.6162005, 0.6214518, 0.6267097, 0.6321951, 0.6379886, 0.6438455, 0.6492178, 0.6535259, 0.6564221,\
        0.6578529, 0.6579473, 0.6569048, 0.6549346, 0.6521955, 0.6487851, 0.6447815, 0.6402727, 0.6353553,\
        0.6301471, 0.6247861, 0.6193589, 0.6137993, 0.6080567, 0.6025179, 0.5969231, 0.5899901, 0.5830585,\
        0.5766487, 0.5708260, 0.5654377, 0.5602930, 0.5552557, 0.5502550, 0.5452644, 0.5402782, 0.5352964,\
        0.5303180, 0.5253387, 0.5203573, 0.5153929, 0.5105143, 0.5058406, 0.5014399, 0.4969534, 0.4919614,\
        0.4868468, 0.4816430, 0.4763158, 0.4708348, 0.4651801, 0.4593394, 0.4533043, 0.4470681, 0.4406240,\
        0.4339651, 0.4270836, 0.4199715, 0.4126205],\
        [0.4567940, 0.4634391, 0.4699448, 0.4763004, 0.4824858, 0.4884578, 0.4941141, 0.4993433, 0.5045007,\
        0.5093333, 0.5139539, 0.5185626, 0.5232102, 0.5278479, 0.5323863, 0.5367326, 0.5408063, 0.5445382,\
        0.5478638, 0.5507173, 0.5530268, 0.5547088, 0.5556661, 0.5557873, 0.5549469, 0.5529998, 0.5497794,\
        0.5451122, 0.5388357, 0.5307661, 0.5206447, 0.5085255, 0.4963029, 0.4864317, 0.4767014, 0.4672014,\
        0.4583624, 0.4503116, 0.4431396, 0.4370065, 0.4321053, 0.4285960, 0.4265789, 0.4261063, 0.4272088,\
        0.4299207, 0.4342961, 0.4404143, 0.4483702, 0.4582361, 0.4699884, 0.4834223, 0.4981454, 0.5133160,\
        0.5277060, 0.5420032, 0.5565354, 0.5712525, 0.5860848, 0.5999254, 0.6103425, 0.6166109, 0.6217866,\
        0.6271001, 0.6324638, 0.6379547, 0.6437012, 0.6496904, 0.6556466, 0.6610340, 0.6652571, 0.6679888,\
        0.6692740, 0.6692817, 0.6681999, 0.6662691, 0.6636267, 0.6602866, 0.6562756, 0.6516678, 0.6465506,\
        0.6410321, 0.6352369, 0.6292539, 0.6231106, 0.6168606, 0.6105923, 0.6038547, 0.5962549, 0.5888506,\
        0.5822533, 0.5764300, 0.5710534, 0.5658376, 0.5606354, 0.5554040, 0.5501480, 0.5448839, 0.5396254,\
        0.5343811, 0.5291579, 0.5239679, 0.5188399, 0.5138263, 0.5089846, 0.5042879, 0.4993681, 0.4941607,\
        0.4889151, 0.4835948, 0.4781474, 0.4725409, 0.4667585, 0.4607916, 0.4546350, 0.4482843, 0.4417346,\
        0.4349803, 0.4280152, 0.4208322, 0.4134236],\
        [0.4566239, 0.4634855, 0.4702187, 0.4768129, 0.4832536, 0.4895236, 0.4955939, 0.5012202, 0.5064165,\
        0.5115444, 0.5165768, 0.5215703, 0.5265564, 0.5315113, 0.5363740, 0.5410693, 0.5455225, 0.5496636,\
        0.5534264, 0.5567446, 0.5595466, 0.5617483, 0.5632451, 0.5639033, 0.5635543, 0.5619962, 0.5590108,\
        0.5543926, 0.5479736, 0.5396359, 0.5293960, 0.5177621, 0.5065311, 0.4970518, 0.4869374, 0.4773725,\
        0.4685626, 0.4603611, 0.4527312, 0.4458593, 0.4400428, 0.4355800, 0.4326915, 0.4315014, 0.4320651,\
        0.4344078, 0.4385588, 0.4445768, 0.4525638, 0.4626456, 0.4748661, 0.4890204, 0.5048605, 0.5218001,\
        0.5383263, 0.5537585, 0.5678732, 0.5810014, 0.5941552, 0.6073836, 0.6182530, 0.6261849, 0.6323548,\
        0.6379092, 0.6433672, 0.6489975, 0.6549441, 0.6611784, 0.6674208, 0.6730696, 0.6773179, 0.6797757,\
        0.6808107, 0.6806744, 0.6794337, 0.6775143, 0.6750210, 0.6717688, 0.6677407, 0.6630374, 0.6577107,\
        0.6518503, 0.6456045, 0.6390935, 0.6323692, 0.6254352, 0.6182099, 0.6104662, 0.6022757, 0.5944812,\
        0.5878934, 0.5822415, 0.5769259, 0.5715929, 0.5661426, 0.5606006, 0.5550189, 0.5494367, 0.5438767,\
        0.5383523, 0.5328744, 0.5274562, 0.5221171, 0.5168761, 0.5117244, 0.5065698, 0.5012779, 0.4960056,\
        0.4907334, 0.4853404, 0.4797849, 0.4740515, 0.4681356, 0.4620364, 0.4557535, 0.4492850, 0.4426280,\
        0.4357782, 0.4287302, 0.4214775, 0.4140131],\
        [0.4559958, 0.4630466, 0.4699762, 0.4767695, 0.4834060, 0.4898622, 0.4961041, 0.5020242, 0.5076258,\
        0.5131121, 0.5185331, 0.5239124, 0.5292638, 0.5345698, 0.5397836, 0.5448419, 0.5496761, 0.5542188,\
        0.5584052, 0.5621724, 0.5654541, 0.5681723, 0.5702237, 0.5714635, 0.5716903, 0.5706500, 0.5680717,\
        0.5637317, 0.5575108, 0.5494334, 0.5397490, 0.5291120, 0.5185721, 0.5083331, 0.4979641, 0.4883296,\
        0.4795985, 0.4711537, 0.4628962, 0.4551165, 0.4482607, 0.4427981, 0.4390586, 0.4372048, 0.4372880,\
        0.4393051, 0.4432433, 0.4491231, 0.4570439, 0.4672250, 0.4799732, 0.4952544, 0.5125800, 0.5312154,\
        0.5496758, 0.5664994, 0.5805957, 0.5922700, 0.6041658, 0.6168212, 0.6277767, 0.6363001, 0.6429650,\
        0.6486656, 0.6541160, 0.6597625, 0.6658029, 0.6722286, 0.6788216, 0.6849819, 0.6894445, 0.6913943,\
        0.6921221, 0.6919881, 0.6903497, 0.6884951, 0.6862856, 0.6830904, 0.6789889, 0.6742298, 0.6686951,\
        0.6624583, 0.6557668, 0.6487842, 0.6415451, 0.6340050, 0.6260732, 0.6176850, 0.6090239, 0.6008092,\
        0.5941613, 0.5886707, 0.5832707, 0.5776345, 0.5717681, 0.5657994, 0.5598222, 0.5538807, 0.5479938,\
        0.5421714, 0.5364217, 0.5307522, 0.5251690, 0.5196674, 0.5142165, 0.5087537, 0.5032480, 0.4978221,\
        0.4924473, 0.4869373, 0.4812456, 0.4753650, 0.4693018, 0.4630618, 0.4566475, 0.4500591, 0.4432944,\
        0.4363499, 0.4292208, 0.4219010, 0.4143837],\
        [0.4549115, 0.4621283, 0.4692306, 0.4761972, 0.4829948, 0.4895747, 0.4958844, 0.5020252, 0.5081176,\
        0.5140348, 0.5198449, 0.5256047, 0.5313316, 0.5370087, 0.5425935, 0.5480279, 0.5532470, 0.5581852,\
        0.5627807, 0.5669761, 0.5707159, 0.5739374, 0.5765535, 0.5784273, 0.5793413, 0.5789873, 0.5770238,\
        0.5731992, 0.5674594, 0.5599600, 0.5510466, 0.5412423, 0.5310731, 0.5207226, 0.5103081, 0.5004888,\
        0.4917595, 0.4827302, 0.4736181, 0.4647127, 0.4566563, 0.4501683, 0.4456492, 0.4432425, 0.4429655,\
        0.4447736, 0.4485985, 0.4543989, 0.4622285, 0.4723237, 0.4852711, 0.5017882, 0.5207629, 0.5411917,\
        0.5616926, 0.5804772, 0.5953264, 0.6067481, 0.6178960, 0.6293205, 0.6395033, 0.6477127, 0.6541735,\
        0.6596090, 0.6648029, 0.6703078, 0.6763058, 0.6827445, 0.6894738, 0.6961089, 0.7010050, 0.7024906,\
        0.7029948, 0.7028771, 0.7008389, 0.6992307, 0.6973531, 0.6939913, 0.6897555, 0.6851258, 0.6793521,\
        0.6726674, 0.6655765, 0.6582524, 0.6506670, 0.6427435, 0.6344253, 0.6257523, 0.6169849, 0.6087147,\
        0.6018176, 0.5959603, 0.5901213, 0.5838790, 0.5773835, 0.5708843, 0.5644649, 0.5581392, 0.5519080,\
        0.5457733, 0.5397376, 0.5338029, 0.5279661, 0.5222151, 0.5165243, 0.5108643, 0.5052342, 0.4996794,\
        0.4940881, 0.4883978, 0.4825221, 0.4764644, 0.4702380, 0.4638497, 0.4573014, 0.4505929, 0.4437221,\
        0.4366857, 0.4294790, 0.4220961, 0.4145301],\
        [0.4533784, 0.4607429, 0.4680042, 0.4751403, 0.4821180, 0.4888933, 0.4954343, 0.5018346, 0.5081890,\
        0.5144037, 0.5205458, 0.5266665, 0.5327699, 0.5388297, 0.5448001, 0.5506221, 0.5562305, 0.5615594,\
        0.5665480, 0.5711447, 0.5753075, 0.5789979, 0.5821614, 0.5846962, 0.5864002, 0.5869088, 0.5857635,\
        0.5826345, 0.5775427, 0.5707715, 0.5626941, 0.5537046, 0.5441096, 0.5340889, 0.5239005, 0.5140489,\
        0.5045662, 0.4947181, 0.4847303, 0.4744277, 0.4650207, 0.4576020, 0.4524720, 0.4496766, 0.4492007,\
        0.4509739, 0.4548714, 0.4607770, 0.4686825, 0.4787544, 0.4914599, 0.5085543, 0.5293528, 0.5512676,\
        0.5732678, 0.5939543, 0.6099003, 0.6219990, 0.6331396, 0.6437539, 0.6530732, 0.6605140, 0.6662356,\
        0.6709594, 0.6755937, 0.6808445, 0.6867474, 0.6930092, 0.6995710, 0.7059668, 0.7107560, 0.7129560,\
        0.7135964, 0.7131554, 0.7116554, 0.7098814, 0.7076662, 0.7040784, 0.6999762, 0.6956443, 0.6893794,\
        0.6821842, 0.6748723, 0.6674296, 0.6597173, 0.6516420, 0.6431833, 0.6344434, 0.6257094, 0.6174600,\
        0.6101354, 0.6035736, 0.5971304, 0.5899435, 0.5827149, 0.5756880, 0.5688376, 0.5621284, 0.5555469,\
        0.5490912, 0.5427616, 0.5365564, 0.5304689, 0.5244866, 0.5185928, 0.5127744, 0.5070223, 0.5012848,\
        0.4955729, 0.4896672, 0.4835616, 0.4773076, 0.4709128, 0.4643762, 0.4576962, 0.4508712, 0.4438987,\
        0.4367754, 0.4294965, 0.4220562, 0.4144474],\
        [0.4514066, 0.4589029, 0.4663128, 0.4736189, 0.4808005, 0.4878371, 0.4947106, 0.5013872, 0.5078437,\
        0.5142184, 0.5206423, 0.5271124, 0.5335943, 0.5400455, 0.5464127, 0.5526328, 0.5586366, 0.5643544,\
        0.5697221, 0.5746892, 0.5792265, 0.5833246, 0.5869731, 0.5901258, 0.5926590, 0.5941651, 0.5940136,\
        0.5916797, 0.5873233, 0.5813834, 0.5742208, 0.5661653, 0.5573712, 0.5478738, 0.5378657, 0.5276521,\
        0.5172876, 0.5067250, 0.4959665, 0.4836542, 0.4731033, 0.4651474, 0.4596643, 0.4566335, 0.4560961,\
        0.4580168, 0.4622193, 0.4684906, 0.4768076, 0.4874420, 0.5009835, 0.5185777, 0.5395075, 0.5615580,\
        0.5835981, 0.6044506, 0.6218060, 0.6358581, 0.6480561, 0.6588485, 0.6677916, 0.6744530, 0.6791860,\
        0.6829151, 0.6867087, 0.6916965, 0.6976906, 0.7037600, 0.7100750, 0.7159264, 0.7203841, 0.7229153,\
        0.7237956, 0.7234160, 0.7220724, 0.7200766, 0.7173895, 0.7138179, 0.7096985, 0.7048419, 0.6980632,\
        0.6907535, 0.6836066, 0.6762910, 0.6686358, 0.6605715, 0.6520982, 0.6433316, 0.6345313, 0.6260407,\
        0.6181031, 0.6106420, 0.6031577, 0.5952149, 0.5875064, 0.5800884, 0.5728577, 0.5657751, 0.5588402,\
        0.5520585, 0.5454315, 0.5389558, 0.5326228, 0.5264191, 0.5203303, 0.5143471, 0.5084767, 0.5027408,\
        0.4968470, 0.4905945, 0.4842650, 0.4778369, 0.4712893, 0.4646151, 0.4578120, 0.4508783, 0.4438118,\
        0.4366091, 0.4292656, 0.4217755, 0.4141314],\
        [0.4490080, 0.4566196, 0.4641630, 0.4716250, 0.4789918, 0.4862558, 0.4934204, 0.5003928, 0.5068132,\
        0.5133724, 0.5201252, 0.5269610, 0.5338267, 0.5406756, 0.5474489, 0.5540774, 0.5604860, 0.5665976,\
        0.5723391, 0.5776519, 0.5825096, 0.5869343, 0.5909731, 0.5946146, 0.5978238, 0.6003290, 0.6011758,\
        0.5997232, 0.5962855, 0.5913343, 0.5851824, 0.5782493, 0.5705137, 0.5616277, 0.5517212, 0.5411202,\
        0.5299063, 0.5180125, 0.5049971, 0.4916267, 0.4812723, 0.4732979, 0.4675998, 0.4643206, 0.4637251,\
        0.4659108, 0.4706385, 0.4774795, 0.4863706, 0.4978396, 0.5124748, 0.5306070, 0.5513902, 0.5728570,\
        0.5935367, 0.6142895, 0.6328318, 0.6486303, 0.6622434, 0.6738741, 0.6830518, 0.6891307, 0.6929218,\
        0.6957963, 0.6987605, 0.7034149, 0.7093993, 0.7154003, 0.7212357, 0.7264600, 0.7304688, 0.7329092,\
        0.7338192, 0.7334344, 0.7320117, 0.7297503, 0.7267068, 0.7228807, 0.7182988, 0.7127179, 0.7059045,\
        0.6988304, 0.6919899, 0.6848559, 0.6773288, 0.6693521, 0.6608907, 0.6520318, 0.6430013, 0.6340659,\
        0.6253899, 0.6169195, 0.6084075, 0.5998789, 0.5918432, 0.5841008, 0.5764881, 0.5690166, 0.5617190,\
        0.5546078, 0.5476835, 0.5409404, 0.5343677, 0.5279487, 0.5216615, 0.5154797, 0.5093736, 0.5033069,\
        0.4971993, 0.4909553, 0.4845554, 0.4780123, 0.4713393, 0.4645442, 0.4576309, 0.4506001, 0.4434502,\
        0.4361783, 0.4287800, 0.4212494, 0.4135791],\
        [0.4461973, 0.4539064, 0.4615644, 0.4691569, 0.4766629, 0.4840493, 0.4912477, 0.4980940, 0.5047914,\
        0.5118942, 0.5190557, 0.5262538, 0.5334967, 0.5407444, 0.5479298, 0.5549761, 0.5618020, 0.5683226,\
        0.5744514, 0.5801083, 0.5852469, 0.5899077, 0.5942443, 0.5983323, 0.6020061, 0.6050942, 0.6066764,\
        0.6062894, 0.6040458, 0.6002775, 0.5951464, 0.5896179, 0.5833095, 0.5750269, 0.5652305, 0.5545073,\
        0.5427453, 0.5296631, 0.5153273, 0.5018727, 0.4912778, 0.4830442, 0.4768568, 0.4729942, 0.4721049,\
        0.4745134, 0.4799413, 0.4874681, 0.4966815, 0.5087281, 0.5241538, 0.5430312, 0.5643760, 0.5861975,\
        0.6064641, 0.6259930, 0.6444530, 0.6610601, 0.6757309, 0.6882096, 0.6979938, 0.7037955, 0.7071351,\
        0.7097778, 0.7126942, 0.7168727, 0.7221830, 0.7277283, 0.7329981, 0.7375697, 0.7410267, 0.7431086,\
        0.7437846, 0.7431883, 0.7415089, 0.7389080, 0.7354781, 0.7312574, 0.7262391, 0.7203667, 0.7137584,\
        0.7069256, 0.7000930, 0.6930410, 0.6856334, 0.6777843, 0.6693053, 0.6602430, 0.6508404, 0.6413576,\
        0.6319579, 0.6226710, 0.6134712, 0.6044765, 0.5959716, 0.5877745, 0.5796753, 0.5717760, 0.5641090,\
        0.5566701, 0.5494524, 0.5424481, 0.5356447, 0.5290217, 0.5225484, 0.5161796, 0.5098442, 0.5034391,\
        0.4971592, 0.4909476, 0.4844826, 0.4778343, 0.4710490, 0.4641477, 0.4571389, 0.4500250, 0.4428052,\
        0.4354767, 0.4280352, 0.4204752, 0.4127894],\
        [0.4429934, 0.4507829, 0.4585380, 0.4662421, 0.4738683, 0.4813680, 0.4886491, 0.4956094, 0.5027575,\
        0.5101879, 0.5175516, 0.5250321, 0.5326331, 0.5402773, 0.5478762, 0.5553447, 0.5625990, 0.5695516,\
        0.5761058, 0.5821535, 0.5875920, 0.5924138, 0.5968837, 0.6014102, 0.6056223, 0.6090802, 0.6112514,\
        0.6117807, 0.6106511, 0.6080207, 0.6041109, 0.6002171, 0.5955152, 0.5874537, 0.5780182, 0.5679033,\
        0.5564466, 0.5433456, 0.5292861, 0.5158374, 0.5043204, 0.4952589, 0.4879760, 0.4829100, 0.4812826,\
        0.4835327, 0.4896277, 0.4979329, 0.5071868, 0.5192681, 0.5350594, 0.5546858, 0.5772583, 0.5999354,\
        0.6206470, 0.6393502, 0.6567763, 0.6733155, 0.6886343, 0.7013126, 0.7111144, 0.7173950, 0.7212458,\
        0.7241884, 0.7272703, 0.7311320, 0.7357584, 0.7406404, 0.7452431, 0.7491253, 0.7519391, 0.7534744,\
        0.7536873, 0.7526680, 0.7505672, 0.7475291, 0.7436569, 0.7390151, 0.7336477, 0.7276212, 0.7211110,\
        0.7143864, 0.7075733, 0.7005448, 0.6933218, 0.6856713, 0.6770873, 0.6676787, 0.6578016, 0.6477495,\
        0.6377103, 0.6277845, 0.6180577, 0.6086893, 0.5998016, 0.5909135, 0.5822507, 0.5739461, 0.5659329,\
        0.5581784, 0.5506746, 0.5434173, 0.5363953, 0.5295867, 0.5229576, 0.5164629, 0.5100558, 0.5037326,\
        0.4973577, 0.4907619, 0.4840888, 0.4773023, 0.4704069, 0.4634126, 0.4563248, 0.4491448, 0.4418710,\
        0.4345006, 0.4270295, 0.4194528, 0.4117635],\
        [0.4394195, 0.4472744, 0.4551133, 0.4629203, 0.4706734, 0.4783496, 0.4859490, 0.4935703, 0.5012008,\
        0.5081988, 0.5155396, 0.5232894, 0.5312604, 0.5393023, 0.5473073, 0.5551898, 0.5628717, 0.5702727,\
        0.5773012, 0.5838408, 0.5897280, 0.5947919, 0.5993056, 0.6041455, 0.6087557, 0.6125709, 0.6152439,\
        0.6165380, 0.6164000, 0.6149468, 0.6125161, 0.6095469, 0.6051935, 0.5977471, 0.5897188, 0.5813263,\
        0.5710007, 0.5584857, 0.5450065, 0.5318392, 0.5199703, 0.5098224, 0.5012004, 0.4944164, 0.4918365,\
        0.4930144, 0.4986557, 0.5072744, 0.5168013, 0.5289216, 0.5446326, 0.5644431, 0.5883611, 0.6129241,\
        0.6347477, 0.6534069, 0.6694315, 0.6850637, 0.7009400, 0.7138733, 0.7236983, 0.7304966, 0.7350041,\
        0.7384126, 0.7416967, 0.7453922, 0.7495488, 0.7538317, 0.7577660, 0.7609335, 0.7630289, 0.7638734,\
        0.7634350, 0.7618066, 0.7591363, 0.7555623, 0.7511888, 0.7460990, 0.7403801, 0.7341523, 0.7275867,\
        0.7208653, 0.7140827, 0.7072168, 0.7002525, 0.6927947, 0.6838620, 0.6739712, 0.6635925, 0.6530202,\
        0.6424499, 0.6320029, 0.6217704, 0.6118480, 0.6022576, 0.5929236, 0.5840292, 0.5754541, 0.5671344,\
        0.5590736, 0.5512883, 0.5437862, 0.5365595, 0.5295849, 0.5228262, 0.5162383, 0.5097642, 0.5033105,\
        0.4967951, 0.4901077, 0.4832886, 0.4763814, 0.4693929, 0.4623256, 0.4551800, 0.4479544, 0.4406458,\
        0.4332505, 0.4257650, 0.4181853, 0.4105056],\
        [0.4355028, 0.4434102, 0.4513222, 0.4592229, 0.4670945, 0.4749247, 0.4827218, 0.4904914, 0.4979451,\
        0.5051060, 0.5128632, 0.5210480, 0.5294307, 0.5378590, 0.5462411, 0.5545071, 0.5625913, 0.5704267,\
        0.5779419, 0.5850528, 0.5916306, 0.5973687, 0.6021329, 0.6069580, 0.6116081, 0.6156408, 0.6187289,\
        0.6206679, 0.6213893, 0.6209559, 0.6195135, 0.6170452, 0.6130015, 0.6071569, 0.6005992, 0.5948288,\
        0.5856404, 0.5737902, 0.5611571, 0.5485996, 0.5367956, 0.5261072, 0.5166285, 0.5090188, 0.5052908,\
        0.5049746, 0.5086259, 0.5163716, 0.5259817, 0.5380411, 0.5533980, 0.5725847, 0.5964111, 0.6243788,\
        0.6500197, 0.6694439, 0.6833790, 0.6971526, 0.7129772, 0.7267040, 0.7368937, 0.7439166, 0.7486986,\
        0.7523580, 0.7557615, 0.7593424, 0.7631825, 0.7670046, 0.7703206, 0.7727599, 0.7740798, 0.7741180,\
        0.7728696, 0.7704739, 0.7671124, 0.7629202, 0.7579842, 0.7523883, 0.7462439, 0.7396974, 0.7329187,\
        0.7260690, 0.7192607, 0.7125414, 0.7058393, 0.6983164, 0.6889184, 0.6786600, 0.6678883, 0.6569252,\
        0.6459728, 0.6351442, 0.6244948, 0.6140250, 0.6037400, 0.5941756, 0.5851813, 0.5763361, 0.5676849,\
        0.5593019, 0.5512322, 0.5434931, 0.5360777, 0.5289570, 0.5220854, 0.5154115, 0.5088809, 0.5023764,\
        0.4957150, 0.4889203, 0.4820153, 0.4750334, 0.4679864, 0.4608760, 0.4536998, 0.4464537, 0.4391326,\
        0.4317319, 0.4242483, 0.4166801, 0.4090237],\
        [0.4312735, 0.4392229, 0.4471989, 0.4551835, 0.4631536, 0.4710832, 0.4789323, 0.4866080, 0.4939938,\
        0.5015592, 0.5098294, 0.5184914, 0.5272463, 0.5359952, 0.5446920, 0.5532852, 0.5617175, 0.5699299,\
        0.5778631, 0.5854582, 0.5926495, 0.5992458, 0.6044307, 0.6092563, 0.6139596, 0.6181899, 0.6216473,\
        0.6241392, 0.6255766, 0.6259520, 0.6252780, 0.6234763, 0.6203846, 0.6161503, 0.6115214, 0.6065622,\
        0.5982041, 0.5878643, 0.5766215, 0.5651032, 0.5539234, 0.5435383, 0.5342156, 0.5265544, 0.5215979,\
        0.5198245, 0.5217297, 0.5273725, 0.5360074, 0.5476549, 0.5628811, 0.5822435, 0.6069522, 0.6381077,\
        0.6681084, 0.6893306, 0.7013273, 0.7138101, 0.7285409, 0.7417511, 0.7515792, 0.7581639, 0.7626188,\
        0.7661418, 0.7695130, 0.7728889, 0.7764458, 0.7799558, 0.7826620, 0.7843282, 0.7848394, 0.7839734,\
        0.7817751, 0.7784820, 0.7743540, 0.7694982, 0.7639430, 0.7577566, 0.7510711, 0.7440571, 0.7368905,\
        0.7297227, 0.7226593, 0.7157324, 0.7087465, 0.7009088, 0.6918003, 0.6813959, 0.6704145, 0.6592573,\
        0.6481184, 0.6371035, 0.6262701, 0.6156529, 0.6053098, 0.5954608, 0.5859437, 0.5766013, 0.5675290,\
        0.5587949, 0.5504392, 0.5424767, 0.5348958, 0.5276569, 0.5206928, 0.5139256, 0.5073041, 0.5007922,\
        0.4940806, 0.4871977, 0.4802498, 0.4732415, 0.4661779, 0.4590602, 0.4518856, 0.4446481, 0.4373403,\
        0.4299560, 0.4224920, 0.4149495, 0.4073301],\
        [0.4267649, 0.4347481, 0.4427828, 0.4508467, 0.4589112, 0.4669428, 0.4748969, 0.4827211, 0.4904327,\
        0.4982650, 0.5069054, 0.5158617, 0.5247904, 0.5337318, 0.5426586, 0.5515094, 0.5602203, 0.5687285,\
        0.5769654, 0.5848467, 0.5922466, 0.5989537, 0.6049610, 0.6104429, 0.6155117, 0.6200536, 0.6238927,\
        0.6268826, 0.6289293, 0.6299845, 0.6300166, 0.6289845, 0.6268856, 0.6238748, 0.6201615, 0.6153369,\
        0.6085022, 0.6000603, 0.5905693, 0.5804223, 0.5703405, 0.5610211, 0.5525226, 0.5449266, 0.5390192,\
        0.5357688, 0.5359138, 0.5397379, 0.5471427, 0.5582186, 0.5735919, 0.5943231, 0.6209072, 0.6529137,\
        0.6857970, 0.7095895, 0.7222471, 0.7340689, 0.7471711, 0.7593508, 0.7680581, 0.7734106, 0.7769489,\
        0.7798961, 0.7832230, 0.7861034, 0.7891221, 0.7925803, 0.7945161, 0.7952864, 0.7950099, 0.7931154,\
        0.7898795, 0.7855916, 0.7807207, 0.7752170, 0.7689888, 0.7621007, 0.7547309, 0.7470900, 0.7393642,\
        0.7316838, 0.7241092, 0.7166381, 0.7092313, 0.7017562, 0.6924502, 0.6819432, 0.6710091, 0.6599003,\
        0.6487841, 0.6377720, 0.6269439, 0.6163886, 0.6062463, 0.5962039, 0.5859458, 0.5760425, 0.5665387,\
        0.5574620, 0.5488388, 0.5406786, 0.5329669, 0.5256563, 0.5186529, 0.5118183, 0.5050396, 0.4983532,\
        0.4917138, 0.4849122, 0.4779974, 0.4710114, 0.4639716, 0.4568835, 0.4497449, 0.4425486, 0.4352841,\
        0.4279414, 0.4205160, 0.4130130, 0.4054433],\
        [0.4220133, 0.4300241, 0.4381156, 0.4462595, 0.4544263, 0.4625913, 0.4707399, 0.4788863, 0.4871176,\
        0.4956252, 0.5044481, 0.5131338, 0.5219858, 0.5310244, 0.5401189, 0.5491657, 0.5580905, 0.5668258,\
        0.5752923, 0.5833826, 0.5909538, 0.5979126, 0.6045107, 0.6106622, 0.6162110, 0.6211388, 0.6253809,\
        0.6288416, 0.6314329, 0.6330902, 0.6337710, 0.6334540, 0.6321590, 0.6299494, 0.6268225, 0.6225433,\
        0.6168668, 0.6099694, 0.6021454, 0.5935507, 0.5847283, 0.5772629, 0.5702656, 0.5627986, 0.5559675,\
        0.5513339, 0.5499733, 0.5524335, 0.5589144, 0.5695200, 0.5847153, 0.6060348, 0.6334677, 0.6652340,\
        0.6974736, 0.7233576, 0.7400423, 0.7528638, 0.7658359, 0.7782976, 0.7857841, 0.7894257, 0.7920537,\
        0.7938188, 0.7972989, 0.7994841, 0.8010392, 0.8044568, 0.8055574, 0.8053591, 0.8040346, 0.8011070,\
        0.7967714, 0.7915706, 0.7861794, 0.7800805, 0.7730850, 0.7653372, 0.7571187, 0.7487019, 0.7402878,\
        0.7319799, 0.7237681, 0.7155417, 0.7072161, 0.6988709, 0.6901615, 0.6802695, 0.6697239, 0.6588809,\
        0.6479521, 0.6370712, 0.6263186, 0.6157262, 0.6052742, 0.5949589, 0.5845994, 0.5744036, 0.5645787,\
        0.5552193, 0.5463723, 0.5380527, 0.5302500, 0.5229264, 0.5159856, 0.5092220, 0.5024087, 0.4956521,\
        0.4889552, 0.4821745, 0.4753033, 0.4683674, 0.4613838, 0.4543589, 0.4472908, 0.4401706, 0.4329840,\
        0.4257146, 0.4183509, 0.4108997, 0.4033898],\
        [0.4170585, 0.4250928, 0.4332397, 0.4414606, 0.4497269, 0.4580288, 0.4663797, 0.4748288, 0.4834779,\
        0.4924368, 0.5012471, 0.5096783, 0.5186085, 0.5278039, 0.5370495, 0.5462448, 0.5553306, 0.5642499,\
        0.5729301, 0.5812767, 0.5891838, 0.5965962, 0.6036436, 0.6102105, 0.6161211, 0.6214158, 0.6260658,\
        0.6299857, 0.6330836, 0.6352854, 0.6365416, 0.6368288, 0.6361492, 0.6345114, 0.6318803, 0.6281575,\
        0.6232890, 0.6173844, 0.6106472, 0.6031761, 0.5956114, 0.5911790, 0.5864152, 0.5787631, 0.5709660,\
        0.5652414, 0.5628387, 0.5644754, 0.5705964, 0.5813851, 0.5966785, 0.6181045, 0.6455496, 0.6770875,\
        0.7099600, 0.7380682, 0.7568091, 0.7693986, 0.7818452, 0.7959613, 0.8033469, 0.8062252, 0.8085291,\
        0.8097749, 0.8119930, 0.8135575, 0.8144490, 0.8157459, 0.8158446, 0.8145205, 0.8118655, 0.8077932,\
        0.8023193, 0.7966991, 0.7909619, 0.7841830, 0.7761941, 0.7673628, 0.7581262, 0.7488174, 0.7396510,\
        0.7307185, 0.7219702, 0.7131794, 0.7040711, 0.6952869, 0.6865308, 0.6770340, 0.6668884, 0.6563733,\
        0.6457037, 0.6350191, 0.6243933, 0.6138304, 0.6031857, 0.5925768, 0.5820593, 0.5716597, 0.5616010,\
        0.5520282, 0.5430151, 0.5345825, 0.5267187, 0.5194111, 0.5126257, 0.5061439, 0.4995028, 0.4926521,\
        0.4858436, 0.4790392, 0.4722056, 0.4653368, 0.4584354, 0.4515033, 0.4445377, 0.4375296, 0.4304610,\
        0.4233064, 0.4160400, 0.4086573, 0.4012110],\
        [0.4119479, 0.4200037, 0.4282020, 0.4364832, 0.4448215, 0.4532208, 0.4617056, 0.4703137, 0.4790689,\
        0.4878798, 0.4964817, 0.5053094, 0.5146781, 0.5240918, 0.5334514, 0.5427396, 0.5519374, 0.5610082,\
        0.5698969, 0.5785377, 0.5868704, 0.5948527, 0.6023507, 0.6090629, 0.6152011, 0.6208500, 0.6259268,\
        0.6303115, 0.6338957, 0.6365960, 0.6383560, 0.6391437, 0.6389455, 0.6377510, 0.6355364, 0.6322711,\
        0.6279636, 0.6226985, 0.6166183, 0.6099748, 0.6040626, 0.6019168, 0.5987472, 0.5907787, 0.5825466,\
        0.5764617, 0.5736671, 0.5749784, 0.5811360, 0.5926839, 0.6092794, 0.6313763, 0.6594439, 0.6916577,\
        0.7284414, 0.7588812, 0.7769965, 0.7863471, 0.7949274, 0.8092333, 0.8193017, 0.8240717, 0.8264560,\
        0.8275700, 0.8282843, 0.8284887, 0.8281349, 0.8273864, 0.8257821, 0.8230114, 0.8190336, 0.8139004,\
        0.8079153, 0.8017387, 0.7953356, 0.7875190, 0.7781359, 0.7679923, 0.7576190, 0.7473607, 0.7374350,\
        0.7279407, 0.7188641, 0.7100565, 0.7010466, 0.6916662, 0.6823970, 0.6728316, 0.6628695, 0.6526080,\
        0.6421839, 0.6317071, 0.6212305, 0.6107703, 0.6004257, 0.5894496, 0.5785436, 0.5678922, 0.5576327,\
        0.5479024, 0.5387878, 0.5303058, 0.5224096, 0.5150667, 0.5083899, 0.5022253, 0.4958078, 0.4890307,\
        0.4822401, 0.4754769, 0.4687166, 0.4619427, 0.4551494, 0.4483353, 0.4414995, 0.4346357, 0.4277272,\
        0.4207407, 0.4136307, 0.4063695, 0.3989982],\
        [0.4067441, 0.4148268, 0.4230619, 0.4313614, 0.4397175, 0.4481487, 0.4566830, 0.4653438, 0.4741204,\
        0.4829428, 0.4917841, 0.5009855, 0.5105125, 0.5199575, 0.5293209, 0.5386311, 0.5478924, 0.5570805,\
        0.5661493, 0.5750448, 0.5837171, 0.5921049, 0.5999511, 0.6068346, 0.6133081, 0.6194119, 0.6249759,\
        0.6298476, 0.6339085, 0.6370716, 0.6392764, 0.6404856, 0.6406787, 0.6398439, 0.6379747, 0.6350780,\
        0.6311987, 0.6264473, 0.6210445, 0.6154823, 0.6108093, 0.6077240, 0.6037826, 0.5970113, 0.5899153,\
        0.5846369, 0.5821521, 0.5834716, 0.5897455, 0.6017610, 0.6196666, 0.6440485, 0.6763442, 0.7133256,\
        0.7541082, 0.7864900, 0.8028317, 0.8090859, 0.8152111, 0.8270113, 0.8372754, 0.8429554, 0.8453581,\
        0.8457996, 0.8451036, 0.8436385, 0.8415274, 0.8387970, 0.8353027, 0.8308956, 0.8255487, 0.8193769,\
        0.8126954, 0.8059829, 0.7990992, 0.7894324, 0.7784281, 0.7669647, 0.7554696, 0.7442813, 0.7336291,\
        0.7236128, 0.7142270, 0.7054093, 0.6966828, 0.6871138, 0.6774721, 0.6677664, 0.6578724, 0.6477830,\
        0.6375561, 0.6272653, 0.6169517, 0.6064452, 0.5957895, 0.5849793, 0.5739784, 0.5631526, 0.5527336,\
        0.5428926, 0.5337450, 0.5253162, 0.5175021, 0.5100781, 0.5032008, 0.4972820, 0.4911195, 0.4846664,\
        0.4780892, 0.4714793, 0.4648548, 0.4582139, 0.4515543, 0.4448777, 0.4381884, 0.4314895, 0.4247744,\
        0.4180113, 0.4111303, 0.4040623, 0.3968479],\
        [0.4015541, 0.4096810, 0.4178847, 0.4261219, 0.4344232, 0.4428138, 0.4513269, 0.4599916, 0.4688164,\
        0.4778005, 0.4869996, 0.4965038, 0.5059789, 0.5153090, 0.5245954, 0.5338776, 0.5431699, 0.5524475,\
        0.5616565, 0.5707303, 0.5795882, 0.5881004, 0.5960299, 0.6033609, 0.6104665, 0.6171703, 0.6232824,\
        0.6286575, 0.6331852, 0.6367816, 0.6393863, 0.6409609, 0.6414849, 0.6409527, 0.6393740, 0.6367820,\
        0.6332516, 0.6289263, 0.6240620, 0.6190810, 0.6144713, 0.6101690, 0.6051112, 0.5991501, 0.5941807,\
        0.5904145, 0.5886957, 0.5901568, 0.5965423, 0.6093847, 0.6288308, 0.6550278, 0.6884005, 0.7335695,\
        0.7806074, 0.8145450, 0.8296145, 0.8365510, 0.8431144, 0.8517926, 0.8592979, 0.8633850, 0.8646942,\
        0.8638417, 0.8614823, 0.8581965, 0.8541601, 0.8494337, 0.8440288, 0.8379229, 0.8311243, 0.8237146,\
        0.8158357, 0.8075482, 0.7986816, 0.7884939, 0.7766384, 0.7641515, 0.7516615, 0.7396186, 0.7283130,\
        0.7178394, 0.7080971, 0.6989776, 0.6901613, 0.6809717, 0.6714720, 0.6618147, 0.6520099, 0.6420640,\
        0.6319968, 0.6218255, 0.6117059, 0.6016136, 0.5906901, 0.5796217, 0.5685042, 0.5575525, 0.5470068,\
        0.5370819, 0.5279479, 0.5196806, 0.5122052, 0.5051713, 0.4980670, 0.4919665, 0.4859961, 0.4798234,\
        0.4735070, 0.4671128, 0.4606726, 0.4541973, 0.4476916, 0.4411623, 0.4346212, 0.4280849, 0.4215676,\
        0.4150608, 0.4084899, 0.4016590, 0.3945491],\
        [0.3966629, 0.4046741, 0.4126683, 0.4207586, 0.4289394, 0.4372191, 0.4456461, 0.4542714, 0.4631249,\
        0.4722227, 0.4815771, 0.4911306, 0.5006446, 0.5099575, 0.5191837, 0.5284339, 0.5377577, 0.5471200,\
        0.5564472, 0.5656584, 0.5746574, 0.5833141, 0.5915002, 0.5993112, 0.6070130, 0.6143086, 0.6209606,\
        0.6268245, 0.6317997, 0.6358035, 0.6387747, 0.6406761, 0.6414914, 0.6412231, 0.6398948, 0.6375581,\
        0.6343060, 0.6302901, 0.6257362, 0.6209271, 0.6160823, 0.6111522, 0.6059497, 0.6009637, 0.5971304,\
        0.5948124, 0.5940663, 0.5957491, 0.6023941, 0.6162723, 0.6376416, 0.6674178, 0.7054378, 0.7528269,\
        0.8018098, 0.8368464, 0.8540644, 0.8627243, 0.8704410, 0.8784184, 0.8830734, 0.8840067, 0.8835804,\
        0.8808360, 0.8765241, 0.8714411, 0.8654707, 0.8587738, 0.8515120, 0.8437378, 0.8354624, 0.8267088,\
        0.8174972, 0.8077068, 0.7972137, 0.7858747, 0.7731911, 0.7597821, 0.7463744, 0.7335458, 0.7216844,\
        0.7109323, 0.7010344, 0.6917989, 0.6829095, 0.6738973, 0.6646158, 0.6551052, 0.6454195, 0.6356127,\
        0.6257271, 0.6156758, 0.6052103, 0.5953317, 0.5846992, 0.5735444, 0.5623295, 0.5512778, 0.5406204,\
        0.5306052, 0.5214769, 0.5133805, 0.5062789, 0.4999240, 0.4932855, 0.4868473, 0.4807817, 0.4747550,\
        0.4686587, 0.4624868, 0.4562510, 0.4499599, 0.4436197, 0.4372385, 0.4308321, 0.4244286, 0.4180654,\
        0.4117706, 0.4055193, 0.3990756, 0.3920176],\
        [0.3916772, 0.3994711, 0.4072904, 0.4152469, 0.4232659, 0.4313606, 0.4396328, 0.4481655, 0.4569930,\
        0.4661099, 0.4754759, 0.4849934, 0.4945017, 0.5038545, 0.5130474, 0.5222879, 0.5316843, 0.5411526,\
        0.5505964, 0.5599449, 0.5691248, 0.5780512, 0.5866694, 0.5950245, 0.6031766, 0.6109822, 0.6181023,\
        0.6244151, 0.6298170, 0.6342081, 0.6375228, 0.6397268, 0.6408094, 0.6407814, 0.6396764, 0.6375571,\
        0.6345228, 0.6307164, 0.6263229, 0.6215442, 0.6165437, 0.6114212, 0.6063584, 0.6018841, 0.5990479,\
        0.5982060, 0.5984379, 0.6006013, 0.6077026, 0.6225426, 0.6456717, 0.6776670, 0.7217825, 0.7742198,\
        0.8260769, 0.8620015, 0.8777530, 0.8863951, 0.8950094, 0.9028363, 0.9050714, 0.9028102, 0.9006616,\
        0.8952537, 0.8893689, 0.8827704, 0.8748594, 0.8662824, 0.8573152, 0.8480093, 0.8383445, 0.8282906,\
        0.8178598, 0.8072347, 0.7957920, 0.7826819, 0.7687129, 0.7542864, 0.7399587, 0.7263725, 0.7140602,\
        0.7030800, 0.6934052, 0.6842115, 0.6753272, 0.6664117, 0.6572760, 0.6478856, 0.6382868, 0.6285683,\
        0.6188591, 0.6093234, 0.5993830, 0.5886509, 0.5779520, 0.5668697, 0.5556585, 0.5445670, 0.5338255,\
        0.5236956, 0.5145165, 0.5065740, 0.4996707, 0.4935435, 0.4874721, 0.4813859, 0.4754432, 0.4695675,\
        0.4636686, 0.4577089, 0.4516794, 0.4455799, 0.4394116, 0.4331775, 0.4268885, 0.4205741, 0.4142880,\
        0.4080842, 0.4019497, 0.3956850, 0.3889904],\
        [0.3864564, 0.3940694, 0.4017606, 0.4096316, 0.4174105, 0.4252210, 0.4332729, 0.4416716, 0.4504327,\
        0.4595124, 0.4688290, 0.4782716, 0.4877165, 0.4970644, 0.5061975, 0.5154973, 0.5250551, 0.5346461,\
        0.5441909, 0.5536739, 0.5630666, 0.5723225, 0.5814053, 0.5903042, 0.5989921, 0.6072135, 0.6146976,\
        0.6214513, 0.6272802, 0.6320484, 0.6356934, 0.6381889, 0.6395300, 0.6397331, 0.6388385, 0.6369148,\
        0.6340613, 0.6304101, 0.6261203, 0.6213648, 0.6163190, 0.6111874, 0.6063013, 0.6022596, 0.6001078,\
        0.6000299, 0.6001845, 0.6027369, 0.6108675, 0.6272848, 0.6532922, 0.6890884, 0.7365612, 0.7956440,\
        0.8545153, 0.8935154, 0.9035950, 0.9110772, 0.9182467, 0.9230175, 0.9231095, 0.9193268, 0.9138745,\
        0.9067044, 0.8999818, 0.8914402, 0.8816124, 0.8714477, 0.8610721, 0.8504834, 0.8396300, 0.8284232,\
        0.8167429, 0.8044130, 0.7921411, 0.7783033, 0.7634098, 0.7481003, 0.7329071, 0.7185273, 0.7059245,\
        0.6951011, 0.6854455, 0.6764530, 0.6676646, 0.6588236, 0.6497632, 0.6404223, 0.6308345, 0.6211010,\
        0.6113363, 0.6017683, 0.5923625, 0.5818213, 0.5709348, 0.5598759, 0.5487367, 0.5376973, 0.5269601,\
        0.5167499, 0.5074167, 0.4996592, 0.4931114, 0.4871588, 0.4814142, 0.4756925, 0.4700008, 0.4643254,\
        0.4586251, 0.4528688, 0.4470412, 0.4411356, 0.4351469, 0.4290686, 0.4228960, 0.4166435, 0.4103758,\
        0.4041993, 0.3981439, 0.3920557, 0.3856634],\
        [0.3811370, 0.3886508, 0.3962965, 0.4039745, 0.4113145, 0.4187516, 0.4265592, 0.4348238, 0.4435198,\
        0.4525540, 0.4618088, 0.4711631, 0.4805011, 0.4897171, 0.4988002, 0.5083298, 0.5180913, 0.5277239,\
        0.5372974, 0.5468799, 0.5564783, 0.5660568, 0.5755666, 0.5849557, 0.5941107, 0.6026557, 0.6106420,\
        0.6179403, 0.6242142, 0.6293505, 0.6333223, 0.6361140, 0.6377216, 0.6381630, 0.6374826, 0.6357521,\
        0.6330706, 0.6295633, 0.6253774, 0.6206779, 0.6156529, 0.6105463, 0.6057262, 0.6017577, 0.5992974,\
        0.5982522, 0.5982085, 0.6013569, 0.6107336, 0.6290070, 0.6580495, 0.6984213, 0.7488848, 0.8098458,\
        0.8733349, 0.9162866, 0.9306931, 0.9368688, 0.9404839, 0.9412493, 0.9383507, 0.9322684, 0.9242423,\
        0.9152947, 0.9061486, 0.8959988, 0.8851429, 0.8739621, 0.8625702, 0.8510081, 0.8392442, 0.8271798,\
        0.8146533, 0.8016007, 0.7877441, 0.7730895, 0.7575912, 0.7416973, 0.7259150, 0.7108262, 0.6977081,\
        0.6872904, 0.6778507, 0.6689273, 0.6601780, 0.6513594, 0.6423119, 0.6329534, 0.6232991, 0.6134976,\
        0.6037003, 0.5939743, 0.5843943, 0.5742705, 0.5636388, 0.5527524, 0.5417858, 0.5309217, 0.5203615,\
        0.5103156, 0.5010539, 0.4933361, 0.4869585, 0.4811363, 0.4755612, 0.4700676, 0.4645952, 0.4591168,\
        0.4536060, 0.4480404, 0.4424052, 0.4366927, 0.4308954, 0.4250000, 0.4189807, 0.4128142, 0.4065480,\
        0.4004101, 0.3944930, 0.3885289, 0.3823185],\
        [0.3756969, 0.3830594, 0.3905090, 0.3978184, 0.4047761, 0.4119174, 0.4195314, 0.4277132, 0.4363866,\
        0.4453986, 0.4546037, 0.4638854, 0.4731568, 0.4823784, 0.4916636, 0.5013417, 0.5109794, 0.5204380,\
        0.5299331, 0.5395654, 0.5493415, 0.5592036, 0.5690694, 0.5788365, 0.5883322, 0.5973453, 0.6060528,\
        0.6139411, 0.6206103, 0.6260946, 0.6304102, 0.6335274, 0.6354301, 0.6361350, 0.6356901, 0.6341707,\
        0.6316770, 0.6283327, 0.6242817, 0.6196845, 0.6147228, 0.6096181, 0.6046659, 0.6002496, 0.5967482,\
        0.5944144, 0.5939526, 0.5973979, 0.6076078, 0.6272664, 0.6580962, 0.7007024, 0.7545151, 0.8182681,\
        0.8820474, 0.9269456, 0.9488494, 0.9572033, 0.9589852, 0.9565391, 0.9505635, 0.9417843, 0.9312231,\
        0.9199254, 0.9085532, 0.8971497, 0.8857643, 0.8739051, 0.8617907, 0.8495293, 0.8371181, 0.8244990,\
        0.8114742, 0.7976442, 0.7829979, 0.7675656, 0.7515873, 0.7354213, 0.7195618, 0.7046056, 0.6914263,\
        0.6805447, 0.6709219, 0.6618757, 0.6530345, 0.6441527, 0.6350664, 0.6256625, 0.6158855, 0.6059186,\
        0.5961597, 0.5864433, 0.5766934, 0.5666723, 0.5562957, 0.5456737, 0.5349710, 0.5243902, 0.5141669,\
        0.5045414, 0.4957449, 0.4880684, 0.4815116, 0.4756245, 0.4700773, 0.4646854, 0.4593527, 0.4540281,\
        0.4486789, 0.4432817, 0.4378225, 0.4322951, 0.4266973, 0.4210217, 0.4152367, 0.4092668, 0.4030920,\
        0.3970068, 0.3911480, 0.3852194, 0.3790834],\
        [0.3701235, 0.3772245, 0.3842911, 0.3911889, 0.3979289, 0.4048674, 0.4123348, 0.4205163, 0.4292248,\
        0.4382346, 0.4473902, 0.4566035, 0.4658339, 0.4750995, 0.4845028, 0.4940874, 0.5034729, 0.5127061,\
        0.5220776, 0.5317371, 0.5416696, 0.5517702, 0.5619238, 0.5720353, 0.5820057, 0.5917471, 0.6011084,\
        0.6093994, 0.6163267, 0.6221916, 0.6269258, 0.6304335, 0.6326822, 0.6336933, 0.6335226, 0.6322500,\
        0.6299788, 0.6268359, 0.6229685, 0.6185364, 0.6137042, 0.6086394, 0.6035229, 0.5985691, 0.5940714,\
        0.5905684, 0.5892537, 0.5923185, 0.6027323, 0.6235590, 0.6568264, 0.7022587, 0.7606797, 0.8276255,\
        0.8922626, 0.9369833, 0.9614075, 0.9713389, 0.9723500, 0.9677219, 0.9591611, 0.9478218, 0.9347713,\
        0.9211187, 0.9082827, 0.8965330, 0.8840689, 0.8714944, 0.8588286, 0.8460831, 0.8332078, 0.8201556,\
        0.8070249, 0.7928379, 0.7775191, 0.7616438, 0.7454388, 0.7292497, 0.7135815, 0.6990716, 0.6860217,\
        0.6746914, 0.6646452, 0.6552997, 0.6462400, 0.6372011, 0.6280292, 0.6186150, 0.6088167, 0.5986302,\
        0.5887494, 0.5789960, 0.5692013, 0.5592606, 0.5491125, 0.5387915, 0.5284062, 0.5181546, 0.5083254,\
        0.4991431, 0.4907475, 0.4832368, 0.4765873, 0.4705819, 0.4649786, 0.4596008, 0.4543361, 0.4491146,\
        0.4438907, 0.4386339, 0.4333264, 0.4279634, 0.4225522, 0.4171062, 0.4116304, 0.4060505, 0.4001085,\
        0.3940826, 0.3881146, 0.3821142, 0.3759724],\
        [0.3645023, 0.3713304, 0.3780742, 0.3846878, 0.3912323, 0.3979178, 0.4052774, 0.4135242, 0.4222818,\
        0.4312555, 0.4403108, 0.4494085, 0.4585415, 0.4677269, 0.4769895, 0.4862773, 0.4954501, 0.5045053,\
        0.5137687, 0.5234642, 0.5335442, 0.5438238, 0.5541567, 0.5645062, 0.5748613, 0.5851568, 0.5951267,\
        0.6036617, 0.6110504, 0.6175353, 0.6228389, 0.6268292, 0.6294890, 0.6308634, 0.6310222, 0.6300485,\
        0.6280475, 0.6251532, 0.6215259, 0.6173375, 0.6127479, 0.6078797, 0.6028079, 0.5975997, 0.5924564,\
        0.5879960, 0.5856199, 0.5877372, 0.5976556, 0.6192119, 0.6563355, 0.7102830, 0.7746324, 0.8478147,\
        0.9087690, 0.9483688, 0.9710964, 0.9805132, 0.9806159, 0.9743899, 0.9638460, 0.9504207, 0.9352945,\
        0.9195574, 0.9041794, 0.8922240, 0.8797831, 0.8668311, 0.8538153, 0.8407992, 0.8276797, 0.8141448,\
        0.8002284, 0.7863037, 0.7710885, 0.7552010, 0.7390484, 0.7230171, 0.7075531, 0.6932936, 0.6804985,\
        0.6690329, 0.6586666, 0.6490026, 0.6396610, 0.6303711, 0.6210163, 0.6115789, 0.6018776, 0.5916833,\
        0.5815292, 0.5716288, 0.5618348, 0.5520292, 0.5421482, 0.5321893, 0.5221868, 0.5122488, 0.5028044,\
        0.4940606, 0.4860034, 0.4786478, 0.4719765, 0.4658792, 0.4601994, 0.4547936, 0.4495517, 0.4443944,\
        0.4392648, 0.4341217, 0.4289387, 0.4237074, 0.4184411, 0.4131682, 0.4079260, 0.4027890, 0.3972852,\
        0.3912073, 0.3851555, 0.3790921, 0.3729298],\
        [0.3589517, 0.3655665, 0.3721008, 0.3785575, 0.3850117, 0.3916624, 0.3989182, 0.4070974, 0.4158119,\
        0.4246139, 0.4334538, 0.4423513, 0.4513032, 0.4602999, 0.4693220, 0.4783168, 0.4872220, 0.4960810,\
        0.5052353, 0.5149289, 0.5251550, 0.5355011, 0.5458004, 0.5561581, 0.5666485, 0.5771571, 0.5872737,\
        0.5964245, 0.6047778, 0.6121509, 0.6181534, 0.6227080, 0.6258423, 0.6276504, 0.6282150, 0.6276101,\
        0.6259355, 0.6233351, 0.6199950, 0.6161224, 0.6119087, 0.6074759, 0.6028310, 0.5978930, 0.5926810,\
        0.5877011, 0.5843987, 0.5853916, 0.5943578, 0.6156677, 0.6536728, 0.7130689, 0.7904834, 0.8655661,\
        0.9234795, 0.9578038, 0.9772164, 0.9850968, 0.9840834, 0.9765961, 0.9646095, 0.9497014, 0.9331949,\
        0.9163631, 0.9011122, 0.8869385, 0.8735012, 0.8601856, 0.8469281, 0.8338046, 0.8208275, 0.8075534,\
        0.7933916, 0.7789874, 0.7639325, 0.7482540, 0.7323306, 0.7166312, 0.7015089, 0.6874895, 0.6747525,\
        0.6632027, 0.6526311, 0.6427172, 0.6331054, 0.6234946, 0.6137539, 0.6040917, 0.5942921, 0.5841310,\
        0.5740161, 0.5641404, 0.5544658, 0.5448898, 0.5353471, 0.5258442, 0.5164068, 0.5069026, 0.4976245,\
        0.4893277, 0.4814987, 0.4742360, 0.4675548, 0.4613937, 0.4556481, 0.4502061, 0.4449689, 0.4398560,\
        0.4348031, 0.4297579, 0.4246797, 0.4195471, 0.4143731, 0.4091997, 0.4040281, 0.3988750, 0.3936794,\
        0.3879758, 0.3820529, 0.3760286, 0.3698867],\
        [0.3535589, 0.3600348, 0.3664622, 0.3728779, 0.3793794, 0.3861523, 0.3934559, 0.4015128, 0.4099372,\
        0.4183283, 0.4268248, 0.4354559, 0.4441717, 0.4529305, 0.4617013, 0.4704517, 0.4791657, 0.4879056,\
        0.4968536, 0.5064371, 0.5168611, 0.5269878, 0.5368855, 0.5469659, 0.5573991, 0.5680372, 0.5785207,\
        0.5885061, 0.5978583, 0.6060769, 0.6128621, 0.6180350, 0.6216979, 0.6240379, 0.6251197, 0.6249760,\
        0.6236899, 0.6214174, 0.6183808, 0.6148484, 0.6110917, 0.6073067, 0.6035079, 0.5994836, 0.5949770,\
        0.5902110, 0.5865836, 0.5869755, 0.5954904, 0.6174169, 0.6588021, 0.7266850, 0.8097923, 0.8769874,\
        0.9275066, 0.9602483, 0.9782631, 0.9850061, 0.9830329, 0.9745724, 0.9615736, 0.9457029, 0.9284296,\
        0.9109542, 0.8944092, 0.8796897, 0.8655952, 0.8518734, 0.8383907, 0.8251332, 0.8122635, 0.7998916,\
        0.7858807, 0.7712922, 0.7563121, 0.7408884, 0.7252195, 0.7097538, 0.6951793, 0.6812920, 0.6685535,\
        0.6569323, 0.6462374, 0.6361677, 0.6263681, 0.6164801, 0.6062693, 0.5959740, 0.5860115, 0.5759429,\
        0.5660030, 0.5563472, 0.5469605, 0.5377351, 0.5285862, 0.5195540, 0.5108018, 0.5024516, 0.4934752,\
        0.4849685, 0.4771447, 0.4699066, 0.4632173, 0.4570197, 0.4512333, 0.4457682, 0.4405383, 0.4354675,\
        0.4304895, 0.4255438, 0.4205716, 0.4155239, 0.4103962, 0.4052815, 0.4002444, 0.3951530, 0.3899801,\
        0.3845226, 0.3787966, 0.3728830, 0.3668061],\
        [0.3483648, 0.3547625, 0.3611512, 0.3675878, 0.3741761, 0.3810737, 0.3884620, 0.3964051, 0.4043674,\
        0.4121915, 0.4203566, 0.4287326, 0.4371977, 0.4457026, 0.4542343, 0.4627938, 0.4714035, 0.4801363,\
        0.4891482, 0.4988920, 0.5090710, 0.5183068, 0.5273333, 0.5369206, 0.5472478, 0.5580642, 0.5690066,\
        0.5797431, 0.5899477, 0.5991605, 0.6069168, 0.6126900, 0.6169518, 0.6200005, 0.6217657, 0.6222002,\
        0.6213701, 0.6194473, 0.6166911, 0.6134375, 0.6100759, 0.6069643, 0.6042465, 0.6016643, 0.5986624,\
        0.5950572, 0.5921945, 0.5934779, 0.6032207, 0.6271035, 0.6755238, 0.7493371, 0.8250759, 0.8845674,\
        0.9284951, 0.9580184, 0.9748301, 0.9807185, 0.9779252, 0.9686702, 0.9549444, 0.9384265, 0.9206461,\
        0.9032085, 0.8862521, 0.8705623, 0.8560410, 0.8421076, 0.8285096, 0.8152094, 0.8020089, 0.7896041,\
        0.7765980, 0.7626609, 0.7481485, 0.7331414, 0.7178706, 0.7025275, 0.6879722, 0.6743376, 0.6616173,\
        0.6499517, 0.6392022, 0.6290541, 0.6191594, 0.6091260, 0.5985813, 0.5878048, 0.5773375, 0.5672067,\
        0.5574591, 0.5481528, 0.5392278, 0.5305039, 0.5217902, 0.5131055, 0.5047035, 0.4967099, 0.4885958,\
        0.4804062, 0.4726912, 0.4655166, 0.4588566, 0.4526633, 0.4468722, 0.4414104, 0.4362039, 0.4311845,\
        0.4262905, 0.4214620, 0.4166278, 0.4116996, 0.4066090, 0.4014454, 0.3965220, 0.3915442, 0.3864163,\
        0.3810697, 0.3754832, 0.3696827, 0.3636876],\
        [0.3433757, 0.3497325, 0.3561122, 0.3625755, 0.3692109, 0.3761251, 0.3833955, 0.3909458, 0.3984020,\
        0.4060189, 0.4140535, 0.4222204, 0.4304217, 0.4386512, 0.4469388, 0.4553198, 0.4638427, 0.4726020,\
        0.4817769, 0.4915213, 0.5008952, 0.5089911, 0.5168992, 0.5260516, 0.5363838, 0.5474231, 0.5587809,\
        0.5701262, 0.5811028, 0.5912987, 0.6000668, 0.6063206, 0.6114922, 0.6155655, 0.6182192, 0.6193568,\
        0.6190557, 0.6175116, 0.6150013, 0.6118926, 0.6086726, 0.6059129, 0.6040099, 0.6027687, 0.6014790,\
        0.5998008, 0.5992080, 0.6042935, 0.6191813, 0.6465643, 0.6989820, 0.7839204, 0.8383253, 0.8868938,\
        0.9252974, 0.9529005, 0.9684573, 0.9731997, 0.9693939, 0.9593145, 0.9450232, 0.9281624, 0.9097726,\
        0.8917845, 0.8749261, 0.8592353, 0.8448774, 0.8311559, 0.8174650, 0.8047845, 0.7917963, 0.7790936,\
        0.7663979, 0.7531956, 0.7394371, 0.7250123, 0.7099927, 0.6953426, 0.6805747, 0.6666244, 0.6537419,\
        0.6420154, 0.6312792, 0.6210812, 0.6110282, 0.6009048, 0.5899383, 0.5788151, 0.5680563, 0.5578672,\
        0.5483339, 0.5394765, 0.5311920, 0.5231982, 0.5150577, 0.5066089, 0.4984760, 0.4907432, 0.4830629,\
        0.4753822, 0.4679594, 0.4609482, 0.4543842, 0.4482486, 0.4424973, 0.4370727, 0.4319129, 0.4269579,\
        0.4221558, 0.4174616, 0.4128141, 0.4080967, 0.4031701, 0.3980486, 0.3930573, 0.3880790, 0.3829718,\
        0.3776799, 0.3721782, 0.3664656, 0.3605473],\
        [0.3385795, 0.3449125, 0.3512866, 0.3577551, 0.3643811, 0.3712210, 0.3782845, 0.3854855, 0.3927214,\
        0.4002334, 0.4080826, 0.4159845, 0.4238642, 0.4317680, 0.4397797, 0.4479486, 0.4563060, 0.4648966,\
        0.4737543, 0.4827214, 0.4912030, 0.4984577, 0.5055823, 0.5147412, 0.5252009, 0.5364012, 0.5480314,\
        0.5598196, 0.5713732, 0.5821694, 0.5914732, 0.5988971, 0.6055475, 0.6108840, 0.6145741, 0.6165088,\
        0.6168215, 0.6157403, 0.6135174, 0.6104679, 0.6070530, 0.6039864, 0.6019235, 0.6005141, 0.5984488,\
        0.5976998, 0.6020200, 0.6166079, 0.6429369, 0.6802716, 0.7371003, 0.8090436, 0.8531659, 0.8889065,\
        0.9222267, 0.9475096, 0.9606263, 0.9632638, 0.9579844, 0.9469219, 0.9319878, 0.9155080, 0.8970940,\
        0.8784595, 0.8611055, 0.8455668, 0.8320922, 0.8195129, 0.8060263, 0.7927235, 0.7805277, 0.7680094,\
        0.7555790, 0.7430073, 0.7301426, 0.7167714, 0.7019221, 0.6869282, 0.6721639, 0.6579064, 0.6447235,\
        0.6328919, 0.6222996, 0.6121426, 0.6017044, 0.5910880, 0.5800257, 0.5688145, 0.5579769, 0.5478369,\
        0.5385673, 0.5302210, 0.5227101, 0.5157386, 0.5086098, 0.5004793, 0.4924996, 0.4848934, 0.4774220,\
        0.4700721, 0.4629493, 0.4561591, 0.4497506, 0.4437264, 0.4380607, 0.4327111, 0.4276252, 0.4227472,\
        0.4180310, 0.4134557, 0.4090106, 0.4045689, 0.3998689, 0.3948501, 0.3897894, 0.3847431, 0.3796198,\
        0.3743517, 0.3689018, 0.3632536, 0.3574000],\
        [0.3339592, 0.3402730, 0.3466344, 0.3530852, 0.3596671, 0.3664086, 0.3733080, 0.3803356, 0.3874983,\
        0.3948851, 0.4024519, 0.4100284, 0.4174959, 0.4250062, 0.4327152, 0.4406360, 0.4487355, 0.4569738,\
        0.4652668, 0.4733948, 0.4809830, 0.4878490, 0.4950076, 0.5039628, 0.5142889, 0.5253636, 0.5370193,\
        0.5490998, 0.5611017, 0.5724464, 0.5826357, 0.5915721, 0.5994593, 0.6060716, 0.6108466, 0.6136169,\
        0.6146642, 0.6142529, 0.6125556, 0.6097625, 0.6061533, 0.6022158, 0.5991616, 0.5951789, 0.5891635,\
        0.5855488, 0.5977012, 0.6295013, 0.6696252, 0.7114631, 0.7573462, 0.8056341, 0.8517214, 0.8912557,\
        0.9224750, 0.9428481, 0.9515151, 0.9508620, 0.9438445, 0.9320995, 0.9161293, 0.8990633, 0.8819491,\
        0.8629597, 0.8451266, 0.8294984, 0.8173527, 0.8068219, 0.7946220, 0.7811074, 0.7683064, 0.7559635,\
        0.7439556, 0.7319788, 0.7197244, 0.7076758, 0.6942469, 0.6783248, 0.6628235, 0.6480800, 0.6344684,\
        0.6223777, 0.6120192, 0.6024144, 0.5915233, 0.5804671, 0.5692392, 0.5580167, 0.5471992, 0.5371785,\
        0.5281977, 0.5203456, 0.5135741, 0.5076083, 0.5020089, 0.4945884, 0.4865871, 0.4789703, 0.4716425,\
        0.4645443, 0.4576979, 0.4511525, 0.4449419, 0.4390743, 0.4335363, 0.4282991, 0.4233196, 0.4185387,\
        0.4138961, 0.4093716, 0.4050351, 0.4008922, 0.3963182, 0.3914473, 0.3864610, 0.3814230, 0.3763051,\
        0.3710596, 0.3656498, 0.3600526, 0.3542537],\
        [0.3294998, 0.3357938, 0.3421343, 0.3485528, 0.3550770, 0.3617248, 0.3685004, 0.3754042, 0.3824545,\
        0.3896738, 0.3970101, 0.4042919, 0.4111939, 0.4182941, 0.4257421, 0.4334257, 0.4412346, 0.4490705,\
        0.4568192, 0.4643246, 0.4714535, 0.4783503, 0.4857029, 0.4942489, 0.5041374, 0.5145227, 0.5259832,\
        0.5383210, 0.5507193, 0.5626159, 0.5736841, 0.5838263, 0.5930131, 0.6009602, 0.6067838, 0.6103992,\
        0.6124068, 0.6130495, 0.6123575, 0.6104439, 0.6075051, 0.6038685, 0.6000941, 0.5957343, 0.5885840,\
        0.5793777, 0.5961146, 0.6508118, 0.6980708, 0.7342618, 0.7681222, 0.8039470, 0.8428156, 0.8849601,\
        0.9186837, 0.9358324, 0.9393047, 0.9348644, 0.9263656, 0.9155333, 0.8997586, 0.8816381, 0.8637006,\
        0.8452663, 0.8276841, 0.8118619, 0.8001454, 0.7915835, 0.7802931, 0.7677592, 0.7550993, 0.7428890,\
        0.7314422, 0.7203335, 0.7081700, 0.6956805, 0.6828632, 0.6678336, 0.6522835, 0.6372692, 0.6232795,\
        0.6106887, 0.6000895, 0.5907401, 0.5802815, 0.5692432, 0.5580230, 0.5468247, 0.5360309, 0.5261496,\
        0.5174828, 0.5100335, 0.5038822, 0.4984946, 0.4933288, 0.4870411, 0.4798343, 0.4726020, 0.4655712,\
        0.4587713, 0.4522206, 0.4459457, 0.4399674, 0.4342927, 0.4289162, 0.4238241, 0.4189891, 0.4143499,\
        0.4098061, 0.4052947, 0.4008667, 0.3967957, 0.3924906, 0.3878446, 0.3830094, 0.3780532, 0.3729833,\
        0.3677780, 0.3624112, 0.3568610, 0.3511111],\
        [0.3251914, 0.3314635, 0.3377774, 0.3441560, 0.3506180, 0.3571757, 0.3638357, 0.3706028, 0.3774835,\
        0.3844727, 0.3915206, 0.3983395, 0.4047123, 0.4116425, 0.4189511, 0.4264447, 0.4339799, 0.4414456,\
        0.4487578, 0.4558547, 0.4627543, 0.4696680, 0.4770292, 0.4852574, 0.4943243, 0.5036997, 0.5153246,\
        0.5279082, 0.5405227, 0.5527513, 0.5643965, 0.5753713, 0.5855628, 0.5946152, 0.6017209, 0.6062282,\
        0.6097102, 0.6119231, 0.6128066, 0.6125774, 0.6115793, 0.6102871, 0.6091742, 0.6089132, 0.6079342,\
        0.6057818, 0.6348640, 0.6953836, 0.7289994, 0.7500512, 0.7689975, 0.7906162, 0.8242995, 0.8695278,\
        0.9076961, 0.9228924, 0.9214204, 0.9142443, 0.9049053, 0.8951199, 0.8814463, 0.8632581, 0.8448088,\
        0.8266048, 0.8098348, 0.7955231, 0.7837823, 0.7746819, 0.7645731, 0.7531311, 0.7409772, 0.7288445,\
        0.7177810, 0.7085306, 0.6970059, 0.6839873, 0.6705661, 0.6560302, 0.6408009, 0.6258339, 0.6120755,\
        0.5990915, 0.5883533, 0.5785453, 0.5684143, 0.5577625, 0.5468088, 0.5357441, 0.5249265, 0.5151482,\
        0.5069712, 0.4999238, 0.4942102, 0.4892225, 0.4842199, 0.4785957, 0.4722910, 0.4657147, 0.4591576,\
        0.4527460, 0.4465379, 0.4405673, 0.4348536, 0.4294023, 0.4242110, 0.4192806, 0.4146185, 0.4101935,\
        0.4058377, 0.4014225, 0.3970692, 0.3928817, 0.3886413, 0.3841670, 0.3794838, 0.3746368, 0.3696424,\
        0.3644949, 0.3591786, 0.3536765, 0.3479733],\
        [0.3210291, 0.3272780, 0.3335625, 0.3398978, 0.3462954, 0.3527625, 0.3593018, 0.3659116, 0.3725804,\
        0.3792707, 0.3858811, 0.3922275, 0.3985174, 0.4053650, 0.4125362, 0.4198609, 0.4271369, 0.4342674,\
        0.4412268, 0.4480298, 0.4547604, 0.4616101, 0.4688505, 0.4766913, 0.4851354, 0.4945827, 0.5059413,\
        0.5182359, 0.5306531, 0.5428061, 0.5546108, 0.5659802, 0.5767243, 0.5864623, 0.5944829, 0.6007163,\
        0.6063555, 0.6104186, 0.6132703, 0.6152886, 0.6170557, 0.6194350, 0.6234395, 0.6300284, 0.6400145,\
        0.6589370, 0.6966060, 0.7317790, 0.7482134, 0.7567260, 0.7650930, 0.7783022, 0.8045644, 0.8449616,\
        0.8832874, 0.9002272, 0.8971055, 0.8902446, 0.8818430, 0.8719481, 0.8589907, 0.8424675, 0.8250794,\
        0.8082557, 0.7920394, 0.7790873, 0.7680982, 0.7585850, 0.7490086, 0.7384458, 0.7268375, 0.7147124,\
        0.7031155, 0.6947284, 0.6845705, 0.6717647, 0.6581879, 0.6439679, 0.6291838, 0.6140964, 0.6009518,\
        0.5888343, 0.5774487, 0.5670570, 0.5568726, 0.5465571, 0.5360662, 0.5253946, 0.5146056, 0.5046288,\
        0.4971517, 0.4907448, 0.4852191, 0.4802890, 0.4754028, 0.4701704, 0.4644960, 0.4585396, 0.4525036,\
        0.4465288, 0.4406989, 0.4350624, 0.4296432, 0.4244446, 0.4194566, 0.4146808, 0.4101653, 0.4059947,\
        0.4020140, 0.3976859, 0.3933961, 0.3891699, 0.3849168, 0.3805225, 0.3759502, 0.3712049, 0.3662929,\
        0.3612117, 0.3559514, 0.3504991, 0.3448417],\
        [0.3170124, 0.3232386, 0.3294937, 0.3357854, 0.3421183, 0.3484943, 0.3549115, 0.3613612, 0.3678225,\
        0.3742530, 0.3805885, 0.3868084, 0.3930954, 0.3996765, 0.4066704, 0.4138211, 0.4208030, 0.4275931,\
        0.4342346, 0.4407857, 0.4473490, 0.4540875, 0.4611982, 0.4688612, 0.4772630, 0.4867567, 0.4975385,\
        0.5092618, 0.5210186, 0.5326373, 0.5442232, 0.5556338, 0.5666208, 0.5768737, 0.5860969, 0.5944490,\
        0.6017746, 0.6076952, 0.6126917, 0.6170563, 0.6215853, 0.6276210, 0.6366439, 0.6498955, 0.6684221,\
        0.6937534, 0.7213479, 0.7403040, 0.7486051, 0.7519298, 0.7563211, 0.7666757, 0.7864363, 0.8156134,\
        0.8500652, 0.8685184, 0.8698229, 0.8646283, 0.8575050, 0.8483462, 0.8361868, 0.8208526, 0.8041988,\
        0.7898615, 0.7753185, 0.7628212, 0.7523449, 0.7430971, 0.7341189, 0.7244453, 0.7135143, 0.7017959,\
        0.6906121, 0.6808930, 0.6706845, 0.6586288, 0.6454650, 0.6318433, 0.6181133, 0.6040927, 0.5901095,\
        0.5781817, 0.5668124, 0.5560957, 0.5458343, 0.5358398, 0.5259988, 0.5161620, 0.5062277, 0.4962958,\
        0.4886731, 0.4824296, 0.4769427, 0.4719427, 0.4670797, 0.4620726, 0.4568103, 0.4513385, 0.4457724,\
        0.4402240, 0.4347776, 0.4294889, 0.4243877, 0.4194765, 0.4147251, 0.4100997, 0.4056517, 0.4015516,\
        0.3978926, 0.3938692, 0.3896848, 0.3855007, 0.3812737, 0.3769362, 0.3724457, 0.3677862, 0.3629520,\
        0.3579376, 0.3527345, 0.3473321, 0.3417192],\
        [0.3131460, 0.3193518, 0.3255802, 0.3318300, 0.3380993, 0.3443867, 0.3506879, 0.3569932, 0.3632844,\
        0.3695368, 0.3757341, 0.3819095, 0.3881754, 0.3946661, 0.4015112, 0.4083692, 0.4149466, 0.4213685,\
        0.4277074, 0.4340185, 0.4404016, 0.4470022, 0.4539902, 0.4615446, 0.4698651, 0.4791636, 0.4895082,\
        0.5005432, 0.5112036, 0.5220042, 0.5331944, 0.5444238, 0.5554344, 0.5660721, 0.5762357, 0.5858798,\
        0.5947768, 0.6027402, 0.6099443, 0.6162339, 0.6226780, 0.6311461, 0.6434722, 0.6607657, 0.6824900,\
        0.7046819, 0.7213207, 0.7303292, 0.7345025, 0.7368997, 0.7412747, 0.7510604, 0.7681526, 0.7937846,\
        0.8229680, 0.8389096, 0.8404236, 0.8368580, 0.8321049, 0.8246742, 0.8139318, 0.8003736, 0.7849196,\
        0.7708888, 0.7579031, 0.7463798, 0.7365722, 0.7278240, 0.7196201, 0.7111032, 0.7008867, 0.6897467,\
        0.6787783, 0.6682682, 0.6573733, 0.6453563, 0.6324332, 0.6192818, 0.6064705, 0.5937877, 0.5803795,\
        0.5678391, 0.5562764, 0.5454105, 0.5351286, 0.5254459, 0.5163527, 0.5074833, 0.4985415, 0.4895360,\
        0.4815535, 0.4749786, 0.4693139, 0.4641846, 0.4592898, 0.4544027, 0.4494067, 0.4442910, 0.4391103,\
        0.4339412, 0.4288554, 0.4239073, 0.4191322, 0.4145444, 0.4101058, 0.4056830, 0.4013408, 0.3973044,\
        0.3936288, 0.3898576, 0.3858940, 0.3818276, 0.3776730, 0.3734018, 0.3689837, 0.3643986, 0.3596349,\
        0.3546833, 0.3495352, 0.3441806, 0.3386096],\
        [0.3094394, 0.3156302, 0.3218368, 0.3280458, 0.3342519, 0.3404541, 0.3466491, 0.3528302, 0.3589879,\
        0.3651163, 0.3712263, 0.3773640, 0.3836153, 0.3900778, 0.3967620, 0.4032365, 0.4093550, 0.4154574,\
        0.4215327, 0.4276093, 0.4337914, 0.4402207, 0.4470481, 0.4544256, 0.4625000, 0.4713740, 0.4810012,\
        0.4909761, 0.5006161, 0.5108431, 0.5216583, 0.5325026, 0.5432524, 0.5539699, 0.5646213, 0.5750987,\
        0.5852524, 0.5950153, 0.6040787, 0.6113612, 0.6184365, 0.6275193, 0.6407151, 0.6588888, 0.6804271,\
        0.6992764, 0.7067916, 0.7087941, 0.7116861, 0.7149702, 0.7209884, 0.7322125, 0.7506962, 0.7764850,\
        0.8031950, 0.8124404, 0.8104004, 0.8079365, 0.8066388, 0.8011301, 0.7918729, 0.7798627, 0.7662833,\
        0.7528642, 0.7404376, 0.7296992, 0.7208602, 0.7126502, 0.7047676, 0.6973893, 0.6879251, 0.6773936,\
        0.6667210, 0.6559657, 0.6446528, 0.6323800, 0.6193471, 0.6062126, 0.5938153, 0.5816249, 0.5692473,\
        0.5571623, 0.5457240, 0.5348946, 0.5246173, 0.5150807, 0.5067083, 0.4986550, 0.4903314, 0.4821162,\
        0.4745473, 0.4679380, 0.4621507, 0.4569168, 0.4519872, 0.4471709, 0.4423541, 0.4374988, 0.4326225,\
        0.4277732, 0.4230080, 0.4183758, 0.4139073, 0.4096364, 0.4056033, 0.4015744, 0.3973305, 0.3933965,\
        0.3896644, 0.3859478, 0.3821227, 0.3781773, 0.3741135, 0.3699179, 0.3655704, 0.3610533, 0.3563529,\
        0.3514587, 0.3463611, 0.3410506, 0.3355176],\
        [0.3059102, 0.3120961, 0.3182843, 0.3244477, 0.3305865, 0.3367045, 0.3428011, 0.3488724, 0.3549162,\
        0.3609377, 0.3669580, 0.3730187, 0.3791752, 0.3854604, 0.3917935, 0.3978940, 0.4038205, 0.4097490,\
        0.4156095, 0.4214416, 0.4273931, 0.4336119, 0.4402207, 0.4473271, 0.4550137, 0.4632985, 0.4720685,\
        0.4810435, 0.4900956, 0.4997035, 0.5099993, 0.5200435, 0.5301572, 0.5406212, 0.5513555, 0.5621823,\
        0.5729541, 0.5835189, 0.5932768, 0.6011219, 0.6080845, 0.6161502, 0.6280830, 0.6446706, 0.6626471,\
        0.6765625, 0.6822979, 0.6835772, 0.6861468, 0.6902574, 0.6978836, 0.7095320, 0.7286476, 0.7525342,\
        0.7742122, 0.7832388, 0.7826596, 0.7815920, 0.7813204, 0.7776637, 0.7701623, 0.7598012, 0.7478620,\
        0.7355301, 0.7234163, 0.7129780, 0.7052456, 0.6982375, 0.6899514, 0.6824655, 0.6738233, 0.6641879,\
        0.6540822, 0.6435782, 0.6322770, 0.6197899, 0.6065676, 0.5934800, 0.5811308, 0.5692450, 0.5575243,\
        0.5460874, 0.5351101, 0.5246019, 0.5144758, 0.5048554, 0.4967386, 0.4894593, 0.4818857, 0.4744168,\
        0.4673884, 0.4610083, 0.4552677, 0.4500234, 0.4451009, 0.4403500, 0.4356702, 0.4310140, 0.4263786,\
        0.4217920, 0.4173015, 0.4129567, 0.4087672, 0.4047224, 0.4009629, 0.3973889, 0.3934457, 0.3895834,\
        0.3858660, 0.3821834, 0.3784398, 0.3745893, 0.3706123, 0.3664925, 0.3622132, 0.3577586, 0.3531153,\
        0.3482722, 0.3432194, 0.3379478, 0.3324482],\
        [0.3025934, 0.3087916, 0.3149470, 0.3210443, 0.3271052, 0.3331377, 0.3391396, 0.3451067, 0.3510392,\
        0.3569464, 0.3628491, 0.3687776, 0.3747587, 0.3807863, 0.3867861, 0.3926559, 0.3984412, 0.4042311,\
        0.4098585, 0.4153984, 0.4210944, 0.4270788, 0.4334266, 0.4401991, 0.4474314, 0.4551043, 0.4631269,\
        0.4713844, 0.4799000, 0.4889027, 0.4982342, 0.5070426, 0.5163158, 0.5263006, 0.5367720, 0.5474118,\
        0.5580178, 0.5683216, 0.5777055, 0.5855798, 0.5922952, 0.5987268, 0.6086083, 0.6229644, 0.6373652,\
        0.6479596, 0.6540448, 0.6573629, 0.6604967, 0.6653740, 0.6734024, 0.6856163, 0.7028406, 0.7231914,\
        0.7419254, 0.7528834, 0.7563834, 0.7572303, 0.7574896, 0.7554017, 0.7497020, 0.7407662, 0.7302407,\
        0.7196281, 0.7081108, 0.6972556, 0.6899143, 0.6829255, 0.6752852, 0.6674561, 0.6591421, 0.6501696,\
        0.6406712, 0.6307398, 0.6200292, 0.6074689, 0.5940690, 0.5809860, 0.5685727, 0.5568396, 0.5456124,\
        0.5348106, 0.5244659, 0.5145908, 0.5049970, 0.4957802, 0.4876847, 0.4805209, 0.4735416, 0.4667086,\
        0.4602002, 0.4541508, 0.4485834, 0.4434277, 0.4385720, 0.4339072, 0.4293521, 0.4248627, 0.4204268,\
        0.4160569, 0.4117909, 0.4077035, 0.4038226, 0.3999986, 0.3963863, 0.3929561, 0.3893882, 0.3857522,\
        0.3821392, 0.3785255, 0.3748553, 0.3710846, 0.3671850, 0.3631359, 0.3589200, 0.3545223, 0.3499296,\
        0.3451309, 0.3401166, 0.3348777, 0.3294056],\
        [0.2996098, 0.3057751, 0.3118247, 0.3178232, 0.3237983, 0.3297467, 0.3356562, 0.3415178, 0.3473307,\
        0.3531042, 0.3588559, 0.3646070, 0.3703704, 0.3761366, 0.3818701, 0.3875446, 0.3931884, 0.3988028,\
        0.4041312, 0.4093424, 0.4148130, 0.4205743, 0.4266458, 0.4330608, 0.4398332, 0.4469340, 0.4543034,\
        0.4618954, 0.4697269, 0.4778135, 0.4859175, 0.4938508, 0.5022833, 0.5115876, 0.5215189, 0.5314659,\
        0.5413519, 0.5510158, 0.5598338, 0.5673178, 0.5736121, 0.5797355, 0.5881194, 0.5997516, 0.6109133,\
        0.6187760, 0.6255754, 0.6311167, 0.6349685, 0.6405457, 0.6493245, 0.6616334, 0.6772885, 0.6952358,\
        0.7129142, 0.7255231, 0.7312109, 0.7334618, 0.7354965, 0.7353787, 0.7314303, 0.7234699, 0.7133398,\
        0.7054369, 0.6954146, 0.6846543, 0.6759053, 0.6680913, 0.6602678, 0.6522921, 0.6440656, 0.6354546,\
        0.6263129, 0.6167282, 0.6071285, 0.5951543, 0.5813195, 0.5680598, 0.5556754, 0.5442224, 0.5335523,\
        0.5234176, 0.5137144, 0.5046170, 0.4957743, 0.4871363, 0.4792880, 0.4722052, 0.4655567, 0.4591896,\
        0.4531269, 0.4474241, 0.4420949, 0.4371001, 0.4323671, 0.4278180, 0.4233922, 0.4190570, 0.4148022,\
        0.4106277, 0.4065432, 0.4026318, 0.3990522, 0.3955263, 0.3920826, 0.3887429, 0.3853738, 0.3819419,\
        0.3784700, 0.3749542, 0.3713652, 0.3676702, 0.3638414, 0.3598570, 0.3556987, 0.3513514, 0.3468025,\
        0.3420411, 0.3370581, 0.3318451, 0.3263942],\
        [0.2968793, 0.3029279, 0.3088417, 0.3147465, 0.3206499, 0.3265248, 0.3323448, 0.3380951, 0.3437737,\
        0.3493896, 0.3549593, 0.3605005, 0.3660244, 0.3715294, 0.3770039, 0.3824406, 0.3878372, 0.3931381,\
        0.3981691, 0.4032162, 0.4085493, 0.4141098, 0.4198972, 0.4259540, 0.4322918, 0.4388694, 0.4456337,\
        0.4525565, 0.4596321, 0.4668246, 0.4740387, 0.4813091, 0.4889633, 0.4973550, 0.5064997, 0.5151372,\
        0.5240210, 0.5330891, 0.5414273, 0.5485349, 0.5546434, 0.5607968, 0.5683326, 0.5774249, 0.5856591,\
        0.5922509, 0.5996345, 0.6075934, 0.6100171, 0.6166588, 0.6260136, 0.6385056, 0.6539729, 0.6712990,\
        0.6885906, 0.7031527, 0.7085772, 0.7109172, 0.7155010, 0.7176728, 0.7157546, 0.7087745, 0.6989001,\
        0.6906417, 0.6820807, 0.6718838, 0.6623986, 0.6536551, 0.6452101, 0.6369073, 0.6285533, 0.6200862,\
        0.6111088, 0.6011422, 0.5913335, 0.5801216, 0.5670538, 0.5541275, 0.5420903, 0.5311733, 0.5212937,\
        0.5120254, 0.5028694, 0.4943820, 0.4862853, 0.4784121, 0.4710492, 0.4642622, 0.4579297, 0.4519410,\
        0.4462609, 0.4408917, 0.4358281, 0.4310399, 0.4264740, 0.4220702, 0.4177831, 0.4135981, 0.4095212,\
        0.4055525, 0.4016797, 0.3979309, 0.3945004, 0.3912022, 0.3879487, 0.3847331, 0.3815043, 0.3782251,\
        0.3748851, 0.3714735, 0.3679702, 0.3643497, 0.3605876, 0.3566622, 0.3525555, 0.3482521, 0.3437397,\
        0.3390081, 0.3340488, 0.3288543, 0.3234174],\
        [0.2940877, 0.3000570, 0.3059209, 0.3117901, 0.3176594, 0.3234766, 0.3292071, 0.3348351, 0.3403596,\
        0.3457920, 0.3511511, 0.3564566, 0.3617234, 0.3669580, 0.3721589, 0.3773196, 0.3824210, 0.3874174,\
        0.3922930, 0.3972255, 0.4023882, 0.4077306, 0.4132061, 0.4189167, 0.4248695, 0.4309767, 0.4371708,\
        0.4434429, 0.4498048, 0.4562514, 0.4627777, 0.4694598, 0.4764879, 0.4840427, 0.4918767, 0.4990642,\
        0.5071515, 0.5157739, 0.5236928, 0.5303806, 0.5360916, 0.5419592, 0.5487704, 0.5562017, 0.5632638,\
        0.5699785, 0.5774868, 0.5845278, 0.5879614, 0.5948269, 0.6038000, 0.6165768, 0.6320795, 0.6490858,\
        0.6664111, 0.6817279, 0.6882059, 0.6917139, 0.6971823, 0.7010035, 0.7019257, 0.6950264, 0.6861335,\
        0.6768372, 0.6676731, 0.6580253, 0.6484910, 0.6392183, 0.6300424, 0.6213620, 0.6126724, 0.6038771,\
        0.5951066, 0.5848888, 0.5744143, 0.5632527, 0.5512245, 0.5391686, 0.5278043, 0.5176380, 0.5087467,\
        0.5006226, 0.4923272, 0.4843587, 0.4768697, 0.4697011, 0.4629149, 0.4565589, 0.4505980, 0.4449730,\
        0.4396453, 0.4345960, 0.4298081, 0.4252542, 0.4208910, 0.4166621, 0.4125220, 0.4084785, 0.4045723,\
        0.4008130, 0.3971625, 0.3936374, 0.3903092, 0.3871260, 0.3840069, 0.3809096, 0.3777971, 0.3746375,\
        0.3714095, 0.3680955, 0.3646754, 0.3611270, 0.3574276, 0.3535565, 0.3494955, 0.3452295, 0.3407464,\
        0.3360366, 0.3310928, 0.3259087, 0.3204783],\
        [0.2913903, 0.2972784, 0.3031034, 0.3089991, 0.3148622, 0.3206229, 0.3262535, 0.3317400, 0.3370847,\
        0.3423045, 0.3474245, 0.3524707, 0.3574641, 0.3624180, 0.3673371, 0.3722179, 0.3770487, 0.3818190,\
        0.3865591, 0.3913704, 0.3963462, 0.4014282, 0.4065663, 0.4119786, 0.4176350, 0.4233253, 0.4289699,\
        0.4346218, 0.4403414, 0.4461600, 0.4521053, 0.4582373, 0.4646390, 0.4713183, 0.4780530, 0.4847418,\
        0.4920788, 0.4998842, 0.5072840, 0.5134017, 0.5182339, 0.5236273, 0.5301084, 0.5367910, 0.5432496,\
        0.5496840, 0.5563193, 0.5623857, 0.5676786, 0.5743673, 0.5835367, 0.5960755, 0.6110024, 0.6270802,\
        0.6434984, 0.6577909, 0.6669013, 0.6728418, 0.6784295, 0.6827246, 0.6839628, 0.6793758, 0.6712116,\
        0.6621019, 0.6527437, 0.6432810, 0.6340116, 0.6248716, 0.6148161, 0.6056802, 0.5969637, 0.5873218,\
        0.5778891, 0.5677782, 0.5571742, 0.5461526, 0.5348923, 0.5238688, 0.5133242, 0.5039435, 0.4960862,\
        0.4888450, 0.4816313, 0.4745115, 0.4676896, 0.4611698, 0.4549637, 0.4490900, 0.4435398, 0.4382852,\
        0.4332993, 0.4285606, 0.4240493, 0.4197435, 0.4156125, 0.4115980, 0.4076175, 0.4036950, 0.3999494,\
        0.3963755, 0.3929197, 0.3895806, 0.3863780, 0.3832904, 0.3802683, 0.3772676, 0.3742517, 0.3711894,\
        0.3680557, 0.3648284, 0.3614855, 0.3580049, 0.3543644, 0.3505432, 0.3465228, 0.3422881, 0.3378269,\
        0.3331308, 0.3281938, 0.3230116, 0.3175795],\
        [0.2889136, 0.2947780, 0.3005788, 0.3065024, 0.3123127, 0.3179888, 0.3234962, 0.3288127, 0.3339443,\
        0.3389178, 0.3437690, 0.3485329, 0.3532384, 0.3579053, 0.3625440, 0.3671577, 0.3717462, 0.3763172,\
        0.3809020, 0.3855544, 0.3903067, 0.3951186, 0.4000007, 0.4052031, 0.4106927, 0.4159742, 0.4210462,\
        0.4261052, 0.4312562, 0.4365400, 0.4419834, 0.4476163, 0.4534565, 0.4594716, 0.4655897, 0.4718701,\
        0.4785229, 0.4854212, 0.4921153, 0.4974620, 0.5013611, 0.5064122, 0.5130556, 0.5191524, 0.5249594,\
        0.5307885, 0.5367138, 0.5424732, 0.5482427, 0.5551498, 0.5643416, 0.5762892, 0.5903051, 0.6052317,\
        0.6209543, 0.6353710, 0.6455350, 0.6524805, 0.6582698, 0.6627761, 0.6641213, 0.6609377, 0.6542182,\
        0.6458438, 0.6368343, 0.6275978, 0.6186376, 0.6107016, 0.6013248, 0.5902100, 0.5807890, 0.5707750,\
        0.5606936, 0.5504969, 0.5401044, 0.5294258, 0.5188026, 0.5090588, 0.4997098, 0.4911683, 0.4840711,\
        0.4775521, 0.4711904, 0.4649175, 0.4588136, 0.4529286, 0.4472863, 0.4419004, 0.4367711, 0.4318882,\
        0.4272367, 0.4227998, 0.4185580, 0.4144949, 0.4106055, 0.4068637, 0.4031031, 0.3992740, 0.3956836,\
        0.3922511, 0.3889420, 0.3857447, 0.3826559, 0.3796577, 0.3767166, 0.3737955, 0.3708598, 0.3678787,\
        0.3648251, 0.3616740, 0.3584017, 0.3549844, 0.3513992, 0.3476243, 0.3436404, 0.3394315, 0.3349855,\
        0.3302948, 0.3253555, 0.3201658, 0.3147234],\
        [0.2866976, 0.2926553, 0.2985444, 0.3043236, 0.3100092, 0.3155830, 0.3209421, 0.3260504, 0.3309263,\
        0.3356141, 0.3401647, 0.3446248, 0.3490310, 0.3534088, 0.3577739, 0.3621348, 0.3664986, 0.3708774,\
        0.3752939, 0.3797755, 0.3843357, 0.3889726, 0.3937325, 0.3988320, 0.4041438, 0.4088812, 0.4133257,\
        0.4178538, 0.4225286, 0.4273693, 0.4323925, 0.4376072, 0.4430001, 0.4485357, 0.4541888, 0.4599920,\
        0.4659804, 0.4720173, 0.4777063, 0.4824660, 0.4865397, 0.4916167, 0.4977195, 0.5029757, 0.5081476,\
        0.5133382, 0.5188358, 0.5243917, 0.5300318, 0.5367185, 0.5456019, 0.5570436, 0.5704328, 0.5849380,\
        0.6009689, 0.6151322, 0.6250463, 0.6312876, 0.6372450, 0.6426027, 0.6444942, 0.6418981, 0.6359530,\
        0.6283544, 0.6199943, 0.6111160, 0.6021441, 0.5945160, 0.5864504, 0.5753879, 0.5646478, 0.5541253,\
        0.5437256, 0.5335970, 0.5237954, 0.5138471, 0.5035591, 0.4950279, 0.4871332, 0.4796759, 0.4731063,\
        0.4671248, 0.4614058, 0.4558218, 0.4503696, 0.4450765, 0.4399646, 0.4350463, 0.4303256, 0.4258018,\
        0.4214719, 0.4173264, 0.4133435, 0.4094982, 0.4058007, 0.4023201, 0.3988980, 0.3952324, 0.3917275,\
        0.3883962, 0.3851975, 0.3821091, 0.3791158, 0.3761978, 0.3733279, 0.3704748, 0.3676072, 0.3646953,\
        0.3617111, 0.3586281, 0.3554205, 0.3520629, 0.3485306, 0.3448000, 0.3408501, 0.3366632, 0.3322266,\
        0.3275332, 0.3225818, 0.3173744, 0.3119127],\
        [0.2846691, 0.2907062, 0.2965531, 0.3022356, 0.3079151, 0.3134162, 0.3185919, 0.3234374, 0.3280033,\
        0.3323607, 0.3365791, 0.3407169, 0.3448169, 0.3489084, 0.3530095, 0.3571314, 0.3612831, 0.3654756,\
        0.3697242, 0.3740459, 0.3784562, 0.3829794, 0.3876775, 0.3926259, 0.3976094, 0.4016750, 0.4056852,\
        0.4098336, 0.4141489, 0.4186376, 0.4233241, 0.4282125, 0.4332666, 0.4384315, 0.4436729, 0.4489883,\
        0.4543541, 0.4596461, 0.4646264, 0.4691206, 0.4734169, 0.4781644, 0.4832555, 0.4880680, 0.4927236,\
        0.4973821, 0.5026517, 0.5079521, 0.5130562, 0.5190046, 0.5274279, 0.5386319, 0.5518309, 0.5665699,\
        0.5821922, 0.5962840, 0.6054881, 0.6098739, 0.6166573, 0.6239694, 0.6262120, 0.6232517, 0.6171870,\
        0.6100865, 0.6024688, 0.5940845, 0.5854812, 0.5773867, 0.5688734, 0.5588938, 0.5483235, 0.5377045,\
        0.5272399, 0.5173295, 0.5083861, 0.5000943, 0.4907378, 0.4826675, 0.4755989, 0.4690672, 0.4631061,\
        0.4576140, 0.4524092, 0.4473749, 0.4424716, 0.4376996, 0.4330679, 0.4285814, 0.4242407, 0.4200491,\
        0.4160173, 0.4121498, 0.4084227, 0.4047897, 0.4012198, 0.3978723, 0.3946302, 0.3912753, 0.3879478,\
        0.3847351, 0.3816388, 0.3786433, 0.3757317, 0.3728845, 0.3700776, 0.3672841, 0.3644758, 0.3616242,\
        0.3587017, 0.3556806, 0.3525338, 0.3492339, 0.3457540, 0.3420680, 0.3381522, 0.3339860, 0.3295550,\
        0.3248517, 0.3198778, 0.3146414, 0.3091505],\
        [0.2827604, 0.2889154, 0.2947771, 0.3004543, 0.3061702, 0.3115206, 0.3164268, 0.3209309, 0.3251230,\
        0.3291045, 0.3329640, 0.3367683, 0.3405630, 0.3443770, 0.3482278, 0.3521259, 0.3560785, 0.3600932,\
        0.3641792, 0.3683478, 0.3726124, 0.3769913, 0.3815005, 0.3860897, 0.3904924, 0.3944130, 0.3982007,\
        0.4020965, 0.4061501, 0.4103544, 0.4147785, 0.4194293, 0.4242399, 0.4291069, 0.4339736, 0.4388313,\
        0.4436594, 0.4483829, 0.4528952, 0.4571673, 0.4613609, 0.4656954, 0.4701279, 0.4744688, 0.4787299,\
        0.4831095, 0.4878670, 0.4926315, 0.4972612, 0.5022453, 0.5103506, 0.5213410, 0.5339902, 0.5481129,\
        0.5627706, 0.5760765, 0.5851130, 0.5910635, 0.6007016, 0.6076493, 0.6094832, 0.6053122, 0.5986117,\
        0.5917092, 0.5844039, 0.5764610, 0.5682626, 0.5600343, 0.5513972, 0.5420028, 0.5321191, 0.5219878,\
        0.5117297, 0.5022995, 0.4940440, 0.4867234, 0.4790635, 0.4718194, 0.4653363, 0.4594414, 0.4540238,\
        0.4489879, 0.4442207, 0.4396341, 0.4351833, 0.4308547, 0.4266455, 0.4225491, 0.4185534, 0.4146540,\
        0.4108793, 0.4072678, 0.4037882, 0.4003886, 0.3970012, 0.3937439, 0.3905923, 0.3874362, 0.3842942,\
        0.3812175, 0.3782258, 0.3753165, 0.3724779, 0.3696933, 0.3669422, 0.3642012, 0.3614449, 0.3586470,\
        0.3557804, 0.3528173, 0.3497292, 0.3464870, 0.3430613, 0.3394230, 0.3355447, 0.3314021, 0.3269765,\
        0.3222585, 0.3172517, 0.3119730, 0.3064420],\
        [0.2808928, 0.2872733, 0.2933863, 0.2992743, 0.3047824, 0.3098315, 0.3143642, 0.3184423, 0.3221988,\
        0.3257685, 0.3292558, 0.3327293, 0.3362309, 0.3397850, 0.3434052, 0.3470984, 0.3508679, 0.3547161,\
        0.3586460, 0.3626617, 0.3667670, 0.3709603, 0.3752177, 0.3794560, 0.3835206, 0.3873292, 0.3910132,\
        0.3947418, 0.3985822, 0.4025475, 0.4067727, 0.4112629, 0.4159219, 0.4205398, 0.4250547, 0.4294975,\
        0.4338797, 0.4381733, 0.4423392, 0.4463762, 0.4503474, 0.4543225, 0.4582923, 0.4622125, 0.4661139,\
        0.4701115, 0.4742748, 0.4784924, 0.4827883, 0.4878627, 0.4955574, 0.5055072, 0.5168062, 0.5298516,\
        0.5432824, 0.5554020, 0.5653773, 0.5743750, 0.5838479, 0.5901766, 0.5918329, 0.5876783, 0.5810547,\
        0.5738949, 0.5664328, 0.5587155, 0.5508966, 0.5429204, 0.5345059, 0.5256122, 0.5165730, 0.5075921,\
        0.4980131, 0.4892712, 0.4817320, 0.4749019, 0.4682940, 0.4619758, 0.4561416, 0.4507882, 0.4458399,\
        0.4412124, 0.4368226, 0.4326054, 0.4285239, 0.4245647, 0.4207210, 0.4169762, 0.4132983, 0.4096507,\
        0.4060612, 0.4026721, 0.3994118, 0.3961884, 0.3929951, 0.3898661, 0.3868072, 0.3837812, 0.3807817,\
        0.3778280, 0.3749348, 0.3721047, 0.3693311, 0.3666014, 0.3638987, 0.3612033, 0.3584926, 0.3557427,\
        0.3529275, 0.3500196, 0.3469897, 0.3438069, 0.3404394, 0.3368549, 0.3330218, 0.3289110, 0.3244975,\
        0.3197655, 0.3147172, 0.3093801, 0.3037962],\
        [0.2789119, 0.2855526, 0.2919111, 0.2978916, 0.3033543, 0.3081388, 0.3122422, 0.3158298, 0.3191083,\
        0.3222547, 0.3253813, 0.3285476, 0.3317837, 0.3351058, 0.3385222, 0.3420348, 0.3456414, 0.3493381,\
        0.3531213, 0.3569871, 0.3609290, 0.3649311, 0.3689567, 0.3729385, 0.3767977, 0.3805077, 0.3841317,\
        0.3877652, 0.3914713, 0.3953115, 0.3993603, 0.4037450, 0.4083348, 0.4126974, 0.4168568, 0.4209306,\
        0.4249512, 0.4289157, 0.4328141, 0.4366374, 0.4403849, 0.4440636, 0.4476765, 0.4512350, 0.4547848,\
        0.4583986, 0.4621293, 0.4660183, 0.4702729, 0.4755003, 0.4824448, 0.4909602, 0.5006566, 0.5128728,\
        0.5248744, 0.5357589, 0.5459080, 0.5556415, 0.5644847, 0.5705296, 0.5722216, 0.5693414, 0.5635169,\
        0.5563985, 0.5488148, 0.5412291, 0.5339093, 0.5265676, 0.5186143, 0.5102172, 0.5019208, 0.4941267,\
        0.4859306, 0.4780282, 0.4710106, 0.4646664, 0.4587313, 0.4531305, 0.4479003, 0.4430431, 0.4385141,\
        0.4342493, 0.4301849, 0.4262716, 0.4224849, 0.4188237, 0.4152891, 0.4118607, 0.4084924, 0.4051031,\
        0.4016205, 0.3983653, 0.3952916, 0.3922287, 0.3891892, 0.3861877, 0.3832302, 0.3803075, 0.3774157,\
        0.3745612, 0.3717518, 0.3689899, 0.3662716, 0.3635879, 0.3609255, 0.3582680, 0.3555962, 0.3528884,\
        0.3501205, 0.3472655, 0.3442940, 0.3411736, 0.3378698, 0.3343474, 0.3305708, 0.3265062, 0.3221212,\
        0.3173888, 0.3122990, 0.3068845, 0.3012317],\
        [0.2766307, 0.2834978, 0.2900671, 0.2962219, 0.3017075, 0.3061982, 0.3098481, 0.3129093, 0.3157035,\
        0.3184569, 0.3212718, 0.3241802, 0.3271940, 0.3303218, 0.3335680, 0.3369297, 0.3403984, 0.3439633,\
        0.3476141, 0.3513408, 0.3551307, 0.3589634, 0.3628051, 0.3666101, 0.3703372, 0.3739776, 0.3775671,\
        0.3811655, 0.3848326, 0.3886308, 0.3926344, 0.3969571, 0.4014195, 0.4054390, 0.4092640, 0.4130468,\
        0.4167913, 0.4205080, 0.4242140, 0.4278831, 0.4314530, 0.4348910, 0.4382114, 0.4414589, 0.4446975,\
        0.4480010, 0.4514481, 0.4551542, 0.4593538, 0.4644338, 0.4707388, 0.4783274, 0.4873935, 0.4981595,\
        0.5078693, 0.5171370, 0.5267355, 0.5363921, 0.5450349, 0.5509917, 0.5531102, 0.5512079, 0.5461747,\
        0.5393235, 0.5317398, 0.5242567, 0.5176022, 0.5114888, 0.5041388, 0.4963770, 0.4889151, 0.4818006,\
        0.4747353, 0.4678596, 0.4615046, 0.4556916, 0.4503072, 0.4452723, 0.4405595, 0.4361468, 0.4319958,\
        0.4280554, 0.4242733, 0.4206081, 0.4170456, 0.4136074, 0.4103213, 0.4071635, 0.4040727, 0.4009982,\
        0.3977085, 0.3945038, 0.3914853, 0.3885281, 0.3856003, 0.3827018, 0.3798344, 0.3769966, 0.3741873,\
        0.3714089, 0.3686650, 0.3659569, 0.3632820, 0.3606337, 0.3580018, 0.3553734, 0.3527324, 0.3500600,\
        0.3473343, 0.3445298, 0.3416169, 0.3385620, 0.3353280, 0.3318764, 0.3281687, 0.3241681, 0.3198383,\
        0.3151404, 0.3100387, 0.3045404, 0.2987969],\
        [0.2738866, 0.2809043, 0.2876044, 0.2938534, 0.2993317, 0.3037010, 0.3069511, 0.3094733, 0.3118354,\
        0.3142891, 0.3168866, 0.3196096, 0.3224533, 0.3254291, 0.3285433, 0.3317886, 0.3351488, 0.3386054,\
        0.3421427, 0.3457475, 0.3494063, 0.3531012, 0.3568065, 0.3604928, 0.3641372, 0.3677371, 0.3713156,\
        0.3749146, 0.3785845, 0.3823807, 0.3863594, 0.3905330, 0.3947033, 0.3985062, 0.4021697, 0.4057740,\
        0.4093299, 0.4128591, 0.4164558, 0.4200663, 0.4235182, 0.4267518, 0.4298162, 0.4327947, 0.4357745,\
        0.4388438, 0.4420990, 0.4456704, 0.4497532, 0.4545957, 0.4604139, 0.4673303, 0.4753929, 0.4841199,\
        0.4921287, 0.5001744, 0.5087988, 0.5183093, 0.5269881, 0.5330106, 0.5356219, 0.5343992, 0.5298511,\
        0.5232417, 0.5157626, 0.5083696, 0.5021106, 0.4975518, 0.4911820, 0.4841843, 0.4774270, 0.4709538,\
        0.4646835, 0.4586523, 0.4529882, 0.4477387, 0.4428671, 0.4383196, 0.4340547, 0.4300370, 0.4262274,\
        0.4225810, 0.4190512, 0.4155949, 0.4121912, 0.4088879, 0.4057852, 0.4028464, 0.3999442, 0.3970233,\
        0.3939873, 0.3909294, 0.3879652, 0.3850723, 0.3822203, 0.3793978, 0.3766014, 0.3738295, 0.3710816,\
        0.3683586, 0.3656619, 0.3629918, 0.3603464, 0.3577210, 0.3551081, 0.3524979, 0.3498778, 0.3472323,\
        0.3445424, 0.3417844, 0.3389294, 0.3359425, 0.3327837, 0.3294100, 0.3257799, 0.3218567, 0.3176084,\
        0.3129971, 0.3079612, 0.3024646, 0.2966315],\
        [0.2705723, 0.2776399, 0.2843487, 0.2905564, 0.2960412, 0.3005659, 0.3032621, 0.3053280, 0.3074125,\
        0.3097339, 0.3122453, 0.3148572, 0.3175757, 0.3204394, 0.3234622, 0.3266293, 0.3299133, 0.3332872,\
        0.3367311, 0.3402328, 0.3437832, 0.3473710, 0.3509797, 0.3545901, 0.3581876, 0.3617708, 0.3653545,\
        0.3689676, 0.3726468, 0.3764284, 0.3803311, 0.3843156, 0.3882411, 0.3919745, 0.3955636, 0.3990772,\
        0.4025309, 0.4058817, 0.4094861, 0.4131758, 0.4165374, 0.4195601, 0.4223807, 0.4251233, 0.4278935,\
        0.4307891, 0.4339065, 0.4373582, 0.4412890, 0.4458637, 0.4512222, 0.4574253, 0.4643707, 0.4716471,\
        0.4788030, 0.4860455, 0.4937640, 0.5030908, 0.5112573, 0.5171340, 0.5203727, 0.5196390, 0.5151743,\
        0.5087463, 0.5016587, 0.4947879, 0.4890244, 0.4843385, 0.4790356, 0.4730740, 0.4670840, 0.4612813,\
        0.4557016, 0.4503737, 0.4453484, 0.4406519, 0.4362700, 0.4321670, 0.4283032, 0.4246405, 0.4211407,\
        0.4177646, 0.4144722, 0.4112175, 0.4079437, 0.4046598, 0.4016689, 0.3988985, 0.3961269, 0.3933129,\
        0.3904353, 0.3875280, 0.3846506, 0.3818187, 0.3790235, 0.3762562, 0.3735117, 0.3707875, 0.3680823,\
        0.3653960, 0.3627288, 0.3600806, 0.3574498, 0.3548333, 0.3522261, 0.3496213, 0.3470098, 0.3443800,\
        0.3417165, 0.3389987, 0.3361992, 0.3332819, 0.3302026, 0.3269116, 0.3233605, 0.3195110, 0.3153436,\
        0.3108510, 0.3059714, 0.3005304, 0.2945982],\
        [0.2666494, 0.2736726, 0.2802840, 0.2863505, 0.2916778, 0.2958687, 0.2985094, 0.3005132, 0.3025171,\
        0.3049109, 0.3074492, 0.3099797, 0.3125916, 0.3153782, 0.3183529, 0.3214829, 0.3247239, 0.3280388,\
        0.3314063, 0.3348198, 0.3382804, 0.3417867, 0.3453304, 0.3488973, 0.3524749, 0.3560591, 0.3596571,\
        0.3632864, 0.3669693, 0.3707228, 0.3745440, 0.3783909, 0.3821843, 0.3858627, 0.3894301, 0.3929159,\
        0.3963111, 0.3995418, 0.4033994, 0.4072419, 0.4104085, 0.4131651, 0.4157532, 0.4182962, 0.4209038,\
        0.4236780, 0.4266997, 0.4300453, 0.4338099, 0.4381033, 0.4430148, 0.4485645, 0.4546547, 0.4610738,\
        0.4676744, 0.4745829, 0.4821596, 0.4904300, 0.4974491, 0.5032385, 0.5075066, 0.5071547, 0.5023106,\
        0.4961480, 0.4897032, 0.4834670, 0.4779632, 0.4730720, 0.4681494, 0.4629618, 0.4577091, 0.4525735,\
        0.4476358, 0.4429329, 0.4384897, 0.4343151, 0.4303973, 0.4267083, 0.4232126, 0.4198731, 0.4166550,\
        0.4135257, 0.4104570, 0.4074241, 0.4043639, 0.4011081, 0.3980508, 0.3953512, 0.3926424, 0.3898979,\
        0.3871129, 0.3843055, 0.3815054, 0.3787294, 0.3759797, 0.3732525, 0.3705439, 0.3678510, 0.3651718,\
        0.3625055, 0.3598516, 0.3572095, 0.3545781, 0.3519556, 0.3493392, 0.3467248, 0.3441069, 0.3414783,\
        0.3388283, 0.3361408, 0.3333914, 0.3305440, 0.3275490, 0.3243454, 0.3208700, 0.3170719, 0.3129314,\
        0.3084832, 0.3038031, 0.2985132, 0.2923706],\
        [0.2621426, 0.2690553, 0.2754992, 0.2813379, 0.2863738, 0.2903530, 0.2932315, 0.2954561, 0.2975882,\
        0.3000783, 0.3026228, 0.3050238, 0.3075347, 0.3102863, 0.3132611, 0.3163950, 0.3196224, 0.3228960,\
        0.3261955, 0.3295268, 0.3329085, 0.3363530, 0.3398575, 0.3434075, 0.3469866, 0.3505850, 0.3542025,\
        0.3578471, 0.3615296, 0.3652556, 0.3690155, 0.3727790, 0.3765027, 0.3801547, 0.3837316, 0.3872470,\
        0.3907198, 0.3942914, 0.3983071, 0.4020813, 0.4048150, 0.4073325, 0.4097557, 0.4121529, 0.4146459,\
        0.4173550, 0.4203299, 0.4235899, 0.4271866, 0.4311996, 0.4356995, 0.4407081, 0.4461784, 0.4520226,\
        0.4581996, 0.4647755, 0.4718162, 0.4790014, 0.4855557, 0.4912735, 0.4957307, 0.4956412, 0.4910141,\
        0.4852603, 0.4792794, 0.4734916, 0.4681718, 0.4632866, 0.4585778, 0.4538823, 0.4492269, 0.4446971,\
        0.4403524, 0.4362208, 0.4323121, 0.4286238, 0.4251417, 0.4218407, 0.4186873, 0.4156444, 0.4126813,\
        0.4097718, 0.4068860, 0.4040427, 0.4012160, 0.3982397, 0.3951092, 0.3922456, 0.3894918, 0.3867552,\
        0.3840101, 0.3812577, 0.3785099, 0.3757765, 0.3730608, 0.3703615, 0.3676756, 0.3650001, 0.3623327,\
        0.3596717, 0.3570161, 0.3543652, 0.3517183, 0.3490747, 0.3464331, 0.3437924, 0.3411505, 0.3385047,\
        0.3358503, 0.3331779, 0.3304689, 0.3276896, 0.3247855, 0.3216800, 0.3182849, 0.3145208, 0.3103400,\
        0.3057369, 0.3008579, 0.2956681, 0.2896278],\
        [0.2571200, 0.2638948, 0.2701589, 0.2757904, 0.2806453, 0.2846106, 0.2877366, 0.2903083, 0.2927146,\
        0.2952074, 0.2976858, 0.2999988, 0.3024711, 0.3052386, 0.3082561, 0.3114239, 0.3146574, 0.3178969,\
        0.3211231, 0.3243643, 0.3276691, 0.3310679, 0.3345578, 0.3381152, 0.3417140, 0.3453367, 0.3489767,\
        0.3526355, 0.3563174, 0.3600229, 0.3637436, 0.3674609, 0.3711517, 0.3748009, 0.3784095, 0.3819975,\
        0.3856139, 0.3893587, 0.3932329, 0.3966909, 0.3993548, 0.4018615, 0.4042349, 0.4065492, 0.4089682,\
        0.4116878, 0.4146810, 0.4178771, 0.4213029, 0.4250382, 0.4291626, 0.4337196, 0.4387042, 0.4440834,\
        0.4498354, 0.4559522, 0.4623544, 0.4687584, 0.4747292, 0.4798398, 0.4832043, 0.4832275, 0.4799367,\
        0.4750459, 0.4696659, 0.4643172, 0.4592520, 0.4545170, 0.4500291, 0.4457069, 0.4415354, 0.4375380,\
        0.4337349, 0.4301298, 0.4267146, 0.4234764, 0.4203992, 0.4174614, 0.4146306, 0.4118625, 0.4091248,\
        0.4064231, 0.4036746, 0.4008800, 0.3981703, 0.3952868, 0.3923041, 0.3893954, 0.3865845, 0.3838249,\
        0.3810863, 0.3783579, 0.3756396, 0.3729336, 0.3702404, 0.3675584, 0.3648843, 0.3622150, 0.3595473,\
        0.3568790, 0.3542088, 0.3515356, 0.3488591, 0.3461794, 0.3434968, 0.3408119, 0.3381261, 0.3354411,\
        0.3327587, 0.3300785, 0.3273922, 0.3246744, 0.3218694, 0.3188832, 0.3155920, 0.3118749, 0.3076651,\
        0.3029595, 0.2978040, 0.2924253, 0.2864443],\
        [0.2516673, 0.2583116, 0.2644281, 0.2699237, 0.2747109, 0.2787518, 0.2821169, 0.2849996, 0.2876401,\
        0.2901940, 0.2926670, 0.2950699, 0.2975823, 0.3003618, 0.3034257, 0.3066270, 0.3098737, 0.3130759,\
        0.3162055, 0.3193301, 0.3225529, 0.3259255, 0.3294300, 0.3330200, 0.3366541, 0.3403077, 0.3439707,\
        0.3476418, 0.3513231, 0.3550146, 0.3587115, 0.3624039, 0.3660810, 0.3697369, 0.3733761, 0.3770156,\
        0.3806824, 0.3843847, 0.3880121, 0.3912836, 0.3940974, 0.3966583, 0.3990766, 0.4013724, 0.4037431,\
        0.4066044, 0.4096795, 0.4128053, 0.4160405, 0.4194899, 0.4232595, 0.4274192, 0.4319841, 0.4369311,\
        0.4422248, 0.4478099, 0.4535593, 0.4592218, 0.4644241, 0.4686616, 0.4711968, 0.4712902, 0.4689748,\
        0.4650788, 0.4604478, 0.4556279, 0.4509171, 0.4464407, 0.4422193, 0.4382392, 0.4344922, 0.4309743,\
        0.4276716, 0.4245564, 0.4215977, 0.4187716, 0.4160634, 0.4134607, 0.4109405, 0.4084450, 0.4058841,\
        0.4033585, 0.4008966, 0.3980451, 0.3952738, 0.3924317, 0.3895252, 0.3866365, 0.3838064, 0.3810286,\
        0.3782859, 0.3755659, 0.3728623, 0.3701721, 0.3674921, 0.3648189, 0.3621484, 0.3594762, 0.3567987,\
        0.3541130, 0.3514169, 0.3487097, 0.3459910, 0.3432616, 0.3405226, 0.3377763, 0.3350259, 0.3322767,\
        0.3295369, 0.3268163, 0.3241224, 0.3214476, 0.3187458, 0.3159090, 0.3127664, 0.3091282, 0.3048922,\
        0.3001301, 0.2948795, 0.2891948, 0.2830516],\
        [0.2458674, 0.2524085, 0.2584263, 0.2638562, 0.2686508, 0.2728075, 0.2763928, 0.2795400, 0.2824047,\
        0.2851043, 0.2877015, 0.2902644, 0.2929119, 0.2957560, 0.2988189, 0.3020267, 0.3052960, 0.3084539,\
        0.3114426, 0.3144005, 0.3175383, 0.3209230, 0.3244833, 0.3281311, 0.3318106, 0.3354954, 0.3391770,\
        0.3428557, 0.3465342, 0.3502144, 0.3538950, 0.3575721, 0.3612415, 0.3649012, 0.3685538, 0.3722044,\
        0.3758513, 0.3794610, 0.3829357, 0.3861421, 0.3890360, 0.3916905, 0.3941851, 0.3965678, 0.3990475,\
        0.4021581, 0.4052464, 0.4082460, 0.4112608, 0.4144081, 0.4178313, 0.4216312, 0.4258177, 0.4303444,\
        0.4351551, 0.4401760, 0.4452708, 0.4502041, 0.4546311, 0.4581007, 0.4600962, 0.4602314, 0.4585336,\
        0.4554475, 0.4515433, 0.4472894, 0.4429985, 0.4388503, 0.4349349, 0.4312947, 0.4279466, 0.4248806,\
        0.4220543, 0.4194033, 0.4168664, 0.4144094, 0.4120239, 0.4097133, 0.4074845, 0.4053102, 0.4029891,\
        0.4002764, 0.3980894, 0.3953652, 0.3925137, 0.3896486, 0.3867683, 0.3839024, 0.3810764, 0.3782953,\
        0.3755519, 0.3728368, 0.3701421, 0.3674613, 0.3647888, 0.3621191, 0.3594463, 0.3567651, 0.3540708,\
        0.3513595, 0.3486289, 0.3458778, 0.3431063, 0.3403155, 0.3375072, 0.3346841, 0.3318501, 0.3290120,\
        0.3261818, 0.3233784, 0.3206282, 0.3179548, 0.3153426, 0.3126812, 0.3097485, 0.3062501, 0.3019591,\
        0.2970449, 0.2916446, 0.2857679, 0.2794425],\
        [0.2397884, 0.2462612, 0.2522281, 0.2576472, 0.2624962, 0.2667876, 0.2705786, 0.2739656, 0.2770599,\
        0.2799605, 0.2827479, 0.2855031, 0.2883175, 0.2912668, 0.2943732, 0.2976084, 0.3009023, 0.3040099,\
        0.3068089, 0.3095177, 0.3126033, 0.3160832, 0.3197490, 0.3234694, 0.3271909, 0.3308969, 0.3345864,\
        0.3382638, 0.3419342, 0.3456013, 0.3492669, 0.3529305, 0.3565909, 0.3602466, 0.3638955, 0.3675317,\
        0.3711378, 0.3746726, 0.3780685, 0.3812564, 0.3842137, 0.3869759, 0.3896088, 0.3922145, 0.3950068,\
        0.3981067, 0.4011122, 0.4040037, 0.4067990, 0.4096353, 0.4127203, 0.4162122, 0.4200713, 0.4241891,\
        0.4285015, 0.4329501, 0.4374142, 0.4416775, 0.4454317, 0.4483057, 0.4499365, 0.4500922, 0.4487917,\
        0.4463015, 0.4430079, 0.4392842, 0.4354234, 0.4316289, 0.4280342, 0.4247281, 0.4217632, 0.4191376,\
        0.4167817, 0.4145816, 0.4124327, 0.4102924, 0.4081733, 0.4060880, 0.4040746, 0.4021843, 0.4004268,\
        0.3978366, 0.3951928, 0.3925070, 0.3896942, 0.3868421, 0.3839860, 0.3811464, 0.3783388, 0.3755690,\
        0.3728344, 0.3701282, 0.3674428, 0.3647705, 0.3621037, 0.3594351, 0.3567573, 0.3540636, 0.3513479,\
        0.3486057, 0.3458340, 0.3430315, 0.3401986, 0.3373373, 0.3344498, 0.3315389, 0.3286076, 0.3256610,\
        0.3227104, 0.3197783, 0.3169042, 0.3141532, 0.3115792, 0.3090868, 0.3063924, 0.3031317, 0.2988238,\
        0.2936992, 0.2880845, 0.2820259, 0.2755411],\
        [0.2334819, 0.2399208, 0.2458773, 0.2513260, 0.2562595, 0.2606962, 0.2646850, 0.2683000, 0.2716274,\
        0.2747529, 0.2777562, 0.2807146, 0.2837017, 0.2867739, 0.2899536, 0.2932238, 0.2965054, 0.2996053,\
        0.3022391, 0.3045882, 0.3077976, 0.3114987, 0.3152861, 0.3190591, 0.3227973, 0.3265025, 0.3301830,\
        0.3338472, 0.3375019, 0.3411516, 0.3447995, 0.3484475, 0.3520952, 0.3557401, 0.3593761, 0.3629902,\
        0.3665596, 0.3700477, 0.3734088, 0.3766048, 0.3796256, 0.3824984, 0.3852832, 0.3880740, 0.3909790,\
        0.3939993, 0.3969870, 0.3998381, 0.4024808, 0.4050273, 0.4077851, 0.4110842, 0.4146913, 0.4183970,\
        0.4221884, 0.4260708, 0.4299563, 0.4336471, 0.4368647, 0.4392991, 0.4406763, 0.4408375, 0.4397954,\
        0.4377278, 0.4349111, 0.4316409, 0.4281782, 0.4247273, 0.4214397, 0.4184399, 0.4158315, 0.4136399,\
        0.4117624, 0.4100135, 0.4082160, 0.4063211, 0.4044123, 0.4024956, 0.4005801, 0.3988076, 0.3970326,\
        0.3947984, 0.3921967, 0.3895028, 0.3867389, 0.3839360, 0.3811246, 0.3783251, 0.3755504, 0.3728060,\
        0.3700916, 0.3674023, 0.3647312, 0.3620703, 0.3594109, 0.3567443, 0.3540616, 0.3513545, 0.3486157,\
        0.3458394, 0.3430221, 0.3401626, 0.3372621, 0.3343237, 0.3313512, 0.3283474, 0.3253133, 0.3222497,\
        0.3191625, 0.3160701, 0.3130051, 0.3100512, 0.3074052, 0.3050374, 0.3024578, 0.2992742, 0.2949958,\
        0.2898063, 0.2840720, 0.2778894, 0.2712873],\
        [0.2269844, 0.2334199, 0.2393986, 0.2449064, 0.2499444, 0.2545335, 0.2587157, 0.2625508, 0.2661094,\
        0.2694648, 0.2726888, 0.2758494, 0.2790061, 0.2822015, 0.2854503, 0.2887288, 0.2919574, 0.2949854,\
        0.2974970, 0.2997936, 0.3034701, 0.3073247, 0.3111386, 0.3148995, 0.3186117, 0.3222876, 0.3259409,\
        0.3295807, 0.3332122, 0.3368393, 0.3404657, 0.3440945, 0.3477263, 0.3513575, 0.3549788, 0.3585741,\
        0.3621194, 0.3655842, 0.3689370, 0.3721550, 0.3752357, 0.3782018, 0.3810998, 0.3839907, 0.3869214,\
        0.3898781, 0.3927853, 0.3955553, 0.3981431, 0.4004862, 0.4029508, 0.4063181, 0.4097050, 0.4129255,\
        0.4161491, 0.4194861, 0.4228727, 0.4261102, 0.4289309, 0.4310559, 0.4322570, 0.4324124, 0.4315341,\
        0.4297578, 0.4272976, 0.4243936, 0.4212780, 0.4181459, 0.4151367, 0.4123852, 0.4100755, 0.4083038,\
        0.4069094, 0.4056202, 0.4041398, 0.4023852, 0.4006120, 0.3988772, 0.3970274, 0.3952586, 0.3934172,\
        0.3912958, 0.3888783, 0.3862922, 0.3836155, 0.3808902, 0.3781462, 0.3754042, 0.3726774, 0.3699723,\
        0.3672898, 0.3646268, 0.3619773, 0.3593335, 0.3566860, 0.3540250, 0.3513402, 0.3486215, 0.3458601,\
        0.3430489, 0.3401838, 0.3372636, 0.3342907, 0.3312705, 0.3282098, 0.3251138, 0.3219825, 0.3188089,\
        0.3155892, 0.3123420, 0.3090890, 0.3058110, 0.3028609, 0.3006080, 0.2979306, 0.2945932, 0.2903519,\
        0.2852648, 0.2795434, 0.2733192, 0.2666523],\
        [0.2203204, 0.2267786, 0.2328050, 0.2383933, 0.2435499, 0.2482959, 0.2526681, 0.2567167, 0.2605002,\
        0.2640813, 0.2675217, 0.2708787, 0.2742002, 0.2775177, 0.2808383, 0.2841363, 0.2873448, 0.2903609,\
        0.2931467, 0.2961325, 0.2997626, 0.3035166, 0.3072407, 0.3109328, 0.3145834, 0.3182082, 0.3218233,\
        0.3254332, 0.3290376, 0.3326379, 0.3362385, 0.3398450, 0.3434594, 0.3470778, 0.3506887, 0.3542736,\
        0.3578087, 0.3612676, 0.3646268, 0.3678714, 0.3710015, 0.3740342, 0.3770004, 0.3799347, 0.3828562,\
        0.3857503, 0.3885694, 0.3912562, 0.3937663, 0.3960988, 0.3987508, 0.4021405, 0.4050827, 0.4076770,\
        0.4102921, 0.4131473, 0.4161451, 0.4190581, 0.4216143, 0.4235370, 0.4246171, 0.4247547, 0.4239687,\
        0.4223837, 0.4201841, 0.4175666, 0.4147407, 0.4119144, 0.4091794, 0.4066036, 0.4044778, 0.4031047,\
        0.4021299, 0.4012552, 0.4000602, 0.3984051, 0.3967150, 0.3950430, 0.3933117, 0.3915427, 0.3896767,\
        0.3876147, 0.3853314, 0.3828780, 0.3803154, 0.3776887, 0.3750298, 0.3723606, 0.3696952, 0.3670416,\
        0.3644021, 0.3617750, 0.3591554, 0.3565356, 0.3539064, 0.3512567, 0.3485748, 0.3458489, 0.3430680,\
        0.3402235, 0.3373102, 0.3343269, 0.3312777, 0.3281710, 0.3250192, 0.3218343, 0.3186206, 0.3153628,\
        0.3120312, 0.3086448, 0.3053279, 0.3019735, 0.2985088, 0.2959489, 0.2930904, 0.2895684, 0.2852845,\
        0.2802475, 0.2745643, 0.2683399, 0.2616440],\
        [0.2135059, 0.2200081, 0.2261021, 0.2317869, 0.2370717, 0.2419774, 0.2465365, 0.2507916, 0.2547923,\
        0.2585913, 0.2622409, 0.2657888, 0.2692736, 0.2727197, 0.2761326, 0.2794954, 0.2827735, 0.2859439,\
        0.2890774, 0.2923989, 0.2960243, 0.2997412, 0.3034221, 0.3070565, 0.3106350, 0.3142053, 0.3177871,\
        0.3213722, 0.3249506, 0.3285212, 0.3320918, 0.3356731, 0.3392709, 0.3428812, 0.3464899, 0.3500756,\
        0.3536135, 0.3570789, 0.3604521, 0.3637216, 0.3668875, 0.3699611, 0.3729612, 0.3759064, 0.3788043,\
        0.3816434, 0.3843971, 0.3870388, 0.3895728, 0.3921071, 0.3949274, 0.3979683, 0.4004375, 0.4024710,\
        0.4045316, 0.4070586, 0.4097833, 0.4124868, 0.4148957, 0.4166963, 0.4176881, 0.4177934, 0.4170375,\
        0.4155636, 0.4135630, 0.4111787, 0.4085663, 0.4060448, 0.4036684, 0.4012638, 0.3992463, 0.3981424,\
        0.3973846, 0.3966530, 0.3956273, 0.3942246, 0.3926719, 0.3910697, 0.3894074, 0.3876688, 0.3858187,\
        0.3838110, 0.3816292, 0.3792959, 0.3768505, 0.3743309, 0.3717670, 0.3691807, 0.3665870, 0.3639946,\
        0.3614072, 0.3588246, 0.3562426, 0.3536546, 0.3510509, 0.3484199, 0.3457480, 0.3430213, 0.3402265,\
        0.3373526, 0.3343928, 0.3313456, 0.3282161, 0.3250167, 0.3217666, 0.3184906, 0.3152098, 0.3119156,\
        0.3085296, 0.3049607, 0.3015063, 0.2983116, 0.2947989, 0.2915319, 0.2882077, 0.2844176, 0.2800010,\
        0.2749282, 0.2692434, 0.2630136, 0.2562973],\
        [0.2065502, 0.2131137, 0.2192908, 0.2250839, 0.2305038, 0.2355706, 0.2403131, 0.2447682, 0.2489779,\
        0.2529868, 0.2568389, 0.2605741, 0.2642247, 0.2678119, 0.2713442, 0.2748183, 0.2782289, 0.2815920,\
        0.2849740, 0.2884759, 0.2921286, 0.2958485, 0.2995530, 0.3031681, 0.3066757, 0.3102188, 0.3137945,\
        0.3173706, 0.3209279, 0.3244652, 0.3279994, 0.3315525, 0.3351369, 0.3387479, 0.3423665, 0.3459661,\
        0.3495187, 0.3529993, 0.3563895, 0.3596792, 0.3628674, 0.3659611, 0.3689715, 0.3719090, 0.3747773,\
        0.3775706, 0.3802783, 0.3828978, 0.3854566, 0.3880321, 0.3906930, 0.3932845, 0.3954956, 0.3972565,\
        0.3990539, 0.4013608, 0.4038366, 0.4063965, 0.4087575, 0.4104645, 0.4113734, 0.4114410, 0.4106558,\
        0.4092156, 0.4073900, 0.4052687, 0.4027848, 0.4004647, 0.3985356, 0.3964243, 0.3946759, 0.3935290,\
        0.3927411, 0.3919895, 0.3910479, 0.3898430, 0.3884517, 0.3869480, 0.3853526, 0.3836590, 0.3818479,\
        0.3798979, 0.3778019, 0.3755729, 0.3732369, 0.3708229, 0.3683566, 0.3658581, 0.3633419, 0.3608172,\
        0.3582886, 0.3557569, 0.3532194, 0.3506700, 0.3480994, 0.3454952, 0.3428421, 0.3401235, 0.3373227,\
        0.3344258, 0.3314234, 0.3283130, 0.3251005, 0.3218005, 0.3184381, 0.3150520, 0.3116934, 0.3084026,\
        0.3051171, 0.3014965, 0.2977102, 0.2942729, 0.2907156, 0.2870877, 0.2833232, 0.2792073, 0.2745885,\
        0.2694074, 0.2636677, 0.2574034, 0.2506551],\
        [0.1994582, 0.2060971, 0.2123691, 0.2182792, 0.2238389, 0.2290671, 0.2339897, 0.2386386, 0.2430499,\
        0.2472614, 0.2513106, 0.2552314, 0.2590520, 0.2627930, 0.2664671, 0.2700819, 0.2736480, 0.2771900,\
        0.2807527, 0.2843818, 0.2880856, 0.2918239, 0.2955386, 0.2991282, 0.3026333, 0.3062207, 0.3098273,\
        0.3134131, 0.3169531, 0.3204476, 0.3239331, 0.3274545, 0.3310329, 0.3346592, 0.3383044, 0.3419327,\
        0.3455109, 0.3490126, 0.3524196, 0.3557229, 0.3589210, 0.3620184, 0.3650226, 0.3679405, 0.3707753,\
        0.3735262, 0.3761919, 0.3787783, 0.3813057, 0.3838012, 0.3862524, 0.3885525, 0.3905697, 0.3923287,\
        0.3941188, 0.3961613, 0.3983851, 0.4007991, 0.4031688, 0.4046869, 0.4054999, 0.4055858, 0.4047215,\
        0.4032282, 0.4015600, 0.3997628, 0.3975413, 0.3953772, 0.3935487, 0.3917937, 0.3902724, 0.3891202,\
        0.3882363, 0.3874166, 0.3864932, 0.3853901, 0.3841161, 0.3827051, 0.3811773, 0.3795354, 0.3777728,\
        0.3758819, 0.3738622, 0.3717242, 0.3694867, 0.3671718, 0.3648004, 0.3623898, 0.3599531, 0.3574991,\
        0.3550329, 0.3525563, 0.3500678, 0.3475626, 0.3450321, 0.3424632, 0.3398389, 0.3371391, 0.3343429,\
        0.3314319, 0.3283934, 0.3252230, 0.3219270, 0.3185211, 0.3150314, 0.3115008, 0.3080044, 0.3046410,\
        0.3014698, 0.2980369, 0.2940744, 0.2902286, 0.2863946, 0.2824616, 0.2783432, 0.2739143, 0.2690635,\
        0.2637276, 0.2578884, 0.2515563, 0.2447551],\
        [0.1922312, 0.1989568, 0.2053332, 0.2113666, 0.2170692, 0.2224584, 0.2275578, 0.2323950, 0.2370014,\
        0.2414095, 0.2456514, 0.2497568, 0.2537513, 0.2576554, 0.2614850, 0.2652541, 0.2689788, 0.2726820,\
        0.2763915, 0.2801292, 0.2838945, 0.2876598, 0.2913777, 0.2950026, 0.2985908, 0.3022408, 0.3058838,\
        0.3094943, 0.3130171, 0.3164451, 0.3198589, 0.3233461, 0.3269351, 0.3306005, 0.3342942, 0.3379673,\
        0.3415805, 0.3451059, 0.3485267, 0.3518354, 0.3550314, 0.3581189, 0.3611042, 0.3639932, 0.3667897,\
        0.3694958, 0.3721137, 0.3746485, 0.3771082, 0.3794934, 0.3817787, 0.3839069, 0.3858366, 0.3876198,\
        0.3894106, 0.3913370, 0.3934267, 0.3956779, 0.3977648, 0.3990711, 0.3998008, 0.4000026, 0.3990930,\
        0.3976049, 0.3960529, 0.3943810, 0.3924991, 0.3905904, 0.3888554, 0.3872831, 0.3859090, 0.3847734,\
        0.3838222, 0.3829348, 0.3819943, 0.3809308, 0.3797241, 0.3783805, 0.3769101, 0.3753177, 0.3736037,\
        0.3717685, 0.3698164, 0.3677576, 0.3656072, 0.3633826, 0.3611004, 0.3587747, 0.3564164, 0.3540331,\
        0.3516298, 0.3492092, 0.3467715, 0.3443137, 0.3418285, 0.3393031, 0.3367183, 0.3340499, 0.3312714,\
        0.3283581, 0.3252922, 0.3220674, 0.3186908, 0.3151816, 0.3115644, 0.3078695, 0.3041737, 0.3006261,\
        0.2972962, 0.2938320, 0.2899635, 0.2859435, 0.2818504, 0.2776317, 0.2732102, 0.2684974, 0.2634119,\
        0.2578977, 0.2519284, 0.2455004, 0.2386243],\
        [0.1848686, 0.1916899, 0.1981777, 0.2043393, 0.2101866, 0.2157361, 0.2210090, 0.2260297, 0.2308256,\
        0.2354249, 0.2398558, 0.2441448, 0.2483153, 0.2523878, 0.2563798, 0.2603076, 0.2641883, 0.2680406,\
        0.2718826, 0.2757252, 0.2795655, 0.2833854, 0.2871594, 0.2908776, 0.2945720, 0.2982665, 0.3019528,\
        0.3056132, 0.3091165, 0.3124218, 0.3157272, 0.3191910, 0.3228262, 0.3265672, 0.3303357, 0.3340684,\
        0.3377217, 0.3412690, 0.3446966, 0.3480004, 0.3511822, 0.3542475, 0.3572030, 0.3600548, 0.3628075,\
        0.3654643, 0.3680278, 0.3705001, 0.3728818, 0.3751658, 0.3773335, 0.3793605, 0.3812439, 0.3830307,\
        0.3848094, 0.3866574, 0.3885961, 0.3905536, 0.3922726, 0.3934750, 0.3941400, 0.3942197, 0.3935045,\
        0.3922388, 0.3907884, 0.3892275, 0.3875614, 0.3858790, 0.3842902, 0.3828421, 0.3815571, 0.3804390,\
        0.3794483, 0.3785089, 0.3775399, 0.3764824, 0.3753066, 0.3740036, 0.3725743, 0.3710218, 0.3693496,\
        0.3675625, 0.3656679, 0.3636766, 0.3616019, 0.3594581, 0.3572579, 0.3550122, 0.3527289, 0.3504138,\
        0.3480711, 0.3457041, 0.3433152, 0.3409044, 0.3384672, 0.3359917, 0.3334573, 0.3308346, 0.3280893,\
        0.3251881, 0.3221059, 0.3188329, 0.3153794, 0.3117743, 0.3080527, 0.3042340, 0.3003396, 0.2966408,\
        0.2930705, 0.2894027, 0.2854811, 0.2813456, 0.2770495, 0.2725819, 0.2678943, 0.2629244, 0.2576121,\
        0.2519110, 0.2457936, 0.2392490, 0.2322791],\
        [0.1773679, 0.1842920, 0.1908967, 0.1971898, 0.2031829, 0.2088917, 0.2143350, 0.2195349, 0.2245153,\
        0.2293012, 0.2339176, 0.2383883, 0.2427352, 0.2469781, 0.2511348, 0.2552219, 0.2592555, 0.2632508,\
        0.2672207, 0.2711718, 0.2751024, 0.2790035, 0.2828652, 0.2866878, 0.2904857, 0.2942678, 0.2980289,\
        0.3017717, 0.3052370, 0.3082961, 0.3114672, 0.3149623, 0.3187106, 0.3225739, 0.3264410, 0.3302407,\
        0.3339316, 0.3374924, 0.3409152, 0.3442010, 0.3473555, 0.3503867, 0.3533026, 0.3561098, 0.3588136,\
        0.3614172, 0.3639226, 0.3663300, 0.3686370, 0.3708370, 0.3729199, 0.3748787, 0.3767234, 0.3784892,\
        0.3802273, 0.3819772, 0.3837332, 0.3854139, 0.3868564, 0.3878991, 0.3884606, 0.3884776, 0.3879102,\
        0.3868857, 0.3856078, 0.3841932, 0.3827012, 0.3811976, 0.3797519, 0.3784095, 0.3771904, 0.3760905,\
        0.3750792, 0.3741041, 0.3731073, 0.3720408, 0.3708734, 0.3695896, 0.3681844, 0.3666587, 0.3650172,\
        0.3632670, 0.3614180, 0.3594820, 0.3574715, 0.3553986, 0.3532730, 0.3511017, 0.3488891, 0.3466381,\
        0.3443510, 0.3420318, 0.3396852, 0.3373159, 0.3349247, 0.3325031, 0.3300291, 0.3274673, 0.3247731,\
        0.3219013, 0.3188158, 0.3155004, 0.3119678, 0.3082637, 0.3044497, 0.3005643, 0.2965814, 0.2926536,\
        0.2887999, 0.2848791, 0.2807839, 0.2764883, 0.2719987, 0.2673061, 0.2623788, 0.2571733, 0.2516448,\
        0.2457558, 0.2394808, 0.2328059, 0.2257275],\
        [0.1697253, 0.1767579, 0.1834837, 0.1899106, 0.1960502, 0.2019167, 0.2075277, 0.2129027, 0.2180632,\
        0.2230314, 0.2278297, 0.2324798, 0.2370019, 0.2414150, 0.2457362, 0.2499815, 0.2541656, 0.2583015,\
        0.2623991, 0.2664641, 0.2704969, 0.2744946, 0.2784549, 0.2823806, 0.2862800, 0.2901611, 0.2940298,\
        0.2978712, 0.3011622, 0.3039007, 0.3070301, 0.3106906, 0.3146396, 0.3186616, 0.3226331, 0.3264918,\
        0.3302060, 0.3337635, 0.3371645, 0.3404164, 0.3435298, 0.3465155, 0.3493831, 0.3521397, 0.3547908,\
        0.3573393, 0.3597864, 0.3621314, 0.3643714, 0.3665017, 0.3685180, 0.3704208, 0.3722215, 0.3739436,\
        0.3756146, 0.3772482, 0.3788247, 0.3802764, 0.3814963, 0.3823762, 0.3828395, 0.3828408, 0.3823819,\
        0.3815424, 0.3804451, 0.3791936, 0.3778614, 0.3765097, 0.3751914, 0.3739436, 0.3727831, 0.3717059,\
        0.3706884, 0.3696922, 0.3686739, 0.3675939, 0.3664230, 0.3651435, 0.3637474, 0.3622345, 0.3606100,\
        0.3588832, 0.3570662, 0.3551723, 0.3532143, 0.3512027, 0.3491445, 0.3470425, 0.3448966, 0.3427049,\
        0.3404671, 0.3381860, 0.3358698, 0.3335300, 0.3311762, 0.3288070, 0.3264014, 0.3239159, 0.3212922,\
        0.3184693, 0.3153964, 0.3120454, 0.3084268, 0.3046041, 0.3006728, 0.2966789, 0.2926039, 0.2885017,\
        0.2843939, 0.2802181, 0.2759030, 0.2714081, 0.2667148, 0.2618056, 0.2566547, 0.2512289, 0.2454940,\
        0.2394196, 0.2329827, 0.2261689, 0.2189715],\
        [0.1619365, 0.1690821, 0.1759319, 0.1824943, 0.1887801, 0.1948030, 0.2005788, 0.2061252, 0.2114616,\
        0.2166081, 0.2215847, 0.2264112, 0.2311064, 0.2356880, 0.2401724, 0.2445744, 0.2489074, 0.2531829,\
        0.2574094, 0.2615926, 0.2657345, 0.2698352, 0.2738948, 0.2779147, 0.2818963, 0.2858346, 0.2897010,\
        0.2933698, 0.2965315, 0.2992824, 0.3025944, 0.3065350, 0.3107229, 0.3148854, 0.3189314, 0.3228206,\
        0.3265320, 0.3300621, 0.3334198, 0.3366201, 0.3396786, 0.3426088, 0.3454213, 0.3481233, 0.3507199,\
        0.3532136, 0.3556048, 0.3578922, 0.3600729, 0.3621436, 0.3641024, 0.3659517, 0.3677003, 0.3693619,\
        0.3709489, 0.3724610, 0.3738748, 0.3751376, 0.3761763, 0.3769160, 0.3772993, 0.3772972, 0.3769197,\
        0.3762208, 0.3752824, 0.3741862, 0.3730012, 0.3717837, 0.3705789, 0.3694175, 0.3683141, 0.3672660,\
        0.3662548, 0.3652510, 0.3642199, 0.3631282, 0.3619490, 0.3606644, 0.3592654, 0.3577516, 0.3561295,\
        0.3544107, 0.3526103, 0.3507442, 0.3488264, 0.3468669, 0.3448698, 0.3428336, 0.3407519, 0.3386163,\
        0.3364208, 0.3341659, 0.3318621, 0.3295306, 0.3271957, 0.3248699, 0.3225365, 0.3201418, 0.3176069,\
        0.3148514, 0.3118091, 0.3084345, 0.3047294, 0.3007732, 0.2967169, 0.2925870, 0.2883763, 0.2841034,\
        0.2797751, 0.2753617, 0.2708185, 0.2661078, 0.2612027, 0.2560800, 0.2507149, 0.2450798, 0.2391467,\
        0.2328905, 0.2262907, 0.2193331, 0.2120095],\
        [0.1539964, 0.1612583, 0.1682345, 0.1749332, 0.1813650, 0.1875424, 0.1934803, 0.1991946, 0.2047031,\
        0.2100236, 0.2151747, 0.2201744, 0.2250399, 0.2297878, 0.2344333, 0.2389902, 0.2434708, 0.2478855,\
        0.2522422, 0.2565463, 0.2608008, 0.2650067, 0.2691637, 0.2732693, 0.2773165, 0.2812842, 0.2851205,\
        0.2887204, 0.2919911, 0.2951189, 0.2985995, 0.3027535, 0.3070585, 0.3112615, 0.3153237, 0.3192044,\
        0.3228806, 0.3263549, 0.3296462, 0.3327776, 0.3357696, 0.3386372, 0.3413908, 0.3440369, 0.3465798,\
        0.3490211, 0.3513607, 0.3535964, 0.3557249, 0.3577429, 0.3596489, 0.3614442, 0.3631340, 0.3647249,\
        0.3662206, 0.3676145, 0.3688844, 0.3699909, 0.3708826, 0.3715081, 0.3718280, 0.3718245, 0.3715078,\
        0.3709169, 0.3701097, 0.3691496, 0.3680953, 0.3669968, 0.3658935, 0.3648124, 0.3637666, 0.3627544,\
        0.3617612, 0.3607627, 0.3597293, 0.3586315, 0.3574441, 0.3561492, 0.3547379, 0.3532104, 0.3515752,\
        0.3498475, 0.3480466, 0.3461924, 0.3443017, 0.3423852, 0.3404446, 0.3384731, 0.3364563, 0.3343768,\
        0.3322193, 0.3299780, 0.3276636, 0.3253080, 0.3229589, 0.3206555, 0.3183923, 0.3161006, 0.3136675,\
        0.3109864, 0.3079866, 0.3046029, 0.3008183, 0.2967482, 0.2925430, 0.2882523, 0.2838796, 0.2794289,\
        0.2748971, 0.2702643, 0.2655001, 0.2605732, 0.2554559, 0.2501234, 0.2445515, 0.2387154, 0.2325913,\
        0.2261574, 0.2193954, 0.2122918, 0.2048375],\
        [0.1458995, 0.1532803, 0.1603844, 0.1672199, 0.1737968, 0.1801271, 0.1862242, 0.1921031, 0.1977797,\
        0.2032705, 0.2085923, 0.2137615, 0.2187942, 0.2237057, 0.2285099, 0.2332199, 0.2378468, 0.2424003,\
        0.2468879, 0.2513148, 0.2556841, 0.2599968, 0.2642515, 0.2684430, 0.2725597, 0.2765779, 0.2804579,\
        0.2841566, 0.2876901, 0.2912285, 0.2950591, 0.2992875, 0.3035372, 0.3077006, 0.3117466, 0.3155915,\
        0.3192025, 0.3225931, 0.3257967, 0.3288460, 0.3317647, 0.3345676, 0.3372628, 0.3398552, 0.3423476,\
        0.3447412, 0.3470350, 0.3492258, 0.3513092, 0.3532805, 0.3551368, 0.3568776, 0.3585048, 0.3600205,\
        0.3614239, 0.3627068, 0.3638509, 0.3648269, 0.3655993, 0.3661331, 0.3664024, 0.3663967, 0.3661256,\
        0.3656178, 0.3649162, 0.3640701, 0.3631279, 0.3621330, 0.3611202, 0.3601139, 0.3591264, 0.3581569,\
        0.3571929, 0.3562127, 0.3551889, 0.3540935, 0.3529015, 0.3515947, 0.3501641, 0.3486110, 0.3469467,\
        0.3451914, 0.3433704, 0.3415101, 0.3396322, 0.3377494, 0.3358621, 0.3339572, 0.3320107, 0.3299927,\
        0.3278749, 0.3256392, 0.3232894, 0.3208659, 0.3184495, 0.3161286, 0.3139255, 0.3117478, 0.3094214,\
        0.3067878, 0.3038119, 0.3004165, 0.2965419, 0.2923728, 0.2880418, 0.2836031, 0.2790716, 0.2744486,\
        0.2697276, 0.2648926, 0.2599204, 0.2547853, 0.2494618, 0.2439260, 0.2381548, 0.2321256, 0.2258171,\
        0.2192100, 0.2122879, 0.2050377, 0.1974502],\
        [0.1376402, 0.1451418, 0.1523748, 0.1593469, 0.1660680, 0.1725491, 0.1788028, 0.1848430, 0.1906840,\
        0.1963412, 0.2018298, 0.2071649, 0.2123615, 0.2174335, 0.2223939, 0.2272549, 0.2320268, 0.2367186,\
        0.2413375, 0.2458884, 0.2503747, 0.2547971, 0.2591538, 0.2634396, 0.2676441, 0.2717514, 0.2757443,\
        0.2796218, 0.2834328, 0.2872985, 0.2913569, 0.2956082, 0.2998558, 0.3040537, 0.3081077, 0.3119077,\
        0.3154276, 0.3187106, 0.3218122, 0.3247746, 0.3276222, 0.3303655, 0.3330084, 0.3355527, 0.3380003,\
        0.3403526, 0.3426081, 0.3447623, 0.3468082, 0.3487390, 0.3505492, 0.3522363, 0.3537995, 0.3552391,\
        0.3565528, 0.3577335, 0.3587674, 0.3596338, 0.3603084, 0.3607682, 0.3609965, 0.3609879, 0.3607509,\
        0.3603073, 0.3596895, 0.3589362, 0.3580873, 0.3571804, 0.3562469, 0.3553097, 0.3543810, 0.3534606,\
        0.3525368, 0.3515883, 0.3505877, 0.3495059, 0.3483164, 0.3469993, 0.3455448, 0.3439550, 0.3422450,\
        0.3404412, 0.3385776, 0.3366899, 0.3348079, 0.3329488, 0.3311124, 0.3292792, 0.3274130, 0.3254688,\
        0.3234019, 0.3211756, 0.3187748, 0.3162344, 0.3136717, 0.3112659, 0.3091001, 0.3070401, 0.3048285,\
        0.3021716, 0.2991191, 0.2956650, 0.2917573, 0.2875431, 0.2831312, 0.2785793, 0.2739114, 0.2691335,\
        0.2642410, 0.2592216, 0.2540570, 0.2487261, 0.2432065, 0.2374764, 0.2315143, 0.2252998, 0.2188137,\
        0.2120384, 0.2049590, 0.1975631, 0.1898415],\
        [0.1292126, 0.1368364, 0.1441986, 0.1513071, 0.1581710, 0.1648009, 0.1712086, 0.1774066, 0.1834085,\
        0.1892282, 0.1948798, 0.2003772, 0.2057341, 0.2109635, 0.2160775, 0.2210873, 0.2260026, 0.2308321,\
        0.2355823, 0.2402585, 0.2448639, 0.2493996, 0.2538648, 0.2582560, 0.2625674, 0.2667930, 0.2709324,\
        0.2750025, 0.2790505, 0.2831501, 0.2873599, 0.2916626, 0.2959744, 0.3002280, 0.3043151, 0.3080591,\
        0.3114630, 0.3146250, 0.3176244, 0.3205097, 0.3233004, 0.3259986, 0.3286007, 0.3311052, 0.3335153,\
        0.3358342, 0.3380608, 0.3401882, 0.3422060, 0.3441035, 0.3458722, 0.3475074, 0.3490074, 0.3503721,\
        0.3516001, 0.3526871, 0.3536240, 0.3543973, 0.3549910, 0.3553902, 0.3555851, 0.3555733, 0.3553622,\
        0.3549685, 0.3544168, 0.3537375, 0.3529639, 0.3521292, 0.3512630, 0.3503883, 0.3495180, 0.3486525,\
        0.3477800, 0.3468779, 0.3459160, 0.3448619, 0.3436854, 0.3423631, 0.3408829, 0.3392469, 0.3374737,\
        0.3355980, 0.3336655, 0.3317251, 0.3298185, 0.3279706, 0.3261827, 0.3244283, 0.3226556, 0.3208018,\
        0.3188065, 0.3166131, 0.3141739, 0.3114885, 0.3086811, 0.3060847, 0.3039229, 0.3019346, 0.2997804,\
        0.2970979, 0.2939246, 0.2903907, 0.2864738, 0.2822449, 0.2777860, 0.2731527, 0.2683746, 0.2634634,\
        0.2584191, 0.2532336, 0.2478932, 0.2423806, 0.2366770, 0.2307631, 0.2246196, 0.2182282, 0.2115714,\
        0.2046334, 0.1974004, 0.1898607, 0.1820052],\
        [0.1206109, 0.1283577, 0.1358493, 0.1430933, 0.1500986, 0.1568752, 0.1634340, 0.1697867, 0.1759459,\
        0.1819244, 0.1877351, 0.1933910, 0.1989047, 0.2042883, 0.2095530, 0.2147093, 0.2197664, 0.2247324,\
        0.2296139, 0.2344161, 0.2391424, 0.2437951, 0.2483745, 0.2528799, 0.2573104, 0.2616674, 0.2659591,\
        0.2702059, 0.2744436, 0.2787143, 0.2830436, 0.2874182, 0.2917923, 0.2961042, 0.3002317, 0.3038998,\
        0.3071889, 0.3102451, 0.3131655, 0.3160024, 0.3187649, 0.3214415, 0.3240184, 0.3264921, 0.3288711,\
        0.3311651, 0.3333743, 0.3354878, 0.3374890, 0.3393618, 0.3410945, 0.3426807, 0.3441188, 0.3454103,\
        0.3465566, 0.3475574, 0.3484084, 0.3491018, 0.3496276, 0.3499770, 0.3501444, 0.3501298, 0.3499394,\
        0.3495855, 0.3490861, 0.3484647, 0.3477494, 0.3469707, 0.3461586, 0.3453380, 0.3445242, 0.3437189,\
        0.3429093, 0.3420695, 0.3411644, 0.3401552, 0.3390059, 0.3376875, 0.3361836, 0.3344943, 0.3326408,\
        0.3306672, 0.3286352, 0.3266123, 0.3246562, 0.3228026, 0.3210580, 0.3193904, 0.3177250, 0.3159734,\
        0.3140693, 0.3119493, 0.3095259, 0.3067461, 0.3036570, 0.3007340, 0.2985366, 0.2964752, 0.2941915,\
        0.2914672, 0.2882748, 0.2846920, 0.2807552, 0.2765116, 0.2720184, 0.2673226, 0.2624545, 0.2574295,\
        0.2522518, 0.2469177, 0.2414173, 0.2357375, 0.2298627, 0.2237763, 0.2174614, 0.2109017, 0.2040815,\
        0.1969865, 0.1896040, 0.1819232, 0.1739350],\
        [0.1118292, 0.1196993, 0.1273200, 0.1346987, 0.1418438, 0.1487648, 0.1554719, 0.1619762, 0.1682891,\
        0.1744226, 0.1803888, 0.1861994, 0.1918664, 0.1974008, 0.2028133, 0.2081136, 0.2133105, 0.2184117,\
        0.2234238, 0.2283520, 0.2332005, 0.2379722, 0.2426692, 0.2472932, 0.2518470, 0.2563358, 0.2607706,\
        0.2651693, 0.2695562, 0.2739543, 0.2783734, 0.2828000, 0.2871961, 0.2914952, 0.2955682, 0.2992265,\
        0.3024962, 0.3054965, 0.3083873, 0.3112225, 0.3139962, 0.3166810, 0.3192494, 0.3216965, 0.3240467,\
        0.3263249, 0.3285318, 0.3306476, 0.3326463, 0.3345046, 0.3362071, 0.3377468, 0.3391240, 0.3403434,\
        0.3414112, 0.3423318, 0.3431060, 0.3437305, 0.3441992, 0.3445072, 0.3446522, 0.3446361, 0.3444639,\
        0.3441437, 0.3436870, 0.3431105, 0.3424375, 0.3416976, 0.3409239, 0.3401464, 0.3393852, 0.3386444,\
        0.3379099, 0.3371503, 0.3363222, 0.3353783, 0.3342741, 0.3329738, 0.3314537, 0.3297086, 0.3277597,\
        0.3256607, 0.3234943, 0.3213548, 0.3193209, 0.3174386, 0.3157243, 0.3141515, 0.3126073, 0.3109490,\
        0.3091258, 0.3071120, 0.3047685, 0.3019899, 0.2988468, 0.2957362, 0.2931996, 0.2908468, 0.2883380,\
        0.2854896, 0.2822435, 0.2786197, 0.2746535, 0.2703846, 0.2658555, 0.2611039, 0.2561575, 0.2510327,\
        0.2457366, 0.2402688, 0.2346231, 0.2287895, 0.2227558, 0.2165079, 0.2100315, 0.2033121, 0.1963359,\
        0.1890900, 0.1815627, 0.1737438, 0.1656249],\
        [0.1028617, 0.1108552, 0.1186044, 0.1261165, 0.1333996, 0.1404627, 0.1473154, 0.1539680, 0.1604312,\
        0.1667161, 0.1728339, 0.1787956, 0.1846122, 0.1902941, 0.1958512, 0.2012928, 0.2066272, 0.2118618,\
        0.2170032, 0.2220569, 0.2270274, 0.2319185, 0.2367337, 0.2414763, 0.2461511, 0.2507648, 0.2553277,\
        0.2598532, 0.2643564, 0.2688479, 0.2733275, 0.2777769, 0.2821560, 0.2863956, 0.2903926, 0.2940493,\
        0.2973494, 0.3003657, 0.3032932, 0.3061723, 0.3089945, 0.3117183, 0.3142913, 0.3167036, 0.3190186,\
        0.3212931, 0.3235195, 0.3256584, 0.3276707, 0.3295252, 0.3312025, 0.3326968, 0.3340126, 0.3351598,\
        0.3361504, 0.3369954, 0.3377010, 0.3382665, 0.3386876, 0.3389610, 0.3390876, 0.3390718, 0.3389182,\
        0.3386303, 0.3382117, 0.3376705, 0.3370246, 0.3363046, 0.3355501, 0.3348006, 0.3340850, 0.3334119,\
        0.3327652, 0.3321055, 0.3313771, 0.3305204, 0.3294821, 0.3282188, 0.3266979, 0.3249032, 0.3228495,\
        0.3205980, 0.3182590, 0.3159672, 0.3138307, 0.3118939, 0.3101746, 0.3086987, 0.3073048, 0.3056924,\
        0.3038753, 0.3019296, 0.2997039, 0.2968673, 0.2937664, 0.2906554, 0.2878029, 0.2850973, 0.2823094,\
        0.2792759, 0.2759197, 0.2722263, 0.2682108, 0.2639003, 0.2593260, 0.2545170, 0.2494965, 0.2442803,\
        0.2388764, 0.2332867, 0.2275077, 0.2215324, 0.2153510, 0.2089522, 0.2023237, 0.1954530, 0.1883280,\
        0.1809371, 0.1732697, 0.1653164, 0.1570693],\
        [0.0937025, 0.1018191, 0.1096960, 0.1173402, 0.1247595, 0.1319623, 0.1389578, 0.1457555, 0.1523656,\
        0.1587983, 0.1650640, 0.1711730, 0.1771356, 0.1829615, 0.1886601, 0.1942400, 0.1997092, 0.2050750,\
        0.2103439, 0.2155214, 0.2206125, 0.2256217, 0.2305531, 0.2354111, 0.2402009, 0.2449292, 0.2496042,\
        0.2542347, 0.2588285, 0.2633880, 0.2679058, 0.2723598, 0.2767101, 0.2808969, 0.2848479, 0.2885046,\
        0.2918574, 0.2949697, 0.2979531, 0.3008901, 0.3037723, 0.3065621, 0.3091515, 0.3114926, 0.3137549,\
        0.3160530, 0.3183315, 0.3205160, 0.3225582, 0.3244188, 0.3260732, 0.3275204, 0.3287728, 0.3298457,\
        0.3307582, 0.3315309, 0.3321762, 0.3326935, 0.3330760, 0.3333201, 0.3334306, 0.3334169, 0.3332853,\
        0.3330338, 0.3326551, 0.3321443, 0.3315117, 0.3307897, 0.3300298, 0.3292878, 0.3286068, 0.3280024,\
        0.3274569, 0.3269189, 0.3263140, 0.3255656, 0.3246124, 0.3234083, 0.3219119, 0.3200882, 0.3179334,\
        0.3155088, 0.3129574, 0.3104779, 0.3082322, 0.3062375, 0.3044433, 0.3030095, 0.3018352, 0.3001739,\
        0.2982898, 0.2962877, 0.2939921, 0.2912497, 0.2882238, 0.2851274, 0.2821006, 0.2791230, 0.2760748,\
        0.2728382, 0.2693389, 0.2655459, 0.2614574, 0.2570869, 0.2524547, 0.2475818, 0.2424866, 0.2371825,\
        0.2316774, 0.2259744, 0.2200717, 0.2139646, 0.2076454, 0.2011049, 0.1943330, 0.1873189, 0.1800520,\
        0.1725220, 0.1647194, 0.1566355, 0.1482628],\
        [0.0843460, 0.0925851, 0.1005887, 0.1083635, 0.1159170, 0.1232571, 0.1303926, 0.1373323, 0.1440858,\
        0.1506627, 0.1570727, 0.1633253, 0.1694303, 0.1753967, 0.1812332, 0.1869483, 0.1925495, 0.1980438,\
        0.2034375, 0.2087365, 0.2139457, 0.2190701, 0.2241139, 0.2290821, 0.2339794, 0.2388115, 0.2435840,\
        0.2483020, 0.2529679, 0.2575785, 0.2621223, 0.2665763, 0.2709048, 0.2750613, 0.2789969, 0.2826779,\
        0.2861051, 0.2893236, 0.2924046, 0.2954046, 0.2983435, 0.3011998, 0.3038381, 0.3060130, 0.3082164,\
        0.3106133, 0.3129776, 0.3152185, 0.3173055, 0.3191815, 0.3208090, 0.3222031, 0.3233904, 0.3243863,\
        0.3252160, 0.3259177, 0.3265135, 0.3269972, 0.3273513, 0.3275689, 0.3276616, 0.3276502, 0.3275466,\
        0.3273428, 0.3270147, 0.3265369, 0.3259055, 0.3251557, 0.3243589, 0.3235975, 0.3229349, 0.3223971,\
        0.3219658, 0.3215745, 0.3211173, 0.3204909, 0.3196320, 0.3185059, 0.3170693, 0.3152581, 0.3130289,\
        0.3104306, 0.3076377, 0.3049377, 0.3025982, 0.3005909, 0.2987393, 0.2972103, 0.2959647, 0.2943302,\
        0.2924034, 0.2902880, 0.2879072, 0.2852006, 0.2822375, 0.2791444, 0.2760099, 0.2728410, 0.2695802,\
        0.2661533, 0.2625013, 0.2585908, 0.2544104, 0.2499631, 0.2452600, 0.2403150, 0.2351414, 0.2297498,\
        0.2241472, 0.2183365, 0.2123174, 0.2060863, 0.1996376, 0.1929637, 0.1860561, 0.1789058, 0.1715034,\
        0.1638399, 0.1559068, 0.1476960, 0.1392007],\
        [0.0747867, 0.0831476, 0.0912767, 0.0991804, 0.1068660, 0.1143410, 0.1216135, 0.1286922, 0.1355858,\
        0.1423033, 0.1488539, 0.1552465, 0.1614901, 0.1675933, 0.1735645, 0.1794113, 0.1851412, 0.1907609,\
        0.1962765, 0.2016938, 0.2070179, 0.2122535, 0.2174052, 0.2224774, 0.2274744, 0.2324003, 0.2372585,\
        0.2420510, 0.2467765, 0.2514288, 0.2559946, 0.2604524, 0.2647729, 0.2689215, 0.2728662, 0.2765884,\
        0.2800931, 0.2834114, 0.2865893, 0.2896663, 0.2926600, 0.2955524, 0.2981771, 0.3001914, 0.3024512,\
        0.3050509, 0.3074721, 0.3097507, 0.3119061, 0.3138104, 0.3153899, 0.3167185, 0.3178499, 0.3187687,\
        0.3195017, 0.3201303, 0.3206948, 0.3211676, 0.3215062, 0.3216955, 0.3217608, 0.3217476, 0.3216803,\
        0.3215434, 0.3212894, 0.3208588, 0.3202198, 0.3194115, 0.3185386, 0.3177247, 0.3170615, 0.3165827,\
        0.3162742, 0.3160591, 0.3157749, 0.3152668, 0.3144852, 0.3134391, 0.3121013, 0.3103674, 0.3081175,\
        0.3053864, 0.3023787, 0.2994926, 0.2970646, 0.2949923, 0.2931061, 0.2914382, 0.2898881, 0.2881659,\
        0.2861907, 0.2839848, 0.2815295, 0.2788089, 0.2758589, 0.2727479, 0.2695305, 0.2662195, 0.2627889,\
        0.2591946, 0.2553947, 0.2513600, 0.2470760, 0.2425394, 0.2377544, 0.2327292, 0.2274725, 0.2219920,\
        0.2162931, 0.2103785, 0.2042480, 0.1978992, 0.1913277, 0.1845272, 0.1774907, 0.1702105, 0.1626785,\
        0.1548868, 0.1468276, 0.1384938, 0.1298789],\
        [0.0650191, 0.0735008, 0.0817540, 0.0897850, 0.0976005, 0.1052079, 0.1126147, 0.1198292, 0.1268596,\
        0.1337143, 0.1404018, 0.1469307, 0.1533093, 0.1595457, 0.1656478, 0.1716229, 0.1774781, 0.1832197,\
        0.1888538, 0.1943859, 0.1998209, 0.2051635, 0.2104180, 0.2155881, 0.2206775, 0.2256887, 0.2306236,\
        0.2354816, 0.2402594, 0.2449492, 0.2495379, 0.2540070, 0.2583331, 0.2624914, 0.2664613, 0.2702322,\
        0.2738095, 0.2772140, 0.2804742, 0.2836130, 0.2866323, 0.2894879, 0.2920532, 0.2943160, 0.2967516,\
        0.2993872, 0.3017637, 0.3040701, 0.3063511, 0.3083072, 0.3097655, 0.3110163, 0.3121424, 0.3129915,\
        0.3135868, 0.3141363, 0.3147048, 0.3152019, 0.3155409, 0.3156933, 0.3157077, 0.3156816, 0.3156612,\
        0.3156160, 0.3154734, 0.3151230, 0.3144734, 0.3135704, 0.3125775, 0.3116774, 0.3110005, 0.3105650,\
        0.3103668, 0.3103641, 0.3102851, 0.3098573, 0.3090918, 0.3080933, 0.3068733, 0.3052911, 0.3030807,\
        0.3002670, 0.2972049, 0.2942553, 0.2916754, 0.2894225, 0.2873923, 0.2855228, 0.2837117, 0.2818075,\
        0.2797175, 0.2774125, 0.2748782, 0.2721116, 0.2691332, 0.2659799, 0.2626839, 0.2592561, 0.2556848,\
        0.2519443, 0.2480062, 0.2438477, 0.2394545, 0.2348204, 0.2299454, 0.2248332, 0.2194887, 0.2139171,\
        0.2081220, 0.2021054, 0.1958672, 0.1894054, 0.1827163, 0.1757950, 0.1686354, 0.1612309, 0.1535746,\
        0.1456593, 0.1374783, 0.1290250, 0.1202934],\
        [0.0550380, 0.0636395, 0.0720153, 0.0801717, 0.0881149, 0.0958521, 0.1033905, 0.1107377, 0.1179015,\
        0.1248899, 0.1317109, 0.1383724, 0.1448824, 0.1512483, 0.1574777, 0.1635775, 0.1695543, 0.1754143,\
        0.1811631, 0.1868062, 0.1923481, 0.1977932, 0.2031454, 0.2084079, 0.2135833, 0.2186732, 0.2236780,\
        0.2285957, 0.2334218, 0.2381484, 0.2427634, 0.2472514, 0.2515945, 0.2557749, 0.2597789, 0.2636003,\
        0.2672433, 0.2707206, 0.2740484, 0.2772368, 0.2802791, 0.2831436, 0.2857956, 0.2883004, 0.2908372,\
        0.2934001, 0.2958260, 0.2981962, 0.3006072, 0.3026450, 0.3038147, 0.3050270, 0.3062839, 0.3070744,\
        0.3074260, 0.3079034, 0.3085438, 0.3091011, 0.3094581, 0.3095624, 0.3094810, 0.3094277, 0.3094700,\
        0.3095319, 0.3095408, 0.3093355, 0.3086759, 0.3076399, 0.3065018, 0.3054876, 0.3048150, 0.3044068,\
        0.3042416, 0.3044830, 0.3046496, 0.3042030, 0.3033628, 0.3023518, 0.3011949, 0.2997398, 0.2975864,\
        0.2947874, 0.2917831, 0.2888471, 0.2861549, 0.2837266, 0.2815050, 0.2794164, 0.2773698, 0.2752637,\
        0.2730194, 0.2705930, 0.2679634, 0.2651254, 0.2620885, 0.2588718, 0.2554929, 0.2519590, 0.2482642,\
        0.2443930, 0.2403265, 0.2360475, 0.2315436, 0.2268076, 0.2218370, 0.2166325, 0.2111963, 0.2055312,\
        0.1996394, 0.1935219, 0.1871783, 0.1806070, 0.1738046, 0.1667671, 0.1594893, 0.1519654, 0.1441893,\
        0.1361548, 0.1278558, 0.1192864, 0.1104411],\
        [0.0448382, 0.0535583, 0.0620552, 0.0703350, 0.0784038, 0.0862683, 0.0939354, 0.1014123, 0.1087064,\
        0.1158251, 0.1227760, 0.1295665, 0.1362041, 0.1426959, 0.1490489, 0.1552697, 0.1613645, 0.1673391,\
        0.1731990, 0.1789491, 0.1845939, 0.1901371, 0.1955823, 0.2009321, 0.2061884, 0.2113518, 0.2164218,\
        0.2213957, 0.2262687, 0.2310329, 0.2356782, 0.2401919, 0.2445604, 0.2487708, 0.2528138, 0.2566852,\
        0.2603875, 0.2639280, 0.2673152, 0.2705532, 0.2736377, 0.2765582, 0.2793190, 0.2819695, 0.2845787,\
        0.2871498, 0.2896469, 0.2920954, 0.2944831, 0.2963767, 0.2975316, 0.2988097, 0.3002429, 0.3009724,\
        0.3009938, 0.3014805, 0.3022375, 0.3028401, 0.3032251, 0.3033044, 0.3030700, 0.3029946, 0.3031237,\
        0.3032638, 0.3034105, 0.3034590, 0.3027719, 0.3015913, 0.3003847, 0.2992688, 0.2986292, 0.2982418,\
        0.2980662, 0.2984055, 0.2986862, 0.2981606, 0.2972672, 0.2962206, 0.2950123, 0.2934729, 0.2913773,\
        0.2887660, 0.2859086, 0.2830407, 0.2803064, 0.2777523, 0.2753570, 0.2730655, 0.2708062, 0.2685031,\
        0.2660919, 0.2635285, 0.2607873, 0.2578576, 0.2547404, 0.2514435, 0.2479751, 0.2443383, 0.2405292,\
        0.2365376, 0.2323504, 0.2279549, 0.2233406, 0.2185005, 0.2134309, 0.2081304, 0.2025995, 0.1968389,\
        0.1908496, 0.1846317, 0.1781845, 0.1715062, 0.1645939, 0.1574440, 0.1500520, 0.1424128, 0.1345210,\
        0.1263712, 0.1179577, 0.1092754, 0.1003190],\
        [0.0344150, 0.0432522, 0.0518686, 0.0602698, 0.0684619, 0.0764512, 0.0842443, 0.0918479, 0.0992690,\
        0.1065147, 0.1135920, 0.1205080, 0.1272697, 0.1338837, 0.1403567, 0.1466948, 0.1529039, 0.1589895,\
        0.1649567, 0.1708100, 0.1765536, 0.1821909, 0.1877249, 0.1931577, 0.1984906, 0.2037238, 0.2088559,\
        0.2138840, 0.2188034, 0.2236071, 0.2282863, 0.2328311, 0.2372309, 0.2414764, 0.2455608, 0.2494810,\
        0.2532380, 0.2568359, 0.2602794, 0.2635712, 0.2667114, 0.2697017, 0.2725550, 0.2753017, 0.2779729,\
        0.2805761, 0.2831004, 0.2855264, 0.2877636, 0.2895999, 0.2910246, 0.2923650, 0.2936382, 0.2943491,\
        0.2945732, 0.2950445, 0.2957356, 0.2963203, 0.2967072, 0.2967961, 0.2965746, 0.2965153, 0.2966624,\
        0.2968343, 0.2970265, 0.2971538, 0.2965210, 0.2953752, 0.2942032, 0.2931839, 0.2924889, 0.2920828,\
        0.2919386, 0.2920577, 0.2920749, 0.2916115, 0.2907659, 0.2896865, 0.2883819, 0.2867528, 0.2846983,\
        0.2822459, 0.2795480, 0.2767735, 0.2740428, 0.2714112, 0.2688807, 0.2664185, 0.2639736, 0.2614892,\
        0.2589138, 0.2562076, 0.2533441, 0.2503088, 0.2470963, 0.2437068, 0.2401422, 0.2364023, 0.2324831,\
        0.2283772, 0.2240750, 0.2195664, 0.2148430, 0.2098982, 0.2047276, 0.1993289, 0.1937010, 0.1878434,\
        0.1817559, 0.1754378, 0.1688882, 0.1621049, 0.1550854, 0.1478262, 0.1403233, 0.1325723, 0.1245683,\
        0.1163065, 0.1077818, 0.0989893, 0.0899244],\
        [0.0237634, 0.0327163, 0.0414504, 0.0499712, 0.0582843, 0.0663959, 0.0743121, 0.0820395, 0.0895845,\
        0.0969539, 0.1041543, 0.1111923, 0.1180745, 0.1248073, 0.1313966, 0.1378485, 0.1441683, 0.1503612,\
        0.1564320, 0.1623849, 0.1682235, 0.1739512, 0.1795702, 0.1850825, 0.1904887, 0.1957886, 0.2009808,\
        0.2060622, 0.2110283, 0.2158732, 0.2205896, 0.2251695, 0.2296048, 0.2338885, 0.2380154, 0.2419831,\
        0.2457918, 0.2494437, 0.2529419, 0.2562891, 0.2594886, 0.2625459, 0.2654733, 0.2682885, 0.2710069,\
        0.2736315, 0.2761487, 0.2785224, 0.2806778, 0.2825284, 0.2840855, 0.2854482, 0.2865941, 0.2873766,\
        0.2878797, 0.2883956, 0.2889832, 0.2895067, 0.2898620, 0.2899870, 0.2899423, 0.2899396, 0.2900493,\
        0.2902047, 0.2903384, 0.2902839, 0.2897620, 0.2888427, 0.2878238, 0.2869089, 0.2862149, 0.2857626,\
        0.2855212, 0.2853999, 0.2851822, 0.2846682, 0.2838329, 0.2827256, 0.2813537, 0.2796746, 0.2776505,\
        0.2753067, 0.2727349, 0.2700490, 0.2673415, 0.2646660, 0.2620363, 0.2594372, 0.2568354, 0.2541899,\
        0.2514605, 0.2486134, 0.2456237, 0.2424747, 0.2391573, 0.2356668, 0.2320008, 0.2281564, 0.2241291,\
        0.2199125, 0.2154989, 0.2108801, 0.2060490, 0.2009995, 0.1957271, 0.1902287, 0.1845024, 0.1785467,\
        0.1723605, 0.1659425, 0.1592912, 0.1524046, 0.1452799, 0.1379139, 0.1303031, 0.1224433, 0.1143302,\
        0.1059593, 0.0973262, 0.0884262, 0.0792551],\
        [0.0128789, 0.0219461, 0.0307961, 0.0394344, 0.0478663, 0.0560976, 0.0641343, 0.0719825, 0.0796484,\
        0.0871383, 0.0944585, 0.1016152, 0.1086145, 0.1154624, 0.1221646, 0.1287267, 0.1351537, 0.1414505,\
        0.1476214, 0.1536703, 0.1596007, 0.1654152, 0.1711162, 0.1767048, 0.1821817, 0.1875463, 0.1927971,\
        0.1979312, 0.2029448, 0.2078326, 0.2125886, 0.2172066, 0.2216801, 0.2260038, 0.2301737, 0.2341878,\
        0.2380458, 0.2417491, 0.2453000, 0.2487014, 0.2519573, 0.2550738, 0.2580597, 0.2609250, 0.2636763,\
        0.2663107, 0.2688121, 0.2711472, 0.2732678, 0.2751335, 0.2767442, 0.2781235, 0.2792609, 0.2801322,\
        0.2807994, 0.2813866, 0.2819426, 0.2824231, 0.2827682, 0.2829540, 0.2830280, 0.2830873, 0.2831804,\
        0.2832806, 0.2833141, 0.2831583, 0.2826985, 0.2819669, 0.2811206, 0.2803135, 0.2796418, 0.2791347,\
        0.2787635, 0.2784416, 0.2780329, 0.2774141, 0.2765327, 0.2753849, 0.2739669, 0.2722620, 0.2702615,\
        0.2679899, 0.2655076, 0.2628913, 0.2602096, 0.2575083, 0.2548060, 0.2520986, 0.2493661, 0.2465801,\
        0.2437103, 0.2407290, 0.2376138, 0.2343482, 0.2309210, 0.2273246, 0.2235538, 0.2196037, 0.2154691,\
        0.2111438, 0.2066213, 0.2018944, 0.1969570, 0.1918033, 0.1864288, 0.1808301, 0.1750046, 0.1689501,\
        0.1626649, 0.1561472, 0.1493951, 0.1424063, 0.1351782, 0.1277076, 0.1199912, 0.1120252, 0.1038056,\
        0.0953284, 0.0865892, 0.0775841, 0.0683091],\
        [0.0017571, 0.0109369, 0.0199011, 0.0286548, 0.0372032, 0.0455519, 0.0537064, 0.0616726, 0.0694564,\
        0.0770637, 0.0845005, 0.0917725, 0.0988856, 0.1058453, 0.1126570, 0.1193259, 0.1258568, 0.1322541,\
        0.1385218, 0.1446635, 0.1506824, 0.1565809, 0.1623609, 0.1680234, 0.1735688, 0.1789965, 0.1843050,\
        0.1894918, 0.1945533, 0.1994853, 0.2042829, 0.2089410, 0.2134545, 0.2178193, 0.2220322, 0.2260915,\
        0.2299968, 0.2337490, 0.2373498, 0.2408020, 0.2441093, 0.2472763, 0.2503088, 0.2532117, 0.2559860,\
        0.2586256, 0.2611150, 0.2634284, 0.2655351, 0.2674116, 0.2690514, 0.2704586, 0.2716341, 0.2725871,\
        0.2733605, 0.2740154, 0.2745852, 0.2750612, 0.2754211, 0.2756602, 0.2758080, 0.2759115, 0.2759960,\
        0.2760448, 0.2760047, 0.2758023, 0.2753870, 0.2747799, 0.2740680, 0.2733513, 0.2727011, 0.2721425,\
        0.2716536, 0.2711704, 0.2706028, 0.2698666, 0.2689094, 0.2677075, 0.2662491, 0.2645264, 0.2625414,\
        0.2603162, 0.2578931, 0.2553255, 0.2526635, 0.2499440, 0.2471860, 0.2443917, 0.2415501, 0.2386423,\
        0.2356460, 0.2325393, 0.2293030, 0.2259215, 0.2223830, 0.2186786, 0.2148012, 0.2107448, 0.2065034,\
        0.2020711, 0.1974414, 0.1926082, 0.1875656, 0.1823084, 0.1768320, 0.1711329, 0.1652078, 0.1590543,\
        0.1526701, 0.1460530, 0.1392008, 0.1321109, 0.1247808, 0.1172075, 0.1093875, 0.1013175, 0.0929937,\
        0.0844123, 0.0755695, 0.0664615, 0.0570844]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=20 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax20vt0
    zgrid_H_Hmax_vmax20vt0=[[0.57080, 0.57220, 0.57363, 0.57511, 0.57664, 0.57824, 0.57992, 0.58170, 0.58357, 0.58554, 0.58761,\
        0.58976, 0.59198, 0.59421, 0.59639, 0.59842, 0.60002, 0.60081, 0.60124, 0.60145, 0.60146, 0.60131,\
        0.60106, 0.60075, 0.60050, 0.60044, 0.60071, 0.60144, 0.60267, 0.60441, 0.60658, 0.60913, 0.61196,\
        0.61501, 0.61821, 0.62150, 0.62482, 0.62815, 0.63144, 0.63465, 0.63775, 0.64072, 0.64351, 0.64609,\
        0.64842, 0.65048, 0.65222, 0.65363, 0.65471, 0.65547, 0.65593, 0.65612, 0.65605, 0.65572, 0.65515,\
        0.65436, 0.65337, 0.65220, 0.65085, 0.64933, 0.64762, 0.64572, 0.64360, 0.64127, 0.63870, 0.63591,\
        0.63290, 0.62970, 0.62637, 0.62294, 0.61949, 0.61609, 0.61278, 0.60955, 0.60631, 0.60276, 0.59830,\
        0.59269, 0.58655, 0.58019, 0.57374, 0.56727, 0.56081, 0.55440, 0.54813, 0.54207, 0.53633, 0.53106,\
        0.52639, 0.52245, 0.51930, 0.51694, 0.51527, 0.51411, 0.51326, 0.51256, 0.51183, 0.51097, 0.50985,\
        0.50835, 0.50628, 0.50273, 0.49504, 0.48468, 0.47337, 0.46209, 0.45151, 0.44209, 0.43404, 0.42736,\
        0.42188, 0.41741, 0.41376, 0.41077, 0.40834, 0.40635, 0.40472, 0.40331, 0.40201, 0.40073, 0.39943],\
        [0.57234, 0.57357, 0.57483, 0.57614, 0.57749, 0.57892, 0.58044, 0.58205, 0.58377, 0.58560, 0.58753,\
        0.58956, 0.59167, 0.59381, 0.59591, 0.59786, 0.59939, 0.60033, 0.60098, 0.60137, 0.60154, 0.60154,\
        0.60140, 0.60117, 0.60096, 0.60096, 0.60137, 0.60234, 0.60392, 0.60608, 0.60871, 0.61171, 0.61499,\
        0.61846, 0.62204, 0.62568, 0.62934, 0.63297, 0.63654, 0.64001, 0.64337, 0.64658, 0.64959, 0.65239,\
        0.65492, 0.65714, 0.65901, 0.66051, 0.66163, 0.66240, 0.66285, 0.66301, 0.66290, 0.66250, 0.66183,\
        0.66093, 0.65981, 0.65850, 0.65701, 0.65533, 0.65346, 0.65136, 0.64903, 0.64644, 0.64358, 0.64043,\
        0.63701, 0.63335, 0.62948, 0.62548, 0.62143, 0.61742, 0.61355, 0.60985, 0.60628, 0.60253, 0.59793,\
        0.59216, 0.58597, 0.57962, 0.57317, 0.56665, 0.56007, 0.55346, 0.54689, 0.54046, 0.53429, 0.52855,\
        0.52343, 0.51909, 0.51568, 0.51321, 0.51153, 0.51039, 0.50953, 0.50874, 0.50783, 0.50667, 0.50515,\
        0.50315, 0.50040, 0.49665, 0.48869, 0.47791, 0.46615, 0.45449, 0.44373, 0.43445, 0.42686, 0.42087,\
        0.41617, 0.41247, 0.40952, 0.40718, 0.40536, 0.40397, 0.40291, 0.40205, 0.40123, 0.40035, 0.39940],\
        [0.57379, 0.57484, 0.57592, 0.57704, 0.57821, 0.57945, 0.58078, 0.58221, 0.58375, 0.58539, 0.58715,\
        0.58901, 0.59095, 0.59291, 0.59484, 0.59662, 0.59809, 0.59924, 0.60014, 0.60073, 0.60113, 0.60136,\
        0.60140, 0.60129, 0.60115, 0.60123, 0.60183, 0.60315, 0.60521, 0.60790, 0.61109, 0.61461, 0.61837,\
        0.62227, 0.62625, 0.63024, 0.63422, 0.63814, 0.64198, 0.64571, 0.64931, 0.65276, 0.65600, 0.65902,\
        0.66176, 0.66417, 0.66619, 0.66779, 0.66894, 0.66971, 0.67016, 0.67032, 0.67018, 0.66972, 0.66896,\
        0.66792, 0.66667, 0.66523, 0.66359, 0.66176, 0.65972, 0.65744, 0.65490, 0.65205, 0.64888, 0.64537,\
        0.64151, 0.63733, 0.63288, 0.62822, 0.62345, 0.61866, 0.61400, 0.60958, 0.60542, 0.60120, 0.59633,\
        0.59083, 0.58496, 0.57889, 0.57266, 0.56625, 0.55966, 0.55292, 0.54609, 0.53928, 0.53265, 0.52637,\
        0.52068, 0.51585, 0.51210, 0.50950, 0.50785, 0.50675, 0.50587, 0.50495, 0.50378, 0.50222, 0.50015,\
        0.49745, 0.49380, 0.48850, 0.48037, 0.46970, 0.45783, 0.44598, 0.43518, 0.42620, 0.41934, 0.41432,\
        0.41061, 0.40777, 0.40557, 0.40388, 0.40264, 0.40181, 0.40134, 0.40103, 0.40066, 0.40015, 0.39951],\
        [0.57517, 0.57604, 0.57693, 0.57784, 0.57881, 0.57985, 0.58098, 0.58221, 0.58355, 0.58499, 0.58655,\
        0.58821, 0.58994, 0.59170, 0.59344, 0.59506, 0.59649, 0.59772, 0.59875, 0.59957, 0.60023, 0.60073,\
        0.60102, 0.60107, 0.60104, 0.60124, 0.60212, 0.60396, 0.60669, 0.61007, 0.61388, 0.61797, 0.62223,\
        0.62656, 0.63091, 0.63525, 0.63952, 0.64371, 0.64780, 0.65177, 0.65560, 0.65927, 0.66274, 0.66598,\
        0.66894, 0.67157, 0.67378, 0.67549, 0.67668, 0.67743, 0.67788, 0.67806, 0.67794, 0.67744, 0.67657,\
        0.67540, 0.67401, 0.67243, 0.67066, 0.66868, 0.66647, 0.66401, 0.66126, 0.65818, 0.65471, 0.65082,\
        0.64651, 0.64181, 0.63674, 0.63137, 0.62578, 0.62009, 0.61445, 0.60904, 0.60406, 0.59936, 0.59440,\
        0.58927, 0.58394, 0.57837, 0.57251, 0.56634, 0.55983, 0.55302, 0.54596, 0.53879, 0.53166, 0.52478,\
        0.51842, 0.51294, 0.50871, 0.50598, 0.50445, 0.50343, 0.50251, 0.50146, 0.50002, 0.49809, 0.49550,\
        0.49203, 0.48733, 0.48085, 0.47212, 0.46133, 0.44935, 0.43731, 0.42642, 0.41780, 0.41196, 0.40823,\
        0.40564, 0.40366, 0.40214, 0.40105, 0.40035, 0.40004, 0.40012, 0.40032, 0.40033, 0.40013, 0.39977],\
        [0.57652, 0.57719, 0.57788, 0.57859, 0.57936, 0.58019, 0.58110, 0.58212, 0.58325, 0.58449, 0.58584,\
        0.58728, 0.58881, 0.59037, 0.59192, 0.59340, 0.59478, 0.59602, 0.59711, 0.59805, 0.59887, 0.59959,\
        0.60008, 0.60037, 0.60053, 0.60095, 0.60233, 0.60503, 0.60866, 0.61284, 0.61733, 0.62198, 0.62671,\
        0.63143, 0.63613, 0.64076, 0.64529, 0.64971, 0.65401, 0.65819, 0.66223, 0.66612, 0.66981, 0.67326,\
        0.67645, 0.67932, 0.68177, 0.68363, 0.68484, 0.68556, 0.68601, 0.68628, 0.68625, 0.68573, 0.68474,\
        0.68342, 0.68190, 0.68019, 0.67829, 0.67615, 0.67377, 0.67113, 0.66819, 0.66488, 0.66113, 0.65688,\
        0.65214, 0.64690, 0.64123, 0.63515, 0.62876, 0.62213, 0.61540, 0.60879, 0.60266, 0.59739, 0.59270,\
        0.58816, 0.58348, 0.57850, 0.57311, 0.56724, 0.56086, 0.55402, 0.54677, 0.53925, 0.53164, 0.52413,\
        0.51703, 0.51072, 0.50576, 0.50282, 0.50156, 0.50065, 0.49966, 0.49849, 0.49687, 0.49462, 0.49155,\
        0.48737, 0.48178, 0.47442, 0.46511, 0.45404, 0.44185, 0.42952, 0.41831, 0.40989, 0.40540, 0.40326,\
        0.40175, 0.40043, 0.39947, 0.39887, 0.39865, 0.39881, 0.39930, 0.39987, 0.40019, 0.40025, 0.40014],\
        [0.57785, 0.57833, 0.57882, 0.57932, 0.57988, 0.58049, 0.58120, 0.58201, 0.58293, 0.58397, 0.58512,\
        0.58637, 0.58771, 0.58910, 0.59051, 0.59190, 0.59324, 0.59450, 0.59564, 0.59664, 0.59748, 0.59814,\
        0.59861, 0.59892, 0.59931, 0.60026, 0.60284, 0.60689, 0.61160, 0.61659, 0.62170, 0.62684, 0.63195,\
        0.63700, 0.64197, 0.64684, 0.65158, 0.65619, 0.66066, 0.66500, 0.66921, 0.67330, 0.67719, 0.68084,\
        0.68423, 0.68735, 0.69012, 0.69221, 0.69342, 0.69411, 0.69459, 0.69501, 0.69517, 0.69464, 0.69352,\
        0.69205, 0.69041, 0.68860, 0.68655, 0.68424, 0.68167, 0.67885, 0.67574, 0.67223, 0.66822, 0.66363,\
        0.65847, 0.65275, 0.64652, 0.63981, 0.63270, 0.62525, 0.61757, 0.60987, 0.60253, 0.59672, 0.59229,\
        0.58830, 0.58423, 0.57978, 0.57482, 0.56925, 0.56302, 0.55615, 0.54873, 0.54091, 0.53286, 0.52480,\
        0.51700, 0.50982, 0.50382, 0.50017, 0.49929, 0.49849, 0.49754, 0.49632, 0.49460, 0.49213, 0.48868,\
        0.48395, 0.47770, 0.46972, 0.45998, 0.44870, 0.43640, 0.42389, 0.41228, 0.40350, 0.40088, 0.40016,\
        0.39921, 0.39828, 0.39763, 0.39736, 0.39749, 0.39798, 0.39873, 0.39954, 0.40015, 0.40046, 0.40059],\
        [0.57920, 0.57949, 0.57977, 0.58007, 0.58042, 0.58083, 0.58133, 0.58194, 0.58268, 0.58354, 0.58452,\
        0.58562, 0.58681, 0.58808, 0.58941, 0.59078, 0.59215, 0.59352, 0.59482, 0.59598, 0.59690, 0.59758,\
        0.59806, 0.59842, 0.59887, 0.60079, 0.60523, 0.61056, 0.61612, 0.62171, 0.62725, 0.63270, 0.63806,\
        0.64333, 0.64849, 0.65353, 0.65842, 0.66316, 0.66774, 0.67218, 0.67653, 0.68080, 0.68490, 0.68873,\
        0.69225, 0.69556, 0.69865, 0.70108, 0.70237, 0.70320, 0.70381, 0.70437, 0.70463, 0.70415, 0.70300,\
        0.70144, 0.69973, 0.69778, 0.69557, 0.69302, 0.69022, 0.68720, 0.68393, 0.68027, 0.67601, 0.67110,\
        0.66555, 0.65941, 0.65272, 0.64552, 0.63787, 0.62983, 0.62153, 0.61321, 0.60535, 0.59881, 0.59417,\
        0.59046, 0.58670, 0.58259, 0.57795, 0.57262, 0.56649, 0.55958, 0.55200, 0.54392, 0.53554, 0.52708,\
        0.51879, 0.51104, 0.50434, 0.49980, 0.49840, 0.49761, 0.49664, 0.49536, 0.49355, 0.49094, 0.48722,\
        0.48211, 0.47543, 0.46708, 0.45712, 0.44582, 0.43370, 0.42150, 0.41032, 0.40209, 0.39976, 0.39897,\
        0.39796, 0.39711, 0.39658, 0.39647, 0.39678, 0.39746, 0.39838, 0.39936, 0.40018, 0.40074, 0.40113],\
        [0.58059, 0.58069, 0.58078, 0.58088, 0.58102, 0.58123, 0.58155, 0.58199, 0.58256, 0.58328, 0.58415,\
        0.58515, 0.58627, 0.58750, 0.58883, 0.59027, 0.59180, 0.59342, 0.59508, 0.59668, 0.59807, 0.59916,\
        0.60029, 0.60149, 0.60313, 0.60633, 0.61122, 0.61688, 0.62270, 0.62847, 0.63413, 0.63967, 0.64511,\
        0.65045, 0.65569, 0.66083, 0.66583, 0.67065, 0.67527, 0.67976, 0.68417, 0.68859, 0.69293, 0.69693,\
        0.70057, 0.70402, 0.70725, 0.70986, 0.71162, 0.71282, 0.71371, 0.71437, 0.71463, 0.71426, 0.71326,\
        0.71178, 0.70996, 0.70784, 0.70539, 0.70257, 0.69951, 0.69622, 0.69279, 0.68899, 0.68450, 0.67926,\
        0.67336, 0.66688, 0.65987, 0.65235, 0.64437, 0.63603, 0.62747, 0.61895, 0.61090, 0.60395, 0.59901,\
        0.59505, 0.59115, 0.58711, 0.58266, 0.57747, 0.57136, 0.56435, 0.55661, 0.54833, 0.53974, 0.53107,\
        0.52258, 0.51466, 0.50786, 0.50300, 0.50042, 0.49883, 0.49744, 0.49593, 0.49400, 0.49129, 0.48740,\
        0.48205, 0.47511, 0.46657, 0.45660, 0.44550, 0.43383, 0.42236, 0.41223, 0.40495, 0.40129, 0.39941,\
        0.39795, 0.39686, 0.39623, 0.39611, 0.39647, 0.39723, 0.39824, 0.39933, 0.40031, 0.40109, 0.40174],\
        [0.58205, 0.58197, 0.58186, 0.58177, 0.58172, 0.58175, 0.58190, 0.58220, 0.58266, 0.58329, 0.58410,\
        0.58507, 0.58621, 0.58749, 0.58894, 0.59056, 0.59240, 0.59446, 0.59675, 0.59919, 0.60159, 0.60384,\
        0.60620, 0.60875, 0.61176, 0.61566, 0.62049, 0.62586, 0.63138, 0.63688, 0.64233, 0.64773, 0.65306,\
        0.65833, 0.66356, 0.66873, 0.67380, 0.67865, 0.68327, 0.68773, 0.69213, 0.69662, 0.70117, 0.70537,\
        0.70920, 0.71281, 0.71613, 0.71894, 0.72113, 0.72277, 0.72400, 0.72484, 0.72522, 0.72501, 0.72418,\
        0.72280, 0.72094, 0.71866, 0.71596, 0.71290, 0.70955, 0.70598, 0.70231, 0.69832, 0.69355, 0.68798,\
        0.68180, 0.67510, 0.66790, 0.66023, 0.65214, 0.64375, 0.63520, 0.62677, 0.61886, 0.61198, 0.60646,\
        0.60185, 0.59746, 0.59329, 0.58892, 0.58377, 0.57756, 0.57037, 0.56245, 0.55403, 0.54535, 0.53662,\
        0.52813, 0.52026, 0.51349, 0.50829, 0.50471, 0.50217, 0.50012, 0.49816, 0.49608, 0.49330, 0.48930,\
        0.48378, 0.47669, 0.46810, 0.45823, 0.44745, 0.43631, 0.42557, 0.41615, 0.40895, 0.40421, 0.40117,\
        0.39897, 0.39741, 0.39649, 0.39620, 0.39650, 0.39727, 0.39833, 0.39946, 0.40055, 0.40156, 0.40247],\
        [0.58359, 0.58334, 0.58305, 0.58277, 0.58255, 0.58242, 0.58243, 0.58262, 0.58301, 0.58362, 0.58445,\
        0.58550, 0.58674, 0.58820, 0.58987, 0.59182, 0.59412, 0.59684, 0.60001, 0.60362, 0.60737, 0.61117,\
        0.61507, 0.61906, 0.62320, 0.62757, 0.63218, 0.63695, 0.64180, 0.64671, 0.65170, 0.65675, 0.66181,\
        0.66688, 0.67198, 0.67714, 0.68228, 0.68716, 0.69173, 0.69617, 0.70053, 0.70505, 0.70963, 0.71398,\
        0.71804, 0.72182, 0.72531, 0.72837, 0.73094, 0.73300, 0.73458, 0.73568, 0.73627, 0.73626, 0.73562,\
        0.73434, 0.73245, 0.73001, 0.72708, 0.72376, 0.72013, 0.71629, 0.71226, 0.70788, 0.70282, 0.69703,\
        0.69070, 0.68391, 0.67668, 0.66901, 0.66099, 0.65271, 0.64435, 0.63614, 0.62841, 0.62148, 0.61551,\
        0.61026, 0.60543, 0.60100, 0.59651, 0.59130, 0.58482, 0.57735, 0.56926, 0.56078, 0.55210, 0.54343,\
        0.53503, 0.52723, 0.52041, 0.51485, 0.51056, 0.50726, 0.50455, 0.50212, 0.49979, 0.49691, 0.49279,\
        0.48712, 0.47994, 0.47138, 0.46168, 0.45121, 0.44051, 0.43023, 0.42109, 0.41363, 0.40799, 0.40387,\
        0.40082, 0.39863, 0.39725, 0.39664, 0.39677, 0.39751, 0.39865, 0.39989, 0.40104, 0.40220, 0.40335],\
        [0.58524, 0.58482, 0.58437, 0.58392, 0.58353, 0.58326, 0.58316, 0.58329, 0.58367, 0.58432, 0.58526,\
        0.58648, 0.58796, 0.58970, 0.59174, 0.59415, 0.59707, 0.60065, 0.60494, 0.60977, 0.61497, 0.62039,\
        0.62592, 0.63139, 0.63656, 0.64126, 0.64550, 0.64949, 0.65344, 0.65754, 0.66191, 0.66652, 0.67124,\
        0.67599, 0.68084, 0.68590, 0.69115, 0.69613, 0.70059, 0.70502, 0.70945, 0.71394, 0.71847, 0.72288,\
        0.72708, 0.73104, 0.73474, 0.73809, 0.74101, 0.74345, 0.74538, 0.74678, 0.74763, 0.74786, 0.74742,\
        0.74624, 0.74431, 0.74169, 0.73851, 0.73489, 0.73095, 0.72675, 0.72232, 0.71752, 0.71220, 0.70631,\
        0.69993, 0.69317, 0.68602, 0.67848, 0.67063, 0.66259, 0.65448, 0.64650, 0.63889, 0.63188, 0.62555,\
        0.61983, 0.61459, 0.60970, 0.60485, 0.59955, 0.59255, 0.58486, 0.57671, 0.56828, 0.55970, 0.55113,\
        0.54282, 0.53505, 0.52812, 0.52223, 0.51741, 0.51350, 0.51027, 0.50745, 0.50476, 0.50168, 0.49743,\
        0.49167, 0.48451, 0.47603, 0.46649, 0.45626, 0.44585, 0.43580, 0.42665, 0.41879, 0.41236, 0.40727,\
        0.40331, 0.40037, 0.39837, 0.39731, 0.39718, 0.39785, 0.39904, 0.40037, 0.40164, 0.40297, 0.40438],\
        [0.58700, 0.58643, 0.58581, 0.58520, 0.58467, 0.58428, 0.58411, 0.58421, 0.58464, 0.58542, 0.58656,\
        0.58806, 0.58989, 0.59205, 0.59459, 0.59759, 0.60129, 0.60584, 0.61118, 0.61720, 0.62377, 0.63074,\
        0.63790, 0.64489, 0.65111, 0.65606, 0.65975, 0.66277, 0.66569, 0.66887, 0.67256, 0.67679, 0.68120,\
        0.68556, 0.69003, 0.69484, 0.70014, 0.70521, 0.70974, 0.71419, 0.71865, 0.72314, 0.72763, 0.73205,\
        0.73634, 0.74046, 0.74436, 0.74800, 0.75128, 0.75410, 0.75636, 0.75803, 0.75913, 0.75963, 0.75942,\
        0.75834, 0.75634, 0.75350, 0.75001, 0.74606, 0.74176, 0.73719, 0.73237, 0.72723, 0.72169, 0.71572,\
        0.70938, 0.70270, 0.69571, 0.68834, 0.68075, 0.67301, 0.66517, 0.65740, 0.64987, 0.64275, 0.63610,\
        0.62993, 0.62416, 0.61867, 0.61319, 0.60716, 0.60017, 0.59253, 0.58449, 0.57623, 0.56782, 0.55939,\
        0.55113, 0.54333, 0.53624, 0.53004, 0.52478, 0.52039, 0.51669, 0.51344, 0.51034, 0.50691, 0.50258,\
        0.49696, 0.48993, 0.48161, 0.47223, 0.46217, 0.45190, 0.44190, 0.43259, 0.42429, 0.41714, 0.41116,\
        0.40629, 0.40249, 0.39973, 0.39811, 0.39762, 0.39814, 0.39937, 0.40077, 0.40222, 0.40383, 0.40559],\
        [0.58888, 0.58817, 0.58740, 0.58664, 0.58597, 0.58548, 0.58526, 0.58539, 0.58593, 0.58690, 0.58835,\
        0.59024, 0.59254, 0.59524, 0.59840, 0.60214, 0.60666, 0.61206, 0.61831, 0.62536, 0.63312, 0.64145,\
        0.65014, 0.65869, 0.66615, 0.67132, 0.67418, 0.67609, 0.67800, 0.68023, 0.68319, 0.68726, 0.69154,\
        0.69548, 0.69967, 0.70419, 0.70925, 0.71427, 0.71899, 0.72354, 0.72805, 0.73255, 0.73703, 0.74145,\
        0.74579, 0.75002, 0.75412, 0.75805, 0.76169, 0.76487, 0.76743, 0.76931, 0.77062, 0.77139, 0.77145,\
        0.77050, 0.76836, 0.76521, 0.76135, 0.75703, 0.75237, 0.74745, 0.74229, 0.73688, 0.73117, 0.72515,\
        0.71883, 0.71226, 0.70542, 0.69828, 0.69102, 0.68364, 0.67606, 0.66844, 0.66095, 0.65371, 0.64675,\
        0.64009, 0.63372, 0.62755, 0.62136, 0.61483, 0.60775, 0.60021, 0.59238, 0.58438, 0.57620, 0.56789,\
        0.55964, 0.55173, 0.54444, 0.53794, 0.53230, 0.52748, 0.52333, 0.51964, 0.51611, 0.51236, 0.50795,\
        0.50251, 0.49578, 0.48770, 0.47847, 0.46852, 0.45832, 0.44829, 0.43877, 0.43003, 0.42222, 0.41542,\
        0.40966, 0.40495, 0.40136, 0.39904, 0.39808, 0.39835, 0.39947, 0.40102, 0.40281, 0.40485, 0.40706],\
        [0.59087, 0.59003, 0.58911, 0.58821, 0.58742, 0.58685, 0.58663, 0.58682, 0.58752, 0.58877, 0.59059,\
        0.59298, 0.59588, 0.59920, 0.60304, 0.60754, 0.61280, 0.61892, 0.62592, 0.63379, 0.64244, 0.65179,\
        0.66165, 0.67165, 0.68073, 0.68628, 0.68779, 0.68869, 0.68997, 0.69140, 0.69335, 0.69743, 0.70197,\
        0.70566, 0.70969, 0.71408, 0.71883, 0.72367, 0.72841, 0.73305, 0.73762, 0.74214, 0.74662, 0.75103,\
        0.75539, 0.75970, 0.76396, 0.76814, 0.77215, 0.77575, 0.77856, 0.78051, 0.78188, 0.78291, 0.78331,\
        0.78252, 0.78013, 0.77653, 0.77227, 0.76761, 0.76262, 0.75738, 0.75194, 0.74632, 0.74049, 0.73443,\
        0.72814, 0.72165, 0.71497, 0.70811, 0.70121, 0.69417, 0.68678, 0.67925, 0.67182, 0.66448, 0.65719,\
        0.65000, 0.64300, 0.63616, 0.62937, 0.62245, 0.61526, 0.60782, 0.60022, 0.59252, 0.58459, 0.57636,\
        0.56802, 0.55994, 0.55243, 0.54564, 0.53966, 0.53444, 0.52988, 0.52577, 0.52186, 0.51781, 0.51330,\
        0.50800, 0.50164, 0.49385, 0.48478, 0.47495, 0.46483, 0.45476, 0.44505, 0.43593, 0.42755, 0.42003,\
        0.41343, 0.40784, 0.40337, 0.40023, 0.39865, 0.39861, 0.39969, 0.40140, 0.40363, 0.40619, 0.40894],\
        [0.59295, 0.59199, 0.59093, 0.58989, 0.58899, 0.58838, 0.58818, 0.58849, 0.58940, 0.59098, 0.59326,\
        0.59621, 0.59980, 0.60382, 0.60836, 0.61354, 0.61945, 0.62617, 0.63374, 0.64216, 0.65138, 0.66126,\
        0.67162, 0.68223, 0.69275, 0.69945, 0.69939, 0.70013, 0.70140, 0.70251, 0.70384, 0.70733, 0.71181,\
        0.71583, 0.71987, 0.72416, 0.72869, 0.73337, 0.73807, 0.74274, 0.74736, 0.75191, 0.75639, 0.76079,\
        0.76514, 0.76947, 0.77383, 0.77822, 0.78257, 0.78665, 0.78974, 0.79153, 0.79276, 0.79398, 0.79470,\
        0.79415, 0.79126, 0.78711, 0.78255, 0.77764, 0.77236, 0.76684, 0.76120, 0.75544, 0.74953, 0.74344,\
        0.73717, 0.73076, 0.72422, 0.71759, 0.71092, 0.70411, 0.69685, 0.68948, 0.68222, 0.67483, 0.66716,\
        0.65940, 0.65178, 0.64436, 0.63707, 0.62983, 0.62254, 0.61520, 0.60784, 0.60041, 0.59277, 0.58448,\
        0.57595, 0.56767, 0.55994, 0.55290, 0.54662, 0.54107, 0.53613, 0.53165, 0.52739, 0.52308, 0.51842,\
        0.51315, 0.50698, 0.49955, 0.49077, 0.48117, 0.47120, 0.46118, 0.45135, 0.44193, 0.43310, 0.42498,\
        0.41767, 0.41128, 0.40596, 0.40190, 0.39943, 0.39896, 0.40025, 0.40219, 0.40493, 0.40811, 0.41144],\
        [0.59509, 0.59402, 0.59281, 0.59163, 0.59065, 0.59002, 0.58989, 0.59037, 0.59156, 0.59355, 0.59637,\
        0.59997, 0.60422, 0.60899, 0.61425, 0.62006, 0.62651, 0.63368, 0.64164, 0.65039, 0.65988, 0.66990,\
        0.68012, 0.68997, 0.69921, 0.70673, 0.70930, 0.71074, 0.71218, 0.71351, 0.71523, 0.71817, 0.72205,\
        0.72606, 0.73013, 0.73433, 0.73871, 0.74324, 0.74788, 0.75257, 0.75725, 0.76186, 0.76635, 0.77074,\
        0.77506, 0.77939, 0.78379, 0.78829, 0.79282, 0.79733, 0.80080, 0.80244, 0.80348, 0.80463, 0.80520,\
        0.80442, 0.80112, 0.79670, 0.79208, 0.78706, 0.78148, 0.77570, 0.76994, 0.76414, 0.75819, 0.75208,\
        0.74582, 0.73946, 0.73302, 0.72650, 0.71991, 0.71313, 0.70608, 0.69893, 0.69197, 0.68458, 0.67641,\
        0.66803, 0.65986, 0.65197, 0.64430, 0.63679, 0.62940, 0.62210, 0.61487, 0.60768, 0.60020, 0.59177,\
        0.58308, 0.57467, 0.56679, 0.55956, 0.55304, 0.54721, 0.54196, 0.53715, 0.53259, 0.52802, 0.52320,\
        0.51786, 0.51173, 0.50456, 0.49622, 0.48702, 0.47733, 0.46745, 0.45761, 0.44802, 0.43886, 0.43030,\
        0.42245, 0.41543, 0.40937, 0.40444, 0.40089, 0.39931, 0.40061, 0.40349, 0.40707, 0.41094, 0.41482],\
        [0.59722, 0.59604, 0.59468, 0.59337, 0.59233, 0.59173, 0.59174, 0.59247, 0.59403, 0.59651, 0.59997,\
        0.60431, 0.60930, 0.61479, 0.62072, 0.62709, 0.63398, 0.64147, 0.64965, 0.65857, 0.66817, 0.67820,\
        0.68826, 0.69770, 0.70600, 0.71337, 0.71798, 0.72069, 0.72265, 0.72440, 0.72644, 0.72921, 0.73268,\
        0.73649, 0.74046, 0.74455, 0.74878, 0.75318, 0.75777, 0.76250, 0.76727, 0.77197, 0.77651, 0.78091,\
        0.78524, 0.78957, 0.79400, 0.79861, 0.80325, 0.80767, 0.81118, 0.81316, 0.81428, 0.81493, 0.81480,\
        0.81329, 0.81003, 0.80567, 0.80097, 0.79585, 0.78979, 0.78381, 0.77812, 0.77240, 0.76644, 0.76029,\
        0.75402, 0.74768, 0.74128, 0.73481, 0.72823, 0.72149, 0.71459, 0.70767, 0.70086, 0.69345, 0.68444,\
        0.67561, 0.66711, 0.65889, 0.65094, 0.64322, 0.63569, 0.62833, 0.62106, 0.61377, 0.60612, 0.59780,\
        0.58920, 0.58082, 0.57289, 0.56555, 0.55886, 0.55280, 0.54729, 0.54222, 0.53738, 0.53259, 0.52759,\
        0.52217, 0.51607, 0.50910, 0.50120, 0.49249, 0.48319, 0.47355, 0.46379, 0.45415, 0.44483, 0.43600,\
        0.42782, 0.42042, 0.41394, 0.40851, 0.40436, 0.40195, 0.40294, 0.40651, 0.41072, 0.41508, 0.41933],\
        [0.59924, 0.59792, 0.59641, 0.59502, 0.59397, 0.59347, 0.59369, 0.59478, 0.59684, 0.59997, 0.60419,\
        0.60932, 0.61508, 0.62128, 0.62784, 0.63471, 0.64195, 0.64964, 0.65792, 0.66690, 0.67657, 0.68669,\
        0.69683, 0.70639, 0.71479, 0.72181, 0.72698, 0.73046, 0.73293, 0.73504, 0.73731, 0.74005, 0.74334,\
        0.74700, 0.75085, 0.75480, 0.75885, 0.76308, 0.76759, 0.77240, 0.77736, 0.78221, 0.78684, 0.79130,\
        0.79573, 0.80019, 0.80476, 0.80950, 0.81413, 0.81832, 0.82160, 0.82366, 0.82466, 0.82479, 0.82393,\
        0.82183, 0.81841, 0.81402, 0.80906, 0.80344, 0.79704, 0.79116, 0.78586, 0.78028, 0.77424, 0.76800,\
        0.76169, 0.75537, 0.74900, 0.74254, 0.73597, 0.72922, 0.72231, 0.71528, 0.70806, 0.69969, 0.69075,\
        0.68212, 0.67355, 0.66515, 0.65698, 0.64907, 0.64140, 0.63390, 0.62647, 0.61896, 0.61113, 0.60288,\
        0.59447, 0.58620, 0.57830, 0.57090, 0.56408, 0.55784, 0.55213, 0.54683, 0.54178, 0.53678, 0.53164,\
        0.52615, 0.52011, 0.51337, 0.50587, 0.49764, 0.48876, 0.47943, 0.46986, 0.46027, 0.45092, 0.44202,\
        0.43375, 0.42629, 0.41976, 0.41437, 0.41040, 0.40843, 0.40918, 0.41222, 0.41633, 0.42077, 0.42513],\
        [0.60081, 0.59944, 0.59791, 0.59651, 0.59549, 0.59514, 0.59569, 0.59726, 0.59998, 0.60397, 0.60906,\
        0.61503, 0.62158, 0.62851, 0.63567, 0.64302, 0.65055, 0.65836, 0.66663, 0.67560, 0.68540, 0.69580,\
        0.70625, 0.71605, 0.72459, 0.73153, 0.73674, 0.74043, 0.74311, 0.74539, 0.74775, 0.75053, 0.75379,\
        0.75742, 0.76122, 0.76503, 0.76885, 0.77284, 0.77723, 0.78214, 0.78739, 0.79248, 0.79727, 0.80189,\
        0.80654, 0.81126, 0.81605, 0.82081, 0.82532, 0.82924, 0.83222, 0.83405, 0.83472, 0.83430, 0.83278,\
        0.83012, 0.82636, 0.82171, 0.81635, 0.81032, 0.80388, 0.79828, 0.79349, 0.78781, 0.78148, 0.77511,\
        0.76882, 0.76254, 0.75621, 0.74978, 0.74319, 0.73638, 0.72932, 0.72191, 0.71391, 0.70513, 0.69664,\
        0.68813, 0.67945, 0.67087, 0.66251, 0.65443, 0.64661, 0.63897, 0.63138, 0.62370, 0.61577, 0.60757,\
        0.59924, 0.59106, 0.58316, 0.57569, 0.56876, 0.56238, 0.55650, 0.55102, 0.54580, 0.54065, 0.53539,\
        0.52986, 0.52388, 0.51736, 0.51022, 0.50243, 0.49401, 0.48506, 0.47572, 0.46628, 0.45702, 0.44823,\
        0.44011, 0.43286, 0.42663, 0.42166, 0.41823, 0.41674, 0.41742, 0.42000, 0.42374, 0.42796, 0.43218],\
        [0.60197, 0.60063, 0.59919, 0.59785, 0.59685, 0.59662, 0.59759, 0.59991, 0.60342, 0.60838, 0.61447,\
        0.62138, 0.62880, 0.63648, 0.64427, 0.65210, 0.65993, 0.66783, 0.67604, 0.68493, 0.69496, 0.70590,\
        0.71679, 0.72684, 0.73544, 0.74222, 0.74719, 0.75066, 0.75318, 0.75537, 0.75771, 0.76052, 0.76388,\
        0.76763, 0.77150, 0.77523, 0.77878, 0.78241, 0.78655, 0.79153, 0.79715, 0.80254, 0.80755, 0.81249,\
        0.81745, 0.82246, 0.82744, 0.83225, 0.83664, 0.84030, 0.84294, 0.84437, 0.84455, 0.84352, 0.84136,\
        0.83813, 0.83394, 0.82896, 0.82334, 0.81727, 0.81116, 0.80574, 0.80086, 0.79455, 0.78791, 0.78157,\
        0.77541, 0.76925, 0.76300, 0.75660, 0.75000, 0.74314, 0.73594, 0.72827, 0.72007, 0.71147, 0.70285,\
        0.69399, 0.68502, 0.67621, 0.66767, 0.65944, 0.65149, 0.64374, 0.63606, 0.62829, 0.62032, 0.61213,\
        0.60383, 0.59562, 0.58764, 0.58008, 0.57302, 0.56651, 0.56049, 0.55487, 0.54951, 0.54423, 0.53888,\
        0.53331, 0.52738, 0.52103, 0.51418, 0.50682, 0.49888, 0.49035, 0.48126, 0.47200, 0.46295, 0.45443,\
        0.44666, 0.43984, 0.43415, 0.42978, 0.42697, 0.42593, 0.42670, 0.42904, 0.43244, 0.43634, 0.44030],\
        [0.60307, 0.60174, 0.60039, 0.59907, 0.59800, 0.59779, 0.59911, 0.60223, 0.60696, 0.61312, 0.62038,\
        0.62836, 0.63671, 0.64519, 0.65363, 0.66198, 0.67018, 0.67825, 0.68644, 0.69537, 0.70566, 0.71712,\
        0.72841, 0.73868, 0.74725, 0.75372, 0.75814, 0.76102, 0.76305, 0.76489, 0.76707, 0.76989, 0.77344,\
        0.77750, 0.78160, 0.78537, 0.78868, 0.79187, 0.79568, 0.80055, 0.80635, 0.81204, 0.81747, 0.82282,\
        0.82816, 0.83348, 0.83868, 0.84356, 0.84788, 0.85133, 0.85363, 0.85458, 0.85416, 0.85245, 0.84961,\
        0.84580, 0.84116, 0.83587, 0.83009, 0.82403, 0.81802, 0.81230, 0.80650, 0.79993, 0.79348, 0.78751,\
        0.78162, 0.77561, 0.76944, 0.76310, 0.75655, 0.74970, 0.74245, 0.73470, 0.72644, 0.71778, 0.70883,\
        0.69962, 0.69041, 0.68136, 0.67262, 0.66425, 0.65623, 0.64845, 0.64074, 0.63295, 0.62497, 0.61676,\
        0.60841, 0.60007, 0.59194, 0.58420, 0.57699, 0.57034, 0.56420, 0.55847, 0.55300, 0.54761, 0.54216,\
        0.53652, 0.53059, 0.52433, 0.51770, 0.51069, 0.50327, 0.49510, 0.48623, 0.47720, 0.46848, 0.46037,\
        0.45312, 0.44690, 0.44187, 0.43820, 0.43603, 0.43545, 0.43643, 0.43872, 0.44191, 0.44554, 0.44919],\
        [0.60426, 0.60287, 0.60149, 0.60019, 0.59900, 0.59863, 0.60035, 0.60450, 0.61064, 0.61823, 0.62680,\
        0.63594, 0.64529, 0.65456, 0.66362, 0.67252, 0.68125, 0.68965, 0.69796, 0.70728, 0.71789, 0.72939,\
        0.74084, 0.75128, 0.75978, 0.76575, 0.76930, 0.77124, 0.77251, 0.77384, 0.77574, 0.77850, 0.78230,\
        0.78686, 0.79137, 0.79528, 0.79854, 0.80140, 0.80513, 0.80989, 0.81548, 0.82129, 0.82706, 0.83279,\
        0.83851, 0.84415, 0.84958, 0.85458, 0.85889, 0.86218, 0.86415, 0.86457, 0.86346, 0.86100, 0.85745,\
        0.85304, 0.84797, 0.84240, 0.83647, 0.83036, 0.82423, 0.81812, 0.81181, 0.80520, 0.79895, 0.79335,\
        0.78764, 0.78172, 0.77561, 0.76935, 0.76291, 0.75619, 0.74901, 0.74130, 0.73300, 0.72419, 0.71495,\
        0.70543, 0.69589, 0.68652, 0.67755, 0.66906, 0.66102, 0.65328, 0.64564, 0.63790, 0.62992, 0.62167,\
        0.61318, 0.60462, 0.59623, 0.58824, 0.58081, 0.57400, 0.56775, 0.56193, 0.55637, 0.55087, 0.54530,\
        0.53954, 0.53352, 0.52721, 0.52063, 0.51378, 0.50662, 0.49886, 0.49029, 0.48161, 0.47335, 0.46583,\
        0.45925, 0.45374, 0.44946, 0.44651, 0.44499, 0.44491, 0.44619, 0.44858, 0.45172, 0.45516, 0.45852],\
        [0.60569, 0.60419, 0.60271, 0.60132, 0.60005, 0.59919, 0.60150, 0.60718, 0.61489, 0.62393, 0.63382,\
        0.64414, 0.65448, 0.66450, 0.67406, 0.68337, 0.69269, 0.70164, 0.71034, 0.71988, 0.73051, 0.74190,\
        0.75337, 0.76400, 0.77257, 0.77792, 0.78023, 0.78094, 0.78136, 0.78219, 0.78377, 0.78637, 0.79029,\
        0.79544, 0.80047, 0.80442, 0.80783, 0.81099, 0.81473, 0.81934, 0.82467, 0.83041, 0.83634, 0.84236,\
        0.84839, 0.85432, 0.85998, 0.86511, 0.86945, 0.87266, 0.87434, 0.87420, 0.87229, 0.86899, 0.86470,\
        0.85972, 0.85426, 0.84845, 0.84241, 0.83624, 0.83000, 0.82370, 0.81731, 0.81091, 0.80490, 0.79926,\
        0.79351, 0.78759, 0.78150, 0.77537, 0.76918, 0.76272, 0.75576, 0.74815, 0.73986, 0.73093, 0.72146,\
        0.71162, 0.70165, 0.69187, 0.68263, 0.67404, 0.66605, 0.65844, 0.65095, 0.64332, 0.63538, 0.62706,\
        0.61839, 0.60952, 0.60074, 0.59236, 0.58464, 0.57764, 0.57129, 0.56540, 0.55975, 0.55413, 0.54839,\
        0.54243, 0.53620, 0.52970, 0.52294, 0.51598, 0.50882, 0.50131, 0.49317, 0.48500, 0.47741, 0.47065,\
        0.46485, 0.46014, 0.45663, 0.45441, 0.45353, 0.45397, 0.45562, 0.45823, 0.46145, 0.46484, 0.46797],\
        [0.60755, 0.60599, 0.60446, 0.60302, 0.60174, 0.60094, 0.60418, 0.61126, 0.62022, 0.63042, 0.64143,\
        0.65284, 0.66416, 0.67489, 0.68475, 0.69416, 0.70364, 0.71309, 0.72232, 0.73205, 0.74255, 0.75366,\
        0.76497, 0.77577, 0.78476, 0.78953, 0.79019, 0.78966, 0.78948, 0.79008, 0.79151, 0.79393, 0.79764,\
        0.80305, 0.80838, 0.81276, 0.81652, 0.82010, 0.82400, 0.82852, 0.83367, 0.83931, 0.84528, 0.85147,\
        0.85773, 0.86391, 0.86976, 0.87498, 0.87932, 0.88251, 0.88400, 0.88323, 0.88037, 0.87616, 0.87116,\
        0.86569, 0.85993, 0.85398, 0.84789, 0.84172, 0.83548, 0.82918, 0.82285, 0.81661, 0.81061, 0.80483,\
        0.79906, 0.79317, 0.78700, 0.78111, 0.77540, 0.76942, 0.76279, 0.75537, 0.74714, 0.73816, 0.72851,\
        0.71835, 0.70793, 0.69768, 0.68810, 0.67940, 0.67154, 0.66415, 0.65687, 0.64940, 0.64152, 0.63312,\
        0.62421, 0.61496, 0.60567, 0.59677, 0.58864, 0.58143, 0.57499, 0.56903, 0.56328, 0.55751, 0.55155,\
        0.54530, 0.53875, 0.53189, 0.52478, 0.51747, 0.51001, 0.50245, 0.49476, 0.48738, 0.48068, 0.47479,\
        0.46984, 0.46594, 0.46320, 0.46167, 0.46140, 0.46235, 0.46440, 0.46732, 0.47074, 0.47420, 0.47720],\
        [0.61002, 0.60854, 0.60716, 0.60603, 0.60542, 0.60622, 0.61015, 0.61739, 0.62682, 0.63769, 0.64948,\
        0.66176, 0.67403, 0.68556, 0.69556, 0.70491, 0.71440, 0.72390, 0.73342, 0.74320, 0.75341, 0.76397,\
        0.77464, 0.78501, 0.79439, 0.79897, 0.79818, 0.79714, 0.79708, 0.79795, 0.79958, 0.80216, 0.80592,\
        0.81099, 0.81622, 0.82085, 0.82493, 0.82879, 0.83282, 0.83730, 0.84232, 0.84785, 0.85380, 0.86006,\
        0.86647, 0.87285, 0.87884, 0.88403, 0.88819, 0.89130, 0.89279, 0.89127, 0.88726, 0.88216, 0.87661,\
        0.87084, 0.86494, 0.85897, 0.85294, 0.84687, 0.84075, 0.83455, 0.82832, 0.82212, 0.81601, 0.81003,\
        0.80418, 0.79812, 0.79180, 0.78655, 0.78174, 0.77643, 0.77020, 0.76301, 0.75492, 0.74598, 0.73625,\
        0.72586, 0.71506, 0.70429, 0.69418, 0.68538, 0.67772, 0.67059, 0.66355, 0.65627, 0.64847, 0.63998,\
        0.63081, 0.62112, 0.61124, 0.60167, 0.59299, 0.58554, 0.57902, 0.57300, 0.56713, 0.56114, 0.55488,\
        0.54825, 0.54127, 0.53396, 0.52639, 0.51861, 0.51070, 0.50282, 0.49558, 0.48915, 0.48338, 0.47837,\
        0.47425, 0.47112, 0.46906, 0.46816, 0.46842, 0.46981, 0.47223, 0.47546, 0.47918, 0.48285, 0.48582],\
        [0.61321, 0.61196, 0.61098, 0.61049, 0.61090, 0.61298, 0.61758, 0.62487, 0.63436, 0.64544, 0.65760,\
        0.67040, 0.68336, 0.69571, 0.70608, 0.71529, 0.72461, 0.73405, 0.74357, 0.75321, 0.76295, 0.77269,\
        0.78221, 0.79106, 0.79841, 0.80324, 0.80433, 0.80429, 0.80490, 0.80630, 0.80841, 0.81133, 0.81518,\
        0.81983, 0.82467, 0.82920, 0.83333, 0.83725, 0.84128, 0.84566, 0.85055, 0.85597, 0.86187, 0.86812,\
        0.87458, 0.88107, 0.88723, 0.89224, 0.89583, 0.89844, 0.89990, 0.89736, 0.89233, 0.88676, 0.88098,\
        0.87514, 0.86929, 0.86346, 0.85763, 0.85179, 0.84588, 0.83988, 0.83376, 0.82757, 0.82130, 0.81498,\
        0.80853, 0.80185, 0.79589, 0.79206, 0.78852, 0.78391, 0.77801, 0.77104, 0.76315, 0.75439, 0.74474,\
        0.73427, 0.72321, 0.71198, 0.70125, 0.69241, 0.68489, 0.67790, 0.67105, 0.66396, 0.65626, 0.64769,\
        0.63823, 0.62808, 0.61760, 0.60730, 0.59793, 0.59029, 0.58361, 0.57745, 0.57139, 0.56513, 0.55846,\
        0.55137, 0.54388, 0.53607, 0.52803, 0.51985, 0.51165, 0.50373, 0.49676, 0.49090, 0.48584, 0.48157,\
        0.47814, 0.47566, 0.47419, 0.47378, 0.47445, 0.47617, 0.47884, 0.48230, 0.48629, 0.49030, 0.49338],\
        [0.61710, 0.61622, 0.61577, 0.61601, 0.61737, 0.62036, 0.62548, 0.63287, 0.64227, 0.65327, 0.66541,\
        0.67826, 0.69130, 0.70375, 0.71458, 0.72431, 0.73378, 0.74326, 0.75275, 0.76219, 0.77143, 0.78028,\
        0.78857, 0.79602, 0.80232, 0.80720, 0.81000, 0.81158, 0.81311, 0.81512, 0.81772, 0.82097, 0.82486,\
        0.82922, 0.83368, 0.83792, 0.84185, 0.84561, 0.84945, 0.85365, 0.85838, 0.86370, 0.86952, 0.87574,\
        0.88213, 0.88853, 0.89486, 0.89976, 0.90255, 0.90411, 0.90392, 0.90061, 0.89561, 0.89012, 0.88444,\
        0.87874, 0.87310, 0.86755, 0.86206, 0.85657, 0.85099, 0.84525, 0.83931, 0.83318, 0.82683, 0.82028,\
        0.81350, 0.80665, 0.80125, 0.79862, 0.79612, 0.79181, 0.78603, 0.77925, 0.77170, 0.76331, 0.75393,\
        0.74356, 0.73243, 0.72107, 0.71029, 0.70104, 0.69314, 0.68601, 0.67926, 0.67238, 0.66481, 0.65616,\
        0.64638, 0.63575, 0.62476, 0.61399, 0.60420, 0.59593, 0.58884, 0.58244, 0.57615, 0.56950, 0.56232,\
        0.55465, 0.54660, 0.53828, 0.52984, 0.52138, 0.51310, 0.50531, 0.49850, 0.49291, 0.48828, 0.48450,\
        0.48159, 0.47960, 0.47856, 0.47852, 0.47945, 0.48133, 0.48407, 0.48753, 0.49150, 0.49571, 0.49914],\
        [0.62160, 0.62114, 0.62125, 0.62219, 0.62429, 0.62794, 0.63344, 0.64087, 0.65010, 0.66084, 0.67271,\
        0.68525, 0.69796, 0.71023, 0.72150, 0.73185, 0.74173, 0.75142, 0.76098, 0.77033, 0.77923, 0.78739,\
        0.79474, 0.80155, 0.80755, 0.81249, 0.81615, 0.81890, 0.82139, 0.82407, 0.82717, 0.83074, 0.83472,\
        0.83892, 0.84308, 0.84697, 0.85053, 0.85391, 0.85740, 0.86131, 0.86587, 0.87111, 0.87692, 0.88316,\
        0.88954, 0.89550, 0.90147, 0.90633, 0.90876, 0.90923, 0.90758, 0.90364, 0.89844, 0.89294, 0.88740,\
        0.88191, 0.87655, 0.87138, 0.86635, 0.86133, 0.85618, 0.85080, 0.84513, 0.83915, 0.83291, 0.82645,\
        0.81995, 0.81386, 0.80921, 0.80633, 0.80368, 0.79951, 0.79385, 0.78734, 0.78033, 0.77258, 0.76367,\
        0.75350, 0.74240, 0.73101, 0.72016, 0.71046, 0.70199, 0.69453, 0.68787, 0.68126, 0.67390, 0.66514,\
        0.65493, 0.64377, 0.63232, 0.62121, 0.61104, 0.60213, 0.59454, 0.58794, 0.58136, 0.57419, 0.56635,\
        0.55800, 0.54935, 0.54056, 0.53177, 0.52314, 0.51488, 0.50724, 0.50059, 0.49512, 0.49070, 0.48722,\
        0.48465, 0.48298, 0.48223, 0.48239, 0.48345, 0.48533, 0.48794, 0.49111, 0.49462, 0.49812, 0.50121],\
        [0.62659, 0.62655, 0.62717, 0.62869, 0.63137, 0.63548, 0.64120, 0.64861, 0.65761, 0.66800, 0.67947,\
        0.69160, 0.70395, 0.71607, 0.72756, 0.73836, 0.74864, 0.75860, 0.76831, 0.77776, 0.78667, 0.79454,\
        0.80134, 0.80779, 0.81367, 0.81870, 0.82286, 0.82637, 0.82962, 0.83292, 0.83650, 0.84041, 0.84456,\
        0.84874, 0.85270, 0.85624, 0.85934, 0.86218, 0.86513, 0.86864, 0.87304, 0.87835, 0.88426, 0.89062,\
        0.89737, 0.90346, 0.90885, 0.91311, 0.91512, 0.91474, 0.91210, 0.90750, 0.90176, 0.89588, 0.89035,\
        0.88494, 0.87982, 0.87510, 0.87063, 0.86621, 0.86159, 0.85664, 0.85130, 0.84558, 0.83959, 0.83345,\
        0.82744, 0.82198, 0.81755, 0.81411, 0.81077, 0.80654, 0.80112, 0.79503, 0.78885, 0.78200, 0.77368,\
        0.76365, 0.75243, 0.74087, 0.72979, 0.71970, 0.71080, 0.70307, 0.69643, 0.69013, 0.68312, 0.67415,\
        0.66329, 0.65148, 0.63957, 0.62817, 0.61773, 0.60849, 0.60059, 0.59378, 0.58686, 0.57896, 0.57027,\
        0.56118, 0.55195, 0.54274, 0.53370, 0.52498, 0.51676, 0.50927, 0.50276, 0.49733, 0.49301, 0.48971,\
        0.48733, 0.48584, 0.48523, 0.48548, 0.48654, 0.48831, 0.49067, 0.49343, 0.49634, 0.49914, 0.50174],\
        [0.63191, 0.63226, 0.63332, 0.63529, 0.63839, 0.64281, 0.64866, 0.65597, 0.66469, 0.67469, 0.68575,\
        0.69751, 0.70955, 0.72152, 0.73311, 0.74420, 0.75479, 0.76497, 0.77484, 0.78442, 0.79367, 0.80169,\
        0.80840, 0.81461, 0.82034, 0.82545, 0.82992, 0.83392, 0.83770, 0.84151, 0.84552, 0.84978, 0.85418,\
        0.85848, 0.86239, 0.86565, 0.86823, 0.87040, 0.87263, 0.87557, 0.87985, 0.88558, 0.89193, 0.89849,\
        0.90572, 0.91234, 0.91764, 0.92124, 0.92255, 0.92135, 0.91783, 0.91244, 0.90599, 0.89956, 0.89371,\
        0.88799, 0.88301, 0.87882, 0.87507, 0.87134, 0.86733, 0.86285, 0.85786, 0.85243, 0.84672, 0.84094,\
        0.83533, 0.83019, 0.82571, 0.82175, 0.81782, 0.81335, 0.80814, 0.80254, 0.79720, 0.79131, 0.78351,\
        0.77333, 0.76172, 0.74983, 0.73846, 0.72808, 0.71890, 0.71101, 0.70426, 0.69821, 0.69190, 0.68233,\
        0.67049, 0.65803, 0.64575, 0.63417, 0.62364, 0.61439, 0.60647, 0.59952, 0.59222, 0.58319, 0.57357,\
        0.56383, 0.55414, 0.54464, 0.53545, 0.52669, 0.51853, 0.51115, 0.50471, 0.49933, 0.49513, 0.49191,\
        0.48961, 0.48819, 0.48763, 0.48788, 0.48886, 0.49046, 0.49254, 0.49493, 0.49742, 0.49986, 0.50218],\
        [0.63743, 0.63810, 0.63950, 0.64183, 0.64523, 0.64986, 0.65576, 0.66295, 0.67134, 0.68089, 0.69155,\
        0.70301, 0.71481, 0.72669, 0.73836, 0.74964, 0.76045, 0.77079, 0.78071, 0.79027, 0.79944, 0.80794,\
        0.81510, 0.82148, 0.82726, 0.83245, 0.83712, 0.84140, 0.84551, 0.84966, 0.85401, 0.85860, 0.86333,\
        0.86792, 0.87196, 0.87505, 0.87713, 0.87858, 0.87997, 0.88208, 0.88604, 0.89261, 0.90029, 0.90767,\
        0.91542, 0.92241, 0.92775, 0.93089, 0.93149, 0.92943, 0.92494, 0.91859, 0.91125, 0.90405, 0.89714,\
        0.89077, 0.88613, 0.88272, 0.87983, 0.87688, 0.87349, 0.86945, 0.86475, 0.85955, 0.85408, 0.84857,\
        0.84323, 0.83821, 0.83358, 0.82921, 0.82481, 0.82009, 0.81503, 0.80991, 0.80500, 0.79980, 0.79235,\
        0.78150, 0.76933, 0.75716, 0.74560, 0.73500, 0.72557, 0.71738, 0.71040, 0.70456, 0.69882, 0.68776,\
        0.67530, 0.66261, 0.65026, 0.63866, 0.62811, 0.61882, 0.61083, 0.60384, 0.59566, 0.58580, 0.57569,\
        0.56561, 0.55572, 0.54610, 0.53687, 0.52814, 0.52006, 0.51278, 0.50644, 0.50118, 0.49702, 0.49380,\
        0.49148, 0.49005, 0.48948, 0.48969, 0.49056, 0.49198, 0.49381, 0.49591, 0.49812, 0.50033, 0.50241],\
        [0.64304, 0.64395, 0.64561, 0.64817, 0.65179, 0.65655, 0.66250, 0.66958, 0.67767, 0.68669, 0.69682,\
        0.70807, 0.71980, 0.73167, 0.74342, 0.75486, 0.76587, 0.77635, 0.78631, 0.79584, 0.80512, 0.81383,\
        0.82152, 0.82825, 0.83422, 0.83952, 0.84429, 0.84869, 0.85293, 0.85724, 0.86177, 0.86661, 0.87169,\
        0.87671, 0.88112, 0.88423, 0.88588, 0.88670, 0.88740, 0.88848, 0.89150, 0.89869, 0.90826, 0.91763,\
        0.92641, 0.93390, 0.93931, 0.94211, 0.94204, 0.93908, 0.93352, 0.92593, 0.91711, 0.90804, 0.89960,\
        0.89314, 0.88947, 0.88715, 0.88514, 0.88293, 0.88011, 0.87639, 0.87183, 0.86674, 0.86144, 0.85613,\
        0.85095, 0.84597, 0.84117, 0.83644, 0.83159, 0.82651, 0.82123, 0.81594, 0.81079, 0.80543, 0.79793,\
        0.78656, 0.77441, 0.76237, 0.75088, 0.74020, 0.73045, 0.72171, 0.71392, 0.70670, 0.69875, 0.68881,\
        0.67714, 0.66493, 0.65287, 0.64138, 0.63078, 0.62123, 0.61265, 0.60445, 0.59577, 0.58633, 0.57640,\
        0.56642, 0.55659, 0.54705, 0.53789, 0.52926, 0.52127, 0.51408, 0.50785, 0.50270, 0.49861, 0.49526,\
        0.49288, 0.49145, 0.49086, 0.49101, 0.49177, 0.49303, 0.49466, 0.49652, 0.49851, 0.50053, 0.50245],\
        [0.64863, 0.64970, 0.65152, 0.65423, 0.65796, 0.66282, 0.66882, 0.67589, 0.68385, 0.69250, 0.70205,\
        0.71307, 0.72471, 0.73655, 0.74836, 0.75997, 0.77122, 0.78194, 0.79203, 0.80171, 0.81116, 0.82004,\
        0.82802, 0.83504, 0.84119, 0.84657, 0.85134, 0.85570, 0.85988, 0.86412, 0.86862, 0.87351, 0.87880,\
        0.88429, 0.88935, 0.89275, 0.89399, 0.89451, 0.89531, 0.89601, 0.89762, 0.90571, 0.91725, 0.92858,\
        0.93864, 0.94675, 0.95229, 0.95480, 0.95411, 0.95027, 0.94361, 0.93469, 0.92424, 0.91311, 0.90259,\
        0.89623, 0.89415, 0.89265, 0.89119, 0.88950, 0.88710, 0.88349, 0.87884, 0.87373, 0.86856, 0.86344,\
        0.85840, 0.85344, 0.84848, 0.84341, 0.83808, 0.83243, 0.82647, 0.82030, 0.81391, 0.80687, 0.79829,\
        0.78809, 0.77690, 0.76554, 0.75443, 0.74378, 0.73370, 0.72422, 0.71526, 0.70648, 0.69743, 0.68771,\
        0.67691, 0.66544, 0.65383, 0.64251, 0.63179, 0.62184, 0.61265, 0.60383, 0.59487, 0.58559, 0.57604,\
        0.56641, 0.55686, 0.54753, 0.53854, 0.53004, 0.52215, 0.51502, 0.50879, 0.50358, 0.49936, 0.49608,\
        0.49379, 0.49241, 0.49184, 0.49195, 0.49263, 0.49375, 0.49521, 0.49688, 0.49867, 0.50052, 0.50232],\
        [0.65414, 0.65528, 0.65715, 0.65990, 0.66367, 0.66855, 0.67460, 0.68176, 0.68986, 0.69852, 0.70786,\
        0.71833, 0.72962, 0.74134, 0.75319, 0.76497, 0.77651, 0.78762, 0.79809, 0.80798, 0.81755, 0.82654,\
        0.83472, 0.84195, 0.84822, 0.85361, 0.85827, 0.86243, 0.86635, 0.87030, 0.87451, 0.87913, 0.88424,\
        0.88983, 0.89550, 0.89950, 0.90049, 0.90142, 0.90327, 0.90526, 0.90866, 0.91704, 0.92896, 0.94120,\
        0.95211, 0.96073, 0.96638, 0.96863, 0.96735, 0.96270, 0.95507, 0.94508, 0.93349, 0.92129, 0.91009,\
        0.90365, 0.90133, 0.89950, 0.89788, 0.89628, 0.89416, 0.89034, 0.88530, 0.88021, 0.87528, 0.87042,\
        0.86556, 0.86065, 0.85558, 0.85019, 0.84434, 0.83794, 0.83099, 0.82351, 0.81550, 0.80689, 0.79767,\
        0.78796, 0.77779, 0.76733, 0.75677, 0.74626, 0.73588, 0.72566, 0.71562, 0.70574, 0.69602, 0.68622,\
        0.67590, 0.66500, 0.65378, 0.64255, 0.63162, 0.62121, 0.61148, 0.60235, 0.59333, 0.58424, 0.57510,\
        0.56590, 0.55672, 0.54768, 0.53891, 0.53056, 0.52276, 0.51564, 0.50935, 0.50396, 0.49962, 0.49649,\
        0.49435, 0.49307, 0.49254, 0.49263, 0.49324, 0.49425, 0.49557, 0.49709, 0.49872, 0.50043, 0.50214],\
        [0.65950, 0.66062, 0.66245, 0.66513, 0.66882, 0.67362, 0.67963, 0.68683, 0.69513, 0.70392, 0.71311,\
        0.72329, 0.73433, 0.74593, 0.75781, 0.76975, 0.78158, 0.79312, 0.80405, 0.81425, 0.82399, 0.83317,\
        0.84155, 0.84897, 0.85534, 0.86067, 0.86512, 0.86893, 0.87242, 0.87589, 0.87957, 0.88360, 0.88802,\
        0.89280, 0.89775, 0.90162, 0.90440, 0.90728, 0.91078, 0.91507, 0.92119, 0.93052, 0.94244, 0.95496,\
        0.96631, 0.97525, 0.98095, 0.98294, 0.98112, 0.97576, 0.96737, 0.95670, 0.94467, 0.93249, 0.92183,\
        0.91454, 0.91030, 0.90735, 0.90487, 0.90264, 0.90037, 0.89582, 0.89054, 0.88599, 0.88160, 0.87710,\
        0.87248, 0.86768, 0.86256, 0.85691, 0.85055, 0.84336, 0.83533, 0.82648, 0.81689, 0.80679, 0.79690,\
        0.78762, 0.77831, 0.76868, 0.75867, 0.74833, 0.73771, 0.72690, 0.71595, 0.70508, 0.69486, 0.68509,\
        0.67503, 0.66444, 0.65342, 0.64217, 0.63095, 0.61999, 0.60961, 0.60013, 0.59127, 0.58267, 0.57403,\
        0.56528, 0.55646, 0.54770, 0.53914, 0.53092, 0.52320, 0.51611, 0.50978, 0.50432, 0.49996, 0.49684,\
        0.49479, 0.49358, 0.49307, 0.49314, 0.49369, 0.49462, 0.49583, 0.49724, 0.49876, 0.50036, 0.50197],\
        [0.66468, 0.66570, 0.66738, 0.66988, 0.67336, 0.67796, 0.68377, 0.69081, 0.69895, 0.70785, 0.71732,\
        0.72757, 0.73858, 0.75018, 0.76212, 0.77420, 0.78622, 0.79802, 0.80936, 0.82006, 0.83020, 0.83970,\
        0.84839, 0.85607, 0.86256, 0.86782, 0.87197, 0.87531, 0.87826, 0.88115, 0.88421, 0.88751, 0.89100,\
        0.89453, 0.89787, 0.90185, 0.90713, 0.91231, 0.91781, 0.92430, 0.93254, 0.94309, 0.95547, 0.96831,\
        0.98002, 0.98923, 0.99496, 0.99670, 0.99440, 0.98843, 0.97949, 0.96847, 0.95642, 0.94454, 0.93411,\
        0.92606, 0.92026, 0.91583, 0.91196, 0.90813, 0.90362, 0.89826, 0.89473, 0.89148, 0.88780, 0.88361,\
        0.87921, 0.87461, 0.86957, 0.86375, 0.85694, 0.84902, 0.84001, 0.82999, 0.81916, 0.80793, 0.79738,\
        0.78829, 0.77954, 0.77049, 0.76088, 0.75066, 0.73988, 0.72866, 0.71712, 0.70558, 0.69493, 0.68503,\
        0.67495, 0.66441, 0.65339, 0.64203, 0.63051, 0.61905, 0.60795, 0.59801, 0.58962, 0.58152, 0.57330,\
        0.56489, 0.55634, 0.54777, 0.53935, 0.53125, 0.52360, 0.51656, 0.51024, 0.50477, 0.50035, 0.49728,\
        0.49526, 0.49406, 0.49354, 0.49357, 0.49406, 0.49492, 0.49605, 0.49737, 0.49881, 0.50032, 0.50183],\
        [0.66968, 0.67050, 0.67194, 0.67416, 0.67732, 0.68160, 0.68710, 0.69386, 0.70184, 0.71084, 0.72062,\
        0.73113, 0.74230, 0.75400, 0.76605, 0.77827, 0.79049, 0.80250, 0.81420, 0.82540, 0.83602, 0.84598,\
        0.85510, 0.86314, 0.86987, 0.87510, 0.87891, 0.88171, 0.88408, 0.88644, 0.88898, 0.89172, 0.89458,\
        0.89747, 0.90072, 0.90523, 0.91094, 0.91715, 0.92394, 0.93185, 0.94146, 0.95301, 0.96604, 0.97939,\
        0.99156, 1.00111, 1.00695, 1.00850, 1.00577, 0.99930, 0.98997, 0.97886, 0.96703, 0.95556, 0.94535,\
        0.93692, 0.93017, 0.92455, 0.91942, 0.91415, 0.90835, 0.90297, 0.90022, 0.89756, 0.89426, 0.89003,\
        0.88582, 0.88155, 0.87670, 0.87083, 0.86367, 0.85514, 0.84534, 0.83449, 0.82290, 0.81113, 0.80013,\
        0.79071, 0.78217, 0.77339, 0.76394, 0.75372, 0.74280, 0.73135, 0.71959, 0.70793, 0.69688, 0.68641,\
        0.67600, 0.66526, 0.65409, 0.64259, 0.63091, 0.61926, 0.60801, 0.59798, 0.58945, 0.58143, 0.57334,\
        0.56502, 0.55654, 0.54803, 0.53966, 0.53161, 0.52402, 0.51703, 0.51078, 0.50541, 0.50107, 0.49794,\
        0.49585, 0.49458, 0.49398, 0.49395, 0.49438, 0.49517, 0.49624, 0.49750, 0.49887, 0.50030, 0.50173],\
        [0.67448, 0.67505, 0.67615, 0.67799, 0.68076, 0.68464, 0.68978, 0.69628, 0.70418, 0.71331, 0.72335,\
        0.73415, 0.74555, 0.75742, 0.76961, 0.78199, 0.79441, 0.80669, 0.81868, 0.83029, 0.84137, 0.85183,\
        0.86147, 0.87004, 0.87719, 0.88254, 0.88601, 0.88821, 0.89008, 0.89211, 0.89441, 0.89695, 0.89968,\
        0.90259, 0.90603, 0.91039, 0.91567, 0.92168, 0.92857, 0.93677, 0.94679, 0.95877, 0.97226, 0.98614,\
        0.99888, 1.00892, 1.01500, 1.01645, 1.01339, 1.00656, 0.99710, 0.98619, 0.97493, 0.96418, 0.95455,\
        0.94631, 0.93933, 0.93321, 0.92747, 0.92172, 0.91599, 0.91106, 0.90750, 0.90445, 0.90077, 0.89623,\
        0.89241, 0.88859, 0.88403, 0.87820, 0.87076, 0.86171, 0.85130, 0.83990, 0.82796, 0.81599, 0.80469,\
        0.79504, 0.78652, 0.77764, 0.76804, 0.75765, 0.74655, 0.73494, 0.72308, 0.71128, 0.69983, 0.68900,\
        0.67816, 0.66705, 0.65564, 0.64401, 0.63232, 0.62080, 0.60979, 0.59976, 0.59088, 0.58260, 0.57430,\
        0.56579, 0.55715, 0.54853, 0.54010, 0.53202, 0.52445, 0.51752, 0.51136, 0.50611, 0.50187, 0.49869,\
        0.49648, 0.49510, 0.49441, 0.49430, 0.49466, 0.49540, 0.49641, 0.49763, 0.49895, 0.50033, 0.50167],\
        [0.67909, 0.67934, 0.68005, 0.68144, 0.68374, 0.68719, 0.69198, 0.69827, 0.70617, 0.71543, 0.72573,\
        0.73681, 0.74847, 0.76054, 0.77288, 0.78535, 0.79790, 0.81043, 0.82272, 0.83466, 0.84614, 0.85705,\
        0.86722, 0.87643, 0.88426, 0.89005, 0.89327, 0.89484, 0.89651, 0.89857, 0.90103, 0.90370, 0.90653,\
        0.90944, 0.91260, 0.91622, 0.92041, 0.92524, 0.93100, 0.93819, 0.94739, 0.95886, 0.97221, 0.98630,\
        0.99951, 1.01012, 1.01661, 1.01817, 1.01498, 1.00809, 0.99893, 0.98880, 0.97870, 0.96930, 0.96093,\
        0.95365, 0.94726, 0.94144, 0.93584, 0.93027, 0.92484, 0.91993, 0.91569, 0.91173, 0.90741, 0.90295,\
        0.89934, 0.89579, 0.89147, 0.88570, 0.87798, 0.86841, 0.85750, 0.84580, 0.83380, 0.82201, 0.81101,\
        0.80137, 0.79236, 0.78300, 0.77303, 0.76230, 0.75091, 0.73911, 0.72717, 0.71533, 0.70379, 0.69249,\
        0.68113, 0.66958, 0.65788, 0.64615, 0.63455, 0.62326, 0.61250, 0.60248, 0.59348, 0.58485, 0.57607,\
        0.56711, 0.55811, 0.54923, 0.54063, 0.53245, 0.52485, 0.51795, 0.51187, 0.50671, 0.50253, 0.49933,\
        0.49704, 0.49556, 0.49478, 0.49459, 0.49488, 0.49557, 0.49656, 0.49774, 0.49904, 0.50037, 0.50163],\
        [0.68353, 0.68341, 0.68366, 0.68454, 0.68634, 0.68934, 0.69381, 0.69996, 0.70790, 0.71732, 0.72788,\
        0.73926, 0.75120, 0.76349, 0.77596, 0.78853, 0.80115, 0.81383, 0.82633, 0.83848, 0.85021, 0.86143,\
        0.87202, 0.88180, 0.89041, 0.89707, 0.90042, 0.90177, 0.90373, 0.90622, 0.90899, 0.91185, 0.91462,\
        0.91721, 0.91965, 0.92203, 0.92449, 0.92724, 0.93066, 0.93540, 0.94224, 0.95178, 0.96384, 0.97723,\
        0.99029, 1.00130, 1.00845, 1.01050, 1.00769, 1.00143, 0.99345, 0.98509, 0.97720, 0.97013, 0.96396,\
        0.95854, 0.95361, 0.94886, 0.94403, 0.93903, 0.93394, 0.92897, 0.92419, 0.91951, 0.91480, 0.91034,\
        0.90644, 0.90277, 0.89857, 0.89294, 0.88479, 0.87463, 0.86331, 0.85148, 0.83964, 0.82821, 0.81759,\
        0.80792, 0.79851, 0.78881, 0.77844, 0.76725, 0.75548, 0.74343, 0.73136, 0.71948, 0.70785, 0.69626,\
        0.68444, 0.67244, 0.66046, 0.64866, 0.63718, 0.62617, 0.61578, 0.60610, 0.59700, 0.58780, 0.57828,\
        0.56869, 0.55922, 0.55000, 0.54116, 0.53283, 0.52515, 0.51823, 0.51219, 0.50710, 0.50297, 0.49975,\
        0.49743, 0.49588, 0.49502, 0.49476, 0.49502, 0.49568, 0.49664, 0.49780, 0.49908, 0.50037, 0.50157],\
        [0.68778, 0.68725, 0.68699, 0.68731, 0.68859, 0.69114, 0.69530, 0.70137, 0.70941, 0.71906, 0.72992,\
        0.74162, 0.75386, 0.76637, 0.77900, 0.79167, 0.80436, 0.81704, 0.82954, 0.84172, 0.85350, 0.86482,\
        0.87558, 0.88561, 0.89460, 0.90182, 0.90609, 0.90874, 0.91143, 0.91448, 0.91766, 0.92069, 0.92332,\
        0.92535, 0.92670, 0.92740, 0.92755, 0.92741, 0.92738, 0.92817, 0.93078, 0.93635, 0.94515, 0.95579,\
        0.96690, 0.97750, 0.98568, 0.98915, 0.98797, 0.98389, 0.97877, 0.97385, 0.96966, 0.96624, 0.96340,\
        0.96084, 0.95821, 0.95523, 0.95169, 0.94754, 0.94287, 0.93786, 0.93265, 0.92735, 0.92210, 0.91716,\
        0.91272, 0.90865, 0.90447, 0.89893, 0.89017, 0.87947, 0.86796, 0.85623, 0.84467, 0.83363, 0.82330,\
        0.81364, 0.80420, 0.79448, 0.78372, 0.77194, 0.75971, 0.74737, 0.73514, 0.72315, 0.71144, 0.69976,\
        0.68750, 0.67512, 0.66296, 0.65115, 0.63980, 0.62901, 0.61892, 0.60954, 0.60050, 0.59066, 0.58035,\
        0.57015, 0.56024, 0.55069, 0.54160, 0.53309, 0.52527, 0.51828, 0.51223, 0.50720, 0.50315, 0.49998,\
        0.49761, 0.49598, 0.49507, 0.49478, 0.49502, 0.49568, 0.49664, 0.49780, 0.49906, 0.50031, 0.50148],\
        [0.69181, 0.69083, 0.69001, 0.68976, 0.69051, 0.69263, 0.69652, 0.70255, 0.71074, 0.72066, 0.73188,\
        0.74396, 0.75654, 0.76929, 0.78203, 0.79477, 0.80745, 0.82002, 0.83237, 0.84438, 0.85600, 0.86717,\
        0.87783, 0.88781, 0.89677, 0.90430, 0.91003, 0.91448, 0.91854, 0.92254, 0.92635, 0.92965, 0.93212,\
        0.93344, 0.93345, 0.93209, 0.92947, 0.92583, 0.92153, 0.91718, 0.91371, 0.91252, 0.91479, 0.91837,\
        0.92318, 0.93077, 0.94055, 0.94765, 0.95161, 0.95301, 0.95368, 0.95462, 0.95604, 0.95777, 0.95945,\
        0.96070, 0.96115, 0.96054, 0.95869, 0.95561, 0.95144, 0.94640, 0.94075, 0.93475, 0.92868, 0.92283,\
        0.91741, 0.91242, 0.90751, 0.90151, 0.89269, 0.88207, 0.87081, 0.85947, 0.84836, 0.83771, 0.82765,\
        0.81813, 0.80894, 0.79947, 0.78800, 0.77566, 0.76311, 0.75056, 0.73814, 0.72594, 0.71396, 0.70197,\
        0.68959, 0.67719, 0.66505, 0.65331, 0.64204, 0.63133, 0.62125, 0.61175, 0.60246, 0.59225, 0.58163,\
        0.57116, 0.56098, 0.55120, 0.54189, 0.53318, 0.52518, 0.51805, 0.51192, 0.50689, 0.50295, 0.49987,\
        0.49745, 0.49579, 0.49486, 0.49459, 0.49486, 0.49556, 0.49655, 0.49772, 0.49898, 0.50022, 0.50136],\
        [0.69551, 0.69404, 0.69264, 0.69184, 0.69210, 0.69385, 0.69751, 0.70355, 0.71193, 0.72215, 0.73376,\
        0.74630, 0.75932, 0.77236, 0.78514, 0.79773, 0.81036, 0.82277, 0.83483, 0.84648, 0.85773, 0.86855,\
        0.87891, 0.88871, 0.89784, 0.90580, 0.91276, 0.91892, 0.92459, 0.92985, 0.93455, 0.93830, 0.94066,\
        0.94123, 0.93971, 0.93600, 0.93023, 0.92271, 0.91382, 0.90397, 0.89407, 0.88383, 0.87305, 0.86150,\
        0.85248, 0.84965, 0.85891, 0.87621, 0.89492, 0.90812, 0.91885, 0.92843, 0.93732, 0.94554, 0.95276,\
        0.95861, 0.96278, 0.96500, 0.96513, 0.96326, 0.95960, 0.95449, 0.94831, 0.94145, 0.93427, 0.92713,\
        0.92026, 0.91376, 0.90748, 0.90075, 0.89215, 0.88221, 0.87168, 0.86101, 0.85048, 0.84026, 0.83041,\
        0.82087, 0.81141, 0.80144, 0.79006, 0.77788, 0.76540, 0.75281, 0.74025, 0.72780, 0.71545, 0.70309,\
        0.69080, 0.67861, 0.66664, 0.65500, 0.64378, 0.63301, 0.62269, 0.61268, 0.60263, 0.59247, 0.58209,\
        0.57167, 0.56144, 0.55153, 0.54204, 0.53311, 0.52489, 0.51753, 0.51120, 0.50606, 0.50215, 0.49918,\
        0.49679, 0.49519, 0.49436, 0.49418, 0.49454, 0.49531, 0.49636, 0.49758, 0.49885, 0.50009, 0.50121],\
        [0.69868, 0.69666, 0.69476, 0.69350, 0.69336, 0.69479, 0.69832, 0.70439, 0.71293, 0.72346, 0.73550,\
        0.74856, 0.76215, 0.77570, 0.78864, 0.80090, 0.81336, 0.82539, 0.83694, 0.84807, 0.85880, 0.86913,\
        0.87905, 0.88851, 0.89745, 0.90613, 0.91443, 0.92220, 0.92949, 0.93619, 0.94197, 0.94635, 0.94873,\
        0.94854, 0.94536, 0.93902, 0.92971, 0.91793, 0.90438, 0.88956, 0.87293, 0.85192, 0.82342, 0.79054,\
        0.76275, 0.75415, 0.76623, 0.79211, 0.82591, 0.85428, 0.87835, 0.89842, 0.91565, 0.93106, 0.94432,\
        0.95530, 0.96364, 0.96899, 0.97127, 0.97062, 0.96741, 0.96213, 0.95529, 0.94738, 0.93887, 0.93017,\
        0.92158, 0.91327, 0.90521, 0.89754, 0.88949, 0.88046, 0.87086, 0.86102, 0.85116, 0.84139, 0.83173,\
        0.82204, 0.81205, 0.80142, 0.79049, 0.77888, 0.76671, 0.75422, 0.74160, 0.72896, 0.71639, 0.70392,\
        0.69166, 0.67961, 0.66782, 0.65632, 0.64511, 0.63420, 0.62357, 0.61316, 0.60288, 0.59256, 0.58222,\
        0.57192, 0.56174, 0.55177, 0.54213, 0.53298, 0.52448, 0.51680, 0.51015, 0.50472, 0.50063, 0.49771,\
        0.49559, 0.49422, 0.49358, 0.49356, 0.49406, 0.49494, 0.49609, 0.49738, 0.49869, 0.49993, 0.50104],\
        [0.70062, 0.69841, 0.69628, 0.69473, 0.69428, 0.69546, 0.69884, 0.70497, 0.71368, 0.72450, 0.73694,\
        0.75053, 0.76477, 0.77910, 0.79278, 0.80504, 0.81662, 0.82783, 0.83867, 0.84915, 0.85928, 0.86907,\
        0.87854, 0.88771, 0.89669, 0.90590, 0.91523, 0.92440, 0.93323, 0.94140, 0.94842, 0.95361, 0.95618,\
        0.95532, 0.95037, 0.94106, 0.92769, 0.91104, 0.89203, 0.87036, 0.84492, 0.81269, 0.76818, 0.71924,\
        0.67639, 0.66485, 0.69070, 0.72824, 0.76513, 0.80096, 0.83773, 0.86858, 0.89391, 0.91589, 0.93515,\
        0.95158, 0.96440, 0.97305, 0.97745, 0.97790, 0.97500, 0.96938, 0.96171, 0.95261, 0.94263, 0.93227,\
        0.92192, 0.91192, 0.90253, 0.89404, 0.88604, 0.87769, 0.86894, 0.85990, 0.85072, 0.84143, 0.83201,\
        0.82234, 0.81223, 0.80163, 0.79076, 0.77940, 0.76746, 0.75508, 0.74243, 0.72966, 0.71694, 0.70446,\
        0.69223, 0.68034, 0.66877, 0.65742, 0.64624, 0.63520, 0.62427, 0.61348, 0.60291, 0.59255, 0.58236,\
        0.57223, 0.56212, 0.55210, 0.54231, 0.53291, 0.52409, 0.51606, 0.50902, 0.50322, 0.49890, 0.49604,\
        0.49415, 0.49305, 0.49262, 0.49279, 0.49344, 0.49446, 0.49574, 0.49712, 0.49850, 0.49976, 0.50086],\
        [0.70178, 0.69956, 0.69735, 0.69562, 0.69493, 0.69590, 0.69917, 0.70530, 0.71412, 0.72517, 0.73793,\
        0.75192, 0.76665, 0.78160, 0.79601, 0.80832, 0.81919, 0.82965, 0.83983, 0.84969, 0.85924, 0.86851,\
        0.87756, 0.88654, 0.89571, 0.90523, 0.91517, 0.92547, 0.93572, 0.94537, 0.95374, 0.95993, 0.96292,\
        0.96156, 0.95484, 0.94215, 0.92389, 0.90154, 0.87552, 0.84274, 0.80303, 0.75725, 0.70758, 0.65684,\
        0.60783, 0.58256, 0.61037, 0.66719, 0.71122, 0.74967, 0.79536, 0.83746, 0.87214, 0.90046, 0.92593,\
        0.94825, 0.96584, 0.97774, 0.98402, 0.98530, 0.98243, 0.97627, 0.96763, 0.95725, 0.94576, 0.93375,\
        0.92176, 0.91029, 0.89992, 0.89087, 0.88267, 0.87464, 0.86649, 0.85812, 0.84955, 0.84076, 0.83169,\
        0.82225, 0.81233, 0.80193, 0.79113, 0.77983, 0.76798, 0.75566, 0.74299, 0.73013, 0.71726, 0.70461,\
        0.69253, 0.68098, 0.66970, 0.65854, 0.64741, 0.63629, 0.62515, 0.61401, 0.60305, 0.59278, 0.58289,\
        0.57290, 0.56280, 0.55268, 0.54271, 0.53305, 0.52391, 0.51551, 0.50808, 0.50195, 0.49751, 0.49461,\
        0.49283, 0.49188, 0.49161, 0.49193, 0.49271, 0.49387, 0.49527, 0.49679, 0.49827, 0.49956, 0.50067],\
        [0.70270, 0.70046, 0.69819, 0.69630, 0.69539, 0.69614, 0.69926, 0.70535, 0.71425, 0.72542, 0.73836,\
        0.75253, 0.76742, 0.78244, 0.79672, 0.80924, 0.82023, 0.83048, 0.84027, 0.84968, 0.85875, 0.86754,\
        0.87616, 0.88483, 0.89381, 0.90338, 0.91397, 0.92531, 0.93686, 0.94793, 0.95771, 0.96510, 0.96878,\
        0.96726, 0.95899, 0.94267, 0.91813, 0.88837, 0.85352, 0.80594, 0.74831, 0.70113, 0.65553, 0.59497,\
        0.54091, 0.52313, 0.55003, 0.60290, 0.66011, 0.69481, 0.74614, 0.80178, 0.84862, 0.88494, 0.91721,\
        0.94628, 0.96880, 0.98357, 0.99120, 0.99285, 0.98968, 0.98277, 0.97306, 0.96137, 0.94843, 0.93490,\
        0.92140, 0.90861, 0.89736, 0.88792, 0.87957, 0.87170, 0.86393, 0.85607, 0.84801, 0.83969, 0.83103,\
        0.82194, 0.81235, 0.80221, 0.79152, 0.78025, 0.76843, 0.75614, 0.74350, 0.73064, 0.71771, 0.70504,\
        0.69309, 0.68186, 0.67086, 0.65985, 0.64880, 0.63769, 0.62653, 0.61535, 0.60432, 0.59412, 0.58424,\
        0.57418, 0.56394, 0.55364, 0.54343, 0.53351, 0.52407, 0.51535, 0.50760, 0.50118, 0.49659, 0.49361,\
        0.49180, 0.49087, 0.49066, 0.49104, 0.49190, 0.49313, 0.49464, 0.49630, 0.49796, 0.49947, 0.50059],\
        [0.70350, 0.70125, 0.69892, 0.69688, 0.69576, 0.69626, 0.69915, 0.70519, 0.71411, 0.72532, 0.73828,\
        0.75243, 0.76720, 0.78197, 0.79604, 0.80868, 0.81999, 0.83036, 0.84005, 0.84920, 0.85791, 0.86629,\
        0.87448, 0.88270, 0.89124, 0.90067, 0.91176, 0.92393, 0.93653, 0.94888, 0.96006, 0.96879, 0.97347,\
        0.97218, 0.96289, 0.94355, 0.91214, 0.87077, 0.82216, 0.75984, 0.69444, 0.65754, 0.59267, 0.50582,\
        0.46822, 0.46083, 0.47609, 0.51945, 0.59723, 0.64967, 0.69874, 0.75646, 0.81971, 0.86731, 0.90972,\
        0.94731, 0.97413, 0.99070, 0.99887, 1.00034, 0.99655, 0.98874, 0.97791, 0.96498, 0.95073, 0.93588,\
        0.92114, 0.90725, 0.89527, 0.88546, 0.87692, 0.86907, 0.86152, 0.85401, 0.84636, 0.83845, 0.83018,\
        0.82147, 0.81224, 0.80244, 0.79184, 0.78057, 0.76880, 0.75660, 0.74408, 0.73136, 0.71856, 0.70592,\
        0.69417, 0.68327, 0.67239, 0.66144, 0.65043, 0.63943, 0.62844, 0.61756, 0.60694, 0.59668, 0.58642,\
        0.57607, 0.56555, 0.55497, 0.54449, 0.53431, 0.52463, 0.51568, 0.50772, 0.50111, 0.49631, 0.49311,\
        0.49113, 0.49008, 0.48980, 0.49013, 0.49097, 0.49221, 0.49374, 0.49546, 0.49724, 0.49893, 0.50005],\
        [0.70421, 0.70194, 0.69960, 0.69744, 0.69612, 0.69640, 0.69912, 0.70500, 0.71384, 0.72499, 0.73786,\
        0.75186, 0.76641, 0.78086, 0.79463, 0.80739, 0.81906, 0.82968, 0.83942, 0.84845, 0.85691, 0.86495,\
        0.87275, 0.88056, 0.88876, 0.89796, 0.90899, 0.92142, 0.93462, 0.94797, 0.96046, 0.97062, 0.97653,\
        0.97581, 0.96577, 0.94380, 0.90744, 0.84961, 0.78108, 0.71003, 0.65170, 0.59290, 0.49114, 0.43368,\
        0.41697, 0.41427, 0.42184, 0.44457, 0.50050, 0.58037, 0.64365, 0.70990, 0.78267, 0.84945, 0.90973,\
        0.95308, 0.98155, 0.99845, 1.00636, 1.00725, 1.00267, 0.99391, 0.98204, 0.96801, 0.95266, 0.93676,\
        0.92111, 0.90654, 0.89400, 0.88365, 0.87479, 0.86685, 0.85939, 0.85210, 0.84475, 0.83717, 0.82924,\
        0.82084, 0.81187, 0.80225, 0.79182, 0.78069, 0.76903, 0.75697, 0.74467, 0.73227, 0.71994, 0.70785,\
        0.69638, 0.68533, 0.67423, 0.66314, 0.65214, 0.64127, 0.63055, 0.62002, 0.60968, 0.59938, 0.58898,\
        0.57833, 0.56747, 0.55656, 0.54581, 0.53540, 0.52554, 0.51644, 0.50837, 0.50162, 0.49657, 0.49307,\
        0.49079, 0.48951, 0.48903, 0.48922, 0.48993, 0.49106, 0.49249, 0.49411, 0.49578, 0.49725, 0.49814],\
        [0.70487, 0.70257, 0.70023, 0.69801, 0.69652, 0.69660, 0.69911, 0.70485, 0.71355, 0.72457, 0.73730,\
        0.75114, 0.76548, 0.77972, 0.79331, 0.80614, 0.81808, 0.82888, 0.83869, 0.84766, 0.85595, 0.86372,\
        0.87116, 0.87860, 0.88646, 0.89537, 0.90581, 0.91769, 0.93094, 0.94495, 0.95858, 0.97016, 0.97748,\
        0.97765, 0.96699, 0.94075, 0.89499, 0.82841, 0.74274, 0.66114, 0.59170, 0.51338, 0.43922, 0.40711,\
        0.39980, 0.39609, 0.40178, 0.41488, 0.45323, 0.51919, 0.59365, 0.67252, 0.74927, 0.83498, 0.91216,\
        0.95926, 0.98836, 1.00515, 1.01261, 1.01287, 1.00754, 0.99796, 0.98522, 0.97031, 0.95410, 0.93742,\
        0.92114, 0.90612, 0.89314, 0.88231, 0.87313, 0.86504, 0.85758, 0.85041, 0.84327, 0.83594, 0.82826,\
        0.82010, 0.81131, 0.80179, 0.79153, 0.78057, 0.76904, 0.75714, 0.74507, 0.73305, 0.72128, 0.70996,\
        0.69892, 0.68738, 0.67586, 0.66459, 0.65359, 0.64287, 0.63243, 0.62224, 0.61219, 0.60208, 0.59161,\
        0.58064, 0.56942, 0.55821, 0.54722, 0.53664, 0.52666, 0.51749, 0.50936, 0.50255, 0.49722, 0.49332,\
        0.49068, 0.48908, 0.48833, 0.48827, 0.48875, 0.48967, 0.49088, 0.49226, 0.49364, 0.49479, 0.49546],\
        [0.70547, 0.70315, 0.70082, 0.69857, 0.69696, 0.69688, 0.69924, 0.70480, 0.71333, 0.72420, 0.73677,\
        0.75046, 0.76470, 0.77893, 0.79267, 0.80558, 0.81745, 0.82826, 0.83809, 0.84705, 0.85522, 0.86276,\
        0.86985, 0.87681, 0.88408, 0.89220, 0.90152, 0.91244, 0.92534, 0.93967, 0.95422, 0.96712, 0.97590,\
        0.97742, 0.96744, 0.93984, 0.88641, 0.80724, 0.71386, 0.62348, 0.54947, 0.47751, 0.42296, 0.40150,\
        0.39501, 0.39218, 0.39639, 0.40536, 0.43314, 0.48194, 0.54983, 0.63276, 0.72238, 0.82680, 0.90857,\
        0.96054, 0.99180, 1.00922, 1.01664, 1.01656, 1.01074, 1.00059, 0.98726, 0.97173, 0.95493, 0.93771,\
        0.92095, 0.90555, 0.89228, 0.88118, 0.87179, 0.86357, 0.85608, 0.84897, 0.84194, 0.83478, 0.82730,\
        0.81933, 0.81071, 0.80131, 0.79114, 0.78023, 0.76876, 0.75695, 0.74502, 0.73321, 0.72175, 0.71074,\
        0.69981, 0.68823, 0.67662, 0.66533, 0.65443, 0.64390, 0.63373, 0.62385, 0.61415, 0.60437, 0.59384,\
        0.58255, 0.57106, 0.55966, 0.54854, 0.53787, 0.52783, 0.51861, 0.51043, 0.50349, 0.49791, 0.49366,\
        0.49066, 0.48871, 0.48763, 0.48725, 0.48743, 0.48804, 0.48894, 0.48999, 0.49102, 0.49186, 0.49233],\
        [0.70604, 0.70368, 0.70134, 0.69910, 0.69742, 0.69720, 0.69939, 0.70483, 0.71323, 0.72395, 0.73639,\
        0.74995, 0.76412, 0.77841, 0.79239, 0.80538, 0.81709, 0.82786, 0.83776, 0.84676, 0.85490, 0.86226,\
        0.86899, 0.87535, 0.88171, 0.88851, 0.89627, 0.90593, 0.91803, 0.93231, 0.94748, 0.96145, 0.97152,\
        0.97444, 0.96608, 0.94042, 0.88449, 0.79542, 0.71124, 0.61791, 0.55664, 0.48123, 0.42436, 0.40134,\
        0.39438, 0.39239, 0.39595, 0.40540, 0.43595, 0.49245, 0.55583, 0.65618, 0.74624, 0.82668, 0.90464,\
        0.95889, 0.99202, 1.01035, 1.01810, 1.01802, 1.01205, 1.00165, 0.98802, 0.97219, 0.95506, 0.93753,\
        0.92047, 0.90479, 0.89137, 0.88020, 0.87070, 0.86239, 0.85485, 0.84773, 0.84076, 0.83369, 0.82633,\
        0.81850, 0.81006, 0.80082, 0.79061, 0.77962, 0.76811, 0.75628, 0.74434, 0.73250, 0.72097, 0.70982,\
        0.69882, 0.68748, 0.67616, 0.66512, 0.65443, 0.64412, 0.63417, 0.62450, 0.61499, 0.60534, 0.59488,\
        0.58359, 0.57209, 0.56070, 0.54959, 0.53894, 0.52891, 0.51967, 0.51141, 0.50430, 0.49846, 0.49395,\
        0.49063, 0.48834, 0.48690, 0.48617, 0.48598, 0.48622, 0.48674, 0.48742, 0.48810, 0.48863, 0.48889],\
        [0.70657, 0.70417, 0.70183, 0.69964, 0.69790, 0.69757, 0.69967, 0.70499, 0.71328, 0.72390, 0.73622,\
        0.74965, 0.76368, 0.77787, 0.79182, 0.80485, 0.81665, 0.82763, 0.83775, 0.84692, 0.85513, 0.86239,\
        0.86880, 0.87456, 0.87993, 0.88535, 0.89141, 0.89910, 0.90972, 0.92354, 0.93899, 0.95365, 0.96456,\
        0.96830, 0.96079, 0.93634, 0.88554, 0.81021, 0.73483, 0.66697, 0.61270, 0.52573, 0.44660, 0.41193,\
        0.40154, 0.39792, 0.40344, 0.42041, 0.46317, 0.53538, 0.61945, 0.70985, 0.77219, 0.83544, 0.90697,\
        0.95827, 0.99082, 1.00929, 1.01727, 1.01735, 1.01147, 1.00111, 0.98750, 0.97165, 0.95450, 0.93692,\
        0.91978, 0.90399, 0.89058, 0.87940, 0.86984, 0.86145, 0.85383, 0.84666, 0.83968, 0.83263, 0.82530,\
        0.81751, 0.80911, 0.79996, 0.78967, 0.77861, 0.76702, 0.75509, 0.74302, 0.73099, 0.71913, 0.70757,\
        0.69642, 0.68550, 0.67464, 0.66395, 0.65353, 0.64344, 0.63365, 0.62410, 0.61461, 0.60490, 0.59457,\
        0.58361, 0.57239, 0.56120, 0.55027, 0.53975, 0.52981, 0.52060, 0.51228, 0.50500, 0.49894, 0.49419,\
        0.49058, 0.48795, 0.48613, 0.48500, 0.48441, 0.48423, 0.48435, 0.48463, 0.48495, 0.48519, 0.48524],\
        [0.70707, 0.70464, 0.70230, 0.70012, 0.69836, 0.69792, 0.69991, 0.70521, 0.71348, 0.72408, 0.73634,\
        0.74967, 0.76349, 0.77736, 0.79092, 0.80390, 0.81618, 0.82764, 0.83815, 0.84764, 0.85603, 0.86332,\
        0.86954, 0.87481, 0.87935, 0.88351, 0.88789, 0.89343, 0.90193, 0.91460, 0.92992, 0.94490, 0.95620,\
        0.96031, 0.95326, 0.93034, 0.88758, 0.82857, 0.76401, 0.70463, 0.65976, 0.59976, 0.50382, 0.44608,\
        0.42156, 0.41496, 0.42502, 0.45939, 0.52888, 0.60525, 0.67574, 0.74142, 0.79273, 0.85279, 0.91423,\
        0.95943, 0.98937, 1.00693, 1.01471, 1.01489, 1.00922, 0.99911, 0.98575, 0.97017, 0.95326, 0.93592,\
        0.91898, 0.90340, 0.89006, 0.87884, 0.86920, 0.86070, 0.85296, 0.84569, 0.83863, 0.83154, 0.82420,\
        0.81635, 0.80776, 0.79837, 0.78821, 0.77716, 0.76548, 0.75342, 0.74117, 0.72892, 0.71678, 0.70490,\
        0.69365, 0.68296, 0.67242, 0.66203, 0.65185, 0.64192, 0.63224, 0.62274, 0.61326, 0.60358, 0.59343,\
        0.58284, 0.57202, 0.56119, 0.55056, 0.54028, 0.53051, 0.52139, 0.51306, 0.50568, 0.49943, 0.49442,\
        0.49051, 0.48752, 0.48531, 0.48375, 0.48272, 0.48211, 0.48180, 0.48168, 0.48165, 0.48159, 0.48142],\
        [0.70755, 0.70508, 0.70272, 0.70056, 0.69879, 0.69826, 0.70021, 0.70552, 0.71385, 0.72452, 0.73685,\
        0.75016, 0.76384, 0.77740, 0.79057, 0.80348, 0.81619, 0.82814, 0.83909, 0.84894, 0.85764, 0.86512,\
        0.87135, 0.87638, 0.88037, 0.88364, 0.88670, 0.89042, 0.89639, 0.90709, 0.92194, 0.93686, 0.94825,\
        0.95282, 0.94723, 0.92832, 0.89474, 0.84896, 0.79546, 0.74103, 0.69360, 0.65193, 0.58997, 0.51059,\
        0.46692, 0.45635, 0.47654, 0.54394, 0.61584, 0.66003, 0.71304, 0.77276, 0.82137, 0.87412, 0.92381,\
        0.96188, 0.98808, 1.00388, 1.01099, 1.01107, 1.00560, 0.99585, 0.98293, 0.96783, 0.95142, 0.93455,\
        0.91804, 0.90279, 0.88964, 0.87844, 0.86871, 0.86006, 0.85215, 0.84473, 0.83753, 0.83035, 0.82296,\
        0.81513, 0.80651, 0.79696, 0.78662, 0.77540, 0.76355, 0.75133, 0.73893, 0.72652, 0.71426, 0.70234,\
        0.69104, 0.68028, 0.66983, 0.65958, 0.64953, 0.63971, 0.63009, 0.62061, 0.61116, 0.60159, 0.59170,\
        0.58150, 0.57113, 0.56075, 0.55051, 0.54055, 0.53102, 0.52204, 0.51376, 0.50632, 0.49990, 0.49465,\
        0.49043, 0.48707, 0.48444, 0.48243, 0.48094, 0.47987, 0.47912, 0.47860, 0.47821, 0.47786, 0.47748],\
        [0.70801, 0.70548, 0.70309, 0.70093, 0.69915, 0.69851, 0.70044, 0.70585, 0.71435, 0.72524, 0.73778,\
        0.75125, 0.76499, 0.77848, 0.79146, 0.80430, 0.81717, 0.82934, 0.84056, 0.85076, 0.85985, 0.86769,\
        0.87417, 0.87926, 0.88308, 0.88594, 0.88833, 0.89089, 0.89474, 0.90312, 0.91691, 0.93111, 0.94220,\
        0.94740, 0.94423, 0.93086, 0.90676, 0.87187, 0.82919, 0.78246, 0.73342, 0.68910, 0.64561, 0.58661,\
        0.52997, 0.51454, 0.55520, 0.62537, 0.67081, 0.70361, 0.75637, 0.81301, 0.85553, 0.89692, 0.93466,\
        0.96513, 0.98700, 1.00048, 1.00652, 1.00628, 1.00092, 0.99156, 0.97919, 0.96472, 0.94899, 0.93280,\
        0.91692, 0.90214, 0.88926, 0.87811, 0.86828, 0.85944, 0.85131, 0.84366, 0.83627, 0.82892, 0.82143,\
        0.81364, 0.80533, 0.79572, 0.78484, 0.77326, 0.76120, 0.74885, 0.73636, 0.72387, 0.71158, 0.69969,\
        0.68834, 0.67749, 0.66699, 0.65676, 0.64675, 0.63696, 0.62736, 0.61791, 0.60852, 0.59908, 0.58952,\
        0.57976, 0.56988, 0.55998, 0.55018, 0.54060, 0.53136, 0.52259, 0.51441, 0.50695, 0.50040, 0.49489,\
        0.49034, 0.48658, 0.48350, 0.48102, 0.47905, 0.47751, 0.47632, 0.47540, 0.47466, 0.47402, 0.47343],\
        [0.70843, 0.70583, 0.70338, 0.70119, 0.69937, 0.69862, 0.70052, 0.70612, 0.71491, 0.72615, 0.73909,\
        0.75294, 0.76700, 0.78071, 0.79382, 0.80657, 0.81911, 0.83107, 0.84231, 0.85278, 0.86234, 0.87076,\
        0.87775, 0.88318, 0.88714, 0.89004, 0.89242, 0.89493, 0.89811, 0.90438, 0.91594, 0.92838, 0.93861,\
        0.94424, 0.94342, 0.93496, 0.91838, 0.89312, 0.86049, 0.82454, 0.78606, 0.74485, 0.70016, 0.65860,\
        0.60696, 0.57998, 0.62931, 0.67861, 0.71890, 0.75954, 0.80721, 0.84902, 0.88217, 0.91455, 0.94412,\
        0.96831, 0.98596, 0.99690, 1.00158, 1.00080, 0.99545, 0.98644, 0.97466, 0.96092, 0.94601, 0.93065,\
        0.91557, 0.90144, 0.88885, 0.87771, 0.86776, 0.85870, 0.85032, 0.84239, 0.83473, 0.82712, 0.81938,\
        0.81133, 0.80287, 0.79330, 0.78226, 0.77053, 0.75837, 0.74597, 0.73347, 0.72100, 0.70871, 0.69681,\
        0.68547, 0.67454, 0.66396, 0.65367, 0.64363, 0.63382, 0.62422, 0.61482, 0.60554, 0.59632, 0.58709,\
        0.57777, 0.56838, 0.55897, 0.54964, 0.54047, 0.53157, 0.52304, 0.51500, 0.50755, 0.50087, 0.49512,\
        0.49022, 0.48603, 0.48248, 0.47949, 0.47703, 0.47501, 0.47338, 0.47206, 0.47099, 0.47008, 0.46927],\
        [0.70881, 0.70612, 0.70359, 0.70130, 0.69940, 0.69864, 0.70060, 0.70631, 0.71541, 0.72711, 0.74058,\
        0.75501, 0.76961, 0.78372, 0.79702, 0.80951, 0.82137, 0.83275, 0.84377, 0.85444, 0.86458, 0.87381,\
        0.88162, 0.88762, 0.89188, 0.89496, 0.89756, 0.90039, 0.90384, 0.90925, 0.91800, 0.92811, 0.93702,\
        0.94268, 0.94353, 0.93856, 0.92718, 0.90919, 0.88577, 0.86094, 0.83673, 0.81100, 0.77515, 0.74322,\
        0.72698, 0.69772, 0.70875, 0.73843, 0.77473, 0.81292, 0.84901, 0.87928, 0.90329, 0.92879, 0.95210,\
        0.97104, 0.98478, 0.99314, 0.99632, 0.99484, 0.98938, 0.98067, 0.96947, 0.95650, 0.94247, 0.92802,\
        0.91379, 0.90040, 0.88814, 0.87708, 0.86701, 0.85773, 0.84905, 0.84081, 0.83283, 0.82492, 0.81684,\
        0.80830, 0.79913, 0.78952, 0.77877, 0.76715, 0.75504, 0.74268, 0.73026, 0.71792, 0.70575, 0.69389,\
        0.68247, 0.67144, 0.66075, 0.65037, 0.64026, 0.63041, 0.62082, 0.61148, 0.60237, 0.59341, 0.58453,\
        0.57565, 0.56673, 0.55781, 0.54894, 0.54020, 0.53165, 0.52340, 0.51553, 0.50814, 0.50136, 0.49535,\
        0.49006, 0.48539, 0.48132, 0.47782, 0.47484, 0.47235, 0.47028, 0.46859, 0.46719, 0.46602, 0.46502],\
        [0.70914, 0.70636, 0.70374, 0.70137, 0.69938, 0.69849, 0.70037, 0.70626, 0.71570, 0.72785, 0.74191,\
        0.75704, 0.77233, 0.78687, 0.80010, 0.81207, 0.82301, 0.83353, 0.84410, 0.85491, 0.86577, 0.87614,\
        0.88516, 0.89194, 0.89641, 0.89954, 0.90215, 0.90496, 0.90851, 0.91360, 0.92068, 0.92885, 0.93647,\
        0.94189, 0.94382, 0.94132, 0.93386, 0.92143, 0.90537, 0.88892, 0.87324, 0.85739, 0.83434, 0.80200,\
        0.78551, 0.77979, 0.78755, 0.80707, 0.83291, 0.85942, 0.88539, 0.90741, 0.92516, 0.94276, 0.95947,\
        0.97333, 0.98335, 0.98915, 0.99077, 0.98850, 0.98283, 0.97435, 0.96369, 0.95149, 0.93838, 0.92487,\
        0.91144, 0.89867, 0.88695, 0.87606, 0.86590, 0.85638, 0.84740, 0.83883, 0.83052, 0.82230, 0.81399,\
        0.80526, 0.79579, 0.78571, 0.77488, 0.76334, 0.75130, 0.73900, 0.72669, 0.71455, 0.70265, 0.69082,\
        0.67930, 0.66817, 0.65738, 0.64691, 0.63673, 0.62684, 0.61725, 0.60798, 0.59907, 0.59042, 0.58192,\
        0.57346, 0.56501, 0.55656, 0.54814, 0.53980, 0.53160, 0.52363, 0.51595, 0.50866, 0.50182, 0.49552,\
        0.48977, 0.48458, 0.47996, 0.47592, 0.47244, 0.46948, 0.46700, 0.46495, 0.46325, 0.46185, 0.46067],\
        [0.70941, 0.70653, 0.70382, 0.70136, 0.69927, 0.69827, 0.70002, 0.70598, 0.71561, 0.72806, 0.74259,\
        0.75839, 0.77449, 0.78969, 0.80276, 0.81365, 0.82312, 0.83241, 0.84235, 0.85324, 0.86493, 0.87677,\
        0.88756, 0.89553, 0.89990, 0.90270, 0.90526, 0.90809, 0.91166, 0.91642, 0.92249, 0.92935, 0.93596,\
        0.94114, 0.94387, 0.94338, 0.93929, 0.93175, 0.92183, 0.91117, 0.90066, 0.89325, 0.88744, 0.87459,\
        0.86419, 0.86358, 0.87085, 0.88152, 0.89024, 0.90107, 0.91645, 0.93069, 0.94296, 0.95459, 0.96557,\
        0.97481, 0.98139, 0.98479, 0.98489, 0.98182, 0.97588, 0.96756, 0.95737, 0.94590, 0.93372, 0.92128,\
        0.90881, 0.89669, 0.88542, 0.87461, 0.86431, 0.85456, 0.84527, 0.83636, 0.82770, 0.81917, 0.81062,\
        0.80187, 0.79219, 0.78178, 0.77081, 0.75927, 0.74728, 0.73503, 0.72276, 0.71072, 0.69896, 0.68731,\
        0.67586, 0.66471, 0.65387, 0.64333, 0.63311, 0.62319, 0.61362, 0.60444, 0.59574, 0.58744, 0.57931,\
        0.57127, 0.56325, 0.55524, 0.54723, 0.53927, 0.53139, 0.52366, 0.51616, 0.50895, 0.50209, 0.49546,\
        0.48919, 0.48346, 0.47830, 0.47374, 0.46977, 0.46637, 0.46350, 0.46111, 0.45915, 0.45754, 0.45622],\
        [0.70960, 0.70660, 0.70380, 0.70129, 0.69913, 0.69810, 0.69976, 0.70550, 0.71495, 0.72738, 0.74207,\
        0.75825, 0.77498, 0.79101, 0.80408, 0.81308, 0.82062, 0.82848, 0.83765, 0.84853, 0.86099, 0.87436,\
        0.88740, 0.89769, 0.90155, 0.90363, 0.90628, 0.90927, 0.91288, 0.91737, 0.92280, 0.92881, 0.93476,\
        0.93984, 0.94331, 0.94461, 0.94352, 0.94027, 0.93562, 0.93074, 0.92725, 0.92795, 0.93281, 0.93493,\
        0.93526, 0.93543, 0.93672, 0.93794, 0.93628, 0.93690, 0.94204, 0.94893, 0.95592, 0.96283, 0.96937,\
        0.97486, 0.97853, 0.97986, 0.97861, 0.97478, 0.96858, 0.96035, 0.95055, 0.93970, 0.92836, 0.91711,\
        0.90619, 0.89492, 0.88355, 0.87257, 0.86211, 0.85214, 0.84259, 0.83337, 0.82437, 0.81549, 0.80658,\
        0.79746, 0.78784, 0.77749, 0.76656, 0.75510, 0.74317, 0.73092, 0.71858, 0.70657, 0.69493, 0.68350,\
        0.67220, 0.66109, 0.65024, 0.63967, 0.62943, 0.61954, 0.61004, 0.60102, 0.59257, 0.58454, 0.57676,\
        0.56909, 0.56147, 0.55385, 0.54623, 0.53860, 0.53099, 0.52345, 0.51602, 0.50878, 0.50175, 0.49481,\
        0.48812, 0.48191, 0.47625, 0.47120, 0.46677, 0.46296, 0.45974, 0.45707, 0.45488, 0.45310, 0.45168],\
        [0.70967, 0.70652, 0.70358, 0.70097, 0.69876, 0.69774, 0.69930, 0.70455, 0.71348, 0.72548, 0.73987,\
        0.75584, 0.77241, 0.78828, 0.80093, 0.80849, 0.81456, 0.82116, 0.82957, 0.84032, 0.85325, 0.86761,\
        0.88205, 0.89403, 0.89852, 0.90153, 0.90478, 0.90813, 0.91189, 0.91628, 0.92132, 0.92682, 0.93239,\
        0.93753, 0.94174, 0.94465, 0.94617, 0.94655, 0.94643, 0.94687, 0.94928, 0.95497, 0.96304, 0.97001,\
        0.97418, 0.97535, 0.97405, 0.97055, 0.96530, 0.96115, 0.95998, 0.96123, 0.96381, 0.96703, 0.97031,\
        0.97301, 0.97447, 0.97418, 0.97185, 0.96741, 0.96097, 0.95282, 0.94333, 0.93294, 0.92214, 0.91156,\
        0.90191, 0.89172, 0.88051, 0.86959, 0.85910, 0.84904, 0.83933, 0.82988, 0.82059, 0.81138, 0.80219,\
        0.79287, 0.78320, 0.77298, 0.76220, 0.75092, 0.73917, 0.72700, 0.71451, 0.70230, 0.69095, 0.67973,\
        0.66852, 0.65741, 0.64652, 0.63595, 0.62574, 0.61592, 0.60655, 0.59773, 0.58952, 0.58174, 0.57425,\
        0.56691, 0.55965, 0.55240, 0.54512, 0.53778, 0.53039, 0.52295, 0.51551, 0.50811, 0.50078, 0.49353,\
        0.48650, 0.47986, 0.47374, 0.46824, 0.46340, 0.45922, 0.45570, 0.45278, 0.45042, 0.44853, 0.44703],\
        [0.70961, 0.70626, 0.70311, 0.70027, 0.69797, 0.69695, 0.69828, 0.70291, 0.71102, 0.72225, 0.73591,\
        0.75106, 0.76652, 0.78083, 0.79210, 0.79960, 0.80508, 0.81076, 0.81847, 0.82896, 0.84197, 0.85660,\
        0.87118, 0.88324, 0.89093, 0.89615, 0.90052, 0.90454, 0.90867, 0.91312, 0.91798, 0.92320, 0.92859,\
        0.93390, 0.93880, 0.94310, 0.94676, 0.95001, 0.95338, 0.95762, 0.96354, 0.97142, 0.98026, 0.98793,\
        0.99268, 0.99378, 0.99138, 0.98618, 0.97945, 0.97319, 0.96887, 0.96677, 0.96640, 0.96712, 0.96824,\
        0.96910, 0.96908, 0.96769, 0.96462, 0.95976, 0.95317, 0.94510, 0.93589, 0.92588, 0.91540, 0.90496,\
        0.89531, 0.88603, 0.87589, 0.86548, 0.85522, 0.84522, 0.83549, 0.82594, 0.81646, 0.80700, 0.79763,\
        0.78823, 0.77853, 0.76841, 0.75782, 0.74680, 0.73537, 0.72359, 0.71147, 0.69917, 0.68772, 0.67633,\
        0.66493, 0.65370, 0.64274, 0.63215, 0.62199, 0.61231, 0.60312, 0.59452, 0.58651, 0.57896, 0.57174,\
        0.56471, 0.55778, 0.55086, 0.54388, 0.53680, 0.52957, 0.52219, 0.51467, 0.50705, 0.49938, 0.49178,\
        0.48436, 0.47730, 0.47076, 0.46484, 0.45962, 0.45513, 0.45135, 0.44825, 0.44577, 0.44382, 0.44231],\
        [0.70942, 0.70584, 0.70243, 0.69933, 0.69688, 0.69573, 0.69669, 0.70052, 0.70756, 0.71785, 0.73060,\
        0.74462, 0.75862, 0.77129, 0.78141, 0.78865, 0.79390, 0.79889, 0.80587, 0.81583, 0.82854, 0.84312,\
        0.85783, 0.87065, 0.88050, 0.88785, 0.89369, 0.89869, 0.90338, 0.90806, 0.91287, 0.91793, 0.92326,\
        0.92877, 0.93428, 0.93966, 0.94493, 0.95025, 0.95596, 0.96246, 0.97005, 0.97851, 0.98691, 0.99377,\
        0.99773, 0.99810, 0.99498, 0.98913, 0.98188, 0.97483, 0.96925, 0.96565, 0.96385, 0.96331, 0.96333,\
        0.96324, 0.96245, 0.96048, 0.95702, 0.95195, 0.94531, 0.93733, 0.92841, 0.91891, 0.90887, 0.89881,\
        0.88938, 0.88011, 0.87047, 0.86054, 0.85057, 0.84075, 0.83111, 0.82162, 0.81212, 0.80257, 0.79308,\
        0.78359, 0.77386, 0.76380, 0.75340, 0.74266, 0.73164, 0.72039, 0.70910, 0.69767, 0.68522, 0.67313,\
        0.66135, 0.64989, 0.63882, 0.62820, 0.61812, 0.60861, 0.59969, 0.59128, 0.58345, 0.57612, 0.56914,\
        0.56240, 0.55578, 0.54917, 0.54248, 0.53563, 0.52855, 0.52119, 0.51355, 0.50567, 0.49765, 0.48963,\
        0.48177, 0.47426, 0.46728, 0.46097, 0.45541, 0.45065, 0.44669, 0.44348, 0.44094, 0.43899, 0.43753],\
        [0.70910, 0.70528, 0.70160, 0.69826, 0.69560, 0.69418, 0.69461, 0.69746, 0.70338, 0.71285, 0.72473,\
        0.73759, 0.75023, 0.76158, 0.77085, 0.77786, 0.78321, 0.78815, 0.79435, 0.80306, 0.81504, 0.82947,\
        0.84448, 0.85808, 0.86928, 0.87814, 0.88524, 0.89117, 0.89645, 0.90140, 0.90623, 0.91115, 0.91646,\
        0.92216, 0.92812, 0.93427, 0.94061, 0.94722, 0.95428, 0.96189, 0.96999, 0.97816, 0.98554, 0.99105,\
        0.99371, 0.99304, 0.98919, 0.98292, 0.97549, 0.96833, 0.96259, 0.95881, 0.95685, 0.95616, 0.95602,\
        0.95577, 0.95483, 0.95275, 0.94924, 0.94417, 0.93758, 0.92967, 0.92095, 0.91217, 0.90315, 0.89335,\
        0.88381, 0.87436, 0.86480, 0.85508, 0.84533, 0.83568, 0.82621, 0.81692, 0.80764, 0.79811, 0.78856,\
        0.77894, 0.76915, 0.75913, 0.74886, 0.73836, 0.72766, 0.71674, 0.70553, 0.69394, 0.68181, 0.66953,\
        0.65750, 0.64586, 0.63468, 0.62405, 0.61402, 0.60466, 0.59594, 0.58779, 0.58021, 0.57310, 0.56638,\
        0.55992, 0.55359, 0.54729, 0.54088, 0.53425, 0.52730, 0.51996, 0.51218, 0.50402, 0.49559, 0.48709,\
        0.47873, 0.47074, 0.46331, 0.45661, 0.45076, 0.44579, 0.44171, 0.43846, 0.43596, 0.43409, 0.43273],\
        [0.70864, 0.70458, 0.70064, 0.69707, 0.69421, 0.69254, 0.69252, 0.69462, 0.69949, 0.70827, 0.71921,\
        0.73093, 0.74238, 0.75273, 0.76143, 0.76841, 0.77412, 0.77944, 0.78543, 0.79313, 0.80369, 0.81779,\
        0.83299, 0.84685, 0.85861, 0.86823, 0.87608, 0.88265, 0.88835, 0.89353, 0.89835, 0.90317, 0.90847,\
        0.91430, 0.92051, 0.92709, 0.93400, 0.94128, 0.94894, 0.95690, 0.96486, 0.97231, 0.97847, 0.98253,\
        0.98378, 0.98195, 0.97727, 0.97055, 0.96298, 0.95596, 0.95068, 0.94764, 0.94651, 0.94656, 0.94702,\
        0.94722, 0.94662, 0.94481, 0.94153, 0.93666, 0.93024, 0.92241, 0.91354, 0.90484, 0.89726, 0.88789,\
        0.87819, 0.86856, 0.85893, 0.84926, 0.83961, 0.83008, 0.82074, 0.81166, 0.80282, 0.79348, 0.78383,\
        0.77413, 0.76432, 0.75432, 0.74413, 0.73377, 0.72327, 0.71259, 0.70141, 0.68970, 0.67760, 0.66535,\
        0.65325, 0.64151, 0.63027, 0.61964, 0.60967, 0.60044, 0.59192, 0.58403, 0.57670, 0.56984, 0.56337,\
        0.55718, 0.55114, 0.54513, 0.53899, 0.53260, 0.52580, 0.51847, 0.51055, 0.50207, 0.49318, 0.48414,\
        0.47523, 0.46672, 0.45884, 0.45178, 0.44567, 0.44056, 0.43645, 0.43325, 0.43086, 0.42915, 0.42796],\
        [0.70799, 0.70369, 0.69952, 0.69573, 0.69273, 0.69093, 0.69077, 0.69268, 0.69730, 0.70490, 0.71463,\
        0.72521, 0.73562, 0.74517, 0.75351, 0.76063, 0.76690, 0.77293, 0.77946, 0.78723, 0.79695, 0.81042,\
        0.82465, 0.83777, 0.84918, 0.85881, 0.86686, 0.87368, 0.87960, 0.88490, 0.88979, 0.89461, 0.89984,\
        0.90559, 0.91180, 0.91847, 0.92555, 0.93300, 0.94077, 0.94863, 0.95616, 0.96275, 0.96775, 0.97046,\
        0.97040, 0.96742, 0.96188, 0.95461, 0.94681, 0.93993, 0.93543, 0.93382, 0.93426, 0.93567, 0.93720,\
        0.93824, 0.93829, 0.93698, 0.93410, 0.92960, 0.92352, 0.91602, 0.90728, 0.89798, 0.89015, 0.88155,\
        0.87215, 0.86250, 0.85280, 0.84311, 0.83351, 0.82404, 0.81475, 0.80569, 0.79693, 0.78800, 0.77865,\
        0.76908, 0.75931, 0.74934, 0.73916, 0.72882, 0.71837, 0.70787, 0.69692, 0.68505, 0.67292, 0.66069,\
        0.64860, 0.63685, 0.62560, 0.61498, 0.60510, 0.59602, 0.58770, 0.58003, 0.57291, 0.56628, 0.56005,\
        0.55410, 0.54833, 0.54260, 0.53674, 0.53060, 0.52397, 0.51668, 0.50860, 0.49975, 0.49035, 0.48072,\
        0.47122, 0.46218, 0.45385, 0.44646, 0.44015, 0.43498, 0.43093, 0.42790, 0.42572, 0.42424, 0.42330],\
        [0.70704, 0.70246, 0.69805, 0.69409, 0.69101, 0.68921, 0.68911, 0.69106, 0.69538, 0.70216, 0.71094,\
        0.72052, 0.73000, 0.73889, 0.74699, 0.75436, 0.76129, 0.76826, 0.77584, 0.78465, 0.79534, 0.80714,\
        0.81918, 0.83073, 0.84108, 0.85009, 0.85787, 0.86463, 0.87060, 0.87598, 0.88099, 0.88590, 0.89103,\
        0.89650, 0.90245, 0.90894, 0.91584, 0.92306, 0.93064, 0.93824, 0.94526, 0.95106, 0.95507, 0.95672,\
        0.95558, 0.95162, 0.94529, 0.93753, 0.92951, 0.92261, 0.91894, 0.91939, 0.92180, 0.92473, 0.92748,\
        0.92949, 0.93029, 0.92955, 0.92713, 0.92301, 0.91734, 0.91036, 0.90237, 0.89331, 0.88431, 0.87528,\
        0.86585, 0.85615, 0.84638, 0.83666, 0.82708, 0.81770, 0.80851, 0.79953, 0.79086, 0.78217, 0.77316,\
        0.76382, 0.75418, 0.74425, 0.73405, 0.72361, 0.71299, 0.70230, 0.69140, 0.67984, 0.66786, 0.65572,\
        0.64368, 0.63194, 0.62070, 0.61014, 0.60038, 0.59146, 0.58330, 0.57580, 0.56886, 0.56240, 0.55636,\
        0.55062, 0.54508, 0.53959, 0.53400, 0.52810, 0.52168, 0.51448, 0.50626, 0.49699, 0.48700, 0.47676,\
        0.46667, 0.45711, 0.44836, 0.44068, 0.43424, 0.42911, 0.42525, 0.42249, 0.42064, 0.41948, 0.41883],\
        [0.70574, 0.70076, 0.69606, 0.69193, 0.68879, 0.68705, 0.68709, 0.68919, 0.69345, 0.69980, 0.70793,\
        0.71664, 0.72527, 0.73356, 0.74147, 0.74908, 0.75667, 0.76459, 0.77324, 0.78291, 0.79347, 0.80431,\
        0.81505, 0.82495, 0.83390, 0.84194, 0.84915, 0.85564, 0.86154, 0.86697, 0.87208, 0.87706, 0.88212,\
        0.88739, 0.89301, 0.89910, 0.90550, 0.91218, 0.91946, 0.92683, 0.93337, 0.93852, 0.94186, 0.94284,\
        0.94103, 0.93639, 0.92948, 0.92146, 0.91371, 0.90708, 0.90382, 0.90697, 0.91083, 0.91488, 0.91867,\
        0.92155, 0.92301, 0.92276, 0.92068, 0.91681, 0.91137, 0.90463, 0.89679, 0.88793, 0.87858, 0.86908,\
        0.85938, 0.84954, 0.83967, 0.82991, 0.82035, 0.81111, 0.80218, 0.79342, 0.78489, 0.77637, 0.76761,\
        0.75851, 0.74903, 0.73917, 0.72896, 0.71845, 0.70767, 0.69673, 0.68585, 0.67447, 0.66266, 0.65062,\
        0.63862, 0.62691, 0.61571, 0.60519, 0.59557, 0.58680, 0.57876, 0.57136, 0.56453, 0.55818, 0.55225,\
        0.54666, 0.54129, 0.53600, 0.53061, 0.52494, 0.51872, 0.51163, 0.50334, 0.49359, 0.48299, 0.47216,\
        0.46155, 0.45152, 0.44241, 0.43451, 0.42802, 0.42304, 0.41950, 0.41716, 0.41574, 0.41498, 0.41467],\
        [0.70406, 0.69857, 0.69348, 0.68905, 0.68575, 0.68403, 0.68424, 0.68657, 0.69107, 0.69754, 0.70519,\
        0.71302, 0.72085, 0.72862, 0.73636, 0.74418, 0.75231, 0.76103, 0.77054, 0.78091, 0.79174, 0.80220,\
        0.81145, 0.81952, 0.82698, 0.83396, 0.84051, 0.84668, 0.85249, 0.85796, 0.86315, 0.86818, 0.87319,\
        0.87831, 0.88366, 0.88929, 0.89509, 0.90110, 0.90823, 0.91547, 0.92147, 0.92607, 0.92915, 0.93003,\
        0.92810, 0.92323, 0.91605, 0.90812, 0.90145, 0.89674, 0.89515, 0.89828, 0.90229, 0.90686, 0.91133,\
        0.91482, 0.91667, 0.91665, 0.91472, 0.91088, 0.90537, 0.89858, 0.89067, 0.88183, 0.87238, 0.86261,\
        0.85267, 0.84265, 0.83267, 0.82286, 0.81334, 0.80423, 0.79558, 0.78717, 0.77891, 0.77062, 0.76213,\
        0.75328, 0.74400, 0.73425, 0.72406, 0.71355, 0.70283, 0.69190, 0.68080, 0.66939, 0.65761, 0.64560,\
        0.63360, 0.62190, 0.61074, 0.60033, 0.59080, 0.58211, 0.57413, 0.56676, 0.55993, 0.55361, 0.54771,\
        0.54218, 0.53689, 0.53171, 0.52646, 0.52091, 0.51481, 0.50779, 0.49927, 0.48912, 0.47809, 0.46686,\
        0.45588, 0.44551, 0.43612, 0.42806, 0.42160, 0.41691, 0.41386, 0.41210, 0.41121, 0.41090, 0.41094],\
        [0.70217, 0.69595, 0.69010, 0.68513, 0.68154, 0.67976, 0.68008, 0.68263, 0.68738, 0.69400, 0.70149,\
        0.70873, 0.71603, 0.72345, 0.73105, 0.73902, 0.74754, 0.75680, 0.76688, 0.77770, 0.78897, 0.79943,\
        0.80672, 0.81338, 0.81966, 0.82575, 0.83176, 0.83769, 0.84345, 0.84898, 0.85426, 0.85933, 0.86430,\
        0.86929, 0.87442, 0.87973, 0.88525, 0.89118, 0.89802, 0.90497, 0.91004, 0.91422, 0.91761, 0.91909,\
        0.91770, 0.91323, 0.90634, 0.89896, 0.89372, 0.89064, 0.89022, 0.89258, 0.89640, 0.90099, 0.90576,\
        0.90948, 0.91127, 0.91109, 0.90907, 0.90508, 0.89927, 0.89222, 0.88416, 0.87522, 0.86566, 0.85573,\
        0.84559, 0.83540, 0.82534, 0.81553, 0.80609, 0.79715, 0.78881, 0.78076, 0.77285, 0.76489, 0.75673,\
        0.74821, 0.73919, 0.72961, 0.71946, 0.70894, 0.69827, 0.68740, 0.67624, 0.66473, 0.65289, 0.64083,\
        0.62877, 0.61699, 0.60587, 0.59560, 0.58614, 0.57746, 0.56944, 0.56200, 0.55510, 0.54868, 0.54272,\
        0.53715, 0.53185, 0.52668, 0.52143, 0.51586, 0.50970, 0.50256, 0.49378, 0.48342, 0.47228, 0.46092,\
        0.44978, 0.43925, 0.42971, 0.42155, 0.41521, 0.41095, 0.40861, 0.40758, 0.40729, 0.40741, 0.40777],\
        [0.69931, 0.69214, 0.68533, 0.67973, 0.67580, 0.67391, 0.67427, 0.67694, 0.68175, 0.68826, 0.69561,\
        0.70297, 0.71020, 0.71752, 0.72506, 0.73311, 0.74189, 0.75145, 0.76171, 0.77242, 0.78301, 0.79228,\
        0.79962, 0.80598, 0.81153, 0.81704, 0.82276, 0.82861, 0.83445, 0.84010, 0.84547, 0.85056, 0.85546,\
        0.86033, 0.86530, 0.87048, 0.87598, 0.88196, 0.88847, 0.89464, 0.89912, 0.90328, 0.90770, 0.91030,\
        0.91003, 0.90664, 0.90080, 0.89457, 0.88987, 0.88730, 0.88710, 0.88912, 0.89273, 0.89720, 0.90191,\
        0.90544, 0.90644, 0.90555, 0.90325, 0.89908, 0.89286, 0.88548, 0.87718, 0.86810, 0.85842, 0.84835,\
        0.83806, 0.82775, 0.81762, 0.80788, 0.79864, 0.78999, 0.78191, 0.77422, 0.76669, 0.75915, 0.75140,\
        0.74329, 0.73466, 0.72537, 0.71537, 0.70471, 0.69411, 0.68333, 0.67212, 0.66051, 0.64858, 0.63644,\
        0.62427, 0.61233, 0.60112, 0.59097, 0.58161, 0.57289, 0.56475, 0.55714, 0.55004, 0.54343, 0.53729,\
        0.53157, 0.52617, 0.52091, 0.51555, 0.50980, 0.50337, 0.49585, 0.48691, 0.47670, 0.46574, 0.45453,\
        0.44350, 0.43303, 0.42352, 0.41538, 0.40917, 0.40551, 0.40417, 0.40396, 0.40421, 0.40468, 0.40526],\
        [0.69275, 0.68559, 0.67846, 0.67249, 0.66831, 0.66632, 0.66670, 0.66942, 0.67424, 0.68065, 0.68798,\
        0.69557, 0.70314, 0.71056, 0.71807, 0.72620, 0.73518, 0.74490, 0.75509, 0.76539, 0.77523, 0.78394,\
        0.79127, 0.79743, 0.80255, 0.80788, 0.81356, 0.81951, 0.82554, 0.83137, 0.83682, 0.84188, 0.84667,\
        0.85138, 0.85621, 0.86131, 0.86677, 0.87265, 0.87875, 0.88447, 0.88942, 0.89428, 0.89940, 0.90306,\
        0.90421, 0.90259, 0.89778, 0.89220, 0.88765, 0.88507, 0.88472, 0.88647, 0.88989, 0.89440, 0.89921,\
        0.90196, 0.90140, 0.89940, 0.89648, 0.89193, 0.88561, 0.87805, 0.86958, 0.86038, 0.85062, 0.84046,\
        0.83006, 0.81963, 0.80945, 0.79979, 0.79083, 0.78250, 0.77476, 0.76746, 0.76037, 0.75330, 0.74605,\
        0.73845, 0.73033, 0.72153, 0.71198, 0.70164, 0.69086, 0.67983, 0.66840, 0.65663, 0.64461, 0.63245,\
        0.62026, 0.60825, 0.59692, 0.58677, 0.57733, 0.56845, 0.56009, 0.55221, 0.54482, 0.53790, 0.53146,\
        0.52549, 0.51990, 0.51448, 0.50894, 0.50289, 0.49610, 0.48830, 0.47936, 0.46941, 0.45882, 0.44799,\
        0.43732, 0.42720, 0.41800, 0.41013, 0.40415, 0.40124, 0.40121, 0.40162, 0.40216, 0.40278, 0.40344],\
        [0.68399, 0.67675, 0.66957, 0.66344, 0.65910, 0.65706, 0.65750, 0.66034, 0.66526, 0.67171, 0.67911,\
        0.68694, 0.69485, 0.70245, 0.71001, 0.71827, 0.72752, 0.73738, 0.74743, 0.75732, 0.76663, 0.77496,\
        0.78213, 0.78818, 0.79345, 0.79876, 0.80443, 0.81055, 0.81684, 0.82289, 0.82838, 0.83329, 0.83784,\
        0.84234, 0.84700, 0.85199, 0.85739, 0.86316, 0.86910, 0.87490, 0.88043, 0.88592, 0.89127, 0.89558,\
        0.89809, 0.89895, 0.89471, 0.88933, 0.88495, 0.88236, 0.88182, 0.88324, 0.88628, 0.89039, 0.89456,\
        0.89629, 0.89505, 0.89243, 0.88877, 0.88379, 0.87738, 0.86978, 0.86124, 0.85199, 0.84220, 0.83200,\
        0.82153, 0.81100, 0.80079, 0.79125, 0.78251, 0.77453, 0.76722, 0.76039, 0.75380, 0.74725, 0.74055,\
        0.73352, 0.72600, 0.71778, 0.70875, 0.69900, 0.68799, 0.67655, 0.66482, 0.65287, 0.64080, 0.62872,\
        0.61670, 0.60485, 0.59355, 0.58309, 0.57333, 0.56415, 0.55547, 0.54726, 0.53949, 0.53216, 0.52532,\
        0.51899, 0.51313, 0.50750, 0.50175, 0.49543, 0.48834, 0.48041, 0.47159, 0.46199, 0.45187, 0.44159,\
        0.43151, 0.42202, 0.41351, 0.40644, 0.40144, 0.39989, 0.40023, 0.40061, 0.40109, 0.40166, 0.40226],\
        [0.67426, 0.66669, 0.65932, 0.65299, 0.64850, 0.64644, 0.64702, 0.65012, 0.65529, 0.66195, 0.66953,\
        0.67752, 0.68556, 0.69338, 0.70109, 0.70963, 0.71937, 0.72936, 0.73923, 0.74876, 0.75769, 0.76576,\
        0.77285, 0.77901, 0.78458, 0.79004, 0.79573, 0.80200, 0.80847, 0.81473, 0.82014, 0.82469, 0.82887,\
        0.83307, 0.83752, 0.84238, 0.84771, 0.85346, 0.85943, 0.86541, 0.87127, 0.87697, 0.88230, 0.88673,\
        0.88973, 0.89056, 0.88815, 0.88407, 0.88037, 0.87802, 0.87738, 0.87844, 0.88087, 0.88404, 0.88686,\
        0.88792, 0.88679, 0.88406, 0.88006, 0.87482, 0.86831, 0.86068, 0.85213, 0.84288, 0.83309, 0.82291,\
        0.81245, 0.80182, 0.79157, 0.78211, 0.77360, 0.76601, 0.75919, 0.75292, 0.74690, 0.74088, 0.73470,\
        0.72824, 0.72139, 0.71385, 0.70503, 0.69522, 0.68449, 0.67293, 0.66098, 0.64890, 0.63682, 0.62489,\
        0.61320, 0.60166, 0.59030, 0.57952, 0.56941, 0.55989, 0.55088, 0.54231, 0.53413, 0.52634, 0.51899,\
        0.51221, 0.50604, 0.50016, 0.49412, 0.48761, 0.48042, 0.47251, 0.46391, 0.45472, 0.44516, 0.43555,\
        0.42622, 0.41758, 0.41004, 0.40414, 0.40058, 0.39987, 0.40018, 0.40039, 0.40071, 0.40113, 0.40160],\
        [0.66428, 0.65622, 0.64847, 0.64180, 0.63705, 0.63495, 0.63580, 0.63934, 0.64495, 0.65195, 0.65976,\
        0.66795, 0.67617, 0.68427, 0.69237, 0.70144, 0.71145, 0.72130, 0.73083, 0.74002, 0.74869, 0.75661,\
        0.76367, 0.76998, 0.77582, 0.78154, 0.78748, 0.79380, 0.80033, 0.80693, 0.81196, 0.81582, 0.81954,\
        0.82344, 0.82767, 0.83238, 0.83767, 0.84347, 0.84954, 0.85564, 0.86158, 0.86720, 0.87227, 0.87643,\
        0.87922, 0.88009, 0.87880, 0.87615, 0.87341, 0.87151, 0.87090, 0.87164, 0.87343, 0.87569, 0.87753,\
        0.87810, 0.87701, 0.87435, 0.87033, 0.86501, 0.85846, 0.85082, 0.84226, 0.83300, 0.82321, 0.81305,\
        0.80260, 0.79192, 0.78160, 0.77222, 0.76396, 0.75684, 0.75063, 0.74501, 0.73960, 0.73411, 0.72836,\
        0.72231, 0.71604, 0.70947, 0.70159, 0.69148, 0.68035, 0.66860, 0.65653, 0.64438, 0.63232, 0.62051,\
        0.60910, 0.59793, 0.58655, 0.57563, 0.56530, 0.55554, 0.54626, 0.53738, 0.52885, 0.52062, 0.51272,\
        0.50538, 0.49889, 0.49277, 0.48650, 0.47984, 0.47263, 0.46485, 0.45653, 0.44780, 0.43886, 0.42998,\
        0.42150, 0.41379, 0.40730, 0.40252, 0.39998, 0.39976, 0.40018, 0.40041, 0.40066, 0.40096, 0.40130],\
        [0.65463, 0.64606, 0.63782, 0.63065, 0.62545, 0.62324, 0.62450, 0.62870, 0.63490, 0.64229, 0.65036,\
        0.65875, 0.66725, 0.67580, 0.68461, 0.69411, 0.70398, 0.71329, 0.72236, 0.73129, 0.73982, 0.74765,\
        0.75467, 0.76104, 0.76702, 0.77294, 0.77906, 0.78552, 0.79232, 0.79908, 0.80309, 0.80626, 0.80970,\
        0.81342, 0.81740, 0.82194, 0.82725, 0.83316, 0.83936, 0.84550, 0.85133, 0.85665, 0.86127, 0.86497,\
        0.86744, 0.86839, 0.86780, 0.86616, 0.86425, 0.86283, 0.86232, 0.86285, 0.86417, 0.86578, 0.86701,\
        0.86726, 0.86616, 0.86363, 0.85972, 0.85446, 0.84794, 0.84028, 0.83168, 0.82238, 0.81254, 0.80228,\
        0.79167, 0.78096, 0.77070, 0.76144, 0.75352, 0.74698, 0.74154, 0.73669, 0.73194, 0.72691, 0.72143,\
        0.71550, 0.70933, 0.70320, 0.69644, 0.68632, 0.67502, 0.66323, 0.65120, 0.63910, 0.62708, 0.61531,\
        0.60390, 0.59285, 0.58188, 0.57115, 0.56084, 0.55099, 0.54156, 0.53249, 0.52372, 0.51521, 0.50693,\
        0.49902, 0.49220, 0.48574, 0.47922, 0.47241, 0.46521, 0.45759, 0.44961, 0.44136, 0.43306, 0.42494,\
        0.41731, 0.41053, 0.40501, 0.40119, 0.39946, 0.39946, 0.40003, 0.40045, 0.40073, 0.40097, 0.40122],\
        [0.64582, 0.63683, 0.62817, 0.62047, 0.61465, 0.61217, 0.61406, 0.61913, 0.62587, 0.63353, 0.64172,\
        0.65023, 0.65893, 0.66782, 0.67701, 0.68657, 0.69613, 0.70500, 0.71378, 0.72271, 0.73124, 0.73896,\
        0.74583, 0.75208, 0.75801, 0.76394, 0.77006, 0.77644, 0.78293, 0.78877, 0.79283, 0.79601, 0.79945,\
        0.80307, 0.80680, 0.81112, 0.81653, 0.82262, 0.82892, 0.83504, 0.84063, 0.84548, 0.84952, 0.85268,\
        0.85483, 0.85582, 0.85566, 0.85465, 0.85332, 0.85225, 0.85186, 0.85230, 0.85337, 0.85463, 0.85554,\
        0.85561, 0.85454, 0.85217, 0.84844, 0.84332, 0.83686, 0.82918, 0.82052, 0.81112, 0.80116, 0.79074,\
        0.77997, 0.76923, 0.75899, 0.74985, 0.74230, 0.73648, 0.73198, 0.72805, 0.72399, 0.71937, 0.71399,\
        0.70791, 0.70139, 0.69494, 0.68779, 0.67864, 0.66797, 0.65660, 0.64487, 0.63300, 0.62112, 0.60942,\
        0.59812, 0.58725, 0.57659, 0.56613, 0.55598, 0.54618, 0.53673, 0.52758, 0.51871, 0.51010, 0.50175,\
        0.49370, 0.48625, 0.47929, 0.47244, 0.46548, 0.45829, 0.45085, 0.44321, 0.43547, 0.42779, 0.42042,\
        0.41362, 0.40772, 0.40308, 0.40009, 0.39896, 0.39921, 0.39996, 0.40051, 0.40083, 0.40106, 0.40125],\
        [0.63818, 0.62902, 0.62023, 0.61235, 0.60608, 0.60301, 0.60596, 0.61169, 0.61851, 0.62602, 0.63404,\
        0.64245, 0.65117, 0.66016, 0.66939, 0.67876, 0.68792, 0.69661, 0.70535, 0.71460, 0.72317, 0.73062,\
        0.73710, 0.74299, 0.74866, 0.75438, 0.76030, 0.76636, 0.77230, 0.77758, 0.78181, 0.78537, 0.78886,\
        0.79241, 0.79606, 0.80023, 0.80583, 0.81200, 0.81834, 0.82438, 0.82961, 0.83384, 0.83717, 0.83976,\
        0.84161, 0.84262, 0.84273, 0.84209, 0.84108, 0.84018, 0.83984, 0.84030, 0.84136, 0.84252, 0.84329,\
        0.84331, 0.84232, 0.84014, 0.83665, 0.83175, 0.82539, 0.81769, 0.80893, 0.79943, 0.78935, 0.77878,\
        0.76794, 0.75714, 0.74687, 0.73773, 0.73044, 0.72548, 0.72218, 0.71925, 0.71584, 0.71151, 0.70608,\
        0.69980, 0.69298, 0.68596, 0.67834, 0.66950, 0.65952, 0.64879, 0.63761, 0.62615, 0.61457, 0.60311,\
        0.59203, 0.58136, 0.57093, 0.56072, 0.55077, 0.54110, 0.53173, 0.52260, 0.51371, 0.50506, 0.49662,\
        0.48843, 0.48067, 0.47331, 0.46617, 0.45907, 0.45191, 0.44466, 0.43736, 0.43012, 0.42307, 0.41641,\
        0.41040, 0.40531, 0.40147, 0.39920, 0.39854, 0.39898, 0.39985, 0.40050, 0.40089, 0.40115, 0.40131],\
        [0.63189, 0.62283, 0.61433, 0.60697, 0.60154, 0.59935, 0.60212, 0.60694, 0.61293, 0.61977, 0.62728,\
        0.63536, 0.64389, 0.65279, 0.66193, 0.67116, 0.68025, 0.68910, 0.69811, 0.70749, 0.71575, 0.72255,\
        0.72834, 0.73363, 0.73886, 0.74426, 0.74988, 0.75559, 0.76111, 0.76612, 0.77047, 0.77433, 0.77800,\
        0.78168, 0.78553, 0.78999, 0.79540, 0.80138, 0.80767, 0.81360, 0.81836, 0.82179, 0.82430, 0.82632,\
        0.82795, 0.82903, 0.82933, 0.82887, 0.82796, 0.82703, 0.82664, 0.82722, 0.82850, 0.82973, 0.83048,\
        0.83051, 0.82962, 0.82765, 0.82447, 0.81987, 0.81368, 0.80596, 0.79714, 0.78760, 0.77744, 0.76681,\
        0.75595, 0.74519, 0.73491, 0.72562, 0.71829, 0.71432, 0.71251, 0.71043, 0.70739, 0.70318, 0.69754,\
        0.69093, 0.68380, 0.67631, 0.66831, 0.65958, 0.65011, 0.64007, 0.62958, 0.61871, 0.60757, 0.59647,\
        0.58569, 0.57520, 0.56498, 0.55498, 0.54524, 0.53575, 0.52651, 0.51748, 0.50861, 0.49990, 0.49140,\
        0.48314, 0.47522, 0.46763, 0.46030, 0.45313, 0.44603, 0.43900, 0.43206, 0.42530, 0.41885, 0.41289,\
        0.40761, 0.40327, 0.40016, 0.39851, 0.39816, 0.39870, 0.39961, 0.40039, 0.40089, 0.40119, 0.40137],\
        [0.62688, 0.61811, 0.61016, 0.60364, 0.59943, 0.59846, 0.60030, 0.60377, 0.60853, 0.61436, 0.62116,\
        0.62875, 0.63697, 0.64566, 0.65467, 0.66387, 0.67310, 0.68237, 0.69182, 0.70110, 0.70854, 0.71442,\
        0.71931, 0.72387, 0.72858, 0.73365, 0.73901, 0.74444, 0.74970, 0.75458, 0.75902, 0.76310, 0.76702,\
        0.77095, 0.77512, 0.77976, 0.78502, 0.79077, 0.79686, 0.80252, 0.80675, 0.80929, 0.81093, 0.81246,\
        0.81400, 0.81525, 0.81573, 0.81532, 0.81440, 0.81334, 0.81265, 0.81348, 0.81524, 0.81656, 0.81724,\
        0.81729, 0.81651, 0.81474, 0.81186, 0.80766, 0.80170, 0.79409, 0.78533, 0.77579, 0.76564, 0.75505,\
        0.74431, 0.73375, 0.72374, 0.71461, 0.70681, 0.70390, 0.70350, 0.70151, 0.69830, 0.69382, 0.68800,\
        0.68119, 0.67378, 0.66595, 0.65771, 0.64904, 0.63998, 0.63062, 0.62093, 0.61078, 0.60020, 0.58951,\
        0.57900, 0.56873, 0.55871, 0.54892, 0.53938, 0.53009, 0.52105, 0.51218, 0.50338, 0.49464, 0.48610,\
        0.47781, 0.46982, 0.46214, 0.45475, 0.44758, 0.44061, 0.43381, 0.42724, 0.42097, 0.41511, 0.40980,\
        0.40522, 0.40158, 0.39912, 0.39794, 0.39782, 0.39843, 0.39938, 0.40024, 0.40083, 0.40120, 0.40140],\
        [0.62289, 0.61445, 0.60704, 0.60120, 0.59755, 0.59652, 0.59777, 0.60059, 0.60430, 0.60919, 0.61528,\
        0.62237, 0.63023, 0.63866, 0.64753, 0.65669, 0.66599, 0.67538, 0.68472, 0.69343, 0.70048, 0.70571,\
        0.70978, 0.71362, 0.71789, 0.72273, 0.72794, 0.73319, 0.73828, 0.74308, 0.74757, 0.75182, 0.75597,\
        0.76016, 0.76455, 0.76926, 0.77437, 0.77980, 0.78533, 0.79038, 0.79409, 0.79616, 0.79730, 0.79855,\
        0.80004, 0.80145, 0.80205, 0.80163, 0.80074, 0.79969, 0.79869, 0.79980, 0.80203, 0.80311, 0.80369,\
        0.80376, 0.80308, 0.80145, 0.79877, 0.79485, 0.78917, 0.78188, 0.77340, 0.76404, 0.75404, 0.74363,\
        0.73313, 0.72296, 0.71361, 0.70548, 0.69902, 0.69571, 0.69448, 0.69196, 0.68834, 0.68354, 0.67758,\
        0.67066, 0.66307, 0.65501, 0.64660, 0.63797, 0.62923, 0.62051, 0.61167, 0.60233, 0.59230, 0.58206,\
        0.57189, 0.56189, 0.55209, 0.54252, 0.53318, 0.52410, 0.51525, 0.50658, 0.49793, 0.48922, 0.48069,\
        0.47242, 0.46445, 0.45680, 0.44944, 0.44237, 0.43557, 0.42905, 0.42286, 0.41707, 0.41177, 0.40709,\
        0.40318, 0.40019, 0.39828, 0.39745, 0.39750, 0.39817, 0.39913, 0.40006, 0.40074, 0.40117, 0.40140],\
        [0.61966, 0.61140, 0.60431, 0.59879, 0.59519, 0.59378, 0.59435, 0.59641, 0.59950, 0.60375, 0.60934,\
        0.61601, 0.62350, 0.63166, 0.64035, 0.64939, 0.65859, 0.66776, 0.67660, 0.68461, 0.69119, 0.69614,\
        0.69973, 0.70312, 0.70704, 0.71182, 0.71697, 0.72207, 0.72700, 0.73170, 0.73619, 0.74055, 0.74487,\
        0.74925, 0.75377, 0.75845, 0.76330, 0.76824, 0.77304, 0.77729, 0.78050, 0.78253, 0.78385, 0.78507,\
        0.78639, 0.78760, 0.78814, 0.78784, 0.78711, 0.78628, 0.78583, 0.78670, 0.78833, 0.78939, 0.78994,\
        0.79001, 0.78940, 0.78790, 0.78536, 0.78156, 0.77623, 0.76942, 0.76138, 0.75238, 0.74267, 0.73256,\
        0.72240, 0.71265, 0.70403, 0.69692, 0.69133, 0.68758, 0.68492, 0.68185, 0.67786, 0.67281, 0.66670,\
        0.65966, 0.65193, 0.64370, 0.63517, 0.62652, 0.61798, 0.60975, 0.60172, 0.59306, 0.58370, 0.57404,\
        0.56431, 0.55465, 0.54513, 0.53579, 0.52665, 0.51775, 0.50906, 0.50055, 0.49203, 0.48350, 0.47512,\
        0.46697, 0.45911, 0.45157, 0.44434, 0.43744, 0.43087, 0.42466, 0.41886, 0.41354, 0.40878, 0.40469,\
        0.40139, 0.39898, 0.39754, 0.39699, 0.39720, 0.39792, 0.39891, 0.39989, 0.40062, 0.40110, 0.40136],\
        [0.61698, 0.60863, 0.60153, 0.59599, 0.59221, 0.59031, 0.59017, 0.59152, 0.59412, 0.59797, 0.60323,\
        0.60953, 0.61664, 0.62451, 0.63301, 0.64188, 0.65086, 0.65966, 0.66795, 0.67532, 0.68141, 0.68612,\
        0.68974, 0.69303, 0.69673, 0.70141, 0.70639, 0.71124, 0.71594, 0.72051, 0.72495, 0.72932, 0.73374,\
        0.73826, 0.74282, 0.74736, 0.75185, 0.75623, 0.76033, 0.76391, 0.76674, 0.76879, 0.77033, 0.77164,\
        0.77284, 0.77379, 0.77423, 0.77408, 0.77358, 0.77306, 0.77294, 0.77354, 0.77460, 0.77551, 0.77605,\
        0.77612, 0.77558, 0.77421, 0.77183, 0.76822, 0.76324, 0.75691, 0.74937, 0.74084, 0.73154, 0.72181,\
        0.71208, 0.70281, 0.69467, 0.68812, 0.68285, 0.67873, 0.67525, 0.67160, 0.66724, 0.66195, 0.65567,\
        0.64851, 0.64064, 0.63230, 0.62369, 0.61503, 0.60661, 0.59870, 0.59117, 0.58318, 0.57457, 0.56556,\
        0.55635, 0.54709, 0.53787, 0.52877, 0.51983, 0.51109, 0.50256, 0.49418, 0.48585, 0.47757, 0.46941,\
        0.46147, 0.45380, 0.44645, 0.43942, 0.43275, 0.42645, 0.42058, 0.41518, 0.41032, 0.40607, 0.40254,\
        0.39979, 0.39790, 0.39684, 0.39655, 0.39689, 0.39767, 0.39868, 0.39969, 0.40048, 0.40100, 0.40129],\
        [0.61477, 0.60611, 0.59861, 0.59286, 0.58870, 0.58623, 0.58543, 0.58618, 0.58836, 0.59193, 0.59685,\
        0.60277, 0.60953, 0.61714, 0.62548, 0.63419, 0.64293, 0.65135, 0.65913, 0.66598, 0.67170, 0.67630,\
        0.68006, 0.68352, 0.68726, 0.69157, 0.69618, 0.70071, 0.70515, 0.70958, 0.71389, 0.71816, 0.72263,\
        0.72728, 0.73183, 0.73614, 0.74020, 0.74400, 0.74747, 0.75049, 0.75301, 0.75506, 0.75676, 0.75820,\
        0.75937, 0.76018, 0.76054, 0.76047, 0.76014, 0.75983, 0.75981, 0.76022, 0.76092, 0.76162, 0.76209,\
        0.76218, 0.76172, 0.76052, 0.75834, 0.75498, 0.75034, 0.74445, 0.73744, 0.72944, 0.72060, 0.71126,\
        0.70200, 0.69330, 0.68558, 0.67915, 0.67387, 0.66945, 0.66547, 0.66137, 0.65671, 0.65119, 0.64475,\
        0.63746, 0.62949, 0.62109, 0.61252, 0.60398, 0.59578, 0.58823, 0.58089, 0.57332, 0.56530, 0.55687,\
        0.54817, 0.53931, 0.53041, 0.52155, 0.51281, 0.50426, 0.49589, 0.48768, 0.47957, 0.47154, 0.46365,\
        0.45597, 0.44855, 0.44144, 0.43468, 0.42828, 0.42230, 0.41678, 0.41178, 0.40737, 0.40362, 0.40060,\
        0.39836, 0.39690, 0.39618, 0.39611, 0.39657, 0.39741, 0.39843, 0.39946, 0.40029, 0.40086, 0.40119],\
        [0.61307, 0.60407, 0.59610, 0.58966, 0.58482, 0.58164, 0.58017, 0.58037, 0.58215, 0.58540, 0.58997,\
        0.59561, 0.60209, 0.60954, 0.61784, 0.62644, 0.63495, 0.64302, 0.65038, 0.65681, 0.66223, 0.66674,\
        0.67060, 0.67421, 0.67793, 0.68196, 0.68619, 0.69044, 0.69468, 0.69894, 0.70306, 0.70712, 0.71161,\
        0.71645, 0.72094, 0.72492, 0.72850, 0.73175, 0.73463, 0.73716, 0.73938, 0.74141, 0.74327, 0.74487,\
        0.74608, 0.74682, 0.74711, 0.74704, 0.74679, 0.74655, 0.74652, 0.74679, 0.74726, 0.74777, 0.74814,\
        0.74824, 0.74790, 0.74690, 0.74499, 0.74190, 0.73755, 0.73205, 0.72555, 0.71810, 0.70977, 0.70082,\
        0.69191, 0.68363, 0.67621, 0.66985, 0.66448, 0.65985, 0.65558, 0.65122, 0.64636, 0.64069, 0.63411,\
        0.62670, 0.61865, 0.61026, 0.60186, 0.59366, 0.58579, 0.57835, 0.57112, 0.56379, 0.55616, 0.54821,\
        0.53995, 0.53147, 0.52285, 0.51423, 0.50571, 0.49736, 0.48920, 0.48118, 0.47329, 0.46552, 0.45792,\
        0.45052, 0.44340, 0.43658, 0.43012, 0.42404, 0.41841, 0.41325, 0.40864, 0.40467, 0.40140, 0.39888,\
        0.39709, 0.39600, 0.39556, 0.39569, 0.39626, 0.39714, 0.39818, 0.39921, 0.40008, 0.40069, 0.40106],\
        [0.61174, 0.60228, 0.59363, 0.58629, 0.58053, 0.57651, 0.57434, 0.57401, 0.57542, 0.57840, 0.58274,\
        0.58818, 0.59455, 0.60201, 0.61025, 0.61872, 0.62703, 0.63480, 0.64178, 0.64782, 0.65296, 0.65735,\
        0.66122, 0.66487, 0.66855, 0.67238, 0.67636, 0.68039, 0.68445, 0.68850, 0.69246, 0.69638, 0.70094,\
        0.70600, 0.71022, 0.71372, 0.71683, 0.71960, 0.72198, 0.72402, 0.72593, 0.72793, 0.72999, 0.73180,\
        0.73308, 0.73375, 0.73393, 0.73380, 0.73352, 0.73326, 0.73318, 0.73333, 0.73365, 0.73400, 0.73428,\
        0.73438, 0.73416, 0.73343, 0.73186, 0.72904, 0.72486, 0.71960, 0.71354, 0.70661, 0.69872, 0.69016,\
        0.68160, 0.67357, 0.66634, 0.66004, 0.65463, 0.64991, 0.64554, 0.64112, 0.63621, 0.63049, 0.62382,\
        0.61633, 0.60827, 0.59997, 0.59181, 0.58389, 0.57626, 0.56893, 0.56178, 0.55461, 0.54727, 0.53969,\
        0.53182, 0.52368, 0.51533, 0.50691, 0.49862, 0.49050, 0.48255, 0.47474, 0.46708, 0.45958, 0.45226,\
        0.44517, 0.43836, 0.43188, 0.42575, 0.42003, 0.41476, 0.40999, 0.40577, 0.40222, 0.39942, 0.39735,\
        0.39596, 0.39520, 0.39500, 0.39529, 0.39596, 0.39689, 0.39794, 0.39898, 0.39987, 0.40052, 0.40092],\
        [0.61058, 0.60042, 0.59089, 0.58248, 0.57563, 0.57067, 0.56780, 0.56701, 0.56814, 0.57098, 0.57525,\
        0.58070, 0.58719, 0.59463, 0.60274, 0.61107, 0.61920, 0.62672, 0.63332, 0.63896, 0.64376, 0.64796,\
        0.65176, 0.65539, 0.65902, 0.66273, 0.66654, 0.67042, 0.67434, 0.67826, 0.68218, 0.68626, 0.69080,\
        0.69555, 0.69937, 0.70249, 0.70524, 0.70766, 0.70965, 0.71122, 0.71276, 0.71472, 0.71708, 0.71917,\
        0.72047, 0.72096, 0.72098, 0.72073, 0.72038, 0.72006, 0.71990, 0.71997, 0.72019, 0.72042, 0.72059,\
        0.72066, 0.72055, 0.72012, 0.71900, 0.71648, 0.71225, 0.70699, 0.70120, 0.69460, 0.68704, 0.67891,\
        0.67073, 0.66295, 0.65587, 0.64964, 0.64426, 0.63956, 0.63527, 0.63096, 0.62619, 0.62055, 0.61385,\
        0.60631, 0.59833, 0.59023, 0.58226, 0.57452, 0.56705, 0.55981, 0.55274, 0.54571, 0.53860, 0.53134,\
        0.52384, 0.51603, 0.50794, 0.49972, 0.49164, 0.48373, 0.47598, 0.46839, 0.46097, 0.45374, 0.44673,\
        0.43996, 0.43349, 0.42735, 0.42160, 0.41626, 0.41139, 0.40703, 0.40320, 0.40004, 0.39769, 0.39603,\
        0.39498, 0.39450, 0.39451, 0.39495, 0.39571, 0.39668, 0.39774, 0.39878, 0.39968, 0.40035, 0.40077],\
        [0.60955, 0.59839, 0.58774, 0.57799, 0.56985, 0.56390, 0.56039, 0.55928, 0.56030, 0.56315, 0.56751,\
        0.57309, 0.57970, 0.58715, 0.59517, 0.60339, 0.61141, 0.61871, 0.62489, 0.63003, 0.63444, 0.63839,\
        0.64208, 0.64567, 0.64926, 0.65291, 0.65663, 0.66041, 0.66423, 0.66809, 0.67202, 0.67611, 0.68041,\
        0.68462, 0.68824, 0.69125, 0.69382, 0.69603, 0.69775, 0.69900, 0.70019, 0.70209, 0.70471, 0.70705,\
        0.70824, 0.70838, 0.70821, 0.70787, 0.70744, 0.70703, 0.70681, 0.70686, 0.70703, 0.70718, 0.70723,\
        0.70724, 0.70715, 0.70693, 0.70634, 0.70420, 0.69977, 0.69437, 0.68860, 0.68211, 0.67486, 0.66712,\
        0.65929, 0.65174, 0.64479, 0.63864, 0.63332, 0.62874, 0.62463, 0.62060, 0.61614, 0.61073, 0.60404,\
        0.59650, 0.58865, 0.58074, 0.57296, 0.56538, 0.55804, 0.55090, 0.54391, 0.53700, 0.53009, 0.52312,\
        0.51599, 0.50852, 0.50068, 0.49269, 0.48479, 0.47705, 0.46951, 0.46215, 0.45499, 0.44805, 0.44135,\
        0.43492, 0.42880, 0.42303, 0.41766, 0.41273, 0.40830, 0.40438, 0.40101, 0.39827, 0.39627, 0.39493,\
        0.39417, 0.39391, 0.39411, 0.39466, 0.39550, 0.39651, 0.39758, 0.39862, 0.39953, 0.40021, 0.40065],\
        [0.60883, 0.59654, 0.58411, 0.57247, 0.56282, 0.55591, 0.55196, 0.55077, 0.55192, 0.55497, 0.55952,\
        0.56526, 0.57197, 0.57942, 0.58738, 0.59550, 0.60347, 0.61065, 0.61624, 0.62076, 0.62475, 0.62846,\
        0.63206, 0.63561, 0.63919, 0.64282, 0.64651, 0.65024, 0.65400, 0.65781, 0.66169, 0.66567, 0.66971,\
        0.67359, 0.67707, 0.68005, 0.68257, 0.68466, 0.68630, 0.68756, 0.68880, 0.69058, 0.69289, 0.69504,\
        0.69599, 0.69589, 0.69561, 0.69520, 0.69475, 0.69434, 0.69411, 0.69415, 0.69429, 0.69440, 0.69441,\
        0.69438, 0.69427, 0.69404, 0.69347, 0.69142, 0.68720, 0.68188, 0.67596, 0.66946, 0.66241, 0.65497,\
        0.64742, 0.64004, 0.63317, 0.62708, 0.62185, 0.61740, 0.61352, 0.60979, 0.60566, 0.60056, 0.59400,\
        0.58659, 0.57894, 0.57127, 0.56371, 0.55633, 0.54914, 0.54211, 0.53521, 0.52840, 0.52164, 0.51491,\
        0.50813, 0.50102, 0.49340, 0.48564, 0.47795, 0.47044, 0.46312, 0.45602, 0.44915, 0.44251, 0.43614,\
        0.43006, 0.42431, 0.41892, 0.41394, 0.40943, 0.40543, 0.40201, 0.39911, 0.39678, 0.39510, 0.39403,\
        0.39350, 0.39344, 0.39378, 0.39445, 0.39535, 0.39639, 0.39748, 0.39852, 0.39943, 0.40012, 0.40055],\
        [0.60761, 0.59423, 0.57907, 0.56518, 0.55404, 0.54642, 0.54241, 0.54153, 0.54311, 0.54654, 0.55135,\
        0.55725, 0.56398, 0.57137, 0.57918, 0.58720, 0.59516, 0.60199, 0.60686, 0.61082, 0.61447, 0.61804,\
        0.62159, 0.62515, 0.62875, 0.63240, 0.63609, 0.63980, 0.64351, 0.64726, 0.65105, 0.65491, 0.65876,\
        0.66247, 0.66587, 0.66887, 0.67144, 0.67356, 0.67526, 0.67665, 0.67799, 0.67955, 0.68129, 0.68277,\
        0.68347, 0.68343, 0.68310, 0.68269, 0.68228, 0.68195, 0.68180, 0.68184, 0.68197, 0.68209, 0.68213,\
        0.68209, 0.68191, 0.68149, 0.68047, 0.67817, 0.67426, 0.66912, 0.66319, 0.65671, 0.64980, 0.64262,\
        0.63531, 0.62805, 0.62117, 0.61512, 0.61001, 0.60565, 0.60192, 0.59837, 0.59434, 0.58934, 0.58319,\
        0.57625, 0.56899, 0.56167, 0.55442, 0.54730, 0.54031, 0.53343, 0.52663, 0.51988, 0.51321, 0.50660,\
        0.50002, 0.49319, 0.48590, 0.47847, 0.47108, 0.46385, 0.45682, 0.45001, 0.44345, 0.43715, 0.43113,\
        0.42542, 0.42005, 0.41505, 0.41046, 0.40635, 0.40278, 0.39979, 0.39736, 0.39545, 0.39410, 0.39329,\
        0.39297, 0.39308, 0.39354, 0.39429, 0.39525, 0.39632, 0.39743, 0.39847, 0.39938, 0.40007, 0.40048],\
        [0.60255, 0.58776, 0.57082, 0.55529, 0.54312, 0.53534, 0.53188, 0.53184, 0.53416, 0.53809, 0.54317,\
        0.54914, 0.55579, 0.56295, 0.57045, 0.57805, 0.58536, 0.59159, 0.59627, 0.60002, 0.60357, 0.60709,\
        0.61065, 0.61425, 0.61788, 0.62157, 0.62529, 0.62900, 0.63267, 0.63634, 0.64003, 0.64378, 0.64752,\
        0.65116, 0.65457, 0.65766, 0.66034, 0.66259, 0.66443, 0.66593, 0.66724, 0.66850, 0.66968, 0.67056,\
        0.67095, 0.67088, 0.67056, 0.67019, 0.66987, 0.66966, 0.66961, 0.66971, 0.66988, 0.67004, 0.67010,\
        0.67001, 0.66969, 0.66899, 0.66758, 0.66507, 0.66123, 0.65619, 0.65027, 0.64381, 0.63706, 0.63017,\
        0.62315, 0.61602, 0.60908, 0.60302, 0.59813, 0.59385, 0.59011, 0.58648, 0.58244, 0.57762, 0.57192,\
        0.56555, 0.55881, 0.55193, 0.54506, 0.53828, 0.53156, 0.52487, 0.51818, 0.51149, 0.50487, 0.49833,\
        0.49182, 0.48517, 0.47826, 0.47121, 0.46419, 0.45730, 0.45060, 0.44413, 0.43791, 0.43197, 0.42632,\
        0.42100, 0.41603, 0.41144, 0.40726, 0.40354, 0.40038, 0.39781, 0.39579, 0.39426, 0.39323, 0.39267,\
        0.39254, 0.39280, 0.39337, 0.39420, 0.39520, 0.39630, 0.39741, 0.39846, 0.39936, 0.40003, 0.40042],\
        [0.59221, 0.57649, 0.55907, 0.54282, 0.53022, 0.52297, 0.52091, 0.52226, 0.52552, 0.52995, 0.53521,\
        0.54111, 0.54750, 0.55426, 0.56122, 0.56813, 0.57464, 0.58028, 0.58485, 0.58866, 0.59220, 0.59570,\
        0.59926, 0.60289, 0.60655, 0.61028, 0.61406, 0.61778, 0.62139, 0.62495, 0.62855, 0.63221, 0.63591,\
        0.63957, 0.64306, 0.64629, 0.64915, 0.65158, 0.65357, 0.65514, 0.65636, 0.65730, 0.65798, 0.65836,\
        0.65841, 0.65821, 0.65789, 0.65759, 0.65738, 0.65730, 0.65738, 0.65758, 0.65783, 0.65803, 0.65808,\
        0.65791, 0.65742, 0.65647, 0.65480, 0.65214, 0.64828, 0.64324, 0.63726, 0.63076, 0.62415, 0.61763,\
        0.61107, 0.60418, 0.59737, 0.59136, 0.58648, 0.58223, 0.57839, 0.57461, 0.57055, 0.56591, 0.56061,\
        0.55476, 0.54854, 0.54214, 0.53571, 0.52931, 0.52293, 0.51650, 0.50994, 0.50330, 0.49673, 0.49026,\
        0.48382, 0.47731, 0.47069, 0.46400, 0.45736, 0.45084, 0.44451, 0.43841, 0.43257, 0.42701, 0.42175,\
        0.41684, 0.41229, 0.40812, 0.40436, 0.40105, 0.39830, 0.39612, 0.39445, 0.39325, 0.39250, 0.39216,\
        0.39221, 0.39260, 0.39327, 0.39416, 0.39520, 0.39631, 0.39743, 0.39847, 0.39934, 0.39999, 0.40034],\
        [0.57865, 0.56285, 0.54556, 0.52917, 0.51641, 0.51032, 0.51067, 0.51370, 0.51775, 0.52247, 0.52769,\
        0.53332, 0.53928, 0.54547, 0.55174, 0.55790, 0.56366, 0.56878, 0.57316, 0.57697, 0.58050, 0.58397,\
        0.58748, 0.59107, 0.59472, 0.59848, 0.60231, 0.60603, 0.60956, 0.61302, 0.61653, 0.62015, 0.62386,\
        0.62758, 0.63121, 0.63462, 0.63770, 0.64034, 0.64248, 0.64411, 0.64521, 0.64585, 0.64608, 0.64600,\
        0.64572, 0.64535, 0.64501, 0.64478, 0.64469, 0.64475, 0.64496, 0.64528, 0.64563, 0.64588, 0.64591,\
        0.64562, 0.64495, 0.64380, 0.64200, 0.63931, 0.63547, 0.63038, 0.62422, 0.61758, 0.61106, 0.60489,\
        0.59888, 0.59236, 0.58592, 0.58014, 0.57518, 0.57086, 0.56690, 0.56302, 0.55896, 0.55450, 0.54956,\
        0.54415, 0.53840, 0.53246, 0.52645, 0.52046, 0.51447, 0.50836, 0.50196, 0.49542, 0.48890, 0.48247,\
        0.47608, 0.46970, 0.46331, 0.45695, 0.45067, 0.44454, 0.43860, 0.43289, 0.42745, 0.42228, 0.41743,\
        0.41294, 0.40883, 0.40512, 0.40180, 0.39891, 0.39655, 0.39471, 0.39335, 0.39242, 0.39191, 0.39177,\
        0.39198, 0.39248, 0.39323, 0.39416, 0.39523, 0.39635, 0.39746, 0.39848, 0.39933, 0.39994, 0.40026],\
        [0.56475, 0.54924, 0.53264, 0.51688, 0.50442, 0.49976, 0.50298, 0.50693, 0.51121, 0.51584, 0.52077,\
        0.52594, 0.53130, 0.53680, 0.54232, 0.54770, 0.55277, 0.55738, 0.56148, 0.56517, 0.56863, 0.57201,\
        0.57542, 0.57890, 0.58248, 0.58616, 0.58991, 0.59357, 0.59707, 0.60046, 0.60392, 0.60754, 0.61128,\
        0.61509, 0.61888, 0.62251, 0.62581, 0.62867, 0.63098, 0.63267, 0.63368, 0.63404, 0.63386, 0.63336,\
        0.63274, 0.63220, 0.63186, 0.63171, 0.63173, 0.63190, 0.63223, 0.63268, 0.63314, 0.63344, 0.63342,\
        0.63299, 0.63213, 0.63081, 0.62896, 0.62637, 0.62272, 0.61766, 0.61128, 0.60445, 0.59798, 0.59207,\
        0.58629, 0.58034, 0.57444, 0.56898, 0.56412, 0.55978, 0.55576, 0.55187, 0.54787, 0.54360, 0.53897,\
        0.53396, 0.52862, 0.52307, 0.51743, 0.51181, 0.50618, 0.50047, 0.49422, 0.48779, 0.48135, 0.47496,\
        0.46864, 0.46238, 0.45621, 0.45013, 0.44420, 0.43845, 0.43292, 0.42763, 0.42259, 0.41784, 0.41339,\
        0.40930, 0.40563, 0.40240, 0.39957, 0.39709, 0.39508, 0.39355, 0.39246, 0.39178, 0.39147, 0.39150,\
        0.39183, 0.39242, 0.39323, 0.39420, 0.39527, 0.39639, 0.39748, 0.39849, 0.39932, 0.39991, 0.40019],\
        [0.55201, 0.53727, 0.52202, 0.50803, 0.49795, 0.49484, 0.49768, 0.50166, 0.50573, 0.51004, 0.51449,\
        0.51904, 0.52369, 0.52843, 0.53317, 0.53781, 0.54221, 0.54630, 0.55005, 0.55352, 0.55681, 0.56003,\
        0.56327, 0.56659, 0.56999, 0.57349, 0.57704, 0.58055, 0.58397, 0.58734, 0.59079, 0.59439, 0.59814,\
        0.60203, 0.60595, 0.60977, 0.61326, 0.61633, 0.61884, 0.62062, 0.62158, 0.62171, 0.62118, 0.62029,\
        0.61938, 0.61869, 0.61839, 0.61837, 0.61847, 0.61871, 0.61911, 0.61965, 0.62022, 0.62058, 0.62049,\
        0.61987, 0.61877, 0.61728, 0.61542, 0.61301, 0.60971, 0.60496, 0.59845, 0.59168, 0.58536, 0.57953,\
        0.57393, 0.56841, 0.56302, 0.55795, 0.55331, 0.54906, 0.54510, 0.54127, 0.53740, 0.53336, 0.52905,\
        0.52440, 0.51943, 0.51420, 0.50886, 0.50354, 0.49817, 0.49262, 0.48667, 0.48042, 0.47409, 0.46777,\
        0.46152, 0.45539, 0.44941, 0.44360, 0.43800, 0.43264, 0.42752, 0.42266, 0.41805, 0.41372, 0.40967,\
        0.40597, 0.40271, 0.39992, 0.39753, 0.39549, 0.39384, 0.39260, 0.39176, 0.39129, 0.39116, 0.39133,\
        0.39177, 0.39243, 0.39328, 0.39426, 0.39532, 0.39641, 0.39747, 0.39844, 0.39926, 0.39986, 0.40005],\
        [0.54105, 0.52755, 0.51414, 0.50234, 0.49456, 0.49197, 0.49360, 0.49706, 0.50098, 0.50492, 0.50878,\
        0.51264, 0.51655, 0.52052, 0.52449, 0.52840, 0.53217, 0.53573, 0.53908, 0.54224, 0.54527, 0.54827,\
        0.55128, 0.55436, 0.55753, 0.56077, 0.56406, 0.56736, 0.57064, 0.57393, 0.57730, 0.58082, 0.58451,\
        0.58836, 0.59229, 0.59619, 0.59981, 0.60303, 0.60573, 0.60766, 0.60864, 0.60867, 0.60785, 0.60672,\
        0.60563, 0.60485, 0.60467, 0.60479, 0.60491, 0.60514, 0.60555, 0.60611, 0.60671, 0.60711, 0.60692,\
        0.60607, 0.60471, 0.60306, 0.60115, 0.59884, 0.59576, 0.59127, 0.58532, 0.57898, 0.57293, 0.56729,\
        0.56195, 0.55682, 0.55189, 0.54722, 0.54288, 0.53884, 0.53502, 0.53134, 0.52767, 0.52392, 0.51995,\
        0.51568, 0.51105, 0.50613, 0.50103, 0.49597, 0.49079, 0.48534, 0.47954, 0.47345, 0.46721, 0.46096,\
        0.45478, 0.44876, 0.44295, 0.43740, 0.43212, 0.42714, 0.42245, 0.41803, 0.41388, 0.40998, 0.40634,\
        0.40301, 0.40013, 0.39775, 0.39577, 0.39411, 0.39281, 0.39185, 0.39125, 0.39097, 0.39099, 0.39127,\
        0.39179, 0.39250, 0.39336, 0.39434, 0.39537, 0.39641, 0.39741, 0.39831, 0.39905, 0.39954, 0.39966],\
        [0.53193, 0.51994, 0.50853, 0.49892, 0.49249, 0.48996, 0.49066, 0.49329, 0.49672, 0.50029, 0.50357,\
        0.50674, 0.50994, 0.51319, 0.51645, 0.51968, 0.52283, 0.52586, 0.52877, 0.53156, 0.53429, 0.53699,\
        0.53972, 0.54251, 0.54537, 0.54832, 0.55132, 0.55436, 0.55742, 0.56053, 0.56374, 0.56708, 0.57059,\
        0.57425, 0.57800, 0.58174, 0.58529, 0.58852, 0.59127, 0.59329, 0.59437, 0.59448, 0.59371, 0.59266,\
        0.59167, 0.59099, 0.59084, 0.59096, 0.59109, 0.59128, 0.59160, 0.59203, 0.59248, 0.59272, 0.59236,\
        0.59137, 0.58990, 0.58817, 0.58621, 0.58386, 0.58080, 0.57668, 0.57158, 0.56603, 0.56055, 0.55533,\
        0.55040, 0.54571, 0.54125, 0.53702, 0.53303, 0.52927, 0.52568, 0.52222, 0.51881, 0.51538, 0.51183,\
        0.50799, 0.50371, 0.49912, 0.49432, 0.48940, 0.48429, 0.47887, 0.47312, 0.46707, 0.46086, 0.45462,\
        0.44848, 0.44253, 0.43687, 0.43155, 0.42659, 0.42200, 0.41774, 0.41378, 0.41011, 0.40668, 0.40348,\
        0.40055, 0.39804, 0.39601, 0.39435, 0.39303, 0.39201, 0.39131, 0.39091, 0.39079, 0.39094, 0.39132,\
        0.39189, 0.39263, 0.39349, 0.39443, 0.39541, 0.39638, 0.39729, 0.39809, 0.39870, 0.39906, 0.39910],\
        [0.52439, 0.51390, 0.50429, 0.49637, 0.49094, 0.48843, 0.48848, 0.49028, 0.49299, 0.49595, 0.49877,\
        0.50139, 0.50398, 0.50660, 0.50921, 0.51181, 0.51438, 0.51690, 0.51935, 0.52175, 0.52411, 0.52648,\
        0.52887, 0.53132, 0.53385, 0.53646, 0.53914, 0.54188, 0.54466, 0.54751, 0.55045, 0.55351, 0.55671,\
        0.56004, 0.56346, 0.56687, 0.57015, 0.57317, 0.57575, 0.57770, 0.57887, 0.57922, 0.57889, 0.57825,\
        0.57761, 0.57718, 0.57705, 0.57711, 0.57720, 0.57730, 0.57743, 0.57760, 0.57771, 0.57759, 0.57704,\
        0.57601, 0.57458, 0.57289, 0.57094, 0.56862, 0.56572, 0.56209, 0.55778, 0.55310, 0.54837, 0.54378,\
        0.53940, 0.53524, 0.53129, 0.52753, 0.52396, 0.52055, 0.51726, 0.51406, 0.51095, 0.50788, 0.50477,\
        0.50146, 0.49753, 0.49327, 0.48872, 0.48391, 0.47880, 0.47335, 0.46754, 0.46144, 0.45517, 0.44886,\
        0.44267, 0.43675, 0.43120, 0.42609, 0.42144, 0.41723, 0.41341, 0.40993, 0.40675, 0.40381, 0.40108,\
        0.39858, 0.39643, 0.39468, 0.39330, 0.39223, 0.39146, 0.39097, 0.39075, 0.39077, 0.39102, 0.39146,\
        0.39208, 0.39282, 0.39366, 0.39455, 0.39546, 0.39633, 0.39713, 0.39780, 0.39828, 0.39851, 0.39845],\
        [0.51814, 0.50896, 0.50075, 0.49416, 0.48957, 0.48719, 0.48681, 0.48792, 0.48992, 0.49226, 0.49459,\
        0.49678, 0.49889, 0.50095, 0.50299, 0.50501, 0.50705, 0.50907, 0.51106, 0.51304, 0.51502, 0.51701,\
        0.51902, 0.52110, 0.52326, 0.52551, 0.52785, 0.53025, 0.53270, 0.53521, 0.53779, 0.54047, 0.54327,\
        0.54617, 0.54914, 0.55211, 0.55500, 0.55767, 0.56000, 0.56184, 0.56309, 0.56375, 0.56392, 0.56382,\
        0.56364, 0.56352, 0.56351, 0.56355, 0.56355, 0.56350, 0.56338, 0.56321, 0.56292, 0.56245, 0.56169,\
        0.56062, 0.55927, 0.55768, 0.55585, 0.55369, 0.55111, 0.54802, 0.54447, 0.54063, 0.53671, 0.53286,\
        0.52914, 0.52560, 0.52222, 0.51899, 0.51589, 0.51288, 0.50995, 0.50707, 0.50427, 0.50159, 0.49888,\
        0.49596, 0.49251, 0.48856, 0.48423, 0.47951, 0.47439, 0.46886, 0.46293, 0.45668, 0.45023, 0.44376,\
        0.43744, 0.43146, 0.42596, 0.42103, 0.41668, 0.41286, 0.40949, 0.40649, 0.40379, 0.40133, 0.39905,\
        0.39696, 0.39516, 0.39370, 0.39256, 0.39172, 0.39116, 0.39085, 0.39077, 0.39090, 0.39123, 0.39172,\
        0.39234, 0.39307, 0.39387, 0.39469, 0.39551, 0.39627, 0.39694, 0.39747, 0.39782, 0.39793, 0.39777],\
        [0.51294, 0.50492, 0.49790, 0.49231, 0.48838, 0.48618, 0.48556, 0.48618, 0.48759, 0.48940, 0.49129,\
        0.49312, 0.49484, 0.49647, 0.49803, 0.49953, 0.50109, 0.50263, 0.50415, 0.50572, 0.50729, 0.50888,\
        0.51050, 0.51218, 0.51395, 0.51582, 0.51780, 0.51983, 0.52191, 0.52402, 0.52619, 0.52842, 0.53073,\
        0.53312, 0.53557, 0.53803, 0.54045, 0.54273, 0.54476, 0.54647, 0.54779, 0.54873, 0.54937, 0.54980,\
        0.55010, 0.55033, 0.55050, 0.55056, 0.55049, 0.55025, 0.54985, 0.54930, 0.54863, 0.54783, 0.54689,\
        0.54579, 0.54453, 0.54310, 0.54147, 0.53960, 0.53742, 0.53491, 0.53209, 0.52907, 0.52597, 0.52289,\
        0.51990, 0.51702, 0.51425, 0.51159, 0.50902, 0.50649, 0.50397, 0.50150, 0.49906, 0.49677, 0.49442,\
        0.49177, 0.48866, 0.48502, 0.48088, 0.47625, 0.47111, 0.46547, 0.45936, 0.45288, 0.44616, 0.43941,\
        0.43285, 0.42671, 0.42118, 0.41639, 0.41234, 0.40892, 0.40600, 0.40347, 0.40122, 0.39919, 0.39732,\
        0.39563, 0.39419, 0.39301, 0.39212, 0.39148, 0.39109, 0.39092, 0.39096, 0.39118, 0.39156, 0.39208,\
        0.39269, 0.39339, 0.39412, 0.39487, 0.39558, 0.39622, 0.39676, 0.39715, 0.39735, 0.39733, 0.39706],\
        [0.50864, 0.50167, 0.49564, 0.49086, 0.48746, 0.48545, 0.48472, 0.48502, 0.48602, 0.48742, 0.48895,\
        0.49048, 0.49191, 0.49323, 0.49444, 0.49559, 0.49672, 0.49782, 0.49890, 0.50005, 0.50122, 0.50241,\
        0.50359, 0.50488, 0.50624, 0.50772, 0.50934, 0.51100, 0.51266, 0.51434, 0.51605, 0.51779, 0.51958,\
        0.52143, 0.52331, 0.52523, 0.52714, 0.52899, 0.53071, 0.53225, 0.53358, 0.53474, 0.53578, 0.53668,\
        0.53744, 0.53801, 0.53837, 0.53849, 0.53834, 0.53792, 0.53724, 0.53634, 0.53531, 0.53422, 0.53315,\
        0.53207, 0.53095, 0.52973, 0.52837, 0.52684, 0.52512, 0.52320, 0.52109, 0.51884, 0.51654, 0.51423,\
        0.51196, 0.50975, 0.50763, 0.50556, 0.50354, 0.50154, 0.49951, 0.49749, 0.49550, 0.49350, 0.49139,\
        0.48897, 0.48610, 0.48270, 0.47874, 0.47419, 0.46904, 0.46327, 0.45695, 0.45016, 0.44307, 0.43592,\
        0.42897, 0.42254, 0.41688, 0.41219, 0.40844, 0.40546, 0.40300, 0.40089, 0.39907, 0.39743, 0.39596,\
        0.39464, 0.39352, 0.39262, 0.39194, 0.39149, 0.39125, 0.39120, 0.39133, 0.39161, 0.39202, 0.39254,\
        0.39313, 0.39378, 0.39443, 0.39508, 0.39567, 0.39618, 0.39658, 0.39682, 0.39688, 0.39673, 0.39634],\
        [0.50505, 0.49905, 0.49387, 0.48976, 0.48681, 0.48502, 0.48429, 0.48441, 0.48515, 0.48627, 0.48755,\
        0.48884, 0.49006, 0.49116, 0.49215, 0.49305, 0.49388, 0.49467, 0.49543, 0.49622, 0.49703, 0.49783,\
        0.49862, 0.49955, 0.50049, 0.50155, 0.50286, 0.50413, 0.50533, 0.50658, 0.50783, 0.50906, 0.51033,\
        0.51163, 0.51294, 0.51430, 0.51570, 0.51712, 0.51849, 0.51980, 0.52103, 0.52229, 0.52363, 0.52494,\
        0.52609, 0.52695, 0.52748, 0.52765, 0.52745, 0.52688, 0.52596, 0.52476, 0.52341, 0.52210, 0.52097,\
        0.52001, 0.51910, 0.51814, 0.51708, 0.51592, 0.51469, 0.51335, 0.51189, 0.51036, 0.50879, 0.50721,\
        0.50564, 0.50408, 0.50257, 0.50110, 0.49962, 0.49815, 0.49660, 0.49500, 0.49336, 0.49162, 0.48971,\
        0.48748, 0.48482, 0.48162, 0.47783, 0.47337, 0.46821, 0.46233, 0.45577, 0.44862, 0.44108, 0.43340,\
        0.42592, 0.41903, 0.41310, 0.40843, 0.40502, 0.40252, 0.40053, 0.39885, 0.39741, 0.39614, 0.39502,\
        0.39403, 0.39319, 0.39253, 0.39205, 0.39175, 0.39163, 0.39167, 0.39186, 0.39218, 0.39260, 0.39310,\
        0.39365, 0.39423, 0.39480, 0.39533, 0.39580, 0.39617, 0.39642, 0.39651, 0.39642, 0.39613, 0.39562],\
        [0.50199, 0.49685, 0.49244, 0.48894, 0.48643, 0.48490, 0.48424, 0.48431, 0.48492, 0.48587, 0.48699,\
        0.48814, 0.48922, 0.49018, 0.49102, 0.49175, 0.49239, 0.49297, 0.49352, 0.49406, 0.49461, 0.49515,\
        0.49571, 0.49632, 0.49697, 0.49774, 0.49867, 0.49953, 0.50027, 0.50116, 0.50196, 0.50267, 0.50349,\
        0.50428, 0.50502, 0.50588, 0.50680, 0.50777, 0.50879, 0.50978, 0.51076, 0.51192, 0.51343, 0.51504,\
        0.51644, 0.51747, 0.51810, 0.51831, 0.51809, 0.51745, 0.51639, 0.51499, 0.51340, 0.51195, 0.51091,\
        0.51020, 0.50958, 0.50893, 0.50816, 0.50737, 0.50662, 0.50583, 0.50494, 0.50404, 0.50313, 0.50220,\
        0.50125, 0.50028, 0.49930, 0.49836, 0.49734, 0.49627, 0.49510, 0.49384, 0.49248, 0.49097, 0.48924,\
        0.48720, 0.48473, 0.48174, 0.47812, 0.47379, 0.46867, 0.46270, 0.45591, 0.44839, 0.44033, 0.43203,\
        0.42387, 0.41631, 0.40990, 0.40511, 0.40210, 0.40015, 0.39864, 0.39739, 0.39632, 0.39539, 0.39456,\
        0.39383, 0.39323, 0.39276, 0.39243, 0.39226, 0.39223, 0.39233, 0.39255, 0.39288, 0.39329, 0.39376,\
        0.39426, 0.39475, 0.39522, 0.39563, 0.39596, 0.39618, 0.39627, 0.39621, 0.39597, 0.39554, 0.39490],\
        [0.49920, 0.49492, 0.49126, 0.48837, 0.48630, 0.48505, 0.48454, 0.48465, 0.48523, 0.48613, 0.48718,\
        0.48826, 0.48927, 0.49015, 0.49090, 0.49151, 0.49203, 0.49248, 0.49289, 0.49328, 0.49368, 0.49407,\
        0.49449, 0.49494, 0.49545, 0.49604, 0.49669, 0.49728, 0.49781, 0.49834, 0.49879, 0.49911, 0.49958,\
        0.49994, 0.50017, 0.50067, 0.50114, 0.50167, 0.50232, 0.50294, 0.50346, 0.50420, 0.50571, 0.50739,\
        0.50879, 0.50981, 0.51043, 0.51063, 0.51043, 0.50982, 0.50880, 0.50742, 0.50578, 0.50434, 0.50362,\
        0.50334, 0.50300, 0.50271, 0.50221, 0.50175, 0.50145, 0.50117, 0.50068, 0.50033, 0.49993, 0.49952,\
        0.49905, 0.49851, 0.49790, 0.49725, 0.49653, 0.49573, 0.49483, 0.49383, 0.49269, 0.49138, 0.48983,\
        0.48797, 0.48571, 0.48295, 0.47957, 0.47543, 0.47041, 0.46442, 0.45744, 0.44956, 0.44097, 0.43199,\
        0.42304, 0.41467, 0.40753, 0.40236, 0.39966, 0.39839, 0.39745, 0.39663, 0.39590, 0.39523, 0.39462,\
        0.39409, 0.39365, 0.39332, 0.39310, 0.39301, 0.39303, 0.39317, 0.39340, 0.39372, 0.39410, 0.39451,\
        0.39494, 0.39534, 0.39569, 0.39597, 0.39615, 0.39622, 0.39614, 0.39592, 0.39552, 0.39494, 0.39417],\
        [0.49696, 0.49337, 0.49038, 0.48804, 0.48640, 0.48545, 0.48514, 0.48536, 0.48601, 0.48693, 0.48799,\
        0.48907, 0.49006, 0.49091, 0.49161, 0.49215, 0.49259, 0.49296, 0.49329, 0.49360, 0.49392, 0.49425,\
        0.49460, 0.49499, 0.49543, 0.49593, 0.49646, 0.49696, 0.49740, 0.49776, 0.49805, 0.49829, 0.49853,\
        0.49867, 0.49883, 0.49913, 0.49937, 0.49961, 0.49983, 0.49996, 0.50012, 0.50008, 0.50108, 0.50222,\
        0.50323, 0.50400, 0.50447, 0.50464, 0.50449, 0.50402, 0.50327, 0.50228, 0.50115, 0.50020, 0.50007,\
        0.50026, 0.49986, 0.49991, 0.49988, 0.49978, 0.49973, 0.49971, 0.49952, 0.49940, 0.49928, 0.49907,\
        0.49878, 0.49840, 0.49795, 0.49744, 0.49689, 0.49627, 0.49556, 0.49476, 0.49381, 0.49268, 0.49130,\
        0.48963, 0.48761, 0.48512, 0.48205, 0.47820, 0.47340, 0.46749, 0.46040, 0.45220, 0.44311, 0.43348,\
        0.42376, 0.41450, 0.40647, 0.40085, 0.39841, 0.39778, 0.39729, 0.39676, 0.39623, 0.39571, 0.39524,\
        0.39481, 0.39447, 0.39421, 0.39405, 0.39400, 0.39404, 0.39417, 0.39439, 0.39467, 0.39500, 0.39535,\
        0.39569, 0.39598, 0.39621, 0.39634, 0.39637, 0.39627, 0.39603, 0.39563, 0.39508, 0.39435, 0.39343],\
        [0.49543, 0.49233, 0.48985, 0.48797, 0.48672, 0.48607, 0.48598, 0.48636, 0.48712, 0.48814, 0.48928,\
        0.49042, 0.49145, 0.49230, 0.49295, 0.49345, 0.49383, 0.49415, 0.49444, 0.49473, 0.49502, 0.49533,\
        0.49567, 0.49604, 0.49645, 0.49692, 0.49742, 0.49790, 0.49831, 0.49861, 0.49884, 0.49906, 0.49929,\
        0.49932, 0.49945, 0.49985, 0.49998, 0.50012, 0.50007, 0.49965, 0.49955, 0.49936, 0.49901, 0.49912,\
        0.49947, 0.49984, 0.50010, 0.50020, 0.50012, 0.49988, 0.49953, 0.49920, 0.49911, 0.49950, 0.49970,\
        0.49969, 0.49949, 0.49950, 0.49995, 0.50013, 0.50016, 0.50015, 0.50011, 0.50006, 0.50002, 0.49988,\
        0.49965, 0.49935, 0.49895, 0.49853, 0.49809, 0.49759, 0.49704, 0.49641, 0.49564, 0.49467, 0.49347,\
        0.49200, 0.49023, 0.48809, 0.48541, 0.48198, 0.47755, 0.47184, 0.46473, 0.45629, 0.44679, 0.43663,\
        0.42629, 0.41634, 0.40748, 0.40082, 0.39928, 0.39896, 0.39842, 0.39788, 0.39736, 0.39687, 0.39641,\
        0.39600, 0.39567, 0.39542, 0.39527, 0.39521, 0.39523, 0.39533, 0.39550, 0.39572, 0.39598, 0.39625,\
        0.39649, 0.39667, 0.39676, 0.39674, 0.39659, 0.39632, 0.39590, 0.39534, 0.39462, 0.39373, 0.39267],\
        [0.49434, 0.49167, 0.48961, 0.48812, 0.48721, 0.48684, 0.48697, 0.48754, 0.48846, 0.48962, 0.49089,\
        0.49214, 0.49324, 0.49411, 0.49474, 0.49518, 0.49552, 0.49581, 0.49608, 0.49636, 0.49666, 0.49697,\
        0.49731, 0.49766, 0.49805, 0.49849, 0.49897, 0.49944, 0.49981, 0.50003, 0.50018, 0.50032, 0.50048,\
        0.50043, 0.50044, 0.50059, 0.50058, 0.50041, 0.50004, 0.49948, 0.49892, 0.49830, 0.49763, 0.49719,\
        0.49702, 0.49701, 0.49705, 0.49708, 0.49707, 0.49705, 0.49708, 0.49727, 0.49772, 0.49838, 0.49889,\
        0.49915, 0.49930, 0.49956, 0.50002, 0.50040, 0.50062, 0.50076, 0.50086, 0.50094, 0.50099, 0.50096,\
        0.50086, 0.50067, 0.50039, 0.50010, 0.49979, 0.49941, 0.49901, 0.49855, 0.49797, 0.49717, 0.49613,\
        0.49486, 0.49338, 0.49162, 0.48944, 0.48657, 0.48267, 0.47734, 0.47031, 0.46169, 0.45186, 0.44134,\
        0.43068, 0.42049, 0.41155, 0.40515, 0.40257, 0.40156, 0.40067, 0.39991, 0.39924, 0.39865, 0.39810,\
        0.39763, 0.39723, 0.39694, 0.39674, 0.39663, 0.39659, 0.39662, 0.39670, 0.39684, 0.39701, 0.39720,\
        0.39734, 0.39739, 0.39733, 0.39714, 0.39682, 0.39636, 0.39577, 0.39503, 0.39414, 0.39309, 0.39188],\
        [0.49352, 0.49126, 0.48956, 0.48841, 0.48780, 0.48769, 0.48804, 0.48880, 0.48989, 0.49121, 0.49264,\
        0.49405, 0.49527, 0.49616, 0.49673, 0.49711, 0.49741, 0.49767, 0.49793, 0.49822, 0.49851, 0.49882,\
        0.49915, 0.49947, 0.49979, 0.50017, 0.50055, 0.50091, 0.50117, 0.50131, 0.50137, 0.50140, 0.50139,\
        0.50129, 0.50117, 0.50105, 0.50083, 0.50044, 0.49986, 0.49913, 0.49831, 0.49745, 0.49660, 0.49590,\
        0.49543, 0.49515, 0.49502, 0.49499, 0.49504, 0.49519, 0.49547, 0.49595, 0.49663, 0.49740, 0.49811,\
        0.49865, 0.49911, 0.49959, 0.50012, 0.50062, 0.50101, 0.50133, 0.50158, 0.50180, 0.50196, 0.50207,\
        0.50210, 0.50206, 0.50195, 0.50182, 0.50166, 0.50146, 0.50123, 0.50094, 0.50055, 0.49995, 0.49909,\
        0.49805, 0.49686, 0.49553, 0.49389, 0.49167, 0.48852, 0.48375, 0.47685, 0.46805, 0.45796, 0.44724,\
        0.43652, 0.42643, 0.41775, 0.41135, 0.40753, 0.40537, 0.40388, 0.40272, 0.40177, 0.40096, 0.40025,\
        0.39963, 0.39911, 0.39871, 0.39843, 0.39823, 0.39810, 0.39802, 0.39799, 0.39800, 0.39808, 0.39817,\
        0.39821, 0.39813, 0.39790, 0.39752, 0.39701, 0.39637, 0.39559, 0.39468, 0.39362, 0.39242, 0.39105],\
        [0.49285, 0.49098, 0.48962, 0.48877, 0.48842, 0.48854, 0.48910, 0.49003, 0.49127, 0.49274, 0.49434,\
        0.49593, 0.49732, 0.49824, 0.49869, 0.49900, 0.49926, 0.49949, 0.49974, 0.50001, 0.50028, 0.50056,\
        0.50083, 0.50110, 0.50135, 0.50162, 0.50188, 0.50212, 0.50227, 0.50233, 0.50231, 0.50223, 0.50210,\
        0.50189, 0.50164, 0.50132, 0.50090, 0.50033, 0.49961, 0.49874, 0.49779, 0.49681, 0.49586, 0.49504,\
        0.49441, 0.49399, 0.49375, 0.49369, 0.49377, 0.49402, 0.49445, 0.49506, 0.49584, 0.49669, 0.49752,\
        0.49826, 0.49894, 0.49960, 0.50025, 0.50087, 0.50141, 0.50188, 0.50229, 0.50264, 0.50294, 0.50318,\
        0.50337, 0.50349, 0.50356, 0.50359, 0.50360, 0.50358, 0.50352, 0.50341, 0.50319, 0.50280, 0.50220,\
        0.50144, 0.50056, 0.49964, 0.49853, 0.49692, 0.49471, 0.49072, 0.48385, 0.47479, 0.46451, 0.45376,\
        0.44313, 0.43326, 0.42473, 0.41807, 0.41337, 0.41018, 0.40793, 0.40622, 0.40486, 0.40373, 0.40277,\
        0.40195, 0.40126, 0.40071, 0.40029, 0.39996, 0.39970, 0.39949, 0.39932, 0.39918, 0.39914, 0.39914,\
        0.39908, 0.39885, 0.39843, 0.39786, 0.39715, 0.39632, 0.39536, 0.39428, 0.39306, 0.39169, 0.39018],\
        [0.49227, 0.49075, 0.48970, 0.48912, 0.48901, 0.48933, 0.49005, 0.49112, 0.49249, 0.49408, 0.49579,\
        0.49751, 0.49907, 0.50005, 0.50037, 0.50064, 0.50087, 0.50107, 0.50129, 0.50153, 0.50176, 0.50199,\
        0.50221, 0.50241, 0.50260, 0.50279, 0.50295, 0.50307, 0.50313, 0.50310, 0.50300, 0.50283, 0.50260,\
        0.50229, 0.50191, 0.50144, 0.50087, 0.50017, 0.49934, 0.49840, 0.49739, 0.49636, 0.49537, 0.49450,\
        0.49381, 0.49332, 0.49303, 0.49294, 0.49304, 0.49335, 0.49384, 0.49452, 0.49533, 0.49623, 0.49714,\
        0.49801, 0.49885, 0.49965, 0.50042, 0.50115, 0.50182, 0.50243, 0.50298, 0.50347, 0.50391, 0.50430,\
        0.50463, 0.50491, 0.50515, 0.50535, 0.50553, 0.50568, 0.50580, 0.50585, 0.50582, 0.50565, 0.50534,\
        0.50490, 0.50439, 0.50383, 0.50311, 0.50200, 0.50071, 0.49750, 0.49034, 0.48109, 0.47083, 0.46023,\
        0.44985, 0.44021, 0.43178, 0.42487, 0.41953, 0.41555, 0.41255, 0.41024, 0.40839, 0.40688, 0.40561,\
        0.40453, 0.40363, 0.40288, 0.40228, 0.40177, 0.40135, 0.40099, 0.40066, 0.40037, 0.40020, 0.40008,\
        0.39989, 0.39949, 0.39887, 0.39810, 0.39721, 0.39620, 0.39507, 0.39381, 0.39243, 0.39091, 0.38925]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=20 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax20vt2.5
    zgrid_H_Hmax_vmax20vt2_5=[[0.32139, 0.32791, 0.33417, 0.34017, 0.34587, 0.35126, 0.35631, 0.36100, 0.36529, 0.36918, 0.37265,\
        0.37571, 0.37843, 0.38087, 0.38314, 0.38535, 0.38750, 0.38955, 0.39136, 0.39276, 0.39346, 0.39336,\
        0.39272, 0.39190, 0.39122, 0.39090, 0.39113, 0.39206, 0.39373, 0.39614, 0.39913, 0.40229, 0.40578,\
        0.40966, 0.41388, 0.41836, 0.42302, 0.42781, 0.43265, 0.43750, 0.44233, 0.44710, 0.45180, 0.45641,\
        0.46094, 0.46536, 0.46964, 0.47374, 0.47758, 0.48110, 0.48425, 0.48690, 0.48896, 0.49053, 0.49202,\
        0.49365, 0.49557, 0.49775, 0.49952, 0.50097, 0.50214, 0.50307, 0.50376, 0.50421, 0.50442, 0.50440,\
        0.50415, 0.50367, 0.50298, 0.50209, 0.50104, 0.49985, 0.49857, 0.49729, 0.49619, 0.49546, 0.49425,\
        0.49241, 0.49056, 0.48882, 0.48712, 0.48538, 0.48355, 0.48161, 0.47959, 0.47752, 0.47540, 0.47326,\
        0.47111, 0.46898, 0.46686, 0.46478, 0.46274, 0.46073, 0.45876, 0.45682, 0.45489, 0.45295, 0.45099,\
        0.44898, 0.44692, 0.44478, 0.44256, 0.44024, 0.43782, 0.43528, 0.43260, 0.42978, 0.42679, 0.42361,\
        0.42022, 0.41659, 0.41269, 0.40846, 0.40383, 0.39861, 0.39247, 0.38573, 0.37872, 0.37153, 0.36415],\
        [0.32761, 0.33408, 0.34030, 0.34625, 0.35191, 0.35725, 0.36225, 0.36686, 0.37106, 0.37482, 0.37811,\
        0.38093, 0.38332, 0.38541, 0.38734, 0.38927, 0.39126, 0.39322, 0.39499, 0.39642, 0.39704, 0.39647,\
        0.39529, 0.39399, 0.39287, 0.39219, 0.39216, 0.39292, 0.39454, 0.39699, 0.40010, 0.40372, 0.40770,\
        0.41206, 0.41674, 0.42165, 0.42672, 0.43187, 0.43703, 0.44218, 0.44725, 0.45225, 0.45715, 0.46195,\
        0.46666, 0.47127, 0.47578, 0.48013, 0.48421, 0.48794, 0.49133, 0.49423, 0.49638, 0.49783, 0.49926,\
        0.50096, 0.50290, 0.50516, 0.50706, 0.50861, 0.50987, 0.51086, 0.51160, 0.51209, 0.51233, 0.51233,\
        0.51208, 0.51160, 0.51088, 0.50996, 0.50884, 0.50757, 0.50621, 0.50483, 0.50362, 0.50283, 0.50145,\
        0.49931, 0.49739, 0.49561, 0.49387, 0.49209, 0.49016, 0.48808, 0.48593, 0.48374, 0.48149, 0.47920,\
        0.47692, 0.47467, 0.47244, 0.47026, 0.46812, 0.46603, 0.46399, 0.46200, 0.46003, 0.45806, 0.45606,\
        0.45402, 0.45191, 0.44973, 0.44747, 0.44510, 0.44264, 0.44006, 0.43734, 0.43448, 0.43146, 0.42825,\
        0.42482, 0.42115, 0.41720, 0.41292, 0.40822, 0.40297, 0.39682, 0.39016, 0.38336, 0.37646, 0.36936],\
        [0.33354, 0.33996, 0.34613, 0.35204, 0.35766, 0.36296, 0.36791, 0.37247, 0.37659, 0.38024, 0.38337,\
        0.38594, 0.38799, 0.38964, 0.39113, 0.39272, 0.39451, 0.39635, 0.39794, 0.39923, 0.39990, 0.39882,\
        0.39728, 0.39560, 0.39411, 0.39310, 0.39282, 0.39342, 0.39500, 0.39755, 0.40090, 0.40494, 0.40948,\
        0.41440, 0.41959, 0.42498, 0.43047, 0.43599, 0.44149, 0.44691, 0.45223, 0.45742, 0.46249, 0.46745,\
        0.47230, 0.47707, 0.48178, 0.48638, 0.49072, 0.49460, 0.49815, 0.50133, 0.50365, 0.50529, 0.50686,\
        0.50865, 0.51063, 0.51273, 0.51465, 0.51628, 0.51762, 0.51868, 0.51947, 0.52001, 0.52030, 0.52032,\
        0.52009, 0.51961, 0.51889, 0.51793, 0.51678, 0.51546, 0.51404, 0.51261, 0.51128, 0.51002, 0.50841,\
        0.50637, 0.50438, 0.50250, 0.50067, 0.49885, 0.49675, 0.49450, 0.49224, 0.48993, 0.48753, 0.48508,\
        0.48265, 0.48027, 0.47793, 0.47562, 0.47337, 0.47118, 0.46907, 0.46702, 0.46501, 0.46299, 0.46095,\
        0.45885, 0.45669, 0.45445, 0.45212, 0.44969, 0.44717, 0.44452, 0.44175, 0.43884, 0.43576, 0.43249,\
        0.42899, 0.42524, 0.42117, 0.41674, 0.41184, 0.40636, 0.40025, 0.39388, 0.38748, 0.38098, 0.37422],\
        [0.33917, 0.34554, 0.35167, 0.35754, 0.36312, 0.36838, 0.37329, 0.37781, 0.38187, 0.38544, 0.38844,\
        0.39082, 0.39251, 0.39367, 0.39462, 0.39576, 0.39735, 0.39906, 0.40044, 0.40122, 0.40144, 0.40044,\
        0.39884, 0.39690, 0.39511, 0.39386, 0.39338, 0.39384, 0.39539, 0.39811, 0.40185, 0.40633, 0.41142,\
        0.41691, 0.42263, 0.42849, 0.43439, 0.44027, 0.44608, 0.45176, 0.45729, 0.46266, 0.46787, 0.47294,\
        0.47789, 0.48275, 0.48758, 0.49243, 0.49712, 0.50112, 0.50476, 0.50803, 0.51064, 0.51271, 0.51458,\
        0.51648, 0.51848, 0.52050, 0.52238, 0.52403, 0.52542, 0.52653, 0.52739, 0.52798, 0.52831, 0.52838,\
        0.52818, 0.52771, 0.52699, 0.52602, 0.52483, 0.52346, 0.52198, 0.52044, 0.51892, 0.51733, 0.51554,\
        0.51354, 0.51150, 0.50950, 0.50753, 0.50553, 0.50328, 0.50092, 0.49856, 0.49612, 0.49354, 0.49090,\
        0.48831, 0.48581, 0.48335, 0.48090, 0.47850, 0.47621, 0.47402, 0.47192, 0.46985, 0.46778, 0.46568,\
        0.46352, 0.46128, 0.45895, 0.45654, 0.45403, 0.45142, 0.44870, 0.44586, 0.44287, 0.43972, 0.43637,\
        0.43280, 0.42894, 0.42476, 0.42017, 0.41511, 0.40950, 0.40341, 0.39721, 0.39139, 0.38532, 0.37889],\
        [0.34451, 0.35083, 0.35692, 0.36274, 0.36827, 0.37349, 0.37836, 0.38285, 0.38688, 0.39037, 0.39329,\
        0.39555, 0.39696, 0.39763, 0.39803, 0.39863, 0.40004, 0.40157, 0.40267, 0.40317, 0.40297, 0.40194,\
        0.40026, 0.39810, 0.39620, 0.39482, 0.39423, 0.39462, 0.39603, 0.39883, 0.40302, 0.40812, 0.41380,\
        0.41983, 0.42604, 0.43232, 0.43860, 0.44480, 0.45087, 0.45678, 0.46250, 0.46802, 0.47333, 0.47847,\
        0.48348, 0.48840, 0.49325, 0.49813, 0.50311, 0.50744, 0.51127, 0.51467, 0.51758, 0.52003, 0.52221,\
        0.52430, 0.52635, 0.52834, 0.53020, 0.53186, 0.53327, 0.53444, 0.53534, 0.53598, 0.53636, 0.53648,\
        0.53632, 0.53589, 0.53518, 0.53419, 0.53296, 0.53153, 0.52996, 0.52829, 0.52656, 0.52475, 0.52281,\
        0.52075, 0.51864, 0.51653, 0.51440, 0.51220, 0.50985, 0.50740, 0.50490, 0.50229, 0.49953, 0.49669,\
        0.49393, 0.49136, 0.48877, 0.48611, 0.48352, 0.48112, 0.47886, 0.47670, 0.47458, 0.47246, 0.47028,\
        0.46803, 0.46570, 0.46327, 0.46074, 0.45813, 0.45542, 0.45261, 0.44969, 0.44662, 0.44340, 0.43997,\
        0.43632, 0.43238, 0.42811, 0.42346, 0.41835, 0.41279, 0.40687, 0.40109, 0.39558, 0.38966, 0.38347],\
        [0.34955, 0.35582, 0.36186, 0.36764, 0.37312, 0.37828, 0.38310, 0.38756, 0.39157, 0.39499, 0.39777,\
        0.39999, 0.40121, 0.40163, 0.40180, 0.40215, 0.40312, 0.40434, 0.40517, 0.40540, 0.40492, 0.40371,\
        0.40182, 0.39963, 0.39790, 0.39640, 0.39584, 0.39640, 0.39776, 0.40020, 0.40505, 0.41078, 0.41696,\
        0.42341, 0.42999, 0.43660, 0.44317, 0.44963, 0.45592, 0.46202, 0.46789, 0.47352, 0.47891, 0.48409,\
        0.48912, 0.49412, 0.49905, 0.50396, 0.50895, 0.51357, 0.51767, 0.52132, 0.52451, 0.52728, 0.52976,\
        0.53204, 0.53419, 0.53621, 0.53806, 0.53973, 0.54117, 0.54237, 0.54331, 0.54401, 0.54445, 0.54462,\
        0.54452, 0.54413, 0.54343, 0.54244, 0.54116, 0.53965, 0.53795, 0.53613, 0.53422, 0.53223, 0.53015,\
        0.52800, 0.52579, 0.52356, 0.52129, 0.51894, 0.51648, 0.51391, 0.51124, 0.50846, 0.50553, 0.50251,\
        0.49959, 0.49701, 0.49423, 0.49123, 0.48844, 0.48595, 0.48364, 0.48142, 0.47924, 0.47705, 0.47478,\
        0.47242, 0.46996, 0.46740, 0.46474, 0.46201, 0.45919, 0.45628, 0.45326, 0.45012, 0.44682, 0.44334,\
        0.43962, 0.43564, 0.43135, 0.42671, 0.42170, 0.41635, 0.41081, 0.40533, 0.39969, 0.39396, 0.38802],\
        [0.35429, 0.36051, 0.36651, 0.37224, 0.37767, 0.38276, 0.38750, 0.39188, 0.39589, 0.39929, 0.40188,\
        0.40390, 0.40513, 0.40568, 0.40595, 0.40631, 0.40697, 0.40774, 0.40825, 0.40825, 0.40762, 0.40633,\
        0.40450, 0.40248, 0.40070, 0.39905, 0.39875, 0.39974, 0.40160, 0.40429, 0.40898, 0.41481, 0.42117,\
        0.42781, 0.43459, 0.44139, 0.44815, 0.45479, 0.46125, 0.46748, 0.47345, 0.47916, 0.48462, 0.48985,\
        0.49485, 0.49989, 0.50502, 0.51005, 0.51503, 0.51975, 0.52408, 0.52798, 0.53144, 0.53451, 0.53724,\
        0.53971, 0.54197, 0.54404, 0.54592, 0.54760, 0.54907, 0.55031, 0.55130, 0.55204, 0.55254, 0.55278,\
        0.55275, 0.55241, 0.55174, 0.55073, 0.54939, 0.54777, 0.54594, 0.54395, 0.54187, 0.53972, 0.53750,\
        0.53524, 0.53294, 0.53060, 0.52820, 0.52573, 0.52315, 0.52046, 0.51765, 0.51471, 0.51165, 0.50854,\
        0.50552, 0.50272, 0.49956, 0.49620, 0.49330, 0.49078, 0.48841, 0.48611, 0.48386, 0.48159, 0.47921,\
        0.47671, 0.47408, 0.47135, 0.46855, 0.46567, 0.46273, 0.45972, 0.45662, 0.45340, 0.45004, 0.44651,\
        0.44276, 0.43878, 0.43451, 0.42995, 0.42510, 0.41999, 0.41473, 0.40936, 0.40388, 0.39838, 0.39260],\
        [0.35874, 0.36491, 0.37087, 0.37656, 0.38194, 0.38695, 0.39156, 0.39580, 0.39970, 0.40310, 0.40570,\
        0.40765, 0.40897, 0.40976, 0.41025, 0.41070, 0.41123, 0.41175, 0.41204, 0.41191, 0.41126, 0.41007,\
        0.40847, 0.40668, 0.40494, 0.40379, 0.40374, 0.40488, 0.40708, 0.41023, 0.41470, 0.42025, 0.42647,\
        0.43306, 0.43985, 0.44671, 0.45354, 0.46026, 0.46681, 0.47313, 0.47917, 0.48492, 0.49043, 0.49575,\
        0.50082, 0.50585, 0.51103, 0.51617, 0.52121, 0.52603, 0.53054, 0.53466, 0.53838, 0.54171, 0.54467,\
        0.54732, 0.54969, 0.55182, 0.55374, 0.55545, 0.55696, 0.55823, 0.55927, 0.56006, 0.56061, 0.56092,\
        0.56098, 0.56072, 0.56009, 0.55906, 0.55764, 0.55589, 0.55389, 0.55173, 0.54948, 0.54718, 0.54484,\
        0.54247, 0.54007, 0.53764, 0.53514, 0.53257, 0.52988, 0.52708, 0.52414, 0.52108, 0.51791, 0.51468,\
        0.51147, 0.50825, 0.50478, 0.50124, 0.49833, 0.49577, 0.49324, 0.49082, 0.48848, 0.48610, 0.48358,\
        0.48089, 0.47806, 0.47513, 0.47215, 0.46913, 0.46607, 0.46295, 0.45976, 0.45648, 0.45307, 0.44951,\
        0.44576, 0.44180, 0.43760, 0.43315, 0.42846, 0.42357, 0.41854, 0.41339, 0.40815, 0.40279, 0.39709],\
        [0.36289, 0.36901, 0.37493, 0.38061, 0.38595, 0.39090, 0.39541, 0.39949, 0.40326, 0.40659, 0.40927,\
        0.41132, 0.41282, 0.41387, 0.41463, 0.41524, 0.41580, 0.41625, 0.41649, 0.41638, 0.41586, 0.41493,\
        0.41368, 0.41229, 0.41106, 0.41037, 0.41058, 0.41182, 0.41407, 0.41730, 0.42158, 0.42679, 0.43269,\
        0.43905, 0.44569, 0.45246, 0.45926, 0.46598, 0.47255, 0.47890, 0.48497, 0.49071, 0.49621, 0.50155,\
        0.50676, 0.51189, 0.51708, 0.52228, 0.52740, 0.53234, 0.53702, 0.54136, 0.54533, 0.54889, 0.55206,\
        0.55486, 0.55733, 0.55953, 0.56149, 0.56325, 0.56480, 0.56613, 0.56721, 0.56804, 0.56864, 0.56902,\
        0.56918, 0.56903, 0.56847, 0.56742, 0.56589, 0.56396, 0.56176, 0.55942, 0.55701, 0.55457, 0.55212,\
        0.54966, 0.54719, 0.54468, 0.54211, 0.53946, 0.53668, 0.53378, 0.53074, 0.52756, 0.52427, 0.52091,\
        0.51748, 0.51398, 0.51037, 0.50686, 0.50376, 0.50093, 0.49814, 0.49555, 0.49310, 0.49062, 0.48790,\
        0.48495, 0.48187, 0.47872, 0.47556, 0.47238, 0.46920, 0.46599, 0.46273, 0.45939, 0.45595, 0.45237,\
        0.44864, 0.44471, 0.44059, 0.43626, 0.43173, 0.42702, 0.42217, 0.41721, 0.41212, 0.40686, 0.40123],\
        [0.36673, 0.37280, 0.37870, 0.38437, 0.38972, 0.39466, 0.39911, 0.40311, 0.40677, 0.41002, 0.41277,\
        0.41498, 0.41670, 0.41803, 0.41907, 0.41992, 0.42063, 0.42119, 0.42153, 0.42160, 0.42134, 0.42077,\
        0.41997, 0.41911, 0.41842, 0.41818, 0.41864, 0.41995, 0.42215, 0.42525, 0.42925, 0.43407, 0.43957,\
        0.44558, 0.45194, 0.45853, 0.46519, 0.47183, 0.47835, 0.48468, 0.49074, 0.49645, 0.50189, 0.50723,\
        0.51250, 0.51775, 0.52302, 0.52830, 0.53352, 0.53860, 0.54346, 0.54803, 0.55224, 0.55605, 0.55942,\
        0.56236, 0.56491, 0.56716, 0.56916, 0.57097, 0.57259, 0.57399, 0.57512, 0.57596, 0.57658, 0.57703,\
        0.57731, 0.57732, 0.57686, 0.57579, 0.57410, 0.57194, 0.56952, 0.56699, 0.56443, 0.56187, 0.55933,\
        0.55680, 0.55427, 0.55172, 0.54911, 0.54639, 0.54355, 0.54057, 0.53743, 0.53415, 0.53074, 0.52723,\
        0.52363, 0.51996, 0.51626, 0.51268, 0.50933, 0.50618, 0.50316, 0.50034, 0.49774, 0.49514, 0.49212,\
        0.48884, 0.48548, 0.48210, 0.47875, 0.47543, 0.47214, 0.46884, 0.46552, 0.46214, 0.45868, 0.45510,\
        0.45139, 0.44751, 0.44346, 0.43924, 0.43484, 0.43027, 0.42556, 0.42071, 0.41569, 0.41041, 0.40473],\
        [0.37027, 0.37627, 0.38215, 0.38783, 0.39319, 0.39815, 0.40259, 0.40656, 0.41017, 0.41342, 0.41625,\
        0.41864, 0.42061, 0.42224, 0.42358, 0.42471, 0.42567, 0.42646, 0.42706, 0.42742, 0.42753, 0.42742,\
        0.42714, 0.42683, 0.42665, 0.42681, 0.42750, 0.42885, 0.43093, 0.43379, 0.43742, 0.44179, 0.44682,\
        0.45240, 0.45841, 0.46472, 0.47118, 0.47765, 0.48403, 0.49028, 0.49632, 0.50196, 0.50734, 0.51266,\
        0.51798, 0.52334, 0.52872, 0.53412, 0.53948, 0.54473, 0.54981, 0.55462, 0.55910, 0.56316, 0.56673,\
        0.56981, 0.57242, 0.57469, 0.57671, 0.57858, 0.58030, 0.58181, 0.58301, 0.58384, 0.58441, 0.58489,\
        0.58532, 0.58554, 0.58524, 0.58413, 0.58222, 0.57977, 0.57711, 0.57440, 0.57171, 0.56907, 0.56647,\
        0.56389, 0.56133, 0.55876, 0.55613, 0.55339, 0.55049, 0.54743, 0.54420, 0.54083, 0.53731, 0.53366,\
        0.52992, 0.52611, 0.52230, 0.51856, 0.51497, 0.51155, 0.50829, 0.50523, 0.50238, 0.49954, 0.49606,\
        0.49247, 0.48882, 0.48523, 0.48172, 0.47829, 0.47491, 0.47154, 0.46816, 0.46475, 0.46128, 0.45771,\
        0.45402, 0.45019, 0.44621, 0.44207, 0.43777, 0.43330, 0.42868, 0.42388, 0.41887, 0.41356, 0.40785],\
        [0.37350, 0.37941, 0.38526, 0.39096, 0.39635, 0.40125, 0.40571, 0.40974, 0.41341, 0.41674, 0.41971,\
        0.42231, 0.42455, 0.42648, 0.42815, 0.42961, 0.43089, 0.43200, 0.43294, 0.43370, 0.43426, 0.43466,\
        0.43493, 0.43517, 0.43548, 0.43600, 0.43687, 0.43821, 0.44011, 0.44264, 0.44583, 0.44969, 0.45419,\
        0.45927, 0.46485, 0.47082, 0.47704, 0.48330, 0.48941, 0.49541, 0.50136, 0.50693, 0.51233, 0.51770,\
        0.52312, 0.52860, 0.53413, 0.53969, 0.54522, 0.55067, 0.55597, 0.56105, 0.56583, 0.57019, 0.57400,\
        0.57720, 0.57985, 0.58210, 0.58414, 0.58608, 0.58792, 0.58959, 0.59090, 0.59169, 0.59214, 0.59260,\
        0.59318, 0.59363, 0.59353, 0.59238, 0.59013, 0.58735, 0.58447, 0.58162, 0.57887, 0.57619, 0.57354,\
        0.57093, 0.56836, 0.56581, 0.56319, 0.56043, 0.55749, 0.55436, 0.55105, 0.54758, 0.54395, 0.54019,\
        0.53632, 0.53237, 0.52839, 0.52447, 0.52064, 0.51693, 0.51338, 0.50998, 0.50670, 0.50329, 0.49958,\
        0.49579, 0.49187, 0.48811, 0.48451, 0.48099, 0.47753, 0.47410, 0.47069, 0.46725, 0.46376, 0.46020,\
        0.45654, 0.45275, 0.44882, 0.44475, 0.44051, 0.43611, 0.43154, 0.42676, 0.42173, 0.41640, 0.41068],\
        [0.37646, 0.38223, 0.38798, 0.39368, 0.39917, 0.40405, 0.40852, 0.41266, 0.41647, 0.41997, 0.42313,\
        0.42597, 0.42850, 0.43075, 0.43276, 0.43457, 0.43621, 0.43771, 0.43906, 0.44026, 0.44134, 0.44228,\
        0.44312, 0.44390, 0.44466, 0.44550, 0.44650, 0.44778, 0.44944, 0.45158, 0.45427, 0.45755, 0.46144,\
        0.46594, 0.47102, 0.47659, 0.48255, 0.48863, 0.49446, 0.50000, 0.50571, 0.51129, 0.51677, 0.52227,\
        0.52784, 0.53349, 0.53921, 0.54496, 0.55068, 0.55634, 0.56188, 0.56725, 0.57236, 0.57708, 0.58120,\
        0.58455, 0.58720, 0.58941, 0.59146, 0.59346, 0.59544, 0.59730, 0.59878, 0.59955, 0.59988, 0.60030,\
        0.60094, 0.60148, 0.60144, 0.60022, 0.59764, 0.59462, 0.59163, 0.58870, 0.58594, 0.58326, 0.58057,\
        0.57791, 0.57536, 0.57286, 0.57030, 0.56754, 0.56455, 0.56134, 0.55794, 0.55438, 0.55066, 0.54678,\
        0.54277, 0.53866, 0.53451, 0.53036, 0.52626, 0.52226, 0.51835, 0.51453, 0.51074, 0.50686, 0.50288,\
        0.49870, 0.49457, 0.49082, 0.48717, 0.48358, 0.48005, 0.47656, 0.47311, 0.46965, 0.46616, 0.46260,\
        0.45896, 0.45520, 0.45131, 0.44727, 0.44308, 0.43872, 0.43416, 0.42938, 0.42435, 0.41900, 0.41328],\
        [0.37921, 0.38478, 0.39036, 0.39591, 0.40133, 0.40640, 0.41106, 0.41539, 0.41941, 0.42312, 0.42652,\
        0.42963, 0.43246, 0.43503, 0.43738, 0.43956, 0.44158, 0.44348, 0.44527, 0.44696, 0.44856, 0.45007,\
        0.45148, 0.45279, 0.45400, 0.45511, 0.45619, 0.45735, 0.45872, 0.46040, 0.46252, 0.46516, 0.46838,\
        0.47222, 0.47669, 0.48178, 0.48739, 0.49338, 0.49915, 0.50429, 0.50964, 0.51513, 0.52070, 0.52636,\
        0.53213, 0.53799, 0.54393, 0.54990, 0.55584, 0.56171, 0.56748, 0.57310, 0.57856, 0.58371, 0.58827,\
        0.59183, 0.59444, 0.59663, 0.59873, 0.60085, 0.60294, 0.60490, 0.60643, 0.60735, 0.60784, 0.60829,\
        0.60878, 0.60905, 0.60871, 0.60731, 0.60477, 0.60177, 0.59874, 0.59572, 0.59301, 0.59041, 0.58758,\
        0.58482, 0.58232, 0.57995, 0.57748, 0.57472, 0.57165, 0.56833, 0.56484, 0.56120, 0.55739, 0.55341,\
        0.54925, 0.54497, 0.54060, 0.53621, 0.53183, 0.52750, 0.52322, 0.51897, 0.51471, 0.51036, 0.50587,\
        0.50123, 0.49727, 0.49359, 0.48983, 0.48611, 0.48250, 0.47896, 0.47547, 0.47199, 0.46849, 0.46493,\
        0.46129, 0.45755, 0.45368, 0.44967, 0.44549, 0.44114, 0.43659, 0.43180, 0.42675, 0.42139, 0.41568],\
        [0.38185, 0.38717, 0.39253, 0.39796, 0.40335, 0.40855, 0.41345, 0.41802, 0.42228, 0.42623, 0.42990,\
        0.43328, 0.43640, 0.43929, 0.44199, 0.44452, 0.44692, 0.44922, 0.45145, 0.45362, 0.45574, 0.45781,\
        0.45980, 0.46166, 0.46329, 0.46465, 0.46577, 0.46675, 0.46775, 0.46893, 0.47043, 0.47237, 0.47484,\
        0.47795, 0.48172, 0.48618, 0.49125, 0.49682, 0.50247, 0.50766, 0.51293, 0.51844, 0.52414, 0.53000,\
        0.53599, 0.54211, 0.54830, 0.55452, 0.56070, 0.56678, 0.57272, 0.57853, 0.58426, 0.58984, 0.59497,\
        0.59888, 0.60153, 0.60385, 0.60612, 0.60837, 0.61054, 0.61250, 0.61408, 0.61515, 0.61584, 0.61630,\
        0.61658, 0.61652, 0.61584, 0.61430, 0.61192, 0.60905, 0.60599, 0.60291, 0.60026, 0.59771, 0.59453,\
        0.59165, 0.58926, 0.58710, 0.58477, 0.58197, 0.57874, 0.57527, 0.57170, 0.56801, 0.56413, 0.56004,\
        0.55574, 0.55126, 0.54666, 0.54201, 0.53734, 0.53268, 0.52804, 0.52340, 0.51874, 0.51401, 0.50925,\
        0.50465, 0.50057, 0.49656, 0.49249, 0.48862, 0.48492, 0.48133, 0.47780, 0.47429, 0.47077, 0.46720,\
        0.46356, 0.45982, 0.45595, 0.45194, 0.44777, 0.44342, 0.43885, 0.43405, 0.42899, 0.42362, 0.41792],\
        [0.38454, 0.38960, 0.39469, 0.40002, 0.40544, 0.41078, 0.41587, 0.42067, 0.42517, 0.42937, 0.43329,\
        0.43693, 0.44034, 0.44353, 0.44654, 0.44940, 0.45215, 0.45483, 0.45747, 0.46008, 0.46269, 0.46530,\
        0.46786, 0.47026, 0.47235, 0.47395, 0.47504, 0.47578, 0.47637, 0.47701, 0.47786, 0.47905, 0.48074,\
        0.48304, 0.48606, 0.48980, 0.49419, 0.49915, 0.50453, 0.50999, 0.51550, 0.52121, 0.52712, 0.53322,\
        0.53948, 0.54587, 0.55236, 0.55886, 0.56530, 0.57159, 0.57765, 0.58352, 0.58936, 0.59518, 0.60069,\
        0.60508, 0.60825, 0.61096, 0.61349, 0.61591, 0.61818, 0.62018, 0.62182, 0.62302, 0.62381, 0.62426,\
        0.62437, 0.62404, 0.62311, 0.62148, 0.61919, 0.61644, 0.61346, 0.61048, 0.60768, 0.60478, 0.60146,\
        0.59858, 0.59630, 0.59437, 0.59220, 0.58923, 0.58570, 0.58207, 0.57846, 0.57478, 0.57088, 0.56667,\
        0.56219, 0.55749, 0.55265, 0.54773, 0.54277, 0.53781, 0.53284, 0.52786, 0.52288, 0.51789, 0.51296,\
        0.50823, 0.50378, 0.49933, 0.49509, 0.49114, 0.48737, 0.48372, 0.48014, 0.47659, 0.47304, 0.46945,\
        0.46578, 0.46203, 0.45815, 0.45412, 0.44994, 0.44556, 0.44098, 0.43616, 0.43107, 0.42570, 0.42000],\
        [0.38742, 0.39231, 0.39712, 0.40227, 0.40781, 0.41325, 0.41849, 0.42347, 0.42816, 0.43258, 0.43672,\
        0.44061, 0.44427, 0.44772, 0.45101, 0.45417, 0.45723, 0.46024, 0.46322, 0.46621, 0.46924, 0.47231,\
        0.47540, 0.47837, 0.48095, 0.48280, 0.48381, 0.48422, 0.48439, 0.48451, 0.48471, 0.48515, 0.48601,\
        0.48748, 0.48973, 0.49280, 0.49657, 0.50109, 0.50628, 0.51182, 0.51759, 0.52356, 0.52974, 0.53610,\
        0.54265, 0.54934, 0.55612, 0.56295, 0.56971, 0.57626, 0.58243, 0.58822, 0.59396, 0.59991, 0.60562,\
        0.61052, 0.61445, 0.61776, 0.62069, 0.62338, 0.62580, 0.62792, 0.62965, 0.63094, 0.63179, 0.63221,\
        0.63219, 0.63166, 0.63057, 0.62887, 0.62662, 0.62396, 0.62106, 0.61809, 0.61511, 0.61203, 0.60886,\
        0.60602, 0.60366, 0.60172, 0.59958, 0.59626, 0.59235, 0.58863, 0.58511, 0.58153, 0.57762, 0.57328,\
        0.56858, 0.56363, 0.55853, 0.55335, 0.54812, 0.54288, 0.53763, 0.53237, 0.52712, 0.52190, 0.51677,\
        0.51177, 0.50692, 0.50219, 0.49782, 0.49377, 0.48991, 0.48618, 0.48253, 0.47892, 0.47532, 0.47168,\
        0.46798, 0.46419, 0.46028, 0.45623, 0.45201, 0.44761, 0.44299, 0.43815, 0.43304, 0.42765, 0.42196],\
        [0.39059, 0.39548, 0.40031, 0.40532, 0.41069, 0.41611, 0.42139, 0.42648, 0.43131, 0.43589, 0.44022,\
        0.44431, 0.44819, 0.45187, 0.45539, 0.45879, 0.46211, 0.46538, 0.46863, 0.47191, 0.47523, 0.47864,\
        0.48214, 0.48565, 0.48882, 0.49097, 0.49178, 0.49182, 0.49161, 0.49129, 0.49091, 0.49062, 0.49067,\
        0.49128, 0.49274, 0.49528, 0.49873, 0.50286, 0.50789, 0.51350, 0.51947, 0.52569, 0.53213, 0.53876,\
        0.54557, 0.55254, 0.55964, 0.56680, 0.57393, 0.58088, 0.58734, 0.59301, 0.59841, 0.60459, 0.61060,\
        0.61594, 0.62049, 0.62437, 0.62776, 0.63077, 0.63342, 0.63570, 0.63755, 0.63893, 0.63983, 0.64022,\
        0.64010, 0.63943, 0.63821, 0.63645, 0.63420, 0.63159, 0.62873, 0.62575, 0.62270, 0.61960, 0.61654,\
        0.61370, 0.61120, 0.60888, 0.60626, 0.60269, 0.59868, 0.59503, 0.59172, 0.58830, 0.58436, 0.57982,\
        0.57484, 0.56961, 0.56426, 0.55884, 0.55338, 0.54789, 0.54240, 0.53691, 0.53143, 0.52600, 0.52065,\
        0.51542, 0.51034, 0.50545, 0.50087, 0.49661, 0.49260, 0.48874, 0.48499, 0.48129, 0.47762, 0.47393,\
        0.47018, 0.46634, 0.46238, 0.45828, 0.45401, 0.44956, 0.44491, 0.44003, 0.43490, 0.42950, 0.42381],\
        [0.39392, 0.39896, 0.40380, 0.40878, 0.41400, 0.41933, 0.42460, 0.42973, 0.43465, 0.43935, 0.44382,\
        0.44806, 0.45210, 0.45596, 0.45966, 0.46325, 0.46675, 0.47021, 0.47366, 0.47711, 0.48059, 0.48414,\
        0.48782, 0.49168, 0.49546, 0.49805, 0.49852, 0.49826, 0.49785, 0.49728, 0.49640, 0.49549, 0.49480,\
        0.49454, 0.49508, 0.49697, 0.50039, 0.50442, 0.50946, 0.51520, 0.52136, 0.52780, 0.53446, 0.54131,\
        0.54834, 0.55556, 0.56293, 0.57040, 0.57790, 0.58534, 0.59250, 0.59850, 0.60382, 0.60988, 0.61600,\
        0.62165, 0.62664, 0.63100, 0.63481, 0.63815, 0.64106, 0.64353, 0.64553, 0.64702, 0.64796, 0.64833,\
        0.64812, 0.64735, 0.64602, 0.64419, 0.64192, 0.63931, 0.63647, 0.63348, 0.63041, 0.62732, 0.62427,\
        0.62137, 0.61861, 0.61585, 0.61278, 0.60916, 0.60528, 0.60158, 0.59846, 0.59511, 0.59103, 0.58616,\
        0.58084, 0.57534, 0.56978, 0.56417, 0.55852, 0.55284, 0.54714, 0.54145, 0.53578, 0.53016, 0.52462,\
        0.51921, 0.51395, 0.50890, 0.50412, 0.49964, 0.49545, 0.49142, 0.48752, 0.48372, 0.47997, 0.47620,\
        0.47238, 0.46847, 0.46444, 0.46028, 0.45595, 0.45145, 0.44675, 0.44183, 0.43667, 0.43125, 0.42555],\
        [0.39708, 0.40233, 0.40730, 0.41236, 0.41757, 0.42284, 0.42809, 0.43322, 0.43818, 0.44295, 0.44751,\
        0.45186, 0.45602, 0.46000, 0.46383, 0.46754, 0.47116, 0.47474, 0.47830, 0.48184, 0.48536, 0.48885,\
        0.49237, 0.49610, 0.50005, 0.50284, 0.50361, 0.50347, 0.50302, 0.50228, 0.50113, 0.49981, 0.49860,\
        0.49759, 0.49711, 0.49802, 0.50103, 0.50574, 0.51119, 0.51716, 0.52350, 0.53009, 0.53689, 0.54387,\
        0.55105, 0.55844, 0.56603, 0.57377, 0.58154, 0.58924, 0.59694, 0.60384, 0.60966, 0.61574, 0.62190,\
        0.62774, 0.63305, 0.63777, 0.64194, 0.64558, 0.64874, 0.65143, 0.65360, 0.65522, 0.65622, 0.65657,\
        0.65629, 0.65540, 0.65397, 0.65206, 0.64973, 0.64710, 0.64424, 0.64125, 0.63817, 0.63507, 0.63198,\
        0.62894, 0.62594, 0.62285, 0.61951, 0.61588, 0.61210, 0.60849, 0.60515, 0.60166, 0.59736, 0.59207,\
        0.58641, 0.58075, 0.57508, 0.56934, 0.56354, 0.55770, 0.55184, 0.54597, 0.54014, 0.53435, 0.52866,\
        0.52308, 0.51767, 0.51247, 0.50752, 0.50285, 0.49845, 0.49419, 0.49013, 0.48622, 0.48237, 0.47851,\
        0.47460, 0.47061, 0.46649, 0.46225, 0.45785, 0.45328, 0.44852, 0.44355, 0.43836, 0.43291, 0.42720],\
        [0.40008, 0.40547, 0.41083, 0.41610, 0.42135, 0.42661, 0.43182, 0.43693, 0.44190, 0.44669, 0.45130,\
        0.45571, 0.45994, 0.46399, 0.46788, 0.47165, 0.47534, 0.47898, 0.48259, 0.48617, 0.48967, 0.49302,\
        0.49620, 0.49940, 0.50306, 0.50591, 0.50728, 0.50756, 0.50722, 0.50642, 0.50521, 0.50375, 0.50228,\
        0.50086, 0.49977, 0.50009, 0.50322, 0.50800, 0.51361, 0.51969, 0.52609, 0.53273, 0.53956, 0.54657,\
        0.55379, 0.56127, 0.56901, 0.57698, 0.58501, 0.59275, 0.60028, 0.60798, 0.61499, 0.62165, 0.62809,\
        0.63419, 0.63977, 0.64478, 0.64922, 0.65313, 0.65652, 0.65942, 0.66178, 0.66355, 0.66462, 0.66496,\
        0.66458, 0.66358, 0.66203, 0.66002, 0.65762, 0.65493, 0.65204, 0.64903, 0.64594, 0.64280, 0.63964,\
        0.63646, 0.63324, 0.62990, 0.62638, 0.62268, 0.61889, 0.61513, 0.61141, 0.60744, 0.60279, 0.59720,\
        0.59142, 0.58586, 0.58019, 0.57436, 0.56844, 0.56247, 0.55647, 0.55047, 0.54449, 0.53856, 0.53272,\
        0.52700, 0.52145, 0.51610, 0.51098, 0.50611, 0.50144, 0.49698, 0.49280, 0.48878, 0.48484, 0.48088,\
        0.47686, 0.47275, 0.46853, 0.46418, 0.45969, 0.45505, 0.45023, 0.44521, 0.43997, 0.43450, 0.42877],\
        [0.40456, 0.40958, 0.41488, 0.42017, 0.42542, 0.43063, 0.43579, 0.44084, 0.44578, 0.45056, 0.45517,\
        0.45960, 0.46386, 0.46793, 0.47184, 0.47562, 0.47930, 0.48293, 0.48656, 0.49016, 0.49367, 0.49696,\
        0.50001, 0.50293, 0.50608, 0.50872, 0.51033, 0.51095, 0.51082, 0.51012, 0.50900, 0.50759, 0.50610,\
        0.50471, 0.50384, 0.50443, 0.50722, 0.51164, 0.51704, 0.52300, 0.52931, 0.53586, 0.54260, 0.54953,\
        0.55667, 0.56410, 0.57189, 0.58008, 0.58857, 0.59687, 0.60463, 0.61260, 0.62032, 0.62764, 0.63454,\
        0.64096, 0.64681, 0.65206, 0.65672, 0.66083, 0.66442, 0.66751, 0.67008, 0.67202, 0.67319, 0.67349,\
        0.67299, 0.67185, 0.67018, 0.66805, 0.66554, 0.66277, 0.65984, 0.65680, 0.65368, 0.65049, 0.64724,\
        0.64393, 0.64052, 0.63698, 0.63329, 0.62945, 0.62549, 0.62145, 0.61727, 0.61273, 0.60756, 0.60172,\
        0.59617, 0.59093, 0.58520, 0.57924, 0.57322, 0.56715, 0.56104, 0.55491, 0.54880, 0.54274, 0.53677,\
        0.53093, 0.52524, 0.51975, 0.51446, 0.50938, 0.50450, 0.49987, 0.49557, 0.49145, 0.48739, 0.48329,\
        0.47913, 0.47488, 0.47054, 0.46608, 0.46149, 0.45677, 0.45187, 0.44679, 0.44151, 0.43601, 0.43027],\
        [0.40983, 0.41454, 0.41956, 0.42468, 0.42981, 0.43491, 0.43996, 0.44493, 0.44980, 0.45453, 0.45912,\
        0.46353, 0.46778, 0.47184, 0.47573, 0.47946, 0.48307, 0.48663, 0.49021, 0.49382, 0.49739, 0.50067,\
        0.50367, 0.50652, 0.50928, 0.51164, 0.51327, 0.51408, 0.51417, 0.51368, 0.51276, 0.51158, 0.51034,\
        0.50931, 0.50895, 0.50986, 0.51242, 0.51642, 0.52144, 0.52712, 0.53321, 0.53956, 0.54611, 0.55284,\
        0.55979, 0.56704, 0.57471, 0.58294, 0.59179, 0.60092, 0.60932, 0.61769, 0.62598, 0.63391, 0.64131,\
        0.64809, 0.65421, 0.65966, 0.66448, 0.66872, 0.67245, 0.67570, 0.67846, 0.68063, 0.68194, 0.68217,\
        0.68146, 0.68017, 0.67840, 0.67613, 0.67347, 0.67058, 0.66761, 0.66455, 0.66141, 0.65816, 0.65481,\
        0.65135, 0.64778, 0.64406, 0.64017, 0.63613, 0.63194, 0.62757, 0.62298, 0.61801, 0.61259, 0.60687,\
        0.60158, 0.59615, 0.59004, 0.58396, 0.57789, 0.57175, 0.56554, 0.55929, 0.55306, 0.54688, 0.54079,\
        0.53482, 0.52901, 0.52338, 0.51795, 0.51273, 0.50772, 0.50297, 0.49850, 0.49422, 0.49001, 0.48575,\
        0.48142, 0.47700, 0.47251, 0.46793, 0.46324, 0.45842, 0.45346, 0.44832, 0.44299, 0.43745, 0.43168],\
        [0.41554, 0.42000, 0.42472, 0.42959, 0.43451, 0.43943, 0.44433, 0.44917, 0.45393, 0.45858, 0.46311,\
        0.46749, 0.47169, 0.47571, 0.47955, 0.48321, 0.48670, 0.49010, 0.49357, 0.49714, 0.50070, 0.50405,\
        0.50709, 0.50991, 0.51250, 0.51469, 0.51629, 0.51723, 0.51753, 0.51729, 0.51666, 0.51582, 0.51497,\
        0.51442, 0.51455, 0.51573, 0.51820, 0.52190, 0.52657, 0.53193, 0.53775, 0.54386, 0.55016, 0.55662,\
        0.56329, 0.57025, 0.57764, 0.58565, 0.59448, 0.60398, 0.61358, 0.62295, 0.63202, 0.64060, 0.64852,\
        0.65567, 0.66203, 0.66762, 0.67253, 0.67685, 0.68064, 0.68398, 0.68689, 0.68931, 0.69087, 0.69094,\
        0.68989, 0.68847, 0.68669, 0.68427, 0.68137, 0.67835, 0.67535, 0.67230, 0.66912, 0.66580, 0.66234,\
        0.65875, 0.65500, 0.65109, 0.64700, 0.64273, 0.63828, 0.63361, 0.62868, 0.62345, 0.61791, 0.61226,\
        0.60667, 0.60072, 0.59453, 0.58852, 0.58249, 0.57628, 0.56996, 0.56359, 0.55723, 0.55093, 0.54473,\
        0.53865, 0.53273, 0.52698, 0.52144, 0.51610, 0.51099, 0.50613, 0.50150, 0.49707, 0.49269, 0.48821,\
        0.48367, 0.47907, 0.47442, 0.46972, 0.46492, 0.46002, 0.45498, 0.44978, 0.44440, 0.43882, 0.43303],\
        [0.42154, 0.42577, 0.43020, 0.43478, 0.43944, 0.44413, 0.44882, 0.45349, 0.45812, 0.46267, 0.46712,\
        0.47143, 0.47558, 0.47955, 0.48333, 0.48692, 0.49026, 0.49341, 0.49670, 0.50024, 0.50381, 0.50721,\
        0.51035, 0.51320, 0.51573, 0.51786, 0.51947, 0.52052, 0.52102, 0.52106, 0.52075, 0.52029, 0.51987,\
        0.51979, 0.52032, 0.52175, 0.52424, 0.52776, 0.53217, 0.53727, 0.54283, 0.54871, 0.55476, 0.56096,\
        0.56732, 0.57398, 0.58106, 0.58874, 0.59752, 0.60764, 0.61824, 0.62865, 0.63858, 0.64783, 0.65627,\
        0.66378, 0.67032, 0.67598, 0.68094, 0.68529, 0.68908, 0.69241, 0.69537, 0.69791, 0.69977, 0.69962,\
        0.69821, 0.69679, 0.69515, 0.69244, 0.68919, 0.68605, 0.68310, 0.68008, 0.67685, 0.67342, 0.66984,\
        0.66610, 0.66219, 0.65808, 0.65377, 0.64926, 0.64455, 0.63961, 0.63442, 0.62897, 0.62328, 0.61745,\
        0.61146, 0.60521, 0.59896, 0.59317, 0.58712, 0.58076, 0.57428, 0.56777, 0.56129, 0.55488, 0.54857,\
        0.54239, 0.53636, 0.53050, 0.52485, 0.51941, 0.51420, 0.50924, 0.50450, 0.49994, 0.49533, 0.49059,\
        0.48581, 0.48103, 0.47625, 0.47143, 0.46654, 0.46155, 0.45644, 0.45118, 0.44575, 0.44014, 0.43432],\
        [0.42772, 0.43172, 0.43587, 0.44015, 0.44451, 0.44893, 0.45338, 0.45785, 0.46232, 0.46675, 0.47111,\
        0.47535, 0.47943, 0.48332, 0.48703, 0.49060, 0.49390, 0.49674, 0.49965, 0.50328, 0.50691, 0.51038,\
        0.51358, 0.51648, 0.51903, 0.52116, 0.52283, 0.52400, 0.52470, 0.52500, 0.52502, 0.52492, 0.52491,\
        0.52523, 0.52610, 0.52775, 0.53029, 0.53373, 0.53799, 0.54291, 0.54832, 0.55403, 0.55991, 0.56588,\
        0.57200, 0.57844, 0.58542, 0.59304, 0.60175, 0.61259, 0.62397, 0.63519, 0.64586, 0.65572, 0.66463,\
        0.67248, 0.67917, 0.68479, 0.68973, 0.69413, 0.69791, 0.70119, 0.70412, 0.70655, 0.70809, 0.70804,\
        0.70684, 0.70535, 0.70354, 0.70049, 0.69696, 0.69378, 0.69101, 0.68799, 0.68462, 0.68102, 0.67728,\
        0.67341, 0.66932, 0.66500, 0.66046, 0.65572, 0.65077, 0.64559, 0.64016, 0.63450, 0.62863, 0.62259,\
        0.61640, 0.61012, 0.60400, 0.59811, 0.59175, 0.58509, 0.57843, 0.57180, 0.56522, 0.55870, 0.55229,\
        0.54600, 0.53987, 0.53391, 0.52815, 0.52260, 0.51727, 0.51217, 0.50729, 0.50257, 0.49774, 0.49273,\
        0.48776, 0.48285, 0.47795, 0.47304, 0.46807, 0.46301, 0.45784, 0.45252, 0.44704, 0.44138, 0.43553],\
        [0.43399, 0.43776, 0.44163, 0.44559, 0.44963, 0.45375, 0.45794, 0.46218, 0.46646, 0.47075, 0.47501,\
        0.47919, 0.48320, 0.48700, 0.49058, 0.49408, 0.49764, 0.50043, 0.50317, 0.50660, 0.51020, 0.51368,\
        0.51693, 0.51987, 0.52245, 0.52463, 0.52638, 0.52768, 0.52856, 0.52910, 0.52941, 0.52964, 0.52997,\
        0.53060, 0.53175, 0.53357, 0.53618, 0.53961, 0.54381, 0.54865, 0.55400, 0.55968, 0.56552, 0.57138,\
        0.57731, 0.58361, 0.59078, 0.59904, 0.60833, 0.61921, 0.63094, 0.64269, 0.65393, 0.66429, 0.67357,\
        0.68174, 0.68863, 0.69406, 0.69892, 0.70343, 0.70724, 0.71051, 0.71329, 0.71543, 0.71664, 0.71666,\
        0.71567, 0.71405, 0.71179, 0.70870, 0.70514, 0.70197, 0.69922, 0.69601, 0.69239, 0.68853, 0.68465,\
        0.68068, 0.67641, 0.67184, 0.66707, 0.66210, 0.65693, 0.65152, 0.64588, 0.64001, 0.63394, 0.62772,\
        0.62139, 0.61506, 0.60884, 0.60263, 0.59586, 0.58911, 0.58238, 0.57566, 0.56898, 0.56237, 0.55586,\
        0.54947, 0.54324, 0.53718, 0.53130, 0.52561, 0.52013, 0.51485, 0.50975, 0.50473, 0.49967, 0.49455,\
        0.48950, 0.48451, 0.47955, 0.47458, 0.46954, 0.46442, 0.45918, 0.45380, 0.44827, 0.44257, 0.43669],\
        [0.44029, 0.44382, 0.44739, 0.45103, 0.45474, 0.45853, 0.46241, 0.46639, 0.47046, 0.47460, 0.47877,\
        0.48290, 0.48688, 0.49061, 0.49398, 0.49706, 0.50056, 0.50386, 0.50689, 0.51021, 0.51372, 0.51717,\
        0.52043, 0.52340, 0.52603, 0.52827, 0.53011, 0.53154, 0.53259, 0.53334, 0.53388, 0.53436, 0.53494,\
        0.53581, 0.53714, 0.53908, 0.54176, 0.54519, 0.54938, 0.55424, 0.55965, 0.56545, 0.57143, 0.57738,\
        0.58320, 0.58929, 0.59657, 0.60550, 0.61559, 0.62683, 0.63884, 0.65099, 0.66272, 0.67347, 0.68292,\
        0.69125, 0.69861, 0.70396, 0.70871, 0.71314, 0.71696, 0.72016, 0.72275, 0.72462, 0.72560, 0.72557,\
        0.72461, 0.72288, 0.72043, 0.71735, 0.71394, 0.71064, 0.70745, 0.70398, 0.70007, 0.69588, 0.69194,\
        0.68797, 0.68340, 0.67854, 0.67354, 0.66838, 0.66301, 0.65739, 0.65153, 0.64545, 0.63918, 0.63277,\
        0.62627, 0.61974, 0.61320, 0.60649, 0.59964, 0.59293, 0.58616, 0.57935, 0.57257, 0.56587, 0.55926,\
        0.55278, 0.54645, 0.54028, 0.53428, 0.52845, 0.52280, 0.51731, 0.51194, 0.50664, 0.50136, 0.49615,\
        0.49106, 0.48606, 0.48107, 0.47605, 0.47095, 0.46576, 0.46046, 0.45502, 0.44944, 0.44370, 0.43778],\
        [0.44658, 0.44985, 0.45311, 0.45640, 0.45975, 0.46318, 0.46672, 0.47041, 0.47424, 0.47821, 0.48229,\
        0.48640, 0.49039, 0.49415, 0.49753, 0.50030, 0.50344, 0.50701, 0.51043, 0.51389, 0.51739, 0.52083,\
        0.52410, 0.52711, 0.52978, 0.53209, 0.53402, 0.53556, 0.53675, 0.53765, 0.53836, 0.53901, 0.53975,\
        0.54074, 0.54216, 0.54416, 0.54684, 0.55028, 0.55448, 0.55940, 0.56494, 0.57098, 0.57733, 0.58369,\
        0.58964, 0.59549, 0.60283, 0.61229, 0.62306, 0.63478, 0.64715, 0.65971, 0.67192, 0.68315, 0.69260,\
        0.70068, 0.70806, 0.71395, 0.71885, 0.72318, 0.72691, 0.73000, 0.73242, 0.73407, 0.73487, 0.73475,\
        0.73373, 0.73190, 0.72937, 0.72628, 0.72286, 0.71933, 0.71572, 0.71185, 0.70765, 0.70318, 0.69939,\
        0.69525, 0.69016, 0.68501, 0.67984, 0.67454, 0.66900, 0.66317, 0.65709, 0.65077, 0.64429, 0.63767,\
        0.63096, 0.62420, 0.61738, 0.61049, 0.60360, 0.59675, 0.58979, 0.58286, 0.57599, 0.56919, 0.56250,\
        0.55593, 0.54950, 0.54322, 0.53710, 0.53113, 0.52531, 0.51962, 0.51402, 0.50847, 0.50299, 0.49767,\
        0.49257, 0.48757, 0.48256, 0.47749, 0.47233, 0.46707, 0.46169, 0.45619, 0.45055, 0.44476, 0.43881],\
        [0.45280, 0.45580, 0.45873, 0.46165, 0.46461, 0.46765, 0.47082, 0.47416, 0.47772, 0.48149, 0.48546,\
        0.48956, 0.49363, 0.49741, 0.50087, 0.50388, 0.50697, 0.51043, 0.51399, 0.51758, 0.52114, 0.52462,\
        0.52792, 0.53097, 0.53370, 0.53608, 0.53808, 0.53971, 0.54100, 0.54200, 0.54281, 0.54354, 0.54432,\
        0.54532, 0.54672, 0.54865, 0.55127, 0.55466, 0.55885, 0.56382, 0.56950, 0.57579, 0.58260, 0.58972,\
        0.59645, 0.60224, 0.60984, 0.61947, 0.63060, 0.64274, 0.65547, 0.66836, 0.68100, 0.69288, 0.70270,\
        0.71055, 0.71769, 0.72378, 0.72890, 0.73327, 0.73695, 0.73993, 0.74220, 0.74370, 0.74436, 0.74413,\
        0.74302, 0.74109, 0.73846, 0.73529, 0.73174, 0.72796, 0.72399, 0.71979, 0.71536, 0.71089, 0.70665,\
        0.70183, 0.69649, 0.69124, 0.68600, 0.68061, 0.67490, 0.66884, 0.66250, 0.65595, 0.64923, 0.64239,\
        0.63546, 0.62847, 0.62143, 0.61438, 0.60733, 0.60026, 0.59320, 0.58619, 0.57922, 0.57233, 0.56555,\
        0.55890, 0.55238, 0.54601, 0.53978, 0.53368, 0.52771, 0.52185, 0.51607, 0.51036, 0.50472, 0.49928,\
        0.49418, 0.48915, 0.48407, 0.47892, 0.47367, 0.46832, 0.46287, 0.45730, 0.45160, 0.44576, 0.43977],\
        [0.45893, 0.46163, 0.46422, 0.46674, 0.46927, 0.47188, 0.47464, 0.47760, 0.48082, 0.48434, 0.48815,\
        0.49223, 0.49642, 0.50030, 0.50374, 0.50704, 0.51038, 0.51391, 0.51759, 0.52130, 0.52496, 0.52851,\
        0.53188, 0.53498, 0.53777, 0.54021, 0.54228, 0.54397, 0.54532, 0.54636, 0.54719, 0.54789, 0.54860,\
        0.54948, 0.55070, 0.55245, 0.55488, 0.55812, 0.56225, 0.56723, 0.57299, 0.57941, 0.58641, 0.59399,\
        0.60156, 0.60820, 0.61623, 0.62625, 0.63782, 0.65042, 0.66351, 0.67665, 0.68941, 0.70142, 0.71172,\
        0.72008, 0.72728, 0.73350, 0.73880, 0.74325, 0.74693, 0.74984, 0.75200, 0.75338, 0.75394, 0.75362,\
        0.75241, 0.75036, 0.74759, 0.74426, 0.74051, 0.73648, 0.73221, 0.72772, 0.72307, 0.71833, 0.71348,\
        0.70827, 0.70279, 0.69737, 0.69210, 0.68664, 0.68069, 0.67434, 0.66772, 0.66091, 0.65396, 0.64689,\
        0.63974, 0.63253, 0.62529, 0.61805, 0.61081, 0.60359, 0.59645, 0.58932, 0.58225, 0.57528, 0.56843,\
        0.56171, 0.55511, 0.54865, 0.54231, 0.53611, 0.53002, 0.52403, 0.51815, 0.51235, 0.50668, 0.50123,\
        0.49600, 0.49083, 0.48562, 0.48034, 0.47498, 0.46954, 0.46400, 0.45835, 0.45259, 0.44670, 0.44067],\
        [0.46493, 0.46732, 0.46954, 0.47164, 0.47371, 0.47586, 0.47816, 0.48069, 0.48353, 0.48673, 0.49030,\
        0.49422, 0.49842, 0.50246, 0.50617, 0.50981, 0.51350, 0.51729, 0.52116, 0.52503, 0.52883, 0.53249,\
        0.53594, 0.53911, 0.54197, 0.54446, 0.54658, 0.54832, 0.54968, 0.55071, 0.55146, 0.55204, 0.55255,\
        0.55316, 0.55405, 0.55543, 0.55751, 0.56048, 0.56444, 0.56939, 0.57522, 0.58166, 0.58846, 0.59554,\
        0.60322, 0.61156, 0.62098, 0.63195, 0.64431, 0.65756, 0.67117, 0.68462, 0.69744, 0.70948, 0.72011,\
        0.72903, 0.73657, 0.74301, 0.74849, 0.75307, 0.75677, 0.75962, 0.76169, 0.76301, 0.76353, 0.76315,\
        0.76182, 0.75960, 0.75664, 0.75310, 0.74913, 0.74484, 0.74029, 0.73554, 0.73061, 0.72553, 0.72029,\
        0.71482, 0.70921, 0.70363, 0.69826, 0.69262, 0.68629, 0.67956, 0.67265, 0.66561, 0.65844, 0.65116,\
        0.64379, 0.63636, 0.62892, 0.62149, 0.61409, 0.60677, 0.59950, 0.59223, 0.58508, 0.57804, 0.57113,\
        0.56434, 0.55768, 0.55115, 0.54473, 0.53843, 0.53225, 0.52617, 0.52021, 0.51438, 0.50871, 0.50323,\
        0.49789, 0.49256, 0.48716, 0.48173, 0.47624, 0.47068, 0.46505, 0.45933, 0.45350, 0.44756, 0.44149],\
        [0.47076, 0.47284, 0.47465, 0.47630, 0.47790, 0.47955, 0.48136, 0.48344, 0.48587, 0.48870, 0.49197,\
        0.49565, 0.49965, 0.50393, 0.50814, 0.51227, 0.51640, 0.52054, 0.52468, 0.52877, 0.53275, 0.53654,\
        0.54009, 0.54335, 0.54627, 0.54882, 0.55098, 0.55273, 0.55407, 0.55502, 0.55564, 0.55598, 0.55617,\
        0.55634, 0.55671, 0.55751, 0.55901, 0.56150, 0.56516, 0.57006, 0.57604, 0.58271, 0.58954, 0.59625,\
        0.60399, 0.61343, 0.62410, 0.63631, 0.64983, 0.66406, 0.67841, 0.69247, 0.70564, 0.71777, 0.72853,\
        0.73775, 0.74558, 0.75226, 0.75795, 0.76269, 0.76644, 0.76921, 0.77116, 0.77244, 0.77301, 0.77261,\
        0.77115, 0.76872, 0.76551, 0.76172, 0.75750, 0.75296, 0.74816, 0.74314, 0.73793, 0.73255, 0.72700,\
        0.72130, 0.71553, 0.70983, 0.70422, 0.69825, 0.69137, 0.68432, 0.67723, 0.67002, 0.66266, 0.65517,\
        0.64759, 0.63995, 0.63231, 0.62469, 0.61713, 0.60963, 0.60219, 0.59489, 0.58771, 0.58061, 0.57366,\
        0.56683, 0.56012, 0.55352, 0.54703, 0.54065, 0.53438, 0.52823, 0.52220, 0.51632, 0.51061, 0.50509,\
        0.49973, 0.49415, 0.48858, 0.48301, 0.47741, 0.47174, 0.46602, 0.46022, 0.45433, 0.44835, 0.44224],\
        [0.47640, 0.47814, 0.47954, 0.48072, 0.48181, 0.48295, 0.48427, 0.48588, 0.48788, 0.49035, 0.49335,\
        0.49688, 0.50091, 0.50537, 0.50999, 0.51460, 0.51919, 0.52373, 0.52820, 0.53255, 0.53672, 0.54066,\
        0.54433, 0.54768, 0.55067, 0.55327, 0.55544, 0.55718, 0.55847, 0.55931, 0.55972, 0.55975, 0.55947,\
        0.55904, 0.55866, 0.55861, 0.55925, 0.56095, 0.56409, 0.56885, 0.57516, 0.58253, 0.59019, 0.59759,\
        0.60529, 0.61440, 0.62566, 0.63918, 0.65428, 0.66992, 0.68518, 0.69988, 0.71378, 0.72621, 0.73709,\
        0.74643, 0.75440, 0.76125, 0.76715, 0.77210, 0.77595, 0.77855, 0.78024, 0.78153, 0.78229, 0.78194,\
        0.78030, 0.77759, 0.77408, 0.77003, 0.76557, 0.76080, 0.75576, 0.75049, 0.74501, 0.73934, 0.73350,\
        0.72754, 0.72151, 0.71546, 0.70932, 0.70273, 0.69564, 0.68857, 0.68146, 0.67414, 0.66661, 0.65891,\
        0.65112, 0.64328, 0.63545, 0.62766, 0.61993, 0.61229, 0.60479, 0.59746, 0.59014, 0.58300, 0.57602,\
        0.56917, 0.56242, 0.55577, 0.54921, 0.54276, 0.53641, 0.53018, 0.52408, 0.51812, 0.51230, 0.50660,\
        0.50094, 0.49537, 0.48982, 0.48417, 0.47846, 0.47269, 0.46688, 0.46101, 0.45508, 0.44905, 0.44292],\
        [0.48179, 0.48319, 0.48416, 0.48485, 0.48543, 0.48607, 0.48689, 0.48804, 0.48964, 0.49179, 0.49459,\
        0.49808, 0.50225, 0.50695, 0.51193, 0.51698, 0.52201, 0.52696, 0.53176, 0.53638, 0.54077, 0.54487,\
        0.54866, 0.55209, 0.55514, 0.55777, 0.55996, 0.56168, 0.56289, 0.56358, 0.56374, 0.56337, 0.56254,\
        0.56134, 0.55999, 0.55878, 0.55814, 0.55863, 0.56083, 0.56518, 0.57182, 0.58035, 0.58965, 0.59805,\
        0.60549, 0.61347, 0.62501, 0.64023, 0.65750, 0.67524, 0.69198, 0.70745, 0.72197, 0.73478, 0.74577,\
        0.75509, 0.76305, 0.76998, 0.77606, 0.78128, 0.78537, 0.78767, 0.78876, 0.79004, 0.79132, 0.79101,\
        0.78910, 0.78607, 0.78223, 0.77793, 0.77327, 0.76829, 0.76305, 0.75754, 0.75180, 0.74585, 0.73971,\
        0.73344, 0.72705, 0.72056, 0.71386, 0.70681, 0.69958, 0.69258, 0.68548, 0.67804, 0.67029, 0.66236,\
        0.65436, 0.64634, 0.63834, 0.63039, 0.62252, 0.61474, 0.60711, 0.59962, 0.59227, 0.58521, 0.57827,\
        0.57141, 0.56462, 0.55792, 0.55129, 0.54476, 0.53833, 0.53201, 0.52581, 0.51975, 0.51382, 0.50798,\
        0.50219, 0.49666, 0.49100, 0.48520, 0.47936, 0.47351, 0.46762, 0.46170, 0.45572, 0.44967, 0.44352],\
        [0.48688, 0.48793, 0.48845, 0.48864, 0.48873, 0.48888, 0.48925, 0.48999, 0.49122, 0.49310, 0.49576,\
        0.49932, 0.50371, 0.50872, 0.51406, 0.51954, 0.52499, 0.53031, 0.53545, 0.54033, 0.54492, 0.54918,\
        0.55307, 0.55657, 0.55966, 0.56232, 0.56450, 0.56619, 0.56732, 0.56785, 0.56773, 0.56693, 0.56546,\
        0.56337, 0.56083, 0.55813, 0.55576, 0.55443, 0.55501, 0.55832, 0.56481, 0.57441, 0.58622, 0.59758,\
        0.60403, 0.60871, 0.62127, 0.63905, 0.65903, 0.67946, 0.69898, 0.71533, 0.73029, 0.74347, 0.75458,\
        0.76378, 0.77158, 0.77848, 0.78467, 0.79007, 0.79466, 0.79669, 0.79674, 0.79782, 0.79997, 0.79944,\
        0.79734, 0.79399, 0.78985, 0.78539, 0.78057, 0.77543, 0.77000, 0.76428, 0.75829, 0.75204, 0.74559,\
        0.73896, 0.73220, 0.72530, 0.71824, 0.71099, 0.70372, 0.69663, 0.68942, 0.68169, 0.67365, 0.66548,\
        0.65728, 0.64910, 0.64098, 0.63292, 0.62493, 0.61703, 0.60920, 0.60146, 0.59436, 0.58742, 0.58048,\
        0.57358, 0.56674, 0.55996, 0.55326, 0.54664, 0.54012, 0.53370, 0.52740, 0.52123, 0.51520, 0.50933,\
        0.50367, 0.49800, 0.49199, 0.48603, 0.48009, 0.47417, 0.46824, 0.46228, 0.45627, 0.45020, 0.44404],\
        [0.49158, 0.49226, 0.49230, 0.49201, 0.49164, 0.49137, 0.49137, 0.49179, 0.49274, 0.49439, 0.49696,\
        0.50065, 0.50537, 0.51075, 0.51648, 0.52235, 0.52819, 0.53387, 0.53931, 0.54444, 0.54921, 0.55360,\
        0.55757, 0.56112, 0.56423, 0.56688, 0.56905, 0.57070, 0.57175, 0.57213, 0.57174, 0.57050, 0.56835,\
        0.56530, 0.56142, 0.55696, 0.55238, 0.54850, 0.54653, 0.54777, 0.55300, 0.56226, 0.57508, 0.58922,\
        0.59527, 0.59899, 0.61458, 0.63538, 0.65847, 0.68128, 0.70249, 0.72152, 0.73810, 0.75217, 0.76355,\
        0.77258, 0.78005, 0.78688, 0.79326, 0.79854, 0.80309, 0.80533, 0.80551, 0.80634, 0.80749, 0.80703,\
        0.80486, 0.80120, 0.79699, 0.79249, 0.78751, 0.78221, 0.77663, 0.77071, 0.76446, 0.75790, 0.75111,\
        0.74413, 0.73700, 0.72974, 0.72239, 0.71502, 0.70776, 0.70066, 0.69314, 0.68491, 0.67654, 0.66817,\
        0.65984, 0.65157, 0.64337, 0.63526, 0.62723, 0.61931, 0.61153, 0.60403, 0.59683, 0.58971, 0.58266,\
        0.57568, 0.56877, 0.56192, 0.55513, 0.54842, 0.54179, 0.53525, 0.52883, 0.52254, 0.51637, 0.51036,\
        0.50445, 0.49853, 0.49257, 0.48659, 0.48063, 0.47468, 0.46873, 0.46275, 0.45673, 0.45065, 0.44448],\
        [0.49576, 0.49601, 0.49558, 0.49486, 0.49411, 0.49352, 0.49327, 0.49349, 0.49430, 0.49583, 0.49838,\
        0.50232, 0.50740, 0.51315, 0.51925, 0.52549, 0.53169, 0.53769, 0.54341, 0.54875, 0.55367, 0.55814,\
        0.56216, 0.56571, 0.56881, 0.57144, 0.57358, 0.57518, 0.57616, 0.57640, 0.57578, 0.57413, 0.57135,\
        0.56733, 0.56209, 0.55572, 0.54857, 0.54144, 0.53582, 0.53381, 0.53643, 0.54310, 0.55331, 0.56472,\
        0.57383, 0.58486, 0.60358, 0.62844, 0.65602, 0.68167, 0.70493, 0.72649, 0.74531, 0.76085, 0.77287,\
        0.78177, 0.78880, 0.79535, 0.80231, 0.80775, 0.81194, 0.81434, 0.81519, 0.81558, 0.81558, 0.81448,\
        0.81199, 0.80831, 0.80402, 0.79935, 0.79408, 0.78866, 0.78297, 0.77684, 0.77029, 0.76341, 0.75626,\
        0.74892, 0.74143, 0.73383, 0.72616, 0.71853, 0.71104, 0.70372, 0.69577, 0.68726, 0.67880, 0.67038,\
        0.66203, 0.65374, 0.64553, 0.63741, 0.62940, 0.62152, 0.61384, 0.60641, 0.59915, 0.59191, 0.58475,\
        0.57770, 0.57072, 0.56378, 0.55690, 0.55008, 0.54333, 0.53667, 0.53012, 0.52367, 0.51734, 0.51112,\
        0.50497, 0.49889, 0.49291, 0.48696, 0.48101, 0.47506, 0.46910, 0.46312, 0.45711, 0.45103, 0.44487],\
        [0.49915, 0.49889, 0.49809, 0.49708, 0.49606, 0.49528, 0.49492, 0.49513, 0.49601, 0.49763, 0.50031,\
        0.50456, 0.50992, 0.51597, 0.52241, 0.52899, 0.53551, 0.54180, 0.54776, 0.55327, 0.55830, 0.56282,\
        0.56682, 0.57034, 0.57338, 0.57595, 0.57803, 0.57958, 0.58051, 0.58066, 0.57985, 0.57786, 0.57449,\
        0.56960, 0.56309, 0.55492, 0.54520, 0.53445, 0.52411, 0.51770, 0.51727, 0.51952, 0.52561, 0.53461,\
        0.54503, 0.56093, 0.58482, 0.61778, 0.65205, 0.68124, 0.70687, 0.73081, 0.75187, 0.76942, 0.78272,\
        0.79171, 0.79876, 0.80531, 0.81219, 0.81790, 0.82196, 0.82431, 0.82518, 0.82505, 0.82411, 0.82220,\
        0.81923, 0.81534, 0.81082, 0.80579, 0.80034, 0.79487, 0.78907, 0.78267, 0.77577, 0.76853, 0.76103,\
        0.75334, 0.74549, 0.73752, 0.72948, 0.72142, 0.71335, 0.70512, 0.69686, 0.68880, 0.68052, 0.67218,\
        0.66388, 0.65564, 0.64748, 0.63940, 0.63142, 0.62358, 0.61591, 0.60844, 0.60116, 0.59385, 0.58668,\
        0.57961, 0.57257, 0.56555, 0.55857, 0.55163, 0.54476, 0.53797, 0.53127, 0.52467, 0.51817, 0.51176,\
        0.50544, 0.49924, 0.49321, 0.48724, 0.48129, 0.47534, 0.46939, 0.46342, 0.45741, 0.45134, 0.44519],\
        [0.50098, 0.50064, 0.49976, 0.49860, 0.49746, 0.49661, 0.49627, 0.49662, 0.49780, 0.49985, 0.50289,\
        0.50732, 0.51288, 0.51918, 0.52591, 0.53281, 0.53964, 0.54621, 0.55238, 0.55804, 0.56312, 0.56763,\
        0.57157, 0.57500, 0.57793, 0.58040, 0.58238, 0.58386, 0.58475, 0.58485, 0.58391, 0.58163, 0.57775,\
        0.57209, 0.56450, 0.55483, 0.54299, 0.52916, 0.51414, 0.50161, 0.49894, 0.49602, 0.49773, 0.50354,\
        0.51125, 0.52973, 0.55813, 0.60258, 0.64457, 0.67845, 0.70734, 0.73351, 0.75677, 0.77690, 0.79294,\
        0.80266, 0.81036, 0.81740, 0.82390, 0.82930, 0.83311, 0.83515, 0.83559, 0.83474, 0.83288, 0.83012,\
        0.82652, 0.82222, 0.81738, 0.81211, 0.80658, 0.80098, 0.79491, 0.78809, 0.78080, 0.77321, 0.76540,\
        0.75738, 0.74920, 0.74086, 0.73242, 0.72391, 0.71536, 0.70683, 0.69852, 0.69024, 0.68194, 0.67369,\
        0.66549, 0.65734, 0.64925, 0.64122, 0.63329, 0.62547, 0.61778, 0.61022, 0.60273, 0.59551, 0.58849,\
        0.58145, 0.57435, 0.56723, 0.56014, 0.55308, 0.54608, 0.53916, 0.53232, 0.52557, 0.51892, 0.51237,\
        0.50593, 0.49964, 0.49352, 0.48749, 0.48152, 0.47556, 0.46962, 0.46366, 0.45766, 0.45161, 0.44548],\
        [0.50171, 0.50144, 0.50058, 0.49942, 0.49830, 0.49752, 0.49729, 0.49786, 0.49939, 0.50193, 0.50549,\
        0.51025, 0.51606, 0.52264, 0.52969, 0.53690, 0.54405, 0.55089, 0.55727, 0.56304, 0.56813, 0.57257,\
        0.57639, 0.57966, 0.58244, 0.58475, 0.58660, 0.58798, 0.58881, 0.58892, 0.58791, 0.58537, 0.58097,\
        0.57456, 0.56600, 0.55511, 0.54166, 0.52573, 0.50855, 0.49212, 0.48163, 0.47527, 0.47275, 0.47318,\
        0.47717, 0.49046, 0.51548, 0.57102, 0.62646, 0.66917, 0.70344, 0.73244, 0.75832, 0.78104, 0.79997,\
        0.81297, 0.82235, 0.83013, 0.83666, 0.84181, 0.84525, 0.84675, 0.84642, 0.84460, 0.84172, 0.83804,\
        0.83372, 0.82890, 0.82367, 0.81814, 0.81240, 0.80648, 0.80004, 0.79291, 0.78528, 0.77742, 0.76935,\
        0.76108, 0.75259, 0.74391, 0.73509, 0.72618, 0.71726, 0.70847, 0.69997, 0.69143, 0.68317, 0.67504,\
        0.66696, 0.65890, 0.65089, 0.64292, 0.63502, 0.62723, 0.61955, 0.61199, 0.60457, 0.59741, 0.59039,\
        0.58326, 0.57604, 0.56881, 0.56160, 0.55442, 0.54730, 0.54025, 0.53327, 0.52638, 0.51960, 0.51294,\
        0.50643, 0.50006, 0.49384, 0.48773, 0.48171, 0.47574, 0.46980, 0.46385, 0.45787, 0.45184, 0.44573],\
        [0.50192, 0.50161, 0.50075, 0.49967, 0.49873, 0.49810, 0.49801, 0.49877, 0.50067, 0.50371, 0.50784,\
        0.51306, 0.51926, 0.52620, 0.53360, 0.54118, 0.54867, 0.55581, 0.56240, 0.56827, 0.57333, 0.57762,\
        0.58125, 0.58433, 0.58692, 0.58904, 0.59069, 0.59190, 0.59266, 0.59279, 0.59181, 0.58898, 0.58393,\
        0.57661, 0.56697, 0.55479, 0.53965, 0.52098, 0.49955, 0.47883, 0.46299, 0.45279, 0.44694, 0.44291,\
        0.43952, 0.44545, 0.47320, 0.53100, 0.59562, 0.65018, 0.69163, 0.72616, 0.75604, 0.78176, 0.80361,\
        0.82118, 0.83387, 0.84307, 0.85001, 0.85512, 0.85823, 0.85901, 0.85757, 0.85450, 0.85043, 0.84576,\
        0.84068, 0.83527, 0.82960, 0.82369, 0.81757, 0.81119, 0.80437, 0.79694, 0.78912, 0.78115, 0.77294,\
        0.76447, 0.75572, 0.74674, 0.73760, 0.72834, 0.71905, 0.70979, 0.70067, 0.69242, 0.68439, 0.67638,\
        0.66839, 0.66041, 0.65245, 0.64452, 0.63666, 0.62888, 0.62123, 0.61376, 0.60651, 0.59943, 0.59232,\
        0.58498, 0.57760, 0.57025, 0.56293, 0.55566, 0.54842, 0.54125, 0.53414, 0.52712, 0.52022, 0.51345,\
        0.50685, 0.50041, 0.49411, 0.48794, 0.48188, 0.47590, 0.46996, 0.46402, 0.45806, 0.45205, 0.44596],\
        [0.50185, 0.50151, 0.50070, 0.49975, 0.49897, 0.49851, 0.49858, 0.49950, 0.50176, 0.50529, 0.50995,\
        0.51567, 0.52232, 0.52970, 0.53754, 0.54554, 0.55341, 0.56089, 0.56773, 0.57371, 0.57870, 0.58277,\
        0.58612, 0.58897, 0.59139, 0.59331, 0.59472, 0.59569, 0.59628, 0.59643, 0.59560, 0.59239, 0.58639,\
        0.57790, 0.56679, 0.55269, 0.53509, 0.51254, 0.48344, 0.45671, 0.43799, 0.42427, 0.41523, 0.40889,\
        0.39804, 0.40036, 0.43202, 0.48505, 0.55183, 0.62153, 0.67494, 0.71627, 0.75102, 0.78141, 0.80842,\
        0.83073, 0.84637, 0.85670, 0.86399, 0.86912, 0.87198, 0.87184, 0.86882, 0.86416, 0.85878, 0.85310,\
        0.84725, 0.84125, 0.83509, 0.82875, 0.82219, 0.81532, 0.80803, 0.80029, 0.79249, 0.78453, 0.77626,\
        0.76761, 0.75863, 0.74939, 0.73998, 0.73048, 0.72097, 0.71155, 0.70252, 0.69402, 0.68585, 0.67784,\
        0.66987, 0.66192, 0.65398, 0.64606, 0.63820, 0.63042, 0.62276, 0.61530, 0.60813, 0.60133, 0.59394,\
        0.58637, 0.57891, 0.57152, 0.56414, 0.55679, 0.54946, 0.54217, 0.53494, 0.52779, 0.52076, 0.51387,\
        0.50719, 0.50069, 0.49431, 0.48808, 0.48200, 0.47602, 0.47010, 0.46419, 0.45825, 0.45226, 0.44619],\
        [0.50167, 0.50132, 0.50060, 0.49978, 0.49913, 0.49885, 0.49918, 0.50033, 0.50285, 0.50671, 0.51180,\
        0.51800, 0.52516, 0.53305, 0.54138, 0.54986, 0.55817, 0.56604, 0.57318, 0.57933, 0.58424, 0.58796,\
        0.59094, 0.59357, 0.59588, 0.59767, 0.59886, 0.59959, 0.59989, 0.59985, 0.59929, 0.59537, 0.58823,\
        0.57850, 0.56547, 0.54835, 0.52634, 0.49759, 0.45896, 0.42540, 0.40577, 0.38717, 0.37252, 0.36164,\
        0.35013, 0.35125, 0.38925, 0.44891, 0.50769, 0.58578, 0.65367, 0.70278, 0.74385, 0.78126, 0.81604,\
        0.84439, 0.86188, 0.87190, 0.87908, 0.88382, 0.88646, 0.88500, 0.87969, 0.87321, 0.86654, 0.85991,\
        0.85335, 0.84679, 0.84015, 0.83337, 0.82638, 0.81912, 0.81152, 0.80363, 0.79573, 0.78779, 0.77942,\
        0.77055, 0.76132, 0.75185, 0.74225, 0.73260, 0.72300, 0.71358, 0.70451, 0.69583, 0.68753, 0.67945,\
        0.67143, 0.66345, 0.65549, 0.64756, 0.63967, 0.63185, 0.62412, 0.61650, 0.60903, 0.60168, 0.59449,\
        0.58728, 0.57998, 0.57263, 0.56524, 0.55784, 0.55043, 0.54304, 0.53569, 0.52841, 0.52123, 0.51420,\
        0.50737, 0.50078, 0.49435, 0.48814, 0.48209, 0.47615, 0.47026, 0.46437, 0.45845, 0.45248, 0.44642],\
        [0.50145, 0.50112, 0.50047, 0.49974, 0.49919, 0.49907, 0.49963, 0.50109, 0.50377, 0.50787, 0.51332,\
        0.51999, 0.52769, 0.53613, 0.54503, 0.55403, 0.56282, 0.57110, 0.57861, 0.58502, 0.58994, 0.59314,\
        0.59559, 0.59810, 0.60040, 0.60219, 0.60334, 0.60395, 0.60402, 0.60349, 0.60179, 0.59710, 0.58979,\
        0.57943, 0.56438, 0.54339, 0.51557, 0.47915, 0.43489, 0.39335, 0.36950, 0.34642, 0.32821, 0.31775,\
        0.30843, 0.30282, 0.33577, 0.41914, 0.47446, 0.54967, 0.62724, 0.68431, 0.73585, 0.78259, 0.82820,\
        0.86494, 0.88251, 0.88955, 0.89659, 0.89959, 0.90101, 0.89727, 0.88945, 0.88138, 0.87361, 0.86619,\
        0.85901, 0.85193, 0.84482, 0.83761, 0.83024, 0.82268, 0.81493, 0.80701, 0.79905, 0.79108, 0.78242,\
        0.77321, 0.76370, 0.75403, 0.74431, 0.73461, 0.72500, 0.71559, 0.70643, 0.69766, 0.68934, 0.68116,\
        0.67304, 0.66499, 0.65698, 0.64901, 0.64108, 0.63321, 0.62539, 0.61762, 0.60988, 0.60220, 0.59515,\
        0.58815, 0.58098, 0.57367, 0.56628, 0.55883, 0.55135, 0.54387, 0.53641, 0.52901, 0.52170, 0.51451,\
        0.50748, 0.50069, 0.49432, 0.48820, 0.48222, 0.47632, 0.47045, 0.46458, 0.45868, 0.45271, 0.44667],\
        [0.50123, 0.50092, 0.50032, 0.49964, 0.49914, 0.49908, 0.49970, 0.50137, 0.50426, 0.50858, 0.51439,\
        0.52155, 0.52982, 0.53888, 0.54838, 0.55795, 0.56723, 0.57589, 0.58371, 0.59049, 0.59569, 0.59835,\
        0.59996, 0.60267, 0.60502, 0.60691, 0.60820, 0.60885, 0.60875, 0.60755, 0.60451, 0.59912, 0.59309,\
        0.58287, 0.56628, 0.54126, 0.50766, 0.46771, 0.42468, 0.38277, 0.34742, 0.31883, 0.30094, 0.29927,\
        0.29484, 0.28682, 0.31919, 0.40128, 0.47000, 0.53185, 0.59781, 0.66686, 0.73027, 0.78539, 0.84413,\
        0.89218, 0.91006, 0.91360, 0.91744, 0.91785, 0.91508, 0.90813, 0.89852, 0.88890, 0.88021, 0.87208,\
        0.86433, 0.85674, 0.84915, 0.84147, 0.83371, 0.82587, 0.81802, 0.81023, 0.80253, 0.79422, 0.78496,\
        0.77535, 0.76561, 0.75583, 0.74607, 0.73638, 0.72684, 0.71750, 0.70845, 0.69973, 0.69125, 0.68288,\
        0.67462, 0.66648, 0.65841, 0.65040, 0.64244, 0.63453, 0.62667, 0.61887, 0.61118, 0.60372, 0.59646,\
        0.58926, 0.58204, 0.57473, 0.56730, 0.55980, 0.55224, 0.54467, 0.53713, 0.52963, 0.52221, 0.51491,\
        0.50778, 0.50091, 0.49452, 0.48841, 0.48245, 0.47656, 0.47070, 0.46484, 0.45894, 0.45298, 0.44693],\
        [0.50101, 0.50070, 0.50013, 0.49944, 0.49892, 0.49889, 0.49961, 0.50129, 0.50420, 0.50871, 0.51490,\
        0.52261, 0.53151, 0.54124, 0.55139, 0.56156, 0.57131, 0.58028, 0.58818, 0.59503, 0.60053, 0.60334,\
        0.60506, 0.60747, 0.60986, 0.61191, 0.61345, 0.61436, 0.61444, 0.61332, 0.61054, 0.60624, 0.60109,\
        0.59087, 0.57459, 0.54619, 0.50610, 0.46275, 0.42144, 0.37968, 0.34055, 0.31278, 0.30168, 0.30329,\
        0.30616, 0.30924, 0.34187, 0.41264, 0.47931, 0.52797, 0.58117, 0.65279, 0.72417, 0.78269, 0.85627,\
        0.91544, 0.93609, 0.94068, 0.94042, 0.93687, 0.92998, 0.91996, 0.90803, 0.89670, 0.88680, 0.87778,\
        0.86945, 0.86132, 0.85316, 0.84494, 0.83669, 0.82847, 0.82036, 0.81242, 0.80457, 0.79596, 0.78641,\
        0.77667, 0.76689, 0.75713, 0.74743, 0.73783, 0.72838, 0.71915, 0.71022, 0.70161, 0.69296, 0.68441,\
        0.67606, 0.66785, 0.65973, 0.65169, 0.64371, 0.63580, 0.62795, 0.62019, 0.61254, 0.60506, 0.59772,\
        0.59046, 0.58318, 0.57580, 0.56831, 0.56073, 0.55310, 0.54546, 0.53783, 0.53026, 0.52279, 0.51545,\
        0.50832, 0.50149, 0.49500, 0.48881, 0.48280, 0.47690, 0.47104, 0.46516, 0.45925, 0.45328, 0.44723],\
        [0.50078, 0.50044, 0.49985, 0.49908, 0.49849, 0.49839, 0.49907, 0.50066, 0.50344, 0.50819, 0.51485,\
        0.52316, 0.53274, 0.54316, 0.55400, 0.56479, 0.57506, 0.58432, 0.59211, 0.59830, 0.60368, 0.60734,\
        0.60989, 0.61234, 0.61481, 0.61709, 0.61902, 0.62044, 0.62118, 0.62100, 0.61972, 0.61732, 0.61303,\
        0.60544, 0.59196, 0.56190, 0.51320, 0.46288, 0.42059, 0.37873, 0.34153, 0.31789, 0.31058, 0.31593,\
        0.32758, 0.34549, 0.38595, 0.44755, 0.50247, 0.53395, 0.57082, 0.63663, 0.71832, 0.78661, 0.86214,\
        0.92302, 0.95190, 0.96128, 0.96071, 0.95460, 0.94470, 0.93213, 0.91833, 0.90506, 0.89313, 0.88340,\
        0.87453, 0.86570, 0.85682, 0.84793, 0.83909, 0.83035, 0.82173, 0.81323, 0.80470, 0.79585, 0.78659,\
        0.77708, 0.76748, 0.75789, 0.74834, 0.73888, 0.72954, 0.72038, 0.71142, 0.70265, 0.69402, 0.68557,\
        0.67726, 0.66904, 0.66090, 0.65284, 0.64486, 0.63696, 0.62915, 0.62144, 0.61383, 0.60634, 0.59896,\
        0.59168, 0.58435, 0.57688, 0.56929, 0.56162, 0.55390, 0.54619, 0.53851, 0.53090, 0.52340, 0.51606,\
        0.50893, 0.50209, 0.49556, 0.48932, 0.48328, 0.47734, 0.47144, 0.46554, 0.45961, 0.45362, 0.44754],\
        [0.50057, 0.50014, 0.49946, 0.49863, 0.49790, 0.49758, 0.49793, 0.49917, 0.50208, 0.50713, 0.51432,\
        0.52329, 0.53356, 0.54466, 0.55615, 0.56756, 0.57839, 0.58809, 0.59615, 0.60205, 0.60693, 0.61086,\
        0.61397, 0.61679, 0.61955, 0.62223, 0.62470, 0.62686, 0.62856, 0.62969, 0.63014, 0.62976, 0.62805,\
        0.62407, 0.61446, 0.59049, 0.53045, 0.47373, 0.42946, 0.38352, 0.34880, 0.33022, 0.32890, 0.34140,\
        0.36521, 0.39620, 0.44215, 0.49681, 0.52282, 0.53625, 0.56103, 0.62224, 0.71667, 0.79853, 0.87399,\
        0.92987, 0.96325, 0.97631, 0.97667, 0.96950, 0.95777, 0.94339, 0.92794, 0.91278, 0.89944, 0.88940,\
        0.87963, 0.86977, 0.85999, 0.85035, 0.84086, 0.83151, 0.82228, 0.81314, 0.80406, 0.79505, 0.78600,\
        0.77681, 0.76750, 0.75815, 0.74881, 0.73953, 0.73034, 0.72125, 0.71225, 0.70332, 0.69479, 0.68651,\
        0.67825, 0.67002, 0.66186, 0.65379, 0.64581, 0.63793, 0.63017, 0.62253, 0.61500, 0.60756, 0.60018,\
        0.59292, 0.58551, 0.57790, 0.57018, 0.56240, 0.55461, 0.54683, 0.53912, 0.53150, 0.52401, 0.51668,\
        0.50955, 0.50268, 0.49614, 0.48990, 0.48383, 0.47785, 0.47191, 0.46597, 0.46000, 0.45397, 0.44787],\
        [0.50044, 0.49997, 0.49918, 0.49824, 0.49735, 0.49676, 0.49677, 0.49781, 0.50061, 0.50579, 0.51358,\
        0.52322, 0.53410, 0.54576, 0.55778, 0.56972, 0.58109, 0.59123, 0.59949, 0.60553, 0.61007, 0.61391,\
        0.61734, 0.62059, 0.62382, 0.62705, 0.63022, 0.63323, 0.63600, 0.63844, 0.64043, 0.64172, 0.64170,\
        0.63895, 0.63053, 0.60987, 0.55882, 0.50939, 0.45434, 0.39957, 0.36504, 0.35175, 0.35932, 0.38088,\
        0.41486, 0.45590, 0.49515, 0.51922, 0.52820, 0.53691, 0.56504, 0.63414, 0.72898, 0.81286, 0.89404,\
        0.94628, 0.97563, 0.98836, 0.98876, 0.98100, 0.96820, 0.95265, 0.93618, 0.92036, 0.90681, 0.89565,\
        0.88408, 0.87305, 0.86239, 0.85204, 0.84195, 0.83206, 0.82230, 0.81263, 0.80309, 0.79399, 0.78512,\
        0.77618, 0.76713, 0.75801, 0.74889, 0.73980, 0.73078, 0.72185, 0.71301, 0.70429, 0.69578, 0.68739,\
        0.67904, 0.67075, 0.66255, 0.65445, 0.64648, 0.63864, 0.63093, 0.62337, 0.61595, 0.60868, 0.60148,\
        0.59418, 0.58653, 0.57873, 0.57087, 0.56300, 0.55514, 0.54734, 0.53961, 0.53201, 0.52456, 0.51728,\
        0.51019, 0.50332, 0.49679, 0.49057, 0.48446, 0.47842, 0.47242, 0.46642, 0.46040, 0.45434, 0.44820],\
        [0.50040, 0.49991, 0.49908, 0.49808, 0.49708, 0.49633, 0.49611, 0.49683, 0.49908, 0.50473, 0.51318,\
        0.52329, 0.53452, 0.54648, 0.55878, 0.57104, 0.58280, 0.59338, 0.60172, 0.60759, 0.61207, 0.61602,\
        0.61981, 0.62358, 0.62743, 0.63134, 0.63530, 0.63923, 0.64305, 0.64666, 0.64990, 0.65239, 0.65322,\
        0.65022, 0.63872, 0.61162, 0.57565, 0.53425, 0.47901, 0.42168, 0.39367, 0.38610, 0.40369, 0.43713,\
        0.47184, 0.49991, 0.51535, 0.52029, 0.52302, 0.53646, 0.57381, 0.64247, 0.73879, 0.83112, 0.90660,\
        0.95824, 0.98620, 0.99765, 0.99735, 0.98896, 0.97542, 0.95912, 0.94200, 0.92569, 0.91125, 0.89830,\
        0.88666, 0.87498, 0.86368, 0.85284, 0.84236, 0.83212, 0.82204, 0.81208, 0.80234, 0.79313, 0.78426,\
        0.77541, 0.76651, 0.75757, 0.74862, 0.73971, 0.73087, 0.72213, 0.71351, 0.70504, 0.69656, 0.68799,\
        0.67951, 0.67114, 0.66289, 0.65477, 0.64680, 0.63898, 0.63133, 0.62382, 0.61647, 0.60930, 0.60238,\
        0.59494, 0.58707, 0.57918, 0.57126, 0.56334, 0.55545, 0.54763, 0.53992, 0.53237, 0.52498, 0.51780,\
        0.51084, 0.50411, 0.49762, 0.49131, 0.48511, 0.47898, 0.47291, 0.46685, 0.46079, 0.45468, 0.44852],\
        [0.50045, 0.49996, 0.49920, 0.49823, 0.49723, 0.49649, 0.49632, 0.49720, 0.49985, 0.50537, 0.51369,\
        0.52374, 0.53494, 0.54685, 0.55910, 0.57131, 0.58305, 0.59378, 0.60222, 0.60783, 0.61249, 0.61687,\
        0.62124, 0.62570, 0.63027, 0.63498, 0.63979, 0.64466, 0.64951, 0.65420, 0.65855, 0.66221, 0.66443,\
        0.66353, 0.65643, 0.63911, 0.60789, 0.56499, 0.50235, 0.44785, 0.43573, 0.44215, 0.46264, 0.48999,\
        0.51026, 0.51604, 0.51433, 0.51229, 0.51807, 0.54159, 0.58764, 0.65291, 0.76403, 0.86536, 0.92980,\
        0.97089, 0.99466, 1.00402, 1.00255, 0.99338, 0.97922, 0.96232, 0.94460, 0.92769, 0.91278, 0.90031,\
        0.88743, 0.87516, 0.86362, 0.85264, 0.84206, 0.83174, 0.82159, 0.81159, 0.80186, 0.79252, 0.78350,\
        0.77460, 0.76572, 0.75684, 0.74801, 0.73924, 0.73056, 0.72196, 0.71345, 0.70504, 0.69659, 0.68802,\
        0.67950, 0.67109, 0.66280, 0.65465, 0.64668, 0.63890, 0.63129, 0.62383, 0.61647, 0.60912, 0.60170,\
        0.59446, 0.58696, 0.57919, 0.57129, 0.56335, 0.55546, 0.54766, 0.53999, 0.53249, 0.52520, 0.51814,\
        0.51133, 0.50478, 0.49839, 0.49195, 0.48566, 0.47947, 0.47334, 0.46724, 0.46114, 0.45500, 0.44880],\
        [0.50062, 0.50024, 0.49957, 0.49867, 0.49778, 0.49720, 0.49729, 0.49855, 0.50170, 0.50714, 0.51496,\
        0.52457, 0.53539, 0.54694, 0.55879, 0.57054, 0.58172, 0.59174, 0.59979, 0.60596, 0.61131, 0.61648,\
        0.62169, 0.62697, 0.63237, 0.63791, 0.64360, 0.64943, 0.65532, 0.66113, 0.66669, 0.67184, 0.67649,\
        0.68030, 0.68269, 0.68306, 0.66916, 0.63035, 0.56427, 0.50552, 0.49704, 0.50419, 0.51150, 0.51867,\
        0.52058, 0.51512, 0.50928, 0.51206, 0.52991, 0.57270, 0.64052, 0.71637, 0.82093, 0.89401, 0.94746,\
        0.98164, 1.00070, 1.00741, 1.00446, 0.99442, 0.97964, 0.96221, 0.94391, 0.92624, 0.91043, 0.89716,\
        0.88509, 0.87335, 0.86217, 0.85147, 0.84111, 0.83096, 0.82093, 0.81102, 0.80129, 0.79190, 0.78274,\
        0.77370, 0.76472, 0.75583, 0.74703, 0.73836, 0.72980, 0.72132, 0.71285, 0.70433, 0.69587, 0.68745,\
        0.67898, 0.67054, 0.66221, 0.65404, 0.64606, 0.63831, 0.63076, 0.62338, 0.61609, 0.60875, 0.60129,\
        0.59399, 0.58655, 0.57883, 0.57094, 0.56302, 0.55513, 0.54736, 0.53975, 0.53233, 0.52514, 0.51819,\
        0.51148, 0.50498, 0.49860, 0.49227, 0.48601, 0.47981, 0.47367, 0.46754, 0.46141, 0.45525, 0.44903],\
        [0.50086, 0.50056, 0.50002, 0.49924, 0.49852, 0.49822, 0.49866, 0.50010, 0.50352, 0.50898, 0.51642,\
        0.52552, 0.53581, 0.54680, 0.55806, 0.56912, 0.57950, 0.58870, 0.59638, 0.60293, 0.60909, 0.61526,\
        0.62142, 0.62759, 0.63381, 0.64017, 0.64671, 0.65349, 0.66048, 0.66756, 0.67454, 0.68147, 0.68910,\
        0.69842, 0.70984, 0.72216, 0.72187, 0.69392, 0.64831, 0.59302, 0.56529, 0.55207, 0.54056, 0.53365,\
        0.52632, 0.51749, 0.51627, 0.53835, 0.58077, 0.65333, 0.72561, 0.79320, 0.86708, 0.92039, 0.96119,\
        0.98904, 1.00380, 1.00779, 1.00326, 0.99233, 0.97702, 0.95918, 0.94047, 0.92232, 0.90570, 0.89222,\
        0.88089, 0.87010, 0.85967, 0.84953, 0.83961, 0.82981, 0.82007, 0.81035, 0.80068, 0.79121, 0.78187,\
        0.77261, 0.76346, 0.75445, 0.74562, 0.73699, 0.72855, 0.72023, 0.71188, 0.70337, 0.69491, 0.68647,\
        0.67796, 0.66946, 0.66107, 0.65286, 0.64487, 0.63714, 0.62967, 0.62243, 0.61535, 0.60830, 0.60118,\
        0.59368, 0.58598, 0.57814, 0.57022, 0.56229, 0.55444, 0.54671, 0.53916, 0.53183, 0.52474, 0.51791,\
        0.51130, 0.50484, 0.49851, 0.49232, 0.48615, 0.47999, 0.47386, 0.46773, 0.46160, 0.45543, 0.44920],\
        [0.50105, 0.50081, 0.50035, 0.49973, 0.49915, 0.49911, 0.49997, 0.50180, 0.50525, 0.51061, 0.51775,\
        0.52639, 0.53614, 0.54655, 0.55719, 0.56757, 0.57726, 0.58586, 0.59325, 0.59974, 0.60675, 0.61387,\
        0.62089, 0.62783, 0.63477, 0.64182, 0.64911, 0.65675, 0.66484, 0.67339, 0.68218, 0.69099, 0.70169,\
        0.71601, 0.73259, 0.74727, 0.75067, 0.73559, 0.70470, 0.66550, 0.63116, 0.59470, 0.56753, 0.55453,\
        0.54594, 0.54015, 0.54909, 0.60046, 0.68191, 0.74252, 0.79158, 0.84834, 0.90484, 0.94531, 0.97418,\
        0.99383, 1.00392, 1.00522, 0.99921, 0.98752, 0.97184, 0.95382, 0.93503, 0.91697, 0.90124, 0.88778,\
        0.87652, 0.86634, 0.85661, 0.84710, 0.83771, 0.82834, 0.81896, 0.80955, 0.80003, 0.79040, 0.78077,\
        0.77123, 0.76184, 0.75264, 0.74369, 0.73505, 0.72672, 0.71862, 0.71058, 0.70235, 0.69379, 0.68512,\
        0.67642, 0.66780, 0.65932, 0.65106, 0.64306, 0.63535, 0.62793, 0.62082, 0.61397, 0.60722, 0.60023,\
        0.59273, 0.58492, 0.57699, 0.56905, 0.56114, 0.55333, 0.54567, 0.53820, 0.53096, 0.52399, 0.51729,\
        0.51084, 0.50454, 0.49834, 0.49222, 0.48611, 0.48000, 0.47389, 0.46779, 0.46167, 0.45551, 0.44930],\
        [0.50118, 0.50093, 0.50047, 0.49991, 0.49949, 0.49962, 0.50065, 0.50287, 0.50655, 0.51187, 0.51879,\
        0.52706, 0.53635, 0.54627, 0.55640, 0.56629, 0.57554, 0.58385, 0.59120, 0.59814, 0.60536, 0.61293,\
        0.62050, 0.62797, 0.63541, 0.64295, 0.65078, 0.65907, 0.66804, 0.67796, 0.68900, 0.70023, 0.71321,\
        0.72987, 0.74790, 0.76307, 0.76960, 0.76365, 0.74620, 0.72115, 0.69091, 0.64877, 0.61540, 0.59421,\
        0.58925, 0.58763, 0.62781, 0.70457, 0.75738, 0.79118, 0.83233, 0.88203, 0.92685, 0.95828, 0.98032,\
        0.99437, 1.00070, 0.99980, 0.99263, 0.98045, 0.96466, 0.94675, 0.92824, 0.91054, 0.89521, 0.88284,\
        0.87215, 0.86250, 0.85337, 0.84445, 0.83555, 0.82659, 0.81752, 0.80834, 0.79897, 0.78916, 0.77927,\
        0.76945, 0.75977, 0.75031, 0.74116, 0.73241, 0.72411, 0.71624, 0.70857, 0.70060, 0.69199, 0.68309,\
        0.67422, 0.66548, 0.65693, 0.64862, 0.64060, 0.63288, 0.62548, 0.61842, 0.61172, 0.60527, 0.59843,\
        0.59095, 0.58316, 0.57527, 0.56738, 0.55955, 0.55182, 0.54424, 0.53686, 0.52972, 0.52285, 0.51630,\
        0.51007, 0.50405, 0.49805, 0.49196, 0.48587, 0.47981, 0.47376, 0.46771, 0.46163, 0.45550, 0.44931],\
        [0.50126, 0.50097, 0.50048, 0.49991, 0.49956, 0.49979, 0.50098, 0.50343, 0.50731, 0.51266, 0.51945,\
        0.52746, 0.53643, 0.54601, 0.55581, 0.56545, 0.57457, 0.58291, 0.59043, 0.59743, 0.60489, 0.61274,\
        0.62052, 0.62820, 0.63587, 0.64367, 0.65177, 0.66038, 0.66975, 0.68021, 0.69213, 0.70549, 0.72070,\
        0.73845, 0.75675, 0.77225, 0.78127, 0.78153, 0.77321, 0.75809, 0.73687, 0.70540, 0.68158, 0.66033,\
        0.64602, 0.64521, 0.69330, 0.75770, 0.79159, 0.82076, 0.85715, 0.89577, 0.93238, 0.96020, 0.97898,\
        0.99001, 0.99405, 0.99177, 0.98395, 0.97164, 0.95605, 0.93858, 0.92073, 0.90402, 0.88974, 0.87797,\
        0.86787, 0.85876, 0.85015, 0.84172, 0.83325, 0.82461, 0.81572, 0.80655, 0.79708, 0.78729, 0.77727,\
        0.76720, 0.75721, 0.74743, 0.73799, 0.72901, 0.72057, 0.71276, 0.70548, 0.69765, 0.68901, 0.68010,\
        0.67121, 0.66244, 0.65387, 0.64554, 0.63749, 0.62976, 0.62235, 0.61525, 0.60842, 0.60214, 0.59561,\
        0.58826, 0.58066, 0.57294, 0.56520, 0.55750, 0.54990, 0.54244, 0.53517, 0.52812, 0.52134, 0.51489,\
        0.50883, 0.50312, 0.49733, 0.49135, 0.48536, 0.47940, 0.47345, 0.46748, 0.46147, 0.45540, 0.44924],\
        [0.50130, 0.50097, 0.50044, 0.49986, 0.49944, 0.49963, 0.50091, 0.50358, 0.50761, 0.51301, 0.51972,\
        0.52758, 0.53635, 0.54574, 0.55541, 0.56503, 0.57429, 0.58294, 0.59089, 0.59828, 0.60572, 0.61334,\
        0.62096, 0.62857, 0.63624, 0.64409, 0.65223, 0.66086, 0.67020, 0.68052, 0.69215, 0.70633, 0.72351,\
        0.74209, 0.76045, 0.77621, 0.78694, 0.79110, 0.78856, 0.78010, 0.76646, 0.74997, 0.73604, 0.72152,\
        0.70607, 0.69913, 0.73946, 0.79288, 0.82251, 0.84892, 0.87736, 0.90537, 0.93336, 0.95665, 0.97271,\
        0.98181, 0.98456, 0.98161, 0.97369, 0.96166, 0.94660, 0.92982, 0.91290, 0.89743, 0.88435, 0.87332,\
        0.86377, 0.85515, 0.84701, 0.83900, 0.83089, 0.82250, 0.81373, 0.80453, 0.79494, 0.78504, 0.77485,\
        0.76452, 0.75420, 0.74405, 0.73423, 0.72489, 0.71614, 0.70800, 0.70062, 0.69323, 0.68483, 0.67613,\
        0.66738, 0.65870, 0.65017, 0.64186, 0.63382, 0.62608, 0.61867, 0.61157, 0.60461, 0.59784, 0.59161,\
        0.58475, 0.57750, 0.57006, 0.56255, 0.55506, 0.54764, 0.54033, 0.53318, 0.52623, 0.51952, 0.51309,\
        0.50702, 0.50152, 0.49602, 0.49030, 0.48456, 0.47879, 0.47298, 0.46713, 0.46121, 0.45520, 0.44910],\
        [0.50130, 0.50093, 0.50032, 0.49967, 0.49922, 0.49939, 0.50070, 0.50345, 0.50755, 0.51296, 0.51962,\
        0.52739, 0.53606, 0.54538, 0.55507, 0.56484, 0.57440, 0.58353, 0.59206, 0.59994, 0.60718, 0.61435,\
        0.62160, 0.62898, 0.63652, 0.64429, 0.65237, 0.66088, 0.67005, 0.68020, 0.69195, 0.70638, 0.72363,\
        0.74221, 0.76038, 0.77622, 0.78797, 0.79449, 0.79563, 0.79210, 0.78546, 0.77820, 0.77131, 0.76405,\
        0.75934, 0.76959, 0.80645, 0.83760, 0.85741, 0.87452, 0.89387, 0.91404, 0.93358, 0.95102, 0.96388,\
        0.97123, 0.97314, 0.97001, 0.96242, 0.95109, 0.93686, 0.92094, 0.90509, 0.89104, 0.87906, 0.86877,\
        0.85977, 0.85162, 0.84391, 0.83629, 0.82849, 0.82030, 0.81161, 0.80239, 0.79268, 0.78255, 0.77210,\
        0.76148, 0.75082, 0.74027, 0.73002, 0.72025, 0.71118, 0.70285, 0.69501, 0.68762, 0.67971, 0.67139,\
        0.66289, 0.65437, 0.64595, 0.63769, 0.62966, 0.62192, 0.61453, 0.60755, 0.60083, 0.59412, 0.58755,\
        0.58086, 0.57391, 0.56678, 0.55955, 0.55231, 0.54511, 0.53799, 0.53099, 0.52417, 0.51756, 0.51118,\
        0.50502, 0.49921, 0.49423, 0.48898, 0.48356, 0.47802, 0.47240, 0.46668, 0.46086, 0.45494, 0.44889],\
        [0.50124, 0.50083, 0.50015, 0.49943, 0.49897, 0.49917, 0.50047, 0.50315, 0.50718, 0.51254, 0.51915,\
        0.52686, 0.53550, 0.54483, 0.55460, 0.56458, 0.57451, 0.58413, 0.59311, 0.60115, 0.60834, 0.61516,\
        0.62208, 0.62924, 0.63667, 0.64438, 0.65238, 0.66077, 0.66974, 0.67968, 0.69130, 0.70540, 0.72203,\
        0.74013, 0.75790, 0.77363, 0.78592, 0.79390, 0.79747, 0.79750, 0.79563, 0.79531, 0.79532, 0.79595,\
        0.80122, 0.81886, 0.84770, 0.87094, 0.88428, 0.89201, 0.90171, 0.91599, 0.93053, 0.94362, 0.95357,\
        0.95932, 0.96057, 0.95752, 0.95062, 0.94040, 0.92741, 0.91256, 0.89775, 0.88499, 0.87388, 0.86420,\
        0.85571, 0.84802, 0.84074, 0.83350, 0.82599, 0.81800, 0.80937, 0.80003, 0.79015, 0.77977, 0.76904,\
        0.75812, 0.74715, 0.73625, 0.72557, 0.71534, 0.70590, 0.69752, 0.68969, 0.68209, 0.67432, 0.66626,\
        0.65799, 0.64964, 0.64134, 0.63316, 0.62517, 0.61742, 0.60997, 0.60309, 0.59653, 0.58995, 0.58341,\
        0.57683, 0.57011, 0.56326, 0.55632, 0.54935, 0.54240, 0.53550, 0.52870, 0.52204, 0.51559, 0.50941,\
        0.50357, 0.49806, 0.49284, 0.48774, 0.48255, 0.47722, 0.47176, 0.46618, 0.46047, 0.45462, 0.44863],\
        [0.50108, 0.50064, 0.49991, 0.49910, 0.49861, 0.49881, 0.50006, 0.50258, 0.50647, 0.51175, 0.51831,\
        0.52599, 0.53461, 0.54396, 0.55381, 0.56396, 0.57418, 0.58423, 0.59372, 0.60189, 0.60875, 0.61528,\
        0.62205, 0.62917, 0.63662, 0.64438, 0.65241, 0.66078, 0.66958, 0.67913, 0.68999, 0.70317, 0.71931,\
        0.73695, 0.75418, 0.76963, 0.78223, 0.79129, 0.79670, 0.79939, 0.80138, 0.80576, 0.81172, 0.81870,\
        0.82990, 0.84797, 0.87057, 0.89009, 0.89970, 0.90154, 0.90516, 0.91411, 0.92467, 0.93461, 0.94230,\
        0.94669, 0.94739, 0.94451, 0.93847, 0.92972, 0.91852, 0.90515, 0.89153, 0.87928, 0.86859, 0.85935,\
        0.85133, 0.84415, 0.83736, 0.83050, 0.82325, 0.81545, 0.80699, 0.79756, 0.78734, 0.77660, 0.76558,\
        0.75443, 0.74327, 0.73215, 0.72118, 0.71058, 0.70077, 0.69229, 0.68447, 0.67679, 0.66902, 0.66105,\
        0.65292, 0.64470, 0.63650, 0.62841, 0.62051, 0.61282, 0.60531, 0.59823, 0.59189, 0.58553, 0.57914,\
        0.57271, 0.56621, 0.55962, 0.55296, 0.54628, 0.53960, 0.53295, 0.52638, 0.51992, 0.51365, 0.50767,\
        0.50211, 0.49684, 0.49171, 0.48669, 0.48163, 0.47645, 0.47113, 0.46567, 0.46005, 0.45427, 0.44834],\
        [0.50070, 0.50015, 0.49937, 0.49852, 0.49800, 0.49815, 0.49931, 0.50164, 0.50539, 0.51061, 0.51713,\
        0.52478, 0.53337, 0.54271, 0.55258, 0.56276, 0.57306, 0.58324, 0.59301, 0.60135, 0.60772, 0.61434,\
        0.62131, 0.62865, 0.63633, 0.64430, 0.65254, 0.66106, 0.66990, 0.67920, 0.68927, 0.70127, 0.71697,\
        0.73376, 0.75011, 0.76505, 0.77784, 0.78800, 0.79545, 0.80093, 0.80654, 0.81363, 0.82240, 0.83280,\
        0.84597, 0.86236, 0.87987, 0.89400, 0.90012, 0.90212, 0.90469, 0.90956, 0.91684, 0.92449, 0.93046,\
        0.93372, 0.93389, 0.93111, 0.92581, 0.91859, 0.90970, 0.89806, 0.88508, 0.87307, 0.86262, 0.85378,\
        0.84630, 0.83975, 0.83357, 0.82717, 0.82012, 0.81227, 0.80398, 0.79456, 0.78384, 0.77277, 0.76155,\
        0.75032, 0.73915, 0.72806, 0.71706, 0.70638, 0.69651, 0.68779, 0.67971, 0.67183, 0.66395, 0.65596,\
        0.64786, 0.63970, 0.63157, 0.62356, 0.61577, 0.60828, 0.60100, 0.59397, 0.58742, 0.58111, 0.57484,\
        0.56857, 0.56227, 0.55593, 0.54955, 0.54316, 0.53678, 0.53043, 0.52413, 0.51792, 0.51182, 0.50594,\
        0.50056, 0.49555, 0.49067, 0.48580, 0.48085, 0.47578, 0.47055, 0.46517, 0.45963, 0.45391, 0.44802],\
        [0.50009, 0.49933, 0.49846, 0.49761, 0.49708, 0.49717, 0.49815, 0.50028, 0.50402, 0.50919, 0.51565,\
        0.52326, 0.53181, 0.54110, 0.55089, 0.56096, 0.57107, 0.58092, 0.59008, 0.59782, 0.60514, 0.61242,\
        0.61990, 0.62769, 0.63577, 0.64411, 0.65271, 0.66159, 0.67079, 0.68038, 0.69063, 0.70231, 0.71617,\
        0.73111, 0.74603, 0.76017, 0.77299, 0.78416, 0.79373, 0.80197, 0.80954, 0.81805, 0.82793, 0.83927,\
        0.85220, 0.86621, 0.87954, 0.88948, 0.89460, 0.89720, 0.89954, 0.90193, 0.90768, 0.91379, 0.91840,\
        0.92072, 0.92030, 0.91745, 0.91249, 0.90624, 0.89866, 0.88883, 0.87678, 0.86529, 0.85528, 0.84699,\
        0.84023, 0.83453, 0.82919, 0.82342, 0.81660, 0.80844, 0.79916, 0.78978, 0.77917, 0.76800, 0.75674,\
        0.74562, 0.73470, 0.72392, 0.71321, 0.70270, 0.69289, 0.68387, 0.67541, 0.66723, 0.65914, 0.65105,\
        0.64291, 0.63476, 0.62666, 0.61868, 0.61094, 0.60361, 0.59660, 0.58967, 0.58309, 0.57678, 0.57059,\
        0.56446, 0.55835, 0.55225, 0.54615, 0.54006, 0.53400, 0.52798, 0.52201, 0.51612, 0.51030, 0.50456,\
        0.49914, 0.49453, 0.48986, 0.48509, 0.48021, 0.47520, 0.47004, 0.46472, 0.45922, 0.45355, 0.44769],\
        [0.49885, 0.49814, 0.49725, 0.49643, 0.49592, 0.49599, 0.49691, 0.49900, 0.50258, 0.50758, 0.51395,\
        0.52151, 0.53001, 0.53920, 0.54885, 0.55872, 0.56852, 0.57797, 0.58672, 0.59470, 0.60232, 0.60999,\
        0.61809, 0.62643, 0.63497, 0.64372, 0.65275, 0.66210, 0.67184, 0.68202, 0.69269, 0.70390, 0.71579,\
        0.72849, 0.74166, 0.75477, 0.76736, 0.77912, 0.78998, 0.79987, 0.80899, 0.81831, 0.82843, 0.83951,\
        0.85132, 0.86313, 0.87365, 0.88148, 0.88624, 0.88901, 0.89105, 0.89349, 0.89828, 0.90257, 0.90623,\
        0.90786, 0.90673, 0.90396, 0.89898, 0.89306, 0.88583, 0.87670, 0.86607, 0.85550, 0.84617, 0.83863,\
        0.83283, 0.82823, 0.82400, 0.81913, 0.81272, 0.80451, 0.79484, 0.78440, 0.77344, 0.76216, 0.75096,\
        0.74010, 0.72965, 0.71951, 0.70942, 0.69921, 0.68941, 0.68012, 0.67131, 0.66284, 0.65454, 0.64631,\
        0.63811, 0.62994, 0.62184, 0.61388, 0.60613, 0.59875, 0.59192, 0.58524, 0.57876, 0.57250, 0.56640,\
        0.56040, 0.55448, 0.54861, 0.54279, 0.53701, 0.53128, 0.52561, 0.52003, 0.51454, 0.50918, 0.50399,\
        0.49902, 0.49413, 0.48936, 0.48457, 0.47970, 0.47471, 0.46958, 0.46429, 0.45883, 0.45318, 0.44734],\
        [0.49697, 0.49648, 0.49574, 0.49503, 0.49461, 0.49473, 0.49565, 0.49768, 0.50103, 0.50582, 0.51214,\
        0.51966, 0.52809, 0.53717, 0.54665, 0.55630, 0.56585, 0.57504, 0.58364, 0.59162, 0.59921, 0.60756,\
        0.61625, 0.62504, 0.63394, 0.64302, 0.65237, 0.66208, 0.67226, 0.68294, 0.69393, 0.70441, 0.71430,\
        0.72486, 0.73637, 0.74843, 0.76058, 0.77248, 0.78391, 0.79471, 0.80483, 0.81454, 0.82446, 0.83485,\
        0.84545, 0.85555, 0.86428, 0.87091, 0.87537, 0.87829, 0.88063, 0.88341, 0.88706, 0.89068, 0.89372,\
        0.89492, 0.89354, 0.89071, 0.88612, 0.88009, 0.87273, 0.86385, 0.85393, 0.84405, 0.83536, 0.82863,\
        0.82391, 0.82059, 0.81768, 0.81401, 0.80816, 0.79951, 0.78927, 0.77812, 0.76664, 0.75518, 0.74404,\
        0.73348, 0.72366, 0.71441, 0.70513, 0.69547, 0.68560, 0.67613, 0.66712, 0.65846, 0.65002, 0.64171,\
        0.63346, 0.62528, 0.61720, 0.60930, 0.60163, 0.59433, 0.58745, 0.58085, 0.57446, 0.56827, 0.56226,\
        0.55640, 0.55066, 0.54503, 0.53948, 0.53401, 0.52861, 0.52331, 0.51811, 0.51305, 0.50816, 0.50344,\
        0.49870, 0.49384, 0.48899, 0.48416, 0.47927, 0.47429, 0.46917, 0.46390, 0.45845, 0.45281, 0.44697],\
        [0.49485, 0.49455, 0.49402, 0.49350, 0.49322, 0.49343, 0.49435, 0.49622, 0.49926, 0.50403, 0.51038,\
        0.51787, 0.52619, 0.53513, 0.54445, 0.55393, 0.56334, 0.57244, 0.58109, 0.58929, 0.59735, 0.60587,\
        0.61471, 0.62364, 0.63267, 0.64186, 0.65130, 0.66110, 0.67131, 0.68195, 0.69276, 0.70239, 0.71038,\
        0.71940, 0.72972, 0.74090, 0.75252, 0.76426, 0.77584, 0.78700, 0.79754, 0.80734, 0.81684, 0.82655,\
        0.83618, 0.84505, 0.85260, 0.85845, 0.86263, 0.86564, 0.86824, 0.87116, 0.87456, 0.87797, 0.88064,\
        0.88169, 0.88069, 0.87792, 0.87348, 0.86745, 0.85994, 0.85106, 0.84132, 0.83167, 0.82330, 0.81721,\
        0.81360, 0.81149, 0.80972, 0.80729, 0.80234, 0.79308, 0.78202, 0.77033, 0.75853, 0.74696, 0.73589,\
        0.72559, 0.71633, 0.70806, 0.69983, 0.69039, 0.68085, 0.67154, 0.66257, 0.65392, 0.64548, 0.63718,\
        0.62894, 0.62077, 0.61273, 0.60492, 0.59736, 0.59011, 0.58321, 0.57660, 0.57023, 0.56410, 0.55818,\
        0.55245, 0.54690, 0.54149, 0.53621, 0.53105, 0.52599, 0.52103, 0.51619, 0.51148, 0.50696, 0.50258,\
        0.49808, 0.49333, 0.48854, 0.48373, 0.47885, 0.47388, 0.46877, 0.46351, 0.45806, 0.45243, 0.44659],\
        [0.49265, 0.49253, 0.49222, 0.49192, 0.49184, 0.49220, 0.49320, 0.49506, 0.49803, 0.50269, 0.50895,\
        0.51628, 0.52443, 0.53319, 0.54236, 0.55172, 0.56108, 0.57024, 0.57910, 0.58767, 0.59613, 0.60470,\
        0.61339, 0.62219, 0.63108, 0.64013, 0.64940, 0.65893, 0.66870, 0.67857, 0.68809, 0.69632, 0.70373,\
        0.71208, 0.72181, 0.73229, 0.74334, 0.75476, 0.76623, 0.77736, 0.78787, 0.79752, 0.80657, 0.81586,\
        0.82480, 0.83275, 0.83944, 0.84473, 0.84861, 0.85151, 0.85421, 0.85733, 0.86097, 0.86460, 0.86741,\
        0.86868, 0.86805, 0.86553, 0.86126, 0.85530, 0.84777, 0.83885, 0.82904, 0.81927, 0.81088, 0.80506,\
        0.80246, 0.80109, 0.79975, 0.79749, 0.79264, 0.78362, 0.77252, 0.76076, 0.74899, 0.73756, 0.72667,\
        0.71655, 0.70758, 0.69991, 0.69241, 0.68394, 0.67505, 0.66620, 0.65755, 0.64911, 0.64085, 0.63270,\
        0.62457, 0.61643, 0.60841, 0.60069, 0.59322, 0.58602, 0.57909, 0.57245, 0.56608, 0.55997, 0.55413,\
        0.54853, 0.54317, 0.53800, 0.53300, 0.52814, 0.52341, 0.51878, 0.51425, 0.50980, 0.50548, 0.50131,\
        0.49704, 0.49255, 0.48793, 0.48322, 0.47840, 0.47346, 0.46837, 0.46311, 0.45766, 0.45202, 0.44617],\
        [0.49046, 0.49052, 0.49044, 0.49037, 0.49052, 0.49109, 0.49229, 0.49433, 0.49749, 0.50204, 0.50794,\
        0.51493, 0.52281, 0.53137, 0.54039, 0.54967, 0.55903, 0.56833, 0.57744, 0.58634, 0.59504, 0.60358,\
        0.61204, 0.62051, 0.62908, 0.63779, 0.64666, 0.65567, 0.66471, 0.67357, 0.68187, 0.68926, 0.69614,\
        0.70419, 0.71340, 0.72296, 0.73337, 0.74444, 0.75570, 0.76668, 0.77700, 0.78644, 0.79526, 0.80415,\
        0.81246, 0.81946, 0.82548, 0.83036, 0.83388, 0.83641, 0.83897, 0.84239, 0.84667, 0.85100, 0.85438,\
        0.85607, 0.85577, 0.85353, 0.84950, 0.84377, 0.83643, 0.82761, 0.81778, 0.80790, 0.79955, 0.79410,\
        0.79150, 0.78999, 0.78824, 0.78524, 0.77983, 0.77140, 0.76090, 0.74956, 0.73822, 0.72728, 0.71683,\
        0.70707, 0.69831, 0.69107, 0.68404, 0.67648, 0.66844, 0.66024, 0.65208, 0.64402, 0.63609, 0.62825,\
        0.62037, 0.61235, 0.60427, 0.59662, 0.58925, 0.58204, 0.57506, 0.56835, 0.56195, 0.55586, 0.55009,\
        0.54463, 0.53945, 0.53453, 0.52982, 0.52528, 0.52088, 0.51657, 0.51233, 0.50812, 0.50391, 0.49974,\
        0.49579, 0.49161, 0.48721, 0.48264, 0.47790, 0.47301, 0.46794, 0.46268, 0.45723, 0.45158, 0.44572],\
        [0.48833, 0.48858, 0.48872, 0.48890, 0.48929, 0.49009, 0.49151, 0.49378, 0.49714, 0.50160, 0.50706,\
        0.51368, 0.52128, 0.52963, 0.53850, 0.54771, 0.55709, 0.56650, 0.57585, 0.58502, 0.59385, 0.60224,\
        0.61043, 0.61844, 0.62655, 0.63482, 0.64318, 0.65157, 0.65986, 0.66787, 0.67538, 0.68238, 0.68935,\
        0.69721, 0.70480, 0.71324, 0.72306, 0.73388, 0.74496, 0.75577, 0.76595, 0.77532, 0.78405, 0.79236,\
        0.79973, 0.80568, 0.81123, 0.81595, 0.81904, 0.82084, 0.82300, 0.82682, 0.83222, 0.83771, 0.84187,\
        0.84399, 0.84393, 0.84189, 0.83811, 0.83278, 0.82594, 0.81749, 0.80788, 0.79836, 0.79035, 0.78482,\
        0.78136, 0.77890, 0.77622, 0.77231, 0.76636, 0.75809, 0.74807, 0.73726, 0.72659, 0.71656, 0.70693,\
        0.69784, 0.68978, 0.68264, 0.67585, 0.66883, 0.66148, 0.65391, 0.64627, 0.63867, 0.63116, 0.62376,\
        0.61638, 0.60876, 0.60075, 0.59301, 0.58553, 0.57818, 0.57107, 0.56426, 0.55781, 0.55173, 0.54603,\
        0.54071, 0.53574, 0.53108, 0.52668, 0.52247, 0.51841, 0.51444, 0.51051, 0.50660, 0.50266, 0.49870,\
        0.49478, 0.49074, 0.48650, 0.48204, 0.47738, 0.47253, 0.46748, 0.46222, 0.45677, 0.45109, 0.44520],\
        [0.48629, 0.48674, 0.48710, 0.48751, 0.48813, 0.48914, 0.49072, 0.49308, 0.49638, 0.50062, 0.50593,\
        0.51236, 0.51976, 0.52792, 0.53664, 0.54575, 0.55510, 0.56456, 0.57403, 0.58342, 0.59259, 0.60108,\
        0.60834, 0.61572, 0.62342, 0.63125, 0.63911, 0.64692, 0.65456, 0.66192, 0.66892, 0.67564, 0.68236,\
        0.68921, 0.69608, 0.70354, 0.71315, 0.72377, 0.73459, 0.74522, 0.75524, 0.76444, 0.77281, 0.78037,\
        0.78682, 0.79221, 0.79726, 0.80182, 0.80449, 0.80531, 0.80701, 0.81136, 0.81841, 0.82531, 0.83018,\
        0.83249, 0.83242, 0.83041, 0.82682, 0.82196, 0.81595, 0.80825, 0.79895, 0.78985, 0.78204, 0.77605,\
        0.77168, 0.76813, 0.76442, 0.75967, 0.75323, 0.74490, 0.73507, 0.72462, 0.71457, 0.70567, 0.69743,\
        0.68908, 0.68155, 0.67466, 0.66804, 0.66137, 0.65450, 0.64744, 0.64028, 0.63310, 0.62598, 0.61903,\
        0.61232, 0.60552, 0.59790, 0.58983, 0.58194, 0.57432, 0.56703, 0.56010, 0.55359, 0.54752, 0.54191,\
        0.53675, 0.53200, 0.52763, 0.52356, 0.51972, 0.51603, 0.51241, 0.50881, 0.50520, 0.50158, 0.49785,\
        0.49398, 0.49000, 0.48583, 0.48145, 0.47684, 0.47202, 0.46697, 0.46172, 0.45624, 0.45055, 0.44463],\
        [0.48436, 0.48500, 0.48559, 0.48623, 0.48706, 0.48823, 0.48991, 0.49225, 0.49537, 0.49942, 0.50465,\
        0.51097, 0.51821, 0.52620, 0.53476, 0.54372, 0.55294, 0.56231, 0.57170, 0.58101, 0.59004, 0.59802,\
        0.60483, 0.61221, 0.61972, 0.62720, 0.63460, 0.64189, 0.64901, 0.65590, 0.66254, 0.66898, 0.67536,\
        0.68177, 0.68832, 0.69568, 0.70500, 0.71461, 0.72502, 0.73543, 0.74518, 0.75399, 0.76182, 0.76864,\
        0.77440, 0.77930, 0.78366, 0.78739, 0.78961, 0.79063, 0.79266, 0.79773, 0.80626, 0.81415, 0.81926,\
        0.82134, 0.82093, 0.81879, 0.81525, 0.81064, 0.80546, 0.79907, 0.79004, 0.78119, 0.77345, 0.76712,\
        0.76205, 0.75766, 0.75316, 0.74780, 0.74102, 0.73262, 0.72284, 0.71255, 0.70304, 0.69496, 0.68778,\
        0.68057, 0.67363, 0.66704, 0.66064, 0.65422, 0.64769, 0.64101, 0.63422, 0.62738, 0.62055, 0.61382,\
        0.60748, 0.60171, 0.59434, 0.58614, 0.57804, 0.57023, 0.56280, 0.55578, 0.54923, 0.54319, 0.53769,\
        0.53271, 0.52823, 0.52418, 0.52049, 0.51704, 0.51374, 0.51049, 0.50722, 0.50387, 0.50044, 0.49693,\
        0.49323, 0.48933, 0.48522, 0.48087, 0.47629, 0.47147, 0.46643, 0.46115, 0.45565, 0.44993, 0.44397],\
        [0.48253, 0.48338, 0.48419, 0.48505, 0.48607, 0.48740, 0.48915, 0.49146, 0.49449, 0.49840, 0.50346,\
        0.50961, 0.51667, 0.52447, 0.53283, 0.54158, 0.55058, 0.55969, 0.56880, 0.57772, 0.58614, 0.59367,\
        0.60081, 0.60833, 0.61564, 0.62274, 0.62973, 0.63661, 0.64333, 0.64988, 0.65625, 0.66252, 0.66876,\
        0.67510, 0.68179, 0.68930, 0.69737, 0.70626, 0.71661, 0.72671, 0.73597, 0.74419, 0.75133, 0.75740,\
        0.76245, 0.76663, 0.77012, 0.77291, 0.77493, 0.77684, 0.78017, 0.78635, 0.79523, 0.80313, 0.80838,\
        0.80994, 0.80893, 0.80673, 0.80325, 0.79856, 0.79353, 0.78746, 0.77966, 0.77150, 0.76404, 0.75764,\
        0.75222, 0.74738, 0.74248, 0.73686, 0.73002, 0.72178, 0.71224, 0.70210, 0.69330, 0.68584, 0.67909,\
        0.67254, 0.66612, 0.65984, 0.65364, 0.64743, 0.64112, 0.63471, 0.62822, 0.62165, 0.61501, 0.60831,\
        0.60168, 0.59581, 0.58918, 0.58147, 0.57354, 0.56576, 0.55830, 0.55125, 0.54471, 0.53873, 0.53335,\
        0.52858, 0.52440, 0.52072, 0.51745, 0.51444, 0.51157, 0.50872, 0.50579, 0.50271, 0.49946, 0.49612,\
        0.49254, 0.48871, 0.48463, 0.48029, 0.47571, 0.47088, 0.46581, 0.46051, 0.45498, 0.44921, 0.44321],\
        [0.48079, 0.48186, 0.48289, 0.48397, 0.48520, 0.48668, 0.48851, 0.49083, 0.49379, 0.49760, 0.50245,\
        0.50835, 0.51520, 0.52278, 0.53089, 0.53934, 0.54801, 0.55676, 0.56547, 0.57397, 0.58207, 0.58971,\
        0.59719, 0.60451, 0.61124, 0.61792, 0.62457, 0.63112, 0.63755, 0.64387, 0.65009, 0.65628, 0.66253,\
        0.66898, 0.67580, 0.68314, 0.69098, 0.69997, 0.70971, 0.71914, 0.72765, 0.73507, 0.74139, 0.74665,\
        0.75092, 0.75427, 0.75687, 0.75888, 0.76071, 0.76315, 0.76732, 0.77393, 0.78230, 0.79024, 0.79592,\
        0.79733, 0.79617, 0.79400, 0.79067, 0.78612, 0.78094, 0.77493, 0.76796, 0.76064, 0.75368, 0.74747,\
        0.74205, 0.73716, 0.73231, 0.72680, 0.72017, 0.71238, 0.70357, 0.69414, 0.68560, 0.67823, 0.67159,\
        0.66530, 0.65917, 0.65312, 0.64709, 0.64101, 0.63484, 0.62861, 0.62233, 0.61600, 0.60959, 0.60300,\
        0.59637, 0.59003, 0.58341, 0.57614, 0.56853, 0.56091, 0.55353, 0.54653, 0.54002, 0.53412, 0.52889,\
        0.52436, 0.52051, 0.51726, 0.51446, 0.51195, 0.50954, 0.50712, 0.50456, 0.50180, 0.49877, 0.49548,\
        0.49194, 0.48813, 0.48405, 0.47970, 0.47508, 0.47022, 0.46512, 0.45978, 0.45420, 0.44839, 0.44235],\
        [0.47912, 0.48041, 0.48168, 0.48300, 0.48445, 0.48610, 0.48805, 0.49040, 0.49329, 0.49692, 0.50151,\
        0.50718, 0.51384, 0.52119, 0.52898, 0.53705, 0.54530, 0.55360, 0.56186, 0.56996, 0.57782, 0.58545,\
        0.59291, 0.59976, 0.60619, 0.61269, 0.61911, 0.62543, 0.63167, 0.63784, 0.64399, 0.65019, 0.65655,\
        0.66316, 0.67017, 0.67766, 0.68579, 0.69464, 0.70380, 0.71244, 0.71998, 0.72644, 0.73188, 0.73634,\
        0.73981, 0.74234, 0.74407, 0.74530, 0.74663, 0.74899, 0.75331, 0.75991, 0.76791, 0.77557, 0.78107,\
        0.78328, 0.78283, 0.78080, 0.77751, 0.77312, 0.76794, 0.76207, 0.75562, 0.74895, 0.74254, 0.73669,\
        0.73149, 0.72686, 0.72246, 0.71746, 0.71112, 0.70369, 0.69570, 0.68704, 0.67887, 0.67156, 0.66496,\
        0.65878, 0.65280, 0.64690, 0.64098, 0.63497, 0.62888, 0.62273, 0.61658, 0.61042, 0.60422, 0.59786,\
        0.59120, 0.58458, 0.57780, 0.57068, 0.56329, 0.55587, 0.54860, 0.54167, 0.53523, 0.52941, 0.52433,\
        0.52006, 0.51659, 0.51383, 0.51157, 0.50959, 0.50768, 0.50568, 0.50347, 0.50099, 0.49813, 0.49491,\
        0.49138, 0.48756, 0.48344, 0.47904, 0.47438, 0.46947, 0.46431, 0.45892, 0.45330, 0.44744, 0.44135],\
        [0.47749, 0.47902, 0.48054, 0.48212, 0.48381, 0.48567, 0.48778, 0.49021, 0.49305, 0.49641, 0.50063,\
        0.50623, 0.51275, 0.51978, 0.52716, 0.53475, 0.54248, 0.55027, 0.55803, 0.56569, 0.57321, 0.58055,\
        0.58761, 0.59424, 0.60069, 0.60714, 0.61338, 0.61956, 0.62568, 0.63177, 0.63788, 0.64414, 0.65063,\
        0.65743, 0.66464, 0.67233, 0.68056, 0.68928, 0.69810, 0.70591, 0.71243, 0.71795, 0.72258, 0.72631,\
        0.72910, 0.73090, 0.73179, 0.73214, 0.73264, 0.73432, 0.73832, 0.74491, 0.75292, 0.76049, 0.76606,\
        0.76885, 0.76906, 0.76733, 0.76413, 0.75983, 0.75473, 0.74906, 0.74301, 0.73685, 0.73092, 0.72551,\
        0.72063, 0.71627, 0.71270, 0.70866, 0.70259, 0.69533, 0.68773, 0.67989, 0.67229, 0.66526, 0.65881,\
        0.65277, 0.64693, 0.64116, 0.63532, 0.62936, 0.62325, 0.61709, 0.61098, 0.60493, 0.59880, 0.59249,\
        0.58595, 0.57925, 0.57239, 0.56532, 0.55808, 0.55080, 0.54365, 0.53681, 0.53042, 0.52467, 0.51972,\
        0.51572, 0.51267, 0.51046, 0.50880, 0.50741, 0.50603, 0.50443, 0.50250, 0.50018, 0.49747, 0.49433,\
        0.49080, 0.48693, 0.48275, 0.47828, 0.47356, 0.46858, 0.46337, 0.45792, 0.45225, 0.44634, 0.44020],\
        [0.47587, 0.47764, 0.47942, 0.48126, 0.48323, 0.48535, 0.48769, 0.49030, 0.49325, 0.49663, 0.50069,\
        0.50599, 0.51209, 0.51863, 0.52545, 0.53245, 0.53957, 0.54677, 0.55397, 0.56115, 0.56824, 0.57521,\
        0.58198, 0.58853, 0.59499, 0.60124, 0.60735, 0.61349, 0.61957, 0.62558, 0.63168, 0.63800, 0.64462,\
        0.65159, 0.65894, 0.66670, 0.67485, 0.68322, 0.69140, 0.69861, 0.70442, 0.70923, 0.71321, 0.71638,\
        0.71866, 0.71991, 0.72005, 0.71952, 0.71901, 0.71952, 0.72293, 0.72992, 0.73844, 0.74620, 0.75189,\
        0.75496, 0.75551, 0.75396, 0.75082, 0.74656, 0.74157, 0.73612, 0.73042, 0.72470, 0.71914, 0.71422,\
        0.70987, 0.70533, 0.70249, 0.69979, 0.69391, 0.68697, 0.67977, 0.67255, 0.66556, 0.65898, 0.65285,\
        0.64705, 0.64142, 0.63582, 0.63010, 0.62419, 0.61804, 0.61174, 0.60557, 0.59961, 0.59352, 0.58724,\
        0.58074, 0.57405, 0.56718, 0.56016, 0.55302, 0.54588, 0.53884, 0.53208, 0.52575, 0.52004, 0.51518,\
        0.51140, 0.50882, 0.50723, 0.50624, 0.50548, 0.50462, 0.50344, 0.50175, 0.49954, 0.49689, 0.49374,\
        0.49015, 0.48620, 0.48193, 0.47737, 0.47257, 0.46752, 0.46225, 0.45675, 0.45102, 0.44506, 0.43887],\
        [0.47419, 0.47620, 0.47825, 0.48038, 0.48264, 0.48507, 0.48770, 0.49058, 0.49376, 0.49739, 0.50157,\
        0.50634, 0.51179, 0.51767, 0.52380, 0.53011, 0.53655, 0.54308, 0.54969, 0.55633, 0.56296, 0.56956,\
        0.57607, 0.58248, 0.58880, 0.59497, 0.60108, 0.60733, 0.61336, 0.61921, 0.62528, 0.63168, 0.63842,\
        0.64550, 0.65291, 0.66062, 0.66851, 0.67638, 0.68382, 0.69037, 0.69574, 0.70007, 0.70358, 0.70631,\
        0.70825, 0.70921, 0.70875, 0.70752, 0.70637, 0.70551, 0.70819, 0.71641, 0.72564, 0.73348, 0.73908,\
        0.74211, 0.74267, 0.74110, 0.73790, 0.73358, 0.72869, 0.72355, 0.71812, 0.71291, 0.70771, 0.70285,\
        0.69985, 0.69510, 0.69200, 0.68904, 0.68407, 0.67792, 0.67140, 0.66487, 0.65855, 0.65254, 0.64687,\
        0.64146, 0.63615, 0.63081, 0.62528, 0.61949, 0.61336, 0.60689, 0.60041, 0.59454, 0.58849, 0.58222,\
        0.57573, 0.56905, 0.56222, 0.55526, 0.54823, 0.54122, 0.53432, 0.52766, 0.52140, 0.51571, 0.51087,\
        0.50724, 0.50515, 0.50428, 0.50398, 0.50381, 0.50348, 0.50270, 0.50126, 0.49911, 0.49636, 0.49308,\
        0.48937, 0.48529, 0.48090, 0.47625, 0.47136, 0.46625, 0.46091, 0.45536, 0.44959, 0.44359, 0.43736],\
        [0.47241, 0.47465, 0.47695, 0.47936, 0.48192, 0.48468, 0.48765, 0.49085, 0.49428, 0.49797, 0.50208,\
        0.50665, 0.51154, 0.51671, 0.52208, 0.52763, 0.53333, 0.53917, 0.54514, 0.55123, 0.55740, 0.56363,\
        0.56988, 0.57612, 0.58235, 0.58855, 0.59480, 0.60119, 0.60696, 0.61251, 0.61863, 0.62516, 0.63200,\
        0.63914, 0.64654, 0.65409, 0.66165, 0.66898, 0.67578, 0.68172, 0.68662, 0.69054, 0.69359, 0.69589,\
        0.69745, 0.69821, 0.69742, 0.69583, 0.69474, 0.69416, 0.69737, 0.70645, 0.71511, 0.72250, 0.72771,\
        0.73046, 0.73080, 0.72902, 0.72569, 0.72120, 0.71630, 0.71178, 0.70643, 0.70150, 0.69730, 0.69236,\
        0.68885, 0.68497, 0.68142, 0.67788, 0.67345, 0.66817, 0.66250, 0.65676, 0.65118, 0.64584, 0.64076,\
        0.63586, 0.63100, 0.62602, 0.62078, 0.61521, 0.60928, 0.60297, 0.59639, 0.59011, 0.58389, 0.57752,\
        0.57099, 0.56431, 0.55752, 0.55066, 0.54376, 0.53691, 0.53018, 0.52369, 0.51757, 0.51196, 0.50712,\
        0.50347, 0.50191, 0.50184, 0.50209, 0.50240, 0.50252, 0.50211, 0.50082, 0.49863, 0.49570, 0.49223,\
        0.48834, 0.48412, 0.47961, 0.47486, 0.46989, 0.46471, 0.45932, 0.45373, 0.44792, 0.44189, 0.43563],\
        [0.47046, 0.47290, 0.47543, 0.47810, 0.48096, 0.48403, 0.48734, 0.49090, 0.49466, 0.49852, 0.50256,\
        0.50677, 0.51108, 0.51552, 0.52012, 0.52489, 0.52984, 0.53496, 0.54029, 0.54582, 0.55154, 0.55742,\
        0.56341, 0.56950, 0.57565, 0.58188, 0.58819, 0.59444, 0.59990, 0.60540, 0.61188, 0.61851, 0.62539,\
        0.63255, 0.63987, 0.64723, 0.65445, 0.66129, 0.66752, 0.67289, 0.67730, 0.68074, 0.68329, 0.68504,\
        0.68603, 0.68620, 0.68536, 0.68412, 0.68358, 0.68482, 0.68972, 0.69749, 0.70557, 0.71261, 0.71733,\
        0.71987, 0.71997, 0.71779, 0.71448, 0.70999, 0.70438, 0.70097, 0.69609, 0.69084, 0.68644, 0.68201,\
        0.67800, 0.67419, 0.67049, 0.66675, 0.66261, 0.65802, 0.65316, 0.64825, 0.64344, 0.63883, 0.63443,\
        0.63017, 0.62588, 0.62136, 0.61646, 0.61115, 0.60545, 0.59933, 0.59276, 0.58615, 0.57963, 0.57309,\
        0.56649, 0.55981, 0.55309, 0.54635, 0.53963, 0.53299, 0.52651, 0.52028, 0.51441, 0.50907, 0.50445,\
        0.50091, 0.49972, 0.50009, 0.50051, 0.50109, 0.50154, 0.50145, 0.50025, 0.49785, 0.49468, 0.49100,\
        0.48694, 0.48258, 0.47796, 0.47313, 0.46810, 0.46287, 0.45745, 0.45183, 0.44600, 0.43996, 0.43367],\
        [0.46829, 0.47089, 0.47361, 0.47650, 0.47959, 0.48293, 0.48654, 0.49042, 0.49455, 0.49882, 0.50272,\
        0.50642, 0.51012, 0.51388, 0.51774, 0.52175, 0.52596, 0.53039, 0.53509, 0.54009, 0.54538, 0.55093,\
        0.55669, 0.56261, 0.56868, 0.57485, 0.58104, 0.58707, 0.59273, 0.59882, 0.60539, 0.61176, 0.61866,\
        0.62580, 0.63302, 0.64018, 0.64708, 0.65348, 0.65919, 0.66404, 0.66791, 0.67082, 0.67279, 0.67392,\
        0.67427, 0.67388, 0.67296, 0.67208, 0.67222, 0.67444, 0.67954, 0.68696, 0.69531, 0.70266, 0.70693,\
        0.71017, 0.71027, 0.70714, 0.70398, 0.70079, 0.69476, 0.69032, 0.68572, 0.68062, 0.67584, 0.67134,\
        0.66713, 0.66318, 0.65937, 0.65560, 0.65172, 0.64769, 0.64355, 0.63941, 0.63536, 0.63149, 0.62782,\
        0.62428, 0.62069, 0.61676, 0.61222, 0.60706, 0.60152, 0.59540, 0.58887, 0.58218, 0.57548, 0.56880,\
        0.56215, 0.55551, 0.54889, 0.54232, 0.53583, 0.52946, 0.52329, 0.51740, 0.51193, 0.50703, 0.50294,\
        0.50004, 0.49875, 0.49872, 0.49911, 0.49974, 0.50017, 0.50018, 0.49902, 0.49639, 0.49305, 0.48922,\
        0.48505, 0.48059, 0.47590, 0.47102, 0.46594, 0.46069, 0.45526, 0.44964, 0.44381, 0.43776, 0.43147],\
        [0.46584, 0.46856, 0.47142, 0.47445, 0.47770, 0.48121, 0.48501, 0.48909, 0.49338, 0.49770, 0.50166,\
        0.50514, 0.50838, 0.51154, 0.51474, 0.51809, 0.52162, 0.52538, 0.52948, 0.53399, 0.53890, 0.54417,\
        0.54972, 0.55550, 0.56147, 0.56756, 0.57371, 0.57982, 0.58594, 0.59232, 0.59853, 0.60488, 0.61198,\
        0.61900, 0.62610, 0.63308, 0.63968, 0.64569, 0.65093, 0.65526, 0.65860, 0.66093, 0.66229, 0.66278,\
        0.66249, 0.66161, 0.66045, 0.65962, 0.65997, 0.66238, 0.66737, 0.67461, 0.68297, 0.69065, 0.69600,\
        0.70165, 0.70149, 0.69739, 0.69395, 0.69056, 0.68561, 0.68044, 0.67539, 0.67026, 0.66528, 0.66059,\
        0.65621, 0.65210, 0.64821, 0.64448, 0.64084, 0.63729, 0.63380, 0.63037, 0.62701, 0.62382, 0.62086,\
        0.61811, 0.61533, 0.61216, 0.60806, 0.60286, 0.59726, 0.59122, 0.58471, 0.57799, 0.57122, 0.56450,\
        0.55786, 0.55130, 0.54485, 0.53850, 0.53229, 0.52626, 0.52045, 0.51497, 0.50995, 0.50554, 0.50196,\
        0.49942, 0.49803, 0.49760, 0.49771, 0.49803, 0.49819, 0.49784, 0.49649, 0.49397, 0.49065, 0.48681,\
        0.48259, 0.47810, 0.47338, 0.46847, 0.46340, 0.45815, 0.45273, 0.44713, 0.44132, 0.43529, 0.42901],\
        [0.46308, 0.46586, 0.46878, 0.47187, 0.47520, 0.47877, 0.48262, 0.48672, 0.49098, 0.49524, 0.49922,\
        0.50264, 0.50559, 0.50830, 0.51099, 0.51382, 0.51676, 0.51989, 0.52343, 0.52752, 0.53213, 0.53718,\
        0.54256, 0.54821, 0.55409, 0.56016, 0.56635, 0.57262, 0.57897, 0.58542, 0.59184, 0.59877, 0.60546,\
        0.61216, 0.61922, 0.62604, 0.63236, 0.63800, 0.64280, 0.64665, 0.64946, 0.65121, 0.65197, 0.65182,\
        0.65094, 0.64954, 0.64799, 0.64682, 0.64684, 0.64888, 0.65353, 0.66060, 0.66901, 0.67731, 0.68459,\
        0.69019, 0.69086, 0.68790, 0.68428, 0.68037, 0.67568, 0.67049, 0.66517, 0.65986, 0.65470, 0.64984,\
        0.64531, 0.64108, 0.63713, 0.63344, 0.63002, 0.62690, 0.62403, 0.62124, 0.61847, 0.61583, 0.61349,\
        0.61150, 0.60959, 0.60737, 0.60413, 0.59893, 0.59306, 0.58681, 0.58023, 0.57349, 0.56673, 0.56005,\
        0.55350, 0.54710, 0.54088, 0.53483, 0.52897, 0.52331, 0.51791, 0.51285, 0.50826, 0.50430, 0.50109,\
        0.49871, 0.49720, 0.49643, 0.49613, 0.49600, 0.49570, 0.49490, 0.49331, 0.49081, 0.48757, 0.48377,\
        0.47958, 0.47510, 0.47039, 0.46549, 0.46044, 0.45522, 0.44985, 0.44429, 0.43853, 0.43254, 0.42629],\
        [0.46000, 0.46277, 0.46567, 0.46875, 0.47204, 0.47557, 0.47933, 0.48331, 0.48740, 0.49148, 0.49532,\
        0.49873, 0.50160, 0.50400, 0.50638, 0.50893, 0.51141, 0.51390, 0.51693, 0.52071, 0.52514, 0.53004,\
        0.53528, 0.54081, 0.54663, 0.55269, 0.55896, 0.56539, 0.57195, 0.57860, 0.58535, 0.59218, 0.59847,\
        0.60549, 0.61253, 0.61915, 0.62518, 0.63045, 0.63485, 0.63825, 0.64056, 0.64177, 0.64194, 0.64121,\
        0.63977, 0.63787, 0.63582, 0.63406, 0.63330, 0.63450, 0.63862, 0.64564, 0.65438, 0.66329, 0.67109,\
        0.67653, 0.67834, 0.67702, 0.67405, 0.67020, 0.66559, 0.66043, 0.65498, 0.64949, 0.64415, 0.63915,\
        0.63454, 0.63025, 0.62626, 0.62259, 0.61933, 0.61660, 0.61435, 0.61218, 0.60987, 0.60759, 0.60571,\
        0.60434, 0.60319, 0.60161, 0.59982, 0.59464, 0.58842, 0.58194, 0.57530, 0.56858, 0.56190, 0.55534,\
        0.54897, 0.54282, 0.53691, 0.53123, 0.52577, 0.52055, 0.51557, 0.51091, 0.50670, 0.50307, 0.50011,\
        0.49781, 0.49616, 0.49507, 0.49433, 0.49369, 0.49289, 0.49165, 0.48978, 0.48720, 0.48397, 0.48021,\
        0.47606, 0.47161, 0.46693, 0.46207, 0.45706, 0.45191, 0.44660, 0.44112, 0.43543, 0.42950, 0.42330],\
        [0.45658, 0.45928, 0.46210, 0.46508, 0.46824, 0.47162, 0.47519, 0.47894, 0.48277, 0.48658, 0.49019,\
        0.49346, 0.49627, 0.49861, 0.50092, 0.50350, 0.50567, 0.50739, 0.51001, 0.51367, 0.51805, 0.52286,\
        0.52797, 0.53338, 0.53913, 0.54521, 0.55158, 0.55817, 0.56492, 0.57177, 0.57867, 0.58551, 0.59231,\
        0.59938, 0.60598, 0.61242, 0.61813, 0.62301, 0.62704, 0.63006, 0.63194, 0.63264, 0.63225, 0.63098,\
        0.62908, 0.62678, 0.62430, 0.62193, 0.62016, 0.62006, 0.62347, 0.63079, 0.64009, 0.64941, 0.65741,\
        0.66304, 0.66568, 0.66554, 0.66341, 0.65994, 0.65552, 0.65041, 0.64489, 0.63923, 0.63372, 0.62862,\
        0.62403, 0.61979, 0.61576, 0.61213, 0.60895, 0.60647, 0.60490, 0.60329, 0.60132, 0.59933, 0.59776,\
        0.59676, 0.59594, 0.59461, 0.59248, 0.58826, 0.58252, 0.57622, 0.56972, 0.56315, 0.55665, 0.55031,\
        0.54420, 0.53838, 0.53285, 0.52762, 0.52265, 0.51791, 0.51338, 0.50909, 0.50515, 0.50176, 0.49896,\
        0.49668, 0.49489, 0.49348, 0.49230, 0.49116, 0.48986, 0.48820, 0.48603, 0.48329, 0.48000, 0.47624,\
        0.47210, 0.46768, 0.46303, 0.45822, 0.45327, 0.44820, 0.44298, 0.43760, 0.43201, 0.42617, 0.42003],\
        [0.45284, 0.45542, 0.45809, 0.46089, 0.46385, 0.46699, 0.47030, 0.47375, 0.47727, 0.48076, 0.48410,\
        0.48719, 0.48993, 0.49240, 0.49484, 0.49747, 0.49937, 0.50037, 0.50295, 0.50665, 0.51107, 0.51580,\
        0.52072, 0.52596, 0.53164, 0.53777, 0.54426, 0.55102, 0.55795, 0.56497, 0.57202, 0.57903, 0.58601,\
        0.59285, 0.59950, 0.60586, 0.61103, 0.61552, 0.61930, 0.62202, 0.62354, 0.62380, 0.62288, 0.62110,\
        0.61885, 0.61634, 0.61369, 0.61106, 0.60866, 0.60686, 0.60942, 0.61761, 0.62742, 0.63688, 0.64487,\
        0.65062, 0.65375, 0.65435, 0.65288, 0.64984, 0.64565, 0.64064, 0.63508, 0.62926, 0.62353, 0.61831,\
        0.61392, 0.60994, 0.60580, 0.60230, 0.59928, 0.59690, 0.59565, 0.59439, 0.59277, 0.59118, 0.58987,\
        0.58891, 0.58797, 0.58657, 0.58423, 0.58045, 0.57539, 0.56959, 0.56344, 0.55718, 0.55096, 0.54491,\
        0.53914, 0.53370, 0.52864, 0.52393, 0.51953, 0.51536, 0.51132, 0.50740, 0.50365, 0.50033, 0.49767,\
        0.49539, 0.49342, 0.49168, 0.49007, 0.48846, 0.48668, 0.48461, 0.48212, 0.47917, 0.47576, 0.47194,\
        0.46778, 0.46336, 0.45874, 0.45397, 0.44909, 0.44411, 0.43900, 0.43375, 0.42829, 0.42256, 0.41650],\
        [0.44883, 0.45122, 0.45368, 0.45624, 0.45894, 0.46178, 0.46477, 0.46788, 0.47106, 0.47424, 0.47732,\
        0.48023, 0.48294, 0.48549, 0.48798, 0.49036, 0.49220, 0.49378, 0.49631, 0.49995, 0.50441, 0.50894,\
        0.51356, 0.51859, 0.52422, 0.53042, 0.53707, 0.54400, 0.55109, 0.55825, 0.56542, 0.57252, 0.57952,\
        0.58637, 0.59302, 0.59896, 0.60349, 0.60783, 0.61155, 0.61400, 0.61524, 0.61515, 0.61372, 0.61142,\
        0.60894, 0.60647, 0.60396, 0.60154, 0.59962, 0.59751, 0.59988, 0.60808, 0.61744, 0.62647, 0.63414,\
        0.63977, 0.64305, 0.64399, 0.64290, 0.64017, 0.63619, 0.63130, 0.62576, 0.61983, 0.61381, 0.60833,\
        0.60418, 0.60089, 0.59670, 0.59321, 0.59040, 0.58823, 0.58678, 0.58555, 0.58424, 0.58294, 0.58178,\
        0.58075, 0.57961, 0.57801, 0.57558, 0.57207, 0.56754, 0.56228, 0.55658, 0.55070, 0.54484, 0.53915,\
        0.53375, 0.52876, 0.52422, 0.52011, 0.51637, 0.51285, 0.50941, 0.50594, 0.50243, 0.49917, 0.49646,\
        0.49403, 0.49180, 0.48970, 0.48766, 0.48559, 0.48336, 0.48089, 0.47807, 0.47488, 0.47131, 0.46739,\
        0.46317, 0.45872, 0.45410, 0.44935, 0.44453, 0.43964, 0.43466, 0.42956, 0.42425, 0.41866, 0.41270],\
        [0.44457, 0.44674, 0.44894, 0.45122, 0.45360, 0.45611, 0.45874, 0.46149, 0.46433, 0.46719, 0.47003,\
        0.47280, 0.47546, 0.47803, 0.48052, 0.48287, 0.48498, 0.48716, 0.48995, 0.49361, 0.49790, 0.50218,\
        0.50644, 0.51128, 0.51693, 0.52329, 0.53012, 0.53720, 0.54442, 0.55167, 0.55888, 0.56597, 0.57289,\
        0.57955, 0.58579, 0.59121, 0.59571, 0.60010, 0.60364, 0.60569, 0.60674, 0.60650, 0.60459, 0.60178,\
        0.59917, 0.59685, 0.59463, 0.59274, 0.59157, 0.59156, 0.59485, 0.60172, 0.61002, 0.61824, 0.62534,\
        0.63062, 0.63376, 0.63473, 0.63374, 0.63114, 0.62731, 0.62255, 0.61708, 0.61120, 0.60506, 0.59928,\
        0.59508, 0.59177, 0.58817, 0.58481, 0.58206, 0.57993, 0.57833, 0.57703, 0.57581, 0.57464, 0.57351,\
        0.57237, 0.57104, 0.56927, 0.56680, 0.56349, 0.55936, 0.55457, 0.54933, 0.54387, 0.53839, 0.53307,\
        0.52807, 0.52353, 0.51954, 0.51609, 0.51308, 0.51031, 0.50757, 0.50466, 0.50152, 0.49833, 0.49537,\
        0.49262, 0.49003, 0.48753, 0.48507, 0.48256, 0.47991, 0.47705, 0.47391, 0.47046, 0.46669, 0.46263,\
        0.45832, 0.45380, 0.44914, 0.44440, 0.43961, 0.43480, 0.42995, 0.42502, 0.41990, 0.41448, 0.40863],\
        [0.44013, 0.44204, 0.44395, 0.44591, 0.44794, 0.45008, 0.45234, 0.45472, 0.45721, 0.45978, 0.46240,\
        0.46504, 0.46767, 0.47027, 0.47282, 0.47532, 0.47779, 0.48043, 0.48351, 0.48715, 0.49120, 0.49530,\
        0.49944, 0.50420, 0.50999, 0.51659, 0.52357, 0.53075, 0.53802, 0.54529, 0.55245, 0.55943, 0.56614,\
        0.57248, 0.57830, 0.58343, 0.58791, 0.59195, 0.59495, 0.59673, 0.59754, 0.59718, 0.59510, 0.59220,\
        0.58950, 0.58722, 0.58538, 0.58421, 0.58413, 0.58574, 0.58987, 0.59643, 0.60414, 0.61172, 0.61826,\
        0.62308, 0.62589, 0.62664, 0.62551, 0.62284, 0.61904, 0.61443, 0.60907, 0.60326, 0.59753, 0.59175,\
        0.58715, 0.58340, 0.57993, 0.57675, 0.57406, 0.57190, 0.57019, 0.56879, 0.56754, 0.56636, 0.56518,\
        0.56392, 0.56242, 0.56053, 0.55806, 0.55492, 0.55110, 0.54671, 0.54190, 0.53685, 0.53175, 0.52679,\
        0.52216, 0.51803, 0.51458, 0.51182, 0.50957, 0.50759, 0.50562, 0.50331, 0.50049, 0.49729, 0.49407,\
        0.49098, 0.48802, 0.48514, 0.48227, 0.47935, 0.47632, 0.47310, 0.46964, 0.46593, 0.46195, 0.45773,\
        0.45328, 0.44866, 0.44393, 0.43915, 0.43437, 0.42961, 0.42488, 0.42012, 0.41523, 0.41001, 0.40427],\
        [0.43558, 0.43720, 0.43879, 0.44041, 0.44207, 0.44383, 0.44570, 0.44770, 0.44985, 0.45215, 0.45458,\
        0.45711, 0.45972, 0.46238, 0.46508, 0.46782, 0.47064, 0.47366, 0.47700, 0.48071, 0.48470, 0.48880,\
        0.49308, 0.49787, 0.50382, 0.51060, 0.51759, 0.52474, 0.53198, 0.53919, 0.54622, 0.55299, 0.55940,\
        0.56535, 0.57076, 0.57553, 0.57966, 0.58310, 0.58561, 0.58707, 0.58750, 0.58678, 0.58486, 0.58224,\
        0.57966, 0.57754, 0.57610, 0.57565, 0.57660, 0.57940, 0.58437, 0.59120, 0.59889, 0.60634, 0.61252,\
        0.61689, 0.61928, 0.61966, 0.61820, 0.61522, 0.61127, 0.60689, 0.60168, 0.59588, 0.59015, 0.58466,\
        0.57983, 0.57573, 0.57214, 0.56898, 0.56630, 0.56408, 0.56228, 0.56078, 0.55945, 0.55819, 0.55691,\
        0.55552, 0.55389, 0.55191, 0.54946, 0.54648, 0.54294, 0.53890, 0.53447, 0.52979, 0.52505, 0.52044,\
        0.51613, 0.51234, 0.50935, 0.50729, 0.50579, 0.50451, 0.50332, 0.50154, 0.49885, 0.49563, 0.49226,\
        0.48893, 0.48567, 0.48246, 0.47924, 0.47596, 0.47258, 0.46903, 0.46528, 0.46132, 0.45713, 0.45272,\
        0.44812, 0.44337, 0.43853, 0.43367, 0.42885, 0.42411, 0.41945, 0.41484, 0.41020, 0.40522, 0.39958],\
        [0.43097, 0.43229, 0.43356, 0.43481, 0.43610, 0.43746, 0.43893, 0.44056, 0.44238, 0.44442, 0.44667,\
        0.44913, 0.45176, 0.45453, 0.45743, 0.46044, 0.46360, 0.46697, 0.47060, 0.47451, 0.47865, 0.48299,\
        0.48761, 0.49279, 0.49890, 0.50547, 0.51217, 0.51921, 0.52638, 0.53345, 0.54028, 0.54675, 0.55278,\
        0.55829, 0.56322, 0.56751, 0.57113, 0.57398, 0.57596, 0.57698, 0.57701, 0.57606, 0.57421, 0.57186,\
        0.56952, 0.56762, 0.56655, 0.56668, 0.56843, 0.57217, 0.57800, 0.58555, 0.59390, 0.60176, 0.60771,\
        0.61178, 0.61373, 0.61362, 0.61175, 0.60833, 0.60378, 0.59958, 0.59456, 0.58883, 0.58310, 0.57766,\
        0.57273, 0.56842, 0.56468, 0.56146, 0.55873, 0.55646, 0.55458, 0.55298, 0.55156, 0.55019, 0.54879,\
        0.54727, 0.54553, 0.54350, 0.54109, 0.53826, 0.53499, 0.53129, 0.52721, 0.52288, 0.51848, 0.51420,\
        0.51021, 0.50666, 0.50396, 0.50262, 0.50178, 0.50085, 0.50029, 0.49888, 0.49629, 0.49316, 0.48979,\
        0.48635, 0.48290, 0.47944, 0.47594, 0.47237, 0.46868, 0.46485, 0.46084, 0.45664, 0.45224, 0.44766,\
        0.44290, 0.43801, 0.43302, 0.42804, 0.42313, 0.41836, 0.41371, 0.40916, 0.40472, 0.40000, 0.39443],\
        [0.42639, 0.42741, 0.42834, 0.42923, 0.43013, 0.43109, 0.43217, 0.43342, 0.43491, 0.43670, 0.43881,\
        0.44122, 0.44391, 0.44683, 0.44995, 0.45326, 0.45677, 0.46049, 0.46446, 0.46867, 0.47311, 0.47780,\
        0.48283, 0.48834, 0.49443, 0.50067, 0.50718, 0.51423, 0.52130, 0.52818, 0.53471, 0.54080, 0.54638,\
        0.55139, 0.55578, 0.55951, 0.56254, 0.56481, 0.56624, 0.56680, 0.56647, 0.56531, 0.56348, 0.56129,\
        0.55912, 0.55741, 0.55657, 0.55705, 0.55931, 0.56372, 0.57040, 0.57899, 0.58845, 0.59709, 0.60344,\
        0.60761, 0.60899, 0.60818, 0.60589, 0.60228, 0.59732, 0.59244, 0.58737, 0.58179, 0.57612, 0.57069,\
        0.56570, 0.56127, 0.55741, 0.55410, 0.55132, 0.54900, 0.54706, 0.54539, 0.54389, 0.54242, 0.54090,\
        0.53925, 0.53742, 0.53535, 0.53300, 0.53034, 0.52734, 0.52398, 0.52026, 0.51625, 0.51215, 0.50824,\
        0.50465, 0.50146, 0.49893, 0.49800, 0.49755, 0.49689, 0.49628, 0.49499, 0.49276, 0.48988, 0.48665,\
        0.48323, 0.47969, 0.47608, 0.47238, 0.46857, 0.46464, 0.46056, 0.45632, 0.45191, 0.44734, 0.44260,\
        0.43770, 0.43266, 0.42751, 0.42235, 0.41732, 0.41249, 0.40784, 0.40322, 0.39876, 0.39414, 0.38878],\
        [0.42190, 0.42262, 0.42322, 0.42376, 0.42430, 0.42487, 0.42554, 0.42641, 0.42756, 0.42911, 0.43110,\
        0.43351, 0.43629, 0.43938, 0.44275, 0.44637, 0.45022, 0.45431, 0.45865, 0.46323, 0.46805, 0.47313,\
        0.47851, 0.48424, 0.49024, 0.49633, 0.50299, 0.50998, 0.51684, 0.52343, 0.52958, 0.53521, 0.54027,\
        0.54472, 0.54852, 0.55165, 0.55408, 0.55577, 0.55667, 0.55678, 0.55611, 0.55475, 0.55287, 0.55073,\
        0.54864, 0.54699, 0.54618, 0.54669, 0.54907, 0.55381, 0.56117, 0.57092, 0.58208, 0.59280, 0.60091,\
        0.60465, 0.60467, 0.60279, 0.59995, 0.59613, 0.59116, 0.58583, 0.58039, 0.57476, 0.56912, 0.56369,\
        0.55866, 0.55415, 0.55021, 0.54685, 0.54402, 0.54168, 0.53972, 0.53803, 0.53647, 0.53492, 0.53329,\
        0.53152, 0.52960, 0.52750, 0.52521, 0.52273, 0.52002, 0.51705, 0.51373, 0.51002, 0.50615, 0.50261,\
        0.49952, 0.49680, 0.49475, 0.49369, 0.49314, 0.49260, 0.49185, 0.49058, 0.48859, 0.48599, 0.48295,\
        0.47963, 0.47610, 0.47241, 0.46857, 0.46460, 0.46047, 0.45619, 0.45175, 0.44717, 0.44244, 0.43758,\
        0.43259, 0.42744, 0.42214, 0.41677, 0.41154, 0.40668, 0.40208, 0.39749, 0.39292, 0.38819, 0.38292],\
        [0.41758, 0.41800, 0.41829, 0.41851, 0.41871, 0.41892, 0.41920, 0.41966, 0.42046, 0.42177, 0.42366,\
        0.42610, 0.42901, 0.43229, 0.43592, 0.43983, 0.44403, 0.44850, 0.45322, 0.45819, 0.46341, 0.46888,\
        0.47461, 0.48059, 0.48676, 0.49315, 0.49982, 0.50643, 0.51300, 0.51923, 0.52492, 0.53003, 0.53451,\
        0.53834, 0.54153, 0.54404, 0.54588, 0.54700, 0.54740, 0.54709, 0.54610, 0.54454, 0.54257, 0.54040,\
        0.53830, 0.53656, 0.53554, 0.53574, 0.53777, 0.54230, 0.54991, 0.56066, 0.57375, 0.58740, 0.59861,\
        0.60192, 0.59977, 0.59700, 0.59362, 0.58945, 0.58449, 0.57906, 0.57343, 0.56772, 0.56206, 0.55663,\
        0.55157, 0.54702, 0.54304, 0.53964, 0.53682, 0.53449, 0.53256, 0.53090, 0.52934, 0.52774, 0.52602,\
        0.52413, 0.52210, 0.51997, 0.51775, 0.51544, 0.51301, 0.51049, 0.50769, 0.50431, 0.50067, 0.49739,\
        0.49471, 0.49241, 0.49065, 0.48952, 0.48880, 0.48813, 0.48727, 0.48598, 0.48413, 0.48173, 0.47887,\
        0.47567, 0.47218, 0.46847, 0.46456, 0.46047, 0.45620, 0.45175, 0.44715, 0.44242, 0.43757, 0.43264,\
        0.42762, 0.42246, 0.41710, 0.41151, 0.40597, 0.40105, 0.39654, 0.39198, 0.38732, 0.38242, 0.37711],\
        [0.41348, 0.41362, 0.41362, 0.41358, 0.41350, 0.41338, 0.41328, 0.41332, 0.41373, 0.41480, 0.41664,\
        0.41915, 0.42220, 0.42567, 0.42953, 0.43373, 0.43826, 0.44308, 0.44819, 0.45355, 0.45917, 0.46502,\
        0.47110, 0.47738, 0.48381, 0.49035, 0.49687, 0.50338, 0.50975, 0.51556, 0.52072, 0.52523, 0.52909,\
        0.53229, 0.53485, 0.53676, 0.53802, 0.53862, 0.53857, 0.53787, 0.53659, 0.53483, 0.53273, 0.53051,\
        0.52835, 0.52644, 0.52504, 0.52459, 0.52575, 0.52941, 0.53654, 0.54769, 0.56204, 0.57742, 0.59033,\
        0.59467, 0.59300, 0.59010, 0.58652, 0.58224, 0.57732, 0.57193, 0.56630, 0.56058, 0.55492, 0.54949,\
        0.54443, 0.53985, 0.53586, 0.53248, 0.52969, 0.52743, 0.52560, 0.52403, 0.52254, 0.52094, 0.51913,\
        0.51712, 0.51498, 0.51279, 0.51062, 0.50850, 0.50627, 0.50416, 0.50204, 0.49905, 0.49571, 0.49271,\
        0.49020, 0.48813, 0.48652, 0.48535, 0.48446, 0.48364, 0.48265, 0.48132, 0.47953, 0.47726, 0.47455,\
        0.47146, 0.46805, 0.46435, 0.46041, 0.45624, 0.45186, 0.44729, 0.44255, 0.43768, 0.43274, 0.42776,\
        0.42279, 0.41777, 0.41257, 0.40696, 0.40105, 0.39602, 0.39142, 0.38676, 0.38193, 0.37686, 0.37142],\
        [0.40967, 0.40953, 0.40929, 0.40903, 0.40876, 0.40841, 0.40797, 0.40758, 0.40754, 0.40835, 0.41021,\
        0.41284, 0.41600, 0.41962, 0.42367, 0.42813, 0.43295, 0.43810, 0.44356, 0.44930, 0.45528, 0.46149,\
        0.46791, 0.47449, 0.48118, 0.48792, 0.49459, 0.50109, 0.50706, 0.51231, 0.51687, 0.52076, 0.52399,\
        0.52656, 0.52850, 0.52984, 0.53057, 0.53071, 0.53026, 0.52924, 0.52771, 0.52575, 0.52352, 0.52124,\
        0.51908, 0.51703, 0.51517, 0.51388, 0.51381, 0.51592, 0.52165, 0.53251, 0.54737, 0.56291, 0.57540,\
        0.58182, 0.58290, 0.58128, 0.57827, 0.57432, 0.56965, 0.56445, 0.55894, 0.55330, 0.54770, 0.54230,\
        0.53725, 0.53268, 0.52870, 0.52536, 0.52265, 0.52050, 0.51882, 0.51745, 0.51612, 0.51458, 0.51268,\
        0.51052, 0.50827, 0.50601, 0.50386, 0.50202, 0.49990, 0.49797, 0.49616, 0.49370, 0.49083, 0.48814,\
        0.48581, 0.48388, 0.48234, 0.48112, 0.48011, 0.47914, 0.47804, 0.47666, 0.47489, 0.47271, 0.47011,\
        0.46712, 0.46378, 0.46012, 0.45617, 0.45196, 0.44750, 0.44282, 0.43797, 0.43297, 0.42792, 0.42289,\
        0.41799, 0.41325, 0.40846, 0.40328, 0.39753, 0.39197, 0.38687, 0.38187, 0.37678, 0.37147, 0.36585],\
        [0.40622, 0.40581, 0.40535, 0.40494, 0.40457, 0.40409, 0.40342, 0.40268, 0.40216, 0.40267, 0.40471,\
        0.40739, 0.41053, 0.41422, 0.41841, 0.42306, 0.42812, 0.43357, 0.43935, 0.44541, 0.45172, 0.45824,\
        0.46494, 0.47179, 0.47872, 0.48566, 0.49251, 0.49901, 0.50458, 0.50921, 0.51322, 0.51655, 0.51917,\
        0.52113, 0.52249, 0.52330, 0.52358, 0.52332, 0.52254, 0.52127, 0.51956, 0.51744, 0.51506, 0.51273,\
        0.51070, 0.50872, 0.50645, 0.50440, 0.50310, 0.50349, 0.50683, 0.51723, 0.53239, 0.54761, 0.55992,\
        0.56762, 0.57086, 0.57094, 0.56901, 0.56575, 0.56154, 0.55669, 0.55142, 0.54595, 0.54046, 0.53513,\
        0.53011, 0.52556, 0.52161, 0.51835, 0.51575, 0.51373, 0.51224, 0.51112, 0.51007, 0.50869, 0.50668,\
        0.50434, 0.50204, 0.49975, 0.49763, 0.49584, 0.49401, 0.49216, 0.49033, 0.48820, 0.48582, 0.48348,\
        0.48138, 0.47958, 0.47808, 0.47682, 0.47572, 0.47464, 0.47345, 0.47202, 0.47026, 0.46814, 0.46562,\
        0.46272, 0.45945, 0.45584, 0.45191, 0.44768, 0.44317, 0.43841, 0.43345, 0.42833, 0.42314, 0.41800,\
        0.41308, 0.40857, 0.40432, 0.39942, 0.39383, 0.38808, 0.38256, 0.37719, 0.37178, 0.36621, 0.36039],\
        [0.40321, 0.40253, 0.40191, 0.40134, 0.40098, 0.40042, 0.39962, 0.39873, 0.39807, 0.39849, 0.40052,\
        0.40289, 0.40585, 0.40955, 0.41380, 0.41856, 0.42382, 0.42950, 0.43555, 0.44188, 0.44845, 0.45522,\
        0.46213, 0.46915, 0.47622, 0.48324, 0.49008, 0.49646, 0.50160, 0.50592, 0.50965, 0.51251, 0.51455,\
        0.51594, 0.51679, 0.51715, 0.51706, 0.51650, 0.51547, 0.51403, 0.51225, 0.51007, 0.50755, 0.50507,\
        0.50327, 0.50177, 0.49914, 0.49668, 0.49480, 0.49427, 0.49638, 0.50627, 0.52041, 0.53445, 0.54613,\
        0.55426, 0.55872, 0.56014, 0.55930, 0.55684, 0.55322, 0.54880, 0.54385, 0.53861, 0.53330, 0.52809,\
        0.52313, 0.51860, 0.51469, 0.51154, 0.50909, 0.50719, 0.50586, 0.50502, 0.50431, 0.50324, 0.50113,\
        0.49858, 0.49628, 0.49404, 0.49197, 0.49012, 0.48834, 0.48657, 0.48476, 0.48283, 0.48077, 0.47873,\
        0.47684, 0.47517, 0.47371, 0.47244, 0.47128, 0.47013, 0.46887, 0.46740, 0.46566, 0.46358, 0.46113,\
        0.45831, 0.45511, 0.45156, 0.44766, 0.44344, 0.43891, 0.43411, 0.42906, 0.42383, 0.41849, 0.41315,\
        0.40801, 0.40341, 0.39954, 0.39492, 0.38949, 0.38381, 0.37814, 0.37250, 0.36682, 0.36101, 0.35497],\
        [0.40088, 0.39986, 0.39914, 0.39841, 0.39788, 0.39723, 0.39640, 0.39560, 0.39518, 0.39566, 0.39719,\
        0.39916, 0.40205, 0.40568, 0.40983, 0.41464, 0.42004, 0.42592, 0.43217, 0.43870, 0.44546, 0.45238,\
        0.45942, 0.46652, 0.47359, 0.48054, 0.48717, 0.49314, 0.49814, 0.50264, 0.50621, 0.50859, 0.51007,\
        0.51095, 0.51136, 0.51137, 0.51104, 0.51029, 0.50906, 0.50751, 0.50581, 0.50380, 0.50119, 0.49873,\
        0.49693, 0.49549, 0.49324, 0.49096, 0.48944, 0.48956, 0.49293, 0.50119, 0.51266, 0.52454, 0.53492,\
        0.54271, 0.54759, 0.54979, 0.54977, 0.54802, 0.54498, 0.54101, 0.53640, 0.53144, 0.52635, 0.52132,\
        0.51648, 0.51197, 0.50807, 0.50507, 0.50286, 0.50106, 0.49980, 0.49914, 0.49856, 0.49767, 0.49569,\
        0.49324, 0.49089, 0.48867, 0.48662, 0.48472, 0.48293, 0.48117, 0.47941, 0.47760, 0.47575, 0.47393,\
        0.47221, 0.47065, 0.46925, 0.46798, 0.46679, 0.46559, 0.46429, 0.46281, 0.46108, 0.45905, 0.45666,\
        0.45392, 0.45080, 0.44732, 0.44347, 0.43927, 0.43475, 0.42993, 0.42485, 0.41955, 0.41410, 0.40858,\
        0.40321, 0.39834, 0.39425, 0.38982, 0.38470, 0.37918, 0.37349, 0.36770, 0.36182, 0.35580, 0.34956],\
        [0.39917, 0.39795, 0.39690, 0.39602, 0.39527, 0.39453, 0.39378, 0.39316, 0.39293, 0.39339, 0.39461,\
        0.39651, 0.39925, 0.40252, 0.40645, 0.41128, 0.41681, 0.42284, 0.42923, 0.43589, 0.44274, 0.44973,\
        0.45679, 0.46386, 0.47084, 0.47763, 0.48404, 0.48987, 0.49504, 0.49954, 0.50285, 0.50465, 0.50560,\
        0.50611, 0.50620, 0.50592, 0.50551, 0.50474, 0.50336, 0.50172, 0.50012, 0.49846, 0.49607, 0.49379,\
        0.49196, 0.49038, 0.48866, 0.48714, 0.48651, 0.48769, 0.49168, 0.49887, 0.50799, 0.51752, 0.52625,\
        0.53321, 0.53796, 0.54045, 0.54092, 0.53969, 0.53712, 0.53355, 0.52927, 0.52458, 0.51974, 0.51496,\
        0.51038, 0.50597, 0.50200, 0.49918, 0.49717, 0.49555, 0.49433, 0.49355, 0.49284, 0.49178, 0.49007,\
        0.48791, 0.48566, 0.48349, 0.48144, 0.47953, 0.47771, 0.47595, 0.47422, 0.47249, 0.47077, 0.46909,\
        0.46750, 0.46603, 0.46469, 0.46344, 0.46224, 0.46102, 0.45971, 0.45824, 0.45653, 0.45454, 0.45223,\
        0.44957, 0.44653, 0.44313, 0.43935, 0.43520, 0.43072, 0.42591, 0.42084, 0.41554, 0.41006, 0.40449,\
        0.39905, 0.39408, 0.38952, 0.38487, 0.37980, 0.37435, 0.36865, 0.36277, 0.35674, 0.35053, 0.34411],\
        [0.39625, 0.39562, 0.39469, 0.39381, 0.39301, 0.39229, 0.39165, 0.39122, 0.39116, 0.39164, 0.39273,\
        0.39448, 0.39686, 0.39977, 0.40362, 0.40857, 0.41420, 0.42031, 0.42675, 0.43343, 0.44029, 0.44724,\
        0.45423, 0.46119, 0.46801, 0.47459, 0.48080, 0.48649, 0.49158, 0.49592, 0.49900, 0.50040, 0.50103,\
        0.50142, 0.50138, 0.50083, 0.50046, 0.49986, 0.49841, 0.49682, 0.49527, 0.49365, 0.49172, 0.48978,\
        0.48811, 0.48669, 0.48550, 0.48478, 0.48506, 0.48700, 0.49113, 0.49746, 0.50479, 0.51242, 0.51963,\
        0.52562, 0.52992, 0.53237, 0.53303, 0.53212, 0.52990, 0.52663, 0.52263, 0.51818, 0.51359, 0.50910,\
        0.50490, 0.50100, 0.49708, 0.49424, 0.49217, 0.49056, 0.48931, 0.48832, 0.48736, 0.48615, 0.48454,\
        0.48259, 0.48050, 0.47841, 0.47639, 0.47447, 0.47263, 0.47086, 0.46914, 0.46746, 0.46582, 0.46423,\
        0.46273, 0.46133, 0.46003, 0.45881, 0.45761, 0.45640, 0.45510, 0.45366, 0.45199, 0.45007, 0.44783,\
        0.44525, 0.44231, 0.43900, 0.43530, 0.43123, 0.42679, 0.42203, 0.41700, 0.41175, 0.40633, 0.40082,\
        0.39534, 0.39015, 0.38519, 0.38021, 0.37499, 0.36948, 0.36371, 0.35773, 0.35155, 0.34517, 0.33858],\
        [0.39294, 0.39279, 0.39225, 0.39158, 0.39092, 0.39034, 0.38990, 0.38967, 0.38978, 0.39033, 0.39141,\
        0.39302, 0.39517, 0.39790, 0.40173, 0.40669, 0.41229, 0.41835, 0.42474, 0.43135, 0.43810, 0.44493,\
        0.45177, 0.45853, 0.46513, 0.47146, 0.47741, 0.48285, 0.48766, 0.49162, 0.49440, 0.49589, 0.49659,\
        0.49693, 0.49686, 0.49637, 0.49591, 0.49524, 0.49404, 0.49262, 0.49116, 0.48966, 0.48807, 0.48652,\
        0.48517, 0.48412, 0.48347, 0.48344, 0.48440, 0.48674, 0.49067, 0.49599, 0.50212, 0.50854, 0.51457,\
        0.51964, 0.52338, 0.52558, 0.52624, 0.52548, 0.52347, 0.52044, 0.51664, 0.51237, 0.50800, 0.50381,\
        0.49984, 0.49637, 0.49294, 0.49006, 0.48783, 0.48608, 0.48466, 0.48343, 0.48223, 0.48087, 0.47926,\
        0.47742, 0.47543, 0.47341, 0.47142, 0.46950, 0.46765, 0.46586, 0.46414, 0.46248, 0.46088, 0.45935,\
        0.45790, 0.45655, 0.45528, 0.45409, 0.45292, 0.45173, 0.45046, 0.44905, 0.44745, 0.44559, 0.44344,\
        0.44096, 0.43812, 0.43491, 0.43132, 0.42733, 0.42296, 0.41824, 0.41326, 0.40809, 0.40272, 0.39724,\
        0.39172, 0.38632, 0.38103, 0.37572, 0.37027, 0.36460, 0.35870, 0.35259, 0.34626, 0.33973, 0.33297],\
        [0.38945, 0.38973, 0.38960, 0.38928, 0.38890, 0.38858, 0.38838, 0.38840, 0.38872, 0.38943, 0.39058,\
        0.39222, 0.39438, 0.39722, 0.40099, 0.40567, 0.41107, 0.41696, 0.42317, 0.42961, 0.43617, 0.44280,\
        0.44940, 0.45591, 0.46223, 0.46826, 0.47390, 0.47903, 0.48349, 0.48712, 0.48974, 0.49136, 0.49223,\
        0.49261, 0.49259, 0.49227, 0.49178, 0.49109, 0.49009, 0.48890, 0.48764, 0.48635, 0.48508, 0.48391,\
        0.48296, 0.48234, 0.48221, 0.48273, 0.48416, 0.48669, 0.49037, 0.49501, 0.50026, 0.50568, 0.51074,\
        0.51499, 0.51814, 0.52000, 0.52055, 0.51982, 0.51794, 0.51508, 0.51147, 0.50733, 0.50303, 0.49924,\
        0.49558, 0.49221, 0.48907, 0.48628, 0.48394, 0.48201, 0.48036, 0.47888, 0.47745, 0.47593, 0.47426,\
        0.47243, 0.47049, 0.46851, 0.46654, 0.46461, 0.46274, 0.46093, 0.45919, 0.45753, 0.45593, 0.45442,\
        0.45301, 0.45168, 0.45045, 0.44928, 0.44814, 0.44698, 0.44576, 0.44441, 0.44288, 0.44110, 0.43904,\
        0.43666, 0.43393, 0.43084, 0.42736, 0.42349, 0.41920, 0.41450, 0.40952, 0.40444, 0.39912, 0.39360,\
        0.38801, 0.38243, 0.37687, 0.37128, 0.36558, 0.35971, 0.35364, 0.34736, 0.34087, 0.33418, 0.32726],\
        [0.38588, 0.38654, 0.38684, 0.38691, 0.38690, 0.38690, 0.38702, 0.38733, 0.38791, 0.38884, 0.39017,\
        0.39196, 0.39424, 0.39709, 0.40071, 0.40525, 0.41042, 0.41605, 0.42200, 0.42818, 0.43448, 0.44082,\
        0.44714, 0.45334, 0.45934, 0.46504, 0.47035, 0.47514, 0.47929, 0.48267, 0.48521, 0.48691, 0.48793,\
        0.48844, 0.48855, 0.48836, 0.48795, 0.48735, 0.48654, 0.48559, 0.48457, 0.48356, 0.48261, 0.48182,\
        0.48128, 0.48110, 0.48142, 0.48238, 0.48413, 0.48676, 0.49026, 0.49445, 0.49904, 0.50368, 0.50789,\
        0.51140, 0.51398, 0.51548, 0.51585, 0.51510, 0.51330, 0.51059, 0.50718, 0.50332, 0.49914, 0.49540,\
        0.49194, 0.48866, 0.48560, 0.48282, 0.48038, 0.47825, 0.47637, 0.47465, 0.47299, 0.47129, 0.46952,\
        0.46765, 0.46570, 0.46371, 0.46173, 0.45978, 0.45788, 0.45604, 0.45427, 0.45258, 0.45097, 0.44946,\
        0.44804, 0.44673, 0.44551, 0.44437, 0.44326, 0.44216, 0.44099, 0.43971, 0.43826, 0.43657, 0.43460,\
        0.43232, 0.42970, 0.42673, 0.42339, 0.41966, 0.41551, 0.41084, 0.40571, 0.40058, 0.39537, 0.38983,\
        0.38415, 0.37841, 0.37262, 0.36678, 0.36084, 0.35474, 0.34848, 0.34203, 0.33537, 0.32851, 0.32143],\
        [0.38223, 0.38328, 0.38399, 0.38448, 0.38487, 0.38525, 0.38572, 0.38636, 0.38725, 0.38846, 0.39004,\
        0.39205, 0.39454, 0.39752, 0.40105, 0.40533, 0.41019, 0.41550, 0.42113, 0.42698, 0.43296, 0.43898,\
        0.44497, 0.45083, 0.45648, 0.46184, 0.46681, 0.47128, 0.47515, 0.47835, 0.48082, 0.48259, 0.48374,\
        0.48440, 0.48466, 0.48463, 0.48437, 0.48392, 0.48331, 0.48260, 0.48186, 0.48114, 0.48053, 0.48011,\
        0.47996, 0.48018, 0.48089, 0.48219, 0.48417, 0.48687, 0.49023, 0.49411, 0.49827, 0.50221, 0.50571,\
        0.50860, 0.51066, 0.51181, 0.51201, 0.51123, 0.50949, 0.50691, 0.50364, 0.50001, 0.49621, 0.49242,\
        0.48889, 0.48558, 0.48248, 0.47963, 0.47706, 0.47474, 0.47263, 0.47067, 0.46879, 0.46691, 0.46501,\
        0.46305, 0.46105, 0.45902, 0.45700, 0.45501, 0.45306, 0.45118, 0.44936, 0.44763, 0.44598, 0.44444,\
        0.44301, 0.44169, 0.44047, 0.43934, 0.43828, 0.43722, 0.43613, 0.43493, 0.43356, 0.43197, 0.43010,\
        0.42790, 0.42538, 0.42251, 0.41930, 0.41575, 0.41183, 0.40739, 0.40217, 0.39675, 0.39147, 0.38589,\
        0.38009, 0.37419, 0.36820, 0.36214, 0.35597, 0.34966, 0.34320, 0.33656, 0.32973, 0.32270, 0.31546],\
        [0.37853, 0.37995, 0.38107, 0.38198, 0.38278, 0.38357, 0.38443, 0.38543, 0.38665, 0.38816, 0.39001,\
        0.39227, 0.39500, 0.39824, 0.40177, 0.40568, 0.41018, 0.41515, 0.42044, 0.42595, 0.43158, 0.43724,\
        0.44287, 0.44837, 0.45367, 0.45868, 0.46332, 0.46750, 0.47114, 0.47419, 0.47660, 0.47841, 0.47968,\
        0.48048, 0.48092, 0.48106, 0.48097, 0.48071, 0.48032, 0.47987, 0.47940, 0.47899, 0.47870, 0.47862,\
        0.47883, 0.47941, 0.48044, 0.48201, 0.48415, 0.48688, 0.49012, 0.49372, 0.49740, 0.50086, 0.50395,\
        0.50633, 0.50792, 0.50875, 0.50881, 0.50803, 0.50637, 0.50391, 0.50080, 0.49724, 0.49355, 0.48983,\
        0.48623, 0.48282, 0.47963, 0.47666, 0.47392, 0.47141, 0.46908, 0.46690, 0.46480, 0.46274, 0.46068,\
        0.45861, 0.45652, 0.45443, 0.45235, 0.45030, 0.44829, 0.44634, 0.44446, 0.44266, 0.44096, 0.43937,\
        0.43789, 0.43654, 0.43532, 0.43420, 0.43317, 0.43217, 0.43115, 0.43004, 0.42877, 0.42727, 0.42548,\
        0.42336, 0.42091, 0.41811, 0.41500, 0.41158, 0.40789, 0.40390, 0.39908, 0.39333, 0.38759, 0.38176,\
        0.37580, 0.36973, 0.36355, 0.35729, 0.35091, 0.34441, 0.33776, 0.33094, 0.32394, 0.31674, 0.30934],\
        [0.37477, 0.37655, 0.37807, 0.37940, 0.38062, 0.38182, 0.38307, 0.38445, 0.38601, 0.38783, 0.38993,\
        0.39239, 0.39524, 0.39852, 0.40203, 0.40586, 0.41016, 0.41485, 0.41982, 0.42499, 0.43025, 0.43555,\
        0.44081, 0.44595, 0.45090, 0.45557, 0.45991, 0.46382, 0.46726, 0.47017, 0.47254, 0.47438, 0.47574,\
        0.47669, 0.47728, 0.47760, 0.47771, 0.47766, 0.47750, 0.47730, 0.47710, 0.47699, 0.47701, 0.47724,\
        0.47776, 0.47864, 0.47993, 0.48170, 0.48396, 0.48669, 0.48983, 0.49321, 0.49662, 0.49980, 0.50252,\
        0.50439, 0.50549, 0.50602, 0.50602, 0.50536, 0.50377, 0.50133, 0.49841, 0.49490, 0.49118, 0.48742,\
        0.48375, 0.48024, 0.47692, 0.47380, 0.47089, 0.46819, 0.46566, 0.46326, 0.46096, 0.45871, 0.45650,\
        0.45430, 0.45211, 0.44992, 0.44776, 0.44563, 0.44354, 0.44151, 0.43955, 0.43768, 0.43590, 0.43423,\
        0.43270, 0.43130, 0.43005, 0.42893, 0.42793, 0.42699, 0.42605, 0.42503, 0.42386, 0.42245, 0.42073,\
        0.41866, 0.41623, 0.41346, 0.41037, 0.40697, 0.40331, 0.39939, 0.39479, 0.38921, 0.38331, 0.37730,\
        0.37118, 0.36496, 0.35862, 0.35219, 0.34563, 0.33895, 0.33212, 0.32513, 0.31796, 0.31060, 0.30304],\
        [0.37092, 0.37306, 0.37497, 0.37670, 0.37834, 0.37994, 0.38159, 0.38335, 0.38526, 0.38738, 0.38974,\
        0.39235, 0.39521, 0.39831, 0.40183, 0.40580, 0.41005, 0.41452, 0.41920, 0.42402, 0.42893, 0.43387,\
        0.43876, 0.44354, 0.44815, 0.45251, 0.45656, 0.46023, 0.46349, 0.46628, 0.46861, 0.47047, 0.47192,\
        0.47299, 0.47374, 0.47424, 0.47454, 0.47470, 0.47478, 0.47482, 0.47489, 0.47504, 0.47535, 0.47586,\
        0.47665, 0.47776, 0.47926, 0.48116, 0.48349, 0.48621, 0.48924, 0.49247, 0.49572, 0.49878, 0.50116,\
        0.50248, 0.50304, 0.50331, 0.50335, 0.50296, 0.50174, 0.49916, 0.49611, 0.49261, 0.48885, 0.48504,\
        0.48130, 0.47768, 0.47423, 0.47098, 0.46791, 0.46502, 0.46229, 0.45970, 0.45721, 0.45479, 0.45242,\
        0.45008, 0.44777, 0.44548, 0.44322, 0.44100, 0.43882, 0.43670, 0.43465, 0.43268, 0.43081, 0.42905,\
        0.42742, 0.42595, 0.42465, 0.42352, 0.42254, 0.42166, 0.42079, 0.41986, 0.41879, 0.41747, 0.41582,\
        0.41376, 0.41131, 0.40852, 0.40542, 0.40196, 0.39818, 0.39415, 0.38954, 0.38421, 0.37841, 0.37237,\
        0.36617, 0.35983, 0.35337, 0.34679, 0.34008, 0.33324, 0.32625, 0.31910, 0.31178, 0.30426, 0.29655],\
        [0.36696, 0.36945, 0.37174, 0.37386, 0.37590, 0.37790, 0.37994, 0.38207, 0.38434, 0.38677, 0.38940,\
        0.39221, 0.39518, 0.39831, 0.40184, 0.40577, 0.40987, 0.41412, 0.41849, 0.42298, 0.42754, 0.43213,\
        0.43667, 0.44112, 0.44540, 0.44947, 0.45326, 0.45671, 0.45980, 0.46249, 0.46478, 0.46666, 0.46818,\
        0.46936, 0.47026, 0.47093, 0.47143, 0.47179, 0.47209, 0.47236, 0.47268, 0.47308, 0.47363, 0.47438,\
        0.47539, 0.47669, 0.47834, 0.48033, 0.48268, 0.48534, 0.48824, 0.49129, 0.49432, 0.49711, 0.49930,\
        0.50018, 0.50033, 0.50044, 0.50050, 0.50034, 0.49941, 0.49711, 0.49378, 0.49016, 0.48637, 0.48253,\
        0.47873, 0.47503, 0.47148, 0.46810, 0.46489, 0.46184, 0.45894, 0.45617, 0.45350, 0.45092, 0.44839,\
        0.44592, 0.44348, 0.44108, 0.43871, 0.43639, 0.43411, 0.43189, 0.42974, 0.42767, 0.42569, 0.42382,\
        0.42208, 0.42051, 0.41914, 0.41798, 0.41701, 0.41617, 0.41537, 0.41452, 0.41354, 0.41232, 0.41072,\
        0.40865, 0.40611, 0.40324, 0.40010, 0.39661, 0.39274, 0.38854, 0.38386, 0.37863, 0.37295, 0.36694,\
        0.36071, 0.35431, 0.34776, 0.34107, 0.33424, 0.32726, 0.32013, 0.31284, 0.30537, 0.29771, 0.28986],\
        [0.36287, 0.36570, 0.36834, 0.37084, 0.37326, 0.37565, 0.37807, 0.38056, 0.38317, 0.38593, 0.38885,\
        0.39193, 0.39515, 0.39851, 0.40205, 0.40577, 0.40961, 0.41356, 0.41763, 0.42180, 0.42603, 0.43028,\
        0.43450, 0.43863, 0.44262, 0.44642, 0.44997, 0.45323, 0.45618, 0.45878, 0.46102, 0.46292, 0.46450,\
        0.46579, 0.46682, 0.46765, 0.46832, 0.46888, 0.46938, 0.46987, 0.47041, 0.47103, 0.47179, 0.47274,\
        0.47392, 0.47537, 0.47710, 0.47914, 0.48145, 0.48401, 0.48675, 0.48956, 0.49228, 0.49470, 0.49647,\
        0.49729, 0.49747, 0.49749, 0.49743, 0.49709, 0.49602, 0.49388, 0.49083, 0.48730, 0.48356, 0.47974,\
        0.47594, 0.47220, 0.46858, 0.46510, 0.46177, 0.45859, 0.45554, 0.45261, 0.44979, 0.44705, 0.44438,\
        0.44177, 0.43921, 0.43669, 0.43422, 0.43179, 0.42941, 0.42709, 0.42483, 0.42265, 0.42055, 0.41856,\
        0.41670, 0.41501, 0.41353, 0.41231, 0.41136, 0.41055, 0.40978, 0.40897, 0.40807, 0.40692, 0.40538,\
        0.40328, 0.40062, 0.39764, 0.39441, 0.39084, 0.38690, 0.38259, 0.37784, 0.37264, 0.36702, 0.36106,\
        0.35484, 0.34840, 0.34179, 0.33502, 0.32808, 0.32099, 0.31374, 0.30632, 0.29872, 0.29093, 0.28295],\
        [0.35861, 0.36177, 0.36476, 0.36762, 0.37039, 0.37315, 0.37592, 0.37875, 0.38169, 0.38477, 0.38799,\
        0.39136, 0.39488, 0.39848, 0.40205, 0.40554, 0.40909, 0.41275, 0.41652, 0.42039, 0.42431, 0.42826,\
        0.43219, 0.43604, 0.43976, 0.44332, 0.44666, 0.44976, 0.45257, 0.45509, 0.45730, 0.45921, 0.46085,\
        0.46222, 0.46338, 0.46435, 0.46518, 0.46592, 0.46661, 0.46730, 0.46802, 0.46884, 0.46978, 0.47089,\
        0.47220, 0.47374, 0.47552, 0.47755, 0.47979, 0.48222, 0.48475, 0.48729, 0.48967, 0.49172, 0.49320,\
        0.49400, 0.49427, 0.49425, 0.49400, 0.49339, 0.49215, 0.49009, 0.48728, 0.48396, 0.48036, 0.47663,\
        0.47286, 0.46913, 0.46547, 0.46193, 0.45851, 0.45522, 0.45204, 0.44899, 0.44603, 0.44315, 0.44035,\
        0.43762, 0.43494, 0.43230, 0.42972, 0.42719, 0.42471, 0.42228, 0.41992, 0.41763, 0.41542, 0.41331,\
        0.41132, 0.40948, 0.40786, 0.40655, 0.40561, 0.40483, 0.40403, 0.40321, 0.40232, 0.40119, 0.39966,\
        0.39751, 0.39480, 0.39173, 0.38839, 0.38472, 0.38070, 0.37630, 0.37151, 0.36630, 0.36071, 0.35478,\
        0.34857, 0.34212, 0.33547, 0.32863, 0.32161, 0.31443, 0.30707, 0.29953, 0.29182, 0.28391, 0.27580],\
        [0.35417, 0.35764, 0.36095, 0.36414, 0.36726, 0.37034, 0.37344, 0.37660, 0.37984, 0.38319, 0.38668,\
        0.39031, 0.39406, 0.39789, 0.40148, 0.40477, 0.40810, 0.41153, 0.41506, 0.41867, 0.42234, 0.42603,\
        0.42970, 0.43330, 0.43679, 0.44014, 0.44330, 0.44625, 0.44895, 0.45139, 0.45358, 0.45550, 0.45718,\
        0.45864, 0.45990, 0.46100, 0.46198, 0.46288, 0.46374, 0.46459, 0.46548, 0.46646, 0.46754, 0.46877,\
        0.47018, 0.47177, 0.47357, 0.47555, 0.47770, 0.47996, 0.48227, 0.48453, 0.48660, 0.48834, 0.48962,\
        0.49036, 0.49064, 0.49056, 0.49016, 0.48935, 0.48797, 0.48595, 0.48331, 0.48020, 0.47676, 0.47315,\
        0.46946, 0.46577, 0.46211, 0.45854, 0.45506, 0.45169, 0.44842, 0.44525, 0.44218, 0.43919, 0.43627,\
        0.43342, 0.43062, 0.42789, 0.42520, 0.42257, 0.41999, 0.41746, 0.41500, 0.41261, 0.41030, 0.40808,\
        0.40598, 0.40401, 0.40224, 0.40078, 0.39985, 0.39905, 0.39816, 0.39726, 0.39628, 0.39505, 0.39341,\
        0.39123, 0.38853, 0.38544, 0.38202, 0.37827, 0.37417, 0.36970, 0.36486, 0.35964, 0.35405, 0.34814,\
        0.34193, 0.33548, 0.32879, 0.32190, 0.31482, 0.30756, 0.30011, 0.29248, 0.28465, 0.27664, 0.26842]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=20 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax20vt5
    zgrid_H_Hmax_vmax20vt5=[[0.3061183, 0.3083599, 0.3101630, 0.3115527, 0.3125563, 0.3132029, 0.3135226, 0.3135455, 0.3133016,\
        0.3128203, 0.3121309, 0.3112625, 0.3102449, 0.3091092, 0.3078887, 0.3066195, 0.3053413, 0.3040979,\
        0.3029366, 0.3019071, 0.3010564, 0.3004224, 0.3000223, 0.2998402, 0.2998019, 0.2994548, 0.2984079,\
        0.2972530, 0.2962184, 0.2954196, 0.2949308, 0.2948014, 0.2950668, 0.2957538, 0.2968735, 0.2983682,\
        0.3001793, 0.3029352, 0.3063851, 0.3103754, 0.3148094, 0.3196100, 0.3247084, 0.3300382, 0.3355318,\
        0.3411178, 0.3467189, 0.3522527, 0.3576377, 0.3628093, 0.3677446, 0.3724836, 0.3771201, 0.3817626,\
        0.3865279, 0.3915426, 0.3967847, 0.4024379, 0.4084184, 0.4145327, 0.4206762, 0.4267685, 0.4327431,\
        0.4385436, 0.4441234, 0.4494461, 0.4544812, 0.4591942, 0.4635259, 0.4673682, 0.4705650, 0.4729842,\
        0.4746642, 0.4758754, 0.4769192, 0.4779202, 0.4788453, 0.4796002, 0.4800656, 0.4802255, 0.4800870,\
        0.4795282, 0.4784429, 0.4768950, 0.4750184, 0.4729195, 0.4706758, 0.4683362, 0.4659120, 0.4633818,\
        0.4607152, 0.4578960, 0.4549237, 0.4517947, 0.4484929, 0.4449966, 0.4412874, 0.4373508, 0.4331706,\
        0.4287254, 0.4239890, 0.4189345, 0.4135389, 0.4077868, 0.4016710, 0.3951919, 0.3883551, 0.3811692,\
        0.3736438, 0.3657880, 0.3576098, 0.3491159, 0.3403112, 0.3311998, 0.3217842, 0.3120662, 0.3020467,\
        0.2917260, 0.2811039, 0.2701799, 0.2589529],\
        [0.3143937, 0.3164275, 0.3180069, 0.3191587, 0.3199129, 0.3203008, 0.3203541, 0.3201042, 0.3195815,\
        0.3188149, 0.3178325, 0.3166617, 0.3153302, 0.3138665, 0.3123014, 0.3106691, 0.3090081, 0.3073624,\
        0.3057824, 0.3043235, 0.3030434, 0.3019926, 0.3011973, 0.3006336, 0.3001951, 0.2997331, 0.2988121,\
        0.2977374, 0.2967714, 0.2960487, 0.2956503, 0.2956253, 0.2960036, 0.2968142, 0.2980827, 0.2997357,\
        0.3018690, 0.3048238, 0.3085186, 0.3127718, 0.3174806, 0.3225672, 0.3279636, 0.3336050, 0.3394249,\
        0.3453516, 0.3513032, 0.3571853, 0.3628917, 0.3683221, 0.3734246, 0.3782515, 0.3829601, 0.3877026,\
        0.3925524, 0.3977983, 0.4033965, 0.4092798, 0.4155006, 0.4218632, 0.4282494, 0.4345726, 0.4407613,\
        0.4467548, 0.4525042, 0.4579764, 0.4631530, 0.4680230, 0.4725541, 0.4766338, 0.4800286, 0.4824436,\
        0.4838302, 0.4846952, 0.4855819, 0.4865874, 0.4875672, 0.4884550, 0.4889537, 0.4891272, 0.4891327,\
        0.4886892, 0.4875237, 0.4858307, 0.4838313, 0.4816298, 0.4793148, 0.4769519, 0.4745474, 0.4720527,\
        0.4694079, 0.4665974, 0.4636466, 0.4605635, 0.4573209, 0.4538842, 0.4502349, 0.4463653, 0.4422633,\
        0.4379043, 0.4332541, 0.4282766, 0.4229424, 0.4172336, 0.4111445, 0.4046795, 0.3978492, 0.3906665,\
        0.3831444, 0.3752943, 0.3671255, 0.3586452, 0.3498587, 0.3407698, 0.3313808, 0.3216931, 0.3117072,\
        0.3014231, 0.2908402, 0.2799576, 0.2687740],\
        [0.3224702, 0.3242975, 0.3256533, 0.3265678, 0.3270736, 0.3272052, 0.3269965, 0.3264802, 0.3256869,\
        0.3246449, 0.3233805, 0.3219184, 0.3202831, 0.3184993, 0.3165941, 0.3145974, 0.3125443, 0.3104763,\
        0.3084433, 0.3065042, 0.3047264, 0.3031779, 0.3019064, 0.3009004, 0.3000829, 0.2993694, 0.2985609,\
        0.2976589, 0.2968467, 0.2962712, 0.2960259, 0.2961608, 0.2966925, 0.2976215, 0.2990138, 0.3009337,\
        0.3034386, 0.3067044, 0.3106351, 0.3151246, 0.3200799, 0.3254253, 0.3310945, 0.3370251, 0.3431537,\
        0.3494117, 0.3557188, 0.3619749, 0.3680504, 0.3737909, 0.3790741, 0.3839435, 0.3886841, 0.3936273,\
        0.3986653, 0.4040496, 0.4100266, 0.4162613, 0.4227286, 0.4293221, 0.4359310, 0.4424663, 0.4488517,\
        0.4550179, 0.4609079, 0.4664877, 0.4717474, 0.4767034, 0.4813861, 0.4857176, 0.4894135, 0.4919413,\
        0.4929045, 0.4932982, 0.4940877, 0.4951435, 0.4960643, 0.4971985, 0.4976234, 0.4977794, 0.4979601,\
        0.4977064, 0.4963805, 0.4945505, 0.4924668, 0.4901667, 0.4877732, 0.4853910, 0.4830239, 0.4805813,\
        0.4779380, 0.4750897, 0.4721376, 0.4691074, 0.4659301, 0.4625433, 0.4589373, 0.4551248, 0.4511020,\
        0.4468376, 0.4422831, 0.4373881, 0.4321139, 0.4264404, 0.4203659, 0.4139017, 0.4070655, 0.3998759,\
        0.3923496, 0.3845001, 0.3763376, 0.3678694, 0.3591006, 0.3500344, 0.3406728, 0.3310164, 0.3210654,\
        0.3108192, 0.3002768, 0.2894369, 0.2782980],\
        [0.3303464, 0.3319682, 0.3331010, 0.3337786, 0.3340381, 0.3339174, 0.3334535, 0.3326809, 0.3316303,\
        0.3303289, 0.3288005, 0.3270665, 0.3251471, 0.3230623, 0.3208338, 0.3184859, 0.3160481, 0.3135564,\
        0.3110566, 0.3086073, 0.3062833, 0.3041773, 0.3023903, 0.3009808, 0.2998776, 0.2989784, 0.2981313,\
        0.2973259, 0.2966560, 0.2962446, 0.2961862, 0.2965308, 0.2972855, 0.2984171, 0.2998878, 0.3021004,\
        0.3050274, 0.3085824, 0.3127453, 0.3174500, 0.3226225, 0.3281940, 0.3341018, 0.3402864, 0.3466889,\
        0.3532469, 0.3598888, 0.3665228, 0.3730145, 0.3791539, 0.3846742, 0.3895126, 0.3941452, 0.3992266,\
        0.4047785, 0.4105252, 0.4167630, 0.4233206, 0.4300660, 0.4368970, 0.4437196, 0.4504540, 0.4570262,\
        0.4633553, 0.4693639, 0.4750149, 0.4803034, 0.4852328, 0.4899519, 0.4944521, 0.4983992, 0.5011297,\
        0.5018475, 0.5019884, 0.5027826, 0.5037734, 0.5046039, 0.5054784, 0.5061761, 0.5064451, 0.5065904,\
        0.5062322, 0.5049434, 0.5031153, 0.5009953, 0.4986052, 0.4961326, 0.4937209, 0.4913936, 0.4890286,\
        0.4863364, 0.4833471, 0.4803749, 0.4774390, 0.4743429, 0.4709789, 0.4673821, 0.4636121, 0.4596742,\
        0.4555206, 0.4510777, 0.4462730, 0.4410553, 0.4354044, 0.4293271, 0.4228464, 0.4159900, 0.4087832,\
        0.4012463, 0.3933939, 0.3852364, 0.3767804, 0.3680302, 0.3589884, 0.3496559, 0.3400328, 0.3301187,\
        0.3199123, 0.3094122, 0.2986167, 0.2875239],\
        [0.3380182, 0.3394354, 0.3403451, 0.3407862, 0.3408015, 0.3404342, 0.3397249, 0.3387102, 0.3374208,\
        0.3358825, 0.3341160, 0.3321385, 0.3299653, 0.3276107, 0.3250901, 0.3224214, 0.3196266, 0.3167343,\
        0.3137828, 0.3108247, 0.3079343, 0.3052200, 0.3028428, 0.3010143, 0.2997413, 0.2986633, 0.2977755,\
        0.2970020, 0.2964160, 0.2961371, 0.2962649, 0.2968443, 0.2978770, 0.2993505, 0.3011773, 0.3035843,\
        0.3067190, 0.3104997, 0.3148667, 0.3197614, 0.3251207, 0.3308828, 0.3369886, 0.3433816, 0.3500064,\
        0.3568063, 0.3637211, 0.3706796, 0.3775807, 0.3842273, 0.3901703, 0.3949332, 0.3994088, 0.4046322,\
        0.4105798, 0.4168974, 0.4235512, 0.4304598, 0.4375038, 0.4445803, 0.4516084, 0.4585284, 0.4652844,\
        0.4717875, 0.4779084, 0.4835967, 0.4889291, 0.4937473, 0.4983216, 0.5028887, 0.5067600, 0.5094302,\
        0.5106361, 0.5111987, 0.5119072, 0.5127402, 0.5135188, 0.5142535, 0.5148541, 0.5151466, 0.5151203,\
        0.5145827, 0.5133720, 0.5116168, 0.5094887, 0.5070866, 0.5045730, 0.5020952, 0.4997224, 0.4974470,\
        0.4945869, 0.4912830, 0.4883387, 0.4856088, 0.4826016, 0.4791841, 0.4755379, 0.4717999, 0.4679656,\
        0.4639511, 0.4596436, 0.4549379, 0.4497679, 0.4441181, 0.4380132, 0.4314945, 0.4246029, 0.4173702,\
        0.4098187, 0.4019628, 0.3938114, 0.3853699, 0.3766412, 0.3676266, 0.3583263, 0.3487395, 0.3388649,\
        0.3287008, 0.3182454, 0.3074963, 0.2964515],\
        [0.3454785, 0.3466911, 0.3473763, 0.3475805, 0.3473541, 0.3467470, 0.3458048, 0.3445665, 0.3430629,\
        0.3413174, 0.3393468, 0.3371636, 0.3347774, 0.3321964, 0.3294298, 0.3264882, 0.3233864, 0.3201445,\
        0.3167919, 0.3133710, 0.3099456, 0.3066141, 0.3035393, 0.3010169, 0.2994698, 0.2985907, 0.2977251,\
        0.2969154, 0.2963250, 0.2961059, 0.2963853, 0.2972005, 0.2984815, 0.3002107, 0.3024230, 0.3051461,\
        0.3085006, 0.3124747, 0.3170183, 0.3220760, 0.3275910, 0.3335068, 0.3397676, 0.3463182, 0.3531035,\
        0.3600670, 0.3671505, 0.3742940, 0.3814347, 0.3884960, 0.3951848, 0.4000757, 0.4044517, 0.4100065,\
        0.4163766, 0.4232288, 0.4303728, 0.4376852, 0.4450533, 0.4523798, 0.4595962, 0.4666718, 0.4735979,\
        0.4803165, 0.4865973, 0.4922324, 0.4976190, 0.5025819, 0.5069946, 0.5113644, 0.5150718, 0.5177615,\
        0.5193988, 0.5203936, 0.5211992, 0.5219648, 0.5226678, 0.5232754, 0.5237163, 0.5238848, 0.5236919,\
        0.5230257, 0.5218166, 0.5201120, 0.5180184, 0.5156495, 0.5131357, 0.5105990, 0.5081047, 0.5055791,\
        0.5024539, 0.4988705, 0.4961501, 0.4937249, 0.4907366, 0.4870958, 0.4833458, 0.4796618, 0.4759700,\
        0.4721345, 0.4679925, 0.4633924, 0.4582493, 0.4525651, 0.4463979, 0.4398171, 0.4328773, 0.4256145,\
        0.4180493, 0.4101934, 0.4020528, 0.3936306, 0.3849281, 0.3759452, 0.3666811, 0.3571343, 0.3473027,\
        0.3371840, 0.3267759, 0.3160756, 0.3050808],\
        [0.3527164, 0.3537228, 0.3541804, 0.3541455, 0.3536791, 0.3528403, 0.3516810, 0.3502426, 0.3485555,\
        0.3466398, 0.3445076, 0.3421654, 0.3396170, 0.3368645, 0.3339110, 0.3307610, 0.3274225, 0.3239086,\
        0.3202410, 0.3164540, 0.3126015, 0.3087678, 0.3050893, 0.3018177, 0.2999506, 0.2993196, 0.2982536,\
        0.2972940, 0.2965737, 0.2962928, 0.2966498, 0.2977049, 0.2993135, 0.3012671, 0.3037474, 0.3067701,\
        0.3103616, 0.3145232, 0.3192234, 0.3244178, 0.3300575, 0.3360910, 0.3424653, 0.3491254, 0.3560126,\
        0.3630632, 0.3702055, 0.3773577, 0.3844185, 0.3912410, 0.3976789, 0.4032175, 0.4087019, 0.4150873,\
        0.4221473, 0.4296083, 0.4372811, 0.4450305, 0.4527435, 0.4603233, 0.4677021, 0.4748724, 0.4818999,\
        0.4888347, 0.4955023, 0.5010741, 0.5063895, 0.5114423, 0.5159768, 0.5201193, 0.5236576, 0.5263761,\
        0.5282721, 0.5295621, 0.5305134, 0.5312842, 0.5319195, 0.5324063, 0.5326935, 0.5327050, 0.5323605,\
        0.5315861, 0.5303438, 0.5286555, 0.5265899, 0.5242385, 0.5217022, 0.5190666, 0.5163578, 0.5134731,\
        0.5102258, 0.5068740, 0.5041370, 0.5017032, 0.4985536, 0.4945511, 0.4907729, 0.4872164, 0.4837044,\
        0.4800859, 0.4761422, 0.4716454, 0.4664851, 0.4607087, 0.4544361, 0.4477720, 0.4407799, 0.4334916,\
        0.4259211, 0.4180740, 0.4099524, 0.4015569, 0.3928870, 0.3839416, 0.3747188, 0.3652164, 0.3554318,\
        0.3453619, 0.3350040, 0.3243551, 0.3134125],\
        [0.3597169, 0.3605129, 0.3607362, 0.3604573, 0.3597517, 0.3586907, 0.3573335, 0.3557242, 0.3538913,\
        0.3518502, 0.3496067, 0.3471607, 0.3445093, 0.3416493, 0.3385785, 0.3352966, 0.3318070, 0.3281181,\
        0.3242474, 0.3202270, 0.3161127, 0.3119999, 0.3080600, 0.3046463, 0.3024011, 0.3009879, 0.2995483,\
        0.2983991, 0.2973590, 0.2968474, 0.2971396, 0.2983721, 0.3002910, 0.3025463, 0.3052780, 0.3085501,\
        0.3123563, 0.3166862, 0.3215182, 0.3268201, 0.3325523, 0.3386695, 0.3451217, 0.3518533, 0.3588018,\
        0.3658936, 0.3730371, 0.3801158, 0.3869842, 0.3933825, 0.3986552, 0.4049592, 0.4124250, 0.4200813,\
        0.4280040, 0.4361275, 0.4443422, 0.4525381, 0.4606080, 0.4684524, 0.4759847, 0.4831725, 0.4901904,\
        0.4971482, 0.5040306, 0.5100358, 0.5154278, 0.5204700, 0.5250448, 0.5290997, 0.5325387, 0.5352684,\
        0.5373015, 0.5387654, 0.5398285, 0.5406205, 0.5412044, 0.5415876, 0.5417390, 0.5416047, 0.5411258,\
        0.5402523, 0.5389596, 0.5372580, 0.5351903, 0.5328209, 0.5302224, 0.5274553, 0.5245410, 0.5214524,\
        0.5181962, 0.5149722, 0.5120162, 0.5091138, 0.5056551, 0.5016884, 0.4980057, 0.4945800, 0.4912105,\
        0.4878215, 0.4841070, 0.4796930, 0.4744283, 0.4684744, 0.4620558, 0.4553050, 0.4482748, 0.4409797,\
        0.4334211, 0.4255970, 0.4175058, 0.4091462, 0.4005166, 0.3916152, 0.3824394, 0.3729864, 0.3632529,\
        0.3532355, 0.3429310, 0.3323361, 0.3214478],\
        [0.3664593, 0.3670373, 0.3670149, 0.3664829, 0.3655376, 0.3642655, 0.3627345, 0.3609902, 0.3590571,\
        0.3569436, 0.3546470, 0.3521589, 0.3494700, 0.3465722, 0.3434596, 0.3401294, 0.3365819, 0.3328233,\
        0.3288695, 0.3247542, 0.3205405, 0.3163398, 0.3123412, 0.3088336, 0.3060691, 0.3038940, 0.3020500,\
        0.3004012, 0.2988908, 0.2980205, 0.2980121, 0.2992672, 0.3014416, 0.3040459, 0.3070861, 0.3105903,\
        0.3145704, 0.3190271, 0.3239512, 0.3293230, 0.3351127, 0.3412819, 0.3477836, 0.3545627, 0.3615558,\
        0.3686869, 0.3758562, 0.3829234, 0.3897415, 0.3963584, 0.4025894, 0.4091788, 0.4171098, 0.4255456,\
        0.4341778, 0.4428952, 0.4516087, 0.4602306, 0.4686559, 0.4767807, 0.4845336, 0.4917143, 0.4986499,\
        0.5057069, 0.5125543, 0.5188493, 0.5245050, 0.5296390, 0.5342440, 0.5382691, 0.5416600, 0.5443870,\
        0.5464799, 0.5480286, 0.5491506, 0.5499495, 0.5504890, 0.5507865, 0.5508237, 0.5505635, 0.5499658,\
        0.5489987, 0.5476453, 0.5459074, 0.5438069, 0.5413837, 0.5386900, 0.5357783, 0.5326887, 0.5294533,\
        0.5261326, 0.5228359, 0.5196200, 0.5163417, 0.5127715, 0.5089880, 0.5053458, 0.5018790, 0.4985477,\
        0.4953390, 0.4918681, 0.4874842, 0.4819483, 0.4757229, 0.4691599, 0.4623614, 0.4553352, 0.4480676,\
        0.4405456, 0.4327620, 0.4247137, 0.4163996, 0.4078183, 0.3989676, 0.3898447, 0.3804461, 0.3707681,\
        0.3608068, 0.3505587, 0.3400203, 0.3291884],\
        [0.3729163, 0.3732633, 0.3729766, 0.3721775, 0.3709907, 0.3695222, 0.3678482, 0.3660133, 0.3640350,\
        0.3619106, 0.3596259, 0.3571628, 0.3545051, 0.3516410, 0.3485635, 0.3452691, 0.3417571, 0.3380319,\
        0.3341078, 0.3300175, 0.3258246, 0.3216377, 0.3176183, 0.3139582, 0.3107835, 0.3080569, 0.3056582,\
        0.3034770, 0.3015855, 0.3002769, 0.2998061, 0.3008780, 0.3032624, 0.3061314, 0.3093802, 0.3130286,\
        0.3171020, 0.3216157, 0.3265732, 0.3319663, 0.3377745, 0.3439656, 0.3504959, 0.3573112, 0.3643500,\
        0.3715464, 0.3788242, 0.3860590, 0.3930199, 0.3996354, 0.4068999, 0.4144792, 0.4227965, 0.4316683,\
        0.4407913, 0.4499707, 0.4590895, 0.4680843, 0.4768566, 0.4852210, 0.4932325, 0.5007862, 0.5075805,\
        0.5146318, 0.5214666, 0.5278496, 0.5336730, 0.5389254, 0.5435851, 0.5476102, 0.5509718, 0.5536789,\
        0.5557799, 0.5573502, 0.5584813, 0.5592631, 0.5597579, 0.5599850, 0.5599284, 0.5595589, 0.5588526,\
        0.5577942, 0.5563738, 0.5545844, 0.5524273, 0.5499205, 0.5471026, 0.5440293, 0.5407635, 0.5373704,\
        0.5339200, 0.5304746, 0.5270443, 0.5235591, 0.5199459, 0.5162597, 0.5126432, 0.5091570, 0.5058181,\
        0.5025984, 0.4992687, 0.4947936, 0.4887231, 0.4822567, 0.4756653, 0.4689174, 0.4619627, 0.4547648,\
        0.4473047, 0.4395771, 0.4315825, 0.4233223, 0.4147965, 0.4060029, 0.3969385, 0.3875992, 0.3779808,\
        0.3680791, 0.3578902, 0.3474105, 0.3366368],\
        [0.3790517, 0.3791470, 0.3785676, 0.3774816, 0.3760527, 0.3744089, 0.3726323, 0.3707626, 0.3688048,\
        0.3667402, 0.3645388, 0.3621704, 0.3596121, 0.3568508, 0.3538811, 0.3507019, 0.3473129, 0.3437159,\
        0.3399199, 0.3359501, 0.3318594, 0.3277380, 0.3237107, 0.3199112, 0.3164317, 0.3132881, 0.3104470,\
        0.3078927, 0.3057115, 0.3040873, 0.3034568, 0.3043045, 0.3063965, 0.3091590, 0.3123727, 0.3159962,\
        0.3200371, 0.3245101, 0.3294248, 0.3347803, 0.3405638, 0.3467483, 0.3532930, 0.3601432, 0.3672360,\
        0.3745149, 0.3819542, 0.3895433, 0.3971220, 0.4043405, 0.4119461, 0.4201939, 0.4290092, 0.4382838,\
        0.4478026, 0.4573443, 0.4667432, 0.4759884, 0.4851479, 0.4937322, 0.5017384, 0.5095523, 0.5167965,\
        0.5238351, 0.5306467, 0.5370648, 0.5429825, 0.5483373, 0.5530665, 0.5571054, 0.5604321, 0.5630938,\
        0.5651674, 0.5667172, 0.5678153, 0.5685536, 0.5690023, 0.5691752, 0.5690427, 0.5685729, 0.5677580,\
        0.5666055, 0.5651147, 0.5632673, 0.5610390, 0.5584232, 0.5554489, 0.5521829, 0.5487142, 0.5451334,\
        0.5415160, 0.5379083, 0.5343144, 0.5307027, 0.5270485, 0.5233787, 0.5197536, 0.5162127, 0.5127559,\
        0.5093080, 0.5055809, 0.5005912, 0.4943060, 0.4880073, 0.4816044, 0.4750238, 0.4682039, 0.4611068,\
        0.4537228, 0.4460584, 0.4381230, 0.4299224, 0.4214578, 0.4127270, 0.4037261, 0.3944506, 0.3848955,\
        0.3750564, 0.3649291, 0.3545100, 0.3437959],\
        [0.3848157, 0.3846273, 0.3837133, 0.3823172, 0.3806530, 0.3788674, 0.3770427, 0.3752075, 0.3733482,\
        0.3714235, 0.3693819, 0.3671779, 0.3647827, 0.3621854, 0.3593874, 0.3563935, 0.3532056, 0.3498213,\
        0.3462384, 0.3424662, 0.3385378, 0.3345189, 0.3305057, 0.3266059, 0.3229105, 0.3194755, 0.3163339,\
        0.3135348, 0.3111886, 0.3094993, 0.3087800, 0.3092641, 0.3108435, 0.3132101, 0.3161444, 0.3195549,\
        0.3234188, 0.3277392, 0.3325241, 0.3377764, 0.3434886, 0.3496394, 0.3561910, 0.3630872, 0.3702548,\
        0.3776177, 0.3851509, 0.3929972, 0.4011733, 0.4091277, 0.4173914, 0.4261908, 0.4354918, 0.4451838,\
        0.4550948, 0.4649896, 0.4745744, 0.4837326, 0.4931113, 0.5023895, 0.5104336, 0.5183641, 0.5259254,\
        0.5331231, 0.5399746, 0.5464219, 0.5524023, 0.5578526, 0.5626729, 0.5667363, 0.5699978, 0.5725730,\
        0.5746047, 0.5761178, 0.5771441, 0.5778102, 0.5782159, 0.5783583, 0.5781681, 0.5775946, 0.5766552,\
        0.5753976, 0.5738361, 0.5719339, 0.5696289, 0.5668802, 0.5637081, 0.5602018, 0.5564886, 0.5526894,\
        0.5488892, 0.5451279, 0.5414057, 0.5376998, 0.5339912, 0.5302816, 0.5265850, 0.5228998, 0.5191768,\
        0.5152659, 0.5108203, 0.5053711, 0.4993473, 0.4933348, 0.4871846, 0.4808143, 0.4741465, 0.4671481,\
        0.4598315, 0.4522245, 0.4443476, 0.4362095, 0.4278108, 0.4191476, 0.4102148, 0.4010067, 0.3915180,\
        0.3817439, 0.3716800, 0.3613227, 0.3506689],\
        [0.3901362, 0.3896132, 0.3883069, 0.3865873, 0.3847157, 0.3828420, 0.3810405, 0.3793241, 0.3776543,\
        0.3759589, 0.3741570, 0.3721836, 0.3700053, 0.3676201, 0.3650434, 0.3622923, 0.3593730, 0.3562761,\
        0.3529822, 0.3494754, 0.3457605, 0.3418773, 0.3379034, 0.3339422, 0.3301005, 0.3264713, 0.3231357,\
        0.3201828, 0.3177365, 0.3159711, 0.3150849, 0.3151998, 0.3162698, 0.3181326, 0.3206329, 0.3236792,\
        0.3272346, 0.3312932, 0.3358604, 0.3409414, 0.3465339, 0.3526231, 0.3591777, 0.3661446, 0.3734430,\
        0.3809571, 0.3885523, 0.3962519, 0.4048306, 0.4137748, 0.4228705, 0.4322916, 0.4420813, 0.4521896,\
        0.4625073, 0.4728490, 0.4828190, 0.4916504, 0.5001303, 0.5100124, 0.5189709, 0.5272825, 0.5351078,\
        0.5424696, 0.5493829, 0.5558457, 0.5618561, 0.5673999, 0.5723635, 0.5764888, 0.5796197, 0.5820244,\
        0.5840488, 0.5855565, 0.5864618, 0.5870239, 0.5873998, 0.5875469, 0.5873188, 0.5866194, 0.5855186,\
        0.5841363, 0.5825065, 0.5805615, 0.5781822, 0.5752744, 0.5718478, 0.5680354, 0.5640277, 0.5599868,\
        0.5560057, 0.5521118, 0.5482900, 0.5445073, 0.5407326, 0.5369448, 0.5331258, 0.5292400, 0.5252080,\
        0.5208772, 0.5160280, 0.5105385, 0.5046457, 0.4986579, 0.4926554, 0.4864451, 0.4798858, 0.4729383,\
        0.4656540, 0.4580881, 0.4502665, 0.4421937, 0.4338652, 0.4252740, 0.4164129, 0.4072752, 0.3978551,\
        0.3881474, 0.3781478, 0.3678528, 0.3572595],\
        [0.3948954, 0.3939498, 0.3921935, 0.3901888, 0.3881812, 0.3862969, 0.3846042, 0.3831029, 0.3817248,\
        0.3803565, 0.3788758, 0.3771909, 0.3752674, 0.3731228, 0.3707986, 0.3683320, 0.3657370, 0.3629954,\
        0.3600620, 0.3568851, 0.3534318, 0.3497130, 0.3457963, 0.3417979, 0.3378540, 0.3340945, 0.3306333,\
        0.3275778, 0.3250447, 0.3231649, 0.3220628, 0.3218092, 0.3223877, 0.3237161, 0.3257021, 0.3282814,\
        0.3314243, 0.3351262, 0.3393943, 0.3442379, 0.3496606, 0.3556560, 0.3622033, 0.3692631, 0.3767687,\
        0.3846090, 0.3926099, 0.4006399, 0.4093251, 0.4187650, 0.4284821, 0.4384521, 0.4486849, 0.4591582,\
        0.4698225, 0.4805979, 0.4913445, 0.5009345, 0.5091220, 0.5184817, 0.5277457, 0.5363916, 0.5444373,\
        0.5519223, 0.5588675, 0.5652938, 0.5712525, 0.5768387, 0.5820330, 0.5863549, 0.5892610, 0.5912925,\
        0.5934417, 0.5950534, 0.5957719, 0.5962263, 0.5965875, 0.5967581, 0.5965007, 0.5956354, 0.5943335,\
        0.5928050, 0.5911042, 0.5891236, 0.5866731, 0.5835735, 0.5798132, 0.5756091, 0.5712580, 0.5669718,\
        0.5628355, 0.5588457, 0.5549571, 0.5511145, 0.5472696, 0.5433836, 0.5394195, 0.5353270, 0.5310271,\
        0.5264095, 0.5213657, 0.5158781, 0.5100760, 0.5041373, 0.4981438, 0.4920191, 0.4854752, 0.4784862,\
        0.4711869, 0.4636503, 0.4558868, 0.4478854, 0.4396325, 0.4311171, 0.4223303, 0.4132647, 0.4039140,\
        0.3942732, 0.3843378, 0.3741046, 0.3635712],\
        [0.3988458, 0.3973141, 0.3951891, 0.3930764, 0.3910513, 0.3892407, 0.3877427, 0.3865570, 0.3855794,\
        0.3846414, 0.3835629, 0.3822107, 0.3805557, 0.3786562, 0.3765935, 0.3744344, 0.3722075, 0.3698837,\
        0.3673812, 0.3645996, 0.3614551, 0.3579209, 0.3540617, 0.3500267, 0.3459990, 0.3421467, 0.3386040,\
        0.3354780, 0.3328631, 0.3308483, 0.3295091, 0.3288887, 0.3289872, 0.3297715, 0.3312003, 0.3332454,\
        0.3358983, 0.3391662, 0.3430644, 0.3476089, 0.3528109, 0.3586726, 0.3651851, 0.3723259, 0.3800566,\
        0.3883109, 0.3969159, 0.4055309, 0.4144958, 0.4241469, 0.4342664, 0.4446728, 0.4552730, 0.4660263,\
        0.4769143, 0.4878688, 0.4987593, 0.5092334, 0.5185580, 0.5278578, 0.5370791, 0.5458309, 0.5539854,\
        0.5615315, 0.5684632, 0.5747860, 0.5805538, 0.5859574, 0.5914027, 0.5961722, 0.5989123, 0.6004320,\
        0.6026669, 0.6044204, 0.6051593, 0.6056021, 0.6059100, 0.6059829, 0.6056039, 0.6046107, 0.6031597,\
        0.6014730, 0.5996423, 0.5975738, 0.5950294, 0.5916965, 0.5875004, 0.5828097, 0.5780941, 0.5736019,\
        0.5693710, 0.5653410, 0.5614246, 0.5575408, 0.5536266, 0.5496347, 0.5455230, 0.5412424, 0.5367304,\
        0.5319182, 0.5267579, 0.5212618, 0.5155136, 0.5096222, 0.5036541, 0.4975273, 0.4908573, 0.4837193,\
        0.4763945, 0.4689064, 0.4612190, 0.4532994, 0.4451271, 0.4366895, 0.4279777, 0.4189842, 0.4097027,\
        0.4001278, 0.3902554, 0.3800826, 0.3696074],\
        [0.4010982, 0.3992421, 0.3973609, 0.3954059, 0.3934387, 0.3917467, 0.3905069, 0.3897296, 0.3892575,\
        0.3888503, 0.3882542, 0.3872578, 0.3858520, 0.3841787, 0.3823663, 0.3805150, 0.3786855, 0.3768375,\
        0.3748339, 0.3725138, 0.3697272, 0.3663853, 0.3625550, 0.3584549, 0.3543420, 0.3504236, 0.3468387,\
        0.3436759, 0.3409965, 0.3388485, 0.3372701, 0.3362871, 0.3359107, 0.3361428, 0.3369855, 0.3384492,\
        0.3405539, 0.3433267, 0.3467960, 0.3509871, 0.3559191, 0.3616019, 0.3680349, 0.3752045, 0.3830824,\
        0.3916189, 0.4006689, 0.4098663, 0.4194223, 0.4295283, 0.4400921, 0.4509287, 0.4618648, 0.4728214,\
        0.4838917, 0.4950199, 0.5062086, 0.5172108, 0.5275663, 0.5373985, 0.5467757, 0.5555777, 0.5637535,\
        0.5713167, 0.5782161, 0.5844061, 0.5899447, 0.5947617, 0.6000130, 0.6050773, 0.6081556, 0.6100954,\
        0.6120341, 0.6136555, 0.6146060, 0.6151154, 0.6153350, 0.6152268, 0.6146657, 0.6135780, 0.6120423,\
        0.6102057, 0.6081442, 0.6058041, 0.6030098, 0.5994063, 0.5947138, 0.5894885, 0.5844752, 0.5798851,\
        0.5756537, 0.5716492, 0.5677419, 0.5638294, 0.5598430, 0.5557388, 0.5514816, 0.5470326, 0.5423470,\
        0.5373839, 0.5321236, 0.5265843, 0.5208225, 0.5149067, 0.5088756, 0.5026442, 0.4957085, 0.4885079,\
        0.4812545, 0.4738737, 0.4662881, 0.4584578, 0.4503661, 0.4420046, 0.4333661, 0.4244432, 0.4152288,\
        0.4057176, 0.3959057, 0.3857906, 0.3753711],\
        [0.4027022, 0.4010709, 0.3994310, 0.3975409, 0.3955457, 0.3939611, 0.3929977, 0.3927067, 0.3928240,\
        0.3930079, 0.3929837, 0.3923333, 0.3911163, 0.3896542, 0.3880710, 0.3864945, 0.3850709, 0.3837553,\
        0.3822998, 0.3804982, 0.3781301, 0.3749713, 0.3710958, 0.3668732, 0.3626717, 0.3587246, 0.3551509,\
        0.3520013, 0.3492934, 0.3470334, 0.3452273, 0.3438871, 0.3430342, 0.3427010, 0.3429308, 0.3437747,\
        0.3452862, 0.3475165, 0.3505102, 0.3543033, 0.3589222, 0.3643833, 0.3706911, 0.3778344, 0.3857766,\
        0.3944514, 0.4038007, 0.4136605, 0.4239363, 0.4346766, 0.4458172, 0.4571880, 0.4685399, 0.4796112,\
        0.4907299, 0.5024011, 0.5139875, 0.5255723, 0.5366611, 0.5470670, 0.5567194, 0.5655615, 0.5736690,\
        0.5812330, 0.5881494, 0.5941881, 0.5995939, 0.6042945, 0.6089170, 0.6136245, 0.6171032, 0.6195554,\
        0.6215115, 0.6230453, 0.6240613, 0.6246063, 0.6247494, 0.6244835, 0.6237551, 0.6225361, 0.6208676,\
        0.6188248, 0.6164448, 0.6136771, 0.6103674, 0.6062682, 0.6011833, 0.5955956, 0.5904882, 0.5859458,\
        0.5818016, 0.5778689, 0.5739859, 0.5700369, 0.5659594, 0.5617273, 0.5573216, 0.5527150, 0.5478746,\
        0.5427737, 0.5374030, 0.5317767, 0.5259264, 0.5198791, 0.5136154, 0.5070090, 0.4999847, 0.4929122,\
        0.4858532, 0.4786209, 0.4711395, 0.4633881, 0.4553669, 0.4470750, 0.4385056, 0.4296498, 0.4204993,\
        0.4110482, 0.4012929, 0.3912317, 0.3808643],\
        [0.4048248, 0.4034136, 0.4017136, 0.3996045, 0.3976544, 0.3961400, 0.3953826, 0.3956426, 0.3964043,\
        0.3970709, 0.3977314, 0.3973465, 0.3962736, 0.3950883, 0.3937186, 0.3923256, 0.3912739, 0.3905727,\
        0.3896337, 0.3883577, 0.3865137, 0.3835024, 0.3794308, 0.3750262, 0.3707700, 0.3668696, 0.3633891,\
        0.3603234, 0.3576406, 0.3553042, 0.3532894, 0.3515959, 0.3502564, 0.3493358, 0.3489219, 0.3491106,\
        0.3499927, 0.3516454, 0.3541302, 0.3574934, 0.3617676, 0.3669741, 0.3731236, 0.3802138, 0.3882168,\
        0.3970676, 0.4067581, 0.4171561, 0.4281035, 0.4395280, 0.4513323, 0.4633665, 0.4754030, 0.4868751,\
        0.4970306, 0.5093170, 0.5219629, 0.5342871, 0.5460319, 0.5569663, 0.5669291, 0.5757858, 0.5836266,\
        0.5910946, 0.5982340, 0.6041155, 0.6093941, 0.6141429, 0.6185864, 0.6227661, 0.6262341, 0.6289034,\
        0.6309535, 0.6324860, 0.6335109, 0.6340451, 0.6341126, 0.6337125, 0.6328253, 0.6314420, 0.6295816,\
        0.6272743, 0.6245243, 0.6212788, 0.6174196, 0.6128003, 0.6074001, 0.6016790, 0.5965440, 0.5920566,\
        0.5880021, 0.5841262, 0.5802421, 0.5762149, 0.5720010, 0.5676117, 0.5630484, 0.5582881, 0.5532997,\
        0.5480598, 0.5425614, 0.5368131, 0.5308296, 0.5246139, 0.5181374, 0.5113474, 0.5042897, 0.4973157,\
        0.4903816, 0.4832440, 0.4758177, 0.4681089, 0.4601390, 0.4519083, 0.4434040, 0.4346116, 0.4255204,\
        0.4161241, 0.4064201, 0.3964078, 0.3860879],\
        [0.4073753, 0.4060569, 0.4042686, 0.4021719, 0.4003999, 0.3986572, 0.3980182, 0.3987508, 0.4001947,\
        0.4011589, 0.4020576, 0.4020894, 0.4013947, 0.4005690, 0.3993993, 0.3980994, 0.3972250, 0.3973511,\
        0.3966338, 0.3957772, 0.3946440, 0.3916545, 0.3871611, 0.3826403, 0.3784592, 0.3747345, 0.3714556,\
        0.3685566, 0.3659599, 0.3635886, 0.3613853, 0.3593366, 0.3574886, 0.3559459, 0.3548503, 0.3543498,\
        0.3545748, 0.3556284, 0.3575865, 0.3605033, 0.3644156, 0.3693479, 0.3753167, 0.3823366, 0.3904210,\
        0.3995335, 0.4095451, 0.4204109, 0.4319685, 0.4440615, 0.4565357, 0.4692301, 0.4820231, 0.4948731,\
        0.5059281, 0.5175452, 0.5304948, 0.5433720, 0.5556704, 0.5670715, 0.5773830, 0.5863823, 0.5937488,\
        0.6007233, 0.6079152, 0.6140139, 0.6193059, 0.6240472, 0.6283636, 0.6322588, 0.6355874, 0.6382741,\
        0.6403605, 0.6418997, 0.6429136, 0.6434105, 0.6433962, 0.6428698, 0.6418248, 0.6402580, 0.6381745,\
        0.6355805, 0.6324668, 0.6287957, 0.6245068, 0.6195662, 0.6140807, 0.6084333, 0.6031564, 0.5985260,\
        0.5944373, 0.5905184, 0.5865725, 0.5823828, 0.5779556, 0.5733736, 0.5686480, 0.5637398, 0.5586057,\
        0.5532206, 0.5475796, 0.5416905, 0.5355637, 0.5292027, 0.5226029, 0.5157768, 0.5088145, 0.5018635,\
        0.4949192, 0.4877837, 0.4803221, 0.4726091, 0.4646782, 0.4565079, 0.4480676, 0.4393347, 0.4302968,\
        0.4209485, 0.4112888, 0.4013190, 0.3910413],\
        [0.4103995, 0.4092359, 0.4076144, 0.4057682, 0.4039588, 0.4024095, 0.4019191, 0.4026692, 0.4043021,\
        0.4056189, 0.4065760, 0.4069462, 0.4067265, 0.4061548, 0.4052594, 0.4042152, 0.4035509, 0.4037729,\
        0.4032842, 0.4026043, 0.4017563, 0.3983435, 0.3938861, 0.3896046, 0.3856981, 0.3823017, 0.3793264,\
        0.3766599, 0.3741995, 0.3718301, 0.3694540, 0.3670384, 0.3646441, 0.3624286, 0.3606058, 0.3593860,\
        0.3589391, 0.3593887, 0.3608201, 0.3632913, 0.3668413, 0.3714958, 0.3772720, 0.3841884, 0.3922821,\
        0.4015976, 0.4120077, 0.4233770, 0.4355271, 0.4482696, 0.4613959, 0.4746586, 0.4878244, 0.5008219,\
        0.5136318, 0.5262349, 0.5393841, 0.5526431, 0.5654124, 0.5771864, 0.5877449, 0.5971498, 0.6045264,\
        0.6111194, 0.6178151, 0.6238883, 0.6292162, 0.6339351, 0.6381431, 0.6418592, 0.6450419, 0.6476598,\
        0.6497208, 0.6512424, 0.6522281, 0.6526712, 0.6525662, 0.6519121, 0.6507090, 0.6489549, 0.6466460,\
        0.6437778, 0.6403438, 0.6363350, 0.6317513, 0.6266394, 0.6211540, 0.6155891, 0.6102890, 0.6054912,\
        0.6011187, 0.5970507, 0.5929798, 0.5884799, 0.5837472, 0.5789647, 0.5740965, 0.5690548, 0.5637758,\
        0.5582360, 0.5524393, 0.5464004, 0.5401350, 0.5336566, 0.5269826, 0.5201516, 0.5132394, 0.5063235,\
        0.4993873, 0.4921603, 0.4845586, 0.4768422, 0.4689738, 0.4608780, 0.4525040, 0.4438254, 0.4348319,\
        0.4255219, 0.4158977, 0.4059630, 0.3957216],\
        [0.4139531, 0.4130523, 0.4117417, 0.4102290, 0.4087591, 0.4076725, 0.4073860, 0.4080578, 0.4093676,\
        0.4106910, 0.4117110, 0.4122785, 0.4123733, 0.4120838, 0.4115109, 0.4108468, 0.4104131, 0.4102347,\
        0.4097977, 0.4089390, 0.4072720, 0.4040084, 0.4001744, 0.3961948, 0.3926656, 0.3896979, 0.3870540,\
        0.3846205, 0.3823140, 0.3799690, 0.3774251, 0.3746210, 0.3716227, 0.3686644, 0.3660664, 0.3641114,\
        0.3630000, 0.3628633, 0.3637874, 0.3658306, 0.3690339, 0.3734258, 0.3790236, 0.3858380, 0.3938983,\
        0.4033500, 0.4141842, 0.4260556, 0.4387719, 0.4521481, 0.4659599, 0.4798923, 0.4935498, 0.5070867,\
        0.5206834, 0.5342349, 0.5479320, 0.5616586, 0.5749622, 0.5871200, 0.5976011, 0.6069537, 0.6148052,\
        0.6215891, 0.6279658, 0.6338461, 0.6391073, 0.6437674, 0.6478736, 0.6514465, 0.6544856, 0.6569955,\
        0.6589900, 0.6604701, 0.6614134, 0.6617888, 0.6615798, 0.6607900, 0.6594300, 0.6574991, 0.6549836,\
        0.6518722, 0.6481698, 0.6438996, 0.6391099, 0.6338925, 0.6284069, 0.6228788, 0.6175451, 0.6125605,\
        0.6079336, 0.6035982, 0.5993159, 0.5942645, 0.5892475, 0.5843378, 0.5793818, 0.5742248, 0.5687918,\
        0.5630794, 0.5571119, 0.5509166, 0.5445166, 0.5379331, 0.5311908, 0.5243266, 0.5173911, 0.5104314,\
        0.5034321, 0.4959634, 0.4883678, 0.4807825, 0.4730393, 0.4650367, 0.4567255, 0.4480888, 0.4391257,\
        0.4298413, 0.4202422, 0.4103346, 0.4001236],\
        [0.4180259, 0.4174722, 0.4165588, 0.4154772, 0.4144612, 0.4137876, 0.4137071, 0.4142918, 0.4153429,\
        0.4165060, 0.4174984, 0.4181656, 0.4184613, 0.4184205, 0.4181309, 0.4177301, 0.4173520, 0.4169566,\
        0.4163026, 0.4151354, 0.4131586, 0.4102268, 0.4066985, 0.4029973, 0.3998017, 0.3971644, 0.3947235,\
        0.3924114, 0.3902346, 0.3879088, 0.3851816, 0.3819740, 0.3782878, 0.3744961, 0.3710916, 0.3684226,\
        0.3666904, 0.3660131, 0.3664679, 0.3681121, 0.3709923, 0.3751472, 0.3806071, 0.3873910, 0.3955124,\
        0.4051375, 0.4162307, 0.4284850, 0.4416916, 0.4556518, 0.4701564, 0.4849548, 0.4995215, 0.5134941,\
        0.5275086, 0.5416052, 0.5557741, 0.5699321, 0.5838171, 0.5966378, 0.6071796, 0.6163586, 0.6244414,\
        0.6315405, 0.6379413, 0.6437367, 0.6489214, 0.6535079, 0.6575167, 0.6609560, 0.6638397, 0.6662086,\
        0.6681064, 0.6695312, 0.6704250, 0.6707184, 0.6703821, 0.6694380, 0.6679236, 0.6658410, 0.6631531,\
        0.6598346, 0.6559051, 0.6514223, 0.6464705, 0.6411629, 0.6356470, 0.6300961, 0.6246743, 0.6194879,\
        0.6145528, 0.6097777, 0.6048498, 0.5995499, 0.5944497, 0.5895167, 0.5845298, 0.5792489, 0.5736243,\
        0.5677076, 0.5615521, 0.5551949, 0.5486628, 0.5419776, 0.5351605, 0.5282327, 0.5212082, 0.5140676,\
        0.5067088, 0.4991360, 0.4918858, 0.4845346, 0.4769389, 0.4690169, 0.4607440, 0.4521248, 0.4431719,\
        0.4338978, 0.4243128, 0.4144247, 0.4042391],\
        [0.4225564, 0.4224041, 0.4219325, 0.4213121, 0.4207410, 0.4204249, 0.4205226, 0.4210712, 0.4219536,\
        0.4229574, 0.4238779, 0.4245778, 0.4249956, 0.4251340, 0.4250408, 0.4247861, 0.4244129, 0.4238710,\
        0.4230105, 0.4216450, 0.4196162, 0.4169103, 0.4137362, 0.4104491, 0.4074471, 0.4048170, 0.4023574,\
        0.3999770, 0.3978400, 0.3954490, 0.3924831, 0.3889395, 0.3844143, 0.3796973, 0.3755301, 0.3722448,\
        0.3699867, 0.3688417, 0.3688763, 0.3701505, 0.3727234, 0.3766532, 0.3819957, 0.3888062, 0.3971362,\
        0.4068800, 0.4180997, 0.4306436, 0.4442675, 0.4587273, 0.4737784, 0.4891837, 0.5045455, 0.5192448,\
        0.5338039, 0.5483447, 0.5628314, 0.5771741, 0.5912015, 0.6044293, 0.6155583, 0.6251458, 0.6335919,\
        0.6410330, 0.6476129, 0.6534410, 0.6585903, 0.6631180, 0.6670472, 0.6703592, 0.6730576, 0.6752344,\
        0.6770021, 0.6783700, 0.6792180, 0.6794087, 0.6789016, 0.6777612, 0.6760954, 0.6739108, 0.6711007,\
        0.6676053, 0.6634723, 0.6588010, 0.6537040, 0.6483019, 0.6427241, 0.6371015, 0.6315475, 0.6261311,\
        0.6208524, 0.6156239, 0.6102906, 0.6048520, 0.5995654, 0.5946100, 0.5895973, 0.5840995, 0.5782052,\
        0.5720496, 0.5656962, 0.5591787, 0.5525209, 0.5457410, 0.5388533, 0.5318653, 0.5247708, 0.5175413,\
        0.5101533, 0.5027511, 0.4955051, 0.4882681, 0.4807481, 0.4728414, 0.4645561, 0.4559192, 0.4469533,\
        0.4376748, 0.4280947, 0.4182206, 0.4080575],\
        [0.4274555, 0.4277330, 0.4277153, 0.4275463, 0.4273846, 0.4273815, 0.4276453, 0.4282033, 0.4289901,\
        0.4298773, 0.4307256, 0.4314255, 0.4319128, 0.4321664, 0.4321961, 0.4320215, 0.4316419, 0.4310077,\
        0.4300213, 0.4285703, 0.4265809, 0.4240776, 0.4212104, 0.4182167, 0.4153194, 0.4125884, 0.4099550,\
        0.4073882, 0.4048976, 0.4020512, 0.3989903, 0.3952407, 0.3895303, 0.3839766, 0.3792899, 0.3755938,\
        0.3729473, 0.3714184, 0.3710768, 0.3719943, 0.3742510, 0.3779330, 0.3831203, 0.3898752, 0.3982390,\
        0.4081676, 0.4196154, 0.4324720, 0.4465109, 0.4614455, 0.4769526, 0.4927128, 0.5085869, 0.5242791,\
        0.5395815, 0.5545977, 0.5693442, 0.5837498, 0.5976758, 0.6109973, 0.6229569, 0.6333102, 0.6423297,\
        0.6501969, 0.6570321, 0.6629497, 0.6680843, 0.6725684, 0.6764559, 0.6796673, 0.6821291, 0.6840246,\
        0.6856118, 0.6869328, 0.6877482, 0.6877952, 0.6870457, 0.6856165, 0.6838030, 0.6816255, 0.6787619,\
        0.6750986, 0.6707665, 0.6659190, 0.6606863, 0.6551867, 0.6495284, 0.6438068, 0.6380985, 0.6324486,\
        0.6268596, 0.6212924, 0.6157073, 0.6101507, 0.6047715, 0.5997256, 0.5945544, 0.5886169, 0.5823968,\
        0.5760070, 0.5694699, 0.5628067, 0.5560375, 0.5491781, 0.5422383, 0.5352217, 0.5281237, 0.5209379,\
        0.5136814, 0.5064196, 0.4992268, 0.4920370, 0.4844683, 0.4764778, 0.4681220, 0.4594354, 0.4504393,\
        0.4411472, 0.4315677, 0.4217063, 0.4115661],\
        [0.4326235, 0.4333410, 0.4337710, 0.4340302, 0.4342420, 0.4345200, 0.4349451, 0.4355453, 0.4362901,\
        0.4371054, 0.4379007, 0.4385935, 0.4391231, 0.4394527, 0.4395632, 0.4394414, 0.4390642, 0.4383877,\
        0.4373495, 0.4358889, 0.4339780, 0.4316502, 0.4290079, 0.4261939, 0.4233298, 0.4204578, 0.4175466,\
        0.4145441, 0.4113595, 0.4078458, 0.4041352, 0.3992502, 0.3928923, 0.3872573, 0.3825289, 0.3786661,\
        0.3757503, 0.3738955, 0.3731943, 0.3737357, 0.3756286, 0.3790012, 0.3839717, 0.3906005, 0.3988884,\
        0.4089944, 0.4207209, 0.4339466, 0.4484727, 0.4639779, 0.4800802, 0.4964416, 0.5129363, 0.5292782,\
        0.5452181, 0.5607021, 0.5757443, 0.5903446, 0.6044727, 0.6179828, 0.6303450, 0.6412974, 0.6508875,\
        0.6592126, 0.6663328, 0.6723375, 0.6774219, 0.6818367, 0.6857285, 0.6889350, 0.6910867, 0.6925803,\
        0.6939121, 0.6951752, 0.6959594, 0.6957529, 0.6947091, 0.6927883, 0.6908364, 0.6889266, 0.6860601,\
        0.6821856, 0.6776604, 0.6726590, 0.6673083, 0.6617220, 0.6559849, 0.6501595, 0.6442940, 0.6384227,\
        0.6325624, 0.6267183, 0.6209095, 0.6152010, 0.6096991, 0.6044289, 0.5987389, 0.5924486, 0.5860413,\
        0.5794910, 0.5728068, 0.5660222, 0.5591625, 0.5522433, 0.5452743, 0.5382610, 0.5312072, 0.5241221,\
        0.5170301, 0.5099687, 0.5029538, 0.4957250, 0.4879444, 0.4798096, 0.4713609, 0.4626163, 0.4535882,\
        0.4442842, 0.4347088, 0.4248643, 0.4147517],\
        [0.4379595, 0.4391169, 0.4399814, 0.4406438, 0.4411990, 0.4417345, 0.4423169, 0.4429796, 0.4437186,\
        0.4444986, 0.4452657, 0.4459606, 0.4465283, 0.4469217, 0.4471012, 0.4470315, 0.4466775, 0.4460017,\
        0.4449668, 0.4435448, 0.4417310, 0.4395549, 0.4370792, 0.4343800, 0.4315121, 0.4284809, 0.4252393,\
        0.4216993, 0.4177326, 0.4131819, 0.4078245, 0.4014142, 0.3953449, 0.3903342, 0.3858476, 0.3818700,\
        0.3786877, 0.3765032, 0.3754156, 0.3755155, 0.3769438, 0.3798920, 0.3845588, 0.3910828, 0.3994456,\
        0.4095383, 0.4214284, 0.4350680, 0.4501983, 0.4664355, 0.4833310, 0.5004703, 0.5175990, 0.5345370,\
        0.5510619, 0.5670131, 0.5823865, 0.5973348, 0.6117916, 0.6255008, 0.6381150, 0.6494488, 0.6594949,\
        0.6682634, 0.6756857, 0.6817405, 0.6866838, 0.6909363, 0.6947911, 0.6981412, 0.6999511, 0.7010847,\
        0.7021205, 0.7031139, 0.7036193, 0.7030747, 0.7016681, 0.6990664, 0.6969569, 0.6958743, 0.6927865,\
        0.6886603, 0.6840309, 0.6789225, 0.6734847, 0.6678432, 0.6620518, 0.6561346, 0.6501172, 0.6440321,\
        0.6379128, 0.6317941, 0.6257184, 0.6197378, 0.6138786, 0.6080226, 0.6018604, 0.5955648, 0.5891347,\
        0.5824692, 0.5756620, 0.5687804, 0.5618526, 0.5548927, 0.5479113, 0.5409170, 0.5339188, 0.5269288,\
        0.5199626, 0.5130257, 0.5060515, 0.4987193, 0.4908832, 0.4826821, 0.4741779, 0.4653990, 0.4563568,\
        0.4470552, 0.4374955, 0.4276779, 0.4176016],\
        [0.4433655, 0.4449587, 0.4462437, 0.4472877, 0.4481618, 0.4489356, 0.4496684, 0.4504024, 0.4511569,\
        0.4519279, 0.4526911, 0.4534072, 0.4540271, 0.4544957, 0.4547570, 0.4547581, 0.4544550, 0.4538164,\
        0.4528247, 0.4514746, 0.4497731, 0.4477413, 0.4454104, 0.4428092, 0.4399444, 0.4367855, 0.4332586,\
        0.4292507, 0.4246164, 0.4191944, 0.4128638, 0.4058499, 0.3995412, 0.3947325, 0.3900815, 0.3856729,\
        0.3820933, 0.3795228, 0.3779805, 0.3775220, 0.3783216, 0.3806516, 0.3848135, 0.3910609, 0.3994169,\
        0.4094935, 0.4216392, 0.4358400, 0.4517228, 0.4688317, 0.4866859, 0.5046633, 0.5224680, 0.5400851,\
        0.5573087, 0.5738387, 0.5895455, 0.6048366, 0.6197035, 0.6336267, 0.6463815, 0.6578982, 0.6682474,\
        0.6774408, 0.6852370, 0.6913273, 0.6960432, 0.7001047, 0.7037709, 0.7068836, 0.7087480, 0.7097588,\
        0.7104772, 0.7109320, 0.7108329, 0.7098634, 0.7079404, 0.7052355, 0.7028628, 0.7017521, 0.6983748,\
        0.6943965, 0.6898538, 0.6846464, 0.6791604, 0.6735246, 0.6677245, 0.6617346, 0.6555655, 0.6492586,\
        0.6428675, 0.6364462, 0.6300451, 0.6237020, 0.6174172, 0.6111260, 0.6047601, 0.5983675, 0.5917765,\
        0.5849214, 0.5780017, 0.5710515, 0.5640776, 0.5570907, 0.5501033, 0.5431265, 0.5361691, 0.5292366,\
        0.5223248, 0.5153994, 0.5083523, 0.5009976, 0.4932058, 0.4850099, 0.4765070, 0.4677368, 0.4587119,\
        0.4494359, 0.4399101, 0.4301339, 0.4201060],\
        [0.4487474, 0.4507714, 0.4524657, 0.4538748, 0.4550488, 0.4560430, 0.4569148, 0.4577178, 0.4584951,\
        0.4592724, 0.4600527, 0.4608149, 0.4615149, 0.4620900, 0.4624666, 0.4625723, 0.4623525, 0.4617823,\
        0.4608644, 0.4596143, 0.4580469, 0.4561732, 0.4539998, 0.4515220, 0.4487122, 0.4455116, 0.4418279,\
        0.4375413, 0.4325239, 0.4266851, 0.4200784, 0.4131186, 0.4066859, 0.4012230, 0.3955815, 0.3903634,\
        0.3862857, 0.3832644, 0.3811673, 0.3799795, 0.3799308, 0.3813989, 0.3847646, 0.3903570, 0.3983588,\
        0.4086192, 0.4213626, 0.4363895, 0.4531743, 0.4711649, 0.4899129, 0.5087869, 0.5273711, 0.5458316,\
        0.5639780, 0.5814249, 0.5976574, 0.6133265, 0.6284018, 0.6424544, 0.6552222, 0.6666824, 0.6770940,\
        0.6866381, 0.6949359, 0.7011399, 0.7057557, 0.7096839, 0.7130579, 0.7157271, 0.7174569, 0.7183742,\
        0.7187585, 0.7186801, 0.7180116, 0.7165988, 0.7144313, 0.7117839, 0.7092440, 0.7067919, 0.7034659,\
        0.6998547, 0.6951290, 0.6897665, 0.6843248, 0.6788000, 0.6730476, 0.6669908, 0.6606497, 0.6540936,\
        0.6474017, 0.6406447, 0.6338782, 0.6271369, 0.6204307, 0.6137594, 0.6071701, 0.6006917, 0.5937379,\
        0.5867583, 0.5798082, 0.5728309, 0.5658271, 0.5588169, 0.5518195, 0.5448478, 0.5379071, 0.5309930,\
        0.5240841, 0.5171293, 0.5100385, 0.5026907, 0.4949247, 0.4867719, 0.4783256, 0.4696105, 0.4606377,\
        0.4514139, 0.4419424, 0.4322244, 0.4222592],\
        [0.4540128, 0.4564642, 0.4585614, 0.4603252, 0.4617849, 0.4629825, 0.4639758, 0.4648341, 0.4656282,\
        0.4664158, 0.4672288, 0.4680644, 0.4688827, 0.4696114, 0.4701556, 0.4704145, 0.4703145, 0.4698383,\
        0.4690188, 0.4678991, 0.4664997, 0.4648186, 0.4628432, 0.4605465, 0.4578771, 0.4547582, 0.4510894,\
        0.4467553, 0.4416541, 0.4357548, 0.4291783, 0.4222704, 0.4154885, 0.4089309, 0.4022371, 0.3962981,\
        0.3916419, 0.3880517, 0.3852589, 0.3830945, 0.3819501, 0.3823702, 0.3847536, 0.3894290, 0.3968108,\
        0.4073709, 0.4210204, 0.4371125, 0.4549235, 0.4737738, 0.4930288, 0.5127148, 0.5323443, 0.5516790,\
        0.5707643, 0.5894439, 0.6066673, 0.6225335, 0.6377887, 0.6520168, 0.6647714, 0.6759156, 0.6859609,\
        0.6956489, 0.7041581, 0.7107029, 0.7155753, 0.7194412, 0.7225247, 0.7248124, 0.7262552, 0.7269218,\
        0.7269483, 0.7264012, 0.7252533, 0.7234655, 0.7210883, 0.7183182, 0.7153911, 0.7122651, 0.7087005,\
        0.7046075, 0.6996329, 0.6943041, 0.6890823, 0.6838025, 0.6781183, 0.6719516, 0.6653822, 0.6585297,\
        0.6515024, 0.6443847, 0.6372360, 0.6300919, 0.6229664, 0.6158584, 0.6087612, 0.6016787, 0.5947921,\
        0.5880326, 0.5811486, 0.5741535, 0.5671114, 0.5600673, 0.5530473, 0.5460640, 0.5391182, 0.5321966,\
        0.5252672, 0.5182718, 0.5111273, 0.5037383, 0.4960253, 0.4879898, 0.4796532, 0.4710302, 0.4621377,\
        0.4529886, 0.4435908, 0.4339477, 0.4240597],\
        [0.4590688, 0.4619464, 0.4644455, 0.4665616, 0.4682990, 0.4696837, 0.4707748, 0.4716626, 0.4724533,\
        0.4732435, 0.4740982, 0.4750344, 0.4760148, 0.4769560, 0.4777385, 0.4782171, 0.4782775, 0.4779130,\
        0.4772112, 0.4762611, 0.4750818, 0.4736449, 0.4719259, 0.4698883, 0.4674618, 0.4645591, 0.4610818,\
        0.4569156, 0.4519660, 0.4462154, 0.4397657, 0.4328412, 0.4256835, 0.4183868, 0.4110581, 0.4042767,\
        0.3985869, 0.3941298, 0.3904589, 0.3869012, 0.3844391, 0.3838384, 0.3853764, 0.3891948, 0.3958983,\
        0.4067463, 0.4214291, 0.4386237, 0.4574787, 0.4773301, 0.4975740, 0.5177219, 0.5378679, 0.5577343,\
        0.5772797, 0.5965599, 0.6147365, 0.6315801, 0.6474973, 0.6622547, 0.6752606, 0.6860704, 0.6951813,\
        0.7046653, 0.7133509, 0.7202004, 0.7253707, 0.7292509, 0.7321064, 0.7340497, 0.7351395, 0.7354550,\
        0.7350909, 0.7341124, 0.7325447, 0.7304092, 0.7277704, 0.7247452, 0.7214343, 0.7178210, 0.7137909,\
        0.7092050, 0.7040156, 0.6985858, 0.6937741, 0.6887758, 0.6830448, 0.6766406, 0.6697495, 0.6625444,\
        0.6551536, 0.6476657, 0.6401406, 0.6326162, 0.6251112, 0.6176251, 0.6101313, 0.6026245, 0.5958246,\
        0.5891426, 0.5821636, 0.5750672, 0.5679475, 0.5608480, 0.5537893, 0.5467787, 0.5398122, 0.5328715,\
        0.5259198, 0.5188976, 0.5117231, 0.5043076, 0.4966557, 0.4887629, 0.4805469, 0.4720265, 0.4632279,\
        0.4541690, 0.4448603, 0.4353071, 0.4255105],\
        [0.4638188, 0.4671222, 0.4700286, 0.4725039, 0.4745199, 0.4760784, 0.4772372, 0.4781163, 0.4788699,\
        0.4796436, 0.4805410, 0.4816036, 0.4827908, 0.4840030, 0.4851095, 0.4859011, 0.4861686, 0.4859218,\
        0.4853522, 0.4846328, 0.4837545, 0.4826202, 0.4812200, 0.4795312, 0.4774495, 0.4748827, 0.4717554,\
        0.4679372, 0.4633083, 0.4578373, 0.4515775, 0.4446739, 0.4373115, 0.4296313, 0.4218185, 0.4142355,\
        0.4073036, 0.4016260, 0.3964592, 0.3909676, 0.3871731, 0.3859415, 0.3872568, 0.3907287, 0.3969607,\
        0.4081289, 0.4235255, 0.4415217, 0.4611661, 0.4817901, 0.5027935, 0.5235569, 0.5442176, 0.5645139,\
        0.5840535, 0.6034151, 0.6225562, 0.6405123, 0.6573296, 0.6729560, 0.6867342, 0.6976896, 0.7065478,\
        0.7152626, 0.7234987, 0.7302628, 0.7354381, 0.7392151, 0.7418121, 0.7433931, 0.7440818, 0.7439867,\
        0.7432030, 0.7418029, 0.7398372, 0.7373518, 0.7344067, 0.7310732, 0.7274008, 0.7233838, 0.7189679,\
        0.7141023, 0.7088449, 0.7035893, 0.6989752, 0.6938640, 0.6878056, 0.6809850, 0.6736807, 0.6660881,\
        0.6583287, 0.6504823, 0.6426051, 0.6347394, 0.6269211, 0.6191923, 0.6116325, 0.6044449, 0.5975271,\
        0.5901699, 0.5828427, 0.5755656, 0.5683394, 0.5611691, 0.5540587, 0.5470086, 0.5400116, 0.5330485,\
        0.5260848, 0.5190678, 0.5119305, 0.5045994, 0.4970338, 0.4891995, 0.4810590, 0.4726287, 0.4639268,\
        0.4549675, 0.4457605, 0.4363105, 0.4266185],\
        [0.4681594, 0.4718856, 0.4752081, 0.4780609, 0.4803710, 0.4820973, 0.4832886, 0.4841078, 0.4847815,\
        0.4855136, 0.4864452, 0.4876598, 0.4891015, 0.4906138, 0.4921167, 0.4933561, 0.4938822, 0.4937668,\
        0.4933505, 0.4929605, 0.4925090, 0.4917142, 0.4906855, 0.4894499, 0.4877975, 0.4856416, 0.4830011,\
        0.4796755, 0.4754758, 0.4703856, 0.4643811, 0.4575624, 0.4501603, 0.4423020, 0.4340731, 0.4256494,\
        0.4173127, 0.4094145, 0.4017800, 0.3944908, 0.3896948, 0.3885615, 0.3906449, 0.3950625, 0.4007592,\
        0.4122639, 0.4278594, 0.4461402, 0.4661794, 0.4871401, 0.5084990, 0.5299824, 0.5514944, 0.5727925,\
        0.5929601, 0.6122894, 0.6316176, 0.6499718, 0.6672626, 0.6834962, 0.6983889, 0.7096822, 0.7188565,\
        0.7271854, 0.7347455, 0.7410775, 0.7459628, 0.7494453, 0.7516758, 0.7528252, 0.7530544, 0.7525009,\
        0.7512709, 0.7494433, 0.7470795, 0.7442337, 0.7409581, 0.7373003, 0.7332914, 0.7289389, 0.7242402,\
        0.7192261, 0.7140339, 0.7089515, 0.7040623, 0.6986569, 0.6921059, 0.6847761, 0.6770508, 0.6690937,\
        0.6609976, 0.6528269, 0.6446329, 0.6364619, 0.6283640, 0.6204080, 0.6126967, 0.6053114, 0.5979928,\
        0.5904931, 0.5830013, 0.5756030, 0.5682886, 0.5610485, 0.5538797, 0.5467802, 0.5397440, 0.5327562,\
        0.5257885, 0.5187968, 0.5117290, 0.5045435, 0.4970909, 0.4892694, 0.4811905, 0.4728488, 0.4642479,\
        0.4553970, 0.4463029, 0.4369686, 0.4273937],\
        [0.4719806, 0.4761150, 0.4798587, 0.4831168, 0.4857581, 0.4876634, 0.4888476, 0.4895435, 0.4900976,\
        0.4907830, 0.4917255, 0.4930980, 0.4948954, 0.4966718, 0.4985357, 0.5003246, 0.5011748, 0.5013502,\
        0.5012278, 0.5012667, 0.5013250, 0.5009058, 0.5003120, 0.4996164, 0.4984374, 0.4967171, 0.4946835,\
        0.4919512, 0.4881938, 0.4836017, 0.4779035, 0.4711684, 0.4638905, 0.4560586, 0.4475440, 0.4383169,\
        0.4282789, 0.4175200, 0.4066074, 0.3971436, 0.3917863, 0.3915584, 0.3949292, 0.4007820, 0.4081910,\
        0.4191533, 0.4342620, 0.4523580, 0.4725378, 0.4938446, 0.5154017, 0.5372478, 0.5595290, 0.5823264,\
        0.6041604, 0.6236390, 0.6423239, 0.6603327, 0.6775444, 0.6936441, 0.7085218, 0.7209873, 0.7309789,\
        0.7394206, 0.7466222, 0.7524958, 0.7569351, 0.7599637, 0.7617010, 0.7623208, 0.7620198, 0.7609637,\
        0.7592635, 0.7569959, 0.7542252, 0.7510084, 0.7473911, 0.7434064, 0.7390768, 0.7344197, 0.7294631,\
        0.7242698, 0.7189570, 0.7136467, 0.7082709, 0.7024082, 0.6954516, 0.6877734, 0.6797517, 0.6715173,\
        0.6631485, 0.6547036, 0.6462329, 0.6377841, 0.6294082, 0.6211650, 0.6131177, 0.6052868, 0.5976409,\
        0.5901211, 0.5826339, 0.5751991, 0.5678277, 0.5605233, 0.5532894, 0.5461289, 0.5390415, 0.5320209,\
        0.5250487, 0.5180856, 0.5110594, 0.5038838, 0.4965276, 0.4888829, 0.4809258, 0.4726910, 0.4642017,\
        0.4554701, 0.4465010, 0.4372949, 0.4278493],\
        [0.4751696, 0.4796736, 0.4838208, 0.4875056, 0.4905416, 0.4926730, 0.4938094, 0.4943136, 0.4947275,\
        0.4954527, 0.4964223, 0.4977617, 0.5001828, 0.5022251, 0.5044029, 0.5064284, 0.5078125, 0.5085968,\
        0.5091006, 0.5095984, 0.5100307, 0.5101930, 0.5101471, 0.5099088, 0.5092581, 0.5081376, 0.5066444,\
        0.5044343, 0.5011442, 0.4971971, 0.4917552, 0.4849952, 0.4781363, 0.4705861, 0.4620087, 0.4524328,\
        0.4410269, 0.4274925, 0.4129886, 0.4003901, 0.3937983, 0.3953896, 0.4002445, 0.4068127, 0.4156002,\
        0.4271538, 0.4419535, 0.4596408, 0.4797057, 0.5013685, 0.5232201, 0.5455246, 0.5681850, 0.5918005,\
        0.6153586, 0.6359610, 0.6542194, 0.6716191, 0.6886933, 0.7048866, 0.7197402, 0.7326384, 0.7432249,\
        0.7518563, 0.7588644, 0.7643268, 0.7682480, 0.7707152, 0.7718472, 0.7718267, 0.7709205, 0.7693299,\
        0.7671444, 0.7644221, 0.7612315, 0.7576354, 0.7536726, 0.7493610, 0.7447117, 0.7397420, 0.7344869,\
        0.7290045, 0.7233635, 0.7175898, 0.7115737, 0.7050355, 0.6977764, 0.6899728, 0.6817972, 0.6733811,\
        0.6648075, 0.6561400, 0.6474331, 0.6387358, 0.6300936, 0.6215493, 0.6131462, 0.6049498, 0.5970588,\
        0.5894413, 0.5819316, 0.5744610, 0.5670298, 0.5596515, 0.5523383, 0.5450996, 0.5379422, 0.5308702,\
        0.5238822, 0.5169597, 0.5100314, 0.5029123, 0.4956508, 0.4881437, 0.4802914, 0.4721663, 0.4638000,\
        0.4552014, 0.4463710, 0.4373059, 0.4280014],\
        [0.4776259, 0.4824230, 0.4869040, 0.4909812, 0.4944657, 0.4969426, 0.4979870, 0.4983064, 0.4985840,\
        0.4995475, 0.5007884, 0.5021924, 0.5046884, 0.5073206, 0.5098427, 0.5121600, 0.5140578, 0.5155186,\
        0.5167023, 0.5177561, 0.5186855, 0.5194233, 0.5199509, 0.5202309, 0.5201575, 0.5196572, 0.5186532,\
        0.5169080, 0.5142297, 0.5105875, 0.5052453, 0.4986906, 0.4926503, 0.4853986, 0.4769106, 0.4682675,\
        0.4564773, 0.4407520, 0.4228628, 0.4061761, 0.3976064, 0.4007323, 0.4069489, 0.4143288, 0.4238292,\
        0.4358058, 0.4504488, 0.4676146, 0.4870519, 0.5087544, 0.5317557, 0.5554788, 0.5792281, 0.6032838,\
        0.6292452, 0.6506021, 0.6674265, 0.6836836, 0.7008141, 0.7172853, 0.7323639, 0.7454070, 0.7561057,\
        0.7646536, 0.7714139, 0.7764520, 0.7797602, 0.7815966, 0.7820457, 0.7812509, 0.7796761, 0.7775568,\
        0.7748862, 0.7716869, 0.7680569, 0.7640775, 0.7597739, 0.7551376, 0.7501589, 0.7448462, 0.7392306,\
        0.7333571, 0.7272625, 0.7209378, 0.7142947, 0.7071961, 0.6996078, 0.6916222, 0.6833264, 0.6747750,\
        0.6660387, 0.6571868, 0.6482797, 0.6393696, 0.6305010, 0.6217079, 0.6130152, 0.6044887, 0.5963855,\
        0.5886806, 0.5810802, 0.5735120, 0.5659803, 0.5585004, 0.5510856, 0.5437470, 0.5364948, 0.5293405,\
        0.5222997, 0.5153930, 0.5086348, 0.5019601, 0.4947068, 0.4870926, 0.4792912, 0.4712827, 0.4630577,\
        0.4546099, 0.4459335, 0.4370223, 0.4278699],\
        [0.4792864, 0.4842634, 0.4889380, 0.4932450, 0.4970377, 0.4999289, 0.5009782, 0.5014087, 0.5019393,\
        0.5031204, 0.5047328, 0.5066848, 0.5092283, 0.5120895, 0.5149467, 0.5176372, 0.5200410, 0.5221308,\
        0.5239701, 0.5256314, 0.5271343, 0.5284565, 0.5295680, 0.5304219, 0.5309430, 0.5310410, 0.5305877,\
        0.5293857, 0.5272174, 0.5238305, 0.5189570, 0.5130263, 0.5069562, 0.4985127, 0.4913525, 0.4859746,\
        0.4747475, 0.4576161, 0.4378045, 0.4201071, 0.4104702, 0.4109193, 0.4160690, 0.4235727, 0.4332455,\
        0.4452719, 0.4597291, 0.4765182, 0.4954700, 0.5171196, 0.5412380, 0.5671989, 0.5944114, 0.6216492,\
        0.6488420, 0.6695397, 0.6823538, 0.6964587, 0.7140021, 0.7311430, 0.7465134, 0.7595214, 0.7698918,\
        0.7778708, 0.7842000, 0.7888009, 0.7912665, 0.7924503, 0.7922214, 0.7904434, 0.7882067, 0.7856422,\
        0.7824847, 0.7787529, 0.7746562, 0.7703031, 0.7656789, 0.7607256, 0.7554029, 0.7497094, 0.7436750,\
        0.7373413, 0.7307379, 0.7238602, 0.7166609, 0.7090785, 0.7011148, 0.6929406, 0.6845090, 0.6758137,\
        0.6669220, 0.6579043, 0.6488227, 0.6397336, 0.6306900, 0.6217390, 0.6129134, 0.6042344, 0.5959395,\
        0.5880130, 0.5801835, 0.5724284, 0.5647439, 0.5571305, 0.5495918, 0.5421338, 0.5347644, 0.5274941,\
        0.5203408, 0.5133391, 0.5065257, 0.4998387, 0.4929084, 0.4855140, 0.4778864, 0.4700508, 0.4619991,\
        0.4537231, 0.4452155, 0.4364695, 0.4274780],\
        [0.4801513, 0.4851876, 0.4898865, 0.4941723, 0.4979071, 0.5007201, 0.5023480, 0.5034850, 0.5046854,\
        0.5062727, 0.5082832, 0.5106870, 0.5135018, 0.5165981, 0.5197700, 0.5228569, 0.5257565, 0.5284294,\
        0.5308884, 0.5331617, 0.5352606, 0.5371740, 0.5388751, 0.5403204, 0.5414452, 0.5421580, 0.5423251,\
        0.5417553, 0.5402041, 0.5373987, 0.5331638, 0.5276018, 0.5207193, 0.5123737, 0.5074367, 0.5038004,\
        0.4939515, 0.4757283, 0.4555170, 0.4385293, 0.4283000, 0.4256569, 0.4284527, 0.4347651, 0.4438435,\
        0.4554892, 0.4696297, 0.4862100, 0.5051576, 0.5266663, 0.5511771, 0.5784936, 0.6084889, 0.6395382,\
        0.6700373, 0.6926952, 0.7004454, 0.7125310, 0.7300659, 0.7473222, 0.7624657, 0.7750581, 0.7847319,\
        0.7914391, 0.7969176, 0.8011491, 0.8026165, 0.8030419, 0.8020754, 0.7993144, 0.7965713, 0.7936753,\
        0.7899347, 0.7855502, 0.7809769, 0.7762969, 0.7713961, 0.7661382, 0.7604501, 0.7543326, 0.7478292,\
        0.7409921, 0.7338615, 0.7264551, 0.7187699, 0.7107951, 0.7025472, 0.6941231, 0.6854728, 0.6765859,\
        0.6675257, 0.6583481, 0.6491054, 0.6398551, 0.6306629, 0.6216049, 0.6127679, 0.6042208, 0.5957586,\
        0.5873649, 0.5792017, 0.5712132, 0.5633503, 0.5555871, 0.5479130, 0.5403269, 0.5328326, 0.5254351,\
        0.5181380, 0.5109602, 0.5039924, 0.4972574, 0.4905314, 0.4834885, 0.4761344, 0.4685217, 0.4606693,\
        0.4525802, 0.4442513, 0.4356774, 0.4268523],\
        [0.4802903, 0.4853053, 0.4899243, 0.4940424, 0.4975086, 0.5002130, 0.5025927, 0.5046745, 0.5066649,\
        0.5088405, 0.5113337, 0.5141755, 0.5173473, 0.5207605, 0.5242801, 0.5277783, 0.5311632, 0.5343849,\
        0.5374284, 0.5402943, 0.5429811, 0.5454769, 0.5477584, 0.5497893, 0.5515165, 0.5528616, 0.5537082,\
        0.5538834, 0.5531400, 0.5511713, 0.5477226, 0.5427395, 0.5364285, 0.5297160, 0.5242555, 0.5184718,\
        0.5079748, 0.4906872, 0.4721175, 0.4566197, 0.4464164, 0.4419603, 0.4425393, 0.4471633, 0.4551265,\
        0.4660146, 0.4796006, 0.4957378, 0.5144684, 0.5361109, 0.5610501, 0.5893502, 0.6208546, 0.6533889,\
        0.6840651, 0.7085338, 0.7208948, 0.7335959, 0.7496972, 0.7661035, 0.7800637, 0.7915062, 0.8005176,\
        0.8057125, 0.8098511, 0.8128456, 0.8138160, 0.8133153, 0.8114532, 0.8084189, 0.8050935, 0.8016087,\
        0.7970514, 0.7919610, 0.7869977, 0.7820910, 0.7769741, 0.7714114, 0.7653151, 0.7587194, 0.7517014,\
        0.7443333, 0.7366697, 0.7287521, 0.7206193, 0.7123141, 0.7038623, 0.6951775, 0.6862249, 0.6771195,\
        0.6678938, 0.6585654, 0.6491666, 0.6397514, 0.6303927, 0.6211795, 0.6122070, 0.6035278, 0.5949437,\
        0.5863635, 0.5779989, 0.5698303, 0.5618097, 0.5539043, 0.5460983, 0.5383885, 0.5307802, 0.5232824,\
        0.5158952, 0.5085831, 0.5014040, 0.4947903, 0.4881772, 0.4813250, 0.4741897, 0.4667868, 0.4591316,\
        0.4512297, 0.4430802, 0.4346794, 0.4260216],\
        [0.4798176, 0.4847950, 0.4893492, 0.4933836, 0.4968026, 0.4997260, 0.5026304, 0.5053952, 0.5080567,\
        0.5108227, 0.5138317, 0.5171338, 0.5207145, 0.5245085, 0.5284188, 0.5323463, 0.5362097, 0.5399547,\
        0.5435503, 0.5469795, 0.5502291, 0.5532827, 0.5561188, 0.5587094, 0.5610163, 0.5629843, 0.5645315,\
        0.5655313, 0.5657639, 0.5648553, 0.5623777, 0.5581892, 0.5526058, 0.5462447, 0.5394041, 0.5309193,\
        0.5187081, 0.5027291, 0.4861398, 0.4728605, 0.4632843, 0.4577499, 0.4565535, 0.4595355, 0.4662396,\
        0.4761335, 0.4889434, 0.5044940, 0.5230113, 0.5450848, 0.5708753, 0.6005936, 0.6337574, 0.6668250,\
        0.6953187, 0.7203918, 0.7390079, 0.7543215, 0.7704291, 0.7865074, 0.7988053, 0.8081992, 0.8156010,\
        0.8202092, 0.8231042, 0.8246949, 0.8247812, 0.8234634, 0.8209311, 0.8174684, 0.8134718, 0.8089774,\
        0.8036909, 0.7980829, 0.7928462, 0.7878028, 0.7825044, 0.7765838, 0.7699906, 0.7628471, 0.7552785,\
        0.7473698, 0.7391774, 0.7307506, 0.7221528, 0.7134687, 0.7047484, 0.6958227, 0.6866560, 0.6774073,\
        0.6680650, 0.6586069, 0.6490524, 0.6394559, 0.6298931, 0.6204514, 0.6112090, 0.6022080, 0.5934954,\
        0.5849552, 0.5765444, 0.5682776, 0.5601402, 0.5521143, 0.5441901, 0.5363685, 0.5286619, 0.5210984,\
        0.5137242, 0.5065870, 0.4995697, 0.4927198, 0.4860033, 0.4791871, 0.4721756, 0.4649335, 0.4574502,\
        0.4497213, 0.4417427, 0.4335092, 0.4250146],\
        [0.4788481, 0.4838166, 0.4883878, 0.4925164, 0.4961954, 0.4995345, 0.5027423, 0.5058595, 0.5089901,\
        0.5122698, 0.5157837, 0.5195613, 0.5235855, 0.5278028, 0.5321376, 0.5365094, 0.5408466, 0.5450936,\
        0.5492110, 0.5531704, 0.5569489, 0.5605243, 0.5638746, 0.5669789, 0.5698162, 0.5723570, 0.5745558,\
        0.5763599, 0.5776548, 0.5780170, 0.5766309, 0.5731519, 0.5680052, 0.5614603, 0.5533600, 0.5429988,\
        0.5296071, 0.5136940, 0.4986636, 0.4875564, 0.4783103, 0.4717504, 0.4690996, 0.4707048, 0.4762767,\
        0.4852047, 0.4964641, 0.5115155, 0.5303790, 0.5534703, 0.5807738, 0.6125288, 0.6478959, 0.6836985,\
        0.7151046, 0.7400757, 0.7588004, 0.7743678, 0.7904439, 0.8058234, 0.8173422, 0.8251939, 0.8308372,\
        0.8343571, 0.8361555, 0.8365642, 0.8356613, 0.8335308, 0.8303309, 0.8262779, 0.8215800, 0.8163067,\
        0.8104966, 0.8044978, 0.7988195, 0.7936078, 0.7880753, 0.7816270, 0.7743905, 0.7666371, 0.7585156,\
        0.7500931, 0.7414039, 0.7324809, 0.7233799, 0.7142007, 0.7050758, 0.6959463, 0.6867435, 0.6775039,\
        0.6681131, 0.6585397, 0.6488206, 0.6390245, 0.6292327, 0.6195360, 0.6100360, 0.6008258, 0.5919835,\
        0.5833899, 0.5749413, 0.5666072, 0.5583789, 0.5502518, 0.5422252, 0.5343049, 0.5265075, 0.5188697,\
        0.5114648, 0.5044131, 0.4976755, 0.4907360, 0.4838999, 0.4770778, 0.4701395, 0.4630161, 0.4556745,\
        0.4480977, 0.4402750, 0.4321980, 0.4238587],\
        [0.4774696, 0.4824621, 0.4871097, 0.4914168, 0.4954267, 0.4991928, 0.5026297, 0.5059521, 0.5094576,\
        0.5132058, 0.5172111, 0.5214711, 0.5259590, 0.5306249, 0.5354042, 0.5402280, 0.5450317, 0.5497603,\
        0.5543699, 0.5588247, 0.5630942, 0.5671496, 0.5709644, 0.5745206, 0.5778151, 0.5808470, 0.5835812,\
        0.5860078, 0.5882416, 0.5900674, 0.5898598, 0.5868015, 0.5820869, 0.5754686, 0.5666549, 0.5553507,\
        0.5413299, 0.5256503, 0.5112410, 0.4997853, 0.4904937, 0.4827263, 0.4792599, 0.4799005, 0.4843344,\
        0.4925341, 0.5032571, 0.5168268, 0.5361947, 0.5609244, 0.5906134, 0.6247827, 0.6625219, 0.7014514,\
        0.7376232, 0.7652037, 0.7820339, 0.7951136, 0.8107783, 0.8247009, 0.8350481, 0.8418133, 0.8460029,\
        0.8482528, 0.8489297, 0.8482702, 0.8464111, 0.8434635, 0.8395623, 0.8348673, 0.8295300, 0.8236717,\
        0.8174517, 0.8111612, 0.8051528, 0.7995518, 0.7936077, 0.7862814, 0.7782845, 0.7699447, 0.7613422,\
        0.7524906, 0.7433884, 0.7340385, 0.7244633, 0.7147316, 0.7050229, 0.6957090, 0.6867331, 0.6776059,\
        0.6681620, 0.6584379, 0.6485238, 0.6385130, 0.6284916, 0.6185444, 0.6087987, 0.5994471, 0.5904825,\
        0.5818138, 0.5732971, 0.5648834, 0.5565659, 0.5483475, 0.5402332, 0.5322309, 0.5243538, 0.5166272,\
        0.5091000, 0.5018648, 0.4950836, 0.4884279, 0.4817184, 0.4749567, 0.4680892, 0.4610617, 0.4538367,\
        0.4463904, 0.4387064, 0.4307723, 0.4225776],\
        [0.4757426, 0.4807744, 0.4855052, 0.4899584, 0.4941826, 0.4982137, 0.5018325, 0.5054640, 0.5094204,\
        0.5136490, 0.5181431, 0.5228872, 0.5278464, 0.5329708, 0.5382011, 0.5434751, 0.5487328, 0.5539203,\
        0.5589912, 0.5639055, 0.5686270, 0.5731203, 0.5773496, 0.5812880, 0.5849459, 0.5883830, 0.5915601,\
        0.5943317, 0.5970265, 0.5999512, 0.6009530, 0.5982089, 0.5945705, 0.5881317, 0.5793170, 0.5679620,\
        0.5535595, 0.5370281, 0.5206436, 0.5068669, 0.4962823, 0.4897811, 0.4872225, 0.4874141, 0.4900964,\
        0.4959597, 0.5054087, 0.5196288, 0.5400586, 0.5665908, 0.5988043, 0.6357181, 0.6769013, 0.7201196,\
        0.7613289, 0.7929372, 0.8094674, 0.8205717, 0.8335786, 0.8450905, 0.8533383, 0.8583398, 0.8609449,\
        0.8618797, 0.8614328, 0.8597459, 0.8569600, 0.8531956, 0.8485726, 0.8432221, 0.8372765, 0.8308760,\
        0.8242053, 0.8175164, 0.8110698, 0.8049382, 0.7981179, 0.7898950, 0.7813506, 0.7726177, 0.7636933,\
        0.7545523, 0.7451731, 0.7355434, 0.7256639, 0.7155670, 0.7054259, 0.6959823, 0.6871896, 0.6779860,\
        0.6683172, 0.6583352, 0.6481726, 0.6379389, 0.6277225, 0.6175851, 0.6075764, 0.5979833, 0.5890508,\
        0.5803037, 0.5716697, 0.5631423, 0.5547229, 0.5464162, 0.5382290, 0.5301697, 0.5222471, 0.5144734,\
        0.5068860, 0.4995940, 0.4927813, 0.4861597, 0.4795314, 0.4728355, 0.4660333, 0.4590850, 0.4519564,\
        0.4446209, 0.4370583, 0.4292527, 0.4211908],\
        [0.4737165, 0.4787917, 0.4835899, 0.4881282, 0.4924221, 0.4964431, 0.5002443, 0.5045048, 0.5089543,\
        0.5136534, 0.5186243, 0.5238444, 0.5292699, 0.5348483, 0.5405225, 0.5462344, 0.5519268, 0.5575464,\
        0.5630450, 0.5683800, 0.5735131, 0.5784054, 0.5830110, 0.5872763, 0.5911827, 0.5948891, 0.5985778,\
        0.6016389, 0.6046154, 0.6075909, 0.6088811, 0.6076309, 0.6047146, 0.5987657, 0.5911727, 0.5806946,\
        0.5661028, 0.5480441, 0.5286557, 0.5104266, 0.4979553, 0.4957700, 0.4950170, 0.4950914, 0.4957537,\
        0.4974693, 0.5053457, 0.5209547, 0.5423125, 0.5700360, 0.6039931, 0.6434815, 0.6884500, 0.7366758,\
        0.7823320, 0.8161213, 0.8349405, 0.8469957, 0.8577876, 0.8665564, 0.8722080, 0.8748449, 0.8755922,\
        0.8752440, 0.8736985, 0.8709709, 0.8672617, 0.8626724, 0.8573070, 0.8512841, 0.8447136, 0.8377185,\
        0.8304577, 0.8231149, 0.8158200, 0.8084581, 0.8005302, 0.7921587, 0.7834870, 0.7746172, 0.7655511,\
        0.7562691, 0.7467611, 0.7370310, 0.7271036, 0.7170485, 0.7070435, 0.6977039, 0.6885045, 0.6786769,\
        0.6685240, 0.6581633, 0.6477017, 0.6372427, 0.6268867, 0.6167203, 0.6067770, 0.5971249, 0.5878916,\
        0.5788805, 0.5700475, 0.5613719, 0.5528399, 0.5444483, 0.5362028, 0.5281152, 0.5201972, 0.5124518,\
        0.5048942, 0.4976426, 0.4907493, 0.4840569, 0.4774162, 0.4707421, 0.4639820, 0.4570943, 0.4500438,\
        0.4428017, 0.4353448, 0.4276541, 0.4197132],\
        [0.4714396, 0.4765661, 0.4814318, 0.4860502, 0.4904431, 0.4946691, 0.4989402, 0.5034126, 0.5081811,\
        0.5132899, 0.5187108, 0.5243873, 0.5302599, 0.5362726, 0.5423702, 0.5484974, 0.5545982, 0.5606185,\
        0.5665072, 0.5722195, 0.5777173, 0.5829662, 0.5879222, 0.5925107, 0.5966269, 0.6003888, 0.6043173,\
        0.6078683, 0.6110806, 0.6138167, 0.6152728, 0.6148909, 0.6125599, 0.6081514, 0.6021779, 0.5930176,\
        0.5788542, 0.5596241, 0.5387452, 0.5194802, 0.5073596, 0.5052546, 0.5049186, 0.5048866, 0.5046475,\
        0.5051981, 0.5109807, 0.5242360, 0.5441279, 0.5715441, 0.6067966, 0.6485852, 0.6962619, 0.7493491,\
        0.7989182, 0.8347160, 0.8557613, 0.8693014, 0.8803995, 0.8874655, 0.8909689, 0.8910021, 0.8898680,\
        0.8884933, 0.8857943, 0.8819208, 0.8772956, 0.8718468, 0.8657047, 0.8589877, 0.8517549, 0.8440843,\
        0.8360936, 0.8279097, 0.8195994, 0.8111117, 0.8024116, 0.7936803, 0.7849383, 0.7760539, 0.7669498,\
        0.7576214, 0.7480861, 0.7383767, 0.7285576, 0.7187547, 0.7092003, 0.6998608, 0.6897210, 0.6791983,\
        0.6685144, 0.6577473, 0.6469762, 0.6362881, 0.6257927, 0.6156330, 0.6059442, 0.5963843, 0.5866905,\
        0.5773656, 0.5683313, 0.5595144, 0.5508785, 0.5424116, 0.5341191, 0.5260205, 0.5181449, 0.5105056,\
        0.5030483, 0.4958128, 0.4888417, 0.4820529, 0.4753579, 0.4686726, 0.4619325, 0.4550884, 0.4481010,\
        0.4409382, 0.4335737, 0.4259857, 0.4181552],\
        [0.4689578, 0.4741542, 0.4791091, 0.4838396, 0.4883843, 0.4928329, 0.4973417, 0.5020549, 0.5071177,\
        0.5126178, 0.5184670, 0.5245680, 0.5308512, 0.5372619, 0.5437489, 0.5502589, 0.5567361, 0.5631225,\
        0.5693616, 0.5754027, 0.5812061, 0.5867464, 0.5920016, 0.5969017, 0.6013011, 0.6052420, 0.6091363,\
        0.6128135, 0.6160856, 0.6187149, 0.6202596, 0.6203299, 0.6187373, 0.6154674, 0.6104753, 0.6031455,\
        0.5900686, 0.5706902, 0.5509536, 0.5339219, 0.5230152, 0.5188379, 0.5179646, 0.5178898, 0.5176824,\
        0.5180357, 0.5214283, 0.5302163, 0.5460603, 0.5711052, 0.6070594, 0.6532628, 0.7058720, 0.7618950,\
        0.8150695, 0.8539588, 0.8738997, 0.8869467, 0.8995399, 0.9063761, 0.9085069, 0.9070219, 0.9044149,\
        0.9019193, 0.8976182, 0.8925722, 0.8870909, 0.8806439, 0.8736963, 0.8662860, 0.8583575, 0.8499383,\
        0.8411264, 0.8320418, 0.8227713, 0.8133496, 0.8038711, 0.7947708, 0.7860099, 0.7770847, 0.7679262,\
        0.7585642, 0.7490274, 0.7393539, 0.7296114, 0.7198942, 0.7102170, 0.7002791, 0.6899276, 0.6790550,\
        0.6679786, 0.6568815, 0.6458488, 0.6349426, 0.6242448, 0.6138913, 0.6039985, 0.5944038, 0.5848663,\
        0.5755156, 0.5664012, 0.5574989, 0.5487888, 0.5402641, 0.5319312, 0.5238136, 0.5159611, 0.5084466,\
        0.5012472, 0.4939731, 0.4868950, 0.4800318, 0.4732930, 0.4665940, 0.4598665, 0.4530572, 0.4461231,\
        0.4390295, 0.4317473, 0.4242523, 0.4165234],\
        [0.4663075, 0.4715999, 0.4766764, 0.4815560, 0.4862760, 0.4909051, 0.4955585, 0.5004221, 0.5058187,\
        0.5117315, 0.5179755, 0.5244428, 0.5310760, 0.5378304, 0.5446598, 0.5515127, 0.5583303, 0.5650487,\
        0.5716012, 0.5779242, 0.5839668, 0.5897025, 0.5951340, 0.6002304, 0.6048008, 0.6089533, 0.6128632,\
        0.6165033, 0.6197122, 0.6222538, 0.6238180, 0.6241061, 0.6229007, 0.6200234, 0.6151219, 0.6072090,\
        0.5949019, 0.5792763, 0.5627685, 0.5485785, 0.5390797, 0.5346813, 0.5337436, 0.5341782, 0.5344732,\
        0.5342417, 0.5346668, 0.5382057, 0.5482269, 0.5690734, 0.6052380, 0.6571553, 0.7164061, 0.7753123,\
        0.8346773, 0.8792353, 0.8913928, 0.9033187, 0.9156402, 0.9231884, 0.9250603, 0.9231883, 0.9194919,\
        0.9149739, 0.9093111, 0.9032493, 0.8964737, 0.8888812, 0.8812408, 0.8731719, 0.8645089, 0.8552694,\
        0.8455718, 0.8355825, 0.8254679, 0.8153671, 0.8053999, 0.7960038, 0.7869648, 0.7777628, 0.7684459,\
        0.7590147, 0.7494536, 0.7397620, 0.7299704, 0.7201373, 0.7102960, 0.7002816, 0.6893312, 0.6780556,\
        0.6667053, 0.6554022, 0.6442142, 0.6331685, 0.6222769, 0.6116077, 0.6014487, 0.5918816, 0.5825557,\
        0.5733194, 0.5642131, 0.5552765, 0.5465268, 0.5379694, 0.5296084, 0.5214561, 0.5135463, 0.5059650,\
        0.4987891, 0.4917209, 0.4847346, 0.4778954, 0.4711602, 0.4644675, 0.4577591, 0.4509844, 0.4441003,\
        0.4370704, 0.4298641, 0.4224553, 0.4148211],\
        [0.4635105, 0.4689267, 0.4741597, 0.4792297, 0.4841684, 0.4890209, 0.4938534, 0.4988506, 0.5045390,\
        0.5108033, 0.5173279, 0.5240555, 0.5309517, 0.5379794, 0.5450948, 0.5522450, 0.5593666, 0.5663873,\
        0.5732263, 0.5798002, 0.5860341, 0.5918793, 0.5973363, 0.6024800, 0.6072025, 0.6114748, 0.6153894,\
        0.6189530, 0.6220553, 0.6244973, 0.6260234, 0.6263652, 0.6252644, 0.6224296, 0.6174160, 0.6095972,\
        0.5989495, 0.5863228, 0.5729620, 0.5614812, 0.5537890, 0.5504086, 0.5504922, 0.5523547, 0.5539013,\
        0.5532779, 0.5505177, 0.5482943, 0.5512244, 0.5657592, 0.6000429, 0.6577541, 0.7231331, 0.7870889,\
        0.8521800, 0.9029290, 0.9161128, 0.9251621, 0.9349837, 0.9405297, 0.9413217, 0.9385521, 0.9336670,\
        0.9275913, 0.9206837, 0.9132181, 0.9050092, 0.8966567, 0.8884476, 0.8796672, 0.8701854, 0.8600311,\
        0.8493689, 0.8384451, 0.8275304, 0.8169173, 0.8069287, 0.7973881, 0.7874993, 0.7778646, 0.7683626,\
        0.7588603, 0.7492618, 0.7394991, 0.7295363, 0.7193781, 0.7090524, 0.6984878, 0.6874382, 0.6760362,\
        0.6645697, 0.6531960, 0.6419970, 0.6309869, 0.6201154, 0.6093090, 0.5988338, 0.5894012, 0.5801324,\
        0.5709022, 0.5617757, 0.5528152, 0.5440529, 0.5354974, 0.5271443, 0.5189876, 0.5110259, 0.5032795,\
        0.4960586, 0.4891951, 0.4823793, 0.4756245, 0.4689300, 0.4622650, 0.4555872, 0.4488527, 0.4420204,\
        0.4350535, 0.4279203, 0.4205936, 0.4130498],\
        [0.4605718, 0.4661352, 0.4715548, 0.4768550, 0.4820708, 0.4872519, 0.4924721, 0.4978382, 0.5036914,\
        0.5099579, 0.5165496, 0.5234043, 0.5304650, 0.5376897, 0.5450308, 0.5524306, 0.5598202, 0.5671189,\
        0.5742323, 0.5810537, 0.5874755, 0.5934222, 0.5988950, 0.6040052, 0.6087051, 0.6129409, 0.6167785,\
        0.6202367, 0.6232309, 0.6255851, 0.6270642, 0.6274063, 0.6263392, 0.6235763, 0.6188246, 0.6119253,\
        0.6031003, 0.5923097, 0.5811741, 0.5719321, 0.5661016, 0.5642811, 0.5660582, 0.5701625, 0.5741907,\
        0.5741450, 0.5685825, 0.5609559, 0.5565960, 0.5639058, 0.5948710, 0.6593877, 0.7284153, 0.7915569,\
        0.8611160, 0.9152473, 0.9395372, 0.9498076, 0.9558185, 0.9581449, 0.9567701, 0.9524582, 0.9462212,\
        0.9388501, 0.9307997, 0.9222809, 0.9133838, 0.9044194, 0.8953422, 0.8856896, 0.8752910, 0.8640984,\
        0.8523636, 0.8404318, 0.8286326, 0.8172784, 0.8066611, 0.7966402, 0.7867348, 0.7770396, 0.7675023,\
        0.7579947, 0.7483904, 0.7385620, 0.7283908, 0.7178101, 0.7068619, 0.6957593, 0.6845298, 0.6730564,\
        0.6615363, 0.6501721, 0.6390839, 0.6283067, 0.6178024, 0.6074492, 0.5971856, 0.5872964, 0.5776915,\
        0.5682738, 0.5590541, 0.5500620, 0.5413143, 0.5328079, 0.5245239, 0.5164431, 0.5085742, 0.5009528,\
        0.4936668, 0.4867361, 0.4799560, 0.4732508, 0.4665968, 0.4599684, 0.4533309, 0.4466450, 0.4398706,\
        0.4329703, 0.4259110, 0.4186653, 0.4112097],\
        [0.4574823, 0.4632061, 0.4688285, 0.4743793, 0.4798987, 0.4854444, 0.4911095, 0.4970395, 0.5028479,\
        0.5089412, 0.5155353, 0.5224273, 0.5295684, 0.5369208, 0.5444302, 0.5520316, 0.5596513, 0.5672045,\
        0.5745895, 0.5816811, 0.5883313, 0.5944045, 0.5999083, 0.6049248, 0.6094225, 0.6134510, 0.6171223,\
        0.6204630, 0.6233847, 0.6257089, 0.6271994, 0.6275929, 0.6266234, 0.6240468, 0.6196881, 0.6135244,\
        0.6056858, 0.5964881, 0.5872121, 0.5797181, 0.5754822, 0.5751611, 0.5785036, 0.5845766, 0.5918602,\
        0.5939160, 0.5867136, 0.5759942, 0.5655934, 0.5656920, 0.5955831, 0.6650623, 0.7354222, 0.7941908,\
        0.8682274, 0.9268618, 0.9564394, 0.9692996, 0.9738614, 0.9736309, 0.9700057, 0.9639644, 0.9563757,\
        0.9479137, 0.9390080, 0.9298806, 0.9206256, 0.9112263, 0.9013997, 0.8910235, 0.8796227, 0.8672310,\
        0.8543118, 0.8413318, 0.8286301, 0.8163977, 0.8048582, 0.7944807, 0.7847954, 0.7752618, 0.7657967,\
        0.7563440, 0.7467954, 0.7369826, 0.7266947, 0.7157722, 0.7042943, 0.6926878, 0.6810340, 0.6692737,\
        0.6575989, 0.6462264, 0.6352909, 0.6248271, 0.6148113, 0.6051846, 0.5951712, 0.5848756, 0.5748880,\
        0.5652454, 0.5559266, 0.5469242, 0.5382332, 0.5298368, 0.5216956, 0.5137573, 0.5060215, 0.4985537,\
        0.4912897, 0.4842878, 0.4774756, 0.4707773, 0.4641497, 0.4575593, 0.4509707, 0.4443443, 0.4376386,\
        0.4308131, 0.4238323, 0.4166689, 0.4093013],\
        [0.4542232, 0.4601073, 0.4659302, 0.4717243, 0.4775273, 0.4833787, 0.4892951, 0.4951816, 0.5010658,\
        0.5075636, 0.5141795, 0.5210324, 0.5281912, 0.5356204, 0.5432489, 0.5510024, 0.5588063, 0.5665789,\
        0.5742223, 0.5816114, 0.5885754, 0.5948791, 0.6003667, 0.6052272, 0.6093313, 0.6130088, 0.6164766,\
        0.6197408, 0.6226785, 0.6250861, 0.6267115, 0.6272841, 0.6265431, 0.6242698, 0.6203302, 0.6147262,\
        0.6076450, 0.5995676, 0.5914866, 0.5851320, 0.5820209, 0.5828392, 0.5871582, 0.5936441, 0.6006902,\
        0.6046627, 0.6012224, 0.5913672, 0.5790422, 0.5690104, 0.5860409, 0.6652846, 0.7436818, 0.8035583,\
        0.8789156, 0.9370317, 0.9685733, 0.9827929, 0.9868183, 0.9849517, 0.9795537, 0.9720286, 0.9633129,\
        0.9540366, 0.9445884, 0.9351701, 0.9258237, 0.9163871, 0.9064471, 0.8954221, 0.8827786, 0.8690142,\
        0.8548572, 0.8408992, 0.8275297, 0.8148326, 0.8027287, 0.7921001, 0.7823227, 0.7727304, 0.7632482,\
        0.7538328, 0.7443870, 0.7347221, 0.7245268, 0.7134536, 0.7015350, 0.6893324, 0.6769927, 0.6646818,\
        0.6526843, 0.6412277, 0.6304239, 0.6202654, 0.6105104, 0.6010618, 0.5914815, 0.5813010, 0.5712597,\
        0.5615593, 0.5522368, 0.5432908, 0.5347144, 0.5264946, 0.5185862, 0.5108730, 0.5032383, 0.4958519,\
        0.4886892, 0.4816921, 0.4748607, 0.4681623, 0.4615581, 0.4550110, 0.4484828, 0.4419315, 0.4353114,\
        0.4285758, 0.4216832, 0.4146055, 0.4073270],\
        [0.4507731, 0.4568043, 0.4628085, 0.4688176, 0.4748644, 0.4809757, 0.4871567, 0.4933948, 0.4997546,\
        0.5060422, 0.5123776, 0.5191018, 0.5262612, 0.5337450, 0.5414507, 0.5493013, 0.5572283, 0.5651588,\
        0.5730070, 0.5806662, 0.5879946, 0.5947770, 0.6005852, 0.6048287, 0.6082283, 0.6115189, 0.6148671,\
        0.6181801, 0.6212831, 0.6239407, 0.6258783, 0.6268133, 0.6264834, 0.6246752, 0.6212574, 0.6162260,\
        0.6097678, 0.6023501, 0.5949382, 0.5890095, 0.5865239, 0.5882069, 0.5932581, 0.5998416, 0.6061658,\
        0.6101228, 0.6093631, 0.6050930, 0.5973507, 0.5809999, 0.5863733, 0.6661443, 0.7552108, 0.8257290,\
        0.8958780, 0.9477586, 0.9775205, 0.9910998, 0.9942362, 0.9911097, 0.9844175, 0.9758289, 0.9663624,\
        0.9566183, 0.9469277, 0.9374535, 0.9282205, 0.9190304, 0.9092998, 0.8978587, 0.8839496, 0.8688537,\
        0.8535551, 0.8387589, 0.8250317, 0.8126390, 0.8010829, 0.7898794, 0.7794129, 0.7694503, 0.7597921,\
        0.7503283, 0.7409657, 0.7315505, 0.7217426, 0.7108564, 0.6985030, 0.6854897, 0.6721731, 0.6590839,\
        0.6466353, 0.6350445, 0.6243380, 0.6145086, 0.6055136, 0.5958851, 0.5863031, 0.5764453, 0.5666152,\
        0.5570632, 0.5478784, 0.5390794, 0.5306717, 0.5226692, 0.5150738, 0.5077736, 0.5004077, 0.4930396,\
        0.4858599, 0.4788578, 0.4720283, 0.4653468, 0.4587791, 0.4522891, 0.4458382, 0.4393833, 0.4328743,\
        0.4262543, 0.4194684, 0.4124825, 0.4052939],\
        [0.4471139, 0.4532692, 0.4594236, 0.4656065, 0.4718460, 0.4781663, 0.4845872, 0.4911290, 0.4977243,\
        0.5038324, 0.5097985, 0.5165150, 0.5237528, 0.5312881, 0.5390242, 0.5469055, 0.5548772, 0.5628742,\
        0.5708168, 0.5786067, 0.5861153, 0.5931374, 0.5991168, 0.6028399, 0.6056871, 0.6088972, 0.6123668,\
        0.6159206, 0.6193830, 0.6224949, 0.6249562, 0.6264732, 0.6267828, 0.6256701, 0.6229911, 0.6187079,\
        0.6129519, 0.6061397, 0.5990221, 0.5925937, 0.5905647, 0.5931670, 0.5986482, 0.6054465, 0.6114571,\
        0.6156819, 0.6172318, 0.6164468, 0.6122062, 0.6023230, 0.6022361, 0.6779425, 0.7748602, 0.8544095,\
        0.9168396, 0.9593893, 0.9840062, 0.9948302, 0.9962316, 0.9918019, 0.9841319, 0.9749143, 0.9651443,\
        0.9553310, 0.9456906, 0.9363069, 0.9272153, 0.9182645, 0.9087000, 0.8969113, 0.8822070, 0.8662287,\
        0.8500840, 0.8345660, 0.8204182, 0.8084006, 0.7976433, 0.7862774, 0.7754400, 0.7651791, 0.7552963,\
        0.7456711, 0.7362457, 0.7269662, 0.7176859, 0.7076813, 0.6950415, 0.6806333, 0.6661083, 0.6521678,\
        0.6392669, 0.6276110, 0.6170508, 0.6072376, 0.5986284, 0.5899159, 0.5801989, 0.5705272, 0.5610055,\
        0.5517542, 0.5428424, 0.5342828, 0.5260773, 0.5182649, 0.5109182, 0.5040769, 0.4971839, 0.4898368,\
        0.4826612, 0.4756925, 0.4689065, 0.4622756, 0.4557695, 0.4493567, 0.4430033, 0.4366701, 0.4303062,\
        0.4238441, 0.4172027, 0.4103213, 0.4032193],\
        [0.4432362, 0.4494865, 0.4557540, 0.4620633, 0.4684343, 0.4748791, 0.4813944, 0.4879379, 0.4943398,\
        0.5001093, 0.5062712, 0.5133843, 0.5207608, 0.5282952, 0.5359870, 0.5438198, 0.5517499, 0.5597104,\
        0.5676124, 0.5753378, 0.5827149, 0.5894486, 0.5949826, 0.5986959, 0.6017288, 0.6053964, 0.6091875,\
        0.6131546, 0.6171827, 0.6209631, 0.6241640, 0.6264912, 0.6276932, 0.6275629, 0.6259472, 0.6227659,\
        0.6180429, 0.6119420, 0.6047166, 0.5980504, 0.5966932, 0.6011033, 0.6063591, 0.6126768, 0.6184021,\
        0.6231088, 0.6264218, 0.6279766, 0.6273728, 0.6257476, 0.6394329, 0.7073452, 0.8089882, 0.8858321,\
        0.9370761, 0.9692254, 0.9870592, 0.9938787, 0.9929042, 0.9870534, 0.9786364, 0.9692140, 0.9596337,\
        0.9502104, 0.9409504, 0.9317571, 0.9226564, 0.9137440, 0.9042998, 0.8921804, 0.8773424, 0.8611112,\
        0.8445540, 0.8285258, 0.8137212, 0.8010130, 0.7908885, 0.7804934, 0.7700523, 0.7598049, 0.7497373,\
        0.7398218, 0.7300659, 0.7204616, 0.7109904, 0.7014750, 0.6887179, 0.6735574, 0.6582659, 0.6437008,\
        0.6304685, 0.6189469, 0.6088302, 0.5994255, 0.5904609, 0.5819065, 0.5728691, 0.5636595, 0.5545638,\
        0.5457291, 0.5372126, 0.5289924, 0.5210241, 0.5133436, 0.5060761, 0.4992900, 0.4927362, 0.4858689,\
        0.4789428, 0.4721247, 0.4654493, 0.4589123, 0.4524970, 0.4461817, 0.4399427, 0.4337517, 0.4275681,\
        0.4213245, 0.4149091, 0.4081807, 0.4011483],\
        [0.4391411, 0.4454560, 0.4517997, 0.4581906, 0.4646386, 0.4711389, 0.4776591, 0.4841108, 0.4903165,\
        0.4962400, 0.5029155, 0.5101687, 0.5174397, 0.5248164, 0.5323656, 0.5400744, 0.5478891, 0.5557297,\
        0.5634895, 0.5710247, 0.5781320, 0.5845158, 0.5898131, 0.5939215, 0.5976264, 0.6015525, 0.6055942,\
        0.6101122, 0.6149138, 0.6195462, 0.6236629, 0.6269976, 0.6293314, 0.6304845, 0.6303193, 0.6287489,\
        0.6257483, 0.6213838, 0.6160477, 0.6116333, 0.6107472, 0.6146123, 0.6191713, 0.6237138, 0.6284207,\
        0.6337075, 0.6394868, 0.6451654, 0.6523533, 0.6665780, 0.6968333, 0.7700321, 0.8537832, 0.9122340,\
        0.9503689, 0.9733523, 0.9849932, 0.9878328, 0.9843781, 0.9771218, 0.9681925, 0.9590058, 0.9501591,\
        0.9416608, 0.9332399, 0.9244661, 0.9151196, 0.9059024, 0.8963772, 0.8845138, 0.8700828, 0.8540741,\
        0.8375143, 0.8214448, 0.8066328, 0.7939212, 0.7835332, 0.7736973, 0.7637249, 0.7535983, 0.7433388,\
        0.7329937, 0.7226559, 0.7123482, 0.7017980, 0.6910947, 0.6785779, 0.6639500, 0.6486987, 0.6338930,\
        0.6203918, 0.6091312, 0.5997309, 0.5907296, 0.5820319, 0.5735010, 0.5648552, 0.5561159, 0.5474854,\
        0.5391339, 0.5311230, 0.5233791, 0.5157426, 0.5081769, 0.5010192, 0.4943587, 0.4879306, 0.4813979,\
        0.4747731, 0.4681709, 0.4616560, 0.4552491, 0.4489485, 0.4427443, 0.4366248, 0.4305789, 0.4245896,\
        0.4186164, 0.4125583, 0.4061989, 0.3992722],\
        [0.4348411, 0.4411920, 0.4475798, 0.4540178, 0.4605081, 0.4670384, 0.4735758, 0.4800699, 0.4865022,\
        0.4930540, 0.5000069, 0.5068957, 0.5137297, 0.5208260, 0.5281763, 0.5357190, 0.5433776, 0.5510569,\
        0.5586375, 0.5659685, 0.5728592, 0.5790884, 0.5844804, 0.5890938, 0.5933155, 0.5974645, 0.6017820,\
        0.6071314, 0.6128532, 0.6184111, 0.6235290, 0.6279962, 0.6316411, 0.6343283, 0.6359652, 0.6365154,\
        0.6360227, 0.6346714, 0.6329689, 0.6319446, 0.6326079, 0.6347449, 0.6368449, 0.6386940, 0.6415915,\
        0.6475376, 0.6574253, 0.6691778, 0.6844630, 0.7133741, 0.7583310, 0.8214456, 0.8816127, 0.9246763,\
        0.9528997, 0.9697525, 0.9770843, 0.9768057, 0.9711565, 0.9626010, 0.9533865, 0.9449031, 0.9373763,\
        0.9303411, 0.9233521, 0.9157125, 0.9063496, 0.8965230, 0.8870514, 0.8754785, 0.8615952, 0.8460171,\
        0.8296509, 0.8137301, 0.7998532, 0.7874189, 0.7767644, 0.7669272, 0.7571337, 0.7470427, 0.7365310,\
        0.7256347, 0.7145435, 0.7034973, 0.6922347, 0.6803736, 0.6674075, 0.6531541, 0.6383108, 0.6236701,\
        0.6099811, 0.5986480, 0.5900038, 0.5816334, 0.5733274, 0.5650217, 0.5566443, 0.5482427, 0.5399969,\
        0.5321134, 0.5246846, 0.5175948, 0.5105393, 0.5031636, 0.4960862, 0.4895398, 0.4831320, 0.4767331,\
        0.4703102, 0.4639055, 0.4575630, 0.4513046, 0.4451336, 0.4390450, 0.4330353, 0.4271092, 0.4212782,\
        0.4155472, 0.4098802, 0.4040752, 0.3975987],\
        [0.4303585, 0.4367205, 0.4431271, 0.4495882, 0.4561024, 0.4626577, 0.4692355, 0.4758271, 0.4824729,\
        0.4892859, 0.4962712, 0.5027842, 0.5093525, 0.5162678, 0.5234478, 0.5308269, 0.5383254, 0.5458436,\
        0.5532628, 0.5604453, 0.5672400, 0.5735037, 0.5791592, 0.5842712, 0.5890428, 0.5937120, 0.5987389,\
        0.6048087, 0.6112372, 0.6176194, 0.6237153, 0.6293475, 0.6343864, 0.6387444, 0.6423832, 0.6453384,\
        0.6477549, 0.6499152, 0.6521992, 0.6548319, 0.6574076, 0.6587700, 0.6581792, 0.6568648, 0.6570305,\
        0.6627064, 0.6780908, 0.6989602, 0.7208971, 0.7549766, 0.7981263, 0.8453584, 0.8882689, 0.9218057,\
        0.9455794, 0.9594925, 0.9644133, 0.9619662, 0.9544382, 0.9445871, 0.9351957, 0.9278869, 0.9222958,\
        0.9170247, 0.9117412, 0.9063605, 0.8982048, 0.8882896, 0.8780594, 0.8664130, 0.8528847, 0.8377969,\
        0.8218501, 0.8058670, 0.7921821, 0.7806486, 0.7701496, 0.7603392, 0.7506375, 0.7405622, 0.7298066,\
        0.7183294, 0.7064321, 0.6945662, 0.6826108, 0.6699648, 0.6565053, 0.6424013, 0.6281568, 0.6142107,\
        0.6011913, 0.5901421, 0.5812207, 0.5730081, 0.5649077, 0.5567739, 0.5485667, 0.5403476, 0.5323175,\
        0.5247799, 0.5179255, 0.5115849, 0.5053839, 0.4987114, 0.4915315, 0.4847892, 0.4783068, 0.4719431,\
        0.4656401, 0.4593933, 0.4532153, 0.4471131, 0.4410821, 0.4351114, 0.4291947, 0.4233427, 0.4175870,\
        0.4119607, 0.4064736, 0.4010382, 0.3947151],\
        [0.4257241, 0.4320764, 0.4384826, 0.4449506, 0.4514765, 0.4580485, 0.4646542, 0.4712918, 0.4779770,\
        0.4847101, 0.4913606, 0.4977853, 0.5043847, 0.5112181, 0.5182657, 0.5255013, 0.5328578, 0.5402401,\
        0.5475401, 0.5546471, 0.5614560, 0.5678839, 0.5739029, 0.5795726, 0.5850453, 0.5905693, 0.5965195,\
        0.6030451, 0.6099626, 0.6170432, 0.6240341, 0.6307789, 0.6371861, 0.6432065, 0.6488352, 0.6541434,\
        0.6593275, 0.6647226, 0.6706379, 0.6768340, 0.6818484, 0.6835555, 0.6807315, 0.6775162, 0.6748782,\
        0.6778036, 0.6958203, 0.7259812, 0.7559384, 0.7888622, 0.8232363, 0.8567472, 0.8864243, 0.9124401,\
        0.9335618, 0.9459707, 0.9494859, 0.9454194, 0.9361944, 0.9248919, 0.9151873, 0.9094347, 0.9062754,\
        0.9027380, 0.8986418, 0.8949025, 0.8884404, 0.8793729, 0.8691370, 0.8575468, 0.8443301, 0.8297381,\
        0.8145661, 0.7997033, 0.7857903, 0.7739369, 0.7634597, 0.7537668, 0.7442671, 0.7343698, 0.7235759,\
        0.7116125, 0.6990916, 0.6865345, 0.6736610, 0.6602396, 0.6462905, 0.6321941, 0.6187590, 0.6060200,\
        0.5939138, 0.5831786, 0.5738052, 0.5652907, 0.5571202, 0.5490294, 0.5409030, 0.5327377, 0.5247025,\
        0.5172496, 0.5108764, 0.5051743, 0.4994904, 0.4932816, 0.4865098, 0.4798001, 0.4733062, 0.4669795,\
        0.4607729, 0.4546650, 0.4486480, 0.4427120, 0.4368381, 0.4309990, 0.4251703, 0.4193539, 0.4135968,\
        0.4079551, 0.4024078, 0.3969355, 0.3910236],\
        [0.4209754, 0.4273007, 0.4336926, 0.4401571, 0.4466854, 0.4532612, 0.4598673, 0.4664912, 0.4731189,\
        0.4797149, 0.4862152, 0.4926387, 0.4991603, 0.5058528, 0.5127640, 0.5198688, 0.5271061, 0.5343819,\
        0.5416013, 0.5486882, 0.5555843, 0.5622526, 0.5686937, 0.5749627, 0.5811769, 0.5875108, 0.5941500,\
        0.6011902, 0.6086974, 0.6164189, 0.6241934, 0.6319253, 0.6395760, 0.6471218, 0.6545480, 0.6618814,\
        0.6692554, 0.6769806, 0.6854221, 0.6942940, 0.7013300, 0.7040293, 0.7002490, 0.7000022, 0.6975723,\
        0.7010121, 0.7206202, 0.7546001, 0.7853140, 0.8124716, 0.8379789, 0.8620125, 0.8840698, 0.9044331,\
        0.9218772, 0.9327436, 0.9350494, 0.9294814, 0.9187921, 0.9064862, 0.8964682, 0.8920935, 0.8902424,\
        0.8882653, 0.8854409, 0.8820745, 0.8767664, 0.8691277, 0.8597074, 0.8486504, 0.8358684, 0.8216012,\
        0.8066848, 0.7922889, 0.7787701, 0.7667806, 0.7563174, 0.7468674, 0.7377766, 0.7283405, 0.7179226,\
        0.7061341, 0.6928806, 0.6793649, 0.6655211, 0.6513814, 0.6370148, 0.6226665, 0.6095509, 0.5993810,\
        0.5875823, 0.5767782, 0.5671073, 0.5582914, 0.5499769, 0.5418927, 0.5338598, 0.5257626, 0.5176000,\
        0.5097803, 0.5037208, 0.4986057, 0.4930949, 0.4871749, 0.4808415, 0.4743870, 0.4680158, 0.4617843,\
        0.4556921, 0.4497287, 0.4438816, 0.4381319, 0.4324474, 0.4267798, 0.4210717, 0.4152886, 0.4094925,\
        0.4038443, 0.3983580, 0.3928920, 0.3871039],\
        [0.4161559, 0.4224399, 0.4288089, 0.4352659, 0.4417923, 0.4483619, 0.4549491, 0.4615312, 0.4680838,\
        0.4745778, 0.4809944, 0.4873673, 0.4937825, 0.5003313, 0.5070890, 0.5140647, 0.5212029, 0.5283888,\
        0.5355398, 0.5426262, 0.5496393, 0.5565775, 0.5634555, 0.5703183, 0.5772422, 0.5843192, 0.5916301,\
        0.5992624, 0.6072235, 0.6154737, 0.6238610, 0.6323790, 0.6410723, 0.6499280, 0.6588770, 0.6678187,\
        0.6766657, 0.6854886, 0.6947474, 0.7042536, 0.7119384, 0.7161168, 0.7176343, 0.7205769, 0.7250061,\
        0.7357900, 0.7579762, 0.7869604, 0.8095883, 0.8269463, 0.8443411, 0.8620980, 0.8797801, 0.8965808,\
        0.9118352, 0.9217765, 0.9229123, 0.9157150, 0.9036460, 0.8916336, 0.8823383, 0.8774096, 0.8753107,\
        0.8739222, 0.8720662, 0.8692035, 0.8646484, 0.8580912, 0.8496916, 0.8395365, 0.8274091, 0.8133059,\
        0.7983111, 0.7837706, 0.7703211, 0.7584225, 0.7481723, 0.7391564, 0.7306868, 0.7219619, 0.7120493,\
        0.7004235, 0.6869640, 0.6724417, 0.6577860, 0.6432110, 0.6288610, 0.6148371, 0.6011108, 0.5909197,\
        0.5806772, 0.5702703, 0.5605792, 0.5516440, 0.5432760, 0.5352738, 0.5274711, 0.5196937, 0.5117391,\
        0.5036632, 0.4970367, 0.4921642, 0.4867532, 0.4809553, 0.4748631, 0.4686388, 0.4624367, 0.4563428,\
        0.4503904, 0.4445866, 0.4389255, 0.4333885, 0.4279384, 0.4225130, 0.4170194, 0.4113477, 0.4054870,\
        0.3998551, 0.3944823, 0.3890401, 0.3832838],\
        [0.4113154, 0.4175450, 0.4238896, 0.4303442, 0.4368709, 0.4434302, 0.4499905, 0.4565250, 0.4630103,\
        0.4694279, 0.4757759, 0.4820826, 0.4884045, 0.4948129, 0.5013941, 0.5082261, 0.5152876, 0.5223672,\
        0.5294148, 0.5364777, 0.5435994, 0.5507981, 0.5580888, 0.5655015, 0.5730804, 0.5808616, 0.5888336,\
        0.5969169, 0.6053113, 0.6139920, 0.6226918, 0.6317190, 0.6412156, 0.6511391, 0.6613417, 0.6715843,\
        0.6814918, 0.6905942, 0.6990626, 0.7083318, 0.7165603, 0.7232556, 0.7296589, 0.7379025, 0.7495591,\
        0.7663237, 0.7874743, 0.8072852, 0.8202790, 0.8303788, 0.8419815, 0.8561569, 0.8722064, 0.8884912,\
        0.9034111, 0.9132734, 0.9130738, 0.9040986, 0.8909166, 0.8785256, 0.8692829, 0.8638842, 0.8614153,\
        0.8602106, 0.8588726, 0.8564805, 0.8524436, 0.8465566, 0.8389882, 0.8299139, 0.8188866, 0.8048896,\
        0.7894240, 0.7742417, 0.7603706, 0.7484703, 0.7385793, 0.7301657, 0.7224080, 0.7145240, 0.7053891,\
        0.6933830, 0.6796148, 0.6648340, 0.6498224, 0.6351620, 0.6212346, 0.6082995, 0.5964927, 0.5844463,\
        0.5736572, 0.5634644, 0.5539156, 0.5450411, 0.5367456, 0.5289137, 0.5214687, 0.5143233, 0.5071541,\
        0.4996434, 0.4925589, 0.4865054, 0.4807134, 0.4748120, 0.4687785, 0.4626862, 0.4566290, 0.4506793,\
        0.4448788, 0.4392455, 0.4337825, 0.4284784, 0.4233025, 0.4181948, 0.4130568, 0.4077257, 0.4020124,\
        0.3963013, 0.3909034, 0.3853961, 0.3796060],\
        [0.4065102, 0.4126702, 0.4190035, 0.4254734, 0.4320051, 0.4385511, 0.4450803, 0.4515680, 0.4579944,\
        0.4643486, 0.4706351, 0.4768787, 0.4831226, 0.4894214, 0.4958342, 0.5025487, 0.5095180, 0.5163935,\
        0.5232426, 0.5302273, 0.5374284, 0.5448532, 0.5524989, 0.5603798, 0.5685220, 0.5769453, 0.5856371,\
        0.5945190, 0.6034110, 0.6117259, 0.6202537, 0.6295283, 0.6396170, 0.6503612, 0.6615403, 0.6728681,\
        0.6838863, 0.6937273, 0.7015864, 0.7091640, 0.7169040, 0.7252200, 0.7350758, 0.7477100, 0.7640305,\
        0.7837009, 0.8015593, 0.8108776, 0.8161371, 0.8218335, 0.8303746, 0.8434720, 0.8603032, 0.8781316,\
        0.8946542, 0.9042512, 0.9030910, 0.8915270, 0.8774000, 0.8645622, 0.8552935, 0.8499885, 0.8477138,\
        0.8468946, 0.8460143, 0.8439909, 0.8402194, 0.8345290, 0.8272329, 0.8189103, 0.8094965, 0.7962769,\
        0.7794490, 0.7631065, 0.7485801, 0.7366753, 0.7273144, 0.7196542, 0.7125391, 0.7049490, 0.6966036,\
        0.6848126, 0.6710252, 0.6562861, 0.6412134, 0.6265758, 0.6132014, 0.6005586, 0.5887482, 0.5772443,\
        0.5663682, 0.5562976, 0.5469615, 0.5382739, 0.5301421, 0.5224970, 0.5153304, 0.5087890, 0.5026015,\
        0.4954257, 0.4879869, 0.4812002, 0.4748763, 0.4687302, 0.4626412, 0.4566014, 0.4506499, 0.4448325,\
        0.4391835, 0.4337223, 0.4284571, 0.4233876, 0.4184964, 0.4137312, 0.4090126, 0.4042216, 0.3989487,\
        0.3932121, 0.3875630, 0.3818970, 0.3760272],\
        [0.4018026, 0.4078720, 0.4142513, 0.4207567, 0.4272844, 0.4338052, 0.4402965, 0.4467359, 0.4531064,\
        0.4594014, 0.4656287, 0.4718123, 0.4779930, 0.4842279, 0.4906010, 0.4972287, 0.5039550, 0.5104445,\
        0.5169770, 0.5238417, 0.5311062, 0.5387104, 0.5466239, 0.5548495, 0.5633975, 0.5722727, 0.5814689,\
        0.5909116, 0.5998220, 0.6077222, 0.6159979, 0.6255055, 0.6360643, 0.6473542, 0.6591209, 0.6711000,\
        0.6829245, 0.6939762, 0.7015082, 0.7061938, 0.7127738, 0.7214909, 0.7330039, 0.7476085, 0.7649981,\
        0.7834851, 0.7971403, 0.7999103, 0.8005466, 0.8041404, 0.8108306, 0.8245968, 0.8435997, 0.8625468,\
        0.8786222, 0.8881285, 0.8867263, 0.8747792, 0.8603487, 0.8478022, 0.8392658, 0.8349209, 0.8336310,\
        0.8336915, 0.8334297, 0.8317188, 0.8280349, 0.8221312, 0.8142858, 0.8056482, 0.7961995, 0.7834365,\
        0.7668463, 0.7498909, 0.7347988, 0.7230596, 0.7145093, 0.7076922, 0.7013298, 0.6936884, 0.6849679,\
        0.6740769, 0.6611341, 0.6469455, 0.6320865, 0.6171620, 0.6040881, 0.5923850, 0.5803253, 0.5689597,\
        0.5583499, 0.5485920, 0.5396052, 0.5312192, 0.5233107, 0.5158608, 0.5087509, 0.5022363, 0.4963526,\
        0.4896317, 0.4824424, 0.4754875, 0.4688965, 0.4625709, 0.4564185, 0.4504049, 0.4445380, 0.4388427,\
        0.4333423, 0.4280498, 0.4229709, 0.4181137, 0.4134841, 0.4090280, 0.4045999, 0.4001453, 0.3952320,\
        0.3897403, 0.3841044, 0.3783755, 0.3724576],\
        [0.3972279, 0.4033136, 0.4098234, 0.4163102, 0.4227859, 0.4292585, 0.4357016, 0.4420873, 0.4483984,\
        0.4546305, 0.4607926, 0.4669081, 0.4730158, 0.4791705, 0.4854397, 0.4918556, 0.4982401, 0.5043808,\
        0.5105273, 0.5173301, 0.5246797, 0.5323947, 0.5404580, 0.5488775, 0.5576330, 0.5666717, 0.5758881,\
        0.5850309, 0.5936468, 0.6014365, 0.6097856, 0.6197747, 0.6306776, 0.6421518, 0.6539816, 0.6658910,\
        0.6773714, 0.6873836, 0.6940055, 0.6985528, 0.7042668, 0.7123785, 0.7240167, 0.7383335, 0.7538638,\
        0.7681214, 0.7770703, 0.7791098, 0.7788945, 0.7810105, 0.7875402, 0.8030881, 0.8235291, 0.8419715,\
        0.8570836, 0.8652738, 0.8635104, 0.8530238, 0.8394819, 0.8280794, 0.8212001, 0.8186829, 0.8190548,\
        0.8204766, 0.8209773, 0.8194616, 0.8158369, 0.8098236, 0.8010705, 0.7915869, 0.7811618, 0.7681020,\
        0.7522631, 0.7353587, 0.7196940, 0.7082424, 0.7008903, 0.6947405, 0.6886417, 0.6814364, 0.6727784,\
        0.6624404, 0.6504574, 0.6371980, 0.6230869, 0.6086301, 0.5944834, 0.5827602, 0.5710399, 0.5598264,\
        0.5495279, 0.5402393, 0.5317861, 0.5238372, 0.5161708, 0.5089167, 0.5023273, 0.4954019, 0.4892386,\
        0.4827737, 0.4759654, 0.4691668, 0.4625755, 0.4562188, 0.4500676, 0.4441012, 0.4383229, 0.4327515,\
        0.4274046, 0.4222845, 0.4173809, 0.4126967, 0.4082727, 0.4041306, 0.3999223, 0.3955746, 0.3909241,\
        0.3858009, 0.3803667, 0.3747125, 0.3688200],\
        [0.3933065, 0.3995433, 0.4058808, 0.4121686, 0.4185426, 0.4249555, 0.4313428, 0.4376664, 0.4439090,\
        0.4500676, 0.4561507, 0.4621780, 0.4681801, 0.4741944, 0.4802501, 0.4863269, 0.4923187, 0.4980698,\
        0.5039116, 0.5109161, 0.5183087, 0.5259845, 0.5340528, 0.5425256, 0.5513256, 0.5603251, 0.5693436,\
        0.5781201, 0.5863631, 0.5941019, 0.6028774, 0.6130503, 0.6239277, 0.6350937, 0.6464656, 0.6577472,\
        0.6682672, 0.6770664, 0.6834338, 0.6881879, 0.6930755, 0.6999141, 0.7105258, 0.7229669, 0.7354592,\
        0.7458157, 0.7517174, 0.7530774, 0.7529327, 0.7549495, 0.7628814, 0.7795033, 0.7998738, 0.8176038,\
        0.8333060, 0.8412328, 0.8396377, 0.8297448, 0.8168852, 0.8071819, 0.8025533, 0.8021818, 0.8042148,\
        0.8071378, 0.8083218, 0.8067084, 0.8029479, 0.7973454, 0.7883809, 0.7780856, 0.7665875, 0.7531975,\
        0.7379625, 0.7216783, 0.7058079, 0.6943318, 0.6876767, 0.6818918, 0.6758675, 0.6688320, 0.6603966,\
        0.6505205, 0.6394061, 0.6272466, 0.6139663, 0.6004618, 0.5866812, 0.5734067, 0.5611702, 0.5499048,\
        0.5398704, 0.5311662, 0.5234779, 0.5161667, 0.5087800, 0.5014537, 0.4949990, 0.4884804, 0.4819750,\
        0.4755034, 0.4689280, 0.4623545, 0.4559099, 0.4496508, 0.4435847, 0.4377107, 0.4320418, 0.4266059,\
        0.4214290, 0.4165061, 0.4117929, 0.4072632, 0.4029463, 0.3990296, 0.3951378, 0.3909503, 0.3864735,\
        0.3816180, 0.3764100, 0.3708975, 0.3650869],\
        [0.3897578, 0.3959696, 0.4020757, 0.4082258, 0.4145602, 0.4209321, 0.4272577, 0.4335055, 0.4396655,\
        0.4457363, 0.4517236, 0.4576405, 0.4635066, 0.4693420, 0.4751536, 0.4809174, 0.4865783, 0.4921437,\
        0.4981651, 0.5050977, 0.5121396, 0.5195353, 0.5274874, 0.5359303, 0.5446918, 0.5535845, 0.5624159,\
        0.5709978, 0.5792408, 0.5874085, 0.5961800, 0.6060870, 0.6163697, 0.6266288, 0.6372001, 0.6477220,\
        0.6573740, 0.6654175, 0.6715964, 0.6764632, 0.6811844, 0.6872154, 0.6955117, 0.7048765, 0.7142025,\
        0.7215026, 0.7250940, 0.7252346, 0.7250542, 0.7266976, 0.7341512, 0.7511635, 0.7721318, 0.7925887,\
        0.8102617, 0.8182294, 0.8181496, 0.8087180, 0.7966411, 0.7890641, 0.7860731, 0.7866780, 0.7894748,\
        0.7927961, 0.7943084, 0.7928272, 0.7889904, 0.7832325, 0.7748962, 0.7645753, 0.7526694, 0.7392295,\
        0.7248313, 0.7103708, 0.6958064, 0.6843197, 0.6764408, 0.6700996, 0.6637950, 0.6566499, 0.6482100,\
        0.6384858, 0.6279348, 0.6170032, 0.6049140, 0.5909203, 0.5770761, 0.5634500, 0.5507159, 0.5392243,\
        0.5293226, 0.5212557, 0.5145946, 0.5082017, 0.5011580, 0.4940122, 0.4872780, 0.4808066, 0.4743635,\
        0.4679534, 0.4615578, 0.4552145, 0.4489879, 0.4429180, 0.4370155, 0.4312851, 0.4257507, 0.4204611,\
        0.4154724, 0.4107979, 0.4063358, 0.4020018, 0.3978949, 0.3940606, 0.3903024, 0.3863123, 0.3820112,\
        0.3773560, 0.3723401, 0.3669771, 0.3612711],\
        [0.3863087, 0.3923750, 0.3983479, 0.4045475, 0.4109127, 0.4172372, 0.4234753, 0.4296241, 0.4356842,\
        0.4416531, 0.4475305, 0.4533215, 0.4590369, 0.4646887, 0.4702836, 0.4758237, 0.4813356, 0.4869654,\
        0.4930511, 0.4995806, 0.5059864, 0.5130272, 0.5208675, 0.5292767, 0.5379734, 0.5467524, 0.5554528,\
        0.5639692, 0.5723228, 0.5807506, 0.5896506, 0.5992485, 0.6081523, 0.6170161, 0.6268424, 0.6368225,\
        0.6458220, 0.6533065, 0.6592616, 0.6641350, 0.6687179, 0.6738779, 0.6799915, 0.6866013, 0.6929577,\
        0.6976883, 0.6996057, 0.6982398, 0.6980168, 0.6989475, 0.7044019, 0.7231027, 0.7455037, 0.7678345,\
        0.7857686, 0.7960596, 0.7991692, 0.7910579, 0.7812374, 0.7744389, 0.7716481, 0.7721263, 0.7745763,\
        0.7773301, 0.7786681, 0.7776066, 0.7741907, 0.7686341, 0.7608532, 0.7509967, 0.7392911, 0.7260177,\
        0.7119497, 0.6993685, 0.6866875, 0.6752249, 0.6664532, 0.6593409, 0.6525943, 0.6452106, 0.6365503,\
        0.6264957, 0.6157571, 0.6053294, 0.5946585, 0.5808698, 0.5665379, 0.5526545, 0.5396579, 0.5279400,\
        0.5179527, 0.5103848, 0.5049016, 0.4993987, 0.4928677, 0.4859355, 0.4792445, 0.4727818, 0.4664355,\
        0.4601634, 0.4539660, 0.4478705, 0.4419119, 0.4361061, 0.4304425, 0.4249102, 0.4195354, 0.4143927,\
        0.4095838, 0.4051963, 0.4011077, 0.3970021, 0.3931033, 0.3893676, 0.3856523, 0.3817694, 0.3776044,\
        0.3731014, 0.3682370, 0.3630051, 0.3574015],\
        [0.3830800, 0.3891721, 0.3952283, 0.4014181, 0.4077089, 0.4139011, 0.4199997, 0.4260223, 0.4319688,\
        0.4378273, 0.4435865, 0.4492439, 0.4548062, 0.4602858, 0.4656986, 0.4710712, 0.4764624, 0.4819882,\
        0.4877627, 0.4936989, 0.4997837, 0.5065504, 0.5144192, 0.5227933, 0.5313797, 0.5400182, 0.5485872,\
        0.5570250, 0.5653712, 0.5737748, 0.5823867, 0.5910059, 0.5989136, 0.6064423, 0.6162913, 0.6259405,\
        0.6342673, 0.6411197, 0.6467130, 0.6514026, 0.6556410, 0.6598795, 0.6643207, 0.6687814, 0.6727379,\
        0.6753465, 0.6757733, 0.6743569, 0.6734386, 0.6745763, 0.6817564, 0.6987686, 0.7201701, 0.7422867,\
        0.7612196, 0.7738047, 0.7777876, 0.7733158, 0.7660894, 0.7603331, 0.7576058, 0.7577326, 0.7596145,\
        0.7617591, 0.7627897, 0.7618880, 0.7588657, 0.7537782, 0.7466457, 0.7375119, 0.7264198, 0.7136244,\
        0.7000220, 0.6871946, 0.6757430, 0.6652052, 0.6563856, 0.6489546, 0.6419834, 0.6345144, 0.6257315,\
        0.6151472, 0.6033860, 0.5927153, 0.5816478, 0.5688450, 0.5549992, 0.5412481, 0.5283056, 0.5166164,\
        0.5064748, 0.4989460, 0.4941860, 0.4891947, 0.4833827, 0.4770834, 0.4707253, 0.4644550, 0.4582806,\
        0.4521998, 0.4462318, 0.4404113, 0.4347698, 0.4293060, 0.4239694, 0.4187012, 0.4135177, 0.4085279,\
        0.4038751, 0.3997071, 0.3959446, 0.3921792, 0.3884762, 0.3848432, 0.3811818, 0.3773661, 0.3732939,\
        0.3689006, 0.3641499, 0.3590225, 0.3535062],\
        [0.3800988, 0.3863265, 0.3925308, 0.3987657, 0.4048814, 0.4108667, 0.4167917, 0.4226779, 0.4285124,\
        0.4342616, 0.4398998, 0.4454196, 0.4508295, 0.4561473, 0.4613984, 0.4666193, 0.4718643, 0.4772016,\
        0.4826738, 0.4882769, 0.4941041, 0.5007804, 0.5085375, 0.5166885, 0.5250433, 0.5334708, 0.5418471,\
        0.5501083, 0.5582695, 0.5663899, 0.5744666, 0.5822936, 0.5895950, 0.5974295, 0.6069346, 0.6156536,\
        0.6229440, 0.6288915, 0.6339687, 0.6383347, 0.6420985, 0.6454473, 0.6485121, 0.6512719, 0.6534993,\
        0.6546779, 0.6543274, 0.6528318, 0.6516821, 0.6531876, 0.6605994, 0.6756203, 0.6959330, 0.7179560,\
        0.7383835, 0.7516957, 0.7564169, 0.7544484, 0.7496011, 0.7451679, 0.7428862, 0.7430297, 0.7446840,\
        0.7464512, 0.7471703, 0.7461967, 0.7433834, 0.7387613, 0.7322919, 0.7240076, 0.7138351, 0.7015313,\
        0.6884154, 0.6757180, 0.6642380, 0.6541606, 0.6456428, 0.6383352, 0.6314497, 0.6241606, 0.6157166,\
        0.6050774, 0.5928438, 0.5809804, 0.5690998, 0.5564014, 0.5430438, 0.5296595, 0.5170549, 0.5059306,\
        0.4963374, 0.4889760, 0.4836994, 0.4788174, 0.4735229, 0.4678103, 0.4618931, 0.4559293, 0.4499975,\
        0.4441477, 0.4384309, 0.4329046, 0.4276212, 0.4225887, 0.4177029, 0.4127886, 0.4078389, 0.4030404,\
        0.3986183, 0.3946659, 0.3910432, 0.3875057, 0.3839844, 0.3804567, 0.3768571, 0.3730960, 0.3690893,\
        0.3647730, 0.3601040, 0.3550544, 0.3496055],\
        [0.3772850, 0.3836665, 0.3899756, 0.3961873, 0.4021892, 0.4080215, 0.4137860, 0.4195606, 0.4253085,\
        0.4309579, 0.4364710, 0.4418455, 0.4470992, 0.4522590, 0.4573575, 0.4624333, 0.4675331, 0.4727071,\
        0.4780034, 0.4834926, 0.4893646, 0.4959682, 0.5032781, 0.5109665, 0.5189772, 0.5271249, 0.5352242,\
        0.5431894, 0.5510198, 0.5587450, 0.5663634, 0.5738418, 0.5813289, 0.5894052, 0.5977768, 0.6057424,\
        0.6115246, 0.6163412, 0.6209195, 0.6249588, 0.6281923, 0.6306967, 0.6325733, 0.6339399, 0.6349933,\
        0.6354669, 0.6347460, 0.6331256, 0.6315782, 0.6323398, 0.6386388, 0.6525390, 0.6729626, 0.6965119,\
        0.7188727, 0.7314773, 0.7360310, 0.7351629, 0.7320455, 0.7288831, 0.7272921, 0.7279963, 0.7299624,\
        0.7316034, 0.7320110, 0.7307291, 0.7278626, 0.7236129, 0.7176382, 0.7099441, 0.7007797, 0.6890166,\
        0.6761553, 0.6635937, 0.6520777, 0.6421594, 0.6340358, 0.6271104, 0.6204393, 0.6132560, 0.6051686,\
        0.5950719, 0.5823684, 0.5696559, 0.5570529, 0.5442653, 0.5313557, 0.5185065, 0.5061354, 0.4958743,\
        0.4870684, 0.4798829, 0.4741479, 0.4690464, 0.4639290, 0.4585827, 0.4530378, 0.4473864, 0.4417165,\
        0.4361096, 0.4306466, 0.4254092, 0.4204888, 0.4159673, 0.4117301, 0.4072985, 0.4026040, 0.3980718,\
        0.3938839, 0.3900566, 0.3864922, 0.3830520, 0.3796399, 0.3761952, 0.3726530, 0.3689393, 0.3649818,\
        0.3607200, 0.3561077, 0.3511123, 0.3457103],\
        [0.3745509, 0.3810856, 0.3874807, 0.3936845, 0.3996378, 0.4053202, 0.4109258, 0.4166659, 0.4223722,\
        0.4279228, 0.4332921, 0.4385011, 0.4435859, 0.4485846, 0.4535345, 0.4584742, 0.4634464, 0.4685002,\
        0.4736977, 0.4791343, 0.4849624, 0.4913287, 0.4981892, 0.5054381, 0.5131229, 0.5209614, 0.5286919,\
        0.5362392, 0.5436214, 0.5508892, 0.5580892, 0.5652824, 0.5726120, 0.5802181, 0.5879013, 0.5948475,\
        0.5992008, 0.6032494, 0.6075064, 0.6112756, 0.6138790, 0.6156090, 0.6164755, 0.6165397, 0.6167157,\
        0.6171566, 0.6160788, 0.6143869, 0.6122262, 0.6112859, 0.6156692, 0.6288639, 0.6507974, 0.6769611,\
        0.6991500, 0.7115402, 0.7159923, 0.7157275, 0.7143297, 0.7123536, 0.7113286, 0.7131197, 0.7158708,\
        0.7173179, 0.7174170, 0.7155711, 0.7122275, 0.7083753, 0.7026715, 0.6951150, 0.6861026, 0.6751958,\
        0.6630344, 0.6507460, 0.6391522, 0.6291502, 0.6215499, 0.6151880, 0.6085631, 0.6012216, 0.5924888,\
        0.5823271, 0.5703443, 0.5577462, 0.5450672, 0.5323821, 0.5200521, 0.5085610, 0.4967982, 0.4867136,\
        0.4783090, 0.4712243, 0.4652553, 0.4599439, 0.4548383, 0.4496835, 0.4444016, 0.4390111, 0.4335779,\
        0.4281984, 0.4229776, 0.4180056, 0.4133852, 0.4093284, 0.4060350, 0.4023505, 0.3978441, 0.3935175,\
        0.3894563, 0.3856913, 0.3821629, 0.3787712, 0.3754218, 0.3720366, 0.3685436, 0.3648717, 0.3609537,\
        0.3567312, 0.3521574, 0.3471966, 0.3418228],\
        [0.3718299, 0.3785390, 0.3850512, 0.3912927, 0.3972019, 0.4026007, 0.4082134, 0.4140854, 0.4197586,\
        0.4251625, 0.4303362, 0.4353418, 0.4402367, 0.4450669, 0.4498704, 0.4546823, 0.4595416, 0.4644961,\
        0.4696089, 0.4749646, 0.4806595, 0.4867545, 0.4932193, 0.5000529, 0.5075135, 0.5149505, 0.5221928,\
        0.5291901, 0.5360078, 0.5427356, 0.5494479, 0.5562039, 0.5630469, 0.5699296, 0.5765489, 0.5822013,\
        0.5864000, 0.5900507, 0.5937860, 0.5969958, 0.5988778, 0.5999517, 0.6000860, 0.5990180, 0.5982824,\
        0.5988931, 0.5970400, 0.5948703, 0.5924222, 0.5901796, 0.5929525, 0.6057224, 0.6277808, 0.6531365,\
        0.6753057, 0.6902802, 0.6958278, 0.6971131, 0.6977328, 0.6971350, 0.6967252, 0.6995005, 0.7027871,\
        0.7032743, 0.7032199, 0.7008964, 0.6967193, 0.6928529, 0.6873655, 0.6800070, 0.6711740, 0.6608695,\
        0.6494610, 0.6376467, 0.6260385, 0.6153620, 0.6083475, 0.6028646, 0.5956468, 0.5877535, 0.5786397,\
        0.5683694, 0.5570370, 0.5451367, 0.5330097, 0.5207003, 0.5085822, 0.4982243, 0.4881493, 0.4783078,\
        0.4699565, 0.4628876, 0.4568106, 0.4513809, 0.4462637, 0.4412213, 0.4361289, 0.4309462, 0.4257058,\
        0.4205182, 0.4155307, 0.4108317, 0.4064590, 0.4025914, 0.4000521, 0.3975347, 0.3932361, 0.3890146,\
        0.3850633, 0.3813786, 0.3779075, 0.3745684, 0.3712739, 0.3679420, 0.3644962, 0.3608649, 0.3569821,\
        0.3527902, 0.3482419, 0.3433003, 0.3379383],\
        [0.3690607, 0.3759901, 0.3826932, 0.3890449, 0.3948478, 0.4000769, 0.4061700, 0.4120488, 0.4175170,\
        0.4226431, 0.4275353, 0.4322889, 0.4369722, 0.4416295, 0.4462903, 0.4509809, 0.4557323, 0.4605864,\
        0.4655987, 0.4708366, 0.4763666, 0.4822330, 0.4884586, 0.4951172, 0.5020417, 0.5089670, 0.5156043,\
        0.5219140, 0.5280589, 0.5341567, 0.5402876, 0.5464690, 0.5526531, 0.5587086, 0.5643558, 0.5692050,\
        0.5731132, 0.5764175, 0.5793830, 0.5817182, 0.5830608, 0.5834714, 0.5828998, 0.5814745, 0.5801591,\
        0.5791425, 0.5769023, 0.5738737, 0.5708293, 0.5693765, 0.5731808, 0.5854546, 0.6054948, 0.6298892,\
        0.6521378, 0.6681652, 0.6768714, 0.6808972, 0.6829729, 0.6838474, 0.6847507, 0.6870185, 0.6891193,\
        0.6893156, 0.6885574, 0.6860528, 0.6819606, 0.6774486, 0.6718937, 0.6648581, 0.6564164, 0.6467087,\
        0.6360167, 0.6248706, 0.6139263, 0.6031069, 0.5949964, 0.5896191, 0.5819363, 0.5733997, 0.5640277,\
        0.5538533, 0.5430805, 0.5320764, 0.5210367, 0.5096066, 0.4980653, 0.4877378, 0.4784758, 0.4696014,\
        0.4616362, 0.4547106, 0.4486595, 0.4432218, 0.4381358, 0.4331991, 0.4282785, 0.4232866, 0.4182000,\
        0.4131455, 0.4083775, 0.4040130, 0.3999864, 0.3964574, 0.3938531, 0.3913575, 0.3879402, 0.3841716,\
        0.3804774, 0.3769585, 0.3736020, 0.3703507, 0.3671306, 0.3638635, 0.3604733, 0.3568879, 0.3530415,\
        0.3488764, 0.3443454, 0.3394113, 0.3340469],\
        [0.3661626, 0.3733668, 0.3803761, 0.3870852, 0.3934173, 0.3995102, 0.4051636, 0.4105438, 0.4155363,\
        0.4202280, 0.4247566, 0.4292239, 0.4336897, 0.4381817, 0.4427109, 0.4472869, 0.4519284, 0.4566682,\
        0.4615521, 0.4666347, 0.4719701, 0.4776021, 0.4835618, 0.4898424, 0.4963225, 0.5028104, 0.5086911,\
        0.5142176, 0.5196393, 0.5250446, 0.5305455, 0.5360983, 0.5415706, 0.5468281, 0.5516829, 0.5558944,\
        0.5593486, 0.5621402, 0.5643683, 0.5659449, 0.5667039, 0.5665514, 0.5654673, 0.5636726, 0.5616367,\
        0.5594243, 0.5565654, 0.5530628, 0.5497932, 0.5490183, 0.5539647, 0.5665705, 0.5861839, 0.6097196,\
        0.6317373, 0.6486462, 0.6595848, 0.6659748, 0.6696055, 0.6716666, 0.6731466, 0.6745972, 0.6754818,\
        0.6751539, 0.6736485, 0.6708700, 0.6669707, 0.6623133, 0.6567570, 0.6500302, 0.6420698, 0.6330156,\
        0.6230459, 0.6124513, 0.6022637, 0.5926569, 0.5835284, 0.5758709, 0.5677015, 0.5587465, 0.5491825,\
        0.5391547, 0.5288497, 0.5186263, 0.5090289, 0.4989079, 0.4881240, 0.4781023, 0.4690420, 0.4607281,\
        0.4532203, 0.4465754, 0.4406891, 0.4353563, 0.4303642, 0.4255564, 0.4208336, 0.4160823, 0.4111542,\
        0.4061551, 0.4015276, 0.3975182, 0.3938367, 0.3905977, 0.3878685, 0.3852382, 0.3822484, 0.3789652,\
        0.3756231, 0.3723487, 0.3691674, 0.3660489, 0.3629349, 0.3597554, 0.3564374, 0.3529094, 0.3491056,\
        0.3449682, 0.3404498, 0.3355141, 0.3301352],\
        [0.3630237, 0.3705260, 0.3779053, 0.3850861, 0.3919740, 0.3983878, 0.4041257, 0.4090672, 0.4134706,\
        0.4176604, 0.4218065, 0.4259984, 0.4302741, 0.4346315, 0.4390520, 0.4435231, 0.4480491, 0.4526532,\
        0.4573732, 0.4622548, 0.4673435, 0.4726753, 0.4782665, 0.4840919, 0.4900581, 0.4958935, 0.5010185,\
        0.5059348, 0.5106597, 0.5153314, 0.5202354, 0.5251945, 0.5299438, 0.5344410, 0.5386344, 0.5422864,\
        0.5452109, 0.5474012, 0.5489400, 0.5498777, 0.5501571, 0.5496057, 0.5481102, 0.5458467, 0.5431956,\
        0.5403911, 0.5373302, 0.5337644, 0.5301207, 0.5291068, 0.5347500, 0.5483907, 0.5685758, 0.5920240,\
        0.6143528, 0.6317655, 0.6439819, 0.6520373, 0.6570096, 0.6598963, 0.6615106, 0.6622851, 0.6621792,\
        0.6610338, 0.6588670, 0.6557817, 0.6519322, 0.6474069, 0.6420810, 0.6357686, 0.6283122, 0.6198833,\
        0.6108465, 0.6006274, 0.5905569, 0.5810017, 0.5716682, 0.5627248, 0.5536918, 0.5442768, 0.5345480,\
        0.5246987, 0.5148569, 0.5050197, 0.4962066, 0.4874510, 0.4777674, 0.4683844, 0.4597152, 0.4518204,\
        0.4447293, 0.4384377, 0.4328326, 0.4277089, 0.4228640, 0.4181887, 0.4136751, 0.4092929, 0.4046471,\
        0.3996922, 0.3952208, 0.3913363, 0.3878562, 0.3847487, 0.3819652, 0.3792922, 0.3765040, 0.3735603,\
        0.3705456, 0.3675394, 0.3645702, 0.3616208, 0.3586448, 0.3555794, 0.3523546, 0.3488999, 0.3451490,\
        0.3410435, 0.3365360, 0.3315914, 0.3261869],\
        [0.3595170, 0.3672804, 0.3749873, 0.3825744, 0.3899337, 0.3968263, 0.4026771, 0.4070166, 0.4108588,\
        0.4146371, 0.4184779, 0.4224658, 0.4266218, 0.4309024, 0.4352502, 0.4396285, 0.4440301, 0.4484721,\
        0.4529874, 0.4576158, 0.4623943, 0.4673473, 0.4724741, 0.4777299, 0.4829955, 0.4880350, 0.4926542,\
        0.4970933, 0.5010338, 0.5050351, 0.5094609, 0.5139316, 0.5178760, 0.5215911, 0.5252687, 0.5284507,\
        0.5308271, 0.5323770, 0.5332359, 0.5336225, 0.5336263, 0.5329155, 0.5311059, 0.5283778, 0.5252553,\
        0.5222704, 0.5196488, 0.5168338, 0.5127652, 0.5101342, 0.5161560, 0.5308240, 0.5517320, 0.5755407,\
        0.5984623, 0.6161122, 0.6293988, 0.6388223, 0.6448536, 0.6482926, 0.6498938, 0.6501172, 0.6491738,\
        0.6471940, 0.6443560, 0.6409171, 0.6371021, 0.6328765, 0.6279461, 0.6222154, 0.6154856, 0.6070540,\
        0.5991512, 0.5894797, 0.5793163, 0.5694060, 0.5596609, 0.5499970, 0.5402641, 0.5304074, 0.5205265,\
        0.5108458, 0.5016914, 0.4925736, 0.4840150, 0.4757277, 0.4670955, 0.4584927, 0.4503407, 0.4428607,\
        0.4361655, 0.4302712, 0.4250513, 0.4202408, 0.4155808, 0.4110013, 0.4065817, 0.4025639, 0.3984693,\
        0.3936823, 0.3892878, 0.3854166, 0.3819845, 0.3789094, 0.3761037, 0.3734333, 0.3707685, 0.3680541,\
        0.3653045, 0.3625504, 0.3598032, 0.3570448, 0.3542321, 0.3513053, 0.3481957, 0.3448324, 0.3411478,\
        0.3370814, 0.3325850, 0.3276252, 0.3221840],\
        [0.3555360, 0.3634706, 0.3713785, 0.3791874, 0.3867730, 0.3938817, 0.3996448, 0.4036355, 0.4073290,\
        0.4109536, 0.4146225, 0.4185219, 0.4226702, 0.4269547, 0.4312707, 0.4355644, 0.4398265, 0.4440751,\
        0.4483429, 0.4526666, 0.4570767, 0.4615872, 0.4661839, 0.4708129, 0.4753690, 0.4797140, 0.4837701,\
        0.4875463, 0.4910061, 0.4945009, 0.4984238, 0.5025261, 0.5053489, 0.5082850, 0.5116785, 0.5144523,\
        0.5162686, 0.5171852, 0.5173611, 0.5172060, 0.5171937, 0.5166971, 0.5146599, 0.5115469, 0.5081210,\
        0.5051327, 0.5031864, 0.5020046, 0.4985080, 0.4949634, 0.5004555, 0.5151661, 0.5360740, 0.5592220,\
        0.5815464, 0.6003529, 0.6152975, 0.6261134, 0.6329558, 0.6367680, 0.6383667, 0.6381681, 0.6365224,\
        0.6337845, 0.6302653, 0.6263965, 0.6226400, 0.6189187, 0.6144851, 0.6090804, 0.6041786, 0.5959843,\
        0.5872113, 0.5779639, 0.5680032, 0.5579391, 0.5479050, 0.5378267, 0.5276248, 0.5174463, 0.5075838,\
        0.4980641, 0.4893763, 0.4809709, 0.4727860, 0.4647648, 0.4566999, 0.4486872, 0.4409923, 0.4338706,\
        0.4275219, 0.4220356, 0.4172999, 0.4129376, 0.4084896, 0.4039830, 0.3996085, 0.3956042, 0.3917126,\
        0.3874552, 0.3833031, 0.3795183, 0.3761196, 0.3730530, 0.3702413, 0.3675913, 0.3650174, 0.3624684,\
        0.3599289, 0.3573977, 0.3548665, 0.3523081, 0.3496761, 0.3469086, 0.3439349, 0.3406818, 0.3370788,\
        0.3330619, 0.3285791, 0.3235977, 0.3181073],\
        [0.3510284, 0.3590242, 0.3669825, 0.3747966, 0.3822783, 0.3890724, 0.3946292, 0.3989920, 0.4029611,\
        0.4066413, 0.4101848, 0.4141425, 0.4184315, 0.4228019, 0.4271113, 0.4313126, 0.4354092, 0.4394284,\
        0.4434069, 0.4473816, 0.4513800, 0.4554097, 0.4594506, 0.4634513, 0.4673346, 0.4710208, 0.4744652,\
        0.4776697, 0.4807056, 0.4837831, 0.4870629, 0.4902459, 0.4924862, 0.4949392, 0.4978948, 0.5001685,\
        0.5014319, 0.5018391, 0.5014769, 0.5007742, 0.5008778, 0.5009068, 0.4986444, 0.4954708, 0.4922016,\
        0.4894835, 0.4878699, 0.4867740, 0.4847797, 0.4840801, 0.4895613, 0.5033083, 0.5222505, 0.5437876,\
        0.5654730, 0.5851681, 0.6016677, 0.6137836, 0.6210407, 0.6252064, 0.6270402, 0.6265734, 0.6243340,\
        0.6210097, 0.6169050, 0.6124370, 0.6087255, 0.6055925, 0.6021782, 0.5964115, 0.5916169, 0.5847435,\
        0.5758060, 0.5663993, 0.5565853, 0.5465796, 0.5365435, 0.5264133, 0.5160251, 0.5055946, 0.4959776,\
        0.4869087, 0.4783875, 0.4702671, 0.4623652, 0.4545905, 0.4468659, 0.4392290, 0.4318420, 0.4249463,\
        0.4188242, 0.4136802, 0.4094641, 0.4057455, 0.4015262, 0.3970704, 0.3928241, 0.3888136, 0.3849333,\
        0.3809999, 0.3771335, 0.3735037, 0.3701739, 0.3671322, 0.3643316, 0.3617108, 0.3592112, 0.3567897,\
        0.3544201, 0.3520835, 0.3497558, 0.3473990, 0.3449588, 0.3423668, 0.3395469, 0.3364219, 0.3329174,\
        0.3289643, 0.3245019, 0.3194927, 0.3139374],\
        [0.3460087, 0.3539715, 0.3618695, 0.3695673, 0.3768551, 0.3834445, 0.3890851, 0.3938617, 0.3981501,\
        0.4018417, 0.4051645, 0.4094533, 0.4140242, 0.4185077, 0.4227889, 0.4268615, 0.4307546, 0.4345063,\
        0.4381578, 0.4417494, 0.4453107, 0.4488475, 0.4523368, 0.4557321, 0.4589783, 0.4620312, 0.4648755,\
        0.4675346, 0.4700807, 0.4726176, 0.4751651, 0.4775216, 0.4795213, 0.4815411, 0.4836469, 0.4852836,\
        0.4861223, 0.4862009, 0.4856835, 0.4850358, 0.4848375, 0.4844564, 0.4827008, 0.4800466, 0.4773153,\
        0.4750548, 0.4735560, 0.4726185, 0.4722122, 0.4739227, 0.4803752, 0.4926889, 0.5094626, 0.5293807,\
        0.5504224, 0.5704346, 0.5879265, 0.6010352, 0.6084409, 0.6135288, 0.6162196, 0.6155250, 0.6126805,\
        0.6090330, 0.6049355, 0.5996125, 0.5960251, 0.5929507, 0.5895541, 0.5845829, 0.5789584, 0.5721523,\
        0.5638639, 0.5547415, 0.5451585, 0.5353353, 0.5255561, 0.5158595, 0.5057054, 0.4954074, 0.4857941,\
        0.4768914, 0.4684886, 0.4604834, 0.4527406, 0.4451568, 0.4376715, 0.4302862, 0.4230852, 0.4162682,\
        0.4101956, 0.4052676, 0.4013859, 0.3981656, 0.3943782, 0.3901011, 0.3859714, 0.3820302, 0.3782194,\
        0.3744736, 0.3708240, 0.3673520, 0.3641115, 0.3611132, 0.3583374, 0.3557498, 0.3533130, 0.3509937,\
        0.3487643, 0.3465977, 0.3444596, 0.3423021, 0.3400607, 0.3376567, 0.3350046, 0.3320213, 0.3286312,\
        0.3247628, 0.3203398, 0.3152996, 0.3096561],\
        [0.3405474, 0.3484185, 0.3562081, 0.3637719, 0.3709196, 0.3774471, 0.3832239, 0.3882863, 0.3927410,\
        0.3963932, 0.4001345, 0.4048354, 0.4096332, 0.4141296, 0.4182980, 0.4221823, 0.4258305, 0.4292815,\
        0.4325764, 0.4357623, 0.4388784, 0.4419331, 0.4448974, 0.4477219, 0.4503643, 0.4528068, 0.4550603,\
        0.4571609, 0.4591615, 0.4611069, 0.4629871, 0.4647264, 0.4663017, 0.4678066, 0.4691996, 0.4702208,\
        0.4706659, 0.4705372, 0.4700177, 0.4694329, 0.4689833, 0.4683446, 0.4670329, 0.4651371, 0.4631172,\
        0.4614002, 0.4602610, 0.4598651, 0.4606594, 0.4637854, 0.4706630, 0.4819070, 0.4970676, 0.5158682,\
        0.5361991, 0.5558414, 0.5731921, 0.5865657, 0.5953677, 0.6023152, 0.6064754, 0.6050780, 0.6016681,\
        0.5978726, 0.5938976, 0.5891771, 0.5849230, 0.5812435, 0.5772865, 0.5723990, 0.5665537, 0.5596736,\
        0.5517350, 0.5430353, 0.5338412, 0.5242499, 0.5146291, 0.5060299, 0.4963298, 0.4862570, 0.4766595,\
        0.4677307, 0.4593672, 0.4514355, 0.4438057, 0.4363780, 0.4290981, 0.4219397, 0.4149059, 0.4081253,\
        0.4019788, 0.3971958, 0.3934607, 0.3901837, 0.3866844, 0.3828492, 0.3789699, 0.3751828, 0.3715002,\
        0.3679088, 0.3644239, 0.3610862, 0.3579335, 0.3549840, 0.3522374, 0.3496807, 0.3472945, 0.3450566,\
        0.3429428, 0.3409238, 0.3389605, 0.3369977, 0.3349597, 0.3327532, 0.3302775, 0.3274406, 0.3241707,\
        0.3204108, 0.3160765, 0.3110256, 0.3052517],\
        [0.3347507, 0.3425060, 0.3501857, 0.3576473, 0.3647238, 0.3712609, 0.3771661, 0.3824345, 0.3871092,\
        0.3913315, 0.3956688, 0.4005333, 0.4052586, 0.4096020, 0.4135584, 0.4172048, 0.4205864, 0.4237217,\
        0.4266413, 0.4294074, 0.4320857, 0.4346929, 0.4371805, 0.4394785, 0.4415481, 0.4433976, 0.4450665,\
        0.4466029, 0.4480459, 0.4494136, 0.4506976, 0.4518792, 0.4529609, 0.4539507, 0.4547815, 0.4553052,\
        0.4554056, 0.4550961, 0.4545329, 0.4539416, 0.4534356, 0.4528756, 0.4520267, 0.4508564, 0.4495756,\
        0.4484867, 0.4478821, 0.4480946, 0.4496849, 0.4535408, 0.4606040, 0.4713857, 0.4858560, 0.5036091,\
        0.5228604, 0.5417252, 0.5587725, 0.5727084, 0.5833343, 0.5916054, 0.5964819, 0.5950252, 0.5915793,\
        0.5876898, 0.5834968, 0.5789695, 0.5744910, 0.5701871, 0.5656590, 0.5604798, 0.5544491, 0.5475056,\
        0.5397309, 0.5314117, 0.5227958, 0.5137061, 0.5038772, 0.4953297, 0.4867893, 0.4771604, 0.4677987,\
        0.4589837, 0.4507165, 0.4428954, 0.4353914, 0.4281086, 0.4210281, 0.4141403, 0.4073615, 0.4007256,\
        0.3947334, 0.3898666, 0.3859537, 0.3824902, 0.3790568, 0.3754942, 0.3718722, 0.3682801, 0.3647560,\
        0.3613122, 0.3579638, 0.3547364, 0.3516577, 0.3487498, 0.3460258, 0.3434900, 0.3411389, 0.3389614,\
        0.3369389, 0.3350442, 0.3332381, 0.3314624, 0.3296311, 0.3276304, 0.3253350, 0.3226349, 0.3194614,\
        0.3158026, 0.3116457, 0.3067412, 0.3007484],\
        [0.3287424, 0.3363828, 0.3439767, 0.3513800, 0.3584372, 0.3650183, 0.3710492, 0.3765267, 0.3815246,\
        0.3862358, 0.3909714, 0.3958411, 0.4005308, 0.4046847, 0.4084034, 0.4118188, 0.4149584, 0.4177931,\
        0.4203293, 0.4226617, 0.4249220, 0.4271470, 0.4292348, 0.4310593, 0.4325815, 0.4338495, 0.4349441,\
        0.4359323, 0.4368444, 0.4376771, 0.4384180, 0.4390743, 0.4396710, 0.4402031, 0.4405975, 0.4407419,\
        0.4405568, 0.4400588, 0.4393909, 0.4387710, 0.4383483, 0.4380839, 0.4377850, 0.4373108, 0.4367336,\
        0.4362861, 0.4362484, 0.4369713, 0.4390039, 0.4431384, 0.4502180, 0.4607906, 0.4748871, 0.4918715,\
        0.5100808, 0.5283469, 0.5453300, 0.5597113, 0.5711148, 0.5795225, 0.5841015, 0.5841841, 0.5815304,\
        0.5777741, 0.5734812, 0.5688874, 0.5641981, 0.5594625, 0.5544822, 0.5489792, 0.5427588, 0.5357356,\
        0.5279635, 0.5198278, 0.5118521, 0.5038088, 0.4945501, 0.4854349, 0.4767720, 0.4677900, 0.4588611,\
        0.4503208, 0.4422670, 0.4346475, 0.4273261, 0.4201750, 0.4132351, 0.4066526, 0.4001794, 0.3937700,\
        0.3879223, 0.3829607, 0.3788002, 0.3751334, 0.3716622, 0.3682228, 0.3647845, 0.3613709, 0.3580050,\
        0.3547006, 0.3514704, 0.3483324, 0.3453096, 0.3424271, 0.3397098, 0.3371777, 0.3348419, 0.3327019,\
        0.3307449, 0.3289468, 0.3272728, 0.3256702, 0.3240474, 0.3222622, 0.3201497, 0.3175688, 0.3144285,\
        0.3107564, 0.3067197, 0.3023629, 0.2964106],\
        [0.3226565, 0.3301987, 0.3377432, 0.3451292, 0.3521976, 0.3588295, 0.3649654, 0.3706132, 0.3758518,\
        0.3808277, 0.3856980, 0.3904899, 0.3950308, 0.3990857, 0.4026541, 0.4059129, 0.4088851, 0.4114683,\
        0.4136209, 0.4154903, 0.4173596, 0.4193168, 0.4211209, 0.4225245, 0.4235103, 0.4242025, 0.4247422,\
        0.4252234, 0.4256646, 0.4260288, 0.4262789, 0.4264418, 0.4265957, 0.4267568, 0.4268333, 0.4267072,\
        0.4263060, 0.4256239, 0.4247993, 0.4241215, 0.4238421, 0.4240124, 0.4243564, 0.4245392, 0.4245724,\
        0.4247136, 0.4252013, 0.4262903, 0.4284464, 0.4324636, 0.4393122, 0.4496608, 0.4634699, 0.4799008,\
        0.4976048, 0.5159300, 0.5330074, 0.5474016, 0.5588127, 0.5670649, 0.5717339, 0.5727554, 0.5710138,\
        0.5676461, 0.5634099, 0.5587368, 0.5538659, 0.5488537, 0.5435806, 0.5378444, 0.5314948, 0.5244739,\
        0.5166354, 0.5082766, 0.5005671, 0.4930820, 0.4845969, 0.4757370, 0.4670193, 0.4583439, 0.4497964,\
        0.4415800, 0.4338202, 0.4265043, 0.4194744, 0.4124836, 0.4054719, 0.3990970, 0.3928523, 0.3867290,\
        0.3810801, 0.3761150, 0.3718140, 0.3680020, 0.3644755, 0.3610885, 0.3577708, 0.3545011, 0.3512767,\
        0.3480989, 0.3449720, 0.3419062, 0.3389196, 0.3360410, 0.3333074, 0.3307561, 0.3284129, 0.3262864,\
        0.3243680, 0.3226331, 0.3210521, 0.3195938, 0.3181782, 0.3166211, 0.3146908, 0.3122238, 0.3090735,\
        0.3051990, 0.3009583, 0.2964838, 0.2908724],\
        [0.3166384, 0.3241118, 0.3316498, 0.3390483, 0.3461314, 0.3527879, 0.3589721, 0.3646990, 0.3700351,\
        0.3750793, 0.3799142, 0.3845431, 0.3888715, 0.3927847, 0.3962700, 0.3994370, 0.4023042, 0.4047064,\
        0.4065016, 0.4078600, 0.4093637, 0.4112468, 0.4129179, 0.4139181, 0.4143592, 0.4144869, 0.4145084,\
        0.4145508, 0.4146111, 0.4146021, 0.4144266, 0.4141248, 0.4138901, 0.4137883, 0.4136584, 0.4133530,\
        0.4128259, 0.4120098, 0.4109941, 0.4102326, 0.4100689, 0.4107445, 0.4117980, 0.4125118, 0.4129647,\
        0.4136059, 0.4145742, 0.4159162, 0.4179249, 0.4214128, 0.4277372, 0.4378763, 0.4517253, 0.4683240,\
        0.4864844, 0.5050445, 0.5218554, 0.5357545, 0.5468470, 0.5550222, 0.5599068, 0.5614880, 0.5603303,\
        0.5572827, 0.5531137, 0.5483718, 0.5433735, 0.5382152, 0.5328107, 0.5269676, 0.5205268, 0.5136790,\
        0.5062516, 0.4975374, 0.4897180, 0.4821641, 0.4741434, 0.4657486, 0.4572866, 0.4488790, 0.4406265,\
        0.4326902, 0.4252253, 0.4182689, 0.4116585, 0.4050389, 0.3980520, 0.3914767, 0.3853785, 0.3795191,\
        0.3740763, 0.3691855, 0.3648526, 0.3609786, 0.3574263, 0.3540757, 0.3508468, 0.3476943, 0.3445938,\
        0.3415307, 0.3384966, 0.3354905, 0.3325231, 0.3296242, 0.3268459, 0.3242493, 0.3218759, 0.3197403,\
        0.3178374, 0.3161325, 0.3145856, 0.3132090, 0.3119894, 0.3106797, 0.3088985, 0.3065118, 0.3034831,\
        0.2993316, 0.2948582, 0.2900153, 0.2843434],\
        [0.3108582, 0.3183077, 0.3258849, 0.3332989, 0.3403612, 0.3469783, 0.3531216, 0.3588107, 0.3641006,\
        0.3690607, 0.3737431, 0.3781535, 0.3822492, 0.3859811, 0.3893466, 0.3923865, 0.3950874, 0.3973280,\
        0.3988762, 0.3998488, 0.4010195, 0.4030357, 0.4046630, 0.4051838, 0.4051089, 0.4047389, 0.4043161,\
        0.4040116, 0.4037792, 0.4035060, 0.4029998, 0.4023000, 0.4017469, 0.4014856, 0.4012215, 0.4007871,\
        0.4002094, 0.3993774, 0.3982597, 0.3975228, 0.3974530, 0.3984829, 0.4000403, 0.4010291, 0.4016845,\
        0.4027281, 0.4041032, 0.4056398, 0.4073932, 0.4099560, 0.4155071, 0.4255644, 0.4398201, 0.4569687,\
        0.4757806, 0.4947428, 0.5113501, 0.5243557, 0.5353198, 0.5437290, 0.5488958, 0.5507684, 0.5498287,\
        0.5468440, 0.5426019, 0.5377363, 0.5326373, 0.5274302, 0.5220350, 0.5162580, 0.5097385, 0.5026450,\
        0.4960583, 0.4877437, 0.4795341, 0.4716756, 0.4637552, 0.4556751, 0.4475309, 0.4394095, 0.4313942,\
        0.4236418, 0.4163837, 0.4097327, 0.4035162, 0.3972163, 0.3905246, 0.3839751, 0.3778753, 0.3721750,\
        0.3669081, 0.3621314, 0.3578444, 0.3539830, 0.3504499, 0.3471482, 0.3440010, 0.3409532, 0.3379663,\
        0.3350114, 0.3320667, 0.3291171, 0.3261596, 0.3232172, 0.3203602, 0.3176922, 0.3152734, 0.3131067,\
        0.3112058, 0.3095188, 0.3079391, 0.3065146, 0.3054534, 0.3043974, 0.3027217, 0.3001564, 0.2972023,\
        0.2931119, 0.2883863, 0.2832123, 0.2773699],\
        [0.3055433, 0.3130451, 0.3206807, 0.3280472, 0.3349975, 0.3414745, 0.3474656, 0.3529937, 0.3581067,\
        0.3628610, 0.3673008, 0.3714422, 0.3752746, 0.3787798, 0.3819527, 0.3847921, 0.3872611, 0.3892632,\
        0.3906970, 0.3917311, 0.3929288, 0.3946015, 0.3958948, 0.3960911, 0.3957114, 0.3950491, 0.3943463,\
        0.3937637, 0.3932642, 0.3927336, 0.3920232, 0.3911807, 0.3904783, 0.3900147, 0.3895987, 0.3890888,\
        0.3884610, 0.3876657, 0.3867886, 0.3862396, 0.3863530, 0.3872876, 0.3886438, 0.3897530, 0.3906746,\
        0.3918588, 0.3933175, 0.3948783, 0.3964791, 0.3982859, 0.4031729, 0.4133554, 0.4281197, 0.4455217,\
        0.4644733, 0.4835094, 0.4996917, 0.5124693, 0.5244885, 0.5335501, 0.5389804, 0.5408624, 0.5397610,\
        0.5365039, 0.5319534, 0.5268392, 0.5216219, 0.5164125, 0.5110792, 0.5054438, 0.4993057, 0.4919542,\
        0.4848934, 0.4772446, 0.4692607, 0.4613479, 0.4534873, 0.4456370, 0.4378120, 0.4300076, 0.4222140,\
        0.4145367, 0.4073350, 0.4008800, 0.3948302, 0.3887242, 0.3824132, 0.3761617, 0.3702229, 0.3646851,\
        0.3595863, 0.3549493, 0.3507619, 0.3469717, 0.3435018, 0.3402717, 0.3372121, 0.3342669, 0.3313905,\
        0.3285438, 0.3256931, 0.3228097, 0.3198710, 0.3168732, 0.3138914, 0.3111259, 0.3086784, 0.3064510,\
        0.3045350, 0.3028931, 0.3012681, 0.2996711, 0.2986387, 0.2976054, 0.2959294, 0.2932968, 0.2900756,\
        0.2860543, 0.2813063, 0.2759663, 0.2699946],\
        [0.3010981, 0.3087595, 0.3162796, 0.3234109, 0.3301070, 0.3363227, 0.3420444, 0.3472926, 0.3521126,\
        0.3565588, 0.3606784, 0.3644998, 0.3680309, 0.3712669, 0.3741975, 0.3768038, 0.3790464, 0.3808721,\
        0.3822760, 0.3834091, 0.3845432, 0.3857232, 0.3865313, 0.3866324, 0.3861807, 0.3854354, 0.3846083,\
        0.3838277, 0.3831009, 0.3823616, 0.3815482, 0.3807021, 0.3799532, 0.3793524, 0.3788207, 0.3782648,\
        0.3776428, 0.3769593, 0.3763272, 0.3759874, 0.3761571, 0.3768762, 0.3779103, 0.3789480, 0.3799544,\
        0.3810838, 0.3823762, 0.3837775, 0.3853503, 0.3877262, 0.3927528, 0.4029884, 0.4172632, 0.4341575,\
        0.4526996, 0.4711707, 0.4874616, 0.5022435, 0.5153818, 0.5249437, 0.5303960, 0.5319344, 0.5302772,\
        0.5264042, 0.5212875, 0.5157561, 0.5103829, 0.5051546, 0.4998566, 0.4941642, 0.4880019, 0.4811105,\
        0.4739239, 0.4664674, 0.4587637, 0.4509905, 0.4432540, 0.4356261, 0.4281517, 0.4207514, 0.4132566,\
        0.4056818, 0.3983887, 0.3920642, 0.3860549, 0.3800772, 0.3740467, 0.3680899, 0.3623819, 0.3570396,\
        0.3521224, 0.3476497, 0.3436025, 0.3399289, 0.3365587, 0.3334216, 0.3304577, 0.3276167, 0.3248511,\
        0.3221148, 0.3193654, 0.3165672, 0.3136850, 0.3106659, 0.3075054, 0.3045799, 0.3021929, 0.2999151,\
        0.2979252, 0.2962607, 0.2946158, 0.2930458, 0.2917258, 0.2903655, 0.2884752, 0.2858357, 0.2824896,\
        0.2784267, 0.2736644, 0.2682563, 0.2622129],\
        [0.2986822, 0.3057836, 0.3126649, 0.3193574, 0.3256789, 0.3315286, 0.3368763, 0.3417391, 0.3461645,\
        0.3502136, 0.3539441, 0.3573972, 0.3605925, 0.3635307, 0.3661986, 0.3685717, 0.3706179, 0.3723135,\
        0.3736777, 0.3748006, 0.3757874, 0.3766160, 0.3770977, 0.3770837, 0.3766233, 0.3758788, 0.3750131,\
        0.3741353, 0.3732817, 0.3724393, 0.3715946, 0.3707761, 0.3700399, 0.3694076, 0.3688437, 0.3682953,\
        0.3677356, 0.3671858, 0.3667359, 0.3665284, 0.3666815, 0.3672040, 0.3679747, 0.3688445, 0.3697676,\
        0.3707917, 0.3719707, 0.3733853, 0.3753110, 0.3785034, 0.3843402, 0.3940341, 0.4070899, 0.4231347,\
        0.4413762, 0.4601237, 0.4779579, 0.4943412, 0.5081315, 0.5179448, 0.5231259, 0.5239683, 0.5213994,\
        0.5166458, 0.5108237, 0.5047069, 0.4991680, 0.4938261, 0.4883853, 0.4826355, 0.4764748, 0.4698525,\
        0.4628815, 0.4556485, 0.4482005, 0.4406211, 0.4330262, 0.4255881, 0.4184803, 0.4116321, 0.4045337,\
        0.3972257, 0.3901027, 0.3835916, 0.3774882, 0.3715386, 0.3656622, 0.3599160, 0.3544165, 0.3492696,\
        0.3445420, 0.3402538, 0.3363797, 0.3328588, 0.3296160, 0.3265849, 0.3237186, 0.3209798, 0.3183237,\
        0.3156961, 0.3130475, 0.3103416, 0.3075562, 0.3046438, 0.3013836, 0.2982060, 0.2958068, 0.2935440,\
        0.2914810, 0.2896545, 0.2879250, 0.2862624, 0.2846546, 0.2829128, 0.2807470, 0.2779682, 0.2745196,\
        0.2703899, 0.2655875, 0.2601353, 0.2540492],\
        [0.2961581, 0.3028569, 0.3093899, 0.3157191, 0.3216413, 0.3270639, 0.3319585, 0.3363479, 0.3402910,\
        0.3438652, 0.3471461, 0.3501900, 0.3530252, 0.3556533, 0.3580572, 0.3602096, 0.3620828, 0.3636611,\
        0.3649562, 0.3660074, 0.3668470, 0.3674479, 0.3677252, 0.3676179, 0.3671550, 0.3664374, 0.3655835,\
        0.3646867, 0.3637996, 0.3629425, 0.3621273, 0.3613738, 0.3607048, 0.3601242, 0.3596096, 0.3591300,\
        0.3586711, 0.3582532, 0.3579368, 0.3578049, 0.3579213, 0.3582909, 0.3588580, 0.3595546, 0.3603587,\
        0.3613078, 0.3624916, 0.3640867, 0.3664572, 0.3702576, 0.3763489, 0.3853401, 0.3973201, 0.4127027,\
        0.4310241, 0.4503302, 0.4693138, 0.4869396, 0.5017827, 0.5119750, 0.5167530, 0.5166932, 0.5130085,\
        0.5072396, 0.5008881, 0.4944109, 0.4884120, 0.4827103, 0.4769809, 0.4710860, 0.4649427, 0.4585142,\
        0.4518140, 0.4448716, 0.4377142, 0.4303658, 0.4228870, 0.4155268, 0.4086404, 0.4024176, 0.3957902,\
        0.3887277, 0.3818375, 0.3753190, 0.3691423, 0.3631819, 0.3573771, 0.3517583, 0.3464110, 0.3414299,\
        0.3368834, 0.3327918, 0.3291187, 0.3257821, 0.3226861, 0.3197612, 0.3169801, 0.3143329, 0.3117802,\
        0.3092505, 0.3066865, 0.3040474, 0.3013212, 0.2985121, 0.2953861, 0.2922558, 0.2895852, 0.2872144,\
        0.2850445, 0.2830452, 0.2811465, 0.2792848, 0.2773833, 0.2752963, 0.2728358, 0.2698519, 0.2662637,\
        0.2620391, 0.2571720, 0.2516706, 0.2455465],\
        [0.2932543, 0.2998519, 0.3063454, 0.3123884, 0.3179204, 0.3228870, 0.3272745, 0.3311203, 0.3345054,\
        0.3375358, 0.3403156, 0.3429206, 0.3453852, 0.3477060, 0.3498569, 0.3518052, 0.3535215, 0.3549868,\
        0.3561975, 0.3571613, 0.3578808, 0.3583354, 0.3584888, 0.3583223, 0.3578637, 0.3571837, 0.3563713,\
        0.3555079, 0.3546542, 0.3538493, 0.3531177, 0.3524755, 0.3519284, 0.3514676, 0.3510705, 0.3507133,\
        0.3503851, 0.3500987, 0.3498915, 0.3498120, 0.3498984, 0.3501603, 0.3505811, 0.3511430, 0.3518580,\
        0.3527879, 0.3540590, 0.3558931, 0.3586505, 0.3628424, 0.3690336, 0.3776526, 0.3889895, 0.4038627,\
        0.4219112, 0.4412807, 0.4607582, 0.4791664, 0.4950271, 0.5057690, 0.5104330, 0.5094833, 0.5049759,\
        0.4982417, 0.4915093, 0.4847200, 0.4781636, 0.4718892, 0.4657541, 0.4596481, 0.4534989, 0.4472477,\
        0.4408347, 0.4342185, 0.4274101, 0.4203997, 0.4130677, 0.4056643, 0.3988993, 0.3928112, 0.3866414,\
        0.3800672, 0.3734893, 0.3671091, 0.3609639, 0.3550176, 0.3492508, 0.3436987, 0.3384432, 0.3335815,\
        0.3291916, 0.3252989, 0.3218524, 0.3187325, 0.3157981, 0.3129643, 0.3102360, 0.3076541, 0.3051972,\
        0.3027360, 0.3002283, 0.2976146, 0.2949055, 0.2920835, 0.2891054, 0.2861204, 0.2833608, 0.2808491,\
        0.2785278, 0.2763429, 0.2742321, 0.2721231, 0.2699258, 0.2675197, 0.2647681, 0.2615522, 0.2577917,\
        0.2534425, 0.2484862, 0.2429201, 0.2367495],\
        [0.2906100, 0.2973155, 0.3035560, 0.3092941, 0.3144525, 0.3189551, 0.3228026, 0.3260500, 0.3288098,\
        0.3312336, 0.3334694, 0.3356203, 0.3377219, 0.3397537, 0.3416719, 0.3434344, 0.3450083, 0.3463678,\
        0.3474951, 0.3483785, 0.3490080, 0.3493702, 0.3494524, 0.3492573, 0.3488149, 0.3481813, 0.3474287,\
        0.3466312, 0.3458544, 0.3451493, 0.3445488, 0.3440659, 0.3436946, 0.3434126, 0.3431902, 0.3430004,\
        0.3428293, 0.3426815, 0.3425788, 0.3425516, 0.3426278, 0.3428247, 0.3431524, 0.3436289, 0.3442994,\
        0.3452549, 0.3466488, 0.3487158, 0.3517853, 0.3562674, 0.3625966, 0.3711740, 0.3824162, 0.3967246,\
        0.4137519, 0.4325154, 0.4517387, 0.4700911, 0.4859878, 0.4975430, 0.5031263, 0.5012143, 0.4964000,\
        0.4897276, 0.4824052, 0.4751186, 0.4680275, 0.4612144, 0.4546627, 0.4483297, 0.4421718, 0.4361187,\
        0.4300308, 0.4237441, 0.4172825, 0.4107894, 0.4038878, 0.3965026, 0.3898377, 0.3836805, 0.3776159,\
        0.3714097, 0.3651576, 0.3589894, 0.3529587, 0.3470707, 0.3413367, 0.3358106, 0.3305907, 0.3257930,\
        0.3215184, 0.3178144, 0.3146143, 0.3117467, 0.3089917, 0.3062189, 0.3035078, 0.3009340, 0.2985517,\
        0.2961204, 0.2936203, 0.2910260, 0.2883291, 0.2855278, 0.2826370, 0.2797442, 0.2769651, 0.2743460,\
        0.2718691, 0.2694902, 0.2671511, 0.2647806, 0.2622926, 0.2595856, 0.2565521, 0.2530955, 0.2491423,\
        0.2446450, 0.2395772, 0.2339281, 0.2276969],\
        [0.2879377, 0.2946307, 0.3007767, 0.3063517, 0.3111745, 0.3152211, 0.3185210, 0.3211287, 0.3231997,\
        0.3249559, 0.3266137, 0.3283148, 0.3300856, 0.3318647, 0.3335756, 0.3351695, 0.3366151, 0.3378812,\
        0.3389362, 0.3397547, 0.3403207, 0.3406264, 0.3406727, 0.3404739, 0.3400617, 0.3394844, 0.3388035,\
        0.3380894, 0.3374143, 0.3368412, 0.3364110, 0.3361342, 0.3359909, 0.3359415, 0.3359423, 0.3359578,\
        0.3359684, 0.3359717, 0.3359807, 0.3360176, 0.3361080, 0.3362778, 0.3365574, 0.3369914, 0.3376524,\
        0.3386560, 0.3401724, 0.3424364, 0.3457473, 0.3504516, 0.3569110, 0.3654723, 0.3764411, 0.3899663,\
        0.4059242, 0.4237178, 0.4421061, 0.4597301, 0.4750531, 0.4864832, 0.4922577, 0.4912453, 0.4866081,\
        0.4801916, 0.4728449, 0.4652575, 0.4577662, 0.4505334, 0.4436334, 0.4371017, 0.4309486, 0.4251395,\
        0.4194861, 0.4135355, 0.4072300, 0.4011140, 0.3948723, 0.3878857, 0.3812251, 0.3749960, 0.3689655,\
        0.3629679, 0.3569822, 0.3510470, 0.3451854, 0.3394003, 0.3337090, 0.3281811, 0.3229483, 0.3181596,\
        0.3139376, 0.3103922, 0.3074393, 0.3048268, 0.3022945, 0.2995305, 0.2968477, 0.2942854, 0.2918554,\
        0.2894198, 0.2869159, 0.2843240, 0.2816362, 0.2788578, 0.2760142, 0.2731606, 0.2703610, 0.2676506,\
        0.2650256, 0.2624542, 0.2598857, 0.2572557, 0.2544873, 0.2514947, 0.2481898, 0.2444918, 0.2403355,\
        0.2356736, 0.2304761, 0.2247267, 0.2184192],\
        [0.2850934, 0.2919008, 0.2981023, 0.3035588, 0.3079807, 0.3116240, 0.3144150, 0.3163500, 0.3176675,\
        0.3186956, 0.3197492, 0.3210328, 0.3225431, 0.3241243, 0.3256482, 0.3270805, 0.3284101, 0.3295994,\
        0.3305983, 0.3313685, 0.3318911, 0.3321644, 0.3321989, 0.3320163, 0.3316498, 0.3311410, 0.3305406,\
        0.3299165, 0.3293525, 0.3289293, 0.3287007, 0.3286746, 0.3288107, 0.3290432, 0.3293082, 0.3295597,\
        0.3297726, 0.3299410, 0.3300738, 0.3301907, 0.3303190, 0.3304933, 0.3307587, 0.3311795, 0.3318489,\
        0.3328999, 0.3345132, 0.3369199, 0.3403956, 0.3452421, 0.3517601, 0.3602163, 0.3707997, 0.3835617,\
        0.3983795, 0.4149297, 0.4321146, 0.4486688, 0.4629759, 0.4735154, 0.4790080, 0.4792030, 0.4755599,\
        0.4696768, 0.4625809, 0.4549720, 0.4472933, 0.4398054, 0.4326640, 0.4259858, 0.4198422, 0.4142566,\
        0.4092178, 0.4038101, 0.3975203, 0.3915165, 0.3854694, 0.3790786, 0.3727034, 0.3665629, 0.3606211,\
        0.3547953, 0.3490450, 0.3433580, 0.3377154, 0.3320872, 0.3264664, 0.3209207, 0.3156348, 0.3108391,\
        0.3065939, 0.3031381, 0.3004141, 0.2979221, 0.2955016, 0.2928643, 0.2902225, 0.2876629, 0.2851737,\
        0.2826902, 0.2801569, 0.2775473, 0.2748526, 0.2720787, 0.2692468, 0.2663920, 0.2635518, 0.2607480,\
        0.2579787, 0.2552193, 0.2524272, 0.2495463, 0.2465103, 0.2432463, 0.2396806, 0.2357446, 0.2313805,\
        0.2265437, 0.2212026, 0.2153379, 0.2089393],\
        [0.2819122, 0.2888883, 0.2952727, 0.3005816, 0.3045755, 0.3081454, 0.3104878, 0.3116948, 0.3122028,\
        0.3124576, 0.3128875, 0.3138236, 0.3152036, 0.3166531, 0.3179749, 0.3192285, 0.3204541, 0.3215884,\
        0.3225488, 0.3232832, 0.3237744, 0.3240307, 0.3240705, 0.3239205, 0.3236177, 0.3231965, 0.3226869,\
        0.3221505, 0.3216931, 0.3214234, 0.3214189, 0.3216883, 0.3221558, 0.3227136, 0.3232737, 0.3237816,\
        0.3242109, 0.3245554, 0.3248236, 0.3250359, 0.3252228, 0.3254254, 0.3256996, 0.3261225, 0.3268010,\
        0.3278796, 0.3295451, 0.3320258, 0.3355808, 0.3404803, 0.3469751, 0.3552605, 0.3654352, 0.3774706,\
        0.3912071, 0.4062202, 0.4222631, 0.4377376, 0.4509039, 0.4605026, 0.4657243, 0.4665270, 0.4637374,\
        0.4585016, 0.4518014, 0.4443632, 0.4366955, 0.4291350, 0.4218809, 0.4151243, 0.4090467, 0.4035166,\
        0.3988842, 0.3941342, 0.3882350, 0.3822858, 0.3763456, 0.3702942, 0.3642311, 0.3582892, 0.3525074,\
        0.3468715, 0.3413621, 0.3359552, 0.3305982, 0.3252081, 0.3197229, 0.3141734, 0.3087682, 0.3040325,\
        0.2998703, 0.2963733, 0.2936614, 0.2911745, 0.2887206, 0.2861804, 0.2835961, 0.2810313, 0.2784923,\
        0.2759485, 0.2733639, 0.2707145, 0.2679906, 0.2651951, 0.2623420, 0.2594527, 0.2565484, 0.2536408,\
        0.2507257, 0.2477819, 0.2447734, 0.2416521, 0.2383612, 0.2348389, 0.2310228, 0.2268539, 0.2222811,\
        0.2172630, 0.2117684, 0.2057762, 0.1992736],\
        [0.2781975, 0.2852669, 0.2916022, 0.2967685, 0.3012777, 0.3049657, 0.3066640, 0.3070622, 0.3067841,\
        0.3063110, 0.3061004, 0.3067939, 0.3082594, 0.3096133, 0.3106222, 0.3116536, 0.3128038, 0.3139088,\
        0.3148429, 0.3155453, 0.3160074, 0.3162568, 0.3163169, 0.3162118, 0.3159924, 0.3156948, 0.3152967,\
        0.3148356, 0.3144721, 0.3143449, 0.3145743, 0.3151898, 0.3160409, 0.3169547, 0.3178230, 0.3185928,\
        0.3192427, 0.3197690, 0.3201814, 0.3205018, 0.3207636, 0.3210123, 0.3213092, 0.3217384, 0.3224137,\
        0.3234854, 0.3251444, 0.3276186, 0.3311605, 0.3360234, 0.3424279, 0.3505216, 0.3603382, 0.3717619,\
        0.3845018, 0.3983232, 0.4134874, 0.4278044, 0.4397638, 0.4484217, 0.4532389, 0.4542161, 0.4519066,\
        0.4471707, 0.4408726, 0.4337052, 0.4261910, 0.4187534, 0.4115742, 0.4047812, 0.3989201, 0.3936593,\
        0.3889349, 0.3842143, 0.3788938, 0.3732519, 0.3675072, 0.3617040, 0.3559028, 0.3501828, 0.3445956,\
        0.3391655, 0.3339072, 0.3288181, 0.3238330, 0.3187961, 0.3135573, 0.3081215, 0.3026121, 0.2977563,\
        0.2938198, 0.2903173, 0.2873702, 0.2847195, 0.2821608, 0.2795936, 0.2770028, 0.2744064, 0.2718087,\
        0.2691944, 0.2665412, 0.2638308, 0.2610536, 0.2582094, 0.2553053, 0.2523526, 0.2493620, 0.2463374,\
        0.2432715, 0.2401449, 0.2369262, 0.2335741, 0.2300400, 0.2262712, 0.2222142, 0.2178182, 0.2130379,\
        0.2078351, 0.2021801, 0.1960509, 0.1894330],\
        [0.2738304, 0.2809757, 0.2874553, 0.2931994, 0.2983573, 0.3015926, 0.3024329, 0.3021979, 0.3013626,\
        0.3004609, 0.2997251, 0.3002563, 0.3019605, 0.3031074, 0.3035923, 0.3043934, 0.3055334, 0.3066086,\
        0.3075135, 0.3081783, 0.3086023, 0.3088571, 0.3089609, 0.3089056, 0.3087793, 0.3086735, 0.3084381,\
        0.3080121, 0.3077540, 0.3077490, 0.3081898, 0.3092238, 0.3104972, 0.3117656, 0.3129262, 0.3139448,\
        0.3148099, 0.3155185, 0.3160802, 0.3165189, 0.3168694, 0.3171779, 0.3175068, 0.3179402, 0.3185924,\
        0.3196141, 0.3211979, 0.3235755, 0.3270040, 0.3317393, 0.3379969, 0.3459080, 0.3554810, 0.3665845,\
        0.3789826, 0.3924571, 0.4063950, 0.4192886, 0.4299770, 0.4376875, 0.4419769, 0.4428237, 0.4406447,\
        0.4361652, 0.4301930, 0.4233627, 0.4160601, 0.4088968, 0.4021272, 0.3955479, 0.3897701, 0.3846127,\
        0.3797807, 0.3749602, 0.3698572, 0.3644828, 0.3589597, 0.3533754, 0.3477974, 0.3422872, 0.3368943,\
        0.3316598, 0.3266319, 0.3218669, 0.3173434, 0.3128029, 0.3079002, 0.3026837, 0.2974074, 0.2924558,\
        0.2882761, 0.2846435, 0.2814543, 0.2785667, 0.2758308, 0.2731490, 0.2704787, 0.2678073, 0.2651271,\
        0.2624254, 0.2596858, 0.2568939, 0.2540404, 0.2511219, 0.2481401, 0.2450991, 0.2420019, 0.2388467,\
        0.2356237, 0.2323140, 0.2288897, 0.2253147, 0.2215472, 0.2175419, 0.2132528, 0.2086355, 0.2036497,\
        0.1982608, 0.1924405, 0.1861670, 0.1794249],\
        [0.2687351, 0.2759179, 0.2824916, 0.2883635, 0.2932461, 0.2962426, 0.2970855, 0.2967805, 0.2958948,\
        0.2949466, 0.2943518, 0.2947221, 0.2959527, 0.2967527, 0.2970098, 0.2976557, 0.2987013, 0.2996834,\
        0.3005397, 0.3011649, 0.3015472, 0.3018294, 0.3020192, 0.3020207, 0.3019656, 0.3021290, 0.3021767,\
        0.3016944, 0.3016602, 0.3017861, 0.3023647, 0.3038955, 0.3055510, 0.3071137, 0.3085175, 0.3097580,\
        0.3108294, 0.3117190, 0.3124329, 0.3129986, 0.3134514, 0.3138337, 0.3142033, 0.3146392, 0.3152481,\
        0.3161744, 0.3176094, 0.3197917, 0.3229956, 0.3275009, 0.3335466, 0.3412761, 0.3506962, 0.3616673,\
        0.3739240, 0.3870459, 0.4001597, 0.4118986, 0.4214984, 0.4283904, 0.4321359, 0.4326690, 0.4303365,\
        0.4258377, 0.4200586, 0.4137346, 0.4067897, 0.3996736, 0.3934595, 0.3872958, 0.3815752, 0.3763292,\
        0.3713549, 0.3664239, 0.3613608, 0.3561240, 0.3507666, 0.3453621, 0.3399741, 0.3346529, 0.3294389,\
        0.3243704, 0.3195037, 0.3149712, 0.3109465, 0.3070896, 0.3025622, 0.2974323, 0.2922939, 0.2874113,\
        0.2830466, 0.2791952, 0.2757593, 0.2726242, 0.2696762, 0.2668277, 0.2640246, 0.2612371, 0.2584464,\
        0.2556362, 0.2527911, 0.2498977, 0.2469461, 0.2439303, 0.2408476, 0.2376966, 0.2344750, 0.2311766,\
        0.2277896, 0.2242953, 0.2206682, 0.2168764, 0.2128835, 0.2086502, 0.2041364, 0.1993032, 0.1941146,\
        0.1885392, 0.1825504, 0.1761271, 0.1692532],\
        [0.2628634, 0.2699863, 0.2764582, 0.2821031, 0.2865860, 0.2894588, 0.2906652, 0.2907078, 0.2901452,\
        0.2894577, 0.2890634, 0.2892587, 0.2899013, 0.2904588, 0.2908394, 0.2914044, 0.2922152, 0.2930675,\
        0.2938381, 0.2944315, 0.2948464, 0.2951762, 0.2954333, 0.2955624, 0.2956826, 0.2959881, 0.2961981,\
        0.2960352, 0.2962030, 0.2965957, 0.2975108, 0.2992227, 0.3011237, 0.3028804, 0.3044706, 0.3059120,\
        0.3071920, 0.3082632, 0.3091286, 0.3098322, 0.3104052, 0.3108780, 0.3113019, 0.3117455, 0.3122992,\
        0.3130915, 0.3143058, 0.3161884, 0.3190415, 0.3231932, 0.3289369, 0.3364530, 0.3457470, 0.3566344,\
        0.3687499, 0.3815157, 0.3940386, 0.4050883, 0.4140952, 0.4204713, 0.4237856, 0.4239459, 0.4212608,\
        0.4164577, 0.4105900, 0.4047599, 0.3991225, 0.3920249, 0.3856819, 0.3796986, 0.3740356, 0.3687033,\
        0.3635907, 0.3585446, 0.3534446, 0.3482478, 0.3429822, 0.3377080, 0.3324794, 0.3273296, 0.3222833,\
        0.3173676, 0.3125988, 0.3080661, 0.3043270, 0.3012818, 0.2971091, 0.2921096, 0.2870693, 0.2822547,\
        0.2778182, 0.2737978, 0.2701463, 0.2667822, 0.2636186, 0.2605810, 0.2576143, 0.2546811, 0.2517550,\
        0.2488159, 0.2458465, 0.2428328, 0.2397635, 0.2366302, 0.2334266, 0.2301473, 0.2267858, 0.2233330,\
        0.2197753, 0.2160942, 0.2122659, 0.2082619, 0.2040498, 0.1995953, 0.1948631, 0.1898187, 0.1844301,\
        0.1786685, 0.1725092, 0.1659317, 0.1589202],\
        [0.2562316, 0.2632170, 0.2694925, 0.2748741, 0.2791157, 0.2820021, 0.2835496, 0.2840600, 0.2839544,\
        0.2836398, 0.2834492, 0.2835643, 0.2839291, 0.2843485, 0.2847687, 0.2852959, 0.2859642, 0.2866906,\
        0.2873787, 0.2879630, 0.2884364, 0.2888348, 0.2891749, 0.2894664, 0.2897775, 0.2901591, 0.2905012,\
        0.2907724, 0.2912166, 0.2919665, 0.2931895, 0.2949368, 0.2969113, 0.2988362, 0.3006020, 0.3022557,\
        0.3037690, 0.3050204, 0.3060280, 0.3068923, 0.3076129, 0.3081947, 0.3086940, 0.3091654, 0.3096696,\
        0.3103066, 0.3112409, 0.3127215, 0.3150846, 0.3187311, 0.3240520, 0.3312964, 0.3404634, 0.3512922,\
        0.3633045, 0.3758226, 0.3879628, 0.3988032, 0.4076713, 0.4138285, 0.4169069, 0.4167480, 0.4136474,\
        0.4083593, 0.4022082, 0.3964303, 0.3910661, 0.3848893, 0.3786338, 0.3726729, 0.3670100, 0.3616117,\
        0.3563948, 0.3512488, 0.3460839, 0.3408664, 0.3356270, 0.3304346, 0.3253445, 0.3203584, 0.3154709,\
        0.3107315, 0.3061303, 0.3016327, 0.2976531, 0.2947322, 0.2909025, 0.2863354, 0.2815524, 0.2768669,\
        0.2724354, 0.2683172, 0.2645027, 0.2609442, 0.2575805, 0.2543519, 0.2512087, 0.2481122, 0.2450330,\
        0.2419479, 0.2388379, 0.2356874, 0.2324834, 0.2292154, 0.2258741, 0.2224510, 0.2189366, 0.2153195,\
        0.2115850, 0.2077146, 0.2036859, 0.1994728, 0.1950464, 0.1903761, 0.1854307, 0.1801794, 0.1745933,\
        0.1686462, 0.1623149, 0.1555802, 0.1484261],\
        [0.2489121, 0.2557343, 0.2618148, 0.2670016, 0.2711346, 0.2741125, 0.2759784, 0.2769403, 0.2773028,\
        0.2773802, 0.2774294, 0.2775965, 0.2778964, 0.2782739, 0.2787061, 0.2792197, 0.2798228, 0.2804756,\
        0.2811207, 0.2817152, 0.2822480, 0.2827320, 0.2831847, 0.2836284, 0.2840990, 0.2846142, 0.2851548,\
        0.2857475, 0.2865089, 0.2875597, 0.2889889, 0.2907842, 0.2927887, 0.2948199, 0.2967742, 0.2986494,\
        0.3004066, 0.3018212, 0.3029542, 0.3040475, 0.3049491, 0.3056451, 0.3062536, 0.3067945, 0.3072806,\
        0.3077711, 0.3083946, 0.3093913, 0.3111242, 0.3140847, 0.3188304, 0.3257411, 0.3347970, 0.3456212,\
        0.3576237, 0.3700846, 0.3821986, 0.3931612, 0.4021289, 0.4082220, 0.4113658, 0.4109562, 0.4075316,\
        0.4019403, 0.3955861, 0.3896828, 0.3839447, 0.3779888, 0.3719308, 0.3660281, 0.3603702, 0.3549421,\
        0.3496722, 0.3444652, 0.3392408, 0.3339696, 0.3286966, 0.3235343, 0.3185802, 0.3137795, 0.3090188,\
        0.3044154, 0.3000933, 0.2957529, 0.2917965, 0.2882830, 0.2845078, 0.2802547, 0.2757471, 0.2712260,\
        0.2668462, 0.2626831, 0.2587536, 0.2550378, 0.2514969, 0.2480868, 0.2447659, 0.2414987, 0.2382555,\
        0.2350122, 0.2317487, 0.2284477, 0.2250950, 0.2216779, 0.2181850, 0.2146053, 0.2109273, 0.2071377,\
        0.2032210, 0.1991589, 0.1949299, 0.1905100, 0.1858730, 0.1809914, 0.1758370, 0.1703823, 0.1646010,\
        0.1584692, 0.1519652, 0.1450707, 0.1377703],\
        [0.2409984, 0.2476702, 0.2535982, 0.2586687, 0.2627826, 0.2658961, 0.2680607, 0.2694305, 0.2702275,\
        0.2706880, 0.2710121, 0.2713298, 0.2716940, 0.2721107, 0.2725809, 0.2731135, 0.2737082, 0.2743441,\
        0.2749892, 0.2756177, 0.2762201, 0.2768020, 0.2773781, 0.2779685, 0.2785966, 0.2792794, 0.2800304,\
        0.2808871, 0.2819210, 0.2832088, 0.2847947, 0.2866567, 0.2887011, 0.2908121, 0.2929066, 0.2949400,\
        0.2968387, 0.2984253, 0.2997666, 0.3012050, 0.3022644, 0.3030480, 0.3038392, 0.3045075, 0.3050316,\
        0.3054281, 0.3057541, 0.3062399, 0.3072445, 0.3093142, 0.3132955, 0.3198379, 0.3288387, 0.3397253,\
        0.3517876, 0.3642911, 0.3764942, 0.3876159, 0.3966649, 0.4036106, 0.4068484, 0.4060214, 0.4021996,\
        0.3966554, 0.3901664, 0.3837937, 0.3776214, 0.3715124, 0.3654734, 0.3596094, 0.3539833, 0.3485811,\
        0.3433301, 0.3381288, 0.3328850, 0.3275568, 0.3221914, 0.3169679, 0.3121535, 0.3076495, 0.3030103,\
        0.2983950, 0.2941585, 0.2899815, 0.2859815, 0.2821591, 0.2782544, 0.2741057, 0.2697682, 0.2653782,\
        0.2610548, 0.2568716, 0.2528589, 0.2490152, 0.2453194, 0.2417410, 0.2382475, 0.2348084, 0.2313961,\
        0.2279870, 0.2245604, 0.2210987, 0.2175861, 0.2140085, 0.2103528, 0.2066064, 0.2027560, 0.1987872,\
        0.1946839, 0.1904277, 0.1859984, 0.1813734, 0.1765287, 0.1714392, 0.1660794, 0.1604243, 0.1544500,\
        0.1481343, 0.1414571, 0.1344010, 0.1269511],\
        [0.2325793, 0.2391320, 0.2449586, 0.2499783, 0.2541319, 0.2574054, 0.2598507, 0.2615865, 0.2627781,\
        0.2636044, 0.2642242, 0.2647538, 0.2652618, 0.2657823, 0.2663328, 0.2669244, 0.2675586, 0.2682262,\
        0.2689112, 0.2695999, 0.2702872, 0.2709775, 0.2716828, 0.2724199, 0.2732083, 0.2740672, 0.2750194,\
        0.2760996, 0.2773551, 0.2788306, 0.2805463, 0.2824809, 0.2845731, 0.2867444, 0.2889237, 0.2910485,\
        0.2930431, 0.2948381, 0.2965003, 0.2981077, 0.2992836, 0.3002264, 0.3013234, 0.3021274, 0.3027626,\
        0.3031816, 0.3032576, 0.3033069, 0.3036410, 0.3046418, 0.3076096, 0.3138380, 0.3228639, 0.3338284,\
        0.3459090, 0.3583862, 0.3706013, 0.3819152, 0.3917664, 0.3992244, 0.4020671, 0.4009682, 0.3966936,\
        0.3910370, 0.3846006, 0.3779996, 0.3715208, 0.3652157, 0.3591153, 0.3532715, 0.3477122, 0.3424081,\
        0.3372704, 0.3321730, 0.3269929, 0.3216568, 0.3161803, 0.3107276, 0.3058928, 0.3020003, 0.2974307,\
        0.2928514, 0.2884816, 0.2842701, 0.2801792, 0.2761704, 0.2721309, 0.2679721, 0.2636985, 0.2593772,\
        0.2550856, 0.2508829, 0.2468009, 0.2428476, 0.2390137, 0.2352796, 0.2316209, 0.2280122, 0.2244295,\
        0.2208503, 0.2172547, 0.2136247, 0.2099438, 0.2061970, 0.2023700, 0.1984489, 0.1944193, 0.1902659,\
        0.1859723, 0.1815205, 0.1768908, 0.1720620, 0.1670119, 0.1617173, 0.1561549, 0.1503020, 0.1441364,\
        0.1376378, 0.1307872, 0.1235681, 0.1159661],\
        [0.2237283, 0.2301982, 0.2359706, 0.2409898, 0.2452217, 0.2486686, 0.2513788, 0.2534462, 0.2549974,\
        0.2561706, 0.2570939, 0.2578700, 0.2585714, 0.2592442, 0.2599171, 0.2606074, 0.2613227, 0.2620627,\
        0.2628223, 0.2635967, 0.2643857, 0.2651944, 0.2660339, 0.2669191, 0.2678674, 0.2688985, 0.2700355,\
        0.2713067, 0.2727436, 0.2743714, 0.2761975, 0.2782027, 0.2803429, 0.2825600, 0.2847936, 0.2869844,\
        0.2890760, 0.2910344, 0.2928667, 0.2945402, 0.2959461, 0.2972110, 0.2984364, 0.2994002, 0.3002156,\
        0.3008482, 0.3006823, 0.3005016, 0.3004969, 0.3006612, 0.3023772, 0.3083668, 0.3173700, 0.3282728,\
        0.3401518, 0.3523478, 0.3642829, 0.3753824, 0.3849901, 0.3920551, 0.3949382, 0.3940044, 0.3901719,\
        0.3846899, 0.3783943, 0.3717905, 0.3651996, 0.3587880, 0.3526502, 0.3468493, 0.3414082, 0.3362880,\
        0.3313778, 0.3265117, 0.3215173, 0.3162857, 0.3108240, 0.3052115, 0.2997736, 0.2960944, 0.2918894,\
        0.2873668, 0.2829250, 0.2786079, 0.2743905, 0.2702300, 0.2660644, 0.2618447, 0.2575638, 0.2532532,\
        0.2489591, 0.2447231, 0.2405724, 0.2365178, 0.2325565, 0.2286763, 0.2248592, 0.2210848, 0.2173324,\
        0.2135816, 0.2098134, 0.2060101, 0.2021551, 0.1982328, 0.1942281, 0.1901263, 0.1859123, 0.1815704,\
        0.1770839, 0.1724351, 0.1676050, 0.1625736, 0.1573199, 0.1518227, 0.1460603, 0.1400117, 0.1336566,\
        0.1269757, 0.1199517, 0.1125685, 0.1048124],\
        [0.2145017, 0.2209242, 0.2266822, 0.2317382, 0.2360745, 0.2397014, 0.2426631, 0.2450354, 0.2469179,\
        0.2484197, 0.2496459, 0.2506864, 0.2516114, 0.2524715, 0.2533019, 0.2541259, 0.2549575, 0.2558037,\
        0.2566674, 0.2575505, 0.2584570, 0.2593940, 0.2603724, 0.2614059, 0.2625109, 0.2637055, 0.2650100,\
        0.2664463, 0.2680350, 0.2697899, 0.2717112, 0.2737816, 0.2759667, 0.2782212, 0.2804947, 0.2827376,\
        0.2849054, 0.2869673, 0.2889026, 0.2906796, 0.2922736, 0.2937247, 0.2950439, 0.2961783, 0.2971498,\
        0.2977800, 0.2977444, 0.2975698, 0.2975087, 0.2977047, 0.2991950, 0.3042098, 0.3129922, 0.3234215,\
        0.3346800, 0.3462094, 0.3574665, 0.3678530, 0.3766439, 0.3829143, 0.3858125, 0.3853585, 0.3822341,\
        0.3773154, 0.3713737, 0.3649497, 0.3584127, 0.3520069, 0.3458949, 0.3401824, 0.3349171, 0.3300690,\
        0.3255101, 0.3210193, 0.3163467, 0.3113298, 0.3060078, 0.3006639, 0.2953214, 0.2907080, 0.2863407,\
        0.2818500, 0.2773548, 0.2729298, 0.2685799, 0.2642792, 0.2599911, 0.2556856, 0.2513544, 0.2470114,\
        0.2426827, 0.2383953, 0.2341693, 0.2300145, 0.2259310, 0.2219109, 0.2179409, 0.2140047, 0.2100844,\
        0.2061619, 0.2022194, 0.1982399, 0.1942070, 0.1901049, 0.1859181, 0.1816315, 0.1772295, 0.1726962,\
        0.1680150, 0.1631684, 0.1581382, 0.1529052, 0.1474498, 0.1417520, 0.1357918, 0.1295495, 0.1230062,\
        0.1161441, 0.1089465, 0.1013985, 0.0934867],\
        [0.2049416, 0.2113479, 0.2171236, 0.2222444, 0.2267026, 0.2305130, 0.2337150, 0.2363715, 0.2385624,\
        0.2403761, 0.2419000, 0.2432125, 0.2443788, 0.2454503, 0.2464652, 0.2474514, 0.2484279, 0.2494078,\
        0.2503998, 0.2514112, 0.2524493, 0.2535234, 0.2546447, 0.2558264, 0.2570834, 0.2584318, 0.2598884,\
        0.2614695, 0.2631885, 0.2650521, 0.2670568, 0.2691865, 0.2714133, 0.2737002, 0.2760059, 0.2782889,\
        0.2805113, 0.2826424, 0.2846561, 0.2865289, 0.2882484, 0.2898187, 0.2912337, 0.2924678, 0.2934789,\
        0.2941499, 0.2944328, 0.2945693, 0.2948524, 0.2956393, 0.2976905, 0.3024945, 0.3101385, 0.3193802,\
        0.3295247, 0.3400222, 0.3502977, 0.3597173, 0.3675793, 0.3731629, 0.3759445, 0.3758807, 0.3733819,\
        0.3690748, 0.3635925, 0.3574610, 0.3510858, 0.3447701, 0.3387380, 0.3331496, 0.3280987, 0.3235893,\
        0.3194936, 0.3155237, 0.3112933, 0.3065618, 0.3012981, 0.2959451, 0.2906802, 0.2856970, 0.2809732,\
        0.2763278, 0.2717244, 0.2671803, 0.2626989, 0.2582651, 0.2538560, 0.2494521, 0.2450454, 0.2406408,\
        0.2362524, 0.2318962, 0.2275857, 0.2233280, 0.2191237, 0.2149672, 0.2108482, 0.2067534, 0.2026675,\
        0.1985742, 0.1944571, 0.1903000, 0.1860870, 0.1818024, 0.1774307, 0.1729565, 0.1683643, 0.1636379,\
        0.1587610, 0.1537165, 0.1484866, 0.1430533, 0.1373978, 0.1315013, 0.1253451, 0.1189109, 0.1121809,\
        0.1051383, 0.0977675, 0.0900541, 0.0819853],\
        [0.1950782, 0.2014952, 0.2073143, 0.2125205, 0.2171129, 0.2211080, 0.2245413, 0.2274655, 0.2299464,\
        0.2320572, 0.2338712, 0.2354569, 0.2368742, 0.2381726, 0.2393915, 0.2405615, 0.2417055, 0.2428408,\
        0.2439807, 0.2451362, 0.2463178, 0.2475365, 0.2488039, 0.2501328, 0.2515370, 0.2530303, 0.2546262,\
        0.2563368, 0.2581705, 0.2601297, 0.2622088, 0.2643931, 0.2666588, 0.2689754, 0.2713085, 0.2736229,\
        0.2758856, 0.2780672, 0.2801427, 0.2820916, 0.2839002, 0.2855602, 0.2870592, 0.2883748, 0.2894724,\
        0.2903174, 0.2909534, 0.2915716, 0.2924805, 0.2941241, 0.2971329, 0.3018214, 0.3080212, 0.3157155,\
        0.3244879, 0.3337801, 0.3429593, 0.3513683, 0.3583557, 0.3633429, 0.3659505, 0.3661153, 0.3640836,\
        0.3602958, 0.3552555, 0.3494422, 0.3432721, 0.3370875, 0.3311603, 0.3257017, 0.3208609, 0.3167007,\
        0.3131262, 0.3098083, 0.3060872, 0.3016169, 0.2964811, 0.2910355, 0.2856870, 0.2805247, 0.2755592,\
        0.2707267, 0.2659814, 0.2613081, 0.2566980, 0.2521375, 0.2476104, 0.2431025, 0.2386062, 0.2341218,\
        0.2296558, 0.2252169, 0.2208129, 0.2164481, 0.2121224, 0.2078312, 0.2035659, 0.1993152, 0.1950658,\
        0.1908030, 0.1865119, 0.1821771, 0.1777832, 0.1733148, 0.1687567, 0.1640936, 0.1593098, 0.1543896,\
        0.1493167, 0.1440745, 0.1386459, 0.1330135, 0.1271595, 0.1210662, 0.1147160, 0.1080915, 0.1011761,\
        0.0939540, 0.0864102, 0.0785310, 0.0703041],\
        [0.1849333, 0.1913833, 0.1972661, 0.2025734, 0.2073084, 0.2114885, 0.2151453, 0.2183240, 0.2210799,\
        0.2234743, 0.2255699, 0.2274267, 0.2290989, 0.2306335, 0.2320696, 0.2334390, 0.2347672, 0.2360744,\
        0.2373772, 0.2386895, 0.2400241, 0.2413932, 0.2428092, 0.2442844, 0.2458313, 0.2474620, 0.2491874,\
        0.2510161, 0.2529532, 0.2549987, 0.2571460, 0.2593814, 0.2616845, 0.2640297, 0.2663878, 0.2687287,\
        0.2710234, 0.2732453, 0.2753711, 0.2773817, 0.2792624, 0.2810012, 0.2825863, 0.2840043, 0.2852463,\
        0.2863287, 0.2873343, 0.2884457, 0.2899468, 0.2922078, 0.2955866, 0.3000828, 0.3053599, 0.3117286,\
        0.3192585, 0.3274302, 0.3355801, 0.3430607, 0.3492751, 0.3537350, 0.3561352, 0.3564022, 0.3546839,\
        0.3512886, 0.3466120, 0.3410802, 0.3351082, 0.3290626, 0.3232430, 0.3178972, 0.3132180, 0.3093390,\
        0.3062211, 0.3035242, 0.3005189, 0.2959405, 0.2909256, 0.2855936, 0.2802417, 0.2749998, 0.2699003,\
        0.2649269, 0.2600524, 0.2552574, 0.2505277, 0.2458498, 0.2412104, 0.2365981, 0.2320058, 0.2274312,\
        0.2228761, 0.2183445, 0.2138399, 0.2093641, 0.2049157, 0.2004904, 0.1960805, 0.1916760, 0.1872652,\
        0.1828348, 0.1783708, 0.1738588, 0.1692841, 0.1646318, 0.1598870, 0.1550344, 0.1500589, 0.1449448,\
        0.1396763, 0.1342372, 0.1286110, 0.1227810, 0.1167303, 0.1104420, 0.1038995, 0.0970864, 0.0899870,\
        0.0825862, 0.0748698, 0.0668248, 0.0584391],\
        [0.1745219, 0.1810234, 0.1869861, 0.1924062, 0.1972899, 0.2016543, 0.2055281, 0.2089505, 0.2119685,\
        0.2146347, 0.2170031, 0.2191265, 0.2210540, 0.2228294, 0.2244908, 0.2260702, 0.2275944, 0.2290856,\
        0.2305624, 0.2320410, 0.2335357, 0.2350598, 0.2366260, 0.2382464, 0.2399325, 0.2416947, 0.2435419,\
        0.2454805, 0.2475133, 0.2496388, 0.2518501, 0.2541347, 0.2564750, 0.2588488, 0.2612315, 0.2635972,\
        0.2659205, 0.2681777, 0.2703479, 0.2724135, 0.2743605, 0.2761784, 0.2778598, 0.2794031, 0.2808204,\
        0.2821542, 0.2834988, 0.2850153, 0.2869256, 0.2894749, 0.2928365, 0.2969694, 0.3015531, 0.3069998,\
        0.3136811, 0.3209725, 0.3282554, 0.3349464, 0.3404980, 0.3444856, 0.3466564, 0.3469374, 0.3454151,\
        0.3423009, 0.3378976, 0.3325804, 0.3267717, 0.3208678, 0.3151649, 0.3099292, 0.3053858, 0.3016714,\
        0.2988656, 0.2963809, 0.2935454, 0.2894187, 0.2846098, 0.2794851, 0.2742480, 0.2690290, 0.2638869,\
        0.2588355, 0.2538686, 0.2489748, 0.2441425, 0.2393602, 0.2346173, 0.2299043, 0.2252148, 0.2205452,\
        0.2158948, 0.2112641, 0.2066540, 0.2020640, 0.1974918, 0.1929327, 0.1883795, 0.1838233, 0.1792531,\
        0.1746568, 0.1700216, 0.1653337, 0.1605791, 0.1557435, 0.1508124, 0.1457709, 0.1406042, 0.1352969,\
        0.1298338, 0.1241990, 0.1183766, 0.1123507, 0.1061051, 0.0996237, 0.0928907, 0.0858906, 0.0786085,\
        0.0710299, 0.0631416, 0.0549307, 0.0463858],\
        [0.1638541, 0.1704224, 0.1764778, 0.1820197, 0.1870561, 0.1916039, 0.1956891, 0.1993458, 0.2026152,\
        0.2055425, 0.2081748, 0.2105591, 0.2127397, 0.2147573, 0.2166481, 0.2184439, 0.2201718, 0.2218552,\
        0.2235140, 0.2251656, 0.2268256, 0.2285079, 0.2302254, 0.2319899, 0.2338122, 0.2357014, 0.2376650,\
        0.2397076, 0.2418309, 0.2440325, 0.2463055, 0.2486388, 0.2510169, 0.2534208, 0.2558294, 0.2582206,\
        0.2605722, 0.2628637, 0.2650768, 0.2671960, 0.2692098, 0.2711108, 0.2728977, 0.2745785, 0.2761776,\
        0.2777462, 0.2793728, 0.2811855, 0.2833381, 0.2859723, 0.2891542, 0.2928154, 0.2968553, 0.3017927,\
        0.3079564, 0.3145094, 0.3210768, 0.3271239, 0.3321057, 0.3356527, 0.3375731, 0.3378095, 0.3364108,\
        0.3335095, 0.3293136, 0.3241378, 0.3184379, 0.3127027, 0.3071925, 0.3020732, 0.2978056, 0.2942620,\
        0.2914466, 0.2888396, 0.2858583, 0.2820811, 0.2776251, 0.2727688, 0.2677132, 0.2625896, 0.2574735,\
        0.2524003, 0.2473817, 0.2424180, 0.2375043, 0.2326335, 0.2277980, 0.2229910, 0.2182067, 0.2134414,\
        0.2086930, 0.2039601, 0.1992417, 0.1945358, 0.1898391, 0.1851467, 0.1804516, 0.1757453, 0.1710178,\
        0.1662578, 0.1614532, 0.1565911, 0.1516581, 0.1466406, 0.1415243, 0.1362951, 0.1309383, 0.1254391,\
        0.1197827, 0.1139538, 0.1079370, 0.1017171, 0.0952785, 0.0886059, 0.0816843, 0.0744989, 0.0670354,\
        0.0592803, 0.0512205, 0.0428440, 0.0341396],\
        [0.1529367, 0.1595842, 0.1657423, 0.1714129, 0.1766050, 0.1813347, 0.1856260, 0.1895093, 0.1930204,\
        0.1961991, 0.1990870, 0.2017256, 0.2041554, 0.2064140, 0.2085356, 0.2105509, 0.2124871, 0.2143677,\
        0.2162134, 0.2180425, 0.2198711, 0.2217136, 0.2235829, 0.2254906, 0.2274468, 0.2294597, 0.2315358,\
        0.2336787, 0.2358892, 0.2381646, 0.2404985, 0.2428810, 0.2452985, 0.2477349, 0.2501720, 0.2525909,\
        0.2549728, 0.2573001, 0.2595572, 0.2617315, 0.2638145, 0.2658027, 0.2676996, 0.2695194, 0.2712915,\
        0.2730662, 0.2749182, 0.2769431, 0.2792430, 0.2819001, 0.2849477, 0.2883764, 0.2922574, 0.2969276,\
        0.3023484, 0.3081201, 0.3141273, 0.3196686, 0.3241191, 0.3272154, 0.3288555, 0.3290100, 0.3277054,\
        0.3250097, 0.3210216, 0.3159359, 0.3102578, 0.3046959, 0.2995892, 0.2947313, 0.2905965, 0.2871063,\
        0.2840898, 0.2812107, 0.2780686, 0.2743987, 0.2701876, 0.2655733, 0.2607079, 0.2557100, 0.2506592,\
        0.2456030, 0.2405662, 0.2355593, 0.2305851, 0.2256419, 0.2207260, 0.2158329, 0.2109585, 0.2060995,\
        0.2012531, 0.1964173, 0.1915896, 0.1867672, 0.1819462, 0.1771212, 0.1722856, 0.1674312, 0.1625485,\
        0.1576271, 0.1526553, 0.1476211, 0.1425117, 0.1373140, 0.1320144, 0.1265990, 0.1210538, 0.1153645,\
        0.1095166, 0.1034955, 0.0972864, 0.0908744, 0.0842449, 0.0773832, 0.0702748, 0.0629058, 0.0552626,\
        0.0473319, 0.0391016, 0.0305598, 0.0216957],\
        [0.1417734, 0.1485104, 0.1547793, 0.1605837, 0.1659331, 0.1708433, 0.1753361, 0.1794387, 0.1831831,\
        0.1866044, 0.1897395, 0.1926257, 0.1952998, 0.1977963, 0.2001477, 0.2023833, 0.2045295, 0.2066100,\
        0.2086454, 0.2106545, 0.2126536, 0.2146572, 0.2166784, 0.2187283, 0.2208165, 0.2229508, 0.2251368,\
        0.2273775, 0.2296733, 0.2320217, 0.2344168, 0.2368498, 0.2393092, 0.2417810, 0.2442500, 0.2467000,\
        0.2491152, 0.2514810, 0.2537846, 0.2560165, 0.2581709, 0.2602479, 0.2622544, 0.2642076, 0.2661373,\
        0.2680885, 0.2701217, 0.2723077, 0.2747174, 0.2774079, 0.2804157, 0.2837770, 0.2875770, 0.2919200,\
        0.2966936, 0.3018592, 0.3075499, 0.3126340, 0.3164680, 0.3190583, 0.3203856, 0.3204348, 0.3192191,\
        0.3167716, 0.3131073, 0.3081745, 0.3024486, 0.2970490, 0.2922106, 0.2876764, 0.2835955, 0.2800002,\
        0.2767448, 0.2735868, 0.2702590, 0.2665734, 0.2624784, 0.2580283, 0.2533184, 0.2484406, 0.2434670,\
        0.2384478, 0.2334142, 0.2283843, 0.2233670, 0.2183658, 0.2133806, 0.2084098, 0.2034510, 0.1985017,\
        0.1935593, 0.1886213, 0.1836848, 0.1787465, 0.1738020, 0.1688459, 0.1638715, 0.1588710, 0.1538354,\
        0.1487548, 0.1436184, 0.1384144, 0.1331310, 0.1277553, 0.1222744, 0.1166751, 0.1109436, 0.1050663,\
        0.0990291, 0.0928179, 0.0864186, 0.0798169, 0.0729987, 0.0659499, 0.0586569, 0.0511061, 0.0432845,\
        0.0351797, 0.0267796, 0.0180731, 0.0090495],\
        [0.1303661, 0.1372009, 0.1435869, 0.1495289, 0.1550367, 0.1601255, 0.1648154, 0.1691310, 0.1731010,\
        0.1767566, 0.1801310, 0.1832579, 0.1861705, 0.1889010, 0.1914796, 0.1939342, 0.1962904, 0.1985712,\
        0.2007973, 0.2029873, 0.2051575, 0.2073225, 0.2094950, 0.2116861, 0.2139051, 0.2161590, 0.2184531,\
        0.2207902, 0.2231707, 0.2255920, 0.2280493, 0.2305348, 0.2330388, 0.2355494, 0.2380539, 0.2405387,\
        0.2429908, 0.2453982, 0.2477510, 0.2500423, 0.2522692, 0.2544340, 0.2565460, 0.2586230, 0.2606930,\
        0.2627949, 0.2649773, 0.2672946, 0.2698006, 0.2725430, 0.2755628, 0.2789053, 0.2826253, 0.2867535,\
        0.2912590, 0.2961795, 0.3016858, 0.3058210, 0.3089000, 0.3109646, 0.3119756, 0.3119072, 0.3107625,\
        0.3085689, 0.3053512, 0.3010097, 0.2952879, 0.2899894, 0.2851453, 0.2806772, 0.2765802, 0.2728390,\
        0.2693478, 0.2659350, 0.2624161, 0.2586522, 0.2545830, 0.2502198, 0.2456147, 0.2408313, 0.2359275,\
        0.2309492, 0.2259289, 0.2208881, 0.2158400, 0.2107918, 0.2057469, 0.2007060, 0.1956686, 0.1906330,\
        0.1855974, 0.1805592, 0.1755155, 0.1704626, 0.1653961, 0.1603106, 0.1551994, 0.1500551, 0.1448692,\
        0.1396320, 0.1343334, 0.1289624, 0.1235074, 0.1179563, 0.1122968, 0.1065159, 0.1006006, 0.0945376,\
        0.0883136, 0.0819148, 0.0753277, 0.0685387, 0.0615341, 0.0543005, 0.0468249, 0.0390941, 0.0310959,\
        0.0228182, 0.0142494, 0.0053788, -0.0038039]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=20 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax20vt7.5
    zgrid_H_Hmax_vmax20vt7_5=[[0.478611, 0.485879, 0.492490, 0.498172, 0.502054, 0.504834, 0.507485, 0.510125, 0.512915, 0.515959,\
        0.519307, 0.522966, 0.526886, 0.530953, 0.535005, 0.538865, 0.542356, 0.545314, 0.547599, 0.549101,\
        0.549737, 0.549454, 0.548225, 0.546037, 0.542892, 0.538799, 0.533783, 0.527886, 0.521176, 0.513742,\
        0.505656, 0.496812, 0.487169, 0.477449, 0.467998, 0.458955, 0.450353, 0.442182, 0.434425, 0.427083,\
        0.420188, 0.413802, 0.408020, 0.402985, 0.399001, 0.396427, 0.394433, 0.392722, 0.391530, 0.391254,\
        0.392388, 0.395445, 0.400834, 0.408663, 0.418641, 0.430060, 0.442128, 0.454055, 0.465036, 0.474262,\
        0.481072, 0.485459, 0.488393, 0.490693, 0.492504, 0.494378, 0.496626, 0.498620, 0.500368, 0.501889,\
        0.503196, 0.504301, 0.505219, 0.505966, 0.506559, 0.507012, 0.507341, 0.507562, 0.507687, 0.507728,\
        0.507696, 0.507597, 0.507438, 0.507219, 0.506939, 0.506592, 0.506172, 0.505668, 0.505068, 0.504360,\
        0.503530, 0.502569, 0.501470, 0.500230, 0.498859, 0.497389, 0.495837, 0.494240, 0.492699, 0.490818,\
        0.488375, 0.485594, 0.482589, 0.479340, 0.475829, 0.472053, 0.468018, 0.463724, 0.459169, 0.454354,\
        0.449281, 0.443951, 0.438365, 0.432524, 0.426427, 0.420075, 0.413474, 0.406651, 0.399489, 0.391238,\
        0.382650],\
        [0.475163, 0.482332, 0.488788, 0.494198, 0.498098, 0.501302, 0.504272, 0.507200, 0.510346, 0.513821,\
        0.517677, 0.521960, 0.526619, 0.531502, 0.536398, 0.541087, 0.545357, 0.549022, 0.551924, 0.553941,\
        0.554988, 0.555016, 0.554006, 0.551953, 0.548858, 0.544723, 0.539556, 0.533389, 0.526285, 0.518344,\
        0.509671, 0.500252, 0.490261, 0.480338, 0.470821, 0.461821, 0.453318, 0.445249, 0.437565, 0.430256,\
        0.423358, 0.416950, 0.411148, 0.406125, 0.402205, 0.399606, 0.397339, 0.395136, 0.393342, 0.392389,\
        0.392899, 0.395556, 0.400881, 0.409566, 0.420753, 0.433520, 0.446958, 0.460236, 0.472500, 0.482784,\
        0.490030, 0.493728, 0.496006, 0.498210, 0.499705, 0.501149, 0.503507, 0.505465, 0.507142, 0.508579,\
        0.509793, 0.510802, 0.511623, 0.512273, 0.512770, 0.513130, 0.513370, 0.513507, 0.513553, 0.513523,\
        0.513428, 0.513277, 0.513074, 0.512821, 0.512517, 0.512155, 0.511726, 0.511217, 0.510613, 0.509899,\
        0.509058, 0.508078, 0.506948, 0.505662, 0.504228, 0.502675, 0.500988, 0.499230, 0.497843, 0.495914,\
        0.493090, 0.490125, 0.487019, 0.483636, 0.479959, 0.476007, 0.471790, 0.467305, 0.462550, 0.457524,\
        0.452227, 0.446663, 0.440830, 0.434724, 0.428337, 0.421650, 0.414626, 0.407198, 0.399408, 0.391746,\
        0.383578],\
        [0.470829, 0.477823, 0.484070, 0.489347, 0.493591, 0.497176, 0.500462, 0.503679, 0.507280, 0.511212,\
        0.515586, 0.520564, 0.526078, 0.531898, 0.537743, 0.543346, 0.548464, 0.552889, 0.556445, 0.559000,\
        0.560467, 0.560806, 0.560015, 0.558105, 0.555080, 0.550924, 0.545616, 0.539157, 0.531591, 0.523019,\
        0.513585, 0.503494, 0.493199, 0.483177, 0.473737, 0.464936, 0.456665, 0.448793, 0.441238, 0.433993,\
        0.427115, 0.420710, 0.414925, 0.409956, 0.406016, 0.403064, 0.400481, 0.397957, 0.395663, 0.394039,\
        0.393856, 0.396045, 0.401648, 0.410960, 0.423368, 0.437339, 0.451896, 0.466255, 0.479616, 0.490878,\
        0.498840, 0.501962, 0.503814, 0.505825, 0.507379, 0.508869, 0.510734, 0.512536, 0.514096, 0.515418,\
        0.516518, 0.517415, 0.518126, 0.518670, 0.519063, 0.519324, 0.519470, 0.519517, 0.519480, 0.519376,\
        0.519215, 0.519009, 0.518762, 0.518478, 0.518153, 0.517781, 0.517350, 0.516844, 0.516247, 0.515537,\
        0.514698, 0.513710, 0.512562, 0.511246, 0.509767, 0.508135, 0.506364, 0.504587, 0.502957, 0.500629,\
        0.497499, 0.494565, 0.491424, 0.487871, 0.483993, 0.479845, 0.475432, 0.470740, 0.465765, 0.460505,\
        0.454965, 0.449147, 0.443051, 0.436671, 0.429995, 0.423001, 0.415658, 0.407950, 0.400079, 0.392383,\
        0.384321],\
        [0.465819, 0.472674, 0.478826, 0.484167, 0.488721, 0.492690, 0.496321, 0.499896, 0.504083, 0.508279,\
        0.513107, 0.518881, 0.525398, 0.532277, 0.539160, 0.545740, 0.551754, 0.556974, 0.561210, 0.564313,\
        0.566195, 0.566831, 0.566251, 0.564498, 0.561584, 0.557468, 0.552080, 0.545368, 0.537352, 0.528146,\
        0.517976, 0.507220, 0.496448, 0.486268, 0.477000, 0.468532, 0.460596, 0.452968, 0.445544, 0.438338,\
        0.431441, 0.424989, 0.419144, 0.414073, 0.409880, 0.406480, 0.403581, 0.400957, 0.398475, 0.396395,\
        0.395705, 0.397605, 0.402810, 0.413659, 0.427071, 0.441810, 0.456976, 0.471814, 0.485670, 0.497499,\
        0.505176, 0.509270, 0.511610, 0.513540, 0.515183, 0.516710, 0.518296, 0.519840, 0.521218, 0.522390,\
        0.523356, 0.524128, 0.524721, 0.525152, 0.525437, 0.525595, 0.525642, 0.525596, 0.525473, 0.525291,\
        0.525063, 0.524800, 0.524511, 0.524197, 0.523856, 0.523480, 0.523055, 0.522562, 0.521981, 0.521288,\
        0.520460, 0.519477, 0.518322, 0.516986, 0.515468, 0.513776, 0.511937, 0.509996, 0.507869, 0.505194,\
        0.502091, 0.499203, 0.495884, 0.492048, 0.487936, 0.483586, 0.478970, 0.474057, 0.468841, 0.463329,\
        0.457528, 0.451446, 0.445083, 0.438438, 0.431505, 0.424281, 0.416781, 0.409091, 0.401365, 0.393313,\
        0.385051],\
        [0.460334, 0.467131, 0.473312, 0.478824, 0.483703, 0.488089, 0.492215, 0.496456, 0.500766, 0.504953,\
        0.510294, 0.517105, 0.524802, 0.532831, 0.540789, 0.548363, 0.555285, 0.561316, 0.566244, 0.569898,\
        0.572177, 0.573076, 0.572680, 0.571098, 0.568365, 0.564402, 0.559056, 0.552199, 0.543819, 0.534056,\
        0.523206, 0.511730, 0.500317, 0.489970, 0.480955, 0.472919, 0.465361, 0.457942, 0.450569, 0.443309,\
        0.436297, 0.429698, 0.423676, 0.418368, 0.413839, 0.410036, 0.406812, 0.404024, 0.401571, 0.399654,\
        0.398740, 0.401578, 0.408222, 0.418842, 0.432321, 0.447190, 0.462386, 0.477003, 0.490161, 0.501245,\
        0.509793, 0.515167, 0.518561, 0.520954, 0.522826, 0.524425, 0.525893, 0.527246, 0.528442, 0.529455,\
        0.530280, 0.530922, 0.531395, 0.531711, 0.531888, 0.531941, 0.531889, 0.531748, 0.531538, 0.531276,\
        0.530979, 0.530659, 0.530327, 0.529987, 0.529634, 0.529260, 0.528850, 0.528380, 0.527826, 0.527160,\
        0.526354, 0.525384, 0.524230, 0.522877, 0.521319, 0.519560, 0.517604, 0.515443, 0.512999, 0.510177,\
        0.507092, 0.503892, 0.500238, 0.496117, 0.491798, 0.487266, 0.482441, 0.477284, 0.471803, 0.466018,\
        0.459943, 0.453590, 0.446965, 0.440070, 0.432908, 0.425491, 0.417848, 0.410033, 0.402063, 0.393950,\
        0.385646],\
        [0.454522, 0.461353, 0.467667, 0.473437, 0.478692, 0.483522, 0.488078, 0.492503, 0.496779, 0.501178,\
        0.507480, 0.515693, 0.524645, 0.533788, 0.542763, 0.551280, 0.559081, 0.565915, 0.571539, 0.575743,\
        0.578392, 0.579492, 0.579221, 0.577818, 0.575369, 0.571732, 0.566616, 0.559771, 0.551155, 0.540963,\
        0.529571, 0.517508, 0.505528, 0.494834, 0.486033, 0.478468, 0.471222, 0.463848, 0.456352, 0.448883,\
        0.441626, 0.434766, 0.428468, 0.422856, 0.417989, 0.413853, 0.410380, 0.407505, 0.405243, 0.403848,\
        0.403928, 0.407567, 0.414994, 0.425687, 0.438901, 0.453516, 0.468414, 0.482584, 0.495183, 0.505656,\
        0.514339, 0.520664, 0.524974, 0.527978, 0.530192, 0.531933, 0.533385, 0.534629, 0.535686, 0.536560,\
        0.537253, 0.537774, 0.538131, 0.538340, 0.538412, 0.538365, 0.538215, 0.537982, 0.537684, 0.537341,\
        0.536973, 0.536596, 0.536222, 0.535857, 0.535497, 0.535133, 0.534745, 0.534308, 0.533791, 0.533162,\
        0.532386, 0.531435, 0.530283, 0.528912, 0.527308, 0.525465, 0.523376, 0.521018, 0.518355, 0.515365,\
        0.512085, 0.508511, 0.504527, 0.500168, 0.495661, 0.490955, 0.485888, 0.480443, 0.474664, 0.468584,\
        0.462224, 0.455598, 0.448716, 0.441583, 0.434209, 0.426607, 0.418800, 0.410813, 0.402679, 0.394459,\
        0.386113],\
        [0.448481, 0.455430, 0.461971, 0.468091, 0.473804, 0.479151, 0.484188, 0.488961, 0.493552, 0.498561,\
        0.506101, 0.515393, 0.525301, 0.535331, 0.545152, 0.554493, 0.563098, 0.570702, 0.577022, 0.581787,\
        0.584791, 0.586000, 0.585742, 0.584512, 0.582483, 0.579409, 0.574769, 0.568107, 0.559375, 0.548893,\
        0.537166, 0.524807, 0.512580, 0.501497, 0.492647, 0.485567, 0.478341, 0.470683, 0.462819, 0.454966,\
        0.447333, 0.440117, 0.433483, 0.427551, 0.422390, 0.418020, 0.414438, 0.411665, 0.409823, 0.409241,\
        0.410633, 0.415004, 0.422690, 0.433337, 0.446274, 0.460570, 0.475147, 0.488903, 0.501146, 0.511456,\
        0.519901, 0.526475, 0.531306, 0.534786, 0.537322, 0.539231, 0.540723, 0.541920, 0.542888, 0.543656,\
        0.544242, 0.544658, 0.544916, 0.545028, 0.545007, 0.544868, 0.544627, 0.544305, 0.543921, 0.543498,\
        0.543059, 0.542624, 0.542208, 0.541820, 0.541458, 0.541109, 0.540752, 0.540357, 0.539887, 0.539302,\
        0.538563, 0.537634, 0.536483, 0.535087, 0.533428, 0.531492, 0.529264, 0.526727, 0.523861, 0.520659,\
        0.517123, 0.513242, 0.508990, 0.504421, 0.499680, 0.494726, 0.489322, 0.483525, 0.477415, 0.471026,\
        0.464375, 0.457477, 0.450342, 0.442979, 0.435402, 0.427623, 0.419660, 0.411530, 0.403262, 0.394932,\
        0.386510],\
        [0.442264, 0.449403, 0.456255, 0.462821, 0.469109, 0.475133, 0.480912, 0.486500, 0.492150, 0.498624,\
        0.506794, 0.516468, 0.526877, 0.537475, 0.547911, 0.557906, 0.567202, 0.575521, 0.582531, 0.587870,\
        0.591269, 0.592493, 0.592067, 0.591020, 0.589560, 0.587326, 0.583445, 0.577077, 0.568272, 0.557608,\
        0.545743, 0.533374, 0.521253, 0.510211, 0.501390, 0.494382, 0.486469, 0.478170, 0.469747, 0.461379,\
        0.453278, 0.445650, 0.438663, 0.432436, 0.427047, 0.422550, 0.418996, 0.416473, 0.415169, 0.415429,\
        0.417789, 0.422800, 0.430686, 0.441234, 0.453913, 0.467949, 0.482361, 0.495907, 0.507645, 0.517711,\
        0.526049, 0.532696, 0.537776, 0.541545, 0.544313, 0.546361, 0.547907, 0.549094, 0.550011, 0.550710,\
        0.551219, 0.551556, 0.551736, 0.551770, 0.551672, 0.551454, 0.551135, 0.550732, 0.550267, 0.549765,\
        0.549254, 0.548759, 0.548302, 0.547894, 0.547533, 0.547206, 0.546887, 0.546541, 0.546126, 0.545592,\
        0.544893, 0.543985, 0.542830, 0.541402, 0.539677, 0.537639, 0.535269, 0.532553, 0.529479, 0.526041,\
        0.522242, 0.518084, 0.513584, 0.508799, 0.503808, 0.498512, 0.492650, 0.486476, 0.480034, 0.473334,\
        0.466392, 0.459223, 0.451840, 0.444252, 0.436477, 0.428530, 0.420429, 0.412190, 0.403827, 0.395387,\
        0.386846],\
        [0.435887, 0.443268, 0.450496, 0.457593, 0.464577, 0.471462, 0.478259, 0.485010, 0.491900, 0.499500,\
        0.508410, 0.518435, 0.529105, 0.540022, 0.550857, 0.561327, 0.571171, 0.580113, 0.587796, 0.593672,\
        0.597486, 0.598791, 0.598103, 0.597366, 0.596525, 0.595280, 0.592442, 0.586274, 0.577359, 0.566602,\
        0.554754, 0.542566, 0.530776, 0.520131, 0.511229, 0.503251, 0.494665, 0.485795, 0.476807, 0.467882,\
        0.459281, 0.451238, 0.443923, 0.437453, 0.431914, 0.427383, 0.423948, 0.421743, 0.420972, 0.421936,\
        0.425003, 0.430495, 0.438531, 0.448960, 0.461378, 0.475150, 0.489393, 0.502909, 0.514401, 0.524276,\
        0.532564, 0.539252, 0.544444, 0.548352, 0.551241, 0.553371, 0.554956, 0.556146, 0.557041, 0.557703,\
        0.558166, 0.558454, 0.558583, 0.558564, 0.558410, 0.558134, 0.557751, 0.557281, 0.556743, 0.556166,\
        0.555582, 0.555027, 0.554528, 0.554100, 0.553744, 0.553443, 0.553168, 0.552878, 0.552523, 0.552045,\
        0.551386, 0.550493, 0.549326, 0.547853, 0.546052, 0.543900, 0.541380, 0.538477, 0.535183, 0.531497,\
        0.527424, 0.522977, 0.518186, 0.513089, 0.507703, 0.501928, 0.495725, 0.489259, 0.482506, 0.475498,\
        0.468268, 0.460834, 0.453205, 0.445392, 0.437415, 0.429298, 0.421065, 0.412738, 0.404317, 0.395767,\
        0.387069],\
        [0.429341, 0.436988, 0.444625, 0.452301, 0.460059, 0.467928, 0.475928, 0.484070, 0.492320, 0.500845,\
        0.510451, 0.520727, 0.531535, 0.542634, 0.553727, 0.564522, 0.574745, 0.584134, 0.592401, 0.598914,\
        0.602696, 0.604382, 0.604364, 0.604055, 0.603653, 0.602973, 0.600863, 0.594716, 0.585907, 0.575227,\
        0.563523, 0.551605, 0.540124, 0.529585, 0.520148, 0.511301, 0.502363, 0.493150, 0.483689, 0.474216,\
        0.465143, 0.456742, 0.449170, 0.442532, 0.436916, 0.432415, 0.429147, 0.427260, 0.426956, 0.428474,\
        0.432062, 0.437899, 0.446023, 0.456294, 0.468415, 0.481885, 0.495842, 0.509125, 0.520899, 0.530980,\
        0.539374, 0.546113, 0.551329, 0.555252, 0.558154, 0.560296, 0.561889, 0.563083, 0.563976, 0.564627,\
        0.565075, 0.565345, 0.565454, 0.565411, 0.565227, 0.564916, 0.564494, 0.563976, 0.563379, 0.562731,\
        0.562076, 0.561459, 0.560917, 0.560471, 0.560122, 0.559851, 0.559622, 0.559391, 0.559099, 0.558677,\
        0.558053, 0.557166, 0.555969, 0.554436, 0.552542, 0.550265, 0.547584, 0.544483, 0.540955, 0.537002,\
        0.532634, 0.527870, 0.522734, 0.517250, 0.511420, 0.505213, 0.498702, 0.491927, 0.484830, 0.477509,\
        0.470001, 0.462308, 0.454433, 0.446390, 0.438200, 0.429892, 0.421503, 0.413062, 0.404577, 0.395922,\
        0.387072],\
        [0.422600, 0.430503, 0.438539, 0.446794, 0.455329, 0.464186, 0.473405, 0.483015, 0.492985, 0.502796,\
        0.512419, 0.522739, 0.533664, 0.544938, 0.556253, 0.567294, 0.577755, 0.587346, 0.595774, 0.602615,\
        0.607085, 0.609500, 0.610482, 0.610742, 0.610562, 0.609687, 0.607071, 0.601548, 0.593371, 0.582870,\
        0.571459, 0.559904, 0.548693, 0.538125, 0.528247, 0.518807, 0.509473, 0.500029, 0.490111, 0.480116,\
        0.470688, 0.462055, 0.454332, 0.447609, 0.441978, 0.437547, 0.434449, 0.432847, 0.432925, 0.434876,\
        0.438864, 0.444970, 0.453153, 0.463246, 0.475017, 0.488199, 0.502063, 0.515425, 0.527545, 0.537976,\
        0.546557, 0.553321, 0.558460, 0.562269, 0.565073, 0.567152, 0.568719, 0.569914, 0.570819, 0.571485,\
        0.571944, 0.572226, 0.572347, 0.572313, 0.572130, 0.571814, 0.571384, 0.570849, 0.570214, 0.569504,\
        0.568777, 0.568097, 0.567513, 0.567052, 0.566714, 0.566472, 0.566287, 0.566109, 0.565877, 0.565508,\
        0.564911, 0.564007, 0.562754, 0.561135, 0.559133, 0.556719, 0.553863, 0.550549, 0.546769, 0.542531,\
        0.537846, 0.532734, 0.527218, 0.521321, 0.515065, 0.508490, 0.501666, 0.494458, 0.486969, 0.479359,\
        0.471596, 0.463653, 0.455530, 0.447244, 0.438821, 0.430294, 0.421698, 0.413070, 0.404412, 0.395651,\
        0.386763],\
        [0.415637, 0.423748, 0.432134, 0.440918, 0.450162, 0.459892, 0.470126, 0.480861, 0.491974, 0.502770,\
        0.513034, 0.523756, 0.535022, 0.546614, 0.558229, 0.569542, 0.580221, 0.589947, 0.598433, 0.605539,\
        0.610785, 0.614115, 0.615943, 0.616695, 0.616539, 0.615246, 0.612216, 0.606934, 0.599033, 0.589091,\
        0.578262, 0.567239, 0.556353, 0.545795, 0.535605, 0.525676, 0.515846, 0.505934, 0.495577, 0.485392,\
        0.475848, 0.467153, 0.459397, 0.452663, 0.447053, 0.442698, 0.439746, 0.438370, 0.438747, 0.441042,\
        0.445368, 0.451742, 0.460053, 0.470056, 0.481466, 0.494237, 0.508301, 0.522137, 0.534749, 0.545555,\
        0.554288, 0.560968, 0.565870, 0.569410, 0.571995, 0.573941, 0.575452, 0.576648, 0.577582, 0.578281,\
        0.578772, 0.579092, 0.579263, 0.579277, 0.579127, 0.578836, 0.578439, 0.577937, 0.577298, 0.576535,\
        0.575737, 0.575000, 0.574378, 0.573906, 0.573584, 0.573370, 0.573212, 0.573070, 0.572885, 0.572563,\
        0.571976, 0.571020, 0.569663, 0.567924, 0.565797, 0.563237, 0.560195, 0.556650, 0.552600, 0.548058,\
        0.543037, 0.537556, 0.531640, 0.525314, 0.518610, 0.511569, 0.504231, 0.496598, 0.488886, 0.481074,\
        0.473081, 0.464887, 0.456504, 0.447960, 0.439286, 0.430513, 0.421673, 0.412802, 0.403934, 0.395073,\
        0.386190],\
        [0.408432, 0.416670, 0.425333, 0.434579, 0.444428, 0.454856, 0.465822, 0.477238, 0.488872, 0.500367,\
        0.511803, 0.523410, 0.535333, 0.547461, 0.559535, 0.571239, 0.582249, 0.592265, 0.601044, 0.608480,\
        0.614257, 0.618286, 0.620743, 0.621846, 0.621661, 0.620024, 0.616594, 0.611062, 0.603406, 0.594093,\
        0.584084, 0.573767, 0.563278, 0.552800, 0.542395, 0.532033, 0.521630, 0.511085, 0.500482, 0.490252,\
        0.480757, 0.472139, 0.464429, 0.457716, 0.452131, 0.447825, 0.444966, 0.443741, 0.444333, 0.446900,\
        0.451538, 0.458242, 0.466867, 0.477101, 0.488530, 0.501115, 0.515399, 0.529733, 0.542818, 0.553951,\
        0.562729, 0.569134, 0.573568, 0.576647, 0.578899, 0.580655, 0.582091, 0.583295, 0.584282, 0.585028,\
        0.585555, 0.585932, 0.586195, 0.586308, 0.586225, 0.585982, 0.585670, 0.585283, 0.584699, 0.583883,\
        0.583020, 0.582254, 0.581599, 0.581121, 0.580831, 0.580637, 0.580471, 0.580320, 0.580153, 0.579866,\
        0.579270, 0.578197, 0.576653, 0.574751, 0.572500, 0.569794, 0.566551, 0.562753, 0.558414, 0.553552,\
        0.548181, 0.542320, 0.535996, 0.529239, 0.522080, 0.514545, 0.506664, 0.498631, 0.490770, 0.482760,\
        0.474509, 0.466031, 0.457363, 0.448544, 0.439610, 0.430589, 0.421502, 0.412375, 0.403299, 0.394352,\
        0.385479],\
        [0.400972, 0.409217, 0.418127, 0.427783, 0.438138, 0.449103, 0.460588, 0.472488, 0.484640, 0.496893,\
        0.509269, 0.521813, 0.534569, 0.547420, 0.560122, 0.572374, 0.583860, 0.594313, 0.603550, 0.611350,\
        0.617565, 0.622106, 0.625022, 0.626395, 0.626227, 0.624397, 0.620708, 0.615013, 0.607387, 0.598431,\
        0.589603, 0.579917, 0.569801, 0.559455, 0.548912, 0.538205, 0.527344, 0.516366, 0.505463, 0.495082,\
        0.485722, 0.477232, 0.469550, 0.462831, 0.457232, 0.452916, 0.450070, 0.448902, 0.449615, 0.452383,\
        0.457310, 0.464404, 0.473542, 0.484436, 0.496547, 0.509655, 0.523918, 0.538378, 0.551806, 0.563226,\
        0.571953, 0.577833, 0.581496, 0.583914, 0.585756, 0.587302, 0.588653, 0.589871, 0.590939, 0.591739,\
        0.592282, 0.592722, 0.593125, 0.593407, 0.593428, 0.593243, 0.593076, 0.592921, 0.592499, 0.591598,\
        0.590693, 0.590005, 0.589302, 0.588818, 0.588598, 0.588412, 0.588164, 0.587924, 0.587713, 0.587436,\
        0.586818, 0.585514, 0.583627, 0.581529, 0.579201, 0.576367, 0.572893, 0.568810, 0.564164, 0.558973,\
        0.553245, 0.546999, 0.540269, 0.533095, 0.525521, 0.517596, 0.509394, 0.501098, 0.492886, 0.484508,\
        0.475901, 0.467082, 0.458095, 0.448986, 0.439795, 0.430552, 0.421271, 0.411945, 0.402594, 0.393621,\
        0.384764],\
        [0.393172, 0.401498, 0.410677, 0.420664, 0.431437, 0.442793, 0.454641, 0.466940, 0.479603, 0.492557,\
        0.505738, 0.519159, 0.532827, 0.546518, 0.559975, 0.572920, 0.585038, 0.596005, 0.605724, 0.613997,\
        0.620633, 0.625604, 0.628911, 0.630563, 0.630527, 0.628715, 0.625026, 0.619441, 0.612198, 0.604152,\
        0.595582, 0.585961, 0.576284, 0.566115, 0.555476, 0.544520, 0.533353, 0.522089, 0.510939, 0.500401,\
        0.491233, 0.482666, 0.474870, 0.468073, 0.462397, 0.457988, 0.455049, 0.453822, 0.454548, 0.457429,\
        0.462596, 0.470075, 0.479769, 0.491443, 0.504503, 0.518244, 0.532818, 0.547534, 0.561352, 0.573155,\
        0.581877, 0.586966, 0.589500, 0.591130, 0.592592, 0.593955, 0.595195, 0.596395, 0.597560, 0.598404,\
        0.598942, 0.599447, 0.600017, 0.600511, 0.600706, 0.600658, 0.600703, 0.600838, 0.600683, 0.599731,\
        0.598903, 0.598474, 0.597669, 0.597193, 0.597085, 0.596871, 0.596431, 0.596004, 0.595627, 0.595261,\
        0.594619, 0.592890, 0.590387, 0.588148, 0.585903, 0.582937, 0.579156, 0.574748, 0.569792, 0.564275,\
        0.558188, 0.551550, 0.544411, 0.536834, 0.528887, 0.520648, 0.512217, 0.503709, 0.495059, 0.486225,\
        0.477195, 0.467987, 0.458655, 0.449248, 0.439809, 0.430377, 0.420988, 0.411667, 0.402414, 0.393246,\
        0.384123],\
        [0.385564, 0.394064, 0.403255, 0.413454, 0.424580, 0.436110, 0.448150, 0.460770, 0.473896, 0.487414,\
        0.501282, 0.515637, 0.530264, 0.544813, 0.559065, 0.572791, 0.585706, 0.597445, 0.607662, 0.616305,\
        0.623350, 0.628753, 0.632479, 0.634495, 0.634755, 0.633197, 0.629780, 0.624555, 0.617764, 0.609803,\
        0.600713, 0.592125, 0.583227, 0.573109, 0.562330, 0.551197, 0.539851, 0.528452, 0.517309, 0.506925,\
        0.497440, 0.488398, 0.480395, 0.473494, 0.467682, 0.463075, 0.459914, 0.458492, 0.459104, 0.462001,\
        0.467341, 0.475164, 0.485357, 0.497606, 0.511453, 0.526150, 0.541327, 0.556466, 0.570752, 0.583091,\
        0.592064, 0.596211, 0.597421, 0.598366, 0.599595, 0.600785, 0.601873, 0.602965, 0.604057, 0.604920,\
        0.605549, 0.606174, 0.606864, 0.607516, 0.607994, 0.608327, 0.608679, 0.609012, 0.609030, 0.608530,\
        0.608032, 0.607659, 0.607055, 0.606632, 0.606438, 0.606089, 0.605483, 0.604802, 0.604105, 0.603369,\
        0.602374, 0.599927, 0.596757, 0.594579, 0.592713, 0.589435, 0.585198, 0.580461, 0.575235, 0.569413,\
        0.562957, 0.555910, 0.548349, 0.540362, 0.532041, 0.523481, 0.514775, 0.505946, 0.496946, 0.487737,\
        0.478265, 0.468652, 0.458973, 0.449274, 0.439592, 0.429974, 0.420476, 0.411171, 0.402109, 0.392725,\
        0.383317],\
        [0.378224, 0.386916, 0.396104, 0.406776, 0.417846, 0.429095, 0.441156, 0.454080, 0.467704, 0.481838,\
        0.496395, 0.511636, 0.527072, 0.542342, 0.557312, 0.571790, 0.585514, 0.598097, 0.608877, 0.617992,\
        0.625549, 0.631477, 0.635722, 0.638239, 0.638987, 0.637920, 0.635022, 0.630345, 0.624042, 0.616339,\
        0.607824, 0.600254, 0.590989, 0.580544, 0.569574, 0.558353, 0.546895, 0.535367, 0.524100, 0.513435,\
        0.503420, 0.494141, 0.486135, 0.479184, 0.473164, 0.468225, 0.464685, 0.462915, 0.463281, 0.466096,\
        0.471562, 0.479726, 0.490460, 0.503461, 0.518067, 0.533513, 0.549210, 0.564648, 0.579180, 0.591894,\
        0.600857, 0.604659, 0.605540, 0.606196, 0.607024, 0.607920, 0.608787, 0.609671, 0.610563, 0.611377,\
        0.612116, 0.612883, 0.613721, 0.614585, 0.615405, 0.616172, 0.616909, 0.617547, 0.617925, 0.617979,\
        0.617872, 0.617673, 0.617337, 0.616981, 0.616613, 0.616070, 0.615279, 0.614293, 0.613145, 0.611763,\
        0.609840, 0.606893, 0.603505, 0.601244, 0.599516, 0.595502, 0.590777, 0.585862, 0.580471, 0.574352,\
        0.567494, 0.559999, 0.551986, 0.543563, 0.534835, 0.525904, 0.516872, 0.507810, 0.498591, 0.488883,\
        0.478941, 0.468963, 0.458978, 0.449009, 0.439090, 0.429261, 0.419572, 0.410056, 0.400683, 0.391400,\
        0.382082],\
        [0.371003, 0.380082, 0.390077, 0.401025, 0.410746, 0.421436, 0.433585, 0.446911, 0.461133, 0.476045,\
        0.491522, 0.507415, 0.523285, 0.539041, 0.554605, 0.569724, 0.584090, 0.597294, 0.608875, 0.618797,\
        0.627077, 0.633677, 0.638578, 0.641761, 0.643195, 0.642847, 0.640704, 0.636804, 0.631273, 0.624390,\
        0.616705, 0.608516, 0.599080, 0.588208, 0.577252, 0.566112, 0.554514, 0.542723, 0.531144, 0.520082,\
        0.509690, 0.500311, 0.492333, 0.485291, 0.478916, 0.473470, 0.469383, 0.467104, 0.467086, 0.469735,\
        0.475327, 0.483924, 0.495353, 0.509255, 0.524704, 0.540706, 0.556627, 0.571995, 0.586240, 0.598391,\
        0.607133, 0.611542, 0.613280, 0.614070, 0.614663, 0.615242, 0.615835, 0.616474, 0.617169, 0.617910,\
        0.618710, 0.619613, 0.620645, 0.621782, 0.622980, 0.624199, 0.625394, 0.626478, 0.627339, 0.627918,\
        0.628240, 0.628331, 0.628207, 0.627901, 0.627405, 0.626649, 0.625583, 0.624207, 0.622516, 0.620433,\
        0.617801, 0.614564, 0.611159, 0.608257, 0.605202, 0.600751, 0.595853, 0.591053, 0.585557, 0.579055,\
        0.571707, 0.563709, 0.555211, 0.546328, 0.537160, 0.527789, 0.518301, 0.508802, 0.499293, 0.489209,\
        0.479040, 0.468843, 0.458629, 0.448431, 0.438289, 0.428243, 0.418333, 0.408588, 0.399046, 0.389750,\
        0.380531],\
        [0.363597, 0.372780, 0.382654, 0.392610, 0.401990, 0.412925, 0.425483, 0.439320, 0.454149, 0.469769,\
        0.486026, 0.502524, 0.518540, 0.534749, 0.550876, 0.566568, 0.581502, 0.595324, 0.607810, 0.618720,\
        0.627861, 0.635249, 0.640940, 0.644951, 0.647268, 0.647857, 0.646691, 0.643785, 0.639216, 0.633160,\
        0.625842, 0.617278, 0.607219, 0.596064, 0.585792, 0.574744, 0.562741, 0.550470, 0.538474, 0.527103,\
        0.516603, 0.507233, 0.499157, 0.491864, 0.484895, 0.478787, 0.474019, 0.471085, 0.470539, 0.472932,\
        0.478691, 0.487938, 0.500419, 0.515505, 0.531827, 0.548216, 0.564035, 0.578943, 0.592569, 0.604104,\
        0.612543, 0.617695, 0.620324, 0.621560, 0.622183, 0.622584, 0.622945, 0.623362, 0.623889, 0.624558,\
        0.625396, 0.626435, 0.627686, 0.629134, 0.630737, 0.632435, 0.634147, 0.635764, 0.637168, 0.638269,\
        0.639023, 0.639419, 0.639470, 0.639194, 0.638583, 0.637596, 0.636196, 0.634369, 0.632104, 0.629377,\
        0.626165, 0.622547, 0.618772, 0.615012, 0.610930, 0.606140, 0.601151, 0.596368, 0.590510, 0.583380,\
        0.575439, 0.566901, 0.557906, 0.548564, 0.538958, 0.529140, 0.519128, 0.508899, 0.498605, 0.488758,\
        0.478642, 0.468334, 0.457943, 0.447549, 0.437211, 0.426975, 0.416882, 0.406984, 0.397363, 0.388027,\
        0.378841],\
        [0.355866, 0.364806, 0.374122, 0.383508, 0.393162, 0.404256, 0.417211, 0.431469, 0.446755, 0.462830,\
        0.479366, 0.495904, 0.512542, 0.529477, 0.546154, 0.562358, 0.577936, 0.592577, 0.606013, 0.617867,\
        0.627836, 0.636055, 0.642646, 0.647647, 0.651034, 0.652762, 0.652781, 0.651056, 0.647586, 0.642410,\
        0.635569, 0.627062, 0.616974, 0.606260, 0.596195, 0.584300, 0.571414, 0.558459, 0.545993, 0.534344,\
        0.523735, 0.514327, 0.506111, 0.498404, 0.490847, 0.484117, 0.478625, 0.474920, 0.473679, 0.475674,\
        0.481606, 0.491812, 0.505870, 0.522548, 0.539992, 0.556668, 0.572088, 0.586284, 0.599037, 0.610095,\
        0.618468, 0.624015, 0.627222, 0.628851, 0.629590, 0.629912, 0.630107, 0.630350, 0.630746, 0.631360,\
        0.632237, 0.633411, 0.634892, 0.636668, 0.638693, 0.640893, 0.643159, 0.645356, 0.647333, 0.648951,\
        0.650114, 0.650778, 0.650956, 0.650677, 0.649940, 0.648705, 0.646934, 0.644614, 0.641761, 0.638403,\
        0.634597, 0.630446, 0.626092, 0.621592, 0.616844, 0.611799, 0.606689, 0.601517, 0.594798, 0.586980,\
        0.578471, 0.569416, 0.559950, 0.550185, 0.540200, 0.530032, 0.519686, 0.509182, 0.498716, 0.488417,\
        0.478057, 0.467549, 0.456964, 0.446387, 0.435885, 0.425506, 0.415286, 0.405289, 0.395623, 0.386268,\
        0.377090],\
        [0.347960, 0.356622, 0.365619, 0.374940, 0.384894, 0.396176, 0.409160, 0.423532, 0.439080, 0.455482,\
        0.472291, 0.489099, 0.506232, 0.523637, 0.540477, 0.556957, 0.573219, 0.588941, 0.603494, 0.616101,\
        0.626777, 0.635865, 0.643487, 0.649642, 0.654282, 0.657338, 0.658730, 0.658372, 0.656177, 0.652066,\
        0.645993, 0.638007, 0.628424, 0.617968, 0.606800, 0.593961, 0.580113, 0.566482, 0.553561, 0.541616,\
        0.530800, 0.521154, 0.512504, 0.504402, 0.496685, 0.489492, 0.483323, 0.478762, 0.476623, 0.477970,\
        0.483923, 0.495222, 0.511487, 0.530657, 0.549869, 0.566800, 0.581368, 0.594927, 0.607044, 0.617443,\
        0.625524, 0.631103, 0.634506, 0.636299, 0.637075, 0.637322, 0.637386, 0.637497, 0.637798, 0.638382,\
        0.639309, 0.640618, 0.642330, 0.644433, 0.646881, 0.649586, 0.652419, 0.655210, 0.657764, 0.659885,\
        0.661411, 0.662268, 0.662491, 0.662159, 0.661280, 0.659784, 0.657610, 0.654760, 0.651284, 0.647263,\
        0.642798, 0.638004, 0.632987, 0.627799, 0.622439, 0.616914, 0.611219, 0.605022, 0.597774, 0.589595,\
        0.580635, 0.571115, 0.561224, 0.551095, 0.540813, 0.530423, 0.519944, 0.509386, 0.498763, 0.488071,\
        0.477319, 0.466512, 0.455698, 0.444943, 0.434312, 0.423852, 0.413583, 0.403529, 0.393852, 0.384488,\
        0.375297],\
        [0.340084, 0.348557, 0.357437, 0.366830, 0.377016, 0.388403, 0.401230, 0.415519, 0.431226, 0.447955,\
        0.465247, 0.482774, 0.500279, 0.517449, 0.533523, 0.549918, 0.566821, 0.583657, 0.599472, 0.612761,\
        0.624237, 0.634392, 0.643241, 0.650729, 0.656788, 0.661337, 0.664274, 0.665468, 0.664753, 0.661940,\
        0.656884, 0.649588, 0.640306, 0.629429, 0.617087, 0.603204, 0.588690, 0.574518, 0.561190, 0.548920,\
        0.537817, 0.527859, 0.518843, 0.510473, 0.502552, 0.495064, 0.488339, 0.482886, 0.479624, 0.479980,\
        0.485649, 0.498085, 0.517281, 0.540056, 0.562185, 0.579692, 0.592038, 0.605019, 0.616773, 0.626527,\
        0.634049, 0.639298, 0.642529, 0.644205, 0.644857, 0.644965, 0.644899, 0.644908, 0.645152, 0.645733,\
        0.646721, 0.648165, 0.650094, 0.652505, 0.655355, 0.658544, 0.661920, 0.665287, 0.668398, 0.670988,\
        0.672813, 0.673742, 0.673877, 0.673424, 0.672402, 0.670641, 0.668028, 0.664596, 0.660446, 0.655707,\
        0.650518, 0.645005, 0.639268, 0.633366, 0.627317, 0.621110, 0.614657, 0.607701, 0.599926, 0.591305,\
        0.581865, 0.571895, 0.561622, 0.551189, 0.540681, 0.530152, 0.519625, 0.509071, 0.498335, 0.487335,\
        0.476234, 0.465137, 0.454102, 0.443186, 0.432454, 0.421975, 0.411788, 0.401806, 0.392113, 0.382674,\
        0.373444],\
        [0.332364, 0.340726, 0.349580, 0.359058, 0.369364, 0.380727, 0.393323, 0.407444, 0.423252, 0.440163,\
        0.457706, 0.475699, 0.493958, 0.509590, 0.524244, 0.540753, 0.558394, 0.576029, 0.592573, 0.607013,\
        0.619888, 0.631464, 0.641760, 0.650739, 0.658345, 0.664506, 0.669118, 0.672028, 0.673003, 0.671732,\
        0.667899, 0.661367, 0.652288, 0.640988, 0.627768, 0.613028, 0.597675, 0.582876, 0.569126, 0.556441,\
        0.544999, 0.534758, 0.525464, 0.516836, 0.508703, 0.501051, 0.493953, 0.487644, 0.483084, 0.482111,\
        0.487156, 0.500740, 0.523432, 0.550465, 0.576629, 0.596378, 0.606849, 0.617784, 0.628546, 0.637367,\
        0.644081, 0.648706, 0.651460, 0.652756, 0.653106, 0.652995, 0.652789, 0.652729, 0.652958, 0.653570,\
        0.654634, 0.656207, 0.658328, 0.661010, 0.664218, 0.667833, 0.671686, 0.675559, 0.679157, 0.682151,\
        0.684204, 0.685034, 0.684866, 0.684214, 0.683107, 0.681077, 0.677959, 0.673886, 0.669007, 0.663496,\
        0.657530, 0.651251, 0.644756, 0.638094, 0.631281, 0.624294, 0.617067, 0.609485, 0.601332, 0.592076,\
        0.582054, 0.571657, 0.561059, 0.550378, 0.539695, 0.529052, 0.518468, 0.507931, 0.497170, 0.485930,\
        0.474607, 0.463316, 0.452118, 0.441073, 0.430250, 0.419738, 0.409665, 0.400062, 0.390221, 0.380698,\
        0.371466],\
        [0.324849, 0.333140, 0.341999, 0.351561, 0.361970, 0.373355, 0.385844, 0.399729, 0.415348, 0.432044,\
        0.449061, 0.466180, 0.482759, 0.497303, 0.512114, 0.529871, 0.548460, 0.566710, 0.583875, 0.599578,\
        0.614019, 0.627171, 0.639023, 0.649569, 0.658779, 0.666600, 0.672947, 0.677672, 0.680518, 0.681044,\
        0.678671, 0.673034, 0.664261, 0.652800, 0.639133, 0.623809, 0.607666, 0.592200, 0.577782, 0.564416,\
        0.552611, 0.542171, 0.532656, 0.523755, 0.515360, 0.507511, 0.500231, 0.493252, 0.487411, 0.484926,\
        0.489208, 0.504154, 0.530230, 0.560965, 0.590118, 0.611576, 0.623883, 0.633399, 0.642208, 0.649689,\
        0.655417, 0.659251, 0.661320, 0.662026, 0.661924, 0.661536, 0.661213, 0.661140, 0.661420, 0.662108,\
        0.663267, 0.664973, 0.667246, 0.670139, 0.673650, 0.677596, 0.681785, 0.686025, 0.689952, 0.693160,\
        0.695376, 0.695916, 0.695191, 0.694255, 0.693216, 0.690833, 0.687110, 0.682362, 0.676716, 0.670392,\
        0.663629, 0.656576, 0.649313, 0.641872, 0.634256, 0.626432, 0.618346, 0.609944, 0.601131, 0.591404,\
        0.581032, 0.570341, 0.559497, 0.548625, 0.537808, 0.527067, 0.516362, 0.505642, 0.494826, 0.483681,\
        0.472354, 0.461002, 0.449725, 0.438604, 0.427705, 0.417090, 0.406824, 0.397118, 0.387718, 0.378423,\
        0.369321],\
        [0.317546, 0.325766, 0.334631, 0.344281, 0.354825, 0.366353, 0.378943, 0.392696, 0.407947, 0.423826,\
        0.439485, 0.455029, 0.470154, 0.484596, 0.500424, 0.519063, 0.538076, 0.556696, 0.574533, 0.591359,\
        0.607130, 0.621724, 0.635088, 0.647173, 0.657957, 0.667404, 0.675444, 0.681961, 0.686748, 0.689323,\
        0.688710, 0.684152, 0.675892, 0.664664, 0.651066, 0.635652, 0.619262, 0.603075, 0.587252, 0.572924,\
        0.560972, 0.550486, 0.540739, 0.531425, 0.522544, 0.514236, 0.506592, 0.499424, 0.492799, 0.488841,\
        0.492199, 0.508925, 0.538060, 0.571541, 0.601947, 0.624201, 0.638600, 0.648470, 0.656292, 0.662739,\
        0.667645, 0.670729, 0.672032, 0.672009, 0.671354, 0.670699, 0.670352, 0.670360, 0.670795, 0.671629,\
        0.672890, 0.674783, 0.677169, 0.680160, 0.683959, 0.688108, 0.692403, 0.696735, 0.700730, 0.703748,\
        0.705695, 0.705950, 0.704905, 0.703616, 0.702324, 0.699374, 0.695097, 0.689753, 0.683306, 0.676162,\
        0.668639, 0.660857, 0.652858, 0.644658, 0.636248, 0.627591, 0.618606, 0.609196, 0.599404, 0.589429,\
        0.578916, 0.568026, 0.556979, 0.545947, 0.535044, 0.524285, 0.513535, 0.502677, 0.491836, 0.480817,\
        0.469565, 0.458232, 0.446952, 0.435822, 0.424919, 0.414301, 0.404009, 0.394258, 0.385034, 0.375979,\
        0.367060],\
        [0.310439, 0.318550, 0.327397, 0.337123, 0.347799, 0.359493, 0.372290, 0.386265, 0.401213, 0.415461,\
        0.429260, 0.443449, 0.458164, 0.473570, 0.490548, 0.508952, 0.527631, 0.546294, 0.564662, 0.582424,\
        0.599326, 0.615239, 0.630014, 0.643524, 0.655790, 0.666775, 0.676377, 0.684486, 0.690995, 0.695666,\
        0.697149, 0.693924, 0.686486, 0.676118, 0.663238, 0.648220, 0.631667, 0.614268, 0.596735, 0.582281,\
        0.570728, 0.560231, 0.550057, 0.540029, 0.530319, 0.521192, 0.512863, 0.505450, 0.498904, 0.494229,\
        0.495892, 0.514910, 0.547376, 0.583166, 0.614095, 0.636884, 0.652222, 0.662367, 0.669776, 0.675768,\
        0.680317, 0.682885, 0.683465, 0.682659, 0.681415, 0.680598, 0.680473, 0.680632, 0.681382, 0.682525,\
        0.683753, 0.686082, 0.688609, 0.691400, 0.695626, 0.699835, 0.703962, 0.707991, 0.711489, 0.713997,\
        0.715295, 0.715213, 0.714006, 0.712212, 0.709850, 0.706330, 0.701653, 0.695794, 0.688445, 0.680589,\
        0.672450, 0.664041, 0.655372, 0.646458, 0.637305, 0.627896, 0.618169, 0.607990, 0.597489, 0.586973,\
        0.576081, 0.564897, 0.553609, 0.542397, 0.531425, 0.520795, 0.510310, 0.499544, 0.488688, 0.477614,\
        0.466370, 0.455078, 0.443850, 0.432781, 0.421957, 0.411474, 0.401423, 0.391739, 0.382492, 0.373544,\
        0.364774],\
        [0.303504, 0.311426, 0.320241, 0.330005, 0.340720, 0.352414, 0.365126, 0.378810, 0.392869, 0.405705,\
        0.418097, 0.431513, 0.446412, 0.462564, 0.479961, 0.498267, 0.516644, 0.535327, 0.554195, 0.572774,\
        0.590702, 0.607830, 0.623810, 0.638560, 0.652228, 0.664687, 0.675719, 0.685133, 0.692810, 0.698638,\
        0.702032, 0.700824, 0.695184, 0.686734, 0.675302, 0.661311, 0.645155, 0.627266, 0.609109, 0.594655,\
        0.582886, 0.571889, 0.560838, 0.549673, 0.538806, 0.528601, 0.519349, 0.511349, 0.505148, 0.500986,\
        0.500593, 0.523418, 0.558560, 0.596008, 0.627000, 0.650131, 0.665750, 0.675483, 0.682381, 0.688307,\
        0.693046, 0.695346, 0.695414, 0.693958, 0.692259, 0.691365, 0.692080, 0.692231, 0.693365, 0.695440,\
        0.696021, 0.699244, 0.702305, 0.704804, 0.709045, 0.713121, 0.716760, 0.719985, 0.722550, 0.724175,\
        0.724675, 0.723990, 0.722256, 0.719687, 0.716287, 0.711871, 0.706388, 0.699638, 0.691790, 0.683632,\
        0.675093, 0.666166, 0.656891, 0.647309, 0.637456, 0.627372, 0.617100, 0.606621, 0.595616, 0.584209,\
        0.572690, 0.561095, 0.549513, 0.538065, 0.526923, 0.516389, 0.506665, 0.496385, 0.485240, 0.474026,\
        0.462793, 0.451586, 0.440471, 0.429525, 0.418831, 0.408478, 0.398566, 0.389105, 0.379991, 0.371158,\
        0.362519],\
        [0.296634, 0.304403, 0.313223, 0.322943, 0.333531, 0.344960, 0.357158, 0.369875, 0.382510, 0.394441,\
        0.405978, 0.419280, 0.434779, 0.451329, 0.468653, 0.486541, 0.504748, 0.523687, 0.543136, 0.562411,\
        0.581236, 0.599445, 0.616224, 0.632123, 0.647264, 0.661252, 0.673707, 0.684320, 0.692893, 0.699537,\
        0.704120, 0.705147, 0.702336, 0.696447, 0.686772, 0.674711, 0.660058, 0.643226, 0.625962, 0.610616,\
        0.597363, 0.585400, 0.572888, 0.560200, 0.548055, 0.536756, 0.526584, 0.517965, 0.511573, 0.508411,\
        0.513707, 0.536155, 0.570773, 0.607837, 0.639462, 0.663905, 0.679981, 0.688452, 0.694433, 0.700326,\
        0.705153, 0.707394, 0.707440, 0.706103, 0.704534, 0.703736, 0.705297, 0.705866, 0.707320, 0.709928,\
        0.711518, 0.714661, 0.718094, 0.721161, 0.724578, 0.727821, 0.730502, 0.732561, 0.733906, 0.734392,\
        0.733888, 0.732338, 0.729775, 0.726267, 0.721805, 0.716302, 0.709649, 0.701864, 0.693938, 0.685672,\
        0.676773, 0.667348, 0.657494, 0.647270, 0.636727, 0.625933, 0.615020, 0.604064, 0.592623, 0.580650,\
        0.568631, 0.556675, 0.544837, 0.533163, 0.521708, 0.510712, 0.501012, 0.491517, 0.480835, 0.469850,\
        0.458809, 0.447792, 0.436869, 0.426115, 0.415609, 0.405431, 0.395706, 0.386448, 0.377515, 0.368829,\
        0.360323],\
        [0.290236, 0.297927, 0.306541, 0.316072, 0.326347, 0.337314, 0.348822, 0.360573, 0.372154, 0.383282,\
        0.394341, 0.408167, 0.423996, 0.440345, 0.457201, 0.474617, 0.492606, 0.511828, 0.531693, 0.551228,\
        0.570354, 0.588975, 0.606724, 0.624116, 0.641008, 0.656645, 0.670674, 0.682745, 0.692421, 0.699903,\
        0.706009, 0.709003, 0.708361, 0.704136, 0.696895, 0.687833, 0.675398, 0.660013, 0.643205, 0.627176,\
        0.612820, 0.599722, 0.585137, 0.571130, 0.558017, 0.545906, 0.535072, 0.526069, 0.520003, 0.519094,\
        0.527576, 0.549225, 0.582114, 0.618359, 0.651131, 0.677701, 0.695673, 0.702692, 0.707707, 0.713052,\
        0.717122, 0.719332, 0.719844, 0.719358, 0.718739, 0.718927, 0.720282, 0.721875, 0.723970, 0.726620,\
        0.729331, 0.732528, 0.735899, 0.738930, 0.741536, 0.743548, 0.744838, 0.745432, 0.745351, 0.744532,\
        0.742869, 0.740271, 0.736699, 0.732154, 0.726634, 0.720127, 0.712661, 0.704463, 0.696051, 0.687181,\
        0.677690, 0.667696, 0.657261, 0.646426, 0.635213, 0.623647, 0.611845, 0.600116, 0.588267, 0.576119,\
        0.563856, 0.551681, 0.539719, 0.528012, 0.516495, 0.505131, 0.494804, 0.485568, 0.475635, 0.465170,\
        0.454492, 0.443771, 0.433116, 0.422619, 0.412370, 0.402462, 0.392965, 0.383884, 0.375117, 0.366580,\
        0.358208],\
        [0.284452, 0.292017, 0.300386, 0.309563, 0.319339, 0.329792, 0.340666, 0.351702, 0.362714, 0.373781,\
        0.385633, 0.399431, 0.414422, 0.429759, 0.445847, 0.462918, 0.481053, 0.500341, 0.520019, 0.539139,\
        0.558128, 0.577086, 0.595792, 0.615074, 0.633755, 0.650877, 0.666647, 0.680805, 0.692574, 0.701610,\
        0.708665, 0.712667, 0.713293, 0.710780, 0.706094, 0.698891, 0.689662, 0.675310, 0.658558, 0.642107,\
        0.626670, 0.611667, 0.596463, 0.582195, 0.568673, 0.556210, 0.545096, 0.535934, 0.530164, 0.530263,\
        0.539346, 0.559595, 0.590716, 0.627737, 0.662583, 0.690226, 0.708850, 0.717842, 0.722786, 0.726820,\
        0.729988, 0.732050, 0.733148, 0.733742, 0.734383, 0.735543, 0.737361, 0.739627, 0.742264, 0.745196,\
        0.748305, 0.751608, 0.754840, 0.757443, 0.759080, 0.759652, 0.759271, 0.758195, 0.756573, 0.754386,\
        0.751500, 0.747765, 0.743088, 0.737450, 0.730881, 0.723450, 0.715296, 0.706710, 0.697760, 0.687971,\
        0.677814, 0.667232, 0.656235, 0.644849, 0.633076, 0.620877, 0.608210, 0.595605, 0.583423, 0.571004,\
        0.558509, 0.546171, 0.534194, 0.522707, 0.511620, 0.500748, 0.490236, 0.480338, 0.470469, 0.460342,\
        0.450026, 0.439642, 0.429301, 0.419104, 0.409149, 0.399530, 0.390300, 0.381421, 0.372823, 0.364433,\
        0.356191],\
        [0.279147, 0.286608, 0.294734, 0.303250, 0.312635, 0.322670, 0.333014, 0.343522, 0.354196, 0.365296,\
        0.377380, 0.390878, 0.404935, 0.418902, 0.434222, 0.450965, 0.469063, 0.488335, 0.507533, 0.525704,\
        0.544652, 0.564496, 0.585053, 0.606019, 0.625488, 0.643378, 0.660980, 0.677826, 0.692757, 0.704040,\
        0.711320, 0.715646, 0.717121, 0.715957, 0.712517, 0.707092, 0.700138, 0.686282, 0.670405, 0.654314,\
        0.638572, 0.623108, 0.608018, 0.593603, 0.580018, 0.567723, 0.556736, 0.547385, 0.541309, 0.541187,\
        0.549834, 0.568864, 0.598520, 0.638977, 0.676372, 0.704025, 0.721912, 0.731907, 0.737308, 0.740855,\
        0.743550, 0.745667, 0.747453, 0.749216, 0.751168, 0.753361, 0.755823, 0.758616, 0.761625, 0.764666,\
        0.767725, 0.770853, 0.773898, 0.775877, 0.776415, 0.775404, 0.773175, 0.770360, 0.767216, 0.763719,\
        0.759653, 0.754771, 0.748937, 0.742147, 0.734491, 0.726108, 0.717132, 0.707615, 0.697732, 0.687704,\
        0.677062, 0.665918, 0.654390, 0.642529, 0.630352, 0.617849, 0.604945, 0.591694, 0.578583, 0.565593,\
        0.552746, 0.540194, 0.528158, 0.516924, 0.506548, 0.496052, 0.485545, 0.475422, 0.465513, 0.455589,\
        0.445583, 0.435534, 0.425518, 0.415638, 0.405994, 0.396681, 0.387726, 0.379068, 0.370648, 0.362405,\
        0.354288],\
        [0.274229, 0.281519, 0.289250, 0.297480, 0.306600, 0.316151, 0.325977, 0.335965, 0.346208, 0.356972,\
        0.368572, 0.381061, 0.393863, 0.407191, 0.422058, 0.438358, 0.455936, 0.474344, 0.492486, 0.510492,\
        0.529728, 0.550563, 0.572850, 0.595574, 0.614363, 0.632911, 0.652749, 0.672320, 0.690012, 0.703373,\
        0.711656, 0.716786, 0.719090, 0.718912, 0.716445, 0.711690, 0.703805, 0.692392, 0.678682, 0.663915,\
        0.648905, 0.633966, 0.619309, 0.605161, 0.591881, 0.580466, 0.570055, 0.560210, 0.553050, 0.551806,\
        0.559949, 0.580170, 0.612878, 0.655454, 0.694407, 0.720026, 0.736062, 0.745493, 0.751019, 0.754693,\
        0.757563, 0.760134, 0.762805, 0.765845, 0.769203, 0.772405, 0.775305, 0.778418, 0.781580, 0.784340,\
        0.786873, 0.789262, 0.791941, 0.792905, 0.792637, 0.789912, 0.785879, 0.781458, 0.776958, 0.772353,\
        0.767244, 0.761245, 0.754198, 0.746164, 0.737327, 0.727921, 0.718158, 0.708110, 0.697674, 0.686831,\
        0.675442, 0.663686, 0.651649, 0.639356, 0.626823, 0.614069, 0.601036, 0.587242, 0.573384, 0.559834,\
        0.546641, 0.533861, 0.521612, 0.510248, 0.500223, 0.490401, 0.480293, 0.470377, 0.460639, 0.450958,\
        0.441264, 0.431550, 0.421862, 0.412294, 0.402981, 0.393979, 0.385291, 0.376855, 0.368610, 0.360508,\
        0.352508],\
        [0.269663, 0.276839, 0.284447, 0.292573, 0.301199, 0.310245, 0.319538, 0.328880, 0.338489, 0.348601,\
        0.359377, 0.370799, 0.382735, 0.395517, 0.409757, 0.425377, 0.442166, 0.459593, 0.477004, 0.494651,\
        0.513771, 0.534874, 0.557478, 0.579290, 0.598653, 0.619448, 0.642156, 0.663963, 0.683224, 0.698205,\
        0.709002, 0.715441, 0.718738, 0.719592, 0.718006, 0.713747, 0.706522, 0.696604, 0.684696, 0.671586,\
        0.657937, 0.644123, 0.630390, 0.616987, 0.604419, 0.594733, 0.584936, 0.574009, 0.565100, 0.561870,\
        0.568704, 0.589727, 0.626172, 0.670802, 0.710563, 0.734995, 0.749589, 0.758313, 0.764033, 0.768448,\
        0.772194, 0.775699, 0.779525, 0.783954, 0.788809, 0.792961, 0.795741, 0.798751, 0.801685, 0.803659,\
        0.805242, 0.806451, 0.807526, 0.807451, 0.806210, 0.802163, 0.797029, 0.791197, 0.785639, 0.780252,\
        0.774282, 0.767165, 0.758794, 0.749373, 0.739210, 0.728628, 0.717961, 0.707535, 0.696879, 0.684887,\
        0.672717, 0.660412, 0.647931, 0.635239, 0.622301, 0.609100, 0.595537, 0.581526, 0.567402, 0.553575,\
        0.540230, 0.527412, 0.515087, 0.503301, 0.493296, 0.484041, 0.474633, 0.465195, 0.455813, 0.446470,\
        0.437129, 0.427776, 0.418430, 0.409148, 0.400105, 0.391453, 0.383042, 0.374812, 0.366726, 0.358750,\
        0.350854],\
        [0.265392, 0.272493, 0.280016, 0.287973, 0.296279, 0.305074, 0.313580, 0.322081, 0.330929, 0.340271,\
        0.350219, 0.360825, 0.372142, 0.384393, 0.397880, 0.412595, 0.428758, 0.445588, 0.462464, 0.479632,\
        0.497735, 0.518916, 0.541517, 0.563007, 0.583695, 0.606562, 0.630741, 0.653869, 0.674559, 0.691813,\
        0.704316, 0.711402, 0.716033, 0.718337, 0.717958, 0.714658, 0.708464, 0.699808, 0.689413, 0.678002,\
        0.666021, 0.653720, 0.641346, 0.629290, 0.618346, 0.609218, 0.599396, 0.587764, 0.577331, 0.571607,\
        0.576237, 0.597284, 0.636248, 0.682288, 0.721607, 0.747441, 0.761919, 0.770163, 0.776536, 0.782592,\
        0.788080, 0.793118, 0.798577, 0.804561, 0.810473, 0.814958, 0.817543, 0.819606, 0.821273, 0.822205,\
        0.822586, 0.822510, 0.821969, 0.820496, 0.817578, 0.812834, 0.806661, 0.799725, 0.793460, 0.787611,\
        0.780869, 0.772489, 0.762581, 0.751601, 0.739978, 0.728049, 0.716100, 0.704373, 0.692966, 0.681036,\
        0.668640, 0.656022, 0.643226, 0.630209, 0.616877, 0.603190, 0.589202, 0.574942, 0.560681, 0.546778,\
        0.533507, 0.520966, 0.509120, 0.497854, 0.487522, 0.478176, 0.469102, 0.460089, 0.451115, 0.442164,\
        0.433218, 0.424269, 0.415322, 0.406399, 0.397636, 0.389234, 0.381029, 0.372960, 0.365003, 0.357134,\
        0.349325],\
        [0.261322, 0.268323, 0.275725, 0.283560, 0.291847, 0.300163, 0.307559, 0.315322, 0.323470, 0.332070,\
        0.341289, 0.351265, 0.362095, 0.373879, 0.386674, 0.400643, 0.416623, 0.432995, 0.449324, 0.466170,\
        0.484463, 0.505439, 0.527188, 0.548163, 0.570167, 0.594122, 0.618763, 0.642560, 0.664097, 0.682271,\
        0.695585, 0.704558, 0.711510, 0.715843, 0.717061, 0.715018, 0.709911, 0.702355, 0.693324, 0.683675,\
        0.673480, 0.662833, 0.651909, 0.641050, 0.630757, 0.621111, 0.611133, 0.600432, 0.590038, 0.582128,\
        0.584135, 0.606488, 0.647917, 0.694761, 0.734747, 0.761441, 0.774725, 0.781536, 0.789034, 0.797731,\
        0.805970, 0.813509, 0.821158, 0.828531, 0.834518, 0.838219, 0.839857, 0.840385, 0.840314, 0.839749,\
        0.838750, 0.837298, 0.835241, 0.832261, 0.828018, 0.822366, 0.815536, 0.808148, 0.801167, 0.794743,\
        0.787037, 0.776974, 0.765240, 0.752598, 0.739524, 0.726306, 0.713111, 0.700286, 0.688162, 0.676003,\
        0.663455, 0.650626, 0.637610, 0.624390, 0.610796, 0.596720, 0.582354, 0.567836, 0.553429, 0.539492,\
        0.526394, 0.514301, 0.503005, 0.492291, 0.482157, 0.472796, 0.463903, 0.455221, 0.446632, 0.438082,\
        0.429546, 0.421019, 0.412522, 0.404071, 0.395636, 0.387346, 0.379247, 0.371287, 0.363430, 0.355646,\
        0.347912],\
        [0.257374, 0.264205, 0.271378, 0.278877, 0.286568, 0.294009, 0.301148, 0.308557, 0.316130, 0.324029,\
        0.332607, 0.342103, 0.352603, 0.364151, 0.376825, 0.390868, 0.406293, 0.421778, 0.437169, 0.453715,\
        0.472356, 0.493309, 0.512982, 0.532717, 0.555352, 0.580186, 0.605670, 0.630063, 0.652043, 0.670819,\
        0.685685, 0.697151, 0.706477, 0.712986, 0.715972, 0.715355, 0.711428, 0.704852, 0.697134, 0.689128,\
        0.680584, 0.671516, 0.661850, 0.651724, 0.641451, 0.631235, 0.621093, 0.611375, 0.602709, 0.594971,\
        0.595572, 0.620413, 0.664115, 0.711434, 0.752227, 0.780288, 0.790424, 0.793767, 0.802996, 0.814687,\
        0.825899, 0.836086, 0.845673, 0.854142, 0.859790, 0.861837, 0.861594, 0.860176, 0.858180, 0.855941,\
        0.853490, 0.850663, 0.847203, 0.842848, 0.837412, 0.830897, 0.823564, 0.815933, 0.808567, 0.801388,\
        0.792185, 0.779809, 0.766176, 0.752004, 0.737705, 0.723557, 0.709639, 0.696060, 0.683246, 0.670489,\
        0.657554, 0.644427, 0.631175, 0.617831, 0.604207, 0.589850, 0.575163, 0.560433, 0.545884, 0.531849,\
        0.518823, 0.507273, 0.496555, 0.486309, 0.476671, 0.467609, 0.458986, 0.450627, 0.442404, 0.434243,\
        0.426101, 0.417966, 0.409859, 0.401803, 0.393680, 0.385576, 0.377602, 0.369745, 0.361975, 0.354266,\
        0.346596],\
        [0.253513, 0.260105, 0.266946, 0.273983, 0.281075, 0.288045, 0.294952, 0.301969, 0.308967, 0.316114,\
        0.324130, 0.333295, 0.343590, 0.355017, 0.367669, 0.381665, 0.396726, 0.410770, 0.424782, 0.440605,\
        0.458430, 0.477403, 0.495603, 0.515475, 0.539251, 0.565049, 0.591419, 0.616395, 0.638875, 0.658849,\
        0.675905, 0.690111, 0.701749, 0.710264, 0.715017, 0.715939, 0.713404, 0.708244, 0.701686, 0.694598,\
        0.687458, 0.679856, 0.671203, 0.661441, 0.650882, 0.640059, 0.629624, 0.620385, 0.613236, 0.609659,\
        0.616220, 0.642135, 0.684427, 0.731491, 0.772226, 0.800952, 0.810967, 0.814118, 0.822544, 0.835552,\
        0.848148, 0.859442, 0.869943, 0.879323, 0.884544, 0.883887, 0.881554, 0.878082, 0.874147, 0.870318,\
        0.866545, 0.862467, 0.857722, 0.852078, 0.845467, 0.837994, 0.829933, 0.821651, 0.813368, 0.804610,\
        0.793501, 0.779696, 0.764741, 0.749471, 0.734297, 0.719527, 0.705710, 0.691847, 0.678051, 0.664590,\
        0.651153, 0.637623, 0.624022, 0.610409, 0.596693, 0.582367, 0.567672, 0.552944, 0.538385, 0.524253,\
        0.510992, 0.499637, 0.489769, 0.480258, 0.471199, 0.462576, 0.454312, 0.446300, 0.438440, 0.430653,\
        0.422881, 0.415086, 0.407266, 0.399453, 0.391648, 0.383819, 0.376022, 0.368285, 0.360604, 0.352967,\
        0.345357],\
        [0.249758, 0.256070, 0.262548, 0.269133, 0.275739, 0.282302, 0.288859, 0.295473, 0.301855, 0.308221,\
        0.315875, 0.324879, 0.334986, 0.346164, 0.358432, 0.371688, 0.385344, 0.398193, 0.411490, 0.426702,\
        0.443483, 0.460992, 0.478875, 0.499138, 0.523141, 0.549013, 0.575524, 0.600938, 0.624529, 0.646226,\
        0.665671, 0.682603, 0.696750, 0.707375, 0.714000, 0.716530, 0.715455, 0.711656, 0.706130, 0.699852,\
        0.694118, 0.687912, 0.680031, 0.670379, 0.659295, 0.647757, 0.636986, 0.628319, 0.623394, 0.624903,\
        0.637539, 0.665245, 0.706326, 0.753378, 0.793874, 0.820589, 0.833761, 0.840338, 0.848718, 0.860244,\
        0.872261, 0.882528, 0.891919, 0.900982, 0.905271, 0.902238, 0.898822, 0.893216, 0.887649, 0.882527,\
        0.877743, 0.872602, 0.866651, 0.859714, 0.851829, 0.843175, 0.834010, 0.824551, 0.814737, 0.803950,\
        0.791175, 0.776456, 0.760749, 0.744891, 0.729307, 0.713997, 0.699209, 0.685831, 0.672022, 0.658201,\
        0.644373, 0.630448, 0.616446, 0.602485, 0.588603, 0.574476, 0.560066, 0.545610, 0.531319, 0.517423,\
        0.504307, 0.492935, 0.483444, 0.474534, 0.465989, 0.457791, 0.449897, 0.442236, 0.434730, 0.427305,\
        0.419892, 0.412425, 0.404863, 0.397279, 0.389725, 0.382118, 0.374490, 0.366876, 0.359289, 0.351725,\
        0.344174],\
        [0.246157, 0.252184, 0.258312, 0.264492, 0.270670, 0.276807, 0.282896, 0.288891, 0.294648, 0.300565,\
        0.308238, 0.317109, 0.326882, 0.337601, 0.349186, 0.361379, 0.373693, 0.385798, 0.398505, 0.413289,\
        0.429316, 0.446221, 0.464290, 0.484575, 0.507457, 0.532611, 0.558697, 0.584413, 0.609154, 0.632709,\
        0.654454, 0.673909, 0.690571, 0.703647, 0.712339, 0.716443, 0.716788, 0.714382, 0.710204, 0.705217,\
        0.700669, 0.695155, 0.688170, 0.678474, 0.666566, 0.654190, 0.643134, 0.635215, 0.632591, 0.638144,\
        0.655245, 0.685732, 0.728095, 0.775970, 0.817096, 0.843582, 0.857646, 0.866033, 0.875023, 0.885533,\
        0.896387, 0.904760, 0.911515, 0.917737, 0.919843, 0.917151, 0.912110, 0.905483, 0.898524, 0.892559,\
        0.887112, 0.881011, 0.873848, 0.865521, 0.856203, 0.846133, 0.835544, 0.824582, 0.813177, 0.800764,\
        0.786429, 0.770750, 0.754459, 0.738357, 0.722972, 0.708147, 0.693408, 0.679282, 0.665361, 0.651388,\
        0.637371, 0.623221, 0.608829, 0.594717, 0.580753, 0.566716, 0.552612, 0.538600, 0.524848, 0.511564,\
        0.499121, 0.487998, 0.478302, 0.469503, 0.461231, 0.453346, 0.445768, 0.438426, 0.431250, 0.424170,\
        0.417115, 0.410005, 0.402762, 0.395390, 0.387969, 0.380501, 0.373005, 0.365502, 0.358006, 0.350516,\
        0.343027],\
        [0.242769, 0.248531, 0.254346, 0.260176, 0.265981, 0.271728, 0.277394, 0.282960, 0.288504, 0.294576,\
        0.301964, 0.310242, 0.319451, 0.329597, 0.340463, 0.351767, 0.363254, 0.374951, 0.387461, 0.401431,\
        0.416615, 0.433007, 0.450823, 0.470494, 0.492342, 0.516437, 0.541843, 0.567566, 0.593156, 0.618223,\
        0.641867, 0.663490, 0.682666, 0.698505, 0.709051, 0.714614, 0.716368, 0.715449, 0.712894, 0.709447,\
        0.705343, 0.700547, 0.695472, 0.685135, 0.672042, 0.658820, 0.647595, 0.640404, 0.639531, 0.647677,\
        0.667793, 0.701370, 0.745612, 0.794898, 0.839023, 0.868134, 0.881791, 0.889821, 0.899542, 0.909073,\
        0.917774, 0.924443, 0.929083, 0.932012, 0.931958, 0.928523, 0.922633, 0.915321, 0.907653, 0.901099,\
        0.894606, 0.887615, 0.879105, 0.869239, 0.858369, 0.846745, 0.834563, 0.821949, 0.808821, 0.794907,\
        0.779573, 0.763126, 0.746312, 0.729973, 0.714971, 0.701188, 0.686817, 0.672502, 0.658386, 0.644289,\
        0.630226, 0.616238, 0.601894, 0.587438, 0.573277, 0.559234, 0.545384, 0.531883, 0.518834, 0.506256,\
        0.494416, 0.483594, 0.473878, 0.465077, 0.456928, 0.449247, 0.441917, 0.434846, 0.427960, 0.421189,\
        0.414468, 0.407715, 0.400802, 0.393607, 0.386286, 0.378923, 0.371538, 0.364139, 0.356734, 0.349322,\
        0.341898],\
        [0.239647, 0.245180, 0.250734, 0.256279, 0.261788, 0.267240, 0.272633, 0.278011, 0.283548, 0.289620,\
        0.296525, 0.304162, 0.312824, 0.322466, 0.332713, 0.343331, 0.354234, 0.365557, 0.377664, 0.390873,\
        0.405311, 0.421062, 0.438311, 0.457165, 0.477921, 0.500745, 0.525007, 0.550309, 0.576355, 0.602572,\
        0.627438, 0.650683, 0.671858, 0.689599, 0.702245, 0.709830, 0.712990, 0.713639, 0.712828, 0.711056,\
        0.708468, 0.705331, 0.700119, 0.688577, 0.674519, 0.660898, 0.649837, 0.643412, 0.643840, 0.653443,\
        0.674589, 0.710161, 0.758457, 0.811201, 0.860152, 0.893918, 0.906597, 0.913707, 0.922261, 0.930219,\
        0.936691, 0.941230, 0.943695, 0.944047, 0.941928, 0.937263, 0.930580, 0.922734, 0.914700, 0.907192,\
        0.900149, 0.892166, 0.881962, 0.870503, 0.858147, 0.845036, 0.831284, 0.817090, 0.802513, 0.787259,\
        0.771190, 0.754322, 0.737101, 0.720316, 0.705138, 0.692063, 0.678833, 0.665119, 0.651166, 0.637003,\
        0.622782, 0.608891, 0.595124, 0.580452, 0.566020, 0.551928, 0.538254, 0.525216, 0.513017, 0.501223,\
        0.489873, 0.479321, 0.469720, 0.460989, 0.452948, 0.445430, 0.438300, 0.431456, 0.424811, 0.418291,\
        0.411832, 0.405363, 0.398713, 0.391702, 0.384540, 0.377313, 0.370051, 0.362764, 0.355455, 0.348126,\
        0.340772],\
        [0.236832, 0.242180, 0.247530, 0.252863, 0.258163, 0.263426, 0.268674, 0.273977, 0.279484, 0.285397,\
        0.291847, 0.298942, 0.307288, 0.316510, 0.326180, 0.336183, 0.346540, 0.357398, 0.368998, 0.381557,\
        0.395187, 0.410287, 0.426744, 0.444396, 0.463724, 0.484948, 0.507844, 0.532244, 0.558096, 0.584702,\
        0.610500, 0.635328, 0.658055, 0.677016, 0.691562, 0.701044, 0.705520, 0.707977, 0.708876, 0.708770,\
        0.707402, 0.704267, 0.697753, 0.686572, 0.673022, 0.659962, 0.649638, 0.644224, 0.646011, 0.657308,\
        0.680005, 0.717611, 0.769298, 0.824887, 0.876017, 0.911713, 0.927630, 0.935827, 0.942664, 0.948538,\
        0.952824, 0.955165, 0.955429, 0.953573, 0.949557, 0.943505, 0.935843, 0.927246, 0.918494, 0.910260,\
        0.902537, 0.893192, 0.881571, 0.868866, 0.855408, 0.841166, 0.826079, 0.810384, 0.794713, 0.778718,\
        0.762176, 0.745240, 0.728102, 0.710977, 0.695990, 0.683266, 0.670701, 0.657614, 0.644001, 0.629842,\
        0.615282, 0.600811, 0.587095, 0.572985, 0.558710, 0.544674, 0.531096, 0.518238, 0.506635, 0.496051,\
        0.485084, 0.474889, 0.465559, 0.457022, 0.449150, 0.441804, 0.434861, 0.428213, 0.421768, 0.415440,\
        0.409153, 0.402826, 0.396360, 0.389617, 0.382680, 0.375634, 0.368520, 0.361358, 0.354156, 0.346917,\
        0.339639],\
        [0.234350, 0.239559, 0.244764, 0.249954, 0.255126, 0.260290, 0.265480, 0.270768, 0.276266, 0.282104,\
        0.288406, 0.295366, 0.303289, 0.311906, 0.320913, 0.330266, 0.340029, 0.350340, 0.361386, 0.373359,\
        0.386452, 0.400875, 0.416040, 0.431997, 0.449623, 0.469123, 0.490399, 0.513496, 0.538631, 0.565178,\
        0.591991, 0.618730, 0.642506, 0.661744, 0.676801, 0.687452, 0.693992, 0.698243, 0.700338, 0.701566,\
        0.700762, 0.696949, 0.690157, 0.679905, 0.667887, 0.656262, 0.647243, 0.643117, 0.646378, 0.659890,\
        0.687013, 0.727386, 0.779419, 0.834546, 0.884107, 0.919434, 0.939637, 0.951083, 0.958324, 0.963062,\
        0.965607, 0.965976, 0.964202, 0.960363, 0.954584, 0.947090, 0.938258, 0.928591, 0.918653, 0.908952,\
        0.899712, 0.889606, 0.877291, 0.864015, 0.850096, 0.835393, 0.819533, 0.802769, 0.786175, 0.769625,\
        0.752879, 0.736263, 0.720242, 0.704708, 0.689584, 0.676163, 0.663311, 0.650429, 0.637214, 0.623317,\
        0.608532, 0.593941, 0.579821, 0.565638, 0.551435, 0.537509, 0.524065, 0.511239, 0.499368, 0.489423,\
        0.479633, 0.470156, 0.461288, 0.453070, 0.445440, 0.438301, 0.431549, 0.425088, 0.418822, 0.412655,\
        0.406498, 0.400286, 0.393968, 0.387457, 0.380748, 0.373896, 0.366944, 0.359917, 0.352830, 0.345688,\
        0.338491],\
        [0.232210, 0.237327, 0.242439, 0.247547, 0.252656, 0.257786, 0.262977, 0.268296, 0.273834, 0.279701,\
        0.286026, 0.292954, 0.300521, 0.308468, 0.316752, 0.325417, 0.334536, 0.344226, 0.354641, 0.365956,\
        0.378347, 0.391810, 0.405416, 0.419769, 0.435902, 0.453869, 0.473546, 0.495105, 0.519207, 0.545812,\
        0.574365, 0.602710, 0.626141, 0.644377, 0.659592, 0.671382, 0.679521, 0.684701, 0.687621, 0.689058,\
        0.688377, 0.684912, 0.678826, 0.670284, 0.660330, 0.650659, 0.643319, 0.640627, 0.645155, 0.659622,\
        0.686819, 0.731990, 0.788136, 0.841466, 0.886439, 0.921334, 0.945322, 0.960127, 0.968767, 0.973277,\
        0.974661, 0.973412, 0.969843, 0.964214, 0.956786, 0.947857, 0.937768, 0.926876, 0.915539, 0.904169,\
        0.893185, 0.881924, 0.869391, 0.855978, 0.842129, 0.827783, 0.812150, 0.794780, 0.777420, 0.760236,\
        0.743307, 0.726936, 0.711888, 0.698513, 0.683850, 0.669552, 0.656159, 0.643275, 0.630590, 0.617592,\
        0.603320, 0.588233, 0.573314, 0.558591, 0.544242, 0.530460, 0.517350, 0.504992, 0.493619, 0.483509,\
        0.474196, 0.465333, 0.456965, 0.449126, 0.441788, 0.434884, 0.428334, 0.422058, 0.415973, 0.409978,\
        0.403950, 0.397864, 0.391685, 0.385334, 0.378808, 0.372132, 0.365338, 0.358448, 0.351479, 0.344437,\
        0.337326],\
        [0.230410, 0.235472, 0.240537, 0.245611, 0.250706, 0.255847, 0.261077, 0.266454, 0.272056, 0.277974,\
        0.284306, 0.291123, 0.298365, 0.305782, 0.313416, 0.321421, 0.329886, 0.338905, 0.348593, 0.359063,\
        0.370364, 0.382329, 0.394673, 0.407986, 0.423166, 0.440038, 0.458430, 0.478503, 0.500864, 0.526888,\
        0.556501, 0.585532, 0.607604, 0.625275, 0.641497, 0.654464, 0.663354, 0.668711, 0.671572, 0.672755,\
        0.672306, 0.669931, 0.665518, 0.659206, 0.651679, 0.644250, 0.638776, 0.637589, 0.643443, 0.659429,\
        0.689129, 0.736950, 0.795714, 0.847120, 0.888772, 0.922941, 0.948613, 0.965285, 0.974966, 0.979448,\
        0.979978, 0.977379, 0.972227, 0.964971, 0.956001, 0.945686, 0.934381, 0.922351, 0.909763, 0.897066,\
        0.884664, 0.872052, 0.858775, 0.845019, 0.831241, 0.817532, 0.803179, 0.785687, 0.767577, 0.750191,\
        0.733515, 0.717247, 0.702481, 0.689306, 0.675874, 0.661935, 0.648332, 0.635349, 0.623071, 0.611205,\
        0.598227, 0.582461, 0.566629, 0.551372, 0.536873, 0.523325, 0.510771, 0.498961, 0.488043, 0.478138,\
        0.469083, 0.460638, 0.452701, 0.445237, 0.438205, 0.431551, 0.425207, 0.419107, 0.413196, 0.407417,\
        0.401596, 0.395608, 0.389511, 0.383277, 0.376894, 0.370369, 0.363720, 0.356962, 0.350108, 0.343167,\
        0.336142],\
        [0.228932, 0.233973, 0.239025, 0.244100, 0.249214, 0.254395, 0.259682, 0.265125, 0.270785, 0.276726,\
        0.283003, 0.289633, 0.296547, 0.303538, 0.310654, 0.318097, 0.325947, 0.334289, 0.343211, 0.352785,\
        0.363016, 0.373828, 0.385239, 0.397790, 0.412267, 0.428413, 0.445974, 0.464927, 0.485546, 0.510263,\
        0.539839, 0.566941, 0.588434, 0.606959, 0.624922, 0.638549, 0.647099, 0.651564, 0.653545, 0.654335,\
        0.654417, 0.653607, 0.651508, 0.647907, 0.643103, 0.638078, 0.634506, 0.634730, 0.641766, 0.659353,\
        0.691545, 0.740198, 0.797269, 0.850114, 0.894425, 0.926812, 0.951268, 0.968108, 0.977966, 0.982122,\
        0.981786, 0.977930, 0.971318, 0.962554, 0.952129, 0.940474, 0.928021, 0.915095, 0.901573, 0.887791,\
        0.874176, 0.860387, 0.846108, 0.831572, 0.817408, 0.803674, 0.789269, 0.772814, 0.755447, 0.738848,\
        0.723711, 0.709189, 0.694306, 0.681042, 0.667311, 0.653307, 0.639460, 0.626109, 0.613571, 0.601985,\
        0.589183, 0.574302, 0.558741, 0.543481, 0.529023, 0.515763, 0.503876, 0.492820, 0.482396, 0.472893,\
        0.464188, 0.456118, 0.448563, 0.441450, 0.434722, 0.428317, 0.422176, 0.416235, 0.410442, 0.404800,\
        0.399286, 0.393400, 0.387387, 0.381265, 0.375010, 0.368620, 0.362102, 0.355467, 0.348724, 0.341881,\
        0.334942],\
        [0.227753, 0.232797, 0.237863, 0.242962, 0.248116, 0.253350, 0.258698, 0.264201, 0.269902, 0.275838,\
        0.282022, 0.288431, 0.294984, 0.301607, 0.308361, 0.315355, 0.322650, 0.330347, 0.338546, 0.347319,\
        0.356711, 0.366765, 0.377649, 0.389776, 0.403630, 0.419329, 0.436590, 0.455536, 0.476770, 0.501574,\
        0.528056, 0.552947, 0.574845, 0.594530, 0.612267, 0.625177, 0.631838, 0.634241, 0.634620, 0.634905,\
        0.635833, 0.637071, 0.637814, 0.637334, 0.635495, 0.632971, 0.631247, 0.632601, 0.640111, 0.657671,\
        0.689863, 0.739325, 0.797186, 0.850493, 0.893500, 0.927542, 0.952453, 0.968996, 0.978349, 0.981743,\
        0.980345, 0.975184, 0.967143, 0.956954, 0.945164, 0.932189, 0.918501, 0.904683, 0.890505, 0.875983,\
        0.861715, 0.847261, 0.832087, 0.816528, 0.801572, 0.787631, 0.773077, 0.757358, 0.741116, 0.725578,\
        0.712329, 0.702305, 0.688323, 0.672957, 0.658260, 0.643863, 0.629716, 0.615937, 0.602662, 0.590689,\
        0.578307, 0.564464, 0.549768, 0.534982, 0.520747, 0.507775, 0.496427, 0.486199, 0.476609, 0.467693,\
        0.459440, 0.451765, 0.444577, 0.437799, 0.431365, 0.425209, 0.419270, 0.413489, 0.407794, 0.402090,\
        0.396622, 0.391068, 0.385260, 0.379282, 0.373156, 0.366891, 0.360493, 0.353971, 0.347332, 0.340584,\
        0.333728],\
        [0.226843, 0.231909, 0.237004, 0.242143, 0.247345, 0.252635, 0.258042, 0.263595, 0.269323, 0.275241,\
        0.281340, 0.287572, 0.293845, 0.300103, 0.306601, 0.313189, 0.319957, 0.327056, 0.334611, 0.342726,\
        0.351494, 0.361043, 0.371595, 0.383481, 0.397039, 0.412417, 0.429529, 0.448767, 0.470660, 0.495102,\
        0.518955, 0.542347, 0.564800, 0.585801, 0.604000, 0.613946, 0.617415, 0.617264, 0.615614, 0.615485,\
        0.617521, 0.621175, 0.625144, 0.628054, 0.629330, 0.629381, 0.629498, 0.631793, 0.639286, 0.655888,\
        0.686377, 0.737674, 0.798979, 0.852153, 0.894935, 0.928339, 0.952499, 0.968055, 0.976319, 0.978544,\
        0.975849, 0.969257, 0.959749, 0.948199, 0.935209, 0.921039, 0.906035, 0.891003, 0.876118, 0.861404,\
        0.847279, 0.833136, 0.817575, 0.801242, 0.785863, 0.771272, 0.756463, 0.741055, 0.725723, 0.711001,\
        0.698173, 0.688380, 0.676647, 0.662378, 0.647836, 0.633512, 0.619370, 0.605534, 0.592340, 0.579844,\
        0.567415, 0.554282, 0.540510, 0.526494, 0.512667, 0.500046, 0.489428, 0.479883, 0.470982, 0.462646,\
        0.454862, 0.447586, 0.440755, 0.434301, 0.428154, 0.422246, 0.416512, 0.410909, 0.405418, 0.399938,\
        0.394404, 0.388877, 0.383197, 0.377347, 0.371339, 0.365187, 0.358897, 0.352479, 0.345937, 0.339277,\
        0.332500],\
        [0.226170, 0.231268, 0.236402, 0.241587, 0.246840, 0.252183, 0.257639, 0.263231, 0.268975, 0.274876,\
        0.280920, 0.287065, 0.293250, 0.299428, 0.305494, 0.311564, 0.317798, 0.324349, 0.331350, 0.338926,\
        0.347211, 0.356372, 0.366641, 0.378309, 0.391654, 0.406638, 0.423433, 0.442412, 0.463571, 0.486152,\
        0.509030, 0.532035, 0.554519, 0.575696, 0.592834, 0.600858, 0.602433, 0.601078, 0.597991, 0.597875,\
        0.600705, 0.606650, 0.613747, 0.620011, 0.624391, 0.627135, 0.629307, 0.632663, 0.640147, 0.656239,\
        0.686870, 0.737335, 0.798667, 0.852871, 0.896655, 0.929052, 0.951295, 0.965097, 0.971789, 0.972565,\
        0.968412, 0.960260, 0.949177, 0.936278, 0.922349, 0.907382, 0.891352, 0.875208, 0.859351, 0.844229,\
        0.830607, 0.818143, 0.803323, 0.786452, 0.770442, 0.755229, 0.740218, 0.724806, 0.709961, 0.697019,\
        0.684420, 0.673618, 0.662252, 0.649390, 0.635836, 0.622248, 0.608603, 0.595013, 0.581963, 0.569393,\
        0.556990, 0.544402, 0.531555, 0.518604, 0.505900, 0.493844, 0.483477, 0.474262, 0.465781, 0.457884,\
        0.450507, 0.443599, 0.437104, 0.430959, 0.425095, 0.419439, 0.413910, 0.408446, 0.403087, 0.397811,\
        0.392369, 0.386858, 0.381239, 0.375476, 0.369565, 0.363510, 0.357318, 0.350993, 0.344540, 0.337963,\
        0.331262],\
        [0.225700, 0.230837, 0.236014, 0.241245, 0.246545, 0.251933, 0.257427, 0.263043, 0.268790, 0.274670,\
        0.280668, 0.286756, 0.292887, 0.298955, 0.304667, 0.310274, 0.316036, 0.322113, 0.328648, 0.335778,\
        0.343654, 0.352455, 0.362394, 0.373703, 0.386559, 0.401067, 0.417453, 0.435766, 0.455802, 0.476991,\
        0.498789, 0.520838, 0.542130, 0.561403, 0.576194, 0.584253, 0.586420, 0.585354, 0.583371, 0.583371,\
        0.586724, 0.593606, 0.602851, 0.612042, 0.619307, 0.625065, 0.630003, 0.634952, 0.642399, 0.657237,\
        0.686117, 0.734144, 0.794007, 0.850310, 0.895962, 0.927218, 0.947758, 0.959565, 0.964426, 0.963696,\
        0.958122, 0.948349, 0.935521, 0.921146, 0.906389, 0.891023, 0.874499, 0.857618, 0.840882, 0.824940,\
        0.811114, 0.800177, 0.787307, 0.770655, 0.754325, 0.739018, 0.724538, 0.709823, 0.694587, 0.682113,\
        0.670273, 0.658986, 0.647575, 0.635431, 0.622745, 0.610105, 0.597536, 0.584322, 0.571500, 0.559071,\
        0.546905, 0.534886, 0.522983, 0.511163, 0.499518, 0.488427, 0.478286, 0.469235, 0.461016, 0.453437,\
        0.446390, 0.439803, 0.433614, 0.427760, 0.422174, 0.416783, 0.411496, 0.406190, 0.400757, 0.395622,\
        0.390360, 0.384926, 0.379361, 0.373664, 0.367830, 0.361859, 0.355753, 0.349513, 0.343142, 0.336642,\
        0.330012],\
        [0.225400, 0.230579, 0.235798, 0.241071, 0.246412, 0.251834, 0.257352, 0.262974, 0.268707, 0.274546,\
        0.280476, 0.286466, 0.292448, 0.298279, 0.303792, 0.309110, 0.314518, 0.320222, 0.326377, 0.333130,\
        0.340637, 0.349072, 0.358623, 0.369478, 0.381800, 0.395737, 0.411452, 0.428908, 0.447848, 0.467846,\
        0.488424, 0.508890, 0.528321, 0.545659, 0.558996, 0.567046, 0.570217, 0.570261, 0.569380, 0.569795,\
        0.573065, 0.579813, 0.589701, 0.601064, 0.610879, 0.620650, 0.630084, 0.637897, 0.645563, 0.658131,\
        0.682933, 0.729087, 0.790004, 0.846524, 0.890914, 0.922606, 0.941719, 0.950948, 0.953711, 0.951684,\
        0.945076, 0.933807, 0.919054, 0.903226, 0.887585, 0.871992, 0.855712, 0.838790, 0.821611, 0.805315,\
        0.790840, 0.779422, 0.767057, 0.752340, 0.736835, 0.721937, 0.708491, 0.695367, 0.681061, 0.667896,\
        0.655901, 0.644471, 0.633124, 0.621433, 0.609271, 0.597225, 0.585393, 0.573158, 0.560888, 0.548829,\
        0.537027, 0.525560, 0.514574, 0.503913, 0.493151, 0.482803, 0.473213, 0.464488, 0.456543, 0.449244,\
        0.442483, 0.436177, 0.430261, 0.424672, 0.419348, 0.414221, 0.409220, 0.404266, 0.399153, 0.393805,\
        0.388474, 0.383057, 0.377532, 0.371890, 0.366123, 0.360227, 0.354198, 0.348037, 0.341742, 0.335314,\
        0.328751],\
        [0.225241, 0.230460, 0.235718, 0.241027, 0.246398, 0.251843, 0.257369, 0.262981, 0.268678, 0.274452,\
        0.280280, 0.286127, 0.291927, 0.297577, 0.302899, 0.307984, 0.313146, 0.318574, 0.324428, 0.330859,\
        0.338023, 0.346084, 0.355212, 0.365570, 0.377310, 0.390581, 0.405492, 0.421991, 0.439865, 0.458657,\
        0.477796, 0.496506, 0.514392, 0.530448, 0.542949, 0.551028, 0.554924, 0.555851, 0.555539, 0.555788,\
        0.558059, 0.563209, 0.571357, 0.581841, 0.594287, 0.609938, 0.627129, 0.640099, 0.649225, 0.660910,\
        0.683418, 0.727190, 0.786658, 0.844129, 0.889988, 0.919586, 0.933703, 0.938573, 0.938891, 0.936045,\
        0.929230, 0.916993, 0.900221, 0.883239, 0.867008, 0.851366, 0.835723, 0.819458, 0.802377, 0.786162,\
        0.771837, 0.759198, 0.746706, 0.733207, 0.718762, 0.704319, 0.691515, 0.679272, 0.666391, 0.653673,\
        0.641635, 0.630209, 0.619083, 0.607914, 0.596386, 0.584812, 0.573435, 0.561919, 0.550316, 0.538779,\
        0.527395, 0.516329, 0.505983, 0.496344, 0.486567, 0.477041, 0.468094, 0.459829, 0.452230, 0.445226,\
        0.438735, 0.432683, 0.427009, 0.421654, 0.416557, 0.411651, 0.406868, 0.402118, 0.397194, 0.391939,\
        0.386595, 0.381196, 0.375714, 0.370128, 0.364427, 0.358602, 0.352647, 0.346560, 0.340337, 0.333977,\
        0.327478],\
        [0.225193, 0.230448, 0.235740, 0.241078, 0.246471, 0.251925, 0.257445, 0.263031, 0.268676, 0.274364,\
        0.280067, 0.285743, 0.291330, 0.296748, 0.301894, 0.306862, 0.311881, 0.317116, 0.322730, 0.328879,\
        0.335721, 0.343409, 0.352098, 0.361933, 0.373050, 0.385569, 0.399585, 0.415176, 0.432051, 0.449746,\
        0.467664, 0.485168, 0.501903, 0.516991, 0.528951, 0.537019, 0.541296, 0.542614, 0.542255, 0.541639,\
        0.542046, 0.544402, 0.549052, 0.556000, 0.566926, 0.587529, 0.616877, 0.638357, 0.650891, 0.664752,\
        0.688684, 0.729790, 0.784352, 0.841104, 0.890458, 0.915965, 0.921515, 0.921239, 0.919297, 0.916035,\
        0.909700, 0.896998, 0.879129, 0.861110, 0.844569, 0.829347, 0.814728, 0.799573, 0.783217, 0.767272,\
        0.752823, 0.739665, 0.727091, 0.714680, 0.701350, 0.687927, 0.675465, 0.663592, 0.651579, 0.639475,\
        0.627594, 0.616336, 0.605573, 0.594862, 0.583998, 0.572992, 0.562029, 0.551057, 0.540080, 0.529158,\
        0.518308, 0.507572, 0.497578, 0.488645, 0.479857, 0.471217, 0.462959, 0.455215, 0.448014, 0.441327,\
        0.435104, 0.429289, 0.423829, 0.418673, 0.413763, 0.409023, 0.404362, 0.399684, 0.394863, 0.389812,\
        0.384598, 0.379282, 0.373872, 0.368357, 0.362728, 0.356976, 0.351094, 0.345078, 0.338924, 0.332630,\
        0.326192],\
        [0.225231, 0.230518, 0.235837, 0.241196, 0.246601, 0.252055, 0.257558, 0.263107, 0.268688, 0.274281,\
        0.279855, 0.285361, 0.290739, 0.295916, 0.300894, 0.305821, 0.310751, 0.315833, 0.321242, 0.327139,\
        0.333676, 0.341002, 0.349254, 0.358564, 0.369053, 0.380830, 0.394003, 0.408661, 0.424628, 0.441444,\
        0.458557, 0.475394, 0.491401, 0.505777, 0.517368, 0.525411, 0.529849, 0.531226, 0.530459, 0.528609,\
        0.526749, 0.525901, 0.526669, 0.528558, 0.531764, 0.545646, 0.588710, 0.626357, 0.645216, 0.665941,\
        0.694813, 0.733872, 0.780127, 0.832160, 0.879445, 0.902140, 0.902157, 0.899651, 0.896175, 0.891680,\
        0.884245, 0.871058, 0.853848, 0.836349, 0.820495, 0.806220, 0.792727, 0.778509, 0.763275, 0.748072,\
        0.733899, 0.720818, 0.708211, 0.696545, 0.684680, 0.672256, 0.660335, 0.648832, 0.637440, 0.625958,\
        0.614196, 0.603052, 0.592814, 0.582545, 0.572178, 0.561705, 0.551203, 0.540725, 0.530335, 0.520102,\
        0.510016, 0.500012, 0.490442, 0.481770, 0.473536, 0.465566, 0.457922, 0.450685, 0.443890, 0.437530,\
        0.431572, 0.425975, 0.420700, 0.415709, 0.410957, 0.406359, 0.401797, 0.397187, 0.392463, 0.387563,\
        0.382499, 0.377304, 0.371991, 0.366564, 0.361016, 0.355341, 0.349533, 0.343588, 0.337502, 0.331270,\
        0.324891],\
        [0.225331, 0.230645, 0.235985, 0.241357, 0.246765, 0.252211, 0.257690, 0.263195, 0.268710, 0.274212,\
        0.279669, 0.285036, 0.290266, 0.295316, 0.300206, 0.305012, 0.309802, 0.314725, 0.319941, 0.325603,\
        0.331856, 0.338835, 0.346669, 0.355476, 0.365365, 0.376441, 0.388813, 0.402595, 0.417714, 0.433841,\
        0.450403, 0.466813, 0.482459, 0.496555, 0.508084, 0.516240, 0.520852, 0.522292, 0.521223, 0.518387,\
        0.514563, 0.510833, 0.508761, 0.507272, 0.502728, 0.497425, 0.542795, 0.597664, 0.628571, 0.663539,\
        0.700394, 0.739061, 0.780867, 0.823382, 0.859286, 0.878327, 0.880947, 0.877751, 0.872441, 0.865302,\
        0.855555, 0.842414, 0.826292, 0.810241, 0.796248, 0.783484, 0.770985, 0.757716, 0.743466, 0.728959,\
        0.715337, 0.702918, 0.690910, 0.679450, 0.668345, 0.657058, 0.645880, 0.634892, 0.624104, 0.613581,\
        0.602499, 0.591204, 0.581107, 0.571156, 0.561159, 0.551104, 0.541020, 0.530971, 0.521086, 0.511518,\
        0.502184, 0.492949, 0.483957, 0.475579, 0.467715, 0.460225, 0.453077, 0.446292, 0.439882, 0.433841,\
        0.428140, 0.422745, 0.417624, 0.412757, 0.408133, 0.403689, 0.399224, 0.394693, 0.390056, 0.385279,\
        0.380351, 0.375281, 0.370079, 0.364748, 0.359288, 0.353694, 0.347962, 0.342088, 0.336067, 0.329897,\
        0.323574],\
        [0.225473, 0.230808, 0.236162, 0.241541, 0.246946, 0.252376, 0.257826, 0.263286, 0.268738, 0.274160,\
        0.279520, 0.284783, 0.289917, 0.294907, 0.299740, 0.304390, 0.309011, 0.313768, 0.318801, 0.324246,\
        0.330235, 0.336893, 0.344339, 0.352681, 0.362021, 0.372450, 0.384061, 0.396956, 0.411331, 0.426920,\
        0.443027, 0.459102, 0.474570, 0.488708, 0.500579, 0.509182, 0.514218, 0.516082, 0.515341, 0.512525,\
        0.508098, 0.502742, 0.499112, 0.496172, 0.490632, 0.490493, 0.522434, 0.572840, 0.618908, 0.663268,\
        0.704392, 0.742328, 0.778663, 0.811821, 0.839161, 0.854961, 0.858579, 0.855087, 0.848193, 0.838645,\
        0.826958, 0.814747, 0.800159, 0.785991, 0.773864, 0.762422, 0.750796, 0.738369, 0.724962, 0.710849,\
        0.697762, 0.686034, 0.674720, 0.663747, 0.653104, 0.642610, 0.632197, 0.621761, 0.611323, 0.601490,\
        0.591655, 0.580862, 0.570609, 0.560743, 0.550996, 0.541276, 0.531564, 0.521873, 0.512323, 0.503269,\
        0.494610, 0.486019, 0.477681, 0.469759, 0.462279, 0.455191, 0.448457, 0.442066, 0.436012, 0.430275,\
        0.424825, 0.419622, 0.414632, 0.409838, 0.405264, 0.400952, 0.396635, 0.392193, 0.387659, 0.382998,\
        0.378193, 0.373242, 0.368150, 0.362918, 0.357548, 0.352036, 0.346380, 0.340576, 0.334620, 0.328509,\
        0.322240],\
        [0.225640, 0.230989, 0.236351, 0.241730, 0.247125, 0.252536, 0.257954, 0.263370, 0.268765, 0.274118,\
        0.279399, 0.284578, 0.289628, 0.294534, 0.299279, 0.303812, 0.308304, 0.312923, 0.317795, 0.323042,\
        0.328790, 0.335156, 0.342252, 0.350179, 0.359032, 0.368900, 0.379867, 0.392043, 0.405726, 0.420693,\
        0.436251, 0.451948, 0.467265, 0.481549, 0.493989, 0.503469, 0.509377, 0.512288, 0.512862, 0.511634,\
        0.508979, 0.505380, 0.501855, 0.499155, 0.495749, 0.492234, 0.521076, 0.572725, 0.623473, 0.668208,\
        0.708487, 0.743041, 0.773063, 0.798981, 0.821188, 0.833653, 0.835599, 0.831087, 0.823607, 0.813101,\
        0.800011, 0.788920, 0.777162, 0.764884, 0.753740, 0.743252, 0.732454, 0.720682, 0.708547, 0.695033,\
        0.682329, 0.670779, 0.659779, 0.649150, 0.638911, 0.629044, 0.619402, 0.609667, 0.599565, 0.589912,\
        0.580514, 0.570664, 0.560798, 0.551132, 0.541622, 0.532217, 0.522892, 0.513601, 0.504309, 0.495545,\
        0.487414, 0.479436, 0.471667, 0.464217, 0.457136, 0.450421, 0.444052, 0.438014, 0.432287, 0.426844,\
        0.421645, 0.416642, 0.411788, 0.407054, 0.402444, 0.398145, 0.393990, 0.389710, 0.385297, 0.380746,\
        0.376051, 0.371209, 0.366219, 0.361083, 0.355801, 0.350370, 0.344789, 0.339053, 0.333160, 0.327106,\
        0.320888],\
        [0.225814, 0.231172, 0.236535, 0.241908, 0.247290, 0.252676, 0.258062, 0.263434, 0.268778, 0.274072,\
        0.279290, 0.284398, 0.289364, 0.294155, 0.298767, 0.303215, 0.307645, 0.312165, 0.316895, 0.321964,\
        0.327494, 0.333599, 0.340383, 0.347943, 0.356375, 0.365772, 0.376241, 0.387924, 0.400941, 0.415025,\
        0.429836, 0.445056, 0.460145, 0.474481, 0.487362, 0.497918, 0.505317, 0.509950, 0.512829, 0.514705,\
        0.516031, 0.517051, 0.517890, 0.518592, 0.518827, 0.520782, 0.547739, 0.588718, 0.632911, 0.676568,\
        0.713104, 0.742610, 0.767203, 0.788265, 0.804460, 0.813757, 0.811924, 0.805638, 0.798130, 0.788582,\
        0.776646, 0.765922, 0.755689, 0.745140, 0.734941, 0.725429, 0.715891, 0.704200, 0.692747, 0.680656,\
        0.668435, 0.656989, 0.646099, 0.635620, 0.625630, 0.616247, 0.607355, 0.598348, 0.588899, 0.579491,\
        0.570265, 0.560933, 0.551526, 0.542172, 0.532930, 0.523837, 0.514922, 0.506177, 0.497403, 0.488887,\
        0.480940, 0.473335, 0.465997, 0.458958, 0.452248, 0.445877, 0.439839, 0.434120, 0.428699, 0.423544,\
        0.418606, 0.413826, 0.409144, 0.404532, 0.400027, 0.395709, 0.391548, 0.387337, 0.383010, 0.378549,\
        0.373946, 0.369197, 0.364300, 0.359252, 0.354053, 0.348699, 0.343190, 0.337520, 0.331687, 0.325688,\
        0.319519],\
        [0.225983, 0.231343, 0.236702, 0.242063, 0.247425, 0.252784, 0.258134, 0.263464, 0.268760, 0.274004,\
        0.279171, 0.284230, 0.289140, 0.293844, 0.298295, 0.302688, 0.307080, 0.311490, 0.316077, 0.320981,\
        0.326316, 0.332187, 0.338694, 0.345928, 0.353982, 0.362947, 0.372927, 0.384038, 0.396328, 0.409430,\
        0.423501, 0.438230, 0.452982, 0.467185, 0.480284, 0.491693, 0.500823, 0.507662, 0.513467, 0.519356,\
        0.525933, 0.533323, 0.540936, 0.547525, 0.553936, 0.567455, 0.592619, 0.619596, 0.651187, 0.685939,\
        0.716067, 0.739472, 0.758242, 0.773279, 0.784793, 0.790946, 0.786621, 0.779348, 0.771707, 0.762953,\
        0.753317, 0.744277, 0.735611, 0.726215, 0.716523, 0.707794, 0.699573, 0.689485, 0.677733, 0.666356,\
        0.655080, 0.644149, 0.633506, 0.623117, 0.613241, 0.604122, 0.595802, 0.587369, 0.578600, 0.569708,\
        0.560816, 0.551873, 0.542860, 0.533828, 0.524849, 0.516025, 0.507440, 0.499112, 0.490860, 0.482746,\
        0.475032, 0.467697, 0.460682, 0.453978, 0.447592, 0.441529, 0.435787, 0.430356, 0.425220, 0.420347,\
        0.415684, 0.411156, 0.406686, 0.402228, 0.397824, 0.393532, 0.389323, 0.385109, 0.380819, 0.376417,\
        0.371886, 0.367214, 0.362397, 0.357429, 0.352307, 0.347026, 0.341584, 0.335977, 0.330202, 0.324254,\
        0.318131],\
        [0.226134, 0.231490, 0.236839, 0.242182, 0.247518, 0.252845, 0.258156, 0.263443, 0.268692, 0.273888,\
        0.279012, 0.284041, 0.288945, 0.293679, 0.298178, 0.302465, 0.306638, 0.310865, 0.315303, 0.320055,\
        0.325217, 0.330882, 0.337140, 0.344080, 0.351782, 0.360325, 0.369772, 0.380152, 0.391417, 0.403649,\
        0.417248, 0.431506, 0.445757, 0.459595, 0.472633, 0.484503, 0.494939, 0.503970, 0.512779, 0.522697,\
        0.534521, 0.548457, 0.563631, 0.576802, 0.585471, 0.600378, 0.621152, 0.641239, 0.664244, 0.689307,\
        0.713261, 0.732086, 0.744755, 0.753497, 0.759507, 0.761533, 0.757951, 0.751535, 0.744593, 0.737007,\
        0.729224, 0.722467, 0.716098, 0.708017, 0.698163, 0.690001, 0.682019, 0.672940, 0.662628, 0.652217,\
        0.642020, 0.631999, 0.621929, 0.611713, 0.601962, 0.593195, 0.585099, 0.577089, 0.568877, 0.560502,\
        0.552032, 0.543479, 0.534838, 0.526121, 0.517372, 0.508751, 0.500450, 0.492420, 0.484571, 0.476885,\
        0.469484, 0.462416, 0.455674, 0.449251, 0.443143, 0.437350, 0.431867, 0.426689, 0.421808, 0.417203,\
        0.412822, 0.408569, 0.404333, 0.400053, 0.395717, 0.391435, 0.387204, 0.382978, 0.378704, 0.374342,\
        0.369866, 0.365260, 0.360513, 0.355617, 0.350565, 0.345352, 0.339974, 0.334425, 0.328703, 0.322803,\
        0.316723],\
        [0.226257, 0.231603, 0.236935, 0.242254, 0.247560, 0.252849, 0.258117, 0.263355, 0.268553, 0.273698,\
        0.278773, 0.283765, 0.288658, 0.293448, 0.298110, 0.302248, 0.306107, 0.310184, 0.314514, 0.319147,\
        0.324162, 0.329643, 0.335678, 0.342348, 0.349726, 0.357874, 0.366831, 0.376596, 0.387138, 0.398636,\
        0.411651, 0.425112, 0.438564, 0.451786, 0.464492, 0.476433, 0.487542, 0.498164, 0.509291, 0.522326,\
        0.538017, 0.556671, 0.577438, 0.596882, 0.603584, 0.614524, 0.630404, 0.647873, 0.667310, 0.688403,\
        0.708062, 0.720741, 0.726621, 0.729154, 0.730498, 0.730502, 0.727202, 0.722208, 0.717203, 0.711226,\
        0.704619, 0.699997, 0.695534, 0.689174, 0.680419, 0.672271, 0.664423, 0.656059, 0.647070, 0.637973,\
        0.629105, 0.620368, 0.611331, 0.601627, 0.592115, 0.583545, 0.575530, 0.567718, 0.559873, 0.551924,\
        0.543867, 0.535706, 0.527445, 0.519078, 0.510583, 0.502058, 0.494026, 0.486274, 0.478706, 0.471336,\
        0.464222, 0.457411, 0.450916, 0.444738, 0.438873, 0.433314, 0.428053, 0.423087, 0.418417, 0.414044,\
        0.409938, 0.405978, 0.401960, 0.397803, 0.393560, 0.389316, 0.385097, 0.380880, 0.376627, 0.372301,\
        0.367874, 0.363327, 0.358643, 0.353813, 0.348826, 0.343676, 0.338357, 0.332863, 0.327191, 0.321336,\
        0.315295],\
        [0.226343, 0.231673, 0.236982, 0.242271, 0.247540, 0.252786, 0.258005, 0.263189, 0.268328, 0.273411,\
        0.278423, 0.283345, 0.288156, 0.292821, 0.297238, 0.301230, 0.305243, 0.309376, 0.313678, 0.318228,\
        0.323115, 0.328431, 0.334262, 0.340686, 0.347770, 0.355569, 0.364127, 0.373480, 0.383701, 0.394938,\
        0.406920, 0.419019, 0.431422, 0.443884, 0.456073, 0.467793, 0.479090, 0.490370, 0.502686, 0.517194,\
        0.534452, 0.554786, 0.576411, 0.594761, 0.605839, 0.614920, 0.627170, 0.642402, 0.659580, 0.678074,\
        0.695058, 0.703067, 0.703696, 0.701961, 0.700354, 0.700172, 0.697015, 0.693372, 0.690222, 0.686159,\
        0.681284, 0.677452, 0.673618, 0.668366, 0.661570, 0.654364, 0.647095, 0.639489, 0.631525, 0.623659,\
        0.616175, 0.608883, 0.601114, 0.592258, 0.583204, 0.574739, 0.566793, 0.559128, 0.551553, 0.543954,\
        0.536270, 0.528479, 0.520590, 0.512630, 0.504537, 0.496273, 0.488282, 0.480640, 0.473247, 0.466096,\
        0.459213, 0.452628, 0.446356, 0.440400, 0.434752, 0.429401, 0.424332, 0.419535, 0.415016, 0.410801,\
        0.406918, 0.403275, 0.399473, 0.395410, 0.391265, 0.387101, 0.382937, 0.378762, 0.374549, 0.370268,\
        0.365893, 0.361403, 0.356781, 0.352013, 0.347088, 0.341997, 0.336733, 0.331291, 0.325665, 0.319851,\
        0.313846],\
        [0.226386, 0.231693, 0.236973, 0.242226, 0.247452, 0.252649, 0.257812, 0.262935, 0.268009, 0.273021,\
        0.277954, 0.282785, 0.287476, 0.291964, 0.296162, 0.300175, 0.304336, 0.308534, 0.312815, 0.317284,\
        0.322050, 0.327211, 0.332853, 0.339049, 0.345862, 0.353345, 0.361543, 0.370506, 0.380298, 0.390928,\
        0.401791, 0.412728, 0.424292, 0.436057, 0.447651, 0.458933, 0.470011, 0.481322, 0.493680, 0.507490,\
        0.524372, 0.544411, 0.565009, 0.582634, 0.595413, 0.603776, 0.614726, 0.628109, 0.642297, 0.657203,\
        0.670057, 0.676436, 0.676739, 0.674586, 0.672571, 0.671419, 0.669388, 0.666936, 0.664665, 0.661995,\
        0.658802, 0.655577, 0.652082, 0.647748, 0.642421, 0.636476, 0.630243, 0.623596, 0.616406, 0.609536,\
        0.603284, 0.597109, 0.590319, 0.582507, 0.574302, 0.566262, 0.558542, 0.551081, 0.543771, 0.536501,\
        0.529168, 0.521702, 0.514110, 0.506476, 0.498817, 0.490814, 0.482940, 0.475370, 0.468094, 0.461104,\
        0.454407, 0.448018, 0.441949, 0.436197, 0.430753, 0.425598, 0.420706, 0.416051, 0.411624, 0.407463,\
        0.403650, 0.400229, 0.396665, 0.392786, 0.388799, 0.384762, 0.380695, 0.376595, 0.372445, 0.368223,\
        0.363908, 0.359478, 0.354918, 0.350211, 0.345346, 0.340312, 0.335101, 0.329706, 0.324123, 0.318346,\
        0.312373],\
        [0.226379, 0.231659, 0.236903, 0.242115, 0.247291, 0.252432, 0.257534, 0.262589, 0.267591, 0.272527,\
        0.277381, 0.282127, 0.286731, 0.291151, 0.295374, 0.299482, 0.303600, 0.307740, 0.311939, 0.316301,\
        0.320937, 0.325945, 0.331407, 0.337390, 0.343950, 0.351129, 0.358960, 0.367458, 0.376593, 0.386197,\
        0.395974, 0.406322, 0.417422, 0.428585, 0.439534, 0.450220, 0.460756, 0.471430, 0.482634, 0.495011,\
        0.510472, 0.529751, 0.549252, 0.566099, 0.578818, 0.588213, 0.598952, 0.608950, 0.619286, 0.630648,\
        0.640580, 0.646413, 0.648050, 0.647295, 0.646033, 0.645041, 0.643910, 0.642501, 0.641065, 0.639509,\
        0.637464, 0.634797, 0.631673, 0.628155, 0.624006, 0.619192, 0.614187, 0.608932, 0.602388, 0.596292,\
        0.590998, 0.585566, 0.579525, 0.572698, 0.565367, 0.557910, 0.550551, 0.543368, 0.536357, 0.529444,\
        0.522482, 0.515315, 0.507925, 0.500433, 0.492947, 0.485316, 0.477689, 0.470266, 0.463120, 0.456274,\
        0.449740, 0.443530, 0.437648, 0.432092, 0.426847, 0.421891, 0.417186, 0.412683, 0.408336, 0.404183,\
        0.400337, 0.396944, 0.393577, 0.389967, 0.386189, 0.382313, 0.378371, 0.374369, 0.370301, 0.366153,\
        0.361905, 0.357542, 0.353046, 0.348401, 0.343596, 0.338617, 0.333457, 0.328108, 0.322564, 0.316821,\
        0.310877],\
        [0.226319, 0.231566, 0.236770, 0.241933, 0.247055, 0.252134, 0.257168, 0.262151, 0.267077, 0.271935,\
        0.276712, 0.281390, 0.285949, 0.290371, 0.294657, 0.298826, 0.302922, 0.306950, 0.311010, 0.315234,\
        0.319732, 0.324593, 0.329887, 0.335672, 0.341994, 0.348882, 0.356350, 0.364384, 0.372916, 0.381819,\
        0.391021, 0.400851, 0.411331, 0.421773, 0.431997, 0.442016, 0.451914, 0.461869, 0.472194, 0.483527,\
        0.497514, 0.515184, 0.533101, 0.549008, 0.561893, 0.572247, 0.581386, 0.588825, 0.595936, 0.604334,\
        0.612341, 0.617705, 0.620531, 0.621233, 0.621096, 0.621043, 0.620925, 0.620286, 0.619556, 0.619087,\
        0.618017, 0.615689, 0.612799, 0.610107, 0.607104, 0.603124, 0.599177, 0.595256, 0.590087, 0.584694,\
        0.579796, 0.574814, 0.569344, 0.563245, 0.556629, 0.549726, 0.542747, 0.535849, 0.529144, 0.522641,\
        0.516140, 0.509300, 0.502081, 0.494717, 0.487321, 0.479877, 0.472465, 0.465217, 0.458221, 0.451524,\
        0.445150, 0.439110, 0.433410, 0.428045, 0.423001, 0.418254, 0.413764, 0.409471, 0.405276, 0.401132,\
        0.397393, 0.393953, 0.390596, 0.387128, 0.383518, 0.379796, 0.375983, 0.372091, 0.368117, 0.364051,\
        0.359879, 0.355587, 0.351158, 0.346578, 0.341834, 0.336911, 0.331801, 0.326494, 0.320986, 0.315274,\
        0.309354],\
        [0.226204, 0.231412, 0.236571, 0.241681, 0.246742, 0.251754, 0.256715, 0.261620, 0.266465, 0.271242,\
        0.275944, 0.280560, 0.285085, 0.289516, 0.293861, 0.298114, 0.302169, 0.306034, 0.309933, 0.314020,\
        0.318390, 0.323117, 0.328261, 0.333867, 0.339970, 0.346592, 0.353735, 0.361378, 0.369475, 0.377977,\
        0.386908, 0.396365, 0.406093, 0.415670, 0.425149, 0.434560, 0.443932, 0.453418, 0.463360, 0.474409,\
        0.487533, 0.502848, 0.518598, 0.533444, 0.546176, 0.556450, 0.564623, 0.571093, 0.576891, 0.582976,\
        0.588863, 0.593402, 0.596726, 0.598304, 0.599100, 0.600010, 0.600940, 0.600786, 0.600466, 0.601020,\
        0.600821, 0.598757, 0.596120, 0.594113, 0.591956, 0.588945, 0.585778, 0.582450, 0.578443, 0.574012,\
        0.569552, 0.564930, 0.559902, 0.554343, 0.548276, 0.541823, 0.535151, 0.528458, 0.521968, 0.515874,\
        0.510040, 0.503606, 0.496515, 0.489214, 0.481851, 0.474506, 0.467250, 0.460167, 0.453331, 0.446793,\
        0.440583, 0.434716, 0.429195, 0.424019, 0.419175, 0.414641, 0.410379, 0.406350, 0.402474, 0.398496,\
        0.394697, 0.391182, 0.387782, 0.384361, 0.380855, 0.377253, 0.373558, 0.369772, 0.365894, 0.361915,\
        0.357824, 0.353607, 0.349250, 0.344737, 0.340055, 0.335189, 0.330129, 0.324864, 0.319389, 0.313703,\
        0.307805],\
        [0.226032, 0.231198, 0.236306, 0.241358, 0.246354, 0.251293, 0.256176, 0.260998, 0.265756, 0.270447,\
        0.275066, 0.279610, 0.284080, 0.288478, 0.292812, 0.297059, 0.301072, 0.304811, 0.308602, 0.312593,\
        0.316869, 0.321490, 0.326505, 0.331954, 0.337865, 0.344251, 0.351116, 0.358443, 0.366211, 0.374403,\
        0.383027, 0.392056, 0.401164, 0.409999, 0.418922, 0.427902, 0.436917, 0.446132, 0.455892, 0.466675,\
        0.478880, 0.492282, 0.506132, 0.520301, 0.532588, 0.542417, 0.550076, 0.556094, 0.561232, 0.566104,\
        0.570671, 0.574588, 0.577668, 0.579745, 0.581241, 0.582645, 0.583839, 0.584368, 0.584711, 0.585310,\
        0.585296, 0.584006, 0.582125, 0.580391, 0.578568, 0.576335, 0.573794, 0.570961, 0.567679, 0.563991,\
        0.560033, 0.555790, 0.551152, 0.546030, 0.540403, 0.534310, 0.527854, 0.521233, 0.514748, 0.508794,\
        0.503765, 0.498095, 0.490942, 0.483648, 0.476343, 0.469104, 0.461992, 0.455072, 0.448405, 0.442038,\
        0.436002, 0.430314, 0.424976, 0.419986, 0.415337, 0.411009, 0.406951, 0.403117, 0.399459, 0.395744,\
        0.392017, 0.388475, 0.385053, 0.381655, 0.378217, 0.374706, 0.371110, 0.367423, 0.363638, 0.359746,\
        0.355737, 0.351598, 0.347314, 0.342872, 0.338255, 0.333449, 0.328440, 0.323217, 0.317773, 0.312109,\
        0.306227],\
        [0.225803, 0.230923, 0.235977, 0.240966, 0.245893, 0.250755, 0.255554, 0.260287, 0.264953, 0.269549,\
        0.274073, 0.278525, 0.282905, 0.287212, 0.291440, 0.295551, 0.299458, 0.303191, 0.306973, 0.310932,\
        0.315153, 0.319698, 0.324611, 0.329926, 0.335667, 0.341848, 0.348472, 0.355531, 0.363010, 0.370889,\
        0.379127, 0.387612, 0.396099, 0.404555, 0.413249, 0.421984, 0.430728, 0.439727, 0.449318, 0.459819,\
        0.471374, 0.483786, 0.496912, 0.510198, 0.521416, 0.530524, 0.537741, 0.543432, 0.548168, 0.552413,\
        0.556306, 0.559762, 0.562660, 0.564957, 0.566815, 0.568421, 0.569735, 0.570653, 0.571330, 0.571840,\
        0.571842, 0.571081, 0.569815, 0.568397, 0.566847, 0.565047, 0.562966, 0.560573, 0.557802, 0.554644,\
        0.551128, 0.547252, 0.542977, 0.538250, 0.533027, 0.527270, 0.520994, 0.514356, 0.507722, 0.501561,\
        0.496488, 0.491309, 0.484772, 0.477750, 0.470636, 0.463571, 0.456637, 0.449899, 0.443414, 0.437230,\
        0.431383, 0.425889, 0.420745, 0.415936, 0.411469, 0.407346, 0.403475, 0.399776, 0.396230, 0.392706,\
        0.389175, 0.385715, 0.382329, 0.378970, 0.375591, 0.372156, 0.368646, 0.365047, 0.361349, 0.357542,\
        0.353614, 0.349553, 0.345345, 0.340975, 0.336428, 0.331687, 0.326734, 0.321555, 0.316140, 0.310493,\
        0.304622],\
        [0.225519, 0.230589, 0.235585, 0.240510, 0.245363, 0.250145, 0.254856, 0.259496, 0.264062, 0.268555,\
        0.272972, 0.277312, 0.281574, 0.285751, 0.289829, 0.293779, 0.297578, 0.301312, 0.305110, 0.309066,\
        0.313261, 0.317752, 0.322581, 0.327780, 0.333371, 0.339369, 0.345777, 0.352592, 0.359799, 0.367371,\
        0.375247, 0.383316, 0.391435, 0.399634, 0.408256, 0.416700, 0.425096, 0.433830, 0.443222, 0.453492,\
        0.464717, 0.476836, 0.489656, 0.501883, 0.511767, 0.520322, 0.527176, 0.532560, 0.537031, 0.540988,\
        0.544584, 0.547820, 0.550641, 0.553026, 0.555029, 0.556712, 0.558077, 0.559112, 0.559854, 0.560291,\
        0.560295, 0.559783, 0.558863, 0.557706, 0.556379, 0.554857, 0.553100, 0.551057, 0.548671, 0.545901,\
        0.542735, 0.539175, 0.535223, 0.530869, 0.526071, 0.520717, 0.514682, 0.508060, 0.501290, 0.495171,\
        0.489707, 0.484255, 0.478138, 0.471513, 0.464694, 0.457872, 0.451158, 0.444630, 0.438347, 0.432361,\
        0.426716, 0.421440, 0.416518, 0.411895, 0.407568, 0.403650, 0.400012, 0.396441, 0.392997, 0.389604,\
        0.386228, 0.382883, 0.379572, 0.376275, 0.372958, 0.369595, 0.366162, 0.362644, 0.359028, 0.355301,\
        0.351451, 0.347467, 0.343334, 0.339039, 0.334565, 0.329894, 0.325006, 0.319879, 0.314496, 0.308859,\
        0.302992],\
        [0.225181, 0.230199, 0.235136, 0.239992, 0.244769, 0.249468, 0.254089, 0.258631, 0.263094, 0.267477,\
        0.271778, 0.275996, 0.280127, 0.284163, 0.288096, 0.291915, 0.295636, 0.299331, 0.303113, 0.307059,\
        0.311230, 0.315677, 0.320433, 0.325526, 0.330977, 0.336802, 0.343007, 0.349591, 0.356540, 0.363829,\
        0.371414, 0.379235, 0.387261, 0.395545, 0.403884, 0.411721, 0.419629, 0.428044, 0.437211, 0.447251,\
        0.458169, 0.469818, 0.481746, 0.492863, 0.502649, 0.511262, 0.517772, 0.522805, 0.527152, 0.531072,\
        0.534641, 0.537868, 0.540722, 0.543175, 0.545235, 0.546927, 0.548267, 0.549266, 0.549940, 0.550276,\
        0.550232, 0.549793, 0.549020, 0.548010, 0.546829, 0.545490, 0.543966, 0.542199, 0.540112, 0.537633,\
        0.534726, 0.531399, 0.527696, 0.523668, 0.519322, 0.514519, 0.508889, 0.502335, 0.495645, 0.489305,\
        0.483402, 0.477585, 0.471509, 0.465130, 0.458589, 0.452032, 0.445568, 0.439275, 0.433213, 0.427434,\
        0.421999, 0.416964, 0.412331, 0.407967, 0.403705, 0.399891, 0.396476, 0.393091, 0.389766, 0.386489,\
        0.383235, 0.380000, 0.376777, 0.373553, 0.370306, 0.367012, 0.363652, 0.360209, 0.356669, 0.353017,\
        0.349242, 0.345331, 0.341271, 0.337050, 0.332652, 0.328059, 0.323248, 0.318190, 0.312853, 0.307224,\
        0.301349],\
        [0.224791, 0.229756, 0.234631, 0.239418, 0.244119, 0.248733, 0.253262, 0.257705, 0.262063, 0.266334,\
        0.270517, 0.274610, 0.278610, 0.282513, 0.286317, 0.290028, 0.293675, 0.297321, 0.301058, 0.304968,\
        0.309109, 0.313511, 0.318192, 0.323178, 0.328490, 0.334145, 0.340151, 0.346508, 0.353205, 0.360224,\
        0.367538, 0.375127, 0.382983, 0.391081, 0.399016, 0.406357, 0.413890, 0.422028, 0.430974, 0.440759,\
        0.451288, 0.462343, 0.473509, 0.484216, 0.494179, 0.502901, 0.508636, 0.513389, 0.517906, 0.522032,\
        0.525784, 0.529182, 0.532184, 0.534741, 0.536841, 0.538507, 0.539769, 0.540657, 0.541196, 0.541394,\
        0.541248, 0.540769, 0.540006, 0.539029, 0.537908, 0.536675, 0.535320, 0.533779, 0.531942, 0.529691,\
        0.526956, 0.523757, 0.520186, 0.516364, 0.512392, 0.508262, 0.503328, 0.496874, 0.490055, 0.483477,\
        0.477213, 0.471107, 0.464966, 0.458715, 0.452396, 0.446095, 0.439896, 0.433864, 0.428045, 0.422481,\
        0.417240, 0.412432, 0.408148, 0.404282, 0.400267, 0.396427, 0.393036, 0.389764, 0.386548, 0.383373,\
        0.380222, 0.377084, 0.373948, 0.370801, 0.367624, 0.364400, 0.361110, 0.357737, 0.354266, 0.350684,\
        0.346978, 0.343135, 0.339144, 0.334993, 0.330669, 0.326158, 0.321438, 0.316476, 0.311226, 0.305628,\
        0.299724],\
        [0.224352, 0.229264, 0.234077, 0.238795, 0.243418, 0.247948, 0.252385, 0.256730, 0.260983, 0.265143,\
        0.269211, 0.273184, 0.277062, 0.280845, 0.284539, 0.288157, 0.291732, 0.295317, 0.298990, 0.302844,\
        0.306949, 0.311295, 0.315888, 0.320755, 0.325920, 0.331401, 0.337205, 0.343331, 0.349771, 0.356509,\
        0.363524, 0.370792, 0.378268, 0.385832, 0.393218, 0.400369, 0.407692, 0.415615, 0.424380, 0.433895,\
        0.443977, 0.454423, 0.464951, 0.475224, 0.484894, 0.493081, 0.498663, 0.503811, 0.508871, 0.513347,\
        0.517433, 0.521165, 0.524431, 0.527146, 0.529300, 0.530931, 0.532092, 0.532836, 0.533210, 0.533248,\
        0.532963, 0.532376, 0.531528, 0.530490, 0.529342, 0.528146, 0.526909, 0.525567, 0.523968, 0.521914,\
        0.519278, 0.516094, 0.512523, 0.508740, 0.504874, 0.501095, 0.496859, 0.490645, 0.483899, 0.477248,\
        0.470826, 0.464582, 0.458417, 0.452270, 0.446147, 0.440096, 0.434175, 0.428436, 0.422901, 0.417575,\
        0.412499, 0.407835, 0.403785, 0.400422, 0.396940, 0.393187, 0.389740, 0.386495, 0.383350, 0.380258,\
        0.377195, 0.374143, 0.371088, 0.368016, 0.364909, 0.361752, 0.358528, 0.355221, 0.351815, 0.348297,\
        0.344654, 0.340872, 0.336941, 0.332851, 0.328593, 0.324157, 0.319529, 0.314684, 0.309579, 0.304156,\
        0.298289],\
        [0.223868, 0.228726, 0.233478, 0.238127, 0.242674, 0.247120, 0.251468, 0.255717, 0.259869, 0.263923,\
        0.267880, 0.271742, 0.275509, 0.279187, 0.282786, 0.286323, 0.289833, 0.293360, 0.296969, 0.300756,\
        0.304809, 0.309070, 0.313547, 0.318275, 0.323281, 0.328580, 0.334175, 0.340065, 0.346236, 0.352673,\
        0.359352, 0.366233, 0.373253, 0.380297, 0.387244, 0.394119, 0.401177, 0.408871, 0.417524, 0.426737,\
        0.436279, 0.446050, 0.455880, 0.465486, 0.474447, 0.482149, 0.488416, 0.494187, 0.499667, 0.504516,\
        0.509079, 0.513306, 0.516944, 0.519868, 0.522094, 0.523703, 0.524769, 0.525366, 0.525575, 0.525453,\
        0.525023, 0.524298, 0.523309, 0.522132, 0.520880, 0.519651, 0.518480, 0.517323, 0.515984, 0.514133,\
        0.511539, 0.508271, 0.504608, 0.500789, 0.496906, 0.493022, 0.488639, 0.483092, 0.476818, 0.470389,\
        0.464049, 0.457849, 0.451763, 0.445761, 0.439845, 0.434047, 0.428422, 0.423014, 0.417838, 0.412840,\
        0.407939, 0.403363, 0.399428, 0.396208, 0.393047, 0.389696, 0.386398, 0.383228, 0.380154, 0.377138,\
        0.374153, 0.371179, 0.368198, 0.365196, 0.362156, 0.359063, 0.355901, 0.352655, 0.349309, 0.345849,\
        0.342262, 0.338534, 0.334653, 0.330610, 0.326399, 0.322018, 0.317462, 0.312713, 0.307725, 0.302427,\
        0.296650],\
        [0.223340, 0.228145, 0.232838, 0.237419, 0.241892, 0.246258, 0.250519, 0.254677, 0.258732, 0.262687,\
        0.266542, 0.270302, 0.273971, 0.277556, 0.281072, 0.284540, 0.287992, 0.291475, 0.295047, 0.298784,\
        0.302731, 0.306857, 0.311181, 0.315749, 0.320584, 0.325695, 0.331078, 0.336724, 0.342618, 0.348743,\
        0.355068, 0.361555, 0.368142, 0.374755, 0.381342, 0.387937, 0.394704, 0.402200, 0.410738, 0.419457,\
        0.428270, 0.437264, 0.446337, 0.455231, 0.463600, 0.471128, 0.477808, 0.483980, 0.489777, 0.495164,\
        0.500327, 0.505153, 0.509225, 0.512393, 0.514712, 0.516348, 0.517359, 0.517823, 0.517889, 0.517640,\
        0.517091, 0.516239, 0.515091, 0.513722, 0.512302, 0.510988, 0.509810, 0.508788, 0.507736, 0.506107,\
        0.503520, 0.500127, 0.496393, 0.492540, 0.488615, 0.484539, 0.480023, 0.474795, 0.468989, 0.462927,\
        0.456845, 0.450846, 0.444954, 0.439166, 0.433488, 0.427957, 0.422634, 0.417582, 0.412844, 0.408379,\
        0.403871, 0.399280, 0.395597, 0.392326, 0.389222, 0.386096, 0.382979, 0.379926, 0.376939, 0.374001,\
        0.371090, 0.368187, 0.365275, 0.362338, 0.359360, 0.356327, 0.353223, 0.350033, 0.346742, 0.343336,\
        0.339798, 0.336116, 0.332274, 0.328263, 0.324074, 0.319712, 0.315184, 0.310489, 0.305557, 0.300249,\
        0.294470],\
        [0.222772, 0.227527, 0.232161, 0.236678, 0.241080, 0.245369, 0.249548, 0.253619, 0.257584, 0.261446,\
        0.265208, 0.268876, 0.272456, 0.275959, 0.279401, 0.282806, 0.286205, 0.289642, 0.293171, 0.296845,\
        0.300684, 0.304633, 0.308782, 0.313182, 0.317845, 0.322768, 0.327937, 0.333337, 0.338951, 0.344760,\
        0.350738, 0.356851, 0.363058, 0.369319, 0.375631, 0.382061, 0.388805, 0.396227, 0.404270, 0.412084,\
        0.419951, 0.428130, 0.436472, 0.444723, 0.452619, 0.459976, 0.466791, 0.473202, 0.479312, 0.485173,\
        0.490833, 0.496134, 0.500692, 0.504163, 0.506613, 0.508411, 0.509447, 0.509800, 0.509776, 0.509463,\
        0.508847, 0.507914, 0.506638, 0.505063, 0.503449, 0.502047, 0.500797, 0.499772, 0.498884, 0.497339,\
        0.494773, 0.491435, 0.487748, 0.483900, 0.479921, 0.475733, 0.471180, 0.466150, 0.460703, 0.455021,\
        0.449281, 0.443593, 0.437998, 0.432497, 0.427097, 0.421842, 0.416814, 0.412100, 0.407773, 0.403918,\
        0.400361, 0.395988, 0.392094, 0.388705, 0.385569, 0.382529, 0.379536, 0.376593, 0.373697, 0.370838,\
        0.367999, 0.365163, 0.362315, 0.359438, 0.356518, 0.353539, 0.350489, 0.347350, 0.344110, 0.340752,\
        0.337260, 0.333618, 0.329809, 0.325817, 0.321628, 0.317241, 0.312677, 0.307980, 0.303112, 0.297799,\
        0.292015],\
        [0.222167, 0.226872, 0.231451, 0.235906, 0.240240, 0.244457, 0.248559, 0.252549, 0.256430, 0.260208,\
        0.263886, 0.267471, 0.270971, 0.274400, 0.277774, 0.281117, 0.284458, 0.287835, 0.291291, 0.294861,\
        0.298545, 0.302342, 0.306341, 0.310586, 0.315086, 0.319827, 0.324784, 0.329937, 0.335271, 0.340770,\
        0.346414, 0.352181, 0.358048, 0.364005, 0.370075, 0.376343, 0.382964, 0.390093, 0.397494, 0.404217,\
        0.411268, 0.418753, 0.426457, 0.434143, 0.441606, 0.448722, 0.455486, 0.461979, 0.468283, 0.474428,\
        0.480365, 0.485907, 0.490748, 0.494562, 0.497339, 0.499463, 0.500617, 0.500929, 0.500921, 0.500615,\
        0.499970, 0.498994, 0.497661, 0.495980, 0.494287, 0.492796, 0.491494, 0.490363, 0.489225, 0.487580,\
        0.485159, 0.482071, 0.478570, 0.474822, 0.470871, 0.466679, 0.462172, 0.457315, 0.452158, 0.446828,\
        0.441461, 0.436154, 0.430941, 0.425806, 0.420727, 0.415749, 0.410999, 0.406594, 0.402577, 0.399028,\
        0.395822, 0.392164, 0.388483, 0.385109, 0.381981, 0.378995, 0.376090, 0.373239, 0.370428, 0.367645,\
        0.364875, 0.362103, 0.359313, 0.356491, 0.353623, 0.350695, 0.347692, 0.344601, 0.341406, 0.338092,\
        0.334643, 0.331040, 0.327264, 0.323291, 0.319097, 0.314661, 0.309987, 0.305153, 0.300295, 0.295050,\
        0.289301],\
        [0.221525, 0.226184, 0.230711, 0.235107, 0.239378, 0.243527, 0.247557, 0.251472, 0.255277, 0.258977,\
        0.262578, 0.266088, 0.269517, 0.272878, 0.276187, 0.279468, 0.282745, 0.286049, 0.289410, 0.292849,\
        0.296379, 0.300037, 0.303893, 0.307990, 0.312344, 0.316913, 0.321658, 0.326561, 0.331614, 0.336808,\
        0.342132, 0.347572, 0.353121, 0.358780, 0.364580, 0.370576, 0.376835, 0.383333, 0.389785, 0.395986,\
        0.402467, 0.409363, 0.416500, 0.423657, 0.430677, 0.437463, 0.444016, 0.450425, 0.456771, 0.463038,\
        0.469101, 0.474741, 0.479695, 0.483745, 0.486870, 0.489162, 0.490522, 0.491075, 0.491165, 0.490894,\
        0.490258, 0.489267, 0.487934, 0.486343, 0.484699, 0.483169, 0.481786, 0.480492, 0.479111, 0.477359,\
        0.475053, 0.472210, 0.468953, 0.465388, 0.461561, 0.457461, 0.453067, 0.448387, 0.443485, 0.438474,\
        0.433482, 0.428598, 0.423846, 0.419169, 0.414471, 0.409760, 0.405271, 0.401188, 0.397507, 0.394200,\
        0.391097, 0.387874, 0.384577, 0.381390, 0.378359, 0.375453, 0.372631, 0.369863, 0.367132, 0.364421,\
        0.361716, 0.359003, 0.356267, 0.353495, 0.350673, 0.347788, 0.344828, 0.341778, 0.338624, 0.335351,\
        0.331942, 0.328380, 0.324643, 0.320705, 0.316530, 0.312075, 0.307310, 0.302235, 0.297227, 0.292067,\
        0.286404],\
        [0.220848, 0.225464, 0.229941, 0.234284, 0.238495, 0.242581, 0.246544, 0.250391, 0.254126, 0.257756,\
        0.261288, 0.264730, 0.268093, 0.271391, 0.274640, 0.277859, 0.281070, 0.284297, 0.287562, 0.290883,\
        0.294282, 0.297797, 0.301488, 0.305441, 0.309676, 0.314074, 0.318596, 0.323240, 0.328008, 0.332900,\
        0.337912, 0.343038, 0.348274, 0.353625, 0.359109, 0.364747, 0.370541, 0.376424, 0.382264, 0.388049,\
        0.393995, 0.400295, 0.406825, 0.413440, 0.419983, 0.426341, 0.432521, 0.438681, 0.444942, 0.451239,\
        0.457355, 0.463031, 0.468035, 0.472212, 0.475514, 0.477949, 0.479526, 0.480351, 0.480613, 0.480431,\
        0.479854, 0.478916, 0.477663, 0.476189, 0.474633, 0.473112, 0.471665, 0.470242, 0.468712, 0.466905,\
        0.464699, 0.462071, 0.459069, 0.455742, 0.452115, 0.448186, 0.443957, 0.439464, 0.434792, 0.430067,\
        0.425427, 0.420980, 0.416753, 0.412646, 0.408451, 0.404007, 0.399741, 0.396010, 0.392610, 0.389484,\
        0.386506, 0.383526, 0.380521, 0.377556, 0.374673, 0.371876, 0.369145, 0.366461, 0.363805, 0.361162,\
        0.358518, 0.355859, 0.353171, 0.350443, 0.347661, 0.344814, 0.341889, 0.338875, 0.335757, 0.332520,\
        0.329149, 0.325627, 0.321936, 0.318055, 0.313946, 0.309544, 0.304818, 0.299807, 0.294569, 0.289197,\
        0.283461],\
        [0.220136, 0.224713, 0.229144, 0.233436, 0.237593, 0.241620, 0.245522, 0.249306, 0.252978, 0.256544,\
        0.260013, 0.263394, 0.266698, 0.269938, 0.273129, 0.276290, 0.279438, 0.282594, 0.285776, 0.289003,\
        0.292293, 0.295670, 0.299174, 0.303048, 0.307173, 0.311360, 0.315626, 0.319992, 0.324470, 0.329063,\
        0.333770, 0.338588, 0.343514, 0.348548, 0.353693, 0.358947, 0.364291, 0.369677, 0.375054, 0.380446,\
        0.385962, 0.391693, 0.397612, 0.403652, 0.409684, 0.415536, 0.421156, 0.426903, 0.433024, 0.439335,\
        0.445478, 0.451143, 0.456129, 0.460326, 0.463701, 0.466253, 0.468011, 0.469053, 0.469498, 0.469449,\
        0.468977, 0.468140, 0.467000, 0.465646, 0.464181, 0.462694, 0.461222, 0.459735, 0.458145, 0.456344,\
        0.454247, 0.451826, 0.449086, 0.446038, 0.442673, 0.438978, 0.434957, 0.430660, 0.426195, 0.421717,\
        0.417389, 0.413349, 0.409663, 0.406198, 0.402659, 0.398659, 0.394635, 0.391088, 0.387873, 0.384888,\
        0.382037, 0.379232, 0.376437, 0.373665, 0.370937, 0.368261, 0.365629, 0.363028, 0.360445, 0.357867,\
        0.355279, 0.352669, 0.350024, 0.347333, 0.344584, 0.341767, 0.338870, 0.335884, 0.332795, 0.329590,\
        0.326252, 0.322764, 0.319114, 0.315298, 0.311303, 0.307030, 0.302330, 0.297320, 0.291993, 0.286393,\
        0.280488],\
        [0.219390, 0.223930, 0.228320, 0.232564, 0.236671, 0.240644, 0.244490, 0.248217, 0.251831, 0.255340,\
        0.258753, 0.262078, 0.265328, 0.268515, 0.271654, 0.274760, 0.277851, 0.280946, 0.284062, 0.287220,\
        0.290443, 0.293764, 0.297254, 0.301027, 0.304884, 0.308766, 0.312742, 0.316820, 0.321006, 0.325304,\
        0.329716, 0.334236, 0.338859, 0.343578, 0.348386, 0.353272, 0.358215, 0.363187, 0.368172, 0.373192,\
        0.378305, 0.383555, 0.388939, 0.394411, 0.399925, 0.405230, 0.410070, 0.415274, 0.421318, 0.427694,\
        0.433833, 0.439411, 0.444277, 0.448390, 0.451755, 0.454375, 0.456263, 0.457469, 0.458075, 0.458170,\
        0.457828, 0.457115, 0.456100, 0.454861, 0.453483, 0.452045, 0.450589, 0.449103, 0.447527, 0.445783,\
        0.443816, 0.441605, 0.439144, 0.436414, 0.433371, 0.429967, 0.426194, 0.422105, 0.417831, 0.413567,\
        0.409514, 0.405810, 0.402599, 0.399700, 0.396642, 0.393201, 0.389629, 0.386300, 0.383245, 0.380392,\
        0.377667, 0.375006, 0.372374, 0.369762, 0.367176, 0.364617, 0.362083, 0.359565, 0.357052, 0.354534,\
        0.351997, 0.349431, 0.346824, 0.344163, 0.341440, 0.338643, 0.335766, 0.332798, 0.329731, 0.326551,\
        0.323240, 0.319776, 0.316143, 0.312353, 0.308467, 0.304484, 0.299871, 0.294743, 0.289258, 0.283474,\
        0.277402],\
        [0.218609, 0.223115, 0.227466, 0.231668, 0.235727, 0.239651, 0.243447, 0.247122, 0.250684, 0.254141,\
        0.257503, 0.260778, 0.263980, 0.267119, 0.270209, 0.273266, 0.276306, 0.279348, 0.282409, 0.285515,\
        0.288693, 0.291987, 0.295457, 0.299096, 0.302615, 0.306211, 0.309915, 0.313713, 0.317612, 0.321627,\
        0.325758, 0.329996, 0.334329, 0.338746, 0.343236, 0.347784, 0.352373, 0.356987, 0.361623, 0.366294,\
        0.371031, 0.375854, 0.380762, 0.385722, 0.390663, 0.395323, 0.399489, 0.404150, 0.410334, 0.416741,\
        0.422761, 0.428115, 0.432716, 0.436629, 0.439914, 0.442556, 0.444527, 0.445845, 0.446578, 0.446805,\
        0.446596, 0.446016, 0.445128, 0.443998, 0.442700, 0.441316, 0.439909, 0.438482, 0.436981, 0.435337,\
        0.433513, 0.431516, 0.429353, 0.426985, 0.424326, 0.421274, 0.417790, 0.413934, 0.409856, 0.405800,\
        0.402049, 0.398684, 0.395828, 0.393248, 0.390547, 0.387597, 0.384518, 0.381513, 0.378665, 0.375966,\
        0.373374, 0.370844, 0.368348, 0.365871, 0.363408, 0.360956, 0.358513, 0.356073, 0.353626, 0.351162,\
        0.348673, 0.346145, 0.343568, 0.340932, 0.338225, 0.335440, 0.332570, 0.329610, 0.326555, 0.323392,\
        0.320104, 0.316660, 0.313023, 0.309176, 0.305190, 0.301396, 0.297192, 0.291874, 0.286227, 0.280307,\
        0.274110],\
        [0.217790, 0.222266, 0.226581, 0.230743, 0.234760, 0.238639, 0.242388, 0.246016, 0.249531, 0.252942,\
        0.256259, 0.259490, 0.262647, 0.265743, 0.268789, 0.271801, 0.274794, 0.277786, 0.280797, 0.283848,\
        0.286967, 0.290183, 0.293506, 0.296875, 0.300225, 0.303648, 0.307129, 0.310658, 0.314283, 0.318034,\
        0.321907, 0.325884, 0.329947, 0.334083, 0.338279, 0.342521, 0.346797, 0.351095, 0.355414, 0.359761,\
        0.364151, 0.368590, 0.373069, 0.377550, 0.381945, 0.386117, 0.390133, 0.394784, 0.400677, 0.406772,\
        0.412473, 0.417434, 0.421587, 0.425183, 0.428356, 0.431007, 0.433017, 0.434390, 0.435209, 0.435547,\
        0.435462, 0.435012, 0.434255, 0.433234, 0.432002, 0.430663, 0.429326, 0.428015, 0.426652, 0.425140,\
        0.423453, 0.421654, 0.419798, 0.417833, 0.415622, 0.412982, 0.409823, 0.406248, 0.402418, 0.398646,\
        0.395234, 0.392231, 0.389587, 0.387154, 0.384698, 0.382107, 0.379418, 0.376732, 0.374117, 0.371591,\
        0.369139, 0.366737, 0.364362, 0.362000, 0.359644, 0.357287, 0.354926, 0.352555, 0.350167, 0.347753,\
        0.345305, 0.342810, 0.340259, 0.337639, 0.334942, 0.332157, 0.329282, 0.326314, 0.323255, 0.320101,\
        0.316835, 0.313420, 0.309798, 0.305898, 0.301726, 0.297565, 0.293377, 0.288355, 0.282767, 0.276813,\
        0.270552],\
        [0.216932, 0.221380, 0.225663, 0.229789, 0.233766, 0.237604, 0.241311, 0.244897, 0.248370, 0.251740,\
        0.255015, 0.258207, 0.261325, 0.264381, 0.267388, 0.270357, 0.273306, 0.276248, 0.279204, 0.282191,\
        0.285228, 0.288330, 0.291489, 0.294677, 0.297881, 0.301143, 0.304395, 0.307640, 0.311009, 0.314530,\
        0.318177, 0.321919, 0.325735, 0.329613, 0.333540, 0.337507, 0.341503, 0.345518, 0.349550, 0.353602,\
        0.357679, 0.361779, 0.365891, 0.369986, 0.374021, 0.377994, 0.382086, 0.386725, 0.392072, 0.397661,\
        0.402934, 0.407421, 0.410910, 0.414128, 0.417246, 0.419934, 0.421925, 0.423275, 0.424135, 0.424569,\
        0.424593, 0.424265, 0.423651, 0.422755, 0.421574, 0.420241, 0.418983, 0.417856, 0.416699, 0.415351,\
        0.413766, 0.412117, 0.410566, 0.409009, 0.407290, 0.405112, 0.402266, 0.399020, 0.395558, 0.392154,\
        0.389041, 0.386265, 0.383774, 0.381456, 0.379171, 0.376831, 0.374432, 0.372013, 0.369618, 0.367266,\
        0.364957, 0.362677, 0.360413, 0.358153, 0.355889, 0.353614, 0.351325, 0.349014, 0.346678, 0.344307,\
        0.341893, 0.339426, 0.336895, 0.334288, 0.331592, 0.328798, 0.325902, 0.322906, 0.319822, 0.316658,\
        0.313406, 0.310031, 0.306471, 0.302596, 0.298382, 0.294015, 0.289474, 0.284462, 0.278936, 0.272998,\
        0.266715],\
        [0.216032, 0.220455, 0.224708, 0.228800, 0.232741, 0.236541, 0.240210, 0.243758, 0.247193, 0.250527,\
        0.253767, 0.256924, 0.260007, 0.263028, 0.265997, 0.268927, 0.271831, 0.274723, 0.277619, 0.280533,\
        0.283477, 0.286458, 0.289471, 0.292509, 0.295575, 0.298678, 0.301654, 0.304624, 0.307794, 0.311136,\
        0.314593, 0.318124, 0.321713, 0.325352, 0.329035, 0.332754, 0.336499, 0.340261, 0.344036, 0.347822,\
        0.351621, 0.355430, 0.359244, 0.363052, 0.366858, 0.370713, 0.374767, 0.379212, 0.384068, 0.389074,\
        0.393838, 0.397836, 0.400688, 0.403615, 0.406798, 0.409551, 0.411395, 0.412617, 0.413500, 0.414036,\
        0.414146, 0.413911, 0.413463, 0.412760, 0.411622, 0.410213, 0.409026, 0.408170, 0.407271, 0.406145,\
        0.404619, 0.403041, 0.401809, 0.400583, 0.399258, 0.397495, 0.394993, 0.392113, 0.389069, 0.386055,\
        0.383231, 0.380655, 0.378302, 0.376096, 0.373948, 0.371791, 0.369605, 0.367400, 0.365195, 0.363003,\
        0.360828, 0.358663, 0.356500, 0.354330, 0.352145, 0.349940, 0.347710, 0.345451, 0.343157, 0.340822,\
        0.338437, 0.335993, 0.333479, 0.330880, 0.328183, 0.325373, 0.322441, 0.319394, 0.316252, 0.313046,\
        0.309785, 0.306422, 0.302902, 0.299119, 0.294811, 0.290281, 0.285515, 0.280391, 0.274850, 0.268912,\
        0.262618],\
        [0.215086, 0.219486, 0.223711, 0.227772, 0.231680, 0.235445, 0.239079, 0.242593, 0.245996, 0.249297,\
        0.252507, 0.255634, 0.258687, 0.261676, 0.264612, 0.267504, 0.270364, 0.273206, 0.276039, 0.278876,\
        0.281724, 0.284587, 0.287463, 0.290347, 0.293232, 0.296083, 0.298831, 0.301636, 0.304690, 0.307903,\
        0.311194, 0.314525, 0.317895, 0.321310, 0.324770, 0.328267, 0.331789, 0.335324, 0.338868, 0.342418,\
        0.345973, 0.349535, 0.353105, 0.356690, 0.360312, 0.364028, 0.367929, 0.372089, 0.376474, 0.380893,\
        0.385036, 0.388534, 0.391381, 0.394186, 0.397225, 0.399892, 0.401462, 0.402506, 0.403445, 0.404107,\
        0.404265, 0.404074, 0.403794, 0.403405, 0.402336, 0.400846, 0.399707, 0.399143, 0.398440, 0.397540,\
        0.396193, 0.394812, 0.393737, 0.392710, 0.391544, 0.389998, 0.387924, 0.385474, 0.382847, 0.380206,\
        0.377672, 0.375304, 0.373097, 0.371008, 0.368982, 0.366970, 0.364947, 0.362910, 0.360862, 0.358811,\
        0.356756, 0.354695, 0.352622, 0.350529, 0.348411, 0.346264, 0.344083, 0.341864, 0.339604, 0.337296,\
        0.334935, 0.332509, 0.330009, 0.327419, 0.324721, 0.321894, 0.318921, 0.315798, 0.312558, 0.309262,\
        0.305963, 0.302588, 0.298977, 0.295134, 0.290871, 0.286271, 0.281377, 0.276153, 0.270561, 0.264600,\
        0.258289],\
        [0.214090, 0.218469, 0.222668, 0.226699, 0.230575, 0.234308, 0.237911, 0.241395, 0.244770, 0.248045,\
        0.251230, 0.254332, 0.257359, 0.260321, 0.263226, 0.266083, 0.268902, 0.271693, 0.274465, 0.277226,\
        0.279981, 0.282733, 0.285480, 0.288214, 0.290923, 0.293584, 0.296200, 0.298892, 0.301822, 0.304899,\
        0.308017, 0.311136, 0.314286, 0.317490, 0.320748, 0.324047, 0.327371, 0.330705, 0.334041, 0.337379,\
        0.340719, 0.344066, 0.347427, 0.350816, 0.354260, 0.357800, 0.361481, 0.365323, 0.369268, 0.373171,\
        0.376821, 0.380046, 0.382892, 0.385619, 0.388313, 0.390597, 0.392145, 0.393218, 0.394142, 0.394823,\
        0.395053, 0.394966, 0.394803, 0.394518, 0.393662, 0.392443, 0.391467, 0.390855, 0.390241, 0.389448,\
        0.388415, 0.387320, 0.386337, 0.385385, 0.384300, 0.382922, 0.381171, 0.379109, 0.376866, 0.374568,\
        0.372313, 0.370155, 0.368103, 0.366139, 0.364229, 0.362339, 0.360447, 0.358544, 0.356626, 0.354692,\
        0.352742, 0.350772, 0.348776, 0.346750, 0.344688, 0.342586, 0.340441, 0.338252, 0.336016, 0.333728,\
        0.331383, 0.328971, 0.326482, 0.323902, 0.321210, 0.318379, 0.315372, 0.312166, 0.308789, 0.305337,\
        0.301941, 0.298557, 0.294888, 0.290936, 0.286643, 0.282003, 0.277035, 0.271733, 0.266085, 0.260088,\
        0.253750],\
        [0.213039, 0.217399, 0.221573, 0.225575, 0.229421, 0.233125, 0.236699, 0.240157, 0.243509, 0.246763,\
        0.249928, 0.253010, 0.256017, 0.258957, 0.261835, 0.264661, 0.267442, 0.270185, 0.272899, 0.275589,\
        0.278259, 0.280911, 0.283543, 0.286151, 0.288728, 0.291275, 0.293824, 0.296453, 0.299229, 0.302169,\
        0.305071, 0.307943, 0.310873, 0.313883, 0.316965, 0.320094, 0.323243, 0.326395, 0.329543, 0.332688,\
        0.335834, 0.338987, 0.342158, 0.345361, 0.348619, 0.351956, 0.355391, 0.358916, 0.362474, 0.365959,\
        0.369246, 0.372250, 0.374993, 0.377560, 0.379943, 0.381967, 0.383506, 0.384654, 0.385555, 0.386201,\
        0.386525, 0.386587, 0.386493, 0.386198, 0.385566, 0.384710, 0.383920, 0.383295, 0.382702, 0.382014,\
        0.381197, 0.380319, 0.379450, 0.378557, 0.377544, 0.376309, 0.374802, 0.373051, 0.371132, 0.369135,\
        0.367136, 0.365180, 0.363286, 0.361449, 0.359650, 0.357869, 0.356087, 0.354293, 0.352481, 0.350646,\
        0.348784, 0.346891, 0.344961, 0.342989, 0.340971, 0.338902, 0.336783, 0.334612, 0.332389, 0.330113,\
        0.327777, 0.325373, 0.322890, 0.320319, 0.317644, 0.314832, 0.311825, 0.308559, 0.305045, 0.301405,\
        0.297874, 0.294372, 0.290640, 0.286602, 0.282238, 0.277537, 0.272503, 0.267136, 0.261431, 0.255390,\
        0.249018],\
        [0.211929, 0.216269, 0.220418, 0.224392, 0.228209, 0.231885, 0.235435, 0.238872, 0.242205, 0.245444,\
        0.248595, 0.251664, 0.254657, 0.257579, 0.260437, 0.263235, 0.265982, 0.268683, 0.271344, 0.273970,\
        0.276564, 0.279130, 0.281667, 0.284175, 0.286657, 0.289127, 0.291618, 0.294183, 0.296868, 0.299652,\
        0.302267, 0.304898, 0.307632, 0.310480, 0.313420, 0.316407, 0.319401, 0.322385, 0.325357, 0.328322,\
        0.331287, 0.334259, 0.337250, 0.340270, 0.343335, 0.346459, 0.349644, 0.352872, 0.356097, 0.359245,\
        0.362240, 0.365029, 0.367610, 0.369995, 0.372156, 0.374012, 0.375511, 0.376684, 0.377589, 0.378242,\
        0.378636, 0.378801, 0.378775, 0.378547, 0.378097, 0.377497, 0.376882, 0.376318, 0.375768, 0.375169,\
        0.374495, 0.373765, 0.373000, 0.372177, 0.371241, 0.370131, 0.368816, 0.367307, 0.365652, 0.363909,\
        0.362133, 0.360364, 0.358621, 0.356908, 0.355216, 0.353533, 0.351845, 0.350143, 0.348418, 0.346665,\
        0.344877, 0.343049, 0.341173, 0.339245, 0.337258, 0.335212, 0.333104, 0.330939, 0.328719, 0.326445,\
        0.324112, 0.321710, 0.319225, 0.316654, 0.313993, 0.311226, 0.308281, 0.305010, 0.301407, 0.297661,\
        0.293953, 0.290228, 0.286330, 0.282168, 0.277702, 0.272917, 0.267808, 0.262372, 0.256608, 0.250517,\
        0.244104],\
        [0.210753, 0.215073, 0.219196, 0.223142, 0.226930, 0.230580, 0.234109, 0.237529, 0.240851, 0.244082,\
        0.247227, 0.250290, 0.253274, 0.256185, 0.259027, 0.261804, 0.264521, 0.267185, 0.269800, 0.272371,\
        0.274901, 0.277394, 0.279853, 0.282281, 0.284688, 0.287090, 0.289513, 0.291987, 0.294523, 0.297059,\
        0.299504, 0.301978, 0.304559, 0.307286, 0.310124, 0.312993, 0.315843, 0.318662, 0.321462, 0.324254,\
        0.327047, 0.329847, 0.332662, 0.335501, 0.338372, 0.341281, 0.344225, 0.347184, 0.350121, 0.352986,\
        0.355728, 0.358310, 0.360715, 0.362928, 0.364923, 0.366658, 0.368107, 0.369278, 0.370195, 0.370875,\
        0.371327, 0.371569, 0.371621, 0.371491, 0.371191, 0.370772, 0.370305, 0.369830, 0.369346, 0.368826,\
        0.368254, 0.367629, 0.366950, 0.366200, 0.365343, 0.364344, 0.363183, 0.361867, 0.360421, 0.358887,\
        0.357303, 0.355699, 0.354094, 0.352496, 0.350903, 0.349308, 0.347701, 0.346076, 0.344424, 0.342738,\
        0.341011, 0.339237, 0.337405, 0.335511, 0.333547, 0.331510, 0.329403, 0.327229, 0.324999, 0.322719,\
        0.320386, 0.317982, 0.315484, 0.312888, 0.310215, 0.307476, 0.304651, 0.301502, 0.297773, 0.293925,\
        0.290026, 0.286069, 0.281969, 0.277646, 0.273052, 0.268163, 0.262965, 0.257453, 0.251623, 0.245476,\
        0.239015],\
        [0.209505, 0.213805, 0.217900, 0.221814, 0.225573, 0.229199, 0.232711, 0.236121, 0.239440, 0.242670,\
        0.245817, 0.248881, 0.251865, 0.254771, 0.257603, 0.260364, 0.263058, 0.265691, 0.268267, 0.270791,\
        0.273266, 0.275699, 0.278093, 0.280456, 0.282798, 0.285132, 0.287476, 0.289843, 0.292225, 0.294585,\
        0.296906, 0.299243, 0.301687, 0.304328, 0.307107, 0.309868, 0.312561, 0.315205, 0.317829, 0.320452,\
        0.323081, 0.325717, 0.328361, 0.331021, 0.333700, 0.336400, 0.339116, 0.341831, 0.344517, 0.347137,\
        0.349655, 0.352043, 0.354276, 0.356337, 0.358200, 0.359840, 0.361240, 0.362400, 0.363329, 0.364040,\
        0.364544, 0.364855, 0.364987, 0.364955, 0.364781, 0.364500, 0.364154, 0.363772, 0.363362, 0.362915,\
        0.362421, 0.361875, 0.361269, 0.360586, 0.359805, 0.358904, 0.357870, 0.356705, 0.355427, 0.354061,\
        0.352636, 0.351174, 0.349691, 0.348196, 0.346691, 0.345172, 0.343636, 0.342074, 0.340482, 0.338852,\
        0.337176, 0.335446, 0.333651, 0.331782, 0.329832, 0.327796, 0.325675, 0.323478, 0.321222, 0.318924,\
        0.316591, 0.314198, 0.311684, 0.309028, 0.306293, 0.303502, 0.300669, 0.297623, 0.293931, 0.289983,\
        0.285936, 0.281792, 0.277504, 0.273016, 0.268283, 0.263277, 0.257980, 0.252383, 0.246481, 0.240273,\
        0.233758],\
        [0.208179, 0.212456, 0.216519, 0.220398, 0.224126, 0.227730, 0.231231, 0.234640, 0.237963, 0.241204,\
        0.244361, 0.247436, 0.250427, 0.253335, 0.256163, 0.258914, 0.261591, 0.264199, 0.266742, 0.269226,\
        0.271656, 0.274038, 0.276378, 0.278684, 0.280965, 0.283234, 0.285501, 0.287771, 0.290037, 0.292286,\
        0.294520, 0.296771, 0.299093, 0.301700, 0.304418, 0.307035, 0.309531, 0.311974, 0.314419, 0.316882,\
        0.319357, 0.321836, 0.324316, 0.326800, 0.329291, 0.331792, 0.334296, 0.336790, 0.339251, 0.341655,\
        0.343974, 0.346183, 0.348260, 0.350187, 0.351940, 0.353502, 0.354859, 0.356006, 0.356947, 0.357689,\
        0.358241, 0.358614, 0.358822, 0.358881, 0.358811, 0.358639, 0.358394, 0.358097, 0.357758, 0.357376,\
        0.356947, 0.356464, 0.355919, 0.355297, 0.354585, 0.353768, 0.352837, 0.351793, 0.350648, 0.349418,\
        0.348122, 0.346778, 0.345398, 0.343992, 0.342562, 0.341109, 0.339629, 0.338120, 0.336576, 0.334991,\
        0.333356, 0.331663, 0.329900, 0.328052, 0.326110, 0.324067, 0.321922, 0.319685, 0.317379, 0.315044,\
        0.312712, 0.310366, 0.307879, 0.305111, 0.302283, 0.299416, 0.296450, 0.293275, 0.289670, 0.285734,\
        0.281606, 0.277328, 0.272886, 0.268246, 0.263377, 0.258252, 0.252851, 0.247165, 0.241185, 0.234909,\
        0.228336],\
        [0.206769, 0.211019, 0.215042, 0.218880, 0.222574, 0.226160, 0.229658, 0.233076, 0.236417, 0.239679,\
        0.242858, 0.245952, 0.248958, 0.251875, 0.254706, 0.257451, 0.260116, 0.262704, 0.265220, 0.267671,\
        0.270063, 0.272401, 0.274694, 0.276949, 0.279177, 0.281386, 0.283585, 0.285778, 0.287965, 0.290148,\
        0.292339, 0.294574, 0.296920, 0.299455, 0.302041, 0.304449, 0.306679, 0.308903, 0.311183, 0.313506,\
        0.315845, 0.318178, 0.320499, 0.322811, 0.325120, 0.327431, 0.329738, 0.332031, 0.334292, 0.336502,\
        0.338641, 0.340689, 0.342628, 0.344437, 0.346099, 0.347596, 0.348917, 0.350053, 0.351005, 0.351776,\
        0.352372, 0.352802, 0.353078, 0.353215, 0.353232, 0.353149, 0.352987, 0.352764, 0.352488, 0.352164,\
        0.351788, 0.351357, 0.350863, 0.350295, 0.349643, 0.348896, 0.348049, 0.347102, 0.346061, 0.344938,\
        0.343745, 0.342496, 0.341201, 0.339867, 0.338499, 0.337098, 0.335664, 0.334194, 0.332687, 0.331136,\
        0.329536, 0.327875, 0.326138, 0.324310, 0.322374, 0.320321, 0.318146, 0.315855, 0.313472, 0.311060,\
        0.308703, 0.306440, 0.304154, 0.301269, 0.298262, 0.295293, 0.292166, 0.288820, 0.285165, 0.281213,\
        0.277028, 0.272649, 0.268083, 0.263313, 0.258317, 0.253076, 0.247572, 0.241793, 0.235733, 0.229385,\
        0.222750],\
        [0.205268, 0.209485, 0.213456, 0.217240, 0.220900, 0.224475, 0.227982, 0.231425, 0.234798, 0.238095,\
        0.241307, 0.244430, 0.247458, 0.250391, 0.253228, 0.255973, 0.258630, 0.261202, 0.263696, 0.266119,\
        0.268476, 0.270777, 0.273028, 0.275239, 0.277418, 0.279575, 0.281717, 0.283851, 0.285983, 0.288123,\
        0.290293, 0.292526, 0.294868, 0.297332, 0.299807, 0.301955, 0.303874, 0.305920, 0.308079, 0.310297,\
        0.312523, 0.314720, 0.316884, 0.319025, 0.321158, 0.323290, 0.325416, 0.327526, 0.329606, 0.331641,\
        0.333619, 0.335523, 0.337339, 0.339048, 0.340632, 0.342077, 0.343368, 0.344497, 0.345460, 0.346257,\
        0.346893, 0.347373, 0.347709, 0.347913, 0.348000, 0.347989, 0.347895, 0.347732, 0.347511, 0.347234,\
        0.346904, 0.346516, 0.346064, 0.345541, 0.344939, 0.344251, 0.343471, 0.342600, 0.341641, 0.340601,\
        0.339489, 0.338314, 0.337084, 0.335807, 0.334486, 0.333124, 0.331721, 0.330279, 0.328796, 0.327270,\
        0.325695, 0.324061, 0.322351, 0.320543, 0.318614, 0.316551, 0.314348, 0.312005, 0.309525, 0.306977,\
        0.304530, 0.302267, 0.300166, 0.297627, 0.294327, 0.291093, 0.287766, 0.284252, 0.280493, 0.276476,\
        0.272224, 0.267757, 0.263085, 0.258201, 0.253091, 0.247740, 0.242135, 0.236265, 0.230122, 0.223701,\
        0.216999],\
        [0.203672, 0.207847, 0.211740, 0.215454, 0.219081, 0.222660, 0.226197, 0.229684, 0.233108, 0.236454,\
        0.239711, 0.242871, 0.245928, 0.248881, 0.251729, 0.254477, 0.257127, 0.259687, 0.262162, 0.264559,\
        0.266887, 0.269154, 0.271367, 0.273538, 0.275673, 0.277783, 0.279876, 0.281962, 0.284049, 0.286151,\
        0.288289, 0.290485, 0.292758, 0.295087, 0.297351, 0.299252, 0.301022, 0.303005, 0.305095, 0.307246,\
        0.309377, 0.311441, 0.313442, 0.315410, 0.317372, 0.319338, 0.321300, 0.323245, 0.325161, 0.327038,\
        0.328869, 0.330645, 0.332353, 0.333977, 0.335499, 0.336901, 0.338171, 0.339296, 0.340270, 0.341092,\
        0.341762, 0.342286, 0.342672, 0.342930, 0.343075, 0.343120, 0.343079, 0.342965, 0.342787, 0.342550,\
        0.342256, 0.341902, 0.341486, 0.341000, 0.340439, 0.339797, 0.339071, 0.338259, 0.337362, 0.336384,\
        0.335332, 0.334212, 0.333032, 0.331795, 0.330507, 0.329169, 0.327784, 0.326354, 0.324882, 0.323368,\
        0.321810, 0.320198, 0.318515, 0.316731, 0.314812, 0.312738, 0.310515, 0.308147, 0.305600, 0.302858,\
        0.300302, 0.297953, 0.295717, 0.293135, 0.290017, 0.286671, 0.283186, 0.279526, 0.275652, 0.271548,\
        0.267214, 0.262661, 0.257892, 0.252904, 0.247689, 0.242236, 0.236534, 0.230574, 0.224348, 0.217853,\
        0.211084],\
        [0.201989, 0.206105, 0.209853, 0.213478, 0.217092, 0.220706, 0.224303, 0.227860, 0.231353, 0.234763,\
        0.238076, 0.241280, 0.244371, 0.247346, 0.250207, 0.252958, 0.255604, 0.258152, 0.260609, 0.262983,\
        0.265283, 0.267518, 0.269697, 0.271829, 0.273924, 0.275992, 0.278041, 0.280081, 0.282123, 0.284180,\
        0.286266, 0.288389, 0.290546, 0.292692, 0.294725, 0.296554, 0.298325, 0.300225, 0.302241, 0.304357,\
        0.306400, 0.308315, 0.310136, 0.311925, 0.313727, 0.315548, 0.317367, 0.319164, 0.320928, 0.322658,\
        0.324356, 0.326017, 0.327633, 0.329187, 0.330659, 0.332030, 0.333285, 0.334410, 0.335396, 0.336240,\
        0.336940, 0.337500, 0.337926, 0.338228, 0.338417, 0.338505, 0.338504, 0.338427, 0.338281, 0.338074,\
        0.337808, 0.337482, 0.337093, 0.336636, 0.336107, 0.335501, 0.334816, 0.334048, 0.333197, 0.332264,\
        0.331254, 0.330172, 0.329024, 0.327814, 0.326545, 0.325218, 0.323836, 0.322404, 0.320928, 0.319411,\
        0.317855, 0.316258, 0.314603, 0.312851, 0.310945, 0.308851, 0.306596, 0.304223, 0.301730, 0.298903,\
        0.296159, 0.293655, 0.291154, 0.288421, 0.285330, 0.281959, 0.278381, 0.274608, 0.270628, 0.266429,\
        0.262009, 0.257367, 0.252506, 0.247421, 0.242108, 0.236559, 0.230764, 0.224717, 0.218410, 0.211840,\
        0.205002],\
        [0.200337, 0.204196, 0.207679, 0.211259, 0.214923, 0.218622, 0.222315, 0.225968, 0.229549, 0.233036,\
        0.236411, 0.239664, 0.242789, 0.245787, 0.248660, 0.251413, 0.254053, 0.256589, 0.259028, 0.261379,\
        0.263652, 0.265856, 0.268002, 0.270098, 0.272154, 0.274181, 0.276186, 0.278181, 0.280175, 0.282178,\
        0.284195, 0.286229, 0.288263, 0.290261, 0.292169, 0.293974, 0.295751, 0.297591, 0.299558, 0.301632,\
        0.303556, 0.305294, 0.306914, 0.308523, 0.310186, 0.311893, 0.313594, 0.315258, 0.316879, 0.318469,\
        0.320042, 0.321603, 0.323142, 0.324640, 0.326076, 0.327426, 0.328673, 0.329801, 0.330800, 0.331663,\
        0.332388, 0.332977, 0.333434, 0.333768, 0.333989, 0.334107, 0.334134, 0.334081, 0.333958, 0.333772,\
        0.333526, 0.333220, 0.332851, 0.332416, 0.331910, 0.331331, 0.330675, 0.329939, 0.329121, 0.328219,\
        0.327234, 0.326174, 0.325043, 0.323847, 0.322585, 0.321256, 0.319862, 0.318412, 0.316915, 0.315378,\
        0.313807, 0.312206, 0.310575, 0.308872, 0.306990, 0.304853, 0.302531, 0.300108, 0.297587, 0.294825,\
        0.291996, 0.289262, 0.286513, 0.283600, 0.280431, 0.277004, 0.273351, 0.269487, 0.265412, 0.261121,\
        0.256611, 0.251881, 0.246929, 0.241753, 0.236347, 0.230706, 0.224822, 0.218690, 0.212304, 0.205659,\
        0.198751],\
        [0.198128, 0.201585, 0.205084, 0.208809, 0.212611, 0.216444, 0.220266, 0.224036, 0.227720, 0.231289,\
        0.234729, 0.238028, 0.241185, 0.244202, 0.247083, 0.249835, 0.252467, 0.254988, 0.257407, 0.259734,\
        0.261980, 0.264154, 0.266266, 0.268326, 0.270345, 0.272330, 0.274293, 0.276241, 0.278182, 0.280124,\
        0.282069, 0.284013, 0.285942, 0.287834, 0.289667, 0.291447, 0.293221, 0.295049, 0.296971, 0.298932,\
        0.300728, 0.302296, 0.303708, 0.305149, 0.306719, 0.308360, 0.309971, 0.311508, 0.312986, 0.314438,\
        0.315893, 0.317366, 0.318845, 0.320306, 0.321718, 0.323055, 0.324299, 0.325433, 0.326444, 0.327323,\
        0.328068, 0.328678, 0.329157, 0.329514, 0.329755, 0.329891, 0.329934, 0.329894, 0.329783, 0.329609,\
        0.329376, 0.329084, 0.328729, 0.328308, 0.327817, 0.327255, 0.326619, 0.325906, 0.325109, 0.324224,\
        0.323250, 0.322195, 0.321068, 0.319874, 0.318610, 0.317269, 0.315850, 0.314364, 0.312830, 0.311259,\
        0.309648, 0.308011, 0.306380, 0.304741, 0.302916, 0.300712, 0.298308, 0.295813, 0.293202, 0.290437,\
        0.287575, 0.284686, 0.281740, 0.278655, 0.275367, 0.271854, 0.268118, 0.264167, 0.260002, 0.255621,\
        0.251023, 0.246205, 0.241165, 0.235900, 0.230405, 0.224676, 0.218706, 0.212492, 0.206027, 0.199307,\
        0.192331],\
        [0.194990, 0.198600, 0.202392, 0.206289, 0.210250, 0.214238, 0.218206, 0.222103, 0.225892, 0.229543,\
        0.233041, 0.236380, 0.239561, 0.242588, 0.245471, 0.248216, 0.250835, 0.253337, 0.255734, 0.258035,\
        0.260252, 0.262395, 0.264473, 0.266498, 0.268478, 0.270423, 0.272341, 0.274240, 0.276128, 0.278007,\
        0.279881, 0.281743, 0.283586, 0.285398, 0.287172, 0.288920, 0.290669, 0.292451, 0.294274, 0.296078,\
        0.297751, 0.299206, 0.300470, 0.301783, 0.303331, 0.304960, 0.306488, 0.307891, 0.309219, 0.310530,\
        0.311873, 0.313274, 0.314717, 0.316158, 0.317556, 0.318887, 0.320132, 0.321273, 0.322294, 0.323186,\
        0.323943, 0.324566, 0.325059, 0.325428, 0.325679, 0.325823, 0.325870, 0.325831, 0.325721, 0.325550,\
        0.325324, 0.325041, 0.324697, 0.324283, 0.323797, 0.323241, 0.322617, 0.321920, 0.321138, 0.320259,\
        0.319280, 0.318212, 0.317073, 0.315874, 0.314607, 0.313249, 0.311788, 0.310247, 0.308662, 0.307049,\
        0.305388, 0.303672, 0.301980, 0.300351, 0.298581, 0.296346, 0.293891, 0.291318, 0.288628, 0.285813,\
        0.282891, 0.279887, 0.276789, 0.273554, 0.270140, 0.266524, 0.262696, 0.258655, 0.254401, 0.249932,\
        0.245246, 0.240341, 0.235214, 0.229862, 0.224281, 0.218467, 0.212414, 0.206119, 0.199577, 0.192785,\
        0.185739],\
        [0.192081, 0.195999, 0.199873, 0.203841, 0.207940, 0.212079, 0.216189, 0.220211, 0.224096, 0.227815,\
        0.231357, 0.234721, 0.237912, 0.240940, 0.243814, 0.246545, 0.249144, 0.251624, 0.253994, 0.256267,\
        0.258453, 0.260563, 0.262607, 0.264595, 0.266537, 0.268440, 0.270314, 0.272164, 0.273998, 0.275817,\
        0.277623, 0.279413, 0.281183, 0.282927, 0.284646, 0.286347, 0.288047, 0.289755, 0.291461, 0.293118,\
        0.294657, 0.296028, 0.297269, 0.298550, 0.300089, 0.301693, 0.303113, 0.304363, 0.305542, 0.306714,\
        0.307949, 0.309305, 0.310742, 0.312179, 0.313567, 0.314891, 0.316138, 0.317286, 0.318315, 0.319213,\
        0.319976, 0.320605, 0.321102, 0.321474, 0.321727, 0.321869, 0.321908, 0.321859, 0.321739, 0.321562,\
        0.321337, 0.321063, 0.320726, 0.320313, 0.319820, 0.319258, 0.318637, 0.317954, 0.317187, 0.316306,\
        0.315305, 0.314202, 0.313031, 0.311819, 0.310557, 0.309187, 0.307670, 0.306049, 0.304397, 0.302747,\
        0.301044, 0.299253, 0.297473, 0.295745, 0.293860, 0.291660, 0.289207, 0.286591, 0.283839, 0.280962,\
        0.277968, 0.274863, 0.271640, 0.268275, 0.264741, 0.261017, 0.257091, 0.252958, 0.248614, 0.244057,\
        0.239284, 0.234292, 0.229079, 0.223642, 0.217976, 0.212079, 0.205945, 0.199571, 0.192954, 0.186089,\
        0.178974],\
        [0.189375, 0.193501, 0.197487, 0.201545, 0.205772, 0.210027, 0.214268, 0.218400, 0.222357, 0.226118,\
        0.229679, 0.233047, 0.236232, 0.239245, 0.242100, 0.244807, 0.247380, 0.249831, 0.252171, 0.254413,\
        0.256566, 0.258642, 0.260651, 0.262603, 0.264505, 0.266367, 0.268196, 0.269999, 0.271779, 0.273541,\
        0.275285, 0.277011, 0.278715, 0.280397, 0.282058, 0.283704, 0.285339, 0.286963, 0.288561, 0.290102,\
        0.291546, 0.292879, 0.294149, 0.295468, 0.296926, 0.298424, 0.299725, 0.300858, 0.301923, 0.302976,\
        0.304099, 0.305452, 0.306921, 0.308363, 0.309727, 0.311033, 0.312285, 0.313442, 0.314472, 0.315368,\
        0.316129, 0.316754, 0.317246, 0.317615, 0.317863, 0.317995, 0.318017, 0.317946, 0.317801, 0.317608,\
        0.317382, 0.317118, 0.316793, 0.316378, 0.315863, 0.315272, 0.314644, 0.313979, 0.313234, 0.312350,\
        0.311309, 0.310141, 0.308909, 0.307668, 0.306434, 0.305079, 0.303488, 0.301771, 0.300030, 0.298333,\
        0.296579, 0.294747, 0.292888, 0.291004, 0.288986, 0.286744, 0.284278, 0.281628, 0.278826, 0.275886,\
        0.272816, 0.269619, 0.266288, 0.262807, 0.259160, 0.255330, 0.251304, 0.247077, 0.242642, 0.237996,\
        0.233136, 0.228058, 0.222761, 0.217239, 0.211491, 0.205512, 0.199298, 0.192847, 0.186155, 0.179218,\
        0.172034],\
        [0.186819, 0.191167, 0.195357, 0.199581, 0.203755, 0.208118, 0.212494, 0.216703, 0.220689, 0.224450,\
        0.227997, 0.231343, 0.234502, 0.237486, 0.240309, 0.242984, 0.245524, 0.247941, 0.250248, 0.252455,\
        0.254574, 0.256615, 0.258589, 0.260503, 0.262367, 0.264188, 0.265974, 0.267730, 0.269460, 0.271168,\
        0.272854, 0.274520, 0.276165, 0.277788, 0.279391, 0.280975, 0.282542, 0.284086, 0.285595, 0.287048,\
        0.288429, 0.289741, 0.291021, 0.292328, 0.293687, 0.295031, 0.296251, 0.297335, 0.298351, 0.299325,\
        0.300342, 0.301770, 0.303270, 0.304706, 0.305999, 0.307268, 0.308548, 0.309711, 0.310723, 0.311608,\
        0.312361, 0.312971, 0.313449, 0.313809, 0.314052, 0.314171, 0.314167, 0.314060, 0.313876, 0.313652,\
        0.313421, 0.313179, 0.312880, 0.312465, 0.311905, 0.311249, 0.310595, 0.309961, 0.309258, 0.308379,\
        0.307278, 0.306014, 0.304693, 0.303378, 0.302173, 0.300899, 0.299205, 0.297420, 0.295612, 0.293816,\
        0.291983, 0.290089, 0.288140, 0.286122, 0.283979, 0.281658, 0.279144, 0.276450, 0.273595, 0.270592,\
        0.267446, 0.264161, 0.260731, 0.257145, 0.253392, 0.249458, 0.245335, 0.241013, 0.236486, 0.231751,\
        0.226804, 0.221641, 0.216259, 0.210655, 0.204824, 0.198765, 0.192473, 0.185946, 0.179179, 0.172171,\
        0.164919],\
        [0.184373, 0.188974, 0.193355, 0.197548, 0.201823, 0.206452, 0.210930, 0.215134, 0.219077, 0.222786,\
        0.226283, 0.229582, 0.232695, 0.235636, 0.238418, 0.241053, 0.243554, 0.245933, 0.248203, 0.250375,\
        0.252458, 0.254464, 0.256402, 0.258279, 0.260106, 0.261887, 0.263631, 0.265342, 0.267025, 0.268683,\
        0.270317, 0.271928, 0.273517, 0.275084, 0.276629, 0.278153, 0.279654, 0.281127, 0.282562, 0.283949,\
        0.285283, 0.286569, 0.287833, 0.289098, 0.290369, 0.291605, 0.292758, 0.293818, 0.294819, 0.295807,\
        0.296910, 0.298282, 0.299745, 0.301195, 0.302307, 0.303550, 0.304925, 0.306059, 0.307015, 0.307886,\
        0.308630, 0.309213, 0.309662, 0.310012, 0.310259, 0.310365, 0.310325, 0.310175, 0.309937, 0.309657,\
        0.309416, 0.309214, 0.308966, 0.308571, 0.307948, 0.307155, 0.306440, 0.305864, 0.305228, 0.304369,\
        0.303191, 0.301805, 0.300402, 0.299012, 0.297726, 0.296408, 0.294739, 0.292923, 0.291059, 0.289172,\
        0.287248, 0.285268, 0.283219, 0.281083, 0.278825, 0.276412, 0.273829, 0.271075, 0.268158, 0.265087,\
        0.261865, 0.258495, 0.254971, 0.251287, 0.247434, 0.243401, 0.239180, 0.234764, 0.230147, 0.225323,\
        0.220289, 0.215041, 0.209575, 0.203888, 0.197977, 0.191839, 0.185469, 0.178866, 0.172026, 0.164947,\
        0.157627],\
        [0.181970, 0.186881, 0.191564, 0.196060, 0.200585, 0.205226, 0.209586, 0.213641, 0.217456, 0.221067,\
        0.224487, 0.227721, 0.230777, 0.233665, 0.236398, 0.238987, 0.241445, 0.243785, 0.246016, 0.248151,\
        0.250200, 0.252171, 0.254073, 0.255916, 0.257706, 0.259450, 0.261154, 0.262824, 0.264463, 0.266074,\
        0.267659, 0.269220, 0.270757, 0.272271, 0.273762, 0.275229, 0.276670, 0.278081, 0.279455, 0.280789,\
        0.282080, 0.283336, 0.284568, 0.285787, 0.286989, 0.288156, 0.289265, 0.290316, 0.291337, 0.292381,\
        0.293532, 0.294840, 0.296237, 0.297584, 0.298630, 0.299959, 0.301432, 0.302401, 0.303262, 0.304152,\
        0.304901, 0.305428, 0.305827, 0.306172, 0.306448, 0.306549, 0.306453, 0.306269, 0.305978, 0.305588,\
        0.305328, 0.305188, 0.305013, 0.304681, 0.304021, 0.302983, 0.302149, 0.301665, 0.301063, 0.300224,\
        0.298978, 0.297516, 0.296047, 0.294600, 0.293185, 0.291709, 0.290043, 0.288222, 0.286316, 0.284358,\
        0.282347, 0.280274, 0.278123, 0.275877, 0.273511, 0.271003, 0.268338, 0.265511, 0.262524, 0.259380,\
        0.256080, 0.252625, 0.249011, 0.245233, 0.241284, 0.237156, 0.232840, 0.228332, 0.223624, 0.218712,\
        0.213591, 0.208258, 0.202709, 0.196940, 0.190949, 0.184732, 0.178286, 0.171608, 0.164695, 0.157546,\
        0.150157],\
        [0.179499, 0.184766, 0.189865, 0.194854, 0.199740, 0.204199, 0.208248, 0.212049, 0.215701, 0.219207,\
        0.222547, 0.225712, 0.228705, 0.231536, 0.234216, 0.236757, 0.239172, 0.241471, 0.243665, 0.245764,\
        0.247778, 0.249716, 0.251586, 0.253395, 0.255152, 0.256861, 0.258529, 0.260160, 0.261759, 0.263327,\
        0.264868, 0.266383, 0.267873, 0.269339, 0.270779, 0.272194, 0.273581, 0.274938, 0.276262, 0.277549,\
        0.278802, 0.280023, 0.281220, 0.282396, 0.283549, 0.284671, 0.285755, 0.286807, 0.287852, 0.288930,\
        0.290088, 0.291340, 0.292642, 0.293901, 0.295095, 0.296419, 0.297729, 0.298612, 0.299435, 0.300371,\
        0.301115, 0.301552, 0.301893, 0.302233, 0.302581, 0.302696, 0.302482, 0.302300, 0.302040, 0.301425,\
        0.301164, 0.301082, 0.300939, 0.300666, 0.300083, 0.298879, 0.297900, 0.297330, 0.296650, 0.295732,\
        0.294499, 0.293056, 0.291548, 0.290029, 0.288496, 0.286896, 0.285173, 0.283322, 0.281374, 0.279352,\
        0.277261, 0.275096, 0.272847, 0.270496, 0.268027, 0.265423, 0.262671, 0.259763, 0.256699, 0.253476,\
        0.250096, 0.246556, 0.242854, 0.238985, 0.234943, 0.230721, 0.226313, 0.221714, 0.216916, 0.211916,\
        0.206710, 0.201292, 0.195660, 0.189810, 0.183740, 0.177444, 0.170922, 0.164170, 0.157185, 0.149966,\
        0.142509],\
        [0.176812, 0.182404, 0.187861, 0.193204, 0.198325, 0.202754, 0.206496, 0.210102, 0.213666, 0.217115,\
        0.220397, 0.223501, 0.226434, 0.229209, 0.231838, 0.234333, 0.236706, 0.238967, 0.241126, 0.243192,\
        0.245175, 0.247083, 0.248923, 0.250703, 0.252428, 0.254106, 0.255741, 0.257338, 0.258900, 0.260431,\
        0.261932, 0.263406, 0.264853, 0.266274, 0.267669, 0.269037, 0.270377, 0.271688, 0.272967, 0.274214,\
        0.275431, 0.276618, 0.277781, 0.278922, 0.280039, 0.281132, 0.282200, 0.283252, 0.284310, 0.285399,\
        0.286542, 0.287744, 0.288976, 0.290199, 0.291414, 0.292642, 0.293783, 0.294722, 0.295576, 0.296428,\
        0.297106, 0.297536, 0.297867, 0.298199, 0.298560, 0.298678, 0.298404, 0.298208, 0.297968, 0.297358,\
        0.297016, 0.296861, 0.296666, 0.296331, 0.295691, 0.294678, 0.293686, 0.292873, 0.292035, 0.291021,\
        0.289780, 0.288357, 0.286833, 0.285255, 0.283630, 0.281934, 0.280138, 0.278236, 0.276235, 0.274148,\
        0.271980, 0.269728, 0.267384, 0.264934, 0.262366, 0.259666, 0.256824, 0.253831, 0.250684, 0.247380,\
        0.243916, 0.240292, 0.236502, 0.232543, 0.228410, 0.224097, 0.219599, 0.214909, 0.210023, 0.204936,\
        0.199644, 0.194143, 0.188429, 0.182498, 0.176348, 0.169976, 0.163378, 0.156552, 0.149495, 0.142205,\
        0.134681],\
        [0.173772, 0.179600, 0.185268, 0.190746, 0.195888, 0.200296, 0.203937, 0.207621, 0.211269, 0.214737,\
        0.217988, 0.221041, 0.223920, 0.226645, 0.229229, 0.231683, 0.234020, 0.236248, 0.238377, 0.240415,\
        0.242371, 0.244253, 0.246067, 0.247821, 0.249521, 0.251171, 0.252778, 0.254344, 0.255875, 0.257372,\
        0.258838, 0.260275, 0.261684, 0.263066, 0.264421, 0.265748, 0.267047, 0.268317, 0.269558, 0.270769,\
        0.271952, 0.273109, 0.274242, 0.275353, 0.276443, 0.277513, 0.278568, 0.279617, 0.280675, 0.281758,\
        0.282877, 0.284033, 0.285210, 0.286388, 0.287555, 0.288692, 0.289746, 0.290680, 0.291523, 0.292288,\
        0.292910, 0.293365, 0.293718, 0.294033, 0.294294, 0.294356, 0.294186, 0.293950, 0.293628, 0.293177,\
        0.292790, 0.292499, 0.292187, 0.291741, 0.291065, 0.290165, 0.289194, 0.288246, 0.287260, 0.286148,\
        0.284869, 0.283438, 0.281893, 0.280267, 0.278570, 0.276795, 0.274927, 0.272962, 0.270900, 0.268746,\
        0.266502, 0.264165, 0.261729, 0.259185, 0.256521, 0.253728, 0.250794, 0.247714, 0.244481, 0.241092,\
        0.237544, 0.233833, 0.229956, 0.225908, 0.221686, 0.217283, 0.212695, 0.207917, 0.202944, 0.197771,\
        0.192394, 0.186810, 0.181014, 0.175003, 0.168774, 0.162325, 0.155652, 0.148752, 0.141624, 0.134265,\
        0.126672],\
        [0.170304, 0.176277, 0.182033, 0.187498, 0.192506, 0.196829, 0.200699, 0.204731, 0.208561, 0.212066,\
        0.215281, 0.218285, 0.221120, 0.223805, 0.226355, 0.228779, 0.231088, 0.233290, 0.235396, 0.237412,\
        0.239348, 0.241209, 0.243003, 0.244737, 0.246415, 0.248043, 0.249625, 0.251167, 0.252670, 0.254139,\
        0.255575, 0.256980, 0.258356, 0.259704, 0.261023, 0.262315, 0.263578, 0.264814, 0.266021, 0.267201,\
        0.268354, 0.269483, 0.270589, 0.271674, 0.272741, 0.273793, 0.274835, 0.275875, 0.276922, 0.277988,\
        0.279078, 0.280189, 0.281313, 0.282434, 0.283536, 0.284596, 0.285584, 0.286480, 0.287283, 0.287987,\
        0.288569, 0.289024, 0.289377, 0.289654, 0.289834, 0.289863, 0.289730, 0.289487, 0.289154, 0.288750,\
        0.288342, 0.287950, 0.287519, 0.286975, 0.286264, 0.285392, 0.284421, 0.283396, 0.282305, 0.281108,\
        0.279777, 0.278313, 0.276737, 0.275065, 0.273307, 0.271463, 0.269527, 0.267496, 0.265368, 0.263143,\
        0.260823, 0.258404, 0.255880, 0.253244, 0.250488, 0.247602, 0.244577, 0.241408, 0.238088, 0.234613,\
        0.230979, 0.227181, 0.223217, 0.219082, 0.214771, 0.210280, 0.205604, 0.200738, 0.195678, 0.190420,\
        0.184959, 0.179292, 0.173414, 0.167324, 0.161017, 0.154491, 0.147743, 0.140771, 0.133571, 0.126142,\
        0.118482],\
        [0.166388, 0.172452, 0.178252, 0.183714, 0.188732, 0.193259, 0.197500, 0.201683, 0.205632, 0.209078,\
        0.212218, 0.215181, 0.217991, 0.220656, 0.223186, 0.225592, 0.227885, 0.230072, 0.232163, 0.234166,\
        0.236088, 0.237936, 0.239716, 0.241434, 0.243097, 0.244708, 0.246272, 0.247793, 0.249275, 0.250720,\
        0.252131, 0.253510, 0.254858, 0.256176, 0.257466, 0.258727, 0.259960, 0.261165, 0.262344, 0.263495,\
        0.264622, 0.265726, 0.266808, 0.267871, 0.268918, 0.269952, 0.270978, 0.272003, 0.273033, 0.274074,\
        0.275130, 0.276196, 0.277266, 0.278326, 0.279361, 0.280351, 0.281276, 0.282120, 0.282873, 0.283527,\
        0.284073, 0.284506, 0.284838, 0.285074, 0.285203, 0.285202, 0.285070, 0.284827, 0.284493, 0.284093,\
        0.283656, 0.283189, 0.282663, 0.282038, 0.281281, 0.280394, 0.279400, 0.278320, 0.277154, 0.275884,\
        0.274497, 0.272990, 0.271372, 0.269652, 0.267838, 0.265931, 0.263930, 0.261832, 0.259635, 0.257339,\
        0.254942, 0.252442, 0.249833, 0.247109, 0.244262, 0.241285, 0.238171, 0.234912, 0.231505, 0.227942,\
        0.224221, 0.220337, 0.216285, 0.212063, 0.207664, 0.203085, 0.198322, 0.193370, 0.188225, 0.182882,\
        0.177337, 0.171588, 0.165630, 0.159461, 0.153076, 0.146474, 0.139651, 0.132606, 0.125335, 0.117837,\
        0.110109],\
        [0.162039, 0.168172, 0.174023, 0.179541, 0.184685, 0.189476, 0.194031, 0.198445, 0.202402, 0.205623,\
        0.208711, 0.211683, 0.214500, 0.217167, 0.219697, 0.222101, 0.224390, 0.226574, 0.228661, 0.230660,\
        0.232576, 0.234418, 0.236192, 0.237902, 0.239555, 0.241154, 0.242706, 0.244212, 0.245678, 0.247105,\
        0.248496, 0.249853, 0.251178, 0.252473, 0.253737, 0.254973, 0.256180, 0.257360, 0.258513, 0.259641,\
        0.260744, 0.261825, 0.262885, 0.263928, 0.264955, 0.265971, 0.266979, 0.267985, 0.268992, 0.270004,\
        0.271022, 0.272042, 0.273057, 0.274056, 0.275026, 0.275949, 0.276811, 0.277598, 0.278299, 0.278905,\
        0.279410, 0.279811, 0.280110, 0.280307, 0.280395, 0.280366, 0.280221, 0.279969, 0.279628, 0.279213,\
        0.278741, 0.278212, 0.277612, 0.276918, 0.276110, 0.275186, 0.274155, 0.273025, 0.271798, 0.270466,\
        0.269023, 0.267467, 0.265801, 0.264031, 0.262162, 0.260195, 0.258129, 0.255965, 0.253699, 0.251331,\
        0.248859, 0.246278, 0.243585, 0.240775, 0.237840, 0.234774, 0.231571, 0.228224, 0.224728, 0.221079,\
        0.217270, 0.213299, 0.209161, 0.204851, 0.200366, 0.195700, 0.190851, 0.185813, 0.180583, 0.175156,\
        0.169529, 0.163698, 0.157660, 0.151412, 0.144950, 0.138272, 0.131375, 0.124257, 0.116915, 0.109348,\
        0.101552],\
        [0.157277, 0.163474, 0.169387, 0.174988, 0.180267, 0.185248, 0.189980, 0.194441, 0.198268, 0.201500,\
        0.204728, 0.207776, 0.210630, 0.213321, 0.215868, 0.218286, 0.220587, 0.222780, 0.224875, 0.226879,\
        0.228800, 0.230644, 0.232418, 0.234127, 0.235777, 0.237372, 0.238916, 0.240414, 0.241868, 0.243283,\
        0.244659, 0.246000, 0.247308, 0.248583, 0.249827, 0.251042, 0.252228, 0.253386, 0.254518, 0.255624,\
        0.256707, 0.257767, 0.258808, 0.259831, 0.260839, 0.261836, 0.262824, 0.263806, 0.264787, 0.265767,\
        0.266745, 0.267717, 0.268678, 0.269618, 0.270524, 0.271383, 0.272183, 0.272911, 0.273558, 0.274114,\
        0.274575, 0.274937, 0.275197, 0.275355, 0.275407, 0.275348, 0.275182, 0.274914, 0.274555, 0.274116,\
        0.273605, 0.273023, 0.272360, 0.271604, 0.270744, 0.269774, 0.268698, 0.267519, 0.266236, 0.264847,\
        0.263349, 0.261741, 0.260025, 0.258203, 0.256278, 0.254251, 0.252123, 0.249892, 0.247558, 0.245118,\
        0.242570, 0.239911, 0.237136, 0.234241, 0.231219, 0.228066, 0.224775, 0.221341, 0.217758, 0.214021,\
        0.210126, 0.206068, 0.201843, 0.197446, 0.192875, 0.188123, 0.183188, 0.178066, 0.172751, 0.167241,\
        0.161533, 0.155621, 0.149503, 0.143177, 0.136638, 0.129885, 0.122914, 0.115723, 0.108311, 0.100674,\
        0.092811],\
        [0.152120, 0.158376, 0.164351, 0.170031, 0.175407, 0.180486, 0.185261, 0.189654, 0.193528, 0.197007,\
        0.200364, 0.203467, 0.206370, 0.209104, 0.211687, 0.214136, 0.216463, 0.218678, 0.220792, 0.222813,\
        0.224747, 0.226602, 0.228385, 0.230100, 0.231754, 0.233350, 0.234894, 0.236388, 0.237838, 0.239245,\
        0.240612, 0.241942, 0.243236, 0.244497, 0.245726, 0.246924, 0.248092, 0.249233, 0.250347, 0.251435,\
        0.252499, 0.253541, 0.254564, 0.255568, 0.256557, 0.257533, 0.258499, 0.259456, 0.260407, 0.261352,\
        0.262289, 0.263215, 0.264122, 0.265003, 0.265848, 0.266646, 0.267385, 0.268054, 0.268646, 0.269151,\
        0.269565, 0.269883, 0.270102, 0.270220, 0.270235, 0.270145, 0.269951, 0.269659, 0.269274, 0.268805,\
        0.268254, 0.267622, 0.266904, 0.266091, 0.265177, 0.264158, 0.263034, 0.261803, 0.260467, 0.259024,\
        0.257473, 0.255812, 0.254043, 0.252167, 0.250185, 0.248099, 0.245908, 0.243612, 0.241209, 0.238697,\
        0.236074, 0.233337, 0.230482, 0.227504, 0.224398, 0.221159, 0.217782, 0.214260, 0.210591, 0.206767,\
        0.202785, 0.198641, 0.194330, 0.189848, 0.185190, 0.180354, 0.175334, 0.170127, 0.164730, 0.159137,\
        0.153347, 0.147355, 0.141159, 0.134754, 0.128139, 0.121311, 0.114266, 0.107003, 0.099520, 0.091814,\
        0.083883]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=20 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax20vt10
    zgrid_H_Hmax_vmax20vt10=[[0.27699, 0.29258, 0.30857, 0.32881, 0.35390, 0.38040, 0.40530, 0.42438, 0.44287, 0.46186, 0.48110,\
        0.50025, 0.51796, 0.53732, 0.55805, 0.57819, 0.59677, 0.61230, 0.62611, 0.63871, 0.64943, 0.65847,\
        0.66642, 0.67387, 0.68115, 0.68826, 0.69501, 0.70113, 0.70645, 0.71039, 0.71289, 0.71418, 0.71459,\
        0.71438, 0.71376, 0.71288, 0.71183, 0.71071, 0.70959, 0.70851, 0.70748, 0.70647, 0.70546, 0.70442,\
        0.70331, 0.70206, 0.70047, 0.69825, 0.69592, 0.69415, 0.69341, 0.69406, 0.69632, 0.70183, 0.71289,\
        0.72670, 0.74148, 0.75592, 0.76883, 0.77900, 0.78540, 0.78797, 0.78783, 0.78609, 0.78335, 0.77994,\
        0.77603, 0.77173, 0.76712, 0.76226, 0.75718, 0.75194, 0.74657, 0.74110, 0.73556, 0.72999, 0.72443,\
        0.71892, 0.71355, 0.70840, 0.70360, 0.69940, 0.69638, 0.69445, 0.69240, 0.69018, 0.68745, 0.68417,\
        0.68049, 0.67649, 0.67213, 0.66735, 0.66214, 0.65651, 0.65052, 0.64420, 0.63762, 0.63080, 0.62380,\
        0.61665, 0.60939, 0.60208, 0.59493, 0.58769, 0.58018, 0.57249, 0.56468, 0.55676, 0.54871, 0.54055,\
        0.53224, 0.52377, 0.51506, 0.50598, 0.49624, 0.48570, 0.47456, 0.46308, 0.45135, 0.43943, 0.42733],\
        [0.26382, 0.27722, 0.29125, 0.31128, 0.33460, 0.35950, 0.38273, 0.40169, 0.42098, 0.44009, 0.45899,\
        0.47748, 0.49553, 0.51722, 0.53986, 0.56086, 0.57981, 0.59642, 0.61274, 0.62717, 0.63880, 0.64821,\
        0.65645, 0.66449, 0.67273, 0.68106, 0.68913, 0.69658, 0.70288, 0.70732, 0.71000, 0.71133, 0.71175,\
        0.71158, 0.71104, 0.71025, 0.70930, 0.70829, 0.70731, 0.70640, 0.70555, 0.70474, 0.70390, 0.70302,\
        0.70207, 0.70101, 0.69967, 0.69744, 0.69523, 0.69372, 0.69347, 0.69501, 0.69875, 0.70456, 0.71609,\
        0.73070, 0.74642, 0.76188, 0.77581, 0.78677, 0.79302, 0.79423, 0.79288, 0.79041, 0.78727, 0.78363,\
        0.77961, 0.77527, 0.77067, 0.76586, 0.76087, 0.75573, 0.75048, 0.74514, 0.73974, 0.73431, 0.72889,\
        0.72352, 0.71828, 0.71325, 0.70856, 0.70446, 0.70161, 0.69993, 0.69777, 0.69584, 0.69302, 0.68951,\
        0.68575, 0.68175, 0.67736, 0.67250, 0.66714, 0.66134, 0.65514, 0.64863, 0.64185, 0.63486, 0.62770,\
        0.62041, 0.61306, 0.60574, 0.59856, 0.59086, 0.58303, 0.57509, 0.56707, 0.55895, 0.55073, 0.54239,\
        0.53392, 0.52532, 0.51656, 0.50754, 0.49789, 0.48719, 0.47602, 0.46459, 0.45297, 0.44117, 0.42921],\
        [0.25233, 0.26459, 0.27856, 0.29557, 0.31543, 0.33755, 0.35929, 0.37931, 0.39870, 0.41758, 0.43581,\
        0.45387, 0.47365, 0.49704, 0.52061, 0.54161, 0.56063, 0.57944, 0.59843, 0.61476, 0.62730, 0.63685,\
        0.64518, 0.65395, 0.66347, 0.67330, 0.68287, 0.69167, 0.69904, 0.70404, 0.70692, 0.70831, 0.70874,\
        0.70867, 0.70829, 0.70765, 0.70681, 0.70591, 0.70506, 0.70432, 0.70366, 0.70302, 0.70231, 0.70149,\
        0.70057, 0.69945, 0.69807, 0.69615, 0.69434, 0.69334, 0.69374, 0.69611, 0.70115, 0.70908, 0.72079,\
        0.73549, 0.75151, 0.76736, 0.78170, 0.79309, 0.79923, 0.79832, 0.79615, 0.79340, 0.79016, 0.78652,\
        0.78254, 0.77829, 0.77381, 0.76914, 0.76432, 0.75937, 0.75431, 0.74918, 0.74399, 0.73878, 0.73359,\
        0.72845, 0.72345, 0.71868, 0.71429, 0.71053, 0.70772, 0.70556, 0.70347, 0.70152, 0.69841, 0.69468,\
        0.69094, 0.68702, 0.68265, 0.67767, 0.67212, 0.66609, 0.65967, 0.65294, 0.64596, 0.63878, 0.63144,\
        0.62401, 0.61652, 0.60904, 0.60143, 0.59356, 0.58549, 0.57734, 0.56911, 0.56078, 0.55234, 0.54375,\
        0.53503, 0.52616, 0.51713, 0.50787, 0.49810, 0.48758, 0.47662, 0.46541, 0.45399, 0.44240, 0.43063],\
        [0.24314, 0.25459, 0.26733, 0.28186, 0.29857, 0.31840, 0.33894, 0.35877, 0.37790, 0.39631, 0.41355,\
        0.43056, 0.45046, 0.47485, 0.49974, 0.52061, 0.53921, 0.56005, 0.58187, 0.60103, 0.61528, 0.62450,\
        0.63258, 0.64249, 0.65375, 0.66530, 0.67635, 0.68631, 0.69453, 0.70018, 0.70376, 0.70515, 0.70560,\
        0.70580, 0.70571, 0.70527, 0.70455, 0.70371, 0.70296, 0.70241, 0.70193, 0.70143, 0.70083, 0.69997,\
        0.69894, 0.69770, 0.69623, 0.69462, 0.69331, 0.69293, 0.69410, 0.69736, 0.70338, 0.71287, 0.72534,\
        0.74015, 0.75610, 0.77178, 0.78572, 0.79618, 0.80086, 0.79989, 0.79780, 0.79522, 0.79215, 0.78868,\
        0.78489, 0.78083, 0.77656, 0.77213, 0.76755, 0.76286, 0.75808, 0.75322, 0.74832, 0.74341, 0.73851,\
        0.73368, 0.72899, 0.72454, 0.72046, 0.71693, 0.71402, 0.71156, 0.70925, 0.70674, 0.70350, 0.69974,\
        0.69616, 0.69239, 0.68800, 0.68281, 0.67700, 0.67070, 0.66403, 0.65708, 0.64989, 0.64252, 0.63500,\
        0.62737, 0.61965, 0.61186, 0.60395, 0.59584, 0.58759, 0.57927, 0.57086, 0.56233, 0.55363, 0.54476,\
        0.53570, 0.52645, 0.51699, 0.50731, 0.49739, 0.48713, 0.47654, 0.46566, 0.45453, 0.44320, 0.43167],\
        [0.23628, 0.24711, 0.25887, 0.27196, 0.28692, 0.30438, 0.32266, 0.34150, 0.35997, 0.37746, 0.39359,\
        0.40852, 0.42687, 0.45215, 0.47710, 0.49701, 0.51579, 0.53938, 0.56427, 0.58686, 0.60370, 0.61092,\
        0.61890, 0.63107, 0.64450, 0.65776, 0.67008, 0.68094, 0.68988, 0.69660, 0.70062, 0.70186, 0.70252,\
        0.70328, 0.70359, 0.70336, 0.70273, 0.70189, 0.70120, 0.70087, 0.70057, 0.70013, 0.69955, 0.69865,\
        0.69747, 0.69607, 0.69456, 0.69316, 0.69230, 0.69252, 0.69444, 0.69872, 0.70591, 0.71619, 0.72919,\
        0.74410, 0.75982, 0.77502, 0.78818, 0.79763, 0.80157, 0.80090, 0.79882, 0.79640, 0.79354, 0.79030,\
        0.78675, 0.78297, 0.77899, 0.77485, 0.77059, 0.76623, 0.76178, 0.75727, 0.75272, 0.74815, 0.74360,\
        0.73912, 0.73478, 0.73065, 0.72684, 0.72343, 0.72043, 0.71772, 0.71507, 0.71219, 0.70887, 0.70525,\
        0.70160, 0.69795, 0.69332, 0.68780, 0.68169, 0.67511, 0.66818, 0.66101, 0.65363, 0.64608, 0.63837,\
        0.63053, 0.62255, 0.61444, 0.60618, 0.59781, 0.58946, 0.58102, 0.57245, 0.56371, 0.55475, 0.54557,\
        0.53615, 0.52648, 0.51657, 0.50646, 0.49638, 0.48639, 0.47614, 0.46558, 0.45475, 0.44369, 0.43241],\
        [0.23169, 0.24217, 0.25342, 0.26572, 0.27936, 0.29418, 0.31071, 0.32846, 0.34608, 0.36267, 0.37785,\
        0.39127, 0.41015, 0.43429, 0.45698, 0.47647, 0.49624, 0.52125, 0.54782, 0.57176, 0.58930, 0.59678,\
        0.60708, 0.62186, 0.63705, 0.65153, 0.66470, 0.67617, 0.68560, 0.69272, 0.69718, 0.69901, 0.70031,\
        0.70163, 0.70221, 0.70215, 0.70160, 0.70078, 0.70005, 0.69997, 0.69976, 0.69932, 0.69865, 0.69765,\
        0.69632, 0.69476, 0.69319, 0.69190, 0.69136, 0.69210, 0.69470, 0.69969, 0.70780, 0.71881, 0.73219,\
        0.74715, 0.76261, 0.77731, 0.78975, 0.79840, 0.80216, 0.80188, 0.79980, 0.79732, 0.79458, 0.79155,\
        0.78828, 0.78480, 0.78115, 0.77737, 0.77348, 0.76950, 0.76544, 0.76132, 0.75717, 0.75299, 0.74883,\
        0.74472, 0.74071, 0.73688, 0.73328, 0.72996, 0.72689, 0.72397, 0.72104, 0.71793, 0.71452, 0.71089,\
        0.70715, 0.70317, 0.69814, 0.69246, 0.68607, 0.67921, 0.67206, 0.66470, 0.65717, 0.64947, 0.64159,\
        0.63354, 0.62533, 0.61695, 0.60844, 0.59986, 0.59134, 0.58274, 0.57401, 0.56505, 0.55584, 0.54634,\
        0.53658, 0.52656, 0.51632, 0.50593, 0.49575, 0.48583, 0.47575, 0.46541, 0.45482, 0.44399, 0.43295],\
        [0.22916, 0.23955, 0.25064, 0.26258, 0.27542, 0.28897, 0.30360, 0.32036, 0.33696, 0.35291, 0.36839,\
        0.38463, 0.40360, 0.42368, 0.44388, 0.46353, 0.48432, 0.50862, 0.53463, 0.55790, 0.57583, 0.58842,\
        0.60129, 0.61647, 0.63224, 0.64716, 0.66064, 0.67234, 0.68200, 0.68939, 0.69442, 0.69748, 0.69958,\
        0.70098, 0.70163, 0.70166, 0.70124, 0.70060, 0.70008, 0.69984, 0.69953, 0.69903, 0.69826, 0.69712,\
        0.69561, 0.69387, 0.69216, 0.69088, 0.69051, 0.69162, 0.69479, 0.70058, 0.70919, 0.72066, 0.73433,\
        0.74932, 0.76459, 0.77890, 0.79091, 0.79915, 0.80275, 0.80266, 0.80062, 0.79808, 0.79539, 0.79256,\
        0.78956, 0.78641, 0.78312, 0.77973, 0.77625, 0.77269, 0.76906, 0.76538, 0.76166, 0.75791, 0.75415,\
        0.75041, 0.74674, 0.74317, 0.73976, 0.73650, 0.73337, 0.73029, 0.72715, 0.72382, 0.72024, 0.71641,\
        0.71233, 0.70781, 0.70265, 0.69679, 0.69005, 0.68292, 0.67562, 0.66816, 0.66053, 0.65271, 0.64469,\
        0.63649, 0.62809, 0.61951, 0.61081, 0.60207, 0.59333, 0.58455, 0.57566, 0.56650, 0.55702, 0.54721,\
        0.53712, 0.52683, 0.51642, 0.50599, 0.49574, 0.48567, 0.47554, 0.46528, 0.45485, 0.44421, 0.43337],\
        [0.22837, 0.23884, 0.25000, 0.26197, 0.27479, 0.28840, 0.30266, 0.31712, 0.33206, 0.34727, 0.36306,\
        0.38011, 0.39851, 0.41702, 0.43586, 0.45566, 0.47730, 0.50127, 0.52535, 0.54774, 0.56694, 0.58317,\
        0.59827, 0.61422, 0.62986, 0.64456, 0.65791, 0.66957, 0.67930, 0.68696, 0.69258, 0.69648, 0.69916,\
        0.70079, 0.70156, 0.70171, 0.70144, 0.70099, 0.70056, 0.70020, 0.69976, 0.69920, 0.69838, 0.69712,\
        0.69540, 0.69340, 0.69148, 0.69008, 0.68973, 0.69100, 0.69446, 0.70060, 0.70984, 0.72176, 0.73571,\
        0.75077, 0.76588, 0.77992, 0.79168, 0.79989, 0.80323, 0.80315, 0.80117, 0.79861, 0.79604, 0.79342,\
        0.79069, 0.78786, 0.78496, 0.78197, 0.77893, 0.77583, 0.77267, 0.76946, 0.76619, 0.76288, 0.75953,\
        0.75616, 0.75280, 0.74949, 0.74623, 0.74304, 0.73987, 0.73667, 0.73333, 0.72978, 0.72594, 0.72179,\
        0.71725, 0.71224, 0.70669, 0.70045, 0.69343, 0.68621, 0.67889, 0.67141, 0.66373, 0.65584, 0.64774,\
        0.63942, 0.63089, 0.62216, 0.61327, 0.60431, 0.59537, 0.58652, 0.57754, 0.56817, 0.55838, 0.54823,\
        0.53781, 0.52726, 0.51672, 0.50640, 0.49612, 0.48570, 0.47545, 0.46523, 0.45491, 0.44443, 0.43375],\
        [0.22891, 0.23952, 0.25080, 0.26290, 0.27588, 0.28961, 0.30325, 0.31617, 0.32955, 0.34383, 0.35924,\
        0.37596, 0.39375, 0.41146, 0.42990, 0.44975, 0.47149, 0.49482, 0.51812, 0.54031, 0.56082, 0.57978,\
        0.59772, 0.61369, 0.62876, 0.64305, 0.65614, 0.66767, 0.67742, 0.68529, 0.69134, 0.69578, 0.69886,\
        0.70077, 0.70173, 0.70203, 0.70191, 0.70158, 0.70119, 0.70079, 0.70031, 0.69978, 0.69896, 0.69761,\
        0.69566, 0.69334, 0.69112, 0.68950, 0.68904, 0.69030, 0.69390, 0.70041, 0.70995, 0.72231, 0.73656,\
        0.75170, 0.76670, 0.78046, 0.79183, 0.79963, 0.80319, 0.80337, 0.80152, 0.79904, 0.79663, 0.79419,\
        0.79172, 0.78922, 0.78670, 0.78415, 0.78156, 0.77894, 0.77627, 0.77355, 0.77077, 0.76791, 0.76497,\
        0.76196, 0.75891, 0.75582, 0.75271, 0.74958, 0.74639, 0.74308, 0.73958, 0.73579, 0.73165, 0.72710,\
        0.72207, 0.71650, 0.71032, 0.70347, 0.69629, 0.68925, 0.68202, 0.67455, 0.66685, 0.65891, 0.65075,\
        0.64236, 0.63375, 0.62494, 0.61592, 0.60671, 0.59760, 0.58892, 0.57980, 0.57012, 0.55995, 0.54943,\
        0.53865, 0.52775, 0.51690, 0.50626, 0.49584, 0.48554, 0.47539, 0.46527, 0.45506, 0.44470, 0.43415],\
        [0.23041, 0.24109, 0.25238, 0.26439, 0.27716, 0.29045, 0.30332, 0.31525, 0.32759, 0.34100, 0.35570,\
        0.37165, 0.38844, 0.40592, 0.42429, 0.44383, 0.46524, 0.48818, 0.51133, 0.53377, 0.55534, 0.57584,\
        0.59492, 0.61192, 0.62746, 0.64181, 0.65482, 0.66630, 0.67609, 0.68413, 0.69047, 0.69527, 0.69870,\
        0.70085, 0.70196, 0.70239, 0.70239, 0.70217, 0.70185, 0.70148, 0.70108, 0.70060, 0.69981, 0.69851,\
        0.69628, 0.69358, 0.69102, 0.68915, 0.68853, 0.68971, 0.69322, 0.69965, 0.70983, 0.72269, 0.73720,\
        0.75242, 0.76733, 0.78086, 0.79190, 0.79946, 0.80311, 0.80355, 0.80198, 0.79963, 0.79721, 0.79490,\
        0.79267, 0.79051, 0.78840, 0.78629, 0.78418, 0.78205, 0.77990, 0.77769, 0.77540, 0.77300, 0.77047,\
        0.76781, 0.76504, 0.76216, 0.75919, 0.75612, 0.75293, 0.74954, 0.74589, 0.74188, 0.73743, 0.73248,\
        0.72697, 0.72085, 0.71412, 0.70687, 0.69956, 0.69248, 0.68520, 0.67768, 0.66993, 0.66194, 0.65372,\
        0.64529, 0.63666, 0.62787, 0.61893, 0.60990, 0.60097, 0.59202, 0.58246, 0.57230, 0.56169, 0.55077,\
        0.53964, 0.52839, 0.51712, 0.50600, 0.49542, 0.48540, 0.47548, 0.46548, 0.45535, 0.44506, 0.43459],\
        [0.23257, 0.24319, 0.25428, 0.26589, 0.27798, 0.29031, 0.30225, 0.31350, 0.32524, 0.33797, 0.35190,\
        0.36709, 0.38347, 0.40076, 0.41799, 0.43683, 0.45787, 0.48055, 0.50384, 0.52694, 0.54939, 0.57080,\
        0.59072, 0.60896, 0.62549, 0.64030, 0.65351, 0.66511, 0.67501, 0.68321, 0.68975, 0.69477, 0.69841,\
        0.70075, 0.70202, 0.70261, 0.70276, 0.70264, 0.70238, 0.70207, 0.70173, 0.70134, 0.70078, 0.69950,\
        0.69691, 0.69389, 0.69110, 0.68908, 0.68835, 0.68951, 0.69319, 0.70002, 0.71035, 0.72337, 0.73800,\
        0.75323, 0.76804, 0.78143, 0.79235, 0.79976, 0.80332, 0.80378, 0.80231, 0.79998, 0.79766, 0.79551,\
        0.79356, 0.79176, 0.79007, 0.78843, 0.78681, 0.78519, 0.78356, 0.78188, 0.78011, 0.77817, 0.77605,\
        0.77372, 0.77121, 0.76851, 0.76567, 0.76267, 0.75948, 0.75605, 0.75228, 0.74807, 0.74334, 0.73802,\
        0.73207, 0.72548, 0.71834, 0.71085, 0.70340, 0.69603, 0.68841, 0.68081, 0.67299, 0.66492, 0.65663,\
        0.64814, 0.63951, 0.63078, 0.62200, 0.61327, 0.60456, 0.59531, 0.58516, 0.57446, 0.56343, 0.55219,\
        0.54080, 0.52932, 0.51781, 0.50641, 0.49573, 0.48585, 0.47597, 0.46597, 0.45582, 0.44554, 0.43511],\
        [0.23519, 0.24562, 0.25634, 0.26732, 0.27844, 0.28945, 0.30016, 0.31109, 0.32245, 0.33455, 0.34765,\
        0.36188, 0.37718, 0.39323, 0.40965, 0.42832, 0.44923, 0.47192, 0.49561, 0.51945, 0.54270, 0.56503,\
        0.58615, 0.60563, 0.62293, 0.63828, 0.65192, 0.66382, 0.67396, 0.68234, 0.68904, 0.69418, 0.69790,\
        0.70037, 0.70186, 0.70263, 0.70290, 0.70288, 0.70268, 0.70238, 0.70203, 0.70161, 0.70100, 0.69969,\
        0.69709, 0.69409, 0.69134, 0.68934, 0.68860, 0.68979, 0.69372, 0.70099, 0.71148, 0.72458, 0.73922,\
        0.75437, 0.76905, 0.78229, 0.79312, 0.80052, 0.80373, 0.80403, 0.80253, 0.80024, 0.79799, 0.79602,\
        0.79437, 0.79298, 0.79176, 0.79061, 0.78948, 0.78837, 0.78729, 0.78617, 0.78492, 0.78345, 0.78172,\
        0.77971, 0.77742, 0.77489, 0.77215, 0.76922, 0.76607, 0.76262, 0.75877, 0.75441, 0.74944, 0.74378,\
        0.73742, 0.73041, 0.72287, 0.71498, 0.70697, 0.69890, 0.69158, 0.68400, 0.67603, 0.66781, 0.65940,\
        0.65083, 0.64214, 0.63340, 0.62467, 0.61599, 0.60724, 0.59785, 0.58729, 0.57624, 0.56497, 0.55356,\
        0.54208, 0.53058, 0.51916, 0.50798, 0.49737, 0.48711, 0.47693, 0.46673, 0.45648, 0.44613, 0.43569],\
        [0.23814, 0.24832, 0.25859, 0.26889, 0.27910, 0.28908, 0.29875, 0.30898, 0.31963, 0.33093, 0.34309,\
        0.35618, 0.37011, 0.38470, 0.40054, 0.41908, 0.43964, 0.46245, 0.48672, 0.51126, 0.53506, 0.55816,\
        0.58046, 0.60116, 0.61930, 0.63551, 0.64984, 0.66226, 0.67275, 0.68138, 0.68825, 0.69351, 0.69732,\
        0.69992, 0.70153, 0.70242, 0.70283, 0.70289, 0.70271, 0.70238, 0.70193, 0.70134, 0.70043, 0.69900,\
        0.69679, 0.69421, 0.69183, 0.69010, 0.68948, 0.69068, 0.69459, 0.70214, 0.71315, 0.72649, 0.74111,\
        0.75606, 0.77047, 0.78343, 0.79394, 0.80090, 0.80403, 0.80423, 0.80269, 0.80040, 0.79821, 0.79643,\
        0.79511, 0.79419, 0.79351, 0.79288, 0.79225, 0.79164, 0.79110, 0.79057, 0.78987, 0.78886, 0.78751,\
        0.78580, 0.78370, 0.78128, 0.77863, 0.77577, 0.77267, 0.76925, 0.76538, 0.76092, 0.75574, 0.74977,\
        0.74303, 0.73559, 0.72761, 0.71930, 0.71086, 0.70282, 0.69537, 0.68723, 0.67895, 0.67054, 0.66196,\
        0.65324, 0.64442, 0.63556, 0.62668, 0.61777, 0.60868, 0.59900, 0.58845, 0.57741, 0.56615, 0.55477,\
        0.54336, 0.53199, 0.52079, 0.50992, 0.49935, 0.48865, 0.47810, 0.46765, 0.45723, 0.44679, 0.43631],\
        [0.24137, 0.25129, 0.26112, 0.27078, 0.28020, 0.28942, 0.29872, 0.30752, 0.31706, 0.32744, 0.33862,\
        0.35061, 0.36341, 0.37722, 0.39274, 0.40972, 0.42926, 0.45225, 0.47696, 0.50206, 0.52602, 0.54980,\
        0.57301, 0.59473, 0.61443, 0.63194, 0.64725, 0.66035, 0.67131, 0.68025, 0.68732, 0.69271, 0.69663,\
        0.69931, 0.70100, 0.70203, 0.70257, 0.70271, 0.70253, 0.70213, 0.70157, 0.70085, 0.69983, 0.69838,\
        0.69652, 0.69450, 0.69275, 0.69161, 0.69136, 0.69264, 0.69665, 0.70456, 0.71604, 0.72951, 0.74395,\
        0.75852, 0.77245, 0.78497, 0.79515, 0.80174, 0.80446, 0.80449, 0.80287, 0.80055, 0.79834, 0.79670,\
        0.79575, 0.79541, 0.79538, 0.79533, 0.79517, 0.79504, 0.79504, 0.79515, 0.79502, 0.79441, 0.79341,\
        0.79201, 0.79004, 0.78768, 0.78511, 0.78233, 0.77930, 0.77594, 0.77211, 0.76759, 0.76224, 0.75597,\
        0.74882, 0.74091, 0.73248, 0.72377, 0.71510, 0.70677, 0.69845, 0.69006, 0.68161, 0.67301, 0.66424,\
        0.65531, 0.64630, 0.63721, 0.62804, 0.61875, 0.60920, 0.59926, 0.58884, 0.57804, 0.56698, 0.55576,\
        0.54450, 0.53329, 0.52223, 0.51137, 0.50061, 0.48989, 0.47918, 0.46855, 0.45798, 0.44745, 0.43693],\
        [0.24484, 0.25453, 0.26398, 0.27308, 0.28174, 0.28992, 0.29778, 0.30593, 0.31478, 0.32436, 0.33464,\
        0.34565, 0.35745, 0.37025, 0.38415, 0.39895, 0.41905, 0.44208, 0.46632, 0.49089, 0.51540, 0.54029,\
        0.56471, 0.58782, 0.60900, 0.62785, 0.64425, 0.65812, 0.66959, 0.67887, 0.68618, 0.69175, 0.69580,\
        0.69855, 0.70032, 0.70155, 0.70223, 0.70244, 0.70227, 0.70180, 0.70112, 0.70032, 0.69938, 0.69809,\
        0.69657, 0.69511, 0.69418, 0.69402, 0.69454, 0.69629, 0.70057, 0.70920, 0.72076, 0.73405, 0.74806,\
        0.76200, 0.77520, 0.78702, 0.79661, 0.80275, 0.80520, 0.80504, 0.80323, 0.80073, 0.79838, 0.79682,\
        0.79629, 0.79662, 0.79742, 0.79803, 0.79834, 0.79870, 0.79921, 0.79994, 0.80036, 0.80009, 0.79941,\
        0.79838, 0.79642, 0.79407, 0.79161, 0.78889, 0.78593, 0.78270, 0.77895, 0.77443, 0.76892, 0.76232,\
        0.75468, 0.74624, 0.73727, 0.72807, 0.71892, 0.70994, 0.70108, 0.69255, 0.68401, 0.67521, 0.66618,\
        0.65701, 0.64776, 0.63842, 0.62896, 0.61932, 0.60939, 0.59919, 0.58896, 0.57842, 0.56757, 0.55654,\
        0.54543, 0.53434, 0.52335, 0.51250, 0.50171, 0.49088, 0.48005, 0.46929, 0.45863, 0.44805, 0.43750],\
        [0.24849, 0.25800, 0.26718, 0.27587, 0.28390, 0.29112, 0.29782, 0.30502, 0.31319, 0.32203, 0.33148,\
        0.34159, 0.35250, 0.36439, 0.37755, 0.39302, 0.41184, 0.43310, 0.45609, 0.48000, 0.50484, 0.53057,\
        0.55620, 0.58064, 0.60311, 0.62350, 0.64096, 0.65554, 0.66750, 0.67715, 0.68475, 0.69057, 0.69485,\
        0.69783, 0.69984, 0.70114, 0.70190, 0.70218, 0.70203, 0.70154, 0.70080, 0.69996, 0.69921, 0.69823,\
        0.69701, 0.69599, 0.69585, 0.69708, 0.69887, 0.70169, 0.70710, 0.71587, 0.72730, 0.74024, 0.75369,\
        0.76682, 0.77905, 0.78999, 0.79876, 0.80441, 0.80677, 0.80636, 0.80411, 0.80111, 0.79839, 0.79682,\
        0.79675, 0.79783, 0.79950, 0.80093, 0.80188, 0.80286, 0.80394, 0.80505, 0.80582, 0.80597, 0.80559,\
        0.80465, 0.80283, 0.80056, 0.79828, 0.79544, 0.79257, 0.78953, 0.78587, 0.78139, 0.77574, 0.76872,\
        0.76047, 0.75135, 0.74173, 0.73195, 0.72227, 0.71286, 0.70376, 0.69502, 0.68622, 0.67706, 0.66771,\
        0.65829, 0.64882, 0.63928, 0.62962, 0.61978, 0.60973, 0.59955, 0.58926, 0.57879, 0.56809, 0.55718,\
        0.54616, 0.53512, 0.52414, 0.51326, 0.50248, 0.49155, 0.48063, 0.46982, 0.45914, 0.44854, 0.43800],\
        [0.25223, 0.26164, 0.27065, 0.27912, 0.28683, 0.29357, 0.29900, 0.30528, 0.31273, 0.32074, 0.32933,\
        0.33860, 0.34872, 0.35996, 0.37280, 0.38793, 0.40534, 0.42505, 0.44693, 0.47039, 0.49507, 0.52137,\
        0.54785, 0.57350, 0.59767, 0.61912, 0.63731, 0.65247, 0.66490, 0.67493, 0.68289, 0.68907, 0.69371,\
        0.69704, 0.69929, 0.70071, 0.70161, 0.70200, 0.70193, 0.70149, 0.70084, 0.70014, 0.69962, 0.69892,\
        0.69792, 0.69711, 0.69742, 0.69956, 0.70326, 0.70778, 0.71453, 0.72382, 0.73527, 0.74801, 0.76101,\
        0.77335, 0.78448, 0.79440, 0.80283, 0.80813, 0.81014, 0.80920, 0.80616, 0.80220, 0.79866, 0.79685,\
        0.79737, 0.79929, 0.80166, 0.80394, 0.80585, 0.80758, 0.80921, 0.81064, 0.81168, 0.81214, 0.81196,\
        0.81110, 0.80953, 0.80744, 0.80505, 0.80205, 0.79942, 0.79647, 0.79278, 0.78841, 0.78261, 0.77500,\
        0.76596, 0.75604, 0.74566, 0.73519, 0.72495, 0.71522, 0.70616, 0.69735, 0.68800, 0.67833, 0.66868,\
        0.65908, 0.64950, 0.63987, 0.63013, 0.62022, 0.61016, 0.60001, 0.58969, 0.57926, 0.56861, 0.55776,\
        0.54675, 0.53568, 0.52461, 0.51362, 0.50268, 0.49176, 0.48089, 0.47013, 0.45948, 0.44892, 0.43843],\
        [0.25595, 0.26527, 0.27421, 0.28259, 0.29018, 0.29686, 0.30229, 0.30735, 0.31352, 0.32051, 0.32820,\
        0.33666, 0.34603, 0.35660, 0.36870, 0.38271, 0.39891, 0.41750, 0.43865, 0.46184, 0.48675, 0.51291,\
        0.53918, 0.56557, 0.59113, 0.61377, 0.63282, 0.64860, 0.66151, 0.67199, 0.68041, 0.68708, 0.69223,\
        0.69605, 0.69867, 0.70027, 0.70144, 0.70195, 0.70196, 0.70163, 0.70114, 0.70069, 0.70040, 0.70003,\
        0.69935, 0.69877, 0.69941, 0.70269, 0.70775, 0.71417, 0.72229, 0.73238, 0.74420, 0.75708, 0.77009,\
        0.78208, 0.79229, 0.80136, 0.80939, 0.81456, 0.81615, 0.81443, 0.81018, 0.80465, 0.79956, 0.79717,\
        0.79867, 0.80159, 0.80470, 0.80774, 0.81048, 0.81291, 0.81503, 0.81679, 0.81805, 0.81870, 0.81867,\
        0.81794, 0.81652, 0.81454, 0.81210, 0.80934, 0.80652, 0.80332, 0.79953, 0.79544, 0.78935, 0.78083,\
        0.77083, 0.76003, 0.74885, 0.73764, 0.72670, 0.71636, 0.70701, 0.69842, 0.68848, 0.67862, 0.66893,\
        0.65935, 0.64982, 0.64024, 0.63055, 0.62068, 0.61058, 0.60035, 0.59018, 0.57986, 0.56922, 0.55833,\
        0.54725, 0.53608, 0.52488, 0.51371, 0.50262, 0.49170, 0.48093, 0.47026, 0.45969, 0.44920, 0.43879],\
        [0.25951, 0.26874, 0.27758, 0.28591, 0.29344, 0.29944, 0.30469, 0.30958, 0.31494, 0.32104, 0.32792,\
        0.33563, 0.34430, 0.35413, 0.36538, 0.37828, 0.39307, 0.41064, 0.43113, 0.45361, 0.47791, 0.50318,\
        0.52881, 0.55576, 0.58247, 0.60671, 0.62700, 0.64357, 0.65706, 0.66810, 0.67713, 0.68445, 0.69027,\
        0.69475, 0.69807, 0.70029, 0.70141, 0.70196, 0.70203, 0.70182, 0.70152, 0.70128, 0.70118, 0.70114,\
        0.70112, 0.70140, 0.70302, 0.70719, 0.71331, 0.72101, 0.73030, 0.74124, 0.75364, 0.76701, 0.78057,\
        0.79316, 0.80322, 0.81221, 0.81971, 0.82438, 0.82541, 0.82279, 0.81719, 0.80982, 0.80238, 0.79876,\
        0.80171, 0.80538, 0.80918, 0.81281, 0.81613, 0.81905, 0.82154, 0.82354, 0.82496, 0.82572, 0.82577,\
        0.82510, 0.82375, 0.82180, 0.81936, 0.81655, 0.81347, 0.81002, 0.80622, 0.80203, 0.79516, 0.78553,\
        0.77467, 0.76311, 0.75122, 0.73927, 0.72756, 0.71628, 0.70555, 0.69614, 0.68722, 0.67788, 0.66850,\
        0.65915, 0.64982, 0.64045, 0.63095, 0.62126, 0.61133, 0.60120, 0.59110, 0.58071, 0.56996, 0.55893,\
        0.54772, 0.53641, 0.52507, 0.51375, 0.50253, 0.49161, 0.48091, 0.47031, 0.45981, 0.44941, 0.43909],\
        [0.26281, 0.27188, 0.28053, 0.28866, 0.29610, 0.30220, 0.30689, 0.31153, 0.31647, 0.32200, 0.32825,\
        0.33533, 0.34338, 0.35256, 0.36308, 0.37516, 0.38914, 0.40567, 0.42461, 0.44543, 0.46774, 0.49115,\
        0.51659, 0.54424, 0.57206, 0.59793, 0.61970, 0.63706, 0.65123, 0.66304, 0.67290, 0.68105, 0.68768,\
        0.69293, 0.69689, 0.69956, 0.70113, 0.70182, 0.70201, 0.70196, 0.70184, 0.70182, 0.70201, 0.70244,\
        0.70323, 0.70474, 0.70769, 0.71269, 0.71965, 0.72831, 0.73851, 0.75019, 0.76322, 0.77721, 0.79141,\
        0.80486, 0.81606, 0.82546, 0.83293, 0.83738, 0.83797, 0.83458, 0.82795, 0.81945, 0.81117, 0.80691,\
        0.80844, 0.81186, 0.81568, 0.81947, 0.82301, 0.82616, 0.82883, 0.83094, 0.83241, 0.83319, 0.83323,\
        0.83254, 0.83114, 0.82910, 0.82651, 0.82346, 0.81999, 0.81607, 0.81161, 0.80614, 0.79845, 0.78848,\
        0.77727, 0.76526, 0.75280, 0.74024, 0.72787, 0.71598, 0.70492, 0.69493, 0.68571, 0.67667, 0.66764,\
        0.65862, 0.64960, 0.64052, 0.63132, 0.62196, 0.61240, 0.60265, 0.59241, 0.58175, 0.57078, 0.55955,\
        0.54816, 0.53670, 0.52525, 0.51387, 0.50263, 0.49168, 0.48093, 0.47034, 0.45990, 0.44959, 0.43937],\
        [0.26581, 0.27467, 0.28301, 0.29069, 0.29758, 0.30350, 0.30852, 0.31318, 0.31796, 0.32317, 0.32899,\
        0.33560, 0.34314, 0.35178, 0.36172, 0.37320, 0.38654, 0.40192, 0.41904, 0.43790, 0.45829, 0.48021,\
        0.50471, 0.53202, 0.56018, 0.58742, 0.61066, 0.62862, 0.64381, 0.65673, 0.66769, 0.67688, 0.68447,\
        0.69055, 0.69520, 0.69851, 0.70046, 0.70138, 0.70179, 0.70194, 0.70205, 0.70232, 0.70290, 0.70394,\
        0.70563, 0.70834, 0.71251, 0.71850, 0.72634, 0.73584, 0.74681, 0.75910, 0.77266, 0.78733, 0.80218,\
        0.81637, 0.82898, 0.83963, 0.84787, 0.85271, 0.85324, 0.84933, 0.84203, 0.83314, 0.82501, 0.82017,\
        0.81944, 0.82133, 0.82437, 0.82778, 0.83116, 0.83425, 0.83689, 0.83896, 0.84037, 0.84106, 0.84099,\
        0.84015, 0.83858, 0.83632, 0.83342, 0.82995, 0.82591, 0.82122, 0.81568, 0.80880, 0.80011, 0.79029,\
        0.77896, 0.76663, 0.75377, 0.74074, 0.72787, 0.71553, 0.70417, 0.69390, 0.68440, 0.67542, 0.66664,\
        0.65794, 0.64924, 0.64049, 0.63162, 0.62259, 0.61334, 0.60373, 0.59352, 0.58274, 0.57157, 0.56014,\
        0.54856, 0.53693, 0.52536, 0.51398, 0.50288, 0.49181, 0.48092, 0.47032, 0.45995, 0.44975, 0.43966],\
        [0.26853, 0.27716, 0.28520, 0.29245, 0.29882, 0.30466, 0.30986, 0.31465, 0.31940, 0.32445, 0.33002,\
        0.33629, 0.34345, 0.35164, 0.36106, 0.37191, 0.38439, 0.39854, 0.41417, 0.43140, 0.45025, 0.47094,\
        0.49414, 0.51956, 0.54695, 0.57439, 0.59821, 0.61797, 0.63499, 0.64937, 0.66166, 0.67209, 0.68077,\
        0.68778, 0.69315, 0.69697, 0.69933, 0.70068, 0.70139, 0.70176, 0.70214, 0.70274, 0.70379, 0.70548,\
        0.70806, 0.71184, 0.71714, 0.72419, 0.73299, 0.74341, 0.75518, 0.76800, 0.78182, 0.79703, 0.81290,\
        0.82802, 0.84181, 0.85377, 0.86329, 0.86917, 0.87004, 0.86570, 0.85776, 0.84855, 0.84031, 0.83475,\
        0.83246, 0.83278, 0.83471, 0.83743, 0.84039, 0.84320, 0.84564, 0.84753, 0.84875, 0.84924, 0.84894,\
        0.84785, 0.84599, 0.84336, 0.84003, 0.83599, 0.83124, 0.82569, 0.81916, 0.81143, 0.80249, 0.79197,\
        0.78015, 0.76751, 0.75433, 0.74094, 0.72770, 0.71491, 0.70297, 0.69253, 0.68316, 0.67428, 0.66568,\
        0.65723, 0.64883, 0.64038, 0.63181, 0.62305, 0.61402, 0.60457, 0.59441, 0.58354, 0.57224, 0.56065,\
        0.54888, 0.53706, 0.52531, 0.51379, 0.50253, 0.49150, 0.48071, 0.47022, 0.45999, 0.44993, 0.43998],\
        [0.27098, 0.27943, 0.28729, 0.29440, 0.30056, 0.30613, 0.31127, 0.31607, 0.32083, 0.32581, 0.33123,\
        0.33729, 0.34416, 0.35198, 0.36092, 0.37112, 0.38268, 0.39562, 0.41005, 0.42606, 0.44358, 0.46275,\
        0.48366, 0.50660, 0.53375, 0.56088, 0.58544, 0.60724, 0.62557, 0.64137, 0.65514, 0.66696, 0.67688,\
        0.68489, 0.69103, 0.69541, 0.69827, 0.70000, 0.70082, 0.70143, 0.70207, 0.70303, 0.70455, 0.70688,\
        0.71028, 0.71502, 0.72136, 0.72945, 0.73931, 0.75080, 0.76360, 0.77721, 0.79137, 0.80686, 0.82359,\
        0.83953, 0.85406, 0.86690, 0.87769, 0.88506, 0.88657, 0.88161, 0.87289, 0.86337, 0.85497, 0.84893,\
        0.84568, 0.84490, 0.84588, 0.84789, 0.85033, 0.85276, 0.85487, 0.85646, 0.85740, 0.85759, 0.85698,\
        0.85554, 0.85327, 0.85019, 0.84630, 0.84162, 0.83611, 0.82971, 0.82229, 0.81373, 0.80394, 0.79297,\
        0.78098, 0.76810, 0.75465, 0.74100, 0.72754, 0.71459, 0.70254, 0.69187, 0.68228, 0.67336, 0.66485,\
        0.65659, 0.64842, 0.64023, 0.63191, 0.62334, 0.61443, 0.60503, 0.59493, 0.58411, 0.57280, 0.56111,\
        0.54916, 0.53712, 0.52514, 0.51339, 0.50197, 0.49099, 0.48039, 0.47010, 0.46006, 0.45018, 0.44039],\
        [0.27316, 0.28145, 0.28918, 0.29629, 0.30248, 0.30777, 0.31273, 0.31750, 0.32226, 0.32722, 0.33257,\
        0.33850, 0.34517, 0.35270, 0.36123, 0.37085, 0.38165, 0.39369, 0.40714, 0.42202, 0.43841, 0.45640,\
        0.47606, 0.49828, 0.52376, 0.54947, 0.57405, 0.59645, 0.61581, 0.63320, 0.64855, 0.66188, 0.67315,\
        0.68225, 0.68919, 0.69409, 0.69727, 0.69913, 0.70019, 0.70101, 0.70187, 0.70314, 0.70509, 0.70799,\
        0.71210, 0.71768, 0.72494, 0.73403, 0.74497, 0.75762, 0.77179, 0.78696, 0.80199, 0.81819, 0.83494,\
        0.85091, 0.86533, 0.87803, 0.88901, 0.89769, 0.89979, 0.89413, 0.88514, 0.87586, 0.86767, 0.86158,\
        0.85797, 0.85664, 0.85706, 0.85856, 0.86056, 0.86259, 0.86433, 0.86556, 0.86614, 0.86596, 0.86496,\
        0.86310, 0.86036, 0.85673, 0.85223, 0.84686, 0.84059, 0.83339, 0.82518, 0.81585, 0.80532, 0.79398,\
        0.78177, 0.76858, 0.75484, 0.74097, 0.72736, 0.71442, 0.70251, 0.69156, 0.68167, 0.67263, 0.66417,\
        0.65604, 0.64806, 0.64009, 0.63197, 0.62357, 0.61472, 0.60529, 0.59525, 0.58461, 0.57335, 0.56158,\
        0.54948, 0.53722, 0.52499, 0.51300, 0.50148, 0.49055, 0.48015, 0.47010, 0.46026, 0.45056, 0.44094],\
        [0.27506, 0.28317, 0.29071, 0.29758, 0.30370, 0.30911, 0.31408, 0.31887, 0.32365, 0.32863, 0.33397,\
        0.33984, 0.34639, 0.35372, 0.36194, 0.37111, 0.38132, 0.39271, 0.40529, 0.41915, 0.43468, 0.45197,\
        0.47139, 0.49351, 0.51615, 0.53965, 0.56348, 0.58595, 0.60670, 0.62556, 0.64237, 0.65727, 0.66999,\
        0.68024, 0.68793, 0.69323, 0.69657, 0.69851, 0.69967, 0.70052, 0.70153, 0.70304, 0.70535, 0.70872,\
        0.71342, 0.71969, 0.72773, 0.73771, 0.74965, 0.76341, 0.77874, 0.79557, 0.81255, 0.82954, 0.84648,\
        0.86210, 0.87566, 0.88708, 0.89629, 0.90342, 0.90545, 0.90052, 0.89361, 0.88522, 0.87771, 0.87209,\
        0.86868, 0.86735, 0.86764, 0.86893, 0.87065, 0.87235, 0.87372, 0.87456, 0.87474, 0.87417, 0.87274,\
        0.87042, 0.86715, 0.86295, 0.85780, 0.85171, 0.84469, 0.83674, 0.82786, 0.81804, 0.80734, 0.79564,\
        0.78268, 0.76898, 0.75491, 0.74081, 0.72703, 0.71397, 0.70195, 0.69097, 0.68104, 0.67200, 0.66360,\
        0.65559, 0.64778, 0.63999, 0.63206, 0.62384, 0.61514, 0.60579, 0.59584, 0.58530, 0.57402, 0.56216,\
        0.54991, 0.53749, 0.52505, 0.51281, 0.50111, 0.49034, 0.48020, 0.47036, 0.46071, 0.45116, 0.44167],\
        [0.27670, 0.28465, 0.29201, 0.29863, 0.30471, 0.31020, 0.31526, 0.32011, 0.32496, 0.32999, 0.33536,\
        0.34123, 0.34773, 0.35495, 0.36298, 0.37184, 0.38153, 0.39207, 0.40378, 0.41716, 0.43205, 0.44852,\
        0.46676, 0.48659, 0.50759, 0.53073, 0.55448, 0.57750, 0.59924, 0.61871, 0.63690, 0.65352, 0.66780,\
        0.67922, 0.68758, 0.69309, 0.69634, 0.69812, 0.69915, 0.69997, 0.70107, 0.70275, 0.70533, 0.70906,\
        0.71420, 0.72098, 0.72961, 0.74029, 0.75313, 0.76799, 0.78428, 0.80169, 0.82090, 0.83991, 0.85776,\
        0.87318, 0.88554, 0.89527, 0.90287, 0.90774, 0.90863, 0.90512, 0.89880, 0.89149, 0.88508, 0.88030,\
        0.87752, 0.87665, 0.87723, 0.87863, 0.88025, 0.88171, 0.88273, 0.88317, 0.88296, 0.88200, 0.88015,\
        0.87735, 0.87356, 0.86876, 0.86295, 0.85615, 0.84839, 0.83973, 0.83018, 0.81981, 0.80870, 0.79658,\
        0.78316, 0.76916, 0.75486, 0.74054, 0.72655, 0.71325, 0.70104, 0.69014, 0.68037, 0.67145, 0.66315,\
        0.65525, 0.64757, 0.63994, 0.63220, 0.62420, 0.61576, 0.60672, 0.59692, 0.58628, 0.57485, 0.56286,\
        0.55051, 0.53800, 0.52551, 0.51323, 0.50144, 0.49084, 0.48080, 0.47103, 0.46147, 0.45202, 0.44263],\
        [0.27809, 0.28591, 0.29320, 0.29990, 0.30576, 0.31118, 0.31627, 0.32119, 0.32612, 0.33123, 0.33667,\
        0.34258, 0.34909, 0.35629, 0.36424, 0.37297, 0.38245, 0.39260, 0.40365, 0.41636, 0.43038, 0.44582,\
        0.46273, 0.48097, 0.50087, 0.52397, 0.54750, 0.57039, 0.59189, 0.61229, 0.63251, 0.65110, 0.66697,\
        0.67952, 0.68843, 0.69386, 0.69666, 0.69800, 0.69878, 0.69952, 0.70058, 0.70232, 0.70506, 0.70905,\
        0.71449, 0.72159, 0.73056, 0.74165, 0.75511, 0.77109, 0.78935, 0.80829, 0.82840, 0.84917, 0.86864,\
        0.88447, 0.89561, 0.90381, 0.90964, 0.91256, 0.91203, 0.90815, 0.90197, 0.89573, 0.89031, 0.88643,\
        0.88448, 0.88437, 0.88560, 0.88738, 0.88905, 0.89032, 0.89105, 0.89107, 0.89049, 0.88925, 0.88700,\
        0.88376, 0.87949, 0.87411, 0.86763, 0.86012, 0.85164, 0.84227, 0.83205, 0.82100, 0.80904, 0.79634,\
        0.78310, 0.76912, 0.75472, 0.74028, 0.72614, 0.71263, 0.70013, 0.68948, 0.67987, 0.67106, 0.66285,\
        0.65503, 0.64744, 0.63992, 0.63233, 0.62452, 0.61637, 0.60766, 0.59806, 0.58728, 0.57568, 0.56359,\
        0.55122, 0.53875, 0.52637, 0.51435, 0.50301, 0.49219, 0.48191, 0.47211, 0.46256, 0.45316, 0.44382],\
        [0.27922, 0.28692, 0.29406, 0.30060, 0.30653, 0.31197, 0.31710, 0.32209, 0.32710, 0.33230, 0.33782,\
        0.34380, 0.35035, 0.35758, 0.36553, 0.37426, 0.38379, 0.39415, 0.40498, 0.41650, 0.42945, 0.44391,\
        0.45998, 0.47789, 0.49810, 0.51979, 0.54274, 0.56565, 0.58716, 0.60835, 0.63044, 0.65044, 0.66762,\
        0.68126, 0.69065, 0.69566, 0.69753, 0.69816, 0.69859, 0.69917, 0.70008, 0.70183, 0.70464, 0.70880,\
        0.71445, 0.72172, 0.73076, 0.74183, 0.75531, 0.77158, 0.79077, 0.81153, 0.83358, 0.85662, 0.87841,\
        0.89607, 0.90617, 0.91289, 0.91683, 0.91786, 0.91591, 0.91135, 0.90525, 0.89905, 0.89407, 0.89084,\
        0.88970, 0.89049, 0.89261, 0.89499, 0.89671, 0.89783, 0.89835, 0.89793, 0.89697, 0.89574, 0.89303,\
        0.88951, 0.88484, 0.87891, 0.87179, 0.86357, 0.85438, 0.84433, 0.83352, 0.82197, 0.80963, 0.79667,\
        0.78323, 0.76911, 0.75461, 0.74011, 0.72596, 0.71257, 0.70041, 0.68944, 0.67973, 0.67093, 0.66274,\
        0.65496, 0.64739, 0.63990, 0.63235, 0.62464, 0.61663, 0.60813, 0.59871, 0.58786, 0.57624, 0.56420,\
        0.55193, 0.53960, 0.52740, 0.51553, 0.50418, 0.49341, 0.48326, 0.47349, 0.46395, 0.45456, 0.44525],\
        [0.28011, 0.28768, 0.29470, 0.30115, 0.30707, 0.31253, 0.31771, 0.32277, 0.32787, 0.33315, 0.33875,\
        0.34479, 0.35140, 0.35865, 0.36662, 0.37534, 0.38481, 0.39489, 0.40540, 0.41650, 0.42872, 0.44244,\
        0.45787, 0.47522, 0.49459, 0.51619, 0.53949, 0.56337, 0.58721, 0.60964, 0.63077, 0.65113, 0.66930,\
        0.68405, 0.69417, 0.69853, 0.69893, 0.69870, 0.69868, 0.69903, 0.69986, 0.70132, 0.70413, 0.70849,\
        0.71436, 0.72171, 0.73063, 0.74132, 0.75415, 0.76962, 0.78818, 0.81042, 0.83593, 0.86190, 0.88542,\
        0.90389, 0.91525, 0.92131, 0.92388, 0.92342, 0.92024, 0.91491, 0.90838, 0.90201, 0.89692, 0.89390,\
        0.89352, 0.89513, 0.89797, 0.90105, 0.90285, 0.90391, 0.90424, 0.90356, 0.90243, 0.90094, 0.89797,\
        0.89460, 0.88955, 0.88309, 0.87533, 0.86642, 0.85653, 0.84584, 0.83449, 0.82265, 0.81039, 0.79747,\
        0.78352, 0.76910, 0.75452, 0.74004, 0.72594, 0.71259, 0.70039, 0.68952, 0.67986, 0.67107, 0.66287,\
        0.65504, 0.64742, 0.63986, 0.63224, 0.62445, 0.61638, 0.60783, 0.59839, 0.58776, 0.57638, 0.56457,\
        0.55255, 0.54047, 0.52850, 0.51679, 0.50541, 0.49484, 0.48489, 0.47513, 0.46558, 0.45619, 0.44688],\
        [0.28076, 0.28822, 0.29520, 0.30160, 0.30743, 0.31287, 0.31808, 0.32320, 0.32837, 0.33373, 0.33939,\
        0.34549, 0.35212, 0.35937, 0.36731, 0.37597, 0.38531, 0.39516, 0.40536, 0.41606, 0.42776, 0.44092,\
        0.45585, 0.47270, 0.49162, 0.51313, 0.53667, 0.56112, 0.58543, 0.60837, 0.63057, 0.65170, 0.67088,\
        0.68679, 0.69786, 0.70208, 0.70148, 0.70013, 0.69931, 0.69903, 0.69930, 0.70061, 0.70364, 0.70836,\
        0.71456, 0.72208, 0.73086, 0.74104, 0.75298, 0.76739, 0.78532, 0.80877, 0.83755, 0.86652, 0.89146,\
        0.91025, 0.92230, 0.92848, 0.93037, 0.92894, 0.92487, 0.91885, 0.91174, 0.90464, 0.89884, 0.89618,\
        0.89664, 0.89905, 0.90202, 0.90525, 0.90749, 0.90873, 0.90904, 0.90846, 0.90721, 0.90522, 0.90249,\
        0.89883, 0.89343, 0.88656, 0.87818, 0.86858, 0.85802, 0.84669, 0.83479, 0.82254, 0.81017, 0.79727,\
        0.78322, 0.76882, 0.75438, 0.74008, 0.72613, 0.71280, 0.70046, 0.68985, 0.68033, 0.67154, 0.66326,\
        0.65531, 0.64754, 0.63982, 0.63202, 0.62402, 0.61569, 0.60690, 0.59746, 0.58713, 0.57615, 0.56473,\
        0.55305, 0.54131, 0.52966, 0.51829, 0.50742, 0.49705, 0.48681, 0.47695, 0.46737, 0.45798, 0.44868],\
        [0.28115, 0.28849, 0.29539, 0.30176, 0.30750, 0.31295, 0.31820, 0.32338, 0.32861, 0.33402, 0.33973,\
        0.34583, 0.35244, 0.35964, 0.36749, 0.37602, 0.38520, 0.39489, 0.40480, 0.41503, 0.42625, 0.43898,\
        0.45357, 0.47022, 0.48907, 0.51023, 0.53335, 0.55783, 0.58248, 0.60599, 0.62905, 0.65084, 0.67114,\
        0.68833, 0.70008, 0.70489, 0.70463, 0.70256, 0.70062, 0.69940, 0.69893, 0.70005, 0.70344, 0.70870,\
        0.71542, 0.72330, 0.73217, 0.74203, 0.75326, 0.76679, 0.78454, 0.80922, 0.84017, 0.87213, 0.89883,\
        0.91726, 0.92880, 0.93470, 0.93618, 0.93425, 0.92971, 0.92328, 0.91578, 0.90815, 0.90163, 0.89922,\
        0.90036, 0.90291, 0.90611, 0.90913, 0.91145, 0.91281, 0.91319, 0.91265, 0.91129, 0.90912, 0.90612,\
        0.90201, 0.89667, 0.88927, 0.88022, 0.86998, 0.85881, 0.84690, 0.83440, 0.82148, 0.80837, 0.79544,\
        0.78201, 0.76817, 0.75418, 0.74026, 0.72665, 0.71364, 0.70162, 0.69092, 0.68128, 0.67236, 0.66391,\
        0.65577, 0.64779, 0.63984, 0.63179, 0.62351, 0.61487, 0.60575, 0.59624, 0.58631, 0.57577, 0.56476,\
        0.55347, 0.54208, 0.53076, 0.51969, 0.50901, 0.49865, 0.48854, 0.47876, 0.46925, 0.45988, 0.45059],\
        [0.28131, 0.28847, 0.29516, 0.30139, 0.30724, 0.31276, 0.31807, 0.32331, 0.32859, 0.33403, 0.33975,\
        0.34583, 0.35236, 0.35943, 0.36710, 0.37539, 0.38430, 0.39372, 0.40333, 0.41313, 0.42394, 0.43633,\
        0.45060, 0.46699, 0.48574, 0.50638, 0.52820, 0.55257, 0.57851, 0.60325, 0.62501, 0.64736, 0.66914,\
        0.68807, 0.70131, 0.70668, 0.70686, 0.70449, 0.70167, 0.69959, 0.69877, 0.69996, 0.70379, 0.70971,\
        0.71715, 0.72571, 0.73508, 0.74511, 0.75606, 0.76900, 0.78590, 0.81066, 0.84392, 0.87761, 0.90552,\
        0.92336, 0.93431, 0.93992, 0.94125, 0.93921, 0.93464, 0.92829, 0.92098, 0.91367, 0.90773, 0.90520,\
        0.90581, 0.90787, 0.91061, 0.91326, 0.91534, 0.91658, 0.91689, 0.91627, 0.91474, 0.91230, 0.90892,\
        0.90459, 0.89894, 0.89084, 0.88127, 0.87054, 0.85890, 0.84655, 0.83364, 0.82024, 0.80652, 0.79338,\
        0.78066, 0.76748, 0.75402, 0.74059, 0.72745, 0.71488, 0.70316, 0.69243, 0.68262, 0.67348, 0.66480,\
        0.65640, 0.64815, 0.63993, 0.63163, 0.62312, 0.61428, 0.60502, 0.59551, 0.58573, 0.57547, 0.56480,\
        0.55385, 0.54279, 0.53177, 0.52089, 0.51023, 0.49991, 0.49008, 0.48057, 0.47119, 0.46188, 0.45260],\
        [0.28130, 0.28830, 0.29486, 0.30101, 0.30686, 0.31239, 0.31774, 0.32301, 0.32832, 0.33378, 0.33947,\
        0.34549, 0.35191, 0.35879, 0.36618, 0.37411, 0.38258, 0.39154, 0.40080, 0.41021, 0.42076, 0.43290,\
        0.44676, 0.46246, 0.47995, 0.49878, 0.52022, 0.54418, 0.56895, 0.59284, 0.61626, 0.64066, 0.66460,\
        0.68534, 0.69995, 0.70691, 0.70783, 0.70530, 0.70183, 0.69931, 0.69869, 0.70031, 0.70467, 0.71129,\
        0.71963, 0.72922, 0.73965, 0.75054, 0.76188, 0.77474, 0.79221, 0.81752, 0.84947, 0.88193, 0.90875,\
        0.92704, 0.93816, 0.94387, 0.94537, 0.94363, 0.93947, 0.93367, 0.92708, 0.92066, 0.91558, 0.91290,\
        0.91260, 0.91382, 0.91575, 0.91775, 0.91934, 0.92025, 0.92030, 0.91943, 0.91760, 0.91477, 0.91084,\
        0.90562, 0.89889, 0.89111, 0.88134, 0.87027, 0.85830, 0.84571, 0.83269, 0.81939, 0.80600, 0.79289,\
        0.78001, 0.76704, 0.75398, 0.74102, 0.72836, 0.71620, 0.70473, 0.69405, 0.68413, 0.67479, 0.66584,\
        0.65714, 0.64859, 0.64009, 0.63156, 0.62289, 0.61399, 0.60481, 0.59531, 0.58550, 0.57534, 0.56488,\
        0.55422, 0.54347, 0.53273, 0.52210, 0.51162, 0.50136, 0.49189, 0.48255, 0.47322, 0.46393, 0.45468],\
        [0.28115, 0.28804, 0.29456, 0.30071, 0.30642, 0.31188, 0.31723, 0.32252, 0.32785, 0.33330, 0.33896,\
        0.34489, 0.35117, 0.35782, 0.36489, 0.37239, 0.38030, 0.38858, 0.39716, 0.40641, 0.41701, 0.42902,\
        0.44246, 0.45734, 0.47355, 0.49126, 0.51191, 0.53457, 0.55795, 0.58122, 0.60513, 0.63221, 0.65848,\
        0.68132, 0.69788, 0.70651, 0.70795, 0.70509, 0.70134, 0.69899, 0.69859, 0.70072, 0.70565, 0.71301,\
        0.72232, 0.73324, 0.74530, 0.75788, 0.77044, 0.78349, 0.80008, 0.82548, 0.85582, 0.88628, 0.91148,\
        0.92916, 0.94036, 0.94639, 0.94840, 0.94730, 0.94391, 0.93901, 0.93340, 0.92797, 0.92358, 0.92087,\
        0.91991, 0.92024, 0.92129, 0.92250, 0.92346, 0.92384, 0.92347, 0.92221, 0.91997, 0.91668, 0.91222,\
        0.90654, 0.89980, 0.89090, 0.88061, 0.86920, 0.85698, 0.84428, 0.83135, 0.81843, 0.80559, 0.79263,\
        0.77960, 0.76669, 0.75394, 0.74142, 0.72923, 0.71747, 0.70622, 0.69564, 0.68571, 0.67617, 0.66693,\
        0.65791, 0.64904, 0.64025, 0.63149, 0.62268, 0.61378, 0.60468, 0.59522, 0.58537, 0.57526, 0.56497,\
        0.55456, 0.54410, 0.53368, 0.52338, 0.51332, 0.50367, 0.49416, 0.48464, 0.47529, 0.46601, 0.45676],\
        [0.28089, 0.28763, 0.29404, 0.30008, 0.30574, 0.31121, 0.31657, 0.32188, 0.32722, 0.33266, 0.33827,\
        0.34412, 0.35024, 0.35668, 0.36345, 0.37053, 0.37793, 0.38561, 0.39369, 0.40278, 0.41339, 0.42529,\
        0.43840, 0.45269, 0.46824, 0.48548, 0.50476, 0.52558, 0.54804, 0.57151, 0.59680, 0.62412, 0.65156,\
        0.67694, 0.69648, 0.70647, 0.70752, 0.70389, 0.69994, 0.69805, 0.69817, 0.70072, 0.70626, 0.71418,\
        0.72432, 0.73655, 0.75057, 0.76565, 0.78045, 0.79420, 0.81057, 0.83346, 0.86125, 0.88939, 0.91291,\
        0.92979, 0.94094, 0.94744, 0.95020, 0.95002, 0.94766, 0.94386, 0.93934, 0.93486, 0.93108, 0.92844,\
        0.92705, 0.92666, 0.92689, 0.92730, 0.92754, 0.92730, 0.92638, 0.92462, 0.92190, 0.91810, 0.91310,\
        0.90675, 0.89899, 0.88981, 0.87913, 0.86737, 0.85491, 0.84208, 0.82921, 0.81658, 0.80432, 0.79163,\
        0.77868, 0.76601, 0.75366, 0.74162, 0.72993, 0.71864, 0.70777, 0.69736, 0.68731, 0.67751, 0.66795,\
        0.65860, 0.64940, 0.64032, 0.63131, 0.62234, 0.61336, 0.60433, 0.59490, 0.58507, 0.57508, 0.56499,\
        0.55482, 0.54465, 0.53454, 0.52458, 0.51483, 0.50532, 0.49589, 0.48655, 0.47727, 0.46804, 0.45883],\
        [0.28055, 0.28713, 0.29333, 0.29921, 0.30490, 0.31043, 0.31582, 0.32115, 0.32649, 0.33192, 0.33749,\
        0.34326, 0.34926, 0.35553, 0.36207, 0.36887, 0.37593, 0.38330, 0.39120, 0.40013, 0.41055, 0.42227,\
        0.43510, 0.44898, 0.46396, 0.48016, 0.49765, 0.51774, 0.53976, 0.56308, 0.58824, 0.61444, 0.64289,\
        0.67077, 0.69349, 0.70527, 0.70583, 0.70154, 0.69814, 0.69675, 0.69742, 0.70052, 0.70602, 0.71401,\
        0.72451, 0.73760, 0.75321, 0.77085, 0.78892, 0.80316, 0.81736, 0.83814, 0.86382, 0.89019, 0.91236,\
        0.92851, 0.93977, 0.94695, 0.95070, 0.95167, 0.95051, 0.94790, 0.94449, 0.94093, 0.93774, 0.93526,\
        0.93361, 0.93268, 0.93221, 0.93189, 0.93142, 0.93052, 0.92898, 0.92665, 0.92338, 0.91906, 0.91352,\
        0.90656, 0.89808, 0.88821, 0.87701, 0.86482, 0.85206, 0.83907, 0.82610, 0.81337, 0.80112, 0.78900,\
        0.77679, 0.76473, 0.75293, 0.74144, 0.73029, 0.71949, 0.70907, 0.69888, 0.68866, 0.67860, 0.66876,\
        0.65910, 0.64959, 0.64021, 0.63094, 0.62173, 0.61257, 0.60340, 0.59405, 0.58446, 0.57471, 0.56486,\
        0.55496, 0.54507, 0.53525, 0.52555, 0.51603, 0.50666, 0.49744, 0.48827, 0.47911, 0.46998, 0.46083],\
        [0.28020, 0.28663, 0.29271, 0.29848, 0.30415, 0.30968, 0.31506, 0.32038, 0.32571, 0.33112, 0.33666,\
        0.34239, 0.34832, 0.35449, 0.36091, 0.36758, 0.37453, 0.38183, 0.38967, 0.39847, 0.40868, 0.42023,\
        0.43282, 0.44639, 0.46097, 0.47663, 0.49366, 0.51273, 0.53334, 0.55511, 0.57800, 0.60319, 0.63243,\
        0.66150, 0.68614, 0.70074, 0.70088, 0.69855, 0.69562, 0.69440, 0.69529, 0.69848, 0.70395, 0.71169,\
        0.72193, 0.73502, 0.75104, 0.76946, 0.78858, 0.80229, 0.81610, 0.83713, 0.86200, 0.88747, 0.90888,\
        0.92477, 0.93672, 0.94497, 0.94995, 0.95220, 0.95233, 0.95095, 0.94865, 0.94598, 0.94337, 0.94112,\
        0.93936, 0.93805, 0.93702, 0.93606, 0.93492, 0.93336, 0.93118, 0.92823, 0.92436, 0.91947, 0.91344,\
        0.90605, 0.89695, 0.88620, 0.87428, 0.86159, 0.84849, 0.83532, 0.82231, 0.80948, 0.79709, 0.78565,\
        0.77421, 0.76286, 0.75170, 0.74081, 0.73018, 0.71982, 0.70967, 0.69962, 0.68942, 0.67927, 0.66923,\
        0.65931, 0.64952, 0.63987, 0.63034, 0.62091, 0.61152, 0.60219, 0.59295, 0.58364, 0.57417, 0.56458,\
        0.55495, 0.54532, 0.53577, 0.52631, 0.51700, 0.50785, 0.49884, 0.48978, 0.48080, 0.47181, 0.46275],\
        [0.27986, 0.28620, 0.29225, 0.29805, 0.30361, 0.30901, 0.31431, 0.31960, 0.32491, 0.33032, 0.33586,\
        0.34157, 0.34748, 0.35363, 0.36003, 0.36671, 0.37373, 0.38115, 0.38910, 0.39776, 0.40790, 0.41924,\
        0.43156, 0.44489, 0.45929, 0.47486, 0.49178, 0.50979, 0.52879, 0.54894, 0.57045, 0.59438, 0.62191,\
        0.64975, 0.67338, 0.68796, 0.69226, 0.69218, 0.69089, 0.69016, 0.69107, 0.69414, 0.69950, 0.70676,\
        0.71623, 0.72851, 0.74372, 0.76090, 0.77783, 0.79229, 0.80962, 0.83128, 0.85598, 0.88046, 0.90146,\
        0.91879, 0.93216, 0.94179, 0.94812, 0.95171, 0.95314, 0.95297, 0.95174, 0.94992, 0.94790, 0.94593,\
        0.94416, 0.94260, 0.94114, 0.93964, 0.93790, 0.93571, 0.93290, 0.92930, 0.92477, 0.91922, 0.91258,\
        0.90472, 0.89500, 0.88345, 0.87087, 0.85767, 0.84422, 0.83091, 0.81805, 0.80580, 0.79386, 0.78234,\
        0.77131, 0.76055, 0.75001, 0.73970, 0.72959, 0.71962, 0.70966, 0.69963, 0.68962, 0.67950, 0.66931,\
        0.65917, 0.64914, 0.63925, 0.62952, 0.61994, 0.61046, 0.60100, 0.59194, 0.58281, 0.57352, 0.56416,\
        0.55477, 0.54541, 0.53610, 0.52688, 0.51775, 0.50870, 0.49976, 0.49108, 0.48238, 0.47357, 0.46459],\
        [0.27955, 0.28581, 0.29181, 0.29756, 0.30305, 0.30834, 0.31357, 0.31881, 0.32412, 0.32954, 0.33510,\
        0.34083, 0.34677, 0.35295, 0.35941, 0.36621, 0.37340, 0.38109, 0.38939, 0.39845, 0.40824, 0.41908,\
        0.43103, 0.44412, 0.45839, 0.47381, 0.49027, 0.50754, 0.52565, 0.54470, 0.56508, 0.58750, 0.61235,\
        0.63756, 0.65918, 0.67392, 0.68125, 0.68375, 0.68408, 0.68410, 0.68505, 0.68774, 0.69253, 0.69937,\
        0.70791, 0.71908, 0.73321, 0.74913, 0.76536, 0.78176, 0.80009, 0.82238, 0.84804, 0.87261, 0.89414,\
        0.91237, 0.92698, 0.93787, 0.94548, 0.95034, 0.95301, 0.95400, 0.95378, 0.95277, 0.95132, 0.94966,\
        0.94795, 0.94623, 0.94445, 0.94251, 0.94025, 0.93751, 0.93410, 0.92984, 0.92460, 0.91826, 0.91076,\
        0.90205, 0.89173, 0.87980, 0.86678, 0.85315, 0.83934, 0.82583, 0.81305, 0.80109, 0.78970, 0.77868,\
        0.76811, 0.75788, 0.74791, 0.73814, 0.72853, 0.71900, 0.70940, 0.69959, 0.68959, 0.67934, 0.66897,\
        0.65863, 0.64839, 0.63831, 0.62842, 0.61878, 0.60948, 0.60045, 0.59120, 0.58196, 0.57277, 0.56359,\
        0.55443, 0.54531, 0.53626, 0.52729, 0.51839, 0.50957, 0.50089, 0.49244, 0.48397, 0.47527, 0.46634],\
        [0.27924, 0.28541, 0.29132, 0.29700, 0.30241, 0.30760, 0.31279, 0.31803, 0.32336, 0.32882, 0.33441,\
        0.34019, 0.34618, 0.35243, 0.35898, 0.36591, 0.37330, 0.38124, 0.38981, 0.39899, 0.40866, 0.41914,\
        0.43075, 0.44361, 0.45775, 0.47309, 0.48933, 0.50617, 0.52360, 0.54178, 0.56115, 0.58196, 0.60421,\
        0.62689, 0.64682, 0.66142, 0.67027, 0.67467, 0.67644, 0.67720, 0.67823, 0.68046, 0.68452, 0.69056,\
        0.69845, 0.70892, 0.72247, 0.73752, 0.75324, 0.77013, 0.78934, 0.81384, 0.84049, 0.86543, 0.88753,\
        0.90642, 0.92176, 0.93357, 0.94226, 0.94827, 0.95207, 0.95412, 0.95483, 0.95458, 0.95367, 0.95234,\
        0.95072, 0.94891, 0.94689, 0.94459, 0.94191, 0.93868, 0.93475, 0.92991, 0.92398, 0.91683, 0.90836,\
        0.89868, 0.88786, 0.87562, 0.86226, 0.84826, 0.83408, 0.82021, 0.80739, 0.79585, 0.78497, 0.77461,\
        0.76462, 0.75493, 0.74547, 0.73618, 0.72701, 0.71790, 0.70877, 0.69935, 0.68917, 0.67870, 0.66816,\
        0.65765, 0.64725, 0.63702, 0.62701, 0.61725, 0.60782, 0.59896, 0.58994, 0.58084, 0.57181, 0.56283,\
        0.55390, 0.54503, 0.53625, 0.52756, 0.51897, 0.51048, 0.50208, 0.49397, 0.48561, 0.47689, 0.46795],\
        [0.27895, 0.28500, 0.29079, 0.29632, 0.30161, 0.30677, 0.31198, 0.31727, 0.32266, 0.32816, 0.33382,\
        0.33966, 0.34572, 0.35204, 0.35867, 0.36570, 0.37321, 0.38129, 0.38997, 0.39916, 0.40869, 0.41892,\
        0.43026, 0.44290, 0.45690, 0.47221, 0.48869, 0.50545, 0.52195, 0.53952, 0.55828, 0.57801, 0.59839,\
        0.61920, 0.63744, 0.65143, 0.66086, 0.66643, 0.66934, 0.67086, 0.67216, 0.67424, 0.67795, 0.68378,\
        0.69163, 0.70186, 0.71495, 0.72848, 0.74247, 0.75892, 0.78039, 0.80663, 0.83283, 0.85816, 0.88103,\
        0.90058, 0.91635, 0.92893, 0.93857, 0.94560, 0.95043, 0.95345, 0.95502, 0.95546, 0.95505, 0.95400,\
        0.95250, 0.95062, 0.94841, 0.94583, 0.94280, 0.93919, 0.93484, 0.92954, 0.92308, 0.91527, 0.90610,\
        0.89566, 0.88414, 0.87140, 0.85765, 0.84332, 0.82887, 0.81470, 0.80137, 0.79039, 0.78022, 0.77045,\
        0.76099, 0.75177, 0.74274, 0.73385, 0.72503, 0.71617, 0.70716, 0.69785, 0.68785, 0.67739, 0.66678,\
        0.65617, 0.64567, 0.63537, 0.62531, 0.61549, 0.60584, 0.59668, 0.58814, 0.57941, 0.57061, 0.56185,\
        0.55315, 0.54454, 0.53604, 0.52765, 0.51940, 0.51134, 0.50362, 0.49571, 0.48705, 0.47826, 0.46931],\
        [0.27869, 0.28464, 0.29029, 0.29561, 0.30072, 0.30595, 0.31124, 0.31659, 0.32203, 0.32761, 0.33334,\
        0.33925, 0.34537, 0.35175, 0.35843, 0.36549, 0.37300, 0.38105, 0.38969, 0.39883, 0.40816, 0.41817,\
        0.42930, 0.44171, 0.45542, 0.47040, 0.48640, 0.50275, 0.51949, 0.53719, 0.55600, 0.57569, 0.59574,\
        0.61454, 0.63092, 0.64406, 0.65363, 0.66000, 0.66397, 0.66648, 0.66855, 0.67122, 0.67557, 0.68264,\
        0.69265, 0.70319, 0.71448, 0.72397, 0.73324, 0.74592, 0.76569, 0.79283, 0.82176, 0.84924, 0.87354,\
        0.89375, 0.91037, 0.92391, 0.93450, 0.94248, 0.94826, 0.95217, 0.95450, 0.95555, 0.95555, 0.95475,\
        0.95332, 0.95138, 0.94899, 0.94617, 0.94286, 0.93896, 0.93431, 0.92873, 0.92195, 0.91370, 0.90396,\
        0.89298, 0.88072, 0.86733, 0.85311, 0.83851, 0.82401, 0.81012, 0.79726, 0.78599, 0.77595, 0.76651,\
        0.75742, 0.74855, 0.73984, 0.73123, 0.72265, 0.71398, 0.70501, 0.69560, 0.68577, 0.67544, 0.66484,\
        0.65419, 0.64365, 0.63332, 0.62331, 0.61369, 0.60447, 0.59542, 0.58655, 0.57784, 0.56921, 0.56065,\
        0.55218, 0.54382, 0.53557, 0.52746, 0.51949, 0.51166, 0.50384, 0.49601, 0.48783, 0.47921, 0.47035],\
        [0.27849, 0.28437, 0.28997, 0.29528, 0.30030, 0.30545, 0.31067, 0.31603, 0.32153, 0.32718, 0.33299,\
        0.33897, 0.34516, 0.35157, 0.35826, 0.36527, 0.37266, 0.38051, 0.38888, 0.39774, 0.40692, 0.41682,\
        0.42784, 0.44003, 0.45336, 0.46776, 0.48307, 0.49917, 0.51623, 0.53442, 0.55365, 0.57350, 0.59308,\
        0.61094, 0.62628, 0.63881, 0.64851, 0.65570, 0.66097, 0.66506, 0.66880, 0.67332, 0.68004, 0.68965,\
        0.70463, 0.71699, 0.72323, 0.72493, 0.72573, 0.73038, 0.74458, 0.77169, 0.80553, 0.83857, 0.86500,\
        0.88631, 0.90400, 0.91870, 0.93017, 0.93904, 0.94573, 0.95046, 0.95347, 0.95500, 0.95532, 0.95467,\
        0.95325, 0.95121, 0.94864, 0.94558, 0.94202, 0.93788, 0.93304, 0.92734, 0.92053, 0.91221, 0.90198,\
        0.89040, 0.87741, 0.86327, 0.84855, 0.83373, 0.81927, 0.80573, 0.79340, 0.78223, 0.77220, 0.76290,\
        0.75401, 0.74536, 0.73684, 0.72839, 0.71998, 0.71151, 0.70273, 0.69335, 0.68339, 0.67302, 0.66241,\
        0.65175, 0.64118, 0.63084, 0.62085, 0.61139, 0.60256, 0.59363, 0.58474, 0.57608, 0.56759, 0.55922,\
        0.55097, 0.54283, 0.53483, 0.52695, 0.51921, 0.51159, 0.50402, 0.49631, 0.48820, 0.47974, 0.47103],\
        [0.27835, 0.28416, 0.28975, 0.29518, 0.30045, 0.30511, 0.31022, 0.31560, 0.32116, 0.32689, 0.33278,\
        0.33885, 0.34509, 0.35154, 0.35821, 0.36513, 0.37233, 0.37986, 0.38778, 0.39621, 0.40516, 0.41511,\
        0.42615, 0.43817, 0.45110, 0.46492, 0.47964, 0.49542, 0.51259, 0.53121, 0.55089, 0.57092, 0.59028,\
        0.60777, 0.62268, 0.63497, 0.64499, 0.65325, 0.66034, 0.66688, 0.67359, 0.68162, 0.69358, 0.71068,\
        0.73029, 0.74142, 0.73969, 0.73062, 0.72055, 0.71441, 0.71897, 0.74624, 0.78906, 0.82693, 0.85476,\
        0.87796, 0.89784, 0.91341, 0.92565, 0.93550, 0.94312, 0.94861, 0.95216, 0.95403, 0.95451, 0.95388,\
        0.95236, 0.95014, 0.94734, 0.94403, 0.94021, 0.93585, 0.93083, 0.92504, 0.91836, 0.91058, 0.90073,\
        0.88808, 0.87382, 0.85883, 0.84368, 0.82880, 0.81447, 0.80100, 0.78921, 0.77852, 0.76870, 0.75955,\
        0.75081, 0.74228, 0.73383, 0.72540, 0.71701, 0.70872, 0.70009, 0.69070, 0.68065, 0.67020, 0.65955,\
        0.64887, 0.63829, 0.62796, 0.61795, 0.60832, 0.59939, 0.59096, 0.58243, 0.57400, 0.56571, 0.55755,\
        0.54952, 0.54160, 0.53381, 0.52613, 0.51857, 0.51111, 0.50376, 0.49622, 0.48819, 0.47990, 0.47137],\
        [0.27824, 0.28397, 0.28946, 0.29467, 0.29937, 0.30453, 0.30986, 0.31532, 0.32094, 0.32675, 0.33274,\
        0.33889, 0.34522, 0.35172, 0.35838, 0.36522, 0.37223, 0.37944, 0.38690, 0.39468, 0.40339, 0.41367,\
        0.42475, 0.43658, 0.44912, 0.46239, 0.47650, 0.49178, 0.50882, 0.52774, 0.54778, 0.56804, 0.58748,\
        0.60490, 0.61952, 0.63167, 0.64216, 0.65177, 0.66120, 0.67111, 0.68222, 0.69597, 0.71534, 0.74167,\
        0.76605, 0.77239, 0.75927, 0.73839, 0.71761, 0.70299, 0.69623, 0.72132, 0.76730, 0.81004, 0.84134,\
        0.86820, 0.89040, 0.90759, 0.92121, 0.93235, 0.94092, 0.94699, 0.95084, 0.95281, 0.95325, 0.95246,\
        0.95071, 0.94821, 0.94510, 0.94148, 0.93738, 0.93276, 0.92752, 0.92147, 0.91453, 0.90682, 0.89762,\
        0.88405, 0.86893, 0.85343, 0.83812, 0.82346, 0.80975, 0.79696, 0.78533, 0.77491, 0.76534, 0.75639,\
        0.74780, 0.73938, 0.73095, 0.72242, 0.71377, 0.70536, 0.69727, 0.68764, 0.67749, 0.66698, 0.65630,\
        0.64561, 0.63505, 0.62477, 0.61489, 0.60540, 0.59636, 0.58805, 0.57983, 0.57166, 0.56360, 0.55567,\
        0.54785, 0.54014, 0.53254, 0.52504, 0.51762, 0.51024, 0.50275, 0.49536, 0.48775, 0.47975, 0.47143],\
        [0.27820, 0.28382, 0.28924, 0.29447, 0.29956, 0.30452, 0.30972, 0.31518, 0.32087, 0.32678, 0.33287,\
        0.33913, 0.34556, 0.35213, 0.35884, 0.36566, 0.37259, 0.37962, 0.38685, 0.39458, 0.40332, 0.41322,\
        0.42409, 0.43566, 0.44783, 0.46062, 0.47414, 0.48878, 0.50540, 0.52436, 0.54447, 0.56472, 0.58421,\
        0.60170, 0.61583, 0.62785, 0.63885, 0.64989, 0.66188, 0.67560, 0.69174, 0.71155, 0.73690, 0.76660,\
        0.79357, 0.79529, 0.77405, 0.74514, 0.71579, 0.69445, 0.68814, 0.70657, 0.74818, 0.79043, 0.82837,\
        0.86038, 0.88470, 0.90272, 0.91816, 0.93065, 0.93980, 0.94602, 0.94978, 0.95152, 0.95165, 0.95051,\
        0.94837, 0.94546, 0.94195, 0.93795, 0.93349, 0.92858, 0.92311, 0.91671, 0.90896, 0.90005, 0.88991,\
        0.87700, 0.86215, 0.84673, 0.83158, 0.81730, 0.80428, 0.79238, 0.78133, 0.77130, 0.76208, 0.75339,\
        0.74500, 0.73671, 0.72834, 0.71975, 0.71084, 0.70171, 0.69322, 0.68402, 0.67391, 0.66338, 0.65269,\
        0.64201, 0.63149, 0.62127, 0.61155, 0.60270, 0.59388, 0.58530, 0.57713, 0.56915, 0.56131, 0.55360,\
        0.54599, 0.53849, 0.53107, 0.52373, 0.51647, 0.50927, 0.50215, 0.49479, 0.48722, 0.47940, 0.47129],\
        [0.27823, 0.28373, 0.28906, 0.29432, 0.29951, 0.30449, 0.30965, 0.31515, 0.32094, 0.32696, 0.33317,\
        0.33956, 0.34611, 0.35280, 0.35962, 0.36653, 0.37352, 0.38056, 0.38772, 0.39531, 0.40393, 0.41372,\
        0.42435, 0.43563, 0.44748, 0.45990, 0.47299, 0.48710, 0.50312, 0.52160, 0.54115, 0.56074, 0.57951,\
        0.59641, 0.61059, 0.62246, 0.63382, 0.64607, 0.66028, 0.67732, 0.69779, 0.72144, 0.74893, 0.77695,\
        0.79661, 0.79886, 0.78065, 0.75051, 0.71858, 0.69507, 0.68877, 0.70596, 0.74240, 0.78542, 0.82594,\
        0.86000, 0.88577, 0.90414, 0.91956, 0.93169, 0.94032, 0.94594, 0.94909, 0.95023, 0.94977, 0.94806,\
        0.94537, 0.94194, 0.93794, 0.93347, 0.92858, 0.92330, 0.91765, 0.91129, 0.90285, 0.89287, 0.88160,\
        0.86850, 0.85396, 0.83892, 0.82418, 0.81037, 0.79799, 0.78704, 0.77696, 0.76761, 0.75886, 0.75052,\
        0.74238, 0.73426, 0.72602, 0.71753, 0.70870, 0.69956, 0.69018, 0.68042, 0.67012, 0.65949, 0.64876,\
        0.63813, 0.62769, 0.61753, 0.60772, 0.59863, 0.59043, 0.58226, 0.57428, 0.56651, 0.55889, 0.55139,\
        0.54400, 0.53669, 0.52946, 0.52228, 0.51516, 0.50805, 0.50099, 0.49392, 0.48656, 0.47892, 0.47100],\
        [0.27841, 0.28374, 0.28890, 0.29395, 0.29898, 0.30411, 0.30951, 0.31519, 0.32113, 0.32729, 0.33364,\
        0.34017, 0.34686, 0.35370, 0.36069, 0.36779, 0.37499, 0.38225, 0.38953, 0.39702, 0.40552, 0.41511,\
        0.42545, 0.43645, 0.44805, 0.46026, 0.47315, 0.48695, 0.50234, 0.51969, 0.53792, 0.55612, 0.57358,\
        0.58964, 0.60333, 0.61467, 0.62615, 0.63918, 0.65476, 0.67365, 0.69612, 0.72219, 0.75089, 0.77818,\
        0.79631, 0.79934, 0.78395, 0.75556, 0.72373, 0.69909, 0.69221, 0.70962, 0.74723, 0.79214, 0.83340,\
        0.86759, 0.89392, 0.91227, 0.92543, 0.93537, 0.94233, 0.94663, 0.94868, 0.94887, 0.94758, 0.94512,\
        0.94175, 0.93769, 0.93311, 0.92813, 0.92276, 0.91698, 0.91085, 0.90510, 0.89694, 0.88560, 0.87312,\
        0.85953, 0.84511, 0.83053, 0.81644, 0.80333, 0.79187, 0.78181, 0.77258, 0.76395, 0.75574, 0.74778,\
        0.73991, 0.73197, 0.72386, 0.71550, 0.70676, 0.69736, 0.68733, 0.67690, 0.66618, 0.65532, 0.64453,\
        0.63396, 0.62371, 0.61378, 0.60415, 0.59516, 0.58694, 0.57903, 0.57130, 0.56375, 0.55636, 0.54909,\
        0.54192, 0.53481, 0.52776, 0.52075, 0.51379, 0.50686, 0.49987, 0.49297, 0.48582, 0.47837, 0.47062],\
        [0.27879, 0.28397, 0.28892, 0.29373, 0.29861, 0.30386, 0.30950, 0.31540, 0.32151, 0.32780, 0.33428,\
        0.34092, 0.34775, 0.35475, 0.36193, 0.36929, 0.37680, 0.38446, 0.39219, 0.39986, 0.40807, 0.41721,\
        0.42715, 0.43785, 0.44927, 0.46140, 0.47425, 0.48788, 0.50250, 0.51839, 0.53473, 0.55097, 0.56655,\
        0.58088, 0.59333, 0.60428, 0.61573, 0.62894, 0.64489, 0.66426, 0.68745, 0.71385, 0.74452, 0.77445,\
        0.79501, 0.79978, 0.78742, 0.76136, 0.73017, 0.70454, 0.69628, 0.71687, 0.75814, 0.80487, 0.84658,\
        0.87994, 0.90441, 0.92124, 0.93246, 0.94009, 0.94499, 0.94758, 0.94822, 0.94726, 0.94498, 0.94165,\
        0.93751, 0.93275, 0.92757, 0.92207, 0.91624, 0.91002, 0.90316, 0.89643, 0.88838, 0.87703, 0.86414,\
        0.85033, 0.83606, 0.82196, 0.80887, 0.79690, 0.78632, 0.77700, 0.76848, 0.76047, 0.75276, 0.74517,\
        0.73755, 0.72974, 0.72165, 0.71324, 0.70453, 0.69484, 0.68418, 0.67318, 0.66201, 0.65086, 0.63994,\
        0.62944, 0.61947, 0.60999, 0.60082, 0.59194, 0.58363, 0.57579, 0.56825, 0.56092, 0.55377, 0.54673,\
        0.53979, 0.53289, 0.52603, 0.51919, 0.51239, 0.50574, 0.49921, 0.49220, 0.48510, 0.47778, 0.47017],\
        [0.27946, 0.28455, 0.28935, 0.29399, 0.29871, 0.30406, 0.30987, 0.31590, 0.32212, 0.32850, 0.33505,\
        0.34179, 0.34872, 0.35585, 0.36320, 0.37078, 0.37859, 0.38663, 0.39485, 0.40291, 0.41078, 0.41945,\
        0.42900, 0.43941, 0.45066, 0.46273, 0.47562, 0.48925, 0.50323, 0.51720, 0.53134, 0.54540, 0.55889,\
        0.57131, 0.58241, 0.59267, 0.60348, 0.61608, 0.63143, 0.65024, 0.67224, 0.69808, 0.73152, 0.76559,\
        0.79119, 0.80003, 0.79090, 0.76791, 0.73883, 0.71254, 0.70113, 0.72963, 0.77459, 0.82079, 0.86214,\
        0.89482, 0.91590, 0.92983, 0.93882, 0.94431, 0.94723, 0.94810, 0.94730, 0.94513, 0.94183, 0.93762,\
        0.93269, 0.92722, 0.92139, 0.91539, 0.90922, 0.90251, 0.89562, 0.88766, 0.87859, 0.86746, 0.85475,\
        0.84115, 0.82722, 0.81358, 0.80117, 0.79074, 0.78121, 0.77263, 0.76473, 0.75724, 0.74997, 0.74271,\
        0.73530, 0.72755, 0.71932, 0.71053, 0.70123, 0.69132, 0.68047, 0.66909, 0.65751, 0.64602, 0.63491,\
        0.62443, 0.61470, 0.60566, 0.59700, 0.58834, 0.58016, 0.57246, 0.56512, 0.55803, 0.55113, 0.54435,\
        0.53765, 0.53099, 0.52434, 0.51767, 0.51095, 0.50421, 0.49773, 0.49121, 0.48434, 0.47717, 0.46970],\
        [0.28048, 0.28557, 0.29033, 0.29493, 0.29971, 0.30499, 0.31075, 0.31678, 0.32299, 0.32938, 0.33595,\
        0.34271, 0.34968, 0.35689, 0.36434, 0.37205, 0.38003, 0.38823, 0.39651, 0.40463, 0.41268, 0.42121,\
        0.43052, 0.44068, 0.45171, 0.46359, 0.47629, 0.48965, 0.50288, 0.51516, 0.52740, 0.53953, 0.55117,\
        0.56196, 0.57180, 0.58109, 0.59074, 0.60192, 0.61570, 0.63374, 0.65513, 0.68076, 0.71468, 0.74712,\
        0.77462, 0.79417, 0.78847, 0.77214, 0.74984, 0.72981, 0.72641, 0.75209, 0.79533, 0.83736, 0.87539,\
        0.90581, 0.92452, 0.93619, 0.94326, 0.94699, 0.94824, 0.94761, 0.94552, 0.94225, 0.93801, 0.93300,\
        0.92735, 0.92121, 0.91471, 0.90816, 0.90174, 0.89486, 0.88719, 0.87870, 0.86902, 0.85789, 0.84552,\
        0.83239, 0.81910, 0.80639, 0.79486, 0.78518, 0.77659, 0.76870, 0.76133, 0.75428, 0.74735, 0.74037,\
        0.73313, 0.72545, 0.71709, 0.70785, 0.69795, 0.68762, 0.67647, 0.66471, 0.65273, 0.64086, 0.62945,\
        0.61885, 0.60933, 0.60055, 0.59220, 0.58410, 0.57633, 0.56895, 0.56190, 0.55510, 0.54848, 0.54198,\
        0.53554, 0.52914, 0.52274, 0.51630, 0.50979, 0.50319, 0.49674, 0.49036, 0.48364, 0.47659, 0.46923],\
        [0.28182, 0.28702, 0.29187, 0.29646, 0.30124, 0.30649, 0.31210, 0.31799, 0.32410, 0.33040, 0.33691,\
        0.34363, 0.35058, 0.35777, 0.36523, 0.37296, 0.38095, 0.38916, 0.39744, 0.40555, 0.41368, 0.42220,\
        0.43138, 0.44133, 0.45207, 0.46357, 0.47570, 0.48817, 0.50031, 0.51172, 0.52280, 0.53357, 0.54382,\
        0.55333, 0.56205, 0.57027, 0.57860, 0.58782, 0.59937, 0.61618, 0.63503, 0.65771, 0.68685, 0.70974,\
        0.73040, 0.75433, 0.76870, 0.76878, 0.75885, 0.74881, 0.75100, 0.77281, 0.80975, 0.84874, 0.88282,\
        0.90980, 0.92814, 0.93918, 0.94512, 0.94755, 0.94752, 0.94572, 0.94258, 0.93842, 0.93345, 0.92780,\
        0.92160, 0.91498, 0.90791, 0.90071, 0.89394, 0.88676, 0.87876, 0.86986, 0.85991, 0.84885, 0.83689,\
        0.82441, 0.81188, 0.80011, 0.78984, 0.78075, 0.77266, 0.76523, 0.75824, 0.75150, 0.74483, 0.73804,\
        0.73097, 0.72340, 0.71510, 0.70579, 0.69541, 0.68429, 0.67248, 0.66020, 0.64777, 0.63550, 0.62371,\
        0.61279, 0.60334, 0.59505, 0.58710, 0.57953, 0.57226, 0.56530, 0.55861, 0.55214, 0.54583, 0.53962,\
        0.53348, 0.52736, 0.52122, 0.51507, 0.50891, 0.50279, 0.49643, 0.48982, 0.48306, 0.47605, 0.46875],\
        [0.28336, 0.28878, 0.29389, 0.29863, 0.30328, 0.30836, 0.31376, 0.31943, 0.32536, 0.33151, 0.33789,\
        0.34450, 0.35135, 0.35845, 0.36582, 0.37345, 0.38133, 0.38942, 0.39761, 0.40570, 0.41385, 0.42239,\
        0.43149, 0.44125, 0.45166, 0.46266, 0.47405, 0.48554, 0.49672, 0.50752, 0.51790, 0.52783, 0.53719,\
        0.54581, 0.55366, 0.56092, 0.56798, 0.57561, 0.58520, 0.59765, 0.60920, 0.62024, 0.63599, 0.65338,\
        0.67132, 0.69943, 0.73392, 0.75446, 0.75943, 0.75958, 0.76648, 0.78665, 0.82017, 0.85806, 0.88930,\
        0.91281, 0.92972, 0.93967, 0.94453, 0.94591, 0.94490, 0.94223, 0.93835, 0.93358, 0.92812, 0.92206,\
        0.91549, 0.90866, 0.90162, 0.89396, 0.88656, 0.87889, 0.87056, 0.86143, 0.85142, 0.84056, 0.82906,\
        0.81738, 0.80602, 0.79516, 0.78560, 0.77704, 0.76929, 0.76213, 0.75536, 0.74881, 0.74228, 0.73560,\
        0.72860, 0.72111, 0.71298, 0.70377, 0.69294, 0.68094, 0.66838, 0.65556, 0.64274, 0.63017, 0.61809,\
        0.60687, 0.59721, 0.58934, 0.58199, 0.57493, 0.56815, 0.56162, 0.55531, 0.54919, 0.54321, 0.53732,\
        0.53147, 0.52563, 0.51976, 0.51386, 0.50793, 0.50191, 0.49570, 0.48921, 0.48249, 0.47552, 0.46827],\
        [0.28494, 0.29057, 0.29595, 0.30099, 0.30557, 0.31037, 0.31550, 0.32094, 0.32665, 0.33261, 0.33882,\
        0.34527, 0.35197, 0.35891, 0.36610, 0.37354, 0.38121, 0.38907, 0.39707, 0.40510, 0.41329, 0.42185,\
        0.43091, 0.44049, 0.45059, 0.46110, 0.47185, 0.48259, 0.49313, 0.50336, 0.51325, 0.52272, 0.53161,\
        0.53976, 0.54706, 0.55357, 0.55950, 0.56523, 0.57110, 0.57640, 0.57871, 0.57827, 0.58181, 0.59252,\
        0.60685, 0.62886, 0.67570, 0.72325, 0.74674, 0.75897, 0.77305, 0.79618, 0.83036, 0.86839, 0.90039,\
        0.91846, 0.93096, 0.93844, 0.94181, 0.94217, 0.94042, 0.93717, 0.93283, 0.92771, 0.92205, 0.91593,\
        0.90913, 0.90206, 0.89503, 0.88744, 0.87957, 0.87142, 0.86279, 0.85355, 0.84365, 0.83311, 0.82208,\
        0.81099, 0.80088, 0.79115, 0.78203, 0.77378, 0.76622, 0.75920, 0.75253, 0.74604, 0.73954, 0.73286,\
        0.72579, 0.71816, 0.70992, 0.70092, 0.68950, 0.67690, 0.66382, 0.65064, 0.63761, 0.62500, 0.61302,\
        0.60187, 0.59241, 0.58444, 0.57729, 0.57058, 0.56418, 0.55803, 0.55209, 0.54631, 0.54065, 0.53507,\
        0.52951, 0.52395, 0.51833, 0.51265, 0.50685, 0.50093, 0.49484, 0.48850, 0.48188, 0.47497, 0.46777],\
        [0.28640, 0.29213, 0.29754, 0.30263, 0.30739, 0.31215, 0.31712, 0.32236, 0.32787, 0.33363, 0.33965,\
        0.34591, 0.35241, 0.35914, 0.36611, 0.37330, 0.38070, 0.38826, 0.39600, 0.40391, 0.41214, 0.42075,\
        0.42977, 0.43922, 0.44906, 0.45918, 0.46945, 0.47968, 0.48973, 0.49957, 0.50920, 0.51856, 0.52739,\
        0.53545, 0.54260, 0.54875, 0.55383, 0.55771, 0.55990, 0.55912, 0.55371, 0.54429, 0.53642, 0.53553,\
        0.53758, 0.54634, 0.59658, 0.66999, 0.71967, 0.74889, 0.77439, 0.80402, 0.83968, 0.87547, 0.90361,\
        0.92050, 0.93001, 0.93516, 0.93705, 0.93651, 0.93422, 0.93067, 0.92616, 0.92091, 0.91525, 0.90953,\
        0.90323, 0.89572, 0.88846, 0.88078, 0.87269, 0.86425, 0.85541, 0.84616, 0.83652, 0.82652, 0.81619,\
        0.80552, 0.79577, 0.78714, 0.77865, 0.77065, 0.76321, 0.75623, 0.74955, 0.74302, 0.73646, 0.72968,\
        0.72246, 0.71446, 0.70535, 0.69566, 0.68447, 0.67182, 0.65860, 0.64530, 0.63228, 0.61986, 0.60838,\
        0.59788, 0.58836, 0.58025, 0.57311, 0.56657, 0.56045, 0.55461, 0.54899, 0.54353, 0.53818, 0.53289,\
        0.52762, 0.52232, 0.51695, 0.51147, 0.50582, 0.49998, 0.49402, 0.48779, 0.48124, 0.47439, 0.46723],\
        [0.28766, 0.29348, 0.29883, 0.30394, 0.30880, 0.31358, 0.31848, 0.32358, 0.32892, 0.33451, 0.34033,\
        0.34639, 0.35268, 0.35919, 0.36591, 0.37284, 0.37995, 0.38722, 0.39466, 0.40242, 0.41068, 0.41930,\
        0.42828, 0.43761, 0.44726, 0.45713, 0.46711, 0.47704, 0.48681, 0.49640, 0.50600, 0.51558, 0.52469,\
        0.53307, 0.54052, 0.54688, 0.55186, 0.55507, 0.55570, 0.55251, 0.54395, 0.52874, 0.50978, 0.50150,\
        0.49966, 0.49606, 0.54830, 0.62277, 0.68710, 0.73963, 0.78100, 0.81585, 0.85098, 0.88184, 0.90480,\
        0.91840, 0.92593, 0.92956, 0.93036, 0.92914, 0.92656, 0.92302, 0.91865, 0.91346, 0.90768, 0.90242,\
        0.89681, 0.88970, 0.88205, 0.87414, 0.86587, 0.85725, 0.84830, 0.83911, 0.82980, 0.82049, 0.81120,\
        0.80221, 0.79272, 0.78367, 0.77527, 0.76739, 0.75999, 0.75298, 0.74624, 0.73959, 0.73287, 0.72591,\
        0.71855, 0.71044, 0.70094, 0.69020, 0.67856, 0.66589, 0.65271, 0.63951, 0.62666, 0.61448, 0.60328,\
        0.59335, 0.58432, 0.57633, 0.56927, 0.56288, 0.55696, 0.55138, 0.54604, 0.54087, 0.53580, 0.53080,\
        0.52579, 0.52074, 0.51560, 0.51033, 0.50492, 0.49927, 0.49332, 0.48711, 0.48059, 0.47377, 0.46664],\
        [0.28864, 0.29462, 0.30019, 0.30511, 0.30989, 0.31467, 0.31953, 0.32455, 0.32976, 0.33519, 0.34084,\
        0.34671, 0.35279, 0.35908, 0.36556, 0.37224, 0.37911, 0.38616, 0.39343, 0.40104, 0.40920, 0.41770,\
        0.42658, 0.43581, 0.44534, 0.45509, 0.46495, 0.47482, 0.48458, 0.49422, 0.50403, 0.51397, 0.52354,\
        0.53254, 0.54080, 0.54809, 0.55410, 0.55842, 0.56040, 0.55931, 0.55457, 0.54377, 0.52407, 0.51313,\
        0.51267, 0.52139, 0.55889, 0.62044, 0.67850, 0.75668, 0.80868, 0.84143, 0.86739, 0.88886, 0.90435,\
        0.91375, 0.91956, 0.92218, 0.92220, 0.92047, 0.91780, 0.91456, 0.91069, 0.90592, 0.90024, 0.89478,\
        0.88939, 0.88280, 0.87536, 0.86742, 0.85906, 0.85034, 0.84133, 0.83216, 0.82310, 0.81446, 0.80602,\
        0.79737, 0.78865, 0.77989, 0.77157, 0.76374, 0.75635, 0.74929, 0.74243, 0.73561, 0.72865, 0.72137,\
        0.71370, 0.70569, 0.69601, 0.68452, 0.67223, 0.65939, 0.64629, 0.63332, 0.62080, 0.60897, 0.59817,\
        0.58865, 0.58013, 0.57248, 0.56562, 0.55941, 0.55369, 0.54834, 0.54324, 0.53832, 0.53352, 0.52878,\
        0.52402, 0.51921, 0.51428, 0.50918, 0.50393, 0.49848, 0.49258, 0.48639, 0.47991, 0.47310, 0.46599],\
        [0.28921, 0.29513, 0.30066, 0.30573, 0.31059, 0.31540, 0.32027, 0.32524, 0.33037, 0.33568, 0.34117,\
        0.34687, 0.35275, 0.35883, 0.36510, 0.37156, 0.37822, 0.38512, 0.39233, 0.39996, 0.40772, 0.41598,\
        0.42475, 0.43392, 0.44339, 0.45311, 0.46300, 0.47299, 0.48304, 0.49311, 0.50327, 0.51350, 0.52360,\
        0.53347, 0.54299, 0.55195, 0.56008, 0.56722, 0.57324, 0.57767, 0.58206, 0.58746, 0.58121, 0.57742,\
        0.57774, 0.59222, 0.62294, 0.67841, 0.75747, 0.82346, 0.85605, 0.87399, 0.88611, 0.89489, 0.90109,\
        0.90768, 0.91241, 0.91422, 0.91341, 0.91111, 0.90841, 0.90562, 0.90247, 0.89830, 0.89320, 0.88775,\
        0.88207, 0.87564, 0.86840, 0.86054, 0.85220, 0.84348, 0.83444, 0.82522, 0.81611, 0.80787, 0.80037,\
        0.79211, 0.78367, 0.77530, 0.76721, 0.75948, 0.75210, 0.74499, 0.73800, 0.73098, 0.72374, 0.71607,\
        0.70765, 0.69883, 0.68920, 0.67765, 0.66521, 0.65236, 0.63942, 0.62677, 0.61479, 0.60358, 0.59331,\
        0.58419, 0.57603, 0.56871, 0.56210, 0.55611, 0.55059, 0.54545, 0.54057, 0.53589, 0.53132, 0.52682,\
        0.52230, 0.51772, 0.51300, 0.50804, 0.50276, 0.49734, 0.49170, 0.48561, 0.47916, 0.47237, 0.46526],\
        [0.28944, 0.29521, 0.30067, 0.30590, 0.31091, 0.31582, 0.32073, 0.32569, 0.33076, 0.33597, 0.34134,\
        0.34688, 0.35259, 0.35849, 0.36457, 0.37083, 0.37729, 0.38398, 0.39096, 0.39822, 0.40586, 0.41412,\
        0.42286, 0.43198, 0.44143, 0.45116, 0.46114, 0.47135, 0.48177, 0.49239, 0.50301, 0.51354, 0.52421,\
        0.53511, 0.54623, 0.55742, 0.56841, 0.57921, 0.59119, 0.60362, 0.61854, 0.63481, 0.64117, 0.64369,\
        0.65043, 0.67789, 0.72764, 0.78979, 0.85661, 0.88912, 0.89975, 0.90264, 0.90246, 0.90090, 0.89997,\
        0.90298, 0.90630, 0.90692, 0.90498, 0.90191, 0.89918, 0.89675, 0.89399, 0.89034, 0.88577, 0.88057,\
        0.87483, 0.86842, 0.86128, 0.85352, 0.84525, 0.83660, 0.82767, 0.81849, 0.80914, 0.80035, 0.79349,\
        0.78591, 0.77793, 0.76993, 0.76209, 0.75449, 0.74714, 0.73998, 0.73286, 0.72563, 0.71813, 0.71014,\
        0.70145, 0.69164, 0.68127, 0.66981, 0.65753, 0.64488, 0.63221, 0.61991, 0.60847, 0.59820, 0.58852,\
        0.57982, 0.57203, 0.56503, 0.55869, 0.55293, 0.54762, 0.54268, 0.53801, 0.53354, 0.52918, 0.52489,\
        0.52060, 0.51624, 0.51176, 0.50704, 0.50200, 0.49660, 0.49088, 0.48479, 0.47833, 0.47154, 0.46443],\
        [0.28952, 0.29521, 0.30066, 0.30591, 0.31102, 0.31602, 0.32096, 0.32593, 0.33095, 0.33609, 0.34135,\
        0.34676, 0.35233, 0.35806, 0.36397, 0.37006, 0.37634, 0.38282, 0.38951, 0.39645, 0.40402, 0.41229,\
        0.42100, 0.43007, 0.43947, 0.44919, 0.45922, 0.46958, 0.48028, 0.49130, 0.50236, 0.51321, 0.52453,\
        0.53651, 0.54927, 0.56289, 0.57723, 0.59155, 0.60840, 0.62981, 0.65427, 0.67733, 0.68881, 0.69510,\
        0.70231, 0.74450, 0.80559, 0.85436, 0.89630, 0.91802, 0.92231, 0.91861, 0.91217, 0.90547, 0.90067,\
        0.90045, 0.90178, 0.90050, 0.89747, 0.89396, 0.89099, 0.88839, 0.88561, 0.88222, 0.87804, 0.87311,\
        0.86747, 0.86109, 0.85399, 0.84628, 0.83811, 0.82960, 0.82089, 0.81209, 0.80314, 0.79436, 0.78668,\
        0.77923, 0.77158, 0.76386, 0.75621, 0.74872, 0.74141, 0.73421, 0.72698, 0.71950, 0.71169, 0.70337,\
        0.69415, 0.68399, 0.67308, 0.66153, 0.64946, 0.63717, 0.62493, 0.61299, 0.60198, 0.59233, 0.58356,\
        0.57546, 0.56810, 0.56143, 0.55538, 0.54985, 0.54475, 0.54001, 0.53553, 0.53124, 0.52708, 0.52297,\
        0.51886, 0.51471, 0.51045, 0.50602, 0.50124, 0.49587, 0.49003, 0.48386, 0.47738, 0.47058, 0.46347],\
        [0.28954, 0.29515, 0.30054, 0.30586, 0.31103, 0.31607, 0.32104, 0.32600, 0.33099, 0.33606, 0.34123,\
        0.34652, 0.35196, 0.35756, 0.36333, 0.36928, 0.37543, 0.38178, 0.38835, 0.39521, 0.40262, 0.41076,\
        0.41928, 0.42819, 0.43747, 0.44712, 0.45713, 0.46751, 0.47824, 0.48927, 0.50042, 0.51192, 0.52393,\
        0.53680, 0.55083, 0.56626, 0.58345, 0.60212, 0.62141, 0.64676, 0.67600, 0.70295, 0.71779, 0.73097,\
        0.75397, 0.80144, 0.84508, 0.87862, 0.90531, 0.92033, 0.92365, 0.91971, 0.91265, 0.90530, 0.89979,\
        0.89738, 0.89620, 0.89346, 0.88988, 0.88621, 0.88296, 0.88010, 0.87723, 0.87396, 0.87005, 0.86536,\
        0.85986, 0.85352, 0.84643, 0.83874, 0.83063, 0.82225, 0.81375, 0.80532, 0.79692, 0.78823, 0.78010,\
        0.77239, 0.76477, 0.75717, 0.74962, 0.74218, 0.73488, 0.72765, 0.72035, 0.71269, 0.70427, 0.69567,\
        0.68603, 0.67559, 0.66452, 0.65298, 0.64120, 0.62941, 0.61789, 0.60672, 0.59613, 0.58708, 0.57882,\
        0.57122, 0.56426, 0.55793, 0.55214, 0.54685, 0.54196, 0.53741, 0.53311, 0.52899, 0.52498, 0.52103,\
        0.51706, 0.51303, 0.50892, 0.50466, 0.50005, 0.49472, 0.48890, 0.48274, 0.47625, 0.46946, 0.46237],\
        [0.28959, 0.29519, 0.30057, 0.30586, 0.31102, 0.31605, 0.32101, 0.32594, 0.33089, 0.33590, 0.34099,\
        0.34619, 0.35152, 0.35700, 0.36265, 0.36850, 0.37455, 0.38085, 0.38744, 0.39443, 0.40182, 0.40945,\
        0.41763, 0.42631, 0.43542, 0.44493, 0.45483, 0.46510, 0.47574, 0.48673, 0.49808, 0.50976, 0.52210,\
        0.53551, 0.55020, 0.56624, 0.58348, 0.60208, 0.62506, 0.65091, 0.67805, 0.70228, 0.72354, 0.74498,\
        0.77375, 0.80998, 0.84254, 0.86987, 0.89090, 0.90396, 0.90900, 0.90804, 0.90372, 0.89834, 0.89364,\
        0.89035, 0.88767, 0.88452, 0.88094, 0.87738, 0.87416, 0.87126, 0.86841, 0.86531, 0.86167, 0.85725,\
        0.85191, 0.84562, 0.83847, 0.83072, 0.82265, 0.81439, 0.80595, 0.79764, 0.78939, 0.78107, 0.77296,\
        0.76514, 0.75749, 0.74991, 0.74236, 0.73489, 0.72754, 0.72026, 0.71289, 0.70545, 0.69666, 0.68723,\
        0.67731, 0.66673, 0.65565, 0.64426, 0.63283, 0.62163, 0.61090, 0.60111, 0.59137, 0.58239, 0.57439,\
        0.56715, 0.56054, 0.55451, 0.54899, 0.54392, 0.53923, 0.53486, 0.53072, 0.52675, 0.52289, 0.51905,\
        0.51517, 0.51120, 0.50710, 0.50282, 0.49825, 0.49310, 0.48743, 0.48135, 0.47492, 0.46817, 0.46110],\
        [0.28965, 0.29530, 0.30072, 0.30591, 0.31098, 0.31596, 0.32089, 0.32579, 0.33070, 0.33564, 0.34065,\
        0.34576, 0.35099, 0.35637, 0.36192, 0.36767, 0.37365, 0.37990, 0.38645, 0.39335, 0.40047, 0.40792,\
        0.41591, 0.42438, 0.43330, 0.44264, 0.45237, 0.46245, 0.47288, 0.48369, 0.49491, 0.50653, 0.51900,\
        0.53267, 0.54759, 0.56376, 0.58124, 0.60075, 0.62205, 0.64561, 0.66947, 0.69214, 0.71412, 0.73816,\
        0.76441, 0.79003, 0.81611, 0.84160, 0.86130, 0.87537, 0.88376, 0.88720, 0.88707, 0.88492, 0.88207,\
        0.87926, 0.87644, 0.87338, 0.87014, 0.86697, 0.86405, 0.86137, 0.85876, 0.85597, 0.85269, 0.84862,\
        0.84352, 0.83727, 0.82999, 0.82206, 0.81395, 0.80594, 0.79759, 0.78929, 0.78114, 0.77303, 0.76506,\
        0.75729, 0.74967, 0.74210, 0.73450, 0.72689, 0.71941, 0.71210, 0.70448, 0.69661, 0.68811, 0.67838,\
        0.66819, 0.65759, 0.64663, 0.63548, 0.62443, 0.61390, 0.60389, 0.59456, 0.58594, 0.57768, 0.57010,\
        0.56320, 0.55691, 0.55117, 0.54590, 0.54105, 0.53655, 0.53234, 0.52836, 0.52453, 0.52078, 0.51705,\
        0.51325, 0.50930, 0.50514, 0.50078, 0.49621, 0.49121, 0.48570, 0.47974, 0.47339, 0.46669, 0.45966],\
        [0.28965, 0.29529, 0.30069, 0.30584, 0.31086, 0.31580, 0.32070, 0.32556, 0.33041, 0.33529, 0.34023,\
        0.34524, 0.35038, 0.35566, 0.36111, 0.36677, 0.37266, 0.37883, 0.38528, 0.39197, 0.39883, 0.40629,\
        0.41411, 0.42237, 0.43111, 0.44028, 0.44983, 0.45970, 0.46985, 0.48029, 0.49109, 0.50250, 0.51496,\
        0.52870, 0.54359, 0.55952, 0.57658, 0.59470, 0.61426, 0.63547, 0.65679, 0.67735, 0.69796, 0.72104,\
        0.74355, 0.76483, 0.78666, 0.80752, 0.82557, 0.84121, 0.85321, 0.86118, 0.86541, 0.86673, 0.86622,\
        0.86468, 0.86253, 0.86002, 0.85736, 0.85477, 0.85238, 0.85015, 0.84795, 0.84561, 0.84285, 0.83929,\
        0.83456, 0.82840, 0.82090, 0.81265, 0.80436, 0.79640, 0.78841, 0.78025, 0.77220, 0.76425, 0.75643,\
        0.74878, 0.74129, 0.73382, 0.72618, 0.71830, 0.71050, 0.70316, 0.69554, 0.68720, 0.67839, 0.66882,\
        0.65873, 0.64831, 0.63766, 0.62688, 0.61612, 0.60607, 0.59745, 0.58863, 0.58051, 0.57290, 0.56582,\
        0.55932, 0.55336, 0.54789, 0.54286, 0.53821, 0.53390, 0.52985, 0.52600, 0.52230, 0.51867, 0.51504,\
        0.51133, 0.50744, 0.50324, 0.49884, 0.49427, 0.48930, 0.48386, 0.47798, 0.47170, 0.46505, 0.45807],\
        [0.28957, 0.29514, 0.30050, 0.30562, 0.31064, 0.31558, 0.32045, 0.32527, 0.33007, 0.33488, 0.33973,\
        0.34466, 0.34969, 0.35487, 0.36021, 0.36577, 0.37156, 0.37762, 0.38399, 0.39068, 0.39777, 0.40481,\
        0.41217, 0.42025, 0.42888, 0.43793, 0.44734, 0.45703, 0.46695, 0.47705, 0.48739, 0.49832, 0.51060,\
        0.52431, 0.53891, 0.55418, 0.57007, 0.58665, 0.60456, 0.62348, 0.64282, 0.66210, 0.68175, 0.70209,\
        0.72072, 0.73999, 0.75878, 0.77592, 0.79201, 0.80781, 0.82213, 0.83370, 0.84148, 0.84576, 0.84745,\
        0.84740, 0.84626, 0.84455, 0.84263, 0.84079, 0.83910, 0.83745, 0.83573, 0.83394, 0.83184, 0.82894,\
        0.82477, 0.81891, 0.81112, 0.80249, 0.79412, 0.78612, 0.77826, 0.77038, 0.76254, 0.75477, 0.74710,\
        0.73960, 0.73231, 0.72510, 0.71761, 0.70955, 0.70101, 0.69353, 0.68573, 0.67732, 0.66837, 0.65890,\
        0.64905, 0.63900, 0.62889, 0.61874, 0.60859, 0.59869, 0.59051, 0.58275, 0.57526, 0.56820, 0.56160,\
        0.55549, 0.54985, 0.54466, 0.53986, 0.53541, 0.53126, 0.52736, 0.52364, 0.52005, 0.51652, 0.51300,\
        0.50941, 0.50567, 0.50166, 0.49719, 0.49246, 0.48741, 0.48196, 0.47611, 0.46987, 0.46327, 0.45633],\
        [0.28946, 0.29490, 0.30013, 0.30531, 0.31038, 0.31532, 0.32017, 0.32494, 0.32968, 0.33441, 0.33917,\
        0.34400, 0.34893, 0.35400, 0.35923, 0.36466, 0.37033, 0.37626, 0.38247, 0.38898, 0.39568, 0.40241,\
        0.40993, 0.41805, 0.42665, 0.43565, 0.44500, 0.45462, 0.46444, 0.47440, 0.48452, 0.49499, 0.50697,\
        0.52043, 0.53437, 0.54869, 0.56340, 0.57861, 0.59451, 0.61127, 0.62912, 0.64719, 0.66516, 0.68244,\
        0.69926, 0.71737, 0.73413, 0.74975, 0.76474, 0.77962, 0.79416, 0.80756, 0.81726, 0.82347, 0.82688,\
        0.82817, 0.82809, 0.82727, 0.82617, 0.82514, 0.82430, 0.82330, 0.82194, 0.82066, 0.81933, 0.81710,\
        0.81356, 0.80851, 0.80048, 0.79163, 0.78316, 0.77512, 0.76737, 0.75976, 0.75221, 0.74467, 0.73713,\
        0.72972, 0.72267, 0.71588, 0.70876, 0.70087, 0.69227, 0.68397, 0.67577, 0.66721, 0.65824, 0.64890,\
        0.63933, 0.62972, 0.62028, 0.61106, 0.60189, 0.59288, 0.58466, 0.57722, 0.57023, 0.56364, 0.55748,\
        0.55173, 0.54641, 0.54147, 0.53689, 0.53263, 0.52864, 0.52487, 0.52127, 0.51777, 0.51433, 0.51089,\
        0.50739, 0.50375, 0.49989, 0.49547, 0.49062, 0.48547, 0.47999, 0.47414, 0.46792, 0.46136, 0.45445],\
        [0.28944, 0.29479, 0.29993, 0.30511, 0.31019, 0.31510, 0.31989, 0.32460, 0.32925, 0.33389, 0.33856,\
        0.34328, 0.34810, 0.35305, 0.35815, 0.36345, 0.36898, 0.37474, 0.38077, 0.38703, 0.39345, 0.40023,\
        0.40784, 0.41592, 0.42448, 0.43348, 0.44285, 0.45252, 0.46241, 0.47250, 0.48285, 0.49376, 0.50537,\
        0.51757, 0.53043, 0.54367, 0.55718, 0.57100, 0.58522, 0.60031, 0.61678, 0.63344, 0.64982, 0.66577,\
        0.68179, 0.69792, 0.71293, 0.72770, 0.74183, 0.75598, 0.77007, 0.78306, 0.79322, 0.80057, 0.80513,\
        0.80758, 0.80851, 0.80863, 0.80835, 0.80809, 0.80817, 0.80792, 0.80654, 0.80559, 0.80501, 0.80324,\
        0.79974, 0.79542, 0.78803, 0.77935, 0.77100, 0.76313, 0.75566, 0.74844, 0.74129, 0.73407, 0.72669,\
        0.71920, 0.71222, 0.70596, 0.69918, 0.69136, 0.68291, 0.67437, 0.66584, 0.65714, 0.64818, 0.63900,\
        0.62971, 0.62050, 0.61169, 0.60343, 0.59515, 0.58690, 0.57916, 0.57205, 0.56544, 0.55926, 0.55347,\
        0.54806, 0.54302, 0.53834, 0.53396, 0.52988, 0.52603, 0.52238, 0.51888, 0.51546, 0.51208, 0.50870,\
        0.50525, 0.50160, 0.49768, 0.49335, 0.48857, 0.48341, 0.47791, 0.47206, 0.46586, 0.45932, 0.45244],\
        [0.28957, 0.29495, 0.30010, 0.30518, 0.31013, 0.31494, 0.31964, 0.32424, 0.32880, 0.33334, 0.33789,\
        0.34251, 0.34721, 0.35203, 0.35700, 0.36216, 0.36753, 0.37314, 0.37901, 0.38514, 0.39164, 0.39867,\
        0.40591, 0.41381, 0.42235, 0.43140, 0.44086, 0.45066, 0.46075, 0.47109, 0.48170, 0.49263, 0.50376,\
        0.51518, 0.52710, 0.53931, 0.55176, 0.56447, 0.57763, 0.59157, 0.60638, 0.62148, 0.63653, 0.65137,\
        0.66613, 0.68065, 0.69493, 0.70882, 0.72171, 0.73506, 0.74820, 0.76003, 0.76965, 0.77706, 0.78243,\
        0.78599, 0.78807, 0.78921, 0.78982, 0.79029, 0.79093, 0.79108, 0.79001, 0.78914, 0.78845, 0.78666,\
        0.78337, 0.77881, 0.77244, 0.76490, 0.75726, 0.74998, 0.74307, 0.73642, 0.72986, 0.72312, 0.71608,\
        0.70854, 0.70102, 0.69516, 0.68854, 0.68101, 0.67287, 0.66444, 0.65588, 0.64719, 0.63833, 0.62938,\
        0.62042, 0.61155, 0.60305, 0.59561, 0.58819, 0.58080, 0.57371, 0.56706, 0.56085, 0.55503, 0.54959,\
        0.54448, 0.53971, 0.53525, 0.53107, 0.52714, 0.52342, 0.51988, 0.51648, 0.51314, 0.50980, 0.50640,\
        0.50302, 0.49942, 0.49544, 0.49109, 0.48634, 0.48120, 0.47571, 0.46987, 0.46368, 0.45716, 0.45031],\
        [0.28980, 0.29527, 0.30044, 0.30538, 0.31016, 0.31481, 0.31938, 0.32387, 0.32831, 0.33274, 0.33719,\
        0.34168, 0.34626, 0.35095, 0.35579, 0.36081, 0.36603, 0.37149, 0.37721, 0.38324, 0.38961, 0.39635,\
        0.40359, 0.41163, 0.42024, 0.42935, 0.43893, 0.44891, 0.45924, 0.46985, 0.48063, 0.49131, 0.50186,\
        0.51307, 0.52426, 0.53557, 0.54710, 0.55894, 0.57122, 0.58410, 0.59751, 0.61123, 0.62505, 0.63871,\
        0.65226, 0.66573, 0.67905, 0.69159, 0.70372, 0.71642, 0.72851, 0.73894, 0.74707, 0.75383, 0.75965,\
        0.76423, 0.76742, 0.76955, 0.77100, 0.77209, 0.77289, 0.77308, 0.77248, 0.77155, 0.77029, 0.76820,\
        0.76501, 0.76066, 0.75511, 0.74871, 0.74213, 0.73577, 0.72965, 0.72374, 0.71801, 0.71183, 0.70525,\
        0.69838, 0.69078, 0.68420, 0.67755, 0.67029, 0.66249, 0.65433, 0.64596, 0.63743, 0.62879, 0.62014,\
        0.61164, 0.60341, 0.59539, 0.58824, 0.58148, 0.57482, 0.56836, 0.56222, 0.55642, 0.55096, 0.54583,\
        0.54101, 0.53648, 0.53222, 0.52821, 0.52442, 0.52082, 0.51738, 0.51406, 0.51081, 0.50754, 0.50410,\
        0.50053, 0.49709, 0.49312, 0.48875, 0.48400, 0.47888, 0.47339, 0.46756, 0.46139, 0.45489, 0.44806],\
        [0.28999, 0.29558, 0.30076, 0.30553, 0.31013, 0.31464, 0.31908, 0.32345, 0.32779, 0.33210, 0.33643,\
        0.34081, 0.34526, 0.34982, 0.35453, 0.35941, 0.36449, 0.36981, 0.37540, 0.38131, 0.38756, 0.39422,\
        0.40150, 0.40954, 0.41812, 0.42726, 0.43692, 0.44706, 0.45762, 0.46851, 0.47963, 0.49077, 0.50149,\
        0.51158, 0.52174, 0.53221, 0.54297, 0.55407, 0.56557, 0.57753, 0.58986, 0.60246, 0.61517, 0.62750,\
        0.63987, 0.65238, 0.66473, 0.67667, 0.68850, 0.69994, 0.71164, 0.72016, 0.72577, 0.73146, 0.73780,\
        0.74328, 0.74725, 0.75007, 0.75217, 0.75369, 0.75459, 0.75479, 0.75429, 0.75318, 0.75144, 0.74899,\
        0.74582, 0.74187, 0.73707, 0.73165, 0.72613, 0.72090, 0.71576, 0.71041, 0.70584, 0.70018, 0.69387,\
        0.68726, 0.68029, 0.67346, 0.66666, 0.65957, 0.65208, 0.64426, 0.63621, 0.62799, 0.61967, 0.61138,\
        0.60334, 0.59576, 0.58850, 0.58170, 0.57531, 0.56917, 0.56323, 0.55755, 0.55214, 0.54703, 0.54219,\
        0.53763, 0.53332, 0.52924, 0.52539, 0.52173, 0.51823, 0.51487, 0.51162, 0.50847, 0.50535, 0.50213,\
        0.49849, 0.49475, 0.49074, 0.48634, 0.48158, 0.47645, 0.47097, 0.46515, 0.45899, 0.45250, 0.44569],\
        [0.28995, 0.29557, 0.30077, 0.30539, 0.30992, 0.31436, 0.31871, 0.32299, 0.32721, 0.33141, 0.33563,\
        0.33989, 0.34422, 0.34866, 0.35323, 0.35798, 0.36293, 0.36811, 0.37359, 0.37939, 0.38559, 0.39231,\
        0.39967, 0.40743, 0.41589, 0.42502, 0.43473, 0.44497, 0.45565, 0.46670, 0.47802, 0.48944, 0.50018,\
        0.50940, 0.51898, 0.52890, 0.53908, 0.54955, 0.56038, 0.57159, 0.58318, 0.59506, 0.60658, 0.61714,\
        0.62857, 0.64035, 0.65202, 0.66349, 0.67487, 0.68633, 0.69793, 0.70285, 0.70584, 0.71060, 0.71798,\
        0.72400, 0.72808, 0.73115, 0.73370, 0.73554, 0.73650, 0.73663, 0.73607, 0.73481, 0.73263, 0.72978,\
        0.72667, 0.72324, 0.71919, 0.71453, 0.70994, 0.70579, 0.70214, 0.69694, 0.69293, 0.68800, 0.68210,\
        0.67582, 0.66926, 0.66261, 0.65590, 0.64901, 0.64186, 0.63442, 0.62677, 0.61897, 0.61110, 0.60324,\
        0.59573, 0.58868, 0.58199, 0.57565, 0.56964, 0.56390, 0.55839, 0.55310, 0.54805, 0.54325, 0.53868,\
        0.53435, 0.53024, 0.52633, 0.52261, 0.51907, 0.51566, 0.51235, 0.50913, 0.50603, 0.50301, 0.49991,\
        0.49641, 0.49248, 0.48833, 0.48387, 0.47907, 0.47394, 0.46846, 0.46264, 0.45649, 0.45002, 0.44322],\
        [0.28965, 0.29505, 0.30001, 0.30489, 0.30954, 0.31398, 0.31827, 0.32246, 0.32658, 0.33068, 0.33478,\
        0.33892, 0.34314, 0.34746, 0.35191, 0.35652, 0.36134, 0.36641, 0.37176, 0.37745, 0.38356, 0.39016,\
        0.39727, 0.40501, 0.41345, 0.42254, 0.43225, 0.44251, 0.45320, 0.46422, 0.47535, 0.48622, 0.49624,\
        0.50598, 0.51575, 0.52543, 0.53517, 0.54511, 0.55533, 0.56586, 0.57666, 0.58752, 0.59733, 0.60705,\
        0.61831, 0.62949, 0.64042, 0.65117, 0.66182, 0.67227, 0.68107, 0.68593, 0.68879, 0.69366, 0.70098,\
        0.70666, 0.71002, 0.71303, 0.71601, 0.71810, 0.71901, 0.71896, 0.71833, 0.71721, 0.71463, 0.71125,\
        0.70835, 0.70551, 0.70229, 0.69825, 0.69448, 0.69117, 0.68805, 0.68411, 0.67998, 0.67540, 0.67007,\
        0.66423, 0.65806, 0.65172, 0.64527, 0.63870, 0.63194, 0.62495, 0.61773, 0.61044, 0.60315, 0.59585,\
        0.58883, 0.58222, 0.57596, 0.57002, 0.56437, 0.55899, 0.55382, 0.54888, 0.54414, 0.53962, 0.53530,\
        0.53118, 0.52724, 0.52348, 0.51988, 0.51644, 0.51312, 0.50987, 0.50663, 0.50345, 0.50047, 0.49735,\
        0.49388, 0.49001, 0.48582, 0.48132, 0.47649, 0.47134, 0.46586, 0.46004, 0.45390, 0.44743, 0.44064],\
        [0.28932, 0.29466, 0.29962, 0.30451, 0.30916, 0.31356, 0.31777, 0.32186, 0.32588, 0.32988, 0.33388,\
        0.33792, 0.34202, 0.34623, 0.35056, 0.35506, 0.35976, 0.36469, 0.36991, 0.37548, 0.38145, 0.38788,\
        0.39485, 0.40247, 0.41080, 0.41981, 0.42946, 0.43967, 0.45028, 0.46115, 0.47204, 0.48265, 0.49279,\
        0.50269, 0.51237, 0.52178, 0.53113, 0.54059, 0.55024, 0.56009, 0.57002, 0.57976, 0.58903, 0.59892,\
        0.60940, 0.61959, 0.62953, 0.63920, 0.64862, 0.65751, 0.66490, 0.67012, 0.67425, 0.67927, 0.68532,\
        0.69013, 0.69325, 0.69616, 0.69952, 0.70156, 0.70239, 0.70217, 0.70140, 0.70092, 0.69817, 0.69449,\
        0.69182, 0.68931, 0.68660, 0.68340, 0.68027, 0.67740, 0.67449, 0.67116, 0.66735, 0.66304, 0.65812,\
        0.65270, 0.64693, 0.64094, 0.63484, 0.62869, 0.62244, 0.61597, 0.60917, 0.60234, 0.59565, 0.58895,\
        0.58244, 0.57624, 0.57036, 0.56477, 0.55946, 0.55438, 0.54953, 0.54488, 0.54042, 0.53615, 0.53205,\
        0.52811, 0.52433, 0.52069, 0.51720, 0.51384, 0.51062, 0.50749, 0.50430, 0.50090, 0.49786, 0.49474,\
        0.49126, 0.48740, 0.48321, 0.47869, 0.47384, 0.46867, 0.46318, 0.45736, 0.45122, 0.44475, 0.43797],\
        [0.28902, 0.29451, 0.29960, 0.30436, 0.30884, 0.31310, 0.31720, 0.32119, 0.32512, 0.32902, 0.33293,\
        0.33687, 0.34088, 0.34498, 0.34921, 0.35359, 0.35817, 0.36298, 0.36807, 0.37349, 0.37930, 0.38558,\
        0.39240, 0.39992, 0.40795, 0.41682, 0.42639, 0.43650, 0.44699, 0.45769, 0.46841, 0.47899, 0.48930,\
        0.49932, 0.50882, 0.51785, 0.52680, 0.53585, 0.54502, 0.55428, 0.56355, 0.57274, 0.58193, 0.59148,\
        0.60099, 0.61020, 0.61908, 0.62744, 0.63553, 0.64325, 0.65003, 0.65559, 0.66043, 0.66534, 0.67034,\
        0.67461, 0.67794, 0.68095, 0.68384, 0.68581, 0.68667, 0.68661, 0.68603, 0.68518, 0.68299, 0.67997,\
        0.67726, 0.67481, 0.67231, 0.66965, 0.66699, 0.66439, 0.66170, 0.65867, 0.65517, 0.65113, 0.64654,\
        0.64148, 0.63608, 0.63046, 0.62474, 0.61904, 0.61339, 0.60760, 0.60129, 0.59484, 0.58861, 0.58245,\
        0.57642, 0.57063, 0.56510, 0.55984, 0.55483, 0.55004, 0.54547, 0.54108, 0.53687, 0.53282, 0.52891,\
        0.52514, 0.52150, 0.51796, 0.51454, 0.51125, 0.50811, 0.50513, 0.50225, 0.49902, 0.49561, 0.49224,\
        0.48865, 0.48475, 0.48053, 0.47599, 0.47112, 0.46594, 0.46043, 0.45460, 0.44845, 0.44199, 0.43522],\
        [0.28863, 0.29427, 0.29953, 0.30413, 0.30841, 0.31253, 0.31652, 0.32042, 0.32426, 0.32809, 0.33192,\
        0.33579, 0.33971, 0.34372, 0.34785, 0.35212, 0.35659, 0.36128, 0.36624, 0.37151, 0.37716, 0.38324,\
        0.38981, 0.39692, 0.40482, 0.41367, 0.42314, 0.43310, 0.44342, 0.45393, 0.46451, 0.47501, 0.48536,\
        0.49543, 0.50473, 0.51337, 0.52208, 0.53086, 0.53965, 0.54844, 0.55723, 0.56601, 0.57483, 0.58374,\
        0.59253, 0.60108, 0.60898, 0.61583, 0.62276, 0.62980, 0.63636, 0.64212, 0.64720, 0.65194, 0.65637,\
        0.66029, 0.66365, 0.66661, 0.66917, 0.67102, 0.67198, 0.67216, 0.67174, 0.67075, 0.66894, 0.66653,\
        0.66404, 0.66164, 0.65927, 0.65688, 0.65451, 0.65214, 0.64964, 0.64685, 0.64360, 0.63982, 0.63551,\
        0.63075, 0.62571, 0.62051, 0.61511, 0.60985, 0.60477, 0.59960, 0.59398, 0.58797, 0.58208, 0.57632,\
        0.57071, 0.56530, 0.56012, 0.55517, 0.55044, 0.54593, 0.54160, 0.53745, 0.53346, 0.52961, 0.52589,\
        0.52228, 0.51875, 0.51531, 0.51193, 0.50865, 0.50551, 0.50256, 0.49978, 0.49678, 0.49332, 0.48976,\
        0.48604, 0.48207, 0.47781, 0.47324, 0.46835, 0.46314, 0.45762, 0.45178, 0.44562, 0.43915, 0.43238],\
        [0.28798, 0.29361, 0.29877, 0.30342, 0.30771, 0.31177, 0.31569, 0.31953, 0.32332, 0.32709, 0.33086,\
        0.33466, 0.33851, 0.34244, 0.34649, 0.35067, 0.35503, 0.35960, 0.36444, 0.36957, 0.37506, 0.38095,\
        0.38731, 0.39423, 0.40200, 0.41056, 0.41981, 0.42956, 0.43965, 0.44993, 0.46030, 0.47062, 0.48075,\
        0.49050, 0.49949, 0.50822, 0.51703, 0.52565, 0.53412, 0.54252, 0.55090, 0.55926, 0.56762, 0.57592,\
        0.58407, 0.59193, 0.59875, 0.60423, 0.61051, 0.61733, 0.62388, 0.62967, 0.63473, 0.63925, 0.64332,\
        0.64695, 0.65020, 0.65308, 0.65549, 0.65727, 0.65832, 0.65867, 0.65841, 0.65753, 0.65603, 0.65403,\
        0.65179, 0.64949, 0.64721, 0.64496, 0.64277, 0.64059, 0.63830, 0.63574, 0.63274, 0.62921, 0.62514,\
        0.62062, 0.61593, 0.61135, 0.60642, 0.60116, 0.59683, 0.59192, 0.58677, 0.58133, 0.57587, 0.57049,\
        0.56526, 0.56021, 0.55535, 0.55069, 0.54624, 0.54198, 0.53789, 0.53397, 0.53018, 0.52652, 0.52297,\
        0.51950, 0.51610, 0.51273, 0.50939, 0.50608, 0.50289, 0.49990, 0.49704, 0.49406, 0.49075, 0.48718,\
        0.48339, 0.47936, 0.47505, 0.47045, 0.46553, 0.46030, 0.45475, 0.44889, 0.44272, 0.43625, 0.42947],\
        [0.28709, 0.29266, 0.29776, 0.30244, 0.30676, 0.31079, 0.31469, 0.31851, 0.32228, 0.32602, 0.32976,\
        0.33351, 0.33730, 0.34117, 0.34513, 0.34923, 0.35349, 0.35797, 0.36269, 0.36769, 0.37304, 0.37878,\
        0.38499, 0.39176, 0.39923, 0.40749, 0.41649, 0.42596, 0.43573, 0.44573, 0.45582, 0.46588, 0.47575,\
        0.48521, 0.49417, 0.50310, 0.51193, 0.52031, 0.52842, 0.53646, 0.54448, 0.55244, 0.56030, 0.56800,\
        0.57543, 0.58235, 0.58831, 0.59356, 0.59954, 0.60613, 0.61269, 0.61823, 0.62308, 0.62736, 0.63105,\
        0.63440, 0.63753, 0.64037, 0.64274, 0.64449, 0.64561, 0.64611, 0.64602, 0.64535, 0.64412, 0.64241,\
        0.64038, 0.63817, 0.63593, 0.63375, 0.63168, 0.62969, 0.62763, 0.62533, 0.62259, 0.61933, 0.61552,\
        0.61120, 0.60672, 0.60272, 0.59948, 0.59402, 0.58942, 0.58477, 0.57993, 0.57493, 0.56988, 0.56489,\
        0.56002, 0.55530, 0.55075, 0.54638, 0.54219, 0.53817, 0.53431, 0.53058, 0.52699, 0.52350, 0.52011,\
        0.51679, 0.51351, 0.51026, 0.50698, 0.50363, 0.50035, 0.49735, 0.49441, 0.49135, 0.48805, 0.48449,\
        0.48068, 0.47661, 0.47226, 0.46762, 0.46268, 0.45742, 0.45185, 0.44596, 0.43977, 0.43328, 0.42649],\
        [0.28601, 0.29156, 0.29671, 0.30141, 0.30562, 0.30963, 0.31354, 0.31738, 0.32117, 0.32491, 0.32862,\
        0.33234, 0.33609, 0.33989, 0.34379, 0.34781, 0.35199, 0.35637, 0.36098, 0.36588, 0.37111, 0.37673,\
        0.38280, 0.38939, 0.39655, 0.40458, 0.41327, 0.42232, 0.43171, 0.44136, 0.45115, 0.46095, 0.47062,\
        0.48004, 0.48923, 0.49829, 0.50686, 0.51479, 0.52247, 0.53022, 0.53795, 0.54554, 0.55293, 0.56007,\
        0.56686, 0.57314, 0.57883, 0.58425, 0.59002, 0.59641, 0.60272, 0.60753, 0.61222, 0.61628, 0.61947,\
        0.62248, 0.62560, 0.62850, 0.63086, 0.63260, 0.63375, 0.63438, 0.63449, 0.63406, 0.63308, 0.63161,\
        0.62972, 0.62757, 0.62531, 0.62314, 0.62117, 0.61936, 0.61756, 0.61557, 0.61312, 0.61011, 0.60668,\
        0.60256, 0.59835, 0.59464, 0.59139, 0.58709, 0.58252, 0.57799, 0.57341, 0.56876, 0.56408, 0.55945,\
        0.55493, 0.55053, 0.54628, 0.54218, 0.53824, 0.53445, 0.53080, 0.52727, 0.52385, 0.52053, 0.51729,\
        0.51410, 0.51096, 0.50783, 0.50468, 0.50145, 0.49811, 0.49496, 0.49188, 0.48870, 0.48535, 0.48176,\
        0.47792, 0.47382, 0.46944, 0.46477, 0.45979, 0.45450, 0.44890, 0.44299, 0.43677, 0.43026, 0.42345],\
        [0.28478, 0.29029, 0.29537, 0.29997, 0.30420, 0.30829, 0.31228, 0.31619, 0.32001, 0.32376, 0.32747,\
        0.33116, 0.33487, 0.33862, 0.34246, 0.34641, 0.35052, 0.35481, 0.35932, 0.36412, 0.36923, 0.37474,\
        0.38069, 0.38718, 0.39431, 0.40207, 0.41011, 0.41861, 0.42755, 0.43684, 0.44632, 0.45588, 0.46538,\
        0.47476, 0.48398, 0.49299, 0.50146, 0.50878, 0.51615, 0.52382, 0.53138, 0.53864, 0.54560, 0.55225,\
        0.55855, 0.56447, 0.57005, 0.57552, 0.58120, 0.58716, 0.59269, 0.59733, 0.60215, 0.60601, 0.60838,\
        0.61108, 0.61441, 0.61748, 0.61984, 0.62147, 0.62260, 0.62338, 0.62371, 0.62351, 0.62279, 0.62154,\
        0.61976, 0.61761, 0.61529, 0.61306, 0.61118, 0.60957, 0.60801, 0.60636, 0.60421, 0.60131, 0.59826,\
        0.59459, 0.59070, 0.58714, 0.58369, 0.57985, 0.57570, 0.57144, 0.56713, 0.56279, 0.55845, 0.55416,\
        0.54995, 0.54586, 0.54190, 0.53807, 0.53437, 0.53080, 0.52735, 0.52400, 0.52074, 0.51756, 0.51445,\
        0.51140, 0.50837, 0.50534, 0.50232, 0.49922, 0.49590, 0.49261, 0.48937, 0.48608, 0.48264, 0.47900,\
        0.47512, 0.47099, 0.46659, 0.46189, 0.45689, 0.45157, 0.44593, 0.43999, 0.43374, 0.42719, 0.42036],\
        [0.28346, 0.28891, 0.29387, 0.29838, 0.30267, 0.30689, 0.31101, 0.31499, 0.31885, 0.32261, 0.32631,\
        0.32998, 0.33366, 0.33737, 0.34115, 0.34504, 0.34907, 0.35328, 0.35770, 0.36238, 0.36738, 0.37275,\
        0.37856, 0.38488, 0.39177, 0.39910, 0.40662, 0.41466, 0.42323, 0.43219, 0.44138, 0.45068, 0.45998,\
        0.46920, 0.47825, 0.48697, 0.49497, 0.50189, 0.50963, 0.51750, 0.52489, 0.53180, 0.53833, 0.54456,\
        0.55050, 0.55616, 0.56163, 0.56702, 0.57248, 0.57793, 0.58307, 0.58783, 0.59244, 0.59581, 0.59781,\
        0.60045, 0.60414, 0.60729, 0.60957, 0.61096, 0.61201, 0.61300, 0.61356, 0.61356, 0.61310, 0.61209,\
        0.61040, 0.60826, 0.60588, 0.60353, 0.60176, 0.60032, 0.59890, 0.59743, 0.59556, 0.59291, 0.58999,\
        0.58673, 0.58326, 0.57983, 0.57641, 0.57280, 0.56898, 0.56502, 0.56101, 0.55697, 0.55294, 0.54897,\
        0.54507, 0.54127, 0.53759, 0.53401, 0.53055, 0.52718, 0.52391, 0.52073, 0.51762, 0.51457, 0.51158,\
        0.50863, 0.50570, 0.50272, 0.49972, 0.49664, 0.49341, 0.49012, 0.48679, 0.48340, 0.47989, 0.47620,\
        0.47229, 0.46814, 0.46371, 0.45899, 0.45397, 0.44862, 0.44295, 0.43697, 0.43067, 0.42409, 0.41723],\
        [0.28214, 0.28758, 0.29253, 0.29702, 0.30131, 0.30564, 0.30984, 0.31386, 0.31774, 0.32149, 0.32517,\
        0.32882, 0.33246, 0.33613, 0.33986, 0.34369, 0.34765, 0.35177, 0.35610, 0.36067, 0.36553, 0.37073,\
        0.37633, 0.38235, 0.38879, 0.39558, 0.40273, 0.41051, 0.41879, 0.42745, 0.43634, 0.44537, 0.45442,\
        0.46340, 0.47218, 0.48058, 0.48837, 0.49578, 0.50381, 0.51153, 0.51854, 0.52499, 0.53109, 0.53695,\
        0.54260, 0.54807, 0.55339, 0.55862, 0.56381, 0.56890, 0.57377, 0.57833, 0.58244, 0.58568, 0.58822,\
        0.59111, 0.59449, 0.59760, 0.59978, 0.60089, 0.60186, 0.60321, 0.60393, 0.60400, 0.60379, 0.60310,\
        0.60144, 0.59937, 0.59708, 0.59482, 0.59305, 0.59164, 0.59027, 0.58881, 0.58700, 0.58468, 0.58200,\
        0.57904, 0.57590, 0.57268, 0.56939, 0.56597, 0.56242, 0.55875, 0.55502, 0.55127, 0.54754, 0.54386,\
        0.54025, 0.53673, 0.53331, 0.52998, 0.52674, 0.52358, 0.52048, 0.51745, 0.51447, 0.51153, 0.50861,\
        0.50574, 0.50292, 0.49999, 0.49697, 0.49389, 0.49071, 0.48743, 0.48408, 0.48064, 0.47708, 0.47335,\
        0.46941, 0.46524, 0.46081, 0.45608, 0.45104, 0.44567, 0.43997, 0.43394, 0.42760, 0.42097, 0.41407],\
        [0.28084, 0.28639, 0.29145, 0.29607, 0.30043, 0.30470, 0.30885, 0.31285, 0.31669, 0.32041, 0.32405,\
        0.32766, 0.33126, 0.33490, 0.33858, 0.34236, 0.34625, 0.35029, 0.35452, 0.35897, 0.36368, 0.36869,\
        0.37403, 0.37973, 0.38576, 0.39212, 0.39892, 0.40639, 0.41436, 0.42270, 0.43128, 0.44000, 0.44877,\
        0.45749, 0.46604, 0.47432, 0.48231, 0.49026, 0.49832, 0.50563, 0.51214, 0.51808, 0.52376, 0.52934,\
        0.53478, 0.54008, 0.54524, 0.55029, 0.55523, 0.56003, 0.56462, 0.56891, 0.57277, 0.57608, 0.57903,\
        0.58202, 0.58512, 0.58793, 0.59002, 0.59134, 0.59249, 0.59377, 0.59455, 0.59474, 0.59465, 0.59408,\
        0.59266, 0.59076, 0.58870, 0.58668, 0.58494, 0.58345, 0.58204, 0.58054, 0.57878, 0.57667, 0.57424,\
        0.57155, 0.56868, 0.56567, 0.56257, 0.55935, 0.55602, 0.55261, 0.54914, 0.54567, 0.54221, 0.53881,\
        0.53547, 0.53222, 0.52906, 0.52597, 0.52294, 0.51996, 0.51702, 0.51412, 0.51126, 0.50843, 0.50557,\
        0.50267, 0.49994, 0.49709, 0.49408, 0.49099, 0.48782, 0.48456, 0.48121, 0.47775, 0.47416, 0.47041,\
        0.46647, 0.46230, 0.45786, 0.45314, 0.44810, 0.44273, 0.43701, 0.43094, 0.42454, 0.41785, 0.41091],\
        [0.27956, 0.28527, 0.29055, 0.29538, 0.29977, 0.30395, 0.30803, 0.31192, 0.31568, 0.31933, 0.32293,\
        0.32650, 0.33008, 0.33367, 0.33732, 0.34104, 0.34487, 0.34883, 0.35296, 0.35728, 0.36184, 0.36665,\
        0.37175, 0.37715, 0.38286, 0.38891, 0.39541, 0.40247, 0.41005, 0.41801, 0.42625, 0.43464, 0.44310,\
        0.45154, 0.45988, 0.46808, 0.47615, 0.48415, 0.49196, 0.49912, 0.50536, 0.51082, 0.51623, 0.52168,\
        0.52699, 0.53214, 0.53716, 0.54203, 0.54674, 0.55129, 0.55563, 0.55971, 0.56346, 0.56686, 0.57000,\
        0.57302, 0.57593, 0.57852, 0.58059, 0.58215, 0.58345, 0.58459, 0.58537, 0.58570, 0.58563, 0.58508,\
        0.58394, 0.58235, 0.58057, 0.57878, 0.57712, 0.57562, 0.57416, 0.57264, 0.57092, 0.56895, 0.56672,\
        0.56426, 0.56162, 0.55883, 0.55592, 0.55290, 0.54978, 0.54660, 0.54337, 0.54014, 0.53694, 0.53379,\
        0.53072, 0.52772, 0.52480, 0.52195, 0.51913, 0.51632, 0.51351, 0.51072, 0.50797, 0.50526, 0.50250,\
        0.49958, 0.49675, 0.49394, 0.49098, 0.48792, 0.48477, 0.48152, 0.47817, 0.47472, 0.47113, 0.46738,\
        0.46344, 0.45928, 0.45487, 0.45017, 0.44516, 0.43981, 0.43408, 0.42798, 0.42152, 0.41475, 0.40775],\
        [0.27819, 0.28411, 0.28964, 0.29479, 0.29947, 0.30345, 0.30728, 0.31100, 0.31465, 0.31824, 0.32179,\
        0.32533, 0.32888, 0.33245, 0.33607, 0.33975, 0.34352, 0.34740, 0.35143, 0.35563, 0.36003, 0.36466,\
        0.36953, 0.37468, 0.38012, 0.38590, 0.39209, 0.39875, 0.40590, 0.41345, 0.42130, 0.42933, 0.43744,\
        0.44557, 0.45368, 0.46171, 0.46966, 0.47746, 0.48496, 0.49188, 0.49793, 0.50304, 0.50855, 0.51404,\
        0.51922, 0.52426, 0.52915, 0.53385, 0.53835, 0.54266, 0.54680, 0.55074, 0.55444, 0.55789, 0.56111,\
        0.56415, 0.56696, 0.56946, 0.57154, 0.57321, 0.57458, 0.57568, 0.57645, 0.57683, 0.57681, 0.57634,\
        0.57541, 0.57411, 0.57261, 0.57103, 0.56950, 0.56802, 0.56656, 0.56503, 0.56337, 0.56151, 0.55945,\
        0.55719, 0.55475, 0.55216, 0.54944, 0.54661, 0.54368, 0.54070, 0.53769, 0.53468, 0.53171, 0.52880,\
        0.52597, 0.52321, 0.52053, 0.51791, 0.51530, 0.51267, 0.50997, 0.50722, 0.50452, 0.50194, 0.49928,\
        0.49643, 0.49354, 0.49067, 0.48772, 0.48468, 0.48154, 0.47831, 0.47497, 0.47153, 0.46795, 0.46421,\
        0.46030, 0.45617, 0.45181, 0.44716, 0.44221, 0.43690, 0.43122, 0.42511, 0.41860, 0.41173, 0.40466],\
        [0.27666, 0.28273, 0.28844, 0.29378, 0.29857, 0.30259, 0.30630, 0.30994, 0.31352, 0.31706, 0.32059,\
        0.32413, 0.32767, 0.33123, 0.33482, 0.33847, 0.34218, 0.34600, 0.34993, 0.35401, 0.35827, 0.36272,\
        0.36740, 0.37232, 0.37752, 0.38303, 0.38891, 0.39519, 0.40189, 0.40902, 0.41649, 0.42411, 0.43182,\
        0.43960, 0.44741, 0.45519, 0.46289, 0.47041, 0.47758, 0.48420, 0.49008, 0.49544, 0.50128, 0.50649,\
        0.51144, 0.51644, 0.52125, 0.52578, 0.53005, 0.53416, 0.53812, 0.54196, 0.54565, 0.54914, 0.55242,\
        0.55547, 0.55825, 0.56070, 0.56278, 0.56450, 0.56590, 0.56700, 0.56777, 0.56818, 0.56821, 0.56785,\
        0.56711, 0.56606, 0.56480, 0.56342, 0.56202, 0.56061, 0.55918, 0.55769, 0.55608, 0.55433, 0.55241,\
        0.55032, 0.54807, 0.54566, 0.54312, 0.54046, 0.53772, 0.53491, 0.53209, 0.52928, 0.52651, 0.52381,\
        0.52120, 0.51868, 0.51622, 0.51382, 0.51144, 0.50899, 0.50641, 0.50368, 0.50089, 0.49839, 0.49578,\
        0.49300, 0.49015, 0.48725, 0.48429, 0.48126, 0.47813, 0.47492, 0.47160, 0.46817, 0.46461, 0.46090,\
        0.45703, 0.45295, 0.44864, 0.44407, 0.43921, 0.43400, 0.42841, 0.42237, 0.41585, 0.40887, 0.40169],\
        [0.27490, 0.28106, 0.28682, 0.29214, 0.29689, 0.30108, 0.30496, 0.30864, 0.31222, 0.31577, 0.31932,\
        0.32287, 0.32643, 0.33000, 0.33359, 0.33721, 0.34088, 0.34462, 0.34847, 0.35244, 0.35656, 0.36085,\
        0.36535, 0.37007, 0.37504, 0.38031, 0.38589, 0.39180, 0.39807, 0.40482, 0.41189, 0.41902, 0.42625,\
        0.43362, 0.44109, 0.44858, 0.45599, 0.46321, 0.47011, 0.47655, 0.48250, 0.48819, 0.49378, 0.49871,\
        0.50368, 0.50875, 0.51348, 0.51781, 0.52185, 0.52574, 0.52957, 0.53336, 0.53705, 0.54060, 0.54393,\
        0.54701, 0.54978, 0.55221, 0.55429, 0.55602, 0.55743, 0.55853, 0.55932, 0.55976, 0.55986, 0.55962,\
        0.55904, 0.55819, 0.55714, 0.55595, 0.55468, 0.55336, 0.55199, 0.55055, 0.54901, 0.54736, 0.54558,\
        0.54365, 0.54157, 0.53934, 0.53697, 0.53447, 0.53187, 0.52922, 0.52656, 0.52392, 0.52133, 0.51883,\
        0.51642, 0.51411, 0.51185, 0.50965, 0.50751, 0.50526, 0.50280, 0.50021, 0.49737, 0.49473, 0.49211,\
        0.48937, 0.48654, 0.48365, 0.48069, 0.47766, 0.47454, 0.47134, 0.46804, 0.46463, 0.46110, 0.45743,\
        0.45359, 0.44957, 0.44534, 0.44087, 0.43611, 0.43105, 0.42562, 0.41976, 0.41338, 0.40638, 0.39902],\
        [0.27292, 0.27911, 0.28489, 0.29020, 0.29502, 0.29937, 0.30337, 0.30710, 0.31073, 0.31435, 0.31797,\
        0.32158, 0.32518, 0.32877, 0.33236, 0.33597, 0.33960, 0.34329, 0.34705, 0.35092, 0.35491, 0.35906,\
        0.36338, 0.36791, 0.37267, 0.37770, 0.38302, 0.38865, 0.39463, 0.40103, 0.40753, 0.41400, 0.42070,\
        0.42767, 0.43479, 0.44197, 0.44909, 0.45604, 0.46272, 0.46904, 0.47501, 0.48069, 0.48609, 0.49115,\
        0.49619, 0.50125, 0.50583, 0.50993, 0.51370, 0.51739, 0.52112, 0.52491, 0.52867, 0.53229, 0.53569,\
        0.53881, 0.54158, 0.54399, 0.54604, 0.54776, 0.54917, 0.55028, 0.55108, 0.55157, 0.55175, 0.55161,\
        0.55118, 0.55051, 0.54963, 0.54860, 0.54747, 0.54625, 0.54495, 0.54358, 0.54213, 0.54059, 0.53894,\
        0.53717, 0.53526, 0.53320, 0.53098, 0.52862, 0.52615, 0.52363, 0.52111, 0.51861, 0.51618, 0.51383,\
        0.51160, 0.50952, 0.50745, 0.50533, 0.50348, 0.50143, 0.49900, 0.49643, 0.49372, 0.49101, 0.48831,\
        0.48557, 0.48275, 0.47986, 0.47690, 0.47387, 0.47076, 0.46757, 0.46428, 0.46090, 0.45739, 0.45375,\
        0.44997, 0.44601, 0.44186, 0.43749, 0.43286, 0.42795, 0.42273, 0.41714, 0.41112, 0.40452, 0.39725],\
        [0.27073, 0.27696, 0.28276, 0.28810, 0.29299, 0.29748, 0.30155, 0.30532, 0.30908, 0.31282, 0.31655,\
        0.32026, 0.32393, 0.32756, 0.33116, 0.33475, 0.33835, 0.34198, 0.34567, 0.34944, 0.35331, 0.35732,\
        0.36148, 0.36583, 0.37038, 0.37518, 0.38024, 0.38561, 0.39132, 0.39734, 0.40313, 0.40890, 0.41515,\
        0.42176, 0.42857, 0.43546, 0.44230, 0.44900, 0.45548, 0.46168, 0.46759, 0.47321, 0.47857, 0.48371,\
        0.48874, 0.49362, 0.49809, 0.50203, 0.50556, 0.50911, 0.51281, 0.51665, 0.52053, 0.52425, 0.52772,\
        0.53088, 0.53365, 0.53602, 0.53802, 0.53970, 0.54110, 0.54222, 0.54305, 0.54359, 0.54384, 0.54381,\
        0.54352, 0.54300, 0.54228, 0.54140, 0.54040, 0.53928, 0.53807, 0.53677, 0.53541, 0.53397, 0.53246,\
        0.53085, 0.52912, 0.52724, 0.52517, 0.52292, 0.52055, 0.51813, 0.51572, 0.51336, 0.51106, 0.50883,\
        0.50673, 0.50488, 0.50316, 0.50091, 0.49917, 0.49735, 0.49496, 0.49241, 0.48977, 0.48707, 0.48435,\
        0.48159, 0.47876, 0.47588, 0.47292, 0.46989, 0.46679, 0.46360, 0.46033, 0.45695, 0.45347, 0.44987,\
        0.44613, 0.44223, 0.43816, 0.43387, 0.42936, 0.42459, 0.41954, 0.41420, 0.40853, 0.40238, 0.39507],\
        [0.26840, 0.27467, 0.28050, 0.28588, 0.29082, 0.29532, 0.29940, 0.30341, 0.30733, 0.31123, 0.31512,\
        0.31894, 0.32269, 0.32637, 0.32999, 0.33357, 0.33713, 0.34071, 0.34432, 0.34800, 0.35176, 0.35563,\
        0.35964, 0.36381, 0.36816, 0.37272, 0.37750, 0.38253, 0.38778, 0.39312, 0.39828, 0.40372, 0.40970,\
        0.41603, 0.42255, 0.42914, 0.43571, 0.44217, 0.44846, 0.45452, 0.46032, 0.46586, 0.47117, 0.47627,\
        0.48120, 0.48588, 0.49020, 0.49404, 0.49752, 0.50103, 0.50474, 0.50867, 0.51271, 0.51650, 0.52003,\
        0.52324, 0.52599, 0.52828, 0.53019, 0.53181, 0.53320, 0.53435, 0.53522, 0.53581, 0.53613, 0.53619,\
        0.53603, 0.53565, 0.53508, 0.53435, 0.53346, 0.53245, 0.53131, 0.53009, 0.52881, 0.52748, 0.52611,\
        0.52468, 0.52314, 0.52145, 0.51953, 0.51739, 0.51507, 0.51272, 0.51041, 0.50816, 0.50599, 0.50389,\
        0.50183, 0.50010, 0.49875, 0.49673, 0.49475, 0.49283, 0.49059, 0.48813, 0.48555, 0.48289, 0.48018,\
        0.47741, 0.47459, 0.47170, 0.46874, 0.46571, 0.46261, 0.45942, 0.45616, 0.45280, 0.44934, 0.44576,\
        0.44206, 0.43821, 0.43420, 0.42999, 0.42555, 0.42087, 0.41592, 0.41066, 0.40500, 0.39889, 0.39176],\
        [0.26594, 0.27228, 0.27817, 0.28362, 0.28862, 0.29320, 0.29747, 0.30154, 0.30556, 0.30968, 0.31374,\
        0.31769, 0.32152, 0.32523, 0.32885, 0.33242, 0.33595, 0.33947, 0.34301, 0.34660, 0.35025, 0.35400,\
        0.35786, 0.36185, 0.36599, 0.37031, 0.37479, 0.37945, 0.38423, 0.38904, 0.39386, 0.39896, 0.40458,\
        0.41063, 0.41684, 0.42311, 0.42939, 0.43561, 0.44169, 0.44758, 0.45325, 0.45868, 0.46389, 0.46890,\
        0.47368, 0.47821, 0.48241, 0.48625, 0.48984, 0.49341, 0.49709, 0.50114, 0.50531, 0.50902, 0.51261,\
        0.51590, 0.51859, 0.52073, 0.52250, 0.52405, 0.52545, 0.52665, 0.52758, 0.52822, 0.52859, 0.52875,\
        0.52870, 0.52846, 0.52804, 0.52743, 0.52667, 0.52575, 0.52470, 0.52353, 0.52232, 0.52109, 0.51986,\
        0.51862, 0.51730, 0.51583, 0.51410, 0.51203, 0.50971, 0.50739, 0.50520, 0.50303, 0.50098, 0.49902,\
        0.49709, 0.49538, 0.49388, 0.49212, 0.49019, 0.48817, 0.48598, 0.48361, 0.48110, 0.47849, 0.47580,\
        0.47305, 0.47022, 0.46733, 0.46437, 0.46134, 0.45823, 0.45504, 0.45177, 0.44842, 0.44497, 0.44141,\
        0.43773, 0.43392, 0.42995, 0.42580, 0.42142, 0.41675, 0.41181, 0.40656, 0.40081, 0.39447, 0.38741],\
        [0.26340, 0.26983, 0.27582, 0.28137, 0.28649, 0.29119, 0.29553, 0.29958, 0.30392, 0.30830, 0.31251,\
        0.31654, 0.32041, 0.32413, 0.32775, 0.33129, 0.33478, 0.33825, 0.34172, 0.34522, 0.34877, 0.35240,\
        0.35612, 0.35994, 0.36389, 0.36797, 0.37217, 0.37650, 0.38091, 0.38538, 0.38995, 0.39478, 0.40006,\
        0.40572, 0.41152, 0.41741, 0.42339, 0.42937, 0.43524, 0.44093, 0.44642, 0.45171, 0.45678, 0.46166,\
        0.46631, 0.47073, 0.47487, 0.47876, 0.48247, 0.48617, 0.48999, 0.49408, 0.49812, 0.50163, 0.50547,\
        0.50887, 0.51140, 0.51331, 0.51487, 0.51636, 0.51783, 0.51913, 0.52013, 0.52079, 0.52120, 0.52144,\
        0.52152, 0.52143, 0.52113, 0.52066, 0.52002, 0.51921, 0.51822, 0.51709, 0.51592, 0.51478, 0.51369,\
        0.51263, 0.51155, 0.51033, 0.50885, 0.50691, 0.50447, 0.50213, 0.50010, 0.49802, 0.49603, 0.49415,\
        0.49233, 0.49063, 0.48900, 0.48727, 0.48539, 0.48338, 0.48121, 0.47890, 0.47644, 0.47388, 0.47122,\
        0.46848, 0.46567, 0.46278, 0.45981, 0.45677, 0.45365, 0.45045, 0.44717, 0.44382, 0.44036, 0.43681,\
        0.43314, 0.42934, 0.42541, 0.42131, 0.41698, 0.41228, 0.40724, 0.40192, 0.39617, 0.38970, 0.38256],\
        [0.26077, 0.26732, 0.27346, 0.27917, 0.28447, 0.28938, 0.29396, 0.29836, 0.30284, 0.30723, 0.31146,\
        0.31550, 0.31937, 0.32308, 0.32667, 0.33018, 0.33362, 0.33703, 0.34044, 0.34386, 0.34732, 0.35084,\
        0.35442, 0.35809, 0.36186, 0.36572, 0.36969, 0.37374, 0.37787, 0.38210, 0.38646, 0.39108, 0.39603,\
        0.40125, 0.40657, 0.41204, 0.41775, 0.42349, 0.42913, 0.43459, 0.43987, 0.44497, 0.44988, 0.45461,\
        0.45914, 0.46347, 0.46760, 0.47153, 0.47536, 0.47917, 0.48306, 0.48704, 0.49093, 0.49467, 0.49869,\
        0.50195, 0.50424, 0.50592, 0.50724, 0.50869, 0.51034, 0.51181, 0.51287, 0.51352, 0.51393, 0.51425,\
        0.51448, 0.51454, 0.51437, 0.51400, 0.51351, 0.51284, 0.51190, 0.51077, 0.50962, 0.50854, 0.50757,\
        0.50669, 0.50583, 0.50486, 0.50368, 0.50202, 0.49952, 0.49708, 0.49507, 0.49307, 0.49114, 0.48928,\
        0.48750, 0.48578, 0.48408, 0.48232, 0.48045, 0.47845, 0.47631, 0.47403, 0.47161, 0.46909, 0.46646,\
        0.46375, 0.46094, 0.45805, 0.45507, 0.45201, 0.44887, 0.44566, 0.44236, 0.43899, 0.43553, 0.43197,\
        0.42829, 0.42448, 0.42055, 0.41651, 0.41230, 0.40762, 0.40234, 0.39691, 0.39103, 0.38450, 0.37730],\
        [0.25806, 0.26477, 0.27107, 0.27698, 0.28252, 0.28771, 0.29264, 0.29744, 0.30198, 0.30631, 0.31051,\
        0.31453, 0.31836, 0.32204, 0.32559, 0.32905, 0.33244, 0.33580, 0.33915, 0.34250, 0.34588, 0.34930,\
        0.35277, 0.35630, 0.35991, 0.36359, 0.36735, 0.37119, 0.37510, 0.37912, 0.38329, 0.38767, 0.39228,\
        0.39710, 0.40185, 0.40701, 0.41254, 0.41805, 0.42341, 0.42859, 0.43361, 0.43848, 0.44321, 0.44778,\
        0.45220, 0.45646, 0.46056, 0.46453, 0.46841, 0.47228, 0.47617, 0.48008, 0.48394, 0.48776, 0.49146,\
        0.49456, 0.49682, 0.49847, 0.49965, 0.50116, 0.50307, 0.50469, 0.50580, 0.50637, 0.50672, 0.50715,\
        0.50758, 0.50783, 0.50772, 0.50742, 0.50711, 0.50664, 0.50574, 0.50457, 0.50344, 0.50243, 0.50156,\
        0.50082, 0.50011, 0.49929, 0.49828, 0.49678, 0.49459, 0.49225, 0.49013, 0.48814, 0.48622, 0.48437,\
        0.48258, 0.48083, 0.47909, 0.47729, 0.47540, 0.47340, 0.47128, 0.46902, 0.46664, 0.46415, 0.46155,\
        0.45885, 0.45605, 0.45316, 0.45017, 0.44709, 0.44392, 0.44067, 0.43735, 0.43396, 0.43048, 0.42691,\
        0.42320, 0.41934, 0.41535, 0.41131, 0.40728, 0.40308, 0.39756, 0.39179, 0.38565, 0.37896, 0.37168],\
        [0.25525, 0.26213, 0.26863, 0.27476, 0.28055, 0.28603, 0.29123, 0.29616, 0.30082, 0.30533, 0.30956,\
        0.31354, 0.31732, 0.32095, 0.32446, 0.32788, 0.33123, 0.33454, 0.33784, 0.34113, 0.34444, 0.34777,\
        0.35115, 0.35457, 0.35804, 0.36157, 0.36516, 0.36882, 0.37256, 0.37639, 0.38036, 0.38449, 0.38878,\
        0.39315, 0.39760, 0.40263, 0.40791, 0.41310, 0.41808, 0.42292, 0.42764, 0.43226, 0.43678, 0.44119,\
        0.44549, 0.44968, 0.45374, 0.45771, 0.46161, 0.46547, 0.46932, 0.47314, 0.47692, 0.48058, 0.48400,\
        0.48692, 0.48921, 0.49096, 0.49241, 0.49405, 0.49592, 0.49759, 0.49875, 0.49929, 0.49962, 0.50021,\
        0.50080, 0.50128, 0.50119, 0.50089, 0.50078, 0.50058, 0.49971, 0.49854, 0.49749, 0.49655, 0.49576,\
        0.49507, 0.49441, 0.49363, 0.49262, 0.49119, 0.48931, 0.48722, 0.48515, 0.48316, 0.48125, 0.47940,\
        0.47759, 0.47580, 0.47401, 0.47218, 0.47027, 0.46827, 0.46615, 0.46390, 0.46154, 0.45907, 0.45649,\
        0.45380, 0.45101, 0.44812, 0.44511, 0.44200, 0.43880, 0.43550, 0.43214, 0.42871, 0.42523, 0.42164,\
        0.41790, 0.41397, 0.40986, 0.40567, 0.40156, 0.39757, 0.39242, 0.38637, 0.37996, 0.37309, 0.36571],\
        [0.25233, 0.25939, 0.26609, 0.27245, 0.27848, 0.28422, 0.28967, 0.29485, 0.29975, 0.30431, 0.30851,\
        0.31244, 0.31618, 0.31977, 0.32324, 0.32662, 0.32994, 0.33322, 0.33648, 0.33973, 0.34299, 0.34626,\
        0.34955, 0.35288, 0.35624, 0.35964, 0.36309, 0.36660, 0.37019, 0.37386, 0.37765, 0.38157, 0.38561,\
        0.38979, 0.39419, 0.39893, 0.40386, 0.40860, 0.41309, 0.41752, 0.42192, 0.42628, 0.43059, 0.43483,\
        0.43901, 0.44311, 0.44712, 0.45105, 0.45492, 0.45873, 0.46250, 0.46622, 0.46984, 0.47331, 0.47651,\
        0.47931, 0.48163, 0.48356, 0.48527, 0.48700, 0.48877, 0.49036, 0.49155, 0.49228, 0.49284, 0.49348,\
        0.49413, 0.49464, 0.49471, 0.49459, 0.49452, 0.49431, 0.49365, 0.49268, 0.49173, 0.49087, 0.49011,\
        0.48942, 0.48873, 0.48792, 0.48689, 0.48553, 0.48385, 0.48197, 0.48002, 0.47808, 0.47618, 0.47432,\
        0.47250, 0.47068, 0.46886, 0.46700, 0.46507, 0.46305, 0.46092, 0.45868, 0.45633, 0.45387, 0.45130,\
        0.44863, 0.44585, 0.44295, 0.43993, 0.43679, 0.43353, 0.43017, 0.42674, 0.42327, 0.41977, 0.41619,\
        0.41245, 0.40846, 0.40422, 0.39984, 0.39558, 0.39124, 0.38618, 0.38029, 0.37381, 0.36685, 0.35938],\
        [0.24925, 0.25650, 0.26340, 0.26997, 0.27623, 0.28220, 0.28790, 0.29333, 0.29845, 0.30307, 0.30724,\
        0.31114, 0.31486, 0.31843, 0.32188, 0.32525, 0.32856, 0.33182, 0.33506, 0.33829, 0.34151, 0.34473,\
        0.34797, 0.35122, 0.35449, 0.35779, 0.36113, 0.36452, 0.36797, 0.37149, 0.37512, 0.37886, 0.38274,\
        0.38678, 0.39104, 0.39556, 0.40019, 0.40430, 0.40827, 0.41232, 0.41642, 0.42053, 0.42463, 0.42870,\
        0.43275, 0.43675, 0.44069, 0.44456, 0.44836, 0.45208, 0.45574, 0.45931, 0.46277, 0.46606, 0.46909,\
        0.47180, 0.47416, 0.47623, 0.47810, 0.47990, 0.48163, 0.48317, 0.48441, 0.48535, 0.48612, 0.48685,\
        0.48752, 0.48803, 0.48827, 0.48833, 0.48828, 0.48807, 0.48756, 0.48682, 0.48602, 0.48524, 0.48451,\
        0.48380, 0.48306, 0.48221, 0.48117, 0.47987, 0.47832, 0.47658, 0.47474, 0.47287, 0.47100, 0.46914,\
        0.46730, 0.46547, 0.46362, 0.46173, 0.45978, 0.45775, 0.45562, 0.45338, 0.45103, 0.44857, 0.44601,\
        0.44334, 0.44057, 0.43768, 0.43466, 0.43148, 0.42815, 0.42470, 0.42119, 0.41764, 0.41411, 0.41056,\
        0.40685, 0.40282, 0.39851, 0.39403, 0.38952, 0.38481, 0.37959, 0.37372, 0.36724, 0.36022, 0.35270],\
        [0.24600, 0.25343, 0.26052, 0.26729, 0.27375, 0.27992, 0.28580, 0.29138, 0.29662, 0.30133, 0.30557,\
        0.30954, 0.31329, 0.31689, 0.32035, 0.32373, 0.32705, 0.33032, 0.33356, 0.33678, 0.33999, 0.34319,\
        0.34638, 0.34958, 0.35279, 0.35601, 0.35925, 0.36253, 0.36585, 0.36925, 0.37272, 0.37630, 0.38001,\
        0.38387, 0.38790, 0.39207, 0.39617, 0.39987, 0.40351, 0.40729, 0.41115, 0.41502, 0.41889, 0.42279,\
        0.42671, 0.43061, 0.43446, 0.43824, 0.44193, 0.44554, 0.44906, 0.45248, 0.45576, 0.45888, 0.46178,\
        0.46442, 0.46680, 0.46895, 0.47093, 0.47278, 0.47451, 0.47607, 0.47739, 0.47849, 0.47942, 0.48025,\
        0.48096, 0.48152, 0.48188, 0.48206, 0.48208, 0.48191, 0.48153, 0.48097, 0.48030, 0.47961, 0.47891,\
        0.47819, 0.47741, 0.47653, 0.47547, 0.47421, 0.47275, 0.47111, 0.46936, 0.46755, 0.46571, 0.46386,\
        0.46201, 0.46015, 0.45829, 0.45638, 0.45442, 0.45238, 0.45024, 0.44799, 0.44563, 0.44316, 0.44060,\
        0.43795, 0.43520, 0.43233, 0.42931, 0.42610, 0.42270, 0.41914, 0.41550, 0.41185, 0.40822, 0.40471,\
        0.40104, 0.39697, 0.39257, 0.38798, 0.38324, 0.37824, 0.37281, 0.36684, 0.36032, 0.35326, 0.34569],\
        [0.24256, 0.25016, 0.25742, 0.26437, 0.27100, 0.27732, 0.28334, 0.28902, 0.29430, 0.29911, 0.30353,\
        0.30762, 0.31147, 0.31512, 0.31863, 0.32204, 0.32539, 0.32869, 0.33196, 0.33520, 0.33841, 0.34161,\
        0.34479, 0.34795, 0.35111, 0.35426, 0.35742, 0.36060, 0.36381, 0.36708, 0.37041, 0.37383, 0.37735,\
        0.38098, 0.38472, 0.38848, 0.39212, 0.39556, 0.39896, 0.40251, 0.40616, 0.40977, 0.41338, 0.41710,\
        0.42090, 0.42470, 0.42845, 0.43211, 0.43567, 0.43913, 0.44249, 0.44574, 0.44885, 0.45181, 0.45459,\
        0.45717, 0.45955, 0.46174, 0.46379, 0.46570, 0.46746, 0.46907, 0.47048, 0.47170, 0.47276, 0.47368,\
        0.47446, 0.47509, 0.47554, 0.47582, 0.47591, 0.47583, 0.47555, 0.47512, 0.47457, 0.47395, 0.47328,\
        0.47256, 0.47177, 0.47086, 0.46980, 0.46857, 0.46716, 0.46559, 0.46390, 0.46213, 0.46031, 0.45847,\
        0.45661, 0.45474, 0.45287, 0.45096, 0.44899, 0.44695, 0.44480, 0.44254, 0.44016, 0.43767, 0.43510,\
        0.43245, 0.42973, 0.42690, 0.42391, 0.42070, 0.41724, 0.41355, 0.40975, 0.40602, 0.40216, 0.39860,\
        0.39490, 0.39079, 0.38634, 0.38164, 0.37670, 0.37147, 0.36584, 0.35973, 0.35312, 0.34600, 0.33838],\
        [0.23890, 0.24666, 0.25409, 0.26119, 0.26797, 0.27443, 0.28056, 0.28632, 0.29168, 0.29663, 0.30119,\
        0.30544, 0.30939, 0.31311, 0.31669, 0.32017, 0.32357, 0.32693, 0.33025, 0.33353, 0.33678, 0.33999,\
        0.34317, 0.34632, 0.34944, 0.35254, 0.35562, 0.35871, 0.36182, 0.36496, 0.36815, 0.37140, 0.37472,\
        0.37811, 0.38154, 0.38496, 0.38830, 0.39153, 0.39476, 0.39808, 0.40152, 0.40476, 0.40807, 0.41165,\
        0.41535, 0.41906, 0.42268, 0.42619, 0.42959, 0.43288, 0.43606, 0.43912, 0.44206, 0.44486, 0.44752,\
        0.45004, 0.45240, 0.45462, 0.45671, 0.45867, 0.46049, 0.46216, 0.46366, 0.46498, 0.46615, 0.46716,\
        0.46803, 0.46873, 0.46927, 0.46962, 0.46980, 0.46980, 0.46962, 0.46929, 0.46883, 0.46827, 0.46763,\
        0.46692, 0.46612, 0.46520, 0.46414, 0.46293, 0.46156, 0.46003, 0.45839, 0.45664, 0.45483, 0.45298,\
        0.45111, 0.44924, 0.44736, 0.44545, 0.44350, 0.44146, 0.43931, 0.43703, 0.43462, 0.43209, 0.42949,\
        0.42684, 0.42416, 0.42140, 0.41848, 0.41530, 0.41182, 0.40804, 0.40394, 0.40021, 0.39636, 0.39248,\
        0.38858, 0.38438, 0.37985, 0.37503, 0.36991, 0.36448, 0.35866, 0.35241, 0.34569, 0.33848, 0.33080],\
        [0.23502, 0.24294, 0.25052, 0.25776, 0.26468, 0.27126, 0.27749, 0.28336, 0.28884, 0.29393, 0.29865,\
        0.30305, 0.30707, 0.31087, 0.31453, 0.31810, 0.32160, 0.32503, 0.32842, 0.33177, 0.33508, 0.33833,\
        0.34152, 0.34467, 0.34776, 0.35081, 0.35383, 0.35684, 0.35984, 0.36286, 0.36590, 0.36898, 0.37210,\
        0.37526, 0.37843, 0.38158, 0.38468, 0.38773, 0.39078, 0.39390, 0.39703, 0.39984, 0.40302, 0.40650,\
        0.41011, 0.41371, 0.41717, 0.42048, 0.42369, 0.42679, 0.42978, 0.43265, 0.43540, 0.43805, 0.44059,\
        0.44303, 0.44537, 0.44760, 0.44973, 0.45174, 0.45362, 0.45536, 0.45693, 0.45835, 0.45960, 0.46070,\
        0.46165, 0.46244, 0.46305, 0.46349, 0.46375, 0.46382, 0.46373, 0.46347, 0.46308, 0.46257, 0.46196,\
        0.46126, 0.46045, 0.45954, 0.45849, 0.45730, 0.45595, 0.45446, 0.45283, 0.45110, 0.44928, 0.44741,\
        0.44552, 0.44364, 0.44176, 0.43987, 0.43794, 0.43593, 0.43379, 0.43150, 0.42904, 0.42644, 0.42377,\
        0.42110, 0.41845, 0.41580, 0.41302, 0.40990, 0.40638, 0.40273, 0.39848, 0.39438, 0.39044, 0.38638,\
        0.38221, 0.37781, 0.37314, 0.36817, 0.36288, 0.35727, 0.35128, 0.34487, 0.33803, 0.33073, 0.32296],\
        [0.23091, 0.23897, 0.24670, 0.25409, 0.26114, 0.26784, 0.27418, 0.28016, 0.28577, 0.29100, 0.29590,\
        0.30044, 0.30446, 0.30837, 0.31217, 0.31586, 0.31946, 0.32300, 0.32649, 0.32993, 0.33330, 0.33661,\
        0.33984, 0.34299, 0.34606, 0.34908, 0.35204, 0.35496, 0.35786, 0.36075, 0.36365, 0.36657, 0.36950,\
        0.37245, 0.37539, 0.37831, 0.38120, 0.38407, 0.38694, 0.38982, 0.39265, 0.39543, 0.39849, 0.40168,\
        0.40524, 0.40871, 0.41191, 0.41497, 0.41797, 0.42088, 0.42367, 0.42634, 0.42890, 0.43138, 0.43379,\
        0.43616, 0.43846, 0.44070, 0.44287, 0.44493, 0.44687, 0.44867, 0.45032, 0.45181, 0.45314, 0.45432,\
        0.45534, 0.45620, 0.45690, 0.45741, 0.45774, 0.45789, 0.45786, 0.45767, 0.45733, 0.45686, 0.45627,\
        0.45558, 0.45478, 0.45387, 0.45283, 0.45166, 0.45035, 0.44888, 0.44726, 0.44552, 0.44367, 0.44177,\
        0.43984, 0.43794, 0.43606, 0.43421, 0.43233, 0.43037, 0.42826, 0.42596, 0.42344, 0.42075, 0.41796,\
        0.41521, 0.41258, 0.41002, 0.40745, 0.40457, 0.40075, 0.39707, 0.39298, 0.38867, 0.38442, 0.38013,\
        0.37572, 0.37110, 0.36624, 0.36109, 0.35563, 0.34984, 0.34368, 0.33712, 0.33015, 0.32274, 0.31488],\
        [0.22656, 0.23477, 0.24264, 0.25017, 0.25735, 0.26418, 0.27064, 0.27674, 0.28247, 0.28782, 0.29279,\
        0.29732, 0.30153, 0.30569, 0.30965, 0.31345, 0.31717, 0.32083, 0.32444, 0.32800, 0.33147, 0.33484,\
        0.33811, 0.34128, 0.34434, 0.34731, 0.35021, 0.35305, 0.35585, 0.35862, 0.36139, 0.36414, 0.36690,\
        0.36965, 0.37240, 0.37513, 0.37784, 0.38053, 0.38323, 0.38592, 0.38861, 0.39135, 0.39425, 0.39736,\
        0.40085, 0.40400, 0.40681, 0.40961, 0.41243, 0.41516, 0.41774, 0.42019, 0.42254, 0.42485, 0.42714,\
        0.42942, 0.43169, 0.43394, 0.43614, 0.43825, 0.44025, 0.44211, 0.44382, 0.44537, 0.44677, 0.44801,\
        0.44910, 0.45004, 0.45080, 0.45139, 0.45179, 0.45200, 0.45203, 0.45188, 0.45158, 0.45113, 0.45055,\
        0.44987, 0.44908, 0.44818, 0.44717, 0.44603, 0.44475, 0.44330, 0.44169, 0.43993, 0.43804, 0.43607,\
        0.43408, 0.43213, 0.43026, 0.42846, 0.42667, 0.42479, 0.42273, 0.42044, 0.41788, 0.41507, 0.41208,\
        0.40918, 0.40655, 0.40401, 0.40150, 0.39906, 0.39527, 0.39121, 0.38702, 0.38265, 0.37821, 0.37369,\
        0.36904, 0.36421, 0.35914, 0.35380, 0.34816, 0.34219, 0.33586, 0.32915, 0.32204, 0.31452, 0.30657],\
        [0.22197, 0.23033, 0.23834, 0.24601, 0.25333, 0.26029, 0.26689, 0.27311, 0.27896, 0.28443, 0.28951,\
        0.29421, 0.29867, 0.30295, 0.30700, 0.31090, 0.31473, 0.31853, 0.32230, 0.32600, 0.32958, 0.33304,\
        0.33635, 0.33952, 0.34257, 0.34550, 0.34833, 0.35109, 0.35379, 0.35645, 0.35908, 0.36168, 0.36428,\
        0.36686, 0.36943, 0.37199, 0.37454, 0.37708, 0.37962, 0.38217, 0.38476, 0.38743, 0.39025, 0.39328,\
        0.39642, 0.39924, 0.40172, 0.40434, 0.40706, 0.40963, 0.41199, 0.41420, 0.41634, 0.41847, 0.42063,\
        0.42283, 0.42507, 0.42733, 0.42957, 0.43174, 0.43379, 0.43570, 0.43745, 0.43904, 0.44048, 0.44178,\
        0.44293, 0.44393, 0.44477, 0.44543, 0.44589, 0.44615, 0.44622, 0.44611, 0.44582, 0.44538, 0.44481,\
        0.44413, 0.44335, 0.44247, 0.44149, 0.44040, 0.43916, 0.43775, 0.43615, 0.43435, 0.43239, 0.43033,\
        0.42825, 0.42623, 0.42434, 0.42260, 0.42093, 0.41919, 0.41722, 0.41494, 0.41236, 0.40950, 0.40626,\
        0.40306, 0.40046, 0.39789, 0.39530, 0.39253, 0.38905, 0.38501, 0.38073, 0.37628, 0.37170, 0.36699,\
        0.36213, 0.35709, 0.35182, 0.34629, 0.34046, 0.33431, 0.32782, 0.32096, 0.31372, 0.30607, 0.29802],\
        [0.21713, 0.22564, 0.23380, 0.24162, 0.24908, 0.25619, 0.26293, 0.26929, 0.27528, 0.28089, 0.28613,\
        0.29104, 0.29568, 0.30007, 0.30422, 0.30820, 0.31215, 0.31614, 0.32009, 0.32395, 0.32766, 0.33120,\
        0.33455, 0.33772, 0.34074, 0.34362, 0.34639, 0.34906, 0.35166, 0.35420, 0.35670, 0.35917, 0.36162,\
        0.36405, 0.36646, 0.36887, 0.37127, 0.37367, 0.37608, 0.37851, 0.38100, 0.38356, 0.38625, 0.38904,\
        0.39182, 0.39439, 0.39678, 0.39927, 0.40192, 0.40428, 0.40639, 0.40838, 0.41028, 0.41225, 0.41430,\
        0.41641, 0.41862, 0.42090, 0.42319, 0.42541, 0.42751, 0.42945, 0.43122, 0.43283, 0.43429, 0.43562,\
        0.43683, 0.43789, 0.43879, 0.43951, 0.44003, 0.44034, 0.44044, 0.44035, 0.44006, 0.43962, 0.43904,\
        0.43835, 0.43757, 0.43671, 0.43578, 0.43475, 0.43358, 0.43223, 0.43065, 0.42883, 0.42678, 0.42459,\
        0.42236, 0.42022, 0.41829, 0.41661, 0.41512, 0.41358, 0.41171, 0.40941, 0.40682, 0.40405, 0.40080,\
        0.39724, 0.39443, 0.39172, 0.38891, 0.38586, 0.38235, 0.37837, 0.37408, 0.36956, 0.36486, 0.35999,\
        0.35496, 0.34972, 0.34426, 0.33854, 0.33254, 0.32622, 0.31956, 0.31255, 0.30517, 0.29740, 0.28924],\
        [0.21204, 0.22070, 0.22902, 0.23699, 0.24461, 0.25187, 0.25877, 0.26529, 0.27145, 0.27723, 0.28265,\
        0.28773, 0.29251, 0.29703, 0.30127, 0.30532, 0.30945, 0.31368, 0.31787, 0.32190, 0.32573, 0.32933,\
        0.33270, 0.33587, 0.33884, 0.34166, 0.34435, 0.34693, 0.34943, 0.35185, 0.35423, 0.35657, 0.35888,\
        0.36118, 0.36345, 0.36572, 0.36798, 0.37026, 0.37254, 0.37486, 0.37723, 0.37967, 0.38218, 0.38474,\
        0.38726, 0.38968, 0.39202, 0.39441, 0.39683, 0.39897, 0.40090, 0.40274, 0.40437, 0.40625, 0.40820,\
        0.41018, 0.41238, 0.41469, 0.41703, 0.41932, 0.42145, 0.42339, 0.42514, 0.42673, 0.42819, 0.42953,\
        0.43077, 0.43189, 0.43287, 0.43365, 0.43422, 0.43457, 0.43468, 0.43459, 0.43430, 0.43383, 0.43322,\
        0.43251, 0.43172, 0.43089, 0.43001, 0.42907, 0.42801, 0.42675, 0.42523, 0.42340, 0.42125, 0.41888,\
        0.41646, 0.41415, 0.41211, 0.41047, 0.40918, 0.40793, 0.40618, 0.40378, 0.40109, 0.39828, 0.39519,\
        0.39170, 0.38849, 0.38546, 0.38237, 0.37907, 0.37543, 0.37141, 0.36708, 0.36249, 0.35769, 0.35269,\
        0.34750, 0.34209, 0.33645, 0.33055, 0.32437, 0.31788, 0.31107, 0.30391, 0.29639, 0.28850, 0.28022],\
        [0.20670, 0.21551, 0.22399, 0.23212, 0.23991, 0.24734, 0.25441, 0.26112, 0.26746, 0.27343, 0.27904,\
        0.28431, 0.28923, 0.29384, 0.29814, 0.30231, 0.30672, 0.31126, 0.31570, 0.31990, 0.32381, 0.32744,\
        0.33080, 0.33393, 0.33684, 0.33959, 0.34219, 0.34467, 0.34706, 0.34938, 0.35164, 0.35386, 0.35605,\
        0.35822, 0.36036, 0.36251, 0.36465, 0.36680, 0.36897, 0.37118, 0.37342, 0.37572, 0.37806, 0.38042,\
        0.38275, 0.38503, 0.38728, 0.38951, 0.39169, 0.39368, 0.39553, 0.39721, 0.39868, 0.40067, 0.40239,\
        0.40419, 0.40642, 0.40874, 0.41113, 0.41348, 0.41563, 0.41752, 0.41920, 0.42074, 0.42215, 0.42349,\
        0.42475, 0.42594, 0.42699, 0.42784, 0.42845, 0.42881, 0.42893, 0.42883, 0.42852, 0.42802, 0.42736,\
        0.42660, 0.42579, 0.42497, 0.42416, 0.42333, 0.42241, 0.42132, 0.41991, 0.41809, 0.41584, 0.41327,\
        0.41063, 0.40810, 0.40588, 0.40421, 0.40305, 0.40217, 0.40052, 0.39797, 0.39517, 0.39224, 0.38908,\
        0.38571, 0.38234, 0.37903, 0.37566, 0.37212, 0.36831, 0.36420, 0.35979, 0.35512, 0.35021, 0.34509,\
        0.33975, 0.33418, 0.32838, 0.32231, 0.31596, 0.30931, 0.30234, 0.29504, 0.28738, 0.27936, 0.27097],\
        [0.20109, 0.21006, 0.21871, 0.22701, 0.23497, 0.24258, 0.24985, 0.25675, 0.26330, 0.26949, 0.27533,\
        0.28081, 0.28593, 0.29070, 0.29515, 0.29947, 0.30420, 0.30907, 0.31368, 0.31797, 0.32191, 0.32551,\
        0.32882, 0.33187, 0.33470, 0.33736, 0.33986, 0.34225, 0.34454, 0.34675, 0.34890, 0.35101, 0.35308,\
        0.35513, 0.35717, 0.35920, 0.36123, 0.36327, 0.36533, 0.36741, 0.36954, 0.37169, 0.37388, 0.37607,\
        0.37825, 0.38040, 0.38251, 0.38459, 0.38659, 0.38847, 0.39022, 0.39185, 0.39346, 0.39528, 0.39683,\
        0.39866, 0.40081, 0.40307, 0.40551, 0.40795, 0.41010, 0.41184, 0.41338, 0.41484, 0.41618, 0.41746,\
        0.41875, 0.42002, 0.42115, 0.42207, 0.42270, 0.42307, 0.42317, 0.42305, 0.42271, 0.42217, 0.42145,\
        0.42062, 0.41976, 0.41894, 0.41820, 0.41750, 0.41676, 0.41588, 0.41468, 0.41295, 0.41060, 0.40780,\
        0.40494, 0.40225, 0.39984, 0.39810, 0.39689, 0.39589, 0.39424, 0.39178, 0.38895, 0.38592, 0.38269,\
        0.37931, 0.37584, 0.37233, 0.36873, 0.36497, 0.36099, 0.35674, 0.35223, 0.34745, 0.34244, 0.33719,\
        0.33171, 0.32600, 0.32003, 0.31381, 0.30730, 0.30049, 0.29337, 0.28593, 0.27813, 0.26999, 0.26148],\
        [0.19520, 0.20435, 0.21316, 0.22164, 0.22978, 0.23759, 0.24506, 0.25218, 0.25896, 0.26540, 0.27150,\
        0.27725, 0.28267, 0.28776, 0.29259, 0.29735, 0.30232, 0.30720, 0.31183, 0.31610, 0.31998, 0.32350,\
        0.32670, 0.32965, 0.33238, 0.33493, 0.33733, 0.33962, 0.34181, 0.34392, 0.34597, 0.34798, 0.34995,\
        0.35189, 0.35382, 0.35575, 0.35768, 0.35961, 0.36157, 0.36355, 0.36555, 0.36758, 0.36962, 0.37168,\
        0.37372, 0.37573, 0.37772, 0.37966, 0.38153, 0.38332, 0.38501, 0.38664, 0.38828, 0.38996, 0.39163,\
        0.39346, 0.39552, 0.39775, 0.40021, 0.40271, 0.40489, 0.40624, 0.40765, 0.40906, 0.41025, 0.41143,\
        0.41275, 0.41412, 0.41536, 0.41633, 0.41697, 0.41730, 0.41737, 0.41723, 0.41687, 0.41628, 0.41550,\
        0.41456, 0.41361, 0.41278, 0.41212, 0.41154, 0.41098, 0.41037, 0.40945, 0.40796, 0.40556, 0.40245,\
        0.39947, 0.39671, 0.39426, 0.39232, 0.39081, 0.38936, 0.38753, 0.38514, 0.38234, 0.37927, 0.37599,\
        0.37255, 0.36899, 0.36532, 0.36154, 0.35759, 0.35343, 0.34904, 0.34440, 0.33951, 0.33438, 0.32900,\
        0.32339, 0.31753, 0.31142, 0.30504, 0.29838, 0.29142, 0.28416, 0.27657, 0.26864, 0.26038, 0.25175],\
        [0.18903, 0.19834, 0.20733, 0.21599, 0.22433, 0.23234, 0.24002, 0.24738, 0.25441, 0.26111, 0.26750,\
        0.27358, 0.27936, 0.28488, 0.29020, 0.29544, 0.30056, 0.30539, 0.31002, 0.31420, 0.31793, 0.32130,\
        0.32436, 0.32718, 0.32980, 0.33224, 0.33455, 0.33674, 0.33884, 0.34086, 0.34282, 0.34473, 0.34661,\
        0.34847, 0.35030, 0.35214, 0.35397, 0.35581, 0.35767, 0.35954, 0.36144, 0.36335, 0.36527, 0.36720,\
        0.36912, 0.37102, 0.37289, 0.37472, 0.37649, 0.37821, 0.37987, 0.38149, 0.38313, 0.38479, 0.38653,\
        0.38839, 0.39045, 0.39269, 0.39513, 0.39768, 0.39986, 0.40050, 0.40202, 0.40344, 0.40434, 0.40537,\
        0.40672, 0.40825, 0.40961, 0.41060, 0.41120, 0.41146, 0.41148, 0.41132, 0.41096, 0.41035, 0.40950,\
        0.40846, 0.40735, 0.40649, 0.40594, 0.40547, 0.40498, 0.40468, 0.40401, 0.40284, 0.40068, 0.39731,\
        0.39415, 0.39129, 0.38873, 0.38655, 0.38464, 0.38279, 0.38069, 0.37823, 0.37540, 0.37230, 0.36897,\
        0.36546, 0.36180, 0.35800, 0.35406, 0.34994, 0.34562, 0.34108, 0.33631, 0.33129, 0.32603, 0.32053,\
        0.31478, 0.30878, 0.30253, 0.29600, 0.28919, 0.28209, 0.27469, 0.26696, 0.25891, 0.25052, 0.24178],\
        [0.18256, 0.19204, 0.20120, 0.21005, 0.21858, 0.22680, 0.23470, 0.24229, 0.24958, 0.25657, 0.26327,\
        0.26969, 0.27586, 0.28182, 0.28759, 0.29315, 0.29837, 0.30350, 0.30812, 0.31209, 0.31560, 0.31879,\
        0.32170, 0.32439, 0.32690, 0.32924, 0.33146, 0.33357, 0.33558, 0.33752, 0.33940, 0.34124, 0.34304,\
        0.34482, 0.34658, 0.34833, 0.35008, 0.35183, 0.35360, 0.35538, 0.35718, 0.35898, 0.36080, 0.36262,\
        0.36443, 0.36622, 0.36800, 0.36974, 0.37144, 0.37310, 0.37473, 0.37635, 0.37799, 0.37967, 0.38144,\
        0.38332, 0.38537, 0.38758, 0.38993, 0.39224, 0.39403, 0.39512, 0.39648, 0.39772, 0.39844, 0.39940,\
        0.40073, 0.40246, 0.40386, 0.40479, 0.40533, 0.40549, 0.40543, 0.40526, 0.40494, 0.40432, 0.40344,\
        0.40235, 0.40107, 0.40017, 0.39970, 0.39936, 0.39878, 0.39868, 0.39806, 0.39691, 0.39491, 0.39185,\
        0.38866, 0.38569, 0.38299, 0.38055, 0.37831, 0.37609, 0.37373, 0.37111, 0.36820, 0.36503, 0.36164,\
        0.35805, 0.35429, 0.35037, 0.34628, 0.34201, 0.33755, 0.33286, 0.32795, 0.32280, 0.31741, 0.31178,\
        0.30590, 0.29976, 0.29336, 0.28670, 0.27975, 0.27251, 0.26496, 0.25710, 0.24892, 0.24041, 0.23155],\
        [0.17578, 0.18543, 0.19477, 0.20380, 0.21252, 0.22094, 0.22906, 0.23689, 0.24443, 0.25169, 0.25869,\
        0.26545, 0.27199, 0.27834, 0.28451, 0.29052, 0.29636, 0.30167, 0.30582, 0.30948, 0.31279, 0.31581,\
        0.31860, 0.32119, 0.32360, 0.32587, 0.32801, 0.33005, 0.33200, 0.33388, 0.33570, 0.33747, 0.33921,\
        0.34092, 0.34261, 0.34429, 0.34597, 0.34765, 0.34934, 0.35104, 0.35274, 0.35446, 0.35618, 0.35791,\
        0.35963, 0.36133, 0.36302, 0.36469, 0.36633, 0.36795, 0.36956, 0.37117, 0.37281, 0.37450, 0.37627,\
        0.37815, 0.38015, 0.38226, 0.38442, 0.38645, 0.38812, 0.38945, 0.39068, 0.39177, 0.39265, 0.39366,\
        0.39498, 0.39657, 0.39788, 0.39874, 0.39924, 0.39931, 0.39919, 0.39901, 0.39873, 0.39810, 0.39721,\
        0.39614, 0.39495, 0.39403, 0.39348, 0.39304, 0.39259, 0.39221, 0.39151, 0.39025, 0.38828, 0.38560,\
        0.38262, 0.37967, 0.37686, 0.37423, 0.37171, 0.36920, 0.36659, 0.36379, 0.36076, 0.35749, 0.35401,\
        0.35033, 0.34647, 0.34243, 0.33821, 0.33380, 0.32919, 0.32437, 0.31932, 0.31404, 0.30851, 0.30275,\
        0.29673, 0.29046, 0.28393, 0.27712, 0.27003, 0.26266, 0.25498, 0.24699, 0.23868, 0.23005, 0.22108]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=25 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax25vt0
    zgrid_H_Hmax_vmax25vt0=[[0.58965, 0.59097, 0.59230, 0.59361, 0.59489, 0.59611, 0.59721, 0.59809, 0.59859, 0.59871, 0.59865,\
        0.59853, 0.59841, 0.59830, 0.59826, 0.59837, 0.59875, 0.59955, 0.60088, 0.60280, 0.60528, 0.60821,\
        0.61143, 0.61480, 0.61818, 0.62146, 0.62457, 0.62750, 0.63022, 0.63276, 0.63514, 0.63739, 0.63954,\
        0.64160, 0.64362, 0.64561, 0.64758, 0.64955, 0.65153, 0.65350, 0.65547, 0.65743, 0.65936, 0.66126,\
        0.66311, 0.66487, 0.66646, 0.66777, 0.66869, 0.66917, 0.66922, 0.66886, 0.66810, 0.66696, 0.66545,\
        0.66365, 0.66168, 0.65964, 0.65762, 0.65565, 0.65370, 0.65175, 0.64974, 0.64765, 0.64542, 0.64302,\
        0.64044, 0.63766, 0.63467, 0.63150, 0.62820, 0.62482, 0.62146, 0.61823, 0.61524, 0.61260, 0.61035,\
        0.60848, 0.60689, 0.60550, 0.60423, 0.60264, 0.59940, 0.59396, 0.58722, 0.57976, 0.57192, 0.56395,\
        0.55610, 0.54862, 0.54172, 0.53563, 0.53046, 0.52626, 0.52295, 0.52035, 0.51823, 0.51640, 0.51472,\
        0.51320, 0.51186, 0.51060, 0.50875, 0.50422, 0.49491, 0.48277, 0.46987, 0.45744, 0.44630, 0.43683,\
        0.42912, 0.42304, 0.41837, 0.41486, 0.41221, 0.41015, 0.40847, 0.40703, 0.40577, 0.40460, 0.40350],\
        [0.59139, 0.59258, 0.59377, 0.59497, 0.59617, 0.59732, 0.59840, 0.59929, 0.59976, 0.59971, 0.59959,\
        0.59949, 0.59940, 0.59931, 0.59926, 0.59934, 0.59971, 0.60057, 0.60210, 0.60438, 0.60734, 0.61083,\
        0.61462, 0.61850, 0.62230, 0.62591, 0.62925, 0.63232, 0.63512, 0.63770, 0.64009, 0.64233, 0.64447,\
        0.64652, 0.64853, 0.65052, 0.65250, 0.65450, 0.65652, 0.65855, 0.66058, 0.66260, 0.66460, 0.66658,\
        0.66857, 0.67052, 0.67234, 0.67386, 0.67492, 0.67546, 0.67552, 0.67514, 0.67435, 0.67312, 0.67144,\
        0.66941, 0.66721, 0.66500, 0.66287, 0.66084, 0.65886, 0.65689, 0.65484, 0.65268, 0.65034, 0.64779,\
        0.64499, 0.64193, 0.63859, 0.63499, 0.63117, 0.62719, 0.62317, 0.61924, 0.61556, 0.61228, 0.60951,\
        0.60722, 0.60527, 0.60349, 0.60199, 0.60055, 0.59753, 0.59178, 0.58478, 0.57704, 0.56884, 0.56047,\
        0.55215, 0.54416, 0.53678, 0.53027, 0.52484, 0.52057, 0.51732, 0.51482, 0.51277, 0.51088, 0.50894,\
        0.50698, 0.50519, 0.50366, 0.50190, 0.49796, 0.48751, 0.47441, 0.46078, 0.44797, 0.43691, 0.42799,\
        0.42112, 0.41602, 0.41242, 0.40999, 0.40834, 0.40715, 0.40620, 0.40540, 0.40468, 0.40400, 0.40335],\
        [0.59286, 0.59387, 0.59490, 0.59594, 0.59699, 0.59801, 0.59896, 0.59974, 0.60011, 0.60002, 0.59998,\
        0.59998, 0.59998, 0.59997, 0.59996, 0.60002, 0.60037, 0.60129, 0.60307, 0.60583, 0.60946, 0.61367,\
        0.61815, 0.62262, 0.62688, 0.63079, 0.63433, 0.63750, 0.64033, 0.64290, 0.64526, 0.64747, 0.64956,\
        0.65159, 0.65357, 0.65555, 0.65754, 0.65956, 0.66162, 0.66371, 0.66581, 0.66788, 0.66992, 0.67195,\
        0.67403, 0.67619, 0.67831, 0.68013, 0.68136, 0.68194, 0.68198, 0.68161, 0.68084, 0.67957, 0.67773,\
        0.67542, 0.67294, 0.67057, 0.66838, 0.66636, 0.66442, 0.66247, 0.66043, 0.65821, 0.65579, 0.65310,\
        0.65011, 0.64678, 0.64310, 0.63905, 0.63468, 0.63005, 0.62527, 0.62051, 0.61597, 0.61189, 0.60846,\
        0.60570, 0.60337, 0.60108, 0.59901, 0.59702, 0.59377, 0.58846, 0.58169, 0.57398, 0.56566, 0.55701,\
        0.54830, 0.53981, 0.53189, 0.52491, 0.51918, 0.51485, 0.51173, 0.50941, 0.50748, 0.50557, 0.50331,\
        0.50073, 0.49829, 0.49586, 0.49282, 0.48714, 0.47700, 0.46392, 0.45010, 0.43725, 0.42663, 0.41872,\
        0.41309, 0.40920, 0.40681, 0.40561, 0.40503, 0.40468, 0.40441, 0.40418, 0.40395, 0.40373, 0.40350],\
        [0.59408, 0.59489, 0.59571, 0.59655, 0.59739, 0.59821, 0.59895, 0.59951, 0.59978, 0.59982, 0.59986,\
        0.59997, 0.60012, 0.60025, 0.60034, 0.60041, 0.60072, 0.60168, 0.60380, 0.60724, 0.61176, 0.61690,\
        0.62221, 0.62733, 0.63204, 0.63623, 0.63989, 0.64307, 0.64587, 0.64837, 0.65065, 0.65278, 0.65482,\
        0.65680, 0.65875, 0.66071, 0.66271, 0.66475, 0.66686, 0.66902, 0.67120, 0.67334, 0.67539, 0.67738,\
        0.67946, 0.68178, 0.68429, 0.68656, 0.68800, 0.68855, 0.68856, 0.68821, 0.68755, 0.68637, 0.68434,\
        0.68163, 0.67884, 0.67637, 0.67423, 0.67230, 0.67047, 0.66860, 0.66658, 0.66434, 0.66184, 0.65904,\
        0.65588, 0.65232, 0.64830, 0.64382, 0.63889, 0.63358, 0.62798, 0.62227, 0.61670, 0.61159, 0.60728,\
        0.60395, 0.60131, 0.59871, 0.59609, 0.59344, 0.58990, 0.58491, 0.57852, 0.57104, 0.56277, 0.55397,\
        0.54494, 0.53597, 0.52747, 0.51988, 0.51373, 0.50936, 0.50650, 0.50443, 0.50262, 0.50074, 0.49828,\
        0.49528, 0.49206, 0.48839, 0.48352, 0.47622, 0.46574, 0.45278, 0.43894, 0.42607, 0.41608, 0.40974,\
        0.40580, 0.40321, 0.40209, 0.40219, 0.40261, 0.40295, 0.40322, 0.40345, 0.40365, 0.40383, 0.40397],\
        [0.59509, 0.59566, 0.59625, 0.59686, 0.59747, 0.59805, 0.59856, 0.59892, 0.59912, 0.59918, 0.59926,\
        0.59942, 0.59969, 0.60005, 0.60032, 0.60047, 0.60074, 0.60172, 0.60430, 0.60876, 0.61450, 0.62080,\
        0.62704, 0.63282, 0.63793, 0.64230, 0.64597, 0.64907, 0.65172, 0.65407, 0.65623, 0.65827, 0.66023,\
        0.66216, 0.66408, 0.66602, 0.66801, 0.67007, 0.67223, 0.67449, 0.67680, 0.67904, 0.68108, 0.68294,\
        0.68485, 0.68717, 0.69010, 0.69308, 0.69478, 0.69520, 0.69523, 0.69494, 0.69443, 0.69353, 0.69129,\
        0.68796, 0.68487, 0.68247, 0.68053, 0.67879, 0.67714, 0.67540, 0.67341, 0.67113, 0.66856, 0.66568,\
        0.66239, 0.65862, 0.65431, 0.64943, 0.64398, 0.63800, 0.63157, 0.62486, 0.61811, 0.61170, 0.60616,\
        0.60200, 0.59884, 0.59600, 0.59327, 0.59037, 0.58675, 0.58199, 0.57596, 0.56877, 0.56063, 0.55180,\
        0.54253, 0.53315, 0.52403, 0.51569, 0.50887, 0.50442, 0.50199, 0.50019, 0.49846, 0.49650, 0.49393,\
        0.49063, 0.48664, 0.48172, 0.47533, 0.46679, 0.45580, 0.44282, 0.42897, 0.41590, 0.40623, 0.40208,\
        0.40014, 0.39894, 0.39905, 0.40024, 0.40128, 0.40202, 0.40265, 0.40323, 0.40379, 0.40432, 0.40481],\
        [0.59592, 0.59625, 0.59659, 0.59695, 0.59732, 0.59767, 0.59797, 0.59817, 0.59827, 0.59830, 0.59834,\
        0.59849, 0.59878, 0.59921, 0.59968, 0.60004, 0.60043, 0.60133, 0.60474, 0.61078, 0.61812, 0.62573,\
        0.63293, 0.63932, 0.64471, 0.64910, 0.65261, 0.65546, 0.65785, 0.65997, 0.66196, 0.66388, 0.66578,\
        0.66766, 0.66954, 0.67147, 0.67346, 0.67555, 0.67777, 0.68013, 0.68263, 0.68508, 0.68718, 0.68881,\
        0.69032, 0.69232, 0.69547, 0.69926, 0.70139, 0.70190, 0.70212, 0.70193, 0.70151, 0.70082, 0.69823,\
        0.69424, 0.69108, 0.68914, 0.68756, 0.68599, 0.68458, 0.68301, 0.68100, 0.67864, 0.67601, 0.67306,\
        0.66968, 0.66575, 0.66121, 0.65600, 0.65010, 0.64352, 0.63633, 0.62867, 0.62074, 0.61287, 0.60564,\
        0.60014, 0.59658, 0.59384, 0.59121, 0.58833, 0.58481, 0.58028, 0.57456, 0.56766, 0.55971, 0.55093,\
        0.54156, 0.53189, 0.52228, 0.51319, 0.50538, 0.50043, 0.49856, 0.49706, 0.49543, 0.49341, 0.49070,\
        0.48706, 0.48239, 0.47646, 0.46895, 0.45957, 0.44827, 0.43546, 0.42203, 0.40930, 0.39960, 0.39699,\
        0.39647, 0.39669, 0.39781, 0.39949, 0.40076, 0.40174, 0.40262, 0.40348, 0.40435, 0.40520, 0.40601],\
        [0.59664, 0.59670, 0.59679, 0.59690, 0.59705, 0.59720, 0.59734, 0.59742, 0.59743, 0.59740, 0.59740,\
        0.59749, 0.59773, 0.59814, 0.59861, 0.59906, 0.59954, 0.60039, 0.60596, 0.61420, 0.62325, 0.63213,\
        0.64018, 0.64701, 0.65246, 0.65663, 0.65976, 0.66217, 0.66416, 0.66597, 0.66776, 0.66958, 0.67143,\
        0.67329, 0.67516, 0.67709, 0.67911, 0.68123, 0.68349, 0.68596, 0.68867, 0.69150, 0.69386, 0.69525,\
        0.69637, 0.69786, 0.70064, 0.70475, 0.70753, 0.70884, 0.70941, 0.70944, 0.70898, 0.70777, 0.70494,\
        0.70110, 0.69796, 0.69698, 0.69566, 0.69404, 0.69298, 0.69160, 0.68941, 0.68688, 0.68419, 0.68122,\
        0.67778, 0.67373, 0.66901, 0.66355, 0.65731, 0.65028, 0.64250, 0.63408, 0.62521, 0.61618, 0.60750,\
        0.60033, 0.59652, 0.59354, 0.59079, 0.58792, 0.58452, 0.58020, 0.57475, 0.56810, 0.56036, 0.55171,\
        0.54239, 0.53266, 0.52287, 0.51347, 0.50517, 0.49956, 0.49747, 0.49584, 0.49410, 0.49196, 0.48904,\
        0.48502, 0.47972, 0.47301, 0.46475, 0.45488, 0.44355, 0.43125, 0.41887, 0.40781, 0.40006, 0.39621,\
        0.39538, 0.39604, 0.39762, 0.39950, 0.40083, 0.40195, 0.40302, 0.40414, 0.40529, 0.40646, 0.40759],\
        [0.59728, 0.59708, 0.59691, 0.59679, 0.59674, 0.59674, 0.59677, 0.59680, 0.59679, 0.59675, 0.59673,\
        0.59683, 0.59713, 0.59766, 0.59839, 0.59917, 0.60016, 0.60306, 0.61039, 0.62013, 0.63046, 0.64032,\
        0.64898, 0.65597, 0.66119, 0.66484, 0.66730, 0.66905, 0.67050, 0.67193, 0.67353, 0.67529, 0.67715,\
        0.67904, 0.68093, 0.68290, 0.68499, 0.68717, 0.68948, 0.69203, 0.69487, 0.69807, 0.70099, 0.70240,\
        0.70359, 0.70519, 0.70782, 0.71140, 0.71438, 0.71627, 0.71724, 0.71748, 0.71698, 0.71556, 0.71303,\
        0.70988, 0.70732, 0.70627, 0.70497, 0.70343, 0.70251, 0.70114, 0.69861, 0.69590, 0.69315, 0.69018,\
        0.68668, 0.68250, 0.67766, 0.67207, 0.66563, 0.65832, 0.65016, 0.64127, 0.63186, 0.62229, 0.61319,\
        0.60562, 0.60015, 0.59599, 0.59266, 0.58960, 0.58621, 0.58203, 0.57676, 0.57030, 0.56275, 0.55430,\
        0.54518, 0.53565, 0.52606, 0.51688, 0.50887, 0.50317, 0.49975, 0.49711, 0.49489, 0.49248, 0.48925,\
        0.48478, 0.47890, 0.47155, 0.46278, 0.45267, 0.44151, 0.42985, 0.41858, 0.40884, 0.40172, 0.39767,\
        0.39622, 0.39656, 0.39802, 0.39985, 0.40125, 0.40245, 0.40373, 0.40512, 0.40659, 0.40809, 0.40955],\
        [0.59789, 0.59742, 0.59700, 0.59667, 0.59645, 0.59636, 0.59637, 0.59644, 0.59653, 0.59659, 0.59667,\
        0.59693, 0.59751, 0.59852, 0.60005, 0.60211, 0.60509, 0.61026, 0.61846, 0.62879, 0.63980, 0.65028,\
        0.65926, 0.66613, 0.67076, 0.67352, 0.67503, 0.67588, 0.67664, 0.67767, 0.67911, 0.68093, 0.68293,\
        0.68492, 0.68686, 0.68890, 0.69116, 0.69350, 0.69589, 0.69859, 0.70157, 0.70492, 0.70799, 0.71015,\
        0.71194, 0.71397, 0.71661, 0.71969, 0.72250, 0.72458, 0.72582, 0.72625, 0.72588, 0.72467, 0.72269,\
        0.72037, 0.71839, 0.71703, 0.71575, 0.71440, 0.71309, 0.71131, 0.70877, 0.70591, 0.70296, 0.69990,\
        0.69622, 0.69190, 0.68702, 0.68141, 0.67491, 0.66750, 0.65920, 0.65012, 0.64052, 0.63080, 0.62156,\
        0.61344, 0.60674, 0.60131, 0.59709, 0.59360, 0.59004, 0.58587, 0.58066, 0.57428, 0.56687, 0.55867,\
        0.54988, 0.54073, 0.53155, 0.52281, 0.51511, 0.50903, 0.50444, 0.50080, 0.49793, 0.49508, 0.49144,\
        0.48643, 0.47993, 0.47204, 0.46288, 0.45265, 0.44169, 0.43057, 0.42004, 0.41099, 0.40417, 0.39990,\
        0.39786, 0.39762, 0.39862, 0.40013, 0.40159, 0.40305, 0.40465, 0.40640, 0.40825, 0.41012, 0.41193],\
        [0.59853, 0.59778, 0.59710, 0.59657, 0.59622, 0.59609, 0.59618, 0.59643, 0.59677, 0.59709, 0.59745,\
        0.59808, 0.59926, 0.60119, 0.60393, 0.60759, 0.61253, 0.61940, 0.62849, 0.63927, 0.65068, 0.66155,\
        0.67071, 0.67718, 0.68082, 0.68235, 0.68261, 0.68237, 0.68235, 0.68297, 0.68436, 0.68639, 0.68872,\
        0.69095, 0.69295, 0.69504, 0.69762, 0.70029, 0.70292, 0.70587, 0.70914, 0.71257, 0.71581, 0.71858,\
        0.72104, 0.72354, 0.72627, 0.72914, 0.73178, 0.73387, 0.73525, 0.73588, 0.73575, 0.73491, 0.73351,\
        0.73185, 0.73029, 0.72895, 0.72766, 0.72624, 0.72457, 0.72244, 0.71975, 0.71670, 0.71347, 0.71003,\
        0.70612, 0.70170, 0.69687, 0.69131, 0.68486, 0.67753, 0.66930, 0.66024, 0.65065, 0.64095, 0.63162,\
        0.62309, 0.61563, 0.60930, 0.60414, 0.59987, 0.59587, 0.59159, 0.58628, 0.57980, 0.57249, 0.56458,\
        0.55621, 0.54754, 0.53884, 0.53048, 0.52291, 0.51645, 0.51113, 0.50675, 0.50307, 0.49960, 0.49546,\
        0.48973, 0.48258, 0.47421, 0.46477, 0.45445, 0.44363, 0.43280, 0.42263, 0.41379, 0.40686, 0.40215,\
        0.39954, 0.39869, 0.39929, 0.40050, 0.40201, 0.40379, 0.40582, 0.40803, 0.41032, 0.41259, 0.41475],\
        [0.59921, 0.59816, 0.59722, 0.59650, 0.59607, 0.59595, 0.59618, 0.59675, 0.59755, 0.59837, 0.59917,\
        0.60048, 0.60256, 0.60560, 0.60962, 0.61476, 0.62127, 0.62942, 0.63928, 0.65044, 0.66212, 0.67328,\
        0.68262, 0.68855, 0.69073, 0.69083, 0.68966, 0.68821, 0.68737, 0.68764, 0.68913, 0.69156, 0.69445,\
        0.69711, 0.69926, 0.70146, 0.70433, 0.70739, 0.71049, 0.71380, 0.71734, 0.72098, 0.72450, 0.72773,\
        0.73074, 0.73366, 0.73658, 0.73941, 0.74195, 0.74402, 0.74549, 0.74630, 0.74644, 0.74597, 0.74506,\
        0.74391, 0.74270, 0.74150, 0.74020, 0.73864, 0.73667, 0.73418, 0.73118, 0.72781, 0.72419, 0.72035,\
        0.71619, 0.71168, 0.70680, 0.70133, 0.69503, 0.68798, 0.68000, 0.67108, 0.66160, 0.65201, 0.64268,\
        0.63391, 0.62595, 0.61896, 0.61299, 0.60788, 0.60330, 0.59875, 0.59316, 0.58640, 0.57923, 0.57170,\
        0.56383, 0.55567, 0.54740, 0.53930, 0.53173, 0.52495, 0.51906, 0.51400, 0.50956, 0.50534, 0.50053,\
        0.49406, 0.48642, 0.47773, 0.46809, 0.45770, 0.44691, 0.43616, 0.42601, 0.41701, 0.40964, 0.40421,\
        0.40079, 0.39946, 0.39985, 0.40093, 0.40259, 0.40479, 0.40735, 0.41010, 0.41288, 0.41557, 0.41809],\
        [0.59988, 0.59852, 0.59736, 0.59649, 0.59598, 0.59592, 0.59634, 0.59727, 0.59867, 0.60026, 0.60179,\
        0.60401, 0.60712, 0.61126, 0.61648, 0.62289, 0.63060, 0.63970, 0.65008, 0.66142, 0.67311, 0.68426,\
        0.69365, 0.69914, 0.69942, 0.69844, 0.69576, 0.69321, 0.69161, 0.69163, 0.69340, 0.69633, 0.69982,\
        0.70302, 0.70568, 0.70838, 0.71148, 0.71488, 0.71843, 0.72216, 0.72605, 0.73000, 0.73385, 0.73751,\
        0.74096, 0.74425, 0.74738, 0.75027, 0.75278, 0.75484, 0.75638, 0.75734, 0.75771, 0.75754, 0.75700,\
        0.75621, 0.75530, 0.75425, 0.75299, 0.75130, 0.74903, 0.74612, 0.74268, 0.73887, 0.73481, 0.73056,\
        0.72610, 0.72140, 0.71640, 0.71095, 0.70493, 0.69832, 0.69072, 0.68195, 0.67268, 0.66334, 0.65413,\
        0.64524, 0.63689, 0.62931, 0.62261, 0.61675, 0.61147, 0.60630, 0.60019, 0.59351, 0.58670, 0.57968,\
        0.57237, 0.56471, 0.55678, 0.54882, 0.54113, 0.53399, 0.52754, 0.52178, 0.51653, 0.51139, 0.50570,\
        0.49892, 0.49110, 0.48225, 0.47252, 0.46207, 0.45122, 0.44038, 0.43005, 0.42067, 0.41267, 0.40634,\
        0.40198, 0.39994, 0.40004, 0.40125, 0.40341, 0.40625, 0.40947, 0.41280, 0.41608, 0.41919, 0.42203],\
        [0.60048, 0.59890, 0.59755, 0.59654, 0.59599, 0.59599, 0.59662, 0.59790, 0.59984, 0.60222, 0.60490,\
        0.60822, 0.61244, 0.61767, 0.62400, 0.63146, 0.64009, 0.64983, 0.66047, 0.67167, 0.68294, 0.69349,\
        0.70193, 0.70642, 0.70655, 0.70443, 0.70081, 0.69777, 0.69520, 0.69509, 0.69748, 0.70099, 0.70488,\
        0.70865, 0.71208, 0.71544, 0.71898, 0.72275, 0.72672, 0.73087, 0.73515, 0.73948, 0.74371, 0.74775,\
        0.75158, 0.75520, 0.75858, 0.76157, 0.76407, 0.76610, 0.76770, 0.76880, 0.76932, 0.76933, 0.76900,\
        0.76847, 0.76774, 0.76683, 0.76563, 0.76387, 0.76129, 0.75788, 0.75389, 0.74955, 0.74501, 0.74034,\
        0.73554, 0.73059, 0.72542, 0.71995, 0.71411, 0.70779, 0.70058, 0.69203, 0.68322, 0.67437, 0.66544,\
        0.65648, 0.64776, 0.63962, 0.63227, 0.62568, 0.61966, 0.61375, 0.60751, 0.60111, 0.59463, 0.58821,\
        0.58150, 0.57429, 0.56659, 0.55861, 0.55067, 0.54309, 0.53606, 0.52960, 0.52356, 0.51761, 0.51130,\
        0.50428, 0.49634, 0.48749, 0.47779, 0.46729, 0.45632, 0.44531, 0.43470, 0.42490, 0.41625, 0.40903,\
        0.40355, 0.40031, 0.40010, 0.40186, 0.40489, 0.40855, 0.41246, 0.41636, 0.42009, 0.42354, 0.42664],\
        [0.60131, 0.59946, 0.59788, 0.59671, 0.59611, 0.59620, 0.59708, 0.59877, 0.60130, 0.60456, 0.60838,\
        0.61288, 0.61822, 0.62451, 0.63182, 0.64016, 0.64950, 0.65968, 0.67036, 0.68111, 0.69156, 0.70108,\
        0.70830, 0.71206, 0.71228, 0.70995, 0.70623, 0.70219, 0.69885, 0.69866, 0.70199, 0.70609, 0.71045,\
        0.71474, 0.71881, 0.72276, 0.72677, 0.73093, 0.73529, 0.73985, 0.74456, 0.74931, 0.75394, 0.75833,\
        0.76246, 0.76638, 0.77004, 0.77320, 0.77564, 0.77755, 0.77922, 0.78049, 0.78105, 0.78105, 0.78080,\
        0.78038, 0.77971, 0.77881, 0.77770, 0.77598, 0.77304, 0.76901, 0.76439, 0.75952, 0.75452, 0.74945,\
        0.74430, 0.73906, 0.73367, 0.72808, 0.72218, 0.71583, 0.70872, 0.70083, 0.69280, 0.68471, 0.67616,\
        0.66710, 0.65798, 0.64933, 0.64141, 0.63425, 0.62768, 0.62140, 0.61518, 0.60899, 0.60291, 0.59705,\
        0.59085, 0.58399, 0.57640, 0.56823, 0.55992, 0.55184, 0.54423, 0.53713, 0.53042, 0.52384, 0.51708,\
        0.50984, 0.50187, 0.49326, 0.48366, 0.47307, 0.46196, 0.45077, 0.43994, 0.42982, 0.42073, 0.41290,\
        0.40657, 0.40211, 0.40104, 0.40394, 0.40786, 0.41221, 0.41666, 0.42099, 0.42505, 0.42873, 0.43198],\
        [0.60239, 0.60022, 0.59840, 0.59701, 0.59633, 0.59659, 0.59783, 0.60006, 0.60332, 0.60746, 0.61233,\
        0.61793, 0.62434, 0.63160, 0.63976, 0.64881, 0.65869, 0.66922, 0.67993, 0.69014, 0.69956, 0.70812,\
        0.71444, 0.71777, 0.71808, 0.71598, 0.71239, 0.70833, 0.70541, 0.70518, 0.70814, 0.71244, 0.71707,\
        0.72170, 0.72618, 0.73055, 0.73492, 0.73940, 0.74410, 0.74903, 0.75416, 0.75937, 0.76443, 0.76913,\
        0.77344, 0.77758, 0.78160, 0.78507, 0.78737, 0.78896, 0.79073, 0.79227, 0.79269, 0.79248, 0.79215,\
        0.79174, 0.79088, 0.78972, 0.78865, 0.78726, 0.78381, 0.77899, 0.77380, 0.76850, 0.76313, 0.75770,\
        0.75222, 0.74667, 0.74103, 0.73522, 0.72917, 0.72275, 0.71586, 0.70859, 0.70131, 0.69402, 0.68589,\
        0.67655, 0.66698, 0.65793, 0.64965, 0.64216, 0.63531, 0.62891, 0.62278, 0.61688, 0.61120, 0.60566,\
        0.59985, 0.59332, 0.58575, 0.57723, 0.56843, 0.55986, 0.55174, 0.54411, 0.53687, 0.52983, 0.52276,\
        0.51545, 0.50772, 0.49943, 0.48973, 0.47906, 0.46785, 0.45658, 0.44566, 0.43548, 0.42630, 0.41839,\
        0.41204, 0.40783, 0.40683, 0.40912, 0.41306, 0.41762, 0.42230, 0.42683, 0.43104, 0.43480, 0.43807],\
        [0.60376, 0.60127, 0.59913, 0.59739, 0.59659, 0.59709, 0.59891, 0.60184, 0.60590, 0.61094, 0.61681,\
        0.62342, 0.63078, 0.63889, 0.64776, 0.65734, 0.66760, 0.67842, 0.68936, 0.69935, 0.70808, 0.71566,\
        0.72128, 0.72432, 0.72477, 0.72307, 0.72003, 0.71672, 0.71446, 0.71435, 0.71663, 0.72047, 0.72500,\
        0.72971, 0.73438, 0.73895, 0.74350, 0.74817, 0.75308, 0.75831, 0.76384, 0.76954, 0.77511, 0.78012,\
        0.78440, 0.78860, 0.79298, 0.79697, 0.79913, 0.80030, 0.80214, 0.80376, 0.80402, 0.80357, 0.80300,\
        0.80233, 0.80099, 0.79921, 0.79753, 0.79723, 0.79261, 0.78721, 0.78183, 0.77634, 0.77072, 0.76500,\
        0.75922, 0.75339, 0.74748, 0.74144, 0.73519, 0.72868, 0.72191, 0.71503, 0.70828, 0.70172, 0.69386,\
        0.68394, 0.67416, 0.66504, 0.65671, 0.64918, 0.64231, 0.63596, 0.62999, 0.62430, 0.61882, 0.61342,\
        0.60781, 0.60163, 0.59401, 0.58501, 0.57580, 0.56686, 0.55837, 0.55035, 0.54271, 0.53533, 0.52804,\
        0.52067, 0.51304, 0.50478, 0.49533, 0.48483, 0.47366, 0.46244, 0.45168, 0.44171, 0.43284, 0.42533,\
        0.41955, 0.41598, 0.41514, 0.41692, 0.42043, 0.42480, 0.42941, 0.43391, 0.43805, 0.44173, 0.44487],\
        [0.60542, 0.60245, 0.59983, 0.59772, 0.59679, 0.59757, 0.60009, 0.60397, 0.60901, 0.61504, 0.62188,\
        0.62940, 0.63759, 0.64644, 0.65588, 0.66582, 0.67621, 0.68710, 0.69822, 0.70833, 0.71684, 0.72391,\
        0.72910, 0.73202, 0.73265, 0.73138, 0.72895, 0.72634, 0.72461, 0.72458, 0.72647, 0.72986, 0.73415,\
        0.73879, 0.74345, 0.74802, 0.75256, 0.75723, 0.76220, 0.76760, 0.77346, 0.77962, 0.78580, 0.79135,\
        0.79537, 0.79945, 0.80403, 0.80809, 0.81064, 0.81218, 0.81367, 0.81479, 0.81497, 0.81438, 0.81338,\
        0.81201, 0.81005, 0.80757, 0.80519, 0.80315, 0.79853, 0.79365, 0.78857, 0.78309, 0.77731, 0.77135,\
        0.76532, 0.75923, 0.75308, 0.74680, 0.74033, 0.73366, 0.72679, 0.71983, 0.71290, 0.70583, 0.69756,\
        0.68843, 0.67925, 0.67053, 0.66253, 0.65524, 0.64858, 0.64241, 0.63659, 0.63102, 0.62558, 0.62009,\
        0.61432, 0.60790, 0.60013, 0.59103, 0.58174, 0.57269, 0.56402, 0.55575, 0.54784, 0.54019, 0.53268,\
        0.52517, 0.51747, 0.50931, 0.50032, 0.49004, 0.47900, 0.46806, 0.45771, 0.44826, 0.44000, 0.43319,\
        0.42817, 0.42525, 0.42465, 0.42621, 0.42936, 0.43341, 0.43778, 0.44206, 0.44599, 0.44941, 0.45229],\
        [0.60749, 0.60386, 0.60065, 0.59810, 0.59697, 0.59812, 0.60151, 0.60658, 0.61282, 0.61991, 0.62766,\
        0.63596, 0.64483, 0.65429, 0.66426, 0.67450, 0.68480, 0.69544, 0.70657, 0.71690, 0.72570, 0.73278,\
        0.73789, 0.74085, 0.74168, 0.74075, 0.73875, 0.73658, 0.73517, 0.73520, 0.73691, 0.74008, 0.74422,\
        0.74879, 0.75337, 0.75777, 0.76209, 0.76658, 0.77144, 0.77684, 0.78290, 0.78940, 0.79603, 0.80222,\
        0.80659, 0.81070, 0.81515, 0.81906, 0.82191, 0.82380, 0.82511, 0.82578, 0.82564, 0.82473, 0.82320,\
        0.82113, 0.81850, 0.81544, 0.81220, 0.80856, 0.80403, 0.79938, 0.79450, 0.78893, 0.78295, 0.77679,\
        0.77055, 0.76427, 0.75793, 0.75144, 0.74476, 0.73782, 0.73065, 0.72325, 0.71564, 0.70765, 0.69933,\
        0.69111, 0.68269, 0.67467, 0.66724, 0.66043, 0.65413, 0.64822, 0.64258, 0.63706, 0.63152, 0.62576,\
        0.61955, 0.61255, 0.60443, 0.59551, 0.58642, 0.57746, 0.56876, 0.56035, 0.55223, 0.54434, 0.53660,\
        0.52889, 0.52103, 0.51285, 0.50407, 0.49407, 0.48349, 0.47316, 0.46351, 0.45482, 0.44737, 0.44141,\
        0.43720, 0.43493, 0.43467, 0.43624, 0.43920, 0.44299, 0.44708, 0.45105, 0.45463, 0.45768, 0.46016],\
        [0.61031, 0.60594, 0.60202, 0.59883, 0.59737, 0.59915, 0.60380, 0.61024, 0.61772, 0.62582, 0.63434,\
        0.64323, 0.65256, 0.66245, 0.67294, 0.68372, 0.69407, 0.70444, 0.71554, 0.72598, 0.73502, 0.74230,\
        0.74757, 0.75069, 0.75169, 0.75092, 0.74907, 0.74704, 0.74571, 0.74577, 0.74747, 0.75066, 0.75488,\
        0.75953, 0.76406, 0.76818, 0.77207, 0.77624, 0.78088, 0.78605, 0.79220, 0.79899, 0.80574, 0.81208,\
        0.81732, 0.82194, 0.82631, 0.83014, 0.83310, 0.83512, 0.83629, 0.83660, 0.83604, 0.83465, 0.83253,\
        0.82979, 0.82650, 0.82278, 0.81875, 0.81436, 0.80964, 0.80483, 0.79967, 0.79389, 0.78771, 0.78136,\
        0.77500, 0.76863, 0.76219, 0.75557, 0.74869, 0.74149, 0.73393, 0.72600, 0.71773, 0.70931, 0.70129,\
        0.69296, 0.68513, 0.67791, 0.67122, 0.66500, 0.65915, 0.65356, 0.64808, 0.64258, 0.63688, 0.63080,\
        0.62412, 0.61662, 0.60825, 0.59931, 0.59032, 0.58150, 0.57281, 0.56429, 0.55597, 0.54787, 0.53988,\
        0.53191, 0.52381, 0.51539, 0.50642, 0.49688, 0.48714, 0.47769, 0.46893, 0.46116, 0.45463, 0.44957,\
        0.44617, 0.44455, 0.44471, 0.44645, 0.44940, 0.45306, 0.45692, 0.46056, 0.46372, 0.46629, 0.46826],\
        [0.61428, 0.60920, 0.60440, 0.60035, 0.59854, 0.60172, 0.60787, 0.61561, 0.62416, 0.63309, 0.64216,\
        0.65135, 0.66082, 0.67083, 0.68165, 0.69323, 0.70418, 0.71465, 0.72542, 0.73578, 0.74496, 0.75247,\
        0.75800, 0.76134, 0.76246, 0.76164, 0.75961, 0.75736, 0.75589, 0.75592, 0.75776, 0.76123, 0.76582,\
        0.77081, 0.77547, 0.77924, 0.78239, 0.78618, 0.79084, 0.79551, 0.80152, 0.80859, 0.81549, 0.82196,\
        0.82770, 0.83277, 0.83729, 0.84116, 0.84416, 0.84618, 0.84717, 0.84713, 0.84610, 0.84414, 0.84139,\
        0.83797, 0.83401, 0.82962, 0.82491, 0.81993, 0.81475, 0.80945, 0.80391, 0.79799, 0.79158, 0.78512,\
        0.77879, 0.77248, 0.76606, 0.75941, 0.75243, 0.74503, 0.73715, 0.72877, 0.71993, 0.71077, 0.70160,\
        0.69410, 0.68730, 0.68090, 0.67492, 0.66929, 0.66391, 0.65863, 0.65332, 0.64782, 0.64196, 0.63559,\
        0.62853, 0.62068, 0.61205, 0.60291, 0.59396, 0.58525, 0.57644, 0.56772, 0.55922, 0.55091, 0.54270,\
        0.53449, 0.52612, 0.51749, 0.50852, 0.49935, 0.49030, 0.48176, 0.47398, 0.46717, 0.46157, 0.45737,\
        0.45474, 0.45376, 0.45438, 0.45641, 0.45950, 0.46317, 0.46690, 0.47023, 0.47292, 0.47493, 0.47633],\
        [0.61986, 0.61436, 0.60905, 0.60449, 0.60305, 0.60751, 0.61464, 0.62322, 0.63250, 0.64195, 0.65129,\
        0.66048, 0.66974, 0.67945, 0.69002, 0.70176, 0.71352, 0.72467, 0.73561, 0.74601, 0.75534, 0.76310,\
        0.76892, 0.77254, 0.77373, 0.77263, 0.77005, 0.76724, 0.76540, 0.76537, 0.76748, 0.77149, 0.77673,\
        0.78235, 0.78743, 0.79104, 0.79296, 0.79617, 0.80140, 0.80614, 0.81181, 0.81859, 0.82545, 0.83197,\
        0.83794, 0.84328, 0.84798, 0.85192, 0.85496, 0.85692, 0.85771, 0.85729, 0.85573, 0.85315, 0.84973,\
        0.84563, 0.84100, 0.83597, 0.83062, 0.82505, 0.81929, 0.81338, 0.80732, 0.80106, 0.79447, 0.78820,\
        0.78213, 0.77604, 0.76977, 0.76321, 0.75625, 0.74879, 0.74076, 0.73215, 0.72303, 0.71364, 0.70462,\
        0.69687, 0.69019, 0.68428, 0.67883, 0.67368, 0.66869, 0.66369, 0.65852, 0.65303, 0.64706, 0.64046,\
        0.63312, 0.62500, 0.61622, 0.60709, 0.59811, 0.58901, 0.57978, 0.57081, 0.56213, 0.55366, 0.54529,\
        0.53689, 0.52834, 0.51957, 0.51061, 0.50166, 0.49324, 0.48562, 0.47874, 0.47280, 0.46803, 0.46461,\
        0.46267, 0.46227, 0.46334, 0.46571, 0.46905, 0.47289, 0.47661, 0.47967, 0.48186, 0.48325, 0.48405],\
        [0.62729, 0.62189, 0.61694, 0.61342, 0.61314, 0.61717, 0.62427, 0.63309, 0.64271, 0.65243, 0.66181,\
        0.67074, 0.67953, 0.68874, 0.69890, 0.71050, 0.72264, 0.73441, 0.74570, 0.75631, 0.76583, 0.77381,\
        0.77991, 0.78385, 0.78515, 0.78356, 0.78007, 0.77643, 0.77405, 0.77395, 0.77648, 0.78128, 0.78738,\
        0.79374, 0.79924, 0.80306, 0.80449, 0.80712, 0.81209, 0.81724, 0.82286, 0.82915, 0.83568, 0.84206,\
        0.84803, 0.85347, 0.85827, 0.86229, 0.86535, 0.86724, 0.86781, 0.86697, 0.86483, 0.86159, 0.85747,\
        0.85270, 0.84743, 0.84180, 0.83588, 0.82974, 0.82341, 0.81689, 0.81018, 0.80328, 0.79680, 0.79106,\
        0.78538, 0.77956, 0.77354, 0.76720, 0.76041, 0.75306, 0.74507, 0.73648, 0.72739, 0.71812, 0.70917,\
        0.70103, 0.69426, 0.68851, 0.68330, 0.67845, 0.67376, 0.66897, 0.66391, 0.65842, 0.65237, 0.64560,\
        0.63804, 0.62970, 0.62073, 0.61147, 0.60221, 0.59239, 0.58278, 0.57364, 0.56487, 0.55633, 0.54787,\
        0.53937, 0.53074, 0.52195, 0.51308, 0.50438, 0.49652, 0.48953, 0.48330, 0.47803, 0.47394, 0.47117,\
        0.46980, 0.46986, 0.47130, 0.47397, 0.47760, 0.48172, 0.48560, 0.48845, 0.49007, 0.49083, 0.49106],\
        [0.63642, 0.63149, 0.62732, 0.62487, 0.62529, 0.62913, 0.63589, 0.64463, 0.65437, 0.66423, 0.67354,\
        0.68209, 0.69027, 0.69905, 0.70911, 0.72038, 0.73228, 0.74414, 0.75561, 0.76640, 0.77610, 0.78421,\
        0.79035, 0.79455, 0.79609, 0.79386, 0.78929, 0.78482, 0.78190, 0.78175, 0.78487, 0.79076, 0.79775,\
        0.80464, 0.81036, 0.81432, 0.81683, 0.81967, 0.82378, 0.82863, 0.83399, 0.83984, 0.84596, 0.85205,\
        0.85786, 0.86324, 0.86802, 0.87206, 0.87515, 0.87699, 0.87733, 0.87603, 0.87324, 0.86928, 0.86448,\
        0.85908, 0.85325, 0.84710, 0.84071, 0.83411, 0.82732, 0.82037, 0.81330, 0.80630, 0.79998, 0.79433,\
        0.78884, 0.78325, 0.77754, 0.77154, 0.76507, 0.75798, 0.75022, 0.74182, 0.73295, 0.72387, 0.71500,\
        0.70680, 0.69984, 0.69378, 0.68847, 0.68380, 0.67928, 0.67461, 0.66960, 0.66413, 0.65804, 0.65116,\
        0.64340, 0.63478, 0.62543, 0.61556, 0.60533, 0.59516, 0.58557, 0.57642, 0.56766, 0.55914, 0.55068,\
        0.54215, 0.53349, 0.52476, 0.51610, 0.50783, 0.50030, 0.49354, 0.48764, 0.48283, 0.47924, 0.47696,\
        0.47600, 0.47638, 0.47804, 0.48085, 0.48463, 0.48902, 0.49326, 0.49595, 0.49697, 0.49719, 0.49701],\
        [0.64676, 0.64241, 0.63894, 0.63721, 0.63801, 0.64175, 0.64822, 0.65679, 0.66663, 0.67670, 0.68595,\
        0.69413, 0.70180, 0.71023, 0.71995, 0.73075, 0.74221, 0.75383, 0.76522, 0.77602, 0.78582, 0.79395,\
        0.79977, 0.80364, 0.80491, 0.80235, 0.79762, 0.79280, 0.78947, 0.78950, 0.79320, 0.80038, 0.80834,\
        0.81564, 0.82162, 0.82598, 0.82916, 0.83219, 0.83579, 0.84008, 0.84496, 0.85033, 0.85599, 0.86171,\
        0.86725, 0.87241, 0.87705, 0.88102, 0.88410, 0.88593, 0.88606, 0.88423, 0.88071, 0.87602, 0.87059,\
        0.86467, 0.85841, 0.85190, 0.84518, 0.83828, 0.83122, 0.82404, 0.81687, 0.80997, 0.80366, 0.79800,\
        0.79255, 0.78712, 0.78184, 0.77633, 0.77031, 0.76362, 0.75621, 0.74811, 0.73951, 0.73068, 0.72196,\
        0.71378, 0.70642, 0.69986, 0.69434, 0.68983, 0.68536, 0.68065, 0.67564, 0.67019, 0.66412, 0.65720,\
        0.64928, 0.64036, 0.63053, 0.62003, 0.60916, 0.59858, 0.58867, 0.57944, 0.57075, 0.56231, 0.55389,\
        0.54535, 0.53668, 0.52798, 0.51945, 0.51136, 0.50397, 0.49734, 0.49160, 0.48711, 0.48392, 0.48198,\
        0.48127, 0.48178, 0.48346, 0.48617, 0.48976, 0.49396, 0.49842, 0.50105, 0.50179, 0.50191, 0.50166],\
        [0.65772, 0.65383, 0.65080, 0.64942, 0.65036, 0.65393, 0.66008, 0.66840, 0.67829, 0.68873, 0.69791,\
        0.70567, 0.71317, 0.72136, 0.73068, 0.74101, 0.75200, 0.76324, 0.77433, 0.78491, 0.79458, 0.80259,\
        0.80809, 0.81136, 0.81208, 0.80994, 0.80592, 0.80122, 0.79808, 0.79908, 0.80349, 0.81135, 0.81994,\
        0.82758, 0.83367, 0.83814, 0.84145, 0.84433, 0.84747, 0.85116, 0.85547, 0.86033, 0.86553, 0.87084,\
        0.87600, 0.88081, 0.88513, 0.88885, 0.89183, 0.89366, 0.89365, 0.89119, 0.88687, 0.88153, 0.87563,\
        0.86938, 0.86289, 0.85621, 0.84934, 0.84232, 0.83517, 0.82793, 0.82074, 0.81380, 0.80737, 0.80171,\
        0.79594, 0.79102, 0.78653, 0.78166, 0.77619, 0.76996, 0.76294, 0.75519, 0.74688, 0.73826, 0.72965,\
        0.72139, 0.71373, 0.70683, 0.70120, 0.69661, 0.69186, 0.68696, 0.68191, 0.67656, 0.67063, 0.66377,\
        0.65575, 0.64654, 0.63628, 0.62523, 0.61384, 0.60268, 0.59227, 0.58295, 0.57440, 0.56608, 0.55767,\
        0.54906, 0.54029, 0.53153, 0.52295, 0.51482, 0.50732, 0.50063, 0.49502, 0.49090, 0.48802, 0.48626,\
        0.48564, 0.48612, 0.48763, 0.49003, 0.49309, 0.49650, 0.49978, 0.50281, 0.50438, 0.50497, 0.50500],\
        [0.66867, 0.66500, 0.66211, 0.66079, 0.66161, 0.66490, 0.67062, 0.67848, 0.68799, 0.69836, 0.70763,\
        0.71542, 0.72315, 0.73146, 0.74065, 0.75068, 0.76130, 0.77216, 0.78288, 0.79307, 0.80230, 0.80999,\
        0.81543, 0.81856, 0.81934, 0.81790, 0.81495, 0.81167, 0.81016, 0.81213, 0.81733, 0.82501, 0.83338,\
        0.84083, 0.84662, 0.85071, 0.85356, 0.85592, 0.85841, 0.86147, 0.86521, 0.86958, 0.87434, 0.87925,\
        0.88401, 0.88834, 0.89211, 0.89532, 0.89789, 0.89952, 0.89941, 0.89620, 0.89122, 0.88554, 0.87949,\
        0.87319, 0.86672, 0.86007, 0.85327, 0.84632, 0.83924, 0.83208, 0.82493, 0.81789, 0.81108, 0.80444,\
        0.79868, 0.79544, 0.79189, 0.78755, 0.78264, 0.77690, 0.77026, 0.76284, 0.75479, 0.74633, 0.73776,\
        0.72940, 0.72157, 0.71456, 0.70862, 0.70347, 0.69833, 0.69327, 0.68820, 0.68311, 0.67751, 0.67084,\
        0.66280, 0.65337, 0.64272, 0.63119, 0.61923, 0.60738, 0.59644, 0.58723, 0.57890, 0.57062, 0.56209,\
        0.55328, 0.54429, 0.53532, 0.52658, 0.51826, 0.51056, 0.50365, 0.49820, 0.49446, 0.49166, 0.48990,\
        0.48921, 0.48953, 0.49077, 0.49276, 0.49528, 0.49806, 0.50093, 0.50370, 0.50570, 0.50681, 0.50726],\
        [0.67900, 0.67526, 0.67224, 0.67074, 0.67129, 0.67419, 0.67942, 0.68664, 0.69535, 0.70504, 0.71446,\
        0.72306, 0.73146, 0.74021, 0.74956, 0.75954, 0.76996, 0.78054, 0.79094, 0.80080, 0.80969, 0.81709,\
        0.82254, 0.82586, 0.82713, 0.82666, 0.82519, 0.82384, 0.82411, 0.82706, 0.83271, 0.84025, 0.84824,\
        0.85518, 0.86021, 0.86335, 0.86523, 0.86665, 0.86831, 0.87067, 0.87389, 0.87786, 0.88228, 0.88683,\
        0.89123, 0.89502, 0.89801, 0.90043, 0.90211, 0.90278, 0.90173, 0.89831, 0.89349, 0.88801, 0.88221,\
        0.87618, 0.86997, 0.86359, 0.85705, 0.85035, 0.84351, 0.83657, 0.82959, 0.82264, 0.81583, 0.80939,\
        0.80437, 0.80134, 0.79787, 0.79380, 0.78961, 0.78425, 0.77791, 0.77079, 0.76297, 0.75459, 0.74596,\
        0.73746, 0.72943, 0.72213, 0.71569, 0.70992, 0.70452, 0.69928, 0.69423, 0.68969, 0.68470, 0.67834,\
        0.67032, 0.66070, 0.64973, 0.63783, 0.62550, 0.61328, 0.60197, 0.59288, 0.58450, 0.57602, 0.56714,\
        0.55791, 0.54855, 0.53927, 0.53028, 0.52179, 0.51399, 0.50719, 0.50192, 0.49788, 0.49486, 0.49294,\
        0.49206, 0.49216, 0.49310, 0.49473, 0.49684, 0.49925, 0.50188, 0.50444, 0.50653, 0.50797, 0.50878],\
        [0.68805, 0.68397, 0.68067, 0.67889, 0.67909, 0.68157, 0.68635, 0.69312, 0.70125, 0.71041, 0.71991,\
        0.72919, 0.73836, 0.74770, 0.75741, 0.76753, 0.77795, 0.78843, 0.79865, 0.80833, 0.81704, 0.82433,\
        0.82988, 0.83356, 0.83550, 0.83610, 0.83606, 0.83638, 0.83811, 0.84199, 0.84807, 0.85569, 0.86350,\
        0.86990, 0.87382, 0.87550, 0.87594, 0.87612, 0.87680, 0.87845, 0.88124, 0.88503, 0.88929, 0.89349,\
        0.89760, 0.90098, 0.90314, 0.90454, 0.90501, 0.90432, 0.90217, 0.89863, 0.89425, 0.88932, 0.88407,\
        0.87856, 0.87283, 0.86690, 0.86078, 0.85448, 0.84803, 0.84146, 0.83483, 0.82824, 0.82187, 0.81608,\
        0.81138, 0.80772, 0.80395, 0.80054, 0.79689, 0.79159, 0.78551, 0.77875, 0.77114, 0.76274, 0.75394,\
        0.74519, 0.73687, 0.72920, 0.72228, 0.71601, 0.71025, 0.70484, 0.69995, 0.69633, 0.69203, 0.68597,\
        0.67801, 0.66822, 0.65698, 0.64486, 0.63243, 0.62035, 0.60937, 0.59983, 0.59103, 0.58207, 0.57255,\
        0.56271, 0.55286, 0.54321, 0.53396, 0.52529, 0.51742, 0.51062, 0.50509, 0.50076, 0.49752, 0.49536,\
        0.49426, 0.49413, 0.49483, 0.49619, 0.49803, 0.50022, 0.50265, 0.50505, 0.50713, 0.50874, 0.50982],\
        [0.69485, 0.69051, 0.68707, 0.68507, 0.68493, 0.68695, 0.69133, 0.69797, 0.70586, 0.71486, 0.72454,\
        0.73439, 0.74427, 0.75423, 0.76437, 0.77477, 0.78534, 0.79589, 0.80616, 0.81580, 0.82448, 0.83183,\
        0.83760, 0.84171, 0.84432, 0.84584, 0.84695, 0.84846, 0.85121, 0.85575, 0.86214, 0.86992, 0.87789,\
        0.88398, 0.88650, 0.88629, 0.88505, 0.88390, 0.88357, 0.88456, 0.88707, 0.89094, 0.89547, 0.89947,\
        0.90317, 0.90619, 0.90783, 0.90821, 0.90738, 0.90537, 0.90231, 0.89851, 0.89438, 0.89008, 0.88553,\
        0.88068, 0.87555, 0.87017, 0.86458, 0.85879, 0.85283, 0.84674, 0.84060, 0.83452, 0.82867, 0.82332,\
        0.81869, 0.81466, 0.81090, 0.80741, 0.80346, 0.79832, 0.79267, 0.78647, 0.77903, 0.77042, 0.76131,\
        0.75226, 0.74363, 0.73560, 0.72826, 0.72160, 0.71560, 0.71029, 0.70597, 0.70270, 0.69884, 0.69316,\
        0.68537, 0.67542, 0.66393, 0.65166, 0.63927, 0.62739, 0.61653, 0.60682, 0.59784, 0.58813, 0.57776,\
        0.56726, 0.55693, 0.54694, 0.53744, 0.52859, 0.52058, 0.51358, 0.50774, 0.50308, 0.49956, 0.49717,\
        0.49585, 0.49553, 0.49606, 0.49726, 0.49894, 0.50098, 0.50326, 0.50553, 0.50758, 0.50928, 0.51055],\
        [0.69855, 0.69470, 0.69147, 0.68942, 0.68899, 0.69056, 0.69440, 0.70063, 0.70893, 0.71838, 0.72856,\
        0.73906, 0.74961, 0.76014, 0.77069, 0.78140, 0.79227, 0.80301, 0.81343, 0.82322, 0.83204, 0.83959,\
        0.84566, 0.85020, 0.85337, 0.85556, 0.85740, 0.85961, 0.86288, 0.86764, 0.87395, 0.88152, 0.88962,\
        0.89590, 0.89672, 0.89460, 0.89194, 0.88965, 0.88846, 0.88892, 0.89127, 0.89538, 0.90057, 0.90505,\
        0.90876, 0.91146, 0.91263, 0.91213, 0.91004, 0.90666, 0.90255, 0.89831, 0.89444, 0.89093, 0.88713,\
        0.88292, 0.87837, 0.87357, 0.86854, 0.86331, 0.85791, 0.85237, 0.84677, 0.84120, 0.83582, 0.83076,\
        0.82612, 0.82185, 0.81780, 0.81380, 0.80944, 0.80448, 0.79927, 0.79377, 0.78624, 0.77715, 0.76767,\
        0.75839, 0.74954, 0.74125, 0.73357, 0.72656, 0.72031, 0.71499, 0.71077, 0.70743, 0.70395, 0.69912,\
        0.69171, 0.68156, 0.66988, 0.65755, 0.64520, 0.63338, 0.62249, 0.61264, 0.60330, 0.59298, 0.58202,\
        0.57109, 0.56046, 0.55025, 0.54058, 0.53157, 0.52334, 0.51607, 0.50987, 0.50485, 0.50104, 0.49839,\
        0.49686, 0.49640, 0.49685, 0.49800, 0.49963, 0.50158, 0.50372, 0.50589, 0.50792, 0.50968, 0.51108],\
        [0.70052, 0.69722, 0.69435, 0.69236, 0.69174, 0.69296, 0.69642, 0.70263, 0.71131, 0.72142, 0.73229,\
        0.74350, 0.75473, 0.76581, 0.77668, 0.78756, 0.79877, 0.80986, 0.82055, 0.83059, 0.83968, 0.84753,\
        0.85397, 0.85890, 0.86246, 0.86502, 0.86717, 0.86959, 0.87290, 0.87746, 0.88318, 0.88962, 0.89629,\
        0.90192, 0.90232, 0.89972, 0.89646, 0.89341, 0.89160, 0.89178, 0.89421, 0.89866, 0.90461, 0.91024,\
        0.91464, 0.91745, 0.91830, 0.91707, 0.91385, 0.90900, 0.90330, 0.89819, 0.89516, 0.89267, 0.88943,\
        0.88563, 0.88153, 0.87723, 0.87275, 0.86807, 0.86322, 0.85823, 0.85317, 0.84809, 0.84308, 0.83821,\
        0.83351, 0.82895, 0.82444, 0.81984, 0.81503, 0.81001, 0.80510, 0.79999, 0.79171, 0.78218, 0.77265,\
        0.76342, 0.75458, 0.74616, 0.73821, 0.73080, 0.72411, 0.71837, 0.71375, 0.71012, 0.70683, 0.70284,\
        0.69588, 0.68573, 0.67420, 0.66203, 0.64975, 0.63784, 0.62666, 0.61630, 0.60629, 0.59582, 0.58487,\
        0.57391, 0.56327, 0.55305, 0.54334, 0.53420, 0.52575, 0.51814, 0.51154, 0.50608, 0.50188, 0.49896,\
        0.49729, 0.49678, 0.49725, 0.49845, 0.50009, 0.50198, 0.50403, 0.50614, 0.50816, 0.50997, 0.51148],\
        [0.70183, 0.69894, 0.69633, 0.69440, 0.69369, 0.69477, 0.69823, 0.70467, 0.71375, 0.72444, 0.73603,\
        0.74799, 0.75994, 0.77162, 0.78281, 0.79356, 0.80491, 0.81649, 0.82756, 0.83790, 0.84731, 0.85553,\
        0.86237, 0.86767, 0.87146, 0.87410, 0.87618, 0.87840, 0.88137, 0.88548, 0.89046, 0.89539, 0.89978,\
        0.90358, 0.90460, 0.90260, 0.89920, 0.89562, 0.89339, 0.89368, 0.89680, 0.90241, 0.90945, 0.91620,\
        0.92147, 0.92466, 0.92545, 0.92375, 0.91969, 0.91367, 0.90647, 0.89997, 0.89824, 0.89620, 0.89279,\
        0.88899, 0.88514, 0.88125, 0.87723, 0.87304, 0.86870, 0.86423, 0.85967, 0.85504, 0.85036, 0.84562,\
        0.84081, 0.83589, 0.83080, 0.82549, 0.81995, 0.81426, 0.80845, 0.80186, 0.79394, 0.78513, 0.77619,\
        0.76744, 0.75889, 0.75053, 0.74234, 0.73444, 0.72703, 0.72044, 0.71495, 0.71063, 0.70701, 0.70287,\
        0.69653, 0.68749, 0.67675, 0.66503, 0.65287, 0.64074, 0.62906, 0.61808, 0.60764, 0.59715, 0.58641,\
        0.57573, 0.56538, 0.55540, 0.54579, 0.53659, 0.52791, 0.51992, 0.51284, 0.50688, 0.50220, 0.49899,\
        0.49724, 0.49674, 0.49726, 0.49854, 0.50027, 0.50211, 0.50416, 0.50628, 0.50832, 0.51018, 0.51179],\
        [0.70274, 0.70022, 0.69780, 0.69592, 0.69518, 0.69632, 0.70007, 0.70692, 0.71639, 0.72766, 0.73996,\
        0.75269, 0.76538, 0.77776, 0.78962, 0.80064, 0.81181, 0.82336, 0.83453, 0.84507, 0.85475, 0.86337,\
        0.87066, 0.87634, 0.88027, 0.88276, 0.88451, 0.88625, 0.88859, 0.89220, 0.89683, 0.90095, 0.90412,\
        0.90644, 0.90677, 0.90475, 0.90115, 0.89713, 0.89444, 0.89529, 0.89991, 0.90729, 0.91578, 0.92368,\
        0.92980, 0.93346, 0.93436, 0.93249, 0.92806, 0.92162, 0.91419, 0.90778, 0.90450, 0.90131, 0.89718,\
        0.89305, 0.88927, 0.88562, 0.88194, 0.87814, 0.87423, 0.87024, 0.86617, 0.86196, 0.85758, 0.85295,\
        0.84800, 0.84269, 0.83696, 0.83081, 0.82427, 0.81741, 0.81024, 0.80265, 0.79478, 0.78676, 0.77877,\
        0.77085, 0.76287, 0.75472, 0.74636, 0.73786, 0.72946, 0.72159, 0.71477, 0.70937, 0.70521, 0.70119,\
        0.69550, 0.68773, 0.67807, 0.66702, 0.65501, 0.64252, 0.63009, 0.61829, 0.60760, 0.59742, 0.58681,\
        0.57670, 0.56700, 0.55752, 0.54814, 0.53892, 0.52998, 0.52157, 0.51396, 0.50743, 0.50229, 0.49878,\
        0.49692, 0.49643, 0.49698, 0.49826, 0.49994, 0.50198, 0.50417, 0.50635, 0.50844, 0.51036, 0.51205],\
        [0.70330, 0.70114, 0.69897, 0.69714, 0.69638, 0.69761, 0.70175, 0.70911, 0.71915, 0.73108, 0.74413,\
        0.75763, 0.77101, 0.78395, 0.79645, 0.80807, 0.81914, 0.83032, 0.84132, 0.85186, 0.86173, 0.87071,\
        0.87851, 0.88470, 0.88876, 0.89091, 0.89229, 0.89362, 0.89511, 0.89819, 0.90267, 0.90656, 0.90922,\
        0.91057, 0.91013, 0.90770, 0.90372, 0.89898, 0.89556, 0.89722, 0.90425, 0.91378, 0.92387, 0.93288,\
        0.93974, 0.94383, 0.94492, 0.94308, 0.93864, 0.93228, 0.92505, 0.91835, 0.91294, 0.90791, 0.90266,\
        0.89789, 0.89388, 0.89028, 0.88678, 0.88321, 0.87964, 0.87610, 0.87252, 0.86877, 0.86471, 0.86021,\
        0.85515, 0.84945, 0.84307, 0.83601, 0.82836, 0.82022, 0.81179, 0.80342, 0.79541, 0.78804, 0.78115,\
        0.77426, 0.76705, 0.75927, 0.75081, 0.74167, 0.73210, 0.72259, 0.71394, 0.70711, 0.70242, 0.69883,\
        0.69426, 0.68767, 0.67912, 0.66878, 0.65694, 0.64401, 0.63055, 0.61736, 0.60550, 0.59540, 0.58604,\
        0.57726, 0.56861, 0.55980, 0.55071, 0.54144, 0.53219, 0.52327, 0.51505, 0.50790, 0.50224, 0.49844,\
        0.49648, 0.49602, 0.49665, 0.49801, 0.49982, 0.50195, 0.50421, 0.50644, 0.50856, 0.51053, 0.51227],\
        [0.70356, 0.70176, 0.69992, 0.69817, 0.69732, 0.69863, 0.70319, 0.71118, 0.72193, 0.73463, 0.74848,\
        0.76278, 0.77683, 0.79006, 0.80252, 0.81450, 0.82580, 0.83680, 0.84755, 0.85795, 0.86785, 0.87707,\
        0.88534, 0.89224, 0.89663, 0.89836, 0.89958, 0.90102, 0.90227, 0.90499, 0.90911, 0.91274, 0.91514,\
        0.91602, 0.91513, 0.91240, 0.90813, 0.90307, 0.89939, 0.90100, 0.91080, 0.92245, 0.93381, 0.94360,\
        0.95091, 0.95526, 0.95648, 0.95473, 0.95042, 0.94420, 0.93697, 0.92962, 0.92262, 0.91587, 0.90935,\
        0.90353, 0.89875, 0.89496, 0.89146, 0.88792, 0.88466, 0.88163, 0.87862, 0.87538, 0.87171, 0.86740,\
        0.86230, 0.85627, 0.84929, 0.84139, 0.83270, 0.82339, 0.81374, 0.80428, 0.79611, 0.78978, 0.78412,\
        0.77834, 0.77197, 0.76469, 0.75626, 0.74659, 0.73584, 0.72453, 0.71363, 0.70482, 0.69992, 0.69720,\
        0.69375, 0.68831, 0.68081, 0.67122, 0.65961, 0.64626, 0.63173, 0.61691, 0.60325, 0.59312, 0.58556,\
        0.57844, 0.57093, 0.56274, 0.55384, 0.54439, 0.53470, 0.52518, 0.51627, 0.50843, 0.50219, 0.49807,\
        0.49606, 0.49566, 0.49641, 0.49794, 0.49993, 0.50208, 0.50433, 0.50656, 0.50870, 0.51069, 0.51249],\
        [0.70354, 0.70211, 0.70062, 0.69904, 0.69804, 0.69928, 0.70437, 0.71315, 0.72473, 0.73824, 0.75289,\
        0.76796, 0.78278, 0.79648, 0.80882, 0.82053, 0.83169, 0.84245, 0.85290, 0.86305, 0.87279, 0.88200,\
        0.89042, 0.89758, 0.90262, 0.90482, 0.90651, 0.90831, 0.91033, 0.91315, 0.91669, 0.91994, 0.92211,\
        0.92281, 0.92186, 0.91930, 0.91557, 0.91167, 0.90954, 0.91254, 0.92182, 0.93358, 0.94514, 0.95501,\
        0.96234, 0.96667, 0.96789, 0.96621, 0.96203, 0.95595, 0.94863, 0.94068, 0.93251, 0.92432, 0.91639,\
        0.90919, 0.90328, 0.89922, 0.89535, 0.89175, 0.88900, 0.88668, 0.88435, 0.88171, 0.87851, 0.87451,\
        0.86947, 0.86322, 0.85575, 0.84716, 0.83765, 0.82749, 0.81702, 0.80681, 0.79850, 0.79317, 0.78843,\
        0.78355, 0.77801, 0.77136, 0.76319, 0.75323, 0.74157, 0.72875, 0.71575, 0.70423, 0.69879, 0.69804,\
        0.69506, 0.69037, 0.68384, 0.67506, 0.66381, 0.65026, 0.63506, 0.61923, 0.60445, 0.59418, 0.58763,\
        0.58154, 0.57469, 0.56677, 0.55777, 0.54792, 0.53762, 0.52738, 0.51773, 0.50917, 0.50229, 0.49786,\
        0.49583, 0.49546, 0.49630, 0.49794, 0.50003, 0.50224, 0.50449, 0.50672, 0.50886, 0.51087, 0.51270],\
        [0.70327, 0.70215, 0.70104, 0.69981, 0.69866, 0.69982, 0.70554, 0.71519, 0.72763, 0.74191, 0.75721,\
        0.77281, 0.78813, 0.80234, 0.81455, 0.82588, 0.83671, 0.84713, 0.85720, 0.86696, 0.87635, 0.88531,\
        0.89363, 0.90070, 0.90598, 0.90964, 0.91251, 0.91526, 0.91816, 0.92140, 0.92482, 0.92786, 0.92992,\
        0.93064, 0.92990, 0.92788, 0.92517, 0.92289, 0.92276, 0.92668, 0.93493, 0.94554, 0.95625, 0.96551,\
        0.97237, 0.97641, 0.97753, 0.97592, 0.97197, 0.96615, 0.95892, 0.95067, 0.94175, 0.93243, 0.92311,\
        0.91430, 0.90671, 0.90097, 0.89696, 0.89427, 0.89257, 0.89120, 0.88964, 0.88765, 0.88502, 0.88145,\
        0.87661, 0.87027, 0.86243, 0.85332, 0.84328, 0.83272, 0.82213, 0.81228, 0.80442, 0.79877, 0.79425,\
        0.78996, 0.78518, 0.77933, 0.77172, 0.76184, 0.74973, 0.73608, 0.72221, 0.71026, 0.70381, 0.70148,\
        0.69822, 0.69411, 0.68852, 0.68067, 0.66990, 0.65639, 0.64102, 0.62517, 0.61072, 0.60007, 0.59326,\
        0.58708, 0.58011, 0.57200, 0.56255, 0.55199, 0.54086, 0.52979, 0.51940, 0.51026, 0.50295, 0.49822,\
        0.49595, 0.49547, 0.49630, 0.49800, 0.50015, 0.50240, 0.50466, 0.50689, 0.50903, 0.51105, 0.51290],\
        [0.70281, 0.70191, 0.70109, 0.70027, 0.69926, 0.70054, 0.70706, 0.71759, 0.73081, 0.74570, 0.76141,\
        0.77720, 0.79238, 0.80640, 0.81887, 0.83016, 0.84072, 0.85077, 0.86042, 0.86970, 0.87860, 0.88706,\
        0.89496, 0.90206, 0.90808, 0.91309, 0.91740, 0.92139, 0.92530, 0.92917, 0.93284, 0.93593, 0.93803,\
        0.93885, 0.93836, 0.93683, 0.93492, 0.93373, 0.93462, 0.93870, 0.94595, 0.95511, 0.96443, 0.97254,\
        0.97861, 0.98224, 0.98330, 0.98194, 0.97852, 0.97338, 0.96673, 0.95874, 0.94962, 0.93966, 0.92926,\
        0.91895, 0.90941, 0.90176, 0.89752, 0.89617, 0.89576, 0.89533, 0.89443, 0.89302, 0.89101, 0.88803,\
        0.88357, 0.87723, 0.86910, 0.85961, 0.84928, 0.83865, 0.82831, 0.81897, 0.81133, 0.80549, 0.80103,\
        0.79709, 0.79300, 0.78820, 0.78155, 0.77209, 0.75986, 0.74589, 0.73192, 0.72007, 0.71210, 0.70726,\
        0.70340, 0.69943, 0.69470, 0.68793, 0.67759, 0.66410, 0.64876, 0.63329, 0.61947, 0.60880, 0.60123,\
        0.59410, 0.58664, 0.57812, 0.56788, 0.55629, 0.54411, 0.53213, 0.52104, 0.51145, 0.50395, 0.49893,\
        0.49625, 0.49556, 0.49632, 0.49805, 0.50024, 0.50253, 0.50481, 0.50704, 0.50919, 0.51122, 0.51310],\
        [0.70228, 0.70153, 0.70090, 0.70040, 0.69979, 0.70171, 0.70932, 0.72064, 0.73441, 0.74969, 0.76563,\
        0.78139, 0.79622, 0.80993, 0.82229, 0.83344, 0.84374, 0.85342, 0.86264, 0.87143, 0.87980, 0.88773,\
        0.89518, 0.90235, 0.90929, 0.91559, 0.92129, 0.92657, 0.93152, 0.93614, 0.94025, 0.94355, 0.94574,\
        0.94660, 0.94612, 0.94461, 0.94274, 0.94153, 0.94216, 0.94544, 0.95130, 0.95869, 0.96616, 0.97271,\
        0.97787, 0.98128, 0.98264, 0.98202, 0.97980, 0.97615, 0.97095, 0.96404, 0.95551, 0.94562, 0.93484,\
        0.92372, 0.91298, 0.90371, 0.89936, 0.89883, 0.89903, 0.89910, 0.89851, 0.89752, 0.89617, 0.89392,\
        0.89006, 0.88377, 0.87533, 0.86553, 0.85508, 0.84456, 0.83453, 0.82556, 0.81808, 0.81223, 0.80773,\
        0.80406, 0.80063, 0.79716, 0.79177, 0.78299, 0.77068, 0.75639, 0.74221, 0.72996, 0.72073, 0.71432,\
        0.70967, 0.70577, 0.70188, 0.69599, 0.68578, 0.67204, 0.65657, 0.64130, 0.62778, 0.61693, 0.60842,\
        0.60078, 0.59340, 0.58456, 0.57316, 0.56021, 0.54687, 0.53397, 0.52221, 0.51218, 0.50439, 0.49913,\
        0.49628, 0.49550, 0.49626, 0.49803, 0.50029, 0.50261, 0.50491, 0.50716, 0.50932, 0.51137, 0.51327],\
        [0.70191, 0.70132, 0.70088, 0.70061, 0.70072, 0.70438, 0.71283, 0.72451, 0.73846, 0.75384, 0.76986,\
        0.78563, 0.80020, 0.81329, 0.82512, 0.83590, 0.84587, 0.85520, 0.86402, 0.87239, 0.88034, 0.88790,\
        0.89518, 0.90256, 0.91014, 0.91746, 0.92433, 0.93076, 0.93671, 0.94208, 0.94667, 0.95019, 0.95237,\
        0.95301, 0.95209, 0.94984, 0.94686, 0.94419, 0.94300, 0.94416, 0.94764, 0.95244, 0.95725, 0.96164,\
        0.96601, 0.97001, 0.97256, 0.97360, 0.97375, 0.97295, 0.97045, 0.96573, 0.95876, 0.94988, 0.93968,\
        0.92885, 0.91831, 0.90949, 0.90449, 0.90269, 0.90223, 0.90203, 0.90151, 0.90084, 0.90003, 0.89852,\
        0.89565, 0.88922, 0.88040, 0.87045, 0.86005, 0.84975, 0.84003, 0.83133, 0.82397, 0.81806, 0.81350,\
        0.80999, 0.80722, 0.80465, 0.80068, 0.79282, 0.78021, 0.76547, 0.75097, 0.73823, 0.72808, 0.72059,\
        0.71527, 0.71136, 0.70779, 0.70247, 0.69261, 0.67831, 0.66260, 0.64736, 0.63388, 0.62283, 0.61402,\
        0.60665, 0.59977, 0.59032, 0.57717, 0.56285, 0.54850, 0.53485, 0.52253, 0.51212, 0.50406, 0.49869,\
        0.49588, 0.49515, 0.49599, 0.49786, 0.50022, 0.50261, 0.50495, 0.50722, 0.50941, 0.51148, 0.51341],\
        [0.70192, 0.70166, 0.70168, 0.70223, 0.70410, 0.70896, 0.71752, 0.72907, 0.74277, 0.75789, 0.77370,\
        0.78945, 0.80377, 0.81604, 0.82724, 0.83759, 0.84722, 0.85623, 0.86473, 0.87278, 0.88046, 0.88785,\
        0.89513, 0.90273, 0.91079, 0.91882, 0.92660, 0.93396, 0.94077, 0.94681, 0.95179, 0.95541, 0.95735,\
        0.95738, 0.95538, 0.95143, 0.94601, 0.94016, 0.93533, 0.93257, 0.93183, 0.93206, 0.93224, 0.93331,\
        0.93763, 0.94419, 0.94971, 0.95387, 0.95835, 0.96242, 0.96423, 0.96300, 0.95870, 0.95185, 0.94326,\
        0.93377, 0.92434, 0.91620, 0.91045, 0.90706, 0.90522, 0.90416, 0.90340, 0.90276, 0.90208, 0.90117,\
        0.89902, 0.89226, 0.88348, 0.87375, 0.86368, 0.85374, 0.84436, 0.83587, 0.82853, 0.82248, 0.81768,\
        0.81400, 0.81121, 0.80874, 0.80542, 0.79880, 0.78602, 0.77135, 0.75688, 0.74386, 0.73303, 0.72467,\
        0.71861, 0.71422, 0.71033, 0.70495, 0.69518, 0.68101, 0.66544, 0.65030, 0.63682, 0.62565, 0.61679,\
        0.60955, 0.60253, 0.59209, 0.57824, 0.56334, 0.54850, 0.53446, 0.52185, 0.51121, 0.50302, 0.49771,\
        0.49509, 0.49454, 0.49552, 0.49751, 0.49999, 0.50248, 0.50489, 0.50721, 0.50943, 0.51154, 0.51350],\
        [0.70243, 0.70268, 0.70344, 0.70509, 0.70834, 0.71407, 0.72268, 0.73388, 0.74705, 0.76153, 0.77664,\
        0.79161, 0.80539, 0.81747, 0.82841, 0.83850, 0.84788, 0.85665, 0.86490, 0.87273, 0.88026, 0.88763,\
        0.89507, 0.90286, 0.91113, 0.91961, 0.92803, 0.93613, 0.94363, 0.95019, 0.95542, 0.95891, 0.96027,\
        0.95917, 0.95536, 0.94870, 0.93936, 0.92846, 0.91815, 0.90969, 0.90186, 0.89208, 0.88235, 0.87926,\
        0.88605, 0.89842, 0.91024, 0.92032, 0.93239, 0.94372, 0.95132, 0.95501, 0.95455, 0.95083, 0.94492,\
        0.93766, 0.92982, 0.92232, 0.91598, 0.91116, 0.90777, 0.90555, 0.90414, 0.90314, 0.90213, 0.90086,\
        0.89788, 0.89212, 0.88421, 0.87519, 0.86575, 0.85633, 0.84733, 0.83902, 0.83164, 0.82529, 0.82000,\
        0.81569, 0.81216, 0.80885, 0.80467, 0.79763, 0.78672, 0.77335, 0.75951, 0.74651, 0.73516, 0.72595,\
        0.71897, 0.71375, 0.70910, 0.70311, 0.69321, 0.67978, 0.66477, 0.64986, 0.63630, 0.62491, 0.61588,\
        0.60833, 0.59992, 0.58930, 0.57607, 0.56150, 0.54679, 0.53279, 0.52020, 0.50963, 0.50157, 0.49650,\
        0.49410, 0.49374, 0.49489, 0.49702, 0.49962, 0.50224, 0.50474, 0.50712, 0.50939, 0.51154, 0.51356],\
        [0.70342, 0.70428, 0.70586, 0.70851, 0.71275, 0.71909, 0.72774, 0.73854, 0.75105, 0.76468, 0.77881,\
        0.79273, 0.80586, 0.81787, 0.82877, 0.83875, 0.84798, 0.85657, 0.86464, 0.87230, 0.87972, 0.88709,\
        0.89465, 0.90258, 0.91087, 0.91962, 0.92851, 0.93718, 0.94523, 0.95219, 0.95749, 0.96057, 0.96092,\
        0.95808, 0.95165, 0.94119, 0.92636, 0.90823, 0.89135, 0.87738, 0.86032, 0.83267, 0.80436, 0.79458,\
        0.80697, 0.82761, 0.85057, 0.87418, 0.89733, 0.91570, 0.93015, 0.94083, 0.94550, 0.94620, 0.94425,\
        0.94015, 0.93438, 0.92766, 0.92082, 0.91460, 0.90956, 0.90600, 0.90372, 0.90215, 0.90048, 0.89840,\
        0.89519, 0.89000, 0.88308, 0.87501, 0.86637, 0.85758, 0.84899, 0.84087, 0.83338, 0.82664, 0.82065,\
        0.81540, 0.81073, 0.80620, 0.80101, 0.79411, 0.78438, 0.77242, 0.75945, 0.74661, 0.73477, 0.72460,\
        0.71650, 0.71035, 0.70489, 0.69811, 0.68845, 0.67577, 0.66137, 0.64664, 0.63279, 0.62088, 0.61146,\
        0.60386, 0.59572, 0.58463, 0.57184, 0.55794, 0.54376, 0.53012, 0.51780, 0.50754, 0.49994, 0.49518,\
        0.49301, 0.49286, 0.49418, 0.49645, 0.49919, 0.50193, 0.50451, 0.50697, 0.50929, 0.51149, 0.51356],\
        [0.70476, 0.70626, 0.70858, 0.71205, 0.71701, 0.72377, 0.73243, 0.74286, 0.75470, 0.76747, 0.78065,\
        0.79371, 0.80623, 0.81793, 0.82871, 0.83860, 0.84770, 0.85615, 0.86405, 0.87155, 0.87881, 0.88604,\
        0.89344, 0.90123, 0.90968, 0.91869, 0.92793, 0.93705, 0.94556, 0.95284, 0.95810, 0.96054, 0.95940,\
        0.95412, 0.94413, 0.92861, 0.90623, 0.87734, 0.85315, 0.83470, 0.80959, 0.76691, 0.71880, 0.69275,\
        0.70756, 0.74027, 0.77760, 0.81915, 0.85207, 0.87342, 0.89854, 0.91921, 0.93067, 0.93783, 0.94153,\
        0.94164, 0.93842, 0.93261, 0.92525, 0.91749, 0.91051, 0.90540, 0.90220, 0.89984, 0.89773, 0.89509,\
        0.89168, 0.88695, 0.88086, 0.87370, 0.86587, 0.85774, 0.84960, 0.84167, 0.83409, 0.82694, 0.82022,\
        0.81392, 0.80806, 0.80263, 0.79690, 0.79006, 0.78112, 0.77014, 0.75786, 0.74514, 0.73278, 0.72150,\
        0.71204, 0.70495, 0.69919, 0.69202, 0.68256, 0.67055, 0.65664, 0.64191, 0.62752, 0.61469, 0.60457,\
        0.59673, 0.58861, 0.57850, 0.56660, 0.55354, 0.54007, 0.52696, 0.51500, 0.50506, 0.49799, 0.49376,\
        0.49193, 0.49200, 0.49347, 0.49585, 0.49868, 0.50154, 0.50422, 0.50674, 0.50913, 0.51139, 0.51352],\
        [0.70631, 0.70839, 0.71136, 0.71546, 0.72095, 0.72800, 0.73665, 0.74675, 0.75802, 0.77004, 0.78238,\
        0.79467, 0.80662, 0.81797, 0.82855, 0.83829, 0.84725, 0.85553, 0.86324, 0.87055, 0.87760, 0.88458,\
        0.89169, 0.89920, 0.90770, 0.91681, 0.92622, 0.93568, 0.94465, 0.95230, 0.95756, 0.95926, 0.95630,\
        0.94787, 0.93335, 0.91154, 0.87924, 0.83724, 0.80315, 0.77791, 0.74509, 0.69721, 0.64121, 0.61195,\
        0.62605, 0.66484, 0.70322, 0.75032, 0.79169, 0.81715, 0.85473, 0.88855, 0.91065, 0.92716, 0.93821,\
        0.94332, 0.94288, 0.93798, 0.93001, 0.92046, 0.91097, 0.90368, 0.89974, 0.89664, 0.89407, 0.89135,\
        0.88799, 0.88364, 0.87819, 0.87178, 0.86468, 0.85716, 0.84946, 0.84177, 0.83418, 0.82674, 0.81945,\
        0.81227, 0.80530, 0.79919, 0.79355, 0.78678, 0.77821, 0.76778, 0.75594, 0.74330, 0.73051, 0.71820,\
        0.70722, 0.69909, 0.69330, 0.68647, 0.67736, 0.66578, 0.65217, 0.63733, 0.62234, 0.60858, 0.59766,\
        0.58983, 0.58192, 0.57250, 0.56145, 0.54922, 0.53648, 0.52395, 0.51242, 0.50276, 0.49618, 0.49252,\
        0.49104, 0.49129, 0.49284, 0.49528, 0.49819, 0.50114, 0.50387, 0.50645, 0.50890, 0.51123, 0.51344],\
        [0.70792, 0.71050, 0.71399, 0.71860, 0.72448, 0.73173, 0.74036, 0.75022, 0.76105, 0.77249, 0.78415,\
        0.79566, 0.80712, 0.81819, 0.82851, 0.83803, 0.84678, 0.85485, 0.86235, 0.86941, 0.87620, 0.88291,\
        0.88979, 0.89715, 0.90529, 0.91399, 0.92328, 0.93299, 0.94251, 0.95078, 0.95635, 0.95757, 0.95275,\
        0.94072, 0.92095, 0.89262, 0.85359, 0.80569, 0.75775, 0.71792, 0.67836, 0.62940, 0.57005, 0.52851,\
        0.55640, 0.60538, 0.64178, 0.68264, 0.73036, 0.77308, 0.81864, 0.85866, 0.89159, 0.91807, 0.93692,\
        0.94688, 0.94890, 0.94466, 0.93600, 0.92471, 0.91260, 0.90183, 0.89665, 0.89347, 0.89071, 0.88795,\
        0.88468, 0.88058, 0.87557, 0.86971, 0.86318, 0.85619, 0.84892, 0.84150, 0.83403, 0.82654, 0.81901,\
        0.81147, 0.80413, 0.79765, 0.79174, 0.78489, 0.77636, 0.76615, 0.75458, 0.74212, 0.72927, 0.71657,\
        0.70483, 0.69616, 0.69005, 0.68315, 0.67412, 0.66271, 0.64927, 0.63447, 0.61919, 0.60470, 0.59385,\
        0.58558, 0.57736, 0.56806, 0.55743, 0.54577, 0.53363, 0.52168, 0.51069, 0.50162, 0.49537, 0.49190,\
        0.49054, 0.49083, 0.49237, 0.49479, 0.49769, 0.50068, 0.50347, 0.50611, 0.50862, 0.51103, 0.51333],\
        [0.70946, 0.71245, 0.71637, 0.72135, 0.72752, 0.73493, 0.74354, 0.75324, 0.76378, 0.77487, 0.78615,\
        0.79727, 0.80820, 0.81877, 0.82870, 0.83792, 0.84642, 0.85424, 0.86148, 0.86824, 0.87469, 0.88104,\
        0.88757, 0.89460, 0.90210, 0.90995, 0.91892, 0.92888, 0.93913, 0.94843, 0.95496, 0.95648, 0.95049,\
        0.93496, 0.90977, 0.87581, 0.83068, 0.77626, 0.72159, 0.67321, 0.62450, 0.56749, 0.51298, 0.47610,\
        0.49637, 0.53959, 0.58421, 0.62756, 0.68735, 0.74223, 0.78987, 0.83564, 0.87943, 0.91519, 0.94014,\
        0.95363, 0.95718, 0.95313, 0.94376, 0.93119, 0.91754, 0.90541, 0.89730, 0.89224, 0.88878, 0.88571,\
        0.88229, 0.87821, 0.87337, 0.86783, 0.86170, 0.85512, 0.84823, 0.84114, 0.83393, 0.82662, 0.81924,\
        0.81183, 0.80455, 0.79797, 0.79190, 0.78461, 0.77579, 0.76556, 0.75420, 0.74209, 0.72969, 0.71755,\
        0.70650, 0.69767, 0.69053, 0.68290, 0.67346, 0.66192, 0.64860, 0.63411, 0.61932, 0.60554, 0.59419,\
        0.58470, 0.57557, 0.56578, 0.55507, 0.54363, 0.53189, 0.52045, 0.50998, 0.50126, 0.49538, 0.49197,\
        0.49052, 0.49068, 0.49208, 0.49438, 0.49721, 0.50019, 0.50302, 0.50571, 0.50830, 0.51081, 0.51320],\
        [0.71084, 0.71415, 0.71839, 0.72365, 0.73003, 0.73755, 0.74617, 0.75576, 0.76614, 0.77704, 0.78819,\
        0.79921, 0.80963, 0.81961, 0.82911, 0.83799, 0.84622, 0.85378, 0.86073, 0.86715, 0.87317, 0.87900,\
        0.88485, 0.89095, 0.89737, 0.90446, 0.91311, 0.92329, 0.93445, 0.94524, 0.95358, 0.95672, 0.95135,\
        0.93386, 0.90274, 0.86317, 0.80939, 0.74378, 0.68444, 0.63467, 0.57018, 0.50075, 0.45525, 0.42988,\
        0.44154, 0.47673, 0.52766, 0.58639, 0.64353, 0.70646, 0.76419, 0.82005, 0.87411, 0.91858, 0.94821,\
        0.96353, 0.96746, 0.96307, 0.95301, 0.93960, 0.92502, 0.91146, 0.90060, 0.89345, 0.88899, 0.88520,\
        0.88122, 0.87677, 0.87181, 0.86634, 0.86042, 0.85413, 0.84757, 0.84082, 0.83394, 0.82699, 0.82003,\
        0.81314, 0.80649, 0.80032, 0.79369, 0.78553, 0.77621, 0.76586, 0.75471, 0.74310, 0.73146, 0.72031,\
        0.71023, 0.70157, 0.69378, 0.68532, 0.67514, 0.66323, 0.64999, 0.63597, 0.62194, 0.60877, 0.59699,\
        0.58647, 0.57622, 0.56559, 0.55442, 0.54289, 0.53135, 0.52030, 0.51034, 0.50219, 0.49631, 0.49263,\
        0.49089, 0.49078, 0.49195, 0.49406, 0.49676, 0.49971, 0.50252, 0.50526, 0.50797, 0.51059, 0.51308],\
        [0.71198, 0.71553, 0.71999, 0.72545, 0.73198, 0.73957, 0.74819, 0.75772, 0.76798, 0.77873, 0.78971,\
        0.80050, 0.81071, 0.82039, 0.82958, 0.83822, 0.84622, 0.85356, 0.86023, 0.86628, 0.87182, 0.87699,\
        0.88196, 0.88694, 0.89218, 0.89830, 0.90621, 0.91632, 0.92837, 0.94094, 0.95174, 0.95780, 0.95549,\
        0.94030, 0.90692, 0.85436, 0.78251, 0.70247, 0.63819, 0.57728, 0.50899, 0.44580, 0.40831, 0.39064,\
        0.39963, 0.42448, 0.47067, 0.52187, 0.57527, 0.65301, 0.73497, 0.80433, 0.87288, 0.92701, 0.95946,\
        0.97489, 0.97831, 0.97334, 0.96274, 0.94883, 0.93364, 0.91897, 0.90635, 0.89741, 0.89167, 0.88656,\
        0.88148, 0.87630, 0.87093, 0.86531, 0.85944, 0.85332, 0.84700, 0.84055, 0.83402, 0.82748, 0.82100,\
        0.81467, 0.80859, 0.80256, 0.79533, 0.78667, 0.77700, 0.76658, 0.75567, 0.74459, 0.73375, 0.72358,\
        0.71442, 0.70630, 0.69854, 0.68922, 0.67817, 0.66584, 0.65267, 0.63912, 0.62572, 0.61297, 0.60114,\
        0.58999, 0.57857, 0.56694, 0.55513, 0.54331, 0.53176, 0.52093, 0.51132, 0.50344, 0.49749, 0.49351,\
        0.49144, 0.49103, 0.49193, 0.49381, 0.49631, 0.49909, 0.50194, 0.50483, 0.50767, 0.51041, 0.51301],\
        [0.71284, 0.71654, 0.72114, 0.72672, 0.73334, 0.74097, 0.74959, 0.75906, 0.76923, 0.77985, 0.79064,\
        0.80122, 0.81135, 0.82095, 0.83003, 0.83855, 0.84644, 0.85363, 0.86008, 0.86581, 0.87088, 0.87539,\
        0.87949, 0.88341, 0.88749, 0.89238, 0.89894, 0.90836, 0.92098, 0.93518, 0.94843, 0.95761, 0.95902,\
        0.94814, 0.91747, 0.84254, 0.74610, 0.65514, 0.57854, 0.51097, 0.45312, 0.40420, 0.37764, 0.36876,\
        0.37240, 0.38796, 0.42209, 0.46419, 0.51012, 0.60758, 0.70350, 0.78492, 0.87190, 0.93667, 0.96968,\
        0.98463, 0.98759, 0.98235, 0.97164, 0.95769, 0.94238, 0.92738, 0.91417, 0.90400, 0.89638, 0.88921,\
        0.88268, 0.87656, 0.87062, 0.86471, 0.85874, 0.85267, 0.84651, 0.84029, 0.83404, 0.82783, 0.82171,\
        0.81573, 0.80979, 0.80350, 0.79615, 0.78738, 0.77760, 0.76721, 0.75655, 0.74593, 0.73574, 0.72629,\
        0.71781, 0.71018, 0.70250, 0.69275, 0.68117, 0.66867, 0.65570, 0.64261, 0.62978, 0.61752, 0.60592,\
        0.59404, 0.58160, 0.56911, 0.55669, 0.54450, 0.53280, 0.52197, 0.51242, 0.50452, 0.49847, 0.49428,\
        0.49197, 0.49131, 0.49200, 0.49368, 0.49601, 0.49866, 0.50154, 0.50455, 0.50750, 0.51033, 0.51302],\
        [0.71337, 0.71714, 0.72181, 0.72744, 0.73409, 0.74174, 0.75033, 0.75977, 0.76987, 0.78042, 0.79111,\
        0.80163, 0.81172, 0.82132, 0.83043, 0.83899, 0.84690, 0.85406, 0.86041, 0.86591, 0.87059, 0.87454,\
        0.87790, 0.88089, 0.88389, 0.88749, 0.89254, 0.90050, 0.91290, 0.92797, 0.94283, 0.95405, 0.95722,\
        0.94614, 0.90995, 0.83196, 0.73029, 0.63489, 0.55089, 0.48365, 0.42899, 0.39157, 0.37200, 0.36524,\
        0.36932, 0.38316, 0.40444, 0.43599, 0.49343, 0.59043, 0.69062, 0.77759, 0.86933, 0.93869, 0.97411,\
        0.98990, 0.99341, 0.98873, 0.97860, 0.96516, 0.95021, 0.93528, 0.92171, 0.91032, 0.90065, 0.89196,\
        0.88419, 0.87719, 0.87068, 0.86441, 0.85826, 0.85215, 0.84605, 0.83995, 0.83388, 0.82786, 0.82192,\
        0.81604, 0.81009, 0.80369, 0.79624, 0.78741, 0.77764, 0.76733, 0.75687, 0.74658, 0.73679, 0.72774,\
        0.71952, 0.71189, 0.70397, 0.69433, 0.68299, 0.67081, 0.65827, 0.64570, 0.63333, 0.62129, 0.60944,\
        0.59721, 0.58444, 0.57149, 0.55862, 0.54608, 0.53415, 0.52315, 0.51346, 0.50538, 0.49913, 0.49481,\
        0.49236, 0.49156, 0.49212, 0.49370, 0.49598, 0.49869, 0.50158, 0.50454, 0.50749, 0.51037, 0.51312],\
        [0.71357, 0.71732, 0.72198, 0.72761, 0.73424, 0.74187, 0.75044, 0.75984, 0.76990, 0.78043, 0.79115,\
        0.80169, 0.81181, 0.82149, 0.83077, 0.83954, 0.84764, 0.85492, 0.86129, 0.86670, 0.87115, 0.87470,\
        0.87750, 0.87977, 0.88189, 0.88442, 0.88820, 0.89459, 0.90534, 0.91992, 0.93513, 0.94713, 0.95085,\
        0.93899, 0.90153, 0.83634, 0.75292, 0.66137, 0.57647, 0.49873, 0.44072, 0.40473, 0.38491, 0.37704,\
        0.38164, 0.39546, 0.41422, 0.45089, 0.52101, 0.62019, 0.71005, 0.79301, 0.87525, 0.93764, 0.97288,\
        0.99006, 0.99513, 0.99189, 0.98303, 0.97061, 0.95636, 0.94170, 0.92776, 0.91518, 0.90405, 0.89416,\
        0.88557, 0.87794, 0.87093, 0.86431, 0.85792, 0.85169, 0.84554, 0.83947, 0.83344, 0.82747, 0.82153,\
        0.81560, 0.80956, 0.80299, 0.79545, 0.78668, 0.77691, 0.76666, 0.75632, 0.74622, 0.73661, 0.72764,\
        0.71933, 0.71148, 0.70344, 0.69407, 0.68334, 0.67182, 0.65993, 0.64794, 0.63600, 0.62413, 0.61216,\
        0.59977, 0.58691, 0.57375, 0.56060, 0.54778, 0.53558, 0.52434, 0.51438, 0.50602, 0.49954, 0.49512,\
        0.49262, 0.49176, 0.49227, 0.49381, 0.49610, 0.49885, 0.50175, 0.50468, 0.50763, 0.51053, 0.51333],\
        [0.71342, 0.71709, 0.72166, 0.72721, 0.73380, 0.74140, 0.74995, 0.75932, 0.76935, 0.77986, 0.79067,\
        0.80137, 0.81150, 0.82140, 0.83106, 0.84022, 0.84866, 0.85622, 0.86278, 0.86825, 0.87263, 0.87598,\
        0.87844, 0.88025, 0.88176, 0.88347, 0.88619, 0.89102, 0.89937, 0.91198, 0.92629, 0.93835, 0.94306,\
        0.93427, 0.90458, 0.84543, 0.77669, 0.70828, 0.63844, 0.56228, 0.48342, 0.42590, 0.40077, 0.39375,\
        0.39776, 0.40899, 0.43878, 0.50165, 0.58265, 0.66675, 0.74691, 0.81744, 0.88339, 0.93497, 0.96820,\
        0.98650, 0.99347, 0.99214, 0.98495, 0.97388, 0.96057, 0.94636, 0.93226, 0.91890, 0.90657, 0.89585,\
        0.88687, 0.87874, 0.87127, 0.86429, 0.85764, 0.85122, 0.84495, 0.83880, 0.83272, 0.82667, 0.82059,\
        0.81442, 0.80818, 0.80209, 0.79435, 0.78519, 0.77532, 0.76507, 0.75478, 0.74474, 0.73515, 0.72609,\
        0.71750, 0.70923, 0.70119, 0.69229, 0.68232, 0.67166, 0.66056, 0.64922, 0.63776, 0.62615, 0.61429,\
        0.60197, 0.58908, 0.57579, 0.56246, 0.54942, 0.53698, 0.52547, 0.51522, 0.50657, 0.49986, 0.49534,\
        0.49280, 0.49194, 0.49243, 0.49398, 0.49626, 0.49899, 0.50191, 0.50489, 0.50787, 0.51080, 0.51363],\
        [0.71295, 0.71644, 0.72085, 0.72628, 0.73278, 0.74035, 0.74890, 0.75829, 0.76830, 0.77872, 0.78939,\
        0.80010, 0.81053, 0.82105, 0.83134, 0.84106, 0.84999, 0.85796, 0.86483, 0.87052, 0.87500, 0.87834,\
        0.88069, 0.88228, 0.88344, 0.88462, 0.88643, 0.88973, 0.89561, 0.90519, 0.91755, 0.92929, 0.93499,\
        0.92920, 0.90629, 0.86223, 0.80965, 0.75220, 0.69247, 0.63068, 0.55426, 0.46597, 0.42605, 0.41407,\
        0.41870, 0.44205, 0.49365, 0.58653, 0.65420, 0.71568, 0.78824, 0.84279, 0.89079, 0.93218, 0.96270,\
        0.98146, 0.98999, 0.99045, 0.98494, 0.97528, 0.96302, 0.94941, 0.93544, 0.92184, 0.90921, 0.89814,\
        0.88830, 0.87954, 0.87162, 0.86428, 0.85734, 0.85068, 0.84422, 0.83791, 0.83169, 0.82550, 0.81924,\
        0.81275, 0.80580, 0.79887, 0.79163, 0.78266, 0.77280, 0.76255, 0.75225, 0.74219, 0.73256, 0.72344,\
        0.71467, 0.70592, 0.69751, 0.68936, 0.68032, 0.67054, 0.66025, 0.64960, 0.63867, 0.62743, 0.61582,\
        0.60370, 0.59085, 0.57753, 0.56411, 0.55091, 0.53827, 0.52651, 0.51600, 0.50710, 0.50018, 0.49554,\
        0.49296, 0.49210, 0.49263, 0.49421, 0.49651, 0.49921, 0.50215, 0.50517, 0.50818, 0.51114, 0.51400],\
        [0.71217, 0.71541, 0.71958, 0.72481, 0.73119, 0.73874, 0.74737, 0.75689, 0.76700, 0.77738, 0.78776,\
        0.79811, 0.80934, 0.82080, 0.83181, 0.84212, 0.85157, 0.86003, 0.86735, 0.87340, 0.87813, 0.88161,\
        0.88402, 0.88562, 0.88670, 0.88760, 0.88877, 0.89077, 0.89432, 0.90017, 0.91038, 0.92189, 0.92805,\
        0.92414, 0.90657, 0.87567, 0.83537, 0.78606, 0.73291, 0.68442, 0.64328, 0.57126, 0.49365, 0.46204,\
        0.47145, 0.52243, 0.60086, 0.65902, 0.70889, 0.76016, 0.81757, 0.86078, 0.89672, 0.93131, 0.95905,\
        0.97722, 0.98636, 0.98797, 0.98376, 0.97533, 0.96406, 0.95114, 0.93751, 0.92394, 0.91110, 0.89952,\
        0.88925, 0.88016, 0.87191, 0.86424, 0.85698, 0.85003, 0.84332, 0.83678, 0.83035, 0.82399, 0.81760,\
        0.81103, 0.80408, 0.79652, 0.78844, 0.77939, 0.76952, 0.75923, 0.74886, 0.73871, 0.72903, 0.71996,\
        0.71142, 0.70305, 0.69463, 0.68641, 0.67784, 0.66875, 0.65919, 0.64921, 0.63884, 0.62804, 0.61671,\
        0.60473, 0.59208, 0.57890, 0.56550, 0.55222, 0.53942, 0.52747, 0.51673, 0.50760, 0.50049, 0.49574,\
        0.49314, 0.49229, 0.49284, 0.49448, 0.49686, 0.49967, 0.50257, 0.50555, 0.50856, 0.51154, 0.51444],\
        [0.71114, 0.71402, 0.71785, 0.72280, 0.72902, 0.73658, 0.74540, 0.75526, 0.76577, 0.77648, 0.78707,\
        0.79777, 0.80936, 0.82125, 0.83266, 0.84335, 0.85324, 0.86222, 0.87010, 0.87665, 0.88176, 0.88547,\
        0.88805, 0.88982, 0.89105, 0.89196, 0.89288, 0.89412, 0.89621, 0.90010, 0.90839, 0.91851, 0.92447,\
        0.92233, 0.90964, 0.88747, 0.85502, 0.81189, 0.76676, 0.72700, 0.69117, 0.64338, 0.58617, 0.53632,\
        0.54865, 0.61039, 0.66595, 0.70186, 0.74027, 0.78396, 0.83286, 0.87478, 0.90635, 0.93571, 0.95937,\
        0.97538, 0.98383, 0.98566, 0.98213, 0.97454, 0.96410, 0.95185, 0.93867, 0.92527, 0.91220, 0.90005,\
        0.88987, 0.88068, 0.87213, 0.86410, 0.85650, 0.84922, 0.84221, 0.83538, 0.82869, 0.82209, 0.81554,\
        0.80893, 0.80201, 0.79410, 0.78524, 0.77577, 0.76573, 0.75532, 0.74482, 0.73451, 0.72468, 0.71564,\
        0.70751, 0.69971, 0.69154, 0.68333, 0.67504, 0.66647, 0.65752, 0.64818, 0.63839, 0.62807, 0.61712,\
        0.60541, 0.59299, 0.57999, 0.56664, 0.55332, 0.54041, 0.52830, 0.51740, 0.50809, 0.50081, 0.49598,\
        0.49335, 0.49249, 0.49306, 0.49472, 0.49716, 0.50004, 0.50294, 0.50592, 0.50895, 0.51197, 0.51491],\
        [0.70992, 0.71235, 0.71571, 0.72026, 0.72623, 0.73379, 0.74292, 0.75340, 0.76473, 0.77632, 0.78768,\
        0.79891, 0.81071, 0.82247, 0.83374, 0.84446, 0.85464, 0.86415, 0.87272, 0.87996, 0.88556, 0.88953,\
        0.89226, 0.89429, 0.89581, 0.89690, 0.89792, 0.89913, 0.90078, 0.90485, 0.91151, 0.91938, 0.92486,\
        0.92484, 0.91765, 0.90294, 0.87754, 0.84175, 0.80478, 0.77443, 0.74231, 0.70288, 0.66110, 0.62226,\
        0.62472, 0.68467, 0.72731, 0.75890, 0.78610, 0.81846, 0.86105, 0.89709, 0.92360, 0.94575, 0.96374,\
        0.97621, 0.98281, 0.98396, 0.98046, 0.97330, 0.96344, 0.95177, 0.93910, 0.92611, 0.91338, 0.90151,\
        0.89088, 0.88122, 0.87225, 0.86383, 0.85585, 0.84823, 0.84088, 0.83372, 0.82670, 0.81976, 0.81287,\
        0.80600, 0.79878, 0.79056, 0.78152, 0.77182, 0.76162, 0.75109, 0.74042, 0.72988, 0.71976, 0.71045,\
        0.70245, 0.69513, 0.68748, 0.67971, 0.67183, 0.66374, 0.65534, 0.64658, 0.63738, 0.62763, 0.61718,\
        0.60586, 0.59371, 0.58086, 0.56755, 0.55418, 0.54118, 0.52898, 0.51798, 0.50859, 0.50122, 0.49630,\
        0.49360, 0.49271, 0.49325, 0.49489, 0.49729, 0.50012, 0.50314, 0.50622, 0.50933, 0.51241, 0.51541],\
        [0.70861, 0.71047, 0.71323, 0.71720, 0.72275, 0.73022, 0.73974, 0.75109, 0.76367, 0.77669, 0.78935,\
        0.80128, 0.81288, 0.82393, 0.83451, 0.84486, 0.85512, 0.86520, 0.87469, 0.88292, 0.88921, 0.89340,\
        0.89610, 0.89826, 0.90012, 0.90134, 0.90249, 0.90387, 0.90599, 0.90991, 0.91576, 0.92238, 0.92766,\
        0.92950, 0.92648, 0.91754, 0.90039, 0.87349, 0.84749, 0.82562, 0.80520, 0.78067, 0.74714, 0.72118,\
        0.71096, 0.75944, 0.79106, 0.81508, 0.83761, 0.86432, 0.89763, 0.92258, 0.94202, 0.95778, 0.97026,\
        0.97879, 0.98296, 0.98284, 0.97889, 0.97177, 0.96224, 0.95103, 0.93885, 0.92637, 0.91414, 0.90258,\
        0.89171, 0.88155, 0.87213, 0.86332, 0.85498, 0.84701, 0.83932, 0.83183, 0.82445, 0.81711, 0.80972,\
        0.80229, 0.79478, 0.78651, 0.77743, 0.76768, 0.75741, 0.74679, 0.73600, 0.72527, 0.71487, 0.70517,\
        0.69688, 0.68994, 0.68292, 0.67568, 0.66825, 0.66060, 0.65268, 0.64444, 0.63582, 0.62669, 0.61687,\
        0.60609, 0.59424, 0.58147, 0.56813, 0.55470, 0.54166, 0.52942, 0.51841, 0.50901, 0.50164, 0.49665,\
        0.49387, 0.49291, 0.49340, 0.49501, 0.49738, 0.50021, 0.50330, 0.50648, 0.50969, 0.51285, 0.51592],\
        [0.70732, 0.70854, 0.71053, 0.71367, 0.71852, 0.72570, 0.73555, 0.74791, 0.76206, 0.77691, 0.79126,\
        0.80403, 0.81491, 0.82468, 0.83406, 0.84366, 0.85382, 0.86451, 0.87523, 0.88498, 0.89239, 0.89683,\
        0.89924, 0.90128, 0.90310, 0.90447, 0.90580, 0.90751, 0.91011, 0.91410, 0.91951, 0.92556, 0.93087,\
        0.93396, 0.93350, 0.92819, 0.91645, 0.89940, 0.88419, 0.86873, 0.85595, 0.84344, 0.82023, 0.80384,\
        0.82110, 0.83958, 0.85014, 0.86290, 0.87851, 0.89986, 0.92501, 0.94379, 0.95782, 0.96870, 0.97678,\
        0.98182, 0.98355, 0.98195, 0.97727, 0.96995, 0.96054, 0.94965, 0.93786, 0.92576, 0.91391, 0.90263,\
        0.89166, 0.88125, 0.87154, 0.86244, 0.85382, 0.84555, 0.83755, 0.82974, 0.82204, 0.81437, 0.80661,\
        0.79869, 0.79081, 0.78243, 0.77333, 0.76358, 0.75332, 0.74268, 0.73185, 0.72105, 0.71059, 0.70094,\
        0.69260, 0.68540, 0.67852, 0.67156, 0.66444, 0.65712, 0.64957, 0.64175, 0.63361, 0.62508, 0.61597,\
        0.60590, 0.59438, 0.58157, 0.56817, 0.55473, 0.54171, 0.52951, 0.51855, 0.50923, 0.50194, 0.49690,\
        0.49404, 0.49302, 0.49349, 0.49509, 0.49748, 0.50034, 0.50347, 0.50673, 0.51003, 0.51329, 0.51644],\
        [0.70621, 0.70674, 0.70783, 0.70986, 0.71359, 0.72004, 0.73001, 0.74337, 0.75919, 0.77604, 0.79204,\
        0.80539, 0.81544, 0.82350, 0.83130, 0.83989, 0.84975, 0.86098, 0.87317, 0.88519, 0.89471, 0.89952,\
        0.90182, 0.90331, 0.90468, 0.90604, 0.90758, 0.90963, 0.91256, 0.91668, 0.92191, 0.92773, 0.93319,\
        0.93716, 0.93852, 0.93623, 0.92985, 0.92105, 0.91178, 0.90042, 0.89180, 0.89243, 0.89133, 0.89511,\
        0.90305, 0.89625, 0.89593, 0.90361, 0.91325, 0.92782, 0.94513, 0.95944, 0.96986, 0.97724, 0.98203,\
        0.98426, 0.98387, 0.98085, 0.97537, 0.96773, 0.95833, 0.94763, 0.93609, 0.92421, 0.91244, 0.90115,\
        0.89038, 0.88011, 0.87038, 0.86114, 0.85233, 0.84383, 0.83557, 0.82748, 0.81951, 0.81162, 0.80371,\
        0.79563, 0.78731, 0.77866, 0.76948, 0.75974, 0.74951, 0.73892, 0.72816, 0.71742, 0.70704, 0.69749,\
        0.68911, 0.68160, 0.67454, 0.66758, 0.66056, 0.65340, 0.64605, 0.63848, 0.63065, 0.62253, 0.61403,\
        0.60470, 0.59349, 0.58073, 0.56742, 0.55411, 0.54122, 0.52914, 0.51829, 0.50906, 0.50184, 0.49684,\
        0.49397, 0.49296, 0.49344, 0.49509, 0.49755, 0.50047, 0.50363, 0.50694, 0.51034, 0.51370, 0.51695],\
        [0.70541, 0.70531, 0.70547, 0.70619, 0.70822, 0.71327, 0.72298, 0.73712, 0.75437, 0.77307, 0.79109,\
        0.80509, 0.81320, 0.81916, 0.82533, 0.83278, 0.84210, 0.85351, 0.86686, 0.88135, 0.89485, 0.90107,\
        0.90217, 0.90304, 0.90419, 0.90569, 0.90751, 0.90987, 0.91307, 0.91730, 0.92250, 0.92830, 0.93401,\
        0.93878, 0.94176, 0.94229, 0.94029, 0.93651, 0.93154, 0.92582, 0.92449, 0.93160, 0.94021, 0.94775,\
        0.94902, 0.94071, 0.93568, 0.93735, 0.94288, 0.95164, 0.96220, 0.97169, 0.97873, 0.98325, 0.98548,\
        0.98552, 0.98339, 0.97916, 0.97295, 0.96498, 0.95556, 0.94500, 0.93367, 0.92192, 0.91015, 0.89884,\
        0.88834, 0.87832, 0.86869, 0.85943, 0.85051, 0.84186, 0.83341, 0.82510, 0.81689, 0.80879, 0.80076,\
        0.79253, 0.78405, 0.77524, 0.76599, 0.75625, 0.74609, 0.73562, 0.72499, 0.71442, 0.70418, 0.69468,\
        0.68613, 0.67830, 0.67093, 0.66377, 0.65665, 0.64948, 0.64217, 0.63466, 0.62690, 0.61886, 0.61054,\
        0.60167, 0.59086, 0.57857, 0.56569, 0.55275, 0.54016, 0.52831, 0.51760, 0.50845, 0.50127, 0.49639,\
        0.49363, 0.49266, 0.49318, 0.49488, 0.49746, 0.50052, 0.50365, 0.50705, 0.51059, 0.51410, 0.51746],\
        [0.70499, 0.70440, 0.70383, 0.70332, 0.70319, 0.70581, 0.71490, 0.72932, 0.74715, 0.76660, 0.78556,\
        0.79984, 0.80601, 0.81078, 0.81585, 0.82225, 0.83075, 0.84166, 0.85504, 0.87041, 0.88528, 0.89412,\
        0.89719, 0.89903, 0.90094, 0.90309, 0.90538, 0.90808, 0.91154, 0.91589, 0.92114, 0.92705, 0.93312,\
        0.93871, 0.94319, 0.94613, 0.94749, 0.94762, 0.94715, 0.94747, 0.95132, 0.95974, 0.96931, 0.97603,\
        0.97656, 0.97133, 0.96610, 0.96436, 0.96604, 0.97027, 0.97581, 0.98106, 0.98490, 0.98691, 0.98703,\
        0.98532, 0.98182, 0.97660, 0.96979, 0.96157, 0.95216, 0.94181, 0.93075, 0.91923, 0.90759, 0.89644,\
        0.88609, 0.87619, 0.86662, 0.85736, 0.84839, 0.83966, 0.83111, 0.82265, 0.81421, 0.80584, 0.79771,\
        0.78948, 0.78099, 0.77214, 0.76286, 0.75316, 0.74308, 0.73273, 0.72227, 0.71189, 0.70183, 0.69231,\
        0.68348, 0.67529, 0.66754, 0.66008, 0.65274, 0.64541, 0.63801, 0.63043, 0.62257, 0.61430, 0.60562,\
        0.59670, 0.58665, 0.57524, 0.56309, 0.55073, 0.53858, 0.52706, 0.51656, 0.50752, 0.50042, 0.49569,\
        0.49304, 0.49213, 0.49267, 0.49436, 0.49691, 0.49991, 0.50333, 0.50701, 0.51078, 0.51447, 0.51796],\
        [0.70491, 0.70399, 0.70297, 0.70175, 0.70010, 0.69965, 0.70738, 0.72090, 0.73796, 0.75644, 0.77375,\
        0.78659, 0.79409, 0.79910, 0.80377, 0.80922, 0.81674, 0.82634, 0.83836, 0.85316, 0.86819, 0.87963,\
        0.88669, 0.89128, 0.89491, 0.89817, 0.90116, 0.90434, 0.90812, 0.91255, 0.91789, 0.92402, 0.93053,\
        0.93693, 0.94275, 0.94768, 0.95167, 0.95500, 0.95829, 0.96259, 0.96907, 0.97765, 0.98623, 0.99200,\
        0.99312, 0.99022, 0.98613, 0.98330, 0.98251, 0.98349, 0.98543, 0.98737, 0.98848, 0.98831, 0.98668,\
        0.98357, 0.97899, 0.97301, 0.96576, 0.95739, 0.94808, 0.93803, 0.92740, 0.91632, 0.90502, 0.89409,\
        0.88376, 0.87383, 0.86422, 0.85493, 0.84594, 0.83721, 0.82868, 0.82022, 0.81172, 0.80311, 0.79485,\
        0.78670, 0.77821, 0.76935, 0.76008, 0.75044, 0.74045, 0.73021, 0.71988, 0.70964, 0.69966, 0.69005,\
        0.68097, 0.67240, 0.66426, 0.65643, 0.64878, 0.64122, 0.63365, 0.62596, 0.61803, 0.60966, 0.60063,\
        0.59166, 0.58203, 0.57136, 0.55996, 0.54824, 0.53663, 0.52552, 0.51531, 0.50645, 0.49950, 0.49488,\
        0.49229, 0.49139, 0.49191, 0.49358, 0.49610, 0.49925, 0.50293, 0.50691, 0.51093, 0.51483, 0.51845],\
        [0.70499, 0.70382, 0.70246, 0.70071, 0.69812, 0.69700, 0.70179, 0.71303, 0.72818, 0.74472, 0.76012,\
        0.77226, 0.78061, 0.78640, 0.79110, 0.79592, 0.80245, 0.80998, 0.81932, 0.83375, 0.84981, 0.86354,\
        0.87377, 0.88117, 0.88678, 0.89133, 0.89523, 0.89909, 0.90315, 0.90752, 0.91300, 0.91941, 0.92637,\
        0.93351, 0.94046, 0.94696, 0.95297, 0.95869, 0.96457, 0.97117, 0.97886, 0.98722, 0.99485, 1.00004,\
        1.00177, 1.00042, 0.99751, 0.99460, 0.99254, 0.99144, 0.99093, 0.99044, 0.98943, 0.98750, 0.98446,\
        0.98024, 0.97484, 0.96831, 0.96075, 0.95232, 0.94320, 0.93357, 0.92353, 0.91310, 0.90238, 0.89162,\
        0.88118, 0.87112, 0.86142, 0.85209, 0.84311, 0.83444, 0.82603, 0.81778, 0.80956, 0.80119, 0.79277,\
        0.78435, 0.77574, 0.76683, 0.75760, 0.74802, 0.73812, 0.72797, 0.71770, 0.70750, 0.69750, 0.68779,\
        0.67846, 0.66954, 0.66098, 0.65275, 0.64475, 0.63691, 0.62913, 0.62132, 0.61339, 0.60527, 0.59655,\
        0.58728, 0.57766, 0.56742, 0.55661, 0.54551, 0.53445, 0.52380, 0.51395, 0.50534, 0.49856, 0.49398,\
        0.49135, 0.49040, 0.49090, 0.49260, 0.49524, 0.49861, 0.50254, 0.50680, 0.51109, 0.51519, 0.51893],\
        [0.70507, 0.70362, 0.70188, 0.69950, 0.69653, 0.69512, 0.69795, 0.70639, 0.71925, 0.73368, 0.74741,\
        0.75891, 0.76771, 0.77430, 0.77959, 0.78459, 0.79006, 0.79544, 0.80178, 0.81719, 0.83403, 0.84904,\
        0.86116, 0.87052, 0.87775, 0.88350, 0.88833, 0.89272, 0.89689, 0.90124, 0.90703, 0.91364, 0.92094,\
        0.92867, 0.93649, 0.94414, 0.95155, 0.95885, 0.96629, 0.97410, 0.98228, 0.99034, 0.99733, 1.00220,\
        1.00438, 1.00405, 1.00207, 0.99943, 0.99684, 0.99455, 0.99248, 0.99033, 0.98777, 0.98453, 0.98043,\
        0.97539, 0.96939, 0.96248, 0.95474, 0.94632, 0.93741, 0.92823, 0.91890, 0.90924, 0.89899, 0.88845,\
        0.87799, 0.86784, 0.85808, 0.84873, 0.83979, 0.83123, 0.82300, 0.81503, 0.80721, 0.79929, 0.79073,\
        0.78210, 0.77339, 0.76448, 0.75530, 0.74583, 0.73605, 0.72599, 0.71571, 0.70539, 0.69532, 0.68551,\
        0.67592, 0.66662, 0.65764, 0.64897, 0.64060, 0.63246, 0.62446, 0.61650, 0.60849, 0.60036, 0.59193,\
        0.58289, 0.57343, 0.56353, 0.55322, 0.54266, 0.53213, 0.52194, 0.51248, 0.50418, 0.49753, 0.49288,\
        0.49011, 0.48908, 0.48957, 0.49136, 0.49421, 0.49787, 0.50212, 0.50668, 0.51125, 0.51555, 0.51939],\
        [0.70502, 0.70327, 0.70113, 0.69832, 0.69535, 0.69382, 0.69544, 0.70160, 0.71225, 0.72455, 0.73668,\
        0.74750, 0.75651, 0.76384, 0.76999, 0.77557, 0.78096, 0.78645, 0.79446, 0.80772, 0.82312, 0.83787,\
        0.85049, 0.86073, 0.86889, 0.87548, 0.88098, 0.88578, 0.89026, 0.89501, 0.90066, 0.90714, 0.91458,\
        0.92275, 0.93115, 0.93949, 0.94771, 0.95588, 0.96412, 0.97246, 0.98074, 0.98851, 0.99508, 0.99980,\
        1.00228, 1.00263, 1.00134, 0.99905, 0.99632, 0.99343, 0.99044, 0.98724, 0.98366, 0.97951, 0.97468,\
        0.96908, 0.96271, 0.95558, 0.94777, 0.93939, 0.93063, 0.92178, 0.91314, 0.90446, 0.89470, 0.88428,\
        0.87392, 0.86381, 0.85406, 0.84474, 0.83587, 0.82744, 0.81940, 0.81165, 0.80402, 0.79625, 0.78803,\
        0.77955, 0.77093, 0.76212, 0.75307, 0.74376, 0.73415, 0.72425, 0.71404, 0.70357, 0.69335, 0.68333,\
        0.67339, 0.66364, 0.65417, 0.64506, 0.63630, 0.62785, 0.61965, 0.61156, 0.60347, 0.59535, 0.58705,\
        0.57834, 0.56921, 0.55968, 0.54981, 0.53974, 0.52966, 0.51987, 0.51074, 0.50270, 0.49610, 0.49127,\
        0.48833, 0.48721, 0.48774, 0.48971, 0.49285, 0.49689, 0.50153, 0.50647, 0.51138, 0.51589, 0.51979],\
        [0.70482, 0.70277, 0.70031, 0.69736, 0.69457, 0.69317, 0.69441, 0.69928, 0.70749, 0.71758, 0.72819,\
        0.73831, 0.74742, 0.75543, 0.76259, 0.76927, 0.77587, 0.78298, 0.79173, 0.80311, 0.81687, 0.83036,\
        0.84231, 0.85243, 0.86083, 0.86780, 0.87370, 0.87889, 0.88374, 0.88871, 0.89417, 0.90032, 0.90780,\
        0.91619, 0.92483, 0.93337, 0.94183, 0.95029, 0.95878, 0.96723, 0.97541, 0.98292, 0.98922, 0.99385,\
        0.99653, 0.99732, 0.99652, 0.99457, 0.99188, 0.98875, 0.98529, 0.98151, 0.97732, 0.97263, 0.96736,\
        0.96147, 0.95493, 0.94775, 0.93997, 0.93167, 0.92302, 0.91420, 0.90585, 0.89785, 0.88875, 0.87887,\
        0.86884, 0.85894, 0.84931, 0.84006, 0.83129, 0.82302, 0.81520, 0.80762, 0.80013, 0.79258, 0.78473,\
        0.77659, 0.76822, 0.75963, 0.75079, 0.74168, 0.73230, 0.72264, 0.71271, 0.70242, 0.69189, 0.68132,\
        0.67082, 0.66053, 0.65054, 0.64095, 0.63179, 0.62307, 0.61471, 0.60656, 0.59848, 0.59040, 0.58222,\
        0.57376, 0.56496, 0.55582, 0.54636, 0.53670, 0.52700, 0.51750, 0.50852, 0.50047, 0.49381, 0.48882,\
        0.48572, 0.48454, 0.48520, 0.48746, 0.49102, 0.49549, 0.50058, 0.50614, 0.51150, 0.51618, 0.52004],\
        [0.70448, 0.70219, 0.69952, 0.69659, 0.69405, 0.69282, 0.69383, 0.69767, 0.70405, 0.71233, 0.72166,\
        0.73118, 0.74033, 0.74896, 0.75716, 0.76521, 0.77350, 0.78244, 0.79229, 0.80291, 0.81443, 0.82579,\
        0.83628, 0.84559, 0.85368, 0.86067, 0.86675, 0.87220, 0.87734, 0.88250, 0.88797, 0.89407, 0.90125,\
        0.90955, 0.91794, 0.92611, 0.93432, 0.94262, 0.95098, 0.95927, 0.96725, 0.97452, 0.98063, 0.98519,\
        0.98800, 0.98906, 0.98859, 0.98690, 0.98435, 0.98118, 0.97756, 0.97353, 0.96907, 0.96413, 0.95869,\
        0.95272, 0.94621, 0.93916, 0.93153, 0.92342, 0.91500, 0.90628, 0.89759, 0.88998, 0.88157, 0.87240,\
        0.86288, 0.85330, 0.84387, 0.83474, 0.82606, 0.81795, 0.81041, 0.80317, 0.79587, 0.78856, 0.78105,\
        0.77328, 0.76525, 0.75695, 0.74835, 0.73946, 0.73029, 0.72086, 0.71122, 0.70123, 0.69031, 0.67913,\
        0.66806, 0.65722, 0.64669, 0.63660, 0.62705, 0.61807, 0.60960, 0.60145, 0.59343, 0.58544, 0.57741,\
        0.56919, 0.56069, 0.55189, 0.54280, 0.53349, 0.52407, 0.51474, 0.50576, 0.49754, 0.49059, 0.48532,\
        0.48201, 0.48079, 0.48166, 0.48439, 0.48862, 0.49383, 0.49963, 0.50589, 0.51162, 0.51633, 0.52000],\
        [0.70398, 0.70151, 0.69875, 0.69589, 0.69356, 0.69243, 0.69313, 0.69604, 0.70113, 0.70822, 0.71665,\
        0.72570, 0.73488, 0.74399, 0.75307, 0.76235, 0.77214, 0.78272, 0.79399, 0.80443, 0.81357, 0.82262,\
        0.83143, 0.83970, 0.84724, 0.85402, 0.86012, 0.86573, 0.87109, 0.87644, 0.88208, 0.88829, 0.89534,\
        0.90324, 0.91065, 0.91790, 0.92553, 0.93338, 0.94133, 0.94932, 0.95706, 0.96413, 0.97010, 0.97461,\
        0.97746, 0.97865, 0.97836, 0.97685, 0.97444, 0.97136, 0.96777, 0.96374, 0.95927, 0.95432, 0.94890,\
        0.94303, 0.93674, 0.93001, 0.92269, 0.91482, 0.90672, 0.89873, 0.89030, 0.88227, 0.87407, 0.86538,\
        0.85633, 0.84711, 0.83791, 0.82890, 0.82027, 0.81226, 0.80497, 0.79823, 0.79127, 0.78425, 0.77709,\
        0.76970, 0.76203, 0.75404, 0.74571, 0.73702, 0.72797, 0.71860, 0.70891, 0.69876, 0.68781, 0.67641,\
        0.66496, 0.65365, 0.64262, 0.63203, 0.62204, 0.61281, 0.60428, 0.59614, 0.58823, 0.58040, 0.57255,\
        0.56457, 0.55635, 0.54786, 0.53906, 0.53000, 0.52077, 0.51147, 0.50232, 0.49372, 0.48629, 0.48057,\
        0.47695, 0.47568, 0.47680, 0.48018, 0.48540, 0.49184, 0.49882, 0.50573, 0.51163, 0.51609, 0.51938],\
        [0.70332, 0.70067, 0.69786, 0.69514, 0.69297, 0.69188, 0.69232, 0.69457, 0.69877, 0.70500, 0.71279,\
        0.72146, 0.73058, 0.73995, 0.74958, 0.75963, 0.77029, 0.78162, 0.79320, 0.80341, 0.81146, 0.81907,\
        0.82666, 0.83408, 0.84111, 0.84764, 0.85369, 0.85937, 0.86487, 0.87038, 0.87614, 0.88237, 0.88924,\
        0.89643, 0.90241, 0.90879, 0.91592, 0.92309, 0.93041, 0.93801, 0.94553, 0.95245, 0.95832, 0.96276,\
        0.96558, 0.96678, 0.96652, 0.96508, 0.96278, 0.95984, 0.95643, 0.95259, 0.94830, 0.94354, 0.93828,\
        0.93260, 0.92667, 0.92050, 0.91371, 0.90609, 0.89825, 0.89052, 0.88263, 0.87464, 0.86653, 0.85814,\
        0.84947, 0.84061, 0.83166, 0.82278, 0.81417, 0.80620, 0.79911, 0.79264, 0.78622, 0.77964, 0.77288,\
        0.76588, 0.75858, 0.75092, 0.74285, 0.73433, 0.72534, 0.71587, 0.70595, 0.69556, 0.68462, 0.67318,\
        0.66152, 0.64986, 0.63840, 0.62731, 0.61684, 0.60731, 0.59874, 0.59070, 0.58294, 0.57529, 0.56764,\
        0.55986, 0.55188, 0.54363, 0.53504, 0.52613, 0.51693, 0.50754, 0.49801, 0.48880, 0.48073, 0.47442,\
        0.47036, 0.46892, 0.47027, 0.47435, 0.48079, 0.48884, 0.49732, 0.50520, 0.51099, 0.51490, 0.51770],\
        [0.70250, 0.69967, 0.69684, 0.69423, 0.69219, 0.69114, 0.69143, 0.69332, 0.69696, 0.70255, 0.70980,\
        0.71808, 0.72698, 0.73632, 0.74607, 0.75632, 0.76711, 0.77831, 0.78937, 0.79913, 0.80695, 0.81410,\
        0.82120, 0.82820, 0.83493, 0.84128, 0.84726, 0.85297, 0.85854, 0.86411, 0.86984, 0.87584, 0.88208,\
        0.88813, 0.89363, 0.89953, 0.90620, 0.91220, 0.91868, 0.92598, 0.93334, 0.94015, 0.94593, 0.95028,\
        0.95297, 0.95403, 0.95365, 0.95216, 0.94991, 0.94714, 0.94401, 0.94051, 0.93659, 0.93217, 0.92718,\
        0.92170, 0.91609, 0.91070, 0.90485, 0.89753, 0.88996, 0.88238, 0.87470, 0.86688, 0.85893, 0.85079,\
        0.84247, 0.83398, 0.82536, 0.81669, 0.80816, 0.80015, 0.79340, 0.78716, 0.78106, 0.77486, 0.76848,\
        0.76185, 0.75491, 0.74759, 0.73980, 0.73147, 0.72252, 0.71294, 0.70282, 0.69227, 0.68124, 0.66972,\
        0.65789, 0.64599, 0.63419, 0.62269, 0.61170, 0.60176, 0.59320, 0.58530, 0.57767, 0.57016, 0.56264,\
        0.55503, 0.54722, 0.53913, 0.53067, 0.52175, 0.51239, 0.50269, 0.49259, 0.48268, 0.47387, 0.46683,\
        0.46215, 0.46034, 0.46173, 0.46639, 0.47398, 0.48378, 0.49460, 0.50356, 0.50846, 0.51182, 0.51437],\
        [0.70153, 0.69855, 0.69567, 0.69308, 0.69112, 0.69012, 0.69038, 0.69213, 0.69551, 0.70063, 0.70736,\
        0.71520, 0.72370, 0.73270, 0.74217, 0.75211, 0.76249, 0.77305, 0.78328, 0.79246, 0.80036, 0.80765,\
        0.81483, 0.82182, 0.82848, 0.83474, 0.84067, 0.84638, 0.85196, 0.85752, 0.86312, 0.86877, 0.87441,\
        0.87986, 0.88513, 0.89067, 0.89631, 0.90097, 0.90679, 0.91398, 0.92111, 0.92778, 0.93348, 0.93770,\
        0.94017, 0.94093, 0.94026, 0.93858, 0.93631, 0.93373, 0.93096, 0.92793, 0.92449, 0.92061, 0.91606,\
        0.91071, 0.90522, 0.90035, 0.89550, 0.88885, 0.88158, 0.87417, 0.86664, 0.85898, 0.85121, 0.84332,\
        0.83535, 0.82729, 0.81910, 0.81079, 0.80261, 0.79489, 0.78808, 0.78188, 0.77592, 0.76997, 0.76390,\
        0.75761, 0.75102, 0.74404, 0.73658, 0.72850, 0.71967, 0.71002, 0.69971, 0.68906, 0.67788, 0.66620,\
        0.65421, 0.64211, 0.63012, 0.61843, 0.60724, 0.59697, 0.58821, 0.58021, 0.57254, 0.56504, 0.55757,\
        0.55002, 0.54230, 0.53430, 0.52588, 0.51685, 0.50710, 0.49673, 0.48600, 0.47544, 0.46586, 0.45796,\
        0.45244, 0.44997, 0.45109, 0.45599, 0.46432, 0.47514, 0.48684, 0.49636, 0.50211, 0.50602, 0.50898],\
        [0.70038, 0.69724, 0.69421, 0.69155, 0.68959, 0.68865, 0.68897, 0.69071, 0.69399, 0.69880, 0.70515,\
        0.71250, 0.72041, 0.72879, 0.73765, 0.74697, 0.75666, 0.76645, 0.77595, 0.78475, 0.79274, 0.80030,\
        0.80778, 0.81498, 0.82169, 0.82791, 0.83378, 0.83948, 0.84508, 0.85059, 0.85603, 0.86136, 0.86657,\
        0.87160, 0.87652, 0.88143, 0.88615, 0.89063, 0.89618, 0.90272, 0.90932, 0.91578, 0.92141, 0.92544,\
        0.92759, 0.92791, 0.92676, 0.92478, 0.92245, 0.92002, 0.91772, 0.91531, 0.91233, 0.90912, 0.90542,\
        0.90021, 0.89505, 0.89032, 0.88548, 0.87959, 0.87288, 0.86577, 0.85844, 0.85094, 0.84334, 0.83569,\
        0.82807, 0.82050, 0.81285, 0.80494, 0.79709, 0.78967, 0.78287, 0.77664, 0.77074, 0.76495, 0.75912,\
        0.75312, 0.74685, 0.74022, 0.73311, 0.72540, 0.71686, 0.70733, 0.69695, 0.68604, 0.67458, 0.66267,\
        0.65048, 0.63824, 0.62616, 0.61447, 0.60340, 0.59309, 0.58384, 0.57546, 0.56757, 0.55994, 0.55239,\
        0.54479, 0.53704, 0.52903, 0.52062, 0.51148, 0.50122, 0.49015, 0.47870, 0.46747, 0.45709, 0.44825,\
        0.44168, 0.43821, 0.43864, 0.44340, 0.45203, 0.46318, 0.47487, 0.48493, 0.49233, 0.49764, 0.50163],\
        [0.69931, 0.69563, 0.69222, 0.68938, 0.68739, 0.68649, 0.68691, 0.68876, 0.69209, 0.69685, 0.70289,\
        0.70968, 0.71678, 0.72434, 0.73241, 0.74096, 0.74987, 0.75894, 0.76789, 0.77648, 0.78466, 0.79255,\
        0.80030, 0.80776, 0.81455, 0.82065, 0.82646, 0.83220, 0.83785, 0.84335, 0.84865, 0.85373, 0.85859,\
        0.86327, 0.86781, 0.87228, 0.87669, 0.88125, 0.88645, 0.89225, 0.89828, 0.90444, 0.91006, 0.91370,\
        0.91548, 0.91530, 0.91351, 0.91117, 0.90885, 0.90654, 0.90470, 0.90317, 0.90045, 0.89774, 0.89485,\
        0.89039, 0.88557, 0.88085, 0.87591, 0.87031, 0.86401, 0.85723, 0.85011, 0.84277, 0.83530, 0.82781,\
        0.82046, 0.81337, 0.80643, 0.79897, 0.79133, 0.78409, 0.77738, 0.77118, 0.76535, 0.75970, 0.75407,\
        0.74832, 0.74232, 0.73599, 0.72925, 0.72196, 0.71393, 0.70474, 0.69428, 0.68301, 0.67117, 0.65899,\
        0.64662, 0.63426, 0.62210, 0.61040, 0.59937, 0.58904, 0.57953, 0.57079, 0.56261, 0.55476, 0.54704,\
        0.53928, 0.53136, 0.52318, 0.51467, 0.50555, 0.49490, 0.48309, 0.47103, 0.45921, 0.44814, 0.43841,\
        0.43068, 0.42586, 0.42518, 0.42957, 0.43842, 0.44981, 0.46167, 0.47234, 0.48100, 0.48771, 0.49292],\
        [0.69701, 0.69296, 0.68924, 0.68625, 0.68423, 0.68340, 0.68392, 0.68590, 0.68937, 0.69425, 0.70024,\
        0.70623, 0.71240, 0.71910, 0.72637, 0.73415, 0.74232, 0.75075, 0.75928, 0.76777, 0.77612, 0.78434,\
        0.79240, 0.80017, 0.80691, 0.81280, 0.81861, 0.82450, 0.83030, 0.83584, 0.84104, 0.84590, 0.85047,\
        0.85483, 0.85908, 0.86329, 0.86758, 0.87211, 0.87708, 0.88247, 0.88814, 0.89395, 0.89943, 0.90206,\
        0.90385, 0.90324, 0.90085, 0.89829, 0.89614, 0.89410, 0.89254, 0.89129, 0.88924, 0.88689, 0.88415,\
        0.88051, 0.87624, 0.87166, 0.86672, 0.86125, 0.85519, 0.84865, 0.84173, 0.83454, 0.82715, 0.81972,\
        0.81244, 0.80556, 0.79917, 0.79221, 0.78492, 0.77792, 0.77137, 0.76529, 0.75958, 0.75409, 0.74866,\
        0.74313, 0.73735, 0.73124, 0.72474, 0.71781, 0.71031, 0.70168, 0.69108, 0.67943, 0.66732, 0.65495,\
        0.64247, 0.63005, 0.61784, 0.60606, 0.59496, 0.58462, 0.57497, 0.56598, 0.55753, 0.54943, 0.54149,\
        0.53349, 0.52527, 0.51670, 0.50779, 0.49830, 0.48746, 0.47549, 0.46320, 0.45110, 0.43964, 0.42931,\
        0.42064, 0.41438, 0.41210, 0.41610, 0.42540, 0.43706, 0.44906, 0.46013, 0.46960, 0.47736, 0.48361],\
        [0.69247, 0.68859, 0.68489, 0.68189, 0.67990, 0.67915, 0.67977, 0.68184, 0.68535, 0.69013, 0.69574,\
        0.70128, 0.70685, 0.71292, 0.71953, 0.72664, 0.73414, 0.74201, 0.75021, 0.75863, 0.76712, 0.77557,\
        0.78383, 0.79162, 0.79830, 0.80421, 0.81023, 0.81646, 0.82249, 0.82811, 0.83321, 0.83785, 0.84214,\
        0.84623, 0.85024, 0.85429, 0.85849, 0.86296, 0.86777, 0.87289, 0.87821, 0.88345, 0.88786, 0.89051,\
        0.89194, 0.89128, 0.88905, 0.88662, 0.88450, 0.88270, 0.88128, 0.88000, 0.87841, 0.87642, 0.87395,\
        0.87080, 0.86699, 0.86266, 0.85781, 0.85242, 0.84651, 0.84013, 0.83338, 0.82632, 0.81899, 0.81156,\
        0.80428, 0.79735, 0.79100, 0.78447, 0.77770, 0.77105, 0.76475, 0.75886, 0.75333, 0.74804, 0.74283,\
        0.73750, 0.73189, 0.72587, 0.71942, 0.71253, 0.70515, 0.69673, 0.68634, 0.67480, 0.66271, 0.65035,\
        0.63791, 0.62553, 0.61334, 0.60155, 0.59041, 0.57995, 0.57012, 0.56089, 0.55218, 0.54385, 0.53570,\
        0.52748, 0.51893, 0.50984, 0.50026, 0.49033, 0.47942, 0.46764, 0.45551, 0.44352, 0.43213, 0.42177,\
        0.41288, 0.40598, 0.40192, 0.40574, 0.41526, 0.42660, 0.43826, 0.44926, 0.45905, 0.46741, 0.47439],\
        [0.68655, 0.68277, 0.67918, 0.67625, 0.67434, 0.67366, 0.67436, 0.67646, 0.67987, 0.68436, 0.68949,\
        0.69477, 0.70014, 0.70585, 0.71203, 0.71859, 0.72548, 0.73283, 0.74075, 0.74915, 0.75778, 0.76636,\
        0.77464, 0.78233, 0.78915, 0.79534, 0.80169, 0.80827, 0.81452, 0.82018, 0.82514, 0.82951, 0.83351,\
        0.83734, 0.84116, 0.84509, 0.84923, 0.85363, 0.85831, 0.86317, 0.86805, 0.87262, 0.87634, 0.87876,\
        0.87979, 0.87926, 0.87753, 0.87543, 0.87346, 0.87182, 0.87049, 0.86930, 0.86798, 0.86632, 0.86418,\
        0.86141, 0.85797, 0.85386, 0.84913, 0.84381, 0.83796, 0.83168, 0.82508, 0.81819, 0.81095, 0.80346,\
        0.79625, 0.78938, 0.78284, 0.77638, 0.76991, 0.76356, 0.75752, 0.75184, 0.74653, 0.74149, 0.73655,\
        0.73146, 0.72598, 0.71996, 0.71337, 0.70624, 0.69857, 0.69009, 0.68016, 0.66905, 0.65724, 0.64508,\
        0.63282, 0.62063, 0.60860, 0.59684, 0.58564, 0.57500, 0.56495, 0.55546, 0.54649, 0.53795, 0.52963,\
        0.52126, 0.51252, 0.50313, 0.49304, 0.48260, 0.47158, 0.46003, 0.44827, 0.43674, 0.42587, 0.41612,\
        0.40798, 0.40204, 0.39941, 0.40198, 0.40941, 0.41924, 0.42985, 0.44025, 0.44985, 0.45837, 0.46572],\
        [0.67973, 0.67588, 0.67231, 0.66943, 0.66758, 0.66697, 0.66773, 0.66984, 0.67316, 0.67744, 0.68230,\
        0.68742, 0.69270, 0.69824, 0.70412, 0.71026, 0.71652, 0.72331, 0.73105, 0.73955, 0.74833, 0.75702,\
        0.76533, 0.77303, 0.78008, 0.78670, 0.79331, 0.79997, 0.80632, 0.81197, 0.81670, 0.82074, 0.82445,\
        0.82805, 0.83170, 0.83554, 0.83964, 0.84400, 0.84856, 0.85319, 0.85767, 0.86171, 0.86495, 0.86706,\
        0.86789, 0.86747, 0.86611, 0.86433, 0.86260, 0.86113, 0.85996, 0.85895, 0.85789, 0.85657, 0.85478,\
        0.85237, 0.84923, 0.84533, 0.84070, 0.83540, 0.82954, 0.82329, 0.81679, 0.81011, 0.80309, 0.79563,\
        0.78833, 0.78136, 0.77466, 0.76814, 0.76175, 0.75558, 0.74972, 0.74425, 0.73916, 0.73440, 0.72979,\
        0.72503, 0.71972, 0.71366, 0.70687, 0.69949, 0.69162, 0.68297, 0.67324, 0.66250, 0.65104, 0.63918,\
        0.62716, 0.61522, 0.60350, 0.59186, 0.58054, 0.56973, 0.55943, 0.54965, 0.54041, 0.53164, 0.52320,\
        0.51477, 0.50590, 0.49631, 0.48597, 0.47523, 0.46419, 0.45292, 0.44167, 0.43081, 0.42076, 0.41202,\
        0.40512, 0.40066, 0.39913, 0.40117, 0.40661, 0.41454, 0.42370, 0.43312, 0.44216, 0.45047, 0.45788],\
        [0.67229, 0.66823, 0.66455, 0.66162, 0.65977, 0.65922, 0.66005, 0.66222, 0.66555, 0.66976, 0.67455,\
        0.67966, 0.68495, 0.69043, 0.69611, 0.70200, 0.70742, 0.71360, 0.72142, 0.73015, 0.73914, 0.74790,\
        0.75620, 0.76392, 0.77110, 0.77793, 0.78463, 0.79125, 0.79759, 0.80320, 0.80762, 0.81132, 0.81480,\
        0.81821, 0.82174, 0.82553, 0.82964, 0.83401, 0.83851, 0.84296, 0.84715, 0.85082, 0.85373, 0.85561,\
        0.85633, 0.85597, 0.85479, 0.85324, 0.85171, 0.85045, 0.84951, 0.84878, 0.84804, 0.84709, 0.84569,\
        0.84365, 0.84081, 0.83709, 0.83254, 0.82721, 0.82125, 0.81491, 0.80843, 0.80183, 0.79501, 0.78767,\
        0.78034, 0.77323, 0.76637, 0.75973, 0.75331, 0.74719, 0.74144, 0.73612, 0.73122, 0.72673, 0.72252,\
        0.71826, 0.71325, 0.70711, 0.70012, 0.69258, 0.68446, 0.67562, 0.66595, 0.65546, 0.64432, 0.63274,\
        0.62096, 0.60921, 0.59767, 0.58627, 0.57503, 0.56412, 0.55358, 0.54349, 0.53390, 0.52486, 0.51629,\
        0.50788, 0.49892, 0.48910, 0.47874, 0.46806, 0.45721, 0.44635, 0.43571, 0.42564, 0.41652, 0.40885,\
        0.40312, 0.39974, 0.39889, 0.40072, 0.40507, 0.41152, 0.41928, 0.42760, 0.43588, 0.44375, 0.45096],\
        [0.66448, 0.66006, 0.65613, 0.65304, 0.65112, 0.65060, 0.65155, 0.65386, 0.65733, 0.66166, 0.66654,\
        0.67173, 0.67710, 0.68257, 0.68811, 0.69355, 0.69824, 0.70433, 0.71243, 0.72147, 0.73056, 0.73926,\
        0.74741, 0.75500, 0.76210, 0.76886, 0.77543, 0.78181, 0.78786, 0.79323, 0.79747, 0.80106, 0.80447,\
        0.80778, 0.81123, 0.81505, 0.81922, 0.82366, 0.82820, 0.83255, 0.83653, 0.83998, 0.84271, 0.84448,\
        0.84512, 0.84472, 0.84358, 0.84211, 0.84071, 0.83967, 0.83904, 0.83867, 0.83832, 0.83780, 0.83685,\
        0.83523, 0.83269, 0.82914, 0.82463, 0.81928, 0.81317, 0.80656, 0.80008, 0.79344, 0.78657, 0.77940,\
        0.77211, 0.76492, 0.75792, 0.75114, 0.74463, 0.73848, 0.73276, 0.72753, 0.72277, 0.71843, 0.71460,\
        0.71111, 0.70665, 0.70035, 0.69318, 0.68539, 0.67703, 0.66805, 0.65839, 0.64809, 0.63723, 0.62594,\
        0.61440, 0.60284, 0.59149, 0.58031, 0.56921, 0.55826, 0.54752, 0.53708, 0.52707, 0.51765, 0.50884,\
        0.50033, 0.49135, 0.48162, 0.47145, 0.46105, 0.45059, 0.44026, 0.43032, 0.42107, 0.41288, 0.40616,\
        0.40136, 0.39880, 0.39846, 0.40027, 0.40402, 0.40942, 0.41604, 0.42332, 0.43080, 0.43810, 0.44497],\
        [0.65653, 0.65164, 0.64731, 0.64393, 0.64186, 0.64135, 0.64247, 0.64504, 0.64880, 0.65338, 0.65849,\
        0.66386, 0.66936, 0.67487, 0.68031, 0.68555, 0.69069, 0.69687, 0.70485, 0.71401, 0.72289, 0.73121,\
        0.73898, 0.74622, 0.75300, 0.75946, 0.76567, 0.77165, 0.77724, 0.78220, 0.78640, 0.79004, 0.79345,\
        0.79677, 0.80021, 0.80415, 0.80840, 0.81300, 0.81768, 0.82200, 0.82581, 0.82915, 0.83190, 0.83368,\
        0.83426, 0.83374, 0.83248, 0.83093, 0.82956, 0.82874, 0.82851, 0.82856, 0.82864, 0.82859, 0.82817,\
        0.82704, 0.82486, 0.82142, 0.81688, 0.81157, 0.80556, 0.79853, 0.79188, 0.78513, 0.77815, 0.77096,\
        0.76368, 0.75643, 0.74932, 0.74240, 0.73576, 0.72951, 0.72378, 0.71862, 0.71395, 0.70964, 0.70583,\
        0.70312, 0.69969, 0.69316, 0.68573, 0.67774, 0.66922, 0.66016, 0.65056, 0.64044, 0.62988, 0.61892,\
        0.60768, 0.59640, 0.58528, 0.57426, 0.56328, 0.55234, 0.54145, 0.53067, 0.52015, 0.51024, 0.50111,\
        0.49242, 0.48350, 0.47406, 0.46425, 0.45427, 0.44432, 0.43463, 0.42542, 0.41699, 0.40967, 0.40381,\
        0.39976, 0.39788, 0.39797, 0.39984, 0.40318, 0.40784, 0.41355, 0.41996, 0.42667, 0.43336, 0.43980],\
        [0.64868, 0.64323, 0.63837, 0.63456, 0.63224, 0.63173, 0.63308, 0.63606, 0.64024, 0.64520, 0.65061,\
        0.65623, 0.66191, 0.66756, 0.67314, 0.67870, 0.68452, 0.69121, 0.69925, 0.70792, 0.71600, 0.72362,\
        0.73081, 0.73751, 0.74378, 0.74974, 0.75547, 0.76095, 0.76606, 0.77069, 0.77476, 0.77843, 0.78188,\
        0.78531, 0.78894, 0.79295, 0.79730, 0.80211, 0.80701, 0.81132, 0.81489, 0.81825, 0.82127, 0.82326,\
        0.82374, 0.82296, 0.82149, 0.81975, 0.81824, 0.81766, 0.81798, 0.81846, 0.81891, 0.81931, 0.81948,\
        0.81898, 0.81725, 0.81388, 0.80910, 0.80367, 0.79794, 0.79106, 0.78395, 0.77689, 0.76972, 0.76244,\
        0.75511, 0.74781, 0.74061, 0.73358, 0.72679, 0.72038, 0.71459, 0.70952, 0.70501, 0.70082, 0.69687,\
        0.69398, 0.69044, 0.68459, 0.67738, 0.66943, 0.66092, 0.65190, 0.64241, 0.63251, 0.62229, 0.61176,\
        0.60089, 0.58991, 0.57901, 0.56816, 0.55733, 0.54649, 0.53560, 0.52461, 0.51360, 0.50312, 0.49382,\
        0.48497, 0.47602, 0.46678, 0.45732, 0.44781, 0.43843, 0.42940, 0.42094, 0.41331, 0.40683, 0.40181,\
        0.39851, 0.39710, 0.39745, 0.39933, 0.40239, 0.40652, 0.41156, 0.41724, 0.42327, 0.42939, 0.43537],\
        [0.64120, 0.63511, 0.62962, 0.62525, 0.62255, 0.62200, 0.62370, 0.62722, 0.63193, 0.63734, 0.64309,\
        0.64896, 0.65484, 0.66069, 0.66655, 0.67253, 0.67892, 0.68609, 0.69418, 0.70223, 0.70917, 0.71615,\
        0.72275, 0.72877, 0.73437, 0.73972, 0.74491, 0.74990, 0.75459, 0.75889, 0.76281, 0.76643, 0.76994,\
        0.77350, 0.77731, 0.78147, 0.78604, 0.79099, 0.79603, 0.80032, 0.80367, 0.80720, 0.81084, 0.81323,\
        0.81348, 0.81227, 0.81059, 0.80873, 0.80691, 0.80653, 0.80764, 0.80838, 0.80902, 0.80982, 0.81060,\
        0.81081, 0.80970, 0.80647, 0.80124, 0.79558, 0.78958, 0.78291, 0.77579, 0.76853, 0.76119, 0.75380,\
        0.74640, 0.73907, 0.73185, 0.72477, 0.71785, 0.71127, 0.70535, 0.70042, 0.69606, 0.69199, 0.68815,\
        0.68459, 0.68048, 0.67496, 0.66816, 0.66048, 0.65215, 0.64327, 0.63392, 0.62425, 0.61437, 0.60434,\
        0.59390, 0.58324, 0.57258, 0.56197, 0.55137, 0.54076, 0.53007, 0.51924, 0.50813, 0.49721, 0.48756,\
        0.47833, 0.46918, 0.45999, 0.45080, 0.44172, 0.43291, 0.42455, 0.41684, 0.40998, 0.40427, 0.40002,\
        0.39738, 0.39637, 0.39689, 0.39871, 0.40159, 0.40536, 0.40989, 0.41499, 0.42044, 0.42603, 0.43156],\
        [0.63433, 0.62761, 0.62143, 0.61640, 0.61318, 0.61253, 0.61472, 0.61892, 0.62419, 0.63000, 0.63602,\
        0.64208, 0.64812, 0.65415, 0.66023, 0.66650, 0.67316, 0.68035, 0.68794, 0.69512, 0.70182, 0.70861,\
        0.71469, 0.71989, 0.72471, 0.72944, 0.73412, 0.73867, 0.74298, 0.74698, 0.75069, 0.75423, 0.75775,\
        0.76140, 0.76534, 0.76967, 0.77440, 0.77942, 0.78440, 0.78877, 0.79250, 0.79637, 0.80050, 0.80336,\
        0.80311, 0.80141, 0.79969, 0.79795, 0.79622, 0.79599, 0.79740, 0.79817, 0.79896, 0.80006, 0.80130,\
        0.80205, 0.80158, 0.79867, 0.79332, 0.78735, 0.78106, 0.77432, 0.76720, 0.75987, 0.75245, 0.74498,\
        0.73754, 0.73019, 0.72302, 0.71603, 0.70909, 0.70242, 0.69649, 0.69155, 0.68720, 0.68311, 0.67914,\
        0.67509, 0.67052, 0.66498, 0.65845, 0.65107, 0.64300, 0.63434, 0.62518, 0.61568, 0.60606, 0.59640,\
        0.58646, 0.57624, 0.56592, 0.55558, 0.54527, 0.53498, 0.52466, 0.51430, 0.50390, 0.49288, 0.48228,\
        0.47239, 0.46290, 0.45367, 0.44468, 0.43600, 0.42775, 0.42007, 0.41309, 0.40698, 0.40196, 0.39838,\
        0.39632, 0.39566, 0.39631, 0.39810, 0.40083, 0.40433, 0.40847, 0.41310, 0.41806, 0.42316, 0.42827],\
        [0.62832, 0.62103, 0.61420, 0.60855, 0.60479, 0.60386, 0.60677, 0.61160, 0.61724, 0.62328, 0.62943,\
        0.63555, 0.64164, 0.64773, 0.65390, 0.66026, 0.66691, 0.67386, 0.68093, 0.68779, 0.69445, 0.70107,\
        0.70643, 0.71065, 0.71474, 0.71893, 0.72319, 0.72740, 0.73139, 0.73507, 0.73854, 0.74193, 0.74541,\
        0.74909, 0.75310, 0.75754, 0.76239, 0.76750, 0.77255, 0.77720, 0.78145, 0.78559, 0.78950, 0.79205,\
        0.79194, 0.79036, 0.78866, 0.78705, 0.78584, 0.78575, 0.78667, 0.78772, 0.78884, 0.79016, 0.79147,\
        0.79224, 0.79170, 0.78899, 0.78424, 0.77841, 0.77205, 0.76527, 0.75817, 0.75085, 0.74341, 0.73594,\
        0.72848, 0.72111, 0.71403, 0.70732, 0.70056, 0.69394, 0.68807, 0.68299, 0.67847, 0.67422, 0.67000,\
        0.66555, 0.66061, 0.65494, 0.64851, 0.64139, 0.63364, 0.62527, 0.61632, 0.60698, 0.59762, 0.58828,\
        0.57874, 0.56894, 0.55897, 0.54893, 0.53891, 0.52893, 0.51895, 0.50886, 0.49853, 0.48775, 0.47692,\
        0.46664, 0.45691, 0.44766, 0.43887, 0.43058, 0.42288, 0.41587, 0.40967, 0.40434, 0.40001, 0.39701,\
        0.39537, 0.39499, 0.39576, 0.39754, 0.40015, 0.40343, 0.40725, 0.41149, 0.41602, 0.42071, 0.42541],\
        [0.62328, 0.61563, 0.60831, 0.60220, 0.59842, 0.59754, 0.60070, 0.60542, 0.61108, 0.61712, 0.62319,\
        0.62920, 0.63519, 0.64122, 0.64735, 0.65363, 0.66012, 0.66676, 0.67346, 0.68006, 0.68652, 0.69258,\
        0.69727, 0.70089, 0.70451, 0.70832, 0.71228, 0.71626, 0.71998, 0.72332, 0.72647, 0.72967, 0.73306,\
        0.73670, 0.74069, 0.74518, 0.75016, 0.75544, 0.76068, 0.76560, 0.77010, 0.77418, 0.77758, 0.77962,\
        0.77984, 0.77876, 0.77726, 0.77587, 0.77499, 0.77497, 0.77575, 0.77695, 0.77834, 0.77979, 0.78101,\
        0.78153, 0.78076, 0.77823, 0.77399, 0.76855, 0.76236, 0.75568, 0.74867, 0.74142, 0.73403, 0.72663,\
        0.71924, 0.71181, 0.70470, 0.69822, 0.69177, 0.68539, 0.67958, 0.67442, 0.66975, 0.66531, 0.66082,\
        0.65604, 0.65076, 0.64489, 0.63846, 0.63156, 0.62416, 0.61618, 0.60753, 0.59843, 0.58933, 0.58022,\
        0.57095, 0.56146, 0.55178, 0.54198, 0.53215, 0.52240, 0.51271, 0.50275, 0.49234, 0.48166, 0.47099,\
        0.46070, 0.45095, 0.44180, 0.43326, 0.42537, 0.41820, 0.41186, 0.40644, 0.40199, 0.39835, 0.39582,\
        0.39451, 0.39436, 0.39524, 0.39702, 0.39954, 0.40264, 0.40620, 0.41012, 0.41428, 0.41858, 0.42291],\
        [0.61915, 0.61139, 0.60400, 0.59769, 0.59395, 0.59321, 0.59553, 0.59985, 0.60533, 0.61121, 0.61701,\
        0.62273, 0.62852, 0.63440, 0.64039, 0.64651, 0.65275, 0.65907, 0.66542, 0.67167, 0.67765, 0.68305,\
        0.68746, 0.69106, 0.69445, 0.69794, 0.70160, 0.70545, 0.70889, 0.71180, 0.71458, 0.71757, 0.72088,\
        0.72442, 0.72827, 0.73273, 0.73788, 0.74342, 0.74889, 0.75395, 0.75847, 0.76233, 0.76529, 0.76703,\
        0.76737, 0.76662, 0.76536, 0.76415, 0.76345, 0.76354, 0.76439, 0.76577, 0.76735, 0.76887, 0.76998,\
        0.77027, 0.76935, 0.76692, 0.76302, 0.75793, 0.75199, 0.74553, 0.73871, 0.73160, 0.72427, 0.71698,\
        0.70989, 0.70254, 0.69527, 0.68878, 0.68250, 0.67642, 0.67076, 0.66562, 0.66088, 0.65632, 0.65166,\
        0.64662, 0.64101, 0.63484, 0.62830, 0.62157, 0.61455, 0.60701, 0.59882, 0.59000, 0.58110, 0.57217,\
        0.56312, 0.55388, 0.54442, 0.53476, 0.52499, 0.51527, 0.50578, 0.49600, 0.48560, 0.47504, 0.46458,\
        0.45447, 0.44489, 0.43596, 0.42775, 0.42030, 0.41366, 0.40792, 0.40318, 0.39947, 0.39657, 0.39457,\
        0.39363, 0.39372, 0.39474, 0.39654, 0.39899, 0.40196, 0.40531, 0.40894, 0.41277, 0.41673, 0.42071],\
        [0.61558, 0.60777, 0.60045, 0.59415, 0.59004, 0.58881, 0.59037, 0.59413, 0.59935, 0.60504, 0.61037,\
        0.61576, 0.62133, 0.62706, 0.63290, 0.63881, 0.64477, 0.65077, 0.65677, 0.66265, 0.66823, 0.67326,\
        0.67757, 0.68126, 0.68465, 0.68799, 0.69143, 0.69497, 0.69805, 0.70059, 0.70302, 0.70583, 0.70913,\
        0.71250, 0.71605, 0.72034, 0.72577, 0.73168, 0.73731, 0.74237, 0.74674, 0.75031, 0.75294, 0.75445,\
        0.75480, 0.75417, 0.75299, 0.75183, 0.75120, 0.75142, 0.75248, 0.75410, 0.75590, 0.75750, 0.75853,\
        0.75868, 0.75766, 0.75530, 0.75162, 0.74677, 0.74105, 0.73481, 0.72829, 0.72146, 0.71417, 0.70683,\
        0.70008, 0.69311, 0.68594, 0.67926, 0.67298, 0.66703, 0.66151, 0.65645, 0.65175, 0.64721, 0.64252,\
        0.63734, 0.63144, 0.62484, 0.61799, 0.61137, 0.60472, 0.59750, 0.58965, 0.58129, 0.57270, 0.56400,\
        0.55518, 0.54618, 0.53694, 0.52740, 0.51759, 0.50764, 0.49801, 0.48840, 0.47831, 0.46803, 0.45785,\
        0.44800, 0.43871, 0.43011, 0.42232, 0.41535, 0.40924, 0.40407, 0.39992, 0.39683, 0.39459, 0.39318,\
        0.39267, 0.39304, 0.39422, 0.39610, 0.39855, 0.40141, 0.40455, 0.40792, 0.41145, 0.41509, 0.41876],\
        [0.61212, 0.60402, 0.59638, 0.58987, 0.58540, 0.58355, 0.58437, 0.58748, 0.59222, 0.59756, 0.60260,\
        0.60788, 0.61337, 0.61903, 0.62475, 0.63046, 0.63614, 0.64184, 0.64754, 0.65314, 0.65847, 0.66335,\
        0.66766, 0.67146, 0.67492, 0.67822, 0.68147, 0.68460, 0.68738, 0.68979, 0.69213, 0.69484, 0.69800,\
        0.70118, 0.70432, 0.70822, 0.71425, 0.72053, 0.72612, 0.73096, 0.73503, 0.73828, 0.74061, 0.74194,\
        0.74224, 0.74158, 0.74030, 0.73897, 0.73827, 0.73864, 0.74002, 0.74199, 0.74406, 0.74578, 0.74679,\
        0.74683, 0.74575, 0.74345, 0.73993, 0.73524, 0.72962, 0.72353, 0.71738, 0.71112, 0.70401, 0.69651,\
        0.68972, 0.68297, 0.67610, 0.66944, 0.66313, 0.65722, 0.65177, 0.64681, 0.64225, 0.63787, 0.63334,\
        0.62825, 0.62219, 0.61508, 0.60760, 0.60101, 0.59459, 0.58765, 0.58016, 0.57224, 0.56403, 0.55563,\
        0.54707, 0.53836, 0.52941, 0.52006, 0.51028, 0.50024, 0.49040, 0.48072, 0.47084, 0.46084, 0.45095,\
        0.44142, 0.43246, 0.42426, 0.41695, 0.41057, 0.40505, 0.40043, 0.39691, 0.39436, 0.39267, 0.39177,\
        0.39166, 0.39231, 0.39368, 0.39566, 0.39814, 0.40092, 0.40387, 0.40699, 0.41026, 0.41362, 0.41701],\
        [0.60842, 0.59974, 0.59148, 0.58444, 0.57941, 0.57691, 0.57703, 0.57943, 0.58349, 0.58836, 0.59351,\
        0.59892, 0.60449, 0.61018, 0.61589, 0.62143, 0.62683, 0.63225, 0.63774, 0.64320, 0.64845, 0.65332,\
        0.65771, 0.66164, 0.66520, 0.66848, 0.67157, 0.67445, 0.67707, 0.67947, 0.68186, 0.68450, 0.68742,\
        0.69045, 0.69348, 0.69756, 0.70400, 0.71017, 0.71525, 0.71966, 0.72336, 0.72623, 0.72827, 0.72949,\
        0.72978, 0.72903, 0.72744, 0.72570, 0.72473, 0.72528, 0.72713, 0.72954, 0.73193, 0.73382, 0.73482,\
        0.73474, 0.73359, 0.73138, 0.72804, 0.72349, 0.71786, 0.71172, 0.70581, 0.70016, 0.69362, 0.68630,\
        0.67932, 0.67254, 0.66582, 0.65922, 0.65287, 0.64692, 0.64149, 0.63664, 0.63226, 0.62813, 0.62394,\
        0.61922, 0.61337, 0.60599, 0.59789, 0.59095, 0.58446, 0.57768, 0.57051, 0.56297, 0.55512, 0.54702,\
        0.53873, 0.53034, 0.52179, 0.51275, 0.50310, 0.49312, 0.48314, 0.47329, 0.46346, 0.45368, 0.44408,\
        0.43486, 0.42624, 0.41845, 0.41168, 0.40598, 0.40122, 0.39719, 0.39419, 0.39213, 0.39089, 0.39042,\
        0.39066, 0.39157, 0.39310, 0.39516, 0.39763, 0.40034, 0.40316, 0.40611, 0.40916, 0.41228, 0.41541],\
        [0.60423, 0.59469, 0.58551, 0.57761, 0.57184, 0.56869, 0.56820, 0.57004, 0.57359, 0.57821, 0.58340,\
        0.58892, 0.59464, 0.60048, 0.60631, 0.61165, 0.61677, 0.62198, 0.62740, 0.63288, 0.63819, 0.64316,\
        0.64769, 0.65176, 0.65542, 0.65873, 0.66175, 0.66451, 0.66706, 0.66948, 0.67193, 0.67455, 0.67740,\
        0.68048, 0.68396, 0.68854, 0.69447, 0.69989, 0.70430, 0.70830, 0.71162, 0.71408, 0.71580, 0.71696,\
        0.71739, 0.71661, 0.71462, 0.71230, 0.71079, 0.71165, 0.71409, 0.71689, 0.71956, 0.72166, 0.72259,\
        0.72230, 0.72106, 0.71897, 0.71592, 0.71166, 0.70601, 0.69976, 0.69394, 0.68835, 0.68218, 0.67540,\
        0.66855, 0.66183, 0.65519, 0.64863, 0.64221, 0.63612, 0.63063, 0.62586, 0.62165, 0.61776, 0.61395,\
        0.60979, 0.60445, 0.59721, 0.58904, 0.58150, 0.57457, 0.56774, 0.56078, 0.55358, 0.54608, 0.53823,\
        0.53010, 0.52193, 0.51392, 0.50542, 0.49580, 0.48584, 0.47584, 0.46597, 0.45624, 0.44669, 0.43739,\
        0.42849, 0.42020, 0.41278, 0.40653, 0.40151, 0.39748, 0.39418, 0.39172, 0.39012, 0.38930, 0.38918,\
        0.38972, 0.39085, 0.39251, 0.39461, 0.39705, 0.39968, 0.40243, 0.40526, 0.40814, 0.41105, 0.41395],\
        [0.59982, 0.58877, 0.57815, 0.56911, 0.56255, 0.55887, 0.55801, 0.55955, 0.56289, 0.56739, 0.57259,\
        0.57819, 0.58400, 0.58993, 0.59576, 0.60097, 0.60592, 0.61104, 0.61655, 0.62220, 0.62768, 0.63282,\
        0.63753, 0.64178, 0.64557, 0.64894, 0.65195, 0.65468, 0.65722, 0.65969, 0.66222, 0.66490, 0.66781,\
        0.67103, 0.67475, 0.67923, 0.68427, 0.68899, 0.69308, 0.69668, 0.69965, 0.70167, 0.70305, 0.70417,\
        0.70485, 0.70421, 0.70181, 0.69923, 0.69738, 0.69853, 0.70111, 0.70401, 0.70678, 0.70924, 0.70998,\
        0.70932, 0.70802, 0.70605, 0.70332, 0.69949, 0.69412, 0.68807, 0.68222, 0.67645, 0.67036, 0.66391,\
        0.65730, 0.65073, 0.64424, 0.63777, 0.63124, 0.62486, 0.61918, 0.61451, 0.61044, 0.60663, 0.60296,\
        0.59914, 0.59393, 0.58704, 0.57929, 0.57172, 0.56458, 0.55774, 0.55097, 0.54410, 0.53698, 0.52941,\
        0.52128, 0.51292, 0.50525, 0.49750, 0.48806, 0.47819, 0.46835, 0.45865, 0.44917, 0.43993, 0.43101,\
        0.42250, 0.41458, 0.40747, 0.40167, 0.39735, 0.39399, 0.39138, 0.38949, 0.38835, 0.38790, 0.38809,\
        0.38888, 0.39019, 0.39196, 0.39410, 0.39651, 0.39908, 0.40176, 0.40448, 0.40720, 0.40991, 0.41260],\
        [0.59388, 0.58099, 0.56882, 0.55872, 0.55153, 0.54760, 0.54674, 0.54834, 0.55173, 0.55626, 0.56147,\
        0.56706, 0.57282, 0.57862, 0.58423, 0.58946, 0.59446, 0.59960, 0.60535, 0.61123, 0.61692, 0.62225,\
        0.62716, 0.63162, 0.63557, 0.63905, 0.64210, 0.64484, 0.64743, 0.64998, 0.65260, 0.65539, 0.65840,\
        0.66171, 0.66540, 0.66951, 0.67383, 0.67794, 0.68159, 0.68472, 0.68723, 0.68897, 0.69014, 0.69106,\
        0.69156, 0.69087, 0.68870, 0.68643, 0.68514, 0.68584, 0.68801, 0.69069, 0.69342, 0.69600, 0.69653,\
        0.69569, 0.69444, 0.69258, 0.69001, 0.68641, 0.68161, 0.67608, 0.67034, 0.66448, 0.65840, 0.65206,\
        0.64561, 0.63921, 0.63295, 0.62671, 0.62017, 0.61334, 0.60718, 0.60275, 0.59873, 0.59489, 0.59113,\
        0.58707, 0.58195, 0.57558, 0.56845, 0.56121, 0.55420, 0.54750, 0.54098, 0.53450, 0.52786, 0.52078,\
        0.51280, 0.50380, 0.49577, 0.48812, 0.47938, 0.47003, 0.46059, 0.45129, 0.44223, 0.43346, 0.42504,\
        0.41706, 0.40967, 0.40305, 0.39765, 0.39389, 0.39107, 0.38900, 0.38760, 0.38685, 0.38673, 0.38719,\
        0.38817, 0.38962, 0.39147, 0.39364, 0.39605, 0.39861, 0.40120, 0.40378, 0.40633, 0.40885, 0.41134],\
        [0.58255, 0.56985, 0.55716, 0.54646, 0.53901, 0.53528, 0.53488, 0.53693, 0.54059, 0.54523, 0.55042,\
        0.55590, 0.56148, 0.56704, 0.57245, 0.57766, 0.58279, 0.58814, 0.59393, 0.59993, 0.60580, 0.61130,\
        0.61646, 0.62118, 0.62535, 0.62897, 0.63209, 0.63489, 0.63754, 0.64021, 0.64298, 0.64591, 0.64902,\
        0.65234, 0.65588, 0.65960, 0.66334, 0.66686, 0.66998, 0.67261, 0.67466, 0.67611, 0.67706, 0.67763,\
        0.67767, 0.67685, 0.67520, 0.67351, 0.67262, 0.67307, 0.67470, 0.67696, 0.67937, 0.68134, 0.68203,\
        0.68156, 0.68044, 0.67873, 0.67631, 0.67299, 0.66872, 0.66369, 0.65821, 0.65240, 0.64630, 0.63996,\
        0.63352, 0.62719, 0.62117, 0.61539, 0.60916, 0.60201, 0.59542, 0.59075, 0.58668, 0.58278, 0.57885,\
        0.57456, 0.56951, 0.56356, 0.55697, 0.55014, 0.54341, 0.53696, 0.53078, 0.52470, 0.51855, 0.51218,\
        0.50499, 0.49604, 0.48721, 0.47894, 0.47040, 0.46156, 0.45267, 0.44393, 0.43545, 0.42730, 0.41953,\
        0.41223, 0.40554, 0.39968, 0.39489, 0.39136, 0.38886, 0.38715, 0.38611, 0.38568, 0.38582, 0.38649,\
        0.38762, 0.38918, 0.39108, 0.39327, 0.39566, 0.39817, 0.40068, 0.40312, 0.40551, 0.40786, 0.41016],\
        [0.56933, 0.55678, 0.54403, 0.53303, 0.52562, 0.52265, 0.52323, 0.52602, 0.53006, 0.53479, 0.53986,\
        0.54510, 0.55037, 0.55562, 0.56079, 0.56589, 0.57104, 0.57643, 0.58216, 0.58813, 0.59407, 0.59979,\
        0.60527, 0.61031, 0.61477, 0.61860, 0.62182, 0.62469, 0.62746, 0.63031, 0.63329, 0.63640, 0.63961,\
        0.64291, 0.64627, 0.64964, 0.65288, 0.65586, 0.65845, 0.66058, 0.66219, 0.66330, 0.66393, 0.66413,\
        0.66383, 0.66295, 0.66163, 0.66037, 0.65972, 0.66001, 0.66122, 0.66301, 0.66494, 0.66650, 0.66723,\
        0.66709, 0.66624, 0.66476, 0.66259, 0.65962, 0.65580, 0.65121, 0.64599, 0.64027, 0.63414, 0.62770,\
        0.62111, 0.61463, 0.60864, 0.60324, 0.59761, 0.59060, 0.58412, 0.57894, 0.57459, 0.57055, 0.56650,\
        0.56212, 0.55716, 0.55152, 0.54534, 0.53887, 0.53241, 0.52622, 0.52041, 0.51470, 0.50882, 0.50273,\
        0.49626, 0.48767, 0.47865, 0.46998, 0.46148, 0.45304, 0.44473, 0.43666, 0.42890, 0.42150, 0.41450,\
        0.40796, 0.40202, 0.39688, 0.39269, 0.38951, 0.38728, 0.38583, 0.38504, 0.38485, 0.38519, 0.38601,\
        0.38726, 0.38888, 0.39081, 0.39298, 0.39532, 0.39774, 0.40015, 0.40248, 0.40473, 0.40692, 0.40905],\
        [0.55628, 0.54387, 0.53118, 0.51989, 0.51254, 0.51097, 0.51294, 0.51647, 0.52077, 0.52540, 0.53017,\
        0.53500, 0.53983, 0.54466, 0.54948, 0.55434, 0.55933, 0.56457, 0.57011, 0.57589, 0.58178, 0.58764,\
        0.59338, 0.59876, 0.60364, 0.60777, 0.61114, 0.61414, 0.61711, 0.62024, 0.62352, 0.62687, 0.63020,\
        0.63349, 0.63669, 0.63974, 0.64256, 0.64505, 0.64715, 0.64880, 0.64999, 0.65073, 0.65103, 0.65091,\
        0.65038, 0.64945, 0.64830, 0.64727, 0.64671, 0.64688, 0.64777, 0.64917, 0.65070, 0.65199, 0.65272,\
        0.65278, 0.65220, 0.65101, 0.64916, 0.64657, 0.64315, 0.63891, 0.63391, 0.62824, 0.62202, 0.61539,\
        0.60855, 0.60171, 0.59548, 0.58992, 0.58429, 0.57813, 0.57219, 0.56705, 0.56260, 0.55850, 0.55442,\
        0.55007, 0.54525, 0.53990, 0.53405, 0.52786, 0.52156, 0.51551, 0.51009, 0.50475, 0.49887, 0.49269,\
        0.48592, 0.47799, 0.46942, 0.46088, 0.45260, 0.44460, 0.43691, 0.42957, 0.42262, 0.41607, 0.40992,\
        0.40421, 0.39903, 0.39456, 0.39092, 0.38814, 0.38619, 0.38496, 0.38438, 0.38436, 0.38484, 0.38577,\
        0.38708, 0.38873, 0.39065, 0.39278, 0.39504, 0.39736, 0.39966, 0.40187, 0.40399, 0.40604, 0.40801],\
        [0.54456, 0.53268, 0.52062, 0.50956, 0.50179, 0.50246, 0.50525, 0.50905, 0.51320, 0.51741, 0.52162,\
        0.52584, 0.53009, 0.53437, 0.53873, 0.54320, 0.54785, 0.55277, 0.55798, 0.56348, 0.56918, 0.57500,\
        0.58080, 0.58641, 0.59165, 0.59616, 0.59991, 0.60322, 0.60649, 0.61005, 0.61373, 0.61736, 0.62084,\
        0.62414, 0.62722, 0.63003, 0.63249, 0.63456, 0.63621, 0.63743, 0.63823, 0.63863, 0.63864, 0.63827,\
        0.63757, 0.63660, 0.63551, 0.63454, 0.63398, 0.63402, 0.63470, 0.63582, 0.63709, 0.63820, 0.63891,\
        0.63910, 0.63873, 0.63781, 0.63630, 0.63411, 0.63107, 0.62709, 0.62221, 0.61650, 0.61008, 0.60311,\
        0.59593, 0.58877, 0.58217, 0.57624, 0.57061, 0.56506, 0.55979, 0.55505, 0.55081, 0.54685, 0.54288,\
        0.53869, 0.53410, 0.52903, 0.52351, 0.51762, 0.51147, 0.50526, 0.50004, 0.49508, 0.48918, 0.48278,\
        0.47577, 0.46804, 0.45986, 0.45167, 0.44378, 0.43630, 0.42929, 0.42274, 0.41666, 0.41102, 0.40579,\
        0.40092, 0.39649, 0.39266, 0.38954, 0.38716, 0.38549, 0.38449, 0.38408, 0.38419, 0.38477, 0.38576,\
        0.38711, 0.38875, 0.39063, 0.39269, 0.39486, 0.39706, 0.39922, 0.40131, 0.40331, 0.40523, 0.40704],\
        [0.53475, 0.52395, 0.51342, 0.50436, 0.49866, 0.49826, 0.50056, 0.50399, 0.50747, 0.51091, 0.51433,\
        0.51778, 0.52131, 0.52495, 0.52873, 0.53268, 0.53685, 0.54130, 0.54607, 0.55116, 0.55654, 0.56216,\
        0.56791, 0.57359, 0.57901, 0.58390, 0.58821, 0.59210, 0.59589, 0.59994, 0.60404, 0.60795, 0.61159,\
        0.61495, 0.61796, 0.62059, 0.62278, 0.62450, 0.62576, 0.62661, 0.62709, 0.62722, 0.62700, 0.62647,\
        0.62566, 0.62466, 0.62358, 0.62260, 0.62195, 0.62188, 0.62244, 0.62342, 0.62452, 0.62552, 0.62621,\
        0.62646, 0.62623, 0.62552, 0.62433, 0.62254, 0.61985, 0.61603, 0.61112, 0.60521, 0.59844, 0.59097,\
        0.58328, 0.57577, 0.56884, 0.56265, 0.55710, 0.55203, 0.54742, 0.54327, 0.53947, 0.53585, 0.53217,\
        0.52825, 0.52396, 0.51924, 0.51409, 0.50858, 0.50278, 0.49679, 0.49129, 0.48602, 0.48009, 0.47346,\
        0.46623, 0.45848, 0.45049, 0.44262, 0.43516, 0.42828, 0.42199, 0.41627, 0.41107, 0.40636, 0.40206,\
        0.39803, 0.39431, 0.39111, 0.38851, 0.38652, 0.38515, 0.38436, 0.38410, 0.38432, 0.38497, 0.38599,\
        0.38733, 0.38893, 0.39076, 0.39273, 0.39480, 0.39687, 0.39888, 0.40082, 0.40273, 0.40450, 0.40613],\
        [0.52684, 0.51744, 0.50876, 0.50182, 0.49752, 0.49647, 0.49804, 0.50064, 0.50325, 0.50574, 0.50825,\
        0.51086, 0.51361, 0.51655, 0.51969, 0.52303, 0.52661, 0.53047, 0.53467, 0.53924, 0.54419, 0.54951,\
        0.55510, 0.56080, 0.56640, 0.57168, 0.57656, 0.58112, 0.58556, 0.59003, 0.59447, 0.59865, 0.60249,\
        0.60595, 0.60896, 0.61149, 0.61351, 0.61497, 0.61592, 0.61648, 0.61674, 0.61670, 0.61636, 0.61573,\
        0.61490, 0.61393, 0.61288, 0.61187, 0.61110, 0.61092, 0.61148, 0.61242, 0.61343, 0.61434, 0.61500,\
        0.61529, 0.61510, 0.61447, 0.61355, 0.61218, 0.60984, 0.60602, 0.60088, 0.59459, 0.58727, 0.57926,\
        0.57102, 0.56304, 0.55576, 0.54943, 0.54406, 0.53951, 0.53559, 0.53212, 0.52892, 0.52578, 0.52253,\
        0.51899, 0.51507, 0.51075, 0.50603, 0.50095, 0.49556, 0.48993, 0.48427, 0.47849, 0.47220, 0.46523,\
        0.45768, 0.44973, 0.44171, 0.43402, 0.42696, 0.42069, 0.41517, 0.41030, 0.40596, 0.40212, 0.39867,\
        0.39545, 0.39249, 0.38992, 0.38782, 0.38623, 0.38515, 0.38456, 0.38444, 0.38473, 0.38541, 0.38643,\
        0.38773, 0.38928, 0.39102, 0.39290, 0.39486, 0.39684, 0.39878, 0.40061, 0.40229, 0.40385, 0.40529],\
        [0.52051, 0.51250, 0.50540, 0.49991, 0.49649, 0.49538, 0.49621, 0.49804, 0.49995, 0.50162, 0.50334,\
        0.50514, 0.50712, 0.50937, 0.51186, 0.51453, 0.51743, 0.52062, 0.52415, 0.52808, 0.53248, 0.53739,\
        0.54278, 0.54848, 0.55428, 0.55994, 0.56536, 0.57052, 0.57551, 0.58038, 0.58509, 0.58950, 0.59355,\
        0.59716, 0.60023, 0.60278, 0.60475, 0.60605, 0.60679, 0.60718, 0.60734, 0.60728, 0.60693, 0.60628,\
        0.60554, 0.60471, 0.60378, 0.60285, 0.60201, 0.60170, 0.60242, 0.60332, 0.60423, 0.60507, 0.60568,\
        0.60602, 0.60580, 0.60510, 0.60432, 0.60329, 0.60136, 0.59732, 0.59182, 0.58503, 0.57714, 0.56849,\
        0.55958, 0.55100, 0.54331, 0.53689, 0.53182, 0.52788, 0.52473, 0.52202, 0.51950, 0.51697, 0.51423,\
        0.51115, 0.50765, 0.50377, 0.49952, 0.49491, 0.48990, 0.48451, 0.47879, 0.47265, 0.46591, 0.45849,\
        0.45048, 0.44213, 0.43386, 0.42615, 0.41939, 0.41373, 0.40904, 0.40503, 0.40152, 0.39849, 0.39581,\
        0.39336, 0.39111, 0.38915, 0.38753, 0.38631, 0.38549, 0.38508, 0.38506, 0.38541, 0.38610, 0.38708,\
        0.38833, 0.38979, 0.39142, 0.39318, 0.39501, 0.39688, 0.39872, 0.40041, 0.40188, 0.40323, 0.40449],\
        [0.51540, 0.50863, 0.50278, 0.49836, 0.49561, 0.49455, 0.49488, 0.49599, 0.49728, 0.49846, 0.49960,\
        0.50073, 0.50200, 0.50365, 0.50551, 0.50748, 0.50965, 0.51212, 0.51490, 0.51808, 0.52180, 0.52620,\
        0.53133, 0.53703, 0.54302, 0.54903, 0.55489, 0.56053, 0.56595, 0.57115, 0.57610, 0.58071, 0.58492,\
        0.58864, 0.59183, 0.59446, 0.59647, 0.59777, 0.59851, 0.59888, 0.59910, 0.59914, 0.59892, 0.59837,\
        0.59784, 0.59725, 0.59656, 0.59586, 0.59525, 0.59512, 0.59572, 0.59654, 0.59734, 0.59809, 0.59863,\
        0.59906, 0.59879, 0.59806, 0.59729, 0.59626, 0.59413, 0.59012, 0.58435, 0.57710, 0.56860, 0.55922,\
        0.54950, 0.54014, 0.53191, 0.52540, 0.52075, 0.51758, 0.51529, 0.51339, 0.51158, 0.50968, 0.50752,\
        0.50494, 0.50190, 0.49849, 0.49474, 0.49056, 0.48589, 0.48071, 0.47498, 0.46861, 0.46148, 0.45357,\
        0.44499, 0.43606, 0.42727, 0.41927, 0.41264, 0.40759, 0.40380, 0.40071, 0.39803, 0.39576, 0.39377,\
        0.39198, 0.39034, 0.38890, 0.38770, 0.38679, 0.38619, 0.38592, 0.38598, 0.38635, 0.38702, 0.38794,\
        0.38909, 0.39044, 0.39194, 0.39355, 0.39522, 0.39692, 0.39856, 0.40007, 0.40141, 0.40261, 0.40370],\
        [0.51127, 0.50558, 0.50078, 0.49722, 0.49500, 0.49403, 0.49407, 0.49471, 0.49555, 0.49637, 0.49712,\
        0.49780, 0.49858, 0.49969, 0.50093, 0.50218, 0.50363, 0.50538, 0.50739, 0.50970, 0.51256, 0.51632,\
        0.52114, 0.52683, 0.53298, 0.53925, 0.54542, 0.55138, 0.55710, 0.56255, 0.56768, 0.57244, 0.57677,\
        0.58061, 0.58391, 0.58663, 0.58874, 0.59022, 0.59117, 0.59176, 0.59214, 0.59236, 0.59236, 0.59216,\
        0.59190, 0.59160, 0.59125, 0.59091, 0.59070, 0.59083, 0.59135, 0.59208, 0.59284, 0.59352, 0.59405,\
        0.59434, 0.59419, 0.59362, 0.59278, 0.59141, 0.58892, 0.58478, 0.57885, 0.57125, 0.56220, 0.55206,\
        0.54140, 0.53104, 0.52202, 0.51536, 0.51132, 0.50912, 0.50776, 0.50663, 0.50545, 0.50414, 0.50256,\
        0.50049, 0.49794, 0.49502, 0.49170, 0.48792, 0.48359, 0.47864, 0.47298, 0.46651, 0.45911, 0.45077,\
        0.44160, 0.43193, 0.42235, 0.41370, 0.40693, 0.40243, 0.39962, 0.39746, 0.39565, 0.39408, 0.39270,\
        0.39145, 0.39030, 0.38926, 0.38839, 0.38771, 0.38727, 0.38710, 0.38719, 0.38755, 0.38816, 0.38899,\
        0.39003, 0.39123, 0.39257, 0.39400, 0.39549, 0.39698, 0.39841, 0.39974, 0.40093, 0.40199, 0.40293],\
        [0.50791, 0.50321, 0.49936, 0.49652, 0.49475, 0.49392, 0.49384, 0.49422, 0.49479, 0.49537, 0.49589,\
        0.49637, 0.49691, 0.49757, 0.49828, 0.49897, 0.49979, 0.50087, 0.50212, 0.50348, 0.50528, 0.50817,\
        0.51264, 0.51829, 0.52450, 0.53087, 0.53718, 0.54330, 0.54917, 0.55475, 0.56000, 0.56486, 0.56928,\
        0.57322, 0.57663, 0.57949, 0.58177, 0.58350, 0.58476, 0.58567, 0.58635, 0.58684, 0.58717, 0.58736,\
        0.58748, 0.58757, 0.58764, 0.58773, 0.58793, 0.58833, 0.58893, 0.58965, 0.59038, 0.59102, 0.59152,\
        0.59178, 0.59171, 0.59125, 0.59034, 0.58874, 0.58601, 0.58174, 0.57572, 0.56791, 0.55843, 0.54761,\
        0.53600, 0.52447, 0.51431, 0.50727, 0.50415, 0.50311, 0.50260, 0.50208, 0.50134, 0.50046, 0.49934,\
        0.49776, 0.49569, 0.49323, 0.49034, 0.48695, 0.48297, 0.47832, 0.47287, 0.46648, 0.45900, 0.45037,\
        0.44068, 0.43027, 0.41973, 0.41001, 0.40265, 0.39862, 0.39680, 0.39567, 0.39463, 0.39365, 0.39273,\
        0.39187, 0.39106, 0.39030, 0.38963, 0.38910, 0.38875, 0.38861, 0.38870, 0.38900, 0.38952, 0.39024,\
        0.39113, 0.39216, 0.39331, 0.39455, 0.39583, 0.39710, 0.39833, 0.39947, 0.40049, 0.40139, 0.40218],\
        [0.50517, 0.50139, 0.49838, 0.49621, 0.49487, 0.49425, 0.49417, 0.49444, 0.49487, 0.49533, 0.49576,\
        0.49614, 0.49654, 0.49697, 0.49741, 0.49786, 0.49836, 0.49895, 0.49955, 0.50006, 0.50071, 0.50233,\
        0.50639, 0.51185, 0.51788, 0.52410, 0.53032, 0.53641, 0.54229, 0.54789, 0.55316, 0.55806, 0.56255,\
        0.56657, 0.57011, 0.57313, 0.57563, 0.57765, 0.57926, 0.58053, 0.58158, 0.58244, 0.58317, 0.58379,\
        0.58436, 0.58491, 0.58545, 0.58602, 0.58664, 0.58734, 0.58811, 0.58889, 0.58962, 0.59025, 0.59074,\
        0.59101, 0.59099, 0.59062, 0.58973, 0.58807, 0.58531, 0.58109, 0.57515, 0.56734, 0.55767, 0.54640,\
        0.53407, 0.52151, 0.51000, 0.50189, 0.50028, 0.50018, 0.50009, 0.49992, 0.49947, 0.49881, 0.49789,\
        0.49664, 0.49501, 0.49300, 0.49055, 0.48756, 0.48397, 0.47971, 0.47464, 0.46855, 0.46123, 0.45255,\
        0.44257, 0.43162, 0.42029, 0.40948, 0.40076, 0.39740, 0.39658, 0.39602, 0.39533, 0.39461, 0.39392,\
        0.39328, 0.39266, 0.39204, 0.39146, 0.39097, 0.39063, 0.39046, 0.39048, 0.39069, 0.39109, 0.39166,\
        0.39238, 0.39322, 0.39417, 0.39519, 0.39625, 0.39731, 0.39832, 0.39926, 0.40010, 0.40084, 0.40147],\
        [0.50289, 0.50000, 0.49777, 0.49624, 0.49534, 0.49497, 0.49498, 0.49524, 0.49563, 0.49605, 0.49644,\
        0.49679, 0.49713, 0.49747, 0.49783, 0.49821, 0.49862, 0.49904, 0.49941, 0.49967, 0.49984, 0.50016,\
        0.50315, 0.50781, 0.51323, 0.51901, 0.52491, 0.53078, 0.53651, 0.54201, 0.54724, 0.55213, 0.55664,\
        0.56074, 0.56439, 0.56758, 0.57032, 0.57264, 0.57460, 0.57627, 0.57772, 0.57900, 0.58017, 0.58127,\
        0.58233, 0.58338, 0.58443, 0.58549, 0.58656, 0.58760, 0.58858, 0.58947, 0.59023, 0.59087, 0.59136,\
        0.59166, 0.59172, 0.59147, 0.59072, 0.58919, 0.58661, 0.58268, 0.57711, 0.56961, 0.56006, 0.54870,\
        0.53608, 0.52314, 0.51122, 0.50270, 0.50067, 0.50041, 0.50016, 0.49997, 0.49975, 0.49908, 0.49818,\
        0.49711, 0.49583, 0.49424, 0.49219, 0.48959, 0.48641, 0.48264, 0.47813, 0.47261, 0.46576, 0.45733,\
        0.44735, 0.43621, 0.42463, 0.41369, 0.40513, 0.40098, 0.39977, 0.39878, 0.39783, 0.39700, 0.39627,\
        0.39567, 0.39510, 0.39446, 0.39383, 0.39328, 0.39287, 0.39262, 0.39253, 0.39261, 0.39286, 0.39325,\
        0.39378, 0.39441, 0.39514, 0.39594, 0.39677, 0.39760, 0.39840, 0.39913, 0.39979, 0.40035, 0.40080],\
        [0.50104, 0.49887, 0.49746, 0.49657, 0.49612, 0.49600, 0.49614, 0.49645, 0.49684, 0.49724, 0.49762,\
        0.49797, 0.49829, 0.49860, 0.49891, 0.49927, 0.49964, 0.49995, 0.50016, 0.50024, 0.50028, 0.50058,\
        0.50234, 0.50581, 0.51033, 0.51545, 0.52086, 0.52636, 0.53182, 0.53714, 0.54224, 0.54707, 0.55157,\
        0.55572, 0.55947, 0.56284, 0.56582, 0.56844, 0.57075, 0.57280, 0.57467, 0.57639, 0.57803, 0.57961,\
        0.58117, 0.58274, 0.58432, 0.58589, 0.58742, 0.58885, 0.59009, 0.59109, 0.59190, 0.59256, 0.59308,\
        0.59342, 0.59356, 0.59349, 0.59302, 0.59179, 0.58956, 0.58620, 0.58135, 0.57452, 0.56543, 0.55429,\
        0.54181, 0.52912, 0.51771, 0.50948, 0.50538, 0.50369, 0.50279, 0.50222, 0.50172, 0.50097, 0.50005,\
        0.49907, 0.49804, 0.49679, 0.49513, 0.49282, 0.49004, 0.48685, 0.48307, 0.47840, 0.47237, 0.46448,\
        0.45472, 0.44365, 0.43223, 0.42164, 0.41328, 0.40806, 0.40528, 0.40340, 0.40190, 0.40069, 0.39967,\
        0.39895, 0.39829, 0.39746, 0.39666, 0.39599, 0.39545, 0.39505, 0.39482, 0.39474, 0.39481, 0.39500,\
        0.39531, 0.39572, 0.39621, 0.39678, 0.39737, 0.39798, 0.39855, 0.39908, 0.39954, 0.39991, 0.40020],\
        [0.49990, 0.49838, 0.49757, 0.49720, 0.49713, 0.49724, 0.49749, 0.49784, 0.49823, 0.49863, 0.49899,\
        0.49932, 0.49961, 0.49984, 0.50008, 0.50032, 0.50053, 0.50064, 0.50065, 0.50058, 0.50057, 0.50096,\
        0.50235, 0.50501, 0.50871, 0.51312, 0.51796, 0.52302, 0.52814, 0.53321, 0.53814, 0.54286, 0.54733,\
        0.55150, 0.55535, 0.55887, 0.56207, 0.56497, 0.56761, 0.57004, 0.57231, 0.57447, 0.57657, 0.57863,\
        0.58069, 0.58276, 0.58486, 0.58695, 0.58897, 0.59082, 0.59236, 0.59348, 0.59429, 0.59499, 0.59561,\
        0.59600, 0.59619, 0.59637, 0.59634, 0.59550, 0.59368, 0.59115, 0.58740, 0.58161, 0.57323, 0.56246,\
        0.55027, 0.53796, 0.52695, 0.51846, 0.51294, 0.50968, 0.50771, 0.50640, 0.50538, 0.50440, 0.50340,\
        0.50243, 0.50149, 0.50045, 0.49912, 0.49694, 0.49458, 0.49200, 0.48905, 0.48549, 0.48064, 0.47350,\
        0.46399, 0.45303, 0.44179, 0.43140, 0.42286, 0.41666, 0.41246, 0.40949, 0.40723, 0.40545, 0.40402,\
        0.40293, 0.40197, 0.40089, 0.39989, 0.39905, 0.39832, 0.39773, 0.39731, 0.39705, 0.39692, 0.39689,\
        0.39696, 0.39712, 0.39738, 0.39770, 0.39807, 0.39844, 0.39879, 0.39911, 0.39936, 0.39954, 0.39964],\
        [0.49945, 0.49841, 0.49801, 0.49804, 0.49827, 0.49853, 0.49883, 0.49918, 0.49955, 0.49992, 0.50021,\
        0.50046, 0.50067, 0.50081, 0.50093, 0.50103, 0.50109, 0.50106, 0.50096, 0.50086, 0.50091, 0.50137,\
        0.50261, 0.50480, 0.50790, 0.51170, 0.51600, 0.52060, 0.52535, 0.53014, 0.53487, 0.53946, 0.54386,\
        0.54803, 0.55196, 0.55561, 0.55901, 0.56217, 0.56512, 0.56790, 0.57056, 0.57313, 0.57565, 0.57816,\
        0.58069, 0.58324, 0.58582, 0.58840, 0.59091, 0.59323, 0.59513, 0.59634, 0.59710, 0.59782, 0.59865,\
        0.59914, 0.59935, 0.59982, 0.60025, 0.59983, 0.59842, 0.59698, 0.59458, 0.59014, 0.58259, 0.57208,\
        0.56010, 0.54811, 0.53727, 0.52845, 0.52193, 0.51742, 0.51434, 0.51216, 0.51052, 0.50917, 0.50799,\
        0.50693, 0.50596, 0.50494, 0.50361, 0.50174, 0.49986, 0.49781, 0.49557, 0.49316, 0.49001, 0.48362,\
        0.47412, 0.46317, 0.45204, 0.44170, 0.43287, 0.42588, 0.42059, 0.41659, 0.41349, 0.41103, 0.40905,\
        0.40744, 0.40602, 0.40468, 0.40344, 0.40236, 0.40142, 0.40061, 0.39997, 0.39951, 0.39915, 0.39889,\
        0.39870, 0.39862, 0.39863, 0.39871, 0.39884, 0.39898, 0.39910, 0.39920, 0.39924, 0.39923, 0.39913],\
        [0.49926, 0.49862, 0.49854, 0.49888, 0.49934, 0.49966, 0.49993, 0.50024, 0.50054, 0.50083, 0.50106,\
        0.50123, 0.50136, 0.50144, 0.50148, 0.50148, 0.50143, 0.50133, 0.50122, 0.50115, 0.50129, 0.50183,\
        0.50299, 0.50490, 0.50758, 0.51090, 0.51474, 0.51892, 0.52332, 0.52782, 0.53233, 0.53678, 0.54110,\
        0.54526, 0.54923, 0.55301, 0.55658, 0.55997, 0.56320, 0.56629, 0.56929, 0.57224, 0.57515, 0.57807,\
        0.58101, 0.58398, 0.58698, 0.58998, 0.59292, 0.59569, 0.59805, 0.59937, 0.60008, 0.60085, 0.60181,\
        0.60253, 0.60305, 0.60373, 0.60433, 0.60435, 0.60380, 0.60322, 0.60188, 0.59895, 0.59208, 0.58163,\
        0.56993, 0.55830, 0.54765, 0.53861, 0.53144, 0.52602, 0.52201, 0.51902, 0.51675, 0.51495, 0.51348,\
        0.51224, 0.51113, 0.51002, 0.50874, 0.50725, 0.50569, 0.50407, 0.50234, 0.50067, 0.49945, 0.49328,\
        0.48367, 0.47287, 0.46195, 0.45171, 0.44269, 0.43514, 0.42905, 0.42419, 0.42029, 0.41713, 0.41454,\
        0.41237, 0.41048, 0.40879, 0.40726, 0.40590, 0.40470, 0.40365, 0.40278, 0.40207, 0.40147, 0.40096,\
        0.40052, 0.40019, 0.39996, 0.39980, 0.39968, 0.39958, 0.39948, 0.39935, 0.39919, 0.39896, 0.39867],\
        [0.49900, 0.49869, 0.49887, 0.49939, 0.50002, 0.50036, 0.50060, 0.50086, 0.50112, 0.50135, 0.50153,\
        0.50167, 0.50175, 0.50178, 0.50177, 0.50172, 0.50164, 0.50153, 0.50145, 0.50146, 0.50170, 0.50231,\
        0.50344, 0.50518, 0.50757, 0.51054, 0.51400, 0.51782, 0.52190, 0.52613, 0.53043, 0.53473, 0.53897,\
        0.54310, 0.54711, 0.55098, 0.55470, 0.55828, 0.56175, 0.56512, 0.56843, 0.57170, 0.57495, 0.57822,\
        0.58151, 0.58483, 0.58817, 0.59149, 0.59473, 0.59777, 0.60037, 0.60203, 0.60302, 0.60398, 0.60501,\
        0.60597, 0.60684, 0.60772, 0.60850, 0.60896, 0.60909, 0.60892, 0.60804, 0.60548, 0.59922, 0.58958,\
        0.57865, 0.56765, 0.55738, 0.54837, 0.54087, 0.53485, 0.53015, 0.52650, 0.52366, 0.52141, 0.51959,\
        0.51809, 0.51679, 0.51558, 0.51434, 0.51305, 0.51174, 0.51042, 0.50910, 0.50785, 0.50584, 0.50022,\
        0.49143, 0.48128, 0.47087, 0.46092, 0.45188, 0.44403, 0.43739, 0.43187, 0.42730, 0.42350, 0.42031,\
        0.41760, 0.41523, 0.41314, 0.41128, 0.40961, 0.40813, 0.40682, 0.40568, 0.40470, 0.40384, 0.40308,\
        0.40241, 0.40183, 0.40135, 0.40094, 0.40058, 0.40024, 0.39991, 0.39956, 0.39917, 0.39874, 0.39825],\
        [0.49855, 0.49850, 0.49882, 0.49939, 0.50001, 0.50046, 0.50078, 0.50106, 0.50131, 0.50152, 0.50169,\
        0.50181, 0.50188, 0.50190, 0.50187, 0.50182, 0.50174, 0.50168, 0.50166, 0.50178, 0.50211, 0.50279,\
        0.50392, 0.50557, 0.50777, 0.51048, 0.51364, 0.51717, 0.52097, 0.52497, 0.52907, 0.53323, 0.53737,\
        0.54148, 0.54550, 0.54944, 0.55328, 0.55703, 0.56070, 0.56431, 0.56788, 0.57142, 0.57496, 0.57852,\
        0.58209, 0.58569, 0.58927, 0.59281, 0.59623, 0.59938, 0.60207, 0.60409, 0.60557, 0.60685, 0.60809,\
        0.60927, 0.61040, 0.61148, 0.61243, 0.61315, 0.61354, 0.61342, 0.61235, 0.60947, 0.60380, 0.59553,\
        0.58581, 0.57575, 0.56610, 0.55736, 0.54980, 0.54349, 0.53833, 0.53419, 0.53087, 0.52820, 0.52604,\
        0.52425, 0.52274, 0.52138, 0.52011, 0.51887, 0.51764, 0.51640, 0.51506, 0.51337, 0.51046, 0.50513,\
        0.49739, 0.48823, 0.47856, 0.46908, 0.46023, 0.45229, 0.44534, 0.43936, 0.43426, 0.42991, 0.42618,\
        0.42296, 0.42015, 0.41765, 0.41543, 0.41344, 0.41166, 0.41007, 0.40865, 0.40739, 0.40627, 0.40525,\
        0.40434, 0.40352, 0.40279, 0.40213, 0.40152, 0.40094, 0.40037, 0.39980, 0.39920, 0.39856, 0.39786]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=25 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax25vt2.5
    zgrid_H_Hmax_vmax25vt2_5=[[0.32886, 0.33481, 0.34057, 0.34616, 0.35160, 0.35692, 0.36217, 0.36736, 0.37255, 0.37778, 0.38310,\
        0.38856, 0.39419, 0.39978, 0.40471, 0.40942, 0.41404, 0.41858, 0.42305, 0.42744, 0.43175, 0.43599,\
        0.44015, 0.44423, 0.44823, 0.45217, 0.45605, 0.45987, 0.46365, 0.46739, 0.47111, 0.47482, 0.47852,\
        0.48225, 0.48603, 0.48987, 0.49375, 0.49760, 0.50086, 0.50358, 0.50647, 0.50957, 0.51288, 0.51635,\
        0.51989, 0.52345, 0.52694, 0.53033, 0.53356, 0.53659, 0.53940, 0.54196, 0.54427, 0.54631, 0.54809,\
        0.54962, 0.55091, 0.55198, 0.55285, 0.55354, 0.55406, 0.55442, 0.55463, 0.55468, 0.55456, 0.55427,\
        0.55380, 0.55314, 0.55226, 0.55116, 0.54982, 0.54823, 0.54637, 0.54427, 0.54193, 0.53936, 0.53660,\
        0.53367, 0.53059, 0.52740, 0.52411, 0.52076, 0.51737, 0.51397, 0.51058, 0.50724, 0.50399, 0.50089,\
        0.49812, 0.49617, 0.49412, 0.49066, 0.48702, 0.48343, 0.47976, 0.47594, 0.47193, 0.46774, 0.46334,\
        0.45867, 0.45368, 0.44836, 0.44273, 0.43682, 0.43067, 0.42428, 0.41767, 0.41083, 0.40377, 0.39647,\
        0.38891, 0.38109, 0.37301, 0.36464, 0.35598, 0.34704, 0.33780, 0.32828, 0.31845, 0.30834, 0.29794],\
        [0.33511, 0.34083, 0.34635, 0.35167, 0.35683, 0.36186, 0.36678, 0.37165, 0.37649, 0.38137, 0.38631,\
        0.39136, 0.39656, 0.40197, 0.40718, 0.41211, 0.41688, 0.42153, 0.42607, 0.43050, 0.43484, 0.43908,\
        0.44322, 0.44726, 0.45121, 0.45508, 0.45887, 0.46258, 0.46624, 0.46984, 0.47340, 0.47692, 0.48041,\
        0.48390, 0.48739, 0.49092, 0.49450, 0.49796, 0.50113, 0.50441, 0.50779, 0.51141, 0.51525, 0.51924,\
        0.52328, 0.52729, 0.53121, 0.53498, 0.53855, 0.54187, 0.54494, 0.54772, 0.55022, 0.55242, 0.55433,\
        0.55596, 0.55734, 0.55849, 0.55943, 0.56018, 0.56077, 0.56121, 0.56150, 0.56165, 0.56163, 0.56144,\
        0.56106, 0.56049, 0.55970, 0.55869, 0.55741, 0.55586, 0.55404, 0.55193, 0.54957, 0.54698, 0.54418,\
        0.54121, 0.53808, 0.53484, 0.53152, 0.52813, 0.52472, 0.52129, 0.51789, 0.51454, 0.51128, 0.50820,\
        0.50545, 0.50353, 0.50132, 0.49749, 0.49405, 0.49064, 0.48711, 0.48339, 0.47946, 0.47537, 0.47111,\
        0.46656, 0.46164, 0.45634, 0.45072, 0.44482, 0.43869, 0.43236, 0.42582, 0.41907, 0.41210, 0.40490,\
        0.39744, 0.38972, 0.38173, 0.37344, 0.36487, 0.35600, 0.34684, 0.33737, 0.32762, 0.31756, 0.30722],\
        [0.34120, 0.34670, 0.35197, 0.35703, 0.36190, 0.36662, 0.37122, 0.37574, 0.38023, 0.38473, 0.38930,\
        0.39398, 0.39871, 0.40392, 0.40932, 0.41449, 0.41944, 0.42422, 0.42886, 0.43336, 0.43773, 0.44197,\
        0.44610, 0.45010, 0.45400, 0.45779, 0.46149, 0.46510, 0.46863, 0.47209, 0.47547, 0.47879, 0.48204,\
        0.48522, 0.48835, 0.49146, 0.49461, 0.49786, 0.50119, 0.50472, 0.50865, 0.51292, 0.51740, 0.52199,\
        0.52658, 0.53109, 0.53545, 0.53961, 0.54352, 0.54715, 0.55047, 0.55347, 0.55615, 0.55850, 0.56053,\
        0.56226, 0.56371, 0.56492, 0.56592, 0.56673, 0.56739, 0.56790, 0.56828, 0.56852, 0.56861, 0.56851,\
        0.56823, 0.56774, 0.56706, 0.56614, 0.56495, 0.56345, 0.56164, 0.55953, 0.55715, 0.55452, 0.55168,\
        0.54865, 0.54548, 0.54219, 0.53883, 0.53542, 0.53198, 0.52856, 0.52516, 0.52184, 0.51862, 0.51559,\
        0.51287, 0.51046, 0.50767, 0.50425, 0.50097, 0.49765, 0.49425, 0.49060, 0.48665, 0.48268, 0.47862,\
        0.47421, 0.46933, 0.46401, 0.45835, 0.45245, 0.44636, 0.44008, 0.43363, 0.42699, 0.42013, 0.41303,\
        0.40569, 0.39807, 0.39017, 0.38198, 0.37349, 0.36470, 0.35560, 0.34620, 0.33650, 0.32651, 0.31622],\
        [0.34716, 0.35243, 0.35746, 0.36226, 0.36685, 0.37126, 0.37553, 0.37972, 0.38386, 0.38803, 0.39227,\
        0.39669, 0.40145, 0.40642, 0.41167, 0.41688, 0.42194, 0.42682, 0.43153, 0.43609, 0.44049, 0.44474,\
        0.44885, 0.45282, 0.45666, 0.46038, 0.46398, 0.46748, 0.47089, 0.47420, 0.47743, 0.48055, 0.48357,\
        0.48647, 0.48922, 0.49187, 0.49451, 0.49740, 0.50079, 0.50479, 0.50939, 0.51439, 0.51957, 0.52479,\
        0.52995, 0.53496, 0.53976, 0.54431, 0.54855, 0.55246, 0.55603, 0.55923, 0.56208, 0.56458, 0.56672,\
        0.56853, 0.57004, 0.57129, 0.57234, 0.57321, 0.57392, 0.57450, 0.57497, 0.57532, 0.57551, 0.57551,\
        0.57531, 0.57492, 0.57434, 0.57354, 0.57243, 0.57098, 0.56918, 0.56706, 0.56466, 0.56199, 0.55910,\
        0.55601, 0.55279, 0.54945, 0.54605, 0.54262, 0.53918, 0.53576, 0.53238, 0.52909, 0.52591, 0.52289,\
        0.52006, 0.51730, 0.51434, 0.51113, 0.50786, 0.50455, 0.50121, 0.49751, 0.49340, 0.48966, 0.48592,\
        0.48165, 0.47673, 0.47131, 0.46559, 0.45968, 0.45363, 0.44745, 0.44110, 0.43458, 0.42785, 0.42088,\
        0.41366, 0.40616, 0.39836, 0.39026, 0.38184, 0.37312, 0.36409, 0.35475, 0.34511, 0.33518, 0.32495],\
        [0.35298, 0.35805, 0.36285, 0.36740, 0.37171, 0.37583, 0.37979, 0.38365, 0.38747, 0.39137, 0.39537,\
        0.39949, 0.40416, 0.40914, 0.41428, 0.41944, 0.42450, 0.42942, 0.43418, 0.43877, 0.44319, 0.44744,\
        0.45153, 0.45547, 0.45925, 0.46288, 0.46640, 0.46979, 0.47308, 0.47627, 0.47935, 0.48233, 0.48517,\
        0.48785, 0.49033, 0.49256, 0.49467, 0.49705, 0.50041, 0.50503, 0.51043, 0.51618, 0.52204, 0.52787,\
        0.53355, 0.53903, 0.54424, 0.54914, 0.55369, 0.55787, 0.56165, 0.56504, 0.56804, 0.57067, 0.57292,\
        0.57479, 0.57634, 0.57762, 0.57871, 0.57963, 0.58039, 0.58102, 0.58158, 0.58205, 0.58238, 0.58247,\
        0.58232, 0.58200, 0.58154, 0.58088, 0.57988, 0.57848, 0.57667, 0.57453, 0.57209, 0.56938, 0.56643,\
        0.56329, 0.56000, 0.55661, 0.55318, 0.54973, 0.54628, 0.54287, 0.53952, 0.53626, 0.53310, 0.53005,\
        0.52711, 0.52417, 0.52113, 0.51795, 0.51467, 0.51131, 0.50781, 0.50391, 0.49976, 0.49648, 0.49311,\
        0.48886, 0.48375, 0.47815, 0.47235, 0.46646, 0.46050, 0.45444, 0.44824, 0.44186, 0.43528, 0.42845,\
        0.42136, 0.41398, 0.40628, 0.39827, 0.38994, 0.38128, 0.37231, 0.36303, 0.35345, 0.34357, 0.33339],\
        [0.35870, 0.36358, 0.36817, 0.37247, 0.37652, 0.38035, 0.38402, 0.38758, 0.39108, 0.39473, 0.39867,\
        0.40277, 0.40724, 0.41208, 0.41711, 0.42218, 0.42721, 0.43212, 0.43688, 0.44148, 0.44590, 0.45014,\
        0.45421, 0.45809, 0.46181, 0.46537, 0.46878, 0.47207, 0.47526, 0.47834, 0.48133, 0.48420, 0.48695,\
        0.48956, 0.49194, 0.49401, 0.49574, 0.49758, 0.50098, 0.50625, 0.51232, 0.51868, 0.52509, 0.53141,\
        0.53754, 0.54341, 0.54896, 0.55417, 0.55899, 0.56339, 0.56736, 0.57090, 0.57403, 0.57678, 0.57913,\
        0.58106, 0.58261, 0.58390, 0.58504, 0.58603, 0.58682, 0.58748, 0.58811, 0.58874, 0.58923, 0.58941,\
        0.58925, 0.58897, 0.58866, 0.58818, 0.58732, 0.58592, 0.58408, 0.58191, 0.57944, 0.57669, 0.57368,\
        0.57046, 0.56711, 0.56367, 0.56020, 0.55673, 0.55329, 0.54989, 0.54656, 0.54331, 0.54016, 0.53709,\
        0.53406, 0.53103, 0.52792, 0.52470, 0.52137, 0.51790, 0.51426, 0.51039, 0.50660, 0.50332, 0.50005,\
        0.49559, 0.49017, 0.48436, 0.47855, 0.47278, 0.46697, 0.46107, 0.45504, 0.44884, 0.44243, 0.43576,\
        0.42881, 0.42155, 0.41396, 0.40603, 0.39777, 0.38918, 0.38027, 0.37104, 0.36151, 0.35167, 0.34155],\
        [0.36431, 0.36902, 0.37342, 0.37750, 0.38128, 0.38483, 0.38824, 0.39159, 0.39473, 0.39795, 0.40188,\
        0.40605, 0.41050, 0.41523, 0.42014, 0.42512, 0.43007, 0.43494, 0.43968, 0.44426, 0.44867, 0.45289,\
        0.45691, 0.46075, 0.46440, 0.46787, 0.47119, 0.47438, 0.47747, 0.48047, 0.48340, 0.48623, 0.48897,\
        0.49163, 0.49418, 0.49644, 0.49834, 0.49986, 0.50357, 0.50909, 0.51544, 0.52214, 0.52890, 0.53556,\
        0.54200, 0.54816, 0.55398, 0.55943, 0.56446, 0.56905, 0.57316, 0.57679, 0.58002, 0.58289, 0.58537,\
        0.58737, 0.58889, 0.59012, 0.59132, 0.59246, 0.59327, 0.59390, 0.59461, 0.59539, 0.59608, 0.59639,\
        0.59610, 0.59585, 0.59569, 0.59543, 0.59472, 0.59325, 0.59135, 0.58918, 0.58671, 0.58393, 0.58085,\
        0.57754, 0.57411, 0.57062, 0.56712, 0.56364, 0.56019, 0.55680, 0.55348, 0.55025, 0.54709, 0.54400,\
        0.54093, 0.53784, 0.53466, 0.53138, 0.52796, 0.52441, 0.52071, 0.51691, 0.51320, 0.50965, 0.50591,\
        0.50133, 0.49565, 0.48982, 0.48422, 0.47868, 0.47309, 0.46739, 0.46155, 0.45554, 0.44932, 0.44283,\
        0.43602, 0.42888, 0.42139, 0.41354, 0.40535, 0.39681, 0.38795, 0.37876, 0.36928, 0.35949, 0.34942],\
        [0.36981, 0.37439, 0.37863, 0.38251, 0.38602, 0.38925, 0.39237, 0.39563, 0.39876, 0.40137, 0.40518,\
        0.40943, 0.41390, 0.41856, 0.42337, 0.42824, 0.43311, 0.43791, 0.44260, 0.44715, 0.45153, 0.45571,\
        0.45970, 0.46348, 0.46705, 0.47043, 0.47364, 0.47673, 0.47973, 0.48268, 0.48559, 0.48842, 0.49119,\
        0.49397, 0.49685, 0.49955, 0.50192, 0.50432, 0.50810, 0.51350, 0.51984, 0.52662, 0.53353, 0.54036,\
        0.54699, 0.55332, 0.55930, 0.56491, 0.57011, 0.57485, 0.57905, 0.58271, 0.58595, 0.58894, 0.59161,\
        0.59371, 0.59518, 0.59629, 0.59756, 0.59896, 0.59979, 0.60040, 0.60120, 0.60208, 0.60286, 0.60324,\
        0.60301, 0.60283, 0.60274, 0.60255, 0.60192, 0.60036, 0.59848, 0.59635, 0.59390, 0.59108, 0.58792,\
        0.58451, 0.58099, 0.57746, 0.57394, 0.57044, 0.56699, 0.56360, 0.56028, 0.55706, 0.55391, 0.55081,\
        0.54772, 0.54457, 0.54134, 0.53798, 0.53448, 0.53083, 0.52705, 0.52319, 0.51930, 0.51534, 0.51104,\
        0.50601, 0.50023, 0.49478, 0.48956, 0.48432, 0.47895, 0.47344, 0.46780, 0.46200, 0.45598, 0.44967,\
        0.44302, 0.43600, 0.42859, 0.42081, 0.41266, 0.40417, 0.39534, 0.38620, 0.37676, 0.36702, 0.35699],\
        [0.37517, 0.37965, 0.38379, 0.38749, 0.39077, 0.39367, 0.39623, 0.39935, 0.40257, 0.40555, 0.40906,\
        0.41314, 0.41751, 0.42207, 0.42677, 0.43153, 0.43630, 0.44103, 0.44566, 0.45016, 0.45450, 0.45865,\
        0.46259, 0.46632, 0.46981, 0.47309, 0.47618, 0.47914, 0.48205, 0.48497, 0.48791, 0.49081, 0.49359,\
        0.49641, 0.49959, 0.50281, 0.50588, 0.50924, 0.51353, 0.51895, 0.52522, 0.53197, 0.53890, 0.54578,\
        0.55246, 0.55886, 0.56490, 0.57058, 0.57587, 0.58075, 0.58505, 0.58863, 0.59175, 0.59478, 0.59773,\
        0.59999, 0.60150, 0.60263, 0.60393, 0.60534, 0.60637, 0.60716, 0.60800, 0.60887, 0.60961, 0.61001,\
        0.61004, 0.60996, 0.60984, 0.60953, 0.60876, 0.60736, 0.60554, 0.60343, 0.60098, 0.59814, 0.59489,\
        0.59136, 0.58776, 0.58420, 0.58067, 0.57716, 0.57369, 0.57029, 0.56697, 0.56375, 0.56061, 0.55752,\
        0.55442, 0.55124, 0.54795, 0.54450, 0.54089, 0.53713, 0.53322, 0.52919, 0.52504, 0.52070, 0.51600,\
        0.51078, 0.50524, 0.49994, 0.49488, 0.48987, 0.48462, 0.47924, 0.47380, 0.46824, 0.46245, 0.45633,\
        0.44982, 0.44290, 0.43556, 0.42782, 0.41971, 0.41125, 0.40245, 0.39334, 0.38394, 0.37424, 0.36426],\
        [0.38036, 0.38476, 0.38887, 0.39245, 0.39549, 0.39826, 0.40038, 0.40285, 0.40626, 0.40960, 0.41317,\
        0.41710, 0.42133, 0.42577, 0.43034, 0.43498, 0.43965, 0.44428, 0.44885, 0.45329, 0.45759, 0.46171,\
        0.46562, 0.46929, 0.47271, 0.47589, 0.47884, 0.48164, 0.48441, 0.48726, 0.49030, 0.49343, 0.49634,\
        0.49895, 0.50248, 0.50630, 0.51021, 0.51443, 0.51930, 0.52495, 0.53125, 0.53797, 0.54486, 0.55171,\
        0.55837, 0.56473, 0.57072, 0.57635, 0.58164, 0.58662, 0.59114, 0.59457, 0.59745, 0.60042, 0.60348,\
        0.60593, 0.60770, 0.60912, 0.61050, 0.61185, 0.61301, 0.61399, 0.61490, 0.61576, 0.61646, 0.61691,\
        0.61709, 0.61707, 0.61690, 0.61647, 0.61562, 0.61429, 0.61253, 0.61040, 0.60791, 0.60502, 0.60172,\
        0.59810, 0.59445, 0.59090, 0.58738, 0.58383, 0.58032, 0.57688, 0.57354, 0.57032, 0.56721, 0.56414,\
        0.56104, 0.55784, 0.55448, 0.55094, 0.54720, 0.54329, 0.53921, 0.53498, 0.53058, 0.52597, 0.52107,\
        0.51586, 0.51053, 0.50531, 0.50035, 0.49542, 0.49009, 0.48480, 0.47960, 0.47431, 0.46876, 0.46282,\
        0.45644, 0.44959, 0.44229, 0.43458, 0.42648, 0.41803, 0.40926, 0.40018, 0.39081, 0.38116, 0.37123],\
        [0.38531, 0.38963, 0.39377, 0.39738, 0.40012, 0.40269, 0.40497, 0.40740, 0.41046, 0.41380, 0.41737,\
        0.42122, 0.42531, 0.42960, 0.43404, 0.43856, 0.44312, 0.44766, 0.45215, 0.45654, 0.46080, 0.46489,\
        0.46877, 0.47242, 0.47579, 0.47886, 0.48166, 0.48426, 0.48680, 0.48950, 0.49257, 0.49606, 0.49953,\
        0.50241, 0.50602, 0.51032, 0.51493, 0.51988, 0.52530, 0.53125, 0.53768, 0.54440, 0.55125, 0.55804,\
        0.56461, 0.57086, 0.57671, 0.58217, 0.58727, 0.59217, 0.59701, 0.60055, 0.60325, 0.60626, 0.60922,\
        0.61179, 0.61386, 0.61556, 0.61709, 0.61850, 0.61976, 0.62086, 0.62184, 0.62270, 0.62340, 0.62389,\
        0.62414, 0.62416, 0.62394, 0.62343, 0.62254, 0.62121, 0.61946, 0.61731, 0.61477, 0.61183, 0.60849,\
        0.60487, 0.60119, 0.59768, 0.59412, 0.59046, 0.58689, 0.58339, 0.58000, 0.57678, 0.57370, 0.57068,\
        0.56759, 0.56437, 0.56095, 0.55729, 0.55340, 0.54931, 0.54504, 0.54060, 0.53600, 0.53121, 0.52621,\
        0.52105, 0.51582, 0.51069, 0.50572, 0.50062, 0.49517, 0.49010, 0.48523, 0.48026, 0.47496, 0.46919,\
        0.46289, 0.45607, 0.44877, 0.44105, 0.43294, 0.42450, 0.41574, 0.40669, 0.39735, 0.38775, 0.37788],\
        [0.39002, 0.39412, 0.39820, 0.40191, 0.40467, 0.40715, 0.40955, 0.41209, 0.41498, 0.41820, 0.42169,\
        0.42543, 0.42939, 0.43354, 0.43784, 0.44223, 0.44668, 0.45113, 0.45554, 0.45988, 0.46411, 0.46818,\
        0.47206, 0.47570, 0.47905, 0.48207, 0.48472, 0.48707, 0.48929, 0.49164, 0.49449, 0.49817, 0.50210,\
        0.50584, 0.51003, 0.51481, 0.52000, 0.52556, 0.53147, 0.53775, 0.54433, 0.55109, 0.55790, 0.56461,\
        0.57107, 0.57717, 0.58283, 0.58804, 0.59282, 0.59731, 0.60191, 0.60598, 0.60914, 0.61220, 0.61514,\
        0.61778, 0.62005, 0.62198, 0.62369, 0.62520, 0.62656, 0.62775, 0.62879, 0.62969, 0.63041, 0.63092,\
        0.63121, 0.63124, 0.63101, 0.63045, 0.62951, 0.62817, 0.62640, 0.62423, 0.62165, 0.61868, 0.61537,\
        0.61180, 0.60815, 0.60455, 0.60083, 0.59706, 0.59346, 0.58984, 0.58634, 0.58312, 0.58011, 0.57715,\
        0.57410, 0.57085, 0.56734, 0.56354, 0.55947, 0.55518, 0.55071, 0.54608, 0.54131, 0.53640, 0.53134,\
        0.52619, 0.52101, 0.51586, 0.51071, 0.50540, 0.50004, 0.49537, 0.49085, 0.48618, 0.48109, 0.47543,\
        0.46915, 0.46230, 0.45496, 0.44719, 0.43906, 0.43061, 0.42187, 0.41285, 0.40357, 0.39402, 0.38422],\
        [0.39463, 0.39833, 0.40227, 0.40588, 0.40890, 0.41155, 0.41408, 0.41671, 0.41957, 0.42269, 0.42608,\
        0.42970, 0.43353, 0.43754, 0.44170, 0.44597, 0.45030, 0.45465, 0.45899, 0.46328, 0.46748, 0.47154,\
        0.47544, 0.47911, 0.48250, 0.48553, 0.48809, 0.49021, 0.49207, 0.49399, 0.49630, 0.49962, 0.50424,\
        0.50908, 0.51415, 0.51957, 0.52535, 0.53143, 0.53777, 0.54435, 0.55108, 0.55789, 0.56465, 0.57128,\
        0.57763, 0.58355, 0.58898, 0.59399, 0.59850, 0.60284, 0.60715, 0.61127, 0.61486, 0.61812, 0.62114,\
        0.62389, 0.62631, 0.62843, 0.63029, 0.63194, 0.63339, 0.63466, 0.63577, 0.63670, 0.63745, 0.63800,\
        0.63831, 0.63836, 0.63811, 0.63752, 0.63655, 0.63519, 0.63340, 0.63121, 0.62861, 0.62564, 0.62234,\
        0.61881, 0.61514, 0.61141, 0.60758, 0.60377, 0.60009, 0.59620, 0.59257, 0.58937, 0.58648, 0.58359,\
        0.58055, 0.57727, 0.57366, 0.56968, 0.56539, 0.56088, 0.55620, 0.55140, 0.54649, 0.54148, 0.53639,\
        0.53124, 0.52607, 0.52090, 0.51572, 0.51052, 0.50550, 0.50089, 0.49654, 0.49211, 0.48716, 0.48150,\
        0.47516, 0.46821, 0.46079, 0.45296, 0.44480, 0.43635, 0.42763, 0.41866, 0.40943, 0.39996, 0.39023],\
        [0.39948, 0.40283, 0.40649, 0.40997, 0.41307, 0.41587, 0.41854, 0.42125, 0.42411, 0.42717, 0.43046,\
        0.43397, 0.43768, 0.44156, 0.44558, 0.44972, 0.45394, 0.45820, 0.46247, 0.46671, 0.47087, 0.47493,\
        0.47885, 0.48259, 0.48608, 0.48922, 0.49182, 0.49378, 0.49534, 0.49704, 0.49919, 0.50238, 0.50731,\
        0.51281, 0.51857, 0.52460, 0.53089, 0.53743, 0.54414, 0.55098, 0.55785, 0.56467, 0.57136, 0.57786,\
        0.58412, 0.58992, 0.59486, 0.59988, 0.60426, 0.60854, 0.61277, 0.61684, 0.62060, 0.62405, 0.62721,\
        0.63008, 0.63265, 0.63493, 0.63694, 0.63870, 0.64025, 0.64160, 0.64276, 0.64374, 0.64453, 0.64511,\
        0.64545, 0.64551, 0.64526, 0.64466, 0.64368, 0.64229, 0.64048, 0.63827, 0.63567, 0.63269, 0.62940,\
        0.62587, 0.62216, 0.61834, 0.61443, 0.61051, 0.60657, 0.60255, 0.59880, 0.59562, 0.59290, 0.58999,\
        0.58691, 0.58362, 0.57987, 0.57565, 0.57109, 0.56635, 0.56148, 0.55653, 0.55151, 0.54643, 0.54130,\
        0.53615, 0.53100, 0.52587, 0.52078, 0.51578, 0.51099, 0.50651, 0.50226, 0.49794, 0.49302, 0.48729,\
        0.48079, 0.47370, 0.46617, 0.45828, 0.45011, 0.44168, 0.43300, 0.42410, 0.41495, 0.40556, 0.39592],\
        [0.40429, 0.40764, 0.41099, 0.41426, 0.41733, 0.42019, 0.42295, 0.42571, 0.42857, 0.43159, 0.43480,\
        0.43821, 0.44180, 0.44555, 0.44945, 0.45347, 0.45758, 0.46174, 0.46594, 0.47012, 0.47425, 0.47830,\
        0.48224, 0.48602, 0.48963, 0.49299, 0.49583, 0.49777, 0.49917, 0.50090, 0.50339, 0.50687, 0.51166,\
        0.51730, 0.52342, 0.52987, 0.53659, 0.54349, 0.55052, 0.55757, 0.56454, 0.57135, 0.57791, 0.58417,\
        0.59021, 0.59618, 0.60065, 0.60512, 0.60981, 0.61420, 0.61847, 0.62257, 0.62643, 0.63002, 0.63333,\
        0.63635, 0.63907, 0.64149, 0.64363, 0.64551, 0.64715, 0.64856, 0.64978, 0.65080, 0.65163, 0.65225,\
        0.65263, 0.65273, 0.65249, 0.65188, 0.65088, 0.64948, 0.64765, 0.64543, 0.64281, 0.63983, 0.63653,\
        0.63298, 0.62922, 0.62533, 0.62133, 0.61728, 0.61320, 0.60915, 0.60534, 0.60214, 0.59942, 0.59620,\
        0.59308, 0.58988, 0.58592, 0.58134, 0.57649, 0.57152, 0.56650, 0.56143, 0.55632, 0.55119, 0.54604,\
        0.54090, 0.53578, 0.53071, 0.52572, 0.52087, 0.51623, 0.51184, 0.50761, 0.50329, 0.49841, 0.49253,\
        0.48585, 0.47862, 0.47100, 0.46308, 0.45493, 0.44656, 0.43797, 0.42916, 0.42011, 0.41082, 0.40129],\
        [0.40901, 0.41234, 0.41555, 0.41867, 0.42166, 0.42452, 0.42731, 0.43009, 0.43295, 0.43593, 0.43907,\
        0.44238, 0.44586, 0.44950, 0.45327, 0.45717, 0.46117, 0.46524, 0.46936, 0.47348, 0.47758, 0.48161,\
        0.48553, 0.48931, 0.49297, 0.49649, 0.49972, 0.50178, 0.50341, 0.50539, 0.50817, 0.51190, 0.51669,\
        0.52235, 0.52862, 0.53532, 0.54233, 0.54953, 0.55680, 0.56402, 0.57107, 0.57784, 0.58425, 0.59018,\
        0.59567, 0.60108, 0.60623, 0.61071, 0.61530, 0.61980, 0.62415, 0.62834, 0.63232, 0.63606, 0.63952,\
        0.64269, 0.64556, 0.64812, 0.65039, 0.65237, 0.65409, 0.65556, 0.65682, 0.65788, 0.65876, 0.65943,\
        0.65987, 0.66000, 0.65979, 0.65919, 0.65818, 0.65675, 0.65491, 0.65267, 0.65003, 0.64704, 0.64372,\
        0.64013, 0.63632, 0.63235, 0.62828, 0.62414, 0.61999, 0.61594, 0.61213, 0.60874, 0.60554, 0.60211,\
        0.59907, 0.59603, 0.59158, 0.58656, 0.58147, 0.57635, 0.57122, 0.56607, 0.56092, 0.55575, 0.55059,\
        0.54546, 0.54036, 0.53534, 0.53041, 0.52563, 0.52102, 0.51659, 0.51225, 0.50778, 0.50282, 0.49688,\
        0.49012, 0.48283, 0.47519, 0.46732, 0.45925, 0.45100, 0.44253, 0.43385, 0.42493, 0.41577, 0.40635],\
        [0.41367, 0.41696, 0.42009, 0.42311, 0.42602, 0.42885, 0.43162, 0.43440, 0.43724, 0.44018, 0.44325,\
        0.44647, 0.44985, 0.45337, 0.45703, 0.46081, 0.46469, 0.46867, 0.47270, 0.47677, 0.48084, 0.48484,\
        0.48872, 0.49244, 0.49603, 0.49949, 0.50273, 0.50532, 0.50752, 0.50997, 0.51308, 0.51703, 0.52189,\
        0.52758, 0.53393, 0.54077, 0.54798, 0.55540, 0.56288, 0.57024, 0.57731, 0.58401, 0.59031, 0.59604,\
        0.60115, 0.60621, 0.61127, 0.61606, 0.62073, 0.62532, 0.62979, 0.63411, 0.63823, 0.64213, 0.64576,\
        0.64909, 0.65211, 0.65482, 0.65721, 0.65929, 0.66109, 0.66260, 0.66389, 0.66498, 0.66590, 0.66664,\
        0.66716, 0.66736, 0.66719, 0.66659, 0.66556, 0.66411, 0.66225, 0.65998, 0.65733, 0.65430, 0.65094,\
        0.64730, 0.64343, 0.63939, 0.63524, 0.63103, 0.62683, 0.62272, 0.61881, 0.61513, 0.61158, 0.60807,\
        0.60481, 0.60129, 0.59627, 0.59111, 0.58599, 0.58083, 0.57566, 0.57048, 0.56529, 0.56010, 0.55493,\
        0.54980, 0.54471, 0.53969, 0.53477, 0.52997, 0.52529, 0.52070, 0.51612, 0.51136, 0.50616, 0.50024,\
        0.49353, 0.48626, 0.47871, 0.47098, 0.46308, 0.45500, 0.44671, 0.43820, 0.42943, 0.42041, 0.41113],\
        [0.41829, 0.42151, 0.42458, 0.42752, 0.43037, 0.43314, 0.43589, 0.43864, 0.44144, 0.44433, 0.44733,\
        0.45046, 0.45374, 0.45715, 0.46069, 0.46436, 0.46813, 0.47200, 0.47595, 0.47995, 0.48398, 0.48799,\
        0.49186, 0.49548, 0.49895, 0.50240, 0.50568, 0.50863, 0.51140, 0.51435, 0.51782, 0.52200, 0.52697,\
        0.53270, 0.53909, 0.54602, 0.55335, 0.56093, 0.56859, 0.57609, 0.58316, 0.58964, 0.59578, 0.60150,\
        0.60646, 0.61137, 0.61631, 0.62120, 0.62600, 0.63073, 0.63535, 0.63983, 0.64414, 0.64822, 0.65204,\
        0.65556, 0.65875, 0.66160, 0.66411, 0.66630, 0.66816, 0.66971, 0.67100, 0.67210, 0.67307, 0.67390,\
        0.67451, 0.67481, 0.67468, 0.67407, 0.67301, 0.67152, 0.66964, 0.66735, 0.66467, 0.66160, 0.65819,\
        0.65448, 0.65054, 0.64642, 0.64218, 0.63789, 0.63361, 0.62940, 0.62531, 0.62136, 0.61749, 0.61365,\
        0.60975, 0.60533, 0.60017, 0.59528, 0.59020, 0.58504, 0.57986, 0.57466, 0.56945, 0.56424, 0.55906,\
        0.55391, 0.54880, 0.54376, 0.53878, 0.53388, 0.52904, 0.52422, 0.51933, 0.51422, 0.50875, 0.50276,\
        0.49610, 0.48893, 0.48159, 0.47411, 0.46647, 0.45863, 0.45056, 0.44224, 0.43365, 0.42478, 0.41564],\
        [0.42286, 0.42601, 0.42901, 0.43188, 0.43466, 0.43739, 0.44008, 0.44279, 0.44554, 0.44837, 0.45130,\
        0.45435, 0.45753, 0.46083, 0.46426, 0.46781, 0.47147, 0.47523, 0.47907, 0.48299, 0.48698, 0.49100,\
        0.49496, 0.49857, 0.50194, 0.50536, 0.50871, 0.51192, 0.51509, 0.51846, 0.52226, 0.52665, 0.53173,\
        0.53750, 0.54390, 0.55085, 0.55824, 0.56592, 0.57372, 0.58140, 0.58857, 0.59465, 0.60025, 0.60587,\
        0.61107, 0.61609, 0.62110, 0.62611, 0.63108, 0.63599, 0.64080, 0.64549, 0.65002, 0.65433, 0.65837,\
        0.66209, 0.66546, 0.66845, 0.67110, 0.67341, 0.67536, 0.67692, 0.67817, 0.67925, 0.68026, 0.68119,\
        0.68195, 0.68238, 0.68229, 0.68164, 0.68050, 0.67897, 0.67706, 0.67476, 0.67203, 0.66891, 0.66542,\
        0.66163, 0.65760, 0.65338, 0.64905, 0.64467, 0.64028, 0.63593, 0.63164, 0.62741, 0.62320, 0.61893,\
        0.61448, 0.60965, 0.60458, 0.59949, 0.59426, 0.58908, 0.58390, 0.57868, 0.57343, 0.56819, 0.56298,\
        0.55780, 0.55265, 0.54754, 0.54247, 0.53743, 0.53238, 0.52729, 0.52205, 0.51658, 0.51076, 0.50451,\
        0.49783, 0.49096, 0.48397, 0.47685, 0.46954, 0.46198, 0.45415, 0.44603, 0.43762, 0.42891, 0.41991],\
        [0.42737, 0.43045, 0.43338, 0.43618, 0.43890, 0.44156, 0.44420, 0.44685, 0.44955, 0.45231, 0.45517,\
        0.45814, 0.46122, 0.46442, 0.46774, 0.47117, 0.47471, 0.47835, 0.48208, 0.48588, 0.48978, 0.49376,\
        0.49780, 0.50159, 0.50495, 0.50834, 0.51175, 0.51516, 0.51863, 0.52231, 0.52636, 0.53092, 0.53607,\
        0.54184, 0.54821, 0.55511, 0.56247, 0.57016, 0.57801, 0.58582, 0.59330, 0.59965, 0.60435, 0.60972,\
        0.61504, 0.62027, 0.62549, 0.63072, 0.63593, 0.64108, 0.64614, 0.65108, 0.65586, 0.66043, 0.66474,\
        0.66870, 0.67226, 0.67540, 0.67816, 0.68061, 0.68270, 0.68429, 0.68544, 0.68646, 0.68751, 0.68854,\
        0.68946, 0.69007, 0.69003, 0.68925, 0.68800, 0.68642, 0.68449, 0.68218, 0.67940, 0.67619, 0.67260,\
        0.66871, 0.66457, 0.66024, 0.65581, 0.65131, 0.64680, 0.64229, 0.63780, 0.63331, 0.62877, 0.62412,\
        0.61929, 0.61421, 0.60896, 0.60361, 0.59823, 0.59306, 0.58787, 0.58257, 0.57725, 0.57198, 0.56672,\
        0.56150, 0.55629, 0.55109, 0.54590, 0.54069, 0.53543, 0.53006, 0.52450, 0.51869, 0.51253, 0.50602,\
        0.49930, 0.49273, 0.48613, 0.47940, 0.47243, 0.46516, 0.45756, 0.44964, 0.44139, 0.43282, 0.42396],\
        [0.43183, 0.43483, 0.43768, 0.44041, 0.44305, 0.44565, 0.44823, 0.45082, 0.45345, 0.45615, 0.45893,\
        0.46181, 0.46480, 0.46790, 0.47111, 0.47444, 0.47787, 0.48139, 0.48499, 0.48866, 0.49240, 0.49624,\
        0.50018, 0.50406, 0.50768, 0.51121, 0.51474, 0.51831, 0.52200, 0.52589, 0.53011, 0.53477, 0.53994,\
        0.54566, 0.55192, 0.55870, 0.56592, 0.57349, 0.58125, 0.58892, 0.59617, 0.60258, 0.60783, 0.61304,\
        0.61845, 0.62394, 0.62948, 0.63504, 0.64056, 0.64601, 0.65135, 0.65657, 0.66164, 0.66652, 0.67113,\
        0.67538, 0.67917, 0.68244, 0.68528, 0.68786, 0.69023, 0.69188, 0.69282, 0.69380, 0.69492, 0.69603,\
        0.69706, 0.69786, 0.69784, 0.69684, 0.69549, 0.69385, 0.69190, 0.68959, 0.68673, 0.68338, 0.67969,\
        0.67569, 0.67142, 0.66696, 0.66240, 0.65779, 0.65316, 0.64850, 0.64382, 0.63908, 0.63426, 0.62931,\
        0.62418, 0.61886, 0.61340, 0.60785, 0.60234, 0.59723, 0.59181, 0.58632, 0.58094, 0.57562, 0.57032,\
        0.56503, 0.55975, 0.55445, 0.54913, 0.54375, 0.53829, 0.53268, 0.52687, 0.52080, 0.51443, 0.50781,\
        0.50115, 0.49468, 0.48836, 0.48197, 0.47531, 0.46828, 0.46088, 0.45311, 0.44500, 0.43656, 0.42781],\
        [0.43623, 0.43914, 0.44191, 0.44456, 0.44712, 0.44965, 0.45215, 0.45468, 0.45724, 0.45987, 0.46258,\
        0.46538, 0.46828, 0.47129, 0.47441, 0.47763, 0.48095, 0.48437, 0.48786, 0.49140, 0.49499, 0.49869,\
        0.50256, 0.50648, 0.51027, 0.51397, 0.51764, 0.52137, 0.52520, 0.52923, 0.53355, 0.53823, 0.54335,\
        0.54894, 0.55502, 0.56157, 0.56854, 0.57587, 0.58341, 0.59089, 0.59779, 0.60406, 0.60983, 0.61545,\
        0.62121, 0.62711, 0.63310, 0.63908, 0.64500, 0.65081, 0.65647, 0.66199, 0.66737, 0.67258, 0.67755,\
        0.68215, 0.68621, 0.68963, 0.69247, 0.69508, 0.69777, 0.69972, 0.70041, 0.70151, 0.70270, 0.70385,\
        0.70486, 0.70555, 0.70545, 0.70444, 0.70303, 0.70130, 0.69926, 0.69700, 0.69393, 0.69037, 0.68665,\
        0.68253, 0.67808, 0.67347, 0.66880, 0.66409, 0.65935, 0.65457, 0.64971, 0.64477, 0.63971, 0.63451,\
        0.62914, 0.62360, 0.61795, 0.61226, 0.60668, 0.60123, 0.59539, 0.58990, 0.58453, 0.57917, 0.57381,\
        0.56845, 0.56307, 0.55767, 0.55222, 0.54669, 0.54106, 0.53527, 0.52929, 0.52307, 0.51661, 0.50997,\
        0.50330, 0.49689, 0.49087, 0.48477, 0.47831, 0.47143, 0.46415, 0.45648, 0.44847, 0.44013, 0.43148],\
        [0.44056, 0.44338, 0.44606, 0.44862, 0.45110, 0.45354, 0.45598, 0.45843, 0.46092, 0.46348, 0.46612,\
        0.46885, 0.47167, 0.47459, 0.47762, 0.48074, 0.48397, 0.48730, 0.49073, 0.49422, 0.49767, 0.50128,\
        0.50513, 0.50903, 0.51290, 0.51671, 0.52050, 0.52433, 0.52826, 0.53236, 0.53669, 0.54133, 0.54634,\
        0.55173, 0.55754, 0.56377, 0.57038, 0.57733, 0.58453, 0.59185, 0.59902, 0.60495, 0.61085, 0.61699,\
        0.62334, 0.62984, 0.63641, 0.64294, 0.64935, 0.65556, 0.66156, 0.66736, 0.67303, 0.67859, 0.68395,\
        0.68895, 0.69337, 0.69701, 0.69988, 0.70254, 0.70523, 0.70738, 0.70862, 0.70978, 0.71098, 0.71206,\
        0.71290, 0.71333, 0.71308, 0.71212, 0.71064, 0.70879, 0.70662, 0.70412, 0.70069, 0.69716, 0.69351,\
        0.68917, 0.68449, 0.67974, 0.67497, 0.67020, 0.66539, 0.66050, 0.65551, 0.65039, 0.64513, 0.63972,\
        0.63414, 0.62841, 0.62257, 0.61667, 0.61075, 0.60471, 0.59890, 0.59354, 0.58812, 0.58269, 0.57724,\
        0.57178, 0.56631, 0.56080, 0.55523, 0.54957, 0.54381, 0.53790, 0.53183, 0.52556, 0.51912, 0.51255,\
        0.50598, 0.49971, 0.49388, 0.48791, 0.48151, 0.47465, 0.46738, 0.45976, 0.45181, 0.44355, 0.43498],\
        [0.44483, 0.44755, 0.45012, 0.45259, 0.45498, 0.45733, 0.45969, 0.46206, 0.46449, 0.46698, 0.46955,\
        0.47221, 0.47497, 0.47782, 0.48076, 0.48380, 0.48694, 0.49019, 0.49359, 0.49711, 0.50063, 0.50412,\
        0.50788, 0.51174, 0.51562, 0.51948, 0.52335, 0.52724, 0.53121, 0.53532, 0.53962, 0.54415, 0.54897,\
        0.55411, 0.55959, 0.56540, 0.57155, 0.57799, 0.58465, 0.59141, 0.59795, 0.60433, 0.61098, 0.61789,\
        0.62502, 0.63227, 0.63954, 0.64672, 0.65370, 0.66038, 0.66670, 0.67275, 0.67864, 0.68451, 0.69029,\
        0.69569, 0.70048, 0.70440, 0.70758, 0.71044, 0.71310, 0.71534, 0.71701, 0.71837, 0.71955, 0.72051,\
        0.72115, 0.72132, 0.72089, 0.71984, 0.71826, 0.71624, 0.71382, 0.71094, 0.70752, 0.70398, 0.70013,\
        0.69544, 0.69056, 0.68572, 0.68091, 0.67612, 0.67127, 0.66631, 0.66120, 0.65594, 0.65053, 0.64494,\
        0.63918, 0.63327, 0.62724, 0.62113, 0.61498, 0.60887, 0.60304, 0.59737, 0.59179, 0.58620, 0.58062,\
        0.57505, 0.56947, 0.56386, 0.55818, 0.55243, 0.54657, 0.54059, 0.53448, 0.52824, 0.52190, 0.51553,\
        0.50924, 0.50319, 0.49739, 0.49138, 0.48483, 0.47785, 0.47054, 0.46293, 0.45501, 0.44680, 0.43831],\
        [0.44902, 0.45163, 0.45410, 0.45646, 0.45875, 0.46101, 0.46328, 0.46558, 0.46794, 0.47037, 0.47288,\
        0.47549, 0.47818, 0.48096, 0.48385, 0.48682, 0.48987, 0.49302, 0.49637, 0.49991, 0.50346, 0.50704,\
        0.51076, 0.51458, 0.51845, 0.52233, 0.52622, 0.53013, 0.53411, 0.53818, 0.54238, 0.54677, 0.55136,\
        0.55620, 0.56129, 0.56665, 0.57228, 0.57816, 0.58422, 0.59037, 0.59657, 0.60330, 0.61076, 0.61854,\
        0.62651, 0.63458, 0.64264, 0.65057, 0.65822, 0.66542, 0.67209, 0.67828, 0.68428, 0.69035, 0.69657,\
        0.70230, 0.70735, 0.71166, 0.71528, 0.71845, 0.72126, 0.72365, 0.72557, 0.72710, 0.72829, 0.72914,\
        0.72957, 0.72949, 0.72885, 0.72762, 0.72586, 0.72361, 0.72092, 0.71778, 0.71424, 0.71040, 0.70612,\
        0.70126, 0.69632, 0.69145, 0.68663, 0.68188, 0.67704, 0.67201, 0.66680, 0.66142, 0.65588, 0.65015,\
        0.64423, 0.63816, 0.63196, 0.62568, 0.61939, 0.61317, 0.60713, 0.60125, 0.59548, 0.58967, 0.58395,\
        0.57827, 0.57259, 0.56688, 0.56110, 0.55526, 0.54933, 0.54331, 0.53720, 0.53103, 0.52481, 0.51864,\
        0.51259, 0.50677, 0.50109, 0.49484, 0.48799, 0.48088, 0.47353, 0.46592, 0.45804, 0.44989, 0.44145],\
        [0.45314, 0.45564, 0.45798, 0.46023, 0.46241, 0.46457, 0.46674, 0.46897, 0.47126, 0.47364, 0.47611,\
        0.47867, 0.48131, 0.48404, 0.48688, 0.48983, 0.49283, 0.49583, 0.49904, 0.50261, 0.50626, 0.50994,\
        0.51369, 0.51752, 0.52138, 0.52526, 0.52915, 0.53305, 0.53699, 0.54098, 0.54507, 0.54927, 0.55361,\
        0.55812, 0.56282, 0.56772, 0.57284, 0.57818, 0.58376, 0.58959, 0.59586, 0.60291, 0.61081, 0.61930,\
        0.62808, 0.63697, 0.64585, 0.65459, 0.66302, 0.67088, 0.67794, 0.68420, 0.69017, 0.69629, 0.70271,\
        0.70888, 0.71429, 0.71899, 0.72303, 0.72654, 0.72958, 0.73217, 0.73428, 0.73593, 0.73713, 0.73786,\
        0.73810, 0.73780, 0.73691, 0.73545, 0.73344, 0.73093, 0.72797, 0.72456, 0.72076, 0.71659, 0.71200,\
        0.70705, 0.70202, 0.69701, 0.69213, 0.68756, 0.68273, 0.67760, 0.67228, 0.66681, 0.66117, 0.65532,\
        0.64928, 0.64305, 0.63669, 0.63025, 0.62381, 0.61743, 0.61117, 0.60507, 0.59902, 0.59299, 0.58721,\
        0.58146, 0.57567, 0.56985, 0.56398, 0.55806, 0.55207, 0.54603, 0.53993, 0.53381, 0.52769, 0.52163,\
        0.51569, 0.50986, 0.50397, 0.49755, 0.49071, 0.48362, 0.47630, 0.46872, 0.46089, 0.45278, 0.44442],\
        [0.45718, 0.45955, 0.46177, 0.46388, 0.46594, 0.46799, 0.47007, 0.47222, 0.47445, 0.47679, 0.47924,\
        0.48177, 0.48437, 0.48704, 0.48983, 0.49280, 0.49587, 0.49884, 0.50190, 0.50545, 0.50915, 0.51291,\
        0.51671, 0.52055, 0.52441, 0.52828, 0.53215, 0.53602, 0.53990, 0.54381, 0.54775, 0.55175, 0.55584,\
        0.56002, 0.56432, 0.56878, 0.57344, 0.57834, 0.58356, 0.58922, 0.59549, 0.60279, 0.61129, 0.62044,\
        0.62995, 0.63961, 0.64927, 0.65883, 0.66814, 0.67689, 0.68457, 0.69084, 0.69666, 0.70302, 0.70956,\
        0.71586, 0.72153, 0.72652, 0.73089, 0.73470, 0.73800, 0.74080, 0.74308, 0.74482, 0.74602, 0.74666,\
        0.74671, 0.74617, 0.74502, 0.74328, 0.74100, 0.73820, 0.73495, 0.73126, 0.72718, 0.72272, 0.71791,\
        0.71282, 0.70760, 0.70229, 0.69755, 0.69334, 0.68832, 0.68298, 0.67758, 0.67208, 0.66639, 0.66045,\
        0.65428, 0.64790, 0.64139, 0.63479, 0.62818, 0.62160, 0.61510, 0.60867, 0.60225, 0.59626, 0.59051,\
        0.58463, 0.57871, 0.57278, 0.56681, 0.56080, 0.55476, 0.54869, 0.54259, 0.53649, 0.53042, 0.52439,\
        0.51841, 0.51242, 0.50625, 0.49975, 0.49304, 0.48607, 0.47883, 0.47132, 0.46354, 0.45550, 0.44719],\
        [0.46114, 0.46338, 0.46545, 0.46743, 0.46935, 0.47128, 0.47325, 0.47532, 0.47750, 0.47982, 0.48226,\
        0.48479, 0.48738, 0.48998, 0.49264, 0.49557, 0.49891, 0.50193, 0.50507, 0.50853, 0.51222, 0.51601,\
        0.51984, 0.52369, 0.52756, 0.53141, 0.53526, 0.53908, 0.54289, 0.54670, 0.55050, 0.55430, 0.55814,\
        0.56201, 0.56595, 0.57000, 0.57424, 0.57877, 0.58373, 0.58934, 0.59584, 0.60349, 0.61236, 0.62207,\
        0.63224, 0.64258, 0.65295, 0.66325, 0.67343, 0.68329, 0.69208, 0.69872, 0.70437, 0.71083, 0.71731,\
        0.72351, 0.72921, 0.73434, 0.73891, 0.74296, 0.74649, 0.74949, 0.75193, 0.75375, 0.75494, 0.75547,\
        0.75534, 0.75456, 0.75313, 0.75109, 0.74850, 0.74540, 0.74185, 0.73789, 0.73354, 0.72884, 0.72384,\
        0.71861, 0.71327, 0.70804, 0.70350, 0.69894, 0.69350, 0.68798, 0.68264, 0.67723, 0.67154, 0.66552,\
        0.65920, 0.65268, 0.64600, 0.63924, 0.63245, 0.62569, 0.61899, 0.61238, 0.60596, 0.60003, 0.59385,\
        0.58774, 0.58169, 0.57563, 0.56956, 0.56347, 0.55736, 0.55125, 0.54514, 0.53903, 0.53295, 0.52690,\
        0.52084, 0.51471, 0.50842, 0.50190, 0.49523, 0.48835, 0.48118, 0.47372, 0.46600, 0.45802, 0.44978],\
        [0.46503, 0.46711, 0.46903, 0.47085, 0.47262, 0.47441, 0.47628, 0.47826, 0.48039, 0.48269, 0.48516,\
        0.48775, 0.49040, 0.49296, 0.49541, 0.49800, 0.50128, 0.50476, 0.50817, 0.51174, 0.51545, 0.51926,\
        0.52311, 0.52697, 0.53083, 0.53467, 0.53848, 0.54225, 0.54599, 0.54969, 0.55336, 0.55699, 0.56059,\
        0.56419, 0.56780, 0.57149, 0.57536, 0.57955, 0.58429, 0.58983, 0.59656, 0.60462, 0.61394, 0.62423,\
        0.63503, 0.64601, 0.65695, 0.66780, 0.67861, 0.68942, 0.69954, 0.70707, 0.71327, 0.71953, 0.72575,\
        0.73171, 0.73727, 0.74239, 0.74706, 0.75127, 0.75500, 0.75819, 0.76078, 0.76267, 0.76383, 0.76426,\
        0.76394, 0.76291, 0.76118, 0.75882, 0.75591, 0.75250, 0.74866, 0.74443, 0.73984, 0.73493, 0.72977,\
        0.72442, 0.71903, 0.71377, 0.70876, 0.70361, 0.69791, 0.69255, 0.68755, 0.68233, 0.67663, 0.67048,\
        0.66401, 0.65731, 0.65047, 0.64355, 0.63660, 0.62967, 0.62282, 0.61610, 0.60958, 0.60320, 0.59689,\
        0.59073, 0.58457, 0.57840, 0.57221, 0.56603, 0.55985, 0.55368, 0.54753, 0.54140, 0.53528, 0.52918,\
        0.52305, 0.51685, 0.51052, 0.50402, 0.49737, 0.49051, 0.48337, 0.47595, 0.46827, 0.46035, 0.45218],\
        [0.46884, 0.47076, 0.47250, 0.47414, 0.47575, 0.47739, 0.47914, 0.48103, 0.48311, 0.48540, 0.48790,\
        0.49061, 0.49342, 0.49614, 0.49850, 0.50094, 0.50409, 0.50769, 0.51134, 0.51505, 0.51883, 0.52267,\
        0.52653, 0.53039, 0.53424, 0.53805, 0.54182, 0.54555, 0.54922, 0.55283, 0.55638, 0.55986, 0.56327,\
        0.56662, 0.56995, 0.57331, 0.57684, 0.58072, 0.58522, 0.59062, 0.59724, 0.60577, 0.61587, 0.62689,\
        0.63838, 0.65000, 0.66148, 0.67270, 0.68376, 0.69484, 0.70571, 0.71475, 0.72206, 0.72855, 0.73458,\
        0.74024, 0.74556, 0.75057, 0.75524, 0.75956, 0.76345, 0.76683, 0.76958, 0.77154, 0.77266, 0.77295,\
        0.77245, 0.77116, 0.76912, 0.76641, 0.76317, 0.75946, 0.75535, 0.75087, 0.74606, 0.74097, 0.73565,\
        0.73018, 0.72466, 0.71917, 0.71368, 0.70801, 0.70218, 0.69717, 0.69254, 0.68748, 0.68162, 0.67526,\
        0.66861, 0.66175, 0.65475, 0.64767, 0.64057, 0.63350, 0.62651, 0.61964, 0.61293, 0.60636, 0.59994,\
        0.59366, 0.58734, 0.58103, 0.57472, 0.56844, 0.56219, 0.55596, 0.54976, 0.54358, 0.53741, 0.53125,\
        0.52506, 0.51881, 0.51248, 0.50604, 0.49943, 0.49255, 0.48538, 0.47797, 0.47034, 0.46249, 0.45440],\
        [0.47257, 0.47431, 0.47586, 0.47730, 0.47873, 0.48021, 0.48183, 0.48363, 0.48565, 0.48792, 0.49045,\
        0.49324, 0.49630, 0.49932, 0.50189, 0.50444, 0.50754, 0.51105, 0.51475, 0.51854, 0.52237, 0.52624,\
        0.53011, 0.53396, 0.53779, 0.54157, 0.54530, 0.54897, 0.55258, 0.55612, 0.55957, 0.56294, 0.56620,\
        0.56936, 0.57244, 0.57551, 0.57871, 0.58226, 0.58653, 0.59195, 0.59885, 0.60765, 0.61831, 0.63001,\
        0.64223, 0.65461, 0.66681, 0.67853, 0.68983, 0.70133, 0.71291, 0.72294, 0.73103, 0.73772, 0.74354,\
        0.74886, 0.75387, 0.75869, 0.76333, 0.76773, 0.77177, 0.77534, 0.77827, 0.78031, 0.78133, 0.78146,\
        0.78079, 0.77925, 0.77685, 0.77378, 0.77021, 0.76623, 0.76188, 0.75718, 0.75217, 0.74690, 0.74143,\
        0.73582, 0.73014, 0.72443, 0.71868, 0.71289, 0.70727, 0.70219, 0.69781, 0.69257, 0.68628, 0.67971,\
        0.67292, 0.66593, 0.65879, 0.65156, 0.64431, 0.63710, 0.62997, 0.62296, 0.61611, 0.60942, 0.60291,\
        0.59641, 0.58995, 0.58349, 0.57706, 0.57068, 0.56436, 0.55807, 0.55182, 0.54558, 0.53935, 0.53312,\
        0.52687, 0.52058, 0.51423, 0.50780, 0.50123, 0.49432, 0.48715, 0.47978, 0.47221, 0.46444, 0.45644],\
        [0.47623, 0.47777, 0.47910, 0.48033, 0.48155, 0.48286, 0.48435, 0.48606, 0.48803, 0.49028, 0.49281,\
        0.49560, 0.49876, 0.50203, 0.50500, 0.50794, 0.51116, 0.51469, 0.51841, 0.52224, 0.52610, 0.52998,\
        0.53384, 0.53767, 0.54147, 0.54521, 0.54890, 0.55252, 0.55607, 0.55955, 0.56293, 0.56621, 0.56937,\
        0.57240, 0.57529, 0.57810, 0.58098, 0.58416, 0.58806, 0.59327, 0.60050, 0.60987, 0.62106, 0.63337,\
        0.64633, 0.65967, 0.67306, 0.68585, 0.69798, 0.71044, 0.72237, 0.73254, 0.74066, 0.74714, 0.75253,\
        0.75736, 0.76199, 0.76659, 0.77121, 0.77569, 0.77985, 0.78358, 0.78677, 0.78893, 0.78972, 0.78963,\
        0.78888, 0.78711, 0.78429, 0.78082, 0.77697, 0.77277, 0.76821, 0.76332, 0.75812, 0.75269, 0.74707,\
        0.74132, 0.73549, 0.72962, 0.72375, 0.71794, 0.71236, 0.70724, 0.70248, 0.69655, 0.69026, 0.68374,\
        0.67690, 0.66981, 0.66254, 0.65517, 0.64779, 0.64043, 0.63316, 0.62602, 0.61902, 0.61220, 0.60551,\
        0.59893, 0.59235, 0.58572, 0.57918, 0.57273, 0.56635, 0.56001, 0.55370, 0.54740, 0.54111, 0.53481,\
        0.52848, 0.52212, 0.51570, 0.50921, 0.50260, 0.49572, 0.48862, 0.48135, 0.47389, 0.46621, 0.45831],\
        [0.47981, 0.48113, 0.48223, 0.48321, 0.48421, 0.48534, 0.48669, 0.48833, 0.49028, 0.49256, 0.49513,\
        0.49793, 0.50100, 0.50455, 0.50799, 0.51137, 0.51485, 0.51849, 0.52227, 0.52612, 0.53000, 0.53388,\
        0.53772, 0.54153, 0.54528, 0.54897, 0.55260, 0.55616, 0.55966, 0.56309, 0.56643, 0.56966, 0.57277,\
        0.57571, 0.57846, 0.58106, 0.58361, 0.58638, 0.58982, 0.59457, 0.60166, 0.61185, 0.62364, 0.63644,\
        0.65006, 0.66445, 0.67957, 0.69490, 0.70849, 0.72186, 0.73376, 0.74347, 0.75096, 0.75672, 0.76139,\
        0.76556, 0.76971, 0.77411, 0.77876, 0.78342, 0.78767, 0.79143, 0.79485, 0.79732, 0.79764, 0.79732,\
        0.79664, 0.79464, 0.79124, 0.78742, 0.78339, 0.77906, 0.77434, 0.76926, 0.76390, 0.75831, 0.75255,\
        0.74667, 0.74071, 0.73471, 0.72873, 0.72283, 0.71710, 0.71154, 0.70582, 0.69982, 0.69385, 0.68746,\
        0.68057, 0.67337, 0.66597, 0.65848, 0.65096, 0.64348, 0.63607, 0.62878, 0.62163, 0.61464, 0.60783,\
        0.60119, 0.59435, 0.58763, 0.58105, 0.57457, 0.56816, 0.56179, 0.55543, 0.54907, 0.54270, 0.53632,\
        0.52991, 0.52345, 0.51694, 0.51034, 0.50358, 0.49674, 0.48986, 0.48274, 0.47539, 0.46782, 0.46003],\
        [0.48331, 0.48440, 0.48522, 0.48593, 0.48669, 0.48763, 0.48887, 0.49047, 0.49244, 0.49479, 0.49751,\
        0.50058, 0.50383, 0.50744, 0.51116, 0.51488, 0.51862, 0.52243, 0.52630, 0.53019, 0.53408, 0.53794,\
        0.54175, 0.54550, 0.54919, 0.55282, 0.55638, 0.55988, 0.56332, 0.56670, 0.57000, 0.57322, 0.57631,\
        0.57922, 0.58189, 0.58429, 0.58650, 0.58884, 0.59184, 0.59635, 0.60367, 0.61380, 0.62549, 0.63835,\
        0.65249, 0.66793, 0.68456, 0.70247, 0.71963, 0.73431, 0.74612, 0.75502, 0.76146, 0.76614, 0.76984,\
        0.77324, 0.77689, 0.78109, 0.78591, 0.79097, 0.79540, 0.79895, 0.80233, 0.80475, 0.80518, 0.80473,\
        0.80376, 0.80132, 0.79754, 0.79355, 0.78954, 0.78517, 0.78027, 0.77500, 0.76946, 0.76373, 0.75784,\
        0.75183, 0.74574, 0.73962, 0.73351, 0.72746, 0.72150, 0.71559, 0.70962, 0.70361, 0.69760, 0.69101,\
        0.68393, 0.67658, 0.66905, 0.66144, 0.65381, 0.64621, 0.63868, 0.63125, 0.62394, 0.61675, 0.60969,\
        0.60268, 0.59586, 0.58925, 0.58272, 0.57626, 0.56984, 0.56343, 0.55702, 0.55059, 0.54415, 0.53768,\
        0.53118, 0.52464, 0.51805, 0.51139, 0.50467, 0.49794, 0.49108, 0.48401, 0.47676, 0.46930, 0.46162],\
        [0.48673, 0.48756, 0.48808, 0.48848, 0.48897, 0.48971, 0.49085, 0.49247, 0.49455, 0.49702, 0.49992,\
        0.50326, 0.50686, 0.51068, 0.51462, 0.51859, 0.52257, 0.52654, 0.53050, 0.53443, 0.53832, 0.54215,\
        0.54591, 0.54959, 0.55320, 0.55673, 0.56020, 0.56362, 0.56699, 0.57032, 0.57359, 0.57680, 0.57990,\
        0.58284, 0.58546, 0.58764, 0.58941, 0.59115, 0.59358, 0.59772, 0.60491, 0.61450, 0.62527, 0.63758,\
        0.65223, 0.66941, 0.68840, 0.70958, 0.73045, 0.74671, 0.75830, 0.76617, 0.77134, 0.77484, 0.77756,\
        0.78024, 0.78345, 0.78756, 0.79261, 0.79823, 0.80318, 0.80681, 0.80988, 0.81188, 0.81243, 0.81182,\
        0.81024, 0.80743, 0.80359, 0.79946, 0.79566, 0.79113, 0.78594, 0.78048, 0.77479, 0.76892, 0.76291,\
        0.75678, 0.75056, 0.74430, 0.73804, 0.73180, 0.72561, 0.71946, 0.71334, 0.70723, 0.70097, 0.69410,\
        0.68689, 0.67941, 0.67176, 0.66405, 0.65633, 0.64864, 0.64101, 0.63346, 0.62600, 0.61864, 0.61139,\
        0.60428, 0.59744, 0.59079, 0.58427, 0.57784, 0.57141, 0.56497, 0.55850, 0.55200, 0.54548, 0.53892,\
        0.53233, 0.52570, 0.51904, 0.51237, 0.50574, 0.49913, 0.49216, 0.48515, 0.47801, 0.47067, 0.46310],\
        [0.49006, 0.49060, 0.49078, 0.49084, 0.49103, 0.49157, 0.49263, 0.49432, 0.49662, 0.49937, 0.50254,\
        0.50616, 0.51008, 0.51417, 0.51836, 0.52257, 0.52673, 0.53084, 0.53488, 0.53885, 0.54273, 0.54651,\
        0.55019, 0.55378, 0.55727, 0.56068, 0.56403, 0.56734, 0.57062, 0.57388, 0.57711, 0.58030, 0.58343,\
        0.58643, 0.58906, 0.59095, 0.59197, 0.59262, 0.59399, 0.59741, 0.60405, 0.61193, 0.62071, 0.63161,\
        0.64671, 0.66707, 0.69184, 0.71813, 0.74111, 0.75780, 0.76874, 0.77553, 0.77963, 0.78220, 0.78423,\
        0.78652, 0.78961, 0.79388, 0.79933, 0.80544, 0.81082, 0.81487, 0.81774, 0.81936, 0.81964, 0.81871,\
        0.81669, 0.81364, 0.80981, 0.80573, 0.80173, 0.79668, 0.79127, 0.78568, 0.77987, 0.77388, 0.76775,\
        0.76150, 0.75515, 0.74873, 0.74228, 0.73583, 0.72941, 0.72302, 0.71664, 0.71023, 0.70360, 0.69670,\
        0.68942, 0.68181, 0.67407, 0.66630, 0.65852, 0.65077, 0.64308, 0.63545, 0.62789, 0.62042, 0.61305,\
        0.60588, 0.59896, 0.59224, 0.58579, 0.57938, 0.57294, 0.56644, 0.55989, 0.55331, 0.54670, 0.54006,\
        0.53339, 0.52668, 0.51995, 0.51319, 0.50639, 0.49957, 0.49294, 0.48617, 0.47919, 0.47197, 0.46451],\
        [0.49327, 0.49349, 0.49329, 0.49299, 0.49287, 0.49318, 0.49417, 0.49598, 0.49858, 0.50176, 0.50531,\
        0.50929, 0.51355, 0.51796, 0.52241, 0.52683, 0.53116, 0.53538, 0.53948, 0.54345, 0.54729, 0.55100,\
        0.55458, 0.55804, 0.56138, 0.56463, 0.56782, 0.57098, 0.57415, 0.57733, 0.58051, 0.58367, 0.58678,\
        0.58986, 0.59261, 0.59417, 0.59388, 0.59235, 0.59121, 0.59261, 0.59692, 0.60238, 0.60871, 0.61729,\
        0.63201, 0.65488, 0.68786, 0.72159, 0.74753, 0.76455, 0.77508, 0.78146, 0.78525, 0.78764, 0.78970,\
        0.79232, 0.79596, 0.80097, 0.80730, 0.81376, 0.81933, 0.82345, 0.82606, 0.82724, 0.82706, 0.82568,\
        0.82325, 0.81994, 0.81599, 0.81169, 0.80704, 0.80180, 0.79632, 0.79065, 0.78472, 0.77861, 0.77237,\
        0.76600, 0.75950, 0.75290, 0.74624, 0.73955, 0.73287, 0.72618, 0.71950, 0.71278, 0.70600, 0.69906,\
        0.69144, 0.68374, 0.67598, 0.66820, 0.66041, 0.65264, 0.64492, 0.63726, 0.62966, 0.62214, 0.61470,\
        0.60735, 0.60016, 0.59374, 0.58740, 0.58097, 0.57444, 0.56784, 0.56120, 0.55453, 0.54784, 0.54112,\
        0.53438, 0.52761, 0.52082, 0.51402, 0.50721, 0.50049, 0.49393, 0.48724, 0.48035, 0.47324, 0.46586],\
        [0.49629, 0.49615, 0.49557, 0.49494, 0.49451, 0.49459, 0.49550, 0.49745, 0.50041, 0.50407, 0.50820,\
        0.51267, 0.51733, 0.52208, 0.52679, 0.53141, 0.53588, 0.54018, 0.54430, 0.54824, 0.55202, 0.55563,\
        0.55908, 0.56237, 0.56552, 0.56856, 0.57153, 0.57449, 0.57749, 0.58059, 0.58374, 0.58687, 0.58992,\
        0.59301, 0.59615, 0.59756, 0.59525, 0.58997, 0.58364, 0.57947, 0.57919, 0.58201, 0.58610, 0.59286,\
        0.60558, 0.62611, 0.66345, 0.71067, 0.74373, 0.76253, 0.77455, 0.78240, 0.78742, 0.79087, 0.79403,\
        0.79803, 0.80342, 0.80987, 0.81694, 0.82362, 0.82910, 0.83293, 0.83505, 0.83559, 0.83477, 0.83281,\
        0.82990, 0.82623, 0.82200, 0.81736, 0.81233, 0.80693, 0.80131, 0.79547, 0.78933, 0.78312, 0.77679,\
        0.77029, 0.76361, 0.75681, 0.74991, 0.74297, 0.73599, 0.72899, 0.72196, 0.71488, 0.70769, 0.70027,\
        0.69281, 0.68522, 0.67753, 0.66979, 0.66202, 0.65427, 0.64655, 0.63890, 0.63132, 0.62383, 0.61645,\
        0.60922, 0.60229, 0.59574, 0.58921, 0.58260, 0.57591, 0.56917, 0.56241, 0.55565, 0.54888, 0.54210,\
        0.53531, 0.52850, 0.52169, 0.51489, 0.50819, 0.50167, 0.49504, 0.48834, 0.48153, 0.47449, 0.46718],\
        [0.49897, 0.49838, 0.49755, 0.49674, 0.49605, 0.49588, 0.49674, 0.49890, 0.50235, 0.50664, 0.51139,\
        0.51639, 0.52150, 0.52658, 0.53155, 0.53634, 0.54091, 0.54525, 0.54935, 0.55323, 0.55691, 0.56038,\
        0.56367, 0.56676, 0.56968, 0.57244, 0.57510, 0.57777, 0.58056, 0.58354, 0.58673, 0.58999, 0.59308,\
        0.59605, 0.59965, 0.60150, 0.59708, 0.58642, 0.57182, 0.55956, 0.55389, 0.55355, 0.55552, 0.55998,\
        0.56927, 0.58462, 0.62132, 0.68043, 0.72261, 0.74770, 0.76546, 0.77771, 0.78603, 0.79208, 0.79766,\
        0.80435, 0.81214, 0.82027, 0.82812, 0.83498, 0.84020, 0.84344, 0.84474, 0.84441, 0.84275, 0.84007,\
        0.83657, 0.83245, 0.82785, 0.82286, 0.81754, 0.81193, 0.80608, 0.79994, 0.79368, 0.78747, 0.78106,\
        0.77439, 0.76751, 0.76047, 0.75332, 0.74609, 0.73880, 0.73148, 0.72411, 0.71667, 0.70914, 0.70153,\
        0.69403, 0.68648, 0.67884, 0.67113, 0.66340, 0.65568, 0.64799, 0.64037, 0.63285, 0.62544, 0.61821,\
        0.61119, 0.60446, 0.59780, 0.59102, 0.58415, 0.57724, 0.57035, 0.56348, 0.55664, 0.54981, 0.54299,\
        0.53618, 0.52937, 0.52255, 0.51575, 0.50899, 0.50235, 0.49585, 0.48940, 0.48273, 0.47575, 0.46847],\
        [0.50070, 0.49999, 0.49933, 0.49856, 0.49768, 0.49718, 0.49813, 0.50079, 0.50483, 0.50975, 0.51509,\
        0.52062, 0.52614, 0.53154, 0.53673, 0.54165, 0.54628, 0.55060, 0.55464, 0.55841, 0.56194, 0.56525,\
        0.56834, 0.57121, 0.57384, 0.57626, 0.57852, 0.58077, 0.58319, 0.58596, 0.58920, 0.59290, 0.59664,\
        0.60036, 0.60511, 0.60759, 0.60163, 0.58312, 0.55731, 0.53451, 0.52415, 0.52193, 0.52190, 0.52334,\
        0.53234, 0.54764, 0.57840, 0.63156, 0.68354, 0.72187, 0.74929, 0.76832, 0.78183, 0.79212, 0.80144,\
        0.81175, 0.82212, 0.83191, 0.84062, 0.84769, 0.85253, 0.85488, 0.85504, 0.85355, 0.85089, 0.84736,\
        0.84321, 0.83857, 0.83356, 0.82824, 0.82263, 0.81677, 0.81066, 0.80430, 0.79804, 0.79180, 0.78523,\
        0.77832, 0.77118, 0.76388, 0.75646, 0.74894, 0.74135, 0.73371, 0.72603, 0.71831, 0.71058, 0.70290,\
        0.69527, 0.68765, 0.67999, 0.67230, 0.66459, 0.65690, 0.64924, 0.64166, 0.63418, 0.62685, 0.61972,\
        0.61283, 0.60616, 0.59950, 0.59251, 0.58540, 0.57832, 0.57131, 0.56436, 0.55747, 0.55062, 0.54379,\
        0.53700, 0.53021, 0.52344, 0.51667, 0.50990, 0.50309, 0.49684, 0.49068, 0.48401, 0.47700, 0.46970],\
        [0.50186, 0.50162, 0.50113, 0.50051, 0.49962, 0.49865, 0.49988, 0.50339, 0.50813, 0.51363, 0.51949,\
        0.52546, 0.53135, 0.53701, 0.54237, 0.54737, 0.55198, 0.55623, 0.56015, 0.56376, 0.56711, 0.57022,\
        0.57310, 0.57572, 0.57805, 0.58005, 0.58179, 0.58344, 0.58527, 0.58756, 0.59060, 0.59461, 0.59958,\
        0.60472, 0.61122, 0.61454, 0.60610, 0.57987, 0.54260, 0.50764, 0.49395, 0.48994, 0.48789, 0.48663,\
        0.49614, 0.51114, 0.54219, 0.59062, 0.64570, 0.69358, 0.72968, 0.75607, 0.77587, 0.79193, 0.80641,\
        0.82042, 0.83339, 0.84475, 0.85425, 0.86155, 0.86592, 0.86701, 0.86564, 0.86279, 0.85900, 0.85458,\
        0.84972, 0.84455, 0.83912, 0.83347, 0.82760, 0.82152, 0.81526, 0.80890, 0.80257, 0.79617, 0.78922,\
        0.78200, 0.77459, 0.76703, 0.75935, 0.75155, 0.74367, 0.73574, 0.72777, 0.71981, 0.71191, 0.70407,\
        0.69637, 0.68872, 0.68102, 0.67330, 0.66561, 0.65794, 0.65031, 0.64274, 0.63528, 0.62797, 0.62084,\
        0.61392, 0.60719, 0.60048, 0.59337, 0.58617, 0.57904, 0.57199, 0.56502, 0.55811, 0.55127, 0.54448,\
        0.53773, 0.53101, 0.52433, 0.51769, 0.51115, 0.50486, 0.49885, 0.49218, 0.48528, 0.47818, 0.47085],\
        [0.50312, 0.50324, 0.50302, 0.50256, 0.50188, 0.50127, 0.50298, 0.50715, 0.51245, 0.51840, 0.52467,\
        0.53099, 0.53715, 0.54302, 0.54848, 0.55348, 0.55802, 0.56213, 0.56586, 0.56927, 0.57239, 0.57527,\
        0.57791, 0.58030, 0.58233, 0.58389, 0.58498, 0.58582, 0.58678, 0.58823, 0.59051, 0.59382, 0.59820,\
        0.60459, 0.61297, 0.61819, 0.60870, 0.57335, 0.52691, 0.48875, 0.46727, 0.45867, 0.45343, 0.44869,\
        0.44795, 0.46262, 0.49948, 0.55504, 0.61274, 0.66578, 0.70904, 0.74230, 0.76883, 0.79138, 0.81191,\
        0.83043, 0.84612, 0.85876, 0.86868, 0.87617, 0.88010, 0.87932, 0.87608, 0.87184, 0.86693, 0.86160,\
        0.85604, 0.85034, 0.84452, 0.83855, 0.83242, 0.82614, 0.81974, 0.81327, 0.80678, 0.80003, 0.79278,\
        0.78533, 0.77770, 0.76992, 0.76200, 0.75395, 0.74580, 0.73758, 0.72934, 0.72114, 0.71306, 0.70524,\
        0.69758, 0.68967, 0.68187, 0.67416, 0.66648, 0.65883, 0.65120, 0.64363, 0.63614, 0.62877, 0.62154,\
        0.61446, 0.60749, 0.60054, 0.59351, 0.58643, 0.57937, 0.57236, 0.56543, 0.55856, 0.55176, 0.54502,\
        0.53834, 0.53171, 0.52515, 0.51866, 0.51230, 0.50609, 0.49979, 0.49319, 0.48630, 0.47918, 0.47185],\
        [0.50452, 0.50501, 0.50517, 0.50515, 0.50515, 0.50577, 0.50809, 0.51234, 0.51785, 0.52406, 0.53059,\
        0.53716, 0.54353, 0.54952, 0.55502, 0.55996, 0.56436, 0.56826, 0.57175, 0.57490, 0.57776, 0.58036,\
        0.58275, 0.58493, 0.58675, 0.58788, 0.58820, 0.58805, 0.58791, 0.58822, 0.58931, 0.59140, 0.59467,\
        0.59968, 0.60778, 0.61521, 0.60055, 0.55390, 0.50149, 0.46211, 0.43879, 0.42644, 0.41686, 0.40726,\
        0.39560, 0.40466, 0.45233, 0.51885, 0.58273, 0.63889, 0.68717, 0.72714, 0.76130, 0.79132, 0.81891,\
        0.84253, 0.86084, 0.87425, 0.88362, 0.89065, 0.89455, 0.89064, 0.88590, 0.88060, 0.87461, 0.86838,\
        0.86215, 0.85597, 0.84977, 0.84349, 0.83709, 0.83057, 0.82394, 0.81722, 0.81036, 0.80324, 0.79589,\
        0.78831, 0.78052, 0.77256, 0.76442, 0.75614, 0.74775, 0.73928, 0.73078, 0.72230, 0.71393, 0.70580,\
        0.69796, 0.69019, 0.68253, 0.67490, 0.66726, 0.65960, 0.65195, 0.64433, 0.63677, 0.62930, 0.62194,\
        0.61466, 0.60743, 0.60025, 0.59330, 0.58636, 0.57941, 0.57248, 0.56561, 0.55881, 0.55207, 0.54540,\
        0.53880, 0.53227, 0.52582, 0.51943, 0.51311, 0.50678, 0.50035, 0.49383, 0.48705, 0.47999, 0.47267],\
        [0.50613, 0.50704, 0.50773, 0.50839, 0.50930, 0.51097, 0.51395, 0.51841, 0.52403, 0.53039, 0.53710,\
        0.54386, 0.55038, 0.55646, 0.56194, 0.56676, 0.57095, 0.57457, 0.57775, 0.58061, 0.58320, 0.58549,\
        0.58756, 0.58954, 0.59132, 0.59215, 0.59162, 0.59035, 0.58904, 0.58810, 0.58790, 0.58870, 0.59054,\
        0.59335, 0.59757, 0.60537, 0.58454, 0.52435, 0.46738, 0.42829, 0.40445, 0.38867, 0.37319, 0.35912,\
        0.34704, 0.35233, 0.39198, 0.46763, 0.54675, 0.60788, 0.66212, 0.71001, 0.75411, 0.79360, 0.82945,\
        0.85803, 0.87826, 0.89216, 0.89989, 0.90470, 0.90589, 0.90044, 0.89583, 0.88934, 0.88211, 0.87494,\
        0.86809, 0.86147, 0.85491, 0.84830, 0.84159, 0.83477, 0.82784, 0.82080, 0.81362, 0.80627, 0.79873,\
        0.79097, 0.78307, 0.77496, 0.76663, 0.75815, 0.74955, 0.74087, 0.73216, 0.72344, 0.71474, 0.70610,\
        0.69793, 0.69065, 0.68321, 0.67564, 0.66799, 0.66030, 0.65259, 0.64489, 0.63724, 0.62964, 0.62214,\
        0.61471, 0.60738, 0.60014, 0.59312, 0.58618, 0.57927, 0.57241, 0.56560, 0.55886, 0.55220, 0.54561,\
        0.53909, 0.53265, 0.52628, 0.51997, 0.51369, 0.50737, 0.50096, 0.49442, 0.48765, 0.48061, 0.47333],\
        [0.50793, 0.50933, 0.51063, 0.51204, 0.51383, 0.51637, 0.51998, 0.52476, 0.53056, 0.53707, 0.54396,\
        0.55089, 0.55755, 0.56370, 0.56913, 0.57379, 0.57769, 0.58097, 0.58380, 0.58638, 0.58873, 0.59071,\
        0.59230, 0.59394, 0.59592, 0.59678, 0.59523, 0.59297, 0.59069, 0.58861, 0.58721, 0.58689, 0.58765,\
        0.58892, 0.58939, 0.58412, 0.55176, 0.49410, 0.43632, 0.39572, 0.37045, 0.34941, 0.32939, 0.31580,\
        0.30724, 0.30328, 0.33090, 0.40484, 0.49390, 0.56902, 0.63308, 0.69260, 0.74705, 0.79695, 0.84335,\
        0.87808, 0.89872, 0.91268, 0.91940, 0.92135, 0.91917, 0.91347, 0.90656, 0.89827, 0.88951, 0.88140,\
        0.87399, 0.86695, 0.86001, 0.85301, 0.84592, 0.83873, 0.83144, 0.82407, 0.81659, 0.80897, 0.80118,\
        0.79338, 0.78541, 0.77713, 0.76863, 0.75996, 0.75120, 0.74238, 0.73353, 0.72469, 0.71594, 0.70743,\
        0.69948, 0.69175, 0.68412, 0.67646, 0.66874, 0.66097, 0.65316, 0.64535, 0.63757, 0.62984, 0.62220,\
        0.61466, 0.60728, 0.60010, 0.59292, 0.58590, 0.57900, 0.57218, 0.56543, 0.55875, 0.55215, 0.54563,\
        0.53919, 0.53282, 0.52653, 0.52029, 0.51406, 0.50782, 0.50148, 0.49491, 0.48810, 0.48107, 0.47380],\
        [0.50989, 0.51179, 0.51371, 0.51584, 0.51841, 0.52168, 0.52586, 0.53102, 0.53706, 0.54376, 0.55084,\
        0.55797, 0.56482, 0.57105, 0.57643, 0.58089, 0.58450, 0.58736, 0.58981, 0.59211, 0.59439, 0.59620,\
        0.59724, 0.59816, 0.60001, 0.60101, 0.59872, 0.59637, 0.59364, 0.59061, 0.58821, 0.58715, 0.58724,\
        0.58730, 0.58397, 0.56721, 0.52491, 0.47039, 0.41896, 0.37915, 0.35036, 0.32545, 0.30385, 0.29252,\
        0.28371, 0.27640, 0.30168, 0.39639, 0.47472, 0.54136, 0.60468, 0.67485, 0.73833, 0.79456, 0.85667,\
        0.90419, 0.92349, 0.93509, 0.93996, 0.93949, 0.93490, 0.92735, 0.91801, 0.90754, 0.89706, 0.88802,\
        0.88006, 0.87257, 0.86513, 0.85763, 0.85006, 0.84242, 0.83474, 0.82702, 0.81926, 0.81146, 0.80361,\
        0.79574, 0.78756, 0.77904, 0.77035, 0.76155, 0.75268, 0.74378, 0.73488, 0.72604, 0.71734, 0.70893,\
        0.70089, 0.69295, 0.68516, 0.67737, 0.66952, 0.66161, 0.65367, 0.64573, 0.63780, 0.62994, 0.62216,\
        0.61448, 0.60695, 0.59958, 0.59248, 0.58549, 0.57861, 0.57182, 0.56511, 0.55848, 0.55193, 0.54546,\
        0.53908, 0.53277, 0.52654, 0.52036, 0.51419, 0.50798, 0.50168, 0.49513, 0.48833, 0.48132, 0.47408],\
        [0.51190, 0.51430, 0.51680, 0.51957, 0.52281, 0.52670, 0.53138, 0.53691, 0.54321, 0.55013, 0.55743,\
        0.56480, 0.57190, 0.57829, 0.58362, 0.58784, 0.59120, 0.59365, 0.59569, 0.59781, 0.60003, 0.60201,\
        0.60293, 0.60367, 0.60483, 0.60516, 0.60367, 0.60158, 0.59875, 0.59500, 0.59205, 0.59094, 0.59123,\
        0.59119, 0.58839, 0.57164, 0.52536, 0.46820, 0.42130, 0.38229, 0.34678, 0.32068, 0.30465, 0.29730,\
        0.28835, 0.28031, 0.31297, 0.40062, 0.47376, 0.53009, 0.58492, 0.65582, 0.72963, 0.79551, 0.86530,\
        0.91811, 0.94359, 0.95550, 0.95928, 0.95713, 0.95065, 0.94117, 0.92984, 0.91761, 0.90562, 0.89526,\
        0.88659, 0.87843, 0.87026, 0.86210, 0.85393, 0.84578, 0.83766, 0.82957, 0.82153, 0.81356, 0.80574,\
        0.79789, 0.78925, 0.78053, 0.77172, 0.76285, 0.75394, 0.74503, 0.73614, 0.72734, 0.71868, 0.71021,\
        0.70190, 0.69409, 0.68627, 0.67831, 0.67029, 0.66222, 0.65413, 0.64603, 0.63796, 0.62994, 0.62203,\
        0.61424, 0.60661, 0.59918, 0.59201, 0.58501, 0.57813, 0.57136, 0.56467, 0.55807, 0.55155, 0.54512,\
        0.53877, 0.53250, 0.52632, 0.52018, 0.51405, 0.50786, 0.50153, 0.49504, 0.48834, 0.48138, 0.47418],\
        [0.51387, 0.51673, 0.51975, 0.52307, 0.52687, 0.53126, 0.53637, 0.54221, 0.54875, 0.55586, 0.56337,\
        0.57100, 0.57841, 0.58510, 0.59043, 0.59425, 0.59742, 0.59973, 0.60143, 0.60373, 0.60599, 0.60807,\
        0.60952, 0.61063, 0.61154, 0.61174, 0.61083, 0.60903, 0.60632, 0.60283, 0.60041, 0.60033, 0.60228,\
        0.60468, 0.60623, 0.59584, 0.54242, 0.47392, 0.42524, 0.38570, 0.35047, 0.32648, 0.31561, 0.31336,\
        0.31275, 0.31423, 0.34948, 0.43044, 0.48656, 0.52716, 0.57083, 0.63689, 0.72018, 0.80448, 0.87201,\
        0.92398, 0.95613, 0.97137, 0.97566, 0.97276, 0.96498, 0.95404, 0.94126, 0.92781, 0.91479, 0.90325,\
        0.89363, 0.88433, 0.87516, 0.86621, 0.85739, 0.84869, 0.84011, 0.83163, 0.82326, 0.81498, 0.80673,\
        0.79841, 0.79010, 0.78148, 0.77269, 0.76382, 0.75493, 0.74606, 0.73723, 0.72850, 0.71991, 0.71154,\
        0.70355, 0.69560, 0.68735, 0.67918, 0.67100, 0.66278, 0.65453, 0.64627, 0.63803, 0.62987, 0.62182,\
        0.61394, 0.60628, 0.59883, 0.59156, 0.58449, 0.57759, 0.57081, 0.56413, 0.55754, 0.55102, 0.54460,\
        0.53826, 0.53201, 0.52585, 0.51976, 0.51369, 0.50755, 0.50125, 0.49482, 0.48816, 0.48124, 0.47407],\
        [0.51573, 0.51898, 0.52244, 0.52623, 0.53047, 0.53526, 0.54069, 0.54677, 0.55347, 0.56071, 0.56833,\
        0.57614, 0.58385, 0.59098, 0.59656, 0.59987, 0.60257, 0.60512, 0.60727, 0.60974, 0.61232, 0.61476,\
        0.61684, 0.61853, 0.61978, 0.62035, 0.62003, 0.61885, 0.61700, 0.61505, 0.61433, 0.61607, 0.62036,\
        0.62612, 0.63084, 0.62268, 0.56830, 0.48529, 0.43779, 0.39449, 0.35769, 0.33771, 0.33310, 0.33746,\
        0.34679, 0.36441, 0.40096, 0.46296, 0.50675, 0.53049, 0.56517, 0.63089, 0.71902, 0.80655, 0.88091,\
        0.93421, 0.96752, 0.98409, 0.98887, 0.98564, 0.97705, 0.96508, 0.95128, 0.93695, 0.92320, 0.91089,\
        0.90005, 0.88936, 0.87937, 0.86969, 0.86024, 0.85101, 0.84198, 0.83313, 0.82445, 0.81589, 0.80740,\
        0.79900, 0.79065, 0.78201, 0.77324, 0.76442, 0.75560, 0.74681, 0.73809, 0.72944, 0.72091, 0.71253,\
        0.70430, 0.69618, 0.68808, 0.67988, 0.67160, 0.66325, 0.65485, 0.64643, 0.63804, 0.62971, 0.62151,\
        0.61352, 0.60582, 0.59835, 0.59101, 0.58390, 0.57698, 0.57020, 0.56351, 0.55691, 0.55037, 0.54392,\
        0.53756, 0.53129, 0.52513, 0.51906, 0.51308, 0.50708, 0.50090, 0.49450, 0.48782, 0.48089, 0.47375],\
        [0.51738, 0.52098, 0.52480, 0.52895, 0.53354, 0.53863, 0.54428, 0.55050, 0.55727, 0.56451, 0.57209,\
        0.57985, 0.58755, 0.59486, 0.60084, 0.60416, 0.60691, 0.60972, 0.61256, 0.61558, 0.61873, 0.62183,\
        0.62470, 0.62720, 0.62919, 0.63050, 0.63102, 0.63081, 0.63022, 0.62993, 0.63095, 0.63407, 0.63923,\
        0.64478, 0.64613, 0.63291, 0.58847, 0.52392, 0.46521, 0.40774, 0.36965, 0.35631, 0.36020, 0.37226,\
        0.38837, 0.41573, 0.45281, 0.49758, 0.51922, 0.53519, 0.56820, 0.63943, 0.73137, 0.80854, 0.89537,\
        0.94827, 0.97872, 0.99455, 0.99915, 0.99558, 0.98639, 0.97366, 0.95906, 0.94399, 0.92953, 0.91628,\
        0.90407, 0.89306, 0.88259, 0.87227, 0.86225, 0.85257, 0.84318, 0.83403, 0.82509, 0.81633, 0.80782,\
        0.79958, 0.79085, 0.78211, 0.77338, 0.76464, 0.75593, 0.74726, 0.73865, 0.73013, 0.72167, 0.71327,\
        0.70486, 0.69669, 0.68867, 0.68046, 0.67209, 0.66361, 0.65507, 0.64651, 0.63797, 0.62948, 0.62111,\
        0.61293, 0.60506, 0.59754, 0.59027, 0.58323, 0.57634, 0.56956, 0.56285, 0.55620, 0.54961, 0.54310,\
        0.53668, 0.53035, 0.52414, 0.51806, 0.51213, 0.50632, 0.50039, 0.49395, 0.48721, 0.48030, 0.47321],\
        [0.51879, 0.52265, 0.52676, 0.53119, 0.53603, 0.54132, 0.54710, 0.55338, 0.56011, 0.56722, 0.57459,\
        0.58206, 0.58935, 0.59607, 0.60174, 0.60613, 0.60981, 0.61339, 0.61709, 0.62096, 0.62496, 0.62896,\
        0.63281, 0.63634, 0.63937, 0.64171, 0.64328, 0.64418, 0.64480, 0.64579, 0.64789, 0.65150, 0.65618,\
        0.65965, 0.65638, 0.63705, 0.59760, 0.55344, 0.49801, 0.43273, 0.39846, 0.38929, 0.39998, 0.41777,\
        0.43511, 0.45839, 0.48349, 0.50386, 0.51780, 0.53902, 0.58354, 0.65515, 0.75208, 0.83654, 0.90643,\
        0.95683, 0.98721, 1.00256, 1.00671, 1.00264, 0.99285, 0.97942, 0.96410, 0.94831, 0.93320, 0.91945,\
        0.90724, 0.89592, 0.88457, 0.87363, 0.86320, 0.85323, 0.84363, 0.83430, 0.82518, 0.81621, 0.80736,\
        0.79867, 0.79025, 0.78171, 0.77310, 0.76448, 0.75588, 0.74734, 0.73888, 0.73051, 0.72221, 0.71396,\
        0.70577, 0.69761, 0.68933, 0.68095, 0.67243, 0.66383, 0.65517, 0.64649, 0.63784, 0.62923, 0.62072,\
        0.61235, 0.60420, 0.59660, 0.58952, 0.58259, 0.57574, 0.56893, 0.56217, 0.55544, 0.54877, 0.54217,\
        0.53566, 0.52923, 0.52292, 0.51673, 0.51070, 0.50491, 0.49918, 0.49282, 0.48620, 0.47939, 0.47241],\
        [0.51989, 0.52397, 0.52829, 0.53293, 0.53793, 0.54335, 0.54919, 0.55543, 0.56205, 0.56894, 0.57600,\
        0.58306, 0.58987, 0.59609, 0.60161, 0.60674, 0.61145, 0.61609, 0.62084, 0.62575, 0.63078, 0.63586,\
        0.64085, 0.64559, 0.64987, 0.65347, 0.65620, 0.65816, 0.65976, 0.66160, 0.66425, 0.66780, 0.67191,\
        0.67476, 0.67200, 0.65802, 0.62977, 0.58807, 0.52978, 0.46867, 0.44797, 0.44585, 0.45483, 0.46612,\
        0.47797, 0.49040, 0.49877, 0.50444, 0.51549, 0.54737, 0.61058, 0.68516, 0.78105, 0.86307, 0.91986,\
        0.96607, 0.99471, 1.00880, 1.01193, 1.00704, 0.99650, 0.98233, 0.96623, 0.94966, 0.93386, 0.91969,\
        0.90740, 0.89602, 0.88442, 0.87337, 0.86291, 0.85294, 0.84333, 0.83398, 0.82480, 0.81574, 0.80678,\
        0.79800, 0.78950, 0.78102, 0.77248, 0.76394, 0.75544, 0.74702, 0.73871, 0.73051, 0.72241, 0.71436,\
        0.70629, 0.69811, 0.68975, 0.68122, 0.67256, 0.66383, 0.65508, 0.64634, 0.63763, 0.62899, 0.62043,\
        0.61202, 0.60387, 0.59624, 0.58910, 0.58215, 0.57525, 0.56836, 0.56149, 0.55466, 0.54787, 0.54115,\
        0.53452, 0.52798, 0.52154, 0.51520, 0.50894, 0.50277, 0.49704, 0.49113, 0.48481, 0.47821, 0.47140],\
        [0.52067, 0.52490, 0.52938, 0.53415, 0.53927, 0.54475, 0.55059, 0.55676, 0.56321, 0.56985, 0.57656,\
        0.58321, 0.58967, 0.59585, 0.60148, 0.60673, 0.61230, 0.61801, 0.62387, 0.62989, 0.63605, 0.64228,\
        0.64848, 0.65453, 0.66020, 0.66520, 0.66917, 0.67207, 0.67441, 0.67676, 0.67995, 0.68351, 0.68750,\
        0.69225, 0.69382, 0.69020, 0.67245, 0.63699, 0.58490, 0.52921, 0.51226, 0.50559, 0.50566, 0.50364,\
        0.50408, 0.50407, 0.50194, 0.50855, 0.53213, 0.58359, 0.67103, 0.74586, 0.82511, 0.89885, 0.94638,\
        0.98100, 1.00335, 1.01409, 1.01525, 1.00907, 0.99758, 0.98257, 0.96564, 0.94821, 0.93155, 0.91677,\
        0.90409, 0.89277, 0.88195, 0.87148, 0.86142, 0.85174, 0.84233, 0.83312, 0.82403, 0.81503, 0.80613,\
        0.79735, 0.78870, 0.78013, 0.77157, 0.76304, 0.75459, 0.74625, 0.73806, 0.73002, 0.72211, 0.71426,\
        0.70636, 0.69823, 0.68979, 0.68111, 0.67233, 0.66352, 0.65472, 0.64598, 0.63731, 0.62873, 0.62026,\
        0.61193, 0.60387, 0.59625, 0.58902, 0.58195, 0.57490, 0.56786, 0.56083, 0.55384, 0.54691, 0.54005,\
        0.53329, 0.52664, 0.52009, 0.51366, 0.50735, 0.50120, 0.49526, 0.48941, 0.48329, 0.47688, 0.47023],\
        [0.52112, 0.52545, 0.53002, 0.53488, 0.54006, 0.54556, 0.55138, 0.55746, 0.56375, 0.57015, 0.57654,\
        0.58282, 0.58889, 0.59469, 0.60023, 0.60627, 0.61268, 0.61939, 0.62632, 0.63342, 0.64065, 0.64799,\
        0.65536, 0.66266, 0.66975, 0.67629, 0.68160, 0.68518, 0.68825, 0.69082, 0.69472, 0.69970, 0.70392,\
        0.71208, 0.71883, 0.72211, 0.71440, 0.69140, 0.65693, 0.61123, 0.57178, 0.54885, 0.53593, 0.52441,\
        0.51599, 0.50997, 0.51047, 0.53439, 0.59118, 0.67621, 0.74902, 0.80642, 0.87725, 0.93633, 0.97172,\
        0.99622, 1.01155, 1.01803, 1.01666, 1.00891, 0.99636, 0.98053, 0.96284, 0.94457, 0.92693, 0.91138,\
        0.89918, 0.88822, 0.87814, 0.86844, 0.85901, 0.84981, 0.84078, 0.83183, 0.82292, 0.81405, 0.80524,\
        0.79646, 0.78771, 0.77901, 0.77036, 0.76176, 0.75328, 0.74496, 0.73684, 0.72894, 0.72119, 0.71351,\
        0.70579, 0.69778, 0.68922, 0.68046, 0.67162, 0.66278, 0.65400, 0.64533, 0.63678, 0.62837, 0.62010,\
        0.61199, 0.60405, 0.59647, 0.58919, 0.58191, 0.57464, 0.56738, 0.56016, 0.55298, 0.54588, 0.53888,\
        0.53200, 0.52525, 0.51863, 0.51215, 0.50584, 0.49986, 0.49386, 0.48788, 0.48180, 0.47551, 0.46899],\
        [0.52123, 0.52561, 0.53023, 0.53513, 0.54034, 0.54585, 0.55164, 0.55766, 0.56383, 0.57005, 0.57620,\
        0.58218, 0.58794, 0.59356, 0.59938, 0.60573, 0.61284, 0.62045, 0.62832, 0.63636, 0.64454, 0.65282,\
        0.66118, 0.66953, 0.67780, 0.68581, 0.69286, 0.69658, 0.70034, 0.70403, 0.70843, 0.71531, 0.72270,\
        0.73221, 0.74170, 0.74800, 0.74682, 0.73577, 0.71546, 0.67620, 0.63054, 0.59431, 0.57007, 0.54692,\
        0.53678, 0.53453, 0.54324, 0.59615, 0.68513, 0.75017, 0.79188, 0.84585, 0.90492, 0.95444, 0.98635,\
        1.00592, 1.01657, 1.01956, 1.01588, 1.00665, 0.99313, 0.97664, 0.95844, 0.93974, 0.92160, 0.90488,\
        0.89295, 0.88320, 0.87385, 0.86485, 0.85607, 0.84742, 0.83883, 0.83022, 0.82154, 0.81281, 0.80405,\
        0.79526, 0.78644, 0.77762, 0.76882, 0.76008, 0.75148, 0.74309, 0.73497, 0.72715, 0.71956, 0.71202,\
        0.70424, 0.69624, 0.68786, 0.67913, 0.67029, 0.66148, 0.65280, 0.64428, 0.63594, 0.62778, 0.61980,\
        0.61202, 0.60449, 0.59708, 0.58946, 0.58187, 0.57434, 0.56685, 0.55941, 0.55205, 0.54477, 0.53762,\
        0.53062, 0.52379, 0.51716, 0.51071, 0.50436, 0.49818, 0.49237, 0.48642, 0.48035, 0.47413, 0.46772],\
        [0.52099, 0.52538, 0.53002, 0.53493, 0.54014, 0.54565, 0.55144, 0.55744, 0.56357, 0.56972, 0.57578,\
        0.58164, 0.58723, 0.59266, 0.59834, 0.60518, 0.61308, 0.62143, 0.63003, 0.63879, 0.64769, 0.65669,\
        0.66577, 0.67485, 0.68380, 0.69246, 0.70061, 0.70595, 0.71051, 0.71579, 0.72200, 0.72996, 0.73924,\
        0.74966, 0.75986, 0.76749, 0.76989, 0.76511, 0.75103, 0.72433, 0.69474, 0.66318, 0.62475, 0.59009,\
        0.57842, 0.59045, 0.61785, 0.69896, 0.75862, 0.79544, 0.83183, 0.87862, 0.92654, 0.96540, 0.99298,\
        1.00964, 1.01750, 1.01814, 1.01274, 1.00238, 0.98817, 0.97129, 0.95292, 0.93432, 0.91690, 0.90212,\
        0.88920, 0.87880, 0.86968, 0.86114, 0.85290, 0.84479, 0.83667, 0.82843, 0.82000, 0.81135, 0.80262,\
        0.79382, 0.78494, 0.77597, 0.76695, 0.75798, 0.74916, 0.74060, 0.73238, 0.72456, 0.71712, 0.70990,\
        0.70235, 0.69428, 0.68581, 0.67705, 0.66824, 0.65953, 0.65100, 0.64270, 0.63465, 0.62681, 0.61916,\
        0.61166, 0.60430, 0.59696, 0.58931, 0.58159, 0.57387, 0.56618, 0.55854, 0.55099, 0.54355, 0.53625,\
        0.52914, 0.52224, 0.51560, 0.50925, 0.50323, 0.49717, 0.49103, 0.48499, 0.47893, 0.47276, 0.46644],\
        [0.52043, 0.52479, 0.52939, 0.53429, 0.53949, 0.54501, 0.55081, 0.55685, 0.56304, 0.56928, 0.57544,\
        0.58140, 0.58710, 0.59262, 0.59838, 0.60553, 0.61381, 0.62255, 0.63155, 0.64076, 0.65011, 0.65959,\
        0.66913, 0.67865, 0.68798, 0.69679, 0.70505, 0.71234, 0.71883, 0.72565, 0.73338, 0.74235, 0.75239,\
        0.76297, 0.77294, 0.78065, 0.78437, 0.78271, 0.77470, 0.76117, 0.74578, 0.72847, 0.70113, 0.66478,\
        0.64280, 0.64791, 0.68998, 0.75653, 0.79855, 0.83517, 0.86813, 0.90139, 0.94091, 0.97287, 0.99565,\
        1.00932, 1.01500, 1.01399, 1.00741, 0.99633, 0.98178, 0.96483, 0.94658, 0.92824, 0.91124, 0.89691,\
        0.88501, 0.87467, 0.86567, 0.85746, 0.84966, 0.84204, 0.83441, 0.82660, 0.81846, 0.80990, 0.80102,\
        0.79227, 0.78331, 0.77411, 0.76477, 0.75546, 0.74631, 0.73746, 0.72903, 0.72107, 0.71362, 0.70671,\
        0.70010, 0.69171, 0.68293, 0.67412, 0.66539, 0.65684, 0.64853, 0.64052, 0.63280, 0.62535, 0.61806,\
        0.61083, 0.60351, 0.59615, 0.58868, 0.58095, 0.57312, 0.56528, 0.55749, 0.54978, 0.54219, 0.53476,\
        0.52753, 0.52054, 0.51385, 0.50748, 0.50145, 0.49554, 0.48950, 0.48350, 0.47749, 0.47140, 0.46518],\
        [0.51955, 0.52383, 0.52837, 0.53322, 0.53840, 0.54392, 0.54977, 0.55590, 0.56224, 0.56870, 0.57516,\
        0.58149, 0.58762, 0.59360, 0.59986, 0.60701, 0.61511, 0.62380, 0.63289, 0.64226, 0.65184, 0.66155,\
        0.67134, 0.68111, 0.69076, 0.70004, 0.70870, 0.71695, 0.72494, 0.73317, 0.74204, 0.75171, 0.76197,\
        0.77223, 0.78150, 0.78856, 0.79237, 0.79245, 0.78904, 0.78347, 0.77727, 0.76886, 0.75321, 0.73162,\
        0.71039, 0.69856, 0.73620, 0.79849, 0.84248, 0.87738, 0.90556, 0.92877, 0.95524, 0.97909, 0.99643,\
        1.00662, 1.01010, 1.00768, 1.00028, 0.98883, 0.97427, 0.95758, 0.93974, 0.92181, 0.90517, 0.89141,\
        0.88017, 0.87039, 0.86170, 0.85379, 0.84637, 0.83919, 0.83206, 0.82475, 0.81704, 0.80878, 0.80001,\
        0.79101, 0.78170, 0.77207, 0.76229, 0.75252, 0.74294, 0.73371, 0.72496, 0.71673, 0.70900, 0.70183,\
        0.69497, 0.68727, 0.67883, 0.67022, 0.66169, 0.65336, 0.64533, 0.63765, 0.63032, 0.62330, 0.61647,\
        0.60963, 0.60270, 0.59541, 0.58778, 0.57998, 0.57206, 0.56412, 0.55621, 0.54838, 0.54068, 0.53314,\
        0.52580, 0.51872, 0.51195, 0.50548, 0.49935, 0.49355, 0.48776, 0.48192, 0.47603, 0.47005, 0.46393],\
        [0.51838, 0.52254, 0.52697, 0.53173, 0.53686, 0.54237, 0.54828, 0.55454, 0.56110, 0.56789, 0.57479,\
        0.58168, 0.58845, 0.59508, 0.60165, 0.60873, 0.61651, 0.62495, 0.63391, 0.64325, 0.65287, 0.66265,\
        0.67251, 0.68235, 0.69211, 0.70170, 0.71088, 0.71996, 0.72903, 0.73831, 0.74799, 0.75808, 0.76832,\
        0.77813, 0.78664, 0.79285, 0.79619, 0.79702, 0.79630, 0.79517, 0.79481, 0.79350, 0.78681, 0.77910,\
        0.76410, 0.75404, 0.79380, 0.85414, 0.89248, 0.91830, 0.93734, 0.95330, 0.96943, 0.98449, 0.99587,\
        1.00226, 1.00344, 0.99974, 0.99175, 0.98020, 0.96591, 0.94979, 0.93279, 0.91580, 0.89983, 0.88669,\
        0.87572, 0.86619, 0.85772, 0.85005, 0.84293, 0.83615, 0.82950, 0.82275, 0.81561, 0.80780, 0.79918,\
        0.78983, 0.77997, 0.76979, 0.75946, 0.74916, 0.73908, 0.72941, 0.72031, 0.71184, 0.70375, 0.69632,\
        0.68923, 0.68173, 0.67372, 0.66546, 0.65721, 0.64915, 0.64140, 0.63405, 0.62714, 0.62060, 0.61430,\
        0.60791, 0.60113, 0.59399, 0.58643, 0.57860, 0.57063, 0.56263, 0.55467, 0.54678, 0.53902, 0.53141,\
        0.52399, 0.51683, 0.51001, 0.50358, 0.49749, 0.49171, 0.48604, 0.48036, 0.47461, 0.46875, 0.46274],\
        [0.51693, 0.52092, 0.52521, 0.52984, 0.53487, 0.54035, 0.54629, 0.55269, 0.55950, 0.56665, 0.57406,\
        0.58159, 0.58904, 0.59624, 0.60310, 0.60999, 0.61748, 0.62564, 0.63440, 0.64363, 0.65320, 0.66297,\
        0.67280, 0.68257, 0.69218, 0.70177, 0.71161, 0.72144, 0.73129, 0.74128, 0.75148, 0.76182, 0.77200,\
        0.78150, 0.78958, 0.79535, 0.79825, 0.79958, 0.80105, 0.80329, 0.80711, 0.81105, 0.81304, 0.81715,\
        0.81613, 0.81924, 0.86451, 0.91251, 0.93562, 0.95010, 0.96071, 0.96984, 0.97878, 0.98707, 0.99327,\
        0.99618, 0.99527, 0.99050, 0.98215, 0.97072, 0.95689, 0.94153, 0.92565, 0.91020, 0.89565, 0.88261,\
        0.87157, 0.86205, 0.85367, 0.84612, 0.83920, 0.83271, 0.82648, 0.82029, 0.81382, 0.80650, 0.79789,\
        0.78822, 0.77783, 0.76708, 0.75621, 0.74540, 0.73481, 0.72466, 0.71517, 0.70662, 0.69871, 0.69086,\
        0.68344, 0.67591, 0.66811, 0.66011, 0.65211, 0.64429, 0.63680, 0.62975, 0.62321, 0.61715, 0.61144,\
        0.60572, 0.59931, 0.59216, 0.58458, 0.57674, 0.56879, 0.56081, 0.55286, 0.54499, 0.53723, 0.52961,\
        0.52217, 0.51495, 0.50807, 0.50171, 0.49574, 0.49004, 0.48448, 0.47892, 0.47329, 0.46754, 0.46163],\
        [0.51524, 0.51902, 0.52309, 0.52754, 0.53243, 0.53783, 0.54377, 0.55026, 0.55727, 0.56477, 0.57265,\
        0.58080, 0.58896, 0.59656, 0.60342, 0.61032, 0.61760, 0.62557, 0.63418, 0.64332, 0.65284, 0.66259,\
        0.67242, 0.68218, 0.69175, 0.70134, 0.71149, 0.72169, 0.73195, 0.74235, 0.75287, 0.76337, 0.77353,\
        0.78294, 0.79104, 0.79725, 0.80077, 0.80319, 0.80666, 0.81149, 0.81796, 0.82558, 0.83428, 0.84527,\
        0.85931, 0.88401, 0.92047, 0.94651, 0.95985, 0.96738, 0.97262, 0.97708, 0.98136, 0.98519, 0.98770,\
        0.98802, 0.98556, 0.98011, 0.97170, 0.96063, 0.94742, 0.93281, 0.91785, 0.90375, 0.89037, 0.87804,\
        0.86721, 0.85773, 0.84936, 0.84185, 0.83502, 0.82868, 0.82269, 0.81691, 0.81111, 0.80454, 0.79594,\
        0.78578, 0.77491, 0.76374, 0.75247, 0.74126, 0.73027, 0.71968, 0.70972, 0.70074, 0.69297, 0.68528,\
        0.67770, 0.67009, 0.66233, 0.65447, 0.64663, 0.63897, 0.63166, 0.62482, 0.61855, 0.61288, 0.60766,\
        0.60262, 0.59676, 0.58961, 0.58207, 0.57433, 0.56649, 0.55862, 0.55078, 0.54300, 0.53533, 0.52779,\
        0.52042, 0.51323, 0.50627, 0.49973, 0.49408, 0.48858, 0.48313, 0.47766, 0.47212, 0.46645, 0.46062],\
        [0.51336, 0.51687, 0.52067, 0.52487, 0.52955, 0.53480, 0.54067, 0.54718, 0.55433, 0.56205, 0.57027,\
        0.57886, 0.58762, 0.59624, 0.60299, 0.60937, 0.61658, 0.62454, 0.63315, 0.64227, 0.65178, 0.66153,\
        0.67143, 0.68137, 0.69125, 0.70099, 0.71087, 0.72087, 0.73120, 0.74179, 0.75251, 0.76311, 0.77331,\
        0.78275, 0.79106, 0.79785, 0.80272, 0.80674, 0.81162, 0.81798, 0.82603, 0.83570, 0.84722, 0.86143,\
        0.88010, 0.90497, 0.93059, 0.95048, 0.96228, 0.96875, 0.97247, 0.97496, 0.97692, 0.97834, 0.97874,\
        0.97755, 0.97428, 0.96867, 0.96061, 0.95021, 0.93777, 0.92392, 0.90952, 0.89611, 0.88409, 0.87272,\
        0.86233, 0.85300, 0.84464, 0.83711, 0.83025, 0.82390, 0.81790, 0.81215, 0.80659, 0.80072, 0.79217,\
        0.78185, 0.77087, 0.75958, 0.74820, 0.73684, 0.72564, 0.71479, 0.70464, 0.69551, 0.68746, 0.67977,\
        0.67211, 0.66441, 0.65662, 0.64878, 0.64098, 0.63338, 0.62615, 0.61944, 0.61332, 0.60784, 0.60316,\
        0.59803, 0.59256, 0.58597, 0.57879, 0.57133, 0.56373, 0.55608, 0.54843, 0.54084, 0.53335, 0.52598,\
        0.51878, 0.51181, 0.50512, 0.49882, 0.49298, 0.48745, 0.48203, 0.47661, 0.47112, 0.46551, 0.45974],\
        [0.51134, 0.51452, 0.51799, 0.52186, 0.52625, 0.53127, 0.53699, 0.54344, 0.55062, 0.55844, 0.56680,\
        0.57551, 0.58427, 0.59251, 0.59966, 0.60677, 0.61437, 0.62255, 0.63130, 0.64051, 0.65003, 0.65979,\
        0.66971, 0.67977, 0.69001, 0.70020, 0.70912, 0.71878, 0.72912, 0.73986, 0.75072, 0.76143, 0.77169,\
        0.78121, 0.78971, 0.79696, 0.80300, 0.80854, 0.81459, 0.82184, 0.83054, 0.84072, 0.85237, 0.86563,\
        0.88055, 0.89732, 0.91824, 0.93665, 0.94987, 0.95801, 0.96251, 0.96496, 0.96636, 0.96696, 0.96657,\
        0.96486, 0.96152, 0.95632, 0.94907, 0.93968, 0.92828, 0.91543, 0.90209, 0.88934, 0.87781, 0.86702,\
        0.85698, 0.84777, 0.83941, 0.83181, 0.82486, 0.81837, 0.81218, 0.80609, 0.80004, 0.79390, 0.78591,\
        0.77619, 0.76559, 0.75457, 0.74341, 0.73223, 0.72111, 0.71022, 0.70003, 0.69098, 0.68267, 0.67473,\
        0.66689, 0.65903, 0.65114, 0.64322, 0.63538, 0.62775, 0.62049, 0.61386, 0.60795, 0.60235, 0.59794,\
        0.59327, 0.58776, 0.58159, 0.57488, 0.56781, 0.56056, 0.55322, 0.54586, 0.53853, 0.53129, 0.52416,\
        0.51722, 0.51052, 0.50414, 0.49807, 0.49219, 0.48660, 0.48116, 0.47575, 0.47029, 0.46472, 0.45898],\
        [0.50925, 0.51206, 0.51513, 0.51859, 0.52260, 0.52729, 0.53278, 0.53909, 0.54621, 0.55404, 0.56239,\
        0.57105, 0.57966, 0.58784, 0.59551, 0.60320, 0.61124, 0.61979, 0.62880, 0.63814, 0.64769, 0.65739,\
        0.66719, 0.67703, 0.68683, 0.69616, 0.70540, 0.71534, 0.72589, 0.73682, 0.74784, 0.75865, 0.76899,\
        0.77862, 0.78735, 0.79506, 0.80194, 0.80848, 0.81526, 0.82292, 0.83170, 0.84150, 0.85209, 0.86323,\
        0.87472, 0.88708, 0.90121, 0.91703, 0.93137, 0.94106, 0.94653, 0.94956, 0.95135, 0.95218, 0.95190,\
        0.95037, 0.94751, 0.94319, 0.93719, 0.92923, 0.91911, 0.90729, 0.89498, 0.88301, 0.87171, 0.86113,\
        0.85123, 0.84206, 0.83364, 0.82594, 0.81886, 0.81221, 0.80580, 0.79944, 0.79300, 0.78628, 0.77845,\
        0.76932, 0.75930, 0.74882, 0.73817, 0.72751, 0.71686, 0.70625, 0.69621, 0.68707, 0.67850, 0.67023,\
        0.66212, 0.65405, 0.64599, 0.63796, 0.63004, 0.62231, 0.61494, 0.60821, 0.60268, 0.59767, 0.59272,\
        0.58799, 0.58272, 0.57689, 0.57059, 0.56394, 0.55708, 0.55011, 0.54309, 0.53609, 0.52916, 0.52232,\
        0.51565, 0.50919, 0.50302, 0.49710, 0.49139, 0.48587, 0.48045, 0.47507, 0.46962, 0.46407, 0.45836],\
        [0.50718, 0.50957, 0.51217, 0.51514, 0.51867, 0.52295, 0.52812, 0.53424, 0.54126, 0.54904, 0.55738,\
        0.56600, 0.57461, 0.58298, 0.59108, 0.59918, 0.60759, 0.61658, 0.62590, 0.63537, 0.64492, 0.65449,\
        0.66404, 0.67351, 0.68277, 0.69178, 0.70104, 0.71111, 0.72191, 0.73307, 0.74422, 0.75510, 0.76550,\
        0.77523, 0.78419, 0.79236, 0.79983, 0.80680, 0.81380, 0.82162, 0.83026, 0.83944, 0.84887, 0.85831,\
        0.86768, 0.87728, 0.88772, 0.90051, 0.91389, 0.92322, 0.92818, 0.93142, 0.93391, 0.93550, 0.93574,\
        0.93470, 0.93261, 0.92944, 0.92499, 0.91887, 0.91029, 0.89941, 0.88790, 0.87644, 0.86540, 0.85493,\
        0.84507, 0.83585, 0.82731, 0.81950, 0.81232, 0.80554, 0.79890, 0.79232, 0.78560, 0.77849, 0.77063,\
        0.76187, 0.75238, 0.74247, 0.73247, 0.72260, 0.71284, 0.70289, 0.69305, 0.68369, 0.67476, 0.66616,\
        0.65775, 0.64946, 0.64124, 0.63310, 0.62510, 0.61733, 0.60990, 0.60298, 0.59725, 0.59255, 0.58774,\
        0.58286, 0.57768, 0.57210, 0.56614, 0.55988, 0.55342, 0.54684, 0.54020, 0.53357, 0.52698, 0.52047,\
        0.51408, 0.50785, 0.50185, 0.49614, 0.49062, 0.48522, 0.47987, 0.47452, 0.46909, 0.46355, 0.45784],\
        [0.50523, 0.50717, 0.50925, 0.51164, 0.51459, 0.51836, 0.52315, 0.52903, 0.53596, 0.54372, 0.55209,\
        0.56079, 0.56955, 0.57820, 0.58667, 0.59507, 0.60388, 0.61336, 0.62291, 0.63243, 0.64191, 0.65131,\
        0.66062, 0.66976, 0.67872, 0.68757, 0.69669, 0.70673, 0.71771, 0.72901, 0.74021, 0.75107, 0.76145,\
        0.77122, 0.78031, 0.78874, 0.79661, 0.80346, 0.81049, 0.81860, 0.82715, 0.83579, 0.84432, 0.85266,\
        0.86087, 0.86933, 0.87869, 0.88955, 0.90092, 0.90762, 0.90993, 0.91291, 0.91605, 0.91862, 0.91926,\
        0.91858, 0.91730, 0.91527, 0.91226, 0.90817, 0.90158, 0.89142, 0.88039, 0.86930, 0.85852, 0.84821,\
        0.83842, 0.82915, 0.82045, 0.81248, 0.80527, 0.79841, 0.79164, 0.78486, 0.77792, 0.77061, 0.76272,\
        0.75418, 0.74510, 0.73569, 0.72626, 0.71715, 0.70855, 0.69981, 0.68995, 0.68034, 0.67115, 0.66231,\
        0.65370, 0.64524, 0.63690, 0.62868, 0.62065, 0.61292, 0.60559, 0.59889, 0.59297, 0.58786, 0.58294,\
        0.57797, 0.57281, 0.56738, 0.56168, 0.55575, 0.54967, 0.54348, 0.53724, 0.53100, 0.52479, 0.51864,\
        0.51257, 0.50661, 0.50079, 0.49530, 0.48997, 0.48469, 0.47941, 0.47409, 0.46868, 0.46314, 0.45743],\
        [0.50350, 0.50499, 0.50651, 0.50826, 0.51051, 0.51365, 0.51800, 0.52366, 0.53052, 0.53834, 0.54681,\
        0.55568, 0.56470, 0.57373, 0.58269, 0.59170, 0.60102, 0.61051, 0.62004, 0.62948, 0.63882, 0.64805,\
        0.65715, 0.66612, 0.67497, 0.68379, 0.69287, 0.70281, 0.71382, 0.72501, 0.73606, 0.74677, 0.75702,\
        0.76673, 0.77582, 0.78423, 0.79183, 0.79846, 0.80625, 0.81481, 0.82322, 0.83135, 0.83920, 0.84683,\
        0.85441, 0.86227, 0.87088, 0.88039, 0.88969, 0.89354, 0.89421, 0.89648, 0.89965, 0.90317, 0.90346,\
        0.90284, 0.90232, 0.90113, 0.89901, 0.89597, 0.89068, 0.88192, 0.87174, 0.86123, 0.85085, 0.84085,\
        0.83126, 0.82205, 0.81316, 0.80495, 0.79773, 0.79087, 0.78406, 0.77719, 0.77013, 0.76272, 0.75485,\
        0.74649, 0.73771, 0.72868, 0.71961, 0.71082, 0.70288, 0.69486, 0.68579, 0.67649, 0.66738, 0.65851,\
        0.64984, 0.64132, 0.63294, 0.62469, 0.61666, 0.60906, 0.60187, 0.59524, 0.58922, 0.58373, 0.57852,\
        0.57338, 0.56816, 0.56281, 0.55730, 0.55164, 0.54589, 0.54009, 0.53425, 0.52842, 0.52261, 0.51685,\
        0.51115, 0.50556, 0.50008, 0.49471, 0.48947, 0.48427, 0.47905, 0.47376, 0.46837, 0.46282, 0.45710],\
        [0.50208, 0.50316, 0.50415, 0.50520, 0.50664, 0.50902, 0.51287, 0.51835, 0.52522, 0.53314, 0.54177,\
        0.55086, 0.56019, 0.56963, 0.57912, 0.58868, 0.59831, 0.60784, 0.61727, 0.62655, 0.63571, 0.64477,\
        0.65375, 0.66266, 0.67154, 0.68052, 0.68984, 0.69983, 0.71040, 0.72118, 0.73187, 0.74228, 0.75232,\
        0.76189, 0.77095, 0.77941, 0.78723, 0.79476, 0.80283, 0.81107, 0.81902, 0.82655, 0.83377, 0.84084,\
        0.84788, 0.85511, 0.86266, 0.87028, 0.87669, 0.87998, 0.88127, 0.88303, 0.88574, 0.88834, 0.88895,\
        0.88873, 0.88843, 0.88763, 0.88594, 0.88305, 0.87815, 0.87078, 0.86177, 0.85206, 0.84226, 0.83272,\
        0.82355, 0.81469, 0.80587, 0.79734, 0.79005, 0.78315, 0.77634, 0.76945, 0.76236, 0.75495, 0.74714,\
        0.73895, 0.73045, 0.72175, 0.71299, 0.70431, 0.69629, 0.68890, 0.68073, 0.67209, 0.66335, 0.65467,\
        0.64610, 0.63764, 0.62931, 0.62111, 0.61307, 0.60542, 0.59874, 0.59200, 0.58575, 0.57994, 0.57443,\
        0.56906, 0.56374, 0.55840, 0.55301, 0.54759, 0.54214, 0.53670, 0.53127, 0.52586, 0.52047, 0.51510,\
        0.50978, 0.50453, 0.49935, 0.49419, 0.48907, 0.48395, 0.47879, 0.47352, 0.46813, 0.46257, 0.45683],\
        [0.50106, 0.50181, 0.50234, 0.50273, 0.50327, 0.50468, 0.50800, 0.51340, 0.52035, 0.52838, 0.53717,\
        0.54646, 0.55607, 0.56587, 0.57577, 0.58571, 0.59555, 0.60518, 0.61453, 0.62360, 0.63256, 0.64148,\
        0.65040, 0.65931, 0.66829, 0.67745, 0.68693, 0.69685, 0.70708, 0.71738, 0.72759, 0.73760, 0.74733,\
        0.75676, 0.76583, 0.77452, 0.78287, 0.79110, 0.79932, 0.80728, 0.81461, 0.82138, 0.82798, 0.83453,\
        0.84104, 0.84749, 0.85384, 0.85972, 0.86439, 0.86730, 0.86901, 0.87070, 0.87275, 0.87459, 0.87552,\
        0.87572, 0.87552, 0.87476, 0.87311, 0.87018, 0.86552, 0.85897, 0.85092, 0.84199, 0.83278, 0.82377,\
        0.81513, 0.80678, 0.79866, 0.79015, 0.78249, 0.77544, 0.76862, 0.76178, 0.75476, 0.74742, 0.73971,\
        0.73169, 0.72349, 0.71518, 0.70683, 0.69868, 0.69086, 0.68338, 0.67564, 0.66753, 0.65920, 0.65081,\
        0.64244, 0.63415, 0.62597, 0.61794, 0.61007, 0.60230, 0.59529, 0.58880, 0.58242, 0.57634, 0.57054,\
        0.56495, 0.55950, 0.55413, 0.54883, 0.54360, 0.53843, 0.53333, 0.52830, 0.52332, 0.51836, 0.51341,\
        0.50845, 0.50348, 0.49856, 0.49366, 0.48872, 0.48370, 0.47859, 0.47334, 0.46794, 0.46237, 0.45660],\
        [0.50043, 0.50098, 0.50120, 0.50112, 0.50083, 0.50101, 0.50387, 0.50929, 0.51625, 0.52430, 0.53314,\
        0.54254, 0.55233, 0.56237, 0.57256, 0.58279, 0.59292, 0.60266, 0.61171, 0.62047, 0.62926, 0.63811,\
        0.64700, 0.65595, 0.66501, 0.67426, 0.68379, 0.69360, 0.70355, 0.71334, 0.72306, 0.73261, 0.74199,\
        0.75125, 0.76035, 0.76929, 0.77808, 0.78677, 0.79526, 0.80317, 0.80973, 0.81559, 0.82169, 0.82785,\
        0.83381, 0.83948, 0.84470, 0.84924, 0.85279, 0.85526, 0.85702, 0.85864, 0.86033, 0.86185, 0.86286,\
        0.86329, 0.86316, 0.86235, 0.86061, 0.85762, 0.85312, 0.84706, 0.83969, 0.83138, 0.82260, 0.81402,\
        0.80597, 0.79791, 0.79000, 0.78211, 0.77460, 0.76761, 0.76091, 0.75426, 0.74742, 0.74025, 0.73269,\
        0.72481, 0.71688, 0.70911, 0.70118, 0.69344, 0.68589, 0.67843, 0.67087, 0.66307, 0.65508, 0.64698,\
        0.63886, 0.63077, 0.62278, 0.61499, 0.60750, 0.60028, 0.59286, 0.58584, 0.57919, 0.57284, 0.56678,\
        0.56098, 0.55538, 0.54997, 0.54472, 0.53964, 0.53473, 0.52998, 0.52535, 0.52082, 0.51631, 0.51179,\
        0.50721, 0.50256, 0.49789, 0.49322, 0.48843, 0.48350, 0.47843, 0.47319, 0.46778, 0.46218, 0.45637],\
        [0.50015, 0.50059, 0.50066, 0.50034, 0.49958, 0.49916, 0.50151, 0.50652, 0.51317, 0.52103, 0.52977,\
        0.53916, 0.54899, 0.55910, 0.56939, 0.57972, 0.58999, 0.59978, 0.60828, 0.61696, 0.62575, 0.63460,\
        0.64348, 0.65244, 0.66152, 0.67077, 0.68021, 0.68982, 0.69935, 0.70886, 0.71813, 0.72715, 0.73615,\
        0.74522, 0.75434, 0.76345, 0.77251, 0.78146, 0.79016, 0.79811, 0.80375, 0.80885, 0.81494, 0.82086,\
        0.82629, 0.83121, 0.83547, 0.83893, 0.84158, 0.84355, 0.84514, 0.84666, 0.84822, 0.84965, 0.85072,\
        0.85126, 0.85117, 0.85031, 0.84848, 0.84545, 0.84104, 0.83527, 0.82842, 0.82070, 0.81209, 0.80362,\
        0.79603, 0.78835, 0.78070, 0.77326, 0.76618, 0.75953, 0.75318, 0.74690, 0.74043, 0.73357, 0.72622,\
        0.71844, 0.71056, 0.70337, 0.69624, 0.68864, 0.68121, 0.67384, 0.66639, 0.65881, 0.65107, 0.64323,\
        0.63533, 0.62744, 0.61962, 0.61193, 0.60453, 0.59740, 0.59004, 0.58284, 0.57594, 0.56936, 0.56307,\
        0.55707, 0.55133, 0.54586, 0.54065, 0.53571, 0.53104, 0.52662, 0.52241, 0.51834, 0.51432, 0.51027,\
        0.50611, 0.50182, 0.49741, 0.49289, 0.48820, 0.48334, 0.47828, 0.47304, 0.46761, 0.46197, 0.45613],\
        [0.50008, 0.50047, 0.50045, 0.49997, 0.49908, 0.49893, 0.50082, 0.50499, 0.51107, 0.51855, 0.52706,\
        0.53630, 0.54603, 0.55607, 0.56624, 0.57639, 0.58624, 0.59537, 0.60421, 0.61323, 0.62211, 0.63096,\
        0.63982, 0.64874, 0.65774, 0.66687, 0.67614, 0.68550, 0.69483, 0.70407, 0.71271, 0.72104, 0.72963,\
        0.73854, 0.74766, 0.75684, 0.76598, 0.77494, 0.78344, 0.79088, 0.79624, 0.80188, 0.80816, 0.81373,\
        0.81849, 0.82277, 0.82621, 0.82870, 0.83051, 0.83192, 0.83322, 0.83466, 0.83624, 0.83775, 0.83892,\
        0.83954, 0.83946, 0.83856, 0.83670, 0.83367, 0.82931, 0.82366, 0.81714, 0.81022, 0.80243, 0.79351,\
        0.78579, 0.77827, 0.77091, 0.76386, 0.75727, 0.75116, 0.74540, 0.73973, 0.73385, 0.72749, 0.72050,\
        0.71291, 0.70495, 0.69751, 0.69102, 0.68397, 0.67675, 0.66947, 0.66213, 0.65470, 0.64716, 0.63954,\
        0.63185, 0.62414, 0.61645, 0.60882, 0.60115, 0.59400, 0.58681, 0.57961, 0.57259, 0.56583, 0.55935,\
        0.55318, 0.54731, 0.54177, 0.53657, 0.53175, 0.52730, 0.52322, 0.51944, 0.51587, 0.51238, 0.50883,\
        0.50511, 0.50117, 0.49700, 0.49262, 0.48800, 0.48317, 0.47811, 0.47286, 0.46739, 0.46172, 0.45583],\
        [0.50011, 0.50044, 0.50031, 0.49965, 0.49881, 0.49874, 0.50035, 0.50404, 0.50964, 0.51674, 0.52495,\
        0.53395, 0.54348, 0.55332, 0.56327, 0.57313, 0.58269, 0.59185, 0.60086, 0.60966, 0.61846, 0.62725,\
        0.63603, 0.64482, 0.65364, 0.66255, 0.67154, 0.68062, 0.68974, 0.69871, 0.70634, 0.71400, 0.72236,\
        0.73121, 0.74031, 0.74948, 0.75857, 0.76737, 0.77560, 0.78288, 0.78915, 0.79532, 0.80176, 0.80626,\
        0.81020, 0.81422, 0.81694, 0.81835, 0.81934, 0.82019, 0.82114, 0.82254, 0.82430, 0.82600, 0.82732,\
        0.82805, 0.82797, 0.82702, 0.82518, 0.82225, 0.81799, 0.81227, 0.80573, 0.79917, 0.79228, 0.78382,\
        0.77564, 0.76797, 0.76074, 0.75405, 0.74797, 0.74252, 0.73754, 0.73272, 0.72767, 0.72204, 0.71562,\
        0.70846, 0.70085, 0.69346, 0.68653, 0.67957, 0.67245, 0.66525, 0.65799, 0.65067, 0.64330, 0.63587,\
        0.62837, 0.62082, 0.61326, 0.60581, 0.59845, 0.59088, 0.58353, 0.57626, 0.56914, 0.56224, 0.55561,\
        0.54928, 0.54329, 0.53767, 0.53247, 0.52773, 0.52349, 0.51974, 0.51641, 0.51338, 0.51046, 0.50745,\
        0.50418, 0.50058, 0.49662, 0.49234, 0.48776, 0.48293, 0.47787, 0.47259, 0.46709, 0.46138, 0.45545],\
        [0.50015, 0.50038, 0.50013, 0.49934, 0.49852, 0.49847, 0.49992, 0.50334, 0.50864, 0.51543, 0.52334,\
        0.53208, 0.54135, 0.55094, 0.56063, 0.57023, 0.57959, 0.58866, 0.59746, 0.60615, 0.61489, 0.62356,\
        0.63216, 0.64070, 0.64924, 0.65779, 0.66638, 0.67496, 0.68345, 0.69146, 0.69866, 0.70608, 0.71454,\
        0.72341, 0.73243, 0.74152, 0.75048, 0.75910, 0.76718, 0.77456, 0.78137, 0.78793, 0.79395, 0.79755,\
        0.80137, 0.80561, 0.80757, 0.80753, 0.80796, 0.80832, 0.80880, 0.81026, 0.81240, 0.81432, 0.81580,\
        0.81675, 0.81660, 0.81555, 0.81380, 0.81106, 0.80703, 0.80139, 0.79462, 0.78801, 0.78096, 0.77313,\
        0.76510, 0.75741, 0.75027, 0.74386, 0.73830, 0.73360, 0.72956, 0.72582, 0.72185, 0.71714, 0.71139,\
        0.70472, 0.69744, 0.68996, 0.68266, 0.67547, 0.66827, 0.66106, 0.65384, 0.64663, 0.63941, 0.63217,\
        0.62487, 0.61748, 0.60996, 0.60243, 0.59507, 0.58763, 0.58018, 0.57282, 0.56561, 0.55860, 0.55184,\
        0.54539, 0.53929, 0.53358, 0.52835, 0.52365, 0.51958, 0.51614, 0.51328, 0.51081, 0.50851, 0.50608,\
        0.50327, 0.49996, 0.49617, 0.49197, 0.48741, 0.48257, 0.47749, 0.47218, 0.46665, 0.46091, 0.45495],\
        [0.50013, 0.50025, 0.49984, 0.49899, 0.49824, 0.49820, 0.49954, 0.50286, 0.50799, 0.51454, 0.52219,\
        0.53064, 0.53964, 0.54896, 0.55840, 0.56776, 0.57693, 0.58582, 0.59445, 0.60303, 0.61156, 0.61996,\
        0.62825, 0.63645, 0.64458, 0.65267, 0.66072, 0.66870, 0.67648, 0.68387, 0.69090, 0.69829, 0.70679,\
        0.71539, 0.72418, 0.73313, 0.74192, 0.75033, 0.75824, 0.76561, 0.77247, 0.77878, 0.78415, 0.78825,\
        0.79218, 0.79592, 0.79732, 0.79621, 0.79626, 0.79626, 0.79647, 0.79800, 0.80054, 0.80255, 0.80414,\
        0.80568, 0.80526, 0.80400, 0.80241, 0.79981, 0.79585, 0.79047, 0.78391, 0.77704, 0.76980, 0.76211,\
        0.75424, 0.74660, 0.73955, 0.73337, 0.72828, 0.72436, 0.72140, 0.71890, 0.71623, 0.71263, 0.70749,\
        0.70101, 0.69386, 0.68633, 0.67879, 0.67136, 0.66402, 0.65676, 0.64957, 0.64244, 0.63536, 0.62833,\
        0.62129, 0.61413, 0.60670, 0.59915, 0.59169, 0.58423, 0.57674, 0.56931, 0.56201, 0.55492, 0.54807,\
        0.54153, 0.53532, 0.52953, 0.52423, 0.51953, 0.51556, 0.51240, 0.51000, 0.50813, 0.50647, 0.50467,\
        0.50235, 0.49927, 0.49559, 0.49140, 0.48685, 0.48200, 0.47691, 0.47159, 0.46604, 0.46028, 0.45430],\
        [0.50007, 0.50003, 0.49950, 0.49867, 0.49803, 0.49811, 0.49951, 0.50275, 0.50771, 0.51404, 0.52142,\
        0.52959, 0.53832, 0.54737, 0.55656, 0.56571, 0.57470, 0.58345, 0.59197, 0.60031, 0.60844, 0.61647,\
        0.62437, 0.63211, 0.63973, 0.64727, 0.65474, 0.66213, 0.66938, 0.67646, 0.68353, 0.69106, 0.69916,\
        0.70708, 0.71569, 0.72457, 0.73310, 0.74117, 0.74881, 0.75599, 0.76265, 0.76861, 0.77368, 0.77784,\
        0.78137, 0.78404, 0.78497, 0.78444, 0.78397, 0.78384, 0.78432, 0.78586, 0.78823, 0.79046, 0.79244,\
        0.79419, 0.79388, 0.79263, 0.79097, 0.78833, 0.78442, 0.77923, 0.77301, 0.76616, 0.75886, 0.75120,\
        0.74338, 0.73576, 0.72876, 0.72274, 0.71802, 0.71484, 0.71300, 0.71177, 0.71056, 0.70832, 0.70377,\
        0.69719, 0.68987, 0.68219, 0.67448, 0.66689, 0.65947, 0.65219, 0.64503, 0.63798, 0.63103, 0.62420,\
        0.61748, 0.61073, 0.60355, 0.59600, 0.58840, 0.58078, 0.57319, 0.56570, 0.55836, 0.55122, 0.54433,\
        0.53773, 0.53146, 0.52559, 0.52020, 0.51544, 0.51149, 0.50854, 0.50657, 0.50529, 0.50425, 0.50304,\
        0.50127, 0.49843, 0.49470, 0.49050, 0.48597, 0.48114, 0.47607, 0.47076, 0.46523, 0.45946, 0.45348],\
        [0.50004, 0.49985, 0.49922, 0.49843, 0.49792, 0.49819, 0.49974, 0.50295, 0.50774, 0.51385, 0.52097,\
        0.52887, 0.53731, 0.54610, 0.55506, 0.56402, 0.57284, 0.58144, 0.58974, 0.59772, 0.60550, 0.61313,\
        0.62055, 0.62774, 0.63475, 0.64166, 0.64853, 0.65539, 0.66222, 0.66907, 0.67608, 0.68341, 0.69095,\
        0.69857, 0.70735, 0.71611, 0.72415, 0.73169, 0.73894, 0.74584, 0.75223, 0.75785, 0.76258, 0.76639,\
        0.76932, 0.77121, 0.77186, 0.77156, 0.77109, 0.77101, 0.77168, 0.77329, 0.77563, 0.77814, 0.78042,\
        0.78202, 0.78226, 0.78137, 0.77964, 0.77692, 0.77305, 0.76804, 0.76208, 0.75540, 0.74820, 0.74061,\
        0.73281, 0.72518, 0.71818, 0.71227, 0.70782, 0.70518, 0.70447, 0.70419, 0.70424, 0.70362, 0.69948,\
        0.69267, 0.68502, 0.67717, 0.66939, 0.66179, 0.65440, 0.64719, 0.64012, 0.63316, 0.62632, 0.61962,\
        0.61316, 0.60694, 0.60033, 0.59269, 0.58488, 0.57712, 0.56947, 0.56196, 0.55462, 0.54750, 0.54063,\
        0.53404, 0.52778, 0.52188, 0.51642, 0.51155, 0.50750, 0.50466, 0.50308, 0.50232, 0.50176, 0.50090,\
        0.49935, 0.49672, 0.49319, 0.48913, 0.48469, 0.47994, 0.47493, 0.46967, 0.46417, 0.45843, 0.45246],\
        [0.49988, 0.49964, 0.49894, 0.49822, 0.49787, 0.49831, 0.50000, 0.50325, 0.50796, 0.51388, 0.52076,\
        0.52838, 0.53654, 0.54507, 0.55379, 0.56255, 0.57123, 0.57969, 0.58780, 0.59548, 0.60283, 0.60999,\
        0.61685, 0.62335, 0.62963, 0.63586, 0.64214, 0.64850, 0.65496, 0.66158, 0.66843, 0.67559, 0.68305,\
        0.69097, 0.69960, 0.70782, 0.71498, 0.72184, 0.72870, 0.73538, 0.74151, 0.74678, 0.75107, 0.75436,\
        0.75669, 0.75802, 0.75839, 0.75809, 0.75766, 0.75765, 0.75848, 0.76029, 0.76286, 0.76570, 0.76828,\
        0.77006, 0.77069, 0.77011, 0.76846, 0.76574, 0.76192, 0.75707, 0.75133, 0.74490, 0.73793, 0.73053,\
        0.72281, 0.71517, 0.70822, 0.70248, 0.69840, 0.69618, 0.69595, 0.69616, 0.69676, 0.69675, 0.69251,\
        0.68605, 0.67860, 0.67090, 0.66328, 0.65586, 0.64868, 0.64168, 0.63479, 0.62796, 0.62119, 0.61454,\
        0.60803, 0.60199, 0.59580, 0.58846, 0.58078, 0.57308, 0.56549, 0.55804, 0.55079, 0.54377, 0.53699,\
        0.53051, 0.52434, 0.51852, 0.51310, 0.50817, 0.50393, 0.50108, 0.49983, 0.49939, 0.49903, 0.49821,\
        0.49670, 0.49429, 0.49108, 0.48726, 0.48300, 0.47839, 0.47348, 0.46830, 0.46286, 0.45717, 0.45123],\
        [0.49899, 0.49897, 0.49849, 0.49799, 0.49785, 0.49847, 0.50028, 0.50365, 0.50831, 0.51406, 0.52070,\
        0.52803, 0.53589, 0.54412, 0.55259, 0.56116, 0.56970, 0.57808, 0.58613, 0.59362, 0.60044, 0.60718,\
        0.61333, 0.61888, 0.62429, 0.62981, 0.63551, 0.64141, 0.64752, 0.65389, 0.66058, 0.66763, 0.67510,\
        0.68304, 0.69129, 0.69894, 0.70525, 0.71161, 0.71824, 0.72488, 0.73080, 0.73565, 0.73940, 0.74211,\
        0.74387, 0.74476, 0.74486, 0.74441, 0.74390, 0.74390, 0.74486, 0.74700, 0.75005, 0.75342, 0.75642,\
        0.75853, 0.75945, 0.75911, 0.75756, 0.75487, 0.75111, 0.74639, 0.74086, 0.73470, 0.72808, 0.72110,\
        0.71364, 0.70599, 0.69923, 0.69387, 0.69010, 0.68805, 0.68746, 0.68756, 0.68772, 0.68670, 0.68305,\
        0.67731, 0.67047, 0.66325, 0.65604, 0.64902, 0.64224, 0.63563, 0.62906, 0.62242, 0.61571, 0.60915,\
        0.60271, 0.59627, 0.59016, 0.58334, 0.57604, 0.56860, 0.56120, 0.55394, 0.54686, 0.54002, 0.53343,\
        0.52715, 0.52118, 0.51558, 0.51038, 0.50566, 0.50154, 0.49864, 0.49737, 0.49676, 0.49622, 0.49532,\
        0.49380, 0.49154, 0.48856, 0.48499, 0.48094, 0.47650, 0.47172, 0.46665, 0.46130, 0.45568, 0.44979],\
        [0.49785, 0.49808, 0.49792, 0.49772, 0.49788, 0.49877, 0.50080, 0.50418, 0.50876, 0.51432, 0.52069,\
        0.52771, 0.53523, 0.54313, 0.55129, 0.55961, 0.56799, 0.57631, 0.58444, 0.59218, 0.59922, 0.60513,\
        0.60980, 0.61407, 0.61855, 0.62341, 0.62861, 0.63409, 0.63986, 0.64596, 0.65246, 0.65939, 0.66673,\
        0.67438, 0.68202, 0.68904, 0.69528, 0.70133, 0.70795, 0.71478, 0.72043, 0.72466, 0.72779, 0.72987,\
        0.73109, 0.73164, 0.73153, 0.73085, 0.73013, 0.73003, 0.73108, 0.73368, 0.73750, 0.74160, 0.74515,\
        0.74760, 0.74874, 0.74852, 0.74703, 0.74437, 0.74066, 0.73603, 0.73068, 0.72479, 0.71851, 0.71223,\
        0.70556, 0.69789, 0.69132, 0.68608, 0.68231, 0.68000, 0.67886, 0.67829, 0.67758, 0.67581, 0.67229,\
        0.66715, 0.66098, 0.65438, 0.64775, 0.64131, 0.63512, 0.62908, 0.62301, 0.61669, 0.61003, 0.60334,\
        0.59732, 0.59093, 0.58452, 0.57785, 0.57088, 0.56376, 0.55665, 0.54965, 0.54283, 0.53625, 0.52994,\
        0.52393, 0.51828, 0.51302, 0.50819, 0.50388, 0.50033, 0.49748, 0.49567, 0.49457, 0.49365, 0.49250,\
        0.49088, 0.48867, 0.48585, 0.48246, 0.47860, 0.47433, 0.46970, 0.46475, 0.45950, 0.45397, 0.44814],\
        [0.49669, 0.49716, 0.49732, 0.49746, 0.49793, 0.49911, 0.50136, 0.50477, 0.50924, 0.51459, 0.52066,\
        0.52732, 0.53445, 0.54194, 0.54971, 0.55769, 0.56578, 0.57393, 0.58202, 0.58994, 0.59726, 0.60218,\
        0.60532, 0.60843, 0.61218, 0.61659, 0.62143, 0.62654, 0.63195, 0.63776, 0.64405, 0.65081, 0.65796,\
        0.66530, 0.67255, 0.67937, 0.68573, 0.69195, 0.69863, 0.70568, 0.71045, 0.71376, 0.71631, 0.71774,\
        0.71842, 0.71880, 0.71871, 0.71775, 0.71672, 0.71651, 0.71745, 0.72068, 0.72564, 0.73065, 0.73471,\
        0.73742, 0.73865, 0.73843, 0.73691, 0.73426, 0.73057, 0.72597, 0.72075, 0.71516, 0.70904, 0.70305,\
        0.69776, 0.69048, 0.68379, 0.67844, 0.67447, 0.67176, 0.67000, 0.66866, 0.66713, 0.66475, 0.66113,\
        0.65630, 0.65064, 0.64463, 0.63863, 0.63285, 0.62736, 0.62205, 0.61670, 0.61094, 0.60451, 0.59777,\
        0.59148, 0.58524, 0.57883, 0.57224, 0.56550, 0.55868, 0.55188, 0.54520, 0.53872, 0.53247, 0.52649,\
        0.52083, 0.51554, 0.51069, 0.50631, 0.50240, 0.49899, 0.49629, 0.49421, 0.49266, 0.49131, 0.48985,\
        0.48806, 0.48581, 0.48304, 0.47978, 0.47607, 0.47194, 0.46745, 0.46263, 0.45749, 0.45204, 0.44629],\
        [0.49555, 0.49630, 0.49679, 0.49726, 0.49801, 0.49940, 0.50185, 0.50533, 0.50969, 0.51479, 0.52052,\
        0.52675, 0.53341, 0.54041, 0.54769, 0.55519, 0.56283, 0.57053, 0.57817, 0.58549, 0.59186, 0.59635,\
        0.59916, 0.60173, 0.60522, 0.60943, 0.61404, 0.61879, 0.62381, 0.62931, 0.63539, 0.64200, 0.64899,\
        0.65613, 0.66319, 0.67000, 0.67659, 0.68321, 0.69016, 0.69695, 0.70004, 0.70271, 0.70503, 0.70571,\
        0.70582, 0.70627, 0.70662, 0.70545, 0.70399, 0.70416, 0.70450, 0.70873, 0.71519, 0.72089, 0.72520,\
        0.72800, 0.72916, 0.72877, 0.72714, 0.72450, 0.72089, 0.71624, 0.71097, 0.70575, 0.70019, 0.69381,\
        0.68820, 0.68192, 0.67570, 0.67041, 0.66629, 0.66323, 0.66093, 0.65895, 0.65675, 0.65387, 0.65004,\
        0.64529, 0.63994, 0.63438, 0.62896, 0.62383, 0.61906, 0.61452, 0.61002, 0.60510, 0.59897, 0.59237,\
        0.58585, 0.57945, 0.57303, 0.56653, 0.55998, 0.55343, 0.54696, 0.54064, 0.53453, 0.52867, 0.52307,\
        0.51778, 0.51286, 0.50841, 0.50450, 0.50104, 0.49784, 0.49504, 0.49274, 0.49083, 0.48910, 0.48733,\
        0.48533, 0.48298, 0.48021, 0.47700, 0.47339, 0.46939, 0.46502, 0.46031, 0.45528, 0.44992, 0.44426],\
        [0.49445, 0.49551, 0.49635, 0.49716, 0.49821, 0.49982, 0.50239, 0.50586, 0.51005, 0.51485, 0.52017,\
        0.52592, 0.53204, 0.53846, 0.54513, 0.55200, 0.55900, 0.56604, 0.57293, 0.57936, 0.58486, 0.58901,\
        0.59198, 0.59469, 0.59806, 0.60210, 0.60658, 0.61089, 0.61547, 0.62064, 0.62656, 0.63310, 0.64000,\
        0.64701, 0.65397, 0.66077, 0.66741, 0.67398, 0.68040, 0.68580, 0.68912, 0.69155, 0.69338, 0.69373,\
        0.69366, 0.69405, 0.69455, 0.69380, 0.69260, 0.69298, 0.69426, 0.69942, 0.70674, 0.71212, 0.71633,\
        0.71915, 0.72012, 0.71938, 0.71754, 0.71495, 0.71165, 0.70710, 0.70135, 0.69619, 0.69082, 0.68474,\
        0.67871, 0.67269, 0.66690, 0.66181, 0.65766, 0.65441, 0.65177, 0.64936, 0.64671, 0.64346, 0.63941,\
        0.63458, 0.62930, 0.62406, 0.61913, 0.61455, 0.61040, 0.60652, 0.60260, 0.59845, 0.59280, 0.58639,\
        0.57991, 0.57349, 0.56709, 0.56070, 0.55434, 0.54806, 0.54192, 0.53599, 0.53029, 0.52487, 0.51970,\
        0.51480, 0.51020, 0.50605, 0.50252, 0.49940, 0.49636, 0.49359, 0.49114, 0.48897, 0.48694, 0.48488,\
        0.48266, 0.48018, 0.47735, 0.47417, 0.47061, 0.46669, 0.46242, 0.45782, 0.45289, 0.44763, 0.44204],\
        [0.49336, 0.49477, 0.49597, 0.49713, 0.49848, 0.50037, 0.50298, 0.50632, 0.51026, 0.51469, 0.51954,\
        0.52474, 0.53024, 0.53599, 0.54196, 0.54810, 0.55435, 0.56059, 0.56666, 0.57227, 0.57715, 0.58110,\
        0.58433, 0.58737, 0.59076, 0.59473, 0.59890, 0.60280, 0.60698, 0.61185, 0.61777, 0.62434, 0.63118,\
        0.63808, 0.64490, 0.65155, 0.65799, 0.66415, 0.66979, 0.67441, 0.67769, 0.67993, 0.68130, 0.68175,\
        0.68179, 0.68194, 0.68207, 0.68176, 0.68158, 0.68263, 0.68562, 0.69148, 0.69827, 0.70296, 0.70739,\
        0.71050, 0.71133, 0.70997, 0.70790, 0.70537, 0.70237, 0.69885, 0.69269, 0.68697, 0.68124, 0.67523,\
        0.66914, 0.66319, 0.65761, 0.65271, 0.64866, 0.64540, 0.64267, 0.64008, 0.63722, 0.63378, 0.62956,\
        0.62459, 0.61917, 0.61407, 0.60968, 0.60544, 0.60174, 0.59834, 0.59470, 0.59062, 0.58556, 0.57971,\
        0.57352, 0.56726, 0.56099, 0.55475, 0.54860, 0.54259, 0.53678, 0.53125, 0.52601, 0.52108, 0.51640,\
        0.51194, 0.50766, 0.50362, 0.50022, 0.49741, 0.49461, 0.49191, 0.48939, 0.48702, 0.48474, 0.48244,\
        0.48001, 0.47738, 0.47448, 0.47127, 0.46774, 0.46388, 0.45969, 0.45517, 0.45033, 0.44516, 0.43966],\
        [0.49218, 0.49397, 0.49558, 0.49710, 0.49868, 0.50071, 0.50341, 0.50658, 0.51020, 0.51420, 0.51853,\
        0.52314, 0.52797, 0.53299, 0.53819, 0.54353, 0.54896, 0.55440, 0.55968, 0.56463, 0.56905, 0.57290,\
        0.57631, 0.57961, 0.58310, 0.58690, 0.59077, 0.59461, 0.59856, 0.60324, 0.60937, 0.61598, 0.62271,\
        0.62944, 0.63605, 0.64242, 0.64846, 0.65407, 0.65903, 0.66309, 0.66609, 0.66812, 0.66929, 0.66976,\
        0.66982, 0.66977, 0.66970, 0.66968, 0.67020, 0.67201, 0.67570, 0.68127, 0.68735, 0.69276, 0.69773,\
        0.70136, 0.70244, 0.70035, 0.69820, 0.69581, 0.69279, 0.68901, 0.68368, 0.67776, 0.67174, 0.66560,\
        0.65945, 0.65351, 0.64803, 0.64328, 0.63940, 0.63633, 0.63376, 0.63127, 0.62844, 0.62501, 0.62083,\
        0.61584, 0.61017, 0.60476, 0.60136, 0.59724, 0.59367, 0.59033, 0.58677, 0.58273, 0.57800, 0.57260,\
        0.56680, 0.56080, 0.55474, 0.54870, 0.54277, 0.53702, 0.53156, 0.52643, 0.52168, 0.51729, 0.51320,\
        0.50930, 0.50546, 0.50167, 0.49828, 0.49551, 0.49280, 0.49011, 0.48749, 0.48496, 0.48246, 0.47995,\
        0.47733, 0.47455, 0.47156, 0.46830, 0.46477, 0.46095, 0.45682, 0.45239, 0.44763, 0.44255, 0.43712],\
        [0.49078, 0.49298, 0.49506, 0.49703, 0.49892, 0.50107, 0.50363, 0.50649, 0.50972, 0.51326, 0.51705,\
        0.52103, 0.52516, 0.52942, 0.53380, 0.53832, 0.54294, 0.54761, 0.55222, 0.55662, 0.56071, 0.56448,\
        0.56802, 0.57151, 0.57512, 0.57889, 0.58275, 0.58667, 0.59082, 0.59576, 0.60184, 0.60824, 0.61472,\
        0.62121, 0.62752, 0.63351, 0.63908, 0.64409, 0.64844, 0.65198, 0.65465, 0.65645, 0.65749, 0.65790,\
        0.65790, 0.65769, 0.65747, 0.65753, 0.65833, 0.66046, 0.66428, 0.66958, 0.67557, 0.68143, 0.68668,\
        0.69057, 0.69199, 0.69074, 0.68865, 0.68620, 0.68307, 0.67908, 0.67404, 0.66829, 0.66220, 0.65597,\
        0.64977, 0.64379, 0.63832, 0.63365, 0.63001, 0.62733, 0.62519, 0.62305, 0.62046, 0.61722, 0.61335,\
        0.60869, 0.60331, 0.59767, 0.59383, 0.59007, 0.58635, 0.58278, 0.57910, 0.57506, 0.57049, 0.56542,\
        0.55995, 0.55424, 0.54842, 0.54261, 0.53690, 0.53141, 0.52625, 0.52152, 0.51727, 0.51346, 0.51002,\
        0.50682, 0.50363, 0.50027, 0.49698, 0.49394, 0.49105, 0.48824, 0.48548, 0.48277, 0.48008, 0.47737,\
        0.47458, 0.47166, 0.46857, 0.46526, 0.46171, 0.45791, 0.45383, 0.44946, 0.44478, 0.43978, 0.43443],\
        [0.48898, 0.49157, 0.49411, 0.49666, 0.49913, 0.50133, 0.50338, 0.50578, 0.50861, 0.51170, 0.51497,\
        0.51835, 0.52178, 0.52526, 0.52882, 0.53252, 0.53637, 0.54036, 0.54438, 0.54835, 0.55219, 0.55591,\
        0.55955, 0.56322, 0.56700, 0.57092, 0.57498, 0.57921, 0.58379, 0.58901, 0.59489, 0.60104, 0.60722,\
        0.61349, 0.61944, 0.62501, 0.63005, 0.63446, 0.63820, 0.64125, 0.64356, 0.64512, 0.64600, 0.64633,\
        0.64622, 0.64584, 0.64542, 0.64531, 0.64601, 0.64807, 0.65181, 0.65704, 0.66312, 0.66924, 0.67468,\
        0.67869, 0.68059, 0.68039, 0.67885, 0.67647, 0.67329, 0.66924, 0.66432, 0.65872, 0.65269, 0.64647,\
        0.64024, 0.63422, 0.62868, 0.62401, 0.62061, 0.61851, 0.61707, 0.61549, 0.61329, 0.61027, 0.60684,\
        0.60310, 0.59773, 0.59241, 0.58764, 0.58343, 0.57947, 0.57564, 0.57178, 0.56768, 0.56322, 0.55837,\
        0.55318, 0.54774, 0.54216, 0.53657, 0.53107, 0.52580, 0.52091, 0.51654, 0.51276, 0.50952, 0.50672,\
        0.50428, 0.50193, 0.49890, 0.49560, 0.49235, 0.48924, 0.48624, 0.48331, 0.48043, 0.47757, 0.47469,\
        0.47174, 0.46868, 0.46548, 0.46211, 0.45853, 0.45474, 0.45070, 0.44639, 0.44179, 0.43687, 0.43160],\
        [0.48662, 0.48947, 0.49234, 0.49528, 0.49819, 0.50036, 0.50195, 0.50406, 0.50666, 0.50938, 0.51216,\
        0.51500, 0.51779, 0.52051, 0.52325, 0.52615, 0.52930, 0.53270, 0.53626, 0.53990, 0.54356, 0.54724,\
        0.55100, 0.55487, 0.55890, 0.56310, 0.56748, 0.57210, 0.57706, 0.58248, 0.58831, 0.59430, 0.60041,\
        0.60644, 0.61190, 0.61707, 0.62158, 0.62531, 0.62843, 0.63100, 0.63295, 0.63425, 0.63496, 0.63518,\
        0.63498, 0.63443, 0.63372, 0.63322, 0.63347, 0.63516, 0.63876, 0.64410, 0.65043, 0.65683, 0.66251,\
        0.66679, 0.66922, 0.66977, 0.66881, 0.66668, 0.66357, 0.65956, 0.65473, 0.64926, 0.64335, 0.63722,\
        0.63107, 0.62507, 0.61944, 0.61461, 0.61134, 0.61000, 0.60951, 0.60853, 0.60683, 0.60428, 0.60046,\
        0.59776, 0.59256, 0.58691, 0.58173, 0.57709, 0.57282, 0.56876, 0.56475, 0.56062, 0.55624, 0.55157,\
        0.54661, 0.54142, 0.53608, 0.53070, 0.52539, 0.52029, 0.51559, 0.51150, 0.50817, 0.50549, 0.50315,\
        0.50123, 0.49966, 0.49705, 0.49372, 0.49039, 0.48716, 0.48402, 0.48095, 0.47792, 0.47490, 0.47186,\
        0.46876, 0.46558, 0.46228, 0.45884, 0.45523, 0.45143, 0.44743, 0.44318, 0.43865, 0.43381, 0.42862],\
        [0.48355, 0.48649, 0.48945, 0.49238, 0.49510, 0.49725, 0.49898, 0.50120, 0.50376, 0.50608, 0.50845,\
        0.51089, 0.51315, 0.51512, 0.51706, 0.51923, 0.52178, 0.52472, 0.52795, 0.53134, 0.53487, 0.53855,\
        0.54244, 0.54656, 0.55089, 0.55544, 0.56020, 0.56521, 0.57052, 0.57615, 0.58207, 0.58814, 0.59427,\
        0.59962, 0.60494, 0.60993, 0.61388, 0.61675, 0.61919, 0.62132, 0.62293, 0.62394, 0.62444, 0.62457,\
        0.62434, 0.62366, 0.62261, 0.62159, 0.62116, 0.62218, 0.62564, 0.63133, 0.63809, 0.64485, 0.65083,\
        0.65543, 0.65830, 0.65937, 0.65885, 0.65699, 0.65401, 0.65008, 0.64537, 0.64004, 0.63429, 0.62834,\
        0.62241, 0.61663, 0.61110, 0.60604, 0.60262, 0.60200, 0.60250, 0.60172, 0.60030, 0.59883, 0.59519,\
        0.59100, 0.58610, 0.58067, 0.57544, 0.57062, 0.56619, 0.56201, 0.55794, 0.55383, 0.54956, 0.54507,\
        0.54034, 0.53540, 0.53029, 0.52512, 0.51999, 0.51504, 0.51046, 0.50652, 0.50356, 0.50147, 0.49946,\
        0.49776, 0.49618, 0.49397, 0.49107, 0.48790, 0.48470, 0.48151, 0.47835, 0.47521, 0.47206, 0.46888,\
        0.46565, 0.46234, 0.45894, 0.45543, 0.45178, 0.44798, 0.44400, 0.43981, 0.43536, 0.43061, 0.42550],\
        [0.47973, 0.48261, 0.48544, 0.48817, 0.49068, 0.49286, 0.49489, 0.49722, 0.49965, 0.50152, 0.50365,\
        0.50594, 0.50786, 0.50905, 0.51022, 0.51177, 0.51385, 0.51651, 0.51955, 0.52278, 0.52619, 0.52990,\
        0.53396, 0.53837, 0.54305, 0.54796, 0.55311, 0.55849, 0.56412, 0.56998, 0.57602, 0.58213, 0.58807,\
        0.59360, 0.59892, 0.60386, 0.60718, 0.60874, 0.61050, 0.61235, 0.61356, 0.61423, 0.61442, 0.61448,\
        0.61439, 0.61369, 0.61230, 0.61084, 0.60974, 0.60972, 0.61318, 0.61956, 0.62678, 0.63385, 0.64009,\
        0.64496, 0.64812, 0.64950, 0.64924, 0.64756, 0.64470, 0.64087, 0.63628, 0.63113, 0.62558, 0.61985,\
        0.61430, 0.60910, 0.60387, 0.59972, 0.59589, 0.59498, 0.59507, 0.59429, 0.59291, 0.59103, 0.58782,\
        0.58351, 0.57867, 0.57356, 0.56853, 0.56381, 0.55942, 0.55528, 0.55129, 0.54729, 0.54320, 0.53892,\
        0.53444, 0.52975, 0.52490, 0.51995, 0.51502, 0.51024, 0.50579, 0.50191, 0.49913, 0.49742, 0.49577,\
        0.49417, 0.49251, 0.49044, 0.48787, 0.48495, 0.48186, 0.47871, 0.47551, 0.47229, 0.46904, 0.46574,\
        0.46239, 0.45897, 0.45546, 0.45186, 0.44817, 0.44435, 0.44040, 0.43626, 0.43190, 0.42725, 0.42223],\
        [0.47520, 0.47789, 0.48049, 0.48297, 0.48527, 0.48740, 0.48949, 0.49166, 0.49376, 0.49566, 0.49770,\
        0.49996, 0.50181, 0.50212, 0.50277, 0.50390, 0.50563, 0.50822, 0.51122, 0.51431, 0.51760, 0.52134,\
        0.52564, 0.53038, 0.53543, 0.54072, 0.54623, 0.55194, 0.55785, 0.56393, 0.57009, 0.57624, 0.58224,\
        0.58801, 0.59355, 0.59858, 0.60153, 0.60117, 0.60268, 0.60420, 0.60479, 0.60520, 0.60486, 0.60487,\
        0.60506, 0.60456, 0.60280, 0.60111, 0.60010, 0.59911, 0.60291, 0.60983, 0.61698, 0.62416, 0.63057,\
        0.63557, 0.63884, 0.64031, 0.64012, 0.63850, 0.63569, 0.63193, 0.62747, 0.62252, 0.61726, 0.61173,\
        0.60643, 0.60233, 0.59774, 0.59347, 0.59008, 0.58821, 0.58724, 0.58611, 0.58450, 0.58224, 0.57906,\
        0.57503, 0.57048, 0.56573, 0.56105, 0.55661, 0.55245, 0.54853, 0.54475, 0.54099, 0.53714, 0.53314,\
        0.52894, 0.52454, 0.51997, 0.51528, 0.51059, 0.50605, 0.50183, 0.49824, 0.49557, 0.49373, 0.49215,\
        0.49058, 0.48887, 0.48685, 0.48445, 0.48172, 0.47876, 0.47566, 0.47247, 0.46920, 0.46586, 0.46246,\
        0.45899, 0.45545, 0.45183, 0.44815, 0.44438, 0.44054, 0.43660, 0.43253, 0.42826, 0.42372, 0.41881],\
        [0.47004, 0.47246, 0.47477, 0.47695, 0.47900, 0.48096, 0.48288, 0.48481, 0.48671, 0.48858, 0.49054,\
        0.49257, 0.49401, 0.49437, 0.49488, 0.49585, 0.49746, 0.50011, 0.50310, 0.50604, 0.50913, 0.51295,\
        0.51760, 0.52274, 0.52814, 0.53375, 0.53957, 0.54558, 0.55173, 0.55796, 0.56422, 0.57040, 0.57643,\
        0.58222, 0.58766, 0.59234, 0.59477, 0.59481, 0.59572, 0.59641, 0.59648, 0.59660, 0.59598, 0.59578,\
        0.59590, 0.59547, 0.59389, 0.59244, 0.59168, 0.59218, 0.59616, 0.60195, 0.60861, 0.61587, 0.62234,\
        0.62729, 0.63047, 0.63183, 0.63152, 0.62981, 0.62698, 0.62325, 0.61887, 0.61413, 0.60933, 0.60421,\
        0.59885, 0.59485, 0.59100, 0.58690, 0.58352, 0.58108, 0.57927, 0.57756, 0.57553, 0.57298, 0.56977,\
        0.56595, 0.56173, 0.55739, 0.55313, 0.54910, 0.54534, 0.54178, 0.53834, 0.53492, 0.53140, 0.52773,\
        0.52386, 0.51979, 0.51553, 0.51113, 0.50672, 0.50247, 0.49857, 0.49522, 0.49259, 0.49055, 0.48881,\
        0.48714, 0.48535, 0.48332, 0.48099, 0.47836, 0.47550, 0.47245, 0.46926, 0.46595, 0.46255, 0.45905,\
        0.45547, 0.45181, 0.44807, 0.44428, 0.44043, 0.43654, 0.43259, 0.42858, 0.42442, 0.42001, 0.41523],\
        [0.46437, 0.46647, 0.46843, 0.47026, 0.47200, 0.47368, 0.47533, 0.47700, 0.47868, 0.48040, 0.48216,\
        0.48383, 0.48508, 0.48585, 0.48661, 0.48777, 0.48956, 0.49210, 0.49501, 0.49795, 0.50093, 0.50497,\
        0.51012, 0.51561, 0.52126, 0.52712, 0.53319, 0.53943, 0.54576, 0.55211, 0.55838, 0.56451, 0.57039,\
        0.57593, 0.58089, 0.58482, 0.58707, 0.58786, 0.58831, 0.58850, 0.58834, 0.58801, 0.58746, 0.58701,\
        0.58671, 0.58613, 0.58513, 0.58431, 0.58432, 0.58590, 0.58963, 0.59493, 0.60152, 0.60896, 0.61523,\
        0.61994, 0.62285, 0.62391, 0.62333, 0.62142, 0.61850, 0.61478, 0.61044, 0.60577, 0.60131, 0.59701,\
        0.59185, 0.58745, 0.58348, 0.57966, 0.57626, 0.57346, 0.57109, 0.56883, 0.56640, 0.56358, 0.56030,\
        0.55661, 0.55269, 0.54874, 0.54494, 0.54141, 0.53815, 0.53509, 0.53211, 0.52911, 0.52599, 0.52270,\
        0.51920, 0.51548, 0.51157, 0.50751, 0.50336, 0.49938, 0.49576, 0.49258, 0.48991, 0.48770, 0.48576,\
        0.48392, 0.48200, 0.47991, 0.47758, 0.47499, 0.47217, 0.46915, 0.46596, 0.46261, 0.45914, 0.45555,\
        0.45185, 0.44806, 0.44420, 0.44027, 0.43631, 0.43233, 0.42836, 0.42438, 0.42033, 0.41610, 0.41146],\
        [0.45831, 0.46003, 0.46161, 0.46306, 0.46443, 0.46575, 0.46709, 0.46846, 0.46989, 0.47139, 0.47293,\
        0.47441, 0.47571, 0.47682, 0.47801, 0.47953, 0.48156, 0.48414, 0.48708, 0.49019, 0.49362, 0.49803,\
        0.50349, 0.50909, 0.51483, 0.52083, 0.52710, 0.53354, 0.54001, 0.54639, 0.55260, 0.55856, 0.56416,\
        0.56928, 0.57369, 0.57707, 0.57919, 0.58020, 0.58055, 0.58049, 0.58012, 0.57956, 0.57889, 0.57826,\
        0.57770, 0.57711, 0.57654, 0.57638, 0.57715, 0.57936, 0.58330, 0.58883, 0.59575, 0.60291, 0.60859,\
        0.61309, 0.61565, 0.61632, 0.61532, 0.61314, 0.61015, 0.60644, 0.60230, 0.59753, 0.59317, 0.58884,\
        0.58425, 0.57982, 0.57574, 0.57195, 0.56852, 0.56548, 0.56274, 0.56008, 0.55731, 0.55427, 0.55091,\
        0.54729, 0.54358, 0.53999, 0.53666, 0.53369, 0.53103, 0.52856, 0.52614, 0.52363, 0.52095, 0.51805,\
        0.51493, 0.51160, 0.50805, 0.50434, 0.50053, 0.49673, 0.49326, 0.49015, 0.48743, 0.48506, 0.48293,\
        0.48089, 0.47882, 0.47663, 0.47426, 0.47166, 0.46884, 0.46582, 0.46261, 0.45923, 0.45568, 0.45200,\
        0.44818, 0.44426, 0.44024, 0.43616, 0.43205, 0.42795, 0.42389, 0.41990, 0.41596, 0.41192, 0.40749],\
        [0.45196, 0.45330, 0.45445, 0.45549, 0.45645, 0.45739, 0.45836, 0.45941, 0.46056, 0.46185, 0.46323,\
        0.46467, 0.46611, 0.46758, 0.46921, 0.47116, 0.47354, 0.47637, 0.47957, 0.48311, 0.48714, 0.49194,\
        0.49743, 0.50305, 0.50877, 0.51490, 0.52139, 0.52801, 0.53454, 0.54088, 0.54694, 0.55263, 0.55787,\
        0.56252, 0.56641, 0.56937, 0.57130, 0.57229, 0.57258, 0.57239, 0.57187, 0.57114, 0.57033, 0.56955,\
        0.56886, 0.56831, 0.56806, 0.56839, 0.56971, 0.57237, 0.57656, 0.58224, 0.58900, 0.59567, 0.60155,\
        0.60622, 0.60825, 0.60872, 0.60726, 0.60472, 0.60186, 0.59808, 0.59403, 0.58955, 0.58507, 0.58066,\
        0.57621, 0.57187, 0.56777, 0.56398, 0.56050, 0.55732, 0.55436, 0.55146, 0.54847, 0.54526, 0.54181,\
        0.53821, 0.53464, 0.53133, 0.52846, 0.52609, 0.52411, 0.52232, 0.52051, 0.51854, 0.51630, 0.51377,\
        0.51101, 0.50806, 0.50486, 0.50142, 0.49790, 0.49432, 0.49093, 0.48783, 0.48505, 0.48253, 0.48022,\
        0.47800, 0.47579, 0.47348, 0.47103, 0.46839, 0.46555, 0.46251, 0.45927, 0.45584, 0.45222, 0.44844,\
        0.44450, 0.44044, 0.43626, 0.43201, 0.42772, 0.42343, 0.41921, 0.41514, 0.41123, 0.40739, 0.40325],\
        [0.44547, 0.44639, 0.44711, 0.44771, 0.44823, 0.44876, 0.44935, 0.45007, 0.45095, 0.45204, 0.45333,\
        0.45481, 0.45646, 0.45829, 0.46038, 0.46281, 0.46563, 0.46886, 0.47249, 0.47652, 0.48104, 0.48612,\
        0.49162, 0.49727, 0.50303, 0.50942, 0.51620, 0.52295, 0.52947, 0.53567, 0.54149, 0.54684, 0.55166,\
        0.55584, 0.55927, 0.56184, 0.56353, 0.56439, 0.56459, 0.56429, 0.56364, 0.56278, 0.56184, 0.56093,\
        0.56015, 0.55964, 0.55957, 0.56019, 0.56182, 0.56470, 0.56896, 0.57451, 0.58092, 0.58748, 0.59372,\
        0.59804, 0.59979, 0.60068, 0.59899, 0.59618, 0.59331, 0.58972, 0.58568, 0.58135, 0.57690, 0.57244,\
        0.56802, 0.56373, 0.55965, 0.55586, 0.55238, 0.54917, 0.54613, 0.54314, 0.54004, 0.53674, 0.53323,\
        0.52960, 0.52607, 0.52296, 0.52052, 0.51880, 0.51757, 0.51651, 0.51536, 0.51392, 0.51208, 0.50986,\
        0.50737, 0.50480, 0.50196, 0.49869, 0.49532, 0.49192, 0.48862, 0.48553, 0.48268, 0.48005, 0.47758,\
        0.47520, 0.47284, 0.47042, 0.46788, 0.46519, 0.46231, 0.45924, 0.45596, 0.45248, 0.44879, 0.44492,\
        0.44086, 0.43665, 0.43232, 0.42789, 0.42340, 0.41888, 0.41443, 0.41017, 0.40618, 0.40232, 0.39842],\
        [0.43894, 0.43944, 0.43973, 0.43988, 0.43997, 0.44008, 0.44028, 0.44066, 0.44127, 0.44219, 0.44344,\
        0.44502, 0.44692, 0.44914, 0.45170, 0.45462, 0.45794, 0.46166, 0.46579, 0.47033, 0.47530, 0.48067,\
        0.48633, 0.49211, 0.49806, 0.50478, 0.51176, 0.51849, 0.52487, 0.53083, 0.53632, 0.54128, 0.54566,\
        0.54937, 0.55235, 0.55455, 0.55597, 0.55666, 0.55672, 0.55630, 0.55553, 0.55454, 0.55347, 0.55244,\
        0.55158, 0.55104, 0.55100, 0.55171, 0.55341, 0.55629, 0.56042, 0.56570, 0.57175, 0.57801, 0.58375,\
        0.58793, 0.59024, 0.59112, 0.59003, 0.58757, 0.58463, 0.58118, 0.57727, 0.57305, 0.56865, 0.56420,\
        0.55979, 0.55552, 0.55148, 0.54773, 0.54431, 0.54117, 0.53820, 0.53527, 0.53222, 0.52894, 0.52541,\
        0.52173, 0.51815, 0.51509, 0.51303, 0.51202, 0.51160, 0.51129, 0.51077, 0.50984, 0.50837, 0.50633,\
        0.50389, 0.50154, 0.49923, 0.49608, 0.49278, 0.48947, 0.48624, 0.48317, 0.48026, 0.47754, 0.47494,\
        0.47244, 0.46995, 0.46742, 0.46480, 0.46205, 0.45913, 0.45603, 0.45271, 0.44919, 0.44544, 0.44148,\
        0.43731, 0.43296, 0.42847, 0.42387, 0.41920, 0.41447, 0.40972, 0.40516, 0.40108, 0.39715, 0.39304],\
        [0.43251, 0.43260, 0.43245, 0.43217, 0.43183, 0.43154, 0.43136, 0.43140, 0.43175, 0.43252, 0.43375,\
        0.43547, 0.43766, 0.44027, 0.44330, 0.44673, 0.45056, 0.45481, 0.45947, 0.46452, 0.46995, 0.47571,\
        0.48172, 0.48793, 0.49442, 0.50131, 0.50811, 0.51463, 0.52074, 0.52638, 0.53149, 0.53602, 0.53993,\
        0.54318, 0.54573, 0.54757, 0.54870, 0.54917, 0.54907, 0.54852, 0.54762, 0.54650, 0.54529, 0.54412,\
        0.54315, 0.54251, 0.54239, 0.54300, 0.54456, 0.54723, 0.55108, 0.55601, 0.56167, 0.56753, 0.57288,\
        0.57705, 0.57969, 0.58078, 0.58026, 0.57844, 0.57579, 0.57255, 0.56882, 0.56473, 0.56042, 0.55600,\
        0.55161, 0.54736, 0.54335, 0.53969, 0.53640, 0.53345, 0.53071, 0.52801, 0.52516, 0.52205, 0.51862,\
        0.51494, 0.51125, 0.50806, 0.50627, 0.50607, 0.50644, 0.50678, 0.50679, 0.50630, 0.50516, 0.50328,\
        0.50082, 0.49833, 0.49604, 0.49323, 0.49011, 0.48691, 0.48375, 0.48069, 0.47775, 0.47495, 0.47226,\
        0.46965, 0.46705, 0.46444, 0.46175, 0.45895, 0.45600, 0.45287, 0.44954, 0.44598, 0.44219, 0.43815,\
        0.43388, 0.42941, 0.42477, 0.42003, 0.41524, 0.41040, 0.40544, 0.40054, 0.39629, 0.39224, 0.38790],\
        [0.42629, 0.42599, 0.42542, 0.42473, 0.42401, 0.42334, 0.42280, 0.42251, 0.42260, 0.42322, 0.42447,\
        0.42635, 0.42883, 0.43185, 0.43533, 0.43925, 0.44360, 0.44837, 0.45354, 0.45911, 0.46502, 0.47123,\
        0.47770, 0.48437, 0.49127, 0.49825, 0.50495, 0.51122, 0.51702, 0.52229, 0.52699, 0.53108, 0.53452,\
        0.53731, 0.53944, 0.54091, 0.54174, 0.54199, 0.54172, 0.54101, 0.53998, 0.53872, 0.53737, 0.53606,\
        0.53493, 0.53413, 0.53382, 0.53419, 0.53543, 0.53773, 0.54120, 0.54576, 0.55108, 0.55663, 0.56179,\
        0.56599, 0.56888, 0.57031, 0.57031, 0.56906, 0.56686, 0.56392, 0.56043, 0.55651, 0.55230, 0.54795,\
        0.54358, 0.53935, 0.53539, 0.53183, 0.52876, 0.52612, 0.52376, 0.52147, 0.51901, 0.51623, 0.51306,\
        0.50955, 0.50589, 0.50250, 0.50075, 0.50141, 0.50226, 0.50299, 0.50334, 0.50312, 0.50231, 0.50056,\
        0.49815, 0.49559, 0.49304, 0.49028, 0.48730, 0.48420, 0.48110, 0.47806, 0.47511, 0.47226, 0.46950,\
        0.46680, 0.46413, 0.46144, 0.45870, 0.45587, 0.45290, 0.44977, 0.44644, 0.44288, 0.43906, 0.43498,\
        0.43063, 0.42603, 0.42125, 0.41636, 0.41150, 0.40671, 0.40187, 0.39684, 0.39220, 0.38774, 0.38308],\
        [0.42040, 0.41972, 0.41877, 0.41771, 0.41668, 0.41570, 0.41485, 0.41424, 0.41405, 0.41452, 0.41580,\
        0.41786, 0.42065, 0.42404, 0.42794, 0.43231, 0.43712, 0.44237, 0.44804, 0.45410, 0.46050, 0.46717,\
        0.47407, 0.48113, 0.48828, 0.49534, 0.50201, 0.50811, 0.51362, 0.51854, 0.52283, 0.52646, 0.52943,\
        0.53176, 0.53347, 0.53459, 0.53514, 0.53516, 0.53471, 0.53386, 0.53269, 0.53130, 0.52981, 0.52834,\
        0.52704, 0.52605, 0.52548, 0.52551, 0.52630, 0.52810, 0.53110, 0.53531, 0.54040, 0.54582, 0.55093,\
        0.55521, 0.55832, 0.56010, 0.56055, 0.55979, 0.55803, 0.55545, 0.55222, 0.54851, 0.54444, 0.54017,\
        0.53584, 0.53162, 0.52769, 0.52426, 0.52144, 0.51924, 0.51744, 0.51572, 0.51382, 0.51153, 0.50878,\
        0.50566, 0.50237, 0.49942, 0.49782, 0.49814, 0.49894, 0.49971, 0.50018, 0.49996, 0.49930, 0.49770,\
        0.49534, 0.49276, 0.49009, 0.48730, 0.48436, 0.48134, 0.47829, 0.47527, 0.47231, 0.46943, 0.46661,\
        0.46385, 0.46112, 0.45839, 0.45562, 0.45277, 0.44981, 0.44670, 0.44340, 0.43987, 0.43607, 0.43198,\
        0.42759, 0.42289, 0.41795, 0.41288, 0.40787, 0.40310, 0.39839, 0.39339, 0.38845, 0.38357, 0.37854],\
        [0.41493, 0.41392, 0.41261, 0.41125, 0.41004, 0.40887, 0.40777, 0.40691, 0.40642, 0.40672, 0.40805,\
        0.41025, 0.41331, 0.41704, 0.42128, 0.42599, 0.43119, 0.43687, 0.44299, 0.44951, 0.45635, 0.46344,\
        0.47072, 0.47809, 0.48542, 0.49254, 0.49920, 0.50518, 0.51045, 0.51507, 0.51897, 0.52214, 0.52464,\
        0.52651, 0.52782, 0.52861, 0.52889, 0.52870, 0.52809, 0.52710, 0.52582, 0.52431, 0.52268, 0.52106,\
        0.51960, 0.51842, 0.51760, 0.51725, 0.51751, 0.51868, 0.52114, 0.52508, 0.53013, 0.53557, 0.54073,\
        0.54510, 0.54839, 0.55044, 0.55125, 0.55090, 0.54953, 0.54729, 0.54435, 0.54086, 0.53696, 0.53279,\
        0.52852, 0.52432, 0.52041, 0.51706, 0.51454, 0.51289, 0.51180, 0.51081, 0.50957, 0.50786, 0.50562,\
        0.50299, 0.50023, 0.49767, 0.49615, 0.49583, 0.49618, 0.49666, 0.49691, 0.49664, 0.49582, 0.49431,\
        0.49216, 0.48967, 0.48700, 0.48420, 0.48130, 0.47832, 0.47530, 0.47230, 0.46934, 0.46643, 0.46358,\
        0.46078, 0.45801, 0.45525, 0.45246, 0.44963, 0.44670, 0.44364, 0.44040, 0.43693, 0.43320, 0.42916,\
        0.42478, 0.42004, 0.41497, 0.40966, 0.40436, 0.39949, 0.39475, 0.38983, 0.38476, 0.37957, 0.37419],\
        [0.40997, 0.40869, 0.40709, 0.40545, 0.40428, 0.40310, 0.40177, 0.40082, 0.40010, 0.40023, 0.40162,\
        0.40378, 0.40709, 0.41103, 0.41545, 0.42037, 0.42586, 0.43188, 0.43840, 0.44532, 0.45255, 0.46001,\
        0.46760, 0.47521, 0.48269, 0.48985, 0.49650, 0.50236, 0.50749, 0.51189, 0.51542, 0.51812, 0.52011,\
        0.52153, 0.52246, 0.52295, 0.52299, 0.52262, 0.52187, 0.52079, 0.51943, 0.51783, 0.51607, 0.51431,\
        0.51272, 0.51141, 0.51042, 0.50977, 0.50951, 0.50993, 0.51177, 0.51557, 0.52078, 0.52637, 0.53157,\
        0.53598, 0.53934, 0.54157, 0.54263, 0.54258, 0.54153, 0.53961, 0.53695, 0.53369, 0.52998, 0.52596,\
        0.52179, 0.51765, 0.51375, 0.51041, 0.50813, 0.50717, 0.50691, 0.50670, 0.50617, 0.50505, 0.50324,\
        0.50094, 0.49852, 0.49627, 0.49469, 0.49393, 0.49375, 0.49377, 0.49365, 0.49318, 0.49222, 0.49072,\
        0.48870, 0.48632, 0.48371, 0.48094, 0.47807, 0.47512, 0.47213, 0.46914, 0.46618, 0.46325, 0.46038,\
        0.45754, 0.45475, 0.45197, 0.44919, 0.44639, 0.44351, 0.44053, 0.43739, 0.43404, 0.43042, 0.42649,\
        0.42220, 0.41751, 0.41242, 0.40699, 0.40138, 0.39628, 0.39144, 0.38640, 0.38114, 0.37566, 0.36995],\
        [0.40554, 0.40408, 0.40237, 0.40058, 0.39950, 0.39853, 0.39708, 0.39609, 0.39547, 0.39564, 0.39680,\
        0.39893, 0.40228, 0.40614, 0.41050, 0.41547, 0.42113, 0.42744, 0.43428, 0.44154, 0.44910, 0.45686,\
        0.46470, 0.47250, 0.48010, 0.48731, 0.49390, 0.49965, 0.50490, 0.50909, 0.51218, 0.51433, 0.51579,\
        0.51676, 0.51735, 0.51758, 0.51744, 0.51693, 0.51607, 0.51493, 0.51355, 0.51193, 0.51008, 0.50817,\
        0.50650, 0.50519, 0.50415, 0.50337, 0.50279, 0.50252, 0.50361, 0.50753, 0.51307, 0.51869, 0.52377,\
        0.52803, 0.53133, 0.53359, 0.53479, 0.53496, 0.53416, 0.53251, 0.53011, 0.52710, 0.52361, 0.51979,\
        0.51578, 0.51179, 0.50802, 0.50466, 0.50243, 0.50232, 0.50285, 0.50325, 0.50339, 0.50280, 0.50129,\
        0.49916, 0.49687, 0.49477, 0.49312, 0.49204, 0.49139, 0.49094, 0.49045, 0.48970, 0.48858, 0.48703,\
        0.48506, 0.48276, 0.48021, 0.47749, 0.47465, 0.47173, 0.46877, 0.46579, 0.46282, 0.45988, 0.45698,\
        0.45413, 0.45131, 0.44853, 0.44577, 0.44301, 0.44021, 0.43733, 0.43433, 0.43113, 0.42768, 0.42391,\
        0.41978, 0.41525, 0.41030, 0.40495, 0.39939, 0.39392, 0.38860, 0.38320, 0.37761, 0.37180, 0.36574],\
        [0.40163, 0.40003, 0.39845, 0.39687, 0.39568, 0.39467, 0.39352, 0.39264, 0.39222, 0.39249, 0.39361,\
        0.39570, 0.39869, 0.40230, 0.40636, 0.41123, 0.41701, 0.42356, 0.43067, 0.43819, 0.44599, 0.45396,\
        0.46198, 0.46992, 0.47764, 0.48497, 0.49171, 0.49776, 0.50285, 0.50664, 0.50918, 0.51069, 0.51157,\
        0.51211, 0.51243, 0.51249, 0.51222, 0.51162, 0.51068, 0.50949, 0.50819, 0.50666, 0.50479, 0.50273,\
        0.50105, 0.49984, 0.49886, 0.49814, 0.49759, 0.49730, 0.49826, 0.50226, 0.50768, 0.51283, 0.51741,\
        0.52129, 0.52436, 0.52654, 0.52778, 0.52808, 0.52747, 0.52603, 0.52387, 0.52112, 0.51789, 0.51433,\
        0.51056, 0.50681, 0.50342, 0.50050, 0.49843, 0.49864, 0.49947, 0.50015, 0.50087, 0.50068, 0.49921,\
        0.49721, 0.49503, 0.49299, 0.49127, 0.48994, 0.48893, 0.48809, 0.48724, 0.48622, 0.48492, 0.48328,\
        0.48131, 0.47904, 0.47654, 0.47386, 0.47105, 0.46816, 0.46521, 0.46223, 0.45926, 0.45631, 0.45339,\
        0.45051, 0.44768, 0.44489, 0.44215, 0.43944, 0.43673, 0.43398, 0.43113, 0.42812, 0.42486, 0.42130,\
        0.41738, 0.41308, 0.40836, 0.40314, 0.39753, 0.39176, 0.38596, 0.38009, 0.37409, 0.36790, 0.36149],\
        [0.39791, 0.39647, 0.39505, 0.39371, 0.39258, 0.39162, 0.39078, 0.39018, 0.39000, 0.39041, 0.39153,\
        0.39341, 0.39601, 0.39920, 0.40280, 0.40761, 0.41356, 0.42030, 0.42760, 0.43529, 0.44323, 0.45131,\
        0.45942, 0.46744, 0.47524, 0.48266, 0.48956, 0.49573, 0.50074, 0.50431, 0.50625, 0.50706, 0.50733,\
        0.50750, 0.50769, 0.50767, 0.50731, 0.50668, 0.50572, 0.50442, 0.50327, 0.50201, 0.50020, 0.49818,\
        0.49652, 0.49537, 0.49454, 0.49401, 0.49384, 0.49433, 0.49625, 0.50010, 0.50442, 0.50854, 0.51229,\
        0.51561, 0.51834, 0.52036, 0.52156, 0.52191, 0.52144, 0.52018, 0.51824, 0.51573, 0.51279, 0.50956,\
        0.50615, 0.50264, 0.49968, 0.49736, 0.49589, 0.49575, 0.49639, 0.49720, 0.49792, 0.49782, 0.49658,\
        0.49477, 0.49276, 0.49081, 0.48905, 0.48755, 0.48627, 0.48512, 0.48397, 0.48269, 0.48121, 0.47947,\
        0.47746, 0.47519, 0.47271, 0.47005, 0.46727, 0.46439, 0.46145, 0.45848, 0.45550, 0.45252, 0.44958,\
        0.44667, 0.44382, 0.44103, 0.43830, 0.43563, 0.43301, 0.43040, 0.42773, 0.42493, 0.42189, 0.41853,\
        0.41481, 0.41071, 0.40626, 0.40126, 0.39548, 0.38937, 0.38315, 0.37685, 0.37044, 0.36388, 0.35712],\
        [0.39363, 0.39287, 0.39184, 0.39082, 0.38994, 0.38922, 0.38869, 0.38843, 0.38855, 0.38915, 0.39029,\
        0.39196, 0.39413, 0.39671, 0.39998, 0.40488, 0.41094, 0.41776, 0.42512, 0.43284, 0.44080, 0.44888,\
        0.45697, 0.46496, 0.47274, 0.48016, 0.48708, 0.49327, 0.49841, 0.50191, 0.50310, 0.50326, 0.50303,\
        0.50289, 0.50320, 0.50315, 0.50268, 0.50208, 0.50127, 0.49977, 0.49869, 0.49761, 0.49599, 0.49425,\
        0.49276, 0.49169, 0.49104, 0.49082, 0.49116, 0.49234, 0.49475, 0.49832, 0.50197, 0.50499, 0.50789,\
        0.51066, 0.51306, 0.51490, 0.51602, 0.51640, 0.51602, 0.51491, 0.51314, 0.51086, 0.50821, 0.50533,\
        0.50240, 0.49943, 0.49663, 0.49466, 0.49347, 0.49316, 0.49350, 0.49407, 0.49449, 0.49430, 0.49333,\
        0.49180, 0.49002, 0.48818, 0.48643, 0.48483, 0.48336, 0.48197, 0.48057, 0.47908, 0.47743, 0.47558,\
        0.47351, 0.47122, 0.46873, 0.46609, 0.46332, 0.46044, 0.45751, 0.45452, 0.45152, 0.44853, 0.44555,\
        0.44261, 0.43972, 0.43691, 0.43417, 0.43154, 0.42900, 0.42652, 0.42404, 0.42146, 0.41865, 0.41548,\
        0.41188, 0.40785, 0.40341, 0.39857, 0.39273, 0.38640, 0.37989, 0.37326, 0.36652, 0.35962, 0.35253],\
        [0.38925, 0.38910, 0.38862, 0.38805, 0.38756, 0.38722, 0.38708, 0.38721, 0.38768, 0.38853, 0.38978,\
        0.39139, 0.39332, 0.39563, 0.39878, 0.40340, 0.40930, 0.41602, 0.42326, 0.43086, 0.43868, 0.44662,\
        0.45456, 0.46241, 0.47003, 0.47730, 0.48407, 0.49012, 0.49516, 0.49836, 0.49926, 0.49924, 0.49887,\
        0.49860, 0.49900, 0.49886, 0.49830, 0.49770, 0.49692, 0.49571, 0.49452, 0.49338, 0.49203, 0.49061,\
        0.48938, 0.48852, 0.48809, 0.48817, 0.48888, 0.49040, 0.49283, 0.49597, 0.49898, 0.50136, 0.50374,\
        0.50614, 0.50832, 0.51000, 0.51103, 0.51139, 0.51111, 0.51014, 0.50849, 0.50637, 0.50400, 0.50143,\
        0.49882, 0.49630, 0.49393, 0.49212, 0.49100, 0.49054, 0.49057, 0.49080, 0.49091, 0.49060, 0.48975,\
        0.48844, 0.48684, 0.48513, 0.48342, 0.48176, 0.48016, 0.47859, 0.47701, 0.47535, 0.47355, 0.47159,\
        0.46945, 0.46712, 0.46462, 0.46197, 0.45919, 0.45632, 0.45337, 0.45038, 0.44735, 0.44432, 0.44130,\
        0.43832, 0.43538, 0.43252, 0.42976, 0.42713, 0.42463, 0.42228, 0.41999, 0.41766, 0.41508, 0.41207,\
        0.40849, 0.40440, 0.39982, 0.39472, 0.38899, 0.38267, 0.37603, 0.36919, 0.36219, 0.35501, 0.34765],\
        [0.38484, 0.38525, 0.38534, 0.38531, 0.38530, 0.38542, 0.38573, 0.38631, 0.38719, 0.38839, 0.38991,\
        0.39166, 0.39360, 0.39582, 0.39872, 0.40304, 0.40863, 0.41503, 0.42196, 0.42927, 0.43682, 0.44448,\
        0.45216, 0.45972, 0.46706, 0.47404, 0.48047, 0.48613, 0.49066, 0.49359, 0.49483, 0.49507, 0.49490,\
        0.49475, 0.49480, 0.49462, 0.49413, 0.49349, 0.49269, 0.49168, 0.49058, 0.48947, 0.48831, 0.48718,\
        0.48623, 0.48560, 0.48540, 0.48570, 0.48660, 0.48818, 0.49038, 0.49296, 0.49544, 0.49760, 0.49972,\
        0.50191, 0.50397, 0.50552, 0.50639, 0.50672, 0.50657, 0.50577, 0.50418, 0.50213, 0.49999, 0.49772,\
        0.49538, 0.49314, 0.49111, 0.48949, 0.48838, 0.48777, 0.48752, 0.48743, 0.48726, 0.48681, 0.48598,\
        0.48479, 0.48333, 0.48172, 0.48005, 0.47836, 0.47668, 0.47499, 0.47326, 0.47146, 0.46954, 0.46748,\
        0.46526, 0.46289, 0.46036, 0.45769, 0.45490, 0.45202, 0.44906, 0.44604, 0.44299, 0.43992, 0.43685,\
        0.43381, 0.43081, 0.42788, 0.42506, 0.42237, 0.41988, 0.41761, 0.41551, 0.41341, 0.41110, 0.40828,\
        0.40467, 0.40038, 0.39567, 0.39041, 0.38459, 0.37826, 0.37156, 0.36459, 0.35740, 0.35001, 0.34242],\
        [0.38040, 0.38134, 0.38199, 0.38251, 0.38303, 0.38366, 0.38447, 0.38553, 0.38688, 0.38853, 0.39044,\
        0.39254, 0.39471, 0.39698, 0.39981, 0.40367, 0.40874, 0.41463, 0.42110, 0.42798, 0.43512, 0.44240,\
        0.44970, 0.45689, 0.46384, 0.47040, 0.47639, 0.48159, 0.48571, 0.48852, 0.49005, 0.49065, 0.49077,\
        0.49073, 0.49065, 0.49042, 0.48997, 0.48937, 0.48861, 0.48772, 0.48675, 0.48576, 0.48479, 0.48389,\
        0.48319, 0.48278, 0.48276, 0.48321, 0.48417, 0.48566, 0.48759, 0.48975, 0.49190, 0.49393, 0.49591,\
        0.49791, 0.49986, 0.50123, 0.50191, 0.50217, 0.50218, 0.50169, 0.50010, 0.49811, 0.49611, 0.49404,\
        0.49195, 0.48994, 0.48814, 0.48665, 0.48553, 0.48478, 0.48431, 0.48395, 0.48355, 0.48295, 0.48208,\
        0.48093, 0.47955, 0.47801, 0.47637, 0.47467, 0.47293, 0.47115, 0.46932, 0.46740, 0.46538, 0.46323,\
        0.46095, 0.45852, 0.45595, 0.45326, 0.45045, 0.44755, 0.44456, 0.44152, 0.43844, 0.43532, 0.43221,\
        0.42910, 0.42602, 0.42301, 0.42008, 0.41730, 0.41474, 0.41248, 0.41053, 0.40865, 0.40659, 0.40403,\
        0.40042, 0.39591, 0.39097, 0.38555, 0.37964, 0.37327, 0.36651, 0.35945, 0.35213, 0.34458, 0.33680],\
        [0.37591, 0.37735, 0.37854, 0.37960, 0.38066, 0.38181, 0.38313, 0.38468, 0.38652, 0.38865, 0.39106,\
        0.39366, 0.39628, 0.39854, 0.40116, 0.40473, 0.40923, 0.41454, 0.42047, 0.42683, 0.43349, 0.44030,\
        0.44715, 0.45388, 0.46038, 0.46648, 0.47202, 0.47680, 0.48062, 0.48337, 0.48508, 0.48599, 0.48638,\
        0.48649, 0.48643, 0.48621, 0.48581, 0.48527, 0.48459, 0.48382, 0.48298, 0.48214, 0.48135, 0.48066,\
        0.48016, 0.47994, 0.48006, 0.48059, 0.48154, 0.48290, 0.48459, 0.48646, 0.48838, 0.49026, 0.49211,\
        0.49393, 0.49560, 0.49680, 0.49742, 0.49768, 0.49772, 0.49732, 0.49598, 0.49417, 0.49228, 0.49036,\
        0.48845, 0.48663, 0.48498, 0.48357, 0.48244, 0.48158, 0.48091, 0.48034, 0.47973, 0.47899, 0.47806,\
        0.47691, 0.47556, 0.47405, 0.47242, 0.47071, 0.46893, 0.46708, 0.46516, 0.46316, 0.46105, 0.45883,\
        0.45648, 0.45400, 0.45139, 0.44867, 0.44584, 0.44291, 0.43990, 0.43683, 0.43371, 0.43055, 0.42738,\
        0.42421, 0.42105, 0.41794, 0.41490, 0.41199, 0.40927, 0.40690, 0.40507, 0.40332, 0.40131, 0.39888,\
        0.39532, 0.39076, 0.38568, 0.38014, 0.37414, 0.36772, 0.36092, 0.35379, 0.34638, 0.33870, 0.33078],\
        [0.37134, 0.37324, 0.37494, 0.37652, 0.37809, 0.37975, 0.38156, 0.38359, 0.38588, 0.38847, 0.39133,\
        0.39443, 0.39763, 0.40036, 0.40247, 0.40565, 0.40968, 0.41445, 0.41982, 0.42565, 0.43180, 0.43811,\
        0.44446, 0.45072, 0.45674, 0.46238, 0.46748, 0.47189, 0.47548, 0.47817, 0.48002, 0.48116, 0.48178,\
        0.48205, 0.48209, 0.48193, 0.48161, 0.48115, 0.48058, 0.47993, 0.47923, 0.47855, 0.47792, 0.47741,\
        0.47708, 0.47700, 0.47723, 0.47779, 0.47870, 0.47994, 0.48143, 0.48307, 0.48479, 0.48651, 0.48819,\
        0.48980, 0.49121, 0.49225, 0.49287, 0.49315, 0.49312, 0.49264, 0.49156, 0.49003, 0.48833, 0.48656,\
        0.48481, 0.48314, 0.48160, 0.48024, 0.47909, 0.47813, 0.47732, 0.47656, 0.47579, 0.47493, 0.47391,\
        0.47273, 0.47137, 0.46987, 0.46824, 0.46651, 0.46469, 0.46279, 0.46081, 0.45873, 0.45655, 0.45426,\
        0.45185, 0.44932, 0.44668, 0.44392, 0.44106, 0.43810, 0.43507, 0.43196, 0.42881, 0.42561, 0.42239,\
        0.41916, 0.41593, 0.41273, 0.40959, 0.40655, 0.40366, 0.40106, 0.39924, 0.39750, 0.39536, 0.39271,\
        0.38918, 0.38477, 0.37971, 0.37413, 0.36810, 0.36165, 0.35481, 0.34764, 0.34015, 0.33238, 0.32435]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=25 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax25vt5
    zgrid_H_Hmax_vmax25vt5=[[0.293382, 0.295214, 0.296795, 0.298151, 0.299214, 0.299620, 0.299339, 0.299038, 0.298749, 0.298470,\
        0.298209, 0.297976, 0.297777, 0.297619, 0.297506, 0.297439, 0.297424, 0.297463, 0.297561, 0.297720,\
        0.297940, 0.298217, 0.298539, 0.298883, 0.299223, 0.299529, 0.299777, 0.299928, 0.299621, 0.299164,\
        0.299141, 0.299962, 0.301969, 0.305067, 0.309058, 0.313790, 0.319120, 0.324918, 0.331056, 0.337416,\
        0.343892, 0.350386, 0.356822, 0.363142, 0.369311, 0.375303, 0.381079, 0.386544, 0.391554, 0.396310,\
        0.401581, 0.406928, 0.412280, 0.417566, 0.422744, 0.427782, 0.432656, 0.437342, 0.441821, 0.446071,\
        0.450072, 0.453808, 0.457272, 0.460473, 0.463437, 0.466199, 0.468781, 0.471169, 0.473318, 0.475184,\
        0.476750, 0.478032, 0.479073, 0.479946, 0.480743, 0.481535, 0.482324, 0.483098, 0.483830, 0.484322,\
        0.484357, 0.483952, 0.483224, 0.482181, 0.480897, 0.479586, 0.478355, 0.477138, 0.475811, 0.474281,\
        0.472491, 0.470415, 0.468055, 0.465438, 0.462602, 0.459583, 0.456407, 0.453090, 0.449639, 0.446054,\
        0.442331, 0.438462, 0.434436, 0.430237, 0.425845, 0.421237, 0.416392, 0.411291, 0.405918, 0.400261,\
        0.394311, 0.388061, 0.381506, 0.374643, 0.367470, 0.359988, 0.352194, 0.344092, 0.335681, 0.326964,\
        0.317943],\
        [0.295740, 0.297143, 0.298302, 0.299274, 0.299971, 0.300063, 0.299882, 0.299703, 0.299503, 0.299290,\
        0.299080, 0.298887, 0.298721, 0.298585, 0.298482, 0.298414, 0.298385, 0.298398, 0.298458, 0.298572,\
        0.298742, 0.298965, 0.299230, 0.299508, 0.299756, 0.299932, 0.300001, 0.299910, 0.299511, 0.299010,\
        0.298964, 0.299838, 0.302066, 0.305489, 0.309870, 0.315030, 0.320812, 0.327071, 0.333668, 0.340476,\
        0.347373, 0.354253, 0.361028, 0.367639, 0.374061, 0.380300, 0.386360, 0.392146, 0.397491, 0.401928,\
        0.407234, 0.412778, 0.418278, 0.423692, 0.428987, 0.434135, 0.439112, 0.443898, 0.448470, 0.452807,\
        0.456881, 0.460670, 0.464155, 0.467343, 0.470275, 0.473021, 0.475631, 0.478089, 0.480310, 0.482211,\
        0.483768, 0.485001, 0.485957, 0.486720, 0.487425, 0.488216, 0.489034, 0.489826, 0.490776, 0.491533,\
        0.491588, 0.491090, 0.490405, 0.489316, 0.487835, 0.486475, 0.485388, 0.484369, 0.483196, 0.481775,\
        0.480048, 0.477984, 0.475597, 0.472934, 0.470053, 0.467000, 0.463809, 0.460496, 0.457064, 0.453512,\
        0.449834, 0.446024, 0.442071, 0.437955, 0.433651, 0.429132, 0.424371, 0.419347, 0.414042, 0.408446,\
        0.402548, 0.396344, 0.389828, 0.383000, 0.375857, 0.368400, 0.360630, 0.352547, 0.344155, 0.335455,\
        0.326449],\
        [0.297639, 0.298504, 0.299146, 0.299674, 0.300012, 0.300068, 0.300007, 0.299969, 0.299901, 0.299794,\
        0.299674, 0.299558, 0.299454, 0.299366, 0.299292, 0.299235, 0.299197, 0.299183, 0.299199, 0.299253,\
        0.299353, 0.299500, 0.299686, 0.299876, 0.300009, 0.300009, 0.299843, 0.299515, 0.299044, 0.298640,\
        0.298734, 0.299822, 0.302270, 0.306003, 0.310766, 0.316352, 0.322585, 0.329305, 0.336365, 0.343625,\
        0.350950, 0.358218, 0.365325, 0.372201, 0.378827, 0.385225, 0.391496, 0.397617, 0.403152, 0.408196,\
        0.413373, 0.418869, 0.424421, 0.429912, 0.435290, 0.440522, 0.445584, 0.450454, 0.455112, 0.459532,\
        0.463683, 0.467529, 0.471031, 0.474181, 0.477032, 0.479707, 0.482325, 0.484889, 0.487219, 0.489157,\
        0.490709, 0.491901, 0.492778, 0.493420, 0.493967, 0.494788, 0.495698, 0.496267, 0.497440, 0.498574,\
        0.498607, 0.497902, 0.497433, 0.496349, 0.494513, 0.493161, 0.492348, 0.491590, 0.490546, 0.489217,\
        0.487543, 0.485467, 0.483019, 0.480281, 0.477334, 0.474235, 0.471021, 0.467711, 0.464298, 0.460774,\
        0.457136, 0.453383, 0.449506, 0.445480, 0.441272, 0.436847, 0.432173, 0.427225, 0.421986, 0.416445,\
        0.410595, 0.404431, 0.397950, 0.391151, 0.384033, 0.376598, 0.368846, 0.360779, 0.352401, 0.343714,\
        0.334721],\
        [0.299154, 0.299368, 0.299495, 0.299662, 0.299790, 0.299853, 0.299921, 0.300023, 0.300085, 0.300097,\
        0.300091, 0.300081, 0.300066, 0.300045, 0.300016, 0.299978, 0.299935, 0.299892, 0.299854, 0.299833,\
        0.299839, 0.299881, 0.299960, 0.300044, 0.300047, 0.299881, 0.299537, 0.299084, 0.298598, 0.298302,\
        0.298571, 0.299849, 0.302611, 0.306678, 0.311815, 0.317814, 0.324483, 0.331653, 0.339166, 0.346875,\
        0.354633, 0.362296, 0.369732, 0.376851, 0.383641, 0.390112, 0.396300, 0.402652, 0.408557, 0.414069,\
        0.419516, 0.425062, 0.430650, 0.436192, 0.441629, 0.446924, 0.452052, 0.456993, 0.461726, 0.466228,\
        0.470464, 0.474385, 0.477923, 0.481020, 0.483726, 0.486241, 0.488808, 0.491527, 0.494032, 0.495975,\
        0.497523, 0.498707, 0.499530, 0.500132, 0.500521, 0.501338, 0.502353, 0.502839, 0.503916, 0.505044,\
        0.505198, 0.504689, 0.504194, 0.503011, 0.501192, 0.499918, 0.499437, 0.498862, 0.497857, 0.496598,\
        0.494963, 0.492846, 0.490299, 0.487466, 0.484444, 0.481287, 0.478048, 0.474744, 0.471352, 0.467848,\
        0.464238, 0.460537, 0.456740, 0.452815, 0.448714, 0.444390, 0.439804, 0.434929, 0.429751, 0.424260,\
        0.418450, 0.412320, 0.405868, 0.399093, 0.391995, 0.384576, 0.376837, 0.368782, 0.360413, 0.351736,\
        0.342753],\
        [0.300208, 0.299806, 0.299592, 0.299510, 0.299476, 0.299513, 0.299681, 0.299946, 0.300153, 0.300302,\
        0.300440, 0.300562, 0.300656, 0.300717, 0.300742, 0.300730, 0.300684, 0.300608, 0.300510, 0.300399,\
        0.300287, 0.300189, 0.300120, 0.300073, 0.299933, 0.299645, 0.299266, 0.298812, 0.298364, 0.298164,\
        0.298624, 0.300204, 0.303200, 0.307569, 0.313068, 0.319461, 0.326542, 0.334132, 0.342072, 0.350213,\
        0.358403, 0.366478, 0.374263, 0.381610, 0.388521, 0.395202, 0.401412, 0.407680, 0.413882, 0.419777,\
        0.425514, 0.431209, 0.436877, 0.442478, 0.447967, 0.453313, 0.458492, 0.463489, 0.468288, 0.472865,\
        0.477188, 0.481215, 0.484844, 0.487916, 0.490418, 0.492678, 0.495055, 0.497899, 0.500660, 0.502582,\
        0.504131, 0.505329, 0.506195, 0.506833, 0.507374, 0.508122, 0.508985, 0.509737, 0.510629, 0.511448,\
        0.511719, 0.511467, 0.510872, 0.509797, 0.508431, 0.507341, 0.506733, 0.506101, 0.505132, 0.503870,\
        0.502221, 0.500056, 0.497401, 0.494485, 0.491396, 0.488166, 0.484895, 0.481617, 0.478249, 0.474740,\
        0.471132, 0.467476, 0.463772, 0.459966, 0.455987, 0.451770, 0.447270, 0.442463, 0.437336, 0.431885,\
        0.426109, 0.420006, 0.413576, 0.406819, 0.399736, 0.392328, 0.384597, 0.376549, 0.368187, 0.359515,\
        0.350538],\
        [0.301060, 0.300412, 0.299863, 0.299524, 0.299254, 0.299181, 0.299397, 0.299793, 0.300173, 0.300525,\
        0.300846, 0.301118, 0.301332, 0.301482, 0.301564, 0.301580, 0.301532, 0.301425, 0.301265, 0.301059,\
        0.300816, 0.300549, 0.300274, 0.300024, 0.299766, 0.299507, 0.299223, 0.298869, 0.298482, 0.298309,\
        0.298846, 0.300661, 0.303972, 0.308699, 0.314573, 0.321337, 0.328786, 0.336747, 0.345065, 0.353598,\
        0.362201, 0.370706, 0.378903, 0.386520, 0.393394, 0.400051, 0.406692, 0.413037, 0.419314, 0.425430,\
        0.431395, 0.437259, 0.443040, 0.448721, 0.454270, 0.459661, 0.464878, 0.469917, 0.474771, 0.479409,\
        0.483800, 0.487936, 0.491741, 0.494902, 0.497191, 0.499215, 0.501454, 0.504140, 0.506861, 0.508960,\
        0.510587, 0.511853, 0.512822, 0.513589, 0.514283, 0.515041, 0.515856, 0.516664, 0.517459, 0.518110,\
        0.518406, 0.518286, 0.517785, 0.516939, 0.515914, 0.514980, 0.514218, 0.513423, 0.512386, 0.511027,\
        0.509268, 0.507033, 0.504352, 0.501389, 0.498239, 0.494891, 0.491583, 0.488379, 0.485029, 0.481454,\
        0.477800, 0.474185, 0.470599, 0.466945, 0.463106, 0.459000, 0.454579, 0.449826, 0.444737, 0.439315,\
        0.433562, 0.427479, 0.421066, 0.414322, 0.407248, 0.399847, 0.392121, 0.384075, 0.375715, 0.367045,\
        0.358072],\
        [0.302377, 0.301539, 0.300686, 0.299954, 0.299303, 0.299021, 0.299275, 0.299850, 0.300401, 0.300963,\
        0.301464, 0.301883, 0.302210, 0.302442, 0.302577, 0.302619, 0.302570, 0.302434, 0.302219, 0.301929,\
        0.301570, 0.301149, 0.300674, 0.300178, 0.299906, 0.299719, 0.299595, 0.299402, 0.299084, 0.298857,\
        0.299175, 0.301201, 0.304990, 0.310162, 0.316404, 0.323491, 0.331239, 0.339493, 0.348111, 0.356960,\
        0.365911, 0.374824, 0.383507, 0.391608, 0.398683, 0.405125, 0.411799, 0.418334, 0.424738, 0.431016,\
        0.437164, 0.443193, 0.449106, 0.454887, 0.460510, 0.465947, 0.471190, 0.476253, 0.481155, 0.485852,\
        0.490262, 0.494422, 0.498391, 0.501717, 0.504039, 0.506081, 0.508301, 0.510755, 0.513212, 0.515325,\
        0.517031, 0.518393, 0.519484, 0.520390, 0.521210, 0.522025, 0.522851, 0.523661, 0.524407, 0.524994,\
        0.525309, 0.525290, 0.524939, 0.524305, 0.523501, 0.522655, 0.521799, 0.520843, 0.519661, 0.518159,\
        0.516270, 0.513956, 0.511248, 0.508236, 0.504986, 0.501539, 0.498184, 0.495122, 0.491726, 0.487953,\
        0.484196, 0.480640, 0.477236, 0.473780, 0.470098, 0.466095, 0.461734, 0.457013, 0.451943, 0.446536,\
        0.440798, 0.434730, 0.428329, 0.421594, 0.414525, 0.407125, 0.399398, 0.391351, 0.382989, 0.374319,\
        0.365346],\
        [0.304205, 0.303179, 0.302036, 0.300887, 0.299803, 0.299303, 0.299561, 0.300338, 0.301102, 0.301816,\
        0.302452, 0.302983, 0.303398, 0.303693, 0.303870, 0.303930, 0.303879, 0.303722, 0.303465, 0.303115,\
        0.302680, 0.302173, 0.301623, 0.301111, 0.300762, 0.300558, 0.300496, 0.300407, 0.300212, 0.300087,\
        0.299929, 0.302300, 0.306629, 0.312164, 0.318671, 0.325972, 0.333915, 0.342359, 0.351171, 0.360225,\
        0.369397, 0.378571, 0.387642, 0.396341, 0.403750, 0.410300, 0.416871, 0.423475, 0.430019, 0.436464,\
        0.442789, 0.448986, 0.455046, 0.460951, 0.466672, 0.472169, 0.477423, 0.482476, 0.487415, 0.492221,\
        0.496640, 0.500729, 0.504673, 0.508061, 0.510715, 0.512997, 0.515251, 0.517564, 0.519825, 0.521861,\
        0.523593, 0.525031, 0.526228, 0.527254, 0.528183, 0.529068, 0.529929, 0.530750, 0.531486, 0.532069,\
        0.532425, 0.532502, 0.532292, 0.531824, 0.531158, 0.530355, 0.529428, 0.528331, 0.526988, 0.525330,\
        0.523308, 0.520904, 0.518139, 0.515070, 0.511763, 0.508320, 0.504975, 0.501827, 0.498181, 0.494143,\
        0.490264, 0.486842, 0.483736, 0.480524, 0.476991, 0.473064, 0.468727, 0.464006, 0.458933, 0.453529,\
        0.447801, 0.441744, 0.435353, 0.428623, 0.421555, 0.414152, 0.406420, 0.398367, 0.390000, 0.381327,\
        0.372353],\
        [0.306583, 0.305407, 0.304060, 0.302656, 0.301391, 0.300647, 0.300706, 0.301554, 0.302428, 0.303233,\
        0.303940, 0.304529, 0.304989, 0.305318, 0.305516, 0.305585, 0.305530, 0.305358, 0.305076, 0.304695,\
        0.304229, 0.303702, 0.303157, 0.302664, 0.302297, 0.302075, 0.301976, 0.301920, 0.301884, 0.301969,\
        0.302584, 0.304995, 0.309290, 0.314875, 0.321443, 0.328804, 0.336811, 0.345326, 0.354218, 0.363354,\
        0.372594, 0.381793, 0.390803, 0.399554, 0.407465, 0.414590, 0.421479, 0.428299, 0.435050, 0.441702,\
        0.448227, 0.454607, 0.460830, 0.466887, 0.472746, 0.478342, 0.483611, 0.488592, 0.493499, 0.498444,\
        0.503019, 0.507118, 0.510985, 0.514416, 0.517322, 0.519852, 0.522207, 0.524478, 0.526637, 0.528608,\
        0.530337, 0.531823, 0.533100, 0.534219, 0.535235, 0.536188, 0.537096, 0.537946, 0.538703, 0.539316,\
        0.539731, 0.539902, 0.539812, 0.539464, 0.538883, 0.538092, 0.537096, 0.535870, 0.534372, 0.532561,\
        0.530405, 0.527899, 0.525061, 0.521938, 0.518596, 0.515133, 0.511660, 0.508126, 0.504226, 0.500060,\
        0.496023, 0.492880, 0.490242, 0.487237, 0.483799, 0.479897, 0.475531, 0.470769, 0.465673, 0.460268,\
        0.454550, 0.448507, 0.442126, 0.435400, 0.428329, 0.420918, 0.413177, 0.405114, 0.396739, 0.388060,\
        0.379084],\
        [0.309544, 0.308298, 0.306885, 0.305449, 0.304202, 0.303410, 0.303313, 0.303836, 0.304581, 0.305333,\
        0.306015, 0.306593, 0.307049, 0.307376, 0.307570, 0.307634, 0.307572, 0.307391, 0.307100, 0.306713,\
        0.306250, 0.305740, 0.305228, 0.304770, 0.304417, 0.304195, 0.304101, 0.304122, 0.304298, 0.304803,\
        0.306063, 0.308675, 0.312814, 0.318237, 0.324682, 0.331953, 0.339894, 0.348367, 0.357234, 0.366352,\
        0.375567, 0.384700, 0.393523, 0.401944, 0.410361, 0.418155, 0.425537, 0.432716, 0.439765, 0.446682,\
        0.453445, 0.460029, 0.466428, 0.472654, 0.478700, 0.484483, 0.489839, 0.494720, 0.499486, 0.504489,\
        0.509209, 0.513488, 0.517403, 0.520919, 0.524007, 0.526733, 0.529208, 0.531502, 0.533630, 0.535568,\
        0.537294, 0.538809, 0.540138, 0.541322, 0.542404, 0.543417, 0.544373, 0.545262, 0.546059, 0.546722,\
        0.547206, 0.547468, 0.547476, 0.547215, 0.546680, 0.545876, 0.544803, 0.543453, 0.541807, 0.539845,\
        0.537556, 0.534941, 0.532020, 0.528831, 0.525430, 0.521877, 0.518211, 0.514381, 0.510295, 0.506019,\
        0.501803, 0.499201, 0.496913, 0.493877, 0.490482, 0.486550, 0.482081, 0.477240, 0.472118, 0.466721,\
        0.461026, 0.455005, 0.448637, 0.441913, 0.434834, 0.427411, 0.419655, 0.411578, 0.403193, 0.394508,\
        0.385530],\
        [0.313086, 0.311845, 0.310463, 0.309089, 0.307909, 0.307121, 0.306858, 0.307072, 0.307561, 0.308142,\
        0.308710, 0.309210, 0.309611, 0.309899, 0.310066, 0.310110, 0.310035, 0.309848, 0.309559, 0.309184,\
        0.308744, 0.308272, 0.307808, 0.307398, 0.307085, 0.306901, 0.306869, 0.307022, 0.307450, 0.308342,\
        0.310012, 0.312793, 0.316829, 0.322040, 0.328263, 0.335332, 0.343100, 0.351433, 0.360191, 0.369227,\
        0.378381, 0.387496, 0.396450, 0.405024, 0.413331, 0.421398, 0.429176, 0.436741, 0.444145, 0.451387,\
        0.458434, 0.465248, 0.471818, 0.478190, 0.484443, 0.490521, 0.496142, 0.501068, 0.505842, 0.510749,\
        0.515502, 0.519914, 0.523951, 0.527596, 0.530847, 0.533737, 0.536329, 0.538680, 0.540821, 0.542758,\
        0.544490, 0.546025, 0.547384, 0.548606, 0.549731, 0.550788, 0.551787, 0.552718, 0.553559, 0.554278,\
        0.554834, 0.555180, 0.555271, 0.555071, 0.554554, 0.553712, 0.552547, 0.551067, 0.549273, 0.547164,\
        0.544741, 0.542012, 0.538998, 0.535731, 0.532250, 0.528592, 0.524778, 0.520798, 0.516662, 0.512499,\
        0.508726, 0.505926, 0.503331, 0.500282, 0.496958, 0.492918, 0.488255, 0.483341, 0.478225, 0.472868,\
        0.467217, 0.461230, 0.454878, 0.448151, 0.441058, 0.433615, 0.425840, 0.417748, 0.409351, 0.400660,\
        0.391682],\
        [0.317162, 0.315978, 0.314672, 0.313384, 0.312266, 0.311460, 0.311050, 0.311013, 0.311238, 0.311599,\
        0.311997, 0.312369, 0.312676, 0.312895, 0.313013, 0.313025, 0.312932, 0.312740, 0.312459, 0.312106,\
        0.311703, 0.311280, 0.310874, 0.310525, 0.310274, 0.310160, 0.310227, 0.310535, 0.311192, 0.312368,\
        0.314286, 0.317158, 0.321095, 0.326085, 0.332035, 0.338829, 0.346343, 0.354457, 0.363042, 0.371951,\
        0.381019, 0.390074, 0.398984, 0.407678, 0.416122, 0.424381, 0.432470, 0.440405, 0.448195, 0.455816,\
        0.463209, 0.470297, 0.477027, 0.483454, 0.489810, 0.496219, 0.502129, 0.507338, 0.512281, 0.517194,\
        0.521978, 0.526497, 0.530676, 0.534481, 0.537898, 0.540941, 0.543650, 0.546073, 0.548252, 0.550213,\
        0.551965, 0.553515, 0.554884, 0.556116, 0.557258, 0.558343, 0.559374, 0.560336, 0.561212, 0.561981,\
        0.562602, 0.563023, 0.563185, 0.563028, 0.562505, 0.561596, 0.560316, 0.558691, 0.556744, 0.554486,\
        0.551928, 0.549082, 0.545970, 0.542618, 0.539055, 0.535309, 0.531406, 0.527375, 0.523281, 0.519284,\
        0.515648, 0.512509, 0.509542, 0.506404, 0.502961, 0.498689, 0.493904, 0.489025, 0.483981, 0.478701,\
        0.473121, 0.467179, 0.460840, 0.454102, 0.446984, 0.439514, 0.431715, 0.423606, 0.415200, 0.406506,\
        0.397531],\
        [0.321707, 0.320604, 0.319387, 0.318179, 0.317102, 0.316264, 0.315725, 0.315480, 0.315468, 0.315604,\
        0.315812, 0.316030, 0.316218, 0.316347, 0.316400, 0.316370, 0.316255, 0.316058, 0.315790, 0.315466,\
        0.315106, 0.314740, 0.314399, 0.314122, 0.313949, 0.313923, 0.314100, 0.314553, 0.315388, 0.316747,\
        0.318794, 0.321677, 0.325484, 0.330232, 0.335875, 0.342338, 0.349532, 0.357359, 0.365711, 0.374457,\
        0.383430, 0.392427, 0.401288, 0.410006, 0.418566, 0.427021, 0.435402, 0.443706, 0.451910, 0.459965,\
        0.467786, 0.475243, 0.482189, 0.488575, 0.494717, 0.501287, 0.507567, 0.513247, 0.518558, 0.523668,\
        0.528589, 0.533258, 0.537613, 0.541608, 0.545211, 0.548416, 0.551243, 0.553743, 0.555977, 0.557984,\
        0.559774, 0.561342, 0.562699, 0.563904, 0.565032, 0.566128, 0.567177, 0.568147, 0.569032, 0.569830,\
        0.570503, 0.570985, 0.571206, 0.571080, 0.570525, 0.569512, 0.568079, 0.566289, 0.564182, 0.561775,\
        0.559079, 0.556115, 0.552904, 0.549467, 0.545828, 0.542013, 0.538056, 0.534005, 0.529944, 0.525997,\
        0.522298, 0.518868, 0.515542, 0.512090, 0.508278, 0.503875, 0.499151, 0.494370, 0.489420, 0.484237,\
        0.478746, 0.472854, 0.466515, 0.459746, 0.452589, 0.445084, 0.437260, 0.429136, 0.420725, 0.412033,\
        0.403067],\
        [0.326644, 0.325629, 0.324498, 0.323354, 0.322300, 0.321421, 0.320768, 0.320347, 0.320127, 0.320055,\
        0.320074, 0.320132, 0.320190, 0.320219, 0.320201, 0.320124, 0.319984, 0.319784, 0.319531, 0.319240,\
        0.318929, 0.318624, 0.318354, 0.318155, 0.318066, 0.318133, 0.318413, 0.318982, 0.319936, 0.321392,\
        0.323474, 0.326294, 0.329926, 0.334397, 0.339695, 0.345777, 0.352586, 0.360055, 0.368105, 0.376635,\
        0.385506, 0.394497, 0.403323, 0.411971, 0.420590, 0.429243, 0.437929, 0.446618, 0.455263, 0.463800,\
        0.472136, 0.480115, 0.487488, 0.494013, 0.499918, 0.506394, 0.512979, 0.519082, 0.524772, 0.530167,\
        0.535313, 0.540196, 0.544778, 0.549010, 0.552840, 0.556229, 0.559180, 0.561757, 0.564052, 0.566130,\
        0.567992, 0.569590, 0.570908, 0.572033, 0.573104, 0.574198, 0.575261, 0.576196, 0.577032, 0.577832,\
        0.578534, 0.579049, 0.579311, 0.579211, 0.578600, 0.577426, 0.575786, 0.573808, 0.571541, 0.568985,\
        0.566151, 0.563066, 0.559758, 0.556243, 0.552535, 0.548659, 0.544652, 0.540570, 0.536488, 0.532492,\
        0.528647, 0.524944, 0.521278, 0.517486, 0.513404, 0.508957, 0.504273, 0.499486, 0.494582, 0.489487,\
        0.484106, 0.478254, 0.471881, 0.465050, 0.457838, 0.450296, 0.442452, 0.434321, 0.425914, 0.417234,\
        0.408285],\
        [0.331897, 0.330970, 0.329912, 0.328815, 0.327768, 0.326843, 0.326086, 0.325514, 0.325115, 0.324857,\
        0.324700, 0.324605, 0.324536, 0.324466, 0.324376, 0.324252, 0.324089, 0.323887, 0.323653, 0.323398,\
        0.323139, 0.322899, 0.322704, 0.322585, 0.322580, 0.322733, 0.323100, 0.323750, 0.324767, 0.326246,\
        0.328288, 0.330980, 0.334385, 0.338538, 0.343446, 0.349093, 0.355449, 0.362478, 0.370131, 0.378345,\
        0.387023, 0.395984, 0.404753, 0.413343, 0.422077, 0.430983, 0.440022, 0.449127, 0.458229, 0.467258,\
        0.476141, 0.484759, 0.492820, 0.499870, 0.506014, 0.512320, 0.518805, 0.525094, 0.531071, 0.536752,\
        0.542165, 0.547315, 0.552180, 0.556712, 0.560833, 0.564455, 0.567543, 0.570180, 0.572533, 0.574716,\
        0.576709, 0.578378, 0.579622, 0.580579, 0.581530, 0.582618, 0.583728, 0.584547, 0.585218, 0.586007,\
        0.586711, 0.587188, 0.587456, 0.587390, 0.586702, 0.585270, 0.583350, 0.581182, 0.578777, 0.576075,\
        0.573092, 0.569885, 0.566488, 0.562904, 0.559132, 0.555192, 0.551121, 0.546976, 0.542820, 0.538713,\
        0.534688, 0.530728, 0.526762, 0.522685, 0.518402, 0.513884, 0.509185, 0.504372, 0.499436, 0.494447,\
        0.489217, 0.483359, 0.476879, 0.469953, 0.462685, 0.455117, 0.447269, 0.439147, 0.430757, 0.422101,\
        0.413180],\
        [0.337397, 0.336549, 0.335551, 0.334485, 0.333431, 0.332454, 0.331602, 0.330898, 0.330344, 0.329924,\
        0.329612, 0.329378, 0.329194, 0.329033, 0.328877, 0.328711, 0.328529, 0.328330, 0.328118, 0.327903,\
        0.327700, 0.327528, 0.327409, 0.327371, 0.327446, 0.327674, 0.328106, 0.328802, 0.329832, 0.331277,\
        0.333216, 0.335722, 0.338852, 0.342644, 0.347116, 0.352272, 0.358106, 0.364606, 0.371751, 0.379504,\
        0.387792, 0.396457, 0.405178, 0.413948, 0.422959, 0.432230, 0.441695, 0.451257, 0.460821, 0.470302,\
        0.479642, 0.488801, 0.497590, 0.505114, 0.511765, 0.518292, 0.524849, 0.531297, 0.537517, 0.543473,\
        0.549172, 0.554622, 0.559818, 0.564719, 0.569229, 0.573177, 0.576424, 0.579090, 0.581484, 0.583805,\
        0.586018, 0.587851, 0.588986, 0.589648, 0.590407, 0.591436, 0.592731, 0.593280, 0.593569, 0.594424,\
        0.595084, 0.595377, 0.595568, 0.595529, 0.594763, 0.592903, 0.590647, 0.588352, 0.585865, 0.583003,\
        0.579839, 0.576510, 0.573046, 0.569410, 0.565576, 0.561557, 0.557399, 0.553157, 0.548888, 0.544635,\
        0.540418, 0.536220, 0.531994, 0.527677, 0.523214, 0.518591, 0.513831, 0.508970, 0.504040, 0.499161,\
        0.494088, 0.488054, 0.481375, 0.474367, 0.467080, 0.459522, 0.451696, 0.443605, 0.435251, 0.426633,\
        0.417752],\
        [0.343079, 0.342299, 0.341345, 0.340293, 0.339219, 0.338186, 0.337244, 0.336423, 0.335735, 0.335177,\
        0.334734, 0.334382, 0.334099, 0.333862, 0.333651, 0.333454, 0.333261, 0.333070, 0.332886, 0.332716,\
        0.332572, 0.332471, 0.332430, 0.332471, 0.332622, 0.332914, 0.333390, 0.334100, 0.335103, 0.336465,\
        0.338252, 0.340526, 0.343338, 0.346727, 0.350720, 0.355336, 0.360585, 0.366476, 0.373013, 0.380183,\
        0.387943, 0.396194, 0.404861, 0.413908, 0.423318, 0.433059, 0.443031, 0.453101, 0.463136, 0.473023,\
        0.482668, 0.492021, 0.501065, 0.509387, 0.516874, 0.523969, 0.530877, 0.537607, 0.544110, 0.550357,\
        0.556353, 0.562122, 0.567678, 0.573002, 0.578020, 0.582474, 0.585937, 0.588588, 0.591018, 0.593509,\
        0.595991, 0.598075, 0.599123, 0.599455, 0.600041, 0.600819, 0.602145, 0.602429, 0.602365, 0.603153,\
        0.603634, 0.603697, 0.603660, 0.603401, 0.602406, 0.600118, 0.597629, 0.595329, 0.592819, 0.589706,\
        0.586300, 0.582876, 0.579396, 0.575734, 0.571828, 0.567708, 0.563433, 0.559065, 0.554657, 0.550242,\
        0.545832, 0.541415, 0.536960, 0.532428, 0.527788, 0.523030, 0.518170, 0.513240, 0.508309, 0.503482,\
        0.498463, 0.491909, 0.485175, 0.478226, 0.471004, 0.463504, 0.455734, 0.447698, 0.439398, 0.430834,\
        0.422004],\
        [0.348880, 0.348156, 0.347230, 0.346176, 0.345069, 0.343974, 0.342944, 0.342018, 0.341217, 0.340544,\
        0.339993, 0.339548, 0.339189, 0.338894, 0.338646, 0.338429, 0.338236, 0.338062, 0.337912, 0.337792,\
        0.337713, 0.337686, 0.337726, 0.337847, 0.338070, 0.338416, 0.338919, 0.339618, 0.340562, 0.341803,\
        0.343400, 0.345405, 0.347865, 0.350821, 0.354303, 0.358339, 0.362955, 0.368177, 0.374035, 0.380557,\
        0.387760, 0.395639, 0.404228, 0.413495, 0.423331, 0.433618, 0.444175, 0.454810, 0.465352, 0.475664,\
        0.485623, 0.495107, 0.504401, 0.513396, 0.521692, 0.529452, 0.536863, 0.543997, 0.550857, 0.557431,\
        0.563737, 0.569833, 0.575761, 0.581503, 0.587071, 0.592314, 0.596152, 0.598853, 0.601445, 0.604171,\
        0.606776, 0.608847, 0.609953, 0.610369, 0.610763, 0.611304, 0.611936, 0.612113, 0.612060, 0.612262,\
        0.612356, 0.612162, 0.611734, 0.610953, 0.609483, 0.607209, 0.604631, 0.602191, 0.599577, 0.595993,\
        0.592368, 0.588945, 0.585541, 0.581864, 0.577851, 0.573592, 0.569171, 0.564656, 0.560096, 0.555516,\
        0.550921, 0.546303, 0.541640, 0.536908, 0.532087, 0.527170, 0.522161, 0.517071, 0.511905, 0.506572,\
        0.500638, 0.494622, 0.488325, 0.481591, 0.474495, 0.467090, 0.459402, 0.451441, 0.443212, 0.434713,\
        0.425946],\
        [0.354744, 0.354062, 0.353147, 0.352073, 0.350918, 0.349753, 0.348638, 0.347617, 0.346718, 0.345953,\
        0.345320, 0.344809, 0.344400, 0.344072, 0.343807, 0.343588, 0.343406, 0.343259, 0.343150, 0.343086,\
        0.343077, 0.343131, 0.343256, 0.343461, 0.343755, 0.344151, 0.344669, 0.345339, 0.346198, 0.347292,\
        0.348670, 0.350381, 0.352469, 0.354971, 0.357923, 0.361358, 0.365312, 0.369831, 0.374969, 0.380801,\
        0.387429, 0.394983, 0.403481, 0.412877, 0.423173, 0.434083, 0.445303, 0.456566, 0.467661, 0.478457,\
        0.488903, 0.498913, 0.508480, 0.517791, 0.526619, 0.534954, 0.542892, 0.550500, 0.557793, 0.564746,\
        0.571371, 0.577792, 0.584133, 0.590276, 0.596203, 0.602135, 0.606775, 0.610058, 0.613082, 0.615951,\
        0.618487, 0.620403, 0.621521, 0.622013, 0.622243, 0.622409, 0.622488, 0.622340, 0.622036, 0.621718,\
        0.621302, 0.620669, 0.619758, 0.618458, 0.616619, 0.614242, 0.611562, 0.608752, 0.605591, 0.601827,\
        0.598136, 0.594757, 0.591548, 0.587775, 0.583577, 0.579146, 0.574560, 0.569891, 0.565181, 0.560444,\
        0.555680, 0.550877, 0.546021, 0.541098, 0.536094, 0.531001, 0.525811, 0.520509, 0.515049, 0.509321,\
        0.503323, 0.497441, 0.491258, 0.484632, 0.477635, 0.470324, 0.462729, 0.454856, 0.446710, 0.438288,\
        0.429590],\
        [0.360616, 0.359960, 0.359036, 0.357923, 0.356705, 0.355461, 0.354259, 0.353150, 0.352170, 0.351336,\
        0.350650, 0.350102, 0.349673, 0.349340, 0.349081, 0.348879, 0.348724, 0.348613, 0.348553, 0.348552,\
        0.348619, 0.348761, 0.348980, 0.349277, 0.349648, 0.350093, 0.350620, 0.351248, 0.352006, 0.352933,\
        0.354076, 0.355478, 0.357183, 0.359226, 0.361645, 0.364476, 0.367767, 0.371574, 0.375977, 0.381082,\
        0.387042, 0.394066, 0.402349, 0.412109, 0.423018, 0.434643, 0.446597, 0.458547, 0.470226, 0.481492,\
        0.492367, 0.502903, 0.512922, 0.522560, 0.531793, 0.540602, 0.549038, 0.557156, 0.564964, 0.572391,\
        0.579350, 0.586033, 0.592887, 0.599613, 0.605987, 0.612203, 0.617534, 0.621779, 0.625361, 0.628440,\
        0.630932, 0.632701, 0.633711, 0.634105, 0.634109, 0.633885, 0.633481, 0.632892, 0.632157, 0.631321,\
        0.630360, 0.629204, 0.627778, 0.625998, 0.623796, 0.621184, 0.618257, 0.615068, 0.611551, 0.607730,\
        0.603907, 0.600530, 0.597470, 0.593282, 0.588887, 0.584295, 0.579542, 0.574730, 0.569893, 0.565022,\
        0.560105, 0.555132, 0.550096, 0.544991, 0.539807, 0.534536, 0.529165, 0.523676, 0.518038, 0.512235,\
        0.506342, 0.500385, 0.494081, 0.487427, 0.480476, 0.473249, 0.465750, 0.457974, 0.449917, 0.441576,\
        0.432951],\
        [0.366439, 0.365793, 0.364838, 0.363663, 0.362363, 0.361029, 0.359738, 0.358551, 0.357507, 0.356628,\
        0.355918, 0.355367, 0.354951, 0.354645, 0.354420, 0.354257, 0.354144, 0.354080, 0.354076, 0.354142,\
        0.354293, 0.354532, 0.354857, 0.355258, 0.355717, 0.356219, 0.356757, 0.357337, 0.357981, 0.358729,\
        0.359625, 0.360715, 0.362039, 0.363632, 0.365531, 0.367776, 0.370422, 0.373541, 0.377232, 0.381625,\
        0.386896, 0.393319, 0.401427, 0.411578, 0.423121, 0.435475, 0.448200, 0.460900, 0.473212, 0.484905,\
        0.496091, 0.506991, 0.517529, 0.527585, 0.537203, 0.546426, 0.555318, 0.563951, 0.572360, 0.580451,\
        0.587896, 0.594666, 0.601980, 0.609460, 0.616456, 0.623011, 0.628855, 0.633821, 0.637972, 0.641334,\
        0.643840, 0.645455, 0.646261, 0.646426, 0.646130, 0.645506, 0.644629, 0.643547, 0.642304, 0.640930,\
        0.639416, 0.637721, 0.635783, 0.633542, 0.630958, 0.628034, 0.624807, 0.621307, 0.617552, 0.613631,\
        0.609755, 0.606165, 0.602419, 0.598090, 0.593730, 0.588978, 0.584054, 0.579150, 0.574235, 0.569259,\
        0.564205, 0.559072, 0.553865, 0.548584, 0.543228, 0.537789, 0.532256, 0.526617, 0.520861, 0.514995,\
        0.509046, 0.502976, 0.496609, 0.489931, 0.483033, 0.475899, 0.468502, 0.460826, 0.452859, 0.444600,\
        0.436047],\
        [0.372159, 0.371502, 0.370490, 0.369227, 0.367826, 0.366390, 0.365009, 0.363750, 0.362661, 0.361763,\
        0.361062, 0.360544, 0.360181, 0.359938, 0.359781, 0.359680, 0.359625, 0.359617, 0.359672, 0.359811,\
        0.360052, 0.360399, 0.360846, 0.361369, 0.361935, 0.362509, 0.363065, 0.363594, 0.364118, 0.364676,\
        0.365321, 0.366102, 0.367059, 0.368224, 0.369630, 0.371320, 0.373357, 0.375834, 0.378876, 0.382645,\
        0.387360, 0.393354, 0.401273, 0.411682, 0.423688, 0.436667, 0.450141, 0.463660, 0.476757, 0.488942,\
        0.500313, 0.511625, 0.522486, 0.532864, 0.542818, 0.552410, 0.561709, 0.570811, 0.579826, 0.588768,\
        0.597224, 0.604400, 0.611993, 0.619850, 0.627342, 0.634292, 0.640570, 0.646075, 0.650735, 0.654412,\
        0.656947, 0.658356, 0.658866, 0.658726, 0.658091, 0.657058, 0.655709, 0.654116, 0.652341, 0.650423,\
        0.648369, 0.646151, 0.643723, 0.641036, 0.638061, 0.634796, 0.631257, 0.627473, 0.623488, 0.619385,\
        0.615278, 0.611202, 0.606974, 0.602587, 0.598230, 0.593034, 0.588042, 0.583177, 0.578246, 0.573183,\
        0.567994, 0.562703, 0.557328, 0.551880, 0.546361, 0.540768, 0.535092, 0.529324, 0.523457, 0.517490,\
        0.511417, 0.505193, 0.498752, 0.492136, 0.485337, 0.478314, 0.471025, 0.463446, 0.455565, 0.447381,\
        0.438895],\
        [0.377716, 0.377026, 0.375927, 0.374546, 0.373020, 0.371471, 0.370001, 0.368682, 0.367566, 0.366678,\
        0.366021, 0.365577, 0.365311, 0.365176, 0.365122, 0.365113, 0.365130, 0.365184, 0.365300, 0.365512,\
        0.365849, 0.366317, 0.366905, 0.367575, 0.368274, 0.368943, 0.369530, 0.370010, 0.370402, 0.370761,\
        0.371153, 0.371636, 0.372249, 0.373017, 0.373967, 0.375140, 0.376610, 0.378493, 0.380954, 0.384208,\
        0.388531, 0.394314, 0.402186, 0.412499, 0.424686, 0.438116, 0.452257, 0.466606, 0.480690, 0.493917,\
        0.505561, 0.516795, 0.527740, 0.538328, 0.548579, 0.558529, 0.568210, 0.577699, 0.587173, 0.596776,\
        0.606134, 0.614514, 0.622647, 0.630768, 0.638560, 0.645798, 0.652386, 0.658287, 0.663409, 0.667449,\
        0.669994, 0.671069, 0.671186, 0.670731, 0.669776, 0.668346, 0.666533, 0.664433, 0.662129, 0.659686,\
        0.657124, 0.654422, 0.651536, 0.648426, 0.645066, 0.641452, 0.637598, 0.633528, 0.629283, 0.624918,\
        0.620483, 0.615973, 0.611332, 0.606542, 0.601400, 0.596269, 0.591654, 0.586956, 0.582013, 0.576834,\
        0.571487, 0.566027, 0.560485, 0.554876, 0.549207, 0.543475, 0.537672, 0.531789, 0.525817, 0.519743,\
        0.513547, 0.507201, 0.500710, 0.494144, 0.487461, 0.480553, 0.473364, 0.465870, 0.458061, 0.449938,\
        0.441507],\
        [0.383045, 0.382295, 0.381071, 0.379539, 0.377869, 0.376201, 0.374646, 0.373282, 0.372162, 0.371312,\
        0.370736, 0.370411, 0.370292, 0.370315, 0.370411, 0.370524, 0.370631, 0.370748, 0.370920, 0.371203,\
        0.371639, 0.372242, 0.372992, 0.373838, 0.374703, 0.375499, 0.376138, 0.376569, 0.376814, 0.376958,\
        0.377096, 0.377297, 0.377597, 0.378010, 0.378545, 0.379239, 0.380171, 0.381484, 0.383391, 0.386176,\
        0.390182, 0.395773, 0.403483, 0.413602, 0.425782, 0.439525, 0.454231, 0.469276, 0.484088, 0.498132,\
        0.510508, 0.521874, 0.532954, 0.543783, 0.554376, 0.564755, 0.574892, 0.584763, 0.594604, 0.604772,\
        0.614890, 0.624365, 0.633307, 0.641860, 0.649911, 0.657306, 0.664006, 0.670090, 0.675601, 0.680152,\
        0.682700, 0.683174, 0.682828, 0.682182, 0.680992, 0.679184, 0.676935, 0.674353, 0.671541, 0.668611,\
        0.665597, 0.662467, 0.659168, 0.655662, 0.651931, 0.647974, 0.643803, 0.639439, 0.634911, 0.630250,\
        0.625477, 0.620587, 0.615542, 0.610291, 0.604856, 0.599899, 0.595380, 0.590707, 0.585608, 0.580224,\
        0.574679, 0.569036, 0.563328, 0.557569, 0.551763, 0.545909, 0.539997, 0.534019, 0.527959, 0.521803,\
        0.515531, 0.509126, 0.502601, 0.496085, 0.489503, 0.482675, 0.475557, 0.468123, 0.460364, 0.452284,\
        0.443893],\
        [0.388071, 0.387225, 0.385830, 0.384116, 0.382291, 0.380511, 0.378886, 0.377497, 0.376396, 0.375613,\
        0.375152, 0.374991, 0.375074, 0.375316, 0.375616, 0.375890, 0.376102, 0.376280, 0.376499, 0.376845,\
        0.377384, 0.378136, 0.379071, 0.380120, 0.381186, 0.382149, 0.382870, 0.383249, 0.383318, 0.383223,\
        0.383107, 0.383050, 0.383077, 0.383181, 0.383346, 0.383590, 0.383994, 0.384721, 0.386027, 0.388262,\
        0.391862, 0.397288, 0.404629, 0.414350, 0.426477, 0.440504, 0.455751, 0.471425, 0.486646, 0.500852,\
        0.514078, 0.526217, 0.537765, 0.549007, 0.560055, 0.570998, 0.581841, 0.592280, 0.602653, 0.613429,\
        0.624116, 0.634312, 0.643921, 0.652931, 0.661228, 0.668659, 0.675212, 0.681116, 0.686690, 0.691947,\
        0.694645, 0.694125, 0.693421, 0.692912, 0.691572, 0.689372, 0.686773, 0.683753, 0.680455, 0.677104,\
        0.673725, 0.670239, 0.666576, 0.662704, 0.658619, 0.654329, 0.649846, 0.645186, 0.640368, 0.635411,\
        0.630328, 0.625122, 0.619792, 0.614373, 0.609024, 0.604026, 0.599407, 0.594438, 0.588962, 0.583296,\
        0.577535, 0.571713, 0.565848, 0.559952, 0.554026, 0.548068, 0.542069, 0.536017, 0.529899, 0.523698,\
        0.517402, 0.511013, 0.504564, 0.498112, 0.491509, 0.484703, 0.477624, 0.470219, 0.462481, 0.454421,\
        0.446053],\
        [0.392694, 0.391702, 0.390084, 0.388175, 0.386210, 0.384341, 0.382676, 0.381289, 0.380232, 0.379538,\
        0.379221, 0.379262, 0.379602, 0.380133, 0.380710, 0.381193, 0.381526, 0.381758, 0.382009, 0.382410,\
        0.383056, 0.383976, 0.385119, 0.386389, 0.387677, 0.388845, 0.389691, 0.390015, 0.389858, 0.389496,\
        0.389139, 0.388855, 0.388649, 0.388490, 0.388328, 0.388144, 0.388012, 0.388106, 0.388701, 0.390183,\
        0.393036, 0.397747, 0.404643, 0.414137, 0.426318, 0.440681, 0.456522, 0.473015, 0.489063, 0.503713,\
        0.517245, 0.529897, 0.542041, 0.553876, 0.565489, 0.577023, 0.588797, 0.600511, 0.611409, 0.622649,\
        0.633759, 0.644418, 0.654489, 0.663885, 0.672422, 0.679821, 0.685989, 0.691329, 0.696278, 0.701402,\
        0.704375, 0.703871, 0.703169, 0.702745, 0.701216, 0.698697, 0.695975, 0.692520, 0.688760, 0.685111,\
        0.681488, 0.677723, 0.673732, 0.669517, 0.665095, 0.660485, 0.655701, 0.650754, 0.645655, 0.640416,\
        0.635052, 0.629582, 0.624040, 0.618502, 0.613102, 0.607974, 0.603031, 0.597637, 0.591839, 0.585954,\
        0.580016, 0.574037, 0.568035, 0.562020, 0.555994, 0.549952, 0.543887, 0.537786, 0.531635, 0.525423,\
        0.519139, 0.512787, 0.506392, 0.499950, 0.493353, 0.486618, 0.479564, 0.472153, 0.464404, 0.456341,\
        0.447980],\
        [0.396765, 0.395545, 0.393675, 0.391615, 0.389568, 0.387665, 0.386003, 0.384651, 0.383658, 0.383066,\
        0.382903, 0.383166, 0.383807, 0.384704, 0.385650, 0.386411, 0.386882, 0.387161, 0.387438, 0.387891,\
        0.388657, 0.389773, 0.391148, 0.392633, 0.394122, 0.395493, 0.396507, 0.396777, 0.396345, 0.395726,\
        0.395168, 0.394680, 0.394264, 0.393878, 0.393426, 0.392822, 0.392144, 0.391562, 0.391340, 0.391867,\
        0.393640, 0.397276, 0.403440, 0.412767, 0.425117, 0.439795, 0.456138, 0.473445, 0.490892, 0.506246,\
        0.519628, 0.532805, 0.545731, 0.558400, 0.570755, 0.582746, 0.594879, 0.607739, 0.620111, 0.632088,\
        0.643619, 0.654563, 0.664893, 0.674561, 0.683355, 0.690796, 0.696491, 0.701209, 0.705641, 0.709736,\
        0.712358, 0.712820, 0.712362, 0.711502, 0.709812, 0.707311, 0.704329, 0.700452, 0.696462, 0.692675,\
        0.688931, 0.684927, 0.680614, 0.676066, 0.671324, 0.666412, 0.661346, 0.656129, 0.650763, 0.645257,\
        0.639628, 0.633909, 0.628148, 0.622419, 0.616800, 0.611316, 0.605822, 0.600070, 0.594172, 0.588189,\
        0.582119, 0.576008, 0.569888, 0.563773, 0.557665, 0.551559, 0.545447, 0.539317, 0.533158, 0.526958,\
        0.520706, 0.514393, 0.507988, 0.501445, 0.495078, 0.488462, 0.481363, 0.473893, 0.466104, 0.458022,\
        0.449659],\
        [0.399958, 0.398408, 0.396459, 0.394395, 0.392362, 0.390499, 0.388899, 0.387618, 0.386704, 0.386209,\
        0.386182, 0.386653, 0.387601, 0.388923, 0.390362, 0.391501, 0.392122, 0.392457, 0.392795, 0.393324,\
        0.394243, 0.395595, 0.397231, 0.398902, 0.400519, 0.401980, 0.403023, 0.403259, 0.402731, 0.401973,\
        0.401241, 0.400539, 0.399886, 0.399244, 0.398515, 0.397476, 0.396274, 0.395022, 0.393947, 0.393449,\
        0.394013, 0.396276, 0.401301, 0.410664, 0.423061, 0.437812, 0.454328, 0.471850, 0.489225, 0.505186,\
        0.520321, 0.534783, 0.548825, 0.562630, 0.576170, 0.589068, 0.601456, 0.615223, 0.628865, 0.641701,\
        0.653608, 0.664626, 0.674961, 0.684697, 0.693635, 0.701190, 0.706642, 0.711015, 0.714911, 0.718185,\
        0.720320, 0.721040, 0.720705, 0.719615, 0.717754, 0.715150, 0.711884, 0.707997, 0.703908, 0.699962,\
        0.696158, 0.691826, 0.687170, 0.682315, 0.677272, 0.672084, 0.666765, 0.661303, 0.655687, 0.649920,\
        0.644026, 0.638043, 0.632024, 0.626026, 0.620092, 0.614212, 0.608297, 0.602259, 0.596222, 0.590097,\
        0.583876, 0.577637, 0.571414, 0.565217, 0.559045, 0.552891, 0.546748, 0.540603, 0.534448, 0.528273,\
        0.522072, 0.515839, 0.509582, 0.503367, 0.497118, 0.490204, 0.482927, 0.475368, 0.467535, 0.459433,\
        0.451069],\
        [0.401620, 0.400280, 0.398543, 0.396605, 0.394668, 0.392922, 0.391449, 0.390278, 0.389448, 0.389025,\
        0.389088, 0.389704, 0.390891, 0.392606, 0.394663, 0.396348, 0.397118, 0.397589, 0.398149, 0.398845,\
        0.400002, 0.401618, 0.403473, 0.405298, 0.406969, 0.408371, 0.409281, 0.409497, 0.409068, 0.408291,\
        0.407390, 0.406443, 0.405476, 0.404467, 0.403315, 0.401866, 0.400228, 0.398354, 0.396495, 0.395058,\
        0.394548, 0.395829, 0.400213, 0.408634, 0.420459, 0.434878, 0.451304, 0.468980, 0.486759, 0.503860,\
        0.520490, 0.536210, 0.551422, 0.566439, 0.581514, 0.596370, 0.609934, 0.623953, 0.638218, 0.651707,\
        0.663818, 0.674572, 0.684587, 0.694167, 0.702884, 0.710258, 0.715890, 0.720253, 0.723782, 0.726504,\
        0.728225, 0.728834, 0.728442, 0.727201, 0.725185, 0.722454, 0.719089, 0.715233, 0.711148, 0.707072,\
        0.702936, 0.698228, 0.693353, 0.688234, 0.682906, 0.677480, 0.671952, 0.666276, 0.660423, 0.654395,\
        0.648222, 0.641950, 0.635632, 0.629317, 0.623036, 0.616791, 0.610561, 0.604349, 0.598111, 0.591700,\
        0.585295, 0.578937, 0.572628, 0.566365, 0.560143, 0.553954, 0.547788, 0.541635, 0.535487, 0.529337,\
        0.523181, 0.517019, 0.510866, 0.504741, 0.498345, 0.491376, 0.484077, 0.476498, 0.468650, 0.460545,\
        0.452191],\
        [0.402874, 0.401790, 0.400246, 0.398416, 0.396622, 0.395074, 0.393804, 0.392779, 0.392031, 0.391644,\
        0.391727, 0.392395, 0.393685, 0.395520, 0.398043, 0.400495, 0.401611, 0.402540, 0.403568, 0.404722,\
        0.406208, 0.408028, 0.409987, 0.411874, 0.413534, 0.414833, 0.415621, 0.415795, 0.415395, 0.414582,\
        0.413516, 0.412290, 0.410939, 0.409454, 0.407779, 0.405866, 0.403725, 0.401331, 0.398889, 0.396588,\
        0.395070, 0.395480, 0.398933, 0.406349, 0.417276, 0.431038, 0.447294, 0.465464, 0.484730, 0.503717,\
        0.520770, 0.537515, 0.553810, 0.569702, 0.585635, 0.601938, 0.617230, 0.632568, 0.647845, 0.662171,\
        0.674473, 0.684498, 0.693795, 0.703143, 0.711652, 0.718844, 0.724541, 0.728916, 0.732226, 0.734577,\
        0.735950, 0.736329, 0.735771, 0.734370, 0.732211, 0.729372, 0.725951, 0.722086, 0.717946, 0.713647,\
        0.709119, 0.704258, 0.699236, 0.693779, 0.688192, 0.682599, 0.676923, 0.671065, 0.664979, 0.658677,\
        0.652203, 0.645615, 0.638969, 0.632310, 0.625672, 0.619068, 0.612505, 0.605986, 0.599444, 0.592880,\
        0.586368, 0.579928, 0.573554, 0.567241, 0.560979, 0.554760, 0.548573, 0.542409, 0.536261, 0.530121,\
        0.523982, 0.517833, 0.511639, 0.505286, 0.498700, 0.491973, 0.484796, 0.477255, 0.469427, 0.461342,\
        0.453013],\
        [0.403990, 0.403089, 0.401698, 0.400024, 0.398428, 0.397155, 0.396169, 0.395313, 0.394636, 0.394266,\
        0.394302, 0.394929, 0.396311, 0.397935, 0.400053, 0.403183, 0.405527, 0.407374, 0.409091, 0.410833,\
        0.412714, 0.414735, 0.416775, 0.418673, 0.420281, 0.421474, 0.422145, 0.422238, 0.421779, 0.420862,\
        0.419590, 0.418040, 0.416249, 0.414223, 0.411954, 0.409451, 0.406748, 0.403879, 0.400859, 0.397558,\
        0.394937, 0.394334, 0.396904, 0.403376, 0.413162, 0.426056, 0.442242, 0.461059, 0.481151, 0.501066,\
        0.520519, 0.539169, 0.556730, 0.573236, 0.589128, 0.605767, 0.623417, 0.640622, 0.657262, 0.672825,\
        0.685930, 0.694671, 0.703178, 0.712204, 0.720463, 0.727441, 0.733021, 0.737280, 0.740374, 0.742418,\
        0.743470, 0.743576, 0.742796, 0.741210, 0.738901, 0.735952, 0.732460, 0.728533, 0.724283, 0.719772,\
        0.714994, 0.709935, 0.704579, 0.698862, 0.693153, 0.687492, 0.681734, 0.675708, 0.669370, 0.662763,\
        0.655961, 0.649033, 0.642039, 0.635026, 0.628026, 0.621061, 0.614140, 0.607254, 0.600415, 0.593740,\
        0.587164, 0.580661, 0.574234, 0.567876, 0.561579, 0.555329, 0.549116, 0.542932, 0.536768, 0.530615,\
        0.524464, 0.518296, 0.512082, 0.505768, 0.499252, 0.492408, 0.485188, 0.477664, 0.469868, 0.461820,\
        0.453533],\
        [0.405098, 0.404385, 0.403220, 0.401802, 0.400395, 0.399446, 0.398767, 0.398067, 0.397457, 0.397160,\
        0.397148, 0.397611, 0.399108, 0.401160, 0.403187, 0.406470, 0.409634, 0.412354, 0.414799, 0.417123,\
        0.419411, 0.421663, 0.423801, 0.425704, 0.427251, 0.428336, 0.428881, 0.428851, 0.428264, 0.427174,\
        0.425647, 0.423731, 0.421452, 0.418820, 0.415855, 0.412606, 0.409161, 0.405602, 0.401834, 0.397464,\
        0.393500, 0.391695, 0.393273, 0.398786, 0.407309, 0.419426, 0.436191, 0.456570, 0.478326, 0.499989,\
        0.521777, 0.542347, 0.561256, 0.578655, 0.594937, 0.612221, 0.630539, 0.648664, 0.666137, 0.682517,\
        0.696976, 0.705867, 0.713690, 0.722014, 0.729721, 0.736301, 0.741582, 0.745575, 0.748378, 0.750096,\
        0.750816, 0.750616, 0.749575, 0.747775, 0.745297, 0.742220, 0.738626, 0.734602, 0.730222, 0.725533,\
        0.720550, 0.715266, 0.709668, 0.703819, 0.697965, 0.692292, 0.686480, 0.680242, 0.673591, 0.666636,\
        0.659479, 0.652199, 0.644853, 0.637488, 0.630136, 0.622825, 0.615573, 0.608400, 0.601347, 0.594494,\
        0.587793, 0.581206, 0.574718, 0.568315, 0.561977, 0.555689, 0.549440, 0.543219, 0.537019, 0.530830,\
        0.524639, 0.518425, 0.512167, 0.505865, 0.499436, 0.492462, 0.485220, 0.477730, 0.469983, 0.461988,\
        0.453756],\
        [0.406304, 0.405832, 0.404974, 0.403911, 0.402886, 0.402164, 0.401651, 0.401130, 0.400672, 0.400575,\
        0.400680, 0.401177, 0.402725, 0.405126, 0.407838, 0.411110, 0.414555, 0.417813, 0.420828, 0.423647,\
        0.426306, 0.428793, 0.431042, 0.432961, 0.434451, 0.435425, 0.435829, 0.435642, 0.434876, 0.433567,\
        0.431749, 0.429442, 0.426645, 0.423351, 0.419573, 0.415370, 0.410864, 0.406193, 0.401315, 0.395657,\
        0.390321, 0.387110, 0.387141, 0.390983, 0.398472, 0.410715, 0.429384, 0.453168, 0.478537, 0.502948,\
        0.526280, 0.547856, 0.567791, 0.586415, 0.603847, 0.621072, 0.639046, 0.657094, 0.674589, 0.690790,\
        0.704805, 0.715673, 0.724287, 0.732227, 0.739399, 0.745488, 0.750337, 0.753937, 0.756355, 0.757688,\
        0.758032, 0.757486, 0.756146, 0.754100, 0.751428, 0.748199, 0.744479, 0.740335, 0.735821, 0.730980,\
        0.725835, 0.720399, 0.714700, 0.708813, 0.702898, 0.697199, 0.691228, 0.684638, 0.677586, 0.670245,\
        0.662727, 0.655102, 0.647419, 0.639719, 0.632037, 0.624405, 0.616860, 0.609444, 0.602207, 0.595159,\
        0.588301, 0.581612, 0.575057, 0.568601, 0.562214, 0.555876, 0.549572, 0.543293, 0.537033, 0.530785,\
        0.524533, 0.518249, 0.511882, 0.505367, 0.498729, 0.491977, 0.484884, 0.477477, 0.469794, 0.461861,\
        0.453693],\
        [0.407660, 0.407476, 0.406974, 0.406310, 0.405665, 0.405179, 0.404838, 0.404571, 0.404440, 0.404577,\
        0.405014, 0.405943, 0.407685, 0.410198, 0.413226, 0.416652, 0.420270, 0.423845, 0.427252, 0.430452,\
        0.433422, 0.436123, 0.438488, 0.440432, 0.441869, 0.442731, 0.442979, 0.442606, 0.441630, 0.440078,\
        0.437964, 0.435276, 0.431972, 0.428001, 0.423332, 0.417990, 0.412087, 0.405802, 0.399215, 0.392217,\
        0.385764, 0.381164, 0.379492, 0.381510, 0.387962, 0.400499, 0.422565, 0.451377, 0.481870, 0.509506,\
        0.533455, 0.555212, 0.575632, 0.595011, 0.613217, 0.630755, 0.648407, 0.666045, 0.683251, 0.699310,\
        0.713327, 0.724955, 0.734445, 0.742489, 0.749344, 0.754969, 0.759318, 0.762426, 0.764374, 0.765252,\
        0.765159, 0.764213, 0.762529, 0.760207, 0.757317, 0.753914, 0.750048, 0.745767, 0.741118, 0.736139,\
        0.730865, 0.725330, 0.719587, 0.713726, 0.707880, 0.702083, 0.695787, 0.688704, 0.681219, 0.673514,\
        0.665669, 0.657734, 0.649748, 0.641746, 0.633763, 0.625835, 0.618012, 0.610351, 0.602903, 0.595685,\
        0.588697, 0.581918, 0.575299, 0.568785, 0.562338, 0.555930, 0.549548, 0.543184, 0.536837, 0.530505,\
        0.524180, 0.517836, 0.511417, 0.504832, 0.498124, 0.491372, 0.484321, 0.476961, 0.469332, 0.461460,\
        0.453360],\
        [0.409175, 0.409307, 0.409177, 0.408908, 0.408631, 0.408441, 0.408368, 0.408418, 0.408646, 0.409139,\
        0.409992, 0.411352, 0.413370, 0.416045, 0.419243, 0.422812, 0.426580, 0.430369, 0.434048, 0.437530,\
        0.440751, 0.443642, 0.446120, 0.448096, 0.449485, 0.450229, 0.450304, 0.449723, 0.448520, 0.446728,\
        0.444349, 0.441341, 0.437609, 0.433030, 0.427493, 0.420957, 0.413500, 0.405349, 0.396927, 0.388650,\
        0.381068, 0.375231, 0.372195, 0.373006, 0.379067, 0.392891, 0.417815, 0.450919, 0.486411, 0.516859,\
        0.541515, 0.563360, 0.583839, 0.603844, 0.622646, 0.640773, 0.658449, 0.675730, 0.692605, 0.708665,\
        0.722884, 0.734875, 0.744766, 0.752914, 0.759549, 0.764733, 0.768527, 0.771065, 0.772471, 0.772827,\
        0.772225, 0.770812, 0.768737, 0.766110, 0.762984, 0.759390, 0.755359, 0.750926, 0.746132, 0.741016,\
        0.735619, 0.729983, 0.724167, 0.718244, 0.712271, 0.706177, 0.699551, 0.692111, 0.684341, 0.676379,\
        0.668284, 0.660101, 0.651863, 0.643603, 0.635354, 0.627157, 0.619064, 0.611146, 0.603490, 0.596110,\
        0.589012, 0.582174, 0.575502, 0.568925, 0.562399, 0.555897, 0.549405, 0.542924, 0.536455, 0.530008,\
        0.523587, 0.517195, 0.510810, 0.504358, 0.497687, 0.490739, 0.483586, 0.476211, 0.468614, 0.460800,\
        0.452771],\
        [0.410832, 0.411296, 0.411538, 0.411658, 0.411756, 0.411910, 0.412173, 0.412584, 0.413202, 0.414103,\
        0.415381, 0.417139, 0.419458, 0.422336, 0.425690, 0.429392, 0.433294, 0.437249, 0.441128, 0.444827,\
        0.448252, 0.451311, 0.453902, 0.455917, 0.457264, 0.457884, 0.457767, 0.456957, 0.455517, 0.453507,\
        0.450933, 0.447726, 0.443729, 0.438722, 0.432482, 0.424881, 0.415974, 0.406047, 0.395915, 0.386274,\
        0.377525, 0.370709, 0.366824, 0.367083, 0.373331, 0.388351, 0.415117, 0.451402, 0.489417, 0.522710,\
        0.549726, 0.572317, 0.592206, 0.612752, 0.632825, 0.651720, 0.669655, 0.686774, 0.703407, 0.719524,\
        0.733809, 0.745818, 0.755690, 0.763744, 0.770129, 0.774829, 0.777968, 0.779846, 0.780664, 0.780444,\
        0.779245, 0.777279, 0.774769, 0.771822, 0.768453, 0.764656, 0.760439, 0.755832, 0.750874, 0.745605,\
        0.740064, 0.734293, 0.728333, 0.722222, 0.715950, 0.709404, 0.702347, 0.694814, 0.686954, 0.678851,\
        0.670590, 0.662228, 0.653798, 0.645333, 0.636864, 0.628433, 0.620092, 0.611903, 0.603957, 0.596425,\
        0.589330, 0.582469, 0.575741, 0.569085, 0.562454, 0.555823, 0.549185, 0.542545, 0.535915, 0.529308,\
        0.522743, 0.516248, 0.509855, 0.503504, 0.496841, 0.489799, 0.482580, 0.475202, 0.467644, 0.459893,\
        0.451942],\
        [0.412606, 0.413408, 0.414020, 0.414525, 0.415007, 0.415536, 0.416172, 0.416971, 0.417993, 0.419309,\
        0.420997, 0.423129, 0.425750, 0.428852, 0.432374, 0.436212, 0.440243, 0.444337, 0.448375, 0.452246,\
        0.455845, 0.459061, 0.461771, 0.463843, 0.465159, 0.465647, 0.465314, 0.464244, 0.462558, 0.460361,\
        0.457690, 0.454464, 0.450461, 0.445331, 0.438690, 0.430278, 0.420165, 0.408889, 0.397257, 0.385789,\
        0.375925, 0.368574, 0.364406, 0.364540, 0.371079, 0.386930, 0.415652, 0.454972, 0.495251, 0.530418,\
        0.559339, 0.583519, 0.604192, 0.624505, 0.644919, 0.664301, 0.682586, 0.699987, 0.716670, 0.732496,\
        0.746457, 0.758008, 0.767390, 0.775103, 0.781199, 0.785337, 0.787635, 0.788744, 0.788973, 0.788140,\
        0.786211, 0.783587, 0.780619, 0.777367, 0.773761, 0.769744, 0.765311, 0.760498, 0.755348, 0.749900,\
        0.744188, 0.738240, 0.732079, 0.725712, 0.719109, 0.712186, 0.704864, 0.697187, 0.689216, 0.681009,\
        0.672642, 0.664163, 0.655601, 0.646986, 0.638349, 0.629735, 0.621201, 0.612817, 0.604672, 0.597021,\
        0.589861, 0.582916, 0.576098, 0.569330, 0.562557, 0.555754, 0.548923, 0.542080, 0.535245, 0.528436,\
        0.521666, 0.514954, 0.508359, 0.501986, 0.495352, 0.488375, 0.481228, 0.473915, 0.466426, 0.458752,\
        0.450887],\
        [0.414468, 0.415609, 0.416588, 0.417476, 0.418344, 0.419261, 0.420286, 0.421481, 0.422906, 0.424625,\
        0.426701, 0.429183, 0.432098, 0.435434, 0.439136, 0.443117, 0.447274, 0.451491, 0.455660, 0.459673,\
        0.463423, 0.466791, 0.469636, 0.471796, 0.473103, 0.473454, 0.472869, 0.471498, 0.469542, 0.467188,\
        0.464531, 0.461507, 0.457838, 0.453022, 0.446416, 0.437497, 0.426225, 0.413301, 0.399831, 0.386823,\
        0.376466, 0.369382, 0.365605, 0.365803, 0.372387, 0.389724, 0.420819, 0.462213, 0.505045, 0.541459,\
        0.570749, 0.595443, 0.617709, 0.638449, 0.658979, 0.678705, 0.697297, 0.715012, 0.731742, 0.747313,\
        0.760856, 0.771499, 0.779823, 0.786911, 0.792772, 0.796290, 0.797529, 0.797776, 0.797419, 0.795928,\
        0.793055, 0.789688, 0.786309, 0.782806, 0.778969, 0.774692, 0.769992, 0.764930, 0.759556, 0.753903,\
        0.747994, 0.741843, 0.735456, 0.728824, 0.721920, 0.714707, 0.707171, 0.699344, 0.691227, 0.682935,\
        0.674507, 0.665964, 0.657324, 0.648611, 0.639859, 0.631116, 0.622451, 0.613971, 0.605823, 0.598091,\
        0.590674, 0.583569, 0.576633, 0.569720, 0.562758, 0.555727, 0.548647, 0.541553, 0.534477, 0.527441,\
        0.520444, 0.513470, 0.506505, 0.499685, 0.493280, 0.486574, 0.479587, 0.472382, 0.464984, 0.457399,\
        0.449627],\
        [0.416388, 0.417866, 0.419207, 0.420471, 0.421723, 0.423027, 0.424441, 0.426027, 0.427841, 0.429941,\
        0.432378, 0.435189, 0.438387, 0.441955, 0.445843, 0.449974, 0.454253, 0.458580, 0.462854, 0.466980,\
        0.470857, 0.474368, 0.477368, 0.479662, 0.481007, 0.481218, 0.480335, 0.478605, 0.476341, 0.473839,\
        0.471292, 0.468695, 0.465760, 0.461827, 0.455885, 0.446899, 0.434464, 0.419279, 0.403165, 0.389071,\
        0.379260, 0.373441, 0.370945, 0.371313, 0.376490, 0.392857, 0.426278, 0.470942, 0.516258, 0.554835,\
        0.584284, 0.607902, 0.630696, 0.652485, 0.673960, 0.694685, 0.713763, 0.731486, 0.747818, 0.763209,\
        0.776828, 0.786288, 0.792997, 0.799093, 0.804462, 0.807283, 0.807829, 0.807239, 0.805921, 0.803488,\
        0.799646, 0.795661, 0.791971, 0.788265, 0.784153, 0.779521, 0.774474, 0.769114, 0.763489, 0.757614,\
        0.751497, 0.745136, 0.738525, 0.731647, 0.724482, 0.717015, 0.709253, 0.701227, 0.693033, 0.684699,\
        0.676251, 0.667689, 0.659016, 0.650251, 0.641429, 0.632600, 0.623836, 0.615242, 0.606969, 0.599101,\
        0.591574, 0.584423, 0.577397, 0.570309, 0.563095, 0.555766, 0.548370, 0.540968, 0.533618, 0.526355,\
        0.519179, 0.512050, 0.504953, 0.498045, 0.491406, 0.484716, 0.477803, 0.470676, 0.463359, 0.455861,\
        0.448184],\
        [0.418336, 0.420147, 0.421841, 0.423473, 0.425100, 0.426780, 0.428572, 0.430530, 0.432712, 0.435166,\
        0.437937, 0.441052, 0.444518, 0.448313, 0.452389, 0.456671, 0.461069, 0.465489, 0.469841, 0.474043,\
        0.478009, 0.481636, 0.484785, 0.487264, 0.488734, 0.488807, 0.487571, 0.485437, 0.482823, 0.480162,\
        0.477775, 0.475773, 0.473941, 0.471558, 0.467169, 0.458881, 0.445643, 0.428277, 0.409357, 0.394023,\
        0.384686, 0.380564, 0.380539, 0.381996, 0.384636, 0.396944, 0.432832, 0.481017, 0.528075, 0.569051,\
        0.599584, 0.621815, 0.643413, 0.665821, 0.688752, 0.711195, 0.731526, 0.749262, 0.764262, 0.778726,\
        0.793483, 0.802339, 0.807603, 0.812705, 0.816580, 0.818453, 0.818349, 0.816894, 0.814450, 0.810976,\
        0.806581, 0.802027, 0.797877, 0.793912, 0.789348, 0.784181, 0.778703, 0.773018, 0.767132, 0.761033,\
        0.754709, 0.748148, 0.741334, 0.734246, 0.726863, 0.719176, 0.711209, 0.703038, 0.694738, 0.686372,\
        0.677943, 0.669394, 0.660713, 0.651926, 0.643074, 0.634207, 0.625378, 0.616655, 0.608131, 0.600010,\
        0.592616, 0.585591, 0.578481, 0.571147, 0.563591, 0.555871, 0.548077, 0.540302, 0.532636, 0.525145,\
        0.517854, 0.510714, 0.503619, 0.496632, 0.489786, 0.482923, 0.475955, 0.468845, 0.461584, 0.454165,\
        0.446582],\
        [0.420283, 0.422420, 0.424458, 0.426445, 0.428432, 0.430473, 0.432621, 0.434930, 0.437449, 0.440226,\
        0.443298, 0.446690, 0.450405, 0.454419, 0.458681, 0.463115, 0.467627, 0.472124, 0.476522, 0.480753,\
        0.484755, 0.488442, 0.491673, 0.494314, 0.496042, 0.495966, 0.494365, 0.491881, 0.488900, 0.486078,\
        0.483848, 0.482480, 0.481914, 0.481643, 0.479934, 0.473445, 0.459986, 0.441036, 0.420476, 0.403175,\
        0.392141, 0.389329, 0.392634, 0.396784, 0.399703, 0.408489, 0.442742, 0.489615, 0.534905, 0.575689,\
        0.607591, 0.630072, 0.653073, 0.677770, 0.703270, 0.728215, 0.750480, 0.769034, 0.781679, 0.792427,\
        0.807388, 0.817739, 0.823525, 0.827387, 0.829648, 0.830152, 0.829003, 0.826556, 0.823107, 0.818818,\
        0.813940, 0.808938, 0.804218, 0.799699, 0.794330, 0.788484, 0.782583, 0.776601, 0.770472, 0.764155,\
        0.757634, 0.750893, 0.743911, 0.736663, 0.729120, 0.721266, 0.713117, 0.704755, 0.696348, 0.688033,\
        0.679662, 0.671122, 0.662427, 0.653626, 0.644777, 0.635931, 0.627130, 0.618410, 0.609820, 0.601533,\
        0.594366, 0.587341, 0.579976, 0.572252, 0.564228, 0.556013, 0.547731, 0.539506, 0.531458, 0.523696,\
        0.516300, 0.509254, 0.502331, 0.495237, 0.488128, 0.481069, 0.474008, 0.466889, 0.459672, 0.452328,\
        0.444838],\
        [0.422200, 0.424656, 0.427026, 0.429353, 0.431683, 0.434064, 0.436545, 0.439174, 0.441998, 0.445061,\
        0.448399, 0.452035, 0.455974, 0.460193, 0.464639, 0.469227, 0.473853, 0.478415, 0.482827, 0.487034,\
        0.491006, 0.494695, 0.497903, 0.500472, 0.502263, 0.502142, 0.500500, 0.497859, 0.494611, 0.491724,\
        0.489599, 0.488826, 0.489279, 0.491031, 0.492982, 0.489517, 0.476075, 0.455771, 0.433583, 0.413834,\
        0.400942, 0.398347, 0.402536, 0.408300, 0.413978, 0.426528, 0.454172, 0.492862, 0.533452, 0.570652,\
        0.601892, 0.630909, 0.658812, 0.688499, 0.719065, 0.747104, 0.770921, 0.790771, 0.803464, 0.810675,\
        0.823772, 0.834061, 0.839834, 0.842555, 0.843178, 0.842116, 0.839667, 0.836128, 0.831745, 0.826717,\
        0.821278, 0.815720, 0.810243, 0.804679, 0.798502, 0.792243, 0.786078, 0.779862, 0.773501, 0.766969,\
        0.760259, 0.753362, 0.746257, 0.738910, 0.731287, 0.723358, 0.715115, 0.706593, 0.698081, 0.689888,\
        0.681495, 0.672881, 0.664119, 0.655288, 0.646462, 0.637699, 0.629045, 0.620543, 0.612278, 0.604501,\
        0.597209, 0.589658, 0.581826, 0.573547, 0.564929, 0.556123, 0.547276, 0.538527, 0.530016, 0.521877,\
        0.514254, 0.507255, 0.500521, 0.493367, 0.486124, 0.478974, 0.471882, 0.464782, 0.457622, 0.450362,\
        0.442969],\
        [0.424061, 0.426826, 0.429516, 0.432168, 0.434823, 0.437521, 0.440307, 0.443224, 0.446317, 0.449625,\
        0.453187, 0.457027, 0.461157, 0.465560, 0.470185, 0.474936, 0.479685, 0.484309, 0.488721, 0.492863,\
        0.496737, 0.500363, 0.503518, 0.505948, 0.507408, 0.507402, 0.505977, 0.503363, 0.500475, 0.497567,\
        0.495438, 0.495545, 0.496613, 0.499117, 0.503701, 0.502934, 0.490515, 0.469596, 0.446931, 0.426975,\
        0.413765, 0.409129, 0.411393, 0.417159, 0.425472, 0.439503, 0.462571, 0.493706, 0.528989, 0.564738,\
        0.598804, 0.630560, 0.662743, 0.698962, 0.737496, 0.769368, 0.793705, 0.813080, 0.827168, 0.836907,\
        0.846267, 0.853310, 0.856991, 0.857867, 0.856627, 0.853867, 0.850020, 0.845369, 0.840098, 0.834349,\
        0.828279, 0.822041, 0.815701, 0.809145, 0.802316, 0.795696, 0.789314, 0.782847, 0.776216, 0.769442,\
        0.762544, 0.755517, 0.748336, 0.740959, 0.733344, 0.725456, 0.717293, 0.708923, 0.700531, 0.692076,\
        0.683412, 0.674588, 0.665682, 0.656784, 0.647980, 0.639337, 0.630904, 0.622725, 0.614852, 0.607281,\
        0.599715, 0.592144, 0.583790, 0.574832, 0.565547, 0.556104, 0.546651, 0.537334, 0.528299, 0.519680,\
        0.511589, 0.504260, 0.497666, 0.490676, 0.483597, 0.476548, 0.469533, 0.462512, 0.455442, 0.448281,\
        0.440996],\
        [0.425842, 0.428907, 0.431904, 0.434866, 0.437827, 0.440820, 0.443884, 0.447058, 0.450380, 0.453890,\
        0.457626, 0.461621, 0.465896, 0.470448, 0.475239, 0.480167, 0.485063, 0.489753, 0.494175, 0.498262,\
        0.502052, 0.505581, 0.508645, 0.510984, 0.512352, 0.512517, 0.511466, 0.509453, 0.506972, 0.504259,\
        0.502677, 0.503878, 0.505720, 0.508650, 0.511791, 0.510195, 0.499301, 0.480054, 0.458719, 0.439660,\
        0.426320, 0.420312, 0.420945, 0.426370, 0.435612, 0.449357, 0.468600, 0.493470, 0.524204, 0.558963,\
        0.595442, 0.630551, 0.667678, 0.712375, 0.761930, 0.797086, 0.820390, 0.838225, 0.852068, 0.863129,\
        0.870489, 0.873461, 0.874116, 0.872680, 0.869344, 0.864832, 0.859638, 0.853972, 0.847906, 0.841486,\
        0.834782, 0.827872, 0.820799, 0.813572, 0.806279, 0.799213, 0.792445, 0.785569, 0.778562, 0.771501,\
        0.764414, 0.757285, 0.750076, 0.742733, 0.735202, 0.727440, 0.719436, 0.711234, 0.702867, 0.694153,\
        0.685161, 0.676054, 0.666940, 0.657929, 0.649125, 0.640603, 0.632402, 0.624533, 0.616981, 0.609694,\
        0.602573, 0.594624, 0.585438, 0.575802, 0.565897, 0.555852, 0.545809, 0.535926, 0.526369, 0.517297,\
        0.508782, 0.500783, 0.494069, 0.487480, 0.480701, 0.473858, 0.466996, 0.460104, 0.453153, 0.446108,\
        0.438940],\
        [0.427520, 0.430876, 0.434170, 0.437428, 0.440677, 0.443946, 0.447263, 0.450662, 0.454177, 0.457844,\
        0.461701, 0.465790, 0.470144, 0.474784, 0.479703, 0.484820, 0.489919, 0.494668, 0.499075, 0.503196,\
        0.506979, 0.510472, 0.513514, 0.515890, 0.517404, 0.517926, 0.517475, 0.516258, 0.514632, 0.513109,\
        0.512757, 0.514098, 0.516268, 0.518553, 0.519351, 0.515542, 0.504511, 0.487777, 0.468293, 0.450169,\
        0.436991, 0.430508, 0.430624, 0.436127, 0.445744, 0.458755, 0.474970, 0.494810, 0.520463, 0.552285,\
        0.589670, 0.630747, 0.676118, 0.730293, 0.787911, 0.827029, 0.848206, 0.863184, 0.874765, 0.886095,\
        0.892470, 0.890948, 0.889552, 0.885995, 0.880508, 0.874405, 0.868113, 0.861667, 0.854979, 0.847989,\
        0.840709, 0.833190, 0.825506, 0.817753, 0.810087, 0.802671, 0.795355, 0.787853, 0.780378, 0.773017,\
        0.765761, 0.758568, 0.751379, 0.744126, 0.736733, 0.729133, 0.721273, 0.713122, 0.704643, 0.695737,\
        0.686464, 0.677063, 0.667690, 0.658510, 0.649666, 0.641242, 0.633254, 0.625646, 0.618311, 0.611110,\
        0.603798, 0.595528, 0.586143, 0.576178, 0.565841, 0.555297, 0.544727, 0.534321, 0.524277, 0.514829,\
        0.506281, 0.498483, 0.491177, 0.484401, 0.477736, 0.471063, 0.464362, 0.457615, 0.450796, 0.443877,\
        0.436829],\
        [0.429074, 0.432713, 0.436294, 0.439837, 0.443363, 0.446890, 0.450442, 0.454041, 0.457716, 0.461497,\
        0.465422, 0.469536, 0.473886, 0.478519, 0.483465, 0.488714, 0.494122, 0.499043, 0.503348, 0.507520,\
        0.511440, 0.515053, 0.518236, 0.520828, 0.522690, 0.523755, 0.524071, 0.523830, 0.523367, 0.523173,\
        0.523812, 0.525419, 0.527406, 0.528708, 0.527675, 0.522271, 0.511346, 0.495423, 0.476220, 0.458412,\
        0.445501, 0.439329, 0.440002, 0.446387, 0.456763, 0.469343, 0.483050, 0.498814, 0.518152, 0.545215,\
        0.582383, 0.631914, 0.688733, 0.748549, 0.808696, 0.851587, 0.873108, 0.886159, 0.892667, 0.900597,\
        0.906576, 0.904487, 0.902166, 0.896454, 0.889321, 0.882116, 0.875198, 0.868331, 0.861237, 0.853795,\
        0.846002, 0.837921, 0.829657, 0.821351, 0.813180, 0.805280, 0.797382, 0.789291, 0.781435, 0.773841,\
        0.766469, 0.759259, 0.752136, 0.745017, 0.737803, 0.730391, 0.722669, 0.714525, 0.705863, 0.696701,\
        0.687192, 0.677455, 0.667739, 0.658308, 0.649367, 0.641019, 0.633245, 0.625901, 0.618729, 0.611389,\
        0.603537, 0.595147, 0.585997, 0.575986, 0.565370, 0.554426, 0.543410, 0.532555, 0.522079, 0.512183,\
        0.503229, 0.495602, 0.488361, 0.481503, 0.474863, 0.468294, 0.461724, 0.455110, 0.448420, 0.441624,\
        0.434693],\
        [0.430484, 0.434399, 0.438261, 0.442082, 0.445874, 0.449651, 0.453424, 0.457208, 0.461020, 0.464883,\
        0.468831, 0.472906, 0.477165, 0.481671, 0.486482, 0.491627, 0.497072, 0.502361, 0.506890, 0.511271,\
        0.515466, 0.519368, 0.522866, 0.525846, 0.528221, 0.529964, 0.531145, 0.531944, 0.532648, 0.533614,\
        0.535122, 0.537113, 0.539002, 0.539666, 0.537542, 0.531026, 0.519256, 0.502547, 0.482967, 0.464862,\
        0.452223, 0.447016, 0.449177, 0.457321, 0.469193, 0.481918, 0.492594, 0.502042, 0.516192, 0.539297,\
        0.576955, 0.636024, 0.704009, 0.763624, 0.831050, 0.881459, 0.900476, 0.908659, 0.912646, 0.915538,\
        0.918285, 0.916140, 0.911132, 0.903824, 0.895612, 0.887891, 0.880997, 0.874055, 0.866720, 0.858899,\
        0.850627, 0.841996, 0.833129, 0.824170, 0.815275, 0.806551, 0.797992, 0.789653, 0.781605, 0.773879,\
        0.766450, 0.759262, 0.752242, 0.745292, 0.738294, 0.731105, 0.723565, 0.715497, 0.706724, 0.697275,\
        0.687388, 0.677128, 0.666908, 0.657109, 0.648004, 0.639718, 0.632208, 0.625242, 0.618423, 0.611244,\
        0.603297, 0.594833, 0.585649, 0.575465, 0.564553, 0.553251, 0.541862, 0.530660, 0.519909, 0.509841,\
        0.500571, 0.492715, 0.485574, 0.478740, 0.472126, 0.465626, 0.459152, 0.452648, 0.446070, 0.439384,\
        0.432560],\
        [0.431730, 0.435917, 0.440054, 0.444149, 0.448206, 0.452230, 0.456222, 0.460186, 0.464126, 0.468056,\
        0.471998, 0.475990, 0.480090, 0.484377, 0.488933, 0.493786, 0.498899, 0.504364, 0.509601, 0.514511,\
        0.519151, 0.523483, 0.527436, 0.530936, 0.533936, 0.536437, 0.538513, 0.540319, 0.542083, 0.544050,\
        0.546364, 0.548891, 0.551058, 0.551758, 0.549401, 0.542274, 0.529312, 0.510980, 0.489783, 0.470434,\
        0.458083, 0.454535, 0.458959, 0.469490, 0.483404, 0.496521, 0.504737, 0.508652, 0.516731, 0.535517,\
        0.573649, 0.636841, 0.710254, 0.776020, 0.860199, 0.919102, 0.934972, 0.938586, 0.937990, 0.935740,\
        0.932566, 0.926979, 0.919049, 0.909472, 0.899592, 0.892653, 0.886065, 0.879096, 0.871518, 0.863306,\
        0.854551, 0.845373, 0.835893, 0.826234, 0.816539, 0.806980, 0.797806, 0.789167, 0.780958, 0.773128,\
        0.765655, 0.758503, 0.751601, 0.744843, 0.738087, 0.731160, 0.723859, 0.715973, 0.707313, 0.697646,\
        0.686985, 0.675919, 0.665008, 0.654708, 0.645361, 0.637132, 0.629964, 0.623566, 0.617432, 0.610927,\
        0.603482, 0.594935, 0.585297, 0.574675, 0.563382, 0.551746, 0.540060, 0.528602, 0.517673, 0.507674,\
        0.498866, 0.490513, 0.483065, 0.476162, 0.469553, 0.463094, 0.456689, 0.450267, 0.443780, 0.437186,\
        0.430455],\
        [0.432793, 0.437247, 0.441657, 0.446026, 0.450350, 0.454626, 0.458844, 0.462997, 0.467076, 0.471080,\
        0.475016, 0.478914, 0.482829, 0.486855, 0.491136, 0.495818, 0.500928, 0.506467, 0.512088, 0.517500,\
        0.522640, 0.527475, 0.531962, 0.536062, 0.539751, 0.543042, 0.545995, 0.548735, 0.551434, 0.554272,\
        0.557355, 0.560579, 0.563477, 0.565032, 0.563554, 0.556846, 0.543124, 0.522509, 0.498101, 0.476453,\
        0.464810, 0.463989, 0.471282, 0.484419, 0.500284, 0.513435, 0.519631, 0.520508, 0.522388, 0.534252,\
        0.568104, 0.631632, 0.708463, 0.787653, 0.876058, 0.938168, 0.961347, 0.965527, 0.961966, 0.955247,\
        0.946974, 0.937316, 0.926518, 0.915368, 0.905444, 0.897805, 0.890803, 0.883583, 0.875624, 0.866948,\
        0.857704, 0.848014, 0.837975, 0.827675, 0.817237, 0.806892, 0.797097, 0.788097, 0.779636, 0.771627,\
        0.764063, 0.756921, 0.750130, 0.743570, 0.737072, 0.730426, 0.723378, 0.715639, 0.706956, 0.696989,\
        0.685512, 0.673577, 0.661863, 0.650943, 0.641266, 0.633069, 0.626304, 0.620628, 0.615429, 0.609934,\
        0.603269, 0.594658, 0.584456, 0.573335, 0.561693, 0.549816, 0.537957, 0.526353, 0.515250, 0.504934,\
        0.495974, 0.488043, 0.480669, 0.473759, 0.467152, 0.460717, 0.454355, 0.447991, 0.441571, 0.435051,\
        0.428395],\
        [0.433652, 0.438369, 0.443051, 0.447696, 0.452294, 0.456832, 0.461295, 0.465661, 0.469907, 0.474015,\
        0.477979, 0.481815, 0.485570, 0.489337, 0.493314, 0.497876, 0.503122, 0.508813, 0.514686, 0.520480,\
        0.526070, 0.531402, 0.536444, 0.541170, 0.545569, 0.549649, 0.553448, 0.557046, 0.560571, 0.564173,\
        0.567975, 0.571990, 0.575967, 0.579128, 0.579824, 0.575172, 0.562066, 0.539516, 0.510680, 0.485279,\
        0.475643, 0.479129, 0.489470, 0.505609, 0.523164, 0.536323, 0.541594, 0.540029, 0.536706, 0.540325,\
        0.567240, 0.639162, 0.723062, 0.801185, 0.889155, 0.950128, 0.977974, 0.984571, 0.980183, 0.970651,\
        0.958783, 0.945848, 0.932688, 0.920325, 0.909951, 0.901818, 0.894824, 0.887286, 0.878790, 0.869616,\
        0.859939, 0.849843, 0.839379, 0.828596, 0.817582, 0.806570, 0.796154, 0.786636, 0.777728, 0.769385,\
        0.761634, 0.754456, 0.747761, 0.741393, 0.735152, 0.728791, 0.722004, 0.714362, 0.705336, 0.694746,\
        0.682755, 0.670014, 0.657423, 0.645760, 0.635643, 0.627419, 0.621054, 0.616143, 0.611916, 0.607359,\
        0.601528, 0.592863, 0.582329, 0.570994, 0.559243, 0.547338, 0.535503, 0.523956, 0.512922, 0.502669,\
        0.493646, 0.485732, 0.478421, 0.471526, 0.464921, 0.458495, 0.452155, 0.445827, 0.439454, 0.432987,\
        0.426388],\
        [0.434284, 0.439260, 0.444214, 0.449137, 0.454017, 0.458835, 0.463565, 0.468178, 0.472636, 0.476903,\
        0.480956, 0.484800, 0.488486, 0.492093, 0.495739, 0.500013, 0.505467, 0.511395, 0.517483, 0.523556,\
        0.529507, 0.535280, 0.540843, 0.546180, 0.551279, 0.556136, 0.560753, 0.565158, 0.569434, 0.573725,\
        0.578193, 0.582988, 0.588178, 0.593308, 0.597044, 0.596115, 0.586074, 0.564443, 0.533377, 0.504044,\
        0.496436, 0.504007, 0.517577, 0.535448, 0.553150, 0.565940, 0.570670, 0.568010, 0.563024, 0.562157,\
        0.583440, 0.666481, 0.751724, 0.831318, 0.912344, 0.964096, 0.989797, 0.996438, 0.991530, 0.980419,\
        0.966346, 0.951192, 0.936228, 0.922621, 0.911605, 0.903968, 0.897607, 0.889505, 0.880480, 0.870975,\
        0.861056, 0.850737, 0.840036, 0.828986, 0.817642, 0.806153, 0.795086, 0.784803, 0.775179, 0.766321,\
        0.758290, 0.751043, 0.744441, 0.738270, 0.732273, 0.726172, 0.719663, 0.712306, 0.703252, 0.692010,\
        0.679119, 0.665424, 0.651851, 0.639300, 0.628600, 0.620268, 0.614234, 0.610001, 0.606628, 0.602579,\
        0.596641, 0.588436, 0.578391, 0.567371, 0.555876, 0.544220, 0.532646, 0.521378, 0.510659, 0.500758,\
        0.491790, 0.483754, 0.476389, 0.469469, 0.462847, 0.456414, 0.450079, 0.443770, 0.437424, 0.430993,\
        0.424437],\
        [0.434667, 0.439896, 0.445118, 0.450322, 0.455491, 0.460604, 0.465628, 0.470525, 0.475250, 0.479747,\
        0.483970, 0.487907, 0.491636, 0.495314, 0.499014, 0.503124, 0.508439, 0.514355, 0.520507, 0.526733,\
        0.532937, 0.539064, 0.545084, 0.550983, 0.556749, 0.562364, 0.567794, 0.572994, 0.577988, 0.582951,\
        0.588107, 0.593693, 0.600127, 0.607245, 0.613326, 0.615558, 0.610369, 0.594611, 0.567335, 0.540673,\
        0.530506, 0.536928, 0.552122, 0.569699, 0.585372, 0.596617, 0.600909, 0.598565, 0.596822, 0.598421,\
        0.622362, 0.696767, 0.778873, 0.863713, 0.936579, 0.977267, 0.997262, 1.001951, 0.996215, 0.984224,\
        0.969016, 0.952645, 0.936535, 0.921820, 0.909653, 0.901807, 0.896725, 0.888892, 0.880099, 0.870751,\
        0.860901, 0.850570, 0.839802, 0.828658, 0.817200, 0.805536, 0.793733, 0.782297, 0.771726, 0.762241,\
        0.753904, 0.746620, 0.740163, 0.734225, 0.728468, 0.722550, 0.716170, 0.709112, 0.700679, 0.688579,\
        0.674709, 0.660096, 0.645545, 0.631978, 0.620512, 0.612046, 0.606294, 0.602468, 0.599679, 0.595786,\
        0.589959, 0.582233, 0.572924, 0.562563, 0.551622, 0.540458, 0.529351, 0.518534, 0.508195, 0.498601,\
        0.489882, 0.481892, 0.474498, 0.467543, 0.460895, 0.454445, 0.448104, 0.441800, 0.435470, 0.429062,\
        0.422533],\
        [0.434776, 0.440250, 0.445734, 0.451217, 0.456680, 0.462097, 0.467434, 0.472652, 0.477696, 0.482500,\
        0.486984, 0.491087, 0.494893, 0.498684, 0.502661, 0.506914, 0.511941, 0.517621, 0.523676, 0.529929,\
        0.536277, 0.542661, 0.549051, 0.555434, 0.561804, 0.568146, 0.574406, 0.580448, 0.586151, 0.591795,\
        0.597846, 0.604482, 0.612170, 0.620367, 0.627484, 0.631286, 0.629268, 0.619198, 0.599262, 0.576901,\
        0.566670, 0.571313, 0.586336, 0.601135, 0.612212, 0.619852, 0.623337, 0.623669, 0.625184, 0.631583,\
        0.656151, 0.728884, 0.814425, 0.890754, 0.948205, 0.982451, 0.998427, 1.001201, 0.994729, 0.982385,\
        0.966877, 0.950218, 0.933844, 0.918772, 0.905926, 0.897103, 0.891838, 0.885522, 0.877762, 0.869020,\
        0.859496, 0.849288, 0.838505, 0.827287, 0.815692, 0.803579, 0.790973, 0.778463, 0.766959, 0.756890,\
        0.748347, 0.741160, 0.734984, 0.729386, 0.723921, 0.718145, 0.711610, 0.703990, 0.695212, 0.683504,\
        0.669486, 0.654430, 0.639190, 0.624646, 0.612133, 0.603585, 0.598379, 0.594706, 0.591718, 0.587828,\
        0.582301, 0.575121, 0.566532, 0.556918, 0.546671, 0.536140, 0.525628, 0.515414, 0.505660, 0.496418,\
        0.487917, 0.480021, 0.472645, 0.465678, 0.459012, 0.452546, 0.446197, 0.439892, 0.433569, 0.427176,\
        0.420666],\
        [0.434589, 0.440295, 0.446031, 0.451785, 0.457538, 0.463261, 0.468920, 0.474473, 0.479870, 0.485046,\
        0.489901, 0.494270, 0.498141, 0.501995, 0.506085, 0.510489, 0.515413, 0.520890, 0.526794, 0.532994,\
        0.539397, 0.545944, 0.552606, 0.559371, 0.566241, 0.573222, 0.580295, 0.587313, 0.593788, 0.599998,\
        0.607007, 0.614717, 0.623151, 0.631727, 0.639142, 0.643628, 0.643269, 0.636403, 0.622775, 0.607181,\
        0.599685, 0.600682, 0.613744, 0.624016, 0.631297, 0.637170, 0.642222, 0.646732, 0.655019, 0.674261,\
        0.709302, 0.787910, 0.859651, 0.915003, 0.954839, 0.980953, 0.993468, 0.994786, 0.987885, 0.975622,\
        0.960454, 0.944316, 0.928641, 0.914463, 0.902446, 0.893660, 0.887440, 0.881365, 0.874335, 0.866214,\
        0.857066, 0.846972, 0.836057, 0.824561, 0.812784, 0.800121, 0.786395, 0.772844, 0.760542, 0.750070,\
        0.741549, 0.734714, 0.729055, 0.723984, 0.718954, 0.713463, 0.706971, 0.698948, 0.689444, 0.677780,\
        0.663989, 0.648992, 0.633664, 0.618769, 0.605288, 0.596451, 0.591629, 0.587807, 0.584205, 0.579967,\
        0.574520, 0.567733, 0.559744, 0.550825, 0.541286, 0.531425, 0.521522, 0.511896, 0.502890, 0.494170,\
        0.485827, 0.478054, 0.470744, 0.463802, 0.457138, 0.450668, 0.444316, 0.438012, 0.431696, 0.425312,\
        0.418816],\
        [0.434083, 0.440003, 0.445977, 0.451989, 0.458019, 0.464038, 0.470010, 0.475890, 0.481631, 0.487181,\
        0.492475, 0.497339, 0.501300, 0.505172, 0.509262, 0.513684, 0.518541, 0.523874, 0.529636, 0.535750,\
        0.542146, 0.548776, 0.555610, 0.562638, 0.569862, 0.577296, 0.584969, 0.592918, 0.600633, 0.607349,\
        0.614987, 0.623400, 0.632286, 0.641024, 0.648558, 0.653558, 0.654721, 0.651250, 0.643803, 0.635618,\
        0.631016, 0.631589, 0.637387, 0.642774, 0.646546, 0.652180, 0.662289, 0.674133, 0.690139, 0.724607,\
        0.772245, 0.832524, 0.880864, 0.920529, 0.953142, 0.974159, 0.983928, 0.984058, 0.976892, 0.964947,\
        0.950536, 0.935500, 0.921187, 0.908663, 0.898415, 0.890053, 0.883367, 0.877196, 0.870497, 0.862796,\
        0.853931, 0.843863, 0.832619, 0.820335, 0.807587, 0.794713, 0.779829, 0.765309, 0.752372, 0.741736,\
        0.733564, 0.727447, 0.722633, 0.718335, 0.713912, 0.708918, 0.702787, 0.694807, 0.684699, 0.672599,\
        0.658832, 0.644133, 0.629360, 0.615365, 0.603099, 0.593655, 0.587187, 0.582189, 0.577629, 0.572774,\
        0.567135, 0.560513, 0.552945, 0.544604, 0.535726, 0.526535, 0.517218, 0.507982, 0.499271, 0.491257,\
        0.483437, 0.475920, 0.468738, 0.461857, 0.455221, 0.448764, 0.442420, 0.436125, 0.429818, 0.423447,\
        0.416961],\
        [0.433240, 0.439354, 0.445544, 0.451795, 0.458083, 0.464378, 0.470639, 0.476816, 0.482850, 0.488678,\
        0.494241, 0.499428, 0.503792, 0.507776, 0.511912, 0.516339, 0.521146, 0.526374, 0.532016, 0.538036,\
        0.544388, 0.551036, 0.557955, 0.565131, 0.572555, 0.580224, 0.588113, 0.596179, 0.604543, 0.612712,\
        0.621173, 0.630166, 0.639419, 0.648357, 0.656149, 0.661870, 0.664806, 0.664873, 0.663053, 0.661315,\
        0.660888, 0.661262, 0.662310, 0.662560, 0.661510, 0.665798, 0.683750, 0.708719, 0.735931, 0.775125,\
        0.816810, 0.859149, 0.894945, 0.922558, 0.947827, 0.964570, 0.971850, 0.970747, 0.963210, 0.951559,\
        0.938092, 0.924580, 0.911903, 0.901154, 0.892490, 0.885233, 0.878926, 0.872909, 0.866456, 0.859044,\
        0.850371, 0.840278, 0.828696, 0.815582, 0.800974, 0.786525, 0.771289, 0.756113, 0.742656, 0.732059,\
        0.724619, 0.719668, 0.716082, 0.712836, 0.709158, 0.704727, 0.699239, 0.691177, 0.680548, 0.667934,\
        0.654026, 0.639619, 0.625588, 0.612777, 0.601938, 0.592074, 0.583989, 0.577559, 0.571906, 0.566320,\
        0.560326, 0.553687, 0.546365, 0.538467, 0.530177, 0.521679, 0.513070, 0.504401, 0.496106, 0.488403,\
        0.480928, 0.473645, 0.466602, 0.459802, 0.453213, 0.446787, 0.440466, 0.434191, 0.427904, 0.421550,\
        0.415079],\
        [0.432048, 0.438330, 0.444711, 0.451175, 0.457696, 0.464240, 0.470762, 0.477202, 0.483481, 0.489500,\
        0.495133, 0.500314, 0.505126, 0.509518, 0.513830, 0.518312, 0.523098, 0.528255, 0.533800, 0.539726,\
        0.546012, 0.552633, 0.559569, 0.566803, 0.574320, 0.582106, 0.590138, 0.598384, 0.607075, 0.616118,\
        0.625392, 0.634964, 0.644633, 0.653932, 0.662221, 0.668874, 0.673594, 0.676739, 0.679388, 0.683048,\
        0.687712, 0.688659, 0.687877, 0.686318, 0.680328, 0.679318, 0.704459, 0.742835, 0.779376, 0.815418,\
        0.847150, 0.876069, 0.901913, 0.924052, 0.942352, 0.954583, 0.959167, 0.956553, 0.948361, 0.936767,\
        0.924142, 0.912720, 0.902149, 0.893078, 0.885878, 0.879666, 0.873992, 0.868354, 0.862202, 0.855049,\
        0.846528, 0.836401, 0.824569, 0.811080, 0.795846, 0.779195, 0.762375, 0.746156, 0.731983, 0.721461,\
        0.715162, 0.711849, 0.709786, 0.707862, 0.705317, 0.701261, 0.696555, 0.688068, 0.676589, 0.663343,\
        0.649174, 0.634892, 0.621365, 0.609247, 0.598434, 0.588942, 0.580476, 0.573191, 0.566692, 0.560457,\
        0.554080, 0.547322, 0.540111, 0.532513, 0.524692, 0.516872, 0.509193, 0.501346, 0.493455, 0.485806,\
        0.478411, 0.471250, 0.464317, 0.457601, 0.451075, 0.444696, 0.438415, 0.432175, 0.425920, 0.419593,\
        0.413142],\
        [0.430499, 0.436921, 0.443464, 0.450111, 0.456834, 0.463597, 0.470350, 0.477028, 0.483545, 0.489783,\
        0.495599, 0.500944, 0.505917, 0.510517, 0.514975, 0.519528, 0.524320, 0.529432, 0.534899, 0.540737,\
        0.546941, 0.553504, 0.560411, 0.567647, 0.575192, 0.583033, 0.591202, 0.599811, 0.608661, 0.618060,\
        0.627848, 0.637917, 0.648067, 0.657914, 0.666941, 0.674660, 0.680920, 0.686367, 0.692196, 0.699167,\
        0.707847, 0.711767, 0.712980, 0.713286, 0.709082, 0.707265, 0.739615, 0.781211, 0.814387, 0.841793,\
        0.865065, 0.884853, 0.902931, 0.921737, 0.936506, 0.945236, 0.947188, 0.942860, 0.933769, 0.922109,\
        0.909909, 0.900250, 0.893031, 0.885374, 0.879098, 0.873693, 0.868638, 0.863454, 0.857655, 0.850778,\
        0.842407, 0.832215, 0.820025, 0.805837, 0.789642, 0.771911, 0.753867, 0.736565, 0.721396, 0.710713,\
        0.706020, 0.704632, 0.703930, 0.703194, 0.702281, 0.700957, 0.694983, 0.684568, 0.672069, 0.658276,\
        0.643873, 0.629557, 0.616135, 0.604707, 0.594541, 0.584894, 0.576342, 0.568698, 0.561701, 0.555009,\
        0.548314, 0.541412, 0.534223, 0.526787, 0.519248, 0.511858, 0.504983, 0.498497, 0.490660, 0.483042,\
        0.475723, 0.468667, 0.461840, 0.455217, 0.448767, 0.442453, 0.436228, 0.430040, 0.423833, 0.417546,\
        0.411125],\
        [0.428595, 0.435126, 0.441800, 0.448597, 0.455488, 0.462433, 0.469382, 0.476271, 0.483022, 0.489532,\
        0.495667, 0.501247, 0.506210, 0.510813, 0.515341, 0.519961, 0.524777, 0.529862, 0.535266, 0.541018,\
        0.547134, 0.553617, 0.560465, 0.567669, 0.575208, 0.583054, 0.591180, 0.599665, 0.608901, 0.618601,\
        0.628686, 0.639142, 0.649808, 0.660360, 0.670359, 0.679329, 0.686922, 0.693745, 0.702101, 0.711855,\
        0.722088, 0.730177, 0.736052, 0.740927, 0.746608, 0.761328, 0.791131, 0.818693, 0.840012, 0.858611,\
        0.875157, 0.890221, 0.905024, 0.919767, 0.931338, 0.937112, 0.936606, 0.930600, 0.920523, 0.909071,\
        0.898372, 0.889538, 0.883178, 0.877288, 0.872028, 0.867341, 0.862848, 0.858109, 0.852679, 0.846103,\
        0.837918, 0.827678, 0.815047, 0.800044, 0.783011, 0.764706, 0.746209, 0.728606, 0.712992, 0.701555,\
        0.698835, 0.698547, 0.698280, 0.697901, 0.697299, 0.695405, 0.689238, 0.678817, 0.666157, 0.652305,\
        0.637958, 0.623733, 0.610228, 0.598300, 0.589016, 0.580085, 0.571745, 0.564040, 0.556819, 0.549863,\
        0.542955, 0.535934, 0.528732, 0.521368, 0.513945, 0.506672, 0.499856, 0.493663, 0.486877, 0.479742,\
        0.472687, 0.465811, 0.459124, 0.452615, 0.446258, 0.440023, 0.433869, 0.427750, 0.421608, 0.415379,\
        0.408999],\
        [0.426349, 0.432956, 0.439728, 0.446641, 0.453662, 0.460747, 0.467846, 0.474897, 0.481830, 0.488564,\
        0.494985, 0.500803, 0.505670, 0.510265, 0.514896, 0.519611, 0.524470, 0.529542, 0.534889, 0.540555,\
        0.546572, 0.552958, 0.559724, 0.566871, 0.574396, 0.582280, 0.590481, 0.599020, 0.608236, 0.617833,\
        0.627949, 0.638674, 0.649852, 0.661181, 0.672299, 0.682812, 0.692157, 0.699949, 0.709705, 0.721312,\
        0.733383, 0.744814, 0.755313, 0.765598, 0.778293, 0.798918, 0.821361, 0.838531, 0.852403, 0.865327,\
        0.878052, 0.890774, 0.904180, 0.917798, 0.927102, 0.930240, 0.927386, 0.919995, 0.909758, 0.897732,\
        0.888705, 0.880546, 0.874213, 0.869049, 0.864583, 0.860525, 0.856520, 0.852169, 0.847068, 0.840792,\
        0.832849, 0.822692, 0.809753, 0.793969, 0.776282, 0.757704, 0.739440, 0.722732, 0.709053, 0.700264,\
        0.695515, 0.693483, 0.692427, 0.691493, 0.690029, 0.686848, 0.680487, 0.670770, 0.658720, 0.645361,\
        0.631481, 0.617747, 0.604831, 0.593410, 0.583736, 0.575072, 0.566966, 0.559326, 0.552045, 0.544972,\
        0.537953, 0.530868, 0.523660, 0.516362, 0.509057, 0.501827, 0.495238, 0.489024, 0.482652, 0.476011,\
        0.469301, 0.462662, 0.456148, 0.449772, 0.443523, 0.437379, 0.431308, 0.425270, 0.419211, 0.413059,\
        0.406736],\
        [0.423780, 0.430433, 0.437271, 0.444265, 0.451377, 0.458559, 0.465754, 0.472898, 0.479918, 0.486719,\
        0.493169, 0.499000, 0.503911, 0.508791, 0.513676, 0.518530, 0.523438, 0.528496, 0.533783, 0.539359,\
        0.545266, 0.551536, 0.558189, 0.565244, 0.572719, 0.580638, 0.589037, 0.597881, 0.606641, 0.615601,\
        0.625566, 0.636506, 0.648163, 0.660208, 0.672308, 0.684168, 0.695535, 0.705011, 0.715188, 0.727443,\
        0.740891, 0.754780, 0.768734, 0.782477, 0.795302, 0.810863, 0.827141, 0.840105, 0.850950, 0.861637,\
        0.873447, 0.886950, 0.901868, 0.915317, 0.923009, 0.923868, 0.918692, 0.909829, 0.899944, 0.889255,\
        0.879343, 0.871454, 0.865351, 0.860612, 0.856701, 0.853151, 0.849534, 0.845451, 0.840556, 0.834498,\
        0.826779, 0.816732, 0.803888, 0.787369, 0.769005, 0.750430, 0.732791, 0.717298, 0.705405, 0.697768,\
        0.692186, 0.688439, 0.686085, 0.684086, 0.681507, 0.677349, 0.670761, 0.661565, 0.650278, 0.637682,\
        0.624551, 0.611586, 0.599383, 0.588485, 0.578875, 0.570242, 0.562240, 0.554672, 0.547405, 0.540310,\
        0.533270, 0.526186, 0.519002, 0.511762, 0.504715, 0.497780, 0.491113, 0.484795, 0.478508, 0.472120,\
        0.465676, 0.459260, 0.452924, 0.446688, 0.440553, 0.434503, 0.428518, 0.422565, 0.416599, 0.410546,\
        0.404300],\
        [0.420923, 0.427592, 0.434466, 0.441510, 0.448677, 0.455912, 0.463152, 0.470328, 0.477354, 0.484114,\
        0.490436, 0.496080, 0.501272, 0.506730, 0.511868, 0.516811, 0.521739, 0.526769, 0.531988, 0.537464,\
        0.543250, 0.549383, 0.555890, 0.562792, 0.570112, 0.577881, 0.586130, 0.594746, 0.602841, 0.611490,\
        0.621513, 0.632729, 0.644822, 0.657421, 0.670151, 0.682622, 0.694255, 0.705271, 0.716686, 0.729400,\
        0.743593, 0.758701, 0.774227, 0.789591, 0.801622, 0.810011, 0.819931, 0.829447, 0.838675, 0.848608,\
        0.860745, 0.876453, 0.894663, 0.909494, 0.917036, 0.916549, 0.909621, 0.899111, 0.888726, 0.878468,\
        0.869076, 0.861517, 0.855847, 0.851618, 0.848231, 0.845123, 0.841797, 0.837814, 0.832871, 0.826804,\
        0.819211, 0.808963, 0.795290, 0.778621, 0.760316, 0.742175, 0.725414, 0.710844, 0.699552, 0.691944,\
        0.686400, 0.682199, 0.678956, 0.675975, 0.672447, 0.667593, 0.660839, 0.652025, 0.641429, 0.629620,\
        0.617306, 0.605207, 0.593834, 0.583439, 0.574064, 0.565530, 0.557609, 0.550114, 0.542898, 0.535841,\
        0.528844, 0.521835, 0.514753, 0.507510, 0.500379, 0.493797, 0.487196, 0.480778, 0.474468, 0.468178,\
        0.461898, 0.455654, 0.449475, 0.443374, 0.437350, 0.431391, 0.425482, 0.419603, 0.413726, 0.407786,\
        0.401643],\
        [0.417824, 0.424482, 0.431368, 0.438434, 0.445624, 0.452875, 0.460120, 0.467288, 0.474293, 0.481033,\
        0.487394, 0.493329, 0.499065, 0.504557, 0.509603, 0.514519, 0.519429, 0.524414, 0.529554, 0.534924,\
        0.540582, 0.546566, 0.552900, 0.559594, 0.566650, 0.574049, 0.581723, 0.589456, 0.597164, 0.605942,\
        0.616164, 0.627680, 0.640140, 0.653104, 0.666161, 0.678968, 0.691348, 0.703046, 0.714003, 0.726862,\
        0.741218, 0.756197, 0.770992, 0.784296, 0.793569, 0.799571, 0.805193, 0.811787, 0.819595, 0.828528,\
        0.840011, 0.856641, 0.877336, 0.896481, 0.906050, 0.905667, 0.899989, 0.888196, 0.876699, 0.866132,\
        0.857125, 0.850204, 0.845298, 0.841818, 0.839045, 0.836381, 0.833305, 0.829262, 0.823882, 0.817301,\
        0.809598, 0.799357, 0.784723, 0.767807, 0.749898, 0.732483, 0.716948, 0.703710, 0.692891, 0.685158,\
        0.679424, 0.674922, 0.671116, 0.667428, 0.663222, 0.657913, 0.651090, 0.642613, 0.632626, 0.621529,\
        0.609949, 0.598646, 0.588074, 0.578246, 0.569186, 0.560822, 0.553014, 0.545612, 0.538477, 0.531491,\
        0.524568, 0.517671, 0.510809, 0.503982, 0.496840, 0.489937, 0.483280, 0.476783, 0.470427, 0.464170,\
        0.457985, 0.451869, 0.445824, 0.439846, 0.433928, 0.428054, 0.422206, 0.416370, 0.410546, 0.404693,\
        0.398643],\
        [0.414541, 0.421167, 0.428047, 0.435114, 0.442297, 0.449531, 0.456748, 0.463883, 0.470864, 0.477619,\
        0.484098, 0.490323, 0.496348, 0.501742, 0.506719, 0.511649, 0.516555, 0.521490, 0.526546, 0.531810,\
        0.537344, 0.543185, 0.549347, 0.555824, 0.562590, 0.569592, 0.576745, 0.583986, 0.591527, 0.600048,\
        0.610216, 0.621938, 0.634642, 0.647776, 0.660876, 0.673584, 0.685599, 0.696413, 0.707462, 0.720703,\
        0.734745, 0.748560, 0.761404, 0.772201, 0.779802, 0.784518, 0.788008, 0.792198, 0.797948, 0.804935,\
        0.813719, 0.830089, 0.852997, 0.874436, 0.886945, 0.889131, 0.884563, 0.874242, 0.862535, 0.851891,\
        0.843334, 0.837349, 0.833623, 0.831193, 0.829123, 0.826884, 0.824118, 0.820052, 0.813900, 0.805992,\
        0.797474, 0.786495, 0.772070, 0.755468, 0.738111, 0.721339, 0.706858, 0.695352, 0.685417, 0.677685,\
        0.671777, 0.667024, 0.662845, 0.658696, 0.654055, 0.648482, 0.641689, 0.633566, 0.624170, 0.613733,\
        0.602820, 0.592259, 0.582306, 0.572944, 0.564194, 0.556030, 0.548366, 0.541085, 0.534061, 0.527169,\
        0.520313, 0.513467, 0.506694, 0.500003, 0.493111, 0.486067, 0.479257, 0.472671, 0.466275, 0.460032,\
        0.453916, 0.447905, 0.441980, 0.436121, 0.430309, 0.424520, 0.418729, 0.412905, 0.407062, 0.401262,\
        0.395275],\
        [0.411149, 0.417732, 0.424593, 0.431639, 0.438787, 0.445969, 0.453127, 0.460203, 0.467142, 0.473890,\
        0.480410, 0.486676, 0.492601, 0.498036, 0.503201, 0.508264, 0.513188, 0.518063, 0.523031, 0.528198,\
        0.533631, 0.539359, 0.545388, 0.551698, 0.558249, 0.564984, 0.571852, 0.578875, 0.586273, 0.594538,\
        0.604454, 0.616192, 0.628964, 0.642062, 0.654975, 0.667315, 0.678755, 0.689238, 0.700210, 0.713117,\
        0.725991, 0.737819, 0.748433, 0.757330, 0.763851, 0.768028, 0.770775, 0.773377, 0.776756, 0.781220,\
        0.788088, 0.803506, 0.827634, 0.850235, 0.864717, 0.869277, 0.865685, 0.856665, 0.845751, 0.835671,\
        0.827821, 0.823130, 0.821071, 0.819981, 0.818571, 0.816522, 0.814168, 0.810497, 0.803719, 0.794517,\
        0.784476, 0.772595, 0.758273, 0.742298, 0.725814, 0.709801, 0.695908, 0.685476, 0.676722, 0.669479,\
        0.663634, 0.658774, 0.654393, 0.649984, 0.645092, 0.639383, 0.632677, 0.624931, 0.616182, 0.606447,\
        0.596160, 0.586140, 0.576577, 0.567535, 0.559038, 0.551068, 0.543564, 0.536432, 0.529553, 0.522788,\
        0.515994, 0.509109, 0.502230, 0.495485, 0.488663, 0.481752, 0.474941, 0.468324, 0.461915, 0.455699,\
        0.449653, 0.443746, 0.437945, 0.432212, 0.426513, 0.420822, 0.415118, 0.409346, 0.403406, 0.397591,\
        0.391729],\
        [0.407745, 0.414282, 0.421118, 0.428116, 0.435187, 0.442275, 0.449336, 0.456324, 0.463189, 0.469885,\
        0.476370, 0.482595, 0.488493, 0.494041, 0.499363, 0.504521, 0.509410, 0.514194, 0.519075, 0.524173,\
        0.529544, 0.535211, 0.541174, 0.547411, 0.553878, 0.560530, 0.567344, 0.574371, 0.581800, 0.590001,\
        0.599577, 0.611091, 0.623698, 0.636526, 0.649093, 0.661097, 0.672395, 0.683194, 0.694191, 0.705830,\
        0.716456, 0.725439, 0.733834, 0.741412, 0.747380, 0.751423, 0.753863, 0.755474, 0.757275, 0.760671,\
        0.768313, 0.783832, 0.806579, 0.829240, 0.844302, 0.849558, 0.846287, 0.837613, 0.827401, 0.818475,\
        0.811399, 0.808238, 0.808197, 0.808516, 0.807640, 0.805159, 0.802977, 0.800141, 0.792856, 0.782998,\
        0.771727, 0.758923, 0.744454, 0.729022, 0.713748, 0.699240, 0.686357, 0.676085, 0.667750, 0.660864,\
        0.655176, 0.650351, 0.645942, 0.641452, 0.636444, 0.630648, 0.624006, 0.616566, 0.608433, 0.599542,\
        0.589729, 0.580028, 0.570730, 0.561922, 0.553631, 0.545842, 0.538501, 0.531529, 0.524826, 0.518252,\
        0.511590, 0.504652, 0.497662, 0.490789, 0.483924, 0.477053, 0.470272, 0.463667, 0.457279, 0.451114,\
        0.445156, 0.439373, 0.433717, 0.428130, 0.422559, 0.416976, 0.411385, 0.405805, 0.400095, 0.394129,\
        0.388200],\
        [0.404464, 0.410974, 0.417764, 0.424654, 0.431584, 0.438526, 0.445449, 0.452314, 0.459075, 0.465685,\
        0.472100, 0.478275, 0.484177, 0.489814, 0.495249, 0.500449, 0.505233, 0.509918, 0.514747, 0.519826,\
        0.525189, 0.530856, 0.536834, 0.543105, 0.549629, 0.556370, 0.563322, 0.570547, 0.578204, 0.586564,\
        0.596011, 0.607111, 0.619217, 0.631470, 0.643502, 0.655102, 0.666246, 0.677150, 0.688181, 0.699120,\
        0.706032, 0.711896, 0.718501, 0.725449, 0.731245, 0.735434, 0.737843, 0.738854, 0.739656, 0.742333,\
        0.750327, 0.766478, 0.789342, 0.812044, 0.826845, 0.831812, 0.828025, 0.818885, 0.808970, 0.801979,\
        0.796578, 0.794400, 0.795386, 0.796490, 0.796150, 0.793533, 0.790930, 0.787254, 0.780277, 0.770534,\
        0.758949, 0.745785, 0.731199, 0.716007, 0.701781, 0.688767, 0.676984, 0.667041, 0.658851, 0.652113,\
        0.646557, 0.641868, 0.637608, 0.633232, 0.628221, 0.622315, 0.615635, 0.608339, 0.600445, 0.592029,\
        0.582880, 0.573586, 0.564562, 0.555971, 0.547872, 0.540257, 0.533073, 0.526248, 0.519711, 0.513374,\
        0.507016, 0.500143, 0.492979, 0.485910, 0.478921, 0.472019, 0.465250, 0.458672, 0.452327, 0.446235,\
        0.440391, 0.434762, 0.429290, 0.423889, 0.418471, 0.412990, 0.407471, 0.401972, 0.396461, 0.390636,\
        0.384659],\
        [0.401546, 0.408043, 0.414674, 0.421337, 0.428037, 0.434774, 0.441521, 0.448236, 0.454871, 0.461375,\
        0.467704, 0.473818, 0.479693, 0.485331, 0.490737, 0.495856, 0.500648, 0.505289, 0.510151, 0.515276,\
        0.520674, 0.526393, 0.532462, 0.538867, 0.545568, 0.552530, 0.559754, 0.567301, 0.575308, 0.583993,\
        0.593632, 0.604275, 0.615436, 0.626831, 0.638108, 0.649063, 0.659662, 0.669966, 0.679870, 0.688333,\
        0.693801, 0.698001, 0.703524, 0.710350, 0.715765, 0.720363, 0.722861, 0.723326, 0.723219, 0.724262,\
        0.731077, 0.747676, 0.771357, 0.794893, 0.810684, 0.815780, 0.811646, 0.802291, 0.793149, 0.787255,\
        0.783552, 0.782083, 0.782696, 0.783638, 0.783383, 0.781447, 0.778417, 0.773870, 0.766909, 0.757534,\
        0.746249, 0.733404, 0.719160, 0.704050, 0.690530, 0.678548, 0.667628, 0.658071, 0.649986, 0.643286,\
        0.637818, 0.633337, 0.629417, 0.625415, 0.620587, 0.614492, 0.607559, 0.600404, 0.592608, 0.584344,\
        0.575615, 0.566707, 0.557958, 0.549582, 0.541675, 0.534240, 0.527209, 0.520490, 0.514031, 0.507818,\
        0.501718, 0.495057, 0.487819, 0.480623, 0.473554, 0.466627, 0.459870, 0.453327, 0.447040, 0.441037,\
        0.435328, 0.429888, 0.424650, 0.419500, 0.414292, 0.408924, 0.403446, 0.397999, 0.392547, 0.386885,\
        0.380992],\
        [0.399529, 0.405653, 0.411881, 0.418164, 0.424552, 0.431043, 0.437594, 0.444147, 0.450645, 0.457034,\
        0.463267, 0.469305, 0.475121, 0.480700, 0.486024, 0.491063, 0.495825, 0.500507, 0.505486, 0.510674,\
        0.516107, 0.521901, 0.528123, 0.534745, 0.541708, 0.548967, 0.556521, 0.564420, 0.572762, 0.581685,\
        0.591298, 0.601401, 0.611779, 0.622320, 0.632681, 0.642710, 0.652351, 0.661499, 0.669814, 0.676605,\
        0.681509, 0.685551, 0.690519, 0.696177, 0.700615, 0.706184, 0.708273, 0.707802, 0.707340, 0.705903,\
        0.709854, 0.727212, 0.751563, 0.775376, 0.792605, 0.799153, 0.796518, 0.788316, 0.780366, 0.774637,\
        0.771298, 0.770042, 0.770279, 0.770836, 0.770558, 0.768857, 0.765626, 0.760641, 0.753596, 0.744521,\
        0.733763, 0.721734, 0.708666, 0.694509, 0.681093, 0.669210, 0.658577, 0.649199, 0.641129, 0.634368,\
        0.628904, 0.624648, 0.621246, 0.617967, 0.613729, 0.607541, 0.599894, 0.592708, 0.584962, 0.576713,\
        0.568138, 0.559449, 0.550901, 0.542707, 0.534990, 0.527751, 0.520895, 0.514264, 0.507752, 0.501488,\
        0.495311, 0.488800, 0.481844, 0.474762, 0.467739, 0.460847, 0.454132, 0.447638, 0.441415, 0.435506,\
        0.429942, 0.424714, 0.419761, 0.414952, 0.410067, 0.404864, 0.399456, 0.394076, 0.388640, 0.383032,\
        0.377207],\
        [0.397650, 0.403497, 0.409195, 0.415018, 0.421082, 0.427337, 0.433701, 0.440098, 0.446461, 0.452735,\
        0.458870, 0.464826, 0.470573, 0.476086, 0.481350, 0.486363, 0.491181, 0.495986, 0.500985, 0.506167,\
        0.511566, 0.517437, 0.523871, 0.530775, 0.538041, 0.545615, 0.553486, 0.561687, 0.570268, 0.579266,\
        0.588621, 0.598166, 0.608028, 0.617742, 0.627020, 0.635903, 0.644370, 0.652252, 0.659234, 0.664976,\
        0.669493, 0.673435, 0.677611, 0.681961, 0.686302, 0.690952, 0.691987, 0.690706, 0.689553, 0.687871,\
        0.691744, 0.708785, 0.732229, 0.755203, 0.772340, 0.780438, 0.780027, 0.774379, 0.767670, 0.762335,\
        0.759151, 0.757949, 0.758031, 0.758397, 0.758069, 0.756375, 0.752986, 0.747747, 0.740618, 0.731735,\
        0.721419, 0.710099, 0.698134, 0.685107, 0.672169, 0.660366, 0.649803, 0.640460, 0.632323, 0.625384,\
        0.619763, 0.615631, 0.612788, 0.610462, 0.607424, 0.601662, 0.593668, 0.585584, 0.577431, 0.569038,\
        0.560468, 0.551861, 0.543411, 0.535336, 0.527796, 0.520778, 0.514142, 0.507700, 0.501104, 0.494714,\
        0.488454, 0.481990, 0.475252, 0.468369, 0.461485, 0.454695, 0.448061, 0.441636, 0.435478, 0.429654,\
        0.424224, 0.419209, 0.414555, 0.410143, 0.405758, 0.400853, 0.395518, 0.390138, 0.384689, 0.379101,\
        0.373320],\
        [0.395566, 0.401099, 0.406254, 0.411744, 0.417590, 0.423672, 0.429883, 0.436140, 0.442379, 0.448545,\
        0.454588, 0.460470, 0.466156, 0.471624, 0.476865, 0.481897, 0.486790, 0.491673, 0.496678, 0.501797,\
        0.507059, 0.513053, 0.519792, 0.527010, 0.534567, 0.542410, 0.550528, 0.558933, 0.567644, 0.576651,\
        0.585876, 0.595177, 0.604392, 0.612976, 0.620916, 0.628546, 0.635823, 0.642549, 0.648461, 0.653382,\
        0.657398, 0.660876, 0.664194, 0.667448, 0.670564, 0.672976, 0.673273, 0.671828, 0.670210, 0.670316,\
        0.676609, 0.692379, 0.714274, 0.736447, 0.753320, 0.762040, 0.763206, 0.759420, 0.753930, 0.749228,\
        0.746469, 0.745596, 0.745868, 0.746337, 0.746079, 0.744366, 0.740804, 0.735330, 0.728082, 0.719296,\
        0.709266, 0.698385, 0.686965, 0.674974, 0.662912, 0.651510, 0.641111, 0.631834, 0.623676, 0.616515,\
        0.610494, 0.606260, 0.603789, 0.601992, 0.600105, 0.594688, 0.586667, 0.578169, 0.569662, 0.561123,\
        0.552542, 0.543972, 0.535539, 0.527484, 0.520099, 0.513344, 0.506880, 0.500681, 0.494247, 0.487722,\
        0.481327, 0.474873, 0.468278, 0.461583, 0.454875, 0.448234, 0.441721, 0.435387, 0.429295, 0.423531,\
        0.418202, 0.413377, 0.408990, 0.404880, 0.400925, 0.396498, 0.391320, 0.385999, 0.380590, 0.375047,\
        0.369319],\
        [0.392857, 0.397912, 0.402917, 0.408358, 0.414134, 0.420111, 0.426199, 0.432332, 0.438457, 0.444525,\
        0.450489, 0.456310, 0.461955, 0.467403, 0.472651, 0.477721, 0.482668, 0.487572, 0.492501, 0.497456,\
        0.502573, 0.508965, 0.516068, 0.523537, 0.531293, 0.539303, 0.547550, 0.556027, 0.564732, 0.573641,\
        0.582706, 0.591823, 0.600615, 0.607455, 0.614040, 0.620525, 0.626742, 0.632489, 0.637496, 0.641635,\
        0.644999, 0.647788, 0.650161, 0.652155, 0.653680, 0.654414, 0.653913, 0.652361, 0.650901, 0.651882,\
        0.659031, 0.674839, 0.696785, 0.720007, 0.736480, 0.745084, 0.746842, 0.743909, 0.739121, 0.735033,\
        0.733059, 0.732961, 0.733779, 0.734642, 0.734669, 0.733008, 0.729227, 0.723442, 0.716027, 0.707297,\
        0.697543, 0.687049, 0.676075, 0.664758, 0.653433, 0.642539, 0.632405, 0.623270, 0.615286, 0.608195,\
        0.601584, 0.597155, 0.594501, 0.592472, 0.589854, 0.584923, 0.577734, 0.569600, 0.561205, 0.552755,\
        0.544299, 0.535848, 0.527426, 0.519227, 0.511889, 0.505621, 0.499203, 0.492979, 0.486703, 0.480303,\
        0.473913, 0.467506, 0.461040, 0.454524, 0.448008, 0.441550, 0.435201, 0.428994, 0.422979, 0.417247,\
        0.411960, 0.407301, 0.403181, 0.399260, 0.395441, 0.391293, 0.386576, 0.381521, 0.376267, 0.370830,\
        0.365188],\
        [0.390119, 0.394889, 0.399776, 0.405168, 0.410888, 0.416759, 0.422720, 0.428729, 0.434746, 0.440726,\
        0.446626, 0.452404, 0.458028, 0.463481, 0.468759, 0.473882, 0.478889, 0.483831, 0.488755, 0.493717,\
        0.499151, 0.505753, 0.512903, 0.520410, 0.528211, 0.536249, 0.544482, 0.552874, 0.561393, 0.569991,\
        0.578577, 0.586934, 0.594384, 0.600380, 0.606269, 0.611814, 0.617140, 0.622101, 0.626289, 0.629598,\
        0.632196, 0.634204, 0.635600, 0.636322, 0.636368, 0.635862, 0.634861, 0.633313, 0.631826, 0.632264,\
        0.638509, 0.655030, 0.680045, 0.704861, 0.720361, 0.729159, 0.731077, 0.728337, 0.723736, 0.720009,\
        0.719257, 0.720358, 0.721829, 0.723255, 0.723843, 0.722356, 0.718248, 0.712017, 0.704397, 0.695765,\
        0.686247, 0.676083, 0.665489, 0.654688, 0.643958, 0.633574, 0.623734, 0.614693, 0.606891, 0.600427,\
        0.593901, 0.588819, 0.585303, 0.582348, 0.578868, 0.573929, 0.567409, 0.559895, 0.551939, 0.543833,\
        0.535706, 0.527589, 0.519380, 0.510992, 0.503070, 0.497489, 0.491315, 0.485028, 0.478777, 0.472497,\
        0.466209, 0.459917, 0.453603, 0.447274, 0.440963, 0.434724, 0.428593, 0.422577, 0.416687, 0.411003,\
        0.405690, 0.401150, 0.397302, 0.393609, 0.389886, 0.385910, 0.381524, 0.376771, 0.371735, 0.366454,\
        0.360928],\
        [0.387717, 0.392390, 0.397218, 0.402457, 0.408010, 0.413701, 0.419497, 0.425370, 0.431282, 0.437187,\
        0.443036, 0.448789, 0.454413, 0.459890, 0.465218, 0.470414, 0.475513, 0.480568, 0.485662, 0.490972,\
        0.496843, 0.503290, 0.510190, 0.517560, 0.525269, 0.533204, 0.541281, 0.549431, 0.557586, 0.565657,\
        0.573497, 0.580835, 0.587286, 0.592838, 0.597952, 0.602497, 0.607084, 0.611455, 0.614801, 0.617143,\
        0.618908, 0.620164, 0.620666, 0.620256, 0.618966, 0.617428, 0.616371, 0.615096, 0.613603, 0.612934,\
        0.615689, 0.632841, 0.659508, 0.684748, 0.703185, 0.713759, 0.715723, 0.713176, 0.708961, 0.705020,\
        0.706174, 0.708409, 0.710006, 0.711928, 0.713436, 0.712278, 0.707654, 0.700882, 0.693144, 0.684594,\
        0.675289, 0.665405, 0.655154, 0.644784, 0.634585, 0.624782, 0.615330, 0.606310, 0.598375, 0.591885,\
        0.585777, 0.580396, 0.575990, 0.572032, 0.567777, 0.562630, 0.556432, 0.549438, 0.542007, 0.534384,\
        0.526715, 0.519167, 0.511623, 0.503604, 0.495601, 0.489087, 0.483013, 0.476808, 0.470603, 0.464424,\
        0.458272, 0.452142, 0.446018, 0.439894, 0.433799, 0.427805, 0.421967, 0.416242, 0.410566, 0.405042,\
        0.399851, 0.395374, 0.391611, 0.388032, 0.384403, 0.380560, 0.376394, 0.371887, 0.367065, 0.361954,\
        0.356561],\
        [0.385681, 0.390315, 0.395125, 0.400223, 0.405494, 0.410939, 0.416544, 0.422275, 0.428089, 0.433931,\
        0.439747, 0.445489, 0.451127, 0.456642, 0.462034, 0.467321, 0.472537, 0.477743, 0.483036, 0.488568,\
        0.494497, 0.500838, 0.507618, 0.514846, 0.522397, 0.530130, 0.537933, 0.545711, 0.553368, 0.560788,\
        0.567816, 0.574251, 0.579917, 0.584833, 0.589147, 0.593001, 0.596834, 0.600584, 0.602876, 0.604128,\
        0.605050, 0.605634, 0.605341, 0.604071, 0.601706, 0.599179, 0.598277, 0.597273, 0.596017, 0.595628,\
        0.596289, 0.614100, 0.640075, 0.665357, 0.685705, 0.697601, 0.700362, 0.698827, 0.696293, 0.693614,\
        0.694920, 0.697012, 0.698244, 0.700332, 0.702766, 0.701917, 0.696812, 0.689808, 0.682045, 0.673615,\
        0.664548, 0.654956, 0.645033, 0.635029, 0.625265, 0.616174, 0.607536, 0.598583, 0.590612, 0.583630,\
        0.577305, 0.571574, 0.566453, 0.561666, 0.556758, 0.551333, 0.545229, 0.538573, 0.531625, 0.524534,\
        0.517271, 0.510182, 0.503769, 0.496177, 0.488333, 0.481211, 0.474701, 0.468409, 0.462229, 0.456146,\
        0.450151, 0.444224, 0.438332, 0.432443, 0.426561, 0.420797, 0.415324, 0.410051, 0.404657, 0.399389,\
        0.394478, 0.390103, 0.386255, 0.382655, 0.379061, 0.375309, 0.371295, 0.366972, 0.362332, 0.357379,\
        0.352117],\
        [0.383948, 0.388522, 0.393237, 0.398143, 0.403188, 0.408418, 0.413840, 0.419444, 0.425180, 0.430978,\
        0.436773, 0.442515, 0.448172, 0.453730, 0.459190, 0.464570, 0.469905, 0.475254, 0.480694, 0.486323,\
        0.492219, 0.498444, 0.505100, 0.512204, 0.519555, 0.527008, 0.534447, 0.541759, 0.548834, 0.555551,\
        0.561780, 0.567386, 0.572285, 0.576492, 0.580110, 0.583307, 0.586265, 0.588773, 0.590185, 0.590641,\
        0.590707, 0.590409, 0.589309, 0.587306, 0.584560, 0.581910, 0.580249, 0.579070, 0.578386, 0.579413,\
        0.585335, 0.601251, 0.624002, 0.647576, 0.667271, 0.679935, 0.685248, 0.686005, 0.684954, 0.683891,\
        0.684322, 0.685511, 0.686700, 0.688398, 0.689893, 0.688786, 0.684322, 0.677923, 0.670598, 0.662582,\
        0.653917, 0.644697, 0.635135, 0.625491, 0.615948, 0.607197, 0.600011, 0.591386, 0.583059, 0.575573,\
        0.568767, 0.562523, 0.556760, 0.551302, 0.545875, 0.540185, 0.534044, 0.527523, 0.520943, 0.514485,\
        0.507672, 0.500404, 0.494373, 0.487566, 0.480119, 0.472954, 0.466232, 0.459836, 0.453669, 0.447688,\
        0.441871, 0.436188, 0.430589, 0.424994, 0.419327, 0.413676, 0.408518, 0.403987, 0.398944, 0.393887,\
        0.389199, 0.384937, 0.381073, 0.377444, 0.373862, 0.370172, 0.366267, 0.362083, 0.357589, 0.352773,\
        0.347630],\
        [0.382472, 0.386950, 0.391516, 0.396216, 0.401080, 0.406110, 0.411364, 0.416879, 0.422571, 0.428344,\
        0.434125, 0.439867, 0.445539, 0.451132, 0.456649, 0.462110, 0.467548, 0.473009, 0.478555, 0.484244,\
        0.490123, 0.496223, 0.502734, 0.509678, 0.516731, 0.523827, 0.530834, 0.537616, 0.544059, 0.550054,\
        0.555497, 0.560300, 0.564419, 0.567876, 0.570750, 0.573145, 0.575103, 0.576464, 0.576987, 0.576725,\
        0.575930, 0.574662, 0.572774, 0.570255, 0.567364, 0.564647, 0.562573, 0.561242, 0.561135, 0.563843,\
        0.572074, 0.587760, 0.609008, 0.631008, 0.650051, 0.663483, 0.670803, 0.673583, 0.673957, 0.673631,\
        0.673604, 0.673954, 0.674494, 0.675154, 0.675310, 0.673814, 0.670193, 0.664930, 0.658588, 0.651387,\
        0.643372, 0.634638, 0.625469, 0.616342, 0.607363, 0.598174, 0.590964, 0.583237, 0.575144, 0.567400,\
        0.560148, 0.553366, 0.547005, 0.540980, 0.535158, 0.529309, 0.523102, 0.516484, 0.509998, 0.504129,\
        0.498182, 0.491152, 0.484617, 0.478023, 0.471069, 0.464125, 0.457430, 0.451032, 0.444914, 0.439055,\
        0.433434, 0.428027, 0.422789, 0.417603, 0.412276, 0.406638, 0.401354, 0.397413, 0.392994, 0.388275,\
        0.383819, 0.379710, 0.375909, 0.372307, 0.368759, 0.365131, 0.361317, 0.357245, 0.352870, 0.348167,\
        0.343124],\
        [0.381250, 0.385616, 0.390016, 0.394519, 0.399196, 0.403961, 0.409112, 0.414613, 0.420296, 0.426052,\
        0.431812, 0.437538, 0.443206, 0.448812, 0.454363, 0.459879, 0.465388, 0.470930, 0.476549, 0.482288,\
        0.488196, 0.494348, 0.500812, 0.507214, 0.513863, 0.520569, 0.527097, 0.533302, 0.539080, 0.544341,\
        0.549002, 0.553004, 0.556324, 0.558985, 0.561046, 0.562567, 0.563552, 0.563913, 0.563550, 0.562488,\
        0.560861, 0.558754, 0.556186, 0.553245, 0.550196, 0.547426, 0.545254, 0.543887, 0.543922, 0.547063,\
        0.555843, 0.571610, 0.592611, 0.614660, 0.634145, 0.648476, 0.657071, 0.661165, 0.662538, 0.662632,\
        0.662308, 0.661901, 0.661477, 0.660966, 0.660052, 0.658260, 0.655317, 0.651257, 0.646192, 0.640129,\
        0.633014, 0.624885, 0.616031, 0.607229, 0.599397, 0.590867, 0.582752, 0.574912, 0.566952, 0.559081,\
        0.551475, 0.544200, 0.537280, 0.530732, 0.524583, 0.518778, 0.512731, 0.505790, 0.499234, 0.493414,\
        0.487698, 0.481320, 0.474816, 0.468272, 0.461589, 0.454880, 0.448311, 0.441991, 0.435970, 0.430257,\
        0.424838, 0.419705, 0.414858, 0.410184, 0.405431, 0.400253, 0.394918, 0.390676, 0.386574, 0.382344,\
        0.378224, 0.374337, 0.370678, 0.367170, 0.363701, 0.360156, 0.356435, 0.352463, 0.348189, 0.343580,\
        0.338619],\
        [0.380306, 0.384572, 0.388806, 0.393052, 0.397352, 0.401955, 0.407210, 0.412748, 0.418416, 0.424129,\
        0.429837, 0.435512, 0.441141, 0.446724, 0.452273, 0.457805, 0.463348, 0.468931, 0.474587, 0.480345,\
        0.486231, 0.492253, 0.498295, 0.504427, 0.510887, 0.517221, 0.523221, 0.528800, 0.533887, 0.538407,\
        0.542290, 0.545489, 0.547994, 0.549828, 0.551038, 0.551677, 0.551760, 0.551234, 0.550024, 0.548145,\
        0.545723, 0.542894, 0.539747, 0.536412, 0.533156, 0.530346, 0.528248, 0.526910, 0.526496, 0.528595,\
        0.536612, 0.552372, 0.574079, 0.597794, 0.618970, 0.634409, 0.643625, 0.648391, 0.650426, 0.650853,\
        0.650349, 0.649296, 0.647913, 0.646316, 0.644530, 0.642537, 0.640228, 0.637374, 0.633712, 0.628990,\
        0.622990, 0.615631, 0.607184, 0.598372, 0.590650, 0.582842, 0.574820, 0.566826, 0.558797, 0.550777,\
        0.542867, 0.535154, 0.527716, 0.520640, 0.514064, 0.508341, 0.503116, 0.496065, 0.489222, 0.483119,\
        0.477217, 0.471112, 0.464826, 0.458432, 0.451943, 0.445424, 0.438992, 0.432777, 0.426879, 0.421339,\
        0.416121, 0.411202, 0.406674, 0.402490, 0.398145, 0.393455, 0.388579, 0.384186, 0.380141, 0.376217,\
        0.372421, 0.368799, 0.365343, 0.361991, 0.358651, 0.355222, 0.351610, 0.347739, 0.343555, 0.339025,\
        0.334128],\
        [0.379667, 0.383881, 0.388012, 0.392104, 0.396256, 0.400807, 0.405961, 0.411417, 0.416986, 0.422590,\
        0.428187, 0.433759, 0.439299, 0.444813, 0.450312, 0.455814, 0.461342, 0.466919, 0.472568, 0.478303,\
        0.484117, 0.489960, 0.495749, 0.501762, 0.507959, 0.513784, 0.519150, 0.524042, 0.528420, 0.532207,\
        0.535323, 0.537724, 0.539414, 0.540418, 0.540772, 0.540544, 0.539807, 0.538518, 0.536546, 0.533871,\
        0.530690, 0.527240, 0.523608, 0.519884, 0.516333, 0.513396, 0.511377, 0.510193, 0.509158, 0.508934,\
        0.515251, 0.531135, 0.554051, 0.579915, 0.603871, 0.620355, 0.629533, 0.634768, 0.637502, 0.638403,\
        0.637914, 0.636392, 0.634201, 0.631675, 0.629100, 0.626943, 0.625304, 0.623649, 0.621387, 0.618056,\
        0.613274, 0.606830, 0.598945, 0.590645, 0.582660, 0.574854, 0.566940, 0.558923, 0.550815, 0.542645,\
        0.534478, 0.526389, 0.518500, 0.510964, 0.503890, 0.497763, 0.492568, 0.486205, 0.479415, 0.473044,\
        0.466961, 0.460919, 0.454808, 0.448600, 0.442294, 0.435935, 0.429620, 0.423490, 0.417715, 0.412382,\
        0.407399, 0.402653, 0.398286, 0.394367, 0.390322, 0.386037, 0.381673, 0.377515, 0.373631, 0.369955,\
        0.366452, 0.363109, 0.359898, 0.356754, 0.353592, 0.350315, 0.346833, 0.343070, 0.338973, 0.334507,\
        0.329658],\
        [0.379338, 0.383571, 0.387701, 0.391789, 0.395959, 0.400436, 0.405402, 0.410641, 0.416003, 0.421410,\
        0.426825, 0.432230, 0.437624, 0.443013, 0.448408, 0.453825, 0.459282, 0.464799, 0.470391, 0.476066,\
        0.481816, 0.487620, 0.493484, 0.499425, 0.504973, 0.510123, 0.514722, 0.518888, 0.522572, 0.525661,\
        0.528039, 0.529658, 0.530553, 0.530766, 0.530286, 0.529189, 0.527715, 0.525839, 0.523249, 0.519809,\
        0.515872, 0.511925, 0.507922, 0.503846, 0.499926, 0.496687, 0.494399, 0.492879, 0.491277, 0.490088,\
        0.495031, 0.510274, 0.533534, 0.560757, 0.586564, 0.603888, 0.613635, 0.620024, 0.623900, 0.625573,\
        0.625370, 0.623598, 0.620814, 0.617675, 0.614321, 0.611939, 0.611030, 0.610437, 0.609331, 0.607183,\
        0.603433, 0.597833, 0.590538, 0.582687, 0.574832, 0.567038, 0.559181, 0.551191, 0.543050, 0.534778,\
        0.526432, 0.518061, 0.509791, 0.501951, 0.494668, 0.488124, 0.482155, 0.475897, 0.469362, 0.462968,\
        0.456828, 0.450846, 0.444899, 0.438891, 0.432774, 0.426570, 0.420357, 0.414271, 0.408575, 0.403475,\
        0.398768, 0.394253, 0.390057, 0.386172, 0.382321, 0.378378, 0.374417, 0.370582, 0.366957, 0.363548,\
        0.360334, 0.357284, 0.354349, 0.351458, 0.348519, 0.345432, 0.342105, 0.338462, 0.334448, 0.330034,\
        0.325214],\
        [0.379288, 0.383606, 0.387813, 0.391977, 0.396207, 0.400643, 0.405366, 0.410313, 0.415386, 0.420520,\
        0.425682, 0.430858, 0.436046, 0.441252, 0.446484, 0.451756, 0.457082, 0.462475, 0.467948, 0.473507,\
        0.479152, 0.484875, 0.490643, 0.496304, 0.501683, 0.505886, 0.509648, 0.513139, 0.516190, 0.518665,\
        0.520359, 0.521200, 0.521330, 0.520866, 0.519670, 0.517602, 0.515430, 0.513216, 0.510198, 0.506109,\
        0.501368, 0.497104, 0.492795, 0.488450, 0.484271, 0.480612, 0.477652, 0.475299, 0.473501, 0.473601,\
        0.479047, 0.492542, 0.514260, 0.541398, 0.566598, 0.585000, 0.596723, 0.604901, 0.610154, 0.612761,\
        0.613140, 0.611371, 0.608143, 0.605016, 0.601534, 0.598689, 0.598195, 0.598108, 0.597558, 0.595987,\
        0.592816, 0.587809, 0.581395, 0.574220, 0.566767, 0.559195, 0.551494, 0.543612, 0.535512, 0.527197,\
        0.518771, 0.510315, 0.501702, 0.493620, 0.486081, 0.479068, 0.472441, 0.465878, 0.459310, 0.452903,\
        0.446778, 0.440910, 0.435169, 0.429394, 0.423484, 0.417447, 0.411370, 0.405357, 0.399654, 0.394763,\
        0.390244, 0.385952, 0.381905, 0.378078, 0.374352, 0.370650, 0.366994, 0.363461, 0.360118, 0.356992,\
        0.354076, 0.351335, 0.348708, 0.346109, 0.343435, 0.340576, 0.337433, 0.333922, 0.329989, 0.325612,\
        0.320799],\
        [0.379451, 0.383898, 0.388224, 0.392483, 0.396756, 0.401124, 0.405623, 0.410263, 0.415006, 0.419814,\
        0.424667, 0.429558, 0.434484, 0.439452, 0.444465, 0.449532, 0.454660, 0.459858, 0.465133, 0.470488,\
        0.475920, 0.481411, 0.486913, 0.492308, 0.497083, 0.500390, 0.503677, 0.506615, 0.509092, 0.511098,\
        0.512188, 0.512215, 0.511591, 0.510603, 0.509148, 0.505886, 0.502926, 0.500526, 0.497055, 0.492648,\
        0.487640, 0.482918, 0.478337, 0.473800, 0.469415, 0.465380, 0.461874, 0.459097, 0.457633, 0.458994,\
        0.465483, 0.478802, 0.499558, 0.525417, 0.549324, 0.568017, 0.581325, 0.590654, 0.596841, 0.600215,\
        0.601385, 0.600026, 0.596714, 0.593765, 0.590924, 0.588301, 0.587263, 0.586854, 0.586181, 0.584618,\
        0.581720, 0.577337, 0.571722, 0.565291, 0.558397, 0.551222, 0.543811, 0.536148, 0.528191, 0.519889,\
        0.511325, 0.503061, 0.494427, 0.485925, 0.477954, 0.470435, 0.463255, 0.456276, 0.449468, 0.442941,\
        0.436820, 0.431114, 0.425652, 0.420165, 0.414475, 0.408605, 0.402708, 0.396897, 0.391377, 0.386461,\
        0.381993, 0.377810, 0.373860, 0.370103, 0.366479, 0.362947, 0.359516, 0.356233, 0.353152, 0.350303,\
        0.347689, 0.345274, 0.342984, 0.340716, 0.338348, 0.335757, 0.332826, 0.329464, 0.325610, 0.321251,\
        0.316416],\
        [0.379740, 0.384341, 0.388799, 0.393151, 0.397433, 0.401695, 0.405991, 0.410331, 0.414725, 0.419174,\
        0.423678, 0.428238, 0.432857, 0.437538, 0.442280, 0.447084, 0.451950, 0.456878, 0.461869, 0.466915,\
        0.472001, 0.477086, 0.482081, 0.486780, 0.490779, 0.493957, 0.496800, 0.499175, 0.501088, 0.502782,\
        0.503383, 0.502513, 0.501202, 0.499693, 0.498245, 0.494733, 0.490665, 0.487398, 0.483543, 0.479014,\
        0.474125, 0.469287, 0.464585, 0.459978, 0.455497, 0.451264, 0.447485, 0.444572, 0.443413, 0.445597,\
        0.453155, 0.467627, 0.488866, 0.513092, 0.535425, 0.553606, 0.567430, 0.577412, 0.584133, 0.588076,\
        0.589644, 0.588885, 0.586490, 0.583785, 0.581196, 0.578974, 0.577520, 0.576540, 0.575423, 0.573636,\
        0.570820, 0.566856, 0.561868, 0.556106, 0.549805, 0.543113, 0.536085, 0.528730, 0.521045, 0.512949,\
        0.504181, 0.495467, 0.487042, 0.478432, 0.470106, 0.462144, 0.454495, 0.447090, 0.439933, 0.433156,\
        0.426966, 0.421442, 0.416349, 0.411220, 0.405733, 0.400013, 0.394267, 0.388666, 0.383354, 0.378477,\
        0.374016, 0.369872, 0.365976, 0.362277, 0.358733, 0.355323, 0.352054, 0.348964, 0.346101, 0.343506,\
        0.341188, 0.339110, 0.337186, 0.335287, 0.333267, 0.330979, 0.328294, 0.325103, 0.321333, 0.316967,\
        0.312071],\
        [0.380060, 0.384822, 0.389412, 0.393852, 0.398158, 0.402307, 0.406357, 0.410383, 0.414417, 0.418485,\
        0.422609, 0.426806, 0.431083, 0.435438, 0.439867, 0.444359, 0.448904, 0.453494, 0.458117, 0.462757,\
        0.467383, 0.471936, 0.476304, 0.480308, 0.483745, 0.486577, 0.488928, 0.490804, 0.492263, 0.493306,\
        0.493311, 0.492010, 0.490204, 0.488191, 0.485865, 0.482411, 0.478235, 0.474166, 0.469937, 0.465404,\
        0.460704, 0.456031, 0.451462, 0.446983, 0.442594, 0.438381, 0.434554, 0.431553, 0.430338, 0.432555,\
        0.440395, 0.455552, 0.477338, 0.501891, 0.523187, 0.540787, 0.554777, 0.565050, 0.572056, 0.576327,\
        0.578250, 0.578145, 0.576601, 0.574416, 0.572129, 0.570054, 0.568355, 0.566894, 0.565306, 0.563214,\
        0.560352, 0.556621, 0.552078, 0.546864, 0.541114, 0.534916, 0.528300, 0.521261, 0.513848, 0.506181,\
        0.497780, 0.488645, 0.479754, 0.470979, 0.462407, 0.454123, 0.446122, 0.438354, 0.430813, 0.423650,\
        0.417242, 0.411869, 0.407198, 0.402500, 0.397142, 0.391565, 0.385997, 0.380588, 0.375451, 0.370668,\
        0.366246, 0.362135, 0.358278, 0.354626, 0.351145, 0.347820, 0.344663, 0.341708, 0.339010, 0.336627,\
        0.334585, 0.332853, 0.331323, 0.329830, 0.328192, 0.326240, 0.323830, 0.320844, 0.317182, 0.312791,\
        0.307775],\
        [0.380318, 0.385242, 0.389951, 0.394448, 0.398732, 0.402766, 0.406596, 0.410310, 0.413972, 0.417640,\
        0.421363, 0.425175, 0.429087, 0.433094, 0.437179, 0.441321, 0.445499, 0.449692, 0.453880, 0.458038,\
        0.462128, 0.466084, 0.469806, 0.473164, 0.476039, 0.478393, 0.480245, 0.481610, 0.482488, 0.482766,\
        0.482162, 0.480615, 0.478459, 0.475929, 0.472949, 0.469291, 0.465107, 0.460759, 0.456347, 0.451885,\
        0.447444, 0.443104, 0.438885, 0.434749, 0.430651, 0.426664, 0.422988, 0.419888, 0.418194, 0.419481,\
        0.426248, 0.441364, 0.463696, 0.488496, 0.510179, 0.528684, 0.543116, 0.553420, 0.560509, 0.565034,\
        0.567375, 0.567877, 0.567016, 0.565361, 0.563388, 0.561408, 0.559549, 0.557740, 0.555763, 0.553371,\
        0.550386, 0.546748, 0.542498, 0.537708, 0.532432, 0.526693, 0.520479, 0.513707, 0.506361, 0.498665,\
        0.490512, 0.481561, 0.472494, 0.463544, 0.454797, 0.446317, 0.438117, 0.430145, 0.422318, 0.414688,\
        0.407807, 0.402500, 0.398092, 0.393575, 0.388511, 0.383172, 0.377827, 0.372620, 0.367654, 0.362990,\
        0.358642, 0.354585, 0.350776, 0.347172, 0.343745, 0.340483, 0.337397, 0.334523, 0.331927, 0.329694,\
        0.327895, 0.326513, 0.325407, 0.324352, 0.323118, 0.321513, 0.319393, 0.316647, 0.313168, 0.308804,\
        0.303572],\
        [0.380426, 0.385511, 0.390331, 0.394861, 0.399083, 0.403003, 0.406632, 0.410031, 0.413300, 0.416545,\
        0.419850, 0.423266, 0.426807, 0.430456, 0.434184, 0.437955, 0.441731, 0.445484, 0.449187, 0.452812,\
        0.456322, 0.459658, 0.462738, 0.465468, 0.467765, 0.469582, 0.470904, 0.471715, 0.471979, 0.471599,\
        0.470458, 0.468562, 0.466072, 0.463132, 0.459748, 0.455882, 0.451631, 0.447205, 0.442780, 0.438477,\
        0.434372, 0.430475, 0.426758, 0.423149, 0.419497, 0.415829, 0.412571, 0.409493, 0.407102, 0.407093,\
        0.411005, 0.426030, 0.449076, 0.474410, 0.497897, 0.518225, 0.532365, 0.542205, 0.549302, 0.554172,\
        0.557011, 0.558077, 0.557769, 0.556557, 0.554863, 0.552981, 0.551040, 0.549011, 0.546753, 0.544103,\
        0.540952, 0.537299, 0.533216, 0.528749, 0.523860, 0.518495, 0.512649, 0.506185, 0.498833, 0.491035,\
        0.482803, 0.474054, 0.465050, 0.456054, 0.447222, 0.438658, 0.430420, 0.422489, 0.414691, 0.406855,\
        0.399372, 0.393944, 0.389379, 0.384821, 0.379966, 0.374890, 0.369776, 0.364771, 0.359973, 0.355443,\
        0.351198, 0.347223, 0.343482, 0.339938, 0.336566, 0.333358, 0.330322, 0.327485, 0.324918, 0.322751,\
        0.321137, 0.320108, 0.319463, 0.318868, 0.318030, 0.316745, 0.314880, 0.312365, 0.309154, 0.305124,\
        0.299742],\
        [0.380299, 0.385554, 0.390498, 0.395080, 0.399256, 0.403058, 0.406451, 0.409490, 0.412322, 0.415112,\
        0.417987, 0.421011, 0.424191, 0.427494, 0.430869, 0.434261, 0.437619, 0.440903, 0.444086, 0.447145,\
        0.450051, 0.452760, 0.455209, 0.457324, 0.459037, 0.460296, 0.461068, 0.461322, 0.461009, 0.460060,\
        0.458424, 0.456133, 0.453309, 0.450063, 0.446433, 0.442422, 0.438113, 0.433674, 0.429308, 0.425220,\
        0.421510, 0.418111, 0.414941, 0.411969, 0.408933, 0.405359, 0.402663, 0.400103, 0.396962, 0.396306,\
        0.398167, 0.413161, 0.436268, 0.462630, 0.488619, 0.508912, 0.521427, 0.530746, 0.538252, 0.543775,\
        0.547220, 0.548799, 0.548914, 0.548026, 0.546548, 0.544771, 0.542827, 0.540704, 0.538288, 0.535444,\
        0.532104, 0.528333, 0.524302, 0.520092, 0.515526, 0.510374, 0.504712, 0.498526, 0.491276, 0.483381,\
        0.475076, 0.466391, 0.457467, 0.448494, 0.439643, 0.431058, 0.422864, 0.415168, 0.407842, 0.400224,\
        0.392859, 0.386701, 0.381533, 0.376684, 0.371814, 0.366867, 0.361922, 0.357082, 0.352437, 0.348044,\
        0.343922, 0.340058, 0.336413, 0.332949, 0.329642, 0.326493, 0.323510, 0.320698, 0.318096, 0.315872,\
        0.314344, 0.313678, 0.313550, 0.313406, 0.312901, 0.311861, 0.310149, 0.307740, 0.304668, 0.300891,\
        0.295959],\
        [0.379848, 0.385281, 0.390379, 0.395066, 0.399276, 0.402934, 0.406026, 0.408627, 0.410956, 0.413255,\
        0.415699, 0.418355, 0.421206, 0.424192, 0.427237, 0.430261, 0.433196, 0.435994, 0.438632, 0.441101,\
        0.443392, 0.445477, 0.447311, 0.448830, 0.449967, 0.450669, 0.450899, 0.450622, 0.449792, 0.448348,\
        0.446253, 0.443557, 0.440404, 0.436941, 0.433193, 0.429122, 0.424796, 0.420384, 0.416087, 0.412208,\
        0.408938, 0.406024, 0.403301, 0.400815, 0.398525, 0.395163, 0.392424, 0.390045, 0.387706, 0.387350,\
        0.391501, 0.404384, 0.425942, 0.451474, 0.476505, 0.496338, 0.508402, 0.518775, 0.527643, 0.534155,\
        0.538209, 0.540165, 0.540528, 0.539816, 0.538472, 0.536801, 0.534934, 0.532851, 0.530424, 0.527494,\
        0.523974, 0.519965, 0.515835, 0.511854, 0.507640, 0.502546, 0.496779, 0.490512, 0.483404, 0.475590,\
        0.467307, 0.458679, 0.449829, 0.440903, 0.432060, 0.423467, 0.415273, 0.407660, 0.400892, 0.393729,\
        0.386441, 0.379983, 0.374330, 0.369140, 0.364133, 0.359195, 0.354330, 0.349600, 0.345075, 0.340811,\
        0.336829, 0.333106, 0.329589, 0.326227, 0.322998, 0.319921, 0.317029, 0.314292, 0.311650, 0.309226,\
        0.307597, 0.307326, 0.307798, 0.307990, 0.307661, 0.306761, 0.305067, 0.302563, 0.299425, 0.295472,\
        0.290515],\
        [0.378971, 0.384581, 0.389851, 0.394681, 0.398936, 0.402515, 0.405277, 0.407350, 0.409100, 0.410883,\
        0.412923, 0.415266, 0.417843, 0.420559, 0.423311, 0.425993, 0.428511, 0.430811, 0.432883, 0.434743,\
        0.436410, 0.437883, 0.439127, 0.440079, 0.440665, 0.440828, 0.440536, 0.439771, 0.438496, 0.436641,\
        0.434141, 0.431030, 0.427532, 0.423933, 0.420222, 0.416176, 0.411888, 0.407585, 0.403419, 0.399690,\
        0.396827, 0.394277, 0.391880, 0.389653, 0.387420, 0.384822, 0.382399, 0.380507, 0.379546, 0.380867,\
        0.386436, 0.398330, 0.418130, 0.441260, 0.463738, 0.482166, 0.495813, 0.508042, 0.518608, 0.525882,\
        0.530255, 0.532317, 0.532690, 0.531970, 0.530660, 0.529088, 0.527380, 0.525483, 0.523225, 0.520388,\
        0.516809, 0.512487, 0.507980, 0.504163, 0.500287, 0.495182, 0.489205, 0.482668, 0.475492, 0.467723,\
        0.459501, 0.450956, 0.442198, 0.433338, 0.424519, 0.415932, 0.407751, 0.400053, 0.393188, 0.386484,\
        0.379672, 0.373268, 0.367421, 0.362010, 0.356866, 0.351882, 0.347034, 0.342355, 0.337909, 0.333756,\
        0.329924, 0.326376, 0.323030, 0.319798, 0.316650, 0.313638, 0.310884, 0.308361, 0.305817, 0.303197,\
        0.301197, 0.301319, 0.302416, 0.302541, 0.302131, 0.301271, 0.299493, 0.296804, 0.293459, 0.289348,\
        0.284338],\
        [0.377562, 0.383321, 0.388749, 0.393751, 0.398170, 0.401724, 0.404058, 0.405511, 0.406640, 0.407916,\
        0.409629, 0.411754, 0.414125, 0.416619, 0.419126, 0.421506, 0.423623, 0.425413, 0.426893, 0.428123,\
        0.429162, 0.430039, 0.430730, 0.431163, 0.431239, 0.430886, 0.430092, 0.428881, 0.427243, 0.425091,\
        0.422284, 0.418765, 0.414831, 0.411148, 0.407693, 0.403734, 0.399543, 0.395464, 0.391595, 0.388178,\
        0.385405, 0.383014, 0.380815, 0.378737, 0.376686, 0.374641, 0.372909, 0.371972, 0.372528, 0.375733,\
        0.383058, 0.395714, 0.412998, 0.432837, 0.453199, 0.471639, 0.487399, 0.501376, 0.512026, 0.519238,\
        0.523484, 0.525332, 0.525448, 0.524516, 0.523124, 0.521639, 0.520162, 0.518592, 0.516696, 0.514180,\
        0.510848, 0.506631, 0.501306, 0.497573, 0.493272, 0.487963, 0.481770, 0.474972, 0.467646, 0.459851,\
        0.451681, 0.443240, 0.434614, 0.425867, 0.417085, 0.408517, 0.400433, 0.392892, 0.385899, 0.379245,\
        0.372735, 0.366500, 0.360649, 0.355156, 0.349934, 0.344906, 0.340043, 0.335368, 0.330949, 0.326875,\
        0.323197, 0.319869, 0.316747, 0.313682, 0.310620, 0.307626, 0.304956, 0.302874, 0.300498, 0.298122,\
        0.296230, 0.296117, 0.296882, 0.296784, 0.296172, 0.295089, 0.293185, 0.290418, 0.286917, 0.282669,\
        0.277618],\
        [0.375527, 0.381369, 0.386880, 0.391976, 0.396509, 0.400068, 0.402010, 0.402916, 0.403512, 0.404332,\
        0.405875, 0.407906, 0.410109, 0.412403, 0.414724, 0.416860, 0.418596, 0.419857, 0.420709, 0.421283,\
        0.421692, 0.421996, 0.422185, 0.422171, 0.421798, 0.420955, 0.419663, 0.418026, 0.416096, 0.413770,\
        0.410835, 0.407018, 0.402597, 0.398756, 0.395531, 0.391804, 0.387844, 0.384040, 0.380502, 0.377365,\
        0.374692, 0.372380, 0.370295, 0.368365, 0.366577, 0.365037, 0.364055, 0.364147, 0.366046, 0.370716,\
        0.379196, 0.392010, 0.407349, 0.425273, 0.444846, 0.463809, 0.481055, 0.495916, 0.506759, 0.513832,\
        0.517787, 0.519208, 0.518836, 0.517482, 0.515888, 0.514477, 0.513276, 0.512115, 0.510722, 0.508673,\
        0.505720, 0.502232, 0.498237, 0.492475, 0.486933, 0.480951, 0.474387, 0.467317, 0.459822, 0.451967,\
        0.443834, 0.435510, 0.427071, 0.418542, 0.409873, 0.401245, 0.393319, 0.385870, 0.378869, 0.372228,\
        0.365858, 0.359768, 0.353991, 0.348515, 0.343286, 0.338248, 0.333368, 0.328663, 0.324212, 0.320155,\
        0.316621, 0.313562, 0.310733, 0.307872, 0.304913, 0.301982, 0.299190, 0.297394, 0.295257, 0.293149,\
        0.291585, 0.291005, 0.290949, 0.290605, 0.289768, 0.288390, 0.286308, 0.283455, 0.279855, 0.275518,\
        0.270437],\
        [0.372811, 0.378647, 0.384119, 0.389122, 0.393467, 0.396795, 0.398676, 0.399480, 0.399856, 0.400344,\
        0.401893, 0.403907, 0.405858, 0.407913, 0.410140, 0.412119, 0.413489, 0.414179, 0.414361, 0.414256,\
        0.414045, 0.413807, 0.413547, 0.413182, 0.412450, 0.411140, 0.409342, 0.407279, 0.405086, 0.402620,\
        0.399657, 0.395753, 0.391319, 0.387315, 0.383825, 0.380289, 0.376679, 0.373203, 0.369989, 0.367116,\
        0.364602, 0.362396, 0.360428, 0.358666, 0.357160, 0.356077, 0.355750, 0.356689, 0.359553, 0.365061,\
        0.373748, 0.385500, 0.399873, 0.418153, 0.438007, 0.457466, 0.475359, 0.490564, 0.501960, 0.509310,\
        0.513033, 0.513949, 0.512938, 0.510961, 0.509042, 0.507716, 0.506780, 0.505959, 0.505060, 0.503514,\
        0.500553, 0.496948, 0.492568, 0.486844, 0.480551, 0.473939, 0.466957, 0.459611, 0.451948, 0.444026,\
        0.435913, 0.427695, 0.419463, 0.411267, 0.402996, 0.394457, 0.386441, 0.378982, 0.371979, 0.365368,\
        0.359091, 0.353124, 0.347455, 0.342061, 0.336896, 0.331903, 0.327037, 0.322293, 0.317747, 0.313601,\
        0.310153, 0.307410, 0.304923, 0.302290, 0.299401, 0.296575, 0.293990, 0.291860, 0.289874, 0.288001,\
        0.286520, 0.285572, 0.284921, 0.284154, 0.283007, 0.281345, 0.279055, 0.276070, 0.272373, 0.267967,\
        0.262859],\
        [0.369410, 0.375160, 0.380493, 0.385286, 0.389349, 0.392429, 0.394362, 0.395378, 0.395927, 0.396613,\
        0.397992, 0.399885, 0.401329, 0.403138, 0.405411, 0.407317, 0.408309, 0.408360, 0.407851, 0.407078,\
        0.406295, 0.405563, 0.404875, 0.404240, 0.403256, 0.401514, 0.399268, 0.396835, 0.394346, 0.391628,\
        0.388428, 0.384601, 0.380422, 0.376430, 0.372788, 0.369339, 0.366016, 0.362879, 0.359998, 0.357407,\
        0.355103, 0.353056, 0.351238, 0.349662, 0.348416, 0.347699, 0.347856, 0.349373, 0.352841, 0.358835,\
        0.367728, 0.379574, 0.394538, 0.412751, 0.432489, 0.452164, 0.470436, 0.486096, 0.498091, 0.505663,\
        0.509162, 0.509608, 0.507985, 0.505250, 0.502869, 0.501704, 0.500901, 0.500150, 0.499340, 0.498068,\
        0.494916, 0.490877, 0.486003, 0.480163, 0.473619, 0.466643, 0.459342, 0.451764, 0.443958, 0.435979,\
        0.427876, 0.419719, 0.411627, 0.403658, 0.395701, 0.387507, 0.379560, 0.372110, 0.365134, 0.358580,\
        0.352403, 0.346564, 0.341033, 0.335773, 0.330736, 0.325863, 0.321080, 0.316344, 0.311698, 0.307320,\
        0.303802, 0.301367, 0.299128, 0.296629, 0.293874, 0.291156, 0.288648, 0.286423, 0.284412, 0.282593,\
        0.281047, 0.279794, 0.278692, 0.277497, 0.275997, 0.274040, 0.271522, 0.268373, 0.264563, 0.260084,\
        0.254940],\
        [0.365372, 0.370989, 0.376142, 0.380713, 0.384549, 0.387504, 0.389537, 0.390819, 0.391698, 0.392609,\
        0.393857, 0.395313, 0.396637, 0.398346, 0.400510, 0.402325, 0.402903, 0.402252, 0.401166, 0.399834,\
        0.398594, 0.397455, 0.396367, 0.395362, 0.394031, 0.392013, 0.389534, 0.386832, 0.383998, 0.380942,\
        0.377525, 0.373724, 0.369728, 0.365838, 0.362226, 0.358901, 0.355840, 0.353042, 0.350510, 0.348233,\
        0.346185, 0.344344, 0.342707, 0.341320, 0.340290, 0.339825, 0.340261, 0.342076, 0.345848, 0.352155,\
        0.361434, 0.373923, 0.389667, 0.407943, 0.427621, 0.447507, 0.466157, 0.482311, 0.494834, 0.502725,\
        0.506041, 0.506160, 0.504278, 0.501197, 0.498347, 0.496991, 0.495956, 0.494910, 0.493661, 0.491729,\
        0.488510, 0.484184, 0.478955, 0.472896, 0.466169, 0.458976, 0.451462, 0.443715, 0.435806, 0.427800,\
        0.419727, 0.411586, 0.403543, 0.395710, 0.387914, 0.380088, 0.372419, 0.365115, 0.358241, 0.351793,\
        0.345744, 0.340059, 0.334695, 0.329609, 0.324758, 0.320087, 0.315501, 0.310874, 0.306251, 0.301790,\
        0.297980, 0.295522, 0.293246, 0.290807, 0.288213, 0.285615, 0.283152, 0.280885, 0.278814, 0.276927,\
        0.275232, 0.273699, 0.272220, 0.270636, 0.268782, 0.266525, 0.263764, 0.260433, 0.256491, 0.251924,\
        0.246731],\
        [0.360771, 0.366241, 0.371222, 0.375610, 0.379299, 0.382211, 0.384357, 0.385876, 0.387021, 0.388077,\
        0.389227, 0.390470, 0.391781, 0.393331, 0.395062, 0.396466, 0.396666, 0.395668, 0.394241, 0.392626,\
        0.391041, 0.389547, 0.388104, 0.386628, 0.384876, 0.382655, 0.380022, 0.377112, 0.373996, 0.370656,\
        0.367055, 0.363224, 0.359313, 0.355527, 0.352027, 0.348884, 0.346106, 0.343665, 0.341513, 0.339590,\
        0.337838, 0.336233, 0.334795, 0.333589, 0.332727, 0.332395, 0.332901, 0.334725, 0.338491, 0.344877,\
        0.354445, 0.367454, 0.383680, 0.402378, 0.422653, 0.443016, 0.462137, 0.478858, 0.491959, 0.500226,\
        0.503440, 0.503335, 0.501523, 0.498773, 0.495805, 0.493594, 0.491826, 0.490084, 0.488062, 0.485387,\
        0.481741, 0.477094, 0.471559, 0.465257, 0.458339, 0.450974, 0.443310, 0.435450, 0.427464, 0.419460,\
        0.411520, 0.403437, 0.395417, 0.387674, 0.380029, 0.372457, 0.365057, 0.357961, 0.351250, 0.344954,\
        0.339070, 0.333569, 0.328403, 0.323516, 0.318873, 0.314462, 0.310210, 0.305844, 0.301222, 0.296977,\
        0.293147, 0.290157, 0.287579, 0.285053, 0.282492, 0.279946, 0.277492, 0.275180, 0.273020, 0.271007,\
        0.269122, 0.267317, 0.265505, 0.263568, 0.261379, 0.258826, 0.255818, 0.252290, 0.248201, 0.243530,\
        0.238271],\
        [0.355689, 0.361023, 0.365860, 0.370115, 0.373717, 0.376627, 0.378873, 0.380566, 0.381886, 0.383025,\
        0.384122, 0.385236, 0.386402, 0.387647, 0.388841, 0.389608, 0.389497, 0.388494, 0.386975, 0.385232,\
        0.383441, 0.381673, 0.379907, 0.378054, 0.375968, 0.373534, 0.370748, 0.367667, 0.364340, 0.360786,\
        0.357027, 0.353132, 0.349237, 0.345521, 0.342150, 0.339225, 0.336767, 0.334723, 0.332996, 0.331471,\
        0.330046, 0.328686, 0.327439, 0.326402, 0.325675, 0.325380, 0.325763, 0.327291, 0.330679, 0.336791,\
        0.346402, 0.359861, 0.376838, 0.396391, 0.417448, 0.438277, 0.457877, 0.475314, 0.489319, 0.498307,\
        0.501638, 0.500848, 0.499034, 0.496452, 0.493470, 0.490651, 0.488054, 0.485452, 0.482569, 0.479122,\
        0.474898, 0.469825, 0.463950, 0.457375, 0.450239, 0.442701, 0.434918, 0.426998, 0.418961, 0.410878,\
        0.403126, 0.395333, 0.387373, 0.379660, 0.372127, 0.364746, 0.357569, 0.350682, 0.344158, 0.338042,\
        0.332347, 0.327060, 0.322127, 0.317449, 0.312980, 0.308790, 0.304887, 0.301147, 0.296532, 0.292182,\
        0.288321, 0.285010, 0.282106, 0.279391, 0.276751, 0.274171, 0.271674, 0.269285, 0.267011, 0.264840,\
        0.262744, 0.260675, 0.258557, 0.256296, 0.253793, 0.250954, 0.247700, 0.243969, 0.239722, 0.234932,\
        0.229591],\
        [0.350202, 0.355424, 0.360151, 0.364318, 0.367872, 0.370794, 0.373113, 0.374917, 0.376334, 0.377504,\
        0.378538, 0.379507, 0.380441, 0.381318, 0.382010, 0.382278, 0.381901, 0.380863, 0.379348, 0.377569,\
        0.375673, 0.373729, 0.371732, 0.369616, 0.367291, 0.364678, 0.361754, 0.358535, 0.355053, 0.351342,\
        0.347452, 0.343472, 0.339543, 0.335850, 0.332587, 0.329892, 0.327788, 0.326189, 0.324944, 0.323868,\
        0.322788, 0.321649, 0.320547, 0.319660, 0.319075, 0.318780, 0.318887, 0.319820, 0.322402, 0.327769,\
        0.337084, 0.350944, 0.368960, 0.389986, 0.411788, 0.432661, 0.452733, 0.470983, 0.486199, 0.497088,\
        0.500661, 0.498909, 0.496741, 0.494003, 0.490820, 0.487509, 0.484189, 0.480770, 0.477066, 0.472873,\
        0.468035, 0.462480, 0.456224, 0.449343, 0.441955, 0.434216, 0.426307, 0.418392, 0.410453, 0.402243,\
        0.394395, 0.386836, 0.379172, 0.371609, 0.364216, 0.357007, 0.350022, 0.343328, 0.336991, 0.331058,\
        0.325557, 0.320500, 0.315850, 0.311430, 0.307035, 0.302961, 0.299182, 0.295525, 0.291406, 0.287170,\
        0.283271, 0.279784, 0.276644, 0.273734, 0.270961, 0.268284, 0.265696, 0.263197, 0.260781, 0.258432,\
        0.256119, 0.253791, 0.251386, 0.248826, 0.246029, 0.242917, 0.239421, 0.235485, 0.231071, 0.226152,\
        0.220714],\
        [0.344377, 0.349513, 0.354167, 0.358280, 0.361811, 0.364746, 0.367109, 0.368968, 0.370417, 0.371564,\
        0.372501, 0.373293, 0.373962, 0.374481, 0.374755, 0.374647, 0.374044, 0.372939, 0.371432, 0.369657,\
        0.367724, 0.365691, 0.363560, 0.361289, 0.358816, 0.356084, 0.353062, 0.349749, 0.346164, 0.342344,\
        0.338352, 0.334285, 0.330286, 0.326556, 0.323350, 0.320878, 0.319160, 0.318045, 0.317335, 0.316768,\
        0.316044, 0.315055, 0.313982, 0.313207, 0.312834, 0.312580, 0.312340, 0.312441, 0.313805, 0.317865,\
        0.326435, 0.340588, 0.359506, 0.381428, 0.403810, 0.425336, 0.446287, 0.465181, 0.480910, 0.492235,\
        0.496921, 0.496175, 0.493929, 0.490975, 0.487536, 0.483812, 0.479910, 0.475808, 0.471396, 0.466537,\
        0.461118, 0.455080, 0.448431, 0.441229, 0.433566, 0.425584, 0.417484, 0.409520, 0.401886, 0.393876,\
        0.385937, 0.378347, 0.370881, 0.363523, 0.356312, 0.349279, 0.342471, 0.335951, 0.329790, 0.324034,\
        0.318704, 0.313843, 0.309504, 0.305546, 0.301205, 0.297079, 0.293340, 0.289617, 0.285719, 0.281732,\
        0.277900, 0.274340, 0.271053, 0.267981, 0.265064, 0.262261, 0.259548, 0.256911, 0.254335, 0.251795,\
        0.249259, 0.246681, 0.244005, 0.241164, 0.238090, 0.234717, 0.230985, 0.226845, 0.222260, 0.217201,\
        0.211655],\
        [0.338271, 0.343351, 0.347963, 0.352052, 0.355576, 0.358521, 0.360900, 0.362765, 0.364190, 0.365265,\
        0.366068, 0.366658, 0.367059, 0.367255, 0.367189, 0.366787, 0.365994, 0.364813, 0.363302, 0.361547,\
        0.359622, 0.357569, 0.355391, 0.353057, 0.350522, 0.347740, 0.344677, 0.341323, 0.337689, 0.333812,\
        0.329758, 0.325627, 0.321552, 0.317725, 0.314488, 0.312219, 0.310922, 0.310283, 0.310116, 0.310143,\
        0.309762, 0.308842, 0.307573, 0.306805, 0.306797, 0.306644, 0.306077, 0.305282, 0.305234, 0.307512,\
        0.314837, 0.329437, 0.349167, 0.371541, 0.394118, 0.417277, 0.439131, 0.458069, 0.473481, 0.484424,\
        0.490016, 0.491058, 0.489583, 0.486822, 0.483308, 0.479322, 0.475005, 0.470387, 0.465420, 0.460014,\
        0.454089, 0.447605, 0.440583, 0.433078, 0.425153, 0.416910, 0.408575, 0.400413, 0.392737, 0.385072,\
        0.377375, 0.369893, 0.362603, 0.355461, 0.348459, 0.341614, 0.334974, 0.328610, 0.322612, 0.317039,\
        0.311860, 0.307093, 0.302932, 0.299348, 0.295514, 0.291300, 0.287413, 0.283628, 0.279807, 0.275966,\
        0.272215, 0.268638, 0.265261, 0.262064, 0.259012, 0.256071, 0.253216, 0.250424, 0.247672, 0.244935,\
        0.242177, 0.239355, 0.236419, 0.233313, 0.229977, 0.226355, 0.222396, 0.218053, 0.213293, 0.208089,\
        0.202425],\
        [0.331931, 0.336986, 0.341588, 0.345679, 0.349211, 0.352160, 0.354531, 0.356360, 0.357714, 0.358670,\
        0.359306, 0.359680, 0.359823, 0.359734, 0.359386, 0.358746, 0.357793, 0.356537, 0.355018, 0.353292,\
        0.351406, 0.349386, 0.347230, 0.344912, 0.342392, 0.339629, 0.336589, 0.333253, 0.329629, 0.325751,\
        0.321684, 0.317548, 0.313465, 0.309536, 0.306143, 0.304043, 0.303209, 0.302941, 0.303155, 0.303945,\
        0.303708, 0.302888, 0.301323, 0.300329, 0.300668, 0.300479, 0.299545, 0.298136, 0.297027, 0.297802,\
        0.304192, 0.319166, 0.339773, 0.363235, 0.387554, 0.411179, 0.432157, 0.450282, 0.465045, 0.475709,\
        0.481912, 0.484169, 0.483635, 0.481349, 0.477975, 0.473903, 0.469346, 0.464391, 0.459035, 0.453224,\
        0.446897, 0.440027, 0.432659, 0.424898, 0.416788, 0.408316, 0.399772, 0.391548, 0.383708, 0.376110,\
        0.368659, 0.361409, 0.354362, 0.347477, 0.340716, 0.334079, 0.327608, 0.321376, 0.315511, 0.310155,\
        0.305200, 0.300452, 0.296388, 0.292787, 0.289119, 0.285211, 0.281333, 0.277523, 0.273735, 0.269965,\
        0.266264, 0.262681, 0.259243, 0.255945, 0.252769, 0.249691, 0.246685, 0.243729, 0.240795, 0.237857,\
        0.234878, 0.231820, 0.228636, 0.225277, 0.221692, 0.217833, 0.213652, 0.209110, 0.204175, 0.198821,\
        0.193032],\
        [0.325403, 0.330465, 0.335090, 0.339208, 0.342761, 0.345713, 0.348053, 0.349811, 0.351047, 0.351844,\
        0.352284, 0.352433, 0.352330, 0.351989, 0.351403, 0.350563, 0.349472, 0.348148, 0.346624, 0.344935,\
        0.343110, 0.341163, 0.339085, 0.336846, 0.334406, 0.331729, 0.328775, 0.325516, 0.321958, 0.318144,\
        0.314113, 0.310022, 0.306102, 0.302248, 0.298783, 0.296782, 0.296135, 0.296081, 0.296583, 0.297595,\
        0.297286, 0.296500, 0.295174, 0.294113, 0.293882, 0.293368, 0.292214, 0.290731, 0.289723, 0.290803,\
        0.296912, 0.311477, 0.332523, 0.356524, 0.381544, 0.405025, 0.424683, 0.442062, 0.456298, 0.466767,\
        0.473310, 0.476302, 0.476496, 0.474700, 0.471563, 0.467532, 0.462885, 0.457757, 0.452180, 0.446120,\
        0.439518, 0.432333, 0.424618, 0.416610, 0.408451, 0.399866, 0.391129, 0.382738, 0.374763, 0.367166,\
        0.359891, 0.352909, 0.346176, 0.339614, 0.333148, 0.326749, 0.320467, 0.314370, 0.308538, 0.303393,\
        0.298742, 0.294209, 0.290086, 0.286334, 0.282644, 0.278871, 0.275065, 0.271275, 0.267507, 0.263769,\
        0.260087, 0.256490, 0.252996, 0.249609, 0.246318, 0.243104, 0.239947, 0.236822, 0.233704, 0.230564,\
        0.227369, 0.224081, 0.220659, 0.217059, 0.213236, 0.209148, 0.204753, 0.200016, 0.194906, 0.189399,\
        0.183478],\
        [0.318732, 0.323837, 0.328514, 0.332683, 0.336275, 0.339231, 0.341523, 0.343172, 0.344246, 0.344841,\
        0.345058, 0.344976, 0.344643, 0.344080, 0.343290, 0.342278, 0.341064, 0.339680, 0.338155, 0.336512,\
        0.334765, 0.332921, 0.330963, 0.328851, 0.326538, 0.323998, 0.321193, 0.318063, 0.314605, 0.310934,\
        0.307035, 0.302886, 0.299250, 0.295682, 0.292544, 0.290533, 0.289667, 0.289556, 0.289956, 0.290412,\
        0.290246, 0.289535, 0.288501, 0.287544, 0.286870, 0.286136, 0.285172, 0.284293, 0.284333, 0.286772,\
        0.293791, 0.307809, 0.327786, 0.350745, 0.374372, 0.396311, 0.416352, 0.433712, 0.447632, 0.457906,\
        0.464568, 0.467937, 0.468576, 0.467134, 0.464208, 0.460269, 0.455632, 0.450469, 0.444827, 0.438677,\
        0.431956, 0.424582, 0.416493, 0.408042, 0.399833, 0.391201, 0.382368, 0.373830, 0.365756, 0.358180,\
        0.351081, 0.344403, 0.338055, 0.331906, 0.325817, 0.319690, 0.313616, 0.307813, 0.301999, 0.296820,\
        0.292334, 0.287981, 0.283863, 0.279984, 0.276212, 0.272446, 0.268666, 0.264889, 0.261128, 0.257395,\
        0.253706, 0.250080, 0.246528, 0.243054, 0.239650, 0.236303, 0.232994, 0.229701, 0.226399, 0.223060,\
        0.219652, 0.216141, 0.212490, 0.208660, 0.204609, 0.200301, 0.195698, 0.190769, 0.185485, 0.179823,\
        0.173767],\
        [0.311970, 0.317157, 0.321907, 0.326146, 0.329799, 0.332770, 0.334998, 0.336497, 0.337356, 0.337705,\
        0.337672, 0.337356, 0.336813, 0.336061, 0.335100, 0.333935, 0.332605, 0.331167, 0.329652, 0.328061,\
        0.326398, 0.324674, 0.322870, 0.320919, 0.318754, 0.316372, 0.313781, 0.310844, 0.307450, 0.303908,\
        0.300440, 0.296330, 0.292675, 0.289388, 0.286584, 0.284607, 0.283524, 0.283139, 0.283166, 0.283234,\
        0.282998, 0.282397, 0.281574, 0.280746, 0.280043, 0.279441, 0.279014, 0.279155, 0.280673, 0.284806,\
        0.293081, 0.306617, 0.324886, 0.346035, 0.368076, 0.389382, 0.409065, 0.425895, 0.439333, 0.449303,\
        0.455893, 0.459370, 0.460186, 0.458901, 0.456081, 0.452209, 0.447632, 0.442537, 0.436964, 0.430863,\
        0.424175, 0.416858, 0.408665, 0.399461, 0.390918, 0.382193, 0.373332, 0.364718, 0.356598, 0.349082,\
        0.342189, 0.335865, 0.329981, 0.324350, 0.318773, 0.313012, 0.306975, 0.301578, 0.296225, 0.290923,\
        0.286222, 0.281830, 0.277658, 0.273672, 0.269802, 0.265980, 0.262176, 0.258383, 0.254606, 0.250854,\
        0.247137, 0.243465, 0.239847, 0.236282, 0.232766, 0.229285, 0.225825, 0.222364, 0.218880, 0.215345,\
        0.211731, 0.208005, 0.204133, 0.200080, 0.195810, 0.191290, 0.186485, 0.181368, 0.175911, 0.170094,\
        0.163900],\
        [0.305203, 0.310503, 0.315295, 0.319618, 0.323376, 0.326387, 0.328535, 0.329833, 0.330412, 0.330461,\
        0.330153, 0.329607, 0.328882, 0.327986, 0.326894, 0.325589, 0.324134, 0.322647, 0.321162, 0.319629,\
        0.318037, 0.316432, 0.314806, 0.313055, 0.311021, 0.308752, 0.306410, 0.303807, 0.300455, 0.296926,\
        0.293542, 0.289905, 0.286393, 0.283276, 0.280667, 0.278716, 0.277459, 0.276776, 0.276450, 0.276217,\
        0.275866, 0.275331, 0.274680, 0.274051, 0.273575, 0.273381, 0.273695, 0.274972, 0.277939, 0.283533,\
        0.292644, 0.305770, 0.322600, 0.342020, 0.362722, 0.383367, 0.402497, 0.418565, 0.431392, 0.440998,\
        0.447405, 0.450796, 0.451555, 0.450213, 0.447342, 0.443458, 0.438941, 0.433989, 0.428605, 0.422650,\
        0.416028, 0.408843, 0.401145, 0.391828, 0.382412, 0.373217, 0.364155, 0.355431, 0.347277, 0.339841,\
        0.333179, 0.327251, 0.321899, 0.316862, 0.311920, 0.306918, 0.301050, 0.295362, 0.290235, 0.285092,\
        0.280256, 0.275734, 0.271450, 0.267348, 0.263370, 0.259465, 0.255601, 0.251763, 0.247946, 0.244152,\
        0.240387, 0.236655, 0.232959, 0.229297, 0.225665, 0.222051, 0.218440, 0.214813, 0.211149, 0.207422,\
        0.203606, 0.199671, 0.195586, 0.191320, 0.186839, 0.182113, 0.177112, 0.171810, 0.166183, 0.160210,\
        0.153875],\
        [0.298615, 0.303892, 0.308644, 0.313105, 0.317052, 0.320141, 0.322181, 0.323208, 0.323423, 0.323112,\
        0.322512, 0.321753, 0.320884, 0.319905, 0.318752, 0.317322, 0.315693, 0.314157, 0.312757, 0.311289,\
        0.309725, 0.308212, 0.306760, 0.305259, 0.303353, 0.301068, 0.298898, 0.296554, 0.293439, 0.290067,\
        0.286719, 0.283359, 0.280119, 0.277194, 0.274719, 0.272779, 0.271390, 0.270471, 0.269871, 0.269416,\
        0.268972, 0.268487, 0.268002, 0.267627, 0.267506, 0.267838, 0.268920, 0.271209, 0.275333, 0.282004,\
        0.291775, 0.304554, 0.319910, 0.337785, 0.357240, 0.376982, 0.395522, 0.411139, 0.423570, 0.432967,\
        0.439200, 0.442374, 0.442870, 0.441249, 0.438142, 0.434126, 0.429629, 0.424863, 0.419788, 0.414097,\
        0.407457, 0.400143, 0.392256, 0.383321, 0.373783, 0.364250, 0.354935, 0.346056, 0.337851, 0.330490,\
        0.324053, 0.318529, 0.313750, 0.309316, 0.304844, 0.300493, 0.295623, 0.289711, 0.284263, 0.279102,\
        0.274205, 0.269577, 0.265179, 0.260960, 0.256872, 0.252872, 0.248930, 0.245024, 0.241146, 0.237291,\
        0.233460, 0.229653, 0.225868, 0.222104, 0.218352, 0.214602, 0.210840, 0.207049, 0.203207, 0.199293,\
        0.195280, 0.191143, 0.186852, 0.182378, 0.177693, 0.172768, 0.167576, 0.162093, 0.156297, 0.150168,\
        0.143691],\
        [0.291893, 0.297118, 0.301917, 0.306636, 0.310884, 0.314067, 0.315950, 0.316620, 0.316367, 0.315641,\
        0.314760, 0.313824, 0.312849, 0.311847, 0.310760, 0.309268, 0.307337, 0.305745, 0.304530, 0.303139,\
        0.301536, 0.300089, 0.298745, 0.297396, 0.295657, 0.293491, 0.291323, 0.288957, 0.286158, 0.283069,\
        0.279910, 0.276781, 0.273786, 0.271057, 0.268701, 0.266782, 0.265309, 0.264229, 0.263445, 0.262848,\
        0.262352, 0.261930, 0.261620, 0.261522, 0.261790, 0.262642, 0.264384, 0.267428, 0.272279, 0.279457,\
        0.289308, 0.301407, 0.315784, 0.332598, 0.351048, 0.369924, 0.387782, 0.403176, 0.415796, 0.425298,\
        0.431417, 0.434269, 0.434301, 0.432179, 0.428640, 0.424352, 0.419803, 0.415212, 0.410519, 0.405326,\
        0.398725, 0.391241, 0.383174, 0.374331, 0.364896, 0.355260, 0.345757, 0.336713, 0.328452, 0.321166,\
        0.314911, 0.309731, 0.305479, 0.301705, 0.297466, 0.293225, 0.288609, 0.283336, 0.278004, 0.272865,\
        0.267951, 0.263262, 0.258776, 0.254458, 0.250268, 0.246171, 0.242140, 0.238153, 0.234198, 0.230267,\
        0.226356, 0.222461, 0.218579, 0.214704, 0.210828, 0.206940, 0.203027, 0.199072, 0.195056, 0.190958,\
        0.186754, 0.182419, 0.177928, 0.173254, 0.168371, 0.163252, 0.157875, 0.152214, 0.146251, 0.139967,\
        0.133347],\
        [0.284999, 0.290362, 0.295416, 0.300295, 0.304943, 0.308058, 0.309748, 0.310016, 0.309189, 0.308024,\
        0.306937, 0.305887, 0.304833, 0.303802, 0.302896, 0.301554, 0.299307, 0.297619, 0.296476, 0.295139,\
        0.293620, 0.292196, 0.290862, 0.289466, 0.287805, 0.285844, 0.283716, 0.281388, 0.278786, 0.275963,\
        0.273048, 0.270155, 0.267387, 0.264843, 0.262608, 0.260732, 0.259225, 0.258058, 0.257172, 0.256503,\
        0.256004, 0.255670, 0.255543, 0.255721, 0.256354, 0.257653, 0.259894, 0.263414, 0.268585, 0.275733,\
        0.285002, 0.296410, 0.310230, 0.326343, 0.344127, 0.362433, 0.379941, 0.395481, 0.408521, 0.418244,\
        0.424237, 0.426630, 0.425994, 0.423171, 0.419036, 0.414364, 0.409709, 0.405230, 0.400784, 0.395885,\
        0.389629, 0.382232, 0.374137, 0.365377, 0.356065, 0.346443, 0.336817, 0.327598, 0.319293, 0.312154,\
        0.306051, 0.301126, 0.297187, 0.293667, 0.289807, 0.285666, 0.281200, 0.276341, 0.271304, 0.266303,\
        0.261437, 0.256735, 0.252195, 0.247799, 0.243520, 0.239332, 0.235210, 0.231135, 0.227093, 0.223075,\
        0.219072, 0.215080, 0.211091, 0.207099, 0.203094, 0.199066, 0.195001, 0.190884, 0.186696, 0.182417,\
        0.178026, 0.173500, 0.168815, 0.163947, 0.158871, 0.153565, 0.148005, 0.142171, 0.136043, 0.129605,\
        0.122841],\
        [0.277926, 0.283522, 0.288885, 0.294100, 0.298963, 0.301468, 0.303337, 0.303232, 0.301845, 0.300322,\
        0.299175, 0.298058, 0.296961, 0.295901, 0.294958, 0.293659, 0.291716, 0.289983, 0.288611, 0.287257,\
        0.285836, 0.284433, 0.283054, 0.281606, 0.279977, 0.278128, 0.276084, 0.273847, 0.271411, 0.268813,\
        0.266136, 0.263476, 0.260923, 0.258559, 0.256451, 0.254643, 0.253149, 0.251958, 0.251040, 0.250363,\
        0.249907, 0.249685, 0.249746, 0.250184, 0.251138, 0.252795, 0.255385, 0.259170, 0.264417, 0.271345,\
        0.280100, 0.290819, 0.303764, 0.319404, 0.336757, 0.354845, 0.372484, 0.388541, 0.402034, 0.411919,\
        0.417749, 0.419513, 0.418017, 0.414374, 0.409625, 0.404550, 0.399879, 0.395500, 0.391093, 0.386135,\
        0.380134, 0.373074, 0.365197, 0.356653, 0.347565, 0.338095, 0.328452, 0.319015, 0.310623, 0.303858,\
        0.298080, 0.293284, 0.289378, 0.285767, 0.282025, 0.278005, 0.273684, 0.269072, 0.264279, 0.259443,\
        0.254658, 0.249976, 0.245410, 0.240958, 0.236605, 0.232331, 0.228120, 0.223954, 0.219819, 0.215705,\
        0.211603, 0.207505, 0.203403, 0.199288, 0.195152, 0.190982, 0.186765, 0.182486, 0.178128, 0.173672,\
        0.169098, 0.164385, 0.159511, 0.154454, 0.149191, 0.143702, 0.137964, 0.131959, 0.125668, 0.119077,\
        0.112170],\
        [0.270479, 0.276231, 0.281723, 0.286872, 0.291177, 0.294045, 0.295904, 0.295749, 0.294375, 0.292856,\
        0.291555, 0.290366, 0.289223, 0.288119, 0.287005, 0.285660, 0.284021, 0.282388, 0.280911, 0.279505,\
        0.278104, 0.276704, 0.275291, 0.273810, 0.272196, 0.270412, 0.268455, 0.266330, 0.264050, 0.261650,\
        0.259193, 0.256752, 0.254403, 0.252214, 0.250243, 0.248529, 0.247090, 0.245928, 0.245036, 0.244402,\
        0.244029, 0.243941, 0.244192, 0.244870, 0.246100, 0.248040, 0.250877, 0.254811, 0.260039, 0.266733,\
        0.275049, 0.285181, 0.297568, 0.312381, 0.329166, 0.347240, 0.365185, 0.381761, 0.395752, 0.406139,\
        0.411831, 0.412718, 0.410278, 0.405857, 0.400792, 0.395642, 0.390931, 0.386448, 0.381836, 0.376722,\
        0.370806, 0.364014, 0.356451, 0.348250, 0.339541, 0.330456, 0.321112, 0.311613, 0.302752, 0.296528,\
        0.291110, 0.286335, 0.282173, 0.278314, 0.274455, 0.270427, 0.266172, 0.261696, 0.257060, 0.252353,\
        0.247647, 0.242994, 0.238417, 0.233922, 0.229504, 0.225153, 0.220855, 0.216597, 0.212366, 0.208151,\
        0.203943, 0.199734, 0.195513, 0.191272, 0.187001, 0.182687, 0.178318, 0.173878, 0.169352, 0.164722,\
        0.159969, 0.155074, 0.150016, 0.144775, 0.139330, 0.133661, 0.127749, 0.121576, 0.115125, 0.108381,\
        0.101331],\
        [0.262501, 0.268274, 0.273686, 0.278571, 0.282596, 0.285487, 0.287126, 0.287304, 0.286410, 0.285134,\
        0.283848, 0.282624, 0.281442, 0.280275, 0.279064, 0.277717, 0.276224, 0.274694, 0.273217, 0.271790,\
        0.270384, 0.268976, 0.267544, 0.266049, 0.264450, 0.262718, 0.260845, 0.258835, 0.256703, 0.254485,\
        0.252227, 0.249991, 0.247836, 0.245822, 0.243997, 0.242398, 0.241048, 0.239960, 0.239138, 0.238590,\
        0.238331, 0.238395, 0.238838, 0.239741, 0.241214, 0.243388, 0.246415, 0.250452, 0.255653, 0.262165,\
        0.270140, 0.279784, 0.291355, 0.304792, 0.321525, 0.339674, 0.357768, 0.374716, 0.389501, 0.401037,\
        0.405626, 0.405485, 0.402469, 0.397711, 0.392620, 0.387552, 0.382662, 0.377895, 0.373001, 0.367705,\
        0.361797, 0.355199, 0.347952, 0.340160, 0.331954, 0.323482, 0.314912, 0.306423, 0.297960, 0.290867,\
        0.285094, 0.280016, 0.275479, 0.271272, 0.267170, 0.263015, 0.258727, 0.254290, 0.249729, 0.245096,\
        0.240443, 0.235809, 0.231220, 0.226688, 0.222211, 0.217786, 0.213404, 0.209053, 0.204723, 0.200404,\
        0.196087, 0.191762, 0.187419, 0.183048, 0.178639, 0.174181, 0.169659, 0.165060, 0.160368, 0.155567,\
        0.150638, 0.145565, 0.140327, 0.134906, 0.129283, 0.123439, 0.117356, 0.111018, 0.104409, 0.097514,\
        0.090321]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=25 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax25vt7.5
    zgrid_H_Hmax_vmax25vt7_5=[[0.364289, 0.370473, 0.375862, 0.380401, 0.384087, 0.387002, 0.389338, 0.391359, 0.393312, 0.395349,\
        0.397508, 0.399741, 0.401955, 0.404038, 0.405879, 0.407378, 0.408450, 0.409033, 0.409085, 0.408591,\
        0.407559, 0.406018, 0.404006, 0.401533, 0.398512, 0.394713, 0.390244, 0.385662, 0.381343, 0.377429,\
        0.373930, 0.370801, 0.367983, 0.365426, 0.363102, 0.360997, 0.359112, 0.357460, 0.356057, 0.354928,\
        0.354096, 0.353586, 0.353424, 0.353634, 0.354235, 0.355247, 0.356683, 0.358550, 0.360850, 0.363576,\
        0.366713, 0.370235, 0.374107, 0.378288, 0.382733, 0.387408, 0.392306, 0.397372, 0.402301, 0.407110,\
        0.411841, 0.416473, 0.420984, 0.425361, 0.429600, 0.433709, 0.437710, 0.441630, 0.445500, 0.449343,\
        0.453167, 0.456957, 0.460679, 0.464296, 0.467789, 0.471171, 0.474457, 0.477576, 0.480345, 0.482583,\
        0.484176, 0.485071, 0.485260, 0.484798, 0.484034, 0.483458, 0.483147, 0.482920, 0.482605, 0.482094,\
        0.481340, 0.480244, 0.478704, 0.476798, 0.474646, 0.472295, 0.469755, 0.467001, 0.463978, 0.460632,\
        0.456938, 0.452895, 0.448507, 0.443779, 0.438716, 0.433321, 0.427599, 0.421555, 0.415192, 0.408513,\
        0.401518, 0.394208, 0.386584, 0.378644, 0.370391, 0.361825, 0.352948, 0.343764, 0.334275, 0.324486,\
        0.314402],\
        [0.369241, 0.375479, 0.380878, 0.385343, 0.388831, 0.391417, 0.393357, 0.395053, 0.396864, 0.398957,\
        0.401324, 0.403855, 0.406398, 0.408800, 0.410919, 0.412634, 0.413844, 0.414470, 0.414464, 0.413805,\
        0.412502, 0.410601, 0.408170, 0.405281, 0.401934, 0.397711, 0.392652, 0.387748, 0.383340, 0.379488,\
        0.376118, 0.373122, 0.370406, 0.367909, 0.365599, 0.363468, 0.361526, 0.359794, 0.358297, 0.357066,\
        0.356131, 0.355523, 0.355272, 0.355407, 0.355954, 0.356936, 0.358370, 0.360267, 0.362631, 0.365459,\
        0.368733, 0.372428, 0.376503, 0.380905, 0.385574, 0.390447, 0.395471, 0.400832, 0.406152, 0.411326,\
        0.416380, 0.421294, 0.426046, 0.430621, 0.435017, 0.439248, 0.443344, 0.447349, 0.451312, 0.455275,\
        0.459259, 0.463246, 0.467181, 0.470995, 0.474645, 0.478171, 0.481688, 0.485177, 0.488312, 0.490811,\
        0.492584, 0.493554, 0.493749, 0.493027, 0.491817, 0.491236, 0.491092, 0.490999, 0.490770, 0.490262,\
        0.489561, 0.488530, 0.486840, 0.484770, 0.482541, 0.480153, 0.477617, 0.474916, 0.471943, 0.468608,\
        0.464894, 0.460821, 0.456403, 0.451644, 0.446549, 0.441122, 0.435368, 0.429295, 0.422905, 0.416201,\
        0.409184, 0.401852, 0.394205, 0.386242, 0.377964, 0.369370, 0.360465, 0.351250, 0.341730, 0.331910,\
        0.321794],\
        [0.373619, 0.379919, 0.385349, 0.389760, 0.393040, 0.395229, 0.396659, 0.397986, 0.399736, 0.402045,\
        0.404793, 0.407772, 0.410767, 0.413585, 0.416057, 0.418044, 0.419428, 0.420118, 0.420051, 0.419192,\
        0.417545, 0.415151, 0.412090, 0.408464, 0.404314, 0.399465, 0.394192, 0.389254, 0.385018, 0.381447,\
        0.378361, 0.375592, 0.373026, 0.370602, 0.368299, 0.366124, 0.364101, 0.362261, 0.360642, 0.359281,\
        0.358216, 0.357483, 0.357117, 0.357154, 0.357624, 0.358555, 0.359969, 0.361882, 0.364302, 0.367225,\
        0.370639, 0.374517, 0.378814, 0.383466, 0.388397, 0.393545, 0.398837, 0.404258, 0.409972, 0.415554,\
        0.420963, 0.426181, 0.431188, 0.435967, 0.440514, 0.444849, 0.449011, 0.453063, 0.457080, 0.461136,\
        0.465275, 0.469484, 0.473679, 0.477734, 0.481539, 0.485108, 0.488736, 0.492686, 0.496251, 0.498976,\
        0.500935, 0.501979, 0.502214, 0.501352, 0.499702, 0.499362, 0.499346, 0.499284, 0.499078, 0.498432,\
        0.497695, 0.496838, 0.494789, 0.492534, 0.490284, 0.487827, 0.485281, 0.482680, 0.479778, 0.476422,\
        0.472653, 0.468534, 0.464078, 0.459283, 0.454149, 0.448683, 0.442892, 0.436785, 0.430367, 0.423638,\
        0.416597, 0.409244, 0.401574, 0.393587, 0.385281, 0.376658, 0.367721, 0.358473, 0.348918, 0.339062,\
        0.328911],\
        [0.377341, 0.383683, 0.389145, 0.393504, 0.396542, 0.398266, 0.399012, 0.399989, 0.401902, 0.404691,\
        0.408041, 0.411635, 0.415202, 0.418523, 0.421415, 0.423726, 0.425326, 0.426112, 0.426001, 0.424940,\
        0.422912, 0.419942, 0.416102, 0.411510, 0.406326, 0.400806, 0.395271, 0.390482, 0.386702, 0.383625,\
        0.380939, 0.378441, 0.376025, 0.373650, 0.371318, 0.369057, 0.366909, 0.364923, 0.363145, 0.361620,\
        0.360392, 0.359503, 0.358993, 0.358903, 0.359270, 0.360127, 0.361502, 0.363414, 0.365876, 0.368889,\
        0.372441, 0.376509, 0.381052, 0.385996, 0.391231, 0.396683, 0.402349, 0.408127, 0.414040, 0.419934,\
        0.425676, 0.431202, 0.436470, 0.441454, 0.446147, 0.450564, 0.454754, 0.458800, 0.462811, 0.466910,\
        0.471185, 0.475645, 0.480176, 0.484571, 0.488623, 0.492187, 0.495601, 0.499973, 0.504035, 0.507031,\
        0.509154, 0.510340, 0.510586, 0.509874, 0.508760, 0.508231, 0.508049, 0.507843, 0.507450, 0.506713,\
        0.505851, 0.504679, 0.502488, 0.500197, 0.497966, 0.495314, 0.492725, 0.490339, 0.487507, 0.484040,\
        0.480169, 0.476003, 0.471515, 0.466680, 0.461502, 0.455990, 0.450157, 0.444014, 0.437566, 0.430814,\
        0.423752, 0.416378, 0.408686, 0.400673, 0.392339, 0.383684, 0.374712, 0.365427, 0.355833, 0.345938,\
        0.335748],\
        [0.380338, 0.386651, 0.392090, 0.396373, 0.398983, 0.400027, 0.399999, 0.400981, 0.403513, 0.407135,\
        0.411311, 0.415652, 0.419875, 0.423757, 0.427116, 0.429795, 0.431657, 0.432584, 0.432476, 0.431257,\
        0.428882, 0.425354, 0.420727, 0.415132, 0.408826, 0.402320, 0.396359, 0.392036, 0.388984, 0.386500,\
        0.384208, 0.381930, 0.379595, 0.377195, 0.374761, 0.372348, 0.370018, 0.367835, 0.365854, 0.364126,\
        0.362698, 0.361618, 0.360931, 0.360684, 0.360918, 0.361675, 0.362988, 0.364882, 0.367373, 0.370465,\
        0.374150, 0.378407, 0.383205, 0.388490, 0.394119, 0.399859, 0.405884, 0.412051, 0.418291, 0.424504,\
        0.430573, 0.436408, 0.441945, 0.447139, 0.451972, 0.456452, 0.460631, 0.464607, 0.468528, 0.472581,\
        0.476939, 0.481664, 0.486619, 0.491473, 0.495950, 0.499814, 0.503221, 0.507521, 0.511801, 0.515148,\
        0.517500, 0.518865, 0.519285, 0.518923, 0.518203, 0.517590, 0.517128, 0.516642, 0.515983, 0.515073,\
        0.513915, 0.512357, 0.510253, 0.507928, 0.505484, 0.502679, 0.500101, 0.497960, 0.495085, 0.491334,\
        0.487373, 0.483210, 0.478701, 0.473822, 0.468590, 0.463026, 0.457148, 0.450968, 0.444493, 0.437721,\
        0.430643, 0.423251, 0.415537, 0.407498, 0.399133, 0.390443, 0.381432, 0.372106, 0.362470, 0.352532,\
        0.342299],\
        [0.382606, 0.388763, 0.394011, 0.398087, 0.400168, 0.399958, 0.399848, 0.401390, 0.405138, 0.409851,\
        0.414949, 0.420074, 0.424967, 0.429423, 0.433266, 0.436341, 0.438509, 0.439638, 0.439611, 0.438327,\
        0.435716, 0.431759, 0.426506, 0.420094, 0.412788, 0.405070, 0.398598, 0.395134, 0.392701, 0.390613,\
        0.388516, 0.386286, 0.383892, 0.381351, 0.378712, 0.376059, 0.373477, 0.371041, 0.368810, 0.366837,\
        0.365170, 0.363862, 0.362962, 0.362522, 0.362593, 0.363221, 0.364448, 0.366305, 0.368813, 0.371977,\
        0.375785, 0.380218, 0.385248, 0.390851, 0.396995, 0.403220, 0.409522, 0.416062, 0.422682, 0.429258,\
        0.435675, 0.441834, 0.447652, 0.453068, 0.458045, 0.462579, 0.466715, 0.470558, 0.474283, 0.478156,\
        0.482484, 0.487431, 0.492882, 0.498281, 0.503185, 0.507591, 0.511682, 0.515973, 0.520156, 0.523642,\
        0.526179, 0.527730, 0.528366, 0.528275, 0.527759, 0.527097, 0.526391, 0.525596, 0.524633, 0.523440,\
        0.521973, 0.520162, 0.517993, 0.515576, 0.512976, 0.510239, 0.507664, 0.505222, 0.502084, 0.498136,\
        0.494275, 0.490181, 0.485633, 0.480690, 0.475396, 0.469773, 0.463845, 0.457631, 0.451136, 0.444352,\
        0.437265, 0.429859, 0.422125, 0.414059, 0.405660, 0.396931, 0.387877, 0.378504, 0.368821, 0.358836,\
        0.348556],\
        [0.384264, 0.390156, 0.395012, 0.398489, 0.399966, 0.399659, 0.399948, 0.402913, 0.407777, 0.413425,\
        0.419318, 0.425132, 0.430631, 0.435620, 0.439930, 0.443411, 0.445919, 0.447319, 0.447474, 0.446260,\
        0.443581, 0.439409, 0.433810, 0.426979, 0.419297, 0.411515, 0.405161, 0.401127, 0.398507, 0.396301,\
        0.394046, 0.391623, 0.388996, 0.386174, 0.383213, 0.380223, 0.377318, 0.374575, 0.372048, 0.369787,\
        0.367841, 0.366265, 0.365115, 0.364447, 0.364319, 0.364787, 0.365902, 0.367704, 0.370219, 0.373452,\
        0.377388, 0.381992, 0.387223, 0.393033, 0.399374, 0.406204, 0.413131, 0.420158, 0.427217, 0.434197,\
        0.440990, 0.447492, 0.453612, 0.459270, 0.464409, 0.469004, 0.473090, 0.476765, 0.480201, 0.483712,\
        0.487828, 0.492916, 0.498691, 0.504738, 0.510252, 0.515366, 0.520169, 0.524777, 0.529032, 0.532604,\
        0.535267, 0.536957, 0.537744, 0.537801, 0.537355, 0.536610, 0.535681, 0.534588, 0.533305, 0.531799,\
        0.530037, 0.527995, 0.525680, 0.523146, 0.520455, 0.517683, 0.514918, 0.512034, 0.508673, 0.504861,\
        0.501052, 0.496928, 0.492264, 0.487251, 0.481896, 0.476208, 0.470229, 0.463988, 0.457487, 0.450706,\
        0.443618, 0.436203, 0.428449, 0.420353, 0.411916, 0.403142, 0.394040, 0.384616, 0.374881, 0.364843,\
        0.354513],\
        [0.385551, 0.391210, 0.395758, 0.398932, 0.400556, 0.401128, 0.402741, 0.406626, 0.412040, 0.418216,\
        0.424635, 0.430957, 0.436937, 0.442378, 0.447110, 0.450981, 0.453853, 0.455588, 0.456039, 0.455050,\
        0.452495, 0.448338, 0.442678, 0.435794, 0.428200, 0.420714, 0.414347, 0.409624, 0.406213, 0.403405,\
        0.400707, 0.397881, 0.394880, 0.391660, 0.388254, 0.384838, 0.381553, 0.378460, 0.375598, 0.373007,\
        0.370741, 0.368858, 0.367417, 0.366482, 0.366118, 0.366393, 0.367368, 0.369097, 0.371614, 0.374926,\
        0.379011, 0.383820, 0.389290, 0.395373, 0.402076, 0.409347, 0.416845, 0.424401, 0.431928, 0.439334,\
        0.446518, 0.453380, 0.459821, 0.465747, 0.471077, 0.475763, 0.479815, 0.483354, 0.486511, 0.489496,\
        0.493070, 0.498419, 0.504547, 0.511049, 0.517337, 0.523217, 0.528696, 0.533751, 0.538243, 0.541955,\
        0.544717, 0.546481, 0.547322, 0.547405, 0.546923, 0.546050, 0.544897, 0.543515, 0.541916, 0.540096,\
        0.538048, 0.535767, 0.533270, 0.530591, 0.527775, 0.524865, 0.521868, 0.518692, 0.515211, 0.511453,\
        0.507520, 0.503244, 0.498518, 0.493485, 0.488072, 0.482308, 0.476276, 0.470025, 0.463544, 0.456785,\
        0.449707, 0.442286, 0.434510, 0.426378, 0.417896, 0.409071, 0.399913, 0.390432, 0.380639, 0.370545,\
        0.360160],\
        [0.386737, 0.392340, 0.396904, 0.400334, 0.402750, 0.404798, 0.407667, 0.412063, 0.417743, 0.424170,\
        0.430881, 0.437528, 0.443853, 0.449648, 0.454735, 0.458959, 0.462189, 0.464306, 0.465162, 0.464555,\
        0.462306, 0.458354, 0.452824, 0.446054, 0.438618, 0.431272, 0.424744, 0.419414, 0.415160, 0.411566,\
        0.408227, 0.404883, 0.401421, 0.397732, 0.393771, 0.389882, 0.386191, 0.382722, 0.379486, 0.376522,\
        0.373893, 0.371664, 0.369895, 0.368653, 0.368013, 0.368056, 0.368861, 0.370496, 0.373010, 0.376420,\
        0.380702, 0.385787, 0.391586, 0.398056, 0.405216, 0.412903, 0.420858, 0.428888, 0.436863, 0.444682,\
        0.452251, 0.459472, 0.466243, 0.472455, 0.478012, 0.482835, 0.486869, 0.490338, 0.493478, 0.496047,\
        0.499019, 0.504091, 0.510911, 0.517885, 0.524734, 0.531251, 0.537330, 0.542862, 0.547678, 0.551581,\
        0.554428, 0.556195, 0.556988, 0.556988, 0.556388, 0.555342, 0.553956, 0.552292, 0.550387, 0.548262,\
        0.545931, 0.543406, 0.540704, 0.537851, 0.534871, 0.531781, 0.528568, 0.525180, 0.521564, 0.517712,\
        0.513618, 0.509211, 0.504460, 0.499392, 0.493899, 0.488036, 0.481959, 0.475740, 0.469319, 0.462607,\
        0.455546, 0.448114, 0.440307, 0.432131, 0.423595, 0.414710, 0.405489, 0.395944, 0.386088, 0.375932,\
        0.365488],\
        [0.388011, 0.393779, 0.398671, 0.402719, 0.406157, 0.409511, 0.413463, 0.418436, 0.424381, 0.430975,\
        0.437851, 0.444691, 0.451242, 0.457295, 0.462665, 0.467181, 0.470719, 0.473221, 0.474575, 0.474502,\
        0.472705, 0.469085, 0.463774, 0.457124, 0.449746, 0.442376, 0.435619, 0.429778, 0.424816, 0.420475,\
        0.416448, 0.412495, 0.408455, 0.404194, 0.399713, 0.395374, 0.391265, 0.387397, 0.383740, 0.380349,\
        0.377316, 0.374706, 0.372573, 0.370984, 0.370025, 0.369794, 0.370392, 0.371907, 0.374410, 0.377935,\
        0.382468, 0.387929, 0.394166, 0.401101, 0.408730, 0.416846, 0.425226, 0.433678, 0.442056, 0.450249,\
        0.458166, 0.465720, 0.472809, 0.479304, 0.485085, 0.490117, 0.494176, 0.497389, 0.500905, 0.503596,\
        0.506836, 0.511794, 0.518295, 0.525389, 0.532551, 0.539510, 0.546076, 0.552063, 0.557242, 0.561370,\
        0.564278, 0.565969, 0.566608, 0.566432, 0.565652, 0.564408, 0.562787, 0.560849, 0.558647, 0.556224,\
        0.553615, 0.550844, 0.547927, 0.544880, 0.541715, 0.538433, 0.535020, 0.531443, 0.527666, 0.523668,\
        0.519420, 0.514885, 0.510043, 0.504873, 0.499322, 0.493337, 0.487264, 0.481162, 0.474856, 0.468203,\
        0.461150, 0.453692, 0.445838, 0.437604, 0.429002, 0.420048, 0.410756, 0.401140, 0.391214, 0.380992,\
        0.370485],\
        [0.389443, 0.395560, 0.400981, 0.405801, 0.410255, 0.414723, 0.419619, 0.425205, 0.431484, 0.438260,\
        0.445260, 0.452216, 0.458899, 0.465125, 0.470710, 0.475445, 0.479173, 0.481965, 0.483880, 0.484487,\
        0.483234, 0.480013, 0.474970, 0.468361, 0.460906, 0.453428, 0.446471, 0.440266, 0.434785, 0.429840,\
        0.425192, 0.420630, 0.415993, 0.411190, 0.406266, 0.401423, 0.396834, 0.392538, 0.388368, 0.384483,\
        0.381017, 0.378003, 0.375474, 0.373496, 0.372173, 0.371622, 0.371971, 0.373333, 0.375805, 0.379445,\
        0.384262, 0.390194, 0.397041, 0.404498, 0.412587, 0.421148, 0.429954, 0.438801, 0.447531, 0.456035,\
        0.464233, 0.472056, 0.479416, 0.486174, 0.492089, 0.497206, 0.501572, 0.504667, 0.508071, 0.511413,\
        0.515225, 0.520232, 0.526452, 0.533418, 0.540671, 0.547890, 0.554825, 0.561229, 0.566799, 0.571185,\
        0.574125, 0.575640, 0.576013, 0.575588, 0.574606, 0.573173, 0.571328, 0.569122, 0.566629, 0.563917,\
        0.561040, 0.558027, 0.554893, 0.551647, 0.548289, 0.544816, 0.541213, 0.537453, 0.533508, 0.529350,\
        0.524949, 0.520272, 0.515289, 0.509964, 0.504249, 0.498193, 0.492272, 0.486399, 0.480234, 0.473614,\
        0.466530, 0.459015, 0.451094, 0.442785, 0.434105, 0.425071, 0.415699, 0.406006, 0.396006, 0.385713,\
        0.375142],\
        [0.391007, 0.397595, 0.403646, 0.409272, 0.414671, 0.420107, 0.425843, 0.432038, 0.438701, 0.445701,\
        0.452827, 0.459852, 0.466585, 0.472888, 0.478636, 0.483564, 0.487290, 0.490060, 0.492555, 0.493972,\
        0.493252, 0.490430, 0.485812, 0.479096, 0.471463, 0.463938, 0.456937, 0.450578, 0.444801, 0.439441,\
        0.434304, 0.429216, 0.424054, 0.418765, 0.413395, 0.408077, 0.402991, 0.398137, 0.393300, 0.388890,\
        0.385001, 0.381577, 0.378621, 0.376211, 0.374474, 0.373557, 0.373613, 0.374785, 0.377196, 0.380931,\
        0.386015, 0.392399, 0.399899, 0.408020, 0.416696, 0.425790, 0.435058, 0.444288, 0.453321, 0.462051,\
        0.470426, 0.478408, 0.485928, 0.492897, 0.498981, 0.503878, 0.508236, 0.511884, 0.515436, 0.519196,\
        0.523479, 0.528646, 0.534760, 0.541596, 0.548832, 0.556174, 0.563360, 0.570123, 0.576119, 0.580839,\
        0.583785, 0.584998, 0.584998, 0.584288, 0.583140, 0.581572, 0.579527, 0.577058, 0.574278, 0.571287,\
        0.568155, 0.564917, 0.561576, 0.558132, 0.554583, 0.550923, 0.547137, 0.543202, 0.539092, 0.534779,\
        0.530232, 0.525420, 0.520311, 0.514869, 0.509079, 0.503075, 0.497288, 0.491643, 0.485537, 0.478846,\
        0.471667, 0.464061, 0.456052, 0.447655, 0.438886, 0.429763, 0.420304, 0.410527, 0.400448, 0.390082,\
        0.379443],\
        [0.392619, 0.399743, 0.406459, 0.412873, 0.419136, 0.425430, 0.431921, 0.438705, 0.445779, 0.453040,\
        0.460312, 0.467385, 0.474077, 0.480308, 0.486111, 0.491341, 0.495021, 0.497099, 0.500001, 0.502073,\
        0.501786, 0.499456, 0.495632, 0.488443, 0.480826, 0.473569, 0.466805, 0.460552, 0.454719, 0.449143,\
        0.443664, 0.438158, 0.432547, 0.426807, 0.420979, 0.415163, 0.409470, 0.403882, 0.398422, 0.393567,\
        0.389302, 0.385454, 0.382029, 0.379135, 0.376938, 0.375615, 0.375342, 0.376291, 0.378617, 0.382430,\
        0.387767, 0.394549, 0.402612, 0.411600, 0.421080, 0.430821, 0.440596, 0.450202, 0.459482, 0.468338,\
        0.476754, 0.484756, 0.492263, 0.499133, 0.505277, 0.510325, 0.514732, 0.518732, 0.522637, 0.526768,\
        0.531395, 0.536731, 0.542832, 0.549578, 0.556751, 0.564109, 0.571408, 0.578403, 0.584810, 0.590023,\
        0.592968, 0.593746, 0.593338, 0.592376, 0.591177, 0.589584, 0.587345, 0.584608, 0.581546, 0.578287,\
        0.574928, 0.571492, 0.567961, 0.564328, 0.560592, 0.556751, 0.552791, 0.548690, 0.544424, 0.539966,\
        0.535291, 0.530373, 0.525190, 0.519733, 0.514038, 0.508247, 0.502591, 0.497014, 0.490706, 0.483808,\
        0.476493, 0.468786, 0.460683, 0.452189, 0.443322, 0.434103, 0.424551, 0.414686, 0.404525, 0.394083,\
        0.383376],\
        [0.394168, 0.401853, 0.409231, 0.416391, 0.423443, 0.430508, 0.437686, 0.445028, 0.452518, 0.460068,\
        0.467522, 0.474660, 0.481251, 0.487196, 0.492710, 0.498081, 0.502128, 0.504050, 0.506531, 0.508429,\
        0.508416, 0.506562, 0.502347, 0.495534, 0.488852, 0.482319, 0.476071, 0.470170, 0.464500, 0.458879,\
        0.453184, 0.447353, 0.441358, 0.435193, 0.428892, 0.422531, 0.416197, 0.409957, 0.403979, 0.398675,\
        0.393967, 0.389634, 0.385674, 0.382248, 0.379561, 0.377812, 0.377196, 0.377907, 0.380135, 0.384040,\
        0.389695, 0.397022, 0.405827, 0.415653, 0.425958, 0.436383, 0.446673, 0.456637, 0.466114, 0.474991,\
        0.483267, 0.491127, 0.498583, 0.505246, 0.511219, 0.516419, 0.521023, 0.525304, 0.529535, 0.533985,\
        0.538875, 0.544357, 0.550471, 0.557144, 0.564219, 0.571500, 0.578759, 0.585731, 0.592234, 0.598006,\
        0.601013, 0.601517, 0.600941, 0.599863, 0.598734, 0.597208, 0.594707, 0.591730, 0.588402, 0.584888,\
        0.581350, 0.577760, 0.574056, 0.570234, 0.566313, 0.562298, 0.558174, 0.553917, 0.549504, 0.544914,\
        0.540128, 0.535132, 0.529915, 0.524491, 0.518910, 0.513277, 0.507685, 0.501911, 0.495375, 0.488318,\
        0.480926, 0.473150, 0.464958, 0.456362, 0.447390, 0.438067, 0.428419, 0.418463, 0.408219, 0.397702,\
        0.386927],\
        [0.395536, 0.403779, 0.411795, 0.419651, 0.427426, 0.435192, 0.443000, 0.450863, 0.458752, 0.466596,\
        0.474270, 0.481553, 0.488117, 0.493690, 0.498593, 0.503589, 0.507782, 0.510551, 0.512731, 0.514120,\
        0.513988, 0.511951, 0.507716, 0.502052, 0.496340, 0.490554, 0.484906, 0.479533, 0.474193, 0.468636,\
        0.462792, 0.456700, 0.450379, 0.443821, 0.437046, 0.430129, 0.423192, 0.416395, 0.409964, 0.404131,\
        0.398893, 0.394012, 0.389454, 0.385481, 0.382317, 0.380165, 0.379228, 0.379716, 0.381854, 0.385878,\
        0.391959, 0.400026, 0.409729, 0.420451, 0.431573, 0.442647, 0.453405, 0.463682, 0.473331, 0.482180,\
        0.490143, 0.497542, 0.504826, 0.511379, 0.517209, 0.522414, 0.527146, 0.531630, 0.536107, 0.540799,\
        0.545884, 0.551475, 0.557599, 0.564196, 0.571138, 0.578262, 0.585383, 0.592155, 0.598125, 0.603620,\
        0.607043, 0.608137, 0.607920, 0.607040, 0.605867, 0.604151, 0.601532, 0.598438, 0.594841, 0.591078,\
        0.587462, 0.583773, 0.579883, 0.575853, 0.571747, 0.567569, 0.563291, 0.558885, 0.554331, 0.549615,\
        0.544728, 0.539663, 0.534419, 0.529011, 0.523470, 0.517834, 0.512070, 0.505978, 0.499330, 0.492293,\
        0.484961, 0.477146, 0.468855, 0.460145, 0.451061, 0.441633, 0.431886, 0.421839, 0.411512, 0.400921,\
        0.390080],\
        [0.396607, 0.405388, 0.414004, 0.422506, 0.430945, 0.439357, 0.447750, 0.456098, 0.464347, 0.472440,\
        0.480319, 0.487832, 0.494575, 0.500063, 0.504637, 0.509185, 0.513189, 0.516228, 0.518359, 0.519468,\
        0.519195, 0.517264, 0.513685, 0.509014, 0.503931, 0.498651, 0.493535, 0.488820, 0.483895, 0.478401,\
        0.472398, 0.466088, 0.459512, 0.452606, 0.445358, 0.437865, 0.430309, 0.422915, 0.415920, 0.409508,\
        0.403711, 0.398273, 0.393170, 0.388727, 0.385163, 0.382688, 0.381509, 0.381842, 0.383931, 0.388077,\
        0.394653, 0.403667, 0.414447, 0.426176, 0.438116, 0.449763, 0.460869, 0.471365, 0.481167, 0.490057,\
        0.497780, 0.504692, 0.511461, 0.517714, 0.523334, 0.528430, 0.533168, 0.537745, 0.542368, 0.547218,\
        0.552435, 0.558097, 0.564217, 0.570738, 0.577529, 0.584415, 0.591306, 0.598084, 0.603789, 0.608685,\
        0.612189, 0.613905, 0.614243, 0.613684, 0.612488, 0.610611, 0.607980, 0.604727, 0.600875, 0.596935,\
        0.593428, 0.589616, 0.585451, 0.581176, 0.576892, 0.572572, 0.568153, 0.563598, 0.558898, 0.554053,\
        0.549063, 0.543925, 0.538638, 0.533205, 0.527633, 0.521910, 0.515967, 0.509664, 0.502893, 0.495997,\
        0.488718, 0.480779, 0.472344, 0.463506, 0.454306, 0.444774, 0.434930, 0.424795, 0.414388, 0.403725,\
        0.392822],\
        [0.397271, 0.406555, 0.415726, 0.424823, 0.433879, 0.442900, 0.451853, 0.460665, 0.469231, 0.477464,\
        0.485376, 0.493054, 0.499981, 0.505530, 0.510224, 0.514560, 0.518393, 0.521442, 0.523562, 0.524610,\
        0.524397, 0.522801, 0.519908, 0.516056, 0.511639, 0.506934, 0.502364, 0.498218, 0.493621, 0.488077,\
        0.481833, 0.475395, 0.468687, 0.461486, 0.453745, 0.445615, 0.437362, 0.429272, 0.421600, 0.414514,\
        0.408047, 0.402097, 0.396652, 0.391885, 0.388052, 0.385389, 0.384113, 0.384442, 0.386622, 0.390950,\
        0.397921, 0.408122, 0.420189, 0.433006, 0.445757, 0.457856, 0.469091, 0.479605, 0.489472, 0.498375,\
        0.505959, 0.512545, 0.518696, 0.524416, 0.529661, 0.534514, 0.539129, 0.543683, 0.548352, 0.553281,\
        0.558572, 0.564273, 0.570377, 0.576835, 0.583523, 0.590170, 0.596565, 0.603099, 0.608960, 0.613724,\
        0.617242, 0.619330, 0.620116, 0.619862, 0.618754, 0.616851, 0.614182, 0.610827, 0.606943, 0.603038,\
        0.599467, 0.595266, 0.590698, 0.586168, 0.581760, 0.577338, 0.572779, 0.568058, 0.563195, 0.558209,\
        0.553105, 0.547880, 0.542526, 0.537036, 0.531394, 0.525572, 0.519517, 0.513171, 0.506564, 0.499770,\
        0.492154, 0.483954, 0.475360, 0.466401, 0.457096, 0.447465, 0.437530, 0.427312, 0.416829, 0.406099,\
        0.395139],\
        [0.397426, 0.407171, 0.416843, 0.426483, 0.436115, 0.445723, 0.455239, 0.464540, 0.473445, 0.481745,\
        0.489338, 0.496608, 0.503681, 0.509678, 0.514842, 0.519355, 0.523215, 0.526298, 0.528473, 0.529619,\
        0.529632, 0.528472, 0.526232, 0.523133, 0.519448, 0.515454, 0.511446, 0.507459, 0.503002, 0.497308,\
        0.490858, 0.484564, 0.477899, 0.470430, 0.462118, 0.453237, 0.444179, 0.435303, 0.426880, 0.419080,\
        0.411971, 0.405558, 0.399844, 0.394881, 0.390928, 0.388251, 0.387073, 0.387625, 0.390189, 0.395131,\
        0.402944, 0.414070, 0.427195, 0.441039, 0.454606, 0.467055, 0.478093, 0.488278, 0.498022, 0.506757,\
        0.514175, 0.520549, 0.526229, 0.531403, 0.536172, 0.540665, 0.545041, 0.549468, 0.554094, 0.559036,\
        0.564363, 0.570085, 0.576160, 0.582544, 0.589227, 0.595934, 0.602112, 0.608241, 0.613988, 0.618769,\
        0.622375, 0.624709, 0.625811, 0.625813, 0.624843, 0.622991, 0.620335, 0.616990, 0.613165, 0.609171,\
        0.605062, 0.600457, 0.595528, 0.590825, 0.586423, 0.581935, 0.577196, 0.572260, 0.567203, 0.562057,\
        0.556828, 0.551502, 0.546064, 0.540491, 0.534760, 0.528837, 0.522684, 0.516277, 0.509608, 0.502549,\
        0.494780, 0.486508, 0.477842, 0.468801, 0.459409, 0.449690, 0.439670, 0.429372, 0.418818, 0.408027,\
        0.397015],\
        [0.396980, 0.407129, 0.417239, 0.427367, 0.437540, 0.447726, 0.457830, 0.467692, 0.477082, 0.485662,\
        0.493004, 0.499197, 0.506436, 0.513114, 0.518778, 0.523582, 0.527601, 0.530807, 0.533127, 0.534485,\
        0.534828, 0.534154, 0.532542, 0.530149, 0.527168, 0.523799, 0.520177, 0.516237, 0.511626, 0.505920,\
        0.499689, 0.493777, 0.487207, 0.479388, 0.470336, 0.460553, 0.450598, 0.440890, 0.431704, 0.423209,\
        0.415504, 0.408646, 0.402678, 0.397632, 0.393720, 0.391206, 0.390332, 0.391350, 0.394564, 0.400505,\
        0.409497, 0.421357, 0.435250, 0.450044, 0.464566, 0.477517, 0.488012, 0.497490, 0.506926, 0.515442,\
        0.522662, 0.528744, 0.533967, 0.538586, 0.542815, 0.546856, 0.550900, 0.555115, 0.559633, 0.564541,\
        0.569884, 0.575643, 0.581710, 0.587937, 0.594455, 0.601390, 0.607719, 0.613713, 0.619275, 0.624023,\
        0.627707, 0.630214, 0.631537, 0.631740, 0.630910, 0.629137, 0.626519, 0.623182, 0.619298, 0.615031,\
        0.610406, 0.605355, 0.600063, 0.595311, 0.591095, 0.586456, 0.581399, 0.576173, 0.570890, 0.565575,\
        0.560214, 0.554780, 0.549242, 0.543570, 0.537732, 0.531691, 0.525411, 0.518849, 0.511940, 0.504554,\
        0.496712, 0.488498, 0.479808, 0.470708, 0.461239, 0.451438, 0.441335, 0.430962, 0.420342, 0.409495,\
        0.398440],\
        [0.395840, 0.406331, 0.416801, 0.427359, 0.438047, 0.448806, 0.459512, 0.469995, 0.480033, 0.489325,\
        0.497338, 0.503717, 0.510157, 0.516601, 0.522381, 0.527370, 0.531567, 0.534956, 0.537498, 0.539151,\
        0.539888, 0.539724, 0.538720, 0.536983, 0.534641, 0.531807, 0.528524, 0.524708, 0.520166, 0.514817,\
        0.509042, 0.503128, 0.496498, 0.488142, 0.478123, 0.467317, 0.456460, 0.445964, 0.436067, 0.426929,\
        0.418672, 0.411393, 0.405173, 0.400105, 0.396361, 0.394121, 0.393713, 0.395418, 0.399479, 0.406414,\
        0.416388, 0.429009, 0.443617, 0.459300, 0.474957, 0.489093, 0.499362, 0.507954, 0.516830, 0.524837,\
        0.531640, 0.537271, 0.541943, 0.545939, 0.549551, 0.553058, 0.556697, 0.560645, 0.565013, 0.569858,\
        0.575203, 0.581042, 0.587255, 0.593421, 0.599435, 0.606371, 0.613097, 0.619274, 0.624839, 0.629590,\
        0.633336, 0.635962, 0.637436, 0.637785, 0.637068, 0.635365, 0.632770, 0.629400, 0.625389, 0.620852,\
        0.615862, 0.610498, 0.605031, 0.600205, 0.596069, 0.590825, 0.585273, 0.579721, 0.574220, 0.568747,\
        0.563258, 0.557711, 0.552063, 0.546277, 0.540315, 0.534140, 0.527709, 0.520969, 0.513863, 0.506353,\
        0.498459, 0.490096, 0.481321, 0.472147, 0.462592, 0.452700, 0.442512, 0.432065, 0.421384, 0.410489,\
        0.399398],\
        [0.393896, 0.404660, 0.415399, 0.426358, 0.437551, 0.448872, 0.460166, 0.471245, 0.481887, 0.491849,\
        0.500695, 0.507491, 0.513791, 0.519936, 0.525649, 0.530729, 0.535096, 0.538707, 0.541530, 0.543535,\
        0.544711, 0.545071, 0.544659, 0.543541, 0.541792, 0.539463, 0.536553, 0.532993, 0.528694, 0.523664,\
        0.518064, 0.511957, 0.504966, 0.496070, 0.484981, 0.473245, 0.461665, 0.450537, 0.440036, 0.430325,\
        0.421559, 0.413880, 0.407427, 0.402355, 0.398781, 0.396754, 0.396897, 0.399426, 0.404450, 0.412300,\
        0.423051, 0.436335, 0.451530, 0.467853, 0.484334, 0.499366, 0.510601, 0.519259, 0.527455, 0.534874,\
        0.541142, 0.546193, 0.550196, 0.553459, 0.556355, 0.559247, 0.562430, 0.566089, 0.570301, 0.575076,\
        0.580390, 0.586257, 0.592780, 0.599437, 0.605517, 0.612115, 0.618827, 0.625115, 0.630743, 0.635532,\
        0.639328, 0.642030, 0.643596, 0.644036, 0.643392, 0.641726, 0.639124, 0.635685, 0.631525, 0.626762,\
        0.621522, 0.615985, 0.610486, 0.605473, 0.600441, 0.594556, 0.588631, 0.582843, 0.577181, 0.571580,\
        0.565975, 0.560309, 0.554538, 0.548621, 0.542521, 0.536198, 0.529610, 0.522705, 0.515429, 0.507756,\
        0.499698, 0.491243, 0.482410, 0.473139, 0.463469, 0.453466, 0.443184, 0.432663, 0.421928, 0.410995,\
        0.399879],\
        [0.390983, 0.401890, 0.412924, 0.424326, 0.436035, 0.447893, 0.459725, 0.471326, 0.482426, 0.492621,\
        0.501566, 0.509463, 0.516288, 0.522550, 0.528326, 0.533538, 0.538109, 0.541989, 0.545145, 0.547552,\
        0.549201, 0.550099, 0.550273, 0.549758, 0.548587, 0.546773, 0.544288, 0.541067, 0.537036, 0.532160,\
        0.526433, 0.519770, 0.511865, 0.502154, 0.490433, 0.478255, 0.466335, 0.454790, 0.443790, 0.433556,\
        0.424301, 0.416216, 0.409472, 0.404241, 0.400638, 0.398752, 0.399550, 0.402919, 0.408851, 0.417584,\
        0.429081, 0.442974, 0.458613, 0.475192, 0.491877, 0.507221, 0.519601, 0.529537, 0.538000, 0.545216,\
        0.551058, 0.555504, 0.558742, 0.561149, 0.563208, 0.565402, 0.568106, 0.571501, 0.575598, 0.580345,\
        0.585644, 0.591328, 0.597737, 0.604869, 0.611611, 0.618358, 0.625072, 0.631408, 0.637086, 0.641908,\
        0.645731, 0.648462, 0.650064, 0.650539, 0.649919, 0.648251, 0.645604, 0.642064, 0.637739, 0.632758,\
        0.627275, 0.621489, 0.615639, 0.609847, 0.603858, 0.597631, 0.591501, 0.585593, 0.579831, 0.574123,\
        0.568399, 0.562601, 0.556685, 0.550614, 0.544355, 0.537873, 0.531130, 0.524079, 0.516664, 0.508809,\
        0.500476, 0.492070, 0.483142, 0.473694, 0.463853, 0.453710, 0.443323, 0.432732, 0.421955, 0.410997,\
        0.399871],\
        [0.387238, 0.398146, 0.409500, 0.421381, 0.433582, 0.445916, 0.458206, 0.470258, 0.481831, 0.492614,\
        0.502175, 0.510332, 0.517579, 0.524167, 0.530192, 0.535648, 0.540500, 0.544712, 0.548257, 0.551114,\
        0.553271, 0.554727, 0.555492, 0.555581, 0.554998, 0.553730, 0.551728, 0.548907, 0.545159, 0.540365,\
        0.534388, 0.527033, 0.518020, 0.507124, 0.494951, 0.482934, 0.470988, 0.459108, 0.447611, 0.436846,\
        0.427095, 0.418580, 0.411482, 0.405951, 0.402123, 0.400534, 0.401854, 0.405864, 0.412570, 0.422081,\
        0.434316, 0.448876, 0.465031, 0.481807, 0.498312, 0.514224, 0.527875, 0.539080, 0.548275, 0.555687,\
        0.561310, 0.565187, 0.567589, 0.569011, 0.570092, 0.571502, 0.573740, 0.576970, 0.581050, 0.585823,\
        0.591329, 0.597058, 0.603105, 0.610447, 0.617794, 0.624931, 0.631805, 0.638216, 0.643933, 0.648765,\
        0.652573, 0.655278, 0.656851, 0.657301, 0.656654, 0.654942, 0.652213, 0.648537, 0.644015, 0.638783,\
        0.633001, 0.626848, 0.620483, 0.613968, 0.607277, 0.600515, 0.594168, 0.588161, 0.582292, 0.576456,\
        0.570583, 0.564619, 0.558523, 0.552264, 0.545816, 0.539155, 0.532249, 0.525066, 0.517576, 0.509763,\
        0.501611, 0.492853, 0.483501, 0.473753, 0.463692, 0.453386, 0.442890, 0.432239, 0.421443, 0.410486,\
        0.399365],\
        [0.383105, 0.394025, 0.405570, 0.417796, 0.430364, 0.443043, 0.455666, 0.468058, 0.480012, 0.491267,\
        0.501314, 0.509906, 0.517660, 0.524713, 0.531132, 0.536950, 0.542172, 0.546790, 0.550787, 0.554144,\
        0.556847, 0.558887, 0.560260, 0.560966, 0.560995, 0.560317, 0.558865, 0.556528, 0.553147, 0.548521,\
        0.542420, 0.534601, 0.524868, 0.513267, 0.500655, 0.488641, 0.476428, 0.463954, 0.451813, 0.440450,\
        0.430177, 0.421227, 0.413783, 0.408029, 0.404226, 0.402829, 0.404239, 0.408518, 0.415690, 0.425796,\
        0.438734, 0.454116, 0.471160, 0.488730, 0.505862, 0.522077, 0.536465, 0.548590, 0.558530, 0.566333,\
        0.571901, 0.575245, 0.576748, 0.577063, 0.577004, 0.577528, 0.579360, 0.582642, 0.586924, 0.591600,\
        0.597262, 0.603668, 0.610015, 0.617184, 0.624682, 0.632061, 0.639098, 0.645589, 0.651327, 0.656128,\
        0.659862, 0.662466, 0.663934, 0.664292, 0.663568, 0.661774, 0.658926, 0.655073, 0.650309, 0.644776,\
        0.638640, 0.632066, 0.625193, 0.618110, 0.610891, 0.603729, 0.597026, 0.590772, 0.584695, 0.578657,\
        0.572576, 0.566391, 0.560062, 0.553567, 0.546890, 0.540015, 0.532920, 0.525578, 0.517965, 0.510080,\
        0.501899, 0.492857, 0.483186, 0.473181, 0.462910, 0.452441, 0.441835, 0.431135, 0.420360, 0.409464,\
        0.398370],\
        [0.378829, 0.389848, 0.401441, 0.413821, 0.426551, 0.439385, 0.452184, 0.464780, 0.476943, 0.488360,\
        0.498697, 0.508128, 0.516577, 0.524208, 0.531117, 0.537382, 0.543054, 0.548150, 0.552664, 0.556578,\
        0.559870, 0.562525, 0.564530, 0.565875, 0.566545, 0.566505, 0.565684, 0.563951, 0.561102, 0.556863,\
        0.550931, 0.543060, 0.533174, 0.521544, 0.509007, 0.496494, 0.483081, 0.469594, 0.456642, 0.444609,\
        0.433794, 0.424410, 0.416638, 0.410676, 0.406800, 0.405363, 0.406687, 0.410961, 0.418303, 0.428774,\
        0.442311, 0.458622, 0.476989, 0.495979, 0.514116, 0.530866, 0.545733, 0.558424, 0.568945, 0.577204,\
        0.582837, 0.585637, 0.586192, 0.585368, 0.583984, 0.583514, 0.585053, 0.588655, 0.593679, 0.598349,\
        0.603632, 0.610459, 0.617486, 0.624849, 0.632432, 0.639922, 0.647048, 0.653576, 0.659290, 0.664002,\
        0.667583, 0.669987, 0.671250, 0.671441, 0.670595, 0.668692, 0.665696, 0.661620, 0.656558, 0.650670,\
        0.644137, 0.637137, 0.629816, 0.622296, 0.614706, 0.607243, 0.600145, 0.593506, 0.587093, 0.580765,\
        0.574404, 0.567929, 0.561302, 0.554510, 0.547550, 0.540415, 0.533091, 0.525544, 0.517716, 0.509496,\
        0.500785, 0.491730, 0.482056, 0.471906, 0.461460, 0.450836, 0.440116, 0.429359, 0.418609, 0.407882,\
        0.396942],\
        [0.374407, 0.385562, 0.397263, 0.409578, 0.422211, 0.434998, 0.447838, 0.460555, 0.472906, 0.484623,\
        0.495533, 0.505456, 0.514514, 0.522741, 0.530156, 0.536908, 0.543092, 0.548737, 0.553836, 0.558365,\
        0.562297, 0.565605, 0.568268, 0.570273, 0.571609, 0.572250, 0.572141, 0.571153, 0.569054, 0.565494,\
        0.560083, 0.552538, 0.542847, 0.531348, 0.518651, 0.505049, 0.490512, 0.476095, 0.462284, 0.449550,\
        0.438189, 0.428374, 0.420253, 0.414012, 0.409893, 0.408199, 0.409236, 0.413270, 0.420514, 0.431102,\
        0.445036, 0.462135, 0.481972, 0.502975, 0.522544, 0.540356, 0.555731, 0.568658, 0.579476, 0.588141,\
        0.593925, 0.596103, 0.595737, 0.594076, 0.591136, 0.589724, 0.591260, 0.595216, 0.600910, 0.606484,\
        0.612006, 0.618646, 0.625873, 0.633408, 0.641066, 0.648587, 0.655710, 0.662198, 0.667823, 0.672371,\
        0.675694, 0.677769, 0.678697, 0.678631, 0.677634, 0.675619, 0.672453, 0.668100, 0.662670, 0.656365,\
        0.649406, 0.641988, 0.634275, 0.626404, 0.618514, 0.610768, 0.603334, 0.596249, 0.589434, 0.582770,\
        0.576068, 0.569227, 0.562223, 0.555063, 0.547759, 0.540316, 0.532722, 0.524942, 0.516916, 0.508542,\
        0.499625, 0.490237, 0.480324, 0.469976, 0.459350, 0.448570, 0.437728, 0.426882, 0.416064, 0.405298,\
        0.394500],\
        [0.369715, 0.380965, 0.392721, 0.404894, 0.417239, 0.429851, 0.442672, 0.455490, 0.468059, 0.480123,\
        0.491416, 0.501813, 0.511693, 0.520394, 0.528234, 0.535488, 0.542243, 0.548509, 0.554266, 0.559478,\
        0.564104, 0.568106, 0.571456, 0.574138, 0.576150, 0.577493, 0.578150, 0.578034, 0.576915, 0.574361,\
        0.569820, 0.562881, 0.553530, 0.542110, 0.529024, 0.514520, 0.499151, 0.483753, 0.468970, 0.455529,\
        0.443638, 0.433377, 0.424859, 0.418232, 0.413701, 0.411538, 0.412060, 0.415608, 0.422504, 0.432996,\
        0.447147, 0.464746, 0.485426, 0.508554, 0.531282, 0.551001, 0.566802, 0.579427, 0.590006, 0.598682,\
        0.604227, 0.606001, 0.605193, 0.602953, 0.598561, 0.597301, 0.599360, 0.603734, 0.609466, 0.615539,\
        0.621654, 0.628284, 0.635465, 0.643008, 0.650653, 0.658100, 0.665097, 0.671436, 0.676881, 0.681178,\
        0.684120, 0.685705, 0.686124, 0.685693, 0.684550, 0.682457, 0.679108, 0.674398, 0.668503, 0.661716,\
        0.654307, 0.646489, 0.638429, 0.630264, 0.622115, 0.614093, 0.606284, 0.598763, 0.591641, 0.584664,\
        0.577549, 0.570250, 0.562784, 0.555181, 0.547471, 0.539669, 0.531771, 0.523734, 0.515489, 0.507027,\
        0.498187, 0.488305, 0.478027, 0.467431, 0.456611, 0.445676, 0.434720, 0.423794, 0.412897, 0.402022,\
        0.391306],\
        [0.364619, 0.375809, 0.387431, 0.399395, 0.411388, 0.423893, 0.436718, 0.449639, 0.462436, 0.474900,\
        0.486858, 0.498233, 0.508305, 0.517032, 0.525239, 0.533062, 0.540475, 0.547446, 0.553939, 0.559904,\
        0.565285, 0.570028, 0.574095, 0.577467, 0.580150, 0.582178, 0.583597, 0.584411, 0.584463, 0.583250,\
        0.579930, 0.573805, 0.564889, 0.553694, 0.540580, 0.525724, 0.509557, 0.492815, 0.476975, 0.462906,\
        0.450477, 0.439712, 0.430722, 0.423600, 0.418498, 0.415659, 0.415425, 0.418219, 0.424512, 0.434748,\
        0.449193, 0.467677, 0.489472, 0.515435, 0.542154, 0.564079, 0.579771, 0.591235, 0.601071, 0.608976,\
        0.613921, 0.615647, 0.614920, 0.612518, 0.609564, 0.608927, 0.611192, 0.615461, 0.620864, 0.626766,\
        0.632901, 0.639384, 0.646354, 0.653763, 0.661293, 0.668513, 0.675188, 0.681221, 0.686353, 0.690307,\
        0.692719, 0.693657, 0.693349, 0.692421, 0.691201, 0.689096, 0.685539, 0.680332, 0.673847, 0.666522,\
        0.658662, 0.650472, 0.642109, 0.633702, 0.625352, 0.617131, 0.609062, 0.601196, 0.593884, 0.586455,\
        0.578789, 0.570927, 0.562915, 0.554801, 0.546629, 0.538425, 0.530197, 0.521904, 0.513398, 0.504425,\
        0.495240, 0.485487, 0.475076, 0.464288, 0.453289, 0.442209, 0.431161, 0.420221, 0.409387, 0.398609,\
        0.387965],\
        [0.359091, 0.370074, 0.381358, 0.392850, 0.404764, 0.417249, 0.430066, 0.443048, 0.456002, 0.468751,\
        0.481156, 0.492961, 0.503115, 0.512162, 0.521041, 0.529613, 0.537788, 0.545548, 0.552857, 0.559651,\
        0.565852, 0.571390, 0.576210, 0.580287, 0.583628, 0.586291, 0.588391, 0.590066, 0.591349, 0.591767,\
        0.590036, 0.584848, 0.576424, 0.565845, 0.553354, 0.538674, 0.521842, 0.503740, 0.486920, 0.472272,\
        0.459071, 0.447660, 0.438123, 0.430414, 0.424600, 0.420895, 0.419659, 0.421403, 0.426775, 0.436511,\
        0.451322, 0.471653, 0.497124, 0.526344, 0.556318, 0.580575, 0.595480, 0.605453, 0.613961, 0.620531,\
        0.624601, 0.626227, 0.626001, 0.624774, 0.623717, 0.624159, 0.626507, 0.630337, 0.635134, 0.640517,\
        0.646209, 0.652163, 0.658623, 0.665755, 0.673121, 0.679922, 0.685928, 0.691468, 0.696033, 0.699465,\
        0.701190, 0.701387, 0.700222, 0.698767, 0.697467, 0.695373, 0.691528, 0.685562, 0.678390, 0.670545,\
        0.662283, 0.653771, 0.645149, 0.636543, 0.628055, 0.619754, 0.611681, 0.603901, 0.596104, 0.587950,\
        0.579630, 0.571138, 0.562522, 0.553849, 0.545176, 0.536536, 0.527949, 0.519432, 0.510902, 0.501930,\
        0.492280, 0.482175, 0.471570, 0.460612, 0.449455, 0.438240, 0.427104, 0.416186, 0.405540, 0.394948,\
        0.384402],\
        [0.353212, 0.363945, 0.374931, 0.386200, 0.397912, 0.410162, 0.422858, 0.435808, 0.448774, 0.461552,\
        0.473924, 0.485526, 0.495947, 0.505915, 0.515820, 0.525236, 0.534222, 0.542835, 0.551038, 0.558738,\
        0.565830, 0.572221, 0.577841, 0.582652, 0.586655, 0.589906, 0.592572, 0.594916, 0.597211, 0.599265,\
        0.599303, 0.595082, 0.587236, 0.578100, 0.567244, 0.553624, 0.536913, 0.518247, 0.500370, 0.484141,\
        0.469543, 0.457416, 0.447324, 0.438963, 0.432328, 0.427600, 0.425139, 0.425524, 0.429568, 0.438306,\
        0.452920, 0.474557, 0.503401, 0.536797, 0.570632, 0.598715, 0.612876, 0.621706, 0.628406, 0.633426,\
        0.636627, 0.638192, 0.638724, 0.639030, 0.639878, 0.641717, 0.644492, 0.647961, 0.652071, 0.656841,\
        0.661905, 0.666915, 0.672361, 0.679015, 0.686277, 0.692531, 0.697264, 0.702068, 0.705829, 0.708277,\
        0.709234, 0.708716, 0.707066, 0.705132, 0.703150, 0.700921, 0.696360, 0.689481, 0.681780, 0.673584,\
        0.665031, 0.656257, 0.647404, 0.638612, 0.630002, 0.621649, 0.613545, 0.605536, 0.597310, 0.588763,\
        0.579865, 0.570732, 0.561491, 0.552246, 0.543067, 0.533983, 0.524990, 0.516130, 0.507591, 0.498885,\
        0.488917, 0.478426, 0.467596, 0.456492, 0.445217, 0.433888, 0.422643, 0.411672, 0.401134, 0.390821,\
        0.380550],\
        [0.347088, 0.357587, 0.368335, 0.379385, 0.390828, 0.402761, 0.415258, 0.428081, 0.440916, 0.453549,\
        0.465788, 0.477437, 0.488474, 0.499327, 0.509944, 0.520039, 0.529811, 0.539334, 0.548514, 0.557199,\
        0.565248, 0.572551, 0.579022, 0.584615, 0.589330, 0.593194, 0.596362, 0.599383, 0.602412, 0.605271,\
        0.606006, 0.602822, 0.596469, 0.590187, 0.582019, 0.570342, 0.554292, 0.535393, 0.516064, 0.497453,\
        0.481649, 0.469194, 0.458555, 0.449467, 0.441946, 0.436108, 0.432271, 0.431034, 0.433344, 0.440480,\
        0.453970, 0.475411, 0.506312, 0.544676, 0.581606, 0.609749, 0.626894, 0.636390, 0.642558, 0.646855,\
        0.649670, 0.651393, 0.652801, 0.654742, 0.657642, 0.661239, 0.664802, 0.668040, 0.671412, 0.675725,\
        0.680389, 0.684154, 0.687843, 0.693608, 0.700618, 0.706071, 0.709833, 0.713181, 0.715761, 0.717112,\
        0.717136, 0.715896, 0.713693, 0.710960, 0.707854, 0.704049, 0.698581, 0.691620, 0.683917, 0.675621,\
        0.666886, 0.657880, 0.648781, 0.639764, 0.630981, 0.622537, 0.614402, 0.606287, 0.597806, 0.588836,\
        0.579342, 0.569565, 0.559712, 0.549920, 0.540277, 0.530812, 0.521466, 0.512066, 0.502848, 0.494185,\
        0.484510, 0.474060, 0.463175, 0.452020, 0.440709, 0.429342, 0.418018, 0.406926, 0.396466, 0.386476,\
        0.376553],\
        [0.340798, 0.351080, 0.361613, 0.372445, 0.383636, 0.395261, 0.407507, 0.420075, 0.432646, 0.445036,\
        0.457137, 0.468928, 0.480509, 0.492032, 0.503231, 0.513953, 0.524547, 0.535087, 0.545340, 0.555070,\
        0.564123, 0.572389, 0.579767, 0.586183, 0.591664, 0.596340, 0.600078, 0.603839, 0.607559, 0.610550,\
        0.611608, 0.609813, 0.606254, 0.602066, 0.597074, 0.587649, 0.572204, 0.552856, 0.532331, 0.512470,\
        0.496339, 0.483379, 0.471855, 0.461967, 0.453581, 0.446664, 0.441430, 0.438443, 0.438757, 0.443916,\
        0.455853, 0.476825, 0.509556, 0.552171, 0.592985, 0.620727, 0.638061, 0.648640, 0.655449, 0.660198,\
        0.663446, 0.665648, 0.667998, 0.671640, 0.676784, 0.682572, 0.687246, 0.690457, 0.693132, 0.697119,\
        0.701877, 0.704551, 0.706621, 0.710658, 0.715936, 0.720093, 0.722905, 0.724850, 0.725975, 0.726065,\
        0.725023, 0.722898, 0.719863, 0.716104, 0.711618, 0.706118, 0.699591, 0.692843, 0.685247, 0.676890,\
        0.667958, 0.658669, 0.649251, 0.639915, 0.630843, 0.622194, 0.614054, 0.606173, 0.597629, 0.588052,\
        0.577879, 0.567486, 0.557085, 0.546811, 0.536770, 0.527047, 0.517650, 0.508344, 0.498595, 0.489392,\
        0.479664, 0.469253, 0.458403, 0.447294, 0.436056, 0.424803, 0.413615, 0.402559, 0.392210, 0.382387,\
        0.372660],\
        [0.334390, 0.344462, 0.354792, 0.365426, 0.376420, 0.387848, 0.399739, 0.411912, 0.424109, 0.436162,\
        0.448046, 0.459865, 0.471736, 0.483656, 0.495436, 0.506888, 0.518477, 0.530225, 0.541613, 0.552384,\
        0.562439, 0.571698, 0.580050, 0.587347, 0.593466, 0.598808, 0.603617, 0.608088, 0.612268, 0.615566,\
        0.617311, 0.617204, 0.615756, 0.613484, 0.609933, 0.602428, 0.588053, 0.568927, 0.548892, 0.529821,\
        0.513424, 0.499261, 0.486751, 0.476250, 0.467137, 0.459323, 0.452858, 0.448166, 0.446428, 0.449540,\
        0.459949, 0.480705, 0.515788, 0.560780, 0.600843, 0.629667, 0.647524, 0.658969, 0.667073, 0.673374,\
        0.678149, 0.681160, 0.684554, 0.689994, 0.697332, 0.705135, 0.710992, 0.714787, 0.717915, 0.721722,\
        0.725627, 0.727757, 0.728883, 0.730583, 0.732964, 0.735006, 0.736238, 0.736666, 0.736292, 0.735027,\
        0.732808, 0.729653, 0.725641, 0.720842, 0.715238, 0.708780, 0.701710, 0.694265, 0.686355, 0.677713,\
        0.668413, 0.658703, 0.648848, 0.639072, 0.629555, 0.620460, 0.611997, 0.604443, 0.596069, 0.585868,\
        0.575188, 0.564363, 0.553560, 0.542906, 0.532521, 0.522555, 0.513169, 0.504482, 0.495341, 0.485034,\
        0.474737, 0.464202, 0.453397, 0.442403, 0.431325, 0.420301, 0.409513, 0.398947, 0.388577, 0.378665,\
        0.368974],\
        [0.327897, 0.337756, 0.347877, 0.358308, 0.369094, 0.380268, 0.391808, 0.403575, 0.415355, 0.426976,\
        0.438499, 0.450213, 0.462287, 0.474620, 0.486936, 0.499102, 0.512012, 0.525067, 0.537451, 0.549128,\
        0.560107, 0.570351, 0.579758, 0.588149, 0.595172, 0.600527, 0.606379, 0.611742, 0.616413, 0.620150,\
        0.622626, 0.623700, 0.623503, 0.622040, 0.618589, 0.611367, 0.598292, 0.582092, 0.564005, 0.546348,\
        0.530249, 0.515694, 0.502890, 0.491927, 0.482178, 0.473833, 0.466527, 0.460319, 0.456634, 0.457802,\
        0.466943, 0.487524, 0.523616, 0.570688, 0.612107, 0.638335, 0.655890, 0.667998, 0.677848, 0.686421,\
        0.694292, 0.698717, 0.703798, 0.711340, 0.720316, 0.728812, 0.735319, 0.740016, 0.744065, 0.748064,\
        0.751206, 0.752360, 0.751881, 0.751073, 0.750620, 0.750257, 0.749566, 0.748326, 0.746424, 0.743774,\
        0.740322, 0.736070, 0.731055, 0.725313, 0.718858, 0.711701, 0.703908, 0.695894, 0.687518, 0.678275,\
        0.668350, 0.658041, 0.647628, 0.637332, 0.627288, 0.617556, 0.608268, 0.599602, 0.591630, 0.581803,\
        0.571144, 0.560195, 0.549203, 0.538309, 0.527625, 0.517331, 0.507645, 0.498549, 0.489616, 0.479748,\
        0.469440, 0.458915, 0.448225, 0.437421, 0.426556, 0.415726, 0.405174, 0.395018, 0.384941, 0.375098,\
        0.365439],\
        [0.321345, 0.330975, 0.340868, 0.351065, 0.361593, 0.372459, 0.383630, 0.394999, 0.406374, 0.417472,\
        0.428431, 0.439980, 0.452365, 0.465361, 0.478638, 0.492136, 0.506239, 0.519875, 0.532818, 0.545167,\
        0.556937, 0.568103, 0.578567, 0.588158, 0.596689, 0.603336, 0.609114, 0.614744, 0.619772, 0.623926,\
        0.626966, 0.628730, 0.629103, 0.627809, 0.624153, 0.617112, 0.606479, 0.592593, 0.576691, 0.561170,\
        0.546330, 0.532324, 0.519519, 0.508054, 0.497815, 0.489557, 0.481938, 0.474441, 0.469158, 0.468461,\
        0.476346, 0.497481, 0.532792, 0.576726, 0.617616, 0.646652, 0.664592, 0.676647, 0.688568, 0.700107,\
        0.710541, 0.719005, 0.726728, 0.735740, 0.745466, 0.753980, 0.760163, 0.764926, 0.769423, 0.773832,\
        0.776654, 0.776495, 0.773759, 0.770327, 0.767420, 0.764887, 0.762326, 0.759438, 0.756043, 0.752033,\
        0.747353, 0.741999, 0.735999, 0.729396, 0.722245, 0.714608, 0.706543, 0.698045, 0.688713, 0.678501,\
        0.667716, 0.656668, 0.645626, 0.634810, 0.624341, 0.614158, 0.604109, 0.595066, 0.586235, 0.576565,\
        0.566079, 0.555193, 0.544202, 0.533260, 0.522414, 0.511763, 0.501833, 0.492625, 0.483504, 0.473888,\
        0.463786, 0.453414, 0.442924, 0.432410, 0.421879, 0.411267, 0.400805, 0.390937, 0.381233, 0.371564,\
        0.361977],\
        [0.314766, 0.324143, 0.333783, 0.343713, 0.353930, 0.364419, 0.375155, 0.386085, 0.397101, 0.407490,\
        0.417678, 0.429174, 0.442121, 0.456008, 0.470438, 0.485312, 0.500190, 0.514011, 0.527363, 0.540233,\
        0.552665, 0.564661, 0.576098, 0.586697, 0.595933, 0.603789, 0.610637, 0.616654, 0.622028, 0.626570,\
        0.630034, 0.632198, 0.632827, 0.631572, 0.627917, 0.621353, 0.611771, 0.599481, 0.586860, 0.574432,\
        0.561342, 0.548116, 0.535428, 0.523700, 0.513380, 0.505139, 0.497359, 0.489075, 0.483470, 0.481207,\
        0.486838, 0.507931, 0.542773, 0.585250, 0.627847, 0.659660, 0.677221, 0.687376, 0.699245, 0.715566,\
        0.729402, 0.741042, 0.751044, 0.760671, 0.771108, 0.779552, 0.784361, 0.788282, 0.792405, 0.796877,\
        0.799285, 0.797786, 0.792523, 0.786803, 0.782221, 0.778014, 0.773906, 0.769561, 0.764808, 0.759527,\
        0.753669, 0.747240, 0.740277, 0.732836, 0.724997, 0.716878, 0.708569, 0.699658, 0.689283, 0.678001,\
        0.666321, 0.654513, 0.642825, 0.631517, 0.620798, 0.610706, 0.600953, 0.590892, 0.581120, 0.571034,\
        0.560486, 0.549652, 0.538774, 0.528020, 0.517380, 0.506657, 0.496377, 0.487112, 0.477766, 0.468083,\
        0.458047, 0.447796, 0.437508, 0.427349, 0.417389, 0.407395, 0.397160, 0.387376, 0.377691, 0.368078,\
        0.358542],\
        [0.308203, 0.317300, 0.326672, 0.336319, 0.346180, 0.356218, 0.366410, 0.376701, 0.386897, 0.396506,\
        0.406261, 0.418063, 0.431849, 0.446569, 0.461749, 0.477144, 0.492284, 0.506859, 0.520748, 0.534045,\
        0.547038, 0.559785, 0.572129, 0.583800, 0.594480, 0.603446, 0.610515, 0.617015, 0.622895, 0.627894,\
        0.631754, 0.634230, 0.635065, 0.633956, 0.630567, 0.624631, 0.616132, 0.605847, 0.596627, 0.586825,\
        0.575011, 0.562423, 0.549904, 0.537955, 0.527166, 0.517908, 0.509526, 0.502281, 0.498491, 0.496442,\
        0.500483, 0.522508, 0.557888, 0.600818, 0.647555, 0.682354, 0.697766, 0.706474, 0.718875, 0.735639,\
        0.751571, 0.764813, 0.775864, 0.784049, 0.794226, 0.803298, 0.806604, 0.809562, 0.812614, 0.815462,\
        0.816166, 0.813185, 0.806777, 0.799981, 0.794287, 0.789004, 0.783802, 0.778304, 0.772421, 0.766001,\
        0.759045, 0.751591, 0.743686, 0.735383, 0.726750, 0.717853, 0.708802, 0.699505, 0.688306, 0.676327,\
        0.663983, 0.651536, 0.639236, 0.627394, 0.616387, 0.606255, 0.596545, 0.586320, 0.575904, 0.565378,\
        0.554643, 0.543788, 0.533023, 0.522595, 0.512628, 0.502680, 0.492330, 0.482317, 0.472478, 0.462533,\
        0.452407, 0.442163, 0.431963, 0.422042, 0.412657, 0.403816, 0.394112, 0.383997, 0.374186, 0.364573,\
        0.355082],\
        [0.301709, 0.310495, 0.319630, 0.329015, 0.338489, 0.348020, 0.357611, 0.367205, 0.376648, 0.385856,\
        0.395482, 0.407790, 0.422217, 0.437280, 0.452639, 0.468137, 0.483558, 0.498654, 0.512798, 0.526392,\
        0.539895, 0.553355, 0.566503, 0.579011, 0.590497, 0.600038, 0.608116, 0.615625, 0.622277, 0.627865,\
        0.632179, 0.635000, 0.636112, 0.635312, 0.632450, 0.627513, 0.620817, 0.613315, 0.606082, 0.597643,\
        0.586490, 0.574636, 0.562366, 0.549978, 0.538258, 0.527891, 0.519274, 0.513231, 0.510836, 0.511890,\
        0.520526, 0.543273, 0.579070, 0.625016, 0.673631, 0.709414, 0.724190, 0.733023, 0.744758, 0.760718,\
        0.776731, 0.789189, 0.800515, 0.807088, 0.814222, 0.822279, 0.826222, 0.828429, 0.829977, 0.830599,\
        0.829195, 0.824898, 0.818079, 0.810418, 0.802693, 0.797912, 0.791498, 0.785439, 0.778684, 0.771246,\
        0.763288, 0.754884, 0.746077, 0.736912, 0.727431, 0.717599, 0.707202, 0.696567, 0.685609, 0.673449,\
        0.660721, 0.647837, 0.635040, 0.622579, 0.611044, 0.601004, 0.591009, 0.580846, 0.570330, 0.559582,\
        0.548681, 0.537752, 0.527005, 0.516707, 0.507318, 0.498297, 0.488006, 0.477634, 0.467405, 0.457200,\
        0.446939, 0.436629, 0.426371, 0.416394, 0.407031, 0.398442, 0.389550, 0.379996, 0.370408, 0.360926,\
        0.351540],\
        [0.295315, 0.303818, 0.312848, 0.321993, 0.331017, 0.340009, 0.349056, 0.358168, 0.367343, 0.376765,\
        0.387154, 0.399606, 0.413666, 0.428295, 0.443259, 0.458466, 0.473765, 0.488849, 0.503306, 0.517179,\
        0.531267, 0.545504, 0.559338, 0.572376, 0.584239, 0.594604, 0.604098, 0.612769, 0.620304, 0.626578,\
        0.631427, 0.634664, 0.636132, 0.635744, 0.633514, 0.629621, 0.624474, 0.618652, 0.612336, 0.604492,\
        0.594823, 0.584580, 0.572568, 0.559408, 0.546550, 0.535251, 0.526518, 0.521375, 0.520835, 0.526095,\
        0.539910, 0.565286, 0.601889, 0.648023, 0.695753, 0.730048, 0.747641, 0.758067, 0.769665, 0.785756,\
        0.802419, 0.813649, 0.823079, 0.829964, 0.835520, 0.840548, 0.843526, 0.844710, 0.844685, 0.843357,\
        0.840171, 0.834737, 0.827356, 0.818970, 0.810764, 0.803256, 0.797256, 0.791170, 0.783448, 0.775080,\
        0.766251, 0.757009, 0.747377, 0.737388, 0.727118, 0.716670, 0.705887, 0.694454, 0.682453, 0.669743,\
        0.656681, 0.643599, 0.630649, 0.617820, 0.605274, 0.595009, 0.585437, 0.575249, 0.564632, 0.553750,\
        0.542737, 0.531730, 0.520960, 0.510685, 0.500726, 0.492335, 0.482760, 0.472585, 0.462294, 0.451991,\
        0.441677, 0.431353, 0.421036, 0.410856, 0.401340, 0.392799, 0.384386, 0.375501, 0.366342, 0.357115,\
        0.347898],\
        [0.289376, 0.297757, 0.306706, 0.315416, 0.323830, 0.332268, 0.340883, 0.349737, 0.358927, 0.368723,\
        0.379637, 0.392081, 0.405508, 0.419345, 0.433550, 0.448177, 0.463105, 0.477991, 0.492467, 0.506599,\
        0.521603, 0.536711, 0.551067, 0.564545, 0.577046, 0.588536, 0.599163, 0.608775, 0.617148, 0.624168,\
        0.629636, 0.633350, 0.635205, 0.635216, 0.633524, 0.630408, 0.626258, 0.621422, 0.615911, 0.609328,\
        0.601719, 0.592851, 0.580271, 0.565801, 0.551704, 0.539640, 0.530927, 0.526808, 0.528587, 0.537786,\
        0.555999, 0.583872, 0.621986, 0.667621, 0.714560, 0.749646, 0.768339, 0.780265, 0.791240, 0.807873,\
        0.824299, 0.836510, 0.845339, 0.851572, 0.855723, 0.858294, 0.859315, 0.858932, 0.857326, 0.854373,\
        0.849749, 0.843287, 0.835244, 0.826306, 0.817338, 0.809123, 0.802481, 0.794916, 0.786404, 0.777350,\
        0.767845, 0.757924, 0.747582, 0.736824, 0.725718, 0.714563, 0.704037, 0.691980, 0.678647, 0.665192,\
        0.651840, 0.638821, 0.626267, 0.614132, 0.602160, 0.591057, 0.580591, 0.569922, 0.559020, 0.547998,\
        0.536956, 0.525952, 0.515116, 0.505301, 0.496266, 0.486802, 0.477273, 0.467303, 0.457109, 0.446855,\
        0.436622, 0.426456, 0.416356, 0.406218, 0.396625, 0.388121, 0.379712, 0.371078, 0.362214, 0.353226,\
        0.344191],\
        [0.283939, 0.292456, 0.301154, 0.309070, 0.316771, 0.324734, 0.333052, 0.341777, 0.351019, 0.360992,\
        0.371969, 0.384052, 0.396927, 0.410052, 0.423350, 0.437381, 0.452016, 0.466831, 0.481477, 0.496091,\
        0.512164, 0.527590, 0.542001, 0.555773, 0.568920, 0.581368, 0.593029, 0.603630, 0.612866, 0.620767,\
        0.626957, 0.631180, 0.633391, 0.633694, 0.632317, 0.629606, 0.625959, 0.621699, 0.616963, 0.611852,\
        0.606405, 0.598118, 0.584142, 0.568250, 0.553179, 0.540688, 0.532239, 0.529245, 0.533198, 0.545715,\
        0.568242, 0.599800, 0.638059, 0.684615, 0.736036, 0.774077, 0.787424, 0.802150, 0.814793, 0.830926,\
        0.846473, 0.858404, 0.866673, 0.871987, 0.874489, 0.874741, 0.873623, 0.871476, 0.868378, 0.864062,\
        0.858236, 0.850791, 0.841962, 0.832318, 0.822599, 0.813467, 0.804980, 0.796534, 0.787603, 0.778063,\
        0.768073, 0.757652, 0.746760, 0.735358, 0.723417, 0.710884, 0.698481, 0.686720, 0.673280, 0.659476,\
        0.645979, 0.633168, 0.621263, 0.610262, 0.600020, 0.587938, 0.576146, 0.564712, 0.553430, 0.542290,\
        0.531366, 0.520673, 0.510024, 0.499436, 0.490918, 0.481638, 0.471951, 0.462009, 0.451909, 0.441770,\
        0.431701, 0.421810, 0.412239, 0.402994, 0.393430, 0.384330, 0.375613, 0.366941, 0.358188, 0.349354,\
        0.340470],\
        [0.278478, 0.286753, 0.294889, 0.302281, 0.309612, 0.317333, 0.325494, 0.334146, 0.343373, 0.353300,\
        0.364040, 0.375588, 0.387746, 0.400039, 0.412538, 0.426279, 0.440859, 0.455871, 0.471180, 0.487110,\
        0.503424, 0.517871, 0.531958, 0.545983, 0.559748, 0.573040, 0.585626, 0.597148, 0.607515, 0.616580,\
        0.623558, 0.628274, 0.630777, 0.631228, 0.629908, 0.627213, 0.623607, 0.619463, 0.614881, 0.609845,\
        0.604521, 0.596466, 0.582078, 0.565869, 0.550637, 0.538279, 0.530420, 0.528611, 0.534400, 0.549290,\
        0.574445, 0.610041, 0.653229, 0.702083, 0.757546, 0.802108, 0.810855, 0.824634, 0.841440, 0.857061,\
        0.870362, 0.880359, 0.886987, 0.890938, 0.891522, 0.889535, 0.886358, 0.882554, 0.878205, 0.872735,\
        0.865835, 0.857366, 0.847543, 0.836911, 0.826144, 0.815837, 0.806307, 0.797049, 0.787293, 0.777308,\
        0.766997, 0.756256, 0.744991, 0.733142, 0.720742, 0.707884, 0.694390, 0.680776, 0.666745, 0.652603,\
        0.638972, 0.626280, 0.614874, 0.604444, 0.593887, 0.582666, 0.570821, 0.559101, 0.547612, 0.536420,\
        0.525696, 0.515621, 0.506116, 0.496381, 0.486296, 0.476543, 0.466700, 0.456746, 0.446727, 0.436718,\
        0.426815, 0.417120, 0.407762, 0.398932, 0.389833, 0.380661, 0.371767, 0.363017, 0.354290, 0.345541,\
        0.336769],\
        [0.272839, 0.280650, 0.288193, 0.295335, 0.302551, 0.310154, 0.318234, 0.326819, 0.335953, 0.345691,\
        0.356052, 0.366967, 0.378246, 0.389662, 0.401765, 0.415368, 0.429895, 0.444978, 0.460487, 0.476414,\
        0.492003, 0.506308, 0.520619, 0.535129, 0.549548, 0.563661, 0.577224, 0.589944, 0.601743, 0.611879,\
        0.619537, 0.624713, 0.627471, 0.627973, 0.626498, 0.623468, 0.619448, 0.614985, 0.610095, 0.603887,\
        0.596776, 0.587578, 0.574297, 0.559008, 0.544442, 0.532772, 0.525853, 0.525393, 0.533015, 0.550223,\
        0.578261, 0.618602, 0.669882, 0.725028, 0.776252, 0.816494, 0.838972, 0.854473, 0.870360, 0.884712,\
        0.895327, 0.901783, 0.906216, 0.907315, 0.906373, 0.902901, 0.897417, 0.892665, 0.887191, 0.880661,\
        0.872733, 0.863088, 0.851981, 0.840039, 0.827949, 0.816228, 0.805263, 0.795230, 0.785297, 0.775157,\
        0.764695, 0.753794, 0.742318, 0.730126, 0.717226, 0.703952, 0.689709, 0.674609, 0.659514, 0.644802,\
        0.630976, 0.618275, 0.606741, 0.597262, 0.586873, 0.575843, 0.564413, 0.552871, 0.541426, 0.530216,\
        0.519514, 0.509813, 0.500786, 0.491344, 0.481319, 0.471317, 0.461384, 0.451467, 0.441561, 0.431702,\
        0.421966, 0.412443, 0.403139, 0.394249, 0.385577, 0.376742, 0.367926, 0.359181, 0.350477, 0.341786,\
        0.333099],\
        [0.267220, 0.274649, 0.281860, 0.288878, 0.295955, 0.303399, 0.311399, 0.319875, 0.328854, 0.338347,\
        0.348314, 0.358679, 0.369353, 0.380375, 0.392163, 0.405174, 0.419377, 0.434093, 0.449094, 0.464285,\
        0.479216, 0.493671, 0.508449, 0.523454, 0.538462, 0.553335, 0.567911, 0.581997, 0.595229, 0.606312,\
        0.614733, 0.620484, 0.623592, 0.624186, 0.622492, 0.618892, 0.614024, 0.608795, 0.603615, 0.596823,\
        0.587771, 0.576989, 0.563806, 0.549337, 0.535674, 0.524975, 0.519256, 0.520409, 0.530247, 0.550585,\
        0.583511, 0.630455, 0.687038, 0.749647, 0.804927, 0.843395, 0.867576, 0.884551, 0.899332, 0.911472,\
        0.919343, 0.923278, 0.924604, 0.923553, 0.920481, 0.915278, 0.908706, 0.902641, 0.895509, 0.888119,\
        0.879092, 0.867959, 0.855176, 0.841632, 0.828155, 0.815193, 0.802828, 0.792142, 0.782084, 0.771824,\
        0.761257, 0.750292, 0.738767, 0.726395, 0.712775, 0.698188, 0.683343, 0.667561, 0.651624, 0.636273,\
        0.622274, 0.610069, 0.598654, 0.588613, 0.578801, 0.568297, 0.557342, 0.546180, 0.534993, 0.523866,\
        0.512887, 0.502871, 0.494263, 0.485104, 0.475491, 0.465690, 0.455883, 0.446121, 0.436413, 0.426757,\
        0.417201, 0.407911, 0.398957, 0.390128, 0.381471, 0.372789, 0.364076, 0.355378, 0.346711, 0.338072,\
        0.329458],\
        [0.261816, 0.268996, 0.276024, 0.282961, 0.289976, 0.297314, 0.305134, 0.313417, 0.322210, 0.331461,\
        0.341079, 0.351017, 0.361291, 0.372029, 0.383502, 0.396008, 0.409561, 0.423385, 0.437432, 0.451870,\
        0.466488, 0.481163, 0.496090, 0.511290, 0.526660, 0.542086, 0.557426, 0.572496, 0.586764, 0.599216,\
        0.608878, 0.615521, 0.619246, 0.620177, 0.618457, 0.614361, 0.608428, 0.601937, 0.595950, 0.588404,\
        0.578388, 0.566378, 0.552777, 0.538622, 0.525642, 0.515874, 0.511422, 0.514377, 0.526815, 0.550893,\
        0.589287, 0.644822, 0.707252, 0.771414, 0.837455, 0.878426, 0.897516, 0.913735, 0.926537, 0.935972,\
        0.941376, 0.943133, 0.942092, 0.938855, 0.933804, 0.927249, 0.919825, 0.912075, 0.903964, 0.895387,\
        0.884902, 0.871722, 0.856796, 0.841392, 0.826604, 0.813130, 0.800714, 0.789043, 0.778151, 0.767472,\
        0.756696, 0.745668, 0.734226, 0.722091, 0.708640, 0.693223, 0.676852, 0.660009, 0.643315, 0.627386,\
        0.612832, 0.601703, 0.591070, 0.580726, 0.570750, 0.560543, 0.550000, 0.539260, 0.528502, 0.517820,\
        0.507116, 0.496608, 0.487637, 0.478647, 0.469296, 0.459752, 0.450182, 0.440685, 0.431296, 0.421968,\
        0.412614, 0.403441, 0.394758, 0.386184, 0.377579, 0.368934, 0.360264, 0.351601, 0.342971, 0.334383,\
        0.325838],\
        [0.256728, 0.263761, 0.270698, 0.277602, 0.284598, 0.291842, 0.299448, 0.307502, 0.316146, 0.325183,\
        0.334476, 0.344033, 0.353943, 0.364361, 0.375492, 0.387516, 0.400261, 0.412781, 0.425706, 0.439580,\
        0.454040, 0.468755, 0.483685, 0.498863, 0.514339, 0.530015, 0.545734, 0.561491, 0.576768, 0.590578,\
        0.601854, 0.609735, 0.614508, 0.616208, 0.614911, 0.610865, 0.604526, 0.597071, 0.589703, 0.580768,\
        0.569630, 0.556602, 0.542397, 0.528157, 0.515497, 0.506388, 0.503013, 0.507674, 0.522748, 0.550749,\
        0.594627, 0.657096, 0.725648, 0.789533, 0.863690, 0.912510, 0.926712, 0.940479, 0.951102, 0.957979,\
        0.961000, 0.960571, 0.957403, 0.952167, 0.945389, 0.937527, 0.929058, 0.920354, 0.911511, 0.901921,\
        0.889456, 0.873525, 0.856203, 0.838827, 0.822556, 0.808761, 0.796714, 0.784548, 0.773081, 0.761986,\
        0.750920, 0.739732, 0.728292, 0.716312, 0.703672, 0.687626, 0.669824, 0.651959, 0.634775, 0.618963,\
        0.604624, 0.592599, 0.582792, 0.572833, 0.562920, 0.552889, 0.542632, 0.532227, 0.521885, 0.511819,\
        0.501959, 0.491702, 0.481819, 0.472409, 0.463045, 0.453655, 0.444313, 0.435121, 0.426160, 0.417423,\
        0.408607, 0.399456, 0.391027, 0.382491, 0.373844, 0.365163, 0.356487, 0.347841, 0.339244, 0.330708,\
        0.322236],\
        [0.251987, 0.258946, 0.265853, 0.272764, 0.279766, 0.286947, 0.294374, 0.302250, 0.310833, 0.319621,\
        0.328529, 0.337662, 0.347157, 0.357145, 0.367737, 0.378953, 0.390514, 0.401864, 0.414029, 0.427755,\
        0.442139, 0.456707, 0.471451, 0.486456, 0.501839, 0.517353, 0.532959, 0.549287, 0.565599, 0.580673,\
        0.593421, 0.603055, 0.609432, 0.612383, 0.612006, 0.608614, 0.602741, 0.594927, 0.585769, 0.575082,\
        0.562544, 0.548428, 0.533488, 0.518881, 0.506175, 0.497312, 0.494552, 0.500397, 0.517481, 0.548464,\
        0.596002, 0.660564, 0.736684, 0.810949, 0.874200, 0.920331, 0.945738, 0.961348, 0.971307, 0.976441,\
        0.977375, 0.974845, 0.969652, 0.962540, 0.954126, 0.944890, 0.935200, 0.925293, 0.915292, 0.904591,\
        0.890002, 0.871820, 0.852677, 0.833829, 0.816031, 0.800668, 0.789234, 0.777819, 0.766507, 0.755269,\
        0.743888, 0.732344, 0.720793, 0.708450, 0.694644, 0.679099, 0.661308, 0.643117, 0.625792, 0.610362,\
        0.597752, 0.585719, 0.575062, 0.565115, 0.555320, 0.545451, 0.535386, 0.525148, 0.514956, 0.505156,\
        0.495583, 0.485682, 0.475804, 0.466194, 0.456767, 0.447455, 0.438287, 0.429346, 0.420772, 0.412759,\
        0.405307, 0.397000, 0.387876, 0.378954, 0.370141, 0.361393, 0.352703, 0.344074, 0.335515, 0.327038,\
        0.318646],\
        [0.247587, 0.254522, 0.261439, 0.268393, 0.275450, 0.282685, 0.290178, 0.298089, 0.306412, 0.314765,\
        0.323152, 0.331790, 0.340821, 0.350310, 0.360266, 0.370606, 0.381100, 0.391647, 0.403358, 0.417158,\
        0.431186, 0.445249, 0.459552, 0.474211, 0.489241, 0.504085, 0.519225, 0.536459, 0.553871, 0.570043,\
        0.584203, 0.595738, 0.604061, 0.608592, 0.609525, 0.607078, 0.601626, 0.593643, 0.583487, 0.571380,\
        0.557525, 0.542360, 0.526658, 0.511541, 0.498503, 0.489431, 0.486623, 0.492753, 0.510675, 0.542889,\
        0.590949, 0.656121, 0.736116, 0.815736, 0.875174, 0.921870, 0.955620, 0.975785, 0.986504, 0.990586,\
        0.989750, 0.985245, 0.978099, 0.969170, 0.959133, 0.948453, 0.937343, 0.925725, 0.913379, 0.900198,\
        0.884429, 0.865536, 0.845821, 0.826691, 0.809112, 0.793450, 0.780966, 0.769739, 0.758706, 0.747528,\
        0.735856, 0.723565, 0.711561, 0.700114, 0.685383, 0.669248, 0.651790, 0.633758, 0.616424, 0.601625,\
        0.589041, 0.577984, 0.567552, 0.557649, 0.547975, 0.538280, 0.528393, 0.518233, 0.507927, 0.498044,\
        0.488634, 0.479054, 0.469382, 0.459808, 0.450397, 0.441161, 0.432122, 0.423339, 0.414914, 0.407074,\
        0.400211, 0.392772, 0.383977, 0.375057, 0.366238, 0.357510, 0.348857, 0.340276, 0.331777, 0.323373,\
        0.315075],\
        [0.243499, 0.250438, 0.257388, 0.264402, 0.271542, 0.278886, 0.286520, 0.294496, 0.302590, 0.310352,\
        0.318118, 0.326275, 0.334881, 0.343907, 0.353314, 0.363044, 0.373063, 0.383591, 0.395375, 0.408355,\
        0.421258, 0.434371, 0.448030, 0.462143, 0.476423, 0.490408, 0.505736, 0.524385, 0.542331, 0.559097,\
        0.574428, 0.587686, 0.597990, 0.604497, 0.607175, 0.605988, 0.601158, 0.593269, 0.582508, 0.569416,\
        0.554585, 0.538631, 0.522345, 0.506782, 0.493314, 0.483693, 0.480147, 0.485460, 0.502881, 0.535395,\
        0.583682, 0.645683, 0.722980, 0.808160, 0.877915, 0.929212, 0.965196, 0.986665, 0.997270, 1.000179,\
        0.997630, 0.991221, 0.982174, 0.971459, 0.959815, 0.947688, 0.935187, 0.921950, 0.906973, 0.890889,\
        0.873849, 0.855189, 0.835822, 0.816956, 0.800480, 0.785583, 0.772447, 0.760891, 0.749985, 0.739051,\
        0.727448, 0.714467, 0.700443, 0.688705, 0.674744, 0.658913, 0.642109, 0.624876, 0.607608, 0.593134,\
        0.581302, 0.570372, 0.560108, 0.550327, 0.540813, 0.531337, 0.521718, 0.511769, 0.501476, 0.491477,\
        0.481890, 0.472333, 0.462759, 0.453262, 0.443923, 0.434787, 0.425884, 0.417242, 0.408901, 0.400967,\
        0.393912, 0.386889, 0.378968, 0.370556, 0.362010, 0.353454, 0.344926, 0.336445, 0.328035, 0.319724,\
        0.311537],\
        [0.239683, 0.246639, 0.253626, 0.260689, 0.267885, 0.275275, 0.282912, 0.290791, 0.298684, 0.305818,\
        0.313131, 0.321012, 0.329329, 0.337993, 0.346981, 0.356313, 0.366087, 0.376569, 0.388087, 0.400230,\
        0.411615, 0.423806, 0.436959, 0.450706, 0.464864, 0.479671, 0.496420, 0.514253, 0.530976, 0.547674,\
        0.563942, 0.578732, 0.591009, 0.599915, 0.604626, 0.605046, 0.601528, 0.593739, 0.582484, 0.568844,\
        0.553546, 0.537280, 0.520834, 0.505172, 0.491498, 0.481331, 0.476638, 0.480079, 0.495264, 0.526534,\
        0.576171, 0.635164, 0.711507, 0.812714, 0.891317, 0.942599, 0.975594, 0.994961, 1.003886, 1.005080,\
        1.000705, 0.992425, 0.981530, 0.969058, 0.955848, 0.942366, 0.928523, 0.914437, 0.897908, 0.879005,\
        0.860331, 0.841967, 0.823598, 0.805388, 0.789118, 0.775180, 0.762435, 0.750891, 0.740179, 0.729674,\
        0.718698, 0.706534, 0.692163, 0.677875, 0.663420, 0.648114, 0.632328, 0.616809, 0.602106, 0.587190,\
        0.574384, 0.563061, 0.552701, 0.542999, 0.533676, 0.524438, 0.515141, 0.505680, 0.495522, 0.485299,\
        0.475374, 0.465649, 0.456055, 0.446611, 0.437365, 0.428361, 0.419634, 0.411205, 0.403096, 0.395382,\
        0.388181, 0.381102, 0.373616, 0.365704, 0.357544, 0.349267, 0.340943, 0.332615, 0.324324, 0.316123,\
        0.308061],\
        [0.236102, 0.243075, 0.250086, 0.257171, 0.264368, 0.271706, 0.279178, 0.286690, 0.293964, 0.300839,\
        0.308192, 0.316087, 0.324228, 0.332588, 0.341216, 0.350185, 0.359610, 0.369632, 0.380256, 0.391014,\
        0.401635, 0.413540, 0.426470, 0.439976, 0.454187, 0.469608, 0.486645, 0.503032, 0.518333, 0.535334,\
        0.552650, 0.568901, 0.583135, 0.594350, 0.601105, 0.603453, 0.601092, 0.593783, 0.582695, 0.569183,\
        0.554103, 0.538195, 0.522237, 0.507125, 0.493898, 0.483782, 0.478304, 0.479580, 0.490942, 0.517674,\
        0.566131, 0.633882, 0.715504, 0.822927, 0.904007, 0.952999, 0.982553, 0.999095, 1.005759, 1.004975,\
        0.998754, 0.988706, 0.976084, 0.961902, 0.947139, 0.932630, 0.917652, 0.900961, 0.883699, 0.862826,\
        0.843426, 0.826230, 0.809787, 0.793081, 0.777417, 0.763524, 0.750919, 0.739397, 0.728894, 0.718742,\
        0.708089, 0.696129, 0.681814, 0.666651, 0.651615, 0.636743, 0.621984, 0.607125, 0.593960, 0.580179,\
        0.567094, 0.555459, 0.545001, 0.535402, 0.526354, 0.517397, 0.508099, 0.499003, 0.489095, 0.478816,\
        0.468718, 0.458871, 0.449255, 0.439870, 0.430737, 0.421893, 0.413381, 0.405223, 0.397387, 0.389885,\
        0.382701, 0.375617, 0.368355, 0.360807, 0.353015, 0.345055, 0.336988, 0.328858, 0.320712, 0.312625,\
        0.304697],\
        [0.232722, 0.239707, 0.246727, 0.253808, 0.260967, 0.268203, 0.275475, 0.282677, 0.289662, 0.296521,\
        0.303960, 0.311773, 0.319658, 0.327679, 0.335933, 0.344495, 0.353442, 0.362818, 0.372552, 0.382464,\
        0.392748, 0.404280, 0.416720, 0.429643, 0.443312, 0.458112, 0.473725, 0.488837, 0.504296, 0.522529,\
        0.540704, 0.558034, 0.573922, 0.587359, 0.596969, 0.601673, 0.599658, 0.592789, 0.582542, 0.569952,\
        0.555884, 0.541112, 0.526437, 0.512744, 0.501002, 0.492234, 0.487533, 0.488269, 0.496828, 0.518779,\
        0.565511, 0.652764, 0.743953, 0.824529, 0.903527, 0.953833, 0.982732, 0.997507, 1.002212, 0.999580,\
        0.991679, 0.980117, 0.966077, 0.950349, 0.933826, 0.918187, 0.904789, 0.885943, 0.864249, 0.842089,\
        0.822714, 0.808195, 0.793555, 0.778696, 0.764124, 0.750666, 0.738164, 0.726340, 0.715855, 0.706027,\
        0.694856, 0.682514, 0.668542, 0.653652, 0.638645, 0.624288, 0.611173, 0.597582, 0.583826, 0.570869,\
        0.558398, 0.546962, 0.536621, 0.527195, 0.518529, 0.510313, 0.501056, 0.491481, 0.481784, 0.471709,\
        0.461668, 0.451847, 0.442297, 0.433028, 0.424052, 0.415397, 0.407114, 0.399237, 0.391691, 0.384413,\
        0.377351, 0.370365, 0.363291, 0.356030, 0.348566, 0.340929, 0.333152, 0.325266, 0.317300, 0.309326,\
        0.301527],\
        [0.229519, 0.236508, 0.243527, 0.250590, 0.257701, 0.264849, 0.271997, 0.279089, 0.286116, 0.293231,\
        0.300636, 0.308105, 0.315569, 0.323196, 0.331057, 0.339185, 0.347619, 0.356383, 0.365467, 0.374910,\
        0.384965, 0.395944, 0.407586, 0.419431, 0.432009, 0.445878, 0.460664, 0.475878, 0.492316, 0.510335,\
        0.528152, 0.545854, 0.562927, 0.578382, 0.591083, 0.597860, 0.596080, 0.590331, 0.581615, 0.570748,\
        0.558494, 0.545631, 0.533018, 0.521630, 0.512531, 0.506784, 0.505374, 0.509307, 0.520183, 0.542810,\
        0.599530, 0.702062, 0.780215, 0.844577, 0.905572, 0.950451, 0.977379, 0.990338, 0.993199, 0.988899,\
        0.979567, 0.966841, 0.951987, 0.935424, 0.917136, 0.899491, 0.884162, 0.865813, 0.843150, 0.819867,\
        0.800242, 0.788087, 0.775707, 0.762496, 0.749210, 0.736577, 0.724752, 0.712369, 0.700966, 0.691363,\
        0.679871, 0.667203, 0.653582, 0.639319, 0.624637, 0.610202, 0.598152, 0.586141, 0.573066, 0.560391,\
        0.548428, 0.537424, 0.527409, 0.518170, 0.509616, 0.502074, 0.493933, 0.483937, 0.473979, 0.464028,\
        0.454163, 0.444512, 0.435146, 0.426090, 0.417341, 0.408918, 0.400895, 0.393308, 0.386047, 0.379012,\
        0.372132, 0.365309, 0.358436, 0.351436, 0.344276, 0.336960, 0.329503, 0.321921, 0.314221, 0.306405,\
        0.298706],\
        [0.226472, 0.233462, 0.240475, 0.247519, 0.254594, 0.261688, 0.268781, 0.275862, 0.282966, 0.290187,\
        0.297562, 0.304671, 0.311732, 0.319029, 0.326534, 0.334241, 0.342182, 0.350396, 0.358935, 0.367897,\
        0.377460, 0.387749, 0.398571, 0.408978, 0.420385, 0.433726, 0.448315, 0.463775, 0.480299, 0.497447,\
        0.514408, 0.532157, 0.549811, 0.566150, 0.579528, 0.587429, 0.588862, 0.585762, 0.579598, 0.571276,\
        0.561560, 0.551250, 0.541288, 0.532818, 0.527173, 0.525740, 0.529776, 0.540502, 0.559889, 0.591991,\
        0.656047, 0.749826, 0.815090, 0.864003, 0.909793, 0.946258, 0.968667, 0.978530, 0.979123, 0.973228,\
        0.962726, 0.949056, 0.933993, 0.918174, 0.899780, 0.880763, 0.862904, 0.843777, 0.822768, 0.800010,\
        0.782239, 0.769264, 0.757777, 0.745775, 0.733286, 0.721120, 0.710877, 0.699822, 0.686775, 0.675504,\
        0.663645, 0.650860, 0.637609, 0.624339, 0.610778, 0.596615, 0.584351, 0.572920, 0.560965, 0.548954,\
        0.537470, 0.526978, 0.517504, 0.508561, 0.499961, 0.492116, 0.484189, 0.475072, 0.465472, 0.455811,\
        0.446242, 0.436876, 0.427811, 0.419083, 0.410660, 0.402550, 0.394839, 0.387540, 0.380537, 0.373738,\
        0.367067, 0.360442, 0.353788, 0.347042, 0.340172, 0.333171, 0.326048, 0.318822, 0.311513, 0.304131,\
        0.296631],\
        [0.223568, 0.230557, 0.237564, 0.244594, 0.251645, 0.258710, 0.265779, 0.272853, 0.279948, 0.287076,\
        0.294154, 0.301029, 0.308002, 0.315137, 0.322347, 0.329656, 0.337126, 0.344822, 0.352817, 0.361196,\
        0.370043, 0.379347, 0.388815, 0.398035, 0.408950, 0.422289, 0.436735, 0.451909, 0.467661, 0.483639,\
        0.500013, 0.517529, 0.535153, 0.551657, 0.565420, 0.574806, 0.579219, 0.579433, 0.576536, 0.571406,\
        0.564787, 0.557461, 0.550420, 0.544981, 0.542821, 0.545838, 0.555713, 0.573383, 0.600344, 0.637933,\
        0.691451, 0.768135, 0.822807, 0.869881, 0.910706, 0.940564, 0.957076, 0.962459, 0.960274, 0.952955,\
        0.941792, 0.927305, 0.911382, 0.897516, 0.879921, 0.861277, 0.842187, 0.821961, 0.803463, 0.784077,\
        0.766337, 0.752635, 0.740896, 0.729337, 0.717143, 0.704636, 0.694176, 0.684500, 0.671946, 0.659522,\
        0.647020, 0.634005, 0.620856, 0.608620, 0.596566, 0.583625, 0.571346, 0.559899, 0.548527, 0.537078,\
        0.525886, 0.515796, 0.507115, 0.498653, 0.490194, 0.482118, 0.474021, 0.465399, 0.456355, 0.447166,\
        0.438011, 0.429013, 0.420320, 0.412042, 0.404063, 0.396333, 0.388967, 0.381942, 0.375188, 0.368622,\
        0.362171, 0.355764, 0.349338, 0.342844, 0.336251, 0.329549, 0.322745, 0.315858, 0.308911, 0.301912,\
        0.294687],\
        [0.220796, 0.227783, 0.234785, 0.241807, 0.248844, 0.255891, 0.262943, 0.269993, 0.277035, 0.284035,\
        0.290922, 0.297686, 0.304631, 0.311605, 0.318506, 0.325414, 0.332426, 0.339621, 0.347070, 0.354832,\
        0.362933, 0.371323, 0.379876, 0.388811, 0.399521, 0.412345, 0.426238, 0.440727, 0.455617, 0.470819,\
        0.486540, 0.503054, 0.520110, 0.536544, 0.550809, 0.561670, 0.568651, 0.572099, 0.572707, 0.571137,\
        0.567972, 0.563857, 0.559705, 0.556894, 0.557356, 0.563497, 0.577624, 0.599892, 0.628922, 0.667931,\
        0.716015, 0.769171, 0.818856, 0.865574, 0.906010, 0.932357, 0.941518, 0.941570, 0.936655, 0.928351,\
        0.917423, 0.903360, 0.886637, 0.871848, 0.855948, 0.839571, 0.822759, 0.801047, 0.782457, 0.765186,\
        0.749294, 0.735967, 0.724212, 0.712921, 0.700957, 0.688380, 0.677005, 0.666361, 0.654865, 0.642817,\
        0.630326, 0.617371, 0.604051, 0.592180, 0.581102, 0.569511, 0.558084, 0.547109, 0.536340, 0.525503,\
        0.514435, 0.504092, 0.496245, 0.488230, 0.480074, 0.471992, 0.463877, 0.455539, 0.446978, 0.438317,\
        0.429660, 0.421075, 0.412716, 0.404951, 0.397536, 0.390226, 0.383213, 0.376488, 0.369996, 0.363672,\
        0.357450, 0.351270, 0.345077, 0.338830, 0.332499, 0.326070, 0.319544, 0.312933, 0.306250, 0.299423,\
        0.292433],\
        [0.218146, 0.225131, 0.232131, 0.239146, 0.246175, 0.253210, 0.260244, 0.267267, 0.274266, 0.281213,\
        0.288089, 0.294924, 0.301759, 0.308450, 0.314973, 0.321466, 0.328040, 0.334768, 0.341710, 0.348914,\
        0.356411, 0.364222, 0.372438, 0.381442, 0.391893, 0.403911, 0.416938, 0.430532, 0.444558, 0.459030,\
        0.474027, 0.489594, 0.505809, 0.521976, 0.536678, 0.548772, 0.557863, 0.564221, 0.568289, 0.570442,\
        0.570963, 0.570190, 0.568771, 0.567953, 0.569715, 0.576690, 0.591787, 0.615476, 0.644830, 0.679557,\
        0.719940, 0.763124, 0.806014, 0.854367, 0.900762, 0.919475, 0.919070, 0.915147, 0.908714, 0.899908,\
        0.889525, 0.876033, 0.860395, 0.845181, 0.829506, 0.814142, 0.802594, 0.783204, 0.762207, 0.744686,\
        0.730205, 0.718091, 0.706594, 0.695154, 0.683313, 0.670923, 0.659055, 0.647802, 0.636832, 0.625761,\
        0.613669, 0.601513, 0.588703, 0.576825, 0.565860, 0.555122, 0.544619, 0.534434, 0.524493, 0.514657,\
        0.504583, 0.494077, 0.485672, 0.477829, 0.469884, 0.461890, 0.453855, 0.445754, 0.437610, 0.429487,\
        0.421415, 0.413350, 0.405268, 0.397749, 0.390947, 0.384174, 0.377568, 0.371171, 0.364959, 0.358887,\
        0.352903, 0.346955, 0.340996, 0.334988, 0.328903, 0.322723, 0.316435, 0.310030, 0.303542, 0.296956,\
        0.290145],\
        [0.215613, 0.222595, 0.229592, 0.236603, 0.243624, 0.250647, 0.257663, 0.264662, 0.271630, 0.278558,\
        0.285445, 0.292305, 0.299080, 0.305472, 0.311603, 0.317722, 0.323917, 0.330239, 0.336742, 0.343486,\
        0.350536, 0.357982, 0.366003, 0.374908, 0.385024, 0.396375, 0.408556, 0.421197, 0.434357, 0.448152,\
        0.462568, 0.477544, 0.492976, 0.508561, 0.523538, 0.536355, 0.546945, 0.555798, 0.563163, 0.569102,\
        0.573527, 0.576299, 0.577583, 0.578338, 0.580490, 0.586882, 0.600656, 0.624663, 0.653457, 0.684531,\
        0.717634, 0.751725, 0.786205, 0.825152, 0.870626, 0.889663, 0.888878, 0.884744, 0.878061, 0.869549,\
        0.859551, 0.846666, 0.831498, 0.817322, 0.803241, 0.787043, 0.774446, 0.757802, 0.738967, 0.722219,\
        0.709150, 0.698481, 0.687575, 0.676030, 0.664275, 0.652320, 0.640440, 0.628892, 0.618286, 0.608829,\
        0.597660, 0.585505, 0.573642, 0.562086, 0.551219, 0.540971, 0.531265, 0.521905, 0.512754, 0.503861,\
        0.494778, 0.485133, 0.476195, 0.467998, 0.460009, 0.452045, 0.444091, 0.436188, 0.428406, 0.420810,\
        0.413395, 0.406054, 0.398578, 0.391332, 0.384699, 0.378314, 0.372083, 0.366010, 0.360083, 0.354269,\
        0.348528, 0.342814, 0.337087, 0.331309, 0.325454, 0.319507, 0.313448, 0.307235, 0.300823, 0.294431,\
        0.287860],\
        [0.213191, 0.220168, 0.227162, 0.234170, 0.241183, 0.248191, 0.255185, 0.262152, 0.269082, 0.275967,\
        0.282800, 0.289559, 0.296147, 0.302322, 0.308225, 0.314099, 0.320010, 0.326001, 0.332139, 0.338506,\
        0.345198, 0.352346, 0.360130, 0.368772, 0.378450, 0.389164, 0.400635, 0.412316, 0.424711, 0.437970,\
        0.451956, 0.466523, 0.481508, 0.496719, 0.511662, 0.524076, 0.535482, 0.546442, 0.556867, 0.566562,\
        0.575136, 0.581862, 0.586097, 0.588475, 0.590922, 0.596876, 0.610904, 0.632688, 0.659347, 0.688184,\
        0.715528, 0.740475, 0.763906, 0.789523, 0.825797, 0.849126, 0.855768, 0.852910, 0.845916, 0.837956,\
        0.829496, 0.817989, 0.801942, 0.788293, 0.775004, 0.760469, 0.746097, 0.730088, 0.713751, 0.699052,\
        0.687383, 0.677623, 0.667391, 0.656292, 0.644836, 0.633480, 0.622051, 0.610282, 0.599744, 0.590970,\
        0.581273, 0.570033, 0.558758, 0.547691, 0.537040, 0.527158, 0.518247, 0.509708, 0.501140, 0.492781,\
        0.484304, 0.475517, 0.466888, 0.458634, 0.450600, 0.442639, 0.434733, 0.426970, 0.419480, 0.412347,\
        0.405552, 0.398863, 0.392037, 0.385270, 0.378861, 0.372741, 0.366812, 0.361033, 0.355378, 0.349819,\
        0.344320, 0.338841, 0.333341, 0.327785, 0.322144, 0.316408, 0.310582, 0.304654, 0.298493, 0.292131,\
        0.285661],\
        [0.210879, 0.217849, 0.224840, 0.231842, 0.238845, 0.245833, 0.252795, 0.259718, 0.266589, 0.273393,\
        0.280106, 0.286679, 0.293013, 0.299024, 0.304835, 0.310598, 0.316308, 0.322023, 0.327846, 0.333884,\
        0.340254, 0.347095, 0.354567, 0.362824, 0.371960, 0.381933, 0.392507, 0.403482, 0.415408, 0.428295,\
        0.441899, 0.456049, 0.470655, 0.485606, 0.499376, 0.510740, 0.522890, 0.535663, 0.548703, 0.561810,\
        0.574629, 0.586005, 0.593736, 0.598058, 0.601312, 0.607431, 0.619895, 0.638586, 0.661664, 0.686626,\
        0.710296, 0.730855, 0.749060, 0.769215, 0.792889, 0.813200, 0.822971, 0.820932, 0.813367, 0.806303,\
        0.799738, 0.790417, 0.775403, 0.760528, 0.746545, 0.733035, 0.719835, 0.704090, 0.689880, 0.677634,\
        0.667139, 0.657693, 0.647842, 0.636947, 0.625593, 0.614989, 0.604771, 0.593447, 0.583046, 0.573975,\
        0.564714, 0.554671, 0.544338, 0.533959, 0.523547, 0.513717, 0.505708, 0.498037, 0.490031, 0.482066,\
        0.474039, 0.465858, 0.457681, 0.449642, 0.441716, 0.433818, 0.425945, 0.418263, 0.410998, 0.404231,\
        0.397938, 0.391773, 0.385528, 0.379303, 0.373255, 0.367419, 0.361761, 0.356247, 0.350847, 0.345533,\
        0.340273, 0.335027, 0.329755, 0.324415, 0.318970, 0.313405, 0.307744, 0.302018, 0.296094, 0.289886,\
        0.283526],\
        [0.208678, 0.215642, 0.222629, 0.229625, 0.236611, 0.243572, 0.250491, 0.257355, 0.264146, 0.270845,\
        0.277420, 0.283821, 0.289984, 0.295875, 0.301619, 0.307310, 0.312816, 0.318266, 0.323799, 0.329529,\
        0.335579, 0.342088, 0.349194, 0.357009, 0.365579, 0.374851, 0.384691, 0.395132, 0.406633, 0.419112,\
        0.432217, 0.445680, 0.459344, 0.472818, 0.485138, 0.496632, 0.509295, 0.523287, 0.537964, 0.553420,\
        0.569791, 0.586227, 0.599533, 0.605154, 0.609060, 0.614759, 0.624893, 0.639956, 0.659071, 0.680600,\
        0.702247, 0.721076, 0.735679, 0.750484, 0.766933, 0.782785, 0.791227, 0.789932, 0.783519, 0.777603,\
        0.771374, 0.762211, 0.748889, 0.734017, 0.719510, 0.706092, 0.695368, 0.682082, 0.669034, 0.657993,\
        0.648346, 0.639363, 0.629952, 0.619141, 0.607296, 0.597114, 0.587738, 0.577590, 0.567657, 0.558411,\
        0.549287, 0.539962, 0.530641, 0.521346, 0.511595, 0.501334, 0.493737, 0.486773, 0.479385, 0.471856,\
        0.464235, 0.456518, 0.448776, 0.441084, 0.433439, 0.425752, 0.417953, 0.410284, 0.403250, 0.396860,\
        0.390924, 0.385139, 0.379340, 0.373553, 0.367864, 0.362318, 0.356919, 0.351649, 0.346486, 0.341405,\
        0.336376, 0.331362, 0.326321, 0.321203, 0.315954, 0.310533, 0.304953, 0.299308, 0.293584, 0.287613,\
        0.281417],\
        [0.206598, 0.213554, 0.220538, 0.227525, 0.234490, 0.241412, 0.248276, 0.255065, 0.261763, 0.268348,\
        0.274793, 0.281064, 0.287132, 0.293007, 0.298748, 0.304255, 0.309479, 0.314666, 0.319932, 0.325365,\
        0.331086, 0.337238, 0.343956, 0.351334, 0.359409, 0.368155, 0.377536, 0.387619, 0.398569, 0.410520,\
        0.422922, 0.435394, 0.447839, 0.460006, 0.471618, 0.483110, 0.495602, 0.509552, 0.524212, 0.540110,\
        0.557993, 0.577093, 0.594164, 0.603562, 0.609046, 0.614780, 0.623107, 0.634831, 0.649983, 0.668220,\
        0.688676, 0.705701, 0.717285, 0.728498, 0.740504, 0.751555, 0.758178, 0.758837, 0.755632, 0.751162,\
        0.745201, 0.736173, 0.723413, 0.708769, 0.695004, 0.682287, 0.671625, 0.660684, 0.649632, 0.639599,\
        0.630634, 0.622275, 0.613715, 0.603567, 0.591808, 0.581285, 0.571804, 0.562331, 0.553002, 0.544005,\
        0.535079, 0.526119, 0.517637, 0.509742, 0.501418, 0.492008, 0.483700, 0.476505, 0.469416, 0.462240,\
        0.454977, 0.447645, 0.440299, 0.433010, 0.425810, 0.418605, 0.411113, 0.403346, 0.396585, 0.390515,\
        0.384752, 0.379151, 0.373615, 0.368129, 0.362728, 0.357441, 0.352277, 0.347230, 0.342285, 0.337420,\
        0.332611, 0.327825, 0.323021, 0.318144, 0.313128, 0.307893, 0.302392, 0.296855, 0.291250, 0.285419,\
        0.279354],\
        [0.204654, 0.211605, 0.218587, 0.225559, 0.232492, 0.239363, 0.246156, 0.252857, 0.259450, 0.265918,\
        0.272239, 0.278394, 0.284374, 0.290188, 0.295837, 0.301143, 0.306142, 0.311154, 0.316199, 0.321335,\
        0.326706, 0.332483, 0.338814, 0.345793, 0.353462, 0.361821, 0.370872, 0.380659, 0.391254, 0.402600,\
        0.414034, 0.425313, 0.436582, 0.447737, 0.458720, 0.469840, 0.481675, 0.494446, 0.507453, 0.521838,\
        0.538979, 0.557633, 0.575098, 0.588594, 0.598148, 0.605701, 0.613460, 0.622474, 0.633474, 0.646901,\
        0.662556, 0.677692, 0.689965, 0.700731, 0.710779, 0.719481, 0.725345, 0.727801, 0.727424, 0.725110,\
        0.720214, 0.711800, 0.699792, 0.685610, 0.672582, 0.660832, 0.650379, 0.640526, 0.630991, 0.621912,\
        0.613526, 0.605671, 0.597719, 0.588418, 0.577557, 0.567069, 0.557337, 0.548044, 0.539165, 0.530706,\
        0.522227, 0.513294, 0.505093, 0.498309, 0.490887, 0.482621, 0.474602, 0.467250, 0.460229, 0.453280,\
        0.446308, 0.439294, 0.432282, 0.425370, 0.418676, 0.412240, 0.405765, 0.398396, 0.391404, 0.385174,\
        0.379349, 0.373776, 0.368361, 0.363060, 0.357868, 0.352791, 0.347830, 0.342980, 0.338228, 0.333559,\
        0.328952, 0.324383, 0.319815, 0.315196, 0.310455, 0.305514, 0.300279, 0.294752, 0.289129, 0.283340,\
        0.277354],\
        [0.202877, 0.209829, 0.216805, 0.223749, 0.230631, 0.237433, 0.244140, 0.250738, 0.257216, 0.263557,\
        0.269749, 0.275778, 0.281634, 0.287302, 0.292733, 0.297840, 0.302780, 0.307749, 0.312593, 0.317399,\
        0.322380, 0.327763, 0.333720, 0.340346, 0.347679, 0.355722, 0.364473, 0.373941, 0.384115, 0.394805,\
        0.405210, 0.415384, 0.425697, 0.436030, 0.446300, 0.456631, 0.467223, 0.478049, 0.488902, 0.500856,\
        0.515918, 0.532548, 0.549435, 0.564994, 0.577972, 0.588311, 0.596715, 0.603975, 0.611216, 0.619204,\
        0.629390, 0.643035, 0.656514, 0.668350, 0.678570, 0.686959, 0.693178, 0.697188, 0.698888, 0.698558,\
        0.694891, 0.687131, 0.676172, 0.663576, 0.651449, 0.640504, 0.630667, 0.621567, 0.613220, 0.604825,\
        0.596852, 0.589314, 0.581549, 0.572874, 0.563235, 0.553393, 0.543810, 0.534616, 0.526045, 0.518363,\
        0.511015, 0.502427, 0.494014, 0.487322, 0.480466, 0.473101, 0.465686, 0.458550, 0.451677, 0.444935,\
        0.438224, 0.431486, 0.424729, 0.418074, 0.411728, 0.405867, 0.400445, 0.393946, 0.386875, 0.380406,\
        0.374455, 0.368854, 0.363493, 0.358308, 0.353267, 0.348356, 0.343566, 0.338886, 0.334303, 0.329804,\
        0.325375, 0.320998, 0.316647, 0.312274, 0.307803, 0.303112, 0.298099, 0.292684, 0.287074, 0.281316,\
        0.275389],\
        [0.201338, 0.208284, 0.215225, 0.222111, 0.228918, 0.235629, 0.242231, 0.248711, 0.255059, 0.261264,\
        0.267315, 0.273201, 0.278911, 0.284425, 0.289710, 0.294757, 0.299698, 0.304550, 0.309110, 0.313500,\
        0.318033, 0.323010, 0.328621, 0.334945, 0.341995, 0.349752, 0.358183, 0.367237, 0.376800, 0.386588,\
        0.396170, 0.405628, 0.415308, 0.424978, 0.434507, 0.443895, 0.453180, 0.462347, 0.471534, 0.481578,\
        0.493345, 0.506874, 0.522234, 0.538868, 0.554224, 0.566463, 0.575489, 0.582012, 0.586967, 0.591252,\
        0.597054, 0.609972, 0.623246, 0.635682, 0.646350, 0.655219, 0.662385, 0.667580, 0.670653, 0.671307,\
        0.668562, 0.661894, 0.652276, 0.641448, 0.630751, 0.620852, 0.611995, 0.603415, 0.595940, 0.588375,\
        0.580762, 0.573418, 0.565901, 0.557848, 0.549203, 0.540194, 0.531040, 0.521964, 0.513435, 0.506322,\
        0.500446, 0.492972, 0.484799, 0.477621, 0.470818, 0.463938, 0.457024, 0.450227, 0.443600, 0.437111,\
        0.430690, 0.424243, 0.417697, 0.411160, 0.404944, 0.399284, 0.394010, 0.388234, 0.381889, 0.375649,\
        0.369755, 0.364191, 0.358893, 0.353804, 0.348886, 0.344115, 0.339471, 0.334937, 0.330499, 0.326143,\
        0.321858, 0.317637, 0.313462, 0.309294, 0.305067, 0.300584, 0.295672, 0.290430, 0.284942, 0.279268,\
        0.273419],\
        [0.200135, 0.207001, 0.213853, 0.220645, 0.227350, 0.233949, 0.240428, 0.246775, 0.252980, 0.259034,\
        0.264930, 0.270661, 0.276217, 0.281591, 0.286781, 0.291816, 0.296744, 0.301450, 0.305636, 0.309528,\
        0.313564, 0.318158, 0.323484, 0.329570, 0.336389, 0.343889, 0.351994, 0.360604, 0.369562, 0.378635,\
        0.387612, 0.396572, 0.405705, 0.414785, 0.423619, 0.432150, 0.440394, 0.448402, 0.456412, 0.464942,\
        0.474494, 0.485370, 0.498503, 0.515199, 0.531751, 0.544610, 0.553758, 0.560019, 0.564350, 0.568054,\
        0.573685, 0.583330, 0.594470, 0.606496, 0.616509, 0.625629, 0.633578, 0.639479, 0.643377, 0.644789,\
        0.643058, 0.637453, 0.628796, 0.619624, 0.610615, 0.601717, 0.593967, 0.586331, 0.579108, 0.572152,\
        0.565120, 0.558082, 0.550905, 0.543465, 0.535733, 0.527665, 0.519153, 0.510332, 0.501840, 0.494788,\
        0.489119, 0.482656, 0.475478, 0.468505, 0.461829, 0.455256, 0.448719, 0.442253, 0.435903, 0.429698,\
        0.423627, 0.417582, 0.411324, 0.404816, 0.398718, 0.393197, 0.387917, 0.382424, 0.376619, 0.370766,\
        0.365085, 0.359650, 0.354458, 0.349478, 0.344681, 0.340038, 0.335527, 0.331127, 0.326816, 0.322578,\
        0.318402, 0.314290, 0.310246, 0.306214, 0.302113, 0.297857, 0.293108, 0.288035, 0.282706, 0.277161,\
        0.271419],\
        [0.199119, 0.205906, 0.212651, 0.219330, 0.225915, 0.232386, 0.238726, 0.244925, 0.250973, 0.256865,\
        0.262592, 0.268152, 0.273541, 0.278760, 0.283818, 0.288735, 0.293507, 0.297994, 0.301931, 0.305317,\
        0.308865, 0.313183, 0.318331, 0.324257, 0.330902, 0.338183, 0.345993, 0.354207, 0.362680, 0.371251,\
        0.379817, 0.388400, 0.397028, 0.405590, 0.413815, 0.421673, 0.429200, 0.436452, 0.443681, 0.451319,\
        0.459858, 0.469863, 0.482234, 0.497802, 0.513285, 0.525036, 0.533709, 0.540002, 0.544730, 0.549055,\
        0.554612, 0.562390, 0.571840, 0.581886, 0.590642, 0.599875, 0.607547, 0.613503, 0.617855, 0.619802,\
        0.619448, 0.615079, 0.606473, 0.598736, 0.591483, 0.583799, 0.576639, 0.569750, 0.563055, 0.556530,\
        0.549999, 0.543347, 0.536532, 0.529638, 0.522791, 0.515823, 0.508174, 0.499832, 0.491751, 0.484760,\
        0.478668, 0.472534, 0.466083, 0.459601, 0.453246, 0.446991, 0.440786, 0.434618, 0.428519, 0.422565,\
        0.416858, 0.411367, 0.405729, 0.399378, 0.393274, 0.387677, 0.382299, 0.376901, 0.371391, 0.365857,\
        0.360425, 0.355174, 0.350128, 0.345280, 0.340612, 0.336101, 0.331721, 0.327449, 0.323260, 0.319128,\
        0.315032, 0.310978, 0.307020, 0.303145, 0.299098, 0.294916, 0.290406, 0.285545, 0.280397, 0.275004,\
        0.269388],\
        [0.198185, 0.204909, 0.211563, 0.218131, 0.224592, 0.230924, 0.237116, 0.243156, 0.249036, 0.254752,\
        0.260299, 0.265672, 0.270873, 0.275900, 0.280758, 0.285442, 0.289913, 0.294065, 0.297737, 0.300710,\
        0.303946, 0.308216, 0.313281, 0.319102, 0.325626, 0.332733, 0.340292, 0.348177, 0.356266, 0.364455,\
        0.372678, 0.380921, 0.389164, 0.397300, 0.405025, 0.412464, 0.419588, 0.426366, 0.433120, 0.440315,\
        0.448470, 0.458180, 0.470055, 0.483969, 0.497217, 0.507638, 0.515779, 0.522329, 0.527812, 0.532815,\
        0.538344, 0.545158, 0.553140, 0.561552, 0.569738, 0.577800, 0.584627, 0.590357, 0.594538, 0.596593,\
        0.597391, 0.594623, 0.587151, 0.579965, 0.573449, 0.566820, 0.560270, 0.553878, 0.547646, 0.541584,\
        0.535561, 0.529322, 0.522792, 0.516257, 0.510099, 0.504338, 0.497532, 0.489816, 0.482215, 0.475311,\
        0.469042, 0.463004, 0.456958, 0.450923, 0.444961, 0.439070, 0.433206, 0.427327, 0.421429, 0.415609,\
        0.410122, 0.405109, 0.400209, 0.394343, 0.388211, 0.382472, 0.377005, 0.371645, 0.366309, 0.361009,\
        0.355805, 0.350751, 0.345874, 0.341179, 0.336654, 0.332282, 0.328039, 0.323902, 0.319843, 0.315828,\
        0.311817, 0.307777, 0.303785, 0.300084, 0.296224, 0.292104, 0.287727, 0.283043, 0.278067, 0.272826,\
        0.267340],\
        [0.197300, 0.203974, 0.210557, 0.217024, 0.223358, 0.229548, 0.235585, 0.241459, 0.247164, 0.252696,\
        0.258051, 0.263225, 0.268217, 0.273024, 0.277637, 0.282034, 0.286165, 0.289939, 0.293247, 0.296144,\
        0.299387, 0.303616, 0.308513, 0.314240, 0.320686, 0.327652, 0.334990, 0.342586, 0.350348, 0.358201,\
        0.366100, 0.374014, 0.381899, 0.389639, 0.397090, 0.404431, 0.411349, 0.417763, 0.424198, 0.431132,\
        0.439003, 0.448227, 0.459013, 0.470837, 0.482180, 0.491808, 0.499738, 0.506731, 0.513072, 0.518560,\
        0.524005, 0.530282, 0.537299, 0.544585, 0.551812, 0.558680, 0.564716, 0.569727, 0.573406, 0.575557,\
        0.576094, 0.573875, 0.568748, 0.562709, 0.556734, 0.550750, 0.544736, 0.538742, 0.532865, 0.527259,\
        0.521876, 0.516163, 0.509826, 0.503491, 0.497696, 0.492488, 0.486469, 0.479615, 0.472658, 0.466043,\
        0.459849, 0.453937, 0.448171, 0.442507, 0.436939, 0.431440, 0.425952, 0.420397, 0.414710, 0.408894,\
        0.403371, 0.398562, 0.393846, 0.388551, 0.382871, 0.377265, 0.371841, 0.366555, 0.361358, 0.356246,\
        0.351245, 0.346385, 0.341687, 0.337158, 0.332790, 0.328568, 0.324472, 0.320480, 0.316567, 0.312702,\
        0.308840, 0.304901, 0.300817, 0.297077, 0.293385, 0.289406, 0.285147, 0.280599, 0.275765, 0.270661,\
        0.265299],\
        [0.196485, 0.203112, 0.209629, 0.215992, 0.222197, 0.228242, 0.234121, 0.239826, 0.245352, 0.250696,\
        0.255854, 0.260823, 0.265599, 0.270176, 0.274540, 0.278664, 0.282506, 0.286019, 0.289197, 0.292221,\
        0.295556, 0.299497, 0.304122, 0.309842, 0.316237, 0.323051, 0.330159, 0.337470, 0.344913, 0.352438,\
        0.360009, 0.367596, 0.375160, 0.382640, 0.390016, 0.397315, 0.403984, 0.410063, 0.416291, 0.423029,\
        0.430537, 0.439054, 0.448610, 0.458762, 0.468659, 0.477610, 0.485542, 0.492917, 0.499852, 0.505501,\
        0.510795, 0.516987, 0.523441, 0.529776, 0.536050, 0.541884, 0.546946, 0.551093, 0.554145, 0.555889,\
        0.556077, 0.554336, 0.550736, 0.546028, 0.540859, 0.535482, 0.529973, 0.524333, 0.518687, 0.513405,\
        0.508826, 0.503971, 0.497797, 0.491807, 0.486303, 0.481069, 0.475487, 0.469378, 0.463061, 0.456851,\
        0.450886, 0.445169, 0.439659, 0.434323, 0.429130, 0.424037, 0.418962, 0.413793, 0.408447, 0.402822,\
        0.397139, 0.392309, 0.387586, 0.382579, 0.377300, 0.371962, 0.366700, 0.361547, 0.356501, 0.351565,\
        0.346751, 0.342080, 0.337566, 0.333212, 0.329012, 0.324951, 0.321010, 0.317170, 0.313411, 0.309715,\
        0.306060, 0.302412, 0.298551, 0.294645, 0.290834, 0.286883, 0.282697, 0.278248, 0.273527, 0.268537,\
        0.263285],\
        [0.195764, 0.202355, 0.208772, 0.215012, 0.221085, 0.226988, 0.232711, 0.238249, 0.243597, 0.248753,\
        0.253716, 0.258482, 0.263049, 0.267409, 0.271551, 0.275455, 0.279103, 0.282489, 0.285668, 0.288806,\
        0.292159, 0.295923, 0.300491, 0.306187, 0.312416, 0.319000, 0.325832, 0.332828, 0.339932, 0.347109,\
        0.354334, 0.361586, 0.368837, 0.376056, 0.383213, 0.390194, 0.396663, 0.402712, 0.408906, 0.415517,\
        0.422587, 0.430313, 0.438760, 0.447641, 0.456457, 0.464807, 0.472605, 0.479981, 0.486840, 0.492735,\
        0.498262, 0.504965, 0.510813, 0.516192, 0.521818, 0.526721, 0.530731, 0.533970, 0.536344, 0.537538,\
        0.537403, 0.536031, 0.533493, 0.529914, 0.525576, 0.520823, 0.515899, 0.510685, 0.505233, 0.500123,\
        0.496041, 0.491854, 0.486429, 0.480851, 0.475531, 0.470349, 0.465014, 0.459392, 0.453590, 0.447782,\
        0.442099, 0.436609, 0.431349, 0.426316, 0.421471, 0.416767, 0.412124, 0.407346, 0.402336, 0.397089,\
        0.391606, 0.386512, 0.381659, 0.376753, 0.371720, 0.366630, 0.361570, 0.356591, 0.351714, 0.346953,\
        0.342322, 0.337837, 0.333509, 0.329339, 0.325318, 0.321429, 0.317649, 0.313961, 0.310354, 0.306809,\
        0.303300, 0.299818, 0.296174, 0.292348, 0.288490, 0.284530, 0.280384, 0.276004, 0.271369, 0.266472,\
        0.261312],\
        [0.195053, 0.201620, 0.207918, 0.214043, 0.219999, 0.225769, 0.231344, 0.236719, 0.241893, 0.246867,\
        0.251640, 0.256214, 0.260588, 0.264758, 0.268720, 0.272469, 0.276007, 0.279360, 0.282602, 0.285871,\
        0.289354, 0.293276, 0.297933, 0.303313, 0.309212, 0.315486, 0.321988, 0.328626, 0.335354, 0.342151,\
        0.349003, 0.355895, 0.362805, 0.369700, 0.376524, 0.383163, 0.389489, 0.395589, 0.401841, 0.408364,\
        0.414873, 0.421795, 0.429327, 0.437248, 0.445216, 0.452968, 0.460408, 0.467521, 0.474216, 0.480442,\
        0.486639, 0.493198, 0.498257, 0.503175, 0.508701, 0.512593, 0.515512, 0.517903, 0.519773, 0.520452,\
        0.519845, 0.518738, 0.517102, 0.514439, 0.510807, 0.506564, 0.502294, 0.497686, 0.492659, 0.487999,\
        0.483931, 0.479854, 0.475182, 0.470182, 0.465163, 0.460153, 0.455047, 0.449778, 0.444378, 0.438926,\
        0.433511, 0.428232, 0.423197, 0.418451, 0.413913, 0.409516, 0.405294, 0.400867, 0.396052, 0.391015,\
        0.385861, 0.380823, 0.375939, 0.371096, 0.366226, 0.361339, 0.356479, 0.351686, 0.346988, 0.342407,\
        0.337958, 0.333658, 0.329517, 0.325538, 0.321710, 0.318007, 0.314396, 0.310856, 0.307389, 0.303990,\
        0.300593, 0.297185, 0.293687, 0.290020, 0.286225, 0.282295, 0.278187, 0.273862, 0.269295, 0.264473,\
        0.259390],\
        [0.194216, 0.200720, 0.206978, 0.213051, 0.218921, 0.224574, 0.230008, 0.235227, 0.240233, 0.245032,\
        0.249627, 0.254024, 0.258226, 0.262236, 0.266061, 0.269708, 0.273198, 0.276574, 0.279915, 0.283338,\
        0.286997, 0.291064, 0.295679, 0.300833, 0.306457, 0.312421, 0.318560, 0.324794, 0.331105, 0.337489,\
        0.343940, 0.350447, 0.356987, 0.363526, 0.370010, 0.376368, 0.382570, 0.388709, 0.394998, 0.401302,\
        0.407068, 0.413305, 0.420185, 0.427395, 0.434657, 0.441805, 0.448771, 0.455537, 0.462074, 0.468402,\
        0.474632, 0.480596, 0.485781, 0.490901, 0.495681, 0.498762, 0.500847, 0.502535, 0.504238, 0.504568,\
        0.503245, 0.502355, 0.501495, 0.499444, 0.496420, 0.492702, 0.488906, 0.484871, 0.480574, 0.476405,\
        0.472485, 0.468552, 0.464321, 0.459806, 0.455136, 0.450374, 0.445515, 0.440550, 0.435497, 0.430370,\
        0.425193, 0.420061, 0.415186, 0.410730, 0.406513, 0.402237, 0.398278, 0.394147, 0.389592, 0.384818,\
        0.379943, 0.375086, 0.370299, 0.365559, 0.360834, 0.356125, 0.351453, 0.346846, 0.342332, 0.337930,\
        0.333662, 0.329544, 0.325591, 0.321810, 0.318192, 0.314700, 0.311277, 0.307876, 0.304522, 0.301274,\
        0.298020, 0.294701, 0.291289, 0.287733, 0.284022, 0.280147, 0.276086, 0.271812, 0.267302, 0.262542,\
        0.257523],\
        [0.193301, 0.199716, 0.205996, 0.212062, 0.217858, 0.223396, 0.228694, 0.233761, 0.238607, 0.243240,\
        0.247669, 0.251904, 0.255955, 0.259834, 0.263555, 0.267139, 0.270619, 0.274046, 0.277495, 0.281070,\
        0.284886, 0.289054, 0.293636, 0.298621, 0.304036, 0.309714, 0.315450, 0.321235, 0.327093, 0.333038,\
        0.339071, 0.345179, 0.351340, 0.357519, 0.363672, 0.369761, 0.375778, 0.381769, 0.387764, 0.393542,\
        0.398946, 0.404847, 0.411334, 0.417985, 0.424616, 0.431148, 0.437582, 0.443947, 0.450245, 0.456446,\
        0.462478, 0.468160, 0.473337, 0.478081, 0.482070, 0.484845, 0.486661, 0.488044, 0.489202, 0.489247,\
        0.488094, 0.487155, 0.486287, 0.484645, 0.482174, 0.479144, 0.475860, 0.472387, 0.468757, 0.465112,\
        0.461507, 0.457831, 0.453945, 0.449810, 0.445469, 0.440972, 0.436359, 0.431677, 0.426962, 0.422187,\
        0.417260, 0.412195, 0.407349, 0.403125, 0.399348, 0.395256, 0.391309, 0.387316, 0.383049, 0.378564,\
        0.373959, 0.369321, 0.364700, 0.360107, 0.355540, 0.351003, 0.346513, 0.342089, 0.337757, 0.333535,\
        0.329445, 0.325504, 0.321732, 0.318149, 0.314757, 0.311519, 0.308333, 0.305089, 0.301794, 0.298653,\
        0.295546, 0.292337, 0.289008, 0.285533, 0.281895, 0.278079, 0.274069, 0.269843, 0.265384, 0.260677,\
        0.255710],\
        [0.192480, 0.198892, 0.205150, 0.211133, 0.216814, 0.222225, 0.227385, 0.232306, 0.236999, 0.241476,\
        0.245751, 0.249838, 0.253757, 0.257525, 0.261166, 0.264710, 0.268196, 0.271678, 0.275228, 0.278930,\
        0.282873, 0.287133, 0.291746, 0.296696, 0.301955, 0.307263, 0.312517, 0.317813, 0.323205, 0.328709,\
        0.334324, 0.340038, 0.345824, 0.351652, 0.357482, 0.363285, 0.369043, 0.374750, 0.380365, 0.385792,\
        0.391112, 0.396777, 0.402845, 0.408962, 0.414987, 0.420889, 0.426745, 0.432667, 0.438673, 0.444665,\
        0.450480, 0.455928, 0.460860, 0.465167, 0.468655, 0.471174, 0.472858, 0.473981, 0.474606, 0.474478,\
        0.473688, 0.472760, 0.471738, 0.470286, 0.468284, 0.465850, 0.463149, 0.460266, 0.457242, 0.454124,\
        0.450923, 0.447583, 0.444030, 0.440226, 0.436183, 0.431937, 0.427543, 0.423101, 0.418726, 0.414390,\
        0.409845, 0.404855, 0.399943, 0.395844, 0.392165, 0.388388, 0.384551, 0.380649, 0.376583, 0.372348,\
        0.367994, 0.363583, 0.359156, 0.354739, 0.350345, 0.345984, 0.341673, 0.337432, 0.333281, 0.329238,\
        0.325323, 0.321552, 0.317948, 0.314545, 0.311380, 0.308447, 0.305595, 0.302591, 0.299419, 0.296315,\
        0.293253, 0.290109, 0.286843, 0.283431, 0.279852, 0.276091, 0.272129, 0.267949, 0.263535, 0.258872,\
        0.253949],\
        [0.191727, 0.198263, 0.204421, 0.210217, 0.215749, 0.221030, 0.226060, 0.230844, 0.235393, 0.239722,\
        0.243851, 0.247802, 0.251599, 0.255270, 0.258845, 0.262357, 0.265851, 0.269380, 0.273006, 0.276799,\
        0.280829, 0.285153, 0.289805, 0.294777, 0.299933, 0.304776, 0.309530, 0.314366, 0.319323, 0.324414,\
        0.329637, 0.334977, 0.340410, 0.345903, 0.351421, 0.356929, 0.362399, 0.367807, 0.373119, 0.378329,\
        0.383540, 0.388945, 0.394565, 0.400185, 0.405675, 0.410942, 0.416192, 0.421667, 0.427394, 0.433186,\
        0.438797, 0.443999, 0.448620, 0.452540, 0.455654, 0.457922, 0.459424, 0.460300, 0.460621, 0.460392,\
        0.459735, 0.458867, 0.457834, 0.456523, 0.454873, 0.452938, 0.450804, 0.448514, 0.446070, 0.443472,\
        0.440711, 0.437751, 0.434540, 0.431048, 0.427285, 0.423278, 0.419059, 0.414758, 0.410665, 0.406826,\
        0.402869, 0.398171, 0.393364, 0.389184, 0.385402, 0.381706, 0.377980, 0.374184, 0.370278, 0.366248,\
        0.362118, 0.357924, 0.353701, 0.349473, 0.345260, 0.341079, 0.336949, 0.332890, 0.328921, 0.325060,\
        0.321320, 0.317716, 0.314264, 0.311003, 0.308011, 0.305395, 0.302998, 0.300303, 0.297273, 0.294195,\
        0.291127, 0.288007, 0.284783, 0.281420, 0.277891, 0.274178, 0.270261, 0.266122, 0.261747, 0.257122,\
        0.252236],\
        [0.190790, 0.197364, 0.203452, 0.209146, 0.214590, 0.219775, 0.224695, 0.229354, 0.233766, 0.237954,\
        0.241943, 0.245764, 0.249448, 0.253028, 0.256540, 0.260021, 0.263515, 0.267069, 0.270738, 0.274578,\
        0.278639, 0.282961, 0.287555, 0.292376, 0.297233, 0.301819, 0.306265, 0.310755, 0.315354, 0.320088,\
        0.324960, 0.329963, 0.335074, 0.340261, 0.345485, 0.350709, 0.355900, 0.361032, 0.366089, 0.371089,\
        0.376098, 0.381195, 0.386374, 0.391531, 0.396543, 0.401247, 0.405936, 0.411009, 0.416526, 0.422154,\
        0.427577, 0.432522, 0.436816, 0.440375, 0.443160, 0.445172, 0.446463, 0.447121, 0.447235, 0.446909,\
        0.446283, 0.445486, 0.444532, 0.443369, 0.441995, 0.440479, 0.438876, 0.437166, 0.435281, 0.433183,\
        0.430864, 0.428300, 0.425438, 0.422245, 0.418748, 0.415003, 0.410979, 0.406675, 0.402699, 0.399252,\
        0.395618, 0.391403, 0.386965, 0.382817, 0.378978, 0.375288, 0.371627, 0.367931, 0.364164, 0.360310,\
        0.356379, 0.352389, 0.348366, 0.344331, 0.340305, 0.336307, 0.332358, 0.328480, 0.324696, 0.321021,\
        0.317466, 0.314037, 0.310739, 0.307590, 0.304656, 0.302188, 0.300357, 0.297893, 0.295042, 0.292074,\
        0.289060, 0.285982, 0.282802, 0.279485, 0.276003, 0.272333, 0.268457, 0.264356, 0.260016, 0.255422,\
        0.250565],\
        [0.189570, 0.196030, 0.202097, 0.207864, 0.213308, 0.218441, 0.223273, 0.227819, 0.232100, 0.236148,\
        0.239999, 0.243692, 0.247264, 0.250755, 0.254203, 0.257647, 0.261126, 0.264683, 0.268362, 0.272202,\
        0.276237, 0.280484, 0.284926, 0.289492, 0.294035, 0.298421, 0.302685, 0.306937, 0.311260, 0.315697,\
        0.320268, 0.324978, 0.329807, 0.334723, 0.339683, 0.344646, 0.349579, 0.354457, 0.359274, 0.364044,\
        0.368802, 0.373576, 0.378350, 0.383055, 0.387596, 0.391930, 0.396233, 0.400976, 0.406286, 0.411721,\
        0.416953, 0.421612, 0.425546, 0.428744, 0.431217, 0.432981, 0.434066, 0.434527, 0.434460, 0.434015,\
        0.433383, 0.432686, 0.431875, 0.430856, 0.429681, 0.428506, 0.427409, 0.426279, 0.424921, 0.423267,\
        0.421357, 0.419192, 0.416684, 0.413767, 0.410493, 0.407028, 0.403386, 0.399174, 0.395202, 0.391804,\
        0.388310, 0.384480, 0.380458, 0.376511, 0.372735, 0.369084, 0.365488, 0.361887, 0.358249, 0.354555,\
        0.350805, 0.347009, 0.343182, 0.339340, 0.335501, 0.331684, 0.327914, 0.324217, 0.320619, 0.317139,\
        0.313785, 0.310553, 0.307442, 0.304454, 0.301599, 0.299105, 0.297320, 0.295200, 0.292647, 0.289878,\
        0.286987, 0.283990, 0.280872, 0.277608, 0.274174, 0.270547, 0.266709, 0.262643, 0.258333, 0.253766,\
        0.248931],\
        [0.188198, 0.194629, 0.200714, 0.206503, 0.211943, 0.217036, 0.221790, 0.226225, 0.230374, 0.234279,\
        0.237988, 0.241549, 0.245008, 0.248409, 0.251789, 0.255188, 0.258639, 0.262179, 0.265837, 0.269641,\
        0.273608, 0.277734, 0.281991, 0.286310, 0.290599, 0.294789, 0.298886, 0.302953, 0.307055, 0.311244,\
        0.315561, 0.320023, 0.324614, 0.329299, 0.334030, 0.338761, 0.343457, 0.348097, 0.352677, 0.357205,\
        0.361697, 0.366163, 0.370584, 0.374921, 0.379132, 0.383241, 0.387410, 0.391924, 0.396844, 0.401983,\
        0.406995, 0.411285, 0.414811, 0.417645, 0.419829, 0.421380, 0.422295, 0.422588, 0.422345, 0.421745,\
        0.421074, 0.420548, 0.419977, 0.419071, 0.417979, 0.417050, 0.416447, 0.415933, 0.415043, 0.413712,\
        0.412138, 0.410368, 0.408218, 0.405552, 0.402444, 0.399150, 0.395814, 0.392022, 0.388231, 0.384732,\
        0.381277, 0.377679, 0.373962, 0.370244, 0.366604, 0.363045, 0.359536, 0.356041, 0.352531, 0.348990,\
        0.345414, 0.341805, 0.338170, 0.334520, 0.330868, 0.327233, 0.323638, 0.320114, 0.316699, 0.313420,\
        0.310286, 0.307275, 0.304379, 0.301634, 0.299008, 0.296634, 0.294618, 0.292557, 0.290224, 0.287655,\
        0.284912, 0.282021, 0.278981, 0.275779, 0.272396, 0.268812, 0.265011, 0.260976, 0.256691, 0.252146,\
        0.247328],\
        [0.186747, 0.193220, 0.199346, 0.205107, 0.210523, 0.215574, 0.220246, 0.224561, 0.228565, 0.232317,\
        0.235875, 0.239300, 0.242642, 0.245949, 0.249260, 0.252608, 0.256022, 0.259529, 0.263148, 0.266894,\
        0.270770, 0.274764, 0.278841, 0.282949, 0.287027, 0.291034, 0.294971, 0.298872, 0.302786, 0.306761,\
        0.310860, 0.315116, 0.319517, 0.324014, 0.328550, 0.333075, 0.337552, 0.341964, 0.346309, 0.350594,\
        0.354828, 0.359014, 0.363143, 0.367198, 0.371181, 0.375142, 0.379205, 0.383516, 0.388110, 0.392887,\
        0.397509, 0.401374, 0.404521, 0.407032, 0.408972, 0.410369, 0.411176, 0.411353, 0.410974, 0.410196,\
        0.409409, 0.409167, 0.409013, 0.408139, 0.406989, 0.406204, 0.406045, 0.406236, 0.405665, 0.404484,\
        0.403138, 0.401721, 0.399904, 0.397451, 0.394567, 0.391471, 0.388255, 0.384823, 0.381316, 0.377892,\
        0.374506, 0.371070, 0.367573, 0.364063, 0.360583, 0.357150, 0.353754, 0.350378, 0.347003, 0.343617,\
        0.340213, 0.336789, 0.333347, 0.329892, 0.326430, 0.322974, 0.319548, 0.316187, 0.312941, 0.309861,\
        0.306964, 0.304200, 0.301510, 0.298958, 0.296557, 0.294304, 0.292221, 0.290142, 0.287910, 0.285483,\
        0.282870, 0.280085, 0.277128, 0.273992, 0.270661, 0.267121, 0.263354, 0.259347, 0.255085, 0.250555,\
        0.245748],\
        [0.185199, 0.191716, 0.197844, 0.203619, 0.209050, 0.214061, 0.218634, 0.222808, 0.226648, 0.230228,\
        0.233624, 0.236905, 0.240129, 0.243343, 0.246583, 0.249880, 0.253254, 0.256722, 0.260294, 0.263975,\
        0.267757, 0.271625, 0.275545, 0.279478, 0.283381, 0.287228, 0.291015, 0.294765, 0.298510, 0.302297,\
        0.306205, 0.310297, 0.314551, 0.318900, 0.323273, 0.327610, 0.331879, 0.336067, 0.340181, 0.344229,\
        0.348219, 0.352157, 0.356041, 0.359873, 0.363674, 0.367493, 0.371410, 0.375499, 0.379761, 0.384078,\
        0.388167, 0.391700, 0.394584, 0.396869, 0.398630, 0.399923, 0.400681, 0.400803, 0.400397, 0.399561,\
        0.398685, 0.398694, 0.398960, 0.398088, 0.396982, 0.396376, 0.396460, 0.397000, 0.396604, 0.395604,\
        0.394436, 0.393167, 0.391515, 0.389333, 0.386738, 0.383887, 0.380866, 0.377696, 0.374443, 0.371183,\
        0.367925, 0.364644, 0.361330, 0.358004, 0.354688, 0.351398, 0.348134, 0.344891, 0.341659, 0.338432,\
        0.335203, 0.331968, 0.328725, 0.325471, 0.322205, 0.318933, 0.315672, 0.312459, 0.309355, 0.306447,\
        0.303795, 0.301304, 0.298840, 0.296453, 0.294200, 0.292056, 0.289990, 0.287917, 0.285741, 0.283405,\
        0.280893, 0.278199, 0.275320, 0.272246, 0.268967, 0.265467, 0.261733, 0.257750, 0.253505, 0.248988,\
        0.244186],\
        [0.183533, 0.190127, 0.196300, 0.202112, 0.207553, 0.212499, 0.216938, 0.220936, 0.224582, 0.227972,\
        0.231194, 0.234326, 0.237432, 0.240557, 0.243735, 0.246987, 0.250325, 0.253757, 0.257285, 0.260904,\
        0.264604, 0.268363, 0.272154, 0.275947, 0.279709, 0.283422, 0.287082, 0.290700, 0.294302, 0.297930,\
        0.301668, 0.305632, 0.309772, 0.314002, 0.318233, 0.322392, 0.326450, 0.330413, 0.334295, 0.338113,\
        0.341876, 0.345591, 0.349262, 0.352901, 0.356530, 0.360188, 0.363921, 0.367758, 0.371669, 0.375541,\
        0.379177, 0.382376, 0.385041, 0.387174, 0.388821, 0.390007, 0.390698, 0.390845, 0.390524, 0.389893,\
        0.389321, 0.389234, 0.389253, 0.388743, 0.388033, 0.387635, 0.387716, 0.387955, 0.387735, 0.387016,\
        0.386023, 0.384812, 0.383269, 0.381314, 0.378992, 0.376398, 0.373611, 0.370682, 0.367660, 0.364593,\
        0.361500, 0.358383, 0.355243, 0.352089, 0.348937, 0.345797, 0.342676, 0.339575, 0.336494, 0.333430,\
        0.330382, 0.327343, 0.324307, 0.321265, 0.318210, 0.315135, 0.312045, 0.308973, 0.305983, 0.303183,\
        0.300738, 0.298508, 0.296265, 0.294068, 0.291950, 0.289899, 0.287881, 0.285837, 0.283703, 0.281432,\
        0.278993, 0.276373, 0.273560, 0.270543, 0.267309, 0.263846, 0.260140, 0.256178, 0.251946, 0.247435,\
        0.242632],\
        [0.181707, 0.188438, 0.194741, 0.200609, 0.206036, 0.210857, 0.215106, 0.218890, 0.222317, 0.225498,\
        0.228537, 0.231522, 0.234518, 0.237568, 0.240698, 0.243919, 0.247234, 0.250642, 0.254136, 0.257707,\
        0.261340, 0.265014, 0.268707, 0.272391, 0.276046, 0.279656, 0.283218, 0.286740, 0.290245, 0.293770,\
        0.297393, 0.301225, 0.305239, 0.309368, 0.313466, 0.317436, 0.321267, 0.324993, 0.328645, 0.332241,\
        0.335791, 0.339303, 0.342780, 0.346236, 0.349690, 0.353168, 0.356692, 0.360263, 0.363840, 0.367327,\
        0.370589, 0.373491, 0.375953, 0.377955, 0.379508, 0.380622, 0.381286, 0.381506, 0.381353, 0.380988,\
        0.380650, 0.380500, 0.380385, 0.380083, 0.379694, 0.379459, 0.379452, 0.379487, 0.379274, 0.378710,\
        0.377848, 0.376715, 0.375276, 0.373498, 0.371404, 0.369050, 0.366498, 0.363800, 0.361000, 0.358134,\
        0.355224, 0.352282, 0.349315, 0.346332, 0.343342, 0.340356, 0.337383, 0.334430, 0.331503, 0.328608,\
        0.325745, 0.322909, 0.320090, 0.317275, 0.314449, 0.311596, 0.308701, 0.305783, 0.302924, 0.300234,\
        0.297908, 0.295847, 0.293810, 0.291788, 0.289799, 0.287837, 0.285874, 0.283869, 0.281776, 0.279553,\
        0.277171, 0.274608, 0.271849, 0.268879, 0.265685, 0.262253, 0.258570, 0.254623, 0.250399, 0.245888,\
        0.241080],\
        [0.179634, 0.186534, 0.193028, 0.199050, 0.204416, 0.209016, 0.213039, 0.216593, 0.219788, 0.222751,\
        0.225607, 0.228454, 0.231359, 0.234357, 0.237463, 0.240676, 0.243987, 0.247388, 0.250866, 0.254407,\
        0.257994, 0.261608, 0.265229, 0.268838, 0.272416, 0.275954, 0.279450, 0.282914, 0.286367, 0.289848,\
        0.293416, 0.297124, 0.300992, 0.305042, 0.308993, 0.312733, 0.316307, 0.319785, 0.323208, 0.326595,\
        0.329950, 0.333273, 0.336568, 0.339845, 0.343118, 0.346404, 0.349709, 0.353020, 0.356295, 0.359457,\
        0.362409, 0.365058, 0.367337, 0.369219, 0.370698, 0.371778, 0.372464, 0.372786, 0.372812, 0.372667,\
        0.372492, 0.372365, 0.372243, 0.372057, 0.371843, 0.371690, 0.371620, 0.371530, 0.371271, 0.370755,\
        0.369966, 0.368907, 0.367565, 0.365933, 0.364026, 0.361884, 0.359551, 0.357073, 0.354486, 0.351822,\
        0.349103, 0.346344, 0.343554, 0.340741, 0.337915, 0.335085, 0.332262, 0.329457, 0.326685, 0.323958,\
        0.321284, 0.318658, 0.316065, 0.313487, 0.310909, 0.308312, 0.305654, 0.302912, 0.300220, 0.297721,\
        0.295487, 0.293471, 0.291540, 0.289639, 0.287752, 0.285867, 0.283958, 0.281995, 0.279939, 0.277756,\
        0.275417, 0.272898, 0.270181, 0.267249, 0.264087, 0.260680, 0.257015, 0.253077, 0.248856, 0.244340,\
        0.239520],\
        [0.177195, 0.184230, 0.190875, 0.197045, 0.202320, 0.206739, 0.210611, 0.213963, 0.216928, 0.219675,\
        0.222359, 0.225091, 0.227936, 0.230918, 0.234034, 0.237268, 0.240602, 0.244017, 0.247497, 0.251026,\
        0.254589, 0.258167, 0.261743, 0.265302, 0.268831, 0.272324, 0.275783, 0.279217, 0.282649, 0.286113,\
        0.289652, 0.293305, 0.297108, 0.301031, 0.304777, 0.308222, 0.311515, 0.314748, 0.317959, 0.321157,\
        0.324336, 0.327486, 0.330607, 0.333705, 0.336794, 0.339883, 0.342970, 0.346038, 0.349047, 0.351936,\
        0.354632, 0.357067, 0.359188, 0.360966, 0.362391, 0.363464, 0.364200, 0.364632, 0.364819, 0.364850,\
        0.364810, 0.364751, 0.364673, 0.364562, 0.364431, 0.364309, 0.364191, 0.364015, 0.363699, 0.363176,\
        0.362417, 0.361410, 0.360150, 0.358638, 0.356889, 0.354930, 0.352796, 0.350521, 0.348139, 0.345674,\
        0.343148, 0.340576, 0.337967, 0.335329, 0.332670, 0.329997, 0.327323, 0.324664, 0.322040, 0.319476,\
        0.316989, 0.314579, 0.312218, 0.309876, 0.307545, 0.305226, 0.302874, 0.300346, 0.297809, 0.295459,\
        0.293307, 0.291328, 0.289452, 0.287623, 0.285807, 0.283982, 0.282121, 0.280194, 0.278172, 0.276022,\
        0.273717, 0.271233, 0.268548, 0.265645, 0.262507, 0.259119, 0.255465, 0.251532, 0.247308, 0.242781,\
        0.237943],\
        [0.174297, 0.181384, 0.188034, 0.194096, 0.199317, 0.203918, 0.207756, 0.210935, 0.213677, 0.216220,\
        0.218757, 0.221413, 0.224246, 0.227262, 0.230431, 0.233720, 0.237101, 0.240551, 0.244052, 0.247589,\
        0.251146, 0.254709, 0.258262, 0.261794, 0.265297, 0.268766, 0.272206, 0.275627, 0.279049, 0.282502,\
        0.286018, 0.289627, 0.293343, 0.297099, 0.300612, 0.303773, 0.306819, 0.309842, 0.312874, 0.315915,\
        0.318942, 0.321933, 0.324883, 0.327803, 0.330707, 0.333600, 0.336480, 0.339326, 0.342103, 0.344762,\
        0.347245, 0.349501, 0.351489, 0.353182, 0.354570, 0.355655, 0.356451, 0.356989, 0.357315, 0.357490,\
        0.357570, 0.357594, 0.357575, 0.357517, 0.357425, 0.357304, 0.357142, 0.356899, 0.356523, 0.355970,\
        0.355212, 0.354236, 0.353038, 0.351624, 0.350006, 0.348206, 0.346249, 0.344164, 0.341975, 0.339705,\
        0.337371, 0.334988, 0.332564, 0.330106, 0.327618, 0.325106, 0.322581, 0.320061, 0.317575, 0.315158,\
        0.312852, 0.310664, 0.308542, 0.306425, 0.304307, 0.302215, 0.300177, 0.297912, 0.295552, 0.293322,\
        0.291253, 0.289325, 0.287493, 0.285713, 0.283947, 0.282167, 0.280344, 0.278450, 0.276456, 0.274333,\
        0.272054, 0.269596, 0.266937, 0.264056, 0.260935, 0.257558, 0.253910, 0.249976, 0.245743, 0.241200,\
        0.236337],\
        [0.170910, 0.177985, 0.184589, 0.190612, 0.195983, 0.200665, 0.204439, 0.207458, 0.209986, 0.212356,\
        0.214790, 0.217425, 0.220312, 0.223422, 0.226692, 0.230068, 0.233517, 0.237019, 0.240556, 0.244116,\
        0.247684, 0.251247, 0.254796, 0.258319, 0.261812, 0.265272, 0.268704, 0.272118, 0.275531, 0.278962,\
        0.282435, 0.285959, 0.289517, 0.293021, 0.296303, 0.299315, 0.302195, 0.305062, 0.307951, 0.310871,\
        0.313772, 0.316611, 0.319389, 0.322129, 0.324848, 0.327553, 0.330238, 0.332885, 0.335463, 0.337929,\
        0.340235, 0.342342, 0.344219, 0.345843, 0.347208, 0.348313, 0.349172, 0.349808, 0.350255, 0.350553,\
        0.350741, 0.350847, 0.350887, 0.350868, 0.350792, 0.350659, 0.350453, 0.350150, 0.349716, 0.349121,\
        0.348346, 0.347382, 0.346228, 0.344892, 0.343384, 0.341722, 0.339925, 0.338014, 0.336008, 0.333926,\
        0.331781, 0.329588, 0.327353, 0.325081, 0.322773, 0.320427, 0.318053, 0.315670, 0.313307, 0.311011,\
        0.308859, 0.306906, 0.305047, 0.303130, 0.301194, 0.299262, 0.297373, 0.295371, 0.293268, 0.291206,\
        0.289245, 0.287387, 0.285606, 0.283869, 0.282143, 0.280399, 0.278607, 0.276740, 0.274769, 0.272668,\
        0.270411, 0.267973, 0.265332, 0.262466, 0.259357, 0.255987, 0.252338, 0.248398, 0.244151, 0.239586,\
        0.234693],\
        [0.167056, 0.174089, 0.180641, 0.186642, 0.192028, 0.196708, 0.200556, 0.203464, 0.205823, 0.208096,\
        0.210488, 0.213169, 0.216196, 0.219471, 0.222880, 0.226361, 0.229890, 0.233451, 0.237034, 0.240626,\
        0.244216, 0.247794, 0.251350, 0.254877, 0.258371, 0.261832, 0.265262, 0.268669, 0.272065, 0.275462,\
        0.278869, 0.282280, 0.285664, 0.288950, 0.292052, 0.294949, 0.297717, 0.300448, 0.303212, 0.306050,\
        0.308842, 0.311522, 0.314116, 0.316674, 0.319214, 0.321738, 0.324244, 0.326717, 0.329126, 0.331429,\
        0.333587, 0.335569, 0.337353, 0.338922, 0.340272, 0.341403, 0.342323, 0.343048, 0.343601, 0.344007,\
        0.344291, 0.344474, 0.344566, 0.344577, 0.344507, 0.344354, 0.344106, 0.343746, 0.343253, 0.342610,\
        0.341806, 0.340839, 0.339714, 0.338438, 0.337023, 0.335482, 0.333829, 0.332080, 0.330248, 0.328346,\
        0.326386, 0.324381, 0.322339, 0.320262, 0.318145, 0.315978, 0.313761, 0.311516, 0.309278, 0.307071,\
        0.305004, 0.303284, 0.301761, 0.299996, 0.298218, 0.296448, 0.294673, 0.292845, 0.290959, 0.289074,\
        0.287238, 0.285465, 0.283744, 0.282053, 0.280365, 0.278651, 0.276886, 0.275041, 0.273091, 0.271008,\
        0.268767, 0.266344, 0.263715, 0.260860, 0.257757, 0.254389, 0.250736, 0.246784, 0.242518, 0.237926,\
        0.232997],\
        [0.162774, 0.169750, 0.176237, 0.182176, 0.187501, 0.192129, 0.195960, 0.198883, 0.201245, 0.203568,\
        0.205958, 0.208745, 0.212023, 0.215521, 0.219075, 0.222653, 0.226254, 0.229874, 0.233505, 0.237135,\
        0.240755, 0.244356, 0.247929, 0.251469, 0.254972, 0.258437, 0.261867, 0.265265, 0.268638, 0.271991,\
        0.275323, 0.278621, 0.281854, 0.284976, 0.287942, 0.290749, 0.293438, 0.296073, 0.298737, 0.301487,\
        0.304161, 0.306652, 0.309041, 0.311429, 0.313804, 0.316154, 0.318495, 0.320819, 0.323087, 0.325255,\
        0.327286, 0.329161, 0.330864, 0.332388, 0.333729, 0.334887, 0.335866, 0.336673, 0.337321, 0.337823,\
        0.338194, 0.338444, 0.338583, 0.338616, 0.338546, 0.338370, 0.338081, 0.337667, 0.337116, 0.336420,\
        0.335577, 0.334595, 0.333484, 0.332255, 0.330919, 0.329485, 0.327964, 0.326366, 0.324699, 0.322971,\
        0.321189, 0.319367, 0.317520, 0.315649, 0.313741, 0.311774, 0.309722, 0.307615, 0.305541, 0.303455,\
        0.301357, 0.299781, 0.298534, 0.296995, 0.295356, 0.293717, 0.292064, 0.290378, 0.288657, 0.286926,\
        0.285213, 0.283531, 0.281876, 0.280234, 0.278582, 0.276898, 0.275157, 0.273331, 0.271397, 0.269329,\
        0.267101, 0.264689, 0.262069, 0.259220, 0.256120, 0.252749, 0.249088, 0.245120, 0.240830, 0.236206,\
        0.231236],\
        [0.158116, 0.165028, 0.171445, 0.177310, 0.182561, 0.187127, 0.190942, 0.194014, 0.196593, 0.199021,\
        0.201376, 0.204357, 0.208027, 0.211717, 0.215351, 0.218982, 0.222635, 0.226306, 0.229984, 0.233655,\
        0.237309, 0.240937, 0.244533, 0.248091, 0.251608, 0.255081, 0.258512, 0.261900, 0.265248, 0.268554,\
        0.271813, 0.275011, 0.278124, 0.281123, 0.283987, 0.286720, 0.289351, 0.291932, 0.294522, 0.297134,\
        0.299673, 0.301940, 0.304130, 0.306400, 0.308627, 0.310796, 0.312987, 0.315194, 0.317350, 0.319401,\
        0.321319, 0.323094, 0.324725, 0.326209, 0.327545, 0.328731, 0.329766, 0.330650, 0.331384, 0.331974,\
        0.332422, 0.332733, 0.332912, 0.332962, 0.332887, 0.332688, 0.332359, 0.331894, 0.331286, 0.330533,\
        0.329643, 0.328633, 0.327524, 0.326332, 0.325066, 0.323730, 0.322331, 0.320875, 0.319368, 0.317807,\
        0.316193, 0.314543, 0.312886, 0.311231, 0.309550, 0.307820, 0.305966, 0.303945, 0.302066, 0.300274,\
        0.298275, 0.296688, 0.295390, 0.294025, 0.292558, 0.291050, 0.289517, 0.287956, 0.286367, 0.284764,\
        0.283161, 0.281566, 0.279977, 0.278384, 0.276770, 0.275114, 0.273394, 0.271586, 0.269666, 0.267609,\
        0.265391, 0.262987, 0.260374, 0.257528, 0.254427, 0.251050, 0.247377, 0.243390, 0.239073, 0.234411,\
        0.229395],\
        [0.153135, 0.159990, 0.166348, 0.172158, 0.177370, 0.181938, 0.185842, 0.189139, 0.192001, 0.194636,\
        0.197253, 0.200623, 0.204502, 0.208123, 0.211710, 0.215348, 0.219037, 0.222756, 0.226481, 0.230193,\
        0.233883, 0.237541, 0.241162, 0.244742, 0.248275, 0.251760, 0.255192, 0.258571, 0.261893, 0.265156,\
        0.268351, 0.271466, 0.274485, 0.277391, 0.280174, 0.282842, 0.285417, 0.287935, 0.290425, 0.292878,\
        0.295217, 0.297354, 0.299424, 0.301636, 0.303698, 0.305647, 0.307719, 0.309851, 0.311920, 0.313862,\
        0.315668, 0.317346, 0.318907, 0.320354, 0.321688, 0.322903, 0.323992, 0.324948, 0.325763, 0.326433,\
        0.326953, 0.327318, 0.327529, 0.327589, 0.327506, 0.327284, 0.326921, 0.326410, 0.325747, 0.324934,\
        0.323987, 0.322938, 0.321820, 0.320659, 0.319459, 0.318215, 0.316926, 0.315604, 0.314255, 0.312863,\
        0.311404, 0.309899, 0.308420, 0.306991, 0.305536, 0.304063, 0.302572, 0.300539, 0.298764, 0.297131,\
        0.295420, 0.293865, 0.292505, 0.291186, 0.289830, 0.288435, 0.287010, 0.285558, 0.284080, 0.282583,\
        0.281074, 0.279557, 0.278030, 0.276484, 0.274905, 0.273275, 0.271575, 0.269782, 0.267873, 0.265826,\
        0.263615, 0.261216, 0.258606, 0.255761, 0.252657, 0.249273, 0.245586, 0.241577, 0.237228, 0.232526,\
        0.227459],\
        [0.147882, 0.154699, 0.161023, 0.166816, 0.172043, 0.176683, 0.180751, 0.184316, 0.187523, 0.190574,\
        0.193770, 0.197468, 0.201133, 0.204539, 0.208067, 0.211719, 0.215453, 0.219226, 0.222999, 0.226753,\
        0.230476, 0.234164, 0.237813, 0.241417, 0.244970, 0.248468, 0.251905, 0.255276, 0.258577, 0.261803,\
        0.264944, 0.267993, 0.270937, 0.273768, 0.276483, 0.279089, 0.281603, 0.284045, 0.286426, 0.288733,\
        0.290927, 0.292997, 0.295027, 0.297078, 0.298948, 0.300704, 0.302734, 0.304819, 0.306803, 0.308625,\
        0.310311, 0.311890, 0.313383, 0.314795, 0.316127, 0.317373, 0.318516, 0.319539, 0.320429, 0.321175,\
        0.321763, 0.322176, 0.322409, 0.322472, 0.322376, 0.322133, 0.321742, 0.321196, 0.320486, 0.319611,\
        0.318598, 0.317493, 0.316356, 0.315225, 0.314097, 0.312943, 0.311750, 0.310545, 0.309357, 0.308149,\
        0.306842, 0.305434, 0.304090, 0.302917, 0.301681, 0.300324, 0.299238, 0.297572, 0.295745, 0.294127,\
        0.292569, 0.291105, 0.289755, 0.288462, 0.287172, 0.285862, 0.284527, 0.283166, 0.281780, 0.280370,\
        0.278940, 0.277490, 0.276017, 0.274513, 0.272965, 0.271359, 0.269676, 0.267895, 0.265996, 0.263955,\
        0.261749, 0.259354, 0.256746, 0.253899, 0.250791, 0.247397, 0.243695, 0.239662, 0.235280, 0.230533,\
        0.225410],\
        [0.142403, 0.149205, 0.155527, 0.161344, 0.166636, 0.171402, 0.175674, 0.179528, 0.183090, 0.186536,\
        0.190063, 0.193739, 0.197317, 0.200775, 0.204359, 0.208074, 0.211883, 0.215720, 0.219543, 0.223332,\
        0.227085, 0.230802, 0.234479, 0.238111, 0.241688, 0.245202, 0.248647, 0.252015, 0.255299, 0.258494,\
        0.261592, 0.264587, 0.267471, 0.270239, 0.272894, 0.275440, 0.277889, 0.280253, 0.282535, 0.284729,\
        0.286823, 0.288824, 0.290767, 0.292653, 0.294427, 0.296182, 0.298104, 0.300093, 0.301972, 0.303653,\
        0.305215, 0.306701, 0.308129, 0.309508, 0.310839, 0.312115, 0.313312, 0.314397, 0.315352, 0.316172,\
        0.316829, 0.317285, 0.317528, 0.317578, 0.317464, 0.317203, 0.316795, 0.316231, 0.315489, 0.314558,\
        0.313467, 0.312288, 0.311116, 0.310020, 0.308982, 0.307925, 0.306810, 0.305686, 0.304655, 0.303662,\
        0.302526, 0.301192, 0.299908, 0.298954, 0.297932, 0.296764, 0.295645, 0.294279, 0.292714, 0.291187,\
        0.289731, 0.288352, 0.287048, 0.285793, 0.284555, 0.283310, 0.282048, 0.280761, 0.279449, 0.278111,\
        0.276745, 0.275351, 0.273923, 0.272454, 0.270933, 0.269345, 0.267675, 0.265904, 0.264011, 0.261974,\
        0.259771, 0.257377, 0.254768, 0.251919, 0.248806, 0.245402, 0.241683, 0.237625, 0.233208, 0.228414,\
        0.223230],\
        [0.136732, 0.143545, 0.149898, 0.155774, 0.161169, 0.166093, 0.170585, 0.174718, 0.178596, 0.182343,\
        0.186065, 0.189776, 0.193409, 0.196987, 0.200633, 0.204442, 0.208343, 0.212248, 0.216112, 0.219926,\
        0.223701, 0.227444, 0.231153, 0.234817, 0.238422, 0.241958, 0.245414, 0.248783, 0.252056, 0.255228,\
        0.258291, 0.261241, 0.264075, 0.266790, 0.269390, 0.271878, 0.274264, 0.276554, 0.278752, 0.280857,\
        0.282870, 0.284798, 0.286656, 0.288447, 0.290181, 0.291920, 0.293731, 0.295571, 0.297314, 0.298891,\
        0.300356, 0.301764, 0.303134, 0.304481, 0.305806, 0.307111, 0.308364, 0.309496, 0.310496, 0.311390,\
        0.312128, 0.312621, 0.312850, 0.312870, 0.312729, 0.312452, 0.312041, 0.311483, 0.310739, 0.309770,\
        0.308595, 0.307322, 0.306091, 0.305038, 0.304125, 0.303177, 0.302133, 0.301041, 0.300138, 0.299335,\
        0.298358, 0.297189, 0.296057, 0.295119, 0.294195, 0.293191, 0.292121, 0.290913, 0.289576, 0.288207,\
        0.286868, 0.285576, 0.284335, 0.283131, 0.281942, 0.280752, 0.279549, 0.278323, 0.277071, 0.275789,\
        0.274474, 0.273123, 0.271731, 0.270289, 0.268787, 0.267214, 0.265553, 0.263786, 0.261896, 0.259860,\
        0.257656, 0.255260, 0.252649, 0.249797, 0.246677, 0.243263, 0.239527, 0.235445, 0.230992, 0.226148,\
        0.220900],\
        [0.130895, 0.137744, 0.144155, 0.150119, 0.155641, 0.160739, 0.165453, 0.169844, 0.173994, 0.177988,\
        0.181891, 0.185724, 0.189487, 0.193215, 0.196983, 0.200878, 0.204853, 0.208815, 0.212698, 0.216515,\
        0.220304, 0.224078, 0.227826, 0.231529, 0.235168, 0.238730, 0.242203, 0.245577, 0.248844, 0.251998,\
        0.255033, 0.257947, 0.260737, 0.263405, 0.265954, 0.268388, 0.270714, 0.272938, 0.275065, 0.277098,\
        0.279040, 0.280902, 0.282692, 0.284423, 0.286115, 0.287802, 0.289508, 0.291204, 0.292822, 0.294325,\
        0.295733, 0.297086, 0.298405, 0.299720, 0.301022, 0.302345, 0.303660, 0.304813, 0.305810, 0.306784,\
        0.307646, 0.308159, 0.308332, 0.308295, 0.308123, 0.307835, 0.307431, 0.306904, 0.306207, 0.305233,\
        0.303982, 0.302623, 0.301324, 0.300306, 0.299516, 0.298658, 0.297703, 0.296721, 0.295875, 0.295109,\
        0.294250, 0.293280, 0.292310, 0.291415, 0.290543, 0.289630, 0.288644, 0.287562, 0.286390, 0.285173,\
        0.283954, 0.282757, 0.281587, 0.280439, 0.279301, 0.278161, 0.277006, 0.275830, 0.274625, 0.273386,\
        0.272110, 0.270791, 0.269424, 0.268001, 0.266512, 0.264945, 0.263287, 0.261520, 0.259628, 0.257588,\
        0.255380, 0.252981, 0.250364, 0.247506, 0.244379, 0.240955, 0.237204, 0.233098, 0.228609, 0.223713,\
        0.218397],\
        [0.124907, 0.131814, 0.138304, 0.144378, 0.150043, 0.155321, 0.160250, 0.164880, 0.169275, 0.173496,\
        0.177592, 0.181595, 0.185529, 0.189433, 0.193362, 0.197353, 0.201398, 0.205404, 0.209272, 0.213063,\
        0.216866, 0.220686, 0.224489, 0.228241, 0.231920, 0.235513, 0.239007, 0.242390, 0.245655, 0.248796,\
        0.251808, 0.254690, 0.257442, 0.260068, 0.262570, 0.264954, 0.267226, 0.269392, 0.271457, 0.273428,\
        0.275311, 0.277114, 0.278848, 0.280528, 0.282170, 0.283794, 0.285407, 0.286990, 0.288512, 0.289952,\
        0.291320, 0.292638, 0.293931, 0.295218, 0.296506, 0.297829, 0.299158, 0.300300, 0.301243, 0.302312,\
        0.303380, 0.303843, 0.303898, 0.303795, 0.303605, 0.303312, 0.302911, 0.302415, 0.301812, 0.300881,\
        0.299593, 0.298227, 0.296965, 0.295947, 0.295129, 0.294318, 0.293460, 0.292601, 0.291806, 0.291055,\
        0.290275, 0.289442, 0.288597, 0.287772, 0.286954, 0.286106, 0.285201, 0.284224, 0.283180, 0.282091,\
        0.280984, 0.279878, 0.278781, 0.277693, 0.276605, 0.275510, 0.274398, 0.273261, 0.272092, 0.270886,\
        0.269637, 0.268339, 0.266987, 0.265573, 0.264088, 0.262520, 0.260858, 0.259085, 0.257185, 0.255137,\
        0.252921, 0.250513, 0.247888, 0.245021, 0.241884, 0.238450, 0.234685, 0.230558, 0.226034, 0.221087,\
        0.215698],\
        [0.118777, 0.125759, 0.132347, 0.138544, 0.144361, 0.149821, 0.154958, 0.159813, 0.164437, 0.168877,\
        0.173177, 0.177370, 0.181490, 0.185574, 0.189661, 0.193777, 0.197910, 0.201957, 0.205757, 0.209506,\
        0.213362, 0.217266, 0.221142, 0.224950, 0.228674, 0.232301, 0.235818, 0.239215, 0.242482, 0.245612,\
        0.248603, 0.251457, 0.254174, 0.256760, 0.259219, 0.261557, 0.263781, 0.265895, 0.267908, 0.269825,\
        0.271655, 0.273407, 0.275092, 0.276723, 0.278314, 0.279874, 0.281408, 0.282906, 0.284353, 0.285740,\
        0.287073, 0.288367, 0.289641, 0.290911, 0.292189, 0.293482, 0.294744, 0.295871, 0.296882, 0.298030,\
        0.299150, 0.299503, 0.299475, 0.299343, 0.299150, 0.298856, 0.298459, 0.297971, 0.297383, 0.296501,\
        0.295313, 0.294046, 0.292863, 0.291848, 0.290983, 0.290178, 0.289382, 0.288601, 0.287857, 0.287139,\
        0.286415, 0.285672, 0.284917, 0.284162, 0.283399, 0.282610, 0.281776, 0.280887, 0.279945, 0.278961,\
        0.277952, 0.276930, 0.275903, 0.274872, 0.273832, 0.272778, 0.271702, 0.270597, 0.269455, 0.268271,\
        0.267038, 0.265751, 0.264404, 0.262989, 0.261497, 0.259920, 0.258246, 0.256460, 0.254546, 0.252485,\
        0.250254, 0.247832, 0.245193, 0.242312, 0.239163, 0.235717, 0.231941, 0.227795, 0.223241, 0.218242,\
        0.212777],\
        [0.112509, 0.119581, 0.126278, 0.132608, 0.138583, 0.144224, 0.149562, 0.154631, 0.159474, 0.164128,\
        0.168634, 0.173024, 0.177330, 0.181585, 0.185814, 0.190032, 0.194217, 0.198281, 0.202003, 0.205791,\
        0.209808, 0.213842, 0.217796, 0.221656, 0.225421, 0.229084, 0.232629, 0.236042, 0.239312, 0.242432,\
        0.245404, 0.248230, 0.250915, 0.253464, 0.255884, 0.258180, 0.260359, 0.262427, 0.264393, 0.266264,\
        0.268047, 0.269754, 0.271393, 0.272978, 0.274517, 0.276020, 0.277489, 0.278919, 0.280306, 0.281648,\
        0.282948, 0.284218, 0.285472, 0.286721, 0.287972, 0.289216, 0.290422, 0.291551, 0.292625, 0.293697,\
        0.294583, 0.294995, 0.295044, 0.294933, 0.294734, 0.294442, 0.294052, 0.293562, 0.292938, 0.292102,\
        0.291060, 0.289933, 0.288845, 0.287863, 0.286991, 0.286190, 0.285428, 0.284696, 0.283993, 0.283311,\
        0.282635, 0.281953, 0.281264, 0.280567, 0.279858, 0.279123, 0.278352, 0.277536, 0.276677, 0.275779,\
        0.274851, 0.273903, 0.272940, 0.271961, 0.270965, 0.269948, 0.268901, 0.267820, 0.266697, 0.265525,\
        0.264299, 0.263012, 0.261658, 0.260232, 0.258724, 0.257128, 0.255433, 0.253625, 0.251690, 0.249609,\
        0.247359, 0.244916, 0.242255, 0.239353, 0.236184, 0.232722, 0.228935, 0.224777, 0.220197, 0.215148,\
        0.209602],\
        [0.106102, 0.113274, 0.120090, 0.126559, 0.132694, 0.138515, 0.144048, 0.149325, 0.154377, 0.159242,\
        0.163950, 0.168534, 0.173021, 0.177435, 0.181792, 0.186088, 0.190288, 0.194303, 0.198089, 0.202073,\
        0.206323, 0.210459, 0.214455, 0.218345, 0.222147, 0.225850, 0.229429, 0.232861, 0.236133, 0.239242,\
        0.242193, 0.244991, 0.247644, 0.250158, 0.252541, 0.254799, 0.256938, 0.258966, 0.260890, 0.262719,\
        0.264461, 0.266125, 0.267723, 0.269263, 0.270756, 0.272207, 0.273621, 0.274996, 0.276334, 0.277634,\
        0.278902, 0.280145, 0.281373, 0.282592, 0.283804, 0.284999, 0.286157, 0.287259, 0.288298, 0.289248,\
        0.289993, 0.290420, 0.290554, 0.290497, 0.290318, 0.290036, 0.289653, 0.289161, 0.288537, 0.287755,\
        0.286832, 0.285836, 0.284848, 0.283922, 0.283072, 0.282289, 0.281553, 0.280854, 0.280184, 0.279536,\
        0.278898, 0.278262, 0.277622, 0.276973, 0.276310, 0.275624, 0.274907, 0.274153, 0.273361, 0.272532,\
        0.271673, 0.270788, 0.269879, 0.268946, 0.267989, 0.267001, 0.265979, 0.264914, 0.263801, 0.262633,\
        0.261404, 0.260107, 0.258737, 0.257288, 0.255753, 0.254127, 0.252400, 0.250562, 0.248598, 0.246490,\
        0.244213, 0.241742, 0.239051, 0.236116, 0.232914, 0.229426, 0.225624, 0.221460, 0.216866, 0.211771,\
        0.206138],\
        [0.099552, 0.106833, 0.113775, 0.120388, 0.126684, 0.132682, 0.138406, 0.143881, 0.149138, 0.154205,\
        0.159112, 0.163886, 0.168549, 0.173119, 0.177604, 0.181998, 0.186275, 0.190410, 0.194468, 0.198686,\
        0.202991, 0.207108, 0.211085, 0.214985, 0.218830, 0.222588, 0.226209, 0.229660, 0.232931, 0.236023,\
        0.238948, 0.241716, 0.244337, 0.246818, 0.249167, 0.251391, 0.253495, 0.255487, 0.257374, 0.259166,\
        0.260870, 0.262496, 0.264054, 0.265554, 0.267003, 0.268409, 0.269775, 0.271104, 0.272398, 0.273660,\
        0.274894, 0.276106, 0.277302, 0.278485, 0.279652, 0.280795, 0.281899, 0.282945, 0.283912, 0.284760,\
        0.285420, 0.285839, 0.286023, 0.286019, 0.285875, 0.285614, 0.285246, 0.284768, 0.284170, 0.283449,\
        0.282623, 0.281737, 0.280843, 0.279983, 0.279175, 0.278420, 0.277711, 0.277040, 0.276399, 0.275780,\
        0.275174, 0.274573, 0.273971, 0.273361, 0.272738, 0.272094, 0.271422, 0.270719, 0.269981, 0.269209,\
        0.268406, 0.267571, 0.266708, 0.265814, 0.264888, 0.263925, 0.262919, 0.261865, 0.260755, 0.259582,\
        0.258341, 0.257024, 0.255626, 0.254143, 0.252569, 0.250900, 0.249131, 0.247252, 0.245251, 0.243108,\
        0.240799, 0.238295, 0.235563, 0.232579, 0.229323, 0.225787, 0.221958, 0.217793, 0.213201, 0.208067,\
        0.202337],\
        [0.092854, 0.100251, 0.107324, 0.114082, 0.120539, 0.126711, 0.132620, 0.138289, 0.143743, 0.149009,\
        0.154109, 0.159069, 0.163906, 0.168636, 0.173265, 0.177792, 0.182214, 0.186545, 0.190856, 0.195226,\
        0.199552, 0.203659, 0.207599, 0.211525, 0.215450, 0.219290, 0.222963, 0.226428, 0.229686, 0.232751,\
        0.235643, 0.238378, 0.240967, 0.243418, 0.245738, 0.247932, 0.250005, 0.251966, 0.253820, 0.255578,\
        0.257248, 0.258839, 0.260362, 0.261824, 0.263234, 0.264599, 0.265924, 0.267212, 0.268466, 0.269692,\
        0.270891, 0.272068, 0.273227, 0.274368, 0.275486, 0.276574, 0.277617, 0.278597, 0.279488, 0.280254,\
        0.280855, 0.281260, 0.281469, 0.281504, 0.281396, 0.281166, 0.280823, 0.280373, 0.279815, 0.279157,\
        0.278415, 0.277622, 0.276814, 0.276021, 0.275262, 0.274544, 0.273866, 0.273222, 0.272607, 0.272014,\
        0.271434, 0.270862, 0.270290, 0.269712, 0.269121, 0.268512, 0.267878, 0.267216, 0.266522, 0.265795,\
        0.265035, 0.264242, 0.263414, 0.262551, 0.261650, 0.260704, 0.259709, 0.258658, 0.257544, 0.256360,\
        0.255098, 0.253751, 0.252316, 0.250786, 0.249160, 0.247435, 0.245609, 0.243677, 0.241629, 0.239447,\
        0.237102, 0.234560, 0.231780, 0.228729, 0.225389, 0.221765, 0.217874, 0.213704, 0.209139, 0.203976,\
        0.198129],\
        [0.086003, 0.093519, 0.100726, 0.107631, 0.114247, 0.120591, 0.126681, 0.132538, 0.138183, 0.143640,\
        0.148930, 0.154073, 0.159086, 0.163982, 0.168769, 0.173453, 0.178043, 0.182559, 0.187035, 0.191481,\
        0.195829, 0.199967, 0.203914, 0.207936, 0.212013, 0.215972, 0.219692, 0.223149, 0.226371, 0.229393,\
        0.232245, 0.234945, 0.237505, 0.239931, 0.242227, 0.244396, 0.246444, 0.248377, 0.250204, 0.251932,\
        0.253571, 0.255130, 0.256620, 0.258048, 0.259423, 0.260751, 0.262039, 0.263290, 0.264508, 0.265697,\
        0.266861, 0.268002, 0.269120, 0.270216, 0.271284, 0.272315, 0.273295, 0.274208, 0.275028, 0.275730,\
        0.276285, 0.276673, 0.276894, 0.276957, 0.276881, 0.276683, 0.276374, 0.275961, 0.275451, 0.274854,\
        0.274187, 0.273474, 0.272743, 0.272015, 0.271309, 0.270632, 0.269986, 0.269370, 0.268780, 0.268210,\
        0.267653, 0.267105, 0.266557, 0.266004, 0.265440, 0.264859, 0.264255, 0.263625, 0.262965, 0.262273,\
        0.261547, 0.260785, 0.259985, 0.259145, 0.258261, 0.257326, 0.256336, 0.255282, 0.254158, 0.252955,\
        0.251664, 0.250280, 0.248796, 0.247208, 0.245515, 0.243719, 0.241820, 0.239821, 0.237715, 0.235487,\
        0.233106, 0.230528, 0.227700, 0.224571, 0.221114, 0.217341, 0.213311, 0.209088, 0.204577, 0.199379,\
        0.193406],\
        [0.078990, 0.086630, 0.093971, 0.101023, 0.107798, 0.114310, 0.120576, 0.126615, 0.132446, 0.138089,\
        0.143564, 0.148888, 0.154077, 0.159143, 0.164096, 0.168946, 0.173703, 0.178380, 0.182986, 0.187505,\
        0.191883, 0.196054, 0.200083, 0.204288, 0.208590, 0.212672, 0.216391, 0.219789, 0.222942, 0.225905,\
        0.228713, 0.231382, 0.233919, 0.236327, 0.238606, 0.240758, 0.242787, 0.244698, 0.246499, 0.248201,\
        0.249813, 0.251344, 0.252804, 0.254201, 0.255544, 0.256840, 0.258094, 0.259311, 0.260495, 0.261650,\
        0.262778, 0.263881, 0.264958, 0.266008, 0.267025, 0.268000, 0.268920, 0.269770, 0.270529, 0.271177,\
        0.271694, 0.272066, 0.272291, 0.272374, 0.272326, 0.272161, 0.271887, 0.271516, 0.271056, 0.270518,\
        0.269919, 0.269278, 0.268615, 0.267949, 0.267293, 0.266658, 0.266047, 0.265459, 0.264893, 0.264345,\
        0.263808, 0.263279, 0.262751, 0.262218, 0.261675, 0.261115, 0.260535, 0.259930, 0.259295, 0.258629,\
        0.257927, 0.257187, 0.256407, 0.255582, 0.254708, 0.253778, 0.252787, 0.251725, 0.250585, 0.249357,\
        0.248032, 0.246602, 0.245060, 0.243403, 0.241629, 0.239743, 0.237753, 0.235667, 0.233488, 0.231208,\
        0.228794, 0.226191, 0.223327, 0.220125, 0.216534, 0.212554, 0.208270, 0.203856, 0.199313, 0.194083,\
        0.188075],\
        [0.071810, 0.079575, 0.087052, 0.094249, 0.101179, 0.107855, 0.114293, 0.120508, 0.126520, 0.132344,\
        0.138000, 0.143501, 0.148864, 0.154101, 0.159223, 0.164239, 0.169158, 0.173988, 0.178728, 0.183359,\
        0.187850, 0.192183, 0.196423, 0.200759, 0.205317, 0.209408, 0.213006, 0.216278, 0.219333, 0.222231,\
        0.224999, 0.227648, 0.230176, 0.232579, 0.234851, 0.236994, 0.239008, 0.240902, 0.242683, 0.244363,\
        0.245950, 0.247455, 0.248889, 0.250259, 0.251574, 0.252841, 0.254065, 0.255252, 0.256404, 0.257526,\
        0.258619, 0.259685, 0.260721, 0.261726, 0.262694, 0.263615, 0.264480, 0.265274, 0.265980, 0.266582,\
        0.267067, 0.267423, 0.267649, 0.267746, 0.267723, 0.267589, 0.267352, 0.267023, 0.266612, 0.266131,\
        0.265593, 0.265016, 0.264414, 0.263804, 0.263198, 0.262604, 0.262026, 0.261467, 0.260924, 0.260396,\
        0.259877, 0.259364, 0.258852, 0.258334, 0.257806, 0.257263, 0.256700, 0.256112, 0.255495, 0.254845,\
        0.254160, 0.253435, 0.252666, 0.251849, 0.250978, 0.250048, 0.249049, 0.247974, 0.246813, 0.245556,\
        0.244192, 0.242712, 0.241105, 0.239367, 0.237498, 0.235504, 0.233399, 0.231202, 0.228931, 0.226587,\
        0.224143, 0.221531, 0.218657, 0.215412, 0.211706, 0.207518, 0.202936, 0.198235, 0.193487, 0.188191,\
        0.182204],\
        [0.064454, 0.072345, 0.079956, 0.087298, 0.094381, 0.101217, 0.107821, 0.114208, 0.120394, 0.126394,\
        0.132224, 0.137899, 0.143434, 0.148839, 0.154126, 0.159306, 0.164384, 0.169368, 0.174256, 0.179042,\
        0.183721, 0.188313, 0.192892, 0.197601, 0.202205, 0.206003, 0.209377, 0.212495, 0.215456, 0.218306,\
        0.221058, 0.223708, 0.226246, 0.228659, 0.230938, 0.233079, 0.235086, 0.236967, 0.238732, 0.240391,\
        0.241958, 0.243442, 0.244853, 0.246200, 0.247490, 0.248732, 0.249929, 0.251088, 0.252212, 0.253303,\
        0.254363, 0.255393, 0.256390, 0.257352, 0.258274, 0.259147, 0.259961, 0.260705, 0.261366, 0.261930,\
        0.262388, 0.262730, 0.262955, 0.263065, 0.263063, 0.262957, 0.262755, 0.262468, 0.262104, 0.261675,\
        0.261193, 0.260672, 0.260126, 0.259568, 0.259007, 0.258452, 0.257907, 0.257374, 0.256853, 0.256343,\
        0.255840, 0.255341, 0.254840, 0.254334, 0.253817, 0.253285, 0.252732, 0.252154, 0.251548, 0.250908,\
        0.250231, 0.249513, 0.248748, 0.247932, 0.247059, 0.246121, 0.245110, 0.244017, 0.242832, 0.241543,\
        0.240138, 0.238603, 0.236928, 0.235103, 0.233126, 0.231005, 0.228759, 0.226420, 0.224025, 0.221598,\
        0.219122, 0.216517, 0.213663, 0.210418, 0.206640, 0.202316, 0.197537, 0.192604, 0.187511, 0.182009,\
        0.175947],\
        [0.056914, 0.064931, 0.072677, 0.080161, 0.087393, 0.094385, 0.101151, 0.107703, 0.114056, 0.120226,\
        0.126225, 0.132069, 0.137769, 0.143338, 0.148786, 0.154123, 0.159357, 0.164493, 0.169535, 0.174487,\
        0.179360, 0.184186, 0.189015, 0.193844, 0.198298, 0.202019, 0.205278, 0.208308, 0.211232, 0.214081,\
        0.216851, 0.219532, 0.222105, 0.224548, 0.226846, 0.228993, 0.230997, 0.232869, 0.234620, 0.236264,\
        0.237813, 0.239279, 0.240672, 0.241999, 0.243270, 0.244490, 0.245665, 0.246800, 0.247898, 0.248961,\
        0.249991, 0.250987, 0.251949, 0.252872, 0.253752, 0.254581, 0.255351, 0.256052, 0.256674, 0.257207,\
        0.257641, 0.257972, 0.258196, 0.258315, 0.258332, 0.258252, 0.258084, 0.257835, 0.257514, 0.257133,\
        0.256702, 0.256232, 0.255736, 0.255224, 0.254705, 0.254186, 0.253671, 0.253163, 0.252663, 0.252169,\
        0.251680, 0.251191, 0.250699, 0.250200, 0.249689, 0.249163, 0.248615, 0.248042, 0.247439, 0.246802,\
        0.246126, 0.245407, 0.244640, 0.243818, 0.242935, 0.241984, 0.240956, 0.239842, 0.238629, 0.237306,\
        0.235858, 0.234270, 0.232526, 0.230613, 0.228522, 0.226258, 0.223846, 0.221327, 0.218765, 0.216220,\
        0.213700, 0.211103, 0.208269, 0.205062, 0.201221, 0.196761, 0.191869, 0.186716, 0.181330, 0.175589,\
        0.169386],\
        [0.049184, 0.057326, 0.065204, 0.072827, 0.080206, 0.087349, 0.094271, 0.100982, 0.107497, 0.113829,\
        0.119991, 0.125996, 0.131856, 0.137581, 0.143183, 0.148670, 0.154050, 0.159330, 0.164516, 0.169614,\
        0.174636, 0.179595, 0.184482, 0.189207, 0.193539, 0.197317, 0.200612, 0.203660, 0.206633, 0.209534,\
        0.212359, 0.215103, 0.217739, 0.220230, 0.222556, 0.224715, 0.226719, 0.228583, 0.230324, 0.231956,\
        0.233493, 0.234946, 0.236325, 0.237638, 0.238893, 0.240096, 0.241253, 0.242368, 0.243443, 0.244482,\
        0.245485, 0.246452, 0.247382, 0.248270, 0.249113, 0.249904, 0.250636, 0.251301, 0.251890, 0.252396,\
        0.252811, 0.253132, 0.253356, 0.253483, 0.253517, 0.253461, 0.253323, 0.253110, 0.252830, 0.252491,\
        0.252106, 0.251682, 0.251230, 0.250759, 0.250278, 0.249791, 0.249304, 0.248819, 0.248337, 0.247857,\
        0.247378, 0.246897, 0.246411, 0.245916, 0.245407, 0.244881, 0.244333, 0.243759, 0.243153, 0.242512,\
        0.241831, 0.241104, 0.240327, 0.239492, 0.238594, 0.237624, 0.236574, 0.235434, 0.234191, 0.232833,\
        0.231343, 0.229704, 0.227897, 0.225899, 0.223696, 0.221285, 0.218687, 0.215956, 0.213170, 0.210448,\
        0.207864, 0.205276, 0.202368, 0.199133, 0.195234, 0.190688, 0.185714, 0.180418, 0.174823, 0.168881,\
        0.162528],\
        [0.041255, 0.049521, 0.057530, 0.065289, 0.072809, 0.080100, 0.087172, 0.094037, 0.100707, 0.107195,\
        0.113512, 0.119670, 0.125681, 0.131555, 0.137301, 0.142928, 0.148443, 0.153852, 0.159160, 0.164372,\
        0.169489, 0.174501, 0.179371, 0.184007, 0.188275, 0.192090, 0.195492, 0.198642, 0.201708, 0.204681,\
        0.207580, 0.210415, 0.213141, 0.215693, 0.218051, 0.220221, 0.222225, 0.224085, 0.225820, 0.227445,\
        0.228975, 0.230420, 0.231791, 0.233095, 0.234340, 0.235531, 0.236674, 0.237774, 0.238832, 0.239850,\
        0.240831, 0.241772, 0.242674, 0.243532, 0.244343, 0.245101, 0.245801, 0.246436, 0.246998, 0.247482,\
        0.247883, 0.248196, 0.248420, 0.248555, 0.248604, 0.248570, 0.248460, 0.248279, 0.248035, 0.247735,\
        0.247390, 0.247007, 0.246594, 0.246160, 0.245712, 0.245255, 0.244792, 0.244327, 0.243861, 0.243393,\
        0.242922, 0.242446, 0.241962, 0.241466, 0.240955, 0.240425, 0.239872, 0.239290, 0.238676, 0.238025,\
        0.237331, 0.236590, 0.235796, 0.234942, 0.234022, 0.233027, 0.231950, 0.230778, 0.229501, 0.228106,\
        0.226575, 0.224891, 0.223028, 0.220961, 0.218659, 0.216106, 0.213323, 0.210370, 0.207325, 0.204344,\
        0.201641, 0.199040, 0.196021, 0.192591, 0.188624, 0.184076, 0.179063, 0.173673, 0.167938, 0.161842,\
        0.155354],\
        [0.033121, 0.041509, 0.049646, 0.057539, 0.065196, 0.072629, 0.079845, 0.086857, 0.093676, 0.100312,\
        0.106777, 0.113081, 0.119235, 0.125247, 0.131127, 0.136882, 0.142518, 0.148039, 0.153449, 0.158744,\
        0.163919, 0.168952, 0.173798, 0.178388, 0.182643, 0.186522, 0.190058, 0.193354, 0.196508, 0.199554,\
        0.202525, 0.205476, 0.208312, 0.210926, 0.213308, 0.215485, 0.217489, 0.219348, 0.221083, 0.222708,\
        0.224237, 0.225682, 0.227051, 0.228352, 0.229592, 0.230778, 0.231913, 0.233002, 0.234047, 0.235050,\
        0.236013, 0.236934, 0.237812, 0.238645, 0.239430, 0.240161, 0.240835, 0.241445, 0.241985, 0.242452,\
        0.242841, 0.243148, 0.243373, 0.243516, 0.243579, 0.243566, 0.243480, 0.243328, 0.243116, 0.242851,\
        0.242542, 0.242194, 0.241816, 0.241415, 0.240995, 0.240563, 0.240122, 0.239674, 0.239221, 0.238762,\
        0.238296, 0.237821, 0.237336, 0.236837, 0.236320, 0.235781, 0.235217, 0.234624, 0.233995, 0.233327,\
        0.232615, 0.231853, 0.231035, 0.230155, 0.229206, 0.228180, 0.227068, 0.225860, 0.224544, 0.223108,\
        0.221535, 0.219806, 0.217898, 0.215778, 0.213404, 0.210732, 0.207775, 0.204637, 0.201394, 0.198168,\
        0.195274, 0.192428, 0.189273, 0.185683, 0.181606, 0.177018, 0.171967, 0.166507, 0.160668, 0.154455,\
        0.147852],\
        [0.024775, 0.033284, 0.041545, 0.049568, 0.057359, 0.064928, 0.072283, 0.079436, 0.086396, 0.093174,\
        0.099778, 0.106219, 0.112507, 0.118649, 0.124653, 0.130524, 0.136268, 0.141886, 0.147378, 0.152738,\
        0.157954, 0.163001, 0.167840, 0.172423, 0.176703, 0.180663, 0.184329, 0.187765, 0.191034, 0.194180,\
        0.197246, 0.200312, 0.203241, 0.205892, 0.208288, 0.210469, 0.212479, 0.214345, 0.216088, 0.217722,\
        0.219260, 0.220712, 0.222087, 0.223393, 0.224635, 0.225821, 0.226954, 0.228037, 0.229075, 0.230068,\
        0.231017, 0.231923, 0.232783, 0.233597, 0.234360, 0.235070, 0.235723, 0.236313, 0.236837, 0.237290,\
        0.237670, 0.237974, 0.238202, 0.238352, 0.238429, 0.238433, 0.238370, 0.238244, 0.238060, 0.237826,\
        0.237548, 0.237231, 0.236884, 0.236510, 0.236116, 0.235706, 0.235282, 0.234848, 0.234404, 0.233951,\
        0.233487, 0.233012, 0.232522, 0.232015, 0.231488, 0.230936, 0.230357, 0.229746, 0.229097, 0.228407,\
        0.227670, 0.226880, 0.226032, 0.225120, 0.224135, 0.223070, 0.221916, 0.220664, 0.219302, 0.217818,\
        0.216197, 0.214421, 0.212468, 0.210309, 0.207895, 0.205142, 0.202026, 0.198730, 0.195384, 0.192047,\
        0.188854, 0.185671, 0.182265, 0.178490, 0.174278, 0.169607, 0.164491, 0.158959, 0.153033, 0.146721,\
        0.140021],\
        [0.016212, 0.024838, 0.033221, 0.041369, 0.049289, 0.056990, 0.064479, 0.071767, 0.078861, 0.085772,\
        0.092509, 0.099079, 0.105492, 0.111754, 0.117871, 0.123848, 0.129688, 0.135391, 0.140954, 0.146369,\
        0.151621, 0.156689, 0.161540, 0.166144, 0.170472, 0.174520, 0.178307, 0.181875, 0.185272, 0.188542,\
        0.191728, 0.194851, 0.197816, 0.200518, 0.202932, 0.205131, 0.207163, 0.209052, 0.210816, 0.212471,\
        0.214027, 0.215496, 0.216885, 0.218202, 0.219454, 0.220646, 0.221783, 0.222867, 0.223903, 0.224891,\
        0.225832, 0.226727, 0.227575, 0.228374, 0.229122, 0.229816, 0.230453, 0.231029, 0.231540, 0.231984,\
        0.232358, 0.232661, 0.232891, 0.233050, 0.233138, 0.233159, 0.233116, 0.233013, 0.232855, 0.232648,\
        0.232396, 0.232107, 0.231785, 0.231435, 0.231062, 0.230670, 0.230261, 0.229837, 0.229400, 0.228949,\
        0.228484, 0.228004, 0.227506, 0.226988, 0.226447, 0.225878, 0.225279, 0.224645, 0.223971, 0.223253,\
        0.222485, 0.221661, 0.220776, 0.219824, 0.218796, 0.217684, 0.216481, 0.215177, 0.213760, 0.212218,\
        0.210538, 0.208703, 0.206693, 0.204480, 0.202026, 0.199233, 0.196006, 0.192600, 0.189137, 0.185669,\
        0.182224, 0.178731, 0.175048, 0.171057, 0.166682, 0.161893, 0.156686, 0.151069, 0.145057, 0.138655,\
        0.131865]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=25 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax25vt10
    zgrid_H_Hmax_vmax25vt10=[[0.4251768, 0.4435350, 0.4632455, 0.4842761, 0.5062616, 0.5278203, 0.5476513, 0.5658983, 0.5836916,\
        0.6023224, 0.6225450, 0.6426040, 0.6616583, 0.6797995, 0.6978690, 0.7175958, 0.7390146, 0.7594391,\
        0.7780984, 0.7942636, 0.8060250, 0.8108608, 0.8141195, 0.8168870, 0.8189376, 0.8202298, 0.8217602,\
        0.8233277, 0.8232444, 0.8225136, 0.8200891, 0.8130003, 0.8033383, 0.7912267, 0.7784540, 0.7660847,\
        0.7542822, 0.7428123, 0.7312558, 0.7192000, 0.7064990, 0.6925982, 0.6773341, 0.6613042, 0.6450611,\
        0.6292763, 0.6147331, 0.6025623, 0.5952135, 0.5910437, 0.5891833, 0.5899439, 0.5935657, 0.6006734,\
        0.6126465, 0.6273015, 0.6432058, 0.6593374, 0.6748116, 0.6885781, 0.6987392, 0.7060737, 0.7116033,\
        0.7157386, 0.7187010, 0.7206476, 0.7217064, 0.7219878, 0.7215891, 0.7205984, 0.7190969, 0.7171612,\
        0.7148653, 0.7122828, 0.7094892, 0.7065649, 0.7036000, 0.7007024, 0.6980144, 0.6957734, 0.6942235,\
        0.6934124, 0.6929435, 0.6926159, 0.6924393, 0.6923045, 0.6922913, 0.6923953, 0.6922250, 0.6915134,\
        0.6900880, 0.6878754, 0.6851341, 0.6821147, 0.6788960, 0.6754443, 0.6716847, 0.6675514, 0.6630094,\
        0.6580528, 0.6526921, 0.6469449, 0.6408323, 0.6343808, 0.6276239, 0.6206034, 0.6133693, 0.6059780,\
        0.5985404, 0.5911336, 0.5830824, 0.5743814, 0.5651888, 0.5556182, 0.5457778, 0.5357704, 0.5256895,\
        0.5156067, 0.5055539, 0.4952666, 0.4845478],\
        [0.4084425, 0.4257027, 0.4444111, 0.4649722, 0.4871326, 0.5089619, 0.5286123, 0.5457001, 0.5625902,\
        0.5814052, 0.6023752, 0.6230897, 0.6425978, 0.6606954, 0.6787366, 0.6990804, 0.7206625, 0.7407236,\
        0.7591705, 0.7751961, 0.7867482, 0.7920127, 0.7956143, 0.7984615, 0.8002333, 0.8005076, 0.8011925,\
        0.8030488, 0.8009829, 0.8002978, 0.7988501, 0.7931074, 0.7824736, 0.7717599, 0.7609773, 0.7509588,\
        0.7417249, 0.7328687, 0.7237712, 0.7137374, 0.7021779, 0.6899719, 0.6759138, 0.6605735, 0.6448192,\
        0.6295087, 0.6156146, 0.6045176, 0.5970687, 0.5919458, 0.5896532, 0.5905626, 0.5950510, 0.6030913,\
        0.6152309, 0.6303911, 0.6469084, 0.6635534, 0.6793565, 0.6936157, 0.7043930, 0.7114572, 0.7170163,\
        0.7212186, 0.7242326, 0.7262120, 0.7272895, 0.7275797, 0.7271833, 0.7261902, 0.7246827, 0.7227380,\
        0.7204300, 0.7178315, 0.7150169, 0.7120645, 0.7090610, 0.7061084, 0.7033377, 0.7010182, 0.6993289,\
        0.6987730, 0.6983305, 0.6979484, 0.6978375, 0.6976069, 0.6975278, 0.6978923, 0.6978311, 0.6971845,\
        0.6957632, 0.6932322, 0.6902487, 0.6871450, 0.6839446, 0.6805433, 0.6768093, 0.6726476, 0.6680254,\
        0.6629534, 0.6574551, 0.6515534, 0.6452706, 0.6386347, 0.6316838, 0.6244717, 0.6170736, 0.6096006,\
        0.6022726, 0.5947689, 0.5861964, 0.5771211, 0.5675503, 0.5575935, 0.5473705, 0.5369903, 0.5265596,\
        0.5161764, 0.5058120, 0.4955487, 0.4851794],\
        [0.3918193, 0.4074701, 0.4242539, 0.4434427, 0.4648209, 0.4868680, 0.5064492, 0.5222771, 0.5381378,\
        0.5570765, 0.5784933, 0.6007206, 0.6214959, 0.6396687, 0.6579981, 0.6783356, 0.6993449, 0.7193010,\
        0.7373552, 0.7530132, 0.7647310, 0.7721283, 0.7767798, 0.7796720, 0.7808398, 0.7803472, 0.7794380,\
        0.7781748, 0.7749838, 0.7723287, 0.7698638, 0.7650651, 0.7574786, 0.7493786, 0.7417067, 0.7349091,\
        0.7289331, 0.7232668, 0.7171893, 0.7099807, 0.7010372, 0.6895059, 0.6760590, 0.6613219, 0.6461360,\
        0.6314412, 0.6182788, 0.6076889, 0.5998227, 0.5939616, 0.5913422, 0.5924261, 0.5973455, 0.6065613,\
        0.6193044, 0.6347288, 0.6514815, 0.6681614, 0.6834277, 0.6963878, 0.7072974, 0.7156699, 0.7218023,\
        0.7263057, 0.7295032, 0.7316057, 0.7327723, 0.7331316, 0.7327924, 0.7318495, 0.7303883, 0.7284879,\
        0.7262236, 0.7236695, 0.7209008, 0.7179970, 0.7150469, 0.7121573, 0.7094727, 0.7071986, 0.7055675,\
        0.7047920, 0.7042671, 0.7038537, 0.7036550, 0.7033680, 0.7032398, 0.7035023, 0.7033849, 0.7026923,\
        0.7011556, 0.6983301, 0.6952205, 0.6921355, 0.6890321, 0.6857337, 0.6820408, 0.6778289, 0.6730910,\
        0.6678757, 0.6622224, 0.6561536, 0.6496873, 0.6428481, 0.6356753, 0.6282298, 0.6206005, 0.6129050,\
        0.6051852, 0.5971185, 0.5887435, 0.5794483, 0.5695153, 0.5591829, 0.5485845, 0.5378238, 0.5270076,\
        0.5162997, 0.5060752, 0.4960865, 0.4854230],\
        [0.3757793, 0.3892690, 0.4038420, 0.4208567, 0.4406072, 0.4608546, 0.4797547, 0.4953122, 0.5109582,\
        0.5301324, 0.5513482, 0.5743840, 0.5965847, 0.6154418, 0.6338646, 0.6536060, 0.6748265, 0.6955612,\
        0.7143294, 0.7305990, 0.7432969, 0.7525127, 0.7585043, 0.7617418, 0.7624519, 0.7609215, 0.7577025,\
        0.7530273, 0.7473781, 0.7426083, 0.7393542, 0.7360313, 0.7317607, 0.7271567, 0.7229510, 0.7196157,\
        0.7171103, 0.7147690, 0.7117512, 0.7072901, 0.7004629, 0.6901574, 0.6773601, 0.6631786, 0.6485061,\
        0.6342537, 0.6213279, 0.6104574, 0.6019033, 0.5959052, 0.5934041, 0.5950938, 0.6007089, 0.6105850,\
        0.6240254, 0.6398175, 0.6567510, 0.6735154, 0.6885511, 0.7007112, 0.7112590, 0.7198194, 0.7263318,\
        0.7311409, 0.7345660, 0.7368431, 0.7381508, 0.7386305, 0.7383993, 0.7375574, 0.7361939, 0.7343906,\
        0.7322254, 0.7297747, 0.7271161, 0.7243320, 0.7215145, 0.7187738, 0.7162484, 0.7141048, 0.7125042,\
        0.7114727, 0.7107838, 0.7102862, 0.7099274, 0.7096193, 0.7094267, 0.7093208, 0.7089394, 0.7079780,\
        0.7061559, 0.7034252, 0.7003085, 0.6972777, 0.6942836, 0.6910978, 0.6874274, 0.6830973, 0.6781815,\
        0.6727960, 0.6669794, 0.6607390, 0.6540802, 0.6470191, 0.6395905, 0.6318523, 0.6238864, 0.6157908,\
        0.6076599, 0.5997238, 0.5910953, 0.5813214, 0.5710444, 0.5603999, 0.5494876, 0.5384052, 0.5272405,\
        0.5160709, 0.5050057, 0.4949551, 0.4851263],\
        [0.3610728, 0.3730083, 0.3857137, 0.4007951, 0.4173938, 0.4348585, 0.4527950, 0.4691607, 0.4856404,\
        0.5036311, 0.5231813, 0.5456012, 0.5686746, 0.5893957, 0.6082624, 0.6276450, 0.6498465, 0.6723772,\
        0.6932471, 0.7097782, 0.7241737, 0.7350650, 0.7423230, 0.7461047, 0.7466052, 0.7441170, 0.7390034,\
        0.7316753, 0.7231899, 0.7165251, 0.7138321, 0.7123515, 0.7107945, 0.7092035, 0.7078325, 0.7072993,\
        0.7078086, 0.7082963, 0.7077429, 0.7053410, 0.7000743, 0.6911206, 0.6792513, 0.6657186, 0.6515248,\
        0.6375593, 0.6246210, 0.6133500, 0.6042205, 0.5978605, 0.5953819, 0.5978176, 0.6046976, 0.6153125,\
        0.6292652, 0.6453630, 0.6623773, 0.6791653, 0.6948170, 0.7065961, 0.7161874, 0.7243770, 0.7309173,\
        0.7358933, 0.7395085, 0.7419667, 0.7434418, 0.7440780, 0.7439961, 0.7432996, 0.7420801, 0.7404222,\
        0.7384061, 0.7361108, 0.7336163, 0.7310067, 0.7283736, 0.7258218, 0.7234705, 0.7214440, 0.7198363,\
        0.7186502, 0.7177863, 0.7171378, 0.7166297, 0.7162052, 0.7158301, 0.7154022, 0.7146944, 0.7134607,\
        0.7115269, 0.7089213, 0.7059338, 0.7028690, 0.6998588, 0.6966791, 0.6929816, 0.6883777, 0.6832228,\
        0.6776761, 0.6717090, 0.6653044, 0.6584513, 0.6511538, 0.6434407, 0.6353639, 0.6269933, 0.6184243,\
        0.6098489, 0.6015353, 0.5923947, 0.5825377, 0.5721344, 0.5612882, 0.5501491, 0.5388567, 0.5275343,\
        0.5162875, 0.5052812, 0.4950523, 0.4851142],\
        [0.3484629, 0.3595140, 0.3714747, 0.3848551, 0.3968267, 0.4133966, 0.4303382, 0.4474755, 0.4642415,\
        0.4809570, 0.4982058, 0.5205204, 0.5446337, 0.5662517, 0.5852247, 0.6044572, 0.6286767, 0.6531756,\
        0.6752405, 0.6938401, 0.7093713, 0.7213028, 0.7296675, 0.7340529, 0.7346327, 0.7317685, 0.7259067,\
        0.7176431, 0.7080876, 0.7005050, 0.7002487, 0.7002030, 0.7000907, 0.7000871, 0.7000785, 0.7005200,\
        0.7027362, 0.7047783, 0.7056205, 0.7044967, 0.7004987, 0.6925228, 0.6815366, 0.6686989, 0.6549968,\
        0.6413047, 0.6283859, 0.6168856, 0.6073889, 0.6006122, 0.5975034, 0.6002205, 0.6084085, 0.6202895,\
        0.6349001, 0.6512499, 0.6681697, 0.6843282, 0.6984654, 0.7110197, 0.7209590, 0.7290615, 0.7355803,\
        0.7406384, 0.7443940, 0.7470173, 0.7486671, 0.7494817, 0.7495799, 0.7490648, 0.7480286, 0.7465566,\
        0.7447312, 0.7426332, 0.7403438, 0.7379460, 0.7355263, 0.7331769, 0.7309936, 0.7290654, 0.7274521,\
        0.7261591, 0.7251387, 0.7243224, 0.7236445, 0.7230436, 0.7224481, 0.7217432, 0.7207611, 0.7193290,\
        0.7173412, 0.7148249, 0.7119409, 0.7088752, 0.7057126, 0.7023847, 0.6985449, 0.6933932, 0.6881298,\
        0.6824980, 0.6764041, 0.6698522, 0.6628083, 0.6552624, 0.6472452, 0.6388123, 0.6300244, 0.6209283,\
        0.6115133, 0.6017585, 0.5929810, 0.5834289, 0.5729525, 0.5619279, 0.5506046, 0.5391764, 0.5278360,\
        0.5168217, 0.5063866, 0.4961167, 0.4854393],\
        [0.3379757, 0.3484667, 0.3596285, 0.3714430, 0.3840167, 0.3975424, 0.4140959, 0.4317067, 0.4487596,\
        0.4657063, 0.4837281, 0.5053753, 0.5288466, 0.5503352, 0.5700660, 0.5906003, 0.6147212, 0.6400862,\
        0.6633142, 0.6831746, 0.6990590, 0.7122933, 0.7214483, 0.7263254, 0.7272344, 0.7246870, 0.7193624,\
        0.7122576, 0.7051724, 0.7013640, 0.7009333, 0.7011272, 0.7010165, 0.7009726, 0.7009783, 0.7011698,\
        0.7026271, 0.7045279, 0.7055648, 0.7047995, 0.7012941, 0.6942776, 0.6841681, 0.6720315, 0.6588577,\
        0.6455227, 0.6328020, 0.6213793, 0.6118972, 0.6050499, 0.6019022, 0.6044964, 0.6135094, 0.6260774,\
        0.6410608, 0.6575039, 0.6743286, 0.6902782, 0.7039946, 0.7156975, 0.7256498, 0.7337628, 0.7402817,\
        0.7453913, 0.7492526, 0.7520217, 0.7538440, 0.7548489, 0.7551490, 0.7548433, 0.7540214, 0.7527676,\
        0.7511649, 0.7492952, 0.7472396, 0.7450778, 0.7428873, 0.7407443, 0.7387215, 0.7368824, 0.7352692,\
        0.7338927, 0.7327321, 0.7317468, 0.7308862, 0.7300896, 0.7292776, 0.7283419, 0.7271494, 0.7255733,\
        0.7235369, 0.7210454, 0.7181764, 0.7150203, 0.7116031, 0.7078132, 0.7032160, 0.6980260, 0.6930615,\
        0.6872847, 0.6810677, 0.6743985, 0.6671627, 0.6593458, 0.6510030, 0.6422142, 0.6330606, 0.6236201,\
        0.6140021, 0.6044999, 0.5948918, 0.5845590, 0.5736718, 0.5623745, 0.5508482, 0.5392698, 0.5278279,\
        0.5167495, 0.5062383, 0.4958773, 0.4851412],\
        [0.3293886, 0.3395353, 0.3502781, 0.3617993, 0.3743761, 0.3885438, 0.4049212, 0.4217381, 0.4389564,\
        0.4571120, 0.4771779, 0.4994376, 0.5209144, 0.5415085, 0.5621333, 0.5840257, 0.6081873, 0.6331607,\
        0.6568438, 0.6776796, 0.6949567, 0.7083520, 0.7176484, 0.7228152, 0.7242005, 0.7224561, 0.7184361,\
        0.7133102, 0.7086921, 0.7061124, 0.7054098, 0.7053282, 0.7052073, 0.7050272, 0.7048917, 0.7050394,\
        0.7058188, 0.7068869, 0.7073562, 0.7063316, 0.7029968, 0.6965915, 0.6871604, 0.6756813, 0.6630626,\
        0.6501914, 0.6378811, 0.6268677, 0.6178692, 0.6116994, 0.6094458, 0.6124144, 0.6207653, 0.6330263,\
        0.6478232, 0.6640364, 0.6806029, 0.6963501, 0.7097748, 0.7209845, 0.7305589, 0.7385406, 0.7450240,\
        0.7501577, 0.7540974, 0.7569932, 0.7589817, 0.7601828, 0.7607004, 0.7606256, 0.7600419, 0.7590302,\
        0.7576726, 0.7560522, 0.7542499, 0.7523405, 0.7503902, 0.7484570, 0.7465918, 0.7448381, 0.7432281,\
        0.7417779, 0.7404859, 0.7393342, 0.7382893, 0.7373004, 0.7362937, 0.7351710, 0.7338190, 0.7321311,\
        0.7300352, 0.7275099, 0.7245748, 0.7212542, 0.7175270, 0.7132894, 0.7084492, 0.7034443, 0.6980982,\
        0.6919327, 0.6857306, 0.6789887, 0.6715226, 0.6633776, 0.6546664, 0.6455061, 0.6360084, 0.6262904,\
        0.6164917, 0.6066830, 0.5964554, 0.5855321, 0.5741976, 0.5626021, 0.5508650, 0.5391125, 0.5274667,\
        0.5160265, 0.5049047, 0.4944146, 0.4841826],\
        [0.3224094, 0.3323841, 0.3430010, 0.3545792, 0.3675080, 0.3821972, 0.3983974, 0.4149053, 0.4322821,\
        0.4512559, 0.4722749, 0.4945877, 0.5153650, 0.5362354, 0.5578348, 0.5807557, 0.6051058, 0.6298871,\
        0.6539299, 0.6758444, 0.6939195, 0.7076728, 0.7171885, 0.7225529, 0.7245208, 0.7238293, 0.7213323,\
        0.7180506, 0.7150835, 0.7131978, 0.7123535, 0.7119848, 0.7116609, 0.7112745, 0.7109046, 0.7107150,\
        0.7108032, 0.7109325, 0.7105289, 0.7088960, 0.7053466, 0.6992540, 0.6904453, 0.6795897, 0.6675101,\
        0.6551816, 0.6434526, 0.6330895, 0.6248525, 0.6195662, 0.6181344, 0.6213051, 0.6291071, 0.6407153,\
        0.6549766, 0.6706728, 0.6865586, 0.7017003, 0.7149003, 0.7260747, 0.7355182, 0.7433910, 0.7498175,\
        0.7549456, 0.7589348, 0.7619363, 0.7640819, 0.7654819, 0.7662286, 0.7664019, 0.7660752, 0.7653225,\
        0.7642240, 0.7628651, 0.7613284, 0.7596843, 0.7579857, 0.7562700, 0.7545648, 0.7528957, 0.7512879,\
        0.7497646, 0.7483417, 0.7470237, 0.7457977, 0.7446287, 0.7434544, 0.7421850, 0.7407136, 0.7389359,\
        0.7367709, 0.7341725, 0.7311238, 0.7276156, 0.7236234, 0.7191110, 0.7141006, 0.7086739, 0.7026015,\
        0.6966169, 0.6906011, 0.6836911, 0.6758578, 0.6672807, 0.6581360, 0.6485614, 0.6386749, 0.6285850,\
        0.6183762, 0.6080318, 0.5973026, 0.5860052, 0.5743778, 0.5625596, 0.5506556, 0.5387726, 0.5270053,\
        0.5154026, 0.5039973, 0.4934716, 0.4833716],\
        [0.3166878, 0.3265380, 0.3370592, 0.3486359, 0.3616276, 0.3762097, 0.3920743, 0.4086322, 0.4262677,\
        0.4453894, 0.4661474, 0.4877435, 0.5096485, 0.5316952, 0.5542160, 0.5778083, 0.6023555, 0.6273360,\
        0.6516653, 0.6747623, 0.6953107, 0.7091647, 0.7182529, 0.7239360, 0.7267249, 0.7272871, 0.7263632,\
        0.7247271, 0.7230815, 0.7218576, 0.7210782, 0.7204973, 0.7198814, 0.7191518, 0.7183580, 0.7175915,\
        0.7168666, 0.7160028, 0.7146129, 0.7121930, 0.7082153, 0.7021978, 0.6940915, 0.6836121, 0.6719961,\
        0.6602720, 0.6492345, 0.6396304, 0.6322028, 0.6277045, 0.6268429, 0.6300980, 0.6374765, 0.6484509,\
        0.6621301, 0.6773756, 0.6926883, 0.7068980, 0.7199410, 0.7311199, 0.7405335, 0.7483346, 0.7546855,\
        0.7597684, 0.7637683, 0.7668488, 0.7691395, 0.7707388, 0.7717244, 0.7721618, 0.7721090, 0.7716270,\
        0.7707935, 0.7697005, 0.7684367, 0.7670702, 0.7656396, 0.7641559, 0.7626188, 0.7610348, 0.7594239,\
        0.7578189, 0.7562568, 0.7547664, 0.7533598, 0.7520236, 0.7507092, 0.7493310, 0.7477767, 0.7459299,\
        0.7436928, 0.7409988, 0.7378116, 0.7341123, 0.7298896, 0.7251438, 0.7199009, 0.7141957, 0.7081370,\
        0.7021401, 0.6958246, 0.6884288, 0.6800169, 0.6709072, 0.6612780, 0.6512482, 0.6409198, 0.6303735,\
        0.6196461, 0.6087097, 0.5974988, 0.5859812, 0.5741971, 0.5622395, 0.5502256, 0.5382788, 0.5265310,\
        0.5151368, 0.5042225, 0.4935440, 0.4829555],\
        [0.3118558, 0.3215061, 0.3317902, 0.3431431, 0.3558848, 0.3701115, 0.3856193, 0.4021297, 0.4197354,\
        0.4384449, 0.4589905, 0.4810543, 0.5037215, 0.5263216, 0.5492191, 0.5733150, 0.5985407, 0.6230437,\
        0.6471135, 0.6706121, 0.6916942, 0.7076321, 0.7182955, 0.7251732, 0.7292905, 0.7314120, 0.7321962,\
        0.7322020, 0.7318594, 0.7314112, 0.7308933, 0.7302155, 0.7292901, 0.7281072, 0.7267257, 0.7252131,\
        0.7235687, 0.7216636, 0.7192352, 0.7159392, 0.7114355, 0.7055053, 0.6975176, 0.6872560, 0.6762469,\
        0.6652331, 0.6549503, 0.6461157, 0.6394262, 0.6355464, 0.6350475, 0.6382870, 0.6452833, 0.6556752,\
        0.6687860, 0.6836793, 0.6990633, 0.7128433, 0.7253521, 0.7364045, 0.7457631, 0.7534627, 0.7596786,\
        0.7646457, 0.7685974, 0.7717208, 0.7741422, 0.7759402, 0.7771731, 0.7778918, 0.7781332, 0.7779323,\
        0.7773620, 0.7765319, 0.7755442, 0.7744686, 0.7733302, 0.7721025, 0.7707466, 0.7692487, 0.7676236,\
        0.7659193, 0.7642003, 0.7625230, 0.7609284, 0.7594323, 0.7580044, 0.7565570, 0.7549586, 0.7530661,\
        0.7507580, 0.7479519, 0.7446044, 0.7407002, 0.7362437, 0.7312583, 0.7257910, 0.7199224, 0.7137970,\
        0.7075337, 0.7008247, 0.6927353, 0.6836873, 0.6740642, 0.6639581, 0.6534630, 0.6426752, 0.6316651,\
        0.6204518, 0.6089980, 0.5973589, 0.5856734, 0.5737505, 0.5616746, 0.5495753, 0.5375814, 0.5258330,\
        0.5145134, 0.5038432, 0.4931626, 0.4823697],\
        [0.3075685, 0.3168583, 0.3266978, 0.3376108, 0.3498796, 0.3635961, 0.3787513, 0.3953038, 0.4117132,\
        0.4295852, 0.4500028, 0.4724750, 0.4959452, 0.5187375, 0.5410661, 0.5650173, 0.5901313, 0.6145595,\
        0.6384960, 0.6624535, 0.6847961, 0.7031394, 0.7161052, 0.7249915, 0.7309966, 0.7350717, 0.7378350,\
        0.7396647, 0.7407846, 0.7412991, 0.7412288, 0.7405695, 0.7393489, 0.7376469, 0.7355714, 0.7332102,\
        0.7305828, 0.7276107, 0.7241192, 0.7198698, 0.7145957, 0.7079808, 0.6997454, 0.6903877, 0.6801894,\
        0.6699438, 0.6604209, 0.6523088, 0.6462458, 0.6428202, 0.6425224, 0.6456617, 0.6522822, 0.6621202,\
        0.6746215, 0.6889967, 0.7039779, 0.7181478, 0.7309306, 0.7420759, 0.7513886, 0.7589060, 0.7648657,\
        0.7695963, 0.7734104, 0.7765311, 0.7790705, 0.7810673, 0.7825502, 0.7835684, 0.7841393, 0.7842342,\
        0.7839159, 0.7833427, 0.7826330, 0.7818579, 0.7810474, 0.7801111, 0.7789495, 0.7775407, 0.7758836,\
        0.7740520, 0.7721533, 0.7702661, 0.7684624, 0.7668037, 0.7652876, 0.7638171, 0.7622207, 0.7603087,\
        0.7579285, 0.7549878, 0.7514473, 0.7473034, 0.7425759, 0.7373049, 0.7315511, 0.7253943, 0.7189095,\
        0.7120663, 0.7045541, 0.6959784, 0.6866231, 0.6766286, 0.6660864, 0.6551341, 0.6438984, 0.6324685,\
        0.6208906, 0.6091661, 0.5972842, 0.5852620, 0.5731052, 0.5608897, 0.5487098, 0.5366661, 0.5248552,\
        0.5133242, 0.5021075, 0.4917256, 0.4813711],\
        [0.3034933, 0.3121958, 0.3214327, 0.3318044, 0.3434447, 0.3563615, 0.3704857, 0.3854563, 0.4008968,\
        0.4184860, 0.4387401, 0.4612359, 0.4849289, 0.5068831, 0.5287554, 0.5526491, 0.5772484, 0.6014555,\
        0.6257595, 0.6509044, 0.6749576, 0.6956703, 0.7114825, 0.7227510, 0.7310220, 0.7374701, 0.7425860,\
        0.7465468, 0.7493907, 0.7510942, 0.7516447, 0.7510995, 0.7496012, 0.7473449, 0.7445218, 0.7412708,\
        0.7376491, 0.7336244, 0.7290882, 0.7238810, 0.7178217, 0.7107472, 0.7026463, 0.6936121, 0.6839624,\
        0.6743961, 0.6655853, 0.6581257, 0.6525751, 0.6494625, 0.6492425, 0.6522221, 0.6584928, 0.6678743,\
        0.6798455, 0.6936861, 0.7087716, 0.7235557, 0.7369533, 0.7483854, 0.7576337, 0.7648176, 0.7703189,\
        0.7746252, 0.7781742, 0.7812403, 0.7838986, 0.7861014, 0.7878183, 0.7891370, 0.7901150, 0.7905360,\
        0.7904413, 0.7901360, 0.7897175, 0.7892235, 0.7887939, 0.7881972, 0.7872177, 0.7859217, 0.7842010,\
        0.7822025, 0.7801140, 0.7779886, 0.7759314, 0.7740903, 0.7725122, 0.7710777, 0.7695406, 0.7676343,\
        0.7651687, 0.7620543, 0.7582708, 0.7538319, 0.7487704, 0.7431351, 0.7369880, 0.7303910, 0.7233698,\
        0.7158463, 0.7076034, 0.6985694, 0.6889867, 0.6786257, 0.6676244, 0.6562091, 0.6445398, 0.6327239,\
        0.6208383, 0.6089490, 0.5969235, 0.5845657, 0.5721947, 0.5598704, 0.5476402, 0.5355778, 0.5237836,\
        0.5123721, 0.5013750, 0.4908232, 0.4803854],\
        [0.2990897, 0.3070377, 0.3158559, 0.3257897, 0.3367604, 0.3487063, 0.3615346, 0.3750930, 0.3896297,\
        0.4063347, 0.4259370, 0.4477040, 0.4699893, 0.4913715, 0.5142539, 0.5383990, 0.5625175, 0.5861942,\
        0.6109825, 0.6374954, 0.6632064, 0.6860913, 0.7046326, 0.7180210, 0.7288477, 0.7381475, 0.7460359,\
        0.7524878, 0.7573658, 0.7604991, 0.7618175, 0.7614419, 0.7596624, 0.7568302, 0.7532502, 0.7491265,\
        0.7445560, 0.7395442, 0.7340329, 0.7279326, 0.7211601, 0.7136867, 0.7055596, 0.6967676, 0.6875484,\
        0.6786006, 0.6704769, 0.6636148, 0.6584755, 0.6555577, 0.6553243, 0.6581158, 0.6640924, 0.6732148,\
        0.6851825, 0.6989713, 0.7143501, 0.7299098, 0.7439900, 0.7557211, 0.7647693, 0.7713598, 0.7761021,\
        0.7797162, 0.7828225, 0.7857808, 0.7885912, 0.7910353, 0.7929530, 0.7944637, 0.7960173, 0.7968263,\
        0.7969203, 0.7969644, 0.7968900, 0.7965960, 0.7965748, 0.7963602, 0.7954857, 0.7944192, 0.7925452,\
        0.7903450, 0.7881212, 0.7857189, 0.7833228, 0.7812570, 0.7796460, 0.7783261, 0.7769193, 0.7750323,\
        0.7724389, 0.7690857, 0.7649922, 0.7601908, 0.7547202, 0.7486315, 0.7419859, 0.7348399, 0.7272196,\
        0.7190984, 0.7104086, 0.7010860, 0.6910344, 0.6800583, 0.6685168, 0.6566380, 0.6445572, 0.6323725,\
        0.6201477, 0.6078885, 0.5956350, 0.5833169, 0.5709307, 0.5585979, 0.5463797, 0.5343367, 0.5225648,\
        0.5112864, 0.5008020, 0.4900145, 0.4793826],\
        [0.2938598, 0.3016600, 0.3103457, 0.3199908, 0.3302887, 0.3412990, 0.3530492, 0.3655906, 0.3792496,\
        0.3947699, 0.4133392, 0.4343533, 0.4560080, 0.4779335, 0.5008563, 0.5248000, 0.5487182, 0.5725396,\
        0.5973344, 0.6242650, 0.6507086, 0.6746093, 0.6945619, 0.7102336, 0.7244304, 0.7370291, 0.7480074,\
        0.7572836, 0.7645214, 0.7693257, 0.7715143, 0.7712949, 0.7691867, 0.7657626, 0.7614610, 0.7565440,\
        0.7511322, 0.7452527, 0.7388813, 0.7319794, 0.7245323, 0.7165849, 0.7082516, 0.6996294, 0.6908466,\
        0.6825945, 0.6752293, 0.6689386, 0.6641124, 0.6612832, 0.6609761, 0.6635939, 0.6693500, 0.6782849,\
        0.6904049, 0.7053582, 0.7213471, 0.7376185, 0.7524282, 0.7644255, 0.7730410, 0.7786698, 0.7822726,\
        0.7848526, 0.7872615, 0.7900386, 0.7930926, 0.7958231, 0.7980537, 0.7994459, 0.8015432, 0.8029294,\
        0.8034975, 0.8039655, 0.8042642, 0.8042815, 0.8044757, 0.8044798, 0.8038177, 0.8028657, 0.8008360,\
        0.7984977, 0.7963039, 0.7935019, 0.7906589, 0.7883122, 0.7866975, 0.7855838, 0.7843858, 0.7824914,\
        0.7796725, 0.7759872, 0.7715124, 0.7662839, 0.7603320, 0.7537056, 0.7464710, 0.7386937, 0.7304214,\
        0.7216831, 0.7125292, 0.7030412, 0.6922953, 0.6806557, 0.6686542, 0.6563835, 0.6439454, 0.6314364,\
        0.6189342, 0.6064770, 0.5940497, 0.5816904, 0.5693555, 0.5570926, 0.5449578, 0.5329933, 0.5212198,\
        0.5096290, 0.4985532, 0.4884804, 0.4781839],\
        [0.2897098, 0.2973366, 0.3059559, 0.3149674, 0.3244498, 0.3346248, 0.3455643, 0.3574001, 0.3704593,\
        0.3853584, 0.4030276, 0.4228329, 0.4439307, 0.4660048, 0.4890296, 0.5127032, 0.5365645, 0.5606873,\
        0.5858872, 0.6123447, 0.6383747, 0.6622125, 0.6829389, 0.7013290, 0.7187019, 0.7342922, 0.7484046,\
        0.7607813, 0.7707283, 0.7774576, 0.7805503, 0.7803608, 0.7778174, 0.7738065, 0.7688841, 0.7633274,\
        0.7572482, 0.7506747, 0.7436011, 0.7360219, 0.7279600, 0.7194939, 0.7107882, 0.7021241, 0.6938109,\
        0.6865695, 0.6801265, 0.6743478, 0.6696997, 0.6668481, 0.6664398, 0.6689781, 0.6747168, 0.6836537,\
        0.6957160, 0.7113935, 0.7289945, 0.7464884, 0.7622936, 0.7746562, 0.7825642, 0.7868068, 0.7889252,\
        0.7901264, 0.7914550, 0.7938097, 0.7973023, 0.8003047, 0.8028167, 0.8048759, 0.8070549, 0.8089474,\
        0.8102537, 0.8112260, 0.8119393, 0.8124080, 0.8127534, 0.8127954, 0.8122936, 0.8111833, 0.8093058,\
        0.8070071, 0.8045063, 0.8013619, 0.7981013, 0.7954238, 0.7937939, 0.7929067, 0.7919764, 0.7899437,\
        0.7867256, 0.7826168, 0.7777167, 0.7720179, 0.7655240, 0.7582856, 0.7503848, 0.7419072, 0.7329178,\
        0.7234460, 0.7134652, 0.7028805, 0.6918912, 0.6802358, 0.6680181, 0.6554713, 0.6427503, 0.6299728,\
        0.6172476, 0.6046869, 0.5922245, 0.5798236, 0.5675436, 0.5553867, 0.5433940, 0.5316139, 0.5200785,\
        0.5088078, 0.4979264, 0.4874546, 0.4770428],\
        [0.2865768, 0.2947394, 0.3027372, 0.3106252, 0.3193360, 0.3288614, 0.3392258, 0.3505886, 0.3632642,\
        0.3777305, 0.3943674, 0.4129425, 0.4334305, 0.4553050, 0.4782002, 0.5018105, 0.5253115, 0.5494604,\
        0.5748461, 0.6009910, 0.6264376, 0.6500073, 0.6718821, 0.6925529, 0.7117029, 0.7297364, 0.7469904,\
        0.7627580, 0.7758213, 0.7847882, 0.7887211, 0.7882502, 0.7851331, 0.7806264, 0.7752904, 0.7693347,\
        0.7628259, 0.7557792, 0.7482025, 0.7401185, 0.7315810, 0.7226885, 0.7136030, 0.7046276, 0.6968480,\
        0.6911328, 0.6855946, 0.6800768, 0.6754135, 0.6724258, 0.6719142, 0.6745547, 0.6806865, 0.6902538,\
        0.7032401, 0.7194722, 0.7376634, 0.7561141, 0.7731105, 0.7862173, 0.7931548, 0.7956974, 0.7963128,\
        0.7959932, 0.7958398, 0.7968668, 0.8006439, 0.8043205, 0.8075031, 0.8103617, 0.8130009, 0.8153360,\
        0.8172372, 0.8187444, 0.8199111, 0.8207623, 0.8212968, 0.8214085, 0.8209508, 0.8198273, 0.8180210,\
        0.8156707, 0.8128569, 0.8095789, 0.8061575, 0.8032216, 0.8013702, 0.8003565, 0.7995457, 0.7970829,\
        0.7933250, 0.7887993, 0.7834954, 0.7773113, 0.7702238, 0.7623043, 0.7536721, 0.7444452, 0.7347123,\
        0.7245181, 0.7138430, 0.7026308, 0.6911680, 0.6792303, 0.6667763, 0.6540040, 0.6410671, 0.6280838,\
        0.6151496, 0.6023707, 0.5899712, 0.5777133, 0.5655354, 0.5534892, 0.5416456, 0.5300843, 0.5188842,\
        0.5081159, 0.4975412, 0.4866077, 0.4758878],\
        [0.2836967, 0.2914410, 0.2988833, 0.3065073, 0.3149126, 0.3240199, 0.3339798, 0.3449906, 0.3573052,\
        0.3711939, 0.3867923, 0.4043873, 0.4243438, 0.4455566, 0.4677355, 0.4906004, 0.5136140, 0.5377084,\
        0.5631570, 0.5893752, 0.6140638, 0.6373388, 0.6599012, 0.6815253, 0.7023220, 0.7228802, 0.7434455,\
        0.7628843, 0.7794494, 0.7910688, 0.7956543, 0.7943226, 0.7906520, 0.7859520, 0.7805478, 0.7745116,\
        0.7678523, 0.7605778, 0.7527184, 0.7443374, 0.7355387, 0.7264798, 0.7174061, 0.7087850, 0.7017925,\
        0.6971364, 0.6917329, 0.6860932, 0.6813151, 0.6781196, 0.6775043, 0.6804499, 0.6874805, 0.6983290,\
        0.7123600, 0.7289286, 0.7469948, 0.7655990, 0.7834287, 0.7976760, 0.8039157, 0.8053269, 0.8049395,\
        0.8035113, 0.8020475, 0.8011580, 0.8042264, 0.8087988, 0.8127942, 0.8162969, 0.8194251, 0.8221934,\
        0.8245674, 0.8265423, 0.8281266, 0.8293121, 0.8300584, 0.8302789, 0.8298716, 0.8287679, 0.8269687,\
        0.8245485, 0.8216034, 0.8182764, 0.8148767, 0.8118650, 0.8095920, 0.8079569, 0.8062493, 0.8033464,\
        0.7992500, 0.7944304, 0.7887854, 0.7821013, 0.7743569, 0.7656843, 0.7562666, 0.7462644, 0.7357949,\
        0.7249413, 0.7138006, 0.7024831, 0.6903655, 0.6778297, 0.6650588, 0.6520901, 0.6390090, 0.6259151,\
        0.6129482, 0.6002032, 0.5877674, 0.5755242, 0.5633878, 0.5513984, 0.5396398, 0.5282059, 0.5171624,\
        0.5064486, 0.4958181, 0.4850982, 0.4744049],\
        [0.2811100, 0.2885365, 0.2957528, 0.3032838, 0.3113406, 0.3200710, 0.3297048, 0.3404226, 0.3524119,\
        0.3658703, 0.3809800, 0.3979796, 0.4167578, 0.4366213, 0.4574639, 0.4790625, 0.5014710, 0.5252380,\
        0.5504325, 0.5758076, 0.5999378, 0.6241094, 0.6471552, 0.6692568, 0.6910728, 0.7138659, 0.7377004,\
        0.7608350, 0.7809693, 0.7952424, 0.8001389, 0.7977466, 0.7941164, 0.7897552, 0.7847042, 0.7789227,\
        0.7723861, 0.7651145, 0.7571778, 0.7486955, 0.7398417, 0.7308630, 0.7221249, 0.7142062, 0.7078717,\
        0.7029745, 0.6973176, 0.6920909, 0.6874102, 0.6840060, 0.6832650, 0.6867021, 0.6950699, 0.7078019,\
        0.7230478, 0.7395455, 0.7566397, 0.7741670, 0.7913362, 0.8054404, 0.8127941, 0.8151682, 0.8148669,\
        0.8131998, 0.8112690, 0.8103663, 0.8120551, 0.8156018, 0.8194560, 0.8231138, 0.8264888, 0.8295536,\
        0.8322744, 0.8346193, 0.8365561, 0.8380397, 0.8390029, 0.8393586, 0.8390215, 0.8379401, 0.8361215,\
        0.8336358, 0.8306055, 0.8272186, 0.8237503, 0.8205138, 0.8176988, 0.8151856, 0.8124861, 0.8090276,\
        0.8046663, 0.7995724, 0.7935540, 0.7863143, 0.7778163, 0.7683160, 0.7580789, 0.7473022, 0.7361202,\
        0.7246055, 0.7127912, 0.7007595, 0.6885124, 0.6758224, 0.6628765, 0.6497949, 0.6366670, 0.6235498,\
        0.6105336, 0.5980030, 0.5855882, 0.5733101, 0.5611357, 0.5491229, 0.5373534, 0.5259171, 0.5148934,\
        0.5042241, 0.4936394, 0.4830649, 0.4725465],\
        [0.2790502, 0.2864061, 0.2937032, 0.3010305, 0.3085617, 0.3168915, 0.3262617, 0.3367164, 0.3483887,\
        0.3614624, 0.3761420, 0.3925494, 0.4099495, 0.4282661, 0.4477983, 0.4683027, 0.4896907, 0.5128251,\
        0.5378250, 0.5626807, 0.5873412, 0.6115457, 0.6342724, 0.6565784, 0.6793304, 0.7036630, 0.7299164,\
        0.7564762, 0.7801432, 0.7964796, 0.8015448, 0.7991669, 0.7960242, 0.7923890, 0.7880137, 0.7827467,\
        0.7765434, 0.7694512, 0.7615906, 0.7531439, 0.7443536, 0.7355337, 0.7270862, 0.7194833, 0.7130618,\
        0.7075358, 0.7024816, 0.6980559, 0.6937173, 0.6902064, 0.6892500, 0.6937178, 0.7040185, 0.7191446,\
        0.7358560, 0.7520062, 0.7671278, 0.7826226, 0.7983483, 0.8122202, 0.8212707, 0.8252844, 0.8258225,\
        0.8244584, 0.8225996, 0.8215548, 0.8222344, 0.8244945, 0.8275703, 0.8309014, 0.8342188, 0.8373873,\
        0.8403159, 0.8429281, 0.8451493, 0.8468965, 0.8480739, 0.8485789, 0.8483220, 0.8472547, 0.8453899,\
        0.8428075, 0.8396473, 0.8361021, 0.8324026, 0.8287653, 0.8252973, 0.8219076, 0.8183151, 0.8142143,\
        0.8094700, 0.8040424, 0.7976709, 0.7897843, 0.7804036, 0.7700433, 0.7589996, 0.7474867, 0.7356634,\
        0.7236042, 0.7112562, 0.6986404, 0.6861413, 0.6733329, 0.6602754, 0.6471466, 0.6340845, 0.6211637,\
        0.6082799, 0.5955880, 0.5834062, 0.5710508, 0.5587773, 0.5466912, 0.5348480, 0.5233032, 0.5121484,\
        0.5015363, 0.4910833, 0.4807174, 0.4704373],\
        [0.2775067, 0.2847885, 0.2919840, 0.2989682, 0.3060677, 0.3143538, 0.3235576, 0.3337353, 0.3450148,\
        0.3575495, 0.3714552, 0.3866569, 0.4028412, 0.4202411, 0.4388519, 0.4586519, 0.4797817, 0.5027922,\
        0.5262009, 0.5500703, 0.5744841, 0.5984310, 0.6211889, 0.6438574, 0.6672734, 0.6922858, 0.7200524,\
        0.7497957, 0.7774908, 0.7969538, 0.8028523, 0.8001876, 0.7972101, 0.7943771, 0.7908256, 0.7862092,\
        0.7804568, 0.7736455, 0.7659469, 0.7576000, 0.7488998, 0.7401929, 0.7318650, 0.7242868, 0.7176842,\
        0.7120435, 0.7073126, 0.7033490, 0.6999969, 0.6969071, 0.6954244, 0.7019535, 0.7155568, 0.7331752,\
        0.7517199, 0.7677049, 0.7796857, 0.7925356, 0.8082108, 0.8221542, 0.8318871, 0.8367898, 0.8378683,\
        0.8366892, 0.8347757, 0.8333676, 0.8332117, 0.8344071, 0.8366061, 0.8393908, 0.8424472, 0.8455614,\
        0.8485775, 0.8513675, 0.8538133, 0.8557943, 0.8571816, 0.8578439, 0.8576691, 0.8565963, 0.8546413,\
        0.8518978, 0.8485178, 0.8446879, 0.8406028, 0.8364256, 0.8322342, 0.8279833, 0.8235210, 0.8186680,\
        0.8132997, 0.8073666, 0.8007516, 0.7919790, 0.7818062, 0.7707080, 0.7589348, 0.7467469, 0.7343641,\
        0.7219482, 0.7095205, 0.6968008, 0.6837202, 0.6705433, 0.6573127, 0.6441281, 0.6311412, 0.6185415,\
        0.6065025, 0.5942358, 0.5812713, 0.5686417, 0.5562747, 0.5441487, 0.5322630, 0.5206231, 0.5092543,\
        0.4983693, 0.4882895, 0.4782695, 0.4682497],\
        [0.2764470, 0.2836160, 0.2905258, 0.2972684, 0.3044647, 0.3126234, 0.3215970, 0.3314066, 0.3421777,\
        0.3540288, 0.3670156, 0.3810956, 0.3962668, 0.4127235, 0.4305423, 0.4496851, 0.4702971, 0.4921124,\
        0.5140092, 0.5371838, 0.5612269, 0.5851110, 0.6082391, 0.6318400, 0.6556319, 0.6803491, 0.7082996,\
        0.7407383, 0.7728679, 0.7967050, 0.8034191, 0.7999477, 0.7978661, 0.7960739, 0.7934093, 0.7894827,\
        0.7842204, 0.7777269, 0.7702193, 0.7619838, 0.7533515, 0.7446832, 0.7363482, 0.7286820, 0.7219270,\
        0.7162125, 0.7115949, 0.7080644, 0.7055833, 0.7037374, 0.7045076, 0.7129512, 0.7293474, 0.7493160,\
        0.7702406, 0.7877556, 0.7965841, 0.8066183, 0.8222654, 0.8361271, 0.8456150, 0.8502214, 0.8509210,\
        0.8493297, 0.8469842, 0.8450322, 0.8441373, 0.8444854, 0.8459214, 0.8481509, 0.8508790, 0.8538576,\
        0.8568840, 0.8597867, 0.8624107, 0.8646027, 0.8661995, 0.8670294, 0.8669367, 0.8658296, 0.8637236,\
        0.8607351, 0.8570335, 0.8527996, 0.8482035, 0.8433815, 0.8384090, 0.8332832, 0.8279292, 0.8222275,\
        0.8160229, 0.8090587, 0.8009587, 0.7923074, 0.7819532, 0.7702878, 0.7578686, 0.7450574, 0.7321268,\
        0.7193138, 0.7068398, 0.6940825, 0.6807389, 0.6673616, 0.6540131, 0.6407502, 0.6276965, 0.6150436,\
        0.6028974, 0.5907761, 0.5783164, 0.5658570, 0.5535878, 0.5415445, 0.5297385, 0.5181756, 0.5069029,\
        0.4961312, 0.4859474, 0.4760234, 0.4661569],\
        [0.2758884, 0.2830608, 0.2899719, 0.2967289, 0.3038816, 0.3117976, 0.3203646, 0.3296762, 0.3398452,\
        0.3509647, 0.3630833, 0.3762130, 0.3904042, 0.4059022, 0.4228195, 0.4409317, 0.4601604, 0.4802856,\
        0.5014897, 0.5244372, 0.5488095, 0.5735085, 0.5979172, 0.6224084, 0.6458185, 0.6695600, 0.6961690,\
        0.7300054, 0.7647946, 0.7923295, 0.8014921, 0.7988092, 0.7982774, 0.7975948, 0.7958279, 0.7926059,\
        0.7878453, 0.7816756, 0.7743607, 0.7662287, 0.7576368, 0.7489558, 0.7405532, 0.7327693, 0.7258916,\
        0.7201466, 0.7157124, 0.7127382, 0.7113901, 0.7121558, 0.7168073, 0.7278377, 0.7452198, 0.7661793,\
        0.7880325, 0.8075484, 0.8182885, 0.8283365, 0.8416259, 0.8539926, 0.8621005, 0.8651298, 0.8643677,\
        0.8616749, 0.8585428, 0.8559468, 0.8544173, 0.8541233, 0.8549771, 0.8567541, 0.8591878, 0.8620217,\
        0.8650267, 0.8680018, 0.8707676, 0.8731509, 0.8749610, 0.8759781, 0.8759771, 0.8748016, 0.8724690,\
        0.8691478, 0.8650358, 0.8603053, 0.8551051, 0.8495585, 0.8437508, 0.8377174, 0.8314418, 0.8248592,\
        0.8178585, 0.8102889, 0.8020673, 0.7924925, 0.7812359, 0.7688967, 0.7558840, 0.7425148, 0.7290535,\
        0.7157019, 0.7025142, 0.6899984, 0.6770279, 0.6637999, 0.6504934, 0.6371970, 0.6240037, 0.6110386,\
        0.5986631, 0.5868216, 0.5748683, 0.5627990, 0.5507788, 0.5389256, 0.5273140, 0.5159850, 0.5049460,\
        0.4942811, 0.4840322, 0.4740625, 0.4642407],\
        [0.2757821, 0.2830881, 0.2901921, 0.2971868, 0.3042481, 0.3116780, 0.3197059, 0.3284370, 0.3379649,\
        0.3483677, 0.3597038, 0.3720229, 0.3853978, 0.3999678, 0.4158309, 0.4326914, 0.4505571, 0.4695124,\
        0.4898588, 0.5125253, 0.5377780, 0.5641541, 0.5907221, 0.6152220, 0.6379456, 0.6613301, 0.6878263,\
        0.7198412, 0.7528936, 0.7794762, 0.7930701, 0.7964614, 0.7979375, 0.7985775, 0.7978735, 0.7954628,\
        0.7912475, 0.7854152, 0.7783010, 0.7702769, 0.7617214, 0.7530154, 0.7445372, 0.7366525, 0.7297052,\
        0.7240193, 0.7199145, 0.7177430, 0.7179803, 0.7214359, 0.7293989, 0.7429929, 0.7618064, 0.7834968,\
        0.8051815, 0.8246668, 0.8389465, 0.8506171, 0.8626558, 0.8736108, 0.8797419, 0.8800465, 0.8769724,\
        0.8727954, 0.8687809, 0.8656081, 0.8636160, 0.8628999, 0.8633769, 0.8648530, 0.8670832, 0.8698121,\
        0.8727974, 0.8758202, 0.8786884, 0.8812331, 0.8832557, 0.8844872, 0.8846136, 0.8833335, 0.8806813,\
        0.8769573, 0.8723868, 0.8671079, 0.8612443, 0.8549141, 0.8482192, 0.8412349, 0.8339997, 0.8265052,\
        0.8186989, 0.8105331, 0.8018613, 0.7914653, 0.7794600, 0.7665831, 0.7531267, 0.7393419, 0.7254764,\
        0.7118790, 0.6989284, 0.6859470, 0.6731469, 0.6601545, 0.6470062, 0.6337864, 0.6206016, 0.6076061,\
        0.5951226, 0.5833367, 0.5716122, 0.5598039, 0.5480007, 0.5363342, 0.5249284, 0.5138786, 0.5031772,\
        0.4925986, 0.4822841, 0.4722811, 0.4624839],\
        [0.2759799, 0.2834522, 0.2907768, 0.2979204, 0.3048199, 0.3118439, 0.3193592, 0.3275251, 0.3364326,\
        0.3461593, 0.3567792, 0.3683721, 0.3810332, 0.3948363, 0.4095831, 0.4251450, 0.4418833, 0.4600607,\
        0.4799375, 0.5022921, 0.5279702, 0.5548993, 0.5819053, 0.6071579, 0.6301100, 0.6539595, 0.6804276,\
        0.7098765, 0.7394198, 0.7643653, 0.7811624, 0.7904350, 0.7955803, 0.7983944, 0.7991986, 0.7978499,\
        0.7942706, 0.7888219, 0.7819520, 0.7740760, 0.7655899, 0.7568872, 0.7483656, 0.7404265, 0.7334764,\
        0.7279369, 0.7242674, 0.7230054, 0.7248249, 0.7305830, 0.7412036, 0.7572227, 0.7781056, 0.8015433,\
        0.8242221, 0.8438677, 0.8588782, 0.8703807, 0.8810314, 0.8913664, 0.8955998, 0.8924777, 0.8871199,\
        0.8817633, 0.8771631, 0.8736732, 0.8714634, 0.8705596, 0.8708672, 0.8722059, 0.8743451, 0.8770326,\
        0.8800174, 0.8830699, 0.8859817, 0.8886176, 0.8908349, 0.8922688, 0.8926164, 0.8911852, 0.8880977,\
        0.8839809, 0.8789813, 0.8731504, 0.8665964, 0.8594416, 0.8518104, 0.8438227, 0.8355768, 0.8271192,\
        0.8183971, 0.8091944, 0.7993088, 0.7885992, 0.7765212, 0.7634620, 0.7498032, 0.7358263, 0.7217668,\
        0.7077800, 0.6946695, 0.6821887, 0.6695401, 0.6567494, 0.6438095, 0.6307667, 0.6177288, 0.6049545,\
        0.5925991, 0.5806381, 0.5688722, 0.5571198, 0.5454000, 0.5338180, 0.5225070, 0.5115914, 0.5010747,\
        0.4906837, 0.4804891, 0.4705695, 0.4608459],\
        [0.2763116, 0.2838723, 0.2912688, 0.2985098, 0.3052230, 0.3119485, 0.3190898, 0.3267800, 0.3351265,\
        0.3442229, 0.3541562, 0.3650161, 0.3768959, 0.3898403, 0.4035886, 0.4180033, 0.4339431, 0.4518120,\
        0.4718366, 0.4944647, 0.5191912, 0.5449593, 0.5712668, 0.5964791, 0.6207009, 0.6453718, 0.6714634,\
        0.6989720, 0.7254783, 0.7497996, 0.7690616, 0.7825720, 0.7914636, 0.7969088, 0.7995342, 0.7995333,\
        0.7967102, 0.7917581, 0.7852299, 0.7775860, 0.7692396, 0.7606020, 0.7520978, 0.7441690, 0.7372822,\
        0.7319449, 0.7287318, 0.7283162, 0.7314890, 0.7391133, 0.7519257, 0.7701706, 0.7933695, 0.8193189,\
        0.8439474, 0.8642341, 0.8784686, 0.8875006, 0.8940923, 0.9015034, 0.9043638, 0.8999732, 0.8938542,\
        0.8882077, 0.8835377, 0.8800601, 0.8778845, 0.8770089, 0.8773312, 0.8786793, 0.8808355, 0.8835499,\
        0.8865548, 0.8896224, 0.8925250, 0.8950396, 0.8974921, 0.8989080, 0.8995688, 0.8978913, 0.8943778,\
        0.8901087, 0.8847825, 0.8784239, 0.8711806, 0.8631776, 0.8545639, 0.8455124, 0.8361946, 0.8267404,\
        0.8171724, 0.8073064, 0.7966790, 0.7852874, 0.7729677, 0.7598367, 0.7461590, 0.7322099, 0.7182992,\
        0.7048675, 0.6916715, 0.6789251, 0.6663623, 0.6537530, 0.6410509, 0.6282657, 0.6154293, 0.6026408,\
        0.5904747, 0.5785473, 0.5666713, 0.5548364, 0.5430758, 0.5314628, 0.5201059, 0.5091428, 0.4986567,\
        0.4885419, 0.4786465, 0.4689223, 0.4593245],\
        [0.2766751, 0.2842361, 0.2914767, 0.2983361, 0.3051055, 0.3118769, 0.3187988, 0.3261031, 0.3339525,\
        0.3424635, 0.3517273, 0.3618183, 0.3727886, 0.3846418, 0.3973213, 0.4108662, 0.4263467, 0.4441028,\
        0.4640175, 0.4860982, 0.5092331, 0.5340750, 0.5598046, 0.5852013, 0.6102134, 0.6354665, 0.6608432,\
        0.6861151, 0.7115793, 0.7365391, 0.7579064, 0.7745551, 0.7866140, 0.7946002, 0.7989555, 0.8002016,\
        0.7984571, 0.7941554, 0.7880956, 0.7807942, 0.7726788, 0.7641883, 0.7557817, 0.7479413, 0.7411838,\
        0.7360816, 0.7332908, 0.7335783, 0.7378242, 0.7469564, 0.7617542, 0.7823845, 0.8082065, 0.8370611,\
        0.8638156, 0.8848673, 0.8975722, 0.9035401, 0.9061612, 0.9085273, 0.9088266, 0.9044520, 0.8982212,\
        0.8927165, 0.8882476, 0.8849654, 0.8829914, 0.8823007, 0.8827735, 0.8842394, 0.8864932, 0.8892964,\
        0.8923462, 0.8953310, 0.8983846, 0.9004305, 0.9028768, 0.9043921, 0.9046852, 0.9029247, 0.8995468,\
        0.8954906, 0.8898036, 0.8829609, 0.8750643, 0.8662051, 0.8565675, 0.8463872, 0.8359201, 0.8254067,\
        0.8149844, 0.8045998, 0.7936550, 0.7816912, 0.7691288, 0.7559913, 0.7424128, 0.7286140, 0.7148499,\
        0.7013596, 0.6886825, 0.6760930, 0.6636234, 0.6511828, 0.6387292, 0.6262812, 0.6138578, 0.6014464,\
        0.5891887, 0.5770207, 0.5649222, 0.5529327, 0.5410807, 0.5293996, 0.5179429, 0.5068593, 0.4964392,\
        0.4865949, 0.4769725, 0.4674376, 0.4579620],\
        [0.2770361, 0.2846224, 0.2918531, 0.2986419, 0.3052664, 0.3118525, 0.3185088, 0.3254549, 0.3328567,\
        0.3408336, 0.3494722, 0.3588276, 0.3689122, 0.3796861, 0.3911051, 0.4036952, 0.4192552, 0.4367393,\
        0.4558034, 0.4763492, 0.4984527, 0.5229429, 0.5484425, 0.5740670, 0.5997067, 0.6251141, 0.6499362,\
        0.6747243, 0.6999337, 0.7253263, 0.7482112, 0.7671698, 0.7817980, 0.7921347, 0.7983433, 0.8007454,\
        0.7997573, 0.7960717, 0.7905862, 0.7837202, 0.7759172, 0.7676599, 0.7594448, 0.7517877, 0.7452400,\
        0.7404141, 0.7380147, 0.7388699, 0.7439431, 0.7542846, 0.7708939, 0.7945493, 0.8237619, 0.8548723,\
        0.8827641, 0.9033782, 0.9144548, 0.9184443, 0.9187822, 0.9174694, 0.9143385, 0.9087983, 0.9021308,\
        0.8963711, 0.8918639, 0.8887269, 0.8870179, 0.8865990, 0.8872987, 0.8889385, 0.8913235, 0.8942325,\
        0.8974290, 0.9002423, 0.9030671, 0.9054878, 0.9075081, 0.9088273, 0.9088644, 0.9072787, 0.9043002,\
        0.8999433, 0.8939773, 0.8868620, 0.8783672, 0.8686452, 0.8579519, 0.8465934, 0.8348985, 0.8232271,\
        0.8119079, 0.8007222, 0.7894969, 0.7776196, 0.7651232, 0.7521263, 0.7387522, 0.7251907, 0.7117557,\
        0.6987656, 0.6859963, 0.6735777, 0.6612533, 0.6489631, 0.6367061, 0.6245449, 0.6126041, 0.6007342,\
        0.5882893, 0.5757591, 0.5634269, 0.5513055, 0.5394051, 0.5277411, 0.5163172, 0.5051554, 0.4948179,\
        0.4851890, 0.4756743, 0.4662158, 0.4568021],\
        [0.2773399, 0.2849767, 0.2923330, 0.2993389, 0.3056723, 0.3118619, 0.3181957, 0.3248031, 0.3318062,\
        0.3393129, 0.3474090, 0.3561531, 0.3655715, 0.3756677, 0.3865250, 0.3988919, 0.4137454, 0.4301185,\
        0.4478902, 0.4673056, 0.4886982, 0.5122418, 0.5371018, 0.5625919, 0.5885575, 0.6137612, 0.6386992,\
        0.6641058, 0.6900277, 0.7157838, 0.7397415, 0.7604066, 0.7771440, 0.7896859, 0.7978013, 0.8013842,\
        0.8008345, 0.7977071, 0.7928096, 0.7863763, 0.7789317, 0.7709921, 0.7630784, 0.7557253, 0.7495013,\
        0.7450366, 0.7430563, 0.7444218, 0.7501761, 0.7615528, 0.7797949, 0.8059901, 0.8388332, 0.8717959,\
        0.8997079, 0.9181925, 0.9279785, 0.9310266, 0.9298212, 0.9260889, 0.9205146, 0.9135102, 0.9061113,\
        0.8996486, 0.8946661, 0.8916591, 0.8902496, 0.8901360, 0.8910849, 0.8929030, 0.8954210, 0.8983728,\
        0.9015679, 0.9045790, 0.9072718, 0.9096666, 0.9115318, 0.9126073, 0.9125306, 0.9110571, 0.9081113,\
        0.9035770, 0.8977908, 0.8903501, 0.8812216, 0.8706144, 0.8588583, 0.8463366, 0.8334136, 0.8204647,\
        0.8081591, 0.7969679, 0.7853337, 0.7734814, 0.7611731, 0.7484284, 0.7353497, 0.7220800, 0.7087664,\
        0.6959721, 0.6836960, 0.6714322, 0.6592107, 0.6470136, 0.6348423, 0.6227255, 0.6107334, 0.5989977,\
        0.5867751, 0.5743100, 0.5619543, 0.5498097, 0.5379472, 0.5264343, 0.5153214, 0.5046414, 0.4943594,\
        0.4844403, 0.4747764, 0.4652632, 0.4558450],\
        [0.2775111, 0.2851193, 0.2924141, 0.2993055, 0.3055864, 0.3116436, 0.3177714, 0.3241082, 0.3307729,\
        0.3378773, 0.3455211, 0.3537915, 0.3627754, 0.3725984, 0.3835135, 0.3958680, 0.4094327, 0.4239761,\
        0.4400713, 0.4582250, 0.4786968, 0.5012999, 0.5250486, 0.5495663, 0.5747851, 0.6002741, 0.6262213,\
        0.6528426, 0.6799130, 0.7066263, 0.7316386, 0.7535540, 0.7720862, 0.7868524, 0.7970893, 0.8021589,\
        0.8022814, 0.7994330, 0.7947095, 0.7886545, 0.7816131, 0.7740942, 0.7666190, 0.7597243, 0.7539831,\
        0.7500293, 0.7485916, 0.7505389, 0.7569595, 0.7693400, 0.7896369, 0.8190405, 0.8540111, 0.8872461,\
        0.9133335, 0.9302391, 0.9389098, 0.9411856, 0.9389355, 0.9336826, 0.9264624, 0.9181196, 0.9096398,\
        0.9020838, 0.8969207, 0.8941134, 0.8929862, 0.8931680, 0.8943612, 0.8963085, 0.8989428, 0.9019826,\
        0.9051318, 0.9081663, 0.9108829, 0.9131852, 0.9149025, 0.9158092, 0.9156567, 0.9142433, 0.9114344,\
        0.9071439, 0.9012241, 0.8934109, 0.8836717, 0.8721481, 0.8593638, 0.8458069, 0.8318863, 0.8179287,\
        0.8041726, 0.7925352, 0.7813430, 0.7696255, 0.7575148, 0.7450541, 0.7323252, 0.7194907, 0.7068060,\
        0.6942410, 0.6818585, 0.6696247, 0.6574432, 0.6452872, 0.6331397, 0.6209806, 0.6087609, 0.5966222,\
        0.5848186, 0.5726688, 0.5604320, 0.5483343, 0.5365366, 0.5251884, 0.5144188, 0.5041605, 0.4939943,\
        0.4838875, 0.4740613, 0.4644772, 0.4550428],\
        [0.2775368, 0.2850435, 0.2921373, 0.2987978, 0.3050833, 0.3111851, 0.3172251, 0.3233578, 0.3297346,\
        0.3364888, 0.3437390, 0.3516002, 0.3602030, 0.3697230, 0.3803880, 0.3923224, 0.4047196, 0.4172573,\
        0.4315693, 0.4482628, 0.4675047, 0.4889598, 0.5114979, 0.5348754, 0.5594835, 0.5852801, 0.6122183,\
        0.6401463, 0.6687551, 0.6969832, 0.7226865, 0.7453022, 0.7654377, 0.7826244, 0.7957343, 0.8027867,\
        0.8030190, 0.8000158, 0.7958727, 0.7902603, 0.7837407, 0.7767990, 0.7699368, 0.7636869, 0.7586250,\
        0.7553885, 0.7547119, 0.7574735, 0.7647386, 0.7779847, 0.8000522, 0.8322698, 0.8686583, 0.9015853,\
        0.9248937, 0.9399023, 0.9475642, 0.9491964, 0.9462566, 0.9400792, 0.9317877, 0.9224037, 0.9130133,\
        0.9048746, 0.8992614, 0.8963865, 0.8954504, 0.8959044, 0.8973706, 0.8994622, 0.9020790, 0.9051740,\
        0.9083065, 0.9112823, 0.9139289, 0.9161048, 0.9176552, 0.9183998, 0.9181532, 0.9167528, 0.9140640,\
        0.9099372, 0.9040962, 0.8961489, 0.8856045, 0.8731077, 0.8594007, 0.8450203, 0.8304578, 0.8162450,\
        0.8030183, 0.7902388, 0.7781489, 0.7662756, 0.7542826, 0.7420797, 0.7296721, 0.7171462, 0.7047698,\
        0.6925704, 0.6802610, 0.6680305, 0.6558584, 0.6437292, 0.6316212, 0.6195133, 0.6074638, 0.5954591,\
        0.5832984, 0.5711280, 0.5589300, 0.5468476, 0.5350491, 0.5237106, 0.5130425, 0.5031741, 0.4931270,\
        0.4830516, 0.4732794, 0.4637386, 0.4543386],\
        [0.2774663, 0.2849033, 0.2918532, 0.2982776, 0.3045844, 0.3107077, 0.3166379, 0.3225697, 0.3286787,\
        0.3351091, 0.3419851, 0.3494248, 0.3575520, 0.3664997, 0.3763669, 0.3870441, 0.3979357, 0.4092389,\
        0.4219387, 0.4371716, 0.4549504, 0.4748642, 0.4962384, 0.5188020, 0.5429861, 0.5690082, 0.5966459,\
        0.6255833, 0.6553881, 0.6845457, 0.7105542, 0.7341582, 0.7557835, 0.7751095, 0.7912085, 0.8009275,\
        0.8008955, 0.7990655, 0.7955999, 0.7907512, 0.7850292, 0.7788927, 0.7728456, 0.7674361, 0.7632562,\
        0.7609552, 0.7612801, 0.7651655, 0.7738283, 0.7884131, 0.8117608, 0.8448786, 0.8804618, 0.9114795,\
        0.9333396, 0.9471166, 0.9540684, 0.9553015, 0.9520068, 0.9453586, 0.9364561, 0.9263643, 0.9162213,\
        0.9073563, 0.9012534, 0.8982025, 0.8977167, 0.8984995, 0.9001870, 0.9025166, 0.9051925, 0.9081767,\
        0.9111965, 0.9140302, 0.9165084, 0.9184839, 0.9198111, 0.9203389, 0.9199177, 0.9184110, 0.9156915,\
        0.9116093, 0.9059306, 0.8978265, 0.8863975, 0.8731577, 0.8587926, 0.8438213, 0.8287707, 0.8142142,\
        0.8005347, 0.7876912, 0.7753522, 0.7633700, 0.7514707, 0.7395020, 0.7274084, 0.7151496, 0.7026330,\
        0.6907077, 0.6786885, 0.6664963, 0.6543249, 0.6422322, 0.6302063, 0.6181807, 0.6060743, 0.5940140,\
        0.5819518, 0.5697414, 0.5575052, 0.5453871, 0.5335236, 0.5220331, 0.5110287, 0.5007857, 0.4913294,\
        0.4818355, 0.4723864, 0.4630198, 0.4537145],\
        [0.2773218, 0.2847882, 0.2918154, 0.2983433, 0.3045020, 0.3103626, 0.3160548, 0.3217496, 0.3275953,\
        0.3337178, 0.3402257, 0.3472148, 0.3547684, 0.3629430, 0.3717243, 0.3809644, 0.3904907, 0.4003066,\
        0.4115348, 0.4255856, 0.4418712, 0.4602852, 0.4804651, 0.5022485, 0.5258912, 0.5521357, 0.5801929,\
        0.6097652, 0.6399097, 0.6686917, 0.6952948, 0.7201207, 0.7424847, 0.7629481, 0.7801908, 0.7912852,\
        0.7948833, 0.7951129, 0.7932845, 0.7898176, 0.7852647, 0.7801796, 0.7751320, 0.7707220, 0.7675829,\
        0.7663868, 0.7678718, 0.7729417, 0.7830091, 0.8003859, 0.8246297, 0.8561761, 0.8894709, 0.9179333,\
        0.9386255, 0.9518434, 0.9585334, 0.9597057, 0.9564095, 0.9497021, 0.9406186, 0.9301829, 0.9194557,\
        0.9096017, 0.9020405, 0.8991379, 0.8999380, 0.9012594, 0.9030760, 0.9055176, 0.9082336, 0.9110846,\
        0.9138971, 0.9164877, 0.9186955, 0.9203768, 0.9213940, 0.9216101, 0.9208901, 0.9191031, 0.9161131,\
        0.9117242, 0.9055239, 0.8968518, 0.8854780, 0.8720850, 0.8574548, 0.8421537, 0.8267025, 0.8116932,\
        0.7977487, 0.7849006, 0.7726882, 0.7607785, 0.7489893, 0.7372204, 0.7254395, 0.7136812, 0.7020212,\
        0.6896569, 0.6772231, 0.6648926, 0.6526827, 0.6406313, 0.6287447, 0.6169517, 0.6050386, 0.5929909,\
        0.5807924, 0.5684586, 0.5561549, 0.5440091, 0.5321274, 0.5205922, 0.5094711, 0.4990718, 0.4897928,\
        0.4807241, 0.4715796, 0.4623987, 0.4531995],\
        [0.2770438, 0.2845688, 0.2917442, 0.2985281, 0.3044481, 0.3099604, 0.3153947, 0.3208639, 0.3264732,\
        0.3323196, 0.3384860, 0.3450370, 0.3520119, 0.3594101, 0.3671715, 0.3751817, 0.3833579, 0.3918768,\
        0.4022854, 0.4147668, 0.4293905, 0.4465447, 0.4657001, 0.4864081, 0.5094715, 0.5359995, 0.5641327,\
        0.5940021, 0.6235860, 0.6519580, 0.6791799, 0.7039917, 0.7260266, 0.7474312, 0.7654366, 0.7783900,\
        0.7857513, 0.7889409, 0.7892818, 0.7875489, 0.7844144, 0.7805395, 0.7765889, 0.7732426, 0.7711997,\
        0.7711765, 0.7739029, 0.7801260, 0.7906982, 0.8082293, 0.8337974, 0.8642244, 0.8955483, 0.9218719,\
        0.9414583, 0.9543779, 0.9611510, 0.9626000, 0.9596586, 0.9532956, 0.9444822, 0.9341877, 0.9233969,\
        0.9131573, 0.9048091, 0.9015200, 0.9026795, 0.9043756, 0.9064006, 0.9087759, 0.9113506, 0.9139665,\
        0.9164720, 0.9187138, 0.9205460, 0.9218323, 0.9224407, 0.9222391, 0.9210912, 0.9188549, 0.9153789,\
        0.9104909, 0.9038431, 0.8947868, 0.8833693, 0.8700809, 0.8555140, 0.8401942, 0.8245876, 0.8092181,\
        0.7951147, 0.7824747, 0.7703692, 0.7584950, 0.7467439, 0.7350627, 0.7234246, 0.7117843, 0.6999829,\
        0.6878550, 0.6754108, 0.6630070, 0.6507584, 0.6387250, 0.6269586, 0.6154836, 0.6041510, 0.5920803,\
        0.5795755, 0.5671261, 0.5548164, 0.5427216, 0.5309415, 0.5195983, 0.5088159, 0.4987063, 0.4892661,\
        0.4801262, 0.4710544, 0.4619679, 0.4528358],\
        [0.2765490, 0.2840538, 0.2911816, 0.2977957, 0.3037362, 0.3092205, 0.3145451, 0.3198721, 0.3253093,\
        0.3309407, 0.3368296, 0.3430162, 0.3495116, 0.3562893, 0.3632865, 0.3704361, 0.3777712, 0.3856611,\
        0.3948385, 0.4053025, 0.4184025, 0.4348913, 0.4534136, 0.4737477, 0.4968275, 0.5211749, 0.5481112,\
        0.5773107, 0.6064383, 0.6347093, 0.6618247, 0.6861259, 0.7098277, 0.7312653, 0.7500172, 0.7650937,\
        0.7754885, 0.7815543, 0.7841935, 0.7842773, 0.7826032, 0.7799388, 0.7770428, 0.7746793, 0.7736230,\
        0.7746564, 0.7785528, 0.7860594, 0.7978888, 0.8149285, 0.8387873, 0.8675072, 0.8971255, 0.9225473,\
        0.9418902, 0.9549206, 0.9620909, 0.9641296, 0.9618823, 0.9562497, 0.9481577, 0.9385504, 0.9284147,\
        0.9188664, 0.9113734, 0.9076518, 0.9073731, 0.9085652, 0.9103376, 0.9124241, 0.9146595, 0.9168891,\
        0.9189659, 0.9207484, 0.9221010, 0.9228940, 0.9230004, 0.9222891, 0.9206183, 0.9178265, 0.9137269,\
        0.9081466, 0.9011320, 0.8919118, 0.8805066, 0.8674425, 0.8531676, 0.8381555, 0.8228671, 0.8077720,\
        0.7936989, 0.7808609, 0.7685433, 0.7565116, 0.7446481, 0.7328957, 0.7212211, 0.7095284, 0.6976209,\
        0.6854703, 0.6731084, 0.6607228, 0.6484536, 0.6363810, 0.6245722, 0.6130927, 0.6018481, 0.5900656,\
        0.5778157, 0.5655575, 0.5534189, 0.5414864, 0.5298810, 0.5187904, 0.5084458, 0.4987173, 0.4891704,\
        0.4798975, 0.4708136, 0.4617563, 0.4526440],\
        [0.2758175, 0.2832335, 0.2902242, 0.2966928, 0.3026275, 0.3081731, 0.3134988, 0.3187732, 0.3241195,\
        0.3296189, 0.3353256, 0.3412691, 0.3474512, 0.3538453, 0.3604051, 0.3670979, 0.3739704, 0.3812102,\
        0.3890330, 0.3976000, 0.4105567, 0.4266406, 0.4443831, 0.4636238, 0.4843548, 0.5067368, 0.5329227,\
        0.5616469, 0.5903419, 0.6190443, 0.6464188, 0.6717530, 0.6950107, 0.7154192, 0.7357087, 0.7530019,\
        0.7657801, 0.7741346, 0.7787379, 0.7804225, 0.7800361, 0.7784130, 0.7763779, 0.7747559, 0.7743792,\
        0.7760837, 0.7806743, 0.7888171, 0.8008119, 0.8167621, 0.8384561, 0.8656891, 0.8945115, 0.9203945,\
        0.9400721, 0.9535571, 0.9614334, 0.9643620, 0.9631194, 0.9585571, 0.9515708, 0.9430917, 0.9341059,\
        0.9256988, 0.9190499, 0.9150681, 0.9136496, 0.9138771, 0.9149657, 0.9164841, 0.9181804, 0.9198692,\
        0.9213930, 0.9226090, 0.9233849, 0.9235964, 0.9231220, 0.9218357, 0.9195966, 0.9162397, 0.9115614,\
        0.9053026, 0.8974478, 0.8882661, 0.8771117, 0.8643691, 0.8505083, 0.8359946, 0.8213150, 0.8070148,\
        0.7931404, 0.7797413, 0.7670277, 0.7547092, 0.7426087, 0.7306398, 0.7187810, 0.7070434, 0.6951018,\
        0.6828004, 0.6704385, 0.6580823, 0.6458091, 0.6336717, 0.6216954, 0.6099255, 0.5984677, 0.5871698,\
        0.5755688, 0.5637804, 0.5519838, 0.5403108, 0.5288869, 0.5178804, 0.5075862, 0.4982009, 0.4888609,\
        0.4797763, 0.4707826, 0.4617488, 0.4526188],\
        [0.2748899, 0.2822093, 0.2891040, 0.2955054, 0.3014171, 0.3069972, 0.3123230, 0.3176026, 0.3229343,\
        0.3283900, 0.3340224, 0.3398626, 0.3459190, 0.3521804, 0.3586260, 0.3652461, 0.3720707, 0.3791937,\
        0.3868518, 0.3959723, 0.4078908, 0.4225619, 0.4390480, 0.4567667, 0.4754582, 0.4957777, 0.5215812,\
        0.5501347, 0.5788055, 0.6069463, 0.6338163, 0.6586080, 0.6811358, 0.7024316, 0.7247909, 0.7434256,\
        0.7576483, 0.7675020, 0.7734879, 0.7763505, 0.7769199, 0.7760421, 0.7745604, 0.7733173, 0.7731581,\
        0.7749233, 0.7794100, 0.7872513, 0.7985989, 0.8120390, 0.8314185, 0.8586344, 0.8893268, 0.9159084,\
        0.9358778, 0.9502316, 0.9591767, 0.9632960, 0.9633324, 0.9601110, 0.9545048, 0.9474184, 0.9397870,\
        0.9325652, 0.9266414, 0.9225991, 0.9204615, 0.9197892, 0.9200493, 0.9208284, 0.9218373, 0.9228617,\
        0.9237280, 0.9242862, 0.9244032, 0.9239585, 0.9228404, 0.9209357, 0.9181147, 0.9142265, 0.9091301,\
        0.9026041, 0.8943937, 0.8846647, 0.8734744, 0.8609758, 0.8475182, 0.8334996, 0.8193053, 0.8052600,\
        0.7916054, 0.7783378, 0.7654613, 0.7529063, 0.7405475, 0.7282832, 0.7160684, 0.7039260, 0.6919380,\
        0.6797766, 0.6674893, 0.6551867, 0.6429521, 0.6308288, 0.6188022, 0.6068528, 0.5954236, 0.5844732,\
        0.5733456, 0.5620195, 0.5506265, 0.5392928, 0.5281169, 0.5171744, 0.5065331, 0.4969376, 0.4885116,\
        0.4798487, 0.4709787, 0.4619352, 0.4527384],\
        [0.2738169, 0.2810510, 0.2878853, 0.2943167, 0.3003426, 0.3056939, 0.3110581, 0.3164047, 0.3217862,\
        0.3272759, 0.3329337, 0.3388004, 0.3449003, 0.3512460, 0.3578485, 0.3647288, 0.3719313, 0.3795424,\
        0.3877647, 0.3970820, 0.4084257, 0.4220338, 0.4374016, 0.4542031, 0.4724049, 0.4928872, 0.5168178,\
        0.5433073, 0.5707459, 0.5980090, 0.6236366, 0.6477668, 0.6717931, 0.6958425, 0.7177718, 0.7364982,\
        0.7513312, 0.7619551, 0.7687132, 0.7722696, 0.7734044, 0.7729283, 0.7716548, 0.7703955, 0.7699567,\
        0.7711213, 0.7745970, 0.7808893, 0.7897738, 0.7995835, 0.8179724, 0.8461797, 0.8781288, 0.9065752,\
        0.9289255, 0.9449133, 0.9553199, 0.9609100, 0.9624498, 0.9607635, 0.9567068, 0.9511411, 0.9449180,\
        0.9388485, 0.9336235, 0.9296847, 0.9271280, 0.9257531, 0.9252285, 0.9252285, 0.9254821, 0.9257701,\
        0.9259101, 0.9257461, 0.9251422, 0.9239821, 0.9221672, 0.9196108, 0.9162091, 0.9117905, 0.9062044,\
        0.8995085, 0.8909293, 0.8808950, 0.8696078, 0.8572619, 0.8441383, 0.8305634, 0.8168204, 0.8030859,\
        0.7896768, 0.7765446, 0.7636415, 0.7509597, 0.7384238, 0.7259255, 0.7133842, 0.7009216, 0.6887714,\
        0.6766445, 0.6643959, 0.6521308, 0.6399726, 0.6280022, 0.6162297, 0.6046546, 0.5933491, 0.5823731,\
        0.5714351, 0.5604379, 0.5494350, 0.5385137, 0.5277569, 0.5172509, 0.5071903, 0.4979182, 0.4891398,\
        0.4804074, 0.4714580, 0.4622924, 0.4529565],\
        [0.2726406, 0.2797958, 0.2865534, 0.2928581, 0.2985710, 0.3042429, 0.3097907, 0.3152324, 0.3206985,\
        0.3262771, 0.3320368, 0.3380299, 0.3442974, 0.3508771, 0.3578123, 0.3651611, 0.3730001, 0.3814147,\
        0.3904705, 0.4002549, 0.4111422, 0.4238375, 0.4384398, 0.4548503, 0.4730968, 0.4933523, 0.5153993,\
        0.5390537, 0.5640190, 0.5896308, 0.6131150, 0.6373101, 0.6631865, 0.6887649, 0.7116886, 0.7312097,\
        0.7464396, 0.7573302, 0.7643637, 0.7681964, 0.7695534, 0.7691789, 0.7678192, 0.7662189, 0.7651156,\
        0.7652138, 0.7671136, 0.7711497, 0.7772279, 0.7861398, 0.8030178, 0.8320813, 0.8650828, 0.8958024,\
        0.9201739, 0.9378023, 0.9499337, 0.9572115, 0.9604117, 0.9603838, 0.9579712, 0.9539807, 0.9491713,\
        0.9442298, 0.9397195, 0.9360127, 0.9332495, 0.9313651, 0.9301706, 0.9294354, 0.9289352, 0.9284677,\
        0.9278540, 0.9269357, 0.9255751, 0.9236572, 0.9210952, 0.9178405, 0.9138715, 0.9090643, 0.9028715,\
        0.8955541, 0.8869016, 0.8767191, 0.8653804, 0.8531496, 0.8402904, 0.8271003, 0.8138906, 0.8006890,\
        0.7874594, 0.7743601, 0.7614595, 0.7487493, 0.7361761, 0.7236371, 0.7110103, 0.6983527, 0.6859885,\
        0.6736442, 0.6612642, 0.6489468, 0.6368384, 0.6250831, 0.6137245, 0.6025917, 0.5915598, 0.5805907,\
        0.5697467, 0.5590023, 0.5483694, 0.5379029, 0.5276814, 0.5177966, 0.5083311, 0.4992523, 0.4903630,\
        0.4813561, 0.4721292, 0.4627242, 0.4531850],\
        [0.2713989, 0.2784916, 0.2852043, 0.2915202, 0.2975235, 0.3031536, 0.3086192, 0.3141160, 0.3196726,\
        0.3253704, 0.3312808, 0.3374615, 0.3439651, 0.3508469, 0.3581735, 0.3660285, 0.3745138, 0.3837232,\
        0.3935987, 0.4036656, 0.4142687, 0.4264511, 0.4405872, 0.4567286, 0.4748067, 0.4944104, 0.5145024,\
        0.5351779, 0.5567352, 0.5785248, 0.6004165, 0.6266310, 0.6547533, 0.6821585, 0.7067902, 0.7271490,\
        0.7423548, 0.7531377, 0.7601295, 0.7639783, 0.7653464, 0.7648931, 0.7632744, 0.7611518, 0.7591965,\
        0.7580824, 0.7584613, 0.7609509, 0.7663222, 0.7762049, 0.7934549, 0.8210418, 0.8527362, 0.8835521,\
        0.9092226, 0.9290277, 0.9431698, 0.9522557, 0.9571923, 0.9588912, 0.9581779, 0.9557901, 0.9523921,\
        0.9485687, 0.9447987, 0.9414175, 0.9385972, 0.9363597, 0.9346188, 0.9332284, 0.9320194, 0.9308194,\
        0.9294618, 0.9277911, 0.9256682, 0.9229755, 0.9196245, 0.9155808, 0.9109285, 0.9058547, 0.9001132,\
        0.8918683, 0.8825141, 0.8720850, 0.8606991, 0.8485687, 0.8359221, 0.8229809, 0.8100703, 0.7975524,\
        0.7845513, 0.7715938, 0.7587727, 0.7461187, 0.7336313, 0.7212500, 0.7088277, 0.6961891, 0.6834141,\
        0.6706935, 0.6580622, 0.6455966, 0.6334356, 0.6217933, 0.6109034, 0.6003491, 0.5895292, 0.5786801,\
        0.5680159, 0.5575490, 0.5472830, 0.5372602, 0.5275569, 0.5182592, 0.5093755, 0.5006386, 0.4917015,\
        0.4823099, 0.4727286, 0.4630509, 0.4532966],\
        [0.2701208, 0.2771715, 0.2838744, 0.2902202, 0.2961794, 0.3018236, 0.3074750, 0.3130361, 0.3186802,\
        0.3245139, 0.3306013, 0.3369958, 0.3437520, 0.3509315, 0.3586065, 0.3668598, 0.3757775, 0.3854252,\
        0.3957614, 0.4060061, 0.4165814, 0.4286015, 0.4424359, 0.4581881, 0.4757258, 0.4944279, 0.5127130,\
        0.5308894, 0.5497953, 0.5695981, 0.5915531, 0.6176635, 0.6468454, 0.6765208, 0.7033995, 0.7231010,\
        0.7380223, 0.7486373, 0.7555300, 0.7593507, 0.7607041, 0.7601568, 0.7582655, 0.7556033, 0.7527868,\
        0.7505057, 0.7495679, 0.7509991, 0.7562548, 0.7674520, 0.7869092, 0.8113159, 0.8398198, 0.8698292,\
        0.8970459, 0.9191685, 0.9352621, 0.9461069, 0.9527923, 0.9562710, 0.9573098, 0.9565517, 0.9545644,\
        0.9518513, 0.9488355, 0.9458363, 0.9430543, 0.9405755, 0.9383921, 0.9364308, 0.9345774, 0.9326940,\
        0.9306315, 0.9282401, 0.9253807, 0.9219349, 0.9178053, 0.9129108, 0.9072819, 0.9013676, 0.8950581,\
        0.8870314, 0.8774807, 0.8669051, 0.8555037, 0.8434793, 0.8310632, 0.8184401, 0.8054685, 0.7930613,\
        0.7807873, 0.7681418, 0.7554634, 0.7428953, 0.7305158, 0.7183415, 0.7061931, 0.6935511, 0.6805188,\
        0.6675170, 0.6546727, 0.6420507, 0.6297441, 0.6179260, 0.6069715, 0.5972394, 0.5868080, 0.5763365,\
        0.5660337, 0.5559204, 0.5460206, 0.5363802, 0.5270696, 0.5181751, 0.5097581, 0.5016519, 0.4924506,\
        0.4827265, 0.4729269, 0.4630713, 0.4531584],\
        [0.2688233, 0.2758416, 0.2825473, 0.2889459, 0.2950465, 0.3008803, 0.3064095, 0.3119387, 0.3176721,\
        0.3236618, 0.3299399, 0.3365479, 0.3435360, 0.3509618, 0.3588882, 0.3673759, 0.3764603, 0.3860975,\
        0.3961027, 0.4065370, 0.4175642, 0.4296283, 0.4431760, 0.4583308, 0.4748983, 0.4922417, 0.5092077,\
        0.5259739, 0.5434889, 0.5624279, 0.5838494, 0.6091529, 0.6382025, 0.6680457, 0.6951618, 0.7166005,\
        0.7322665, 0.7430519, 0.7500454, 0.7540189, 0.7555289, 0.7550492, 0.7530351, 0.7499675, 0.7464025,\
        0.7430334, 0.7407772, 0.7408966, 0.7451126, 0.7554185, 0.7728304, 0.7961865, 0.8248836, 0.8554005,\
        0.8841443, 0.9083287, 0.9261943, 0.9387425, 0.9472392, 0.9526041, 0.9554760, 0.9563792, 0.9557936,\
        0.9541650, 0.9518876, 0.9492826, 0.9465817, 0.9439251, 0.9413704, 0.9389075, 0.9364743, 0.9339685,\
        0.9312595, 0.9282013, 0.9246530, 0.9205043, 0.9156843, 0.9101130, 0.9036217, 0.8966719, 0.8896018,\
        0.8813183, 0.8717444, 0.8611504, 0.8497751, 0.8378439, 0.8256312, 0.8135161, 0.8020011, 0.7892868,\
        0.7766258, 0.7640607, 0.7514996, 0.7389817, 0.7266067, 0.7144934, 0.7024739, 0.6899242, 0.6769127,\
        0.6638812, 0.6509993, 0.6383519, 0.6259975, 0.6140065, 0.6026537, 0.5927511, 0.5833272, 0.5735824,\
        0.5637760, 0.5540582, 0.5445032, 0.5351619, 0.5260785, 0.5172897, 0.5087890, 0.5003645, 0.4916072,\
        0.4822266, 0.4725320, 0.4626619, 0.4526794],\
        [0.2675181, 0.2744984, 0.2811923, 0.2876231, 0.2938295, 0.2997910, 0.3051247, 0.3107041, 0.3166113,\
        0.3227907, 0.3292625, 0.3360654, 0.3432438, 0.3508453, 0.3589180, 0.3675010, 0.3766067, 0.3861936,\
        0.3961553, 0.4065244, 0.4175089, 0.4294509, 0.4426201, 0.4570536, 0.4725175, 0.4885034, 0.5044580,\
        0.5205452, 0.5374023, 0.5557288, 0.5765201, 0.6007729, 0.6282486, 0.6572438, 0.6849668, 0.7081795,\
        0.7246324, 0.7357821, 0.7432468, 0.7477464, 0.7497516, 0.7496544, 0.7478143, 0.7446109, 0.7405138,\
        0.7361644, 0.7324707, 0.7307147, 0.7326460, 0.7403880, 0.7557485, 0.7790708, 0.8085634, 0.8397918,\
        0.8693127, 0.8955359, 0.9156487, 0.9301165, 0.9406911, 0.9481498, 0.9529551, 0.9555291, 0.9562969,\
        0.9556814, 0.9540771, 0.9518248, 0.9491930, 0.9463721, 0.9434771, 0.9405553, 0.9375939, 0.9345264,\
        0.9312394, 0.9275841, 0.9234030, 0.9185812, 0.9131105, 0.9070469, 0.9002723, 0.8924416, 0.8841980,\
        0.8753071, 0.8655322, 0.8549071, 0.8435618, 0.8316781, 0.8195054, 0.8073482, 0.7956466, 0.7839017,\
        0.7716259, 0.7592914, 0.7469218, 0.7344606, 0.7219360, 0.7096656, 0.6976673, 0.6853009, 0.6725647,\
        0.6597167, 0.6469926, 0.6345308, 0.6224178, 0.6107354, 0.5995806, 0.5896396, 0.5802024, 0.5707982,\
        0.5613717, 0.5519976, 0.5427405, 0.5336373, 0.5246921, 0.5158719, 0.5071603, 0.4986212, 0.4901055,\
        0.4811161, 0.4716311, 0.4618251, 0.4518304],\
        [0.2662237, 0.2731521, 0.2797984, 0.2861714, 0.2922422, 0.2978354, 0.3033738, 0.3094022, 0.3155513,\
        0.3219196, 0.3285675, 0.3355334, 0.3428521, 0.3505585, 0.3586854, 0.3672569, 0.3762831, 0.3857708,\
        0.3957454, 0.4058567, 0.4165279, 0.4282130, 0.4409490, 0.4546942, 0.4692321, 0.4841953, 0.4993479,\
        0.5150478, 0.5314079, 0.5489725, 0.5686281, 0.5910034, 0.6163355, 0.6447233, 0.6729597, 0.6974454,\
        0.7143034, 0.7263797, 0.7349239, 0.7404358, 0.7433627, 0.7440646, 0.7428125, 0.7398678, 0.7355791,\
        0.7304812, 0.7253792, 0.7214211, 0.7202174, 0.7241774, 0.7367503, 0.7600514, 0.7915189, 0.8222969,\
        0.8527615, 0.8809787, 0.9036276, 0.9205752, 0.9337219, 0.9434746, 0.9502376, 0.9544006, 0.9563850,\
        0.9566329, 0.9555680, 0.9535655, 0.9509346, 0.9479124, 0.9446653, 0.9412930, 0.9378312, 0.9342509,\
        0.9304570, 0.9262915, 0.9215543, 0.9160624, 0.9097945, 0.9030552, 0.8956143, 0.8872520, 0.8783231,\
        0.8689070, 0.8589051, 0.8482539, 0.8369557, 0.8251076, 0.8129474, 0.8008001, 0.7890505, 0.7774778,\
        0.7657063, 0.7538105, 0.7418270, 0.7296349, 0.7170186, 0.7041356, 0.6923489, 0.6802465, 0.6676975,\
        0.6550526, 0.6425965, 0.6304892, 0.6188109, 0.6077436, 0.5973544, 0.5870890, 0.5774617, 0.5681247,\
        0.5589090, 0.5497868, 0.5407757, 0.5318931, 0.5231277, 0.5144027, 0.5055781, 0.4970704, 0.4887689,\
        0.4798566, 0.4704159, 0.4606248, 0.4506202],\
        [0.2649714, 0.2718414, 0.2784268, 0.2847366, 0.2907703, 0.2965897, 0.3024225, 0.3083671, 0.3145866,\
        0.3210866, 0.3278771, 0.3349688, 0.3423786, 0.3501279, 0.3582360, 0.3667140, 0.3755558, 0.3847259,\
        0.3941320, 0.4039766, 0.4147069, 0.4261498, 0.4384538, 0.4516282, 0.4655324, 0.4799492, 0.4947366,\
        0.5099906, 0.5256160, 0.5421557, 0.5603119, 0.5805553, 0.6041185, 0.6314338, 0.6589204, 0.6828181,\
        0.7011747, 0.7151012, 0.7252703, 0.7321552, 0.7363991, 0.7383558, 0.7381868, 0.7360048, 0.7320018,\
        0.7265795, 0.7204267, 0.7145369, 0.7102038, 0.7093685, 0.7172537, 0.7396979, 0.7710036, 0.8032715,\
        0.8375010, 0.8673827, 0.8919816, 0.9119257, 0.9275842, 0.9394865, 0.9480079, 0.9534958, 0.9564177,\
        0.9572734, 0.9565347, 0.9546169, 0.9518666, 0.9485602, 0.9449075, 0.9410554, 0.9370878, 0.9330185,\
        0.9287776, 0.9242025, 0.9190433, 0.9129938, 0.9058349, 0.8979746, 0.8899171, 0.8810590, 0.8716658,\
        0.8619071, 0.8517855, 0.8412039, 0.8300380, 0.8182547, 0.8061021, 0.7940483, 0.7822354, 0.7706338,\
        0.7591068, 0.7476346, 0.7362019, 0.7246738, 0.7128415, 0.7004493, 0.6877874, 0.6751501, 0.6624161,\
        0.6498541, 0.6376943, 0.6260592, 0.6149154, 0.6040348, 0.5939339, 0.5842039, 0.5746817, 0.5654275,\
        0.5563614, 0.5474309, 0.5386233, 0.5299516, 0.5214426, 0.5131191, 0.5049535, 0.4966387, 0.4877955,\
        0.4785332, 0.4689378, 0.4590856, 0.4490549],\
        [0.2638012, 0.2706202, 0.2771554, 0.2834337, 0.2895146, 0.2955170, 0.3014639, 0.3074052, 0.3136885,\
        0.3203132, 0.3272267, 0.3344109, 0.3418687, 0.3496110, 0.3576488, 0.3659865, 0.3746169, 0.3835194,\
        0.3926980, 0.4023409, 0.4125822, 0.4235543, 0.4353913, 0.4480833, 0.4615405, 0.4756556, 0.4903468,\
        0.5051896, 0.5199009, 0.5355210, 0.5527186, 0.5719470, 0.5937920, 0.6194120, 0.6453561, 0.6684866,\
        0.6876719, 0.7030999, 0.7147427, 0.7229879, 0.7288788, 0.7325355, 0.7339876, 0.7331537, 0.7300196,\
        0.7248519, 0.7182791, 0.7112977, 0.7052058, 0.7012890, 0.7024875, 0.7237661, 0.7562512, 0.7933296,\
        0.8269864, 0.8580395, 0.8848022, 0.9063599, 0.9236142, 0.9372679, 0.9470255, 0.9533166, 0.9567238,\
        0.9578205, 0.9571226, 0.9550742, 0.9520455, 0.9483383, 0.9441927, 0.9397946, 0.9352755, 0.9307000,\
        0.9260358, 0.9211265, 0.9156961, 0.9093623, 0.9016649, 0.8928443, 0.8836356, 0.8740504, 0.8641765,\
        0.8541615, 0.8440449, 0.8336968, 0.8228344, 0.8112356, 0.7990105, 0.7868991, 0.7749636, 0.7632569,\
        0.7518059, 0.7406323, 0.7297347, 0.7189794, 0.7079002, 0.6962441, 0.6830029, 0.6695488, 0.6564794,\
        0.6439480, 0.6320991, 0.6210096, 0.6107050, 0.6008108, 0.5906824, 0.5810055, 0.5716508, 0.5625559,\
        0.5536591, 0.5449053, 0.5362663, 0.5277505, 0.5194164, 0.5113785, 0.5036826, 0.4956154, 0.4863575,\
        0.4768143, 0.4670815, 0.4571726, 0.4471250],\
        [0.2627532, 0.2695396, 0.2760416, 0.2822888, 0.2883346, 0.2942624, 0.3001822, 0.3063281, 0.3128501,\
        0.3196445, 0.3266699, 0.3339150, 0.3413822, 0.3490783, 0.3570101, 0.3651826, 0.3736010, 0.3822814,\
        0.3912789, 0.4005791, 0.4101247, 0.4205765, 0.4319440, 0.4441896, 0.4572136, 0.4709116, 0.4851524,\
        0.4995683, 0.5137474, 0.5289393, 0.5458229, 0.5647957, 0.5863180, 0.6099499, 0.6332797, 0.6551598,\
        0.6744357, 0.6907362, 0.7030222, 0.7129156, 0.7207712, 0.7264679, 0.7300947, 0.7312368, 0.7295861,\
        0.7252971, 0.7189699, 0.7116644, 0.7050174, 0.7010080, 0.7001860, 0.7162234, 0.7479882, 0.7844244,\
        0.8204942, 0.8550716, 0.8831036, 0.9047047, 0.9236216, 0.9379235, 0.9478920, 0.9541942, 0.9574928,\
        0.9583896, 0.9574078, 0.9549915, 0.9515112, 0.9472732, 0.9425303, 0.9374927, 0.9323348, 0.9271826,\
        0.9220584, 0.9168076, 0.9110993, 0.9045656, 0.8964232, 0.8867232, 0.8765541, 0.8662022, 0.8558164,\
        0.8455561, 0.8355135, 0.8255725, 0.8152280, 0.8039043, 0.7917591, 0.7793312, 0.7671118, 0.7552065,\
        0.7436874, 0.7326104, 0.7219999, 0.7117855, 0.7014359, 0.6891685, 0.6760763, 0.6626461, 0.6495396,\
        0.6371451, 0.6256654, 0.6150615, 0.6052737, 0.5962421, 0.5867282, 0.5773447, 0.5682639, 0.5594234,\
        0.5507540, 0.5421972, 0.5337110, 0.5252780, 0.5169226, 0.5087528, 0.5008065, 0.4926652, 0.4838319,\
        0.4744444, 0.4647478, 0.4548530, 0.4448240],\
        [0.2618603, 0.2686424, 0.2751412, 0.2813816, 0.2873989, 0.2932483, 0.2991121, 0.3055135, 0.3122570,\
        0.3191828, 0.3262764, 0.3335410, 0.3409812, 0.3486000, 0.3563995, 0.3643833, 0.3725607, 0.3809483,\
        0.3895284, 0.3981335, 0.4074128, 0.4173643, 0.4282968, 0.4401284, 0.4526774, 0.4657890, 0.4792951,\
        0.4929810, 0.5069965, 0.5222262, 0.5390945, 0.5578475, 0.5785223, 0.6000937, 0.6217337, 0.6424073,\
        0.6611049, 0.6772380, 0.6906300, 0.7024255, 0.7118605, 0.7197599, 0.7261902, 0.7298988, 0.7302558,\
        0.7274519, 0.7220288, 0.7148240, 0.7073058, 0.7017407, 0.6999674, 0.7147062, 0.7473268, 0.7876830,\
        0.8258803, 0.8607836, 0.8899736, 0.9120968, 0.9289452, 0.9417638, 0.9506518, 0.9560966, 0.9586780,\
        0.9589422, 0.9573685, 0.9543658, 0.9502783, 0.9453949, 0.9399598, 0.9341853, 0.9282715, 0.9224159,\
        0.9167364, 0.9110704, 0.9049440, 0.8976990, 0.8890861, 0.8790632, 0.8683693, 0.8574507, 0.8465953,\
        0.8360538, 0.8260152, 0.8165219, 0.8069974, 0.7958405, 0.7835663, 0.7710062, 0.7585588, 0.7464367,\
        0.7347526, 0.7235678, 0.7129176, 0.7025999, 0.6922579, 0.6804778, 0.6676326, 0.6544818, 0.6415710,\
        0.6294158, 0.6184444, 0.6085277, 0.5990653, 0.5903917, 0.5817817, 0.5730838, 0.5644818, 0.5560116,\
        0.5476420, 0.5393257, 0.5310164, 0.5226705, 0.5142375, 0.5057111, 0.4975807, 0.4894664, 0.4808013,\
        0.4715824, 0.4619842, 0.4521489, 0.4421689],\
        [0.2611426, 0.2679551, 0.2744951, 0.2807923, 0.2868896, 0.2928640, 0.2989129, 0.3053379, 0.3120805,\
        0.3190111, 0.3260983, 0.3333354, 0.3407176, 0.3482388, 0.3558917, 0.3636703, 0.3715759, 0.3796265,\
        0.3878834, 0.3965386, 0.4047286, 0.4140556, 0.4247217, 0.4362118, 0.4482871, 0.4607900, 0.4735985,\
        0.4866793, 0.5003833, 0.5156548, 0.5323205, 0.5505390, 0.5700648, 0.5902496, 0.6105847, 0.6300331,\
        0.6478279, 0.6636697, 0.6777779, 0.6904500, 0.7014174, 0.7118730, 0.7218838, 0.7283678, 0.7309429,\
        0.7302659, 0.7267660, 0.7206123, 0.7125176, 0.7044204, 0.7005993, 0.7192254, 0.7527667, 0.7935066,\
        0.8355264, 0.8726797, 0.9008894, 0.9220245, 0.9369814, 0.9475758, 0.9545979, 0.9585663, 0.9599732,\
        0.9592765, 0.9568809, 0.9531343, 0.9483340, 0.9427352, 0.9365569, 0.9299885, 0.9232128, 0.9164789,\
        0.9101028, 0.9039529, 0.8972528, 0.8894344, 0.8803612, 0.8701825, 0.8592561, 0.8479768, 0.8367255,\
        0.8258862, 0.8156265, 0.8062475, 0.7971232, 0.7864645, 0.7743795, 0.7618633, 0.7493373, 0.7370628,\
        0.7252241, 0.7138394, 0.7029962, 0.6925984, 0.6819784, 0.6705487, 0.6582590, 0.6455621, 0.6329502,\
        0.6209737, 0.6104265, 0.6018261, 0.5930382, 0.5846554, 0.5765482, 0.5684713, 0.5604077, 0.5523736,\
        0.5443568, 0.5363254, 0.5282412, 0.5200723, 0.5118024, 0.5033710, 0.4948984, 0.4864968, 0.4776954,\
        0.4684691, 0.4589086, 0.4491221, 0.4391979],\
        [0.2606032, 0.2674789, 0.2741005, 0.2805111, 0.2867741, 0.2929737, 0.2992089, 0.3056275, 0.3122666,\
        0.3191126, 0.3261378, 0.3333155, 0.3406242, 0.3480447, 0.3555573, 0.3631404, 0.3707730, 0.3784382,\
        0.3861158, 0.3936708, 0.4015031, 0.4111182, 0.4216695, 0.4328409, 0.4444597, 0.4564150, 0.4686701,\
        0.4813473, 0.4948675, 0.5095360, 0.5256483, 0.5432094, 0.5617925, 0.5808951, 0.5998658, 0.6179928,\
        0.6346091, 0.6496550, 0.6636929, 0.6770445, 0.6897973, 0.7035248, 0.7166243, 0.7248962, 0.7293864,\
        0.7316637, 0.7319293, 0.7290682, 0.7228877, 0.7160752, 0.7163456, 0.7336577, 0.7652173, 0.8047692,\
        0.8479327, 0.8875183, 0.9151761, 0.9330058, 0.9454471, 0.9536920, 0.9586304, 0.9608726, 0.9608889,\
        0.9590696, 0.9557431, 0.9511866, 0.9456403, 0.9393193, 0.9324175, 0.9250967, 0.9174741, 0.9097138,\
        0.9024119, 0.8957532, 0.8884901, 0.8802472, 0.8709371, 0.8606747, 0.8496862, 0.8382330, 0.8266543,\
        0.8156584, 0.8054023, 0.7956035, 0.7864417, 0.7760653, 0.7644497, 0.7521750, 0.7397185, 0.7273660,\
        0.7154514, 0.7041610, 0.6930551, 0.6824779, 0.6717361, 0.6604927, 0.6486960, 0.6365503, 0.6243771,\
        0.6126171, 0.6019437, 0.5940057, 0.5864944, 0.5788457, 0.5712897, 0.5637561, 0.5561972, 0.5485994,\
        0.5409489, 0.5332212, 0.5253843, 0.5174126, 0.5093336, 0.5012740, 0.4926458, 0.4836738, 0.4745597,\
        0.4651993, 0.4556029, 0.4458321, 0.4359549],\
        [0.2602301, 0.2671921, 0.2739149, 0.2804507, 0.2868785, 0.2933000, 0.2997423, 0.3061031, 0.3126287,\
        0.3193919, 0.3263530, 0.3334698, 0.3407102, 0.3480472, 0.3554525, 0.3628920, 0.3703233, 0.3776981,\
        0.3849832, 0.3922844, 0.4002211, 0.4094238, 0.4195971, 0.4303583, 0.4415098, 0.4529630, 0.4647221,\
        0.4769124, 0.4897839, 0.5037261, 0.5193423, 0.5362190, 0.5538189, 0.5717334, 0.5894450, 0.6063566,\
        0.6212462, 0.6348415, 0.6483065, 0.6621858, 0.6769951, 0.6933200, 0.7075260, 0.7154999, 0.7215326,\
        0.7279563, 0.7349701, 0.7386636, 0.7372676, 0.7341352, 0.7369177, 0.7530891, 0.7834753, 0.8218056,\
        0.8627816, 0.8988995, 0.9255879, 0.9423002, 0.9526522, 0.9587314, 0.9616882, 0.9622422, 0.9608796,\
        0.9579508, 0.9537189, 0.9483902, 0.9421427, 0.9351531, 0.9276064, 0.9196774, 0.9114796, 0.9029673,\
        0.8947629, 0.8872566, 0.8793887, 0.8707391, 0.8612413, 0.8509840, 0.8401254, 0.8288205, 0.8171450,\
        0.8055757, 0.7964059, 0.7864422, 0.7765279, 0.7659687, 0.7544985, 0.7424116, 0.7301449, 0.7178667,\
        0.7056098, 0.6945718, 0.6836043, 0.6727349, 0.6618550, 0.6507713, 0.6394549, 0.6279864, 0.6165081,\
        0.6054070, 0.5954709, 0.5873650, 0.5802194, 0.5731880, 0.5661580, 0.5591000, 0.5519803, 0.5447752,\
        0.5374651, 0.5300265, 0.5224253, 0.5146068, 0.5064934, 0.4981199, 0.4895654, 0.4805515, 0.4712568,\
        0.4617589, 0.4520962, 0.4423182, 0.4324782],\
        [0.2600024, 0.2670644, 0.2738904, 0.2805253, 0.2870355, 0.2935143, 0.3000113, 0.3064272, 0.3129912,\
        0.3197571, 0.3266939, 0.3337705, 0.3409623, 0.3482463, 0.3555965, 0.3629784, 0.3703460, 0.3776462,\
        0.3848608, 0.3921503, 0.4000214, 0.4088859, 0.4186164, 0.4289085, 0.4395816, 0.4505604, 0.4618555,\
        0.4735643, 0.4858597, 0.4991353, 0.5140967, 0.5299325, 0.5463090, 0.5628708, 0.5791625, 0.5944456,\
        0.6072175, 0.6190687, 0.6314344, 0.6447190, 0.6590495, 0.6738568, 0.6858820, 0.6942483, 0.7015926,\
        0.7138398, 0.7317262, 0.7458429, 0.7511415, 0.7514990, 0.7550678, 0.7692809, 0.7993782, 0.8396370,\
        0.8799622, 0.9119823, 0.9347946, 0.9493077, 0.9576204, 0.9617428, 0.9629482, 0.9620195, 0.9594534,\
        0.9555722, 0.9505835, 0.9446208, 0.9377907, 0.9302283, 0.9221250, 0.9136981, 0.9051541, 0.8964527,\
        0.8875627, 0.8790172, 0.8703666, 0.8612365, 0.8515316, 0.8413280, 0.8307654, 0.8200024, 0.8093108,\
        0.7984708, 0.7878972, 0.7778354, 0.7674601, 0.7565960, 0.7450471, 0.7329559, 0.7209082, 0.7092783,\
        0.6972966, 0.6856488, 0.6745094, 0.6634473, 0.6524501, 0.6415106, 0.6306907, 0.6200585, 0.6095334,\
        0.5992269, 0.5899743, 0.5819310, 0.5747737, 0.5679909, 0.5613100, 0.5546162, 0.5478479, 0.5409652,\
        0.5339391, 0.5267436, 0.5193493, 0.5117027, 0.5036647, 0.4951604, 0.4863285, 0.4771681, 0.4677399,\
        0.4581285, 0.4483988, 0.4386064, 0.4288008],\
        [0.2598956, 0.2670690, 0.2740016, 0.2807203, 0.2872530, 0.2936406, 0.3000199, 0.3066239, 0.3133571,\
        0.3201907, 0.3271326, 0.3341857, 0.3413463, 0.3486061, 0.3559517, 0.3633625, 0.3708083, 0.3782477,\
        0.3856335, 0.3929609, 0.4006516, 0.4092907, 0.4186231, 0.4284448, 0.4386556, 0.4491997, 0.4600830,\
        0.4714012, 0.4833839, 0.4963901, 0.5100596, 0.5244776, 0.5394701, 0.5544801, 0.5689022, 0.5819536,\
        0.5930373, 0.6033299, 0.6139656, 0.6253464, 0.6367473, 0.6467980, 0.6542262, 0.6595438, 0.6660553,\
        0.6828007, 0.7155305, 0.7450543, 0.7590010, 0.7635584, 0.7689329, 0.7827739, 0.8108580, 0.8564446,\
        0.8976427, 0.9252737, 0.9429759, 0.9539239, 0.9598369, 0.9621285, 0.9618545, 0.9597336, 0.9562365,\
        0.9516577, 0.9461574, 0.9397915, 0.9325681, 0.9245580, 0.9159778, 0.9071085, 0.8981352, 0.8891308,\
        0.8799465, 0.8707424, 0.8614291, 0.8518220, 0.8418761, 0.8317092, 0.8214735, 0.8112239, 0.8010899,\
        0.7910627, 0.7802838, 0.7696273, 0.7589527, 0.7479632, 0.7364097, 0.7241678, 0.7117405, 0.7010966,\
        0.6895179, 0.6775934, 0.6659923, 0.6546483, 0.6435229, 0.6326701, 0.6222625, 0.6125154, 0.6032317,\
        0.5937510, 0.5848995, 0.5770228, 0.5699188, 0.5632643, 0.5567986, 0.5503604, 0.5438522, 0.5372117,\
        0.5303953, 0.5233677, 0.5160989, 0.5085692, 0.5006995, 0.4920220, 0.4829420, 0.4735947, 0.4640306,\
        0.4543179, 0.4445276, 0.4347229, 0.4249550],\
        [0.2598824, 0.2671803, 0.2742351, 0.2810668, 0.2876850, 0.2940701, 0.3003404, 0.3070696, 0.3138783,\
        0.3207298, 0.3276560, 0.3346786, 0.3418097, 0.3490568, 0.3564239, 0.3639119, 0.3715183, 0.3792354,\
        0.3870372, 0.3947971, 0.4023523, 0.4105769, 0.4194133, 0.4287761, 0.4385835, 0.4487648, 0.4592918,\
        0.4702061, 0.4816295, 0.4936746, 0.5063655, 0.5198528, 0.5336577, 0.5472138, 0.5599529, 0.5713344,\
        0.5811654, 0.5899494, 0.5983609, 0.6072734, 0.6149000, 0.6194662, 0.6211162, 0.6202616, 0.6206644,\
        0.6281823, 0.6723304, 0.7258256, 0.7539883, 0.7667802, 0.7780366, 0.7979922, 0.8312184, 0.8740728,\
        0.9113383, 0.9336639, 0.9471389, 0.9549717, 0.9587389, 0.9595074, 0.9580964, 0.9551181, 0.9510020,\
        0.9460229, 0.9403147, 0.9338618, 0.9265195, 0.9182181, 0.9092319, 0.9000293, 0.8907753, 0.8814105,\
        0.8718964, 0.8622608, 0.8524818, 0.8424799, 0.8322576, 0.8220332, 0.8120812, 0.8022692, 0.7923388,\
        0.7823441, 0.7719670, 0.7613743, 0.7507055, 0.7398914, 0.7287440, 0.7169130, 0.7041404, 0.6924691,\
        0.6814619, 0.6696575, 0.6578417, 0.6462911, 0.6350733, 0.6242442, 0.6140074, 0.6047807, 0.5964935,\
        0.5880015, 0.5798183, 0.5722985, 0.5653859, 0.5588836, 0.5525871, 0.5463356, 0.5400141, 0.5335420,\
        0.5268609, 0.5199214, 0.5126657, 0.5050328, 0.4970137, 0.4884164, 0.4792869, 0.4698309, 0.4601582,\
        0.4503576, 0.4405135, 0.4307012, 0.4209787],\
        [0.2599308, 0.2673642, 0.2745563, 0.2815347, 0.2883411, 0.2950335, 0.3015259, 0.3079565, 0.3146040,\
        0.3213707, 0.3282335, 0.3351998, 0.3422853, 0.3495077, 0.3568861, 0.3644401, 0.3721901, 0.3801534,\
        0.3883319, 0.3966645, 0.4044518, 0.4121564, 0.4205572, 0.4295871, 0.4391309, 0.4490822, 0.4593590,\
        0.4699182, 0.4807584, 0.4919274, 0.5038119, 0.5165700, 0.5293925, 0.5417800, 0.5532989, 0.5636151,\
        0.5726373, 0.5805578, 0.5876939, 0.5943548, 0.5981508, 0.5981940, 0.5953197, 0.5874495, 0.5786275,\
        0.5733781, 0.6012456, 0.6737306, 0.7304124, 0.7608872, 0.7849233, 0.8118877, 0.8483210, 0.8865344,\
        0.9166776, 0.9350716, 0.9458295, 0.9516833, 0.9540320, 0.9537786, 0.9516257, 0.9481253, 0.9436853,\
        0.9385794, 0.9329572, 0.9267989, 0.9197506, 0.9113652, 0.9020464, 0.8925905, 0.8830477, 0.8733395,\
        0.8634903, 0.8535343, 0.8434709, 0.8332322, 0.8227456, 0.8122542, 0.8024921, 0.7930261, 0.7833764,\
        0.7735038, 0.7633339, 0.7529206, 0.7424092, 0.7319212, 0.7215159, 0.7108974, 0.6991508, 0.6864412,\
        0.6741060, 0.6619287, 0.6499362, 0.6383085, 0.6271466, 0.6164600, 0.6062579, 0.5973259, 0.5895765,\
        0.5820444, 0.5746605, 0.5676455, 0.5610384, 0.5547431, 0.5486167, 0.5425202, 0.5363371, 0.5299760,\
        0.5233690, 0.5164653, 0.5092012, 0.5014590, 0.4932685, 0.4846079, 0.4754528, 0.4659325, 0.4561765,\
        0.4462983, 0.4364038, 0.4265872, 0.4169186],\
        [0.2600043, 0.2675783, 0.2749067, 0.2820151, 0.2889462, 0.2957458, 0.3023587, 0.3088141, 0.3153609,\
        0.3220300, 0.3288041, 0.3356880, 0.3427008, 0.3498674, 0.3572159, 0.3647756, 0.3725738, 0.3806226,\
        0.3888769, 0.3971137, 0.4051541, 0.4131497, 0.4215508, 0.4305289, 0.4400339, 0.4499561, 0.4601751,\
        0.4705833, 0.4811248, 0.4918947, 0.5032111, 0.5151059, 0.5270509, 0.5385742, 0.5493365, 0.5591288,\
        0.5679145, 0.5757601, 0.5825067, 0.5874645, 0.5896716, 0.5887344, 0.5832059, 0.5724753, 0.5584893,\
        0.5492832, 0.5657476, 0.6248051, 0.6978542, 0.7555257, 0.7975361, 0.8317560, 0.8643373, 0.8943792,\
        0.9163755, 0.9306548, 0.9395898, 0.9444293, 0.9460521, 0.9452426, 0.9426885, 0.9389392, 0.9344034,\
        0.9293561, 0.9239789, 0.9184257, 0.9123452, 0.9041677, 0.8946121, 0.8848109, 0.8748674, 0.8648089,\
        0.8546621, 0.8444774, 0.8343046, 0.8241059, 0.8136615, 0.8028064, 0.7929837, 0.7837621, 0.7743510,\
        0.7646490, 0.7546186, 0.7443048, 0.7338592, 0.7234878, 0.7134560, 0.7039028, 0.6925446, 0.6796435,\
        0.6667750, 0.6542650, 0.6421737, 0.6306080, 0.6197000, 0.6095450, 0.5999692, 0.5911780, 0.5835871,\
        0.5765001, 0.5696781, 0.5631264, 0.5568504, 0.5507902, 0.5448424, 0.5388896, 0.5328191, 0.5265316,\
        0.5199487, 0.5130268, 0.5057559, 0.4980067, 0.4896432, 0.4808335, 0.4715957, 0.4620034, 0.4521683,\
        0.4422130, 0.4322639, 0.4224407, 0.4128340],\
        [0.2600658, 0.2677827, 0.2752418, 0.2824514, 0.2894246, 0.2961840, 0.3028268, 0.3094343, 0.3160103,\
        0.3226198, 0.3293007, 0.3360813, 0.3429913, 0.3500620, 0.3573257, 0.3648133, 0.3725490, 0.3805378,\
        0.3887454, 0.3970951, 0.4052356, 0.4134724, 0.4221429, 0.4313405, 0.4410468, 0.4511673, 0.4615682,\
        0.4720977, 0.4826220, 0.4931774, 0.5040705, 0.5153334, 0.5266201, 0.5375995, 0.5480405, 0.5578175,\
        0.5669587, 0.5756349, 0.5835919, 0.5890096, 0.5919979, 0.5935127, 0.5889242, 0.5806284, 0.5692851,\
        0.5604898, 0.5723231, 0.6131037, 0.6872768, 0.7790409, 0.8244796, 0.8576680, 0.8803146, 0.8982907,\
        0.9116231, 0.9223757, 0.9301839, 0.9345297, 0.9357339, 0.9345773, 0.9318036, 0.9279644, 0.9234722,\
        0.9185839, 0.9134038, 0.9081928, 0.9037084, 0.8964406, 0.8864124, 0.8761875, 0.8659282, 0.8556282,\
        0.8452755, 0.8349318, 0.8247429, 0.8148302, 0.8049233, 0.7945195, 0.7843350, 0.7748564, 0.7654767,\
        0.7558894, 0.7459512, 0.7356371, 0.7250894, 0.7145794, 0.7041062, 0.6943534, 0.6836176, 0.6713902,\
        0.6587911, 0.6463971, 0.6344487, 0.6230908, 0.6125198, 0.6028785, 0.5940950, 0.5858074, 0.5783413,\
        0.5714925, 0.5650197, 0.5588187, 0.5528352, 0.5470034, 0.5412336, 0.5354210, 0.5294560, 0.5232309,\
        0.5166459, 0.5096303, 0.5022172, 0.4944046, 0.4860227, 0.4771327, 0.4678134, 0.4581454, 0.4482316,\
        0.4381952, 0.4281798, 0.4183398, 0.4088008],\
        [0.2600772, 0.2679423, 0.2755393, 0.2828613, 0.2899003, 0.2966868, 0.3033661, 0.3099976, 0.3165594,\
        0.3230982, 0.3296724, 0.3363291, 0.3431079, 0.3500449, 0.3571732, 0.3645218, 0.3721107, 0.3799420,\
        0.3879846, 0.3961545, 0.4045120, 0.4131922, 0.4222753, 0.4318653, 0.4419551, 0.4524628, 0.4632603,\
        0.4741826, 0.4850040, 0.4954262, 0.5060204, 0.5168953, 0.5277532, 0.5385039, 0.5490474, 0.5593168,\
        0.5693549, 0.5795251, 0.5908731, 0.6005936, 0.6076424, 0.6133289, 0.6115516, 0.6093151, 0.6048817,\
        0.6099000, 0.6281666, 0.6732202, 0.7765629, 0.8642729, 0.8835117, 0.8924877, 0.8969277, 0.8991126,\
        0.9045037, 0.9129710, 0.9200359, 0.9237750, 0.9242862, 0.9226530, 0.9196727, 0.9157810, 0.9113855,\
        0.9067352, 0.9017959, 0.8965299, 0.8918764, 0.8853000, 0.8760658, 0.8660482, 0.8558772, 0.8456170,\
        0.8352287, 0.8247834, 0.8145141, 0.8047789, 0.7953629, 0.7854226, 0.7755322, 0.7660187, 0.7566886,\
        0.7472459, 0.7374386, 0.7271062, 0.7163172, 0.7055794, 0.6951898, 0.6847372, 0.6739627, 0.6623313,\
        0.6502748, 0.6383184, 0.6267965, 0.6158203, 0.6055922, 0.5965257, 0.5882724, 0.5805988, 0.5734856,\
        0.5668871, 0.5606740, 0.5547363, 0.5489919, 0.5433627, 0.5377605, 0.5320855, 0.5262306, 0.5200853,\
        0.5135348, 0.5064637, 0.4988525, 0.4909487, 0.4825103, 0.4735564, 0.4641723, 0.4544448, 0.4444687,\
        0.4343585, 0.4242655, 0.4143880, 0.4049196],\
        [0.2599973, 0.2680161, 0.2757665, 0.2832385, 0.2904140, 0.2972654, 0.3039524, 0.3105393, 0.3170098,\
        0.3234363, 0.3298803, 0.3363936, 0.3430194, 0.3497954, 0.3567539, 0.3639223, 0.3713204, 0.3789593,\
        0.3868449, 0.3950008, 0.4035183, 0.4124682, 0.4219572, 0.4320062, 0.4425689, 0.4535614, 0.4648648,\
        0.4763304, 0.4877896, 0.4989027, 0.5089617, 0.5192671, 0.5298261, 0.5406194, 0.5516602, 0.5629651,\
        0.5745795, 0.5867554, 0.6009582, 0.6193740, 0.6350927, 0.6457760, 0.6502026, 0.6508663, 0.6529844,\
        0.6757739, 0.7338855, 0.8194761, 0.8990518, 0.9325049, 0.9324716, 0.9228362, 0.9120648, 0.9029000,\
        0.9004107, 0.9058442, 0.9114962, 0.9137837, 0.9127209, 0.9102882, 0.9070715, 0.9031012, 0.8987994,\
        0.8943595, 0.8895774, 0.8843973, 0.8788345, 0.8720107, 0.8635502, 0.8542127, 0.8445711, 0.8347163,\
        0.8245372, 0.8140976, 0.8038028, 0.7940436, 0.7848508, 0.7754915, 0.7660936, 0.7568603, 0.7477632,\
        0.7386153, 0.7291184, 0.7189247, 0.7079780, 0.6967942, 0.6861995, 0.6755183, 0.6645538, 0.6531927,\
        0.6416095, 0.6301787, 0.6193351, 0.6091854, 0.5993401, 0.5907836, 0.5829727, 0.5756962, 0.5689101,\
        0.5625638, 0.5565726, 0.5508439, 0.5452921, 0.5398343, 0.5343825, 0.5288389, 0.5230964, 0.5170495,\
        0.5105973, 0.5036018, 0.4959613, 0.4877971, 0.4791897, 0.4701443, 0.4607130, 0.4509650, 0.4409766,\
        0.4308384, 0.4206811, 0.4107353, 0.4013350],\
        [0.2597794, 0.2679495, 0.2758568, 0.2834944, 0.2908609, 0.2979636, 0.3045779, 0.3109741, 0.3172999,\
        0.3235871, 0.3298860, 0.3362453, 0.3427077, 0.3493104, 0.3560854, 0.3630600, 0.3702572, 0.3777012,\
        0.3854338, 0.3935484, 0.4021214, 0.4112999, 0.4211939, 0.4316982, 0.4427273, 0.4541940, 0.4659764,\
        0.4778923, 0.4896751, 0.5009166, 0.5113311, 0.5214971, 0.5319796, 0.5430168, 0.5548044, 0.5675482,\
        0.5814635, 0.5970826, 0.6162307, 0.6401421, 0.6642202, 0.6801740, 0.6901269, 0.6980264, 0.7075980,\
        0.7426567, 0.7964154, 0.8650692, 0.9267167, 0.9516687, 0.9494497, 0.9357035, 0.9198644, 0.9067112,\
        0.9001778, 0.9020290, 0.9050986, 0.9051638, 0.9016405, 0.8981613, 0.8946604, 0.8905915, 0.8863025,\
        0.8818015, 0.8769075, 0.8714420, 0.8652133, 0.8579438, 0.8496967, 0.8409647, 0.8320773, 0.8229545,\
        0.8133020, 0.8029838, 0.7928680, 0.7832648, 0.7740949, 0.7650744, 0.7561089, 0.7472402, 0.7384760,\
        0.7297552, 0.7208392, 0.7110836, 0.7001585, 0.6888276, 0.6776375, 0.6665719, 0.6554694, 0.6442894,\
        0.6331132, 0.6221297, 0.6118306, 0.6031529, 0.5943374, 0.5857855, 0.5781299, 0.5711028, 0.5645695,\
        0.5584441, 0.5526444, 0.5470864, 0.5416883, 0.5363694, 0.5310458, 0.5256223, 0.5199843, 0.5140105,\
        0.5076396, 0.5007667, 0.4930251, 0.4847143, 0.4759750, 0.4668628, 0.4574282, 0.4477281, 0.4378217,\
        0.4277721, 0.4176587, 0.4076474, 0.3982978],\
        [0.2593762, 0.2676829, 0.2757286, 0.2834991, 0.2909561, 0.2979999, 0.3047021, 0.3111089, 0.3173336,\
        0.3234921, 0.3296512, 0.3358608, 0.3421632, 0.3485959, 0.3551924, 0.3619825, 0.3689942, 0.3762584,\
        0.3838191, 0.3917533, 0.4002689, 0.4097707, 0.4200426, 0.4309150, 0.4423093, 0.4541492, 0.4663100,\
        0.4785724, 0.4905583, 0.5018669, 0.5125332, 0.5228122, 0.5333708, 0.5446862, 0.5571344, 0.5711364,\
        0.5871925, 0.6054249, 0.6272457, 0.6537925, 0.6821670, 0.7014498, 0.7127254, 0.7300429, 0.7553049,\
        0.7971387, 0.8353595, 0.8787071, 0.9181697, 0.9383048, 0.9393818, 0.9295373, 0.9162722, 0.9047195,\
        0.8982896, 0.8976709, 0.8976028, 0.8958003, 0.8912176, 0.8865587, 0.8823251, 0.8780091, 0.8735727,\
        0.8689155, 0.8638146, 0.8580025, 0.8512217, 0.8433957, 0.8349456, 0.8265844, 0.8185053, 0.8101957,\
        0.8014204, 0.7914401, 0.7815206, 0.7720753, 0.7630537, 0.7543048, 0.7456957, 0.7371555, 0.7286625,\
        0.7202725, 0.7120446, 0.7032407, 0.6922419, 0.6806429, 0.6691100, 0.6577705, 0.6466198, 0.6356792,\
        0.6250110, 0.6146233, 0.6043823, 0.5960096, 0.5885077, 0.5807552, 0.5734541, 0.5666908, 0.5603873,\
        0.5544588, 0.5488245, 0.5434057, 0.5381258, 0.5329100, 0.5276840, 0.5223657, 0.5168398, 0.5109129,\
        0.5044016, 0.4974967, 0.4898396, 0.4815132, 0.4727372, 0.4636269, 0.4542583, 0.4446936, 0.4349903,\
        0.4252063, 0.4154102, 0.4057056, 0.3964834],\
        [0.2587467, 0.2671655, 0.2753197, 0.2831938, 0.2907512, 0.2979212, 0.3045998, 0.3109135, 0.3170479,\
        0.3231044, 0.3291465, 0.3352242, 0.3413818, 0.3476598, 0.3540961, 0.3607265, 0.3675875, 0.3747203,\
        0.3821792, 0.3900561, 0.3985695, 0.4081825, 0.4186183, 0.4296534, 0.4412292, 0.4532740, 0.4656660,\
        0.4782114, 0.4905963, 0.5022106, 0.5126346, 0.5228023, 0.5333948, 0.5448390, 0.5575184, 0.5718437,\
        0.5881835, 0.6067298, 0.6288312, 0.6542420, 0.6797022, 0.6996777, 0.7166956, 0.7391981, 0.7695350,\
        0.8045172, 0.8339967, 0.8640733, 0.8906346, 0.9070446, 0.9120988, 0.9086970, 0.9013117, 0.8939551,\
        0.8891737, 0.8871644, 0.8857009, 0.8830299, 0.8788034, 0.8739978, 0.8692959, 0.8647248, 0.8601758,\
        0.8554433, 0.8502440, 0.8442636, 0.8371434, 0.8286829, 0.8195976, 0.8113044, 0.8040292, 0.7962992,\
        0.7879018, 0.7787778, 0.7694158, 0.7603238, 0.7516038, 0.7431960, 0.7349525, 0.7267134, 0.7183802,\
        0.7099442, 0.7018575, 0.6934632, 0.6831079, 0.6717433, 0.6602461, 0.6489054, 0.6378636, 0.6272541,\
        0.6172244, 0.6078938, 0.5990045, 0.5903191, 0.5828048, 0.5756423, 0.5687983, 0.5623523, 0.5562877,\
        0.5505457, 0.5450568, 0.5397497, 0.5345535, 0.5293993, 0.5242232, 0.5189704, 0.5135801, 0.5078889,\
        0.5013911, 0.4942044, 0.4864792, 0.4781426, 0.4693834, 0.4603395, 0.4511057, 0.4417563, 0.4323549,\
        0.4229662, 0.4136837, 0.4046335, 0.3956726],\
        [0.2578630, 0.2663648, 0.2745907, 0.2825251, 0.2901474, 0.2974091, 0.3040554, 0.3103068, 0.3164021,\
        0.3224003, 0.3283581, 0.3343291, 0.3403634, 0.3465079, 0.3528075, 0.3593071, 0.3660553, 0.3731113,\
        0.3805547, 0.3885027, 0.3971360, 0.4066563, 0.4169337, 0.4278743, 0.4394229, 0.4514720, 0.4638758,\
        0.4764448, 0.4889058, 0.5006211, 0.5108970, 0.5211126, 0.5317645, 0.5431643, 0.5556156, 0.5694149,\
        0.5848674, 0.6025564, 0.6220912, 0.6448819, 0.6674296, 0.6868385, 0.7051451, 0.7284908, 0.7563110,\
        0.7829930, 0.8067986, 0.8317183, 0.8516870, 0.8664768, 0.8759197, 0.8789676, 0.8778983, 0.8753919,\
        0.8731328, 0.8715710, 0.8700341, 0.8676174, 0.8639902, 0.8595257, 0.8548048, 0.8501995, 0.8457286,\
        0.8411092, 0.8360100, 0.8301635, 0.8232271, 0.8144111, 0.8042825, 0.7959597, 0.7888453, 0.7815681,\
        0.7736468, 0.7652398, 0.7566032, 0.7480449, 0.7397530, 0.7317944, 0.7240031, 0.7160954, 0.7079662,\
        0.6995407, 0.6911335, 0.6825998, 0.6728520, 0.6621250, 0.6510175, 0.6399288, 0.6291329, 0.6188926,\
        0.6093969, 0.6008167, 0.5927722, 0.5848462, 0.5775018, 0.5706613, 0.5641887, 0.5580521, 0.5522246,\
        0.5466593, 0.5412979, 0.5360773, 0.5309329, 0.5257980, 0.5206072, 0.5153173, 0.5099367, 0.5044823,\
        0.4985591, 0.4909927, 0.4829595, 0.4745539, 0.4658367, 0.4569059, 0.4478570, 0.4387733, 0.4297222,\
        0.4207497, 0.4118808, 0.4031615, 0.3944771],\
        [0.2567127, 0.2652715, 0.2735343, 0.2814741, 0.2890436, 0.2961590, 0.3028510, 0.3092609, 0.3154051,\
        0.3213887, 0.3272918, 0.3331795, 0.3391114, 0.3451433, 0.3513279, 0.3577188, 0.3643755, 0.3713720,\
        0.3788081, 0.3868190, 0.3955429, 0.4048788, 0.4147839, 0.4254834, 0.4368656, 0.4487511, 0.4609488,\
        0.4732237, 0.4852336, 0.4965036, 0.5072116, 0.5178165, 0.5286003, 0.5398550, 0.5518266, 0.5646981,\
        0.5785785, 0.5931927, 0.6105273, 0.6315417, 0.6517896, 0.6699541, 0.6874941, 0.7084622, 0.7326466,\
        0.7551833, 0.7758972, 0.7961905, 0.8096727, 0.8244767, 0.8382567, 0.8458910, 0.8500332, 0.8519916,\
        0.8524210, 0.8521329, 0.8514595, 0.8499411, 0.8471238, 0.8431092, 0.8385015, 0.8340471, 0.8299398,\
        0.8256313, 0.8208213, 0.8151523, 0.8091806, 0.8007981, 0.7905811, 0.7816678, 0.7740707, 0.7667537,\
        0.7592243, 0.7514352, 0.7434879, 0.7355061, 0.7276380, 0.7201932, 0.7130091, 0.7054429, 0.6974932,\
        0.6892526, 0.6807638, 0.6719757, 0.6624867, 0.6522881, 0.6416871, 0.6310066, 0.6205338, 0.6106739,\
        0.6017649, 0.5936724, 0.5862638, 0.5790987, 0.5722546, 0.5657822, 0.5596394, 0.5537835, 0.5481757,\
        0.5427710, 0.5375170, 0.5323587, 0.5272398, 0.5220970, 0.5168484, 0.5114058, 0.5057906, 0.5000704,\
        0.4940125, 0.4868889, 0.4789784, 0.4706406, 0.4620261, 0.4532468, 0.4444078, 0.4356051, 0.4269153,\
        0.4183703, 0.4098961, 0.4013311, 0.3928572],\
        [0.2552970, 0.2638972, 0.2721822, 0.2801199, 0.2876678, 0.2947888, 0.3015306, 0.3079736, 0.3141292,\
        0.3201063, 0.3259685, 0.3317868, 0.3376319, 0.3435681, 0.3496540, 0.3559479, 0.3625135, 0.3694261,\
        0.3767781, 0.3846758, 0.3932014, 0.4022247, 0.4119045, 0.4224425, 0.4336219, 0.4452498, 0.4571289,\
        0.4690220, 0.4806472, 0.4917948, 0.5026489, 0.5135003, 0.5243349, 0.5354029, 0.5468873, 0.5588825,\
        0.5714260, 0.5848162, 0.6008264, 0.6189456, 0.6368246, 0.6538464, 0.6707081, 0.6891699, 0.7092666,\
        0.7284482, 0.7466848, 0.7630707, 0.7756852, 0.7910303, 0.8046300, 0.8136864, 0.8214811, 0.8269729,\
        0.8292843, 0.8300672, 0.8306784, 0.8304441, 0.8285577, 0.8249515, 0.8203421, 0.8160082, 0.8127276,\
        0.8086675, 0.8044762, 0.7986892, 0.7931622, 0.7856404, 0.7763693, 0.7675373, 0.7596360, 0.7522470,\
        0.7449972, 0.7377618, 0.7304960, 0.7231654, 0.7155835, 0.7084386, 0.7021632, 0.6948220, 0.6870387,\
        0.6789194, 0.6704895, 0.6616937, 0.6524075, 0.6426399, 0.6325718, 0.6224324, 0.6124042, 0.6027714,\
        0.5945602, 0.5870784, 0.5801049, 0.5734581, 0.5670816, 0.5609825, 0.5551464, 0.5495417, 0.5441291,\
        0.5388632, 0.5336939, 0.5285715, 0.5234520, 0.5182923, 0.5130196, 0.5074606, 0.5014769, 0.4955732,\
        0.4892785, 0.4822653, 0.4745612, 0.4663881, 0.4579281, 0.4493203, 0.4406877, 0.4321436, 0.4237831,\
        0.4156701, 0.4077860, 0.3997344, 0.3912889],\
        [0.2536243, 0.2622610, 0.2705717, 0.2785268, 0.2860986, 0.2932903, 0.3001152, 0.3064421, 0.3126222,\
        0.3185964, 0.3244130, 0.3301633, 0.3359309, 0.3417848, 0.3477849, 0.3539886, 0.3604563, 0.3672536,\
        0.3744489, 0.3820988, 0.3902155, 0.3988446, 0.4085135, 0.4189290, 0.4298624, 0.4411817, 0.4527216,\
        0.4642873, 0.4756894, 0.4868394, 0.4978279, 0.5086477, 0.5193934, 0.5303112, 0.5414770, 0.5529818,\
        0.5650131, 0.5780513, 0.5927225, 0.6081801, 0.6237629, 0.6398674, 0.6559018, 0.6722931, 0.6888042,\
        0.7049935, 0.7212788, 0.7366877, 0.7504134, 0.7641548, 0.7762637, 0.7860776, 0.7951626, 0.8027282,\
        0.8052842, 0.8064798, 0.8083952, 0.8094256, 0.8083895, 0.8051580, 0.8006621, 0.7962985, 0.7938350,\
        0.7901362, 0.7861066, 0.7811659, 0.7754686, 0.7687162, 0.7608160, 0.7527870, 0.7452230, 0.7380722,\
        0.7311491, 0.7244981, 0.7179190, 0.7114453, 0.7047166, 0.6969917, 0.6910676, 0.6841735, 0.6766246,\
        0.6686847, 0.6604031, 0.6517614, 0.6427330, 0.6333757, 0.6238559, 0.6143843, 0.6051532, 0.5961858,\
        0.5881628, 0.5810247, 0.5743849, 0.5680907, 0.5620662, 0.5562802, 0.5507086, 0.5453200, 0.5400743,\
        0.5349239, 0.5298158, 0.5246990, 0.5195389, 0.5143277, 0.5090612, 0.5036845, 0.4976556, 0.4912353,\
        0.4846148, 0.4775188, 0.4699084, 0.4618835, 0.4535818, 0.4451377, 0.4366814, 0.4283407, 0.4202242,\
        0.4123839, 0.4049069, 0.3975592, 0.3891961],\
        [0.2517027, 0.2603772, 0.2687232, 0.2767141, 0.2843171, 0.2914856, 0.2981508, 0.3046948, 0.3109666,\
        0.3168986, 0.3226383, 0.3283133, 0.3340118, 0.3397980, 0.3457264, 0.3518491, 0.3582196, 0.3648932,\
        0.3719244, 0.3793622, 0.3872646, 0.3957980, 0.4051548, 0.4151976, 0.4257787, 0.4367462, 0.4479560,\
        0.4592482, 0.4704804, 0.4815783, 0.4925280, 0.5032517, 0.5140339, 0.5249324, 0.5359579, 0.5472392,\
        0.5589931, 0.5714892, 0.5847283, 0.5978615, 0.6130023, 0.6284768, 0.6434337, 0.6580654, 0.6724407,\
        0.6866713, 0.7017863, 0.7163244, 0.7295942, 0.7421932, 0.7533828, 0.7631112, 0.7717237, 0.7784568,\
        0.7815589, 0.7832461, 0.7852176, 0.7865130, 0.7859518, 0.7833635, 0.7796842, 0.7760934, 0.7732454,\
        0.7701636, 0.7665396, 0.7622591, 0.7572210, 0.7513451, 0.7446688, 0.7376631, 0.7309210, 0.7244586,\
        0.7178433, 0.7118926, 0.7060621, 0.6998915, 0.6944747, 0.6873304, 0.6805452, 0.6736767, 0.6663543,\
        0.6586443, 0.6506033, 0.6422324, 0.6335270, 0.6245753, 0.6155817, 0.6067601, 0.5982474, 0.5900719,\
        0.5824293, 0.5754809, 0.5690554, 0.5630009, 0.5572267, 0.5516787, 0.5463177, 0.5411047, 0.5359960,\
        0.5309400, 0.5258764, 0.5207407, 0.5154872, 0.5101303, 0.5047378, 0.4992296, 0.4933086, 0.4867548,\
        0.4799006, 0.4727077, 0.4651356, 0.4572250, 0.4490643, 0.4407620, 0.4324397, 0.4242345, 0.4162892,\
        0.4086638, 0.4010563, 0.3938778, 0.3862593],\
        [0.2495365, 0.2582500, 0.2666398, 0.2746861, 0.2823651, 0.2896525, 0.2965659, 0.3031243, 0.3092338,\
        0.3150007, 0.3206234, 0.3262269, 0.3318756, 0.3376158, 0.3434915, 0.3495481, 0.3558328, 0.3623952,\
        0.3692876, 0.3765696, 0.3843244, 0.3926670, 0.4016628, 0.4113162, 0.4214878, 0.4320787, 0.4429762,\
        0.4540289, 0.4650982, 0.4760881, 0.4869363, 0.4976707, 0.5085698, 0.5194839, 0.5304363, 0.5415956,\
        0.5531585, 0.5652737, 0.5779307, 0.5912745, 0.6055378, 0.6196977, 0.6333433, 0.6465762, 0.6596579,\
        0.6729682, 0.6867751, 0.6995627, 0.7125573, 0.7240266, 0.7341053, 0.7428487, 0.7500704, 0.7554646,\
        0.7586386, 0.7603917, 0.7616992, 0.7624752, 0.7621131, 0.7603890, 0.7577413, 0.7548959, 0.7522659,\
        0.7496202, 0.7466165, 0.7430725, 0.7389422, 0.7341777, 0.7287329, 0.7226973, 0.7169682, 0.7120919,\
        0.7056620, 0.6999664, 0.6951882, 0.6892462, 0.6834902, 0.6771210, 0.6704025, 0.6635133, 0.6563393,\
        0.6488714, 0.6411502, 0.6331715, 0.6248712, 0.6163026, 0.6077996, 0.5995599, 0.5916795, 0.5841045,\
        0.5769268, 0.5702367, 0.5639975, 0.5581202, 0.5525252, 0.5471512, 0.5419483, 0.5368696, 0.5318673,\
        0.5268879, 0.5218663, 0.5167167, 0.5113487, 0.5057537, 0.5001140, 0.4944473, 0.4884112, 0.4819042,\
        0.4750171, 0.4678075, 0.4602887, 0.4524898, 0.4444669, 0.4362971, 0.4280796, 0.4199439, 0.4120676,\
        0.4047113, 0.3977729, 0.3904141, 0.3830156],\
        [0.2471272, 0.2558763, 0.2643085, 0.2724136, 0.2801846, 0.2876233, 0.2947241, 0.3013392, 0.3072227,\
        0.3127739, 0.3183105, 0.3238908, 0.3295289, 0.3352528, 0.3410996, 0.3471097, 0.3533260, 0.3597947,\
        0.3665678, 0.3737070, 0.3812887, 0.3893936, 0.3980754, 0.4073252, 0.4170339, 0.4272549, 0.4378807,\
        0.4487265, 0.4596458, 0.4705492, 0.4814054, 0.4922663, 0.5031736, 0.5140016, 0.5248580, 0.5359158,\
        0.5473686, 0.5593761, 0.5720473, 0.5854557, 0.5992607, 0.6124424, 0.6248550, 0.6369211, 0.6489604,\
        0.6612543, 0.6737515, 0.6861972, 0.6985688, 0.7079115, 0.7171836, 0.7246102, 0.7301430, 0.7343555,\
        0.7370217, 0.7381553, 0.7385981, 0.7388627, 0.7386594, 0.7376649, 0.7358503, 0.7336487, 0.7314970,\
        0.7294432, 0.7271497, 0.7243557, 0.7212139, 0.7176882, 0.7136807, 0.7087421, 0.7032993, 0.7008525,\
        0.6960313, 0.6895935, 0.6843647, 0.6787897, 0.6729752, 0.6668334, 0.6603522, 0.6536020, 0.6465958,\
        0.6393749, 0.6320308, 0.6246083, 0.6169163, 0.6086767, 0.6006165, 0.5929625, 0.5855567, 0.5784153,\
        0.5715963, 0.5651550, 0.5590905, 0.5533572, 0.5478959, 0.5426483, 0.5375591, 0.5325753, 0.5276459,\
        0.5227210, 0.5177443, 0.5126274, 0.5072122, 0.5014170, 0.4955226, 0.4895971, 0.4834061, 0.4768624,\
        0.4699866, 0.4628211, 0.4553956, 0.4477372, 0.4398798, 0.4318692, 0.4237737, 0.4157001, 0.4078008,\
        0.4003404, 0.3936475, 0.3868304, 0.3797392],\
        [0.2444764, 0.2532518, 0.2617124, 0.2698515, 0.2776643, 0.2851462, 0.2922691, 0.2988926, 0.3045612,\
        0.3100593, 0.3156769, 0.3213233, 0.3269965, 0.3327342, 0.3385762, 0.3445604, 0.3507265, 0.3571174,\
        0.3637815, 0.3707739, 0.3781579, 0.3860027, 0.3943778, 0.4031729, 0.4123840, 0.4223410, 0.4327683,\
        0.4434289, 0.4541904, 0.4649904, 0.4758155, 0.4866824, 0.4975544, 0.5083195, 0.5190597, 0.5299949,\
        0.5413400, 0.5532716, 0.5658898, 0.5791419, 0.5925954, 0.6051403, 0.6168624, 0.6279959, 0.6391206,\
        0.6504895, 0.6620086, 0.6734177, 0.6839478, 0.6931340, 0.7022521, 0.7077157, 0.7116070, 0.7150165,\
        0.7173577, 0.7174242, 0.7168244, 0.7167836, 0.7167726, 0.7164622, 0.7151937, 0.7134571, 0.7117808,\
        0.7104433, 0.7089680, 0.7067460, 0.7045423, 0.7022044, 0.6994663, 0.6964837, 0.6916722, 0.6888105,\
        0.6849018, 0.6794201, 0.6739171, 0.6683815, 0.6626642, 0.6567021, 0.6504569, 0.6439226, 0.6371163,\
        0.6301195, 0.6231181, 0.6163725, 0.6097859, 0.6022726, 0.5942972, 0.5869551, 0.5798306, 0.5729618,\
        0.5663928, 0.5601512, 0.5542370, 0.5486230, 0.5432650, 0.5381101, 0.5331003, 0.5281758, 0.5232807,\
        0.5183686, 0.5134051, 0.5083476, 0.5030406, 0.4971069, 0.4909507, 0.4847236, 0.4783386, 0.4717210,\
        0.4648577, 0.4577680, 0.4504771, 0.4430069, 0.4353741, 0.4275927, 0.4196947, 0.4117800, 0.4040409,\
        0.3967410, 0.3900401, 0.3834117, 0.3765718],\
        [0.2415908, 0.2503808, 0.2588509, 0.2669903, 0.2747832, 0.2821973, 0.2891524, 0.2954673, 0.3011205,\
        0.3069889, 0.3128556, 0.3185990, 0.3243229, 0.3300935, 0.3359507, 0.3419281, 0.3480611, 0.3543890,\
        0.3609546, 0.3678024, 0.3749753, 0.3825058, 0.3903920, 0.3985926, 0.4076424, 0.4175235, 0.4277833,\
        0.4382208, 0.4487640, 0.4593838, 0.4700691, 0.4808109, 0.4915699, 0.5022529, 0.5128594, 0.5236188,\
        0.5347847, 0.5465361, 0.5589302, 0.5718224, 0.5847792, 0.5972831, 0.6085624, 0.6188760, 0.6293407,\
        0.6400013, 0.6505725, 0.6607693, 0.6702737, 0.6790142, 0.6866253, 0.6912805, 0.6945416, 0.6975293,\
        0.7000532, 0.6990914, 0.6977119, 0.6975258, 0.6975718, 0.6977877, 0.6967992, 0.6954788, 0.6942492,\
        0.6934390, 0.6925790, 0.6910145, 0.6895059, 0.6879454, 0.6860449, 0.6835964, 0.6803210, 0.6770528,\
        0.6732595, 0.6685434, 0.6633625, 0.6579855, 0.6524443, 0.6467166, 0.6407563, 0.6345043, 0.6279337,\
        0.6211241, 0.6142916, 0.6079256, 0.6025943, 0.5964543, 0.5887093, 0.5812995, 0.5742926, 0.5676066,\
        0.5612240, 0.5551425, 0.5493540, 0.5438379, 0.5385608, 0.5334759, 0.5285219, 0.5236294, 0.5187306,\
        0.5137720, 0.5087300, 0.5035931, 0.4982766, 0.4923629, 0.4860765, 0.4796730, 0.4731526, 0.4664804,\
        0.4596440, 0.4526545, 0.4455346, 0.4383055, 0.4309758, 0.4235321, 0.4159472, 0.4082669, 0.4008146,\
        0.3937488, 0.3870198, 0.3803870, 0.3736389],\
        [0.2384839, 0.2472804, 0.2557477, 0.2638700, 0.2716230, 0.2789647, 0.2858258, 0.2921427, 0.2981124,\
        0.3042177, 0.3100459, 0.3157854, 0.3215476, 0.3273644, 0.3332542, 0.3392406, 0.3453555, 0.3516356,\
        0.3581200, 0.3648455, 0.3718424, 0.3791258, 0.3866971, 0.3946497, 0.4034494, 0.4131401, 0.4230790,\
        0.4331594, 0.4433791, 0.4537196, 0.4641550, 0.4746581, 0.4851908, 0.4957008, 0.5061296, 0.5166465,\
        0.5275406, 0.5389984, 0.5510584, 0.5635348, 0.5760557, 0.5881195, 0.5989584, 0.6089640, 0.6192845,\
        0.6295387, 0.6392340, 0.6482923, 0.6567388, 0.6644546, 0.6709326, 0.6756329, 0.6790757, 0.6819011,\
        0.6837732, 0.6834541, 0.6823572, 0.6819028, 0.6818207, 0.6817769, 0.6811880, 0.6802546, 0.6793989,\
        0.6787457, 0.6780626, 0.6771023, 0.6760234, 0.6748198, 0.6732962, 0.6712475, 0.6685942, 0.6654802,\
        0.6618164, 0.6574947, 0.6526699, 0.6475475, 0.6422499, 0.6368240, 0.6312382, 0.6253731, 0.6191086,\
        0.6125160, 0.6058691, 0.5994017, 0.5943967, 0.5889041, 0.5822780, 0.5753725, 0.5686443, 0.5621785,\
        0.5559813, 0.5500477, 0.5443688, 0.5389321, 0.5337175, 0.5286877, 0.5237778, 0.5189030, 0.5139798,\
        0.5089298, 0.5037333, 0.4984160, 0.4929143, 0.4870291, 0.4807895, 0.4743546, 0.4677926, 0.4611172,\
        0.4543361, 0.4474684, 0.4405438, 0.4335946, 0.4266397, 0.4196615, 0.4125787, 0.4052583, 0.3979313,\
        0.3911029, 0.3843743, 0.3776862, 0.3709387],\
        [0.2351749, 0.2439765, 0.2524408, 0.2605506, 0.2682835, 0.2756125, 0.2825214, 0.2890622, 0.2954200,\
        0.3014433, 0.3071089, 0.3128612, 0.3186918, 0.3245795, 0.3305178, 0.3365242, 0.3426312, 0.3488775,\
        0.3553019, 0.3619412, 0.3688295, 0.3760019, 0.3835157, 0.3915031, 0.4001776, 0.4093295, 0.4186477,\
        0.4282140, 0.4380215, 0.4480055, 0.4581062, 0.4682775, 0.4784789, 0.4886722, 0.4988337, 0.5090671,\
        0.5195986, 0.5306796, 0.5423839, 0.5544450, 0.5664652, 0.5779828, 0.5886164, 0.5987048, 0.6092452,\
        0.6192196, 0.6279279, 0.6357847, 0.6431725, 0.6500452, 0.6560094, 0.6607970, 0.6645437, 0.6674139,\
        0.6691832, 0.6695903, 0.6692232, 0.6688462, 0.6686142, 0.6683597, 0.6678697, 0.6671883, 0.6665071,\
        0.6659030, 0.6652894, 0.6645598, 0.6636990, 0.6626553, 0.6612894, 0.6594501, 0.6570600, 0.6541215,\
        0.6506167, 0.6465384, 0.6419763, 0.6370894, 0.6320392, 0.6269453, 0.6218283, 0.6165060, 0.6106528,\
        0.6043155, 0.5980057, 0.5920304, 0.5866394, 0.5812549, 0.5753177, 0.5690376, 0.5627327, 0.5565629,\
        0.5505828, 0.5448045, 0.5392271, 0.5338504, 0.5286764, 0.5236905, 0.5188257, 0.5139627, 0.5090175,\
        0.5038859, 0.4985108, 0.4930217, 0.4873469, 0.4814046, 0.4751942, 0.4687889, 0.4622532, 0.4556239,\
        0.4489271, 0.4421938, 0.4354668, 0.4287981, 0.4222356, 0.4157942, 0.4094044, 0.4028576, 0.3958161,\
        0.3887725, 0.3819199, 0.3751555, 0.3683848],\
        [0.2316849, 0.2404969, 0.2489674, 0.2570816, 0.2648238, 0.2721821, 0.2791622, 0.2858018, 0.2921295,\
        0.2980929, 0.3039372, 0.3098313, 0.3157958, 0.3217817, 0.3277739, 0.3337995, 0.3399011, 0.3461223,\
        0.3525037, 0.3590851, 0.3659088, 0.3730294, 0.3805302, 0.3885303, 0.3970540, 0.4055905, 0.4142033,\
        0.4232665, 0.4326661, 0.4422663, 0.4519834, 0.4617622, 0.4715616, 0.4813552, 0.4911531, 0.5010566,\
        0.5110503, 0.5217354, 0.5331701, 0.5448719, 0.5564855, 0.5677236, 0.5784748, 0.5890044, 0.5995717,\
        0.6090894, 0.6164026, 0.6230473, 0.6295706, 0.6360056, 0.6417891, 0.6467265, 0.6508360, 0.6540533,\
        0.6562171, 0.6572853, 0.6575555, 0.6574527, 0.6572097, 0.6568548, 0.6563537, 0.6557371, 0.6550887,\
        0.6544548, 0.6538122, 0.6531087, 0.6522931, 0.6512845, 0.6499587, 0.6481880, 0.6458947, 0.6430605,\
        0.6396858, 0.6357768, 0.6313893, 0.6266653, 0.6218000, 0.6169841, 0.6123519, 0.6077473, 0.6024538,\
        0.5963823, 0.5903798, 0.5846570, 0.5792343, 0.5738660, 0.5682776, 0.5624664, 0.5565764, 0.5507313,\
        0.5449941, 0.5393842, 0.5339032, 0.5285604, 0.5233884, 0.5184253, 0.5136291, 0.5087811, 0.5037379,\
        0.4986213, 0.4930843, 0.4874175, 0.4816026, 0.4755973, 0.4693978, 0.4630392, 0.4565671, 0.4500196,\
        0.4434291, 0.4368326, 0.4302814, 0.4238457, 0.4176050, 0.4116134, 0.4058289, 0.3998801, 0.3931785,\
        0.3862370, 0.3793865, 0.3726211, 0.3658677],\
        [0.2280332, 0.2368655, 0.2453564, 0.2534950, 0.2612713, 0.2686807, 0.2757278, 0.2824251, 0.2887803,\
        0.2948336, 0.3007604, 0.3068196, 0.3129523, 0.3190277, 0.3250492, 0.3310759, 0.3371655, 0.3433639,\
        0.3497105, 0.3562437, 0.3630057, 0.3700473, 0.3774265, 0.3851805, 0.3932283, 0.4012100, 0.4094720,\
        0.4182721, 0.4273344, 0.4365529, 0.4458646, 0.4552227, 0.4645907, 0.4739473, 0.4832976, 0.4926432,\
        0.5019824, 0.5125873, 0.5238275, 0.5351347, 0.5463628, 0.5573501, 0.5680852, 0.5787087, 0.5891555,\
        0.5976525, 0.6041940, 0.6099267, 0.6159887, 0.6225167, 0.6282027, 0.6333206, 0.6379232, 0.6417149,\
        0.6444850, 0.6461914, 0.6469981, 0.6471914, 0.6470222, 0.6466340, 0.6460927, 0.6454448, 0.6447375,\
        0.6440038, 0.6432509, 0.6424605, 0.6415866, 0.6405402, 0.6391906, 0.6374055, 0.6351076, 0.6323032,\
        0.6290259, 0.6252576, 0.6209855, 0.6163490, 0.6115934, 0.6069429, 0.6026111, 0.5986201, 0.5938666,\
        0.5883102, 0.5826604, 0.5771738, 0.5718591, 0.5665948, 0.5612442, 0.5557719, 0.5502328, 0.5446997,\
        0.5392165, 0.5337905, 0.5284084, 0.5230698, 0.5178302, 0.5128178, 0.5081142, 0.5034363, 0.4981739,\
        0.4928634, 0.4873182, 0.4815706, 0.4756779, 0.4696401, 0.4634615, 0.4571646, 0.4507816, 0.4443442,\
        0.4378807, 0.4314225, 0.4250168, 0.4187411, 0.4126991, 0.4069599, 0.4014923, 0.3958987, 0.3897190,\
        0.3831984, 0.3765863, 0.3699597, 0.3633037],\
        [0.2242357, 0.2331001, 0.2416270, 0.2498091, 0.2576413, 0.2651229, 0.2722589, 0.2790594, 0.2855420,\
        0.2917523, 0.2978076, 0.3040258, 0.3102803, 0.3163551, 0.3223419, 0.3283406, 0.3344100, 0.3405868,\
        0.3469012, 0.3533843, 0.3600693, 0.3669892, 0.3741644, 0.3815720, 0.3891026, 0.3966959, 0.4048626,\
        0.4134012, 0.4221132, 0.4309332, 0.4398275, 0.4487607, 0.4577017, 0.4666339, 0.4755567, 0.4844980,\
        0.4937921, 0.5042231, 0.5146778, 0.5254228, 0.5361727, 0.5467125, 0.5570328, 0.5671248, 0.5766275,\
        0.5847157, 0.5914862, 0.5965826, 0.6033024, 0.6098881, 0.6150483, 0.6204292, 0.6257221, 0.6301982,\
        0.6336465, 0.6359582, 0.6372082, 0.6376460, 0.6375614, 0.6371736, 0.6366003, 0.6358948, 0.6350862,\
        0.6342029, 0.6332808, 0.6323378, 0.6313529, 0.6302428, 0.6288529, 0.6270173, 0.6246362, 0.6217759,\
        0.6185788, 0.6149900, 0.6108131, 0.6062143, 0.6015795, 0.5971081, 0.5929807, 0.5890589, 0.5847106,\
        0.5797958, 0.5746423, 0.5694870, 0.5643898, 0.5593083, 0.5541786, 0.5489774, 0.5437309, 0.5384837,\
        0.5332601, 0.5280437, 0.5227870, 0.5174492, 0.5120629, 0.5068431, 0.5020792, 0.4975488, 0.4923737,\
        0.4868915, 0.4812719, 0.4754963, 0.4695863, 0.4635583, 0.4574268, 0.4512135, 0.4449454, 0.4386483,\
        0.4323420, 0.4260415, 0.4197728, 0.4136025, 0.4076667, 0.4021032, 0.3968172, 0.3915286, 0.3858461,\
        0.3797995, 0.3735368, 0.3671494, 0.3606628],\
        [0.2203038, 0.2292124, 0.2377899, 0.2460324, 0.2539393, 0.2615145, 0.2687685, 0.2757207, 0.2824056,\
        0.2888850, 0.2952631, 0.3016388, 0.3077874, 0.3137016, 0.3195924, 0.3255544, 0.3316104, 0.3377730,\
        0.3440590, 0.3504901, 0.3570891, 0.3638744, 0.3708498, 0.3779945, 0.3852820, 0.3927818, 0.4006306,\
        0.4087762, 0.4170641, 0.4254513, 0.4339269, 0.4424478, 0.4509866, 0.4595466, 0.4681664, 0.4769613,\
        0.4861887, 0.4957769, 0.5055832, 0.5158490, 0.5260107, 0.5358279, 0.5454003, 0.5547133, 0.5634902,\
        0.5713570, 0.5782174, 0.5844411, 0.5912140, 0.5975340, 0.6017805, 0.6081697, 0.6141197, 0.6192004,\
        0.6233500, 0.6262205, 0.6278064, 0.6284014, 0.6283745, 0.6280175, 0.6274539, 0.6267163, 0.6258113,\
        0.6247483, 0.6236186, 0.6224827, 0.6213589, 0.6201990, 0.6187899, 0.6169246, 0.6144016, 0.6113458,\
        0.6081961, 0.6048922, 0.6008306, 0.5963066, 0.5918445, 0.5875758, 0.5835526, 0.5796297, 0.5755069,\
        0.5710535, 0.5663711, 0.5615944, 0.5567852, 0.5519407, 0.5470375, 0.5420709, 0.5370691, 0.5320775,\
        0.5271180, 0.5221500, 0.5170812, 0.5118120, 0.5063129, 0.5007479, 0.4958499, 0.4911163, 0.4860473,\
        0.4806304, 0.4750124, 0.4692483, 0.4633657, 0.4573868, 0.4513329, 0.4452287, 0.4391025, 0.4329807,\
        0.4268774, 0.4207878, 0.4147019, 0.4086499, 0.4027873, 0.3974255, 0.3923842, 0.3872987, 0.3819510,\
        0.3762876, 0.3703645, 0.3642452, 0.3579639],\
        [0.2162443, 0.2252083, 0.2338487, 0.2421651, 0.2501604, 0.2578433, 0.2652300, 0.2723482, 0.2792435,\
        0.2859849, 0.2926509, 0.2991784, 0.3051537, 0.3108503, 0.3166962, 0.3226727, 0.3287462, 0.3349108,\
        0.3411758, 0.3475575, 0.3540728, 0.3607348, 0.3675482, 0.3745120, 0.3816388, 0.3889780, 0.3965681,\
        0.4043578, 0.4121452, 0.4201019, 0.4281900, 0.4363179, 0.4444712, 0.4526860, 0.4610271, 0.4695863,\
        0.4784395, 0.4875360, 0.4969213, 0.5066459, 0.5160341, 0.5247961, 0.5334656, 0.5420092, 0.5502534,\
        0.5579822, 0.5650938, 0.5717978, 0.5783539, 0.5843511, 0.5899049, 0.5965679, 0.6025371, 0.6082479,\
        0.6131749, 0.6165357, 0.6183351, 0.6190067, 0.6190080, 0.6187565, 0.6182865, 0.6175752, 0.6166477,\
        0.6153695, 0.6140236, 0.6126832, 0.6113680, 0.6102196, 0.6087876, 0.6069657, 0.6043604, 0.6009325,\
        0.5977858, 0.5945612, 0.5907504, 0.5864728, 0.5822049, 0.5780968, 0.5741626, 0.5703038, 0.5663620,\
        0.5622422, 0.5579543, 0.5535485, 0.5490557, 0.5444771, 0.5398040, 0.5350428, 0.5302341, 0.5254515,\
        0.5207510, 0.5160697, 0.5112531, 0.5061666, 0.5006959, 0.4951020, 0.4898256, 0.4847656, 0.4795884,\
        0.4741897, 0.4686064, 0.4628873, 0.4570665, 0.4511701, 0.4452223, 0.4392505, 0.4332891, 0.4273745,\
        0.4215286, 0.4157334, 0.4099378, 0.4041248, 0.3984562, 0.3932508, 0.3883120, 0.3833623, 0.3782294,\
        0.3728404, 0.3671962, 0.3613248, 0.3552501],\
        [0.2120609, 0.2210896, 0.2298027, 0.2382020, 0.2462933, 0.2540875, 0.2616027, 0.2688664, 0.2759166,\
        0.2827956, 0.2895197, 0.2959787, 0.3017919, 0.3076115, 0.3136177, 0.3196925, 0.3258183, 0.3320001,\
        0.3382510, 0.3445876, 0.3510265, 0.3575809, 0.3642603, 0.3710740, 0.3780387, 0.3851794, 0.3925059,\
        0.3998693, 0.4071982, 0.4148847, 0.4226461, 0.4303774, 0.4381296, 0.4459863, 0.4540211, 0.4622893,\
        0.4708177, 0.4796063, 0.4886751, 0.4980391, 0.5060772, 0.5136066, 0.5215186, 0.5293409, 0.5371770,\
        0.5448432, 0.5520755, 0.5588637, 0.5653103, 0.5714636, 0.5776380, 0.5841387, 0.5905272, 0.5968410,\
        0.6025245, 0.6062186, 0.6081977, 0.6090011, 0.6090055, 0.6090543, 0.6088091, 0.6081035, 0.6074321,\
        0.6058020, 0.6042962, 0.6028447, 0.6011838, 0.6001307, 0.5985892, 0.5966994, 0.5941592, 0.5907810,\
        0.5874626, 0.5841121, 0.5804466, 0.5764895, 0.5724749, 0.5685506, 0.5647462, 0.5610085, 0.5572504,\
        0.5534076, 0.5494587, 0.5454025, 0.5412321, 0.5369318, 0.5324889, 0.5279070, 0.5232280, 0.5185656,\
        0.5140882, 0.5097347, 0.5051236, 0.5002594, 0.4948311, 0.4892293, 0.4837663, 0.4784463, 0.4731197,\
        0.4676871, 0.4621323, 0.4564755, 0.4507430, 0.4449583, 0.4391430, 0.4333231, 0.4275372, 0.4218418,\
        0.4162912, 0.4108740, 0.4054733, 0.4000375, 0.3946342, 0.3895147, 0.3846101, 0.3797352, 0.3747415,\
        0.3695451, 0.3641159, 0.3584536, 0.3525665],\
        [0.2077542, 0.2168554, 0.2256483, 0.2341362, 0.2423259, 0.2502284, 0.2578592, 0.2652382, 0.2723857,\
        0.2793084, 0.2859698, 0.2922543, 0.2981746, 0.3043004, 0.3104879, 0.3166678, 0.3228496, 0.3290500,\
        0.3352874, 0.3415811, 0.3479490, 0.3544063, 0.3609655, 0.3676371, 0.3744295, 0.3813384, 0.3883146,\
        0.3952442, 0.4024063, 0.4099604, 0.4173414, 0.4246020, 0.4319032, 0.4393748, 0.4470808, 0.4550358,\
        0.4632278, 0.4716284, 0.4801621, 0.4884492, 0.4955250, 0.5026677, 0.5100171, 0.5169235, 0.5245210,\
        0.5322138, 0.5394050, 0.5460402, 0.5523057, 0.5584284, 0.5646703, 0.5711556, 0.5777579, 0.5842659,\
        0.5900960, 0.5942330, 0.5966987, 0.5979552, 0.5981444, 0.5986757, 0.5987601, 0.5979798, 0.5977525,\
        0.5959632, 0.5942718, 0.5927486, 0.5910826, 0.5896852, 0.5880637, 0.5860284, 0.5834673, 0.5803710,\
        0.5770777, 0.5736825, 0.5701157, 0.5663925, 0.5626243, 0.5589087, 0.5552763, 0.5517018, 0.5481384,\
        0.5445492, 0.5409102, 0.5371902, 0.5333435, 0.5293269, 0.5251186, 0.5207091, 0.5161103, 0.5114317,\
        0.5070037, 0.5031584, 0.4985924, 0.4936964, 0.4884388, 0.4829498, 0.4774614, 0.4720333, 0.4666195,\
        0.4611641, 0.4556443, 0.4500657, 0.4444458, 0.4388026, 0.4331502, 0.4275031, 0.4218923, 0.4163913,\
        0.4111149, 0.4061285, 0.4011733, 0.3960525, 0.3909510, 0.3859921, 0.3811660, 0.3763663, 0.3714803,\
        0.3664279, 0.3611651, 0.3556734, 0.3499477],\
        [0.2033235, 0.2125033, 0.2213812, 0.2299613, 0.2382501, 0.2462568, 0.2539931, 0.2614706, 0.2686964,\
        0.2756643, 0.2823486, 0.2887360, 0.2949441, 0.3011646, 0.3074113, 0.3136446, 0.3198602, 0.3260680,\
        0.3322858, 0.3385344, 0.3448333, 0.3511988, 0.3576443, 0.3641796, 0.3708089, 0.3775249, 0.3843082,\
        0.3911807, 0.3983019, 0.4054152, 0.4122194, 0.4188987, 0.4257033, 0.4327882, 0.4401732, 0.4478076,\
        0.4556235, 0.4635347, 0.4713823, 0.4788581, 0.4857830, 0.4927030, 0.4989454, 0.5050317, 0.5127709,\
        0.5205221, 0.5273639, 0.5335472, 0.5394007, 0.5452034, 0.5512325, 0.5575837, 0.5641079, 0.5704862,\
        0.5761964, 0.5806689, 0.5837448, 0.5856096, 0.5866142, 0.5873747, 0.5876476, 0.5873254, 0.5867620,\
        0.5854732, 0.5838751, 0.5822799, 0.5806542, 0.5790390, 0.5772645, 0.5751578, 0.5726362, 0.5697224,\
        0.5665532, 0.5632259, 0.5597725, 0.5562360, 0.5526889, 0.5491919, 0.5457618, 0.5423770, 0.5390101,\
        0.5356556, 0.5323106, 0.5289272, 0.5254092, 0.5216726, 0.5177019, 0.5134985, 0.5090006, 0.5042993,\
        0.4996118, 0.4959088, 0.4916316, 0.4867836, 0.4816473, 0.4763109, 0.4708996, 0.4654829, 0.4600668,\
        0.4546343, 0.4491759, 0.4436981, 0.4382173, 0.4327499, 0.4273025, 0.4218676, 0.4164430, 0.4110918,\
        0.4059784, 0.4013260, 0.3968460, 0.3920927, 0.3873100, 0.3825759, 0.3778904, 0.3731959, 0.3684161,\
        0.3634850, 0.3583570, 0.3530052, 0.3474152],\
        [0.1987668, 0.2080303, 0.2169975, 0.2256725, 0.2340613, 0.2421712, 0.2500105, 0.2575862, 0.2649018,\
        0.2719554, 0.2787467, 0.2853028, 0.2917060, 0.2980486, 0.3043504, 0.3106161, 0.3168503, 0.3230519,\
        0.3292403, 0.3354394, 0.3416688, 0.3479445, 0.3542799, 0.3606847, 0.3671650, 0.3737241, 0.3803726,\
        0.3871459, 0.3940443, 0.4007847, 0.4070255, 0.4131007, 0.4194304, 0.4261981, 0.4333204, 0.4406563,\
        0.4480919, 0.4555223, 0.4628140, 0.4698155, 0.4764856, 0.4828729, 0.4887957, 0.4950630, 0.5027693,\
        0.5101214, 0.5159870, 0.5214608, 0.5266970, 0.5319103, 0.5375322, 0.5436422, 0.5499473, 0.5561034,\
        0.5617320, 0.5664459, 0.5700379, 0.5725523, 0.5742224, 0.5753105, 0.5758399, 0.5758185, 0.5753212,\
        0.5743199, 0.5729706, 0.5714660, 0.5698724, 0.5681885, 0.5663353, 0.5642122, 0.5617611, 0.5589896,\
        0.5559616, 0.5527517, 0.5494264, 0.5460561, 0.5427126, 0.5394417, 0.5362399, 0.5330630, 0.5298768,\
        0.5267170, 0.5236472, 0.5206153, 0.5174422, 0.5139590, 0.5101810, 0.5062395, 0.5019228, 0.4972771,\
        0.4927493, 0.4885601, 0.4842771, 0.4795946, 0.4746122, 0.4694343, 0.4641503, 0.4588217, 0.4534754,\
        0.4481183, 0.4427565, 0.4374053, 0.4320893, 0.4268332, 0.4216456, 0.4165006, 0.4113317, 0.4061314,\
        0.4011796, 0.3966931, 0.3925158, 0.3881880, 0.3837452, 0.3792612, 0.3747514, 0.3701878, 0.3655217,\
        0.3607028, 0.3556906, 0.3504559, 0.3449798],\
        [0.1940812, 0.2034328, 0.2124931, 0.2212660, 0.2297569, 0.2379719, 0.2459176, 0.2536000, 0.2610241,\
        0.2681955, 0.2751272, 0.2818506, 0.2884167, 0.2948720, 0.3012388, 0.3075577, 0.3138082, 0.3199876,\
        0.3261373, 0.3322836, 0.3384429, 0.3446297, 0.3508567, 0.3571330, 0.3634652, 0.3698581, 0.3763173,\
        0.3828379, 0.3893520, 0.3956597, 0.4015100, 0.4070653, 0.4130480, 0.4196713, 0.4266255, 0.4336951,\
        0.4407660, 0.4477644, 0.4546166, 0.4612548, 0.4676576, 0.4738533, 0.4799937, 0.4866037, 0.4940213,\
        0.5000303, 0.5049028, 0.5098236, 0.5143930, 0.5187236, 0.5238687, 0.5297821, 0.5357919, 0.5416689,\
        0.5472373, 0.5521606, 0.5561637, 0.5591941, 0.5613648, 0.5628162, 0.5636257, 0.5638504, 0.5635557,\
        0.5628032, 0.5616970, 0.5603547, 0.5588467, 0.5571871, 0.5553433, 0.5532643, 0.5509130, 0.5482830,\
        0.5454004, 0.5423191, 0.5391182, 0.5358941, 0.5327393, 0.5297050, 0.5267666, 0.5238293, 0.5207935,\
        0.5177345, 0.5148989, 0.5122422, 0.5094471, 0.5061869, 0.5024567, 0.4986864, 0.4946140, 0.4901274,\
        0.4856596, 0.4813055, 0.4769079, 0.4722955, 0.4674523, 0.4624334, 0.4573021, 0.4521094, 0.4468870,\
        0.4416529, 0.4364224, 0.4312205, 0.4260863, 0.4210642, 0.4161824, 0.4114300, 0.4066895, 0.4017640,\
        0.3969985, 0.3927062, 0.3886191, 0.3845237, 0.3803432, 0.3760848, 0.3717524, 0.3673284, 0.3627795,\
        0.3580679, 0.3531589, 0.3480245, 0.3426440],\
        [0.1892636, 0.1987073, 0.2078641, 0.2167381, 0.2253341, 0.2336578, 0.2417155, 0.2495139, 0.2570604,\
        0.2643657, 0.2714470, 0.2783313, 0.2850518, 0.2916344, 0.2980926, 0.3044846, 0.3107149, 0.3168451,\
        0.3229561, 0.3290525, 0.3351431, 0.3412424, 0.3473624, 0.3535103, 0.3596884, 0.3658943, 0.3721160,\
        0.3783171, 0.3844102, 0.3902503, 0.3957099, 0.4008115, 0.4067863, 0.4134483, 0.4202684, 0.4270561,\
        0.4337501, 0.4403405, 0.4468129, 0.4531443, 0.4593321, 0.4654260, 0.4715664, 0.4779257, 0.4842561,\
        0.4896405, 0.4942408, 0.4988168, 0.5027292, 0.5058679, 0.5107901, 0.5166859, 0.5221489, 0.5276099,\
        0.5331505, 0.5382737, 0.5425560, 0.5459235, 0.5484483, 0.5502067, 0.5512655, 0.5517086, 0.5516308,\
        0.5511143, 0.5502369, 0.5490775, 0.5476973, 0.5461248, 0.5443573, 0.5423758, 0.5401605, 0.5376937,\
        0.5349678, 0.5320127, 0.5289166, 0.5258073, 0.5228168, 0.5200265, 0.5173958, 0.5147761, 0.5119004,\
        0.5087500, 0.5060731, 0.5037842, 0.5013227, 0.4982945, 0.4946491, 0.4908924, 0.4869277, 0.4826879,\
        0.4783399, 0.4739743, 0.4695488, 0.4649876, 0.4602645, 0.4554012, 0.4504379, 0.4454136, 0.4403577,\
        0.4352893, 0.4302238, 0.4251882, 0.4202374, 0.4154478, 0.4108692, 0.4065242, 0.4024100, 0.3979310,\
        0.3933792, 0.3891445, 0.3851237, 0.3811596, 0.3771573, 0.3730787, 0.3689049, 0.3646145, 0.3601796,\
        0.3555691, 0.3507531, 0.3457056, 0.3404055],\
        [0.1843100, 0.1938492, 0.2031059, 0.2120840, 0.2207883, 0.2292245, 0.2373991, 0.2453198, 0.2529960,\
        0.2604402, 0.2676700, 0.2747081, 0.2815800, 0.2883111, 0.2949321, 0.3013988, 0.3074676, 0.3135775,\
        0.3196823, 0.3257354, 0.3317583, 0.3377731, 0.3437909, 0.3498142, 0.3558394, 0.3618556, 0.3678392,\
        0.3737446, 0.3794960, 0.3850009, 0.3902154, 0.3953987, 0.4013392, 0.4078470, 0.4144175, 0.4208262,\
        0.4270819, 0.4332562, 0.4393761, 0.4454287, 0.4514025, 0.4573177, 0.4632181, 0.4690827, 0.4746739,\
        0.4796601, 0.4840887, 0.4881684, 0.4915559, 0.4945219, 0.4995467, 0.5050124, 0.5093077, 0.5142618,\
        0.5199337, 0.5252270, 0.5295536, 0.5330450, 0.5357860, 0.5377659, 0.5390010, 0.5396035, 0.5397165,\
        0.5394230, 0.5387613, 0.5377846, 0.5365524, 0.5351044, 0.5334558, 0.5316097, 0.5295675, 0.5273046,\
        0.5247634, 0.5219314, 0.5189154, 0.5158836, 0.5130122, 0.5104603, 0.5081466, 0.5059173, 0.5034093,\
        0.5000670, 0.4973791, 0.4952587, 0.4928700, 0.4900031, 0.4866292, 0.4829830, 0.4791273, 0.4750647,\
        0.4708596, 0.4665682, 0.4621931, 0.4577114, 0.4531125, 0.4484089, 0.4436273, 0.4387976, 0.4339452,\
        0.4290864, 0.4242270, 0.4193767, 0.4145954, 0.4100192, 0.4057277, 0.4016538, 0.3978829, 0.3939699,\
        0.3898217, 0.3857852, 0.3818859, 0.3780440, 0.3741821, 0.3702492, 0.3662120, 0.3620427, 0.3577137,\
        0.3531964, 0.3484637, 0.3434913, 0.3382584],\
        [0.1792162, 0.1888541, 0.1982134, 0.2072983, 0.2161136, 0.2246651, 0.2329597, 0.2410056, 0.2488129,\
        0.2563944, 0.2637659, 0.2709456, 0.2779510, 0.2847922, 0.2914491, 0.2977877, 0.3039179, 0.3102236,\
        0.3163296, 0.3223227, 0.3282764, 0.3342148, 0.3401416, 0.3460524, 0.3519383, 0.3577844, 0.3635665,\
        0.3692484, 0.3747864, 0.3801553, 0.3854160, 0.3908164, 0.3966630, 0.4028921, 0.4090881, 0.4149736,\
        0.4207075, 0.4264604, 0.4322583, 0.4380547, 0.4438002, 0.4494706, 0.4550453, 0.4604556, 0.4655535,\
        0.4701969, 0.4743764, 0.4781389, 0.4815275, 0.4850570, 0.4895404, 0.4939548, 0.4976318, 0.5021201,\
        0.5083445, 0.5133775, 0.5173165, 0.5207303, 0.5236016, 0.5257330, 0.5270348, 0.5276927, 0.5279626,\
        0.5278959, 0.5274326, 0.5266219, 0.5255394, 0.5242338, 0.5227280, 0.5210296, 0.5191834, 0.5171825,\
        0.5148841, 0.5121718, 0.5092256, 0.5062690, 0.5034720, 0.5011313, 0.4990768, 0.4970537, 0.4948607,\
        0.4918985, 0.4891755, 0.4868340, 0.4843963, 0.4816168, 0.4784500, 0.4749792, 0.4712741, 0.4673722,\
        0.4633152, 0.4591411, 0.4548678, 0.4504997, 0.4460425, 0.4415110, 0.4369266, 0.4323139, 0.4276978,\
        0.4230975, 0.4185076, 0.4138918, 0.4092614, 0.4048390, 0.4009089, 0.3971443, 0.3935732, 0.3899882,\
        0.3862669, 0.3825210, 0.3788102, 0.3751143, 0.3713866, 0.3675825, 0.3636657, 0.3596052, 0.3553722,\
        0.3509391, 0.3462802, 0.3413720, 0.3361943],\
        [0.1739774, 0.1837164, 0.1931809, 0.2023747, 0.2113029, 0.2199714, 0.2283870, 0.2365580, 0.2444945,\
        0.2522080, 0.2597121, 0.2670200, 0.2741417, 0.2810757, 0.2877941, 0.2942673, 0.3007518, 0.3069373,\
        0.3128771, 0.3187837, 0.3246803, 0.3305624, 0.3364184, 0.3422368, 0.3480063, 0.3537139, 0.3593430,\
        0.3648750, 0.3702991, 0.3756359, 0.3809723, 0.3864704, 0.3922651, 0.3982872, 0.4040864, 0.4092920,\
        0.4144870, 0.4198790, 0.4254254, 0.4310054, 0.4365223, 0.4419254, 0.4471751, 0.4522090, 0.4569401,\
        0.4612983, 0.4652758, 0.4689278, 0.4724042, 0.4760176, 0.4799640, 0.4839124, 0.4878580, 0.4926742,\
        0.4987341, 0.5023784, 0.5057719, 0.5090470, 0.5120517, 0.5143071, 0.5155211, 0.5160733, 0.5164960,\
        0.5167201, 0.5164002, 0.5157220, 0.5147810, 0.5136192, 0.5122839, 0.5107078, 0.5090403, 0.5073453,\
        0.5054012, 0.5027788, 0.4999477, 0.4971411, 0.4945217, 0.4922721, 0.4902993, 0.4883299, 0.4861508,\
        0.4836206, 0.4810485, 0.4785849, 0.4760663, 0.4733222, 0.4702873, 0.4669783, 0.4634355, 0.4596957,\
        0.4557934, 0.4517599, 0.4476198, 0.4433921, 0.4390960, 0.4347527, 0.4303827, 0.4260054, 0.4216465,\
        0.4173449, 0.4131140, 0.4088623, 0.4044487, 0.4001034, 0.3964302, 0.3930017, 0.3896410, 0.3862810,\
        0.3828541, 0.3793743, 0.3758685, 0.3723330, 0.3687410, 0.3650583, 0.3612514, 0.3572892, 0.3531428,\
        0.3487850, 0.3441905, 0.3393364, 0.3342029],\
        [0.1685883, 0.1784308, 0.1880022, 0.1973065, 0.2063487, 0.2151346, 0.2236709, 0.2319654, 0.2400272,\
        0.2478664, 0.2554938, 0.2629192, 0.2701500, 0.2771895, 0.2840452, 0.2907627, 0.2973010, 0.3032912,\
        0.3091823, 0.3150703, 0.3209575, 0.3268182, 0.3326299, 0.3383790, 0.3440573, 0.3496580, 0.3551752,\
        0.3606068, 0.3659627, 0.3712789, 0.3766272, 0.3820989, 0.3877413, 0.3934579, 0.3987897, 0.4034551,\
        0.4083114, 0.4134908, 0.4189018, 0.4243120, 0.4296010, 0.4347337, 0.4396816, 0.4444018, 0.4488443,\
        0.4529755, 0.4568036, 0.4603926, 0.4638728, 0.4674147, 0.4711099, 0.4749381, 0.4790302, 0.4836650,\
        0.4883141, 0.4918981, 0.4950312, 0.4981408, 0.5012135, 0.5035582, 0.5045385, 0.5048316, 0.5054505,\
        0.5061465, 0.5057524, 0.5051965, 0.5044064, 0.5033451, 0.5022659, 0.5007585, 0.4992389, 0.4977662,\
        0.4961023, 0.4937332, 0.4911035, 0.4885073, 0.4860678, 0.4838687, 0.4818508, 0.4798463, 0.4777022,\
        0.4753639, 0.4729240, 0.4704529, 0.4679023, 0.4651842, 0.4622446, 0.4590742, 0.4556886, 0.4521117,\
        0.4483701, 0.4444912, 0.4405027, 0.4364329, 0.4323130, 0.4281740, 0.4240384, 0.4199146, 0.4158132,\
        0.4118014, 0.4079874, 0.4042840, 0.4003172, 0.3962193, 0.3925788, 0.3892565, 0.3860501, 0.3828655,\
        0.3796484, 0.3763810, 0.3730618, 0.3696827, 0.3662234, 0.3626563, 0.3589515, 0.3550792, 0.3510110,\
        0.3467200, 0.3421811, 0.3373717, 0.3322722],\
        [0.1630437, 0.1729913, 0.1826710, 0.1920869, 0.2012436, 0.2101466, 0.2188024, 0.2272179, 0.2354010,\
        0.2433599, 0.2511029, 0.2586370, 0.2659676, 0.2730982, 0.2800308, 0.2867504, 0.2931660, 0.2992582,\
        0.3052336, 0.3111880, 0.3171250, 0.3229997, 0.3287898, 0.3344890, 0.3400967, 0.3456146, 0.3510470,\
        0.3564025, 0.3616994, 0.3669703, 0.3722595, 0.3776025, 0.3829788, 0.3882464, 0.3931329, 0.3976687,\
        0.4023267, 0.4074131, 0.4128100, 0.4180361, 0.4230652, 0.4279216, 0.4325905, 0.4370426, 0.4412489,\
        0.4451944, 0.4488957, 0.4524102, 0.4558349, 0.4592799, 0.4628263, 0.4665274, 0.4704494, 0.4745678,\
        0.4785342, 0.4819909, 0.4850838, 0.4880573, 0.4908887, 0.4930112, 0.4940191, 0.4944118, 0.4951764,\
        0.4960049, 0.4956133, 0.4951581, 0.4945119, 0.4935888, 0.4926077, 0.4913321, 0.4899622, 0.4885594,\
        0.4869319, 0.4848855, 0.4825646, 0.4801969, 0.4779104, 0.4757576, 0.4737053, 0.4716585, 0.4695243,\
        0.4672636, 0.4648981, 0.4624525, 0.4599109, 0.4572313, 0.4543780, 0.4513346, 0.4481013, 0.4446884,\
        0.4411130, 0.4373982, 0.4335722, 0.4296695, 0.4257328, 0.4218092, 0.4179323, 0.4140978, 0.4102606,\
        0.4064409, 0.4029715, 0.3997636, 0.3963000, 0.3925865, 0.3890888, 0.3858476, 0.3827505, 0.3797034,\
        0.3766469, 0.3735480, 0.3703884, 0.3671513, 0.3638161, 0.3603575, 0.3567475, 0.3529578, 0.3489603,\
        0.3447283, 0.3402369, 0.3354635, 0.3303883],\
        [0.1573381, 0.1673922, 0.1771812, 0.1867091, 0.1959803, 0.2050000, 0.2137738, 0.2223079, 0.2306088,\
        0.2386828, 0.2465358, 0.2541722, 0.2615940, 0.2687993, 0.2757785, 0.2825052, 0.2889438, 0.2951202,\
        0.3011552, 0.3072176, 0.3132374, 0.3191364, 0.3249130, 0.3305724, 0.3361221, 0.3415721, 0.3469344,\
        0.3522232, 0.3574571, 0.3626587, 0.3678485, 0.3730279, 0.3781533, 0.3831223, 0.3878336, 0.3923492,\
        0.3968905, 0.4020631, 0.4073102, 0.4121809, 0.4168948, 0.4214787, 0.4258909, 0.4301094, 0.4341160,\
        0.4379012, 0.4414798, 0.4448981, 0.4482285, 0.4515536, 0.4549453, 0.4584532, 0.4620904, 0.4657811,\
        0.4693381, 0.4726065, 0.4756028, 0.4783923, 0.4808876, 0.4828014, 0.4839894, 0.4847381, 0.4854712,\
        0.4860086, 0.4859666, 0.4856182, 0.4850655, 0.4843090, 0.4833988, 0.4823058, 0.4810754, 0.4797250,\
        0.4781692, 0.4763358, 0.4742813, 0.4721317, 0.4699861, 0.4678884, 0.4658267, 0.4637515, 0.4616099,\
        0.4593737, 0.4570429, 0.4546242, 0.4521114, 0.4494846, 0.4467219, 0.4438065, 0.4407283, 0.4374846,\
        0.4340824, 0.4305400, 0.4268850, 0.4231530, 0.4193952, 0.4156846, 0.4120841, 0.4085957, 0.4051365,\
        0.4014786, 0.3981514, 0.3952471, 0.3921649, 0.3889175, 0.3857067, 0.3826285, 0.3796584, 0.3767384,\
        0.3738175, 0.3708585, 0.3678348, 0.3647235, 0.3615012, 0.3581425, 0.3546201, 0.3509060, 0.3469724,\
        0.3427925, 0.3383412, 0.3335957, 0.3285361],\
        [0.1514660, 0.1616274, 0.1715264, 0.1811667, 0.1905523, 0.1996880, 0.2085787, 0.2172296, 0.2256459,\
        0.2338324, 0.2417928, 0.2495294, 0.2570415, 0.2643243, 0.2713670, 0.2781540, 0.2846789, 0.2909723,\
        0.2971292, 0.3033021, 0.3093464, 0.3152475, 0.3210061, 0.3266278, 0.3321251, 0.3375147, 0.3428143,\
        0.3480404, 0.3532086, 0.3583319, 0.3634166, 0.3684526, 0.3734050, 0.3782241, 0.3828924, 0.3874936,\
        0.3922464, 0.3973447, 0.4021365, 0.4065619, 0.4110188, 0.4153719, 0.4195470, 0.4235619, 0.4273987,\
        0.4310398, 0.4344933, 0.4377935, 0.4409965, 0.4441701, 0.4473771, 0.4506565, 0.4540048, 0.4573589,\
        0.4606104, 0.4636683, 0.4665008, 0.4690887, 0.4713500, 0.4731528, 0.4744552, 0.4753902, 0.4761125,\
        0.4765771, 0.4766788, 0.4764744, 0.4760402, 0.4754107, 0.4746091, 0.4736430, 0.4725252, 0.4712538,\
        0.4697949, 0.4681244, 0.4662716, 0.4643037, 0.4622847, 0.4602489, 0.4581971, 0.4561072, 0.4539525,\
        0.4517170, 0.4493989, 0.4470019, 0.4445255, 0.4419625, 0.4393005, 0.4365236, 0.4336128, 0.4305501,\
        0.4273304, 0.4239695, 0.4204973, 0.4169439, 0.4133519, 0.4098277, 0.4064940, 0.4033579, 0.4003272,\
        0.3971481, 0.3939541, 0.3910870, 0.3882416, 0.3853250, 0.3823988, 0.3795231, 0.3767046, 0.3739155,\
        0.3711198, 0.3682837, 0.3653786, 0.3623785, 0.3592579, 0.3559904, 0.3525483, 0.3489035, 0.3450278,\
        0.3408939, 0.3364761, 0.3317513, 0.3266990],\
        [0.1454217, 0.1556914, 0.1657008, 0.1754536, 0.1849534, 0.1942046, 0.2032113, 0.2119780, 0.2205087,\
        0.2288070, 0.2368754, 0.2447148, 0.2523236, 0.2596978, 0.2668312, 0.2737199, 0.2803728, 0.2868265,\
        0.2931515, 0.2993685, 0.3054144, 0.3113228, 0.3170602, 0.3226451, 0.3280917, 0.3334238, 0.3386659,\
        0.3438345, 0.3489404, 0.3539901, 0.3589844, 0.3639142, 0.3687607, 0.3735085, 0.3781721, 0.3828206,\
        0.3875495, 0.3923164, 0.3968180, 0.4010003, 0.4054619, 0.4095627, 0.4135014, 0.4173538, 0.4210510,\
        0.4245592, 0.4278832, 0.4310459, 0.4340906, 0.4370797, 0.4400754, 0.4431162, 0.4461989, 0.4492762,\
        0.4522738, 0.4551220, 0.4577714, 0.4601776, 0.4622788, 0.4640105, 0.4653581, 0.4663758, 0.4671239,\
        0.4676022, 0.4677882, 0.4677020, 0.4673853, 0.4668704, 0.4661781, 0.4653199, 0.4643025, 0.4631231,\
        0.4617696, 0.4602369, 0.4585423, 0.4567223, 0.4548144, 0.4528423, 0.4508122, 0.4487177, 0.4465499,\
        0.4443053, 0.4419888, 0.4396086, 0.4371715, 0.4346797, 0.4321296, 0.4295088, 0.4267884, 0.4239274,\
        0.4209007, 0.4177273, 0.4144552, 0.4111139, 0.4076903, 0.4043002, 0.4012127, 0.3983834, 0.3956375,\
        0.3928261, 0.3899802, 0.3872456, 0.3845690, 0.3818838, 0.3791902, 0.3765097, 0.3738482, 0.3711917,\
        0.3685153, 0.3657913, 0.3629921, 0.3600914, 0.3570624, 0.3538779, 0.3505095, 0.3469283, 0.3431052,\
        0.3390120, 0.3346221, 0.3299113, 0.3248589],\
        [0.1391998, 0.1495782, 0.1596985, 0.1695638, 0.1791778, 0.1885441, 0.1976664, 0.2065486, 0.2151938,\
        0.2236049, 0.2317838, 0.2397309, 0.2474456, 0.2549261, 0.2621712, 0.2691842, 0.2759787, 0.2825819,\
        0.2890232, 0.2953026, 0.3014352, 0.3073353, 0.3130539, 0.3186105, 0.3240031, 0.3292780, 0.3344700,\
        0.3395907, 0.3446437, 0.3496318, 0.3545558, 0.3594116, 0.3641924, 0.3688972, 0.3735421, 0.3781605,\
        0.3827696, 0.3873087, 0.3916690, 0.3959646, 0.4002080, 0.4039158, 0.4076877, 0.4114582, 0.4150344,\
        0.4184113, 0.4216048, 0.4246150, 0.4274713, 0.4302427, 0.4330064, 0.4358104, 0.4386557, 0.4415007,\
        0.4442818, 0.4469359, 0.4494104, 0.4516598, 0.4536389, 0.4553109, 0.4566655, 0.4577214, 0.4585034,\
        0.4590207, 0.4592760, 0.4592846, 0.4590722, 0.4586626, 0.4580729, 0.4573139, 0.4563916, 0.4553074,\
        0.4540596, 0.4526498, 0.4510882, 0.4493934, 0.4475847, 0.4456763, 0.4436751, 0.4415833, 0.4394042,\
        0.4371468, 0.4348261, 0.4324599, 0.4300633, 0.4276466, 0.4252169, 0.4227729, 0.4202806, 0.4176572,\
        0.4148320, 0.4118359, 0.4087697, 0.4056988, 0.4025783, 0.3992487, 0.3964133, 0.3937912, 0.3912511,\
        0.3886991, 0.3861329, 0.3835974, 0.3810924, 0.3785926, 0.3760872, 0.3735780, 0.3710639, 0.3685342,\
        0.3659703, 0.3633491, 0.3606458, 0.3578344, 0.3548883, 0.3517795, 0.3484790, 0.3449565, 0.3411819,\
        0.3371252, 0.3327583, 0.3280560, 0.3229965],\
        [0.1327948, 0.1432824, 0.1535136, 0.1634917, 0.1732197, 0.1827010, 0.1919389, 0.2009367, 0.2096972,\
        0.2182228, 0.2265153, 0.2345756, 0.2424042, 0.2500017, 0.2573703, 0.2645161, 0.2714513, 0.2781924,\
        0.2847529, 0.2911348, 0.2973121, 0.3031885, 0.3089844, 0.3145158, 0.3198314, 0.3250510, 0.3302098,\
        0.3352985, 0.3403107, 0.3452498, 0.3501210, 0.3549256, 0.3596623, 0.3643321, 0.3689424, 0.3735020,\
        0.3780036, 0.3824117, 0.3866957, 0.3908635, 0.3947878, 0.3984067, 0.4021618, 0.4058936, 0.4092969,\
        0.4125410, 0.4156198, 0.4184703, 0.4211036, 0.4236202, 0.4261345, 0.4287123, 0.4313534, 0.4340077,\
        0.4366068, 0.4390885, 0.4414043, 0.4435161, 0.4453920, 0.4470080, 0.4483523, 0.4494264, 0.4502386,\
        0.4507974, 0.4511127, 0.4512002, 0.4510786, 0.4507647, 0.4502704, 0.4496039, 0.4487722, 0.4477810,\
        0.4466353, 0.4453392, 0.4438976, 0.4423163, 0.4406012, 0.4387579, 0.4367913, 0.4347081, 0.4325205,\
        0.4302487, 0.4279211, 0.4255684, 0.4232146, 0.4208743, 0.4185635, 0.4163086, 0.4140997, 0.4117846,\
        0.4091667, 0.4063083, 0.4034202, 0.4006052, 0.3978120, 0.3948639, 0.3921058, 0.3895901, 0.3871870,\
        0.3848172, 0.3824545, 0.3801033, 0.3777632, 0.3754238, 0.3730755, 0.3707128, 0.3683297, 0.3659155,\
        0.3634544, 0.3609265, 0.3583094, 0.3555784, 0.3527072, 0.3496677, 0.3464300, 0.3429626, 0.3392332,\
        0.3352099, 0.3308625, 0.3261639, 0.3210911],\
        [0.1262014, 0.1367983, 0.1471407, 0.1572314, 0.1670734, 0.1766697, 0.1860233, 0.1951372, 0.2040139,\
        0.2126558, 0.2210647, 0.2292424, 0.2371903, 0.2449107, 0.2524074, 0.2596870, 0.2667586, 0.2736319,\
        0.2803114, 0.2867871, 0.2930231, 0.2990630, 0.3049411, 0.3103279, 0.3155198, 0.3207149, 0.3258792,\
        0.3309547, 0.3359339, 0.3408323, 0.3456637, 0.3504331, 0.3551394, 0.3597799, 0.3643529, 0.3688545,\
        0.3732722, 0.3775837, 0.3817659, 0.3857911, 0.3896139, 0.3932881, 0.3970152, 0.4005319, 0.4037276,\
        0.4068872, 0.4099023, 0.4125897, 0.4149479, 0.4171656, 0.4194193, 0.4217944, 0.4242763, 0.4267846,\
        0.4292340, 0.4315633, 0.4337337, 0.4357198, 0.4375024, 0.4390652, 0.4403933, 0.4414766, 0.4423123,\
        0.4429060, 0.4432707, 0.4434238, 0.4433819, 0.4431559, 0.4427508, 0.4421704, 0.4414220, 0.4405174,\
        0.4394681, 0.4382807, 0.4369545, 0.4354842, 0.4338636, 0.4320898, 0.4301647, 0.4280971, 0.4259049,\
        0.4236186, 0.4212821, 0.4189451, 0.4166433, 0.4143856, 0.4121789, 0.4100865, 0.4082125, 0.4063635,\
        0.4039479, 0.4011829, 0.3984509, 0.3958402, 0.3932433, 0.3906176, 0.3880696, 0.3856712, 0.3833826,\
        0.3811498, 0.3789408, 0.3767426, 0.3745475, 0.3723466, 0.3701309, 0.3678923, 0.3656222, 0.3633092,\
        0.3609389, 0.3584935, 0.3559525, 0.3532928, 0.3504890, 0.3475129, 0.3443338, 0.3409187, 0.3372328,\
        0.3332414, 0.3289112, 0.3242124, 0.3191207],\
        [0.1194140, 0.1301206, 0.1405741, 0.1507775, 0.1607334, 0.1704446, 0.1799140, 0.1891443, 0.1981380,\
        0.2068974, 0.2154248, 0.2237222, 0.2317920, 0.2396374, 0.2472625, 0.2546730, 0.2618757, 0.2688762,\
        0.2756764, 0.2822731, 0.2886699, 0.2948896, 0.3007051, 0.3058789, 0.3109800, 0.3162794, 0.3215021,\
        0.3265649, 0.3315035, 0.3363620, 0.3411638, 0.3459126, 0.3506019, 0.3552221, 0.3597641, 0.3642178,\
        0.3685698, 0.3728027, 0.3768977, 0.3808366, 0.3846152, 0.3882735, 0.3918390, 0.3951806, 0.3983147,\
        0.4014271, 0.4044379, 0.4069373, 0.4089370, 0.4108231, 0.4128233, 0.4150403, 0.4174263, 0.4198354,\
        0.4221583, 0.4243460, 0.4263767, 0.4282410, 0.4299350, 0.4314493, 0.4327634, 0.4338533, 0.4347055,\
        0.4353231, 0.4357231, 0.4359287, 0.4359582, 0.4358162, 0.4354968, 0.4349953, 0.4343192, 0.4334893,\
        0.4325283, 0.4314473, 0.4302389, 0.4288844, 0.4273644, 0.4256680, 0.4237941, 0.4217522, 0.4195632,\
        0.4172651, 0.4149172, 0.4125989, 0.4103703, 0.4082249, 0.4061195, 0.4041151, 0.4024765, 0.4013802,\
        0.3991351, 0.3964826, 0.3938925, 0.3913987, 0.3889559, 0.3865414, 0.3841882, 0.3819298, 0.3797577,\
        0.3776429, 0.3755583, 0.3734861, 0.3714137, 0.3693303, 0.3672260, 0.3650912, 0.3629159, 0.3606885,\
        0.3583954, 0.3560203, 0.3535444, 0.3509465, 0.3482021, 0.3452836, 0.3421596, 0.3387951, 0.3351525,\
        0.3311929, 0.3268790, 0.3221776, 0.3170618],\
        [0.1124276, 0.1232436, 0.1338083, 0.1441241, 0.1541937, 0.1640198, 0.1736050, 0.1829518, 0.1920628,\
        0.2009404, 0.2095870, 0.2180050, 0.2261972, 0.2341668, 0.2419178, 0.2494547, 0.2567820, 0.2639025,\
        0.2708160, 0.2775178, 0.2839943, 0.2901873, 0.2959214, 0.3009357, 0.3062966, 0.3118909, 0.3171326,\
        0.3221253, 0.3269954, 0.3318131, 0.3365994, 0.3413463, 0.3460351, 0.3506476, 0.3551690, 0.3595866,\
        0.3638877, 0.3680584, 0.3720855, 0.3759600, 0.3796839, 0.3832693, 0.3867102, 0.3899745, 0.3930927,\
        0.3961472, 0.3991048, 0.4013315, 0.4029573, 0.4045652, 0.4063440, 0.4084668, 0.4108400, 0.4131799,\
        0.4153751, 0.4174189, 0.4193077, 0.4210470, 0.4226530, 0.4241280, 0.4254394, 0.4265387, 0.4273996,\
        0.4280259, 0.4284425, 0.4286853, 0.4287809, 0.4287261, 0.4284944, 0.4280642, 0.4274427, 0.4266672,\
        0.4257823, 0.4248077, 0.4237262, 0.4224982, 0.4210881, 0.4194783, 0.4176677, 0.4156665, 0.4134965,\
        0.4111985, 0.4088392, 0.4065361, 0.4044095, 0.4024259, 0.4004848, 0.3985981, 0.3971138, 0.3960738,\
        0.3942903, 0.3919219, 0.3895089, 0.3871536, 0.3848564, 0.3826089, 0.3804197, 0.3782999, 0.3762458,\
        0.3742404, 0.3722636, 0.3702978, 0.3683286, 0.3663435, 0.3643315, 0.3622820, 0.3601841, 0.3580263,\
        0.3557956, 0.3534772, 0.3510542, 0.3485072, 0.3458134, 0.3429462, 0.3398739, 0.3365596, 0.3329617,\
        0.3290361, 0.3247396, 0.3200340, 0.3148897],\
        [0.1052366, 0.1161622, 0.1268377, 0.1372658, 0.1474488, 0.1573894, 0.1670901, 0.1765533, 0.1857816,\
        0.1947773, 0.2035430, 0.2120811, 0.2203945, 0.2284861, 0.2363593, 0.2440171, 0.2514620, 0.2586941,\
        0.2657095, 0.2724961, 0.2790235, 0.2852222, 0.2909807, 0.2963865, 0.3022374, 0.3077151, 0.3127455,\
        0.3175763, 0.3223583, 0.3271512, 0.3319508, 0.3367230, 0.3414317, 0.3460505, 0.3505619, 0.3549537,\
        0.3592153, 0.3633360, 0.3673066, 0.3711212, 0.3747798, 0.3782846, 0.3816325, 0.3848208, 0.3878665,\
        0.3907767, 0.3934240, 0.3954704, 0.3970203, 0.3984898, 0.4000915, 0.4021825, 0.4045920, 0.4068250,\
        0.4088564, 0.4107543, 0.4124984, 0.4141026, 0.4156163, 0.4170699, 0.4184032, 0.4195173, 0.4203759,\
        0.4209923, 0.4213999, 0.4216580, 0.4218190, 0.4218665, 0.4217343, 0.4213690, 0.4207741, 0.4200192,\
        0.4191909, 0.4183261, 0.4173885, 0.4163013, 0.4150079, 0.4134912, 0.4117556, 0.4098153, 0.4076894,\
        0.4054248, 0.4030833, 0.4007861, 0.3987890, 0.3969536, 0.3951923, 0.3935315, 0.3921300, 0.3908621,\
        0.3892745, 0.3872907, 0.3851462, 0.3829842, 0.3808518, 0.3787618, 0.3767210, 0.3747330, 0.3727940,\
        0.3708925, 0.3690130, 0.3671395, 0.3652577, 0.3633545, 0.3614179, 0.3594365, 0.3573994, 0.3552950,\
        0.3531112, 0.3508348, 0.3484508, 0.3459422, 0.3432886, 0.3404651, 0.3374407, 0.3341769, 0.3306273,\
        0.3267404, 0.3224646, 0.3177549, 0.3125776],\
        [0.0978360, 0.1088708, 0.1196569, 0.1301968, 0.1404928, 0.1505474, 0.1603631, 0.1699422, 0.1792872,\
        0.1884005, 0.1972843, 0.2059413, 0.2143739, 0.2225847, 0.2305760, 0.2383497, 0.2459064, 0.2532441,\
        0.2603566, 0.2672307, 0.2738427, 0.2801649, 0.2862137, 0.2921654, 0.2980700, 0.3033360, 0.3081098,\
        0.3127928, 0.3175256, 0.3223483, 0.3272115, 0.3320441, 0.3367933, 0.3414292, 0.3459376, 0.3503105,\
        0.3545405, 0.3586193, 0.3625389, 0.3662941, 0.3698830, 0.3733051, 0.3765595, 0.3796453, 0.3825581,\
        0.3852636, 0.3876562, 0.3896180, 0.3912254, 0.3927200, 0.3943423, 0.3963563, 0.3986395, 0.4006683,\
        0.4025348, 0.4043155, 0.4059174, 0.4073782, 0.4087832, 0.4102482, 0.4116501, 0.4127722, 0.4136085,\
        0.4142023, 0.4145693, 0.4148032, 0.4150364, 0.4152202, 0.4152118, 0.4149093, 0.4142998, 0.4135113,\
        0.4127116, 0.4119638, 0.4111973, 0.4102631, 0.4090812, 0.4076563, 0.4060010, 0.4041428, 0.4020937,\
        0.3998988, 0.3976579, 0.3954822, 0.3935423, 0.3917914, 0.3901567, 0.3886398, 0.3872599, 0.3859155,\
        0.3844120, 0.3826762, 0.3807855, 0.3788348, 0.3768802, 0.3749478, 0.3730488, 0.3711869, 0.3693590,\
        0.3675561, 0.3657658, 0.3639741, 0.3621671, 0.3603316, 0.3584554, 0.3565268, 0.3545345, 0.3524677,\
        0.3503149, 0.3480646, 0.3457039, 0.3432188, 0.3405921, 0.3378023, 0.3348207, 0.3316079, 0.3281126,\
        0.3242729, 0.3200243, 0.3153117, 0.3100970],\
        [0.0902206, 0.1013642, 0.1122604, 0.1229116, 0.1333200, 0.1434881, 0.1534180, 0.1631123, 0.1725731,\
        0.1818027, 0.1908035, 0.1995775, 0.2081270, 0.2164539, 0.2245598, 0.2324453, 0.2401098, 0.2475503,\
        0.2547605, 0.2617308, 0.2684502, 0.2749174, 0.2811621, 0.2872273, 0.2930146, 0.2982439, 0.3030286,\
        0.3076885, 0.3124661, 0.3174137, 0.3224043, 0.3273294, 0.3321299, 0.3367843, 0.3412898, 0.3456465,\
        0.3498500, 0.3538921, 0.3577643, 0.3614602, 0.3649766, 0.3683123, 0.3714658, 0.3744329, 0.3771999,\
        0.3797320, 0.3819738, 0.3838961, 0.3855672, 0.3871469, 0.3888193, 0.3907064, 0.3927139, 0.3946035,\
        0.3963683, 0.3980413, 0.3995188, 0.4008698, 0.4021328, 0.4036654, 0.4051926, 0.4062565, 0.4070440,\
        0.4076386, 0.4079459, 0.4080648, 0.4084041, 0.4087719, 0.4089088, 0.4086862, 0.4080107, 0.4071156,\
        0.4063184, 0.4056880, 0.4051274, 0.4043397, 0.4032376, 0.4018985, 0.4003206, 0.3985480, 0.3966040,\
        0.3945266, 0.3924305, 0.3904081, 0.3885492, 0.3868591, 0.3853028, 0.3838581, 0.3824975, 0.3811480,\
        0.3797115, 0.3781377, 0.3764470, 0.3746894, 0.3729076, 0.3711286, 0.3693661, 0.3676249, 0.3659030,\
        0.3641934, 0.3624855, 0.3607668, 0.3590242, 0.3572446, 0.3554156, 0.3535256, 0.3515634, 0.3495186,\
        0.3473806, 0.3451395, 0.3427847, 0.3403054, 0.3376888, 0.3349185, 0.3319706, 0.3288085, 0.3253763,\
        0.3215978, 0.3173875, 0.3126744, 0.3074165],\
        [0.0823854, 0.0936372, 0.1046429, 0.1154047, 0.1259248, 0.1362055, 0.1462488, 0.1560572, 0.1656327,\
        0.1749774, 0.1840935, 0.1929827, 0.2016469, 0.2100873, 0.2183048, 0.2262993, 0.2340692, 0.2416112,\
        0.2489200, 0.2559886, 0.2628113, 0.2693873, 0.2757189, 0.2817846, 0.2874948, 0.2927495, 0.2976273,\
        0.3023070, 0.3072555, 0.3124313, 0.3175901, 0.3226140, 0.3274544, 0.3321126, 0.3366071, 0.3409473,\
        0.3451290, 0.3491400, 0.3529680, 0.3566049, 0.3600469, 0.3632933, 0.3663429, 0.3691916, 0.3718277,\
        0.3742306, 0.3763801, 0.3782826, 0.3799985, 0.3816356, 0.3833085, 0.3850749, 0.3868864, 0.3886431,\
        0.3903052, 0.3918577, 0.3932721, 0.3945729, 0.3958435, 0.3973875, 0.3989022, 0.3998525, 0.4005994,\
        0.4012481, 0.4015725, 0.4014060, 0.4019717, 0.4024878, 0.4027335, 0.4026527, 0.4018866, 0.4008423,\
        0.4000860, 0.3995220, 0.3991174, 0.3984228, 0.3973840, 0.3961182, 0.3946348, 0.3929636, 0.3911454,\
        0.3892292, 0.3872951, 0.3854214, 0.3836664, 0.3820477, 0.3805516, 0.3791539, 0.3778192, 0.3764948,\
        0.3751226, 0.3736683, 0.3721324, 0.3705380, 0.3689124, 0.3672762, 0.3656420, 0.3640147, 0.3623930,\
        0.3607710, 0.3591391, 0.3574858, 0.3557986, 0.3540645, 0.3522713, 0.3504072, 0.3484614, 0.3464237,\
        0.3442847, 0.3420354, 0.3396676, 0.3371735, 0.3345454, 0.3317738, 0.3288438, 0.3257270, 0.3223685,\
        0.3186750, 0.3145234, 0.3098111, 0.3045013],\
        [0.0743253, 0.0856848, 0.0967993, 0.1076709, 0.1183018, 0.1286940, 0.1388498, 0.1487711, 0.1584600,\
        0.1679184, 0.1771481, 0.1861508, 0.1949276, 0.2034795, 0.2118066, 0.2199082, 0.2277824, 0.2354256,\
        0.2428330, 0.2499985, 0.2569157, 0.2635770, 0.2699686, 0.2760573, 0.2817896, 0.2871337, 0.2921397,\
        0.2969967, 0.3021719, 0.3075445, 0.3128402, 0.3179285, 0.3227661, 0.3273960, 0.3318671, 0.3361938,\
        0.3403624, 0.3443499, 0.3481376, 0.3517158, 0.3550821, 0.3582380, 0.3611856, 0.3639243, 0.3664497,\
        0.3687554, 0.3708412, 0.3727270, 0.3744624, 0.3761199, 0.3777687, 0.3794419, 0.3811203, 0.3827581,\
        0.3843215, 0.3857920, 0.3871651, 0.3884704, 0.3897901, 0.3911979, 0.3925029, 0.3934901, 0.3942605,\
        0.3948952, 0.3952191, 0.3952672, 0.3957474, 0.3962445, 0.3965144, 0.3964667, 0.3958036, 0.3948714,\
        0.3941240, 0.3935668, 0.3930834, 0.3924178, 0.3914758, 0.3903010, 0.3889261, 0.3873815, 0.3857077,\
        0.3839548, 0.3821838, 0.3804550, 0.3788124, 0.3772752, 0.3758396, 0.3744869, 0.3731872, 0.3719020,\
        0.3705932, 0.3692356, 0.3678230, 0.3663643, 0.3648742, 0.3633663, 0.3618496, 0.3603278, 0.3587993,\
        0.3572585, 0.3556963, 0.3541016, 0.3524619, 0.3507645, 0.3489969, 0.3471474, 0.3452055, 0.3431614,\
        0.3410063, 0.3387320, 0.3363316, 0.3338002, 0.3311344, 0.3283324, 0.3253920, 0.3223026, 0.3190235,\
        0.3154535, 0.3114087, 0.3066882, 0.3013107],\
        [0.0660356, 0.0775020, 0.0887245, 0.0997050, 0.1104456, 0.1209484, 0.1312154, 0.1412484, 0.1510493,\
        0.1606200, 0.1699618, 0.1790763, 0.1879641, 0.1966259, 0.2050614, 0.2132694, 0.2212477, 0.2289927,\
        0.2364998, 0.2437627, 0.2507737, 0.2575221, 0.2639915, 0.2701580, 0.2759989, 0.2815202, 0.2867940,\
        0.2919975, 0.2973430, 0.3027855, 0.3081771, 0.3132675, 0.3180272, 0.3225883, 0.3270337, 0.3313637,\
        0.3355372, 0.3395121, 0.3432631, 0.3467814, 0.3500695, 0.3531342, 0.3559830, 0.3586213, 0.3610524,\
        0.3632803, 0.3653151, 0.3671802, 0.3689148, 0.3705683, 0.3721848, 0.3737870, 0.3753701, 0.3769129,\
        0.3783950, 0.3798051, 0.3811464, 0.3824430, 0.3837314, 0.3850095, 0.3861819, 0.3871552, 0.3879358,\
        0.3885394, 0.3889318, 0.3892148, 0.3895991, 0.3899890, 0.3902039, 0.3901130, 0.3896318, 0.3889296,\
        0.3882539, 0.3876722, 0.3871085, 0.3864275, 0.3855487, 0.3844674, 0.3832055, 0.3817927, 0.3802663,\
        0.3786706, 0.3770548, 0.3754650, 0.3739363, 0.3724866, 0.3711172, 0.3698155, 0.3685595, 0.3673216,\
        0.3660750, 0.3648006, 0.3634899, 0.3621442, 0.3607701, 0.3593748, 0.3579635, 0.3565375, 0.3550943,\
        0.3536278, 0.3521290, 0.3505865, 0.3489875, 0.3473191, 0.3455683, 0.3437235, 0.3417745, 0.3397120,\
        0.3375275, 0.3352131, 0.3327620, 0.3301708, 0.3274391, 0.3245704, 0.3215744, 0.3184683, 0.3152494,\
        0.3118357, 0.3080302, 0.3033350, 0.2977774],\
        [0.0575115, 0.0690840, 0.0804135, 0.0915019, 0.1023512, 0.1129634, 0.1233403, 0.1334837, 0.1433954,\
        0.1530769, 0.1625295, 0.1717543, 0.1807519, 0.1895225, 0.1980657, 0.2063800, 0.2144633, 0.2223120,\
        0.2299216, 0.2372862, 0.2443983, 0.2512485, 0.2578251, 0.2641176, 0.2701258, 0.2758776, 0.2814475,\
        0.2869550, 0.2924972, 0.2980672, 0.3035598, 0.3085453, 0.3131338, 0.3176107, 0.3220648, 0.3264415,\
        0.3306490, 0.3346225, 0.3383357, 0.3417886, 0.3449930, 0.3479642, 0.3507166, 0.3532620, 0.3556102,\
        0.3577716, 0.3597607, 0.3616002, 0.3633210, 0.3649578, 0.3665407, 0.3680873, 0.3695996, 0.3710693,\
        0.3724863, 0.3738456, 0.3751509, 0.3764144, 0.3776453, 0.3788274, 0.3799101, 0.3808448, 0.3816165,\
        0.3822267, 0.3826883, 0.3830633, 0.3834193, 0.3837226, 0.3838737, 0.3837849, 0.3834331, 0.3829043,\
        0.3823243, 0.3817502, 0.3811531, 0.3804634, 0.3796268, 0.3786263, 0.3774713, 0.3761850, 0.3747989,\
        0.3733503, 0.3718785, 0.3704199, 0.3690028, 0.3676433, 0.3663451, 0.3651010, 0.3638957, 0.3627100,\
        0.3615245, 0.3603241, 0.3591000, 0.3578498, 0.3565745, 0.3552769, 0.3539582, 0.3526177, 0.3512514,\
        0.3498522, 0.3484106, 0.3469144, 0.3453502, 0.3437042, 0.3419628, 0.3401140, 0.3381483, 0.3360573,\
        0.3338332, 0.3314670, 0.3289505, 0.3262803, 0.3234585, 0.3204878, 0.3173778, 0.3141733, 0.3109447,\
        0.3076371, 0.3040637, 0.3000165, 0.2940398],\
        [0.0487484, 0.0604260, 0.0718615, 0.0830568, 0.0940136, 0.1047340, 0.1152196, 0.1254721, 0.1354931,\
        0.1452841, 0.1548461, 0.1641801, 0.1732865, 0.1821654, 0.1908161, 0.1992373, 0.2074270, 0.2153819,\
        0.2230983, 0.2305711, 0.2377947, 0.2447630, 0.2514711, 0.2579189, 0.2641191, 0.2701058, 0.2759401,\
        0.2816985, 0.2874377, 0.2931569, 0.2986972, 0.3034933, 0.3079201, 0.3123831, 0.3169445, 0.3214394,\
        0.3257101, 0.3296820, 0.3333451, 0.3367190, 0.3398299, 0.3427031, 0.3453600, 0.3478181, 0.3500913,\
        0.3521933, 0.3541397, 0.3559508, 0.3576509, 0.3592655, 0.3608167, 0.3623187, 0.3637769, 0.3651898,\
        0.3665537, 0.3678669, 0.3691312, 0.3703503, 0.3715223, 0.3726311, 0.3736473, 0.3745417, 0.3752999,\
        0.3759218, 0.3764223, 0.3768325, 0.3771747, 0.3774296, 0.3775455, 0.3774754, 0.3772124, 0.3768012,\
        0.3763068, 0.3757664, 0.3751736, 0.3744951, 0.3736987, 0.3727700, 0.3717123, 0.3705424, 0.3692853,\
        0.3679712, 0.3666313, 0.3652948, 0.3639846, 0.3627149, 0.3614909, 0.3603089, 0.3591592, 0.3580283,\
        0.3569021, 0.3557687, 0.3546198, 0.3534509, 0.3522599, 0.3510461, 0.3498081, 0.3485427, 0.3472445,\
        0.3459054, 0.3445148, 0.3430596, 0.3415254, 0.3398966, 0.3381585, 0.3362983, 0.3343071, 0.3321793,\
        0.3299088, 0.3274851, 0.3248941, 0.3221305, 0.3192044, 0.3161188, 0.3128499, 0.3094352, 0.3060511,\
        0.3028783, 0.2990212, 0.2948282, 0.2891885],\
        [0.0397418, 0.0515236, 0.0630640, 0.0743649, 0.0854281, 0.0962553, 0.1068483, 0.1172086, 0.1273376,\
        0.1372367, 0.1469070, 0.1563491, 0.1655635, 0.1745502, 0.1833086, 0.1918376, 0.2001353, 0.2081994,\
        0.2160270, 0.2236145, 0.2309587, 0.2380571, 0.2449103, 0.2515246, 0.2579167, 0.2641173, 0.2701691,\
        0.2761147, 0.2819710, 0.2876871, 0.2930721, 0.2978840, 0.3023326, 0.3069238, 0.3117416, 0.3164149,\
        0.3207480, 0.3246885, 0.3282707, 0.3315441, 0.3345486, 0.3373180, 0.3398801, 0.3422558, 0.3444610,\
        0.3465096, 0.3484166, 0.3501991, 0.3518764, 0.3534679, 0.3549906, 0.3564567, 0.3578727, 0.3592405,\
        0.3605601, 0.3618310, 0.3630535, 0.3642268, 0.3653455, 0.3663965, 0.3673607, 0.3682198, 0.3689628,\
        0.3695887, 0.3701062, 0.3705288, 0.3708627, 0.3710955, 0.3712001, 0.3711535, 0.3709540, 0.3706252,\
        0.3702017, 0.3697064, 0.3691409, 0.3684911, 0.3677402, 0.3668791, 0.3659101, 0.3648457, 0.3637052,\
        0.3625125, 0.3612924, 0.3600683, 0.3588588, 0.3576765, 0.3565269, 0.3554091, 0.3543171, 0.3532416,\
        0.3521724, 0.3511000, 0.3500169, 0.3489176, 0.3477982, 0.3466558, 0.3454867, 0.3442861, 0.3430472,\
        0.3417608, 0.3404153, 0.3389966, 0.3374884, 0.3358734, 0.3341341, 0.3322560, 0.3302308, 0.3280576,\
        0.3257377, 0.3232598, 0.3205946, 0.3177240, 0.3146810, 0.3115111, 0.3081275, 0.3044151, 0.3007555,\
        0.2973634, 0.2935996, 0.2889285, 0.2833135],\
        [0.0304875, 0.0423722, 0.0540164, 0.0654218, 0.0765900, 0.0875228, 0.0982218, 0.1086885, 0.1189242,\
        0.1289302, 0.1387075, 0.1482568, 0.1575785, 0.1666726, 0.1755388, 0.1841762, 0.1925836, 0.2007593,\
        0.2087015, 0.2164085, 0.2238791, 0.2311139, 0.2381166, 0.2448961, 0.2514681, 0.2578554, 0.2640834,\
        0.2701681, 0.2760968, 0.2818023, 0.2871585, 0.2920787, 0.2966904, 0.3015508, 0.3066246, 0.3114463,\
        0.3157785, 0.3196124, 0.3230687, 0.3262204, 0.3291070, 0.3317683, 0.3342380, 0.3365382, 0.3386835,\
        0.3406862, 0.3425586, 0.3443150, 0.3459706, 0.3475410, 0.3490396, 0.3504769, 0.3518597, 0.3531916,\
        0.3544740, 0.3557072, 0.3568906, 0.3580217, 0.3590945, 0.3600988, 0.3610217, 0.3618506, 0.3625777,\
        0.3632006, 0.3637227, 0.3641486, 0.3644786, 0.3647044, 0.3648120, 0.3647901, 0.3646390, 0.3643719,\
        0.3640076, 0.3635606, 0.3630347, 0.3624253, 0.3617249, 0.3609298, 0.3600429, 0.3590739, 0.3580381,\
        0.3569543, 0.3558423, 0.3547206, 0.3536048, 0.3525056, 0.3514286, 0.3503746, 0.3493400, 0.3483188,\
        0.3473034, 0.3462862, 0.3452605, 0.3442206, 0.3431616, 0.3420790, 0.3409677, 0.3398217, 0.3386330,\
        0.3373917, 0.3360852, 0.3346984, 0.3332136, 0.3316107, 0.3298685, 0.3279681, 0.3258989, 0.3236673,\
        0.3212939, 0.3187794, 0.3160634, 0.3130699, 0.3098594, 0.3066162, 0.3033811, 0.2993911, 0.2954772,\
        0.2916350, 0.2874711, 0.2825828, 0.2768630],\
        [0.0209811, 0.0329677, 0.0447145, 0.0562230, 0.0674949, 0.0785319, 0.0893356, 0.0999072, 0.1102483,\
        0.1203599, 0.1302431, 0.1398985, 0.1493266, 0.1585277, 0.1675015, 0.1762478, 0.1847657, 0.1930545,\
        0.2011135, 0.2089423, 0.2165416, 0.2239139, 0.2310643, 0.2380015, 0.2447384, 0.2512900, 0.2576688,\
        0.2638764, 0.2698921, 0.2756673, 0.2811440, 0.2863205, 0.2913398, 0.2964566, 0.3016268, 0.3065453,\
        0.3107469, 0.3143542, 0.3176641, 0.3206930, 0.3234543, 0.3260068, 0.3283918, 0.3306278, 0.3327245,\
        0.3346903, 0.3365350, 0.3382701, 0.3399080, 0.3414611, 0.3429406, 0.3443557, 0.3457129, 0.3470165,\
        0.3482685, 0.3494695, 0.3506188, 0.3517133, 0.3527479, 0.3537145, 0.3546041, 0.3554077, 0.3561190,\
        0.3567352, 0.3572562, 0.3576823, 0.3580119, 0.3582393, 0.3583570, 0.3583598, 0.3582486, 0.3580310,\
        0.3577180, 0.3573188, 0.3568380, 0.3562753, 0.3556288, 0.3548983, 0.3540877, 0.3532053, 0.3522635,\
        0.3512771, 0.3502621, 0.3492333, 0.3482034, 0.3471818, 0.3461738, 0.3451811, 0.3442020, 0.3432322,\
        0.3422664, 0.3412984, 0.3403222, 0.3393321, 0.3383229, 0.3372891, 0.3362248, 0.3351230, 0.3339750,\
        0.3327704, 0.3314962, 0.3301366, 0.3286728, 0.3270824, 0.3253396, 0.3234173, 0.3212951, 0.3189801,\
        0.3165313, 0.3140089, 0.3113216, 0.3082246, 0.3047296, 0.3013022, 0.2980172, 0.2942517, 0.2899796,\
        0.2856568, 0.2810383, 0.2758580, 0.2699794],\
        [0.0112186, 0.0233060, 0.0351540, 0.0467643, 0.0581386, 0.0692784, 0.0801852, 0.0908604, 0.1013054,\
        0.1115212, 0.1215090, 0.1312693, 0.1408030, 0.1501102, 0.1591913, 0.1680461, 0.1766747, 0.1850769,\
        0.1932530, 0.2012038, 0.2089311, 0.2164382, 0.2237308, 0.2308164, 0.2377046, 0.2444047, 0.2509226,\
        0.2572555, 0.2633885, 0.2692985, 0.2749732, 0.2804471, 0.2858224, 0.2912032, 0.2965528, 0.3015736,\
        0.3054117, 0.3087145, 0.3119876, 0.3149119, 0.3175325, 0.3199824, 0.3223004, 0.3244898, 0.3265523,\
        0.3284926, 0.3303183, 0.3320387, 0.3336642, 0.3352053, 0.3366715, 0.3380708, 0.3394094, 0.3406916,\
        0.3419198, 0.3430948, 0.3442159, 0.3452805, 0.3462842, 0.3472210, 0.3480840, 0.3488668, 0.3495640,\
        0.3501725, 0.3506905, 0.3511166, 0.3514485, 0.3516823, 0.3518140, 0.3518409, 0.3517643, 0.3515890,\
        0.3513217, 0.3509689, 0.3505348, 0.3500213, 0.3494296, 0.3487617, 0.3480223, 0.3472187, 0.3463613,\
        0.3454620, 0.3445336, 0.3435882, 0.3426364, 0.3416860, 0.3407421, 0.3398067, 0.3388792, 0.3379569,\
        0.3370355, 0.3361102, 0.3351755, 0.3342262, 0.3332566, 0.3322609, 0.3312325, 0.3301640, 0.3290464,\
        0.3278690, 0.3266188, 0.3252801, 0.3238339, 0.3222567, 0.3205197, 0.3185865, 0.3164155, 0.3139832,\
        0.3113817, 0.3088470, 0.3063286, 0.3033069, 0.2994732, 0.2957966, 0.2921800, 0.2882514, 0.2838955,\
        0.2792494, 0.2742608, 0.2687848, 0.2627132],\
        [0.0011962, 0.0133829, 0.0253309, 0.0370416, 0.0485168, 0.0597579, 0.0707664, 0.0815437, 0.0920911,\
        0.1024097, 0.1125006, 0.1223646, 0.1320025, 0.1414149, 0.1506022, 0.1595648, 0.1683032, 0.1768179,\
        0.1851099, 0.1931807, 0.2010329, 0.2086702, 0.2160977, 0.2233218, 0.2303496, 0.2371872, 0.2438378,\
        0.2502997, 0.2565666, 0.2626331, 0.2685076, 0.2742252, 0.2798435, 0.2853974, 0.2908200, 0.2957647,\
        0.2991997, 0.3026399, 0.3060812, 0.3088227, 0.3112679, 0.3136483, 0.3159305, 0.3180957, 0.3201402,\
        0.3220673, 0.3238834, 0.3255969, 0.3272168, 0.3287521, 0.3302112, 0.3316013, 0.3329281, 0.3341957,\
        0.3354068, 0.3365623, 0.3376618, 0.3387032, 0.3396831, 0.3405970, 0.3414396, 0.3422059, 0.3428916,\
        0.3434934, 0.3440089, 0.3444361, 0.3447726, 0.3450158, 0.3451632, 0.3452138, 0.3451686, 0.3450309,\
        0.3448054, 0.3444968, 0.3441089, 0.3436448, 0.3431071, 0.3424991, 0.3418257, 0.3410938, 0.3403121,\
        0.3394904, 0.3386391, 0.3377682, 0.3368864, 0.3360003, 0.3351145, 0.3342312, 0.3333504, 0.3324702,\
        0.3315873, 0.3306978, 0.3297967, 0.3288789, 0.3279388, 0.3269703, 0.3259666, 0.3249200, 0.3238213,\
        0.3226600, 0.3214232, 0.3200960, 0.3186601, 0.3170933, 0.3153671, 0.3134432, 0.3112636, 0.3087334,\
        0.3058180, 0.3031616, 0.3007632, 0.2979639, 0.2940666, 0.2900530, 0.2860408, 0.2818170, 0.2772617,\
        0.2723617, 0.2670816, 0.2613431, 0.2550732],\
        [-0.0090901, 0.0031947, 0.0152412, 0.0270510, 0.0386256, 0.0499665, 0.0610751, 0.0719528, 0.0826010,\
        0.0930208, 0.1032133, 0.1131794, 0.1229201, 0.1324361, 0.1417282, 0.1507970, 0.1596435, 0.1682687,\
        0.1766741, 0.1848616, 0.1928341, 0.2005953, 0.2081497, 0.2155026, 0.2226594, 0.2296245, 0.2364006,\
        0.2429880, 0.2493861, 0.2555974, 0.2616326, 0.2675121, 0.2732531, 0.2788319, 0.2841190, 0.2887982,\
        0.2926774, 0.2963840, 0.2999933, 0.3022118, 0.3046074, 0.3069864, 0.3092645, 0.3114250, 0.3134662,\
        0.3153918, 0.3172082, 0.3189232, 0.3205450, 0.3220814, 0.3235402, 0.3249278, 0.3262495, 0.3275095,\
        0.3287101, 0.3298528, 0.3309374, 0.3319624, 0.3329254, 0.3338228, 0.3346507, 0.3354051, 0.3360824,\
        0.3366797, 0.3371945, 0.3376245, 0.3379677, 0.3382221, 0.3383864, 0.3384601, 0.3384441, 0.3383411,\
        0.3381543, 0.3378875, 0.3375444, 0.3371282, 0.3366423, 0.3360907, 0.3354783, 0.3348113, 0.3340974,\
        0.3333446, 0.3325616, 0.3317566, 0.3309367, 0.3301077, 0.3292735, 0.3284362, 0.3275961, 0.3267519,\
        0.3259010, 0.3250397, 0.3241638, 0.3232682, 0.3223474, 0.3213952, 0.3204046, 0.3193677, 0.3182754,\
        0.3171171, 0.3158805, 0.3145512, 0.3131124, 0.3115437, 0.3098203, 0.3079100, 0.3057661, 0.3033027,\
        0.3001843, 0.2971370, 0.2945505, 0.2915294, 0.2878181, 0.2837202, 0.2794413, 0.2749429, 0.2701479,\
        0.2650080, 0.2594804, 0.2535123, 0.2470535]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=30 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax30vt0
    zgrid_H_Hmax_vmax30vt0=[[0.59929, 0.59972, 0.59992, 0.60007, 0.60041, 0.60099, 0.60186, 0.60309, 0.60476, 0.60701, 0.60995,\
        0.61365, 0.61808, 0.62315, 0.62868, 0.63452, 0.64053, 0.64658, 0.65262, 0.65859, 0.66446, 0.67022,\
        0.67584, 0.68131, 0.68655, 0.69146, 0.69587, 0.69935, 0.70114, 0.70186, 0.70204, 0.70189, 0.70154,\
        0.70105, 0.70051, 0.70000, 0.69958, 0.69929, 0.69917, 0.69928, 0.69965, 0.70026, 0.70105, 0.70195,\
        0.70289, 0.70380, 0.70463, 0.70532, 0.70584, 0.70617, 0.70628, 0.70615, 0.70577, 0.70514, 0.70427,\
        0.70315, 0.70184, 0.70038, 0.69888, 0.69757, 0.69699, 0.69535, 0.69255, 0.69024, 0.68834, 0.68646,\
        0.68440, 0.68208, 0.67949, 0.67669, 0.67374, 0.67063, 0.66732, 0.66381, 0.66014, 0.65636, 0.65252,\
        0.64864, 0.64474, 0.64082, 0.63685, 0.63281, 0.62869, 0.62445, 0.62006, 0.61548, 0.61069, 0.60562,\
        0.60009, 0.59361, 0.58655, 0.57937, 0.57216, 0.56498, 0.55784, 0.55081, 0.54395, 0.53732, 0.53102,\
        0.52516, 0.51983, 0.51516, 0.51123, 0.50806, 0.50563, 0.50380, 0.50244, 0.50143, 0.50073, 0.50014,\
        0.49883, 0.49671, 0.49417, 0.49137, 0.48836, 0.48520, 0.48190, 0.47847, 0.47490, 0.47119, 0.46734],\
        [0.60005, 0.60002, 0.59995, 0.59992, 0.60008, 0.60049, 0.60120, 0.60230, 0.60387, 0.60607, 0.60909,\
        0.61304, 0.61786, 0.62338, 0.62937, 0.63560, 0.64189, 0.64813, 0.65424, 0.66020, 0.66598, 0.67160,\
        0.67705, 0.68231, 0.68732, 0.69201, 0.69618, 0.69940, 0.70119, 0.70198, 0.70218, 0.70208, 0.70181,\
        0.70139, 0.70093, 0.70056, 0.70036, 0.70032, 0.70051, 0.70099, 0.70181, 0.70291, 0.70422, 0.70563,\
        0.70704, 0.70838, 0.70959, 0.71062, 0.71145, 0.71205, 0.71240, 0.71248, 0.71228, 0.71179, 0.71102,\
        0.70998, 0.70868, 0.70720, 0.70563, 0.70420, 0.70335, 0.70125, 0.69810, 0.69616, 0.69454, 0.69277,\
        0.69072, 0.68831, 0.68556, 0.68260, 0.67955, 0.67635, 0.67290, 0.66918, 0.66528, 0.66128, 0.65725,\
        0.65320, 0.64916, 0.64509, 0.64100, 0.63683, 0.63256, 0.62815, 0.62356, 0.61874, 0.61364, 0.60822,\
        0.60236, 0.59565, 0.58859, 0.58144, 0.57420, 0.56691, 0.55962, 0.55238, 0.54525, 0.53831, 0.53165,\
        0.52540, 0.51970, 0.51469, 0.51051, 0.50725, 0.50484, 0.50312, 0.50186, 0.50090, 0.50024, 0.49998,\
        0.49894, 0.49703, 0.49470, 0.49211, 0.48933, 0.48639, 0.48331, 0.48009, 0.47673, 0.47324, 0.46959],\
        [0.60029, 0.59986, 0.59962, 0.59941, 0.59934, 0.59955, 0.60011, 0.60112, 0.60261, 0.60477, 0.60795,\
        0.61230, 0.61771, 0.62386, 0.63041, 0.63710, 0.64370, 0.65010, 0.65623, 0.66209, 0.66770, 0.67307,\
        0.67821, 0.68311, 0.68771, 0.69194, 0.69562, 0.69848, 0.70040, 0.70123, 0.70154, 0.70157, 0.70144,\
        0.70111, 0.70068, 0.70049, 0.70060, 0.70091, 0.70149, 0.70246, 0.70387, 0.70561, 0.70755, 0.70953,\
        0.71146, 0.71325, 0.71485, 0.71624, 0.71737, 0.71825, 0.71885, 0.71915, 0.71915, 0.71883, 0.71820,\
        0.71726, 0.71604, 0.71459, 0.71302, 0.71146, 0.70982, 0.70747, 0.70478, 0.70289, 0.70127, 0.69946,\
        0.69733, 0.69479, 0.69177, 0.68858, 0.68551, 0.68228, 0.67866, 0.67467, 0.67049, 0.66625, 0.66202,\
        0.65781, 0.65362, 0.64943, 0.64521, 0.64091, 0.63650, 0.63191, 0.62710, 0.62199, 0.61651, 0.61057,\
        0.60403, 0.59723, 0.59059, 0.58360, 0.57638, 0.56904, 0.56163, 0.55420, 0.54682, 0.53955, 0.53251,\
        0.52582, 0.51964, 0.51417, 0.50964, 0.50619, 0.50381, 0.50222, 0.50107, 0.50014, 0.49948, 0.49903,\
        0.49817, 0.49665, 0.49467, 0.49239, 0.48989, 0.48721, 0.48439, 0.48142, 0.47831, 0.47504, 0.47162],\
        [0.60094, 0.60009, 0.59941, 0.59882, 0.59839, 0.59825, 0.59857, 0.59948, 0.60094, 0.60310, 0.60658,\
        0.61161, 0.61786, 0.62483, 0.63208, 0.63928, 0.64620, 0.65271, 0.65879, 0.66447, 0.66980, 0.67483,\
        0.67958, 0.68401, 0.68809, 0.69173, 0.69487, 0.69738, 0.69909, 0.69997, 0.70046, 0.70052, 0.70055,\
        0.70021, 0.69972, 0.69966, 0.70018, 0.70098, 0.70210, 0.70375, 0.70596, 0.70851, 0.71119, 0.71381,\
        0.71627, 0.71851, 0.72050, 0.72221, 0.72365, 0.72480, 0.72565, 0.72619, 0.72640, 0.72627, 0.72580,\
        0.72499, 0.72387, 0.72248, 0.72089, 0.71915, 0.71718, 0.71486, 0.71246, 0.71038, 0.70848, 0.70646,\
        0.70414, 0.70142, 0.69804, 0.69457, 0.69170, 0.68852, 0.68461, 0.68025, 0.67575, 0.67127, 0.66684,\
        0.66248, 0.65817, 0.65387, 0.64954, 0.64514, 0.64060, 0.63586, 0.63085, 0.62549, 0.61971, 0.61345,\
        0.60675, 0.60005, 0.59331, 0.58620, 0.57894, 0.57156, 0.56405, 0.55645, 0.54882, 0.54123, 0.53378,\
        0.52660, 0.51987, 0.51381, 0.50873, 0.50495, 0.50257, 0.50117, 0.50016, 0.49925, 0.49856, 0.49797,\
        0.49716, 0.49591, 0.49426, 0.49231, 0.49012, 0.48774, 0.48520, 0.48250, 0.47965, 0.47664, 0.47345],\
        [0.60201, 0.60071, 0.59950, 0.59842, 0.59750, 0.59688, 0.59678, 0.59736, 0.59872, 0.60103, 0.60516,\
        0.61129, 0.61868, 0.62666, 0.63469, 0.64243, 0.64963, 0.65619, 0.66212, 0.66752, 0.67249, 0.67713,\
        0.68144, 0.68538, 0.68888, 0.69190, 0.69444, 0.69649, 0.69797, 0.69886, 0.69929, 0.69930, 0.69921,\
        0.69871, 0.69812, 0.69803, 0.69886, 0.70045, 0.70236, 0.70505, 0.70836, 0.71190, 0.71539, 0.71866,\
        0.72162, 0.72427, 0.72659, 0.72860, 0.73030, 0.73170, 0.73279, 0.73356, 0.73400, 0.73408, 0.73379,\
        0.73313, 0.73212, 0.73080, 0.72920, 0.72736, 0.72527, 0.72297, 0.72061, 0.71835, 0.71613, 0.71378,\
        0.71115, 0.70808, 0.70447, 0.70095, 0.69839, 0.69509, 0.69065, 0.68585, 0.68103, 0.67633, 0.67177,\
        0.66729, 0.66287, 0.65848, 0.65407, 0.64959, 0.64495, 0.64010, 0.63495, 0.62944, 0.62350, 0.61714,\
        0.61048, 0.60369, 0.59657, 0.58932, 0.58205, 0.57463, 0.56704, 0.55928, 0.55142, 0.54352, 0.53568,\
        0.52801, 0.52067, 0.51390, 0.50804, 0.50362, 0.50116, 0.49995, 0.49899, 0.49821, 0.49755, 0.49692,\
        0.49614, 0.49507, 0.49369, 0.49203, 0.49014, 0.48806, 0.48581, 0.48339, 0.48082, 0.47807, 0.47513],\
        [0.60349, 0.60173, 0.60001, 0.59844, 0.59695, 0.59576, 0.59521, 0.59548, 0.59661, 0.59908, 0.60437,\
        0.61202, 0.62072, 0.62974, 0.63857, 0.64682, 0.65425, 0.66076, 0.66643, 0.67143, 0.67597, 0.68019,\
        0.68410, 0.68759, 0.69056, 0.69299, 0.69493, 0.69653, 0.69782, 0.69858, 0.69876, 0.69861, 0.69824,\
        0.69755, 0.69672, 0.69632, 0.69705, 0.69922, 0.70255, 0.70687, 0.71158, 0.71620, 0.72047, 0.72429,\
        0.72766, 0.73060, 0.73317, 0.73540, 0.73732, 0.73894, 0.74025, 0.74125, 0.74192, 0.74221, 0.74213,\
        0.74164, 0.74075, 0.73950, 0.73790, 0.73602, 0.73387, 0.73154, 0.72911, 0.72664, 0.72413, 0.72147,\
        0.71853, 0.71524, 0.71168, 0.70828, 0.70520, 0.70140, 0.69654, 0.69138, 0.68634, 0.68152, 0.67686,\
        0.67229, 0.66779, 0.66333, 0.65886, 0.65433, 0.64965, 0.64474, 0.63952, 0.63393, 0.62793, 0.62151,\
        0.61474, 0.60765, 0.60028, 0.59311, 0.58588, 0.57842, 0.57072, 0.56282, 0.55475, 0.54659, 0.53841,\
        0.53030, 0.52241, 0.51494, 0.50818, 0.50267, 0.49961, 0.49863, 0.49791, 0.49728, 0.49668, 0.49605,\
        0.49528, 0.49431, 0.49310, 0.49167, 0.49005, 0.48825, 0.48628, 0.48415, 0.48185, 0.47937, 0.47669],\
        [0.60544, 0.60327, 0.60116, 0.59908, 0.59691, 0.59515, 0.59436, 0.59479, 0.59635, 0.59946, 0.60592,\
        0.61475, 0.62451, 0.63443, 0.64396, 0.65266, 0.66024, 0.66660, 0.67186, 0.67634, 0.68040, 0.68423,\
        0.68785, 0.69104, 0.69362, 0.69556, 0.69696, 0.69817, 0.69938, 0.70002, 0.69992, 0.69954, 0.69899,\
        0.69810, 0.69694, 0.69605, 0.69658, 0.69895, 0.70415, 0.71036, 0.71640, 0.72191, 0.72674, 0.73091,\
        0.73448, 0.73757, 0.74025, 0.74261, 0.74469, 0.74648, 0.74799, 0.74921, 0.75010, 0.75063, 0.75075,\
        0.75044, 0.74968, 0.74850, 0.74692, 0.74501, 0.74281, 0.74040, 0.73785, 0.73520, 0.73243, 0.72950,\
        0.72634, 0.72291, 0.71930, 0.71564, 0.71186, 0.70749, 0.70238, 0.69696, 0.69183, 0.68699, 0.68226,\
        0.67759, 0.67300, 0.66848, 0.66398, 0.65943, 0.65475, 0.64984, 0.64461, 0.63901, 0.63299, 0.62655,\
        0.61972, 0.61254, 0.60520, 0.59794, 0.59057, 0.58299, 0.57518, 0.56713, 0.55891, 0.55055, 0.54212,\
        0.53370, 0.52541, 0.51743, 0.51006, 0.50383, 0.50006, 0.49850, 0.49767, 0.49693, 0.49624, 0.49552,\
        0.49469, 0.49373, 0.49262, 0.49136, 0.48995, 0.48839, 0.48669, 0.48483, 0.48280, 0.48058, 0.47816],\
        [0.60790, 0.60528, 0.60273, 0.60021, 0.59736, 0.59511, 0.59441, 0.59563, 0.59864, 0.60316, 0.61039,\
        0.61980, 0.63024, 0.64084, 0.65095, 0.66003, 0.66772, 0.67382, 0.67851, 0.68234, 0.68588, 0.68941,\
        0.69292, 0.69605, 0.69846, 0.70016, 0.70134, 0.70232, 0.70338, 0.70388, 0.70372, 0.70320, 0.70244,\
        0.70143, 0.70020, 0.69903, 0.70017, 0.70386, 0.70986, 0.71688, 0.72357, 0.72946, 0.73446, 0.73863,\
        0.74213, 0.74514, 0.74780, 0.75019, 0.75234, 0.75426, 0.75594, 0.75736, 0.75848, 0.75925, 0.75960,\
        0.75947, 0.75884, 0.75772, 0.75617, 0.75423, 0.75197, 0.74946, 0.74677, 0.74394, 0.74097, 0.73783,\
        0.73449, 0.73091, 0.72712, 0.72312, 0.71881, 0.71401, 0.70867, 0.70309, 0.69789, 0.69297, 0.68809,\
        0.68327, 0.67856, 0.67398, 0.66947, 0.66495, 0.66030, 0.65543, 0.65025, 0.64469, 0.63870, 0.63227,\
        0.62544, 0.61830, 0.61098, 0.60358, 0.59604, 0.58833, 0.58040, 0.57223, 0.56389, 0.55542, 0.54687,\
        0.53829, 0.52979, 0.52158, 0.51399, 0.50758, 0.50299, 0.50043, 0.49870, 0.49747, 0.49647, 0.49548,\
        0.49445, 0.49340, 0.49231, 0.49114, 0.48990, 0.48855, 0.48708, 0.48548, 0.48371, 0.48175, 0.47958],\
        [0.61097, 0.60783, 0.60462, 0.60137, 0.59803, 0.59576, 0.59537, 0.59761, 0.60219, 0.60845, 0.61668,\
        0.62667, 0.63764, 0.64877, 0.65937, 0.66882, 0.67661, 0.68243, 0.68642, 0.68947, 0.69255, 0.69588,\
        0.69946, 0.70273, 0.70523, 0.70704, 0.70832, 0.70929, 0.71005, 0.71037, 0.71017, 0.70960, 0.70880,\
        0.70790, 0.70707, 0.70693, 0.70899, 0.71352, 0.71977, 0.72671, 0.73329, 0.73898, 0.74363, 0.74740,\
        0.75052, 0.75324, 0.75571, 0.75803, 0.76021, 0.76221, 0.76402, 0.76561, 0.76696, 0.76800, 0.76859,\
        0.76864, 0.76813, 0.76708, 0.76553, 0.76356, 0.76123, 0.75861, 0.75578, 0.75279, 0.74966, 0.74638,\
        0.74289, 0.73916, 0.73517, 0.73088, 0.72623, 0.72115, 0.71571, 0.71013, 0.70475, 0.69956, 0.69444,\
        0.68938, 0.68451, 0.67988, 0.67539, 0.67091, 0.66632, 0.66153, 0.65643, 0.65094, 0.64500, 0.63862,\
        0.63181, 0.62468, 0.61732, 0.60980, 0.60214, 0.59434, 0.58631, 0.57804, 0.56965, 0.56118, 0.55264,\
        0.54404, 0.53548, 0.52718, 0.51945, 0.51275, 0.50748, 0.50368, 0.50091, 0.49898, 0.49745, 0.49595,\
        0.49458, 0.49335, 0.49220, 0.49108, 0.48994, 0.48876, 0.48751, 0.48614, 0.48462, 0.48291, 0.48099],\
        [0.61487, 0.61124, 0.60739, 0.60341, 0.59977, 0.59809, 0.59778, 0.60080, 0.60693, 0.61475, 0.62410,\
        0.63476, 0.64622, 0.65780, 0.66882, 0.67860, 0.68658, 0.69225, 0.69553, 0.69778, 0.70072, 0.70407,\
        0.70770, 0.71108, 0.71381, 0.71587, 0.71735, 0.71837, 0.71898, 0.71916, 0.71889, 0.71833, 0.71766,\
        0.71714, 0.71712, 0.71824, 0.72124, 0.72622, 0.73252, 0.73916, 0.74522, 0.75020, 0.75404, 0.75701,\
        0.75944, 0.76165, 0.76382, 0.76602, 0.76819, 0.77025, 0.77213, 0.77384, 0.77541, 0.77675, 0.77761,\
        0.77784, 0.77746, 0.77646, 0.77491, 0.77291, 0.77050, 0.76776, 0.76479, 0.76168, 0.75843, 0.75504,\
        0.75146, 0.74762, 0.74348, 0.73898, 0.73411, 0.72886, 0.72334, 0.71771, 0.71215, 0.70669, 0.70130,\
        0.69595, 0.69089, 0.68625, 0.68180, 0.67735, 0.67282, 0.66812, 0.66312, 0.65772, 0.65185, 0.64549,\
        0.63868, 0.63151, 0.62409, 0.61648, 0.60874, 0.60087, 0.59275, 0.58440, 0.57605, 0.56772, 0.55931,\
        0.55080, 0.54226, 0.53388, 0.52596, 0.51885, 0.51282, 0.50797, 0.50419, 0.50134, 0.49901, 0.49677,\
        0.49501, 0.49358, 0.49232, 0.49118, 0.49011, 0.48906, 0.48799, 0.48684, 0.48556, 0.48410, 0.48241],\
        [0.61983, 0.61594, 0.61184, 0.60782, 0.60457, 0.60321, 0.60342, 0.60691, 0.61371, 0.62239, 0.63249,\
        0.64366, 0.65546, 0.66737, 0.67869, 0.68866, 0.69670, 0.70235, 0.70540, 0.70768, 0.71053, 0.71392,\
        0.71752, 0.72097, 0.72393, 0.72627, 0.72796, 0.72907, 0.72964, 0.72973, 0.72940, 0.72886, 0.72836,\
        0.72827, 0.72903, 0.73114, 0.73497, 0.74044, 0.74689, 0.75328, 0.75866, 0.76259, 0.76523, 0.76705,\
        0.76855, 0.77010, 0.77191, 0.77398, 0.77618, 0.77831, 0.78021, 0.78193, 0.78368, 0.78542, 0.78656,\
        0.78692, 0.78670, 0.78573, 0.78417, 0.78219, 0.77970, 0.77681, 0.77372, 0.77051, 0.76720, 0.76377,\
        0.76015, 0.75627, 0.75203, 0.74740, 0.74237, 0.73701, 0.73140, 0.72569, 0.71998, 0.71429, 0.70864,\
        0.70306, 0.69782, 0.69321, 0.68875, 0.68427, 0.67976, 0.67515, 0.67028, 0.66498, 0.65915, 0.65277,\
        0.64591, 0.63867, 0.63114, 0.62343, 0.61559, 0.60762, 0.59941, 0.59108, 0.58296, 0.57491, 0.56675,\
        0.55839, 0.54988, 0.54138, 0.53316, 0.52552, 0.51870, 0.51285, 0.50796, 0.50393, 0.50047, 0.49770,\
        0.49573, 0.49408, 0.49266, 0.49144, 0.49039, 0.48945, 0.48855, 0.48761, 0.48656, 0.48533, 0.48387],\
        [0.62596, 0.62207, 0.61811, 0.61448, 0.61182, 0.61080, 0.61187, 0.61573, 0.62246, 0.63131, 0.64166,\
        0.65299, 0.66488, 0.67685, 0.68830, 0.69818, 0.70588, 0.71137, 0.71505, 0.71798, 0.72109, 0.72460,\
        0.72833, 0.73196, 0.73518, 0.73779, 0.73971, 0.74094, 0.74152, 0.74154, 0.74113, 0.74054, 0.74012,\
        0.74030, 0.74157, 0.74435, 0.74885, 0.75487, 0.76168, 0.76805, 0.77276, 0.77541, 0.77655, 0.77701,\
        0.77743, 0.77829, 0.77977, 0.78178, 0.78409, 0.78636, 0.78824, 0.78977, 0.79151, 0.79384, 0.79532,\
        0.79560, 0.79575, 0.79471, 0.79315, 0.79133, 0.78873, 0.78565, 0.78250, 0.77921, 0.77587, 0.77248,\
        0.76891, 0.76504, 0.76077, 0.75605, 0.75091, 0.74544, 0.73976, 0.73398, 0.72817, 0.72236, 0.71659,\
        0.71094, 0.70566, 0.70087, 0.69623, 0.69161, 0.68708, 0.68257, 0.67783, 0.67262, 0.66679, 0.66035,\
        0.65337, 0.64598, 0.63833, 0.63049, 0.62254, 0.61446, 0.60623, 0.59806, 0.59034, 0.58267, 0.57479,\
        0.56660, 0.55809, 0.54940, 0.54077, 0.53251, 0.52488, 0.51804, 0.51205, 0.50686, 0.50243, 0.49922,\
        0.49692, 0.49486, 0.49318, 0.49184, 0.49078, 0.48993, 0.48918, 0.48845, 0.48763, 0.48663, 0.48539],\
        [0.63320, 0.62950, 0.62585, 0.62268, 0.62054, 0.62000, 0.62157, 0.62565, 0.63226, 0.64099, 0.65125,\
        0.66245, 0.67405, 0.68566, 0.69689, 0.70649, 0.71394, 0.71966, 0.72406, 0.72780, 0.73149, 0.73540,\
        0.73947, 0.74345, 0.74704, 0.74999, 0.75214, 0.75349, 0.75410, 0.75405, 0.75350, 0.75272, 0.75215,\
        0.75233, 0.75376, 0.75684, 0.76173, 0.76823, 0.77560, 0.78233, 0.78649, 0.78766, 0.78720, 0.78632,\
        0.78570, 0.78595, 0.78724, 0.78930, 0.79181, 0.79438, 0.79628, 0.79753, 0.79887, 0.80156, 0.80346,\
        0.80389, 0.80420, 0.80327, 0.80185, 0.80024, 0.79742, 0.79417, 0.79113, 0.78772, 0.78437, 0.78114,\
        0.77767, 0.77387, 0.76963, 0.76485, 0.75959, 0.75402, 0.74830, 0.74250, 0.73666, 0.73082, 0.72503,\
        0.71938, 0.71402, 0.70897, 0.70409, 0.69929, 0.69468, 0.69030, 0.68570, 0.68054, 0.67465, 0.66804,\
        0.66086, 0.65329, 0.64549, 0.63755, 0.62953, 0.62148, 0.61345, 0.60563, 0.59818, 0.59082, 0.58325,\
        0.57521, 0.56664, 0.55766, 0.54854, 0.53959, 0.53113, 0.52336, 0.51640, 0.51031, 0.50524, 0.50144,\
        0.49833, 0.49574, 0.49379, 0.49232, 0.49125, 0.49047, 0.48989, 0.48936, 0.48877, 0.48801, 0.48699],\
        [0.64133, 0.63792, 0.63461, 0.63182, 0.63008, 0.62989, 0.63170, 0.63581, 0.64226, 0.65077, 0.66085,\
        0.67179, 0.68285, 0.69360, 0.70409, 0.71344, 0.72107, 0.72723, 0.73233, 0.73686, 0.74126, 0.74576,\
        0.75038, 0.75490, 0.75901, 0.76237, 0.76477, 0.76622, 0.76685, 0.76674, 0.76595, 0.76479, 0.76381,\
        0.76363, 0.76483, 0.76780, 0.77269, 0.77930, 0.78706, 0.79450, 0.79832, 0.79801, 0.79638, 0.79461,\
        0.79312, 0.79300, 0.79437, 0.79658, 0.79923, 0.80203, 0.80410, 0.80548, 0.80699, 0.80917, 0.81106,\
        0.81199, 0.81222, 0.81161, 0.81036, 0.80853, 0.80575, 0.80264, 0.79974, 0.79588, 0.79265, 0.78978,\
        0.78631, 0.78265, 0.77855, 0.77367, 0.76824, 0.76259, 0.75687, 0.75112, 0.74534, 0.73953, 0.73376,\
        0.72808, 0.72260, 0.71733, 0.71224, 0.70730, 0.70262, 0.69826, 0.69372, 0.68855, 0.68250, 0.67563,\
        0.66818, 0.66040, 0.65244, 0.64443, 0.63643, 0.62852, 0.62077, 0.61330, 0.60612, 0.59907, 0.59179,\
        0.58389, 0.57524, 0.56589, 0.55618, 0.54652, 0.53726, 0.52864, 0.52082, 0.51392, 0.50804, 0.50321,\
        0.49933, 0.49665, 0.49445, 0.49283, 0.49174, 0.49107, 0.49065, 0.49035, 0.49001, 0.48948, 0.48866],\
        [0.65012, 0.64699, 0.64395, 0.64142, 0.63990, 0.63985, 0.64169, 0.64566, 0.65184, 0.66008, 0.66996,\
        0.68077, 0.69145, 0.70132, 0.71096, 0.71989, 0.72759, 0.73415, 0.73985, 0.74507, 0.75015, 0.75531,\
        0.76058, 0.76580, 0.77058, 0.77444, 0.77706, 0.77854, 0.77919, 0.77909, 0.77800, 0.77624, 0.77456,\
        0.77370, 0.77430, 0.77679, 0.78125, 0.78735, 0.79453, 0.80164, 0.80534, 0.80531, 0.80376, 0.80185,\
        0.79996, 0.79989, 0.80152, 0.80391, 0.80660, 0.80935, 0.81165, 0.81349, 0.81528, 0.81721, 0.81891,\
        0.81996, 0.82028, 0.81983, 0.81866, 0.81678, 0.81420, 0.81120, 0.80787, 0.80399, 0.80108, 0.79839,\
        0.79451, 0.79126, 0.78747, 0.78229, 0.77663, 0.77098, 0.76537, 0.75976, 0.75410, 0.74836, 0.74259,\
        0.73686, 0.73124, 0.72579, 0.72052, 0.71547, 0.71071, 0.70617, 0.70155, 0.69629, 0.69002, 0.68281,\
        0.67505, 0.66707, 0.65900, 0.65096, 0.64302, 0.63527, 0.62776, 0.62057, 0.61367, 0.60691, 0.59993,\
        0.59224, 0.58351, 0.57372, 0.56336, 0.55300, 0.54303, 0.53368, 0.52513, 0.51747, 0.51081, 0.50526,\
        0.50104, 0.49770, 0.49507, 0.49331, 0.49225, 0.49169, 0.49147, 0.49141, 0.49132, 0.49104, 0.49042],\
        [0.65928, 0.65638, 0.65350, 0.65107, 0.64958, 0.64948, 0.65114, 0.65480, 0.66057, 0.66838, 0.67799,\
        0.68879, 0.69950, 0.70878, 0.71764, 0.72606, 0.73370, 0.74050, 0.74664, 0.75238, 0.75802, 0.76375,\
        0.76965, 0.77560, 0.78121, 0.78570, 0.78848, 0.78980, 0.79045, 0.79060, 0.78915, 0.78658, 0.78405,\
        0.78225, 0.78198, 0.78375, 0.78759, 0.79289, 0.79888, 0.80492, 0.80892, 0.81013, 0.80960, 0.80837,\
        0.80731, 0.80751, 0.80909, 0.81146, 0.81415, 0.81685, 0.81933, 0.82151, 0.82353, 0.82543, 0.82702,\
        0.82808, 0.82846, 0.82810, 0.82701, 0.82519, 0.82272, 0.81974, 0.81636, 0.81281, 0.80963, 0.80629,\
        0.80240, 0.79977, 0.79612, 0.79019, 0.78450, 0.77907, 0.77373, 0.76836, 0.76286, 0.75721, 0.75142,\
        0.74559, 0.73981, 0.73415, 0.72867, 0.72342, 0.71840, 0.71355, 0.70861, 0.70320, 0.69677, 0.68917,\
        0.68121, 0.67312, 0.66500, 0.65696, 0.64910, 0.64147, 0.63413, 0.62711, 0.62037, 0.61378, 0.60705,\
        0.59970, 0.59098, 0.58065, 0.56966, 0.55874, 0.54825, 0.53836, 0.52921, 0.52092, 0.51357, 0.50728,\
        0.50213, 0.49821, 0.49556, 0.49379, 0.49276, 0.49234, 0.49232, 0.49251, 0.49270, 0.49267, 0.49226],\
        [0.66854, 0.66579, 0.66292, 0.66042, 0.65879, 0.65848, 0.65982, 0.66303, 0.66821, 0.67538, 0.68442,\
        0.69482, 0.70532, 0.71474, 0.72353, 0.73180, 0.73942, 0.74636, 0.75275, 0.75881, 0.76480, 0.77090,\
        0.77723, 0.78377, 0.79024, 0.79554, 0.79837, 0.79947, 0.79996, 0.80053, 0.79861, 0.79542, 0.79215,\
        0.78929, 0.78798, 0.78892, 0.79230, 0.79728, 0.80248, 0.80774, 0.81178, 0.81396, 0.81466, 0.81461,\
        0.81462, 0.81535, 0.81699, 0.81930, 0.82194, 0.82465, 0.82724, 0.82963, 0.83182, 0.83377, 0.83535,\
        0.83641, 0.83683, 0.83654, 0.83552, 0.83379, 0.83140, 0.82846, 0.82514, 0.82162, 0.81810, 0.81446,\
        0.81081, 0.80744, 0.80284, 0.79705, 0.79196, 0.78697, 0.78198, 0.77690, 0.77159, 0.76599, 0.76014,\
        0.75415, 0.74816, 0.74225, 0.73648, 0.73091, 0.72551, 0.72018, 0.71472, 0.70882, 0.70203, 0.69438,\
        0.68653, 0.67845, 0.67033, 0.66233, 0.65453, 0.64699, 0.63973, 0.63274, 0.62598, 0.61934, 0.61261,\
        0.60546, 0.59685, 0.58594, 0.57464, 0.56353, 0.55282, 0.54262, 0.53307, 0.52427, 0.51634, 0.50939,\
        0.50354, 0.49917, 0.49626, 0.49436, 0.49332, 0.49300, 0.49319, 0.49365, 0.49413, 0.49437, 0.49415],\
        [0.67761, 0.67490, 0.67189, 0.66915, 0.66726, 0.66666, 0.66764, 0.67035, 0.67485, 0.68118, 0.68940,\
        0.69922, 0.70961, 0.71953, 0.72872, 0.73715, 0.74484, 0.75183, 0.75829, 0.76445, 0.77056, 0.77678,\
        0.78321, 0.78988, 0.79668, 0.80260, 0.80581, 0.80730, 0.80796, 0.80788, 0.80597, 0.80288, 0.79890,\
        0.79524, 0.79283, 0.79279, 0.79574, 0.80085, 0.80594, 0.81082, 0.81493, 0.81781, 0.81959, 0.82072,\
        0.82174, 0.82311, 0.82500, 0.82736, 0.82998, 0.83270, 0.83538, 0.83792, 0.84024, 0.84227, 0.84389,\
        0.84497, 0.84543, 0.84520, 0.84425, 0.84258, 0.84024, 0.83734, 0.83400, 0.83039, 0.82664, 0.82277,\
        0.81882, 0.81463, 0.80970, 0.80446, 0.79961, 0.79489, 0.79018, 0.78538, 0.78026, 0.77463, 0.76863,\
        0.76243, 0.75617, 0.74996, 0.74385, 0.73787, 0.73199, 0.72611, 0.72005, 0.71358, 0.70647, 0.69899,\
        0.69120, 0.68309, 0.67496, 0.66701, 0.65929, 0.65179, 0.64452, 0.63745, 0.63050, 0.62352, 0.61632,\
        0.60852, 0.59942, 0.58903, 0.57823, 0.56742, 0.55681, 0.54654, 0.53676, 0.52761, 0.51924, 0.51182,\
        0.50558, 0.50077, 0.49729, 0.49504, 0.49390, 0.49365, 0.49403, 0.49476, 0.49555, 0.49607, 0.49604],\
        [0.68617, 0.68338, 0.68005, 0.67697, 0.67476, 0.67388, 0.67458, 0.67694, 0.68086, 0.68633, 0.69368,\
        0.70320, 0.71380, 0.72420, 0.73380, 0.74245, 0.75017, 0.75708, 0.76341, 0.76946, 0.77548, 0.78162,\
        0.78791, 0.79430, 0.80085, 0.80689, 0.81106, 0.81341, 0.81442, 0.81417, 0.81241, 0.80930, 0.80526,\
        0.80116, 0.79766, 0.79661, 0.79918, 0.80405, 0.80947, 0.81445, 0.81875, 0.82218, 0.82482, 0.82692,\
        0.82883, 0.83083, 0.83308, 0.83558, 0.83825, 0.84100, 0.84375, 0.84640, 0.84885, 0.85097, 0.85265,\
        0.85377, 0.85427, 0.85409, 0.85319, 0.85157, 0.84925, 0.84632, 0.84290, 0.83914, 0.83516, 0.83103,\
        0.82674, 0.82220, 0.81733, 0.81236, 0.80757, 0.80293, 0.79833, 0.79379, 0.78880, 0.78300, 0.77674,\
        0.77031, 0.76377, 0.75722, 0.75073, 0.74432, 0.73795, 0.73152, 0.72489, 0.71794, 0.71066, 0.70316,\
        0.69525, 0.68699, 0.67891, 0.67107, 0.66343, 0.65596, 0.64864, 0.64140, 0.63416, 0.62673, 0.61887,\
        0.61024, 0.60069, 0.59101, 0.58104, 0.57080, 0.56049, 0.55030, 0.54041, 0.53101, 0.52229, 0.51446,\
        0.50774, 0.50234, 0.49833, 0.49575, 0.49447, 0.49426, 0.49480, 0.49578, 0.49687, 0.49770, 0.49785],\
        [0.69383, 0.69078, 0.68699, 0.68355, 0.68108, 0.68003, 0.68066, 0.68300, 0.68681, 0.69178, 0.69840,\
        0.70799, 0.71888, 0.72957, 0.73941, 0.74813, 0.75572, 0.76235, 0.76833, 0.77404, 0.77981, 0.78581,\
        0.79200, 0.79821, 0.80464, 0.81075, 0.81549, 0.81859, 0.82014, 0.82021, 0.81878, 0.81601, 0.81228,\
        0.80817, 0.80452, 0.80302, 0.80521, 0.80947, 0.81440, 0.81920, 0.82354, 0.82732, 0.83056, 0.83342,\
        0.83608, 0.83868, 0.84131, 0.84400, 0.84674, 0.84952, 0.85232, 0.85509, 0.85767, 0.85991, 0.86164,\
        0.86279, 0.86331, 0.86318, 0.86233, 0.86074, 0.85840, 0.85538, 0.85180, 0.84784, 0.84363, 0.83925,\
        0.83472, 0.83003, 0.82520, 0.82036, 0.81562, 0.81099, 0.80644, 0.80197, 0.79701, 0.79075, 0.78431,\
        0.77772, 0.77090, 0.76401, 0.75715, 0.75034, 0.74351, 0.73657, 0.72942, 0.72200, 0.71431, 0.70643,\
        0.69829, 0.69008, 0.68228, 0.67468, 0.66714, 0.65967, 0.65226, 0.64486, 0.63736, 0.62957, 0.62128,\
        0.61231, 0.60281, 0.59339, 0.58398, 0.57423, 0.56422, 0.55413, 0.54419, 0.53459, 0.52555, 0.51728,\
        0.51002, 0.50398, 0.49939, 0.49646, 0.49503, 0.49482, 0.49548, 0.49664, 0.49794, 0.49903, 0.49940],\
        [0.69986, 0.69636, 0.69227, 0.68867, 0.68609, 0.68500, 0.68575, 0.68849, 0.69294, 0.69834, 0.70517,\
        0.71461, 0.72537, 0.73609, 0.74594, 0.75453, 0.76177, 0.76788, 0.77325, 0.77839, 0.78375, 0.78964,\
        0.79597, 0.80236, 0.80879, 0.81490, 0.82001, 0.82370, 0.82587, 0.82651, 0.82569, 0.82358, 0.82054,\
        0.81711, 0.81417, 0.81294, 0.81416, 0.81722, 0.82117, 0.82534, 0.82943, 0.83330, 0.83695, 0.84038,\
        0.84366, 0.84680, 0.84981, 0.85269, 0.85547, 0.85825, 0.86110, 0.86399, 0.86674, 0.86912, 0.87088,\
        0.87199, 0.87253, 0.87243, 0.87163, 0.87006, 0.86767, 0.86448, 0.86067, 0.85646, 0.85201, 0.84740,\
        0.84270, 0.83792, 0.83311, 0.82831, 0.82356, 0.81887, 0.81419, 0.80938, 0.80389, 0.79736, 0.79142,\
        0.78470, 0.77753, 0.77033, 0.76318, 0.75605, 0.74884, 0.74147, 0.73386, 0.72595, 0.71773, 0.70921,\
        0.70061, 0.69281, 0.68555, 0.67814, 0.67061, 0.66309, 0.65560, 0.64809, 0.64044, 0.63249, 0.62409,\
        0.61519, 0.60593, 0.59678, 0.58763, 0.57811, 0.56827, 0.55825, 0.54825, 0.53847, 0.52912, 0.52041,\
        0.51257, 0.50589, 0.50065, 0.49727, 0.49561, 0.49534, 0.49606, 0.49732, 0.49865, 0.49970, 0.50011],\
        [0.70279, 0.69952, 0.69581, 0.69240, 0.68983, 0.68875, 0.68970, 0.69299, 0.69841, 0.70500, 0.71272,\
        0.72232, 0.73305, 0.74376, 0.75354, 0.76185, 0.76855, 0.77389, 0.77839, 0.78269, 0.78744, 0.79312,\
        0.79976, 0.80646, 0.81316, 0.81951, 0.82500, 0.82925, 0.83208, 0.83346, 0.83344, 0.83221, 0.83012,\
        0.82765, 0.82550, 0.82444, 0.82490, 0.82673, 0.82948, 0.83276, 0.83633, 0.84009, 0.84395, 0.84784,\
        0.85167, 0.85532, 0.85872, 0.86178, 0.86454, 0.86722, 0.87007, 0.87311, 0.87609, 0.87865, 0.88037,\
        0.88129, 0.88184, 0.88179, 0.88099, 0.87949, 0.87702, 0.87356, 0.86944, 0.86495, 0.86024, 0.85543,\
        0.85059, 0.84575, 0.84092, 0.83611, 0.83131, 0.82647, 0.82150, 0.81622, 0.81036, 0.80432, 0.79846,\
        0.79108, 0.78359, 0.77624, 0.76892, 0.76157, 0.75408, 0.74638, 0.73841, 0.73013, 0.72154, 0.71272,\
        0.70401, 0.69651, 0.68934, 0.68173, 0.67400, 0.66636, 0.65883, 0.65131, 0.64364, 0.63572, 0.62744,\
        0.61882, 0.61000, 0.60113, 0.59201, 0.58257, 0.57278, 0.56276, 0.55270, 0.54276, 0.53312, 0.52398,\
        0.51559, 0.50825, 0.50236, 0.49836, 0.49631, 0.49588, 0.49661, 0.49792, 0.49918, 0.49993, 0.50012],\
        [0.70446, 0.70159, 0.69837, 0.69516, 0.69260, 0.69147, 0.69252, 0.69628, 0.70255, 0.71049, 0.71972,\
        0.73017, 0.74134, 0.75229, 0.76211, 0.77012, 0.77616, 0.78054, 0.78395, 0.78720, 0.79118, 0.79643,\
        0.80319, 0.81061, 0.81790, 0.82470, 0.83067, 0.83551, 0.83903, 0.84119, 0.84206, 0.84182, 0.84076,\
        0.83926, 0.83779, 0.83681, 0.83662, 0.83732, 0.83883, 0.84108, 0.84398, 0.84749, 0.85148, 0.85577,\
        0.86014, 0.86433, 0.86816, 0.87143, 0.87407, 0.87650, 0.87929, 0.88250, 0.88570, 0.88857, 0.89018,\
        0.89045, 0.89124, 0.89122, 0.89026, 0.88902, 0.88639, 0.88251, 0.87802, 0.87322, 0.86828, 0.86328,\
        0.85832, 0.85343, 0.84859, 0.84376, 0.83886, 0.83384, 0.82860, 0.82301, 0.81704, 0.81082, 0.80408,\
        0.79650, 0.78920, 0.78190, 0.77454, 0.76707, 0.75941, 0.75148, 0.74325, 0.73473, 0.72599, 0.71724,\
        0.70892, 0.70141, 0.69360, 0.68531, 0.67727, 0.66958, 0.66210, 0.65466, 0.64710, 0.63932, 0.63126,\
        0.62295, 0.61445, 0.60578, 0.59684, 0.58751, 0.57775, 0.56771, 0.55761, 0.54757, 0.53770, 0.52816,\
        0.51920, 0.51118, 0.50454, 0.49976, 0.49714, 0.49644, 0.49718, 0.49855, 0.49977, 0.50032, 0.50024],\
        [0.70579, 0.70332, 0.70045, 0.69739, 0.69488, 0.69364, 0.69466, 0.69885, 0.70621, 0.71557, 0.72623,\
        0.73780, 0.74975, 0.76127, 0.77137, 0.77921, 0.78454, 0.78786, 0.79017, 0.79232, 0.79544, 0.80085,\
        0.80777, 0.81561, 0.82339, 0.83068, 0.83715, 0.84256, 0.84676, 0.84970, 0.85145, 0.85218, 0.85213,\
        0.85154, 0.85065, 0.84968, 0.84887, 0.84846, 0.84873, 0.84987, 0.85202, 0.85521, 0.85932, 0.86408,\
        0.86904, 0.87381, 0.87818, 0.88184, 0.88426, 0.88618, 0.88886, 0.89240, 0.89563, 0.89863, 0.90031,\
        0.89924, 0.90073, 0.90061, 0.89926, 0.89862, 0.89552, 0.89112, 0.88632, 0.88127, 0.87609, 0.87092,\
        0.86586, 0.86095, 0.85612, 0.85127, 0.84629, 0.84110, 0.83562, 0.82978, 0.82356, 0.81690, 0.80970,\
        0.80222, 0.79496, 0.78762, 0.78024, 0.77273, 0.76496, 0.75687, 0.74847, 0.73979, 0.73094, 0.72212,\
        0.71363, 0.70552, 0.69705, 0.68843, 0.68038, 0.67283, 0.66554, 0.65828, 0.65089, 0.64329, 0.63546,\
        0.62740, 0.61914, 0.61070, 0.60200, 0.59284, 0.58303, 0.57301, 0.56301, 0.55300, 0.54298, 0.53308,\
        0.52355, 0.51476, 0.50722, 0.50151, 0.49805, 0.49699, 0.49783, 0.49929, 0.50032, 0.50065, 0.50039],\
        [0.70697, 0.70477, 0.70218, 0.69956, 0.69732, 0.69604, 0.69715, 0.70186, 0.71032, 0.72079, 0.73252,\
        0.74504, 0.75784, 0.77015, 0.78087, 0.78869, 0.79339, 0.79566, 0.79717, 0.79879, 0.80071, 0.80658,\
        0.81391, 0.82190, 0.82995, 0.83758, 0.84447, 0.85040, 0.85522, 0.85886, 0.86139, 0.86299, 0.86386,\
        0.86412, 0.86379, 0.86282, 0.86132, 0.85977, 0.85873, 0.85872, 0.86007, 0.86294, 0.86726, 0.87263,\
        0.87837, 0.88370, 0.88854, 0.89314, 0.89537, 0.89668, 0.89925, 0.90313, 0.90642, 0.90895, 0.91022,\
        0.91003, 0.91044, 0.91017, 0.90898, 0.90740, 0.90400, 0.89944, 0.89441, 0.88913, 0.88371, 0.87831,\
        0.87317, 0.86831, 0.86355, 0.85870, 0.85365, 0.84831, 0.84263, 0.83657, 0.83010, 0.82321, 0.81593,\
        0.80848, 0.80102, 0.79354, 0.78619, 0.77869, 0.77084, 0.76262, 0.75408, 0.74527, 0.73625, 0.72713,\
        0.71803, 0.70893, 0.69980, 0.69123, 0.68354, 0.67636, 0.66935, 0.66230, 0.65509, 0.64766, 0.63999,\
        0.63210, 0.62400, 0.61571, 0.60720, 0.59821, 0.58832, 0.57857, 0.56894, 0.55915, 0.54910, 0.53890,\
        0.52880, 0.51919, 0.51056, 0.50350, 0.49882, 0.49761, 0.49871, 0.50012, 0.50085, 0.50096, 0.50056],\
        [0.70821, 0.70632, 0.70419, 0.70215, 0.70034, 0.69949, 0.70131, 0.70685, 0.71557, 0.72644, 0.73867,\
        0.75171, 0.76505, 0.77804, 0.78964, 0.79774, 0.80171, 0.80354, 0.80480, 0.80652, 0.80921, 0.81443,\
        0.82154, 0.82945, 0.83754, 0.84536, 0.85257, 0.85892, 0.86426, 0.86847, 0.87161, 0.87390, 0.87554,\
        0.87660, 0.87691, 0.87596, 0.87367, 0.87086, 0.86848, 0.86730, 0.86784, 0.87044, 0.87505, 0.88123,\
        0.88807, 0.89423, 0.89876, 0.90432, 0.90735, 0.90919, 0.91188, 0.91526, 0.91830, 0.92045, 0.92149,\
        0.92159, 0.92131, 0.92046, 0.91883, 0.91632, 0.91258, 0.90781, 0.90248, 0.89696, 0.89117, 0.88543,\
        0.88028, 0.87559, 0.87095, 0.86612, 0.86099, 0.85551, 0.84966, 0.84341, 0.83676, 0.82975, 0.82243,\
        0.81494, 0.80735, 0.79983, 0.79255, 0.78506, 0.77707, 0.76872, 0.76010, 0.75120, 0.74199, 0.73253,\
        0.72285, 0.71300, 0.70322, 0.69469, 0.68739, 0.68055, 0.67377, 0.66686, 0.65975, 0.65242, 0.64485,\
        0.63705, 0.62901, 0.62070, 0.61204, 0.60282, 0.59336, 0.58447, 0.57556, 0.56616, 0.55616, 0.54571,\
        0.53512, 0.52478, 0.51515, 0.50678, 0.50073, 0.49931, 0.50018, 0.50111, 0.50152, 0.50143, 0.50087],\
        [0.70967, 0.70818, 0.70657, 0.70512, 0.70414, 0.70445, 0.70722, 0.71305, 0.72167, 0.73239, 0.74455,\
        0.75756, 0.77089, 0.78390, 0.79564, 0.80408, 0.80827, 0.81063, 0.81250, 0.81482, 0.81826, 0.82345,\
        0.83022, 0.83792, 0.84595, 0.85384, 0.86123, 0.86790, 0.87367, 0.87833, 0.88181, 0.88453, 0.88678,\
        0.88843, 0.88954, 0.88873, 0.88546, 0.88136, 0.87774, 0.87545, 0.87523, 0.87759, 0.88260, 0.88971,\
        0.89775, 0.90506, 0.91062, 0.91594, 0.92008, 0.92310, 0.92600, 0.92892, 0.93143, 0.93313, 0.93386,\
        0.93371, 0.93287, 0.93133, 0.92899, 0.92574, 0.92150, 0.91638, 0.91070, 0.90471, 0.89840, 0.89220,\
        0.88728, 0.88294, 0.87844, 0.87358, 0.86834, 0.86271, 0.85668, 0.85027, 0.84350, 0.83641, 0.82909,\
        0.82162, 0.81410, 0.80667, 0.79943, 0.79180, 0.78353, 0.77511, 0.76652, 0.75762, 0.74832, 0.73864,\
        0.72864, 0.71851, 0.70867, 0.69991, 0.69252, 0.68575, 0.67897, 0.67202, 0.66489, 0.65758, 0.65005,\
        0.64229, 0.63427, 0.62594, 0.61722, 0.60807, 0.59915, 0.59129, 0.58314, 0.57412, 0.56415, 0.55348,\
        0.54250, 0.53165, 0.52146, 0.51262, 0.50621, 0.50335, 0.50284, 0.50283, 0.50272, 0.50227, 0.50146],\
        [0.71144, 0.71041, 0.70941, 0.70871, 0.70873, 0.71013, 0.71363, 0.71963, 0.72800, 0.73830, 0.74999,\
        0.76250, 0.77528, 0.78763, 0.79861, 0.80713, 0.81269, 0.81640, 0.81947, 0.82280, 0.82702, 0.83251,\
        0.83923, 0.84681, 0.85479, 0.86272, 0.87021, 0.87704, 0.88315, 0.88828, 0.89175, 0.89454, 0.89736,\
        0.89899, 0.90074, 0.90011, 0.89593, 0.89095, 0.88649, 0.88330, 0.88243, 0.88464, 0.89026, 0.89826,\
        0.90746, 0.91593, 0.92305, 0.92907, 0.93397, 0.93780, 0.94094, 0.94358, 0.94558, 0.94674, 0.94698,\
        0.94634, 0.94488, 0.94263, 0.93955, 0.93561, 0.93079, 0.92520, 0.91902, 0.91241, 0.90545, 0.89885,\
        0.89472, 0.89063, 0.88607, 0.88105, 0.87562, 0.86982, 0.86364, 0.85708, 0.85021, 0.84308, 0.83578,\
        0.82838, 0.82096, 0.81360, 0.80625, 0.79832, 0.78994, 0.78170, 0.77334, 0.76459, 0.75530, 0.74553,\
        0.73544, 0.72531, 0.71557, 0.70671, 0.69897, 0.69201, 0.68491, 0.67768, 0.67044, 0.66308, 0.65555,\
        0.64781, 0.63986, 0.63168, 0.62328, 0.61481, 0.60677, 0.59933, 0.59165, 0.58285, 0.57282, 0.56197,\
        0.55071, 0.53954, 0.52908, 0.52004, 0.51320, 0.50898, 0.50680, 0.50558, 0.50464, 0.50365, 0.50245],\
        [0.71353, 0.71301, 0.71266, 0.71275, 0.71368, 0.71594, 0.71998, 0.72607, 0.73415, 0.74392, 0.75492,\
        0.76667, 0.77862, 0.79016, 0.80058, 0.80923, 0.81590, 0.82107, 0.82553, 0.83004, 0.83513, 0.84112,\
        0.84803, 0.85567, 0.86369, 0.87169, 0.87923, 0.88601, 0.89217, 0.89797, 0.90145, 0.90402, 0.90693,\
        0.90878, 0.90982, 0.90868, 0.90479, 0.89984, 0.89509, 0.89135, 0.89031, 0.89210, 0.89916, 0.90831,\
        0.91848, 0.92809, 0.93640, 0.94331, 0.94887, 0.95318, 0.95646, 0.95884, 0.96035, 0.96093, 0.96058,\
        0.95930, 0.95716, 0.95419, 0.95040, 0.94579, 0.94039, 0.93431, 0.92768, 0.92068, 0.91362, 0.90746,\
        0.90295, 0.89854, 0.89368, 0.88835, 0.88269, 0.87674, 0.87041, 0.86371, 0.85674, 0.84960, 0.84235,\
        0.83504, 0.82768, 0.82026, 0.81263, 0.80453, 0.79642, 0.78860, 0.78063, 0.77210, 0.76287, 0.75306,\
        0.74295, 0.73286, 0.72319, 0.71427, 0.70630, 0.69894, 0.69113, 0.68360, 0.67627, 0.66886, 0.66127,\
        0.65355, 0.64571, 0.63779, 0.62987, 0.62209, 0.61471, 0.60768, 0.60042, 0.59172, 0.58161, 0.57067,\
        0.55924, 0.54786, 0.53720, 0.52790, 0.52050, 0.51518, 0.51160, 0.50915, 0.50726, 0.50556, 0.50386],\
        [0.71590, 0.71591, 0.71620, 0.71702, 0.71870, 0.72160, 0.72603, 0.73215, 0.73993, 0.74913, 0.75941,\
        0.77032, 0.78139, 0.79216, 0.80222, 0.81111, 0.81866, 0.82507, 0.83083, 0.83649, 0.84248, 0.84907,\
        0.85632, 0.86412, 0.87226, 0.88046, 0.88816, 0.89483, 0.90055, 0.90625, 0.91044, 0.91343, 0.91598,\
        0.91768, 0.91812, 0.91672, 0.91334, 0.90883, 0.90357, 0.90082, 0.90071, 0.90201, 0.91011, 0.92076,\
        0.93171, 0.94203, 0.95107, 0.95859, 0.96457, 0.96909, 0.97232, 0.97439, 0.97538, 0.97534, 0.97432,\
        0.97236, 0.96952, 0.96584, 0.96136, 0.95613, 0.95019, 0.94367, 0.93670, 0.92954, 0.92256, 0.91634,\
        0.91104, 0.90606, 0.90084, 0.89517, 0.88934, 0.88334, 0.87686, 0.87000, 0.86294, 0.85582, 0.84866,\
        0.84147, 0.83421, 0.82682, 0.81921, 0.81135, 0.80350, 0.79599, 0.78842, 0.78007, 0.77081, 0.76091,\
        0.75075, 0.74067, 0.73096, 0.72187, 0.71341, 0.70523, 0.69702, 0.68964, 0.68233, 0.67479, 0.66708,\
        0.65933, 0.65160, 0.64395, 0.63642, 0.62912, 0.62211, 0.61525, 0.60805, 0.59957, 0.58983, 0.57906,\
        0.56751, 0.55594, 0.54508, 0.53550, 0.52757, 0.52137, 0.51670, 0.51316, 0.51034, 0.50790, 0.50563],\
        [0.71849, 0.71902, 0.71990, 0.72135, 0.72363, 0.72701, 0.73170, 0.73781, 0.74529, 0.75397, 0.76358,\
        0.77373, 0.78402, 0.79409, 0.80387, 0.81309, 0.82136, 0.82876, 0.83560, 0.84227, 0.84910, 0.85631,\
        0.86395, 0.87194, 0.88018, 0.88860, 0.89669, 0.90339, 0.90914, 0.91443, 0.91884, 0.92220, 0.92470,\
        0.92623, 0.92649, 0.92524, 0.92253, 0.91887, 0.91520, 0.91366, 0.91454, 0.91791, 0.92560, 0.93609,\
        0.94719, 0.95772, 0.96702, 0.97475, 0.98082, 0.98527, 0.98824, 0.98987, 0.99028, 0.98959, 0.98787,\
        0.98521, 0.98167, 0.97732, 0.97222, 0.96642, 0.96001, 0.95309, 0.94584, 0.93849, 0.93137, 0.92476,\
        0.91873, 0.91301, 0.90724, 0.90125, 0.89549, 0.88954, 0.88279, 0.87570, 0.86861, 0.86159, 0.85462,\
        0.84763, 0.84056, 0.83335, 0.82596, 0.81847, 0.81103, 0.80380, 0.79650, 0.78817, 0.77869, 0.76866,\
        0.75843, 0.74831, 0.73852, 0.72920, 0.72030, 0.71168, 0.70350, 0.69609, 0.68855, 0.68068, 0.67274,\
        0.66492, 0.65729, 0.64983, 0.64256, 0.63548, 0.62856, 0.62164, 0.61436, 0.60626, 0.59719, 0.58660,\
        0.57484, 0.56312, 0.55216, 0.54239, 0.53406, 0.52721, 0.52171, 0.51729, 0.51365, 0.51051, 0.50765],\
        [0.72122, 0.72224, 0.72364, 0.72562, 0.72839, 0.73213, 0.73699, 0.74304, 0.75026, 0.75849, 0.76754,\
        0.77712, 0.78691, 0.79655, 0.80612, 0.81549, 0.82423, 0.83235, 0.84002, 0.84752, 0.85507, 0.86286,\
        0.87093, 0.87915, 0.88730, 0.89552, 0.90392, 0.91102, 0.91711, 0.92246, 0.92698, 0.93055, 0.93315,\
        0.93472, 0.93515, 0.93440, 0.93268, 0.93049, 0.92879, 0.92875, 0.93097, 0.93581, 0.94363, 0.95363,\
        0.96435, 0.97466, 0.98380, 0.99137, 0.99721, 1.00132, 1.00383, 1.00489, 1.00466, 1.00327, 1.00086,\
        0.99751, 0.99333, 0.98838, 0.98272, 0.97643, 0.96959, 0.96233, 0.95479, 0.94718, 0.93972, 0.93261,\
        0.92587, 0.91942, 0.91310, 0.90696, 0.90132, 0.89503, 0.88776, 0.88053, 0.87357, 0.86682, 0.86017,\
        0.85348, 0.84668, 0.83972, 0.83260, 0.82540, 0.81826, 0.81122, 0.80403, 0.79554, 0.78592, 0.77585,\
        0.76562, 0.75550, 0.74567, 0.73621, 0.72710, 0.71837, 0.71017, 0.70247, 0.69449, 0.68616, 0.67795,\
        0.67010, 0.66256, 0.65525, 0.64810, 0.64103, 0.63400, 0.62685, 0.61941, 0.61152, 0.60299, 0.59235,\
        0.58047, 0.56887, 0.55805, 0.54830, 0.53976, 0.53249, 0.52639, 0.52130, 0.51697, 0.51321, 0.50982],\
        [0.72404, 0.72549, 0.72735, 0.72977, 0.73292, 0.73693, 0.74192, 0.74791, 0.75489, 0.76275, 0.77134,\
        0.78050, 0.79005, 0.79971, 0.80908, 0.81835, 0.82735, 0.83596, 0.84424, 0.85236, 0.86049, 0.86878,\
        0.87729, 0.88591, 0.89420, 0.90196, 0.91030, 0.91791, 0.92448, 0.93011, 0.93484, 0.93860, 0.94139,\
        0.94319, 0.94401, 0.94397, 0.94337, 0.94273, 0.94280, 0.94438, 0.94800, 0.95392, 0.96207, 0.97181,\
        0.98211, 0.99198, 1.00069, 1.00781, 1.01314, 1.01669, 1.01857, 1.01895, 1.01801, 1.01594, 1.01287,\
        1.00891, 1.00416, 0.99870, 0.99260, 0.98591, 0.97872, 0.97114, 0.96330, 0.95537, 0.94751, 0.93983,\
        0.93241, 0.92523, 0.91828, 0.91165, 0.90532, 0.89853, 0.89122, 0.88429, 0.87776, 0.87150, 0.86530,\
        0.85901, 0.85252, 0.84579, 0.83885, 0.83179, 0.82466, 0.81746, 0.80989, 0.80138, 0.79214, 0.78217,\
        0.77203, 0.76204, 0.75229, 0.74279, 0.73356, 0.72463, 0.71607, 0.70782, 0.69951, 0.69076, 0.68246,\
        0.67472, 0.66736, 0.66017, 0.65302, 0.64581, 0.63848, 0.63091, 0.62300, 0.61459, 0.60542, 0.59500,\
        0.58394, 0.57303, 0.56267, 0.55314, 0.54460, 0.53710, 0.53061, 0.52501, 0.52015, 0.51587, 0.51200],\
        [0.72688, 0.72872, 0.73097, 0.73376, 0.73721, 0.74144, 0.74652, 0.75246, 0.75925, 0.76680, 0.77500,\
        0.78376, 0.79300, 0.80256, 0.81204, 0.82144, 0.83066, 0.83963, 0.84835, 0.85691, 0.86545, 0.87411,\
        0.88297, 0.89203, 0.90096, 0.90897, 0.91698, 0.92466, 0.93153, 0.93746, 0.94242, 0.94641, 0.94942,\
        0.95150, 0.95278, 0.95346, 0.95390, 0.95460, 0.95617, 0.95920, 0.96407, 0.97090, 0.97947, 0.98917,\
        0.99916, 1.00858, 1.01674, 1.02322, 1.02786, 1.03067, 1.03179, 1.03143, 1.02980, 1.02709, 1.02345,\
        1.01899, 1.01382, 1.00799, 1.00157, 0.99461, 0.98717, 0.97934, 0.97123, 0.96297, 0.95467, 0.94644,\
        0.93835, 0.93041, 0.92267, 0.91513, 0.90769, 0.90020, 0.89342, 0.88715, 0.88132, 0.87573, 0.87010,\
        0.86424, 0.85805, 0.85149, 0.84461, 0.83747, 0.83015, 0.82260, 0.81470, 0.80632, 0.79725, 0.78733,\
        0.77750, 0.76785, 0.75834, 0.74890, 0.73954, 0.73029, 0.72119, 0.71218, 0.70313, 0.69432, 0.68628,\
        0.67887, 0.67176, 0.66466, 0.65742, 0.64993, 0.64216, 0.63404, 0.62546, 0.61626, 0.60627, 0.59600,\
        0.58600, 0.57596, 0.56620, 0.55702, 0.54860, 0.54102, 0.53429, 0.52834, 0.52308, 0.51838, 0.51412],\
        [0.72969, 0.73187, 0.73446, 0.73756, 0.74128, 0.74568, 0.75083, 0.75675, 0.76339, 0.77071, 0.77861,\
        0.78700, 0.79584, 0.80529, 0.81501, 0.82466, 0.83414, 0.84340, 0.85243, 0.86128, 0.87006, 0.87893,\
        0.88798, 0.89716, 0.90641, 0.91515, 0.92347, 0.93131, 0.93840, 0.94458, 0.94975, 0.95392, 0.95712,\
        0.95944, 0.96109, 0.96232, 0.96354, 0.96524, 0.96793, 0.97207, 0.97793, 0.98554, 0.99457, 1.00440,\
        1.01424, 1.02326, 1.03084, 1.03660, 1.04042, 1.04240, 1.04273, 1.04165, 1.03940, 1.03617, 1.03212,\
        1.02735, 1.02194, 1.01595, 1.00939, 1.00231, 0.99476, 0.98678, 0.97847, 0.96992, 0.96123, 0.95248,\
        0.94375, 0.93509, 0.92653, 0.91812, 0.90990, 0.90217, 0.89539, 0.88964, 0.88458, 0.87974, 0.87472,\
        0.86928, 0.86331, 0.85681, 0.84983, 0.84247, 0.83480, 0.82683, 0.81852, 0.80979, 0.80053, 0.79125,\
        0.78211, 0.77305, 0.76390, 0.75457, 0.74508, 0.73549, 0.72586, 0.71620, 0.70659, 0.69764, 0.68990,\
        0.68289, 0.67601, 0.66891, 0.66143, 0.65355, 0.64525, 0.63654, 0.62735, 0.61766, 0.60752, 0.59747,\
        0.58778, 0.57824, 0.56896, 0.56012, 0.55186, 0.54429, 0.53742, 0.53124, 0.52569, 0.52067, 0.51609],\
        [0.73244, 0.73492, 0.73780, 0.74117, 0.74510, 0.74966, 0.75489, 0.76080, 0.76738, 0.77456, 0.78228,\
        0.79047, 0.79917, 0.80853, 0.81829, 0.82810, 0.83780, 0.84731, 0.85658, 0.86561, 0.87448, 0.88340,\
        0.89264, 0.90204, 0.91155, 0.92085, 0.92964, 0.93779, 0.94513, 0.95150, 0.95683, 0.96109, 0.96436,\
        0.96677, 0.96855, 0.97004, 0.97164, 0.97386, 0.97717, 0.98197, 0.98849, 0.99667, 1.00613, 1.01620,\
        1.02601, 1.03473, 1.04175, 1.04676, 1.04974, 1.05089, 1.05048, 1.04880, 1.04610, 1.04258, 1.03838,\
        1.03357, 1.02820, 1.02229, 1.01583, 1.00885, 1.00135, 0.99336, 0.98496, 0.97621, 0.96720, 0.95802,\
        0.94874, 0.93944, 0.93020, 0.92110, 0.91232, 0.90419, 0.89734, 0.89213, 0.88789, 0.88380, 0.87932,\
        0.87420, 0.86834, 0.86176, 0.85454, 0.84683, 0.83874, 0.83034, 0.82164, 0.81262, 0.80347, 0.79461,\
        0.78624, 0.77791, 0.76916, 0.75993, 0.75028, 0.74039, 0.73039, 0.72039, 0.71060, 0.70150, 0.69386,\
        0.68717, 0.68035, 0.67304, 0.66516, 0.65674, 0.64787, 0.63860, 0.62895, 0.61901, 0.60895, 0.59909,\
        0.58943, 0.58013, 0.57117, 0.56259, 0.55450, 0.54696, 0.54003, 0.53371, 0.52795, 0.52269, 0.51786],\
        [0.73510, 0.73784, 0.74098, 0.74458, 0.74870, 0.75340, 0.75871, 0.76465, 0.77120, 0.77835, 0.78603,\
        0.79418, 0.80287, 0.81218, 0.82189, 0.83178, 0.84166, 0.85138, 0.86087, 0.87007, 0.87894, 0.88764,\
        0.89690, 0.90675, 0.91668, 0.92644, 0.93568, 0.94417, 0.95174, 0.95825, 0.96363, 0.96787, 0.97103,\
        0.97327, 0.97486, 0.97616, 0.97763, 0.97978, 0.98310, 0.98800, 0.99468, 1.00311, 1.01286, 1.02318,\
        1.03307, 1.04157, 1.04806, 1.05234, 1.05453, 1.05496, 1.05400, 1.05199, 1.04918, 1.04573, 1.04174,\
        1.03725, 1.03227, 1.02678, 1.02072, 1.01409, 1.00685, 0.99904, 0.99068, 0.98186, 0.97264, 0.96313,\
        0.95343, 0.94365, 0.93389, 0.92432, 0.91515, 0.90676, 0.89981, 0.89497, 0.89159, 0.88814, 0.88401,\
        0.87906, 0.87316, 0.86633, 0.85873, 0.85058, 0.84205, 0.83329, 0.82435, 0.81530, 0.80623, 0.79772,\
        0.79045, 0.78279, 0.77430, 0.76501, 0.75515, 0.74498, 0.73476, 0.72471, 0.71509, 0.70624, 0.69873,\
        0.69197, 0.68487, 0.67708, 0.66858, 0.65950, 0.65001, 0.64022, 0.63022, 0.62010, 0.60999, 0.60012,\
        0.59069, 0.58167, 0.57295, 0.56457, 0.55659, 0.54910, 0.54215, 0.53573, 0.52984, 0.52441, 0.51940],\
        [0.73763, 0.74061, 0.74397, 0.74777, 0.75206, 0.75688, 0.76227, 0.76825, 0.77483, 0.78201, 0.78975,\
        0.79798, 0.80668, 0.81593, 0.82563, 0.83559, 0.84563, 0.85559, 0.86534, 0.87481, 0.88388, 0.89235,\
        0.90083, 0.91136, 0.92195, 0.93217, 0.94176, 0.95052, 0.95826, 0.96484, 0.97016, 0.97420, 0.97702,\
        0.97879, 0.97978, 0.98039, 0.98110, 0.98247, 0.98506, 0.98930, 0.99547, 1.00359, 1.01332, 1.02381,\
        1.03380, 1.04210, 1.04808, 1.05171, 1.05329, 1.05328, 1.05218, 1.05032, 1.04791, 1.04505, 1.04177,\
        1.03808, 1.03393, 1.02925, 1.02394, 1.01796, 1.01124, 1.00380, 0.99566, 0.98689, 0.97760, 0.96789,\
        0.95790, 0.94777, 0.93768, 0.92784, 0.91852, 0.91008, 0.90306, 0.89849, 0.89598, 0.89271, 0.88868,\
        0.88379, 0.87771, 0.87046, 0.86232, 0.85363, 0.84467, 0.83565, 0.82672, 0.81799, 0.80962, 0.80210,\
        0.79532, 0.78786, 0.77929, 0.76971, 0.75948, 0.74900, 0.73865, 0.72873, 0.71951, 0.71123, 0.70392,\
        0.69690, 0.68931, 0.68079, 0.67148, 0.66164, 0.65150, 0.64126, 0.63101, 0.62082, 0.61072, 0.60084,\
        0.59172, 0.58294, 0.57436, 0.56607, 0.55817, 0.55073, 0.54378, 0.53733, 0.53135, 0.52583, 0.52069],\
        [0.74002, 0.74320, 0.74676, 0.75074, 0.75518, 0.76011, 0.76557, 0.77158, 0.77818, 0.78539, 0.79321,\
        0.80153, 0.81026, 0.81952, 0.82926, 0.83933, 0.84958, 0.85981, 0.86988, 0.87968, 0.88928, 0.89864,\
        0.90720, 0.91716, 0.92775, 0.93816, 0.94798, 0.95691, 0.96474, 0.97128, 0.97641, 0.98008, 0.98232,\
        0.98328, 0.98322, 0.98255, 0.98179, 0.98157, 0.98253, 0.98520, 0.98991, 0.99683, 1.00588, 1.01624,\
        1.02624, 1.03423, 1.03966, 1.04284, 1.04422, 1.04436, 1.04381, 1.04288, 1.04162, 1.04004, 1.03812,\
        1.03581, 1.03303, 1.02962, 1.02547, 1.02047, 1.01454, 1.00768, 0.99994, 0.99138, 0.98213, 0.97233,\
        0.96217, 0.95183, 0.94154, 0.93157, 0.92226, 0.91402, 0.90739, 0.90310, 0.90049, 0.89680, 0.89304,\
        0.88830, 0.88187, 0.87396, 0.86512, 0.85581, 0.84641, 0.83721, 0.82843, 0.82028, 0.81293, 0.80643,\
        0.80009, 0.79269, 0.78379, 0.77369, 0.76293, 0.75206, 0.74155, 0.73175, 0.72293, 0.71516, 0.70815,\
        0.70114, 0.69307, 0.68370, 0.67348, 0.66285, 0.65211, 0.64148, 0.63111, 0.62106, 0.61131, 0.60186,\
        0.59278, 0.58395, 0.57535, 0.56709, 0.55925, 0.55186, 0.54494, 0.53850, 0.53251, 0.52693, 0.52173],\
        [0.74224, 0.74561, 0.74934, 0.75347, 0.75805, 0.76309, 0.76861, 0.77463, 0.78119, 0.78836, 0.79620,\
        0.80456, 0.81339, 0.82275, 0.83260, 0.84284, 0.85332, 0.86387, 0.87430, 0.88437, 0.89415, 0.90392,\
        0.91345, 0.92342, 0.93391, 0.94436, 0.95428, 0.96332, 0.97117, 0.97759, 0.98242, 0.98555, 0.98698,\
        0.98679, 0.98522, 0.98267, 0.97967, 0.97696, 0.97530, 0.97532, 0.97732, 0.98159, 0.98865, 0.99818,\
        1.00779, 1.01502, 1.01975, 1.02315, 1.02528, 1.02654, 1.02773, 1.02891, 1.02983, 1.03040, 1.03059,\
        1.03036, 1.02954, 1.02794, 1.02538, 1.02171, 1.01685, 1.01080, 1.00361, 0.99538, 0.98629, 0.97650,\
        0.96624, 0.95576, 0.94533, 0.93529, 0.92601, 0.91793, 0.91150, 0.90691, 0.90338, 0.89999, 0.89704,\
        0.89247, 0.88532, 0.87649, 0.86682, 0.85687, 0.84703, 0.83764, 0.82901, 0.82137, 0.81485, 0.80926,\
        0.80365, 0.79651, 0.78719, 0.77641, 0.76501, 0.75367, 0.74292, 0.73316, 0.72461, 0.71723, 0.71059,\
        0.70379, 0.69540, 0.68517, 0.67411, 0.66278, 0.65153, 0.64062, 0.63023, 0.62045, 0.61126, 0.60239,\
        0.59338, 0.58440, 0.57575, 0.56753, 0.55977, 0.55248, 0.54565, 0.53927, 0.53331, 0.52774, 0.52251],\
        [0.74428, 0.74780, 0.75168, 0.75596, 0.76067, 0.76582, 0.77140, 0.77742, 0.78390, 0.79091, 0.79861,\
        0.80704, 0.81604, 0.82554, 0.83553, 0.84592, 0.85663, 0.86754, 0.87842, 0.88889, 0.89856, 0.90859,\
        0.91879, 0.92923, 0.93991, 0.95050, 0.96056, 0.96969, 0.97752, 0.98378, 0.98822, 0.99069, 0.99109,\
        0.98947, 0.98600, 0.98099, 0.97500, 0.96882, 0.96345, 0.95974, 0.95775, 0.95719, 0.95963, 0.96671,\
        0.97461, 0.97984, 0.98356, 0.98948, 0.99427, 0.99816, 1.00307, 1.00804, 1.01235, 1.01604, 1.01920,\
        1.02180, 1.02362, 1.02440, 1.02388, 1.02189, 1.01836, 1.01329, 1.00679, 0.99900, 0.99013, 0.98041,\
        0.97010, 0.95949, 0.94891, 0.93872, 0.92934, 0.92117, 0.91457, 0.90961, 0.90594, 0.90316, 0.90077,\
        0.89569, 0.88732, 0.87753, 0.86711, 0.85656, 0.84631, 0.83670, 0.82807, 0.82069, 0.81469, 0.80989,\
        0.80521, 0.79856, 0.78874, 0.77729, 0.76529, 0.75347, 0.74240, 0.73252, 0.72408, 0.71696, 0.71055,\
        0.70385, 0.69522, 0.68453, 0.67298, 0.66120, 0.64959, 0.63845, 0.62805, 0.61852, 0.60983, 0.60155,\
        0.59272, 0.58384, 0.57533, 0.56729, 0.55971, 0.55259, 0.54591, 0.53965, 0.53378, 0.52826, 0.52307],\
        [0.74612, 0.74976, 0.75377, 0.75818, 0.76301, 0.76827, 0.77395, 0.78002, 0.78643, 0.79325, 0.80078,\
        0.80923, 0.81832, 0.82791, 0.83797, 0.84845, 0.85930, 0.87045, 0.88181, 0.89311, 0.90326, 0.91300,\
        0.92346, 0.93439, 0.94548, 0.95638, 0.96666, 0.97592, 0.98376, 0.98984, 0.99386, 0.99557, 0.99483,\
        0.99157, 0.98588, 0.97799, 0.96835, 0.95773, 0.94745, 0.93901, 0.93229, 0.92491, 0.91770, 0.91878,\
        0.91911, 0.91991, 0.92346, 0.93898, 0.94900, 0.95780, 0.96990, 0.98056, 0.98939, 0.99719, 1.00422,\
        1.01047, 1.01564, 1.01936, 1.02131, 1.02130, 1.01929, 1.01535, 1.00963, 1.00234, 0.99373, 0.98409,\
        0.97373, 0.96296, 0.95216, 0.94170, 0.93199, 0.92344, 0.91638, 0.91096, 0.90708, 0.90434, 0.90142,\
        0.89568, 0.88686, 0.87663, 0.86578, 0.85483, 0.84422, 0.83434, 0.82555, 0.81810, 0.81214, 0.80776,\
        0.80401, 0.79766, 0.78768, 0.77594, 0.76357, 0.75135, 0.73989, 0.72972, 0.72119, 0.71426, 0.70796,\
        0.70100, 0.69229, 0.68168, 0.67008, 0.65813, 0.64629, 0.63497, 0.62449, 0.61506, 0.60663, 0.59864,\
        0.59038, 0.58205, 0.57401, 0.56634, 0.55909, 0.55225, 0.54579, 0.53970, 0.53396, 0.52854, 0.52341],\
        [0.74774, 0.75148, 0.75559, 0.76010, 0.76504, 0.77042, 0.77624, 0.78245, 0.78893, 0.79562, 0.80301,\
        0.81136, 0.82035, 0.82989, 0.83992, 0.85039, 0.86122, 0.87231, 0.88358, 0.89497, 0.90555, 0.91596,\
        0.92718, 0.93884, 0.95052, 0.96187, 0.97247, 0.98193, 0.98982, 0.99575, 0.99936, 1.00031, 0.99835,\
        0.99333, 0.98526, 0.97428, 0.96066, 0.94489, 0.92834, 0.91385, 0.90225, 0.88815, 0.86948, 0.85283,\
        0.83408, 0.81960, 0.83828, 0.86722, 0.88732, 0.90692, 0.93038, 0.94712, 0.96148, 0.97445, 0.98625,\
        0.99699, 1.00622, 1.01340, 1.01815, 1.02033, 1.01996, 1.01721, 1.01229, 1.00551, 0.99717, 0.98760,\
        0.97715, 0.96618, 0.95505, 0.94417, 0.93392, 0.92470, 0.91689, 0.91071, 0.90609, 0.90270, 0.89896,\
        0.89261, 0.88401, 0.87389, 0.86299, 0.85186, 0.84098, 0.83080, 0.82175, 0.81407, 0.80773, 0.80284,\
        0.79989, 0.79361, 0.78403, 0.77252, 0.76010, 0.74758, 0.73568, 0.72501, 0.71615, 0.70938, 0.70379,\
        0.69647, 0.68763, 0.67727, 0.66585, 0.65391, 0.64195, 0.63044, 0.61978, 0.61030, 0.60203, 0.59440,\
        0.58686, 0.57933, 0.57196, 0.56484, 0.55802, 0.55152, 0.54534, 0.53948, 0.53390, 0.52861, 0.52358],\
        [0.74913, 0.75294, 0.75711, 0.76168, 0.76669, 0.77217, 0.77813, 0.78457, 0.79138, 0.79828, 0.80541,\
        0.81336, 0.82208, 0.83147, 0.84141, 0.85179, 0.86250, 0.87336, 0.88411, 0.89434, 0.90526, 0.91762,\
        0.93012, 0.94267, 0.95503, 0.96691, 0.97791, 0.98763, 0.99563, 1.00148, 1.00474, 1.00497, 1.00181,\
        0.99500, 0.98446, 0.97033, 0.95292, 0.93226, 0.90875, 0.88662, 0.86742, 0.84700, 0.81715, 0.78146,\
        0.75203, 0.73621, 0.75259, 0.78651, 0.82009, 0.85290, 0.88399, 0.90775, 0.92993, 0.94898, 0.96628,\
        0.98236, 0.99628, 1.00728, 1.01501, 1.01944, 1.02071, 1.01910, 1.01495, 1.00862, 1.00050, 0.99096,\
        0.98039, 0.96916, 0.95765, 0.94623, 0.93527, 0.92517, 0.91632, 0.90910, 0.90355, 0.89887, 0.89447,\
        0.88808, 0.87977, 0.86999, 0.85927, 0.84813, 0.83705, 0.82656, 0.81721, 0.80944, 0.80314, 0.79782,\
        0.79379, 0.78785, 0.77895, 0.76788, 0.75556, 0.74285, 0.73051, 0.71922, 0.70965, 0.70257, 0.69760,\
        0.69092, 0.68219, 0.67211, 0.66096, 0.64914, 0.63712, 0.62541, 0.61452, 0.60492, 0.59686, 0.58978,\
        0.58297, 0.57623, 0.56956, 0.56302, 0.55668, 0.55056, 0.54469, 0.53906, 0.53368, 0.52853, 0.52361],\
        [0.75030, 0.75414, 0.75833, 0.76291, 0.76793, 0.77343, 0.77944, 0.78599, 0.79311, 0.80045, 0.80723,\
        0.81481, 0.82332, 0.83259, 0.84246, 0.85277, 0.86340, 0.87421, 0.88508, 0.89582, 0.90703, 0.91967,\
        0.93285, 0.94610, 0.95909, 0.97148, 0.98290, 0.99293, 1.00111, 1.00696, 1.00996, 1.00957, 1.00530,\
        0.99671, 0.98360, 0.96615, 0.94509, 0.92114, 0.89268, 0.85966, 0.82547, 0.79844, 0.75957, 0.71027,\
        0.67475, 0.65741, 0.67892, 0.72631, 0.76304, 0.79732, 0.83210, 0.86625, 0.89764, 0.92222, 0.94582,\
        0.96808, 0.98705, 1.00192, 1.01256, 1.01909, 1.02184, 1.02124, 1.01772, 1.01175, 1.00378, 0.99422,\
        0.98349, 0.97198, 0.96005, 0.94806, 0.93636, 0.92528, 0.91519, 0.90660, 0.90030, 0.89502, 0.88966,\
        0.88321, 0.87519, 0.86578, 0.85534, 0.84431, 0.83312, 0.82231, 0.81256, 0.80473, 0.79891, 0.79389,\
        0.78894, 0.78262, 0.77396, 0.76318, 0.75093, 0.73804, 0.72535, 0.71366, 0.70351, 0.69621, 0.69085,\
        0.68464, 0.67659, 0.66698, 0.65616, 0.64453, 0.63251, 0.62063, 0.60952, 0.59997, 0.59226, 0.58564,\
        0.57941, 0.57328, 0.56720, 0.56118, 0.55528, 0.54952, 0.54394, 0.53855, 0.53335, 0.52836, 0.52355],\
        [0.75124, 0.75507, 0.75924, 0.76379, 0.76875, 0.77415, 0.78004, 0.78641, 0.79323, 0.80026, 0.80748,\
        0.81532, 0.82393, 0.83324, 0.84312, 0.85343, 0.86406, 0.87493, 0.88614, 0.89793, 0.90946, 0.92202,\
        0.93550, 0.94921, 0.96269, 0.97553, 0.98735, 0.99771, 1.00612, 1.01206, 1.01493, 1.01410, 1.00888,\
        0.99862, 0.98283, 0.96147, 0.93544, 0.90680, 0.87486, 0.83270, 0.77617, 0.73571, 0.70592, 0.65718,\
        0.59286, 0.57105, 0.60435, 0.66721, 0.71011, 0.74239, 0.78132, 0.82763, 0.86724, 0.89689, 0.92775,\
        0.95619, 0.97984, 0.99820, 1.01135, 1.01965, 1.02359, 1.02375, 1.02069, 1.01494, 1.00702, 0.99739,\
        0.98648, 0.97469, 0.96237, 0.94988, 0.93753, 0.92562, 0.91448, 0.90455, 0.89697, 0.89138, 0.88565,\
        0.87903, 0.87115, 0.86204, 0.85191, 0.84109, 0.82998, 0.81900, 0.80879, 0.80056, 0.79537, 0.79086,\
        0.78569, 0.77898, 0.77021, 0.75943, 0.74714, 0.73405, 0.72105, 0.70931, 0.69995, 0.69223, 0.68614,\
        0.67971, 0.67194, 0.66268, 0.65217, 0.64075, 0.62884, 0.61693, 0.60572, 0.59630, 0.58894, 0.58258,\
        0.57666, 0.57090, 0.56521, 0.55957, 0.55400, 0.54854, 0.54321, 0.53802, 0.53300, 0.52814, 0.52343],\
        [0.75198, 0.75578, 0.75989, 0.76435, 0.76920, 0.77444, 0.78009, 0.78614, 0.79251, 0.79923, 0.80686,\
        0.81514, 0.82402, 0.83349, 0.84347, 0.85387, 0.86458, 0.87547, 0.88649, 0.89779, 0.91039, 0.92390,\
        0.93786, 0.95194, 0.96577, 0.97898, 0.99114, 1.00181, 1.01049, 1.01659, 1.01947, 1.01840, 1.01252,\
        1.00091, 0.98260, 0.95697, 0.92448, 0.88776, 0.84636, 0.79613, 0.72910, 0.68703, 0.65964, 0.56909,\
        0.49061, 0.48808, 0.51175, 0.59180, 0.65893, 0.69133, 0.73461, 0.79768, 0.83887, 0.87705, 0.91470,\
        0.94814, 0.97550, 0.99663, 1.01173, 1.02131, 1.02605, 1.02667, 1.02383, 1.01815, 1.01019, 1.00043,\
        0.98933, 0.97730, 0.96469, 0.95184, 0.93905, 0.92663, 0.91492, 0.90448, 0.89606, 0.88939, 0.88304,\
        0.87611, 0.86822, 0.85930, 0.84948, 0.83902, 0.82826, 0.81760, 0.80761, 0.79950, 0.79450, 0.78994,\
        0.78445, 0.77732, 0.76823, 0.75728, 0.74489, 0.73165, 0.71835, 0.70602, 0.69672, 0.68996, 0.68365,\
        0.67686, 0.66897, 0.65981, 0.64949, 0.63830, 0.62662, 0.61492, 0.60387, 0.59468, 0.58731, 0.58088,\
        0.57497, 0.56931, 0.56379, 0.55836, 0.55301, 0.54775, 0.54259, 0.53757, 0.53267, 0.52792, 0.52331],\
        [0.75254, 0.75628, 0.76032, 0.76468, 0.76937, 0.77444, 0.77987, 0.78567, 0.79186, 0.79862, 0.80628,\
        0.81473, 0.82380, 0.83344, 0.84359, 0.85417, 0.86507, 0.87618, 0.88740, 0.89886, 0.91179, 0.92567,\
        0.93988, 0.95417, 0.96822, 0.98168, 0.99412, 1.00506, 1.01399, 1.02030, 1.02330, 1.02219, 1.01600,\
        1.00354, 0.98341, 0.95414, 0.91508, 0.86881, 0.80917, 0.73665, 0.69349, 0.66033, 0.56845, 0.45180,\
        0.41674, 0.42092, 0.43567, 0.48904, 0.57039, 0.64404, 0.68887, 0.74852, 0.80499, 0.85976, 0.90603,\
        0.94343, 0.97397, 0.99733, 1.01375, 1.02404, 1.02913, 1.02987, 1.02703, 1.02126, 1.01317, 1.00326,\
        0.99199, 0.97978, 0.96698, 0.95394, 0.94096, 0.92835, 0.91647, 0.90572, 0.89668, 0.88917, 0.88207,\
        0.87468, 0.86664, 0.85781, 0.84828, 0.83827, 0.82813, 0.81828, 0.80936, 0.80219, 0.79663, 0.79140,\
        0.78535, 0.77766, 0.76811, 0.75691, 0.74447, 0.73136, 0.71824, 0.70611, 0.69685, 0.69024, 0.68349,\
        0.67614, 0.66789, 0.65861, 0.64835, 0.63737, 0.62602, 0.61477, 0.60424, 0.59511, 0.58732, 0.58051,\
        0.57434, 0.56856, 0.56303, 0.55764, 0.55237, 0.54721, 0.54216, 0.53723, 0.53242, 0.52775, 0.52321],\
        [0.75295, 0.75663, 0.76058, 0.76482, 0.76939, 0.77429, 0.77955, 0.78522, 0.79138, 0.79818, 0.80583,\
        0.81428, 0.82343, 0.83320, 0.84352, 0.85430, 0.86550, 0.87705, 0.88898, 0.90131, 0.91390, 0.92739,\
        0.94145, 0.95573, 0.96987, 0.98348, 0.99611, 1.00726, 1.01639, 1.02289, 1.02605, 1.02503, 1.01878,\
        1.00597, 0.98489, 0.95328, 0.90871, 0.85039, 0.77253, 0.69648, 0.66585, 0.60723, 0.45550, 0.40319,\
        0.39984, 0.39821, 0.40349, 0.42304, 0.46637, 0.56634, 0.64137, 0.70909, 0.78136, 0.84636, 0.89743,\
        0.94059, 0.97507, 1.00013, 1.01714, 1.02756, 1.03256, 1.03312, 1.03007, 1.02410, 1.01581, 1.00574,\
        0.99433, 0.98201, 0.96915, 0.95608, 0.94311, 0.93057, 0.91876, 0.90805, 0.89871, 0.89034, 0.88250,\
        0.87464, 0.86637, 0.85755, 0.84824, 0.83866, 0.82917, 0.82021, 0.81227, 0.80566, 0.79998, 0.79446,\
        0.78795, 0.77958, 0.76948, 0.75804, 0.74571, 0.73303, 0.72068, 0.70959, 0.70099, 0.69327, 0.68536,\
        0.67727, 0.66853, 0.65898, 0.64868, 0.63783, 0.62678, 0.61596, 0.60586, 0.59666, 0.58840, 0.58112,\
        0.57459, 0.56858, 0.56289, 0.55743, 0.55213, 0.54697, 0.54194, 0.53704, 0.53228, 0.52764, 0.52314],\
        [0.75324, 0.75685, 0.76072, 0.76487, 0.76931, 0.77408, 0.77922, 0.78479, 0.79092, 0.79780, 0.80535,\
        0.81373, 0.82292, 0.83279, 0.84323, 0.85419, 0.86563, 0.87754, 0.88986, 0.90241, 0.91514, 0.92839,\
        0.94222, 0.95641, 0.97056, 0.98424, 0.99698, 1.00825, 1.01750, 1.02411, 1.02736, 1.02639, 1.02013,\
        1.00715, 0.98546, 0.95190, 0.90111, 0.82600, 0.72746, 0.65876, 0.62108, 0.51400, 0.41675, 0.39952,\
        0.39685, 0.39158, 0.39243, 0.40200, 0.42532, 0.48831, 0.59217, 0.67264, 0.75528, 0.83321, 0.89323,\
        0.94292, 0.97934, 1.00461, 1.02134, 1.03133, 1.03589, 1.03606, 1.03268, 1.02643, 1.01792, 1.00768,\
        0.99618, 0.98383, 0.97101, 0.95804, 0.94525, 0.93291, 0.92130, 0.91067, 0.90104, 0.89227, 0.88395,\
        0.87570, 0.86720, 0.85830, 0.84908, 0.83979, 0.83077, 0.82245, 0.81516, 0.80901, 0.80365, 0.79839,\
        0.79142, 0.78226, 0.77161, 0.76002, 0.74795, 0.73587, 0.72435, 0.71403, 0.70511, 0.69654, 0.68828,\
        0.67973, 0.67052, 0.66064, 0.65020, 0.63938, 0.62848, 0.61784, 0.60778, 0.59845, 0.58997, 0.58237,\
        0.57552, 0.56921, 0.56330, 0.55767, 0.55226, 0.54702, 0.54195, 0.53702, 0.53224, 0.52761, 0.52312],\
        [0.75342, 0.75698, 0.76078, 0.76485, 0.76920, 0.77388, 0.77891, 0.78437, 0.79036, 0.79701, 0.80455,\
        0.81305, 0.82232, 0.83222, 0.84270, 0.85372, 0.86526, 0.87731, 0.88978, 0.90241, 0.91509, 0.92816,\
        0.94186, 0.95600, 0.97016, 0.98388, 0.99665, 1.00794, 1.01719, 1.02378, 1.02698, 1.02591, 1.01948,\
        1.00619, 0.98383, 0.94849, 0.89145, 0.80226, 0.69754, 0.62108, 0.56341, 0.47669, 0.41166, 0.39568,\
        0.39071, 0.38621, 0.38707, 0.39438, 0.41108, 0.46357, 0.54852, 0.63027, 0.72620, 0.81235, 0.89614,\
        0.94971, 0.98517, 1.00949, 1.02538, 1.03465, 1.03862, 1.03831, 1.03453, 1.02799, 1.01928, 1.00892,\
        0.99737, 0.98506, 0.97237, 0.95961, 0.94709, 0.93505, 0.92371, 0.91320, 0.90359, 0.89461, 0.88602,\
        0.87757, 0.86886, 0.85978, 0.85047, 0.84119, 0.83234, 0.82429, 0.81732, 0.81149, 0.80653, 0.80164,\
        0.79424, 0.78442, 0.77351, 0.76205, 0.75040, 0.73896, 0.72811, 0.71813, 0.70891, 0.70026, 0.69208,\
        0.68308, 0.67340, 0.66317, 0.65255, 0.64168, 0.63081, 0.62017, 0.61002, 0.60052, 0.59186, 0.58404,\
        0.57692, 0.57034, 0.56416, 0.55831, 0.55271, 0.54733, 0.54214, 0.53714, 0.53231, 0.52764, 0.52314],\
        [0.75352, 0.75703, 0.76079, 0.76480, 0.76910, 0.77371, 0.77868, 0.78406, 0.78993, 0.79641, 0.80391,\
        0.81245, 0.82170, 0.83154, 0.84194, 0.85285, 0.86427, 0.87620, 0.88861, 0.90121, 0.91360, 0.92652,\
        0.94024, 0.95444, 0.96867, 0.98239, 0.99512, 1.00633, 1.01546, 1.02188, 1.02487, 1.02353, 1.01676,\
        1.00303, 0.98017, 0.94474, 0.89089, 0.81479, 0.72405, 0.63828, 0.56283, 0.47770, 0.41451, 0.39546,\
        0.38833, 0.38428, 0.38725, 0.39781, 0.41671, 0.47269, 0.55294, 0.64960, 0.74307, 0.82010, 0.89748,\
        0.95353, 0.98950, 1.01317, 1.02827, 1.03685, 1.04025, 1.03949, 1.03537, 1.02857, 1.01970, 1.00926,\
        0.99772, 0.98551, 0.97301, 0.96054, 0.94839, 0.93676, 0.92578, 0.91548, 0.90580, 0.89677, 0.88853,\
        0.88009, 0.87112, 0.86170, 0.85204, 0.84248, 0.83343, 0.82525, 0.81820, 0.81227, 0.80703, 0.80147,\
        0.79436, 0.78491, 0.77440, 0.76344, 0.75242, 0.74166, 0.73144, 0.72191, 0.71306, 0.70483, 0.69629,\
        0.68670, 0.67662, 0.66614, 0.65536, 0.64442, 0.63352, 0.62285, 0.61257, 0.60287, 0.59405, 0.58609,\
        0.57871, 0.57183, 0.56535, 0.55923, 0.55340, 0.54782, 0.54249, 0.53737, 0.53245, 0.52773, 0.52319],\
        [0.75353, 0.75702, 0.76075, 0.76474, 0.76901, 0.77360, 0.77855, 0.78392, 0.78980, 0.79635, 0.80378,\
        0.81209, 0.82114, 0.83081, 0.84100, 0.85166, 0.86274, 0.87423, 0.88611, 0.89831, 0.91043, 0.92348,\
        0.93749, 0.95191, 0.96622, 0.97992, 0.99253, 1.00354, 1.01243, 1.01857, 1.02124, 1.01956, 1.01243,\
        0.99842, 0.97567, 0.94185, 0.89527, 0.83719, 0.76349, 0.68446, 0.60996, 0.50928, 0.43210, 0.40477,\
        0.39502, 0.38840, 0.39481, 0.41390, 0.45252, 0.52985, 0.62192, 0.69110, 0.77563, 0.84669, 0.90962,\
        0.95830, 0.99230, 1.01507, 1.02950, 1.03753, 1.04049, 1.03937, 1.03499, 1.02801, 1.01903, 1.00856,\
        0.99707, 0.98501, 0.97275, 0.96063, 0.94892, 0.93781, 0.92739, 0.91760, 0.90833, 0.89977, 0.89180,\
        0.88314, 0.87374, 0.86376, 0.85351, 0.84336, 0.83375, 0.82505, 0.81755, 0.81123, 0.80563, 0.79968,\
        0.79237, 0.78369, 0.77400, 0.76383, 0.75360, 0.74358, 0.73401, 0.72497, 0.71644, 0.70814, 0.69938,\
        0.68986, 0.67973, 0.66915, 0.65828, 0.64732, 0.63642, 0.62574, 0.61544, 0.60567, 0.59671, 0.58848,\
        0.58078, 0.57356, 0.56676, 0.56034, 0.55424, 0.54845, 0.54292, 0.53766, 0.53263, 0.52783, 0.52324],\
        [0.75347, 0.75694, 0.76066, 0.76465, 0.76893, 0.77353, 0.77850, 0.78389, 0.78982, 0.79641, 0.80373,\
        0.81181, 0.82062, 0.83005, 0.83997, 0.85030, 0.86093, 0.87179, 0.88281, 0.89399, 0.90601, 0.91963,\
        0.93411, 0.94878, 0.96311, 0.97667, 0.98904, 0.99975, 1.00831, 1.01412, 1.01647, 1.01453, 1.00725,\
        0.99342, 0.97161, 0.94040, 0.89902, 0.84697, 0.78393, 0.71885, 0.66353, 0.62037, 0.50049, 0.43396,\
        0.41102, 0.40099, 0.41344, 0.45362, 0.53431, 0.62173, 0.68124, 0.73379, 0.80030, 0.86687, 0.92228,\
        0.96332, 0.99398, 1.01536, 1.02908, 1.03664, 1.03925, 1.03788, 1.03332, 1.02622, 1.01719, 1.00673,\
        0.99533, 0.98343, 0.97143, 0.95968, 0.94847, 0.93799, 0.92833, 0.91944, 0.91122, 0.90354, 0.89541,\
        0.88632, 0.87636, 0.86566, 0.85460, 0.84364, 0.83319, 0.82368, 0.81542, 0.80851, 0.80259, 0.79656,\
        0.78943, 0.78133, 0.77248, 0.76320, 0.75380, 0.74454, 0.73559, 0.72702, 0.71874, 0.71048, 0.70179,\
        0.69251, 0.68247, 0.67189, 0.66105, 0.65012, 0.63928, 0.62867, 0.61842, 0.60871, 0.59963, 0.59103,\
        0.58296, 0.57539, 0.56826, 0.56153, 0.55515, 0.54911, 0.54338, 0.53795, 0.53280, 0.52791, 0.52326],\
        [0.75330, 0.75677, 0.76050, 0.76451, 0.76882, 0.77346, 0.77847, 0.78389, 0.78980, 0.79630, 0.80350,\
        0.81146, 0.82008, 0.82928, 0.83896, 0.84899, 0.85925, 0.86961, 0.87997, 0.89051, 0.90220, 0.91619,\
        0.93090, 0.94555, 0.95968, 0.97290, 0.98488, 0.99519, 1.00337, 1.00886, 1.01100, 1.00901, 1.00199,\
        0.98896, 0.96891, 0.94110, 0.90540, 0.86241, 0.81398, 0.75750, 0.70063, 0.66770, 0.59869, 0.50090,\
        0.46067, 0.44416, 0.46367, 0.53856, 0.62169, 0.66913, 0.71253, 0.77231, 0.82863, 0.88096, 0.92804,\
        0.96625, 0.99455, 1.01447, 1.02734, 1.03438, 1.03666, 1.03508, 1.03038, 1.02323, 1.01418, 1.00377,\
        0.99246, 0.98071, 0.96894, 0.95753, 0.94681, 0.93700, 0.92819, 0.92031, 0.91313, 0.90602, 0.89811,\
        0.88911, 0.87857, 0.86704, 0.85510, 0.84323, 0.83183, 0.82132, 0.81214, 0.80453, 0.79821, 0.79221,\
        0.78557, 0.77816, 0.77011, 0.76166, 0.75306, 0.74451, 0.73614, 0.72800, 0.72002, 0.71203, 0.70377,\
        0.69468, 0.68464, 0.67412, 0.66339, 0.65261, 0.64190, 0.63140, 0.62122, 0.61149, 0.60223, 0.59339,\
        0.58504, 0.57716, 0.56972, 0.56267, 0.55601, 0.54972, 0.54378, 0.53818, 0.53291, 0.52793, 0.52322],\
        [0.75301, 0.75648, 0.76023, 0.76427, 0.76863, 0.77333, 0.77840, 0.78387, 0.78977, 0.79616, 0.80323,\
        0.81106, 0.81949, 0.82849, 0.83799, 0.84787, 0.85799, 0.86820, 0.87845, 0.88897, 0.90067, 0.91419,\
        0.92847, 0.94262, 0.95618, 0.96881, 0.98022, 0.99004, 0.99784, 1.00309, 1.00520, 1.00348, 0.99717,\
        0.98554, 0.96796, 0.94419, 0.91481, 0.88160, 0.84658, 0.80308, 0.75049, 0.71488, 0.67236, 0.59840,\
        0.53309, 0.51518, 0.56445, 0.64230, 0.68701, 0.71386, 0.75728, 0.81296, 0.85823, 0.90045, 0.93741,\
        0.96982, 0.99500, 1.01302, 1.02473, 1.03108, 1.03295, 1.03113, 1.02631, 1.01911, 1.01008, 0.99971,\
        0.98848, 0.97685, 0.96524, 0.95410, 0.94378, 0.93455, 0.92654, 0.91963, 0.91351, 0.90753, 0.90060,\
        0.89120, 0.87983, 0.86748, 0.85479, 0.84216, 0.82993, 0.81845, 0.80829, 0.80017, 0.79371, 0.78775,\
        0.78145, 0.77457, 0.76716, 0.75940, 0.75148, 0.74355, 0.73571, 0.72796, 0.72025, 0.71246, 0.70449,\
        0.69568, 0.68583, 0.67560, 0.66515, 0.65461, 0.64411, 0.63376, 0.62365, 0.61387, 0.60444, 0.59543,\
        0.58690, 0.57876, 0.57102, 0.56367, 0.55673, 0.55019, 0.54405, 0.53829, 0.53289, 0.52783, 0.52308],\
        [0.75258, 0.75605, 0.75981, 0.76388, 0.76830, 0.77308, 0.77823, 0.78376, 0.78970, 0.79612, 0.80306,\
        0.81059, 0.81878, 0.82760, 0.83701, 0.84691, 0.85716, 0.86762, 0.87818, 0.88900, 0.90061, 0.91351,\
        0.92689, 0.94006, 0.95267, 0.96446, 0.97516, 0.98443, 0.99188, 0.99703, 0.99934, 0.99821, 0.99305,\
        0.98331, 0.96864, 0.94900, 0.92494, 0.89794, 0.86998, 0.83558, 0.79672, 0.76482, 0.72948, 0.66921,\
        0.60851, 0.60195, 0.63963, 0.69843, 0.73381, 0.76427, 0.80201, 0.84654, 0.88420, 0.91712, 0.94829,\
        0.97480, 0.99598, 1.01150, 1.02166, 1.02706, 1.02837, 1.02624, 1.02127, 1.01403, 1.00501, 0.99469,\
        0.98351, 0.97193, 0.96040, 0.94937, 0.93929, 0.93050, 0.92311, 0.91700, 0.91174, 0.90681, 0.90090,\
        0.89120, 0.87929, 0.86651, 0.85345, 0.84047, 0.82785, 0.81583, 0.80477, 0.79634, 0.78992, 0.78382,\
        0.77753, 0.77088, 0.76387, 0.75660, 0.74921, 0.74180, 0.73442, 0.72703, 0.71954, 0.71175, 0.70353,\
        0.69503, 0.68597, 0.67630, 0.66627, 0.65604, 0.64579, 0.63562, 0.62563, 0.61587, 0.60639, 0.59726,\
        0.58850, 0.58009, 0.57205, 0.56441, 0.55720, 0.55044, 0.54411, 0.53820, 0.53270, 0.52758, 0.52281],\
        [0.75196, 0.75542, 0.75918, 0.76329, 0.76776, 0.77261, 0.77784, 0.78346, 0.78944, 0.79581, 0.80257,\
        0.80982, 0.81777, 0.82643, 0.83582, 0.84588, 0.85650, 0.86750, 0.87869, 0.89000, 0.90158, 0.91367,\
        0.92579, 0.93763, 0.94901, 0.95976, 0.96966, 0.97839, 0.98558, 0.99078, 0.99352, 0.99329, 0.98964,\
        0.98218, 0.97072, 0.95534, 0.93655, 0.91552, 0.89295, 0.86708, 0.83891, 0.81393, 0.78816, 0.75004,\
        0.71920, 0.69372, 0.70564, 0.74901, 0.77758, 0.80686, 0.83951, 0.87645, 0.90938, 0.93481, 0.95888,\
        0.98007, 0.99725, 1.00999, 1.01832, 1.02255, 1.02314, 1.02061, 1.01545, 1.00814, 0.99914, 0.98887,\
        0.97774, 0.96617, 0.95460, 0.94352, 0.93346, 0.92487, 0.91792, 0.91235, 0.90757, 0.90247, 0.89698,\
        0.88781, 0.87634, 0.86382, 0.85094, 0.83813, 0.82574, 0.81410, 0.80360, 0.79476, 0.78736, 0.78058,\
        0.77395, 0.76723, 0.76037, 0.75338, 0.74635, 0.73935, 0.73238, 0.72539, 0.71821, 0.71064, 0.70253,\
        0.69426, 0.68565, 0.67643, 0.66677, 0.65686, 0.64687, 0.63691, 0.62707, 0.61740, 0.60794, 0.59871,\
        0.58972, 0.58102, 0.57269, 0.56478, 0.55733, 0.55036, 0.54387, 0.53785, 0.53228, 0.52713, 0.52236],\
        [0.75114, 0.75456, 0.75831, 0.76242, 0.76692, 0.77183, 0.77715, 0.78285, 0.78887, 0.79508, 0.80153,\
        0.80866, 0.81631, 0.82475, 0.83413, 0.84443, 0.85553, 0.86724, 0.87927, 0.89126, 0.90288, 0.91395,\
        0.92455, 0.93483, 0.94484, 0.95451, 0.96362, 0.97187, 0.97892, 0.98434, 0.98772, 0.98865, 0.98676,\
        0.98182, 0.97375, 0.96273, 0.94925, 0.93404, 0.91750, 0.89973, 0.88168, 0.86478, 0.84582, 0.81946,\
        0.78765, 0.76721, 0.78392, 0.81216, 0.82707, 0.84698, 0.87060, 0.89901, 0.92655, 0.94881, 0.96809,\
        0.98478, 0.99832, 1.00833, 1.01472, 1.01765, 1.01742, 1.01441, 1.00901, 1.00164, 0.99267, 0.98247,\
        0.97141, 0.95984, 0.94817, 0.93687, 0.92656, 0.91794, 0.91130, 0.90600, 0.90181, 0.89659, 0.89022,\
        0.88181, 0.87129, 0.85953, 0.84727, 0.83502, 0.82323, 0.81234, 0.80270, 0.79367, 0.78526, 0.77769,\
        0.77056, 0.76360, 0.75668, 0.74980, 0.74299, 0.73628, 0.72968, 0.72313, 0.71643, 0.70939, 0.70183,\
        0.69371, 0.68512, 0.67608, 0.66668, 0.65704, 0.64730, 0.63754, 0.62786, 0.61831, 0.60889, 0.59958,\
        0.59039, 0.58143, 0.57283, 0.56467, 0.55701, 0.54987, 0.54326, 0.53718, 0.53158, 0.52644, 0.52171],\
        [0.75008, 0.75343, 0.75713, 0.76122, 0.76572, 0.77064, 0.77600, 0.78179, 0.78795, 0.79436, 0.80082,\
        0.80716, 0.81414, 0.82229, 0.83162, 0.84213, 0.85370, 0.86610, 0.87899, 0.89176, 0.90341, 0.91335,\
        0.92231, 0.93104, 0.93975, 0.94842, 0.95685, 0.96477, 0.97183, 0.97765, 0.98185, 0.98410, 0.98412,\
        0.98178, 0.97708, 0.97026, 0.96175, 0.95208, 0.94176, 0.93127, 0.92118, 0.91148, 0.90039, 0.88818,\
        0.87699, 0.86952, 0.87433, 0.87948, 0.88224, 0.88961, 0.90170, 0.91952, 0.94106, 0.95978, 0.97548,\
        0.98853, 0.99888, 1.00633, 1.01081, 1.01240, 1.01129, 1.00775, 1.00210, 0.99467, 0.98576, 0.97571,\
        0.96479, 0.95332, 0.94160, 0.93003, 0.91923, 0.91019, 0.90401, 0.89897, 0.89448, 0.88945, 0.88291,\
        0.87475, 0.86499, 0.85409, 0.84261, 0.83107, 0.81989, 0.80947, 0.80009, 0.79109, 0.78258, 0.77462,\
        0.76710, 0.75986, 0.75280, 0.74588, 0.73913, 0.73260, 0.72631, 0.72020, 0.71407, 0.70760, 0.70058,\
        0.69261, 0.68407, 0.67515, 0.66595, 0.65654, 0.64701, 0.63745, 0.62793, 0.61847, 0.60909, 0.59974,\
        0.59036, 0.58116, 0.57233, 0.56398, 0.55616, 0.54891, 0.54224, 0.53613, 0.53056, 0.52548, 0.52084],\
        [0.74875, 0.75200, 0.75562, 0.75963, 0.76407, 0.76895, 0.77428, 0.78004, 0.78622, 0.79276, 0.79921,\
        0.80433, 0.81092, 0.81883, 0.82804, 0.83864, 0.85051, 0.86339, 0.87685, 0.89026, 0.90211, 0.91064,\
        0.91818, 0.92569, 0.93336, 0.94126, 0.94923, 0.95701, 0.96425, 0.97062, 0.97576, 0.97940, 0.98135,\
        0.98152, 0.98000, 0.97702, 0.97296, 0.96830, 0.96352, 0.95912, 0.95545, 0.95255, 0.95044, 0.95063,\
        0.95335, 0.95508, 0.94999, 0.93991, 0.93259, 0.93070, 0.93450, 0.94379, 0.95688, 0.97011, 0.98175,\
        0.99136, 0.99877, 1.00384, 1.00650, 1.00677, 1.00478, 1.00071, 0.99482, 0.98735, 0.97857, 0.96875,\
        0.95813, 0.94695, 0.93541, 0.92381, 0.91259, 0.90282, 0.89658, 0.89204, 0.88726, 0.88189, 0.87532,\
        0.86738, 0.85815, 0.84795, 0.83723, 0.82641, 0.81585, 0.80565, 0.79634, 0.78774, 0.77930, 0.77120,\
        0.76344, 0.75596, 0.74869, 0.74164, 0.73483, 0.72833, 0.72222, 0.71649, 0.71093, 0.70496, 0.69811,\
        0.69049, 0.68219, 0.67347, 0.66448, 0.65528, 0.64596, 0.63657, 0.62717, 0.61779, 0.60841, 0.59898,\
        0.58946, 0.58010, 0.57111, 0.56263, 0.55471, 0.54741, 0.54074, 0.53468, 0.52919, 0.52422, 0.51973],\
        [0.74716, 0.75027, 0.75376, 0.75765, 0.76197, 0.76673, 0.77192, 0.77749, 0.78337, 0.78933, 0.79484,\
        0.80019, 0.80688, 0.81437, 0.82328, 0.83382, 0.84577, 0.85872, 0.87213, 0.88520, 0.89640, 0.90446,\
        0.91162, 0.91851, 0.92551, 0.93295, 0.94072, 0.94855, 0.95614, 0.96316, 0.96931, 0.97434, 0.97811,\
        0.98059, 0.98189, 0.98225, 0.98199, 0.98154, 0.98132, 0.98173, 0.98308, 0.98554, 0.98934, 0.99461,\
        0.99977, 1.00104, 0.99500, 0.98366, 0.97267, 0.96552, 0.96338, 0.96605, 0.97224, 0.97981, 0.98712,\
        0.99330, 0.99793, 1.00077, 1.00172, 1.00074, 0.99790, 0.99333, 0.98721, 0.97975, 0.97117, 0.96170,\
        0.95155, 0.94090, 0.92993, 0.91884, 0.90794, 0.89822, 0.89133, 0.88595, 0.88063, 0.87476, 0.86796,\
        0.86009, 0.85120, 0.84152, 0.83142, 0.82125, 0.81139, 0.80223, 0.79291, 0.78409, 0.77564, 0.76747,\
        0.75955, 0.75187, 0.74440, 0.73714, 0.73014, 0.72350, 0.71736, 0.71184, 0.70684, 0.70164, 0.69494,\
        0.68744, 0.67940, 0.67096, 0.66222, 0.65325, 0.64411, 0.63487, 0.62557, 0.61623, 0.60681, 0.59726,\
        0.58764, 0.57818, 0.56911, 0.56056, 0.55262, 0.54535, 0.53874, 0.53280, 0.52746, 0.52268, 0.51838],\
        [0.74529, 0.74822, 0.75154, 0.75526, 0.75942, 0.76400, 0.76898, 0.77427, 0.77976, 0.78525, 0.79062,\
        0.79629, 0.80243, 0.80893, 0.81746, 0.82786, 0.83968, 0.85235, 0.86518, 0.87729, 0.88762, 0.89591,\
        0.90331, 0.90978, 0.91631, 0.92365, 0.93146, 0.93949, 0.94753, 0.95527, 0.96241, 0.96872, 0.97411,\
        0.97859, 0.98228, 0.98539, 0.98821, 0.99108, 0.99434, 0.99829, 1.00311, 1.00887, 1.01540, 1.02198,\
        1.02679, 1.02723, 1.02172, 1.01166, 1.00053, 0.99135, 0.98569, 0.98378, 0.98491, 0.98778, 0.99115,\
        0.99414, 0.99619, 0.99700, 0.99637, 0.99424, 0.99062, 0.98560, 0.97930, 0.97190, 0.96358, 0.95455,\
        0.94500, 0.93511, 0.92504, 0.91495, 0.90504, 0.89576, 0.88783, 0.88114, 0.87484, 0.86829, 0.86111,\
        0.85315, 0.84441, 0.83508, 0.82542, 0.81578, 0.80634, 0.79735, 0.78881, 0.78016, 0.77173, 0.76353,\
        0.75552, 0.74768, 0.74001, 0.73252, 0.72525, 0.71831, 0.71189, 0.70626, 0.70144, 0.69660, 0.69048,\
        0.68338, 0.67572, 0.66763, 0.65919, 0.65046, 0.64150, 0.63237, 0.62312, 0.61379, 0.60432, 0.59466,\
        0.58494, 0.57542, 0.56631, 0.55776, 0.54987, 0.54269, 0.53624, 0.53048, 0.52538, 0.52085, 0.51682],\
        [0.74316, 0.74587, 0.74898, 0.75250, 0.75647, 0.76085, 0.76561, 0.77065, 0.77585, 0.78106, 0.78625,\
        0.79153, 0.79674, 0.80277, 0.81110, 0.82130, 0.83286, 0.84506, 0.85715, 0.86840, 0.87823, 0.88659,\
        0.89384, 0.89987, 0.90626, 0.91389, 0.92173, 0.92998, 0.93855, 0.94702, 0.95505, 0.96246, 0.96919,\
        0.97529, 0.98086, 0.98611, 0.99126, 0.99657, 1.00228, 1.00857, 1.01548, 1.02287, 1.03028, 1.03677,\
        1.04083, 1.04085, 1.03613, 1.02762, 1.01751, 1.00805, 1.00069, 0.99600, 0.99370, 0.99307, 0.99327,\
        0.99354, 0.99336, 0.99238, 0.99037, 0.98723, 0.98294, 0.97753, 0.97111, 0.96379, 0.95577, 0.94722,\
        0.93833, 0.92927, 0.92021, 0.91126, 0.90226, 0.89315, 0.88462, 0.87693, 0.86969, 0.86244, 0.85483,\
        0.84668, 0.83798, 0.82884, 0.81947, 0.81020, 0.80148, 0.79274, 0.78430, 0.77595, 0.76767, 0.75950,\
        0.75144, 0.74349, 0.73565, 0.72794, 0.72040, 0.71312, 0.70629, 0.70030, 0.69547, 0.69063, 0.68499,\
        0.67845, 0.67126, 0.66356, 0.65546, 0.64698, 0.63818, 0.62911, 0.61985, 0.61052, 0.60101, 0.59121,\
        0.58140, 0.57184, 0.56274, 0.55425, 0.54647, 0.53946, 0.53324, 0.52776, 0.52296, 0.51876, 0.51506],\
        [0.74078, 0.74324, 0.74610, 0.74941, 0.75317, 0.75737, 0.76196, 0.76684, 0.77189, 0.77697, 0.78198,\
        0.78686, 0.79175, 0.79746, 0.80512, 0.81497, 0.82617, 0.83775, 0.84904, 0.85950, 0.86881, 0.87690,\
        0.88389, 0.89012, 0.89686, 0.90435, 0.91177, 0.92027, 0.92943, 0.93855, 0.94727, 0.95551, 0.96325,\
        0.97055, 0.97751, 0.98429, 0.99106, 0.99801, 1.00527, 1.01291, 1.02085, 1.02881, 1.03621, 1.04218,\
        1.04561, 1.04550, 1.04156, 1.03446, 1.02562, 1.01665, 1.00878, 1.00262, 0.99822, 0.99523, 0.99308,\
        0.99125, 0.98926, 0.98680, 0.98365, 0.97967, 0.97483, 0.96913, 0.96262, 0.95542, 0.94769, 0.93961,\
        0.93135, 0.92303, 0.91481, 0.90685, 0.89863, 0.88965, 0.88089, 0.87266, 0.86480, 0.85701, 0.84904,\
        0.84072, 0.83202, 0.82301, 0.81385, 0.80469, 0.79616, 0.78805, 0.77990, 0.77175, 0.76361, 0.75549,\
        0.74741, 0.73938, 0.73142, 0.72356, 0.71582, 0.70830, 0.70124, 0.69510, 0.68986, 0.68478, 0.67924,\
        0.67305, 0.66625, 0.65893, 0.65116, 0.64294, 0.63429, 0.62522, 0.61583, 0.60634, 0.59679, 0.58691,\
        0.57707, 0.56751, 0.55846, 0.55006, 0.54246, 0.53570, 0.52978, 0.52466, 0.52026, 0.51646, 0.51316],\
        [0.73817, 0.74035, 0.74294, 0.74600, 0.74956, 0.75362, 0.75812, 0.76298, 0.76807, 0.77323, 0.77832,\
        0.78327, 0.78820, 0.79362, 0.80043, 0.80988, 0.82045, 0.83118, 0.84152, 0.85113, 0.85981, 0.86754,\
        0.87447, 0.88101, 0.88776, 0.89467, 0.90181, 0.91086, 0.92058, 0.93005, 0.93916, 0.94790, 0.95628,\
        0.96436, 0.97223, 0.97998, 0.98775, 0.99564, 1.00372, 1.01197, 1.02022, 1.02813, 1.03515, 1.04054,\
        1.04350, 1.04342, 1.04019, 1.03428, 1.02668, 1.01853, 1.01081, 1.00410, 0.99860, 0.99416, 0.99047,\
        0.98714, 0.98381, 0.98022, 0.97617, 0.97155, 0.96629, 0.96040, 0.95388, 0.94681, 0.93934, 0.93168,\
        0.92400, 0.91630, 0.90853, 0.90093, 0.89329, 0.88488, 0.87634, 0.86801, 0.85990, 0.85184, 0.84367,\
        0.83526, 0.82657, 0.81769, 0.80885, 0.80015, 0.79172, 0.78370, 0.77575, 0.76775, 0.75971, 0.75163,\
        0.74353, 0.73545, 0.72740, 0.71945, 0.71163, 0.70401, 0.69685, 0.69045, 0.68471, 0.67921, 0.67353,\
        0.66745, 0.66089, 0.65388, 0.64641, 0.63846, 0.63000, 0.62095, 0.61132, 0.60144, 0.59174, 0.58191,\
        0.57208, 0.56254, 0.55354, 0.54528, 0.53789, 0.53145, 0.52592, 0.52126, 0.51733, 0.51401, 0.51117],\
        [0.73537, 0.73722, 0.73951, 0.74231, 0.74568, 0.74963, 0.75413, 0.75911, 0.76443, 0.76994, 0.77548,\
        0.78092, 0.78629, 0.79189, 0.79850, 0.80689, 0.81627, 0.82576, 0.83494, 0.84355, 0.85150, 0.85880,\
        0.86561, 0.87221, 0.87889, 0.88581, 0.89344, 0.90269, 0.91228, 0.92161, 0.93074, 0.93964, 0.94832,\
        0.95680, 0.96514, 0.97339, 0.98163, 0.98990, 0.99824, 1.00655, 1.01466, 1.02221, 1.02870, 1.03356,\
        1.03619, 1.03622, 1.03360, 1.02870, 1.02222, 1.01500, 1.00780, 1.00114, 0.99524, 0.99010, 0.98553,\
        0.98126, 0.97704, 0.97265, 0.96796, 0.96288, 0.95737, 0.95140, 0.94496, 0.93805, 0.93076, 0.92339,\
        0.91631, 0.90928, 0.90183, 0.89442, 0.88701, 0.87918, 0.87109, 0.86297, 0.85490, 0.84682, 0.83865,\
        0.83028, 0.82170, 0.81292, 0.80420, 0.79598, 0.78785, 0.77989, 0.77200, 0.76406, 0.75605, 0.74798,\
        0.73985, 0.73171, 0.72359, 0.71556, 0.70772, 0.70008, 0.69275, 0.68600, 0.67977, 0.67382, 0.66788,\
        0.66175, 0.65530, 0.64849, 0.64128, 0.63362, 0.62542, 0.61655, 0.60687, 0.59669, 0.58664, 0.57659,\
        0.56664, 0.55706, 0.54810, 0.53998, 0.53286, 0.52679, 0.52175, 0.51763, 0.51427, 0.51151, 0.50919],\
        [0.73241, 0.73389, 0.73583, 0.73835, 0.74152, 0.74538, 0.74995, 0.75516, 0.76090, 0.76701, 0.77330,\
        0.77959, 0.78578, 0.79192, 0.79839, 0.80569, 0.81354, 0.82149, 0.82928, 0.83678, 0.84391, 0.85069,\
        0.85726, 0.86382, 0.87059, 0.87781, 0.88588, 0.89494, 0.90401, 0.91299, 0.92194, 0.93076, 0.93944,\
        0.94802, 0.95647, 0.96482, 0.97309, 0.98131, 0.98948, 0.99750, 1.00519, 1.01222, 1.01818, 1.02259,\
        1.02500, 1.02517, 1.02307, 1.01901, 1.01352, 1.00723, 1.00076, 0.99452, 0.98875, 0.98347, 0.97856,\
        0.97383, 0.96910, 0.96422, 0.95911, 0.95375, 0.94812, 0.94221, 0.93597, 0.92931, 0.92216, 0.91480,\
        0.90815, 0.90224, 0.89511, 0.88785, 0.88056, 0.87309, 0.86541, 0.85763, 0.84980, 0.84191, 0.83392,\
        0.82576, 0.81743, 0.80893, 0.80036, 0.79226, 0.78440, 0.77654, 0.76866, 0.76071, 0.75269, 0.74457,\
        0.73639, 0.72817, 0.71995, 0.71179, 0.70383, 0.69612, 0.68860, 0.68150, 0.67484, 0.66847, 0.66222,\
        0.65594, 0.64949, 0.64280, 0.63578, 0.62836, 0.62048, 0.61199, 0.60251, 0.59211, 0.58152, 0.57105,\
        0.56087, 0.55119, 0.54225, 0.53428, 0.52745, 0.52184, 0.51738, 0.51391, 0.51121, 0.50906, 0.50731],\
        [0.72933, 0.73038, 0.73193, 0.73411, 0.73704, 0.74083, 0.74549, 0.75101, 0.75728, 0.76414, 0.77139,\
        0.77880, 0.78610, 0.79297, 0.79930, 0.80551, 0.81171, 0.81797, 0.82430, 0.83063, 0.83689, 0.84310,\
        0.84936, 0.85581, 0.86262, 0.86997, 0.87801, 0.88661, 0.89533, 0.90409, 0.91277, 0.92130, 0.92979,\
        0.93821, 0.94651, 0.95464, 0.96262, 0.97045, 0.97814, 0.98563, 0.99275, 0.99922, 1.00469, 1.00874,\
        1.01102, 1.01133, 1.00968, 1.00631, 1.00166, 0.99627, 0.99061, 0.98505, 0.97978, 0.97479, 0.96997,\
        0.96517, 0.96025, 0.95512, 0.94978, 0.94425, 0.93861, 0.93287, 0.92700, 0.92083, 0.91399, 0.90645,\
        0.89941, 0.89420, 0.88784, 0.88093, 0.87391, 0.86677, 0.85950, 0.85213, 0.84466, 0.83711, 0.82944,\
        0.82162, 0.81367, 0.80565, 0.79753, 0.78942, 0.78149, 0.77361, 0.76569, 0.75769, 0.74960, 0.74141,\
        0.73315, 0.72483, 0.71649, 0.70814, 0.69989, 0.69199, 0.68429, 0.67688, 0.66981, 0.66303, 0.65645,\
        0.64996, 0.64344, 0.63679, 0.62987, 0.62258, 0.61487, 0.60674, 0.59753, 0.58688, 0.57596, 0.56520,\
        0.55483, 0.54505, 0.53611, 0.52829, 0.52179, 0.51671, 0.51295, 0.51024, 0.50828, 0.50681, 0.50563],\
        [0.72618, 0.72675, 0.72784, 0.72960, 0.73224, 0.73589, 0.74064, 0.74647, 0.75330, 0.76096, 0.76922,\
        0.77782, 0.78641, 0.79438, 0.80074, 0.80562, 0.80998, 0.81461, 0.81960, 0.82484, 0.83026, 0.83588,\
        0.84177, 0.84804, 0.85479, 0.86209, 0.86992, 0.87815, 0.88655, 0.89501, 0.90325, 0.91135, 0.91956,\
        0.92767, 0.93560, 0.94329, 0.95071, 0.95789, 0.96489, 0.97169, 0.97818, 0.98412, 0.98918, 0.99299,\
        0.99521, 0.99567, 0.99437, 0.99155, 0.98760, 0.98301, 0.97821, 0.97351, 0.96900, 0.96465, 0.96028,\
        0.95572, 0.95085, 0.94565, 0.94017, 0.93455, 0.92896, 0.92344, 0.91800, 0.91263, 0.90659, 0.89931,\
        0.89222, 0.88621, 0.88012, 0.87367, 0.86703, 0.86029, 0.85347, 0.84655, 0.83953, 0.83240, 0.82515,\
        0.81777, 0.81020, 0.80252, 0.79477, 0.78686, 0.77895, 0.77102, 0.76303, 0.75495, 0.74676, 0.73846,\
        0.73009, 0.72166, 0.71321, 0.70473, 0.69625, 0.68797, 0.67992, 0.67214, 0.66465, 0.65746, 0.65052,\
        0.64377, 0.63713, 0.63045, 0.62357, 0.61627, 0.60840, 0.60007, 0.59095, 0.58060, 0.56977, 0.55898,\
        0.54856, 0.53876, 0.52987, 0.52219, 0.51605, 0.51159, 0.50867, 0.50685, 0.50569, 0.50489, 0.50427],\
        [0.72302, 0.72306, 0.72360, 0.72486, 0.72709, 0.73053, 0.73529, 0.74139, 0.74874, 0.75711, 0.76624,\
        0.77583, 0.78548, 0.79458, 0.80143, 0.80462, 0.80733, 0.81079, 0.81480, 0.81916, 0.82385, 0.82890,\
        0.83440, 0.84045, 0.84707, 0.85423, 0.86183, 0.86972, 0.87774, 0.88571, 0.89348, 0.90121, 0.90909,\
        0.91670, 0.92412, 0.93123, 0.93788, 0.94421, 0.95038, 0.95641, 0.96225, 0.96772, 0.97251, 0.97621,\
        0.97846, 0.97907, 0.97804, 0.97561, 0.97219, 0.96828, 0.96433, 0.96060, 0.95711, 0.95368, 0.95003,\
        0.94596, 0.94134, 0.93617, 0.93059, 0.92486, 0.91932, 0.91403, 0.90878, 0.90417, 0.89910, 0.89225,\
        0.88518, 0.87869, 0.87248, 0.86627, 0.86000, 0.85371, 0.84737, 0.84096, 0.83443, 0.82778, 0.82101,\
        0.81414, 0.80706, 0.79958, 0.79214, 0.78445, 0.77660, 0.76865, 0.76059, 0.75242, 0.74411, 0.73568,\
        0.72716, 0.71858, 0.70999, 0.70136, 0.69263, 0.68397, 0.67551, 0.66730, 0.65937, 0.65173, 0.64439,\
        0.63734, 0.63052, 0.62379, 0.61694, 0.60965, 0.60159, 0.59293, 0.58370, 0.57366, 0.56313, 0.55254,\
        0.54224, 0.53255, 0.52377, 0.51625, 0.51045, 0.50672, 0.50484, 0.50401, 0.50364, 0.50345, 0.50333],\
        [0.71995, 0.71940, 0.71930, 0.71994, 0.72164, 0.72472, 0.72940, 0.73569, 0.74345, 0.75237, 0.76210,\
        0.77224, 0.78228, 0.79144, 0.79824, 0.80081, 0.80315, 0.80629, 0.80973, 0.81348, 0.81756, 0.82207,\
        0.82720, 0.83303, 0.83952, 0.84652, 0.85389, 0.86145, 0.86906, 0.87659, 0.88398, 0.89133, 0.89864,\
        0.90551, 0.91248, 0.91896, 0.92461, 0.92994, 0.93523, 0.94047, 0.94568, 0.95080, 0.95548, 0.95923,\
        0.96160, 0.96236, 0.96150, 0.95930, 0.95621, 0.95282, 0.94967, 0.94703, 0.94477, 0.94252, 0.93984,\
        0.93643, 0.93217, 0.92710, 0.92142, 0.91548, 0.90987, 0.90512, 0.89984, 0.89513, 0.89031, 0.88416,\
        0.87751, 0.87103, 0.86482, 0.85879, 0.85289, 0.84707, 0.84127, 0.83541, 0.82941, 0.82324, 0.81693,\
        0.81056, 0.80422, 0.79734, 0.78982, 0.78216, 0.77435, 0.76640, 0.75829, 0.75003, 0.74161, 0.73302,\
        0.72430, 0.71551, 0.70671, 0.69786, 0.68887, 0.67990, 0.67106, 0.66241, 0.65400, 0.64588, 0.63810,\
        0.63068, 0.62361, 0.61679, 0.60999, 0.60281, 0.59465, 0.58582, 0.57647, 0.56661, 0.55640, 0.54613,\
        0.53613, 0.52670, 0.51817, 0.51089, 0.50540, 0.50249, 0.50192, 0.50204, 0.50230, 0.50259, 0.50285],\
        [0.71706, 0.71589, 0.71508, 0.71496, 0.71596, 0.71853, 0.72301, 0.72942, 0.73750, 0.74680, 0.75684,\
        0.76710, 0.77696, 0.78560, 0.79191, 0.79541, 0.79825, 0.80132, 0.80444, 0.80782, 0.81141, 0.81540,\
        0.82018, 0.82586, 0.83227, 0.83915, 0.84628, 0.85350, 0.86070, 0.86778, 0.87474, 0.88157, 0.88819,\
        0.89456, 0.90116, 0.90689, 0.91124, 0.91559, 0.92013, 0.92456, 0.92914, 0.93408, 0.93889, 0.94284,\
        0.94544, 0.94636, 0.94557, 0.94341, 0.94043, 0.93735, 0.93490, 0.93344, 0.93266, 0.93183, 0.93028,\
        0.92763, 0.92378, 0.91885, 0.91312, 0.90696, 0.90099, 0.89648, 0.89171, 0.88663, 0.88144, 0.87561,\
        0.86934, 0.86306, 0.85701, 0.85123, 0.84571, 0.84040, 0.83520, 0.82996, 0.82455, 0.81887, 0.81290,\
        0.80677, 0.80069, 0.79438, 0.78729, 0.77983, 0.77211, 0.76418, 0.75605, 0.74773, 0.73921, 0.73048,\
        0.72154, 0.71244, 0.70332, 0.69423, 0.68505, 0.67581, 0.66660, 0.65752, 0.64863, 0.64000, 0.63172,\
        0.62387, 0.61649, 0.60949, 0.60261, 0.59541, 0.58736, 0.57861, 0.56937, 0.55974, 0.54987, 0.54001,\
        0.53045, 0.52148, 0.51343, 0.50668, 0.50176, 0.49982, 0.50053, 0.50112, 0.50171, 0.50228, 0.50280],\
        [0.71445, 0.71268, 0.71113, 0.71015, 0.71026, 0.71213, 0.71631, 0.72279, 0.73113, 0.74069, 0.75084,\
        0.76098, 0.77048, 0.77867, 0.78497, 0.78939, 0.79286, 0.79602, 0.79910, 0.80237, 0.80551, 0.80895,\
        0.81347, 0.81913, 0.82555, 0.83231, 0.83919, 0.84603, 0.85277, 0.85936, 0.86580, 0.87207, 0.87816,\
        0.88411, 0.88990, 0.89468, 0.89810, 0.90185, 0.90587, 0.90944, 0.91333, 0.91837, 0.92353, 0.92775,\
        0.93069, 0.93179, 0.93096, 0.92870, 0.92566, 0.92262, 0.92067, 0.92054, 0.92152, 0.92226, 0.92189,\
        0.91999, 0.91650, 0.91170, 0.90600, 0.89984, 0.89389, 0.88873, 0.88375, 0.87851, 0.87301, 0.86715,\
        0.86104, 0.85492, 0.84905, 0.84355, 0.83846, 0.83372, 0.82919, 0.82467, 0.81996, 0.81483, 0.80921,\
        0.80320, 0.79724, 0.79122, 0.78459, 0.77739, 0.76981, 0.76192, 0.75380, 0.74544, 0.73687, 0.72805,\
        0.71895, 0.70957, 0.70005, 0.69075, 0.68134, 0.67181, 0.66225, 0.65274, 0.64337, 0.63422, 0.62541,\
        0.61708, 0.60934, 0.60213, 0.59512, 0.58786, 0.57998, 0.57148, 0.56250, 0.55319, 0.54373, 0.53436,\
        0.52535, 0.51702, 0.50973, 0.50390, 0.50018, 0.49936, 0.50017, 0.50093, 0.50165, 0.50239, 0.50310],\
        [0.71219, 0.70989, 0.70767, 0.70583, 0.70488, 0.70583, 0.70964, 0.71622, 0.72479, 0.73450, 0.74462,\
        0.75452, 0.76367, 0.77160, 0.77802, 0.78303, 0.78708, 0.79062, 0.79394, 0.79716, 0.80000, 0.80295,\
        0.80736, 0.81323, 0.81968, 0.82626, 0.83278, 0.83915, 0.84535, 0.85137, 0.85723, 0.86292, 0.86843,\
        0.87373, 0.87864, 0.88275, 0.88610, 0.88946, 0.89289, 0.89594, 0.89928, 0.90490, 0.91016, 0.91442,\
        0.91799, 0.91930, 0.91821, 0.91584, 0.91282, 0.90960, 0.90782, 0.90922, 0.91214, 0.91435, 0.91502,\
        0.91376, 0.91047, 0.90564, 0.89994, 0.89382, 0.88773, 0.88200, 0.87647, 0.87084, 0.86503, 0.85899,\
        0.85284, 0.84677, 0.84103, 0.83578, 0.83113, 0.82701, 0.82325, 0.81957, 0.81568, 0.81128, 0.80616,\
        0.80044, 0.79461, 0.78857, 0.78202, 0.77494, 0.76743, 0.75958, 0.75146, 0.74310, 0.73451, 0.72569,\
        0.71658, 0.70714, 0.69742, 0.68770, 0.67790, 0.66800, 0.65806, 0.64817, 0.63837, 0.62875, 0.61941,\
        0.61057, 0.60248, 0.59514, 0.58804, 0.58074, 0.57298, 0.56472, 0.55604, 0.54709, 0.53808, 0.52923,\
        0.52085, 0.51326, 0.50685, 0.50208, 0.49946, 0.49917, 0.50012, 0.50097, 0.50182, 0.50271, 0.50359],\
        [0.71026, 0.70755, 0.70487, 0.70236, 0.70044, 0.70033, 0.70371, 0.71034, 0.71906, 0.72877, 0.73868,\
        0.74823, 0.75701, 0.76472, 0.77123, 0.77662, 0.78118, 0.78519, 0.78885, 0.79221, 0.79522, 0.79829,\
        0.80268, 0.80876, 0.81498, 0.82115, 0.82714, 0.83290, 0.83845, 0.84383, 0.84905, 0.85412, 0.85903,\
        0.86369, 0.86797, 0.87173, 0.87506, 0.87826, 0.88145, 0.88470, 0.88865, 0.89411, 0.89860, 0.90313,\
        0.90780, 0.90917, 0.90751, 0.90514, 0.90292, 0.89966, 0.89822, 0.90103, 0.90516, 0.90819, 0.90966,\
        0.90896, 0.90565, 0.90052, 0.89464, 0.88835, 0.88198, 0.87578, 0.86972, 0.86367, 0.85751, 0.85124,\
        0.84493, 0.83877, 0.83304, 0.82798, 0.82375, 0.82028, 0.81734, 0.81458, 0.81166, 0.80821, 0.80376,\
        0.79837, 0.79252, 0.78627, 0.77957, 0.77244, 0.76492, 0.75707, 0.74895, 0.74058, 0.73201, 0.72323,\
        0.71422, 0.70489, 0.69505, 0.68487, 0.67462, 0.66435, 0.65408, 0.64386, 0.63375, 0.62379, 0.61405,\
        0.60474, 0.59643, 0.58896, 0.58171, 0.57432, 0.56661, 0.55852, 0.55011, 0.54151, 0.53293, 0.52462,\
        0.51686, 0.51000, 0.50442, 0.50059, 0.49875, 0.49873, 0.49972, 0.50083, 0.50193, 0.50305, 0.50415],\
        [0.70853, 0.70552, 0.70261, 0.69982, 0.69747, 0.69695, 0.69955, 0.70589, 0.71450, 0.72390, 0.73335,\
        0.74239, 0.75070, 0.75812, 0.76461, 0.77027, 0.77526, 0.77979, 0.78399, 0.78794, 0.79172, 0.79577,\
        0.80074, 0.80590, 0.81146, 0.81699, 0.82226, 0.82725, 0.83204, 0.83670, 0.84125, 0.84570, 0.85003,\
        0.85417, 0.85804, 0.86159, 0.86490, 0.86814, 0.87145, 0.87507, 0.87934, 0.88425, 0.88909, 0.89437,\
        0.89944, 0.90045, 0.89894, 0.89660, 0.89540, 0.89312, 0.89290, 0.89570, 0.89966, 0.90300, 0.90511,\
        0.90522, 0.90188, 0.89612, 0.88973, 0.88302, 0.87630, 0.86972, 0.86328, 0.85690, 0.85047, 0.84396,\
        0.83743, 0.83108, 0.82522, 0.82023, 0.81636, 0.81355, 0.81145, 0.80957, 0.80759, 0.80529, 0.80174,\
        0.79650, 0.79042, 0.78389, 0.77699, 0.76975, 0.76218, 0.75431, 0.74617, 0.73780, 0.72920, 0.72043,\
        0.71150, 0.70221, 0.69224, 0.68183, 0.67130, 0.66076, 0.65024, 0.63982, 0.62954, 0.61947, 0.60960,\
        0.60011, 0.59160, 0.58380, 0.57627, 0.56871, 0.56096, 0.55297, 0.54476, 0.53647, 0.52829, 0.52045,\
        0.51328, 0.50707, 0.50217, 0.49903, 0.49775, 0.49793, 0.49902, 0.50043, 0.50184, 0.50328, 0.50468],\
        [0.70681, 0.70344, 0.70028, 0.69746, 0.69537, 0.69511, 0.69763, 0.70339, 0.71127, 0.71997, 0.72869,\
        0.73701, 0.74471, 0.75173, 0.75810, 0.76391, 0.76930, 0.77439, 0.77930, 0.78409, 0.78885, 0.79373,\
        0.79865, 0.80363, 0.80881, 0.81362, 0.81800, 0.82208, 0.82602, 0.82990, 0.83378, 0.83764, 0.84147,\
        0.84519, 0.84878, 0.85221, 0.85554, 0.85888, 0.86239, 0.86625, 0.87060, 0.87539, 0.88042, 0.88552,\
        0.88965, 0.89121, 0.89064, 0.88930, 0.88823, 0.88745, 0.88807, 0.89056, 0.89408, 0.89745, 0.89989,\
        0.90127, 0.89777, 0.89141, 0.88439, 0.87729, 0.87032, 0.86355, 0.85696, 0.85044, 0.84388, 0.83724,\
        0.83054, 0.82394, 0.81780, 0.81270, 0.80910, 0.80691, 0.80559, 0.80443, 0.80295, 0.80167, 0.79916,\
        0.79386, 0.78764, 0.78096, 0.77396, 0.76665, 0.75907, 0.75120, 0.74306, 0.73466, 0.72601, 0.71713,\
        0.70810, 0.69884, 0.68882, 0.67840, 0.66779, 0.65712, 0.64649, 0.63596, 0.62563, 0.61563, 0.60605,\
        0.59665, 0.58775, 0.57950, 0.57162, 0.56384, 0.55601, 0.54805, 0.53999, 0.53194, 0.52407, 0.51663,\
        0.50994, 0.50434, 0.50008, 0.49745, 0.49649, 0.49684, 0.49807, 0.49972, 0.50152, 0.50337, 0.50511],\
        [0.70494, 0.70110, 0.69763, 0.69485, 0.69322, 0.69351, 0.69635, 0.70172, 0.70876, 0.71662, 0.72444,\
        0.73185, 0.73880, 0.74532, 0.75148, 0.75738, 0.76311, 0.76880, 0.77449, 0.78024, 0.78601, 0.79169,\
        0.79704, 0.80219, 0.80690, 0.81078, 0.81408, 0.81715, 0.82020, 0.82333, 0.82657, 0.82991, 0.83330,\
        0.83671, 0.84009, 0.84345, 0.84681, 0.85025, 0.85391, 0.85787, 0.86221, 0.86687, 0.87165, 0.87619,\
        0.87975, 0.88165, 0.88203, 0.88167, 0.88130, 0.88141, 0.88250, 0.88476, 0.88773, 0.89069, 0.89300,\
        0.89366, 0.89049, 0.88455, 0.87766, 0.87061, 0.86371, 0.85706, 0.85059, 0.84420, 0.83775, 0.83117,\
        0.82445, 0.81770, 0.81120, 0.80575, 0.80235, 0.80063, 0.79981, 0.79912, 0.79779, 0.79649, 0.79410,\
        0.78951, 0.78364, 0.77714, 0.77023, 0.76300, 0.75547, 0.74766, 0.73955, 0.73116, 0.72246, 0.71341,\
        0.70406, 0.69467, 0.68488, 0.67461, 0.66406, 0.65341, 0.64275, 0.63219, 0.62182, 0.61181, 0.60237,\
        0.59320, 0.58420, 0.57569, 0.56754, 0.55956, 0.55163, 0.54368, 0.53572, 0.52783, 0.52019, 0.51302,\
        0.50667, 0.50154, 0.49784, 0.49562, 0.49491, 0.49544, 0.49687, 0.49882, 0.50104, 0.50333, 0.50544],\
        [0.70275, 0.69837, 0.69451, 0.69163, 0.69027, 0.69097, 0.69402, 0.69919, 0.70602, 0.71325, 0.72007,\
        0.72649, 0.73262, 0.73860, 0.74450, 0.75043, 0.75649, 0.76273, 0.76922, 0.77593, 0.78278, 0.78957,\
        0.79597, 0.80143, 0.80536, 0.80802, 0.81011, 0.81216, 0.81437, 0.81684, 0.81954, 0.82243, 0.82547,\
        0.82864, 0.83189, 0.83519, 0.83857, 0.84207, 0.84578, 0.84975, 0.85399, 0.85841, 0.86282, 0.86688,\
        0.87012, 0.87223, 0.87328, 0.87370, 0.87402, 0.87468, 0.87597, 0.87795, 0.88031, 0.88254, 0.88394,\
        0.88360, 0.88066, 0.87552, 0.86927, 0.86272, 0.85628, 0.85009, 0.84408, 0.83812, 0.83205, 0.82576,\
        0.81928, 0.81270, 0.80617, 0.80021, 0.79698, 0.79525, 0.79427, 0.79340, 0.79217, 0.79051, 0.78788,\
        0.78379, 0.77848, 0.77237, 0.76574, 0.75871, 0.75134, 0.74364, 0.73563, 0.72731, 0.71865, 0.70960,\
        0.70015, 0.69059, 0.68080, 0.67063, 0.66019, 0.64960, 0.63901, 0.62849, 0.61812, 0.60795, 0.59834,\
        0.58939, 0.58062, 0.57208, 0.56379, 0.55568, 0.54767, 0.53971, 0.53181, 0.52404, 0.51654, 0.50954,\
        0.50336, 0.49850, 0.49518, 0.49331, 0.49286, 0.49366, 0.49540, 0.49776, 0.50044, 0.50313, 0.50562],\
        [0.69995, 0.69476, 0.69026, 0.68711, 0.68581, 0.68675, 0.69008, 0.69559, 0.70260, 0.70908, 0.71486,\
        0.72032, 0.72571, 0.73120, 0.73688, 0.74285, 0.74917, 0.75592, 0.76313, 0.77073, 0.77860, 0.78648,\
        0.79396, 0.80002, 0.80327, 0.80456, 0.80558, 0.80680, 0.80837, 0.81033, 0.81262, 0.81515, 0.81791,\
        0.82092, 0.82410, 0.82735, 0.83071, 0.83421, 0.83791, 0.84181, 0.84587, 0.85001, 0.85405, 0.85775,\
        0.86080, 0.86303, 0.86449, 0.86544, 0.86624, 0.86719, 0.86849, 0.87013, 0.87186, 0.87322, 0.87368,\
        0.87260, 0.86965, 0.86507, 0.85954, 0.85372, 0.84803, 0.84261, 0.83738, 0.83217, 0.82674, 0.82096,\
        0.81490, 0.80880, 0.80307, 0.79754, 0.79334, 0.79077, 0.78907, 0.78766, 0.78609, 0.78406, 0.78124,\
        0.77737, 0.77248, 0.76680, 0.76052, 0.75378, 0.74664, 0.73913, 0.73126, 0.72306, 0.71457, 0.70570,\
        0.69634, 0.68663, 0.67672, 0.66657, 0.65620, 0.64571, 0.63522, 0.62485, 0.61467, 0.60464, 0.59498,\
        0.58593, 0.57716, 0.56859, 0.56023, 0.55203, 0.54396, 0.53599, 0.52812, 0.52042, 0.51300, 0.50609,\
        0.50003, 0.49528, 0.49208, 0.49041, 0.49021, 0.49131, 0.49345, 0.49632, 0.49954, 0.50267, 0.50554],\
        [0.69425, 0.68892, 0.68405, 0.68065, 0.67928, 0.68031, 0.68382, 0.68955, 0.69648, 0.70274, 0.70785,\
        0.71274, 0.71768, 0.72287, 0.72844, 0.73445, 0.74100, 0.74817, 0.75598, 0.76431, 0.77294, 0.78156,\
        0.78972, 0.79636, 0.79883, 0.79965, 0.80017, 0.80097, 0.80213, 0.80377, 0.80583, 0.80804, 0.81056,\
        0.81352, 0.81668, 0.81987, 0.82314, 0.82660, 0.83025, 0.83402, 0.83787, 0.84172, 0.84544, 0.84885,\
        0.85176, 0.85404, 0.85571, 0.85694, 0.85798, 0.85904, 0.86024, 0.86151, 0.86263, 0.86323, 0.86288,\
        0.86121, 0.85812, 0.85384, 0.84889, 0.84384, 0.83905, 0.83465, 0.83049, 0.82630, 0.82175, 0.81662,\
        0.81097, 0.80515, 0.79970, 0.79455, 0.78996, 0.78652, 0.78397, 0.78186, 0.77976, 0.77735, 0.77434,\
        0.77054, 0.76593, 0.76060, 0.75469, 0.74828, 0.74144, 0.73415, 0.72643, 0.71834, 0.71001, 0.70135,\
        0.69207, 0.68239, 0.67248, 0.66237, 0.65209, 0.64170, 0.63131, 0.62110, 0.61126, 0.60170, 0.59204,\
        0.58272, 0.57379, 0.56514, 0.55671, 0.54845, 0.54033, 0.53232, 0.52444, 0.51676, 0.50937, 0.50244,\
        0.49636, 0.49156, 0.48833, 0.48675, 0.48676, 0.48822, 0.49083, 0.49424, 0.49802, 0.50165, 0.50510],\
        [0.68580, 0.68054, 0.67560, 0.67201, 0.67046, 0.67137, 0.67476, 0.68021, 0.68675, 0.69303, 0.69846,\
        0.70345, 0.70835, 0.71351, 0.71913, 0.72523, 0.73193, 0.73942, 0.74771, 0.75656, 0.76563, 0.77447,\
        0.78246, 0.78855, 0.79182, 0.79323, 0.79400, 0.79477, 0.79580, 0.79729, 0.79917, 0.80109, 0.80342,\
        0.80647, 0.80966, 0.81268, 0.81580, 0.81921, 0.82279, 0.82641, 0.83001, 0.83357, 0.83701, 0.84020,\
        0.84299, 0.84524, 0.84696, 0.84827, 0.84935, 0.85038, 0.85141, 0.85236, 0.85299, 0.85298, 0.85199,\
        0.84982, 0.84650, 0.84231, 0.83775, 0.83337, 0.82954, 0.82631, 0.82343, 0.82051, 0.81704, 0.81265,\
        0.80735, 0.80147, 0.79593, 0.79069, 0.78590, 0.78188, 0.77861, 0.77583, 0.77320, 0.77041, 0.76721,\
        0.76342, 0.75899, 0.75394, 0.74834, 0.74228, 0.73576, 0.72877, 0.72122, 0.71312, 0.70478, 0.69628,\
        0.68720, 0.67770, 0.66793, 0.65796, 0.64783, 0.63757, 0.62724, 0.61702, 0.60725, 0.59804, 0.58858,\
        0.57928, 0.57029, 0.56158, 0.55309, 0.54478, 0.53660, 0.52851, 0.52055, 0.51281, 0.50537, 0.49828,\
        0.49197, 0.48698, 0.48364, 0.48208, 0.48231, 0.48418, 0.48738, 0.49145, 0.49581, 0.50004, 0.50445],\
        [0.67602, 0.67048, 0.66530, 0.66140, 0.65953, 0.66015, 0.66328, 0.66845, 0.67474, 0.68116, 0.68714,\
        0.69265, 0.69793, 0.70332, 0.70918, 0.71535, 0.72205, 0.72977, 0.73848, 0.74773, 0.75700, 0.76579,\
        0.77349, 0.77945, 0.78334, 0.78559, 0.78697, 0.78809, 0.78928, 0.79076, 0.79249, 0.79440, 0.79676,\
        0.79986, 0.80301, 0.80569, 0.80859, 0.81205, 0.81561, 0.81903, 0.82233, 0.82557, 0.82877, 0.83181,\
        0.83450, 0.83668, 0.83832, 0.83952, 0.84047, 0.84135, 0.84221, 0.84292, 0.84323, 0.84281, 0.84137,\
        0.83878, 0.83515, 0.83088, 0.82650, 0.82263, 0.81971, 0.81771, 0.81622, 0.81470, 0.81252, 0.80895,\
        0.80376, 0.79818, 0.79204, 0.78628, 0.78105, 0.77654, 0.77273, 0.76942, 0.76635, 0.76325, 0.75987,\
        0.75606, 0.75174, 0.74689, 0.74156, 0.73580, 0.72965, 0.72305, 0.71582, 0.70770, 0.69912, 0.69073,\
        0.68188, 0.67260, 0.66304, 0.65329, 0.64339, 0.63335, 0.62315, 0.61281, 0.60264, 0.59350, 0.58452,\
        0.57547, 0.56652, 0.55777, 0.54923, 0.54087, 0.53262, 0.52442, 0.51626, 0.50825, 0.50057, 0.49318,\
        0.48655, 0.48130, 0.47777, 0.47619, 0.47663, 0.47899, 0.48296, 0.48804, 0.49354, 0.49894, 0.50416],\
        [0.66569, 0.65959, 0.65388, 0.64941, 0.64701, 0.64725, 0.65020, 0.65531, 0.66169, 0.66845, 0.67499,\
        0.68114, 0.68700, 0.69284, 0.69895, 0.70512, 0.71156, 0.71954, 0.72874, 0.73829, 0.74761, 0.75627,\
        0.76379, 0.76981, 0.77417, 0.77715, 0.77921, 0.78084, 0.78236, 0.78399, 0.78581, 0.78788, 0.79038,\
        0.79332, 0.79627, 0.79875, 0.80157, 0.80528, 0.80876, 0.81187, 0.81481, 0.81773, 0.82072, 0.82368,\
        0.82633, 0.82842, 0.82985, 0.83075, 0.83142, 0.83208, 0.83279, 0.83340, 0.83360, 0.83300, 0.83131,\
        0.82839, 0.82445, 0.81998, 0.81560, 0.81202, 0.80989, 0.80909, 0.80884, 0.80863, 0.80783, 0.80532,\
        0.80042, 0.79387, 0.78727, 0.78094, 0.77525, 0.77034, 0.76615, 0.76250, 0.75915, 0.75583, 0.75233,\
        0.74850, 0.74425, 0.73954, 0.73438, 0.72885, 0.72303, 0.71689, 0.71026, 0.70269, 0.69395, 0.68520,\
        0.67628, 0.66711, 0.65775, 0.64828, 0.63871, 0.62904, 0.61920, 0.60914, 0.59902, 0.58958, 0.58049,\
        0.57142, 0.56244, 0.55362, 0.54500, 0.53657, 0.52826, 0.51993, 0.51146, 0.50295, 0.49473, 0.48694,\
        0.47998, 0.47439, 0.47059, 0.46890, 0.46949, 0.47236, 0.47726, 0.48370, 0.49094, 0.49796, 0.50404],\
        [0.65544, 0.64864, 0.64218, 0.63687, 0.63370, 0.63349, 0.63643, 0.64181, 0.64857, 0.65580, 0.66291,\
        0.66967, 0.67612, 0.68240, 0.68864, 0.69471, 0.70097, 0.70952, 0.71920, 0.72881, 0.73801, 0.74648,\
        0.75387, 0.75995, 0.76468, 0.76824, 0.77092, 0.77308, 0.77501, 0.77692, 0.77895, 0.78121, 0.78378,\
        0.78660, 0.78944, 0.79218, 0.79528, 0.79892, 0.80212, 0.80490, 0.80745, 0.81003, 0.81286, 0.81587,\
        0.81858, 0.82055, 0.82164, 0.82205, 0.82226, 0.82263, 0.82326, 0.82394, 0.82424, 0.82375, 0.82207,\
        0.81898, 0.81474, 0.81013, 0.80574, 0.80229, 0.80074, 0.80091, 0.80137, 0.80186, 0.80217, 0.79994,\
        0.79523, 0.78824, 0.78100, 0.77424, 0.76826, 0.76314, 0.75880, 0.75503, 0.75157, 0.74815, 0.74460,\
        0.74077, 0.73659, 0.73197, 0.72689, 0.72145, 0.71580, 0.71001, 0.70385, 0.69674, 0.68824, 0.67929,\
        0.67027, 0.66117, 0.65200, 0.64281, 0.63364, 0.62444, 0.61515, 0.60572, 0.59590, 0.58603, 0.57649,\
        0.56717, 0.55800, 0.54902, 0.54026, 0.53174, 0.52338, 0.51497, 0.50622, 0.49710, 0.48820, 0.47983,\
        0.47238, 0.46631, 0.46210, 0.46014, 0.46075, 0.46401, 0.46979, 0.47762, 0.48679, 0.49623, 0.50316],\
        [0.64585, 0.63837, 0.63110, 0.62480, 0.62057, 0.61984, 0.62305, 0.62901, 0.63630, 0.64398, 0.65155,\
        0.65881, 0.66572, 0.67237, 0.67884, 0.68525, 0.69224, 0.70099, 0.71044, 0.71964, 0.72853, 0.73676,\
        0.74398, 0.75008, 0.75506, 0.75906, 0.76226, 0.76490, 0.76724, 0.76948, 0.77177, 0.77424, 0.77692,\
        0.77978, 0.78273, 0.78575, 0.78899, 0.79234, 0.79540, 0.79800, 0.80030, 0.80256, 0.80526, 0.80849,\
        0.81132, 0.81316, 0.81379, 0.81344, 0.81304, 0.81305, 0.81365, 0.81461, 0.81517, 0.81504, 0.81372,\
        0.81078, 0.80628, 0.80171, 0.79778, 0.79474, 0.79345, 0.79352, 0.79399, 0.79441, 0.79422, 0.79166,\
        0.78668, 0.78000, 0.77279, 0.76595, 0.75994, 0.75488, 0.75067, 0.74703, 0.74366, 0.74027, 0.73672,\
        0.73292, 0.72882, 0.72430, 0.71923, 0.71367, 0.70799, 0.70231, 0.69624, 0.68925, 0.68121, 0.67254,\
        0.66363, 0.65464, 0.64566, 0.63676, 0.62798, 0.61926, 0.61045, 0.60138, 0.59180, 0.58189, 0.57212,\
        0.56254, 0.55312, 0.54389, 0.53491, 0.52621, 0.51774, 0.50926, 0.50038, 0.49064, 0.48102, 0.47200,\
        0.46395, 0.45728, 0.45248, 0.45002, 0.45038, 0.45379, 0.46017, 0.46899, 0.47934, 0.48973, 0.49726],\
        [0.63740, 0.62944, 0.62162, 0.61449, 0.60899, 0.60759, 0.61143, 0.61806, 0.62569, 0.63358, 0.64138,\
        0.64890, 0.65612, 0.66309, 0.66995, 0.67696, 0.68460, 0.69323, 0.70205, 0.71071, 0.71934, 0.72730,\
        0.73425, 0.74025, 0.74539, 0.74972, 0.75333, 0.75639, 0.75910, 0.76165, 0.76421, 0.76690, 0.76976,\
        0.77279, 0.77593, 0.77917, 0.78247, 0.78571, 0.78867, 0.79124, 0.79351, 0.79571, 0.79835, 0.80166,\
        0.80449, 0.80609, 0.80641, 0.80490, 0.80390, 0.80351, 0.80404, 0.80549, 0.80614, 0.80662, 0.80598,\
        0.80355, 0.79966, 0.79495, 0.79143, 0.78878, 0.78732, 0.78682, 0.78666, 0.78625, 0.78486, 0.78164,\
        0.77646, 0.76989, 0.76284, 0.75613, 0.75034, 0.74563, 0.74185, 0.73864, 0.73556, 0.73231, 0.72878,\
        0.72500, 0.72101, 0.71667, 0.71167, 0.70585, 0.69991, 0.69420, 0.68805, 0.68112, 0.67337, 0.66501,\
        0.65631, 0.64749, 0.63868, 0.63002, 0.62159, 0.61332, 0.60487, 0.59595, 0.58657, 0.57688, 0.56711,\
        0.55739, 0.54775, 0.53822, 0.52888, 0.51987, 0.51118, 0.50241, 0.49309, 0.48313, 0.47313, 0.46362,\
        0.45501, 0.44767, 0.44210, 0.43888, 0.43865, 0.44189, 0.44854, 0.45786, 0.46857, 0.47894, 0.48709],\
        [0.63035, 0.62225, 0.61436, 0.60728, 0.60147, 0.59929, 0.60351, 0.60998, 0.61726, 0.62489, 0.63257,\
        0.64012, 0.64746, 0.65464, 0.66181, 0.66918, 0.67701, 0.68530, 0.69366, 0.70201, 0.71063, 0.71820,\
        0.72463, 0.73042, 0.73567, 0.74028, 0.74422, 0.74759, 0.75059, 0.75341, 0.75622, 0.75914, 0.76223,\
        0.76550, 0.76890, 0.77235, 0.77577, 0.77903, 0.78202, 0.78468, 0.78708, 0.78944, 0.79208, 0.79500,\
        0.79738, 0.79868, 0.79914, 0.79670, 0.79506, 0.79438, 0.79482, 0.79616, 0.79687, 0.79789, 0.79788,\
        0.79592, 0.79288, 0.78910, 0.78580, 0.78328, 0.78155, 0.78038, 0.77930, 0.77778, 0.77521, 0.77112,\
        0.76548, 0.75875, 0.75168, 0.74510, 0.73964, 0.73554, 0.73253, 0.73006, 0.72751, 0.72447, 0.72089,\
        0.71703, 0.71314, 0.70908, 0.70439, 0.69849, 0.69225, 0.68616, 0.67974, 0.67274, 0.66510, 0.65695,\
        0.64845, 0.63978, 0.63110, 0.62257, 0.61436, 0.60649, 0.59848, 0.58973, 0.58057, 0.57111, 0.56147,\
        0.55174, 0.54195, 0.53210, 0.52228, 0.51275, 0.50372, 0.49461, 0.48495, 0.47489, 0.46477, 0.45501,\
        0.44597, 0.43802, 0.43161, 0.42735, 0.42615, 0.42891, 0.43568, 0.44532, 0.45621, 0.46671, 0.47549],\
        [0.62473, 0.61675, 0.60916, 0.60273, 0.59826, 0.59670, 0.59956, 0.60453, 0.61084, 0.61777, 0.62506,\
        0.63240, 0.63965, 0.64686, 0.65415, 0.66166, 0.66948, 0.67758, 0.68581, 0.69420, 0.70254, 0.70926,\
        0.71486, 0.72048, 0.72593, 0.73079, 0.73495, 0.73852, 0.74172, 0.74473, 0.74775, 0.75091, 0.75428,\
        0.75785, 0.76155, 0.76526, 0.76888, 0.77228, 0.77538, 0.77816, 0.78069, 0.78310, 0.78551, 0.78780,\
        0.78952, 0.79037, 0.78999, 0.78808, 0.78618, 0.78519, 0.78528, 0.78615, 0.78719, 0.78825, 0.78856,\
        0.78752, 0.78544, 0.78275, 0.78005, 0.77770, 0.77573, 0.77393, 0.77196, 0.76937, 0.76571, 0.76071,\
        0.75441, 0.74722, 0.73989, 0.73326, 0.72815, 0.72487, 0.72296, 0.72157, 0.71981, 0.71706, 0.71327,\
        0.70910, 0.70520, 0.70130, 0.69677, 0.69103, 0.68467, 0.67817, 0.67141, 0.66423, 0.65659, 0.64855,\
        0.64021, 0.63168, 0.62308, 0.61460, 0.60643, 0.59875, 0.59105, 0.58274, 0.57392, 0.56474, 0.55531,\
        0.54568, 0.53585, 0.52579, 0.51550, 0.50521, 0.49570, 0.48635, 0.47659, 0.46652, 0.45641, 0.44658,\
        0.43734, 0.42899, 0.42188, 0.41651, 0.41392, 0.41594, 0.42292, 0.43291, 0.44396, 0.45461, 0.46385],\
        [0.62033, 0.61262, 0.60540, 0.59946, 0.59567, 0.59459, 0.59649, 0.60075, 0.60556, 0.61172, 0.61854,\
        0.62550, 0.63249, 0.63954, 0.64676, 0.65425, 0.66199, 0.66994, 0.67801, 0.68613, 0.69382, 0.69967,\
        0.70463, 0.71047, 0.71625, 0.72129, 0.72551, 0.72914, 0.73243, 0.73557, 0.73876, 0.74215, 0.74582,\
        0.74975, 0.75383, 0.75788, 0.76176, 0.76535, 0.76859, 0.77148, 0.77405, 0.77636, 0.77843, 0.78009,\
        0.78110, 0.78122, 0.78026, 0.77841, 0.77649, 0.77524, 0.77498, 0.77560, 0.77673, 0.77797, 0.77874,\
        0.77861, 0.77756, 0.77587, 0.77385, 0.77176, 0.76965, 0.76737, 0.76466, 0.76118, 0.75661, 0.75076,\
        0.74372, 0.73591, 0.72808, 0.72116, 0.71623, 0.71395, 0.71344, 0.71338, 0.71271, 0.71045, 0.70629,\
        0.70150, 0.69748, 0.69344, 0.68872, 0.68308, 0.67674, 0.67001, 0.66297, 0.65560, 0.64790, 0.63994,\
        0.63176, 0.62339, 0.61488, 0.60648, 0.59844, 0.59085, 0.58331, 0.57532, 0.56683, 0.55794, 0.54873,\
        0.53928, 0.52957, 0.51954, 0.50908, 0.49838, 0.48823, 0.47842, 0.46848, 0.45843, 0.44842, 0.43873,\
        0.42958, 0.42122, 0.41388, 0.40786, 0.40377, 0.40479, 0.41217, 0.42226, 0.43318, 0.44371, 0.45307],\
        [0.61682, 0.60943, 0.60249, 0.59688, 0.59327, 0.59204, 0.59326, 0.59628, 0.60037, 0.60630, 0.61261,\
        0.61910, 0.62567, 0.63241, 0.63945, 0.64683, 0.65444, 0.66215, 0.66985, 0.67732, 0.68410, 0.68970,\
        0.69486, 0.70080, 0.70672, 0.71171, 0.71578, 0.71935, 0.72266, 0.72589, 0.72919, 0.73279, 0.73680,\
        0.74118, 0.74571, 0.75017, 0.75435, 0.75814, 0.76151, 0.76445, 0.76699, 0.76914, 0.77084, 0.77194,\
        0.77226, 0.77167, 0.77016, 0.76803, 0.76588, 0.76435, 0.76381, 0.76433, 0.76563, 0.76727, 0.76869,\
        0.76944, 0.76937, 0.76856, 0.76717, 0.76538, 0.76322, 0.76062, 0.75739, 0.75327, 0.74802, 0.74150,\
        0.73377, 0.72529, 0.71693, 0.70964, 0.70451, 0.70339, 0.70421, 0.70541, 0.70605, 0.70489, 0.70029,\
        0.69518, 0.69050, 0.68586, 0.68075, 0.67496, 0.66853, 0.66163, 0.65436, 0.64680, 0.63903, 0.63113,\
        0.62316, 0.61507, 0.60675, 0.59848, 0.59071, 0.58319, 0.57567, 0.56782, 0.55953, 0.55083, 0.54181,\
        0.53254, 0.52304, 0.51322, 0.50296, 0.49209, 0.48138, 0.47103, 0.46088, 0.45085, 0.44106, 0.43169,\
        0.42295, 0.41504, 0.40816, 0.40253, 0.39869, 0.39906, 0.40537, 0.41462, 0.42471, 0.43462, 0.44368],\
        [0.61374, 0.60664, 0.60001, 0.59430, 0.59042, 0.58872, 0.58920, 0.59156, 0.59563, 0.60101, 0.60670,\
        0.61279, 0.61883, 0.62516, 0.63203, 0.63932, 0.64678, 0.65420, 0.66143, 0.66827, 0.67451, 0.68009,\
        0.68550, 0.69121, 0.69692, 0.70163, 0.70548, 0.70899, 0.71234, 0.71564, 0.71900, 0.72278, 0.72718,\
        0.73209, 0.73719, 0.74210, 0.74659, 0.75055, 0.75399, 0.75694, 0.75942, 0.76139, 0.76276, 0.76339,\
        0.76313, 0.76189, 0.75977, 0.75712, 0.75451, 0.75256, 0.75178, 0.75232, 0.75400, 0.75628, 0.75851,\
        0.76017, 0.76097, 0.76090, 0.76004, 0.75853, 0.75639, 0.75361, 0.75005, 0.74554, 0.73990, 0.73299,\
        0.72482, 0.71574, 0.70689, 0.69992, 0.69500, 0.69403, 0.69525, 0.69707, 0.69858, 0.69899, 0.69461,\
        0.68901, 0.68366, 0.67838, 0.67282, 0.66675, 0.66014, 0.65305, 0.64557, 0.63783, 0.62995, 0.62207,\
        0.61435, 0.60672, 0.59877, 0.59077, 0.58312, 0.57566, 0.56815, 0.56036, 0.55215, 0.54352, 0.53456,\
        0.52541, 0.51609, 0.50641, 0.49616, 0.48533, 0.47447, 0.46393, 0.45372, 0.44384, 0.43438, 0.42550,\
        0.41740, 0.41030, 0.40437, 0.39980, 0.39710, 0.39787, 0.40268, 0.41006, 0.41865, 0.42749, 0.43587],\
        [0.61062, 0.60335, 0.59678, 0.59091, 0.58663, 0.58438, 0.58425, 0.58611, 0.58974, 0.59460, 0.60029,\
        0.60619, 0.61154, 0.61747, 0.62436, 0.63172, 0.63906, 0.64616, 0.65292, 0.65924, 0.66506, 0.67048,\
        0.67574, 0.68103, 0.68614, 0.69061, 0.69444, 0.69797, 0.70142, 0.70483, 0.70818, 0.71210, 0.71698,\
        0.72254, 0.72828, 0.73367, 0.73840, 0.74245, 0.74589, 0.74880, 0.75120, 0.75304, 0.75420, 0.75449,\
        0.75376, 0.75194, 0.74917, 0.74582, 0.74253, 0.74002, 0.73895, 0.73965, 0.74194, 0.74514, 0.74835,\
        0.75089, 0.75243, 0.75293, 0.75246, 0.75116, 0.74906, 0.74617, 0.74242, 0.73775, 0.73201, 0.72503,\
        0.71680, 0.70764, 0.69828, 0.69147, 0.68709, 0.68556, 0.68622, 0.68782, 0.68928, 0.68925, 0.68624,\
        0.68139, 0.67606, 0.67053, 0.66469, 0.65839, 0.65160, 0.64431, 0.63662, 0.62868, 0.62066, 0.61273,\
        0.60513, 0.59790, 0.59046, 0.58288, 0.57546, 0.56815, 0.56073, 0.55300, 0.54480, 0.53610, 0.52703,\
        0.51781, 0.50858, 0.49895, 0.48864, 0.47798, 0.46731, 0.45690, 0.44688, 0.43732, 0.42832, 0.42003,\
        0.41268, 0.40653, 0.40176, 0.39826, 0.39668, 0.39790, 0.40173, 0.40740, 0.41445, 0.42205, 0.42954],\
        [0.60739, 0.59948, 0.59243, 0.58629, 0.58157, 0.57878, 0.57809, 0.57945, 0.58261, 0.58720, 0.59280,\
        0.59839, 0.60325, 0.60919, 0.61659, 0.62416, 0.63137, 0.63813, 0.64443, 0.65026, 0.65564, 0.66070,\
        0.66557, 0.67033, 0.67487, 0.67902, 0.68279, 0.68634, 0.68985, 0.69336, 0.69682, 0.70098, 0.70637,\
        0.71259, 0.71906, 0.72484, 0.72966, 0.73365, 0.73701, 0.73985, 0.74220, 0.74399, 0.74506, 0.74518,\
        0.74414, 0.74186, 0.73844, 0.73430, 0.73017, 0.72694, 0.72552, 0.72647, 0.72967, 0.73409, 0.73838,\
        0.74171, 0.74378, 0.74461, 0.74435, 0.74315, 0.74107, 0.73809, 0.73419, 0.72946, 0.72385, 0.71706,\
        0.70891, 0.70004, 0.69099, 0.68366, 0.67898, 0.67686, 0.67674, 0.67767, 0.67857, 0.67828, 0.67608,\
        0.67223, 0.66743, 0.66205, 0.65618, 0.64978, 0.64287, 0.63543, 0.62753, 0.61937, 0.61124, 0.60326,\
        0.59569, 0.58867, 0.58168, 0.57464, 0.56762, 0.56058, 0.55336, 0.54575, 0.53758, 0.52875, 0.51935,\
        0.50977, 0.50040, 0.49080, 0.48065, 0.47028, 0.45996, 0.44992, 0.44032, 0.43125, 0.42282, 0.41519,\
        0.40857, 0.40325, 0.39946, 0.39702, 0.39625, 0.39755, 0.40078, 0.40552, 0.41140, 0.41788, 0.42445],\
        [0.60417, 0.59556, 0.58746, 0.58048, 0.57511, 0.57175, 0.57056, 0.57148, 0.57425, 0.57851, 0.58372,\
        0.58914, 0.59445, 0.60103, 0.60920, 0.61679, 0.62373, 0.63012, 0.63597, 0.64130, 0.64620, 0.65078,\
        0.65514, 0.65936, 0.66339, 0.66720, 0.67081, 0.67432, 0.67784, 0.68146, 0.68534, 0.68995, 0.69562,\
        0.70237, 0.70952, 0.71544, 0.72010, 0.72391, 0.72713, 0.72989, 0.73224, 0.73408, 0.73521, 0.73533,\
        0.73419, 0.73160, 0.72763, 0.72271, 0.71769, 0.71370, 0.71189, 0.71313, 0.71758, 0.72352, 0.72881,\
        0.73264, 0.73492, 0.73579, 0.73552, 0.73432, 0.73223, 0.72913, 0.72499, 0.72008, 0.71471, 0.70831,\
        0.70001, 0.69092, 0.68218, 0.67493, 0.66996, 0.66732, 0.66656, 0.66685, 0.66727, 0.66691, 0.66522,\
        0.66216, 0.65801, 0.65299, 0.64725, 0.64087, 0.63392, 0.62643, 0.61833, 0.60990, 0.60170, 0.59380,\
        0.58635, 0.57941, 0.57274, 0.56616, 0.55960, 0.55295, 0.54603, 0.53864, 0.53059, 0.52170, 0.51194,\
        0.50170, 0.49206, 0.48239, 0.47246, 0.46243, 0.45256, 0.44304, 0.43401, 0.42558, 0.41785, 0.41093,\
        0.40503, 0.40044, 0.39743, 0.39586, 0.39571, 0.39710, 0.39994, 0.40407, 0.40909, 0.41463, 0.42034],\
        [0.60008, 0.59048, 0.58115, 0.57319, 0.56712, 0.56325, 0.56168, 0.56228, 0.56480, 0.56884, 0.57395,\
        0.57971, 0.58613, 0.59381, 0.60207, 0.60930, 0.61596, 0.62207, 0.62751, 0.63238, 0.63678, 0.64085,\
        0.64468, 0.64835, 0.65191, 0.65537, 0.65877, 0.66218, 0.66570, 0.66947, 0.67370, 0.67869, 0.68470,\
        0.69175, 0.69914, 0.70480, 0.70924, 0.71289, 0.71601, 0.71874, 0.72113, 0.72311, 0.72441, 0.72470,\
        0.72363, 0.72095, 0.71661, 0.71107, 0.70539, 0.70095, 0.69904, 0.70054, 0.70651, 0.71390, 0.71961,\
        0.72345, 0.72559, 0.72622, 0.72567, 0.72437, 0.72231, 0.71910, 0.71456, 0.70905, 0.70358, 0.69764,\
        0.68918, 0.68012, 0.67165, 0.66465, 0.65966, 0.65679, 0.65565, 0.65558, 0.65577, 0.65546, 0.65418,\
        0.65173, 0.64815, 0.64355, 0.63799, 0.63162, 0.62466, 0.61727, 0.60915, 0.60026, 0.59196, 0.58413,\
        0.57682, 0.57005, 0.56369, 0.55755, 0.55146, 0.54525, 0.53871, 0.53164, 0.52382, 0.51508, 0.50524,\
        0.49464, 0.48431, 0.47426, 0.46434, 0.45461, 0.44523, 0.43632, 0.42800, 0.42033, 0.41341, 0.40730,\
        0.40217, 0.39835, 0.39602, 0.39501, 0.39526, 0.39677, 0.39943, 0.40303, 0.40732, 0.41205, 0.41697],\
        [0.59256, 0.58259, 0.57265, 0.56406, 0.55751, 0.55333, 0.55159, 0.55209, 0.55454, 0.55857, 0.56383,\
        0.57008, 0.57727, 0.58534, 0.59357, 0.60099, 0.60788, 0.61393, 0.61906, 0.62352, 0.62748, 0.63106,\
        0.63437, 0.63753, 0.64065, 0.64376, 0.64691, 0.65017, 0.65363, 0.65744, 0.66180, 0.66691, 0.67290,\
        0.67964, 0.68639, 0.69209, 0.69665, 0.70036, 0.70353, 0.70628, 0.70875, 0.71089, 0.71239, 0.71287,\
        0.71193, 0.70932, 0.70486, 0.69896, 0.69325, 0.68918, 0.68798, 0.69048, 0.69718, 0.70481, 0.70998,\
        0.71348, 0.71533, 0.71555, 0.71444, 0.71296, 0.71105, 0.70776, 0.70280, 0.69660, 0.69043, 0.68394,\
        0.67605, 0.66755, 0.65957, 0.65294, 0.64816, 0.64536, 0.64421, 0.64411, 0.64432, 0.64418, 0.64324,\
        0.64127, 0.63818, 0.63396, 0.62856, 0.62206, 0.61488, 0.60758, 0.59982, 0.59072, 0.58207, 0.57418,\
        0.56702, 0.56051, 0.55451, 0.54882, 0.54322, 0.53749, 0.53137, 0.52462, 0.51704, 0.50846, 0.49871,\
        0.48771, 0.47678, 0.46632, 0.45634, 0.44690, 0.43804, 0.42982, 0.42229, 0.41548, 0.40948, 0.40435,\
        0.40004, 0.39695, 0.39511, 0.39448, 0.39498, 0.39656, 0.39909, 0.40224, 0.40590, 0.40995, 0.41418],\
        [0.58153, 0.57186, 0.56194, 0.55316, 0.54644, 0.54224, 0.54060, 0.54125, 0.54382, 0.54799, 0.55348,\
        0.56009, 0.56767, 0.57591, 0.58426, 0.59218, 0.59959, 0.60565, 0.61059, 0.61480, 0.61847, 0.62164,\
        0.62445, 0.62713, 0.62983, 0.63262, 0.63550, 0.63853, 0.64182, 0.64551, 0.64973, 0.65463, 0.66022,\
        0.66631, 0.67237, 0.67785, 0.68252, 0.68644, 0.68979, 0.69266, 0.69518, 0.69733, 0.69882, 0.69926,\
        0.69827, 0.69549, 0.69108, 0.68548, 0.68043, 0.67724, 0.67692, 0.68007, 0.68631, 0.69313, 0.69825,\
        0.70173, 0.70346, 0.70353, 0.70167, 0.69989, 0.69808, 0.69445, 0.68924, 0.68290, 0.67626, 0.66930,\
        0.66172, 0.65385, 0.64644, 0.64025, 0.63584, 0.63339, 0.63258, 0.63275, 0.63320, 0.63330, 0.63264,\
        0.63100, 0.62828, 0.62442, 0.61918, 0.61241, 0.60456, 0.59686, 0.58889, 0.58018, 0.57163, 0.56384,\
        0.55691, 0.55076, 0.54521, 0.54003, 0.53495, 0.52969, 0.52397, 0.51748, 0.50997, 0.50120, 0.49111,\
        0.48005, 0.46891, 0.45827, 0.44836, 0.43927, 0.43101, 0.42357, 0.41692, 0.41102, 0.40592, 0.40188,\
        0.39858, 0.39599, 0.39452, 0.39416, 0.39482, 0.39638, 0.39870, 0.40152, 0.40470, 0.40817, 0.41181],\
        [0.56930, 0.55969, 0.54986, 0.54106, 0.53439, 0.53047, 0.52926, 0.53029, 0.53311, 0.53744, 0.54311,\
        0.54995, 0.55776, 0.56621, 0.57481, 0.58313, 0.59075, 0.59711, 0.60222, 0.60642, 0.61005, 0.61289,\
        0.61517, 0.61740, 0.61976, 0.62225, 0.62484, 0.62757, 0.63056, 0.63393, 0.63779, 0.64221, 0.64718,\
        0.65252, 0.65794, 0.66307, 0.66771, 0.67178, 0.67528, 0.67825, 0.68072, 0.68261, 0.68369, 0.68368,\
        0.68226, 0.67932, 0.67516, 0.67045, 0.66641, 0.66415, 0.66445, 0.66759, 0.67292, 0.67887, 0.68396,\
        0.68754, 0.68942, 0.68951, 0.68786, 0.68574, 0.68319, 0.67933, 0.67413, 0.66800, 0.66140, 0.65448,\
        0.64724, 0.63994, 0.63307, 0.62730, 0.62332, 0.62145, 0.62125, 0.62194, 0.62275, 0.62308, 0.62258,\
        0.62106, 0.61852, 0.61495, 0.61000, 0.60298, 0.59451, 0.58598, 0.57747, 0.56885, 0.56057, 0.55308,\
        0.54655, 0.54089, 0.53589, 0.53127, 0.52672, 0.52194, 0.51656, 0.51021, 0.50257, 0.49342, 0.48296,\
        0.47181, 0.46063, 0.45003, 0.44034, 0.43171, 0.42417, 0.41761, 0.41193, 0.40702, 0.40273, 0.39931,\
        0.39695, 0.39509, 0.39406, 0.39395, 0.39471, 0.39623, 0.39834, 0.40087, 0.40365, 0.40663, 0.40975],\
        [0.55710, 0.54744, 0.53761, 0.52872, 0.52212, 0.51875, 0.51833, 0.51991, 0.52296, 0.52735, 0.53306,\
        0.54002, 0.54802, 0.55670, 0.56559, 0.57421, 0.58209, 0.58883, 0.59433, 0.59878, 0.60270, 0.60510,\
        0.60683, 0.60870, 0.61080, 0.61306, 0.61529, 0.61761, 0.62019, 0.62311, 0.62643, 0.63020, 0.63440,\
        0.63897, 0.64372, 0.64846, 0.65297, 0.65707, 0.66065, 0.66362, 0.66590, 0.66738, 0.66789, 0.66725,\
        0.66535, 0.66227, 0.65844, 0.65454, 0.65144, 0.64994, 0.65061, 0.65355, 0.65821, 0.66352, 0.66837,\
        0.67200, 0.67402, 0.67430, 0.67305, 0.67080, 0.66773, 0.66363, 0.65855, 0.65275, 0.64655, 0.64008,\
        0.63342, 0.62675, 0.62045, 0.61504, 0.61139, 0.61032, 0.61093, 0.61218, 0.61337, 0.61380, 0.61325,\
        0.61160, 0.60887, 0.60529, 0.60051, 0.59329, 0.58439, 0.57519, 0.56610, 0.55734, 0.54923, 0.54212,\
        0.53612, 0.53110, 0.52677, 0.52276, 0.51874, 0.51436, 0.50928, 0.50298, 0.49497, 0.48542, 0.47470,\
        0.46340, 0.45219, 0.44168, 0.43231, 0.42428, 0.41757, 0.41199, 0.40734, 0.40352, 0.40043, 0.39757,\
        0.39559, 0.39430, 0.39369, 0.39381, 0.39465, 0.39611, 0.39806, 0.40032, 0.40272, 0.40527, 0.40795],\
        [0.54580, 0.53621, 0.52644, 0.51742, 0.51069, 0.50818, 0.50890, 0.51093, 0.51396, 0.51817, 0.52373,\
        0.53069, 0.53886, 0.54782, 0.55706, 0.56604, 0.57430, 0.58150, 0.58751, 0.59245, 0.59633, 0.59847,\
        0.59993, 0.60153, 0.60344, 0.60551, 0.60725, 0.60905, 0.61116, 0.61352, 0.61618, 0.61919, 0.62256,\
        0.62627, 0.63032, 0.63461, 0.63893, 0.64300, 0.64656, 0.64942, 0.65143, 0.65247, 0.65240, 0.65115,\
        0.64877, 0.64553, 0.64195, 0.63867, 0.63630, 0.63536, 0.63621, 0.63893, 0.64318, 0.64811, 0.65278,\
        0.65641, 0.65856, 0.65906, 0.65805, 0.65579, 0.65249, 0.64828, 0.64337, 0.63804, 0.63250, 0.62682,\
        0.62101, 0.61518, 0.60965, 0.60483, 0.60126, 0.60117, 0.60239, 0.60398, 0.60546, 0.60567, 0.60485,\
        0.60289, 0.59960, 0.59553, 0.59016, 0.58283, 0.57392, 0.56441, 0.55494, 0.54598, 0.53798, 0.53129,\
        0.52596, 0.52173, 0.51816, 0.51481, 0.51128, 0.50722, 0.50227, 0.49592, 0.48765, 0.47782, 0.46684,\
        0.45532, 0.44396, 0.43348, 0.42445, 0.41709, 0.41133, 0.40685, 0.40321, 0.40025, 0.39805, 0.39608,\
        0.39460, 0.39373, 0.39345, 0.39377, 0.39466, 0.39604, 0.39780, 0.39980, 0.40183, 0.40405, 0.40637],\
        [0.53601, 0.52683, 0.51756, 0.50889, 0.50192, 0.50055, 0.50221, 0.50406, 0.50664, 0.51030, 0.51546,\
        0.52233, 0.53069, 0.54001, 0.54965, 0.55907, 0.56778, 0.57547, 0.58197, 0.58721, 0.59108, 0.59353,\
        0.59516, 0.59659, 0.59827, 0.59997, 0.60105, 0.60226, 0.60396, 0.60569, 0.60761, 0.60984, 0.61231,\
        0.61508, 0.61834, 0.62214, 0.62623, 0.63021, 0.63366, 0.63634, 0.63806, 0.63868, 0.63811, 0.63635,\
        0.63353, 0.63008, 0.62665, 0.62387, 0.62211, 0.62153, 0.62236, 0.62485, 0.62885, 0.63362, 0.63820,\
        0.64182, 0.64405, 0.64470, 0.64382, 0.64160, 0.63824, 0.63402, 0.62934, 0.62461, 0.61999, 0.61540,\
        0.61075, 0.60600, 0.60144, 0.59782, 0.59526, 0.59508, 0.59620, 0.59774, 0.59909, 0.59874, 0.59741,\
        0.59501, 0.59126, 0.58644, 0.58035, 0.57270, 0.56373, 0.55405, 0.54432, 0.53517, 0.52722, 0.52099,\
        0.51649, 0.51323, 0.51053, 0.50787, 0.50481, 0.50098, 0.49604, 0.48956, 0.48124, 0.47124, 0.45999,\
        0.44811, 0.43640, 0.42577, 0.41699, 0.41036, 0.40563, 0.40235, 0.39982, 0.39765, 0.39605, 0.39481,\
        0.39391, 0.39344, 0.39343, 0.39389, 0.39479, 0.39607, 0.39760, 0.39924, 0.40107, 0.40304, 0.40502],\
        [0.52801, 0.51964, 0.51149, 0.50431, 0.49907, 0.49737, 0.49827, 0.49957, 0.50136, 0.50411, 0.50858,\
        0.51530, 0.52394, 0.53367, 0.54377, 0.55365, 0.56283, 0.57099, 0.57791, 0.58346, 0.58758, 0.59039,\
        0.59228, 0.59375, 0.59510, 0.59625, 0.59702, 0.59788, 0.59905, 0.60011, 0.60134, 0.60285, 0.60439,\
        0.60613, 0.60845, 0.61169, 0.61553, 0.61932, 0.62256, 0.62497, 0.62637, 0.62665, 0.62574, 0.62365,\
        0.62053, 0.61687, 0.61355, 0.61135, 0.61022, 0.60989, 0.61042, 0.61248, 0.61633, 0.62105, 0.62552,\
        0.62904, 0.63123, 0.63193, 0.63115, 0.62901, 0.62570, 0.62157, 0.61716, 0.61318, 0.60978, 0.60648,\
        0.60317, 0.59978, 0.59637, 0.59371, 0.59204, 0.59171, 0.59234, 0.59328, 0.59375, 0.59301, 0.59113,\
        0.58811, 0.58386, 0.57841, 0.57171, 0.56371, 0.55459, 0.54478, 0.53485, 0.52547, 0.51748, 0.51175,\
        0.50834, 0.50623, 0.50444, 0.50242, 0.49980, 0.49623, 0.49135, 0.48485, 0.47650, 0.46639, 0.45484,\
        0.44246, 0.43012, 0.41900, 0.41030, 0.40447, 0.40082, 0.39864, 0.39711, 0.39581, 0.39482, 0.39411,\
        0.39366, 0.39351, 0.39369, 0.39424, 0.39512, 0.39629, 0.39765, 0.39915, 0.40071, 0.40230, 0.40390],\
        [0.52176, 0.51443, 0.50763, 0.50211, 0.49840, 0.49678, 0.49686, 0.49753, 0.49845, 0.49999, 0.50342,\
        0.51009, 0.51909, 0.52924, 0.53973, 0.55000, 0.55960, 0.56819, 0.57551, 0.58141, 0.58584, 0.58893,\
        0.59101, 0.59247, 0.59358, 0.59442, 0.59507, 0.59572, 0.59646, 0.59716, 0.59795, 0.59882, 0.59960,\
        0.60035, 0.60154, 0.60407, 0.60750, 0.61087, 0.61369, 0.61570, 0.61678, 0.61682, 0.61578, 0.61368,\
        0.61063, 0.60699, 0.60386, 0.60257, 0.60215, 0.60206, 0.60197, 0.60321, 0.60686, 0.61137, 0.61547,\
        0.61864, 0.62060, 0.62125, 0.62056, 0.61862, 0.61557, 0.61170, 0.60759, 0.60449, 0.60273, 0.60072,\
        0.59858, 0.59634, 0.59400, 0.59203, 0.59075, 0.59024, 0.59028, 0.59037, 0.58995, 0.58857, 0.58608,\
        0.58246, 0.57766, 0.57169, 0.56455, 0.55628, 0.54707, 0.53723, 0.52724, 0.51771, 0.50955, 0.50427,\
        0.50237, 0.50148, 0.50042, 0.49892, 0.49666, 0.49334, 0.48863, 0.48225, 0.47400, 0.46389, 0.45213,\
        0.43922, 0.42602, 0.41389, 0.40480, 0.39990, 0.39759, 0.39643, 0.39564, 0.39499, 0.39450, 0.39418,\
        0.39402, 0.39405, 0.39431, 0.39486, 0.39568, 0.39674, 0.39795, 0.39923, 0.40049, 0.40176, 0.40300],\
        [0.51703, 0.51081, 0.50524, 0.50086, 0.49843, 0.49754, 0.49750, 0.49780, 0.49816, 0.49870, 0.50071,\
        0.50754, 0.51674, 0.52703, 0.53770, 0.54821, 0.55813, 0.56708, 0.57477, 0.58101, 0.58570, 0.58897,\
        0.59111, 0.59250, 0.59344, 0.59414, 0.59471, 0.59526, 0.59585, 0.59645, 0.59708, 0.59770, 0.59823,\
        0.59864, 0.59917, 0.60031, 0.60269, 0.60513, 0.60720, 0.60867, 0.60940, 0.60931, 0.60838, 0.60666,\
        0.60433, 0.60172, 0.59972, 0.59945, 0.59926, 0.59914, 0.59890, 0.59940, 0.60189, 0.60522, 0.60833,\
        0.61080, 0.61236, 0.61288, 0.61232, 0.61072, 0.60825, 0.60515, 0.60186, 0.59986, 0.59968, 0.59861,\
        0.59708, 0.59541, 0.59368, 0.59214, 0.59098, 0.59021, 0.58964, 0.58891, 0.58763, 0.58549, 0.58234,\
        0.57813, 0.57283, 0.56645, 0.55902, 0.55065, 0.54149, 0.53186, 0.52214, 0.51289, 0.50489, 0.49993,\
        0.49983, 0.49957, 0.49881, 0.49751, 0.49550, 0.49247, 0.48806, 0.48199, 0.47406, 0.46421, 0.45250,\
        0.43930, 0.42534, 0.41191, 0.40134, 0.39748, 0.39691, 0.39646, 0.39593, 0.39554, 0.39528, 0.39514,\
        0.39509, 0.39513, 0.39534, 0.39579, 0.39649, 0.39741, 0.39844, 0.39949, 0.40047, 0.40137, 0.40226],\
        [0.51354, 0.50845, 0.50407, 0.50089, 0.49946, 0.49920, 0.49927, 0.49960, 0.49988, 0.50065, 0.50264,\
        0.50850, 0.51710, 0.52708, 0.53763, 0.54818, 0.55828, 0.56752, 0.57556, 0.58210, 0.58699, 0.59029,\
        0.59232, 0.59354, 0.59434, 0.59495, 0.59550, 0.59603, 0.59660, 0.59723, 0.59786, 0.59847, 0.59902,\
        0.59956, 0.59995, 0.60014, 0.60065, 0.60173, 0.60286, 0.60371, 0.60410, 0.60395, 0.60328, 0.60219,\
        0.60096, 0.60008, 0.60003, 0.60012, 0.59983, 0.59962, 0.59964, 0.60010, 0.60062, 0.60198, 0.60373,\
        0.60531, 0.60637, 0.60673, 0.60633, 0.60522, 0.60359, 0.60177, 0.60040, 0.60008, 0.59970, 0.59890,\
        0.59766, 0.59626, 0.59482, 0.59347, 0.59231, 0.59127, 0.59016, 0.58869, 0.58659, 0.58369, 0.57988,\
        0.57511, 0.56937, 0.56270, 0.55516, 0.54685, 0.53797, 0.52880, 0.51977, 0.51143, 0.50472, 0.50116,\
        0.50043, 0.50003, 0.49922, 0.49796, 0.49619, 0.49354, 0.48958, 0.48402, 0.47671, 0.46751, 0.45632,\
        0.44332, 0.42928, 0.41557, 0.40469, 0.40051, 0.40001, 0.39903, 0.39814, 0.39760, 0.39726, 0.39706,\
        0.39693, 0.39677, 0.39677, 0.39704, 0.39754, 0.39827, 0.39910, 0.39987, 0.40051, 0.40111, 0.40165],\
        [0.51099, 0.50697, 0.50371, 0.50159, 0.50076, 0.50079, 0.50105, 0.50153, 0.50219, 0.50352, 0.50634,\
        0.51173, 0.51955, 0.52895, 0.53916, 0.54959, 0.55976, 0.56922, 0.57757, 0.58441, 0.58944, 0.59261,\
        0.59433, 0.59525, 0.59588, 0.59644, 0.59698, 0.59752, 0.59810, 0.59873, 0.59936, 0.59988, 0.60026,\
        0.60049, 0.60039, 0.60000, 0.59973, 0.59983, 0.60014, 0.60042, 0.60051, 0.60035, 0.59994, 0.59943,\
        0.59905, 0.59907, 0.59952, 0.59998, 0.60016, 0.60015, 0.60010, 0.59996, 0.59984, 0.60014, 0.60085,\
        0.60165, 0.60225, 0.60247, 0.60226, 0.60166, 0.60086, 0.60016, 0.59992, 0.60016, 0.60033, 0.59997,\
        0.59911, 0.59800, 0.59675, 0.59551, 0.59432, 0.59309, 0.59153, 0.58941, 0.58658, 0.58297, 0.57854,\
        0.57328, 0.56721, 0.56038, 0.55285, 0.54476, 0.53633, 0.52784, 0.51971, 0.51251, 0.50698, 0.50376,\
        0.50242, 0.50175, 0.50097, 0.49988, 0.49844, 0.49630, 0.49290, 0.48801, 0.48165, 0.47361, 0.46344,\
        0.45116, 0.43768, 0.42465, 0.41436, 0.40849, 0.40574, 0.40378, 0.40222, 0.40115, 0.40039, 0.39987,\
        0.39951, 0.39888, 0.39861, 0.39862, 0.39881, 0.39930, 0.39989, 0.40034, 0.40067, 0.40093, 0.40115],\
        [0.50909, 0.50602, 0.50367, 0.50226, 0.50180, 0.50202, 0.50260, 0.50347, 0.50475, 0.50683, 0.51031,\
        0.51572, 0.52309, 0.53197, 0.54177, 0.55197, 0.56210, 0.57171, 0.58036, 0.58755, 0.59272, 0.59561,\
        0.59676, 0.59722, 0.59769, 0.59823, 0.59877, 0.59927, 0.59976, 0.60028, 0.60074, 0.60100, 0.60107,\
        0.60091, 0.60046, 0.59980, 0.59916, 0.59872, 0.59849, 0.59836, 0.59822, 0.59804, 0.59783, 0.59770,\
        0.59777, 0.59814, 0.59873, 0.59931, 0.59966, 0.59975, 0.59962, 0.59931, 0.59897, 0.59883, 0.59897,\
        0.59926, 0.59954, 0.59969, 0.59966, 0.59949, 0.59932, 0.59933, 0.59966, 0.60023, 0.60071, 0.60082,\
        0.60051, 0.59982, 0.59882, 0.59772, 0.59658, 0.59533, 0.59348, 0.59079, 0.58729, 0.58304, 0.57810,\
        0.57246, 0.56617, 0.55929, 0.55190, 0.54415, 0.53626, 0.52852, 0.52129, 0.51506, 0.51028, 0.50720,\
        0.50553, 0.50460, 0.50386, 0.50301, 0.50192, 0.50031, 0.49751, 0.49333, 0.48824, 0.48194, 0.47324,\
        0.46185, 0.44895, 0.43641, 0.42598, 0.41860, 0.41378, 0.41039, 0.40786, 0.40596, 0.40453, 0.40343,\
        0.40249, 0.40150, 0.40091, 0.40055, 0.40035, 0.40048, 0.40073, 0.40086, 0.40089, 0.40084, 0.40074],\
        [0.50761, 0.50534, 0.50372, 0.50285, 0.50274, 0.50323, 0.50417, 0.50554, 0.50746, 0.51023, 0.51426,\
        0.51983, 0.52699, 0.53548, 0.54490, 0.55481, 0.56480, 0.57444, 0.58331, 0.59089, 0.59634, 0.59884,\
        0.59920, 0.59912, 0.59952, 0.60003, 0.60050, 0.60089, 0.60123, 0.60153, 0.60173, 0.60174, 0.60155,\
        0.60111, 0.60044, 0.59962, 0.59878, 0.59807, 0.59752, 0.59712, 0.59683, 0.59662, 0.59651, 0.59656,\
        0.59681, 0.59728, 0.59788, 0.59843, 0.59880, 0.59892, 0.59879, 0.59847, 0.59809, 0.59780, 0.59768,\
        0.59772, 0.59784, 0.59797, 0.59809, 0.59824, 0.59848, 0.59892, 0.59956, 0.60032, 0.60101, 0.60143,\
        0.60149, 0.60118, 0.60056, 0.59965, 0.59868, 0.59772, 0.59567, 0.59244, 0.58836, 0.58362, 0.57829,\
        0.57240, 0.56602, 0.55920, 0.55204, 0.54471, 0.53739, 0.53036, 0.52393, 0.51844, 0.51418, 0.51124,\
        0.50943, 0.50835, 0.50762, 0.50696, 0.50613, 0.50484, 0.50263, 0.49922, 0.49565, 0.49172, 0.48468,\
        0.47385, 0.46121, 0.44879, 0.43797, 0.42943, 0.42303, 0.41823, 0.41455, 0.41169, 0.40943, 0.40761,\
        0.40605, 0.40469, 0.40363, 0.40283, 0.40226, 0.40193, 0.40172, 0.40149, 0.40120, 0.40084, 0.40041],\
        [0.50639, 0.50480, 0.50378, 0.50341, 0.50367, 0.50450, 0.50583, 0.50769, 0.51019, 0.51355, 0.51801,\
        0.52376, 0.53081, 0.53901, 0.54806, 0.55763, 0.56734, 0.57683, 0.58568, 0.59341, 0.59925, 0.60142,\
        0.60140, 0.60108, 0.60129, 0.60164, 0.60196, 0.60221, 0.60238, 0.60247, 0.60243, 0.60223, 0.60182,\
        0.60120, 0.60040, 0.59948, 0.59854, 0.59768, 0.59697, 0.59642, 0.59602, 0.59577, 0.59569, 0.59579,\
        0.59608, 0.59653, 0.59705, 0.59754, 0.59787, 0.59799, 0.59789, 0.59762, 0.59728, 0.59697, 0.59678,\
        0.59673, 0.59680, 0.59696, 0.59721, 0.59757, 0.59807, 0.59874, 0.59956, 0.60045, 0.60126, 0.60186,\
        0.60217, 0.60214, 0.60178, 0.60112, 0.60048, 0.59994, 0.59758, 0.59389, 0.58943, 0.58439, 0.57884,\
        0.57286, 0.56651, 0.55985, 0.55300, 0.54611, 0.53936, 0.53297, 0.52721, 0.52230, 0.51843, 0.51564,\
        0.51380, 0.51265, 0.51191, 0.51133, 0.51067, 0.50965, 0.50800, 0.50570, 0.50362, 0.50150, 0.49589,\
        0.48509, 0.47266, 0.46038, 0.44933, 0.44002, 0.43251, 0.42654, 0.42179, 0.41796, 0.41485, 0.41227,\
        0.41008, 0.40822, 0.40667, 0.40541, 0.40439, 0.40359, 0.40292, 0.40228, 0.40162, 0.40092, 0.40016]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=30 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax30vt2.5
    zgrid_H_Hmax_vmax30vt2_5=[[0.28713, 0.29685, 0.30627, 0.31540, 0.32424, 0.33279, 0.34105, 0.34903, 0.35673, 0.36416, 0.37132,\
        0.37822, 0.38486, 0.39127, 0.39743, 0.40338, 0.40911, 0.41465, 0.42001, 0.42519, 0.43022, 0.43511,\
        0.43988, 0.44455, 0.44915, 0.45373, 0.45832, 0.46299, 0.46780, 0.47283, 0.47812, 0.48370, 0.48956,\
        0.49556, 0.50110, 0.50598, 0.51073, 0.51543, 0.52004, 0.52451, 0.52879, 0.53283, 0.53657, 0.54001,\
        0.54313, 0.54595, 0.54849, 0.55081, 0.55297, 0.55501, 0.55700, 0.55896, 0.56092, 0.56286, 0.56477,\
        0.56659, 0.56827, 0.56974, 0.57093, 0.57178, 0.57228, 0.57244, 0.57231, 0.57194, 0.57139, 0.57070,\
        0.56992, 0.56907, 0.56814, 0.56712, 0.56602, 0.56482, 0.56352, 0.56215, 0.56071, 0.55921, 0.55763,\
        0.55592, 0.55404, 0.55197, 0.54968, 0.54717, 0.54445, 0.54152, 0.53838, 0.53504, 0.53151, 0.52780,\
        0.52393, 0.51993, 0.51583, 0.51165, 0.50745, 0.50327, 0.49920, 0.49546, 0.49213, 0.48839, 0.48417,\
        0.47954, 0.47462, 0.46942, 0.46388, 0.45795, 0.45166, 0.44505, 0.43813, 0.43093, 0.42345, 0.41567,\
        0.40759, 0.39920, 0.39049, 0.38145, 0.37207, 0.36236, 0.35231, 0.34193, 0.33121, 0.32015, 0.30876],\
        [0.29697, 0.30656, 0.31585, 0.32484, 0.33354, 0.34194, 0.35005, 0.35788, 0.36542, 0.37268, 0.37966,\
        0.38638, 0.39284, 0.39904, 0.40501, 0.41074, 0.41626, 0.42158, 0.42670, 0.43165, 0.43644, 0.44107,\
        0.44556, 0.44994, 0.45422, 0.45845, 0.46267, 0.46695, 0.47137, 0.47601, 0.48094, 0.48622, 0.49184,\
        0.49761, 0.50335, 0.50873, 0.51399, 0.51917, 0.52423, 0.52912, 0.53376, 0.53809, 0.54208, 0.54570,\
        0.54893, 0.55179, 0.55434, 0.55662, 0.55873, 0.56074, 0.56270, 0.56469, 0.56671, 0.56877, 0.57083,\
        0.57283, 0.57471, 0.57637, 0.57775, 0.57873, 0.57929, 0.57946, 0.57931, 0.57892, 0.57834, 0.57764,\
        0.57688, 0.57606, 0.57517, 0.57420, 0.57314, 0.57196, 0.57068, 0.56933, 0.56794, 0.56652, 0.56504,\
        0.56344, 0.56165, 0.55964, 0.55739, 0.55491, 0.55220, 0.54928, 0.54615, 0.54281, 0.53927, 0.53554,\
        0.53166, 0.52763, 0.52350, 0.51931, 0.51511, 0.51094, 0.50692, 0.50327, 0.50010, 0.49632, 0.49209,\
        0.48739, 0.48256, 0.47754, 0.47211, 0.46624, 0.45999, 0.45344, 0.44662, 0.43953, 0.43217, 0.42452,\
        0.41657, 0.40831, 0.39972, 0.39080, 0.38153, 0.37193, 0.36198, 0.35169, 0.34107, 0.33011, 0.31881],\
        [0.30655, 0.31601, 0.32517, 0.33402, 0.34258, 0.35084, 0.35880, 0.36647, 0.37385, 0.38094, 0.38775,\
        0.39429, 0.40056, 0.40657, 0.41233, 0.41785, 0.42316, 0.42825, 0.43315, 0.43786, 0.44240, 0.44678,\
        0.45100, 0.45508, 0.45904, 0.46291, 0.46673, 0.47058, 0.47453, 0.47870, 0.48318, 0.48804, 0.49331,\
        0.49901, 0.50477, 0.51074, 0.51666, 0.52247, 0.52808, 0.53345, 0.53851, 0.54320, 0.54746, 0.55126,\
        0.55460, 0.55750, 0.56001, 0.56223, 0.56425, 0.56618, 0.56810, 0.57008, 0.57215, 0.57432, 0.57655,\
        0.57876, 0.58085, 0.58274, 0.58433, 0.58549, 0.58610, 0.58625, 0.58611, 0.58569, 0.58507, 0.58437,\
        0.58364, 0.58287, 0.58203, 0.58112, 0.58010, 0.57895, 0.57766, 0.57628, 0.57492, 0.57360, 0.57225,\
        0.57078, 0.56908, 0.56713, 0.56490, 0.56243, 0.55973, 0.55682, 0.55369, 0.55035, 0.54680, 0.54305,\
        0.53914, 0.53508, 0.53093, 0.52674, 0.52254, 0.51842, 0.51448, 0.51084, 0.50734, 0.50356, 0.49934,\
        0.49445, 0.48991, 0.48522, 0.47987, 0.47400, 0.46780, 0.46133, 0.45461, 0.44766, 0.44045, 0.43296,\
        0.42516, 0.41704, 0.40858, 0.39978, 0.39063, 0.38113, 0.37129, 0.36110, 0.35057, 0.33970, 0.32850],\
        [0.31587, 0.32520, 0.33423, 0.34295, 0.35137, 0.35949, 0.36730, 0.37482, 0.38204, 0.38897, 0.39560,\
        0.40196, 0.40804, 0.41385, 0.41940, 0.42472, 0.42980, 0.43467, 0.43935, 0.44383, 0.44814, 0.45228,\
        0.45624, 0.46005, 0.46369, 0.46720, 0.47061, 0.47399, 0.47745, 0.48111, 0.48510, 0.48952, 0.49436,\
        0.49964, 0.50597, 0.51257, 0.51916, 0.52560, 0.53179, 0.53768, 0.54318, 0.54823, 0.55278, 0.55679,\
        0.56023, 0.56314, 0.56558, 0.56768, 0.56956, 0.57136, 0.57320, 0.57515, 0.57726, 0.57953, 0.58193,\
        0.58437, 0.58667, 0.58878, 0.59065, 0.59205, 0.59268, 0.59279, 0.59270, 0.59231, 0.59161, 0.59092,\
        0.59027, 0.58955, 0.58877, 0.58791, 0.58695, 0.58583, 0.58447, 0.58302, 0.58167, 0.58047, 0.57930,\
        0.57797, 0.57637, 0.57445, 0.57222, 0.56973, 0.56703, 0.56412, 0.56101, 0.55767, 0.55410, 0.55032,\
        0.54637, 0.54228, 0.53809, 0.53388, 0.52968, 0.52557, 0.52161, 0.51781, 0.51403, 0.51004, 0.50559,\
        0.50077, 0.49693, 0.49252, 0.48704, 0.48117, 0.47503, 0.46867, 0.46210, 0.45532, 0.44830, 0.44098,\
        0.43335, 0.42538, 0.41707, 0.40839, 0.39936, 0.38998, 0.38024, 0.37015, 0.35971, 0.34894, 0.33783],\
        [0.32494, 0.33415, 0.34304, 0.35163, 0.35991, 0.36789, 0.37556, 0.38293, 0.38999, 0.39675, 0.40322,\
        0.40939, 0.41528, 0.42089, 0.42624, 0.43134, 0.43621, 0.44086, 0.44532, 0.44958, 0.45367, 0.45759,\
        0.46133, 0.46488, 0.46824, 0.47142, 0.47443, 0.47736, 0.48031, 0.48345, 0.48698, 0.49106, 0.49581,\
        0.50130, 0.50771, 0.51473, 0.52184, 0.52882, 0.53555, 0.54193, 0.54786, 0.55329, 0.55813, 0.56234,\
        0.56588, 0.56877, 0.57110, 0.57302, 0.57470, 0.57633, 0.57804, 0.57994, 0.58205, 0.58438, 0.58696,\
        0.58968, 0.59220, 0.59442, 0.59656, 0.59835, 0.59896, 0.59905, 0.59909, 0.59881, 0.59799, 0.59739,\
        0.59684, 0.59618, 0.59542, 0.59460, 0.59370, 0.59264, 0.59115, 0.58950, 0.58819, 0.58716, 0.58620,\
        0.58506, 0.58353, 0.58160, 0.57932, 0.57679, 0.57407, 0.57119, 0.56809, 0.56476, 0.56116, 0.55734,\
        0.55333, 0.54918, 0.54495, 0.54069, 0.53645, 0.53230, 0.52824, 0.52427, 0.52028, 0.51613, 0.51175,\
        0.50743, 0.50356, 0.49893, 0.49333, 0.48767, 0.48166, 0.47544, 0.46908, 0.46254, 0.45574, 0.44863,\
        0.44118, 0.43337, 0.42519, 0.41665, 0.40774, 0.39846, 0.38882, 0.37883, 0.36850, 0.35782, 0.34681],\
        [0.33376, 0.34284, 0.35161, 0.36006, 0.36821, 0.37605, 0.38358, 0.39080, 0.39771, 0.40431, 0.41060,\
        0.41660, 0.42230, 0.42771, 0.43286, 0.43774, 0.44239, 0.44682, 0.45106, 0.45512, 0.45900, 0.46273,\
        0.46627, 0.46963, 0.47276, 0.47566, 0.47833, 0.48082, 0.48327, 0.48590, 0.48898, 0.49283, 0.49766,\
        0.50347, 0.51013, 0.51741, 0.52490, 0.53233, 0.53951, 0.54632, 0.55265, 0.55844, 0.56358, 0.56800,\
        0.57163, 0.57447, 0.57663, 0.57830, 0.57971, 0.58111, 0.58268, 0.58452, 0.58661, 0.58892, 0.59159,\
        0.59468, 0.59751, 0.59971, 0.60198, 0.60393, 0.60486, 0.60520, 0.60532, 0.60509, 0.60447, 0.60394,\
        0.60344, 0.60280, 0.60206, 0.60124, 0.60036, 0.59942, 0.59765, 0.59572, 0.59456, 0.59374, 0.59303,\
        0.59206, 0.59056, 0.58854, 0.58617, 0.58356, 0.58084, 0.57800, 0.57495, 0.57161, 0.56798, 0.56409,\
        0.56000, 0.55577, 0.55146, 0.54713, 0.54283, 0.53859, 0.53443, 0.53031, 0.52618, 0.52196, 0.51767,\
        0.51340, 0.50907, 0.50414, 0.49906, 0.49364, 0.48767, 0.48166, 0.47560, 0.46934, 0.46280, 0.45591,\
        0.44865, 0.44100, 0.43297, 0.42455, 0.41576, 0.40659, 0.39705, 0.38716, 0.37692, 0.36633, 0.35542],\
        [0.34233, 0.35128, 0.35993, 0.36826, 0.37627, 0.38398, 0.39136, 0.39844, 0.40520, 0.41164, 0.41777,\
        0.42359, 0.42910, 0.43432, 0.43925, 0.44392, 0.44835, 0.45256, 0.45658, 0.46043, 0.46413, 0.46769,\
        0.47109, 0.47430, 0.47728, 0.47999, 0.48239, 0.48451, 0.48650, 0.48863, 0.49126, 0.49488, 0.49976,\
        0.50595, 0.51301, 0.52062, 0.52844, 0.53622, 0.54376, 0.55092, 0.55760, 0.56370, 0.56913, 0.57377,\
        0.57751, 0.58028, 0.58220, 0.58356, 0.58466, 0.58578, 0.58718, 0.58900, 0.59113, 0.59328, 0.59579,\
        0.59913, 0.60245, 0.60496, 0.60729, 0.60925, 0.61052, 0.61120, 0.61148, 0.61140, 0.61104, 0.61059,\
        0.61009, 0.60947, 0.60874, 0.60791, 0.60696, 0.60579, 0.60399, 0.60212, 0.60110, 0.60034, 0.59978,\
        0.59891, 0.59735, 0.59521, 0.59271, 0.58999, 0.58728, 0.58457, 0.58159, 0.57824, 0.57454, 0.57055,\
        0.56635, 0.56201, 0.55760, 0.55318, 0.54879, 0.54446, 0.54019, 0.53597, 0.53174, 0.52747, 0.52315,\
        0.51877, 0.51423, 0.50940, 0.50441, 0.49878, 0.49304, 0.48742, 0.48173, 0.47582, 0.46955, 0.46287,\
        0.45578, 0.44828, 0.44039, 0.43210, 0.42342, 0.41436, 0.40492, 0.39512, 0.38497, 0.37448, 0.36367],\
        [0.35065, 0.35949, 0.36801, 0.37621, 0.38410, 0.39167, 0.39892, 0.40585, 0.41246, 0.41876, 0.42472,\
        0.43037, 0.43570, 0.44072, 0.44544, 0.44989, 0.45409, 0.45807, 0.46187, 0.46552, 0.46904, 0.47245,\
        0.47574, 0.47888, 0.48180, 0.48443, 0.48668, 0.48852, 0.49014, 0.49185, 0.49396, 0.49735, 0.50256,\
        0.50907, 0.51648, 0.52439, 0.53249, 0.54053, 0.54834, 0.55576, 0.56269, 0.56905, 0.57475, 0.57964,\
        0.58353, 0.58622, 0.58784, 0.58884, 0.58960, 0.59041, 0.59162, 0.59346, 0.59586, 0.59798, 0.59989,\
        0.60346, 0.60703, 0.61006, 0.61262, 0.61470, 0.61621, 0.61716, 0.61765, 0.61777, 0.61760, 0.61726,\
        0.61679, 0.61618, 0.61545, 0.61457, 0.61353, 0.61223, 0.61063, 0.60908, 0.60794, 0.60711, 0.60637,\
        0.60534, 0.60372, 0.60153, 0.59896, 0.59604, 0.59342, 0.59096, 0.58803, 0.58460, 0.58079, 0.57668,\
        0.57235, 0.56787, 0.56334, 0.55882, 0.55434, 0.54992, 0.54557, 0.54125, 0.53695, 0.53262, 0.52825,\
        0.52378, 0.51915, 0.51429, 0.50912, 0.50360, 0.49821, 0.49292, 0.48764, 0.48208, 0.47604, 0.46953,\
        0.46258, 0.45523, 0.44747, 0.43930, 0.43073, 0.42177, 0.41242, 0.40271, 0.39266, 0.38227, 0.37155],\
        [0.35874, 0.36745, 0.37585, 0.38392, 0.39168, 0.39912, 0.40624, 0.41304, 0.41951, 0.42565, 0.43147,\
        0.43695, 0.44210, 0.44692, 0.45143, 0.45565, 0.45960, 0.46334, 0.46691, 0.47035, 0.47371, 0.47699,\
        0.48019, 0.48330, 0.48625, 0.48893, 0.49119, 0.49286, 0.49418, 0.49578, 0.49754, 0.50040, 0.50607,\
        0.51298, 0.52066, 0.52878, 0.53706, 0.54528, 0.55325, 0.56081, 0.56789, 0.57440, 0.58030, 0.58547,\
        0.58961, 0.59228, 0.59347, 0.59416, 0.59470, 0.59520, 0.59622, 0.59807, 0.60067, 0.60321, 0.60541,\
        0.60866, 0.61218, 0.61540, 0.61814, 0.62037, 0.62206, 0.62321, 0.62390, 0.62420, 0.62419, 0.62394,\
        0.62351, 0.62293, 0.62218, 0.62127, 0.62019, 0.61891, 0.61747, 0.61608, 0.61489, 0.61386, 0.61283,\
        0.61154, 0.60979, 0.60755, 0.60487, 0.60185, 0.59948, 0.59727, 0.59422, 0.59064, 0.58670, 0.58245,\
        0.57795, 0.57333, 0.56867, 0.56405, 0.55949, 0.55499, 0.55057, 0.54619, 0.54183, 0.53746, 0.53304,\
        0.52853, 0.52387, 0.51901, 0.51392, 0.50867, 0.50344, 0.49831, 0.49349, 0.48819, 0.48226, 0.47586,\
        0.46904, 0.46183, 0.45420, 0.44615, 0.43769, 0.42881, 0.41956, 0.40994, 0.39997, 0.38968, 0.37907],\
        [0.36658, 0.37517, 0.38345, 0.39140, 0.39904, 0.40635, 0.41334, 0.42000, 0.42634, 0.43234, 0.43800,\
        0.44332, 0.44830, 0.45293, 0.45722, 0.46120, 0.46491, 0.46839, 0.47171, 0.47492, 0.47809, 0.48124,\
        0.48437, 0.48748, 0.49049, 0.49330, 0.49576, 0.49746, 0.49834, 0.50023, 0.50246, 0.50554, 0.51097,\
        0.51787, 0.52560, 0.53379, 0.54215, 0.55044, 0.55846, 0.56606, 0.57314, 0.57965, 0.58561, 0.59095,\
        0.59543, 0.59831, 0.59888, 0.59952, 0.60011, 0.60048, 0.60151, 0.60345, 0.60605, 0.60880, 0.61156,\
        0.61470, 0.61804, 0.62123, 0.62402, 0.62635, 0.62815, 0.62946, 0.63029, 0.63073, 0.63083, 0.63066,\
        0.63027, 0.62970, 0.62895, 0.62803, 0.62695, 0.62571, 0.62437, 0.62304, 0.62177, 0.62055, 0.61925,\
        0.61771, 0.61579, 0.61347, 0.61081, 0.60803, 0.60558, 0.60309, 0.59993, 0.59621, 0.59220, 0.58782,\
        0.58313, 0.57834, 0.57357, 0.56887, 0.56424, 0.55970, 0.55522, 0.55080, 0.54641, 0.54202, 0.53758,\
        0.53307, 0.52844, 0.52366, 0.51874, 0.51373, 0.50875, 0.50395, 0.49943, 0.49405, 0.48814, 0.48182,\
        0.47512, 0.46806, 0.46057, 0.45265, 0.44428, 0.43549, 0.42631, 0.41677, 0.40690, 0.39671, 0.38621],\
        [0.37419, 0.38266, 0.39082, 0.39865, 0.40616, 0.41334, 0.42021, 0.42674, 0.43294, 0.43881, 0.44433,\
        0.44950, 0.45431, 0.45876, 0.46284, 0.46658, 0.47001, 0.47320, 0.47624, 0.47921, 0.48217, 0.48517,\
        0.48821, 0.49130, 0.49439, 0.49729, 0.49984, 0.50180, 0.50287, 0.50495, 0.50774, 0.51151, 0.51684,\
        0.52355, 0.53117, 0.53931, 0.54766, 0.55595, 0.56395, 0.57147, 0.57842, 0.58475, 0.59051, 0.59579,\
        0.60026, 0.60323, 0.60427, 0.60502, 0.60577, 0.60651, 0.60776, 0.60974, 0.61228, 0.61511, 0.61809,\
        0.62124, 0.62446, 0.62756, 0.63033, 0.63267, 0.63454, 0.63592, 0.63684, 0.63736, 0.63754, 0.63742,\
        0.63706, 0.63650, 0.63576, 0.63484, 0.63377, 0.63257, 0.63127, 0.62993, 0.62857, 0.62717, 0.62564,\
        0.62388, 0.62181, 0.61941, 0.61676, 0.61401, 0.61129, 0.60840, 0.60507, 0.60132, 0.59730, 0.59273,\
        0.58779, 0.58287, 0.57805, 0.57331, 0.56864, 0.56406, 0.55956, 0.55513, 0.55073, 0.54634, 0.54191,\
        0.53743, 0.53287, 0.52820, 0.52346, 0.51867, 0.51394, 0.50931, 0.50459, 0.49930, 0.49352, 0.48731,\
        0.48079, 0.47392, 0.46660, 0.45880, 0.45051, 0.44179, 0.43268, 0.42322, 0.41344, 0.40336, 0.39298],\
        [0.38156, 0.38992, 0.39795, 0.40566, 0.41305, 0.42011, 0.42684, 0.43324, 0.43932, 0.44506, 0.45045,\
        0.45548, 0.46014, 0.46442, 0.46829, 0.47178, 0.47492, 0.47780, 0.48052, 0.48320, 0.48594, 0.48877,\
        0.49168, 0.49469, 0.49782, 0.50083, 0.50340, 0.50559, 0.50755, 0.51001, 0.51327, 0.51752, 0.52298,\
        0.52960, 0.53710, 0.54515, 0.55345, 0.56171, 0.56964, 0.57702, 0.58375, 0.58978, 0.59505, 0.60003,\
        0.60439, 0.60749, 0.60932, 0.61056, 0.61166, 0.61286, 0.61443, 0.61651, 0.61907, 0.62194, 0.62499,\
        0.62815, 0.63130, 0.63431, 0.63702, 0.63934, 0.64121, 0.64261, 0.64357, 0.64412, 0.64433, 0.64423,\
        0.64389, 0.64333, 0.64258, 0.64167, 0.64061, 0.63942, 0.63813, 0.63676, 0.63531, 0.63375, 0.63202,\
        0.63006, 0.62783, 0.62532, 0.62258, 0.61969, 0.61668, 0.61345, 0.60989, 0.60601, 0.60185, 0.59695,\
        0.59184, 0.58696, 0.58216, 0.57741, 0.57273, 0.56814, 0.56363, 0.55920, 0.55482, 0.55045, 0.54606,\
        0.54164, 0.53715, 0.53260, 0.52800, 0.52337, 0.51874, 0.51409, 0.50925, 0.50404, 0.49842, 0.49229,\
        0.48604, 0.47944, 0.47230, 0.46459, 0.45636, 0.44768, 0.43862, 0.42925, 0.41957, 0.40962, 0.39938],\
        [0.38870, 0.39694, 0.40486, 0.41244, 0.41970, 0.42663, 0.43324, 0.43951, 0.44546, 0.45107, 0.45634,\
        0.46125, 0.46578, 0.46990, 0.47359, 0.47683, 0.47966, 0.48218, 0.48453, 0.48689, 0.48939, 0.49207,\
        0.49486, 0.49766, 0.50079, 0.50388, 0.50672, 0.50935, 0.51199, 0.51502, 0.51876, 0.52340, 0.52904,\
        0.53566, 0.54309, 0.55107, 0.55933, 0.56756, 0.57545, 0.58269, 0.58912, 0.59491, 0.59990, 0.60430,\
        0.60853, 0.61183, 0.61421, 0.61602, 0.61762, 0.61928, 0.62120, 0.62350, 0.62616, 0.62909, 0.63218,\
        0.63534, 0.63845, 0.64139, 0.64404, 0.64631, 0.64815, 0.64952, 0.65046, 0.65100, 0.65119, 0.65109,\
        0.65073, 0.65017, 0.64942, 0.64850, 0.64744, 0.64626, 0.64495, 0.64353, 0.64199, 0.64029, 0.63839,\
        0.63625, 0.63385, 0.63118, 0.62827, 0.62516, 0.62185, 0.61828, 0.61441, 0.61019, 0.60552, 0.60032,\
        0.59551, 0.59080, 0.58604, 0.58127, 0.57656, 0.57196, 0.56747, 0.56306, 0.55871, 0.55438, 0.55005,\
        0.54569, 0.54128, 0.53682, 0.53232, 0.52777, 0.52318, 0.51848, 0.51357, 0.50833, 0.50264, 0.49676,\
        0.49102, 0.48472, 0.47770, 0.47002, 0.46179, 0.45313, 0.44413, 0.43484, 0.42529, 0.41548, 0.40540],\
        [0.39562, 0.40374, 0.41153, 0.41900, 0.42613, 0.43293, 0.43940, 0.44554, 0.45136, 0.45685, 0.46200,\
        0.46680, 0.47122, 0.47522, 0.47874, 0.48176, 0.48427, 0.48638, 0.48831, 0.49029, 0.49252, 0.49510,\
        0.49792, 0.50067, 0.50362, 0.50684, 0.50997, 0.51305, 0.51626, 0.51984, 0.52403, 0.52900, 0.53483,\
        0.54150, 0.54889, 0.55682, 0.56505, 0.57331, 0.58124, 0.58841, 0.59451, 0.59978, 0.60479, 0.60909,\
        0.61301, 0.61637, 0.61912, 0.62143, 0.62354, 0.62566, 0.62796, 0.63052, 0.63335, 0.63639, 0.63955,\
        0.64273, 0.64582, 0.64872, 0.65132, 0.65353, 0.65530, 0.65661, 0.65749, 0.65797, 0.65811, 0.65797,\
        0.65759, 0.65700, 0.65624, 0.65531, 0.65425, 0.65305, 0.65172, 0.65026, 0.64863, 0.64680, 0.64475,\
        0.64244, 0.63985, 0.63698, 0.63386, 0.63050, 0.62688, 0.62300, 0.61880, 0.61426, 0.60934, 0.60426,\
        0.59943, 0.59465, 0.58980, 0.58494, 0.58018, 0.57559, 0.57113, 0.56677, 0.56246, 0.55818, 0.55390,\
        0.54960, 0.54526, 0.54087, 0.53643, 0.53193, 0.52733, 0.52259, 0.51765, 0.51243, 0.50693, 0.50147,\
        0.49597, 0.48977, 0.48276, 0.47501, 0.46672, 0.45807, 0.44915, 0.43999, 0.43059, 0.42095, 0.41106],\
        [0.40231, 0.41031, 0.41799, 0.42532, 0.43232, 0.43899, 0.44532, 0.45132, 0.45700, 0.46236, 0.46739,\
        0.47209, 0.47641, 0.48032, 0.48375, 0.48658, 0.48877, 0.49044, 0.49188, 0.49344, 0.49534, 0.49780,\
        0.50068, 0.50358, 0.50662, 0.50989, 0.51325, 0.51671, 0.52037, 0.52441, 0.52899, 0.53425, 0.54024,\
        0.54696, 0.55432, 0.56218, 0.57036, 0.57865, 0.58672, 0.59406, 0.59995, 0.60489, 0.60952, 0.61376,\
        0.61760, 0.62104, 0.62407, 0.62679, 0.62935, 0.63191, 0.63460, 0.63747, 0.64053, 0.64373, 0.64699,\
        0.65022, 0.65332, 0.65620, 0.65877, 0.66093, 0.66263, 0.66385, 0.66461, 0.66500, 0.66506, 0.66486,\
        0.66443, 0.66382, 0.66303, 0.66209, 0.66101, 0.65980, 0.65845, 0.65694, 0.65524, 0.65331, 0.65111,\
        0.64863, 0.64584, 0.64276, 0.63939, 0.63577, 0.63187, 0.62769, 0.62322, 0.61845, 0.61345, 0.60838,\
        0.60341, 0.59850, 0.59347, 0.58842, 0.58363, 0.57908, 0.57468, 0.57037, 0.56611, 0.56188, 0.55764,\
        0.55339, 0.54910, 0.54477, 0.54036, 0.53587, 0.53126, 0.52651, 0.52156, 0.51644, 0.51120, 0.50597,\
        0.50052, 0.49442, 0.48731, 0.47939, 0.47105, 0.46245, 0.45366, 0.44467, 0.43548, 0.42606, 0.41638],\
        [0.40878, 0.41667, 0.42422, 0.43143, 0.43830, 0.44482, 0.45100, 0.45685, 0.46238, 0.46759, 0.47249,\
        0.47708, 0.48132, 0.48516, 0.48854, 0.49129, 0.49321, 0.49438, 0.49530, 0.49647, 0.49804, 0.50021,\
        0.50321, 0.50634, 0.50959, 0.51301, 0.51658, 0.52033, 0.52434, 0.52873, 0.53362, 0.53910, 0.54521,\
        0.55195, 0.55926, 0.56701, 0.57506, 0.58324, 0.59131, 0.59891, 0.60477, 0.60958, 0.61404, 0.61822,\
        0.62210, 0.62567, 0.62896, 0.63204, 0.63502, 0.63800, 0.64107, 0.64428, 0.64761, 0.65102, 0.65442,\
        0.65773, 0.66087, 0.66376, 0.66632, 0.66845, 0.67007, 0.67117, 0.67179, 0.67204, 0.67201, 0.67173,\
        0.67125, 0.67059, 0.66977, 0.66881, 0.66772, 0.66650, 0.66514, 0.66360, 0.66185, 0.65982, 0.65749,\
        0.65482, 0.65183, 0.64852, 0.64491, 0.64102, 0.63686, 0.63243, 0.62773, 0.62278, 0.61766, 0.61246,\
        0.60730, 0.60221, 0.59684, 0.59168, 0.58697, 0.58253, 0.57820, 0.57394, 0.56972, 0.56551, 0.56131,\
        0.55710, 0.55285, 0.54855, 0.54416, 0.53967, 0.53504, 0.53024, 0.52527, 0.52015, 0.51495, 0.50970,\
        0.50432, 0.49838, 0.49098, 0.48294, 0.47466, 0.46622, 0.45764, 0.44891, 0.43998, 0.43082, 0.42140],\
        [0.41504, 0.42282, 0.43025, 0.43733, 0.44406, 0.45043, 0.45645, 0.46213, 0.46749, 0.47253, 0.47728,\
        0.48173, 0.48587, 0.48963, 0.49297, 0.49578, 0.49754, 0.49822, 0.49869, 0.49946, 0.50117, 0.50316,\
        0.50604, 0.50926, 0.51264, 0.51618, 0.51992, 0.52389, 0.52816, 0.53280, 0.53791, 0.54354, 0.54973,\
        0.55646, 0.56367, 0.57125, 0.57908, 0.58698, 0.59471, 0.60194, 0.60819, 0.61340, 0.61806, 0.62236,\
        0.62639, 0.63016, 0.63372, 0.63714, 0.64050, 0.64388, 0.64733, 0.65088, 0.65452, 0.65817, 0.66176,\
        0.66518, 0.66837, 0.67129, 0.67387, 0.67602, 0.67758, 0.67852, 0.67895, 0.67904, 0.67890, 0.67855,\
        0.67800, 0.67731, 0.67647, 0.67547, 0.67437, 0.67314, 0.67178, 0.67024, 0.66845, 0.66635, 0.66389,\
        0.66104, 0.65783, 0.65428, 0.65043, 0.64630, 0.64190, 0.63724, 0.63234, 0.62723, 0.62194, 0.61655,\
        0.61109, 0.60553, 0.59990, 0.59492, 0.59041, 0.58608, 0.58180, 0.57755, 0.57333, 0.56913, 0.56494,\
        0.56075, 0.55653, 0.55225, 0.54787, 0.54337, 0.53869, 0.53383, 0.52876, 0.52352, 0.51811, 0.51255,\
        0.50674, 0.50040, 0.49320, 0.48550, 0.47753, 0.46941, 0.46116, 0.45276, 0.44416, 0.43531, 0.42617],\
        [0.42110, 0.42876, 0.43608, 0.44303, 0.44961, 0.45584, 0.46169, 0.46719, 0.47234, 0.47717, 0.48172,\
        0.48603, 0.49004, 0.49366, 0.49682, 0.49957, 0.50137, 0.50190, 0.50229, 0.50302, 0.50458, 0.50668,\
        0.50936, 0.51247, 0.51584, 0.51945, 0.52329, 0.52739, 0.53182, 0.53662, 0.54187, 0.54759, 0.55381,\
        0.56050, 0.56759, 0.57497, 0.58253, 0.59011, 0.59752, 0.60456, 0.61096, 0.61657, 0.62158, 0.62615,\
        0.63042, 0.63445, 0.63830, 0.64205, 0.64576, 0.64951, 0.65333, 0.65725, 0.66121, 0.66516, 0.66896,\
        0.67251, 0.67574, 0.67867, 0.68132, 0.68355, 0.68511, 0.68585, 0.68601, 0.68593, 0.68573, 0.68530,\
        0.68468, 0.68398, 0.68310, 0.68207, 0.68095, 0.67974, 0.67838, 0.67685, 0.67507, 0.67292, 0.67033,\
        0.66729, 0.66384, 0.66005, 0.65597, 0.65161, 0.64700, 0.64215, 0.63708, 0.63181, 0.62637, 0.62079,\
        0.61511, 0.60935, 0.60372, 0.59873, 0.59423, 0.58989, 0.58555, 0.58124, 0.57697, 0.57274, 0.56855,\
        0.56437, 0.56017, 0.55592, 0.55155, 0.54703, 0.54231, 0.53734, 0.53212, 0.52664, 0.52088, 0.51484,\
        0.50841, 0.50161, 0.49479, 0.48742, 0.47985, 0.47217, 0.46435, 0.45635, 0.44811, 0.43959, 0.43075],\
        [0.42696, 0.43452, 0.44171, 0.44854, 0.45499, 0.46105, 0.46674, 0.47203, 0.47695, 0.48153, 0.48583,\
        0.48994, 0.49384, 0.49735, 0.50021, 0.50269, 0.50444, 0.50536, 0.50602, 0.50693, 0.50839, 0.51042,\
        0.51295, 0.51591, 0.51920, 0.52278, 0.52665, 0.53081, 0.53532, 0.54020, 0.54552, 0.55128, 0.55749,\
        0.56412, 0.57109, 0.57829, 0.58559, 0.59288, 0.60010, 0.60707, 0.61356, 0.61945, 0.62478, 0.62968,\
        0.63423, 0.63854, 0.64269, 0.64675, 0.65079, 0.65488, 0.65905, 0.66332, 0.66764, 0.67192, 0.67599,\
        0.67968, 0.68290, 0.68580, 0.68850, 0.69093, 0.69259, 0.69306, 0.69286, 0.69265, 0.69252, 0.69198,\
        0.69121, 0.69061, 0.68971, 0.68856, 0.68748, 0.68630, 0.68495, 0.68346, 0.68172, 0.67954, 0.67681,\
        0.67355, 0.66985, 0.66582, 0.66151, 0.65696, 0.65217, 0.64717, 0.64196, 0.63657, 0.63101, 0.62532,\
        0.61954, 0.61379, 0.60825, 0.60316, 0.59853, 0.59397, 0.58944, 0.58501, 0.58065, 0.57637, 0.57215,\
        0.56799, 0.56382, 0.55960, 0.55526, 0.55073, 0.54595, 0.54088, 0.53548, 0.52972, 0.52360, 0.51711,\
        0.51029, 0.50329, 0.49618, 0.48908, 0.48193, 0.47473, 0.46738, 0.45981, 0.45195, 0.44377, 0.43521],\
        [0.43262, 0.44008, 0.44717, 0.45387, 0.46019, 0.46610, 0.47162, 0.47672, 0.48140, 0.48567, 0.48962,\
        0.49345, 0.49721, 0.50066, 0.50343, 0.50571, 0.50746, 0.50869, 0.50968, 0.51080, 0.51230, 0.51425,\
        0.51666, 0.51948, 0.52265, 0.52616, 0.52998, 0.53414, 0.53865, 0.54355, 0.54887, 0.55462, 0.56079,\
        0.56736, 0.57425, 0.58133, 0.58846, 0.59552, 0.60259, 0.60957, 0.61615, 0.62225, 0.62785, 0.63304,\
        0.63788, 0.64248, 0.64690, 0.65124, 0.65558, 0.65997, 0.66448, 0.66909, 0.67376, 0.67839, 0.68281,\
        0.68671, 0.68983, 0.69256, 0.69525, 0.69784, 0.69979, 0.70001, 0.69953, 0.69923, 0.69932, 0.69860,\
        0.69751, 0.69730, 0.69625, 0.69489, 0.69405, 0.69287, 0.69148, 0.69005, 0.68841, 0.68623, 0.68333,\
        0.67979, 0.67581, 0.67155, 0.66705, 0.66234, 0.65742, 0.65229, 0.64699, 0.64150, 0.63587, 0.63013,\
        0.62434, 0.61860, 0.61306, 0.60787, 0.60298, 0.59813, 0.59340, 0.58884, 0.58437, 0.58000, 0.57576,\
        0.57161, 0.56750, 0.56334, 0.55905, 0.55454, 0.54973, 0.54457, 0.53901, 0.53300, 0.52656, 0.51968,\
        0.51246, 0.50506, 0.49783, 0.49086, 0.48411, 0.47739, 0.47050, 0.46332, 0.45582, 0.44793, 0.43964],\
        [0.43810, 0.44547, 0.45245, 0.45905, 0.46523, 0.47101, 0.47636, 0.48129, 0.48577, 0.48972, 0.49319,\
        0.49662, 0.50022, 0.50355, 0.50638, 0.50869, 0.51054, 0.51200, 0.51327, 0.51460, 0.51617, 0.51809,\
        0.52040, 0.52308, 0.52613, 0.52953, 0.53326, 0.53735, 0.54181, 0.54666, 0.55193, 0.55763, 0.56375,\
        0.57026, 0.57710, 0.58416, 0.59126, 0.59827, 0.60526, 0.61218, 0.61881, 0.62506, 0.63090, 0.63634,\
        0.64146, 0.64631, 0.65099, 0.65558, 0.66017, 0.66482, 0.66960, 0.67452, 0.67951, 0.68448, 0.68928,\
        0.69362, 0.69656, 0.69900, 0.70171, 0.70422, 0.70604, 0.70657, 0.70635, 0.70610, 0.70592, 0.70518,\
        0.70420, 0.70383, 0.70259, 0.70135, 0.70078, 0.69945, 0.69801, 0.69666, 0.69515, 0.69297, 0.68981,\
        0.68591, 0.68166, 0.67721, 0.67258, 0.66775, 0.66273, 0.65753, 0.65216, 0.64663, 0.64096, 0.63519,\
        0.62938, 0.62361, 0.61799, 0.61259, 0.60739, 0.60229, 0.59742, 0.59275, 0.58810, 0.58361, 0.57934,\
        0.57525, 0.57123, 0.56717, 0.56297, 0.55852, 0.55372, 0.54852, 0.54284, 0.53665, 0.52996, 0.52279,\
        0.51526, 0.50756, 0.50004, 0.49309, 0.48674, 0.48044, 0.47390, 0.46704, 0.45981, 0.45217, 0.44410],\
        [0.44340, 0.45068, 0.45758, 0.46407, 0.47014, 0.47577, 0.48098, 0.48576, 0.49011, 0.49389, 0.49677,\
        0.49972, 0.50316, 0.50647, 0.50934, 0.51173, 0.51371, 0.51536, 0.51684, 0.51833, 0.51997, 0.52188,\
        0.52410, 0.52667, 0.52958, 0.53284, 0.53646, 0.54043, 0.54479, 0.54954, 0.55472, 0.56032, 0.56636,\
        0.57280, 0.57961, 0.58673, 0.59397, 0.60106, 0.60800, 0.61486, 0.62155, 0.62794, 0.63399, 0.63967,\
        0.64503, 0.65013, 0.65504, 0.65983, 0.66462, 0.66947, 0.67447, 0.67965, 0.68493, 0.69012, 0.69509,\
        0.69983, 0.70310, 0.70553, 0.70815, 0.71050, 0.71219, 0.71301, 0.71316, 0.71303, 0.71271, 0.71208,\
        0.71130, 0.71053, 0.70947, 0.70841, 0.70750, 0.70624, 0.70479, 0.70338, 0.70182, 0.69956, 0.69605,\
        0.69175, 0.68732, 0.68282, 0.67812, 0.67321, 0.66811, 0.66286, 0.65746, 0.65192, 0.64623, 0.64044,\
        0.63460, 0.62876, 0.62300, 0.61738, 0.61190, 0.60660, 0.60156, 0.59672, 0.59172, 0.58713, 0.58292,\
        0.57894, 0.57505, 0.57113, 0.56706, 0.56272, 0.55800, 0.55280, 0.54708, 0.54079, 0.53394, 0.52660,\
        0.51886, 0.51091, 0.50305, 0.49624, 0.49023, 0.48414, 0.47777, 0.47108, 0.46401, 0.45654, 0.44863],\
        [0.44852, 0.45573, 0.46256, 0.46896, 0.47492, 0.48042, 0.48547, 0.49008, 0.49429, 0.49820, 0.50086,\
        0.50339, 0.50659, 0.50972, 0.51254, 0.51496, 0.51702, 0.51881, 0.52043, 0.52202, 0.52371, 0.52560,\
        0.52775, 0.53020, 0.53297, 0.53608, 0.53954, 0.54337, 0.54759, 0.55221, 0.55725, 0.56272, 0.56863,\
        0.57497, 0.58172, 0.58884, 0.59626, 0.60351, 0.61049, 0.61746, 0.62430, 0.63089, 0.63716, 0.64309,\
        0.64869, 0.65401, 0.65912, 0.66409, 0.66903, 0.67402, 0.67917, 0.68455, 0.69012, 0.69557, 0.70050,\
        0.70518, 0.70900, 0.71196, 0.71458, 0.71683, 0.71851, 0.71952, 0.71995, 0.71998, 0.71973, 0.71921,\
        0.71851, 0.71768, 0.71670, 0.71565, 0.71454, 0.71323, 0.71174, 0.71011, 0.70816, 0.70551, 0.70176,\
        0.69721, 0.69289, 0.68846, 0.68370, 0.67871, 0.67356, 0.66829, 0.66290, 0.65736, 0.65168, 0.64587,\
        0.63996, 0.63402, 0.62809, 0.62224, 0.61649, 0.61092, 0.60556, 0.60023, 0.59505, 0.59057, 0.58653,\
        0.58271, 0.57899, 0.57525, 0.57137, 0.56719, 0.56259, 0.55747, 0.55176, 0.54545, 0.53855, 0.53116,\
        0.52339, 0.51549, 0.50785, 0.50104, 0.49481, 0.48856, 0.48212, 0.47544, 0.46844, 0.46106, 0.45324],\
        [0.45345, 0.46062, 0.46739, 0.47373, 0.47960, 0.48497, 0.48988, 0.49432, 0.49825, 0.50189, 0.50498,\
        0.50764, 0.51050, 0.51338, 0.51606, 0.51844, 0.52053, 0.52237, 0.52407, 0.52571, 0.52742, 0.52927,\
        0.53134, 0.53366, 0.53629, 0.53923, 0.54252, 0.54617, 0.55022, 0.55467, 0.55954, 0.56485, 0.57061,\
        0.57680, 0.58340, 0.59040, 0.59769, 0.60518, 0.61260, 0.61991, 0.62705, 0.63392, 0.64047, 0.64666,\
        0.65250, 0.65804, 0.66335, 0.66850, 0.67355, 0.67862, 0.68385, 0.68934, 0.69508, 0.70099, 0.70612,\
        0.71081, 0.71486, 0.71820, 0.72098, 0.72326, 0.72499, 0.72615, 0.72678, 0.72699, 0.72687, 0.72647,\
        0.72585, 0.72506, 0.72410, 0.72301, 0.72178, 0.72035, 0.71869, 0.71674, 0.71436, 0.71129, 0.70736,\
        0.70292, 0.69874, 0.69425, 0.68935, 0.68425, 0.67907, 0.67382, 0.66846, 0.66295, 0.65727, 0.65144,\
        0.64546, 0.63939, 0.63328, 0.62718, 0.62115, 0.61522, 0.60938, 0.60361, 0.59847, 0.59422, 0.59031,\
        0.58661, 0.58305, 0.57953, 0.57588, 0.57191, 0.56750, 0.56251, 0.55689, 0.55061, 0.54373, 0.53636,\
        0.52868, 0.52095, 0.51348, 0.50656, 0.50000, 0.49346, 0.48682, 0.48006, 0.47306, 0.46571, 0.45794],\
        [0.45820, 0.46532, 0.47207, 0.47838, 0.48419, 0.48944, 0.49421, 0.49861, 0.50243, 0.50590, 0.50905,\
        0.51189, 0.51465, 0.51734, 0.51987, 0.52217, 0.52422, 0.52607, 0.52778, 0.52943, 0.53112, 0.53290,\
        0.53487, 0.53706, 0.53952, 0.54228, 0.54538, 0.54883, 0.55268, 0.55693, 0.56162, 0.56675, 0.57233,\
        0.57836, 0.58482, 0.59169, 0.59897, 0.60669, 0.61455, 0.62231, 0.62985, 0.63709, 0.64395, 0.65042,\
        0.65652, 0.66230, 0.66784, 0.67320, 0.67839, 0.68349, 0.68875, 0.69440, 0.70013, 0.70622, 0.71182,\
        0.71672, 0.72095, 0.72450, 0.72743, 0.72980, 0.73162, 0.73290, 0.73370, 0.73407, 0.73409, 0.73382,\
        0.73329, 0.73254, 0.73160, 0.73046, 0.72911, 0.72752, 0.72564, 0.72340, 0.72067, 0.71734, 0.71341,\
        0.70914, 0.70478, 0.70006, 0.69502, 0.68981, 0.68463, 0.67945, 0.67414, 0.66867, 0.66301, 0.65714,\
        0.65108, 0.64487, 0.63856, 0.63223, 0.62592, 0.61970, 0.61362, 0.60782, 0.60268, 0.59828, 0.59433,\
        0.59062, 0.58722, 0.58395, 0.58057, 0.57687, 0.57269, 0.56788, 0.56237, 0.55616, 0.54932, 0.54200,\
        0.53440, 0.52675, 0.51930, 0.51218, 0.50532, 0.49849, 0.49166, 0.48481, 0.47780, 0.47046, 0.46268],\
        [0.46274, 0.46983, 0.47656, 0.48290, 0.48873, 0.49388, 0.49846, 0.50277, 0.50665, 0.51012, 0.51328,\
        0.51618, 0.51891, 0.52149, 0.52390, 0.52610, 0.52810, 0.52991, 0.53158, 0.53320, 0.53481, 0.53651,\
        0.53835, 0.54039, 0.54267, 0.54523, 0.54812, 0.55136, 0.55499, 0.55903, 0.56351, 0.56846, 0.57388,\
        0.57977, 0.58614, 0.59297, 0.60036, 0.60836, 0.61660, 0.62480, 0.63278, 0.64042, 0.64765, 0.65444,\
        0.66081, 0.66685, 0.67266, 0.67833, 0.68381, 0.68896, 0.69409, 0.70013, 0.70626, 0.71232, 0.71801,\
        0.72305, 0.72737, 0.73102, 0.73404, 0.73648, 0.73837, 0.73977, 0.74070, 0.74121, 0.74137, 0.74121,\
        0.74077, 0.74007, 0.73913, 0.73794, 0.73649, 0.73474, 0.73264, 0.73014, 0.72717, 0.72367, 0.71969,\
        0.71537, 0.71078, 0.70589, 0.70069, 0.69536, 0.69031, 0.68522, 0.67997, 0.67454, 0.66889, 0.66298,\
        0.65682, 0.65044, 0.64393, 0.63736, 0.63083, 0.62441, 0.61820, 0.61238, 0.60715, 0.60258, 0.59844,\
        0.59459, 0.59145, 0.58848, 0.58540, 0.58199, 0.57808, 0.57348, 0.56810, 0.56195, 0.55513, 0.54782,\
        0.54022, 0.53257, 0.52503, 0.51770, 0.51055, 0.50348, 0.49650, 0.48961, 0.48260, 0.47526, 0.46743],\
        [0.46707, 0.47412, 0.48083, 0.48719, 0.49317, 0.49839, 0.50281, 0.50701, 0.51088, 0.51440, 0.51760,\
        0.52053, 0.52325, 0.52577, 0.52809, 0.53021, 0.53213, 0.53387, 0.53548, 0.53701, 0.53853, 0.54011,\
        0.54180, 0.54366, 0.54575, 0.54810, 0.55076, 0.55377, 0.55716, 0.56097, 0.56523, 0.56999, 0.57526,\
        0.58107, 0.58744, 0.59437, 0.60194, 0.61021, 0.61880, 0.62742, 0.63585, 0.64394, 0.65158, 0.65873,\
        0.66540, 0.67171, 0.67783, 0.68395, 0.69002, 0.69564, 0.70055, 0.70669, 0.71320, 0.71933, 0.72494,\
        0.72991, 0.73420, 0.73782, 0.74084, 0.74330, 0.74525, 0.74672, 0.74776, 0.74839, 0.74867, 0.74861,\
        0.74826, 0.74760, 0.74666, 0.74543, 0.74388, 0.74198, 0.73970, 0.73699, 0.73382, 0.73017, 0.72609,\
        0.72165, 0.71689, 0.71183, 0.70651, 0.70119, 0.69629, 0.69117, 0.68589, 0.68051, 0.67488, 0.66892,\
        0.66263, 0.65607, 0.64934, 0.64255, 0.63581, 0.62922, 0.62290, 0.61699, 0.61160, 0.60673, 0.60223,\
        0.59857, 0.59579, 0.59308, 0.59025, 0.58716, 0.58356, 0.57919, 0.57393, 0.56779, 0.56093, 0.55355,\
        0.54589, 0.53815, 0.53049, 0.52300, 0.51566, 0.50844, 0.50136, 0.49441, 0.48742, 0.48003, 0.47210],\
        [0.47121, 0.47821, 0.48483, 0.49109, 0.49708, 0.50257, 0.50709, 0.51126, 0.51514, 0.51871, 0.52197,\
        0.52494, 0.52767, 0.53016, 0.53242, 0.53446, 0.53629, 0.53795, 0.53946, 0.54088, 0.54228, 0.54371,\
        0.54523, 0.54690, 0.54877, 0.55089, 0.55331, 0.55607, 0.55920, 0.56277, 0.56680, 0.57136, 0.57648,\
        0.58223, 0.58865, 0.59581, 0.60366, 0.61215, 0.62107, 0.63012, 0.63903, 0.64761, 0.65572, 0.66329,\
        0.67032, 0.67691, 0.68330, 0.68989, 0.69686, 0.70326, 0.70902, 0.71503, 0.72129, 0.72722, 0.73259,\
        0.73733, 0.74142, 0.74491, 0.74784, 0.75027, 0.75222, 0.75375, 0.75486, 0.75559, 0.75596, 0.75600,\
        0.75572, 0.75511, 0.75417, 0.75290, 0.75127, 0.74925, 0.74681, 0.74392, 0.74058, 0.73678, 0.73258,\
        0.72801, 0.72313, 0.71800, 0.71271, 0.70750, 0.70243, 0.69711, 0.69185, 0.68656, 0.68095, 0.67491,\
        0.66846, 0.66167, 0.65470, 0.64769, 0.64075, 0.63401, 0.62757, 0.62154, 0.61599, 0.61096, 0.60651,\
        0.60299, 0.60020, 0.59757, 0.59491, 0.59218, 0.58897, 0.58485, 0.57965, 0.57345, 0.56646, 0.55895,\
        0.55117, 0.54331, 0.53553, 0.52791, 0.52048, 0.51322, 0.50613, 0.49918, 0.49218, 0.48468, 0.47657],\
        [0.47515, 0.48213, 0.48870, 0.49476, 0.50047, 0.50607, 0.51099, 0.51537, 0.51937, 0.52303, 0.52637,\
        0.52941, 0.53216, 0.53463, 0.53685, 0.53883, 0.54058, 0.54213, 0.54353, 0.54482, 0.54607, 0.54733,\
        0.54866, 0.55012, 0.55176, 0.55363, 0.55579, 0.55828, 0.56114, 0.56443, 0.56821, 0.57254, 0.57749,\
        0.58314, 0.58958, 0.59686, 0.60500, 0.61386, 0.62321, 0.63274, 0.64218, 0.65130, 0.65995, 0.66806,\
        0.67556, 0.68253, 0.68922, 0.69608, 0.70384, 0.71133, 0.71802, 0.72431, 0.73031, 0.73587, 0.74083,\
        0.74519, 0.74896, 0.75220, 0.75497, 0.75731, 0.75925, 0.76079, 0.76196, 0.76276, 0.76322, 0.76334,\
        0.76312, 0.76255, 0.76162, 0.76032, 0.75862, 0.75650, 0.75393, 0.75089, 0.74739, 0.74344, 0.73910,\
        0.73442, 0.72945, 0.72428, 0.71900, 0.71372, 0.70843, 0.70305, 0.69788, 0.69269, 0.68703, 0.68088,\
        0.67420, 0.66714, 0.65991, 0.65267, 0.64555, 0.63867, 0.63210, 0.62595, 0.62029, 0.61519, 0.61078,\
        0.60717, 0.60423, 0.60163, 0.59909, 0.59687, 0.59413, 0.59022, 0.58501, 0.57864, 0.57145, 0.56375,\
        0.55581, 0.54782, 0.53993, 0.53223, 0.52477, 0.51755, 0.51056, 0.50371, 0.49668, 0.48899, 0.48065],\
        [0.47890, 0.48591, 0.49256, 0.49866, 0.50429, 0.50975, 0.51482, 0.51940, 0.52357, 0.52737, 0.53081,\
        0.53392, 0.53672, 0.53920, 0.54139, 0.54331, 0.54497, 0.54642, 0.54769, 0.54883, 0.54991, 0.55098,\
        0.55210, 0.55332, 0.55472, 0.55633, 0.55821, 0.56041, 0.56298, 0.56598, 0.56947, 0.57353, 0.57825,\
        0.58374, 0.59012, 0.59747, 0.60587, 0.61514, 0.62500, 0.63508, 0.64508, 0.65477, 0.66403, 0.67281,\
        0.68102, 0.68861, 0.69593, 0.70361, 0.71198, 0.72019, 0.72755, 0.73408, 0.73988, 0.74500, 0.74946,\
        0.75331, 0.75666, 0.75958, 0.76214, 0.76437, 0.76627, 0.76782, 0.76902, 0.76988, 0.77041, 0.77060,\
        0.77044, 0.76991, 0.76898, 0.76765, 0.76590, 0.76370, 0.76102, 0.75784, 0.75418, 0.75009, 0.74562,\
        0.74082, 0.73577, 0.73053, 0.72520, 0.71983, 0.71446, 0.70913, 0.70397, 0.69866, 0.69291, 0.68668,\
        0.67970, 0.67231, 0.66481, 0.65737, 0.65011, 0.64310, 0.63642, 0.63014, 0.62436, 0.61916, 0.61463,\
        0.61084, 0.60771, 0.60507, 0.60283, 0.60103, 0.59863, 0.59495, 0.58965, 0.58302, 0.57558, 0.56770,\
        0.55962, 0.55152, 0.54354, 0.53578, 0.52829, 0.52109, 0.51417, 0.50739, 0.50040, 0.49254, 0.48408],\
        [0.48240, 0.48946, 0.49626, 0.50249, 0.50817, 0.51358, 0.51870, 0.52344, 0.52778, 0.53173, 0.53530,\
        0.53850, 0.54135, 0.54386, 0.54603, 0.54790, 0.54948, 0.55081, 0.55194, 0.55292, 0.55381, 0.55466,\
        0.55556, 0.55654, 0.55767, 0.55901, 0.56060, 0.56250, 0.56475, 0.56743, 0.57060, 0.57433, 0.57875,\
        0.58398, 0.59021, 0.59761, 0.60617, 0.61584, 0.62625, 0.63691, 0.64746, 0.65768, 0.66752, 0.67705,\
        0.68629, 0.69494, 0.70323, 0.71209, 0.72119, 0.72987, 0.73758, 0.74417, 0.74970, 0.75432, 0.75819,\
        0.76147, 0.76433, 0.76689, 0.76922, 0.77135, 0.77322, 0.77479, 0.77602, 0.77691, 0.77749, 0.77775,\
        0.77765, 0.77714, 0.77620, 0.77485, 0.77307, 0.77082, 0.76803, 0.76472, 0.76091, 0.75666, 0.75205,\
        0.74713, 0.74197, 0.73664, 0.73122, 0.72575, 0.72030, 0.71490, 0.70956, 0.70408, 0.69849, 0.69213,\
        0.68466, 0.67694, 0.66925, 0.66170, 0.65434, 0.64724, 0.64045, 0.63404, 0.62809, 0.62270, 0.61794,\
        0.61387, 0.61049, 0.60776, 0.60563, 0.60389, 0.60186, 0.59858, 0.59304, 0.58615, 0.57855, 0.57056,\
        0.56242, 0.55427, 0.54623, 0.53840, 0.53086, 0.52362, 0.51669, 0.50988, 0.50280, 0.49519, 0.48670],\
        [0.48562, 0.49260, 0.49954, 0.50587, 0.51177, 0.51735, 0.52261, 0.52752, 0.53203, 0.53613, 0.53984,\
        0.54315, 0.54607, 0.54861, 0.55078, 0.55260, 0.55409, 0.55530, 0.55628, 0.55708, 0.55777, 0.55840,\
        0.55906, 0.55978, 0.56064, 0.56169, 0.56297, 0.56455, 0.56648, 0.56881, 0.57161, 0.57497, 0.57900,\
        0.58385, 0.58972, 0.59687, 0.60559, 0.61577, 0.62678, 0.63798, 0.64897, 0.65958, 0.66985, 0.67995,\
        0.69024, 0.70061, 0.71016, 0.72037, 0.73053, 0.73983, 0.74779, 0.75429, 0.75946, 0.76352, 0.76675,\
        0.76941, 0.77175, 0.77395, 0.77609, 0.77816, 0.78006, 0.78169, 0.78294, 0.78383, 0.78443, 0.78475,\
        0.78473, 0.78422, 0.78322, 0.78183, 0.78007, 0.77780, 0.77492, 0.77146, 0.76749, 0.76308, 0.75832,\
        0.75326, 0.74798, 0.74253, 0.73698, 0.73138, 0.72577, 0.72017, 0.71458, 0.70896, 0.70327, 0.69633,\
        0.68864, 0.68087, 0.67316, 0.66560, 0.65823, 0.65109, 0.64420, 0.63763, 0.63146, 0.62577, 0.62066,\
        0.61619, 0.61243, 0.60937, 0.60697, 0.60503, 0.60305, 0.59991, 0.59445, 0.58763, 0.58012, 0.57222,\
        0.56414, 0.55601, 0.54795, 0.54006, 0.53240, 0.52504, 0.51799, 0.51119, 0.50448, 0.49707, 0.48826],\
        [0.48868, 0.49534, 0.50224, 0.50893, 0.51519, 0.52106, 0.52655, 0.53165, 0.53634, 0.54060, 0.54445,\
        0.54787, 0.55088, 0.55346, 0.55563, 0.55741, 0.55882, 0.55990, 0.56072, 0.56133, 0.56180, 0.56220,\
        0.56261, 0.56307, 0.56364, 0.56439, 0.56537, 0.56661, 0.56819, 0.57014, 0.57255, 0.57549, 0.57907,\
        0.58344, 0.58882, 0.59555, 0.60425, 0.61489, 0.62643, 0.63799, 0.64919, 0.65996, 0.67043, 0.68076,\
        0.69116, 0.70237, 0.71497, 0.72759, 0.73933, 0.74949, 0.75772, 0.76406, 0.76879, 0.77225, 0.77484,\
        0.77688, 0.77870, 0.78058, 0.78262, 0.78474, 0.78675, 0.78850, 0.78980, 0.79063, 0.79120, 0.79157,\
        0.79165, 0.79112, 0.78993, 0.78849, 0.78685, 0.78461, 0.78160, 0.77798, 0.77384, 0.76927, 0.76435,\
        0.75916, 0.75374, 0.74816, 0.74245, 0.73667, 0.73082, 0.72493, 0.71896, 0.71283, 0.70629, 0.69912,\
        0.69175, 0.68413, 0.67657, 0.66913, 0.66184, 0.65468, 0.64769, 0.64093, 0.63447, 0.62842, 0.62284,\
        0.61785, 0.61350, 0.60987, 0.60691, 0.60442, 0.60199, 0.59874, 0.59376, 0.58745, 0.58033, 0.57273,\
        0.56484, 0.55681, 0.54877, 0.54081, 0.53299, 0.52539, 0.51806, 0.51102, 0.50410, 0.49668, 0.48829],\
        [0.49205, 0.49831, 0.50518, 0.51215, 0.51873, 0.52487, 0.53058, 0.53587, 0.54073, 0.54515, 0.54914,\
        0.55268, 0.55577, 0.55841, 0.56059, 0.56233, 0.56365, 0.56460, 0.56525, 0.56566, 0.56591, 0.56608,\
        0.56623, 0.56642, 0.56671, 0.56715, 0.56780, 0.56871, 0.56992, 0.57148, 0.57346, 0.57593, 0.57901,\
        0.58286, 0.58771, 0.59395, 0.60235, 0.61322, 0.62497, 0.63648, 0.64747, 0.65810, 0.66867, 0.67947,\
        0.69061, 0.70308, 0.71844, 0.73356, 0.74711, 0.75828, 0.76681, 0.77296, 0.77723, 0.78014, 0.78215,\
        0.78365, 0.78499, 0.78662, 0.78875, 0.79108, 0.79328, 0.79527, 0.79662, 0.79735, 0.79784, 0.79821,\
        0.79834, 0.79780, 0.79616, 0.79468, 0.79340, 0.79116, 0.78796, 0.78418, 0.77989, 0.77517, 0.77010,\
        0.76476, 0.75922, 0.75349, 0.74762, 0.74162, 0.73550, 0.72926, 0.72285, 0.71620, 0.70922, 0.70201,\
        0.69447, 0.68697, 0.67965, 0.67244, 0.66526, 0.65811, 0.65100, 0.64402, 0.63723, 0.63073, 0.62461,\
        0.61897, 0.61390, 0.60949, 0.60582, 0.60268, 0.59961, 0.59630, 0.59179, 0.58608, 0.57950, 0.57229,\
        0.56469, 0.55683, 0.54884, 0.54082, 0.53283, 0.52493, 0.51717, 0.50963, 0.50234, 0.49499, 0.48719],\
        [0.49621, 0.50242, 0.50910, 0.51598, 0.52263, 0.52891, 0.53477, 0.54022, 0.54522, 0.54978, 0.55390,\
        0.55756, 0.56074, 0.56345, 0.56565, 0.56736, 0.56860, 0.56941, 0.56988, 0.57008, 0.57011, 0.57004,\
        0.56994, 0.56986, 0.56987, 0.57000, 0.57033, 0.57088, 0.57170, 0.57284, 0.57436, 0.57632, 0.57884,\
        0.58210, 0.58638, 0.59210, 0.59995, 0.61061, 0.62191, 0.63262, 0.64278, 0.65284, 0.66335, 0.67504,\
        0.68868, 0.70402, 0.72028, 0.73759, 0.75323, 0.76555, 0.77438, 0.78033, 0.78421, 0.78674, 0.78843,\
        0.78960, 0.79055, 0.79199, 0.79447, 0.79729, 0.79966, 0.80190, 0.80340, 0.80414, 0.80456, 0.80477,\
        0.80467, 0.80384, 0.80204, 0.80054, 0.79977, 0.79718, 0.79381, 0.78997, 0.78557, 0.78070, 0.77550,\
        0.77005, 0.76439, 0.75853, 0.75249, 0.74628, 0.73989, 0.73332, 0.72651, 0.71943, 0.71204, 0.70437,\
        0.69677, 0.68966, 0.68269, 0.67572, 0.66866, 0.66150, 0.65425, 0.64699, 0.63983, 0.63285, 0.62615,\
        0.61982, 0.61394, 0.60864, 0.60412, 0.60066, 0.59728, 0.59370, 0.58941, 0.58416, 0.57805, 0.57126,\
        0.56396, 0.55630, 0.54840, 0.54035, 0.53222, 0.52405, 0.51589, 0.50779, 0.50007, 0.49305, 0.48577],\
        [0.50084, 0.50718, 0.51368, 0.52034, 0.52691, 0.53321, 0.53915, 0.54469, 0.54981, 0.55449, 0.55872,\
        0.56248, 0.56577, 0.56856, 0.57081, 0.57250, 0.57365, 0.57431, 0.57459, 0.57459, 0.57440, 0.57411,\
        0.57377, 0.57343, 0.57315, 0.57298, 0.57298, 0.57317, 0.57359, 0.57429, 0.57529, 0.57665, 0.57849,\
        0.58099, 0.58449, 0.58950, 0.59673, 0.60621, 0.61603, 0.62492, 0.63357, 0.64263, 0.65252, 0.66432,\
        0.67912, 0.69643, 0.71647, 0.73805, 0.75673, 0.77037, 0.77949, 0.78536, 0.78909, 0.79157, 0.79340,\
        0.79478, 0.79559, 0.79685, 0.79981, 0.80345, 0.80629, 0.80861, 0.81020, 0.81104, 0.81137, 0.81131,\
        0.81082, 0.80972, 0.80811, 0.80653, 0.80498, 0.80241, 0.79914, 0.79537, 0.79084, 0.78581, 0.78053,\
        0.77502, 0.76927, 0.76330, 0.75711, 0.75071, 0.74410, 0.73726, 0.73016, 0.72278, 0.71512, 0.70730,\
        0.69973, 0.69273, 0.68598, 0.67921, 0.67221, 0.66496, 0.65751, 0.64994, 0.64238, 0.63492, 0.62765,\
        0.62067, 0.61406, 0.60791, 0.60241, 0.59829, 0.59499, 0.59137, 0.58717, 0.58219, 0.57642, 0.56995,\
        0.56292, 0.55546, 0.54767, 0.53965, 0.53146, 0.52318, 0.51486, 0.50656, 0.49871, 0.49175, 0.48468],\
        [0.50539, 0.51191, 0.51842, 0.52497, 0.53144, 0.53771, 0.54367, 0.54927, 0.55446, 0.55923, 0.56355,\
        0.56742, 0.57082, 0.57370, 0.57602, 0.57772, 0.57879, 0.57930, 0.57938, 0.57917, 0.57879, 0.57830,\
        0.57775, 0.57717, 0.57662, 0.57615, 0.57581, 0.57564, 0.57566, 0.57587, 0.57630, 0.57697, 0.57793,\
        0.57933, 0.58147, 0.58483, 0.59002, 0.59687, 0.60513, 0.61072, 0.61786, 0.62586, 0.63449, 0.64527,\
        0.65963, 0.67914, 0.70494, 0.73390, 0.75646, 0.77134, 0.78090, 0.78710, 0.79122, 0.79418, 0.79673,\
        0.79916, 0.80072, 0.80243, 0.80604, 0.81003, 0.81335, 0.81581, 0.81739, 0.81816, 0.81829, 0.81790,\
        0.81705, 0.81572, 0.81403, 0.81218, 0.81007, 0.80738, 0.80412, 0.80033, 0.79552, 0.79045, 0.78520,\
        0.77969, 0.77389, 0.76783, 0.76152, 0.75497, 0.74819, 0.74117, 0.73390, 0.72639, 0.71869, 0.71094,\
        0.70340, 0.69634, 0.68975, 0.68306, 0.67601, 0.66857, 0.66083, 0.65291, 0.64494, 0.63702, 0.62927,\
        0.62176, 0.61460, 0.60792, 0.60202, 0.59728, 0.59344, 0.58962, 0.58538, 0.58050, 0.57491, 0.56867,\
        0.56185, 0.55455, 0.54688, 0.53893, 0.53078, 0.52253, 0.51432, 0.50635, 0.49877, 0.49142, 0.48412],\
        [0.50992, 0.51656, 0.52313, 0.52965, 0.53607, 0.54231, 0.54827, 0.55389, 0.55914, 0.56396, 0.56836,\
        0.57231, 0.57581, 0.57882, 0.58125, 0.58300, 0.58399, 0.58432, 0.58420, 0.58382, 0.58328, 0.58265,\
        0.58192, 0.58113, 0.58033, 0.57957, 0.57891, 0.57838, 0.57798, 0.57770, 0.57751, 0.57738, 0.57726,\
        0.57719, 0.57724, 0.57762, 0.57867, 0.58084, 0.58404, 0.58720, 0.59369, 0.60111, 0.60849, 0.61780,\
        0.63078, 0.65137, 0.68550, 0.72425, 0.74999, 0.76616, 0.77702, 0.78469, 0.79024, 0.79446, 0.79819,\
        0.80222, 0.80593, 0.80954, 0.81371, 0.81787, 0.82131, 0.82373, 0.82514, 0.82564, 0.82542, 0.82463,\
        0.82338, 0.82173, 0.81978, 0.81757, 0.81506, 0.81210, 0.80861, 0.80446, 0.79963, 0.79478, 0.78963,\
        0.78412, 0.77828, 0.77215, 0.76575, 0.75909, 0.75219, 0.74506, 0.73772, 0.73021, 0.72259, 0.71500,\
        0.70760, 0.70061, 0.69412, 0.68730, 0.68003, 0.67229, 0.66419, 0.65588, 0.64751, 0.63920, 0.63105,\
        0.62316, 0.61564, 0.60864, 0.60242, 0.59721, 0.59284, 0.58867, 0.58423, 0.57929, 0.57375, 0.56761,\
        0.56092, 0.55376, 0.54620, 0.53834, 0.53027, 0.52210, 0.51401, 0.50625, 0.49887, 0.49130, 0.48386],\
        [0.51438, 0.52111, 0.52774, 0.53428, 0.54069, 0.54691, 0.55287, 0.55851, 0.56378, 0.56864, 0.57307,\
        0.57707, 0.58065, 0.58380, 0.58642, 0.58829, 0.58921, 0.58933, 0.58902, 0.58850, 0.58789, 0.58720,\
        0.58637, 0.58540, 0.58437, 0.58335, 0.58238, 0.58149, 0.58068, 0.57990, 0.57908, 0.57810, 0.57681,\
        0.57505, 0.57260, 0.56925, 0.56491, 0.55995, 0.55586, 0.55582, 0.56114, 0.56836, 0.57554, 0.58289,\
        0.59276, 0.60991, 0.64933, 0.70025, 0.73181, 0.75204, 0.76660, 0.77778, 0.78638, 0.79308, 0.79894,\
        0.80534, 0.81161, 0.81730, 0.82248, 0.82696, 0.83038, 0.83257, 0.83357, 0.83359, 0.83284, 0.83154,\
        0.82980, 0.82774, 0.82541, 0.82282, 0.81993, 0.81665, 0.81290, 0.80862, 0.80394, 0.79913, 0.79393,\
        0.78837, 0.78248, 0.77629, 0.76981, 0.76308, 0.75610, 0.74892, 0.74156, 0.73410, 0.72662, 0.71925,\
        0.71213, 0.70537, 0.69869, 0.69169, 0.68412, 0.67599, 0.66747, 0.65877, 0.65004, 0.64140, 0.63297,\
        0.62484, 0.61710, 0.60990, 0.60337, 0.59781, 0.59307, 0.58850, 0.58377, 0.57864, 0.57302, 0.56689,\
        0.56027, 0.55320, 0.54574, 0.53799, 0.53001, 0.52191, 0.51379, 0.50577, 0.49813, 0.49101, 0.48376],\
        [0.51872, 0.52553, 0.53221, 0.53878, 0.54520, 0.55143, 0.55740, 0.56306, 0.56835, 0.57322, 0.57765,\
        0.58165, 0.58524, 0.58848, 0.59134, 0.59349, 0.59434, 0.59421, 0.59379, 0.59323, 0.59263, 0.59204,\
        0.59119, 0.59007, 0.58885, 0.58760, 0.58634, 0.58511, 0.58390, 0.58266, 0.58123, 0.57942, 0.57701,\
        0.57366, 0.56887, 0.56203, 0.55254, 0.54031, 0.52748, 0.52233, 0.52733, 0.53349, 0.54053, 0.54473,\
        0.55005, 0.56505, 0.60340, 0.65851, 0.70078, 0.72873, 0.74998, 0.76712, 0.78056, 0.79111, 0.80030,\
        0.80963, 0.81841, 0.82604, 0.83237, 0.83727, 0.84062, 0.84238, 0.84275, 0.84205, 0.84060, 0.83864,\
        0.83633, 0.83376, 0.83098, 0.82798, 0.82472, 0.82114, 0.81719, 0.81285, 0.80820, 0.80330, 0.79804,\
        0.79244, 0.78650, 0.78025, 0.77371, 0.76691, 0.75988, 0.75265, 0.74529, 0.73787, 0.73050, 0.72329,\
        0.71634, 0.70966, 0.70298, 0.69590, 0.68798, 0.67939, 0.67045, 0.66140, 0.65239, 0.64354, 0.63493,\
        0.62667, 0.61885, 0.61159, 0.60501, 0.59921, 0.59401, 0.58902, 0.58391, 0.57852, 0.57275, 0.56656,\
        0.55994, 0.55293, 0.54557, 0.53791, 0.53004, 0.52204, 0.51401, 0.50607, 0.49851, 0.49127, 0.48399],\
        [0.52290, 0.52976, 0.53650, 0.54310, 0.54955, 0.55579, 0.56179, 0.56747, 0.57278, 0.57765, 0.58206,\
        0.58599, 0.58950, 0.59269, 0.59567, 0.59831, 0.59913, 0.59882, 0.59854, 0.59809, 0.59755, 0.59731,\
        0.59646, 0.59522, 0.59390, 0.59249, 0.59097, 0.58941, 0.58785, 0.58619, 0.58418, 0.58162, 0.57826,\
        0.57366, 0.56716, 0.55788, 0.54497, 0.52797, 0.50812, 0.49612, 0.49934, 0.50069, 0.50935, 0.50860,\
        0.51005, 0.52624, 0.56440, 0.61949, 0.66524, 0.70031, 0.72982, 0.75466, 0.77408, 0.78937, 0.80266,\
        0.81534, 0.82667, 0.83612, 0.84355, 0.84891, 0.85211, 0.85323, 0.85269, 0.85102, 0.84867, 0.84593,\
        0.84295, 0.83981, 0.83653, 0.83310, 0.82948, 0.82561, 0.82145, 0.81700, 0.81227, 0.80727, 0.80195,\
        0.79631, 0.79032, 0.78401, 0.77742, 0.77057, 0.76348, 0.75620, 0.74880, 0.74136, 0.73399, 0.72680,\
        0.71987, 0.71318, 0.70655, 0.69946, 0.69114, 0.68213, 0.67288, 0.66361, 0.65444, 0.64548, 0.63681,\
        0.62852, 0.62070, 0.61345, 0.60685, 0.60088, 0.59538, 0.59001, 0.58451, 0.57882, 0.57286, 0.56659,\
        0.55996, 0.55299, 0.54569, 0.53814, 0.53038, 0.52252, 0.51466, 0.50697, 0.49952, 0.49199, 0.48454],\
        [0.52688, 0.53378, 0.54055, 0.54719, 0.55366, 0.55994, 0.56596, 0.57168, 0.57702, 0.58192, 0.58631,\
        0.59015, 0.59348, 0.59647, 0.59928, 0.60191, 0.60322, 0.60343, 0.60344, 0.60325, 0.60298, 0.60292,\
        0.60219, 0.60104, 0.59973, 0.59825, 0.59651, 0.59463, 0.59275, 0.59074, 0.58819, 0.58490, 0.58076,\
        0.57533, 0.56777, 0.55711, 0.54253, 0.52366, 0.50165, 0.48221, 0.47462, 0.47387, 0.47626, 0.47267,\
        0.47173, 0.48394, 0.52887, 0.58603, 0.63147, 0.67259, 0.71035, 0.74248, 0.76789, 0.78835, 0.80625,\
        0.82264, 0.83657, 0.84776, 0.85622, 0.86199, 0.86496, 0.86516, 0.86333, 0.86043, 0.85704, 0.85341,\
        0.84967, 0.84590, 0.84209, 0.83821, 0.83421, 0.83004, 0.82565, 0.82103, 0.81618, 0.81106, 0.80567,\
        0.79999, 0.79392, 0.78755, 0.78093, 0.77403, 0.76687, 0.75951, 0.75201, 0.74446, 0.73696, 0.72961,\
        0.72248, 0.71557, 0.70872, 0.70140, 0.69299, 0.68388, 0.67455, 0.66525, 0.65608, 0.64714, 0.63850,\
        0.63024, 0.62246, 0.61524, 0.60865, 0.60264, 0.59701, 0.59123, 0.58534, 0.57940, 0.57329, 0.56694,\
        0.56029, 0.55335, 0.54612, 0.53865, 0.53100, 0.52325, 0.51549, 0.50780, 0.50019, 0.49273, 0.48527],\
        [0.53063, 0.53755, 0.54434, 0.55099, 0.55749, 0.56380, 0.56987, 0.57563, 0.58103, 0.58597, 0.59038,\
        0.59418, 0.59735, 0.60017, 0.60293, 0.60540, 0.60711, 0.60802, 0.60853, 0.60879, 0.60894, 0.60896,\
        0.60855, 0.60771, 0.60655, 0.60508, 0.60323, 0.60112, 0.59893, 0.59662, 0.59348, 0.58936, 0.58457,\
        0.57861, 0.57036, 0.55850, 0.54221, 0.52106, 0.49288, 0.46357, 0.44791, 0.44321, 0.43693, 0.43035,\
        0.42328, 0.43116, 0.47991, 0.55005, 0.59806, 0.64535, 0.69147, 0.73037, 0.76136, 0.78792, 0.81128,\
        0.83157, 0.84821, 0.86112, 0.87048, 0.87663, 0.87934, 0.87815, 0.87455, 0.87022, 0.86570, 0.86109,\
        0.85652, 0.85205, 0.84767, 0.84332, 0.83892, 0.83442, 0.82977, 0.82495, 0.81992, 0.81467, 0.80916,\
        0.80336, 0.79720, 0.79088, 0.78425, 0.77729, 0.77003, 0.76254, 0.75487, 0.74711, 0.73935, 0.73168,\
        0.72415, 0.71679, 0.70948, 0.70194, 0.69353, 0.68455, 0.67541, 0.66629, 0.65728, 0.64847, 0.63993,\
        0.63175, 0.62400, 0.61676, 0.61011, 0.60412, 0.59839, 0.59220, 0.58615, 0.58011, 0.57393, 0.56755,\
        0.56091, 0.55399, 0.54681, 0.53941, 0.53184, 0.52417, 0.51648, 0.50879, 0.50118, 0.49370, 0.48619],\
        [0.53411, 0.54104, 0.54783, 0.55448, 0.56098, 0.56731, 0.57342, 0.57924, 0.58469, 0.58970, 0.59419,\
        0.59805, 0.60120, 0.60399, 0.60670, 0.60916, 0.61116, 0.61264, 0.61373, 0.61456, 0.61518, 0.61556,\
        0.61558, 0.61516, 0.61433, 0.61306, 0.61134, 0.60923, 0.60681, 0.60400, 0.60008, 0.59512, 0.58983,\
        0.58355, 0.57476, 0.56106, 0.54058, 0.51404, 0.47745, 0.43663, 0.41383, 0.40844, 0.39056, 0.37664,\
        0.36309, 0.35695, 0.38816, 0.48112, 0.55501, 0.61249, 0.66836, 0.71404, 0.75142, 0.78682, 0.81692,\
        0.84169, 0.86167, 0.87654, 0.88660, 0.89281, 0.89537, 0.89195, 0.88619, 0.88047, 0.87478, 0.86904,\
        0.86349, 0.85827, 0.85329, 0.84844, 0.84361, 0.83874, 0.83379, 0.82872, 0.82350, 0.81810, 0.81245,\
        0.80653, 0.80037, 0.79414, 0.78740, 0.78034, 0.77296, 0.76529, 0.75739, 0.74934, 0.74122, 0.73311,\
        0.72504, 0.71705, 0.70913, 0.70118, 0.69293, 0.68435, 0.67561, 0.66682, 0.65809, 0.64949, 0.64111,\
        0.63303, 0.62532, 0.61801, 0.61112, 0.60458, 0.59843, 0.59272, 0.58689, 0.58092, 0.57477, 0.56840,\
        0.56177, 0.55487, 0.54773, 0.54037, 0.53285, 0.52524, 0.51761, 0.51000, 0.50245, 0.49492, 0.48727],\
        [0.53732, 0.54422, 0.55098, 0.55760, 0.56409, 0.57041, 0.57655, 0.58242, 0.58794, 0.59298, 0.59750,\
        0.60144, 0.60468, 0.60759, 0.61037, 0.61297, 0.61530, 0.61730, 0.61899, 0.62044, 0.62166, 0.62260,\
        0.62317, 0.62332, 0.62297, 0.62211, 0.62070, 0.61874, 0.61621, 0.61290, 0.60845, 0.60298, 0.59707,\
        0.59049, 0.58160, 0.56639, 0.53848, 0.50018, 0.45788, 0.41393, 0.38240, 0.36329, 0.33753, 0.31938,\
        0.30515, 0.30045, 0.32429, 0.40617, 0.49490, 0.56276, 0.63574, 0.69016, 0.73810, 0.78085, 0.81969,\
        0.85295, 0.87756, 0.89498, 0.90559, 0.91114, 0.91178, 0.90650, 0.89877, 0.89166, 0.88453, 0.87725,\
        0.87058, 0.86458, 0.85898, 0.85357, 0.84825, 0.84296, 0.83767, 0.83233, 0.82691, 0.82135, 0.81561,\
        0.80968, 0.80362, 0.79713, 0.79029, 0.78318, 0.77566, 0.76777, 0.75959, 0.75121, 0.74270, 0.73411,\
        0.72547, 0.71680, 0.70817, 0.69987, 0.69182, 0.68373, 0.67545, 0.66705, 0.65862, 0.65027, 0.64208,\
        0.63413, 0.62647, 0.61913, 0.61212, 0.60542, 0.59927, 0.59356, 0.58780, 0.58189, 0.57580, 0.56946,\
        0.56285, 0.55597, 0.54883, 0.54149, 0.53399, 0.52639, 0.51877, 0.51121, 0.50378, 0.49623, 0.48832],\
        [0.54024, 0.54708, 0.55377, 0.56034, 0.56676, 0.57305, 0.57917, 0.58508, 0.59067, 0.59575, 0.60019,\
        0.60413, 0.60757, 0.61072, 0.61374, 0.61664, 0.61937, 0.62191, 0.62424, 0.62637, 0.62828, 0.62992,\
        0.63120, 0.63204, 0.63235, 0.63206, 0.63112, 0.62949, 0.62709, 0.62375, 0.61932, 0.61383, 0.60741,\
        0.59992, 0.59114, 0.57882, 0.54306, 0.49345, 0.44565, 0.40198, 0.36457, 0.33188, 0.30105, 0.28792,\
        0.27587, 0.27313, 0.30381, 0.38073, 0.45272, 0.52600, 0.60723, 0.67207, 0.72573, 0.77314, 0.82312,\
        0.86927, 0.89772, 0.91724, 0.92812, 0.93195, 0.92995, 0.92309, 0.91374, 0.90438, 0.89480, 0.88552,\
        0.87775, 0.87102, 0.86476, 0.85871, 0.85280, 0.84703, 0.84134, 0.83571, 0.83008, 0.82439, 0.81857,\
        0.81257, 0.80627, 0.79962, 0.79301, 0.78587, 0.77816, 0.76999, 0.76151, 0.75279, 0.74390, 0.73490,\
        0.72580, 0.71663, 0.70744, 0.69867, 0.69096, 0.68322, 0.67527, 0.66716, 0.65901, 0.65089, 0.64290,\
        0.63510, 0.62754, 0.62027, 0.61332, 0.60678, 0.60072, 0.59477, 0.58894, 0.58309, 0.57705, 0.57075,\
        0.56414, 0.55724, 0.55009, 0.54273, 0.53521, 0.52758, 0.51989, 0.51220, 0.50455, 0.49688, 0.48910],\
        [0.54288, 0.54962, 0.55622, 0.56268, 0.56900, 0.57518, 0.58122, 0.58710, 0.59275, 0.59795, 0.60236,\
        0.60623, 0.60984, 0.61331, 0.61670, 0.62001, 0.62325, 0.62637, 0.62937, 0.63223, 0.63492, 0.63737,\
        0.63949, 0.64117, 0.64228, 0.64272, 0.64242, 0.64133, 0.63938, 0.63644, 0.63246, 0.62746, 0.62129,\
        0.61325, 0.60399, 0.59862, 0.55303, 0.49297, 0.44179, 0.39907, 0.35971, 0.32650, 0.30343, 0.29115,\
        0.27730, 0.27203, 0.30904, 0.38015, 0.44807, 0.51530, 0.58615, 0.65269, 0.71281, 0.76859, 0.83081,\
        0.89354, 0.92496, 0.94334, 0.95264, 0.95417, 0.94966, 0.94077, 0.92944, 0.91728, 0.90489, 0.89381,\
        0.88513, 0.87775, 0.87064, 0.86376, 0.85716, 0.85083, 0.84473, 0.83880, 0.83296, 0.82713, 0.82125,\
        0.81523, 0.80901, 0.80258, 0.79585, 0.78843, 0.78039, 0.77192, 0.76312, 0.75409, 0.74490, 0.73561,\
        0.72626, 0.71693, 0.70775, 0.69905, 0.69103, 0.68317, 0.67527, 0.66731, 0.65935, 0.65143, 0.64363,\
        0.63600, 0.62857, 0.62141, 0.61454, 0.60801, 0.60183, 0.59593, 0.59023, 0.58450, 0.57853, 0.57223,\
        0.56559, 0.55865, 0.55145, 0.54405, 0.53649, 0.52880, 0.52102, 0.51315, 0.50513, 0.49734, 0.48988],\
        [0.54523, 0.55187, 0.55833, 0.56464, 0.57080, 0.57680, 0.58267, 0.58839, 0.59394, 0.59918, 0.60357,\
        0.60757, 0.61149, 0.61536, 0.61921, 0.62303, 0.62682, 0.63057, 0.63427, 0.63790, 0.64143, 0.64478,\
        0.64786, 0.65050, 0.65256, 0.65387, 0.65437, 0.65404, 0.65282, 0.65059, 0.64733, 0.64324, 0.63844,\
        0.63279, 0.62624, 0.61077, 0.56096, 0.49383, 0.44217, 0.40112, 0.36300, 0.33360, 0.31699, 0.30901,\
        0.30015, 0.29034, 0.33885, 0.41700, 0.46471, 0.51337, 0.56930, 0.63507, 0.70536, 0.77088, 0.84308,\
        0.90619, 0.94840, 0.96905, 0.97685, 0.97604, 0.96911, 0.95805, 0.94459, 0.93013, 0.91575, 0.90273,\
        0.89338, 0.88483, 0.87641, 0.86851, 0.86113, 0.85422, 0.84770, 0.84147, 0.83543, 0.82950, 0.82357,\
        0.81758, 0.81146, 0.80518, 0.79840, 0.79057, 0.78220, 0.77345, 0.76439, 0.75512, 0.74570, 0.73623,\
        0.72677, 0.71747, 0.70851, 0.69999, 0.69162, 0.68343, 0.67543, 0.66752, 0.65967, 0.65192, 0.64429,\
        0.63683, 0.62956, 0.62253, 0.61575, 0.60923, 0.60301, 0.59721, 0.59174, 0.58618, 0.58025, 0.57389,\
        0.56716, 0.56013, 0.55286, 0.54540, 0.53778, 0.53006, 0.52225, 0.51439, 0.50655, 0.49886, 0.49104],\
        [0.54733, 0.55384, 0.56015, 0.56628, 0.57222, 0.57799, 0.58358, 0.58898, 0.59417, 0.59909, 0.60366,\
        0.60812, 0.61254, 0.61693, 0.62130, 0.62566, 0.63003, 0.63442, 0.63882, 0.64323, 0.64762, 0.65193,\
        0.65605, 0.65980, 0.66295, 0.66527, 0.66667, 0.66726, 0.66705, 0.66575, 0.66325, 0.66002, 0.65651,\
        0.65228, 0.64417, 0.62254, 0.57183, 0.50273, 0.44755, 0.40430, 0.36754, 0.34257, 0.33125, 0.32790,\
        0.32784, 0.33812, 0.38779, 0.44577, 0.47926, 0.51281, 0.55872, 0.62520, 0.70394, 0.77670, 0.86350,\
        0.93369, 0.97259, 0.99222, 0.99841, 0.99554, 0.98656, 0.97359, 0.95836, 0.94229, 0.92669, 0.91297,\
        0.90197, 0.89130, 0.88147, 0.87257, 0.86446, 0.85701, 0.85011, 0.84362, 0.83742, 0.83138, 0.82540,\
        0.81938, 0.81323, 0.80679, 0.79974, 0.79190, 0.78342, 0.77450, 0.76527, 0.75582, 0.74625, 0.73666,\
        0.72715, 0.71784, 0.70889, 0.70035, 0.69187, 0.68361, 0.67558, 0.66770, 0.65995, 0.65234, 0.64487,\
        0.63759, 0.63051, 0.62364, 0.61701, 0.61061, 0.60448, 0.59879, 0.59362, 0.58819, 0.58215, 0.57563,\
        0.56876, 0.56161, 0.55424, 0.54671, 0.53905, 0.53129, 0.52349, 0.51569, 0.50794, 0.50018, 0.49219],\
        [0.54921, 0.55557, 0.56172, 0.56765, 0.57337, 0.57887, 0.58415, 0.58920, 0.59397, 0.59846, 0.60325,\
        0.60830, 0.61326, 0.61816, 0.62303, 0.62793, 0.63286, 0.63785, 0.64293, 0.64809, 0.65332, 0.65859,\
        0.66379, 0.66875, 0.67316, 0.67662, 0.67894, 0.68051, 0.68168, 0.68162, 0.67975, 0.67700, 0.67406,\
        0.67023, 0.66090, 0.63681, 0.58859, 0.52235, 0.45996, 0.40974, 0.37241, 0.35242, 0.34921, 0.35245,\
        0.35765, 0.37836, 0.42160, 0.46105, 0.48766, 0.51419, 0.55773, 0.62719, 0.70914, 0.78317, 0.88101,\
        0.95445, 0.99282, 1.01076, 1.01543, 1.01106, 1.00059, 0.98622, 0.96970, 0.95250, 0.93591, 0.92096,\
        0.90780, 0.89586, 0.88523, 0.87560, 0.86691, 0.85905, 0.85186, 0.84517, 0.83884, 0.83271, 0.82667,\
        0.82057, 0.81429, 0.80765, 0.80046, 0.79259, 0.78408, 0.77507, 0.76572, 0.75615, 0.74648, 0.73682,\
        0.72728, 0.71791, 0.70875, 0.69987, 0.69165, 0.68359, 0.67563, 0.66780, 0.66014, 0.65265, 0.64534,\
        0.63825, 0.63137, 0.62472, 0.61831, 0.61217, 0.60631, 0.60084, 0.59607, 0.59043, 0.58406, 0.57729,\
        0.57024, 0.56297, 0.55551, 0.54791, 0.54020, 0.53242, 0.52460, 0.51679, 0.50899, 0.50114, 0.49307],\
        [0.55089, 0.55711, 0.56309, 0.56884, 0.57434, 0.57960, 0.58462, 0.58941, 0.59401, 0.59859, 0.60343,\
        0.60860, 0.61388, 0.61917, 0.62449, 0.62985, 0.63528, 0.64082, 0.64650, 0.65233, 0.65833, 0.66448,\
        0.67074, 0.67695, 0.68278, 0.68761, 0.69085, 0.69321, 0.69614, 0.69789, 0.69684, 0.69486, 0.69246,\
        0.68958, 0.68167, 0.65874, 0.61419, 0.54829, 0.47858, 0.41978, 0.38227, 0.36709, 0.37493, 0.38652,\
        0.38845, 0.40643, 0.44034, 0.47471, 0.50224, 0.52431, 0.56597, 0.63901, 0.72639, 0.80415, 0.90180,\
        0.96893, 1.00565, 1.02283, 1.02676, 1.02161, 1.01033, 0.99514, 0.97780, 0.95977, 0.94226, 0.92615,\
        0.91177, 0.89907, 0.88762, 0.87738, 0.86831, 0.86020, 0.85285, 0.84606, 0.83966, 0.83348, 0.82737,\
        0.82119, 0.81478, 0.80799, 0.80075, 0.79280, 0.78425, 0.77519, 0.76574, 0.75607, 0.74632, 0.73663,\
        0.72709, 0.71779, 0.70873, 0.69996, 0.69163, 0.68351, 0.67553, 0.66774, 0.66016, 0.65279, 0.64565,\
        0.63875, 0.63208, 0.62567, 0.61952, 0.61369, 0.60825, 0.60340, 0.59858, 0.59228, 0.58559, 0.57865,\
        0.57147, 0.56411, 0.55658, 0.54893, 0.54117, 0.53336, 0.52550, 0.51763, 0.50972, 0.50172, 0.49365],\
        [0.55241, 0.55850, 0.56434, 0.56992, 0.57524, 0.58031, 0.58513, 0.58975, 0.59430, 0.59896, 0.60386,\
        0.60908, 0.61451, 0.62007, 0.62571, 0.63145, 0.63730, 0.64330, 0.64949, 0.65588, 0.66251, 0.66940,\
        0.67654, 0.68387, 0.69113, 0.69756, 0.70207, 0.70554, 0.70985, 0.71338, 0.71488, 0.71536, 0.71539,\
        0.71497, 0.70990, 0.69203, 0.65257, 0.58272, 0.50468, 0.44357, 0.41182, 0.39900, 0.41219, 0.42614,\
        0.42920, 0.44129, 0.46422, 0.48911, 0.51385, 0.54318, 0.58640, 0.65846, 0.75988, 0.84565, 0.91994,\
        0.97820, 1.01246, 1.02881, 1.03240, 1.02702, 1.01550, 1.00001, 0.98226, 0.96369, 0.94547, 0.92850,\
        0.91334, 0.90008, 0.88819, 0.87775, 0.86859, 0.86046, 0.85310, 0.84630, 0.83988, 0.83368, 0.82755,\
        0.82132, 0.81483, 0.80788, 0.80035, 0.79256, 0.78404, 0.77489, 0.76534, 0.75556, 0.74574, 0.73600,\
        0.72649, 0.71733, 0.70858, 0.70008, 0.69157, 0.68324, 0.67520, 0.66744, 0.65994, 0.65271, 0.64574,\
        0.63902, 0.63256, 0.62637, 0.62045, 0.61483, 0.60951, 0.60441, 0.59903, 0.59302, 0.58646, 0.57954,\
        0.57235, 0.56495, 0.55739, 0.54971, 0.54192, 0.53408, 0.52618, 0.51824, 0.51026, 0.50221, 0.49411],\
        [0.55379, 0.55977, 0.56548, 0.57094, 0.57612, 0.58105, 0.58574, 0.59024, 0.59469, 0.59930, 0.60427,\
        0.60959, 0.61514, 0.62087, 0.62674, 0.63275, 0.63892, 0.64529, 0.65187, 0.65869, 0.66579, 0.67320,\
        0.68096, 0.68905, 0.69732, 0.70505, 0.71137, 0.71705, 0.72289, 0.72840, 0.73307, 0.73721, 0.74117,\
        0.74438, 0.74381, 0.73444, 0.70753, 0.64594, 0.56642, 0.50132, 0.47493, 0.46659, 0.46735, 0.47060,\
        0.47265, 0.47846, 0.49235, 0.51073, 0.53834, 0.57742, 0.63138, 0.70877, 0.81921, 0.89181, 0.94521,\
        0.98770, 1.01599, 1.03023, 1.03320, 1.02777, 1.01635, 1.00095, 0.98318, 0.96439, 0.94571, 0.92808,\
        0.91228, 0.89865, 0.88693, 0.87679, 0.86789, 0.85993, 0.85268, 0.84593, 0.83954, 0.83334, 0.82721,\
        0.82099, 0.81455, 0.80775, 0.80047, 0.79238, 0.78354, 0.77418, 0.76448, 0.75460, 0.74468, 0.73488,\
        0.72534, 0.71622, 0.70763, 0.69939, 0.69083, 0.68248, 0.67448, 0.66681, 0.65944, 0.65235, 0.64554,\
        0.63900, 0.63273, 0.62673, 0.62098, 0.61546, 0.61008, 0.60465, 0.59902, 0.59316, 0.58680, 0.57999,\
        0.57284, 0.56546, 0.55790, 0.55021, 0.54242, 0.53456, 0.52663, 0.51866, 0.51065, 0.50261, 0.49449],\
        [0.55505, 0.56093, 0.56656, 0.57192, 0.57702, 0.58187, 0.58649, 0.59091, 0.59525, 0.59971, 0.60476,\
        0.61015, 0.61576, 0.62157, 0.62756, 0.63375, 0.64016, 0.64678, 0.65366, 0.66081, 0.66824, 0.67597,\
        0.68406, 0.69256, 0.70148, 0.71037, 0.71869, 0.72660, 0.73445, 0.74223, 0.74981, 0.75730, 0.76470,\
        0.77124, 0.77468, 0.77090, 0.75241, 0.70711, 0.64534, 0.57884, 0.54359, 0.52604, 0.51394, 0.50694,\
        0.50583, 0.50778, 0.51750, 0.53860, 0.57910, 0.63274, 0.70635, 0.79667, 0.86799, 0.91955, 0.96233,\
        0.99483, 1.01685, 1.02820, 1.03015, 1.02460, 1.01346, 0.99845, 0.98099, 0.96233, 0.94353, 0.92553,\
        0.90923, 0.89564, 0.88454, 0.87498, 0.86651, 0.85882, 0.85173, 0.84506, 0.83869, 0.83249, 0.82635,\
        0.82015, 0.81375, 0.80703, 0.79981, 0.79156, 0.78248, 0.77296, 0.76314, 0.75317, 0.74316, 0.73327,\
        0.72362, 0.71434, 0.70554, 0.69725, 0.68911, 0.68108, 0.67330, 0.66581, 0.65860, 0.65167, 0.64502,\
        0.63865, 0.63254, 0.62669, 0.62107, 0.61563, 0.61026, 0.60478, 0.59909, 0.59319, 0.58684, 0.58007,\
        0.57297, 0.56563, 0.55810, 0.55044, 0.54266, 0.53480, 0.52687, 0.51889, 0.51086, 0.50281, 0.49470],\
        [0.55618, 0.56200, 0.56756, 0.57287, 0.57793, 0.58276, 0.58738, 0.59182, 0.59618, 0.60065, 0.60556,\
        0.61082, 0.61636, 0.62215, 0.62818, 0.63446, 0.64101, 0.64783, 0.65493, 0.66231, 0.66998, 0.67794,\
        0.68620, 0.69489, 0.70439, 0.71443, 0.72438, 0.73418, 0.74395, 0.75377, 0.76362, 0.77350, 0.78314,\
        0.79167, 0.79710, 0.79590, 0.78286, 0.75473, 0.71608, 0.65517, 0.60340, 0.56780, 0.54299, 0.52878,\
        0.52764, 0.53675, 0.55298, 0.58536, 0.64681, 0.70622, 0.78569, 0.85098, 0.89530, 0.93574, 0.97075,\
        0.99719, 1.01460, 1.02325, 1.02403, 1.01831, 1.00755, 0.99315, 0.97636, 0.95824, 0.93976, 0.92190,\
        0.90573, 0.89249, 0.88198, 0.87296, 0.86486, 0.85741, 0.85043, 0.84379, 0.83739, 0.83115, 0.82496,\
        0.81873, 0.81231, 0.80549, 0.79796, 0.78973, 0.78070, 0.77116, 0.76132, 0.75131, 0.74125, 0.73128,\
        0.72151, 0.71208, 0.70306, 0.69475, 0.68694, 0.67926, 0.67176, 0.66448, 0.65745, 0.65068, 0.64417,\
        0.63794, 0.63196, 0.62622, 0.62070, 0.61535, 0.61008, 0.60476, 0.59920, 0.59308, 0.58660, 0.57981,\
        0.57274, 0.56545, 0.55798, 0.55037, 0.54265, 0.53482, 0.52691, 0.51893, 0.51091, 0.50286, 0.49479],\
        [0.55719, 0.56295, 0.56847, 0.57376, 0.57881, 0.58365, 0.58831, 0.59285, 0.59732, 0.60179, 0.60647,\
        0.61149, 0.61686, 0.62254, 0.62854, 0.63486, 0.64150, 0.64845, 0.65571, 0.66329, 0.67118, 0.67938,\
        0.68790, 0.69682, 0.70679, 0.71761, 0.72868, 0.73985, 0.75109, 0.76239, 0.77370, 0.78487, 0.79549,\
        0.80464, 0.81064, 0.81096, 0.80275, 0.78437, 0.75381, 0.70854, 0.66633, 0.62490, 0.58603, 0.56290,\
        0.56240, 0.57991, 0.61784, 0.67866, 0.73313, 0.77725, 0.83404, 0.88003, 0.91443, 0.94681, 0.97494,\
        0.99612, 1.00969, 1.01591, 1.01553, 1.00960, 0.99929, 0.98573, 0.96994, 0.95282, 0.93522, 0.91803,\
        0.90258, 0.89007, 0.87996, 0.87122, 0.86330, 0.85593, 0.84893, 0.84221, 0.83570, 0.82934, 0.82304,\
        0.81671, 0.81019, 0.80333, 0.79580, 0.78735, 0.77833, 0.76885, 0.75906, 0.74909, 0.73905, 0.72906,\
        0.71923, 0.70975, 0.70089, 0.69257, 0.68479, 0.67731, 0.67001, 0.66291, 0.65603, 0.64938, 0.64300,\
        0.63686, 0.63098, 0.62532, 0.61986, 0.61457, 0.60937, 0.60411, 0.59853, 0.59244, 0.58593, 0.57915,\
        0.57214, 0.56492, 0.55755, 0.55003, 0.54238, 0.53463, 0.52677, 0.51882, 0.51082, 0.50282, 0.49481],\
        [0.55805, 0.56377, 0.56926, 0.57454, 0.57960, 0.58446, 0.58915, 0.59373, 0.59828, 0.60268, 0.60715,\
        0.61195, 0.61712, 0.62267, 0.62861, 0.63492, 0.64161, 0.64866, 0.65607, 0.66381, 0.67190, 0.68039,\
        0.68936, 0.69875, 0.70891, 0.71997, 0.73167, 0.74368, 0.75583, 0.76797, 0.77994, 0.79147, 0.80205,\
        0.81084, 0.81651, 0.81743, 0.81194, 0.79885, 0.77752, 0.75090, 0.72567, 0.69962, 0.66919, 0.64263,\
        0.63226, 0.63934, 0.67826, 0.74734, 0.79488, 0.82740, 0.86556, 0.89991, 0.92932, 0.95527, 0.97696,\
        0.99305, 1.00295, 1.00680, 1.00523, 0.99907, 0.98927, 0.97675, 0.96231, 0.94668, 0.93053, 0.91464,\
        0.89999, 0.88848, 0.87866, 0.86996, 0.86198, 0.85448, 0.84732, 0.84040, 0.83368, 0.82709, 0.82059,\
        0.81410, 0.80746, 0.80034, 0.79272, 0.78438, 0.77545, 0.76609, 0.75644, 0.74659, 0.73667, 0.72676,\
        0.71696, 0.70741, 0.69848, 0.69043, 0.68277, 0.67537, 0.66817, 0.66117, 0.65438, 0.64782, 0.64151,\
        0.63544, 0.62961, 0.62399, 0.61854, 0.61323, 0.60803, 0.60287, 0.59727, 0.59119, 0.58475, 0.57805,\
        0.57114, 0.56405, 0.55680, 0.54941, 0.54189, 0.53425, 0.52649, 0.51862, 0.51064, 0.50259, 0.49471],\
        [0.55874, 0.56443, 0.56990, 0.57516, 0.58022, 0.58509, 0.58977, 0.59428, 0.59867, 0.60299, 0.60740,\
        0.61205, 0.61705, 0.62246, 0.62832, 0.63462, 0.64136, 0.64850, 0.65603, 0.66391, 0.67215, 0.68079,\
        0.68998, 0.69984, 0.70997, 0.72124, 0.73334, 0.74582, 0.75838, 0.77079, 0.78277, 0.79398, 0.80387,\
        0.81169, 0.81650, 0.81730, 0.81334, 0.80441, 0.79132, 0.77663, 0.76309, 0.75052, 0.73710, 0.72305,\
        0.71069, 0.70790, 0.74202, 0.80458, 0.84598, 0.87258, 0.89643, 0.91921, 0.94209, 0.96175, 0.97751,\
        0.98869, 0.99495, 0.99642, 0.99359, 0.98717, 0.97794, 0.96663, 0.95386, 0.94015, 0.92597, 0.91199,\
        0.89924, 0.88784, 0.87796, 0.86907, 0.86084, 0.85306, 0.84560, 0.83838, 0.83134, 0.82444, 0.81764,\
        0.81092, 0.80419, 0.79710, 0.78927, 0.78096, 0.77218, 0.76300, 0.75354, 0.74391, 0.73419, 0.72450,\
        0.71491, 0.70553, 0.69671, 0.68862, 0.68093, 0.67351, 0.66630, 0.65931, 0.65254, 0.64601, 0.63973,\
        0.63369, 0.62788, 0.62226, 0.61678, 0.61139, 0.60601, 0.60079, 0.59537, 0.58938, 0.58307, 0.57652,\
        0.56976, 0.56282, 0.55574, 0.54852, 0.54117, 0.53369, 0.52609, 0.51837, 0.51056, 0.50274, 0.49492],\
        [0.55926, 0.56489, 0.57032, 0.57557, 0.58064, 0.58552, 0.59019, 0.59459, 0.59872, 0.60294, 0.60727,\
        0.61175, 0.61656, 0.62184, 0.62763, 0.63393, 0.64073, 0.64798, 0.65564, 0.66366, 0.67202, 0.68066,\
        0.68955, 0.69881, 0.70960, 0.72147, 0.73387, 0.74651, 0.75911, 0.77136, 0.78294, 0.79344, 0.80232,\
        0.80896, 0.81270, 0.81304, 0.80996, 0.80412, 0.79681, 0.78967, 0.78367, 0.77728, 0.76838, 0.75855,\
        0.75310, 0.76361, 0.80838, 0.85816, 0.89044, 0.90970, 0.92538, 0.93964, 0.95394, 0.96674, 0.97675,\
        0.98327, 0.98602, 0.98510, 0.98096, 0.97424, 0.96561, 0.95565, 0.94477, 0.93330, 0.92144, 0.90937,\
        0.89771, 0.88721, 0.87742, 0.86829, 0.85970, 0.85154, 0.84372, 0.83612, 0.82871, 0.82144, 0.81429,\
        0.80721, 0.80021, 0.79306, 0.78538, 0.77720, 0.76861, 0.75966, 0.75045, 0.74107, 0.73163, 0.72225,\
        0.71306, 0.70410, 0.69541, 0.68714, 0.67926, 0.67170, 0.66440, 0.65735, 0.65054, 0.64398, 0.63768,\
        0.63164, 0.62582, 0.62019, 0.61470, 0.60925, 0.60375, 0.59820, 0.59283, 0.58708, 0.58096, 0.57459,\
        0.56801, 0.56127, 0.55438, 0.54736, 0.54022, 0.53295, 0.52555, 0.51803, 0.51042, 0.50278, 0.49501],\
        [0.55956, 0.56513, 0.57051, 0.57572, 0.58077, 0.58567, 0.59038, 0.59485, 0.59895, 0.60292, 0.60686,\
        0.61101, 0.61560, 0.62075, 0.62650, 0.63285, 0.63974, 0.64713, 0.65495, 0.66315, 0.67166, 0.68046,\
        0.68948, 0.69879, 0.70949, 0.72128, 0.73361, 0.74613, 0.75848, 0.77032, 0.78129, 0.79096, 0.79884,\
        0.80443, 0.80727, 0.80720, 0.80474, 0.80135, 0.79865, 0.79774, 0.79963, 0.80266, 0.80512, 0.80720,\
        0.81350, 0.83479, 0.87382, 0.90364, 0.92598, 0.93831, 0.94731, 0.95525, 0.96288, 0.96961, 0.97447,\
        0.97680, 0.97630, 0.97308, 0.96760, 0.96053, 0.95250, 0.94395, 0.93509, 0.92605, 0.91682, 0.90706,\
        0.89677, 0.88656, 0.87666, 0.86721, 0.85825, 0.84973, 0.84155, 0.83360, 0.82580, 0.81814, 0.81069,\
        0.80341, 0.79608, 0.78884, 0.78124, 0.77323, 0.76485, 0.75616, 0.74722, 0.73812, 0.72896, 0.71988,\
        0.71105, 0.70262, 0.69405, 0.68561, 0.67757, 0.66987, 0.66245, 0.65529, 0.64838, 0.64174, 0.63538,\
        0.62930, 0.62346, 0.61782, 0.61231, 0.60689, 0.60149, 0.59592, 0.59029, 0.58452, 0.57854, 0.57234,\
        0.56595, 0.55942, 0.55275, 0.54595, 0.53904, 0.53201, 0.52486, 0.51756, 0.51009, 0.50240, 0.49476],\
        [0.55965, 0.56512, 0.57042, 0.57555, 0.58054, 0.58541, 0.59016, 0.59479, 0.59918, 0.60265, 0.60593,\
        0.60969, 0.61408, 0.61915, 0.62492, 0.63136, 0.63840, 0.64597, 0.65400, 0.66241, 0.67114, 0.68021,\
        0.68974, 0.69986, 0.70965, 0.72083, 0.73285, 0.74504, 0.75697, 0.76828, 0.77860, 0.78753, 0.79467,\
        0.79960, 0.80209, 0.80219, 0.80070, 0.80007, 0.80219, 0.80772, 0.81795, 0.83405, 0.85938, 0.88853,\
        0.89843, 0.91158, 0.92964, 0.94006, 0.94980, 0.95642, 0.96082, 0.96432, 0.96734, 0.96956, 0.97037,\
        0.96924, 0.96595, 0.96062, 0.95381, 0.94630, 0.93885, 0.93175, 0.92485, 0.91810, 0.91160, 0.90452,\
        0.89539, 0.88521, 0.87509, 0.86538, 0.85616, 0.84739, 0.83896, 0.83075, 0.82265, 0.81460, 0.80671,\
        0.79944, 0.79208, 0.78462, 0.77701, 0.76914, 0.76098, 0.75256, 0.74391, 0.73510, 0.72619, 0.71730,\
        0.70858, 0.70015, 0.69194, 0.68374, 0.67573, 0.66795, 0.66041, 0.65311, 0.64606, 0.63930, 0.63284,\
        0.62668, 0.62082, 0.61516, 0.60963, 0.60416, 0.59875, 0.59319, 0.58752, 0.58176, 0.57586, 0.56980,\
        0.56361, 0.55728, 0.55084, 0.54428, 0.53762, 0.53085, 0.52396, 0.51694, 0.50977, 0.50241, 0.49477],\
        [0.55951, 0.56487, 0.57003, 0.57503, 0.57989, 0.58461, 0.58922, 0.59367, 0.59775, 0.60096, 0.60404,\
        0.60765, 0.61196, 0.61704, 0.62290, 0.62949, 0.63673, 0.64454, 0.65281, 0.66147, 0.67042, 0.67960,\
        0.68892, 0.69816, 0.70851, 0.71996, 0.73175, 0.74353, 0.75495, 0.76572, 0.77548, 0.78391, 0.79066,\
        0.79548, 0.79833, 0.79958, 0.80028, 0.80267, 0.80829, 0.81789, 0.83244, 0.85282, 0.87832, 0.89953,\
        0.91880, 0.93549, 0.94809, 0.95548, 0.96035, 0.96363, 0.96559, 0.96664, 0.96695, 0.96633, 0.96438,\
        0.96074, 0.95528, 0.94817, 0.94004, 0.93196, 0.92505, 0.91950, 0.91437, 0.90920, 0.90470, 0.90008,\
        0.89187, 0.88204, 0.87206, 0.86238, 0.85312, 0.84429, 0.83580, 0.82751, 0.81930, 0.81103, 0.80277,\
        0.79504, 0.78774, 0.78030, 0.77272, 0.76498, 0.75704, 0.74891, 0.74059, 0.73209, 0.72346, 0.71473,\
        0.70604, 0.69770, 0.68969, 0.68173, 0.67380, 0.66597, 0.65830, 0.65083, 0.64361, 0.63667, 0.63005,\
        0.62380, 0.61789, 0.61224, 0.60670, 0.60113, 0.59564, 0.59011, 0.58447, 0.57875, 0.57294, 0.56703,\
        0.56101, 0.55489, 0.54867, 0.54235, 0.53594, 0.52942, 0.52280, 0.51605, 0.50916, 0.50209, 0.49450],\
        [0.55916, 0.56437, 0.56937, 0.57419, 0.57882, 0.58329, 0.58757, 0.59158, 0.59516, 0.59824, 0.60134,\
        0.60498, 0.60932, 0.61449, 0.62050, 0.62730, 0.63479, 0.64286, 0.65142, 0.66037, 0.66958, 0.67893,\
        0.68828, 0.69772, 0.70805, 0.71923, 0.73058, 0.74181, 0.75269, 0.76295, 0.77232, 0.78050, 0.78725,\
        0.79242, 0.79612, 0.79887, 0.80192, 0.80662, 0.81411, 0.82501, 0.83952, 0.85687, 0.87428, 0.88893,\
        0.90736, 0.92821, 0.94337, 0.95244, 0.95774, 0.96091, 0.96255, 0.96296, 0.96222, 0.96023, 0.95678,\
        0.95166, 0.94483, 0.93646, 0.92714, 0.91820, 0.91171, 0.90795, 0.90472, 0.89995, 0.89626, 0.89201,\
        0.88525, 0.87657, 0.86726, 0.85796, 0.84894, 0.84028, 0.83192, 0.82377, 0.81568, 0.80754, 0.79921,\
        0.79117, 0.78347, 0.77589, 0.76833, 0.76072, 0.75304, 0.74523, 0.73729, 0.72918, 0.72090, 0.71246,\
        0.70392, 0.69568, 0.68778, 0.67986, 0.67190, 0.66398, 0.65615, 0.64848, 0.64103, 0.63387, 0.62705,\
        0.62063, 0.61465, 0.60905, 0.60361, 0.59802, 0.59240, 0.58680, 0.58118, 0.57552, 0.56981, 0.56403,\
        0.55818, 0.55226, 0.54626, 0.54017, 0.53399, 0.52771, 0.52132, 0.51480, 0.50807, 0.50100, 0.49359],\
        [0.55863, 0.56367, 0.56847, 0.57306, 0.57744, 0.58159, 0.58550, 0.58909, 0.59234, 0.59532, 0.59840,\
        0.60197, 0.60631, 0.61161, 0.61783, 0.62487, 0.63261, 0.64097, 0.64985, 0.65912, 0.66868, 0.67840,\
        0.68821, 0.69815, 0.70829, 0.71874, 0.72937, 0.73994, 0.75027, 0.76012, 0.76926, 0.77744, 0.78449,\
        0.79035, 0.79515, 0.79939, 0.80407, 0.81015, 0.81829, 0.82882, 0.84147, 0.85517, 0.86833, 0.88079,\
        0.89556, 0.91378, 0.92891, 0.93909, 0.94610, 0.95085, 0.95365, 0.95471, 0.95415, 0.95197, 0.94810,\
        0.94248, 0.93519, 0.92643, 0.91657, 0.90663, 0.90035, 0.89793, 0.89611, 0.89209, 0.88798, 0.88334,\
        0.87714, 0.86943, 0.86091, 0.85218, 0.84358, 0.83525, 0.82719, 0.81931, 0.81144, 0.80352, 0.79538,\
        0.78700, 0.77899, 0.77127, 0.76374, 0.75632, 0.74893, 0.74151, 0.73400, 0.72635, 0.71854, 0.71058,\
        0.70252, 0.69437, 0.68629, 0.67822, 0.67012, 0.66203, 0.65400, 0.64610, 0.63839, 0.63096, 0.62387,\
        0.61721, 0.61107, 0.60546, 0.60013, 0.59457, 0.58891, 0.58329, 0.57769, 0.57209, 0.56647, 0.56083,\
        0.55514, 0.54941, 0.54362, 0.53776, 0.53179, 0.52572, 0.51952, 0.51319, 0.50666, 0.49975, 0.49240],\
        [0.55795, 0.56280, 0.56740, 0.57175, 0.57586, 0.57970, 0.58327, 0.58655, 0.58956, 0.59243, 0.59543,\
        0.59888, 0.60318, 0.60863, 0.61505, 0.62229, 0.63028, 0.63890, 0.64807, 0.65768, 0.66760, 0.67775,\
        0.68799, 0.69815, 0.70814, 0.71798, 0.72787, 0.73779, 0.74764, 0.75721, 0.76628, 0.77465, 0.78219,\
        0.78889, 0.79485, 0.80038, 0.80609, 0.81273, 0.82077, 0.83032, 0.84102, 0.85214, 0.86308, 0.87417,\
        0.88674, 0.90084, 0.91237, 0.92156, 0.93011, 0.93683, 0.94129, 0.94363, 0.94399, 0.94243, 0.93895,\
        0.93363, 0.92668, 0.91858, 0.90981, 0.90079, 0.89416, 0.89077, 0.88799, 0.88431, 0.87991, 0.87481,\
        0.86865, 0.86141, 0.85347, 0.84528, 0.83714, 0.82920, 0.82150, 0.81400, 0.80640, 0.79833, 0.79021,\
        0.78197, 0.77394, 0.76624, 0.75885, 0.75168, 0.74466, 0.73769, 0.73068, 0.72355, 0.71626, 0.70876,\
        0.70113, 0.69312, 0.68494, 0.67671, 0.66843, 0.66014, 0.65189, 0.64373, 0.63575, 0.62802, 0.62063,\
        0.61365, 0.60721, 0.60149, 0.59610, 0.59067, 0.58513, 0.57958, 0.57404, 0.56850, 0.56298, 0.55745,\
        0.55192, 0.54638, 0.54079, 0.53513, 0.52937, 0.52347, 0.51741, 0.51120, 0.50489, 0.49825, 0.49091],\
        [0.55715, 0.56183, 0.56623, 0.57035, 0.57421, 0.57779, 0.58109, 0.58413, 0.58698, 0.58976, 0.59269,\
        0.59606, 0.60028, 0.60584, 0.61234, 0.61969, 0.62783, 0.63667, 0.64608, 0.65594, 0.66615, 0.67661,\
        0.68721, 0.69770, 0.70732, 0.71649, 0.72573, 0.73514, 0.74465, 0.75408, 0.76324, 0.77192, 0.78002,\
        0.78752, 0.79454, 0.80110, 0.80734, 0.81415, 0.82185, 0.83039, 0.83951, 0.84887, 0.85825, 0.86792,\
        0.87827, 0.88867, 0.89626, 0.90437, 0.91395, 0.92203, 0.92778, 0.93141, 0.93295, 0.93243, 0.92988,\
        0.92533, 0.91913, 0.91208, 0.90541, 0.89847, 0.89116, 0.88574, 0.88138, 0.87697, 0.87207, 0.86651,\
        0.86014, 0.85300, 0.84535, 0.83754, 0.82983, 0.82227, 0.81482, 0.80760, 0.80049, 0.79246, 0.78421,\
        0.77606, 0.76818, 0.76066, 0.75354, 0.74674, 0.74018, 0.73372, 0.72727, 0.72073, 0.71397, 0.70685,\
        0.69933, 0.69163, 0.68354, 0.67522, 0.66678, 0.65829, 0.64982, 0.64142, 0.63317, 0.62516, 0.61748,\
        0.61023, 0.60342, 0.59744, 0.59207, 0.58670, 0.58126, 0.57578, 0.57028, 0.56480, 0.55935, 0.55393,\
        0.54855, 0.54318, 0.53779, 0.53235, 0.52678, 0.52105, 0.51509, 0.50887, 0.50249, 0.49598, 0.48898],\
        [0.55629, 0.56080, 0.56502, 0.56895, 0.57259, 0.57596, 0.57906, 0.58194, 0.58468, 0.58742, 0.59035,\
        0.59378, 0.59809, 0.60346, 0.60976, 0.61709, 0.62534, 0.63432, 0.64386, 0.65385, 0.66417, 0.67471,\
        0.68530, 0.69560, 0.70500, 0.71387, 0.72270, 0.73178, 0.74115, 0.75063, 0.76000, 0.76907, 0.77769,\
        0.78582, 0.79346, 0.80059, 0.80740, 0.81445, 0.82182, 0.82952, 0.83744, 0.84546, 0.85352, 0.86169,\
        0.86993, 0.87766, 0.88439, 0.89181, 0.90066, 0.90858, 0.91469, 0.91935, 0.92193, 0.92262, 0.92129,\
        0.91776, 0.91233, 0.90581, 0.89981, 0.89421, 0.88726, 0.88094, 0.87539, 0.87007, 0.86452, 0.85846,\
        0.85177, 0.84448, 0.83684, 0.82921, 0.82189, 0.81473, 0.80739, 0.80011, 0.79299, 0.78533, 0.77732,\
        0.76937, 0.76172, 0.75451, 0.74777, 0.74145, 0.73544, 0.72958, 0.72373, 0.71779, 0.71166, 0.70514,\
        0.69796, 0.69022, 0.68207, 0.67366, 0.66508, 0.65644, 0.64779, 0.63920, 0.63073, 0.62247, 0.61456,\
        0.60718, 0.60041, 0.59414, 0.58847, 0.58298, 0.57751, 0.57201, 0.56652, 0.56105, 0.55564, 0.55030,\
        0.54504, 0.53983, 0.53465, 0.52943, 0.52410, 0.51856, 0.51274, 0.50658, 0.50009, 0.49368, 0.48690],\
        [0.55539, 0.55976, 0.56382, 0.56759, 0.57107, 0.57428, 0.57725, 0.58004, 0.58272, 0.58544, 0.58839,\
        0.59182, 0.59604, 0.60113, 0.60721, 0.61455, 0.62287, 0.63190, 0.64146, 0.65142, 0.66166, 0.67203,\
        0.68232, 0.69223, 0.70149, 0.71021, 0.71870, 0.72764, 0.73709, 0.74679, 0.75648, 0.76595, 0.77506,\
        0.78369, 0.79171, 0.79910, 0.80659, 0.81390, 0.82096, 0.82794, 0.83492, 0.84187, 0.84880, 0.85570,\
        0.86250, 0.86901, 0.87538, 0.88232, 0.88995, 0.89672, 0.90301, 0.90833, 0.91136, 0.91328, 0.91343,\
        0.91107, 0.90615, 0.90065, 0.89433, 0.88833, 0.88186, 0.87538, 0.86925, 0.86328, 0.85718, 0.85068,\
        0.84364, 0.83608, 0.82822, 0.82049, 0.81349, 0.80692, 0.79989, 0.79241, 0.78508, 0.77751, 0.76973,\
        0.76203, 0.75466, 0.74781, 0.74154, 0.73582, 0.73047, 0.72528, 0.72003, 0.71463, 0.70909, 0.70319,\
        0.69638, 0.68862, 0.68042, 0.67194, 0.66328, 0.65455, 0.64580, 0.63709, 0.62849, 0.62005, 0.61189,\
        0.60430, 0.59754, 0.59124, 0.58533, 0.57963, 0.57400, 0.56840, 0.56282, 0.55731, 0.55188, 0.54658,\
        0.54141, 0.53636, 0.53138, 0.52641, 0.52134, 0.51607, 0.51050, 0.50457, 0.49828, 0.49174, 0.48490],\
        [0.55448, 0.55873, 0.56267, 0.56632, 0.56969, 0.57280, 0.57570, 0.57844, 0.58111, 0.58383, 0.58677,\
        0.59011, 0.59402, 0.59868, 0.60476, 0.61224, 0.62057, 0.62953, 0.63897, 0.64875, 0.65873, 0.66878,\
        0.67870, 0.68832, 0.69746, 0.70572, 0.71369, 0.72278, 0.73256, 0.74260, 0.75263, 0.76250, 0.77208,\
        0.78126, 0.78993, 0.79802, 0.80575, 0.81281, 0.81937, 0.82571, 0.83192, 0.83804, 0.84408, 0.85004,\
        0.85592, 0.86175, 0.86769, 0.87400, 0.88060, 0.88698, 0.89326, 0.89810, 0.90119, 0.90448, 0.90603,\
        0.90535, 0.90063, 0.89468, 0.88853, 0.88204, 0.87548, 0.86896, 0.86261, 0.85634, 0.84993, 0.84316,\
        0.83588, 0.82808, 0.81990, 0.81169, 0.80470, 0.79884, 0.79197, 0.78460, 0.77709, 0.76947, 0.76178,\
        0.75425, 0.74712, 0.74064, 0.73491, 0.72989, 0.72534, 0.72091, 0.71628, 0.71126, 0.70600, 0.70057,\
        0.69401, 0.68651, 0.67841, 0.66996, 0.66130, 0.65256, 0.64380, 0.63509, 0.62648, 0.61802, 0.60975,\
        0.60177, 0.59500, 0.58872, 0.58261, 0.57666, 0.57079, 0.56499, 0.55925, 0.55362, 0.54811, 0.54279,\
        0.53767, 0.53275, 0.52798, 0.52328, 0.51852, 0.51356, 0.50828, 0.50259, 0.49643, 0.48986, 0.48294],\
        [0.55357, 0.55773, 0.56159, 0.56515, 0.56845, 0.57152, 0.57439, 0.57713, 0.57982, 0.58258, 0.58554,\
        0.58884, 0.59264, 0.59721, 0.60318, 0.61049, 0.61861, 0.62732, 0.63648, 0.64594, 0.65556, 0.66519,\
        0.67466, 0.68380, 0.69242, 0.69977, 0.70789, 0.71766, 0.72786, 0.73819, 0.74849, 0.75864, 0.76858,\
        0.77825, 0.78763, 0.79664, 0.80434, 0.81081, 0.81685, 0.82269, 0.82835, 0.83385, 0.83922, 0.84452,\
        0.84980, 0.85512, 0.86060, 0.86632, 0.87222, 0.87810, 0.88369, 0.88832, 0.89212, 0.89575, 0.89787,\
        0.89874, 0.89488, 0.88828, 0.88153, 0.87478, 0.86814, 0.86167, 0.85536, 0.84910, 0.84268, 0.83588,\
        0.82855, 0.82070, 0.81252, 0.80412, 0.79640, 0.79046, 0.78385, 0.77664, 0.76911, 0.76143, 0.75374,\
        0.74626, 0.73928, 0.73310, 0.72792, 0.72371, 0.72013, 0.71668, 0.71278, 0.70804, 0.70257, 0.69728,\
        0.69117, 0.68397, 0.67604, 0.66767, 0.65908, 0.65040, 0.64173, 0.63314, 0.62467, 0.61638, 0.60838,\
        0.60081, 0.59354, 0.58679, 0.58033, 0.57405, 0.56789, 0.56182, 0.55585, 0.55001, 0.54436, 0.53895,\
        0.53382, 0.52900, 0.52443, 0.52002, 0.51559, 0.51095, 0.50595, 0.50051, 0.49441, 0.48784, 0.48089],\
        [0.55267, 0.55677, 0.56057, 0.56409, 0.56736, 0.57041, 0.57330, 0.57609, 0.57885, 0.58169, 0.58472,\
        0.58809, 0.59200, 0.59673, 0.60251, 0.60932, 0.61699, 0.62530, 0.63406, 0.64311, 0.65230, 0.66147,\
        0.67046, 0.67907, 0.68707, 0.69457, 0.70324, 0.71306, 0.72333, 0.73371, 0.74405, 0.75428, 0.76434,\
        0.77420, 0.78381, 0.79287, 0.80070, 0.80720, 0.81307, 0.81873, 0.82410, 0.82917, 0.83406, 0.83889,\
        0.84375, 0.84867, 0.85370, 0.85887, 0.86414, 0.86938, 0.87437, 0.87883, 0.88275, 0.88606, 0.88818,\
        0.88837, 0.88525, 0.87949, 0.87286, 0.86616, 0.85966, 0.85343, 0.84741, 0.84145, 0.83532, 0.82878,\
        0.82163, 0.81386, 0.80585, 0.79807, 0.79020, 0.78314, 0.77619, 0.76891, 0.76133, 0.75358, 0.74583,\
        0.73831, 0.73133, 0.72532, 0.72065, 0.71733, 0.71491, 0.71268, 0.70989, 0.70583, 0.70042, 0.69478,\
        0.68849, 0.68124, 0.67332, 0.66502, 0.65654, 0.64801, 0.63952, 0.63113, 0.62290, 0.61487, 0.60713,\
        0.59965, 0.59229, 0.58520, 0.57839, 0.57176, 0.56527, 0.55889, 0.55262, 0.54652, 0.54065, 0.53507,\
        0.52987, 0.52509, 0.52070, 0.51660, 0.51254, 0.50822, 0.50341, 0.49806, 0.49211, 0.48560, 0.47866],\
        [0.55178, 0.55584, 0.55961, 0.56311, 0.56638, 0.56946, 0.57240, 0.57526, 0.57812, 0.58107, 0.58422,\
        0.58771, 0.59171, 0.59644, 0.60200, 0.60839, 0.61560, 0.62344, 0.63174, 0.64033, 0.64908, 0.65782,\
        0.66641, 0.67472, 0.68274, 0.69081, 0.69954, 0.70906, 0.71903, 0.72918, 0.73931, 0.74934, 0.75923,\
        0.76894, 0.77835, 0.78720, 0.79509, 0.80180, 0.80790, 0.81383, 0.81913, 0.82387, 0.82839, 0.83291,\
        0.83749, 0.84210, 0.84672, 0.85135, 0.85598, 0.86056, 0.86497, 0.86904, 0.87262, 0.87543, 0.87696,\
        0.87653, 0.87360, 0.86853, 0.86238, 0.85603, 0.84993, 0.84418, 0.83869, 0.83327, 0.82771, 0.82172,\
        0.81503, 0.80749, 0.79938, 0.79169, 0.78401, 0.77652, 0.76913, 0.76163, 0.75395, 0.74615, 0.73833,\
        0.73070, 0.72357, 0.71750, 0.71318, 0.71081, 0.70964, 0.70877, 0.70737, 0.70449, 0.69955, 0.69307,\
        0.68590, 0.67822, 0.67016, 0.66191, 0.65359, 0.64529, 0.63708, 0.62898, 0.62103, 0.61326, 0.60570,\
        0.59827, 0.59093, 0.58371, 0.57666, 0.56974, 0.56293, 0.55621, 0.54962, 0.54320, 0.53702, 0.53119,\
        0.52582, 0.52100, 0.51676, 0.51297, 0.50936, 0.50544, 0.50076, 0.49548, 0.48958, 0.48314, 0.47623],\
        [0.55087, 0.55491, 0.55867, 0.56218, 0.56548, 0.56860, 0.57161, 0.57457, 0.57755, 0.58064, 0.58392,\
        0.58752, 0.59154, 0.59612, 0.60140, 0.60748, 0.61428, 0.62165, 0.62948, 0.63762, 0.64594, 0.65431,\
        0.66262, 0.67082, 0.67895, 0.68726, 0.69602, 0.70522, 0.71478, 0.72453, 0.73424, 0.74381, 0.75325,\
        0.76256, 0.77161, 0.78021, 0.78808, 0.79509, 0.80177, 0.80827, 0.81342, 0.81776, 0.82201, 0.82640,\
        0.83086, 0.83526, 0.83950, 0.84358, 0.84754, 0.85143, 0.85521, 0.85874, 0.86181, 0.86404, 0.86497,\
        0.86406, 0.86106, 0.85627, 0.85047, 0.84452, 0.83899, 0.83395, 0.82917, 0.82446, 0.81968, 0.81445,\
        0.80838, 0.80158, 0.79359, 0.78562, 0.77782, 0.77012, 0.76249, 0.75482, 0.74708, 0.73928, 0.73148,\
        0.72380, 0.71648, 0.71009, 0.70577, 0.70439, 0.70424, 0.70443, 0.70427, 0.70250, 0.69749, 0.69035,\
        0.68253, 0.67447, 0.66633, 0.65819, 0.65013, 0.64218, 0.63435, 0.62662, 0.61900, 0.61151, 0.60414,\
        0.59684, 0.58956, 0.58231, 0.57510, 0.56794, 0.56084, 0.55381, 0.54688, 0.54011, 0.53357, 0.52740,\
        0.52174, 0.51677, 0.51257, 0.50905, 0.50594, 0.50262, 0.49813, 0.49278, 0.48687, 0.48046, 0.47359],\
        [0.54994, 0.55397, 0.55773, 0.56126, 0.56459, 0.56778, 0.57088, 0.57394, 0.57705, 0.58027, 0.58369,\
        0.58739, 0.59144, 0.59588, 0.60086, 0.60664, 0.61299, 0.61988, 0.62724, 0.63494, 0.64288, 0.65092,\
        0.65902, 0.66713, 0.67531, 0.68367, 0.69230, 0.70120, 0.71041, 0.71971, 0.72880, 0.73767, 0.74645,\
        0.75520, 0.76384, 0.77223, 0.78022, 0.78781, 0.79525, 0.80208, 0.80663, 0.81057, 0.81475, 0.81926,\
        0.82379, 0.82808, 0.83197, 0.83545, 0.83868, 0.84183, 0.84495, 0.84791, 0.85046, 0.85221, 0.85268,\
        0.85142, 0.84823, 0.84337, 0.83759, 0.83189, 0.82701, 0.82288, 0.81891, 0.81489, 0.81094, 0.80670,\
        0.80112, 0.79454, 0.78717, 0.77936, 0.77155, 0.76379, 0.75608, 0.74838, 0.74068, 0.73301, 0.72541,\
        0.71794, 0.71069, 0.70411, 0.69942, 0.69865, 0.69875, 0.69914, 0.69924, 0.69752, 0.69259, 0.68546,\
        0.67759, 0.66954, 0.66155, 0.65372, 0.64609, 0.63863, 0.63131, 0.62406, 0.61685, 0.60966, 0.60255,\
        0.59544, 0.58827, 0.58103, 0.57372, 0.56638, 0.55904, 0.55172, 0.54447, 0.53734, 0.53042, 0.52383,\
        0.51776, 0.51248, 0.50821, 0.50483, 0.50194, 0.49922, 0.49497, 0.48972, 0.48389, 0.47755, 0.47076],\
        [0.54894, 0.55297, 0.55675, 0.56030, 0.56368, 0.56693, 0.57010, 0.57327, 0.57650, 0.57985, 0.58341,\
        0.58722, 0.59135, 0.59582, 0.60066, 0.60586, 0.61163, 0.61802, 0.62493, 0.63223, 0.63980, 0.64755,\
        0.65542, 0.66343, 0.67158, 0.67993, 0.68845, 0.69708, 0.70597, 0.71478, 0.72296, 0.73085, 0.73882,\
        0.74694, 0.75515, 0.76336, 0.77144, 0.77934, 0.78694, 0.79348, 0.79824, 0.80218, 0.80671, 0.81157,\
        0.81632, 0.82061, 0.82413, 0.82692, 0.82931, 0.83168, 0.83416, 0.83659, 0.83871, 0.84015, 0.84041,\
        0.83895, 0.83552, 0.83038, 0.82427, 0.81843, 0.81423, 0.81128, 0.80810, 0.80456, 0.80121, 0.79787,\
        0.79297, 0.78679, 0.77993, 0.77259, 0.76503, 0.75740, 0.74977, 0.74217, 0.73464, 0.72724, 0.72005,\
        0.71314, 0.70652, 0.70026, 0.69588, 0.69387, 0.69319, 0.69285, 0.69201, 0.68956, 0.68484, 0.67830,\
        0.67088, 0.66323, 0.65571, 0.64843, 0.64142, 0.63463, 0.62798, 0.62136, 0.61469, 0.60784, 0.60086,\
        0.59411, 0.58715, 0.57993, 0.57255, 0.56507, 0.55752, 0.54996, 0.54244, 0.53500, 0.52772, 0.52072,\
        0.51416, 0.50837, 0.50391, 0.50073, 0.49781, 0.49495, 0.49112, 0.48626, 0.48065, 0.47447, 0.46778],\
        [0.54786, 0.55189, 0.55568, 0.55926, 0.56267, 0.56597, 0.56921, 0.57247, 0.57579, 0.57925, 0.58291,\
        0.58682, 0.59101, 0.59547, 0.60003, 0.60476, 0.60999, 0.61594, 0.62248, 0.62942, 0.63663, 0.64405,\
        0.65167, 0.65950, 0.66756, 0.67588, 0.68443, 0.69309, 0.70178, 0.70976, 0.71649, 0.72321, 0.73033,\
        0.73784, 0.74564, 0.75363, 0.76167, 0.76958, 0.77704, 0.78354, 0.78882, 0.79344, 0.79839, 0.80348,\
        0.80850, 0.81289, 0.81599, 0.81793, 0.81937, 0.82094, 0.82289, 0.82488, 0.82665, 0.82795, 0.82829,\
        0.82689, 0.82324, 0.81776, 0.81127, 0.80475, 0.80109, 0.79937, 0.79680, 0.79380, 0.79092, 0.78783,\
        0.78367, 0.77832, 0.77213, 0.76536, 0.75822, 0.75088, 0.74346, 0.73607, 0.72879, 0.72174, 0.71502,\
        0.70885, 0.70309, 0.69757, 0.69271, 0.68946, 0.68742, 0.68576, 0.68369, 0.68044, 0.67565, 0.66954,\
        0.66270, 0.65569, 0.64885, 0.64234, 0.63616, 0.63023, 0.62441, 0.61858, 0.61263, 0.60646, 0.60000,\
        0.59328, 0.58631, 0.57906, 0.57160, 0.56400, 0.55630, 0.54856, 0.54084, 0.53318, 0.52565, 0.51835,\
        0.51141, 0.50506, 0.49999, 0.49706, 0.49422, 0.49113, 0.48736, 0.48273, 0.47733, 0.47130, 0.46471],\
        [0.54665, 0.55069, 0.55448, 0.55807, 0.56150, 0.56484, 0.56813, 0.57144, 0.57483, 0.57835, 0.58207,\
        0.58602, 0.59022, 0.59461, 0.59903, 0.60316, 0.60790, 0.61360, 0.61985, 0.62644, 0.63326, 0.64031,\
        0.64759, 0.65514, 0.66300, 0.67119, 0.67971, 0.68852, 0.69733, 0.70365, 0.70878, 0.71456, 0.72103,\
        0.72801, 0.73544, 0.74321, 0.75116, 0.75903, 0.76649, 0.77321, 0.77909, 0.78447, 0.78976, 0.79503,\
        0.80026, 0.80484, 0.80739, 0.80840, 0.80885, 0.80965, 0.81135, 0.81295, 0.81432, 0.81560, 0.81629,\
        0.81532, 0.81143, 0.80548, 0.79919, 0.79272, 0.78884, 0.78684, 0.78488, 0.78265, 0.78028, 0.77754,\
        0.77399, 0.76943, 0.76399, 0.75785, 0.75121, 0.74424, 0.73710, 0.72996, 0.72297, 0.71627, 0.70988,\
        0.70412, 0.69945, 0.69373, 0.68861, 0.68447, 0.68116, 0.67818, 0.67491, 0.67085, 0.66578, 0.65988,\
        0.65353, 0.64719, 0.64118, 0.63561, 0.63044, 0.62552, 0.62066, 0.61571, 0.61056, 0.60516, 0.59931,\
        0.59266, 0.58562, 0.57833, 0.57081, 0.56313, 0.55534, 0.54750, 0.53967, 0.53190, 0.52428, 0.51690,\
        0.50988, 0.50350, 0.49834, 0.49466, 0.49144, 0.48803, 0.48409, 0.47946, 0.47414, 0.46818, 0.46165],\
        [0.54529, 0.54932, 0.55310, 0.55669, 0.56013, 0.56347, 0.56678, 0.57011, 0.57352, 0.57706, 0.58078,\
        0.58469, 0.58880, 0.59304, 0.59718, 0.60057, 0.60545, 0.61110, 0.61708, 0.62327, 0.62963, 0.63621,\
        0.64304, 0.65018, 0.65767, 0.66551, 0.67364, 0.68182, 0.68935, 0.69502, 0.69969, 0.70512, 0.71118,\
        0.71772, 0.72479, 0.73237, 0.74026, 0.74816, 0.75575, 0.76277, 0.76915, 0.77503, 0.78062, 0.78603,\
        0.79119, 0.79552, 0.79761, 0.79814, 0.79803, 0.79827, 0.79992, 0.80100, 0.80189, 0.80306, 0.80403,\
        0.80380, 0.79976, 0.79318, 0.78680, 0.78105, 0.77699, 0.77457, 0.77285, 0.77123, 0.76945, 0.76724,\
        0.76430, 0.76046, 0.75574, 0.75023, 0.74410, 0.73752, 0.73067, 0.72377, 0.71704, 0.71073, 0.70467,\
        0.69883, 0.69386, 0.68862, 0.68337, 0.67862, 0.67435, 0.67025, 0.66595, 0.66113, 0.65569, 0.64979,\
        0.64379, 0.63808, 0.63294, 0.62844, 0.62444, 0.62068, 0.61687, 0.61281, 0.60835, 0.60347, 0.59799,\
        0.59170, 0.58485, 0.57762, 0.57011, 0.56241, 0.55459, 0.54672, 0.53887, 0.53111, 0.52354, 0.51625,\
        0.50941, 0.50326, 0.49799, 0.49356, 0.48962, 0.48569, 0.48141, 0.47661, 0.47121, 0.46522, 0.45868],\
        [0.54375, 0.54775, 0.55152, 0.55508, 0.55849, 0.56182, 0.56510, 0.56841, 0.57179, 0.57530, 0.57896,\
        0.58279, 0.58676, 0.59076, 0.59460, 0.59843, 0.60322, 0.60864, 0.61418, 0.61985, 0.62566, 0.63166,\
        0.63793, 0.64453, 0.65150, 0.65878, 0.66624, 0.67353, 0.68011, 0.68557, 0.69042, 0.69558, 0.70122,\
        0.70731, 0.71405, 0.72148, 0.72936, 0.73736, 0.74512, 0.75239, 0.75908, 0.76522, 0.77090, 0.77615,\
        0.78083, 0.78445, 0.78646, 0.78711, 0.78718, 0.78751, 0.78844, 0.78923, 0.78989, 0.79064, 0.79114,\
        0.79036, 0.78651, 0.78029, 0.77400, 0.76861, 0.76472, 0.76233, 0.76090, 0.75984, 0.75868, 0.75710,\
        0.75480, 0.75163, 0.74756, 0.74265, 0.73701, 0.73079, 0.72421, 0.71751, 0.71088, 0.70481, 0.69954,\
        0.69352, 0.68797, 0.68256, 0.67720, 0.67204, 0.66708, 0.66216, 0.65703, 0.65154, 0.64568, 0.63966,\
        0.63385, 0.62867, 0.62442, 0.62111, 0.61843, 0.61593, 0.61323, 0.61004, 0.60620, 0.60160, 0.59647,\
        0.59059, 0.58399, 0.57688, 0.56943, 0.56176, 0.55396, 0.54613, 0.53835, 0.53068, 0.52323, 0.51611,\
        0.50946, 0.50342, 0.49798, 0.49304, 0.48848, 0.48399, 0.47929, 0.47419, 0.46859, 0.46248, 0.45585],\
        [0.54199, 0.54596, 0.54967, 0.55319, 0.55655, 0.55982, 0.56305, 0.56630, 0.56961, 0.57304, 0.57661,\
        0.58033, 0.58419, 0.58814, 0.59216, 0.59643, 0.60122, 0.60612, 0.61100, 0.61607, 0.62125, 0.62659,\
        0.63220, 0.63817, 0.64453, 0.65120, 0.65798, 0.66457, 0.67063, 0.67604, 0.68106, 0.68615, 0.69151,\
        0.69722, 0.70372, 0.71099, 0.71885, 0.72698, 0.73485, 0.74225, 0.74905, 0.75521, 0.76073, 0.76560,\
        0.76970, 0.77280, 0.77471, 0.77563, 0.77603, 0.77644, 0.77705, 0.77762, 0.77805, 0.77825, 0.77781,\
        0.77597, 0.77199, 0.76636, 0.76046, 0.75545, 0.75194, 0.74996, 0.74908, 0.74867, 0.74821, 0.74728,\
        0.74561, 0.74306, 0.73959, 0.73522, 0.73003, 0.72413, 0.71773, 0.71123, 0.70488, 0.69838, 0.69311,\
        0.68741, 0.68164, 0.67599, 0.67044, 0.66499, 0.65958, 0.65409, 0.64837, 0.64234, 0.63607, 0.62985,\
        0.62410, 0.61932, 0.61595, 0.61395, 0.61273, 0.61156, 0.60997, 0.60765, 0.60444, 0.60030, 0.59531,\
        0.58956, 0.58310, 0.57609, 0.56870, 0.56108, 0.55336, 0.54562, 0.53796, 0.53044, 0.52316, 0.51620,\
        0.50966, 0.50362, 0.49802, 0.49274, 0.48772, 0.48275, 0.47762, 0.47216, 0.46629, 0.45996, 0.45317],\
        [0.53998, 0.54390, 0.54755, 0.55099, 0.55427, 0.55745, 0.56058, 0.56372, 0.56693, 0.57023, 0.57368,\
        0.57729, 0.58107, 0.58503, 0.58923, 0.59378, 0.59866, 0.60284, 0.60716, 0.61178, 0.61633, 0.62092,\
        0.62582, 0.63113, 0.63690, 0.64301, 0.64926, 0.65539, 0.66119, 0.66660, 0.67178, 0.67697, 0.68235,\
        0.68808, 0.69441, 0.70131, 0.70923, 0.71743, 0.72517, 0.73253, 0.73927, 0.74523, 0.75037, 0.75473,\
        0.75827, 0.76096, 0.76276, 0.76386, 0.76453, 0.76509, 0.76564, 0.76610, 0.76627, 0.76592, 0.76465,\
        0.76198, 0.75767, 0.75217, 0.74651, 0.74181, 0.73882, 0.73758, 0.73752, 0.73793, 0.73820, 0.73791,\
        0.73683, 0.73482, 0.73186, 0.72798, 0.72322, 0.71763, 0.71130, 0.70471, 0.69872, 0.69251, 0.68650,\
        0.68065, 0.67479, 0.66901, 0.66332, 0.65769, 0.65205, 0.64625, 0.64018, 0.63379, 0.62720, 0.62076,\
        0.61498, 0.61048, 0.60793, 0.60741, 0.60774, 0.60786, 0.60726, 0.60571, 0.60306, 0.59928, 0.59438,\
        0.58862, 0.58216, 0.57517, 0.56782, 0.56028, 0.55266, 0.54506, 0.53757, 0.53025, 0.52316, 0.51636,\
        0.50988, 0.50377, 0.49802, 0.49254, 0.48720, 0.48183, 0.47629, 0.47045, 0.46426, 0.45766, 0.45065],\
        [0.53771, 0.54154, 0.54510, 0.54844, 0.55161, 0.55466, 0.55767, 0.56067, 0.56372, 0.56687, 0.57015,\
        0.57362, 0.57728, 0.58118, 0.58535, 0.58978, 0.59421, 0.59823, 0.60255, 0.60699, 0.61081, 0.61458,\
        0.61879, 0.62350, 0.62876, 0.63446, 0.64036, 0.64622, 0.65190, 0.65738, 0.66275, 0.66818, 0.67380,\
        0.67977, 0.68620, 0.69323, 0.70129, 0.70887, 0.71612, 0.72340, 0.72995, 0.73549, 0.74006, 0.74379,\
        0.74677, 0.74906, 0.75074, 0.75193, 0.75281, 0.75355, 0.75421, 0.75469, 0.75475, 0.75405, 0.75223,\
        0.74898, 0.74428, 0.73856, 0.73277, 0.72816, 0.72575, 0.72552, 0.72653, 0.72784, 0.72882, 0.72909,\
        0.72847, 0.72689, 0.72434, 0.72086, 0.71651, 0.71131, 0.70517, 0.69833, 0.69204, 0.68586, 0.67965,\
        0.67355, 0.66755, 0.66168, 0.65595, 0.65031, 0.64464, 0.63880, 0.63266, 0.62615, 0.61940, 0.61281,\
        0.60713, 0.60292, 0.60108, 0.60216, 0.60387, 0.60494, 0.60515, 0.60422, 0.60191, 0.59825, 0.59339,\
        0.58756, 0.58103, 0.57401, 0.56668, 0.55922, 0.55173, 0.54432, 0.53705, 0.52997, 0.52312, 0.51650,\
        0.51012, 0.50395, 0.49808, 0.49249, 0.48689, 0.48115, 0.47520, 0.46897, 0.46242, 0.45551, 0.44824],\
        [0.53514, 0.53887, 0.54231, 0.54552, 0.54855, 0.55145, 0.55428, 0.55710, 0.55996, 0.56291, 0.56599,\
        0.56926, 0.57273, 0.57646, 0.58043, 0.58459, 0.58878, 0.59297, 0.59739, 0.60180, 0.60439, 0.60749,\
        0.61121, 0.61535, 0.62026, 0.62576, 0.63148, 0.63720, 0.64285, 0.64844, 0.65404, 0.65978, 0.66576,\
        0.67208, 0.67885, 0.68618, 0.69388, 0.70047, 0.70777, 0.71513, 0.72125, 0.72611, 0.72993, 0.73293,\
        0.73529, 0.73718, 0.73871, 0.73995, 0.74099, 0.74195, 0.74284, 0.74354, 0.74373, 0.74300, 0.74096,\
        0.73740, 0.73237, 0.72630, 0.72007, 0.71515, 0.71329, 0.71440, 0.71654, 0.71864, 0.72018, 0.72084,\
        0.72051, 0.71919, 0.71689, 0.71369, 0.70965, 0.70484, 0.69904, 0.69225, 0.68548, 0.67893, 0.67247,\
        0.66614, 0.65999, 0.65407, 0.64839, 0.64288, 0.63742, 0.63183, 0.62591, 0.61958, 0.61295, 0.60640,\
        0.60103, 0.59780, 0.59693, 0.59870, 0.60101, 0.60258, 0.60349, 0.60313, 0.60095, 0.59721, 0.59219,\
        0.58619, 0.57951, 0.57243, 0.56513, 0.55777, 0.55046, 0.54327, 0.53628, 0.52949, 0.52293, 0.51658,\
        0.51041, 0.50438, 0.49848, 0.49268, 0.48675, 0.48063, 0.47427, 0.46765, 0.46072, 0.45347, 0.44590],\
        [0.53224, 0.53585, 0.53916, 0.54221, 0.54506, 0.54778, 0.55042, 0.55303, 0.55565, 0.55836, 0.56119,\
        0.56420, 0.56741, 0.57087, 0.57456, 0.57845, 0.58247, 0.58662, 0.59088, 0.59459, 0.59690, 0.59992,\
        0.60328, 0.60676, 0.61166, 0.61717, 0.62278, 0.62843, 0.63408, 0.63979, 0.64565, 0.65173, 0.65810,\
        0.66478, 0.67182, 0.67916, 0.68648, 0.69352, 0.70121, 0.70775, 0.71311, 0.71703, 0.71995, 0.72217,\
        0.72387, 0.72535, 0.72674, 0.72801, 0.72917, 0.73036, 0.73160, 0.73277, 0.73341, 0.73300, 0.73110,\
        0.72754, 0.72242, 0.71618, 0.70954, 0.70401, 0.70250, 0.70513, 0.70788, 0.71042, 0.71229, 0.71303,\
        0.71278, 0.71158, 0.70936, 0.70629, 0.70234, 0.69762, 0.69195, 0.68539, 0.67853, 0.67169, 0.66497,\
        0.65842, 0.65213, 0.64618, 0.64060, 0.63537, 0.63033, 0.62525, 0.61986, 0.61403, 0.60789, 0.60191,\
        0.59702, 0.59447, 0.59416, 0.59579, 0.59813, 0.60015, 0.60199, 0.60235, 0.60007, 0.59594, 0.59052,\
        0.58422, 0.57738, 0.57025, 0.56301, 0.55579, 0.54870, 0.54181, 0.53515, 0.52872, 0.52251, 0.51649,\
        0.51063, 0.50487, 0.49905, 0.49299, 0.48668, 0.48017, 0.47341, 0.46638, 0.45908, 0.45148, 0.44360],\
        [0.52901, 0.53247, 0.53560, 0.53847, 0.54114, 0.54366, 0.54607, 0.54844, 0.55080, 0.55322, 0.55576,\
        0.55846, 0.56136, 0.56449, 0.56786, 0.57144, 0.57519, 0.57904, 0.58281, 0.58611, 0.58888, 0.59182,\
        0.59490, 0.59826, 0.60352, 0.60889, 0.61435, 0.61997, 0.62561, 0.63141, 0.63753, 0.64398, 0.65071,\
        0.65772, 0.66496, 0.67235, 0.67973, 0.68709, 0.69422, 0.70048, 0.70517, 0.70792, 0.71002, 0.71154,\
        0.71252, 0.71360, 0.71492, 0.71621, 0.71744, 0.71883, 0.72052, 0.72243, 0.72391, 0.72416, 0.72274,\
        0.71945, 0.71452, 0.70854, 0.70222, 0.69694, 0.69566, 0.69781, 0.70030, 0.70289, 0.70494, 0.70533,\
        0.70499, 0.70390, 0.70155, 0.69852, 0.69460, 0.68983, 0.68423, 0.67786, 0.67105, 0.66409, 0.65716,\
        0.65040, 0.64396, 0.63797, 0.63253, 0.62766, 0.62320, 0.61886, 0.61423, 0.60908, 0.60358, 0.59826,\
        0.59401, 0.59164, 0.59133, 0.59268, 0.59495, 0.59752, 0.60012, 0.60128, 0.59871, 0.59381, 0.58789,\
        0.58132, 0.57438, 0.56727, 0.56015, 0.55316, 0.54637, 0.53984, 0.53358, 0.52757, 0.52176, 0.51611,\
        0.51055, 0.50503, 0.49933, 0.49306, 0.48648, 0.47964, 0.47252, 0.46511, 0.45742, 0.44947, 0.44127],\
        [0.52540, 0.52868, 0.53162, 0.53430, 0.53676, 0.53906, 0.54124, 0.54334, 0.54542, 0.54753, 0.54973,\
        0.55208, 0.55462, 0.55740, 0.56041, 0.56365, 0.56707, 0.57057, 0.57399, 0.57719, 0.58020, 0.58328,\
        0.58661, 0.59062, 0.59563, 0.60072, 0.60621, 0.61189, 0.61741, 0.62327, 0.62969, 0.63651, 0.64360,\
        0.65086, 0.65824, 0.66565, 0.67299, 0.68011, 0.68677, 0.69254, 0.69646, 0.69849, 0.70011, 0.70112,\
        0.70136, 0.70206, 0.70338, 0.70463, 0.70585, 0.70743, 0.70954, 0.71255, 0.71520, 0.71639, 0.71572,\
        0.71298, 0.70828, 0.70266, 0.69710, 0.69248, 0.69062, 0.69133, 0.69319, 0.69545, 0.69733, 0.69730,\
        0.69683, 0.69575, 0.69344, 0.69034, 0.68645, 0.68173, 0.67620, 0.66997, 0.66322, 0.65620, 0.64910,\
        0.64213, 0.63551, 0.62944, 0.62410, 0.61957, 0.61577, 0.61232, 0.60855, 0.60400, 0.59908, 0.59420,\
        0.59038, 0.58818, 0.58775, 0.58882, 0.59093, 0.59356, 0.59608, 0.59709, 0.59462, 0.58971, 0.58372,\
        0.57716, 0.57030, 0.56335, 0.55647, 0.54978, 0.54337, 0.53728, 0.53150, 0.52599, 0.52064, 0.51537,\
        0.51007, 0.50462, 0.49890, 0.49271, 0.48605, 0.47898, 0.47153, 0.46376, 0.45570, 0.44739, 0.43886],\
        [0.52138, 0.52446, 0.52719, 0.52966, 0.53192, 0.53400, 0.53593, 0.53776, 0.53953, 0.54131, 0.54316,\
        0.54513, 0.54729, 0.54969, 0.55234, 0.55524, 0.55834, 0.56156, 0.56479, 0.56799, 0.57119, 0.57456,\
        0.57829, 0.58261, 0.58752, 0.59277, 0.59852, 0.60419, 0.60933, 0.61535, 0.62220, 0.62942, 0.63680,\
        0.64423, 0.65166, 0.65900, 0.66611, 0.67281, 0.67883, 0.68377, 0.68715, 0.68908, 0.69025, 0.69075,\
        0.69078, 0.69116, 0.69209, 0.69318, 0.69445, 0.69629, 0.69882, 0.70317, 0.70703, 0.70923, 0.70958,\
        0.70785, 0.70318, 0.69756, 0.69225, 0.68793, 0.68552, 0.68510, 0.68600, 0.68746, 0.68861, 0.68877,\
        0.68824, 0.68702, 0.68487, 0.68187, 0.67806, 0.67344, 0.66803, 0.66192, 0.65524, 0.64817, 0.64093,\
        0.63374, 0.62688, 0.62066, 0.61533, 0.61102, 0.60772, 0.60517, 0.60223, 0.59802, 0.59334, 0.58899,\
        0.58556, 0.58350, 0.58295, 0.58372, 0.58544, 0.58758, 0.58938, 0.58969, 0.58758, 0.58334, 0.57782,\
        0.57161, 0.56506, 0.55843, 0.55191, 0.54563, 0.53969, 0.53412, 0.52891, 0.52398, 0.51918, 0.51436,\
        0.50934, 0.50395, 0.49826, 0.49219, 0.48548, 0.47819, 0.47043, 0.46229, 0.45386, 0.44519, 0.43633],\
        [0.51693, 0.51975, 0.52225, 0.52452, 0.52660, 0.52848, 0.53017, 0.53171, 0.53318, 0.53462, 0.53610,\
        0.53768, 0.53945, 0.54147, 0.54378, 0.54638, 0.54922, 0.55224, 0.55538, 0.55863, 0.56204, 0.56573,\
        0.56982, 0.57442, 0.57951, 0.58501, 0.59079, 0.59626, 0.60129, 0.60792, 0.61533, 0.62287, 0.63038,\
        0.63786, 0.64525, 0.65242, 0.65920, 0.66541, 0.67078, 0.67503, 0.67796, 0.67966, 0.68047, 0.68064,\
        0.68050, 0.68053, 0.68096, 0.68179, 0.68314, 0.68532, 0.68875, 0.69353, 0.69849, 0.70161, 0.70336,\
        0.70379, 0.69891, 0.69277, 0.68710, 0.68254, 0.67960, 0.67833, 0.67834, 0.67900, 0.67963, 0.67976,\
        0.67926, 0.67806, 0.67606, 0.67322, 0.66956, 0.66509, 0.65984, 0.65387, 0.64727, 0.64020, 0.63283,\
        0.62541, 0.61828, 0.61182, 0.60640, 0.60219, 0.59904, 0.59686, 0.59420, 0.59046, 0.58628, 0.58241,\
        0.57934, 0.57742, 0.57674, 0.57719, 0.57842, 0.57992, 0.58100, 0.58086, 0.57896, 0.57535, 0.57049,\
        0.56484, 0.55878, 0.55260, 0.54652, 0.54072, 0.53532, 0.53037, 0.52582, 0.52156, 0.51742, 0.51319,\
        0.50865, 0.50360, 0.49801, 0.49183, 0.48489, 0.47730, 0.46918, 0.46066, 0.45185, 0.44281, 0.43363],\
        [0.51199, 0.51450, 0.51676, 0.51887, 0.52081, 0.52252, 0.52397, 0.52522, 0.52638, 0.52751, 0.52863,\
        0.52982, 0.53120, 0.53286, 0.53487, 0.53722, 0.53989, 0.54279, 0.54591, 0.54924, 0.55285, 0.55685,\
        0.56129, 0.56622, 0.57159, 0.57730, 0.58315, 0.58886, 0.59473, 0.60173, 0.60940, 0.61693, 0.62438,\
        0.63180, 0.63908, 0.64602, 0.65244, 0.65815, 0.66293, 0.66660, 0.66908, 0.67044, 0.67092, 0.67078,\
        0.67035, 0.66997, 0.66993, 0.67040, 0.67162, 0.67392, 0.67758, 0.68247, 0.68779, 0.69203, 0.69583,\
        0.69772, 0.69304, 0.68656, 0.68059, 0.67579, 0.67247, 0.67065, 0.67005, 0.67018, 0.67049, 0.67055,\
        0.67011, 0.66902, 0.66717, 0.66451, 0.66105, 0.65678, 0.65173, 0.64594, 0.63947, 0.63244, 0.62500,\
        0.61738, 0.60996, 0.60330, 0.59780, 0.59360, 0.59040, 0.58779, 0.58501, 0.58165, 0.57800, 0.57458,\
        0.57181, 0.56999, 0.56921, 0.56936, 0.57014, 0.57110, 0.57166, 0.57127, 0.56954, 0.56644, 0.56221,\
        0.55718, 0.55168, 0.54601, 0.54043, 0.53516, 0.53035, 0.52606, 0.52226, 0.51879, 0.51542, 0.51188,\
        0.50795, 0.50346, 0.49810, 0.49155, 0.48424, 0.47628, 0.46776, 0.45883, 0.44962, 0.44023, 0.43073],\
        [0.50647, 0.50859, 0.51069, 0.51267, 0.51454, 0.51614, 0.51733, 0.51827, 0.51917, 0.52004, 0.52082,\
        0.52161, 0.52263, 0.52397, 0.52573, 0.52792, 0.53050, 0.53337, 0.53649, 0.53991, 0.54373, 0.54803,\
        0.55284, 0.55815, 0.56389, 0.56993, 0.57615, 0.58251, 0.58924, 0.59668, 0.60427, 0.61143, 0.61871,\
        0.62606, 0.63322, 0.63993, 0.64598, 0.65121, 0.65545, 0.65860, 0.66061, 0.66157, 0.66168, 0.66116,\
        0.66033, 0.65949, 0.65890, 0.65886, 0.65967, 0.66165, 0.66503, 0.66967, 0.67493, 0.67991, 0.68396,\
        0.68538, 0.68257, 0.67736, 0.67192, 0.66729, 0.66393, 0.66193, 0.66109, 0.66100, 0.66121, 0.66130,\
        0.66097, 0.66003, 0.65836, 0.65590, 0.65263, 0.64858, 0.64376, 0.63821, 0.63193, 0.62499, 0.61757,\
        0.60991, 0.60220, 0.59549, 0.58989, 0.58540, 0.58177, 0.57865, 0.57557, 0.57229, 0.56893, 0.56580,\
        0.56324, 0.56147, 0.56060, 0.56052, 0.56097, 0.56156, 0.56183, 0.56134, 0.55980, 0.55714, 0.55346,\
        0.54902, 0.54409, 0.53894, 0.53385, 0.52909, 0.52487, 0.52129, 0.51832, 0.51572, 0.51320, 0.51042,\
        0.50704, 0.50279, 0.49745, 0.49093, 0.48338, 0.47505, 0.46611, 0.45676, 0.44714, 0.43738, 0.42758],\
        [0.50017, 0.50207, 0.50409, 0.50590, 0.50782, 0.50940, 0.51028, 0.51079, 0.51154, 0.51230, 0.51273,\
        0.51313, 0.51384, 0.51493, 0.51650, 0.51864, 0.52125, 0.52415, 0.52728, 0.53076, 0.53476, 0.53938,\
        0.54461, 0.55035, 0.55653, 0.56303, 0.56977, 0.57672, 0.58399, 0.59157, 0.59895, 0.60592, 0.61330,\
        0.62071, 0.62778, 0.63424, 0.63993, 0.64470, 0.64844, 0.65107, 0.65261, 0.65312, 0.65279, 0.65184,\
        0.65054, 0.64914, 0.64794, 0.64722, 0.64732, 0.64859, 0.65134, 0.65553, 0.66061, 0.66562, 0.66947,\
        0.67092, 0.66936, 0.66563, 0.66117, 0.65712, 0.65407, 0.65226, 0.65154, 0.65156, 0.65191, 0.65216,\
        0.65202, 0.65126, 0.64976, 0.64746, 0.64437, 0.64052, 0.63595, 0.63068, 0.62466, 0.61784, 0.61044,\
        0.60299, 0.59534, 0.58831, 0.58238, 0.57743, 0.57327, 0.56962, 0.56618, 0.56278, 0.55947, 0.55644,\
        0.55395, 0.55218, 0.55123, 0.55101, 0.55129, 0.55171, 0.55188, 0.55143, 0.55011, 0.54784, 0.54467,\
        0.54076, 0.53635, 0.53168, 0.52704, 0.52272, 0.51905, 0.51619, 0.51409, 0.51245, 0.51087, 0.50888,\
        0.50607, 0.50208, 0.49676, 0.49011, 0.48229, 0.47356, 0.46418, 0.45439, 0.44436, 0.43425, 0.42416],\
        [0.49323, 0.49518, 0.49698, 0.49864, 0.50062, 0.50223, 0.50277, 0.50268, 0.50352, 0.50435, 0.50438,\
        0.50442, 0.50498, 0.50591, 0.50733, 0.50956, 0.51234, 0.51531, 0.51841, 0.52188, 0.52607, 0.53106,\
        0.53673, 0.54296, 0.54963, 0.55664, 0.56388, 0.57131, 0.57887, 0.58642, 0.59365, 0.60080, 0.60844,\
        0.61594, 0.62286, 0.62904, 0.63434, 0.63865, 0.64189, 0.64403, 0.64507, 0.64511, 0.64429, 0.64286,\
        0.64104, 0.63908, 0.63721, 0.63571, 0.63489, 0.63517, 0.63703, 0.64072, 0.64566, 0.65066, 0.65447,\
        0.65614, 0.65537, 0.65269, 0.64914, 0.64575, 0.64324, 0.64189, 0.64162, 0.64206, 0.64276, 0.64332,\
        0.64341, 0.64283, 0.64148, 0.63929, 0.63631, 0.63258, 0.62821, 0.62326, 0.61763, 0.61099, 0.60344,\
        0.59580, 0.58835, 0.58123, 0.57495, 0.56956, 0.56492, 0.56079, 0.55699, 0.55337, 0.54995, 0.54687,\
        0.54432, 0.54249, 0.54147, 0.54121, 0.54147, 0.54191, 0.54215, 0.54185, 0.54079, 0.53889, 0.53617,\
        0.53276, 0.52884, 0.52461, 0.52034, 0.51636, 0.51311, 0.51094, 0.50976, 0.50913, 0.50852, 0.50737,\
        0.50516, 0.50148, 0.49611, 0.48918, 0.48097, 0.47179, 0.46194, 0.45169, 0.44125, 0.43080, 0.42046],\
        [0.48577, 0.48772, 0.48943, 0.49106, 0.49274, 0.49404, 0.49447, 0.49443, 0.49507, 0.49578, 0.49569,\
        0.49566, 0.49620, 0.49711, 0.49852, 0.50095, 0.50396, 0.50703, 0.51001, 0.51338, 0.51780, 0.52322,\
        0.52937, 0.53609, 0.54327, 0.55078, 0.55850, 0.56634, 0.57419, 0.58192, 0.58945, 0.59690, 0.60457,\
        0.61188, 0.61849, 0.62430, 0.62918, 0.63303, 0.63579, 0.63743, 0.63798, 0.63750, 0.63618, 0.63424,\
        0.63192, 0.62942, 0.62693, 0.62471, 0.62296, 0.62205, 0.62282, 0.62611, 0.63123, 0.63644, 0.64036,\
        0.64222, 0.64186, 0.63976, 0.63676, 0.63388, 0.63193, 0.63127, 0.63172, 0.63282, 0.63404, 0.63498,\
        0.63533, 0.63491, 0.63363, 0.63147, 0.62848, 0.62475, 0.62042, 0.61570, 0.61066, 0.60442, 0.59671,\
        0.58878, 0.58110, 0.57389, 0.56740, 0.56169, 0.55669, 0.55222, 0.54812, 0.54427, 0.54066, 0.53741,\
        0.53470, 0.53274, 0.53167, 0.53147, 0.53188, 0.53250, 0.53295, 0.53289, 0.53213, 0.53057, 0.52826,\
        0.52531, 0.52187, 0.51810, 0.51418, 0.51040, 0.50738, 0.50584, 0.50560, 0.50593, 0.50625, 0.50594,\
        0.50434, 0.50087, 0.49535, 0.48803, 0.47934, 0.46967, 0.45933, 0.44863, 0.43779, 0.42704, 0.41649],\
        [0.47788, 0.47979, 0.48140, 0.48285, 0.48415, 0.48509, 0.48553, 0.48573, 0.48612, 0.48652, 0.48669,\
        0.48692, 0.48756, 0.48868, 0.49042, 0.49297, 0.49608, 0.49931, 0.50217, 0.50537, 0.51025, 0.51617,\
        0.52271, 0.52988, 0.53754, 0.54553, 0.55368, 0.56189, 0.57005, 0.57809, 0.58599, 0.59383, 0.60146,\
        0.60829, 0.61452, 0.61993, 0.62436, 0.62773, 0.63001, 0.63119, 0.63125, 0.63026, 0.62838, 0.62593,\
        0.62318, 0.62026, 0.61730, 0.61458, 0.61225, 0.61026, 0.60965, 0.61292, 0.61871, 0.62422, 0.62820,\
        0.63007, 0.62978, 0.62779, 0.62494, 0.62226, 0.62077, 0.62094, 0.62234, 0.62425, 0.62607, 0.62740,\
        0.62797, 0.62762, 0.62631, 0.62405, 0.62093, 0.61702, 0.61249, 0.60759, 0.60291, 0.59754, 0.58956,\
        0.58137, 0.57351, 0.56622, 0.55963, 0.55379, 0.54862, 0.54397, 0.53970, 0.53568, 0.53189, 0.52843,\
        0.52547, 0.52331, 0.52220, 0.52218, 0.52289, 0.52385, 0.52461, 0.52484, 0.52437, 0.52312, 0.52117,\
        0.51864, 0.51570, 0.51248, 0.50905, 0.50552, 0.50244, 0.50146, 0.50202, 0.50304, 0.50407, 0.50450,\
        0.50351, 0.50019, 0.49431, 0.48648, 0.47728, 0.46713, 0.45634, 0.44521, 0.43401, 0.42298, 0.41228],\
        [0.46970, 0.47150, 0.47295, 0.47416, 0.47514, 0.47583, 0.47623, 0.47649, 0.47680, 0.47716, 0.47755,\
        0.47813, 0.47909, 0.48056, 0.48265, 0.48539, 0.48858, 0.49191, 0.49507, 0.49867, 0.50413, 0.51016,\
        0.51687, 0.52442, 0.53254, 0.54094, 0.54945, 0.55797, 0.56640, 0.57469, 0.58281, 0.59068, 0.59804,\
        0.60474, 0.61075, 0.61579, 0.61971, 0.62257, 0.62439, 0.62516, 0.62479, 0.62328, 0.62081, 0.61781,\
        0.61475, 0.61162, 0.60834, 0.60539, 0.60333, 0.60104, 0.59954, 0.60342, 0.60950, 0.61488, 0.61869,\
        0.62034, 0.61981, 0.61760, 0.61454, 0.61170, 0.61046, 0.61163, 0.61414, 0.61684, 0.61918, 0.62081,\
        0.62148, 0.62107, 0.61958, 0.61710, 0.61371, 0.60951, 0.60462, 0.59929, 0.59405, 0.58836, 0.58106,\
        0.57312, 0.56537, 0.55815, 0.55164, 0.54587, 0.54075, 0.53614, 0.53186, 0.52781, 0.52393, 0.52029,\
        0.51706, 0.51461, 0.51345, 0.51374, 0.51495, 0.51634, 0.51744, 0.51795, 0.51772, 0.51672, 0.51504,\
        0.51287, 0.51044, 0.50790, 0.50526, 0.50245, 0.49977, 0.49885, 0.49942, 0.50056, 0.50184, 0.50276,\
        0.50221, 0.49892, 0.49251, 0.48422, 0.47463, 0.46411, 0.45296, 0.44148, 0.42996, 0.41870, 0.40791],\
        [0.46135, 0.46299, 0.46425, 0.46521, 0.46592, 0.46641, 0.46673, 0.46700, 0.46732, 0.46778, 0.46843,\
        0.46938, 0.47076, 0.47267, 0.47516, 0.47821, 0.48167, 0.48536, 0.48925, 0.49382, 0.49936, 0.50497,\
        0.51183, 0.51985, 0.52837, 0.53708, 0.54583, 0.55453, 0.56312, 0.57153, 0.57969, 0.58749, 0.59471,\
        0.60126, 0.60712, 0.61171, 0.61501, 0.61729, 0.61868, 0.61913, 0.61843, 0.61648, 0.61333, 0.60964,\
        0.60636, 0.60333, 0.59992, 0.59704, 0.59539, 0.59419, 0.59456, 0.59840, 0.60351, 0.60847, 0.61200,\
        0.61328, 0.61231, 0.60970, 0.60642, 0.60337, 0.60200, 0.60447, 0.60782, 0.61097, 0.61359, 0.61535,\
        0.61597, 0.61533, 0.61348, 0.61059, 0.60682, 0.60223, 0.59692, 0.59118, 0.58525, 0.57890, 0.57177,\
        0.56417, 0.55669, 0.54973, 0.54348, 0.53799, 0.53316, 0.52882, 0.52477, 0.52087, 0.51706, 0.51338,\
        0.50998, 0.50719, 0.50588, 0.50673, 0.50852, 0.51031, 0.51167, 0.51238, 0.51233, 0.51150, 0.50997,\
        0.50801, 0.50601, 0.50419, 0.50250, 0.50090, 0.49889, 0.49765, 0.49766, 0.49839, 0.49942, 0.50025,\
        0.49972, 0.49588, 0.48940, 0.48101, 0.47130, 0.46062, 0.44928, 0.43756, 0.42579, 0.41433, 0.40352],\
        [0.45296, 0.45441, 0.45545, 0.45618, 0.45666, 0.45698, 0.45721, 0.45747, 0.45786, 0.45848, 0.45942,\
        0.46077, 0.46261, 0.46501, 0.46799, 0.47150, 0.47543, 0.47971, 0.48432, 0.48940, 0.49480, 0.50027,\
        0.50796, 0.51643, 0.52514, 0.53396, 0.54277, 0.55151, 0.56010, 0.56848, 0.57658, 0.58429, 0.59148,\
        0.59805, 0.60361, 0.60742, 0.60992, 0.61155, 0.61254, 0.61279, 0.61190, 0.60967, 0.60603, 0.60121,\
        0.59770, 0.59461, 0.59152, 0.58910, 0.58794, 0.58809, 0.59005, 0.59414, 0.59922, 0.60445, 0.60783,\
        0.60872, 0.60728, 0.60427, 0.60095, 0.59855, 0.59793, 0.60052, 0.60360, 0.60666, 0.60931, 0.61105,\
        0.61146, 0.61040, 0.60797, 0.60448, 0.60021, 0.59510, 0.58930, 0.58305, 0.57649, 0.56961, 0.56234,\
        0.55492, 0.54771, 0.54108, 0.53524, 0.53024, 0.52594, 0.52211, 0.51851, 0.51499, 0.51147, 0.50801,\
        0.50476, 0.50199, 0.50035, 0.50198, 0.50408, 0.50599, 0.50742, 0.50821, 0.50826, 0.50752, 0.50605,\
        0.50408, 0.50229, 0.50107, 0.49993, 0.49895, 0.49750, 0.49627, 0.49585, 0.49607, 0.49655, 0.49674,\
        0.49544, 0.49141, 0.48507, 0.47690, 0.46736, 0.45678, 0.44546, 0.43367, 0.42175, 0.41012, 0.39935],\
        [0.44466, 0.44592, 0.44675, 0.44725, 0.44752, 0.44767, 0.44781, 0.44807, 0.44855, 0.44936, 0.45060,\
        0.45236, 0.45470, 0.45765, 0.46120, 0.46529, 0.46985, 0.47480, 0.48013, 0.48583, 0.49185, 0.49846,\
        0.50606, 0.51429, 0.52282, 0.53150, 0.54019, 0.54880, 0.55724, 0.56544, 0.57333, 0.58083, 0.58784,\
        0.59418, 0.59911, 0.60238, 0.60404, 0.60501, 0.60563, 0.60578, 0.60471, 0.60207, 0.59857, 0.59323,\
        0.58884, 0.58536, 0.58255, 0.58075, 0.58037, 0.58168, 0.58489, 0.58990, 0.59604, 0.60172, 0.60524,\
        0.60602, 0.60416, 0.60101, 0.59797, 0.59619, 0.59620, 0.59807, 0.60070, 0.60347, 0.60609, 0.60774,\
        0.60782, 0.60618, 0.60301, 0.59872, 0.59380, 0.58810, 0.58177, 0.57498, 0.56788, 0.56054, 0.55307,\
        0.54566, 0.53866, 0.53237, 0.52705, 0.52271, 0.51917, 0.51610, 0.51317, 0.51021, 0.50719, 0.50421,\
        0.50147, 0.49942, 0.49862, 0.49977, 0.50160, 0.50328, 0.50457, 0.50532, 0.50541, 0.50476, 0.50337,\
        0.50136, 0.49940, 0.49840, 0.49738, 0.49636, 0.49518, 0.49408, 0.49339, 0.49307, 0.49284, 0.49212,\
        0.49006, 0.48600, 0.47994, 0.47217, 0.46301, 0.45277, 0.44171, 0.43010, 0.41825, 0.40658, 0.39592],\
        [0.43657, 0.43765, 0.43828, 0.43857, 0.43864, 0.43864, 0.43869, 0.43893, 0.43950, 0.44050, 0.44205,\
        0.44424, 0.44710, 0.45064, 0.45482, 0.45958, 0.46484, 0.47053, 0.47662, 0.48310, 0.49003, 0.49751,\
        0.50511, 0.51290, 0.52111, 0.52952, 0.53796, 0.54629, 0.55442, 0.56228, 0.56979, 0.57686, 0.58334,\
        0.58895, 0.59330, 0.59604, 0.59723, 0.59770, 0.59783, 0.59761, 0.59616, 0.59297, 0.58887, 0.58394,\
        0.57928, 0.57555, 0.57291, 0.57161, 0.57197, 0.57423, 0.57846, 0.58444, 0.59146, 0.59813, 0.60296,\
        0.60416, 0.60189, 0.59873, 0.59616, 0.59472, 0.59477, 0.59619, 0.59836, 0.60088, 0.60363, 0.60510,\
        0.60474, 0.60237, 0.59842, 0.59340, 0.58766, 0.58130, 0.57440, 0.56710, 0.55952, 0.55181, 0.54410,\
        0.53665, 0.52977, 0.52379, 0.51901, 0.51550, 0.51296, 0.51085, 0.50873, 0.50644, 0.50404, 0.50168,\
        0.49959, 0.49820, 0.49794, 0.49891, 0.50042, 0.50171, 0.50278, 0.50344, 0.50353, 0.50296, 0.50171,\
        0.49982, 0.49776, 0.49618, 0.49485, 0.49357, 0.49229, 0.49109, 0.49010, 0.48926, 0.48833, 0.48686,\
        0.48428, 0.48017, 0.47441, 0.46711, 0.45850, 0.44879, 0.43823, 0.42707, 0.41563, 0.40434, 0.39391],\
        [0.42883, 0.42976, 0.43021, 0.43030, 0.43019, 0.43004, 0.42999, 0.43022, 0.43085, 0.43203, 0.43387,\
        0.43647, 0.43986, 0.44402, 0.44888, 0.45435, 0.46035, 0.46678, 0.47360, 0.48078, 0.48830, 0.49610,\
        0.50376, 0.51157, 0.51964, 0.52781, 0.53591, 0.54386, 0.55156, 0.55895, 0.56592, 0.57238, 0.57814,\
        0.58297, 0.58660, 0.58881, 0.58974, 0.58981, 0.58932, 0.58821, 0.58601, 0.58251, 0.57811, 0.57326,\
        0.56865, 0.56491, 0.56240, 0.56146, 0.56236, 0.56529, 0.57026, 0.57700, 0.58484, 0.59277, 0.59936,\
        0.60149, 0.59903, 0.59621, 0.59404, 0.59292, 0.59304, 0.59427, 0.59632, 0.59895, 0.60136, 0.60235,\
        0.60155, 0.59838, 0.59367, 0.58798, 0.58160, 0.57465, 0.56724, 0.55949, 0.55153, 0.54352, 0.53563,\
        0.52812, 0.52130, 0.51556, 0.51130, 0.50872, 0.50741, 0.50638, 0.50511, 0.50347, 0.50163, 0.49993,\
        0.49842, 0.49754, 0.49754, 0.49837, 0.49955, 0.50068, 0.50164, 0.50222, 0.50224, 0.50163, 0.50036,\
        0.49840, 0.49613, 0.49405, 0.49221, 0.49052, 0.48892, 0.48742, 0.48604, 0.48470, 0.48319, 0.48117,\
        0.47826, 0.47415, 0.46871, 0.46195, 0.45398, 0.44496, 0.43508, 0.42457, 0.41377, 0.40312, 0.39293],\
        [0.42154, 0.42238, 0.42269, 0.42261, 0.42233, 0.42203, 0.42190, 0.42209, 0.42276, 0.42406, 0.42616,\
        0.42915, 0.43307, 0.43785, 0.44340, 0.44960, 0.45632, 0.46348, 0.47097, 0.47870, 0.48656, 0.49436,\
        0.50217, 0.51028, 0.51833, 0.52623, 0.53395, 0.54143, 0.54861, 0.55541, 0.56175, 0.56750, 0.57250,\
        0.57656, 0.57949, 0.58118, 0.58172, 0.58132, 0.58015, 0.57818, 0.57524, 0.57131, 0.56667, 0.56177,\
        0.55718, 0.55345, 0.55103, 0.55026, 0.55144, 0.55474, 0.56014, 0.56732, 0.57564, 0.58409, 0.59116,\
        0.59445, 0.59378, 0.59204, 0.59064, 0.59004, 0.59040, 0.59165, 0.59361, 0.59591, 0.59775, 0.59823,\
        0.59698, 0.59339, 0.58830, 0.58222, 0.57542, 0.56807, 0.56030, 0.55223, 0.54401, 0.53582, 0.52784,\
        0.52032, 0.51358, 0.50806, 0.50420, 0.50257, 0.50266, 0.50258, 0.50207, 0.50106, 0.49956, 0.49836,\
        0.49740, 0.49694, 0.49712, 0.49787, 0.49890, 0.49996, 0.50084, 0.50138, 0.50121, 0.50032, 0.49875,\
        0.49655, 0.49404, 0.49156, 0.48923, 0.48706, 0.48504, 0.48313, 0.48131, 0.47950, 0.47752, 0.47513,\
        0.47204, 0.46804, 0.46297, 0.45679, 0.44955, 0.44133, 0.43226, 0.42252, 0.41236, 0.40218, 0.39231],\
        [0.41481, 0.41564, 0.41588, 0.41567, 0.41523, 0.41482, 0.41461, 0.41476, 0.41542, 0.41678, 0.41905,\
        0.42239, 0.42681, 0.43222, 0.43844, 0.44534, 0.45275, 0.46057, 0.46866, 0.47692, 0.48520, 0.49335,\
        0.50137, 0.50941, 0.51720, 0.52474, 0.53199, 0.53893, 0.54551, 0.55168, 0.55734, 0.56236, 0.56662,\
        0.56995, 0.57222, 0.57337, 0.57342, 0.57249, 0.57066, 0.56795, 0.56434, 0.55993, 0.55498, 0.54991,\
        0.54522, 0.54144, 0.53899, 0.53821, 0.53940, 0.54278, 0.54830, 0.55561, 0.56401, 0.57246, 0.57954,\
        0.58391, 0.58549, 0.58560, 0.58546, 0.58567, 0.58645, 0.58782, 0.58959, 0.59138, 0.59257, 0.59253,\
        0.59083, 0.58726, 0.58218, 0.57602, 0.56907, 0.56154, 0.55359, 0.54537, 0.53704, 0.52882, 0.52090,\
        0.51353, 0.50698, 0.50183, 0.49860, 0.49768, 0.49854, 0.49899, 0.49903, 0.49865, 0.49764, 0.49679,\
        0.49626, 0.49614, 0.49650, 0.49727, 0.49828, 0.49931, 0.50023, 0.50078, 0.50024, 0.49882, 0.49678,\
        0.49423, 0.49141, 0.48854, 0.48575, 0.48311, 0.48061, 0.47824, 0.47598, 0.47374, 0.47139, 0.46876,\
        0.46563, 0.46182, 0.45718, 0.45164, 0.44518, 0.43784, 0.42968, 0.42080, 0.41134, 0.40158, 0.39205],\
        [0.40871, 0.40966, 0.40993, 0.40966, 0.40912, 0.40863, 0.40839, 0.40851, 0.40909, 0.41037, 0.41269,\
        0.41630, 0.42121, 0.42720, 0.43406, 0.44159, 0.44961, 0.45801, 0.46665, 0.47541, 0.48414, 0.49272,\
        0.50101, 0.50872, 0.51613, 0.52320, 0.52992, 0.53628, 0.54225, 0.54776, 0.55273, 0.55706, 0.56062,\
        0.56329, 0.56495, 0.56555, 0.56510, 0.56363, 0.56121, 0.55787, 0.55368, 0.54879, 0.54347, 0.53813,\
        0.53324, 0.52932, 0.52675, 0.52577, 0.52670, 0.52988, 0.53530, 0.54255, 0.55088, 0.55929, 0.56663,\
        0.57204, 0.57539, 0.57732, 0.57861, 0.57979, 0.58114, 0.58270, 0.58432, 0.58570, 0.58638, 0.58591,\
        0.58397, 0.58045, 0.57553, 0.56950, 0.56263, 0.55512, 0.54718, 0.53897, 0.53069, 0.52258, 0.51490,\
        0.50793, 0.50190, 0.49728, 0.49474, 0.49406, 0.49464, 0.49534, 0.49573, 0.49575, 0.49539, 0.49501,\
        0.49485, 0.49501, 0.49553, 0.49638, 0.49744, 0.49858, 0.49967, 0.50022, 0.49897, 0.49691, 0.49432,\
        0.49136, 0.48817, 0.48491, 0.48170, 0.47859, 0.47562, 0.47279, 0.47009, 0.46747, 0.46483, 0.46206,\
        0.45898, 0.45545, 0.45130, 0.44644, 0.44081, 0.43438, 0.42720, 0.41932, 0.41076, 0.40160, 0.39229],\
        [0.40327, 0.40452, 0.40501, 0.40480, 0.40423, 0.40376, 0.40359, 0.40369, 0.40412, 0.40513, 0.40728,\
        0.41107, 0.41642, 0.42295, 0.43035, 0.43838, 0.44690, 0.45576, 0.46484, 0.47400, 0.48310, 0.49190,\
        0.50010, 0.50783, 0.51490, 0.52148, 0.52765, 0.53342, 0.53877, 0.54365, 0.54798, 0.55166, 0.55460,\
        0.55667, 0.55778, 0.55789, 0.55697, 0.55503, 0.55210, 0.54826, 0.54358, 0.53825, 0.53256, 0.52690,\
        0.52173, 0.51759, 0.51494, 0.51366, 0.51402, 0.51681, 0.52199, 0.52906, 0.53732, 0.54581, 0.55353,\
        0.55983, 0.56454, 0.56796, 0.57055, 0.57270, 0.57466, 0.57648, 0.57806, 0.57918, 0.57953, 0.57880,\
        0.57676, 0.57334, 0.56864, 0.56286, 0.55621, 0.54890, 0.54113, 0.53308, 0.52499, 0.51711, 0.50978,\
        0.50337, 0.49810, 0.49414, 0.49186, 0.49111, 0.49137, 0.49198, 0.49252, 0.49281, 0.49290, 0.49293,\
        0.49308, 0.49345, 0.49409, 0.49497, 0.49601, 0.49712, 0.49809, 0.49820, 0.49663, 0.49414, 0.49115,\
        0.48783, 0.48430, 0.48067, 0.47705, 0.47351, 0.47008, 0.46680, 0.46368, 0.46071, 0.45785, 0.45500,\
        0.45206, 0.44886, 0.44526, 0.44110, 0.43628, 0.43075, 0.42452, 0.41765, 0.41021, 0.40204, 0.39266],\
        [0.39840, 0.40020, 0.40123, 0.40138, 0.40093, 0.40065, 0.40065, 0.40074, 0.40095, 0.40144, 0.40305,\
        0.40696, 0.41270, 0.41961, 0.42737, 0.43574, 0.44457, 0.45373, 0.46310, 0.47254, 0.48190, 0.49099,\
        0.49957, 0.50675, 0.51332, 0.51942, 0.52506, 0.53027, 0.53505, 0.53934, 0.54309, 0.54621, 0.54860,\
        0.55017, 0.55083, 0.55052, 0.54921, 0.54689, 0.54359, 0.53936, 0.53433, 0.52866, 0.52265, 0.51672,\
        0.51131, 0.50683, 0.50433, 0.50284, 0.50232, 0.50468, 0.50944, 0.51617, 0.52435, 0.53296, 0.54101,\
        0.54794, 0.55362, 0.55816, 0.56181, 0.56482, 0.56735, 0.56946, 0.57110, 0.57211, 0.57230, 0.57145,\
        0.56942, 0.56616, 0.56173, 0.55627, 0.54996, 0.54297, 0.53551, 0.52777, 0.51999, 0.51243, 0.50546,\
        0.49958, 0.49499, 0.49162, 0.48955, 0.48868, 0.48866, 0.48906, 0.48956, 0.48998, 0.49029, 0.49058,\
        0.49093, 0.49142, 0.49209, 0.49289, 0.49377, 0.49456, 0.49500, 0.49457, 0.49293, 0.49034, 0.48716,\
        0.48359, 0.47978, 0.47583, 0.47185, 0.46790, 0.46404, 0.46033, 0.45681, 0.45350, 0.45043, 0.44757,\
        0.44481, 0.44201, 0.43896, 0.43551, 0.43147, 0.42675, 0.42130, 0.41519, 0.40851, 0.40119, 0.39177],\
        [0.39388, 0.39640, 0.39835, 0.39966, 0.40002, 0.39995, 0.39997, 0.39996, 0.39996, 0.39998, 0.40051,\
        0.40449, 0.41031, 0.41730, 0.42515, 0.43363, 0.44257, 0.45185, 0.46133, 0.47085, 0.48026, 0.48932,\
        0.49754, 0.50475, 0.51116, 0.51687, 0.52206, 0.52678, 0.53104, 0.53483, 0.53808, 0.54072, 0.54267,\
        0.54384, 0.54414, 0.54352, 0.54194, 0.53937, 0.53584, 0.53140, 0.52617, 0.52031, 0.51409, 0.50801,\
        0.50267, 0.49812, 0.49544, 0.49386, 0.49323, 0.49491, 0.49889, 0.50487, 0.51294, 0.52155, 0.52963,\
        0.53686, 0.54313, 0.54845, 0.55289, 0.55656, 0.55957, 0.56196, 0.56371, 0.56473, 0.56488, 0.56403,\
        0.56210, 0.55906, 0.55495, 0.54987, 0.54397, 0.53741, 0.53038, 0.52307, 0.51573, 0.50861, 0.50208,\
        0.49670, 0.49261, 0.48961, 0.48765, 0.48664, 0.48635, 0.48651, 0.48686, 0.48725, 0.48763, 0.48801,\
        0.48843, 0.48893, 0.48951, 0.49013, 0.49071, 0.49108, 0.49099, 0.49013, 0.48833, 0.48569, 0.48240,\
        0.47868, 0.47466, 0.47045, 0.46616, 0.46184, 0.45759, 0.45346, 0.44953, 0.44589, 0.44262, 0.43974,\
        0.43721, 0.43483, 0.43237, 0.42959, 0.42628, 0.42225, 0.41736, 0.41158, 0.40499, 0.39747, 0.38858],\
        [0.38937, 0.39261, 0.39544, 0.39781, 0.39920, 0.39955, 0.39968, 0.39977, 0.39984, 0.39994, 0.40071,\
        0.40390, 0.40923, 0.41593, 0.42359, 0.43195, 0.44081, 0.45002, 0.45942, 0.46885, 0.47811, 0.48695,\
        0.49507, 0.50232, 0.50843, 0.51378, 0.51859, 0.52290, 0.52673, 0.53009, 0.53294, 0.53521, 0.53683,\
        0.53770, 0.53777, 0.53696, 0.53523, 0.53256, 0.52896, 0.52448, 0.51924, 0.51343, 0.50725, 0.50105,\
        0.49597, 0.49178, 0.48880, 0.48704, 0.48655, 0.48786, 0.49115, 0.49646, 0.50413, 0.51208, 0.51966,\
        0.52682, 0.53339, 0.53919, 0.54414, 0.54828, 0.55164, 0.55427, 0.55615, 0.55722, 0.55742, 0.55666,\
        0.55490, 0.55213, 0.54837, 0.54373, 0.53831, 0.53226, 0.52575, 0.51898, 0.51219, 0.50566, 0.49972,\
        0.49479, 0.49094, 0.48806, 0.48609, 0.48490, 0.48434, 0.48422, 0.48435, 0.48461, 0.48491, 0.48525,\
        0.48561, 0.48600, 0.48641, 0.48677, 0.48699, 0.48693, 0.48638, 0.48516, 0.48316, 0.48041, 0.47705,\
        0.47322, 0.46904, 0.46462, 0.46007, 0.45544, 0.45082, 0.44629, 0.44195, 0.43795, 0.43443, 0.43155,\
        0.42925, 0.42733, 0.42545, 0.42331, 0.42066, 0.41722, 0.41275, 0.40709, 0.40030, 0.39277, 0.38403],\
        [0.38459, 0.38838, 0.39174, 0.39454, 0.39648, 0.39757, 0.39818, 0.39859, 0.39897, 0.39957, 0.40098,\
        0.40404, 0.40889, 0.41515, 0.42246, 0.43053, 0.43916, 0.44814, 0.45732, 0.46651, 0.47552, 0.48413,\
        0.49213, 0.49921, 0.50496, 0.51010, 0.51467, 0.51864, 0.52210, 0.52513, 0.52767, 0.52967, 0.53106,\
        0.53175, 0.53169, 0.53082, 0.52909, 0.52648, 0.52297, 0.51861, 0.51354, 0.50804, 0.50229, 0.49646,\
        0.49134, 0.48720, 0.48414, 0.48227, 0.48177, 0.48291, 0.48589, 0.49084, 0.49747, 0.50421, 0.51101,\
        0.51791, 0.52458, 0.53063, 0.53587, 0.54026, 0.54383, 0.54660, 0.54858, 0.54974, 0.55003, 0.54942,\
        0.54788, 0.54541, 0.54205, 0.53788, 0.53300, 0.52752, 0.52160, 0.51542, 0.50923, 0.50334, 0.49800,\
        0.49342, 0.48972, 0.48685, 0.48475, 0.48335, 0.48251, 0.48209, 0.48195, 0.48198, 0.48211, 0.48229,\
        0.48249, 0.48269, 0.48285, 0.48290, 0.48275, 0.48228, 0.48134, 0.47979, 0.47758, 0.47471, 0.47126,\
        0.46733, 0.46303, 0.45846, 0.45369, 0.44881, 0.44388, 0.43899, 0.43425, 0.42982, 0.42599, 0.42303,\
        0.42101, 0.41958, 0.41823, 0.41664, 0.41457, 0.41165, 0.40751, 0.40210, 0.39520, 0.38741, 0.37866],\
        [0.37938, 0.38356, 0.38727, 0.39040, 0.39280, 0.39450, 0.39569, 0.39663, 0.39755, 0.39878, 0.40077,\
        0.40399, 0.40862, 0.41452, 0.42146, 0.42917, 0.43746, 0.44612, 0.45497, 0.46384, 0.47251, 0.48079,\
        0.48842, 0.49510, 0.50081, 0.50604, 0.51042, 0.51400, 0.51715, 0.51993, 0.52227, 0.52410, 0.52535,\
        0.52596, 0.52588, 0.52506, 0.52347, 0.52107, 0.51784, 0.51377, 0.50894, 0.50376, 0.49858, 0.49321,\
        0.48823, 0.48408, 0.48095, 0.47901, 0.47843, 0.47939, 0.48200, 0.48623, 0.49162, 0.49739, 0.50357,\
        0.51019, 0.51687, 0.52296, 0.52823, 0.53266, 0.53628, 0.53911, 0.54115, 0.54238, 0.54279, 0.54235,\
        0.54105, 0.53891, 0.53598, 0.53231, 0.52800, 0.52314, 0.51786, 0.51230, 0.50667, 0.50139, 0.49658,\
        0.49233, 0.48872, 0.48579, 0.48352, 0.48186, 0.48072, 0.47999, 0.47955, 0.47931, 0.47919, 0.47914,\
        0.47910, 0.47905, 0.47891, 0.47863, 0.47812, 0.47726, 0.47597, 0.47413, 0.47171, 0.46870, 0.46515,\
        0.46114, 0.45675, 0.45206, 0.44714, 0.44207, 0.43692, 0.43174, 0.42664, 0.42177, 0.41747, 0.41432,\
        0.41263, 0.41173, 0.41080, 0.40952, 0.40792, 0.40549, 0.40141, 0.39599, 0.38920, 0.38131, 0.37252],\
        [0.37371, 0.37820, 0.38220, 0.38564, 0.38847, 0.39071, 0.39249, 0.39404, 0.39560, 0.39746, 0.39997,\
        0.40344, 0.40803, 0.41370, 0.42031, 0.42767, 0.43558, 0.44386, 0.45233, 0.46082, 0.46912, 0.47704,\
        0.48436, 0.49093, 0.49677, 0.50208, 0.50588, 0.50894, 0.51187, 0.51450, 0.51673, 0.51848, 0.51968,\
        0.52029, 0.52027, 0.51960, 0.51824, 0.51618, 0.51341, 0.50989, 0.50553, 0.50042, 0.49576, 0.49081,\
        0.48603, 0.48191, 0.47876, 0.47676, 0.47606, 0.47678, 0.47896, 0.48246, 0.48695, 0.49204, 0.49757,\
        0.50389, 0.51047, 0.51626, 0.52128, 0.52555, 0.52908, 0.53187, 0.53392, 0.53521, 0.53572, 0.53546,\
        0.53441, 0.53262, 0.53011, 0.52695, 0.52323, 0.51904, 0.51446, 0.50959, 0.50448, 0.49956, 0.49533,\
        0.49132, 0.48774, 0.48470, 0.48223, 0.48031, 0.47887, 0.47781, 0.47706, 0.47651, 0.47611, 0.47577,\
        0.47546, 0.47510, 0.47465, 0.47403, 0.47316, 0.47196, 0.47035, 0.46825, 0.46562, 0.46247, 0.45883,\
        0.45474, 0.45028, 0.44551, 0.44051, 0.43533, 0.43005, 0.42472, 0.41941, 0.41419, 0.40930, 0.40572,\
        0.40445, 0.40407, 0.40329, 0.40198, 0.40048, 0.39854, 0.39433, 0.38882, 0.38210, 0.37431, 0.36558],\
        [0.36761, 0.37237, 0.37665, 0.38042, 0.38365, 0.38639, 0.38875, 0.39091, 0.39309, 0.39554, 0.39853,\
        0.40228, 0.40693, 0.41247, 0.41882, 0.42585, 0.43339, 0.44128, 0.44936, 0.45744, 0.46535, 0.47293,\
        0.48002, 0.48652, 0.49246, 0.49757, 0.50049, 0.50342, 0.50632, 0.50891, 0.51109, 0.51280, 0.51401,\
        0.51467, 0.51477, 0.51428, 0.51322, 0.51157, 0.50936, 0.50661, 0.50334, 0.49932, 0.49409, 0.48901,\
        0.48429, 0.48024, 0.47711, 0.47507, 0.47425, 0.47473, 0.47650, 0.47944, 0.48334, 0.48799, 0.49332,\
        0.49947, 0.50522, 0.51032, 0.51488, 0.51885, 0.52220, 0.52489, 0.52690, 0.52822, 0.52884, 0.52874,\
        0.52794, 0.52648, 0.52438, 0.52172, 0.51857, 0.51503, 0.51120, 0.50717, 0.50304, 0.49874, 0.49434,\
        0.49026, 0.48658, 0.48340, 0.48074, 0.47857, 0.47683, 0.47547, 0.47440, 0.47354, 0.47282, 0.47218,\
        0.47155, 0.47088, 0.47010, 0.46915, 0.46795, 0.46643, 0.46453, 0.46218, 0.45937, 0.45609, 0.45235,\
        0.44820, 0.44369, 0.43889, 0.43385, 0.42864, 0.42333, 0.41798, 0.41268, 0.40751, 0.40244, 0.39832,\
        0.39701, 0.39657, 0.39567, 0.39428, 0.39256, 0.39014, 0.38616, 0.38074, 0.37413, 0.36647, 0.35785]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=30 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax30vt5
    zgrid_H_Hmax_vmax30vt5=[[0.28501, 0.28700, 0.28888, 0.29082, 0.29294, 0.29531, 0.29790, 0.30034, 0.30218, 0.30405, 0.30600,\
        0.30806, 0.31020, 0.31243, 0.31472, 0.31708, 0.31950, 0.32197, 0.32450, 0.32709, 0.32973, 0.33243,\
        0.33521, 0.33805, 0.34098, 0.34399, 0.34709, 0.35030, 0.35362, 0.35707, 0.36066, 0.36442, 0.36835,\
        0.37250, 0.37691, 0.38161, 0.38663, 0.39196, 0.39747, 0.40273, 0.40784, 0.41299, 0.41816, 0.42334,\
        0.42848, 0.43353, 0.43845, 0.44319, 0.44772, 0.45198, 0.45594, 0.45958, 0.46289, 0.46591, 0.46874,\
        0.47148, 0.47423, 0.47705, 0.47991, 0.48272, 0.48532, 0.48774, 0.49034, 0.49351, 0.49711, 0.50084,\
        0.50454, 0.50814, 0.51156, 0.51474, 0.51764, 0.52023, 0.52249, 0.52441, 0.52600, 0.52727, 0.52823,\
        0.52890, 0.52930, 0.52945, 0.52937, 0.52908, 0.52860, 0.52795, 0.52714, 0.52617, 0.52506, 0.52381,\
        0.52242, 0.52089, 0.51921, 0.51737, 0.51537, 0.51319, 0.51082, 0.50825, 0.50545, 0.50239, 0.49902,\
        0.49522, 0.49094, 0.48639, 0.48169, 0.47690, 0.47206, 0.46716, 0.46221, 0.45721, 0.45211, 0.44689,\
        0.44152, 0.43595, 0.43016, 0.42411, 0.41778, 0.41116, 0.40424, 0.39701, 0.38945, 0.38158, 0.37339],\
        [0.28882, 0.29022, 0.29147, 0.29280, 0.29439, 0.29628, 0.29841, 0.30062, 0.30288, 0.30508, 0.30730,\
        0.30957, 0.31190, 0.31427, 0.31669, 0.31915, 0.32165, 0.32419, 0.32677, 0.32939, 0.33206, 0.33479,\
        0.33757, 0.34042, 0.34334, 0.34634, 0.34944, 0.35263, 0.35593, 0.35935, 0.36290, 0.36659, 0.37046,\
        0.37454, 0.37887, 0.38350, 0.38848, 0.39383, 0.39949, 0.40511, 0.41064, 0.41623, 0.42184, 0.42745,\
        0.43299, 0.43843, 0.44373, 0.44883, 0.45370, 0.45828, 0.46253, 0.46640, 0.46986, 0.47296, 0.47582,\
        0.47859, 0.48143, 0.48441, 0.48753, 0.49067, 0.49346, 0.49582, 0.49830, 0.50163, 0.50545, 0.50933,\
        0.51317, 0.51687, 0.52036, 0.52358, 0.52650, 0.52907, 0.53128, 0.53314, 0.53463, 0.53579, 0.53663,\
        0.53717, 0.53743, 0.53745, 0.53723, 0.53681, 0.53621, 0.53545, 0.53453, 0.53347, 0.53228, 0.53096,\
        0.52951, 0.52792, 0.52618, 0.52430, 0.52226, 0.52005, 0.51765, 0.51504, 0.51221, 0.50912, 0.50572,\
        0.50189, 0.49756, 0.49299, 0.48830, 0.48355, 0.47876, 0.47395, 0.46911, 0.46423, 0.45928, 0.45422,\
        0.44901, 0.44359, 0.43793, 0.43201, 0.42579, 0.41926, 0.41242, 0.40526, 0.39777, 0.38996, 0.38181],\
        [0.29244, 0.29318, 0.29372, 0.29439, 0.29539, 0.29682, 0.29866, 0.30085, 0.30333, 0.30586, 0.30838,\
        0.31091, 0.31344, 0.31599, 0.31854, 0.32112, 0.32370, 0.32631, 0.32894, 0.33160, 0.33430, 0.33704,\
        0.33983, 0.34268, 0.34560, 0.34859, 0.35167, 0.35485, 0.35813, 0.36152, 0.36502, 0.36866, 0.37246,\
        0.37643, 0.38064, 0.38514, 0.39000, 0.39524, 0.40092, 0.40696, 0.41308, 0.41921, 0.42532, 0.43139,\
        0.43737, 0.44323, 0.44892, 0.45441, 0.45964, 0.46458, 0.46916, 0.47332, 0.47698, 0.48015, 0.48300,\
        0.48580, 0.48872, 0.49185, 0.49521, 0.49875, 0.50187, 0.50440, 0.50709, 0.51044, 0.51425, 0.51818,\
        0.52207, 0.52581, 0.52933, 0.53257, 0.53546, 0.53799, 0.54013, 0.54188, 0.54326, 0.54428, 0.54497,\
        0.54536, 0.54546, 0.54532, 0.54496, 0.54439, 0.54365, 0.54276, 0.54173, 0.54057, 0.53929, 0.53789,\
        0.53636, 0.53471, 0.53291, 0.53098, 0.52889, 0.52662, 0.52417, 0.52151, 0.51861, 0.51545, 0.51196,\
        0.50807, 0.50378, 0.49925, 0.49459, 0.48990, 0.48519, 0.48048, 0.47575, 0.47101, 0.46623, 0.46134,\
        0.45629, 0.45103, 0.44552, 0.43971, 0.43360, 0.42716, 0.42039, 0.41330, 0.40586, 0.39810, 0.39000],\
        [0.29598, 0.29597, 0.29577, 0.29578, 0.29623, 0.29732, 0.29908, 0.30137, 0.30397, 0.30672, 0.30950,\
        0.31225, 0.31498, 0.31769, 0.32038, 0.32305, 0.32572, 0.32839, 0.33106, 0.33376, 0.33647, 0.33923,\
        0.34202, 0.34487, 0.34778, 0.35077, 0.35384, 0.35700, 0.36026, 0.36363, 0.36710, 0.37070, 0.37442,\
        0.37829, 0.38236, 0.38671, 0.39147, 0.39673, 0.40251, 0.40889, 0.41548, 0.42212, 0.42873, 0.43527,\
        0.44169, 0.44796, 0.45405, 0.45992, 0.46552, 0.47083, 0.47579, 0.48032, 0.48425, 0.48750, 0.49031,\
        0.49315, 0.49623, 0.49950, 0.50308, 0.50677, 0.51018, 0.51320, 0.51627, 0.51969, 0.52344, 0.52733,\
        0.53120, 0.53494, 0.53845, 0.54167, 0.54452, 0.54698, 0.54901, 0.55063, 0.55185, 0.55271, 0.55323,\
        0.55344, 0.55338, 0.55307, 0.55253, 0.55180, 0.55092, 0.54989, 0.54873, 0.54747, 0.54609, 0.54460,\
        0.54299, 0.54127, 0.53941, 0.53742, 0.53526, 0.53294, 0.53042, 0.52769, 0.52472, 0.52147, 0.51791,\
        0.51400, 0.50976, 0.50527, 0.50066, 0.49604, 0.49141, 0.48678, 0.48217, 0.47758, 0.47297, 0.46827,\
        0.46340, 0.45830, 0.45292, 0.44723, 0.44121, 0.43485, 0.42815, 0.42111, 0.41373, 0.40601, 0.39796],\
        [0.29943, 0.29858, 0.29781, 0.29728, 0.29720, 0.29801, 0.29977, 0.30221, 0.30499, 0.30791, 0.31086,\
        0.31379, 0.31666, 0.31949, 0.32228, 0.32504, 0.32777, 0.33048, 0.33319, 0.33590, 0.33863, 0.34138,\
        0.34418, 0.34702, 0.34992, 0.35289, 0.35595, 0.35910, 0.36236, 0.36572, 0.36918, 0.37276, 0.37645,\
        0.38025, 0.38420, 0.38840, 0.39303, 0.39836, 0.40445, 0.41109, 0.41807, 0.42514, 0.43220, 0.43916,\
        0.44599, 0.45265, 0.45911, 0.46535, 0.47132, 0.47698, 0.48231, 0.48728, 0.49166, 0.49504, 0.49782,\
        0.50076, 0.50410, 0.50760, 0.51132, 0.51508, 0.51870, 0.52211, 0.52551, 0.52906, 0.53281, 0.53665,\
        0.54048, 0.54418, 0.54767, 0.55084, 0.55364, 0.55599, 0.55789, 0.55934, 0.56038, 0.56105, 0.56138,\
        0.56141, 0.56117, 0.56067, 0.55995, 0.55904, 0.55799, 0.55682, 0.55554, 0.55416, 0.55268, 0.55110,\
        0.54942, 0.54762, 0.54569, 0.54363, 0.54141, 0.53902, 0.53643, 0.53363, 0.53059, 0.52728, 0.52367,\
        0.51976, 0.51557, 0.51116, 0.50663, 0.50206, 0.49748, 0.49291, 0.48839, 0.48396, 0.47955, 0.47505,\
        0.47036, 0.46541, 0.46016, 0.45457, 0.44863, 0.44233, 0.43569, 0.42870, 0.42136, 0.41368, 0.40566],\
        [0.30239, 0.30131, 0.30026, 0.29931, 0.29868, 0.29934, 0.30119, 0.30374, 0.30661, 0.30961, 0.31264,\
        0.31564, 0.31860, 0.32150, 0.32434, 0.32714, 0.32990, 0.33264, 0.33536, 0.33807, 0.34080, 0.34354,\
        0.34632, 0.34915, 0.35203, 0.35499, 0.35803, 0.36118, 0.36443, 0.36780, 0.37129, 0.37490, 0.37861,\
        0.38241, 0.38631, 0.39042, 0.39496, 0.40018, 0.40659, 0.41364, 0.42097, 0.42840, 0.43583, 0.44315,\
        0.45033, 0.45732, 0.46412, 0.47069, 0.47700, 0.48296, 0.48857, 0.49392, 0.49887, 0.50260, 0.50560,\
        0.50880, 0.51229, 0.51598, 0.51980, 0.52366, 0.52743, 0.53110, 0.53474, 0.53843, 0.54220, 0.54601,\
        0.54978, 0.55344, 0.55688, 0.56002, 0.56275, 0.56498, 0.56671, 0.56797, 0.56881, 0.56927, 0.56941,\
        0.56924, 0.56880, 0.56811, 0.56719, 0.56610, 0.56489, 0.56357, 0.56216, 0.56067, 0.55909, 0.55742,\
        0.55565, 0.55377, 0.55178, 0.54965, 0.54736, 0.54490, 0.54225, 0.53938, 0.53628, 0.53292, 0.52929,\
        0.52540, 0.52127, 0.51694, 0.51249, 0.50798, 0.50344, 0.49892, 0.49447, 0.49020, 0.48601, 0.48171,\
        0.47719, 0.47238, 0.46724, 0.46173, 0.45584, 0.44960, 0.44300, 0.43604, 0.42874, 0.42110, 0.41311],\
        [0.30564, 0.30462, 0.30340, 0.30223, 0.30143, 0.30196, 0.30378, 0.30622, 0.30900, 0.31194, 0.31493,\
        0.31792, 0.32087, 0.32377, 0.32662, 0.32942, 0.33217, 0.33490, 0.33760, 0.34030, 0.34301, 0.34573,\
        0.34848, 0.35128, 0.35414, 0.35707, 0.36010, 0.36323, 0.36649, 0.36989, 0.37343, 0.37712, 0.38094,\
        0.38485, 0.38883, 0.39290, 0.39746, 0.40302, 0.40950, 0.41669, 0.42425, 0.43195, 0.43966, 0.44728,\
        0.45473, 0.46200, 0.46907, 0.47593, 0.48255, 0.48880, 0.49456, 0.50005, 0.50527, 0.50959, 0.51326,\
        0.51685, 0.52058, 0.52445, 0.52840, 0.53236, 0.53627, 0.54011, 0.54390, 0.54769, 0.55149, 0.55527,\
        0.55899, 0.56259, 0.56600, 0.56912, 0.57179, 0.57390, 0.57543, 0.57646, 0.57707, 0.57732, 0.57725,\
        0.57691, 0.57629, 0.57539, 0.57425, 0.57297, 0.57158, 0.57013, 0.56860, 0.56700, 0.56532, 0.56356,\
        0.56170, 0.55975, 0.55769, 0.55549, 0.55314, 0.55061, 0.54790, 0.54497, 0.54182, 0.53843, 0.53480,\
        0.53093, 0.52685, 0.52261, 0.51825, 0.51381, 0.50934, 0.50488, 0.50048, 0.49641, 0.49243, 0.48828,\
        0.48389, 0.47920, 0.47414, 0.46869, 0.46284, 0.45663, 0.45006, 0.44314, 0.43587, 0.42826, 0.42031],\
        [0.30944, 0.30848, 0.30732, 0.30625, 0.30567, 0.30608, 0.30753, 0.30966, 0.31219, 0.31494, 0.31779,\
        0.32067, 0.32353, 0.32636, 0.32915, 0.33190, 0.33461, 0.33730, 0.33996, 0.34262, 0.34528, 0.34797,\
        0.35068, 0.35344, 0.35626, 0.35915, 0.36215, 0.36527, 0.36853, 0.37195, 0.37556, 0.37936, 0.38336,\
        0.38752, 0.39177, 0.39598, 0.40049, 0.40624, 0.41290, 0.42019, 0.42788, 0.43578, 0.44371, 0.45155,\
        0.45922, 0.46668, 0.47393, 0.48101, 0.48794, 0.49455, 0.50051, 0.50612, 0.51147, 0.51627, 0.52058,\
        0.52467, 0.52874, 0.53284, 0.53695, 0.54104, 0.54507, 0.54904, 0.55294, 0.55680, 0.56061, 0.56436,\
        0.56800, 0.57153, 0.57490, 0.57802, 0.58069, 0.58266, 0.58395, 0.58474, 0.58513, 0.58515, 0.58489,\
        0.58438, 0.58360, 0.58250, 0.58113, 0.57963, 0.57809, 0.57652, 0.57488, 0.57317, 0.57139, 0.56954,\
        0.56760, 0.56558, 0.56345, 0.56118, 0.55877, 0.55618, 0.55340, 0.55043, 0.54724, 0.54383, 0.54019,\
        0.53636, 0.53234, 0.52817, 0.52390, 0.51957, 0.51521, 0.51088, 0.50667, 0.50270, 0.49881, 0.49473,\
        0.49043, 0.48585, 0.48085, 0.47542, 0.46958, 0.46339, 0.45685, 0.44996, 0.44273, 0.43515, 0.42723],\
        [0.31378, 0.31294, 0.31195, 0.31109, 0.31068, 0.31101, 0.31214, 0.31391, 0.31610, 0.31857, 0.32118,\
        0.32387, 0.32658, 0.32928, 0.33196, 0.33462, 0.33724, 0.33985, 0.34245, 0.34504, 0.34764, 0.35027,\
        0.35292, 0.35563, 0.35839, 0.36124, 0.36420, 0.36728, 0.37052, 0.37396, 0.37762, 0.38153, 0.38571,\
        0.39017, 0.39488, 0.39965, 0.40427, 0.40995, 0.41660, 0.42395, 0.43176, 0.43982, 0.44795, 0.45599,\
        0.46382, 0.47139, 0.47872, 0.48587, 0.49295, 0.49995, 0.50624, 0.51208, 0.51763, 0.52282, 0.52764,\
        0.53222, 0.53667, 0.54105, 0.54537, 0.54960, 0.55374, 0.55780, 0.56178, 0.56568, 0.56949, 0.57318,\
        0.57672, 0.58013, 0.58343, 0.58658, 0.58934, 0.59118, 0.59214, 0.59270, 0.59291, 0.59271, 0.59226,\
        0.59165, 0.59075, 0.58944, 0.58779, 0.58607, 0.58442, 0.58277, 0.58103, 0.57921, 0.57733, 0.57538,\
        0.57337, 0.57128, 0.56908, 0.56675, 0.56427, 0.56162, 0.55879, 0.55577, 0.55255, 0.54912, 0.54549,\
        0.54168, 0.53771, 0.53362, 0.52944, 0.52522, 0.52100, 0.51683, 0.51277, 0.50886, 0.50498, 0.50095,\
        0.49676, 0.49229, 0.48733, 0.48187, 0.47602, 0.46984, 0.46334, 0.45650, 0.44931, 0.44176, 0.43386],\
        [0.31861, 0.31793, 0.31714, 0.31647, 0.31618, 0.31644, 0.31731, 0.31874, 0.32059, 0.32273, 0.32506,\
        0.32750, 0.33000, 0.33252, 0.33504, 0.33756, 0.34007, 0.34257, 0.34507, 0.34758, 0.35010, 0.35265,\
        0.35523, 0.35786, 0.36056, 0.36335, 0.36624, 0.36927, 0.37246, 0.37587, 0.37954, 0.38351, 0.38782,\
        0.39248, 0.39740, 0.40250, 0.40769, 0.41355, 0.42029, 0.42775, 0.43572, 0.44397, 0.45231, 0.46055,\
        0.46854, 0.47618, 0.48349, 0.49060, 0.49766, 0.50473, 0.51148, 0.51774, 0.52364, 0.52920, 0.53447,\
        0.53949, 0.54433, 0.54902, 0.55355, 0.55794, 0.56217, 0.56629, 0.57032, 0.57426, 0.57808, 0.58170,\
        0.58511, 0.58834, 0.59146, 0.59455, 0.59752, 0.59926, 0.59984, 0.60029, 0.60039, 0.59997, 0.59936,\
        0.59867, 0.59772, 0.59619, 0.59419, 0.59227, 0.59063, 0.58894, 0.58709, 0.58515, 0.58316, 0.58112,\
        0.57903, 0.57687, 0.57461, 0.57222, 0.56967, 0.56696, 0.56408, 0.56101, 0.55775, 0.55430, 0.55067,\
        0.54688, 0.54296, 0.53893, 0.53485, 0.53073, 0.52664, 0.52259, 0.51864, 0.51476, 0.51089, 0.50693,\
        0.50282, 0.49844, 0.49346, 0.48794, 0.48208, 0.47596, 0.46952, 0.46274, 0.45559, 0.44808, 0.44021],\
        [0.32387, 0.32335, 0.32273, 0.32222, 0.32199, 0.32218, 0.32286, 0.32399, 0.32551, 0.32732, 0.32934,\
        0.33150, 0.33375, 0.33605, 0.33838, 0.34073, 0.34309, 0.34546, 0.34784, 0.35023, 0.35266, 0.35511,\
        0.35761, 0.36016, 0.36278, 0.36548, 0.36829, 0.37123, 0.37435, 0.37768, 0.38130, 0.38525, 0.38958,\
        0.39431, 0.39943, 0.40469, 0.41037, 0.41666, 0.42370, 0.43141, 0.43961, 0.44812, 0.45671, 0.46519,\
        0.47338, 0.48112, 0.48838, 0.49541, 0.50247, 0.50966, 0.51662, 0.52321, 0.52945, 0.53538, 0.54104,\
        0.54647, 0.55168, 0.55669, 0.56146, 0.56598, 0.57029, 0.57443, 0.57848, 0.58246, 0.58630, 0.58988,\
        0.59314, 0.59616, 0.59905, 0.60194, 0.60468, 0.60645, 0.60717, 0.60749, 0.60745, 0.60700, 0.60631,\
        0.60546, 0.60432, 0.60263, 0.60037, 0.59833, 0.59688, 0.59509, 0.59308, 0.59102, 0.58893, 0.58679,\
        0.58461, 0.58238, 0.58007, 0.57761, 0.57499, 0.57221, 0.56927, 0.56616, 0.56286, 0.55939, 0.55575,\
        0.55196, 0.54806, 0.54409, 0.54007, 0.53605, 0.53205, 0.52809, 0.52420, 0.52035, 0.51648, 0.51254,\
        0.50844, 0.50404, 0.49905, 0.49349, 0.48773, 0.48171, 0.47538, 0.46867, 0.46158, 0.45410, 0.44625],\
        [0.32945, 0.32907, 0.32861, 0.32821, 0.32801, 0.32814, 0.32863, 0.32951, 0.33072, 0.33222, 0.33392,\
        0.33579, 0.33777, 0.33983, 0.34194, 0.34409, 0.34628, 0.34849, 0.35073, 0.35301, 0.35531, 0.35767,\
        0.36007, 0.36252, 0.36505, 0.36766, 0.37037, 0.37320, 0.37619, 0.37939, 0.38288, 0.38672, 0.39098,\
        0.39567, 0.40077, 0.40637, 0.41249, 0.41926, 0.42672, 0.43478, 0.44332, 0.45213, 0.46103, 0.46981,\
        0.47830, 0.48626, 0.49348, 0.50041, 0.50757, 0.51475, 0.52180, 0.52859, 0.53509, 0.54133, 0.54734,\
        0.55313, 0.55871, 0.56404, 0.56905, 0.57370, 0.57802, 0.58214, 0.58618, 0.59018, 0.59408, 0.59765,\
        0.60078, 0.60367, 0.60642, 0.60906, 0.61143, 0.61310, 0.61399, 0.61433, 0.61424, 0.61378, 0.61303,\
        0.61201, 0.61066, 0.60886, 0.60671, 0.60475, 0.60313, 0.60117, 0.59904, 0.59687, 0.59467, 0.59243,\
        0.59015, 0.58785, 0.58548, 0.58295, 0.58024, 0.57737, 0.57437, 0.57121, 0.56788, 0.56437, 0.56069,\
        0.55689, 0.55300, 0.54906, 0.54509, 0.54113, 0.53720, 0.53330, 0.52944, 0.52559, 0.52171, 0.51773,\
        0.51355, 0.50903, 0.50401, 0.49859, 0.49300, 0.48716, 0.48095, 0.47431, 0.46726, 0.45980, 0.45198],\
        [0.33527, 0.33503, 0.33468, 0.33435, 0.33416, 0.33421, 0.33455, 0.33519, 0.33613, 0.33732, 0.33873,\
        0.34030, 0.34200, 0.34380, 0.34568, 0.34762, 0.34962, 0.35166, 0.35375, 0.35588, 0.35807, 0.36030,\
        0.36260, 0.36496, 0.36739, 0.36989, 0.37249, 0.37519, 0.37803, 0.38106, 0.38433, 0.38797, 0.39209,\
        0.39677, 0.40187, 0.40762, 0.41415, 0.42142, 0.42935, 0.43783, 0.44674, 0.45591, 0.46514, 0.47424,\
        0.48305, 0.49148, 0.49891, 0.50570, 0.51275, 0.51988, 0.52695, 0.53385, 0.54054, 0.54703, 0.55331,\
        0.55942, 0.56533, 0.57101, 0.57630, 0.58105, 0.58532, 0.58937, 0.59335, 0.59733, 0.60125, 0.60483,\
        0.60796, 0.61081, 0.61347, 0.61591, 0.61802, 0.61958, 0.62051, 0.62090, 0.62083, 0.62036, 0.61956,\
        0.61845, 0.61700, 0.61521, 0.61321, 0.61124, 0.60933, 0.60728, 0.60506, 0.60278, 0.60046, 0.59807,\
        0.59567, 0.59331, 0.59089, 0.58826, 0.58540, 0.58243, 0.57938, 0.57618, 0.57280, 0.56923, 0.56550,\
        0.56166, 0.55775, 0.55382, 0.54988, 0.54596, 0.54207, 0.53821, 0.53436, 0.53050, 0.52658, 0.52254,\
        0.51829, 0.51373, 0.50877, 0.50350, 0.49805, 0.49237, 0.48626, 0.47965, 0.47261, 0.46518, 0.45738],\
        [0.34126, 0.34112, 0.34085, 0.34057, 0.34037, 0.34034, 0.34052, 0.34096, 0.34164, 0.34256, 0.34367,\
        0.34496, 0.34639, 0.34793, 0.34957, 0.35129, 0.35308, 0.35494, 0.35686, 0.35885, 0.36090, 0.36302,\
        0.36521, 0.36747, 0.36980, 0.37220, 0.37469, 0.37726, 0.37994, 0.38276, 0.38577, 0.38907, 0.39284,\
        0.39734, 0.40248, 0.40848, 0.41549, 0.42327, 0.43166, 0.44056, 0.44985, 0.45937, 0.46893, 0.47831,\
        0.48724, 0.49574, 0.50372, 0.51070, 0.51773, 0.52486, 0.53195, 0.53893, 0.54576, 0.55242, 0.55892,\
        0.56527, 0.57147, 0.57749, 0.58315, 0.58803, 0.59212, 0.59610, 0.60004, 0.60400, 0.60788, 0.61147,\
        0.61468, 0.61758, 0.62020, 0.62253, 0.62449, 0.62597, 0.62691, 0.62733, 0.62728, 0.62683, 0.62601,\
        0.62486, 0.62338, 0.62164, 0.61970, 0.61770, 0.61564, 0.61347, 0.61117, 0.60879, 0.60633, 0.60381,\
        0.60126, 0.59880, 0.59632, 0.59353, 0.59041, 0.58736, 0.58430, 0.58109, 0.57764, 0.57397, 0.57015,\
        0.56624, 0.56230, 0.55836, 0.55443, 0.55053, 0.54666, 0.54281, 0.53896, 0.53508, 0.53113, 0.52704,\
        0.52275, 0.51819, 0.51334, 0.50823, 0.50291, 0.49738, 0.49127, 0.48463, 0.47759, 0.47019, 0.46242],\
        [0.34734, 0.34728, 0.34708, 0.34681, 0.34657, 0.34645, 0.34649, 0.34673, 0.34719, 0.34785, 0.34869,\
        0.34971, 0.35087, 0.35216, 0.35355, 0.35505, 0.35664, 0.35831, 0.36006, 0.36189, 0.36381, 0.36580,\
        0.36788, 0.37004, 0.37228, 0.37459, 0.37698, 0.37945, 0.38198, 0.38461, 0.38735, 0.39027, 0.39354,\
        0.39736, 0.40246, 0.40920, 0.41676, 0.42497, 0.43375, 0.44300, 0.45261, 0.46244, 0.47232, 0.48203,\
        0.49111, 0.49890, 0.50711, 0.51486, 0.52224, 0.52951, 0.53669, 0.54377, 0.55070, 0.55749, 0.56412,\
        0.57063, 0.57701, 0.58329, 0.58940, 0.59472, 0.59824, 0.60228, 0.60634, 0.61034, 0.61421, 0.61782,\
        0.62111, 0.62407, 0.62671, 0.62900, 0.63090, 0.63234, 0.63328, 0.63372, 0.63370, 0.63326, 0.63245,\
        0.63129, 0.62983, 0.62812, 0.62621, 0.62418, 0.62203, 0.61977, 0.61738, 0.61489, 0.61231, 0.60967,\
        0.60700, 0.60435, 0.60171, 0.59859, 0.59519, 0.59219, 0.58919, 0.58596, 0.58240, 0.57857, 0.57461,\
        0.57061, 0.56662, 0.56266, 0.55873, 0.55484, 0.55098, 0.54713, 0.54326, 0.53936, 0.53537, 0.53125,\
        0.52694, 0.52241, 0.51764, 0.51265, 0.50745, 0.50194, 0.49579, 0.48916, 0.48217, 0.47482, 0.46711],\
        [0.35345, 0.35346, 0.35328, 0.35300, 0.35271, 0.35249, 0.35239, 0.35245, 0.35270, 0.35312, 0.35372,\
        0.35448, 0.35539, 0.35643, 0.35759, 0.35887, 0.36025, 0.36173, 0.36331, 0.36499, 0.36676, 0.36864,\
        0.37061, 0.37267, 0.37483, 0.37707, 0.37938, 0.38177, 0.38421, 0.38671, 0.38927, 0.39193, 0.39487,\
        0.39860, 0.40381, 0.41053, 0.41827, 0.42671, 0.43571, 0.44518, 0.45500, 0.46504, 0.47516, 0.48518,\
        0.49488, 0.50333, 0.51091, 0.51870, 0.52633, 0.53379, 0.54112, 0.54832, 0.55536, 0.56224, 0.56895,\
        0.57552, 0.58195, 0.58820, 0.59431, 0.60036, 0.60417, 0.60813, 0.61233, 0.61642, 0.62035, 0.62401,\
        0.62738, 0.63041, 0.63310, 0.63542, 0.63732, 0.63876, 0.63971, 0.64017, 0.64017, 0.63975, 0.63894,\
        0.63780, 0.63636, 0.63467, 0.63277, 0.63071, 0.62851, 0.62616, 0.62369, 0.62110, 0.61840, 0.61562,\
        0.61277, 0.60987, 0.60682, 0.60335, 0.60001, 0.59709, 0.59411, 0.59082, 0.58705, 0.58297, 0.57884,\
        0.57474, 0.57070, 0.56673, 0.56280, 0.55891, 0.55504, 0.55118, 0.54729, 0.54336, 0.53933, 0.53518,\
        0.53086, 0.52634, 0.52160, 0.51664, 0.51142, 0.50582, 0.49973, 0.49322, 0.48632, 0.47906, 0.47144],\
        [0.35953, 0.35957, 0.35940, 0.35909, 0.35873, 0.35840, 0.35817, 0.35806, 0.35811, 0.35832, 0.35870,\
        0.35923, 0.35990, 0.36071, 0.36165, 0.36271, 0.36388, 0.36518, 0.36659, 0.36811, 0.36975, 0.37150,\
        0.37337, 0.37535, 0.37743, 0.37961, 0.38188, 0.38421, 0.38661, 0.38908, 0.39162, 0.39421, 0.39697,\
        0.40062, 0.40587, 0.41246, 0.42009, 0.42853, 0.43758, 0.44712, 0.45702, 0.46716, 0.47736, 0.48741,\
        0.49703, 0.50597, 0.51420, 0.52219, 0.53004, 0.53772, 0.54524, 0.55258, 0.55975, 0.56672, 0.57350,\
        0.58009, 0.58649, 0.59264, 0.59829, 0.60398, 0.60921, 0.61376, 0.61815, 0.62239, 0.62640, 0.63015,\
        0.63361, 0.63673, 0.63950, 0.64188, 0.64383, 0.64529, 0.64626, 0.64674, 0.64676, 0.64635, 0.64556,\
        0.64443, 0.64301, 0.64133, 0.63943, 0.63734, 0.63508, 0.63267, 0.63011, 0.62741, 0.62458, 0.62164,\
        0.61859, 0.61543, 0.61209, 0.60859, 0.60523, 0.60210, 0.59902, 0.59564, 0.59144, 0.58705, 0.58277,\
        0.57862, 0.57456, 0.57058, 0.56664, 0.56275, 0.55887, 0.55499, 0.55108, 0.54711, 0.54304, 0.53885,\
        0.53449, 0.52996, 0.52521, 0.52022, 0.51495, 0.50932, 0.50330, 0.49688, 0.49008, 0.48294, 0.47543],\
        [0.36552, 0.36558, 0.36538, 0.36501, 0.36457, 0.36413, 0.36375, 0.36349, 0.36337, 0.36340, 0.36358,\
        0.36390, 0.36436, 0.36496, 0.36568, 0.36653, 0.36751, 0.36863, 0.36987, 0.37124, 0.37274, 0.37437,\
        0.37614, 0.37803, 0.38005, 0.38219, 0.38442, 0.38674, 0.38912, 0.39158, 0.39420, 0.39701, 0.39971,\
        0.40332, 0.40836, 0.41470, 0.42210, 0.43037, 0.43932, 0.44881, 0.45869, 0.46882, 0.47903, 0.48909,\
        0.49874, 0.50800, 0.51678, 0.52518, 0.53335, 0.54131, 0.54906, 0.55660, 0.56393, 0.57102, 0.57787,\
        0.58449, 0.59093, 0.59722, 0.60313, 0.60878, 0.61425, 0.61928, 0.62395, 0.62835, 0.63248, 0.63633,\
        0.63989, 0.64313, 0.64601, 0.64849, 0.65051, 0.65203, 0.65303, 0.65352, 0.65354, 0.65313, 0.65234,\
        0.65121, 0.64980, 0.64812, 0.64621, 0.64409, 0.64178, 0.63930, 0.63665, 0.63384, 0.63087, 0.62776,\
        0.62451, 0.62112, 0.61759, 0.61400, 0.61047, 0.60706, 0.60371, 0.59993, 0.59525, 0.59071, 0.58644,\
        0.58230, 0.57824, 0.57425, 0.57030, 0.56640, 0.56250, 0.55859, 0.55464, 0.55062, 0.54651, 0.54227,\
        0.53787, 0.53329, 0.52850, 0.52346, 0.51815, 0.51250, 0.50650, 0.50014, 0.49346, 0.48650, 0.47912],\
        [0.37135, 0.37140, 0.37115, 0.37070, 0.37015, 0.36960, 0.36910, 0.36870, 0.36844, 0.36831, 0.36832,\
        0.36846, 0.36873, 0.36913, 0.36965, 0.37031, 0.37111, 0.37205, 0.37313, 0.37436, 0.37572, 0.37723,\
        0.37889, 0.38070, 0.38265, 0.38475, 0.38697, 0.38927, 0.39164, 0.39403, 0.39654, 0.39954, 0.40250,\
        0.40607, 0.41090, 0.41696, 0.42409, 0.43212, 0.44088, 0.45023, 0.46000, 0.47007, 0.48028, 0.49046,\
        0.50039, 0.50983, 0.51894, 0.52776, 0.53630, 0.54459, 0.55263, 0.56041, 0.56795, 0.57523, 0.58223,\
        0.58894, 0.59542, 0.60187, 0.60815, 0.61409, 0.61973, 0.62500, 0.62989, 0.63445, 0.63869, 0.64265,\
        0.64632, 0.64969, 0.65272, 0.65534, 0.65747, 0.65906, 0.66009, 0.66059, 0.66059, 0.66015, 0.65934,\
        0.65820, 0.65678, 0.65509, 0.65316, 0.65099, 0.64863, 0.64607, 0.64332, 0.64039, 0.63728, 0.63399,\
        0.63054, 0.62694, 0.62321, 0.61942, 0.61560, 0.61177, 0.60778, 0.60332, 0.59848, 0.59412, 0.59000,\
        0.58589, 0.58181, 0.57778, 0.57382, 0.56989, 0.56597, 0.56202, 0.55802, 0.55394, 0.54976, 0.54546,\
        0.54099, 0.53635, 0.53150, 0.52642, 0.52107, 0.51541, 0.50942, 0.50309, 0.49656, 0.48985, 0.48257],\
        [0.37697, 0.37698, 0.37665, 0.37609, 0.37542, 0.37475, 0.37413, 0.37363, 0.37325, 0.37300, 0.37288,\
        0.37287, 0.37298, 0.37319, 0.37354, 0.37402, 0.37465, 0.37543, 0.37636, 0.37745, 0.37868, 0.38006,\
        0.38161, 0.38332, 0.38520, 0.38726, 0.38945, 0.39175, 0.39411, 0.39645, 0.39855, 0.40109, 0.40449,\
        0.40836, 0.41316, 0.41901, 0.42588, 0.43365, 0.44219, 0.45134, 0.46097, 0.47094, 0.48109, 0.49131,\
        0.50141, 0.51121, 0.52073, 0.52999, 0.53896, 0.54762, 0.55598, 0.56405, 0.57186, 0.57941, 0.58669,\
        0.59362, 0.60029, 0.60699, 0.61354, 0.61977, 0.62562, 0.63107, 0.63612, 0.64080, 0.64514, 0.64920,\
        0.65299, 0.65651, 0.65972, 0.66253, 0.66481, 0.66648, 0.66753, 0.66801, 0.66798, 0.66748, 0.66662,\
        0.66545, 0.66401, 0.66229, 0.66031, 0.65808, 0.65564, 0.65299, 0.65015, 0.64709, 0.64383, 0.64035,\
        0.63669, 0.63287, 0.62891, 0.62485, 0.62069, 0.61641, 0.61190, 0.60707, 0.60218, 0.59790, 0.59374,\
        0.58950, 0.58532, 0.58124, 0.57725, 0.57328, 0.56931, 0.56530, 0.56123, 0.55708, 0.55283, 0.54844,\
        0.54390, 0.53918, 0.53426, 0.52912, 0.52374, 0.51810, 0.51218, 0.50601, 0.49969, 0.49315, 0.48573],\
        [0.38231, 0.38225, 0.38180, 0.38110, 0.38030, 0.37950, 0.37879, 0.37821, 0.37776, 0.37744, 0.37723,\
        0.37711, 0.37707, 0.37713, 0.37731, 0.37763, 0.37810, 0.37874, 0.37955, 0.38050, 0.38160, 0.38286,\
        0.38427, 0.38587, 0.38767, 0.38966, 0.39182, 0.39408, 0.39642, 0.39881, 0.40093, 0.40317, 0.40636,\
        0.41027, 0.41501, 0.42069, 0.42732, 0.43484, 0.44315, 0.45213, 0.46161, 0.47146, 0.48153, 0.49172,\
        0.50197, 0.51219, 0.52222, 0.53198, 0.54139, 0.55045, 0.55916, 0.56754, 0.57565, 0.58355, 0.59123,\
        0.59861, 0.60566, 0.61261, 0.61940, 0.62587, 0.63193, 0.63755, 0.64273, 0.64751, 0.65193, 0.65607,\
        0.65997, 0.66366, 0.66709, 0.67015, 0.67263, 0.67439, 0.67543, 0.67588, 0.67578, 0.67518, 0.67422,\
        0.67301, 0.67154, 0.66978, 0.66771, 0.66538, 0.66283, 0.66009, 0.65714, 0.65395, 0.65051, 0.64684,\
        0.64296, 0.63890, 0.63469, 0.63034, 0.62586, 0.62123, 0.61641, 0.61146, 0.60662, 0.60211, 0.59765,\
        0.59313, 0.58880, 0.58467, 0.58063, 0.57661, 0.57257, 0.56847, 0.56432, 0.56008, 0.55572, 0.55124,\
        0.54661, 0.54180, 0.53679, 0.53159, 0.52616, 0.52052, 0.51469, 0.50875, 0.50278, 0.49606, 0.48832],\
        [0.38729, 0.38712, 0.38650, 0.38563, 0.38468, 0.38378, 0.38301, 0.38240, 0.38194, 0.38160, 0.38135,\
        0.38115, 0.38101, 0.38093, 0.38095, 0.38112, 0.38146, 0.38199, 0.38268, 0.38353, 0.38451, 0.38562,\
        0.38688, 0.38833, 0.39001, 0.39193, 0.39403, 0.39623, 0.39842, 0.40063, 0.40282, 0.40515, 0.40813,\
        0.41189, 0.41647, 0.42194, 0.42833, 0.43561, 0.44372, 0.45254, 0.46192, 0.47171, 0.48173, 0.49193,\
        0.50237, 0.51301, 0.52356, 0.53380, 0.54366, 0.55313, 0.56220, 0.57088, 0.57927, 0.58752, 0.59572,\
        0.60358, 0.61114, 0.61852, 0.62566, 0.63242, 0.63872, 0.64451, 0.64982, 0.65468, 0.65916, 0.66336,\
        0.66735, 0.67120, 0.67489, 0.67828, 0.68106, 0.68288, 0.68384, 0.68425, 0.68411, 0.68333, 0.68221,\
        0.68093, 0.67946, 0.67763, 0.67540, 0.67290, 0.67022, 0.66738, 0.66432, 0.66098, 0.65735, 0.65346,\
        0.64933, 0.64502, 0.64055, 0.63593, 0.63117, 0.62626, 0.62123, 0.61615, 0.61115, 0.60629, 0.60147,\
        0.59672, 0.59230, 0.58814, 0.58404, 0.57992, 0.57577, 0.57156, 0.56729, 0.56294, 0.55848, 0.55389,\
        0.54914, 0.54422, 0.53911, 0.53381, 0.52831, 0.52261, 0.51673, 0.51070, 0.50440, 0.49751, 0.49001],\
        [0.39182, 0.39148, 0.39063, 0.38956, 0.38848, 0.38752, 0.38673, 0.38615, 0.38573, 0.38545, 0.38522,\
        0.38500, 0.38477, 0.38456, 0.38444, 0.38448, 0.38471, 0.38516, 0.38579, 0.38655, 0.38742, 0.38838,\
        0.38946, 0.39072, 0.39223, 0.39403, 0.39608, 0.39821, 0.40025, 0.40225, 0.40436, 0.40671, 0.40958,\
        0.41314, 0.41749, 0.42271, 0.42884, 0.43588, 0.44380, 0.45250, 0.46187, 0.47173, 0.48187, 0.49214,\
        0.50275, 0.51390, 0.52489, 0.53552, 0.54578, 0.55565, 0.56512, 0.57415, 0.58275, 0.59117, 0.59983,\
        0.60835, 0.61662, 0.62463, 0.63227, 0.63942, 0.64601, 0.65201, 0.65747, 0.66242, 0.66696, 0.67119,\
        0.67526, 0.67925, 0.68318, 0.68696, 0.69021, 0.69206, 0.69277, 0.69322, 0.69312, 0.69199, 0.69064,\
        0.68931, 0.68785, 0.68591, 0.68339, 0.68062, 0.67780, 0.67488, 0.67171, 0.66820, 0.66434, 0.66018,\
        0.65579, 0.65122, 0.64650, 0.64163, 0.63662, 0.63148, 0.62625, 0.62098, 0.61574, 0.61056, 0.60542,\
        0.60044, 0.59596, 0.59176, 0.58753, 0.58324, 0.57893, 0.57458, 0.57018, 0.56570, 0.56111, 0.55639,\
        0.55152, 0.54648, 0.54125, 0.53582, 0.53020, 0.52437, 0.51833, 0.51203, 0.50539, 0.49837, 0.49113],\
        [0.39576, 0.39517, 0.39404, 0.39279, 0.39162, 0.39064, 0.38990, 0.38941, 0.38912, 0.38897, 0.38883,\
        0.38864, 0.38836, 0.38803, 0.38776, 0.38767, 0.38784, 0.38826, 0.38887, 0.38959, 0.39038, 0.39120,\
        0.39207, 0.39307, 0.39433, 0.39596, 0.39793, 0.39998, 0.40185, 0.40370, 0.40568, 0.40794, 0.41066,\
        0.41399, 0.41807, 0.42298, 0.42880, 0.43557, 0.44329, 0.45191, 0.46135, 0.47145, 0.48200, 0.49276,\
        0.50375, 0.51505, 0.52619, 0.53706, 0.54765, 0.55794, 0.56791, 0.57743, 0.58635, 0.59471, 0.60363,\
        0.61305, 0.62221, 0.63099, 0.63927, 0.64691, 0.65384, 0.66009, 0.66575, 0.67085, 0.67545, 0.67973,\
        0.68387, 0.68798, 0.69211, 0.69615, 0.69998, 0.70191, 0.70245, 0.70291, 0.70278, 0.70130, 0.69976,\
        0.69829, 0.69685, 0.69466, 0.69163, 0.68854, 0.68559, 0.68260, 0.67932, 0.67559, 0.67143, 0.66696,\
        0.66229, 0.65747, 0.65251, 0.64742, 0.64219, 0.63684, 0.63140, 0.62593, 0.62046, 0.61504, 0.60973,\
        0.60464, 0.60002, 0.59560, 0.59107, 0.58655, 0.58204, 0.57753, 0.57298, 0.56837, 0.56364, 0.55879,\
        0.55377, 0.54859, 0.54322, 0.53765, 0.53188, 0.52588, 0.51965, 0.51315, 0.50635, 0.49932, 0.49213],\
        [0.39890, 0.39792, 0.39655, 0.39523, 0.39406, 0.39313, 0.39249, 0.39216, 0.39208, 0.39214, 0.39219,\
        0.39208, 0.39177, 0.39133, 0.39090, 0.39068, 0.39082, 0.39129, 0.39196, 0.39271, 0.39345, 0.39415,\
        0.39479, 0.39548, 0.39643, 0.39777, 0.39957, 0.40149, 0.40326, 0.40500, 0.40684, 0.40893, 0.41143,\
        0.41447, 0.41820, 0.42273, 0.42817, 0.43460, 0.44207, 0.45059, 0.46012, 0.47054, 0.48167, 0.49330,\
        0.50482, 0.51601, 0.52715, 0.53816, 0.54903, 0.55978, 0.57036, 0.58067, 0.59045, 0.59933, 0.60841,\
        0.61823, 0.62808, 0.63765, 0.64668, 0.65492, 0.66224, 0.66877, 0.67471, 0.68008, 0.68482, 0.68917,\
        0.69346, 0.69779, 0.70209, 0.70632, 0.70996, 0.71225, 0.71319, 0.71347, 0.71300, 0.71165, 0.70994,\
        0.70815, 0.70626, 0.70360, 0.70011, 0.69672, 0.69365, 0.69061, 0.68717, 0.68313, 0.67857, 0.67374,\
        0.66877, 0.66372, 0.65855, 0.65326, 0.64783, 0.64228, 0.63664, 0.63096, 0.62529, 0.61968, 0.61423,\
        0.60903, 0.60417, 0.59939, 0.59454, 0.58979, 0.58509, 0.58041, 0.57571, 0.57095, 0.56609, 0.56108,\
        0.55592, 0.55059, 0.54506, 0.53934, 0.53341, 0.52726, 0.52087, 0.51423, 0.50736, 0.50029, 0.49313],\
        [0.40051, 0.39939, 0.39815, 0.39692, 0.39583, 0.39500, 0.39451, 0.39438, 0.39457, 0.39495, 0.39528,\
        0.39532, 0.39502, 0.39445, 0.39384, 0.39349, 0.39367, 0.39431, 0.39512, 0.39593, 0.39668, 0.39731,\
        0.39775, 0.39810, 0.39876, 0.39978, 0.40130, 0.40303, 0.40468, 0.40627, 0.40793, 0.40978, 0.41197,\
        0.41464, 0.41793, 0.42198, 0.42693, 0.43290, 0.44001, 0.44833, 0.45786, 0.46854, 0.48014, 0.49225,\
        0.50428, 0.51594, 0.52725, 0.53843, 0.54963, 0.56086, 0.57208, 0.58325, 0.59421, 0.60415, 0.61353,\
        0.62361, 0.63411, 0.64457, 0.65452, 0.66350, 0.67123, 0.67797, 0.68430, 0.69021, 0.69521, 0.69981,\
        0.70442, 0.70907, 0.71356, 0.71771, 0.72115, 0.72346, 0.72459, 0.72477, 0.72411, 0.72270, 0.72079,\
        0.71857, 0.71601, 0.71285, 0.70919, 0.70551, 0.70213, 0.69886, 0.69518, 0.69067, 0.68560, 0.68039,\
        0.67518, 0.66994, 0.66461, 0.65914, 0.65352, 0.64776, 0.64191, 0.63602, 0.63014, 0.62435, 0.61871,\
        0.61328, 0.60806, 0.60293, 0.59787, 0.59292, 0.58804, 0.58321, 0.57837, 0.57347, 0.56846, 0.56330,\
        0.55798, 0.55249, 0.54681, 0.54094, 0.53485, 0.52856, 0.52204, 0.51531, 0.50840, 0.50135, 0.49417],\
        [0.40116, 0.40025, 0.39914, 0.39806, 0.39707, 0.39635, 0.39601, 0.39611, 0.39658, 0.39732, 0.39807,\
        0.39828, 0.39807, 0.39742, 0.39661, 0.39609, 0.39639, 0.39738, 0.39840, 0.39928, 0.40007, 0.40067,\
        0.40101, 0.40117, 0.40165, 0.40246, 0.40363, 0.40499, 0.40635, 0.40769, 0.40907, 0.41060, 0.41239,\
        0.41459, 0.41734, 0.42079, 0.42510, 0.43045, 0.43702, 0.44495, 0.45432, 0.46509, 0.47706, 0.48975,\
        0.50247, 0.51457, 0.52603, 0.53750, 0.54916, 0.56096, 0.57276, 0.58446, 0.59606, 0.60684, 0.61741,\
        0.62848, 0.63996, 0.65150, 0.66264, 0.67269, 0.68090, 0.68762, 0.69427, 0.70108, 0.70667, 0.71180,\
        0.71679, 0.72162, 0.72615, 0.73016, 0.73338, 0.73556, 0.73664, 0.73669, 0.73584, 0.73422, 0.73200,\
        0.72930, 0.72617, 0.72258, 0.71866, 0.71469, 0.71086, 0.70706, 0.70292, 0.69791, 0.69232, 0.68682,\
        0.68147, 0.67612, 0.67067, 0.66503, 0.65921, 0.65323, 0.64716, 0.64104, 0.63495, 0.62894, 0.62307,\
        0.61737, 0.61183, 0.60642, 0.60113, 0.59599, 0.59090, 0.58592, 0.58097, 0.57594, 0.57078, 0.56546,\
        0.55997, 0.55432, 0.54849, 0.54247, 0.53625, 0.52982, 0.52319, 0.51637, 0.50941, 0.50235, 0.49514],\
        [0.40175, 0.40097, 0.39996, 0.39894, 0.39798, 0.39730, 0.39710, 0.39743, 0.39817, 0.39913, 0.40032,\
        0.40076, 0.40078, 0.40023, 0.39938, 0.39863, 0.39913, 0.40056, 0.40182, 0.40282, 0.40364, 0.40424,\
        0.40457, 0.40478, 0.40512, 0.40569, 0.40649, 0.40743, 0.40841, 0.40939, 0.41039, 0.41150, 0.41282,\
        0.41445, 0.41654, 0.41925, 0.42276, 0.42729, 0.43308, 0.44036, 0.44929, 0.45993, 0.47211, 0.48546,\
        0.49917, 0.51139, 0.52298, 0.53506, 0.54754, 0.56013, 0.57259, 0.58465, 0.59597, 0.60791, 0.62031,\
        0.63276, 0.64541, 0.65814, 0.67068, 0.68236, 0.69144, 0.69799, 0.70490, 0.71225, 0.71881, 0.72464,\
        0.73003, 0.73502, 0.73951, 0.74336, 0.74636, 0.74833, 0.74920, 0.74902, 0.74791, 0.74599, 0.74337,\
        0.74019, 0.73654, 0.73249, 0.72820, 0.72381, 0.71942, 0.71494, 0.71008, 0.70460, 0.69872, 0.69309,\
        0.68770, 0.68231, 0.67673, 0.67090, 0.66485, 0.65863, 0.65231, 0.64596, 0.63963, 0.63338, 0.62727,\
        0.62131, 0.61549, 0.60982, 0.60429, 0.59887, 0.59361, 0.58860, 0.58356, 0.57838, 0.57304, 0.56754,\
        0.56189, 0.55608, 0.55010, 0.54395, 0.53761, 0.53108, 0.52436, 0.51744, 0.51037, 0.50319, 0.49599],\
        [0.40237, 0.40170, 0.40078, 0.39977, 0.39875, 0.39802, 0.39793, 0.39855, 0.39961, 0.40077, 0.40202,\
        0.40289, 0.40319, 0.40297, 0.40240, 0.40197, 0.40265, 0.40409, 0.40551, 0.40667, 0.40756, 0.40818,\
        0.40856, 0.40879, 0.40902, 0.40936, 0.40981, 0.41034, 0.41089, 0.41143, 0.41199, 0.41261, 0.41337,\
        0.41435, 0.41569, 0.41753, 0.42006, 0.42354, 0.42828, 0.43459, 0.44275, 0.45290, 0.46490, 0.47834,\
        0.49227, 0.50497, 0.51762, 0.53112, 0.54491, 0.55861, 0.57205, 0.58518, 0.59779, 0.61022, 0.62331,\
        0.63682, 0.65054, 0.66429, 0.67789, 0.69121, 0.70229, 0.70944, 0.71683, 0.72443, 0.73158, 0.73802,\
        0.74378, 0.74889, 0.75333, 0.75700, 0.75977, 0.76147, 0.76203, 0.76152, 0.76008, 0.75779, 0.75474,\
        0.75105, 0.74686, 0.74233, 0.73756, 0.73267, 0.72770, 0.72257, 0.71713, 0.71130, 0.70528, 0.69945,\
        0.69401, 0.68854, 0.68277, 0.67671, 0.67039, 0.66389, 0.65730, 0.65069, 0.64412, 0.63763, 0.63128,\
        0.62507, 0.61902, 0.61310, 0.60729, 0.60156, 0.59634, 0.59135, 0.58617, 0.58079, 0.57524, 0.56955,\
        0.56372, 0.55776, 0.55165, 0.54539, 0.53897, 0.53236, 0.52557, 0.51860, 0.51145, 0.50415, 0.49696],\
        [0.40311, 0.40252, 0.40166, 0.40065, 0.39957, 0.39875, 0.39873, 0.39970, 0.40112, 0.40262, 0.40404,\
        0.40516, 0.40584, 0.40608, 0.40609, 0.40629, 0.40709, 0.40840, 0.40980, 0.41101, 0.41195, 0.41260,\
        0.41300, 0.41320, 0.41332, 0.41342, 0.41354, 0.41366, 0.41378, 0.41386, 0.41393, 0.41402, 0.41417,\
        0.41445, 0.41496, 0.41583, 0.41723, 0.41944, 0.42282, 0.42782, 0.43485, 0.44416, 0.45561, 0.46867,\
        0.48239, 0.49598, 0.51069, 0.52630, 0.54173, 0.55673, 0.57124, 0.58540, 0.59949, 0.61261, 0.62651,\
        0.64103, 0.65573, 0.67029, 0.68438, 0.69755, 0.71022, 0.72027, 0.72898, 0.73720, 0.74485, 0.75176,\
        0.75780, 0.76294, 0.76724, 0.77072, 0.77326, 0.77465, 0.77481, 0.77387, 0.77206, 0.76939, 0.76587,\
        0.76163, 0.75690, 0.75185, 0.74659, 0.74119, 0.73568, 0.73002, 0.72415, 0.71810, 0.71199, 0.70608,\
        0.70046, 0.69476, 0.68872, 0.68234, 0.67572, 0.66892, 0.66205, 0.65517, 0.64835, 0.64163, 0.63506,\
        0.62864, 0.62240, 0.61632, 0.61041, 0.60474, 0.59949, 0.59423, 0.58876, 0.58311, 0.57732, 0.57143,\
        0.56544, 0.55934, 0.55313, 0.54679, 0.54030, 0.53366, 0.52686, 0.51990, 0.51280, 0.50561, 0.49851],\
        [0.40405, 0.40355, 0.40276, 0.40179, 0.40075, 0.39998, 0.40004, 0.40132, 0.40306, 0.40484, 0.40651,\
        0.40792, 0.40899, 0.40975, 0.41036, 0.41107, 0.41209, 0.41338, 0.41473, 0.41594, 0.41688, 0.41754,\
        0.41791, 0.41805, 0.41802, 0.41788, 0.41768, 0.41741, 0.41708, 0.41670, 0.41627, 0.41580, 0.41534,\
        0.41490, 0.41457, 0.41441, 0.41459, 0.41535, 0.41710, 0.42041, 0.42596, 0.43418, 0.44498, 0.45781,\
        0.47201, 0.48731, 0.50408, 0.52138, 0.53845, 0.55491, 0.57053, 0.58511, 0.59894, 0.61430, 0.63008,\
        0.64586, 0.66155, 0.67697, 0.69187, 0.70602, 0.71932, 0.73107, 0.74117, 0.75017, 0.75835, 0.76568,\
        0.77188, 0.77688, 0.78091, 0.78413, 0.78643, 0.78752, 0.78716, 0.78569, 0.78355, 0.78057, 0.77652,\
        0.77170, 0.76643, 0.76088, 0.75515, 0.74928, 0.74329, 0.73720, 0.73101, 0.72476, 0.71855, 0.71251,\
        0.70664, 0.70067, 0.69435, 0.68765, 0.68071, 0.67362, 0.66646, 0.65932, 0.65226, 0.64533, 0.63856,\
        0.63197, 0.62558, 0.61940, 0.61346, 0.60783, 0.60245, 0.59696, 0.59117, 0.58522, 0.57920, 0.57313,\
        0.56701, 0.56081, 0.55453, 0.54813, 0.54161, 0.53496, 0.52819, 0.52129, 0.51431, 0.50731, 0.50030],\
        [0.40526, 0.40490, 0.40428, 0.40351, 0.40275, 0.40231, 0.40266, 0.40394, 0.40575, 0.40769, 0.40958,\
        0.41127, 0.41273, 0.41397, 0.41509, 0.41624, 0.41752, 0.41889, 0.42024, 0.42144, 0.42237, 0.42299,\
        0.42331, 0.42334, 0.42314, 0.42275, 0.42222, 0.42156, 0.42081, 0.41996, 0.41903, 0.41803, 0.41696,\
        0.41584, 0.41469, 0.41354, 0.41250, 0.41174, 0.41164, 0.41292, 0.41661, 0.42359, 0.43377, 0.44650,\
        0.46132, 0.47819, 0.49688, 0.51617, 0.53520, 0.55351, 0.57082, 0.58700, 0.60230, 0.61836, 0.63516,\
        0.65194, 0.66856, 0.68495, 0.70094, 0.71620, 0.73036, 0.74297, 0.75384, 0.76330, 0.77182, 0.77954,\
        0.78584, 0.79045, 0.79400, 0.79683, 0.79881, 0.79957, 0.79861, 0.79657, 0.79425, 0.79109, 0.78639,\
        0.78096, 0.77523, 0.76928, 0.76313, 0.75684, 0.75046, 0.74401, 0.73754, 0.73107, 0.72469, 0.71842,\
        0.71224, 0.70598, 0.69943, 0.69246, 0.68524, 0.67787, 0.67045, 0.66307, 0.65579, 0.64867, 0.64172,\
        0.63498, 0.62846, 0.62217, 0.61615, 0.61040, 0.60484, 0.59918, 0.59312, 0.58696, 0.58079, 0.57461,\
        0.56840, 0.56215, 0.55581, 0.54939, 0.54286, 0.53623, 0.52949, 0.52265, 0.51575, 0.50879, 0.50171],\
        [0.40681, 0.40665, 0.40629, 0.40583, 0.40548, 0.40548, 0.40609, 0.40740, 0.40920, 0.41122, 0.41327,\
        0.41522, 0.41703, 0.41869, 0.42025, 0.42178, 0.42332, 0.42483, 0.42625, 0.42746, 0.42838, 0.42896,\
        0.42918, 0.42908, 0.42867, 0.42802, 0.42716, 0.42612, 0.42495, 0.42365, 0.42224, 0.42072, 0.41909,\
        0.41734, 0.41546, 0.41344, 0.41131, 0.40914, 0.40719, 0.40616, 0.40754, 0.41315, 0.42270, 0.43504,\
        0.44987, 0.46756, 0.48813, 0.51014, 0.53183, 0.55259, 0.57213, 0.59060, 0.60795, 0.62476, 0.64210,\
        0.65968, 0.67717, 0.69447, 0.71163, 0.72797, 0.74296, 0.75617, 0.76733, 0.77664, 0.78500, 0.79298,\
        0.79934, 0.80338, 0.80635, 0.80860, 0.80994, 0.81009, 0.80879, 0.80651, 0.80391, 0.80041, 0.79496,\
        0.78916, 0.78318, 0.77695, 0.77046, 0.76381, 0.75707, 0.75032, 0.74358, 0.73689, 0.73027, 0.72373,\
        0.71722, 0.71063, 0.70380, 0.69667, 0.68923, 0.68160, 0.67394, 0.66635, 0.65890, 0.65161, 0.64452,\
        0.63763, 0.63097, 0.62455, 0.61836, 0.61238, 0.60653, 0.60061, 0.59447, 0.58826, 0.58205, 0.57585,\
        0.56961, 0.56333, 0.55698, 0.55056, 0.54403, 0.53742, 0.53071, 0.52391, 0.51703, 0.51005, 0.50293],\
        [0.40869, 0.40879, 0.40874, 0.40867, 0.40873, 0.40912, 0.41000, 0.41141, 0.41324, 0.41534, 0.41753,\
        0.41971, 0.42182, 0.42384, 0.42578, 0.42766, 0.42946, 0.43116, 0.43269, 0.43395, 0.43487, 0.43540,\
        0.43552, 0.43525, 0.43463, 0.43370, 0.43250, 0.43108, 0.42949, 0.42775, 0.42588, 0.42388, 0.42174,\
        0.41944, 0.41694, 0.41421, 0.41121, 0.40796, 0.40453, 0.40138, 0.40010, 0.40413, 0.41266, 0.42375,\
        0.43745, 0.45492, 0.47725, 0.50278, 0.52822, 0.55216, 0.57415, 0.59446, 0.61361, 0.63203, 0.65047,\
        0.66913, 0.68773, 0.70609, 0.72419, 0.74136, 0.75701, 0.77069, 0.78189, 0.79051, 0.79792, 0.80550,\
        0.81153, 0.81536, 0.81784, 0.81939, 0.81998, 0.81948, 0.81785, 0.81532, 0.81208, 0.80779, 0.80225,\
        0.79639, 0.79033, 0.78390, 0.77709, 0.77010, 0.76307, 0.75606, 0.74908, 0.74215, 0.73526, 0.72841,\
        0.72156, 0.71464, 0.70757, 0.70027, 0.69259, 0.68473, 0.67688, 0.66914, 0.66154, 0.65413, 0.64692,\
        0.63991, 0.63312, 0.62653, 0.62013, 0.61389, 0.60772, 0.60156, 0.59538, 0.58921, 0.58305, 0.57687,\
        0.57065, 0.56438, 0.55804, 0.55161, 0.54510, 0.53851, 0.53182, 0.52503, 0.51815, 0.51115, 0.50401],\
        [0.41088, 0.41128, 0.41157, 0.41188, 0.41233, 0.41308, 0.41422, 0.41579, 0.41772, 0.41991, 0.42225,\
        0.42464, 0.42702, 0.42936, 0.43164, 0.43383, 0.43591, 0.43783, 0.43951, 0.44085, 0.44179, 0.44228,\
        0.44230, 0.44186, 0.44100, 0.43977, 0.43823, 0.43643, 0.43442, 0.43225, 0.42994, 0.42749, 0.42488,\
        0.42209, 0.41905, 0.41573, 0.41208, 0.40811, 0.40389, 0.39956, 0.39657, 0.39822, 0.40393, 0.41284,\
        0.42396, 0.43958, 0.46377, 0.49402, 0.52446, 0.55261, 0.57768, 0.59964, 0.62054, 0.64041, 0.66011,\
        0.68020, 0.70031, 0.71986, 0.73867, 0.75629, 0.77228, 0.78620, 0.79750, 0.80541, 0.81197, 0.81819,\
        0.82320, 0.82652, 0.82844, 0.82927, 0.82913, 0.82801, 0.82592, 0.82295, 0.81914, 0.81443, 0.80890,\
        0.80296, 0.79679, 0.79012, 0.78294, 0.77565, 0.76839, 0.76119, 0.75401, 0.74685, 0.73969, 0.73252,\
        0.72531, 0.71804, 0.71065, 0.70308, 0.69522, 0.68722, 0.67925, 0.67141, 0.66372, 0.65623, 0.64893,\
        0.64183, 0.63492, 0.62818, 0.62159, 0.61510, 0.60866, 0.60228, 0.59605, 0.58997, 0.58389, 0.57776,\
        0.57157, 0.56531, 0.55897, 0.55256, 0.54606, 0.53948, 0.53279, 0.52602, 0.51913, 0.51211, 0.50494],\
        [0.41335, 0.41405, 0.41468, 0.41536, 0.41618, 0.41725, 0.41866, 0.42041, 0.42248, 0.42480, 0.42730,\
        0.42989, 0.43253, 0.43516, 0.43774, 0.44025, 0.44262, 0.44478, 0.44664, 0.44811, 0.44911, 0.44957,\
        0.44948, 0.44886, 0.44776, 0.44623, 0.44433, 0.44214, 0.43972, 0.43713, 0.43439, 0.43151, 0.42846,\
        0.42519, 0.42163, 0.41774, 0.41345, 0.40876, 0.40372, 0.39832, 0.39340, 0.39208, 0.39491, 0.40238,\
        0.40925, 0.42087, 0.44816, 0.48406, 0.52067, 0.55448, 0.58390, 0.60831, 0.62982, 0.65029, 0.67103,\
        0.69268, 0.71461, 0.73544, 0.75479, 0.77254, 0.78845, 0.80204, 0.81278, 0.82053, 0.82641, 0.83124,\
        0.83491, 0.83725, 0.83835, 0.83840, 0.83753, 0.83577, 0.83314, 0.82969, 0.82545, 0.82043, 0.81478,\
        0.80874, 0.80243, 0.79543, 0.78785, 0.78040, 0.77307, 0.76576, 0.75843, 0.75105, 0.74361, 0.73611,\
        0.72853, 0.72087, 0.71311, 0.70522, 0.69718, 0.68911, 0.68109, 0.67320, 0.66548, 0.65794, 0.65059,\
        0.64343, 0.63643, 0.62958, 0.62285, 0.61620, 0.60960, 0.60304, 0.59680, 0.59077, 0.58470, 0.57858,\
        0.57239, 0.56614, 0.55981, 0.55340, 0.54691, 0.54033, 0.53364, 0.52686, 0.51996, 0.51293, 0.50575],\
        [0.41603, 0.41703, 0.41800, 0.41902, 0.42018, 0.42156, 0.42322, 0.42518, 0.42742, 0.42991, 0.43257,\
        0.43537, 0.43824, 0.44113, 0.44402, 0.44683, 0.44950, 0.45194, 0.45404, 0.45568, 0.45676, 0.45721,\
        0.45702, 0.45622, 0.45488, 0.45305, 0.45080, 0.44822, 0.44539, 0.44237, 0.43921, 0.43591, 0.43242,\
        0.42866, 0.42452, 0.41994, 0.41481, 0.40900, 0.40234, 0.39478, 0.38784, 0.38380, 0.38400, 0.38821,\
        0.39192, 0.40236, 0.43446, 0.47538, 0.51812, 0.55796, 0.59193, 0.61848, 0.64053, 0.66153, 0.68349,\
        0.70726, 0.73119, 0.75311, 0.77261, 0.79002, 0.80542, 0.81826, 0.82823, 0.83548, 0.84061, 0.84422,\
        0.84653, 0.84765, 0.84773, 0.84691, 0.84528, 0.84287, 0.83970, 0.83577, 0.83112, 0.82579, 0.81989,\
        0.81352, 0.80671, 0.79927, 0.79175, 0.78447, 0.77723, 0.76989, 0.76243, 0.75484, 0.74712, 0.73927,\
        0.73131, 0.72325, 0.71511, 0.70690, 0.69867, 0.69052, 0.68247, 0.67457, 0.66685, 0.65931, 0.65195,\
        0.64475, 0.63770, 0.63078, 0.62398, 0.61727, 0.61065, 0.60420, 0.59793, 0.59170, 0.58552, 0.57935,\
        0.57315, 0.56689, 0.56056, 0.55414, 0.54764, 0.54105, 0.53436, 0.52757, 0.52065, 0.51361, 0.50641],\
        [0.41888, 0.42018, 0.42147, 0.42281, 0.42428, 0.42595, 0.42786, 0.43004, 0.43246, 0.43512, 0.43797,\
        0.44096, 0.44405, 0.44720, 0.45036, 0.45348, 0.45647, 0.45922, 0.46161, 0.46347, 0.46469, 0.46516,\
        0.46488, 0.46391, 0.46233, 0.46022, 0.45763, 0.45466, 0.45141, 0.44797, 0.44441, 0.44071, 0.43679,\
        0.43250, 0.42766, 0.42216, 0.41587, 0.40856, 0.39975, 0.38958, 0.38009, 0.37326, 0.37027, 0.37120,\
        0.37645, 0.39253, 0.42506, 0.47080, 0.51899, 0.56257, 0.59939, 0.62799, 0.65164, 0.67406, 0.69828,\
        0.72527, 0.75107, 0.77333, 0.79236, 0.80898, 0.82344, 0.83523, 0.84419, 0.85052, 0.85466, 0.85700,\
        0.85790, 0.85768, 0.85662, 0.85488, 0.85248, 0.84943, 0.84570, 0.84131, 0.83626, 0.83061, 0.82439,\
        0.81764, 0.81035, 0.80261, 0.79528, 0.78830, 0.78113, 0.77374, 0.76614, 0.75833, 0.75031, 0.74211,\
        0.73375, 0.72529, 0.71676, 0.70826, 0.69985, 0.69159, 0.68349, 0.67561, 0.66792, 0.66040, 0.65305,\
        0.64585, 0.63878, 0.63184, 0.62501, 0.61828, 0.61167, 0.60522, 0.59888, 0.59255, 0.58630, 0.58008,\
        0.57385, 0.56757, 0.56122, 0.55479, 0.54827, 0.54166, 0.53495, 0.52814, 0.52121, 0.51415, 0.50694],\
        [0.42186, 0.42344, 0.42502, 0.42666, 0.42842, 0.43036, 0.43252, 0.43491, 0.43753, 0.44037, 0.44340,\
        0.44658, 0.44988, 0.45326, 0.45669, 0.46010, 0.46342, 0.46653, 0.46926, 0.47141, 0.47282, 0.47333,\
        0.47297, 0.47184, 0.47007, 0.46772, 0.46483, 0.46148, 0.45780, 0.45396, 0.45003, 0.44599, 0.44169,\
        0.43687, 0.43121, 0.42447, 0.41658, 0.40739, 0.39607, 0.38308, 0.37100, 0.36167, 0.35635, 0.35612,\
        0.36308, 0.38128, 0.41678, 0.46781, 0.52041, 0.56654, 0.60531, 0.63662, 0.66321, 0.68844, 0.71664,\
        0.74697, 0.77428, 0.79586, 0.81384, 0.82935, 0.84252, 0.85301, 0.86066, 0.86575, 0.86865, 0.86960,\
        0.86892, 0.86724, 0.86502, 0.86236, 0.85921, 0.85553, 0.85126, 0.84641, 0.84100, 0.83503, 0.82856,\
        0.82160, 0.81421, 0.80664, 0.79939, 0.79228, 0.78497, 0.77745, 0.76967, 0.76161, 0.75329, 0.74473,\
        0.73598, 0.72709, 0.71817, 0.70933, 0.70075, 0.69232, 0.68423, 0.67640, 0.66876, 0.66128, 0.65395,\
        0.64677, 0.63972, 0.63277, 0.62594, 0.61919, 0.61256, 0.60602, 0.59959, 0.59326, 0.58700, 0.58076,\
        0.57449, 0.56818, 0.56179, 0.55533, 0.54878, 0.54214, 0.53540, 0.52857, 0.52162, 0.51454, 0.50732],\
        [0.42492, 0.42677, 0.42862, 0.43053, 0.43256, 0.43476, 0.43714, 0.43975, 0.44256, 0.44558, 0.44879,\
        0.45214, 0.45563, 0.45922, 0.46288, 0.46658, 0.47023, 0.47372, 0.47684, 0.47937, 0.48104, 0.48165,\
        0.48119, 0.47991, 0.47803, 0.47555, 0.47242, 0.46871, 0.46462, 0.46039, 0.45616, 0.45190, 0.44736,\
        0.44212, 0.43561, 0.42736, 0.41719, 0.40532, 0.39122, 0.37583, 0.36166, 0.35047, 0.34379, 0.34324,\
        0.35138, 0.37234, 0.41032, 0.46349, 0.52007, 0.56853, 0.60928, 0.64445, 0.67546, 0.70537, 0.73720,\
        0.76988, 0.79882, 0.81912, 0.83569, 0.85004, 0.86214, 0.87137, 0.87749, 0.88107, 0.88262, 0.88205,\
        0.87950, 0.87625, 0.87291, 0.86942, 0.86558, 0.86128, 0.85647, 0.85117, 0.84539, 0.83917, 0.83252,\
        0.82550, 0.81821, 0.81084, 0.80359, 0.79626, 0.78874, 0.78105, 0.77308, 0.76477, 0.75612, 0.74720,\
        0.73806, 0.72878, 0.71945, 0.71015, 0.70110, 0.69277, 0.68484, 0.67707, 0.66945, 0.66200, 0.65471,\
        0.64755, 0.64052, 0.63361, 0.62678, 0.62003, 0.61337, 0.60678, 0.60030, 0.59394, 0.58766, 0.58139,\
        0.57509, 0.56873, 0.56229, 0.55577, 0.54917, 0.54249, 0.53571, 0.52885, 0.52187, 0.51478, 0.50755],\
        [0.42803, 0.43013, 0.43223, 0.43439, 0.43667, 0.43909, 0.44170, 0.44451, 0.44751, 0.45071, 0.45408,\
        0.45760, 0.46125, 0.46500, 0.46886, 0.47280, 0.47676, 0.48063, 0.48420, 0.48716, 0.48921, 0.48998,\
        0.48937, 0.48799, 0.48616, 0.48372, 0.48046, 0.47641, 0.47190, 0.46733, 0.46293, 0.45863, 0.45413,\
        0.44880, 0.44172, 0.43196, 0.41901, 0.40320, 0.38641, 0.36915, 0.35331, 0.34087, 0.33344, 0.33268,\
        0.34084, 0.36103, 0.39749, 0.45427, 0.51659, 0.56704, 0.61144, 0.65116, 0.68629, 0.72198, 0.75715,\
        0.78930, 0.81813, 0.83958, 0.85614, 0.87005, 0.88172, 0.89010, 0.89453, 0.89640, 0.89644, 0.89436,\
        0.88934, 0.88466, 0.88043, 0.87625, 0.87172, 0.86675, 0.86137, 0.85561, 0.84949, 0.84303, 0.83627,\
        0.82925, 0.82205, 0.81476, 0.80742, 0.79993, 0.79236, 0.78459, 0.77643, 0.76784, 0.75886, 0.74957,\
        0.74007, 0.73045, 0.72081, 0.71128, 0.70207, 0.69365, 0.68559, 0.67772, 0.67006, 0.66261, 0.65534,\
        0.64822, 0.64123, 0.63435, 0.62756, 0.62083, 0.61416, 0.60754, 0.60101, 0.59463, 0.58832, 0.58201,\
        0.57564, 0.56921, 0.56270, 0.55610, 0.54944, 0.54269, 0.53587, 0.52897, 0.52197, 0.51486, 0.50762],\
        [0.43116, 0.43348, 0.43581, 0.43821, 0.44070, 0.44334, 0.44616, 0.44915, 0.45234, 0.45572, 0.45925,\
        0.46291, 0.46669, 0.47056, 0.47455, 0.47867, 0.48288, 0.48710, 0.49111, 0.49452, 0.49694, 0.49809,\
        0.49724, 0.49593, 0.49442, 0.49226, 0.48901, 0.48465, 0.47970, 0.47491, 0.47052, 0.46643, 0.46233,\
        0.45749, 0.45053, 0.43982, 0.42451, 0.40554, 0.38492, 0.36476, 0.34719, 0.33388, 0.32613, 0.32526,\
        0.33296, 0.35139, 0.38397, 0.43840, 0.50112, 0.55603, 0.61008, 0.65530, 0.69419, 0.73610, 0.77686,\
        0.80837, 0.83581, 0.85801, 0.87522, 0.88924, 0.90085, 0.90869, 0.91188, 0.91208, 0.91008, 0.90540,\
        0.89866, 0.89289, 0.88795, 0.88312, 0.87775, 0.87200, 0.86598, 0.85972, 0.85325, 0.84658, 0.83973,\
        0.83273, 0.82560, 0.81838, 0.81104, 0.80354, 0.79594, 0.78811, 0.77973, 0.77081, 0.76148, 0.75183,\
        0.74199, 0.73208, 0.72224, 0.71267, 0.70364, 0.69488, 0.68637, 0.67830, 0.67057, 0.66311, 0.65586,\
        0.64879, 0.64185, 0.63503, 0.62828, 0.62159, 0.61495, 0.60833, 0.60179, 0.59536, 0.58900, 0.58261,\
        0.57616, 0.56962, 0.56300, 0.55631, 0.54956, 0.54274, 0.53586, 0.52891, 0.52189, 0.51476, 0.50752],\
        [0.43429, 0.43681, 0.43935, 0.44195, 0.44465, 0.44748, 0.45048, 0.45366, 0.45702, 0.46056, 0.46426,\
        0.46806, 0.47194, 0.47589, 0.47994, 0.48414, 0.48852, 0.49301, 0.49737, 0.50116, 0.50379, 0.50514,\
        0.50484, 0.50395, 0.50285, 0.50104, 0.49799, 0.49345, 0.48810, 0.48341, 0.47931, 0.47561, 0.47225,\
        0.46861, 0.46288, 0.45212, 0.43464, 0.41175, 0.38668, 0.36339, 0.34413, 0.33036, 0.32279, 0.32210,\
        0.32957, 0.34721, 0.37747, 0.42114, 0.47125, 0.52813, 0.59681, 0.65259, 0.69545, 0.74372, 0.79347,\
        0.82608, 0.85339, 0.87666, 0.89472, 0.90928, 0.92047, 0.92724, 0.92944, 0.92803, 0.92383, 0.91723,\
        0.90937, 0.90165, 0.89625, 0.89020, 0.88362, 0.87697, 0.87024, 0.86346, 0.85662, 0.84974, 0.84281,\
        0.83583, 0.82879, 0.82168, 0.81447, 0.80711, 0.79954, 0.79163, 0.78291, 0.77362, 0.76392, 0.75394,\
        0.74378, 0.73358, 0.72351, 0.71372, 0.70434, 0.69538, 0.68683, 0.67871, 0.67095, 0.66349, 0.65628,\
        0.64926, 0.64239, 0.63563, 0.62895, 0.62233, 0.61574, 0.60916, 0.60261, 0.59613, 0.58970, 0.58320,\
        0.57661, 0.56994, 0.56319, 0.55638, 0.54951, 0.54260, 0.53566, 0.52866, 0.52161, 0.51448, 0.50724],\
        [0.43739, 0.44010, 0.44282, 0.44560, 0.44847, 0.45148, 0.45464, 0.45799, 0.46152, 0.46523, 0.46909,\
        0.47305, 0.47704, 0.48102, 0.48505, 0.48927, 0.49371, 0.49837, 0.50295, 0.50695, 0.50993, 0.51169,\
        0.51226, 0.51206, 0.51133, 0.50979, 0.50705, 0.50266, 0.49739, 0.49353, 0.48994, 0.48664, 0.48414,\
        0.48221, 0.47914, 0.46950, 0.44979, 0.42213, 0.39218, 0.36547, 0.34454, 0.33104, 0.32436, 0.32405,\
        0.33100, 0.34719, 0.37412, 0.40807, 0.44220, 0.48639, 0.56152, 0.64070, 0.69256, 0.74416, 0.79196,\
        0.83444, 0.86929, 0.89682, 0.91691, 0.93172, 0.94169, 0.94680, 0.94737, 0.94421, 0.93816, 0.93005,\
        0.92098, 0.91228, 0.90475, 0.89681, 0.88911, 0.88152, 0.87404, 0.86671, 0.85951, 0.85242, 0.84541,\
        0.83846, 0.83152, 0.82456, 0.81752, 0.81036, 0.80305, 0.79485, 0.78570, 0.77610, 0.76611, 0.75582,\
        0.74536, 0.73488, 0.72452, 0.71443, 0.70473, 0.69559, 0.68704, 0.67892, 0.67117, 0.66375, 0.65658,\
        0.64962, 0.64282, 0.63615, 0.62955, 0.62301, 0.61650, 0.61000, 0.60347, 0.59695, 0.59041, 0.58375,\
        0.57698, 0.57013, 0.56322, 0.55626, 0.54928, 0.54227, 0.53524, 0.52820, 0.52112, 0.51399, 0.50676],\
        [0.44046, 0.44333, 0.44622, 0.44915, 0.45217, 0.45532, 0.45863, 0.46212, 0.46580, 0.46967, 0.47371,\
        0.47786, 0.48201, 0.48602, 0.48997, 0.49417, 0.49869, 0.50349, 0.50826, 0.51249, 0.51585, 0.51819,\
        0.51956, 0.52011, 0.51995, 0.51891, 0.51672, 0.51327, 0.50937, 0.50619, 0.50319, 0.50033, 0.49885,\
        0.49831, 0.49882, 0.49128, 0.46971, 0.43800, 0.40278, 0.37037, 0.34844, 0.33684, 0.33211, 0.33220,\
        0.33762, 0.35043, 0.37259, 0.40077, 0.42393, 0.45236, 0.51510, 0.61813, 0.68900, 0.74807, 0.80131,\
        0.84736, 0.88672, 0.91896, 0.94139, 0.95588, 0.96411, 0.96704, 0.96550, 0.96036, 0.95253, 0.94292,\
        0.93252, 0.92226, 0.91250, 0.90310, 0.89419, 0.88550, 0.87725, 0.86937, 0.86181, 0.85451, 0.84744,\
        0.84053, 0.83371, 0.82689, 0.81998, 0.81285, 0.80525, 0.79689, 0.78786, 0.77816, 0.76797, 0.75743,\
        0.74670, 0.73593, 0.72530, 0.71496, 0.70505, 0.69577, 0.68713, 0.67899, 0.67126, 0.66388, 0.65677,\
        0.64988, 0.64315, 0.63655, 0.63004, 0.62359, 0.61718, 0.61077, 0.60433, 0.59778, 0.59104, 0.58416,\
        0.57719, 0.57014, 0.56306, 0.55594, 0.54882, 0.54170, 0.53459, 0.52749, 0.52040, 0.51327, 0.50608],\
        [0.44348, 0.44650, 0.44952, 0.45258, 0.45573, 0.45900, 0.46242, 0.46603, 0.46983, 0.47385, 0.47807,\
        0.48246, 0.48689, 0.49102, 0.49476, 0.49900, 0.50373, 0.50871, 0.51361, 0.51807, 0.52183, 0.52477,\
        0.52688, 0.52824, 0.52888, 0.52872, 0.52766, 0.52575, 0.52347, 0.52128, 0.51925, 0.51771, 0.51731,\
        0.51817, 0.51883, 0.51239, 0.49186, 0.45737, 0.41447, 0.37725, 0.35639, 0.34932, 0.34782, 0.34826,\
        0.35157, 0.35972, 0.37509, 0.40008, 0.42681, 0.45355, 0.50086, 0.59076, 0.68230, 0.75274, 0.81301,\
        0.86894, 0.91199, 0.94571, 0.96801, 0.98088, 0.98668, 0.98704, 0.98311, 0.97585, 0.96617, 0.95496,\
        0.94305, 0.93115, 0.91964, 0.90869, 0.89829, 0.88865, 0.87972, 0.87132, 0.86340, 0.85591, 0.84879,\
        0.84195, 0.83526, 0.82860, 0.82182, 0.81471, 0.80704, 0.79870, 0.78966, 0.77988, 0.76952, 0.75875,\
        0.74776, 0.73673, 0.72584, 0.71529, 0.70524, 0.69584, 0.68710, 0.67893, 0.67122, 0.66388, 0.65683,\
        0.65000, 0.64335, 0.63682, 0.63038, 0.62400, 0.61765, 0.61131, 0.60493, 0.59835, 0.59142, 0.58433,\
        0.57715, 0.56992, 0.56265, 0.55538, 0.54811, 0.54088, 0.53368, 0.52652, 0.51941, 0.51231, 0.50516],\
        [0.44644, 0.44959, 0.45273, 0.45590, 0.45915, 0.46251, 0.46602, 0.46972, 0.47361, 0.47772, 0.48207,\
        0.48667, 0.49152, 0.49618, 0.49971, 0.50389, 0.50890, 0.51406, 0.51913, 0.52384, 0.52799, 0.53149,\
        0.53431, 0.53652, 0.53814, 0.53917, 0.53958, 0.53941, 0.53882, 0.53800, 0.53716, 0.53673, 0.53715,\
        0.53826, 0.53811, 0.53212, 0.51506, 0.48269, 0.43561, 0.39009, 0.37061, 0.37080, 0.37268, 0.37323,\
        0.37576, 0.38185, 0.39504, 0.42049, 0.45479, 0.48398, 0.51627, 0.57722, 0.67662, 0.75579, 0.82846,\
        0.90258, 0.94812, 0.97746, 0.99571, 1.00520, 1.00795, 1.00554, 0.99916, 0.98980, 0.97831, 0.96548,\
        0.95205, 0.93860, 0.92555, 0.91313, 0.90146, 0.89101, 0.88139, 0.87244, 0.86417, 0.85652, 0.84939,\
        0.84265, 0.83614, 0.82970, 0.82310, 0.81614, 0.80861, 0.80036, 0.79123, 0.78132, 0.77077, 0.75977,\
        0.74852, 0.73724, 0.72611, 0.71535, 0.70516, 0.69566, 0.68688, 0.67871, 0.67104, 0.66374, 0.65675,\
        0.64999, 0.64340, 0.63693, 0.63054, 0.62419, 0.61786, 0.61151, 0.60507, 0.59838, 0.59139, 0.58417,\
        0.57682, 0.56941, 0.56197, 0.55454, 0.54714, 0.53978, 0.53249, 0.52526, 0.51812, 0.51106, 0.50400],\
        [0.44936, 0.45262, 0.45585, 0.45911, 0.46243, 0.46586, 0.46943, 0.47319, 0.47714, 0.48130, 0.48567,\
        0.49031, 0.49528, 0.50055, 0.50482, 0.50918, 0.51423, 0.51955, 0.52481, 0.52979, 0.53433, 0.53835,\
        0.54185, 0.54490, 0.54757, 0.54993, 0.55198, 0.55365, 0.55484, 0.55548, 0.55569, 0.55587, 0.55639,\
        0.55703, 0.55638, 0.55148, 0.53837, 0.51268, 0.46854, 0.41795, 0.39603, 0.40048, 0.40343, 0.40493,\
        0.40999, 0.42005, 0.43937, 0.46740, 0.49462, 0.51201, 0.52968, 0.57760, 0.67649, 0.75916, 0.84642,\
        0.93440, 0.98023, 1.00601, 1.02023, 1.02626, 1.02608, 1.02114, 1.01259, 1.00135, 0.98823, 0.97393,\
        0.95910, 0.94428, 0.92993, 0.91643, 0.90402, 0.89261, 0.88209, 0.87256, 0.86397, 0.85622, 0.84914,\
        0.84258, 0.83633, 0.83017, 0.82386, 0.81714, 0.80979, 0.80161, 0.79247, 0.78245, 0.77171, 0.76047,\
        0.74897, 0.73743, 0.72608, 0.71512, 0.70475, 0.69520, 0.68645, 0.67834, 0.67071, 0.66347, 0.65653,\
        0.64983, 0.64329, 0.63687, 0.63050, 0.62415, 0.61777, 0.61135, 0.60487, 0.59812, 0.59099, 0.58364,\
        0.57615, 0.56859, 0.56099, 0.55341, 0.54588, 0.53841, 0.53102, 0.52372, 0.51652, 0.50949, 0.50256],\
        [0.45223, 0.45558, 0.45890, 0.46222, 0.46558, 0.46905, 0.47266, 0.47646, 0.48047, 0.48467, 0.48903,\
        0.49357, 0.49838, 0.50373, 0.50903, 0.51418, 0.51956, 0.52511, 0.53061, 0.53589, 0.54081, 0.54531,\
        0.54941, 0.55323, 0.55691, 0.56057, 0.56428, 0.56788, 0.57096, 0.57307, 0.57413, 0.57455, 0.57472,\
        0.57463, 0.57351, 0.56967, 0.55971, 0.53927, 0.50519, 0.46008, 0.43207, 0.42884, 0.43348, 0.43930,\
        0.44839, 0.46349, 0.48557, 0.50884, 0.52237, 0.52879, 0.54198, 0.58746, 0.68494, 0.76455, 0.85927,\
        0.94903, 0.99865, 1.02477, 1.03751, 1.04155, 1.03939, 1.03264, 1.02247, 1.00980, 0.99539, 0.97990,\
        0.96391, 0.94793, 0.93244, 0.91791, 0.90474, 0.89255, 0.88137, 0.87146, 0.86270, 0.85493, 0.84800,\
        0.84169, 0.83579, 0.83001, 0.82409, 0.81772, 0.81061, 0.80252, 0.79340, 0.78328, 0.77231, 0.76080,\
        0.74903, 0.73727, 0.72574, 0.71462, 0.70415, 0.69458, 0.68588, 0.67781, 0.67023, 0.66304, 0.65615,\
        0.64950, 0.64301, 0.63663, 0.63027, 0.62389, 0.61743, 0.61083, 0.60416, 0.59737, 0.59018, 0.58277,\
        0.57516, 0.56745, 0.55971, 0.55198, 0.54432, 0.53676, 0.52929, 0.52193, 0.51464, 0.50749, 0.50076],\
        [0.45505, 0.45850, 0.46188, 0.46524, 0.46863, 0.47211, 0.47573, 0.47956, 0.48363, 0.48793, 0.49237,\
        0.49690, 0.50188, 0.50740, 0.51316, 0.51893, 0.52478, 0.53066, 0.53648, 0.54209, 0.54737, 0.55230,\
        0.55691, 0.56135, 0.56585, 0.57065, 0.57589, 0.58142, 0.58654, 0.59020, 0.59211, 0.59288, 0.59288,\
        0.59224, 0.59057, 0.58742, 0.57958, 0.56078, 0.52902, 0.48866, 0.46116, 0.45726, 0.46452, 0.47418,\
        0.48582, 0.50099, 0.51995, 0.53443, 0.54144, 0.54554, 0.55982, 0.60661, 0.69774, 0.77863, 0.87384,\
        0.95656, 1.00582, 1.03320, 1.04630, 1.04992, 1.04697, 1.03931, 1.02823, 1.01470, 0.99947, 0.98318,\
        0.96636, 0.94947, 0.93296, 0.91728, 0.90298, 0.89035, 0.87908, 0.86908, 0.86029, 0.85263, 0.84593,\
        0.83998, 0.83451, 0.82922, 0.82378, 0.81786, 0.81113, 0.80329, 0.79416, 0.78379, 0.77251, 0.76068,\
        0.74864, 0.73669, 0.72506, 0.71390, 0.70339, 0.69386, 0.68520, 0.67714, 0.66958, 0.66242, 0.65559,\
        0.64899, 0.64255, 0.63620, 0.62987, 0.62348, 0.61696, 0.61019, 0.60308, 0.59614, 0.58911, 0.58164,\
        0.57389, 0.56602, 0.55811, 0.55024, 0.54247, 0.53483, 0.52734, 0.51999, 0.51273, 0.50542, 0.49831],\
        [0.45785, 0.46138, 0.46481, 0.46820, 0.47159, 0.47505, 0.47865, 0.48248, 0.48659, 0.49105, 0.49583,\
        0.50053, 0.50566, 0.51138, 0.51743, 0.52364, 0.52991, 0.53619, 0.54236, 0.54833, 0.55398, 0.55928,\
        0.56428, 0.56915, 0.57422, 0.57983, 0.58626, 0.59350, 0.60052, 0.60582, 0.60913, 0.61116, 0.61216,\
        0.61215, 0.61083, 0.60817, 0.60148, 0.58263, 0.55017, 0.50801, 0.48129, 0.48559, 0.49757, 0.50953,\
        0.52163, 0.53479, 0.54802, 0.55714, 0.56155, 0.56692, 0.58489, 0.63179, 0.72061, 0.81403, 0.89231,\
        0.96023, 1.00664, 1.03412, 1.04780, 1.05173, 1.04881, 1.04100, 1.02969, 1.01589, 1.00037, 0.98372,\
        0.96647, 0.94904, 0.93185, 0.91528, 0.89993, 0.88703, 0.87566, 0.86559, 0.85683, 0.84933, 0.84294,\
        0.83743, 0.83250, 0.82778, 0.82292, 0.81752, 0.81123, 0.80370, 0.79451, 0.78381, 0.77216, 0.75998,\
        0.74767, 0.73556, 0.72392, 0.71291, 0.70258, 0.69308, 0.68436, 0.67626, 0.66870, 0.66158, 0.65480,\
        0.64826, 0.64188, 0.63558, 0.62928, 0.62292, 0.61644, 0.60977, 0.60283, 0.59562, 0.58813, 0.58034,\
        0.57235, 0.56427, 0.55619, 0.54818, 0.54031, 0.53261, 0.52512, 0.51787, 0.51086, 0.50415, 0.49736],\
        [0.46062, 0.46424, 0.46773, 0.47113, 0.47450, 0.47791, 0.48144, 0.48522, 0.48932, 0.49382, 0.49879,\
        0.50403, 0.50945, 0.51542, 0.52177, 0.52833, 0.53499, 0.54165, 0.54820, 0.55455, 0.56059, 0.56625,\
        0.57153, 0.57664, 0.58200, 0.58811, 0.59528, 0.60367, 0.61211, 0.61923, 0.62479, 0.62909, 0.63231,\
        0.63430, 0.63467, 0.63276, 0.62609, 0.61054, 0.58213, 0.53860, 0.50619, 0.51598, 0.53174, 0.54560,\
        0.55611, 0.56648, 0.57602, 0.58190, 0.58558, 0.59176, 0.61239, 0.66418, 0.76703, 0.85314, 0.91456,\
        0.96635, 1.00555, 1.03075, 1.04402, 1.04815, 1.04555, 1.03806, 1.02705, 1.01351, 0.99818, 0.98165,\
        0.96439, 0.94686, 0.92947, 0.91273, 0.89718, 0.88345, 0.87152, 0.86119, 0.85240, 0.84508, 0.83907,\
        0.83409, 0.82979, 0.82572, 0.82148, 0.81664, 0.81076, 0.80333, 0.79408, 0.78316, 0.77114, 0.75859,\
        0.74598, 0.73370, 0.72207, 0.71135, 0.70144, 0.69197, 0.68314, 0.67502, 0.66750, 0.66045, 0.65375,\
        0.64730, 0.64099, 0.63474, 0.62848, 0.62215, 0.61572, 0.60916, 0.60246, 0.59500, 0.58696, 0.57878,\
        0.57051, 0.56220, 0.55394, 0.54579, 0.53782, 0.53006, 0.52257, 0.51539, 0.50855, 0.50190, 0.49518],\
        [0.46339, 0.46710, 0.47065, 0.47408, 0.47741, 0.48073, 0.48415, 0.48783, 0.49189, 0.49635, 0.50131,\
        0.50697, 0.51297, 0.51935, 0.52606, 0.53297, 0.53998, 0.54699, 0.55392, 0.56067, 0.56714, 0.57318,\
        0.57871, 0.58392, 0.58936, 0.59581, 0.60376, 0.61290, 0.62235, 0.63113, 0.63894, 0.64581, 0.65173,\
        0.65630, 0.65861, 0.65772, 0.65266, 0.64143, 0.62101, 0.58629, 0.55580, 0.55336, 0.56819, 0.58309,\
        0.58876, 0.59629, 0.60400, 0.60819, 0.61270, 0.62202, 0.65154, 0.72695, 0.82948, 0.88519, 0.93112,\
        0.97183, 1.00354, 1.02490, 1.03665, 1.04045, 1.03809, 1.03112, 1.02075, 1.00789, 0.99319, 0.97719,\
        0.96033, 0.94305, 0.92581, 0.90911, 0.89343, 0.87915, 0.86662, 0.85592, 0.84704, 0.83993, 0.83437,\
        0.83004, 0.82645, 0.82312, 0.81953, 0.81522, 0.80976, 0.80262, 0.79316, 0.78182, 0.76937, 0.75642,\
        0.74349, 0.73099, 0.71926, 0.70858, 0.69906, 0.68985, 0.68122, 0.67326, 0.66589, 0.65898, 0.65241,\
        0.64607, 0.63985, 0.63367, 0.62743, 0.62108, 0.61458, 0.60789, 0.60087, 0.59330, 0.58518, 0.57678,\
        0.56829, 0.55978, 0.55135, 0.54308, 0.53500, 0.52718, 0.51965, 0.51246, 0.50569, 0.49910, 0.49236],\
        [0.46614, 0.46997, 0.47362, 0.47708, 0.48039, 0.48361, 0.48686, 0.49033, 0.49433, 0.49901, 0.50408,\
        0.50995, 0.51640, 0.52320, 0.53028, 0.53752, 0.54483, 0.55215, 0.55941, 0.56654, 0.57346, 0.58000,\
        0.58587, 0.59112, 0.59661, 0.60332, 0.61186, 0.62148, 0.63158, 0.64159, 0.65123, 0.66039, 0.66885,\
        0.67613, 0.68062, 0.68063, 0.67750, 0.66987, 0.65472, 0.62996, 0.60616, 0.59606, 0.60413, 0.61711,\
        0.62003, 0.62474, 0.63097, 0.63696, 0.64672, 0.66676, 0.71729, 0.80724, 0.87097, 0.90835, 0.94357,\
        0.97537, 1.00021, 1.01721, 1.02667, 1.02960, 1.02727, 1.02087, 1.01137, 0.99950, 0.98580, 0.97069,\
        0.95456, 0.93779, 0.92084, 0.90426, 0.88838, 0.87375, 0.86078, 0.84974, 0.84079, 0.83393, 0.82895,\
        0.82539, 0.82266, 0.82012, 0.81717, 0.81328, 0.80805, 0.80111, 0.79142, 0.77969, 0.76681, 0.75345,\
        0.74019, 0.72751, 0.71562, 0.70468, 0.69522, 0.68667, 0.67856, 0.67095, 0.66385, 0.65715, 0.65077,\
        0.64458, 0.63848, 0.63236, 0.62613, 0.61972, 0.61305, 0.60611, 0.59884, 0.59108, 0.58287, 0.57435,\
        0.56568, 0.55701, 0.54843, 0.54004, 0.53189, 0.52402, 0.51646, 0.50919, 0.50224, 0.49587, 0.48926],\
        [0.46887, 0.47286, 0.47663, 0.48017, 0.48349, 0.48664, 0.48970, 0.49286, 0.49637, 0.50102, 0.50668,\
        0.51302, 0.51988, 0.52706, 0.53445, 0.54196, 0.54951, 0.55704, 0.56452, 0.57193, 0.57924, 0.58640,\
        0.59289, 0.59813, 0.60380, 0.61084, 0.61940, 0.62912, 0.63955, 0.65029, 0.66115, 0.67197, 0.68233,\
        0.69152, 0.69900, 0.69960, 0.69924, 0.69567, 0.68718, 0.67329, 0.65716, 0.64538, 0.64435, 0.64794,\
        0.64880, 0.65257, 0.66095, 0.67156, 0.69262, 0.72713, 0.78239, 0.84861, 0.89659, 0.92778, 0.95377,\
        0.97687, 0.99518, 1.00773, 1.01455, 1.01627, 1.01377, 1.00794, 0.99948, 0.98887, 0.97648, 0.96257,\
        0.94742, 0.93138, 0.91478, 0.89820, 0.88234, 0.86742, 0.85405, 0.84270, 0.83371, 0.82720, 0.82294,\
        0.82035, 0.81864, 0.81701, 0.81472, 0.81114, 0.80577, 0.79837, 0.78872, 0.77685, 0.76359, 0.74976,\
        0.73614, 0.72340, 0.71173, 0.70094, 0.69155, 0.68321, 0.67546, 0.66820, 0.66141, 0.65498, 0.64883,\
        0.64284, 0.63688, 0.63085, 0.62465, 0.61816, 0.61131, 0.60406, 0.59652, 0.58861, 0.58025, 0.57158,\
        0.56274, 0.55390, 0.54519, 0.53671, 0.52853, 0.52065, 0.51313, 0.50605, 0.49915, 0.49256, 0.48601],\
        [0.47156, 0.47574, 0.47968, 0.48336, 0.48678, 0.48994, 0.49287, 0.49585, 0.49919, 0.50335, 0.50956,\
        0.51637, 0.52356, 0.53100, 0.53861, 0.54629, 0.55398, 0.56162, 0.56917, 0.57662, 0.58403, 0.59147,\
        0.59888, 0.60440, 0.61023, 0.61739, 0.62588, 0.63551, 0.64594, 0.65692, 0.66827, 0.67994, 0.69168,\
        0.70164, 0.70990, 0.71489, 0.71784, 0.71859, 0.71654, 0.71127, 0.70241, 0.69089, 0.68514, 0.68077,\
        0.67496, 0.67572, 0.69364, 0.71165, 0.74709, 0.78830, 0.82949, 0.87403, 0.91045, 0.93730, 0.95811,\
        0.97500, 0.98788, 0.99636, 1.00053, 1.00089, 0.99814, 0.99292, 0.98567, 0.97659, 0.96578, 0.95331,\
        0.93933, 0.92418, 0.90838, 0.89189, 0.87569, 0.86043, 0.84667, 0.83500, 0.82597, 0.81991, 0.81661,\
        0.81524, 0.81474, 0.81412, 0.81256, 0.80938, 0.80398, 0.79622, 0.78614, 0.77381, 0.75998, 0.74552,\
        0.73135, 0.71849, 0.70761, 0.69755, 0.68806, 0.67964, 0.67206, 0.66510, 0.65863, 0.65251, 0.64663,\
        0.64086, 0.63508, 0.62917, 0.62302, 0.61652, 0.60958, 0.60211, 0.59428, 0.58609, 0.57749, 0.56858,\
        0.55954, 0.55051, 0.54166, 0.53311, 0.52494, 0.51713, 0.50961, 0.50253, 0.49613, 0.48943, 0.48276],\
        [0.47416, 0.47856, 0.48272, 0.48660, 0.49020, 0.49354, 0.49654, 0.49919, 0.50301, 0.50756, 0.51345,\
        0.52025, 0.52752, 0.53507, 0.54277, 0.55052, 0.55825, 0.56588, 0.57335, 0.58064, 0.58772, 0.59466,\
        0.60168, 0.60843, 0.61507, 0.62250, 0.63097, 0.64042, 0.65066, 0.66147, 0.67267, 0.68414, 0.69573,\
        0.70719, 0.71739, 0.72581, 0.73237, 0.73702, 0.73939, 0.73882, 0.73444, 0.72688, 0.72023, 0.71197,\
        0.70167, 0.69371, 0.71315, 0.75481, 0.79621, 0.83270, 0.86244, 0.89138, 0.91824, 0.94006, 0.95685,\
        0.96941, 0.97812, 0.98314, 0.98484, 0.98384, 0.98084, 0.97635, 0.97053, 0.96326, 0.95429, 0.94346,\
        0.93076, 0.91637, 0.90086, 0.88492, 0.86864, 0.85311, 0.83901, 0.82702, 0.81790, 0.81239, 0.81038,\
        0.81051, 0.81133, 0.81176, 0.81097, 0.80828, 0.80320, 0.79509, 0.78410, 0.77095, 0.75633, 0.74106,\
        0.72603, 0.71235, 0.70191, 0.69289, 0.68378, 0.67562, 0.66832, 0.66168, 0.65555, 0.64976, 0.64418,\
        0.63867, 0.63309, 0.62732, 0.62126, 0.61481, 0.60788, 0.60035, 0.59219, 0.58358, 0.57463, 0.56542,\
        0.55611, 0.54686, 0.53784, 0.52922, 0.52113, 0.51357, 0.50627, 0.49908, 0.49254, 0.48605, 0.47943],\
        [0.47661, 0.48126, 0.48568, 0.48980, 0.49359, 0.49714, 0.50057, 0.50347, 0.50725, 0.51207, 0.51788,\
        0.52451, 0.53169, 0.53921, 0.54689, 0.55462, 0.56230, 0.56984, 0.57717, 0.58421, 0.59092, 0.59736,\
        0.60406, 0.61120, 0.61843, 0.62613, 0.63459, 0.64386, 0.65383, 0.66432, 0.67512, 0.68603, 0.69717,\
        0.70965, 0.72208, 0.73317, 0.74263, 0.75032, 0.75587, 0.75868, 0.75819, 0.75475, 0.74950, 0.74242,\
        0.73502, 0.73345, 0.75661, 0.79889, 0.83345, 0.86037, 0.88153, 0.90126, 0.92131, 0.93863, 0.95186,\
        0.96102, 0.96640, 0.96846, 0.96788, 0.96554, 0.96233, 0.95874, 0.95466, 0.94951, 0.94264, 0.93361,\
        0.92228, 0.90879, 0.89352, 0.87753, 0.86136, 0.84578, 0.83157, 0.81942, 0.81018, 0.80526, 0.80500,\
        0.80676, 0.80880, 0.81016, 0.81005, 0.80769, 0.80243, 0.79397, 0.78234, 0.76830, 0.75281, 0.73678,\
        0.72105, 0.70651, 0.69505, 0.68668, 0.67861, 0.67106, 0.66420, 0.65797, 0.65221, 0.64678, 0.64153,\
        0.63629, 0.63093, 0.62531, 0.61933, 0.61291, 0.60602, 0.59837, 0.58992, 0.58094, 0.57163, 0.56211,\
        0.55252, 0.54301, 0.53377, 0.52501, 0.51697, 0.50979, 0.50312, 0.49619, 0.48930, 0.48264, 0.47601],\
        [0.47883, 0.48373, 0.48843, 0.49286, 0.49686, 0.50051, 0.50409, 0.50765, 0.51170, 0.51659, 0.52235,\
        0.52885, 0.53590, 0.54330, 0.55090, 0.55854, 0.56611, 0.57351, 0.58066, 0.58751, 0.59401, 0.60025,\
        0.60669, 0.61354, 0.62077, 0.62850, 0.63687, 0.64595, 0.65567, 0.66590, 0.67652, 0.68741, 0.69882,\
        0.71168, 0.72504, 0.73768, 0.74910, 0.75898, 0.76700, 0.77278, 0.77598, 0.77656, 0.77506, 0.77257,\
        0.77250, 0.78137, 0.80270, 0.83462, 0.86103, 0.87943, 0.89279, 0.90657, 0.92162, 0.93489, 0.94487,\
        0.95109, 0.95365, 0.95309, 0.95032, 0.94658, 0.94315, 0.94070, 0.93875, 0.93606, 0.93148, 0.92427,\
        0.91421, 0.90151, 0.88654, 0.87037, 0.85415, 0.83872, 0.82483, 0.81309, 0.80424, 0.79997, 0.80179,\
        0.80449, 0.80728, 0.80933, 0.80977, 0.80774, 0.80243, 0.79337, 0.78077, 0.76571, 0.74936, 0.73276,\
        0.71685, 0.70252, 0.69064, 0.68134, 0.67340, 0.66626, 0.65983, 0.65401, 0.64865, 0.64360, 0.63869,\
        0.63376, 0.62863, 0.62317, 0.61723, 0.61071, 0.60367, 0.59597, 0.58730, 0.57805, 0.56844, 0.55863,\
        0.54878, 0.53904, 0.52956, 0.52053, 0.51229, 0.50539, 0.49913, 0.49252, 0.48576, 0.47910, 0.47248],\
        [0.48075, 0.48586, 0.49081, 0.49559, 0.49992, 0.50370, 0.50747, 0.51142, 0.51579, 0.52083, 0.52658,\
        0.53298, 0.53991, 0.54719, 0.55467, 0.56218, 0.56961, 0.57683, 0.58377, 0.59044, 0.59682, 0.60278,\
        0.60879, 0.61525, 0.62222, 0.62978, 0.63797, 0.64684, 0.65637, 0.66647, 0.67711, 0.68835, 0.70030,\
        0.71305, 0.72650, 0.73981, 0.75231, 0.76361, 0.77342, 0.78159, 0.78805, 0.79244, 0.79520, 0.79764,\
        0.80300, 0.81664, 0.83644, 0.86058, 0.88129, 0.89286, 0.90025, 0.90959, 0.92026, 0.92993, 0.93705,\
        0.94076, 0.94092, 0.93801, 0.93314, 0.92786, 0.92408, 0.92304, 0.92364, 0.92364, 0.92141, 0.91592,\
        0.90677, 0.89438, 0.87941, 0.86314, 0.84692, 0.83187, 0.81883, 0.80840, 0.80125, 0.79891, 0.80056,\
        0.80333, 0.80650, 0.80899, 0.80981, 0.80806, 0.80290, 0.79293, 0.77898, 0.76281, 0.74562, 0.72854,\
        0.71266, 0.69855, 0.68646, 0.67659, 0.66843, 0.66144, 0.65532, 0.64987, 0.64491, 0.64025, 0.63570,\
        0.63107, 0.62619, 0.62092, 0.61508, 0.60847, 0.60091, 0.59307, 0.58437, 0.57492, 0.56506, 0.55501,\
        0.54495, 0.53505, 0.52541, 0.51615, 0.50739, 0.50014, 0.49442, 0.48825, 0.48183, 0.47535, 0.46885],\
        [0.48233, 0.48756, 0.49262, 0.49755, 0.50218, 0.50631, 0.51041, 0.51470, 0.51937, 0.52457, 0.53035,\
        0.53671, 0.54353, 0.55071, 0.55807, 0.56547, 0.57274, 0.57974, 0.58638, 0.59267, 0.59871, 0.60435,\
        0.61001, 0.61611, 0.62277, 0.63004, 0.63800, 0.64665, 0.65601, 0.66602, 0.67666, 0.68794, 0.69991,\
        0.71272, 0.72624, 0.73975, 0.75270, 0.76471, 0.77553, 0.78510, 0.79353, 0.80086, 0.80642, 0.81290,\
        0.82220, 0.83664, 0.85546, 0.87618, 0.89426, 0.90072, 0.90410, 0.90996, 0.91709, 0.92398, 0.92896,\
        0.93082, 0.92910, 0.92426, 0.91759, 0.91089, 0.90649, 0.90709, 0.91040, 0.91283, 0.91285, 0.90907,\
        0.90041, 0.88771, 0.87219, 0.85553, 0.83930, 0.82477, 0.81296, 0.80433, 0.79915, 0.79765, 0.79923,\
        0.80238, 0.80594, 0.80875, 0.80970, 0.80775, 0.80187, 0.79107, 0.77627, 0.75920, 0.74126, 0.72360,\
        0.70742, 0.69357, 0.68164, 0.67169, 0.66345, 0.65656, 0.65068, 0.64558, 0.64101, 0.63674, 0.63256,\
        0.62825, 0.62362, 0.61852, 0.61287, 0.60652, 0.59908, 0.59056, 0.58136, 0.57159, 0.56147, 0.55122,\
        0.54102, 0.53107, 0.52149, 0.51237, 0.50381, 0.49620, 0.49000, 0.48399, 0.47781, 0.47151, 0.46514],\
        [0.48360, 0.48891, 0.49396, 0.49881, 0.50364, 0.50828, 0.51280, 0.51743, 0.52235, 0.52770, 0.53353,\
        0.53986, 0.54661, 0.55370, 0.56097, 0.56827, 0.57542, 0.58221, 0.58849, 0.59420, 0.59950, 0.60480,\
        0.61026, 0.61607, 0.62239, 0.62935, 0.63701, 0.64544, 0.65464, 0.66458, 0.67518, 0.68634, 0.69798,\
        0.71099, 0.72442, 0.73776, 0.75068, 0.76284, 0.77397, 0.78396, 0.79277, 0.80078, 0.80911, 0.81840,\
        0.83000, 0.84467, 0.86174, 0.87909, 0.89290, 0.89901, 0.90230, 0.90619, 0.91124, 0.91673, 0.92070,\
        0.92164, 0.91874, 0.91258, 0.90486, 0.89787, 0.89387, 0.89518, 0.89970, 0.90331, 0.90525, 0.90424,\
        0.89542, 0.88120, 0.86435, 0.84708, 0.83087, 0.81685, 0.80631, 0.79961, 0.79595, 0.79548, 0.79758,\
        0.80121, 0.80524, 0.80833, 0.80923, 0.80677, 0.79990, 0.78812, 0.77251, 0.75471, 0.73621, 0.71817,\
        0.70142, 0.68776, 0.67622, 0.66643, 0.65827, 0.65152, 0.64588, 0.64111, 0.63694, 0.63309, 0.62930,\
        0.62530, 0.62088, 0.61589, 0.61025, 0.60400, 0.59669, 0.58773, 0.57809, 0.56800, 0.55765, 0.54723,\
        0.53694, 0.52700, 0.51762, 0.50890, 0.50079, 0.49321, 0.48639, 0.48008, 0.47390, 0.46769, 0.46143],\
        [0.48460, 0.49004, 0.49515, 0.50002, 0.50500, 0.50989, 0.51471, 0.51959, 0.52470, 0.53015, 0.53602,\
        0.54231, 0.54901, 0.55601, 0.56322, 0.57047, 0.57757, 0.58426, 0.59027, 0.59544, 0.60001, 0.60474,\
        0.60982, 0.61526, 0.62118, 0.62774, 0.63506, 0.64324, 0.65229, 0.66217, 0.67281, 0.68419, 0.69632,\
        0.70866, 0.72125, 0.73411, 0.74672, 0.75867, 0.76969, 0.77964, 0.78860, 0.79707, 0.80598, 0.81654,\
        0.82900, 0.84317, 0.85817, 0.87226, 0.88316, 0.88976, 0.89394, 0.89767, 0.90221, 0.90784, 0.91184,\
        0.91304, 0.90982, 0.90281, 0.89516, 0.88874, 0.88553, 0.88640, 0.89002, 0.89378, 0.89664, 0.89784,\
        0.88892, 0.87268, 0.85471, 0.83715, 0.82143, 0.80812, 0.79843, 0.79347, 0.79145, 0.79220, 0.79519,\
        0.79954, 0.80417, 0.80760, 0.80842, 0.80535, 0.79742, 0.78449, 0.76791, 0.74929, 0.73035, 0.71265,\
        0.69645, 0.68230, 0.67059, 0.66085, 0.65281, 0.64623, 0.64086, 0.63644, 0.63268, 0.62928, 0.62591,\
        0.62225, 0.61803, 0.61304, 0.60718, 0.60043, 0.59290, 0.58409, 0.57434, 0.56407, 0.55355, 0.54300,\
        0.53265, 0.52271, 0.51346, 0.50516, 0.49731, 0.48981, 0.48287, 0.47639, 0.47014, 0.46396, 0.45776],\
        [0.48534, 0.49099, 0.49635, 0.50137, 0.50630, 0.51123, 0.51617, 0.52119, 0.52640, 0.53190, 0.53775,\
        0.54398, 0.55059, 0.55751, 0.56466, 0.57190, 0.57902, 0.58578, 0.59179, 0.59658, 0.60034, 0.60434,\
        0.60892, 0.61383, 0.61921, 0.62527, 0.63220, 0.64010, 0.64898, 0.65878, 0.66940, 0.68076, 0.69264,\
        0.70439, 0.71658, 0.72906, 0.74128, 0.75288, 0.76361, 0.77336, 0.78221, 0.79054, 0.79933, 0.80986,\
        0.82216, 0.83526, 0.84817, 0.85974, 0.86893, 0.87553, 0.88052, 0.88522, 0.89070, 0.89672, 0.90131,\
        0.90391, 0.90143, 0.89405, 0.88655, 0.88066, 0.87762, 0.87782, 0.88031, 0.88337, 0.88541, 0.88407,\
        0.87547, 0.86028, 0.84250, 0.82530, 0.81090, 0.79936, 0.79085, 0.78660, 0.78575, 0.78769, 0.79179,\
        0.79716, 0.80258, 0.80652, 0.80736, 0.80361, 0.79447, 0.78030, 0.76261, 0.74302, 0.72323, 0.70528,\
        0.69002, 0.67619, 0.66454, 0.65490, 0.64701, 0.64063, 0.63554, 0.63149, 0.62819, 0.62529, 0.62240,\
        0.61914, 0.61515, 0.61016, 0.60408, 0.59704, 0.58905, 0.58006, 0.57019, 0.55979, 0.54915, 0.53855,\
        0.52817, 0.51819, 0.50882, 0.50061, 0.49323, 0.48601, 0.47919, 0.47271, 0.46646, 0.46031, 0.45416],\
        [0.48574, 0.49157, 0.49722, 0.50238, 0.50725, 0.51219, 0.51717, 0.52223, 0.52745, 0.53290, 0.53866,\
        0.54478, 0.55125, 0.55806, 0.56512, 0.57232, 0.57950, 0.58644, 0.59278, 0.59787, 0.60056, 0.60391,\
        0.60771, 0.61186, 0.61653, 0.62198, 0.62844, 0.63606, 0.64478, 0.65448, 0.66497, 0.67602, 0.68732,\
        0.69874, 0.71092, 0.72309, 0.73486, 0.74604, 0.75646, 0.76599, 0.77468, 0.78289, 0.79129, 0.80094,\
        0.81224, 0.82378, 0.83458, 0.84404, 0.85183, 0.85812, 0.86370, 0.86954, 0.87607, 0.88271, 0.88823,\
        0.89142, 0.88986, 0.88385, 0.87703, 0.87155, 0.86852, 0.86816, 0.86968, 0.87158, 0.87201, 0.86876,\
        0.85993, 0.84577, 0.82868, 0.81188, 0.79883, 0.78931, 0.78236, 0.77888, 0.77883, 0.78177, 0.78703,\
        0.79360, 0.80006, 0.80506, 0.80621, 0.80170, 0.79115, 0.77558, 0.75670, 0.73625, 0.71576, 0.69697,\
        0.68206, 0.66913, 0.65794, 0.64857, 0.64087, 0.63470, 0.62988, 0.62623, 0.62342, 0.62108, 0.61874,\
        0.61595, 0.61229, 0.60741, 0.60111, 0.59369, 0.58525, 0.57586, 0.56575, 0.55519, 0.54447, 0.53386,\
        0.52358, 0.51375, 0.50442, 0.49602, 0.48887, 0.48202, 0.47540, 0.46902, 0.46283, 0.45672, 0.45063],\
        [0.48575, 0.49163, 0.49715, 0.50244, 0.50764, 0.51269, 0.51769, 0.52270, 0.52782, 0.53314, 0.53873,\
        0.54466, 0.55093, 0.55754, 0.56443, 0.57151, 0.57865, 0.58566, 0.59225, 0.59791, 0.60113, 0.60323,\
        0.60596, 0.60924, 0.61311, 0.61787, 0.62386, 0.63123, 0.63986, 0.64948, 0.65984, 0.67069, 0.68184,\
        0.69329, 0.70508, 0.71661, 0.72782, 0.73862, 0.74877, 0.75811, 0.76667, 0.77475, 0.78285, 0.79163,\
        0.80127, 0.81078, 0.81938, 0.82686, 0.83329, 0.83897, 0.84473, 0.85147, 0.85916, 0.86677, 0.87292,\
        0.87623, 0.87550, 0.87120, 0.86550, 0.86052, 0.85756, 0.85693, 0.85788, 0.85891, 0.85818, 0.85393,\
        0.84506, 0.83174, 0.81552, 0.79951, 0.78733, 0.77872, 0.77279, 0.76992, 0.77041, 0.77413, 0.78054,\
        0.78859, 0.79680, 0.80319, 0.80508, 0.79969, 0.78729, 0.76999, 0.74995, 0.72902, 0.70869, 0.69029,\
        0.67481, 0.66191, 0.65103, 0.64193, 0.63444, 0.62844, 0.62386, 0.62059, 0.61831, 0.61657, 0.61486,\
        0.61262, 0.60933, 0.60470, 0.59832, 0.59032, 0.58127, 0.57142, 0.56100, 0.55027, 0.53949, 0.52892,\
        0.51882, 0.50938, 0.50064, 0.49234, 0.48494, 0.47812, 0.47163, 0.46536, 0.45924, 0.45320, 0.44716],\
        [0.48545, 0.49143, 0.49703, 0.50243, 0.50772, 0.51280, 0.51773, 0.52261, 0.52755, 0.53263, 0.53796,\
        0.54360, 0.54958, 0.55590, 0.56252, 0.56935, 0.57626, 0.58304, 0.58938, 0.59469, 0.59827, 0.60068,\
        0.60308, 0.60578, 0.60893, 0.61297, 0.61852, 0.62579, 0.63445, 0.64407, 0.65434, 0.66505, 0.67609,\
        0.68741, 0.69874, 0.70955, 0.72036, 0.73096, 0.74093, 0.75010, 0.75851, 0.76637, 0.77406, 0.78189,\
        0.78986, 0.79740, 0.80394, 0.80961, 0.81477, 0.81950, 0.82491, 0.83243, 0.84148, 0.85022, 0.85704,\
        0.86073, 0.86067, 0.85733, 0.85232, 0.84758, 0.84469, 0.84416, 0.84517, 0.84601, 0.84486, 0.84032,\
        0.83177, 0.81953, 0.80472, 0.78983, 0.77776, 0.76878, 0.76271, 0.75975, 0.76026, 0.76439, 0.77178,\
        0.78146, 0.79169, 0.79997, 0.80361, 0.79663, 0.78181, 0.76273, 0.74170, 0.72059, 0.70074, 0.68277,\
        0.66738, 0.65456, 0.64393, 0.63513, 0.62786, 0.62196, 0.61750, 0.61456, 0.61282, 0.61168, 0.61060,\
        0.60901, 0.60604, 0.60124, 0.59476, 0.58633, 0.57677, 0.56652, 0.55586, 0.54500, 0.53418, 0.52366,\
        0.51374, 0.50461, 0.49623, 0.48837, 0.48109, 0.47433, 0.46793, 0.46175, 0.45571, 0.44974, 0.44376],\
        [0.48488, 0.49107, 0.49697, 0.50247, 0.50761, 0.51256, 0.51734, 0.52199, 0.52663, 0.53138, 0.53634,\
        0.54160, 0.54721, 0.55316, 0.55940, 0.56586, 0.57237, 0.57873, 0.58461, 0.58960, 0.59340, 0.59630,\
        0.59888, 0.60144, 0.60407, 0.60740, 0.61264, 0.62010, 0.62893, 0.63855, 0.64870, 0.65923, 0.67001,\
        0.68090, 0.69160, 0.70208, 0.71285, 0.72339, 0.73322, 0.74220, 0.75036, 0.75788, 0.76501, 0.77188,\
        0.77843, 0.78435, 0.78936, 0.79365, 0.79776, 0.80138, 0.80596, 0.81428, 0.82484, 0.83461, 0.84209,\
        0.84620, 0.84650, 0.84343, 0.83834, 0.83324, 0.83023, 0.83022, 0.83204, 0.83342, 0.83243, 0.82798,\
        0.81986, 0.80858, 0.79524, 0.78118, 0.76886, 0.75914, 0.75224, 0.74847, 0.74833, 0.75226, 0.76020,\
        0.77120, 0.78345, 0.79468, 0.79834, 0.78916, 0.77291, 0.75293, 0.73144, 0.71034, 0.69123, 0.67411,\
        0.65926, 0.64683, 0.63665, 0.62831, 0.62134, 0.61544, 0.61087, 0.60817, 0.60698, 0.60630, 0.60568,\
        0.60486, 0.60285, 0.59746, 0.59020, 0.58134, 0.57144, 0.56096, 0.55019, 0.53935, 0.52860, 0.51815,\
        0.50837, 0.49956, 0.49158, 0.48415, 0.47716, 0.47056, 0.46428, 0.45820, 0.45225, 0.44634, 0.44042],\
        [0.48399, 0.49036, 0.49636, 0.50191, 0.50711, 0.51198, 0.51654, 0.52088, 0.52512, 0.52942, 0.53392,\
        0.53872, 0.54387, 0.54937, 0.55517, 0.56117, 0.56722, 0.57311, 0.57859, 0.58338, 0.58736, 0.59065,\
        0.59354, 0.59622, 0.59880, 0.60152, 0.60683, 0.61485, 0.62376, 0.63321, 0.64310, 0.65336, 0.66386,\
        0.67441, 0.68485, 0.69524, 0.70589, 0.71626, 0.72585, 0.73452, 0.74232, 0.74941, 0.75593, 0.76196,\
        0.76744, 0.77223, 0.77627, 0.77977, 0.78305, 0.78640, 0.79114, 0.79994, 0.81084, 0.82100, 0.82891,\
        0.83335, 0.83368, 0.83035, 0.82460, 0.81837, 0.81478, 0.81574, 0.81904, 0.82145, 0.82087, 0.81649,\
        0.80870, 0.79782, 0.78519, 0.77199, 0.75973, 0.74939, 0.74146, 0.73638, 0.73487, 0.73785, 0.74570,\
        0.75730, 0.77008, 0.78042, 0.78300, 0.77524, 0.76002, 0.74064, 0.71961, 0.69918, 0.68092, 0.66472,\
        0.65054, 0.63866, 0.62914, 0.62158, 0.61521, 0.60936, 0.60429, 0.60166, 0.60095, 0.60036, 0.59975,\
        0.59925, 0.59791, 0.59264, 0.58446, 0.57514, 0.56506, 0.55458, 0.54393, 0.53332, 0.52286, 0.51263,\
        0.50298, 0.49459, 0.48703, 0.47994, 0.47322, 0.46682, 0.46067, 0.45470, 0.44883, 0.44300, 0.43714],\
        [0.48276, 0.48931, 0.49540, 0.50104, 0.50632, 0.51111, 0.51541, 0.51932, 0.52304, 0.52678, 0.53073,\
        0.53500, 0.53965, 0.54466, 0.54998, 0.55550, 0.56108, 0.56656, 0.57173, 0.57642, 0.58057, 0.58422,\
        0.58753, 0.59065, 0.59371, 0.59714, 0.60309, 0.61084, 0.61919, 0.62814, 0.63762, 0.64756, 0.65780,\
        0.66820, 0.67863, 0.68909, 0.69954, 0.70961, 0.71885, 0.72708, 0.73441, 0.74101, 0.74699, 0.75237,\
        0.75711, 0.76117, 0.76460, 0.76766, 0.77077, 0.77459, 0.78031, 0.78891, 0.79926, 0.80936, 0.81760,\
        0.82233, 0.82235, 0.81842, 0.81211, 0.80440, 0.79991, 0.80196, 0.80657, 0.80999, 0.80983, 0.80490,\
        0.79744, 0.78694, 0.77482, 0.76223, 0.75021, 0.73955, 0.73073, 0.72420, 0.72073, 0.72193, 0.72922,\
        0.74096, 0.75331, 0.76235, 0.76449, 0.75822, 0.74489, 0.72709, 0.70742, 0.68837, 0.67102, 0.65534,\
        0.64155, 0.63007, 0.62127, 0.61485, 0.60962, 0.60430, 0.59896, 0.59585, 0.59463, 0.59377, 0.59290,\
        0.59178, 0.58942, 0.58434, 0.57662, 0.56746, 0.55754, 0.54729, 0.53702, 0.52690, 0.51705, 0.50758,\
        0.49832, 0.49014, 0.48275, 0.47586, 0.46934, 0.46311, 0.45711, 0.45125, 0.44547, 0.43971, 0.43390],\
        [0.48121, 0.48803, 0.49436, 0.50016, 0.50547, 0.51010, 0.51401, 0.51737, 0.52045, 0.52352, 0.52684,\
        0.53054, 0.53466, 0.53918, 0.54401, 0.54906, 0.55422, 0.55934, 0.56430, 0.56897, 0.57331, 0.57735,\
        0.58122, 0.58509, 0.58929, 0.59446, 0.60092, 0.60745, 0.61486, 0.62314, 0.63216, 0.64179, 0.65184,\
        0.66216, 0.67260, 0.68306, 0.69339, 0.70323, 0.71205, 0.71972, 0.72653, 0.73267, 0.73822, 0.74316,\
        0.74743, 0.75104, 0.75409, 0.75690, 0.76003, 0.76424, 0.77037, 0.77880, 0.78877, 0.79874, 0.80755,\
        0.81278, 0.81187, 0.80672, 0.80022, 0.79290, 0.78831, 0.78949, 0.79361, 0.79756, 0.79803, 0.79264,\
        0.78507, 0.77531, 0.76397, 0.75206, 0.74049, 0.72990, 0.72071, 0.71315, 0.70773, 0.70641, 0.71318,\
        0.72511, 0.73680, 0.74495, 0.74695, 0.74168, 0.72998, 0.71401, 0.69619, 0.67856, 0.66188, 0.64645,\
        0.63268, 0.62118, 0.61280, 0.60775, 0.60390, 0.59957, 0.59432, 0.59046, 0.58815, 0.58654, 0.58500,\
        0.58299, 0.57974, 0.57450, 0.56722, 0.55850, 0.54896, 0.53914, 0.52943, 0.52001, 0.51089, 0.50222,\
        0.49404, 0.48594, 0.47860, 0.47184, 0.46549, 0.45942, 0.45356, 0.44782, 0.44214, 0.43645, 0.43070],\
        [0.47927, 0.48649, 0.49319, 0.49926, 0.50460, 0.50901, 0.51242, 0.51508, 0.51736, 0.51967, 0.52230,\
        0.52543, 0.52905, 0.53310, 0.53748, 0.54209, 0.54686, 0.55170, 0.55650, 0.56120, 0.56577, 0.57023,\
        0.57469, 0.57932, 0.58441, 0.59025, 0.59663, 0.60310, 0.61006, 0.61788, 0.62655, 0.63595, 0.64587,\
        0.65612, 0.66651, 0.67689, 0.68706, 0.69669, 0.70510, 0.71214, 0.71850, 0.72430, 0.72956, 0.73424,\
        0.73825, 0.74158, 0.74434, 0.74688, 0.74982, 0.75398, 0.76009, 0.76835, 0.77814, 0.78835, 0.79809,\
        0.80295, 0.80058, 0.79383, 0.78654, 0.77982, 0.77579, 0.77600, 0.77909, 0.78235, 0.78276, 0.77890,\
        0.77208, 0.76314, 0.75270, 0.74159, 0.73068, 0.72063, 0.71178, 0.70425, 0.69831, 0.69599, 0.70216,\
        0.71262, 0.72305, 0.73043, 0.73239, 0.72775, 0.71704, 0.70262, 0.68652, 0.66996, 0.65374, 0.63839,\
        0.62445, 0.61253, 0.60370, 0.59983, 0.59660, 0.59258, 0.58803, 0.58405, 0.58104, 0.57864, 0.57630,\
        0.57347, 0.56955, 0.56410, 0.55706, 0.54874, 0.53963, 0.53027, 0.52118, 0.51264, 0.50441, 0.49625,\
        0.48863, 0.48125, 0.47427, 0.46775, 0.46160, 0.45572, 0.45001, 0.44440, 0.43882, 0.43321, 0.42752],\
        [0.47681, 0.48450, 0.49171, 0.49818, 0.50366, 0.50782, 0.51064, 0.51245, 0.51380, 0.51525, 0.51720,\
        0.51981, 0.52300, 0.52663, 0.53059, 0.53479, 0.53921, 0.54380, 0.54850, 0.55327, 0.55807, 0.56293,\
        0.56791, 0.57313, 0.57874, 0.58479, 0.59115, 0.59766, 0.60456, 0.61221, 0.62071, 0.63000, 0.63985,\
        0.65002, 0.66028, 0.67043, 0.68025, 0.68940, 0.69733, 0.70403, 0.71021, 0.71577, 0.72088, 0.72548,\
        0.72938, 0.73251, 0.73496, 0.73704, 0.73941, 0.74302, 0.74871, 0.75668, 0.76628, 0.77626, 0.78480,\
        0.78838, 0.78559, 0.77871, 0.77092, 0.76429, 0.76053, 0.76045, 0.76298, 0.76581, 0.76656, 0.76406,\
        0.75855, 0.75070, 0.74121, 0.73093, 0.72079, 0.71159, 0.70369, 0.69686, 0.69197, 0.69097, 0.69553,\
        0.70369, 0.71271, 0.71952, 0.72161, 0.71757, 0.70736, 0.69359, 0.67859, 0.66261, 0.64664, 0.63135,\
        0.61734, 0.60537, 0.59637, 0.59151, 0.58806, 0.58427, 0.58022, 0.57638, 0.57305, 0.57008, 0.56709,\
        0.56361, 0.55922, 0.55361, 0.54672, 0.53872, 0.52997, 0.52096, 0.51236, 0.50465, 0.49760, 0.49015,\
        0.48299, 0.47617, 0.46967, 0.46350, 0.45762, 0.45195, 0.44642, 0.44096, 0.43550, 0.42998, 0.42435],\
        [0.47362, 0.48183, 0.48961, 0.49664, 0.50242, 0.50645, 0.50861, 0.50944, 0.50977, 0.51032, 0.51164,\
        0.51385, 0.51672, 0.51999, 0.52355, 0.52736, 0.53144, 0.53581, 0.54045, 0.54531, 0.55034, 0.55555,\
        0.56097, 0.56663, 0.57258, 0.57879, 0.58520, 0.59177, 0.59868, 0.60627, 0.61472, 0.62401, 0.63384,\
        0.64390, 0.65394, 0.66374, 0.67306, 0.68163, 0.68915, 0.69576, 0.70175, 0.70700, 0.71206, 0.71671,\
        0.72055, 0.72353, 0.72565, 0.72702, 0.72837, 0.73089, 0.73579, 0.74339, 0.75265, 0.76178, 0.76859,\
        0.77087, 0.76792, 0.76125, 0.75337, 0.74661, 0.74290, 0.74312, 0.74603, 0.74923, 0.75066, 0.74929,\
        0.74503, 0.73831, 0.72979, 0.72025, 0.71080, 0.70246, 0.69555, 0.68997, 0.68652, 0.68647, 0.69035,\
        0.69715, 0.70526, 0.71171, 0.71413, 0.71081, 0.70177, 0.68779, 0.67220, 0.65614, 0.64029, 0.62521,\
        0.61126, 0.59939, 0.59049, 0.58431, 0.57976, 0.57565, 0.57165, 0.56785, 0.56435, 0.56101, 0.55757,\
        0.55366, 0.54896, 0.54329, 0.53657, 0.52889, 0.52050, 0.51175, 0.50345, 0.49639, 0.48999, 0.48354,\
        0.47710, 0.47086, 0.46485, 0.45908, 0.45352, 0.44811, 0.44279, 0.43749, 0.43215, 0.42673, 0.42117],\
        [0.46949, 0.47816, 0.48652, 0.49418, 0.50056, 0.50471, 0.50616, 0.50594, 0.50536, 0.50500, 0.50582,\
        0.50784, 0.51049, 0.51345, 0.51660, 0.51997, 0.52370, 0.52789, 0.53250, 0.53745, 0.54271, 0.54823,\
        0.55402, 0.56005, 0.56630, 0.57272, 0.57927, 0.58595, 0.59289, 0.60039, 0.60882, 0.61818, 0.62799,\
        0.63789, 0.64762, 0.65698, 0.66577, 0.67381, 0.68099, 0.68735, 0.69298, 0.69804, 0.70305, 0.70779,\
        0.71137, 0.71420, 0.71609, 0.71657, 0.71650, 0.71742, 0.72127, 0.72873, 0.73788, 0.74636, 0.75204,\
        0.75331, 0.74987, 0.74304, 0.73496, 0.72778, 0.72392, 0.72505, 0.72932, 0.73343, 0.73562, 0.73518,\
        0.73194, 0.72623, 0.71867, 0.70988, 0.70088, 0.69345, 0.68740, 0.68298, 0.68078, 0.68151, 0.68538,\
        0.69175, 0.69907, 0.70551, 0.70849, 0.70500, 0.69559, 0.68172, 0.66583, 0.64962, 0.63400, 0.61949,\
        0.60615, 0.59423, 0.58476, 0.57752, 0.57185, 0.56707, 0.56278, 0.55884, 0.55518, 0.55164, 0.54795,\
        0.54380, 0.53894, 0.53329, 0.52684, 0.51957, 0.51164, 0.50350, 0.49550, 0.48883, 0.48278, 0.47689,\
        0.47107, 0.46538, 0.45986, 0.45451, 0.44930, 0.44418, 0.43909, 0.43397, 0.42878, 0.42345, 0.41797],\
        [0.46422, 0.47321, 0.48201, 0.49027, 0.49738, 0.50198, 0.50299, 0.50167, 0.50094, 0.49977, 0.50028,\
        0.50220, 0.50458, 0.50720, 0.50993, 0.51280, 0.51615, 0.52019, 0.52481, 0.52987, 0.53531, 0.54113,\
        0.54725, 0.55363, 0.56019, 0.56688, 0.57366, 0.58053, 0.58760, 0.59507, 0.60350, 0.61287, 0.62254,\
        0.63215, 0.64145, 0.65029, 0.65851, 0.66602, 0.67277, 0.67877, 0.68414, 0.68906, 0.69385, 0.69830,\
        0.70118, 0.70389, 0.70580, 0.70522, 0.70377, 0.70303, 0.70575, 0.71363, 0.72300, 0.73124, 0.73644,\
        0.73707, 0.73288, 0.72550, 0.71736, 0.70979, 0.70539, 0.70801, 0.71419, 0.71888, 0.72154, 0.72186,\
        0.71937, 0.71443, 0.70777, 0.70008, 0.69174, 0.68468, 0.67921, 0.67556, 0.67419, 0.67547, 0.67945,\
        0.68554, 0.69260, 0.69943, 0.70278, 0.69806, 0.68767, 0.67383, 0.65816, 0.64209, 0.62676, 0.61308,\
        0.60065, 0.58885, 0.57873, 0.57057, 0.56398, 0.55849, 0.55375, 0.54956, 0.54577, 0.54216, 0.53842,\
        0.53419, 0.52925, 0.52366, 0.51761, 0.51093, 0.50335, 0.49588, 0.48858, 0.48197, 0.47603, 0.47044,\
        0.46506, 0.45984, 0.45477, 0.44983, 0.44498, 0.44016, 0.43532, 0.43040, 0.42535, 0.42013, 0.41471],\
        [0.45772, 0.46683, 0.47580, 0.48425, 0.49153, 0.49664, 0.49827, 0.49701, 0.49614, 0.49531, 0.49559,\
        0.49706, 0.49907, 0.50139, 0.50371, 0.50596, 0.50893, 0.51296, 0.51761, 0.52271, 0.52831, 0.53439,\
        0.54085, 0.54758, 0.55448, 0.56149, 0.56856, 0.57573, 0.58307, 0.59079, 0.59917, 0.60827, 0.61762,\
        0.62677, 0.63551, 0.64372, 0.65132, 0.65824, 0.66446, 0.67002, 0.67501, 0.67956, 0.68374, 0.68727,\
        0.68984, 0.69192, 0.69312, 0.69187, 0.68974, 0.68882, 0.69154, 0.69916, 0.70850, 0.71676, 0.72223,\
        0.72285, 0.71783, 0.70938, 0.70177, 0.69504, 0.69143, 0.69461, 0.70105, 0.70506, 0.70796, 0.70904,\
        0.70699, 0.70255, 0.69657, 0.68961, 0.68229, 0.67571, 0.67060, 0.66737, 0.66639, 0.66792, 0.67187,\
        0.67767, 0.68437, 0.69043, 0.69247, 0.68756, 0.67737, 0.66400, 0.64890, 0.63321, 0.61797, 0.60455,\
        0.59316, 0.58181, 0.57167, 0.56306, 0.55585, 0.54976, 0.54457, 0.54010, 0.53623, 0.53270, 0.52911,\
        0.52499, 0.52001, 0.51433, 0.50861, 0.50311, 0.49587, 0.48860, 0.48170, 0.47532, 0.46951, 0.46414,\
        0.45908, 0.45426, 0.44960, 0.44505, 0.44056, 0.43605, 0.43147, 0.42676, 0.42187, 0.41675, 0.41140],\
        [0.45006, 0.45906, 0.46795, 0.47628, 0.48342, 0.48854, 0.49096, 0.49121, 0.49084, 0.49060, 0.49100,\
        0.49218, 0.49389, 0.49591, 0.49785, 0.49967, 0.50237, 0.50658, 0.51113, 0.51613, 0.52183, 0.52818,\
        0.53497, 0.54206, 0.54930, 0.55664, 0.56404, 0.57153, 0.57919, 0.58716, 0.59553, 0.60435, 0.61321,\
        0.62171, 0.62975, 0.63726, 0.64415, 0.65039, 0.65598, 0.66097, 0.66541, 0.66934, 0.67271, 0.67537,\
        0.67718, 0.67814, 0.67794, 0.67631, 0.67437, 0.67409, 0.67727, 0.68437, 0.69349, 0.70221, 0.70885,\
        0.71052, 0.70500, 0.69584, 0.68814, 0.68253, 0.68015, 0.68235, 0.68701, 0.69100, 0.69414, 0.69577,\
        0.69418, 0.69027, 0.68496, 0.67872, 0.67217, 0.66611, 0.66125, 0.65815, 0.65724, 0.65871, 0.66240,\
        0.66768, 0.67340, 0.67780, 0.67851, 0.67399, 0.66480, 0.65249, 0.63838, 0.62348, 0.60867, 0.59507,\
        0.58392, 0.57342, 0.56363, 0.55492, 0.54735, 0.54082, 0.53522, 0.53048, 0.52655, 0.52326, 0.52010,\
        0.51635, 0.51145, 0.50549, 0.49955, 0.49454, 0.48825, 0.48135, 0.47473, 0.46858, 0.46296, 0.45782,\
        0.45307, 0.44861, 0.44434, 0.44018, 0.43604, 0.43185, 0.42754, 0.42304, 0.41831, 0.41330, 0.40800],\
        [0.44145, 0.45021, 0.45889, 0.46703, 0.47406, 0.47936, 0.48263, 0.48419, 0.48488, 0.48539, 0.48617,\
        0.48737, 0.48895, 0.49073, 0.49255, 0.49452, 0.49743, 0.50136, 0.50544, 0.51017, 0.51600, 0.52265,\
        0.52978, 0.53718, 0.54473, 0.55235, 0.56004, 0.56780, 0.57572, 0.58388, 0.59232, 0.60091, 0.60906,\
        0.61677, 0.62405, 0.63081, 0.63694, 0.64241, 0.64727, 0.65156, 0.65532, 0.65848, 0.66099, 0.66273,\
        0.66356, 0.66336, 0.66198, 0.65971, 0.65772, 0.65772, 0.66107, 0.66797, 0.67713, 0.68655, 0.69433,\
        0.69750, 0.69208, 0.68363, 0.67584, 0.67045, 0.66826, 0.66949, 0.67289, 0.67667, 0.67981, 0.68141,\
        0.68062, 0.67763, 0.67310, 0.66757, 0.66163, 0.65593, 0.65116, 0.64794, 0.64679, 0.64797, 0.65128,\
        0.65596, 0.66073, 0.66392, 0.66383, 0.65948, 0.65115, 0.63995, 0.62711, 0.61340, 0.59955, 0.58661,\
        0.57509, 0.56465, 0.55504, 0.54633, 0.53858, 0.53174, 0.52578, 0.52074, 0.51673, 0.51376, 0.51133,\
        0.50815, 0.50390, 0.49758, 0.49163, 0.48601, 0.48006, 0.47371, 0.46746, 0.46160, 0.45623, 0.45136,\
        0.44693, 0.44283, 0.43896, 0.43519, 0.43142, 0.42755, 0.42350, 0.41922, 0.41466, 0.40975, 0.40450],\
        [0.43221, 0.44068, 0.44916, 0.45720, 0.46427, 0.46993, 0.47400, 0.47668, 0.47845, 0.47981, 0.48114,\
        0.48263, 0.48430, 0.48613, 0.48808, 0.49034, 0.49324, 0.49672, 0.50031, 0.50489, 0.51105, 0.51802,\
        0.52542, 0.53306, 0.54081, 0.54862, 0.55648, 0.56440, 0.57242, 0.58058, 0.58887, 0.59706, 0.60461,\
        0.61162, 0.61824, 0.62429, 0.62961, 0.63421, 0.63823, 0.64177, 0.64477, 0.64713, 0.64875, 0.64957,\
        0.64948, 0.64831, 0.64592, 0.64268, 0.64003, 0.63974, 0.64295, 0.64992, 0.65942, 0.66924, 0.67701,\
        0.67999, 0.67668, 0.66983, 0.66275, 0.65753, 0.65524, 0.65595, 0.65876, 0.66232, 0.66545, 0.66724,\
        0.66710, 0.66499, 0.66128, 0.65644, 0.65099, 0.64550, 0.64062, 0.63701, 0.63536, 0.63612, 0.63911,\
        0.64342, 0.64764, 0.65020, 0.64971, 0.64544, 0.63754, 0.62710, 0.61550, 0.60314, 0.59024, 0.57779,\
        0.56631, 0.55582, 0.54623, 0.53752, 0.52967, 0.52266, 0.51640, 0.51102, 0.50687, 0.50402, 0.50269,\
        0.49965, 0.49568, 0.49003, 0.48380, 0.47782, 0.47184, 0.46579, 0.45987, 0.45430, 0.44922, 0.44466,\
        0.44058, 0.43688, 0.43343, 0.43008, 0.42668, 0.42312, 0.41933, 0.41526, 0.41088, 0.40611, 0.40087],\
        [0.42274, 0.43093, 0.43931, 0.44738, 0.45463, 0.46073, 0.46553, 0.46914, 0.47188, 0.47410, 0.47608,\
        0.47803, 0.48002, 0.48209, 0.48431, 0.48679, 0.48965, 0.49284, 0.49636, 0.50110, 0.50742, 0.51451,\
        0.52201, 0.52972, 0.53754, 0.54540, 0.55329, 0.56119, 0.56913, 0.57708, 0.58496, 0.59255, 0.59957,\
        0.60610, 0.61226, 0.61769, 0.62212, 0.62570, 0.62880, 0.63157, 0.63384, 0.63539, 0.63613, 0.63606,\
        0.63520, 0.63340, 0.63027, 0.62581, 0.62181, 0.62094, 0.62356, 0.63083, 0.64098, 0.65112, 0.65881,\
        0.66198, 0.66005, 0.65463, 0.64833, 0.64337, 0.64112, 0.64183, 0.64466, 0.64830, 0.65162, 0.65375,\
        0.65419, 0.65282, 0.64986, 0.64565, 0.64062, 0.63526, 0.63014, 0.62594, 0.62352, 0.62377, 0.62669,\
        0.63103, 0.63519, 0.63763, 0.63709, 0.63288, 0.62503, 0.61458, 0.60367, 0.59260, 0.58054, 0.56860,\
        0.55732, 0.54688, 0.53731, 0.52860, 0.52076, 0.51374, 0.50738, 0.50170, 0.49748, 0.49471, 0.49312,\
        0.49040, 0.48631, 0.48118, 0.47538, 0.46945, 0.46353, 0.45767, 0.45200, 0.44669, 0.44189, 0.43766,\
        0.43397, 0.43071, 0.42772, 0.42481, 0.42180, 0.41855, 0.41499, 0.41110, 0.40688, 0.40235, 0.39713],\
        [0.41346, 0.42146, 0.42987, 0.43806, 0.44557, 0.45208, 0.45751, 0.46192, 0.46550, 0.46851, 0.47118,\
        0.47368, 0.47612, 0.47859, 0.48116, 0.48392, 0.48695, 0.49031, 0.49420, 0.49910, 0.50521, 0.51216,\
        0.51954, 0.52715, 0.53486, 0.54262, 0.55037, 0.55810, 0.56578, 0.57336, 0.58075, 0.58777, 0.59428,\
        0.60037, 0.60617, 0.61104, 0.61445, 0.61679, 0.61888, 0.62097, 0.62262, 0.62337, 0.62317, 0.62222,\
        0.62068, 0.61860, 0.61536, 0.61003, 0.60369, 0.60298, 0.60399, 0.61222, 0.62304, 0.63358, 0.64156,\
        0.64515, 0.64389, 0.63912, 0.63318, 0.62831, 0.62624, 0.62752, 0.63091, 0.63496, 0.63865, 0.64119,\
        0.64214, 0.64141, 0.63911, 0.63550, 0.63090, 0.62570, 0.62039, 0.61554, 0.61209, 0.61176, 0.61501,\
        0.61974, 0.62418, 0.62688, 0.62658, 0.62255, 0.61479, 0.60395, 0.59285, 0.58215, 0.57075, 0.55923,\
        0.54815, 0.53782, 0.52831, 0.51963, 0.51184, 0.50498, 0.49875, 0.49316, 0.48881, 0.48574, 0.48330,\
        0.48040, 0.47653, 0.47178, 0.46642, 0.46078, 0.45504, 0.44936, 0.44388, 0.43878, 0.43424, 0.43035,\
        0.42708, 0.42430, 0.42182, 0.41939, 0.41677, 0.41382, 0.41045, 0.40668, 0.40249, 0.39801, 0.39301],\
        [0.40492, 0.41286, 0.42138, 0.42971, 0.43741, 0.44426, 0.45018, 0.45522, 0.45952, 0.46325, 0.46658,\
        0.46968, 0.47265, 0.47559, 0.47859, 0.48174, 0.48512, 0.48885, 0.49312, 0.49818, 0.50414, 0.51078,\
        0.51787, 0.52521, 0.53268, 0.54018, 0.54766, 0.55507, 0.56236, 0.56948, 0.57635, 0.58285, 0.58893,\
        0.59463, 0.60000, 0.60430, 0.60648, 0.60734, 0.60838, 0.61003, 0.61123, 0.61111, 0.60993, 0.60803,\
        0.60571, 0.60326, 0.60013, 0.59482, 0.58831, 0.58601, 0.58779, 0.59560, 0.60619, 0.61733, 0.62600,\
        0.63013, 0.62907, 0.62422, 0.61824, 0.61332, 0.61132, 0.61388, 0.61809, 0.62261, 0.62675, 0.62967,\
        0.63104, 0.63081, 0.62913, 0.62614, 0.62204, 0.61717, 0.61199, 0.60691, 0.60260, 0.60152, 0.60546,\
        0.61035, 0.61510, 0.61826, 0.61833, 0.61455, 0.60704, 0.59655, 0.58473, 0.57308, 0.56140, 0.54988,\
        0.53888, 0.52867, 0.51930, 0.51069, 0.50287, 0.49613, 0.49008, 0.48469, 0.48026, 0.47674, 0.47369,\
        0.47047, 0.46666, 0.46221, 0.45723, 0.45191, 0.44643, 0.44093, 0.43559, 0.43065, 0.42632, 0.42275,\
        0.41990, 0.41764, 0.41571, 0.41379, 0.41158, 0.40889, 0.40566, 0.40199, 0.39790, 0.39336, 0.38843],\
        [0.39810, 0.40590, 0.41436, 0.42263, 0.43037, 0.43740, 0.44368, 0.44921, 0.45409, 0.45843, 0.46239,\
        0.46607, 0.46959, 0.47306, 0.47654, 0.48014, 0.48394, 0.48807, 0.49269, 0.49793, 0.50372, 0.51002,\
        0.51675, 0.52375, 0.53086, 0.53799, 0.54507, 0.55204, 0.55885, 0.56543, 0.57172, 0.57767, 0.58323,\
        0.58839, 0.59304, 0.59651, 0.59774, 0.59755, 0.59772, 0.59885, 0.59958, 0.59849, 0.59642, 0.59367,\
        0.59051, 0.58711, 0.58296, 0.57746, 0.57207, 0.56969, 0.57197, 0.57929, 0.59013, 0.60201, 0.61185,\
        0.61689, 0.61584, 0.61045, 0.60438, 0.60031, 0.59828, 0.60231, 0.60654, 0.61134, 0.61599, 0.61909,\
        0.62071, 0.62088, 0.61979, 0.61754, 0.61407, 0.60964, 0.60500, 0.60048, 0.59667, 0.59583, 0.59874,\
        0.60298, 0.60800, 0.61163, 0.61200, 0.60817, 0.60032, 0.58957, 0.57727, 0.56473, 0.55242, 0.54060,\
        0.52954, 0.51943, 0.51030, 0.50190, 0.49416, 0.48737, 0.48140, 0.47612, 0.47159, 0.46773, 0.46425,\
        0.46076, 0.45695, 0.45270, 0.44803, 0.44303, 0.43782, 0.43251, 0.42728, 0.42239, 0.41819, 0.41492,\
        0.41250, 0.41075, 0.40940, 0.40799, 0.40615, 0.40373, 0.40058, 0.39693, 0.39292, 0.38846, 0.38357],\
        [0.39358, 0.40100, 0.40901, 0.41695, 0.42452, 0.43157, 0.43805, 0.44393, 0.44927, 0.45414, 0.45864,\
        0.46288, 0.46694, 0.47093, 0.47491, 0.47898, 0.48322, 0.48771, 0.49254, 0.49779, 0.50346, 0.50955,\
        0.51596, 0.52257, 0.52925, 0.53592, 0.54251, 0.54895, 0.55519, 0.56115, 0.56680, 0.57207, 0.57692,\
        0.58127, 0.58487, 0.58725, 0.58801, 0.58768, 0.58736, 0.58738, 0.58696, 0.58531, 0.58264, 0.57923,\
        0.57526, 0.57071, 0.56542, 0.55958, 0.55457, 0.55246, 0.55489, 0.56230, 0.57353, 0.58621, 0.59767,\
        0.60401, 0.60346, 0.59803, 0.59264, 0.58942, 0.58894, 0.59193, 0.59590, 0.60137, 0.60609, 0.60895,\
        0.61079, 0.61125, 0.61074, 0.60946, 0.60683, 0.60276, 0.59874, 0.59511, 0.59237, 0.59187, 0.59388,\
        0.59763, 0.60256, 0.60625, 0.60664, 0.60237, 0.59366, 0.58225, 0.56947, 0.55632, 0.54345, 0.53130,\
        0.52012, 0.51004, 0.50118, 0.49306, 0.48552, 0.47878, 0.47280, 0.46751, 0.46286, 0.45875, 0.45498,\
        0.45129, 0.44747, 0.44338, 0.43897, 0.43428, 0.42936, 0.42428, 0.41913, 0.41419, 0.41002, 0.40706,\
        0.40503, 0.40367, 0.40286, 0.40190, 0.40030, 0.39806, 0.39509, 0.39156, 0.38761, 0.38324, 0.37844],\
        [0.39037, 0.39750, 0.40500, 0.41246, 0.41973, 0.42670, 0.43326, 0.43938, 0.44507, 0.45037, 0.45535,\
        0.46008, 0.46465, 0.46913, 0.47360, 0.47813, 0.48277, 0.48758, 0.49259, 0.49775, 0.50331, 0.50924,\
        0.51533, 0.52151, 0.52771, 0.53387, 0.53990, 0.54574, 0.55132, 0.55660, 0.56151, 0.56601, 0.57001,\
        0.57340, 0.57597, 0.57746, 0.57778, 0.57725, 0.57641, 0.57545, 0.57404, 0.57179, 0.56866, 0.56475,\
        0.56011, 0.55470, 0.54848, 0.54198, 0.53664, 0.53429, 0.53650, 0.54411, 0.55588, 0.56912, 0.58092,\
        0.58797, 0.58901, 0.58585, 0.58184, 0.57943, 0.57945, 0.58192, 0.58605, 0.59131, 0.59554, 0.59852,\
        0.60080, 0.60144, 0.60150, 0.60127, 0.59996, 0.59626, 0.59279, 0.58995, 0.58814, 0.58801, 0.58979,\
        0.59316, 0.59724, 0.60053, 0.60032, 0.59517, 0.58588, 0.57413, 0.56107, 0.54754, 0.53427, 0.52193,\
        0.51078, 0.50070, 0.49203, 0.48417, 0.47689, 0.47025, 0.46428, 0.45893, 0.45414, 0.44983, 0.44585,\
        0.44204, 0.43823, 0.43428, 0.43012, 0.42572, 0.42114, 0.41638, 0.41141, 0.40637, 0.40209, 0.39947,\
        0.39778, 0.39666, 0.39606, 0.39526, 0.39387, 0.39179, 0.38907, 0.38576, 0.38197, 0.37773, 0.37304],\
        [0.38784, 0.39469, 0.40179, 0.40881, 0.41580, 0.42264, 0.42923, 0.43552, 0.44148, 0.44712, 0.45249,\
        0.45764, 0.46265, 0.46758, 0.47249, 0.47743, 0.48245, 0.48757, 0.49281, 0.49819, 0.50353, 0.50904,\
        0.51471, 0.52042, 0.52610, 0.53170, 0.53713, 0.54232, 0.54720, 0.55173, 0.55584, 0.55948, 0.56258,\
        0.56501, 0.56665, 0.56736, 0.56715, 0.56621, 0.56482, 0.56310, 0.56094, 0.55813, 0.55460, 0.55034,\
        0.54534, 0.53950, 0.53271, 0.52530, 0.51919, 0.51619, 0.51752, 0.52544, 0.53794, 0.55139, 0.56294,\
        0.57040, 0.57309, 0.57222, 0.57011, 0.56876, 0.56919, 0.57155, 0.57542, 0.57999, 0.58414, 0.58746,\
        0.58991, 0.59124, 0.59195, 0.59226, 0.59151, 0.58919, 0.58659, 0.58451, 0.58336, 0.58351, 0.58506,\
        0.58768, 0.59060, 0.59241, 0.59106, 0.58555, 0.57648, 0.56498, 0.55205, 0.53847, 0.52494, 0.51249,\
        0.50177, 0.49204, 0.48338, 0.47560, 0.46844, 0.46186, 0.45587, 0.45043, 0.44551, 0.44102, 0.43688,\
        0.43298, 0.42921, 0.42542, 0.42147, 0.41733, 0.41310, 0.40879, 0.40423, 0.39941, 0.39521, 0.39256,\
        0.39092, 0.38988, 0.38916, 0.38834, 0.38706, 0.38518, 0.38269, 0.37963, 0.37604, 0.37195, 0.36739],\
        [0.38576, 0.39228, 0.39898, 0.40576, 0.41255, 0.41928, 0.42587, 0.43227, 0.43842, 0.44432, 0.45000,\
        0.45550, 0.46087, 0.46617, 0.47145, 0.47675, 0.48210, 0.48749, 0.49287, 0.49816, 0.50342, 0.50869,\
        0.51392, 0.51914, 0.52429, 0.52931, 0.53412, 0.53863, 0.54278, 0.54652, 0.54979, 0.55255, 0.55473,\
        0.55624, 0.55701, 0.55697, 0.55616, 0.55469, 0.55273, 0.55038, 0.54762, 0.54436, 0.54050, 0.53603,\
        0.53094, 0.52517, 0.51855, 0.51050, 0.50322, 0.50005, 0.49993, 0.50840, 0.52141, 0.53461, 0.54582,\
        0.55348, 0.55719, 0.55788, 0.55724, 0.55693, 0.55789, 0.56036, 0.56403, 0.56825, 0.57233, 0.57585,\
        0.57862, 0.58059, 0.58190, 0.58259, 0.58238, 0.58122, 0.57967, 0.57836, 0.57772, 0.57798, 0.57914,\
        0.58086, 0.58241, 0.58265, 0.58029, 0.57467, 0.56602, 0.55511, 0.54273, 0.52957, 0.51609, 0.50307,\
        0.49299, 0.48376, 0.47524, 0.46745, 0.46029, 0.45369, 0.44763, 0.44208, 0.43700, 0.43235, 0.42805,\
        0.42408, 0.42036, 0.41675, 0.41302, 0.40904, 0.40505, 0.40122, 0.39704, 0.39270, 0.38887, 0.38613,\
        0.38435, 0.38318, 0.38231, 0.38139, 0.38015, 0.37842, 0.37613, 0.37328, 0.36988, 0.36596, 0.36154],\
        [0.38411, 0.39035, 0.39678, 0.40332, 0.40987, 0.41650, 0.42310, 0.42957, 0.43585, 0.44193, 0.44783,\
        0.45358, 0.45922, 0.46480, 0.47036, 0.47594, 0.48155, 0.48718, 0.49273, 0.49802, 0.50314, 0.50803,\
        0.51278, 0.51749, 0.52213, 0.52659, 0.53078, 0.53461, 0.53802, 0.54095, 0.54336, 0.54524, 0.54652,\
        0.54715, 0.54709, 0.54631, 0.54486, 0.54280, 0.54024, 0.53731, 0.53407, 0.53043, 0.52630, 0.52165,\
        0.51657, 0.51107, 0.50514, 0.49854, 0.49032, 0.48655, 0.48743, 0.49525, 0.50714, 0.51951, 0.53021,\
        0.53781, 0.54192, 0.54332, 0.54356, 0.54405, 0.54563, 0.54844, 0.55217, 0.55632, 0.56040, 0.56409,\
        0.56720, 0.56966, 0.57149, 0.57264, 0.57304, 0.57273, 0.57205, 0.57140, 0.57112, 0.57136, 0.57207,\
        0.57291, 0.57326, 0.57228, 0.56915, 0.56342, 0.55520, 0.54494, 0.53333, 0.52111, 0.50888, 0.49642,\
        0.48558, 0.47618, 0.46759, 0.45971, 0.45246, 0.44577, 0.43959, 0.43390, 0.42868, 0.42387, 0.41939,\
        0.41529, 0.41164, 0.40823, 0.40476, 0.40085, 0.39693, 0.39329, 0.38950, 0.38569, 0.38230, 0.37967,\
        0.37781, 0.37651, 0.37550, 0.37450, 0.37326, 0.37163, 0.36949, 0.36681, 0.36359, 0.35983, 0.35553],\
        [0.38287, 0.38888, 0.39502, 0.40125, 0.40764, 0.41426, 0.42087, 0.42737, 0.43371, 0.43988, 0.44591,\
        0.45181, 0.45761, 0.46337, 0.46911, 0.47486, 0.48063, 0.48642, 0.49217, 0.49777, 0.50260, 0.50678,\
        0.51102, 0.51531, 0.51949, 0.52345, 0.52705, 0.53022, 0.53288, 0.53499, 0.53655, 0.53756, 0.53798,\
        0.53777, 0.53690, 0.53540, 0.53330, 0.53063, 0.52745, 0.52392, 0.52025, 0.51635, 0.51191, 0.50703,\
        0.50184, 0.49631, 0.49045, 0.48407, 0.47774, 0.47443, 0.47600, 0.48299, 0.49373, 0.50554, 0.51579,\
        0.52334, 0.52736, 0.52877, 0.52936, 0.53047, 0.53276, 0.53614, 0.54018, 0.54445, 0.54860, 0.55244,\
        0.55585, 0.55874, 0.56104, 0.56268, 0.56364, 0.56399, 0.56394, 0.56377, 0.56371, 0.56385, 0.56412,\
        0.56421, 0.56363, 0.56178, 0.55810, 0.55231, 0.54445, 0.53483, 0.52393, 0.51242, 0.50143, 0.49021,\
        0.47903, 0.46915, 0.46030, 0.45227, 0.44490, 0.43809, 0.43177, 0.42592, 0.42056, 0.41564, 0.41098,\
        0.40662, 0.40301, 0.39972, 0.39639, 0.39269, 0.38892, 0.38532, 0.38181, 0.37843, 0.37542, 0.37298,\
        0.37114, 0.36978, 0.36870, 0.36765, 0.36643, 0.36486, 0.36285, 0.36031, 0.35722, 0.35359, 0.34941],\
        [0.38208, 0.38789, 0.39371, 0.39953, 0.40602, 0.41264, 0.41919, 0.42563, 0.43193, 0.43811, 0.44416,\
        0.45011, 0.45598, 0.46180, 0.46760, 0.47339, 0.47918, 0.48494, 0.49060, 0.49596, 0.50042, 0.50438,\
        0.50844, 0.51247, 0.51631, 0.51981, 0.52287, 0.52541, 0.52734, 0.52864, 0.52933, 0.52949, 0.52911,\
        0.52811, 0.52644, 0.52423, 0.52153, 0.51826, 0.51447, 0.51028, 0.50617, 0.50213, 0.49730, 0.49211,\
        0.48663, 0.48086, 0.47486, 0.46886, 0.46379, 0.46146, 0.46338, 0.46989, 0.47987, 0.49118, 0.50158,\
        0.50962, 0.51326, 0.51423, 0.51497, 0.51666, 0.51984, 0.52402, 0.52851, 0.53295, 0.53717, 0.54113,\
        0.54480, 0.54807, 0.55082, 0.55293, 0.55437, 0.55520, 0.55558, 0.55570, 0.55572, 0.55573, 0.55562,\
        0.55514, 0.55390, 0.55145, 0.54742, 0.54161, 0.53405, 0.52501, 0.51480, 0.50366, 0.49278, 0.48234,\
        0.47186, 0.46204, 0.45310, 0.44498, 0.43753, 0.43063, 0.42417, 0.41813, 0.41262, 0.40770, 0.40306,\
        0.39835, 0.39462, 0.39126, 0.38789, 0.38438, 0.38082, 0.37736, 0.37406, 0.37099, 0.36828, 0.36604,\
        0.36429, 0.36296, 0.36187, 0.36083, 0.35965, 0.35816, 0.35624, 0.35381, 0.35084, 0.34730, 0.34320],\
        [0.38172, 0.38747, 0.39328, 0.39923, 0.40538, 0.41168, 0.41800, 0.42427, 0.43045, 0.43654, 0.44253,\
        0.44843, 0.45426, 0.46004, 0.46578, 0.47148, 0.47713, 0.48267, 0.48800, 0.49287, 0.49710, 0.50106,\
        0.50511, 0.50899, 0.51253, 0.51561, 0.51816, 0.52011, 0.52136, 0.52185, 0.52164, 0.52096, 0.51991,\
        0.51820, 0.51567, 0.51275, 0.50956, 0.50574, 0.50142, 0.49670, 0.49204, 0.48756, 0.48253, 0.47707,\
        0.47131, 0.46531, 0.45923, 0.45350, 0.44903, 0.44721, 0.44927, 0.45552, 0.46502, 0.47603, 0.48674,\
        0.49514, 0.49888, 0.49990, 0.50116, 0.50344, 0.50774, 0.51275, 0.51760, 0.52212, 0.52631, 0.53033,\
        0.53425, 0.53791, 0.54108, 0.54360, 0.54541, 0.54656, 0.54718, 0.54742, 0.54743, 0.54726, 0.54686,\
        0.54600, 0.54434, 0.54152, 0.53727, 0.53146, 0.52415, 0.51555, 0.50604, 0.49573, 0.48487, 0.47440,\
        0.46425, 0.45464, 0.44577, 0.43768, 0.43026, 0.42335, 0.41681, 0.41056, 0.40479, 0.39984, 0.39527,\
        0.39071, 0.38665, 0.38305, 0.37958, 0.37613, 0.37270, 0.36939, 0.36627, 0.36342, 0.36094, 0.35888,\
        0.35727, 0.35602, 0.35500, 0.35403, 0.35293, 0.35153, 0.34970, 0.34736, 0.34446, 0.34099, 0.33694],\
        [0.38171, 0.38747, 0.39331, 0.39927, 0.40516, 0.41112, 0.41716, 0.42319, 0.42918, 0.43510, 0.44093,\
        0.44670, 0.45240, 0.45804, 0.46362, 0.46913, 0.47454, 0.47977, 0.48474, 0.48930, 0.49343, 0.49735,\
        0.50124, 0.50489, 0.50812, 0.51076, 0.51283, 0.51426, 0.51491, 0.51462, 0.51344, 0.51186, 0.51033,\
        0.50816, 0.50464, 0.50092, 0.49729, 0.49298, 0.48821, 0.48318, 0.47814, 0.47312, 0.46783, 0.46221,\
        0.45628, 0.45016, 0.44406, 0.43841, 0.43402, 0.43212, 0.43406, 0.44026, 0.44964, 0.46045, 0.47089,\
        0.47916, 0.48412, 0.48679, 0.48919, 0.49244, 0.49740, 0.50271, 0.50771, 0.51221, 0.51617, 0.52017,\
        0.52439, 0.52849, 0.53207, 0.53493, 0.53699, 0.53830, 0.53897, 0.53917, 0.53906, 0.53872, 0.53811,\
        0.53704, 0.53518, 0.53219, 0.52783, 0.52204, 0.51487, 0.50649, 0.49724, 0.48733, 0.47688, 0.46648,\
        0.45643, 0.44696, 0.43822, 0.43026, 0.42298, 0.41619, 0.40969, 0.40334, 0.39737, 0.39220, 0.38751,\
        0.38304, 0.37889, 0.37509, 0.37150, 0.36803, 0.36466, 0.36145, 0.35846, 0.35577, 0.35345, 0.35156,\
        0.35009, 0.34898, 0.34809, 0.34725, 0.34628, 0.34499, 0.34325, 0.34097, 0.33812, 0.33467, 0.33064],\
        [0.38192, 0.38766, 0.39341, 0.39915, 0.40492, 0.41071, 0.41650, 0.42228, 0.42802, 0.43371, 0.43932,\
        0.44488, 0.45036, 0.45578, 0.46112, 0.46636, 0.47147, 0.47638, 0.48102, 0.48533, 0.48933, 0.49311,\
        0.49674, 0.50011, 0.50298, 0.50513, 0.50672, 0.50770, 0.50787, 0.50696, 0.50480, 0.50210, 0.50016,\
        0.49773, 0.49352, 0.48907, 0.48470, 0.47997, 0.47489, 0.46962, 0.46430, 0.45897, 0.45350, 0.44780,\
        0.44186, 0.43576, 0.42973, 0.42414, 0.41958, 0.41705, 0.41859, 0.42506, 0.43464, 0.44538, 0.45565,\
        0.46416, 0.47036, 0.47483, 0.47875, 0.48312, 0.48827, 0.49369, 0.49884, 0.50334, 0.50680, 0.51082,\
        0.51554, 0.52012, 0.52405, 0.52712, 0.52929, 0.53059, 0.53116, 0.53117, 0.53082, 0.53028, 0.52955,\
        0.52846, 0.52664, 0.52367, 0.51928, 0.51348, 0.50643, 0.49817, 0.48896, 0.47909, 0.46878, 0.45845,\
        0.44844, 0.43903, 0.43040, 0.42261, 0.41556, 0.40900, 0.40265, 0.39635, 0.39032, 0.38492, 0.38003,\
        0.37550, 0.37127, 0.36734, 0.36364, 0.36012, 0.35676, 0.35360, 0.35068, 0.34807, 0.34587, 0.34410,\
        0.34278, 0.34185, 0.34116, 0.34052, 0.33971, 0.33856, 0.33690, 0.33467, 0.33182, 0.32836, 0.32430],\
        [0.38227, 0.38798, 0.39365, 0.39925, 0.40484, 0.41041, 0.41594, 0.42144, 0.42690, 0.43230, 0.43764,\
        0.44292, 0.44813, 0.45326, 0.45830, 0.46322, 0.46799, 0.47255, 0.47687, 0.48091, 0.48467, 0.48820,\
        0.49148, 0.49445, 0.49688, 0.49856, 0.49969, 0.50022, 0.49994, 0.49852, 0.49575, 0.49241, 0.48947,\
        0.48616, 0.48184, 0.47701, 0.47205, 0.46691, 0.46156, 0.45610, 0.45061, 0.44515, 0.43965, 0.43403,\
        0.42823, 0.42232, 0.41651, 0.41118, 0.40674, 0.40346, 0.40436, 0.41145, 0.42125, 0.43193, 0.44213,\
        0.45097, 0.45815, 0.46403, 0.46930, 0.47457, 0.48007, 0.48559, 0.49079, 0.49514, 0.49854, 0.50286,\
        0.50821, 0.51313, 0.51722, 0.52035, 0.52247, 0.52362, 0.52393, 0.52360, 0.52289, 0.52208, 0.52131,\
        0.52041, 0.51889, 0.51618, 0.51186, 0.50600, 0.49896, 0.49068, 0.48129, 0.47121, 0.46077, 0.45036,\
        0.44030, 0.43088, 0.42231, 0.41469, 0.40788, 0.40158, 0.39532, 0.38913, 0.38321, 0.37775, 0.37275,\
        0.36812, 0.36382, 0.35979, 0.35601, 0.35242, 0.34904, 0.34587, 0.34296, 0.34039, 0.33823, 0.33655,\
        0.33538, 0.33466, 0.33423, 0.33385, 0.33327, 0.33227, 0.33069, 0.32845, 0.32556, 0.32204, 0.31792],\
        [0.38267, 0.38836, 0.39398, 0.39953, 0.40486, 0.41014, 0.41538, 0.42058, 0.42573, 0.43082, 0.43585,\
        0.44081, 0.44569, 0.45048, 0.45517, 0.45973, 0.46413, 0.46833, 0.47230, 0.47600, 0.47944, 0.48261,\
        0.48547, 0.48793, 0.48984, 0.49110, 0.49173, 0.49171, 0.49083, 0.48888, 0.48586, 0.48225, 0.47848,\
        0.47439, 0.46973, 0.46464, 0.45934, 0.45390, 0.44835, 0.44274, 0.43717, 0.43171, 0.42636, 0.42097,\
        0.41545, 0.40990, 0.40447, 0.39957, 0.39574, 0.39316, 0.39443, 0.40095, 0.41041, 0.42073, 0.43073,\
        0.43973, 0.44756, 0.45443, 0.46072, 0.46676, 0.47269, 0.47841, 0.48369, 0.48831, 0.49252, 0.49734,\
        0.50273, 0.50758, 0.51161, 0.51466, 0.51662, 0.51750, 0.51744, 0.51666, 0.51543, 0.51423, 0.51346,\
        0.51294, 0.51199, 0.50989, 0.50599, 0.50001, 0.49276, 0.48399, 0.47412, 0.46362, 0.45289, 0.44229,\
        0.43211, 0.42260, 0.41400, 0.40652, 0.39990, 0.39373, 0.38768, 0.38172, 0.37599, 0.37060, 0.36558,\
        0.36090, 0.35654, 0.35245, 0.34859, 0.34495, 0.34152, 0.33832, 0.33538, 0.33278, 0.33061, 0.32896,\
        0.32792, 0.32744, 0.32732, 0.32727, 0.32697, 0.32614, 0.32462, 0.32233, 0.31932, 0.31568, 0.31146],\
        [0.38308, 0.38870, 0.39419, 0.39953, 0.40472, 0.40979, 0.41477, 0.41966, 0.42448, 0.42923, 0.43390,\
        0.43850, 0.44301, 0.44743, 0.45174, 0.45591, 0.45992, 0.46373, 0.46731, 0.47064, 0.47368, 0.47642,\
        0.47880, 0.48073, 0.48212, 0.48289, 0.48300, 0.48239, 0.48094, 0.47857, 0.47531, 0.47144, 0.46716,\
        0.46252, 0.45749, 0.45216, 0.44665, 0.44105, 0.43538, 0.42969, 0.42406, 0.41867, 0.41361, 0.40862,\
        0.40343, 0.39835, 0.39347, 0.38924, 0.38624, 0.38513, 0.38728, 0.39328, 0.40196, 0.41173, 0.42133,\
        0.43022, 0.43840, 0.44597, 0.45304, 0.45974, 0.46614, 0.47219, 0.47780, 0.48296, 0.48794, 0.49313,\
        0.49840, 0.50310, 0.50703, 0.50997, 0.51171, 0.51226, 0.51178, 0.51054, 0.50870, 0.50688, 0.50608,\
        0.50605, 0.50578, 0.50432, 0.50196, 0.49632, 0.48758, 0.47776, 0.46718, 0.45617, 0.44510, 0.43430,\
        0.42402, 0.41441, 0.40570, 0.39833, 0.39190, 0.38587, 0.38003, 0.37431, 0.36877, 0.36350, 0.35852,\
        0.35384, 0.34945, 0.34532, 0.34142, 0.33774, 0.33426, 0.33101, 0.32801, 0.32534, 0.32308, 0.32138,\
        0.32042, 0.32021, 0.32048, 0.32083, 0.32085, 0.32021, 0.31870, 0.31629, 0.31307, 0.30924, 0.30491],\
        [0.38344, 0.38899, 0.39436, 0.39952, 0.40452, 0.40936, 0.41405, 0.41864, 0.42311, 0.42749, 0.43178,\
        0.43598, 0.44010, 0.44411, 0.44802, 0.45178, 0.45538, 0.45878, 0.46194, 0.46485, 0.46746, 0.46973,\
        0.47161, 0.47302, 0.47389, 0.47414, 0.47372, 0.47257, 0.47064, 0.46789, 0.46439, 0.46026, 0.45565,\
        0.45064, 0.44529, 0.43974, 0.43411, 0.42846, 0.42280, 0.41712, 0.41148, 0.40610, 0.40137, 0.39684,\
        0.39204, 0.38742, 0.38324, 0.37990, 0.37793, 0.37802, 0.38088, 0.38675, 0.39498, 0.40439, 0.41337,\
        0.42196, 0.43037, 0.43849, 0.44621, 0.45351, 0.46038, 0.46684, 0.47287, 0.47854, 0.48401, 0.48941,\
        0.49461, 0.49929, 0.50321, 0.50611, 0.50766, 0.50780, 0.50690, 0.50540, 0.50317, 0.50056, 0.49966,\
        0.49976, 0.49982, 0.49838, 0.49583, 0.49049, 0.48155, 0.47118, 0.46011, 0.44870, 0.43733, 0.42640,\
        0.41617, 0.40669, 0.39804, 0.39068, 0.38428, 0.37833, 0.37261, 0.36707, 0.36169, 0.35652, 0.35161,\
        0.34695, 0.34256, 0.33842, 0.33451, 0.33080, 0.32729, 0.32399, 0.32094, 0.31819, 0.31579, 0.31392,\
        0.31295, 0.31305, 0.31377, 0.31455, 0.31492, 0.31447, 0.31293, 0.31030, 0.30676, 0.30265, 0.29819],\
        [0.38373, 0.38922, 0.39447, 0.39951, 0.40426, 0.40882, 0.41323, 0.41748, 0.42160, 0.42558, 0.42946,\
        0.43325, 0.43694, 0.44053, 0.44401, 0.44735, 0.45052, 0.45349, 0.45623, 0.45869, 0.46085, 0.46265,\
        0.46403, 0.46494, 0.46529, 0.46503, 0.46411, 0.46247, 0.46012, 0.45704, 0.45330, 0.44898, 0.44415,\
        0.43887, 0.43326, 0.42754, 0.42187, 0.41628, 0.41072, 0.40518, 0.39970, 0.39439, 0.38981, 0.38549,\
        0.38120, 0.37717, 0.37371, 0.37122, 0.37019, 0.37115, 0.37452, 0.38040, 0.38835, 0.39735, 0.40576,\
        0.41430, 0.42313, 0.43185, 0.44016, 0.44798, 0.45531, 0.46217, 0.46859, 0.47464, 0.48042, 0.48597,\
        0.49118, 0.49588, 0.49986, 0.50284, 0.50424, 0.50390, 0.50250, 0.50094, 0.49890, 0.49622, 0.49470,\
        0.49411, 0.49341, 0.49160, 0.48818, 0.48242, 0.47402, 0.46385, 0.45273, 0.44116, 0.42959, 0.41858,\
        0.40857, 0.39947, 0.39112, 0.38375, 0.37724, 0.37125, 0.36558, 0.36012, 0.35484, 0.34976, 0.34489,\
        0.34027, 0.33590, 0.33178, 0.32788, 0.32417, 0.32064, 0.31731, 0.31423, 0.31145, 0.30897, 0.30680,\
        0.30564, 0.30614, 0.30726, 0.30840, 0.30911, 0.30881, 0.30716, 0.30427, 0.30041, 0.29600, 0.29127],\
        [0.38395, 0.38935, 0.39445, 0.39929, 0.40383, 0.40816, 0.41229, 0.41620, 0.41993, 0.42349, 0.42694,\
        0.43028, 0.43353, 0.43669, 0.43973, 0.44264, 0.44538, 0.44791, 0.45021, 0.45222, 0.45392, 0.45526,\
        0.45617, 0.45659, 0.45646, 0.45573, 0.45433, 0.45226, 0.44953, 0.44617, 0.44224, 0.43780, 0.43286,\
        0.42742, 0.42157, 0.41569, 0.41006, 0.40463, 0.39925, 0.39390, 0.38865, 0.38365, 0.37911, 0.37494,\
        0.37104, 0.36754, 0.36472, 0.36295, 0.36264, 0.36418, 0.36785, 0.37365, 0.38116, 0.38955, 0.39805,\
        0.40702, 0.41661, 0.42597, 0.43480, 0.44307, 0.45078, 0.45797, 0.46469, 0.47101, 0.47698, 0.48261,\
        0.48784, 0.49254, 0.49656, 0.49966, 0.50099, 0.50020, 0.49847, 0.49664, 0.49457, 0.49215, 0.49016,\
        0.48862, 0.48691, 0.48426, 0.48011, 0.47400, 0.46585, 0.45606, 0.44520, 0.43373, 0.42208, 0.41088,\
        0.40117, 0.39250, 0.38452, 0.37726, 0.37070, 0.36465, 0.35896, 0.35352, 0.34829, 0.34325, 0.33841,\
        0.33380, 0.32945, 0.32538, 0.32153, 0.31785, 0.31433, 0.31100, 0.30791, 0.30517, 0.30279, 0.30059,\
        0.29904, 0.29983, 0.30098, 0.30224, 0.30315, 0.30295, 0.30109, 0.29789, 0.29384, 0.28921, 0.28419],\
        [0.38409, 0.38941, 0.39434, 0.39891, 0.40327, 0.40742, 0.41126, 0.41480, 0.41810, 0.42121, 0.42418,\
        0.42706, 0.42986, 0.43258, 0.43519, 0.43767, 0.43998, 0.44208, 0.44393, 0.44551, 0.44676, 0.44765,\
        0.44811, 0.44809, 0.44753, 0.44636, 0.44454, 0.44208, 0.43900, 0.43538, 0.43130, 0.42683, 0.42195,\
        0.41652, 0.41046, 0.40437, 0.39889, 0.39363, 0.38844, 0.38331, 0.37831, 0.37357, 0.36918, 0.36518,\
        0.36158, 0.35850, 0.35617, 0.35492, 0.35509, 0.35698, 0.36080, 0.36651, 0.37380, 0.38212, 0.39105,\
        0.40077, 0.41102, 0.42079, 0.42999, 0.43858, 0.44658, 0.45403, 0.46097, 0.46744, 0.47350, 0.47913,\
        0.48429, 0.48888, 0.49274, 0.49558, 0.49674, 0.49607, 0.49442, 0.49240, 0.49015, 0.48772, 0.48534,\
        0.48300, 0.48034, 0.47684, 0.47205, 0.46567, 0.45765, 0.44823, 0.43780, 0.42674, 0.41539, 0.40404,\
        0.39434, 0.38608, 0.37834, 0.37116, 0.36457, 0.35845, 0.35272, 0.34728, 0.34206, 0.33702, 0.33217,\
        0.32755, 0.32322, 0.31922, 0.31549, 0.31186, 0.30836, 0.30506, 0.30195, 0.29925, 0.29701, 0.29503,\
        0.29383, 0.29401, 0.29485, 0.29587, 0.29654, 0.29612, 0.29414, 0.29084, 0.28670, 0.28197, 0.27678],\
        [0.38416, 0.38947, 0.39429, 0.39866, 0.40284, 0.40673, 0.41021, 0.41330, 0.41610, 0.41871, 0.42119,\
        0.42359, 0.42594, 0.42821, 0.43040, 0.43246, 0.43435, 0.43603, 0.43745, 0.43859, 0.43942, 0.43988,\
        0.43994, 0.43953, 0.43858, 0.43704, 0.43486, 0.43204, 0.42864, 0.42477, 0.42057, 0.41612, 0.41144,\
        0.40635, 0.40030, 0.39411, 0.38862, 0.38343, 0.37837, 0.37343, 0.36865, 0.36413, 0.35993, 0.35612,\
        0.35276, 0.34998, 0.34799, 0.34706, 0.34748, 0.34953, 0.35339, 0.35907, 0.36639, 0.37504, 0.38475,\
        0.39533, 0.40591, 0.41596, 0.42544, 0.43430, 0.44254, 0.45016, 0.45723, 0.46377, 0.46982, 0.47536,\
        0.48036, 0.48472, 0.48825, 0.49069, 0.49170, 0.49128, 0.48984, 0.48783, 0.48547, 0.48287, 0.48011,\
        0.47713, 0.47370, 0.46948, 0.46416, 0.45755, 0.44963, 0.44057, 0.43066, 0.42020, 0.40962, 0.39938,\
        0.38930, 0.38056, 0.37268, 0.36544, 0.35876, 0.35258, 0.34680, 0.34134, 0.33612, 0.33108, 0.32620,\
        0.32152, 0.31714, 0.31323, 0.30975, 0.30620, 0.30264, 0.29943, 0.29637, 0.29368, 0.29145, 0.28967,\
        0.28857, 0.28832, 0.28865, 0.28911, 0.28919, 0.28836, 0.28629, 0.28306, 0.27896, 0.27420, 0.26893],\
        [0.38412, 0.38954, 0.39439, 0.39871, 0.40265, 0.40615, 0.40914, 0.41168, 0.41391, 0.41595, 0.41791,\
        0.41983, 0.42173, 0.42359, 0.42538, 0.42704, 0.42853, 0.42980, 0.43081, 0.43153, 0.43194, 0.43202,\
        0.43172, 0.43097, 0.42970, 0.42785, 0.42538, 0.42227, 0.41856, 0.41443, 0.41014, 0.40570, 0.40112,\
        0.39642, 0.39072, 0.38478, 0.37923, 0.37402, 0.36904, 0.36424, 0.35965, 0.35532, 0.35131, 0.34769,\
        0.34453, 0.34196, 0.34015, 0.33935, 0.33983, 0.34186, 0.34566, 0.35133, 0.35878, 0.36780, 0.37811,\
        0.38923, 0.40042, 0.41098, 0.42083, 0.42999, 0.43843, 0.44619, 0.45331, 0.45985, 0.46583, 0.47123,\
        0.47600, 0.48006, 0.48325, 0.48537, 0.48628, 0.48598, 0.48473, 0.48279, 0.48037, 0.47759, 0.47449,\
        0.47100, 0.46698, 0.46220, 0.45647, 0.44968, 0.44183, 0.43308, 0.42369, 0.41391, 0.40375, 0.39390,\
        0.38435, 0.37543, 0.36734, 0.35995, 0.35318, 0.34693, 0.34111, 0.33565, 0.33044, 0.32542, 0.32052,\
        0.31573, 0.31119, 0.30726, 0.30420, 0.30103, 0.29722, 0.29395, 0.29094, 0.28826, 0.28601, 0.28424,\
        0.28304, 0.28242, 0.28219, 0.28202, 0.28149, 0.28022, 0.27797, 0.27475, 0.27069, 0.26594, 0.26064],\
        [0.38387, 0.38950, 0.39450, 0.39883, 0.40256, 0.40560, 0.40796, 0.40984, 0.41144, 0.41290, 0.41433,\
        0.41578, 0.41726, 0.41873, 0.42013, 0.42143, 0.42255, 0.42344, 0.42404, 0.42436, 0.42438, 0.42410,\
        0.42349, 0.42247, 0.42093, 0.41884, 0.41617, 0.41287, 0.40889, 0.40450, 0.40017, 0.39578, 0.39127,\
        0.38653, 0.38128, 0.37576, 0.37037, 0.36523, 0.36034, 0.35568, 0.35125, 0.34711, 0.34329, 0.33985,\
        0.33686, 0.33441, 0.33267, 0.33184, 0.33218, 0.33402, 0.33764, 0.34322, 0.35078, 0.36013, 0.37090,\
        0.38253, 0.39433, 0.40556, 0.41593, 0.42545, 0.43410, 0.44195, 0.44909, 0.45559, 0.46145, 0.46668,\
        0.47121, 0.47498, 0.47786, 0.47974, 0.48055, 0.48030, 0.47915, 0.47728, 0.47483, 0.47190, 0.46851,\
        0.46464, 0.46017, 0.45498, 0.44894, 0.44202, 0.43422, 0.42568, 0.41667, 0.40768, 0.39830, 0.38846,\
        0.37904, 0.37018, 0.36201, 0.35452, 0.34766, 0.34136, 0.33553, 0.33009, 0.32494, 0.32000, 0.31513,\
        0.31030, 0.30554, 0.30123, 0.29837, 0.29551, 0.29194, 0.28855, 0.28549, 0.28279, 0.28048, 0.27862,\
        0.27720, 0.27619, 0.27543, 0.27466, 0.27356, 0.27186, 0.26939, 0.26608, 0.26202, 0.25728, 0.25196],\
        [0.38322, 0.38913, 0.39440, 0.39889, 0.40242, 0.40489, 0.40651, 0.40766, 0.40861, 0.40949, 0.41040,\
        0.41142, 0.41253, 0.41363, 0.41468, 0.41564, 0.41644, 0.41698, 0.41720, 0.41711, 0.41675, 0.41615,\
        0.41528, 0.41405, 0.41231, 0.41001, 0.40721, 0.40386, 0.39977, 0.39525, 0.39085, 0.38645, 0.38194,\
        0.37723, 0.37223, 0.36705, 0.36190, 0.35693, 0.35217, 0.34766, 0.34341, 0.33945, 0.33582, 0.33256,\
        0.32972, 0.32737, 0.32561, 0.32461, 0.32466, 0.32611, 0.32936, 0.33475, 0.34236, 0.35199, 0.36319,\
        0.37533, 0.38771, 0.39964, 0.41068, 0.42059, 0.42941, 0.43732, 0.44446, 0.45089, 0.45664, 0.46169,\
        0.46599, 0.46949, 0.47212, 0.47381, 0.47452, 0.47428, 0.47318, 0.47133, 0.46886, 0.46581, 0.46222,\
        0.45806, 0.45327, 0.44778, 0.44152, 0.43450, 0.42676, 0.41838, 0.40944, 0.40047, 0.39192, 0.38259,\
        0.37338, 0.36463, 0.35647, 0.34896, 0.34208, 0.33576, 0.32995, 0.32455, 0.31950, 0.31470, 0.30998,\
        0.30521, 0.30049, 0.29608, 0.29267, 0.28958, 0.28627, 0.28297, 0.27990, 0.27715, 0.27475, 0.27272,\
        0.27104, 0.26964, 0.26838, 0.26707, 0.26546, 0.26336, 0.26062, 0.25718, 0.25305, 0.24828, 0.24294],\
        [0.38194, 0.38813, 0.39370, 0.39841, 0.40187, 0.40371, 0.40449, 0.40495, 0.40532, 0.40568, 0.40611,\
        0.40676, 0.40757, 0.40834, 0.40905, 0.40971, 0.41024, 0.41048, 0.41032, 0.40982, 0.40911, 0.40821,\
        0.40711, 0.40573, 0.40385, 0.40131, 0.39840, 0.39500, 0.39097, 0.38656, 0.38211, 0.37765, 0.37312,\
        0.36846, 0.36363, 0.35871, 0.35380, 0.34903, 0.34445, 0.34011, 0.33604, 0.33228, 0.32885, 0.32579,\
        0.32311, 0.32085, 0.31905, 0.31782, 0.31742, 0.31827, 0.32096, 0.32600, 0.33361, 0.34353, 0.35516,\
        0.36782, 0.38078, 0.39341, 0.40509, 0.41531, 0.42422, 0.43216, 0.43932, 0.44572, 0.45137, 0.45626,\
        0.46035, 0.46363, 0.46604, 0.46756, 0.46818, 0.46791, 0.46683, 0.46500, 0.46250, 0.45938, 0.45565,\
        0.45130, 0.44628, 0.44057, 0.43416, 0.42707, 0.41940, 0.41124, 0.40267, 0.39365, 0.38497, 0.37613,\
        0.36727, 0.35871, 0.35064, 0.34317, 0.33630, 0.33002, 0.32425, 0.31891, 0.31395, 0.30931, 0.30481,\
        0.30016, 0.29542, 0.29103, 0.28721, 0.28377, 0.28042, 0.27716, 0.27408, 0.27127, 0.26874, 0.26651,\
        0.26454, 0.26276, 0.26105, 0.25926, 0.25720, 0.25472, 0.25170, 0.24808, 0.24384, 0.23902, 0.23364],\
        [0.37978, 0.38616, 0.39193, 0.39689, 0.40051, 0.40156, 0.40159, 0.40158, 0.40156, 0.40153, 0.40151,\
        0.40185, 0.40247, 0.40289, 0.40325, 0.40362, 0.40394, 0.40396, 0.40341, 0.40253, 0.40150, 0.40033,\
        0.39902, 0.39747, 0.39546, 0.39282, 0.38977, 0.38629, 0.38236, 0.37810, 0.37370, 0.36925, 0.36474,\
        0.36014, 0.35546, 0.35074, 0.34606, 0.34150, 0.33711, 0.33296, 0.32909, 0.32553, 0.32232, 0.31947,\
        0.31700, 0.31487, 0.31307, 0.31162, 0.31069, 0.31077, 0.31265, 0.31718, 0.32479, 0.33505, 0.34713,\
        0.36022, 0.37363, 0.38676, 0.39897, 0.40939, 0.41834, 0.42638, 0.43362, 0.44005, 0.44563, 0.45038,\
        0.45429, 0.45738, 0.45961, 0.46099, 0.46152, 0.46121, 0.46013, 0.45830, 0.45579, 0.45263, 0.44882,\
        0.44435, 0.43919, 0.43333, 0.42679, 0.41966, 0.41201, 0.40398, 0.39568, 0.38697, 0.37820, 0.36947,\
        0.36083, 0.35244, 0.34449, 0.33708, 0.33027, 0.32406, 0.31835, 0.31305, 0.30811, 0.30359, 0.29922,\
        0.29467, 0.29002, 0.28562, 0.28161, 0.27793, 0.27446, 0.27115, 0.26802, 0.26510, 0.26243, 0.25998,\
        0.25772, 0.25558, 0.25347, 0.25125, 0.24878, 0.24595, 0.24265, 0.23882, 0.23445, 0.22953, 0.22408],\
        [0.37657, 0.38296, 0.38871, 0.39355, 0.39687, 0.39788, 0.39780, 0.39760, 0.39735, 0.39707, 0.39682,\
        0.39689, 0.39718, 0.39729, 0.39736, 0.39745, 0.39750, 0.39727, 0.39645, 0.39532, 0.39405, 0.39265,\
        0.39110, 0.38930, 0.38711, 0.38442, 0.38132, 0.37783, 0.37397, 0.36984, 0.36554, 0.36115, 0.35671,\
        0.35221, 0.34767, 0.34313, 0.33865, 0.33429, 0.33011, 0.32616, 0.32249, 0.31914, 0.31615, 0.31354,\
        0.31131, 0.30939, 0.30771, 0.30618, 0.30473, 0.30399, 0.30486, 0.30871, 0.31642, 0.32709, 0.33950,\
        0.35280, 0.36635, 0.37956, 0.39184, 0.40248, 0.41162, 0.41996, 0.42742, 0.43391, 0.43944, 0.44406,\
        0.44781, 0.45072, 0.45280, 0.45406, 0.45450, 0.45416, 0.45307, 0.45125, 0.44875, 0.44558, 0.44174,\
        0.43722, 0.43199, 0.42603, 0.41940, 0.41222, 0.40460, 0.39662, 0.38841, 0.37993, 0.37130, 0.36268,\
        0.35417, 0.34590, 0.33802, 0.33067, 0.32393, 0.31782, 0.31224, 0.30701, 0.30195, 0.29742, 0.29307,\
        0.28862, 0.28413, 0.27980, 0.27572, 0.27191, 0.26832, 0.26492, 0.26169, 0.25865, 0.25581, 0.25313,\
        0.25059, 0.24812, 0.24564, 0.24303, 0.24019, 0.23703, 0.23346, 0.22942, 0.22488, 0.21984, 0.21431],\
        [0.37227, 0.37852, 0.38402, 0.38848, 0.39149, 0.39285, 0.39313, 0.39297, 0.39264, 0.39226, 0.39191,\
        0.39173, 0.39166, 0.39154, 0.39138, 0.39119, 0.39090, 0.39034, 0.38938, 0.38812, 0.38666, 0.38506,\
        0.38328, 0.38126, 0.37891, 0.37616, 0.37304, 0.36957, 0.36579, 0.36177, 0.35759, 0.35330, 0.34896,\
        0.34458, 0.34019, 0.33583, 0.33154, 0.32739, 0.32341, 0.31966, 0.31619, 0.31303, 0.31025, 0.30789,\
        0.30593, 0.30426, 0.30276, 0.30145, 0.29966, 0.29842, 0.29850, 0.30148, 0.30949, 0.32037, 0.33278,\
        0.34592, 0.35922, 0.37215, 0.38416, 0.39481, 0.40434, 0.41313, 0.42085, 0.42739, 0.43283, 0.43731,\
        0.44090, 0.44366, 0.44560, 0.44675, 0.44713, 0.44675, 0.44565, 0.44384, 0.44136, 0.43821, 0.43440,\
        0.42990, 0.42468, 0.41869, 0.41197, 0.40474, 0.39717, 0.38927, 0.38112, 0.37275, 0.36426, 0.35574,\
        0.34733, 0.33914, 0.33130, 0.32397, 0.31727, 0.31126, 0.30584, 0.30081, 0.29578, 0.29108, 0.28664,\
        0.28224, 0.27787, 0.27360, 0.26951, 0.26563, 0.26194, 0.25844, 0.25510, 0.25192, 0.24889, 0.24599,\
        0.24317, 0.24039, 0.23756, 0.23461, 0.23143, 0.22796, 0.22412, 0.21987, 0.21516, 0.20998, 0.20434],\
        [0.36694, 0.37297, 0.37818, 0.38235, 0.38525, 0.38688, 0.38751, 0.38757, 0.38733, 0.38695, 0.38655,\
        0.38620, 0.38589, 0.38556, 0.38518, 0.38473, 0.38415, 0.38334, 0.38223, 0.38085, 0.37926, 0.37749,\
        0.37553, 0.37334, 0.37086, 0.36805, 0.36492, 0.36149, 0.35779, 0.35388, 0.34982, 0.34566, 0.34144,\
        0.33720, 0.33297, 0.32879, 0.32470, 0.32074, 0.31696, 0.31342, 0.31014, 0.30718, 0.30458, 0.30244,\
        0.30070, 0.29927, 0.29792, 0.29668, 0.29527, 0.29422, 0.29462, 0.29781, 0.30517, 0.31553, 0.32731,\
        0.33984, 0.35257, 0.36501, 0.37671, 0.38742, 0.39723, 0.40627, 0.41409, 0.42051, 0.42579, 0.43010,\
        0.43354, 0.43616, 0.43799, 0.43905, 0.43937, 0.43897, 0.43786, 0.43607, 0.43361, 0.43051, 0.42676,\
        0.42234, 0.41721, 0.41130, 0.40451, 0.39722, 0.38969, 0.38186, 0.37377, 0.36549, 0.35710, 0.34869,\
        0.34037, 0.33223, 0.32441, 0.31705, 0.31033, 0.30441, 0.29909, 0.29411, 0.28925, 0.28455, 0.28005,\
        0.27565, 0.27132, 0.26709, 0.26300, 0.25907, 0.25530, 0.25169, 0.24822, 0.24489, 0.24168, 0.23855,\
        0.23547, 0.23240, 0.22926, 0.22598, 0.22250, 0.21874, 0.21464, 0.21017, 0.20528, 0.19995, 0.19419],\
        [0.36073, 0.36652, 0.37148, 0.37547, 0.37836, 0.38018, 0.38112, 0.38144, 0.38137, 0.38108, 0.38068,\
        0.38025, 0.37979, 0.37929, 0.37873, 0.37806, 0.37725, 0.37624, 0.37499, 0.37350, 0.37180, 0.36991,\
        0.36781, 0.36550, 0.36292, 0.36007, 0.35695, 0.35356, 0.34995, 0.34615, 0.34222, 0.33819, 0.33412,\
        0.33003, 0.32597, 0.32197, 0.31807, 0.31431, 0.31074, 0.30740, 0.30432, 0.30156, 0.29913, 0.29717,\
        0.29560, 0.29430, 0.29316, 0.29215, 0.29135, 0.29123, 0.29268, 0.29671, 0.30344, 0.31236, 0.32298,\
        0.33455, 0.34648, 0.35831, 0.36966, 0.38033, 0.39028, 0.39947, 0.40702, 0.41316, 0.41824, 0.42239,\
        0.42570, 0.42821, 0.42995, 0.43095, 0.43123, 0.43080, 0.42969, 0.42792, 0.42550, 0.42245, 0.41876,\
        0.41444, 0.40946, 0.40375, 0.39705, 0.38973, 0.38216, 0.37436, 0.36633, 0.35814, 0.34986, 0.34156,\
        0.33333, 0.32527, 0.31748, 0.31008, 0.30326, 0.29740, 0.29216, 0.28722, 0.28243, 0.27778, 0.27327,\
        0.26886, 0.26455, 0.26033, 0.25622, 0.25224, 0.24839, 0.24467, 0.24108, 0.23758, 0.23418, 0.23083,\
        0.22750, 0.22415, 0.22072, 0.21715, 0.21338, 0.20935, 0.20501, 0.20031, 0.19524, 0.18976, 0.18387]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=30 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax30vt7.5
    zgrid_H_Hmax_vmax30vt7_5=[[0.241675, 0.249325, 0.256469, 0.263126, 0.269320, 0.275076, 0.280421, 0.285382, 0.289984, 0.294251,\
        0.298200, 0.301850, 0.305213, 0.308299, 0.311115, 0.313668, 0.315961, 0.317998, 0.319782, 0.321317,\
        0.322609, 0.323667, 0.324499, 0.325121, 0.325549, 0.325804, 0.325909, 0.325891, 0.325779, 0.325605,\
        0.325401, 0.325200, 0.325034, 0.324934, 0.324930, 0.325051, 0.325322, 0.325768, 0.326410, 0.327271,\
        0.328368, 0.329720, 0.331342, 0.333250, 0.335454, 0.337966, 0.340791, 0.343935, 0.347394, 0.351161,\
        0.355219, 0.359543, 0.364097, 0.368838, 0.373726, 0.378742, 0.383913, 0.389288, 0.394802, 0.400125,\
        0.405162, 0.409987, 0.414592, 0.418958, 0.423068, 0.426917, 0.430513, 0.433875, 0.437030, 0.440010,\
        0.442845, 0.445560, 0.448175, 0.450704, 0.453157, 0.455536, 0.457834, 0.460026, 0.462069, 0.463912,\
        0.465505, 0.466819, 0.467865, 0.468684, 0.469314, 0.469756, 0.469958, 0.469834, 0.469303, 0.468299,\
        0.466765, 0.464687, 0.462112, 0.459120, 0.455788, 0.452168, 0.448294, 0.444184, 0.439848, 0.435290,\
        0.430513, 0.425512, 0.420282, 0.414812, 0.409091, 0.403103, 0.396835, 0.390272, 0.383402, 0.376212,\
        0.368693, 0.360838, 0.352642, 0.344102, 0.335216, 0.325984, 0.316406, 0.306484, 0.296221, 0.285618,\
        0.274678],\
        [0.252239, 0.259792, 0.266819, 0.273342, 0.279387, 0.284985, 0.290167, 0.294963, 0.299401, 0.303508,\
        0.307303, 0.310803, 0.314021, 0.316966, 0.319643, 0.322058, 0.324211, 0.326105, 0.327739, 0.329117,\
        0.330242, 0.331121, 0.331764, 0.332184, 0.332398, 0.332428, 0.332298, 0.332038, 0.331679, 0.331256,\
        0.330804, 0.330359, 0.329956, 0.329627, 0.329406, 0.329321, 0.329399, 0.329665, 0.330143, 0.330854,\
        0.331818, 0.333052, 0.334576, 0.336403, 0.338548, 0.341025, 0.343841, 0.347004, 0.350514, 0.354367,\
        0.358548, 0.363031, 0.367773, 0.372718, 0.377797, 0.382953, 0.388218, 0.393740, 0.399590, 0.405291,\
        0.410686, 0.415842, 0.420736, 0.425345, 0.429648, 0.433642, 0.437338, 0.440764, 0.443957, 0.446960,\
        0.449814, 0.452549, 0.455187, 0.457745, 0.460234, 0.462668, 0.465045, 0.467340, 0.469499, 0.471449,\
        0.473113, 0.474441, 0.475455, 0.476243, 0.476890, 0.477420, 0.477763, 0.477777, 0.477365, 0.476464,\
        0.474971, 0.472846, 0.470175, 0.467095, 0.463703, 0.460057, 0.456181, 0.452088, 0.447783, 0.443269,\
        0.438547, 0.433613, 0.428458, 0.423071, 0.417437, 0.411538, 0.405358, 0.398879, 0.392087, 0.384968,\
        0.377514, 0.369716, 0.361570, 0.353074, 0.344226, 0.335027, 0.325478, 0.315584, 0.305345, 0.294765,\
        0.283848],\
        [0.262609, 0.270067, 0.276976, 0.283361, 0.289254, 0.294688, 0.299701, 0.304327, 0.308601, 0.312547,\
        0.316190, 0.319544, 0.322623, 0.325433, 0.327980, 0.330264, 0.332285, 0.334042, 0.335533, 0.336758,\
        0.337719, 0.338422, 0.338875, 0.339092, 0.339088, 0.338888, 0.338516, 0.338006, 0.337391, 0.336710,\
        0.336001, 0.335304, 0.334656, 0.334093, 0.333649, 0.333353, 0.333234, 0.333317, 0.333627, 0.334184,\
        0.335009, 0.336122, 0.337540, 0.339280, 0.341359, 0.343791, 0.346589, 0.349763, 0.353319, 0.357256,\
        0.361564, 0.366219, 0.371181, 0.376384, 0.381729, 0.387085, 0.392398, 0.397977, 0.404046, 0.410192,\
        0.416056, 0.421611, 0.426842, 0.431726, 0.436243, 0.440389, 0.444181, 0.447657, 0.450871, 0.453881,\
        0.456739, 0.459484, 0.462136, 0.464706, 0.467210, 0.469673, 0.472117, 0.474516, 0.476797, 0.478870,\
        0.480619, 0.481933, 0.482843, 0.483526, 0.484133, 0.484735, 0.485254, 0.485396, 0.485073, 0.484322,\
        0.482894, 0.480668, 0.477853, 0.474670, 0.471232, 0.467579, 0.463720, 0.459657, 0.455393, 0.450930,\
        0.446270, 0.441409, 0.436338, 0.431042, 0.425502, 0.419699, 0.413612, 0.407221, 0.400509, 0.393463,\
        0.386072, 0.378329, 0.370229, 0.361772, 0.352957, 0.343787, 0.334263, 0.324391, 0.314172, 0.303612,\
        0.292714],\
        [0.272789, 0.280156, 0.286947, 0.293190, 0.298922, 0.304185, 0.309023, 0.313476, 0.317581, 0.321370,\
        0.324863, 0.328078, 0.331025, 0.333708, 0.336131, 0.338293, 0.340190, 0.341818, 0.343173, 0.344251,\
        0.345052, 0.345580, 0.345844, 0.345855, 0.345630, 0.345194, 0.344574, 0.343804, 0.342924, 0.341974,\
        0.341000, 0.340042, 0.339143, 0.338340, 0.337669, 0.337159, 0.336840, 0.336738, 0.336877, 0.337277,\
        0.337960, 0.338946, 0.340253, 0.341899, 0.343903, 0.346281, 0.349050, 0.352223, 0.355813, 0.359824,\
        0.364255, 0.369088, 0.374290, 0.379804, 0.385533, 0.391295, 0.396815, 0.402451, 0.408787, 0.415216,\
        0.421463, 0.427407, 0.432991, 0.438170, 0.442914, 0.447215, 0.451094, 0.454603, 0.457815, 0.460811,\
        0.463659, 0.466406, 0.469069, 0.471643, 0.474128, 0.476579, 0.479066, 0.481571, 0.483973, 0.486176,\
        0.488059, 0.489345, 0.490051, 0.490560, 0.491058, 0.491644, 0.492440, 0.492657, 0.492280, 0.491819,\
        0.490508, 0.488066, 0.485067, 0.481808, 0.478365, 0.474742, 0.470925, 0.466907, 0.462691, 0.458284,\
        0.453691, 0.448910, 0.443930, 0.438732, 0.433296, 0.427595, 0.421607, 0.415307, 0.408678, 0.401703,\
        0.394373, 0.386680, 0.378621, 0.370198, 0.361410, 0.352263, 0.342759, 0.332904, 0.322701, 0.312157,\
        0.301274],\
        [0.282784, 0.290064, 0.296735, 0.302832, 0.308396, 0.313479, 0.318132, 0.322406, 0.326344, 0.329976,\
        0.333327, 0.336410, 0.339232, 0.341798, 0.344107, 0.346155, 0.347937, 0.349445, 0.350670, 0.351605,\
        0.352249, 0.352604, 0.352678, 0.352482, 0.352033, 0.351356, 0.350478, 0.349440, 0.348283, 0.347056,\
        0.345806, 0.344581, 0.343425, 0.342379, 0.341477, 0.340753, 0.340234, 0.339945, 0.339911, 0.340153,\
        0.340692, 0.341546, 0.342737, 0.344282, 0.346202, 0.348515, 0.351241, 0.354399, 0.358005, 0.362073,\
        0.366610, 0.371608, 0.377042, 0.382868, 0.389025, 0.395386, 0.401515, 0.407410, 0.413801, 0.420434,\
        0.427001, 0.433312, 0.439251, 0.444740, 0.449724, 0.454184, 0.458137, 0.461651, 0.464832, 0.467789,\
        0.470612, 0.473361, 0.476047, 0.478632, 0.481063, 0.483418, 0.485910, 0.488548, 0.491048, 0.493313,\
        0.495405, 0.496711, 0.497101, 0.497457, 0.497831, 0.498141, 0.499238, 0.499440, 0.498822, 0.498847,\
        0.497636, 0.494844, 0.491752, 0.488511, 0.485131, 0.481583, 0.477832, 0.473865, 0.469696, 0.465345,\
        0.460822, 0.456127, 0.451245, 0.446155, 0.440829, 0.435238, 0.429353, 0.423148, 0.416601, 0.409695,\
        0.402420, 0.394772, 0.386748, 0.378351, 0.369584, 0.360453, 0.350962, 0.341119, 0.330928, 0.320394,\
        0.309524],\
        [0.292598, 0.299797, 0.306350, 0.312292, 0.317677, 0.322567, 0.327027, 0.331117, 0.334886, 0.338368,\
        0.341585, 0.344546, 0.347255, 0.349713, 0.351917, 0.353861, 0.355537, 0.356933, 0.358035, 0.358833,\
        0.359321, 0.359503, 0.359386, 0.358981, 0.358305, 0.357380, 0.356237, 0.354919, 0.353475, 0.351959,\
        0.350425, 0.348926, 0.347510, 0.346218, 0.345087, 0.344149, 0.343431, 0.342957, 0.342752, 0.342835,\
        0.343227, 0.343948, 0.345018, 0.346456, 0.348283, 0.350520, 0.353189, 0.356313, 0.359914, 0.364013,\
        0.368628, 0.373763, 0.379400, 0.385480, 0.391929, 0.398700, 0.405390, 0.411954, 0.418742, 0.425711,\
        0.432645, 0.439345, 0.445662, 0.451483, 0.456728, 0.461354, 0.465367, 0.468854, 0.471966, 0.474860,\
        0.477644, 0.480391, 0.483127, 0.485778, 0.488152, 0.490253, 0.492665, 0.495521, 0.498115, 0.500276,\
        0.502414, 0.503766, 0.504164, 0.504460, 0.504726, 0.504951, 0.505572, 0.505806, 0.505429, 0.505112,\
        0.503704, 0.501022, 0.498025, 0.494886, 0.491617, 0.488173, 0.484492, 0.480566, 0.476433, 0.472131,\
        0.467679, 0.463073, 0.458297, 0.453323, 0.448116, 0.442642, 0.436864, 0.430754, 0.424287, 0.417445,\
        0.410220, 0.402607, 0.394609, 0.386230, 0.377475, 0.368353, 0.358869, 0.349032, 0.338847, 0.328321,\
        0.317460],\
        [0.302236, 0.309364, 0.315797, 0.321578, 0.326770, 0.331452, 0.335705, 0.339605, 0.343208, 0.346550,\
        0.349645, 0.352499, 0.355108, 0.357468, 0.359576, 0.361425, 0.363003, 0.364295, 0.365281, 0.365944,\
        0.366276, 0.366282, 0.365972, 0.365357, 0.364450, 0.363272, 0.361854, 0.360245, 0.358501, 0.356685,\
        0.354860, 0.353083, 0.351405, 0.349870, 0.348512, 0.347364, 0.346451, 0.345796, 0.345422, 0.345348,\
        0.345594, 0.346180, 0.347125, 0.348451, 0.350179, 0.352330, 0.354929, 0.358001, 0.361572, 0.365671,\
        0.370328, 0.375564, 0.381378, 0.387699, 0.394354, 0.401318, 0.408644, 0.416000, 0.423439, 0.430937,\
        0.438347, 0.445496, 0.452231, 0.458420, 0.463962, 0.468779, 0.472845, 0.476262, 0.479264, 0.482082,\
        0.484821, 0.487546, 0.490345, 0.493137, 0.495632, 0.497214, 0.499525, 0.502467, 0.505194, 0.507398,\
        0.509308, 0.510626, 0.511284, 0.511627, 0.511845, 0.512033, 0.512257, 0.512258, 0.511870, 0.511082,\
        0.509507, 0.507083, 0.504208, 0.501138, 0.497950, 0.494599, 0.490961, 0.487040, 0.482921, 0.478661,\
        0.474280, 0.469769, 0.465105, 0.460253, 0.455173, 0.449820, 0.444153, 0.438137, 0.431745, 0.424959,\
        0.417773, 0.410186, 0.402203, 0.393831, 0.385079, 0.375957, 0.366474, 0.356637, 0.346454, 0.335932,\
        0.325076],\
        [0.311701, 0.318772, 0.325088, 0.330697, 0.335676, 0.340129, 0.344161, 0.347865, 0.351309, 0.354526,\
        0.357522, 0.360287, 0.362810, 0.365084, 0.367103, 0.368863, 0.370350, 0.371546, 0.372422, 0.372949,\
        0.373117, 0.372941, 0.372435, 0.371611, 0.370472, 0.369034, 0.367331, 0.365418, 0.363361, 0.361234,\
        0.359111, 0.357055, 0.355119, 0.353346, 0.351769, 0.350418, 0.349316, 0.348486, 0.347948, 0.347720,\
        0.347821, 0.348272, 0.349092, 0.350303, 0.351927, 0.353986, 0.356505, 0.359510, 0.363030, 0.367099,\
        0.371753, 0.377036, 0.382990, 0.389625, 0.396728, 0.404023, 0.411779, 0.419815, 0.427973, 0.436115,\
        0.444092, 0.451751, 0.458947, 0.465544, 0.471426, 0.476492, 0.480633, 0.483921, 0.486761, 0.489533,\
        0.492275, 0.494925, 0.497769, 0.500640, 0.503240, 0.504939, 0.507117, 0.509813, 0.512416, 0.514627,\
        0.516417, 0.517701, 0.518475, 0.518887, 0.519086, 0.519159, 0.519113, 0.518841, 0.518218, 0.517133,\
        0.515439, 0.513130, 0.510386, 0.507387, 0.504229, 0.500904, 0.497254, 0.493298, 0.489178, 0.484960,\
        0.480651, 0.476237, 0.471690, 0.466967, 0.462017, 0.456789, 0.451233, 0.445308, 0.438983, 0.432242,\
        0.425081, 0.417505, 0.409523, 0.401147, 0.392388, 0.383259, 0.373769, 0.363927, 0.353742, 0.343219,\
        0.332366],\
        [0.320994, 0.328030, 0.334235, 0.339659, 0.344401, 0.348596, 0.352385, 0.355890, 0.359188, 0.362307,\
        0.365236, 0.367940, 0.370394, 0.372589, 0.374524, 0.376196, 0.377595, 0.378701, 0.379472, 0.379859,\
        0.379847, 0.379471, 0.378767, 0.377737, 0.376367, 0.374664, 0.372665, 0.370432, 0.368047, 0.365600,\
        0.363177, 0.360847, 0.358661, 0.356661, 0.354878, 0.353335, 0.352055, 0.351057, 0.350360, 0.349982,\
        0.349942, 0.350259, 0.350954, 0.352048, 0.353566, 0.355530, 0.357966, 0.360899, 0.364358, 0.368375,\
        0.372990, 0.378251, 0.384219, 0.390990, 0.398643, 0.406439, 0.414762, 0.423551, 0.432477, 0.441318,\
        0.449906, 0.458104, 0.465785, 0.472809, 0.479050, 0.484453, 0.488787, 0.491870, 0.494513, 0.497302,\
        0.500200, 0.502833, 0.505643, 0.508468, 0.510985, 0.513113, 0.515294, 0.517695, 0.520052, 0.522126,\
        0.523806, 0.525038, 0.525828, 0.526250, 0.526396, 0.526328, 0.526050, 0.525509, 0.524625, 0.523310,\
        0.521502, 0.519210, 0.516528, 0.513569, 0.510399, 0.507010, 0.503327, 0.499353, 0.495244, 0.491072,\
        0.486830, 0.482508, 0.478079, 0.473486, 0.468667, 0.463564, 0.458118, 0.452277, 0.446006, 0.439293,\
        0.432139, 0.424557, 0.416561, 0.408167, 0.399392, 0.390247, 0.380744, 0.370892, 0.360701, 0.350177,\
        0.339325],\
        [0.330110, 0.337144, 0.343252, 0.348479, 0.352949, 0.356846, 0.360367, 0.363669, 0.366844, 0.369910,\
        0.372821, 0.375501, 0.377901, 0.380023, 0.381876, 0.383455, 0.384755, 0.385772, 0.386449, 0.386687,\
        0.386455, 0.385851, 0.384950, 0.383726, 0.382128, 0.380153, 0.377844, 0.375273, 0.372545, 0.369772,\
        0.367054, 0.364462, 0.362046, 0.359838, 0.357864, 0.356145, 0.354698, 0.353542, 0.352694, 0.352171,\
        0.351991, 0.352175, 0.352745, 0.353723, 0.355136, 0.357008, 0.359366, 0.362234, 0.365639, 0.369607,\
        0.374177, 0.379392, 0.385294, 0.391880, 0.399278, 0.408154, 0.417641, 0.427363, 0.437102, 0.446650,\
        0.455837, 0.464547, 0.472700, 0.480163, 0.486688, 0.492365, 0.497279, 0.500122, 0.502781, 0.505618,\
        0.508565, 0.511346, 0.514095, 0.516785, 0.519269, 0.521533, 0.523730, 0.525942, 0.528066, 0.529954,\
        0.531497, 0.532639, 0.533376, 0.533741, 0.533788, 0.533556, 0.533054, 0.532259, 0.531125, 0.529603,\
        0.527666, 0.525327, 0.522639, 0.519667, 0.516453, 0.512996, 0.509276, 0.505316, 0.501217, 0.497072,\
        0.492868, 0.488618, 0.484307, 0.479836, 0.475140, 0.470160, 0.464820, 0.459052, 0.452816, 0.446107,\
        0.438937, 0.431327, 0.423301, 0.414878, 0.406076, 0.396908, 0.387388, 0.377523, 0.367324, 0.356796,\
        0.345945],\
        [0.339033, 0.346119, 0.352159, 0.357177, 0.361326, 0.364869, 0.368091, 0.371197, 0.374283, 0.377357,\
        0.380328, 0.383035, 0.385391, 0.387436, 0.389211, 0.390688, 0.391861, 0.392768, 0.393363, 0.393447,\
        0.392916, 0.392041, 0.390962, 0.389564, 0.387732, 0.385474, 0.382844, 0.379917, 0.376829, 0.373731,\
        0.370736, 0.367912, 0.365294, 0.362905, 0.360763, 0.358884, 0.357284, 0.355979, 0.354986, 0.354322,\
        0.354006, 0.354057, 0.354501, 0.355363, 0.356672, 0.358459, 0.360751, 0.363575, 0.366953, 0.370908,\
        0.375475, 0.380710, 0.386702, 0.393544, 0.401364, 0.410707, 0.420975, 0.431511, 0.442009, 0.452223,\
        0.461950, 0.471071, 0.479601, 0.487550, 0.494387, 0.499811, 0.505017, 0.508633, 0.511586, 0.514517,\
        0.517435, 0.520247, 0.522952, 0.525539, 0.527964, 0.530226, 0.532379, 0.534448, 0.536381, 0.538086,\
        0.539475, 0.540494, 0.541125, 0.541379, 0.541282, 0.540863, 0.540134, 0.539091, 0.537717, 0.535992,\
        0.533907, 0.531475, 0.528730, 0.525709, 0.522441, 0.518934, 0.515195, 0.511256, 0.507180, 0.503030,\
        0.498809, 0.494610, 0.490423, 0.486035, 0.481438, 0.476583, 0.471349, 0.465636, 0.459405, 0.452668,\
        0.445453, 0.437795, 0.429722, 0.421259, 0.412423, 0.403229, 0.393688, 0.383809, 0.373600, 0.363068,\
        0.352218],\
        [0.347723, 0.354942, 0.360974, 0.365782, 0.369536, 0.372643, 0.375544, 0.378481, 0.381522, 0.384677,\
        0.387822, 0.390633, 0.392934, 0.394894, 0.396607, 0.397976, 0.398980, 0.399716, 0.400183, 0.400099,\
        0.399187, 0.398023, 0.396794, 0.395229, 0.393119, 0.390570, 0.387627, 0.384319, 0.380855, 0.377452,\
        0.374226, 0.371219, 0.368442, 0.365905, 0.363619, 0.361598, 0.359857, 0.358412, 0.357280, 0.356477,\
        0.356023, 0.355940, 0.356254, 0.356996, 0.358202, 0.359909, 0.362152, 0.364962, 0.368361, 0.372368,\
        0.377010, 0.382349, 0.388550, 0.395921, 0.404514, 0.414262, 0.424989, 0.436141, 0.447296, 0.458129,\
        0.468342, 0.477726, 0.486353, 0.494567, 0.502125, 0.507759, 0.512824, 0.517024, 0.520501, 0.523652,\
        0.526627, 0.529442, 0.532101, 0.534606, 0.536953, 0.539151, 0.541220, 0.543162, 0.544936, 0.546475,\
        0.547709, 0.548584, 0.549075, 0.549175, 0.548899, 0.548265, 0.547295, 0.546002, 0.544389, 0.542456,\
        0.540206, 0.537651, 0.534814, 0.531721, 0.528393, 0.524844, 0.521092, 0.517166, 0.513108, 0.508958,\
        0.504747, 0.500598, 0.496484, 0.492059, 0.487549, 0.482841, 0.477713, 0.472023, 0.465751, 0.458943,\
        0.451654, 0.443927, 0.435797, 0.427286, 0.418414, 0.409191, 0.399630, 0.389737, 0.379520, 0.368985,\
        0.358138],\
        [0.356099, 0.363564, 0.369706, 0.374339, 0.377587, 0.380123, 0.382715, 0.385589, 0.388637, 0.391908,\
        0.395348, 0.398373, 0.400628, 0.402518, 0.404147, 0.405393, 0.406232, 0.406737, 0.406872, 0.406438,\
        0.405355, 0.403955, 0.402429, 0.400612, 0.398123, 0.395358, 0.392143, 0.388392, 0.384551, 0.380917,\
        0.377549, 0.374432, 0.371550, 0.368901, 0.366494, 0.364344, 0.362471, 0.360891, 0.359621, 0.358679,\
        0.358084, 0.357859, 0.358034, 0.358646, 0.359741, 0.361367, 0.363573, 0.366401, 0.369881, 0.374029,\
        0.378853, 0.384390, 0.390776, 0.398366, 0.407702, 0.418204, 0.429489, 0.441189, 0.452932, 0.464374,\
        0.475128, 0.484748, 0.493198, 0.501230, 0.509017, 0.515509, 0.520940, 0.525539, 0.529447, 0.532874,\
        0.535973, 0.538816, 0.541442, 0.543877, 0.546138, 0.548248, 0.550220, 0.552046, 0.553686, 0.555081,\
        0.556166, 0.556888, 0.557215, 0.557134, 0.556647, 0.555772, 0.554542, 0.552987, 0.551129, 0.548980,\
        0.546549, 0.543849, 0.540897, 0.537714, 0.534317, 0.530724, 0.526952, 0.523029, 0.518987, 0.514862,\
        0.510699, 0.506558, 0.502303, 0.497849, 0.493476, 0.488954, 0.483917, 0.478180, 0.471800, 0.464879,\
        0.457490, 0.449684, 0.441492, 0.432935, 0.424028, 0.414780, 0.405201, 0.395297, 0.385075, 0.374540,\
        0.363697],\
        [0.364017, 0.371843, 0.378304, 0.382916, 0.385489, 0.387200, 0.389595, 0.392731, 0.395858, 0.399248,\
        0.402893, 0.406115, 0.408521, 0.410410, 0.411903, 0.412973, 0.413606, 0.413817, 0.413572, 0.412790,\
        0.411473, 0.409773, 0.407792, 0.405454, 0.402692, 0.399780, 0.396269, 0.391928, 0.387834, 0.384156,\
        0.380786, 0.377648, 0.374710, 0.371974, 0.369459, 0.367189, 0.365185, 0.363470, 0.362060, 0.360974,\
        0.360229, 0.359850, 0.359869, 0.360331, 0.361293, 0.362823, 0.364989, 0.367854, 0.371466, 0.375851,\
        0.381003, 0.386913, 0.393687, 0.401718, 0.411501, 0.422535, 0.434316, 0.446497, 0.458752, 0.470780,\
        0.482240, 0.492470, 0.500801, 0.508847, 0.516545, 0.523361, 0.529198, 0.534179, 0.538443, 0.542134,\
        0.545382, 0.548278, 0.550891, 0.553266, 0.555443, 0.557459, 0.559336, 0.561064, 0.562598, 0.563872,\
        0.564820, 0.565386, 0.565537, 0.565254, 0.564531, 0.563385, 0.561868, 0.560035, 0.557923, 0.555550,\
        0.552926, 0.550062, 0.546977, 0.543687, 0.540210, 0.536561, 0.532758, 0.528822, 0.524781, 0.520665,\
        0.516503, 0.512297, 0.507987, 0.503598, 0.499302, 0.494943, 0.489922, 0.484007, 0.477452, 0.470393,\
        0.462899, 0.455018, 0.446773, 0.438179, 0.429245, 0.419980, 0.410388, 0.400478, 0.390255, 0.379724,\
        0.368890],\
        [0.371281, 0.379482, 0.386474, 0.391546, 0.393274, 0.393806, 0.396134, 0.400209, 0.403650, 0.407271,\
        0.410915, 0.414118, 0.416627, 0.418508, 0.419863, 0.420724, 0.421098, 0.420988, 0.420374, 0.419232,\
        0.417578, 0.415479, 0.412983, 0.410090, 0.406845, 0.403370, 0.399200, 0.394620, 0.390782, 0.387350,\
        0.384115, 0.381016, 0.378046, 0.375229, 0.372603, 0.370206, 0.368066, 0.366208, 0.364650, 0.363410,\
        0.362503, 0.361950, 0.361785, 0.362062, 0.362853, 0.364248, 0.366342, 0.369230, 0.372993, 0.377683,\
        0.383305, 0.389792, 0.397143, 0.405751, 0.415833, 0.427159, 0.439313, 0.451890, 0.464536, 0.476974,\
        0.488983, 0.500096, 0.508955, 0.516935, 0.524542, 0.531479, 0.537596, 0.542902, 0.547472, 0.551403,\
        0.554798, 0.557760, 0.560376, 0.562708, 0.564807, 0.566730, 0.568527, 0.570187, 0.571648, 0.572828,\
        0.573652, 0.574059, 0.574023, 0.573531, 0.572551, 0.571096, 0.569259, 0.567132, 0.564761, 0.562159,\
        0.559330, 0.556287, 0.553049, 0.549635, 0.546061, 0.542340, 0.538485, 0.534514, 0.530448, 0.526309,\
        0.522114, 0.517868, 0.513568, 0.509258, 0.504994, 0.500686, 0.495466, 0.489268, 0.482565, 0.475392,\
        0.467818, 0.459886, 0.451611, 0.442999, 0.434052, 0.424778, 0.415183, 0.405273, 0.395054, 0.384532,\
        0.373711],\
        [0.377758, 0.386146, 0.393380, 0.399408, 0.400764, 0.400978, 0.403245, 0.407870, 0.412116, 0.416001,\
        0.419580, 0.422655, 0.425078, 0.426849, 0.428021, 0.428640, 0.428726, 0.428285, 0.427312, 0.425802,\
        0.423766, 0.421233, 0.418223, 0.414740, 0.410775, 0.406267, 0.401207, 0.397136, 0.394009, 0.390896,\
        0.387799, 0.384729, 0.381701, 0.378776, 0.376017, 0.373475, 0.371183, 0.369166, 0.367447, 0.366039,\
        0.364952, 0.364199, 0.363812, 0.363852, 0.364410, 0.365603, 0.367559, 0.370409, 0.374271, 0.379244,\
        0.385387, 0.392632, 0.400622, 0.409646, 0.420047, 0.431716, 0.444262, 0.457230, 0.470184, 0.482791,\
        0.494889, 0.506219, 0.516083, 0.524700, 0.532548, 0.539679, 0.546045, 0.551639, 0.556487, 0.560638,\
        0.564171, 0.567199, 0.569839, 0.572158, 0.574189, 0.576020, 0.577763, 0.579401, 0.580827, 0.581936,\
        0.582654, 0.582891, 0.582644, 0.581954, 0.580708, 0.578883, 0.576688, 0.574262, 0.571640, 0.568810,\
        0.565764, 0.562519, 0.559107, 0.555550, 0.551861, 0.548047, 0.544116, 0.540080, 0.535955, 0.531755,\
        0.527496, 0.523186, 0.518839, 0.514474, 0.510070, 0.505405, 0.499881, 0.493786, 0.487049, 0.479803,\
        0.472199, 0.464263, 0.455993, 0.447385, 0.438442, 0.429170, 0.419579, 0.409676, 0.399468, 0.388960,\
        0.378157],\
        [0.383527, 0.392027, 0.399036, 0.404855, 0.408099, 0.409961, 0.412854, 0.417060, 0.421415, 0.425376,\
        0.428831, 0.431693, 0.433897, 0.435444, 0.436369, 0.436710, 0.436492, 0.435724, 0.434408, 0.432543,\
        0.430137, 0.427202, 0.423748, 0.419781, 0.415302, 0.410360, 0.405350, 0.401468, 0.398311, 0.395185,\
        0.392093, 0.388971, 0.385812, 0.382719, 0.379792, 0.377079, 0.374608, 0.372408, 0.370506, 0.368913,\
        0.367626, 0.366643, 0.365984, 0.365717, 0.365956, 0.366851, 0.368568, 0.371272, 0.375112, 0.380212,\
        0.386670, 0.394535, 0.403276, 0.412681, 0.423610, 0.435835, 0.448940, 0.462434, 0.475798, 0.488560,\
        0.500700, 0.512333, 0.522820, 0.532083, 0.540352, 0.547773, 0.554408, 0.560288, 0.565417, 0.569783,\
        0.573440, 0.576526, 0.579213, 0.581594, 0.583583, 0.585296, 0.587032, 0.588716, 0.590150, 0.591191,\
        0.591830, 0.591877, 0.591339, 0.590505, 0.589005, 0.586677, 0.584121, 0.581418, 0.578576, 0.575525,\
        0.572236, 0.568758, 0.565146, 0.561426, 0.557601, 0.553670, 0.549635, 0.545501, 0.541277, 0.536975,\
        0.532604, 0.528172, 0.523681, 0.519115, 0.514402, 0.509355, 0.503816, 0.497759, 0.490842, 0.483583,\
        0.476031, 0.468152, 0.459924, 0.451341, 0.442414, 0.433154, 0.423574, 0.413684, 0.403492, 0.393004,\
        0.382225],\
        [0.388688, 0.397683, 0.405017, 0.411099, 0.415718, 0.419378, 0.423148, 0.427343, 0.431547, 0.435347,\
        0.438559, 0.441117, 0.443005, 0.444241, 0.444862, 0.444899, 0.444374, 0.443296, 0.441666, 0.439485,\
        0.436758, 0.433499, 0.429727, 0.425473, 0.420807, 0.415911, 0.411203, 0.407165, 0.403662, 0.400335,\
        0.397144, 0.393873, 0.390468, 0.387145, 0.384020, 0.381108, 0.378421, 0.375996, 0.373878, 0.372083,\
        0.370582, 0.369336, 0.368345, 0.367682, 0.367490, 0.367965, 0.369316, 0.371745, 0.375417, 0.380447,\
        0.386873, 0.394643, 0.403789, 0.414371, 0.426252, 0.439291, 0.453156, 0.467380, 0.481445, 0.494749,\
        0.507068, 0.518811, 0.529601, 0.539296, 0.547948, 0.555662, 0.562553, 0.568720, 0.574162, 0.578763,\
        0.582533, 0.585658, 0.588391, 0.590996, 0.593046, 0.594538, 0.596397, 0.598177, 0.599659, 0.600604,\
        0.601158, 0.601007, 0.600095, 0.599113, 0.597380, 0.594309, 0.591600, 0.588626, 0.585630, 0.582350,\
        0.578756, 0.574997, 0.571161, 0.567259, 0.563279, 0.559206, 0.555035, 0.550765, 0.546402, 0.541950,\
        0.537416, 0.532798, 0.528085, 0.523241, 0.518191, 0.512821, 0.507034, 0.500688, 0.493863, 0.486768,\
        0.479373, 0.471603, 0.463435, 0.454884, 0.445976, 0.436731, 0.427167, 0.417296, 0.407126, 0.396664,\
        0.385914],\
        [0.392997, 0.402615, 0.410747, 0.417667, 0.423564, 0.428687, 0.433435, 0.437962, 0.442118, 0.445694,\
        0.448579, 0.450757, 0.452259, 0.453129, 0.453412, 0.453138, 0.452321, 0.450964, 0.449066, 0.446625,\
        0.443650, 0.440161, 0.436197, 0.431821, 0.427150, 0.422385, 0.417811, 0.413653, 0.409886, 0.406356,\
        0.402951, 0.399430, 0.395701, 0.392136, 0.388811, 0.385666, 0.382704, 0.379985, 0.377607, 0.375599,\
        0.373880, 0.372344, 0.370948, 0.369774, 0.369014, 0.368925, 0.369775, 0.371805, 0.375204, 0.380087,\
        0.386473, 0.394287, 0.403834, 0.415308, 0.428093, 0.442013, 0.456753, 0.471827, 0.486663, 0.500801,\
        0.513501, 0.525339, 0.536355, 0.546388, 0.555365, 0.563309, 0.570360, 0.576750, 0.582580, 0.587471,\
        0.591329, 0.594560, 0.597270, 0.600138, 0.602553, 0.604060, 0.606019, 0.607871, 0.609334, 0.610258,\
        0.610617, 0.610261, 0.609219, 0.607724, 0.605216, 0.602234, 0.599228, 0.595969, 0.592950, 0.589335,\
        0.585305, 0.581221, 0.577153, 0.573058, 0.568901, 0.564658, 0.560315, 0.555870, 0.551324, 0.546679,\
        0.541934, 0.537082, 0.532102, 0.526954, 0.521571, 0.515864, 0.509746, 0.503194, 0.496427, 0.489564,\
        0.482372, 0.474698, 0.466566, 0.458032, 0.449136, 0.439905, 0.430359, 0.420510, 0.410368, 0.399938,\
        0.389224],\
        [0.396340, 0.406606, 0.415689, 0.423841, 0.431146, 0.437663, 0.443452, 0.448509, 0.452754, 0.456126,\
        0.458646, 0.460396, 0.461471, 0.461953, 0.461897, 0.461329, 0.460256, 0.458670, 0.456561, 0.453927,\
        0.450779, 0.447148, 0.443090, 0.438692, 0.434082, 0.429432, 0.424933, 0.420714, 0.416779, 0.413041,\
        0.409366, 0.405571, 0.401626, 0.397865, 0.394321, 0.390878, 0.387552, 0.384417, 0.381716, 0.379506,\
        0.377594, 0.375744, 0.373851, 0.372019, 0.370523, 0.369711, 0.369922, 0.371448, 0.374511, 0.379255,\
        0.385766, 0.394140, 0.404209, 0.415879, 0.429278, 0.443980, 0.459595, 0.475598, 0.491170, 0.505663,\
        0.519085, 0.531415, 0.542850, 0.553306, 0.562634, 0.570752, 0.577768, 0.584102, 0.590334, 0.595782,\
        0.599549, 0.603149, 0.606166, 0.609173, 0.611832, 0.613948, 0.615966, 0.617813, 0.619246, 0.620114,\
        0.620324, 0.619805, 0.618560, 0.616615, 0.613906, 0.610764, 0.607334, 0.603898, 0.600630, 0.596360,\
        0.591820, 0.587437, 0.583153, 0.578847, 0.574484, 0.570032, 0.565478, 0.560816, 0.556047, 0.551168,\
        0.546175, 0.541056, 0.535791, 0.530343, 0.524660, 0.518681, 0.512352, 0.505674, 0.498853, 0.492306,\
        0.485199, 0.477497, 0.469337, 0.460790, 0.451894, 0.442676, 0.433150, 0.423329, 0.413220, 0.402828,\
        0.392156],\
        [0.398164, 0.409292, 0.419592, 0.429233, 0.438119, 0.446109, 0.453041, 0.458759, 0.463178, 0.466359,\
        0.468489, 0.469785, 0.470425, 0.470530, 0.470166, 0.469353, 0.468082, 0.466331, 0.464081, 0.461324,\
        0.458074, 0.454372, 0.450287, 0.445920, 0.441394, 0.436851, 0.432423, 0.428196, 0.424176, 0.420302,\
        0.416460, 0.412539, 0.408531, 0.404579, 0.400694, 0.396862, 0.393074, 0.389287, 0.386210, 0.383859,\
        0.381812, 0.379626, 0.377102, 0.374416, 0.371985, 0.370284, 0.369731, 0.370665, 0.373342, 0.377930,\
        0.384502, 0.393007, 0.403376, 0.415791, 0.429832, 0.445177, 0.461524, 0.478447, 0.495198, 0.510188,\
        0.523916, 0.536847, 0.548932, 0.560006, 0.569829, 0.578172, 0.584983, 0.590726, 0.596422, 0.602820,\
        0.607243, 0.611291, 0.614922, 0.618244, 0.621213, 0.623773, 0.626019, 0.627927, 0.629359, 0.630177,\
        0.630283, 0.629620, 0.628179, 0.625984, 0.623110, 0.619734, 0.616036, 0.612183, 0.608076, 0.603333,\
        0.598376, 0.593761, 0.589238, 0.584653, 0.580041, 0.575336, 0.570521, 0.565601, 0.560572, 0.555426,\
        0.550155, 0.544750, 0.539194, 0.533461, 0.527522, 0.521351, 0.514951, 0.508393, 0.501912, 0.495264,\
        0.487824, 0.479949, 0.471715, 0.463141, 0.454244, 0.445040, 0.435540, 0.425752, 0.415683, 0.405336,\
        0.394712],\
        [0.398286, 0.410481, 0.422334, 0.433680, 0.444286, 0.453866, 0.462068, 0.468552, 0.473170, 0.476120,\
        0.477821, 0.478659, 0.478891, 0.478668, 0.478061, 0.477080, 0.475691, 0.473852, 0.471536, 0.468728,\
        0.465439, 0.461718, 0.457653, 0.453349, 0.448925, 0.444495, 0.440160, 0.435982, 0.431970, 0.428077,\
        0.424218, 0.420308, 0.416305, 0.412213, 0.408015, 0.403753, 0.399237, 0.394531, 0.391172, 0.388761,\
        0.386635, 0.384066, 0.380700, 0.376891, 0.373314, 0.370581, 0.369170, 0.369449, 0.371704, 0.376132,\
        0.382814, 0.391633, 0.402500, 0.415495, 0.429968, 0.445721, 0.462480, 0.479862, 0.497075, 0.513042,\
        0.527739, 0.541595, 0.554567, 0.566468, 0.577003, 0.585807, 0.592587, 0.597638, 0.602861, 0.608940,\
        0.614566, 0.619330, 0.623529, 0.627285, 0.630628, 0.633552, 0.636060, 0.638107, 0.639593, 0.640401,\
        0.640439, 0.639657, 0.638051, 0.635667, 0.632596, 0.628973, 0.624935, 0.620568, 0.615830, 0.610683,\
        0.605403, 0.600434, 0.595412, 0.590466, 0.585590, 0.580559, 0.575431, 0.570219, 0.564899, 0.559456,\
        0.553885, 0.548179, 0.542329, 0.536321, 0.530146, 0.523808, 0.517345, 0.510851, 0.504390, 0.497454,\
        0.489845, 0.481904, 0.473644, 0.465065, 0.456177, 0.446996, 0.437529, 0.427783, 0.417761, 0.407466,\
        0.396897],\
        [0.397534, 0.410751, 0.424146, 0.437178, 0.449511, 0.460749, 0.470362, 0.477730, 0.482506, 0.485102,\
        0.486334, 0.486753, 0.486652, 0.486188, 0.485435, 0.484387, 0.482975, 0.481129, 0.478824, 0.476037,\
        0.472764, 0.469064, 0.465049, 0.460837, 0.456534, 0.452238, 0.448031, 0.443965, 0.440054, 0.436264,\
        0.432526, 0.428751, 0.424846, 0.420722, 0.416286, 0.411448, 0.406052, 0.400739, 0.397044, 0.394335,\
        0.392093, 0.389054, 0.384476, 0.379228, 0.374349, 0.370525, 0.368230, 0.367833, 0.369639, 0.373897,\
        0.380779, 0.390354, 0.402182, 0.415226, 0.429995, 0.446016, 0.462888, 0.480348, 0.497959, 0.514842,\
        0.530845, 0.545895, 0.559863, 0.572658, 0.584046, 0.593483, 0.600703, 0.605800, 0.610975, 0.616672,\
        0.622332, 0.627500, 0.632137, 0.636296, 0.640005, 0.643261, 0.646036, 0.648268, 0.649858, 0.650700,\
        0.650703, 0.649822, 0.648067, 0.645498, 0.642208, 0.638310, 0.633911, 0.629085, 0.623872, 0.618351,\
        0.612722, 0.607149, 0.601513, 0.596377, 0.591130, 0.585636, 0.580173, 0.574656, 0.569021, 0.563257,\
        0.557365, 0.551346, 0.545196, 0.538910, 0.532490, 0.525949, 0.519314, 0.512601, 0.505736, 0.498581,\
        0.491156, 0.483334, 0.475114, 0.466555, 0.457693, 0.448545, 0.439121, 0.429425, 0.419460, 0.409224,\
        0.398717],\
        [0.396800, 0.410793, 0.425321, 0.439739, 0.453606, 0.466441, 0.477588, 0.486057, 0.490881, 0.492898,\
        0.493707, 0.493843, 0.493544, 0.492961, 0.492169, 0.491173, 0.489837, 0.488045, 0.485829, 0.483150,\
        0.479932, 0.476273, 0.472337, 0.468251, 0.464103, 0.459970, 0.455929, 0.452033, 0.448304, 0.444726,\
        0.441239, 0.437739, 0.434082, 0.430095, 0.425600, 0.420467, 0.414763, 0.409116, 0.404308, 0.400540,\
        0.397909, 0.394237, 0.387839, 0.381008, 0.374906, 0.370093, 0.366982, 0.365933, 0.367274, 0.371296,\
        0.378214, 0.388057, 0.400412, 0.414700, 0.430412, 0.446866, 0.463687, 0.480903, 0.498691, 0.516699,\
        0.534157, 0.550385, 0.565132, 0.578500, 0.590644, 0.600613, 0.608003, 0.613882, 0.619359, 0.624968,\
        0.630566, 0.635887, 0.640800, 0.645273, 0.649296, 0.652849, 0.655884, 0.658323, 0.660057, 0.660969,\
        0.660960, 0.659988, 0.658079, 0.655314, 0.651792, 0.647612, 0.642866, 0.637636, 0.632005, 0.626082,\
        0.620015, 0.613925, 0.607955, 0.602367, 0.596304, 0.590441, 0.584734, 0.578911, 0.572927, 0.566813,\
        0.560584, 0.554242, 0.547786, 0.541213, 0.534528, 0.527743, 0.520868, 0.513897, 0.506772, 0.499474,\
        0.492191, 0.484381, 0.476165, 0.467627, 0.458799, 0.449696, 0.440325, 0.430689, 0.420787, 0.410617,\
        0.400179],\
        [0.396227, 0.410670, 0.425860, 0.441256, 0.456293, 0.470421, 0.482986, 0.492937, 0.497743, 0.498995,\
        0.499733, 0.499822, 0.499518, 0.498951, 0.498212, 0.497369, 0.496200, 0.494452, 0.492415, 0.489971,\
        0.486809, 0.483194, 0.479381, 0.475478, 0.471528, 0.467590, 0.463747, 0.460067, 0.456587, 0.453309,\
        0.450190, 0.447119, 0.443905, 0.440290, 0.436006, 0.430882, 0.424989, 0.418727, 0.412620, 0.407093,\
        0.402581, 0.397460, 0.389622, 0.381869, 0.374966, 0.369417, 0.365635, 0.364004, 0.364896, 0.368656,\
        0.375556, 0.385703, 0.398993, 0.415068, 0.432369, 0.449528, 0.466213, 0.483062, 0.500817, 0.520119,\
        0.538922, 0.556020, 0.571004, 0.584135, 0.595862, 0.606411, 0.614605, 0.621370, 0.627441, 0.633269,\
        0.638943, 0.644377, 0.649474, 0.654176, 0.658446, 0.662247, 0.665522, 0.668172, 0.670070, 0.671079,\
        0.671075, 0.670001, 0.667918, 0.664945, 0.661191, 0.656740, 0.651671, 0.646069, 0.640036, 0.633695,\
        0.627180, 0.620621, 0.614124, 0.607655, 0.601139, 0.595107, 0.589198, 0.582970, 0.576577, 0.570092,\
        0.563520, 0.556855, 0.550090, 0.543223, 0.536261, 0.529210, 0.522083, 0.514893, 0.507715, 0.500749,\
        0.493049, 0.485052, 0.476812, 0.468298, 0.459513, 0.450464, 0.441155, 0.431585, 0.421753, 0.411656,\
        0.401291],\
        [0.395091, 0.409877, 0.425567, 0.441597, 0.457350, 0.472201, 0.485429, 0.495973, 0.501564, 0.503333,\
        0.504350, 0.504718, 0.504619, 0.504218, 0.503633, 0.502918, 0.501876, 0.500184, 0.498468, 0.496397,\
        0.493187, 0.489659, 0.486061, 0.482439, 0.478735, 0.475009, 0.471389, 0.467958, 0.464765, 0.461841,\
        0.459183, 0.456692, 0.454138, 0.451159, 0.447355, 0.442428, 0.436323, 0.429264, 0.421624, 0.413727,\
        0.405644, 0.397901, 0.390207, 0.382253, 0.374934, 0.368879, 0.364565, 0.362434, 0.362934, 0.366502,\
        0.373543, 0.384434, 0.399361, 0.417841, 0.437352, 0.455528, 0.471771, 0.488238, 0.506764, 0.526651,\
        0.546278, 0.563832, 0.578439, 0.590733, 0.601820, 0.612139, 0.621117, 0.628654, 0.635291, 0.641439,\
        0.647281, 0.652842, 0.658081, 0.662943, 0.667385, 0.671371, 0.674845, 0.677690, 0.679748, 0.680870,\
        0.680883, 0.679669, 0.677374, 0.674197, 0.670234, 0.665543, 0.660180, 0.654227, 0.647798, 0.641030,\
        0.634063, 0.627024, 0.620002, 0.613042, 0.606272, 0.600045, 0.593520, 0.586703, 0.579887, 0.573048,\
        0.566150, 0.559171, 0.552101, 0.544941, 0.537697, 0.530378, 0.522993, 0.515554, 0.508046, 0.500378,\
        0.493067, 0.485249, 0.477062, 0.468590, 0.459858, 0.450869, 0.441626, 0.432128, 0.422371, 0.412351,\
        0.402063],\
        [0.393580, 0.408483, 0.424461, 0.440814, 0.456848, 0.471878, 0.485089, 0.495359, 0.501932, 0.505556,\
        0.507394, 0.508312, 0.508649, 0.508596, 0.508271, 0.507697, 0.506762, 0.505402, 0.503925, 0.501853,\
        0.498843, 0.495565, 0.492312, 0.489125, 0.485679, 0.482148, 0.478771, 0.475615, 0.472713, 0.470145,\
        0.467990, 0.466201, 0.464520, 0.462460, 0.459405, 0.454845, 0.448562, 0.440672, 0.431527, 0.421552,\
        0.411221, 0.401220, 0.391978, 0.383342, 0.375558, 0.369051, 0.364263, 0.361683, 0.361857, 0.365362,\
        0.372739, 0.384621, 0.401914, 0.423670, 0.446442, 0.466414, 0.481753, 0.495806, 0.515363, 0.536326,\
        0.556566, 0.574465, 0.588301, 0.599028, 0.609722, 0.619573, 0.628419, 0.636205, 0.643155, 0.649540,\
        0.655537, 0.661215, 0.666562, 0.671524, 0.676051, 0.680129, 0.683736, 0.686735, 0.688897, 0.690113,\
        0.690187, 0.688734, 0.686189, 0.682856, 0.678742, 0.673869, 0.668251, 0.661960, 0.655136, 0.647936,\
        0.640516, 0.633012, 0.625532, 0.618176, 0.611063, 0.604173, 0.597038, 0.589891, 0.582782, 0.575652,\
        0.568458, 0.561179, 0.553815, 0.546370, 0.538851, 0.531270, 0.523641, 0.515985, 0.508344, 0.500787,\
        0.493082, 0.485155, 0.476976, 0.468544, 0.459864, 0.450936, 0.441760, 0.432334, 0.422654, 0.412714,\
        0.402505],\
        [0.391921, 0.406754, 0.422810, 0.439167, 0.455128, 0.470031, 0.483174, 0.493864, 0.501499, 0.505962,\
        0.508757, 0.510475, 0.511452, 0.511902, 0.511955, 0.511655, 0.510985, 0.509944, 0.508546, 0.506578,\
        0.503971, 0.500992, 0.498212, 0.495625, 0.492275, 0.488914, 0.485847, 0.483000, 0.480345, 0.478056,\
        0.476373, 0.475338, 0.474711, 0.473864, 0.471846, 0.467845, 0.461483, 0.452837, 0.442370, 0.430724,\
        0.418593, 0.406693, 0.395707, 0.385992, 0.377552, 0.370530, 0.365238, 0.362180, 0.362043, 0.365628,\
        0.373701, 0.386647, 0.406011, 0.432359, 0.459482, 0.482611, 0.498558, 0.510912, 0.528954, 0.549292,\
        0.569047, 0.587157, 0.600729, 0.609979, 0.619218, 0.628204, 0.636566, 0.644211, 0.651200, 0.657667,\
        0.663737, 0.669483, 0.674899, 0.679908, 0.684418, 0.688448, 0.692071, 0.695170, 0.697308, 0.698433,\
        0.698701, 0.696774, 0.694089, 0.690729, 0.686532, 0.681583, 0.675748, 0.669124, 0.661899, 0.654266,\
        0.646398, 0.638444, 0.630524, 0.622728, 0.615094, 0.607535, 0.599980, 0.592610, 0.585293, 0.577913,\
        0.570439, 0.562873, 0.555227, 0.547510, 0.539732, 0.531904, 0.524042, 0.516176, 0.508368, 0.500671,\
        0.492811, 0.484794, 0.476609, 0.468205, 0.459567, 0.450691, 0.441577, 0.432220, 0.422616, 0.412754,\
        0.402626],\
        [0.390557, 0.405204, 0.420954, 0.436978, 0.452578, 0.467153, 0.480118, 0.490919, 0.499005, 0.504649,\
        0.508584, 0.511271, 0.513033, 0.514109, 0.514660, 0.514772, 0.514469, 0.513755, 0.512601, 0.510936,\
        0.508778, 0.506330, 0.504025, 0.501625, 0.498294, 0.495293, 0.492646, 0.490157, 0.487637, 0.485452,\
        0.484149, 0.483806, 0.484289, 0.484942, 0.484232, 0.480975, 0.474671, 0.465374, 0.453786, 0.440801,\
        0.427258, 0.413896, 0.401429, 0.390563, 0.381407, 0.373810, 0.367923, 0.364264, 0.363671, 0.367274,\
        0.376432, 0.392493, 0.414702, 0.443439, 0.473892, 0.500473, 0.517155, 0.530819, 0.546784, 0.564703,\
        0.582076, 0.598263, 0.611532, 0.620899, 0.629288, 0.637441, 0.645256, 0.652614, 0.659478, 0.665887,\
        0.671921, 0.677656, 0.683097, 0.688128, 0.692525, 0.696340, 0.699788, 0.702796, 0.704786, 0.705716,\
        0.705522, 0.703648, 0.701011, 0.697595, 0.693423, 0.688603, 0.682536, 0.675567, 0.667942, 0.659879,\
        0.651572, 0.643184, 0.634839, 0.626618, 0.618544, 0.610578, 0.602710, 0.595069, 0.587506, 0.579849,\
        0.572085, 0.564237, 0.556325, 0.548359, 0.540346, 0.532294, 0.524216, 0.516126, 0.508022, 0.499941,\
        0.492131, 0.484180, 0.476008, 0.467616, 0.459002, 0.450161, 0.441095, 0.431798, 0.422264, 0.412482,\
        0.402434],\
        [0.389399, 0.403763, 0.419020, 0.434454, 0.449481, 0.463585, 0.476256, 0.487027, 0.495606, 0.502210,\
        0.507253, 0.510924, 0.513508, 0.515275, 0.516420, 0.517051, 0.517216, 0.516926, 0.516172, 0.514947,\
        0.513303, 0.511386, 0.509330, 0.506952, 0.504145, 0.501570, 0.499271, 0.497168, 0.494573, 0.492281,\
        0.491373, 0.491499, 0.492788, 0.495116, 0.495776, 0.493397, 0.487368, 0.477544, 0.465091, 0.451178,\
        0.436736, 0.422484, 0.409067, 0.397217, 0.387450, 0.379235, 0.372644, 0.368185, 0.366797, 0.369918,\
        0.379278, 0.396541, 0.423107, 0.454657, 0.486420, 0.513282, 0.532621, 0.548193, 0.563989, 0.580624,\
        0.595655, 0.608707, 0.620862, 0.630598, 0.638940, 0.646731, 0.654191, 0.661304, 0.668006, 0.674270,\
        0.680154, 0.685770, 0.691150, 0.696194, 0.700462, 0.703989, 0.707135, 0.709715, 0.711399, 0.712019,\
        0.711460, 0.709658, 0.706877, 0.703102, 0.699785, 0.694763, 0.688420, 0.681133, 0.673123, 0.664647,\
        0.655926, 0.647129, 0.638388, 0.629797, 0.621400, 0.613200, 0.605173, 0.597285, 0.589410, 0.581425,\
        0.573354, 0.565236, 0.557087, 0.548907, 0.540696, 0.532455, 0.524198, 0.515951, 0.507737, 0.499593,\
        0.491554, 0.483473, 0.475240, 0.466819, 0.458196, 0.449365, 0.440327, 0.431076, 0.421605, 0.411898,\
        0.401933],\
        [0.388291, 0.402381, 0.416968, 0.431626, 0.445975, 0.459566, 0.471954, 0.482776, 0.491843, 0.499241,\
        0.505160, 0.509673, 0.513018, 0.515490, 0.517292, 0.518534, 0.519266, 0.519508, 0.519270, 0.518570,\
        0.517464, 0.516030, 0.514317, 0.512300, 0.510057, 0.507845, 0.505786, 0.503749, 0.500864, 0.498900,\
        0.498597, 0.498891, 0.500398, 0.503409, 0.505019, 0.503594, 0.498112, 0.488266, 0.475397, 0.461161,\
        0.446481, 0.432016, 0.418369, 0.406171, 0.395793, 0.386915, 0.379583, 0.374167, 0.371551, 0.373507,\
        0.382764, 0.401571, 0.431363, 0.466053, 0.498116, 0.525109, 0.545655, 0.562359, 0.578248, 0.594552,\
        0.608740, 0.620163, 0.630667, 0.639977, 0.648259, 0.655939, 0.663255, 0.670251, 0.676846, 0.682936,\
        0.688561, 0.693930, 0.699140, 0.703950, 0.708072, 0.711393, 0.714131, 0.716212, 0.717446, 0.717681,\
        0.716836, 0.714927, 0.712096, 0.708622, 0.704744, 0.699608, 0.693252, 0.685695, 0.677321, 0.668470,\
        0.659373, 0.650202, 0.641099, 0.632178, 0.623520, 0.615164, 0.607068, 0.599045, 0.590848, 0.582510,\
        0.574154, 0.565814, 0.557483, 0.549145, 0.540784, 0.532395, 0.523990, 0.515618, 0.507358, 0.499187,\
        0.490933, 0.482668, 0.474324, 0.465832, 0.457164, 0.448313, 0.439279, 0.430055, 0.420634, 0.411001,\
        0.401119],\
        [0.386865, 0.400734, 0.414489, 0.428360, 0.442078, 0.455208, 0.467364, 0.478257, 0.487741, 0.495812,\
        0.502466, 0.507637, 0.511650, 0.514828, 0.517345, 0.519281, 0.520671, 0.521536, 0.521897, 0.521786,\
        0.521247, 0.520335, 0.519093, 0.517559, 0.515818, 0.513994, 0.512132, 0.510111, 0.507897, 0.506467,\
        0.506287, 0.506874, 0.508420, 0.510641, 0.511849, 0.510452, 0.505523, 0.496294, 0.483878, 0.470194,\
        0.455991, 0.441947, 0.428648, 0.416564, 0.405901, 0.396624, 0.388733, 0.382450, 0.378259, 0.377837,\
        0.385410, 0.405963, 0.439448, 0.479281, 0.512669, 0.538036, 0.557920, 0.574392, 0.589751, 0.605079,\
        0.619126, 0.630620, 0.640524, 0.649418, 0.657587, 0.665234, 0.672519, 0.679520, 0.686124, 0.692083,\
        0.697387, 0.702429, 0.707326, 0.711749, 0.715498, 0.718501, 0.720798, 0.722361, 0.723091, 0.722889,\
        0.721712, 0.719589, 0.716638, 0.713015, 0.708721, 0.703506, 0.697020, 0.689122, 0.680439, 0.671290,\
        0.661880, 0.652377, 0.642932, 0.633682, 0.624746, 0.616202, 0.608037, 0.599975, 0.591485, 0.582900,\
        0.574376, 0.565916, 0.557489, 0.549066, 0.540620, 0.532131, 0.523596, 0.515050, 0.506620, 0.498368,\
        0.490032, 0.481691, 0.473248, 0.464657, 0.455908, 0.447004, 0.437950, 0.428736, 0.419345, 0.409768,\
        0.399980],\
        [0.384549, 0.397795, 0.411109, 0.424530, 0.437792, 0.450552, 0.462510, 0.473436, 0.483199, 0.491748,\
        0.498994, 0.504746, 0.509423, 0.513357, 0.516662, 0.519368, 0.521488, 0.523044, 0.524067, 0.524592,\
        0.524663, 0.524328, 0.523634, 0.522636, 0.521404, 0.520012, 0.518501, 0.516895, 0.515385, 0.514435,\
        0.514302, 0.514878, 0.516011, 0.517205, 0.517371, 0.515320, 0.510164, 0.501535, 0.490623, 0.478173,\
        0.464951, 0.451799, 0.439273, 0.427700, 0.417227, 0.407909, 0.399798, 0.393157, 0.387695, 0.383912,\
        0.386290, 0.405540, 0.445187, 0.490045, 0.525819, 0.551043, 0.570130, 0.585865, 0.601076, 0.616106,\
        0.629723, 0.641039, 0.650558, 0.659144, 0.667269, 0.674927, 0.682185, 0.689231, 0.695948, 0.701844,\
        0.706902, 0.711568, 0.715905, 0.719751, 0.722960, 0.725459, 0.727228, 0.728234, 0.728418, 0.727723,\
        0.726126, 0.723656, 0.720388, 0.716396, 0.711667, 0.706062, 0.699296, 0.691264, 0.682458, 0.673129,\
        0.663482, 0.653691, 0.643915, 0.634307, 0.625015, 0.616157, 0.607740, 0.599507, 0.591020, 0.582471,\
        0.573973, 0.565527, 0.557104, 0.548675, 0.540216, 0.531699, 0.523097, 0.514383, 0.505563, 0.497086,\
        0.488950, 0.480592, 0.472026, 0.463292, 0.454417, 0.445424, 0.436327, 0.427118, 0.417754, 0.408169,\
        0.398426],\
        [0.381611, 0.394265, 0.407193, 0.420315, 0.433210, 0.445650, 0.457411, 0.468303, 0.478197, 0.487000,\
        0.494610, 0.500985, 0.506402, 0.511185, 0.515359, 0.518891, 0.521785, 0.524073, 0.525796, 0.526994,\
        0.527713, 0.527999, 0.527902, 0.527479, 0.526791, 0.525902, 0.524875, 0.523796, 0.522841, 0.522246,\
        0.522143, 0.522460, 0.522952, 0.523124, 0.522207, 0.519388, 0.514097, 0.506323, 0.496682, 0.485256,\
        0.473183, 0.461271, 0.449913, 0.439257, 0.429356, 0.420282, 0.412235, 0.405554, 0.399925, 0.394604,\
        0.391990, 0.409666, 0.453902, 0.500345, 0.537275, 0.563897, 0.583258, 0.598736, 0.614160, 0.629141,\
        0.642170, 0.652718, 0.661379, 0.669452, 0.677719, 0.685482, 0.692608, 0.699633, 0.706326, 0.712096,\
        0.716943, 0.721142, 0.724826, 0.727970, 0.730500, 0.732359, 0.733505, 0.733898, 0.733488, 0.732235,\
        0.730125, 0.727176, 0.723432, 0.718930, 0.713658, 0.707516, 0.700364, 0.692369, 0.683545, 0.674115,\
        0.664296, 0.654261, 0.644168, 0.634173, 0.624445, 0.615153, 0.606370, 0.597955, 0.589659, 0.581346,\
        0.573027, 0.564701, 0.556356, 0.547984, 0.539574, 0.531112, 0.522574, 0.513930, 0.505201, 0.496596,\
        0.488111, 0.479481, 0.470669, 0.461717, 0.452665, 0.443544, 0.434380, 0.425185, 0.415930, 0.406456,\
        0.396715],\
        [0.378536, 0.390857, 0.403385, 0.415989, 0.428455, 0.440571, 0.452112, 0.462911, 0.472845, 0.481820,\
        0.489757, 0.496668, 0.502830, 0.508524, 0.513598, 0.517957, 0.521625, 0.524655, 0.527098, 0.528995,\
        0.530389, 0.531326, 0.531858, 0.532039, 0.531930, 0.531594, 0.531101, 0.530539, 0.530030, 0.529690,\
        0.529558, 0.529533, 0.529359, 0.528611, 0.526743, 0.523235, 0.517792, 0.510499, 0.501550, 0.491222,\
        0.480521, 0.470197, 0.460414, 0.451059, 0.442040, 0.433393, 0.425320, 0.418161, 0.412214, 0.408205,\
        0.408670, 0.427807, 0.470132, 0.514954, 0.550682, 0.578004, 0.598400, 0.614653, 0.630021, 0.644627,\
        0.657119, 0.666523, 0.673619, 0.680519, 0.689232, 0.697557, 0.704416, 0.711390, 0.717733, 0.723112,\
        0.727498, 0.731075, 0.734005, 0.736350, 0.738102, 0.739227, 0.739676, 0.739401, 0.738349, 0.736481,\
        0.733779, 0.730249, 0.725919, 0.720820, 0.714970, 0.708387, 0.701084, 0.692894, 0.683945, 0.674431,\
        0.664488, 0.654262, 0.643896, 0.633531, 0.623332, 0.613533, 0.604450, 0.596008, 0.587884, 0.579836,\
        0.571729, 0.563547, 0.555299, 0.547001, 0.538665, 0.530301, 0.521911, 0.513491, 0.505011, 0.496307,\
        0.487333, 0.478264, 0.469111, 0.459884, 0.450612, 0.441324, 0.432051, 0.422829, 0.413718, 0.404761,\
        0.395212],\
        [0.375365, 0.387376, 0.399455, 0.411508, 0.423588, 0.435383, 0.446670, 0.457328, 0.467268, 0.476417,\
        0.484731, 0.492237, 0.499128, 0.505657, 0.511543, 0.516646, 0.521043, 0.524805, 0.527976, 0.530586,\
        0.532672, 0.534277, 0.535454, 0.536259, 0.536751, 0.536989, 0.537036, 0.536961, 0.536832, 0.536691,\
        0.536510, 0.536162, 0.535404, 0.533892, 0.531232, 0.527090, 0.521310, 0.513987, 0.505423, 0.496117,\
        0.486905, 0.478552, 0.470761, 0.463034, 0.455161, 0.447114, 0.439032, 0.431350, 0.425060, 0.422329,\
        0.427830, 0.449410, 0.487980, 0.528876, 0.564611, 0.593192, 0.614436, 0.631519, 0.647117, 0.661982,\
        0.674755, 0.683098, 0.688322, 0.693039, 0.701317, 0.710837, 0.718139, 0.724722, 0.730435, 0.735065,\
        0.738642, 0.741350, 0.743373, 0.744819, 0.745718, 0.746047, 0.745753, 0.744775, 0.743048, 0.740523,\
        0.737172, 0.732994, 0.728013, 0.722266, 0.715795, 0.708651, 0.700984, 0.692730, 0.683802, 0.674253,\
        0.664231, 0.653886, 0.643339, 0.632708, 0.622124, 0.611785, 0.602184, 0.593949, 0.586133, 0.578277,\
        0.570294, 0.562188, 0.553986, 0.545723, 0.537430, 0.529132, 0.520853, 0.512584, 0.504204, 0.495368,\
        0.486099, 0.476688, 0.467220, 0.457723, 0.448220, 0.438743, 0.429321, 0.419965, 0.410688, 0.401618,\
        0.392408],\
        [0.372056, 0.383689, 0.395224, 0.406924, 0.418732, 0.430151, 0.441116, 0.451588, 0.461510, 0.470841,\
        0.479573, 0.487746, 0.495454, 0.502722, 0.509220, 0.514938, 0.520019, 0.524513, 0.528421, 0.531753,\
        0.534535, 0.536813, 0.538641, 0.540077, 0.541175, 0.541992, 0.542575, 0.542967, 0.543193, 0.543242,\
        0.543046, 0.542460, 0.541260, 0.539166, 0.535887, 0.531195, 0.525007, 0.517445, 0.508914, 0.500199,\
        0.492363, 0.486544, 0.481049, 0.475100, 0.468583, 0.461427, 0.453640, 0.445668, 0.438986, 0.436669,\
        0.443592, 0.464612, 0.499412, 0.540006, 0.577603, 0.608335, 0.630405, 0.648132, 0.664207, 0.679901,\
        0.694271, 0.702432, 0.706944, 0.711067, 0.717801, 0.726173, 0.733491, 0.739559, 0.744372, 0.747900,\
        0.750315, 0.751886, 0.752834, 0.753285, 0.753276, 0.752780, 0.751728, 0.750037, 0.747625, 0.744423,\
        0.740393, 0.735529, 0.729858, 0.723442, 0.716372, 0.708714, 0.700498, 0.692320, 0.683359, 0.673729,\
        0.663660, 0.653285, 0.642699, 0.631997, 0.621304, 0.610860, 0.601171, 0.592794, 0.584863, 0.576926,\
        0.568874, 0.560705, 0.552450, 0.544143, 0.535816, 0.527494, 0.519200, 0.510932, 0.502532, 0.493590,\
        0.484180, 0.474580, 0.464899, 0.455185, 0.445475, 0.435812, 0.426244, 0.416782, 0.407317, 0.397897,\
        0.388847],\
        [0.368642, 0.380067, 0.391414, 0.402811, 0.414055, 0.424857, 0.435416, 0.445663, 0.455533, 0.465019,\
        0.474135, 0.482910, 0.491362, 0.499356, 0.506367, 0.512687, 0.518492, 0.523757, 0.528423, 0.532477,\
        0.535949, 0.538891, 0.541362, 0.543423, 0.545128, 0.546523, 0.547641, 0.548499, 0.549090, 0.549366,\
        0.549233, 0.548536, 0.547070, 0.544602, 0.540906, 0.535826, 0.529329, 0.521553, 0.512864, 0.503955,\
        0.497653, 0.494881, 0.491242, 0.486891, 0.481853, 0.476011, 0.469077, 0.461170, 0.453847, 0.450532,\
        0.456328, 0.476121, 0.510764, 0.552484, 0.590672, 0.622637, 0.646546, 0.665039, 0.681505, 0.697917,\
        0.713192, 0.723403, 0.729172, 0.733584, 0.738694, 0.744797, 0.750786, 0.755807, 0.759359, 0.761386,\
        0.762292, 0.762491, 0.762227, 0.761619, 0.760685, 0.759370, 0.757578, 0.755195, 0.752111, 0.748237,\
        0.743522, 0.737953, 0.731568, 0.724456, 0.716791, 0.708895, 0.701093, 0.692165, 0.682673, 0.672885,\
        0.662818, 0.652523, 0.642058, 0.631497, 0.620964, 0.610697, 0.601146, 0.592473, 0.584115, 0.575827,\
        0.567498, 0.559110, 0.550683, 0.542244, 0.533805, 0.525360, 0.516901, 0.508465, 0.499951, 0.490959,\
        0.481561, 0.471915, 0.462130, 0.452275, 0.442400, 0.432563, 0.422858, 0.413406, 0.404197, 0.394829,\
        0.385478],\
        [0.365042, 0.376344, 0.387675, 0.399017, 0.409113, 0.419264, 0.429497, 0.439501, 0.449259, 0.458832,\
        0.468248, 0.477485, 0.486459, 0.494961, 0.502700, 0.509759, 0.516443, 0.522553, 0.527985, 0.532742,\
        0.536876, 0.540457, 0.543554, 0.546229, 0.548535, 0.550508, 0.552169, 0.553513, 0.554506, 0.555079,\
        0.555118, 0.554465, 0.552928, 0.550306, 0.546424, 0.541179, 0.534601, 0.526904, 0.518585, 0.510734,\
        0.505549, 0.503246, 0.500496, 0.497489, 0.493901, 0.489874, 0.484622, 0.477402, 0.469647, 0.464714,\
        0.468494, 0.488110, 0.526514, 0.571626, 0.610017, 0.641291, 0.665575, 0.684177, 0.701359, 0.718715,\
        0.734636, 0.746259, 0.753402, 0.757778, 0.761215, 0.765143, 0.769431, 0.773087, 0.775041, 0.775086,\
        0.774191, 0.772886, 0.771345, 0.769675, 0.767846, 0.765757, 0.763275, 0.760249, 0.756532, 0.752012,\
        0.746622, 0.740349, 0.733234, 0.725374, 0.716935, 0.708163, 0.699535, 0.690809, 0.681381, 0.671630,\
        0.661674, 0.651572, 0.641367, 0.631112, 0.620899, 0.610902, 0.601431, 0.592448, 0.583595, 0.574816,\
        0.566064, 0.557331, 0.548636, 0.539996, 0.531403, 0.522809, 0.514121, 0.505284, 0.496632, 0.487728,\
        0.478407, 0.468783, 0.458971, 0.449047, 0.439066, 0.429076, 0.419169, 0.409573, 0.400488, 0.391329,\
        0.382012],\
        [0.361097, 0.372138, 0.383030, 0.393394, 0.403049, 0.413363, 0.423406, 0.433057, 0.442594, 0.452181,\
        0.461832, 0.471455, 0.480888, 0.489890, 0.498233, 0.506299, 0.514018, 0.520969, 0.527114, 0.532517,\
        0.537267, 0.541450, 0.545146, 0.548420, 0.551321, 0.553879, 0.556099, 0.557963, 0.559417, 0.560377,\
        0.560715, 0.560273, 0.558866, 0.556313, 0.552469, 0.547278, 0.540833, 0.533442, 0.525734, 0.518753,\
        0.513655, 0.510392, 0.507806, 0.505441, 0.503116, 0.501048, 0.498297, 0.492741, 0.486056, 0.480481,\
        0.482925, 0.504620, 0.548109, 0.595334, 0.632694, 0.663818, 0.689396, 0.707652, 0.725723, 0.743310,\
        0.759528, 0.771584, 0.778918, 0.782693, 0.783871, 0.785792, 0.788371, 0.790590, 0.790793, 0.788214,\
        0.785550, 0.782771, 0.779959, 0.777313, 0.774668, 0.771880, 0.768792, 0.765199, 0.760908, 0.755778,\
        0.749734, 0.742770, 0.734933, 0.726328, 0.717119, 0.707527, 0.698029, 0.688989, 0.679622, 0.669963,\
        0.660172, 0.650333, 0.640480, 0.630630, 0.620814, 0.611119, 0.601663, 0.592314, 0.582956, 0.573649,\
        0.564401, 0.555243, 0.546212, 0.537332, 0.528590, 0.519922, 0.511219, 0.502284, 0.493287, 0.484271,\
        0.474921, 0.465300, 0.455496, 0.445576, 0.435580, 0.425530, 0.415439, 0.405432, 0.396226, 0.387413,\
        0.378367],\
        [0.356758, 0.367459, 0.377910, 0.387929, 0.397792, 0.407907, 0.417230, 0.426219, 0.435418, 0.445000,\
        0.454887, 0.464916, 0.474891, 0.484593, 0.493900, 0.502943, 0.511433, 0.519053, 0.525785, 0.531745,\
        0.537049, 0.541793, 0.546061, 0.549918, 0.553411, 0.556562, 0.559367, 0.561794, 0.563780, 0.565225,\
        0.565997, 0.565932, 0.564849, 0.562571, 0.558961, 0.553982, 0.547757, 0.540630, 0.533211, 0.526310,\
        0.520627, 0.516287, 0.512948, 0.510350, 0.508449, 0.507220, 0.505910, 0.503595, 0.500620, 0.497852,\
        0.502192, 0.527984, 0.571748, 0.617006, 0.652846, 0.684608, 0.714827, 0.736027, 0.753557, 0.769936,\
        0.786408, 0.798092, 0.804342, 0.807105, 0.806236, 0.806351, 0.806872, 0.806941, 0.804862, 0.800046,\
        0.796235, 0.791812, 0.787883, 0.784478, 0.781087, 0.777689, 0.774109, 0.770050, 0.765248, 0.759542,\
        0.752866, 0.745228, 0.736691, 0.727376, 0.717496, 0.707405, 0.697415, 0.687458, 0.677664, 0.667912,\
        0.658230, 0.648660, 0.639200, 0.629806, 0.620407, 0.610916, 0.601238, 0.591666, 0.581930, 0.572096,\
        0.562317, 0.552691, 0.543286, 0.534146, 0.525275, 0.516589, 0.507953, 0.499348, 0.490065, 0.480669,\
        0.471167, 0.461517, 0.451748, 0.441900, 0.432003, 0.422085, 0.412156, 0.402223, 0.392788, 0.383831,\
        0.374795],\
        [0.352057, 0.362443, 0.372645, 0.382704, 0.392814, 0.402318, 0.410487, 0.418697, 0.427607, 0.437279,\
        0.447456, 0.457930, 0.468533, 0.479092, 0.489453, 0.499405, 0.508577, 0.516698, 0.523873, 0.530314,\
        0.536126, 0.541396, 0.546211, 0.550638, 0.554723, 0.558481, 0.561900, 0.564941, 0.567529, 0.569559,\
        0.570893, 0.571364, 0.570784, 0.568965, 0.565750, 0.561076, 0.555044, 0.547972, 0.540401, 0.532986,\
        0.526293, 0.520608, 0.515980, 0.512417, 0.509972, 0.508622, 0.508092, 0.508181, 0.509191, 0.512545,\
        0.523996, 0.551353, 0.592359, 0.635463, 0.674671, 0.710497, 0.743225, 0.767200, 0.784212, 0.797672,\
        0.812620, 0.823444, 0.828520, 0.829783, 0.828443, 0.826575, 0.824592, 0.821804, 0.817361, 0.811564,\
        0.805804, 0.799405, 0.795398, 0.791227, 0.787029, 0.783139, 0.779228, 0.774815, 0.769550, 0.763277,\
        0.755969, 0.747667, 0.738451, 0.728438, 0.717820, 0.706897, 0.696095, 0.685618, 0.675362, 0.665369,\
        0.655693, 0.646354, 0.637294, 0.628383, 0.619454, 0.610328, 0.600902, 0.590758, 0.580377, 0.569951,\
        0.559628, 0.549530, 0.539738, 0.530314, 0.521318, 0.512693, 0.503997, 0.495153, 0.486049, 0.476619,\
        0.467053, 0.457408, 0.447713, 0.437996, 0.428280, 0.418604, 0.409073, 0.399639, 0.389972, 0.380592,\
        0.371351],\
        [0.347023, 0.357071, 0.366955, 0.376686, 0.386164, 0.394869, 0.402620, 0.410282, 0.419218, 0.429169,\
        0.439635, 0.450507, 0.461707, 0.473070, 0.484374, 0.495294, 0.505221, 0.513579, 0.521154, 0.528088,\
        0.534392, 0.540162, 0.545505, 0.550496, 0.555177, 0.559558, 0.563622, 0.567322, 0.570579, 0.573283,\
        0.575294, 0.576440, 0.576526, 0.575334, 0.572655, 0.568349, 0.562440, 0.555191, 0.547090, 0.538740,\
        0.530703, 0.523399, 0.517110, 0.512068, 0.508497, 0.506605, 0.506585, 0.508792, 0.514047, 0.524137,\
        0.542209, 0.570947, 0.611361, 0.657700, 0.704721, 0.748950, 0.781298, 0.802362, 0.817402, 0.829577,\
        0.840462, 0.848196, 0.851534, 0.851382, 0.848995, 0.845452, 0.841100, 0.835788, 0.829374, 0.822183,\
        0.814812, 0.808004, 0.802719, 0.797306, 0.792354, 0.788237, 0.784212, 0.779528, 0.773785, 0.766899,\
        0.758926, 0.749957, 0.740084, 0.729407, 0.718056, 0.706274, 0.694632, 0.683407, 0.672523, 0.662124,\
        0.652338, 0.643177, 0.634509, 0.626087, 0.617627, 0.608818, 0.599194, 0.588833, 0.577951, 0.566985,\
        0.556169, 0.545646, 0.535499, 0.525765, 0.516540, 0.508097, 0.499929, 0.490762, 0.481483, 0.472046,\
        0.462505, 0.452922, 0.443345, 0.433809, 0.424309, 0.414821, 0.405384, 0.396106, 0.386671, 0.377226,\
        0.367862],\
        [0.341686, 0.351341, 0.360775, 0.369939, 0.378674, 0.386730, 0.394063, 0.401387, 0.410892, 0.421053,\
        0.431544, 0.442639, 0.454311, 0.466309, 0.478322, 0.489952, 0.500322, 0.509144, 0.517462, 0.524986,\
        0.531757, 0.537996, 0.543856, 0.549412, 0.554697, 0.559718, 0.564452, 0.568852, 0.572834, 0.576286,\
        0.579063, 0.580996, 0.581882, 0.581475, 0.579483, 0.575626, 0.569785, 0.562160, 0.553244, 0.543670,\
        0.534057, 0.524933, 0.516747, 0.509922, 0.504908, 0.502223, 0.502516, 0.506690, 0.516073, 0.532473,\
        0.557499, 0.590380, 0.633149, 0.688394, 0.746264, 0.794496, 0.820991, 0.837055, 0.850068, 0.861171,\
        0.868920, 0.872746, 0.873415, 0.871341, 0.867554, 0.862472, 0.856051, 0.848574, 0.840374, 0.831852,\
        0.823504, 0.815861, 0.809063, 0.802648, 0.797274, 0.793197, 0.789198, 0.784204, 0.777847, 0.770231,\
        0.761529, 0.751875, 0.741374, 0.730106, 0.718151, 0.705639, 0.693042, 0.680740, 0.668926, 0.657912,\
        0.647893, 0.638868, 0.630609, 0.622682, 0.614603, 0.606091, 0.596575, 0.585693, 0.574385, 0.563007,\
        0.551813, 0.540978, 0.530596, 0.520639, 0.511006, 0.502037, 0.494231, 0.485518, 0.476332, 0.466973,\
        0.457518, 0.448036, 0.438601, 0.429278, 0.420066, 0.410813, 0.401416, 0.392177, 0.382914, 0.373557,\
        0.364185],\
        [0.336119, 0.345351, 0.354305, 0.362927, 0.371127, 0.378843, 0.386257, 0.394222, 0.403633, 0.413156,\
        0.423155, 0.434325, 0.446367, 0.458819, 0.471245, 0.483130, 0.493840, 0.503718, 0.513059, 0.521013,\
        0.528129, 0.534806, 0.541189, 0.547321, 0.553220, 0.558890, 0.564315, 0.569446, 0.574197, 0.578447,\
        0.582050, 0.584838, 0.586618, 0.587135, 0.586012, 0.582744, 0.576964, 0.568811, 0.558886, 0.547930,\
        0.536634, 0.525610, 0.515425, 0.506668, 0.500012, 0.496282, 0.496536, 0.502144, 0.514823, 0.536484,\
        0.568638, 0.609439, 0.655290, 0.720188, 0.789063, 0.833804, 0.854404, 0.865823, 0.878756, 0.889879,\
        0.895420, 0.895365, 0.893478, 0.888821, 0.883826, 0.877394, 0.868947, 0.859534, 0.849774, 0.840119,\
        0.830964, 0.822596, 0.815052, 0.808273, 0.802580, 0.798371, 0.794256, 0.788703, 0.781459, 0.772979,\
        0.763482, 0.753111, 0.741987, 0.730185, 0.717712, 0.704652, 0.690944, 0.677221, 0.664230, 0.652424,\
        0.642075, 0.633177, 0.625404, 0.618086, 0.610283, 0.601528, 0.591894, 0.581032, 0.569544, 0.557933,\
        0.546506, 0.535494, 0.525073, 0.515231, 0.505687, 0.496409, 0.488059, 0.479588, 0.470664, 0.461472,\
        0.452144, 0.442771, 0.433452, 0.424316, 0.415463, 0.406746, 0.397563, 0.388248, 0.378939, 0.369611,\
        0.360272],\
        [0.330415, 0.339239, 0.347757, 0.355952, 0.363821, 0.371441, 0.379104, 0.387363, 0.396387, 0.404704,\
        0.414283, 0.425708, 0.438094, 0.450855, 0.463569, 0.475865, 0.487512, 0.498618, 0.508079, 0.515977,\
        0.523345, 0.530506, 0.537461, 0.544189, 0.550701, 0.557020, 0.563144, 0.569028, 0.574579, 0.579663,\
        0.584117, 0.587773, 0.590461, 0.591975, 0.591930, 0.589507, 0.583822, 0.575018, 0.564002, 0.551667,\
        0.538749, 0.525896, 0.513755, 0.503038, 0.494602, 0.489530, 0.489215, 0.495409, 0.510157, 0.535472,\
        0.572633, 0.622046, 0.680482, 0.752761, 0.827583, 0.868166, 0.883234, 0.889442, 0.902362, 0.913833,\
        0.917440, 0.915359, 0.910566, 0.903319, 0.897907, 0.889532, 0.879004, 0.867974, 0.856952, 0.846426,\
        0.836733, 0.828042, 0.820353, 0.813630, 0.807956, 0.803303, 0.799047, 0.792340, 0.784103, 0.774779,\
        0.764457, 0.753307, 0.741506, 0.729171, 0.716186, 0.702132, 0.687349, 0.672336, 0.658093, 0.645402,\
        0.634675, 0.625903, 0.618704, 0.612273, 0.605290, 0.596160, 0.586065, 0.575039, 0.563480, 0.551805,\
        0.540299, 0.529192, 0.518817, 0.509422, 0.500335, 0.491167, 0.482307, 0.473607, 0.464749, 0.455691,\
        0.446492, 0.437218, 0.427948, 0.418839, 0.410168, 0.401931, 0.393212, 0.383928, 0.374642, 0.365374,\
        0.356105],\
        [0.324664, 0.333118, 0.341264, 0.349131, 0.356773, 0.364305, 0.371935, 0.379855, 0.387890, 0.395811,\
        0.405616, 0.417321, 0.429872, 0.442711, 0.455527, 0.468091, 0.480234, 0.491639, 0.501313, 0.509379,\
        0.517270, 0.525110, 0.532710, 0.540029, 0.547122, 0.554064, 0.560882, 0.567535, 0.573915, 0.579861,\
        0.585178, 0.589667, 0.593163, 0.595564, 0.596699, 0.595576, 0.589991, 0.580455, 0.568484, 0.555015,\
        0.540740, 0.526307, 0.512401, 0.499822, 0.489555, 0.482860, 0.481376, 0.487194, 0.502803, 0.530807,\
        0.573509, 0.632662, 0.700430, 0.767495, 0.848019, 0.902149, 0.917641, 0.921299, 0.929109, 0.935713,\
        0.936493, 0.932227, 0.924909, 0.916288, 0.907593, 0.897019, 0.885590, 0.873339, 0.861380, 0.850302,\
        0.840416, 0.831813, 0.824392, 0.817944, 0.812196, 0.806748, 0.800877, 0.793913, 0.785436, 0.775411,\
        0.764198, 0.752137, 0.739493, 0.726491, 0.713268, 0.698764, 0.682418, 0.665898, 0.650375, 0.636778,\
        0.625707, 0.617063, 0.610321, 0.604735, 0.598468, 0.589748, 0.579102, 0.567867, 0.556322, 0.544766,\
        0.533396, 0.522282, 0.511688, 0.502700, 0.494191, 0.485218, 0.476345, 0.467553, 0.458711, 0.449752,\
        0.440684, 0.431534, 0.422311, 0.413045, 0.404158, 0.396047, 0.387749, 0.378956, 0.369931, 0.360826,\
        0.351684],\
        [0.318930, 0.327055, 0.334883, 0.342489, 0.349951, 0.357367, 0.364853, 0.372486, 0.380281, 0.388568,\
        0.398268, 0.409745, 0.422020, 0.434570, 0.447182, 0.459649, 0.471702, 0.482902, 0.492710, 0.501334,\
        0.510212, 0.518846, 0.527067, 0.534888, 0.542474, 0.549984, 0.557479, 0.564917, 0.572163, 0.579017,\
        0.585227, 0.590522, 0.594674, 0.597605, 0.599458, 0.600053, 0.594408, 0.584563, 0.572216, 0.558150,\
        0.542994, 0.527392, 0.512070, 0.497884, 0.485885, 0.477413, 0.474237, 0.478702, 0.493827, 0.523169,\
        0.570312, 0.637550, 0.712072, 0.780719, 0.871920, 0.935278, 0.954274, 0.957872, 0.958659, 0.957804,\
        0.953594, 0.945957, 0.936003, 0.924982, 0.913550, 0.901666, 0.888586, 0.875282, 0.862695, 0.851394,\
        0.841670, 0.833545, 0.826777, 0.820915, 0.815345, 0.809325, 0.802272, 0.795120, 0.785969, 0.774899,\
        0.762549, 0.749392, 0.735683, 0.721531, 0.707239, 0.692678, 0.675403, 0.657870, 0.641302, 0.626836,\
        0.615570, 0.607314, 0.600544, 0.595419, 0.588967, 0.580629, 0.570524, 0.559512, 0.548178, 0.536956,\
        0.526090, 0.515436, 0.504733, 0.495263, 0.487222, 0.478740, 0.470066, 0.461342, 0.452565, 0.443713,\
        0.434794, 0.425843, 0.416870, 0.407721, 0.398462, 0.390348, 0.382190, 0.373686, 0.364922, 0.356019,\
        0.347041],\
        [0.313250, 0.321075, 0.328628, 0.336028, 0.343359, 0.350690, 0.358099, 0.365678, 0.373583, 0.382170,\
        0.391928, 0.402922, 0.414517, 0.426446, 0.438606, 0.450754, 0.462572, 0.473689, 0.483858, 0.493384,\
        0.502797, 0.512036, 0.520687, 0.528796, 0.536729, 0.544736, 0.552893, 0.561137, 0.569300, 0.577134,\
        0.584327, 0.590510, 0.595304, 0.598493, 0.600429, 0.600575, 0.596049, 0.587177, 0.575365, 0.561409,\
        0.545962, 0.529720, 0.513469, 0.498111, 0.484715, 0.474600, 0.469490, 0.471768, 0.484797, 0.513106,\
        0.561573, 0.631703, 0.710223, 0.789520, 0.885801, 0.953353, 0.980232, 0.985738, 0.982835, 0.976303,\
        0.967137, 0.955768, 0.942902, 0.929330, 0.915614, 0.901884, 0.887650, 0.873669, 0.860760, 0.849518,\
        0.840270, 0.832990, 0.827292, 0.822488, 0.817712, 0.812115, 0.805291, 0.796745, 0.785838, 0.773165,\
        0.759328, 0.744934, 0.730199, 0.714914, 0.699040, 0.683241, 0.666370, 0.648819, 0.631777, 0.616480,\
        0.605044, 0.597718, 0.591392, 0.585462, 0.578777, 0.570573, 0.560917, 0.550280, 0.539230, 0.528399,\
        0.518367, 0.508901, 0.499177, 0.489457, 0.480693, 0.472178, 0.463616, 0.454995, 0.446319, 0.437588,\
        0.428819, 0.420073, 0.411487, 0.403130, 0.394142, 0.385344, 0.376891, 0.368401, 0.359777, 0.351040,\
        0.342228],\
        [0.307626, 0.315160, 0.322475, 0.329745, 0.337016, 0.344324, 0.351733, 0.359354, 0.367370, 0.376051,\
        0.385654, 0.396152, 0.406965, 0.418236, 0.429889, 0.441679, 0.453305, 0.464480, 0.475076, 0.485235,\
        0.495175, 0.504852, 0.513571, 0.521678, 0.529824, 0.538279, 0.547099, 0.556178, 0.565305, 0.574201,\
        0.582518, 0.589816, 0.595560, 0.599241, 0.601063, 0.600542, 0.596671, 0.589208, 0.578564, 0.565310,\
        0.550128, 0.533805, 0.517208, 0.501297, 0.487142, 0.475965, 0.469269, 0.469145, 0.478772, 0.503155,\
        0.549776, 0.622979, 0.709077, 0.800990, 0.904207, 0.967375, 0.995293, 1.001976, 0.997597, 0.987767,\
        0.975022, 0.960565, 0.945145, 0.929300, 0.913505, 0.898158, 0.883175, 0.868791, 0.855736, 0.844708,\
        0.836146, 0.830019, 0.825780, 0.822502, 0.819058, 0.814274, 0.807105, 0.797110, 0.784587, 0.769900,\
        0.754174, 0.738457, 0.723085, 0.707319, 0.690802, 0.674000, 0.656942, 0.639861, 0.623409, 0.608077,\
        0.596602, 0.589493, 0.582853, 0.576116, 0.568811, 0.560483, 0.551050, 0.540723, 0.529866, 0.519109,\
        0.509691, 0.501827, 0.492666, 0.483249, 0.474255, 0.465608, 0.457075, 0.448548, 0.439989, 0.431390,\
        0.422756, 0.414107, 0.405579, 0.397380, 0.388933, 0.380215, 0.371633, 0.363114, 0.354563, 0.345956,\
        0.337297],\
        [0.302007, 0.309223, 0.316395, 0.323653, 0.330948, 0.338285, 0.345724, 0.353374, 0.361394, 0.369962,\
        0.379191, 0.388999, 0.399231, 0.409973, 0.421152, 0.432622, 0.444105, 0.455331, 0.466179, 0.476692,\
        0.486965, 0.496828, 0.505283, 0.513366, 0.521720, 0.530621, 0.540122, 0.550052, 0.560159, 0.570161,\
        0.579709, 0.588352, 0.595517, 0.600333, 0.602181, 0.601381, 0.598005, 0.591847, 0.582603, 0.570374,\
        0.555880, 0.539995, 0.523668, 0.507950, 0.493965, 0.482847, 0.475789, 0.474347, 0.481019, 0.500058,\
        0.539227, 0.616301, 0.723455, 0.820360, 0.916778, 0.973661, 1.000046, 1.007012, 1.002218, 0.990854,\
        0.976069, 0.959682, 0.942660, 0.925324, 0.908022, 0.891592, 0.876065, 0.861343, 0.848100, 0.837231,\
        0.829415, 0.824644, 0.822137, 0.820679, 0.818896, 0.815332, 0.808382, 0.797050, 0.782135, 0.764687,\
        0.746568, 0.729293, 0.713688, 0.698263, 0.681356, 0.664314, 0.647451, 0.631228, 0.616634, 0.604760,\
        0.593729, 0.583701, 0.575242, 0.567317, 0.559272, 0.550676, 0.541360, 0.531365, 0.520810, 0.509883,\
        0.499996, 0.492835, 0.484822, 0.476103, 0.467388, 0.458843, 0.450410, 0.442007, 0.433584, 0.425133,\
        0.416671, 0.408143, 0.399568, 0.391365, 0.383137, 0.374706, 0.366227, 0.357761, 0.349292, 0.340803,\
        0.332290],\
        [0.296113, 0.303172, 0.310467, 0.317837, 0.325207, 0.332588, 0.340042, 0.347669, 0.355596, 0.363940,\
        0.372759, 0.382026, 0.391714, 0.401865, 0.412568, 0.423766, 0.435111, 0.446286, 0.457138, 0.467636,\
        0.477734, 0.487175, 0.495727, 0.503990, 0.512549, 0.521894, 0.532095, 0.542840, 0.553860, 0.564917,\
        0.575691, 0.585714, 0.594315, 0.600463, 0.603330, 0.603234, 0.600436, 0.595807, 0.587955, 0.576830,\
        0.563338, 0.548357, 0.532871, 0.518086, 0.505335, 0.495818, 0.490455, 0.490244, 0.497175, 0.515316,\
        0.552909, 0.635220, 0.751003, 0.840500, 0.920328, 0.970402, 0.995368, 1.002466, 0.997743, 0.985876,\
        0.970215, 0.953025, 0.935696, 0.918300, 0.900304, 0.883480, 0.867419, 0.852251, 0.838618, 0.827637,\
        0.820462, 0.817142, 0.816418, 0.816730, 0.816490, 0.814268, 0.808242, 0.796024, 0.777426, 0.756830,\
        0.736191, 0.717083, 0.700728, 0.685998, 0.670078, 0.653763, 0.637590, 0.622101, 0.608642, 0.598079,\
        0.587970, 0.577189, 0.567484, 0.558591, 0.549941, 0.541113, 0.531912, 0.522360, 0.512539, 0.502408,\
        0.492498, 0.484388, 0.476578, 0.468416, 0.460104, 0.451818, 0.443586, 0.435369, 0.427109, 0.418797,\
        0.410539, 0.402335, 0.393888, 0.385572, 0.377339, 0.369045, 0.360699, 0.352337, 0.343973, 0.335608,\
        0.327243],\
        [0.290460, 0.297626, 0.305002, 0.312435, 0.319856, 0.327255, 0.334682, 0.342228, 0.350001, 0.358093,\
        0.366552, 0.375381, 0.384578, 0.394178, 0.404384, 0.415358, 0.426506, 0.437469, 0.448092, 0.458309,\
        0.468047, 0.477188, 0.485749, 0.494079, 0.502675, 0.512450, 0.523319, 0.534700, 0.546427, 0.558377,\
        0.570282, 0.581641, 0.591691, 0.599436, 0.604051, 0.605456, 0.604239, 0.601018, 0.594233, 0.584394,\
        0.572283, 0.558662, 0.544443, 0.531090, 0.520450, 0.514120, 0.512855, 0.516906, 0.527925, 0.550651,\
        0.594522, 0.676650, 0.792664, 0.867870, 0.922678, 0.962567, 0.984568, 0.991036, 0.986208, 0.974190,\
        0.958298, 0.940955, 0.924191, 0.908968, 0.891683, 0.874220, 0.857824, 0.842457, 0.828425, 0.816923,\
        0.810127, 0.808316, 0.809002, 0.810559, 0.810950, 0.809025, 0.802961, 0.788394, 0.768001, 0.745632,\
        0.723560, 0.703165, 0.686774, 0.672267, 0.657605, 0.642545, 0.627469, 0.612655, 0.598911, 0.588568,\
        0.578635, 0.568478, 0.558671, 0.549426, 0.540511, 0.531601, 0.522509, 0.513331, 0.504224, 0.494885,\
        0.485367, 0.476692, 0.468602, 0.460605, 0.452585, 0.444577, 0.436606, 0.428644, 0.420609, 0.412397,\
        0.404161, 0.396185, 0.388011, 0.379770, 0.371564, 0.363354, 0.355123, 0.346881, 0.338639, 0.330408,\
        0.322194],\
        [0.285397, 0.292712, 0.300069, 0.307501, 0.314930, 0.322294, 0.329638, 0.337052, 0.344640, 0.352490,\
        0.360654, 0.369154, 0.378007, 0.387267, 0.397126, 0.407701, 0.418465, 0.429029, 0.439237, 0.449040,\
        0.458403, 0.467307, 0.475834, 0.484239, 0.492996, 0.503099, 0.514193, 0.525798, 0.537873, 0.550474,\
        0.563376, 0.576049, 0.587724, 0.597431, 0.604034, 0.607222, 0.607620, 0.605512, 0.600467, 0.592397,\
        0.582251, 0.570502, 0.557722, 0.545778, 0.537479, 0.535399, 0.540413, 0.551115, 0.567726, 0.595278,\
        0.640458, 0.723129, 0.828361, 0.882391, 0.920792, 0.953038, 0.971102, 0.975625, 0.969955, 0.957655,\
        0.941867, 0.924907, 0.907713, 0.894610, 0.879353, 0.862894, 0.847353, 0.832858, 0.819231, 0.807180,\
        0.800303, 0.799833, 0.800637, 0.802426, 0.802220, 0.797814, 0.790555, 0.774345, 0.753459, 0.731088,\
        0.709707, 0.690099, 0.673529, 0.659037, 0.645051, 0.631031, 0.617146, 0.603739, 0.590477, 0.578915,\
        0.568521, 0.558494, 0.548834, 0.539632, 0.530787, 0.522013, 0.513014, 0.503950, 0.495196, 0.486330,\
        0.477394, 0.468816, 0.460665, 0.452749, 0.444931, 0.437168, 0.429464, 0.421813, 0.414144, 0.406197,\
        0.397902, 0.389951, 0.381967, 0.373882, 0.365774, 0.357663, 0.349552, 0.341442, 0.333338, 0.325248,\
        0.317191],\
        [0.280723, 0.288162, 0.295576, 0.303027, 0.310405, 0.317679, 0.324885, 0.332123, 0.339509, 0.347138,\
        0.355070, 0.363338, 0.371979, 0.381058, 0.390668, 0.400771, 0.411007, 0.421022, 0.430690, 0.440017,\
        0.449020, 0.457725, 0.466240, 0.474815, 0.483892, 0.493926, 0.504795, 0.516175, 0.528154, 0.541169,\
        0.554920, 0.568803, 0.582054, 0.593729, 0.602506, 0.607455, 0.609532, 0.609128, 0.606010, 0.600124,\
        0.592839, 0.583465, 0.571983, 0.560757, 0.553958, 0.555921, 0.568498, 0.588090, 0.609420, 0.640470,\
        0.688471, 0.767773, 0.842562, 0.887317, 0.919865, 0.944909, 0.957312, 0.958294, 0.950990, 0.938212,\
        0.922595, 0.907246, 0.891798, 0.877320, 0.863462, 0.849228, 0.835925, 0.823896, 0.812686, 0.802010,\
        0.795454, 0.793129, 0.792469, 0.792219, 0.789792, 0.783186, 0.772249, 0.755636, 0.735393, 0.714034,\
        0.693919, 0.676194, 0.660339, 0.646149, 0.632656, 0.619257, 0.606072, 0.593220, 0.580627, 0.568792,\
        0.557949, 0.547810, 0.538248, 0.529218, 0.520644, 0.512322, 0.503530, 0.494570, 0.485925, 0.477372,\
        0.468860, 0.460561, 0.452556, 0.444790, 0.437164, 0.429614, 0.422137, 0.414765, 0.407493, 0.400178,\
        0.392024, 0.383933, 0.375951, 0.367969, 0.359978, 0.351992, 0.344022, 0.336071, 0.328131, 0.320195,\
        0.312294],\
        [0.276310, 0.283879, 0.291396, 0.298855, 0.306189, 0.313341, 0.320363, 0.327387, 0.334563, 0.341997,\
        0.349745, 0.357839, 0.366312, 0.375204, 0.384540, 0.394225, 0.403952, 0.413369, 0.422436, 0.431285,\
        0.439961, 0.448491, 0.456969, 0.465596, 0.474667, 0.484439, 0.494985, 0.505575, 0.517227, 0.530580,\
        0.545001, 0.559866, 0.574389, 0.587630, 0.598390, 0.605271, 0.609634, 0.611595, 0.610936, 0.608022,\
        0.603487, 0.596878, 0.586428, 0.575223, 0.568053, 0.571939, 0.591132, 0.619504, 0.648237, 0.680388,\
        0.725257, 0.785887, 0.843537, 0.887697, 0.919624, 0.937885, 0.943209, 0.939603, 0.930564, 0.917964,\
        0.902538, 0.888280, 0.874575, 0.860427, 0.846714, 0.834011, 0.823295, 0.814617, 0.807387, 0.800117,\
        0.792469, 0.786965, 0.783448, 0.780181, 0.775043, 0.766305, 0.753087, 0.735325, 0.715727, 0.696713,\
        0.678052, 0.661467, 0.646622, 0.633181, 0.620324, 0.607272, 0.594306, 0.581750, 0.569493, 0.557780,\
        0.546803, 0.536575, 0.527105, 0.518285, 0.509891, 0.502143, 0.493942, 0.485135, 0.476581, 0.468221,\
        0.460009, 0.451997, 0.444235, 0.436700, 0.429306, 0.421959, 0.414644, 0.407447, 0.400371, 0.393232,\
        0.385629, 0.377770, 0.369906, 0.362050, 0.354197, 0.346360, 0.338559, 0.330808, 0.323091, 0.315350,\
        0.307595],\
        [0.272052, 0.279733, 0.287335, 0.294838, 0.302148, 0.309180, 0.315979, 0.322758, 0.329732, 0.337007,\
        0.344611, 0.352553, 0.360846, 0.369502, 0.378505, 0.387754, 0.397022, 0.405868, 0.414361, 0.422820,\
        0.431231, 0.439600, 0.447982, 0.456496, 0.465317, 0.474577, 0.484231, 0.494160, 0.505737, 0.519184,\
        0.533941, 0.549484, 0.564898, 0.579204, 0.591319, 0.600615, 0.608084, 0.612750, 0.614712, 0.614526,\
        0.612507, 0.608114, 0.599762, 0.590306, 0.581291, 0.582000, 0.607760, 0.646273, 0.681267, 0.710573,\
        0.742776, 0.783290, 0.831587, 0.880954, 0.916300, 0.928556, 0.926659, 0.919074, 0.908931, 0.898296,\
        0.884562, 0.870771, 0.857274, 0.843484, 0.829764, 0.817667, 0.809119, 0.803177, 0.798479, 0.792746,\
        0.785106, 0.778215, 0.772426, 0.766692, 0.759408, 0.749097, 0.734840, 0.716401, 0.695989, 0.679000,\
        0.662135, 0.646416, 0.632324, 0.619755, 0.607862, 0.595138, 0.582259, 0.569762, 0.557689, 0.546133,\
        0.535187, 0.524913, 0.515532, 0.507136, 0.498852, 0.491141, 0.483409, 0.475216, 0.466987, 0.458887,\
        0.450950, 0.443215, 0.435725, 0.428488, 0.421409, 0.414306, 0.407113, 0.400028, 0.393144, 0.386167,\
        0.378913, 0.371411, 0.363796, 0.356135, 0.348456, 0.340789, 0.333173, 0.325660, 0.318268, 0.310878,\
        0.303260],\
        [0.267867, 0.275628, 0.283289, 0.290822, 0.298142, 0.305084, 0.311600, 0.318121, 0.324948, 0.332127,\
        0.339626, 0.347421, 0.355507, 0.363877, 0.372491, 0.381244, 0.389930, 0.398245, 0.406435, 0.414645,\
        0.422854, 0.431078, 0.439322, 0.447621, 0.456078, 0.464800, 0.473877, 0.483603, 0.494720, 0.507540,\
        0.522225, 0.538181, 0.554137, 0.569320, 0.583129, 0.595249, 0.605290, 0.612194, 0.616460, 0.618767,\
        0.619187, 0.617401, 0.613353, 0.607914, 0.601018, 0.598677, 0.630842, 0.675415, 0.707794, 0.729078,\
        0.749268, 0.775039, 0.812037, 0.862914, 0.904619, 0.911871, 0.905360, 0.897415, 0.887768, 0.878000,\
        0.866493, 0.853740, 0.840602, 0.827243, 0.813566, 0.801138, 0.793949, 0.789291, 0.785127, 0.779928,\
        0.773157, 0.765929, 0.758885, 0.751643, 0.743086, 0.731996, 0.717704, 0.700391, 0.680654, 0.662714,\
        0.646394, 0.631168, 0.617377, 0.605488, 0.594151, 0.582105, 0.569593, 0.557314, 0.545464, 0.534144,\
        0.523396, 0.513172, 0.503643, 0.495684, 0.487927, 0.480285, 0.472734, 0.465010, 0.457193, 0.449418,\
        0.441767, 0.434295, 0.427062, 0.420149, 0.413506, 0.406768, 0.399759, 0.392813, 0.386028, 0.379202,\
        0.372203, 0.365015, 0.357687, 0.350270, 0.342801, 0.335316, 0.327875, 0.320575, 0.313544, 0.306835,\
        0.299941],\
        [0.263702, 0.271508, 0.279184, 0.286697, 0.293987, 0.300829, 0.307024, 0.313379, 0.320203, 0.327381,\
        0.334807, 0.342445, 0.350292, 0.358341, 0.366550, 0.374820, 0.383001, 0.390991, 0.398906, 0.406876,\
        0.414886, 0.422997, 0.431093, 0.439127, 0.447197, 0.455476, 0.464174, 0.473607, 0.484164, 0.496104,\
        0.510673, 0.526618, 0.542501, 0.558164, 0.573383, 0.587744, 0.600287, 0.608852, 0.615431, 0.620371,\
        0.623590, 0.625369, 0.626316, 0.627248, 0.629678, 0.640656, 0.670606, 0.704774, 0.725052, 0.737045,\
        0.748287, 0.764336, 0.791142, 0.834610, 0.873998, 0.884866, 0.881490, 0.875360, 0.867273, 0.858205,\
        0.848031, 0.836450, 0.823838, 0.811393, 0.798940, 0.786989, 0.779187, 0.774155, 0.769803, 0.764703,\
        0.758231, 0.750736, 0.742915, 0.734906, 0.725811, 0.714418, 0.700038, 0.683019, 0.664553, 0.646862,\
        0.630968, 0.616377, 0.602076, 0.590742, 0.579691, 0.568213, 0.556342, 0.544490, 0.532977, 0.522011,\
        0.511665, 0.501887, 0.492561, 0.484447, 0.476933, 0.469561, 0.462235, 0.454847, 0.447388, 0.439933,\
        0.432574, 0.425364, 0.418319, 0.411631, 0.405485, 0.399302, 0.392496, 0.385723, 0.379045, 0.372371,\
        0.365603, 0.358701, 0.351668, 0.344527, 0.337300, 0.330016, 0.322721, 0.315526, 0.308648, 0.302233,\
        0.295990],\
        [0.259539, 0.267366, 0.275020, 0.282439, 0.289518, 0.296083, 0.302126, 0.308626, 0.315659, 0.322895,\
        0.330229, 0.337666, 0.345235, 0.352946, 0.360770, 0.368638, 0.376461, 0.384195, 0.391884, 0.399572,\
        0.407382, 0.415474, 0.423411, 0.431109, 0.438758, 0.446628, 0.454995, 0.464153, 0.474432, 0.486301,\
        0.500335, 0.515027, 0.529926, 0.545431, 0.561370, 0.576918, 0.590704, 0.601715, 0.611309, 0.619002,\
        0.624916, 0.630244, 0.636382, 0.644825, 0.657592, 0.677094, 0.701378, 0.720027, 0.729975, 0.735353,\
        0.741425, 0.753041, 0.774076, 0.805531, 0.835479, 0.852010, 0.855362, 0.852000, 0.845451, 0.837540,\
        0.828892, 0.818774, 0.806174, 0.794168, 0.782597, 0.771899, 0.763868, 0.758226, 0.753516, 0.748366,\
        0.741804, 0.733647, 0.724891, 0.716476, 0.707215, 0.695625, 0.681166, 0.664584, 0.647065, 0.630120,\
        0.614999, 0.602156, 0.588812, 0.576668, 0.565555, 0.554392, 0.542998, 0.531581, 0.520431, 0.509870,\
        0.499999, 0.490677, 0.481861, 0.473775, 0.466289, 0.459092, 0.452003, 0.444907, 0.437751, 0.430571,\
        0.423488, 0.416600, 0.409781, 0.402961, 0.397208, 0.391326, 0.385029, 0.378572, 0.372116, 0.365655,\
        0.359139, 0.352529, 0.345808, 0.338974, 0.332032, 0.324996, 0.317881, 0.310727, 0.303789, 0.297508,\
        0.291404],\
        [0.255386, 0.263240, 0.270887, 0.278245, 0.285195, 0.291626, 0.297724, 0.304476, 0.311674, 0.318850,\
        0.325977, 0.333123, 0.340361, 0.347722, 0.355197, 0.362737, 0.370289, 0.377823, 0.385338, 0.392853,\
        0.400624, 0.408677, 0.416321, 0.423537, 0.430681, 0.438123, 0.446163, 0.455046, 0.465044, 0.476471,\
        0.489288, 0.502522, 0.516067, 0.531053, 0.547350, 0.563611, 0.578771, 0.592369, 0.604487, 0.614314,\
        0.621809, 0.629686, 0.639701, 0.653458, 0.672280, 0.694525, 0.710360, 0.718410, 0.722442, 0.724289,\
        0.728015, 0.738253, 0.755869, 0.778782, 0.801953, 0.820739, 0.827464, 0.826697, 0.821521, 0.814971,\
        0.807815, 0.800280, 0.788034, 0.775753, 0.764465, 0.754590, 0.746925, 0.741223, 0.736420, 0.731432,\
        0.724992, 0.715795, 0.705586, 0.696586, 0.687099, 0.675336, 0.661355, 0.645533, 0.628879, 0.612754,\
        0.598245, 0.586116, 0.574403, 0.562900, 0.551915, 0.541001, 0.529957, 0.518930, 0.508159, 0.497917,\
        0.488564, 0.479666, 0.471249, 0.463371, 0.455988, 0.448950, 0.442100, 0.435297, 0.428425, 0.421455,\
        0.414565, 0.408041, 0.401806, 0.395207, 0.389194, 0.383443, 0.377496, 0.371391, 0.365228, 0.359040,\
        0.352814, 0.346523, 0.340143, 0.333654, 0.327049, 0.320338, 0.313542, 0.306660, 0.299700, 0.293466,\
        0.287370],\
        [0.251254, 0.259169, 0.266883, 0.274318, 0.281397, 0.288107, 0.294670, 0.301503, 0.308482, 0.315332,\
        0.322070, 0.328808, 0.335655, 0.342660, 0.349815, 0.357081, 0.364417, 0.371803, 0.379250, 0.386823,\
        0.394643, 0.402521, 0.409613, 0.416196, 0.422762, 0.429769, 0.437486, 0.446021, 0.455487, 0.465966,\
        0.477290, 0.489034, 0.501045, 0.515790, 0.532250, 0.548782, 0.564982, 0.580482, 0.595016, 0.605082,\
        0.612776, 0.621984, 0.633530, 0.648600, 0.667658, 0.687527, 0.697806, 0.701650, 0.704182, 0.705070,\
        0.707776, 0.719062, 0.735054, 0.754297, 0.774804, 0.791952, 0.798080, 0.799732, 0.795481, 0.790599,\
        0.784677, 0.777646, 0.767521, 0.755994, 0.744895, 0.735443, 0.728394, 0.723111, 0.718396, 0.713653,\
        0.708321, 0.698386, 0.686998, 0.676587, 0.666171, 0.654541, 0.641362, 0.626521, 0.610728, 0.595694,\
        0.582264, 0.570480, 0.559578, 0.549030, 0.538648, 0.528141, 0.517412, 0.506760, 0.496475, 0.486702,\
        0.477592, 0.469012, 0.460886, 0.453215, 0.445980, 0.439119, 0.432532, 0.426070, 0.419542, 0.412766,\
        0.405877, 0.399584, 0.393658, 0.387601, 0.381644, 0.375887, 0.370127, 0.364299, 0.358428, 0.352539,\
        0.346631, 0.340689, 0.334682, 0.328579, 0.322353, 0.316010, 0.309619, 0.303243, 0.296755, 0.290240,\
        0.283913],\
        [0.247142, 0.255162, 0.263022, 0.270668, 0.278060, 0.285212, 0.292230, 0.299179, 0.305843, 0.312217,\
        0.318409, 0.324634, 0.331053, 0.337710, 0.344576, 0.351601, 0.358744, 0.365987, 0.373344, 0.380854,\
        0.388516, 0.396015, 0.402690, 0.408768, 0.414751, 0.421408, 0.428893, 0.437045, 0.445847, 0.455330,\
        0.465414, 0.475991, 0.487529, 0.501674, 0.516903, 0.532726, 0.549009, 0.564831, 0.578924, 0.589270,\
        0.597910, 0.607404, 0.618208, 0.631122, 0.646290, 0.660959, 0.670410, 0.674939, 0.677514, 0.679417,\
        0.684188, 0.696102, 0.711097, 0.728770, 0.747094, 0.760857, 0.767661, 0.769682, 0.768114, 0.764942,\
        0.760306, 0.753881, 0.745339, 0.735054, 0.724289, 0.715015, 0.708829, 0.704243, 0.699594, 0.694630,\
        0.689138, 0.679985, 0.668120, 0.656548, 0.645234, 0.633680, 0.621592, 0.608165, 0.593296, 0.579292,\
        0.566625, 0.555292, 0.544998, 0.535318, 0.525773, 0.515862, 0.505467, 0.495153, 0.485325, 0.475949,\
        0.467103, 0.458758, 0.450850, 0.443349, 0.436249, 0.429556, 0.423241, 0.417176, 0.411119, 0.404747,\
        0.397908, 0.391635, 0.385793, 0.379999, 0.374249, 0.368593, 0.362980, 0.357370, 0.351760, 0.346164,\
        0.340586, 0.335014, 0.329414, 0.323738, 0.317927, 0.311938, 0.305857, 0.299805, 0.293572, 0.287196,\
        0.280774],\
        [0.243021, 0.251174, 0.259230, 0.267153, 0.274914, 0.282500, 0.289881, 0.296924, 0.303418, 0.309292,\
        0.314807, 0.320452, 0.326459, 0.332812, 0.339428, 0.346233, 0.353175, 0.360230, 0.367389, 0.374636,\
        0.381877, 0.388850, 0.395278, 0.401024, 0.406476, 0.413082, 0.420522, 0.428258, 0.436361, 0.444966,\
        0.454150, 0.464059, 0.475157, 0.487802, 0.501072, 0.515830, 0.531476, 0.546443, 0.559477, 0.570069,\
        0.579152, 0.587959, 0.596861, 0.606210, 0.616894, 0.628196, 0.637436, 0.642998, 0.646425, 0.649911,\
        0.656274, 0.667923, 0.683402, 0.701357, 0.718729, 0.730199, 0.736028, 0.738693, 0.739563, 0.738793,\
        0.735507, 0.729955, 0.723018, 0.714135, 0.703899, 0.694926, 0.689233, 0.684940, 0.680396, 0.675106,\
        0.668398, 0.659105, 0.647750, 0.635922, 0.624208, 0.612726, 0.601531, 0.589741, 0.576269, 0.563175,\
        0.551138, 0.540267, 0.530541, 0.521736, 0.513189, 0.504045, 0.494095, 0.484145, 0.474616, 0.465555,\
        0.456981, 0.448876, 0.441177, 0.433823, 0.426811, 0.420227, 0.414141, 0.408463, 0.402839, 0.396939,\
        0.390486, 0.384233, 0.378352, 0.372653, 0.367048, 0.361520, 0.356051, 0.350625, 0.345244, 0.339921,\
        0.334666, 0.329472, 0.324302, 0.319094, 0.313771, 0.308196, 0.302205, 0.296428, 0.290438, 0.284180,\
        0.277754],\
        [0.238848, 0.247137, 0.255392, 0.263587, 0.271691, 0.279659, 0.287394, 0.294677, 0.301139, 0.306370,\
        0.310998, 0.316090, 0.321799, 0.327932, 0.334342, 0.340937, 0.347657, 0.354466, 0.361334, 0.368212,\
        0.374992, 0.381496, 0.387543, 0.393033, 0.398353, 0.405374, 0.412642, 0.419758, 0.427121, 0.434952,\
        0.443398, 0.452625, 0.462832, 0.474024, 0.486062, 0.499383, 0.513902, 0.527392, 0.539140, 0.549339,\
        0.558237, 0.566263, 0.573629, 0.580599, 0.588014, 0.596681, 0.605227, 0.610707, 0.614738, 0.619206,\
        0.625592, 0.637704, 0.654650, 0.672833, 0.689412, 0.699738, 0.704943, 0.708431, 0.711481, 0.713464,\
        0.711341, 0.706173, 0.700826, 0.693572, 0.684481, 0.676426, 0.670495, 0.665843, 0.661155, 0.655467,\
        0.647934, 0.638076, 0.626742, 0.615122, 0.603459, 0.592274, 0.581541, 0.570551, 0.558803, 0.546990,\
        0.535736, 0.525326, 0.516032, 0.508053, 0.500404, 0.491983, 0.482683, 0.473275, 0.464139, 0.455410,\
        0.447140, 0.439325, 0.431892, 0.424716, 0.417747, 0.411180, 0.405217, 0.399829, 0.394491, 0.388843,\
        0.382900, 0.376943, 0.371171, 0.365567, 0.360078, 0.354674, 0.349341, 0.344076, 0.338891, 0.333807,\
        0.328849, 0.324023, 0.319287, 0.314553, 0.309757, 0.304898, 0.299517, 0.293512, 0.287449, 0.281196,\
        0.274767],\
        [0.234583, 0.242979, 0.251398, 0.259811, 0.268175, 0.276426, 0.284449, 0.292021, 0.298628, 0.303154,\
        0.306620, 0.311494, 0.317106, 0.323099, 0.329329, 0.335709, 0.342180, 0.348700, 0.355229, 0.361716,\
        0.368078, 0.374208, 0.380036, 0.385657, 0.391615, 0.398505, 0.405024, 0.411365, 0.418032, 0.425183,\
        0.432977, 0.441519, 0.450880, 0.461086, 0.472191, 0.484364, 0.497360, 0.508507, 0.518886, 0.528480,\
        0.536994, 0.544561, 0.551289, 0.557457, 0.563673, 0.570440, 0.577029, 0.581888, 0.586005, 0.590679,\
        0.596468, 0.609582, 0.626999, 0.643586, 0.659162, 0.670324, 0.677029, 0.681754, 0.686009, 0.689225,\
        0.688192, 0.683862, 0.678895, 0.672549, 0.664979, 0.657847, 0.651963, 0.647034, 0.642195, 0.636310,\
        0.628376, 0.617870, 0.605958, 0.594862, 0.583652, 0.572851, 0.562444, 0.552069, 0.541553, 0.531014,\
        0.520665, 0.510823, 0.501722, 0.494319, 0.487236, 0.479479, 0.471062, 0.462374, 0.453764, 0.445432,\
        0.437506, 0.430046, 0.422998, 0.416152, 0.409236, 0.402613, 0.396781, 0.391491, 0.386302, 0.380927,\
        0.375359, 0.369729, 0.364164, 0.358706, 0.353344, 0.348064, 0.342860, 0.337736, 0.332712, 0.327823,\
        0.323113, 0.318622, 0.314318, 0.310024, 0.305576, 0.301072, 0.296322, 0.290504, 0.284401, 0.278152,\
        0.271740],\
        [0.230197, 0.238656, 0.247179, 0.255725, 0.264232, 0.272609, 0.280715, 0.288307, 0.294905, 0.298835,\
        0.301959, 0.307012, 0.312543, 0.318384, 0.324421, 0.330570, 0.336766, 0.342967, 0.349138, 0.355245,\
        0.361241, 0.367088, 0.372797, 0.378513, 0.384516, 0.390822, 0.396858, 0.402821, 0.408959, 0.415549,\
        0.422797, 0.430734, 0.439353, 0.448706, 0.458853, 0.469693, 0.480531, 0.490304, 0.499714, 0.508582,\
        0.516649, 0.524039, 0.530675, 0.536678, 0.542402, 0.548018, 0.553179, 0.557523, 0.561585, 0.566415,\
        0.573792, 0.586941, 0.601198, 0.615760, 0.631854, 0.644100, 0.652267, 0.658211, 0.662978, 0.665944,\
        0.665707, 0.662580, 0.657861, 0.651916, 0.645261, 0.638894, 0.633230, 0.628180, 0.623414, 0.617666,\
        0.609571, 0.599385, 0.586795, 0.575508, 0.564693, 0.554319, 0.544302, 0.534456, 0.524959, 0.515703,\
        0.506231, 0.497183, 0.488778, 0.481421, 0.474506, 0.467282, 0.459591, 0.451601, 0.443545, 0.435623,\
        0.428031, 0.420947, 0.414384, 0.408127, 0.401624, 0.394912, 0.389092, 0.383742, 0.378568, 0.373357,\
        0.368052, 0.362696, 0.357356, 0.352073, 0.346858, 0.341710, 0.336632, 0.331634, 0.326741, 0.321995,\
        0.317457, 0.313222, 0.309358, 0.305566, 0.301331, 0.296902, 0.292228, 0.286889, 0.281055, 0.274941,\
        0.268610],\
        [0.225693, 0.234162, 0.242724, 0.251318, 0.259851, 0.268197, 0.276173, 0.283484, 0.289624, 0.294083,\
        0.298131, 0.302887, 0.308187, 0.313819, 0.319640, 0.325539, 0.331439, 0.337297, 0.343099, 0.348837,\
        0.354501, 0.360089, 0.365635, 0.371228, 0.376957, 0.382758, 0.388462, 0.394115, 0.399868, 0.406020,\
        0.412862, 0.420289, 0.428187, 0.436685, 0.445843, 0.455407, 0.464846, 0.473758, 0.482254, 0.490294,\
        0.497893, 0.505170, 0.511873, 0.517888, 0.523446, 0.528513, 0.532914, 0.536824, 0.540787, 0.545832,\
        0.553566, 0.564858, 0.577801, 0.592762, 0.608937, 0.621110, 0.629547, 0.636191, 0.641408, 0.644440,\
        0.644551, 0.642019, 0.637704, 0.632006, 0.625636, 0.619880, 0.614472, 0.609124, 0.604366, 0.598993,\
        0.590490, 0.580163, 0.568504, 0.557099, 0.546506, 0.536655, 0.527247, 0.517786, 0.508962, 0.501328,\
        0.492611, 0.484272, 0.476515, 0.469381, 0.462591, 0.455722, 0.448575, 0.441176, 0.433638, 0.426092,\
        0.418735, 0.411941, 0.405845, 0.400037, 0.394130, 0.387791, 0.381903, 0.376479, 0.371296, 0.366184,\
        0.361063, 0.355925, 0.350795, 0.345696, 0.340642, 0.335637, 0.330690, 0.325812, 0.321032, 0.316397,\
        0.311961, 0.307809, 0.304284, 0.301159, 0.297218, 0.292815, 0.288150, 0.283036, 0.277471, 0.271556,\
        0.265362],\
        [0.221107, 0.229538, 0.238087, 0.246662, 0.255140, 0.263366, 0.271139, 0.278200, 0.284267, 0.289322,\
        0.293954, 0.298765, 0.303916, 0.309364, 0.314978, 0.320625, 0.326210, 0.331704, 0.337122, 0.342494,\
        0.347835, 0.353156, 0.358478, 0.363835, 0.369245, 0.374668, 0.380053, 0.385430, 0.390922, 0.396764,\
        0.403278, 0.410242, 0.417329, 0.424982, 0.433315, 0.441939, 0.450422, 0.458542, 0.466301, 0.473742,\
        0.480938, 0.487934, 0.494568, 0.500606, 0.506247, 0.511201, 0.515168, 0.518844, 0.522658, 0.527330,\
        0.534434, 0.544887, 0.557972, 0.572907, 0.587916, 0.599627, 0.607832, 0.615140, 0.621082, 0.624471,\
        0.624696, 0.622271, 0.618476, 0.613223, 0.606370, 0.601289, 0.596125, 0.590550, 0.585256, 0.579202,\
        0.570956, 0.560965, 0.550029, 0.539083, 0.528933, 0.519801, 0.511412, 0.502817, 0.493983, 0.486962,\
        0.479483, 0.471924, 0.464733, 0.457946, 0.451396, 0.444844, 0.438134, 0.431238, 0.424199, 0.417049,\
        0.409822, 0.403068, 0.397397, 0.391890, 0.386277, 0.380506, 0.374873, 0.369522, 0.364392, 0.359377,\
        0.354409, 0.349457, 0.344521, 0.339607, 0.334722, 0.329873, 0.325069, 0.320323, 0.315656, 0.311130,\
        0.306835, 0.302723, 0.299091, 0.296302, 0.292754, 0.288606, 0.284067, 0.279126, 0.273775, 0.268057,\
        0.262018],\
        [0.216507, 0.224868, 0.233372, 0.241894, 0.250280, 0.258364, 0.265968, 0.272908, 0.279063, 0.284503,\
        0.289541, 0.294522, 0.299643, 0.304969, 0.310416, 0.315828, 0.321081, 0.326174, 0.331186, 0.336189,\
        0.341215, 0.346267, 0.351340, 0.356435, 0.361548, 0.366665, 0.371780, 0.376937, 0.382251, 0.387905,\
        0.394069, 0.400510, 0.406649, 0.413623, 0.421456, 0.429419, 0.437138, 0.444526, 0.451633, 0.458539,\
        0.465307, 0.471941, 0.478337, 0.484381, 0.490064, 0.495028, 0.498887, 0.502649, 0.506442, 0.510271,\
        0.516459, 0.526725, 0.539837, 0.553890, 0.567277, 0.578364, 0.587205, 0.595062, 0.601646, 0.605485,\
        0.605832, 0.603167, 0.599898, 0.595728, 0.589038, 0.583507, 0.578258, 0.572626, 0.566687, 0.559931,\
        0.551742, 0.542265, 0.532070, 0.521631, 0.512032, 0.503757, 0.496223, 0.488787, 0.480904, 0.473770,\
        0.466884, 0.460047, 0.453415, 0.447036, 0.440813, 0.434590, 0.428258, 0.421800, 0.415254, 0.408637,\
        0.401819, 0.395066, 0.389392, 0.384040, 0.378681, 0.373282, 0.367950, 0.362781, 0.357777, 0.352891,\
        0.348076, 0.343300, 0.338551, 0.333824, 0.329119, 0.324439, 0.319798, 0.315210, 0.310682, 0.306249,\
        0.302138, 0.298375, 0.294798, 0.291649, 0.288244, 0.284340, 0.279988, 0.275220, 0.270052, 0.264509,\
        0.258621],\
        [0.211998, 0.220273, 0.228727, 0.237183, 0.245464, 0.253412, 0.260886, 0.267771, 0.274024, 0.279717,\
        0.285038, 0.290205, 0.295371, 0.300617, 0.305944, 0.311151, 0.316027, 0.320661, 0.325241, 0.329887,\
        0.334627, 0.339431, 0.344264, 0.349107, 0.353954, 0.358809, 0.363692, 0.368651, 0.373776, 0.379171,\
        0.384858, 0.390606, 0.396291, 0.402965, 0.410503, 0.417880, 0.424868, 0.431536, 0.438017, 0.444399,\
        0.450721, 0.456976, 0.463097, 0.468987, 0.474502, 0.479372, 0.483542, 0.487435, 0.491168, 0.494661,\
        0.500223, 0.510730, 0.523590, 0.536253, 0.548061, 0.558526, 0.567645, 0.575683, 0.582207, 0.586306,\
        0.587089, 0.585066, 0.582025, 0.577903, 0.572423, 0.566757, 0.561148, 0.555247, 0.548867, 0.541730,\
        0.533554, 0.524491, 0.515295, 0.505658, 0.496364, 0.488989, 0.482066, 0.475232, 0.468295, 0.461562,\
        0.455064, 0.448728, 0.442579, 0.436620, 0.430772, 0.424891, 0.418883, 0.412778, 0.406650, 0.400481,\
        0.394193, 0.387836, 0.382037, 0.376663, 0.371445, 0.366290, 0.361213, 0.356252, 0.351418, 0.346691,\
        0.342042, 0.337447, 0.332889, 0.328356, 0.323842, 0.319343, 0.314878, 0.310486, 0.306176, 0.301889,\
        0.297830, 0.294243, 0.290805, 0.287480, 0.284020, 0.280212, 0.275998, 0.271377, 0.266360, 0.260965,\
        0.255210],\
        [0.207723, 0.215922, 0.224336, 0.232712, 0.240867, 0.248669, 0.256020, 0.262854, 0.269161, 0.275005,\
        0.280513, 0.285832, 0.291078, 0.296314, 0.301569, 0.306577, 0.310964, 0.315070, 0.319212, 0.323552,\
        0.328070, 0.332674, 0.337294, 0.341904, 0.346508, 0.351128, 0.355797, 0.360559, 0.365472, 0.370580,\
        0.375874, 0.381292, 0.386976, 0.393419, 0.400397, 0.407167, 0.413387, 0.419360, 0.425262, 0.431156,\
        0.437049, 0.442934, 0.448774, 0.454475, 0.459869, 0.464769, 0.469146, 0.473180, 0.477076, 0.481383,\
        0.487771, 0.497968, 0.509349, 0.520042, 0.530195, 0.539836, 0.548784, 0.556714, 0.563077, 0.567184,\
        0.568530, 0.567421, 0.564714, 0.560792, 0.555874, 0.550437, 0.544701, 0.538561, 0.531925, 0.524709,\
        0.516748, 0.507910, 0.499510, 0.491537, 0.483041, 0.475789, 0.469188, 0.462780, 0.456399, 0.450102,\
        0.443955, 0.437988, 0.432227, 0.426664, 0.421216, 0.415699, 0.409956, 0.404093, 0.398271, 0.392422,\
        0.386492, 0.380592, 0.374964, 0.369650, 0.364547, 0.359574, 0.354708, 0.349951, 0.345304, 0.340756,\
        0.336288, 0.331881, 0.327522, 0.323198, 0.318891, 0.314584, 0.310289, 0.306080, 0.302062, 0.298040,\
        0.294112, 0.290488, 0.287041, 0.283631, 0.280101, 0.276306, 0.272164, 0.267645, 0.262743, 0.257464,\
        0.251820],\
        [0.203889, 0.212054, 0.220417, 0.228654, 0.236623, 0.244238, 0.251435, 0.258180, 0.264477, 0.270378,\
        0.275975, 0.281376, 0.286674, 0.291939, 0.297173, 0.301827, 0.305683, 0.309262, 0.313026, 0.317179,\
        0.321584, 0.326048, 0.330477, 0.334861, 0.339231, 0.343631, 0.348097, 0.352667, 0.357373, 0.362241,\
        0.367283, 0.372532, 0.378119, 0.384191, 0.390581, 0.396754, 0.402385, 0.407808, 0.413249, 0.418733,\
        0.424239, 0.429775, 0.435367, 0.440951, 0.446341, 0.451344, 0.455917, 0.460211, 0.464572, 0.469677,\
        0.476558, 0.485643, 0.495395, 0.504570, 0.513188, 0.522092, 0.530765, 0.538524, 0.544765, 0.548987,\
        0.550792, 0.550258, 0.547909, 0.544236, 0.539633, 0.534466, 0.528828, 0.522587, 0.515836, 0.508752,\
        0.501346, 0.493208, 0.485290, 0.477961, 0.470658, 0.463834, 0.457495, 0.451391, 0.445363, 0.439383,\
        0.433502, 0.427796, 0.422328, 0.417099, 0.412051, 0.406965, 0.401452, 0.395773, 0.390168, 0.384573,\
        0.378963, 0.373412, 0.368033, 0.362879, 0.357922, 0.353120, 0.348444, 0.343884, 0.339430, 0.335071,\
        0.330794, 0.326584, 0.322430, 0.318327, 0.314259, 0.310172, 0.306037, 0.301944, 0.298078, 0.294326,\
        0.290589, 0.286995, 0.283511, 0.280033, 0.276441, 0.272626, 0.268511, 0.264051, 0.259227, 0.254034,\
        0.248475],\
        [0.200905, 0.209047, 0.217183, 0.225125, 0.232804, 0.240163, 0.247154, 0.253752, 0.259964, 0.265829,\
        0.271413, 0.276791, 0.282026, 0.287140, 0.292026, 0.296355, 0.299977, 0.303170, 0.306690, 0.310864,\
        0.315270, 0.319624, 0.323855, 0.327998, 0.332127, 0.336313, 0.340593, 0.344986, 0.349507, 0.354178,\
        0.359024, 0.364090, 0.369442, 0.375093, 0.380882, 0.386503, 0.391784, 0.396867, 0.401971, 0.407134,\
        0.412298, 0.417499, 0.422892, 0.428476, 0.433988, 0.439144, 0.443891, 0.448418, 0.453104, 0.458494,\
        0.465118, 0.472989, 0.481351, 0.489376, 0.497162, 0.505520, 0.513917, 0.521459, 0.527552, 0.531998,\
        0.534156, 0.533853, 0.531693, 0.528178, 0.523725, 0.518828, 0.513524, 0.507304, 0.500531, 0.493705,\
        0.486741, 0.479524, 0.472376, 0.465579, 0.459022, 0.452742, 0.446746, 0.440928, 0.435170, 0.429408,\
        0.423682, 0.418129, 0.412862, 0.407868, 0.403101, 0.398430, 0.393203, 0.387759, 0.382345, 0.376972,\
        0.371635, 0.366374, 0.361249, 0.356297, 0.351520, 0.346898, 0.342412, 0.338046, 0.333787, 0.329625,\
        0.325545, 0.321533, 0.317582, 0.313699, 0.309905, 0.306115, 0.302154, 0.298224, 0.294455, 0.290807,\
        0.287206, 0.283665, 0.280164, 0.276636, 0.272993, 0.269149, 0.265034, 0.260602, 0.255825, 0.250689,\
        0.245190],\
        [0.199524, 0.207061, 0.214631, 0.222103, 0.229396, 0.236436, 0.243166, 0.249558, 0.255612, 0.261353,\
        0.266825, 0.272075, 0.277130, 0.281969, 0.286486, 0.290510, 0.293970, 0.297079, 0.300532, 0.304878,\
        0.309264, 0.313458, 0.317444, 0.321303, 0.325176, 0.329164, 0.333283, 0.337523, 0.341882, 0.346375,\
        0.351031, 0.355884, 0.360954, 0.366210, 0.371529, 0.376742, 0.381759, 0.386638, 0.391496, 0.396388,\
        0.401266, 0.406135, 0.411384, 0.417113, 0.422866, 0.428143, 0.432953, 0.437549, 0.442278, 0.447543,\
        0.453644, 0.460558, 0.467903, 0.475291, 0.482767, 0.490572, 0.498553, 0.505672, 0.511431, 0.516372,\
        0.518832, 0.518245, 0.516107, 0.512696, 0.508169, 0.503429, 0.498562, 0.492562, 0.485998, 0.479471,\
        0.472987, 0.466529, 0.460191, 0.454067, 0.448134, 0.442368, 0.436750, 0.431237, 0.425749, 0.420171,\
        0.414495, 0.408992, 0.403861, 0.399064, 0.394429, 0.389849, 0.384983, 0.379881, 0.374726, 0.369592,\
        0.364509, 0.359509, 0.354627, 0.349893, 0.345315, 0.340887, 0.336595, 0.332426, 0.328366, 0.324402,\
        0.320524, 0.316716, 0.312957, 0.309246, 0.305691, 0.302359, 0.298630, 0.294803, 0.291103, 0.287507,\
        0.283972, 0.280467, 0.276961, 0.273400, 0.269717, 0.265841, 0.261714, 0.257289, 0.252535, 0.247433,\
        0.241970],\
        [0.198243, 0.205435, 0.212447, 0.219423, 0.226312, 0.233010, 0.239444, 0.245580, 0.251413, 0.256956,\
        0.262238, 0.267282, 0.272096, 0.276652, 0.280881, 0.284712, 0.288169, 0.291482, 0.295140, 0.299333,\
        0.303552, 0.307510, 0.311210, 0.314725, 0.318336, 0.322171, 0.326175, 0.330288, 0.334498, 0.338820,\
        0.343282, 0.347911, 0.352710, 0.357640, 0.362617, 0.367540, 0.372355, 0.377083, 0.381777, 0.386471,\
        0.391151, 0.395870, 0.401025, 0.407009, 0.413040, 0.418213, 0.422898, 0.427378, 0.431913, 0.436825,\
        0.442361, 0.448554, 0.455241, 0.462228, 0.469455, 0.476902, 0.484306, 0.490899, 0.496585, 0.502209,\
        0.504779, 0.503175, 0.501049, 0.497919, 0.493261, 0.488529, 0.483573, 0.477959, 0.471879, 0.465788,\
        0.459864, 0.454137, 0.448596, 0.443196, 0.437878, 0.432600, 0.427347, 0.422139, 0.416980, 0.411676,\
        0.405991, 0.400442, 0.395451, 0.390749, 0.386199, 0.381665, 0.377000, 0.372184, 0.367300, 0.362418,\
        0.357583, 0.352827, 0.348181, 0.343667, 0.339297, 0.335071, 0.330981, 0.327013, 0.323153, 0.319389,\
        0.315715, 0.312122, 0.308575, 0.304998, 0.301457, 0.298384, 0.295065, 0.291448, 0.287859, 0.284335,\
        0.280851, 0.277376, 0.273871, 0.270290, 0.266576, 0.262673, 0.258528, 0.254097, 0.249350, 0.244261,\
        0.238817],\
        [0.196954, 0.203770, 0.210276, 0.216904, 0.223463, 0.229833, 0.235954, 0.241799, 0.247361, 0.252650,\
        0.257685, 0.262479, 0.267036, 0.271337, 0.275357, 0.279088, 0.282603, 0.286093, 0.289811, 0.293811,\
        0.297825, 0.301617, 0.305081, 0.308159, 0.311560, 0.315352, 0.319295, 0.323298, 0.327361, 0.331508,\
        0.335775, 0.340188, 0.344751, 0.349431, 0.354164, 0.358885, 0.363555, 0.368176, 0.372776, 0.377389,\
        0.382057, 0.386912, 0.392242, 0.398509, 0.404230, 0.408969, 0.413448, 0.417721, 0.421909, 0.426353,\
        0.431346, 0.436987, 0.443223, 0.449910, 0.456888, 0.463984, 0.470913, 0.477309, 0.483134, 0.488277,\
        0.490321, 0.488918, 0.486539, 0.483292, 0.478965, 0.474191, 0.469136, 0.463723, 0.458074, 0.452483,\
        0.447175, 0.442214, 0.437500, 0.432851, 0.428143, 0.423333, 0.418398, 0.413409, 0.408582, 0.403802,\
        0.398251, 0.392684, 0.387648, 0.382932, 0.378387, 0.373892, 0.369357, 0.364750, 0.360103, 0.355459,\
        0.350860, 0.346337, 0.341917, 0.337621, 0.333462, 0.329443, 0.325561, 0.321800, 0.318141, 0.314571,\
        0.311090, 0.307712, 0.304435, 0.301102, 0.297676, 0.294528, 0.291384, 0.288049, 0.284639, 0.281221,\
        0.277803, 0.274363, 0.270868, 0.267277, 0.263543, 0.259616, 0.255451, 0.251008, 0.246254, 0.241165,\
        0.235723],\
        [0.195033, 0.201398, 0.208012, 0.214529, 0.220825, 0.226879, 0.232673, 0.238197, 0.243452, 0.248448,\
        0.253200, 0.257721, 0.262019, 0.266093, 0.269945, 0.273601, 0.277136, 0.280676, 0.284343, 0.288142,\
        0.291940, 0.295599, 0.298814, 0.301508, 0.304930, 0.308806, 0.312687, 0.316566, 0.320471, 0.324437,\
        0.328509, 0.332726, 0.337097, 0.341593, 0.346162, 0.350749, 0.355321, 0.359874, 0.364431, 0.369037,\
        0.373769, 0.378766, 0.384211, 0.390038, 0.395315, 0.399925, 0.404332, 0.408432, 0.412160, 0.416098,\
        0.420611, 0.425807, 0.431727, 0.438194, 0.444936, 0.451704, 0.458250, 0.464326, 0.469693, 0.473784,\
        0.475516, 0.474759, 0.472479, 0.469166, 0.464991, 0.460275, 0.455248, 0.450002, 0.444671, 0.439509,\
        0.434803, 0.430654, 0.426843, 0.422966, 0.418820, 0.414461, 0.409834, 0.404890, 0.400189, 0.395703,\
        0.390595, 0.385265, 0.380228, 0.375488, 0.370940, 0.366484, 0.362049, 0.357605, 0.353157, 0.348729,\
        0.344351, 0.340049, 0.335847, 0.331761, 0.327808, 0.323998, 0.320328, 0.316781, 0.313327, 0.309944,\
        0.306634, 0.303420, 0.300357, 0.297308, 0.294095, 0.290968, 0.287873, 0.284700, 0.281448, 0.278146,\
        0.274802, 0.271406, 0.267929, 0.264337, 0.260590, 0.256645, 0.252461, 0.248001, 0.243233, 0.238133,\
        0.232680],\
        [0.193352, 0.199787, 0.206206, 0.212443, 0.218418, 0.224132, 0.229581, 0.234762, 0.239682, 0.244356,\
        0.248801, 0.253036, 0.257077, 0.260936, 0.264631, 0.268197, 0.271693, 0.275195, 0.278757, 0.282368,\
        0.285944, 0.289348, 0.292408, 0.295303, 0.298820, 0.302605, 0.306342, 0.310075, 0.313818, 0.317597,\
        0.321478, 0.325523, 0.329748, 0.334124, 0.338592, 0.343097, 0.347605, 0.352111, 0.356638, 0.361234,\
        0.365972, 0.370938, 0.376163, 0.381463, 0.386448, 0.391017, 0.395331, 0.399285, 0.402516, 0.406078,\
        0.410223, 0.415001, 0.420738, 0.427071, 0.433580, 0.439998, 0.446118, 0.451710, 0.456450, 0.459832,\
        0.461324, 0.460842, 0.458778, 0.455542, 0.451447, 0.446812, 0.441904, 0.436866, 0.431780, 0.426902,\
        0.422684, 0.419376, 0.416608, 0.413509, 0.409749, 0.405766, 0.401598, 0.396738, 0.392042, 0.387517,\
        0.382747, 0.377797, 0.372940, 0.368272, 0.363771, 0.359378, 0.355044, 0.350742, 0.346469, 0.342237,\
        0.338065, 0.333973, 0.329978, 0.326095, 0.322340, 0.318730, 0.315275, 0.311954, 0.308713, 0.305518,\
        0.302373, 0.299309, 0.296377, 0.293483, 0.290508, 0.287510, 0.284500, 0.281438, 0.278307, 0.275106,\
        0.271837, 0.268487, 0.265034, 0.261449, 0.257695, 0.253736, 0.249534, 0.245055, 0.240269, 0.235148,\
        0.229674],\
        [0.192177, 0.198659, 0.204781, 0.210610, 0.216209, 0.221566, 0.226658, 0.231480, 0.236045, 0.240374,\
        0.244495, 0.248434, 0.252215, 0.255860, 0.259393, 0.262845, 0.266259, 0.269676, 0.273117, 0.276564,\
        0.279958, 0.283222, 0.286331, 0.289469, 0.292907, 0.296536, 0.300155, 0.303775, 0.307377, 0.310965,\
        0.314664, 0.318568, 0.322701, 0.327018, 0.331438, 0.335898, 0.340359, 0.344816, 0.349293, 0.353833,\
        0.358486, 0.363289, 0.368210, 0.373112, 0.377802, 0.382190, 0.386266, 0.389908, 0.393125, 0.396552,\
        0.400359, 0.404650, 0.410437, 0.416640, 0.422854, 0.428843, 0.434470, 0.439546, 0.443740, 0.446642,\
        0.447927, 0.447532, 0.445626, 0.442483, 0.438439, 0.433871, 0.429141, 0.424440, 0.419622, 0.414821,\
        0.410869, 0.408317, 0.406746, 0.404480, 0.400756, 0.396975, 0.393026, 0.388592, 0.384047, 0.379552,\
        0.374985, 0.370337, 0.365718, 0.361205, 0.356813, 0.352521, 0.348303, 0.344141, 0.340033, 0.335985,\
        0.332009, 0.328118, 0.324325, 0.320637, 0.317065, 0.313635, 0.310390, 0.307312, 0.304299, 0.301289,\
        0.298315, 0.295415, 0.292596, 0.289807, 0.286984, 0.284120, 0.281214, 0.278252, 0.275218, 0.272104,\
        0.268901, 0.265596, 0.262169, 0.258594, 0.254838, 0.250869, 0.246651, 0.242152, 0.237343, 0.232197,\
        0.226693],\
        [0.191010, 0.197413, 0.203334, 0.208830, 0.214115, 0.219145, 0.223885, 0.228338, 0.232530, 0.236497,\
        0.240278, 0.243911, 0.247427, 0.250854, 0.254216, 0.257539, 0.260847, 0.264160, 0.267485, 0.270803,\
        0.274082, 0.277292, 0.280454, 0.283673, 0.287045, 0.290538, 0.294068, 0.297612, 0.301096, 0.304515,\
        0.308058, 0.311862, 0.315966, 0.320282, 0.324695, 0.329128, 0.333540, 0.337929, 0.342318, 0.346747,\
        0.351249, 0.355831, 0.360453, 0.365020, 0.369416, 0.373560, 0.377403, 0.380914, 0.384203, 0.387570,\
        0.391197, 0.395561, 0.401091, 0.406954, 0.412767, 0.418207, 0.423299, 0.427918, 0.431710, 0.434263,\
        0.435361, 0.434992, 0.433204, 0.430124, 0.426079, 0.421526, 0.416952, 0.412830, 0.408598, 0.403613,\
        0.399905, 0.397744, 0.396837, 0.395139, 0.391786, 0.388136, 0.384325, 0.380241, 0.375990, 0.371687,\
        0.367338, 0.362954, 0.358581, 0.354267, 0.350033, 0.345880, 0.341799, 0.337786, 0.333842, 0.329973,\
        0.326186, 0.322492, 0.318900, 0.315409, 0.312012, 0.308722, 0.305651, 0.302839, 0.300049, 0.297227,\
        0.294419, 0.291665, 0.288959, 0.286269, 0.283559, 0.280806, 0.277999, 0.275126, 0.272174, 0.269131,\
        0.265985, 0.262721, 0.259320, 0.255755, 0.252000, 0.248023, 0.243791, 0.239273, 0.234439, 0.229262,\
        0.223722],\
        [0.189742, 0.195938, 0.201606, 0.207009, 0.212116, 0.216862, 0.221253, 0.225325, 0.229125, 0.232711,\
        0.236138, 0.239455, 0.242702, 0.245909, 0.249096, 0.252281, 0.255475, 0.258686, 0.261910, 0.265134,\
        0.268341, 0.271523, 0.274705, 0.277936, 0.281257, 0.284657, 0.288094, 0.291531, 0.294915, 0.298256,\
        0.301701, 0.305444, 0.309589, 0.313947, 0.318367, 0.322771, 0.327115, 0.331400, 0.335660, 0.339934,\
        0.344245, 0.348591, 0.352935, 0.357209, 0.361336, 0.365258, 0.368940, 0.372400, 0.375745, 0.379173,\
        0.382919, 0.387300, 0.392393, 0.397862, 0.403193, 0.407980, 0.412564, 0.416862, 0.420425, 0.422690,\
        0.423561, 0.423238, 0.421627, 0.418587, 0.414485, 0.409943, 0.405351, 0.401766, 0.398731, 0.393999,\
        0.390391, 0.388226, 0.386875, 0.385127, 0.382426, 0.379159, 0.375611, 0.371839, 0.367903, 0.363875,\
        0.359790, 0.355675, 0.351557, 0.347468, 0.343426, 0.339440, 0.335517, 0.331664, 0.327889, 0.324200,\
        0.320600, 0.317100, 0.313712, 0.310433, 0.307236, 0.304067, 0.301047, 0.298488, 0.295905, 0.293274,\
        0.290640, 0.288027, 0.285434, 0.282842, 0.280226, 0.277565, 0.274843, 0.272047, 0.269164, 0.266180,\
        0.263081, 0.259851, 0.256471, 0.252917, 0.249163, 0.245180, 0.240935, 0.236398, 0.231537, 0.226327,\
        0.220744],\
        [0.188486, 0.194507, 0.200063, 0.205389, 0.210301, 0.214743, 0.218764, 0.222430, 0.225815, 0.228999,\
        0.232057, 0.235051, 0.238027, 0.241015, 0.244031, 0.247081, 0.250166, 0.253284, 0.256426, 0.259580,\
        0.262738, 0.265899, 0.269078, 0.272300, 0.275581, 0.278915, 0.282276, 0.285636, 0.288972, 0.292308,\
        0.295743, 0.299455, 0.303681, 0.308043, 0.312449, 0.316807, 0.321049, 0.325186, 0.329274, 0.333361,\
        0.337467, 0.341582, 0.345674, 0.349695, 0.353593, 0.357331, 0.360900, 0.364334, 0.367728, 0.371234,\
        0.375038, 0.379300, 0.384025, 0.388992, 0.393760, 0.398101, 0.402288, 0.406396, 0.409945, 0.411867,\
        0.412322, 0.412142, 0.410936, 0.407917, 0.403700, 0.399298, 0.394996, 0.391453, 0.388396, 0.384761,\
        0.381500, 0.379142, 0.377339, 0.375431, 0.373018, 0.370130, 0.366914, 0.363467, 0.359855, 0.356135,\
        0.352346, 0.348517, 0.344671, 0.340827, 0.337001, 0.333205, 0.329454, 0.325770, 0.322173, 0.318672,\
        0.315259, 0.311945, 0.308759, 0.305698, 0.302752, 0.299811, 0.296905, 0.294359, 0.291917, 0.289453,\
        0.286973, 0.284490, 0.282004, 0.279503, 0.276969, 0.274385, 0.271734, 0.269002, 0.266175, 0.263238,\
        0.260176, 0.256973, 0.253609, 0.250062, 0.246309, 0.242319, 0.238063, 0.233508, 0.228621, 0.223374,\
        0.217743],\
        [0.187373, 0.193452, 0.199001, 0.204133, 0.208724, 0.212791, 0.216405, 0.219637, 0.222580, 0.225339,\
        0.228013, 0.230679, 0.233389, 0.236168, 0.239024, 0.241952, 0.244940, 0.247979, 0.251054, 0.254156,\
        0.257277, 0.260419, 0.263587, 0.266795, 0.270048, 0.273342, 0.276664, 0.279997, 0.283344, 0.286739,\
        0.290272, 0.294087, 0.298231, 0.302532, 0.306909, 0.311205, 0.315293, 0.319227, 0.323111, 0.326999,\
        0.330903, 0.334805, 0.338675, 0.342477, 0.346182, 0.349771, 0.353248, 0.356653, 0.360065, 0.363596,\
        0.367367, 0.371454, 0.375827, 0.380314, 0.384667, 0.388768, 0.392701, 0.396609, 0.400109, 0.401571,\
        0.401402, 0.401466, 0.400881, 0.397883, 0.393736, 0.389479, 0.385462, 0.381909, 0.378706, 0.375542,\
        0.372635, 0.370226, 0.368178, 0.366143, 0.363853, 0.361232, 0.358325, 0.355197, 0.351908, 0.348506,\
        0.345029, 0.341503, 0.337945, 0.334366, 0.330776, 0.327188, 0.323619, 0.320104, 0.316693, 0.313404,\
        0.310184, 0.307031, 0.304041, 0.301184, 0.298428, 0.295723, 0.293041, 0.290534, 0.288147, 0.285788,\
        0.283425, 0.281049, 0.278656, 0.276235, 0.273772, 0.271252, 0.268658, 0.265977, 0.263193, 0.260292,\
        0.257257, 0.254071, 0.250717, 0.247173, 0.243416, 0.239420, 0.235154, 0.230583, 0.225671, 0.220386,\
        0.214702],\
        [0.186296, 0.192515, 0.198202, 0.203093, 0.207260, 0.210946, 0.214147, 0.216923, 0.219395, 0.221705,\
        0.223981, 0.226321, 0.228778, 0.231369, 0.234086, 0.236910, 0.239819, 0.242793, 0.245816, 0.248878,\
        0.251970, 0.255093, 0.258250, 0.261446, 0.264684, 0.267962, 0.271274, 0.274620, 0.278012, 0.281490,\
        0.285122, 0.288984, 0.293079, 0.297332, 0.301688, 0.305906, 0.309756, 0.313438, 0.317111, 0.320814,\
        0.324539, 0.328255, 0.331931, 0.335544, 0.339082, 0.342541, 0.345932, 0.349290, 0.352678, 0.356173,\
        0.359849, 0.363739, 0.367802, 0.371923, 0.375962, 0.379841, 0.383567, 0.387097, 0.389993, 0.391276,\
        0.391286, 0.391128, 0.390305, 0.387783, 0.384059, 0.380081, 0.376251, 0.372736, 0.369526, 0.366541,\
        0.363827, 0.361443, 0.359304, 0.357219, 0.355007, 0.352580, 0.349932, 0.347097, 0.344116, 0.341029,\
        0.337870, 0.334657, 0.331402, 0.328107, 0.324773, 0.321410, 0.318032, 0.314674, 0.311433, 0.308412,\
        0.305441, 0.302379, 0.299578, 0.296912, 0.294317, 0.291776, 0.289282, 0.286878, 0.284555, 0.282271,\
        0.279991, 0.277698, 0.275379, 0.273024, 0.270619, 0.268149, 0.265601, 0.262958, 0.260206, 0.257329,\
        0.254310, 0.251133, 0.247779, 0.244229, 0.240464, 0.236459, 0.232184, 0.227602, 0.222669, 0.217345,\
        0.211604],\
        [0.185070, 0.191343, 0.196946, 0.201633, 0.205705, 0.209149, 0.211967, 0.214269, 0.216235, 0.218067,\
        0.219937, 0.221962, 0.224192, 0.226626, 0.229235, 0.231980, 0.234828, 0.237752, 0.240733, 0.243761,\
        0.246829, 0.249935, 0.253081, 0.256269, 0.259502, 0.262779, 0.266102, 0.269477, 0.272921, 0.276469,\
        0.280164, 0.284046, 0.288114, 0.292331, 0.296626, 0.300742, 0.304254, 0.307716, 0.311222, 0.314782,\
        0.318367, 0.321928, 0.325430, 0.328872, 0.332261, 0.335601, 0.338901, 0.342189, 0.345511, 0.348922,\
        0.352466, 0.356155, 0.359949, 0.363769, 0.367520, 0.371127, 0.374522, 0.377554, 0.379868, 0.381078,\
        0.381311, 0.380936, 0.379794, 0.377535, 0.374349, 0.370766, 0.367173, 0.363771, 0.360627, 0.357749,\
        0.355149, 0.352819, 0.350687, 0.348621, 0.346493, 0.344226, 0.341795, 0.339218, 0.336523, 0.333739,\
        0.330893, 0.328000, 0.325064, 0.322070, 0.319008, 0.315892, 0.312737, 0.309526, 0.306357, 0.303638,\
        0.301124, 0.298153, 0.295425, 0.292883, 0.290416, 0.288004, 0.285643, 0.283340, 0.281089, 0.278868,\
        0.276652, 0.274420, 0.272159, 0.269854, 0.267493, 0.265061, 0.262545, 0.259928, 0.257197, 0.254333,\
        0.251321, 0.248142, 0.244778, 0.241213, 0.237429, 0.233409, 0.229126, 0.224540, 0.219595, 0.214235,\
        0.208429],\
        [0.183613, 0.189898, 0.195405, 0.200130, 0.204197, 0.207445, 0.209874, 0.211659, 0.213072, 0.214395,\
        0.215858, 0.217595, 0.219641, 0.221963, 0.224501, 0.227194, 0.229997, 0.232881, 0.235827, 0.238824,\
        0.241867, 0.244956, 0.248091, 0.251273, 0.254506, 0.257791, 0.261131, 0.264537, 0.268022, 0.271613,\
        0.275334, 0.279202, 0.283209, 0.287311, 0.291401, 0.295229, 0.298682, 0.302036, 0.305444, 0.308904,\
        0.312397, 0.315826, 0.319157, 0.322433, 0.325684, 0.328912, 0.332115, 0.335308, 0.338529, 0.341819,\
        0.345209, 0.348698, 0.352250, 0.355798, 0.359261, 0.362554, 0.365573, 0.368156, 0.370073, 0.371143,\
        0.371381, 0.370888, 0.369631, 0.367534, 0.364725, 0.361508, 0.358178, 0.354939, 0.351902, 0.349124,\
        0.346626, 0.344385, 0.342326, 0.340338, 0.338316, 0.336192, 0.333948, 0.331598, 0.329163, 0.326664,\
        0.324118, 0.321544, 0.318941, 0.316271, 0.313487, 0.310622, 0.307759, 0.304822, 0.301584, 0.298970,\
        0.296748, 0.294147, 0.291537, 0.289067, 0.286693, 0.284381, 0.282118, 0.279900, 0.277715, 0.275549,\
        0.273382, 0.271196, 0.268976, 0.266707, 0.264377, 0.261970, 0.259474, 0.256873, 0.254151, 0.251291,\
        0.248275, 0.245085, 0.241701, 0.238106, 0.234289, 0.230240, 0.225943, 0.221364, 0.216429, 0.211041,\
        0.205159],\
        [0.181916, 0.188316, 0.193973, 0.198828, 0.202857, 0.205875, 0.207864, 0.209072, 0.209875, 0.210665,\
        0.211735, 0.213231, 0.215158, 0.217427, 0.219933, 0.222595, 0.225364, 0.228211, 0.231119, 0.234082,\
        0.237097, 0.240165, 0.243286, 0.246463, 0.249696, 0.252989, 0.256345, 0.259772, 0.263281, 0.266887,\
        0.270600, 0.274421, 0.278329, 0.282267, 0.286133, 0.289791, 0.293211, 0.296522, 0.299831, 0.303212,\
        0.306660, 0.309956, 0.313082, 0.316189, 0.319322, 0.322451, 0.325550, 0.328623, 0.331709, 0.334849,\
        0.338070, 0.341363, 0.344689, 0.347983, 0.351163, 0.354139, 0.356801, 0.359011, 0.360618, 0.361511,\
        0.361665, 0.361101, 0.359825, 0.357858, 0.355313, 0.352389, 0.349302, 0.346235, 0.343320, 0.340651,\
        0.338271, 0.336161, 0.334238, 0.332382, 0.330487, 0.328496, 0.326411, 0.324259, 0.322064, 0.319829,\
        0.317558, 0.315285, 0.313031, 0.310722, 0.308214, 0.305546, 0.302948, 0.300550, 0.297581, 0.294846,\
        0.292521, 0.290149, 0.287737, 0.285380, 0.283094, 0.280867, 0.278683, 0.276533, 0.274405, 0.272285,\
        0.270156, 0.268003, 0.265811, 0.263565, 0.261252, 0.258858, 0.256370, 0.253773, 0.251051, 0.248187,\
        0.245161, 0.241951, 0.238536, 0.234898, 0.231025, 0.226919, 0.222586, 0.218019, 0.213139, 0.207760,\
        0.201778],\
        [0.179912, 0.186493, 0.192379, 0.197479, 0.201568, 0.204346, 0.205875, 0.206459, 0.206616, 0.206873,\
        0.207593, 0.208924, 0.210820, 0.213098, 0.215598, 0.218235, 0.220966, 0.223768, 0.226630, 0.229550,\
        0.232530, 0.235570, 0.238673, 0.241839, 0.245069, 0.248365, 0.251729, 0.255166, 0.258681, 0.262279,\
        0.265959, 0.269710, 0.273504, 0.277287, 0.280986, 0.284531, 0.287915, 0.291199, 0.294460, 0.297784,\
        0.301178, 0.304282, 0.307135, 0.310105, 0.313157, 0.316206, 0.319193, 0.322116, 0.325031, 0.327999,\
        0.331043, 0.334149, 0.337268, 0.340326, 0.343240, 0.345918, 0.348261, 0.350159, 0.351505, 0.352221,\
        0.352270, 0.351653, 0.350391, 0.348535, 0.346187, 0.343493, 0.340615, 0.337704, 0.334903, 0.332339,\
        0.330095, 0.328168, 0.326450, 0.324781, 0.323033, 0.321161, 0.319198, 0.317214, 0.315247, 0.313267,\
        0.311234, 0.309204, 0.307294, 0.305422, 0.303207, 0.300676, 0.298236, 0.295979, 0.293481, 0.290934,\
        0.288570, 0.286282, 0.284009, 0.281769, 0.279575, 0.277425, 0.275309, 0.273216, 0.271134, 0.269050,\
        0.266950, 0.264819, 0.262642, 0.260406, 0.258098, 0.255705, 0.253214, 0.250611, 0.247881, 0.245005,\
        0.241963, 0.238731, 0.235281, 0.231587, 0.227634, 0.223425, 0.218999, 0.214407, 0.209631, 0.204417,\
        0.198322],\
        [0.177483, 0.184231, 0.190315, 0.195607, 0.199833, 0.202614, 0.203752, 0.203747, 0.203299, 0.203097,\
        0.203549, 0.204801, 0.206766, 0.209088, 0.211572, 0.214163, 0.216837, 0.219576, 0.222374, 0.225237,\
        0.228170, 0.231175, 0.234253, 0.237403, 0.240624, 0.243915, 0.247278, 0.250712, 0.254216, 0.257788,\
        0.261419, 0.265092, 0.268776, 0.272429, 0.276001, 0.279456, 0.282793, 0.286047, 0.289273, 0.292520,\
        0.295742, 0.298601, 0.301266, 0.304204, 0.307201, 0.310186, 0.313046, 0.315769, 0.318476, 0.321255,\
        0.324126, 0.327062, 0.329996, 0.332844, 0.335516, 0.337924, 0.339986, 0.341619, 0.342743, 0.343298,\
        0.343247, 0.342589, 0.341347, 0.339585, 0.337396, 0.334895, 0.332199, 0.329423, 0.326706, 0.324217,\
        0.322116, 0.320432, 0.319001, 0.317580, 0.315996, 0.314213, 0.312323, 0.310470, 0.308732, 0.307023,\
        0.305198, 0.303308, 0.301669, 0.300211, 0.298378, 0.296028, 0.293752, 0.291550, 0.289285, 0.286974,\
        0.284715, 0.282514, 0.280347, 0.278209, 0.276102, 0.274023, 0.271966, 0.269922, 0.267878, 0.265822,\
        0.263741, 0.261621, 0.259449, 0.257212, 0.254898, 0.252493, 0.249988, 0.247368, 0.244621, 0.241727,\
        0.238667, 0.235414, 0.231933, 0.228187, 0.224138, 0.219779, 0.215161, 0.210409, 0.205622, 0.200652,\
        0.194592],\
        [0.174518, 0.181365, 0.187549, 0.192941, 0.197326, 0.200174, 0.201212, 0.200836, 0.200068, 0.199629,\
        0.199926, 0.201131, 0.203210, 0.205515, 0.207904, 0.210404, 0.212995, 0.215646, 0.218358, 0.221146,\
        0.224020, 0.226982, 0.230028, 0.233155, 0.236358, 0.239636, 0.242988, 0.246407, 0.249889, 0.253425,\
        0.256999, 0.260592, 0.264176, 0.267718, 0.271184, 0.274555, 0.277829, 0.281027, 0.284175, 0.287279,\
        0.290270, 0.293041, 0.295729, 0.298564, 0.301457, 0.304402, 0.307101, 0.309545, 0.312016, 0.314612,\
        0.317325, 0.320114, 0.322896, 0.325561, 0.328014, 0.330178, 0.331993, 0.333398, 0.334335, 0.334754,\
        0.334622, 0.333931, 0.332709, 0.331023, 0.328971, 0.326652, 0.324140, 0.321494, 0.318822, 0.316344,\
        0.314356, 0.312985, 0.311940, 0.310823, 0.309414, 0.307683, 0.305813, 0.304037, 0.302529, 0.301106,\
        0.299509, 0.297780, 0.296294, 0.294955, 0.293344, 0.291374, 0.289305, 0.287231, 0.285129, 0.283001,\
        0.280883, 0.278790, 0.276722, 0.274675, 0.272647, 0.270634, 0.268630, 0.266626, 0.264613, 0.262577,\
        0.260506, 0.258389, 0.256212, 0.253962, 0.251630, 0.249203, 0.246671, 0.244023, 0.241249, 0.238331,\
        0.235252, 0.231985, 0.228491, 0.224716, 0.220595, 0.216074, 0.211176, 0.206082, 0.201056, 0.195914,\
        0.190030],\
        [0.170964, 0.177829, 0.184003, 0.189347, 0.193631, 0.196534, 0.197758, 0.197635, 0.197109, 0.196797,\
        0.197122, 0.198314, 0.200268, 0.202376, 0.204561, 0.206953, 0.209445, 0.211979, 0.214576, 0.217271,\
        0.220078, 0.222992, 0.226000, 0.229095, 0.232272, 0.235528, 0.238859, 0.242256, 0.245708, 0.249200,\
        0.252714, 0.256229, 0.259720, 0.263162, 0.266531, 0.269814, 0.273009, 0.276125, 0.279170, 0.282135,\
        0.284988, 0.287716, 0.290400, 0.293134, 0.295932, 0.298809, 0.301236, 0.303372, 0.305650, 0.308090,\
        0.310658, 0.313333, 0.315996, 0.318503, 0.320753, 0.322690, 0.324280, 0.325489, 0.326275, 0.326591,\
        0.326401, 0.325691, 0.324484, 0.322856, 0.320924, 0.318797, 0.316515, 0.314041, 0.311401, 0.308850,\
        0.306864, 0.305897, 0.305309, 0.304486, 0.303274, 0.301566, 0.299751, 0.298056, 0.296698, 0.295427,\
        0.294043, 0.292577, 0.291202, 0.289874, 0.288396, 0.286696, 0.284856, 0.282953, 0.281009, 0.279037,\
        0.277055, 0.275077, 0.273107, 0.271145, 0.269189, 0.267235, 0.265277, 0.263307, 0.261316, 0.259293,\
        0.257226, 0.255102, 0.252910, 0.250639, 0.248276, 0.245814, 0.243243, 0.240555, 0.237742, 0.234792,\
        0.231689, 0.228412, 0.224927, 0.221176, 0.217059, 0.212456, 0.207320, 0.201785, 0.196523, 0.191154,\
        0.185180],\
        [0.166829, 0.173654, 0.179751, 0.184982, 0.189156, 0.192064, 0.193612, 0.194085, 0.194083, 0.194153,\
        0.194680, 0.195849, 0.197561, 0.199434, 0.201451, 0.203808, 0.206183, 0.208554, 0.211004, 0.213597,\
        0.216340, 0.219206, 0.222171, 0.225225, 0.228366, 0.231592, 0.234895, 0.238263, 0.241679, 0.245124,\
        0.248577, 0.252015, 0.255418, 0.258764, 0.262037, 0.265226, 0.268329, 0.271347, 0.274283, 0.277131,\
        0.279883, 0.282550, 0.285179, 0.287815, 0.290466, 0.293040, 0.295296, 0.297320, 0.299473, 0.301759,\
        0.304168, 0.306758, 0.309330, 0.311688, 0.313738, 0.315451, 0.316833, 0.317876, 0.318545, 0.318794,\
        0.318580, 0.317871, 0.316678, 0.315082, 0.313243, 0.311320, 0.309353, 0.307169, 0.304608, 0.302034,\
        0.299907, 0.299333, 0.299051, 0.298362, 0.297330, 0.295776, 0.294128, 0.292598, 0.291275, 0.290054,\
        0.288812, 0.287535, 0.286267, 0.284987, 0.283608, 0.282084, 0.280435, 0.278701, 0.276910, 0.275079,\
        0.273224, 0.271356, 0.269479, 0.267596, 0.265705, 0.263803, 0.261885, 0.259943, 0.257968, 0.255951,\
        0.253879, 0.251742, 0.249527, 0.247223, 0.244820, 0.242310, 0.239686, 0.236943, 0.234077, 0.231080,\
        0.227941, 0.224647, 0.221181, 0.217506, 0.213511, 0.208985, 0.203789, 0.198146, 0.192424, 0.186595,\
        0.180325],\
        [0.162167, 0.168930, 0.174943, 0.180089, 0.184230, 0.187259, 0.189187, 0.190242, 0.190828, 0.191359,\
        0.192148, 0.193344, 0.194903, 0.196677, 0.198698, 0.200973, 0.203156, 0.205320, 0.207604, 0.210112,\
        0.212812, 0.215636, 0.218549, 0.221549, 0.224642, 0.227829, 0.231099, 0.234435, 0.237813, 0.241208,\
        0.244598, 0.247960, 0.251275, 0.254525, 0.257698, 0.260786, 0.263787, 0.266700, 0.269526, 0.272264,\
        0.274918, 0.277501, 0.280036, 0.282542, 0.285002, 0.287346, 0.289492, 0.291507, 0.293556, 0.295692,\
        0.297967, 0.300447, 0.302905, 0.305099, 0.306945, 0.308432, 0.309623, 0.310534, 0.311120, 0.311337,\
        0.311136, 0.310462, 0.309297, 0.307709, 0.305926, 0.304183, 0.302551, 0.300834, 0.298429, 0.296063,\
        0.294271, 0.293423, 0.292985, 0.292368, 0.291428, 0.290157, 0.288749, 0.287385, 0.286134, 0.284963,\
        0.283810, 0.282646, 0.281468, 0.280254, 0.278963, 0.277566, 0.276065, 0.274477, 0.272824, 0.271121,\
        0.269380, 0.267612, 0.265821, 0.264010, 0.262178, 0.260321, 0.258435, 0.256514, 0.254549, 0.252531,\
        0.250449, 0.248291, 0.246045, 0.243701, 0.241247, 0.238676, 0.235983, 0.233168, 0.230231, 0.227171,\
        0.223976, 0.220635, 0.217153, 0.213551, 0.209776, 0.205523, 0.200321, 0.194491, 0.188384, 0.182071,\
        0.175425],\
        [0.157054, 0.163762, 0.169723, 0.174845, 0.179043, 0.182273, 0.184590, 0.186175, 0.187313, 0.188306,\
        0.189397, 0.190716, 0.192283, 0.194066, 0.196061, 0.198180, 0.200232, 0.202206, 0.204339, 0.206828,\
        0.209525, 0.212303, 0.215142, 0.218065, 0.221096, 0.224238, 0.227475, 0.230779, 0.234118, 0.237463,\
        0.240788, 0.244070, 0.247293, 0.250442, 0.253510, 0.256489, 0.259380, 0.262181, 0.264894, 0.267522,\
        0.270070, 0.272548, 0.274968, 0.277333, 0.279626, 0.281813, 0.283874, 0.285855, 0.287840, 0.289894,\
        0.292067, 0.294359, 0.296615, 0.298638, 0.300303, 0.301586, 0.302625, 0.303444, 0.303969, 0.304177,\
        0.304018, 0.303412, 0.302323, 0.300764, 0.299065, 0.297477, 0.296045, 0.294503, 0.292542, 0.290560,\
        0.288995, 0.287996, 0.287332, 0.286658, 0.285796, 0.284725, 0.283535, 0.282333, 0.281179, 0.280073,\
        0.278989, 0.277901, 0.276793, 0.275645, 0.274431, 0.273133, 0.271747, 0.270280, 0.268745, 0.267153,\
        0.265514, 0.263835, 0.262120, 0.260372, 0.258590, 0.256770, 0.254910, 0.253003, 0.251041, 0.249015,\
        0.246916, 0.244731, 0.242449, 0.240057, 0.237544, 0.234900, 0.232123, 0.229216, 0.226188, 0.223046,\
        0.219777, 0.216352, 0.212760, 0.209104, 0.205469, 0.201611, 0.196500, 0.190390, 0.183969, 0.177297,\
        0.170322],\
        [0.151560, 0.158243, 0.164197, 0.169358, 0.173680, 0.177157, 0.179854, 0.181917, 0.183554, 0.184988,\
        0.186404, 0.187921, 0.189588, 0.191409, 0.193360, 0.195364, 0.197323, 0.199183, 0.201274, 0.203838,\
        0.206546, 0.209231, 0.211946, 0.214764, 0.217723, 0.220821, 0.224029, 0.227305, 0.230607, 0.233900,\
        0.237154, 0.240350, 0.243473, 0.246514, 0.249467, 0.252329, 0.255101, 0.257783, 0.260378, 0.262890,\
        0.265324, 0.267686, 0.269981, 0.272211, 0.274364, 0.276430, 0.278410, 0.280337, 0.282263, 0.284236,\
        0.286280, 0.288367, 0.290391, 0.292214, 0.293729, 0.294916, 0.295863, 0.296602, 0.297076, 0.297268,\
        0.297135, 0.296587, 0.295603, 0.294225, 0.292712, 0.291266, 0.289893, 0.288448, 0.286835, 0.285220,\
        0.283848, 0.282815, 0.282020, 0.281278, 0.280455, 0.279512, 0.278479, 0.277416, 0.276362, 0.275328,\
        0.274304, 0.273275, 0.272224, 0.271133, 0.269984, 0.268765, 0.267471, 0.266101, 0.264664, 0.263166,\
        0.261613, 0.260011, 0.258362, 0.256667, 0.254925, 0.253135, 0.251291, 0.249390, 0.247424, 0.245385,\
        0.243262, 0.241046, 0.238722, 0.236278, 0.233700, 0.230975, 0.228098, 0.225077, 0.221932, 0.218689,\
        0.215348, 0.211846, 0.208058, 0.204105, 0.200368, 0.196384, 0.191430, 0.185459, 0.178949, 0.172082,\
        0.164878],\
        [0.145748, 0.152439, 0.158432, 0.163685, 0.168176, 0.171920, 0.174983, 0.177481, 0.179572, 0.181420,\
        0.183173, 0.184934, 0.186758, 0.188663, 0.190632, 0.192618, 0.194577, 0.196537, 0.198712, 0.201316,\
        0.203916, 0.206395, 0.208931, 0.211624, 0.214513, 0.217580, 0.220773, 0.224029, 0.227295, 0.230528,\
        0.233703, 0.236801, 0.239812, 0.242733, 0.245561, 0.248296, 0.250940, 0.253496, 0.255966, 0.258355,\
        0.260668, 0.262907, 0.265078, 0.267179, 0.269209, 0.271167, 0.273063, 0.274921, 0.276773, 0.278648,\
        0.280549, 0.282446, 0.284261, 0.285901, 0.287293, 0.288419, 0.289311, 0.289983, 0.290416, 0.290582,\
        0.290443, 0.289950, 0.289094, 0.287942, 0.286647, 0.285341, 0.284051, 0.282727, 0.281349, 0.279996,\
        0.278790, 0.277787, 0.276945, 0.276167, 0.275366, 0.274502, 0.273579, 0.272623, 0.271656, 0.270689,\
        0.269720, 0.268740, 0.267735, 0.266691, 0.265597, 0.264441, 0.263218, 0.261927, 0.260569, 0.259148,\
        0.257667, 0.256128, 0.254532, 0.252879, 0.251169, 0.249398, 0.247563, 0.245659, 0.243682, 0.241622,\
        0.239471, 0.237217, 0.234849, 0.232351, 0.229707, 0.226898, 0.223911, 0.220750, 0.217453, 0.214077,\
        0.210661, 0.207169, 0.203347, 0.198960, 0.194939, 0.190656, 0.185635, 0.179788, 0.173310, 0.166366,\
        0.159027],\
        [0.139662, 0.146396, 0.152466, 0.157849, 0.162539, 0.166561, 0.169974, 0.172873, 0.175380, 0.177623,\
        0.179716, 0.181748, 0.183775, 0.185822, 0.187887, 0.189957, 0.192035, 0.194179, 0.196524, 0.199104,\
        0.201445, 0.203686, 0.206040, 0.208620, 0.211465, 0.214531, 0.217730, 0.220974, 0.224197, 0.227360,\
        0.230438, 0.233420, 0.236304, 0.239090, 0.241780, 0.244376, 0.246883, 0.249305, 0.251644, 0.253905,\
        0.256090, 0.258205, 0.260251, 0.262229, 0.264143, 0.265997, 0.267802, 0.269576, 0.271339, 0.273104,\
        0.274866, 0.276595, 0.278237, 0.279726, 0.281013, 0.282077, 0.282921, 0.283549, 0.283949, 0.284098,\
        0.283970, 0.283541, 0.282821, 0.281865, 0.280768, 0.279615, 0.278443, 0.277253, 0.276053, 0.274888,\
        0.273817, 0.272871, 0.272034, 0.271252, 0.270473, 0.269663, 0.268816, 0.267937, 0.267040, 0.266131,\
        0.265208, 0.264267, 0.263300, 0.262295, 0.261243, 0.260136, 0.258969, 0.257739, 0.256444, 0.255084,\
        0.253660, 0.252172, 0.250617, 0.248996, 0.247306, 0.245544, 0.243708, 0.241793, 0.239795, 0.237708,\
        0.235522, 0.233227, 0.230810, 0.228259, 0.225554, 0.222668, 0.219571, 0.216253, 0.212758, 0.209193,\
        0.205649, 0.202107, 0.198442, 0.194000, 0.189470, 0.184763, 0.179533, 0.173654, 0.167174, 0.160188,\
        0.152763],\
        [0.133332, 0.140139, 0.146320, 0.151861, 0.156768, 0.161069, 0.164818, 0.168093, 0.170989, 0.173608,\
        0.176041, 0.178365, 0.180631, 0.182872, 0.185099, 0.187321, 0.189556, 0.191846, 0.194233, 0.196641,\
        0.198890, 0.201020, 0.203248, 0.205753, 0.208606, 0.211715, 0.214941, 0.218171, 0.221336, 0.224403,\
        0.227358, 0.230201, 0.232936, 0.235568, 0.238105, 0.240551, 0.242912, 0.245191, 0.247394, 0.249522,\
        0.251579, 0.253567, 0.255488, 0.257347, 0.259146, 0.260894, 0.262600, 0.264279, 0.265943, 0.267595,\
        0.269227, 0.270812, 0.272311, 0.273677, 0.274873, 0.275876, 0.276679, 0.277275, 0.277655, 0.277803,\
        0.277703, 0.277349, 0.276756, 0.275965, 0.275041, 0.274042, 0.273006, 0.271953, 0.270901, 0.269880,\
        0.268922, 0.268045, 0.267238, 0.266475, 0.265722, 0.264954, 0.264159, 0.263338, 0.262494, 0.261629,\
        0.260743, 0.259832, 0.258893, 0.257916, 0.256896, 0.255827, 0.254702, 0.253518, 0.252271, 0.250959,\
        0.249579, 0.248128, 0.246604, 0.245003, 0.243322, 0.241559, 0.239710, 0.237774, 0.235747, 0.233624,\
        0.231396, 0.229052, 0.226582, 0.223975, 0.221218, 0.218278, 0.215098, 0.211625, 0.207900, 0.204076,\
        0.200366, 0.196668, 0.192779, 0.188413, 0.183687, 0.178658, 0.173193, 0.167194, 0.160654, 0.153612,\
        0.146115],\
        [0.126778, 0.133684, 0.139999, 0.145719, 0.150854, 0.155434, 0.159507, 0.163137, 0.166401, 0.169381,\
        0.172153, 0.174782, 0.177317, 0.179790, 0.182225, 0.184635, 0.187039, 0.189450, 0.191860, 0.194203,\
        0.196405, 0.198495, 0.200628, 0.203099, 0.206023, 0.209215, 0.212469, 0.215657, 0.218725, 0.221655,\
        0.224452, 0.227124, 0.229684, 0.232144, 0.234513, 0.236797, 0.239003, 0.241136, 0.243197, 0.245190,\
        0.247117, 0.248978, 0.250775, 0.252513, 0.254197, 0.255835, 0.257437, 0.259014, 0.260573, 0.262114,\
        0.263627, 0.265090, 0.266471, 0.267736, 0.268854, 0.269803, 0.270570, 0.271144, 0.271517, 0.271678,\
        0.271619, 0.271343, 0.270864, 0.270216, 0.269444, 0.268591, 0.267691, 0.266770, 0.265849, 0.264949,\
        0.264091, 0.263283, 0.262522, 0.261790, 0.261067, 0.260334, 0.259580, 0.258800, 0.257993, 0.257160,\
        0.256300, 0.255410, 0.254489, 0.253531, 0.252532, 0.251488, 0.250393, 0.249242, 0.248031, 0.246754,\
        0.245406, 0.243982, 0.242477, 0.240886, 0.239204, 0.237427, 0.235554, 0.233584, 0.231518, 0.229350,\
        0.227073, 0.224672, 0.222138, 0.219466, 0.216656, 0.213692, 0.210492, 0.206906, 0.202969, 0.198908,\
        0.194982, 0.191053, 0.186908, 0.182406, 0.177525, 0.172272, 0.166599, 0.160452, 0.153812, 0.146692,\
        0.139118],\
        [0.120010, 0.127035, 0.133503, 0.139415, 0.144785, 0.149641, 0.154027, 0.157995, 0.161612, 0.164942,\
        0.168050, 0.170993, 0.173814, 0.176550, 0.179222, 0.181849, 0.184442, 0.186999, 0.189500, 0.191900,\
        0.194161, 0.196297, 0.198420, 0.200873, 0.203881, 0.207153, 0.210380, 0.213457, 0.216361, 0.219098,\
        0.221690, 0.224157, 0.226517, 0.228786, 0.230974, 0.233087, 0.235132, 0.237113, 0.239032, 0.240890,\
        0.242686, 0.244421, 0.246095, 0.247712, 0.249279, 0.250803, 0.252295, 0.253765, 0.255218, 0.256652,\
        0.258058, 0.259417, 0.260701, 0.261884, 0.262937, 0.263840, 0.264579, 0.265140, 0.265515, 0.265699,\
        0.265688, 0.265487, 0.265112, 0.264589, 0.263952, 0.263233, 0.262463, 0.261666, 0.260862, 0.260069,\
        0.259301, 0.258563, 0.257853, 0.257160, 0.256470, 0.255769, 0.255046, 0.254296, 0.253515, 0.252702,\
        0.251857, 0.250978, 0.250064, 0.249115, 0.248127, 0.247096, 0.246020, 0.244891, 0.243705, 0.242452,\
        0.241126, 0.239718, 0.238222, 0.236631, 0.234936, 0.233134, 0.231223, 0.229206, 0.227088, 0.224868,\
        0.222535, 0.220070, 0.217457, 0.214695, 0.211805, 0.208812, 0.205681, 0.202078, 0.197941, 0.193726,\
        0.189519, 0.185275, 0.180848, 0.176126, 0.171052, 0.165601, 0.159740, 0.153438, 0.146678, 0.139460,\
        0.131799],\
        [0.113032, 0.120193, 0.126827, 0.132940, 0.138547, 0.143676, 0.148364, 0.152658, 0.156612, 0.160282,\
        0.163723, 0.166983, 0.170104, 0.173120, 0.176054, 0.178924, 0.181738, 0.184491, 0.187169, 0.189744,\
        0.192201, 0.194557, 0.196889, 0.199360, 0.202430, 0.205639, 0.208694, 0.211548, 0.214202, 0.216682,\
        0.219022, 0.221251, 0.223390, 0.225453, 0.227450, 0.229387, 0.231268, 0.233097, 0.234874, 0.236598,\
        0.238268, 0.239879, 0.241431, 0.242926, 0.244372, 0.245779, 0.247157, 0.248517, 0.249865, 0.251199,\
        0.252509, 0.253779, 0.254986, 0.256102, 0.257104, 0.257971, 0.258688, 0.259243, 0.259629, 0.259840,\
        0.259878, 0.259750, 0.259469, 0.259056, 0.258538, 0.257940, 0.257290, 0.256607, 0.255911, 0.255215,\
        0.254530, 0.253859, 0.253203, 0.252551, 0.251896, 0.251225, 0.250528, 0.249800, 0.249035, 0.248232,\
        0.247390, 0.246511, 0.245595, 0.244643, 0.243655, 0.242628, 0.241560, 0.240444, 0.239272, 0.238034,\
        0.236720, 0.235319, 0.233823, 0.232222, 0.230507, 0.228669, 0.226705, 0.224623, 0.222438, 0.220155,\
        0.217764, 0.215233, 0.212528, 0.209645, 0.206623, 0.203506, 0.200374, 0.196830, 0.192618, 0.188236,\
        0.183794, 0.179264, 0.174559, 0.169594, 0.164306, 0.158656, 0.152614, 0.146155, 0.139264, 0.131936,\
        0.124178],\
        [0.105843, 0.113151, 0.119961, 0.126281, 0.132126, 0.137522, 0.142504, 0.147111, 0.151391, 0.155390,\
        0.159157, 0.162735, 0.166162, 0.169471, 0.172684, 0.175820, 0.178888, 0.181888, 0.184814, 0.187658,\
        0.190423, 0.193139, 0.195885, 0.198793, 0.201731, 0.204564, 0.207303, 0.209833, 0.212157, 0.214322,\
        0.216372, 0.218341, 0.220246, 0.222098, 0.223902, 0.225662, 0.227379, 0.229057, 0.230696, 0.232294,\
        0.233842, 0.235335, 0.236767, 0.238140, 0.239463, 0.240747, 0.242008, 0.243256, 0.244500, 0.245741,\
        0.246968, 0.248165, 0.249308, 0.250374, 0.251337, 0.252177, 0.252880, 0.253435, 0.253834, 0.254076,\
        0.254163, 0.254101, 0.253904, 0.253588, 0.253175, 0.252687, 0.252145, 0.251566, 0.250967, 0.250359,\
        0.249752, 0.249147, 0.248543, 0.247936, 0.247315, 0.246672, 0.245998, 0.245285, 0.244529, 0.243727,\
        0.242879, 0.241989, 0.241060, 0.240094, 0.239094, 0.238060, 0.236991, 0.235878, 0.234713, 0.233482,\
        0.232170, 0.230767, 0.229263, 0.227646, 0.225904, 0.224020, 0.221988, 0.219820, 0.217544, 0.215186,\
        0.212738, 0.210150, 0.207356, 0.204330, 0.201152, 0.197888, 0.194543, 0.190890, 0.186746, 0.182303,\
        0.177696, 0.172940, 0.167991, 0.162791, 0.157288, 0.151442, 0.145221, 0.138604, 0.131576, 0.124129,\
        0.116266],\
        [0.098438, 0.105903, 0.112895, 0.119424, 0.125505, 0.131164, 0.136429, 0.141338, 0.145931, 0.150250,\
        0.154335, 0.158228, 0.161962, 0.165568, 0.169071, 0.172489, 0.175834, 0.179115, 0.182335, 0.185497,\
        0.188616, 0.191717, 0.194834, 0.197947, 0.200809, 0.203533, 0.205996, 0.208147, 0.210085, 0.211900,\
        0.213645, 0.215349, 0.217022, 0.218668, 0.220285, 0.221873, 0.223431, 0.224964, 0.226473, 0.227953,\
        0.229391, 0.230772, 0.232088, 0.233338, 0.234535, 0.235693, 0.236831, 0.237967, 0.239110, 0.240264,\
        0.241419, 0.242558, 0.243656, 0.244684, 0.245619, 0.246441, 0.247136, 0.247693, 0.248108, 0.248380,\
        0.248512, 0.248511, 0.248388, 0.248159, 0.247839, 0.247447, 0.247001, 0.246516, 0.246005, 0.245479,\
        0.244943, 0.244401, 0.243850, 0.243285, 0.242700, 0.242085, 0.241430, 0.240729, 0.239975, 0.239165,\
        0.238302, 0.237390, 0.236436, 0.235445, 0.234423, 0.233371, 0.232290, 0.231174, 0.230010, 0.228778,\
        0.227461, 0.226044, 0.224521, 0.222884, 0.221114, 0.219183, 0.217069, 0.214787, 0.212389, 0.209932,\
        0.207426, 0.204792, 0.201923, 0.198769, 0.195443, 0.192024, 0.188456, 0.184622, 0.180431, 0.175932,\
        0.171202, 0.166266, 0.161110, 0.155698, 0.149991, 0.143954, 0.137559, 0.130784, 0.123616, 0.116046,\
        0.108071],\
        [0.090811, 0.098439, 0.105617, 0.112355, 0.118670, 0.124583, 0.130124, 0.135323, 0.140217, 0.144843,\
        0.149237, 0.153436, 0.157473, 0.161376, 0.165170, 0.168876, 0.172509, 0.176082, 0.179607, 0.183094,\
        0.186556, 0.190002, 0.193410, 0.196672, 0.199656, 0.202425, 0.204583, 0.206276, 0.207806, 0.209278,\
        0.210733, 0.212193, 0.213655, 0.215112, 0.216558, 0.217986, 0.219392, 0.220785, 0.222174, 0.223553,\
        0.224896, 0.226178, 0.227382, 0.228509, 0.229574, 0.230601, 0.231613, 0.232633, 0.233678, 0.234753,\
        0.235851, 0.236948, 0.238015, 0.239019, 0.239936, 0.240746, 0.241435, 0.241997, 0.242428, 0.242728,\
        0.242900, 0.242953, 0.242895, 0.242740, 0.242502, 0.242195, 0.241835, 0.241433, 0.241002, 0.240549,\
        0.240080, 0.239596, 0.239095, 0.238573, 0.238023, 0.237435, 0.236800, 0.236107, 0.235351, 0.234527,\
        0.233638, 0.232693, 0.231703, 0.230678, 0.229622, 0.228539, 0.227435, 0.226309, 0.225143, 0.223908,\
        0.222574, 0.221129, 0.219574, 0.217909, 0.216118, 0.214151, 0.211949, 0.209526, 0.206970, 0.204377,\
        0.201796, 0.199091, 0.196125, 0.192880, 0.189439, 0.185851, 0.182085, 0.178075, 0.173771, 0.169179,\
        0.164329, 0.159238, 0.153903, 0.148301, 0.142404, 0.136186, 0.129622, 0.122692, 0.115384, 0.107687,\
        0.099597],\
        [0.082954, 0.090749, 0.098114, 0.105060, 0.111604, 0.117765, 0.123570, 0.129048, 0.134230, 0.139150,\
        0.143842, 0.148337, 0.152667, 0.156859, 0.160938, 0.164925, 0.168839, 0.172696, 0.176510, 0.180294,\
        0.184058, 0.187802, 0.191497, 0.195079, 0.198452, 0.201248, 0.202742, 0.203940, 0.205135, 0.206326,\
        0.207538, 0.208801, 0.210090, 0.211384, 0.212683, 0.213970, 0.215232, 0.216487, 0.217769, 0.219071,\
        0.220344, 0.221540, 0.222637, 0.223638, 0.224568, 0.225457, 0.226338, 0.227241, 0.228189, 0.229194,\
        0.230248, 0.231323, 0.232375, 0.233368, 0.234274, 0.235074, 0.235760, 0.236325, 0.236769, 0.237092,\
        0.237300, 0.237399, 0.237398, 0.237308, 0.237140, 0.236907, 0.236621, 0.236294, 0.235933, 0.235546,\
        0.235137, 0.234707, 0.234254, 0.233774, 0.233258, 0.232698, 0.232082, 0.231398, 0.230637, 0.229793,\
        0.228869, 0.227879, 0.226843, 0.225774, 0.224675, 0.223548, 0.222405, 0.221262, 0.220098, 0.218859,\
        0.217497, 0.216004, 0.214394, 0.212684, 0.210878, 0.208912, 0.206634, 0.204046, 0.201334, 0.198580,\
        0.195869, 0.193032, 0.189955, 0.186626, 0.183080, 0.179345, 0.175408, 0.171235, 0.166795, 0.162082,\
        0.157104, 0.151867, 0.146368, 0.140593, 0.134522, 0.128133, 0.121406, 0.114325, 0.106877, 0.099052,\
        0.090845],\
        [0.074858, 0.082822, 0.090374, 0.097525, 0.104291, 0.110693, 0.116752, 0.122496, 0.127953, 0.133154,\
        0.138128, 0.142905, 0.147515, 0.151983, 0.156333, 0.160586, 0.164762, 0.168877, 0.172945, 0.176978,\
        0.180979, 0.184942, 0.188842, 0.192620, 0.196135, 0.198738, 0.199884, 0.200906, 0.201947, 0.202958,\
        0.203982, 0.205129, 0.206286, 0.207442, 0.208630, 0.209810, 0.210928, 0.212035, 0.213226, 0.214490,\
        0.215724, 0.216850, 0.217841, 0.218712, 0.219499, 0.220247, 0.220995, 0.221777, 0.222627, 0.223570,\
        0.224602, 0.225676, 0.226732, 0.227722, 0.228619, 0.229410, 0.230089, 0.230654, 0.231107, 0.231449,\
        0.231686, 0.231824, 0.231870, 0.231835, 0.231727, 0.231558, 0.231337, 0.231073, 0.230774, 0.230446,\
        0.230091, 0.229710, 0.229302, 0.228861, 0.228379, 0.227847, 0.227252, 0.226579, 0.225815, 0.224948,\
        0.223979, 0.222929, 0.221834, 0.220720, 0.219577, 0.218387, 0.217177, 0.216007, 0.214859, 0.213626,\
        0.212222, 0.210652, 0.208957, 0.207168, 0.205320, 0.203402, 0.201093, 0.198341, 0.195503, 0.192624,\
        0.189718, 0.186698, 0.183483, 0.180041, 0.176379, 0.172506, 0.168416, 0.164089, 0.159507, 0.154660,\
        0.149546, 0.144165, 0.138512, 0.132575, 0.126340, 0.119790, 0.112908, 0.105680, 0.098094, 0.090141,\
        0.081816],\
        [0.066515, 0.074647, 0.082384, 0.089736, 0.096719, 0.103351, 0.109653, 0.115650, 0.121368, 0.126834,\
        0.132075, 0.137119, 0.141992, 0.146717, 0.151319, 0.155816, 0.160227, 0.164565, 0.168842, 0.173064,\
        0.177225, 0.181308, 0.185263, 0.188990, 0.192256, 0.194655, 0.196120, 0.197214, 0.198220, 0.199138,\
        0.200049, 0.201179, 0.202213, 0.203244, 0.204372, 0.205504, 0.206472, 0.207384, 0.208515, 0.209805,\
        0.211033, 0.212095, 0.212975, 0.213708, 0.214350, 0.214957, 0.215576, 0.216237, 0.216982, 0.217870,\
        0.218907, 0.220011, 0.221084, 0.222072, 0.222957, 0.223733, 0.224400, 0.224961, 0.225418, 0.225773,\
        0.226033, 0.226202, 0.226288, 0.226298, 0.226240, 0.226125, 0.225959, 0.225750, 0.225504, 0.225226,\
        0.224918, 0.224581, 0.224212, 0.223807, 0.223357, 0.222853, 0.222281, 0.221624, 0.220863, 0.219976,\
        0.218954, 0.217822, 0.216650, 0.215501, 0.214337, 0.213067, 0.211731, 0.210510, 0.209415, 0.208212,\
        0.206737, 0.205058, 0.203261, 0.201364, 0.199409, 0.197432, 0.195105, 0.192325, 0.189388, 0.186379,\
        0.183297, 0.180096, 0.176723, 0.173146, 0.169353, 0.165342, 0.161106, 0.156632, 0.151906, 0.146920,\
        0.141667, 0.136142, 0.130339, 0.124249, 0.117859, 0.111155, 0.104124, 0.096753, 0.089033, 0.080953,\
        0.072510],\
        [0.057914, 0.066214, 0.074132, 0.081680, 0.088872, 0.095725, 0.102259, 0.108497, 0.114461, 0.120176,\
        0.125668, 0.130960, 0.136076, 0.141039, 0.145869, 0.150584, 0.155198, 0.159723, 0.164163, 0.168517,\
        0.172771, 0.176889, 0.180805, 0.184400, 0.187488, 0.189877, 0.191584, 0.192863, 0.193931, 0.194882,\
        0.195835, 0.196868, 0.197830, 0.198802, 0.199888, 0.201042, 0.201847, 0.202490, 0.203648, 0.205035,\
        0.206257, 0.207243, 0.208003, 0.208596, 0.209095, 0.209571, 0.210082, 0.210633, 0.211251, 0.212081,\
        0.213171, 0.214342, 0.215435, 0.216410, 0.217269, 0.218021, 0.218670, 0.219220, 0.219675, 0.220039,\
        0.220315, 0.220508, 0.220625, 0.220672, 0.220656, 0.220585, 0.220464, 0.220301, 0.220100, 0.219864,\
        0.219596, 0.219295, 0.218960, 0.218585, 0.218164, 0.217686, 0.217138, 0.216503, 0.215754, 0.214862,\
        0.213790, 0.212545, 0.211253, 0.210088, 0.208984, 0.207647, 0.206064, 0.204736, 0.203743, 0.202600,\
        0.201009, 0.199200, 0.197308, 0.195315, 0.193249, 0.191077, 0.188641, 0.185878, 0.182898, 0.179785,\
        0.176557, 0.173195, 0.169666, 0.165943, 0.162012, 0.157860, 0.153481, 0.148862, 0.143994, 0.138867,\
        0.133472, 0.127805, 0.121856, 0.115618, 0.109079, 0.102227, 0.095053, 0.087544, 0.079691, 0.071486,\
        0.062924],\
        [0.049047, 0.057512, 0.065608, 0.073345, 0.080738, 0.087803, 0.094557, 0.101022, 0.107217, 0.113166,\
        0.118890, 0.124412, 0.129753, 0.134932, 0.139967, 0.144872, 0.149660, 0.154335, 0.158899, 0.163342,\
        0.167640, 0.171751, 0.175604, 0.179097, 0.182104, 0.184539, 0.186425, 0.187896, 0.189105, 0.190165,\
        0.191172, 0.192169, 0.193138, 0.194119, 0.195162, 0.196193, 0.196945, 0.197580, 0.198763, 0.200156,\
        0.201305, 0.202214, 0.202856, 0.203328, 0.203710, 0.204072, 0.204527, 0.205008, 0.205454, 0.206195,\
        0.207438, 0.208700, 0.209778, 0.210709, 0.211527, 0.212245, 0.212869, 0.213405, 0.213854, 0.214220,\
        0.214507, 0.214718, 0.214859, 0.214936, 0.214953, 0.214917, 0.214832, 0.214706, 0.214540, 0.214338,\
        0.214101, 0.213829, 0.213519, 0.213168, 0.212769, 0.212313, 0.211786, 0.211173, 0.210449, 0.209576,\
        0.208491, 0.207115, 0.205603, 0.204419, 0.203500, 0.202207, 0.200309, 0.198794, 0.197787, 0.196582,\
        0.194944, 0.193062, 0.191070, 0.188975, 0.186775, 0.184433, 0.181872, 0.179059, 0.176028, 0.172826,\
        0.169476, 0.165972, 0.162296, 0.158430, 0.154356, 0.150063, 0.145542, 0.140781, 0.135771, 0.130502,\
        0.124967, 0.119158, 0.113066, 0.106683, 0.100000, 0.093007, 0.085693, 0.078049, 0.070066, 0.061738,\
        0.053058],\
        [0.039905, 0.048532, 0.056801, 0.064721, 0.072307, 0.079573, 0.086536, 0.093214, 0.099626, 0.105792,\
        0.111732, 0.117465, 0.123010, 0.128384, 0.133601, 0.138673, 0.143607, 0.148406, 0.153063, 0.157566,\
        0.161886, 0.165979, 0.169782, 0.173217, 0.176207, 0.178709, 0.180743, 0.182393, 0.183765, 0.184958,\
        0.186049, 0.187084, 0.188086, 0.189078, 0.190072, 0.191009, 0.191822, 0.192649, 0.193729, 0.194940,\
        0.196009, 0.196853, 0.197437, 0.197856, 0.198180, 0.198481, 0.198914, 0.199379, 0.199682, 0.200297,\
        0.201833, 0.203070, 0.204060, 0.204924, 0.205691, 0.206371, 0.206969, 0.207487, 0.207927, 0.208293,\
        0.208585, 0.208809, 0.208968, 0.209066, 0.209108, 0.209099, 0.209043, 0.208944, 0.208805, 0.208627,\
        0.208413, 0.208161, 0.207869, 0.207532, 0.207146, 0.206700, 0.206186, 0.205586, 0.204882, 0.204040,\
        0.203003, 0.201597, 0.199791, 0.198546, 0.197652, 0.196396, 0.194570, 0.192925, 0.191584, 0.190174,\
        0.188494, 0.186577, 0.184505, 0.182306, 0.179977, 0.177491, 0.174809, 0.171910, 0.168804, 0.165513,\
        0.162051, 0.158419, 0.154606, 0.150599, 0.146385, 0.141952, 0.137290, 0.132388, 0.127237, 0.121828,\
        0.116153, 0.110203, 0.103971, 0.097448, 0.090625, 0.083493, 0.076044, 0.068268, 0.060158, 0.051708,\
        0.042912]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=30 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax30vt10
    zgrid_H_Hmax_vmax30vt10=[[0.460188, 0.471631, 0.482878, 0.493838, 0.504117, 0.513837, 0.523994, 0.535067, 0.547189, 0.560135,\
        0.573560, 0.586893, 0.598955, 0.607215, 0.613580, 0.618313, 0.621395, 0.623199, 0.624042, 0.624129,\
        0.623598, 0.622542, 0.621048, 0.619232, 0.617245, 0.615244, 0.613345, 0.611577, 0.609889, 0.608223,\
        0.606581, 0.605007, 0.603532, 0.602146, 0.600483, 0.595894, 0.589027, 0.580807, 0.571567, 0.561541,\
        0.550950, 0.540033, 0.529068, 0.518379, 0.508359, 0.499607, 0.492935, 0.487319, 0.482870, 0.479730,\
        0.478086, 0.478081, 0.479786, 0.483318, 0.489185, 0.497842, 0.509501, 0.522643, 0.535570, 0.546990,\
        0.556014, 0.562519, 0.567156, 0.570894, 0.574424, 0.577875, 0.581048, 0.583954, 0.586652, 0.589063,\
        0.591467, 0.594453, 0.599238, 0.604346, 0.609535, 0.614778, 0.619974, 0.625030, 0.629867, 0.634426,\
        0.638667, 0.642562, 0.646094, 0.649251, 0.652025, 0.654393, 0.656311, 0.657706, 0.658515, 0.658726,\
        0.658387, 0.657573, 0.656354, 0.654768, 0.652827, 0.650526, 0.647856, 0.644810, 0.641386, 0.637585,\
        0.633414, 0.628885, 0.624014, 0.618826, 0.613350, 0.607628, 0.601726, 0.595778, 0.589830, 0.583709,\
        0.577252, 0.570403, 0.563163, 0.555545, 0.547571, 0.539269, 0.530669, 0.521798, 0.512696, 0.503412,\
        0.493433],\
        [0.451164, 0.462125, 0.472808, 0.483085, 0.492720, 0.502040, 0.511604, 0.522437, 0.534545, 0.547514,\
        0.561091, 0.574767, 0.587295, 0.597345, 0.605746, 0.611793, 0.615684, 0.618225, 0.619734, 0.620425,\
        0.620424, 0.619774, 0.618523, 0.616790, 0.614773, 0.612718, 0.610829, 0.609158, 0.607588, 0.605985,\
        0.604367, 0.602853, 0.601483, 0.600179, 0.598874, 0.595481, 0.589684, 0.582299, 0.573714, 0.564181,\
        0.553941, 0.543261, 0.532456, 0.521907, 0.512101, 0.503697, 0.496622, 0.490829, 0.486326, 0.483251,\
        0.481925, 0.482538, 0.485106, 0.489449, 0.496013, 0.506539, 0.519809, 0.534468, 0.548911, 0.561472,\
        0.570710, 0.576458, 0.579877, 0.582584, 0.585600, 0.588890, 0.591729, 0.594227, 0.596788, 0.598786,\
        0.600856, 0.603333, 0.607779, 0.613065, 0.618420, 0.623785, 0.629073, 0.634185, 0.639039, 0.643575,\
        0.647760, 0.651572, 0.655001, 0.658043, 0.660709, 0.663000, 0.664866, 0.666190, 0.666853, 0.666829,\
        0.666209, 0.665128, 0.663684, 0.661915, 0.659818, 0.657372, 0.654555, 0.651356, 0.647769, 0.643796,\
        0.639443, 0.634722, 0.629653, 0.624261, 0.618577, 0.612644, 0.606524, 0.600341, 0.594215, 0.587915,\
        0.581160, 0.573974, 0.566371, 0.558363, 0.549974, 0.541235, 0.532171, 0.522792, 0.513056, 0.502920,\
        0.493185],\
        [0.441613, 0.452045, 0.462120, 0.471737, 0.480852, 0.489661, 0.498616, 0.509323, 0.521132, 0.533660,\
        0.546938, 0.560768, 0.574249, 0.586452, 0.596925, 0.604366, 0.609257, 0.612912, 0.615283, 0.616804,\
        0.617562, 0.617484, 0.616545, 0.614852, 0.612660, 0.610352, 0.608311, 0.606676, 0.605203, 0.603581,\
        0.601878, 0.600441, 0.599286, 0.598362, 0.597135, 0.594528, 0.589967, 0.583677, 0.575964, 0.567091,\
        0.557326, 0.546973, 0.536390, 0.526002, 0.516281, 0.507636, 0.500230, 0.494814, 0.490056, 0.487008,\
        0.486068, 0.487483, 0.491399, 0.497442, 0.505256, 0.516986, 0.531496, 0.547315, 0.563126, 0.577076,\
        0.586511, 0.591199, 0.593122, 0.594643, 0.597193, 0.600260, 0.602775, 0.604741, 0.606958, 0.608832,\
        0.610839, 0.613541, 0.617524, 0.622473, 0.627791, 0.633197, 0.638533, 0.643668, 0.648501, 0.652969,\
        0.657050, 0.660735, 0.664012, 0.666879, 0.669370, 0.671546, 0.673380, 0.674675, 0.675183, 0.674849,\
        0.673869, 0.672482, 0.670815, 0.668884, 0.666654, 0.664078, 0.661122, 0.657772, 0.654025, 0.649881,\
        0.645347, 0.640437, 0.635174, 0.629588, 0.623717, 0.617610, 0.611334, 0.604983, 0.598719, 0.592034,\
        0.584866, 0.577304, 0.569307, 0.560878, 0.552046, 0.542853, 0.533344, 0.523561, 0.513566, 0.503562,\
        0.493541],\
        [0.431780, 0.441736, 0.451309, 0.460458, 0.469217, 0.477799, 0.486765, 0.496852, 0.507885, 0.519454,\
        0.532104, 0.546266, 0.560989, 0.574963, 0.587062, 0.596098, 0.603103, 0.607787, 0.611025, 0.613683,\
        0.615500, 0.616195, 0.615658, 0.613961, 0.611391, 0.608485, 0.605947, 0.604204, 0.602827, 0.601088,\
        0.599253, 0.598006, 0.597391, 0.597011, 0.596210, 0.594253, 0.590699, 0.585463, 0.578653, 0.570480,\
        0.561217, 0.551190, 0.540792, 0.530469, 0.520689, 0.511906, 0.504625, 0.498460, 0.493533, 0.490667,\
        0.490163, 0.492369, 0.497596, 0.505675, 0.515787, 0.528659, 0.543836, 0.560206, 0.576736, 0.592141,\
        0.601975, 0.605824, 0.607079, 0.607931, 0.609745, 0.611949, 0.613931, 0.615589, 0.617295, 0.619067,\
        0.621162, 0.623979, 0.627785, 0.632455, 0.637633, 0.643008, 0.648351, 0.653480, 0.658253, 0.662601,\
        0.666524, 0.670040, 0.673130, 0.675750, 0.677955, 0.679934, 0.681766, 0.683134, 0.683487, 0.682716,\
        0.681268, 0.679561, 0.677706, 0.675656, 0.673328, 0.670640, 0.667548, 0.664047, 0.660139, 0.655824,\
        0.651107, 0.646003, 0.640543, 0.634761, 0.628704, 0.622425, 0.615989, 0.609472, 0.602818, 0.595694,\
        0.588270, 0.580365, 0.571970, 0.563106, 0.553822, 0.544181, 0.534255, 0.524134, 0.513969, 0.503922,\
        0.493656],\
        [0.421882, 0.431474, 0.440706, 0.449593, 0.458219, 0.466801, 0.475706, 0.485253, 0.495461, 0.505571,\
        0.517609, 0.532485, 0.548676, 0.564239, 0.577887, 0.589056, 0.597739, 0.602691, 0.607343, 0.611659,\
        0.614839, 0.616509, 0.616506, 0.614833, 0.611719, 0.607739, 0.603981, 0.601654, 0.600189, 0.598224,\
        0.596597, 0.595964, 0.596175, 0.596605, 0.596439, 0.595120, 0.592427, 0.588110, 0.582097, 0.574551,\
        0.565722, 0.555939, 0.545621, 0.535236, 0.525265, 0.516150, 0.508195, 0.501501, 0.496800, 0.494106,\
        0.493943, 0.497089, 0.503516, 0.513551, 0.526050, 0.540493, 0.556306, 0.572720, 0.588757, 0.603674,\
        0.614332, 0.619315, 0.621196, 0.622029, 0.622986, 0.624181, 0.625393, 0.626572, 0.627877, 0.629472,\
        0.631575, 0.634435, 0.638180, 0.642717, 0.647795, 0.653134, 0.658480, 0.663598, 0.668284, 0.672450,\
        0.676148, 0.679463, 0.682369, 0.684701, 0.686447, 0.688050, 0.689867, 0.691507, 0.691707, 0.690275,\
        0.688255, 0.686316, 0.684352, 0.682239, 0.679856, 0.677062, 0.673819, 0.670160, 0.666094, 0.661606,\
        0.656697, 0.651386, 0.645712, 0.639719, 0.633454, 0.626967, 0.620302, 0.613472, 0.606406, 0.599112,\
        0.591400, 0.583138, 0.574347, 0.565043, 0.555309, 0.545234, 0.534902, 0.524406, 0.513847, 0.503332,\
        0.492987],\
        [0.412071, 0.421423, 0.430477, 0.439289, 0.447966, 0.456674, 0.465608, 0.474853, 0.484216, 0.493628,\
        0.505234, 0.521221, 0.538923, 0.555717, 0.570546, 0.582851, 0.592170, 0.598761, 0.605470, 0.611568,\
        0.616181, 0.618974, 0.619718, 0.618292, 0.614754, 0.609483, 0.603452, 0.599020, 0.597081, 0.595769,\
        0.594947, 0.595234, 0.596447, 0.597807, 0.598285, 0.597421, 0.595501, 0.591920, 0.586493, 0.579435,\
        0.570912, 0.561233, 0.550854, 0.540276, 0.529992, 0.520450, 0.512030, 0.505140, 0.500245, 0.497235,\
        0.497177, 0.500900, 0.509507, 0.521695, 0.536430, 0.552558, 0.569020, 0.585494, 0.601231, 0.615329,\
        0.626094, 0.632201, 0.634875, 0.635813, 0.636210, 0.636581, 0.637045, 0.637663, 0.638561, 0.639919,\
        0.641943, 0.644802, 0.648547, 0.653063, 0.658119, 0.663451, 0.668824, 0.673974, 0.678577, 0.682488,\
        0.685875, 0.688946, 0.691724, 0.693842, 0.694940, 0.695959, 0.697489, 0.699544, 0.699491, 0.697217,\
        0.694739, 0.692856, 0.690821, 0.688669, 0.686276, 0.683345, 0.679899, 0.676080, 0.671869, 0.667208,\
        0.662088, 0.656546, 0.650632, 0.644398, 0.637893, 0.631158, 0.624221, 0.617087, 0.609733, 0.602112,\
        0.594071, 0.585603, 0.576413, 0.566655, 0.556483, 0.546004, 0.535307, 0.524472, 0.513570, 0.502712,\
        0.492275],\
        [0.402412, 0.411643, 0.420669, 0.429565, 0.438464, 0.447494, 0.456787, 0.466394, 0.476266, 0.486617,\
        0.498798, 0.515273, 0.533124, 0.550246, 0.565687, 0.578876, 0.589599, 0.598519, 0.606574, 0.613883,\
        0.619804, 0.623853, 0.625678, 0.624998, 0.621676, 0.615886, 0.608436, 0.601788, 0.598634, 0.597346,\
        0.596589, 0.597393, 0.599396, 0.601188, 0.602088, 0.601561, 0.600164, 0.596948, 0.591845, 0.585176,\
        0.576787, 0.567029, 0.556445, 0.545566, 0.534905, 0.524946, 0.516122, 0.508839, 0.503379, 0.499900,\
        0.499198, 0.504687, 0.515593, 0.530179, 0.547172, 0.565283, 0.582692, 0.598470, 0.615014, 0.628926,\
        0.639292, 0.645532, 0.648368, 0.649165, 0.649091, 0.648825, 0.648666, 0.648753, 0.649228, 0.650291,\
        0.652167, 0.655006, 0.658804, 0.663393, 0.668500, 0.673833, 0.679225, 0.684507, 0.689096, 0.692679,\
        0.695693, 0.698467, 0.701070, 0.703009, 0.703726, 0.704330, 0.705398, 0.706554, 0.706134, 0.703751,\
        0.701363, 0.699487, 0.697231, 0.694973, 0.692661, 0.689431, 0.685697, 0.681779, 0.677466, 0.672619,\
        0.667252, 0.661439, 0.655251, 0.648744, 0.641964, 0.634947, 0.627718, 0.620283, 0.612624, 0.604686,\
        0.596690, 0.587808, 0.578075, 0.567859, 0.557291, 0.546467, 0.535478, 0.524420, 0.513393, 0.502499,\
        0.491914],\
        [0.392872, 0.402133, 0.411241, 0.420353, 0.429642, 0.439190, 0.449212, 0.459885, 0.471339, 0.483911,\
        0.498264, 0.514063, 0.530895, 0.547625, 0.563319, 0.577372, 0.589483, 0.599909, 0.609544, 0.618150,\
        0.625415, 0.630936, 0.634285, 0.635043, 0.632900, 0.627924, 0.621001, 0.614107, 0.609146, 0.606153,\
        0.604316, 0.604443, 0.606139, 0.607563, 0.608201, 0.607738, 0.606131, 0.602876, 0.598080, 0.591749,\
        0.583194, 0.573179, 0.562289, 0.551042, 0.539991, 0.529661, 0.520508, 0.512916, 0.507183, 0.503658,\
        0.503154, 0.510074, 0.522880, 0.539212, 0.558010, 0.578216, 0.597895, 0.614318, 0.630185, 0.643956,\
        0.653888, 0.659570, 0.661840, 0.662100, 0.661509, 0.660748, 0.660123, 0.659767, 0.659818, 0.660521,\
        0.662184, 0.665003, 0.668917, 0.673668, 0.678916, 0.684234, 0.689441, 0.694949, 0.699640, 0.702955,\
        0.705717, 0.708244, 0.710466, 0.712055, 0.712841, 0.713301, 0.713773, 0.713926, 0.713037, 0.711003,\
        0.708674, 0.706356, 0.703804, 0.701264, 0.699076, 0.694952, 0.691096, 0.687301, 0.682925, 0.677827,\
        0.672145, 0.666015, 0.659518, 0.652709, 0.645624, 0.638296, 0.630748, 0.622998, 0.615072, 0.607070,\
        0.598806, 0.589229, 0.579098, 0.568543, 0.557666, 0.546573, 0.535367, 0.524157, 0.513105, 0.502411,\
        0.491624],\
        [0.383502, 0.392821, 0.402050, 0.411556, 0.421349, 0.431449, 0.442381, 0.454446, 0.467776, 0.482389,\
        0.497930, 0.514060, 0.530350, 0.546673, 0.562578, 0.577542, 0.591071, 0.602931, 0.613707, 0.623542,\
        0.632262, 0.639508, 0.644834, 0.647718, 0.647631, 0.644370, 0.638597, 0.631898, 0.625891, 0.621281,\
        0.618165, 0.616780, 0.616711, 0.616877, 0.616595, 0.615490, 0.613321, 0.609864, 0.605145, 0.598611,\
        0.589666, 0.579437, 0.568222, 0.556575, 0.545161, 0.534568, 0.525289, 0.517753, 0.512393, 0.509830,\
        0.511371, 0.518813, 0.531815, 0.548751, 0.568340, 0.589521, 0.610633, 0.629282, 0.645910, 0.659829,\
        0.669389, 0.673998, 0.675078, 0.674504, 0.673393, 0.672296, 0.671403, 0.670733, 0.670367, 0.670611,\
        0.671969, 0.674793, 0.678916, 0.683884, 0.689367, 0.694839, 0.699522, 0.704835, 0.709587, 0.713078,\
        0.715842, 0.718179, 0.720081, 0.721411, 0.722129, 0.722415, 0.722391, 0.721901, 0.720668, 0.718683,\
        0.716263, 0.713603, 0.710720, 0.707695, 0.704190, 0.699829, 0.696393, 0.692883, 0.688290, 0.682770,\
        0.676695, 0.670214, 0.663388, 0.656254, 0.648839, 0.641169, 0.633267, 0.625150, 0.616809, 0.608145,\
        0.599024, 0.589620, 0.579431, 0.568667, 0.557561, 0.546274, 0.534920, 0.523580, 0.512309, 0.501300,\
        0.490727],\
        [0.374294, 0.383681, 0.393219, 0.403354, 0.413291, 0.423743, 0.435593, 0.449037, 0.464068, 0.480525,\
        0.497752, 0.513860, 0.529730, 0.545940, 0.562178, 0.578018, 0.593122, 0.606212, 0.617904, 0.629038,\
        0.639333, 0.648470, 0.656067, 0.661582, 0.664202, 0.663102, 0.658507, 0.652004, 0.645350, 0.639566,\
        0.635051, 0.631919, 0.629890, 0.628341, 0.626683, 0.624489, 0.621452, 0.617347, 0.611954, 0.604793,\
        0.595900, 0.585668, 0.574037, 0.561947, 0.550238, 0.539537, 0.530373, 0.523241, 0.518682, 0.517413,\
        0.520411, 0.528496, 0.541526, 0.558511, 0.578238, 0.599621, 0.621708, 0.642349, 0.660516, 0.675380,\
        0.685059, 0.688091, 0.687581, 0.686224, 0.684737, 0.683527, 0.682602, 0.681797, 0.681038, 0.680674,\
        0.681548, 0.684436, 0.688992, 0.694068, 0.699760, 0.705345, 0.710236, 0.715031, 0.719497, 0.723099,\
        0.725930, 0.728173, 0.729875, 0.731008, 0.731571, 0.731633, 0.731236, 0.730323, 0.728801, 0.726683,\
        0.724093, 0.721141, 0.717867, 0.714249, 0.710180, 0.705969, 0.702264, 0.698661, 0.693333, 0.687268,\
        0.680804, 0.673981, 0.666828, 0.659360, 0.651592, 0.643550, 0.635261, 0.626744, 0.618003, 0.609005,\
        0.599644, 0.589809, 0.579281, 0.568260, 0.556952, 0.545521, 0.534101, 0.522782, 0.511549, 0.500394,\
        0.489837],\
        [0.365276, 0.374753, 0.384593, 0.394803, 0.404600, 0.415553, 0.428346, 0.442976, 0.459226, 0.476725,\
        0.494651, 0.511316, 0.527637, 0.544321, 0.561090, 0.577502, 0.592923, 0.607318, 0.620965, 0.633695,\
        0.645604, 0.656566, 0.666359, 0.674603, 0.680371, 0.681674, 0.678012, 0.671657, 0.664766, 0.658315,\
        0.652646, 0.647950, 0.644148, 0.640887, 0.637721, 0.634234, 0.630084, 0.625000, 0.618746, 0.611143,\
        0.602320, 0.591687, 0.579328, 0.566840, 0.554974, 0.544338, 0.535485, 0.528951, 0.525304, 0.525188,\
        0.529256, 0.537928, 0.551096, 0.568136, 0.588076, 0.609880, 0.632495, 0.654107, 0.673223, 0.688809,\
        0.699345, 0.700663, 0.699105, 0.697456, 0.695749, 0.694645, 0.693862, 0.693130, 0.692073, 0.691028,\
        0.691148, 0.694018, 0.699519, 0.704597, 0.710293, 0.715852, 0.720924, 0.725575, 0.729757, 0.733267,\
        0.736072, 0.738234, 0.739794, 0.740756, 0.741134, 0.740959, 0.740259, 0.739026, 0.737243, 0.734925,\
        0.732125, 0.728895, 0.725260, 0.721231, 0.716873, 0.712424, 0.708094, 0.703374, 0.697503, 0.691193,\
        0.684432, 0.677300, 0.669836, 0.662027, 0.653883, 0.645437, 0.636729, 0.627787, 0.618631, 0.609355,\
        0.600116, 0.589570, 0.578598, 0.567308, 0.555809, 0.544251, 0.532796, 0.521607, 0.510853, 0.500411,\
        0.489226],\
        [0.356459, 0.365884, 0.375568, 0.385407, 0.395511, 0.407042, 0.420683, 0.436242, 0.453199, 0.470981,\
        0.488788, 0.506205, 0.523849, 0.541490, 0.559000, 0.576140, 0.592618, 0.608135, 0.622912, 0.637000,\
        0.650294, 0.662708, 0.674107, 0.684309, 0.692947, 0.696834, 0.693964, 0.688141, 0.681878, 0.675564,\
        0.669217, 0.663389, 0.658210, 0.653515, 0.648980, 0.644219, 0.638877, 0.632671, 0.625416, 0.617033,\
        0.607441, 0.596063, 0.583554, 0.570980, 0.559152, 0.548733, 0.540306, 0.534426, 0.531641, 0.532487,\
        0.537405, 0.546601, 0.559956, 0.577110, 0.597394, 0.619578, 0.642493, 0.664537, 0.683850, 0.698703,\
        0.708331, 0.711143, 0.710312, 0.708700, 0.707065, 0.705984, 0.705241, 0.704574, 0.703475, 0.702257,\
        0.701826, 0.704846, 0.710132, 0.715821, 0.721425, 0.726856, 0.731860, 0.736363, 0.740317, 0.743650,\
        0.746333, 0.748380, 0.749801, 0.750596, 0.750773, 0.750358, 0.749382, 0.747872, 0.745844, 0.743314,\
        0.740296, 0.736802, 0.732841, 0.728441, 0.723678, 0.718670, 0.713429, 0.707702, 0.701402, 0.694787,\
        0.687614, 0.680205, 0.672452, 0.664284, 0.655721, 0.646831, 0.637677, 0.628295, 0.618665, 0.608680,\
        0.598403, 0.588270, 0.577263, 0.565797, 0.554122, 0.542421, 0.530858, 0.519593, 0.508799, 0.498327,\
        0.487385],\
        [0.347949, 0.357293, 0.366876, 0.376734, 0.387163, 0.398932, 0.413205, 0.429376, 0.446703, 0.464635,\
        0.482669, 0.500804, 0.519282, 0.537682, 0.555913, 0.573810, 0.591150, 0.607665, 0.623510, 0.638661,\
        0.652975, 0.666334, 0.678526, 0.689175, 0.697646, 0.703198, 0.703004, 0.699670, 0.695177, 0.689963,\
        0.683428, 0.677136, 0.671125, 0.665451, 0.659883, 0.654051, 0.647588, 0.640217, 0.631784, 0.622218,\
        0.611439, 0.599484, 0.586908, 0.574369, 0.562685, 0.552560, 0.544592, 0.539331, 0.537298, 0.538953,\
        0.544610, 0.554339, 0.567908, 0.584981, 0.605460, 0.628068, 0.651218, 0.673440, 0.692903, 0.707081,\
        0.715880, 0.719833, 0.720434, 0.719576, 0.718379, 0.717387, 0.716655, 0.715998, 0.715228, 0.714594,\
        0.715088, 0.717873, 0.722494, 0.727857, 0.733263, 0.738439, 0.743202, 0.747450, 0.751131, 0.754218,\
        0.756704, 0.758592, 0.759858, 0.760472, 0.760432, 0.759764, 0.758515, 0.756739, 0.754479, 0.751746,\
        0.748524, 0.744782, 0.740502, 0.735696, 0.730415, 0.724720, 0.718619, 0.712067, 0.705124, 0.697758,\
        0.690354, 0.682810, 0.674765, 0.666169, 0.657113, 0.647721, 0.638094, 0.628297, 0.618343, 0.608172,\
        0.597700, 0.586814, 0.575489, 0.563800, 0.551937, 0.540073, 0.528353, 0.516857, 0.505595, 0.494967,\
        0.484474],\
        [0.339895, 0.349257, 0.358943, 0.369077, 0.379976, 0.392262, 0.406695, 0.422970, 0.440384, 0.458439,\
        0.476828, 0.495511, 0.514231, 0.533049, 0.551858, 0.570377, 0.588283, 0.605634, 0.622595, 0.638545,\
        0.653536, 0.667481, 0.680071, 0.690799, 0.699136, 0.705076, 0.707415, 0.706469, 0.704330, 0.699895,\
        0.694525, 0.688540, 0.682287, 0.676215, 0.670088, 0.663517, 0.656111, 0.647621, 0.637953, 0.627142,\
        0.615322, 0.602769, 0.589818, 0.577165, 0.565612, 0.555781, 0.548226, 0.543483, 0.542057, 0.544380,\
        0.550730, 0.561132, 0.575253, 0.592430, 0.612919, 0.635734, 0.658698, 0.680450, 0.699541, 0.713557,\
        0.722579, 0.727506, 0.729406, 0.729576, 0.729047, 0.728431, 0.727961, 0.727650, 0.727509, 0.727797,\
        0.729099, 0.731898, 0.736010, 0.740821, 0.745784, 0.750562, 0.754948, 0.758828, 0.762151, 0.764905,\
        0.767118, 0.768810, 0.769908, 0.770319, 0.770038, 0.769097, 0.767553, 0.765503, 0.763025, 0.760119,\
        0.756724, 0.752758, 0.748157, 0.742907, 0.737045, 0.730634, 0.723729, 0.716362, 0.708553, 0.700521,\
        0.693124, 0.685375, 0.676873, 0.667690, 0.658030, 0.648067, 0.637927, 0.627701, 0.617464, 0.607214,\
        0.596519, 0.585077, 0.573332, 0.561385, 0.549333, 0.537311, 0.525481, 0.513992, 0.502766, 0.491968,\
        0.481454],\
        [0.332383, 0.341880, 0.351826, 0.362379, 0.373824, 0.386594, 0.401065, 0.417167, 0.434460, 0.452454,\
        0.470840, 0.489482, 0.508365, 0.527554, 0.546845, 0.566013, 0.584860, 0.603133, 0.620353, 0.636572,\
        0.652002, 0.666427, 0.679465, 0.690653, 0.699539, 0.705991, 0.709623, 0.710674, 0.709710, 0.706753,\
        0.702771, 0.696946, 0.691581, 0.685666, 0.679523, 0.672596, 0.664476, 0.654968, 0.644094, 0.632031,\
        0.619090, 0.605717, 0.592290, 0.579489, 0.568067, 0.558483, 0.551233, 0.546845, 0.545833, 0.548646,\
        0.555597, 0.566774, 0.581955, 0.600371, 0.620874, 0.643223, 0.665592, 0.686331, 0.704404, 0.718801,\
        0.728645, 0.734564, 0.737558, 0.738705, 0.738938, 0.738911, 0.738983, 0.739312, 0.739991, 0.741195,\
        0.743199, 0.746184, 0.750016, 0.754332, 0.758782, 0.763083, 0.767000, 0.770415, 0.773303, 0.775622,\
        0.777460, 0.778934, 0.779881, 0.780045, 0.779494, 0.778264, 0.776379, 0.774031, 0.771359, 0.768335,\
        0.764823, 0.760667, 0.755747, 0.750020, 0.743531, 0.736386, 0.728720, 0.720681, 0.712447, 0.704306,\
        0.696375, 0.688021, 0.678719, 0.668743, 0.658385, 0.647814, 0.637129, 0.626403, 0.615708, 0.605054,\
        0.594123, 0.582581, 0.570689, 0.558595, 0.546406, 0.534245, 0.522277, 0.510819, 0.500248, 0.489214,\
        0.478473],\
        [0.325419, 0.335125, 0.345406, 0.356411, 0.368345, 0.381446, 0.395908, 0.411812, 0.428931, 0.446656,\
        0.464749, 0.483149, 0.501929, 0.521243, 0.540758, 0.560285, 0.579777, 0.598744, 0.615905, 0.632447,\
        0.648407, 0.663385, 0.676999, 0.688898, 0.698697, 0.705567, 0.710206, 0.712809, 0.713299, 0.711779,\
        0.708486, 0.704163, 0.699151, 0.694082, 0.688388, 0.681424, 0.672787, 0.662373, 0.650359, 0.637063,\
        0.622892, 0.608377, 0.594329, 0.581611, 0.570337, 0.560888, 0.553765, 0.549508, 0.548660, 0.551713,\
        0.559044, 0.570815, 0.586828, 0.606270, 0.627809, 0.650501, 0.672737, 0.693037, 0.710541, 0.724705,\
        0.734905, 0.741472, 0.745236, 0.747149, 0.748089, 0.748733, 0.749519, 0.750674, 0.752285, 0.754390,\
        0.757057, 0.760335, 0.764073, 0.767981, 0.771940, 0.775790, 0.779205, 0.782064, 0.784507, 0.786288,\
        0.787580, 0.788806, 0.789722, 0.789476, 0.788666, 0.787181, 0.784852, 0.782178, 0.779371, 0.776314,\
        0.772765, 0.768466, 0.763227, 0.756979, 0.749798, 0.741860, 0.733401, 0.724692, 0.716045, 0.707741,\
        0.699579, 0.690310, 0.679881, 0.669059, 0.658044, 0.646920, 0.635735, 0.624515, 0.613259, 0.602122,\
        0.591009, 0.579547, 0.567667, 0.555530, 0.543285, 0.531049, 0.518924, 0.507020, 0.496047, 0.485777,\
        0.475314],\
        [0.318947, 0.328886, 0.339511, 0.350941, 0.363311, 0.376732, 0.391215, 0.407016, 0.423807, 0.440963,\
        0.458546, 0.476590, 0.495117, 0.514231, 0.533474, 0.552768, 0.572063, 0.590795, 0.608692, 0.626188,\
        0.642949, 0.658563, 0.672750, 0.685208, 0.695418, 0.703116, 0.709406, 0.713660, 0.715749, 0.715753,\
        0.713925, 0.710724, 0.706556, 0.702292, 0.696935, 0.690135, 0.681099, 0.669879, 0.656852, 0.642482,\
        0.627237, 0.611744, 0.597175, 0.584244, 0.572846, 0.563307, 0.556073, 0.551683, 0.550713, 0.553705,\
        0.561110, 0.573195, 0.589858, 0.610382, 0.633527, 0.657609, 0.680588, 0.700850, 0.718096, 0.731798,\
        0.741853, 0.748593, 0.752719, 0.755096, 0.756572, 0.757844, 0.759395, 0.761474, 0.764105, 0.767135,\
        0.770417, 0.774047, 0.777905, 0.781415, 0.784894, 0.788449, 0.791426, 0.793523, 0.795704, 0.796842,\
        0.797468, 0.798113, 0.799391, 0.798327, 0.797401, 0.795752, 0.792773, 0.789831, 0.787004, 0.784018,\
        0.780510, 0.776120, 0.770549, 0.763708, 0.755730, 0.746884, 0.737495, 0.727909, 0.718474, 0.709471,\
        0.700830, 0.690713, 0.679658, 0.668350, 0.656910, 0.645393, 0.633847, 0.622319, 0.610777, 0.599101,\
        0.588000, 0.576436, 0.564483, 0.552314, 0.540082, 0.527908, 0.515890, 0.504183, 0.493049, 0.482546,\
        0.472173],\
        [0.312898, 0.323048, 0.333970, 0.345746, 0.358479, 0.372267, 0.387169, 0.402970, 0.418771, 0.435027,\
        0.451944, 0.469541, 0.487825, 0.506489, 0.524834, 0.543605, 0.562701, 0.581592, 0.600070, 0.618594,\
        0.636079, 0.652266, 0.667000, 0.679997, 0.690962, 0.700134, 0.707925, 0.713756, 0.717479, 0.719127,\
        0.718878, 0.717057, 0.714076, 0.710201, 0.705237, 0.698373, 0.689298, 0.677328, 0.663564, 0.648430,\
        0.632426, 0.616376, 0.601292, 0.587679, 0.575891, 0.566054, 0.558458, 0.553668, 0.552303, 0.554950,\
        0.562132, 0.574250, 0.591492, 0.613367, 0.638449, 0.664634, 0.689340, 0.709948, 0.726692, 0.739853,\
        0.749520, 0.756067, 0.760173, 0.762685, 0.764456, 0.766221, 0.768491, 0.771512, 0.775254, 0.779316,\
        0.783079, 0.786876, 0.791349, 0.794351, 0.797297, 0.800775, 0.803378, 0.804929, 0.806492, 0.807308,\
        0.807493, 0.807674, 0.807862, 0.806930, 0.805619, 0.803428, 0.800151, 0.797092, 0.794339, 0.791443,\
        0.788019, 0.783582, 0.777639, 0.770086, 0.761173, 0.751286, 0.740817, 0.730119, 0.719510, 0.709195,\
        0.699040, 0.688860, 0.677979, 0.666632, 0.655037, 0.643309, 0.631537, 0.619841, 0.608539, 0.597473,\
        0.585403, 0.573377, 0.561232, 0.549002, 0.536810, 0.524790, 0.513038, 0.501575, 0.490441, 0.479688,\
        0.469192],\
        [0.307221, 0.317529, 0.328650, 0.340626, 0.353533, 0.367432, 0.382295, 0.397686, 0.412796, 0.428323,\
        0.444582, 0.461528, 0.479079, 0.496802, 0.514465, 0.533095, 0.552391, 0.571901, 0.591398, 0.610383,\
        0.628082, 0.644751, 0.660109, 0.673904, 0.686079, 0.696791, 0.705914, 0.713249, 0.718635, 0.721967,\
        0.723309, 0.722867, 0.720896, 0.717532, 0.712650, 0.705982, 0.696444, 0.684324, 0.670460, 0.654923,\
        0.638325, 0.621552, 0.605664, 0.591544, 0.579680, 0.569413, 0.561210, 0.555808, 0.553853, 0.555968,\
        0.562748, 0.574704, 0.592240, 0.615638, 0.642922, 0.671136, 0.697580, 0.718926, 0.735637, 0.748516,\
        0.757780, 0.763911, 0.767691, 0.770029, 0.771833, 0.773905, 0.776783, 0.780663, 0.785507, 0.790877,\
        0.795150, 0.798243, 0.803430, 0.806760, 0.809504, 0.812522, 0.814824, 0.816280, 0.817235, 0.817646,\
        0.817534, 0.817174, 0.816512, 0.815249, 0.813388, 0.810796, 0.807614, 0.804432, 0.801572, 0.798552,\
        0.795190, 0.790745, 0.784329, 0.775894, 0.765911, 0.754911, 0.743314, 0.731442, 0.719568, 0.707996,\
        0.697092, 0.686465, 0.675574, 0.664269, 0.652643, 0.640820, 0.628912, 0.616985, 0.605023, 0.593843,\
        0.582261, 0.570150, 0.557875, 0.545578, 0.533399, 0.521504, 0.510033, 0.498811, 0.487684, 0.476847,\
        0.466278],\
        [0.301932, 0.312321, 0.323490, 0.335468, 0.348296, 0.361954, 0.376282, 0.390905, 0.405579, 0.420754,\
        0.436458, 0.452604, 0.469095, 0.485783, 0.503105, 0.521793, 0.541426, 0.561495, 0.581473, 0.600603,\
        0.618802, 0.636131, 0.652199, 0.666834, 0.680141, 0.692219, 0.702923, 0.712004, 0.719184, 0.724245,\
        0.727143, 0.727988, 0.726937, 0.724061, 0.719256, 0.712191, 0.702613, 0.691266, 0.677753, 0.661986,\
        0.644958, 0.627578, 0.610716, 0.596589, 0.584808, 0.573528, 0.564529, 0.558448, 0.555838, 0.557371,\
        0.563782, 0.575748, 0.593895, 0.618883, 0.647966, 0.677431, 0.704464, 0.727021, 0.744596, 0.757638,\
        0.766572, 0.772132, 0.775357, 0.777284, 0.778866, 0.781051, 0.784463, 0.789061, 0.794668, 0.801099,\
        0.806300, 0.810075, 0.814522, 0.818412, 0.821464, 0.824089, 0.826066, 0.827280, 0.827835, 0.827808,\
        0.827276, 0.826351, 0.825035, 0.823235, 0.820907, 0.818057, 0.814852, 0.811600, 0.808450, 0.805246,\
        0.801811, 0.797251, 0.790218, 0.780710, 0.769621, 0.757590, 0.745000, 0.732134, 0.719225, 0.706520,\
        0.694876, 0.684069, 0.673044, 0.661666, 0.649987, 0.638100, 0.626141, 0.614314, 0.602691, 0.590865,\
        0.578961, 0.566768, 0.554413, 0.542048, 0.529821, 0.517901, 0.506474, 0.495519, 0.484580, 0.473860,\
        0.463341],\
        [0.297188, 0.307457, 0.318472, 0.330266, 0.342832, 0.356084, 0.369835, 0.383839, 0.398072, 0.412861,\
        0.427996, 0.443332, 0.458973, 0.475077, 0.492030, 0.510333, 0.530209, 0.550494, 0.570477, 0.589777,\
        0.608520, 0.626590, 0.643262, 0.658542, 0.672839, 0.686343, 0.698849, 0.709929, 0.719090, 0.725926,\
        0.730334, 0.732397, 0.732233, 0.729904, 0.725377, 0.718575, 0.709616, 0.699012, 0.685450, 0.669412,\
        0.652307, 0.634868, 0.618294, 0.604162, 0.590197, 0.578027, 0.568547, 0.561944, 0.558722, 0.559702,\
        0.565911, 0.578359, 0.597800, 0.624384, 0.654904, 0.685402, 0.712772, 0.735842, 0.754089, 0.767406,\
        0.775957, 0.780740, 0.783277, 0.784726, 0.785888, 0.787940, 0.792036, 0.797376, 0.803662, 0.810425,\
        0.816468, 0.821405, 0.825894, 0.829830, 0.832993, 0.835421, 0.837085, 0.837962, 0.838111, 0.837619,\
        0.836581, 0.835080, 0.833153, 0.830795, 0.828011, 0.824851, 0.821444, 0.817951, 0.814443, 0.810843,\
        0.806916, 0.802040, 0.794409, 0.783818, 0.771917, 0.759157, 0.745863, 0.732336, 0.718913, 0.706006,\
        0.693848, 0.682183, 0.670657, 0.659042, 0.647248, 0.635278, 0.623194, 0.611209, 0.599790, 0.587918,\
        0.575676, 0.563336, 0.550913, 0.538500, 0.526212, 0.514186, 0.502621, 0.491770, 0.481247, 0.470788,\
        0.460398],\
        [0.292764, 0.302820, 0.313570, 0.325077, 0.337305, 0.350141, 0.363425, 0.377022, 0.390915, 0.405208,\
        0.419607, 0.434072, 0.449045, 0.464806, 0.481516, 0.499632, 0.519393, 0.539375, 0.559122, 0.578576,\
        0.597721, 0.616302, 0.633129, 0.648653, 0.663849, 0.678995, 0.693634, 0.707035, 0.718379, 0.726985,\
        0.732846, 0.736085, 0.736820, 0.735136, 0.731085, 0.724740, 0.716268, 0.705640, 0.692040, 0.676761,\
        0.660102, 0.642751, 0.625650, 0.609388, 0.594809, 0.583087, 0.573646, 0.566724, 0.562921, 0.563324,\
        0.569387, 0.582663, 0.604104, 0.632510, 0.664275, 0.695620, 0.722981, 0.745840, 0.764332, 0.777959,\
        0.785949, 0.789627, 0.791535, 0.792817, 0.793645, 0.795068, 0.800019, 0.806538, 0.813414, 0.820428,\
        0.826869, 0.832416, 0.837164, 0.841129, 0.844236, 0.846457, 0.847791, 0.848265, 0.847945, 0.846931,\
        0.845324, 0.843217, 0.840674, 0.837739, 0.834453, 0.830876, 0.827092, 0.823169, 0.819100, 0.814745,\
        0.809768, 0.803518, 0.795222, 0.784726, 0.772639, 0.759551, 0.745867, 0.731934, 0.718134, 0.704991,\
        0.692551, 0.680239, 0.668265, 0.656398, 0.644492, 0.632474, 0.620301, 0.607933, 0.595826, 0.584287,\
        0.572223, 0.559880, 0.547449, 0.535053, 0.522796, 0.510787, 0.499256, 0.488538, 0.478149, 0.467833,\
        0.457531],\
        [0.288555, 0.298340, 0.308795, 0.319985, 0.331869, 0.344310, 0.357178, 0.370393, 0.383948, 0.397743,\
        0.411288, 0.424740, 0.439234, 0.454932, 0.471784, 0.489953, 0.509057, 0.527997, 0.547112, 0.566442,\
        0.585930, 0.604797, 0.621086, 0.636518, 0.652707, 0.669932, 0.687370, 0.703625, 0.717047, 0.727305,\
        0.734586, 0.739006, 0.740681, 0.739720, 0.736229, 0.730315, 0.722039, 0.711295, 0.698656, 0.684348,\
        0.668249, 0.650977, 0.633460, 0.616657, 0.601813, 0.589828, 0.580415, 0.573203, 0.568736, 0.568429,\
        0.574241, 0.588302, 0.612260, 0.643387, 0.676484, 0.707728, 0.734601, 0.756486, 0.774539, 0.788767,\
        0.796214, 0.798526, 0.800271, 0.801901, 0.803144, 0.804722, 0.809814, 0.816835, 0.824111, 0.831218,\
        0.837764, 0.843494, 0.848335, 0.852254, 0.855201, 0.857142, 0.858086, 0.858082, 0.857219, 0.855610,\
        0.853373, 0.850614, 0.847424, 0.843872, 0.840019, 0.835920, 0.831620, 0.827123, 0.822350, 0.817097,\
        0.811001, 0.803569, 0.794683, 0.784165, 0.772071, 0.758885, 0.745039, 0.730889, 0.716729, 0.702892,\
        0.690099, 0.677818, 0.665658, 0.653621, 0.641656, 0.629694, 0.617670, 0.605593, 0.593371, 0.581138,\
        0.568858, 0.556462, 0.544056, 0.531762, 0.519688, 0.507907, 0.496603, 0.485884, 0.475449, 0.465111,\
        0.454800],\
        [0.284659, 0.294126, 0.304237, 0.315117, 0.326670, 0.338711, 0.351116, 0.363816, 0.376766, 0.389856,\
        0.402440, 0.414955, 0.429471, 0.445239, 0.461978, 0.479661, 0.497710, 0.515616, 0.533893, 0.552498,\
        0.571364, 0.589704, 0.605767, 0.621586, 0.639092, 0.658916, 0.679583, 0.698918, 0.714523, 0.726601,\
        0.735412, 0.741090, 0.743767, 0.743586, 0.740706, 0.735299, 0.727532, 0.717609, 0.705798, 0.692160,\
        0.676636, 0.659409, 0.641863, 0.625188, 0.610413, 0.598346, 0.589057, 0.581482, 0.576190, 0.574967,\
        0.580629, 0.596017, 0.623316, 0.657959, 0.693415, 0.724345, 0.748800, 0.767461, 0.783064, 0.796699,\
        0.804705, 0.807487, 0.809769, 0.812022, 0.814270, 0.817274, 0.822258, 0.828809, 0.835849, 0.842731,\
        0.849083, 0.854676, 0.859385, 0.863124, 0.865807, 0.867381, 0.867851, 0.867286, 0.865797, 0.863519,\
        0.860588, 0.857127, 0.853243, 0.849025, 0.844543, 0.839842, 0.834940, 0.829802, 0.824312, 0.818243,\
        0.811279, 0.803236, 0.793778, 0.782827, 0.770587, 0.757331, 0.743439, 0.729274, 0.715130, 0.701206,\
        0.687988, 0.675233, 0.662775, 0.650565, 0.638567, 0.626715, 0.614879, 0.602858, 0.590562, 0.578060,\
        0.565540, 0.553062, 0.540700, 0.528559, 0.516784, 0.505473, 0.494416, 0.483625, 0.473044, 0.462586,\
        0.452190],\
        [0.281204, 0.290352, 0.300093, 0.310681, 0.321880, 0.333471, 0.345331, 0.357379, 0.369508, 0.381480,\
        0.392917, 0.405485, 0.420358, 0.435817, 0.451905, 0.468528, 0.485350, 0.502280, 0.519565, 0.536783,\
        0.554259, 0.571560, 0.588050, 0.604422, 0.623548, 0.646480, 0.670082, 0.692069, 0.710435, 0.724720,\
        0.735253, 0.742291, 0.746033, 0.746666, 0.744415, 0.739551, 0.732379, 0.723234, 0.712423, 0.700101,\
        0.684529, 0.667287, 0.650007, 0.633807, 0.619574, 0.607978, 0.598836, 0.590991, 0.584883, 0.582232,\
        0.587600, 0.605764, 0.637453, 0.676287, 0.714831, 0.746110, 0.766460, 0.779093, 0.790049, 0.801165,\
        0.810082, 0.815584, 0.819501, 0.822980, 0.826489, 0.830571, 0.835728, 0.841839, 0.848341, 0.854705,\
        0.860586, 0.865779, 0.870151, 0.873575, 0.875900, 0.877019, 0.876929, 0.875716, 0.873524, 0.870510,\
        0.866826, 0.862612, 0.857990, 0.853063, 0.847904, 0.842555, 0.837013, 0.831229, 0.825088, 0.818384,\
        0.810765, 0.801851, 0.792015, 0.780828, 0.768362, 0.754979, 0.741046, 0.726920, 0.712978, 0.699361,\
        0.685576, 0.672271, 0.659460, 0.647058, 0.635004, 0.623244, 0.611642, 0.599734, 0.587309, 0.574673,\
        0.562053, 0.549567, 0.537301, 0.525333, 0.513733, 0.502869, 0.492210, 0.481410, 0.470722, 0.460133,\
        0.449627],\
        [0.278293, 0.287237, 0.296705, 0.306897, 0.317657, 0.328718, 0.339973, 0.351355, 0.362781, 0.374179,\
        0.385798, 0.398681, 0.412415, 0.426831, 0.441878, 0.457316, 0.472965, 0.488841, 0.504785, 0.520064,\
        0.536454, 0.553571, 0.570608, 0.587915, 0.609296, 0.634509, 0.660208, 0.684455, 0.705369, 0.721891,\
        0.734164, 0.742634, 0.747481, 0.748914, 0.747237, 0.742819, 0.736064, 0.727389, 0.717128, 0.705064,\
        0.690184, 0.673553, 0.657006, 0.641584, 0.628011, 0.616755, 0.607737, 0.600387, 0.594213, 0.589291,\
        0.591843, 0.612479, 0.652223, 0.698691, 0.740422, 0.772682, 0.787879, 0.792706, 0.798289, 0.808057,\
        0.817024, 0.823994, 0.829540, 0.834426, 0.839137, 0.844038, 0.849384, 0.855151, 0.861047, 0.866716,\
        0.871901, 0.876468, 0.880328, 0.883344, 0.885247, 0.885840, 0.885114, 0.883180, 0.880224, 0.876426,\
        0.871948, 0.866939, 0.861542, 0.855874, 0.850015, 0.844001, 0.837816, 0.831404, 0.824674, 0.817518,\
        0.809713, 0.800675, 0.790082, 0.778255, 0.765416, 0.751835, 0.737806, 0.723611, 0.709446, 0.695631,\
        0.682003, 0.668538, 0.655517, 0.642957, 0.630807, 0.618999, 0.607540, 0.595922, 0.583433, 0.570820,\
        0.558259, 0.545873, 0.533786, 0.522118, 0.510858, 0.499734, 0.489615, 0.479050, 0.468309, 0.457618,\
        0.447020],\
        [0.275911, 0.284743, 0.294015, 0.303828, 0.314058, 0.324505, 0.335131, 0.345907, 0.356816, 0.367923,\
        0.379502, 0.391833, 0.404645, 0.418139, 0.432109, 0.446326, 0.460686, 0.475114, 0.489229, 0.503263,\
        0.519586, 0.537457, 0.555867, 0.575625, 0.598385, 0.623921, 0.650707, 0.676757, 0.699979, 0.718200,\
        0.732229, 0.742218, 0.748163, 0.750318, 0.749094, 0.744954, 0.738353, 0.729707, 0.719309, 0.707223,\
        0.693141, 0.677764, 0.662491, 0.648102, 0.635187, 0.624152, 0.615156, 0.608152, 0.602442, 0.597054,\
        0.594718, 0.617292, 0.665590, 0.719377, 0.765029, 0.799094, 0.810855, 0.811575, 0.813632, 0.819628,\
        0.826916, 0.833864, 0.840254, 0.846225, 0.851891, 0.857357, 0.862762, 0.868165, 0.873428, 0.878285,\
        0.882577, 0.886321, 0.889482, 0.892055, 0.893515, 0.893534, 0.892141, 0.889445, 0.885710, 0.881119,\
        0.875822, 0.869989, 0.863790, 0.857366, 0.850808, 0.844148, 0.837354, 0.830339, 0.823002, 0.815312,\
        0.807332, 0.798429, 0.787190, 0.774818, 0.761649, 0.747880, 0.733740, 0.719486, 0.705368, 0.691281,\
        0.677511, 0.664014, 0.650899, 0.638247, 0.626030, 0.614072, 0.602190, 0.590733, 0.578785, 0.566488,\
        0.554143, 0.541939, 0.530048, 0.518698, 0.508306, 0.498653, 0.487507, 0.476531, 0.465679, 0.454932,\
        0.444288],\
        [0.273941, 0.282690, 0.291835, 0.301326, 0.310948, 0.320747, 0.330765, 0.340992, 0.351438, 0.362175,\
        0.373336, 0.384984, 0.397110, 0.409797, 0.422729, 0.435737, 0.448830, 0.461958, 0.475109, 0.488984,\
        0.505552, 0.523937, 0.543484, 0.564735, 0.588267, 0.614111, 0.641497, 0.668815, 0.693253, 0.713510,\
        0.729704, 0.741264, 0.748163, 0.750871, 0.749959, 0.745970, 0.739375, 0.730572, 0.719874, 0.707517,\
        0.694185, 0.680537, 0.666826, 0.653620, 0.641378, 0.630511, 0.621373, 0.614184, 0.608940, 0.605839,\
        0.605296, 0.632713, 0.683973, 0.739067, 0.786299, 0.817182, 0.830379, 0.832067, 0.831968, 0.834300,\
        0.838989, 0.844978, 0.851543, 0.858205, 0.864541, 0.870274, 0.875482, 0.880391, 0.884983, 0.888948,\
        0.892123, 0.894926, 0.897109, 0.899179, 0.900181, 0.899671, 0.897662, 0.894252, 0.889816, 0.884469,\
        0.878344, 0.871661, 0.864642, 0.857462, 0.850230, 0.842979, 0.835661, 0.828140, 0.820209, 0.811717,\
        0.802838, 0.793557, 0.782621, 0.770259, 0.756997, 0.743143, 0.728933, 0.714575, 0.700315, 0.686358,\
        0.672488, 0.658903, 0.645705, 0.633004, 0.620859, 0.609080, 0.597323, 0.585644, 0.573887, 0.561902,\
        0.549827, 0.537831, 0.526082, 0.514742, 0.503956, 0.494369, 0.484180, 0.473458, 0.462704, 0.452007,\
        0.441389],\
        [0.272240, 0.280849, 0.289772, 0.298929, 0.308021, 0.317270, 0.326759, 0.336487, 0.346468, 0.356750,\
        0.367402, 0.378473, 0.389992, 0.401907, 0.413755, 0.425638, 0.437712, 0.450047, 0.462949, 0.477241,\
        0.493816, 0.512102, 0.532476, 0.554713, 0.578477, 0.603739, 0.632972, 0.661802, 0.687194, 0.709127,\
        0.727377, 0.739997, 0.747438, 0.750466, 0.749781, 0.745954, 0.739441, 0.730629, 0.719909, 0.707873,\
        0.695396, 0.683006, 0.670824, 0.658832, 0.647268, 0.636572, 0.627347, 0.620316, 0.616426, 0.617487,\
        0.628913, 0.662158, 0.712289, 0.762878, 0.808849, 0.836534, 0.848623, 0.850202, 0.848716, 0.848688,\
        0.851354, 0.856342, 0.862848, 0.870013, 0.876862, 0.882528, 0.887145, 0.891323, 0.895151, 0.898252,\
        0.900078, 0.901831, 0.903163, 0.904243, 0.904673, 0.903747, 0.901305, 0.897419, 0.892494, 0.886424,\
        0.879431, 0.871864, 0.864017, 0.856099, 0.848239, 0.840478, 0.832769, 0.824962, 0.816754, 0.807671,\
        0.797942, 0.788188, 0.777166, 0.764839, 0.751587, 0.737742, 0.723564, 0.709240, 0.694928, 0.680919,\
        0.667055, 0.653406, 0.640105, 0.627300, 0.615210, 0.603986, 0.592410, 0.580718, 0.569046, 0.557303,\
        0.545496, 0.533718, 0.522116, 0.510901, 0.500254, 0.490138, 0.480217, 0.469923, 0.459420, 0.448864,\
        0.438327],\
        [0.270717, 0.279108, 0.287689, 0.296375, 0.305109, 0.313972, 0.323030, 0.332304, 0.341810, 0.351583,\
        0.361671, 0.372117, 0.382929, 0.393974, 0.404892, 0.415927, 0.427270, 0.439044, 0.451675, 0.465881,\
        0.482155, 0.500693, 0.522171, 0.545355, 0.570165, 0.597355, 0.627094, 0.656059, 0.683089, 0.706861,\
        0.725321, 0.738009, 0.745524, 0.748761, 0.748397, 0.744948, 0.738823, 0.730387, 0.720056, 0.708437,\
        0.696770, 0.685959, 0.675315, 0.664527, 0.653615, 0.643083, 0.633890, 0.627381, 0.625414, 0.630926,\
        0.648777, 0.683435, 0.729739, 0.778341, 0.825990, 0.856623, 0.866670, 0.865972, 0.862818, 0.861248,\
        0.862688, 0.867053, 0.873520, 0.881086, 0.888535, 0.893781, 0.897368, 0.900538, 0.903327, 0.905291,\
        0.906257, 0.906899, 0.907371, 0.907592, 0.907301, 0.906019, 0.903388, 0.899315, 0.893911, 0.886944,\
        0.878987, 0.870514, 0.861852, 0.853234, 0.844803, 0.836614, 0.828635, 0.820762, 0.812827, 0.804127,\
        0.793997, 0.783138, 0.771459, 0.758896, 0.745615, 0.731826, 0.717729, 0.703638, 0.689487, 0.675382,\
        0.661467, 0.647757, 0.634360, 0.621408, 0.608989, 0.597689, 0.586914, 0.575635, 0.564266, 0.552825,\
        0.541318, 0.529788, 0.518328, 0.507167, 0.496660, 0.486524, 0.476505, 0.466348, 0.456017, 0.445593,\
        0.435149],\
        [0.269365, 0.277506, 0.285738, 0.294027, 0.302393, 0.310900, 0.319576, 0.328416, 0.337429, 0.346643,\
        0.356100, 0.365829, 0.375813, 0.385942, 0.396120, 0.406522, 0.417305, 0.428457, 0.440505, 0.454327,\
        0.470610, 0.489822, 0.511673, 0.534931, 0.561039, 0.590668, 0.619906, 0.647515, 0.675944, 0.702123,\
        0.721326, 0.733693, 0.741415, 0.745176, 0.745525, 0.742902, 0.737659, 0.730135, 0.720772, 0.710261,\
        0.699684, 0.690061, 0.680937, 0.671335, 0.660973, 0.650470, 0.641218, 0.635151, 0.634686, 0.642769,\
        0.662627, 0.696927, 0.744316, 0.795940, 0.849458, 0.881449, 0.884481, 0.879711, 0.874737, 0.872034,\
        0.872787, 0.876894, 0.883298, 0.890723, 0.898405, 0.903346, 0.905989, 0.908083, 0.909610, 0.910387,\
        0.910454, 0.910145, 0.909696, 0.909112, 0.908202, 0.906648, 0.904140, 0.900353, 0.893865, 0.885781,\
        0.876865, 0.867540, 0.858127, 0.848870, 0.839931, 0.831380, 0.823190, 0.815221, 0.807440, 0.800108,\
        0.789353, 0.777604, 0.765305, 0.752472, 0.739194, 0.725570, 0.711614, 0.697523, 0.683676, 0.669762,\
        0.655895, 0.642185, 0.628760, 0.615830, 0.603614, 0.592163, 0.581416, 0.570614, 0.559646, 0.548568,\
        0.537412, 0.526206, 0.514972, 0.503812, 0.493275, 0.483194, 0.473109, 0.462932, 0.452647, 0.442291,\
        0.431914],\
        [0.268210, 0.276126, 0.284057, 0.291994, 0.299988, 0.308143, 0.316452, 0.324851, 0.333335, 0.341940,\
        0.350710, 0.359673, 0.368829, 0.378151, 0.387652, 0.397453, 0.407700, 0.417880, 0.428885, 0.441930,\
        0.457769, 0.476715, 0.497870, 0.520224, 0.545228, 0.574348, 0.602805, 0.630594, 0.661411, 0.690987,\
        0.710901, 0.724872, 0.734031, 0.739149, 0.740894, 0.739727, 0.735959, 0.729910, 0.722044, 0.713019,\
        0.703718, 0.695538, 0.687963, 0.679541, 0.669514, 0.658710, 0.649063, 0.643015, 0.643363, 0.653023,\
        0.674291, 0.708987, 0.759432, 0.820044, 0.878315, 0.907480, 0.901616, 0.893195, 0.886295, 0.882331,\
        0.882482, 0.886514, 0.892815, 0.899558, 0.905828, 0.910553, 0.912979, 0.914160, 0.914485, 0.914057,\
        0.913034, 0.911687, 0.910265, 0.908862, 0.907330, 0.905358, 0.902680, 0.898129, 0.891280, 0.882573,\
        0.872967, 0.862959, 0.852907, 0.843094, 0.833719, 0.824858, 0.816500, 0.808518, 0.800196, 0.792144,\
        0.782335, 0.770764, 0.758374, 0.745480, 0.732313, 0.719090, 0.705616, 0.691819, 0.678007, 0.664214,\
        0.650476, 0.636872, 0.623501, 0.610596, 0.598638, 0.587405, 0.576641, 0.566018, 0.555346, 0.544597,\
        0.533801, 0.523002, 0.512228, 0.501408, 0.490749, 0.480333, 0.470014, 0.459703, 0.449366, 0.439011,\
        0.428663],\
        [0.267275, 0.275017, 0.282720, 0.290366, 0.297997, 0.305820, 0.313748, 0.321658, 0.329562, 0.337507,\
        0.345548, 0.353724, 0.362063, 0.370591, 0.379375, 0.388540, 0.398015, 0.406750, 0.416482, 0.428346,\
        0.443016, 0.460517, 0.480013, 0.500317, 0.522402, 0.549320, 0.577340, 0.606497, 0.638127, 0.668390,\
        0.693057, 0.711055, 0.723057, 0.730494, 0.734434, 0.735424, 0.733701, 0.729581, 0.723591, 0.716457,\
        0.709073, 0.702302, 0.695874, 0.688939, 0.678803, 0.667293, 0.656844, 0.650320, 0.650857, 0.661705,\
        0.685884, 0.723043, 0.773302, 0.836193, 0.894322, 0.921505, 0.920145, 0.910044, 0.900384, 0.894340,\
        0.893554, 0.897252, 0.903106, 0.908870, 0.913786, 0.917283, 0.918989, 0.919219, 0.918356, 0.916663,\
        0.914394, 0.911845, 0.909315, 0.907020, 0.904844, 0.902197, 0.898912, 0.893942, 0.886483, 0.877466,\
        0.867458, 0.856964, 0.846395, 0.836109, 0.826381, 0.817291, 0.808703, 0.800759, 0.792658, 0.783818,\
        0.774016, 0.762885, 0.750766, 0.737996, 0.724907, 0.712084, 0.699707, 0.686060, 0.672380, 0.658776,\
        0.645276, 0.631937, 0.618838, 0.605991, 0.594083, 0.583139, 0.572452, 0.561918, 0.551423, 0.540915,\
        0.530401, 0.519934, 0.509588, 0.499312, 0.488408, 0.477652, 0.467077, 0.456597, 0.446164, 0.435769,\
        0.425420],\
        [0.266552, 0.274184, 0.281744, 0.289216, 0.296630, 0.304085, 0.311523, 0.318864, 0.326127, 0.333372,\
        0.340659, 0.348040, 0.355551, 0.363228, 0.371109, 0.379190, 0.387228, 0.395022, 0.403695, 0.414129,\
        0.427414, 0.443429, 0.460870, 0.478768, 0.498679, 0.525018, 0.553404, 0.582345, 0.612630, 0.644085,\
        0.671714, 0.693463, 0.709169, 0.719527, 0.726447, 0.730232, 0.730950, 0.728953, 0.724915, 0.719692,\
        0.714127, 0.708750, 0.703798, 0.697846, 0.687507, 0.675286, 0.663785, 0.656488, 0.656771, 0.667957,\
        0.693449, 0.735116, 0.788763, 0.852340, 0.906244, 0.931760, 0.935330, 0.928232, 0.918716, 0.910982,\
        0.908689, 0.910967, 0.915153, 0.919288, 0.922536, 0.924459, 0.924859, 0.923840, 0.921661, 0.918600,\
        0.914956, 0.911056, 0.907241, 0.903869, 0.901121, 0.897805, 0.893540, 0.887848, 0.880218, 0.871034,\
        0.860790, 0.849951, 0.838957, 0.828245, 0.818229, 0.809109, 0.800533, 0.792252, 0.784105, 0.775244,\
        0.765468, 0.754661, 0.742925, 0.730436, 0.717381, 0.704127, 0.692170, 0.679687, 0.666589, 0.653391,\
        0.640261, 0.627315, 0.614715, 0.602730, 0.590823, 0.579528, 0.568757, 0.558243, 0.547844, 0.537482,\
        0.527126, 0.516786, 0.506474, 0.496115, 0.485537, 0.474814, 0.464134, 0.453531, 0.443004, 0.432554,\
        0.422189],\
        [0.265996, 0.273569, 0.281058, 0.288436, 0.295692, 0.302798, 0.309708, 0.316430, 0.323014, 0.329539,\
        0.336078, 0.342680, 0.349375, 0.356181, 0.363095, 0.370061, 0.376977, 0.383956, 0.391569, 0.400562,\
        0.412867, 0.428100, 0.444481, 0.461948, 0.482079, 0.506808, 0.534251, 0.562301, 0.591583, 0.622509,\
        0.650283, 0.674291, 0.693276, 0.707190, 0.717814, 0.724737, 0.727936, 0.727861, 0.725463, 0.721784,\
        0.717684, 0.713644, 0.709526, 0.703725, 0.694614, 0.681512, 0.668946, 0.661066, 0.661371, 0.673226,\
        0.700903, 0.747869, 0.805040, 0.864399, 0.912891, 0.938861, 0.947177, 0.944813, 0.938079, 0.931496,\
        0.927987, 0.927784, 0.929307, 0.931082, 0.932216, 0.932245, 0.930978, 0.928437, 0.924788, 0.920267,\
        0.915167, 0.909851, 0.904733, 0.900097, 0.896484, 0.892537, 0.887537, 0.881187, 0.873278, 0.863951,\
        0.853546, 0.842460, 0.831120, 0.819997, 0.809656, 0.800580, 0.792238, 0.783975, 0.775610, 0.766717,\
        0.757046, 0.746539, 0.735262, 0.723304, 0.710720, 0.697870, 0.685568, 0.673381, 0.660823, 0.648086,\
        0.635380, 0.622834, 0.610546, 0.598662, 0.587262, 0.576121, 0.565354, 0.554866, 0.544537, 0.534272,\
        0.523999, 0.513672, 0.503306, 0.492960, 0.482488, 0.471847, 0.461144, 0.450465, 0.439859, 0.429356,\
        0.418972],\
        [0.265531, 0.273073, 0.280524, 0.287838, 0.294940, 0.301735, 0.308160, 0.314261, 0.320162, 0.325990,\
        0.331824, 0.337708, 0.343658, 0.349678, 0.355753, 0.361861, 0.368019, 0.374414, 0.381492, 0.390089,\
        0.401910, 0.416223, 0.431942, 0.449567, 0.470088, 0.494076, 0.519858, 0.546713, 0.575022, 0.602981,\
        0.629941, 0.655818, 0.678073, 0.696033, 0.709809, 0.719844, 0.724979, 0.726161, 0.724771, 0.722050,\
        0.718876, 0.715609, 0.711928, 0.706727, 0.697726, 0.684175, 0.671310, 0.663525, 0.664629, 0.678504,\
        0.709353, 0.758899, 0.816853, 0.870814, 0.914729, 0.943277, 0.956463, 0.959097, 0.956108, 0.951456,\
        0.947667, 0.945522, 0.944510, 0.943762, 0.942576, 0.940523, 0.937404, 0.933194, 0.927995, 0.921994,\
        0.915449, 0.908710, 0.902389, 0.896983, 0.892181, 0.887276, 0.881524, 0.874584, 0.866323, 0.856819,\
        0.846299, 0.835069, 0.823509, 0.812130, 0.801457, 0.792456, 0.784214, 0.775998, 0.767532, 0.758580,\
        0.748981, 0.738710, 0.727887, 0.716644, 0.704843, 0.692439, 0.680001, 0.667690, 0.655327, 0.642928,\
        0.630613, 0.618476, 0.606549, 0.594905, 0.583670, 0.572735, 0.562079, 0.551668, 0.541425, 0.531264,\
        0.521089, 0.510812, 0.500425, 0.490059, 0.479572, 0.468911, 0.458158, 0.447406, 0.436727, 0.426172,\
        0.415771],\
        [0.265071, 0.272577, 0.279991, 0.287252, 0.294244, 0.300778, 0.306750, 0.312222, 0.317474, 0.322678,\
        0.327908, 0.333187, 0.338522, 0.343913, 0.349365, 0.354905, 0.360630, 0.366789, 0.373869, 0.382678,\
        0.394005, 0.407231, 0.422255, 0.439816, 0.460225, 0.483259, 0.507780, 0.533068, 0.559019, 0.584875,\
        0.612095, 0.639665, 0.665270, 0.687199, 0.704676, 0.716322, 0.722088, 0.723575, 0.722466, 0.720112,\
        0.717337, 0.714347, 0.710635, 0.705077, 0.695815, 0.682674, 0.670470, 0.663386, 0.665452, 0.680790,\
        0.713179, 0.761937, 0.818747, 0.873766, 0.917776, 0.947226, 0.964177, 0.971156, 0.971766, 0.969223,\
        0.965765, 0.962470, 0.959482, 0.956483, 0.953076, 0.948962, 0.943978, 0.938092, 0.931373, 0.923978,\
        0.916138, 0.908166, 0.900566, 0.894287, 0.888518, 0.882577, 0.875982, 0.868415, 0.859757, 0.850056,\
        0.839481, 0.828260, 0.816655, 0.805279, 0.794912, 0.785651, 0.777119, 0.768704, 0.760076, 0.751028,\
        0.741427, 0.731253, 0.720654, 0.710004, 0.699071, 0.686953, 0.674626, 0.662330, 0.650074, 0.637897,\
        0.625905, 0.614177, 0.602657, 0.591291, 0.580185, 0.569388, 0.558860, 0.548557, 0.538424, 0.528401,\
        0.518398, 0.508261, 0.497956, 0.487496, 0.476860, 0.466077, 0.455220, 0.444375, 0.433617, 0.423010,\
        0.412598],\
        [0.264534, 0.271973, 0.279311, 0.286491, 0.293406, 0.299808, 0.305258, 0.310117, 0.314825, 0.319554,\
        0.324333, 0.329162, 0.334045, 0.338995, 0.344043, 0.349259, 0.354787, 0.360894, 0.368006, 0.376659,\
        0.387215, 0.399613, 0.414208, 0.431609, 0.451567, 0.473580, 0.496836, 0.520918, 0.545414, 0.570473,\
        0.597089, 0.625498, 0.654078, 0.680061, 0.700690, 0.712819, 0.718455, 0.719583, 0.718305, 0.715981,\
        0.713315, 0.710365, 0.706437, 0.699988, 0.690298, 0.678205, 0.667068, 0.660863, 0.663559, 0.679150,\
        0.711727, 0.760680, 0.817771, 0.873470, 0.918199, 0.950656, 0.970671, 0.981132, 0.984819, 0.984280,\
        0.981480, 0.977625, 0.973245, 0.968434, 0.963099, 0.957114, 0.950401, 0.942955, 0.934842, 0.926211,\
        0.917299, 0.908453, 0.900137, 0.892587, 0.885593, 0.878584, 0.871107, 0.862876, 0.853776, 0.843853,\
        0.833277, 0.822279, 0.811030, 0.799559, 0.789418, 0.779946, 0.770983, 0.762213, 0.753340, 0.744156,\
        0.734518, 0.724348, 0.713660, 0.702778, 0.692406, 0.681084, 0.669179, 0.657091, 0.644972, 0.632935,\
        0.621142, 0.609772, 0.598661, 0.587532, 0.576647, 0.566034, 0.555657, 0.545474, 0.535446, 0.525553,\
        0.515782, 0.505999, 0.495733, 0.485103, 0.474258, 0.463318, 0.452341, 0.441392, 0.430549, 0.419890,\
        0.409472],\
        [0.263868, 0.271185, 0.278373, 0.285370, 0.292065, 0.298222, 0.303308, 0.307708, 0.312122, 0.316605,\
        0.321120, 0.325666, 0.330262, 0.334947, 0.339778, 0.344853, 0.350330, 0.356450, 0.363534, 0.371916,\
        0.381821, 0.393422, 0.407616, 0.424715, 0.443963, 0.464969, 0.487249, 0.510288, 0.533284, 0.557219,\
        0.583090, 0.611961, 0.642887, 0.670928, 0.693191, 0.707452, 0.712894, 0.713641, 0.712373, 0.710201,\
        0.707703, 0.704815, 0.700758, 0.693925, 0.683984, 0.672585, 0.662415, 0.656963, 0.660065, 0.675811,\
        0.708641, 0.759132, 0.818150, 0.875696, 0.921431, 0.953527, 0.975713, 0.988999, 0.995256, 0.996507,\
        0.994488, 0.990450, 0.985149, 0.978971, 0.972073, 0.964510, 0.956307, 0.947506, 0.938189, 0.928497,\
        0.918659, 0.908997, 0.899827, 0.891317, 0.883250, 0.875240, 0.866907, 0.857998, 0.848415, 0.838210,\
        0.827580, 0.816819, 0.806247, 0.795670, 0.784877, 0.774929, 0.765537, 0.756403, 0.747272, 0.737956,\
        0.728318, 0.718253, 0.707693, 0.696852, 0.686246, 0.675319, 0.663826, 0.652001, 0.640039, 0.628084,\
        0.616298, 0.604987, 0.594241, 0.583561, 0.573019, 0.562655, 0.552456, 0.542394, 0.532438, 0.522571,\
        0.512826, 0.503332, 0.493338, 0.482532, 0.471597, 0.460578, 0.449518, 0.438482, 0.427556, 0.416843,\
        0.406429],\
        [0.263057, 0.270201, 0.277165, 0.283867, 0.290175, 0.295855, 0.300586, 0.304954, 0.309447, 0.313910,\
        0.318319, 0.322719, 0.327170, 0.331735, 0.336495, 0.341559, 0.347081, 0.353263, 0.360346, 0.368577,\
        0.378185, 0.389493, 0.403129, 0.419185, 0.437191, 0.456881, 0.477938, 0.499509, 0.520765, 0.543625,\
        0.569302, 0.599285, 0.630579, 0.659109, 0.682897, 0.698763, 0.704152, 0.705763, 0.705498, 0.704080,\
        0.702207, 0.699708, 0.695443, 0.688282, 0.678411, 0.667444, 0.657885, 0.652893, 0.656102, 0.671599,\
        0.703988, 0.754642, 0.813813, 0.870481, 0.917761, 0.953679, 0.978771, 0.994663, 1.003108, 1.005909,\
        1.004672, 1.000664, 0.994786, 0.987621, 0.979524, 0.970710, 0.961312, 0.951425, 0.941143, 0.930581,\
        0.919927, 0.909483, 0.899571, 0.890206, 0.881273, 0.872403, 0.863280, 0.853711, 0.843617, 0.833044,\
        0.822193, 0.811397, 0.800853, 0.790515, 0.780161, 0.770118, 0.760470, 0.751064, 0.741721, 0.732299,\
        0.722702, 0.712880, 0.702651, 0.691958, 0.681137, 0.670138, 0.658807, 0.647182, 0.635384, 0.623550,\
        0.611795, 0.600267, 0.589841, 0.579612, 0.569416, 0.559302, 0.549285, 0.539346, 0.529447, 0.519537,\
        0.509572, 0.499603, 0.489881, 0.479569, 0.468821, 0.457855, 0.446784, 0.435697, 0.424699, 0.413927,\
        0.403521],\
        [0.262128, 0.269070, 0.275781, 0.282167, 0.288090, 0.293360, 0.297907, 0.302445, 0.307150, 0.311628,\
        0.315986, 0.320326, 0.324733, 0.329289, 0.334084, 0.339231, 0.344868, 0.351164, 0.358310, 0.366506,\
        0.375982, 0.387046, 0.399959, 0.414522, 0.430840, 0.448641, 0.467507, 0.486838, 0.507015, 0.527703,\
        0.551862, 0.581984, 0.613799, 0.642702, 0.667320, 0.684406, 0.693308, 0.697531, 0.699511, 0.699894,\
        0.699514, 0.697444, 0.692516, 0.684695, 0.674605, 0.663768, 0.654483, 0.649647, 0.652645, 0.667354,\
        0.698369, 0.748505, 0.807820, 0.864648, 0.913645, 0.952270, 0.980049, 0.998185, 1.008432, 1.012521,\
        1.011991, 1.008119, 1.001894, 0.994041, 0.985069, 0.975327, 0.965046, 0.954381, 0.943428, 0.932259,\
        0.920960, 0.909727, 0.899042, 0.889132, 0.879563, 0.869932, 0.860064, 0.849870, 0.839291, 0.828316,\
        0.817088, 0.806047, 0.795553, 0.785350, 0.775254, 0.765302, 0.755568, 0.746006, 0.736514, 0.727006,\
        0.717404, 0.707684, 0.697854, 0.687264, 0.676410, 0.665387, 0.654145, 0.642682, 0.631061, 0.619428,\
        0.608025, 0.596928, 0.586242, 0.576022, 0.565979, 0.556036, 0.546175, 0.536370, 0.526572, 0.516718,\
        0.506734, 0.496764, 0.486882, 0.476684, 0.466093, 0.455228, 0.444206, 0.433123, 0.422079, 0.411238,\
        0.400841],\
        [0.261130, 0.267874, 0.274355, 0.280495, 0.286203, 0.291417, 0.296250, 0.300983, 0.305460, 0.309820,\
        0.314122, 0.318449, 0.322886, 0.327513, 0.332421, 0.337714, 0.343517, 0.349968, 0.357218, 0.365416,\
        0.374708, 0.385219, 0.396997, 0.410196, 0.424781, 0.440317, 0.456598, 0.473611, 0.491213, 0.507243,\
        0.527964, 0.556719, 0.589564, 0.621102, 0.648584, 0.668773, 0.681754, 0.689926, 0.695286, 0.698623,\
        0.700026, 0.697723, 0.691958, 0.683403, 0.672910, 0.661955, 0.652688, 0.647816, 0.650482, 0.664241,\
        0.693414, 0.741960, 0.801095, 0.858462, 0.909367, 0.950214, 0.979827, 0.999601, 1.011240, 1.016363,\
        1.016436, 1.012733, 1.006306, 0.997986, 0.988403, 0.978020, 0.967160, 0.956036, 0.944762, 0.933366,\
        0.921819, 0.910131, 0.898744, 0.888420, 0.878130, 0.867649, 0.856998, 0.846220, 0.835280, 0.824057,\
        0.812487, 0.801023, 0.790561, 0.780451, 0.770412, 0.760487, 0.750725, 0.741094, 0.731512, 0.721939,\
        0.712378, 0.702533, 0.692676, 0.682439, 0.671753, 0.660823, 0.649716, 0.638446, 0.627031, 0.615542,\
        0.604256, 0.593459, 0.582929, 0.572706, 0.562703, 0.552851, 0.543116, 0.533455, 0.523814, 0.514150,\
        0.504436, 0.494482, 0.484387, 0.474099, 0.463546, 0.452770, 0.441848, 0.430853, 0.419852, 0.408968,\
        0.398564],\
        [0.260116, 0.266685, 0.272984, 0.278962, 0.284579, 0.289847, 0.294849, 0.299615, 0.304064, 0.308357,\
        0.312643, 0.317013, 0.321541, 0.326300, 0.331373, 0.336854, 0.342851, 0.349480, 0.356852, 0.365065,\
        0.374180, 0.384180, 0.394976, 0.406832, 0.419422, 0.432365, 0.445572, 0.458870, 0.471729, 0.484412,\
        0.500801, 0.528078, 0.561300, 0.596360, 0.628124, 0.651725, 0.668561, 0.681155, 0.690829, 0.697491,\
        0.700436, 0.698834, 0.692856, 0.683892, 0.673036, 0.661830, 0.652402, 0.647382, 0.649786, 0.662911,\
        0.690411, 0.735443, 0.791895, 0.849883, 0.904491, 0.946796, 0.977620, 0.998673, 1.011454, 1.017433,\
        1.018012, 1.014472, 1.007922, 0.999285, 0.989291, 0.978500, 0.967321, 0.956032, 0.944789, 0.933612,\
        0.922388, 0.910990, 0.899609, 0.888219, 0.876793, 0.865205, 0.853645, 0.842308, 0.831219, 0.820157,\
        0.808735, 0.797140, 0.786323, 0.775861, 0.765616, 0.755604, 0.745833, 0.736232, 0.726642, 0.716956,\
        0.707473, 0.698184, 0.688021, 0.677684, 0.667070, 0.656260, 0.645342, 0.634347, 0.623252, 0.612002,\
        0.600712, 0.590054, 0.579685, 0.569479, 0.559473, 0.549664, 0.540031, 0.530531, 0.521074, 0.511544,\
        0.501992, 0.492187, 0.482043, 0.471693, 0.461152, 0.450465, 0.439699, 0.428914, 0.418151, 0.407450,\
        0.397134],\
        [0.259131, 0.265556, 0.271715, 0.277582, 0.283144, 0.288422, 0.293458, 0.298253, 0.302762, 0.307090,\
        0.311444, 0.315928, 0.320607, 0.325543, 0.330812, 0.336500, 0.342700, 0.349506, 0.357001, 0.365246,\
        0.374253, 0.383973, 0.394292, 0.404766, 0.415094, 0.425404, 0.435591, 0.445488, 0.455056, 0.465294,\
        0.479581, 0.502146, 0.532354, 0.568777, 0.604938, 0.631648, 0.652243, 0.669676, 0.684762, 0.695989,\
        0.701362, 0.700356, 0.694559, 0.685473, 0.674333, 0.662761, 0.652977, 0.647618, 0.649705, 0.662452,\
        0.688657, 0.729130, 0.779033, 0.835957, 0.894171, 0.939696, 0.972521, 0.995091, 1.009019, 1.015771,\
        1.016769, 1.013352, 1.006706, 0.997840, 0.987571, 0.976540, 0.965230, 0.953989, 0.943039, 0.932451,\
        0.922062, 0.911330, 0.899759, 0.887681, 0.874961, 0.862000, 0.849359, 0.837442, 0.826365, 0.815886,\
        0.805321, 0.793807, 0.782231, 0.771163, 0.760572, 0.750422, 0.740690, 0.731287, 0.721956, 0.712244,\
        0.702099, 0.692987, 0.683222, 0.672858, 0.662231, 0.651506, 0.640792, 0.630140, 0.619536, 0.608843,\
        0.597920, 0.587079, 0.576511, 0.566201, 0.556145, 0.546343, 0.536793, 0.527476, 0.518300, 0.508992,\
        0.499258, 0.489701, 0.479655, 0.469301, 0.458790, 0.448209, 0.437637, 0.427156, 0.416831, 0.406679,\
        0.396624],\
        [0.258207, 0.264516, 0.270569, 0.276349, 0.281856, 0.287106, 0.292124, 0.296934, 0.301513, 0.305938,\
        0.310455, 0.315129, 0.320007, 0.325148, 0.330626, 0.336518, 0.342907, 0.349870, 0.357466, 0.365728,\
        0.374633, 0.384074, 0.393779, 0.403123, 0.411741, 0.419944, 0.427821, 0.435402, 0.442988, 0.451653,\
        0.463500, 0.480596, 0.504516, 0.538521, 0.576102, 0.607181, 0.631702, 0.654182, 0.675976, 0.694116,\
        0.703423, 0.702766, 0.696413, 0.687423, 0.675868, 0.663773, 0.653391, 0.647286, 0.648550, 0.660725,\
        0.686545, 0.725333, 0.771130, 0.823927, 0.880903, 0.929228, 0.964585, 0.988970, 1.004099, 1.011534,\
        1.012831, 1.009452, 1.002687, 0.993627, 0.983166, 0.972001, 0.960666, 0.949565, 0.938993, 0.929120,\
        0.919962, 0.910760, 0.899541, 0.886295, 0.871824, 0.857255, 0.843406, 0.830844, 0.819764, 0.809756,\
        0.800086, 0.788734, 0.776969, 0.765639, 0.754849, 0.744613, 0.734971, 0.725925, 0.717303, 0.708545,\
        0.698750, 0.688627, 0.678355, 0.667758, 0.657032, 0.646339, 0.635792, 0.625458, 0.615344, 0.605366,\
        0.594773, 0.583797, 0.573068, 0.562661, 0.552561, 0.542755, 0.533249, 0.524080, 0.515267, 0.506643,\
        0.497499, 0.487407, 0.477161, 0.466778, 0.456318, 0.445853, 0.435469, 0.425265, 0.415344, 0.405704,\
        0.395890],\
        [0.257367, 0.263587, 0.269558, 0.275272, 0.280728, 0.285935, 0.290909, 0.295667, 0.300270, 0.304903,\
        0.309687, 0.314597, 0.319691, 0.325041, 0.330717, 0.336794, 0.343342, 0.350422, 0.358076, 0.366310,\
        0.375070, 0.384192, 0.393325, 0.401840, 0.409353, 0.416227, 0.422731, 0.429095, 0.435745, 0.443517,\
        0.453604, 0.467153, 0.485677, 0.514757, 0.550105, 0.581355, 0.607652, 0.633919, 0.662443, 0.689389,\
        0.704888, 0.704283, 0.698424, 0.688366, 0.676220, 0.663676, 0.652566, 0.645132, 0.644355, 0.654793,\
        0.682018, 0.724796, 0.768798, 0.813874, 0.867284, 0.917267, 0.954807, 0.980980, 0.997151, 1.005017,\
        1.006392, 1.002911, 0.995964, 0.986712, 0.976106, 0.964878, 0.953563, 0.942579, 0.932278, 0.922836,\
        0.914519, 0.907783, 0.897771, 0.882483, 0.866143, 0.850179, 0.835242, 0.821983, 0.810963, 0.801078,\
        0.791405, 0.780958, 0.769891, 0.758868, 0.748165, 0.737924, 0.728326, 0.719558, 0.711590, 0.704226,\
        0.695138, 0.684088, 0.673050, 0.662101, 0.651248, 0.640558, 0.630120, 0.620003, 0.610120, 0.600333,\
        0.590227, 0.579580, 0.568997, 0.558663, 0.548610, 0.538826, 0.529310, 0.520127, 0.511442, 0.503115,\
        0.494249, 0.484437, 0.474270, 0.463971, 0.453619, 0.443284, 0.433045, 0.422990, 0.413217, 0.403777,\
        0.394169],\
        [0.256626, 0.262783, 0.268698, 0.274367, 0.279790, 0.284977, 0.289943, 0.294709, 0.299369, 0.304239,\
        0.309232, 0.314340, 0.319629, 0.325164, 0.331013, 0.337239, 0.343904, 0.351052, 0.358709, 0.366863,\
        0.375439, 0.384261, 0.392974, 0.401005, 0.407953, 0.414222, 0.420231, 0.426323, 0.432949, 0.440783,\
        0.450697, 0.463721, 0.481400, 0.505568, 0.533482, 0.559949, 0.584126, 0.610335, 0.641770, 0.673968,\
        0.694920, 0.696794, 0.693324, 0.684864, 0.673639, 0.661415, 0.649772, 0.640557, 0.635928, 0.639859,\
        0.666216, 0.722352, 0.765106, 0.802046, 0.853696, 0.904693, 0.944440, 0.972169, 0.988769, 0.996535,\
        0.997661, 0.993915, 0.986717, 0.977259, 0.966543, 0.955338, 0.944099, 0.933129, 0.922930, 0.913630,\
        0.904559, 0.898086, 0.888650, 0.873295, 0.856652, 0.840384, 0.825004, 0.810994, 0.800061, 0.790806,\
        0.781370, 0.771594, 0.761378, 0.750971, 0.740583, 0.730371, 0.720635, 0.711872, 0.703973, 0.696401,\
        0.687888, 0.677470, 0.666505, 0.655539, 0.644710, 0.634070, 0.623689, 0.613737, 0.604151, 0.594393,\
        0.584561, 0.574433, 0.564225, 0.554143, 0.544268, 0.534601, 0.525098, 0.515727, 0.506753, 0.498599,\
        0.489974, 0.480678, 0.470876, 0.460818, 0.450651, 0.440475, 0.430363, 0.420373, 0.410537, 0.400931,\
        0.391676],\
        [0.255994, 0.262112, 0.267997, 0.273648, 0.279069, 0.284274, 0.289296, 0.294203, 0.299114, 0.304038,\
        0.309101, 0.314343, 0.319784, 0.325469, 0.331452, 0.337788, 0.344524, 0.351692, 0.359302, 0.367327,\
        0.375691, 0.384245, 0.392723, 0.400609, 0.407387, 0.413662, 0.419899, 0.426449, 0.433748, 0.442390,\
        0.453132, 0.466843, 0.484264, 0.504507, 0.525130, 0.545608, 0.565915, 0.588241, 0.616612, 0.647164,\
        0.669369, 0.678959, 0.680223, 0.675873, 0.667480, 0.656623, 0.644862, 0.633883, 0.625058, 0.617232,\
        0.615280, 0.688946, 0.749615, 0.792772, 0.843735, 0.894329, 0.935982, 0.963585, 0.979248, 0.986196,\
        0.986789, 0.982695, 0.975228, 0.965550, 0.954742, 0.943703, 0.932784, 0.921735, 0.911253, 0.902234,\
        0.892846, 0.883522, 0.872745, 0.858708, 0.843387, 0.828226, 0.813626, 0.800048, 0.788753, 0.779622,\
        0.770670, 0.761503, 0.752051, 0.742356, 0.732451, 0.722351, 0.712264, 0.703099, 0.695353, 0.687425,\
        0.678828, 0.669081, 0.658632, 0.647991, 0.637412, 0.626968, 0.616643, 0.606584, 0.597446, 0.588051,\
        0.578443, 0.568704, 0.558898, 0.549157, 0.539575, 0.530181, 0.520912, 0.511616, 0.502161, 0.493882,\
        0.485613, 0.476649, 0.467174, 0.457391, 0.447454, 0.437474, 0.427525, 0.417668, 0.407940, 0.398416,\
        0.389288],\
        [0.255472, 0.261576, 0.267457, 0.273117, 0.278565, 0.283818, 0.288916, 0.293922, 0.298923, 0.303986,\
        0.309185, 0.314552, 0.320114, 0.325911, 0.331990, 0.338395, 0.345164, 0.352314, 0.359842, 0.367711,\
        0.375842, 0.384108, 0.392327, 0.400178, 0.407279, 0.414164, 0.421186, 0.428665, 0.437003, 0.446702,\
        0.458336, 0.472398, 0.488814, 0.505643, 0.521271, 0.536964, 0.553440, 0.572108, 0.594894, 0.620607,\
        0.642218, 0.656153, 0.662793, 0.663273, 0.658583, 0.649835, 0.638312, 0.625509, 0.612961, 0.599274,\
        0.581197, 0.632975, 0.714821, 0.777286, 0.833754, 0.888107, 0.929668, 0.954474, 0.967945, 0.973767,\
        0.973894, 0.969589, 0.961944, 0.952049, 0.941067, 0.930282, 0.920312, 0.909947, 0.898120, 0.888792,\
        0.878591, 0.867468, 0.855071, 0.841337, 0.827603, 0.814509, 0.801461, 0.789110, 0.778166, 0.768694,\
        0.759855, 0.751121, 0.742317, 0.733365, 0.724141, 0.714433, 0.704315, 0.694653, 0.686351, 0.678107,\
        0.669339, 0.659867, 0.649859, 0.639642, 0.629479, 0.619463, 0.609502, 0.599505, 0.590213, 0.581219,\
        0.571995, 0.562621, 0.553185, 0.543792, 0.534551, 0.525549, 0.516791, 0.508105, 0.499406, 0.490438,\
        0.481661, 0.472668, 0.463354, 0.453797, 0.444099, 0.434351, 0.424622, 0.414982, 0.405563, 0.396345,\
        0.387234],\
        [0.255057, 0.261170, 0.267071, 0.272764, 0.278262, 0.283586, 0.288767, 0.293850, 0.298890, 0.304104,\
        0.309451, 0.314926, 0.320577, 0.326450, 0.332588, 0.339030, 0.345802, 0.352914, 0.360354, 0.368075,\
        0.375996, 0.383998, 0.391936, 0.399707, 0.407596, 0.415493, 0.423594, 0.432197, 0.441632, 0.452257,\
        0.464387, 0.478117, 0.492877, 0.506866, 0.519482, 0.531980, 0.545375, 0.560706, 0.578762, 0.598636,\
        0.617549, 0.633108, 0.643840, 0.648868, 0.648122, 0.642114, 0.631463, 0.617136, 0.600039, 0.580336,\
        0.569235, 0.601923, 0.683472, 0.754904, 0.814848, 0.875818, 0.919732, 0.941677, 0.953581, 0.959047,\
        0.959299, 0.955154, 0.947538, 0.937523, 0.926191, 0.915212, 0.906595, 0.897805, 0.886538, 0.875131,\
        0.863589, 0.851043, 0.837398, 0.823238, 0.810502, 0.800030, 0.788665, 0.777533, 0.767253, 0.757879,\
        0.749102, 0.740632, 0.732332, 0.724081, 0.715682, 0.706723, 0.696763, 0.686911, 0.677712, 0.668821,\
        0.659764, 0.650346, 0.640610, 0.630751, 0.621026, 0.611590, 0.602248, 0.592750, 0.583424, 0.574414,\
        0.565417, 0.556344, 0.547225, 0.538136, 0.529181, 0.520514, 0.512311, 0.504346, 0.495748, 0.486802,\
        0.477773, 0.468684, 0.459457, 0.450090, 0.440633, 0.431153, 0.421712, 0.412324, 0.403006, 0.394134,\
        0.385275],\
        [0.254739, 0.260879, 0.266820, 0.272568, 0.278136, 0.283546, 0.288834, 0.294046, 0.299257, 0.304516,\
        0.309901, 0.315430, 0.321133, 0.327048, 0.333214, 0.339665, 0.346424, 0.353498, 0.360870, 0.368497,\
        0.376302, 0.384196, 0.392130, 0.400223, 0.408729, 0.417556, 0.426725, 0.436415, 0.446827, 0.458121,\
        0.470352, 0.483335, 0.496394, 0.508295, 0.518940, 0.529278, 0.540187, 0.552390, 0.566216, 0.581184,\
        0.596355, 0.611415, 0.624600, 0.633207, 0.636322, 0.634163, 0.626317, 0.612240, 0.594149, 0.574659,\
        0.565550, 0.595232, 0.664988, 0.738870, 0.801712, 0.861879, 0.902462, 0.923803, 0.936306, 0.942747,\
        0.943874, 0.940242, 0.932817, 0.922947, 0.911723, 0.899795, 0.891562, 0.883121, 0.872811, 0.861315,\
        0.849024, 0.835623, 0.821241, 0.806968, 0.794332, 0.785145, 0.775557, 0.765723, 0.756199, 0.747135,\
        0.738453, 0.730097, 0.722114, 0.714396, 0.706559, 0.698367, 0.688670, 0.678638, 0.668950, 0.659545,\
        0.650203, 0.640770, 0.631209, 0.621589, 0.612148, 0.603246, 0.594451, 0.585447, 0.576468, 0.567615,\
        0.558811, 0.549987, 0.541141, 0.532303, 0.523528, 0.514952, 0.506870, 0.499606, 0.491366, 0.482501,\
        0.473534, 0.464514, 0.455427, 0.446268, 0.437068, 0.427880, 0.418795, 0.409922, 0.401130, 0.392289,\
        0.383424],\
        [0.254500, 0.260685, 0.266682, 0.272498, 0.278149, 0.283657, 0.289055, 0.294390, 0.299707, 0.305040,\
        0.310459, 0.316013, 0.321736, 0.327664, 0.333833, 0.340275, 0.347013, 0.354057, 0.361398, 0.369008,\
        0.376830, 0.384801, 0.392923, 0.401434, 0.410526, 0.420125, 0.430205, 0.440832, 0.452053, 0.463823,\
        0.475956, 0.488103, 0.499721, 0.510129, 0.519424, 0.528201, 0.537078, 0.546499, 0.556621, 0.567321,\
        0.578585, 0.590898, 0.603918, 0.614369, 0.621038, 0.624732, 0.623676, 0.615572, 0.602532, 0.584951,\
        0.574584, 0.602640, 0.684089, 0.758654, 0.818746, 0.859956, 0.888314, 0.906263, 0.919235, 0.926820,\
        0.929033, 0.925904, 0.918471, 0.908709, 0.898344, 0.887289, 0.877562, 0.868472, 0.858526, 0.847514,\
        0.835437, 0.822113, 0.807590, 0.793942, 0.782249, 0.772480, 0.763386, 0.754340, 0.745428, 0.736705,\
        0.728096, 0.719649, 0.711716, 0.704405, 0.696751, 0.688528, 0.679383, 0.669626, 0.659823, 0.650169,\
        0.640667, 0.631273, 0.621925, 0.612562, 0.603227, 0.594638, 0.586361, 0.577903, 0.569348, 0.560780,\
        0.552208, 0.543623, 0.535035, 0.526452, 0.517847, 0.509243, 0.500943, 0.493464, 0.485835, 0.477507,\
        0.468857, 0.460074, 0.451220, 0.442320, 0.433401, 0.424503, 0.415692, 0.407163, 0.399109, 0.390393,\
        0.381486],\
        [0.254321, 0.260563, 0.266628, 0.272523, 0.278262, 0.283866, 0.289365, 0.294795, 0.300190, 0.305589,\
        0.311046, 0.316615, 0.322340, 0.328259, 0.334410, 0.340827, 0.347537, 0.354558, 0.361899, 0.369556,\
        0.377514, 0.385736, 0.394157, 0.403001, 0.412633, 0.422835, 0.433635, 0.445028, 0.456932, 0.469124,\
        0.481189, 0.492638, 0.503148, 0.512538, 0.520870, 0.528519, 0.535826, 0.542955, 0.549857, 0.556396,\
        0.562712, 0.569523, 0.577645, 0.586602, 0.596775, 0.609137, 0.619929, 0.621468, 0.615295, 0.606499,\
        0.595614, 0.642744, 0.723648, 0.787186, 0.833400, 0.863254, 0.882590, 0.895583, 0.906099, 0.913135,\
        0.916121, 0.913072, 0.905013, 0.895142, 0.884830, 0.874321, 0.864161, 0.854426, 0.844551, 0.834146,\
        0.822874, 0.810549, 0.797038, 0.783641, 0.772005, 0.761864, 0.752622, 0.743802, 0.735257, 0.726890,\
        0.718423, 0.709697, 0.701480, 0.694268, 0.686797, 0.678611, 0.669696, 0.660204, 0.650461, 0.640728,\
        0.631182, 0.621925, 0.612909, 0.604011, 0.595126, 0.586655, 0.578564, 0.570464, 0.562266, 0.553990,\
        0.545657, 0.537295, 0.528954, 0.520687, 0.512419, 0.503952, 0.495695, 0.487950, 0.480241, 0.472200,\
        0.463876, 0.455396, 0.446841, 0.438248, 0.429648, 0.421071, 0.412514, 0.403884, 0.395784, 0.387762,\
        0.379238],\
        [0.254179, 0.260486, 0.266626, 0.272606, 0.278436, 0.284133, 0.289718, 0.295225, 0.300689, 0.306131,\
        0.311616, 0.317192, 0.322903, 0.328795, 0.334909, 0.341285, 0.347956, 0.354950, 0.362293, 0.370010,\
        0.378132, 0.386707, 0.395745, 0.404929, 0.414679, 0.425261, 0.436600, 0.448598, 0.461096, 0.473772,\
        0.486016, 0.497042, 0.506806, 0.515440, 0.523146, 0.530125, 0.536458, 0.542019, 0.546431, 0.549154,\
        0.549804, 0.548749, 0.547724, 0.550046, 0.559517, 0.579801, 0.607528, 0.621662, 0.626985, 0.631795,\
        0.642212, 0.687171, 0.755918, 0.808539, 0.843116, 0.865218, 0.879374, 0.889213, 0.896523, 0.902281,\
        0.905113, 0.901743, 0.892634, 0.882295, 0.871688, 0.861074, 0.850665, 0.840592, 0.830792, 0.821076,\
        0.810846, 0.799276, 0.786901, 0.774288, 0.762732, 0.752383, 0.742929, 0.734068, 0.725690, 0.717746,\
        0.709819, 0.701059, 0.692567, 0.684904, 0.677207, 0.668999, 0.660184, 0.650845, 0.641174, 0.631407,\
        0.621831, 0.612754, 0.604170, 0.595684, 0.587306, 0.579132, 0.571194, 0.563317, 0.555382, 0.547352,\
        0.539228, 0.531040, 0.522867, 0.514881, 0.507220, 0.499153, 0.490803, 0.482742, 0.474823, 0.466841,\
        0.458737, 0.450549, 0.442320, 0.434071, 0.425817, 0.417614, 0.409583, 0.401610, 0.393419, 0.385242,\
        0.376847],\
        [0.254047, 0.260426, 0.266645, 0.272713, 0.278635, 0.284422, 0.290085, 0.295643, 0.301144, 0.306638,\
        0.312144, 0.317709, 0.323389, 0.329235, 0.335295, 0.341613, 0.348228, 0.355176, 0.362495, 0.370222,\
        0.378403, 0.387095, 0.396320, 0.405874, 0.416001, 0.426969, 0.438725, 0.451159, 0.464098, 0.477248,\
        0.490056, 0.501143, 0.510211, 0.518395, 0.525933, 0.532840, 0.538989, 0.544025, 0.547268, 0.547623,\
        0.543765, 0.535081, 0.523494, 0.515880, 0.518777, 0.533966, 0.571104, 0.606435, 0.634629, 0.663416,\
        0.693376, 0.730842, 0.783364, 0.822932, 0.846730, 0.862949, 0.873955, 0.881680, 0.887475, 0.892133,\
        0.892527, 0.888305, 0.879306, 0.868839, 0.858173, 0.847569, 0.837028, 0.826737, 0.817002, 0.807751,\
        0.798817, 0.787933, 0.776350, 0.764728, 0.753674, 0.743435, 0.733911, 0.724929, 0.716467, 0.708837,\
        0.701519, 0.693378, 0.684629, 0.676387, 0.668286, 0.659925, 0.651130, 0.641891, 0.632302, 0.622521,\
        0.612835, 0.603820, 0.595682, 0.587671, 0.579723, 0.571890, 0.564172, 0.556496, 0.548772, 0.540945,\
        0.532993, 0.524929, 0.516807, 0.508738, 0.501411, 0.493910, 0.485618, 0.477428, 0.469393, 0.461429,\
        0.453487, 0.445568, 0.437680, 0.429809, 0.421914, 0.413995, 0.406239, 0.398991, 0.390967, 0.382722,\
        0.374362],\
        [0.253899, 0.260350, 0.266651, 0.272807, 0.278823, 0.284705, 0.290454, 0.296075, 0.301607, 0.307118,\
        0.312609, 0.318137, 0.323765, 0.329547, 0.335536, 0.341778, 0.348317, 0.355194, 0.362451, 0.370127,\
        0.378255, 0.386850, 0.395907, 0.405750, 0.416409, 0.427747, 0.439784, 0.452443, 0.465549, 0.478796,\
        0.491660, 0.502949, 0.512224, 0.520738, 0.528773, 0.536324, 0.543191, 0.548958, 0.552889, 0.553623,\
        0.548832, 0.536381, 0.517516, 0.500284, 0.499858, 0.499142, 0.535325, 0.587619, 0.646602, 0.696286,\
        0.730593, 0.761180, 0.793271, 0.821133, 0.841028, 0.855005, 0.864645, 0.871158, 0.875451, 0.877580,\
        0.876463, 0.871595, 0.863509, 0.853822, 0.843789, 0.833787, 0.823483, 0.812962, 0.803474, 0.794509,\
        0.785801, 0.776105, 0.765584, 0.754923, 0.744571, 0.734721, 0.725346, 0.716362, 0.707610, 0.699733,\
        0.692707, 0.684882, 0.676543, 0.668192, 0.659874, 0.651406, 0.642639, 0.633526, 0.624105, 0.614467,\
        0.604763, 0.595774, 0.587840, 0.580163, 0.572541, 0.564981, 0.557477, 0.549989, 0.542453, 0.534807,\
        0.527010, 0.519059, 0.511014, 0.502988, 0.495121, 0.487750, 0.479870, 0.471811, 0.463803, 0.455897,\
        0.448105, 0.440442, 0.432915, 0.425482, 0.418027, 0.410407, 0.402588, 0.395214, 0.387830, 0.379918,\
        0.371707],\
        [0.253709, 0.260230, 0.266609, 0.272849, 0.278956, 0.284934, 0.290784, 0.296502, 0.302078, 0.307541,\
        0.312975, 0.318441, 0.323998, 0.329703, 0.335606, 0.341757, 0.348201, 0.354983, 0.362148, 0.369737,\
        0.377791, 0.386348, 0.395494, 0.405420, 0.416162, 0.427622, 0.439741, 0.452410, 0.465423, 0.478429,\
        0.490830, 0.502122, 0.512553, 0.522093, 0.531232, 0.540084, 0.548485, 0.556091, 0.562443, 0.566749,\
        0.566278, 0.556452, 0.538860, 0.522751, 0.516898, 0.519736, 0.550344, 0.610317, 0.681859, 0.731011,\
        0.758654, 0.777774, 0.794191, 0.812470, 0.829868, 0.843087, 0.852192, 0.857793, 0.860552, 0.860799,\
        0.858432, 0.853387, 0.846062, 0.837382, 0.828418, 0.819826, 0.810640, 0.800062, 0.790688, 0.782097,\
        0.773460, 0.764353, 0.754733, 0.744965, 0.735389, 0.726127, 0.717088, 0.708303, 0.699961, 0.691621,\
        0.684123, 0.676426, 0.668362, 0.660123, 0.651800, 0.643351, 0.634691, 0.625770, 0.616618, 0.607356,\
        0.598077, 0.589154, 0.581040, 0.573359, 0.565857, 0.558449, 0.551102, 0.543773, 0.536407, 0.528935,\
        0.521291, 0.513442, 0.505447, 0.497483, 0.489583, 0.481799, 0.473940, 0.465980, 0.458042, 0.450216,\
        0.442560, 0.435129, 0.427965, 0.421043, 0.414194, 0.407120, 0.399744, 0.392263, 0.384749, 0.376961,\
        0.368885],\
        [0.253450, 0.260036, 0.266484, 0.272800, 0.278987, 0.285048, 0.290984, 0.296781, 0.302382, 0.307807,\
        0.313179, 0.318576, 0.324057, 0.329675, 0.335483, 0.341531, 0.347866, 0.354536, 0.361587, 0.369070,\
        0.377040, 0.385571, 0.394754, 0.404652, 0.415284, 0.426662, 0.438693, 0.451217, 0.464015, 0.476805,\
        0.489272, 0.501084, 0.511878, 0.522390, 0.532941, 0.543530, 0.554017, 0.564117, 0.573563, 0.582850,\
        0.591646, 0.589746, 0.579484, 0.568711, 0.565281, 0.574595, 0.605558, 0.679497, 0.732118, 0.763556,\
        0.780057, 0.789035, 0.795739, 0.805680, 0.818729, 0.830374, 0.838618, 0.843056, 0.844210, 0.842863,\
        0.839585, 0.834591, 0.827988, 0.820241, 0.812181, 0.805237, 0.798262, 0.788458, 0.779033, 0.770336,\
        0.761779, 0.753012, 0.743988, 0.734925, 0.726093, 0.717579, 0.709068, 0.700321, 0.692088, 0.683982,\
        0.676116, 0.668300, 0.660331, 0.652211, 0.643983, 0.635655, 0.627195, 0.618528, 0.609646, 0.600706,\
        0.591803, 0.583112, 0.574916, 0.567143, 0.559628, 0.552269, 0.545008, 0.537796, 0.530578, 0.523277,\
        0.515795, 0.508037, 0.500037, 0.491985, 0.483968, 0.475991, 0.468026, 0.460067, 0.452172, 0.444410,\
        0.436853, 0.429587, 0.422718, 0.416307, 0.410208, 0.403872, 0.396767, 0.389301, 0.381684, 0.373900,\
        0.365908],\
        [0.253098, 0.259740, 0.266248, 0.272624, 0.278871, 0.284990, 0.290975, 0.296809, 0.302423, 0.307831,\
        0.313166, 0.318506, 0.323913, 0.329444, 0.335152, 0.341090, 0.347307, 0.353852, 0.360773, 0.368125,\
        0.375971, 0.384392, 0.393490, 0.403255, 0.413702, 0.424945, 0.436810, 0.449107, 0.461628, 0.474154,\
        0.486483, 0.498339, 0.509732, 0.521478, 0.533573, 0.546014, 0.558791, 0.571732, 0.584306, 0.596806,\
        0.612827, 0.623318, 0.626752, 0.626146, 0.627986, 0.644211, 0.685851, 0.739613, 0.769093, 0.786027,\
        0.793710, 0.796067, 0.797129, 0.800963, 0.809513, 0.819023, 0.825810, 0.828514, 0.827697, 0.824708,\
        0.820703, 0.816033, 0.810227, 0.803420, 0.796344, 0.790100, 0.783958, 0.775926, 0.767322, 0.758889,\
        0.750533, 0.742047, 0.733411, 0.724822, 0.716552, 0.708817, 0.701139, 0.692677, 0.684335, 0.676248,\
        0.668311, 0.660431, 0.652503, 0.644481, 0.636370, 0.628202, 0.620020, 0.611716, 0.603043, 0.594362,\
        0.585730, 0.577263, 0.569110, 0.561289, 0.553729, 0.546359, 0.539124, 0.531981, 0.524871, 0.517724,\
        0.510424, 0.502759, 0.494708, 0.486522, 0.478341, 0.470208, 0.462136, 0.454143, 0.446266, 0.438550,\
        0.431047, 0.423842, 0.417096, 0.411012, 0.405573, 0.400089, 0.393265, 0.385890, 0.378336, 0.370629,\
        0.362745],\
        [0.252635, 0.259322, 0.265875, 0.272296, 0.278583, 0.284731, 0.290728, 0.296549, 0.302163, 0.307591,\
        0.312914, 0.318211, 0.323551, 0.328996, 0.334605, 0.340432, 0.346527, 0.352939, 0.359719, 0.366918,\
        0.374594, 0.382806, 0.391611, 0.401116, 0.411513, 0.422640, 0.434314, 0.446358, 0.458584, 0.470790,\
        0.482778, 0.494428, 0.506604, 0.519557, 0.532933, 0.546895, 0.561612, 0.577277, 0.593809, 0.610083,\
        0.629515, 0.649631, 0.667904, 0.680082, 0.681890, 0.695895, 0.725797, 0.761249, 0.783774, 0.794720,\
        0.798247, 0.797730, 0.796376, 0.797137, 0.802173, 0.809741, 0.814696, 0.815084, 0.811934, 0.807243,\
        0.802617, 0.798453, 0.793424, 0.787581, 0.781557, 0.775747, 0.769754, 0.762839, 0.755259, 0.747471,\
        0.739559, 0.731411, 0.723045, 0.714738, 0.706787, 0.699363, 0.692309, 0.684466, 0.676411, 0.668432,\
        0.660540, 0.652690, 0.644832, 0.636924, 0.628928, 0.620866, 0.612926, 0.605169, 0.596920, 0.588303,\
        0.579831, 0.571526, 0.563452, 0.555629, 0.548032, 0.540623, 0.533371, 0.526246, 0.519192, 0.512115,\
        0.504964, 0.497390, 0.489255, 0.480963, 0.472659, 0.464409, 0.456257, 0.448239, 0.440390, 0.432738,\
        0.425294, 0.418084, 0.411247, 0.405158, 0.399984, 0.394829, 0.388698, 0.381824, 0.374574, 0.367079,\
        0.359370],\
        [0.252043, 0.258764, 0.265350, 0.271801, 0.278111, 0.284273, 0.290268, 0.296073, 0.301683, 0.307123,\
        0.312433, 0.317689, 0.322964, 0.328327, 0.333838, 0.339556, 0.345531, 0.351812, 0.358449, 0.365490,\
        0.372986, 0.380990, 0.389564, 0.398839, 0.409051, 0.419996, 0.431445, 0.443244, 0.455224, 0.467221,\
        0.479146, 0.491180, 0.503766, 0.516951, 0.530986, 0.545748, 0.561368, 0.578160, 0.596280, 0.615491,\
        0.637580, 0.662137, 0.686562, 0.704637, 0.707540, 0.720249, 0.741537, 0.764996, 0.782176, 0.791166,\
        0.793996, 0.793400, 0.792074, 0.792391, 0.795874, 0.801495, 0.804546, 0.802432, 0.797429, 0.791743,\
        0.786752, 0.782338, 0.777718, 0.772636, 0.767325, 0.761908, 0.756186, 0.749896, 0.743109, 0.736070,\
        0.728824, 0.721151, 0.713006, 0.704874, 0.697272, 0.690116, 0.683143, 0.675829, 0.668210, 0.660483,\
        0.652734, 0.644985, 0.637250, 0.629517, 0.621699, 0.613663, 0.605629, 0.598216, 0.590428, 0.582202,\
        0.573954, 0.565821, 0.557854, 0.550069, 0.542452, 0.534991, 0.527686, 0.520539, 0.513509, 0.506412,\
        0.499099, 0.491589, 0.483540, 0.475247, 0.466893, 0.458582, 0.450388, 0.442373, 0.434592, 0.427070,\
        0.419788, 0.412688, 0.405720, 0.399506, 0.394408, 0.389380, 0.383705, 0.377343, 0.370488, 0.363286,\
        0.355807],\
        [0.251311, 0.258054, 0.264661, 0.271130, 0.277454, 0.283622, 0.289620, 0.295428, 0.301039, 0.306460,\
        0.311739, 0.316945, 0.322152, 0.327433, 0.332852, 0.338466, 0.344328, 0.350486, 0.356987, 0.363881,\
        0.371220, 0.379064, 0.387486, 0.396597, 0.406550, 0.417204, 0.428382, 0.439967, 0.451790, 0.463703,\
        0.475646, 0.487711, 0.500144, 0.513621, 0.527916, 0.542687, 0.558043, 0.574218, 0.591535, 0.611292,\
        0.634449, 0.659290, 0.682506, 0.699684, 0.709696, 0.722874, 0.740018, 0.757131, 0.770678, 0.779007,\
        0.782642, 0.783345, 0.783209, 0.784036, 0.786700, 0.790100, 0.791657, 0.788856, 0.783417, 0.777607,\
        0.772302, 0.767574, 0.763036, 0.758399, 0.753590, 0.748547, 0.743138, 0.737266, 0.731018, 0.724665,\
        0.718289, 0.711370, 0.703447, 0.695550, 0.688176, 0.681154, 0.674237, 0.667176, 0.659894, 0.652443,\
        0.644883, 0.637265, 0.629666, 0.622183, 0.614749, 0.606992, 0.598800, 0.591271, 0.583771, 0.575949,\
        0.568002, 0.560086, 0.552266, 0.544558, 0.536952, 0.529438, 0.522052, 0.514841, 0.507802, 0.500870,\
        0.493405, 0.485705, 0.477684, 0.469424, 0.461071, 0.452744, 0.444541, 0.436556, 0.428881, 0.421572,\
        0.414600, 0.407908, 0.401385, 0.395064, 0.389649, 0.384426, 0.378870, 0.372807, 0.366275, 0.359356,\
        0.352117],\
        [0.250429, 0.257183, 0.263800, 0.270276, 0.276606, 0.282779, 0.288780, 0.294593, 0.300204, 0.305593,\
        0.310831, 0.315978, 0.321113, 0.326314, 0.331646, 0.337167, 0.342927, 0.348974, 0.355354, 0.362117,\
        0.369318, 0.377025, 0.385331, 0.394342, 0.404046, 0.414268, 0.425183, 0.436630, 0.448391, 0.460315,\
        0.472330, 0.484447, 0.496871, 0.510337, 0.524240, 0.538317, 0.552677, 0.567547, 0.583419, 0.601647,\
        0.623508, 0.646733, 0.668126, 0.684896, 0.697887, 0.713024, 0.727897, 0.741232, 0.752912, 0.761405,\
        0.766265, 0.768560, 0.769830, 0.771303, 0.773386, 0.775331, 0.775618, 0.773135, 0.768537, 0.763231,\
        0.758086, 0.753378, 0.748986, 0.744692, 0.740305, 0.735634, 0.730538, 0.724979, 0.719009, 0.713095,\
        0.707584, 0.701917, 0.694326, 0.686622, 0.679320, 0.672334, 0.665481, 0.658592, 0.651566, 0.644373,\
        0.637014, 0.629519, 0.621985, 0.614678, 0.607833, 0.600645, 0.592748, 0.584926, 0.577326, 0.569685,\
        0.561984, 0.554291, 0.546653, 0.539077, 0.531535, 0.523990, 0.516490, 0.509269, 0.502060, 0.495092,\
        0.487696, 0.479893, 0.471833, 0.463589, 0.455259, 0.446947, 0.438756, 0.430803, 0.423232, 0.416185,\
        0.409621, 0.403272, 0.397251, 0.391164, 0.385445, 0.379962, 0.374355, 0.368418, 0.362094, 0.355402,\
        0.348375],\
        [0.249391, 0.256146, 0.262761, 0.269235, 0.275563, 0.281734, 0.287733, 0.293540, 0.299127, 0.304511,\
        0.309707, 0.314783, 0.319840, 0.324964, 0.330221, 0.335663, 0.341337, 0.347288, 0.353562, 0.360207,\
        0.367276, 0.374831, 0.382948, 0.391704, 0.401029, 0.411000, 0.421862, 0.433284, 0.445046, 0.457035,\
        0.469208, 0.481608, 0.494386, 0.507324, 0.520252, 0.533227, 0.546313, 0.559767, 0.574106, 0.590242,\
        0.609331, 0.630204, 0.650099, 0.667564, 0.683139, 0.698664, 0.710123, 0.720475, 0.731950, 0.741093,\
        0.746913, 0.750470, 0.752960, 0.755117, 0.757090, 0.758431, 0.758357, 0.756380, 0.752778, 0.748289,\
        0.743643, 0.739275, 0.735205, 0.731302, 0.727380, 0.723113, 0.718301, 0.713105, 0.707269, 0.701452,\
        0.696338, 0.691126, 0.684542, 0.677364, 0.670298, 0.663438, 0.656707, 0.649998, 0.643218, 0.636291,\
        0.629164, 0.621814, 0.614275, 0.606751, 0.600295, 0.593613, 0.586092, 0.578450, 0.570896, 0.563388,\
        0.555886, 0.548408, 0.540976, 0.533594, 0.526214, 0.518740, 0.511093, 0.503566, 0.497175, 0.489623,\
        0.481989, 0.474110, 0.466027, 0.457808, 0.449527, 0.441262, 0.433105, 0.425171, 0.417631, 0.410770,\
        0.404851, 0.398858, 0.392907, 0.387078, 0.381376, 0.375779, 0.370115, 0.364226, 0.358029, 0.351501,\
        0.344647],\
        [0.248195, 0.254938, 0.261540, 0.268001, 0.274315, 0.280476, 0.286472, 0.292282, 0.297879, 0.303239,\
        0.308358, 0.313338, 0.318315, 0.323377, 0.328578, 0.333959, 0.339564, 0.345436, 0.351620, 0.358162,\
        0.365109, 0.372505, 0.380385, 0.388744, 0.397589, 0.407640, 0.418595, 0.429983, 0.441712, 0.453724,\
        0.465986, 0.478488, 0.491161, 0.503669, 0.515817, 0.527703, 0.539525, 0.551653, 0.564569, 0.578900,\
        0.595267, 0.613187, 0.631579, 0.649430, 0.665802, 0.679950, 0.689902, 0.699154, 0.711410, 0.720459,\
        0.726335, 0.730579, 0.734040, 0.736902, 0.739112, 0.740472, 0.740595, 0.739289, 0.736698, 0.733054,\
        0.728960, 0.725148, 0.721529, 0.718034, 0.714672, 0.710929, 0.706247, 0.701344, 0.696235, 0.690362,\
        0.685219, 0.679959, 0.674013, 0.667524, 0.660888, 0.654290, 0.647760, 0.641275, 0.634772, 0.628154,\
        0.621315, 0.614208, 0.606826, 0.599347, 0.592412, 0.585916, 0.578890, 0.571602, 0.564272, 0.556955,\
        0.549655, 0.542383, 0.535166, 0.528021, 0.520923, 0.513746, 0.506262, 0.498602, 0.491401, 0.484037,\
        0.476290, 0.468347, 0.460272, 0.452115, 0.443928, 0.435767, 0.427695, 0.419810, 0.412253, 0.405238,\
        0.399597, 0.394419, 0.388655, 0.382964, 0.377347, 0.371752, 0.366078, 0.360221, 0.354106, 0.347695,\
        0.340975],\
        [0.246839, 0.253560, 0.260137, 0.266570, 0.272856, 0.278991, 0.284969, 0.290779, 0.296396, 0.301733,\
        0.306730, 0.311598, 0.316520, 0.321553, 0.326723, 0.332064, 0.337617, 0.343426, 0.349538, 0.355996,\
        0.362845, 0.370124, 0.377871, 0.386132, 0.395074, 0.404974, 0.415577, 0.426721, 0.438294, 0.450209,\
        0.462403, 0.474798, 0.487221, 0.499354, 0.510945, 0.521813, 0.532487, 0.543478, 0.555195, 0.568004,\
        0.582079, 0.597165, 0.614277, 0.631893, 0.647913, 0.661283, 0.671920, 0.682030, 0.692723, 0.700433,\
        0.705786, 0.710324, 0.714587, 0.718079, 0.720612, 0.722332, 0.722912, 0.722319, 0.720904, 0.718095,\
        0.714176, 0.711186, 0.708023, 0.704782, 0.701894, 0.698868, 0.694302, 0.689573, 0.684663, 0.679365,\
        0.674148, 0.668875, 0.663264, 0.657292, 0.651106, 0.644829, 0.638539, 0.632299, 0.626115, 0.619859,\
        0.613339, 0.606523, 0.599414, 0.592088, 0.585044, 0.578286, 0.571448, 0.564463, 0.557408, 0.550331,\
        0.543244, 0.536164, 0.529130, 0.522201, 0.515401, 0.508676, 0.501640, 0.493871, 0.486109, 0.478385,\
        0.470535, 0.462575, 0.454554, 0.446516, 0.438494, 0.430519, 0.422626, 0.414892, 0.407483, 0.400645,\
        0.394748, 0.389640, 0.384331, 0.378883, 0.373392, 0.367851, 0.362204, 0.356383, 0.350329, 0.344004,\
        0.337386],\
        [0.245327, 0.252015, 0.258554, 0.264945, 0.271185, 0.277271, 0.283199, 0.288960, 0.294534, 0.299842,\
        0.304699, 0.309507, 0.314456, 0.319510, 0.324676, 0.329991, 0.335503, 0.341262, 0.347316, 0.353710,\
        0.360490, 0.367702, 0.375402, 0.383682, 0.392686, 0.402354, 0.412525, 0.423351, 0.434667, 0.446355,\
        0.458324, 0.470469, 0.482629, 0.494557, 0.505536, 0.515384, 0.525124, 0.535274, 0.546128, 0.557862,\
        0.570572, 0.584448, 0.600030, 0.616606, 0.631730, 0.644643, 0.655578, 0.665392, 0.674314, 0.681359,\
        0.686683, 0.691391, 0.696032, 0.699802, 0.702560, 0.704787, 0.705905, 0.705775, 0.705772, 0.704303,\
        0.699672, 0.697615, 0.694871, 0.691799, 0.689053, 0.686018, 0.682096, 0.677657, 0.672950, 0.667995,\
        0.662919, 0.657743, 0.652389, 0.646819, 0.641045, 0.635088, 0.629007, 0.622963, 0.617109, 0.611276,\
        0.605071, 0.598387, 0.591492, 0.584397, 0.577410, 0.570603, 0.563861, 0.557104, 0.550320, 0.543506,\
        0.536643, 0.529729, 0.522810, 0.515992, 0.509335, 0.502789, 0.496032, 0.488396, 0.480496, 0.472596,\
        0.464682, 0.456755, 0.448846, 0.440991, 0.433216, 0.425531, 0.417937, 0.410458, 0.403286, 0.396736,\
        0.390860, 0.385506, 0.380264, 0.374956, 0.369570, 0.364091, 0.358484, 0.352701, 0.346696, 0.340434,\
        0.333893],\
        [0.243666, 0.250311, 0.256801, 0.263137, 0.269314, 0.275329, 0.281171, 0.286822, 0.292245, 0.297355,\
        0.302123, 0.307119, 0.312202, 0.317303, 0.322466, 0.327754, 0.333229, 0.338944, 0.344950, 0.351293,\
        0.358021, 0.365181, 0.372831, 0.381032, 0.389820, 0.399163, 0.409168, 0.419741, 0.430741, 0.442087,\
        0.453691, 0.465425, 0.477102, 0.488416, 0.498770, 0.508114, 0.517313, 0.527015, 0.537407, 0.548559,\
        0.560651, 0.573955, 0.588667, 0.603998, 0.617365, 0.629716, 0.640279, 0.649239, 0.657047, 0.663681,\
        0.669244, 0.674209, 0.678815, 0.682722, 0.685846, 0.688356, 0.689884, 0.690446, 0.691089, 0.690362,\
        0.686987, 0.684275, 0.681742, 0.678979, 0.676201, 0.673159, 0.669580, 0.665512, 0.661091, 0.656391,\
        0.651489, 0.646462, 0.641353, 0.636152, 0.630789, 0.625160, 0.619224, 0.613201, 0.607616, 0.602209,\
        0.596302, 0.589791, 0.583050, 0.576197, 0.569374, 0.562665, 0.556061, 0.549524, 0.543018, 0.536494,\
        0.529883, 0.523124, 0.516239, 0.509402, 0.502796, 0.496182, 0.489396, 0.482070, 0.474365, 0.466531,\
        0.458674, 0.450850, 0.443107, 0.435494, 0.428039, 0.420744, 0.413586, 0.406515, 0.399540, 0.393191,\
        0.387337, 0.381855, 0.376552, 0.371269, 0.365931, 0.360494, 0.354920, 0.349168, 0.343201, 0.336986,\
        0.330501],\
        [0.241866, 0.248460, 0.254893, 0.261164, 0.267269, 0.273202, 0.278948, 0.284483, 0.289766, 0.294751,\
        0.299641, 0.304782, 0.309908, 0.314994, 0.320119, 0.325364, 0.330795, 0.336466, 0.342428, 0.348725,\
        0.355402, 0.362506, 0.370081, 0.378165, 0.386769, 0.395904, 0.405657, 0.415896, 0.426485, 0.437399,\
        0.448544, 0.459752, 0.470792, 0.481358, 0.491144, 0.500191, 0.509124, 0.518780, 0.528995, 0.539757,\
        0.551367, 0.564069, 0.577744, 0.591258, 0.604080, 0.616231, 0.625980, 0.633683, 0.640663, 0.647126,\
        0.652885, 0.658052, 0.662738, 0.666836, 0.670255, 0.672959, 0.674826, 0.675935, 0.676452, 0.675753,\
        0.673620, 0.671125, 0.668630, 0.666036, 0.663311, 0.660339, 0.656987, 0.653233, 0.649116, 0.644656,\
        0.639899, 0.634990, 0.630102, 0.625274, 0.620366, 0.615141, 0.609400, 0.603197, 0.597674, 0.592505,\
        0.586825, 0.580650, 0.574176, 0.567567, 0.560948, 0.554408, 0.547987, 0.541688, 0.535482, 0.529298,\
        0.523008, 0.516456, 0.509590, 0.502579, 0.495983, 0.489394, 0.482567, 0.475386, 0.467890, 0.460225,\
        0.452513, 0.444846, 0.437306, 0.429966, 0.422873, 0.416022, 0.409368, 0.402879, 0.396312, 0.389944,\
        0.384041, 0.378479, 0.373113, 0.367810, 0.362481, 0.357064, 0.351510, 0.345781, 0.339840, 0.333658,\
        0.327212],\
        [0.239937, 0.246476, 0.252847, 0.259050, 0.265081, 0.270935, 0.276601, 0.282067, 0.287328, 0.292424,\
        0.297493, 0.302590, 0.307610, 0.312585, 0.317629, 0.322814, 0.328195, 0.333821, 0.339737, 0.345987,\
        0.352610, 0.359646, 0.367131, 0.375096, 0.383564, 0.392552, 0.402023, 0.411802, 0.421878, 0.432332,\
        0.443012, 0.453694, 0.464138, 0.474112, 0.483458, 0.492237, 0.501020, 0.510937, 0.520857, 0.531066,\
        0.542056, 0.554007, 0.566671, 0.579471, 0.592110, 0.603935, 0.612291, 0.618189, 0.624779, 0.631414,\
        0.637316, 0.642626, 0.647481, 0.651798, 0.655445, 0.658342, 0.660423, 0.661677, 0.662077, 0.661453,\
        0.659860, 0.657740, 0.655421, 0.652972, 0.650358, 0.647512, 0.644362, 0.640888, 0.637096, 0.632906,\
        0.628241, 0.623338, 0.618595, 0.614130, 0.609696, 0.604940, 0.599526, 0.593563, 0.587897, 0.582593,\
        0.577063, 0.571162, 0.564968, 0.558606, 0.552198, 0.545845, 0.539621, 0.533565, 0.527672, 0.521874,\
        0.516011, 0.509795, 0.503048, 0.496075, 0.489276, 0.482573, 0.475713, 0.468609, 0.461275, 0.453791,\
        0.446257, 0.438771, 0.431436, 0.424360, 0.417628, 0.411226, 0.405005, 0.398868, 0.392709, 0.386610,\
        0.380785, 0.375227, 0.369842, 0.364528, 0.359199, 0.353789, 0.348247, 0.342531, 0.336606, 0.330444,\
        0.324022],\
        [0.237895, 0.244375, 0.250682, 0.256816, 0.262776, 0.268559, 0.274164, 0.279594, 0.284871, 0.290048,\
        0.295199, 0.300304, 0.305159, 0.309996, 0.314961, 0.320090, 0.325422, 0.331001, 0.336870, 0.343067,\
        0.349627, 0.356583, 0.363962, 0.371788, 0.380080, 0.388855, 0.398062, 0.407239, 0.416872, 0.426977,\
        0.437272, 0.447500, 0.457460, 0.467011, 0.476120, 0.484974, 0.494130, 0.503668, 0.512734, 0.522108,\
        0.532372, 0.543587, 0.555458, 0.567600, 0.579641, 0.590682, 0.597563, 0.602219, 0.609599, 0.616502,\
        0.622295, 0.627664, 0.632757, 0.637339, 0.641180, 0.644238, 0.646441, 0.647702, 0.648005, 0.647395,\
        0.646020, 0.644158, 0.642058, 0.639794, 0.637333, 0.634651, 0.631703, 0.628473, 0.625062, 0.621303,\
        0.616687, 0.611588, 0.606884, 0.602717, 0.598627, 0.594155, 0.589056, 0.583541, 0.578046, 0.572684,\
        0.567231, 0.561531, 0.555575, 0.549435, 0.543218, 0.537037, 0.530992, 0.525152, 0.519545, 0.514124,\
        0.508729, 0.503038, 0.496415, 0.489503, 0.482622, 0.475790, 0.468889, 0.461845, 0.454649, 0.447345,\
        0.440000, 0.432696, 0.425539, 0.418672, 0.412244, 0.406297, 0.400537, 0.394674, 0.388852, 0.383088,\
        0.377464, 0.372004, 0.366662, 0.361368, 0.356051, 0.350651, 0.345116, 0.339407, 0.333489, 0.327336,\
        0.320926],\
        [0.235752, 0.242172, 0.248415, 0.254482, 0.260374, 0.266092, 0.271642, 0.277036, 0.282300, 0.287472,\
        0.292579, 0.297561, 0.302337, 0.307157, 0.312096, 0.317185, 0.322471, 0.328002, 0.333821, 0.339962,\
        0.346452, 0.353316, 0.360572, 0.368227, 0.376274, 0.384669, 0.393282, 0.402111, 0.411600, 0.421529,\
        0.431521, 0.441357, 0.450903, 0.460107, 0.469016, 0.477816, 0.486750, 0.495630, 0.504017, 0.512553,\
        0.522199, 0.532761, 0.543807, 0.554946, 0.565704, 0.575116, 0.582077, 0.588032, 0.595434, 0.601894,\
        0.607454, 0.612875, 0.618379, 0.623269, 0.627211, 0.630427, 0.632763, 0.633983, 0.634148, 0.633486,\
        0.632176, 0.630444, 0.628566, 0.626543, 0.624238, 0.621744, 0.619034, 0.615932, 0.612867, 0.609977,\
        0.605501, 0.599975, 0.595369, 0.591243, 0.587227, 0.582900, 0.578143, 0.573065, 0.567880, 0.562676,\
        0.557374, 0.551871, 0.546130, 0.540185, 0.534129, 0.528087, 0.522184, 0.516513, 0.511120, 0.505998,\
        0.500882, 0.495503, 0.489275, 0.482578, 0.475782, 0.468961, 0.462089, 0.455132, 0.448087, 0.440979,\
        0.433847, 0.426738, 0.419736, 0.412994, 0.406752, 0.401124, 0.395836, 0.390386, 0.384913, 0.379470,\
        0.374089, 0.368784, 0.363534, 0.358294, 0.353008, 0.347626, 0.342101, 0.336396, 0.330478, 0.324325,\
        0.317917],\
        [0.233521, 0.239883, 0.246063, 0.252065, 0.257890, 0.263544, 0.269035, 0.274377, 0.279590, 0.284693,\
        0.289687, 0.294543, 0.299295, 0.304142, 0.309070, 0.314113, 0.319347, 0.324830, 0.330597, 0.336676,\
        0.343092, 0.349860, 0.356988, 0.364472, 0.372282, 0.380361, 0.388637, 0.397211, 0.406566, 0.416277,\
        0.425939, 0.435369, 0.444500, 0.453331, 0.461922, 0.470371, 0.478728, 0.486852, 0.494584, 0.502552,\
        0.511812, 0.521750, 0.531850, 0.541774, 0.551222, 0.559710, 0.566978, 0.573711, 0.580570, 0.586880,\
        0.592624, 0.598141, 0.604353, 0.609361, 0.613199, 0.616687, 0.619293, 0.620427, 0.620366, 0.619651,\
        0.618375, 0.616622, 0.614980, 0.613328, 0.611077, 0.608737, 0.606445, 0.603364, 0.600264, 0.598299,\
        0.594376, 0.589008, 0.584211, 0.579925, 0.575801, 0.571549, 0.567048, 0.562329, 0.557487, 0.552563,\
        0.547513, 0.542257, 0.536744, 0.530984, 0.525066, 0.519132, 0.513344, 0.507826, 0.502565, 0.497651,\
        0.492734, 0.487456, 0.481614, 0.475297, 0.468737, 0.462060, 0.455304, 0.448487, 0.441630, 0.434761,\
        0.427899, 0.421047, 0.414234, 0.407588, 0.401428, 0.396111, 0.391119, 0.386092, 0.380986, 0.375848,\
        0.370709, 0.365581, 0.360451, 0.355289, 0.350052, 0.344699, 0.339188, 0.333486, 0.327564, 0.321403,\
        0.314986],\
        [0.231216, 0.237521, 0.243641, 0.249578, 0.255338, 0.260927, 0.266353, 0.271629, 0.276770, 0.281788,\
        0.286691, 0.291491, 0.296253, 0.301070, 0.305921, 0.310885, 0.316062, 0.321494, 0.327207, 0.333223,\
        0.339561, 0.346233, 0.353244, 0.360587, 0.368238, 0.376166, 0.384368, 0.392952, 0.402022, 0.411355,\
        0.420596, 0.429555, 0.438221, 0.446625, 0.454804, 0.462795, 0.470593, 0.478146, 0.485519, 0.493214,\
        0.501802, 0.510955, 0.519955, 0.528545, 0.536855, 0.544739, 0.552039, 0.558925, 0.565568, 0.571863,\
        0.577859, 0.583866, 0.590030, 0.594928, 0.598926, 0.602816, 0.605898, 0.606814, 0.606466, 0.605821,\
        0.604756, 0.602802, 0.601348, 0.600244, 0.597922, 0.595661, 0.593617, 0.590872, 0.588046, 0.585564,\
        0.582082, 0.577527, 0.572924, 0.568587, 0.564405, 0.560208, 0.555902, 0.551482, 0.546982, 0.542405,\
        0.537693, 0.532755, 0.527516, 0.521955, 0.516170, 0.510351, 0.504651, 0.499348, 0.494294, 0.489454,\
        0.484599, 0.479447, 0.473864, 0.467874, 0.461595, 0.455134, 0.448561, 0.441929, 0.435296, 0.428717,\
        0.422217, 0.415744, 0.409246, 0.402849, 0.396909, 0.391700, 0.386841, 0.382037, 0.377192, 0.372301,\
        0.367375, 0.362417, 0.357415, 0.352345, 0.347171, 0.341857, 0.336366, 0.330667, 0.324737, 0.318560,\
        0.312125],\
        [0.228851, 0.235101, 0.241162, 0.247038, 0.252733, 0.258255, 0.263613, 0.268817, 0.273881, 0.278818,\
        0.283646, 0.288394, 0.293119, 0.297869, 0.302606, 0.307505, 0.312634, 0.318017, 0.323672, 0.329619,\
        0.335875, 0.342452, 0.349358, 0.356593, 0.364147, 0.372015, 0.380213, 0.388771, 0.397641, 0.406665,\
        0.415413, 0.423823, 0.431996, 0.439957, 0.447712, 0.455269, 0.462629, 0.469816, 0.476964, 0.484381,\
        0.492309, 0.500577, 0.508357, 0.515472, 0.522840, 0.530274, 0.537429, 0.544235, 0.550731, 0.556974,\
        0.563083, 0.569169, 0.575044, 0.580163, 0.584588, 0.588613, 0.591709, 0.592734, 0.592529, 0.592016,\
        0.591097, 0.589450, 0.588015, 0.586706, 0.584792, 0.582681, 0.580556, 0.578143, 0.575544, 0.572799,\
        0.569483, 0.565519, 0.561302, 0.557117, 0.553003, 0.548904, 0.544773, 0.540614, 0.536449, 0.532262,\
        0.527959, 0.523410, 0.518521, 0.513214, 0.507533, 0.501949, 0.496408, 0.491252, 0.486343, 0.481542,\
        0.476698, 0.471632, 0.466238, 0.460507, 0.454493, 0.448273, 0.441915, 0.435489, 0.429086, 0.422819,\
        0.416773, 0.410853, 0.404814, 0.398765, 0.393085, 0.387878, 0.383011, 0.378287, 0.373591, 0.368874,\
        0.364118, 0.359310, 0.354432, 0.349458, 0.344356, 0.339090, 0.333626, 0.327933, 0.321991, 0.315790,\
        0.309328],\
        [0.226435, 0.232634, 0.238640, 0.244458, 0.250091, 0.255548, 0.260837, 0.265968, 0.270955, 0.275813,\
        0.280566, 0.285242, 0.289880, 0.294506, 0.299173, 0.304027, 0.309107, 0.314432, 0.320019, 0.325885,\
        0.332047, 0.338523, 0.345325, 0.352461, 0.359933, 0.367746, 0.375909, 0.384420, 0.393215, 0.402015,\
        0.410106, 0.417980, 0.425725, 0.433293, 0.440670, 0.447862, 0.454885, 0.461785, 0.468671, 0.475701,\
        0.482953, 0.490229, 0.496762, 0.502721, 0.509504, 0.516584, 0.523458, 0.530027, 0.536264, 0.542298,\
        0.548313, 0.554320, 0.560097, 0.565350, 0.569976, 0.573915, 0.576797, 0.578236, 0.578551, 0.578262,\
        0.577475, 0.576221, 0.574826, 0.573336, 0.571576, 0.569624, 0.567561, 0.565320, 0.562863, 0.560126,\
        0.556967, 0.553369, 0.549509, 0.545570, 0.541628, 0.537678, 0.533719, 0.529790, 0.525943, 0.522174,\
        0.518338, 0.514220, 0.509727, 0.504865, 0.499269, 0.493860, 0.488614, 0.483553, 0.478687, 0.473903,\
        0.469074, 0.464079, 0.458834, 0.453316, 0.447545, 0.441568, 0.435437, 0.429220, 0.423014, 0.416990,\
        0.411403, 0.406245, 0.400820, 0.395057, 0.389519, 0.384337, 0.379461, 0.374769, 0.370159, 0.365562,\
        0.360939, 0.356260, 0.351497, 0.346620, 0.341595, 0.336387, 0.330959, 0.325279, 0.319323, 0.313089,\
        0.306590],\
        [0.223983, 0.230135, 0.236090, 0.241853, 0.247428, 0.252823, 0.258044, 0.263105, 0.268017, 0.272799,\
        0.277474, 0.282070, 0.286620, 0.291164, 0.295770, 0.300536, 0.305533, 0.310779, 0.316279, 0.322042,\
        0.328089, 0.334445, 0.341130, 0.348153, 0.355518, 0.363227, 0.371271, 0.379614, 0.388136, 0.396457,\
        0.404224, 0.411860, 0.419341, 0.426602, 0.433672, 0.440574, 0.447328, 0.453967, 0.460550, 0.467133,\
        0.473685, 0.479999, 0.485780, 0.491336, 0.497520, 0.503994, 0.510400, 0.516599, 0.522376, 0.527971,\
        0.533771, 0.539686, 0.545421, 0.550702, 0.555342, 0.559182, 0.562023, 0.563757, 0.564534, 0.564597,\
        0.564074, 0.563063, 0.561727, 0.560176, 0.558449, 0.556601, 0.554659, 0.552573, 0.550260, 0.547639,\
        0.544652, 0.541320, 0.537751, 0.534085, 0.530382, 0.526626, 0.522834, 0.519094, 0.515516, 0.512166,\
        0.508849, 0.505148, 0.500951, 0.496456, 0.491382, 0.486120, 0.481028, 0.476087, 0.471273, 0.466515,\
        0.461720, 0.456798, 0.451687, 0.446359, 0.440818, 0.435089, 0.429204, 0.423207, 0.417172, 0.411244,\
        0.405799, 0.401546, 0.396672, 0.391246, 0.385909, 0.380831, 0.376006, 0.371364, 0.366826, 0.362326,\
        0.357814, 0.353248, 0.348593, 0.343812, 0.338871, 0.333731, 0.328355, 0.322703, 0.316741, 0.310463,\
        0.303914],\
        [0.221505, 0.227614, 0.233524, 0.239238, 0.244760, 0.250096, 0.255255, 0.260247, 0.265088, 0.269797,\
        0.274397, 0.278916, 0.283390, 0.287863, 0.292399, 0.297071, 0.301955, 0.307099, 0.312484, 0.318109,\
        0.324010, 0.330220, 0.336763, 0.343643, 0.350860, 0.358403, 0.366240, 0.374299, 0.382421, 0.390350,\
        0.397987, 0.405576, 0.412858, 0.419864, 0.426697, 0.433383, 0.439926, 0.446338, 0.452631, 0.458804,\
        0.464802, 0.470518, 0.475913, 0.481232, 0.486812, 0.492587, 0.498416, 0.504171, 0.509232, 0.514138,\
        0.519751, 0.525590, 0.531263, 0.536492, 0.541061, 0.544801, 0.547629, 0.549574, 0.550730, 0.551182,\
        0.550968, 0.550145, 0.548843, 0.547241, 0.545504, 0.543735, 0.541946, 0.540052, 0.537923, 0.535462,\
        0.532650, 0.529535, 0.526200, 0.522802, 0.519412, 0.515889, 0.512260, 0.508676, 0.505296, 0.502267,\
        0.499461, 0.496096, 0.492208, 0.487943, 0.483301, 0.478431, 0.473561, 0.468766, 0.464040, 0.459342,\
        0.454608, 0.449775, 0.444798, 0.439652, 0.434339, 0.428871, 0.423267, 0.417541, 0.411732, 0.405971,\
        0.400416, 0.396139, 0.391917, 0.387059, 0.382083, 0.377216, 0.372517, 0.367967, 0.363517, 0.359113,\
        0.354704, 0.350245, 0.345694, 0.341011, 0.336157, 0.331096, 0.325792, 0.320201, 0.314264, 0.307940,\
        0.301318],\
        [0.219011, 0.225085, 0.230956, 0.236626, 0.242101, 0.247384, 0.252484, 0.257413, 0.262187, 0.266825,\
        0.271352, 0.275799, 0.280202, 0.284606, 0.289068, 0.293650, 0.298418, 0.303430, 0.308658, 0.314101,\
        0.319816, 0.325853, 0.332227, 0.338933, 0.345961, 0.353289, 0.360873, 0.368631, 0.376434, 0.384148,\
        0.391740, 0.399208, 0.406231, 0.413042, 0.419727, 0.426277, 0.432675, 0.438911, 0.444973, 0.450835,\
        0.456454, 0.461798, 0.466906, 0.471931, 0.477033, 0.482234, 0.487466, 0.492564, 0.496840, 0.501171,\
        0.506721, 0.512361, 0.517883, 0.522977, 0.527423, 0.531012, 0.533752, 0.535832, 0.537335, 0.538201,\
        0.538321, 0.537654, 0.536324, 0.534613, 0.532823, 0.531128, 0.529535, 0.527901, 0.526010, 0.523718,\
        0.521068, 0.518168, 0.515005, 0.511806, 0.508876, 0.505610, 0.502158, 0.498781, 0.495635, 0.492817,\
        0.490127, 0.487053, 0.483503, 0.479552, 0.475269, 0.470761, 0.466161, 0.461552, 0.456955, 0.452353,\
        0.447710, 0.442986, 0.438149, 0.433185, 0.428097, 0.422902, 0.417617, 0.412238, 0.406733, 0.401272,\
        0.396128, 0.391519, 0.387215, 0.382719, 0.378084, 0.373464, 0.368935, 0.364511, 0.360169, 0.355869,\
        0.351566, 0.347215, 0.342768, 0.338182, 0.333416, 0.328437, 0.323218, 0.317731, 0.311912, 0.305615,\
        0.298882],\
        [0.216511, 0.222556, 0.228396, 0.234031, 0.239464, 0.244699, 0.249745, 0.254615, 0.259324, 0.263895,\
        0.268352, 0.272728, 0.277060, 0.281391, 0.285773, 0.290264, 0.294920, 0.299776, 0.304809, 0.310013,\
        0.315512, 0.321361, 0.327548, 0.334052, 0.340857, 0.347941, 0.355258, 0.362736, 0.370277, 0.377794,\
        0.385221, 0.392457, 0.399385, 0.406141, 0.412774, 0.419265, 0.425583, 0.431708, 0.437616, 0.443271,\
        0.448639, 0.453721, 0.458573, 0.463297, 0.467988, 0.472676, 0.477304, 0.481708, 0.485762, 0.490050,\
        0.495003, 0.500235, 0.505464, 0.510309, 0.514631, 0.517977, 0.520457, 0.522618, 0.524487, 0.525821,\
        0.526319, 0.525789, 0.524330, 0.522398, 0.520511, 0.518893, 0.517546, 0.516255, 0.514664, 0.512518,\
        0.509964, 0.507352, 0.504434, 0.501149, 0.498785, 0.495848, 0.492627, 0.489522, 0.486601, 0.483884,\
        0.481200, 0.478281, 0.474994, 0.471342, 0.467378, 0.463184, 0.458849, 0.454444, 0.450003, 0.445529,\
        0.441005, 0.436407, 0.431716, 0.426929, 0.422056, 0.417119, 0.412158, 0.407196, 0.402086, 0.396823,\
        0.391860, 0.387226, 0.382822, 0.378440, 0.374020, 0.369605, 0.365245, 0.360958, 0.356735, 0.352547,\
        0.348356, 0.344116, 0.339778, 0.335290, 0.330608, 0.325697, 0.320544, 0.315161, 0.309556, 0.303582,\
        0.296871],\
        [0.214017, 0.220041, 0.225857, 0.231464, 0.236863, 0.242055, 0.247051, 0.251863, 0.256510, 0.261014,\
        0.265403, 0.269707, 0.273963, 0.278214, 0.282504, 0.286884, 0.291400, 0.296081, 0.300898, 0.305816,\
        0.311110, 0.316784, 0.322773, 0.329047, 0.335601, 0.342421, 0.349471, 0.356687, 0.363989, 0.371299,\
        0.378539, 0.385627, 0.392517, 0.399252, 0.405887, 0.412376, 0.418661, 0.424736, 0.430572, 0.436105,\
        0.441302, 0.446178, 0.450795, 0.455232, 0.459562, 0.463811, 0.467956, 0.471952, 0.475862, 0.479970,\
        0.484462, 0.489228, 0.493991, 0.498522, 0.502771, 0.505780, 0.507740, 0.510046, 0.512303, 0.514158,\
        0.515094, 0.514721, 0.513018, 0.510728, 0.508760, 0.507218, 0.506125, 0.505204, 0.503964, 0.501947,\
        0.499415, 0.497061, 0.494485, 0.491559, 0.489047, 0.486416, 0.483555, 0.480710, 0.477975, 0.475341,\
        0.472697, 0.469883, 0.466788, 0.463381, 0.459687, 0.455760, 0.451664, 0.447460, 0.443186, 0.438859,\
        0.434476, 0.430019, 0.425477, 0.420859, 0.416183, 0.411460, 0.406739, 0.402102, 0.397422, 0.392428,\
        0.387575, 0.382969, 0.378551, 0.374223, 0.369927, 0.365661, 0.361443, 0.357285, 0.353180, 0.349108,\
        0.345034, 0.340911, 0.336688, 0.332304, 0.327702, 0.322836, 0.317690, 0.312290, 0.306730, 0.301102,\
        0.294402],\
        [0.211537, 0.217549, 0.223352, 0.228939, 0.234309, 0.239462, 0.244409, 0.249166, 0.253751, 0.258189,\
        0.262507, 0.266737, 0.270913, 0.275074, 0.279259, 0.283507, 0.287851, 0.292301, 0.296834, 0.301535,\
        0.306698, 0.312211, 0.317974, 0.323977, 0.330245, 0.336788, 0.343575, 0.350544, 0.357621, 0.364737,\
        0.371823, 0.378820, 0.385696, 0.392458, 0.399117, 0.405634, 0.411893, 0.417985, 0.423836, 0.429310,\
        0.434379, 0.439076, 0.443473, 0.447648, 0.451666, 0.455562, 0.459350, 0.463052, 0.466758, 0.470618,\
        0.474727, 0.479031, 0.483368, 0.487580, 0.491428, 0.494140, 0.496023, 0.498434, 0.500904, 0.503200,\
        0.504533, 0.504424, 0.502500, 0.499860, 0.497962, 0.496520, 0.495546, 0.494762, 0.493656, 0.491873,\
        0.489635, 0.487354, 0.484958, 0.482419, 0.479937, 0.477438, 0.474846, 0.472236, 0.469668, 0.467136,\
        0.464564, 0.461851, 0.458913, 0.455707, 0.452235, 0.448531, 0.444642, 0.440620, 0.436510, 0.432339,\
        0.428115, 0.423816, 0.419419, 0.414949, 0.410469, 0.405929, 0.401360, 0.396890, 0.392418, 0.387787,\
        0.383154, 0.378646, 0.374274, 0.370000, 0.365788, 0.361627, 0.357520, 0.353472, 0.349476, 0.345516,\
        0.341558, 0.337559, 0.333461, 0.329193, 0.324680, 0.319852, 0.314668, 0.309125, 0.303286, 0.297358,\
        0.290921],\
        [0.209084, 0.215094, 0.220893, 0.226469, 0.231813, 0.236929, 0.241827, 0.246526, 0.251048, 0.255419,\
        0.259665, 0.263819, 0.267912, 0.271978, 0.276051, 0.280163, 0.284337, 0.288587, 0.292932, 0.297493,\
        0.302472, 0.307752, 0.313226, 0.318887, 0.324830, 0.331093, 0.337628, 0.344362, 0.351227, 0.358165,\
        0.365122, 0.372053, 0.378926, 0.385727, 0.392437, 0.398983, 0.405201, 0.411460, 0.417408, 0.422842,\
        0.427793, 0.432318, 0.436505, 0.440447, 0.444206, 0.447821, 0.451326, 0.454769, 0.458231, 0.461809,\
        0.465556, 0.469443, 0.473363, 0.477152, 0.480548, 0.483257, 0.485539, 0.487924, 0.490400, 0.492650,\
        0.494070, 0.494123, 0.492529, 0.490249, 0.488364, 0.486958, 0.485911, 0.484989, 0.483841, 0.482256,\
        0.480307, 0.478186, 0.475951, 0.473628, 0.471277, 0.468903, 0.466492, 0.464066, 0.461649, 0.459230,\
        0.456757, 0.454158, 0.451363, 0.448329, 0.445044, 0.441525, 0.437809, 0.433943, 0.429977, 0.425959,\
        0.421909, 0.417799, 0.413553, 0.409167, 0.404895, 0.400586, 0.396155, 0.391799, 0.387446, 0.383031,\
        0.378599, 0.374221, 0.369928, 0.365716, 0.361570, 0.357485, 0.353461, 0.349499, 0.345595, 0.341735,\
        0.337890, 0.334016, 0.330054, 0.325926, 0.321532, 0.316766, 0.311560, 0.305944, 0.299908, 0.293719,\
        0.287140],\
        [0.206671, 0.212691, 0.218500, 0.224069, 0.229387, 0.234460, 0.239305, 0.243944, 0.248401, 0.252703,\
        0.256877, 0.260953, 0.264961, 0.268933, 0.272897, 0.276883, 0.280917, 0.285026, 0.289259, 0.293712,\
        0.298453, 0.303459, 0.308572, 0.313782, 0.319379, 0.325386, 0.331691, 0.338199, 0.344853, 0.351616,\
        0.358450, 0.365319, 0.372188, 0.379022, 0.385770, 0.392332, 0.398719, 0.405320, 0.411256, 0.416607,\
        0.421441, 0.425787, 0.429774, 0.433517, 0.437071, 0.440464, 0.443730, 0.446924, 0.450123, 0.453402,\
        0.456804, 0.460312, 0.463844, 0.467265, 0.470401, 0.473149, 0.475631, 0.478045, 0.480387, 0.482397,\
        0.483657, 0.483792, 0.482761, 0.481104, 0.479465, 0.478093, 0.476955, 0.475887, 0.474676, 0.473185,\
        0.471420, 0.469463, 0.467378, 0.465207, 0.462986, 0.460735, 0.458461, 0.456178, 0.453893, 0.451593,\
        0.449239, 0.446768, 0.444120, 0.441244, 0.438121, 0.434758, 0.431184, 0.427442, 0.423589, 0.419698,\
        0.415827, 0.411959, 0.407952, 0.403575, 0.399287, 0.395299, 0.391040, 0.386778, 0.382521, 0.378244,\
        0.373958, 0.369699, 0.365490, 0.361340, 0.357249, 0.353217, 0.349247, 0.345344, 0.341508, 0.337730,\
        0.333985, 0.330232, 0.326412, 0.322448, 0.318227, 0.313588, 0.308374, 0.302646, 0.296641, 0.290169,\
        0.283305],\
        [0.204318, 0.210365, 0.216195, 0.221755, 0.227036, 0.232057, 0.236841, 0.241415, 0.245805, 0.250037,\
        0.254139, 0.258138, 0.262064, 0.265945, 0.269808, 0.273682, 0.277596, 0.281587, 0.285706, 0.290016,\
        0.294564, 0.299367, 0.303936, 0.308590, 0.313928, 0.319754, 0.325846, 0.332112, 0.338535, 0.345108,\
        0.351813, 0.358613, 0.365468, 0.372335, 0.379166, 0.385931, 0.392706, 0.399371, 0.405061, 0.410450,\
        0.415205, 0.419335, 0.423135, 0.426740, 0.430152, 0.433373, 0.436431, 0.439383, 0.442309, 0.445286,\
        0.448362, 0.451535, 0.454744, 0.457890, 0.460862, 0.463605, 0.466149, 0.468544, 0.470738, 0.472540,\
        0.473677, 0.473943, 0.473357, 0.472220, 0.470911, 0.469655, 0.468491, 0.467341, 0.466084, 0.464635,\
        0.462977, 0.461145, 0.459184, 0.457133, 0.455023, 0.452877, 0.450712, 0.448542, 0.446374, 0.444196,\
        0.441974, 0.439650, 0.437159, 0.434443, 0.431467, 0.428237, 0.424784, 0.421141, 0.417356, 0.413534,\
        0.409801, 0.406185, 0.402557, 0.398443, 0.394093, 0.390020, 0.385915, 0.381766, 0.377603, 0.373430,\
        0.369254, 0.365091, 0.360957, 0.356862, 0.352811, 0.348810, 0.344867, 0.340992, 0.337195, 0.333473,\
        0.329806, 0.326154, 0.322461, 0.318663, 0.314668, 0.310285, 0.305180, 0.299265, 0.293046, 0.286399,\
        0.279290],\
        [0.202060, 0.208161, 0.214013, 0.219535, 0.224754, 0.229707, 0.234422, 0.238928, 0.243249, 0.247412,\
        0.251444, 0.255370, 0.259218, 0.263014, 0.266784, 0.270557, 0.274360, 0.278230, 0.282206, 0.286326,\
        0.290604, 0.294940, 0.298989, 0.303333, 0.308659, 0.314352, 0.320189, 0.326150, 0.332293, 0.338650,\
        0.345209, 0.351926, 0.358752, 0.365635, 0.372535, 0.379420, 0.386235, 0.392712, 0.398667, 0.404318,\
        0.408941, 0.412745, 0.416432, 0.420010, 0.423355, 0.426453, 0.429329, 0.432042, 0.434689, 0.437368,\
        0.440145, 0.443032, 0.445988, 0.448934, 0.451786, 0.454491, 0.457028, 0.459375, 0.461456, 0.463128,\
        0.464221, 0.464625, 0.464367, 0.463621, 0.462617, 0.461522, 0.460399, 0.459230, 0.457958, 0.456534,\
        0.454942, 0.453197, 0.451331, 0.449373, 0.447353, 0.445291, 0.443209, 0.441128, 0.439059, 0.437000,\
        0.434923, 0.432768, 0.430456, 0.427909, 0.425074, 0.421957, 0.418616, 0.415079, 0.411325, 0.407490,\
        0.403806, 0.400353, 0.396876, 0.393062, 0.388940, 0.384856, 0.380803, 0.376739, 0.372663, 0.368577,\
        0.364487, 0.360402, 0.356331, 0.352280, 0.348254, 0.344262, 0.340316, 0.336435, 0.332640, 0.328941,\
        0.325323, 0.321742, 0.318133, 0.314440, 0.310636, 0.306634, 0.301856, 0.295870, 0.289277, 0.282316,\
        0.274965],\
        [0.199995, 0.206187, 0.211975, 0.217389, 0.222513, 0.227385, 0.232028, 0.236465, 0.240720, 0.244818,\
        0.248784, 0.252643, 0.256419, 0.260138, 0.263824, 0.267501, 0.271197, 0.274938, 0.278753, 0.282653,\
        0.286617, 0.290556, 0.294439, 0.298782, 0.303925, 0.309348, 0.314788, 0.320324, 0.326124, 0.332241,\
        0.338643, 0.345261, 0.352027, 0.358884, 0.365779, 0.372659, 0.379430, 0.385947, 0.392148, 0.397914,\
        0.402270, 0.405714, 0.409598, 0.413283, 0.416601, 0.419622, 0.422340, 0.424811, 0.427173, 0.429564,\
        0.432073, 0.434730, 0.437500, 0.440307, 0.443069, 0.445728, 0.448235, 0.450536, 0.452547, 0.454158,\
        0.455260, 0.455794, 0.455781, 0.455328, 0.454576, 0.453648, 0.452607, 0.451466, 0.450207, 0.448808,\
        0.447262, 0.445578, 0.443781, 0.441896, 0.439945, 0.437946, 0.435923, 0.433902, 0.431911, 0.429963,\
        0.428039, 0.426077, 0.423977, 0.421627, 0.418934, 0.415889, 0.412645, 0.409307, 0.405598, 0.401690,\
        0.398016, 0.394603, 0.391158, 0.387489, 0.383603, 0.379640, 0.375663, 0.371677, 0.367680, 0.363671,\
        0.359655, 0.355635, 0.351616, 0.347599, 0.343587, 0.339584, 0.335605, 0.331679, 0.327842, 0.324125,\
        0.320526, 0.316986, 0.313407, 0.309706, 0.305873, 0.302004, 0.297631, 0.291597, 0.284877, 0.277731,\
        0.270206],\
        [0.198453, 0.204458, 0.209963, 0.215222, 0.220254, 0.225054, 0.229632, 0.234007, 0.238201, 0.242240,\
        0.246147, 0.249947, 0.253661, 0.257312, 0.260922, 0.264512, 0.268106, 0.271723, 0.275381, 0.279088,\
        0.282834, 0.286617, 0.290539, 0.294872, 0.299718, 0.304744, 0.309587, 0.314571, 0.320001, 0.325889,\
        0.332138, 0.338639, 0.345308, 0.352078, 0.358892, 0.365685, 0.372369, 0.378834, 0.384956, 0.390449,\
        0.394772, 0.398518, 0.402842, 0.406502, 0.409783, 0.412808, 0.415395, 0.417604, 0.419678, 0.421800,\
        0.424081, 0.426570, 0.429225, 0.431947, 0.434646, 0.437263, 0.439745, 0.442020, 0.444000, 0.445598,\
        0.446739, 0.447388, 0.447564, 0.447331, 0.446784, 0.446008, 0.445065, 0.443979, 0.442756, 0.441390,\
        0.439881, 0.438242, 0.436497, 0.434670, 0.432777, 0.430824, 0.428832, 0.426840, 0.424896, 0.423037,\
        0.421268, 0.419521, 0.417670, 0.415563, 0.413051, 0.410020, 0.406716, 0.403764, 0.400309, 0.396303,\
        0.392592, 0.389061, 0.385553, 0.381936, 0.378183, 0.374343, 0.370462, 0.366558, 0.362636, 0.358699,\
        0.354749, 0.350788, 0.346816, 0.342831, 0.338827, 0.334800, 0.330763, 0.326748, 0.322814, 0.319028,\
        0.315420, 0.311911, 0.308341, 0.304564, 0.300557, 0.296449, 0.291918, 0.286229, 0.279657, 0.272516,\
        0.264933],\
        [0.196307, 0.202227, 0.207682, 0.212915, 0.217918, 0.222679, 0.227208, 0.231531, 0.235674, 0.239663,\
        0.243522, 0.247273, 0.250936, 0.254531, 0.258076, 0.261590, 0.265092, 0.268598, 0.272124, 0.275681,\
        0.279285, 0.282978, 0.286860, 0.291067, 0.295626, 0.300259, 0.304336, 0.308786, 0.313930, 0.319646,\
        0.325754, 0.332115, 0.338637, 0.345255, 0.351915, 0.358553, 0.365079, 0.371382, 0.377308, 0.382640,\
        0.387262, 0.391560, 0.395812, 0.399388, 0.402778, 0.405962, 0.408406, 0.410314, 0.412132, 0.414029,\
        0.416124, 0.418516, 0.421143, 0.423828, 0.426475, 0.429059, 0.431533, 0.433809, 0.435791, 0.437403,\
        0.438594, 0.439345, 0.439668, 0.439605, 0.439220, 0.438579, 0.437733, 0.436717, 0.435545, 0.434222,\
        0.432750, 0.431143, 0.429435, 0.427662, 0.425829, 0.423916, 0.421930, 0.419929, 0.417989, 0.416180,\
        0.414550, 0.413034, 0.411448, 0.409623, 0.407397, 0.404486, 0.400802, 0.397937, 0.394843, 0.391016,\
        0.387237, 0.383601, 0.380019, 0.376406, 0.372723, 0.368977, 0.365187, 0.361366, 0.357520, 0.353651,\
        0.349763, 0.345857, 0.341934, 0.337986, 0.333996, 0.329947, 0.325836, 0.321696, 0.317604, 0.313673,\
        0.310022, 0.306554, 0.302990, 0.299156, 0.295013, 0.290650, 0.285818, 0.280191, 0.273767, 0.266710,\
        0.259144],\
        [0.193583, 0.199509, 0.205112, 0.210457, 0.215495, 0.220239, 0.224734, 0.229017, 0.233120, 0.237072,\
        0.240895, 0.244611, 0.248236, 0.251789, 0.255283, 0.258735, 0.262160, 0.265572, 0.268989, 0.272427,\
        0.275912, 0.279489, 0.283223, 0.287164, 0.291259, 0.295238, 0.298916, 0.303062, 0.308059, 0.313644,\
        0.319597, 0.325770, 0.332076, 0.338470, 0.344910, 0.351335, 0.357655, 0.363760, 0.369518, 0.374811,\
        0.379626, 0.384102, 0.388279, 0.392051, 0.395583, 0.398940, 0.401130, 0.402812, 0.404509, 0.406261,\
        0.408201, 0.410576, 0.413291, 0.415951, 0.418520, 0.421076, 0.423571, 0.425881, 0.427887, 0.429520,\
        0.430760, 0.431601, 0.432043, 0.432111, 0.431855, 0.431327, 0.430573, 0.429629, 0.428522, 0.427255,\
        0.425824, 0.424238, 0.422550, 0.420833, 0.419090, 0.417230, 0.415221, 0.413175, 0.411198, 0.409372,\
        0.407836, 0.406560, 0.405198, 0.403587, 0.401649, 0.399083, 0.395546, 0.392219, 0.388976, 0.385405,\
        0.381718, 0.378066, 0.374455, 0.370847, 0.367212, 0.363540, 0.359833, 0.356093, 0.352322, 0.348519,\
        0.344689, 0.340836, 0.336965, 0.333068, 0.329117, 0.325069, 0.320892, 0.316608, 0.312314, 0.308137,\
        0.304348, 0.300970, 0.297323, 0.293402, 0.289149, 0.284558, 0.279496, 0.273798, 0.267421, 0.260426,\
        0.252898],\
        [0.190783, 0.196778, 0.202494, 0.207959, 0.213005, 0.217727, 0.222192, 0.226446, 0.230523, 0.234452,\
        0.238255, 0.241951, 0.245555, 0.249081, 0.252540, 0.255946, 0.259310, 0.262647, 0.265973, 0.269305,\
        0.272671, 0.276102, 0.279629, 0.283254, 0.286910, 0.290477, 0.294007, 0.297951, 0.302684, 0.308073,\
        0.313792, 0.319690, 0.325691, 0.331781, 0.337942, 0.344114, 0.350204, 0.356105, 0.361715, 0.366959,\
        0.371826, 0.376353, 0.380552, 0.384419, 0.387977, 0.391077, 0.393377, 0.395169, 0.396872, 0.398591,\
        0.400408, 0.402865, 0.405778, 0.408302, 0.410722, 0.413272, 0.415836, 0.418216, 0.420250, 0.421886,\
        0.423165, 0.424090, 0.424632, 0.424800, 0.424645, 0.424213, 0.423538, 0.422664, 0.421632, 0.420445,\
        0.419071, 0.417495, 0.415787, 0.414118, 0.412549, 0.410795, 0.408711, 0.406595, 0.404584, 0.402690,\
        0.401147, 0.400073, 0.398822, 0.397266, 0.395381, 0.392892, 0.389761, 0.386431, 0.383068, 0.379574,\
        0.375983, 0.372375, 0.368783, 0.365201, 0.361616, 0.358015, 0.354390, 0.350733, 0.347038, 0.343299,\
        0.339520, 0.335713, 0.331893, 0.328063, 0.324193, 0.320204, 0.316003, 0.311579, 0.307105, 0.302728,\
        0.298585, 0.295135, 0.291419, 0.287375, 0.282983, 0.278203, 0.272955, 0.267149, 0.260748, 0.253770,\
        0.246263],\
        [0.187969, 0.194164, 0.200067, 0.205443, 0.210424, 0.215117, 0.219562, 0.223800, 0.227867, 0.231790,\
        0.235591, 0.239285, 0.242886, 0.246403, 0.249845, 0.253221, 0.256542, 0.259820, 0.263069, 0.266307,\
        0.269554, 0.272832, 0.276155, 0.279514, 0.282882, 0.286245, 0.289717, 0.293560, 0.298019, 0.303035,\
        0.308416, 0.313936, 0.319517, 0.325226, 0.331068, 0.336969, 0.342819, 0.348512, 0.353961, 0.359113,\
        0.363947, 0.368464, 0.372660, 0.376520, 0.380008, 0.383011, 0.385444, 0.387452, 0.389281, 0.391100,\
        0.393082, 0.395620, 0.398476, 0.400739, 0.403025, 0.405646, 0.408312, 0.410795, 0.412835, 0.414411,\
        0.415720, 0.416750, 0.417381, 0.417610, 0.417536, 0.417195, 0.416581, 0.415760, 0.414811, 0.413739,\
        0.412468, 0.410908, 0.409112, 0.407404, 0.406167, 0.404674, 0.402383, 0.400197, 0.398204, 0.396325,\
        0.394761, 0.393551, 0.392266, 0.390700, 0.388775, 0.386368, 0.383485, 0.380312, 0.376991, 0.373559,\
        0.370047, 0.366503, 0.362958, 0.359426, 0.355904, 0.352381, 0.348845, 0.345279, 0.341665, 0.337990,\
        0.334253, 0.330473, 0.326688, 0.322932, 0.319190, 0.315351, 0.311231, 0.306689, 0.301994, 0.297606,\
        0.293354, 0.289408, 0.285430, 0.281187, 0.276606, 0.271634, 0.266211, 0.260281, 0.253814, 0.246812,\
        0.239302],\
        [0.185026, 0.191347, 0.197263, 0.202654, 0.207674, 0.212377, 0.216821, 0.221064, 0.225140, 0.229076,\
        0.232893, 0.236606, 0.240224, 0.243752, 0.247195, 0.250560, 0.253854, 0.257088, 0.260275, 0.263430,\
        0.266569, 0.269709, 0.272860, 0.276026, 0.279213, 0.282459, 0.285871, 0.289615, 0.293819, 0.298466,\
        0.303490, 0.308504, 0.313533, 0.318816, 0.324339, 0.329976, 0.335585, 0.341054, 0.346315, 0.351329,\
        0.356074, 0.360533, 0.364687, 0.368506, 0.371945, 0.374949, 0.377509, 0.379721, 0.381749, 0.383760,\
        0.385931, 0.388394, 0.390947, 0.393273, 0.395618, 0.398260, 0.400959, 0.403565, 0.405573, 0.406945,\
        0.408330, 0.409532, 0.410241, 0.410452, 0.410458, 0.410238, 0.409655, 0.408852, 0.407983, 0.407060,\
        0.405966, 0.404468, 0.402610, 0.400648, 0.399750, 0.398660, 0.396274, 0.394012, 0.391986, 0.390131,\
        0.388512, 0.387088, 0.385646, 0.383997, 0.382030, 0.379677, 0.376944, 0.373920, 0.370703, 0.367354,\
        0.363920, 0.360444, 0.356962, 0.353495, 0.350049, 0.346619, 0.343186, 0.339725, 0.336205, 0.332601,\
        0.328898, 0.325115, 0.321319, 0.317603, 0.314011, 0.310412, 0.306536, 0.302075, 0.297131, 0.292519,\
        0.288076, 0.283755, 0.279417, 0.274891, 0.270071, 0.264886, 0.259282, 0.253213, 0.246653, 0.239596,\
        0.232055],\
        [0.181872, 0.188245, 0.194182, 0.199669, 0.204772, 0.209498, 0.213961, 0.218232, 0.222336, 0.226303,\
        0.230157, 0.233910, 0.237566, 0.241125, 0.244589, 0.247960, 0.251244, 0.254449, 0.257588, 0.260673,\
        0.263721, 0.266747, 0.269765, 0.272790, 0.275851, 0.278998, 0.282321, 0.285928, 0.289900, 0.294253,\
        0.298918, 0.303203, 0.307659, 0.312569, 0.317823, 0.323226, 0.328586, 0.333800, 0.338831, 0.343663,\
        0.348279, 0.352649, 0.356739, 0.360513, 0.363931, 0.366969, 0.369645, 0.372034, 0.374255, 0.376441,\
        0.378712, 0.381112, 0.383556, 0.385957, 0.388409, 0.391009, 0.393687, 0.396279, 0.398184, 0.399407,\
        0.400963, 0.402365, 0.403154, 0.403200, 0.403332, 0.403328, 0.402710, 0.401894, 0.401100, 0.400298,\
        0.399387, 0.397979, 0.396213, 0.394451, 0.393231, 0.391926, 0.389959, 0.387811, 0.385778, 0.383907,\
        0.382202, 0.380609, 0.378998, 0.377234, 0.375216, 0.372888, 0.370248, 0.367341, 0.364228, 0.360968,\
        0.357611, 0.354203, 0.350786, 0.347391, 0.344032, 0.340708, 0.337397, 0.334063, 0.330663, 0.327149,\
        0.323484, 0.319667, 0.315781, 0.312001, 0.308515, 0.305161, 0.301521, 0.297200, 0.292258, 0.287362,\
        0.282628, 0.277976, 0.273296, 0.268464, 0.263381, 0.257970, 0.252174, 0.245953, 0.239280, 0.232145,\
        0.224552],\
        [0.178521, 0.184963, 0.190986, 0.196588, 0.201754, 0.206467, 0.210984, 0.215314, 0.219457, 0.223467,\
        0.227377, 0.231193, 0.234911, 0.238523, 0.242026, 0.245419, 0.248708, 0.251899, 0.255003, 0.258035,\
        0.261010, 0.263944, 0.266856, 0.269772, 0.272726, 0.275769, 0.278969, 0.282396, 0.286092, 0.290035,\
        0.294048, 0.297895, 0.301984, 0.306596, 0.311647, 0.316838, 0.321908, 0.326804, 0.331550, 0.336163,\
        0.340622, 0.344886, 0.348905, 0.352634, 0.356042, 0.359118, 0.361889, 0.364416, 0.366789, 0.369105,\
        0.371441, 0.373830, 0.376249, 0.378682, 0.381164, 0.383715, 0.386272, 0.388650, 0.390546, 0.392063,\
        0.393592, 0.394984, 0.395800, 0.395943, 0.396147, 0.396216, 0.395683, 0.394950, 0.394206, 0.393431,\
        0.392499, 0.391224, 0.389665, 0.388085, 0.386655, 0.385160, 0.383369, 0.381398, 0.379431, 0.377553,\
        0.375774, 0.374052, 0.372302, 0.370429, 0.368354, 0.366032, 0.363451, 0.360630, 0.357609, 0.354429,\
        0.351139, 0.347789, 0.344429, 0.341102, 0.337835, 0.334629, 0.331460, 0.328280, 0.325032, 0.321652,\
        0.318062, 0.314206, 0.310162, 0.306151, 0.302614, 0.299441, 0.295768, 0.291501, 0.286731, 0.281793,\
        0.276864, 0.271946, 0.266973, 0.261856, 0.256513, 0.250873, 0.244882, 0.238500, 0.231701, 0.224471,\
        0.216808],\
        [0.174990, 0.181520, 0.187638, 0.193319, 0.198506, 0.203249, 0.207970, 0.212344, 0.216501, 0.220562,\
        0.224553, 0.228461, 0.232264, 0.235949, 0.239507, 0.242937, 0.246243, 0.249432, 0.252515, 0.255506,\
        0.258422, 0.261283, 0.264109, 0.266930, 0.269782, 0.272709, 0.275761, 0.278980, 0.282378, 0.285916,\
        0.289492, 0.293076, 0.296877, 0.301088, 0.306021, 0.310943, 0.315609, 0.320087, 0.324488, 0.328857,\
        0.333154, 0.337309, 0.341254, 0.344937, 0.348327, 0.351425, 0.354259, 0.356881, 0.359360, 0.361764,\
        0.364149, 0.366543, 0.368953, 0.371380, 0.373833, 0.376300, 0.378715, 0.380946, 0.382874, 0.384540,\
        0.386052, 0.387335, 0.388183, 0.388604, 0.388837, 0.388849, 0.388501, 0.387915, 0.387225, 0.386442,\
        0.385495, 0.384307, 0.382913, 0.381434, 0.379943, 0.378374, 0.376647, 0.374792, 0.372898, 0.371029,\
        0.369201, 0.367386, 0.365531, 0.363569, 0.361445, 0.359119, 0.356578, 0.353826, 0.350879, 0.347767,\
        0.344529, 0.341218, 0.337895, 0.334622, 0.331441, 0.328362, 0.325354, 0.322354, 0.319287, 0.316095,\
        0.312675, 0.308830, 0.304640, 0.300449, 0.296637, 0.293288, 0.289560, 0.285324, 0.280658, 0.275739,\
        0.270703, 0.265592, 0.260382, 0.255017, 0.249434, 0.243575, 0.237392, 0.230848, 0.223915, 0.216579,\
        0.208833],\
        [0.171283, 0.177917, 0.184147, 0.189947, 0.195286, 0.200394, 0.205093, 0.209299, 0.213438, 0.217578,\
        0.221691, 0.225722, 0.229633, 0.233406, 0.237032, 0.240510, 0.243843, 0.247039, 0.250111, 0.253073,\
        0.255943, 0.258741, 0.261492, 0.264226, 0.266976, 0.269780, 0.272675, 0.275686, 0.278820, 0.282053,\
        0.285360, 0.288768, 0.292409, 0.296498, 0.301115, 0.305540, 0.309655, 0.313608, 0.317633, 0.321771,\
        0.325920, 0.329972, 0.333843, 0.337470, 0.340823, 0.343911, 0.346768, 0.349442, 0.351983, 0.354440,\
        0.356852, 0.359246, 0.361635, 0.364025, 0.366416, 0.368785, 0.371079, 0.373217, 0.375141, 0.376845,\
        0.378344, 0.379585, 0.380490, 0.381059, 0.381364, 0.381413, 0.381182, 0.380721, 0.380097, 0.379331,\
        0.378398, 0.377273, 0.375978, 0.374568, 0.373079, 0.371501, 0.369811, 0.368025, 0.366186, 0.364332,\
        0.362475, 0.360599, 0.358668, 0.356640, 0.354476, 0.352148, 0.349640, 0.346948, 0.344070, 0.341016,\
        0.337813, 0.334515, 0.331197, 0.327950, 0.324841, 0.321889, 0.319056, 0.316258, 0.313375, 0.310370,\
        0.307303, 0.303567, 0.299254, 0.294982, 0.290958, 0.287142, 0.283177, 0.278863, 0.274198, 0.269265,\
        0.264148, 0.258891, 0.253489, 0.247912, 0.242116, 0.236056, 0.229690, 0.222985, 0.215916, 0.208467,\
        0.200630],\
        [0.167396, 0.174144, 0.180510, 0.186496, 0.192153, 0.197492, 0.201848, 0.206015, 0.210221, 0.214526,\
        0.218817, 0.222999, 0.227032, 0.230901, 0.234602, 0.238133, 0.241499, 0.244710, 0.247777, 0.250717,\
        0.253548, 0.256292, 0.258975, 0.261625, 0.264274, 0.266954, 0.269694, 0.272518, 0.275433, 0.278441,\
        0.281554, 0.284819, 0.288339, 0.292219, 0.296358, 0.300322, 0.303903, 0.307274, 0.310978, 0.314957,\
        0.318978, 0.322933, 0.326726, 0.330274, 0.333551, 0.336587, 0.339424, 0.342107, 0.344673, 0.347152,\
        0.349567, 0.351941, 0.354289, 0.356619, 0.358929, 0.361199, 0.363389, 0.365449, 0.367340, 0.369039,\
        0.370528, 0.371771, 0.372727, 0.373388, 0.373773, 0.373893, 0.373755, 0.373389, 0.372830, 0.372100,\
        0.371197, 0.370122, 0.368890, 0.367531, 0.366068, 0.364508, 0.362851, 0.361111, 0.359311, 0.357473,\
        0.355602, 0.353687, 0.351704, 0.349627, 0.347433, 0.345106, 0.342635, 0.340007, 0.337203, 0.334208,\
        0.331030, 0.327716, 0.324360, 0.321094, 0.318031, 0.315196, 0.312539, 0.309967, 0.307304, 0.304237,\
        0.301624, 0.298251, 0.293786, 0.289363, 0.285087, 0.280911, 0.276659, 0.272182, 0.267434, 0.262437,\
        0.257232, 0.251847, 0.246284, 0.240525, 0.234541, 0.228297, 0.221760, 0.214901, 0.207697, 0.200133,\
        0.192198],\
        [0.163321, 0.170185, 0.176676, 0.182788, 0.188498, 0.193672, 0.198237, 0.202530, 0.206890, 0.211494,\
        0.215994, 0.220323, 0.224471, 0.228435, 0.232210, 0.235796, 0.239198, 0.242427, 0.245494, 0.248417,\
        0.251214, 0.253909, 0.256526, 0.259094, 0.261644, 0.264203, 0.266800, 0.269455, 0.272186, 0.275008,\
        0.277946, 0.281042, 0.284351, 0.287885, 0.291525, 0.295037, 0.298159, 0.301090, 0.304657, 0.308537,\
        0.312390, 0.316244, 0.319955, 0.323381, 0.326524, 0.329451, 0.332225, 0.334883, 0.337444, 0.339917,\
        0.342311, 0.344640, 0.346923, 0.349173, 0.351391, 0.353561, 0.355655, 0.357639, 0.359484, 0.361161,\
        0.362645, 0.363904, 0.364907, 0.365639, 0.366099, 0.366294, 0.366235, 0.365943, 0.365443, 0.364754,\
        0.363888, 0.362855, 0.361669, 0.360351, 0.358919, 0.357385, 0.355758, 0.354053, 0.352284, 0.350463,\
        0.348589, 0.346652, 0.344635, 0.342522, 0.340304, 0.337979, 0.335550, 0.333002, 0.330293, 0.327376,\
        0.324225, 0.320874, 0.317429, 0.314082, 0.311037, 0.308302, 0.305788, 0.303396, 0.301050, 0.298081,\
        0.295100, 0.291850, 0.287679, 0.283252, 0.278823, 0.274405, 0.269914, 0.265258, 0.260386, 0.255290,\
        0.249983, 0.244475, 0.238768, 0.232849, 0.226697, 0.220287, 0.213591, 0.206585, 0.199250, 0.191571,\
        0.183537],\
        [0.159051, 0.166032, 0.172636, 0.178843, 0.184610, 0.189874, 0.194637, 0.198934, 0.203812, 0.208665,\
        0.213278, 0.217704, 0.221949, 0.225998, 0.229844, 0.233483, 0.236922, 0.240170, 0.243240, 0.246147,\
        0.248913, 0.251560, 0.254114, 0.256602, 0.259054, 0.261498, 0.263959, 0.266463, 0.269029, 0.271678,\
        0.274436, 0.277329, 0.280376, 0.283552, 0.286757, 0.289844, 0.292714, 0.295592, 0.299026, 0.302599,\
        0.306157, 0.309959, 0.313587, 0.316804, 0.319725, 0.322485, 0.325158, 0.327768, 0.330309, 0.332758,\
        0.335104, 0.337359, 0.339551, 0.341703, 0.343819, 0.345887, 0.347887, 0.349794, 0.351585, 0.353237,\
        0.354719, 0.356000, 0.357048, 0.357841, 0.358370, 0.358632, 0.358639, 0.358405, 0.357954, 0.357304,\
        0.356472, 0.355474, 0.354324, 0.353041, 0.351638, 0.350131, 0.348532, 0.346854, 0.345112, 0.343310,\
        0.341445, 0.339502, 0.337464, 0.335321, 0.333076, 0.330747, 0.328361, 0.325914, 0.323341, 0.320539,\
        0.317439, 0.314057, 0.310494, 0.306975, 0.303940, 0.301292, 0.298852, 0.296528, 0.294163, 0.291410,\
        0.288276, 0.284831, 0.280861, 0.276543, 0.272075, 0.267521, 0.262861, 0.258048, 0.253045, 0.247835,\
        0.242416, 0.236787, 0.230946, 0.224882, 0.218579, 0.212017, 0.205174, 0.198031, 0.190570, 0.182777,\
        0.174642],\
        [0.154589, 0.161693, 0.168422, 0.174754, 0.180667, 0.186152, 0.191270, 0.196270, 0.201302, 0.206003,\
        0.210613, 0.215106, 0.219438, 0.223570, 0.227482, 0.231173, 0.234647, 0.237913, 0.240986, 0.243880,\
        0.246615, 0.249216, 0.251706, 0.254116, 0.256472, 0.258804, 0.261138, 0.263499, 0.265909, 0.268390,\
        0.270962, 0.273640, 0.276425, 0.279286, 0.282159, 0.284974, 0.287737, 0.290609, 0.293754, 0.296986,\
        0.300303, 0.304208, 0.307658, 0.310486, 0.313094, 0.315651, 0.318207, 0.320764, 0.323283, 0.325699,\
        0.327969, 0.330113, 0.332184, 0.334221, 0.336229, 0.338194, 0.340096, 0.341921, 0.343656, 0.345280,\
        0.346766, 0.348074, 0.349167, 0.350017, 0.350604, 0.350924, 0.350982, 0.350792, 0.350377, 0.349757,\
        0.348954, 0.347983, 0.346863, 0.345608, 0.344232, 0.342748, 0.341171, 0.339516, 0.337798, 0.336022,\
        0.334178, 0.332244, 0.330196, 0.328028, 0.325747, 0.323391, 0.321028, 0.318704, 0.316324, 0.313685,\
        0.310671, 0.307296, 0.303691, 0.300001, 0.296914, 0.294297, 0.291831, 0.289431, 0.286931, 0.284142,\
        0.280991, 0.277474, 0.273561, 0.269319, 0.264853, 0.260229, 0.255456, 0.250519, 0.245395, 0.240071,\
        0.234538, 0.228791, 0.222824, 0.216626, 0.210184, 0.203483, 0.196504, 0.189231, 0.181649, 0.173746,\
        0.165511],\
        [0.149934, 0.157176, 0.164051, 0.170550, 0.176668, 0.182429, 0.187908, 0.193217, 0.198320, 0.203158,\
        0.207855, 0.212463, 0.216903, 0.221120, 0.225096, 0.228834, 0.232341, 0.235626, 0.238702, 0.241583,\
        0.244288, 0.246841, 0.249269, 0.251599, 0.253861, 0.256084, 0.258296, 0.260520, 0.262780, 0.265097,\
        0.267486, 0.269956, 0.272504, 0.275107, 0.277733, 0.280362, 0.283021, 0.285794, 0.288739, 0.291839,\
        0.295215, 0.298933, 0.301928, 0.304241, 0.306554, 0.308922, 0.311364, 0.313875, 0.316392, 0.318776,\
        0.320932, 0.322909, 0.324824, 0.326737, 0.328640, 0.330500, 0.332299, 0.334033, 0.335704, 0.337302,\
        0.338796, 0.340138, 0.341279, 0.342181, 0.342820, 0.343184, 0.343277, 0.343115, 0.342721, 0.342121,\
        0.341337, 0.340387, 0.339291, 0.338060, 0.336707, 0.335243, 0.333681, 0.332040, 0.330342, 0.328598,\
        0.326794, 0.324887, 0.322843, 0.320656, 0.318332, 0.315911, 0.313509, 0.311304, 0.309195, 0.306743,\
        0.303825, 0.300492, 0.296910, 0.293339, 0.290125, 0.287325, 0.284714, 0.282145, 0.279475, 0.276572,\
        0.273356, 0.269796, 0.265891, 0.261677, 0.257212, 0.252544, 0.247692, 0.242657, 0.237428, 0.231997,\
        0.226354, 0.220493, 0.214406, 0.208083, 0.201513, 0.194682, 0.187576, 0.180181, 0.172484, 0.164474,\
        0.156142],\
        [0.145081, 0.152476, 0.159520, 0.166210, 0.172554, 0.178576, 0.184322, 0.189824, 0.195067, 0.200051,\
        0.204948, 0.209747, 0.214318, 0.218614, 0.222646, 0.226428, 0.229968, 0.233274, 0.236354, 0.239222,\
        0.241897, 0.244402, 0.246765, 0.249015, 0.251184, 0.253301, 0.255393, 0.257486, 0.259603, 0.261761,\
        0.263976, 0.266255, 0.268595, 0.270986, 0.273418, 0.275890, 0.278428, 0.281079, 0.283882, 0.286864,\
        0.290047, 0.293252, 0.295865, 0.297998, 0.300125, 0.302327, 0.304658, 0.307116, 0.309669, 0.312037,\
        0.314007, 0.315732, 0.317460, 0.319262, 0.321074, 0.322830, 0.324513, 0.326140, 0.327737, 0.329310,\
        0.330823, 0.332207, 0.333397, 0.334347, 0.335029, 0.335426, 0.335536, 0.335381, 0.334992, 0.334399,\
        0.333624, 0.332688, 0.331610, 0.330401, 0.329069, 0.327621, 0.326067, 0.324429, 0.322739, 0.321033,\
        0.319295, 0.317438, 0.315404, 0.313215, 0.310866, 0.308365, 0.305828, 0.303639, 0.301889, 0.299539,\
        0.296664, 0.293405, 0.289929, 0.286475, 0.283234, 0.280258, 0.277450, 0.274680, 0.271823, 0.268773,\
        0.265456, 0.261836, 0.257906, 0.253684, 0.249204, 0.244497, 0.239581, 0.234464, 0.229143, 0.223613,\
        0.217866, 0.211896, 0.205696, 0.199256, 0.192566, 0.185615, 0.178390, 0.170879, 0.163072, 0.154959,\
        0.146530],\
        [0.140020, 0.147580, 0.154805, 0.161698, 0.168268, 0.174534, 0.180518, 0.186234, 0.191680, 0.196882,\
        0.201978, 0.206998, 0.211658, 0.216005, 0.220083, 0.223908, 0.227485, 0.230817, 0.233907, 0.236763,\
        0.239406, 0.241860, 0.244156, 0.246326, 0.248402, 0.250414, 0.252391, 0.254358, 0.256337, 0.258346,\
        0.260399, 0.262504, 0.264663, 0.266874, 0.269138, 0.271461, 0.273864, 0.276374, 0.279013, 0.281783,\
        0.284631, 0.287375, 0.289777, 0.291874, 0.293893, 0.295970, 0.298168, 0.300530, 0.303133, 0.305519,\
        0.307162, 0.308532, 0.310077, 0.311824, 0.313570, 0.315212, 0.316760, 0.318255, 0.319758, 0.321311,\
        0.322863, 0.324299, 0.325532, 0.326522, 0.327240, 0.327657, 0.327762, 0.327589, 0.327188, 0.326591,\
        0.325817, 0.324887, 0.323822, 0.322635, 0.321326, 0.319893, 0.318343, 0.316694, 0.314987, 0.313306,\
        0.311676, 0.309905, 0.307855, 0.305682, 0.303368, 0.300822, 0.298259, 0.295965, 0.294138, 0.291868,\
        0.289089, 0.285962, 0.282641, 0.279293, 0.276051, 0.272960, 0.269976, 0.267011, 0.263968, 0.260761,\
        0.257326, 0.253623, 0.249640, 0.245383, 0.240869, 0.236117, 0.231142, 0.225951, 0.220546, 0.214923,\
        0.209078, 0.203006, 0.196698, 0.190148, 0.183345, 0.176280, 0.168943, 0.161323, 0.153411, 0.145197,\
        0.136675],\
        [0.134733, 0.142465, 0.149879, 0.156978, 0.163771, 0.170273, 0.176499, 0.182463, 0.188182, 0.193703,\
        0.199117, 0.204183, 0.208830, 0.213212, 0.217341, 0.221220, 0.224849, 0.228218, 0.231322, 0.234168,\
        0.236776, 0.239176, 0.241402, 0.243489, 0.245473, 0.247383, 0.249248, 0.251095, 0.252945, 0.254815,\
        0.256720, 0.258668, 0.260666, 0.262717, 0.264826, 0.267001, 0.269257, 0.271605, 0.274052, 0.276575,\
        0.279112, 0.281546, 0.283775, 0.285820, 0.287792, 0.289803, 0.291924, 0.294224, 0.296713, 0.298992,\
        0.300243, 0.301293, 0.302734, 0.304487, 0.306166, 0.307663, 0.309062, 0.310399, 0.311774, 0.313321,\
        0.314946, 0.316437, 0.317689, 0.318697, 0.319453, 0.319884, 0.319953, 0.319730, 0.319304, 0.318699,\
        0.317916, 0.316982, 0.315928, 0.314763, 0.313480, 0.312067, 0.310524, 0.308862, 0.307109, 0.305390,\
        0.303910, 0.302300, 0.300160, 0.297986, 0.295715, 0.293221, 0.290722, 0.288406, 0.286259, 0.283917,\
        0.281219, 0.278218, 0.275028, 0.271770, 0.268536, 0.265366, 0.262242, 0.259108, 0.255894, 0.252533,\
        0.248971, 0.245170, 0.241115, 0.236801, 0.232235, 0.227429, 0.222392, 0.217130, 0.211644, 0.205934,\
        0.199997, 0.193826, 0.187417, 0.180761, 0.173852, 0.166680, 0.159236, 0.151512, 0.143499, 0.135189,\
        0.126575],\
        [0.129202, 0.137109, 0.144712, 0.152015, 0.159028, 0.165760, 0.172226, 0.178442, 0.184430, 0.190221,\
        0.195793, 0.200928, 0.205659, 0.210147, 0.214356, 0.218311, 0.222016, 0.225441, 0.228566, 0.231399,\
        0.233966, 0.236307, 0.238459, 0.240463, 0.242354, 0.244165, 0.245925, 0.247659, 0.249388, 0.251131,\
        0.252900, 0.254707, 0.256560, 0.258465, 0.260427, 0.262455, 0.264555, 0.266732, 0.268979, 0.271268,\
        0.273549, 0.275753, 0.277834, 0.279804, 0.281728, 0.283680, 0.285720, 0.287874, 0.290058, 0.291931,\
        0.293216, 0.294286, 0.295640, 0.297264, 0.298823, 0.300155, 0.301441, 0.302623, 0.303813, 0.305392,\
        0.307123, 0.308638, 0.309851, 0.310838, 0.311654, 0.312115, 0.312096, 0.311775, 0.311326, 0.310728,\
        0.309925, 0.308973, 0.307926, 0.306785, 0.305533, 0.304138, 0.302608, 0.300957, 0.299189, 0.297420,\
        0.295947, 0.294358, 0.292318, 0.290132, 0.287850, 0.285444, 0.283008, 0.280637, 0.278301, 0.275842,\
        0.273149, 0.270221, 0.267118, 0.263920, 0.260690, 0.257460, 0.254224, 0.250948, 0.247583, 0.244077,\
        0.240388, 0.236481, 0.232339, 0.227953, 0.223323, 0.218454, 0.213349, 0.208014, 0.202450, 0.196655,\
        0.190627, 0.184363, 0.177856, 0.171100, 0.164089, 0.156814, 0.149269, 0.141445, 0.133335, 0.124931,\
        0.116229],\
        [0.123405, 0.131486, 0.139276, 0.146778, 0.154001, 0.160954, 0.167647, 0.174093, 0.180301, 0.186265,\
        0.191926, 0.197186, 0.202138, 0.206787, 0.211068, 0.215134, 0.218957, 0.222458, 0.225605, 0.228416,\
        0.230933, 0.233206, 0.235281, 0.237201, 0.239003, 0.240721, 0.242382, 0.244011, 0.245630, 0.247256,\
        0.248904, 0.250583, 0.252303, 0.254072, 0.255895, 0.257778, 0.259724, 0.261731, 0.263787, 0.265868,\
        0.267937, 0.269955, 0.271900, 0.273780, 0.275630, 0.277498, 0.279411, 0.281357, 0.283238, 0.284876,\
        0.286192, 0.287365, 0.288644, 0.290054, 0.291435, 0.292692, 0.293872, 0.294974, 0.296111, 0.297670,\
        0.299394, 0.300858, 0.301972, 0.302883, 0.303805, 0.304347, 0.304153, 0.303690, 0.303248, 0.302689,\
        0.301845, 0.300871, 0.299829, 0.298702, 0.297467, 0.296086, 0.294569, 0.292935, 0.291212, 0.289499,\
        0.287880, 0.286178, 0.284226, 0.282085, 0.279821, 0.277461, 0.275052, 0.272633, 0.270183, 0.267629,\
        0.264907, 0.262001, 0.258940, 0.255770, 0.252532, 0.249248, 0.245918, 0.242521, 0.239022, 0.235383,\
        0.231570, 0.227555, 0.223319, 0.218850, 0.214145, 0.209203, 0.204026, 0.198615, 0.192971, 0.187092,\
        0.180977, 0.174621, 0.168020, 0.161168, 0.154059, 0.146686, 0.139043, 0.131123, 0.122919, 0.114425,\
        0.105635],\
        [0.117321, 0.125573, 0.133543, 0.141236, 0.148659, 0.155818, 0.162720, 0.169370, 0.175767, 0.181892,\
        0.187706, 0.193190, 0.198398, 0.203050, 0.207402, 0.211661, 0.215667, 0.219255, 0.222405, 0.225171,\
        0.227622, 0.229819, 0.231815, 0.233654, 0.235374, 0.237005, 0.238577, 0.240114, 0.241634, 0.243156,\
        0.244694, 0.246259, 0.247859, 0.249503, 0.251195, 0.252939, 0.254736, 0.256581, 0.258462, 0.260361,\
        0.262251, 0.264109, 0.265923, 0.267698, 0.269452, 0.271208, 0.272972, 0.274719, 0.276380, 0.277870,\
        0.279172, 0.280373, 0.281588, 0.282844, 0.284083, 0.285258, 0.286376, 0.287474, 0.288667, 0.290085,\
        0.291600, 0.292955, 0.294024, 0.294922, 0.295805, 0.296320, 0.296068, 0.295571, 0.295096, 0.294510,\
        0.293679, 0.292707, 0.291657, 0.290520, 0.289274, 0.287900, 0.286399, 0.284790, 0.283108, 0.281403,\
        0.279693, 0.277903, 0.275953, 0.273841, 0.271601, 0.269263, 0.266855, 0.264396, 0.261871, 0.259245,\
        0.256480, 0.253565, 0.250510, 0.247339, 0.244078, 0.240740, 0.237327, 0.233822, 0.230204, 0.226443,\
        0.222512, 0.218388, 0.214053, 0.209495, 0.204707, 0.199686, 0.194431, 0.188942, 0.183216, 0.177254,\
        0.171052, 0.164606, 0.157913, 0.150968, 0.143764, 0.136297, 0.128559, 0.120546, 0.112251, 0.103669,\
        0.094794],\
        [0.110930, 0.119346, 0.127489, 0.135363, 0.142972, 0.150322, 0.157415, 0.164252, 0.170826, 0.177118,\
        0.183105, 0.188760, 0.194025, 0.198793, 0.203361, 0.207956, 0.212191, 0.215812, 0.218909, 0.221598,\
        0.223968, 0.226088, 0.228009, 0.229774, 0.231421, 0.232978, 0.234474, 0.235930, 0.237365, 0.238797,\
        0.240238, 0.241701, 0.243194, 0.244723, 0.246294, 0.247910, 0.249568, 0.251264, 0.252988, 0.254726,\
        0.256459, 0.258174, 0.259861, 0.261522, 0.263166, 0.264798, 0.266415, 0.267993, 0.269490, 0.270868,\
        0.272123, 0.273300, 0.274454, 0.275606, 0.276742, 0.277845, 0.278922, 0.280012, 0.281179, 0.282457,\
        0.283770, 0.284977, 0.286000, 0.286864, 0.287568, 0.287921, 0.287792, 0.287377, 0.286854, 0.286209,\
        0.285394, 0.284438, 0.283379, 0.282224, 0.280962, 0.279585, 0.278094, 0.276503, 0.274837, 0.273120,\
        0.271353, 0.269501, 0.267525, 0.265415, 0.263183, 0.260848, 0.258428, 0.255931, 0.253349, 0.250664,\
        0.247854, 0.244910, 0.241835, 0.238642, 0.235344, 0.231949, 0.228455, 0.224854, 0.221126, 0.217252,\
        0.213209, 0.208978, 0.204542, 0.199890, 0.195015, 0.189910, 0.184573, 0.179001, 0.173193, 0.167146,\
        0.160857, 0.154323, 0.147540, 0.140503, 0.133207, 0.125648, 0.117819, 0.109716, 0.101332, 0.092663,\
        0.083705],\
        [0.104214, 0.112786, 0.121092, 0.129135, 0.136917, 0.144443, 0.151712, 0.158723, 0.165467, 0.171929,\
        0.178087, 0.183911, 0.189355, 0.194408, 0.199189, 0.204280, 0.208546, 0.212041, 0.215016, 0.217605,\
        0.219894, 0.221945, 0.223804, 0.225512, 0.227101, 0.228601, 0.230035, 0.231426, 0.232791, 0.234147,\
        0.235507, 0.236881, 0.238279, 0.239706, 0.241168, 0.242665, 0.244197, 0.245760, 0.247344, 0.248939,\
        0.250533, 0.252115, 0.253680, 0.255226, 0.256753, 0.258261, 0.259742, 0.261179, 0.262545, 0.263824,\
        0.265017, 0.266145, 0.267238, 0.268313, 0.269370, 0.270409, 0.271441, 0.272492, 0.273589, 0.274734,\
        0.275878, 0.276941, 0.277865, 0.278625, 0.279179, 0.279433, 0.279346, 0.278993, 0.278467, 0.277797,\
        0.276980, 0.276030, 0.274965, 0.273795, 0.272519, 0.271134, 0.269643, 0.268057, 0.266389, 0.264651,\
        0.262842, 0.260943, 0.258935, 0.256808, 0.254566, 0.252219, 0.249776, 0.247241, 0.244610, 0.241871,\
        0.239013, 0.236029, 0.232919, 0.229687, 0.226341, 0.222883, 0.219312, 0.215618, 0.211788, 0.207807,\
        0.203657, 0.199321, 0.194785, 0.190038, 0.185071, 0.179878, 0.174456, 0.168799, 0.162906, 0.156773,\
        0.150397, 0.143775, 0.136903, 0.129777, 0.122391, 0.114742, 0.106824, 0.098632, 0.090162, 0.081408,\
        0.072368],\
        [0.097156, 0.105875, 0.114333, 0.122532, 0.130474, 0.138161, 0.145592, 0.152764, 0.159670, 0.166300,\
        0.172639, 0.178672, 0.184400, 0.189881, 0.195299, 0.200440, 0.204396, 0.207702, 0.210565, 0.213081,\
        0.215318, 0.217327, 0.219150, 0.220823, 0.222376, 0.223837, 0.225228, 0.226571, 0.227883, 0.229179,\
        0.230472, 0.231772, 0.233088, 0.234427, 0.235792, 0.237184, 0.238604, 0.240047, 0.241507, 0.242975,\
        0.244444, 0.245906, 0.247354, 0.248786, 0.250199, 0.251589, 0.252948, 0.254262, 0.255518, 0.256707,\
        0.257829, 0.258898, 0.259930, 0.260937, 0.261927, 0.262904, 0.263880, 0.264867, 0.265875, 0.266895,\
        0.267891, 0.268812, 0.269611, 0.270249, 0.270682, 0.270862, 0.270771, 0.270442, 0.269924, 0.269247,\
        0.268426, 0.267473, 0.266401, 0.265218, 0.263928, 0.262532, 0.261034, 0.259440, 0.257759, 0.255996,\
        0.254150, 0.252210, 0.250167, 0.248013, 0.245748, 0.243377, 0.240903, 0.238328, 0.235648, 0.232858,\
        0.229949, 0.226917, 0.223760, 0.220479, 0.217076, 0.213551, 0.209901, 0.206118, 0.202191, 0.198108,\
        0.193854, 0.189415, 0.184780, 0.179936, 0.174876, 0.169594, 0.164083, 0.158339, 0.152359, 0.146140,\
        0.139677, 0.132967, 0.126007, 0.118792, 0.111318, 0.103580, 0.095574, 0.087296, 0.078741, 0.069904,\
        0.060783]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=40 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax40vt0
    zgrid_H_Hmax_vmax40vt0=[[0.62956, 0.63335, 0.63685, 0.64010, 0.64313, 0.64598, 0.64868, 0.65127, 0.65380, 0.65631, 0.65885,\
        0.66148, 0.66427, 0.66729, 0.67062, 0.67431, 0.67843, 0.68300, 0.68798, 0.69327, 0.69846, 0.70247,\
        0.70584, 0.70907, 0.71219, 0.71513, 0.71783, 0.72026, 0.72240, 0.72427, 0.72589, 0.72730, 0.72852,\
        0.72960, 0.73056, 0.73141, 0.73217, 0.73283, 0.73339, 0.73384, 0.73418, 0.73443, 0.73459, 0.73470,\
        0.73476, 0.73480, 0.73482, 0.73483, 0.73480, 0.73471, 0.73452, 0.73419, 0.73369, 0.73297, 0.73202,\
        0.73081, 0.72934, 0.72762, 0.72568, 0.72354, 0.72125, 0.71884, 0.71634, 0.71379, 0.71120, 0.70859,\
        0.70599, 0.70341, 0.70089, 0.69847, 0.69629, 0.69470, 0.69302, 0.69049, 0.68764, 0.68445, 0.68092,\
        0.67726, 0.67364, 0.67005, 0.66645, 0.66278, 0.65898, 0.65505, 0.65100, 0.64688, 0.64274, 0.63858,\
        0.63444, 0.63028, 0.62610, 0.62186, 0.61753, 0.61305, 0.60839, 0.60349, 0.59830, 0.59268, 0.58653,\
        0.58010, 0.57363, 0.56712, 0.56049, 0.55372, 0.54680, 0.53979, 0.53272, 0.52561, 0.51850, 0.51137,\
        0.50424, 0.49709, 0.48992, 0.48271, 0.47545, 0.46811, 0.46070, 0.45318, 0.44555, 0.43780, 0.42990],\
        [0.63614, 0.63968, 0.64293, 0.64590, 0.64865, 0.65120, 0.65359, 0.65587, 0.65806, 0.66021, 0.66238,\
        0.66462, 0.66700, 0.66961, 0.67253, 0.67584, 0.67960, 0.68386, 0.68859, 0.69368, 0.69873, 0.70329,\
        0.70746, 0.71145, 0.71523, 0.71872, 0.72187, 0.72463, 0.72703, 0.72907, 0.73082, 0.73232, 0.73363,\
        0.73479, 0.73585, 0.73681, 0.73768, 0.73847, 0.73914, 0.73970, 0.74014, 0.74046, 0.74069, 0.74085,\
        0.74098, 0.74111, 0.74125, 0.74139, 0.74152, 0.74161, 0.74160, 0.74146, 0.74112, 0.74055, 0.73971,\
        0.73858, 0.73715, 0.73545, 0.73350, 0.73135, 0.72903, 0.72660, 0.72410, 0.72155, 0.71898, 0.71642,\
        0.71387, 0.71135, 0.70890, 0.70655, 0.70442, 0.70297, 0.70132, 0.69869, 0.69591, 0.69269, 0.68890,\
        0.68507, 0.68140, 0.67782, 0.67422, 0.67052, 0.66664, 0.66256, 0.65834, 0.65406, 0.64978, 0.64554,\
        0.64133, 0.63716, 0.63297, 0.62874, 0.62441, 0.61994, 0.61527, 0.61036, 0.60516, 0.59954, 0.59312,\
        0.58661, 0.58026, 0.57386, 0.56728, 0.56050, 0.55355, 0.54651, 0.53943, 0.53234, 0.52527, 0.51822,\
        0.51119, 0.50416, 0.49711, 0.49003, 0.48291, 0.47572, 0.46846, 0.46110, 0.45362, 0.44603, 0.43829],\
        [0.64252, 0.64582, 0.64880, 0.65150, 0.65396, 0.65623, 0.65832, 0.66028, 0.66215, 0.66395, 0.66574,\
        0.66757, 0.66951, 0.67167, 0.67413, 0.67698, 0.68031, 0.68415, 0.68848, 0.69320, 0.69818, 0.70335,\
        0.70855, 0.71352, 0.71812, 0.72226, 0.72589, 0.72900, 0.73163, 0.73382, 0.73566, 0.73722, 0.73859,\
        0.73982, 0.74096, 0.74204, 0.74305, 0.74398, 0.74480, 0.74548, 0.74601, 0.74640, 0.74668, 0.74689,\
        0.74707, 0.74728, 0.74753, 0.74782, 0.74813, 0.74841, 0.74861, 0.74867, 0.74853, 0.74812, 0.74741,\
        0.74636, 0.74497, 0.74327, 0.74130, 0.73910, 0.73676, 0.73430, 0.73180, 0.72927, 0.72675, 0.72425,\
        0.72179, 0.71939, 0.71706, 0.71487, 0.71291, 0.71123, 0.70934, 0.70690, 0.70415, 0.70081, 0.69677,\
        0.69286, 0.68924, 0.68567, 0.68207, 0.67832, 0.67431, 0.67001, 0.66554, 0.66104, 0.65660, 0.65225,\
        0.64799, 0.64379, 0.63960, 0.63538, 0.63105, 0.62656, 0.62185, 0.61685, 0.61148, 0.60557, 0.59899,\
        0.59276, 0.58667, 0.58041, 0.57386, 0.56702, 0.55999, 0.55288, 0.54578, 0.53872, 0.53171, 0.52475,\
        0.51783, 0.51092, 0.50402, 0.49708, 0.49011, 0.48308, 0.47597, 0.46877, 0.46146, 0.45402, 0.44644],\
        [0.64871, 0.65176, 0.65448, 0.65691, 0.65909, 0.66107, 0.66289, 0.66456, 0.66612, 0.66759, 0.66901,\
        0.67044, 0.67195, 0.67363, 0.67561, 0.67800, 0.68088, 0.68432, 0.68832, 0.69284, 0.69789, 0.70373,\
        0.70995, 0.71589, 0.72130, 0.72606, 0.73014, 0.73354, 0.73633, 0.73860, 0.74047, 0.74204, 0.74342,\
        0.74470, 0.74592, 0.74712, 0.74828, 0.74938, 0.75037, 0.75121, 0.75185, 0.75230, 0.75261, 0.75284,\
        0.75305, 0.75332, 0.75368, 0.75413, 0.75463, 0.75513, 0.75556, 0.75585, 0.75591, 0.75569, 0.75511,\
        0.75415, 0.75279, 0.75107, 0.74905, 0.74681, 0.74441, 0.74193, 0.73943, 0.73693, 0.73448, 0.73207,\
        0.72973, 0.72746, 0.72529, 0.72325, 0.72136, 0.71953, 0.71752, 0.71513, 0.71229, 0.70885, 0.70488,\
        0.70099, 0.69735, 0.69371, 0.69005, 0.68624, 0.68202, 0.67738, 0.67257, 0.66781, 0.66319, 0.65873,\
        0.65442, 0.65020, 0.64602, 0.64181, 0.63749, 0.63298, 0.62822, 0.62311, 0.61758, 0.61154, 0.60517,\
        0.59906, 0.59309, 0.58689, 0.58025, 0.57325, 0.56607, 0.55888, 0.55176, 0.54474, 0.53781, 0.53097,\
        0.52417, 0.51741, 0.51065, 0.50387, 0.49706, 0.49019, 0.48324, 0.47620, 0.46906, 0.46178, 0.45437],\
        [0.65470, 0.65750, 0.65996, 0.66212, 0.66404, 0.66576, 0.66731, 0.66873, 0.67003, 0.67121, 0.67231,\
        0.67337, 0.67445, 0.67568, 0.67720, 0.67915, 0.68166, 0.68483, 0.68870, 0.69333, 0.69881, 0.70535,\
        0.71235, 0.71907, 0.72515, 0.73042, 0.73483, 0.73841, 0.74125, 0.74350, 0.74530, 0.74680, 0.74813,\
        0.74941, 0.75070, 0.75203, 0.75338, 0.75470, 0.75591, 0.75693, 0.75770, 0.75821, 0.75852, 0.75872,\
        0.75894, 0.75925, 0.75971, 0.76032, 0.76103, 0.76177, 0.76245, 0.76298, 0.76329, 0.76327, 0.76284,\
        0.76196, 0.76062, 0.75886, 0.75677, 0.75444, 0.75197, 0.74946, 0.74696, 0.74451, 0.74213, 0.73984,\
        0.73763, 0.73551, 0.73350, 0.73159, 0.72974, 0.72788, 0.72583, 0.72344, 0.72061, 0.71726, 0.71352,\
        0.70968, 0.70588, 0.70205, 0.69821, 0.69431, 0.68976, 0.68459, 0.67935, 0.67432, 0.66954, 0.66499,\
        0.66064, 0.65643, 0.65227, 0.64809, 0.64379, 0.63928, 0.63449, 0.62933, 0.62377, 0.61781, 0.61167,\
        0.60558, 0.59955, 0.59329, 0.58640, 0.57909, 0.57172, 0.56445, 0.55735, 0.55040, 0.54359, 0.53688,\
        0.53023, 0.52363, 0.51703, 0.51042, 0.50377, 0.49706, 0.49028, 0.48341, 0.47643, 0.46933, 0.46209],\
        [0.66051, 0.66305, 0.66524, 0.66713, 0.66879, 0.67027, 0.67161, 0.67282, 0.67392, 0.67489, 0.67573,\
        0.67646, 0.67717, 0.67798, 0.67909, 0.68068, 0.68294, 0.68601, 0.68999, 0.69502, 0.70127, 0.70852,\
        0.71613, 0.72342, 0.72998, 0.73557, 0.74013, 0.74373, 0.74647, 0.74855, 0.75016, 0.75149, 0.75271,\
        0.75395, 0.75529, 0.75676, 0.75834, 0.75994, 0.76144, 0.76270, 0.76362, 0.76418, 0.76445, 0.76458,\
        0.76474, 0.76507, 0.76562, 0.76640, 0.76734, 0.76834, 0.76929, 0.77008, 0.77064, 0.77085, 0.77059,\
        0.76979, 0.76844, 0.76661, 0.76441, 0.76196, 0.75941, 0.75685, 0.75437, 0.75198, 0.74969, 0.74752,\
        0.74545, 0.74348, 0.74162, 0.73983, 0.73807, 0.73625, 0.73424, 0.73192, 0.72918, 0.72600, 0.72244,\
        0.71865, 0.71473, 0.71069, 0.70659, 0.70235, 0.69728, 0.69142, 0.68581, 0.68058, 0.67568, 0.67106,\
        0.66670, 0.66251, 0.65840, 0.65427, 0.65001, 0.64553, 0.64075, 0.63559, 0.63003, 0.62414, 0.61805,\
        0.61191, 0.60576, 0.59933, 0.59202, 0.58436, 0.57682, 0.56956, 0.56254, 0.55572, 0.54906, 0.54252,\
        0.53604, 0.52960, 0.52317, 0.51673, 0.51025, 0.50371, 0.49710, 0.49040, 0.48359, 0.47666, 0.46959],\
        [0.66611, 0.66840, 0.67031, 0.67193, 0.67334, 0.67460, 0.67576, 0.67683, 0.67781, 0.67866, 0.67933,\
        0.67984, 0.68023, 0.68069, 0.68144, 0.68278, 0.68494, 0.68813, 0.69246, 0.69807, 0.70517, 0.71319,\
        0.72136, 0.72908, 0.73592, 0.74164, 0.74617, 0.74957, 0.75203, 0.75378, 0.75506, 0.75611, 0.75713,\
        0.75828, 0.75966, 0.76129, 0.76314, 0.76510, 0.76699, 0.76859, 0.76970, 0.77029, 0.77045, 0.77044,\
        0.77049, 0.77080, 0.77145, 0.77241, 0.77361, 0.77488, 0.77609, 0.77713, 0.77795, 0.77842, 0.77836,\
        0.77763, 0.77624, 0.77430, 0.77195, 0.76936, 0.76669, 0.76409, 0.76163, 0.75932, 0.75714, 0.75509,\
        0.75316, 0.75134, 0.74962, 0.74796, 0.74632, 0.74460, 0.74270, 0.74051, 0.73793, 0.73492, 0.73151,\
        0.72775, 0.72370, 0.71939, 0.71482, 0.70984, 0.70408, 0.69783, 0.69208, 0.68672, 0.68169, 0.67700,\
        0.67262, 0.66848, 0.66444, 0.66039, 0.65620, 0.65178, 0.64702, 0.64187, 0.63631, 0.63038, 0.62418,\
        0.61779, 0.61122, 0.60431, 0.59673, 0.58885, 0.58137, 0.57425, 0.56740, 0.56076, 0.55428, 0.54792,\
        0.54162, 0.53536, 0.52910, 0.52283, 0.51652, 0.51015, 0.50371, 0.49718, 0.49054, 0.48379, 0.47689],\
        [0.67151, 0.67353, 0.67515, 0.67648, 0.67764, 0.67871, 0.67974, 0.68074, 0.68169, 0.68254, 0.68319,\
        0.68358, 0.68375, 0.68390, 0.68437, 0.68556, 0.68781, 0.69133, 0.69631, 0.70282, 0.71073, 0.71936,\
        0.72799, 0.73602, 0.74301, 0.74867, 0.75295, 0.75595, 0.75792, 0.75914, 0.75995, 0.76061, 0.76135,\
        0.76236, 0.76375, 0.76555, 0.76773, 0.77015, 0.77257, 0.77464, 0.77603, 0.77663, 0.77661, 0.77635,\
        0.77622, 0.77649, 0.77723, 0.77840, 0.77986, 0.78144, 0.78288, 0.78409, 0.78515, 0.78595, 0.78612,\
        0.78543, 0.78397, 0.78190, 0.77936, 0.77657, 0.77378, 0.77115, 0.76874, 0.76653, 0.76448, 0.76256,\
        0.76075, 0.75905, 0.75746, 0.75594, 0.75444, 0.75288, 0.75114, 0.74913, 0.74675, 0.74393, 0.74063,\
        0.73688, 0.73267, 0.72802, 0.72292, 0.71727, 0.71104, 0.70459, 0.69853, 0.69291, 0.68766, 0.68284,\
        0.67848, 0.67440, 0.67045, 0.66649, 0.66239, 0.65805, 0.65334, 0.64820, 0.64258, 0.63653, 0.63007,\
        0.62327, 0.61610, 0.60850, 0.60044, 0.59270, 0.58556, 0.57869, 0.57204, 0.56560, 0.55932, 0.55314,\
        0.54703, 0.54094, 0.53485, 0.52874, 0.52259, 0.51638, 0.51011, 0.50375, 0.49729, 0.49071, 0.48399],\
        [0.67670, 0.67841, 0.67972, 0.68075, 0.68165, 0.68254, 0.68348, 0.68448, 0.68552, 0.68650, 0.68729,\
        0.68773, 0.68778, 0.68769, 0.68793, 0.68910, 0.69164, 0.69571, 0.70158, 0.70907, 0.71771, 0.72687,\
        0.73589, 0.74415, 0.75115, 0.75659, 0.76041, 0.76278, 0.76404, 0.76457, 0.76475, 0.76492, 0.76532,\
        0.76616, 0.76753, 0.76951, 0.77207, 0.77505, 0.77816, 0.78091, 0.78272, 0.78331, 0.78299, 0.78238,\
        0.78201, 0.78222, 0.78309, 0.78444, 0.78614, 0.78808, 0.78975, 0.79090, 0.79213, 0.79336, 0.79379,\
        0.79309, 0.79150, 0.78935, 0.78661, 0.78357, 0.78065, 0.77803, 0.77573, 0.77366, 0.77174, 0.76991,\
        0.76820, 0.76660, 0.76512, 0.76373, 0.76238, 0.76100, 0.75947, 0.75768, 0.75553, 0.75290, 0.74973,\
        0.74597, 0.74159, 0.73662, 0.73105, 0.72493, 0.71838, 0.71169, 0.70525, 0.69927, 0.69360, 0.68862,\
        0.68433, 0.68033, 0.67645, 0.67259, 0.66860, 0.66436, 0.65971, 0.65457, 0.64887, 0.64262, 0.63586,\
        0.62863, 0.62094, 0.61281, 0.60445, 0.59688, 0.58988, 0.58310, 0.57660, 0.57033, 0.56423, 0.55824,\
        0.55229, 0.54636, 0.54042, 0.53446, 0.52847, 0.52242, 0.51631, 0.51012, 0.50383, 0.49743, 0.49089],\
        [0.68164, 0.68304, 0.68398, 0.68467, 0.68531, 0.68603, 0.68691, 0.68798, 0.68918, 0.69042, 0.69155,\
        0.69226, 0.69232, 0.69209, 0.69215, 0.69339, 0.69655, 0.70144, 0.70821, 0.71650, 0.72575, 0.73538,\
        0.74475, 0.75321, 0.76016, 0.76524, 0.76839, 0.76990, 0.77023, 0.76991, 0.76937, 0.76898, 0.76901,\
        0.76965, 0.77101, 0.77316, 0.77609, 0.77968, 0.78364, 0.78735, 0.78983, 0.79037, 0.78967, 0.78867,\
        0.78801, 0.78816, 0.78924, 0.79075, 0.79246, 0.79480, 0.79684, 0.79755, 0.79883, 0.80048, 0.80109,\
        0.80038, 0.79869, 0.79666, 0.79363, 0.79033, 0.78733, 0.78479, 0.78266, 0.78077, 0.77896, 0.77719,\
        0.77551, 0.77396, 0.77255, 0.77128, 0.77008, 0.76888, 0.76757, 0.76605, 0.76415, 0.76175, 0.75872,\
        0.75496, 0.75044, 0.74517, 0.73923, 0.73273, 0.72585, 0.71888, 0.71206, 0.70556, 0.69939, 0.69449,\
        0.69032, 0.68632, 0.68245, 0.67867, 0.67481, 0.67070, 0.66614, 0.66099, 0.65519, 0.64874, 0.64168,\
        0.63409, 0.62606, 0.61774, 0.60950, 0.60178, 0.59448, 0.58760, 0.58116, 0.57503, 0.56910, 0.56326,\
        0.55745, 0.55165, 0.54585, 0.54002, 0.53416, 0.52826, 0.52231, 0.51629, 0.51018, 0.50396, 0.49760],\
        [0.68633, 0.68736, 0.68788, 0.68820, 0.68855, 0.68912, 0.68998, 0.69114, 0.69256, 0.69413, 0.69571,\
        0.69701, 0.69715, 0.69705, 0.69715, 0.69839, 0.70250, 0.70840, 0.71589, 0.72473, 0.73444, 0.74447,\
        0.75417, 0.76284, 0.76975, 0.77436, 0.77662, 0.77703, 0.77627, 0.77499, 0.77368, 0.77274, 0.77242,\
        0.77289, 0.77425, 0.77655, 0.77982, 0.78401, 0.78881, 0.79367, 0.79727, 0.79772, 0.79676, 0.79553,\
        0.79461, 0.79461, 0.79603, 0.79775, 0.79901, 0.80140, 0.80368, 0.80459, 0.80576, 0.80716, 0.80772,\
        0.80714, 0.80566, 0.80355, 0.80032, 0.79699, 0.79399, 0.79156, 0.78970, 0.78798, 0.78621, 0.78441,\
        0.78267, 0.78111, 0.77974, 0.77853, 0.77744, 0.77641, 0.77534, 0.77408, 0.77248, 0.77034, 0.76749,\
        0.76377, 0.75913, 0.75360, 0.74731, 0.74046, 0.73328, 0.72601, 0.71887, 0.71204, 0.70581, 0.70092,\
        0.69648, 0.69231, 0.68839, 0.68469, 0.68100, 0.67706, 0.67261, 0.66747, 0.66156, 0.65489, 0.64756,\
        0.63968, 0.63142, 0.62298, 0.61466, 0.60669, 0.59913, 0.59214, 0.58576, 0.57977, 0.57396, 0.56823,\
        0.56253, 0.55683, 0.55112, 0.54540, 0.53966, 0.53391, 0.52811, 0.52226, 0.51632, 0.51029, 0.50412],\
        [0.69073, 0.69132, 0.69137, 0.69127, 0.69134, 0.69176, 0.69263, 0.69393, 0.69559, 0.69747, 0.69930,\
        0.70120, 0.70183, 0.70231, 0.70308, 0.70503, 0.70960, 0.71612, 0.72412, 0.73332, 0.74332, 0.75361,\
        0.76359, 0.77251, 0.77945, 0.78352, 0.78467, 0.78376, 0.78182, 0.77959, 0.77758, 0.77616, 0.77560,\
        0.77602, 0.77745, 0.77989, 0.78344, 0.78817, 0.79368, 0.79929, 0.80376, 0.80512, 0.80468, 0.80362,\
        0.80266, 0.80246, 0.80378, 0.80548, 0.80681, 0.80868, 0.81059, 0.81191, 0.81304, 0.81401, 0.81437,\
        0.81383, 0.81243, 0.81022, 0.80724, 0.80405, 0.80102, 0.79866, 0.79705, 0.79540, 0.79355, 0.79158,\
        0.78968, 0.78804, 0.78665, 0.78544, 0.78440, 0.78350, 0.78263, 0.78164, 0.78033, 0.77850, 0.77590,\
        0.77227, 0.76753, 0.76173, 0.75511, 0.74792, 0.74045, 0.73295, 0.72563, 0.71873, 0.71253, 0.70724,\
        0.70252, 0.69814, 0.69416, 0.69057, 0.68710, 0.68338, 0.67909, 0.67397, 0.66792, 0.66102, 0.65341,\
        0.64528, 0.63682, 0.62826, 0.61980, 0.61163, 0.60386, 0.59678, 0.59048, 0.58459, 0.57886, 0.57318,\
        0.56752, 0.56187, 0.55622, 0.55059, 0.54497, 0.53934, 0.53370, 0.52801, 0.52226, 0.51642, 0.51045],\
        [0.69476, 0.69484, 0.69434, 0.69384, 0.69365, 0.69397, 0.69491, 0.69640, 0.69826, 0.70051, 0.70265,\
        0.70469, 0.70626, 0.70767, 0.70952, 0.71258, 0.71752, 0.72428, 0.73249, 0.74181, 0.75187, 0.76222,\
        0.77230, 0.78143, 0.78855, 0.79209, 0.79181, 0.78953, 0.78654, 0.78354, 0.78100, 0.77929, 0.77867,\
        0.77926, 0.78098, 0.78363, 0.78733, 0.79265, 0.79909, 0.80541, 0.81046, 0.81311, 0.81373, 0.81326,\
        0.81256, 0.81233, 0.81300, 0.81414, 0.81536, 0.81679, 0.81826, 0.81950, 0.82049, 0.82117, 0.82131,\
        0.82074, 0.81940, 0.81732, 0.81465, 0.81171, 0.80888, 0.80656, 0.80476, 0.80293, 0.80093, 0.79869,\
        0.79653, 0.79479, 0.79332, 0.79198, 0.79086, 0.79001, 0.78930, 0.78853, 0.78751, 0.78602, 0.78375,\
        0.78028, 0.77542, 0.76933, 0.76236, 0.75487, 0.74716, 0.73949, 0.73208, 0.72513, 0.71884, 0.71326,\
        0.70828, 0.70375, 0.69974, 0.69628, 0.69303, 0.68959, 0.68550, 0.68037, 0.67415, 0.66698, 0.65909,\
        0.65072, 0.64209, 0.63341, 0.62487, 0.61663, 0.60884, 0.60173, 0.59541, 0.58952, 0.58378, 0.57806,\
        0.57238, 0.56674, 0.56113, 0.55556, 0.55004, 0.54455, 0.53906, 0.53355, 0.52800, 0.52235, 0.51658],\
        [0.69828, 0.69771, 0.69672, 0.69595, 0.69560, 0.69583, 0.69688, 0.69875, 0.70081, 0.70332, 0.70593,\
        0.70842, 0.71077, 0.71318, 0.71610, 0.72006, 0.72547, 0.73240, 0.74062, 0.74985, 0.75972, 0.76980,\
        0.77960, 0.78850, 0.79566, 0.79876, 0.79692, 0.79372, 0.79019, 0.78682, 0.78404, 0.78226, 0.78180,\
        0.78285, 0.78532, 0.78855, 0.79218, 0.79804, 0.80590, 0.81326, 0.81904, 0.82258, 0.82403, 0.82409,\
        0.82358, 0.82316, 0.82322, 0.82371, 0.82447, 0.82543, 0.82647, 0.82743, 0.82820, 0.82865, 0.82864,\
        0.82804, 0.82678, 0.82489, 0.82251, 0.81987, 0.81724, 0.81485, 0.81270, 0.81055, 0.80826, 0.80580,\
        0.80342, 0.80146, 0.79974, 0.79813, 0.79676, 0.79587, 0.79520, 0.79452, 0.79369, 0.79257, 0.79075,\
        0.78754, 0.78253, 0.77608, 0.76877, 0.76103, 0.75317, 0.74541, 0.73796, 0.73098, 0.72460, 0.71886,\
        0.71372, 0.70915, 0.70517, 0.70175, 0.69864, 0.69553, 0.69167, 0.68649, 0.68002, 0.67253, 0.66436,\
        0.65578, 0.64703, 0.63829, 0.62974, 0.62154, 0.61384, 0.60679, 0.60041, 0.59446, 0.58860, 0.58278,\
        0.57704, 0.57137, 0.56578, 0.56028, 0.55486, 0.54950, 0.54418, 0.53887, 0.53352, 0.52809, 0.52254],\
        [0.70060, 0.69962, 0.69858, 0.69780, 0.69750, 0.69762, 0.69858, 0.70088, 0.70353, 0.70640, 0.70941,\
        0.71245, 0.71551, 0.71878, 0.72257, 0.72725, 0.73308, 0.74012, 0.74824, 0.75719, 0.76663, 0.77614,\
        0.78517, 0.79306, 0.79880, 0.80082, 0.79918, 0.79628, 0.79300, 0.78971, 0.78698, 0.78536, 0.78525,\
        0.78697, 0.79070, 0.79567, 0.79980, 0.80612, 0.81514, 0.82345, 0.82981, 0.83380, 0.83559, 0.83580,\
        0.83519, 0.83441, 0.83388, 0.83373, 0.83393, 0.83440, 0.83503, 0.83567, 0.83619, 0.83646, 0.83634,\
        0.83574, 0.83459, 0.83291, 0.83078, 0.82836, 0.82583, 0.82333, 0.82086, 0.81835, 0.81572, 0.81302,\
        0.81040, 0.80803, 0.80587, 0.80389, 0.80218, 0.80104, 0.80019, 0.79937, 0.79855, 0.79766, 0.79647,\
        0.79368, 0.78836, 0.78153, 0.77399, 0.76614, 0.75826, 0.75053, 0.74312, 0.73615, 0.72972, 0.72389,\
        0.71867, 0.71407, 0.71009, 0.70670, 0.70374, 0.70092, 0.69723, 0.69198, 0.68518, 0.67737, 0.66897,\
        0.66025, 0.65145, 0.64272, 0.63423, 0.62612, 0.61851, 0.61151, 0.60512, 0.59914, 0.59308, 0.58717,\
        0.58139, 0.57570, 0.57013, 0.56469, 0.55939, 0.55419, 0.54906, 0.54395, 0.53882, 0.53362, 0.52831],\
        [0.70191, 0.70134, 0.70040, 0.69982, 0.69983, 0.70002, 0.70101, 0.70344, 0.70655, 0.70985, 0.71328,\
        0.71680, 0.72048, 0.72443, 0.72889, 0.73410, 0.74022, 0.74730, 0.75523, 0.76379, 0.77265, 0.78139,\
        0.78941, 0.79601, 0.80033, 0.80161, 0.80026, 0.79841, 0.79573, 0.79274, 0.79033, 0.78906, 0.78950,\
        0.79191, 0.79662, 0.80383, 0.81062, 0.81847, 0.82764, 0.83619, 0.84268, 0.84660, 0.84817, 0.84804,\
        0.84698, 0.84566, 0.84452, 0.84377, 0.84344, 0.84347, 0.84374, 0.84410, 0.84442, 0.84455, 0.84439,\
        0.84382, 0.84280, 0.84130, 0.83937, 0.83710, 0.83460, 0.83196, 0.82920, 0.82632, 0.82333, 0.82028,\
        0.81729, 0.81445, 0.81179, 0.80934, 0.80720, 0.80548, 0.80408, 0.80286, 0.80179, 0.80083, 0.80007,\
        0.79778, 0.79209, 0.78519, 0.77773, 0.77003, 0.76232, 0.75476, 0.74748, 0.74058, 0.73417, 0.72828,\
        0.72297, 0.71826, 0.71419, 0.71073, 0.70775, 0.70492, 0.70148, 0.69619, 0.68911, 0.68113, 0.67266,\
        0.66395, 0.65520, 0.64656, 0.63818, 0.63017, 0.62264, 0.61564, 0.60915, 0.60296, 0.59689, 0.59113,\
        0.58535, 0.57963, 0.57409, 0.56875, 0.56359, 0.55858, 0.55367, 0.54880, 0.54392, 0.53897, 0.53390],\
        [0.70342, 0.70326, 0.70276, 0.70254, 0.70277, 0.70338, 0.70474, 0.70715, 0.71031, 0.71384, 0.71758,\
        0.72151, 0.72565, 0.73011, 0.73505, 0.74062, 0.74691, 0.75394, 0.76162, 0.76973, 0.77798, 0.78594,\
        0.79306, 0.79863, 0.80218, 0.80341, 0.80283, 0.80145, 0.79919, 0.79655, 0.79478, 0.79399, 0.79530,\
        0.79886, 0.80467, 0.81319, 0.82253, 0.83227, 0.84213, 0.85083, 0.85708, 0.86042, 0.86121, 0.86027,\
        0.85847, 0.85648, 0.85475, 0.85349, 0.85275, 0.85245, 0.85246, 0.85263, 0.85281, 0.85287, 0.85271,\
        0.85223, 0.85134, 0.85002, 0.84825, 0.84607, 0.84354, 0.84072, 0.83767, 0.83442, 0.83102, 0.82755,\
        0.82410, 0.82074, 0.81753, 0.81450, 0.81172, 0.80925, 0.80707, 0.80516, 0.80352, 0.80211, 0.80067,\
        0.79808, 0.79319, 0.78691, 0.77996, 0.77271, 0.76538, 0.75813, 0.75108, 0.74432, 0.73794, 0.73200,\
        0.72655, 0.72164, 0.71731, 0.71354, 0.71025, 0.70712, 0.70347, 0.69826, 0.69141, 0.68360, 0.67529,\
        0.66676, 0.65820, 0.64974, 0.64152, 0.63363, 0.62615, 0.61913, 0.61254, 0.60628, 0.60042, 0.59473,\
        0.58881, 0.58308, 0.57762, 0.57242, 0.56746, 0.56267, 0.55801, 0.55341, 0.54880, 0.54413, 0.53932],\
        [0.70522, 0.70552, 0.70557, 0.70577, 0.70632, 0.70736, 0.70906, 0.71156, 0.71473, 0.71837, 0.72232,\
        0.72653, 0.73101, 0.73582, 0.74107, 0.74685, 0.75321, 0.76013, 0.76753, 0.77520, 0.78284, 0.79016,\
        0.79671, 0.80170, 0.80492, 0.80641, 0.80646, 0.80552, 0.80379, 0.80197, 0.80113, 0.80093, 0.80334,\
        0.80828, 0.81542, 0.82470, 0.83530, 0.84641, 0.85717, 0.86620, 0.87205, 0.87433, 0.87386, 0.87177,\
        0.86906, 0.86641, 0.86419, 0.86259, 0.86161, 0.86116, 0.86106, 0.86115, 0.86129, 0.86135, 0.86125,\
        0.86089, 0.86016, 0.85900, 0.85736, 0.85523, 0.85263, 0.84961, 0.84624, 0.84260, 0.83876, 0.83482,\
        0.83086, 0.82695, 0.82313, 0.81944, 0.81591, 0.81257, 0.80945, 0.80659, 0.80407, 0.80196, 0.79981,\
        0.79684, 0.79257, 0.78715, 0.78098, 0.77440, 0.76761, 0.76078, 0.75404, 0.74747, 0.74115, 0.73514,\
        0.72950, 0.72427, 0.71949, 0.71518, 0.71130, 0.70766, 0.70375, 0.69871, 0.69223, 0.68481, 0.67689,\
        0.66872, 0.66047, 0.65228, 0.64426, 0.63650, 0.62908, 0.62204, 0.61539, 0.60914, 0.60329, 0.59737,\
        0.59155, 0.58596, 0.58067, 0.57570, 0.57099, 0.56648, 0.56211, 0.55781, 0.55350, 0.54911, 0.54459],\
        [0.70732, 0.70807, 0.70864, 0.70931, 0.71028, 0.71170, 0.71371, 0.71636, 0.71961, 0.72333, 0.72742,\
        0.73182, 0.73652, 0.74156, 0.74699, 0.75287, 0.75921, 0.76600, 0.77313, 0.78041, 0.78752, 0.79422,\
        0.80039, 0.80524, 0.80857, 0.81048, 0.81117, 0.81094, 0.81015, 0.80942, 0.80938, 0.81048, 0.81370,\
        0.81931, 0.82710, 0.83682, 0.84796, 0.85979, 0.87133, 0.88090, 0.88624, 0.88701, 0.88496, 0.88165,\
        0.87814, 0.87501, 0.87255, 0.87085, 0.86988, 0.86949, 0.86948, 0.86964, 0.86982, 0.86994, 0.86995,\
        0.86974, 0.86920, 0.86820, 0.86667, 0.86456, 0.86186, 0.85861, 0.85489, 0.85081, 0.84650, 0.84206,\
        0.83759, 0.83313, 0.82870, 0.82433, 0.82002, 0.81577, 0.81162, 0.80764, 0.80398, 0.80085, 0.79823,\
        0.79516, 0.79135, 0.78670, 0.78135, 0.77549, 0.76932, 0.76297, 0.75657, 0.75021, 0.74396, 0.73787,\
        0.73199, 0.72636, 0.72101, 0.71597, 0.71126, 0.70690, 0.70272, 0.69794, 0.69186, 0.68503, 0.67768,\
        0.67000, 0.66215, 0.65428, 0.64649, 0.63889, 0.63154, 0.62448, 0.61774, 0.61133, 0.60517, 0.59930,\
        0.59375, 0.58834, 0.58330, 0.57862, 0.57422, 0.57003, 0.56598, 0.56200, 0.55801, 0.55394, 0.54972],\
        [0.70965, 0.71084, 0.71190, 0.71305, 0.71445, 0.71625, 0.71855, 0.72140, 0.72477, 0.72859, 0.73279,\
        0.73732, 0.74217, 0.74734, 0.75286, 0.75875, 0.76503, 0.77164, 0.77855, 0.78559, 0.79240, 0.79854,\
        0.80436, 0.80936, 0.81307, 0.81556, 0.81699, 0.81764, 0.81788, 0.81820, 0.81913, 0.82122, 0.82504,\
        0.83090, 0.83874, 0.84835, 0.85932, 0.87105, 0.88270, 0.89283, 0.89758, 0.89654, 0.89316, 0.88902,\
        0.88516, 0.88198, 0.87964, 0.87817, 0.87749, 0.87744, 0.87773, 0.87810, 0.87839, 0.87860, 0.87876,\
        0.87876, 0.87841, 0.87757, 0.87614, 0.87403, 0.87119, 0.86767, 0.86355, 0.85901, 0.85421, 0.84928,\
        0.84432, 0.83934, 0.83437, 0.82938, 0.82433, 0.81923, 0.81408, 0.80896, 0.80408, 0.79985, 0.79661,\
        0.79366, 0.79031, 0.78631, 0.78165, 0.77645, 0.77085, 0.76497, 0.75892, 0.75277, 0.74660, 0.74043,\
        0.73431, 0.72825, 0.72228, 0.71642, 0.71073, 0.70538, 0.70069, 0.69605, 0.69077, 0.68471, 0.67800,\
        0.67085, 0.66343, 0.65590, 0.64837, 0.64094, 0.63366, 0.62658, 0.61975, 0.61318, 0.60695, 0.60117,\
        0.59548, 0.59029, 0.58560, 0.58127, 0.57722, 0.57337, 0.56966, 0.56602, 0.56237, 0.55863, 0.55472],\
        [0.71216, 0.71377, 0.71529, 0.71690, 0.71872, 0.72090, 0.72350, 0.72656, 0.73009, 0.73403, 0.73834,\
        0.74298, 0.74792, 0.75316, 0.75871, 0.76459, 0.77076, 0.77720, 0.78387, 0.79078, 0.79773, 0.80380,\
        0.80939, 0.81442, 0.81851, 0.82161, 0.82382, 0.82538, 0.82659, 0.82787, 0.82964, 0.83237, 0.83648,\
        0.84222, 0.84964, 0.85855, 0.86861, 0.87923, 0.88953, 0.89805, 0.90205, 0.90081, 0.89728, 0.89344,\
        0.89000, 0.88731, 0.88551, 0.88460, 0.88451, 0.88506, 0.88589, 0.88661, 0.88705, 0.88734, 0.88767,\
        0.88794, 0.88777, 0.88705, 0.88571, 0.88356, 0.88056, 0.87672, 0.87216, 0.86712, 0.86183, 0.85645,\
        0.85106, 0.84567, 0.84025, 0.83476, 0.82914, 0.82334, 0.81737, 0.81128, 0.80529, 0.79994, 0.79624,\
        0.79330, 0.79021, 0.78660, 0.78241, 0.77770, 0.77256, 0.76708, 0.76134, 0.75540, 0.74930, 0.74307,\
        0.73674, 0.73031, 0.72380, 0.71723, 0.71065, 0.70422, 0.69876, 0.69460, 0.68996, 0.68450, 0.67831,\
        0.67160, 0.66457, 0.65736, 0.65008, 0.64281, 0.63561, 0.62854, 0.62163, 0.61491, 0.60843, 0.60224,\
        0.59675, 0.59204, 0.58777, 0.58380, 0.58008, 0.57655, 0.57318, 0.56989, 0.56659, 0.56320, 0.55961],\
        [0.71480, 0.71680, 0.71876, 0.72079, 0.72302, 0.72556, 0.72847, 0.73177, 0.73549, 0.73957, 0.74400,\
        0.74874, 0.75375, 0.75903, 0.76460, 0.77044, 0.77654, 0.78284, 0.78922, 0.79576, 0.80278, 0.80926,\
        0.81509, 0.82032, 0.82482, 0.82852, 0.83149, 0.83389, 0.83598, 0.83804, 0.84043, 0.84349, 0.84756,\
        0.85285, 0.85943, 0.86718, 0.87577, 0.88460, 0.89271, 0.89873, 0.90157, 0.90074, 0.89826, 0.89552,\
        0.89306, 0.89134, 0.89048, 0.89041, 0.89114, 0.89251, 0.89410, 0.89530, 0.89592, 0.89626, 0.89674,\
        0.89735, 0.89722, 0.89656, 0.89528, 0.89302, 0.88984, 0.88567, 0.88060, 0.87503, 0.86929, 0.86354,\
        0.85783, 0.85215, 0.84645, 0.84063, 0.83463, 0.82839, 0.82189, 0.81520, 0.80856, 0.80255, 0.79818,\
        0.79483, 0.79159, 0.78800, 0.78400, 0.77958, 0.77475, 0.76957, 0.76408, 0.75832, 0.75230, 0.74605,\
        0.73956, 0.73287, 0.72600, 0.71898, 0.71193, 0.70513, 0.69934, 0.69488, 0.69029, 0.68496, 0.67898,\
        0.67254, 0.66579, 0.65885, 0.65179, 0.64469, 0.63759, 0.63055, 0.62361, 0.61681, 0.61021, 0.60395,\
        0.59850, 0.59409, 0.59009, 0.58636, 0.58288, 0.57964, 0.57658, 0.57364, 0.57070, 0.56766, 0.56441],\
        [0.71751, 0.71989, 0.72224, 0.72468, 0.72730, 0.73019, 0.73340, 0.73697, 0.74089, 0.74516, 0.74973,\
        0.75457, 0.75965, 0.76497, 0.77053, 0.77635, 0.78244, 0.78874, 0.79508, 0.80132, 0.80823, 0.81504,\
        0.82127, 0.82688, 0.83183, 0.83611, 0.83977, 0.84293, 0.84577, 0.84846, 0.85124, 0.85436, 0.85810,\
        0.86265, 0.86811, 0.87439, 0.88124, 0.88814, 0.89433, 0.89884, 0.90083, 0.90026, 0.89841, 0.89650,\
        0.89515, 0.89473, 0.89514, 0.89611, 0.89776, 0.90002, 0.90239, 0.90413, 0.90511, 0.90570, 0.90630,\
        0.90686, 0.90674, 0.90601, 0.90459, 0.90216, 0.89885, 0.89431, 0.88869, 0.88261, 0.87651, 0.87051,\
        0.86463, 0.85882, 0.85301, 0.84707, 0.84093, 0.83451, 0.82779, 0.82089, 0.81407, 0.80784, 0.80266,\
        0.79850, 0.79470, 0.79073, 0.78665, 0.78230, 0.77761, 0.77261, 0.76731, 0.76170, 0.75578, 0.74953,\
        0.74298, 0.73614, 0.72908, 0.72189, 0.71475, 0.70799, 0.70209, 0.69712, 0.69198, 0.68625, 0.68017,\
        0.67383, 0.66728, 0.66055, 0.65368, 0.64673, 0.63974, 0.63277, 0.62587, 0.61910, 0.61258, 0.60649,\
        0.60121, 0.59672, 0.59270, 0.58901, 0.58566, 0.58263, 0.57986, 0.57726, 0.57470, 0.57203, 0.56912],\
        [0.72026, 0.72298, 0.72570, 0.72852, 0.73151, 0.73474, 0.73826, 0.74210, 0.74626, 0.75072, 0.75546,\
        0.76043, 0.76561, 0.77096, 0.77652, 0.78232, 0.78843, 0.79486, 0.80146, 0.80793, 0.81468, 0.82150,\
        0.82799, 0.83397, 0.83936, 0.84418, 0.84845, 0.85228, 0.85574, 0.85893, 0.86193, 0.86490, 0.86808,\
        0.87169, 0.87586, 0.88055, 0.88561, 0.89069, 0.89530, 0.89880, 0.90035, 0.89978, 0.89838, 0.89732,\
        0.89716, 0.89824, 0.90028, 0.90244, 0.90512, 0.90812, 0.91097, 0.91319, 0.91468, 0.91564, 0.91630,\
        0.91662, 0.91634, 0.91536, 0.91357, 0.91086, 0.90717, 0.90226, 0.89623, 0.88976, 0.88349, 0.87740,\
        0.87148, 0.86570, 0.85994, 0.85408, 0.84801, 0.84165, 0.83497, 0.82808, 0.82122, 0.81476, 0.80901,\
        0.80401, 0.79923, 0.79472, 0.79044, 0.78597, 0.78125, 0.77632, 0.77115, 0.76568, 0.75985, 0.75364,\
        0.74707, 0.74018, 0.73304, 0.72581, 0.71867, 0.71193, 0.70589, 0.70056, 0.69439, 0.68810, 0.68186,\
        0.67557, 0.66915, 0.66258, 0.65587, 0.64905, 0.64218, 0.63529, 0.62847, 0.62180, 0.61541, 0.60949,\
        0.60425, 0.59967, 0.59545, 0.59167, 0.58836, 0.58550, 0.58301, 0.58075, 0.57858, 0.57631, 0.57375],\
        [0.72300, 0.72604, 0.72911, 0.73227, 0.73560, 0.73916, 0.74299, 0.74711, 0.75153, 0.75622, 0.76116,\
        0.76630, 0.77160, 0.77703, 0.78258, 0.78837, 0.79450, 0.80101, 0.80781, 0.81464, 0.82155, 0.82847,\
        0.83517, 0.84146, 0.84724, 0.85251, 0.85730, 0.86168, 0.86570, 0.86930, 0.87242, 0.87508, 0.87756,\
        0.88014, 0.88298, 0.88609, 0.88945, 0.89285, 0.89604, 0.89873, 0.90017, 0.89948, 0.89867, 0.89894,\
        0.90032, 0.90293, 0.90637, 0.90990, 0.91351, 0.91706, 0.92028, 0.92286, 0.92470, 0.92590, 0.92653,\
        0.92661, 0.92601, 0.92464, 0.92241, 0.91921, 0.91497, 0.90962, 0.90325, 0.89665, 0.89041, 0.88424,\
        0.87838, 0.87276, 0.86721, 0.86159, 0.85577, 0.84965, 0.84317, 0.83638, 0.82949, 0.82279, 0.81649,\
        0.81064, 0.80511, 0.80006, 0.79540, 0.79058, 0.78565, 0.78071, 0.77566, 0.77031, 0.76458, 0.75842,\
        0.75186, 0.74495, 0.73779, 0.73051, 0.72329, 0.71634, 0.70975, 0.70330, 0.69674, 0.69030, 0.68403,\
        0.67783, 0.67153, 0.66507, 0.65847, 0.65176, 0.64497, 0.63817, 0.63143, 0.62484, 0.61852, 0.61264,\
        0.60730, 0.60249, 0.59809, 0.59423, 0.59094, 0.58822, 0.58599, 0.58410, 0.58235, 0.58049, 0.57832],\
        [0.72570, 0.72903, 0.73241, 0.73590, 0.73955, 0.74343, 0.74756, 0.75196, 0.75665, 0.76159, 0.76677,\
        0.77212, 0.77761, 0.78316, 0.78874, 0.79449, 0.80071, 0.80736, 0.81431, 0.82140, 0.82855, 0.83567,\
        0.84260, 0.84918, 0.85530, 0.86091, 0.86608, 0.87090, 0.87541, 0.87944, 0.88266, 0.88492, 0.88658,\
        0.88821, 0.88984, 0.89149, 0.89340, 0.89532, 0.89709, 0.89881, 0.89997, 0.89956, 0.90042, 0.90264,\
        0.90578, 0.90971, 0.91413, 0.91860, 0.92291, 0.92691, 0.93040, 0.93319, 0.93520, 0.93645, 0.93697,\
        0.93677, 0.93579, 0.93396, 0.93122, 0.92750, 0.92277, 0.91705, 0.91057, 0.90395, 0.89746, 0.89106,\
        0.88538, 0.88001, 0.87472, 0.86944, 0.86402, 0.85829, 0.85209, 0.84543, 0.83849, 0.83150, 0.82472,\
        0.81825, 0.81216, 0.80655, 0.80130, 0.79588, 0.79067, 0.78577, 0.78085, 0.77563, 0.76997, 0.76385,\
        0.75729, 0.75038, 0.74319, 0.73584, 0.72845, 0.72114, 0.71393, 0.70676, 0.69969, 0.69302, 0.68685,\
        0.68075, 0.67451, 0.66809, 0.66154, 0.65488, 0.64816, 0.64142, 0.63472, 0.62816, 0.62183, 0.61585,\
        0.61031, 0.60524, 0.60065, 0.59667, 0.59336, 0.59074, 0.58878, 0.58727, 0.58597, 0.58458, 0.58279],\
        [0.72833, 0.73193, 0.73559, 0.73937, 0.74332, 0.74749, 0.75191, 0.75660, 0.76156, 0.76677, 0.77220,\
        0.77781, 0.78354, 0.78932, 0.79502, 0.80080, 0.80708, 0.81384, 0.82090, 0.82816, 0.83552, 0.84288,\
        0.85009, 0.85696, 0.86336, 0.86921, 0.87458, 0.87965, 0.88456, 0.88906, 0.89251, 0.89432, 0.89512,\
        0.89606, 0.89686, 0.89718, 0.89818, 0.89908, 0.89967, 0.90078, 0.90177, 0.90299, 0.90549, 0.90913,\
        0.91351, 0.91840, 0.92353, 0.92859, 0.93335, 0.93763, 0.94127, 0.94413, 0.94614, 0.94730, 0.94761,\
        0.94707, 0.94567, 0.94336, 0.94012, 0.93592, 0.93077, 0.92478, 0.91819, 0.91135, 0.90441, 0.89798,\
        0.89264, 0.88742, 0.88231, 0.87741, 0.87252, 0.86729, 0.86144, 0.85490, 0.84784, 0.84056, 0.83333,\
        0.82635, 0.81974, 0.81352, 0.80756, 0.80168, 0.79626, 0.79153, 0.78674, 0.78158, 0.77595, 0.76983,\
        0.76327, 0.75635, 0.74913, 0.74170, 0.73414, 0.72651, 0.71884, 0.71114, 0.70354, 0.69670, 0.69065,\
        0.68451, 0.67817, 0.67168, 0.66509, 0.65844, 0.65175, 0.64503, 0.63834, 0.63175, 0.62533, 0.61920,\
        0.61342, 0.60807, 0.60323, 0.59904, 0.59560, 0.59305, 0.59134, 0.59024, 0.58943, 0.58854, 0.58717],\
        [0.73086, 0.73470, 0.73862, 0.74266, 0.74688, 0.75132, 0.75601, 0.76097, 0.76620, 0.77169, 0.77738,\
        0.78325, 0.78923, 0.79531, 0.80128, 0.80717, 0.81348, 0.82025, 0.82738, 0.83477, 0.84233, 0.84994,\
        0.85744, 0.86464, 0.87132, 0.87731, 0.88266, 0.88769, 0.89279, 0.89766, 0.90136, 0.90294, 0.90332,\
        0.90382, 0.90412, 0.90393, 0.90435, 0.90484, 0.90535, 0.90640, 0.90797, 0.91030, 0.91371, 0.91813,\
        0.92324, 0.92873, 0.93432, 0.93973, 0.94472, 0.94912, 0.95277, 0.95556, 0.95744, 0.95838, 0.95840,\
        0.95748, 0.95563, 0.95285, 0.94913, 0.94449, 0.93899, 0.93275, 0.92599, 0.91897, 0.91201, 0.90568,\
        0.90017, 0.89472, 0.88964, 0.88518, 0.88096, 0.87638, 0.87091, 0.86442, 0.85716, 0.84955, 0.84193,\
        0.83452, 0.82746, 0.82076, 0.81436, 0.80827, 0.80277, 0.79807, 0.79319, 0.78802, 0.78237, 0.77620,\
        0.76963, 0.76272, 0.75550, 0.74802, 0.74033, 0.73249, 0.72457, 0.71666, 0.70898, 0.70205, 0.69561,\
        0.68909, 0.68245, 0.67577, 0.66909, 0.66243, 0.65573, 0.64901, 0.64228, 0.63561, 0.62906, 0.62273,\
        0.61669, 0.61103, 0.60588, 0.60137, 0.59770, 0.59516, 0.59369, 0.59299, 0.59268, 0.59232, 0.59139],\
        [0.73327, 0.73733, 0.74147, 0.74574, 0.75021, 0.75489, 0.75983, 0.76504, 0.77052, 0.77626, 0.78223,\
        0.78833, 0.79450, 0.80074, 0.80695, 0.81312, 0.81957, 0.82639, 0.83358, 0.84107, 0.84879, 0.85663,\
        0.86446, 0.87203, 0.87905, 0.88521, 0.89038, 0.89503, 0.90004, 0.90492, 0.90855, 0.91050, 0.91129,\
        0.91167, 0.91180, 0.91178, 0.91199, 0.91247, 0.91328, 0.91469, 0.91686, 0.91993, 0.92398, 0.92892,\
        0.93447, 0.94032, 0.94616, 0.95174, 0.95680, 0.96117, 0.96471, 0.96732, 0.96895, 0.96959, 0.96924,\
        0.96791, 0.96561, 0.96237, 0.95820, 0.95317, 0.94734, 0.94086, 0.93393, 0.92681, 0.91980, 0.91326,\
        0.90725, 0.90149, 0.89644, 0.89240, 0.88898, 0.88519, 0.88011, 0.87354, 0.86600, 0.85805, 0.85011,\
        0.84240, 0.83504, 0.82806, 0.82148, 0.81536, 0.80984, 0.80483, 0.79984, 0.79472, 0.78898, 0.78272,\
        0.77619, 0.76936, 0.76218, 0.75469, 0.74695, 0.73902, 0.73100, 0.72302, 0.71531, 0.70808, 0.70117,\
        0.69420, 0.68715, 0.68023, 0.67347, 0.66679, 0.66009, 0.65334, 0.64654, 0.63975, 0.63303, 0.62648,\
        0.62017, 0.61421, 0.60872, 0.60383, 0.59985, 0.59725, 0.59591, 0.59552, 0.59562, 0.59579, 0.59534],\
        [0.73555, 0.73979, 0.74413, 0.74861, 0.75328, 0.75817, 0.76332, 0.76875, 0.77446, 0.78044, 0.78666,\
        0.79303, 0.79936, 0.80570, 0.81209, 0.81853, 0.82516, 0.83209, 0.83933, 0.84689, 0.85471, 0.86276,\
        0.87089, 0.87889, 0.88637, 0.89286, 0.89802, 0.90229, 0.90704, 0.91165, 0.91526, 0.91758, 0.91887,\
        0.91954, 0.91988, 0.92013, 0.92054, 0.92129, 0.92253, 0.92444, 0.92718, 0.93084, 0.93541, 0.94075,\
        0.94660, 0.95267, 0.95864, 0.96425, 0.96927, 0.97350, 0.97683, 0.97917, 0.98048, 0.98074, 0.97998,\
        0.97822, 0.97548, 0.97180, 0.96723, 0.96183, 0.95570, 0.94899, 0.94187, 0.93458, 0.92738, 0.92048,\
        0.91399, 0.90797, 0.90277, 0.89875, 0.89607, 0.89316, 0.88848, 0.88165, 0.87377, 0.86559, 0.85749,\
        0.84966, 0.84219, 0.83511, 0.82847, 0.82232, 0.81667, 0.81143, 0.80637, 0.80129, 0.79528, 0.78906,\
        0.78276, 0.77610, 0.76902, 0.76158, 0.75385, 0.74590, 0.73784, 0.72978, 0.72189, 0.71427, 0.70685,\
        0.69941, 0.69200, 0.68493, 0.67816, 0.67150, 0.66480, 0.65799, 0.65109, 0.64415, 0.63725, 0.63047,\
        0.62390, 0.61767, 0.61190, 0.60676, 0.60253, 0.59959, 0.59813, 0.59792, 0.59811, 0.59857, 0.59875],\
        [0.73770, 0.74210, 0.74660, 0.75125, 0.75608, 0.76115, 0.76649, 0.77210, 0.77799, 0.78416, 0.79057,\
        0.79720, 0.80376, 0.81024, 0.81680, 0.82345, 0.83027, 0.83729, 0.84458, 0.85214, 0.85998, 0.86811,\
        0.87646, 0.88484, 0.89282, 0.89970, 0.90505, 0.90954, 0.91403, 0.91833, 0.92191, 0.92453, 0.92626,\
        0.92735, 0.92807, 0.92870, 0.92948, 0.93062, 0.93232, 0.93475, 0.93803, 0.94220, 0.94722, 0.95290,\
        0.95901, 0.96524, 0.97128, 0.97686, 0.98176, 0.98579, 0.98884, 0.99084, 0.99176, 0.99162, 0.99044,\
        0.98825, 0.98509, 0.98101, 0.97607, 0.97034, 0.96394, 0.95698, 0.94965, 0.94213, 0.93465, 0.92738,\
        0.92047, 0.91411, 0.90857, 0.90418, 0.90109, 0.89920, 0.89500, 0.88783, 0.87986, 0.87176, 0.86378,\
        0.85606, 0.84867, 0.84164, 0.83501, 0.82881, 0.82304, 0.81760, 0.81232, 0.80687, 0.80090, 0.79514,\
        0.78927, 0.78281, 0.77586, 0.76851, 0.76085, 0.75293, 0.74484, 0.73667, 0.72853, 0.72049, 0.71254,\
        0.70462, 0.69698, 0.68981, 0.68313, 0.67656, 0.66984, 0.66292, 0.65588, 0.64877, 0.64167, 0.63466,\
        0.62786, 0.62139, 0.61540, 0.61009, 0.60568, 0.60243, 0.60063, 0.60033, 0.60021, 0.60025, 0.60060],\
        [0.73971, 0.74424, 0.74887, 0.75365, 0.75863, 0.76384, 0.76931, 0.77507, 0.78112, 0.78743, 0.79394,\
        0.80066, 0.80749, 0.81426, 0.82107, 0.82794, 0.83492, 0.84204, 0.84932, 0.85682, 0.86456, 0.87260,\
        0.88095, 0.88952, 0.89787, 0.90497, 0.91078, 0.91584, 0.92053, 0.92485, 0.92853, 0.93142, 0.93353,\
        0.93503, 0.93616, 0.93718, 0.93835, 0.93989, 0.94202, 0.94491, 0.94866, 0.95329, 0.95871, 0.96473,\
        0.97109, 0.97747, 0.98357, 0.98910, 0.99384, 0.99762, 1.00035, 1.00198, 1.00250, 1.00195, 1.00035,\
        0.99776, 0.99422, 0.98980, 0.98455, 0.97855, 0.97189, 0.96471, 0.95715, 0.94938, 0.94157, 0.93392,\
        0.92658, 0.91976, 0.91369, 0.90856, 0.90446, 0.90186, 0.89780, 0.89129, 0.88394, 0.87638, 0.86887,\
        0.86151, 0.85436, 0.84748, 0.84091, 0.83469, 0.82880, 0.82319, 0.81771, 0.81219, 0.80660, 0.80126,\
        0.79559, 0.78928, 0.78245, 0.77526, 0.76775, 0.75993, 0.75184, 0.74356, 0.73518, 0.72678, 0.71840,\
        0.71014, 0.70216, 0.69487, 0.68847, 0.68199, 0.67514, 0.66804, 0.66082, 0.65353, 0.64622, 0.63899,\
        0.63195, 0.62526, 0.61909, 0.61363, 0.60909, 0.60569, 0.60361, 0.60265, 0.60216, 0.60189, 0.60151],\
        [0.74158, 0.74621, 0.75096, 0.75585, 0.76092, 0.76623, 0.77181, 0.77769, 0.78388, 0.79033, 0.79698,\
        0.80381, 0.81083, 0.81788, 0.82496, 0.83207, 0.83922, 0.84642, 0.85368, 0.86103, 0.86853, 0.87628,\
        0.88437, 0.89278, 0.90120, 0.90878, 0.91539, 0.92123, 0.92646, 0.93110, 0.93503, 0.93821, 0.94065,\
        0.94250, 0.94396, 0.94529, 0.94675, 0.94860, 0.95105, 0.95430, 0.95843, 0.96344, 0.96924, 0.97560,\
        0.98223, 0.98880, 0.99497, 1.00046, 1.00504, 1.00857, 1.01097, 1.01222, 1.01234, 1.01139, 1.00942,\
        1.00649, 1.00266, 0.99797, 0.99249, 0.98628, 0.97944, 0.97206, 0.96428, 0.95625, 0.94812, 0.94007,\
        0.93227, 0.92489, 0.91814, 0.91219, 0.90717, 0.90291, 0.89829, 0.89265, 0.88631, 0.87965, 0.87286,\
        0.86603, 0.85925, 0.85259, 0.84610, 0.83986, 0.83388, 0.82814, 0.82258, 0.81714, 0.81184, 0.80664,\
        0.80121, 0.79514, 0.78853, 0.78163, 0.77439, 0.76674, 0.75869, 0.75031, 0.74174, 0.73307, 0.72442,\
        0.71593, 0.70779, 0.70057, 0.69437, 0.68769, 0.68047, 0.67317, 0.66581, 0.65834, 0.65081, 0.64333,\
        0.63604, 0.62911, 0.62274, 0.61711, 0.61243, 0.60884, 0.60638, 0.60488, 0.60395, 0.60324, 0.60234],\
        [0.74332, 0.74805, 0.75287, 0.75784, 0.76299, 0.76837, 0.77401, 0.77996, 0.78626, 0.79290, 0.79977,\
        0.80674, 0.81390, 0.82120, 0.82856, 0.83593, 0.84328, 0.85058, 0.85781, 0.86498, 0.87216, 0.87949,\
        0.88713, 0.89518, 0.90362, 0.91191, 0.91944, 0.92613, 0.93202, 0.93712, 0.94140, 0.94486, 0.94756,\
        0.94964, 0.95129, 0.95277, 0.95435, 0.95632, 0.95892, 0.96236, 0.96673, 0.97205, 0.97818, 0.98489,\
        0.99183, 0.99863, 1.00492, 1.01040, 1.01484, 1.01813, 1.02021, 1.02110, 1.02088, 1.01960, 1.01734,\
        1.01417, 1.01014, 1.00530, 0.99970, 0.99339, 0.98644, 0.97893, 0.97098, 0.96272, 0.95428, 0.94584,\
        0.93754, 0.92956, 0.92206, 0.91518, 0.90899, 0.90343, 0.89824, 0.89310, 0.88774, 0.88205, 0.87606,\
        0.86984, 0.86345, 0.85700, 0.85058, 0.84430, 0.83822, 0.83237, 0.82676, 0.82136, 0.81616, 0.81105,\
        0.80578, 0.80003, 0.79385, 0.78747, 0.78067, 0.77326, 0.76526, 0.75681, 0.74808, 0.73923, 0.73042,\
        0.72185, 0.71380, 0.70665, 0.70038, 0.69290, 0.68541, 0.67812, 0.67071, 0.66310, 0.65534, 0.64756,\
        0.63997, 0.63275, 0.62612, 0.62028, 0.61539, 0.61155, 0.60875, 0.60681, 0.60543, 0.60429, 0.60300],\
        [0.74495, 0.74975, 0.75464, 0.75967, 0.76487, 0.77028, 0.77594, 0.78191, 0.78824, 0.79498, 0.80207,\
        0.80927, 0.81669, 0.82428, 0.83194, 0.83960, 0.84720, 0.85466, 0.86192, 0.86895, 0.87583, 0.88270,\
        0.88988, 0.89775, 0.90641, 0.91525, 0.92354, 0.93098, 0.93750, 0.94306, 0.94768, 0.95137, 0.95421,\
        0.95634, 0.95799, 0.95939, 0.96086, 0.96270, 0.96519, 0.96857, 0.97300, 0.97848, 0.98489, 0.99194,\
        0.99924, 1.00633, 1.01280, 1.01831, 1.02266, 1.02573, 1.02754, 1.02815, 1.02765, 1.02615, 1.02373,\
        1.02047, 1.01641, 1.01159, 1.00601, 0.99973, 0.99279, 0.98524, 0.97719, 0.96875, 0.96005, 0.95124,\
        0.94247, 0.93389, 0.92566, 0.91788, 0.91063, 0.90397, 0.89825, 0.89356, 0.88899, 0.88414, 0.87887,\
        0.87317, 0.86710, 0.86079, 0.85437, 0.84799, 0.84177, 0.83579, 0.83010, 0.82469, 0.81953, 0.81451,\
        0.80940, 0.80402, 0.79846, 0.79279, 0.78653, 0.77937, 0.77141, 0.76289, 0.75403, 0.74503, 0.73609,\
        0.72743, 0.71924, 0.71163, 0.70414, 0.69659, 0.68985, 0.68290, 0.67551, 0.66772, 0.65967, 0.65154,\
        0.64356, 0.63598, 0.62902, 0.62290, 0.61776, 0.61365, 0.61054, 0.60825, 0.60652, 0.60503, 0.60346],\
        [0.74648, 0.75134, 0.75629, 0.76137, 0.76660, 0.77203, 0.77770, 0.78364, 0.78991, 0.79662, 0.80383,\
        0.81142, 0.81921, 0.82714, 0.83515, 0.84316, 0.85107, 0.85879, 0.86620, 0.87322, 0.87989, 0.88644,\
        0.89319, 0.90099, 0.91007, 0.91935, 0.92816, 0.93615, 0.94315, 0.94908, 0.95393, 0.95773, 0.96055,\
        0.96255, 0.96394, 0.96499, 0.96603, 0.96739, 0.96943, 0.97246, 0.97667, 0.98213, 0.98872, 0.99609,\
        1.00376, 1.01121, 1.01792, 1.02353, 1.02783, 1.03075, 1.03236, 1.03278, 1.03215, 1.03060, 1.02822,\
        1.02509, 1.02121, 1.01661, 1.01126, 1.00519, 0.99840, 0.99094, 0.98289, 0.97435, 0.96544, 0.95632,\
        0.94713, 0.93803, 0.92917, 0.92067, 0.91268, 0.90539, 0.89931, 0.89470, 0.89062, 0.88636, 0.88161,\
        0.87626, 0.87036, 0.86404, 0.85751, 0.85094, 0.84451, 0.83834, 0.83252, 0.82706, 0.82195, 0.81707,\
        0.81228, 0.80746, 0.80260, 0.79766, 0.79191, 0.78487, 0.77690, 0.76832, 0.75935, 0.75022, 0.74115,\
        0.73236, 0.72397, 0.71595, 0.70814, 0.70103, 0.69451, 0.68768, 0.68019, 0.67212, 0.66368, 0.65509,\
        0.64663, 0.63860, 0.63124, 0.62479, 0.61936, 0.61499, 0.61163, 0.60909, 0.60712, 0.60541, 0.60365],\
        [0.74793, 0.75284, 0.75784, 0.76297, 0.76824, 0.77370, 0.77938, 0.78531, 0.79154, 0.79821, 0.80560,\
        0.81347, 0.82159, 0.82986, 0.83822, 0.84660, 0.85491, 0.86301, 0.87076, 0.87796, 0.88460, 0.89109,\
        0.89799, 0.90574, 0.91496, 0.92445, 0.93354, 0.94185, 0.94914, 0.95528, 0.96022, 0.96397, 0.96658,\
        0.96821, 0.96906, 0.96943, 0.96966, 0.97015, 0.97131, 0.97356, 0.97719, 0.98235, 0.98893, 0.99656,\
        1.00464, 1.01248, 1.01949, 1.02525, 1.02957, 1.03244, 1.03397, 1.03437, 1.03383, 1.03248, 1.03042,\
        1.02769, 1.02429, 1.02018, 1.01532, 1.00967, 1.00322, 0.99599, 0.98806, 0.97952, 0.97049, 0.96113,\
        0.95160, 0.94208, 0.93273, 0.92374, 0.91531, 0.90769, 0.90139, 0.89684, 0.89286, 0.88894, 0.88448,\
        0.87927, 0.87331, 0.86681, 0.85999, 0.85311, 0.84637, 0.83995, 0.83395, 0.82843, 0.82336, 0.81868,\
        0.81429, 0.81006, 0.80594, 0.80172, 0.79639, 0.78935, 0.78148, 0.77291, 0.76385, 0.75458, 0.74537,\
        0.73647, 0.72800, 0.72003, 0.71257, 0.70576, 0.69929, 0.69240, 0.68459, 0.67613, 0.66721, 0.65806,\
        0.64901, 0.64042, 0.63259, 0.62576, 0.62004, 0.61546, 0.61193, 0.60927, 0.60719, 0.60538, 0.60352],\
        [0.74930, 0.75427, 0.75932, 0.76450, 0.76982, 0.77533, 0.78107, 0.78706, 0.79337, 0.80013, 0.80760,\
        0.81560, 0.82392, 0.83245, 0.84112, 0.84987, 0.85862, 0.86724, 0.87554, 0.88323, 0.88998, 0.89638,\
        0.90376, 0.91197, 0.92107, 0.93051, 0.93967, 0.94811, 0.95553, 0.96173, 0.96660, 0.97012, 0.97231,\
        0.97331, 0.97332, 0.97264, 0.97164, 0.97077, 0.97055, 0.97148, 0.97403, 0.97847, 0.98477, 0.99252,\
        1.00096, 1.00920, 1.01654, 1.02252, 1.02696, 1.02991, 1.03158, 1.03223, 1.03209, 1.03130, 1.02994,\
        1.02801, 1.02545, 1.02217, 1.01809, 1.01312, 1.00722, 1.00040, 0.99272, 0.98428, 0.97521, 0.96569,\
        0.95591, 0.94606, 0.93637, 0.92706, 0.91838, 0.91067, 0.90431, 0.89931, 0.89550, 0.89192, 0.88755,\
        0.88223, 0.87598, 0.86907, 0.86179, 0.85446, 0.84733, 0.84058, 0.83436, 0.82874, 0.82372, 0.81925,\
        0.81523, 0.81150, 0.80783, 0.80377, 0.79860, 0.79246, 0.78502, 0.77653, 0.76735, 0.75790, 0.74854,\
        0.73955, 0.73111, 0.72334, 0.71624, 0.70972, 0.70344, 0.69636, 0.68832, 0.67955, 0.67009, 0.66026,\
        0.65050, 0.64126, 0.63290, 0.62567, 0.61969, 0.61496, 0.61137, 0.60870, 0.60666, 0.60490, 0.60306],\
        [0.75060, 0.75563, 0.76075, 0.76598, 0.77137, 0.77695, 0.78277, 0.78889, 0.79538, 0.80231, 0.80979,\
        0.81779, 0.82619, 0.83488, 0.84379, 0.85287, 0.86204, 0.87121, 0.88020, 0.88875, 0.89632, 0.90209,\
        0.90994, 0.91859, 0.92780, 0.93722, 0.94639, 0.95486, 0.96229, 0.96841, 0.97308, 0.97619, 0.97776,\
        0.97788, 0.97674, 0.97462, 0.97194, 0.96918, 0.96696, 0.96593, 0.96674, 0.96986, 0.97542, 0.98302,\
        0.99167, 1.00019, 1.00779, 1.01408, 1.01883, 1.02211, 1.02424, 1.02558, 1.02633, 1.02661, 1.02644,\
        1.02578, 1.02453, 1.02250, 1.01956, 1.01556, 1.01045, 1.00421, 0.99691, 0.98867, 0.97965, 0.97003,\
        0.96005, 0.94995, 0.93998, 0.93043, 0.92159, 0.91375, 0.90712, 0.90181, 0.89876, 0.89514, 0.89069,\
        0.88504, 0.87825, 0.87071, 0.86283, 0.85495, 0.84733, 0.84020, 0.83372, 0.82798, 0.82299, 0.81872,\
        0.81502, 0.81172, 0.80850, 0.80484, 0.80046, 0.79487, 0.78768, 0.77913, 0.76973, 0.76001, 0.75043,\
        0.74134, 0.73296, 0.72539, 0.71861, 0.71234, 0.70603, 0.69904, 0.69123, 0.68228, 0.67218, 0.66150,\
        0.65090, 0.64094, 0.63203, 0.62442, 0.61824, 0.61344, 0.60988, 0.60733, 0.60549, 0.60395, 0.60221],\
        [0.75184, 0.75693, 0.76212, 0.76742, 0.77288, 0.77853, 0.78444, 0.79068, 0.79733, 0.80438, 0.81189,\
        0.81989, 0.82831, 0.83708, 0.84614, 0.85544, 0.86496, 0.87459, 0.88418, 0.89342, 0.90182, 0.90889,\
        0.91658, 0.92536, 0.93471, 0.94422, 0.95344, 0.96193, 0.96931, 0.97529, 0.97964, 0.98222, 0.98297,\
        0.98196, 0.97936, 0.97544, 0.97061, 0.96543, 0.96056, 0.95681, 0.95506, 0.95605, 0.96008, 0.96694,\
        0.97545, 0.98382, 0.99144, 0.99823, 1.00369, 1.00773, 1.01092, 1.01364, 1.01600, 1.01801, 1.01967,\
        1.02089, 1.02148, 1.02120, 1.01979, 1.01708, 1.01298, 1.00750, 1.00070, 0.99275, 0.98382, 0.97416,\
        0.96402, 0.95368, 0.94346, 0.93366, 0.92464, 0.91669, 0.91011, 0.90515, 0.90154, 0.89805, 0.89368,\
        0.88746, 0.87987, 0.87156, 0.86299, 0.85450, 0.84636, 0.83882, 0.83204, 0.82615, 0.82118, 0.81708,\
        0.81369, 0.81083, 0.80825, 0.80551, 0.80193, 0.79673, 0.78952, 0.78062, 0.77081, 0.76072, 0.75086,\
        0.74162, 0.73325, 0.72589, 0.71944, 0.71359, 0.70772, 0.70112, 0.69368, 0.68436, 0.67329, 0.66154,\
        0.64999, 0.63928, 0.62985, 0.62194, 0.61564, 0.61088, 0.60747, 0.60514, 0.60360, 0.60247, 0.60093],\
        [0.75302, 0.75818, 0.76343, 0.76881, 0.77434, 0.78007, 0.78605, 0.79234, 0.79903, 0.80619, 0.81378,\
        0.82180, 0.83022, 0.83900, 0.84810, 0.85750, 0.86718, 0.87710, 0.88714, 0.89689, 0.90577, 0.91405,\
        0.92257, 0.93177, 0.94144, 0.95118, 0.96056, 0.96913, 0.97650, 0.98230, 0.98627, 0.98820, 0.98798,\
        0.98564, 0.98129, 0.97521, 0.96781, 0.95967, 0.95153, 0.94432, 0.93914, 0.93704, 0.93826, 0.94303,\
        0.95057, 0.95760, 0.96458, 0.97262, 0.97965, 0.98522, 0.99048, 0.99573, 1.00068, 1.00527, 1.00952,\
        1.01334, 1.01642, 1.01840, 1.01895, 1.01783, 1.01497, 1.01038, 1.00419, 0.99658, 0.98779, 0.97809,\
        0.96778, 0.95719, 0.94666, 0.93656, 0.92725, 0.91911, 0.91246, 0.90744, 0.90378, 0.90072, 0.89615,\
        0.88889, 0.88039, 0.87135, 0.86215, 0.85309, 0.84445, 0.83648, 0.82938, 0.82333, 0.81838, 0.81444,\
        0.81132, 0.80883, 0.80682, 0.80506, 0.80270, 0.79800, 0.79022, 0.78072, 0.77042, 0.75992, 0.74972,\
        0.74025, 0.73183, 0.72462, 0.71855, 0.71330, 0.70841, 0.70320, 0.69609, 0.68541, 0.67294, 0.66004,\
        0.64758, 0.63619, 0.62633, 0.61825, 0.61198, 0.60739, 0.60425, 0.60219, 0.60090, 0.60030, 0.59904],\
        [0.75414, 0.75936, 0.76469, 0.77014, 0.77576, 0.78156, 0.78759, 0.79393, 0.80066, 0.80787, 0.81550,\
        0.82352, 0.83190, 0.84062, 0.84965, 0.85900, 0.86865, 0.87862, 0.88885, 0.89905, 0.90857, 0.91786,\
        0.92744, 0.93744, 0.94769, 0.95787, 0.96755, 0.97630, 0.98370, 0.98936, 0.99294, 0.99416, 0.99285,\
        0.98898, 0.98264, 0.97408, 0.96372, 0.95215, 0.94022, 0.92898, 0.91969, 0.91399, 0.91097, 0.90993,\
        0.91409, 0.91725, 0.92242, 0.93395, 0.94429, 0.95274, 0.96203, 0.97153, 0.98028, 0.98840, 0.99610,\
        1.00333, 1.00962, 1.01444, 1.01735, 1.01810, 1.01663, 1.01304, 1.00749, 1.00024, 0.99158, 0.98183,\
        0.97133, 0.96044, 0.94953, 0.93899, 0.92923, 0.92065, 0.91359, 0.90823, 0.90428, 0.90104, 0.89616,\
        0.88833, 0.87939, 0.86993, 0.86031, 0.85081, 0.84172, 0.83332, 0.82589, 0.81967, 0.81475, 0.81101,\
        0.80814, 0.80582, 0.80403, 0.80282, 0.80138, 0.79696, 0.78887, 0.77907, 0.76844, 0.75760, 0.74705,\
        0.73727, 0.72868, 0.72153, 0.71575, 0.71095, 0.70670, 0.70250, 0.69559, 0.68376, 0.67038, 0.65670,\
        0.64357, 0.63167, 0.62156, 0.61350, 0.60743, 0.60313, 0.60040, 0.59871, 0.59769, 0.59713, 0.59616],\
        [0.75517, 0.76046, 0.76586, 0.77141, 0.77711, 0.78300, 0.78910, 0.79548, 0.80225, 0.80948, 0.81709,\
        0.82506, 0.83337, 0.84198, 0.85087, 0.86004, 0.86949, 0.87923, 0.88930, 0.89965, 0.90997, 0.92050,\
        0.93129, 0.94230, 0.95334, 0.96412, 0.97424, 0.98329, 0.99082, 0.99640, 0.99961, 1.00011, 0.99764,\
        0.99208, 0.98352, 0.97220, 0.95851, 0.94301, 0.92679, 0.91128, 0.89829, 0.88960, 0.88171, 0.86906,\
        0.85636, 0.85588, 0.85970, 0.87534, 0.89457, 0.90889, 0.92584, 0.94130, 0.95513, 0.96771, 0.97979,\
        0.99137, 1.00165, 1.00984, 1.01543, 1.01821, 1.01822, 1.01564, 1.01073, 1.00382, 0.99526, 0.98543,\
        0.97469, 0.96344, 0.95206, 0.94095, 0.93052, 0.92120, 0.91336, 0.90727, 0.90275, 0.89831, 0.89293,\
        0.88560, 0.87688, 0.86742, 0.85765, 0.84788, 0.83842, 0.82962, 0.82181, 0.81537, 0.81052, 0.80712,\
        0.80460, 0.80235, 0.80038, 0.79906, 0.79721, 0.79280, 0.78526, 0.77570, 0.76505, 0.75399, 0.74309,\
        0.73291, 0.72398, 0.71673, 0.71115, 0.70650, 0.70199, 0.69690, 0.68955, 0.67859, 0.66547, 0.65166,\
        0.63820, 0.62598, 0.61579, 0.60800, 0.60240, 0.59855, 0.59623, 0.59492, 0.59418, 0.59366, 0.59288],\
        [0.75611, 0.76146, 0.76694, 0.77257, 0.77839, 0.78438, 0.79058, 0.79700, 0.80380, 0.81101, 0.81857,\
        0.82647, 0.83467, 0.84316, 0.85189, 0.86084, 0.87000, 0.87940, 0.88911, 0.89932, 0.91062, 0.92241,\
        0.93442, 0.94648, 0.95839, 0.96986, 0.98053, 0.98997, 0.99772, 1.00331, 1.00623, 1.00605, 1.00240,\
        0.99504, 0.98402, 0.96964, 0.95234, 0.93238, 0.91069, 0.89000, 0.87292, 0.85920, 0.84167, 0.81023,\
        0.77158, 0.76429, 0.77380, 0.80226, 0.83233, 0.85730, 0.88252, 0.90521, 0.92626, 0.94386, 0.96141,\
        0.97842, 0.99340, 1.00534, 1.01375, 1.01858, 1.02001, 1.01836, 1.01401, 1.00737, 0.99886, 0.98890,\
        0.97789, 0.96622, 0.95431, 0.94253, 0.93130, 0.92102, 0.91209, 0.90482, 0.89953, 0.89471, 0.88893,\
        0.88185, 0.87351, 0.86427, 0.85453, 0.84464, 0.83493, 0.82575, 0.81753, 0.81079, 0.80600, 0.80308,\
        0.80120, 0.79916, 0.79716, 0.79525, 0.79253, 0.78777, 0.78055, 0.77132, 0.76079, 0.74962, 0.73839,\
        0.72770, 0.71820, 0.71060, 0.70526, 0.70074, 0.69585, 0.69002, 0.68220, 0.67175, 0.65922, 0.64568,\
        0.63215, 0.61970, 0.60947, 0.60221, 0.59733, 0.59413, 0.59223, 0.59122, 0.59069, 0.59028, 0.58968],\
        [0.75693, 0.76234, 0.76788, 0.77360, 0.77952, 0.78565, 0.79199, 0.79850, 0.80530, 0.81243, 0.81990,\
        0.82773, 0.83586, 0.84426, 0.85287, 0.86165, 0.87058, 0.87970, 0.88924, 0.89973, 0.91150, 0.92418,\
        0.93717, 0.95016, 0.96288, 0.97505, 0.98630, 0.99620, 1.00427, 1.00998, 1.01275, 1.01200, 1.00721,\
        0.99798, 0.98422, 0.96634, 0.94520, 0.92106, 0.89241, 0.86189, 0.83550, 0.81103, 0.77073, 0.71509,\
        0.67301, 0.65914, 0.68754, 0.73080, 0.76307, 0.79948, 0.83145, 0.86637, 0.89505, 0.91805, 0.94269,\
        0.96602, 0.98610, 1.00180, 1.01289, 1.01956, 1.02222, 1.02134, 1.01741, 1.01094, 1.00240, 0.99226,\
        0.98094, 0.96886, 0.95640, 0.94396, 0.93189, 0.92061, 0.91057, 0.90230, 0.89601, 0.89079, 0.88501,\
        0.87812, 0.87006, 0.86106, 0.85144, 0.84155, 0.83170, 0.82226, 0.81365, 0.80647, 0.80169, 0.79930,\
        0.79792, 0.79632, 0.79431, 0.79182, 0.78829, 0.78306, 0.77582, 0.76676, 0.75636, 0.74514, 0.73366,\
        0.72250, 0.71232, 0.70399, 0.69874, 0.69449, 0.68946, 0.68322, 0.67515, 0.66497, 0.65297, 0.63986,\
        0.62651, 0.61390, 0.60344, 0.59686, 0.59286, 0.59031, 0.58879, 0.58799, 0.58758, 0.58727, 0.58680],\
        [0.75764, 0.76307, 0.76865, 0.77443, 0.78043, 0.78670, 0.79321, 0.79986, 0.80660, 0.81362, 0.82104,\
        0.82883, 0.83695, 0.84534, 0.85393, 0.86267, 0.87153, 0.88057, 0.89002, 0.90046, 0.91285, 0.92615,\
        0.93979, 0.95345, 0.96683, 0.97961, 0.99143, 1.00183, 1.01030, 1.01625, 1.01902, 1.01791, 1.01218,\
        1.00114, 0.98440, 0.96220, 0.93613, 0.90811, 0.87395, 0.82995, 0.78554, 0.74928, 0.70477, 0.65112,\
        0.59458, 0.57630, 0.61632, 0.66869, 0.70158, 0.73806, 0.78256, 0.83116, 0.86458, 0.89527, 0.92631,\
        0.95622, 0.98106, 1.00000, 1.01328, 1.02140, 1.02497, 1.02460, 1.02091, 1.01448, 1.00584, 0.99548,\
        0.98386, 0.97139, 0.95846, 0.94544, 0.93268, 0.92055, 0.90950, 0.90034, 0.89364, 0.88796, 0.88198,\
        0.87507, 0.86712, 0.85827, 0.84879, 0.83899, 0.82918, 0.81971, 0.81098, 0.80354, 0.79869, 0.79676,\
        0.79557, 0.79411, 0.79198, 0.78898, 0.78480, 0.77910, 0.77169, 0.76269, 0.75240, 0.74125, 0.72970,\
        0.71830, 0.70777, 0.69918, 0.69352, 0.68905, 0.68390, 0.67741, 0.66922, 0.65926, 0.64775, 0.63523,\
        0.62243, 0.61022, 0.59992, 0.59373, 0.59002, 0.58767, 0.58627, 0.58551, 0.58512, 0.58483, 0.58443],\
        [0.75821, 0.76365, 0.76923, 0.77501, 0.78103, 0.78734, 0.79395, 0.80073, 0.80741, 0.81443, 0.82188,\
        0.82974, 0.83794, 0.84643, 0.85512, 0.86398, 0.87297, 0.88217, 0.89188, 0.90268, 0.91499, 0.92840,\
        0.94231, 0.95634, 0.97017, 0.98345, 0.99576, 1.00665, 1.01556, 1.02186, 1.02482, 1.02362, 1.01729,\
        1.00482, 0.98529, 0.95827, 0.92470, 0.89070, 0.85305, 0.80045, 0.73721, 0.69795, 0.66678, 0.58574,\
        0.49832, 0.49802, 0.52930, 0.60319, 0.65715, 0.68895, 0.73782, 0.79101, 0.83522, 0.87616, 0.91536,\
        0.95090, 0.97902, 1.00023, 1.01503, 1.02410, 1.02819, 1.02806, 1.02442, 1.01790, 1.00909, 0.99850,\
        0.98661, 0.97382, 0.96054, 0.94713, 0.93393, 0.92132, 0.90976, 0.90006, 0.89289, 0.88668, 0.88028,\
        0.87310, 0.86503, 0.85620, 0.84683, 0.83721, 0.82763, 0.81843, 0.81003, 0.80308, 0.79851, 0.79625,\
        0.79479, 0.79304, 0.79051, 0.78697, 0.78221, 0.77606, 0.76846, 0.75948, 0.74936, 0.73843, 0.72707,\
        0.71581, 0.70538, 0.69697, 0.69095, 0.68579, 0.68011, 0.67325, 0.66492, 0.65512, 0.64409, 0.63229,\
        0.62040, 0.60929, 0.60010, 0.59345, 0.58917, 0.58648, 0.58485, 0.58393, 0.58343, 0.58308, 0.58266],\
        [0.75867, 0.76407, 0.76962, 0.77535, 0.78130, 0.78751, 0.79398, 0.80064, 0.80755, 0.81478, 0.82240,\
        0.83042, 0.83879, 0.84747, 0.85640, 0.86552, 0.87480, 0.88430, 0.89429, 0.90527, 0.91746, 0.93066,\
        0.94451, 0.95867, 0.97275, 0.98638, 0.99911, 1.01043, 1.01978, 1.02648, 1.02979, 1.02876, 1.02228,\
        1.00902, 0.98749, 0.95672, 0.91781, 0.87385, 0.82374, 0.75982, 0.69087, 0.65207, 0.57888, 0.46666,\
        0.42467, 0.43011, 0.44639, 0.48903, 0.57951, 0.64755, 0.69532, 0.75156, 0.80725, 0.86067, 0.91080,\
        0.94874, 0.97915, 1.00207, 1.01787, 1.02742, 1.03166, 1.03151, 1.02773, 1.02103, 1.01201, 1.00120,\
        0.98908, 0.97607, 0.96259, 0.94900, 0.93566, 0.92297, 0.91142, 0.90168, 0.89381, 0.88700, 0.87997,\
        0.87226, 0.86385, 0.85490, 0.84562, 0.83624, 0.82704, 0.81834, 0.81053, 0.80408, 0.79976, 0.79751,\
        0.79552, 0.79317, 0.79002, 0.78584, 0.78051, 0.77396, 0.76619, 0.75728, 0.74741, 0.73688, 0.72601,\
        0.71527, 0.70525, 0.69705, 0.69080, 0.68487, 0.67840, 0.67098, 0.66241, 0.65271, 0.64211, 0.63101,\
        0.62002, 0.60988, 0.60128, 0.59457, 0.58976, 0.58652, 0.58448, 0.58327, 0.58256, 0.58206, 0.58155],\
        [0.75901, 0.76437, 0.76986, 0.77550, 0.78133, 0.78738, 0.79367, 0.80026, 0.80733, 0.81479, 0.82264,\
        0.83088, 0.83947, 0.84839, 0.85760, 0.86708, 0.87677, 0.88664, 0.89675, 0.90769, 0.91961, 0.93245,\
        0.94608, 0.96018, 0.97438, 0.98824, 1.00126, 1.01293, 1.02264, 1.02973, 1.03341, 1.03272, 1.02646,\
        1.01305, 0.99034, 0.95570, 0.90814, 0.85243, 0.78819, 0.71317, 0.64869, 0.58037, 0.47699, 0.42382,\
        0.39627, 0.38578, 0.40096, 0.43024, 0.48920, 0.58119, 0.65440, 0.71209, 0.77398, 0.84151, 0.90002,\
        0.94553, 0.98019, 1.00488, 1.02128, 1.03090, 1.03500, 1.03461, 1.03059, 1.02365, 1.01440, 1.00340,\
        0.99112, 0.97800, 0.96445, 0.95087, 0.93762, 0.92508, 0.91362, 0.90353, 0.89569, 0.88863, 0.88072,\
        0.87224, 0.86336, 0.85422, 0.84500, 0.83590, 0.82716, 0.81908, 0.81199, 0.80632, 0.80244, 0.79976,\
        0.79721, 0.79419, 0.79031, 0.78545, 0.77957, 0.77267, 0.76478, 0.75600, 0.74650, 0.73651, 0.72637,\
        0.71647, 0.70731, 0.69957, 0.69288, 0.68590, 0.67850, 0.67045, 0.66158, 0.65192, 0.64163, 0.63109,\
        0.62079, 0.61130, 0.60309, 0.59636, 0.59113, 0.58740, 0.58495, 0.58342, 0.58246, 0.58178, 0.58111],\
        [0.75927, 0.76458, 0.77000, 0.77554, 0.78125, 0.78716, 0.79335, 0.79995, 0.80704, 0.81463, 0.82269,\
        0.83114, 0.83994, 0.84908, 0.85855, 0.86835, 0.87848, 0.88885, 0.89918, 0.90970, 0.92097, 0.93331,\
        0.94664, 0.96063, 0.97487, 0.98885, 1.00206, 1.01394, 1.02389, 1.03123, 1.03517, 1.03475, 1.02876,\
        1.01554, 0.99251, 0.95519, 0.89678, 0.81697, 0.74238, 0.66406, 0.58461, 0.49507, 0.43268, 0.39152,\
        0.34272, 0.31612, 0.33927, 0.38667, 0.43537, 0.48135, 0.57872, 0.65949, 0.74227, 0.82716, 0.89320,\
        0.94517, 0.98274, 1.00828, 1.02467, 1.03399, 1.03772, 1.03699, 1.03267, 1.02548, 1.01605, 1.00491,\
        0.99254, 0.97939, 0.96589, 0.95244, 0.93942, 0.92723, 0.91622, 0.90680, 0.89951, 0.89089, 0.88177,\
        0.87248, 0.86314, 0.85384, 0.84470, 0.83589, 0.82762, 0.82014, 0.81372, 0.80862, 0.80488, 0.80201,\
        0.79916, 0.79560, 0.79099, 0.78548, 0.77912, 0.77195, 0.76403, 0.75544, 0.74636, 0.73700, 0.72765,\
        0.71868, 0.71048, 0.70327, 0.69591, 0.68801, 0.67984, 0.67127, 0.66214, 0.65247, 0.64241, 0.63224,\
        0.62237, 0.61323, 0.60519, 0.59838, 0.59285, 0.58878, 0.58602, 0.58425, 0.58307, 0.58218, 0.58132],\
        [0.75946, 0.76473, 0.77010, 0.77556, 0.78116, 0.78694, 0.79301, 0.79957, 0.80665, 0.81439, 0.82262,\
        0.83124, 0.84018, 0.84943, 0.85901, 0.86895, 0.87930, 0.89003, 0.90067, 0.91036, 0.92087, 0.93279,\
        0.94591, 0.95984, 0.97409, 0.98812, 1.00140, 1.01335, 1.02336, 1.03073, 1.03467, 1.03422, 1.02817,\
        1.01485, 0.99171, 0.95375, 0.88629, 0.78670, 0.70246, 0.61584, 0.54114, 0.46257, 0.41161, 0.36020,\
        0.29951, 0.27686, 0.30271, 0.35776, 0.40725, 0.45175, 0.52993, 0.61480, 0.72366, 0.81538, 0.89376,\
        0.94916, 0.98657, 1.01151, 1.02728, 1.03604, 1.03932, 1.03824, 1.03366, 1.02630, 1.01675, 1.00554,\
        0.99316, 0.98005, 0.96665, 0.95336, 0.94060, 0.92876, 0.91820, 0.90918, 0.90096, 0.89184, 0.88209,\
        0.87233, 0.86274, 0.85339, 0.84440, 0.83589, 0.82804, 0.82105, 0.81511, 0.81035, 0.80668, 0.80373,\
        0.80082, 0.79674, 0.79153, 0.78553, 0.77884, 0.77151, 0.76364, 0.75531, 0.74668, 0.73793, 0.72930,\
        0.72103, 0.71332, 0.70602, 0.69846, 0.69032, 0.68185, 0.67301, 0.66375, 0.65410, 0.64417, 0.63423,\
        0.62456, 0.61553, 0.60741, 0.60041, 0.59470, 0.59047, 0.58758, 0.58567, 0.58433, 0.58324, 0.58215],\
        [0.75960, 0.76486, 0.77019, 0.77562, 0.78117, 0.78685, 0.79272, 0.79900, 0.80622, 0.81419, 0.82256,\
        0.83125, 0.84021, 0.84943, 0.85891, 0.86866, 0.87870, 0.88896, 0.89904, 0.90836, 0.91880, 0.93070,\
        0.94383, 0.95780, 0.97207, 0.98609, 0.99933, 1.01118, 1.02104, 1.02821, 1.03186, 1.03102, 1.02439,\
        1.01024, 0.98599, 0.94742, 0.88566, 0.79866, 0.70129, 0.60820, 0.54040, 0.45491, 0.40635, 0.37039,\
        0.31228, 0.28343, 0.31408, 0.37043, 0.40877, 0.45996, 0.54653, 0.63372, 0.72341, 0.81239, 0.89600,\
        0.95283, 0.98933, 1.01322, 1.02821, 1.03644, 1.03937, 1.03806, 1.03334, 1.02590, 1.01633, 1.00515,\
        0.99283, 0.97981, 0.96652, 0.95337, 0.94075, 0.92904, 0.91850, 0.90916, 0.90044, 0.89100, 0.88118,\
        0.87140, 0.86184, 0.85262, 0.84384, 0.83562, 0.82812, 0.82149, 0.81584, 0.81123, 0.80753, 0.80440,\
        0.80110, 0.79688, 0.79149, 0.78525, 0.77840, 0.77107, 0.76334, 0.75533, 0.74714, 0.73894, 0.73089,\
        0.72312, 0.71565, 0.70828, 0.70066, 0.69266, 0.68423, 0.67538, 0.66613, 0.65653, 0.64671, 0.63688,\
        0.62729, 0.61821, 0.60991, 0.60263, 0.59672, 0.59246, 0.58960, 0.58767, 0.58622, 0.58490, 0.58353],\
        [0.75970, 0.76495, 0.77029, 0.77574, 0.78132, 0.78706, 0.79297, 0.79918, 0.80639, 0.81431, 0.82261,\
        0.83122, 0.84007, 0.84913, 0.85833, 0.86764, 0.87697, 0.88619, 0.89505, 0.90440, 0.91521, 0.92731,\
        0.94063, 0.95471, 0.96898, 0.98292, 0.99599, 1.00760, 1.01715, 1.02392, 1.02708, 1.02562, 1.01823,\
        1.00314, 0.97786, 0.93889, 0.88571, 0.82172, 0.74118, 0.66637, 0.61248, 0.49200, 0.39831, 0.39936,\
        0.36460, 0.33560, 0.36425, 0.40943, 0.44026, 0.52062, 0.60615, 0.68459, 0.75800, 0.82982, 0.90182,\
        0.95462, 0.98944, 1.01242, 1.02689, 1.03485, 1.03764, 1.03627, 1.03157, 1.02419, 1.01472, 1.00365,\
        0.99145, 0.97856, 0.96538, 0.95230, 0.93970, 0.92790, 0.91709, 0.90719, 0.89787, 0.88860, 0.87911,\
        0.86963, 0.86034, 0.85138, 0.84287, 0.83493, 0.82770, 0.82129, 0.81578, 0.81116, 0.80731, 0.80400,\
        0.80084, 0.79635, 0.79073, 0.78442, 0.77760, 0.77040, 0.76291, 0.75523, 0.74749, 0.73978, 0.73222,\
        0.72485, 0.71764, 0.71041, 0.70298, 0.69516, 0.68690, 0.67819, 0.66906, 0.65956, 0.64984, 0.64009,\
        0.63053, 0.62139, 0.61291, 0.60536, 0.59921, 0.59494, 0.59216, 0.59029, 0.58870, 0.58710, 0.58539],\
        [0.75974, 0.76500, 0.77036, 0.77587, 0.78155, 0.78747, 0.79370, 0.80032, 0.80718, 0.81470, 0.82274,\
        0.83116, 0.83983, 0.84866, 0.85754, 0.86635, 0.87498, 0.88334, 0.89168, 0.90082, 0.91104, 0.92324,\
        0.93680, 0.95095, 0.96515, 0.97888, 0.99163, 1.00287, 1.01199, 1.01828, 1.02093, 1.01894, 1.01111,\
        0.99596, 0.97184, 0.93767, 0.89429, 0.84053, 0.77566, 0.71232, 0.68249, 0.63280, 0.44474, 0.41336,\
        0.41031, 0.40043, 0.41992, 0.45402, 0.51168, 0.62198, 0.66648, 0.72227, 0.78605, 0.84668, 0.90893,\
        0.95458, 0.98698, 1.00916, 1.02338, 1.03130, 1.03415, 1.03291, 1.02837, 1.02117, 1.01190, 1.00104,\
        0.98903, 0.97631, 0.96324, 0.95019, 0.93752, 0.92553, 0.91443, 0.90419, 0.89463, 0.88545, 0.87632,\
        0.86723, 0.85830, 0.84967, 0.84145, 0.83376, 0.82672, 0.82043, 0.81495, 0.81024, 0.80613, 0.80236,\
        0.79882, 0.79455, 0.78909, 0.78294, 0.77632, 0.76936, 0.76217, 0.75487, 0.74755, 0.74030, 0.73319,\
        0.72622, 0.71934, 0.71240, 0.70524, 0.69772, 0.68977, 0.68132, 0.67238, 0.66303, 0.65342, 0.64375,\
        0.63423, 0.62508, 0.61654, 0.60892, 0.60262, 0.59799, 0.59536, 0.59358, 0.59174, 0.58972, 0.58757],\
        [0.75969, 0.76496, 0.77036, 0.77592, 0.78169, 0.78774, 0.79412, 0.80077, 0.80763, 0.81494, 0.82277,\
        0.83099, 0.83951, 0.84817, 0.85681, 0.86527, 0.87340, 0.88118, 0.88881, 0.89678, 0.90674, 0.91936,\
        0.93301, 0.94702, 0.96092, 0.97426, 0.98657, 0.99734, 1.00597, 1.01181, 1.01407, 1.01186, 1.00417,\
        0.98993, 0.96826, 0.93895, 0.90245, 0.85885, 0.80784, 0.74601, 0.70040, 0.70411, 0.62758, 0.49117,\
        0.45987, 0.45284, 0.47170, 0.54231, 0.61219, 0.66771, 0.70532, 0.77232, 0.82879, 0.87003, 0.91426,\
        0.95324, 0.98312, 1.00438, 1.01833, 1.02624, 1.02922, 1.02818, 1.02387, 1.01695, 1.00795, 0.99737,\
        0.98563, 0.97313, 0.96021, 0.94721, 0.93444, 0.92223, 0.91091, 0.90065, 0.89107, 0.88201, 0.87317,\
        0.86446, 0.85590, 0.84759, 0.83964, 0.83215, 0.82523, 0.81898, 0.81346, 0.80865, 0.80437, 0.80027,\
        0.79628, 0.79192, 0.78674, 0.78089, 0.77455, 0.76789, 0.76105, 0.75412, 0.74721, 0.74040, 0.73373,\
        0.72719, 0.72071, 0.71418, 0.70743, 0.70030, 0.69279, 0.68466, 0.67594, 0.66675, 0.65727, 0.64770,\
        0.63826, 0.62918, 0.62070, 0.61310, 0.60668, 0.60180, 0.59945, 0.59742, 0.59509, 0.59251, 0.58988],\
        [0.75953, 0.76480, 0.77021, 0.77579, 0.78159, 0.78766, 0.79403, 0.80067, 0.80752, 0.81478, 0.82250,\
        0.83063, 0.83908, 0.84770, 0.85630, 0.86468, 0.87265, 0.88016, 0.88739, 0.89502, 0.90474, 0.91679,\
        0.92990, 0.94333, 0.95662, 0.96935, 0.98108, 0.99130, 0.99947, 1.00496, 1.00707, 1.00503, 0.99804,\
        0.98536, 0.96650, 0.94137, 0.91066, 0.87740, 0.84001, 0.79157, 0.73599, 0.69837, 0.66929, 0.60489,\
        0.54158, 0.51811, 0.55174, 0.62923, 0.67082, 0.70510, 0.75364, 0.81543, 0.85793, 0.88787, 0.92026,\
        0.95305, 0.97975, 0.99944, 1.01266, 1.02031, 1.02328, 1.02238, 1.01831, 1.01168, 1.00302, 0.99277,\
        0.98136, 0.96915, 0.95647, 0.94360, 0.93081, 0.91841, 0.90683, 0.89654, 0.88729, 0.87849, 0.86994,\
        0.86153, 0.85330, 0.84528, 0.83756, 0.83022, 0.82336, 0.81707, 0.81144, 0.80653, 0.80220, 0.79801,\
        0.79370, 0.78914, 0.78406, 0.77844, 0.77238, 0.76603, 0.75951, 0.75293, 0.74641, 0.74000, 0.73376,\
        0.72767, 0.72168, 0.71566, 0.70947, 0.70295, 0.69587, 0.68804, 0.67955, 0.67053, 0.66120, 0.65176,\
        0.64244, 0.63348, 0.62514, 0.61767, 0.61137, 0.60665, 0.60359, 0.60110, 0.59820, 0.59510, 0.59213],\
        [0.75923, 0.76448, 0.76987, 0.77543, 0.78121, 0.78722, 0.79346, 0.79998, 0.80688, 0.81417, 0.82185,\
        0.82996, 0.83845, 0.84721, 0.85603, 0.86466, 0.87286, 0.88051, 0.88782, 0.89551, 0.90470, 0.91562,\
        0.92761, 0.94003, 0.95241, 0.96433, 0.97535, 0.98500, 0.99277, 0.99806, 1.00028, 0.99879, 0.99299,\
        0.98238, 0.96674, 0.94627, 0.92205, 0.89649, 0.86986, 0.83902, 0.80433, 0.75627, 0.70641, 0.67260,\
        0.60963, 0.58670, 0.62207, 0.67696, 0.71702, 0.75862, 0.80234, 0.84318, 0.87787, 0.90394, 0.92992,\
        0.95589, 0.97822, 0.99535, 1.00716, 1.01410, 1.01678, 1.01586, 1.01195, 1.00558, 0.99725, 0.98738,\
        0.97636, 0.96453, 0.95219, 0.93962, 0.92704, 0.91466, 0.90287, 0.89279, 0.88375, 0.87515, 0.86679,\
        0.85862, 0.85063, 0.84286, 0.83534, 0.82813, 0.82129, 0.81489, 0.80907, 0.80398, 0.79953, 0.79531,\
        0.79094, 0.78629, 0.78123, 0.77575, 0.76991, 0.76380, 0.75756, 0.75129, 0.74510, 0.73906, 0.73322,\
        0.72758, 0.72210, 0.71665, 0.71109, 0.70520, 0.69865, 0.69120, 0.68296, 0.67414, 0.66497, 0.65567,\
        0.64650, 0.63769, 0.62951, 0.62218, 0.61597, 0.61104, 0.60725, 0.60397, 0.60057, 0.59734, 0.59422],\
        [0.75877, 0.76397, 0.76931, 0.77483, 0.78054, 0.78648, 0.79263, 0.79905, 0.80591, 0.81312, 0.82072,\
        0.82882, 0.83745, 0.84652, 0.85583, 0.86506, 0.87387, 0.88201, 0.88954, 0.89709, 0.90554, 0.91523,\
        0.92585, 0.93699, 0.94825, 0.95923, 0.96950, 0.97860, 0.98605, 0.99132, 0.99390, 0.99327, 0.98901,\
        0.98083, 0.96871, 0.95307, 0.93491, 0.91569, 0.89594, 0.87637, 0.85583, 0.82292, 0.77752, 0.74858,\
        0.70573, 0.68183, 0.69923, 0.73925, 0.77316, 0.80839, 0.84252, 0.87192, 0.89757, 0.91982, 0.94072,\
        0.96077, 0.97841, 0.99238, 1.00220, 1.00797, 1.01005, 1.00890, 1.00500, 0.99883, 0.99081, 0.98133,\
        0.97075, 0.95938, 0.94751, 0.93541, 0.92334, 0.91152, 0.90022, 0.89000, 0.88078, 0.87213, 0.86383,\
        0.85579, 0.84799, 0.84042, 0.83309, 0.82603, 0.81923, 0.81274, 0.80666, 0.80131, 0.79684, 0.79259,\
        0.78817, 0.78344, 0.77837, 0.77294, 0.76721, 0.76128, 0.75524, 0.74920, 0.74327, 0.73753, 0.73204,\
        0.72682, 0.72183, 0.71696, 0.71204, 0.70678, 0.70087, 0.69383, 0.68590, 0.67733, 0.66834, 0.65919,\
        0.65017, 0.64152, 0.63347, 0.62624, 0.61999, 0.61477, 0.61041, 0.60656, 0.60291, 0.59955, 0.59619],\
        [0.75813, 0.76326, 0.76852, 0.77395, 0.77958, 0.78544, 0.79157, 0.79797, 0.80466, 0.81158, 0.81897,\
        0.82703, 0.83585, 0.84536, 0.85536, 0.86549, 0.87529, 0.88424, 0.89201, 0.89912, 0.90668, 0.91502,\
        0.92416, 0.93392, 0.94399, 0.95400, 0.96353, 0.97215, 0.97939, 0.98479, 0.98792, 0.98837, 0.98586,\
        0.98025, 0.97164, 0.96046, 0.94742, 0.93329, 0.91849, 0.90384, 0.88930, 0.86838, 0.84225, 0.81902,\
        0.78553, 0.77438, 0.78710, 0.81142, 0.83147, 0.85237, 0.87585, 0.89776, 0.91684, 0.93450, 0.95090,\
        0.96612, 0.97949, 0.99019, 0.99773, 1.00203, 1.00324, 1.00166, 0.99765, 0.99159, 0.98384, 0.97475,\
        0.96463, 0.95377, 0.94244, 0.93093, 0.91951, 0.90846, 0.89782, 0.88757, 0.87811, 0.86931, 0.86099,\
        0.85303, 0.84538, 0.83801, 0.83089, 0.82403, 0.81739, 0.81098, 0.80480, 0.79919, 0.79468, 0.79023,\
        0.78560, 0.78071, 0.77552, 0.77006, 0.76436, 0.75850, 0.75257, 0.74668, 0.74092, 0.73540, 0.73017,\
        0.72529, 0.72075, 0.71643, 0.71214, 0.70752, 0.70212, 0.69566, 0.68820, 0.67989, 0.67106, 0.66207,\
        0.65319, 0.64469, 0.63676, 0.62958, 0.62325, 0.61778, 0.61303, 0.60882, 0.60499, 0.60150, 0.59771],\
        [0.75732, 0.76234, 0.76749, 0.77279, 0.77828, 0.78402, 0.79004, 0.79638, 0.80279, 0.80929, 0.81639,\
        0.82436, 0.83336, 0.84338, 0.85419, 0.86540, 0.87648, 0.88666, 0.89502, 0.90139, 0.90762, 0.91441,\
        0.92206, 0.93049, 0.93943, 0.94853, 0.95741, 0.96563, 0.97279, 0.97845, 0.98225, 0.98389, 0.98319,\
        0.98010, 0.97477, 0.96755, 0.95897, 0.94956, 0.93980, 0.93024, 0.92104, 0.90999, 0.89750, 0.88829,\
        0.87355, 0.86495, 0.86727, 0.87544, 0.88266, 0.89120, 0.90527, 0.92088, 0.93484, 0.94781, 0.95992,\
        0.97101, 0.98063, 0.98827, 0.99351, 0.99621, 0.99639, 0.99423, 0.98999, 0.98395, 0.97643, 0.96771,\
        0.95807, 0.94776, 0.93700, 0.92606, 0.91522, 0.90481, 0.89461, 0.88467, 0.87527, 0.86645, 0.85814,\
        0.85026, 0.84276, 0.83560, 0.82873, 0.82214, 0.81579, 0.80969, 0.80394, 0.79861, 0.79331, 0.78830,\
        0.78328, 0.77810, 0.77272, 0.76714, 0.76138, 0.75551, 0.74961, 0.74377, 0.73808, 0.73266, 0.72759,\
        0.72295, 0.71876, 0.71495, 0.71130, 0.70744, 0.70274, 0.69686, 0.68976, 0.68161, 0.67291, 0.66405,\
        0.65533, 0.64697, 0.63917, 0.63204, 0.62565, 0.62000, 0.61498, 0.61046, 0.60628, 0.60223, 0.59812],\
        [0.75633, 0.76123, 0.76622, 0.77134, 0.77663, 0.78212, 0.78785, 0.79381, 0.79984, 0.80603, 0.81282,\
        0.82062, 0.82975, 0.84024, 0.85183, 0.86409, 0.87645, 0.88814, 0.89777, 0.90337, 0.90751, 0.91272,\
        0.91909, 0.92638, 0.93435, 0.94269, 0.95105, 0.95902, 0.96619, 0.97219, 0.97673, 0.97958, 0.98061,\
        0.97985, 0.97746, 0.97374, 0.96911, 0.96404, 0.95902, 0.95451, 0.95067, 0.94714, 0.94498, 0.94506,\
        0.94147, 0.93493, 0.92927, 0.92688, 0.92545, 0.92651, 0.93250, 0.94141, 0.95051, 0.95923, 0.96744,\
        0.97490, 0.98127, 0.98615, 0.98923, 0.99034, 0.98944, 0.98662, 0.98207, 0.97599, 0.96866, 0.96029,\
        0.95113, 0.94138, 0.93124, 0.92086, 0.91038, 0.90032, 0.89074, 0.88124, 0.87206, 0.86335, 0.85513,\
        0.84737, 0.84005, 0.83312, 0.82653, 0.82026, 0.81425, 0.80847, 0.80291, 0.79746, 0.79197, 0.78652,\
        0.78109, 0.77559, 0.76995, 0.76418, 0.75830, 0.75235, 0.74640, 0.74052, 0.73481, 0.72938, 0.72433,\
        0.71979, 0.71582, 0.71239, 0.70934, 0.70636, 0.70284, 0.69754, 0.69033, 0.68218, 0.67360, 0.66491,\
        0.65639, 0.64822, 0.64056, 0.63351, 0.62711, 0.62136, 0.61617, 0.61141, 0.60691, 0.60246, 0.59810],\
        [0.75520, 0.75994, 0.76474, 0.76964, 0.77466, 0.77981, 0.78512, 0.79056, 0.79612, 0.80196, 0.80830,\
        0.81576, 0.82493, 0.83581, 0.84801, 0.86098, 0.87408, 0.88655, 0.89698, 0.90150, 0.90473, 0.90935,\
        0.91494, 0.92141, 0.92865, 0.93643, 0.94443, 0.95227, 0.95954, 0.96594, 0.97119, 0.97516, 0.97777,\
        0.97907, 0.97922, 0.97849, 0.97723, 0.97586, 0.97479, 0.97442, 0.97495, 0.97645, 0.97904, 0.98165,\
        0.98123, 0.97683, 0.97037, 0.96397, 0.95857, 0.95551, 0.95592, 0.95911, 0.96358, 0.96840, 0.97313,\
        0.97744, 0.98100, 0.98348, 0.98462, 0.98425, 0.98230, 0.97881, 0.97390, 0.96774, 0.96055, 0.95252,\
        0.94385, 0.93468, 0.92518, 0.91555, 0.90585, 0.89592, 0.88664, 0.87744, 0.86849, 0.85994, 0.85187,\
        0.84427, 0.83715, 0.83048, 0.82420, 0.81826, 0.81258, 0.80706, 0.80157, 0.79602, 0.79036, 0.78464,\
        0.77889, 0.77307, 0.76718, 0.76119, 0.75514, 0.74906, 0.74299, 0.73702, 0.73121, 0.72566, 0.72049,\
        0.71588, 0.71197, 0.70875, 0.70602, 0.70357, 0.70106, 0.69617, 0.68913, 0.68120, 0.67289, 0.66449,\
        0.65625, 0.64833, 0.64086, 0.63393, 0.62759, 0.62182, 0.61655, 0.61168, 0.60706, 0.60254, 0.59802],\
        [0.75393, 0.75851, 0.76312, 0.76778, 0.77250, 0.77728, 0.78212, 0.78702, 0.79204, 0.79731, 0.80307,\
        0.80993, 0.81908, 0.83029, 0.84285, 0.85605, 0.86911, 0.88107, 0.89039, 0.89570, 0.89987, 0.90450,\
        0.90969, 0.91562, 0.92234, 0.92976, 0.93759, 0.94541, 0.95284, 0.95959, 0.96549, 0.97042, 0.97438,\
        0.97741, 0.97966, 0.98137, 0.98282, 0.98435, 0.98629, 0.98890, 0.99231, 0.99642, 1.00075, 1.00406,\
        1.00456, 1.00147, 0.99561, 0.98853, 0.98169, 0.97639, 0.97343, 0.97269, 0.97344, 0.97496, 0.97674,\
        0.97839, 0.97958, 0.98002, 0.97948, 0.97779, 0.97488, 0.97075, 0.96548, 0.95921, 0.95213, 0.94441,\
        0.93623, 0.92766, 0.91875, 0.90972, 0.90118, 0.89200, 0.88245, 0.87331, 0.86454, 0.85618, 0.84828,\
        0.84089, 0.83400, 0.82760, 0.82165, 0.81604, 0.81067, 0.80541, 0.80007, 0.79440, 0.78855, 0.78260,\
        0.77658, 0.77050, 0.76435, 0.75815, 0.75191, 0.74567, 0.73946, 0.73336, 0.72741, 0.72168, 0.71627,\
        0.71140, 0.70739, 0.70424, 0.70153, 0.69908, 0.69639, 0.69207, 0.68582, 0.67850, 0.67069, 0.66273,\
        0.65486, 0.64726, 0.64004, 0.63330, 0.62706, 0.62134, 0.61608, 0.61120, 0.60658, 0.60208, 0.59745],\
        [0.75256, 0.75698, 0.76140, 0.76584, 0.77028, 0.77472, 0.77915, 0.78356, 0.78799, 0.79256, 0.79746,\
        0.80358, 0.81291, 0.82430, 0.83694, 0.84995, 0.86250, 0.87369, 0.88258, 0.88893, 0.89394, 0.89869,\
        0.90369, 0.90926, 0.91562, 0.92285, 0.93067, 0.93855, 0.94610, 0.95312, 0.95951, 0.96521, 0.97022,\
        0.97462, 0.97853, 0.98211, 0.98559, 0.98922, 0.99322, 0.99772, 1.00271, 1.00792, 1.01271, 1.01605,\
        1.01682, 1.01448, 1.00946, 1.00281, 0.99579, 0.98951, 0.98470, 0.98151, 0.97963, 0.97863, 0.97806,\
        0.97758, 0.97685, 0.97562, 0.97369, 0.97089, 0.96715, 0.96245, 0.95684, 0.95043, 0.94341, 0.93597,\
        0.92828, 0.92037, 0.91206, 0.90331, 0.89481, 0.88639, 0.87752, 0.86870, 0.86018, 0.85204, 0.84436,\
        0.83719, 0.83054, 0.82443, 0.81880, 0.81354, 0.80845, 0.80334, 0.79809, 0.79243, 0.78648, 0.78036,\
        0.77413, 0.76782, 0.76145, 0.75504, 0.74862, 0.74221, 0.73587, 0.72963, 0.72354, 0.71765, 0.71201,\
        0.70672, 0.70236, 0.69934, 0.69654, 0.69386, 0.69078, 0.68657, 0.68097, 0.67434, 0.66714, 0.65970,\
        0.65228, 0.64505, 0.63814, 0.63162, 0.62554, 0.61991, 0.61472, 0.60989, 0.60529, 0.60066, 0.59598],\
        [0.75111, 0.75539, 0.75965, 0.76390, 0.76814, 0.77233, 0.77647, 0.78053, 0.78454, 0.78858, 0.79296,\
        0.79889, 0.80752, 0.81880, 0.83116, 0.84362, 0.85546, 0.86602, 0.87477, 0.88171, 0.88739, 0.89248,\
        0.89746, 0.90283, 0.90878, 0.91601, 0.92398, 0.93186, 0.93939, 0.94650, 0.95317, 0.95940, 0.96519,\
        0.97060, 0.97570, 0.98062, 0.98550, 0.99050, 0.99573, 1.00124, 1.00691, 1.01239, 1.01708, 1.02020,\
        1.02101, 1.01916, 1.01490, 1.00896, 1.00230, 0.99582, 0.99016, 0.98560, 0.98207, 0.97932, 0.97706,\
        0.97496, 0.97276, 0.97023, 0.96719, 0.96350, 0.95910, 0.95393, 0.94802, 0.94147, 0.93444, 0.92718,\
        0.91992, 0.91276, 0.90530, 0.89704, 0.88866, 0.88037, 0.87198, 0.86360, 0.85542, 0.84757, 0.84013,\
        0.83318, 0.82678, 0.82094, 0.81564, 0.81076, 0.80603, 0.80103, 0.79578, 0.79015, 0.78416, 0.77793,\
        0.77154, 0.76504, 0.75848, 0.75188, 0.74529, 0.73873, 0.73225, 0.72589, 0.71970, 0.71375, 0.70806,\
        0.70263, 0.69780, 0.69441, 0.69136, 0.68827, 0.68478, 0.68050, 0.67524, 0.66914, 0.66250, 0.65558,\
        0.64863, 0.64181, 0.63523, 0.62898, 0.62308, 0.61757, 0.61244, 0.60767, 0.60321, 0.59868, 0.59387],\
        [0.74959, 0.75375, 0.75790, 0.76203, 0.76614, 0.77021, 0.77423, 0.77819, 0.78210, 0.78610, 0.79050,\
        0.79596, 0.80404, 0.81476, 0.82627, 0.83780, 0.84878, 0.85872, 0.86732, 0.87457, 0.88077, 0.88633,\
        0.89160, 0.89685, 0.90217, 0.90997, 0.91797, 0.92552, 0.93273, 0.93969, 0.94642, 0.95293, 0.95923,\
        0.96531, 0.97122, 0.97699, 0.98271, 0.98846, 0.99428, 1.00016, 1.00591, 1.01122, 1.01557, 1.01838,\
        1.01914, 1.01763, 1.01400, 1.00878, 1.00269, 0.99643, 0.99056, 0.98539, 0.98097, 0.97719, 0.97381,\
        0.97062, 0.96737, 0.96388, 0.96001, 0.95566, 0.95077, 0.94528, 0.93916, 0.93246, 0.92536, 0.91815,\
        0.91111, 0.90449, 0.89793, 0.89022, 0.88218, 0.87416, 0.86615, 0.85819, 0.85038, 0.84283, 0.83565,\
        0.82892, 0.82274, 0.81717, 0.81216, 0.80766, 0.80352, 0.79888, 0.79348, 0.78773, 0.78165, 0.77532,\
        0.76881, 0.76216, 0.75544, 0.74867, 0.74192, 0.73523, 0.72864, 0.72218, 0.71591, 0.70993, 0.70435,\
        0.69900, 0.69401, 0.68982, 0.68614, 0.68251, 0.67860, 0.67414, 0.66901, 0.66325, 0.65704, 0.65058,\
        0.64407, 0.63766, 0.63144, 0.62546, 0.61976, 0.61438, 0.60934, 0.60457, 0.60006, 0.59569, 0.59102],\
        [0.74801, 0.75208, 0.75615, 0.76022, 0.76429, 0.76837, 0.77246, 0.77655, 0.78069, 0.78502, 0.78985,\
        0.79576, 0.80333, 0.81246, 0.82252, 0.83279, 0.84274, 0.85200, 0.86035, 0.86775, 0.87437, 0.88046,\
        0.88628, 0.89202, 0.89835, 0.90588, 0.91287, 0.91946, 0.92603, 0.93262, 0.93921, 0.94580, 0.95237,\
        0.95886, 0.96522, 0.97145, 0.97755, 0.98356, 0.98947, 0.99525, 1.00073, 1.00562, 1.00954, 1.01202,\
        1.01270, 1.01142, 1.00831, 1.00374, 0.99826, 0.99245, 0.98677, 0.98152, 0.97680, 0.97254, 0.96858,\
        0.96474, 0.96082, 0.95668, 0.95224, 0.94744, 0.94225, 0.93661, 0.93040, 0.92361, 0.91641, 0.90916,\
        0.90223, 0.89579, 0.88956, 0.88264, 0.87524, 0.86769, 0.86012, 0.85260, 0.84519, 0.83798, 0.83106,\
        0.82454, 0.81855, 0.81322, 0.80850, 0.80414, 0.80025, 0.79609, 0.79084, 0.78508, 0.77897, 0.77258,\
        0.76597, 0.75920, 0.75234, 0.74544, 0.73854, 0.73173, 0.72505, 0.71853, 0.71218, 0.70601, 0.70047,\
        0.69509, 0.68991, 0.68520, 0.68087, 0.67666, 0.67231, 0.66762, 0.66247, 0.65689, 0.65098, 0.64489,\
        0.63876, 0.63274, 0.62688, 0.62121, 0.61573, 0.61046, 0.60556, 0.60092, 0.59638, 0.59201, 0.58753],\
        [0.74634, 0.75034, 0.75436, 0.75842, 0.76254, 0.76673, 0.77101, 0.77541, 0.77997, 0.78481, 0.79012,\
        0.79622, 0.80319, 0.81099, 0.81954, 0.82844, 0.83730, 0.84583, 0.85383, 0.86126, 0.86818, 0.87479,\
        0.88132, 0.88811, 0.89553, 0.90253, 0.90770, 0.91313, 0.91904, 0.92520, 0.93153, 0.93804, 0.94471,\
        0.95140, 0.95797, 0.96433, 0.97045, 0.97632, 0.98195, 0.98730, 0.99228, 0.99665, 1.00010, 1.00227,\
        1.00287, 1.00178, 0.99907, 0.99505, 0.99016, 0.98488, 0.97963, 0.97467, 0.97008, 0.96582, 0.96172,\
        0.95763, 0.95337, 0.94885, 0.94403, 0.93893, 0.93361, 0.92802, 0.92197, 0.91520, 0.90790, 0.90068,\
        0.89393, 0.88762, 0.88141, 0.87493, 0.86810, 0.86109, 0.85403, 0.84698, 0.84001, 0.83318, 0.82655,\
        0.82023, 0.81438, 0.80925, 0.80500, 0.80101, 0.79695, 0.79280, 0.78785, 0.78223, 0.77616, 0.76974,\
        0.76307, 0.75621, 0.74923, 0.74219, 0.73517, 0.72822, 0.72146, 0.71494, 0.70869, 0.70236, 0.69640,\
        0.69092, 0.68551, 0.68036, 0.67547, 0.67073, 0.66594, 0.66097, 0.65574, 0.65023, 0.64450, 0.63866,\
        0.63285, 0.62717, 0.62169, 0.61638, 0.61115, 0.60596, 0.60107, 0.59671, 0.59230, 0.58795, 0.58359],\
        [0.74456, 0.74850, 0.75248, 0.75655, 0.76075, 0.76511, 0.76966, 0.77445, 0.77951, 0.78487, 0.79055,\
        0.79647, 0.80279, 0.80963, 0.81684, 0.82441, 0.83221, 0.84001, 0.84762, 0.85496, 0.86204, 0.86897,\
        0.87594, 0.88311, 0.89036, 0.89650, 0.90114, 0.90610, 0.91169, 0.91745, 0.92338, 0.92974, 0.93644,\
        0.94321, 0.94980, 0.95605, 0.96189, 0.96731, 0.97236, 0.97707, 0.98139, 0.98519, 0.98821, 0.99013,\
        0.99068, 0.98973, 0.98733, 0.98375, 0.97937, 0.97465, 0.96996, 0.96554, 0.96143, 0.95754, 0.95369,\
        0.94967, 0.94535, 0.94064, 0.93558, 0.93027, 0.92489, 0.91955, 0.91401, 0.90754, 0.90011, 0.89302,\
        0.88629, 0.87991, 0.87370, 0.86743, 0.86103, 0.85453, 0.84799, 0.84147, 0.83499, 0.82860, 0.82234,\
        0.81627, 0.81052, 0.80540, 0.80164, 0.79841, 0.79431, 0.78987, 0.78490, 0.77935, 0.77331, 0.76688,\
        0.76017, 0.75323, 0.74615, 0.73899, 0.73182, 0.72473, 0.71783, 0.71124, 0.70514, 0.69941, 0.69290,\
        0.68675, 0.68091, 0.67530, 0.66993, 0.66469, 0.65950, 0.65425, 0.64888, 0.64335, 0.63772, 0.63204,\
        0.62645, 0.62106, 0.61596, 0.61107, 0.60622, 0.60120, 0.59636, 0.59199, 0.58775, 0.58353, 0.57929],\
        [0.74263, 0.74649, 0.75043, 0.75451, 0.75878, 0.76330, 0.76812, 0.77327, 0.77881, 0.78471, 0.79086,\
        0.79690, 0.80260, 0.80823, 0.81400, 0.82032, 0.82718, 0.83432, 0.84154, 0.84870, 0.85577, 0.86278,\
        0.86978, 0.87675, 0.88337, 0.88906, 0.89388, 0.89878, 0.90424, 0.90948, 0.91484, 0.92108, 0.92784,\
        0.93464, 0.94111, 0.94705, 0.95239, 0.95712, 0.96136, 0.96522, 0.96880, 0.97204, 0.97471, 0.97648,\
        0.97704, 0.97622, 0.97405, 0.97077, 0.96679, 0.96257, 0.95852, 0.95483, 0.95149, 0.94829, 0.94498,\
        0.94132, 0.93716, 0.93243, 0.92718, 0.92164, 0.91615, 0.91100, 0.90626, 0.90066, 0.89323, 0.88593,\
        0.87906, 0.87256, 0.86632, 0.86022, 0.85416, 0.84813, 0.84213, 0.83617, 0.83025, 0.82438, 0.81860,\
        0.81295, 0.80754, 0.80253, 0.79880, 0.79572, 0.79177, 0.78722, 0.78215, 0.77657, 0.77052, 0.76408,\
        0.75732, 0.75031, 0.74313, 0.73586, 0.72857, 0.72133, 0.71421, 0.70736, 0.70095, 0.69500, 0.68874,\
        0.68236, 0.67613, 0.67009, 0.66425, 0.65857, 0.65300, 0.64748, 0.64195, 0.63638, 0.63076, 0.62516,\
        0.61968, 0.61449, 0.60969, 0.60524, 0.60082, 0.59614, 0.59144, 0.58706, 0.58289, 0.57879, 0.57470],\
        [0.74051, 0.74427, 0.74813, 0.75218, 0.75649, 0.76111, 0.76611, 0.77153, 0.77741, 0.78371, 0.79034,\
        0.79697, 0.80223, 0.80618, 0.81047, 0.81581, 0.82197, 0.82860, 0.83546, 0.84239, 0.84931, 0.85619,\
        0.86301, 0.86965, 0.87587, 0.88145, 0.88650, 0.89152, 0.89674, 0.90122, 0.90603, 0.91243, 0.91933,\
        0.92608, 0.93231, 0.93780, 0.94246, 0.94630, 0.94952, 0.95238, 0.95515, 0.95788, 0.96035, 0.96212,\
        0.96280, 0.96212, 0.96009, 0.95697, 0.95322, 0.94941, 0.94600, 0.94320, 0.94086, 0.93863, 0.93612,\
        0.93305, 0.92923, 0.92461, 0.91925, 0.91341, 0.90765, 0.90243, 0.89788, 0.89285, 0.88601, 0.87882,\
        0.87190, 0.86536, 0.85916, 0.85324, 0.84752, 0.84195, 0.83651, 0.83115, 0.82585, 0.82059, 0.81539,\
        0.81031, 0.80551, 0.80134, 0.79770, 0.79372, 0.78948, 0.78478, 0.77959, 0.77394, 0.76785, 0.76137,\
        0.75457, 0.74751, 0.74025, 0.73287, 0.72547, 0.71812, 0.71085, 0.70368, 0.69703, 0.69074, 0.68434,\
        0.67777, 0.67121, 0.66477, 0.65849, 0.65240, 0.64647, 0.64068, 0.63500, 0.62938, 0.62377, 0.61817,\
        0.61270, 0.60758, 0.60297, 0.59883, 0.59469, 0.59036, 0.58600, 0.58179, 0.57774, 0.57379, 0.56986],\
        [0.73817, 0.74178, 0.74552, 0.74947, 0.75373, 0.75836, 0.76343, 0.76896, 0.77495, 0.78134, 0.78795,\
        0.79429, 0.79952, 0.80230, 0.80581, 0.81072, 0.81650, 0.82281, 0.82935, 0.83600, 0.84268, 0.84934,\
        0.85591, 0.86227, 0.86826, 0.87379, 0.87896, 0.88395, 0.88875, 0.89313, 0.89810, 0.90448, 0.91135,\
        0.91789, 0.92376, 0.92871, 0.93257, 0.93540, 0.93743, 0.93911, 0.94100, 0.94335, 0.94585, 0.94786,\
        0.94878, 0.94827, 0.94631, 0.94318, 0.93944, 0.93585, 0.93306, 0.93130, 0.93022, 0.92919, 0.92766,\
        0.92530, 0.92195, 0.91759, 0.91224, 0.90622, 0.89997, 0.89485, 0.88987, 0.88453, 0.87822, 0.87141,\
        0.86465, 0.85819, 0.85212, 0.84643, 0.84108, 0.83601, 0.83115, 0.82644, 0.82181, 0.81722, 0.81265,\
        0.80814, 0.80379, 0.79976, 0.79595, 0.79186, 0.78740, 0.78253, 0.77722, 0.77148, 0.76533, 0.75881,\
        0.75197, 0.74485, 0.73752, 0.73006, 0.72256, 0.71517, 0.70800, 0.70081, 0.69379, 0.68698, 0.68015,\
        0.67323, 0.66630, 0.65944, 0.65273, 0.64621, 0.63994, 0.63390, 0.62808, 0.62243, 0.61686, 0.61125,\
        0.60573, 0.60066, 0.59614, 0.59211, 0.58816, 0.58415, 0.58012, 0.57617, 0.57232, 0.56855, 0.56482],\
        [0.73559, 0.73899, 0.74253, 0.74632, 0.75044, 0.75497, 0.75996, 0.76542, 0.77131, 0.77750, 0.78372,\
        0.78948, 0.79402, 0.79698, 0.80057, 0.80532, 0.81092, 0.81701, 0.82325, 0.82957, 0.83596, 0.84237,\
        0.84869, 0.85483, 0.86067, 0.86618, 0.87139, 0.87639, 0.88120, 0.88600, 0.89132, 0.89752, 0.90415,\
        0.91015, 0.91567, 0.92005, 0.92312, 0.92490, 0.92564, 0.92597, 0.92689, 0.92907, 0.93195, 0.93443,\
        0.93574, 0.93546, 0.93351, 0.93021, 0.92625, 0.92262, 0.92036, 0.91983, 0.92025, 0.92055, 0.92005,\
        0.91843, 0.91558, 0.91161, 0.90648, 0.90035, 0.89421, 0.88833, 0.88269, 0.87683, 0.87052, 0.86391,\
        0.85732, 0.85102, 0.84514, 0.83974, 0.83480, 0.83028, 0.82606, 0.82205, 0.81813, 0.81423, 0.81028,\
        0.80629, 0.80230, 0.79833, 0.79433, 0.79009, 0.78546, 0.78043, 0.77499, 0.76916, 0.76295, 0.75639,\
        0.74952, 0.74238, 0.73501, 0.72746, 0.71983, 0.71229, 0.70520, 0.69846, 0.69096, 0.68357, 0.67624,\
        0.66888, 0.66150, 0.65419, 0.64702, 0.64008, 0.63345, 0.62715, 0.62119, 0.61555, 0.61012, 0.60462,\
        0.59901, 0.59399, 0.58952, 0.58544, 0.58157, 0.57777, 0.57400, 0.57030, 0.56669, 0.56314, 0.55962],\
        [0.73275, 0.73588, 0.73916, 0.74268, 0.74657, 0.75089, 0.75570, 0.76097, 0.76662, 0.77249, 0.77826,\
        0.78354, 0.78793, 0.79157, 0.79539, 0.79994, 0.80549, 0.81135, 0.81718, 0.82313, 0.82922, 0.83538,\
        0.84150, 0.84745, 0.85319, 0.85867, 0.86393, 0.86903, 0.87405, 0.87921, 0.88480, 0.89092, 0.89707,\
        0.90270, 0.90820, 0.91189, 0.91428, 0.91521, 0.91476, 0.91364, 0.91340, 0.91582, 0.91948, 0.92256,\
        0.92434, 0.92434, 0.92239, 0.91885, 0.91452, 0.91060, 0.90867, 0.90964, 0.91173, 0.91327, 0.91368,\
        0.91266, 0.91012, 0.90642, 0.90213, 0.89583, 0.88909, 0.88252, 0.87611, 0.86970, 0.86316, 0.85654,\
        0.85005, 0.84388, 0.83822, 0.83315, 0.82868, 0.82476, 0.82123, 0.81796, 0.81478, 0.81156, 0.80823,\
        0.80470, 0.80098, 0.79705, 0.79290, 0.78845, 0.78364, 0.77844, 0.77287, 0.76695, 0.76068, 0.75410,\
        0.74723, 0.74010, 0.73273, 0.72516, 0.71743, 0.70959, 0.70183, 0.69504, 0.68787, 0.68030, 0.67257,\
        0.66476, 0.65692, 0.64913, 0.64147, 0.63407, 0.62705, 0.62049, 0.61437, 0.60865, 0.60340, 0.59815,\
        0.59262, 0.58754, 0.58299, 0.57887, 0.57502, 0.57134, 0.56777, 0.56430, 0.56093, 0.55761, 0.55433],\
        [0.72967, 0.73247, 0.73539, 0.73857, 0.74213, 0.74616, 0.75071, 0.75573, 0.76112, 0.76668, 0.77216,\
        0.77726, 0.78181, 0.78597, 0.79021, 0.79497, 0.80042, 0.80585, 0.81110, 0.81667, 0.82252, 0.82849,\
        0.83442, 0.84023, 0.84587, 0.85133, 0.85663, 0.86183, 0.86704, 0.87241, 0.87807, 0.88402, 0.89000,\
        0.89603, 0.90090, 0.90373, 0.90599, 0.90644, 0.90525, 0.90330, 0.90156, 0.90495, 0.90928, 0.91264,\
        0.91489, 0.91528, 0.91339, 0.90967, 0.90518, 0.90117, 0.89948, 0.90205, 0.90532, 0.90758, 0.90865,\
        0.90810, 0.90561, 0.90143, 0.89693, 0.89101, 0.88394, 0.87681, 0.86981, 0.86293, 0.85611, 0.84940,\
        0.84292, 0.83687, 0.83141, 0.82669, 0.82273, 0.81946, 0.81668, 0.81418, 0.81175, 0.80920, 0.80642,\
        0.80332, 0.79979, 0.79585, 0.79155, 0.78688, 0.78187, 0.77650, 0.77080, 0.76478, 0.75847, 0.75189,\
        0.74505, 0.73798, 0.73069, 0.72320, 0.71554, 0.70781, 0.70017, 0.69264, 0.68511, 0.67727, 0.66918,\
        0.66094, 0.65264, 0.64435, 0.63619, 0.62829, 0.62083, 0.61399, 0.60774, 0.60182, 0.59648, 0.59132,\
        0.58607, 0.58107, 0.57651, 0.57235, 0.56852, 0.56493, 0.56152, 0.55826, 0.55511, 0.55203, 0.54899],\
        [0.72637, 0.72878, 0.73127, 0.73402, 0.73718, 0.74085, 0.74510, 0.74987, 0.75504, 0.76041, 0.76576,\
        0.77088, 0.77568, 0.78026, 0.78489, 0.78983, 0.79508, 0.80000, 0.80486, 0.81023, 0.81595, 0.82179,\
        0.82756, 0.83322, 0.83875, 0.84416, 0.84946, 0.85471, 0.85999, 0.86540, 0.87100, 0.87677, 0.88257,\
        0.88808, 0.89237, 0.89532, 0.89768, 0.89799, 0.89679, 0.89521, 0.89454, 0.89761, 0.90115, 0.90440,\
        0.90728, 0.90821, 0.90642, 0.90271, 0.89854, 0.89545, 0.89483, 0.89734, 0.90065, 0.90304, 0.90473,\
        0.90470, 0.90205, 0.89749, 0.89189, 0.88548, 0.87829, 0.87086, 0.86351, 0.85632, 0.84931, 0.84251,\
        0.83604, 0.83006, 0.82478, 0.82039, 0.81698, 0.81443, 0.81246, 0.81076, 0.80905, 0.80711, 0.80481,\
        0.80205, 0.79868, 0.79465, 0.79016, 0.78529, 0.78006, 0.77452, 0.76869, 0.76259, 0.75625, 0.74969,\
        0.74292, 0.73596, 0.72881, 0.72148, 0.71398, 0.70639, 0.69881, 0.69091, 0.68287, 0.67461, 0.66613,\
        0.65747, 0.64871, 0.63996, 0.63131, 0.62289, 0.61489, 0.60770, 0.60147, 0.59549, 0.58990, 0.58462,\
        0.57949, 0.57459, 0.57004, 0.56589, 0.56208, 0.55857, 0.55531, 0.55223, 0.54930, 0.54646, 0.54367],\
        [0.72291, 0.72485, 0.72684, 0.72907, 0.73176, 0.73504, 0.73899, 0.74355, 0.74859, 0.75390, 0.75926,\
        0.76451, 0.76956, 0.77446, 0.77934, 0.78430, 0.78925, 0.79399, 0.79879, 0.80403, 0.80971, 0.81541,\
        0.82097, 0.82645, 0.83185, 0.83717, 0.84242, 0.84763, 0.85286, 0.85816, 0.86357, 0.86904, 0.87440,\
        0.87933, 0.88337, 0.88643, 0.88850, 0.88916, 0.88873, 0.88817, 0.88865, 0.89097, 0.89406, 0.89736,\
        0.90089, 0.90227, 0.90049, 0.89710, 0.89364, 0.89142, 0.89130, 0.89330, 0.89618, 0.89887, 0.90139,\
        0.90225, 0.89868, 0.89325, 0.88672, 0.87960, 0.87212, 0.86454, 0.85707, 0.84979, 0.84274, 0.83595,\
        0.82952, 0.82360, 0.81844, 0.81435, 0.81151, 0.80976, 0.80866, 0.80776, 0.80673, 0.80531, 0.80335,\
        0.80073, 0.79737, 0.79328, 0.78863, 0.78355, 0.77812, 0.77240, 0.76644, 0.76028, 0.75393, 0.74741,\
        0.74074, 0.73393, 0.72698, 0.71987, 0.71255, 0.70496, 0.69717, 0.68922, 0.68090, 0.67228, 0.66341,\
        0.65437, 0.64521, 0.63604, 0.62698, 0.61813, 0.60960, 0.60176, 0.59541, 0.58946, 0.58374, 0.57830,\
        0.57310, 0.56820, 0.56365, 0.55950, 0.55572, 0.55229, 0.54916, 0.54627, 0.54356, 0.54096, 0.53842],\
        [0.71933, 0.72076, 0.72216, 0.72380, 0.72596, 0.72883, 0.73250, 0.73693, 0.74194, 0.74730, 0.75278,\
        0.75821, 0.76350, 0.76865, 0.77370, 0.77869, 0.78359, 0.78838, 0.79324, 0.79840, 0.80404, 0.80944,\
        0.81467, 0.81994, 0.82519, 0.83039, 0.83552, 0.84059, 0.84564, 0.85071, 0.85582, 0.86090, 0.86580,\
        0.87029, 0.87410, 0.87710, 0.87919, 0.88031, 0.88075, 0.88117, 0.88229, 0.88445, 0.88736, 0.89065,\
        0.89376, 0.89513, 0.89392, 0.89132, 0.88877, 0.88725, 0.88733, 0.88896, 0.89153, 0.89443, 0.89742,\
        0.89846, 0.89425, 0.88773, 0.88052, 0.87298, 0.86534, 0.85779, 0.85042, 0.84329, 0.83642, 0.82981,\
        0.82352, 0.81770, 0.81262, 0.80873, 0.80649, 0.80566, 0.80544, 0.80528, 0.80485, 0.80384, 0.80205,\
        0.79945, 0.79598, 0.79173, 0.78686, 0.78155, 0.77591, 0.77004, 0.76397, 0.75776, 0.75141, 0.74496,\
        0.73842, 0.73178, 0.72505, 0.71820, 0.71118, 0.70382, 0.69600, 0.68776, 0.67913, 0.67018, 0.66100,\
        0.65164, 0.64215, 0.63264, 0.62325, 0.61418, 0.60563, 0.59748, 0.59041, 0.58406, 0.57806, 0.57238,\
        0.56702, 0.56201, 0.55740, 0.55322, 0.54947, 0.54612, 0.54312, 0.54041, 0.53793, 0.53558, 0.53330],\
        [0.71570, 0.71658, 0.71732, 0.71829, 0.71986, 0.72231, 0.72576, 0.73015, 0.73526, 0.74077, 0.74644,\
        0.75207, 0.75758, 0.76292, 0.76811, 0.77319, 0.77816, 0.78308, 0.78809, 0.79335, 0.79881, 0.80367,\
        0.80859, 0.81372, 0.81882, 0.82385, 0.82880, 0.83364, 0.83840, 0.84314, 0.84787, 0.85253, 0.85700,\
        0.86112, 0.86473, 0.86769, 0.86995, 0.87154, 0.87269, 0.87384, 0.87540, 0.87758, 0.88027, 0.88314,\
        0.88558, 0.88674, 0.88624, 0.88467, 0.88303, 0.88211, 0.88231, 0.88365, 0.88578, 0.88820, 0.89012,\
        0.88989, 0.88615, 0.87995, 0.87275, 0.86526, 0.85780, 0.85055, 0.84357, 0.83687, 0.83042, 0.82421,\
        0.81826, 0.81269, 0.80773, 0.80387, 0.80227, 0.80246, 0.80296, 0.80336, 0.80342, 0.80279, 0.80106,\
        0.79825, 0.79450, 0.78992, 0.78474, 0.77917, 0.77333, 0.76732, 0.76118, 0.75494, 0.74862, 0.74225,\
        0.73584, 0.72938, 0.72286, 0.71625, 0.70955, 0.70265, 0.69482, 0.68627, 0.67738, 0.66822, 0.65883,\
        0.64924, 0.63952, 0.62975, 0.62005, 0.61069, 0.60215, 0.59405, 0.58635, 0.57938, 0.57293, 0.56690,\
        0.56127, 0.55607, 0.55133, 0.54709, 0.54335, 0.54008, 0.53722, 0.53471, 0.53246, 0.53037, 0.52836],\
        [0.71209, 0.71238, 0.71240, 0.71265, 0.71360, 0.71561, 0.71891, 0.72341, 0.72872, 0.73448, 0.74037,\
        0.74620, 0.75187, 0.75736, 0.76267, 0.76784, 0.77291, 0.77795, 0.78304, 0.78821, 0.79328, 0.79795,\
        0.80291, 0.80795, 0.81277, 0.81763, 0.82235, 0.82685, 0.83121, 0.83553, 0.83984, 0.84408, 0.84815,\
        0.85197, 0.85540, 0.85838, 0.86083, 0.86282, 0.86450, 0.86614, 0.86799, 0.87017, 0.87259, 0.87499,\
        0.87694, 0.87796, 0.87791, 0.87713, 0.87622, 0.87574, 0.87600, 0.87702, 0.87852, 0.87999, 0.88064,\
        0.87945, 0.87581, 0.87018, 0.86347, 0.85643, 0.84949, 0.84286, 0.83657, 0.83059, 0.82483, 0.81925,\
        0.81389, 0.80885, 0.80437, 0.80084, 0.79973, 0.80047, 0.80122, 0.80188, 0.80228, 0.80205, 0.80043,\
        0.79714, 0.79276, 0.78767, 0.78211, 0.77626, 0.77025, 0.76414, 0.75797, 0.75174, 0.74549, 0.73921,\
        0.73293, 0.72665, 0.72031, 0.71387, 0.70722, 0.70030, 0.69275, 0.68436, 0.67549, 0.66629, 0.65682,\
        0.64713, 0.63730, 0.62738, 0.61748, 0.60769, 0.59845, 0.59044, 0.58262, 0.57521, 0.56828, 0.56182,\
        0.55585, 0.55039, 0.54548, 0.54114, 0.53739, 0.53420, 0.53150, 0.52920, 0.52720, 0.52539, 0.52365],\
        [0.70851, 0.70826, 0.70750, 0.70703, 0.70738, 0.70893, 0.71218, 0.71694, 0.72257, 0.72859, 0.73468,\
        0.74066, 0.74644, 0.75200, 0.75737, 0.76261, 0.76776, 0.77287, 0.77800, 0.78310, 0.78809, 0.79296,\
        0.79795, 0.80260, 0.80705, 0.81188, 0.81630, 0.82028, 0.82411, 0.82798, 0.83185, 0.83568, 0.83938,\
        0.84291, 0.84622, 0.84922, 0.85186, 0.85415, 0.85619, 0.85814, 0.86015, 0.86226, 0.86443, 0.86644,\
        0.86803, 0.86895, 0.86916, 0.86886, 0.86842, 0.86822, 0.86846, 0.86912, 0.86995, 0.87049, 0.87012,\
        0.86822, 0.86450, 0.85926, 0.85312, 0.84674, 0.84058, 0.83484, 0.82954, 0.82454, 0.81971, 0.81499,\
        0.81040, 0.80611, 0.80245, 0.79992, 0.79888, 0.79920, 0.79991, 0.80055, 0.80111, 0.80127, 0.79927,\
        0.79534, 0.79028, 0.78465, 0.77873, 0.77267, 0.76655, 0.76042, 0.75428, 0.74812, 0.74196, 0.73580,\
        0.72966, 0.72355, 0.71742, 0.71114, 0.70452, 0.69753, 0.69016, 0.68209, 0.67342, 0.66433, 0.65491,\
        0.64524, 0.63541, 0.62549, 0.61560, 0.60591, 0.59654, 0.58768, 0.57934, 0.57144, 0.56402, 0.55711,\
        0.55075, 0.54498, 0.53984, 0.53539, 0.53162, 0.52851, 0.52598, 0.52393, 0.52221, 0.52068, 0.51923],\
        [0.70487, 0.70420, 0.70271, 0.70165, 0.70161, 0.70259, 0.70598, 0.71114, 0.71704, 0.72326, 0.72948,\
        0.73550, 0.74128, 0.74682, 0.75219, 0.75744, 0.76262, 0.76779, 0.77295, 0.77808, 0.78315, 0.78815,\
        0.79304, 0.79743, 0.80201, 0.80696, 0.81072, 0.81390, 0.81711, 0.82054, 0.82403, 0.82744, 0.83077,\
        0.83404, 0.83724, 0.84028, 0.84307, 0.84557, 0.84783, 0.84994, 0.85199, 0.85401, 0.85593, 0.85764,\
        0.85895, 0.85976, 0.86006, 0.86000, 0.85982, 0.85974, 0.85989, 0.86023, 0.86050, 0.86032, 0.85921,\
        0.85679, 0.85293, 0.84790, 0.84223, 0.83654, 0.83130, 0.82670, 0.82263, 0.81885, 0.81514, 0.81140,\
        0.80766, 0.80412, 0.80111, 0.79903, 0.79805, 0.79802, 0.79851, 0.79906, 0.79997, 0.80021, 0.79700,\
        0.79219, 0.78655, 0.78052, 0.77437, 0.76822, 0.76213, 0.75609, 0.75008, 0.74407, 0.73804, 0.73200,\
        0.72600, 0.72007, 0.71420, 0.70821, 0.70187, 0.69498, 0.68762, 0.67971, 0.67125, 0.66234, 0.65306,\
        0.64350, 0.63374, 0.62388, 0.61404, 0.60437, 0.59478, 0.58538, 0.57646, 0.56803, 0.56010, 0.55271,\
        0.54594, 0.53982, 0.53444, 0.52984, 0.52605, 0.52304, 0.52072, 0.51894, 0.51754, 0.51633, 0.51517],\
        [0.70075, 0.69984, 0.69819, 0.69685, 0.69664, 0.69763, 0.70121, 0.70634, 0.71227, 0.71857, 0.72478,\
        0.73070, 0.73634, 0.74175, 0.74703, 0.75224, 0.75743, 0.76263, 0.76784, 0.77304, 0.77822, 0.78333,\
        0.78835, 0.79327, 0.79853, 0.80259, 0.80534, 0.80752, 0.81016, 0.81337, 0.81652, 0.81951, 0.82242,\
        0.82541, 0.82854, 0.83165, 0.83455, 0.83716, 0.83950, 0.84165, 0.84366, 0.84555, 0.84725, 0.84869,\
        0.84977, 0.85044, 0.85072, 0.85073, 0.85062, 0.85056, 0.85061, 0.85071, 0.85062, 0.84997, 0.84839,\
        0.84561, 0.84162, 0.83667, 0.83131, 0.82623, 0.82198, 0.81869, 0.81608, 0.81369, 0.81119, 0.80846,\
        0.80551, 0.80255, 0.79994, 0.79800, 0.79685, 0.79638, 0.79642, 0.79675, 0.79723, 0.79635, 0.79252,\
        0.78721, 0.78125, 0.77506, 0.76888, 0.76283, 0.75693, 0.75114, 0.74540, 0.73962, 0.73379, 0.72788,\
        0.72198, 0.71620, 0.71062, 0.70500, 0.69895, 0.69228, 0.68503, 0.67728, 0.66903, 0.66033, 0.65124,\
        0.64183, 0.63218, 0.62236, 0.61248, 0.60261, 0.59280, 0.58313, 0.57378, 0.56486, 0.55644, 0.54860,\
        0.54140, 0.53493, 0.52928, 0.52452, 0.52071, 0.51783, 0.51576, 0.51432, 0.51326, 0.51239, 0.51152],\
        [0.69513, 0.69448, 0.69327, 0.69235, 0.69244, 0.69402, 0.69747, 0.70226, 0.70817, 0.71449, 0.72050,\
        0.72613, 0.73147, 0.73666, 0.74178, 0.74690, 0.75208, 0.75730, 0.76257, 0.76786, 0.77313, 0.77837,\
        0.78354, 0.78865, 0.79349, 0.79721, 0.79963, 0.80101, 0.80347, 0.80672, 0.80947, 0.81198, 0.81443,\
        0.81710, 0.82023, 0.82346, 0.82642, 0.82900, 0.83129, 0.83336, 0.83527, 0.83700, 0.83849, 0.83969,\
        0.84054, 0.84104, 0.84122, 0.84118, 0.84104, 0.84092, 0.84090, 0.84089, 0.84066, 0.83982, 0.83804,\
        0.83511, 0.83103, 0.82610, 0.82090, 0.81628, 0.81299, 0.81118, 0.81018, 0.80922, 0.80792, 0.80612,\
        0.80380, 0.80119, 0.79871, 0.79662, 0.79502, 0.79391, 0.79317, 0.79260, 0.79172, 0.78959, 0.78559,\
        0.78025, 0.77430, 0.76820, 0.76223, 0.75649, 0.75097, 0.74561, 0.74027, 0.73486, 0.72930, 0.72356,\
        0.71770, 0.71192, 0.70661, 0.70161, 0.69573, 0.68927, 0.68226, 0.67474, 0.66673, 0.65826, 0.64940,\
        0.64017, 0.63064, 0.62088, 0.61093, 0.60086, 0.59085, 0.58092, 0.57121, 0.56188, 0.55302, 0.54475,\
        0.53715, 0.53033, 0.52439, 0.51946, 0.51563, 0.51292, 0.51119, 0.51015, 0.50949, 0.50895, 0.50836],\
        [0.68854, 0.68830, 0.68773, 0.68749, 0.68821, 0.69028, 0.69383, 0.69857, 0.70477, 0.71086, 0.71638,\
        0.72151, 0.72644, 0.73133, 0.73626, 0.74130, 0.74646, 0.75172, 0.75706, 0.76243, 0.76778, 0.77306,\
        0.77822, 0.78314, 0.78754, 0.79100, 0.79339, 0.79514, 0.79771, 0.80066, 0.80283, 0.80495, 0.80694,\
        0.80919, 0.81253, 0.81595, 0.81878, 0.82115, 0.82324, 0.82514, 0.82690, 0.82846, 0.82975, 0.83072,\
        0.83134, 0.83165, 0.83168, 0.83152, 0.83126, 0.83104, 0.83097, 0.83103, 0.83092, 0.83019, 0.82848,\
        0.82560, 0.82159, 0.81674, 0.81163, 0.80729, 0.80485, 0.80467, 0.80530, 0.80556, 0.80530, 0.80435,\
        0.80244, 0.79986, 0.79721, 0.79467, 0.79239, 0.79042, 0.78865, 0.78687, 0.78465, 0.78148, 0.77708,\
        0.77173, 0.76591, 0.76007, 0.75447, 0.74924, 0.74432, 0.73958, 0.73482, 0.72990, 0.72472, 0.71924,\
        0.71347, 0.70751, 0.70192, 0.69749, 0.69216, 0.68601, 0.67930, 0.67206, 0.66432, 0.65612, 0.64749,\
        0.63847, 0.62910, 0.61940, 0.60944, 0.59928, 0.58901, 0.57879, 0.56875, 0.55904, 0.54981, 0.54115,\
        0.53320, 0.52606, 0.51985, 0.51473, 0.51088, 0.50840, 0.50711, 0.50655, 0.50631, 0.50609, 0.50573],\
        [0.68146, 0.68155, 0.68156, 0.68198, 0.68333, 0.68594, 0.68998, 0.69545, 0.70166, 0.70701, 0.71185,\
        0.71641, 0.72093, 0.72554, 0.73032, 0.73529, 0.74047, 0.74581, 0.75124, 0.75669, 0.76209, 0.76737,\
        0.77243, 0.77713, 0.78126, 0.78461, 0.78722, 0.78946, 0.79192, 0.79438, 0.79648, 0.79849, 0.80028,\
        0.80189, 0.80605, 0.80933, 0.81166, 0.81357, 0.81533, 0.81701, 0.81859, 0.82000, 0.82112, 0.82186,\
        0.82223, 0.82233, 0.82221, 0.82191, 0.82149, 0.82111, 0.82102, 0.82133, 0.82161, 0.82126, 0.81987,\
        0.81728, 0.81357, 0.80911, 0.80443, 0.80038, 0.79880, 0.79988, 0.80163, 0.80253, 0.80306, 0.80301,\
        0.80134, 0.79846, 0.79523, 0.79197, 0.78885, 0.78589, 0.78302, 0.78003, 0.77662, 0.77252, 0.76763,\
        0.76214, 0.75642, 0.75088, 0.74579, 0.74123, 0.73711, 0.73317, 0.72915, 0.72484, 0.72016, 0.71508,\
        0.70963, 0.70386, 0.69816, 0.69337, 0.68834, 0.68258, 0.67619, 0.66924, 0.66179, 0.65386, 0.64547,\
        0.63667, 0.62748, 0.61789, 0.60793, 0.59765, 0.58719, 0.57670, 0.56635, 0.55633, 0.54678, 0.53783,\
        0.52960, 0.52220, 0.51578, 0.51050, 0.50660, 0.50440, 0.50370, 0.50371, 0.50385, 0.50387, 0.50368],\
        [0.67402, 0.67433, 0.67474, 0.67565, 0.67747, 0.68050, 0.68485, 0.69040, 0.69635, 0.70163, 0.70616,\
        0.71034, 0.71463, 0.71909, 0.72379, 0.72879, 0.73406, 0.73952, 0.74507, 0.75062, 0.75607, 0.76134,\
        0.76631, 0.77086, 0.77485, 0.77820, 0.78099, 0.78346, 0.78585, 0.78815, 0.79030, 0.79238, 0.79441,\
        0.79715, 0.80093, 0.80330, 0.80473, 0.80613, 0.80755, 0.80897, 0.81038, 0.81168, 0.81265, 0.81314,\
        0.81325, 0.81315, 0.81292, 0.81252, 0.81195, 0.81136, 0.81117, 0.81195, 0.81289, 0.81308, 0.81218,\
        0.81008, 0.80692, 0.80299, 0.79992, 0.79667, 0.79548, 0.79649, 0.79839, 0.79948, 0.80079, 0.80168,\
        0.79993, 0.79639, 0.79237, 0.78830, 0.78431, 0.78040, 0.77649, 0.77241, 0.76798, 0.76306, 0.75764,\
        0.75191, 0.74620, 0.74092, 0.73638, 0.73265, 0.72951, 0.72657, 0.72342, 0.71979, 0.71563, 0.71103,\
        0.70605, 0.70078, 0.69528, 0.69002, 0.68477, 0.67913, 0.67297, 0.66629, 0.65911, 0.65143, 0.64329,\
        0.63471, 0.62570, 0.61627, 0.60636, 0.59599, 0.58533, 0.57458, 0.56397, 0.55369, 0.54391, 0.53476,\
        0.52637, 0.51886, 0.51235, 0.50701, 0.50308, 0.50117, 0.50131, 0.50180, 0.50217, 0.50232, 0.50219],\
        [0.66629, 0.66667, 0.66728, 0.66845, 0.67050, 0.67367, 0.67799, 0.68324, 0.68884, 0.69415, 0.69876,\
        0.70300, 0.70739, 0.71188, 0.71661, 0.72173, 0.72721, 0.73287, 0.73858, 0.74424, 0.74976, 0.75504,\
        0.75997, 0.76444, 0.76836, 0.77173, 0.77462, 0.77719, 0.77958, 0.78187, 0.78410, 0.78636, 0.78881,\
        0.79175, 0.79476, 0.79666, 0.79769, 0.79875, 0.79988, 0.80106, 0.80229, 0.80349, 0.80435, 0.80459,\
        0.80439, 0.80413, 0.80386, 0.80345, 0.80283, 0.80217, 0.80164, 0.80319, 0.80481, 0.80544, 0.80509,\
        0.80355, 0.80113, 0.79812, 0.79533, 0.79333, 0.79249, 0.79317, 0.79465, 0.79614, 0.79775, 0.79857,\
        0.79651, 0.79262, 0.78811, 0.78342, 0.77872, 0.77401, 0.76924, 0.76428, 0.75902, 0.75340, 0.74746,\
        0.74142, 0.73562, 0.73050, 0.72649, 0.72370, 0.72174, 0.71997, 0.71776, 0.71479, 0.71105, 0.70683,\
        0.70228, 0.69726, 0.69196, 0.68663, 0.68126, 0.67564, 0.66964, 0.66318, 0.65624, 0.64880, 0.64088,\
        0.63248, 0.62364, 0.61439, 0.60466, 0.59421, 0.58329, 0.57230, 0.56150, 0.55107, 0.54117, 0.53194,\
        0.52353, 0.51606, 0.50968, 0.50455, 0.50100, 0.49958, 0.50021, 0.50080, 0.50119, 0.50133, 0.50118],\
        [0.65835, 0.65864, 0.65924, 0.66042, 0.66247, 0.66555, 0.66965, 0.67456, 0.67985, 0.68507, 0.68993,\
        0.69458, 0.69926, 0.70391, 0.70876, 0.71417, 0.71999, 0.72591, 0.73180, 0.73759, 0.74320, 0.74853,\
        0.75347, 0.75791, 0.76180, 0.76516, 0.76808, 0.77068, 0.77309, 0.77541, 0.77770, 0.78006, 0.78258,\
        0.78523, 0.78766, 0.78940, 0.79052, 0.79146, 0.79240, 0.79337, 0.79439, 0.79537, 0.79603, 0.79609,\
        0.79575, 0.79537, 0.79504, 0.79464, 0.79411, 0.79365, 0.79368, 0.79512, 0.79692, 0.79778, 0.79788,\
        0.79700, 0.79507, 0.79292, 0.79080, 0.78934, 0.78879, 0.78923, 0.79032, 0.79158, 0.79263, 0.79259,\
        0.79055, 0.78685, 0.78226, 0.77728, 0.77213, 0.76687, 0.76148, 0.75588, 0.75000, 0.74382, 0.73745,\
        0.73110, 0.72512, 0.71999, 0.71642, 0.71466, 0.71407, 0.71358, 0.71234, 0.70991, 0.70634, 0.70222,\
        0.69797, 0.69317, 0.68807, 0.68284, 0.67750, 0.67196, 0.66611, 0.65985, 0.65314, 0.64593, 0.63819,\
        0.62991, 0.62114, 0.61193, 0.60227, 0.59180, 0.58080, 0.56974, 0.55887, 0.54839, 0.53848, 0.52930,\
        0.52101, 0.51376, 0.50770, 0.50302, 0.50001, 0.49913, 0.49963, 0.50028, 0.50065, 0.50075, 0.50056],\
        [0.65031, 0.65037, 0.65075, 0.65174, 0.65358, 0.65642, 0.66023, 0.66485, 0.66994, 0.67516, 0.68030,\
        0.68535, 0.69035, 0.69531, 0.70044, 0.70633, 0.71256, 0.71871, 0.72476, 0.73069, 0.73643, 0.74187,\
        0.74686, 0.75129, 0.75514, 0.75846, 0.76135, 0.76394, 0.76634, 0.76866, 0.77097, 0.77333, 0.77575,\
        0.77814, 0.78027, 0.78193, 0.78314, 0.78410, 0.78496, 0.78579, 0.78659, 0.78727, 0.78766, 0.78763,\
        0.78730, 0.78687, 0.78646, 0.78606, 0.78570, 0.78557, 0.78600, 0.78721, 0.78870, 0.78973, 0.79011,\
        0.78968, 0.78851, 0.78704, 0.78561, 0.78460, 0.78420, 0.78443, 0.78507, 0.78577, 0.78605, 0.78531,\
        0.78312, 0.77957, 0.77508, 0.77004, 0.76470, 0.75915, 0.75341, 0.74743, 0.74116, 0.73463, 0.72797,\
        0.72142, 0.71529, 0.71001, 0.70662, 0.70599, 0.70678, 0.70752, 0.70713, 0.70519, 0.70152, 0.69734,\
        0.69306, 0.68846, 0.68358, 0.67852, 0.67332, 0.66793, 0.66227, 0.65624, 0.64976, 0.64277, 0.63520,\
        0.62702, 0.61821, 0.60883, 0.59897, 0.58862, 0.57780, 0.56683, 0.55602, 0.54561, 0.53579, 0.52675,\
        0.51869, 0.51177, 0.50618, 0.50208, 0.49962, 0.49883, 0.49918, 0.49985, 0.50029, 0.50041, 0.50018],\
        [0.64232, 0.64201, 0.64202, 0.64264, 0.64412, 0.64661, 0.65014, 0.65454, 0.65955, 0.66487, 0.67029,\
        0.67571, 0.68111, 0.68655, 0.69225, 0.69857, 0.70499, 0.71128, 0.71746, 0.72354, 0.72948, 0.73510,\
        0.74017, 0.74458, 0.74836, 0.75159, 0.75440, 0.75692, 0.75929, 0.76158, 0.76387, 0.76619, 0.76852,\
        0.77075, 0.77274, 0.77438, 0.77566, 0.77667, 0.77751, 0.77823, 0.77884, 0.77928, 0.77945, 0.77933,\
        0.77897, 0.77852, 0.77808, 0.77771, 0.77749, 0.77758, 0.77814, 0.77919, 0.78043, 0.78147, 0.78204,\
        0.78202, 0.78146, 0.78061, 0.77973, 0.77906, 0.77874, 0.77876, 0.77895, 0.77902, 0.77861, 0.77732,\
        0.77489, 0.77137, 0.76697, 0.76200, 0.75666, 0.75106, 0.74523, 0.73914, 0.73275, 0.72609, 0.71931,\
        0.71275, 0.70683, 0.70164, 0.69838, 0.69851, 0.70002, 0.70162, 0.70151, 0.70013, 0.69649, 0.69215,\
        0.68774, 0.68320, 0.67848, 0.67363, 0.66864, 0.66346, 0.65803, 0.65226, 0.64604, 0.63930, 0.63194,\
        0.62387, 0.61503, 0.60548, 0.59549, 0.58515, 0.57446, 0.56363, 0.55295, 0.54268, 0.53304, 0.52422,\
        0.51644, 0.50991, 0.50481, 0.50128, 0.49923, 0.49854, 0.49881, 0.49945, 0.49999, 0.50019, 0.49997],\
        [0.63459, 0.63381, 0.63332, 0.63341, 0.63440, 0.63650, 0.63976, 0.64403, 0.64906, 0.65453, 0.66023,\
        0.66601, 0.67184, 0.67778, 0.68397, 0.69049, 0.69710, 0.70357, 0.70987, 0.71612, 0.72237, 0.72826,\
        0.73343, 0.73779, 0.74143, 0.74450, 0.74717, 0.74960, 0.75190, 0.75415, 0.75642, 0.75871, 0.76099,\
        0.76316, 0.76512, 0.76679, 0.76814, 0.76921, 0.77006, 0.77071, 0.77117, 0.77143, 0.77144, 0.77121,\
        0.77081, 0.77032, 0.76985, 0.76949, 0.76935, 0.76953, 0.77010, 0.77103, 0.77214, 0.77316, 0.77388,\
        0.77419, 0.77410, 0.77374, 0.77328, 0.77286, 0.77256, 0.77236, 0.77213, 0.77166, 0.77067, 0.76891,\
        0.76623, 0.76265, 0.75832, 0.75346, 0.74824, 0.74278, 0.73711, 0.73120, 0.72497, 0.71843, 0.71172,\
        0.70526, 0.69984, 0.69544, 0.69272, 0.69243, 0.69360, 0.69483, 0.69472, 0.69321, 0.69011, 0.68606,\
        0.68172, 0.67728, 0.67275, 0.66812, 0.66338, 0.65847, 0.65332, 0.64784, 0.64192, 0.63546, 0.62837,\
        0.62051, 0.61176, 0.60215, 0.59205, 0.58161, 0.57093, 0.56019, 0.54967, 0.53960, 0.53019, 0.52164,\
        0.51418, 0.50799, 0.50330, 0.50026, 0.49865, 0.49815, 0.49846, 0.49914, 0.49976, 0.50006, 0.49987],\
        [0.62733, 0.62605, 0.62495, 0.62439, 0.62479, 0.62645, 0.62947, 0.63370, 0.63882, 0.64449, 0.65044,\
        0.65653, 0.66270, 0.66900, 0.67548, 0.68215, 0.68888, 0.69553, 0.70198, 0.70840, 0.71520, 0.72145,\
        0.72666, 0.73087, 0.73427, 0.73712, 0.73962, 0.74193, 0.74416, 0.74639, 0.74865, 0.75096, 0.75326,\
        0.75546, 0.75748, 0.75922, 0.76066, 0.76179, 0.76265, 0.76326, 0.76363, 0.76375, 0.76363, 0.76330,\
        0.76282, 0.76226, 0.76175, 0.76137, 0.76122, 0.76140, 0.76194, 0.76280, 0.76383, 0.76487, 0.76572,\
        0.76630, 0.76656, 0.76657, 0.76641, 0.76615, 0.76584, 0.76544, 0.76486, 0.76393, 0.76246, 0.76031,\
        0.75738, 0.75370, 0.74941, 0.74468, 0.73966, 0.73449, 0.72919, 0.72372, 0.71795, 0.71179, 0.70541,\
        0.69924, 0.69413, 0.69026, 0.68779, 0.68694, 0.68717, 0.68750, 0.68703, 0.68539, 0.68258, 0.67892,\
        0.67486, 0.67063, 0.66633, 0.66197, 0.65753, 0.65293, 0.64809, 0.64291, 0.63730, 0.63116, 0.62439,\
        0.61687, 0.60837, 0.59879, 0.58852, 0.57790, 0.56716, 0.55651, 0.54617, 0.53634, 0.52722, 0.51900,\
        0.51187, 0.50603, 0.50167, 0.49902, 0.49784, 0.49765, 0.49812, 0.49889, 0.49962, 0.49999, 0.49983],\
        [0.62081, 0.61902, 0.61728, 0.61599, 0.61568, 0.61686, 0.61969, 0.62395, 0.62922, 0.63506, 0.64119,\
        0.64747, 0.65384, 0.66032, 0.66694, 0.67368, 0.68049, 0.68726, 0.69387, 0.70077, 0.70826, 0.71469,\
        0.71978, 0.72367, 0.72673, 0.72933, 0.73167, 0.73389, 0.73607, 0.73830, 0.74060, 0.74298, 0.74539,\
        0.74772, 0.74986, 0.75172, 0.75326, 0.75447, 0.75537, 0.75595, 0.75625, 0.75628, 0.75605, 0.75561,\
        0.75502, 0.75438, 0.75377, 0.75332, 0.75311, 0.75324, 0.75373, 0.75453, 0.75556, 0.75664, 0.75764,\
        0.75844, 0.75897, 0.75925, 0.75930, 0.75914, 0.75879, 0.75822, 0.75734, 0.75603, 0.75419, 0.75170,\
        0.74854, 0.74475, 0.74046, 0.73584, 0.73107, 0.72628, 0.72151, 0.71671, 0.71170, 0.70615, 0.70023,\
        0.69451, 0.68955, 0.68568, 0.68302, 0.68151, 0.68077, 0.68016, 0.67908, 0.67720, 0.67446, 0.67105,\
        0.66725, 0.66328, 0.65925, 0.65519, 0.65107, 0.64680, 0.64229, 0.63743, 0.63212, 0.62625, 0.61976,\
        0.61260, 0.60453, 0.59497, 0.58449, 0.57374, 0.56300, 0.55249, 0.54240, 0.53289, 0.52415, 0.51633,\
        0.50960, 0.50414, 0.50021, 0.49796, 0.49709, 0.49715, 0.49779, 0.49869, 0.49953, 0.49998, 0.49984],\
        [0.61523, 0.61307, 0.61077, 0.60870, 0.60757, 0.60828, 0.61094, 0.61529, 0.62069, 0.62660, 0.63274,\
        0.63902, 0.64539, 0.65187, 0.65847, 0.66522, 0.67207, 0.67902, 0.68612, 0.69369, 0.70131, 0.70764,\
        0.71246, 0.71593, 0.71862, 0.72100, 0.72327, 0.72547, 0.72766, 0.72992, 0.73232, 0.73486, 0.73746,\
        0.74000, 0.74233, 0.74435, 0.74602, 0.74731, 0.74824, 0.74883, 0.74909, 0.74904, 0.74872, 0.74817,\
        0.74746, 0.74668, 0.74595, 0.74537, 0.74505, 0.74509, 0.74552, 0.74630, 0.74736, 0.74854, 0.74969,\
        0.75069, 0.75144, 0.75192, 0.75210, 0.75200, 0.75160, 0.75087, 0.74975, 0.74816, 0.74601, 0.74325,\
        0.73988, 0.73596, 0.73164, 0.72711, 0.72258, 0.71821, 0.71404, 0.71003, 0.70615, 0.70132, 0.69576,\
        0.69026, 0.68534, 0.68129, 0.67822, 0.67601, 0.67439, 0.67291, 0.67116, 0.66890, 0.66604, 0.66271,\
        0.65907, 0.65532, 0.65156, 0.64782, 0.64404, 0.64012, 0.63592, 0.63136, 0.62633, 0.62068, 0.61425,\
        0.60717, 0.59942, 0.58993, 0.57952, 0.56886, 0.55830, 0.54806, 0.53832, 0.52924, 0.52097, 0.51364,\
        0.50742, 0.50245, 0.49898, 0.49712, 0.49647, 0.49668, 0.49743, 0.49843, 0.49939, 0.50002, 0.49994],\
        [0.61075, 0.60844, 0.60584, 0.60321, 0.60122, 0.60162, 0.60386, 0.60833, 0.61366, 0.61937, 0.62528,\
        0.63133, 0.63748, 0.64377, 0.65019, 0.65680, 0.66359, 0.67060, 0.67791, 0.68552, 0.69297, 0.69944,\
        0.70414, 0.70725, 0.70969, 0.71206, 0.71443, 0.71672, 0.71898, 0.72133, 0.72388, 0.72666, 0.72956,\
        0.73240, 0.73498, 0.73718, 0.73897, 0.74035, 0.74133, 0.74194, 0.74218, 0.74207, 0.74166, 0.74100,\
        0.74016, 0.73922, 0.73832, 0.73757, 0.73710, 0.73701, 0.73738, 0.73817, 0.73931, 0.74062, 0.74196,\
        0.74314, 0.74408, 0.74470, 0.74497, 0.74489, 0.74443, 0.74357, 0.74227, 0.74046, 0.73809, 0.73511,\
        0.73156, 0.72752, 0.72314, 0.71865, 0.71431, 0.71039, 0.70682, 0.70343, 0.70098, 0.69687, 0.69140,\
        0.68598, 0.68104, 0.67678, 0.67327, 0.67042, 0.66801, 0.66575, 0.66335, 0.66063, 0.65752, 0.65410,\
        0.65050, 0.64688, 0.64335, 0.63992, 0.63651, 0.63293, 0.62902, 0.62469, 0.61991, 0.61454, 0.60804,\
        0.60049, 0.59258, 0.58347, 0.57346, 0.56317, 0.55298, 0.54317, 0.53392, 0.52539, 0.51769, 0.51096,\
        0.50535, 0.50104, 0.49807, 0.49646, 0.49595, 0.49622, 0.49698, 0.49798, 0.49894, 0.49960, 0.49944],\
        [0.60736, 0.60517, 0.60266, 0.60001, 0.59796, 0.59781, 0.59973, 0.60369, 0.60838, 0.61350, 0.61890,\
        0.62447, 0.63020, 0.63608, 0.64215, 0.64844, 0.65497, 0.66178, 0.66887, 0.67612, 0.68314, 0.68934,\
        0.69411, 0.69737, 0.69993, 0.70259, 0.70525, 0.70774, 0.71013, 0.71263, 0.71538, 0.71848, 0.72182,\
        0.72504, 0.72789, 0.73026, 0.73216, 0.73362, 0.73466, 0.73529, 0.73554, 0.73540, 0.73492, 0.73415,\
        0.73317, 0.73206, 0.73096, 0.73000, 0.72932, 0.72909, 0.72938, 0.73022, 0.73148, 0.73299, 0.73453,\
        0.73591, 0.73700, 0.73772, 0.73803, 0.73793, 0.73740, 0.73644, 0.73500, 0.73305, 0.73052, 0.72741,\
        0.72372, 0.71958, 0.71516, 0.71069, 0.70644, 0.70293, 0.69999, 0.69720, 0.69469, 0.69125, 0.68639,\
        0.68127, 0.67639, 0.67198, 0.66810, 0.66470, 0.66163, 0.65869, 0.65568, 0.65248, 0.64903, 0.64539,\
        0.64169, 0.63809, 0.63471, 0.63158, 0.62855, 0.62534, 0.62167, 0.61741, 0.61272, 0.60777, 0.60187,\
        0.59376, 0.58530, 0.57622, 0.56658, 0.55675, 0.54708, 0.53784, 0.52922, 0.52136, 0.51435, 0.50828,\
        0.50329, 0.49959, 0.49717, 0.49584, 0.49543, 0.49570, 0.49638, 0.49724, 0.49800, 0.49838, 0.49797],\
        [0.60485, 0.60294, 0.60080, 0.59843, 0.59662, 0.59631, 0.59787, 0.60081, 0.60455, 0.60887, 0.61353,\
        0.61843, 0.62355, 0.62887, 0.63441, 0.64019, 0.64625, 0.65259, 0.65917, 0.66585, 0.67231, 0.67813,\
        0.68291, 0.68665, 0.68979, 0.69285, 0.69583, 0.69864, 0.70126, 0.70402, 0.70697, 0.71050, 0.71443,\
        0.71807, 0.72114, 0.72362, 0.72559, 0.72711, 0.72823, 0.72892, 0.72919, 0.72905, 0.72852, 0.72766,\
        0.72655, 0.72527, 0.72395, 0.72275, 0.72183, 0.72140, 0.72162, 0.72253, 0.72399, 0.72575, 0.72753,\
        0.72909, 0.73030, 0.73106, 0.73137, 0.73122, 0.73061, 0.72956, 0.72803, 0.72599, 0.72341, 0.72024,\
        0.71649, 0.71227, 0.70786, 0.70355, 0.69943, 0.69614, 0.69352, 0.69099, 0.68832, 0.68497, 0.68073,\
        0.67604, 0.67135, 0.66688, 0.66273, 0.65890, 0.65529, 0.65177, 0.64821, 0.64453, 0.64069, 0.63674,\
        0.63281, 0.62909, 0.62577, 0.62290, 0.62028, 0.61750, 0.61406, 0.60966, 0.60466, 0.59968, 0.59399,\
        0.58623, 0.57753, 0.56845, 0.55912, 0.54979, 0.54072, 0.53215, 0.52427, 0.51719, 0.51101, 0.50572,\
        0.50137, 0.49825, 0.49629, 0.49522, 0.49489, 0.49509, 0.49562, 0.49624, 0.49671, 0.49675, 0.49610],\
        [0.60295, 0.60131, 0.59945, 0.59747, 0.59599, 0.59563, 0.59664, 0.59878, 0.60174, 0.60522, 0.60905,\
        0.61316, 0.61754, 0.62216, 0.62702, 0.63214, 0.63754, 0.64323, 0.64914, 0.65516, 0.66107, 0.66656,\
        0.67143, 0.67563, 0.67939, 0.68294, 0.68634, 0.68956, 0.69266, 0.69578, 0.69898, 0.70307, 0.70767,\
        0.71158, 0.71475, 0.71724, 0.71922, 0.72081, 0.72202, 0.72281, 0.72314, 0.72302, 0.72248, 0.72157,\
        0.72035, 0.71891, 0.71739, 0.71596, 0.71476, 0.71408, 0.71420, 0.71524, 0.71698, 0.71905, 0.72108,\
        0.72280, 0.72407, 0.72482, 0.72506, 0.72481, 0.72411, 0.72297, 0.72139, 0.71933, 0.71677, 0.71365,\
        0.70996, 0.70571, 0.70132, 0.69728, 0.69352, 0.69029, 0.68756, 0.68495, 0.68214, 0.67882, 0.67490,\
        0.67054, 0.66603, 0.66155, 0.65722, 0.65306, 0.64902, 0.64503, 0.64100, 0.63687, 0.63264, 0.62832,\
        0.62406, 0.62007, 0.61665, 0.61398, 0.61179, 0.60946, 0.60637, 0.60178, 0.59638, 0.59092, 0.58480,\
        0.57739, 0.56901, 0.56019, 0.55125, 0.54246, 0.53404, 0.52621, 0.51914, 0.51294, 0.50772, 0.50339,\
        0.49984, 0.49722, 0.49555, 0.49463, 0.49431, 0.49441, 0.49473, 0.49507, 0.49521, 0.49492, 0.49404],\
        [0.60146, 0.60001, 0.59840, 0.59679, 0.59558, 0.59519, 0.59580, 0.59735, 0.59963, 0.60231, 0.60526,\
        0.60858, 0.61217, 0.61600, 0.62008, 0.62443, 0.62906, 0.63396, 0.63911, 0.64445, 0.64984, 0.65510,\
        0.66007, 0.66469, 0.66900, 0.67309, 0.67702, 0.68078, 0.68444, 0.68813, 0.69206, 0.69671, 0.70145,\
        0.70544, 0.70861, 0.71099, 0.71296, 0.71466, 0.71603, 0.71695, 0.71738, 0.71730, 0.71679, 0.71588,\
        0.71461, 0.71305, 0.71138, 0.70979, 0.70832, 0.70731, 0.70732, 0.70853, 0.71063, 0.71306, 0.71531,\
        0.71712, 0.71837, 0.71903, 0.71912, 0.71873, 0.71790, 0.71668, 0.71506, 0.71303, 0.71055, 0.70758,\
        0.70409, 0.70001, 0.69571, 0.69179, 0.68820, 0.68498, 0.68208, 0.67927, 0.67629, 0.67295, 0.66916,\
        0.66500, 0.66061, 0.65614, 0.65168, 0.64727, 0.64290, 0.63853, 0.63411, 0.62962, 0.62503, 0.62037,\
        0.61574, 0.61135, 0.60763, 0.60506, 0.60321, 0.60111, 0.59820, 0.59367, 0.58806, 0.58209, 0.57554,\
        0.56817, 0.56008, 0.55162, 0.54315, 0.53494, 0.52722, 0.52018, 0.51395, 0.50865, 0.50441, 0.50123,\
        0.49847, 0.49631, 0.49488, 0.49407, 0.49372, 0.49368, 0.49376, 0.49380, 0.49361, 0.49301, 0.49187],\
        [0.60027, 0.59900, 0.59758, 0.59622, 0.59520, 0.59480, 0.59514, 0.59621, 0.59787, 0.59979, 0.60203,\
        0.60464, 0.60745, 0.61047, 0.61374, 0.61725, 0.62103, 0.62508, 0.62942, 0.63407, 0.63897, 0.64403,\
        0.64912, 0.65411, 0.65893, 0.66358, 0.66806, 0.67240, 0.67666, 0.68097, 0.68550, 0.69034, 0.69509,\
        0.69930, 0.70248, 0.70468, 0.70671, 0.70861, 0.71022, 0.71133, 0.71187, 0.71185, 0.71140, 0.71057,\
        0.70936, 0.70775, 0.70594, 0.70443, 0.70284, 0.70143, 0.70136, 0.70272, 0.70521, 0.70794, 0.71029,\
        0.71208, 0.71322, 0.71368, 0.71355, 0.71295, 0.71198, 0.71068, 0.70903, 0.70702, 0.70465, 0.70184,\
        0.69852, 0.69471, 0.69067, 0.68683, 0.68328, 0.68002, 0.67696, 0.67395, 0.67082, 0.66741, 0.66365,\
        0.65957, 0.65525, 0.65077, 0.64621, 0.64161, 0.63699, 0.63233, 0.62762, 0.62285, 0.61801, 0.61310,\
        0.60818, 0.60346, 0.59941, 0.59672, 0.59478, 0.59250, 0.58933, 0.58491, 0.57941, 0.57323, 0.56645,\
        0.55905, 0.55116, 0.54305, 0.53504, 0.52744, 0.52047, 0.51427, 0.50892, 0.50449, 0.50105, 0.49879,\
        0.49693, 0.49533, 0.49420, 0.49350, 0.49312, 0.49292, 0.49276, 0.49249, 0.49198, 0.49107, 0.48966],\
        [0.59922, 0.59803, 0.59677, 0.59561, 0.59474, 0.59434, 0.59450, 0.59519, 0.59631, 0.59772, 0.59944,\
        0.60140, 0.60342, 0.60566, 0.60814, 0.61083, 0.61373, 0.61689, 0.62038, 0.62430, 0.62872, 0.63362,\
        0.63884, 0.64417, 0.64946, 0.65462, 0.65962, 0.66449, 0.66927, 0.67406, 0.67893, 0.68384, 0.68856,\
        0.69274, 0.69595, 0.69832, 0.70052, 0.70270, 0.70457, 0.70591, 0.70654, 0.70657, 0.70621, 0.70554,\
        0.70456, 0.70309, 0.70100, 0.69978, 0.69842, 0.69707, 0.69700, 0.69836, 0.70102, 0.70371, 0.70592,\
        0.70759, 0.70854, 0.70872, 0.70828, 0.70743, 0.70632, 0.70495, 0.70327, 0.70125, 0.69896, 0.69625,\
        0.69308, 0.68953, 0.68580, 0.68211, 0.67861, 0.67530, 0.67212, 0.66895, 0.66568, 0.66219, 0.65842,\
        0.65437, 0.65006, 0.64556, 0.64091, 0.63617, 0.63136, 0.62648, 0.62156, 0.61660, 0.61162, 0.60661,\
        0.60159, 0.59680, 0.59270, 0.58957, 0.58699, 0.58421, 0.58070, 0.57621, 0.57076, 0.56454, 0.55769,\
        0.55031, 0.54258, 0.53476, 0.52720, 0.52021, 0.51403, 0.50873, 0.50433, 0.50083, 0.49828, 0.49661,\
        0.49539, 0.49436, 0.49355, 0.49296, 0.49252, 0.49216, 0.49174, 0.49117, 0.49033, 0.48911, 0.48742],\
        [0.59717, 0.59655, 0.59565, 0.59477, 0.59410, 0.59377, 0.59383, 0.59429, 0.59509, 0.59613, 0.59740,\
        0.59876, 0.60014, 0.60173, 0.60349, 0.60540, 0.60746, 0.60973, 0.61232, 0.61545, 0.61937, 0.62414,\
        0.62951, 0.63516, 0.64083, 0.64641, 0.65183, 0.65710, 0.66226, 0.66736, 0.67241, 0.67734, 0.68197,\
        0.68606, 0.68942, 0.69216, 0.69462, 0.69695, 0.69900, 0.70055, 0.70124, 0.70132, 0.70110, 0.70062,\
        0.69991, 0.69880, 0.69699, 0.69559, 0.69447, 0.69367, 0.69381, 0.69517, 0.69752, 0.69985, 0.70187,\
        0.70342, 0.70416, 0.70401, 0.70323, 0.70214, 0.70088, 0.69948, 0.69777, 0.69576, 0.69347, 0.69083,\
        0.68782, 0.68451, 0.68102, 0.67751, 0.67408, 0.67075, 0.66749, 0.66422, 0.66084, 0.65728, 0.65348,\
        0.64942, 0.64511, 0.64058, 0.63586, 0.63101, 0.62605, 0.62102, 0.61595, 0.61085, 0.60581, 0.60082,\
        0.59582, 0.59105, 0.58682, 0.58323, 0.57998, 0.57662, 0.57272, 0.56806, 0.56257, 0.55636, 0.54952,\
        0.54221, 0.53462, 0.52705, 0.51988, 0.51350, 0.50817, 0.50386, 0.50047, 0.49796, 0.49623, 0.49509,\
        0.49428, 0.49362, 0.49303, 0.49250, 0.49198, 0.49142, 0.49074, 0.48986, 0.48868, 0.48713, 0.48515],\
        [0.59465, 0.59456, 0.59413, 0.59363, 0.59322, 0.59304, 0.59313, 0.59350, 0.59411, 0.59492, 0.59585,\
        0.59682, 0.59781, 0.59887, 0.60001, 0.60123, 0.60253, 0.60396, 0.60564, 0.60788, 0.61122, 0.61591,\
        0.62145, 0.62734, 0.63327, 0.63910, 0.64479, 0.65031, 0.65569, 0.66095, 0.66606, 0.67096, 0.67552,\
        0.67960, 0.68311, 0.68612, 0.68880, 0.69123, 0.69333, 0.69492, 0.69577, 0.69606, 0.69601, 0.69571,\
        0.69516, 0.69424, 0.69298, 0.69179, 0.69091, 0.69050, 0.69085, 0.69207, 0.69395, 0.69596, 0.69778,\
        0.69918, 0.69975, 0.69933, 0.69834, 0.69710, 0.69575, 0.69424, 0.69250, 0.69049, 0.68821, 0.68563,\
        0.68275, 0.67963, 0.67634, 0.67299, 0.66965, 0.66634, 0.66304, 0.65970, 0.65626, 0.65265, 0.64882,\
        0.64475, 0.64043, 0.63588, 0.63111, 0.62616, 0.62110, 0.61597, 0.61081, 0.60561, 0.60049, 0.59552,\
        0.59059, 0.58585, 0.58147, 0.57748, 0.57370, 0.56982, 0.56554, 0.56067, 0.55513, 0.54894, 0.54219,\
        0.53501, 0.52760, 0.52024, 0.51338, 0.50757, 0.50316, 0.49995, 0.49761, 0.49599, 0.49492, 0.49422,\
        0.49371, 0.49324, 0.49274, 0.49218, 0.49152, 0.49073, 0.48975, 0.48854, 0.48702, 0.48514, 0.48285],\
        [0.59184, 0.59222, 0.59227, 0.59218, 0.59209, 0.59213, 0.59234, 0.59275, 0.59333, 0.59403, 0.59481,\
        0.59560, 0.59637, 0.59712, 0.59785, 0.59855, 0.59924, 0.59997, 0.60085, 0.60209, 0.60471, 0.60944,\
        0.61507, 0.62099, 0.62695, 0.63283, 0.63858, 0.64418, 0.64963, 0.65491, 0.66000, 0.66483, 0.66933,\
        0.67340, 0.67700, 0.68018, 0.68299, 0.68547, 0.68758, 0.68919, 0.69022, 0.69074, 0.69090, 0.69078,\
        0.69039, 0.68972, 0.68886, 0.68803, 0.68745, 0.68730, 0.68774, 0.68879, 0.69029, 0.69193, 0.69343,\
        0.69451, 0.69488, 0.69447, 0.69349, 0.69223, 0.69080, 0.68921, 0.68742, 0.68540, 0.68313, 0.68062,\
        0.67785, 0.67488, 0.67175, 0.66854, 0.66529, 0.66202, 0.65872, 0.65536, 0.65189, 0.64826, 0.64443,\
        0.64036, 0.63605, 0.63149, 0.62668, 0.62167, 0.61651, 0.61131, 0.60615, 0.60097, 0.59577, 0.59075,\
        0.58584, 0.58108, 0.57656, 0.57226, 0.56808, 0.56379, 0.55921, 0.55416, 0.54858, 0.54248, 0.53589,\
        0.52895, 0.52181, 0.51473, 0.50812, 0.50277, 0.49936, 0.49725, 0.49586, 0.49495, 0.49439, 0.49401,\
        0.49367, 0.49326, 0.49272, 0.49202, 0.49114, 0.49007, 0.48877, 0.48720, 0.48533, 0.48310, 0.48049],\
        [0.58881, 0.58961, 0.59011, 0.59044, 0.59072, 0.59104, 0.59146, 0.59200, 0.59266, 0.59341, 0.59419,\
        0.59497, 0.59569, 0.59633, 0.59688, 0.59732, 0.59768, 0.59801, 0.59840, 0.59896, 0.60098, 0.60545,\
        0.61068, 0.61625, 0.62194, 0.62762, 0.63324, 0.63874, 0.64410, 0.64930, 0.65429, 0.65902, 0.66343,\
        0.66747, 0.67111, 0.67435, 0.67722, 0.67973, 0.68185, 0.68350, 0.68468, 0.68540, 0.68576, 0.68583,\
        0.68563, 0.68523, 0.68470, 0.68420, 0.68389, 0.68393, 0.68439, 0.68528, 0.68647, 0.68775, 0.68889,\
        0.68966, 0.68987, 0.68947, 0.68857, 0.68735, 0.68590, 0.68428, 0.68246, 0.68044, 0.67820, 0.67575,\
        0.67308, 0.67023, 0.66723, 0.66414, 0.66098, 0.65777, 0.65450, 0.65115, 0.64768, 0.64407, 0.64026,\
        0.63623, 0.63196, 0.62742, 0.62262, 0.61756, 0.61231, 0.60699, 0.60184, 0.59674, 0.59157, 0.58650,\
        0.58155, 0.57673, 0.57207, 0.56754, 0.56306, 0.55849, 0.55369, 0.54854, 0.54299, 0.53703, 0.53072,\
        0.52414, 0.51747, 0.51091, 0.50479, 0.49984, 0.49741, 0.49607, 0.49528, 0.49485, 0.49461, 0.49440,\
        0.49410, 0.49363, 0.49292, 0.49199, 0.49082, 0.48941, 0.48775, 0.48582, 0.48358, 0.48100, 0.47804],\
        [0.58557, 0.58678, 0.58770, 0.58844, 0.58909, 0.58974, 0.59043, 0.59119, 0.59204, 0.59294, 0.59386,\
        0.59476, 0.59559, 0.59630, 0.59684, 0.59723, 0.59751, 0.59778, 0.59816, 0.59898, 0.60080, 0.60393,\
        0.60816, 0.61299, 0.61814, 0.62341, 0.62871, 0.63397, 0.63912, 0.64414, 0.64896, 0.65355, 0.65784,\
        0.66181, 0.66541, 0.66866, 0.67154, 0.67406, 0.67619, 0.67790, 0.67919, 0.68008, 0.68062, 0.68088,\
        0.68089, 0.68073, 0.68048, 0.68025, 0.68017, 0.68032, 0.68078, 0.68152, 0.68245, 0.68343, 0.68425,\
        0.68477, 0.68484, 0.68443, 0.68359, 0.68242, 0.68101, 0.67939, 0.67758, 0.67558, 0.67337, 0.67098,\
        0.66840, 0.66565, 0.66277, 0.65979, 0.65672, 0.65357, 0.65035, 0.64704, 0.64361, 0.64004, 0.63628,\
        0.63232, 0.62812, 0.62366, 0.61891, 0.61388, 0.60858, 0.60311, 0.59785, 0.59283, 0.58774, 0.58266,\
        0.57767, 0.57277, 0.56799, 0.56329, 0.55860, 0.55385, 0.54893, 0.54376, 0.53832, 0.53259, 0.52664,\
        0.52057, 0.51455, 0.50883, 0.50378, 0.49999, 0.49763, 0.49636, 0.49578, 0.49556, 0.49547, 0.49531,\
        0.49493, 0.49426, 0.49328, 0.49202, 0.49049, 0.48870, 0.48666, 0.48433, 0.48171, 0.47877, 0.47548],\
        [0.58214, 0.58372, 0.58505, 0.58618, 0.58721, 0.58820, 0.58921, 0.59025, 0.59135, 0.59249, 0.59364,\
        0.59478, 0.59584, 0.59674, 0.59743, 0.59789, 0.59821, 0.59856, 0.59907, 0.59991, 0.60129, 0.60356,\
        0.60683, 0.61083, 0.61529, 0.62003, 0.62489, 0.62979, 0.63464, 0.63940, 0.64400, 0.64840, 0.65254,\
        0.65639, 0.65992, 0.66312, 0.66598, 0.66849, 0.67063, 0.67239, 0.67377, 0.67479, 0.67550, 0.67593,\
        0.67614, 0.67620, 0.67618, 0.67617, 0.67625, 0.67649, 0.67693, 0.67754, 0.67827, 0.67898, 0.67956,\
        0.67987, 0.67982, 0.67939, 0.67858, 0.67747, 0.67610, 0.67451, 0.67273, 0.67076, 0.66860, 0.66627,\
        0.66377, 0.66112, 0.65834, 0.65546, 0.65247, 0.64940, 0.64625, 0.64299, 0.63962, 0.63611, 0.63244,\
        0.62857, 0.62449, 0.62015, 0.61553, 0.61060, 0.60537, 0.59996, 0.59460, 0.58942, 0.58428, 0.57917,\
        0.57413, 0.56916, 0.56428, 0.55946, 0.55465, 0.54980, 0.54485, 0.53974, 0.53446, 0.52903, 0.52351,\
        0.51802, 0.51274, 0.50791, 0.50387, 0.50089, 0.49877, 0.49751, 0.49696, 0.49681, 0.49676, 0.49655,\
        0.49598, 0.49501, 0.49366, 0.49200, 0.49006, 0.48786, 0.48541, 0.48269, 0.47969, 0.47639, 0.47275],\
        [0.57850, 0.58044, 0.58214, 0.58366, 0.58506, 0.58641, 0.58774, 0.58909, 0.59048, 0.59191, 0.59336,\
        0.59479, 0.59616, 0.59737, 0.59829, 0.59884, 0.59919, 0.59958, 0.60007, 0.60067, 0.60171, 0.60346,\
        0.60601, 0.60928, 0.61308, 0.61724, 0.62161, 0.62609, 0.63059, 0.63504, 0.63938, 0.64354, 0.64750,\
        0.65120, 0.65463, 0.65775, 0.66056, 0.66304, 0.66517, 0.66697, 0.66842, 0.66955, 0.67039, 0.67098,\
        0.67137, 0.67162, 0.67178, 0.67194, 0.67214, 0.67244, 0.67285, 0.67336, 0.67392, 0.67443, 0.67481,\
        0.67496, 0.67482, 0.67435, 0.67356, 0.67249, 0.67116, 0.66962, 0.66788, 0.66596, 0.66386, 0.66159,\
        0.65917, 0.65661, 0.65393, 0.65113, 0.64823, 0.64524, 0.64216, 0.63897, 0.63567, 0.63224, 0.62867,\
        0.62492, 0.62097, 0.61679, 0.61235, 0.60759, 0.60248, 0.59713, 0.59172, 0.58639, 0.58115, 0.57597,\
        0.57087, 0.56584, 0.56089, 0.55600, 0.55114, 0.54626, 0.54134, 0.53635, 0.53129, 0.52618, 0.52112,\
        0.51620, 0.51160, 0.50752, 0.50417, 0.50165, 0.49987, 0.49880, 0.49834, 0.49824, 0.49820, 0.49790,\
        0.49706, 0.49568, 0.49390, 0.49179, 0.48942, 0.48680, 0.48393, 0.48082, 0.47745, 0.47379, 0.46983],\
        [0.57463, 0.57692, 0.57897, 0.58086, 0.58262, 0.58431, 0.58597, 0.58764, 0.58934, 0.59106, 0.59281,\
        0.59456, 0.59627, 0.59784, 0.59907, 0.59967, 0.59991, 0.60020, 0.60054, 0.60101, 0.60184, 0.60324,\
        0.60530, 0.60799, 0.61121, 0.61483, 0.61871, 0.62276, 0.62687, 0.63099, 0.63503, 0.63895, 0.64269,\
        0.64622, 0.64951, 0.65253, 0.65527, 0.65770, 0.65982, 0.66164, 0.66314, 0.66436, 0.66531, 0.66603,\
        0.66657, 0.66697, 0.66728, 0.66756, 0.66785, 0.66819, 0.66857, 0.66900, 0.66943, 0.66979, 0.67001,\
        0.67004, 0.66981, 0.66931, 0.66853, 0.66749, 0.66621, 0.66471, 0.66302, 0.66115, 0.65911, 0.65692,\
        0.65457, 0.65209, 0.64949, 0.64678, 0.64396, 0.64105, 0.63804, 0.63493, 0.63171, 0.62837, 0.62489,\
        0.62126, 0.61746, 0.61346, 0.60921, 0.60466, 0.59969, 0.59434, 0.58891, 0.58351, 0.57819, 0.57295,\
        0.56781, 0.56274, 0.55776, 0.55284, 0.54797, 0.54313, 0.53829, 0.53345, 0.52863, 0.52387, 0.51923,\
        0.51483, 0.51079, 0.50728, 0.50441, 0.50224, 0.50074, 0.49988, 0.49954, 0.49944, 0.49945, 0.49908,\
        0.49787, 0.49602, 0.49377, 0.49121, 0.48842, 0.48539, 0.48215, 0.47866, 0.47493, 0.47094, 0.46665],\
        [0.57052, 0.57313, 0.57552, 0.57775, 0.57985, 0.58187, 0.58385, 0.58582, 0.58780, 0.58981, 0.59183,\
        0.59386, 0.59585, 0.59772, 0.59931, 0.60007, 0.60009, 0.60024, 0.60047, 0.60086, 0.60157, 0.60274,\
        0.60445, 0.60671, 0.60945, 0.61259, 0.61602, 0.61965, 0.62339, 0.62716, 0.63090, 0.63456, 0.63807,\
        0.64142, 0.64455, 0.64745, 0.65009, 0.65247, 0.65457, 0.65638, 0.65792, 0.65920, 0.66024, 0.66107,\
        0.66173, 0.66225, 0.66267, 0.66305, 0.66340, 0.66376, 0.66412, 0.66449, 0.66481, 0.66505, 0.66516,\
        0.66509, 0.66480, 0.66426, 0.66348, 0.66246, 0.66122, 0.65977, 0.65814, 0.65632, 0.65435, 0.65222,\
        0.64995, 0.64754, 0.64502, 0.64239, 0.63965, 0.63682, 0.63388, 0.63085, 0.62770, 0.62445, 0.62106,\
        0.61753, 0.61385, 0.60999, 0.60591, 0.60155, 0.59666, 0.59139, 0.58599, 0.58060, 0.57527, 0.57002,\
        0.56486, 0.55979, 0.55481, 0.54991, 0.54507, 0.54030, 0.53558, 0.53092, 0.52634, 0.52189, 0.51764,\
        0.51367, 0.51009, 0.50700, 0.50449, 0.50259, 0.50129, 0.50054, 0.50020, 0.50003, 0.50008, 0.49969,\
        0.49800, 0.49571, 0.49305, 0.49012, 0.48695, 0.48357, 0.47997, 0.47615, 0.47209, 0.46778, 0.46319]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=40 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax40vt2.5
    zgrid_H_Hmax_vmax40vt2_5=[[0.38137, 0.39006, 0.39846, 0.40655, 0.41431, 0.42172, 0.42873, 0.43531, 0.44143, 0.44707, 0.45223,\
        0.45694, 0.46126, 0.46527, 0.46906, 0.47271, 0.47627, 0.47978, 0.48311, 0.48600, 0.48846, 0.49078,\
        0.49297, 0.49490, 0.49664, 0.49892, 0.50206, 0.50564, 0.50952, 0.51361, 0.51784, 0.52217, 0.52656,\
        0.53095, 0.53529, 0.53954, 0.54363, 0.54750, 0.55111, 0.55440, 0.55736, 0.55999, 0.56233, 0.56444,\
        0.56637, 0.56821, 0.57001, 0.57185, 0.57379, 0.57591, 0.57826, 0.58088, 0.58374, 0.58671, 0.58938,\
        0.59121, 0.59203, 0.59222, 0.59226, 0.59224, 0.59204, 0.59149, 0.59068, 0.58979, 0.58893, 0.58819,\
        0.58753, 0.58684, 0.58604, 0.58521, 0.58439, 0.58357, 0.58275, 0.58188, 0.58096, 0.57998, 0.57895,\
        0.57784, 0.57660, 0.57523, 0.57372, 0.57207, 0.57028, 0.56831, 0.56615, 0.56379, 0.56123, 0.55848,\
        0.55555, 0.55247, 0.54926, 0.54591, 0.54244, 0.53884, 0.53511, 0.53124, 0.52721, 0.52301, 0.51862,\
        0.51401, 0.50913, 0.50395, 0.49832, 0.49200, 0.48526, 0.47834, 0.47132, 0.46415, 0.45677, 0.44916,\
        0.44131, 0.43323, 0.42492, 0.41640, 0.40765, 0.39867, 0.38946, 0.38000, 0.37030, 0.36033, 0.35009],\
        [0.38999, 0.39864, 0.40701, 0.41508, 0.42284, 0.43024, 0.43725, 0.44383, 0.44991, 0.45547, 0.46048,\
        0.46499, 0.46908, 0.47285, 0.47640, 0.47982, 0.48321, 0.48670, 0.49023, 0.49316, 0.49529, 0.49758,\
        0.49984, 0.50179, 0.50350, 0.50571, 0.50874, 0.51226, 0.51609, 0.52015, 0.52438, 0.52873, 0.53315,\
        0.53759, 0.54201, 0.54635, 0.55054, 0.55450, 0.55818, 0.56149, 0.56443, 0.56698, 0.56920, 0.57115,\
        0.57293, 0.57460, 0.57625, 0.57793, 0.57973, 0.58173, 0.58402, 0.58665, 0.58963, 0.59295, 0.59618,\
        0.59831, 0.59906, 0.59914, 0.59924, 0.59935, 0.59932, 0.59873, 0.59785, 0.59691, 0.59598, 0.59526,\
        0.59468, 0.59399, 0.59311, 0.59219, 0.59132, 0.59048, 0.58963, 0.58872, 0.58775, 0.58673, 0.58568,\
        0.58455, 0.58329, 0.58186, 0.58029, 0.57860, 0.57678, 0.57478, 0.57258, 0.57015, 0.56750, 0.56466,\
        0.56164, 0.55849, 0.55520, 0.55180, 0.54829, 0.54467, 0.54092, 0.53703, 0.53300, 0.52880, 0.52440,\
        0.51978, 0.51489, 0.50968, 0.50402, 0.49771, 0.49105, 0.48431, 0.47755, 0.47063, 0.46345, 0.45600,\
        0.44830, 0.44035, 0.43219, 0.42381, 0.41522, 0.40640, 0.39734, 0.38804, 0.37849, 0.36867, 0.35858],\
        [0.39819, 0.40679, 0.41513, 0.42319, 0.43094, 0.43835, 0.44538, 0.45197, 0.45805, 0.46355, 0.46843,\
        0.47272, 0.47654, 0.48004, 0.48333, 0.48648, 0.48959, 0.49285, 0.49660, 0.49994, 0.50171, 0.50392,\
        0.50621, 0.50826, 0.51025, 0.51257, 0.51546, 0.51883, 0.52256, 0.52654, 0.53072, 0.53505, 0.53948,\
        0.54397, 0.54846, 0.55289, 0.55718, 0.56126, 0.56503, 0.56840, 0.57131, 0.57378, 0.57586, 0.57766,\
        0.57927, 0.58080, 0.58230, 0.58384, 0.58548, 0.58733, 0.58953, 0.59210, 0.59504, 0.59846, 0.60204,\
        0.60445, 0.60554, 0.60596, 0.60622, 0.60640, 0.60635, 0.60587, 0.60506, 0.60414, 0.60320, 0.60247,\
        0.60188, 0.60115, 0.60014, 0.59915, 0.59823, 0.59734, 0.59646, 0.59548, 0.59442, 0.59333, 0.59224,\
        0.59110, 0.58979, 0.58827, 0.58662, 0.58488, 0.58302, 0.58100, 0.57874, 0.57622, 0.57346, 0.57049,\
        0.56737, 0.56412, 0.56076, 0.55730, 0.55374, 0.55008, 0.54630, 0.54240, 0.53835, 0.53413, 0.52970,\
        0.52503, 0.52007, 0.51475, 0.50898, 0.50272, 0.49621, 0.48977, 0.48339, 0.47677, 0.46979, 0.46248,\
        0.45491, 0.44710, 0.43908, 0.43086, 0.42244, 0.41379, 0.40490, 0.39577, 0.38637, 0.37671, 0.36676],\
        [0.40597, 0.41451, 0.42280, 0.43083, 0.43856, 0.44599, 0.45305, 0.45969, 0.46581, 0.47130, 0.47607,\
        0.48013, 0.48363, 0.48686, 0.48993, 0.49283, 0.49565, 0.49840, 0.50190, 0.50538, 0.50780, 0.51004,\
        0.51230, 0.51449, 0.51670, 0.51915, 0.52200, 0.52525, 0.52886, 0.53274, 0.53684, 0.54112, 0.54553,\
        0.55003, 0.55457, 0.55909, 0.56351, 0.56773, 0.57163, 0.57508, 0.57800, 0.58038, 0.58231, 0.58395,\
        0.58541, 0.58681, 0.58822, 0.58966, 0.59116, 0.59285, 0.59500, 0.59760, 0.60048, 0.60386, 0.60733,\
        0.61000, 0.61163, 0.61255, 0.61309, 0.61338, 0.61338, 0.61302, 0.61235, 0.61151, 0.61064, 0.60985,\
        0.60910, 0.60825, 0.60723, 0.60618, 0.60517, 0.60420, 0.60323, 0.60215, 0.60099, 0.59981, 0.59867,\
        0.59751, 0.59612, 0.59444, 0.59268, 0.59089, 0.58901, 0.58696, 0.58464, 0.58200, 0.57908, 0.57597,\
        0.57272, 0.56937, 0.56593, 0.56240, 0.55880, 0.55510, 0.55129, 0.54737, 0.54330, 0.53906, 0.53460,\
        0.52988, 0.52486, 0.51946, 0.51365, 0.50744, 0.50105, 0.49491, 0.48909, 0.48269, 0.47581, 0.46860,\
        0.46114, 0.45347, 0.44561, 0.43757, 0.42932, 0.42085, 0.41215, 0.40319, 0.39396, 0.38445, 0.37466],\
        [0.41331, 0.42178, 0.43001, 0.43798, 0.44569, 0.45311, 0.46020, 0.46691, 0.47313, 0.47869, 0.48342,\
        0.48722, 0.49032, 0.49329, 0.49626, 0.49898, 0.50165, 0.50434, 0.50742, 0.51063, 0.51339, 0.51584,\
        0.51819, 0.52050, 0.52287, 0.52542, 0.52826, 0.53143, 0.53491, 0.53867, 0.54267, 0.54687, 0.55123,\
        0.55572, 0.56029, 0.56488, 0.56943, 0.57382, 0.57791, 0.58150, 0.58445, 0.58673, 0.58851, 0.58999,\
        0.59133, 0.59265, 0.59404, 0.59547, 0.59694, 0.59848, 0.60065, 0.60337, 0.60637, 0.60965, 0.61290,\
        0.61565, 0.61765, 0.61899, 0.61984, 0.62032, 0.62044, 0.62021, 0.61968, 0.61894, 0.61811, 0.61726,\
        0.61638, 0.61541, 0.61435, 0.61324, 0.61214, 0.61104, 0.60993, 0.60876, 0.60751, 0.60625, 0.60500,\
        0.60372, 0.60219, 0.60038, 0.59851, 0.59665, 0.59475, 0.59268, 0.59027, 0.58746, 0.58435, 0.58106,\
        0.57768, 0.57423, 0.57071, 0.56713, 0.56348, 0.55974, 0.55592, 0.55198, 0.54790, 0.54365, 0.53918,\
        0.53445, 0.52941, 0.52402, 0.51827, 0.51221, 0.50602, 0.50040, 0.49488, 0.48835, 0.48149, 0.47435,\
        0.46700, 0.45947, 0.45178, 0.44393, 0.43588, 0.42761, 0.41910, 0.41032, 0.40126, 0.39191, 0.38227],\
        [0.42022, 0.42860, 0.43674, 0.44463, 0.45227, 0.45965, 0.46676, 0.47353, 0.47987, 0.48559, 0.49041,\
        0.49405, 0.49660, 0.49927, 0.50227, 0.50496, 0.50757, 0.51026, 0.51313, 0.51608, 0.51886, 0.52143,\
        0.52388, 0.52630, 0.52876, 0.53137, 0.53420, 0.53730, 0.54068, 0.54431, 0.54819, 0.55228, 0.55656,\
        0.56100, 0.56556, 0.57020, 0.57485, 0.57941, 0.58374, 0.58756, 0.59058, 0.59276, 0.59437, 0.59574,\
        0.59702, 0.59832, 0.59976, 0.60128, 0.60284, 0.60453, 0.60673, 0.60945, 0.61248, 0.61567, 0.61880,\
        0.62156, 0.62378, 0.62543, 0.62657, 0.62726, 0.62754, 0.62745, 0.62704, 0.62638, 0.62557, 0.62466,\
        0.62367, 0.62260, 0.62146, 0.62026, 0.61905, 0.61783, 0.61659, 0.61530, 0.61396, 0.61259, 0.61119,\
        0.60970, 0.60802, 0.60615, 0.60420, 0.60223, 0.60024, 0.59813, 0.59561, 0.59255, 0.58919, 0.58573,\
        0.58224, 0.57871, 0.57513, 0.57150, 0.56780, 0.56404, 0.56021, 0.55627, 0.55221, 0.54797, 0.54353,\
        0.53882, 0.53382, 0.52851, 0.52291, 0.51711, 0.51134, 0.50583, 0.49991, 0.49349, 0.48675, 0.47970,\
        0.47246, 0.46509, 0.45761, 0.44997, 0.44215, 0.43410, 0.42578, 0.41719, 0.40830, 0.39911, 0.38961],\
        [0.42670, 0.43496, 0.44299, 0.45076, 0.45830, 0.46559, 0.47263, 0.47941, 0.48586, 0.49176, 0.49674,\
        0.50046, 0.50275, 0.50517, 0.50796, 0.51068, 0.51332, 0.51599, 0.51875, 0.52154, 0.52427, 0.52687,\
        0.52939, 0.53188, 0.53441, 0.53704, 0.53985, 0.54287, 0.54613, 0.54963, 0.55336, 0.55732, 0.56148,\
        0.56583, 0.57033, 0.57496, 0.57965, 0.58435, 0.58891, 0.59307, 0.59627, 0.59832, 0.59979, 0.60117,\
        0.60249, 0.60387, 0.60540, 0.60703, 0.60876, 0.61070, 0.61301, 0.61573, 0.61874, 0.62187, 0.62493,\
        0.62772, 0.63009, 0.63196, 0.63333, 0.63423, 0.63468, 0.63472, 0.63441, 0.63380, 0.63299, 0.63202,\
        0.63094, 0.62977, 0.62852, 0.62723, 0.62590, 0.62456, 0.62318, 0.62177, 0.62031, 0.61880, 0.61723,\
        0.61556, 0.61374, 0.61177, 0.60972, 0.60761, 0.60545, 0.60316, 0.60045, 0.59711, 0.59355, 0.58998,\
        0.58642, 0.58284, 0.57921, 0.57553, 0.57179, 0.56802, 0.56419, 0.56029, 0.55626, 0.55208, 0.54769,\
        0.54305, 0.53814, 0.53295, 0.52751, 0.52190, 0.51624, 0.51053, 0.50454, 0.49826, 0.49165, 0.48463,\
        0.47751, 0.47035, 0.46312, 0.45574, 0.44816, 0.44034, 0.43223, 0.42382, 0.41509, 0.40605, 0.39670],\
        [0.43277, 0.44090, 0.44878, 0.45641, 0.46378, 0.47090, 0.47778, 0.48444, 0.49087, 0.49690, 0.50188,\
        0.50559, 0.50832, 0.51083, 0.51347, 0.51615, 0.51881, 0.52148, 0.52419, 0.52691, 0.52958, 0.53219,\
        0.53474, 0.53726, 0.53981, 0.54244, 0.54520, 0.54814, 0.55127, 0.55462, 0.55819, 0.56198, 0.56599,\
        0.57020, 0.57459, 0.57913, 0.58377, 0.58846, 0.59313, 0.59760, 0.60107, 0.60314, 0.60474, 0.60622,\
        0.60770, 0.60924, 0.61090, 0.61270, 0.61464, 0.61681, 0.61927, 0.62204, 0.62504, 0.62815, 0.63121,\
        0.63405, 0.63654, 0.63860, 0.64018, 0.64127, 0.64187, 0.64203, 0.64178, 0.64120, 0.64036, 0.63933,\
        0.63815, 0.63687, 0.63552, 0.63411, 0.63267, 0.63119, 0.62969, 0.62814, 0.62654, 0.62489, 0.62315,\
        0.62131, 0.61933, 0.61723, 0.61501, 0.61271, 0.61028, 0.60765, 0.60463, 0.60113, 0.59746, 0.59386,\
        0.59028, 0.58667, 0.58298, 0.57924, 0.57547, 0.57170, 0.56790, 0.56405, 0.56010, 0.55600, 0.55171,\
        0.54717, 0.54237, 0.53730, 0.53198, 0.52647, 0.52082, 0.51500, 0.50899, 0.50281, 0.49613, 0.48906,\
        0.48216, 0.47530, 0.46837, 0.46128, 0.45396, 0.44637, 0.43846, 0.43022, 0.42165, 0.41275, 0.40353],\
        [0.43844, 0.44642, 0.45415, 0.46161, 0.46878, 0.47568, 0.48230, 0.48867, 0.49480, 0.50074, 0.50579,\
        0.50979, 0.51301, 0.51584, 0.51859, 0.52131, 0.52402, 0.52672, 0.52942, 0.53210, 0.53476, 0.53736,\
        0.53992, 0.54246, 0.54500, 0.54760, 0.55029, 0.55312, 0.55611, 0.55929, 0.56267, 0.56627, 0.57009,\
        0.57413, 0.57836, 0.58275, 0.58724, 0.59179, 0.59628, 0.60063, 0.60433, 0.60695, 0.60898, 0.61078,\
        0.61254, 0.61433, 0.61621, 0.61822, 0.62040, 0.62278, 0.62540, 0.62826, 0.63131, 0.63445, 0.63755,\
        0.64048, 0.64310, 0.64534, 0.64711, 0.64837, 0.64911, 0.64936, 0.64915, 0.64856, 0.64767, 0.64655,\
        0.64527, 0.64388, 0.64241, 0.64089, 0.63933, 0.63773, 0.63609, 0.63442, 0.63268, 0.63087, 0.62897,\
        0.62695, 0.62480, 0.62252, 0.62010, 0.61755, 0.61482, 0.61185, 0.60855, 0.60493, 0.60118, 0.59751,\
        0.59391, 0.59025, 0.58648, 0.58266, 0.57885, 0.57509, 0.57135, 0.56760, 0.56377, 0.55980, 0.55563,\
        0.55122, 0.54654, 0.54158, 0.53636, 0.53089, 0.52519, 0.51927, 0.51311, 0.50664, 0.49974, 0.49302,\
        0.48656, 0.48008, 0.47347, 0.46667, 0.45961, 0.45223, 0.44451, 0.43642, 0.42799, 0.41922, 0.41011],\
        [0.44374, 0.45158, 0.45915, 0.46643, 0.47339, 0.48003, 0.48638, 0.49243, 0.49814, 0.50381, 0.50904,\
        0.51341, 0.51707, 0.52027, 0.52325, 0.52611, 0.52891, 0.53167, 0.53441, 0.53711, 0.53977, 0.54238,\
        0.54494, 0.54747, 0.55000, 0.55254, 0.55515, 0.55785, 0.56068, 0.56367, 0.56684, 0.57022, 0.57383,\
        0.57766, 0.58170, 0.58591, 0.59022, 0.59457, 0.59888, 0.60310, 0.60693, 0.61005, 0.61261, 0.61487,\
        0.61699, 0.61911, 0.62128, 0.62355, 0.62597, 0.62856, 0.63136, 0.63435, 0.63749, 0.64070, 0.64388,\
        0.64693, 0.64971, 0.65213, 0.65409, 0.65552, 0.65639, 0.65670, 0.65649, 0.65586, 0.65489, 0.65367,\
        0.65228, 0.65078, 0.64919, 0.64755, 0.64587, 0.64416, 0.64240, 0.64059, 0.63872, 0.63676, 0.63469,\
        0.63250, 0.63017, 0.62768, 0.62503, 0.62221, 0.61919, 0.61592, 0.61240, 0.60866, 0.60483, 0.60105,\
        0.59739, 0.59363, 0.58969, 0.58575, 0.58192, 0.57820, 0.57458, 0.57097, 0.56730, 0.56349, 0.55948,\
        0.55522, 0.55067, 0.54582, 0.54067, 0.53522, 0.52949, 0.52347, 0.51715, 0.51049, 0.50362, 0.49714,\
        0.49107, 0.48485, 0.47850, 0.47197, 0.46515, 0.45797, 0.45040, 0.44244, 0.43411, 0.42544, 0.41644],\
        [0.44868, 0.45639, 0.46383, 0.47095, 0.47771, 0.48410, 0.49018, 0.49606, 0.50169, 0.50711, 0.51224,\
        0.51679, 0.52076, 0.52428, 0.52751, 0.53056, 0.53349, 0.53635, 0.53916, 0.54190, 0.54459, 0.54722,\
        0.54980, 0.55232, 0.55481, 0.55730, 0.55981, 0.56237, 0.56502, 0.56780, 0.57074, 0.57388, 0.57725,\
        0.58086, 0.58470, 0.58873, 0.59286, 0.59707, 0.60132, 0.60550, 0.60941, 0.61287, 0.61588, 0.61859,\
        0.62113, 0.62361, 0.62610, 0.62866, 0.63133, 0.63415, 0.63712, 0.64026, 0.64351, 0.64683, 0.65014,\
        0.65333, 0.65630, 0.65893, 0.66110, 0.66271, 0.66369, 0.66404, 0.66380, 0.66307, 0.66199, 0.66065,\
        0.65915, 0.65753, 0.65584, 0.65409, 0.65230, 0.65048, 0.64861, 0.64668, 0.64468, 0.64258, 0.64035,\
        0.63799, 0.63546, 0.63276, 0.62988, 0.62679, 0.62348, 0.61995, 0.61620, 0.61230, 0.60836, 0.60448,\
        0.60073, 0.59671, 0.59255, 0.58849, 0.58466, 0.58106, 0.57761, 0.57421, 0.57075, 0.56714, 0.56331,\
        0.55921, 0.55480, 0.55006, 0.54498, 0.53956, 0.53382, 0.52776, 0.52140, 0.51481, 0.50819, 0.50193,\
        0.49589, 0.48964, 0.48350, 0.47724, 0.47064, 0.46360, 0.45613, 0.44825, 0.44000, 0.43142, 0.42251],\
        [0.45329, 0.46088, 0.46821, 0.47522, 0.48185, 0.48802, 0.49381, 0.49946, 0.50508, 0.51043, 0.51547,\
        0.52008, 0.52424, 0.52800, 0.53146, 0.53470, 0.53780, 0.54078, 0.54368, 0.54649, 0.54924, 0.55191,\
        0.55450, 0.55702, 0.55948, 0.56190, 0.56430, 0.56672, 0.56918, 0.57174, 0.57443, 0.57730, 0.58041,\
        0.58378, 0.58744, 0.59133, 0.59533, 0.59942, 0.60365, 0.60786, 0.61188, 0.61560, 0.61899, 0.62211,\
        0.62504, 0.62789, 0.63071, 0.63356, 0.63649, 0.63953, 0.64269, 0.64597, 0.64935, 0.65280, 0.65625,\
        0.65962, 0.66280, 0.66567, 0.66808, 0.66990, 0.67100, 0.67135, 0.67102, 0.67017, 0.66895, 0.66748,\
        0.66585, 0.66413, 0.66234, 0.66050, 0.65861, 0.65669, 0.65472, 0.65269, 0.65057, 0.64834, 0.64597,\
        0.64344, 0.64073, 0.63781, 0.63469, 0.63134, 0.62776, 0.62396, 0.61997, 0.61584, 0.61168, 0.60756,\
        0.60348, 0.59927, 0.59494, 0.59083, 0.58711, 0.58371, 0.58052, 0.57738, 0.57416, 0.57078, 0.56716,\
        0.56323, 0.55897, 0.55434, 0.54933, 0.54395, 0.53822, 0.53218, 0.52588, 0.51942, 0.51299, 0.50673,\
        0.50043, 0.49433, 0.48854, 0.48255, 0.47609, 0.46913, 0.46170, 0.45385, 0.44564, 0.43711, 0.42830],\
        [0.45756, 0.46504, 0.47230, 0.47927, 0.48585, 0.49193, 0.49751, 0.50292, 0.50842, 0.51370, 0.51868,\
        0.52332, 0.52760, 0.53153, 0.53517, 0.53860, 0.54185, 0.54497, 0.54798, 0.55089, 0.55370, 0.55643,\
        0.55905, 0.56159, 0.56402, 0.56638, 0.56868, 0.57094, 0.57322, 0.57554, 0.57796, 0.58055, 0.58337,\
        0.58649, 0.58995, 0.59377, 0.59780, 0.60179, 0.60599, 0.61025, 0.61440, 0.61835, 0.62205, 0.62552,\
        0.62882, 0.63201, 0.63515, 0.63829, 0.64146, 0.64471, 0.64805, 0.65148, 0.65499, 0.65857, 0.66216,\
        0.66572, 0.66913, 0.67227, 0.67497, 0.67703, 0.67826, 0.67859, 0.67813, 0.67710, 0.67571, 0.67410,\
        0.67238, 0.67057, 0.66870, 0.66677, 0.66481, 0.66280, 0.66076, 0.65864, 0.65643, 0.65408, 0.65158,\
        0.64889, 0.64600, 0.64288, 0.63952, 0.63592, 0.63208, 0.62801, 0.62374, 0.61932, 0.61482, 0.61031,\
        0.60581, 0.60130, 0.59686, 0.59281, 0.58932, 0.58625, 0.58340, 0.58056, 0.57762, 0.57447, 0.57106,\
        0.56731, 0.56320, 0.55867, 0.55374, 0.54840, 0.54270, 0.53670, 0.53047, 0.52411, 0.51775, 0.51142,\
        0.50515, 0.49922, 0.49377, 0.48793, 0.48148, 0.47449, 0.46702, 0.45916, 0.45097, 0.44250, 0.43378],\
        [0.46149, 0.46886, 0.47605, 0.48302, 0.48968, 0.49582, 0.50131, 0.50658, 0.51181, 0.51694, 0.52185,\
        0.52649, 0.53084, 0.53490, 0.53871, 0.54229, 0.54569, 0.54895, 0.55208, 0.55509, 0.55800, 0.56080,\
        0.56348, 0.56604, 0.56846, 0.57077, 0.57297, 0.57510, 0.57718, 0.57927, 0.58143, 0.58372, 0.58623,\
        0.58906, 0.59226, 0.59590, 0.60010, 0.60415, 0.60838, 0.61272, 0.61702, 0.62118, 0.62515, 0.62893,\
        0.63255, 0.63605, 0.63948, 0.64288, 0.64629, 0.64973, 0.65323, 0.65679, 0.66042, 0.66411, 0.66784,\
        0.67156, 0.67520, 0.67863, 0.68166, 0.68404, 0.68545, 0.68571, 0.68503, 0.68379, 0.68222, 0.68050,\
        0.67871, 0.67686, 0.67493, 0.67294, 0.67090, 0.66884, 0.66673, 0.66455, 0.66226, 0.65982, 0.65720,\
        0.65437, 0.65131, 0.64800, 0.64443, 0.64059, 0.63650, 0.63215, 0.62758, 0.62284, 0.61796, 0.61302,\
        0.60805, 0.60315, 0.59855, 0.59462, 0.59149, 0.58886, 0.58639, 0.58386, 0.58117, 0.57824, 0.57503,\
        0.57148, 0.56751, 0.56309, 0.55821, 0.55291, 0.54723, 0.54125, 0.53507, 0.52878, 0.52248, 0.51626,\
        0.51021, 0.50455, 0.49922, 0.49319, 0.48666, 0.47957, 0.47200, 0.46410, 0.45593, 0.44754, 0.43892],\
        [0.46508, 0.47230, 0.47938, 0.48631, 0.49303, 0.49941, 0.50482, 0.50996, 0.51505, 0.52006, 0.52492,\
        0.52958, 0.53399, 0.53815, 0.54208, 0.54580, 0.54935, 0.55273, 0.55599, 0.55912, 0.56214, 0.56503,\
        0.56778, 0.57039, 0.57283, 0.57510, 0.57722, 0.57922, 0.58114, 0.58302, 0.58491, 0.58690, 0.58911,\
        0.59167, 0.59461, 0.59782, 0.60179, 0.60630, 0.61081, 0.61533, 0.61980, 0.62416, 0.62836, 0.63240,\
        0.63629, 0.64006, 0.64375, 0.64739, 0.65101, 0.65463, 0.65827, 0.66194, 0.66566, 0.66942, 0.67324,\
        0.67710, 0.68094, 0.68463, 0.68800, 0.69077, 0.69245, 0.69256, 0.69161, 0.69017, 0.68846, 0.68667,\
        0.68489, 0.68304, 0.68106, 0.67901, 0.67692, 0.67481, 0.67267, 0.67045, 0.66811, 0.66560, 0.66287,\
        0.65991, 0.65670, 0.65321, 0.64945, 0.64540, 0.64106, 0.63645, 0.63160, 0.62653, 0.62130, 0.61596,\
        0.61058, 0.60529, 0.60043, 0.59658, 0.59394, 0.59178, 0.58965, 0.58736, 0.58485, 0.58211, 0.57910,\
        0.57572, 0.57190, 0.56757, 0.56273, 0.55743, 0.55175, 0.54576, 0.53959, 0.53333, 0.52708, 0.52096,\
        0.51507, 0.50947, 0.50391, 0.49794, 0.49140, 0.48417, 0.47649, 0.46856, 0.46045, 0.45218, 0.44372],\
        [0.46833, 0.47536, 0.48229, 0.48907, 0.49561, 0.50184, 0.50756, 0.51285, 0.51797, 0.52299, 0.52786,\
        0.53255, 0.53703, 0.54129, 0.54533, 0.54917, 0.55284, 0.55635, 0.55973, 0.56299, 0.56612, 0.56912,\
        0.57197, 0.57464, 0.57712, 0.57940, 0.58146, 0.58335, 0.58513, 0.58684, 0.58852, 0.59021, 0.59212,\
        0.59449, 0.59740, 0.60064, 0.60442, 0.60889, 0.61354, 0.61820, 0.62281, 0.62733, 0.63172, 0.63597,\
        0.64009, 0.64410, 0.64801, 0.65186, 0.65567, 0.65945, 0.66322, 0.66698, 0.67074, 0.67454, 0.67840,\
        0.68234, 0.68631, 0.69020, 0.69380, 0.69687, 0.69901, 0.69886, 0.69775, 0.69625, 0.69446, 0.69265,\
        0.69096, 0.68918, 0.68714, 0.68502, 0.68289, 0.68075, 0.67859, 0.67637, 0.67401, 0.67144, 0.66862,\
        0.66554, 0.66218, 0.65855, 0.65461, 0.65037, 0.64582, 0.64098, 0.63588, 0.63054, 0.62502, 0.61938,\
        0.61373, 0.60820, 0.60310, 0.59922, 0.59716, 0.59524, 0.59330, 0.59110, 0.58866, 0.58605, 0.58322,\
        0.58005, 0.57637, 0.57210, 0.56726, 0.56192, 0.55618, 0.55015, 0.54393, 0.53765, 0.53139, 0.52526,\
        0.51933, 0.51361, 0.50793, 0.50207, 0.49547, 0.48804, 0.48030, 0.47244, 0.46448, 0.45641, 0.44818],\
        [0.47129, 0.47808, 0.48482, 0.49145, 0.49785, 0.50398, 0.50980, 0.51530, 0.52058, 0.52569, 0.53064,\
        0.53541, 0.53998, 0.54433, 0.54847, 0.55242, 0.55619, 0.55982, 0.56331, 0.56668, 0.56994, 0.57306,\
        0.57603, 0.57881, 0.58136, 0.58366, 0.58570, 0.58750, 0.58916, 0.59079, 0.59235, 0.59378, 0.59534,\
        0.59755, 0.60057, 0.60397, 0.60781, 0.61213, 0.61673, 0.62142, 0.62610, 0.63073, 0.63526, 0.63968,\
        0.64400, 0.64820, 0.65231, 0.65634, 0.66031, 0.66424, 0.66813, 0.67197, 0.67576, 0.67953, 0.68336,\
        0.68731, 0.69135, 0.69537, 0.69907, 0.70214, 0.70420, 0.70445, 0.70356, 0.70215, 0.70035, 0.69855,\
        0.69706, 0.69539, 0.69314, 0.69100, 0.68886, 0.68667, 0.68454, 0.68235, 0.67999, 0.67737, 0.67446,\
        0.67126, 0.66779, 0.66402, 0.65994, 0.65554, 0.65082, 0.64578, 0.64046, 0.63492, 0.62920, 0.62340,\
        0.61765, 0.61215, 0.60724, 0.60361, 0.60122, 0.59920, 0.59732, 0.59500, 0.59248, 0.58997, 0.58738,\
        0.58443, 0.58089, 0.57665, 0.57174, 0.56630, 0.56045, 0.55432, 0.54803, 0.54166, 0.53531, 0.52905,\
        0.52294, 0.51697, 0.51104, 0.50497, 0.49827, 0.49090, 0.48332, 0.47569, 0.46803, 0.46026, 0.45232],\
        [0.47402, 0.48054, 0.48705, 0.49355, 0.49993, 0.50599, 0.51185, 0.51750, 0.52296, 0.52823, 0.53331,\
        0.53819, 0.54285, 0.54729, 0.55152, 0.55556, 0.55942, 0.56313, 0.56672, 0.57020, 0.57358, 0.57685,\
        0.57996, 0.58287, 0.58553, 0.58790, 0.58993, 0.59166, 0.59321, 0.59481, 0.59648, 0.59775, 0.59897,\
        0.60112, 0.60415, 0.60768, 0.61157, 0.61582, 0.62033, 0.62498, 0.62968, 0.63437, 0.63900, 0.64356,\
        0.64802, 0.65239, 0.65666, 0.66085, 0.66498, 0.66905, 0.67307, 0.67700, 0.68081, 0.68451, 0.68824,\
        0.69216, 0.69622, 0.70027, 0.70404, 0.70706, 0.70902, 0.70965, 0.70919, 0.70803, 0.70645, 0.70477,\
        0.70330, 0.70149, 0.69904, 0.69711, 0.69491, 0.69261, 0.69056, 0.68845, 0.68610, 0.68340, 0.68039,\
        0.67708, 0.67350, 0.66962, 0.66544, 0.66091, 0.65605, 0.65086, 0.64538, 0.63967, 0.63384, 0.62798,\
        0.62226, 0.61690, 0.61220, 0.60850, 0.60571, 0.60345, 0.60135, 0.59876, 0.59615, 0.59380, 0.59152,\
        0.58885, 0.58543, 0.58113, 0.57608, 0.57048, 0.56448, 0.55822, 0.55181, 0.54532, 0.53881, 0.53235,\
        0.52595, 0.51960, 0.51326, 0.50677, 0.49995, 0.49284, 0.48562, 0.47840, 0.47117, 0.46380, 0.45623],\
        [0.47663, 0.48281, 0.48903, 0.49534, 0.50163, 0.50775, 0.51375, 0.51959, 0.52525, 0.53070, 0.53593,\
        0.54093, 0.54569, 0.55022, 0.55452, 0.55862, 0.56254, 0.56631, 0.56997, 0.57354, 0.57704, 0.58044,\
        0.58371, 0.58678, 0.58957, 0.59207, 0.59417, 0.59582, 0.59726, 0.59875, 0.60065, 0.60205, 0.60337,\
        0.60549, 0.60840, 0.61185, 0.61569, 0.61986, 0.62427, 0.62886, 0.63353, 0.63823, 0.64293, 0.64759,\
        0.65218, 0.65669, 0.66111, 0.66543, 0.66969, 0.67390, 0.67807, 0.68216, 0.68605, 0.68966, 0.69319,\
        0.69711, 0.70124, 0.70531, 0.70903, 0.71201, 0.71401, 0.71491, 0.71483, 0.71401, 0.71272, 0.71120,\
        0.70957, 0.70762, 0.70542, 0.70340, 0.70098, 0.69867, 0.69678, 0.69472, 0.69234, 0.68953, 0.68639,\
        0.68297, 0.67930, 0.67535, 0.67110, 0.66649, 0.66151, 0.65619, 0.65057, 0.64476, 0.63885, 0.63298,\
        0.62731, 0.62204, 0.61739, 0.61352, 0.61036, 0.60767, 0.60510, 0.60238, 0.59970, 0.59758, 0.59562,\
        0.59328, 0.58991, 0.58542, 0.58014, 0.57433, 0.56817, 0.56180, 0.55527, 0.54864, 0.54196, 0.53525,\
        0.52852, 0.52176, 0.51495, 0.50806, 0.50117, 0.49422, 0.48744, 0.48078, 0.47408, 0.46719, 0.46001],\
        [0.47927, 0.48506, 0.49086, 0.49689, 0.50314, 0.50940, 0.51562, 0.52170, 0.52757, 0.53320, 0.53858,\
        0.54370, 0.54856, 0.55315, 0.55750, 0.56163, 0.56558, 0.56938, 0.57307, 0.57669, 0.58027, 0.58380,\
        0.58725, 0.59050, 0.59342, 0.59602, 0.59836, 0.59995, 0.60140, 0.60292, 0.60470, 0.60639, 0.60810,\
        0.61029, 0.61309, 0.61641, 0.62014, 0.62419, 0.62849, 0.63297, 0.63759, 0.64228, 0.64702, 0.65175,\
        0.65646, 0.66110, 0.66565, 0.67010, 0.67446, 0.67880, 0.68314, 0.68747, 0.69160, 0.69524, 0.69856,\
        0.70239, 0.70667, 0.71073, 0.71436, 0.71730, 0.71935, 0.72044, 0.72065, 0.72013, 0.71909, 0.71770,\
        0.71605, 0.71413, 0.71202, 0.70982, 0.70746, 0.70523, 0.70323, 0.70112, 0.69866, 0.69569, 0.69241,\
        0.68891, 0.68517, 0.68118, 0.67690, 0.67224, 0.66717, 0.66173, 0.65599, 0.65007, 0.64410, 0.63821,\
        0.63256, 0.62731, 0.62261, 0.61853, 0.61502, 0.61191, 0.60901, 0.60618, 0.60356, 0.60145, 0.59957,\
        0.59761, 0.59406, 0.58924, 0.58369, 0.57771, 0.57145, 0.56501, 0.55841, 0.55168, 0.54485, 0.53792,\
        0.53090, 0.52376, 0.51652, 0.50920, 0.50198, 0.49531, 0.48915, 0.48312, 0.47700, 0.47059, 0.46380],\
        [0.48213, 0.48754, 0.49289, 0.49842, 0.50469, 0.51124, 0.51770, 0.52399, 0.53004, 0.53583, 0.54135,\
        0.54657, 0.55150, 0.55614, 0.56051, 0.56464, 0.56857, 0.57235, 0.57602, 0.57965, 0.58326, 0.58688,\
        0.59049, 0.59398, 0.59703, 0.59958, 0.60205, 0.60394, 0.60557, 0.60725, 0.60905, 0.61092, 0.61291,\
        0.61523, 0.61800, 0.62121, 0.62481, 0.62872, 0.63289, 0.63726, 0.64180, 0.64646, 0.65121, 0.65602,\
        0.66084, 0.66562, 0.67031, 0.67487, 0.67931, 0.68373, 0.68823, 0.69282, 0.69736, 0.70130, 0.70476,\
        0.70856, 0.71266, 0.71662, 0.72014, 0.72301, 0.72509, 0.72632, 0.72673, 0.72643, 0.72558, 0.72431,\
        0.72271, 0.72084, 0.71877, 0.71655, 0.71425, 0.71200, 0.70981, 0.70750, 0.70486, 0.70180, 0.69843,\
        0.69486, 0.69106, 0.68708, 0.68283, 0.67814, 0.67298, 0.66740, 0.66154, 0.65551, 0.64946, 0.64353,\
        0.63786, 0.63257, 0.62776, 0.62346, 0.61964, 0.61620, 0.61302, 0.61007, 0.60741, 0.60514, 0.60315,\
        0.60100, 0.59726, 0.59221, 0.58652, 0.58051, 0.57428, 0.56786, 0.56127, 0.55452, 0.54760, 0.54053,\
        0.53330, 0.52592, 0.51840, 0.51083, 0.50345, 0.49704, 0.49128, 0.48579, 0.48019, 0.47420, 0.46771],\
        [0.48539, 0.49059, 0.49554, 0.50097, 0.50712, 0.51367, 0.52022, 0.52662, 0.53279, 0.53869, 0.54430,\
        0.54959, 0.55456, 0.55922, 0.56358, 0.56767, 0.57154, 0.57525, 0.57885, 0.58242, 0.58600, 0.58963,\
        0.59331, 0.59707, 0.60039, 0.60302, 0.60548, 0.60767, 0.60964, 0.61156, 0.61352, 0.61557, 0.61776,\
        0.62021, 0.62299, 0.62613, 0.62960, 0.63337, 0.63740, 0.64165, 0.64609, 0.65070, 0.65546, 0.66032,\
        0.66526, 0.67022, 0.67509, 0.67979, 0.68429, 0.68870, 0.69331, 0.69815, 0.70290, 0.70728, 0.71121,\
        0.71515, 0.71914, 0.72297, 0.72638, 0.72919, 0.73126, 0.73256, 0.73310, 0.73295, 0.73224, 0.73106,\
        0.72952, 0.72769, 0.72564, 0.72344, 0.72114, 0.71880, 0.71641, 0.71387, 0.71106, 0.70791, 0.70448,\
        0.70083, 0.69695, 0.69301, 0.68886, 0.68414, 0.67884, 0.67311, 0.66709, 0.66093, 0.65480, 0.64881,\
        0.64309, 0.63772, 0.63278, 0.62828, 0.62419, 0.62045, 0.61701, 0.61383, 0.61092, 0.60827, 0.60572,\
        0.60285, 0.59904, 0.59404, 0.58854, 0.58274, 0.57670, 0.57043, 0.56395, 0.55724, 0.55033, 0.54321,\
        0.53590, 0.52844, 0.52087, 0.51334, 0.50613, 0.49960, 0.49411, 0.48907, 0.48386, 0.47815, 0.47184],\
        [0.48905, 0.49435, 0.49936, 0.50437, 0.51042, 0.51682, 0.52330, 0.52969, 0.53589, 0.54183, 0.54749,\
        0.55282, 0.55781, 0.56246, 0.56677, 0.57079, 0.57455, 0.57813, 0.58161, 0.58507, 0.58856, 0.59210,\
        0.59569, 0.59938, 0.60300, 0.60607, 0.60881, 0.61131, 0.61362, 0.61583, 0.61803, 0.62027, 0.62263,\
        0.62518, 0.62798, 0.63106, 0.63443, 0.63806, 0.64194, 0.64605, 0.65038, 0.65492, 0.65966, 0.66459,\
        0.66966, 0.67483, 0.67997, 0.68491, 0.68951, 0.69386, 0.69845, 0.70358, 0.70862, 0.71334, 0.71772,\
        0.72190, 0.72593, 0.72970, 0.73303, 0.73578, 0.73783, 0.73915, 0.73975, 0.73968, 0.73904, 0.73793,\
        0.73643, 0.73463, 0.73259, 0.73038, 0.72803, 0.72558, 0.72302, 0.72028, 0.71730, 0.71404, 0.71054,\
        0.70680, 0.70288, 0.69900, 0.69499, 0.69010, 0.68461, 0.67871, 0.67251, 0.66622, 0.66000, 0.65395,\
        0.64817, 0.64272, 0.63764, 0.63295, 0.62862, 0.62461, 0.62088, 0.61738, 0.61407, 0.61087, 0.60760,\
        0.60393, 0.59967, 0.59506, 0.58999, 0.58456, 0.57884, 0.57283, 0.56653, 0.55995, 0.55311, 0.54605,\
        0.53879, 0.53139, 0.52395, 0.51660, 0.50956, 0.50320, 0.49796, 0.49305, 0.48810, 0.48249, 0.47617],\
        [0.49292, 0.49829, 0.50355, 0.50872, 0.51443, 0.52057, 0.52687, 0.53317, 0.53933, 0.54527, 0.55093,\
        0.55627, 0.56126, 0.56587, 0.57012, 0.57403, 0.57765, 0.58105, 0.58434, 0.58765, 0.59105, 0.59452,\
        0.59803, 0.60166, 0.60545, 0.60893, 0.61207, 0.61493, 0.61758, 0.62009, 0.62253, 0.62497, 0.62748,\
        0.63011, 0.63293, 0.63596, 0.63922, 0.64272, 0.64645, 0.65041, 0.65461, 0.65905, 0.66375, 0.66871,\
        0.67391, 0.67932, 0.68483, 0.69019, 0.69508, 0.69952, 0.70419, 0.70943, 0.71468, 0.71969, 0.72439,\
        0.72881, 0.73294, 0.73671, 0.74001, 0.74272, 0.74475, 0.74606, 0.74666, 0.74661, 0.74599, 0.74489,\
        0.74341, 0.74161, 0.73956, 0.73731, 0.73489, 0.73233, 0.72960, 0.72669, 0.72355, 0.72017, 0.71657,\
        0.71278, 0.70888, 0.70499, 0.70081, 0.69569, 0.69012, 0.68405, 0.67767, 0.67126, 0.66497, 0.65888,\
        0.65305, 0.64753, 0.64234, 0.63748, 0.63293, 0.62866, 0.62461, 0.62075, 0.61700, 0.61325, 0.60937,\
        0.60518, 0.60073, 0.59611, 0.59131, 0.58627, 0.58091, 0.57519, 0.56913, 0.56272, 0.55602, 0.54907,\
        0.54195, 0.53472, 0.52751, 0.52047, 0.51380, 0.50776, 0.50237, 0.49765, 0.49288, 0.48706, 0.48056],\
        [0.49691, 0.50232, 0.50767, 0.51307, 0.51873, 0.52469, 0.53082, 0.53699, 0.54308, 0.54898, 0.55463,\
        0.55996, 0.56492, 0.56949, 0.57366, 0.57745, 0.58089, 0.58407, 0.58710, 0.59020, 0.59354, 0.59704,\
        0.60060, 0.60436, 0.60821, 0.61194, 0.61541, 0.61860, 0.62157, 0.62436, 0.62703, 0.62965, 0.63228,\
        0.63498, 0.63780, 0.64077, 0.64393, 0.64729, 0.65086, 0.65466, 0.65871, 0.66303, 0.66765, 0.67260,\
        0.67789, 0.68351, 0.68940, 0.69536, 0.70074, 0.70553, 0.71047, 0.71574, 0.72111, 0.72632, 0.73126,\
        0.73587, 0.74013, 0.74396, 0.74727, 0.74996, 0.75196, 0.75323, 0.75379, 0.75369, 0.75304, 0.75191,\
        0.75040, 0.74858, 0.74650, 0.74420, 0.74170, 0.73901, 0.73614, 0.73306, 0.72977, 0.72625, 0.72253,\
        0.71863, 0.71460, 0.71043, 0.70588, 0.70080, 0.69527, 0.68898, 0.68241, 0.67595, 0.66966, 0.66357,\
        0.65773, 0.65215, 0.64687, 0.64186, 0.63711, 0.63260, 0.62826, 0.62404, 0.61988, 0.61568, 0.61133,\
        0.60677, 0.60206, 0.59738, 0.59283, 0.58813, 0.58311, 0.57768, 0.57184, 0.56561, 0.55906, 0.55226,\
        0.54531, 0.53831, 0.53137, 0.52465, 0.51831, 0.51250, 0.50730, 0.50276, 0.49764, 0.49146, 0.48478],\
        [0.50114, 0.50654, 0.51198, 0.51749, 0.52315, 0.52900, 0.53500, 0.54105, 0.54705, 0.55291, 0.55854,\
        0.56386, 0.56880, 0.57333, 0.57741, 0.58108, 0.58436, 0.58729, 0.58999, 0.59272, 0.59592, 0.59974,\
        0.60336, 0.60726, 0.61128, 0.61521, 0.61894, 0.62242, 0.62565, 0.62867, 0.63154, 0.63431, 0.63703,\
        0.63976, 0.64256, 0.64546, 0.64851, 0.65172, 0.65513, 0.65875, 0.66263, 0.66679, 0.67129, 0.67619,\
        0.68152, 0.68726, 0.69336, 0.69970, 0.70572, 0.71120, 0.71664, 0.72219, 0.72775, 0.73316, 0.73829,\
        0.74309, 0.74748, 0.75139, 0.75475, 0.75744, 0.75940, 0.76060, 0.76107, 0.76087, 0.76013, 0.75893,\
        0.75737, 0.75551, 0.75339, 0.75102, 0.74844, 0.74563, 0.74261, 0.73937, 0.73591, 0.73223, 0.72834,\
        0.72425, 0.71997, 0.71545, 0.71060, 0.70542, 0.69985, 0.69313, 0.68660, 0.68027, 0.67408, 0.66804,\
        0.66221, 0.65661, 0.65125, 0.64612, 0.64121, 0.63647, 0.63187, 0.62735, 0.62285, 0.61830, 0.61362,\
        0.60881, 0.60389, 0.59919, 0.59483, 0.59036, 0.58559, 0.58039, 0.57471, 0.56862, 0.56220, 0.55556,\
        0.54878, 0.54199, 0.53531, 0.52886, 0.52276, 0.51711, 0.51191, 0.50694, 0.50148, 0.49529, 0.48865],\
        [0.50579, 0.51109, 0.51651, 0.52201, 0.52764, 0.53342, 0.53931, 0.54525, 0.55117, 0.55698, 0.56260,\
        0.56792, 0.57286, 0.57735, 0.58137, 0.58493, 0.58809, 0.59085, 0.59322, 0.59541, 0.59801, 0.60188,\
        0.60614, 0.61039, 0.61464, 0.61878, 0.62272, 0.62641, 0.62985, 0.63305, 0.63606, 0.63893, 0.64170,\
        0.64443, 0.64717, 0.64998, 0.65290, 0.65597, 0.65920, 0.66265, 0.66634, 0.67032, 0.67467, 0.67946,\
        0.68478, 0.69062, 0.69689, 0.70349, 0.71014, 0.71647, 0.72259, 0.72861, 0.73448, 0.74013, 0.74546,\
        0.75041, 0.75493, 0.75895, 0.76237, 0.76509, 0.76701, 0.76812, 0.76844, 0.76809, 0.76720, 0.76589,\
        0.76426, 0.76236, 0.76019, 0.75776, 0.75508, 0.75216, 0.74899, 0.74560, 0.74197, 0.73811, 0.73401,\
        0.72967, 0.72509, 0.72020, 0.71494, 0.70924, 0.70287, 0.69647, 0.69046, 0.68440, 0.67833, 0.67235,\
        0.66654, 0.66093, 0.65552, 0.65030, 0.64524, 0.64032, 0.63550, 0.63073, 0.62597, 0.62117, 0.61630,\
        0.61138, 0.60649, 0.60183, 0.59736, 0.59301, 0.58844, 0.58336, 0.57776, 0.57174, 0.56540, 0.55886,\
        0.55223, 0.54563, 0.53915, 0.53289, 0.52693, 0.52130, 0.51594, 0.51060, 0.50491, 0.49876, 0.49224],\
        [0.51066, 0.51587, 0.52119, 0.52663, 0.53218, 0.53786, 0.54364, 0.54949, 0.55533, 0.56111, 0.56673,\
        0.57208, 0.57705, 0.58152, 0.58546, 0.58894, 0.59206, 0.59479, 0.59706, 0.59893, 0.60122, 0.60496,\
        0.60941, 0.61393, 0.61838, 0.62268, 0.62677, 0.63061, 0.63419, 0.63751, 0.64060, 0.64350, 0.64626,\
        0.64894, 0.65160, 0.65429, 0.65706, 0.65997, 0.66304, 0.66631, 0.66981, 0.67360, 0.67777, 0.68243,\
        0.68771, 0.69372, 0.70033, 0.70735, 0.71454, 0.72161, 0.72844, 0.73499, 0.74125, 0.74717, 0.75269,\
        0.75779, 0.76244, 0.76658, 0.77009, 0.77284, 0.77474, 0.77572, 0.77584, 0.77526, 0.77416, 0.77273,\
        0.77102, 0.76908, 0.76687, 0.76439, 0.76162, 0.75858, 0.75528, 0.75173, 0.74793, 0.74387, 0.73956,\
        0.73498, 0.73009, 0.72486, 0.71923, 0.71316, 0.70672, 0.70041, 0.69460, 0.68857, 0.68250, 0.67655,\
        0.67077, 0.66516, 0.65972, 0.65442, 0.64926, 0.64419, 0.63918, 0.63422, 0.62926, 0.62429, 0.61931,\
        0.61434, 0.60947, 0.60474, 0.60015, 0.59616, 0.59165, 0.58652, 0.58089, 0.57485, 0.56854, 0.56207,\
        0.55556, 0.54909, 0.54276, 0.53662, 0.53072, 0.52505, 0.51952, 0.51397, 0.50822, 0.50217, 0.49563],\
        [0.51563, 0.52074, 0.52596, 0.53128, 0.53672, 0.54228, 0.54794, 0.55367, 0.55944, 0.56518, 0.57081,\
        0.57623, 0.58128, 0.58580, 0.58964, 0.59295, 0.59611, 0.59890, 0.60130, 0.60336, 0.60585, 0.60935,\
        0.61357, 0.61806, 0.62255, 0.62693, 0.63109, 0.63501, 0.63866, 0.64202, 0.64512, 0.64800, 0.65069,\
        0.65326, 0.65577, 0.65830, 0.66091, 0.66365, 0.66657, 0.66969, 0.67303, 0.67666, 0.68065, 0.68515,\
        0.69035, 0.69648, 0.70356, 0.71119, 0.71904, 0.72682, 0.73431, 0.74142, 0.74809, 0.75427, 0.75997,\
        0.76518, 0.76993, 0.77418, 0.77781, 0.78063, 0.78249, 0.78334, 0.78318, 0.78227, 0.78094, 0.77936,\
        0.77761, 0.77566, 0.77344, 0.77091, 0.76804, 0.76488, 0.76144, 0.75774, 0.75378, 0.74956, 0.74505,\
        0.74023, 0.73509, 0.72960, 0.72374, 0.71755, 0.71119, 0.70503, 0.69911, 0.69270, 0.68662, 0.68071,\
        0.67496, 0.66936, 0.66390, 0.65854, 0.65328, 0.64808, 0.64293, 0.63780, 0.63269, 0.62759, 0.62252,\
        0.61754, 0.61269, 0.60804, 0.60375, 0.59971, 0.59494, 0.58970, 0.58395, 0.57783, 0.57149, 0.56507,\
        0.55865, 0.55229, 0.54606, 0.54000, 0.53411, 0.52837, 0.52271, 0.51701, 0.51116, 0.50509, 0.49849],\
        [0.52064, 0.52565, 0.53073, 0.53592, 0.54121, 0.54662, 0.55213, 0.55774, 0.56340, 0.56908, 0.57472,\
        0.58022, 0.58543, 0.59010, 0.59388, 0.59676, 0.59988, 0.60291, 0.60550, 0.60800, 0.61082, 0.61429,\
        0.61833, 0.62269, 0.62712, 0.63148, 0.63565, 0.63958, 0.64323, 0.64657, 0.64961, 0.65238, 0.65493,\
        0.65732, 0.65962, 0.66194, 0.66434, 0.66691, 0.66969, 0.67271, 0.67597, 0.67949, 0.68336, 0.68773,\
        0.69287, 0.69912, 0.70673, 0.71509, 0.72368, 0.73217, 0.74033, 0.74798, 0.75503, 0.76147, 0.76729,\
        0.77255, 0.77732, 0.78167, 0.78546, 0.78833, 0.79013, 0.79087, 0.79034, 0.78901, 0.78745, 0.78576,\
        0.78398, 0.78208, 0.77989, 0.77730, 0.77434, 0.77105, 0.76748, 0.76365, 0.75955, 0.75517, 0.75050,\
        0.74550, 0.74017, 0.73449, 0.72849, 0.72222, 0.71584, 0.70945, 0.70294, 0.69677, 0.69080, 0.68489,\
        0.67917, 0.67359, 0.66810, 0.66269, 0.65734, 0.65202, 0.64673, 0.64146, 0.63620, 0.63098, 0.62582,\
        0.62078, 0.61592, 0.61132, 0.60698, 0.60265, 0.59794, 0.59265, 0.58673, 0.58049, 0.57413, 0.56776,\
        0.56142, 0.55517, 0.54903, 0.54301, 0.53711, 0.53130, 0.52551, 0.51965, 0.51361, 0.50725, 0.50039],\
        [0.52566, 0.53053, 0.53547, 0.54049, 0.54561, 0.55084, 0.55617, 0.56161, 0.56713, 0.57272, 0.57831,\
        0.58386, 0.58923, 0.59419, 0.59827, 0.60075, 0.60357, 0.60675, 0.60971, 0.61264, 0.61580, 0.61937,\
        0.62336, 0.62761, 0.63196, 0.63626, 0.64039, 0.64427, 0.64785, 0.65110, 0.65401, 0.65661, 0.65893,\
        0.66105, 0.66306, 0.66507, 0.66722, 0.66959, 0.67227, 0.67526, 0.67855, 0.68209, 0.68594, 0.69029,\
        0.69551, 0.70208, 0.71020, 0.71919, 0.72851, 0.73773, 0.74654, 0.75471, 0.76214, 0.76879, 0.77470,\
        0.77991, 0.78455, 0.78891, 0.79291, 0.79581, 0.79741, 0.79814, 0.79708, 0.79535, 0.79370, 0.79189,\
        0.79013, 0.78837, 0.78625, 0.78357, 0.78046, 0.77704, 0.77336, 0.76943, 0.76523, 0.76074, 0.75594,\
        0.75080, 0.74533, 0.73952, 0.73342, 0.72708, 0.72060, 0.71405, 0.70755, 0.70134, 0.69508, 0.68914,\
        0.68347, 0.67789, 0.67237, 0.66689, 0.66145, 0.65602, 0.65059, 0.64517, 0.63976, 0.63440, 0.62910,\
        0.62393, 0.61895, 0.61420, 0.60967, 0.60521, 0.60058, 0.59510, 0.58896, 0.58265, 0.57634, 0.57006,\
        0.56384, 0.55769, 0.55164, 0.54567, 0.53976, 0.53389, 0.52799, 0.52198, 0.51576, 0.50923, 0.50235],\
        [0.53064, 0.53537, 0.54013, 0.54496, 0.54988, 0.55490, 0.56002, 0.56526, 0.57059, 0.57600, 0.58147,\
        0.58696, 0.59236, 0.59747, 0.60173, 0.60473, 0.60757, 0.61071, 0.61393, 0.61721, 0.62067, 0.62441,\
        0.62843, 0.63264, 0.63693, 0.64115, 0.64520, 0.64900, 0.65247, 0.65558, 0.65829, 0.66063, 0.66263,\
        0.66437, 0.66597, 0.66757, 0.66935, 0.67147, 0.67406, 0.67712, 0.68060, 0.68437, 0.68837, 0.69283,\
        0.69827, 0.70531, 0.71387, 0.72344, 0.73346, 0.74343, 0.75293, 0.76166, 0.76944, 0.77628, 0.78228,\
        0.78740, 0.79169, 0.79579, 0.79994, 0.80284, 0.80421, 0.80454, 0.80329, 0.80157, 0.79981, 0.79785,\
        0.79606, 0.79459, 0.79251, 0.78962, 0.78635, 0.78283, 0.77907, 0.77507, 0.77081, 0.76625, 0.76137,\
        0.75614, 0.75057, 0.74468, 0.73850, 0.73210, 0.72555, 0.71893, 0.71233, 0.70580, 0.69939, 0.69360,\
        0.68795, 0.68233, 0.67673, 0.67116, 0.66561, 0.66006, 0.65449, 0.64891, 0.64333, 0.63777, 0.63229,\
        0.62690, 0.62167, 0.61662, 0.61173, 0.60693, 0.60205, 0.59644, 0.59035, 0.58419, 0.57806, 0.57196,\
        0.56590, 0.55989, 0.55392, 0.54800, 0.54210, 0.53618, 0.53021, 0.52409, 0.51778, 0.51120, 0.50437],\
        [0.53557, 0.54013, 0.54470, 0.54932, 0.55401, 0.55879, 0.56367, 0.56866, 0.57375, 0.57894, 0.58419,\
        0.58948, 0.59473, 0.59976, 0.60416, 0.60782, 0.61116, 0.61456, 0.61806, 0.62166, 0.62540, 0.62933,\
        0.63343, 0.63765, 0.64190, 0.64605, 0.65002, 0.65370, 0.65703, 0.65994, 0.66239, 0.66440, 0.66598,\
        0.66721, 0.66824, 0.66926, 0.67051, 0.67225, 0.67470, 0.67794, 0.68184, 0.68612, 0.69054, 0.69521,\
        0.70088, 0.70832, 0.71732, 0.72750, 0.73832, 0.74915, 0.75945, 0.76880, 0.77695, 0.78392, 0.79002,\
        0.79534, 0.79910, 0.80292, 0.80665, 0.80934, 0.81061, 0.81064, 0.80954, 0.80786, 0.80594, 0.80390,\
        0.80203, 0.80071, 0.79846, 0.79530, 0.79195, 0.78839, 0.78459, 0.78057, 0.77630, 0.77173, 0.76680,\
        0.76151, 0.75588, 0.74992, 0.74369, 0.73725, 0.73065, 0.72398, 0.71731, 0.71072, 0.70439, 0.69843,\
        0.69267, 0.68689, 0.68115, 0.67548, 0.66982, 0.66414, 0.65841, 0.65265, 0.64687, 0.64109, 0.63535,\
        0.62968, 0.62411, 0.61864, 0.61325, 0.60786, 0.60234, 0.59673, 0.59102, 0.58522, 0.57938, 0.57353,\
        0.56766, 0.56179, 0.55592, 0.55006, 0.54417, 0.53824, 0.53221, 0.52605, 0.51970, 0.51312, 0.50632],\
        [0.54044, 0.54481, 0.54917, 0.55355, 0.55799, 0.56251, 0.56713, 0.57184, 0.57667, 0.58157, 0.58655,\
        0.59156, 0.59658, 0.60153, 0.60619, 0.61040, 0.61431, 0.61814, 0.62199, 0.62591, 0.62993, 0.63406,\
        0.63828, 0.64254, 0.64677, 0.65087, 0.65475, 0.65830, 0.66145, 0.66412, 0.66627, 0.66786, 0.66892,\
        0.66950, 0.66977, 0.66997, 0.67042, 0.67154, 0.67371, 0.67713, 0.68172, 0.68697, 0.69224, 0.69734,\
        0.70317, 0.71069, 0.72001, 0.73093, 0.74274, 0.75462, 0.76591, 0.77606, 0.78470, 0.79175, 0.79780,\
        0.80333, 0.80741, 0.81084, 0.81386, 0.81600, 0.81697, 0.81682, 0.81574, 0.81408, 0.81211, 0.81005,\
        0.80812, 0.80625, 0.80374, 0.80058, 0.79731, 0.79373, 0.78991, 0.78593, 0.78171, 0.77715, 0.77221,\
        0.76689, 0.76120, 0.75521, 0.74894, 0.74247, 0.73585, 0.72916, 0.72247, 0.71589, 0.70953, 0.70346,\
        0.69749, 0.69145, 0.68560, 0.67984, 0.67407, 0.66825, 0.66235, 0.65639, 0.65038, 0.64435, 0.63832,\
        0.63232, 0.62637, 0.62045, 0.61454, 0.60862, 0.60271, 0.59700, 0.59151, 0.58604, 0.58050, 0.57488,\
        0.56920, 0.56347, 0.55769, 0.55189, 0.54602, 0.54009, 0.53405, 0.52787, 0.52151, 0.51495, 0.50817],\
        [0.54523, 0.54939, 0.55352, 0.55766, 0.56183, 0.56607, 0.57040, 0.57484, 0.57938, 0.58402, 0.58872,\
        0.59347, 0.59829, 0.60325, 0.60815, 0.61279, 0.61720, 0.62147, 0.62571, 0.62996, 0.63424, 0.63857,\
        0.64292, 0.64724, 0.65148, 0.65553, 0.65931, 0.66273, 0.66568, 0.66809, 0.66988, 0.67099, 0.67142,\
        0.67122, 0.67051, 0.66958, 0.66885, 0.66891, 0.67043, 0.67389, 0.67933, 0.68613, 0.69301, 0.69889,\
        0.70463, 0.71159, 0.72117, 0.73315, 0.74631, 0.75951, 0.77199, 0.78321, 0.79262, 0.80010, 0.80617,\
        0.81158, 0.81581, 0.81901, 0.82141, 0.82293, 0.82347, 0.82307, 0.82190, 0.82019, 0.81818, 0.81603,\
        0.81386, 0.81156, 0.80887, 0.80579, 0.80250, 0.79878, 0.79505, 0.79118, 0.78704, 0.78253, 0.77759,\
        0.77222, 0.76650, 0.76048, 0.75419, 0.74771, 0.74108, 0.73438, 0.72768, 0.72107, 0.71462, 0.70832,\
        0.70206, 0.69590, 0.69006, 0.68424, 0.67834, 0.67236, 0.66627, 0.66009, 0.65384, 0.64754, 0.64122,\
        0.63489, 0.62857, 0.62224, 0.61590, 0.60956, 0.60334, 0.59751, 0.59216, 0.58693, 0.58161, 0.57617,\
        0.57062, 0.56499, 0.55929, 0.55354, 0.54770, 0.54178, 0.53574, 0.52956, 0.52321, 0.51667, 0.50992],\
        [0.54993, 0.55388, 0.55777, 0.56164, 0.56553, 0.56949, 0.57353, 0.57768, 0.58196, 0.58637, 0.59087,\
        0.59542, 0.60015, 0.60516, 0.61023, 0.61518, 0.61997, 0.62464, 0.62923, 0.63379, 0.63834, 0.64286,\
        0.64733, 0.65172, 0.65595, 0.65996, 0.66364, 0.66691, 0.66965, 0.67177, 0.67317, 0.67376, 0.67349,\
        0.67236, 0.67046, 0.66806, 0.66563, 0.66398, 0.66415, 0.66710, 0.67323, 0.68200, 0.69178, 0.69944,\
        0.70427, 0.70955, 0.71985, 0.73367, 0.74868, 0.76341, 0.77715, 0.78962, 0.80024, 0.80830, 0.81487,\
        0.82020, 0.82427, 0.82715, 0.82901, 0.82995, 0.83003, 0.82932, 0.82797, 0.82615, 0.82403, 0.82174,\
        0.81934, 0.81678, 0.81394, 0.81083, 0.80744, 0.80381, 0.80016, 0.79633, 0.79229, 0.78785, 0.78287,\
        0.77746, 0.77171, 0.76567, 0.75938, 0.75290, 0.74626, 0.73955, 0.73283, 0.72616, 0.71959, 0.71310,\
        0.70669, 0.70053, 0.69462, 0.68863, 0.68261, 0.67645, 0.67015, 0.66373, 0.65723, 0.65067, 0.64407,\
        0.63744, 0.63080, 0.62415, 0.61750, 0.61089, 0.60445, 0.59852, 0.59326, 0.58812, 0.58286, 0.57748,\
        0.57199, 0.56641, 0.56077, 0.55504, 0.54924, 0.54333, 0.53730, 0.53113, 0.52479, 0.51828, 0.51156],\
        [0.55456, 0.55827, 0.56191, 0.56550, 0.56911, 0.57277, 0.57652, 0.58040, 0.58443, 0.58866, 0.59309,\
        0.59761, 0.60224, 0.60729, 0.61247, 0.61763, 0.62271, 0.62770, 0.63261, 0.63744, 0.64221, 0.64690,\
        0.65149, 0.65593, 0.66016, 0.66411, 0.66768, 0.67079, 0.67331, 0.67512, 0.67611, 0.67615, 0.67513,\
        0.67299, 0.66973, 0.66554, 0.66086, 0.65661, 0.65430, 0.65567, 0.66163, 0.67168, 0.68425, 0.69616,\
        0.69874, 0.70273, 0.71561, 0.73221, 0.74958, 0.76598, 0.78084, 0.79416, 0.80602, 0.81559, 0.82304,\
        0.82864, 0.83257, 0.83509, 0.83645, 0.83687, 0.83650, 0.83547, 0.83389, 0.83191, 0.82966, 0.82721,\
        0.82462, 0.82186, 0.81891, 0.81574, 0.81236, 0.80881, 0.80516, 0.80136, 0.79748, 0.79306, 0.78795,\
        0.78249, 0.77675, 0.77072, 0.76445, 0.75796, 0.75133, 0.74461, 0.73785, 0.73112, 0.72444, 0.71786,\
        0.71142, 0.70519, 0.69904, 0.69294, 0.68680, 0.68046, 0.67393, 0.66727, 0.66053, 0.65373, 0.64688,\
        0.64000, 0.63310, 0.62621, 0.61938, 0.61268, 0.60623, 0.60026, 0.59494, 0.58968, 0.58429, 0.57883,\
        0.57333, 0.56777, 0.56214, 0.55644, 0.55065, 0.54476, 0.53874, 0.53259, 0.52627, 0.51978, 0.51310],\
        [0.55910, 0.56257, 0.56594, 0.56926, 0.57257, 0.57594, 0.57940, 0.58300, 0.58679, 0.59083, 0.59521,\
        0.59990, 0.60459, 0.60958, 0.61482, 0.62014, 0.62545, 0.63070, 0.63586, 0.64093, 0.64589, 0.65072,\
        0.65539, 0.65986, 0.66407, 0.66794, 0.67138, 0.67431, 0.67658, 0.67807, 0.67864, 0.67811, 0.67634,\
        0.67316, 0.66847, 0.66230, 0.65493, 0.64718, 0.64088, 0.63906, 0.64343, 0.65282, 0.66464, 0.67552,\
        0.68318, 0.69257, 0.70838, 0.72851, 0.74865, 0.76681, 0.78294, 0.79739, 0.81068, 0.82196, 0.83056,\
        0.83661, 0.84049, 0.84266, 0.84355, 0.84351, 0.84275, 0.84142, 0.83963, 0.83748, 0.83506, 0.83246,\
        0.82971, 0.82681, 0.82376, 0.82053, 0.81715, 0.81363, 0.81000, 0.80627, 0.80242, 0.79786, 0.79261,\
        0.78722, 0.78157, 0.77559, 0.76933, 0.76285, 0.75621, 0.74947, 0.74267, 0.73587, 0.72912, 0.72245,\
        0.71592, 0.70953, 0.70325, 0.69711, 0.69089, 0.68430, 0.67751, 0.67064, 0.66370, 0.65669, 0.64963,\
        0.64254, 0.63544, 0.62839, 0.62145, 0.61473, 0.60837, 0.60255, 0.59711, 0.59141, 0.58575, 0.58018,\
        0.57463, 0.56906, 0.56343, 0.55774, 0.55196, 0.54608, 0.54008, 0.53394, 0.52765, 0.52119, 0.51453],\
        [0.56355, 0.56678, 0.56987, 0.57290, 0.57592, 0.57899, 0.58218, 0.58552, 0.58906, 0.59286, 0.59706,\
        0.60171, 0.60666, 0.61183, 0.61721, 0.62269, 0.62820, 0.63365, 0.63902, 0.64428, 0.64938, 0.65432,\
        0.65904, 0.66350, 0.66765, 0.67142, 0.67471, 0.67741, 0.67941, 0.68056, 0.68067, 0.67957, 0.67703,\
        0.67284, 0.66677, 0.65867, 0.64856, 0.63685, 0.62501, 0.61798, 0.61935, 0.62629, 0.63542, 0.64563,\
        0.65867, 0.67519, 0.69667, 0.72186, 0.74489, 0.76493, 0.78287, 0.79941, 0.81473, 0.82778, 0.83755,\
        0.84404, 0.84783, 0.84965, 0.85014, 0.84973, 0.84868, 0.84713, 0.84516, 0.84285, 0.84029, 0.83754,\
        0.83464, 0.83162, 0.82848, 0.82519, 0.82178, 0.81823, 0.81456, 0.81073, 0.80660, 0.80184, 0.79678,\
        0.79171, 0.78618, 0.78022, 0.77396, 0.76749, 0.76086, 0.75409, 0.74725, 0.74038, 0.73353, 0.72675,\
        0.72009, 0.71358, 0.70725, 0.70113, 0.69471, 0.68773, 0.68076, 0.67377, 0.66669, 0.65952, 0.65229,\
        0.64502, 0.63777, 0.63059, 0.62356, 0.61677, 0.61035, 0.60442, 0.59867, 0.59275, 0.58703, 0.58143,\
        0.57586, 0.57028, 0.56464, 0.55895, 0.55317, 0.54730, 0.54131, 0.53519, 0.52893, 0.52249, 0.51587],\
        [0.56791, 0.57088, 0.57370, 0.57643, 0.57916, 0.58195, 0.58487, 0.58799, 0.59132, 0.59488, 0.59886,\
        0.60355, 0.60868, 0.61407, 0.61962, 0.62527, 0.63094, 0.63658, 0.64211, 0.64751, 0.65272, 0.65772,\
        0.66245, 0.66687, 0.67092, 0.67454, 0.67763, 0.68008, 0.68176, 0.68251, 0.68212, 0.68040, 0.67708,\
        0.67189, 0.66450, 0.65460, 0.64207, 0.62706, 0.61023, 0.59590, 0.59199, 0.59354, 0.59816, 0.60626,\
        0.62389, 0.64721, 0.67769, 0.70963, 0.73562, 0.75836, 0.77914, 0.79885, 0.81744, 0.83296, 0.84396,\
        0.85079, 0.85442, 0.85591, 0.85607, 0.85545, 0.85428, 0.85264, 0.85056, 0.84812, 0.84540, 0.84250,\
        0.83946, 0.83632, 0.83308, 0.82971, 0.82623, 0.82261, 0.81883, 0.81484, 0.81054, 0.80585, 0.80107,\
        0.79610, 0.79055, 0.78454, 0.77829, 0.77185, 0.76522, 0.75844, 0.75153, 0.74456, 0.73759, 0.73068,\
        0.72386, 0.71717, 0.71062, 0.70414, 0.69733, 0.69047, 0.68364, 0.67665, 0.66949, 0.66220, 0.65482,\
        0.64740, 0.64002, 0.63272, 0.62558, 0.61865, 0.61198, 0.60553, 0.59931, 0.59371, 0.58817, 0.58262,\
        0.57704, 0.57143, 0.56578, 0.56007, 0.55429, 0.54842, 0.54245, 0.53635, 0.53010, 0.52370, 0.51711],\
        [0.57218, 0.57488, 0.57741, 0.57984, 0.58227, 0.58478, 0.58747, 0.59041, 0.59366, 0.59717, 0.60093,\
        0.60565, 0.61087, 0.61638, 0.62208, 0.62788, 0.63370, 0.63948, 0.64514, 0.65064, 0.65592, 0.66094,\
        0.66564, 0.66998, 0.67389, 0.67731, 0.68015, 0.68230, 0.68360, 0.68388, 0.68292, 0.68047, 0.67627,\
        0.66999, 0.66121, 0.64946, 0.63448, 0.61657, 0.59619, 0.57658, 0.56304, 0.55465, 0.55247, 0.55769,\
        0.57367, 0.60244, 0.64350, 0.68417, 0.71738, 0.74505, 0.77051, 0.79515, 0.81888, 0.83737, 0.84950,\
        0.85656, 0.86004, 0.86127, 0.86130, 0.86072, 0.85967, 0.85809, 0.85596, 0.85338, 0.85049, 0.84741,\
        0.84422, 0.84095, 0.83759, 0.83412, 0.83053, 0.82679, 0.82288, 0.81876, 0.81440, 0.80982, 0.80512,\
        0.80018, 0.79447, 0.78843, 0.78224, 0.77588, 0.76928, 0.76247, 0.75548, 0.74839, 0.74127, 0.73419,\
        0.72719, 0.72027, 0.71338, 0.70637, 0.69938, 0.69298, 0.68632, 0.67934, 0.67209, 0.66468, 0.65716,\
        0.64962, 0.64212, 0.63472, 0.62747, 0.62043, 0.61359, 0.60697, 0.60074, 0.59501, 0.58940, 0.58380,\
        0.57817, 0.57251, 0.56683, 0.56110, 0.55532, 0.54945, 0.54349, 0.53740, 0.53119, 0.52481, 0.51826],\
        [0.57633, 0.57876, 0.58099, 0.58310, 0.58522, 0.58746, 0.58992, 0.59270, 0.59592, 0.59962, 0.60352,\
        0.60806, 0.61323, 0.61880, 0.62461, 0.63053, 0.63648, 0.64238, 0.64814, 0.65371, 0.65903, 0.66403,\
        0.66866, 0.67286, 0.67658, 0.67975, 0.68230, 0.68409, 0.68497, 0.68471, 0.68306, 0.67972, 0.67439,\
        0.66675, 0.65633, 0.64231, 0.62383, 0.60110, 0.57509, 0.55111, 0.53179, 0.51391, 0.50540, 0.50367,\
        0.51113, 0.53807, 0.59007, 0.64535, 0.69049, 0.72396, 0.75556, 0.78926, 0.81921, 0.83998, 0.85328,\
        0.86093, 0.86453, 0.86574, 0.86593, 0.86576, 0.86515, 0.86379, 0.86161, 0.85881, 0.85567, 0.85237,\
        0.84900, 0.84558, 0.84207, 0.83847, 0.83473, 0.83084, 0.82677, 0.82251, 0.81805, 0.81340, 0.80855,\
        0.80335, 0.79767, 0.79180, 0.78583, 0.77960, 0.77304, 0.76617, 0.75906, 0.75181, 0.74451, 0.73725,\
        0.73008, 0.72300, 0.71600, 0.70905, 0.70231, 0.69574, 0.68898, 0.68186, 0.67447, 0.66691, 0.65926,\
        0.65160, 0.64400, 0.63652, 0.62921, 0.62211, 0.61525, 0.60868, 0.60249, 0.59655, 0.59073, 0.58497,\
        0.57924, 0.57352, 0.56779, 0.56204, 0.55624, 0.55038, 0.54443, 0.53837, 0.53217, 0.52583, 0.51932],\
        [0.58036, 0.58250, 0.58440, 0.58619, 0.58800, 0.58996, 0.59217, 0.59477, 0.59788, 0.60156, 0.60574,\
        0.61043, 0.61565, 0.62129, 0.62719, 0.63322, 0.63928, 0.64528, 0.65113, 0.65675, 0.66207, 0.66702,\
        0.67154, 0.67557, 0.67906, 0.68194, 0.68414, 0.68553, 0.68594, 0.68510, 0.68267, 0.67826, 0.67145,\
        0.66192, 0.64933, 0.63263, 0.60956, 0.57979, 0.54568, 0.51596, 0.49603, 0.47716, 0.46396, 0.45501,\
        0.45295, 0.46544, 0.51528, 0.58803, 0.65851, 0.69597, 0.73084, 0.77302, 0.81198, 0.83756, 0.85403,\
        0.86377, 0.86821, 0.86969, 0.87038, 0.87110, 0.87131, 0.87024, 0.86782, 0.86459, 0.86107, 0.85748,\
        0.85389, 0.85027, 0.84659, 0.84280, 0.83887, 0.83478, 0.83051, 0.82607, 0.82144, 0.81662, 0.81155,\
        0.80615, 0.80042, 0.79490, 0.78920, 0.78310, 0.77653, 0.76953, 0.76223, 0.75477, 0.74726, 0.73981,\
        0.73248, 0.72532, 0.71835, 0.71157, 0.70499, 0.69843, 0.69145, 0.68410, 0.67652, 0.66881, 0.66103,\
        0.65328, 0.64561, 0.63807, 0.63073, 0.62361, 0.61676, 0.61023, 0.60400, 0.59795, 0.59196, 0.58604,\
        0.58020, 0.57440, 0.56864, 0.56286, 0.55706, 0.55121, 0.54527, 0.53923, 0.53307, 0.52676, 0.52029],\
        [0.58424, 0.58607, 0.58762, 0.58906, 0.59056, 0.59226, 0.59424, 0.59662, 0.59964, 0.60339, 0.60775,\
        0.61265, 0.61803, 0.62378, 0.62980, 0.63594, 0.64211, 0.64820, 0.65412, 0.65977, 0.66509, 0.66998,\
        0.67437, 0.67819, 0.68140, 0.68394, 0.68576, 0.68675, 0.68669, 0.68529, 0.68209, 0.67651, 0.66791,\
        0.65570, 0.63974, 0.61972, 0.59116, 0.55512, 0.51260, 0.46925, 0.44726, 0.43490, 0.42068, 0.40483,\
        0.39473, 0.39372, 0.41813, 0.50564, 0.60702, 0.66061, 0.69767, 0.74390, 0.79331, 0.82796, 0.85169,\
        0.86647, 0.87247, 0.87420, 0.87560, 0.87768, 0.87909, 0.87807, 0.87487, 0.87087, 0.86681, 0.86286,\
        0.85898, 0.85512, 0.85121, 0.84718, 0.84299, 0.83864, 0.83412, 0.82945, 0.82461, 0.81960, 0.81440,\
        0.80900, 0.80351, 0.79807, 0.79247, 0.78641, 0.77973, 0.77251, 0.76495, 0.75721, 0.74945, 0.74179,\
        0.73430, 0.72707, 0.72012, 0.71345, 0.70698, 0.70045, 0.69329, 0.68580, 0.67810, 0.67026, 0.66240,\
        0.65458, 0.64687, 0.63931, 0.63195, 0.62483, 0.61798, 0.61143, 0.60516, 0.59906, 0.59292, 0.58690,\
        0.58098, 0.57514, 0.56935, 0.56357, 0.55777, 0.55193, 0.54601, 0.54000, 0.53387, 0.52760, 0.52117],\
        [0.58793, 0.58942, 0.59059, 0.59165, 0.59284, 0.59432, 0.59618, 0.59835, 0.60123, 0.60514, 0.60969,\
        0.61479, 0.62035, 0.62626, 0.63242, 0.63869, 0.64497, 0.65115, 0.65713, 0.66283, 0.66813, 0.67296,\
        0.67721, 0.68082, 0.68371, 0.68589, 0.68730, 0.68789, 0.68743, 0.68559, 0.68179, 0.67527, 0.66504,\
        0.64984, 0.62842, 0.60215, 0.56598, 0.52395, 0.47937, 0.42115, 0.39443, 0.38179, 0.35985, 0.34121,\
        0.33095, 0.32601, 0.34935, 0.42785, 0.53645, 0.61760, 0.66204, 0.71417, 0.76949, 0.81192, 0.84958,\
        0.87330, 0.88024, 0.88159, 0.88346, 0.88694, 0.88969, 0.88776, 0.88282, 0.87773, 0.87305, 0.86866,\
        0.86442, 0.86024, 0.85601, 0.85165, 0.84712, 0.84244, 0.83760, 0.83265, 0.82757, 0.82239, 0.81710,\
        0.81176, 0.80642, 0.80110, 0.79562, 0.78951, 0.78256, 0.77500, 0.76711, 0.75906, 0.75102, 0.74311,\
        0.73544, 0.72809, 0.72111, 0.71447, 0.70799, 0.70128, 0.69430, 0.68686, 0.67909, 0.67119, 0.66329,\
        0.65545, 0.64773, 0.64017, 0.63282, 0.62570, 0.61883, 0.61223, 0.60585, 0.59963, 0.59350, 0.58747,\
        0.58155, 0.57570, 0.56991, 0.56414, 0.55836, 0.55254, 0.54665, 0.54067, 0.53458, 0.52836, 0.52197],\
        [0.59141, 0.59251, 0.59324, 0.59388, 0.59473, 0.59601, 0.59788, 0.60025, 0.60304, 0.60691, 0.61159,\
        0.61688, 0.62263, 0.62873, 0.63505, 0.64146, 0.64786, 0.65414, 0.66019, 0.66592, 0.67124, 0.67602,\
        0.68016, 0.68355, 0.68613, 0.68790, 0.68891, 0.68913, 0.68839, 0.68632, 0.68228, 0.67537, 0.66444,\
        0.64795, 0.62388, 0.59067, 0.54597, 0.49444, 0.44768, 0.39708, 0.36057, 0.33425, 0.30558, 0.29631,\
        0.29041, 0.28922, 0.31735, 0.38827, 0.47631, 0.55904, 0.61339, 0.68776, 0.74829, 0.79919, 0.85465,\
        0.89090, 0.89608, 0.89690, 0.89795, 0.90109, 0.90311, 0.89848, 0.89164, 0.88533, 0.88007, 0.87507,\
        0.87029, 0.86569, 0.86104, 0.85624, 0.85126, 0.84615, 0.84093, 0.83564, 0.83030, 0.82491, 0.81950,\
        0.81413, 0.80889, 0.80378, 0.79849, 0.79219, 0.78480, 0.77685, 0.76862, 0.76027, 0.75193, 0.74374,\
        0.73582, 0.72828, 0.72120, 0.71456, 0.70821, 0.70182, 0.69481, 0.68722, 0.67941, 0.67152, 0.66365,\
        0.65585, 0.64817, 0.64065, 0.63332, 0.62620, 0.61932, 0.61266, 0.60620, 0.59989, 0.59380, 0.58781,\
        0.58191, 0.57609, 0.57032, 0.56457, 0.55882, 0.55304, 0.54719, 0.54125, 0.53521, 0.52902, 0.52268],\
        [0.59460, 0.59527, 0.59550, 0.59568, 0.59614, 0.59716, 0.59892, 0.60140, 0.60454, 0.60857, 0.61343,\
        0.61893, 0.62490, 0.63120, 0.63769, 0.64426, 0.65079, 0.65717, 0.66329, 0.66908, 0.67442, 0.67921,\
        0.68330, 0.68652, 0.68878, 0.69014, 0.69075, 0.69066, 0.68978, 0.68777, 0.68390, 0.67721, 0.66658,\
        0.65070, 0.62786, 0.59369, 0.54311, 0.48814, 0.43849, 0.39154, 0.34938, 0.31999, 0.30430, 0.29633,\
        0.28414, 0.28356, 0.32056, 0.39266, 0.45979, 0.52606, 0.59516, 0.67019, 0.73138, 0.78504, 0.85578,\
        0.91280, 0.92577, 0.92620, 0.92424, 0.92257, 0.91908, 0.91165, 0.90239, 0.89422, 0.88843, 0.88218,\
        0.87664, 0.87153, 0.86634, 0.86093, 0.85536, 0.84971, 0.84405, 0.83839, 0.83274, 0.82710, 0.82149,\
        0.81594, 0.81053, 0.80540, 0.80052, 0.79377, 0.78603, 0.77786, 0.76941, 0.76080, 0.75219, 0.74370,\
        0.73547, 0.72765, 0.72034, 0.71360, 0.70733, 0.70111, 0.69420, 0.68668, 0.67897, 0.67121, 0.66347,\
        0.65578, 0.64820, 0.64075, 0.63347, 0.62637, 0.61949, 0.61281, 0.60634, 0.60005, 0.59394, 0.58797,\
        0.58210, 0.57632, 0.57059, 0.56488, 0.55918, 0.55343, 0.54763, 0.54174, 0.53574, 0.52961, 0.52332],\
        [0.59740, 0.59756, 0.59729, 0.59704, 0.59711, 0.59779, 0.59941, 0.60204, 0.60561, 0.61003, 0.61519,\
        0.62095, 0.62716, 0.63367, 0.64036, 0.64709, 0.65376, 0.66024, 0.66644, 0.67229, 0.67768, 0.68254,\
        0.68667, 0.68984, 0.69184, 0.69279, 0.69303, 0.69273, 0.69186, 0.69018, 0.68690, 0.68082, 0.67090,\
        0.65597, 0.63348, 0.59887, 0.55132, 0.49529, 0.43882, 0.38810, 0.34675, 0.32122, 0.31020, 0.30491,\
        0.29352, 0.28789, 0.32765, 0.39837, 0.45273, 0.50837, 0.59134, 0.66375, 0.71981, 0.76640, 0.84585,\
        0.92506, 0.95521, 0.95913, 0.95469, 0.94778, 0.93899, 0.92808, 0.91619, 0.90625, 0.89763, 0.88959,\
        0.88347, 0.87781, 0.87185, 0.86560, 0.85928, 0.85301, 0.84686, 0.84080, 0.83484, 0.82893, 0.82305,\
        0.81719, 0.81131, 0.80537, 0.79943, 0.79341, 0.78614, 0.77807, 0.76954, 0.76075, 0.75189, 0.74309,\
        0.73450, 0.72629, 0.71863, 0.71161, 0.70520, 0.69902, 0.69232, 0.68520, 0.67782, 0.67034, 0.66281,\
        0.65530, 0.64786, 0.64051, 0.63330, 0.62625, 0.61938, 0.61271, 0.60627, 0.60002, 0.59391, 0.58795,\
        0.58213, 0.57640, 0.57072, 0.56507, 0.55942, 0.55373, 0.54798, 0.54215, 0.53620, 0.53012, 0.52388],\
        [0.59956, 0.59913, 0.59857, 0.59806, 0.59781, 0.59817, 0.59965, 0.60256, 0.60658, 0.61142, 0.61693,\
        0.62297, 0.62943, 0.63616, 0.64305, 0.64996, 0.65677, 0.66336, 0.66964, 0.67553, 0.68097, 0.68593,\
        0.69026, 0.69359, 0.69545, 0.69604, 0.69606, 0.69574, 0.69494, 0.69388, 0.69163, 0.68628, 0.67716,\
        0.66376, 0.64337, 0.61214, 0.56549, 0.50691, 0.44182, 0.38426, 0.34679, 0.32634, 0.31730, 0.31271,\
        0.30624, 0.29988, 0.34108, 0.40571, 0.45135, 0.49936, 0.57093, 0.65007, 0.71382, 0.75650, 0.85135,\
        0.94586, 0.97819, 0.98545, 0.98116, 0.97169, 0.95954, 0.94581, 0.93178, 0.91884, 0.90709, 0.89756,\
        0.89104, 0.88452, 0.87731, 0.86994, 0.86278, 0.85589, 0.84925, 0.84283, 0.83656, 0.83040, 0.82428,\
        0.81814, 0.81190, 0.80544, 0.79902, 0.79293, 0.78578, 0.77777, 0.76919, 0.76028, 0.75119, 0.74209,\
        0.73312, 0.72446, 0.71632, 0.70893, 0.70222, 0.69608, 0.68980, 0.68314, 0.67619, 0.66904, 0.66179,\
        0.65449, 0.64721, 0.63999, 0.63287, 0.62587, 0.61903, 0.61237, 0.60593, 0.59972, 0.59366, 0.58777,\
        0.58201, 0.57635, 0.57075, 0.56516, 0.55957, 0.55394, 0.54825, 0.54247, 0.53658, 0.53055, 0.52437],\
        [0.60032, 0.59994, 0.59948, 0.59898, 0.59856, 0.59874, 0.60017, 0.60340, 0.60778, 0.61294, 0.61873,\
        0.62503, 0.63172, 0.63867, 0.64575, 0.65285, 0.65982, 0.66654, 0.67289, 0.67880, 0.68424, 0.68923,\
        0.69381, 0.69765, 0.69962, 0.70010, 0.70029, 0.70035, 0.69970, 0.69941, 0.69860, 0.69381, 0.68552,\
        0.67478, 0.65782, 0.62788, 0.58106, 0.52304, 0.45288, 0.38833, 0.35592, 0.33941, 0.33198, 0.32835,\
        0.32681, 0.33361, 0.36732, 0.41702, 0.45907, 0.50002, 0.55867, 0.63391, 0.71231, 0.77502, 0.86381,\
        0.94795, 0.98858, 1.00199, 1.00054, 0.99120, 0.97774, 0.96232, 0.94648, 0.93139, 0.91789, 0.90711,\
        0.89921, 0.89095, 0.88196, 0.87347, 0.86556, 0.85815, 0.85113, 0.84440, 0.83789, 0.83152, 0.82522,\
        0.81893, 0.81261, 0.80629, 0.80005, 0.79302, 0.78543, 0.77726, 0.76859, 0.75956, 0.75030, 0.74095,\
        0.73164, 0.72253, 0.71384, 0.70606, 0.69957, 0.69331, 0.68725, 0.68096, 0.67437, 0.66754, 0.66055,\
        0.65347, 0.64636, 0.63927, 0.63224, 0.62531, 0.61850, 0.61184, 0.60537, 0.59915, 0.59321, 0.58744,\
        0.58179, 0.57621, 0.57068, 0.56516, 0.55963, 0.55406, 0.54843, 0.54272, 0.53688, 0.53092, 0.52480],\
        [0.60065, 0.60060, 0.60031, 0.59995, 0.59954, 0.59996, 0.60165, 0.60493, 0.60939, 0.61470, 0.62066,\
        0.62714, 0.63402, 0.64117, 0.64845, 0.65575, 0.66290, 0.66977, 0.67621, 0.68213, 0.68751, 0.69240,\
        0.69691, 0.70117, 0.70385, 0.70522, 0.70617, 0.70691, 0.70740, 0.70787, 0.70735, 0.70305, 0.69748,\
        0.69032, 0.67989, 0.65270, 0.60561, 0.54944, 0.47889, 0.41520, 0.38198, 0.36605, 0.35938, 0.35585,\
        0.35915, 0.37088, 0.39294, 0.43143, 0.47437, 0.52048, 0.57083, 0.63374, 0.71784, 0.80874, 0.88491,\
        0.95129, 0.99377, 1.01177, 1.01347, 1.00548, 0.99207, 0.97597, 0.95901, 0.94254, 0.92760, 0.91500,\
        0.90473, 0.89460, 0.88472, 0.87574, 0.86744, 0.85970, 0.85241, 0.84548, 0.83880, 0.83229, 0.82587,\
        0.81949, 0.81309, 0.80667, 0.80001, 0.79277, 0.78496, 0.77664, 0.76788, 0.75876, 0.74941, 0.73991,\
        0.73040, 0.72099, 0.71185, 0.70331, 0.69669, 0.69097, 0.68511, 0.67902, 0.67265, 0.66604, 0.65925,\
        0.65234, 0.64538, 0.63841, 0.63149, 0.62463, 0.61788, 0.61125, 0.60478, 0.59859, 0.59275, 0.58708,\
        0.58151, 0.57600, 0.57054, 0.56508, 0.55962, 0.55412, 0.54855, 0.54289, 0.53712, 0.53122, 0.52515],\
        [0.60093, 0.60113, 0.60104, 0.60085, 0.60082, 0.60165, 0.60362, 0.60690, 0.61132, 0.61664, 0.62268,\
        0.62928, 0.63630, 0.64361, 0.65109, 0.65859, 0.66596, 0.67304, 0.67964, 0.68560, 0.69092, 0.69577,\
        0.70019, 0.70459, 0.70827, 0.71098, 0.71317, 0.71514, 0.71698, 0.71856, 0.71912, 0.71790, 0.71615,\
        0.71375, 0.71065, 0.69184, 0.65206, 0.59157, 0.51646, 0.45478, 0.42364, 0.40896, 0.40047, 0.39377,\
        0.39716, 0.40980, 0.42463, 0.45587, 0.49363, 0.54520, 0.60612, 0.66549, 0.74498, 0.84300, 0.91390,\
        0.96697, 1.00204, 1.01911, 1.02193, 1.01503, 1.00215, 0.98600, 0.96852, 0.95112, 0.93480, 0.92006,\
        0.90670, 0.89531, 0.88588, 0.87689, 0.86843, 0.86052, 0.85309, 0.84605, 0.83928, 0.83269, 0.82620,\
        0.81973, 0.81321, 0.80654, 0.79959, 0.79221, 0.78432, 0.77594, 0.76714, 0.75800, 0.74863, 0.73912,\
        0.72960, 0.72019, 0.71111, 0.70273, 0.69569, 0.68960, 0.68363, 0.67752, 0.67120, 0.66468, 0.65800,\
        0.65121, 0.64435, 0.63749, 0.63066, 0.62390, 0.61723, 0.61069, 0.60431, 0.59819, 0.59237, 0.58673,\
        0.58120, 0.57575, 0.57034, 0.56494, 0.55954, 0.55410, 0.54860, 0.54300, 0.53730, 0.53145, 0.52545],\
        [0.60116, 0.60159, 0.60174, 0.60186, 0.60229, 0.60346, 0.60565, 0.60895, 0.61333, 0.61863, 0.62469,\
        0.63135, 0.63848, 0.64593, 0.65358, 0.66129, 0.66891, 0.67627, 0.68315, 0.68928, 0.69457, 0.69947,\
        0.70432, 0.70899, 0.71335, 0.71722, 0.72075, 0.72415, 0.72751, 0.73074, 0.73358, 0.73601, 0.73849,\
        0.74098, 0.74094, 0.73105, 0.70493, 0.65444, 0.59167, 0.52061, 0.48922, 0.46407, 0.45104, 0.44357,\
        0.44111, 0.44663, 0.46240, 0.49634, 0.53747, 0.59040, 0.65577, 0.72072, 0.78922, 0.87737, 0.94225,\
        0.98396, 1.01114, 1.02484, 1.02702, 1.02053, 1.00816, 0.99227, 0.97468, 0.95677, 0.93952, 0.92357,\
        0.90931, 0.89728, 0.88681, 0.87736, 0.86870, 0.86070, 0.85321, 0.84613, 0.83934, 0.83272, 0.82620,\
        0.81966, 0.81303, 0.80619, 0.79905, 0.79156, 0.78362, 0.77522, 0.76642, 0.75732, 0.74801, 0.73861,\
        0.72924, 0.72006, 0.71130, 0.70323, 0.69593, 0.68924, 0.68286, 0.67651, 0.67008, 0.66353, 0.65687,\
        0.65012, 0.64333, 0.63654, 0.62979, 0.62311, 0.61656, 0.61014, 0.60390, 0.59785, 0.59203, 0.58638,\
        0.58087, 0.57544, 0.57008, 0.56474, 0.55940, 0.55402, 0.54858, 0.54305, 0.53741, 0.53163, 0.52568],\
        [0.60137, 0.60200, 0.60241, 0.60287, 0.60367, 0.60513, 0.60749, 0.61086, 0.61522, 0.62049, 0.62655,\
        0.63325, 0.64046, 0.64804, 0.65584, 0.66374, 0.67159, 0.67927, 0.68657, 0.69314, 0.69854, 0.70349,\
        0.70864, 0.71376, 0.71879, 0.72366, 0.72843, 0.73321, 0.73809, 0.74309, 0.74819, 0.75346, 0.75893,\
        0.76391, 0.76588, 0.76018, 0.74099, 0.70313, 0.65044, 0.59480, 0.55773, 0.52518, 0.50651, 0.49748,\
        0.48930, 0.49080, 0.50939, 0.54639, 0.59751, 0.65892, 0.71862, 0.77851, 0.83884, 0.90426, 0.95954,\
        0.99537, 1.01726, 1.02783, 1.02887, 1.02237, 1.01043, 0.99495, 0.97755, 0.95953, 0.94188, 0.92540,\
        0.91080, 0.89829, 0.88699, 0.87712, 0.86833, 0.86030, 0.85283, 0.84577, 0.83901, 0.83242, 0.82590,\
        0.81934, 0.81266, 0.80574, 0.79852, 0.79093, 0.78292, 0.77451, 0.76574, 0.75671, 0.74752, 0.73828,\
        0.72914, 0.72025, 0.71179, 0.70387, 0.69642, 0.68934, 0.68254, 0.67588, 0.66925, 0.66258, 0.65587,\
        0.64910, 0.64233, 0.63557, 0.62887, 0.62227, 0.61582, 0.60953, 0.60342, 0.59746, 0.59162, 0.58597,\
        0.58047, 0.57508, 0.56976, 0.56447, 0.55919, 0.55388, 0.54850, 0.54304, 0.53746, 0.53174, 0.52586],\
        [0.60153, 0.60235, 0.60300, 0.60374, 0.60482, 0.60651, 0.60902, 0.61244, 0.61682, 0.62208, 0.62814,\
        0.63488, 0.64216, 0.64983, 0.65776, 0.66580, 0.67382, 0.68171, 0.68938, 0.69662, 0.70244, 0.70748,\
        0.71284, 0.71843, 0.72413, 0.72989, 0.73571, 0.74169, 0.74788, 0.75433, 0.76105, 0.76802, 0.77492,\
        0.78075, 0.78334, 0.77921, 0.76440, 0.73673, 0.69929, 0.65885, 0.62237, 0.58806, 0.55827, 0.54020,\
        0.53686, 0.54423, 0.57299, 0.61742, 0.67680, 0.73089, 0.78022, 0.83162, 0.88589, 0.93339, 0.97242,\
        1.00126, 1.01912, 1.02735, 1.02740, 1.02085, 1.00933, 0.99439, 0.97743, 0.95963, 0.94194, 0.92516,\
        0.90993, 0.89681, 0.88579, 0.87607, 0.86735, 0.85939, 0.85200, 0.84502, 0.83832, 0.83179, 0.82532,\
        0.81879, 0.81211, 0.80521, 0.79796, 0.79028, 0.78222, 0.77380, 0.76508, 0.75614, 0.74708, 0.73802,\
        0.72909, 0.72043, 0.71219, 0.70439, 0.69687, 0.68950, 0.68238, 0.67544, 0.66860, 0.66179, 0.65497,\
        0.64815, 0.64135, 0.63459, 0.62791, 0.62135, 0.61496, 0.60875, 0.60274, 0.59684, 0.59105, 0.58543,\
        0.57997, 0.57463, 0.56936, 0.56414, 0.55892, 0.55367, 0.54837, 0.54297, 0.53746, 0.53180, 0.52598],\
        [0.60165, 0.60261, 0.60345, 0.60438, 0.60565, 0.60751, 0.61012, 0.61361, 0.61800, 0.62327, 0.62935,\
        0.63613, 0.64347, 0.65124, 0.65928, 0.66742, 0.67551, 0.68339, 0.69096, 0.69815, 0.70451, 0.71041,\
        0.71642, 0.72262, 0.72899, 0.73551, 0.74220, 0.74910, 0.75623, 0.76362, 0.77122, 0.77884, 0.78602,\
        0.79175, 0.79427, 0.79128, 0.78060, 0.76164, 0.73636, 0.70691, 0.67714, 0.64582, 0.61254, 0.58972,\
        0.59111, 0.59939, 0.62833, 0.68363, 0.73980, 0.78381, 0.82441, 0.86659, 0.91004, 0.94847, 0.97941,\
        1.00246, 1.01701, 1.02347, 1.02283, 1.01632, 1.00528, 0.99101, 0.97470, 0.95741, 0.94003, 0.92334,\
        0.90799, 0.89483, 0.88398, 0.87447, 0.86592, 0.85808, 0.85079, 0.84391, 0.83731, 0.83087, 0.82448,\
        0.81802, 0.81136, 0.80442, 0.79718, 0.78952, 0.78146, 0.77306, 0.76438, 0.75553, 0.74660, 0.73770,\
        0.72894, 0.72044, 0.71229, 0.70452, 0.69695, 0.68947, 0.68218, 0.67507, 0.66805, 0.66109, 0.65416,\
        0.64725, 0.64039, 0.63359, 0.62690, 0.62034, 0.61395, 0.60777, 0.60180, 0.59597, 0.59028, 0.58474,\
        0.57935, 0.57408, 0.56888, 0.56373, 0.55858, 0.55341, 0.54817, 0.54284, 0.53740, 0.53181, 0.52605],\
        [0.60166, 0.60274, 0.60368, 0.60473, 0.60612, 0.60806, 0.61075, 0.61427, 0.61869, 0.62398, 0.63009,\
        0.63692, 0.64434, 0.65221, 0.66036, 0.66861, 0.67676, 0.68458, 0.69183, 0.69833, 0.70538, 0.71245,\
        0.71929, 0.72614, 0.73312, 0.74027, 0.74761, 0.75514, 0.76284, 0.77067, 0.77849, 0.78604, 0.79280,\
        0.79792, 0.80016, 0.79815, 0.79084, 0.77819, 0.76129, 0.74188, 0.72214, 0.70090, 0.68124, 0.66383,\
        0.65299, 0.65493, 0.68128, 0.73803, 0.78993, 0.82453, 0.85493, 0.88520, 0.91988, 0.95319, 0.98003,\
        0.99955, 1.01167, 1.01678, 1.01565, 1.00928, 0.99879, 0.98528, 0.96981, 0.95330, 0.93659, 0.92042,\
        0.90564, 0.89285, 0.88202, 0.87262, 0.86418, 0.85645, 0.84927, 0.84248, 0.83598, 0.82965, 0.82339,\
        0.81705, 0.81049, 0.80355, 0.79627, 0.78865, 0.78060, 0.77221, 0.76359, 0.75483, 0.74602, 0.73726,\
        0.72862, 0.72021, 0.71206, 0.70421, 0.69661, 0.68920, 0.68189, 0.67468, 0.66753, 0.66043, 0.65338,\
        0.64637, 0.63942, 0.63257, 0.62583, 0.61924, 0.61283, 0.60662, 0.60065, 0.59491, 0.58934, 0.58392,\
        0.57862, 0.57344, 0.56832, 0.56325, 0.55818, 0.55307, 0.54791, 0.54266, 0.53728, 0.53176, 0.52606],\
        [0.60156, 0.60268, 0.60366, 0.60474, 0.60617, 0.60815, 0.61085, 0.61440, 0.61883, 0.62415, 0.63030,\
        0.63719, 0.64470, 0.65269, 0.66098, 0.66940, 0.67771, 0.68569, 0.69317, 0.70019, 0.70714, 0.71437,\
        0.72162, 0.72893, 0.73639, 0.74403, 0.75182, 0.75973, 0.76768, 0.77556, 0.78317, 0.79020, 0.79619,\
        0.80049, 0.80234, 0.80105, 0.79627, 0.78828, 0.77798, 0.76672, 0.75574, 0.74548, 0.73727, 0.72659,\
        0.71467, 0.71720, 0.74637, 0.79570, 0.83550, 0.86086, 0.88106, 0.90158, 0.92786, 0.95511, 0.97771,\
        0.99413, 1.00415, 1.00806, 1.00649, 1.00027, 0.99035, 0.97767, 0.96317, 0.94768, 0.93195, 0.91668,\
        0.90238, 0.89031, 0.87983, 0.87054, 0.86218, 0.85454, 0.84744, 0.84075, 0.83435, 0.82813, 0.82199,\
        0.81582, 0.80947, 0.80276, 0.79548, 0.78771, 0.77959, 0.77120, 0.76264, 0.75397, 0.74528, 0.73664,\
        0.72813, 0.71978, 0.71164, 0.70377, 0.69621, 0.68885, 0.68155, 0.67427, 0.66701, 0.65978, 0.65260,\
        0.64548, 0.63844, 0.63151, 0.62472, 0.61809, 0.61164, 0.60541, 0.59944, 0.59377, 0.58831, 0.58300,\
        0.57781, 0.57271, 0.56769, 0.56270, 0.55770, 0.55268, 0.54759, 0.54241, 0.53711, 0.53165, 0.52601],\
        [0.60128, 0.60240, 0.60334, 0.60439, 0.60578, 0.60774, 0.61042, 0.61397, 0.61841, 0.62376, 0.62995,\
        0.63690, 0.64451, 0.65262, 0.66109, 0.66973, 0.67832, 0.68663, 0.69449, 0.70187, 0.70892, 0.71605,\
        0.72338, 0.73095, 0.73875, 0.74673, 0.75483, 0.76295, 0.77094, 0.77863, 0.78580, 0.79212, 0.79722,\
        0.80067, 0.80210, 0.80129, 0.79842, 0.79405, 0.78901, 0.78438, 0.78121, 0.77981, 0.77858, 0.77507,\
        0.77482, 0.79011, 0.81734, 0.85076, 0.87510, 0.89132, 0.90489, 0.91961, 0.93785, 0.95749, 0.97479,\
        0.98769, 0.99546, 0.99809, 0.99601, 0.98986, 0.98045, 0.96862, 0.95518, 0.94086, 0.92631, 0.91233,\
        0.89964, 0.88772, 0.87734, 0.86816, 0.85990, 0.85235, 0.84533, 0.83872, 0.83240, 0.82627, 0.82023,\
        0.81421, 0.80806, 0.80151, 0.79425, 0.78643, 0.77829, 0.76994, 0.76145, 0.75290, 0.74434, 0.73584,\
        0.72747, 0.71926, 0.71120, 0.70324, 0.69580, 0.68857, 0.68122, 0.67384, 0.66646, 0.65911, 0.65181,\
        0.64457, 0.63743, 0.63042, 0.62356, 0.61689, 0.61043, 0.60421, 0.59827, 0.59265, 0.58725, 0.58202,\
        0.57692, 0.57192, 0.56698, 0.56208, 0.55717, 0.55223, 0.54722, 0.54211, 0.53687, 0.53148, 0.52591],\
        [0.60078, 0.60183, 0.60269, 0.60365, 0.60497, 0.60685, 0.60948, 0.61299, 0.61744, 0.62280, 0.62903,\
        0.63603, 0.64371, 0.65195, 0.66059, 0.66948, 0.67841, 0.68713, 0.69535, 0.70291, 0.70996, 0.71697,\
        0.72432, 0.73208, 0.74015, 0.74842, 0.75674, 0.76496, 0.77290, 0.78034, 0.78702, 0.79265, 0.79694,\
        0.79965, 0.80071, 0.80030, 0.79900, 0.79768, 0.79713, 0.79832, 0.80249, 0.81062, 0.82295, 0.83945,\
        0.85272, 0.86541, 0.88429, 0.90017, 0.90874, 0.91635, 0.92428, 0.93394, 0.94603, 0.95932, 0.97153,\
        0.98081, 0.98621, 0.98744, 0.98471, 0.97851, 0.96951, 0.95848, 0.94615, 0.93312, 0.91981, 0.90673,\
        0.89493, 0.88419, 0.87429, 0.86536, 0.85727, 0.84984, 0.84293, 0.83639, 0.83013, 0.82405, 0.81806,\
        0.81208, 0.80602, 0.79965, 0.79241, 0.78467, 0.77662, 0.76837, 0.75999, 0.75157, 0.74315, 0.73482,\
        0.72662, 0.71861, 0.71085, 0.70343, 0.69600, 0.68841, 0.68087, 0.67335, 0.66585, 0.65838, 0.65096,\
        0.64362, 0.63638, 0.62927, 0.62235, 0.61564, 0.60919, 0.60303, 0.59713, 0.59152, 0.58617, 0.58101,\
        0.57599, 0.57107, 0.56622, 0.56140, 0.55657, 0.55171, 0.54678, 0.54174, 0.53658, 0.53125, 0.52574],\
        [0.59993, 0.60087, 0.60167, 0.60257, 0.60378, 0.60554, 0.60807, 0.61153, 0.61595, 0.62132, 0.62755,\
        0.63458, 0.64230, 0.65061, 0.65938, 0.66847, 0.67770, 0.68683, 0.69550, 0.70318, 0.70991, 0.71674,\
        0.72421, 0.73223, 0.74061, 0.74916, 0.75769, 0.76600, 0.77390, 0.78113, 0.78743, 0.79254, 0.79625,\
        0.79848, 0.79942, 0.79959, 0.79988, 0.80152, 0.80500, 0.81110, 0.82103, 0.83621, 0.85850, 0.88999,\
        0.91125, 0.91618, 0.92356, 0.92941, 0.93125, 0.93331, 0.93700, 0.94276, 0.95049, 0.95919, 0.96732,\
        0.97344, 0.97662, 0.97646, 0.97300, 0.96662, 0.95790, 0.94755, 0.93630, 0.92470, 0.91292, 0.90105,\
        0.88995, 0.87992, 0.87062, 0.86209, 0.85425, 0.84701, 0.84022, 0.83378, 0.82758, 0.82153, 0.81554,\
        0.80949, 0.80332, 0.79689, 0.78994, 0.78242, 0.77455, 0.76645, 0.75822, 0.74996, 0.74171, 0.73355,\
        0.72551, 0.71767, 0.71011, 0.70288, 0.69554, 0.68794, 0.68033, 0.67273, 0.66513, 0.65756, 0.65004,\
        0.64259, 0.63526, 0.62807, 0.62107, 0.61432, 0.60786, 0.60172, 0.59589, 0.59034, 0.58504, 0.57995,\
        0.57501, 0.57017, 0.56540, 0.56066, 0.55591, 0.55113, 0.54627, 0.54131, 0.53622, 0.53096, 0.52552],\
        [0.59850, 0.59953, 0.60040, 0.60124, 0.60234, 0.60393, 0.60629, 0.60966, 0.61406, 0.61940, 0.62559,\
        0.63257, 0.64027, 0.64858, 0.65739, 0.66656, 0.67593, 0.68530, 0.69433, 0.70210, 0.70816, 0.71508,\
        0.72298, 0.73145, 0.74022, 0.74908, 0.75783, 0.76629, 0.77423, 0.78139, 0.78752, 0.79239, 0.79585,\
        0.79799, 0.79920, 0.80044, 0.80269, 0.80661, 0.81269, 0.82154, 0.83392, 0.85044, 0.87088, 0.89266,\
        0.91395, 0.92700, 0.93406, 0.93778, 0.93896, 0.93981, 0.94183, 0.94551, 0.95063, 0.95641, 0.96170,\
        0.96542, 0.96677, 0.96536, 0.96119, 0.95454, 0.94595, 0.93608, 0.92573, 0.91562, 0.90557, 0.89515,\
        0.88481, 0.87525, 0.86646, 0.85835, 0.85084, 0.84385, 0.83725, 0.83093, 0.82479, 0.81878, 0.81280,\
        0.80671, 0.80044, 0.79402, 0.78718, 0.77985, 0.77215, 0.76422, 0.75615, 0.74806, 0.74000, 0.73201,\
        0.72414, 0.71640, 0.70882, 0.70151, 0.69435, 0.68704, 0.67954, 0.67193, 0.66428, 0.65662, 0.64901,\
        0.64148, 0.63406, 0.62680, 0.61974, 0.61293, 0.60643, 0.60030, 0.59454, 0.58907, 0.58386, 0.57885,\
        0.57398, 0.56922, 0.56452, 0.55985, 0.55518, 0.55048, 0.54570, 0.54081, 0.53579, 0.53061, 0.52522],\
        [0.59690, 0.59808, 0.59902, 0.59984, 0.60086, 0.60219, 0.60427, 0.60756, 0.61190, 0.61715, 0.62322,\
        0.63008, 0.63767, 0.64589, 0.65463, 0.66373, 0.67300, 0.68219, 0.69088, 0.69805, 0.70424, 0.71223,\
        0.72093, 0.72995, 0.73913, 0.74831, 0.75733, 0.76599, 0.77408, 0.78137, 0.78759, 0.79254, 0.79614,\
        0.79858, 0.80051, 0.80293, 0.80651, 0.81172, 0.81897, 0.82863, 0.84089, 0.85556, 0.87160, 0.88754,\
        0.90387, 0.91787, 0.92690, 0.93215, 0.93498, 0.93697, 0.93934, 0.94263, 0.94672, 0.95099, 0.95460,\
        0.95673, 0.95674, 0.95435, 0.94954, 0.94261, 0.93405, 0.92447, 0.91457, 0.90568, 0.89735, 0.88808,\
        0.87881, 0.87000, 0.86179, 0.85416, 0.84707, 0.84040, 0.83405, 0.82789, 0.82185, 0.81586, 0.80995,\
        0.80408, 0.79779, 0.79125, 0.78437, 0.77711, 0.76952, 0.76171, 0.75380, 0.74587, 0.73801, 0.73023,\
        0.72257, 0.71500, 0.70747, 0.69999, 0.69315, 0.68602, 0.67859, 0.67096, 0.66325, 0.65553, 0.64785,\
        0.64026, 0.63279, 0.62548, 0.61836, 0.61149, 0.60495, 0.59885, 0.59315, 0.58778, 0.58265, 0.57771,\
        0.57291, 0.56821, 0.56358, 0.55898, 0.55439, 0.54975, 0.54505, 0.54024, 0.53529, 0.53017, 0.52485],\
        [0.59524, 0.59656, 0.59764, 0.59863, 0.59966, 0.60054, 0.60223, 0.60551, 0.60971, 0.61472, 0.62055,\
        0.62718, 0.63458, 0.64263, 0.65121, 0.66013, 0.66917, 0.67801, 0.68620, 0.69335, 0.70060, 0.70939,\
        0.71860, 0.72802, 0.73752, 0.74699, 0.75627, 0.76518, 0.77355, 0.78114, 0.78771, 0.79304, 0.79709,\
        0.80011, 0.80283, 0.80603, 0.81027, 0.81598, 0.82340, 0.83267, 0.84366, 0.85589, 0.86856, 0.88099,\
        0.89296, 0.90339, 0.91248, 0.91914, 0.92394, 0.92779, 0.93144, 0.93543, 0.93961, 0.94347, 0.94632,\
        0.94755, 0.94672, 0.94363, 0.93834, 0.93115, 0.92263, 0.91346, 0.90397, 0.89536, 0.88815, 0.88025,\
        0.87211, 0.86417, 0.85663, 0.84957, 0.84296, 0.83670, 0.83069, 0.82477, 0.81885, 0.81285, 0.80686,\
        0.80110, 0.79512, 0.78848, 0.78153, 0.77425, 0.76670, 0.75897, 0.75116, 0.74339, 0.73572, 0.72820,\
        0.72082, 0.71356, 0.70651, 0.69965, 0.69242, 0.68506, 0.67750, 0.66980, 0.66203, 0.65426, 0.64654,\
        0.63892, 0.63143, 0.62410, 0.61697, 0.61007, 0.60347, 0.59739, 0.59180, 0.58651, 0.58144, 0.57654,\
        0.57179, 0.56715, 0.56257, 0.55804, 0.55351, 0.54894, 0.54431, 0.53958, 0.53471, 0.52966, 0.52440],\
        [0.59354, 0.59500, 0.59628, 0.59748, 0.59862, 0.59951, 0.60109, 0.60395, 0.60763, 0.61218, 0.61763,\
        0.62396, 0.63109, 0.63893, 0.64732, 0.65606, 0.66490, 0.67358, 0.68186, 0.68983, 0.69815, 0.70694,\
        0.71619, 0.72578, 0.73550, 0.74519, 0.75469, 0.76388, 0.77258, 0.78060, 0.78774, 0.79370, 0.79828,\
        0.80193, 0.80520, 0.80881, 0.81330, 0.81900, 0.82605, 0.83440, 0.84382, 0.85387, 0.86396, 0.87361,\
        0.88248, 0.89041, 0.89787, 0.90422, 0.91027, 0.91556, 0.92043, 0.92558, 0.93050, 0.93462, 0.93735,\
        0.93821, 0.93691, 0.93341, 0.92785, 0.92050, 0.91198, 0.90331, 0.89494, 0.88683, 0.87957, 0.87239,\
        0.86511, 0.85795, 0.85109, 0.84463, 0.83856, 0.83280, 0.82723, 0.82167, 0.81597, 0.81003, 0.80386,\
        0.79787, 0.79200, 0.78557, 0.77863, 0.77131, 0.76373, 0.75600, 0.74824, 0.74058, 0.73310, 0.72585,\
        0.71881, 0.71187, 0.70495, 0.69837, 0.69133, 0.68386, 0.67619, 0.66839, 0.66056, 0.65276, 0.64503,\
        0.63742, 0.62995, 0.62267, 0.61560, 0.60877, 0.60223, 0.59616, 0.59057, 0.58528, 0.58022, 0.57535,\
        0.57062, 0.56601, 0.56149, 0.55700, 0.55253, 0.54804, 0.54348, 0.53882, 0.53402, 0.52904, 0.52385],\
        [0.59177, 0.59337, 0.59482, 0.59623, 0.59768, 0.59929, 0.60078, 0.60264, 0.60554, 0.60951, 0.61450,\
        0.62046, 0.62732, 0.63495, 0.64317, 0.65177, 0.66053, 0.66925, 0.67783, 0.68637, 0.69511, 0.70412,\
        0.71352, 0.72323, 0.73308, 0.74291, 0.75259, 0.76200, 0.77101, 0.77949, 0.78728, 0.79413, 0.79956,\
        0.80342, 0.80686, 0.81067, 0.81522, 0.82068, 0.82713, 0.83446, 0.84243, 0.85067, 0.85878, 0.86643,\
        0.87344, 0.87986, 0.88586, 0.89153, 0.89768, 0.90289, 0.90850, 0.91487, 0.92061, 0.92528, 0.92823,\
        0.92903, 0.92751, 0.92378, 0.91823, 0.91102, 0.90251, 0.89424, 0.88646, 0.87891, 0.87177, 0.86487,\
        0.85811, 0.85153, 0.84527, 0.83939, 0.83390, 0.82872, 0.82371, 0.81865, 0.81335, 0.80769, 0.80161,\
        0.79553, 0.78938, 0.78280, 0.77575, 0.76831, 0.76063, 0.75282, 0.74502, 0.73739, 0.73005, 0.72308,\
        0.71647, 0.71005, 0.70342, 0.69665, 0.68975, 0.68226, 0.67449, 0.66663, 0.65877, 0.65098, 0.64328,\
        0.63572, 0.62833, 0.62114, 0.61419, 0.60752, 0.60116, 0.59510, 0.58942, 0.58407, 0.57898, 0.57410,\
        0.56938, 0.56479, 0.56030, 0.55586, 0.55144, 0.54701, 0.54253, 0.53795, 0.53323, 0.52833, 0.52320],\
        [0.58991, 0.59160, 0.59317, 0.59469, 0.59625, 0.59784, 0.59926, 0.60076, 0.60315, 0.60662, 0.61117,\
        0.61678, 0.62337, 0.63082, 0.63893, 0.64747, 0.65624, 0.66507, 0.67390, 0.68276, 0.69174, 0.70095,\
        0.71051, 0.72036, 0.73028, 0.74016, 0.74992, 0.75946, 0.76868, 0.77748, 0.78571, 0.79318, 0.79939,\
        0.80346, 0.80717, 0.81125, 0.81585, 0.82105, 0.82690, 0.83331, 0.84006, 0.84690, 0.85353, 0.85977,\
        0.86554, 0.87093, 0.87612, 0.88134, 0.88674, 0.89188, 0.89805, 0.90465, 0.91082, 0.91610, 0.91936,\
        0.92022, 0.91860, 0.91467, 0.90922, 0.90302, 0.89474, 0.88656, 0.87882, 0.87144, 0.86440, 0.85766,\
        0.85119, 0.84502, 0.83923, 0.83387, 0.82897, 0.82444, 0.82010, 0.81570, 0.81094, 0.80568, 0.80005,\
        0.79373, 0.78714, 0.78022, 0.77291, 0.76527, 0.75739, 0.74942, 0.74149, 0.73379, 0.72649, 0.71971,\
        0.71350, 0.70777, 0.70227, 0.69549, 0.68788, 0.68012, 0.67227, 0.66441, 0.65659, 0.64886, 0.64125,\
        0.63380, 0.62652, 0.61944, 0.61262, 0.60609, 0.59988, 0.59387, 0.58817, 0.58278, 0.57766, 0.57276,\
        0.56804, 0.56345, 0.55898, 0.55458, 0.55022, 0.54585, 0.54144, 0.53694, 0.53231, 0.52749, 0.52244],\
        [0.58797, 0.58971, 0.59130, 0.59281, 0.59427, 0.59566, 0.59694, 0.59837, 0.60049, 0.60356, 0.60771,\
        0.61299, 0.61937, 0.62670, 0.63475, 0.64329, 0.65211, 0.66107, 0.67010, 0.67918, 0.68833, 0.69762,\
        0.70735, 0.71726, 0.72712, 0.73691, 0.74663, 0.75619, 0.76549, 0.77440, 0.78274, 0.79024, 0.79643,\
        0.80127, 0.80590, 0.81050, 0.81522, 0.82022, 0.82558, 0.83125, 0.83706, 0.84284, 0.84839, 0.85362,\
        0.85850, 0.86316, 0.86775, 0.87249, 0.87747, 0.88281, 0.88882, 0.89515, 0.90162, 0.90742, 0.91072,\
        0.91170, 0.91018, 0.90602, 0.90024, 0.89448, 0.88725, 0.87932, 0.87162, 0.86427, 0.85728, 0.85063,\
        0.84434, 0.83843, 0.83299, 0.82807, 0.82372, 0.81986, 0.81632, 0.81273, 0.80862, 0.80363, 0.79795,\
        0.79174, 0.78493, 0.77767, 0.77006, 0.76215, 0.75403, 0.74583, 0.73769, 0.72980, 0.72236, 0.71558,\
        0.70957, 0.70407, 0.69863, 0.69228, 0.68488, 0.67718, 0.66941, 0.66165, 0.65396, 0.64638, 0.63892,\
        0.63162, 0.62449, 0.61756, 0.61085, 0.60440, 0.59824, 0.59236, 0.58673, 0.58135, 0.57621, 0.57129,\
        0.56655, 0.56197, 0.55751, 0.55314, 0.54882, 0.54452, 0.54019, 0.53578, 0.53124, 0.52651, 0.52154],\
        [0.58597, 0.58771, 0.58929, 0.59073, 0.59207, 0.59331, 0.59450, 0.59587, 0.59776, 0.60047, 0.60425,\
        0.60923, 0.61546, 0.62275, 0.63082, 0.63938, 0.64825, 0.65730, 0.66645, 0.67570, 0.68505, 0.69457,\
        0.70429, 0.71396, 0.72354, 0.73312, 0.74270, 0.75219, 0.76145, 0.77034, 0.77866, 0.78617, 0.79264,\
        0.79825, 0.80363, 0.80865, 0.81346, 0.81831, 0.82333, 0.82846, 0.83361, 0.83862, 0.84338, 0.84786,\
        0.85207, 0.85613, 0.86022, 0.86452, 0.86919, 0.87437, 0.88015, 0.88641, 0.89291, 0.89866, 0.90159,\
        0.90295, 0.90188, 0.89774, 0.89212, 0.88607, 0.87929, 0.87190, 0.86443, 0.85717, 0.85024, 0.84367,\
        0.83749, 0.83175, 0.82655, 0.82198, 0.81810, 0.81488, 0.81218, 0.80958, 0.80638, 0.80184, 0.79608,\
        0.78960, 0.78251, 0.77497, 0.76708, 0.75892, 0.75056, 0.74211, 0.73370, 0.72552, 0.71780, 0.71076,\
        0.70467, 0.69947, 0.69375, 0.68759, 0.68069, 0.67336, 0.66586, 0.65835, 0.65089, 0.64353, 0.63629,\
        0.62919, 0.62225, 0.61548, 0.60891, 0.60256, 0.59649, 0.59069, 0.58511, 0.57974, 0.57459, 0.56965,\
        0.56489, 0.56030, 0.55584, 0.55150, 0.54723, 0.54300, 0.53876, 0.53444, 0.53000, 0.52538, 0.52050],\
        [0.58394, 0.58569, 0.58724, 0.58864, 0.58990, 0.59106, 0.59220, 0.59351, 0.59522, 0.59762, 0.60099,\
        0.60567, 0.61185, 0.61920, 0.62729, 0.63585, 0.64471, 0.65377, 0.66295, 0.67226, 0.68172, 0.69135,\
        0.70098, 0.71026, 0.71942, 0.72873, 0.73814, 0.74751, 0.75669, 0.76551, 0.77381, 0.78145, 0.78837,\
        0.79470, 0.80060, 0.80582, 0.81065, 0.81544, 0.82028, 0.82510, 0.82980, 0.83428, 0.83847, 0.84237,\
        0.84603, 0.84957, 0.85317, 0.85701, 0.86126, 0.86608, 0.87149, 0.87738, 0.88334, 0.88842, 0.89160,\
        0.89319, 0.89242, 0.88886, 0.88377, 0.87786, 0.87132, 0.86430, 0.85711, 0.85002, 0.84319, 0.83671,\
        0.83063, 0.82501, 0.81994, 0.81560, 0.81209, 0.80938, 0.80742, 0.80586, 0.80373, 0.79983, 0.79386,\
        0.78701, 0.77966, 0.77191, 0.76385, 0.75553, 0.74701, 0.73835, 0.72967, 0.72116, 0.71312, 0.70580,\
        0.69921, 0.69394, 0.68842, 0.68239, 0.67583, 0.66889, 0.66176, 0.65457, 0.64742, 0.64035, 0.63338,\
        0.62653, 0.61982, 0.61326, 0.60685, 0.60064, 0.59465, 0.58889, 0.58333, 0.57797, 0.57279, 0.56782,\
        0.56302, 0.55841, 0.55396, 0.54964, 0.54542, 0.54125, 0.53710, 0.53290, 0.52857, 0.52406, 0.51929],\
        [0.58194, 0.58370, 0.58526, 0.58665, 0.58790, 0.58906, 0.59022, 0.59152, 0.59313, 0.59528, 0.59823,\
        0.60263, 0.60892, 0.61632, 0.62436, 0.63282, 0.64157, 0.65051, 0.65958, 0.66879, 0.67811, 0.68754,\
        0.69692, 0.70576, 0.71457, 0.72373, 0.73302, 0.74228, 0.75133, 0.76006, 0.76837, 0.77615, 0.78339,\
        0.79014, 0.79636, 0.80183, 0.80681, 0.81170, 0.81654, 0.82125, 0.82572, 0.82984, 0.83361, 0.83704,\
        0.84022, 0.84327, 0.84634, 0.84964, 0.85335, 0.85762, 0.86248, 0.86777, 0.87305, 0.87764, 0.88089,\
        0.88248, 0.88199, 0.87929, 0.87493, 0.86948, 0.86328, 0.85658, 0.84965, 0.84275, 0.83607, 0.82973,\
        0.82378, 0.81826, 0.81326, 0.80903, 0.80584, 0.80349, 0.80195, 0.80108, 0.79952, 0.79592, 0.79021,\
        0.78346, 0.77611, 0.76836, 0.76029, 0.75196, 0.74340, 0.73466, 0.72581, 0.71699, 0.70857, 0.70120,\
        0.69466, 0.68880, 0.68309, 0.67708, 0.67071, 0.66405, 0.65725, 0.65042, 0.64361, 0.63688, 0.63024,\
        0.62368, 0.61723, 0.61089, 0.60469, 0.59863, 0.59272, 0.58698, 0.58141, 0.57602, 0.57080, 0.56577,\
        0.56093, 0.55628, 0.55181, 0.54751, 0.54333, 0.53924, 0.53519, 0.53111, 0.52692, 0.52255, 0.51790],\
        [0.57999, 0.58180, 0.58341, 0.58485, 0.58616, 0.58740, 0.58866, 0.59005, 0.59173, 0.59389, 0.59677,\
        0.60103, 0.60721, 0.61438, 0.62216, 0.63036, 0.63885, 0.64753, 0.65634, 0.66526, 0.67423, 0.68317,\
        0.69191, 0.70033, 0.70916, 0.71830, 0.72751, 0.73662, 0.74552, 0.75413, 0.76239, 0.77026, 0.77769,\
        0.78464, 0.79104, 0.79678, 0.80210, 0.80726, 0.81228, 0.81706, 0.82142, 0.82532, 0.82876, 0.83182,\
        0.83456, 0.83710, 0.83960, 0.84226, 0.84529, 0.84887, 0.85305, 0.85769, 0.86241, 0.86666, 0.86986,\
        0.87157, 0.87147, 0.86947, 0.86584, 0.86096, 0.85518, 0.84879, 0.84210, 0.83539, 0.82889, 0.82275,\
        0.81700, 0.81164, 0.80672, 0.80251, 0.79953, 0.79767, 0.79623, 0.79526, 0.79351, 0.79016, 0.78507,\
        0.77879, 0.77176, 0.76423, 0.75633, 0.74815, 0.73974, 0.73113, 0.72232, 0.71338, 0.70455, 0.69685,\
        0.69036, 0.68416, 0.67807, 0.67188, 0.66552, 0.65903, 0.65250, 0.64599, 0.63956, 0.63321, 0.62692,\
        0.62069, 0.61452, 0.60842, 0.60241, 0.59650, 0.59067, 0.58494, 0.57934, 0.57388, 0.56858, 0.56348,\
        0.55857, 0.55388, 0.54938, 0.54508, 0.54095, 0.53693, 0.53299, 0.52905, 0.52502, 0.52080, 0.51629],\
        [0.57813, 0.58001, 0.58172, 0.58326, 0.58471, 0.58611, 0.58754, 0.58914, 0.59106, 0.59350, 0.59677,\
        0.60119, 0.60674, 0.61335, 0.62067, 0.62845, 0.63654, 0.64484, 0.65326, 0.66176, 0.67027, 0.67870,\
        0.68697, 0.69518, 0.70377, 0.71277, 0.72183, 0.73071, 0.73935, 0.74777, 0.75595, 0.76385, 0.77139,\
        0.77852, 0.78514, 0.79123, 0.79692, 0.80234, 0.80765, 0.81263, 0.81697, 0.82068, 0.82387, 0.82662,\
        0.82898, 0.83103, 0.83292, 0.83484, 0.83706, 0.83983, 0.84327, 0.84732, 0.85163, 0.85569, 0.85893,\
        0.86088, 0.86120, 0.85981, 0.85682, 0.85249, 0.84712, 0.84102, 0.83452, 0.82796, 0.82163, 0.81575,\
        0.81031, 0.80522, 0.80051, 0.79650, 0.79364, 0.79172, 0.79027, 0.78888, 0.78685, 0.78362, 0.77901,\
        0.77326, 0.76669, 0.75953, 0.75195, 0.74408, 0.73598, 0.72771, 0.71927, 0.71066, 0.70200, 0.69396,\
        0.68678, 0.67998, 0.67337, 0.66684, 0.66034, 0.65390, 0.64757, 0.64137, 0.63533, 0.62939, 0.62351,\
        0.61762, 0.61174, 0.60589, 0.60007, 0.59428, 0.58850, 0.58276, 0.57709, 0.57153, 0.56613, 0.56093,\
        0.55593, 0.55117, 0.54664, 0.54233, 0.53822, 0.53428, 0.53045, 0.52666, 0.52282, 0.51879, 0.51446],\
        [0.57634, 0.57834, 0.58018, 0.58188, 0.58351, 0.58513, 0.58681, 0.58869, 0.59090, 0.59365, 0.59717,\
        0.60155, 0.60680, 0.61291, 0.61971, 0.62699, 0.63459, 0.64241, 0.65037, 0.65840, 0.66643, 0.67443,\
        0.68239, 0.69040, 0.69868, 0.70743, 0.71616, 0.72462, 0.73289, 0.74105, 0.74911, 0.75701, 0.76466,\
        0.77198, 0.77888, 0.78536, 0.79144, 0.79720, 0.80294, 0.80810, 0.81232, 0.81586, 0.81888, 0.82142,\
        0.82348, 0.82508, 0.82632, 0.82743, 0.82873, 0.83058, 0.83327, 0.83682, 0.84092, 0.84499, 0.84841,\
        0.85067, 0.85144, 0.85058, 0.84813, 0.84428, 0.83926, 0.83339, 0.82701, 0.82052, 0.81428, 0.80868,\
        0.80367, 0.79886, 0.79446, 0.79071, 0.78784, 0.78572, 0.78401, 0.78226, 0.77998, 0.77678, 0.77249,\
        0.76720, 0.76108, 0.75435, 0.74717, 0.73968, 0.73200, 0.72419, 0.71632, 0.70835, 0.70013, 0.69167,\
        0.68363, 0.67607, 0.66885, 0.66189, 0.65516, 0.64868, 0.64249, 0.63660, 0.63097, 0.62551, 0.62009,\
        0.61458, 0.60897, 0.60336, 0.59775, 0.59204, 0.58626, 0.58045, 0.57466, 0.56897, 0.56343, 0.55809,\
        0.55298, 0.54813, 0.54354, 0.53922, 0.53513, 0.53125, 0.52754, 0.52391, 0.52028, 0.51648, 0.51235],\
        [0.57463, 0.57677, 0.57877, 0.58067, 0.58252, 0.58440, 0.58636, 0.58853, 0.59103, 0.59401, 0.59764,\
        0.60197, 0.60701, 0.61275, 0.61907, 0.62584, 0.63292, 0.64021, 0.64765, 0.65518, 0.66275, 0.67036,\
        0.67802, 0.68585, 0.69396, 0.70237, 0.71047, 0.71831, 0.72611, 0.73398, 0.74191, 0.74983, 0.75760,\
        0.76513, 0.77233, 0.77919, 0.78573, 0.79201, 0.79808, 0.80326, 0.80727, 0.81071, 0.81369, 0.81615,\
        0.81805, 0.81931, 0.81994, 0.82016, 0.82045, 0.82129, 0.82319, 0.82638, 0.83050, 0.83479, 0.83853,\
        0.84117, 0.84235, 0.84194, 0.83994, 0.83648, 0.83175, 0.82602, 0.81969, 0.81322, 0.80698, 0.80139,\
        0.79690, 0.79237, 0.78824, 0.78470, 0.78183, 0.77954, 0.77755, 0.77551, 0.77305, 0.76985, 0.76578,\
        0.76084, 0.75512, 0.74879, 0.74201, 0.73491, 0.72762, 0.72025, 0.71287, 0.70551, 0.69750, 0.68880,\
        0.68023, 0.67206, 0.66429, 0.65691, 0.64991, 0.64334, 0.63725, 0.63165, 0.62648, 0.62159, 0.61676,\
        0.61172, 0.60633, 0.60085, 0.59544, 0.58981, 0.58396, 0.57800, 0.57204, 0.56616, 0.56045, 0.55495,\
        0.54971, 0.54475, 0.54009, 0.53573, 0.53165, 0.52782, 0.52421, 0.52074, 0.51733, 0.51381, 0.50994],\
        [0.57295, 0.57525, 0.57743, 0.57955, 0.58166, 0.58381, 0.58608, 0.58855, 0.59130, 0.59445, 0.59810,\
        0.60238, 0.60728, 0.61272, 0.61861, 0.62488, 0.63142, 0.63817, 0.64506, 0.65207, 0.65916, 0.66635,\
        0.67367, 0.68120, 0.68900, 0.69692, 0.70436, 0.71160, 0.71894, 0.72655, 0.73441, 0.74238, 0.75029,\
        0.75803, 0.76551, 0.77270, 0.77958, 0.78615, 0.79226, 0.79745, 0.80151, 0.80510, 0.80822, 0.81076,\
        0.81269, 0.81383, 0.81391, 0.81320, 0.81246, 0.81224, 0.81325, 0.81629, 0.82070, 0.82537, 0.82950,\
        0.83251, 0.83404, 0.83396, 0.83231, 0.82918, 0.82470, 0.81907, 0.81266, 0.80616, 0.80020, 0.79458,\
        0.78995, 0.78570, 0.78180, 0.77841, 0.77555, 0.77313, 0.77095, 0.76871, 0.76612, 0.76293, 0.75902,\
        0.75434, 0.74895, 0.74296, 0.73651, 0.72975, 0.72280, 0.71571, 0.70847, 0.70127, 0.69354, 0.68491,\
        0.67618, 0.66768, 0.65953, 0.65180, 0.64453, 0.63784, 0.63181, 0.62650, 0.62184, 0.61763, 0.61355,\
        0.60923, 0.60431, 0.59884, 0.59345, 0.58766, 0.58158, 0.57537, 0.56917, 0.56307, 0.55716, 0.55148,\
        0.54609, 0.54101, 0.53626, 0.53185, 0.52777, 0.52397, 0.52043, 0.51709, 0.51390, 0.51072, 0.50718],\
        [0.57127, 0.57374, 0.57612, 0.57847, 0.58084, 0.58327, 0.58585, 0.58862, 0.59164, 0.59495, 0.59859,\
        0.60283, 0.60759, 0.61274, 0.61822, 0.62399, 0.63000, 0.63619, 0.64253, 0.64899, 0.65557, 0.66228,\
        0.66915, 0.67621, 0.68343, 0.69064, 0.69759, 0.70446, 0.71140, 0.71883, 0.72672, 0.73479, 0.74283,\
        0.75074, 0.75844, 0.76585, 0.77293, 0.77959, 0.78565, 0.79088, 0.79525, 0.79908, 0.80242, 0.80512,\
        0.80731, 0.80873, 0.80839, 0.80662, 0.80504, 0.80396, 0.80385, 0.80712, 0.81194, 0.81699, 0.82149,\
        0.82478, 0.82651, 0.82659, 0.82516, 0.82234, 0.81812, 0.81259, 0.80613, 0.79936, 0.79350, 0.78807,\
        0.78325, 0.77897, 0.77516, 0.77183, 0.76898, 0.76650, 0.76421, 0.76186, 0.75921, 0.75607, 0.75228,\
        0.74781, 0.74267, 0.73696, 0.73076, 0.72423, 0.71753, 0.71074, 0.70363, 0.69610, 0.68846, 0.68015,\
        0.67152, 0.66293, 0.65455, 0.64655, 0.63903, 0.63217, 0.62614, 0.62109, 0.61697, 0.61351, 0.61030,\
        0.60692, 0.60289, 0.59776, 0.59177, 0.58547, 0.57899, 0.57247, 0.56599, 0.55966, 0.55353, 0.54767,\
        0.54212, 0.53692, 0.53208, 0.52762, 0.52351, 0.51972, 0.51621, 0.51293, 0.50989, 0.50704, 0.50397],\
        [0.56955, 0.57218, 0.57476, 0.57733, 0.57995, 0.58267, 0.58554, 0.58861, 0.59192, 0.59550, 0.59931,\
        0.60344, 0.60793, 0.61272, 0.61779, 0.62308, 0.62856, 0.63420, 0.63998, 0.64588, 0.65191, 0.65808,\
        0.66441, 0.67090, 0.67750, 0.68410, 0.69060, 0.69704, 0.70358, 0.71103, 0.71906, 0.72719, 0.73530,\
        0.74333, 0.75117, 0.75872, 0.76589, 0.77257, 0.77863, 0.78395, 0.78857, 0.79262, 0.79614, 0.79905,\
        0.80166, 0.80402, 0.80331, 0.80048, 0.79823, 0.79690, 0.79663, 0.79975, 0.80451, 0.80974, 0.81449,\
        0.81798, 0.81968, 0.81964, 0.81824, 0.81571, 0.81185, 0.80640, 0.80001, 0.79330, 0.78706, 0.78148,\
        0.77650, 0.77211, 0.76829, 0.76499, 0.76215, 0.75966, 0.75735, 0.75499, 0.75236, 0.74928, 0.74562,\
        0.74132, 0.73639, 0.73088, 0.72487, 0.71845, 0.71184, 0.70537, 0.69858, 0.69108, 0.68323, 0.67502,\
        0.66652, 0.65793, 0.64946, 0.64126, 0.63351, 0.62643, 0.62029, 0.61540, 0.61180, 0.60909, 0.60669,\
        0.60409, 0.60106, 0.59586, 0.58950, 0.58279, 0.57597, 0.56915, 0.56243, 0.55587, 0.54956, 0.54352,\
        0.53782, 0.53249, 0.52756, 0.52304, 0.51892, 0.51514, 0.51163, 0.50835, 0.50527, 0.50252, 0.50009],\
        [0.56774, 0.57052, 0.57328, 0.57605, 0.57891, 0.58188, 0.58502, 0.58837, 0.59196, 0.59580, 0.59984,\
        0.60385, 0.60807, 0.61254, 0.61720, 0.62204, 0.62701, 0.63211, 0.63732, 0.64265, 0.64811, 0.65372,\
        0.65948, 0.66540, 0.67144, 0.67753, 0.68362, 0.68976, 0.69621, 0.70372, 0.71170, 0.71973, 0.72780,\
        0.73586, 0.74379, 0.75142, 0.75861, 0.76527, 0.77131, 0.77668, 0.78141, 0.78557, 0.78924, 0.79248,\
        0.79567, 0.79840, 0.79730, 0.79431, 0.79177, 0.79048, 0.79087, 0.79360, 0.79811, 0.80333, 0.80834,\
        0.81199, 0.81335, 0.81276, 0.81109, 0.80879, 0.80555, 0.80015, 0.79355, 0.78682, 0.78042, 0.77463,\
        0.76948, 0.76500, 0.76114, 0.75785, 0.75506, 0.75264, 0.75039, 0.74811, 0.74558, 0.74260, 0.73907,\
        0.73491, 0.73015, 0.72482, 0.71899, 0.71269, 0.70590, 0.69941, 0.69292, 0.68565, 0.67790, 0.66979,\
        0.66142, 0.65292, 0.64444, 0.63615, 0.62821, 0.62085, 0.61444, 0.60954, 0.60642, 0.60435, 0.60254,\
        0.60018, 0.59699, 0.59225, 0.58604, 0.57928, 0.57231, 0.56532, 0.55842, 0.55170, 0.54523, 0.53906,\
        0.53323, 0.52778, 0.52275, 0.51818, 0.51407, 0.51033, 0.50683, 0.50353, 0.50042, 0.49755, 0.49489],\
        [0.56579, 0.56870, 0.57161, 0.57456, 0.57761, 0.58080, 0.58417, 0.58775, 0.59154, 0.59551, 0.59956,\
        0.60367, 0.60781, 0.61202, 0.61634, 0.62075, 0.62525, 0.62983, 0.63449, 0.63925, 0.64414, 0.64918,\
        0.65440, 0.65979, 0.66534, 0.67104, 0.67689, 0.68299, 0.68960, 0.69693, 0.70471, 0.71238, 0.72032,\
        0.72843, 0.73641, 0.74405, 0.75121, 0.75779, 0.76374, 0.76905, 0.77374, 0.77788, 0.78154, 0.78481,\
        0.78767, 0.78940, 0.78877, 0.78660, 0.78458, 0.78372, 0.78458, 0.78741, 0.79187, 0.79719, 0.80259,\
        0.80660, 0.80717, 0.80555, 0.80328, 0.80087, 0.79800, 0.79288, 0.78630, 0.77957, 0.77314, 0.76727,\
        0.76204, 0.75750, 0.75364, 0.75040, 0.74771, 0.74543, 0.74335, 0.74125, 0.73888, 0.73606, 0.73266,\
        0.72863, 0.72399, 0.71883, 0.71321, 0.70719, 0.70084, 0.69388, 0.68722, 0.68011, 0.67254, 0.66460,\
        0.65641, 0.64806, 0.63970, 0.63143, 0.62343, 0.61588, 0.60910, 0.60380, 0.60121, 0.59952, 0.59780,\
        0.59548, 0.59203, 0.58736, 0.58152, 0.57494, 0.56801, 0.56097, 0.55399, 0.54716, 0.54058, 0.53431,\
        0.52839, 0.52284, 0.51772, 0.51309, 0.50903, 0.50539, 0.50195, 0.49865, 0.49558, 0.49263, 0.48959],\
        [0.56365, 0.56666, 0.56969, 0.57278, 0.57598, 0.57934, 0.58288, 0.58663, 0.59058, 0.59469, 0.59887,\
        0.60302, 0.60705, 0.61105, 0.61508, 0.61913, 0.62320, 0.62730, 0.63143, 0.63563, 0.63996, 0.64446,\
        0.64915, 0.65407, 0.65922, 0.66462, 0.67032, 0.67641, 0.68302, 0.69020, 0.69762, 0.70489, 0.71295,\
        0.72115, 0.72915, 0.73673, 0.74378, 0.75020, 0.75597, 0.76109, 0.76559, 0.76952, 0.77292, 0.77579,\
        0.77797, 0.77902, 0.77862, 0.77727, 0.77602, 0.77579, 0.77715, 0.78026, 0.78491, 0.79055, 0.79648,\
        0.80103, 0.80033, 0.79777, 0.79501, 0.79215, 0.78869, 0.78382, 0.77775, 0.77132, 0.76506, 0.75926,\
        0.75407, 0.74957, 0.74576, 0.74263, 0.74010, 0.73805, 0.73624, 0.73442, 0.73231, 0.72969, 0.72643,\
        0.72248, 0.71791, 0.71284, 0.70742, 0.70154, 0.69538, 0.68870, 0.68180, 0.67468, 0.66725, 0.65953,\
        0.65159, 0.64350, 0.63537, 0.62730, 0.61943, 0.61198, 0.60525, 0.59971, 0.59685, 0.59502, 0.59302,\
        0.59039, 0.58677, 0.58209, 0.57642, 0.57002, 0.56320, 0.55619, 0.54918, 0.54230, 0.53567, 0.52936,\
        0.52340, 0.51781, 0.51259, 0.50786, 0.50384, 0.50039, 0.49703, 0.49379, 0.49068, 0.48760, 0.48432],\
        [0.56130, 0.56438, 0.56748, 0.57065, 0.57395, 0.57742, 0.58108, 0.58494, 0.58901, 0.59324, 0.59756,\
        0.60171, 0.60565, 0.60952, 0.61332, 0.61709, 0.62080, 0.62445, 0.62807, 0.63174, 0.63554, 0.63952,\
        0.64374, 0.64823, 0.65304, 0.65821, 0.66377, 0.66981, 0.67634, 0.68329, 0.69043, 0.69791, 0.70601,\
        0.71418, 0.72211, 0.72956, 0.73639, 0.74256, 0.74805, 0.75286, 0.75702, 0.76058, 0.76353, 0.76583,\
        0.76735, 0.76793, 0.76758, 0.76671, 0.76608, 0.76643, 0.76823, 0.77165, 0.77646, 0.78215, 0.78781,\
        0.79171, 0.79156, 0.78907, 0.78605, 0.78269, 0.77874, 0.77388, 0.76821, 0.76217, 0.75621, 0.75063,\
        0.74559, 0.74120, 0.73750, 0.73453, 0.73224, 0.73051, 0.72909, 0.72766, 0.72588, 0.72351, 0.72039,\
        0.71648, 0.71191, 0.70682, 0.70147, 0.69584, 0.68966, 0.68315, 0.67635, 0.66933, 0.66209, 0.65463,\
        0.64700, 0.63927, 0.63151, 0.62381, 0.61629, 0.60917, 0.60287, 0.59779, 0.59408, 0.59137, 0.58870,\
        0.58555, 0.58161, 0.57679, 0.57114, 0.56483, 0.55808, 0.55111, 0.54410, 0.53721, 0.53057, 0.52427,\
        0.51834, 0.51280, 0.50759, 0.50272, 0.49867, 0.49530, 0.49207, 0.48889, 0.48574, 0.48253, 0.47910],\
        [0.55870, 0.56180, 0.56493, 0.56814, 0.57148, 0.57499, 0.57870, 0.58262, 0.58673, 0.59100, 0.59531,\
        0.59948, 0.60352, 0.60734, 0.61099, 0.61456, 0.61799, 0.62124, 0.62438, 0.62753, 0.63082, 0.63433,\
        0.63812, 0.64224, 0.64675, 0.65171, 0.65715, 0.66313, 0.66962, 0.67655, 0.68383, 0.69149, 0.69949,\
        0.70756, 0.71536, 0.72255, 0.72909, 0.73492, 0.74002, 0.74442, 0.74815, 0.75124, 0.75366, 0.75536,\
        0.75629, 0.75644, 0.75596, 0.75527, 0.75497, 0.75568, 0.75781, 0.76145, 0.76631, 0.77175, 0.77676,\
        0.78000, 0.78050, 0.77880, 0.77597, 0.77245, 0.76829, 0.76345, 0.75804, 0.75237, 0.74677, 0.74150,\
        0.73671, 0.73251, 0.72896, 0.72617, 0.72417, 0.72285, 0.72192, 0.72099, 0.71964, 0.71752, 0.71454,\
        0.71065, 0.70599, 0.70102, 0.69547, 0.68984, 0.68380, 0.67743, 0.67080, 0.66399, 0.65701, 0.64988,\
        0.64264, 0.63535, 0.62807, 0.62089, 0.61391, 0.60723, 0.60102, 0.59616, 0.59206, 0.58847, 0.58497,\
        0.58116, 0.57677, 0.57169, 0.56593, 0.55959, 0.55286, 0.54590, 0.53889, 0.53199, 0.52535, 0.51909,\
        0.51328, 0.50789, 0.50288, 0.49814, 0.49399, 0.49048, 0.48722, 0.48403, 0.48082, 0.47750, 0.47394],\
        [0.55584, 0.55891, 0.56201, 0.56520, 0.56852, 0.57202, 0.57571, 0.57962, 0.58372, 0.58796, 0.59226,\
        0.59653, 0.60069, 0.60441, 0.60797, 0.61151, 0.61476, 0.61762, 0.62027, 0.62293, 0.62576, 0.62885,\
        0.63226, 0.63606, 0.64030, 0.64507, 0.65042, 0.65638, 0.66291, 0.66994, 0.67739, 0.68520, 0.69323,\
        0.70131, 0.70884, 0.71564, 0.72187, 0.72731, 0.73195, 0.73586, 0.73910, 0.74168, 0.74357, 0.74471,\
        0.74508, 0.74479, 0.74404, 0.74326, 0.74299, 0.74380, 0.74613, 0.74996, 0.75486, 0.76007, 0.76465,\
        0.76764, 0.76852, 0.76746, 0.76501, 0.76162, 0.75748, 0.75273, 0.74755, 0.74220, 0.73697, 0.73208,\
        0.72763, 0.72368, 0.72030, 0.71766, 0.71598, 0.71513, 0.71476, 0.71446, 0.71358, 0.71167, 0.70878,\
        0.70504, 0.70009, 0.69504, 0.68956, 0.68380, 0.67780, 0.67157, 0.66516, 0.65861, 0.65196, 0.64522,\
        0.63844, 0.63165, 0.62492, 0.61834, 0.61202, 0.60607, 0.60054, 0.59523, 0.59044, 0.58603, 0.58171,\
        0.57721, 0.57230, 0.56687, 0.56091, 0.55448, 0.54769, 0.54071, 0.53367, 0.52675, 0.52011, 0.51390,\
        0.50821, 0.50303, 0.49823, 0.49370, 0.48960, 0.48595, 0.48258, 0.47930, 0.47600, 0.47257, 0.46889],\
        [0.55270, 0.55570, 0.55873, 0.56184, 0.56508, 0.56849, 0.57211, 0.57594, 0.57996, 0.58415, 0.58842,\
        0.59270, 0.59684, 0.60048, 0.60421, 0.60799, 0.61111, 0.61355, 0.61567, 0.61786, 0.62031, 0.62307,\
        0.62617, 0.62967, 0.63367, 0.63827, 0.64355, 0.64953, 0.65616, 0.66334, 0.67097, 0.67891, 0.68702,\
        0.69503, 0.70199, 0.70877, 0.71477, 0.71972, 0.72385, 0.72723, 0.72996, 0.73208, 0.73351, 0.73412,\
        0.73397, 0.73323, 0.73211, 0.73104, 0.73053, 0.73121, 0.73362, 0.73772, 0.74281, 0.74799, 0.75240,\
        0.75533, 0.75643, 0.75577, 0.75368, 0.75050, 0.74652, 0.74193, 0.73696, 0.73190, 0.72705, 0.72260,\
        0.71858, 0.71501, 0.71180, 0.70919, 0.70786, 0.70749, 0.70763, 0.70808, 0.70773, 0.70579, 0.70273,\
        0.69912, 0.69435, 0.68893, 0.68335, 0.67757, 0.67163, 0.66556, 0.65939, 0.65315, 0.64688, 0.64059,\
        0.63430, 0.62804, 0.62188, 0.61589, 0.61015, 0.60479, 0.59961, 0.59414, 0.58881, 0.58372, 0.57871,\
        0.57358, 0.56817, 0.56237, 0.55616, 0.54958, 0.54271, 0.53567, 0.52859, 0.52163, 0.51496, 0.50876,\
        0.50320, 0.49821, 0.49360, 0.48928, 0.48528, 0.48160, 0.47812, 0.47474, 0.47132, 0.46777, 0.46398],\
        [0.54930, 0.55218, 0.55507, 0.55805, 0.56115, 0.56442, 0.56790, 0.57160, 0.57550, 0.57958, 0.58376,\
        0.58797, 0.59206, 0.59595, 0.60005, 0.60407, 0.60698, 0.60886, 0.61043, 0.61225, 0.61444, 0.61699,\
        0.61986, 0.62310, 0.62685, 0.63129, 0.63652, 0.64257, 0.64935, 0.65670, 0.66447, 0.67248, 0.68051,\
        0.68822, 0.69536, 0.70228, 0.70774, 0.71209, 0.71576, 0.71861, 0.72080, 0.72258, 0.72367, 0.72380,\
        0.72315, 0.72200, 0.72048, 0.71902, 0.71816, 0.71842, 0.72089, 0.72547, 0.73093, 0.73626, 0.74065,\
        0.74356, 0.74475, 0.74430, 0.74244, 0.73949, 0.73570, 0.73131, 0.72654, 0.72171, 0.71725, 0.71331,\
        0.70978, 0.70674, 0.70395, 0.70113, 0.70020, 0.70025, 0.70050, 0.70180, 0.70186, 0.69974, 0.69628,\
        0.69237, 0.68768, 0.68237, 0.67677, 0.67103, 0.66520, 0.65932, 0.65344, 0.64756, 0.64172, 0.63591,\
        0.63014, 0.62443, 0.61881, 0.61333, 0.60798, 0.60277, 0.59778, 0.59239, 0.58685, 0.58133, 0.57580,\
        0.57016, 0.56430, 0.55815, 0.55169, 0.54495, 0.53799, 0.53089, 0.52379, 0.51679, 0.51007, 0.50386,\
        0.49842, 0.49362, 0.48919, 0.48502, 0.48111, 0.47740, 0.47385, 0.47035, 0.46681, 0.46313, 0.45922],\
        [0.54564, 0.54835, 0.55107, 0.55384, 0.55675, 0.55983, 0.56312, 0.56663, 0.57037, 0.57429, 0.57836,\
        0.58250, 0.58666, 0.59085, 0.59519, 0.59924, 0.60199, 0.60326, 0.60441, 0.60607, 0.60821, 0.61067,\
        0.61338, 0.61639, 0.61989, 0.62415, 0.62937, 0.63556, 0.64253, 0.65007, 0.65795, 0.66597, 0.67391,\
        0.68155, 0.68876, 0.69529, 0.70030, 0.70426, 0.70779, 0.71011, 0.71160, 0.71331, 0.71428, 0.71386,\
        0.71265, 0.71131, 0.70946, 0.70762, 0.70669, 0.70616, 0.70884, 0.71416, 0.71999, 0.72552, 0.72992,\
        0.73273, 0.73386, 0.73343, 0.73168, 0.72888, 0.72530, 0.72117, 0.71662, 0.71191, 0.70780, 0.70450,\
        0.70143, 0.69886, 0.69673, 0.69443, 0.69344, 0.69347, 0.69403, 0.69489, 0.69476, 0.69279, 0.68930,\
        0.68516, 0.68044, 0.67523, 0.66973, 0.66411, 0.65845, 0.65282, 0.64726, 0.64179, 0.63642, 0.63113,\
        0.62592, 0.62075, 0.61564, 0.61064, 0.60572, 0.60060, 0.59552, 0.59021, 0.58457, 0.57879, 0.57289,\
        0.56685, 0.56063, 0.55419, 0.54751, 0.54062, 0.53358, 0.52646, 0.51936, 0.51239, 0.50569, 0.49951,\
        0.49419, 0.48947, 0.48512, 0.48103, 0.47714, 0.47341, 0.46977, 0.46616, 0.46249, 0.45867, 0.45463],\
        [0.54176, 0.54425, 0.54673, 0.54926, 0.55192, 0.55475, 0.55780, 0.56108, 0.56461, 0.56835, 0.57226,\
        0.57631, 0.58045, 0.58466, 0.58885, 0.59261, 0.59524, 0.59661, 0.59778, 0.59952, 0.60171, 0.60419,\
        0.60683, 0.60964, 0.61284, 0.61691, 0.62218, 0.62860, 0.63584, 0.64357, 0.65153, 0.65951, 0.66732,\
        0.67476, 0.68165, 0.68768, 0.69252, 0.69652, 0.70013, 0.70191, 0.70227, 0.70444, 0.70545, 0.70431,\
        0.70237, 0.70114, 0.69933, 0.69719, 0.69654, 0.69614, 0.69936, 0.70450, 0.71051, 0.71622, 0.72055,\
        0.72312, 0.72398, 0.72342, 0.72167, 0.71894, 0.71554, 0.71175, 0.70762, 0.70299, 0.69903, 0.69634,\
        0.69376, 0.69153, 0.68965, 0.68803, 0.68711, 0.68694, 0.68720, 0.68738, 0.68675, 0.68477, 0.68151,\
        0.67741, 0.67272, 0.66761, 0.66225, 0.65679, 0.65136, 0.64601, 0.64081, 0.63578, 0.63093, 0.62623,\
        0.62160, 0.61698, 0.61232, 0.60771, 0.60325, 0.59877, 0.59342, 0.58786, 0.58207, 0.57608, 0.56992,\
        0.56360, 0.55711, 0.55043, 0.54359, 0.53659, 0.52950, 0.52239, 0.51535, 0.50850, 0.50197, 0.49594,\
        0.49061, 0.48585, 0.48147, 0.47736, 0.47344, 0.46964, 0.46591, 0.46217, 0.45836, 0.45440, 0.45022],\
        [0.53767, 0.53991, 0.54211, 0.54434, 0.54670, 0.54924, 0.55200, 0.55502, 0.55830, 0.56181, 0.56553,\
        0.56941, 0.57341, 0.57743, 0.58132, 0.58475, 0.58738, 0.58922, 0.59084, 0.59275, 0.59502, 0.59758,\
        0.60023, 0.60293, 0.60582, 0.60968, 0.61516, 0.62196, 0.62950, 0.63739, 0.64537, 0.65324, 0.66082,\
        0.66795, 0.67443, 0.68008, 0.68481, 0.68877, 0.69199, 0.69346, 0.69407, 0.69562, 0.69630, 0.69489,\
        0.69268, 0.69108, 0.68938, 0.68782, 0.68733, 0.68821, 0.69141, 0.69640, 0.70270, 0.70845, 0.71263,\
        0.71484, 0.71521, 0.71440, 0.71261, 0.70984, 0.70650, 0.70308, 0.69950, 0.69546, 0.69174, 0.68899,\
        0.68668, 0.68468, 0.68301, 0.68167, 0.68078, 0.68035, 0.68012, 0.67968, 0.67856, 0.67641, 0.67323,\
        0.66921, 0.66460, 0.65960, 0.65437, 0.64910, 0.64391, 0.63888, 0.63407, 0.62951, 0.62521, 0.62116,\
        0.61723, 0.61321, 0.60895, 0.60457, 0.60024, 0.59582, 0.59080, 0.58525, 0.57937, 0.57323, 0.56688,\
        0.56036, 0.55368, 0.54685, 0.53989, 0.53283, 0.52573, 0.51866, 0.51172, 0.50503, 0.49870, 0.49282,\
        0.48749, 0.48266, 0.47822, 0.47403, 0.47003, 0.46613, 0.46227, 0.45839, 0.45443, 0.45032, 0.44599],\
        [0.53343, 0.53538, 0.53724, 0.53914, 0.54115, 0.54335, 0.54580, 0.54852, 0.55152, 0.55478, 0.55826,\
        0.56192, 0.56569, 0.56946, 0.57308, 0.57635, 0.57909, 0.58137, 0.58347, 0.58570, 0.58816, 0.59081,\
        0.59355, 0.59628, 0.59908, 0.60280, 0.60880, 0.61605, 0.62381, 0.63175, 0.63964, 0.64730, 0.65456,\
        0.66129, 0.66734, 0.67260, 0.67701, 0.68060, 0.68325, 0.68475, 0.68558, 0.68627, 0.68623, 0.68495,\
        0.68300, 0.68119, 0.67965, 0.67863, 0.67871, 0.68035, 0.68391, 0.68926, 0.69567, 0.70167, 0.70598,\
        0.70804, 0.70750, 0.70641, 0.70464, 0.70180, 0.69839, 0.69520, 0.69189, 0.68842, 0.68513, 0.68239,\
        0.68014, 0.67824, 0.67666, 0.67537, 0.67440, 0.67366, 0.67295, 0.67197, 0.67040, 0.66801, 0.66477,\
        0.66077, 0.65621, 0.65128, 0.64617, 0.64107, 0.63612, 0.63142, 0.62702, 0.62293, 0.61921, 0.61588,\
        0.61280, 0.60959, 0.60580, 0.60150, 0.59737, 0.59295, 0.58795, 0.58241, 0.57646, 0.57022, 0.56376,\
        0.55712, 0.55033, 0.54341, 0.53639, 0.52932, 0.52226, 0.51526, 0.50843, 0.50186, 0.49571, 0.48998,\
        0.48469, 0.47982, 0.47529, 0.47100, 0.46688, 0.46286, 0.45886, 0.45483, 0.45070, 0.44643, 0.44194],\
        [0.52909, 0.53070, 0.53220, 0.53371, 0.53533, 0.53716, 0.53927, 0.54167, 0.54438, 0.54737, 0.55059,\
        0.55399, 0.55751, 0.56105, 0.56448, 0.56770, 0.57062, 0.57328, 0.57585, 0.57848, 0.58126, 0.58417,\
        0.58715, 0.59018, 0.59341, 0.59764, 0.60396, 0.61129, 0.61900, 0.62681, 0.63447, 0.64180, 0.64866,\
        0.65492, 0.66048, 0.66526, 0.66922, 0.67234, 0.67455, 0.67587, 0.67652, 0.67665, 0.67612, 0.67482,\
        0.67307, 0.67134, 0.66999, 0.66938, 0.66993, 0.67206, 0.67598, 0.68157, 0.68820, 0.69475, 0.70041,\
        0.70309, 0.70097, 0.69950, 0.69760, 0.69466, 0.69135, 0.68811, 0.68492, 0.68177, 0.67879, 0.67618,\
        0.67396, 0.67208, 0.67049, 0.66915, 0.66799, 0.66693, 0.66579, 0.66435, 0.66237, 0.65971, 0.65632,\
        0.65226, 0.64769, 0.64277, 0.63772, 0.63274, 0.62803, 0.62367, 0.61968, 0.61605, 0.61284, 0.61025,\
        0.60822, 0.60620, 0.60336, 0.59925, 0.59494, 0.59023, 0.58502, 0.57937, 0.57335, 0.56706, 0.56054,\
        0.55385, 0.54702, 0.54007, 0.53306, 0.52603, 0.51903, 0.51215, 0.50546, 0.49904, 0.49302, 0.48740,\
        0.48215, 0.47725, 0.47264, 0.46824, 0.46399, 0.45982, 0.45567, 0.45147, 0.44717, 0.44272, 0.43807],\
        [0.52470, 0.52594, 0.52703, 0.52812, 0.52933, 0.53077, 0.53251, 0.53459, 0.53701, 0.53972, 0.54268,\
        0.54582, 0.54909, 0.55243, 0.55575, 0.55899, 0.56211, 0.56514, 0.56817, 0.57127, 0.57451, 0.57787,\
        0.58136, 0.58503, 0.58914, 0.59423, 0.60059, 0.60760, 0.61507, 0.62261, 0.62993, 0.63683, 0.64319,\
        0.64891, 0.65391, 0.65815, 0.66158, 0.66419, 0.66597, 0.66697, 0.66729, 0.66702, 0.66616, 0.66477,\
        0.66309, 0.66147, 0.66027, 0.65987, 0.66066, 0.66300, 0.66707, 0.67277, 0.67965, 0.68698, 0.69385,\
        0.69862, 0.69634, 0.69337, 0.69079, 0.68780, 0.68461, 0.68144, 0.67837, 0.67541, 0.67265, 0.67017,\
        0.66799, 0.66610, 0.66445, 0.66297, 0.66161, 0.66023, 0.65871, 0.65687, 0.65453, 0.65159, 0.64800,\
        0.64381, 0.63915, 0.63419, 0.62912, 0.62419, 0.61967, 0.61567, 0.61211, 0.60892, 0.60610, 0.60410,\
        0.60315, 0.60246, 0.60125, 0.59752, 0.59254, 0.58731, 0.58183, 0.57605, 0.56999, 0.56370, 0.55720,\
        0.55053, 0.54373, 0.53683, 0.52987, 0.52291, 0.51602, 0.50928, 0.50276, 0.49652, 0.49062, 0.48507,\
        0.47984, 0.47491, 0.47022, 0.46571, 0.46133, 0.45700, 0.45268, 0.44830, 0.44382, 0.43919, 0.43435],\
        [0.52032, 0.52117, 0.52182, 0.52245, 0.52322, 0.52426, 0.52564, 0.52740, 0.52955, 0.53200, 0.53471,\
        0.53760, 0.54064, 0.54380, 0.54704, 0.55034, 0.55369, 0.55709, 0.56059, 0.56422, 0.56801, 0.57199,\
        0.57617, 0.58064, 0.58558, 0.59124, 0.59762, 0.60458, 0.61189, 0.61911, 0.62600, 0.63239, 0.63818,\
        0.64330, 0.64770, 0.65133, 0.65420, 0.65629, 0.65761, 0.65822, 0.65818, 0.65757, 0.65645, 0.65494,\
        0.65323, 0.65162, 0.65044, 0.65005, 0.65082, 0.65307, 0.65701, 0.66259, 0.66945, 0.67692, 0.68386,\
        0.68828, 0.68831, 0.68611, 0.68352, 0.68071, 0.67777, 0.67483, 0.67195, 0.66920, 0.66661, 0.66425,\
        0.66212, 0.66021, 0.65847, 0.65685, 0.65527, 0.65361, 0.65175, 0.64956, 0.64691, 0.64369, 0.63988,\
        0.63550, 0.63070, 0.62564, 0.62050, 0.61550, 0.61108, 0.60745, 0.60434, 0.60161, 0.59922, 0.59763,\
        0.59732, 0.59724, 0.59652, 0.59355, 0.58881, 0.58356, 0.57804, 0.57229, 0.56631, 0.56011, 0.55371,\
        0.54715, 0.54045, 0.53365, 0.52680, 0.51996, 0.51320, 0.50660, 0.50026, 0.49418, 0.48841, 0.48294,\
        0.47773, 0.47277, 0.46800, 0.46338, 0.45886, 0.45437, 0.44987, 0.44531, 0.44064, 0.43581, 0.43079],\
        [0.51602, 0.51645, 0.51663, 0.51679, 0.51712, 0.51775, 0.51877, 0.52025, 0.52216, 0.52439, 0.52686,\
        0.52950, 0.53231, 0.53530, 0.53849, 0.54188, 0.54545, 0.54923, 0.55320, 0.55740, 0.56183, 0.56650,\
        0.57144, 0.57672, 0.58243, 0.58862, 0.59528, 0.60231, 0.60939, 0.61623, 0.62263, 0.62845, 0.63362,\
        0.63810, 0.64185, 0.64487, 0.64715, 0.64870, 0.64955, 0.64975, 0.64936, 0.64845, 0.64710, 0.64544,\
        0.64364, 0.64194, 0.64064, 0.64005, 0.64053, 0.64239, 0.64589, 0.65109, 0.65765, 0.66483, 0.67142,\
        0.67595, 0.67756, 0.67698, 0.67530, 0.67311, 0.67065, 0.66809, 0.66551, 0.66300, 0.66060, 0.65836,\
        0.65628, 0.65436, 0.65254, 0.65078, 0.64899, 0.64707, 0.64492, 0.64244, 0.63951, 0.63604, 0.63200,\
        0.62740, 0.62240, 0.61723, 0.61203, 0.60685, 0.60234, 0.59909, 0.59630, 0.59395, 0.59208, 0.59096,\
        0.59065, 0.59051, 0.58966, 0.58726, 0.58339, 0.57867, 0.57350, 0.56801, 0.56225, 0.55627, 0.55007,\
        0.54370, 0.53718, 0.53055, 0.52386, 0.51717, 0.51055, 0.50410, 0.49792, 0.49203, 0.48639, 0.48099,\
        0.47581, 0.47081, 0.46597, 0.46123, 0.45656, 0.45191, 0.44723, 0.44248, 0.43761, 0.43258, 0.42736],\
        [0.51186, 0.51185, 0.51153, 0.51123, 0.51115, 0.51139, 0.51207, 0.51330, 0.51504, 0.51710, 0.51934,\
        0.52172, 0.52427, 0.52709, 0.53022, 0.53369, 0.53750, 0.54164, 0.54610, 0.55087, 0.55596, 0.56134,\
        0.56705, 0.57310, 0.57952, 0.58629, 0.59335, 0.60047, 0.60736, 0.61382, 0.61969, 0.62491, 0.62944,\
        0.63326, 0.63636, 0.63876, 0.64045, 0.64148, 0.64187, 0.64167, 0.64095, 0.63977, 0.63822, 0.63640,\
        0.63447, 0.63262, 0.63108, 0.63014, 0.63011, 0.63131, 0.63411, 0.63871, 0.64487, 0.65179, 0.65828,\
        0.66322, 0.66600, 0.66684, 0.66635, 0.66504, 0.66324, 0.66118, 0.65898, 0.65676, 0.65457, 0.65246,\
        0.65045, 0.64852, 0.64664, 0.64475, 0.64277, 0.64062, 0.63822, 0.63547, 0.63231, 0.62865, 0.62441,\
        0.61956, 0.61426, 0.60892, 0.60380, 0.59865, 0.59400, 0.59065, 0.58802, 0.58594, 0.58443, 0.58355,\
        0.58318, 0.58286, 0.58196, 0.57999, 0.57686, 0.57286, 0.56825, 0.56321, 0.55785, 0.55220, 0.54630,\
        0.54020, 0.53393, 0.52753, 0.52105, 0.51456, 0.50812, 0.50183, 0.49582, 0.49009, 0.48457, 0.47923,\
        0.47405, 0.46901, 0.46409, 0.45923, 0.45442, 0.44960, 0.44473, 0.43978, 0.43471, 0.42948, 0.42405],\
        [0.50789, 0.50743, 0.50660, 0.50588, 0.50549, 0.50538, 0.50571, 0.50675, 0.50840, 0.51032, 0.51233,\
        0.51441, 0.51667, 0.51927, 0.52233, 0.52588, 0.52991, 0.53440, 0.53933, 0.54466, 0.55038, 0.55645,\
        0.56287, 0.56963, 0.57671, 0.58403, 0.59147, 0.59875, 0.60556, 0.61166, 0.61703, 0.62166, 0.62556,\
        0.62873, 0.63120, 0.63299, 0.63413, 0.63465, 0.63460, 0.63404, 0.63303, 0.63162, 0.62991, 0.62796,\
        0.62591, 0.62388, 0.62208, 0.62072, 0.62006, 0.62042, 0.62226, 0.62614, 0.63201, 0.63892, 0.64557,\
        0.65098, 0.65465, 0.65661, 0.65719, 0.65678, 0.65569, 0.65418, 0.65240, 0.65049, 0.64852, 0.64655,\
        0.64461, 0.64269, 0.64076, 0.63877, 0.63663, 0.63428, 0.63163, 0.62864, 0.62527, 0.62145, 0.61710,\
        0.61203, 0.60631, 0.60067, 0.59545, 0.59041, 0.58587, 0.58230, 0.57960, 0.57761, 0.57627, 0.57550,\
        0.57509, 0.57470, 0.57388, 0.57227, 0.56977, 0.56646, 0.56250, 0.55804, 0.55317, 0.54795, 0.54244,\
        0.53668, 0.53072, 0.52460, 0.51839, 0.51214, 0.50592, 0.49985, 0.49400, 0.48839, 0.48294, 0.47764,\
        0.47245, 0.46737, 0.46235, 0.45737, 0.45240, 0.44741, 0.44235, 0.43720, 0.43192, 0.42647, 0.42083],\
        [0.50412, 0.50323, 0.50196, 0.50087, 0.50037, 0.49999, 0.50000, 0.50089, 0.50248, 0.50424, 0.50602,\
        0.50773, 0.50965, 0.51198, 0.51495, 0.51857, 0.52278, 0.52758, 0.53294, 0.53879, 0.54508, 0.55177,\
        0.55883, 0.56621, 0.57385, 0.58166, 0.58949, 0.59704, 0.60381, 0.60954, 0.61445, 0.61856, 0.62187,\
        0.62444, 0.62631, 0.62754, 0.62817, 0.62823, 0.62779, 0.62691, 0.62566, 0.62409, 0.62226, 0.62024,\
        0.61812, 0.61598, 0.61396, 0.61224, 0.61102, 0.61052, 0.61118, 0.61423, 0.62012, 0.62727, 0.63415,\
        0.63991, 0.64418, 0.64692, 0.64832, 0.64866, 0.64822, 0.64722, 0.64585, 0.64424, 0.64248, 0.64065,\
        0.63878, 0.63688, 0.63492, 0.63284, 0.63057, 0.62802, 0.62514, 0.62189, 0.61828, 0.61430, 0.60995,\
        0.60491, 0.59871, 0.59270, 0.58714, 0.58201, 0.57748, 0.57379, 0.57099, 0.56901, 0.56773, 0.56701,\
        0.56664, 0.56629, 0.56563, 0.56440, 0.56246, 0.55980, 0.55650, 0.55265, 0.54834, 0.54362, 0.53856,\
        0.53320, 0.52759, 0.52180, 0.51588, 0.50990, 0.50394, 0.49809, 0.49241, 0.48689, 0.48150, 0.47621,\
        0.47100, 0.46585, 0.46073, 0.45562, 0.45049, 0.44532, 0.44007, 0.43471, 0.42921, 0.42355, 0.41769],\
        [0.50039, 0.49917, 0.49778, 0.49658, 0.49585, 0.49539, 0.49536, 0.49605, 0.49736, 0.49889, 0.50051,\
        0.50171, 0.50343, 0.50533, 0.50827, 0.51192, 0.51623, 0.52127, 0.52700, 0.53330, 0.54008, 0.54730,\
        0.55488, 0.56275, 0.57085, 0.57903, 0.58711, 0.59479, 0.60150, 0.60713, 0.61180, 0.61547, 0.61826,\
        0.62027, 0.62163, 0.62238, 0.62256, 0.62223, 0.62146, 0.62032, 0.61889, 0.61722, 0.61535, 0.61334,\
        0.61124, 0.60909, 0.60699, 0.60508, 0.60358, 0.60262, 0.60213, 0.60421, 0.61062, 0.61790, 0.62472,\
        0.63051, 0.63501, 0.63818, 0.64011, 0.64099, 0.64105, 0.64048, 0.63945, 0.63809, 0.63651, 0.63480,\
        0.63299, 0.63110, 0.62911, 0.62697, 0.62460, 0.62188, 0.61874, 0.61519, 0.61129, 0.60701, 0.60245,\
        0.59753, 0.59122, 0.58481, 0.57887, 0.57351, 0.56886, 0.56505, 0.56217, 0.56016, 0.55893, 0.55829,\
        0.55801, 0.55781, 0.55741, 0.55658, 0.55516, 0.55311, 0.55045, 0.54722, 0.54349, 0.53930, 0.53472,\
        0.52979, 0.52458, 0.51913, 0.51353, 0.50784, 0.50215, 0.49651, 0.49098, 0.48555, 0.48021, 0.47492,\
        0.46967, 0.46444, 0.45921, 0.45396, 0.44867, 0.44331, 0.43785, 0.43228, 0.42657, 0.42068, 0.41460],\
        [0.49585, 0.49504, 0.49384, 0.49272, 0.49191, 0.49144, 0.49141, 0.49192, 0.49292, 0.49419, 0.49553,\
        0.49669, 0.49811, 0.49963, 0.50265, 0.50610, 0.51032, 0.51558, 0.52160, 0.52824, 0.53540, 0.54302,\
        0.55099, 0.55924, 0.56765, 0.57606, 0.58426, 0.59193, 0.59869, 0.60447, 0.60901, 0.61231, 0.61461,\
        0.61615, 0.61708, 0.61746, 0.61731, 0.61666, 0.61561, 0.61427, 0.61275, 0.61105, 0.60922, 0.60731,\
        0.60536, 0.60334, 0.60132, 0.59949, 0.59804, 0.59718, 0.59692, 0.59934, 0.60506, 0.61144, 0.61758,\
        0.62297, 0.62734, 0.63060, 0.63277, 0.63398, 0.63436, 0.63409, 0.63330, 0.63213, 0.63068, 0.62904,\
        0.62726, 0.62537, 0.62336, 0.62118, 0.61874, 0.61590, 0.61248, 0.60853, 0.60433, 0.59967, 0.59464,\
        0.58922, 0.58307, 0.57662, 0.57047, 0.56488, 0.56005, 0.55612, 0.55317, 0.55116, 0.55000, 0.54950,\
        0.54940, 0.54945, 0.54937, 0.54895, 0.54804, 0.54657, 0.54452, 0.54190, 0.53874, 0.53510, 0.53102,\
        0.52654, 0.52172, 0.51664, 0.51135, 0.50595, 0.50050, 0.49508, 0.48970, 0.48436, 0.47905, 0.47375,\
        0.46844, 0.46312, 0.45777, 0.45237, 0.44690, 0.44135, 0.43569, 0.42990, 0.42396, 0.41785, 0.41154],\
        [0.49094, 0.49063, 0.48984, 0.48898, 0.48829, 0.48789, 0.48787, 0.48826, 0.48903, 0.49005, 0.49117,\
        0.49232, 0.49363, 0.49543, 0.49812, 0.50100, 0.50522, 0.51067, 0.51683, 0.52366, 0.53107, 0.53895,\
        0.54719, 0.55568, 0.56427, 0.57281, 0.58109, 0.58884, 0.59580, 0.60169, 0.60607, 0.60894, 0.61078,\
        0.61196, 0.61261, 0.61277, 0.61242, 0.61156, 0.61027, 0.60880, 0.60728, 0.60565, 0.60390, 0.60216,\
        0.60045, 0.59868, 0.59692, 0.59541, 0.59436, 0.59402, 0.59487, 0.59780, 0.60230, 0.60735, 0.61247,\
        0.61716, 0.62113, 0.62423, 0.62641, 0.62773, 0.62827, 0.62816, 0.62752, 0.62644, 0.62505, 0.62343,\
        0.62164, 0.61973, 0.61767, 0.61544, 0.61296, 0.61010, 0.60650, 0.60194, 0.59736, 0.59237, 0.58690,\
        0.58102, 0.57472, 0.56823, 0.56193, 0.55616, 0.55115, 0.54709, 0.54408, 0.54212, 0.54110, 0.54081,\
        0.54098, 0.54135, 0.54164, 0.54166, 0.54126, 0.54033, 0.53885, 0.53681, 0.53422, 0.53110, 0.52751,\
        0.52349, 0.51907, 0.51434, 0.50936, 0.50423, 0.49902, 0.49380, 0.48856, 0.48329, 0.47800, 0.47267,\
        0.46729, 0.46186, 0.45638, 0.45081, 0.44516, 0.43941, 0.43354, 0.42753, 0.42137, 0.41502, 0.40848],\
        [0.48596, 0.48609, 0.48576, 0.48526, 0.48482, 0.48458, 0.48462, 0.48498, 0.48563, 0.48650, 0.48752,\
        0.48868, 0.49007, 0.49191, 0.49429, 0.49723, 0.50151, 0.50672, 0.51273, 0.51960, 0.52713, 0.53514,\
        0.54351, 0.55209, 0.56075, 0.56932, 0.57761, 0.58539, 0.59244, 0.59844, 0.60269, 0.60512, 0.60661,\
        0.60761, 0.60818, 0.60829, 0.60791, 0.60700, 0.60551, 0.60391, 0.60254, 0.60105, 0.59944, 0.59789,\
        0.59642, 0.59497, 0.59362, 0.59257, 0.59205, 0.59235, 0.59380, 0.59666, 0.60050, 0.60464, 0.60882,\
        0.61276, 0.61620, 0.61898, 0.62100, 0.62227, 0.62284, 0.62277, 0.62217, 0.62112, 0.61971, 0.61804,\
        0.61618, 0.61420, 0.61208, 0.60973, 0.60717, 0.60433, 0.60087, 0.59593, 0.59067, 0.58520, 0.57933,\
        0.57309, 0.56656, 0.55992, 0.55347, 0.54750, 0.54230, 0.53809, 0.53504, 0.53318, 0.53238, 0.53240,\
        0.53293, 0.53367, 0.53438, 0.53484, 0.53492, 0.53451, 0.53356, 0.53205, 0.52999, 0.52739, 0.52427,\
        0.52069, 0.51667, 0.51228, 0.50759, 0.50270, 0.49771, 0.49267, 0.48754, 0.48233, 0.47703, 0.47165,\
        0.46619, 0.46065, 0.45501, 0.44927, 0.44343, 0.43748, 0.43139, 0.42515, 0.41876, 0.41218, 0.40541],\
        [0.48099, 0.48153, 0.48164, 0.48155, 0.48142, 0.48141, 0.48159, 0.48201, 0.48265, 0.48350, 0.48452,\
        0.48573, 0.48722, 0.48910, 0.49152, 0.49466, 0.49878, 0.50339, 0.50922, 0.51608, 0.52360, 0.53161,\
        0.53997, 0.54854, 0.55715, 0.56563, 0.57380, 0.58146, 0.58836, 0.59416, 0.59830, 0.60061, 0.60206,\
        0.60312, 0.60381, 0.60399, 0.60375, 0.60309, 0.60157, 0.59972, 0.59855, 0.59723, 0.59582, 0.59448,\
        0.59324, 0.59212, 0.59120, 0.59064, 0.59063, 0.59139, 0.59311, 0.59580, 0.59915, 0.60268, 0.60612,\
        0.60939, 0.61231, 0.61471, 0.61648, 0.61760, 0.61808, 0.61797, 0.61732, 0.61621, 0.61472, 0.61293,\
        0.61094, 0.60886, 0.60665, 0.60414, 0.60131, 0.59830, 0.59457, 0.58972, 0.58418, 0.57830, 0.57207,\
        0.56554, 0.55877, 0.55195, 0.54531, 0.53913, 0.53370, 0.52932, 0.52625, 0.52454, 0.52406, 0.52448,\
        0.52543, 0.52660, 0.52772, 0.52861, 0.52912, 0.52918, 0.52871, 0.52770, 0.52613, 0.52400, 0.52135,\
        0.51818, 0.51454, 0.51048, 0.50606, 0.50140, 0.49661, 0.49170, 0.48666, 0.48146, 0.47614, 0.47069,\
        0.46512, 0.45943, 0.45363, 0.44772, 0.44168, 0.43551, 0.42920, 0.42274, 0.41611, 0.40930, 0.40229],\
        [0.47606, 0.47699, 0.47753, 0.47785, 0.47808, 0.47835, 0.47874, 0.47930, 0.48004, 0.48097, 0.48207,\
        0.48339, 0.48499, 0.48696, 0.48944, 0.49253, 0.49621, 0.50053, 0.50648, 0.51316, 0.52051, 0.52839,\
        0.53663, 0.54506, 0.55351, 0.56181, 0.56975, 0.57714, 0.58373, 0.58919, 0.59320, 0.59577, 0.59745,\
        0.59868, 0.59956, 0.59987, 0.59980, 0.59949, 0.59843, 0.59665, 0.59530, 0.59407, 0.59288, 0.59176,\
        0.59078, 0.58999, 0.58946, 0.58932, 0.58972, 0.59079, 0.59262, 0.59514, 0.59813, 0.60114, 0.60400,\
        0.60678, 0.60925, 0.61127, 0.61275, 0.61365, 0.61398, 0.61376, 0.61301, 0.61180, 0.61018, 0.60820,\
        0.60601, 0.60376, 0.60147, 0.59884, 0.59572, 0.59232, 0.58827, 0.58338, 0.57777, 0.57169, 0.56523,\
        0.55848, 0.55154, 0.54455, 0.53773, 0.53132, 0.52564, 0.52105, 0.51793, 0.51645, 0.51641, 0.51732,\
        0.51873, 0.52030, 0.52179, 0.52305, 0.52395, 0.52441, 0.52437, 0.52379, 0.52267, 0.52098, 0.51875,\
        0.51598, 0.51271, 0.50897, 0.50482, 0.50034, 0.49572, 0.49091, 0.48589, 0.48067, 0.47528, 0.46973,\
        0.46403, 0.45819, 0.45222, 0.44611, 0.43987, 0.43348, 0.42695, 0.42025, 0.41339, 0.40634, 0.39910],\
        [0.47118, 0.47249, 0.47345, 0.47417, 0.47478, 0.47538, 0.47604, 0.47682, 0.47774, 0.47882, 0.48009,\
        0.48156, 0.48330, 0.48538, 0.48790, 0.49096, 0.49468, 0.49934, 0.50458, 0.51078, 0.51783, 0.52547,\
        0.53350, 0.54172, 0.54993, 0.55795, 0.56559, 0.57265, 0.57889, 0.58410, 0.58810, 0.59095, 0.59294,\
        0.59438, 0.59537, 0.59589, 0.59603, 0.59583, 0.59505, 0.59379, 0.59255, 0.59144, 0.59043, 0.58955,\
        0.58885, 0.58836, 0.58818, 0.58840, 0.58910, 0.59038, 0.59223, 0.59457, 0.59717, 0.59973, 0.60231,\
        0.60477, 0.60686, 0.60851, 0.60969, 0.61037, 0.61051, 0.61013, 0.60926, 0.60793, 0.60617, 0.60401,\
        0.60153, 0.59908, 0.59659, 0.59377, 0.59046, 0.58669, 0.58235, 0.57732, 0.57165, 0.56547, 0.55890,\
        0.55204, 0.54500, 0.53792, 0.53099, 0.52441, 0.51849, 0.51363, 0.51041, 0.50926, 0.50979, 0.51123,\
        0.51304, 0.51492, 0.51670, 0.51825, 0.51945, 0.52023, 0.52055, 0.52035, 0.51961, 0.51832, 0.51648,\
        0.51408, 0.51116, 0.50774, 0.50386, 0.49960, 0.49505, 0.49024, 0.48519, 0.47991, 0.47442, 0.46874,\
        0.46289, 0.45689, 0.45073, 0.44442, 0.43796, 0.43136, 0.42460, 0.41767, 0.41057, 0.40329, 0.39581],\
        [0.46634, 0.46803, 0.46939, 0.47052, 0.47151, 0.47247, 0.47345, 0.47451, 0.47568, 0.47699, 0.47847,\
        0.48014, 0.48205, 0.48425, 0.48682, 0.48986, 0.49347, 0.49774, 0.50272, 0.50868, 0.51548, 0.52287,\
        0.53064, 0.53857, 0.54648, 0.55417, 0.56146, 0.56818, 0.57413, 0.57915, 0.58316, 0.58621, 0.58847,\
        0.59013, 0.59129, 0.59201, 0.59233, 0.59226, 0.59176, 0.59093, 0.59000, 0.58911, 0.58833, 0.58770,\
        0.58727, 0.58707, 0.58719, 0.58769, 0.58862, 0.59003, 0.59190, 0.59412, 0.59653, 0.59893, 0.60129,\
        0.60333, 0.60499, 0.60629, 0.60720, 0.60766, 0.60762, 0.60708, 0.60605, 0.60460, 0.60274, 0.60044,\
        0.59780, 0.59507, 0.59225, 0.58912, 0.58555, 0.58150, 0.57690, 0.57172, 0.56598, 0.55977, 0.55318,\
        0.54631, 0.53929, 0.53223, 0.52531, 0.51872, 0.51270, 0.50763, 0.50420, 0.50351, 0.50473, 0.50654,\
        0.50856, 0.51058, 0.51250, 0.51421, 0.51562, 0.51665, 0.51724, 0.51736, 0.51695, 0.51601, 0.51450,\
        0.51244, 0.50982, 0.50669, 0.50309, 0.49900, 0.49448, 0.48963, 0.48450, 0.47912, 0.47350, 0.46768,\
        0.46166, 0.45547, 0.44912, 0.44260, 0.43593, 0.42911, 0.42212, 0.41496, 0.40763, 0.40011, 0.39240],\
        [0.46153, 0.46358, 0.46534, 0.46687, 0.46827, 0.46960, 0.47093, 0.47232, 0.47380, 0.47539, 0.47714,\
        0.47905, 0.48117, 0.48353, 0.48616, 0.48914, 0.49254, 0.49646, 0.50115, 0.50693, 0.51348, 0.52061,\
        0.52808, 0.53569, 0.54324, 0.55056, 0.55749, 0.56387, 0.56955, 0.57443, 0.57844, 0.58162, 0.58408,\
        0.58594, 0.58729, 0.58819, 0.58870, 0.58884, 0.58862, 0.58814, 0.58754, 0.58695, 0.58643, 0.58607,\
        0.58590, 0.58597, 0.58635, 0.58707, 0.58818, 0.58968, 0.59156, 0.59373, 0.59608, 0.59849, 0.60062,\
        0.60218, 0.60343, 0.60444, 0.60515, 0.60546, 0.60528, 0.60457, 0.60337, 0.60178, 0.59980, 0.59738,\
        0.59460, 0.59161, 0.58842, 0.58494, 0.58108, 0.57677, 0.57197, 0.56666, 0.56087, 0.55467, 0.54812,\
        0.54135, 0.53445, 0.52756, 0.52084, 0.51448, 0.50871, 0.50385, 0.50048, 0.50043, 0.50176, 0.50344,\
        0.50532, 0.50727, 0.50916, 0.51091, 0.51241, 0.51360, 0.51440, 0.51476, 0.51464, 0.51399, 0.51278,\
        0.51100, 0.50864, 0.50572, 0.50229, 0.49832, 0.49382, 0.48894, 0.48373, 0.47822, 0.47247, 0.46648,\
        0.46029, 0.45391, 0.44735, 0.44063, 0.43374, 0.42669, 0.41947, 0.41209, 0.40453, 0.39678, 0.38884],\
        [0.45671, 0.45914, 0.46128, 0.46322, 0.46502, 0.46674, 0.46845, 0.47020, 0.47202, 0.47394, 0.47600,\
        0.47821, 0.48059, 0.48315, 0.48590, 0.48886, 0.49210, 0.49580, 0.50028, 0.50563, 0.51190, 0.51874,\
        0.52588, 0.53311, 0.54026, 0.54719, 0.55375, 0.55980, 0.56523, 0.56995, 0.57394, 0.57720, 0.57980,\
        0.58183, 0.58337, 0.58446, 0.58517, 0.58553, 0.58559, 0.58544, 0.58516, 0.58486, 0.58463, 0.58453,\
        0.58462, 0.58493, 0.58553, 0.58643, 0.58767, 0.58924, 0.59112, 0.59322, 0.59547, 0.59769, 0.59960,\
        0.60091, 0.60196, 0.60283, 0.60345, 0.60371, 0.60346, 0.60262, 0.60121, 0.59943, 0.59730, 0.59472,\
        0.59176, 0.58851, 0.58501, 0.58121, 0.57706, 0.57252, 0.56754, 0.56214, 0.55634, 0.55018, 0.54375,\
        0.53715, 0.53048, 0.52389, 0.51754, 0.51164, 0.50646, 0.50241, 0.50017, 0.49999, 0.50057, 0.50167,\
        0.50314, 0.50482, 0.50656, 0.50824, 0.50976, 0.51102, 0.51196, 0.51251, 0.51261, 0.51221, 0.51125,\
        0.50970, 0.50753, 0.50475, 0.50139, 0.49747, 0.49302, 0.48810, 0.48280, 0.47717, 0.47126, 0.46510,\
        0.45872, 0.45214, 0.44538, 0.43845, 0.43135, 0.42408, 0.41664, 0.40903, 0.40124, 0.39327, 0.38510],\
        [0.45188, 0.45467, 0.45720, 0.45953, 0.46173, 0.46386, 0.46596, 0.46808, 0.47027, 0.47256, 0.47497,\
        0.47751, 0.48020, 0.48304, 0.48600, 0.48906, 0.49222, 0.49560, 0.49957, 0.50478, 0.51083, 0.51733,\
        0.52406, 0.53086, 0.53758, 0.54408, 0.55026, 0.55598, 0.56116, 0.56573, 0.56966, 0.57295, 0.57566,\
        0.57783, 0.57953, 0.58081, 0.58172, 0.58232, 0.58265, 0.58278, 0.58281, 0.58281, 0.58285, 0.58301,\
        0.58334, 0.58387, 0.58465, 0.58570, 0.58704, 0.58865, 0.59049, 0.59251, 0.59459, 0.59657, 0.59824,\
        0.59950, 0.60052, 0.60138, 0.60202, 0.60235, 0.60215, 0.60125, 0.59968, 0.59766, 0.59525, 0.59242,\
        0.58922, 0.58571, 0.58192, 0.57784, 0.57345, 0.56870, 0.56360, 0.55814, 0.55236, 0.54629, 0.54003,\
        0.53366, 0.52730, 0.52109, 0.51521, 0.50988, 0.50538, 0.50207, 0.50030, 0.49998, 0.50003, 0.50060,\
        0.50165, 0.50301, 0.50453, 0.50607, 0.50753, 0.50880, 0.50981, 0.51049, 0.51076, 0.51057, 0.50984,\
        0.50850, 0.50648, 0.50377, 0.50043, 0.49653, 0.49206, 0.48708, 0.48166, 0.47589, 0.46981, 0.46347,\
        0.45690, 0.45013, 0.44317, 0.43603, 0.42872, 0.42123, 0.41358, 0.40575, 0.39775, 0.38955, 0.38117],\
        [0.44699, 0.45014, 0.45305, 0.45578, 0.45838, 0.46090, 0.46340, 0.46591, 0.46849, 0.47115, 0.47393,\
        0.47685, 0.47990, 0.48309, 0.48638, 0.48971, 0.49299, 0.49622, 0.49991, 0.50474, 0.51036, 0.51638,\
        0.52261, 0.52891, 0.53516, 0.54122, 0.54700, 0.55240, 0.55732, 0.56173, 0.56558, 0.56888, 0.57165,\
        0.57393, 0.57578, 0.57723, 0.57834, 0.57916, 0.57974, 0.58015, 0.58046, 0.58073, 0.58104, 0.58144,\
        0.58199, 0.58271, 0.58365, 0.58482, 0.58623, 0.58786, 0.58966, 0.59157, 0.59350, 0.59532, 0.59689,\
        0.59816, 0.59921, 0.60007, 0.60080, 0.60135, 0.60127, 0.60032, 0.59859, 0.59627, 0.59351, 0.59037,\
        0.58690, 0.58312, 0.57908, 0.57476, 0.57016, 0.56527, 0.56008, 0.55460, 0.54887, 0.54294, 0.53687,\
        0.53077, 0.52475, 0.51896, 0.51356, 0.50877, 0.50482, 0.50195, 0.50029, 0.49964, 0.49948, 0.49977,\
        0.50051, 0.50158, 0.50286, 0.50423, 0.50558, 0.50681, 0.50784, 0.50859, 0.50900, 0.50898, 0.50845,\
        0.50733, 0.50548, 0.50280, 0.49943, 0.49550, 0.49094, 0.48582, 0.48025, 0.47431, 0.46805, 0.46153,\
        0.45478, 0.44782, 0.44067, 0.43333, 0.42582, 0.41813, 0.41027, 0.40223, 0.39401, 0.38561, 0.37701],\
        [0.44203, 0.44553, 0.44881, 0.45192, 0.45491, 0.45783, 0.46072, 0.46362, 0.46658, 0.46963, 0.47279,\
        0.47608, 0.47953, 0.48311, 0.48682, 0.49059, 0.49429, 0.49769, 0.50110, 0.50542, 0.51039, 0.51578,\
        0.52143, 0.52719, 0.53294, 0.53856, 0.54395, 0.54901, 0.55369, 0.55792, 0.56167, 0.56495, 0.56776,\
        0.57013, 0.57210, 0.57371, 0.57501, 0.57603, 0.57685, 0.57751, 0.57807, 0.57859, 0.57914, 0.57976,\
        0.58051, 0.58141, 0.58248, 0.58375, 0.58521, 0.58684, 0.58860, 0.59043, 0.59225, 0.59397, 0.59551,\
        0.59684, 0.59797, 0.59893, 0.59983, 0.60056, 0.60070, 0.59961, 0.59760, 0.59494, 0.59184, 0.58840,\
        0.58466, 0.58065, 0.57639, 0.57188, 0.56713, 0.56214, 0.55690, 0.55144, 0.54580, 0.54002, 0.53417,\
        0.52836, 0.52269, 0.51730, 0.51234, 0.50799, 0.50443, 0.50179, 0.50010, 0.49921, 0.49888, 0.49898,\
        0.49949, 0.50032, 0.50139, 0.50258, 0.50379, 0.50494, 0.50594, 0.50671, 0.50718, 0.50728, 0.50693,\
        0.50603, 0.50444, 0.50193, 0.49846, 0.49436, 0.48957, 0.48424, 0.47848, 0.47236, 0.46593, 0.45924,\
        0.45232, 0.44518, 0.43784, 0.43032, 0.42261, 0.41473, 0.40667, 0.39844, 0.39002, 0.38141, 0.37261],\
        [0.43696, 0.44081, 0.44445, 0.44793, 0.45130, 0.45459, 0.45786, 0.46114, 0.46447, 0.46789, 0.47142,\
        0.47509, 0.47890, 0.48287, 0.48699, 0.49123, 0.49551, 0.49958, 0.50268, 0.50628, 0.51054, 0.51529,\
        0.52035, 0.52557, 0.53083, 0.53601, 0.54102, 0.54577, 0.55020, 0.55425, 0.55790, 0.56113, 0.56396,\
        0.56640, 0.56847, 0.57022, 0.57169, 0.57291, 0.57393, 0.57481, 0.57560, 0.57636, 0.57712, 0.57794,\
        0.57886, 0.57991, 0.58110, 0.58245, 0.58395, 0.58558, 0.58730, 0.58907, 0.59082, 0.59250, 0.59403,\
        0.59541, 0.59665, 0.59777, 0.59879, 0.59963, 0.60012, 0.59873, 0.59632, 0.59337, 0.59001, 0.58633,\
        0.58238, 0.57818, 0.57376, 0.56912, 0.56427, 0.55922, 0.55398, 0.54858, 0.54304, 0.53743, 0.53182,\
        0.52630, 0.52096, 0.51593, 0.51134, 0.50734, 0.50404, 0.50152, 0.49979, 0.49874, 0.49823, 0.49817,\
        0.49848, 0.49911, 0.49997, 0.50097, 0.50203, 0.50305, 0.50397, 0.50470, 0.50517, 0.50532, 0.50507,\
        0.50434, 0.50302, 0.50094, 0.49740, 0.49284, 0.48774, 0.48220, 0.47626, 0.46997, 0.46339, 0.45654,\
        0.44946, 0.44216, 0.43465, 0.42696, 0.41908, 0.41101, 0.40277, 0.39435, 0.38574, 0.37694, 0.36794]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=40 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax40vt5
    zgrid_H_Hmax_vmax40vt5=[[0.26602, 0.27182, 0.27733, 0.28256, 0.28754, 0.29227, 0.29677, 0.30107, 0.30517, 0.30910, 0.31288,\
        0.31651, 0.32003, 0.32343, 0.32675, 0.33000, 0.33319, 0.33634, 0.33946, 0.34257, 0.34567, 0.34879,\
        0.35193, 0.35510, 0.35831, 0.36158, 0.36491, 0.36833, 0.37182, 0.37543, 0.37915, 0.38300, 0.38702,\
        0.39118, 0.39546, 0.39957, 0.40309, 0.40658, 0.41031, 0.41432, 0.41858, 0.42303, 0.42760, 0.43223,\
        0.43686, 0.44143, 0.44590, 0.45022, 0.45434, 0.45823, 0.46185, 0.46516, 0.46816, 0.47086, 0.47332,\
        0.47561, 0.47774, 0.47977, 0.48183, 0.48396, 0.48596, 0.48774, 0.48970, 0.49192, 0.49428, 0.49684,\
        0.49964, 0.50248, 0.50525, 0.50788, 0.51031, 0.51251, 0.51446, 0.51613, 0.51752, 0.51861, 0.51940,\
        0.51988, 0.52007, 0.51997, 0.51958, 0.51892, 0.51800, 0.51684, 0.51545, 0.51384, 0.51202, 0.51001,\
        0.50779, 0.50537, 0.50274, 0.49988, 0.49675, 0.49329, 0.48935, 0.48505, 0.48068, 0.47636, 0.47210,\
        0.46787, 0.46365, 0.45939, 0.45509, 0.45072, 0.44627, 0.44169, 0.43695, 0.43201, 0.42684, 0.42140,\
        0.41569, 0.40967, 0.40336, 0.39675, 0.38984, 0.38263, 0.37513, 0.36735, 0.35927, 0.35091, 0.34226],\
        [0.27404, 0.27964, 0.28495, 0.28999, 0.29475, 0.29927, 0.30356, 0.30764, 0.31152, 0.31523, 0.31878,\
        0.32219, 0.32549, 0.32867, 0.33178, 0.33481, 0.33779, 0.34073, 0.34364, 0.34655, 0.34945, 0.35237,\
        0.35531, 0.35828, 0.36130, 0.36437, 0.36750, 0.37070, 0.37398, 0.37735, 0.38083, 0.38443, 0.38817,\
        0.39207, 0.39611, 0.40009, 0.40377, 0.40760, 0.41174, 0.41621, 0.42093, 0.42585, 0.43089, 0.43598,\
        0.44104, 0.44603, 0.45089, 0.45557, 0.46005, 0.46427, 0.46820, 0.47177, 0.47497, 0.47781, 0.48039,\
        0.48281, 0.48506, 0.48713, 0.48926, 0.49170, 0.49408, 0.49566, 0.49778, 0.50024, 0.50279, 0.50549,\
        0.50839, 0.51131, 0.51413, 0.51679, 0.51923, 0.52142, 0.52334, 0.52497, 0.52631, 0.52733, 0.52805,\
        0.52844, 0.52853, 0.52832, 0.52781, 0.52702, 0.52598, 0.52470, 0.52319, 0.52147, 0.51955, 0.51744,\
        0.51514, 0.51265, 0.50995, 0.50703, 0.50387, 0.50041, 0.49630, 0.49181, 0.48740, 0.48312, 0.47897,\
        0.47485, 0.47073, 0.46658, 0.46240, 0.45816, 0.45386, 0.44944, 0.44486, 0.44008, 0.43504, 0.42973,\
        0.42411, 0.41819, 0.41195, 0.40541, 0.39856, 0.39142, 0.38398, 0.37625, 0.36824, 0.35995, 0.35137],\
        [0.28183, 0.28724, 0.29236, 0.29718, 0.30174, 0.30604, 0.31011, 0.31396, 0.31762, 0.32110, 0.32443,\
        0.32761, 0.33068, 0.33365, 0.33653, 0.33934, 0.34211, 0.34483, 0.34754, 0.35024, 0.35293, 0.35565,\
        0.35839, 0.36116, 0.36397, 0.36684, 0.36976, 0.37274, 0.37579, 0.37891, 0.38210, 0.38537, 0.38875,\
        0.39224, 0.39582, 0.39946, 0.40353, 0.40790, 0.41260, 0.41763, 0.42292, 0.42838, 0.43394, 0.43951,\
        0.44504, 0.45045, 0.45572, 0.46079, 0.46563, 0.47021, 0.47447, 0.47836, 0.48179, 0.48476, 0.48741,\
        0.49001, 0.49253, 0.49457, 0.49672, 0.49926, 0.50230, 0.50401, 0.50630, 0.50895, 0.51170, 0.51454,\
        0.51747, 0.52039, 0.52320, 0.52583, 0.52823, 0.53037, 0.53223, 0.53380, 0.53505, 0.53598, 0.53659,\
        0.53688, 0.53684, 0.53650, 0.53585, 0.53492, 0.53373, 0.53230, 0.53065, 0.52880, 0.52677, 0.52455,\
        0.52214, 0.51955, 0.51676, 0.51374, 0.51046, 0.50681, 0.50259, 0.49812, 0.49374, 0.48959, 0.48558,\
        0.48159, 0.47756, 0.47350, 0.46942, 0.46532, 0.46117, 0.45692, 0.45252, 0.44789, 0.44300, 0.43780,\
        0.43229, 0.42644, 0.42027, 0.41378, 0.40699, 0.39990, 0.39252, 0.38486, 0.37691, 0.36868, 0.36017],\
        [0.28941, 0.29463, 0.29954, 0.30416, 0.30850, 0.31258, 0.31642, 0.32005, 0.32348, 0.32673, 0.32982,\
        0.33278, 0.33562, 0.33836, 0.34102, 0.34361, 0.34616, 0.34867, 0.35117, 0.35366, 0.35615, 0.35866,\
        0.36120, 0.36377, 0.36638, 0.36904, 0.37176, 0.37452, 0.37734, 0.38020, 0.38310, 0.38604, 0.38903,\
        0.39211, 0.39537, 0.39902, 0.40313, 0.40789, 0.41316, 0.41881, 0.42470, 0.43073, 0.43683, 0.44291,\
        0.44890, 0.45475, 0.46042, 0.46587, 0.47107, 0.47601, 0.48065, 0.48489, 0.48861, 0.49174, 0.49440,\
        0.49704, 0.50019, 0.50228, 0.50461, 0.50731, 0.51023, 0.51274, 0.51528, 0.51804, 0.52091, 0.52383,\
        0.52677, 0.52965, 0.53240, 0.53496, 0.53729, 0.53934, 0.54111, 0.54257, 0.54371, 0.54454, 0.54502,\
        0.54518, 0.54500, 0.54450, 0.54369, 0.54259, 0.54123, 0.53964, 0.53784, 0.53584, 0.53367, 0.53133,\
        0.52881, 0.52611, 0.52321, 0.52007, 0.51664, 0.51285, 0.50866, 0.50426, 0.49989, 0.49592, 0.49205,\
        0.48814, 0.48417, 0.48017, 0.47618, 0.47220, 0.46822, 0.46415, 0.45994, 0.45548, 0.45073, 0.44564,\
        0.44020, 0.43442, 0.42830, 0.42186, 0.41512, 0.40808, 0.40076, 0.39315, 0.38527, 0.37711, 0.36867],\
        [0.29678, 0.30180, 0.30651, 0.31091, 0.31503, 0.31889, 0.32251, 0.32590, 0.32910, 0.33212, 0.33498,\
        0.33770, 0.34031, 0.34283, 0.34526, 0.34763, 0.34996, 0.35226, 0.35455, 0.35683, 0.35912, 0.36143,\
        0.36377, 0.36614, 0.36856, 0.37102, 0.37354, 0.37610, 0.37871, 0.38134, 0.38400, 0.38666, 0.38932,\
        0.39204, 0.39494, 0.39833, 0.40263, 0.40787, 0.41373, 0.41997, 0.42645, 0.43304, 0.43967, 0.44624,\
        0.45269, 0.45897, 0.46502, 0.47082, 0.47636, 0.48163, 0.48663, 0.49128, 0.49540, 0.49872, 0.50155,\
        0.50429, 0.50742, 0.51020, 0.51289, 0.51576, 0.51873, 0.52161, 0.52445, 0.52735, 0.53031, 0.53327,\
        0.53620, 0.53902, 0.54169, 0.54414, 0.54635, 0.54829, 0.54993, 0.55126, 0.55228, 0.55296, 0.55331,\
        0.55332, 0.55298, 0.55230, 0.55131, 0.55002, 0.54848, 0.54671, 0.54473, 0.54259, 0.54027, 0.53780,\
        0.53517, 0.53236, 0.52935, 0.52611, 0.52260, 0.51878, 0.51467, 0.51041, 0.50621, 0.50228, 0.49844,\
        0.49454, 0.49057, 0.48659, 0.48267, 0.47881, 0.47500, 0.47115, 0.46713, 0.46285, 0.45823, 0.45324,\
        0.44786, 0.44211, 0.43603, 0.42963, 0.42292, 0.41594, 0.40867, 0.40113, 0.39331, 0.38523, 0.37686],\
        [0.30394, 0.30877, 0.31327, 0.31746, 0.32136, 0.32499, 0.32837, 0.33153, 0.33449, 0.33727, 0.33990,\
        0.34239, 0.34477, 0.34705, 0.34926, 0.35142, 0.35353, 0.35562, 0.35770, 0.35977, 0.36186, 0.36397,\
        0.36612, 0.36830, 0.37053, 0.37282, 0.37516, 0.37754, 0.37998, 0.38245, 0.38493, 0.38741, 0.38986,\
        0.39234, 0.39495, 0.39810, 0.40254, 0.40822, 0.41459, 0.42135, 0.42834, 0.43543, 0.44254, 0.44958,\
        0.45647, 0.46315, 0.46955, 0.47566, 0.48147, 0.48704, 0.49232, 0.49730, 0.50183, 0.50550, 0.50870,\
        0.51181, 0.51502, 0.51817, 0.52125, 0.52438, 0.52752, 0.53063, 0.53370, 0.53675, 0.53978, 0.54277,\
        0.54566, 0.54842, 0.55098, 0.55330, 0.55537, 0.55715, 0.55864, 0.55983, 0.56070, 0.56123, 0.56143,\
        0.56128, 0.56077, 0.55990, 0.55870, 0.55721, 0.55545, 0.55349, 0.55134, 0.54903, 0.54658, 0.54398,\
        0.54125, 0.53835, 0.53526, 0.53196, 0.52842, 0.52463, 0.52063, 0.51653, 0.51247, 0.50853, 0.50465,\
        0.50076, 0.49675, 0.49277, 0.48890, 0.48516, 0.48154, 0.47792, 0.47413, 0.47002, 0.46551, 0.46058,\
        0.45523, 0.44951, 0.44344, 0.43706, 0.43039, 0.42345, 0.41625, 0.40877, 0.40103, 0.39302, 0.38474],\
        [0.31091, 0.31554, 0.31982, 0.32380, 0.32747, 0.33087, 0.33402, 0.33694, 0.33966, 0.34220, 0.34459,\
        0.34685, 0.34899, 0.35105, 0.35304, 0.35498, 0.35688, 0.35876, 0.36063, 0.36251, 0.36440, 0.36632,\
        0.36828, 0.37029, 0.37234, 0.37446, 0.37664, 0.37889, 0.38120, 0.38356, 0.38597, 0.38839, 0.39082,\
        0.39327, 0.39583, 0.39880, 0.40333, 0.40931, 0.41599, 0.42310, 0.43047, 0.43797, 0.44550, 0.45296,\
        0.46026, 0.46732, 0.47406, 0.48043, 0.48643, 0.49221, 0.49776, 0.50292, 0.50771, 0.51188, 0.51561,\
        0.51917, 0.52269, 0.52617, 0.52961, 0.53302, 0.53639, 0.53970, 0.54295, 0.54612, 0.54922, 0.55221,\
        0.55507, 0.55775, 0.56019, 0.56237, 0.56427, 0.56588, 0.56720, 0.56822, 0.56892, 0.56930, 0.56934,\
        0.56903, 0.56833, 0.56726, 0.56584, 0.56412, 0.56214, 0.55997, 0.55763, 0.55517, 0.55258, 0.54988,\
        0.54706, 0.54410, 0.54097, 0.53764, 0.53412, 0.53039, 0.52650, 0.52253, 0.51855, 0.51461, 0.51069,\
        0.50674, 0.50273, 0.49876, 0.49493, 0.49126, 0.48786, 0.48450, 0.48094, 0.47699, 0.47256, 0.46764,\
        0.46229, 0.45655, 0.45049, 0.44413, 0.43750, 0.43062, 0.42348, 0.41608, 0.40842, 0.40050, 0.39230],\
        [0.31767, 0.32210, 0.32618, 0.32993, 0.33337, 0.33654, 0.33945, 0.34213, 0.34461, 0.34691, 0.34906,\
        0.35108, 0.35300, 0.35483, 0.35660, 0.35832, 0.36002, 0.36169, 0.36337, 0.36505, 0.36676, 0.36850,\
        0.37028, 0.37211, 0.37401, 0.37597, 0.37801, 0.38014, 0.38237, 0.38469, 0.38710, 0.38960, 0.39216,\
        0.39482, 0.39772, 0.40099, 0.40545, 0.41125, 0.41796, 0.42523, 0.43286, 0.44069, 0.44858, 0.45641,\
        0.46408, 0.47150, 0.47857, 0.48520, 0.49132, 0.49709, 0.50296, 0.50839, 0.51343, 0.51803, 0.52227,\
        0.52630, 0.53023, 0.53408, 0.53787, 0.54158, 0.54520, 0.54871, 0.55211, 0.55539, 0.55853, 0.56153,\
        0.56434, 0.56694, 0.56926, 0.57127, 0.57297, 0.57439, 0.57553, 0.57638, 0.57690, 0.57710, 0.57699,\
        0.57653, 0.57566, 0.57437, 0.57270, 0.57073, 0.56853, 0.56614, 0.56362, 0.56100, 0.55830, 0.55551,\
        0.55263, 0.54963, 0.54649, 0.54318, 0.53970, 0.53605, 0.53225, 0.52837, 0.52444, 0.52051, 0.51657,\
        0.51260, 0.50861, 0.50466, 0.50080, 0.49717, 0.49402, 0.49094, 0.48758, 0.48374, 0.47933, 0.47438,\
        0.46897, 0.46320, 0.45713, 0.45080, 0.44422, 0.43740, 0.43035, 0.42304, 0.41548, 0.40765, 0.39954],\
        [0.32425, 0.32847, 0.33234, 0.33586, 0.33907, 0.34200, 0.34467, 0.34711, 0.34934, 0.35140, 0.35332,\
        0.35510, 0.35679, 0.35840, 0.35996, 0.36147, 0.36296, 0.36444, 0.36592, 0.36742, 0.36895, 0.37052,\
        0.37214, 0.37381, 0.37555, 0.37737, 0.37929, 0.38132, 0.38348, 0.38579, 0.38826, 0.39091, 0.39370,\
        0.39661, 0.39975, 0.40344, 0.40796, 0.41364, 0.42029, 0.42764, 0.43546, 0.44354, 0.45173, 0.45989,\
        0.46790, 0.47566, 0.48307, 0.49003, 0.49639, 0.50215, 0.50802, 0.51371, 0.51903, 0.52400, 0.52869,\
        0.53318, 0.53754, 0.54180, 0.54596, 0.54999, 0.55386, 0.55756, 0.56109, 0.56446, 0.56764, 0.57062,\
        0.57338, 0.57590, 0.57811, 0.57993, 0.58140, 0.58260, 0.58357, 0.58426, 0.58458, 0.58457, 0.58432,\
        0.58376, 0.58271, 0.58118, 0.57925, 0.57702, 0.57458, 0.57198, 0.56928, 0.56652, 0.56372, 0.56088,\
        0.55797, 0.55497, 0.55185, 0.54859, 0.54516, 0.54158, 0.53787, 0.53406, 0.53018, 0.52627, 0.52232,\
        0.51836, 0.51440, 0.51049, 0.50669, 0.50318, 0.50013, 0.49721, 0.49398, 0.49019, 0.48572, 0.48068,\
        0.47520, 0.46939, 0.46332, 0.45703, 0.45053, 0.44380, 0.43685, 0.42965, 0.42220, 0.41448, 0.40647],\
        [0.33063, 0.33465, 0.33830, 0.34160, 0.34457, 0.34726, 0.34968, 0.35187, 0.35387, 0.35568, 0.35736,\
        0.35892, 0.36038, 0.36178, 0.36312, 0.36443, 0.36572, 0.36701, 0.36832, 0.36965, 0.37101, 0.37241,\
        0.37387, 0.37539, 0.37699, 0.37868, 0.38048, 0.38240, 0.38449, 0.38678, 0.38931, 0.39211, 0.39519,\
        0.39846, 0.40170, 0.40555, 0.41027, 0.41601, 0.42270, 0.43015, 0.43813, 0.44644, 0.45490, 0.46335,\
        0.47167, 0.47972, 0.48741, 0.49467, 0.50141, 0.50738, 0.51321, 0.51895, 0.52449, 0.52978, 0.53486,\
        0.53978, 0.54458, 0.54926, 0.55380, 0.55817, 0.56231, 0.56618, 0.56982, 0.57325, 0.57645, 0.57939,\
        0.58206, 0.58454, 0.58667, 0.58827, 0.58946, 0.59043, 0.59124, 0.59181, 0.59190, 0.59160, 0.59126,\
        0.59068, 0.58946, 0.58764, 0.58543, 0.58295, 0.58028, 0.57748, 0.57461, 0.57173, 0.56886, 0.56599,\
        0.56309, 0.56013, 0.55707, 0.55387, 0.55051, 0.54700, 0.54336, 0.53960, 0.53577, 0.53187, 0.52795,\
        0.52401, 0.52009, 0.51625, 0.51257, 0.50916, 0.50608, 0.50312, 0.49993, 0.49617, 0.49156, 0.48640,\
        0.48086, 0.47504, 0.46901, 0.46280, 0.45641, 0.44981, 0.44299, 0.43592, 0.42859, 0.42099, 0.41310],\
        [0.33682, 0.34064, 0.34407, 0.34714, 0.34988, 0.35232, 0.35449, 0.35643, 0.35818, 0.35976, 0.36120,\
        0.36253, 0.36377, 0.36495, 0.36609, 0.36721, 0.36832, 0.36943, 0.37057, 0.37173, 0.37294, 0.37419,\
        0.37551, 0.37689, 0.37836, 0.37991, 0.38158, 0.38339, 0.38539, 0.38761, 0.39013, 0.39298, 0.39620,\
        0.39969, 0.40322, 0.40724, 0.41217, 0.41811, 0.42497, 0.43259, 0.44077, 0.44931, 0.45802, 0.46674,\
        0.47531, 0.48361, 0.49154, 0.49890, 0.50576, 0.51210, 0.51813, 0.52401, 0.52974, 0.53532, 0.54075,\
        0.54607, 0.55129, 0.55639, 0.56135, 0.56608, 0.57049, 0.57452, 0.57821, 0.58166, 0.58491, 0.58780,\
        0.59025, 0.59270, 0.59487, 0.59619, 0.59710, 0.59784, 0.59847, 0.59894, 0.59882, 0.59810, 0.59774,\
        0.59726, 0.59578, 0.59364, 0.59118, 0.58850, 0.58562, 0.58263, 0.57960, 0.57663, 0.57372, 0.57087,\
        0.56802, 0.56514, 0.56215, 0.55903, 0.55575, 0.55231, 0.54872, 0.54501, 0.54121, 0.53733, 0.53342,\
        0.52950, 0.52562, 0.52182, 0.51819, 0.51476, 0.51155, 0.50841, 0.50509, 0.50125, 0.49654, 0.49141,\
        0.48586, 0.48008, 0.47416, 0.46810, 0.46187, 0.45543, 0.44877, 0.44186, 0.43467, 0.42720, 0.41942],\
        [0.34283, 0.34644, 0.34965, 0.35248, 0.35498, 0.35717, 0.35909, 0.36079, 0.36228, 0.36362, 0.36483,\
        0.36594, 0.36697, 0.36795, 0.36889, 0.36983, 0.37076, 0.37171, 0.37269, 0.37370, 0.37477, 0.37589,\
        0.37707, 0.37833, 0.37967, 0.38110, 0.38264, 0.38432, 0.38617, 0.38826, 0.39066, 0.39344, 0.39662,\
        0.40009, 0.40389, 0.40828, 0.41357, 0.41983, 0.42697, 0.43484, 0.44327, 0.45207, 0.46104, 0.47000,\
        0.47877, 0.48724, 0.49539, 0.50291, 0.50984, 0.51639, 0.52266, 0.52874, 0.53470, 0.54055, 0.54631,\
        0.55200, 0.55762, 0.56315, 0.56854, 0.57366, 0.57837, 0.58253, 0.58620, 0.58960, 0.59290, 0.59589,\
        0.59786, 0.60015, 0.60242, 0.60358, 0.60433, 0.60489, 0.60531, 0.60551, 0.60518, 0.60442, 0.60386,\
        0.60313, 0.60144, 0.59910, 0.59649, 0.59368, 0.59061, 0.58742, 0.58426, 0.58122, 0.57832, 0.57553,\
        0.57279, 0.57001, 0.56714, 0.56410, 0.56090, 0.55752, 0.55397, 0.55029, 0.54650, 0.54263, 0.53872,\
        0.53480, 0.53091, 0.52711, 0.52343, 0.51989, 0.51646, 0.51301, 0.50937, 0.50528, 0.50071, 0.49567,\
        0.49012, 0.48451, 0.47879, 0.47294, 0.46692, 0.46070, 0.45423, 0.44749, 0.44045, 0.43311, 0.42546],\
        [0.34865, 0.35205, 0.35503, 0.35763, 0.35987, 0.36181, 0.36348, 0.36493, 0.36618, 0.36728, 0.36826,\
        0.36915, 0.36998, 0.37076, 0.37153, 0.37229, 0.37306, 0.37386, 0.37469, 0.37558, 0.37651, 0.37751,\
        0.37858, 0.37973, 0.38096, 0.38227, 0.38368, 0.38522, 0.38690, 0.38879, 0.39096, 0.39351, 0.39656,\
        0.40015, 0.40396, 0.40874, 0.41451, 0.42120, 0.42871, 0.43689, 0.44560, 0.45466, 0.46389, 0.47310,\
        0.48207, 0.49055, 0.49859, 0.50635, 0.51353, 0.52031, 0.52681, 0.53313, 0.53932, 0.54544, 0.55151,\
        0.55753, 0.56352, 0.56945, 0.57527, 0.58085, 0.58594, 0.59023, 0.59378, 0.59704, 0.60026, 0.60328,\
        0.60529, 0.60725, 0.60916, 0.61039, 0.61112, 0.61155, 0.61175, 0.61167, 0.61120, 0.61043, 0.60955,\
        0.60835, 0.60652, 0.60414, 0.60145, 0.59854, 0.59525, 0.59187, 0.58860, 0.58553, 0.58269, 0.58002,\
        0.57743, 0.57480, 0.57205, 0.56911, 0.56597, 0.56262, 0.55910, 0.55543, 0.55163, 0.54775, 0.54381,\
        0.53986, 0.53594, 0.53207, 0.52828, 0.52456, 0.52086, 0.51709, 0.51309, 0.50874, 0.50400, 0.49875,\
        0.49369, 0.48838, 0.48297, 0.47740, 0.47164, 0.46565, 0.45939, 0.45283, 0.44596, 0.43875, 0.43121],\
        [0.35427, 0.35746, 0.36022, 0.36257, 0.36457, 0.36625, 0.36766, 0.36885, 0.36986, 0.37073, 0.37149,\
        0.37217, 0.37281, 0.37341, 0.37401, 0.37461, 0.37524, 0.37590, 0.37660, 0.37737, 0.37819, 0.37909,\
        0.38007, 0.38113, 0.38226, 0.38348, 0.38478, 0.38617, 0.38768, 0.38933, 0.39120, 0.39339, 0.39602,\
        0.39921, 0.40349, 0.40887, 0.41519, 0.42235, 0.43024, 0.43874, 0.44772, 0.45704, 0.46652, 0.47600,\
        0.48526, 0.49394, 0.50180, 0.50956, 0.51692, 0.52391, 0.53063, 0.53717, 0.54361, 0.54999, 0.55633,\
        0.56266, 0.56898, 0.57526, 0.58145, 0.58748, 0.59310, 0.59759, 0.60101, 0.60420, 0.60728, 0.61005,\
        0.61225, 0.61410, 0.61568, 0.61681, 0.61749, 0.61781, 0.61784, 0.61755, 0.61694, 0.61603, 0.61485,\
        0.61333, 0.61132, 0.60887, 0.60607, 0.60297, 0.59955, 0.59603, 0.59267, 0.58964, 0.58691, 0.58441,\
        0.58202, 0.57956, 0.57693, 0.57407, 0.57097, 0.56764, 0.56412, 0.56043, 0.55660, 0.55268, 0.54869,\
        0.54469, 0.54068, 0.53671, 0.53277, 0.52885, 0.52489, 0.52081, 0.51651, 0.51189, 0.50693, 0.50195,\
        0.49688, 0.49191, 0.48684, 0.48156, 0.47608, 0.47034, 0.46430, 0.45793, 0.45121, 0.44413, 0.43670],\
        [0.35970, 0.36268, 0.36520, 0.36731, 0.36905, 0.37047, 0.37162, 0.37256, 0.37333, 0.37397, 0.37452,\
        0.37501, 0.37546, 0.37590, 0.37634, 0.37680, 0.37730, 0.37784, 0.37843, 0.37909, 0.37983, 0.38065,\
        0.38155, 0.38254, 0.38361, 0.38476, 0.38599, 0.38727, 0.38863, 0.39007, 0.39165, 0.39346, 0.39571,\
        0.39881, 0.40330, 0.40910, 0.41588, 0.42345, 0.43168, 0.44046, 0.44967, 0.45918, 0.46886, 0.47855,\
        0.48808, 0.49731, 0.50530, 0.51277, 0.52011, 0.52724, 0.53416, 0.54092, 0.54759, 0.55421, 0.56081,\
        0.56742, 0.57402, 0.58056, 0.58701, 0.59331, 0.59932, 0.60420, 0.60783, 0.61105, 0.61401, 0.61663,\
        0.61881, 0.62058, 0.62198, 0.62298, 0.62356, 0.62376, 0.62363, 0.62318, 0.62240, 0.62131, 0.61991,\
        0.61814, 0.61596, 0.61338, 0.61044, 0.60718, 0.60366, 0.60002, 0.59664, 0.59369, 0.59105, 0.58877,\
        0.58662, 0.58435, 0.58182, 0.57900, 0.57591, 0.57257, 0.56902, 0.56529, 0.56140, 0.55741, 0.55336,\
        0.54926, 0.54516, 0.54106, 0.53696, 0.53285, 0.52866, 0.52434, 0.51982, 0.51504, 0.51002, 0.50487,\
        0.49992, 0.49544, 0.49057, 0.48553, 0.48031, 0.47484, 0.46902, 0.46282, 0.45623, 0.44926, 0.44193],\
        [0.36494, 0.36770, 0.36998, 0.37183, 0.37331, 0.37446, 0.37535, 0.37604, 0.37657, 0.37700, 0.37734,\
        0.37765, 0.37794, 0.37823, 0.37854, 0.37888, 0.37926, 0.37970, 0.38020, 0.38078, 0.38144, 0.38220,\
        0.38305, 0.38399, 0.38504, 0.38616, 0.38735, 0.38859, 0.38986, 0.39118, 0.39256, 0.39411, 0.39607,\
        0.39904, 0.40370, 0.40979, 0.41687, 0.42471, 0.43316, 0.44211, 0.45145, 0.46109, 0.47088, 0.48064,\
        0.49014, 0.49914, 0.50762, 0.51546, 0.52299, 0.53031, 0.53744, 0.54443, 0.55132, 0.55816, 0.56501,\
        0.57188, 0.57873, 0.58551, 0.59213, 0.59851, 0.60459, 0.60982, 0.61398, 0.61745, 0.62044, 0.62300,\
        0.62512, 0.62680, 0.62807, 0.62893, 0.62939, 0.62948, 0.62921, 0.62860, 0.62767, 0.62641, 0.62481,\
        0.62286, 0.62052, 0.61782, 0.61478, 0.61143, 0.60786, 0.60422, 0.60088, 0.59786, 0.59522, 0.59321,\
        0.59135, 0.58924, 0.58674, 0.58390, 0.58078, 0.57740, 0.57379, 0.56999, 0.56602, 0.56195, 0.55779,\
        0.55359, 0.54938, 0.54515, 0.54091, 0.53662, 0.53226, 0.52778, 0.52312, 0.51829, 0.51331, 0.50833,\
        0.50361, 0.49889, 0.49420, 0.48936, 0.48442, 0.47920, 0.47358, 0.46753, 0.46104, 0.45415, 0.44689],\
        [0.36996, 0.37251, 0.37455, 0.37613, 0.37733, 0.37821, 0.37884, 0.37928, 0.37959, 0.37980, 0.37997,\
        0.38011, 0.38025, 0.38042, 0.38061, 0.38085, 0.38114, 0.38149, 0.38192, 0.38243, 0.38304, 0.38375,\
        0.38457, 0.38550, 0.38654, 0.38768, 0.38889, 0.39015, 0.39144, 0.39274, 0.39409, 0.39558, 0.39740,\
        0.40009, 0.40492, 0.41113, 0.41831, 0.42624, 0.43476, 0.44376, 0.45313, 0.46277, 0.47259, 0.48242,\
        0.49197, 0.50090, 0.50954, 0.51772, 0.52556, 0.53316, 0.54056, 0.54778, 0.55489, 0.56194, 0.56901,\
        0.57613, 0.58327, 0.59031, 0.59713, 0.60369, 0.60985, 0.61527, 0.61979, 0.62351, 0.62661, 0.62917,\
        0.63124, 0.63284, 0.63399, 0.63473, 0.63507, 0.63503, 0.63463, 0.63389, 0.63281, 0.63140, 0.62964,\
        0.62755, 0.62510, 0.62231, 0.61921, 0.61584, 0.61232, 0.60877, 0.60540, 0.60225, 0.59961, 0.59791,\
        0.59628, 0.59420, 0.59164, 0.58872, 0.58553, 0.58209, 0.57841, 0.57452, 0.57045, 0.56626, 0.56200,\
        0.55769, 0.55336, 0.54901, 0.54465, 0.54023, 0.53575, 0.53116, 0.52645, 0.52162, 0.51672, 0.51186,\
        0.50709, 0.50244, 0.49773, 0.49307, 0.48846, 0.48350, 0.47803, 0.47206, 0.46563, 0.45880, 0.45158],\
        [0.37477, 0.37709, 0.37887, 0.38019, 0.38111, 0.38171, 0.38208, 0.38227, 0.38236, 0.38238, 0.38238,\
        0.38238, 0.38241, 0.38247, 0.38258, 0.38274, 0.38296, 0.38325, 0.38362, 0.38408, 0.38465, 0.38533,\
        0.38613, 0.38706, 0.38813, 0.38931, 0.39060, 0.39194, 0.39332, 0.39474, 0.39618, 0.39778, 0.39989,\
        0.40268, 0.40720, 0.41320, 0.42024, 0.42807, 0.43652, 0.44545, 0.45473, 0.46427, 0.47399, 0.48382,\
        0.49368, 0.50296, 0.51138, 0.51975, 0.52795, 0.53591, 0.54362, 0.55110, 0.55842, 0.56566, 0.57292,\
        0.58029, 0.58777, 0.59518, 0.60227, 0.60905, 0.61532, 0.62087, 0.62558, 0.62949, 0.63268, 0.63525,\
        0.63726, 0.63877, 0.63981, 0.64043, 0.64065, 0.64048, 0.63996, 0.63909, 0.63788, 0.63635, 0.63448,\
        0.63227, 0.62974, 0.62690, 0.62379, 0.62046, 0.61702, 0.61357, 0.61026, 0.60722, 0.60473, 0.60292,\
        0.60125, 0.59909, 0.59639, 0.59338, 0.59014, 0.58663, 0.58287, 0.57886, 0.57467, 0.57036, 0.56597,\
        0.56155, 0.55712, 0.55267, 0.54821, 0.54370, 0.53914, 0.53449, 0.52977, 0.52497, 0.52015, 0.51535,\
        0.51060, 0.50587, 0.50113, 0.49679, 0.49261, 0.48778, 0.48234, 0.47639, 0.46998, 0.46316, 0.45598],\
        [0.37933, 0.38143, 0.38295, 0.38398, 0.38461, 0.38493, 0.38503, 0.38500, 0.38488, 0.38473, 0.38459,\
        0.38448, 0.38442, 0.38441, 0.38446, 0.38458, 0.38475, 0.38500, 0.38533, 0.38575, 0.38629, 0.38694,\
        0.38774, 0.38868, 0.38978, 0.39104, 0.39243, 0.39390, 0.39540, 0.39699, 0.39864, 0.40027, 0.40257,\
        0.40568, 0.41004, 0.41573, 0.42251, 0.43014, 0.43843, 0.44720, 0.45632, 0.46567, 0.47514, 0.48468,\
        0.49419, 0.50354, 0.51268, 0.52161, 0.53030, 0.53870, 0.54677, 0.55455, 0.56208, 0.56947, 0.57688,\
        0.58445, 0.59229, 0.60020, 0.60765, 0.61466, 0.62109, 0.62675, 0.63158, 0.63556, 0.63878, 0.64132,\
        0.64326, 0.64467, 0.64560, 0.64609, 0.64619, 0.64590, 0.64526, 0.64428, 0.64296, 0.64132, 0.63936,\
        0.63708, 0.63450, 0.63164, 0.62854, 0.62528, 0.62192, 0.61859, 0.61540, 0.61249, 0.61000, 0.60793,\
        0.60594, 0.60363, 0.60087, 0.59783, 0.59457, 0.59100, 0.58713, 0.58300, 0.57866, 0.57421, 0.56970,\
        0.56518, 0.56066, 0.55614, 0.55161, 0.54704, 0.54243, 0.53777, 0.53305, 0.52829, 0.52353, 0.51880,\
        0.51412, 0.50949, 0.50499, 0.50097, 0.49690, 0.49193, 0.48644, 0.48045, 0.47402, 0.46720, 0.46004],\
        [0.38365, 0.38551, 0.38675, 0.38747, 0.38780, 0.38784, 0.38769, 0.38744, 0.38714, 0.38684, 0.38659,\
        0.38641, 0.38629, 0.38625, 0.38628, 0.38638, 0.38654, 0.38676, 0.38707, 0.38747, 0.38798, 0.38863,\
        0.38942, 0.39036, 0.39148, 0.39281, 0.39432, 0.39594, 0.39753, 0.39918, 0.40103, 0.40289, 0.40529,\
        0.40856, 0.41289, 0.41838, 0.42491, 0.43232, 0.44042, 0.44903, 0.45798, 0.46710, 0.47628, 0.48547,\
        0.49471, 0.50420, 0.51395, 0.52355, 0.53283, 0.54172, 0.55020, 0.55828, 0.56604, 0.57359, 0.58109,\
        0.58879, 0.59690, 0.60526, 0.61327, 0.62065, 0.62728, 0.63304, 0.63789, 0.64186, 0.64503, 0.64749,\
        0.64932, 0.65061, 0.65141, 0.65178, 0.65174, 0.65134, 0.65059, 0.64950, 0.64810, 0.64637, 0.64434,\
        0.64201, 0.63939, 0.63653, 0.63346, 0.63026, 0.62699, 0.62375, 0.62064, 0.61776, 0.61518, 0.61283,\
        0.61051, 0.60800, 0.60518, 0.60211, 0.59881, 0.59516, 0.59118, 0.58690, 0.58239, 0.57779, 0.57317,\
        0.56858, 0.56400, 0.55943, 0.55486, 0.55026, 0.54563, 0.54096, 0.53626, 0.53154, 0.52684, 0.52218,\
        0.51760, 0.51315, 0.50890, 0.50491, 0.50060, 0.49565, 0.49016, 0.48415, 0.47769, 0.47087, 0.46374],\
        [0.38767, 0.38928, 0.39022, 0.39063, 0.39065, 0.39041, 0.39002, 0.38957, 0.38912, 0.38872, 0.38839,\
        0.38817, 0.38805, 0.38802, 0.38807, 0.38818, 0.38836, 0.38859, 0.38888, 0.38926, 0.38977, 0.39041,\
        0.39119, 0.39212, 0.39323, 0.39458, 0.39619, 0.39799, 0.39972, 0.40126, 0.40315, 0.40527, 0.40786,\
        0.41123, 0.41554, 0.42089, 0.42723, 0.43445, 0.44240, 0.45090, 0.45974, 0.46874, 0.47774, 0.48668,\
        0.49578, 0.50555, 0.51576, 0.52593, 0.53575, 0.54513, 0.55403, 0.56246, 0.57051, 0.57826, 0.58589,\
        0.59373, 0.60213, 0.61103, 0.61952, 0.62724, 0.63403, 0.63982, 0.64462, 0.64849, 0.65153, 0.65384,\
        0.65552, 0.65666, 0.65731, 0.65753, 0.65737, 0.65685, 0.65599, 0.65482, 0.65334, 0.65155, 0.64945,\
        0.64708, 0.64444, 0.64158, 0.63853, 0.63537, 0.63216, 0.62898, 0.62590, 0.62298, 0.62025, 0.61764,\
        0.61503, 0.61229, 0.60935, 0.60618, 0.60277, 0.59905, 0.59499, 0.59052, 0.58582, 0.58108, 0.57638,\
        0.57175, 0.56715, 0.56257, 0.55799, 0.55337, 0.54873, 0.54406, 0.53936, 0.53467, 0.53001, 0.52541,\
        0.52090, 0.51653, 0.51231, 0.50815, 0.50375, 0.49887, 0.49343, 0.48740, 0.48092, 0.47411, 0.46703],\
        [0.39135, 0.39270, 0.39332, 0.39339, 0.39311, 0.39261, 0.39201, 0.39139, 0.39082, 0.39035, 0.39000,\
        0.38979, 0.38971, 0.38973, 0.38985, 0.39003, 0.39026, 0.39052, 0.39081, 0.39118, 0.39168, 0.39234,\
        0.39313, 0.39402, 0.39507, 0.39635, 0.39794, 0.39985, 0.40165, 0.40334, 0.40524, 0.40747, 0.41018,\
        0.41360, 0.41787, 0.42310, 0.42929, 0.43637, 0.44424, 0.45271, 0.46160, 0.47067, 0.47974, 0.48871,\
        0.49777, 0.50773, 0.51836, 0.52892, 0.53917, 0.54900, 0.55834, 0.56716, 0.57558, 0.58372, 0.59163,\
        0.59978, 0.60877, 0.61796, 0.62673, 0.63464, 0.64149, 0.64720, 0.65183, 0.65549, 0.65831, 0.66041,\
        0.66190, 0.66286, 0.66335, 0.66342, 0.66312, 0.66248, 0.66153, 0.66028, 0.65872, 0.65688, 0.65474,\
        0.65232, 0.64965, 0.64678, 0.64374, 0.64060, 0.63742, 0.63425, 0.63114, 0.62813, 0.62523, 0.62238,\
        0.61950, 0.61652, 0.61336, 0.61002, 0.60645, 0.60264, 0.59848, 0.59375, 0.58886, 0.58405, 0.57934,\
        0.57473, 0.57016, 0.56559, 0.56101, 0.55639, 0.55174, 0.54705, 0.54235, 0.53766, 0.53300, 0.52840,\
        0.52389, 0.51950, 0.51519, 0.51088, 0.50642, 0.50161, 0.49620, 0.49011, 0.48363, 0.47688, 0.46989],\
        [0.39464, 0.39569, 0.39595, 0.39569, 0.39512, 0.39440, 0.39363, 0.39289, 0.39225, 0.39176, 0.39144,\
        0.39129, 0.39130, 0.39144, 0.39167, 0.39197, 0.39230, 0.39261, 0.39291, 0.39326, 0.39377, 0.39449,\
        0.39532, 0.39615, 0.39709, 0.39829, 0.39968, 0.40154, 0.40344, 0.40528, 0.40724, 0.40952, 0.41226,\
        0.41564, 0.41982, 0.42490, 0.43093, 0.43789, 0.44573, 0.45430, 0.46339, 0.47278, 0.48221, 0.49159,\
        0.50112, 0.51118, 0.52173, 0.53242, 0.54299, 0.55326, 0.56309, 0.57235, 0.58117, 0.58993, 0.59863,\
        0.60727, 0.61660, 0.62603, 0.63495, 0.64293, 0.64970, 0.65520, 0.65954, 0.66289, 0.66541, 0.66724,\
        0.66850, 0.66925, 0.66957, 0.66949, 0.66905, 0.66829, 0.66724, 0.66592, 0.66431, 0.66241, 0.66022,\
        0.65775, 0.65505, 0.65214, 0.64909, 0.64594, 0.64274, 0.63954, 0.63637, 0.63324, 0.63015, 0.62707,\
        0.62393, 0.62068, 0.61726, 0.61365, 0.60983, 0.60576, 0.60134, 0.59643, 0.59150, 0.58673, 0.58211,\
        0.57758, 0.57307, 0.56853, 0.56396, 0.55934, 0.55466, 0.54995, 0.54522, 0.54049, 0.53578, 0.53113,\
        0.52655, 0.52204, 0.51758, 0.51309, 0.50848, 0.50363, 0.49822, 0.49214, 0.48576, 0.47915, 0.47233],\
        [0.39741, 0.39812, 0.39800, 0.39744, 0.39665, 0.39577, 0.39489, 0.39408, 0.39343, 0.39296, 0.39272,\
        0.39269, 0.39286, 0.39317, 0.39357, 0.39403, 0.39452, 0.39495, 0.39527, 0.39557, 0.39607, 0.39689,\
        0.39786, 0.39866, 0.39941, 0.40062, 0.40199, 0.40363, 0.40543, 0.40727, 0.40922, 0.41144, 0.41409,\
        0.41734, 0.42132, 0.42618, 0.43200, 0.43883, 0.44664, 0.45536, 0.46481, 0.47472, 0.48477, 0.49474,\
        0.50466, 0.51481, 0.52527, 0.53599, 0.54684, 0.55764, 0.56812, 0.57796, 0.58708, 0.59620, 0.60590,\
        0.61547, 0.62523, 0.63492, 0.64402, 0.65203, 0.65864, 0.66380, 0.66771, 0.67064, 0.67280, 0.67433,\
        0.67534, 0.67589, 0.67603, 0.67579, 0.67520, 0.67432, 0.67318, 0.67179, 0.67013, 0.66819, 0.66594,\
        0.66341, 0.66063, 0.65766, 0.65456, 0.65138, 0.64814, 0.64488, 0.64161, 0.63835, 0.63507, 0.63176,\
        0.62836, 0.62482, 0.62112, 0.61719, 0.61302, 0.60857, 0.60377, 0.59878, 0.59390, 0.58926, 0.58481,\
        0.58040, 0.57596, 0.57146, 0.56688, 0.56224, 0.55752, 0.55276, 0.54796, 0.54316, 0.53837, 0.53361,\
        0.52889, 0.52422, 0.51954, 0.51482, 0.50994, 0.50481, 0.49932, 0.49347, 0.48737, 0.48100, 0.47439],\
        [0.39946, 0.39974, 0.39929, 0.39857, 0.39768, 0.39673, 0.39581, 0.39501, 0.39438, 0.39400, 0.39388,\
        0.39405, 0.39445, 0.39498, 0.39558, 0.39625, 0.39698, 0.39759, 0.39796, 0.39821, 0.39866, 0.39961,\
        0.40071, 0.40158, 0.40233, 0.40342, 0.40471, 0.40617, 0.40776, 0.40943, 0.41124, 0.41329, 0.41572,\
        0.41870, 0.42237, 0.42689, 0.43239, 0.43897, 0.44668, 0.45552, 0.46536, 0.47595, 0.48683, 0.49738,\
        0.50770, 0.51784, 0.52820, 0.53897, 0.55018, 0.56166, 0.57310, 0.58398, 0.59360, 0.60286, 0.61323,\
        0.62370, 0.63410, 0.64424, 0.65366, 0.66179, 0.66820, 0.67289, 0.67624, 0.67868, 0.68046, 0.68169,\
        0.68245, 0.68280, 0.68278, 0.68239, 0.68165, 0.68063, 0.67939, 0.67794, 0.67626, 0.67426, 0.67194,\
        0.66930, 0.66641, 0.66335, 0.66018, 0.65692, 0.65362, 0.65027, 0.64689, 0.64348, 0.64002, 0.63648,\
        0.63283, 0.62902, 0.62502, 0.62078, 0.61628, 0.61148, 0.60642, 0.60126, 0.59634, 0.59189, 0.58763,\
        0.58333, 0.57893, 0.57442, 0.56982, 0.56512, 0.56034, 0.55549, 0.55060, 0.54568, 0.54076, 0.53586,\
        0.53097, 0.52609, 0.52119, 0.51621, 0.51108, 0.50569, 0.50003, 0.49454, 0.48868, 0.48253, 0.47615],\
        [0.40007, 0.40017, 0.39979, 0.39914, 0.39829, 0.39736, 0.39647, 0.39572, 0.39519, 0.39493, 0.39499,\
        0.39540, 0.39612, 0.39697, 0.39777, 0.39864, 0.39966, 0.40052, 0.40103, 0.40138, 0.40189, 0.40284,\
        0.40391, 0.40485, 0.40571, 0.40669, 0.40782, 0.40906, 0.41040, 0.41182, 0.41336, 0.41511, 0.41719,\
        0.41976, 0.42296, 0.42699, 0.43199, 0.43815, 0.44558, 0.45435, 0.46445, 0.47570, 0.48766, 0.49936,\
        0.50956, 0.51948, 0.52978, 0.54071, 0.55236, 0.56462, 0.57725, 0.58975, 0.60091, 0.61054, 0.62096,\
        0.63187, 0.64287, 0.65359, 0.66354, 0.67200, 0.67825, 0.68229, 0.68495, 0.68690, 0.68836, 0.68933,\
        0.68986, 0.69004, 0.68991, 0.68938, 0.68846, 0.68728, 0.68593, 0.68445, 0.68274, 0.68069, 0.67825,\
        0.67545, 0.67241, 0.66921, 0.66593, 0.66259, 0.65920, 0.65576, 0.65225, 0.64868, 0.64503, 0.64128,\
        0.63739, 0.63333, 0.62906, 0.62455, 0.61978, 0.61474, 0.60950, 0.60419, 0.59922, 0.59494, 0.59080,\
        0.58649, 0.58203, 0.57747, 0.57280, 0.56802, 0.56314, 0.55817, 0.55314, 0.54808, 0.54301, 0.53793,\
        0.53285, 0.52775, 0.52262, 0.51743, 0.51213, 0.50673, 0.50135, 0.49569, 0.48984, 0.48385, 0.47769],\
        [0.39977, 0.39998, 0.39988, 0.39938, 0.39862, 0.39775, 0.39695, 0.39633, 0.39595, 0.39586, 0.39612,\
        0.39678, 0.39788, 0.39921, 0.40026, 0.40136, 0.40259, 0.40366, 0.40445, 0.40508, 0.40578, 0.40668,\
        0.40766, 0.40859, 0.40946, 0.41035, 0.41129, 0.41229, 0.41333, 0.41443, 0.41561, 0.41695, 0.41856,\
        0.42057, 0.42314, 0.42648, 0.43077, 0.43625, 0.44311, 0.45152, 0.46153, 0.47300, 0.48551, 0.49788,\
        0.50857, 0.51882, 0.52938, 0.54070, 0.55289, 0.56590, 0.57952, 0.59324, 0.60578, 0.61683, 0.62800,\
        0.63955, 0.65117, 0.66252, 0.67316, 0.68231, 0.68857, 0.69171, 0.69366, 0.69525, 0.69652, 0.69732,\
        0.69765, 0.69769, 0.69749, 0.69683, 0.69572, 0.69435, 0.69288, 0.69137, 0.68965, 0.68752, 0.68490,\
        0.68188, 0.67861, 0.67525, 0.67184, 0.66841, 0.66492, 0.66135, 0.65771, 0.65397, 0.65014, 0.64619,\
        0.64209, 0.63781, 0.63332, 0.62859, 0.62363, 0.61846, 0.61316, 0.60791, 0.60300, 0.59867, 0.59440,\
        0.58985, 0.58526, 0.58061, 0.57585, 0.57095, 0.56593, 0.56081, 0.55562, 0.55039, 0.54514, 0.53988,\
        0.53459, 0.52928, 0.52394, 0.51854, 0.51310, 0.50763, 0.50210, 0.49652, 0.49082, 0.48500, 0.47907],\
        [0.39961, 0.40001, 0.39997, 0.39954, 0.39884, 0.39805, 0.39736, 0.39693, 0.39679, 0.39694, 0.39745,\
        0.39836, 0.39976, 0.40154, 0.40309, 0.40454, 0.40597, 0.40725, 0.40831, 0.40922, 0.41009, 0.41100,\
        0.41192, 0.41279, 0.41360, 0.41436, 0.41510, 0.41583, 0.41655, 0.41727, 0.41802, 0.41887, 0.41989,\
        0.42122, 0.42300, 0.42545, 0.42878, 0.43329, 0.43924, 0.44688, 0.45634, 0.46751, 0.47986, 0.49243,\
        0.50445, 0.51555, 0.52678, 0.53879, 0.55168, 0.56536, 0.57968, 0.59419, 0.60804, 0.62105, 0.63381,\
        0.64645, 0.65878, 0.67063, 0.68180, 0.69181, 0.69873, 0.70086, 0.70244, 0.70386, 0.70501, 0.70570,\
        0.70595, 0.70589, 0.70556, 0.70479, 0.70355, 0.70203, 0.70041, 0.69882, 0.69706, 0.69476, 0.69191,\
        0.68856, 0.68500, 0.68145, 0.67793, 0.67440, 0.67079, 0.66709, 0.66329, 0.65938, 0.65537, 0.65124,\
        0.64696, 0.64249, 0.63781, 0.63292, 0.62783, 0.62259, 0.61730, 0.61211, 0.60722, 0.60273, 0.59805,\
        0.59327, 0.58857, 0.58384, 0.57897, 0.57392, 0.56872, 0.56341, 0.55805, 0.55264, 0.54720, 0.54175,\
        0.53626, 0.53075, 0.52520, 0.51960, 0.51399, 0.50839, 0.50286, 0.49730, 0.49166, 0.48605, 0.48037],\
        [0.39959, 0.40009, 0.40011, 0.39974, 0.39909, 0.39834, 0.39779, 0.39765, 0.39787, 0.39837, 0.39921,\
        0.40050, 0.40225, 0.40431, 0.40630, 0.40813, 0.40982, 0.41134, 0.41264, 0.41377, 0.41479, 0.41574,\
        0.41661, 0.41740, 0.41810, 0.41870, 0.41922, 0.41967, 0.42005, 0.42037, 0.42065, 0.42093, 0.42129,\
        0.42183, 0.42269, 0.42406, 0.42620, 0.42942, 0.43409, 0.44057, 0.44909, 0.45963, 0.47177, 0.48478,\
        0.49772, 0.50985, 0.52218, 0.53539, 0.54921, 0.56350, 0.57835, 0.59382, 0.60911, 0.62411, 0.63893,\
        0.65296, 0.66594, 0.67795, 0.68907, 0.69888, 0.70614, 0.70947, 0.71133, 0.71274, 0.71379, 0.71443,\
        0.71467, 0.71456, 0.71412, 0.71327, 0.71201, 0.71043, 0.70871, 0.70690, 0.70486, 0.70231, 0.69921,\
        0.69545, 0.69158, 0.68787, 0.68424, 0.68060, 0.67686, 0.67299, 0.66899, 0.66490, 0.66073, 0.65644,\
        0.65200, 0.64737, 0.64255, 0.63752, 0.63233, 0.62703, 0.62171, 0.61648, 0.61143, 0.60651, 0.60155,\
        0.59671, 0.59195, 0.58719, 0.58219, 0.57693, 0.57151, 0.56599, 0.56043, 0.55483, 0.54922, 0.54359,\
        0.53792, 0.53222, 0.52647, 0.52069, 0.51489, 0.50910, 0.50338, 0.49780, 0.49241, 0.48708, 0.48170],\
        [0.39971, 0.40028, 0.40038, 0.40009, 0.39950, 0.39873, 0.39826, 0.39856, 0.39933, 0.40029, 0.40165,\
        0.40339, 0.40549, 0.40780, 0.41008, 0.41222, 0.41418, 0.41592, 0.41743, 0.41874, 0.41986, 0.42084,\
        0.42167, 0.42237, 0.42293, 0.42335, 0.42364, 0.42380, 0.42382, 0.42373, 0.42352, 0.42321, 0.42287,\
        0.42255, 0.42239, 0.42255, 0.42329, 0.42495, 0.42799, 0.43293, 0.44021, 0.44997, 0.46188, 0.47517,\
        0.48886, 0.50224, 0.51647, 0.53148, 0.54647, 0.56141, 0.57642, 0.59213, 0.60908, 0.62717, 0.64462,\
        0.66012, 0.67345, 0.68515, 0.69583, 0.70533, 0.71262, 0.71717, 0.71984, 0.72155, 0.72269, 0.72337,\
        0.72366, 0.72357, 0.72311, 0.72224, 0.72097, 0.71936, 0.71750, 0.71540, 0.71299, 0.71010, 0.70663,\
        0.70259, 0.69848, 0.69463, 0.69086, 0.68707, 0.68315, 0.67905, 0.67481, 0.67052, 0.66619, 0.66178,\
        0.65721, 0.65245, 0.64749, 0.64235, 0.63705, 0.63167, 0.62627, 0.62093, 0.61567, 0.61047, 0.60531,\
        0.60025, 0.59546, 0.59075, 0.58550, 0.57994, 0.57425, 0.56851, 0.56276, 0.55700, 0.55123, 0.54544,\
        0.53961, 0.53375, 0.52784, 0.52190, 0.51592, 0.50995, 0.50406, 0.49847, 0.49331, 0.48829, 0.48318],\
        [0.40002, 0.40068, 0.40087, 0.40068, 0.40020, 0.39933, 0.39886, 0.39964, 0.40118, 0.40281, 0.40473,\
        0.40694, 0.40938, 0.41194, 0.41447, 0.41686, 0.41904, 0.42098, 0.42267, 0.42410, 0.42530, 0.42628,\
        0.42706, 0.42766, 0.42806, 0.42828, 0.42832, 0.42819, 0.42787, 0.42736, 0.42666, 0.42578, 0.42473,\
        0.42355, 0.42234, 0.42123, 0.42045, 0.42035, 0.42144, 0.42447, 0.43021, 0.43915, 0.45098, 0.46483,\
        0.47973, 0.49520, 0.51173, 0.52827, 0.54470, 0.56070, 0.57623, 0.59186, 0.61089, 0.63263, 0.65259,\
        0.66921, 0.68236, 0.69323, 0.70346, 0.71262, 0.71985, 0.72489, 0.72816, 0.73026, 0.73160, 0.73242,\
        0.73282, 0.73282, 0.73242, 0.73159, 0.73033, 0.72866, 0.72663, 0.72425, 0.72148, 0.71824, 0.71449,\
        0.71032, 0.70604, 0.70190, 0.69784, 0.69382, 0.68967, 0.68525, 0.68067, 0.67617, 0.67174, 0.66725,\
        0.66258, 0.65770, 0.65260, 0.64733, 0.64193, 0.63646, 0.63095, 0.62546, 0.62002, 0.61462, 0.60929,\
        0.60409, 0.59953, 0.59449, 0.58874, 0.58283, 0.57689, 0.57096, 0.56505, 0.55915, 0.55325, 0.54733,\
        0.54138, 0.53540, 0.52938, 0.52332, 0.51725, 0.51120, 0.50524, 0.49963, 0.49469, 0.48986, 0.48493],\
        [0.40060, 0.40137, 0.40169, 0.40163, 0.40121, 0.40052, 0.40027, 0.40150, 0.40363, 0.40596, 0.40844,\
        0.41109, 0.41387, 0.41669, 0.41944, 0.42204, 0.42441, 0.42651, 0.42832, 0.42983, 0.43107, 0.43203,\
        0.43274, 0.43321, 0.43344, 0.43345, 0.43324, 0.43281, 0.43215, 0.43125, 0.43009, 0.42866, 0.42695,\
        0.42497, 0.42275, 0.42040, 0.41810, 0.41617, 0.41512, 0.41585, 0.41973, 0.42783, 0.43984, 0.45456,\
        0.47107, 0.48899, 0.50734, 0.52601, 0.54467, 0.56272, 0.58023, 0.59837, 0.61943, 0.64242, 0.66387,\
        0.68107, 0.69349, 0.70350, 0.71286, 0.72130, 0.72810, 0.73313, 0.73662, 0.73896, 0.74052, 0.74152,\
        0.74209, 0.74225, 0.74198, 0.74123, 0.73998, 0.73823, 0.73603, 0.73339, 0.73031, 0.72679, 0.72283,\
        0.71852, 0.71406, 0.70960, 0.70518, 0.70080, 0.69637, 0.69144, 0.68642, 0.68178, 0.67737, 0.67286,\
        0.66811, 0.66308, 0.65783, 0.65243, 0.64691, 0.64132, 0.63569, 0.63006, 0.62445, 0.61890, 0.61345,\
        0.60823, 0.60324, 0.59765, 0.59163, 0.58551, 0.57939, 0.57333, 0.56731, 0.56132, 0.55532, 0.54930,\
        0.54326, 0.53719, 0.53110, 0.52500, 0.51892, 0.51292, 0.50710, 0.50170, 0.49671, 0.49182, 0.48698],\
        [0.40151, 0.40245, 0.40298, 0.40316, 0.40308, 0.40298, 0.40341, 0.40490, 0.40722, 0.40993, 0.41282,\
        0.41584, 0.41892, 0.42198, 0.42493, 0.42770, 0.43022, 0.43245, 0.43435, 0.43591, 0.43714, 0.43805,\
        0.43866, 0.43898, 0.43902, 0.43880, 0.43833, 0.43761, 0.43662, 0.43536, 0.43379, 0.43188, 0.42959,\
        0.42689, 0.42378, 0.42032, 0.41665, 0.41303, 0.40988, 0.40806, 0.40952, 0.41687, 0.42927, 0.44469,\
        0.46227, 0.48166, 0.50257, 0.52463, 0.54655, 0.56751, 0.58778, 0.60867, 0.63126, 0.65479, 0.67716,\
        0.69508, 0.70638, 0.71546, 0.72369, 0.73106, 0.73718, 0.74190, 0.74533, 0.74775, 0.74944, 0.75063,\
        0.75142, 0.75181, 0.75173, 0.75110, 0.74986, 0.74803, 0.74563, 0.74275, 0.73942, 0.73566, 0.73151,\
        0.72703, 0.72233, 0.71753, 0.71269, 0.70784, 0.70287, 0.69726, 0.69191, 0.68737, 0.68314, 0.67865,\
        0.67375, 0.66853, 0.66311, 0.65756, 0.65192, 0.64622, 0.64047, 0.63470, 0.62893, 0.62319, 0.61752,\
        0.61194, 0.60630, 0.60038, 0.59422, 0.58797, 0.58177, 0.57565, 0.56958, 0.56353, 0.55746, 0.55137,\
        0.54526, 0.53914, 0.53301, 0.52692, 0.52088, 0.51498, 0.50930, 0.50395, 0.49884, 0.49404, 0.48931],\
        [0.40278, 0.40397, 0.40481, 0.40538, 0.40583, 0.40640, 0.40746, 0.40926, 0.41175, 0.41468, 0.41786,\
        0.42115, 0.42448, 0.42775, 0.43088, 0.43380, 0.43644, 0.43876, 0.44071, 0.44228, 0.44348, 0.44430,\
        0.44477, 0.44491, 0.44474, 0.44428, 0.44354, 0.44252, 0.44123, 0.43963, 0.43770, 0.43537, 0.43260,\
        0.42931, 0.42546, 0.42108, 0.41629, 0.41133, 0.40660, 0.40269, 0.40101, 0.40786, 0.42026, 0.43547,\
        0.45332, 0.47422, 0.49811, 0.52413, 0.55006, 0.57426, 0.59719, 0.62007, 0.64332, 0.66653, 0.68851,\
        0.70649, 0.71793, 0.72671, 0.73438, 0.74110, 0.74667, 0.75101, 0.75424, 0.75662, 0.75838, 0.75974,\
        0.76077, 0.76146, 0.76164, 0.76115, 0.75993, 0.75798, 0.75538, 0.75224, 0.74868, 0.74472, 0.74039,\
        0.73571, 0.73074, 0.72555, 0.72021, 0.71472, 0.70895, 0.70286, 0.69741, 0.69312, 0.68923, 0.68463,\
        0.67942, 0.67394, 0.66833, 0.66265, 0.65692, 0.65112, 0.64526, 0.63935, 0.63341, 0.62745, 0.62148,\
        0.61548, 0.60937, 0.60310, 0.59669, 0.59033, 0.58412, 0.57800, 0.57192, 0.56583, 0.55971, 0.55355,\
        0.54738, 0.54122, 0.53508, 0.52901, 0.52304, 0.51724, 0.51165, 0.50628, 0.50121, 0.49659, 0.49190],\
        [0.40440, 0.40591, 0.40714, 0.40818, 0.40920, 0.41040, 0.41201, 0.41419, 0.41690, 0.42004, 0.42342,\
        0.42694, 0.43047, 0.43392, 0.43721, 0.44026, 0.44300, 0.44537, 0.44735, 0.44890, 0.45002, 0.45072,\
        0.45102, 0.45096, 0.45055, 0.44982, 0.44879, 0.44748, 0.44587, 0.44396, 0.44170, 0.43904, 0.43588,\
        0.43213, 0.42770, 0.42256, 0.41685, 0.41090, 0.40519, 0.40044, 0.39813, 0.40288, 0.41335, 0.42633,\
        0.44315, 0.46527, 0.49265, 0.52377, 0.55458, 0.58217, 0.60694, 0.63094, 0.65403, 0.67603, 0.69602,\
        0.71310, 0.72600, 0.73587, 0.74401, 0.75078, 0.75617, 0.76026, 0.76328, 0.76557, 0.76736, 0.76885,\
        0.77015, 0.77119, 0.77169, 0.77136, 0.77013, 0.76802, 0.76514, 0.76174, 0.75797, 0.75386, 0.74937,\
        0.74448, 0.73921, 0.73362, 0.72779, 0.72174, 0.71551, 0.70929, 0.70368, 0.69943, 0.69573, 0.69057,\
        0.68490, 0.67913, 0.67339, 0.66764, 0.66185, 0.65598, 0.65003, 0.64400, 0.63789, 0.63171, 0.62545,\
        0.61907, 0.61255, 0.60589, 0.59923, 0.59278, 0.58660, 0.58052, 0.57442, 0.56827, 0.56208, 0.55585,\
        0.54961, 0.54339, 0.53724, 0.53119, 0.52529, 0.51959, 0.51412, 0.50891, 0.50401, 0.49926, 0.49469],\
        [0.40636, 0.40822, 0.40986, 0.41141, 0.41299, 0.41476, 0.41687, 0.41943, 0.42243, 0.42578, 0.42937,\
        0.43308, 0.43679, 0.44040, 0.44383, 0.44699, 0.44981, 0.45222, 0.45419, 0.45569, 0.45670, 0.45725,\
        0.45735, 0.45704, 0.45636, 0.45534, 0.45400, 0.45237, 0.45045, 0.44823, 0.44567, 0.44271, 0.43925,\
        0.43515, 0.43024, 0.42443, 0.41780, 0.41079, 0.40428, 0.39869, 0.39617, 0.39938, 0.40703, 0.41498,\
        0.42938, 0.45228, 0.48439, 0.52214, 0.55842, 0.59008, 0.61638, 0.64005, 0.66246, 0.68343, 0.70211,\
        0.71831, 0.73191, 0.74316, 0.75249, 0.75998, 0.76559, 0.76956, 0.77241, 0.77464, 0.77645, 0.77803,\
        0.77954, 0.78101, 0.78191, 0.78165, 0.78039, 0.77804, 0.77478, 0.77108, 0.76715, 0.76296, 0.75837,\
        0.75327, 0.74769, 0.74170, 0.73543, 0.72897, 0.72248, 0.71621, 0.71060, 0.70602, 0.70150, 0.69584,\
        0.68984, 0.68396, 0.67821, 0.67248, 0.66669, 0.66079, 0.65478, 0.64865, 0.64241, 0.63604, 0.62955,\
        0.62289, 0.61605, 0.60908, 0.60210, 0.59554, 0.58945, 0.58333, 0.57715, 0.57090, 0.56459, 0.55825,\
        0.55190, 0.54560, 0.53941, 0.53337, 0.52752, 0.52191, 0.51657, 0.51151, 0.50672, 0.50217, 0.49768],\
        [0.40857, 0.41080, 0.41289, 0.41492, 0.41702, 0.41931, 0.42189, 0.42483, 0.42813, 0.43173, 0.43553,\
        0.43943, 0.44332, 0.44708, 0.45064, 0.45389, 0.45678, 0.45922, 0.46117, 0.46259, 0.46347, 0.46382,\
        0.46369, 0.46311, 0.46212, 0.46078, 0.45910, 0.45712, 0.45484, 0.45229, 0.44942, 0.44618, 0.44246,\
        0.43807, 0.43278, 0.42633, 0.41856, 0.40977, 0.40143, 0.39433, 0.39020, 0.39144, 0.39631, 0.39802,\
        0.40935, 0.43121, 0.46862, 0.51573, 0.55765, 0.59257, 0.62080, 0.64537, 0.66764, 0.68791, 0.70602,\
        0.72188, 0.73631, 0.74915, 0.76016, 0.76892, 0.77510, 0.77899, 0.78169, 0.78401, 0.78592, 0.78749,\
        0.78903, 0.79088, 0.79237, 0.79176, 0.79061, 0.78792, 0.78406, 0.78008, 0.77611, 0.77195, 0.76732,\
        0.76203, 0.75610, 0.74970, 0.74302, 0.73622, 0.72948, 0.72303, 0.71710, 0.71168, 0.70622, 0.70027,\
        0.69417, 0.68842, 0.68282, 0.67719, 0.67143, 0.66553, 0.65947, 0.65328, 0.64695, 0.64048, 0.63384,\
        0.62701, 0.62000, 0.61286, 0.60576, 0.59917, 0.59290, 0.58652, 0.58013, 0.57372, 0.56724, 0.56071,\
        0.55421, 0.54779, 0.54152, 0.53545, 0.52962, 0.52406, 0.51880, 0.51385, 0.50918, 0.50473, 0.50009],\
        [0.41098, 0.41359, 0.41609, 0.41858, 0.42116, 0.42392, 0.42694, 0.43026, 0.43388, 0.43774, 0.44178,\
        0.44587, 0.44993, 0.45385, 0.45752, 0.46086, 0.46381, 0.46627, 0.46820, 0.46953, 0.47024, 0.47037,\
        0.46996, 0.46908, 0.46777, 0.46607, 0.46402, 0.46164, 0.45896, 0.45602, 0.45279, 0.44923, 0.44521,\
        0.44054, 0.43491, 0.42789, 0.41899, 0.40785, 0.39564, 0.38515, 0.37775, 0.37472, 0.37412, 0.37464,\
        0.38150, 0.39920, 0.44021, 0.49982, 0.54944, 0.58700, 0.61899, 0.64672, 0.66961, 0.68902, 0.70664,\
        0.72347, 0.73951, 0.75432, 0.76744, 0.77801, 0.78505, 0.78876, 0.79124, 0.79407, 0.79621, 0.79777,\
        0.79919, 0.80067, 0.80264, 0.80148, 0.80060, 0.79717, 0.79275, 0.78871, 0.78479, 0.78079, 0.77619,\
        0.77064, 0.76428, 0.75743, 0.75035, 0.74322, 0.73620, 0.72943, 0.72301, 0.71686, 0.71070, 0.70435,\
        0.69823, 0.69274, 0.68731, 0.68179, 0.67608, 0.67016, 0.66409, 0.65788, 0.65153, 0.64503, 0.63834,\
        0.63145, 0.62439, 0.61723, 0.61017, 0.60342, 0.59669, 0.58993, 0.58333, 0.57671, 0.56998, 0.56320,\
        0.55647, 0.54987, 0.54348, 0.53733, 0.53147, 0.52592, 0.52068, 0.51574, 0.51103, 0.50640, 0.50153],\
        [0.41352, 0.41648, 0.41938, 0.42230, 0.42532, 0.42851, 0.43193, 0.43561, 0.43954, 0.44369, 0.44796,\
        0.45227, 0.45651, 0.46057, 0.46436, 0.46778, 0.47078, 0.47327, 0.47517, 0.47640, 0.47694, 0.47681,\
        0.47611, 0.47490, 0.47325, 0.47118, 0.46871, 0.46588, 0.46274, 0.45932, 0.45563, 0.45163, 0.44722,\
        0.44216, 0.43613, 0.42856, 0.41867, 0.40542, 0.38860, 0.37220, 0.36002, 0.35301, 0.34935, 0.34788,\
        0.35153, 0.36712, 0.40799, 0.47543, 0.53348, 0.57789, 0.61551, 0.64639, 0.66995, 0.68855, 0.70593,\
        0.72445, 0.74247, 0.75923, 0.77447, 0.78733, 0.79557, 0.79924, 0.80171, 0.80524, 0.80789, 0.80964,\
        0.81096, 0.81208, 0.81271, 0.81178, 0.80974, 0.80588, 0.80144, 0.79718, 0.79322, 0.78950, 0.78493,\
        0.77892, 0.77198, 0.76466, 0.75722, 0.74978, 0.74246, 0.73534, 0.72846, 0.72177, 0.71519, 0.70874,\
        0.70270, 0.69713, 0.69175, 0.68629, 0.68057, 0.67463, 0.66856, 0.66239, 0.65610, 0.64966, 0.64302,\
        0.63617, 0.62911, 0.62193, 0.61472, 0.60755, 0.60033, 0.59340, 0.58676, 0.57987, 0.57277, 0.56563,\
        0.55860, 0.55177, 0.54520, 0.53895, 0.53301, 0.52741, 0.52213, 0.51711, 0.51228, 0.50746, 0.50245],\
        [0.41612, 0.41942, 0.42268, 0.42598, 0.42939, 0.43297, 0.43676, 0.44078, 0.44502, 0.44944, 0.45397,\
        0.45851, 0.46294, 0.46715, 0.47103, 0.47452, 0.47756, 0.48009, 0.48198, 0.48313, 0.48347, 0.48306,\
        0.48202, 0.48050, 0.47853, 0.47609, 0.47319, 0.46986, 0.46615, 0.46215, 0.45788, 0.45330, 0.44829,\
        0.44262, 0.43592, 0.42752, 0.41631, 0.40025, 0.37806, 0.35565, 0.33890, 0.32999, 0.32581, 0.32367,\
        0.32630, 0.34393, 0.38600, 0.44844, 0.51059, 0.56522, 0.61130, 0.64537, 0.66994, 0.68904, 0.70707,\
        0.72698, 0.74690, 0.76495, 0.78091, 0.79584, 0.80503, 0.81064, 0.81461, 0.81826, 0.82113, 0.82302,\
        0.82412, 0.82455, 0.82410, 0.82241, 0.81943, 0.81527, 0.81056, 0.80585, 0.80161, 0.79794, 0.79328,\
        0.78636, 0.77881, 0.77113, 0.76344, 0.75577, 0.74818, 0.74073, 0.73347, 0.72641, 0.71958, 0.71305,\
        0.70696, 0.70131, 0.69601, 0.69060, 0.68477, 0.67878, 0.67277, 0.66674, 0.66061, 0.65434, 0.64784,\
        0.64108, 0.63407, 0.62684, 0.61945, 0.61195, 0.60441, 0.59748, 0.59057, 0.58313, 0.57546, 0.56786,\
        0.56048, 0.55338, 0.54663, 0.54023, 0.53419, 0.52850, 0.52313, 0.51801, 0.51304, 0.50809, 0.50299],\
        [0.41873, 0.42233, 0.42592, 0.42956, 0.43331, 0.43723, 0.44134, 0.44567, 0.45021, 0.45490, 0.45969,\
        0.46446, 0.46909, 0.47346, 0.47743, 0.48094, 0.48400, 0.48656, 0.48849, 0.48959, 0.48973, 0.48898,\
        0.48762, 0.48584, 0.48360, 0.48084, 0.47750, 0.47361, 0.46928, 0.46460, 0.45962, 0.45428, 0.44843,\
        0.44183, 0.43405, 0.42431, 0.41107, 0.39144, 0.36497, 0.33733, 0.31611, 0.30783, 0.30485, 0.30336,\
        0.30453, 0.32368, 0.36743, 0.42517, 0.48653, 0.54745, 0.60239, 0.64240, 0.66883, 0.68862, 0.70874,\
        0.73219, 0.75519, 0.77418, 0.78916, 0.80123, 0.81424, 0.82332, 0.82907, 0.83307, 0.83588, 0.83758,\
        0.83822, 0.83782, 0.83630, 0.83357, 0.82970, 0.82494, 0.81974, 0.81456, 0.80975, 0.80530, 0.79956,\
        0.79215, 0.78443, 0.77671, 0.76896, 0.76115, 0.75334, 0.74561, 0.73802, 0.73066, 0.72358, 0.71690,\
        0.71072, 0.70509, 0.69996, 0.69439, 0.68833, 0.68241, 0.67660, 0.67084, 0.66501, 0.65901, 0.65275,\
        0.64616, 0.63922, 0.63196, 0.62443, 0.61678, 0.60926, 0.60209, 0.59450, 0.58614, 0.57781, 0.56974,\
        0.56201, 0.55465, 0.54769, 0.54114, 0.53498, 0.52919, 0.52372, 0.51849, 0.51341, 0.50837, 0.50323],\
        [0.42130, 0.42517, 0.42904, 0.43297, 0.43702, 0.44122, 0.44562, 0.45022, 0.45502, 0.45997, 0.46501,\
        0.47002, 0.47487, 0.47939, 0.48343, 0.48691, 0.48991, 0.49249, 0.49451, 0.49558, 0.49557, 0.49442,\
        0.49280, 0.49087, 0.48850, 0.48550, 0.48173, 0.47725, 0.47224, 0.46685, 0.46110, 0.45489, 0.44800,\
        0.44012, 0.43088, 0.41959, 0.40407, 0.38178, 0.35287, 0.32280, 0.29804, 0.28867, 0.28501, 0.28373,\
        0.28749, 0.30892, 0.35012, 0.40294, 0.46206, 0.52436, 0.58813, 0.63815, 0.66706, 0.68619, 0.71067,\
        0.74299, 0.77059, 0.79048, 0.80588, 0.81897, 0.83083, 0.83980, 0.84558, 0.84933, 0.85176, 0.85299,\
        0.85295, 0.85160, 0.84892, 0.84501, 0.84009, 0.83448, 0.82854, 0.82259, 0.81676, 0.81079, 0.80400,\
        0.79649, 0.78898, 0.78153, 0.77386, 0.76596, 0.75794, 0.74994, 0.74205, 0.73439, 0.72704, 0.72012,\
        0.71370, 0.70784, 0.70239, 0.69665, 0.69089, 0.68536, 0.67999, 0.67467, 0.66927, 0.66366, 0.65773,\
        0.65136, 0.64453, 0.63722, 0.62952, 0.62158, 0.61361, 0.60570, 0.59726, 0.58831, 0.57957, 0.57116,\
        0.56312, 0.55552, 0.54837, 0.54166, 0.53539, 0.52949, 0.52393, 0.51861, 0.51346, 0.50836, 0.50321],\
        [0.42380, 0.42790, 0.43200, 0.43618, 0.44046, 0.44490, 0.44953, 0.45436, 0.45938, 0.46455, 0.46982,\
        0.47506, 0.48014, 0.48486, 0.48898, 0.49232, 0.49512, 0.49766, 0.49977, 0.50078, 0.50072, 0.49919,\
        0.49756, 0.49562, 0.49327, 0.49016, 0.48601, 0.48092, 0.47525, 0.46921, 0.46275, 0.45570, 0.44769,\
        0.43833, 0.42717, 0.41431, 0.39976, 0.37392, 0.34339, 0.31429, 0.29098, 0.27663, 0.27003, 0.26890,\
        0.27619, 0.29766, 0.33670, 0.38759, 0.43948, 0.49495, 0.56727, 0.63419, 0.66609, 0.68791, 0.72279,\
        0.76379, 0.79507, 0.81481, 0.83021, 0.84329, 0.85343, 0.86004, 0.86395, 0.86658, 0.86830, 0.86882,\
        0.86792, 0.86549, 0.86153, 0.85629, 0.85016, 0.84352, 0.83666, 0.82977, 0.82286, 0.81573, 0.80811,\
        0.80022, 0.79313, 0.78600, 0.77833, 0.77026, 0.76200, 0.75372, 0.74553, 0.73755, 0.72989, 0.72264,\
        0.71585, 0.70949, 0.70339, 0.69774, 0.69266, 0.68775, 0.68297, 0.67823, 0.67339, 0.66829, 0.66277,\
        0.65670, 0.64998, 0.64261, 0.63463, 0.62617, 0.61735, 0.60819, 0.59880, 0.58973, 0.58082, 0.57213,\
        0.56383, 0.55600, 0.54867, 0.54182, 0.53543, 0.52945, 0.52381, 0.51843, 0.51323, 0.50811, 0.50298],\
        [0.42619, 0.43048, 0.43478, 0.43914, 0.44362, 0.44824, 0.45304, 0.45803, 0.46321, 0.46856, 0.47401,\
        0.47946, 0.48476, 0.48968, 0.49399, 0.49714, 0.49962, 0.50206, 0.50403, 0.50503, 0.50487, 0.50372,\
        0.50213, 0.50022, 0.49799, 0.49490, 0.49041, 0.48471, 0.47857, 0.47212, 0.46521, 0.45753, 0.44859,\
        0.43781, 0.42469, 0.40850, 0.39062, 0.36502, 0.33489, 0.30707, 0.28541, 0.27058, 0.26243, 0.26065,\
        0.26769, 0.28875, 0.32644, 0.37595, 0.42016, 0.45921, 0.53131, 0.62466, 0.66352, 0.69131, 0.74289,\
        0.79294, 0.82333, 0.84149, 0.85620, 0.86954, 0.87821, 0.88178, 0.88283, 0.88404, 0.88495, 0.88452,\
        0.88263, 0.87900, 0.87359, 0.86688, 0.85944, 0.85171, 0.84391, 0.83613, 0.82836, 0.82049, 0.81250,\
        0.80469, 0.79760, 0.79041, 0.78237, 0.77402, 0.76550, 0.75693, 0.74843, 0.74013, 0.73214, 0.72455,\
        0.71740, 0.71072, 0.70453, 0.69904, 0.69421, 0.68981, 0.68564, 0.68156, 0.67737, 0.67288, 0.66787,\
        0.66216, 0.65561, 0.64816, 0.63983, 0.63074, 0.62105, 0.61094, 0.60081, 0.59129, 0.58184, 0.57275,\
        0.56416, 0.55612, 0.54862, 0.54165, 0.53516, 0.52910, 0.52340, 0.51799, 0.51277, 0.50766, 0.50258],\
        [0.42847, 0.43291, 0.43736, 0.44186, 0.44647, 0.45121, 0.45611, 0.46120, 0.46648, 0.47193, 0.47750,\
        0.48311, 0.48857, 0.49359, 0.49796, 0.50131, 0.50367, 0.50598, 0.50783, 0.50887, 0.50892, 0.50810,\
        0.50667, 0.50478, 0.50245, 0.49957, 0.49466, 0.48861, 0.48261, 0.47622, 0.46927, 0.46144, 0.45200,\
        0.44006, 0.42516, 0.40769, 0.38432, 0.35642, 0.32840, 0.30371, 0.28392, 0.27060, 0.26292, 0.26027,\
        0.26481, 0.28126, 0.31658, 0.36782, 0.41174, 0.44579, 0.51401, 0.60968, 0.66023, 0.69896, 0.76818,\
        0.82268, 0.84819, 0.86381, 0.87772, 0.89312, 0.90106, 0.90254, 0.90117, 0.90123, 0.90109, 0.89936,\
        0.89635, 0.89143, 0.88442, 0.87614, 0.86747, 0.85877, 0.85016, 0.84166, 0.83326, 0.82494, 0.81677,\
        0.80902, 0.80197, 0.79401, 0.78571, 0.77718, 0.76842, 0.75956, 0.75075, 0.74214, 0.73384, 0.72595,\
        0.71854, 0.71169, 0.70553, 0.70017, 0.69557, 0.69160, 0.68806, 0.68468, 0.68123, 0.67745, 0.67305,\
        0.66779, 0.66146, 0.65394, 0.64523, 0.63549, 0.62499, 0.61413, 0.60349, 0.59285, 0.58263, 0.57309,\
        0.56420, 0.55593, 0.54827, 0.54117, 0.53460, 0.52848, 0.52275, 0.51733, 0.51212, 0.50705, 0.50203],\
        [0.43062, 0.43517, 0.43973, 0.44433, 0.44901, 0.45382, 0.45877, 0.46389, 0.46918, 0.47464, 0.48024,\
        0.48591, 0.49150, 0.49668, 0.50076, 0.50422, 0.50707, 0.50950, 0.51143, 0.51265, 0.51302, 0.51255,\
        0.51133, 0.50939, 0.50666, 0.50281, 0.49774, 0.49311, 0.48826, 0.48235, 0.47582, 0.46857, 0.45949,\
        0.44692, 0.42946, 0.40625, 0.37977, 0.35252, 0.32678, 0.30390, 0.28752, 0.27832, 0.27297, 0.27008,\
        0.27221, 0.28306, 0.31411, 0.36402, 0.41210, 0.44875, 0.50999, 0.60314, 0.66173, 0.71239, 0.78019,\
        0.83683, 0.86664, 0.88183, 0.89311, 0.90858, 0.91850, 0.92092, 0.91978, 0.91812, 0.91595, 0.91257,\
        0.90793, 0.90169, 0.89304, 0.88345, 0.87389, 0.86450, 0.85531, 0.84629, 0.83745, 0.82879, 0.82038,\
        0.81225, 0.80415, 0.79625, 0.78836, 0.77975, 0.77074, 0.76160, 0.75252, 0.74362, 0.73505, 0.72690,\
        0.71929, 0.71232, 0.70614, 0.70088, 0.69658, 0.69311, 0.69021, 0.68759, 0.68494, 0.68195, 0.67828,\
        0.67359, 0.66756, 0.66001, 0.65092, 0.64050, 0.62908, 0.61715, 0.60517, 0.59374, 0.58309, 0.57318,\
        0.56399, 0.55550, 0.54768, 0.54046, 0.53381, 0.52765, 0.52190, 0.51649, 0.51132, 0.50631, 0.50138],\
        [0.43266, 0.43729, 0.44191, 0.44657, 0.45128, 0.45610, 0.46104, 0.46612, 0.47136, 0.47674, 0.48224,\
        0.48783, 0.49340, 0.49872, 0.50308, 0.50667, 0.50989, 0.51267, 0.51493, 0.51653, 0.51735, 0.51735,\
        0.51649, 0.51473, 0.51193, 0.50797, 0.50351, 0.50044, 0.49679, 0.49143, 0.48575, 0.48005, 0.47272,\
        0.46115, 0.44350, 0.41763, 0.38636, 0.35974, 0.33644, 0.31525, 0.30007, 0.29552, 0.29314, 0.29065,\
        0.29476, 0.30289, 0.33001, 0.37164, 0.42451, 0.47032, 0.51731, 0.59872, 0.66505, 0.72456, 0.79514,\
        0.85654, 0.88852, 0.90525, 0.91390, 0.92673, 0.93568, 0.93846, 0.93715, 0.93388, 0.92945, 0.92383,\
        0.91695, 0.90858, 0.89873, 0.88861, 0.87864, 0.86890, 0.85935, 0.84999, 0.84083, 0.83190, 0.82323,\
        0.81485, 0.80678, 0.79910, 0.79076, 0.78174, 0.77244, 0.76305, 0.75372, 0.74459, 0.73579, 0.72745,\
        0.71968, 0.71262, 0.70641, 0.70121, 0.69725, 0.69433, 0.69209, 0.69023, 0.68841, 0.68630, 0.68349,\
        0.67951, 0.67392, 0.66640, 0.65693, 0.64576, 0.63335, 0.62025, 0.60712, 0.59475, 0.58350, 0.57317,\
        0.56365, 0.55491, 0.54690, 0.53957, 0.53284, 0.52664, 0.52089, 0.51551, 0.51040, 0.50548, 0.50065],\
        [0.43459, 0.43927, 0.44393, 0.44860, 0.45332, 0.45810, 0.46298, 0.46798, 0.47310, 0.47832, 0.48364,\
        0.48901, 0.49433, 0.49948, 0.50427, 0.50852, 0.51232, 0.51566, 0.51846, 0.52063, 0.52208, 0.52272,\
        0.52252, 0.52142, 0.51940, 0.51665, 0.51380, 0.51150, 0.50867, 0.50458, 0.50019, 0.49677, 0.49252,\
        0.48373, 0.47069, 0.44871, 0.41499, 0.38562, 0.36220, 0.34143, 0.32629, 0.32047, 0.31929, 0.32058,\
        0.32934, 0.34476, 0.37184, 0.41453, 0.47101, 0.50946, 0.53638, 0.59269, 0.67679, 0.74456, 0.82412,\
        0.89899, 0.92660, 0.93943, 0.94616, 0.95205, 0.95608, 0.95640, 0.95340, 0.94817, 0.94142, 0.93340,\
        0.92416, 0.91379, 0.90285, 0.89221, 0.88207, 0.87218, 0.86240, 0.85279, 0.84338, 0.83422, 0.82533,\
        0.81675, 0.80859, 0.80097, 0.79217, 0.78289, 0.77343, 0.76389, 0.75439, 0.74509, 0.73612, 0.72763,\
        0.71976, 0.71265, 0.70646, 0.70140, 0.69774, 0.69531, 0.69368, 0.69252, 0.69153, 0.69035, 0.68852,\
        0.68544, 0.68046, 0.67307, 0.66322, 0.65126, 0.63782, 0.62360, 0.60937, 0.59602, 0.58408, 0.57318,\
        0.56324, 0.55421, 0.54600, 0.53854, 0.53173, 0.52550, 0.51977, 0.51443, 0.50940, 0.50458, 0.49988],\
        [0.43643, 0.44114, 0.44582, 0.45048, 0.45516, 0.45989, 0.46468, 0.46956, 0.47453, 0.47958, 0.48469,\
        0.48982, 0.49493, 0.50003, 0.50522, 0.51013, 0.51462, 0.51863, 0.52212, 0.52502, 0.52727, 0.52878,\
        0.52954, 0.52952, 0.52882, 0.52768, 0.52651, 0.52548, 0.52419, 0.52234, 0.52035, 0.51894, 0.51691,\
        0.51239, 0.50645, 0.49496, 0.46361, 0.42810, 0.40007, 0.37626, 0.35883, 0.35201, 0.35294, 0.35922,\
        0.37430, 0.40182, 0.43353, 0.48034, 0.51495, 0.53008, 0.54727, 0.60588, 0.70365, 0.76950, 0.85919,\
        0.93870, 0.96622, 0.97513, 0.97785, 0.97840, 0.97733, 0.97402, 0.96838, 0.96083, 0.95180, 0.94156,\
        0.93027, 0.91816, 0.90576, 0.89466, 0.88477, 0.87465, 0.86460, 0.85474, 0.84511, 0.83576, 0.82667,\
        0.81780, 0.80903, 0.80015, 0.79208, 0.78319, 0.77379, 0.76418, 0.75458, 0.74515, 0.73607, 0.72749,\
        0.71957, 0.71248, 0.70640, 0.70153, 0.69814, 0.69607, 0.69492, 0.69437, 0.69412, 0.69383, 0.69307,\
        0.69113, 0.68702, 0.67988, 0.66961, 0.65683, 0.64235, 0.62707, 0.61195, 0.59782, 0.58483, 0.57318,\
        0.56279, 0.55344, 0.54502, 0.53741, 0.53052, 0.52427, 0.51856, 0.51328, 0.50834, 0.50365, 0.49908],\
        [0.43821, 0.44294, 0.44760, 0.45224, 0.45687, 0.46152, 0.46622, 0.47098, 0.47581, 0.48071, 0.48566,\
        0.49064, 0.49571, 0.50098, 0.50648, 0.51190, 0.51702, 0.52174, 0.52599, 0.52973, 0.53291, 0.53547,\
        0.53739, 0.53870, 0.53951, 0.54006, 0.54062, 0.54138, 0.54219, 0.54285, 0.54330, 0.54356, 0.54322,\
        0.54206, 0.53960, 0.53054, 0.50474, 0.47050, 0.43747, 0.41000, 0.39423, 0.38954, 0.39551, 0.40742,\
        0.42846, 0.46025, 0.49391, 0.52183, 0.53493, 0.54109, 0.55956, 0.63188, 0.72853, 0.79481, 0.88633,\
        0.95716, 0.98953, 1.00080, 1.00273, 1.00053, 0.99601, 0.98956, 0.98132, 0.97153, 0.96046, 0.94838,\
        0.93552, 0.92226, 0.90939, 0.89845, 0.88718, 0.87643, 0.86601, 0.85588, 0.84605, 0.83653, 0.82733,\
        0.81836, 0.80955, 0.80092, 0.79220, 0.78314, 0.77369, 0.76402, 0.75433, 0.74483, 0.73567, 0.72705,\
        0.71913, 0.71210, 0.70617, 0.70155, 0.69840, 0.69657, 0.69576, 0.69566, 0.69598, 0.69643, 0.69666,\
        0.69607, 0.69325, 0.68645, 0.67574, 0.66214, 0.64666, 0.63028, 0.61403, 0.59890, 0.58514, 0.57304,\
        0.56228, 0.55264, 0.54399, 0.53622, 0.52924, 0.52296, 0.51728, 0.51208, 0.50726, 0.50270, 0.49830],\
        [0.43995, 0.44468, 0.44934, 0.45394, 0.45851, 0.46309, 0.46769, 0.47235, 0.47707, 0.48187, 0.48674,\
        0.49170, 0.49680, 0.50229, 0.50813, 0.51400, 0.51968, 0.52507, 0.53009, 0.53471, 0.53889, 0.54259,\
        0.54579, 0.54852, 0.55086, 0.55301, 0.55526, 0.55788, 0.56093, 0.56402, 0.56653, 0.56801, 0.56832,\
        0.56789, 0.56579, 0.55732, 0.53746, 0.50739, 0.47071, 0.44345, 0.43224, 0.43351, 0.44402, 0.46065,\
        0.48252, 0.50657, 0.52607, 0.53867, 0.54628, 0.55872, 0.59061, 0.66764, 0.75006, 0.81946, 0.90553,\
        0.96616, 1.00019, 1.01512, 1.01871, 1.01610, 1.00997, 1.00152, 0.99134, 0.97977, 0.96713, 0.95369,\
        0.93975, 0.92575, 0.91225, 0.89966, 0.88838, 0.87736, 0.86661, 0.85621, 0.84618, 0.83654, 0.82729,\
        0.81839, 0.80975, 0.80115, 0.79219, 0.78285, 0.77324, 0.76347, 0.75372, 0.74416, 0.73497, 0.72634,\
        0.71844, 0.71149, 0.70569, 0.70128, 0.69833, 0.69672, 0.69616, 0.69634, 0.69700, 0.69788, 0.69874,\
        0.69928, 0.69838, 0.69200, 0.68098, 0.66681, 0.65055, 0.63316, 0.61560, 0.59901, 0.58522, 0.57291,\
        0.56181, 0.55184, 0.54292, 0.53497, 0.52789, 0.52160, 0.51596, 0.51086, 0.50617, 0.50178, 0.49755],\
        [0.44168, 0.44642, 0.45106, 0.45562, 0.46014, 0.46465, 0.46917, 0.47374, 0.47839, 0.48314, 0.48805,\
        0.49314, 0.49842, 0.50412, 0.51023, 0.51645, 0.52262, 0.52861, 0.53437, 0.53985, 0.54503, 0.54989,\
        0.55440, 0.55854, 0.56231, 0.56588, 0.56955, 0.57373, 0.57880, 0.58417, 0.58838, 0.59074, 0.59082,\
        0.59009, 0.58865, 0.58309, 0.56880, 0.54474, 0.50808, 0.48107, 0.47432, 0.48023, 0.49099, 0.50593,\
        0.52117, 0.53452, 0.54443, 0.55138, 0.56073, 0.58588, 0.63284, 0.70347, 0.78079, 0.84915, 0.91433,\
        0.96999, 1.00441, 1.02161, 1.02702, 1.02504, 1.01854, 1.00917, 0.99786, 0.98516, 0.97149, 0.95717,\
        0.94256, 0.92805, 0.91409, 0.90120, 0.88909, 0.87750, 0.86637, 0.85570, 0.84548, 0.83573, 0.82646,\
        0.81763, 0.80913, 0.80060, 0.79157, 0.78211, 0.77239, 0.76255, 0.75277, 0.74319, 0.73401, 0.72539,\
        0.71754, 0.71066, 0.70495, 0.70066, 0.69793, 0.69654, 0.69616, 0.69649, 0.69727, 0.69826, 0.69921,\
        0.69984, 0.70000, 0.69523, 0.68474, 0.67054, 0.65391, 0.63599, 0.61789, 0.60094, 0.58613, 0.57312,\
        0.56146, 0.55105, 0.54181, 0.53365, 0.52648, 0.52017, 0.51460, 0.50962, 0.50510, 0.50089, 0.49687],\
        [0.44344, 0.44818, 0.45280, 0.45733, 0.46180, 0.46625, 0.47071, 0.47522, 0.47983, 0.48458, 0.48955,\
        0.49485, 0.50051, 0.50639, 0.51268, 0.51921, 0.52580, 0.53232, 0.53873, 0.54500, 0.55111, 0.55708,\
        0.56286, 0.56836, 0.57344, 0.57817, 0.58294, 0.58802, 0.59441, 0.60168, 0.60701, 0.61019, 0.61100,\
        0.61101, 0.61093, 0.60861, 0.60139, 0.59131, 0.56204, 0.53278, 0.52128, 0.52425, 0.53111, 0.53985,\
        0.54861, 0.55668, 0.56445, 0.57431, 0.59089, 0.62801, 0.68358, 0.75337, 0.82782, 0.89512, 0.93828,\
        0.97825, 1.00752, 1.02386, 1.02979, 1.02837, 1.02203, 1.01244, 1.00069, 0.98749, 0.97332, 0.95859,\
        0.94365, 0.92889, 0.91466, 0.90122, 0.88871, 0.87672, 0.86526, 0.85433, 0.84394, 0.83408, 0.82475,\
        0.81595, 0.80761, 0.79922, 0.79027, 0.78083, 0.77110, 0.76127, 0.75149, 0.74195, 0.73281, 0.72425,\
        0.71648, 0.70968, 0.70408, 0.69991, 0.69739, 0.69616, 0.69589, 0.69629, 0.69709, 0.69806, 0.69898,\
        0.69977, 0.70031, 0.69666, 0.68716, 0.67330, 0.65664, 0.63856, 0.62032, 0.60301, 0.58743, 0.57351,\
        0.56112, 0.55018, 0.54059, 0.53223, 0.52497, 0.51867, 0.51320, 0.50838, 0.50405, 0.50006, 0.49626],\
        [0.44524, 0.44999, 0.45460, 0.45911, 0.46354, 0.46795, 0.47236, 0.47684, 0.48144, 0.48620, 0.49119,\
        0.49652, 0.50243, 0.50873, 0.51534, 0.52218, 0.52913, 0.53611, 0.54307, 0.54999, 0.55690, 0.56382,\
        0.57076, 0.57758, 0.58391, 0.58940, 0.59525, 0.60095, 0.60784, 0.61563, 0.62197, 0.62632, 0.62894,\
        0.63080, 0.63241, 0.63306, 0.63256, 0.62814, 0.61054, 0.58533, 0.57098, 0.56547, 0.56893, 0.57070,\
        0.57547, 0.58290, 0.59519, 0.61642, 0.64812, 0.69211, 0.74270, 0.80495, 0.86498, 0.91734, 0.95770,\
        0.98777, 1.01005, 1.02342, 1.02856, 1.02717, 1.02105, 1.01163, 0.99996, 0.98677, 0.97258, 0.95781,\
        0.94284, 0.92807, 0.91381, 0.90009, 0.88723, 0.87494, 0.86322, 0.85209, 0.84157, 0.83163, 0.82226,\
        0.81338, 0.80511, 0.79719, 0.78836, 0.77903, 0.76939, 0.75963, 0.74993, 0.74046, 0.73141, 0.72296,\
        0.71530, 0.70865, 0.70321, 0.69924, 0.69685, 0.69572, 0.69552, 0.69596, 0.69678, 0.69777, 0.69881,\
        0.70004, 0.70090, 0.69785, 0.68886, 0.67523, 0.65858, 0.64039, 0.62206, 0.60470, 0.58841, 0.57355,\
        0.56049, 0.54909, 0.53919, 0.53065, 0.52333, 0.51709, 0.51175, 0.50714, 0.50305, 0.49932, 0.49576],\
        [0.44710, 0.45187, 0.45649, 0.46099, 0.46540, 0.46977, 0.47416, 0.47863, 0.48324, 0.48806, 0.49315,\
        0.49855, 0.50459, 0.51124, 0.51816, 0.52529, 0.53256, 0.53990, 0.54728, 0.55470, 0.56219, 0.56982,\
        0.57765, 0.58564, 0.59344, 0.59933, 0.60563, 0.61261, 0.62001, 0.62776, 0.63468, 0.64017, 0.64437,\
        0.64783, 0.65098, 0.65371, 0.65528, 0.65308, 0.64368, 0.63048, 0.61923, 0.60911, 0.61019, 0.60363,\
        0.60672, 0.61658, 0.63745, 0.67030, 0.70709, 0.74642, 0.78753, 0.83995, 0.88969, 0.93278, 0.96708,\
        0.99231, 1.00984, 1.02018, 1.02394, 1.02221, 1.01625, 1.00719, 0.99595, 0.98318, 0.96936, 0.95489,\
        0.94013, 0.92548, 0.91141, 0.89811, 0.88476, 0.87214, 0.86022, 0.84899, 0.83841, 0.82848, 0.81920,\
        0.81046, 0.80179, 0.79424, 0.78591, 0.77679, 0.76729, 0.75767, 0.74810, 0.73876, 0.72984, 0.72153,\
        0.71404, 0.70758, 0.70235, 0.69860, 0.69634, 0.69529, 0.69514, 0.69563, 0.69654, 0.69769, 0.69898,\
        0.70059, 0.70177, 0.69893, 0.69003, 0.67642, 0.65967, 0.64120, 0.62237, 0.60439, 0.58778, 0.57269,\
        0.55934, 0.54765, 0.53752, 0.52886, 0.52154, 0.51541, 0.51027, 0.50591, 0.50211, 0.49867, 0.49539],\
        [0.44904, 0.45385, 0.45849, 0.46299, 0.46740, 0.47176, 0.47613, 0.48059, 0.48524, 0.49017, 0.49547,\
        0.50117, 0.50734, 0.51407, 0.52118, 0.52853, 0.53603, 0.54364, 0.55131, 0.55905, 0.56689, 0.57492,\
        0.58320, 0.59181, 0.60039, 0.60783, 0.61479, 0.62243, 0.63041, 0.63839, 0.64578, 0.65213, 0.65743,\
        0.66197, 0.66609, 0.66981, 0.67250, 0.67307, 0.67141, 0.66934, 0.66712, 0.66244, 0.65549, 0.64674,\
        0.64336, 0.65082, 0.67799, 0.71704, 0.75405, 0.79009, 0.82725, 0.87107, 0.91109, 0.94437, 0.97175,\
        0.99221, 1.00608, 1.01386, 1.01622, 1.01400, 1.00816, 0.99961, 0.98906, 0.97705, 0.96395, 0.95007,\
        0.93571, 0.92123, 0.90697, 0.89356, 0.88070, 0.86819, 0.85629, 0.84505, 0.83452, 0.82467, 0.81559,\
        0.80746, 0.80051, 0.79201, 0.78324, 0.77418, 0.76487, 0.75545, 0.74608, 0.73691, 0.72815, 0.72000,\
        0.71270, 0.70646, 0.70149, 0.69795, 0.69582, 0.69484, 0.69477, 0.69535, 0.69639, 0.69776, 0.69946,\
        0.70144, 0.70239, 0.69945, 0.69056, 0.67691, 0.66001, 0.64122, 0.62175, 0.60273, 0.58612, 0.57120,\
        0.55775, 0.54587, 0.53559, 0.52686, 0.51959, 0.51362, 0.50874, 0.50471, 0.50126, 0.49816, 0.49519],\
        [0.45108, 0.45593, 0.46061, 0.46514, 0.46956, 0.47391, 0.47827, 0.48272, 0.48739, 0.49243, 0.49791,\
        0.50384, 0.51023, 0.51710, 0.52434, 0.53185, 0.53953, 0.54732, 0.55516, 0.56307, 0.57107, 0.57920,\
        0.58755, 0.59619, 0.60521, 0.61388, 0.62217, 0.63057, 0.63910, 0.64750, 0.65541, 0.66248, 0.66854,\
        0.67373, 0.67843, 0.68300, 0.68713, 0.69058, 0.69504, 0.70197, 0.70871, 0.70935, 0.70104, 0.69139,\
        0.68377, 0.68172, 0.70938, 0.75394, 0.79079, 0.82501, 0.85786, 0.89286, 0.92503, 0.95084, 0.97214,\
        0.98827, 0.99899, 1.00460, 1.00567, 1.00295, 0.99726, 0.98933, 0.97971, 0.96878, 0.95674, 0.94376,\
        0.93003, 0.91596, 0.90202, 0.88855, 0.87573, 0.86334, 0.85150, 0.84037, 0.82997, 0.82027, 0.81126,\
        0.80309, 0.79595, 0.78840, 0.77998, 0.77119, 0.76218, 0.75304, 0.74391, 0.73496, 0.72637, 0.71838,\
        0.71124, 0.70522, 0.70052, 0.69720, 0.69519, 0.69433, 0.69436, 0.69506, 0.69624, 0.69777, 0.69964,\
        0.70177, 0.70255, 0.69936, 0.69044, 0.67678, 0.65981, 0.64088, 0.62131, 0.60241, 0.58515, 0.56973,\
        0.55599, 0.54388, 0.53343, 0.52464, 0.51745, 0.51171, 0.50718, 0.50356, 0.50053, 0.49781, 0.49517],\
        [0.45321, 0.45813, 0.46286, 0.46744, 0.47189, 0.47627, 0.48062, 0.48504, 0.48968, 0.49476, 0.50040,\
        0.50654, 0.51316, 0.52020, 0.52759, 0.53524, 0.54304, 0.55094, 0.55889, 0.56686, 0.57488, 0.58302,\
        0.59134, 0.60002, 0.60937, 0.61878, 0.62800, 0.63714, 0.64623, 0.65515, 0.66365, 0.67144, 0.67823,\
        0.68391, 0.68907, 0.69475, 0.70122, 0.70813, 0.71735, 0.72866, 0.73849, 0.74213, 0.73826, 0.73124,\
        0.72482, 0.72456, 0.75212, 0.79648, 0.82595, 0.85272, 0.87783, 0.90366, 0.92913, 0.95087, 0.96821,\
        0.98099, 0.98911, 0.99282, 0.99269, 0.98949, 0.98398, 0.97679, 0.96834, 0.95879, 0.94817, 0.93642,\
        0.92359, 0.90996, 0.89640, 0.88318, 0.87027, 0.85782, 0.84601, 0.83500, 0.82487, 0.81551, 0.80665,\
        0.79851, 0.79130, 0.78404, 0.77620, 0.76791, 0.75931, 0.75053, 0.74171, 0.73299, 0.72458, 0.71671,\
        0.70967, 0.70380, 0.69931, 0.69621, 0.69439, 0.69365, 0.69381, 0.69464, 0.69597, 0.69763, 0.69956,\
        0.70160, 0.70223, 0.69883, 0.68985, 0.67621, 0.65922, 0.64022, 0.62060, 0.60176, 0.58412, 0.56827,\
        0.55419, 0.54178, 0.53110, 0.52222, 0.51513, 0.50970, 0.50561, 0.50250, 0.49996, 0.49766, 0.49536],\
        [0.45544, 0.46044, 0.46524, 0.46988, 0.47440, 0.47884, 0.48322, 0.48759, 0.49213, 0.49718, 0.50294,\
        0.50931, 0.51614, 0.52336, 0.53088, 0.53864, 0.54654, 0.55452, 0.56253, 0.57052, 0.57851, 0.58662,\
        0.59500, 0.60386, 0.61331, 0.62302, 0.63273, 0.64239, 0.65196, 0.66138, 0.67050, 0.67912, 0.68691,\
        0.69328, 0.69897, 0.70619, 0.71500, 0.72518, 0.73718, 0.75022, 0.76150, 0.76773, 0.76799, 0.76531,\
        0.76356, 0.77154, 0.79643, 0.83201, 0.85775, 0.87569, 0.89121, 0.90926, 0.92903, 0.94681, 0.96106,\
        0.97122, 0.97715, 0.97913, 0.97782, 0.97408, 0.96876, 0.96242, 0.95535, 0.94752, 0.93866, 0.92850,\
        0.91695, 0.90411, 0.89071, 0.87746, 0.86437, 0.85173, 0.83984, 0.82896, 0.81919, 0.81048, 0.80228,\
        0.79431, 0.78698, 0.77980, 0.77235, 0.76454, 0.75640, 0.74804, 0.73957, 0.73112, 0.72288, 0.71509,\
        0.70806, 0.70223, 0.69791, 0.69502, 0.69336, 0.69274, 0.69300, 0.69397, 0.69544, 0.69722, 0.69910,\
        0.70098, 0.70157, 0.69803, 0.68895, 0.67532, 0.65839, 0.63935, 0.61941, 0.60002, 0.58266, 0.56683,\
        0.55244, 0.53965, 0.52865, 0.51961, 0.51263, 0.50758, 0.50408, 0.50158, 0.49959, 0.49775, 0.49578],\
        [0.45775, 0.46285, 0.46773, 0.47246, 0.47707, 0.48161, 0.48610, 0.49051, 0.49486, 0.49980, 0.50572,\
        0.51226, 0.51923, 0.52657, 0.53419, 0.54202, 0.55001, 0.55806, 0.56611, 0.57411, 0.58206, 0.59007,\
        0.59849, 0.60744, 0.61690, 0.62667, 0.63657, 0.64650, 0.65640, 0.66620, 0.67582, 0.68517, 0.69416,\
        0.70195, 0.70870, 0.71711, 0.72735, 0.73927, 0.75265, 0.76660, 0.77915, 0.78782, 0.79066, 0.79315,\
        0.79504, 0.80752, 0.83133, 0.85981, 0.88210, 0.89274, 0.90037, 0.91205, 0.92656, 0.94056, 0.95206,\
        0.95999, 0.96398, 0.96427, 0.96165, 0.95725, 0.95206, 0.94664, 0.94114, 0.93533, 0.92860, 0.92027,\
        0.91017, 0.89845, 0.88516, 0.87149, 0.85800, 0.84507, 0.83304, 0.82221, 0.81275, 0.80474, 0.79740,\
        0.78987, 0.78269, 0.77571, 0.76862, 0.76126, 0.75360, 0.74567, 0.73758, 0.72943, 0.72140, 0.71370,\
        0.70661, 0.70063, 0.69647, 0.69371, 0.69208, 0.69147, 0.69178, 0.69285, 0.69449, 0.69645, 0.69842,\
        0.70014, 0.70081, 0.69703, 0.68776, 0.67414, 0.65741, 0.63865, 0.61892, 0.59956, 0.58202, 0.56583,\
        0.55095, 0.53763, 0.52615, 0.51684, 0.50994, 0.50538, 0.50263, 0.50088, 0.49950, 0.49809, 0.49644],\
        [0.46014, 0.46534, 0.47031, 0.47512, 0.47982, 0.48449, 0.48920, 0.49389, 0.49830, 0.50299, 0.50890,\
        0.51544, 0.52244, 0.52980, 0.53745, 0.54534, 0.55338, 0.56150, 0.56962, 0.57767, 0.58562, 0.59347,\
        0.60159, 0.61062, 0.62002, 0.62970, 0.63958, 0.64958, 0.65962, 0.66965, 0.67956, 0.68927, 0.69872,\
        0.70775, 0.71635, 0.72589, 0.73694, 0.74948, 0.76316, 0.77724, 0.79040, 0.80047, 0.80518, 0.81048,\
        0.81776, 0.83165, 0.85340, 0.87800, 0.89790, 0.90117, 0.90422, 0.91155, 0.92191, 0.93285, 0.94214,\
        0.94826, 0.95052, 0.94908, 0.94490, 0.93957, 0.93447, 0.92999, 0.92609, 0.92257, 0.91829, 0.91210,\
        0.90278, 0.89186, 0.87879, 0.86487, 0.85104, 0.83784, 0.82569, 0.81494, 0.80570, 0.79800, 0.79141,\
        0.78479, 0.77821, 0.77168, 0.76504, 0.75816, 0.75097, 0.74350, 0.73580, 0.72799, 0.72022, 0.71269,\
        0.70567, 0.69964, 0.69533, 0.69234, 0.69049, 0.68971, 0.68993, 0.69103, 0.69281, 0.69503, 0.69736,\
        0.69938, 0.70028, 0.69573, 0.68597, 0.67244, 0.65615, 0.63804, 0.61913, 0.60043, 0.58228, 0.56534,\
        0.54979, 0.53581, 0.52374, 0.51399, 0.50706, 0.50315, 0.50141, 0.50049, 0.49969, 0.49868, 0.49729],\
        [0.46259, 0.46790, 0.47296, 0.47782, 0.48258, 0.48733, 0.49220, 0.49726, 0.50212, 0.50684, 0.51244,\
        0.51879, 0.52568, 0.53298, 0.54061, 0.54849, 0.55656, 0.56474, 0.57294, 0.58110, 0.58916, 0.59709,\
        0.60509, 0.61362, 0.62266, 0.63206, 0.64176, 0.65167, 0.66172, 0.67183, 0.68193, 0.69190, 0.70164,\
        0.71147, 0.72134, 0.73179, 0.74322, 0.75565, 0.76876, 0.78194, 0.79429, 0.80414, 0.81148, 0.81938,\
        0.82980, 0.84428, 0.86276, 0.88170, 0.89467, 0.89860, 0.90119, 0.90662, 0.91475, 0.92393, 0.93188,\
        0.93671, 0.93758, 0.93449, 0.92842, 0.92179, 0.91680, 0.91330, 0.91062, 0.90955, 0.90752, 0.90422,\
        0.89709, 0.88487, 0.87144, 0.85735, 0.84339, 0.83012, 0.81799, 0.80749, 0.79880, 0.79163, 0.78546,\
        0.77958, 0.77370, 0.76775, 0.76164, 0.75526, 0.74857, 0.74155, 0.73424, 0.72676, 0.71926, 0.71198,\
        0.70521, 0.69925, 0.69437, 0.69075, 0.68839, 0.68723, 0.68720, 0.68818, 0.68999, 0.69238, 0.69500,\
        0.69735, 0.69784, 0.69258, 0.68283, 0.66982, 0.65430, 0.63705, 0.61887, 0.60045, 0.58234, 0.56503,\
        0.54891, 0.53431, 0.52161, 0.51130, 0.50410, 0.50101, 0.50060, 0.50045, 0.50011, 0.49942, 0.49824],\
        [0.46510, 0.47053, 0.47567, 0.48057, 0.48532, 0.49004, 0.49489, 0.49991, 0.50504, 0.51022, 0.51584,\
        0.52206, 0.52881, 0.53599, 0.54353, 0.55137, 0.55943, 0.56764, 0.57590, 0.58413, 0.59226, 0.60032,\
        0.60801, 0.61606, 0.62462, 0.63364, 0.64306, 0.65280, 0.66277, 0.67287, 0.68305, 0.69332, 0.70353,\
        0.71364, 0.72412, 0.73501, 0.74643, 0.75832, 0.77034, 0.78194, 0.79242, 0.80162, 0.81098, 0.82117,\
        0.83302, 0.84691, 0.86209, 0.87607, 0.88567, 0.89041, 0.89361, 0.89832, 0.90540, 0.91407, 0.92159,\
        0.92563, 0.92562, 0.92146, 0.91347, 0.90506, 0.90034, 0.89783, 0.89596, 0.89646, 0.89566, 0.89298,\
        0.88708, 0.87590, 0.86269, 0.84883, 0.83510, 0.82217, 0.81039, 0.80015, 0.79235, 0.78578, 0.78004,\
        0.77469, 0.76940, 0.76403, 0.75846, 0.75259, 0.74638, 0.73979, 0.73285, 0.72565, 0.71835, 0.71117,\
        0.70443, 0.69830, 0.69283, 0.68850, 0.68547, 0.68376, 0.68331, 0.68401, 0.68565, 0.68793, 0.69035,\
        0.69208, 0.69146, 0.68673, 0.67799, 0.66608, 0.65170, 0.63548, 0.61806, 0.60007, 0.58218, 0.56478,\
        0.54830, 0.53322, 0.52006, 0.50937, 0.50191, 0.49940, 0.50012, 0.50053, 0.50055, 0.50013, 0.49916],\
        [0.46764, 0.47321, 0.47844, 0.48338, 0.48810, 0.49271, 0.49742, 0.50249, 0.50776, 0.51318, 0.51891,\
        0.52506, 0.53167, 0.53870, 0.54610, 0.55383, 0.56182, 0.57001, 0.57830, 0.58653, 0.59448, 0.60218,\
        0.60983, 0.61754, 0.62566, 0.63430, 0.64344, 0.65300, 0.66287, 0.67293, 0.68306, 0.69318, 0.70349,\
        0.71424, 0.72510, 0.73609, 0.74723, 0.75840, 0.76930, 0.77949, 0.78866, 0.79725, 0.80693, 0.81810,\
        0.83019, 0.84286, 0.85528, 0.86603, 0.87380, 0.87882, 0.88290, 0.88794, 0.89487, 0.90366, 0.91121,\
        0.91442, 0.91402, 0.91010, 0.90183, 0.89221, 0.88678, 0.88407, 0.88289, 0.88292, 0.88232, 0.87966,\
        0.87392, 0.86445, 0.85248, 0.83944, 0.82637, 0.81415, 0.80368, 0.79425, 0.78669, 0.78053, 0.77518,\
        0.77024, 0.76544, 0.76058, 0.75551, 0.75013, 0.74437, 0.73817, 0.73155, 0.72455, 0.71730, 0.70998,\
        0.70286, 0.69621, 0.69023, 0.68523, 0.68148, 0.67910, 0.67810, 0.67836, 0.67965, 0.68158, 0.68355,\
        0.68462, 0.68359, 0.67943, 0.67191, 0.66141, 0.64845, 0.63350, 0.61704, 0.59961, 0.58208, 0.56466,\
        0.54793, 0.53254, 0.51914, 0.50846, 0.50139, 0.49882, 0.49930, 0.50023, 0.50064, 0.50056, 0.49987],\
        [0.47020, 0.47594, 0.48129, 0.48629, 0.49102, 0.49556, 0.50012, 0.50515, 0.51047, 0.51595, 0.52167,\
        0.52773, 0.53417, 0.54100, 0.54821, 0.55575, 0.56359, 0.57168, 0.57993, 0.58820, 0.59621, 0.60356,\
        0.61065, 0.61790, 0.62563, 0.63396, 0.64289, 0.65232, 0.66215, 0.67222, 0.68240, 0.69261, 0.70306,\
        0.71392, 0.72478, 0.73561, 0.74635, 0.75686, 0.76688, 0.77616, 0.78464, 0.79279, 0.80193, 0.81275,\
        0.82407, 0.83515, 0.84528, 0.85371, 0.86008, 0.86495, 0.86963, 0.87545, 0.88303, 0.89195, 0.89918,\
        0.90143, 0.90087, 0.89751, 0.89010, 0.88115, 0.87481, 0.87140, 0.86997, 0.86950, 0.86859, 0.86594,\
        0.86063, 0.85236, 0.84174, 0.82980, 0.81766, 0.80615, 0.79655, 0.78864, 0.78167, 0.77584, 0.77080,\
        0.76621, 0.76181, 0.75738, 0.75277, 0.74783, 0.74247, 0.73662, 0.73025, 0.72337, 0.71605, 0.70842,\
        0.70074, 0.69340, 0.68666, 0.68086, 0.67629, 0.67315, 0.67149, 0.67122, 0.67208, 0.67362, 0.67520,\
        0.67596, 0.67496, 0.67147, 0.66518, 0.65618, 0.64476, 0.63127, 0.61609, 0.59959, 0.58221, 0.56466,\
        0.54770, 0.53207, 0.51854, 0.50790, 0.50105, 0.49821, 0.49814, 0.49920, 0.50005, 0.50051, 0.50024],\
        [0.47273, 0.47868, 0.48419, 0.48929, 0.49408, 0.49867, 0.50326, 0.50809, 0.51322, 0.51856, 0.52415,\
        0.53003, 0.53625, 0.54283, 0.54977, 0.55704, 0.56462, 0.57246, 0.58051, 0.58865, 0.59665, 0.60371,\
        0.61020, 0.61701, 0.62444, 0.63258, 0.64141, 0.65082, 0.66068, 0.67086, 0.68125, 0.69183, 0.70249,\
        0.71297, 0.72351, 0.73403, 0.74439, 0.75439, 0.76386, 0.77266, 0.78085, 0.78884, 0.79736, 0.80694,\
        0.81672, 0.82597, 0.83402, 0.84044, 0.84534, 0.84956, 0.85433, 0.86065, 0.86856, 0.87692, 0.88333,\
        0.88590, 0.88526, 0.88174, 0.87536, 0.86802, 0.86215, 0.85865, 0.85707, 0.85640, 0.85540, 0.85292,\
        0.84818, 0.84093, 0.83148, 0.82065, 0.80955, 0.79961, 0.79064, 0.78330, 0.77699, 0.77154, 0.76679,\
        0.76249, 0.75842, 0.75437, 0.75014, 0.74560, 0.74060, 0.73505, 0.72886, 0.72202, 0.71456, 0.70653,\
        0.69814, 0.68991, 0.68219, 0.67540, 0.66989, 0.66589, 0.66352, 0.66269, 0.66316, 0.66443, 0.66584,\
        0.66661, 0.66597, 0.66327, 0.65820, 0.65066, 0.64078, 0.62877, 0.61487, 0.59926, 0.58212, 0.56452,\
        0.54738, 0.53155, 0.51787, 0.50713, 0.50009, 0.49696, 0.49658, 0.49757, 0.49889, 0.49995, 0.50021],\
        [0.47518, 0.48138, 0.48708, 0.49230, 0.49715, 0.50179, 0.50633, 0.51100, 0.51588, 0.52098, 0.52633,\
        0.53195, 0.53788, 0.54415, 0.55074, 0.55765, 0.56486, 0.57230, 0.57992, 0.58757, 0.59499, 0.60180,\
        0.60812, 0.61479, 0.62211, 0.63023, 0.63910, 0.64861, 0.65858, 0.66889, 0.67942, 0.69013, 0.70074,\
        0.71106, 0.72138, 0.73166, 0.74174, 0.75142, 0.76058, 0.76916, 0.77725, 0.78516, 0.79323, 0.80153,\
        0.80923, 0.81676, 0.82283, 0.82725, 0.83043, 0.83342, 0.83774, 0.84427, 0.85243, 0.86049, 0.86648,\
        0.86915, 0.86849, 0.86495, 0.85934, 0.85323, 0.84831, 0.84541, 0.84426, 0.84390, 0.84321, 0.84118,\
        0.83714, 0.83084, 0.82249, 0.81273, 0.80250, 0.79340, 0.78556, 0.77865, 0.77268, 0.76751, 0.76299,\
        0.75894, 0.75515, 0.75141, 0.74752, 0.74332, 0.73863, 0.73331, 0.72725, 0.72036, 0.71266, 0.70425,\
        0.69506, 0.68566, 0.67679, 0.66889, 0.66233, 0.65742, 0.65432, 0.65299, 0.65318, 0.65437, 0.65588,\
        0.65695, 0.65688, 0.65508, 0.65119, 0.64505, 0.63661, 0.62596, 0.61316, 0.59823, 0.58151, 0.56398,\
        0.54674, 0.53075, 0.51690, 0.50597, 0.49876, 0.49537, 0.49463, 0.49548, 0.49703, 0.49864, 0.49926],\
        [0.47746, 0.48395, 0.48990, 0.49524, 0.50006, 0.50470, 0.50918, 0.51367, 0.51830, 0.52311, 0.52815,\
        0.53346, 0.53905, 0.54494, 0.55114, 0.55763, 0.56437, 0.57130, 0.57834, 0.58534, 0.59211, 0.59846,\
        0.60478, 0.61148, 0.61883, 0.62707, 0.63614, 0.64584, 0.65598, 0.66639, 0.67695, 0.68751, 0.69794,\
        0.70829, 0.71859, 0.72877, 0.73868, 0.74817, 0.75715, 0.76561, 0.77363, 0.78138, 0.78894, 0.79589,\
        0.80261, 0.80858, 0.81250, 0.81502, 0.81635, 0.81759, 0.82100, 0.82782, 0.83660, 0.84480, 0.85058,\
        0.85306, 0.85220, 0.84856, 0.84325, 0.83783, 0.83385, 0.83202, 0.83188, 0.83236, 0.83236, 0.83101,\
        0.82777, 0.82244, 0.81513, 0.80635, 0.79729, 0.78873, 0.78114, 0.77445, 0.76863, 0.76360, 0.75924,\
        0.75538, 0.75182, 0.74835, 0.74475, 0.74083, 0.73640, 0.73127, 0.72526, 0.71823, 0.71009, 0.70088,\
        0.69085, 0.68043, 0.67042, 0.66139, 0.65377, 0.64793, 0.64411, 0.64237, 0.64246, 0.64382, 0.64568,\
        0.64728, 0.64793, 0.64708, 0.64434, 0.63948, 0.63234, 0.62288, 0.61111, 0.59693, 0.58041, 0.56294,\
        0.54563, 0.52952, 0.51557, 0.50460, 0.49721, 0.49342, 0.49225, 0.49275, 0.49407, 0.49540, 0.49561],\
        [0.47944, 0.48624, 0.49251, 0.49812, 0.50291, 0.50738, 0.51170, 0.51599, 0.52036, 0.52487, 0.52958,\
        0.53453, 0.53975, 0.54524, 0.55101, 0.55704, 0.56328, 0.56967, 0.57611, 0.58249, 0.58870, 0.59475,\
        0.60096, 0.60750, 0.61487, 0.62340, 0.63282, 0.64280, 0.65314, 0.66365, 0.67418, 0.68462, 0.69494,\
        0.70518, 0.71546, 0.72561, 0.73543, 0.74478, 0.75361, 0.76193, 0.76980, 0.77731, 0.78449, 0.79136,\
        0.79792, 0.80093, 0.80307, 0.80427, 0.80426, 0.80365, 0.80593, 0.81353, 0.82327, 0.83155, 0.83698,\
        0.83899, 0.83767, 0.83365, 0.82815, 0.82287, 0.81966, 0.81924, 0.82060, 0.82225, 0.82315, 0.82258,\
        0.82012, 0.81569, 0.80943, 0.80150, 0.79296, 0.78474, 0.77717, 0.77045, 0.76461, 0.75962, 0.75536,\
        0.75165, 0.74828, 0.74503, 0.74167, 0.73798, 0.73375, 0.72875, 0.72274, 0.71553, 0.70686, 0.69673,\
        0.68565, 0.67423, 0.66317, 0.65308, 0.64444, 0.63769, 0.63321, 0.63119, 0.63142, 0.63320, 0.63565,\
        0.63793, 0.63936, 0.63944, 0.63778, 0.63408, 0.62810, 0.61964, 0.60853, 0.59478, 0.57874, 0.56142,\
        0.54404, 0.52777, 0.51365, 0.50270, 0.49531, 0.49109, 0.48942, 0.48940, 0.49019, 0.49094, 0.49085],\
        [0.48102, 0.48806, 0.49456, 0.50037, 0.50525, 0.50962, 0.51378, 0.51787, 0.52198, 0.52620, 0.53057,\
        0.53516, 0.53999, 0.54508, 0.55042, 0.55600, 0.56176, 0.56764, 0.57356, 0.57944, 0.58522, 0.59097,\
        0.59688, 0.60301, 0.61063, 0.61969, 0.62952, 0.63980, 0.65034, 0.66094, 0.67148, 0.68184, 0.69197,\
        0.70203, 0.71239, 0.72249, 0.73216, 0.74133, 0.74996, 0.75805, 0.76564, 0.77277, 0.77944, 0.78552,\
        0.79032, 0.79288, 0.79424, 0.79481, 0.79457, 0.79426, 0.79667, 0.80478, 0.81416, 0.82166, 0.82633,\
        0.82766, 0.82572, 0.82117, 0.81521, 0.80965, 0.80690, 0.80827, 0.81128, 0.81403, 0.81577, 0.81591,\
        0.81402, 0.81014, 0.80478, 0.79766, 0.78922, 0.78087, 0.77314, 0.76627, 0.76034, 0.75533, 0.75113,\
        0.74755, 0.74435, 0.74129, 0.73812, 0.73460, 0.73050, 0.72557, 0.71947, 0.71198, 0.70303, 0.69189,\
        0.67964, 0.66721, 0.65524, 0.64426, 0.63476, 0.62718, 0.62207, 0.61989, 0.62055, 0.62306, 0.62623,\
        0.62920, 0.63138, 0.63230, 0.63160, 0.62894, 0.62401, 0.61652, 0.60613, 0.59270, 0.57683, 0.55953,\
        0.54204, 0.52557, 0.51123, 0.50000, 0.49264, 0.48830, 0.48622, 0.48565, 0.48584, 0.48605, 0.48566],\
        [0.48211, 0.48933, 0.49593, 0.50176, 0.50680, 0.51124, 0.51533, 0.51926, 0.52314, 0.52706, 0.53111,\
        0.53535, 0.53980, 0.54450, 0.54944, 0.55461, 0.55995, 0.56542, 0.57094, 0.57645, 0.58193, 0.58741,\
        0.59294, 0.59890, 0.60710, 0.61656, 0.62666, 0.63715, 0.64783, 0.65853, 0.66911, 0.67948, 0.68963,\
        0.69970, 0.70978, 0.71960, 0.72900, 0.73788, 0.74620, 0.75394, 0.76111, 0.76769, 0.77361, 0.77864,\
        0.78238, 0.78459, 0.78573, 0.78630, 0.78684, 0.78851, 0.79326, 0.80113, 0.80832, 0.81458, 0.81842,\
        0.81908, 0.81660, 0.81172, 0.80562, 0.80015, 0.79802, 0.80077, 0.80464, 0.80783, 0.81018, 0.81088,\
        0.80927, 0.80526, 0.79980, 0.79313, 0.78491, 0.77652, 0.76863, 0.76158, 0.75553, 0.75049, 0.74636,\
        0.74293, 0.73991, 0.73702, 0.73396, 0.73051, 0.72646, 0.72154, 0.71530, 0.70718, 0.69725, 0.68555,\
        0.67260, 0.65947, 0.64686, 0.63532, 0.62528, 0.61711, 0.61142, 0.60918, 0.61063, 0.61406, 0.61792,\
        0.62143, 0.62416, 0.62576, 0.62587, 0.62411, 0.62010, 0.61346, 0.60400, 0.59084, 0.57481, 0.55741,\
        0.53978, 0.52309, 0.50862, 0.49751, 0.48999, 0.48541, 0.48291, 0.48178, 0.48137, 0.48107, 0.48039],\
        [0.48272, 0.49007, 0.49679, 0.50269, 0.50779, 0.51228, 0.51635, 0.52014, 0.52381, 0.52746, 0.53120,\
        0.53510, 0.53920, 0.54355, 0.54814, 0.55296, 0.55797, 0.56314, 0.56840, 0.57372, 0.57910, 0.58459,\
        0.59042, 0.59719, 0.60517, 0.61436, 0.62446, 0.63502, 0.64577, 0.65653, 0.66716, 0.67758, 0.68777,\
        0.69776, 0.70750, 0.71694, 0.72595, 0.73444, 0.74234, 0.74963, 0.75628, 0.76221, 0.76733, 0.77147,\
        0.77446, 0.77631, 0.77739, 0.77827, 0.77971, 0.78272, 0.78818, 0.79536, 0.80260, 0.80862, 0.81220,\
        0.81259, 0.80983, 0.80504, 0.79965, 0.79534, 0.79416, 0.79649, 0.80002, 0.80305, 0.80603, 0.80721,\
        0.80567, 0.80147, 0.79522, 0.78801, 0.77983, 0.77138, 0.76333, 0.75612, 0.74996, 0.74492, 0.74090,\
        0.73767, 0.73487, 0.73213, 0.72912, 0.72559, 0.72141, 0.71643, 0.71020, 0.70158, 0.69054, 0.67805,\
        0.66466, 0.65117, 0.63829, 0.62665, 0.61665, 0.60844, 0.60257, 0.60036, 0.60288, 0.60701, 0.61110,\
        0.61479, 0.61782, 0.61986, 0.62058, 0.61960, 0.61641, 0.61031, 0.60078, 0.58814, 0.57244, 0.55508,\
        0.53740, 0.52060, 0.50590, 0.49489, 0.48750, 0.48267, 0.47970, 0.47801, 0.47703, 0.47624, 0.47523],\
        [0.48287, 0.49033, 0.49708, 0.50306, 0.50830, 0.51286, 0.51690, 0.52056, 0.52402, 0.52741, 0.53085,\
        0.53443, 0.53822, 0.54226, 0.54656, 0.55112, 0.55591, 0.56089, 0.56604, 0.57134, 0.57680, 0.58252,\
        0.58869, 0.59559, 0.60355, 0.61283, 0.62292, 0.63345, 0.64418, 0.65495, 0.66561, 0.67605, 0.68620,\
        0.69597, 0.70540, 0.71444, 0.72298, 0.73097, 0.73839, 0.74516, 0.75122, 0.75648, 0.76086, 0.76424,\
        0.76660, 0.76806, 0.76900, 0.77000, 0.77182, 0.77524, 0.78068, 0.78773, 0.79538, 0.80191, 0.80624,\
        0.80704, 0.80412, 0.79993, 0.79548, 0.79234, 0.79157, 0.79322, 0.79608, 0.79942, 0.80277, 0.80415,\
        0.80230, 0.79741, 0.79056, 0.78261, 0.77407, 0.76541, 0.75716, 0.74975, 0.74349, 0.73848, 0.73465,\
        0.73170, 0.72919, 0.72664, 0.72362, 0.71985, 0.71522, 0.70983, 0.70351, 0.69461, 0.68282, 0.66975,\
        0.65611, 0.64256, 0.62976, 0.61843, 0.60926, 0.60206, 0.59704, 0.59569, 0.59816, 0.60215, 0.60579,\
        0.60929, 0.61236, 0.61457, 0.61565, 0.61532, 0.61301, 0.60778, 0.59864, 0.58569, 0.56988, 0.55254,\
        0.53500, 0.51847, 0.50400, 0.49311, 0.48551, 0.48025, 0.47675, 0.47447, 0.47293, 0.47166, 0.47030],\
        [0.48267, 0.49027, 0.49706, 0.50314, 0.50851, 0.51311, 0.51707, 0.52058, 0.52381, 0.52693, 0.53007,\
        0.53336, 0.53687, 0.54065, 0.54474, 0.54912, 0.55378, 0.55870, 0.56386, 0.56927, 0.57495, 0.58100,\
        0.58753, 0.59466, 0.60275, 0.61202, 0.62196, 0.63235, 0.64298, 0.65370, 0.66435, 0.67477, 0.68486,\
        0.69451, 0.70361, 0.71202, 0.71998, 0.72744, 0.73432, 0.74053, 0.74600, 0.75063, 0.75432, 0.75703,\
        0.75878, 0.75975, 0.76031, 0.76104, 0.76264, 0.76578, 0.77087, 0.77770, 0.78544, 0.79283, 0.79874,\
        0.80076, 0.79804, 0.79459, 0.79136, 0.78916, 0.78870, 0.79002, 0.79259, 0.79584, 0.79895, 0.80020,\
        0.79762, 0.79211, 0.78481, 0.77646, 0.76758, 0.75864, 0.75012, 0.74249, 0.73609, 0.73114, 0.72757,\
        0.72503, 0.72292, 0.72064, 0.71760, 0.71346, 0.70817, 0.70169, 0.69466, 0.68535, 0.67368, 0.66068,\
        0.64720, 0.63393, 0.62153, 0.61069, 0.60248, 0.59736, 0.59366, 0.59295, 0.59503, 0.59834, 0.60155,\
        0.60488, 0.60782, 0.60989, 0.61095, 0.61101, 0.60961, 0.60531, 0.59668, 0.58312, 0.56693, 0.54957,\
        0.53231, 0.51640, 0.50302, 0.49210, 0.48390, 0.47809, 0.47403, 0.47118, 0.46910, 0.46736, 0.46565],\
        [0.48216, 0.49004, 0.49708, 0.50327, 0.50860, 0.51311, 0.51693, 0.52023, 0.52320, 0.52603, 0.52889,\
        0.53190, 0.53517, 0.53876, 0.54270, 0.54699, 0.55162, 0.55657, 0.56183, 0.56742, 0.57339, 0.57981,\
        0.58681, 0.59451, 0.60284, 0.61175, 0.62137, 0.63153, 0.64203, 0.65266, 0.66323, 0.67359, 0.68355,\
        0.69297, 0.70155, 0.70938, 0.71687, 0.72379, 0.73009, 0.73573, 0.74063, 0.74468, 0.74776, 0.74984,\
        0.75096, 0.75131, 0.75123, 0.75127, 0.75210, 0.75443, 0.75881, 0.76524, 0.77304, 0.78098, 0.78753,\
        0.79074, 0.79023, 0.78825, 0.78622, 0.78490, 0.78479, 0.78597, 0.78816, 0.79081, 0.79303, 0.79336,\
        0.79062, 0.78516, 0.77787, 0.76942, 0.76036, 0.75118, 0.74238, 0.73445, 0.72785, 0.72293, 0.71970,\
        0.71768, 0.71610, 0.71417, 0.71124, 0.70662, 0.70095, 0.69357, 0.68524, 0.67539, 0.66385, 0.65119,\
        0.63821, 0.62560, 0.61399, 0.60403, 0.59676, 0.59271, 0.59066, 0.59068, 0.59261, 0.59554, 0.59864,\
        0.60183, 0.60431, 0.60591, 0.60640, 0.60629, 0.60590, 0.60219, 0.59299, 0.57934, 0.56302, 0.54575,\
        0.52888, 0.51349, 0.50039, 0.49012, 0.48206, 0.47595, 0.47144, 0.46810, 0.46552, 0.46336, 0.46129],\
        [0.48131, 0.48955, 0.49695, 0.50326, 0.50850, 0.51288, 0.51651, 0.51956, 0.52223, 0.52475, 0.52730,\
        0.53006, 0.53312, 0.53657, 0.54044, 0.54472, 0.54940, 0.55447, 0.55990, 0.56571, 0.57195, 0.57870,\
        0.58605, 0.59409, 0.60257, 0.61132, 0.62075, 0.63076, 0.64113, 0.65165, 0.66213, 0.67237, 0.68215,\
        0.69112, 0.69894, 0.70667, 0.71373, 0.71995, 0.72561, 0.73070, 0.73509, 0.73861, 0.74114, 0.74262,\
        0.74310, 0.74274, 0.74182, 0.74084, 0.74045, 0.74149, 0.74482, 0.75079, 0.75871, 0.76708, 0.77421,\
        0.77873, 0.78043, 0.78034, 0.77967, 0.77922, 0.77947, 0.78057, 0.78232, 0.78421, 0.78544, 0.78500,\
        0.78218, 0.77701, 0.77000, 0.76170, 0.75263, 0.74330, 0.73422, 0.72594, 0.71905, 0.71406, 0.71121,\
        0.70979, 0.70874, 0.70712, 0.70438, 0.69997, 0.69290, 0.68502, 0.67585, 0.66549, 0.65398, 0.64172,\
        0.62939, 0.61775, 0.60733, 0.59873, 0.59266, 0.58926, 0.58805, 0.58867, 0.59079, 0.59379, 0.59702,\
        0.59977, 0.60183, 0.60291, 0.60245, 0.60100, 0.60137, 0.59885, 0.58834, 0.57408, 0.55779, 0.54084,\
        0.52451, 0.51003, 0.49781, 0.48791, 0.47994, 0.47369, 0.46887, 0.46514, 0.46216, 0.45961, 0.45722],\
        [0.48003, 0.48862, 0.49629, 0.50278, 0.50809, 0.51240, 0.51582, 0.51858, 0.52091, 0.52308, 0.52532,\
        0.52783, 0.53074, 0.53411, 0.53798, 0.54234, 0.54715, 0.55238, 0.55803, 0.56408, 0.57056, 0.57751,\
        0.58499, 0.59300, 0.60146, 0.61041, 0.61990, 0.62987, 0.64014, 0.65054, 0.66088, 0.67099, 0.68068,\
        0.68973, 0.69799, 0.70486, 0.71051, 0.71572, 0.72071, 0.72533, 0.72931, 0.73238, 0.73439, 0.73531,\
        0.73518, 0.73408, 0.73224, 0.73007, 0.72822, 0.72753, 0.72946, 0.73509, 0.74347, 0.75254, 0.76053,\
        0.76633, 0.76971, 0.77123, 0.77177, 0.77209, 0.77269, 0.77374, 0.77511, 0.77639, 0.77692, 0.77600,\
        0.77316, 0.76830, 0.76168, 0.75368, 0.74476, 0.73538, 0.72610, 0.71751, 0.71024, 0.70504, 0.70253,\
        0.70159, 0.70077, 0.69926, 0.69607, 0.69145, 0.68436, 0.67590, 0.66635, 0.65579, 0.64440, 0.63256,\
        0.62089, 0.61030, 0.60154, 0.59418, 0.58912, 0.58637, 0.58570, 0.58677, 0.58917, 0.59241, 0.59585,\
        0.59870, 0.60053, 0.60094, 0.59960, 0.59720, 0.59537, 0.59099, 0.58090, 0.56697, 0.55118, 0.53489,\
        0.51919, 0.50529, 0.49482, 0.48530, 0.47750, 0.47124, 0.46626, 0.46227, 0.45897, 0.45609, 0.45339],\
        [0.47828, 0.48726, 0.49518, 0.50191, 0.50743, 0.51171, 0.51492, 0.51732, 0.51925, 0.52102, 0.52294,\
        0.52522, 0.52801, 0.53138, 0.53533, 0.53984, 0.54485, 0.55033, 0.55623, 0.56252, 0.56921, 0.57631,\
        0.58382, 0.59176, 0.60022, 0.60928, 0.61888, 0.62884, 0.63901, 0.64922, 0.65934, 0.66921, 0.67871,\
        0.68768, 0.69584, 0.70191, 0.70628, 0.71067, 0.71518, 0.71952, 0.72322, 0.72589, 0.72738, 0.72774,\
        0.72706, 0.72533, 0.72259, 0.71928, 0.71610, 0.71349, 0.71355, 0.71926, 0.72864, 0.73862, 0.74748,\
        0.75431, 0.75885, 0.76147, 0.76286, 0.76374, 0.76460, 0.76565, 0.76680, 0.76771, 0.76788, 0.76678,\
        0.76405, 0.75954, 0.75336, 0.74578, 0.73714, 0.72788, 0.71852, 0.70981, 0.70240, 0.69722, 0.69458,\
        0.69338, 0.69220, 0.69020, 0.68675, 0.68168, 0.67477, 0.66636, 0.65682, 0.64638, 0.63529, 0.62396,\
        0.61298, 0.60295, 0.59560, 0.58976, 0.58563, 0.58350, 0.58325, 0.58461, 0.58725, 0.59073, 0.59456,\
        0.59812, 0.60029, 0.59898, 0.59629, 0.59269, 0.58835, 0.58181, 0.57166, 0.55841, 0.54348, 0.52823,\
        0.51381, 0.50098, 0.49058, 0.48206, 0.47473, 0.46863, 0.46360, 0.45945, 0.45591, 0.45276, 0.44980],\
        [0.47605, 0.48556, 0.49394, 0.50100, 0.50672, 0.51094, 0.51385, 0.51581, 0.51724, 0.51857, 0.52015,\
        0.52223, 0.52496, 0.52840, 0.53252, 0.53726, 0.54256, 0.54833, 0.55453, 0.56109, 0.56800, 0.57523,\
        0.58279, 0.59070, 0.59907, 0.60823, 0.61787, 0.62776, 0.63774, 0.64768, 0.65743, 0.66687, 0.67584,\
        0.68413, 0.69127, 0.69660, 0.70064, 0.70467, 0.70898, 0.71324, 0.71678, 0.71902, 0.71987, 0.71963,\
        0.71846, 0.71630, 0.71285, 0.70839, 0.70457, 0.70097, 0.69910, 0.70541, 0.71569, 0.72619, 0.73565,\
        0.74316, 0.74833, 0.75147, 0.75325, 0.75440, 0.75542, 0.75653, 0.75765, 0.75848, 0.75859, 0.75758,\
        0.75511, 0.75104, 0.74539, 0.73833, 0.73013, 0.72118, 0.71195, 0.70311, 0.69629, 0.69109, 0.68779,\
        0.68564, 0.68361, 0.68092, 0.67704, 0.67171, 0.66487, 0.65669, 0.64744, 0.63737, 0.62676, 0.61606,\
        0.60600, 0.59723, 0.59027, 0.58529, 0.58195, 0.58032, 0.58035, 0.58184, 0.58451, 0.58798, 0.59178,\
        0.59525, 0.59692, 0.59506, 0.59133, 0.58651, 0.58046, 0.57250, 0.56202, 0.54927, 0.53517, 0.52100,\
        0.50809, 0.49700, 0.48704, 0.47876, 0.47179, 0.46591, 0.46093, 0.45668, 0.45297, 0.44961, 0.44641],\
        [0.47327, 0.48351, 0.49262, 0.50024, 0.50610, 0.51013, 0.51262, 0.51403, 0.51489, 0.51572, 0.51694,\
        0.51885, 0.52160, 0.52520, 0.52959, 0.53466, 0.54031, 0.54645, 0.55298, 0.55985, 0.56701, 0.57442,\
        0.58210, 0.59008, 0.59852, 0.60757, 0.61705, 0.62673, 0.63639, 0.64592, 0.65516, 0.66398, 0.67221,\
        0.67961, 0.68585, 0.69072, 0.69460, 0.69828, 0.70230, 0.70652, 0.70990, 0.71164, 0.71151, 0.71053,\
        0.70886, 0.70633, 0.70293, 0.69705, 0.69264, 0.68937, 0.68911, 0.69510, 0.70485, 0.71529, 0.72517,\
        0.73307, 0.73834, 0.74141, 0.74313, 0.74428, 0.74540, 0.74666, 0.74795, 0.74895, 0.74927, 0.74856,\
        0.74653, 0.74301, 0.73796, 0.73150, 0.72387, 0.71543, 0.70670, 0.69832, 0.69121, 0.68576, 0.68167,\
        0.67845, 0.67542, 0.67193, 0.66751, 0.66192, 0.65510, 0.64718, 0.63836, 0.62886, 0.61891, 0.60885,\
        0.59952, 0.59175, 0.58551, 0.58094, 0.57801, 0.57665, 0.57678, 0.57821, 0.58067, 0.58377, 0.58697,\
        0.58950, 0.59022, 0.58838, 0.58442, 0.57892, 0.57194, 0.56322, 0.55260, 0.54027, 0.52690, 0.51348,\
        0.50152, 0.49163, 0.48299, 0.47539, 0.46884, 0.46318, 0.45829, 0.45400, 0.45016, 0.44662, 0.44321],\
        [0.46976, 0.48086, 0.49090, 0.49929, 0.50537, 0.50920, 0.51121, 0.51196, 0.51216, 0.51244, 0.51332,\
        0.51512, 0.51798, 0.52187, 0.52663, 0.53211, 0.53818, 0.54473, 0.55164, 0.55885, 0.56631, 0.57396,\
        0.58180, 0.58987, 0.59832, 0.60722, 0.61646, 0.62579, 0.63502, 0.64401, 0.65262, 0.66071, 0.66813,\
        0.67468, 0.68020, 0.68468, 0.68840, 0.69185, 0.69545, 0.69933, 0.70213, 0.70348, 0.70195, 0.70002,\
        0.69773, 0.69435, 0.69037, 0.68474, 0.68000, 0.67756, 0.67881, 0.68483, 0.69434, 0.70504, 0.71585,\
        0.72399, 0.72878, 0.73124, 0.73252, 0.73351, 0.73473, 0.73629, 0.73797, 0.73936, 0.74011, 0.73989,\
        0.73841, 0.73552, 0.73113, 0.72532, 0.71831, 0.71040, 0.70209, 0.69407, 0.68683, 0.68079, 0.67586,\
        0.67165, 0.66765, 0.66336, 0.65839, 0.65251, 0.64567, 0.63797, 0.62962, 0.62083, 0.61175, 0.60243,\
        0.59383, 0.58656, 0.58075, 0.57646, 0.57369, 0.57239, 0.57244, 0.57364, 0.57569, 0.57818, 0.58058,\
        0.58217, 0.58219, 0.58016, 0.57614, 0.57041, 0.56308, 0.55415, 0.54368, 0.53193, 0.51946, 0.50688,\
        0.49573, 0.48690, 0.47910, 0.47214, 0.46598, 0.46054, 0.45573, 0.45142, 0.44748, 0.44377, 0.44016],\
        [0.46526, 0.47729, 0.48834, 0.49763, 0.50415, 0.50803, 0.50956, 0.50957, 0.50906, 0.50880, 0.50935,\
        0.51113, 0.51423, 0.51852, 0.52375, 0.52971, 0.53625, 0.54323, 0.55056, 0.55815, 0.56593, 0.57385,\
        0.58188, 0.59004, 0.59840, 0.60715, 0.61610, 0.62499, 0.63368, 0.64203, 0.64992, 0.65722, 0.66382,\
        0.66957, 0.67443, 0.67844, 0.68183, 0.68490, 0.68791, 0.69074, 0.69264, 0.69311, 0.69110, 0.68807,\
        0.68469, 0.68047, 0.67561, 0.67035, 0.66608, 0.66439, 0.66657, 0.67315, 0.68329, 0.69545, 0.70752,\
        0.71537, 0.71910, 0.72065, 0.72141, 0.72222, 0.72365, 0.72576, 0.72803, 0.72999, 0.73129, 0.73165,\
        0.73081, 0.72858, 0.72488, 0.71968, 0.71322, 0.70583, 0.69777, 0.68976, 0.68232, 0.67575, 0.67007,\
        0.66501, 0.66019, 0.65521, 0.64973, 0.64355, 0.63663, 0.62907, 0.62110, 0.61298, 0.60487, 0.59663,\
        0.58839, 0.58139, 0.57578, 0.57162, 0.56887, 0.56749, 0.56733, 0.56818, 0.56973, 0.57158, 0.57321,\
        0.57404, 0.57352, 0.57127, 0.56724, 0.56153, 0.55424, 0.54547, 0.53541, 0.52440, 0.51307, 0.50236,\
        0.49207, 0.48331, 0.47582, 0.46922, 0.46332, 0.45805, 0.45329, 0.44895, 0.44491, 0.44105, 0.43726],\
        [0.45951, 0.47250, 0.48466, 0.49494, 0.50247, 0.50675, 0.50767, 0.50680, 0.50565, 0.50493, 0.50521,\
        0.50705, 0.51056, 0.51537, 0.52112, 0.52758, 0.53459, 0.54202, 0.54977, 0.55774, 0.56587, 0.57408,\
        0.58234, 0.59064, 0.59896, 0.60746, 0.61598, 0.62433, 0.63238, 0.64002, 0.64713, 0.65361, 0.65937,\
        0.66434, 0.66849, 0.67191, 0.67473, 0.67714, 0.67923, 0.68084, 0.68153, 0.68082, 0.67840, 0.67477,\
        0.67044, 0.66546, 0.66005, 0.65478, 0.65076, 0.64946, 0.65221, 0.65956, 0.67085, 0.68452, 0.69762,\
        0.70548, 0.70820, 0.70929, 0.70992, 0.71072, 0.71258, 0.71557, 0.71856, 0.72108, 0.72298, 0.72394,\
        0.72371, 0.72213, 0.71910, 0.71445, 0.70820, 0.70131, 0.69330, 0.68510, 0.67735, 0.67032, 0.66404,\
        0.65834, 0.65291, 0.64738, 0.64147, 0.63503, 0.62800, 0.62045, 0.61265, 0.60511, 0.59741, 0.58985,\
        0.58240, 0.57578, 0.57037, 0.56626, 0.56348, 0.56194, 0.56151, 0.56198, 0.56303, 0.56430, 0.56530,\
        0.56557, 0.56465, 0.56225, 0.55826, 0.55271, 0.54570, 0.53735, 0.52788, 0.51765, 0.50721, 0.49732,\
        0.48841, 0.48024, 0.47303, 0.46663, 0.46090, 0.45571, 0.45097, 0.44658, 0.44244, 0.43843, 0.43446],\
        [0.45229, 0.46619, 0.47966, 0.49179, 0.50108, 0.50523, 0.50535, 0.50353, 0.50213, 0.50120, 0.50126,\
        0.50326, 0.50734, 0.51268, 0.51891, 0.52582, 0.53327, 0.54112, 0.54927, 0.55762, 0.56608, 0.57458,\
        0.58308, 0.59153, 0.59990, 0.60804, 0.61602, 0.62375, 0.63111, 0.63799, 0.64429, 0.64993, 0.65484,\
        0.65899, 0.66236, 0.66504, 0.66710, 0.66864, 0.66967, 0.67005, 0.66952, 0.66778, 0.66473, 0.66059,\
        0.65564, 0.65011, 0.64429, 0.63880, 0.63464, 0.63312, 0.63594, 0.64391, 0.65604, 0.67012, 0.68319,\
        0.69178, 0.69559, 0.69748, 0.69874, 0.69993, 0.70242, 0.70649, 0.70995, 0.71284, 0.71532, 0.71685,\
        0.71705, 0.71594, 0.71361, 0.70963, 0.70334, 0.69582, 0.68798, 0.67967, 0.67164, 0.66425, 0.65757,\
        0.65147, 0.64565, 0.63978, 0.63356, 0.62690, 0.61981, 0.61225, 0.60433, 0.59690, 0.58980, 0.58262,\
        0.57575, 0.56956, 0.56438, 0.56034, 0.55749, 0.55577, 0.55507, 0.55519, 0.55583, 0.55663, 0.55718,\
        0.55706, 0.55589, 0.55342, 0.54953, 0.54425, 0.53766, 0.52992, 0.52117, 0.51174, 0.50220, 0.49322,\
        0.48499, 0.47740, 0.47050, 0.46428, 0.45864, 0.45349, 0.44874, 0.44429, 0.44004, 0.43589, 0.43174],\
        [0.44347, 0.45803, 0.47257, 0.48606, 0.49684, 0.50227, 0.50220, 0.50013, 0.49886, 0.49819, 0.49838,\
        0.50059, 0.50504, 0.51068, 0.51722, 0.52447, 0.53230, 0.54053, 0.54906, 0.55775, 0.56651, 0.57526,\
        0.58392, 0.59243, 0.60068, 0.60853, 0.61604, 0.62316, 0.62982, 0.63593, 0.64141, 0.64619, 0.65023,\
        0.65352, 0.65605, 0.65788, 0.65905, 0.65961, 0.65954, 0.65876, 0.65712, 0.65447, 0.65078, 0.64617,\
        0.64085, 0.63504, 0.62896, 0.62322, 0.61878, 0.61640, 0.61865, 0.62718, 0.63994, 0.65412, 0.66703,\
        0.67656, 0.68245, 0.68606, 0.68864, 0.69107, 0.69450, 0.69891, 0.70223, 0.70536, 0.70855, 0.71053,\
        0.71075, 0.70963, 0.70778, 0.70478, 0.69882, 0.69056, 0.68192, 0.67328, 0.66499, 0.65736, 0.65047,\
        0.64422, 0.63831, 0.63234, 0.62595, 0.61905, 0.61200, 0.60465, 0.59672, 0.58915, 0.58204, 0.57516,\
        0.56867, 0.56283, 0.55784, 0.55387, 0.55095, 0.54908, 0.54814, 0.54797, 0.54831, 0.54880, 0.54908,\
        0.54875, 0.54747, 0.54502, 0.54129, 0.53631, 0.53022, 0.52321, 0.51536, 0.50675, 0.49802, 0.48979,\
        0.48203, 0.47479, 0.46812, 0.46202, 0.45645, 0.45132, 0.44654, 0.44202, 0.43766, 0.43337, 0.42905],\
        [0.43315, 0.44800, 0.46311, 0.47727, 0.48889, 0.49543, 0.49724, 0.49690, 0.49590, 0.49574, 0.49669,\
        0.49941, 0.50370, 0.50930, 0.51599, 0.52351, 0.53165, 0.54022, 0.54908, 0.55807, 0.56709, 0.57603,\
        0.58477, 0.59324, 0.60130, 0.60888, 0.61595, 0.62249, 0.62847, 0.63381, 0.63846, 0.64239, 0.64555,\
        0.64794, 0.64959, 0.65051, 0.65074, 0.65030, 0.64916, 0.64730, 0.64467, 0.64123, 0.63696, 0.63198,\
        0.62646, 0.62071, 0.61475, 0.60862, 0.60459, 0.60118, 0.60236, 0.61161, 0.62472, 0.63888, 0.65185,\
        0.66225, 0.66985, 0.67530, 0.67951, 0.68336, 0.68751, 0.69181, 0.69549, 0.69913, 0.70307, 0.70519,\
        0.70474, 0.70292, 0.70072, 0.69810, 0.69206, 0.68369, 0.67469, 0.66576, 0.65729, 0.64954, 0.64260,\
        0.63644, 0.63076, 0.62503, 0.61869, 0.61140, 0.60412, 0.59717, 0.58928, 0.58156, 0.57432, 0.56755,\
        0.56130, 0.55570, 0.55088, 0.54695, 0.54398, 0.54197, 0.54086, 0.54050, 0.54066, 0.54102, 0.54120,\
        0.54083, 0.53958, 0.53723, 0.53368, 0.52897, 0.52332, 0.51707, 0.51039, 0.50293, 0.49478, 0.48687,\
        0.47933, 0.47228, 0.46575, 0.45974, 0.45422, 0.44910, 0.44429, 0.43971, 0.43525, 0.43083, 0.42634],\
        [0.42182, 0.43667, 0.45200, 0.46633, 0.47810, 0.48592, 0.48999, 0.49164, 0.49227, 0.49313, 0.49492,\
        0.49804, 0.50247, 0.50818, 0.51506, 0.52284, 0.53126, 0.54014, 0.54928, 0.55853, 0.56776, 0.57683,\
        0.58562, 0.59400, 0.60185, 0.60911, 0.61574, 0.62171, 0.62701, 0.63160, 0.63544, 0.63851, 0.64079,\
        0.64228, 0.64302, 0.64302, 0.64232, 0.64092, 0.63878, 0.63592, 0.63242, 0.62832, 0.62355, 0.61832,\
        0.61269, 0.60714, 0.60219, 0.59598, 0.59197, 0.58949, 0.59130, 0.59954, 0.61212, 0.62587, 0.63868,\
        0.64964, 0.65846, 0.66540, 0.67104, 0.67603, 0.68079, 0.68541, 0.68978, 0.69417, 0.69845, 0.70081,\
        0.69875, 0.69627, 0.69310, 0.68917, 0.68304, 0.67495, 0.66601, 0.65703, 0.64852, 0.64075, 0.63389,\
        0.62798, 0.62282, 0.61768, 0.61186, 0.60477, 0.59642, 0.58911, 0.58133, 0.57362, 0.56639, 0.55974,\
        0.55369, 0.54830, 0.54362, 0.53973, 0.53671, 0.53459, 0.53335, 0.53291, 0.53304, 0.53344, 0.53371,\
        0.53347, 0.53238, 0.53020, 0.52683, 0.52231, 0.51687, 0.51107, 0.50554, 0.49945, 0.49176, 0.48396,\
        0.47655, 0.46962, 0.46321, 0.45729, 0.45182, 0.44673, 0.44192, 0.43729, 0.43275, 0.42820, 0.42356],\
        [0.41038, 0.42518, 0.44059, 0.45496, 0.46700, 0.47592, 0.48179, 0.48540, 0.48780, 0.48999, 0.49267,\
        0.49628, 0.50103, 0.50707, 0.51427, 0.52235, 0.53106, 0.54020, 0.54960, 0.55907, 0.56847, 0.57764,\
        0.58643, 0.59470, 0.60232, 0.60922, 0.61537, 0.62077, 0.62541, 0.62926, 0.63230, 0.63453, 0.63594,\
        0.63655, 0.63638, 0.63549, 0.63391, 0.63165, 0.62863, 0.62481, 0.62050, 0.61598, 0.61075, 0.60527,\
        0.59987, 0.59417, 0.58946, 0.58465, 0.58100, 0.57993, 0.58288, 0.59075, 0.60252, 0.61524, 0.62758,\
        0.63882, 0.64844, 0.65643, 0.66311, 0.66890, 0.67419, 0.67915, 0.68389, 0.68842, 0.69232, 0.69400,\
        0.69216, 0.68892, 0.68483, 0.67973, 0.67310, 0.66504, 0.65620, 0.64728, 0.63884, 0.63117, 0.62437,\
        0.61867, 0.61423, 0.60996, 0.60435, 0.59820, 0.58949, 0.58095, 0.57286, 0.56520, 0.55812, 0.55169,\
        0.54591, 0.54075, 0.53623, 0.53239, 0.52933, 0.52711, 0.52579, 0.52535, 0.52560, 0.52622, 0.52676,\
        0.52681, 0.52599, 0.52406, 0.52090, 0.51652, 0.51108, 0.50505, 0.49981, 0.49453, 0.48773, 0.48045,\
        0.47332, 0.46659, 0.46032, 0.45452, 0.44914, 0.44411, 0.43932, 0.43468, 0.43009, 0.42544, 0.42066],\
        [0.40054, 0.41514, 0.43035, 0.44456, 0.45679, 0.46650, 0.47373, 0.47896, 0.48295, 0.48646, 0.49012,\
        0.49436, 0.49950, 0.50602, 0.51361, 0.52199, 0.53097, 0.54035, 0.54996, 0.55962, 0.56915, 0.57838,\
        0.58714, 0.59530, 0.60266, 0.60916, 0.61481, 0.61963, 0.62360, 0.62673, 0.62901, 0.63042, 0.63099,\
        0.63073, 0.62970, 0.62795, 0.62554, 0.62259, 0.61897, 0.61426, 0.60889, 0.60433, 0.59895, 0.59293,\
        0.58749, 0.58207, 0.57731, 0.57333, 0.57084, 0.57102, 0.57489, 0.58282, 0.59388, 0.60586, 0.61806,\
        0.62956, 0.63971, 0.64836, 0.65563, 0.66188, 0.66743, 0.67250, 0.67718, 0.68130, 0.68434, 0.68533,\
        0.68381, 0.68046, 0.67582, 0.66999, 0.66288, 0.65465, 0.64577, 0.63689, 0.62859, 0.62118, 0.61447,\
        0.60855, 0.60468, 0.60145, 0.59567, 0.58863, 0.58044, 0.57186, 0.56370, 0.55621, 0.54945, 0.54342,\
        0.53803, 0.53319, 0.52888, 0.52513, 0.52203, 0.51972, 0.51834, 0.51797, 0.51849, 0.51951, 0.52050,\
        0.52096, 0.52050, 0.51886, 0.51596, 0.51182, 0.50651, 0.50040, 0.49471, 0.48928, 0.48303, 0.47627,\
        0.46951, 0.46303, 0.45696, 0.45132, 0.44608, 0.44114, 0.43643, 0.43182, 0.42720, 0.42248, 0.41757],\
        [0.39420, 0.40785, 0.42222, 0.43591, 0.44806, 0.45824, 0.46641, 0.47289, 0.47819, 0.48290, 0.48755,\
        0.49258, 0.49838, 0.50526, 0.51312, 0.52173, 0.53093, 0.54051, 0.55030, 0.56010, 0.56972, 0.57897,\
        0.58766, 0.59562, 0.60270, 0.60880, 0.61398, 0.61821, 0.62152, 0.62395, 0.62549, 0.62614, 0.62590,\
        0.62481, 0.62297, 0.62043, 0.61720, 0.61363, 0.60988, 0.60497, 0.59815, 0.59300, 0.58746, 0.58142,\
        0.57568, 0.57037, 0.56581, 0.56239, 0.56079, 0.56188, 0.56639, 0.57442, 0.58516, 0.59724, 0.60989,\
        0.62155, 0.63210, 0.64106, 0.64851, 0.65482, 0.66034, 0.66527, 0.66962, 0.67319, 0.67552, 0.67607,\
        0.67457, 0.67123, 0.66637, 0.66017, 0.65273, 0.64428, 0.63527, 0.62633, 0.61813, 0.61119, 0.60532,\
        0.59877, 0.59456, 0.59083, 0.58517, 0.57794, 0.56989, 0.56161, 0.55375, 0.54665, 0.54044, 0.53500,\
        0.53018, 0.52581, 0.52181, 0.51818, 0.51507, 0.51268, 0.51123, 0.51097, 0.51190, 0.51350, 0.51506,\
        0.51601, 0.51595, 0.51459, 0.51188, 0.50799, 0.50316, 0.49709, 0.49079, 0.48458, 0.47818, 0.47161,\
        0.46510, 0.45887, 0.45304, 0.44761, 0.44254, 0.43776, 0.43316, 0.42862, 0.42403, 0.41927, 0.41425],\
        [0.39011, 0.40289, 0.41620, 0.42916, 0.44101, 0.45136, 0.46013, 0.46750, 0.47383, 0.47956, 0.48513,\
        0.49095, 0.49738, 0.50461, 0.51269, 0.52150, 0.53087, 0.54061, 0.55052, 0.56041, 0.57010, 0.57937,\
        0.58796, 0.59558, 0.60228, 0.60817, 0.61284, 0.61642, 0.61905, 0.62080, 0.62167, 0.62163, 0.62063,\
        0.61872, 0.61614, 0.61301, 0.60901, 0.60444, 0.60070, 0.59580, 0.58903, 0.58248, 0.57631, 0.57017,\
        0.56434, 0.55909, 0.55471, 0.55164, 0.55051, 0.55210, 0.55703, 0.56541, 0.57662, 0.58953, 0.60223,\
        0.61445, 0.62548, 0.63437, 0.64154, 0.64756, 0.65281, 0.65743, 0.66141, 0.66450, 0.66636, 0.66661,\
        0.66508, 0.66179, 0.65691, 0.65060, 0.64303, 0.63444, 0.62526, 0.61617, 0.60795, 0.60130, 0.59613,\
        0.59046, 0.58497, 0.57984, 0.57376, 0.56653, 0.55861, 0.55062, 0.54316, 0.53665, 0.53120, 0.52660,\
        0.52254, 0.51879, 0.51521, 0.51178, 0.50872, 0.50630, 0.50478, 0.50460, 0.50612, 0.50842, 0.51055,\
        0.51201, 0.51232, 0.51116, 0.50843, 0.50433, 0.49962, 0.49359, 0.48672, 0.47987, 0.47312, 0.46649,\
        0.46011, 0.45409, 0.44849, 0.44331, 0.43847, 0.43388, 0.42944, 0.42503, 0.42051, 0.41574, 0.41062],\
        [0.38720, 0.39931, 0.41183, 0.42410, 0.43554, 0.44585, 0.45495, 0.46294, 0.47006, 0.47661, 0.48295,\
        0.48942, 0.49633, 0.50390, 0.51221, 0.52121, 0.53072, 0.54056, 0.55053, 0.56046, 0.57017, 0.57945,\
        0.58809, 0.59580, 0.60227, 0.60739, 0.61127, 0.61410, 0.61603, 0.61713, 0.61742, 0.61682, 0.61514,\
        0.61239, 0.60899, 0.60547, 0.60166, 0.59580, 0.59096, 0.58544, 0.57889, 0.57207, 0.56547, 0.55918,\
        0.55333, 0.54814, 0.54388, 0.54094, 0.53989, 0.54154, 0.54666, 0.55553, 0.56759, 0.58148, 0.59552,\
        0.60895, 0.61981, 0.62794, 0.63437, 0.63987, 0.64474, 0.64906, 0.65273, 0.65551, 0.65710, 0.65723,\
        0.65571, 0.65254, 0.64779, 0.64159, 0.63412, 0.62562, 0.61642, 0.60711, 0.59922, 0.59254, 0.58698,\
        0.58132, 0.57541, 0.56930, 0.56260, 0.55515, 0.54723, 0.53938, 0.53226, 0.52642, 0.52194, 0.51839,\
        0.51527, 0.51228, 0.50928, 0.50613, 0.50319, 0.50098, 0.49956, 0.49951, 0.50171, 0.50444, 0.50698,\
        0.50888, 0.50951, 0.50843, 0.50554, 0.50091, 0.49534, 0.48899, 0.48188, 0.47461, 0.46753, 0.46078,\
        0.45446, 0.44863, 0.44329, 0.43839, 0.43382, 0.42946, 0.42521, 0.42096, 0.41657, 0.41183, 0.40662],\
        [0.38522, 0.39691, 0.40881, 0.42042, 0.43140, 0.44156, 0.45083, 0.45924, 0.46695, 0.47415, 0.48110,\
        0.48805, 0.49530, 0.50313, 0.51166, 0.52085, 0.53047, 0.54033, 0.55026, 0.56012, 0.56975, 0.57897,\
        0.58758, 0.59531, 0.60154, 0.60595, 0.60892, 0.61097, 0.61225, 0.61273, 0.61252, 0.61160, 0.60943,\
        0.60581, 0.60141, 0.59726, 0.59287, 0.58717, 0.58115, 0.57492, 0.56825, 0.56139, 0.55469, 0.54834,\
        0.54254, 0.53747, 0.53332, 0.53033, 0.52898, 0.53019, 0.53514, 0.54450, 0.55776, 0.57348, 0.58984,\
        0.60436, 0.61422, 0.62087, 0.62645, 0.63149, 0.63608, 0.64022, 0.64375, 0.64643, 0.64796, 0.64810,\
        0.64671, 0.64375, 0.63927, 0.63338, 0.62625, 0.61811, 0.60937, 0.60051, 0.59229, 0.58536, 0.57899,\
        0.57270, 0.56620, 0.55940, 0.55213, 0.54438, 0.53634, 0.52846, 0.52146, 0.51626, 0.51295, 0.51062,\
        0.50848, 0.50624, 0.50413, 0.50134, 0.49861, 0.49678, 0.49592, 0.49648, 0.49863, 0.50129, 0.50411,\
        0.50646, 0.50729, 0.50610, 0.50289, 0.49773, 0.49113, 0.48388, 0.47624, 0.46858, 0.46121, 0.45434,\
        0.44808, 0.44245, 0.43740, 0.43282, 0.42857, 0.42447, 0.42040, 0.41632, 0.41213, 0.40749, 0.40217],\
        [0.38383, 0.39530, 0.40668, 0.41771, 0.42826, 0.43824, 0.44759, 0.45632, 0.46450, 0.47224, 0.47970,\
        0.48702, 0.49442, 0.50234, 0.51115, 0.52050, 0.53012, 0.53987, 0.54964, 0.55928, 0.56867, 0.57763,\
        0.58596, 0.59334, 0.59934, 0.60332, 0.60532, 0.60675, 0.60759, 0.60743, 0.60665, 0.60574, 0.60344,\
        0.59893, 0.59380, 0.58863, 0.58328, 0.57729, 0.57086, 0.56421, 0.55739, 0.55054, 0.54387, 0.53758,\
        0.53190, 0.52702, 0.52307, 0.52003, 0.51811, 0.51829, 0.52252, 0.53226, 0.54675, 0.56404, 0.58207,\
        0.59783, 0.60613, 0.61193, 0.61734, 0.62232, 0.62687, 0.63104, 0.63466, 0.63742, 0.63907, 0.63938,\
        0.63822, 0.63556, 0.63146, 0.62602, 0.61946, 0.61194, 0.60350, 0.59532, 0.58706, 0.57930, 0.57202,\
        0.56491, 0.55772, 0.55032, 0.54262, 0.53462, 0.52650, 0.51857, 0.51139, 0.50657, 0.50462, 0.50343,\
        0.50216, 0.50043, 0.49931, 0.49711, 0.49474, 0.49327, 0.49290, 0.49380, 0.49586, 0.49853, 0.50173,\
        0.50444, 0.50537, 0.50366, 0.49956, 0.49353, 0.48611, 0.47803, 0.46976, 0.46166, 0.45404, 0.44709,\
        0.44092, 0.43552, 0.43081, 0.42662, 0.42274, 0.41891, 0.41498, 0.41095, 0.40701, 0.40264, 0.39715],\
        [0.38267, 0.39389, 0.40494, 0.41556, 0.42578, 0.43563, 0.44507, 0.45408, 0.46267, 0.47089, 0.47885,\
        0.48662, 0.49425, 0.50218, 0.51110, 0.52033, 0.52972, 0.53918, 0.54860, 0.55787, 0.56683, 0.57532,\
        0.58314, 0.59001, 0.59556, 0.59896, 0.60018, 0.60134, 0.60207, 0.60111, 0.59969, 0.59851, 0.59635,\
        0.59117, 0.58536, 0.57938, 0.57324, 0.56680, 0.56009, 0.55326, 0.54638, 0.53958, 0.53302, 0.52687,\
        0.52132, 0.51668, 0.51314, 0.51027, 0.50782, 0.50652, 0.50933, 0.51949, 0.53498, 0.55283, 0.57055,\
        0.58516, 0.59455, 0.60124, 0.60743, 0.61266, 0.61731, 0.62176, 0.62566, 0.62868, 0.63057, 0.63115,\
        0.63029, 0.62801, 0.62436, 0.61945, 0.61350, 0.60695, 0.59950, 0.59090, 0.58228, 0.57385, 0.56574,\
        0.55785, 0.55003, 0.54216, 0.53417, 0.52609, 0.51809, 0.51051, 0.50358, 0.49858, 0.49737, 0.49665,\
        0.49580, 0.49468, 0.49358, 0.49215, 0.49056, 0.48963, 0.48969, 0.49085, 0.49302, 0.49593, 0.49920,\
        0.50219, 0.50332, 0.50068, 0.49531, 0.48819, 0.47994, 0.47115, 0.46232, 0.45383, 0.44599, 0.43900,\
        0.43297, 0.42786, 0.42354, 0.41980, 0.41635, 0.41285, 0.40902, 0.40478, 0.40083, 0.39675, 0.39130],\
        [0.38151, 0.39257, 0.40341, 0.41365, 0.42365, 0.43348, 0.44309, 0.45239, 0.46137, 0.47003, 0.47846,\
        0.48678, 0.49513, 0.50332, 0.51157, 0.52026, 0.52920, 0.53820, 0.54714, 0.55587, 0.56424, 0.57207,\
        0.57917, 0.58525, 0.58995, 0.59281, 0.59412, 0.59491, 0.59506, 0.59376, 0.59188, 0.58979, 0.58667,\
        0.58171, 0.57571, 0.56929, 0.56265, 0.55586, 0.54897, 0.54208, 0.53525, 0.52858, 0.52221, 0.51623,\
        0.51082, 0.50627, 0.50330, 0.50079, 0.49822, 0.49621, 0.49799, 0.50842, 0.52404, 0.54143, 0.55821,\
        0.57233, 0.58287, 0.59089, 0.59774, 0.60290, 0.60781, 0.61279, 0.61704, 0.62036, 0.62258, 0.62347,\
        0.62293, 0.62101, 0.61788, 0.61361, 0.60806, 0.60191, 0.59533, 0.58670, 0.57760, 0.56857, 0.55979,\
        0.55129, 0.54300, 0.53483, 0.52672, 0.51873, 0.51107, 0.50399, 0.49849, 0.49374, 0.49139, 0.49028,\
        0.48943, 0.48856, 0.48764, 0.48665, 0.48576, 0.48537, 0.48579, 0.48715, 0.48939, 0.49229, 0.49552,\
        0.49843, 0.49997, 0.49629, 0.48957, 0.48146, 0.47250, 0.46317, 0.45391, 0.44512, 0.43712, 0.43015,\
        0.42433, 0.41956, 0.41566, 0.41240, 0.40942, 0.40630, 0.40267, 0.39833, 0.39401, 0.38966, 0.38444],\
        [0.38017, 0.39090, 0.40146, 0.41162, 0.42166, 0.43166, 0.44154, 0.45119, 0.46053, 0.46956, 0.47832,\
        0.48692, 0.49542, 0.50365, 0.51171, 0.51999, 0.52843, 0.53690, 0.54524, 0.55331, 0.56096, 0.56802,\
        0.57428, 0.57949, 0.58339, 0.58582, 0.58698, 0.58727, 0.58670, 0.58513, 0.58281, 0.57990, 0.57605,\
        0.57100, 0.56499, 0.55841, 0.55153, 0.54454, 0.53758, 0.53077, 0.52408, 0.51759, 0.51145, 0.50572,\
        0.50051, 0.49607, 0.49310, 0.49085, 0.48882, 0.48818, 0.49144, 0.50094, 0.51531, 0.53145, 0.54718,\
        0.56095, 0.57214, 0.58108, 0.58827, 0.59391, 0.59939, 0.60458, 0.60901, 0.61258, 0.61515, 0.61634,\
        0.61605, 0.61442, 0.61175, 0.60840, 0.60407, 0.59776, 0.59060, 0.58196, 0.57257, 0.56311, 0.55387,\
        0.54497, 0.53641, 0.52815, 0.52011, 0.51229, 0.50505, 0.49877, 0.49325, 0.48901, 0.48615, 0.48444,\
        0.48329, 0.48237, 0.48155, 0.48084, 0.48035, 0.48033, 0.48098, 0.48238, 0.48448, 0.48707, 0.48975,\
        0.49181, 0.49197, 0.48831, 0.48149, 0.47304, 0.46375, 0.45414, 0.44466, 0.43571, 0.42764, 0.42076,\
        0.41523, 0.41085, 0.40731, 0.40449, 0.40187, 0.39903, 0.39552, 0.39134, 0.38688, 0.38219, 0.37695],\
        [0.37872, 0.38898, 0.39929, 0.40949, 0.41978, 0.43014, 0.44042, 0.45045, 0.46014, 0.46944, 0.47836,\
        0.48694, 0.49522, 0.50332, 0.51136, 0.51937, 0.52736, 0.53525, 0.54292, 0.55026, 0.55712, 0.56334,\
        0.56873, 0.57309, 0.57625, 0.57815, 0.57889, 0.57864, 0.57749, 0.57547, 0.57268, 0.56917, 0.56485,\
        0.55963, 0.55361, 0.54700, 0.54004, 0.53295, 0.52600, 0.51944, 0.51303, 0.50665, 0.50078, 0.49529,\
        0.49030, 0.48616, 0.48315, 0.48117, 0.48028, 0.48143, 0.48617, 0.49548, 0.50848, 0.52327, 0.53793,\
        0.55124, 0.56265, 0.57211, 0.57984, 0.58625, 0.59200, 0.59719, 0.60160, 0.60535, 0.60834, 0.60972,\
        0.60951, 0.60801, 0.60548, 0.60282, 0.60000, 0.59422, 0.58579, 0.57659, 0.56694, 0.55722, 0.54773,\
        0.53863, 0.53000, 0.52187, 0.51413, 0.50660, 0.49940, 0.49350, 0.48833, 0.48414, 0.48102, 0.47886,\
        0.47734, 0.47622, 0.47535, 0.47472, 0.47439, 0.47450, 0.47516, 0.47641, 0.47815, 0.48013, 0.48192,\
        0.48279, 0.48172, 0.47784, 0.47135, 0.46311, 0.45390, 0.44433, 0.43486, 0.42595, 0.41798, 0.41117,\
        0.40604, 0.40218, 0.39881, 0.39627, 0.39376, 0.39096, 0.38756, 0.38357, 0.37914, 0.37433, 0.36904],\
        [0.37744, 0.38721, 0.39715, 0.40748, 0.41821, 0.42907, 0.43982, 0.45025, 0.46026, 0.46975, 0.47871,\
        0.48716, 0.49521, 0.50308, 0.51086, 0.51853, 0.52602, 0.53328, 0.54024, 0.54680, 0.55283, 0.55819,\
        0.56272, 0.56626, 0.56869, 0.56997, 0.57016, 0.56935, 0.56764, 0.56512, 0.56186, 0.55790, 0.55325,\
        0.54790, 0.54192, 0.53539, 0.52844, 0.52134, 0.51434, 0.50811, 0.50243, 0.49596, 0.49023, 0.48493,\
        0.48020, 0.47631, 0.47352, 0.47203, 0.47221, 0.47480, 0.48068, 0.49017, 0.50258, 0.51644, 0.53008,\
        0.54292, 0.55440, 0.56419, 0.57237, 0.57924, 0.58520, 0.59040, 0.59484, 0.59880, 0.60205, 0.60336,\
        0.60304, 0.60165, 0.59913, 0.59631, 0.59304, 0.58749, 0.57948, 0.57027, 0.56057, 0.55079, 0.54124,\
        0.53211, 0.52354, 0.51565, 0.50845, 0.50153, 0.49463, 0.48856, 0.48343, 0.47919, 0.47586, 0.47335,\
        0.47148, 0.47007, 0.46903, 0.46830, 0.46792, 0.46794, 0.46842, 0.46933, 0.47055, 0.47180, 0.47263,\
        0.47243, 0.47049, 0.46636, 0.46010, 0.45224, 0.44339, 0.43414, 0.42498, 0.41633, 0.40882, 0.40226,\
        0.39730, 0.39384, 0.39074, 0.38805, 0.38544, 0.38256, 0.37919, 0.37530, 0.37091, 0.36608, 0.36076],\
        [0.37673, 0.38606, 0.39566, 0.40612, 0.41734, 0.42870, 0.43989, 0.45068, 0.46095, 0.47059, 0.47954,\
        0.48779, 0.49549, 0.50298, 0.51041, 0.51760, 0.52449, 0.53106, 0.53726, 0.54300, 0.54820, 0.55271,\
        0.55640, 0.55915, 0.56086, 0.56149, 0.56106, 0.55966, 0.55738, 0.55434, 0.55064, 0.54633, 0.54145,\
        0.53603, 0.53017, 0.52387, 0.51701, 0.51000, 0.50321, 0.49669, 0.49166, 0.48565, 0.47994, 0.47477,\
        0.47024, 0.46658, 0.46407, 0.46305, 0.46402, 0.46759, 0.47432, 0.48435, 0.49708, 0.50999, 0.52281,\
        0.53556, 0.54719, 0.55719, 0.56559, 0.57266, 0.57873, 0.58398, 0.58854, 0.59249, 0.59541, 0.59669,\
        0.59637, 0.59497, 0.59258, 0.58939, 0.58531, 0.57953, 0.57190, 0.56301, 0.55349, 0.54383, 0.53437,\
        0.52536, 0.51691, 0.50915, 0.50244, 0.49603, 0.48954, 0.48360, 0.47847, 0.47415, 0.47061, 0.46780,\
        0.46560, 0.46389, 0.46257, 0.46161, 0.46101, 0.46077, 0.46090, 0.46136, 0.46198, 0.46250, 0.46250,\
        0.46147, 0.45893, 0.45459, 0.44851, 0.44106, 0.43273, 0.42404, 0.41557, 0.40762, 0.40055, 0.39499,\
        0.38997, 0.38616, 0.38293, 0.38001, 0.37715, 0.37409, 0.37065, 0.36675, 0.36236, 0.35751, 0.35218],\
        [0.37692, 0.38604, 0.39556, 0.40612, 0.41759, 0.42926, 0.44074, 0.45180, 0.46227, 0.47201, 0.48094,\
        0.48902, 0.49633, 0.50332, 0.51019, 0.51670, 0.52285, 0.52862, 0.53401, 0.53893, 0.54329, 0.54699,\
        0.54989, 0.55189, 0.55290, 0.55287, 0.55181, 0.54979, 0.54693, 0.54337, 0.53926, 0.53467, 0.52963,\
        0.52416, 0.51844, 0.51267, 0.50614, 0.49890, 0.49261, 0.48640, 0.48081, 0.47524, 0.46982, 0.46483,\
        0.46047, 0.45697, 0.45464, 0.45390, 0.45527, 0.45933, 0.46651, 0.47679, 0.48923, 0.50193, 0.51546,\
        0.52892, 0.54083, 0.55086, 0.55921, 0.56626, 0.57231, 0.57754, 0.58204, 0.58574, 0.58835, 0.58956,\
        0.58935, 0.58793, 0.58546, 0.58197, 0.57735, 0.57131, 0.56382, 0.55520, 0.54594, 0.53649, 0.52723,\
        0.51847, 0.51033, 0.50264, 0.49594, 0.48998, 0.48403, 0.47841, 0.47337, 0.46899, 0.46528, 0.46220,\
        0.45968, 0.45764, 0.45599, 0.45470, 0.45375, 0.45312, 0.45280, 0.45273, 0.45276, 0.45264, 0.45199,\
        0.45039, 0.44744, 0.44298, 0.43706, 0.43003, 0.42234, 0.41440, 0.40660, 0.40089, 0.39429, 0.38829,\
        0.38322, 0.37900, 0.37541, 0.37215, 0.36898, 0.36566, 0.36206, 0.35805, 0.35362, 0.34872, 0.34337],\
        [0.37817, 0.38733, 0.39682, 0.40750, 0.41904, 0.43079, 0.44238, 0.45356, 0.46414, 0.47395, 0.48287,\
        0.49082, 0.49790, 0.50425, 0.51019, 0.51579, 0.52105, 0.52596, 0.53051, 0.53461, 0.53817, 0.54110,\
        0.54327, 0.54457, 0.54493, 0.54428, 0.54260, 0.53996, 0.53650, 0.53242, 0.52792, 0.52312, 0.51799,\
        0.51247, 0.50674, 0.50133, 0.49584, 0.48874, 0.48224, 0.47619, 0.47051, 0.46508, 0.45990, 0.45512,\
        0.45091, 0.44750, 0.44519, 0.44442, 0.44576, 0.44983, 0.45710, 0.46753, 0.48025, 0.49411, 0.50916,\
        0.52323, 0.53515, 0.54486, 0.55288, 0.55971, 0.56565, 0.57081, 0.57519, 0.57868, 0.58108, 0.58221,\
        0.58202, 0.58060, 0.57802, 0.57429, 0.56935, 0.56312, 0.55567, 0.54724, 0.53822, 0.52902, 0.51998,\
        0.51151, 0.50395, 0.49678, 0.49012, 0.48415, 0.47848, 0.47311, 0.46817, 0.46375, 0.45988, 0.45655,\
        0.45373, 0.45134, 0.44933, 0.44765, 0.44627, 0.44516, 0.44431, 0.44369, 0.44317, 0.44255, 0.44147,\
        0.43953, 0.43636, 0.43183, 0.42608, 0.41943, 0.41239, 0.40562, 0.39865, 0.39284, 0.38736, 0.38161,\
        0.37645, 0.37197, 0.36802, 0.36440, 0.36088, 0.35728, 0.35345, 0.34929, 0.34475, 0.33978, 0.33437],\
        [0.38035, 0.38990, 0.39963, 0.41021, 0.42153, 0.43312, 0.44462, 0.45578, 0.46638, 0.47622, 0.48508,\
        0.49282, 0.49946, 0.50510, 0.51009, 0.51468, 0.51899, 0.52302, 0.52673, 0.53004, 0.53285, 0.53507,\
        0.53657, 0.53726, 0.53705, 0.53584, 0.53360, 0.53038, 0.52632, 0.52170, 0.51679, 0.51188, 0.50677,\
        0.50126, 0.49552, 0.49010, 0.48459, 0.47833, 0.47206, 0.46612, 0.46053, 0.45526, 0.45029, 0.44571,\
        0.44163, 0.43822, 0.43576, 0.43466, 0.43552, 0.43912, 0.44619, 0.45704, 0.47110, 0.48724, 0.50365,\
        0.51820, 0.52971, 0.53863, 0.54605, 0.55261, 0.55848, 0.56362, 0.56794, 0.57132, 0.57362, 0.57471,\
        0.57454, 0.57311, 0.57046, 0.56659, 0.56148, 0.55515, 0.54772, 0.53943, 0.53063, 0.52172, 0.51296,\
        0.50456, 0.49748, 0.49090, 0.48458, 0.47869, 0.47313, 0.46787, 0.46297, 0.45848, 0.45445, 0.45088,\
        0.44776, 0.44503, 0.44266, 0.44056, 0.43871, 0.43706, 0.43564, 0.43446, 0.43345, 0.43247, 0.43119,\
        0.42915, 0.42592, 0.42138, 0.41576, 0.40945, 0.40285, 0.39682, 0.39107, 0.38528, 0.37985, 0.37449,\
        0.36943, 0.36482, 0.36060, 0.35666, 0.35281, 0.34892, 0.34484, 0.34049, 0.33580, 0.33072, 0.32523],\
        [0.38303, 0.39307, 0.40328, 0.41364, 0.42459, 0.43586, 0.44714, 0.45816, 0.46869, 0.47850, 0.48731,\
        0.49481, 0.50075, 0.50555, 0.50954, 0.51312, 0.51650, 0.51970, 0.52263, 0.52521, 0.52734, 0.52891,\
        0.52983, 0.52999, 0.52930, 0.52765, 0.52497, 0.52122, 0.51659, 0.51153, 0.50609, 0.50111, 0.49601,\
        0.49053, 0.48488, 0.47930, 0.47367, 0.46779, 0.46188, 0.45619, 0.45081, 0.44575, 0.44102, 0.43665,\
        0.43270, 0.42928, 0.42655, 0.42486, 0.42483, 0.42737, 0.43379, 0.44501, 0.46067, 0.47907, 0.49754,\
        0.51307, 0.52359, 0.53123, 0.53809, 0.54461, 0.55061, 0.55589, 0.56029, 0.56369, 0.56599, 0.56710,\
        0.56698, 0.56560, 0.56296, 0.55906, 0.55391, 0.54757, 0.54018, 0.53200, 0.52340, 0.51484, 0.50673,\
        0.49882, 0.49170, 0.48538, 0.47933, 0.47356, 0.46805, 0.46282, 0.45788, 0.45328, 0.44906, 0.44524,\
        0.44182, 0.43877, 0.43604, 0.43355, 0.43122, 0.42902, 0.42700, 0.42523, 0.42376, 0.42254, 0.42131,\
        0.41947, 0.41631, 0.41173, 0.40621, 0.40026, 0.39424, 0.38849, 0.38304, 0.37762, 0.37231, 0.36714,\
        0.36217, 0.35748, 0.35307, 0.34885, 0.34472, 0.34054, 0.33621, 0.33165, 0.32678, 0.32157, 0.31597],\
        [0.38572, 0.39590, 0.40628, 0.41681, 0.42760, 0.43859, 0.44958, 0.46035, 0.47069, 0.48036, 0.48910,\
        0.49652, 0.50189, 0.50544, 0.50826, 0.51088, 0.51345, 0.51592, 0.51818, 0.52011, 0.52163, 0.52263,\
        0.52304, 0.52275, 0.52168, 0.51974, 0.51679, 0.51268, 0.50745, 0.50225, 0.49656, 0.49111, 0.48581,\
        0.48027, 0.47459, 0.46889, 0.46319, 0.45747, 0.45183, 0.44642, 0.44132, 0.43654, 0.43208, 0.42796,\
        0.42420, 0.42083, 0.41781, 0.41541, 0.41420, 0.41501, 0.41999, 0.43120, 0.44825, 0.46882, 0.48998,\
        0.50661, 0.51493, 0.52141, 0.52846, 0.53554, 0.54204, 0.54766, 0.55227, 0.55580, 0.55819, 0.55941,\
        0.55941, 0.55815, 0.55563, 0.55184, 0.54678, 0.54053, 0.53323, 0.52516, 0.51670, 0.50837, 0.50067,\
        0.49373, 0.48685, 0.48051, 0.47454, 0.46880, 0.46328, 0.45799, 0.45295, 0.44818, 0.44374, 0.43966,\
        0.43595, 0.43260, 0.42957, 0.42675, 0.42400, 0.42125, 0.41859, 0.41623, 0.41430, 0.41285, 0.41187,\
        0.41061, 0.40763, 0.40291, 0.39749, 0.39183, 0.38614, 0.38059, 0.37522, 0.36994, 0.36474, 0.35966,\
        0.35472, 0.34996, 0.34539, 0.34095, 0.33655, 0.33211, 0.32754, 0.32276, 0.31770, 0.31233, 0.30661],\
        [0.38832, 0.39860, 0.40902, 0.41957, 0.43023, 0.44094, 0.45158, 0.46199, 0.47199, 0.48135, 0.48980,\
        0.49682, 0.50159, 0.50411, 0.50592, 0.50778, 0.50977, 0.51168, 0.51337, 0.51474, 0.51572, 0.51621,\
        0.51617, 0.51551, 0.51411, 0.51196, 0.50897, 0.50478, 0.49936, 0.49347, 0.48784, 0.48201, 0.47628,\
        0.47051, 0.46469, 0.45887, 0.45310, 0.44744, 0.44199, 0.43685, 0.43206, 0.42761, 0.42346, 0.41960,\
        0.41610, 0.41297, 0.40984, 0.40660, 0.40443, 0.40299, 0.40567, 0.41633, 0.43413, 0.45540, 0.47679,\
        0.49318, 0.50154, 0.50879, 0.51748, 0.52569, 0.53294, 0.53904, 0.54394, 0.54766, 0.55024, 0.55165,\
        0.55186, 0.55084, 0.54855, 0.54500, 0.54018, 0.53416, 0.52705, 0.51913, 0.51082, 0.50275, 0.49541,\
        0.48879, 0.48238, 0.47619, 0.47022, 0.46444, 0.45884, 0.45343, 0.44822, 0.44325, 0.43855, 0.43418,\
        0.43016, 0.42654, 0.42328, 0.42024, 0.41721, 0.41398, 0.41066, 0.40771, 0.40542, 0.40356, 0.40281,\
        0.40239, 0.39968, 0.39479, 0.38942, 0.38389, 0.37834, 0.37288, 0.36754, 0.36230, 0.35714, 0.35207,\
        0.34712, 0.34228, 0.33756, 0.33292, 0.32829, 0.32361, 0.31880, 0.31380, 0.30855, 0.30301, 0.29715],\
        [0.39067, 0.40112, 0.41145, 0.42187, 0.43231, 0.44269, 0.45290, 0.46284, 0.47232, 0.48114, 0.48903,\
        0.49556, 0.49986, 0.50122, 0.50243, 0.50386, 0.50554, 0.50705, 0.50822, 0.50911, 0.50964, 0.50964,\
        0.50919, 0.50826, 0.50649, 0.50408, 0.50107, 0.49686, 0.49136, 0.48537, 0.47934, 0.47328, 0.46724,\
        0.46122, 0.45521, 0.44924, 0.44339, 0.43773, 0.43239, 0.42750, 0.42305, 0.41896, 0.41513, 0.41150,\
        0.40818, 0.40545, 0.40308, 0.39859, 0.39545, 0.39292, 0.39402, 0.40336, 0.42020, 0.44013, 0.45976,\
        0.47560, 0.48654, 0.49624, 0.50636, 0.51562, 0.52362, 0.53016, 0.53533, 0.53929, 0.54212, 0.54382,\
        0.54433, 0.54365, 0.54173, 0.53856, 0.53415, 0.52852, 0.52177, 0.51413, 0.50603, 0.49825, 0.49132,\
        0.48483, 0.47853, 0.47237, 0.46633, 0.46045, 0.45471, 0.44912, 0.44372, 0.43850, 0.43352, 0.42882,\
        0.42448, 0.42058, 0.41715, 0.41405, 0.41096, 0.40745, 0.40351, 0.39998, 0.39754, 0.39550, 0.39446,\
        0.39368, 0.39115, 0.38667, 0.38149, 0.37608, 0.37063, 0.36522, 0.35990, 0.35466, 0.34949, 0.34440,\
        0.33938, 0.33445, 0.32958, 0.32475, 0.31991, 0.31500, 0.30997, 0.30476, 0.29932, 0.29360, 0.28758],\
        [0.39242, 0.40282, 0.41328, 0.42360, 0.43375, 0.44371, 0.45340, 0.46274, 0.47156, 0.47966, 0.48674,\
        0.49235, 0.49584, 0.49738, 0.49838, 0.49953, 0.50102, 0.50213, 0.50277, 0.50324, 0.50347, 0.50294,\
        0.50204, 0.50101, 0.49884, 0.49606, 0.49275, 0.48843, 0.48309, 0.47715, 0.47100, 0.46477, 0.45854,\
        0.45232, 0.44614, 0.44003, 0.43406, 0.42835, 0.42306, 0.41838, 0.41431, 0.41054, 0.40706, 0.40363,\
        0.40030, 0.39768, 0.39550, 0.39152, 0.38750, 0.38521, 0.38669, 0.39444, 0.40839, 0.42566, 0.44350,\
        0.45941, 0.47260, 0.48435, 0.49548, 0.50557, 0.51426, 0.52105, 0.52642, 0.53065, 0.53381, 0.53588,\
        0.53680, 0.53654, 0.53510, 0.53247, 0.52864, 0.52360, 0.51742, 0.51026, 0.50259, 0.49517, 0.48830,\
        0.48173, 0.47532, 0.46902, 0.46283, 0.45676, 0.45083, 0.44505, 0.43942, 0.43396, 0.42869, 0.42364,\
        0.41891, 0.41467, 0.41108, 0.40804, 0.40509, 0.40174, 0.39740, 0.39358, 0.39064, 0.38838, 0.38670,\
        0.38499, 0.38225, 0.37822, 0.37339, 0.36819, 0.36287, 0.35752, 0.35221, 0.34695, 0.34175, 0.33660,\
        0.33151, 0.32647, 0.32146, 0.31645, 0.31141, 0.30629, 0.30104, 0.29562, 0.28999, 0.28410, 0.27792],\
        [0.39406, 0.40430, 0.41469, 0.42475, 0.43448, 0.44391, 0.45301, 0.46167, 0.46975, 0.47705, 0.48329,\
        0.48812, 0.49130, 0.49306, 0.49415, 0.49514, 0.49619, 0.49686, 0.49712, 0.49721, 0.49709, 0.49619,\
        0.49493, 0.49342, 0.49110, 0.48804, 0.48438, 0.47992, 0.47467, 0.46884, 0.46271, 0.45642, 0.45009,\
        0.44375, 0.43745, 0.43124, 0.42519, 0.41939, 0.41404, 0.40946, 0.40590, 0.40234, 0.39908, 0.39589,\
        0.39265, 0.38986, 0.38712, 0.38380, 0.38061, 0.37912, 0.38098, 0.38749, 0.39857, 0.41323, 0.42966,\
        0.44548, 0.45976, 0.47272, 0.48470, 0.49568, 0.50470, 0.51153, 0.51713, 0.52169, 0.52530, 0.52782,\
        0.52920, 0.52944, 0.52856, 0.52657, 0.52346, 0.51923, 0.51385, 0.50738, 0.50019, 0.49304, 0.48606,\
        0.47923, 0.47255, 0.46599, 0.45957, 0.45328, 0.44714, 0.44114, 0.43530, 0.42962, 0.42408, 0.41871,\
        0.41356, 0.40885, 0.40499, 0.40203, 0.39905, 0.39571, 0.39159, 0.38761, 0.38428, 0.38155, 0.37918,\
        0.37672, 0.37365, 0.36975, 0.36517, 0.36018, 0.35499, 0.34971, 0.34442, 0.33914, 0.33390, 0.32868,\
        0.32350, 0.31834, 0.31318, 0.30800, 0.30277, 0.29745, 0.29200, 0.28638, 0.28055, 0.27448, 0.26814],\
        [0.39581, 0.40599, 0.41578, 0.42524, 0.43440, 0.44323, 0.45168, 0.45965, 0.46700, 0.47355, 0.47907,\
        0.48336, 0.48635, 0.48822, 0.48940, 0.49026, 0.49092, 0.49127, 0.49126, 0.49100, 0.49039, 0.48928,\
        0.48772, 0.48575, 0.48317, 0.47992, 0.47603, 0.47146, 0.46625, 0.46053, 0.45446, 0.44820, 0.44185,\
        0.43549, 0.42915, 0.42291, 0.41683, 0.41099, 0.40558, 0.40090, 0.39769, 0.39449, 0.39121, 0.38812,\
        0.38508, 0.38222, 0.37945, 0.37671, 0.37451, 0.37393, 0.37609, 0.38174, 0.39094, 0.40358, 0.41868,\
        0.43390, 0.44827, 0.46161, 0.47393, 0.48501, 0.49415, 0.50151, 0.50740, 0.51242, 0.51662, 0.51963,\
        0.52147, 0.52222, 0.52193, 0.52062, 0.51832, 0.51502, 0.51067, 0.50510, 0.49842, 0.49132, 0.48408,\
        0.47692, 0.46990, 0.46306, 0.45637, 0.44986, 0.44350, 0.43731, 0.43129, 0.42542, 0.41971, 0.41412,\
        0.40864, 0.40341, 0.39910, 0.39598, 0.39283, 0.38932, 0.38543, 0.38152, 0.37795, 0.37477, 0.37181,\
        0.36879, 0.36537, 0.36141, 0.35693, 0.35208, 0.34699, 0.34177, 0.33650, 0.33120, 0.32591, 0.32062,\
        0.31533, 0.31005, 0.30474, 0.29940, 0.29398, 0.28847, 0.28283, 0.27701, 0.27100, 0.26475, 0.25824],\
        [0.39675, 0.40674, 0.41592, 0.42477, 0.43334, 0.44157, 0.44940, 0.45672, 0.46341, 0.46931, 0.47427,\
        0.47816, 0.48098, 0.48287, 0.48408, 0.48483, 0.48525, 0.48533, 0.48507, 0.48449, 0.48354, 0.48215,\
        0.48032, 0.47802, 0.47518, 0.47174, 0.46771, 0.46308, 0.45790, 0.45227, 0.44630, 0.44012, 0.43383,\
        0.42751, 0.42122, 0.41504, 0.40903, 0.40329, 0.39793, 0.39344, 0.38990, 0.38677, 0.38367, 0.38067,\
        0.37780, 0.37510, 0.37260, 0.37047, 0.36916, 0.36939, 0.37191, 0.37727, 0.38566, 0.39703, 0.41049,\
        0.42442, 0.43812, 0.45117, 0.46328, 0.47416, 0.48347, 0.49121, 0.49736, 0.50319, 0.50788, 0.51124,\
        0.51350, 0.51474, 0.51501, 0.51436, 0.51282, 0.51041, 0.50713, 0.50283, 0.49665, 0.48927, 0.48175,\
        0.47431, 0.46703, 0.45994, 0.45304, 0.44632, 0.43979, 0.43344, 0.42726, 0.42127, 0.41546, 0.40983,\
        0.40435, 0.39883, 0.39416, 0.39033, 0.38675, 0.38304, 0.37914, 0.37523, 0.37148, 0.36794, 0.36453,\
        0.36105, 0.35732, 0.35322, 0.34873, 0.34393, 0.33889, 0.33371, 0.32844, 0.32312, 0.31776, 0.31239,\
        0.30701, 0.30159, 0.29614, 0.29063, 0.28505, 0.27935, 0.27352, 0.26752, 0.26132, 0.25489, 0.24822]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=40 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax40vt7.5
    zgrid_H_Hmax_vmax40vt7_5=[[0.284535, 0.287365, 0.289475, 0.291108, 0.292470, 0.293713, 0.294926, 0.296150, 0.297378, 0.298559,\
        0.299589, 0.300092, 0.300111, 0.300002, 0.299822, 0.299617, 0.299417, 0.299236, 0.299078, 0.298943,\
        0.298829, 0.298736, 0.298660, 0.298602, 0.298560, 0.298529, 0.298506, 0.298484, 0.298458, 0.298417,\
        0.298359, 0.298308, 0.298335, 0.298523, 0.298937, 0.299625, 0.300617, 0.301942, 0.303623, 0.305683,\
        0.308146, 0.311039, 0.314387, 0.318219, 0.322560, 0.327436, 0.332869, 0.338875, 0.345465, 0.352646,\
        0.360426, 0.368844, 0.378057, 0.388445, 0.400694, 0.416717, 0.433723, 0.449948, 0.463926, 0.474112,\
        0.479021, 0.478203, 0.473016, 0.465463, 0.457130, 0.449114, 0.442131, 0.436607, 0.432739, 0.430539,\
        0.429875, 0.430525, 0.432214, 0.434654, 0.437575, 0.440743, 0.443964, 0.447092, 0.450017, 0.452660,\
        0.454973, 0.456926, 0.458511, 0.459734, 0.460610, 0.461163, 0.461418, 0.461405, 0.461156, 0.460701,\
        0.460068, 0.459280, 0.458354, 0.457300, 0.456124, 0.454829, 0.453415, 0.451878, 0.450216, 0.448421,\
        0.446486, 0.444405, 0.442169, 0.439771, 0.437203, 0.434459, 0.431534, 0.428422, 0.425119, 0.421620,\
        0.417921, 0.414021, 0.409917, 0.405612, 0.401128, 0.396624, 0.390972, 0.384786, 0.378380, 0.371793,\
        0.364998],\
        [0.288661, 0.290955, 0.292473, 0.293544, 0.294417, 0.295248, 0.296120, 0.297058, 0.298032, 0.298960,\
        0.299681, 0.300090, 0.300307, 0.300304, 0.300183, 0.300023, 0.299867, 0.299730, 0.299616, 0.299520,\
        0.299441, 0.299379, 0.299332, 0.299303, 0.299288, 0.299285, 0.299290, 0.299298, 0.299303, 0.299298,\
        0.299257, 0.299189, 0.299203, 0.299409, 0.299874, 0.300635, 0.301719, 0.303147, 0.304943, 0.307131,\
        0.309737, 0.312791, 0.316326, 0.320377, 0.324978, 0.330165, 0.335971, 0.342426, 0.349550, 0.357359,\
        0.365866, 0.375080, 0.385036, 0.396512, 0.410194, 0.426444, 0.443987, 0.460933, 0.475728, 0.486619,\
        0.491271, 0.488396, 0.480673, 0.470980, 0.460984, 0.451769, 0.444019, 0.438125, 0.434238, 0.432311,\
        0.432137, 0.433406, 0.435767, 0.438867, 0.442388, 0.446069, 0.449706, 0.453153, 0.456308, 0.459106,\
        0.461503, 0.463483, 0.465045, 0.466207, 0.466995, 0.467441, 0.467576, 0.467434, 0.467054, 0.466472,\
        0.465724, 0.464834, 0.463820, 0.462691, 0.461452, 0.460103, 0.458645, 0.457073, 0.455381, 0.453563,\
        0.451610, 0.449513, 0.447262, 0.444850, 0.442268, 0.439510, 0.436570, 0.433443, 0.430125, 0.426610,\
        0.422897, 0.418984, 0.414873, 0.410583, 0.406164, 0.401367, 0.395561, 0.389472, 0.383205, 0.376801,\
        0.370199],\
        [0.292487, 0.294155, 0.295013, 0.295506, 0.295901, 0.296344, 0.296900, 0.297567, 0.298303, 0.299027,\
        0.299631, 0.300051, 0.300287, 0.300336, 0.300266, 0.300156, 0.300055, 0.299976, 0.299913, 0.299861,\
        0.299818, 0.299785, 0.299765, 0.299759, 0.299764, 0.299778, 0.299798, 0.299817, 0.299836, 0.299860,\
        0.299861, 0.299780, 0.299815, 0.300089, 0.300650, 0.301522, 0.302726, 0.304279, 0.306203, 0.308524,\
        0.311273, 0.314487, 0.318204, 0.322468, 0.327324, 0.332821, 0.339007, 0.345930, 0.353631, 0.362140,\
        0.371492, 0.381776, 0.392945, 0.404863, 0.419771, 0.436476, 0.454025, 0.470952, 0.485759, 0.496677,\
        0.500809, 0.495670, 0.486063, 0.474831, 0.463633, 0.453578, 0.445327, 0.439271, 0.435553, 0.434075,\
        0.434550, 0.436576, 0.439710, 0.443531, 0.447673, 0.451853, 0.455867, 0.459581, 0.462911, 0.465805,\
        0.468233, 0.470185, 0.471674, 0.472730, 0.473393, 0.473700, 0.473687, 0.473389, 0.472851, 0.472122,\
        0.471242, 0.470240, 0.469131, 0.467920, 0.466611, 0.465204, 0.463697, 0.462085, 0.460360, 0.458515,\
        0.456539, 0.454422, 0.452151, 0.449719, 0.447117, 0.444336, 0.441372, 0.438220, 0.434874, 0.431329,\
        0.427582, 0.423627, 0.419460, 0.415070, 0.410397, 0.405229, 0.399569, 0.393645, 0.387599, 0.381481,\
        0.375138],\
        [0.295984, 0.296893, 0.297072, 0.297048, 0.297030, 0.297150, 0.297448, 0.297895, 0.298434, 0.299022,\
        0.299599, 0.300047, 0.300249, 0.300285, 0.300222, 0.300134, 0.300078, 0.300052, 0.300034, 0.300018,\
        0.300003, 0.299991, 0.299987, 0.299994, 0.300007, 0.300020, 0.300034, 0.300028, 0.300004, 0.299993,\
        0.300008, 0.300007, 0.300175, 0.300597, 0.301312, 0.302338, 0.303688, 0.305379, 0.307435, 0.309887,\
        0.312771, 0.316131, 0.320014, 0.324469, 0.329554, 0.335331, 0.341864, 0.349223, 0.357475, 0.366666,\
        0.376798, 0.387887, 0.400209, 0.413599, 0.428785, 0.445704, 0.463105, 0.479428, 0.493273, 0.502630,\
        0.504869, 0.499482, 0.489574, 0.477520, 0.465726, 0.455233, 0.446692, 0.440594, 0.437156, 0.436248,\
        0.437484, 0.440353, 0.444315, 0.448869, 0.453605, 0.458221, 0.462528, 0.466419, 0.469836, 0.472752,\
        0.475148, 0.477013, 0.478370, 0.479273, 0.479773, 0.479915, 0.479733, 0.479256, 0.478534, 0.477636,\
        0.476614, 0.475494, 0.474285, 0.472989, 0.471606, 0.470135, 0.468574, 0.466917, 0.465157, 0.463282,\
        0.461280, 0.459138, 0.456844, 0.454386, 0.451756, 0.448946, 0.445951, 0.442764, 0.439380, 0.435795,\
        0.431999, 0.427982, 0.423727, 0.419200, 0.414331, 0.409048, 0.403389, 0.397494, 0.391749, 0.386003,\
        0.379936],\
        [0.299033, 0.299000, 0.298699, 0.298325, 0.297993, 0.297887, 0.298014, 0.298305, 0.298669, 0.299121,\
        0.299648, 0.300093, 0.300292, 0.300307, 0.300209, 0.300083, 0.300032, 0.300037, 0.300040, 0.300040,\
        0.300037, 0.300030, 0.300029, 0.300041, 0.300049, 0.300034, 0.300018, 0.299940, 0.299826, 0.299757,\
        0.299803, 0.300006, 0.300389, 0.301024, 0.301946, 0.303159, 0.304671, 0.306502, 0.308684, 0.311253,\
        0.314253, 0.317735, 0.321751, 0.326362, 0.331630, 0.337629, 0.344435, 0.352140, 0.360839, 0.370621,\
        0.381489, 0.393198, 0.405923, 0.420356, 0.436300, 0.453583, 0.471108, 0.486896, 0.499051, 0.507222,\
        0.508635, 0.503422, 0.492707, 0.480500, 0.468767, 0.458060, 0.449155, 0.442899, 0.439701, 0.439372,\
        0.441387, 0.445103, 0.449872, 0.455106, 0.460342, 0.465268, 0.469730, 0.473667, 0.477057, 0.479904,\
        0.482204, 0.483918, 0.485078, 0.485786, 0.486091, 0.486050, 0.485695, 0.485018, 0.484083, 0.482998,\
        0.481831, 0.480595, 0.479287, 0.477902, 0.476440, 0.474900, 0.473281, 0.471576, 0.469776, 0.467869,\
        0.465839, 0.463670, 0.461348, 0.458860, 0.456196, 0.453350, 0.450315, 0.447087, 0.443661, 0.440029,\
        0.436181, 0.432102, 0.427770, 0.423150, 0.418196, 0.412877, 0.407223, 0.401421, 0.395996, 0.390596,\
        0.384708],\
        [0.300803, 0.300684, 0.300231, 0.299624, 0.299107, 0.298876, 0.298908, 0.299125, 0.299302, 0.299571,\
        0.300044, 0.300427, 0.300617, 0.300597, 0.300426, 0.300180, 0.299994, 0.299984, 0.299982, 0.299978,\
        0.299960, 0.299951, 0.299947, 0.299945, 0.299931, 0.299898, 0.299840, 0.299721, 0.299575, 0.299469,\
        0.299546, 0.299950, 0.300598, 0.301518, 0.302685, 0.304093, 0.305760, 0.307717, 0.310001, 0.312660,\
        0.315745, 0.319313, 0.323423, 0.328139, 0.333532, 0.339676, 0.346653, 0.354561, 0.363513, 0.373648,\
        0.385112, 0.397813, 0.410846, 0.425462, 0.441904, 0.459715, 0.477964, 0.494663, 0.506633, 0.513844,\
        0.514894, 0.508892, 0.497737, 0.486893, 0.475309, 0.464081, 0.454111, 0.447196, 0.443999, 0.444099,\
        0.446755, 0.451192, 0.456646, 0.462426, 0.467995, 0.473032, 0.477448, 0.481268, 0.484491, 0.487150,\
        0.489314, 0.490826, 0.491698, 0.492207, 0.492285, 0.492050, 0.491560, 0.490660, 0.489470, 0.488191,\
        0.486893, 0.485555, 0.484150, 0.482671, 0.481122, 0.479507, 0.477823, 0.476065, 0.474223, 0.472282,\
        0.470222, 0.468025, 0.465672, 0.463149, 0.460446, 0.457556, 0.454477, 0.451203, 0.447731, 0.444053,\
        0.440159, 0.436035, 0.431660, 0.427011, 0.422067, 0.416835, 0.411382, 0.405906, 0.400752, 0.395299,\
        0.389458],\
        [0.301826, 0.302347, 0.302053, 0.301400, 0.300899, 0.300557, 0.300505, 0.300657, 0.300700, 0.300808,\
        0.301127, 0.301383, 0.301468, 0.301364, 0.301106, 0.300761, 0.300450, 0.300293, 0.300211, 0.300159,\
        0.300102, 0.300090, 0.300070, 0.300031, 0.299984, 0.299958, 0.299917, 0.299777, 0.299668, 0.299528,\
        0.299604, 0.300091, 0.301093, 0.302312, 0.303697, 0.305266, 0.307050, 0.309093, 0.311442, 0.314151,\
        0.317279, 0.320888, 0.325043, 0.329812, 0.335264, 0.341473, 0.348516, 0.356475, 0.365445, 0.375531,\
        0.386860, 0.399507, 0.413279, 0.428595, 0.445568, 0.463852, 0.482805, 0.501050, 0.514885, 0.523637,\
        0.526096, 0.521720, 0.512481, 0.501009, 0.488751, 0.475831, 0.463055, 0.454634, 0.451030, 0.451150,\
        0.454055, 0.458906, 0.464807, 0.470912, 0.476581, 0.481452, 0.485549, 0.489089, 0.492019, 0.494306,\
        0.496306, 0.497628, 0.498073, 0.498488, 0.498283, 0.497841, 0.497330, 0.496139, 0.494661, 0.493215,\
        0.491827, 0.490401, 0.488892, 0.487309, 0.485662, 0.483962, 0.482206, 0.480390, 0.478502, 0.476527,\
        0.474438, 0.472211, 0.469824, 0.467261, 0.464512, 0.461574, 0.458444, 0.455122, 0.451604, 0.447885,\
        0.443956, 0.439806, 0.435423, 0.430795, 0.425926, 0.420845, 0.415633, 0.410421, 0.405232, 0.399826,\
        0.394113],\
        [0.302947, 0.304046, 0.304218, 0.303925, 0.303540, 0.303225, 0.303077, 0.303048, 0.302997, 0.302978,\
        0.303034, 0.303063, 0.302974, 0.302742, 0.302391, 0.301983, 0.301607, 0.301331, 0.301146, 0.301018,\
        0.300928, 0.300875, 0.300832, 0.300782, 0.300734, 0.300700, 0.300646, 0.300543, 0.300477, 0.300395,\
        0.300613, 0.301243, 0.302327, 0.303657, 0.305145, 0.306792, 0.308628, 0.310698, 0.313057, 0.315765,\
        0.318885, 0.322485, 0.326633, 0.331400, 0.336854, 0.343062, 0.350089, 0.357998, 0.366849, 0.376688,\
        0.387538, 0.399625, 0.414136, 0.430247, 0.447807, 0.466579, 0.485918, 0.505471, 0.523569, 0.536388,\
        0.542390, 0.541468, 0.535273, 0.525305, 0.512932, 0.495206, 0.477466, 0.466707, 0.462003, 0.461177,\
        0.463567, 0.468326, 0.474323, 0.480443, 0.485942, 0.490348, 0.493745, 0.496894, 0.499482, 0.501294,\
        0.502891, 0.504011, 0.504311, 0.504437, 0.504097, 0.503509, 0.502777, 0.501369, 0.499703, 0.498134,\
        0.496703, 0.495173, 0.493535, 0.491830, 0.490068, 0.488276, 0.486439, 0.484554, 0.482618, 0.480610,\
        0.478495, 0.476237, 0.473812, 0.471202, 0.468401, 0.465407, 0.462222, 0.458849, 0.455287, 0.451533,\
        0.447580, 0.443422, 0.439052, 0.434470, 0.429689, 0.424744, 0.419694, 0.414596, 0.409445, 0.404178,\
        0.398425],\
        [0.304338, 0.306001, 0.306743, 0.306913, 0.306807, 0.306614, 0.306429, 0.306259, 0.306071, 0.305876,\
        0.305686, 0.305463, 0.305162, 0.304771, 0.304309, 0.303824, 0.303378, 0.303015, 0.302745, 0.302553,\
        0.302425, 0.302345, 0.302296, 0.302261, 0.302238, 0.302225, 0.302207, 0.302185, 0.302191, 0.302290,\
        0.302635, 0.303327, 0.304367, 0.305657, 0.307124, 0.308750, 0.310556, 0.312584, 0.314890, 0.317537,\
        0.320593, 0.324130, 0.328219, 0.332933, 0.338339, 0.344499, 0.351470, 0.359298, 0.368033, 0.377741,\
        0.388569, 0.400876, 0.415088, 0.431330, 0.449401, 0.469131, 0.490192, 0.512430, 0.534236, 0.551678,\
        0.561940, 0.564509, 0.562151, 0.556649, 0.545930, 0.525677, 0.500247, 0.486081, 0.477995, 0.474312,\
        0.475155, 0.479195, 0.484853, 0.490579, 0.495496, 0.499354, 0.501806, 0.504355, 0.506518, 0.508081,\
        0.509281, 0.510064, 0.510324, 0.510219, 0.509762, 0.509016, 0.507970, 0.506509, 0.504809, 0.503140,\
        0.501575, 0.499872, 0.498095, 0.496250, 0.494344, 0.492467, 0.490531, 0.488559, 0.486574, 0.484539,\
        0.482399, 0.480111, 0.477643, 0.474979, 0.472115, 0.469057, 0.465812, 0.462387, 0.458783, 0.454998,\
        0.451029, 0.446872, 0.442524, 0.437988, 0.433280, 0.428422, 0.423445, 0.418364, 0.413144, 0.407646,\
        0.401510],\
        [0.306070, 0.308304, 0.309631, 0.310303, 0.310550, 0.310544, 0.310391, 0.310140, 0.309810, 0.309423,\
        0.308994, 0.308522, 0.307998, 0.307428, 0.306832, 0.306251, 0.305725, 0.305289, 0.304953, 0.304713,\
        0.304557, 0.304469, 0.304433, 0.304433, 0.304459, 0.304503, 0.304561, 0.304638, 0.304765, 0.305005,\
        0.305444, 0.306145, 0.307110, 0.308294, 0.309654, 0.311173, 0.312870, 0.314784, 0.316970, 0.319494,\
        0.322427, 0.325845, 0.329823, 0.334437, 0.339755, 0.345840, 0.352741, 0.360500, 0.369157, 0.378798,\
        0.389630, 0.401886, 0.415886, 0.432102, 0.450651, 0.471559, 0.494952, 0.520300, 0.545635, 0.567581,\
        0.582032, 0.586183, 0.586928, 0.586940, 0.582813, 0.564976, 0.535709, 0.512833, 0.497674, 0.489603,\
        0.488226, 0.490986, 0.495378, 0.500635, 0.504194, 0.507391, 0.509577, 0.511516, 0.513216, 0.514521,\
        0.515448, 0.515988, 0.516117, 0.515872, 0.515286, 0.514386, 0.513173, 0.511654, 0.509936, 0.508165,\
        0.506382, 0.504519, 0.502595, 0.500542, 0.498518, 0.496577, 0.494481, 0.492394, 0.490375, 0.488323,\
        0.486159, 0.483836, 0.481319, 0.478589, 0.475651, 0.472520, 0.469211, 0.465732, 0.462089, 0.458279,\
        0.454299, 0.450146, 0.445818, 0.441318, 0.436653, 0.431835, 0.426868, 0.421738, 0.416383, 0.410677,\
        0.404503],\
        [0.308173, 0.310988, 0.312900, 0.314086, 0.314719, 0.314946, 0.314876, 0.314585, 0.314127, 0.313549,\
        0.312888, 0.312173, 0.311424, 0.310663, 0.309919, 0.309225, 0.308614, 0.308108, 0.307720, 0.307448,\
        0.307280, 0.307203, 0.307200, 0.307254, 0.307351, 0.307481, 0.307638, 0.307829, 0.308076, 0.308422,\
        0.308917, 0.309597, 0.310469, 0.311514, 0.312712, 0.314061, 0.315582, 0.317314, 0.319315, 0.321652,\
        0.324402, 0.327645, 0.331461, 0.335929, 0.341124, 0.347112, 0.353945, 0.361659, 0.370268, 0.379789,\
        0.390364, 0.402369, 0.416177, 0.432542, 0.451561, 0.473413, 0.498442, 0.526582, 0.554963, 0.580220,\
        0.598238, 0.601556, 0.604099, 0.606971, 0.607439, 0.593646, 0.563360, 0.536910, 0.517722, 0.506065,\
        0.502186, 0.503109, 0.505307, 0.509034, 0.512116, 0.514639, 0.516623, 0.518257, 0.519627, 0.520688,\
        0.521411, 0.521778, 0.521778, 0.521419, 0.520722, 0.519705, 0.518386, 0.516801, 0.515023, 0.513135,\
        0.511175, 0.509138, 0.507024, 0.504836, 0.502703, 0.500578, 0.498241, 0.496059, 0.494052, 0.491975,\
        0.489774, 0.487412, 0.484837, 0.482027, 0.479000, 0.475787, 0.472410, 0.468881, 0.465202, 0.461371,\
        0.457383, 0.453235, 0.448921, 0.444442, 0.439797, 0.434986, 0.429997, 0.424802, 0.419345, 0.413553,\
        0.407379],\
        [0.310653, 0.314068, 0.316571, 0.318278, 0.319314, 0.319800, 0.319843, 0.319535, 0.318962, 0.318197,\
        0.317310, 0.316355, 0.315379, 0.314422, 0.313519, 0.312704, 0.312002, 0.311432, 0.311003, 0.310712,\
        0.310549, 0.310500, 0.310548, 0.310674, 0.310862, 0.311097, 0.311369, 0.311677, 0.312034, 0.312461,\
        0.312983, 0.313622, 0.314384, 0.315266, 0.316267, 0.317397, 0.318684, 0.320174, 0.321928, 0.324019,\
        0.326527, 0.329538, 0.333139, 0.337416, 0.342450, 0.348316, 0.355076, 0.362774, 0.371417, 0.380917,\
        0.390988, 0.402098, 0.416281, 0.432928, 0.452330, 0.474982, 0.501402, 0.531326, 0.561015, 0.586396,\
        0.603655, 0.610228, 0.612322, 0.612666, 0.608555, 0.595398, 0.574780, 0.552681, 0.534461, 0.522165,\
        0.516120, 0.514550, 0.515232, 0.517204, 0.519450, 0.521471, 0.523179, 0.524600, 0.525761, 0.526643,\
        0.527218, 0.527461, 0.527358, 0.526909, 0.526122, 0.525014, 0.523611, 0.521949, 0.520085, 0.518074,\
        0.515955, 0.513744, 0.511457, 0.509122, 0.506779, 0.504385, 0.501895, 0.499699, 0.497660, 0.495481,\
        0.493219, 0.490827, 0.488186, 0.485271, 0.482138, 0.478839, 0.475400, 0.471828, 0.468121, 0.464276,\
        0.460284, 0.456138, 0.451833, 0.447363, 0.442722, 0.437899, 0.432878, 0.427627, 0.422106, 0.416269,\
        0.410090],\
        [0.313509, 0.317554, 0.320664, 0.322903, 0.324355, 0.325114, 0.325282, 0.324967, 0.324278, 0.323324,\
        0.322206, 0.321008, 0.319803, 0.318646, 0.317581, 0.316641, 0.315849, 0.315219, 0.314757, 0.314459,\
        0.314315, 0.314310, 0.314424, 0.314639, 0.314933, 0.315286, 0.315683, 0.316112, 0.316569, 0.317059,\
        0.317591, 0.318172, 0.318810, 0.319510, 0.320284, 0.321155, 0.322161, 0.323356, 0.324808, 0.326595,\
        0.328805, 0.331528, 0.334861, 0.338896, 0.343723, 0.349428, 0.356086, 0.363762, 0.372505, 0.382338,\
        0.393168, 0.404114, 0.417344, 0.433705, 0.453195, 0.476365, 0.504026, 0.535143, 0.565381, 0.589529,\
        0.604906, 0.612221, 0.613117, 0.609942, 0.602452, 0.592089, 0.577719, 0.561329, 0.546383, 0.535160,\
        0.528339, 0.525196, 0.524480, 0.525189, 0.526544, 0.528040, 0.529446, 0.530675, 0.531688, 0.532450,\
        0.532926, 0.533086, 0.532910, 0.532391, 0.531533, 0.530350, 0.528866, 0.527118, 0.525152, 0.523012,\
        0.520738, 0.518357, 0.515894, 0.513376, 0.510824, 0.508245, 0.505700, 0.503350, 0.501092, 0.498761,\
        0.496441, 0.494061, 0.491335, 0.488275, 0.485029, 0.481658, 0.478173, 0.474573, 0.470851, 0.466998,\
        0.463005, 0.458863, 0.454562, 0.450093, 0.445446, 0.440605, 0.435551, 0.430257, 0.424691, 0.418825,\
        0.412636],\
        [0.316733, 0.321450, 0.325197, 0.327991, 0.329874, 0.330914, 0.331207, 0.330874, 0.330056, 0.328896,\
        0.327533, 0.326084, 0.324645, 0.323286, 0.322058, 0.320993, 0.320114, 0.319430, 0.318943, 0.318648,\
        0.318532, 0.318581, 0.318775, 0.319091, 0.319503, 0.319987, 0.320517, 0.321068, 0.321623, 0.322168,\
        0.322698, 0.323211, 0.323711, 0.324211, 0.324733, 0.325311, 0.325994, 0.326847, 0.327945, 0.329375,\
        0.331231, 0.333614, 0.336625, 0.340366, 0.344935, 0.350425, 0.356921, 0.364496, 0.373206, 0.383079,\
        0.393993, 0.405440, 0.418650, 0.434865, 0.454397, 0.477721, 0.505975, 0.538766, 0.570091, 0.594221,\
        0.607579, 0.611919, 0.610293, 0.605057, 0.598002, 0.588991, 0.578179, 0.566019, 0.554360, 0.544865,\
        0.538302, 0.534533, 0.532919, 0.532745, 0.533395, 0.534419, 0.535538, 0.536595, 0.537498, 0.538184,\
        0.538601, 0.538712, 0.538489, 0.537919, 0.537002, 0.535749, 0.534184, 0.532339, 0.530257, 0.527979,\
        0.525546, 0.522992, 0.520346, 0.517638, 0.514892, 0.512139, 0.509436, 0.506835, 0.504298, 0.501779,\
        0.499397, 0.497107, 0.494183, 0.490945, 0.487631, 0.484233, 0.480734, 0.477128, 0.473404, 0.469553,\
        0.465563, 0.461424, 0.457124, 0.452651, 0.447992, 0.443130, 0.438046, 0.432717, 0.427118, 0.421226,\
        0.415026],\
        [0.320312, 0.325755, 0.330187, 0.333573, 0.335910, 0.337238, 0.337644, 0.337269, 0.336286, 0.334888,\
        0.333254, 0.331538, 0.329858, 0.328297, 0.326907, 0.325723, 0.324761, 0.324028, 0.323522, 0.323235,\
        0.323154, 0.323264, 0.323544, 0.323970, 0.324511, 0.325136, 0.325808, 0.326489, 0.327144, 0.327744,\
        0.328267, 0.328703, 0.329054, 0.329337, 0.329583, 0.329837, 0.330161, 0.330628, 0.331325, 0.332349,\
        0.333802, 0.335794, 0.338432, 0.341828, 0.346085, 0.351303, 0.357571, 0.364957, 0.373487, 0.383110,\
        0.393664, 0.405546, 0.419600, 0.436279, 0.456257, 0.480165, 0.509233, 0.542675, 0.574549, 0.599078,\
        0.609305, 0.610451, 0.606284, 0.600476, 0.593907, 0.586608, 0.578095, 0.568815, 0.559770, 0.552020,\
        0.546209, 0.542444, 0.540442, 0.539759, 0.539942, 0.540618, 0.541513, 0.542440, 0.543272, 0.543920,\
        0.544314, 0.544403, 0.544154, 0.543546, 0.542576, 0.541253, 0.539599, 0.537646, 0.535434, 0.533007,\
        0.530406, 0.527671, 0.524836, 0.521931, 0.518986, 0.516032, 0.513104, 0.510226, 0.507396, 0.504634,\
        0.502064, 0.499690, 0.496377, 0.493190, 0.489954, 0.486592, 0.483112, 0.479519, 0.475805, 0.471962,\
        0.467977, 0.463841, 0.459539, 0.455059, 0.450385, 0.445502, 0.440389, 0.435028, 0.429399, 0.423484,\
        0.417270],\
        [0.324222, 0.330463, 0.335648, 0.339683, 0.342510, 0.344133, 0.344632, 0.344169, 0.342968, 0.341279,\
        0.339338, 0.337335, 0.335407, 0.333644, 0.332099, 0.330801, 0.329762, 0.328983, 0.328460, 0.328183,\
        0.328138, 0.328310, 0.328678, 0.329217, 0.329893, 0.330668, 0.331494, 0.332317, 0.333083, 0.333744,\
        0.334262, 0.334617, 0.334809, 0.334858, 0.334805, 0.334707, 0.334636, 0.334679, 0.334933, 0.335506,\
        0.336511, 0.338065, 0.340287, 0.343291, 0.347190, 0.352090, 0.358083, 0.365244, 0.373610, 0.383185,\
        0.394038, 0.406344, 0.420683, 0.437859, 0.458558, 0.483554, 0.513333, 0.546162, 0.576781, 0.598723,\
        0.608522, 0.608267, 0.602688, 0.596586, 0.591059, 0.584881, 0.578022, 0.570751, 0.563663, 0.557425,\
        0.552514, 0.549101, 0.547087, 0.546212, 0.546164, 0.546647, 0.547413, 0.548273, 0.549081, 0.549729,\
        0.550131, 0.550225, 0.549965, 0.549328, 0.548306, 0.546908, 0.545154, 0.543077, 0.540718, 0.538124,\
        0.535343, 0.532417, 0.529384, 0.526276, 0.523121, 0.519945, 0.516769, 0.513607, 0.510461, 0.507337,\
        0.504217, 0.500967, 0.497986, 0.495235, 0.492131, 0.488809, 0.485361, 0.481790, 0.478091, 0.474256,\
        0.470276, 0.466139, 0.461832, 0.457341, 0.452650, 0.447743, 0.442602, 0.437210, 0.431550, 0.425610,\
        0.419377],\
        [0.328427, 0.335556, 0.341586, 0.346351, 0.349723, 0.351655, 0.352216, 0.351600, 0.350099, 0.348049,\
        0.345755, 0.343444, 0.341266, 0.339308, 0.337616, 0.336211, 0.335099, 0.334276, 0.333734, 0.333462,\
        0.333445, 0.333671, 0.334121, 0.334770, 0.335584, 0.336518, 0.337511, 0.338493, 0.339387, 0.340123,\
        0.340644, 0.340917, 0.340941, 0.340740, 0.340365, 0.339887, 0.339391, 0.338975, 0.338749, 0.338832,\
        0.339350, 0.340428, 0.342196, 0.344774, 0.348281, 0.352827, 0.358518, 0.365450, 0.373702, 0.383319,\
        0.394303, 0.406922, 0.421681, 0.439266, 0.460473, 0.485974, 0.516084, 0.548522, 0.578234, 0.599338,\
        0.607489, 0.606142, 0.600601, 0.594717, 0.589335, 0.583906, 0.578198, 0.572348, 0.566712, 0.561713,\
        0.557684, 0.554785, 0.552998, 0.552174, 0.552097, 0.552538, 0.553283, 0.554151, 0.554992, 0.555682,\
        0.556124, 0.556242, 0.555983, 0.555320, 0.554243, 0.552759, 0.550889, 0.548668, 0.546139, 0.543359,\
        0.540381, 0.537252, 0.534014, 0.530697, 0.527327, 0.523923, 0.520500, 0.517064, 0.513616, 0.510148,\
        0.506644, 0.503169, 0.500263, 0.497544, 0.494287, 0.490969, 0.487550, 0.483998, 0.480306, 0.476472,\
        0.472489, 0.468346, 0.464028, 0.459522, 0.454810, 0.449875, 0.444702, 0.439277, 0.433585, 0.427615,\
        0.421358],\
        [0.332875, 0.340997, 0.347991, 0.353601, 0.357597, 0.359864, 0.360451, 0.359591, 0.357678, 0.355178,\
        0.352482, 0.349852, 0.347430, 0.345287, 0.343458, 0.341952, 0.340768, 0.339897, 0.339327, 0.339045,\
        0.339041, 0.339304, 0.339819, 0.340567, 0.341514, 0.342612, 0.343788, 0.344952, 0.346002, 0.346836,\
        0.347373, 0.347566, 0.347411, 0.346943, 0.346226, 0.345344, 0.344394, 0.343489, 0.342751, 0.342310,\
        0.342307, 0.342881, 0.344168, 0.346298, 0.349393, 0.353568, 0.358937, 0.365619, 0.373751, 0.383468,\
        0.394828, 0.407639, 0.422469, 0.440264, 0.461681, 0.487257, 0.517323, 0.549496, 0.578796, 0.599461,\
        0.606304, 0.603726, 0.598462, 0.593356, 0.588436, 0.583584, 0.578702, 0.573877, 0.569330, 0.565333,\
        0.562111, 0.559789, 0.558371, 0.557765, 0.557821, 0.558355, 0.559185, 0.560140, 0.561070, 0.561846,\
        0.562360, 0.562522, 0.562270, 0.561576, 0.560434, 0.558851, 0.556846, 0.554454, 0.551728, 0.548736,\
        0.545540, 0.542197, 0.538747, 0.535219, 0.531635, 0.528009, 0.524354, 0.520675, 0.516977, 0.513271,\
        0.509598, 0.506095, 0.502941, 0.499677, 0.496426, 0.493166, 0.489764, 0.486202, 0.482493, 0.478644,\
        0.474648, 0.470490, 0.466155, 0.461626, 0.456886, 0.451919, 0.446710, 0.441246, 0.435515, 0.429509,\
        0.423222],\
        [0.337494, 0.346721, 0.354827, 0.361437, 0.366173, 0.368815, 0.369393, 0.368168, 0.365690, 0.362637,\
        0.359510, 0.356570, 0.353923, 0.351611, 0.349649, 0.348040, 0.346778, 0.345847, 0.345231, 0.344917,\
        0.344898, 0.345166, 0.345717, 0.346540, 0.347607, 0.348869, 0.350244, 0.351621, 0.352862, 0.353827,\
        0.354400, 0.354515, 0.354171, 0.353418, 0.352338, 0.351032, 0.349608, 0.348189, 0.346910, 0.345918,\
        0.345368, 0.345415, 0.346209, 0.347886, 0.350572, 0.354379, 0.359413, 0.365800, 0.373703, 0.383339,\
        0.394844, 0.407856, 0.422886, 0.440904, 0.462397, 0.487873, 0.517573, 0.549127, 0.577674, 0.597580,\
        0.604302, 0.601817, 0.596954, 0.592498, 0.588111, 0.583772, 0.579525, 0.575469, 0.571767, 0.568597,\
        0.566107, 0.564378, 0.563411, 0.563142, 0.563451, 0.564192, 0.565201, 0.566315, 0.567389, 0.568294,\
        0.568911, 0.569133, 0.568887, 0.568149, 0.566926, 0.565228, 0.563065, 0.560469, 0.557509, 0.554271,\
        0.550837, 0.547267, 0.543601, 0.539865, 0.536073, 0.532239, 0.528371, 0.524480, 0.520579, 0.516695,\
        0.512878, 0.509199, 0.505659, 0.502165, 0.498846, 0.495556, 0.492083, 0.488447, 0.484689, 0.480806,\
        0.476782, 0.472598, 0.468237, 0.463678, 0.458903, 0.453895, 0.448641, 0.443130, 0.437353, 0.431303,\
        0.424977],\
        [0.342192, 0.352629, 0.362011, 0.369835, 0.375469, 0.378524, 0.379092, 0.377352, 0.374096, 0.370410,\
        0.366872, 0.363665, 0.360821, 0.358342, 0.356237, 0.354512, 0.353154, 0.352140, 0.351449, 0.351069,\
        0.350993, 0.351223, 0.351765, 0.352622, 0.353780, 0.355196, 0.356784, 0.358409, 0.359891, 0.361033,\
        0.361668, 0.361707, 0.361157, 0.360099, 0.358642, 0.356899, 0.354988, 0.353038, 0.351196, 0.349629,\
        0.348511, 0.348018, 0.348316, 0.349554, 0.351860, 0.355338, 0.360074, 0.366155, 0.373708, 0.382939,\
        0.394156, 0.407436, 0.423183, 0.441684, 0.463087, 0.488039, 0.517067, 0.547949, 0.575864, 0.595383,\
        0.602409, 0.600338, 0.596333, 0.592290, 0.588290, 0.584371, 0.580631, 0.577191, 0.574182, 0.571726,\
        0.569915, 0.568786, 0.568325, 0.568472, 0.569127, 0.570163, 0.571432, 0.572768, 0.574028, 0.575098,\
        0.575850, 0.576145, 0.575892, 0.575081, 0.573755, 0.571930, 0.569585, 0.566741, 0.563495, 0.559973,\
        0.556278, 0.552474, 0.548595, 0.544656, 0.540666, 0.536633, 0.532569, 0.528487, 0.524408, 0.520359,\
        0.516381, 0.512511, 0.508758, 0.505129, 0.501651, 0.498197, 0.494505, 0.490742, 0.486914, 0.482983,\
        0.478917, 0.474696, 0.470297, 0.465698, 0.460878, 0.455819, 0.450510, 0.444941, 0.439107, 0.433003,\
        0.426628],\
        [0.346881, 0.358594, 0.369386, 0.378693, 0.385487, 0.388860, 0.389570, 0.387107, 0.382817, 0.378554,\
        0.374706, 0.371299, 0.368262, 0.365580, 0.363291, 0.361415, 0.359929, 0.358798, 0.357994, 0.357500,\
        0.357314, 0.357444, 0.357914, 0.358746, 0.359946, 0.361490, 0.363294, 0.365205, 0.366991, 0.368375,\
        0.369108, 0.369063, 0.368280, 0.366896, 0.365057, 0.362883, 0.360487, 0.357996, 0.355574, 0.353408,\
        0.351703, 0.350660, 0.350469, 0.351298, 0.353283, 0.356523, 0.361078, 0.366977, 0.374268, 0.383093,\
        0.393661, 0.407343, 0.424551, 0.443758, 0.464794, 0.488529, 0.516527, 0.546826, 0.574443, 0.594365,\
        0.601342, 0.599037, 0.595987, 0.592493, 0.588846, 0.585284, 0.581982, 0.579078, 0.576686, 0.574891,\
        0.573738, 0.573226, 0.573313, 0.573934, 0.575000, 0.576403, 0.578002, 0.579610, 0.581068, 0.582326,\
        0.583250, 0.583634, 0.583343, 0.582397, 0.580944, 0.578998, 0.576449, 0.573290, 0.569687, 0.565837,\
        0.561867, 0.557830, 0.553744, 0.549610, 0.545429, 0.541207, 0.536955, 0.532691, 0.528437, 0.524222,\
        0.520076, 0.516025, 0.512087, 0.508271, 0.504561, 0.500812, 0.496937, 0.493081, 0.489195, 0.485201,\
        0.481077, 0.476805, 0.472357, 0.467707, 0.462830, 0.457707, 0.452329, 0.446690, 0.440785, 0.434615,\
        0.428180],\
        [0.351520, 0.364531, 0.376748, 0.387653, 0.396227, 0.399560, 0.400615, 0.396882, 0.392036, 0.387419,\
        0.383364, 0.379765, 0.376447, 0.373442, 0.370878, 0.368799, 0.367145, 0.365849, 0.364878, 0.364216,\
        0.363855, 0.363812, 0.364127, 0.364850, 0.366018, 0.367635, 0.369638, 0.371866, 0.374033, 0.375751,\
        0.376627, 0.376473, 0.375407, 0.373683, 0.371487, 0.368918, 0.366058, 0.363028, 0.360007, 0.357218,\
        0.354903, 0.353299, 0.352627, 0.353081, 0.354819, 0.357953, 0.362536, 0.368549, 0.375940, 0.384868,\
        0.396307, 0.410648, 0.428564, 0.448522, 0.469384, 0.490831, 0.517406, 0.546847, 0.573380, 0.592967,\
        0.600543, 0.598749, 0.596153, 0.592981, 0.589627, 0.586412, 0.583546, 0.581169, 0.579381, 0.578243,\
        0.577767, 0.577905, 0.578581, 0.579724, 0.581244, 0.583055, 0.585052, 0.586998, 0.588591, 0.590044,\
        0.591181, 0.591665, 0.591298, 0.590096, 0.588497, 0.586483, 0.583704, 0.580110, 0.576055, 0.571845,\
        0.567603, 0.563349, 0.559070, 0.554750, 0.550381, 0.545969, 0.541528, 0.537079, 0.532646, 0.528256,\
        0.523932, 0.519696, 0.515556, 0.511508, 0.507510, 0.503480, 0.499421, 0.495548, 0.491579, 0.487482,\
        0.483277, 0.478942, 0.474437, 0.469724, 0.464775, 0.459572, 0.454108, 0.448380, 0.442391, 0.436141,\
        0.429634],\
        [0.356167, 0.370535, 0.384218, 0.396423, 0.406325, 0.411310, 0.411825, 0.408175, 0.403160, 0.397855,\
        0.393454, 0.389433, 0.385574, 0.382007, 0.379036, 0.376713, 0.374840, 0.373310, 0.372110, 0.371224,\
        0.370623, 0.370321, 0.370383, 0.370892, 0.371921, 0.373520, 0.375666, 0.378211, 0.380833, 0.383014,\
        0.384104, 0.383764, 0.382331, 0.380289, 0.377824, 0.374945, 0.371668, 0.368101, 0.364457, 0.361014,\
        0.358062, 0.355880, 0.354726, 0.354827, 0.356379, 0.359530, 0.364373, 0.370902, 0.378973, 0.388295,\
        0.399510, 0.415910, 0.435126, 0.456161, 0.478229, 0.500296, 0.523417, 0.549043, 0.572549, 0.590322,\
        0.598826, 0.598614, 0.596357, 0.593466, 0.590461, 0.587679, 0.585330, 0.583538, 0.582389, 0.581942,\
        0.582196, 0.583041, 0.584340, 0.586057, 0.588068, 0.590289, 0.592700, 0.595153, 0.596683, 0.598335,\
        0.599695, 0.600218, 0.599757, 0.598200, 0.596454, 0.594449, 0.591366, 0.587118, 0.582537, 0.577983,\
        0.573507, 0.569063, 0.564604, 0.560100, 0.555536, 0.550923, 0.546281, 0.541637, 0.537015, 0.532439,\
        0.527928, 0.523496, 0.519147, 0.514870, 0.510636, 0.506424, 0.502297, 0.498263, 0.494068, 0.489821,\
        0.485523, 0.481122, 0.476555, 0.471767, 0.466727, 0.461422, 0.455850, 0.450015, 0.443923, 0.437578,\
        0.430987],\
        [0.360973, 0.376821, 0.392499, 0.406666, 0.417972, 0.424441, 0.425595, 0.422487, 0.417054, 0.411014,\
        0.405490, 0.400455, 0.395724, 0.391220, 0.387775, 0.385220, 0.383048, 0.381172, 0.379678, 0.378536,\
        0.377630, 0.376976, 0.376686, 0.376867, 0.377625, 0.379068, 0.381244, 0.384036, 0.387117, 0.389906,\
        0.391351, 0.390632, 0.388731, 0.386527, 0.383984, 0.380938, 0.377306, 0.373192, 0.368886, 0.364748,\
        0.361126, 0.358343, 0.356691, 0.356435, 0.357812, 0.361029, 0.366252, 0.373586, 0.383015, 0.394223,\
        0.406670, 0.423196, 0.443222, 0.465012, 0.487419, 0.509650, 0.530917, 0.552312, 0.571886, 0.586817,\
        0.594988, 0.596969, 0.595839, 0.593629, 0.591233, 0.589093, 0.587427, 0.586340, 0.585895, 0.586176,\
        0.587229, 0.588894, 0.590781, 0.593180, 0.595736, 0.598377, 0.601155, 0.603925, 0.605842, 0.607478,\
        0.608829, 0.609262, 0.608642, 0.607057, 0.605070, 0.602775, 0.599130, 0.594130, 0.589103, 0.584302,\
        0.579648, 0.575028, 0.570390, 0.565690, 0.560909, 0.556069, 0.551206, 0.546350, 0.541525, 0.536752,\
        0.532046, 0.527415, 0.522859, 0.518370, 0.513935, 0.509556, 0.505252, 0.500952, 0.496569, 0.492192,\
        0.487819, 0.483368, 0.478735, 0.473853, 0.468696, 0.463260, 0.457554, 0.451589, 0.445375, 0.438922,\
        0.432234],\
        [0.366263, 0.383459, 0.401506, 0.418725, 0.432181, 0.439968, 0.441661, 0.438516, 0.432558, 0.425605,\
        0.418786, 0.412446, 0.406559, 0.401024, 0.397241, 0.394390, 0.391747, 0.389354, 0.387528, 0.386170,\
        0.384892, 0.383789, 0.383071, 0.382832, 0.383171, 0.384280, 0.386324, 0.389216, 0.392564, 0.395839,\
        0.397968, 0.396434, 0.394216, 0.392327, 0.389962, 0.386928, 0.382987, 0.378271, 0.373243, 0.368369,\
        0.364048, 0.360638, 0.358460, 0.357811, 0.358962, 0.362171, 0.367677, 0.375681, 0.386311, 0.399378,\
        0.413593, 0.430851, 0.451390, 0.473700, 0.495740, 0.516706, 0.536563, 0.555132, 0.571479, 0.584000,\
        0.591596, 0.594634, 0.594737, 0.593530, 0.592048, 0.590820, 0.590063, 0.589851, 0.590190, 0.591210,\
        0.593023, 0.595823, 0.598026, 0.601282, 0.604537, 0.607689, 0.610808, 0.613677, 0.615948, 0.617653,\
        0.618770, 0.619007, 0.618237, 0.616551, 0.614174, 0.611053, 0.606710, 0.601357, 0.595973, 0.590999,\
        0.586161, 0.581312, 0.576474, 0.571548, 0.566502, 0.561397, 0.556285, 0.551199, 0.546159, 0.541179,\
        0.536271, 0.531439, 0.526683, 0.521993, 0.517358, 0.512770, 0.508214, 0.503652, 0.499092, 0.494606,\
        0.490200, 0.485720, 0.481006, 0.475996, 0.470682, 0.465079, 0.459209, 0.453090, 0.446737, 0.440160,\
        0.433366],\
        [0.372665, 0.391791, 0.412090, 0.432675, 0.448671, 0.457601, 0.459110, 0.455138, 0.448203, 0.440200,\
        0.432231, 0.424768, 0.417947, 0.412026, 0.407500, 0.403971, 0.400742, 0.397692, 0.395603, 0.394168,\
        0.392363, 0.390744, 0.389616, 0.388936, 0.388727, 0.389296, 0.391054, 0.393912, 0.397260, 0.400254,\
        0.402105, 0.400526, 0.399017, 0.398012, 0.395802, 0.393017, 0.388704, 0.383256, 0.377460, 0.371836,\
        0.366805, 0.362751, 0.360017, 0.358919, 0.359749, 0.362790, 0.368316, 0.376564, 0.387591, 0.401191,\
        0.417489, 0.436380, 0.457867, 0.480913, 0.503364, 0.522724, 0.540831, 0.557324, 0.571447, 0.582285,\
        0.589300, 0.592793, 0.593816, 0.593643, 0.593280, 0.593222, 0.593601, 0.594485, 0.595738, 0.597521,\
        0.600110, 0.603415, 0.606988, 0.610667, 0.614532, 0.618168, 0.621546, 0.624524, 0.626910, 0.628604,\
        0.629516, 0.629505, 0.628495, 0.626523, 0.623659, 0.619865, 0.615051, 0.609460, 0.603740, 0.598366,\
        0.593170, 0.587912, 0.582905, 0.577689, 0.572290, 0.566872, 0.561491, 0.556161, 0.550894, 0.545699,\
        0.540585, 0.535556, 0.530605, 0.525723, 0.520894, 0.516097, 0.511310, 0.506514, 0.501743, 0.497108,\
        0.492759, 0.488244, 0.483394, 0.478193, 0.472668, 0.466856, 0.460791, 0.454497, 0.447990, 0.441280,\
        0.434371],\
        [0.379287, 0.402705, 0.425441, 0.448163, 0.466507, 0.476207, 0.476395, 0.470798, 0.462671, 0.453756,\
        0.444984, 0.436810, 0.429452, 0.423112, 0.417907, 0.413600, 0.409797, 0.406400, 0.404023, 0.402247,\
        0.399846, 0.397866, 0.396441, 0.395413, 0.394610, 0.394442, 0.395855, 0.398718, 0.401970, 0.404697,\
        0.406053, 0.405615, 0.404751, 0.403705, 0.401733, 0.399264, 0.394181, 0.387977, 0.381484, 0.375145,\
        0.369417, 0.364718, 0.361413, 0.359822, 0.360235, 0.362934, 0.368204, 0.376342, 0.387626, 0.402054,\
        0.419267, 0.439489, 0.461857, 0.484897, 0.506836, 0.526453, 0.543822, 0.559118, 0.571950, 0.581806,\
        0.588449, 0.592195, 0.593887, 0.594633, 0.595507, 0.596850, 0.598529, 0.600642, 0.603113, 0.605802,\
        0.609267, 0.613188, 0.617407, 0.621619, 0.625768, 0.629670, 0.633207, 0.636247, 0.638632, 0.640230,\
        0.640929, 0.640638, 0.639314, 0.636974, 0.633656, 0.629383, 0.624223, 0.618418, 0.612378, 0.606447,\
        0.600393, 0.594767, 0.589810, 0.584047, 0.578185, 0.572445, 0.566793, 0.561211, 0.555706, 0.550289,\
        0.544968, 0.539745, 0.534611, 0.529551, 0.524545, 0.519571, 0.514607, 0.509640, 0.504686, 0.500014,\
        0.495695, 0.490974, 0.485882, 0.480407, 0.474609, 0.468549, 0.462266, 0.455782, 0.449112, 0.442261,\
        0.435234],\
        [0.383820, 0.409588, 0.436606, 0.462119, 0.482964, 0.493458, 0.490844, 0.483642, 0.474816, 0.465500,\
        0.456433, 0.447997, 0.440398, 0.433759, 0.428092, 0.423249, 0.419037, 0.415429, 0.412535, 0.410041,\
        0.407535, 0.405350, 0.403661, 0.402372, 0.401191, 0.400433, 0.401657, 0.404502, 0.407587, 0.410134,\
        0.411578, 0.411813, 0.411254, 0.410019, 0.407895, 0.404473, 0.398969, 0.392438, 0.385395, 0.378372,\
        0.371966, 0.366642, 0.362775, 0.360682, 0.360635, 0.362886, 0.367693, 0.375360, 0.386274, 0.400751,\
        0.419205, 0.440828, 0.464152, 0.487518, 0.509328, 0.528837, 0.545987, 0.560845, 0.573206, 0.582749,\
        0.589381, 0.593442, 0.595801, 0.597304, 0.599510, 0.602345, 0.605419, 0.608754, 0.612338, 0.616151,\
        0.620323, 0.624764, 0.629310, 0.633776, 0.638045, 0.642017, 0.645589, 0.648619, 0.650935, 0.652380,\
        0.652837, 0.652236, 0.650560, 0.647837, 0.644117, 0.639469, 0.634009, 0.627934, 0.621504, 0.614907,\
        0.608257, 0.602583, 0.597007, 0.590257, 0.584071, 0.578088, 0.572171, 0.566326, 0.560571, 0.554923,\
        0.549394, 0.543984, 0.538681, 0.533461, 0.528302, 0.523184, 0.518097, 0.513057, 0.508136, 0.503469,\
        0.498753, 0.493799, 0.488376, 0.482543, 0.476428, 0.470099, 0.463588, 0.456911, 0.450076, 0.443087,\
        0.435943],\
        [0.387325, 0.415308, 0.444549, 0.472024, 0.493210, 0.502936, 0.499232, 0.492735, 0.484161, 0.475130,\
        0.466310, 0.458021, 0.450469, 0.443755, 0.437875, 0.432741, 0.428252, 0.424365, 0.421041, 0.418119,\
        0.415466, 0.413163, 0.411297, 0.409821, 0.408677, 0.408307, 0.409432, 0.411796, 0.414547, 0.416922,\
        0.418446, 0.418941, 0.418444, 0.416926, 0.414196, 0.409942, 0.404075, 0.397152, 0.389406, 0.381653,\
        0.374593, 0.368679, 0.364294, 0.361746, 0.361282, 0.363109, 0.367428, 0.374485, 0.384712, 0.399081,\
        0.418572, 0.441535, 0.465816, 0.489639, 0.511468, 0.530772, 0.547857, 0.562824, 0.575451, 0.585313,\
        0.592440, 0.597061, 0.600382, 0.603086, 0.606337, 0.610245, 0.614397, 0.618773, 0.623342, 0.628040,\
        0.632823, 0.637625, 0.642345, 0.646850, 0.651052, 0.654926, 0.658424, 0.661383, 0.663582, 0.664836,\
        0.665019, 0.664078, 0.662033, 0.658936, 0.654859, 0.649897, 0.644164, 0.637810, 0.631014, 0.623967,\
        0.616944, 0.610228, 0.603139, 0.596261, 0.590101, 0.583846, 0.577616, 0.571481, 0.565454, 0.559564,\
        0.553829, 0.548246, 0.542791, 0.537430, 0.532135, 0.526885, 0.521681, 0.516551, 0.511545, 0.506666,\
        0.501784, 0.496580, 0.490677, 0.484445, 0.478019, 0.471435, 0.464710, 0.457851, 0.450860, 0.443739,\
        0.436483],\
        [0.387143, 0.417186, 0.449600, 0.478748, 0.499162, 0.507389, 0.505344, 0.499035, 0.491207, 0.482975,\
        0.474745, 0.466869, 0.459594, 0.453025, 0.447154, 0.441920, 0.437265, 0.433152, 0.429542, 0.426368,\
        0.423588, 0.421229, 0.419337, 0.417934, 0.417105, 0.417128, 0.418260, 0.420322, 0.422759, 0.424963,\
        0.426475, 0.427017, 0.426434, 0.424577, 0.421262, 0.416337, 0.409879, 0.402184, 0.393564, 0.385186,\
        0.377504, 0.371030, 0.366194, 0.363289, 0.362533, 0.364108, 0.368166, 0.374886, 0.384702, 0.398834,\
        0.418982, 0.442765, 0.467687, 0.491620, 0.513278, 0.532438, 0.549606, 0.565150, 0.578840, 0.589695,\
        0.597470, 0.603611, 0.608158, 0.612150, 0.616141, 0.620537, 0.625325, 0.630454, 0.635808, 0.641182,\
        0.646387, 0.651368, 0.656119, 0.660505, 0.664437, 0.668039, 0.671380, 0.674242, 0.676284, 0.677311,\
        0.677192, 0.675876, 0.673460, 0.670010, 0.665621, 0.660405, 0.654450, 0.647854, 0.640740, 0.633273,\
        0.625658, 0.618009, 0.610337, 0.603196, 0.596502, 0.589681, 0.583087, 0.576633, 0.570307, 0.564164,\
        0.558235, 0.552503, 0.546921, 0.541436, 0.536008, 0.530613, 0.525250, 0.519939, 0.514705, 0.509536,\
        0.504340, 0.498701, 0.492453, 0.485960, 0.479300, 0.472508, 0.465599, 0.458578, 0.451447, 0.444204,\
        0.436846],\
        [0.387284, 0.418840, 0.453844, 0.484630, 0.504940, 0.512631, 0.511330, 0.505132, 0.497544, 0.489797,\
        0.482107, 0.474686, 0.467829, 0.461587, 0.455901, 0.450712, 0.445993, 0.441744, 0.437967, 0.434655,\
        0.431814, 0.429474, 0.427682, 0.426495, 0.426012, 0.426372, 0.427631, 0.429610, 0.431899, 0.434009,\
        0.435499, 0.436027, 0.435330, 0.433181, 0.429376, 0.423798, 0.416511, 0.407790, 0.398464, 0.389369,\
        0.380941, 0.373903, 0.368706, 0.365571, 0.364678, 0.366240, 0.370424, 0.377335, 0.387262, 0.401482,\
        0.421352, 0.444978, 0.470112, 0.494336, 0.515310, 0.533799, 0.551129, 0.567629, 0.583052, 0.596615,\
        0.605063, 0.612798, 0.618903, 0.623759, 0.628157, 0.632741, 0.637822, 0.643436, 0.649400, 0.655296,\
        0.660676, 0.665547, 0.670210, 0.674406, 0.677827, 0.680945, 0.684080, 0.686876, 0.688677, 0.689438,\
        0.689005, 0.687238, 0.684503, 0.680731, 0.676090, 0.670716, 0.664624, 0.657859, 0.650470, 0.642582,\
        0.634397, 0.626116, 0.617963, 0.610196, 0.602683, 0.595431, 0.588533, 0.581725, 0.575052, 0.568657,\
        0.562570, 0.556735, 0.551062, 0.545472, 0.539907, 0.534335, 0.528750, 0.523159, 0.517569, 0.511947,\
        0.506169, 0.500047, 0.493707, 0.487080, 0.480258, 0.473305, 0.466244, 0.459083, 0.451827, 0.444476,\
        0.437025],\
        [0.391918, 0.423795, 0.458502, 0.489523, 0.509536, 0.517263, 0.516705, 0.511261, 0.503702, 0.496242,\
        0.488637, 0.481595, 0.475276, 0.469516, 0.464145, 0.459095, 0.454386, 0.450076, 0.446220, 0.442857,\
        0.440025, 0.437770, 0.436147, 0.435225, 0.435077, 0.435760, 0.437237, 0.439312, 0.441638, 0.443784,\
        0.445324, 0.445887, 0.445154, 0.442839, 0.438692, 0.432564, 0.424520, 0.414964, 0.404694, 0.394430,\
        0.385045, 0.377468, 0.372077, 0.368833, 0.367880, 0.369584, 0.374239, 0.381889, 0.392448, 0.406518,\
        0.425249, 0.447577, 0.471986, 0.496344, 0.516149, 0.534289, 0.552359, 0.570318, 0.587517, 0.603216,\
        0.615756, 0.625251, 0.632398, 0.637626, 0.641938, 0.646358, 0.651442, 0.657285, 0.663714, 0.670112,\
        0.675387, 0.679569, 0.684027, 0.688263, 0.690855, 0.693293, 0.696158, 0.698945, 0.700291, 0.700719,\
        0.699995, 0.697638, 0.694854, 0.690633, 0.685951, 0.680557, 0.674442, 0.667632, 0.660016, 0.651709,\
        0.642979, 0.634114, 0.625371, 0.616927, 0.608832, 0.601174, 0.593938, 0.586626, 0.579550, 0.572959,\
        0.566806, 0.560944, 0.555230, 0.549554, 0.543843, 0.538058, 0.532189, 0.526239, 0.520210, 0.514087,\
        0.507826, 0.501401, 0.494758, 0.487915, 0.480940, 0.473849, 0.466656, 0.459371, 0.452004, 0.444554,\
        0.437020],\
        [0.395908, 0.428184, 0.462238, 0.492188, 0.511842, 0.520325, 0.520815, 0.516351, 0.509431, 0.501678,\
        0.494164, 0.487661, 0.482080, 0.476936, 0.471942, 0.467058, 0.462388, 0.458068, 0.454200, 0.450857,\
        0.448094, 0.445968, 0.444545, 0.443892, 0.444066, 0.445077, 0.446844, 0.449158, 0.451685, 0.454016,\
        0.455727, 0.456433, 0.455805, 0.453526, 0.449266, 0.442777, 0.434063, 0.423512, 0.411915, 0.400319,\
        0.389790, 0.381897, 0.376668, 0.373363, 0.372196, 0.373948, 0.379170, 0.387793, 0.399207, 0.412816,\
        0.429779, 0.449936, 0.472081, 0.493899, 0.514306, 0.534041, 0.553849, 0.574236, 0.594324, 0.612936,\
        0.628850, 0.640629, 0.648598, 0.653575, 0.657156, 0.661036, 0.665886, 0.671628, 0.678171, 0.685301,\
        0.690302, 0.692950, 0.696345, 0.701493, 0.703311, 0.705288, 0.707672, 0.709804, 0.710857, 0.710804,\
        0.709554, 0.707059, 0.703829, 0.699106, 0.695178, 0.689583, 0.683684, 0.677038, 0.669205, 0.660431,\
        0.651141, 0.641691, 0.632348, 0.623299, 0.614681, 0.606631, 0.598935, 0.590960, 0.583605, 0.577024,\
        0.570969, 0.565183, 0.559478, 0.553728, 0.547856, 0.541822, 0.535616, 0.529249, 0.522733, 0.516085,\
        0.509328, 0.502494, 0.495552, 0.488520, 0.481405, 0.474183, 0.466861, 0.459457, 0.451983, 0.444444,\
        0.436835],\
        [0.397715, 0.430936, 0.464911, 0.494082, 0.513037, 0.521913, 0.523450, 0.520069, 0.513819, 0.506291,\
        0.498925, 0.493110, 0.488484, 0.483982, 0.479324, 0.474551, 0.469910, 0.465619, 0.461808, 0.458551,\
        0.455908, 0.453942, 0.452723, 0.452325, 0.452798, 0.454139, 0.456250, 0.458910, 0.461777, 0.464433,\
        0.466441, 0.467406, 0.467041, 0.465062, 0.461011, 0.454447, 0.445227, 0.433641, 0.420488, 0.407033,\
        0.395162, 0.387613, 0.383188, 0.379558, 0.377657, 0.379053, 0.384710, 0.394115, 0.405871, 0.419272,\
        0.434883, 0.452813, 0.472701, 0.493753, 0.514351, 0.535137, 0.556917, 0.580117, 0.604666, 0.627301,\
        0.645840, 0.659149, 0.667497, 0.671459, 0.673492, 0.676644, 0.681236, 0.686581, 0.692219, 0.699476,\
        0.704568, 0.706460, 0.708611, 0.712730, 0.715272, 0.717094, 0.718775, 0.720071, 0.720540, 0.719976,\
        0.718306, 0.715557, 0.711845, 0.707589, 0.703051, 0.697296, 0.692389, 0.686011, 0.677791, 0.668460,\
        0.658616, 0.648631, 0.638745, 0.629123, 0.619877, 0.611057, 0.602525, 0.594342, 0.587203, 0.580975,\
        0.575196, 0.569560, 0.563889, 0.558062, 0.552003, 0.545679, 0.539091, 0.532263, 0.525229, 0.518027,\
        0.510706, 0.503371, 0.496135, 0.488980, 0.481737, 0.474358, 0.466886, 0.459354, 0.451776, 0.444151,\
        0.436475],\
        [0.404374, 0.435475, 0.467308, 0.494651, 0.512865, 0.522171, 0.524768, 0.522565, 0.517310, 0.510518,\
        0.503598, 0.498689, 0.494683, 0.490656, 0.486198, 0.481413, 0.476801, 0.472616, 0.468955, 0.465865,\
        0.463394, 0.461606, 0.460582, 0.460405, 0.461140, 0.462795, 0.465274, 0.468352, 0.471659, 0.474745,\
        0.477164, 0.478451, 0.478437, 0.477078, 0.473660, 0.467414, 0.458040, 0.445680, 0.431198, 0.416026,\
        0.402373, 0.396038, 0.392557, 0.387700, 0.384376, 0.384791, 0.390795, 0.401241, 0.413489, 0.427111,\
        0.441943, 0.457990, 0.475656, 0.496049, 0.517444, 0.539630, 0.563496, 0.590220, 0.618878, 0.645983,\
        0.666704, 0.680386, 0.688580, 0.690990, 0.690790, 0.693572, 0.698028, 0.702887, 0.707669, 0.713054,\
        0.717844, 0.720364, 0.722447, 0.724965, 0.727028, 0.728446, 0.729426, 0.729912, 0.729686, 0.728574,\
        0.726499, 0.723482, 0.719651, 0.715238, 0.710285, 0.705292, 0.700721, 0.694139, 0.685270, 0.675419,\
        0.665157, 0.654787, 0.644502, 0.634422, 0.624606, 0.615055, 0.605802, 0.597454, 0.590877, 0.585210,\
        0.579730, 0.574220, 0.568557, 0.562624, 0.556339, 0.549678, 0.542660, 0.535336, 0.527768, 0.520026,\
        0.512172, 0.504245, 0.496727, 0.489526, 0.482021, 0.474399, 0.466743, 0.459072, 0.451389, 0.443685,\
        0.435948],\
        [0.412373, 0.441158, 0.469720, 0.494178, 0.511599, 0.521492, 0.525116, 0.524121, 0.520087, 0.514474,\
        0.508728, 0.504079, 0.500121, 0.496689, 0.492165, 0.487308, 0.482876, 0.478969, 0.475598, 0.472773,\
        0.470530, 0.468936, 0.468084, 0.468079, 0.469017, 0.470938, 0.473767, 0.477290, 0.481097, 0.484610,\
        0.487536, 0.489166, 0.489340, 0.489003, 0.486674, 0.481140, 0.472201, 0.459542, 0.444487, 0.428979,\
        0.415696, 0.408487, 0.403458, 0.397709, 0.392931, 0.391875, 0.397739, 0.410040, 0.423857, 0.438281,\
        0.453147, 0.468216, 0.483648, 0.502171, 0.525644, 0.550073, 0.575501, 0.605703, 0.639947, 0.669792,\
        0.690244, 0.703211, 0.709574, 0.711388, 0.710869, 0.713114, 0.716809, 0.720845, 0.724819, 0.728780,\
        0.732226, 0.734568, 0.736258, 0.737721, 0.738834, 0.739474, 0.739658, 0.739346, 0.738418, 0.736744,\
        0.734251, 0.730946, 0.726925, 0.722341, 0.717388, 0.712358, 0.707048, 0.700088, 0.690971, 0.680999,\
        0.670611, 0.660103, 0.649652, 0.639363, 0.629289, 0.619476, 0.610080, 0.601702, 0.595646, 0.590208,\
        0.584768, 0.579255, 0.573547, 0.567464, 0.560899, 0.553841, 0.546341, 0.538482, 0.530363, 0.522106,\
        0.513867, 0.505896, 0.498307, 0.490216, 0.482189, 0.474266, 0.466415, 0.458610, 0.450829, 0.443053,\
        0.435262],\
        [0.419721, 0.446505, 0.472376, 0.494487, 0.510736, 0.520568, 0.524855, 0.524907, 0.522070, 0.517637,\
        0.512776, 0.508265, 0.504303, 0.500761, 0.496204, 0.491927, 0.488068, 0.484689, 0.481775, 0.479316,\
        0.477354, 0.475971, 0.475266, 0.475370, 0.476429, 0.478543, 0.481653, 0.485551, 0.489923, 0.493697,\
        0.496801, 0.499122, 0.499130, 0.500145, 0.498990, 0.494374, 0.486815, 0.474094, 0.459253, 0.444387,\
        0.431692, 0.422619, 0.415802, 0.409773, 0.404583, 0.403293, 0.411118, 0.423822, 0.438784, 0.454189,\
        0.470065, 0.486551, 0.502139, 0.521086, 0.543658, 0.569106, 0.597383, 0.629097, 0.665134, 0.696460,\
        0.714397, 0.725175, 0.730786, 0.732812, 0.733364, 0.734771, 0.737190, 0.740030, 0.742835, 0.745398,\
        0.747528, 0.749036, 0.749964, 0.750435, 0.750496, 0.750169, 0.749471, 0.748365, 0.746756, 0.744527,\
        0.741598, 0.737948, 0.733631, 0.728757, 0.723459, 0.717798, 0.711482, 0.703870, 0.695026, 0.685249,\
        0.674997, 0.664613, 0.654265, 0.644066, 0.634110, 0.624527, 0.615591, 0.607864, 0.601630, 0.595927,\
        0.590220, 0.584626, 0.578870, 0.572595, 0.565678, 0.558153, 0.550110, 0.541667, 0.532955, 0.524130,\
        0.515377, 0.506892, 0.498605, 0.490273, 0.482003, 0.473885, 0.465885, 0.457968, 0.450104, 0.442265,\
        0.434429],\
        [0.425817, 0.450890, 0.474729, 0.495430, 0.510204, 0.519470, 0.524095, 0.524958, 0.523137, 0.519697,\
        0.515543, 0.511265, 0.507046, 0.502773, 0.499093, 0.495653, 0.492611, 0.489936, 0.487614, 0.485592,\
        0.483957, 0.482816, 0.482244, 0.482397, 0.483481, 0.485702, 0.489056, 0.493037, 0.497875, 0.502114,\
        0.504781, 0.507659, 0.508924, 0.509863, 0.509063, 0.505864, 0.499062, 0.487212, 0.473565, 0.459641,\
        0.447328, 0.437578, 0.430121, 0.424466, 0.421071, 0.422500, 0.430659, 0.443342, 0.458490, 0.474928,\
        0.492146, 0.512173, 0.531193, 0.550125, 0.570583, 0.594077, 0.625081, 0.657722, 0.690664, 0.718066,\
        0.735486, 0.745561, 0.751206, 0.754054, 0.755420, 0.756462, 0.757829, 0.759509, 0.761036, 0.762144,\
        0.762944, 0.763429, 0.763414, 0.762817, 0.761759, 0.760394, 0.758794, 0.756928, 0.754687, 0.751934,\
        0.748558, 0.744504, 0.739779, 0.734438, 0.728543, 0.722085, 0.714924, 0.706920, 0.698048, 0.688389,\
        0.678462, 0.668420, 0.658397, 0.648530, 0.638959, 0.629854, 0.621466, 0.614071, 0.607652, 0.601675,\
        0.595719, 0.590254, 0.584525, 0.577981, 0.570614, 0.562544, 0.553902, 0.544828, 0.535474, 0.526000,\
        0.516561, 0.507264, 0.498280, 0.489826, 0.481493, 0.473271, 0.465168, 0.457163, 0.449229, 0.441337,\
        0.433462],\
        [0.430421, 0.453949, 0.475892, 0.494497, 0.508611, 0.517868, 0.522769, 0.524254, 0.523264, 0.520679,\
        0.517224, 0.513391, 0.509443, 0.505587, 0.502067, 0.499303, 0.496878, 0.494949, 0.493298, 0.491714,\
        0.490435, 0.489625, 0.489207, 0.489388, 0.490436, 0.492605, 0.496360, 0.500585, 0.504991, 0.509684,\
        0.513215, 0.516035, 0.517857, 0.518584, 0.517641, 0.514335, 0.507738, 0.498214, 0.486050, 0.473215,\
        0.461685, 0.452308, 0.445382, 0.441054, 0.440035, 0.443748, 0.452817, 0.465575, 0.479786, 0.498876,\
        0.518691, 0.542188, 0.564250, 0.583227, 0.602651, 0.625880, 0.655392, 0.686418, 0.714261, 0.737500,\
        0.753681, 0.763830, 0.769944, 0.773911, 0.776103, 0.776886, 0.777415, 0.778290, 0.778692, 0.778224,\
        0.777715, 0.777288, 0.776353, 0.774599, 0.772366, 0.769963, 0.767518, 0.764984, 0.762201, 0.758983,\
        0.755175, 0.750679, 0.745462, 0.739537, 0.732934, 0.725663, 0.717705, 0.709074, 0.699942, 0.690670,\
        0.681253, 0.671665, 0.662082, 0.652682, 0.643618, 0.635048, 0.627134, 0.619979, 0.613480, 0.607319,\
        0.601454, 0.596282, 0.590469, 0.583467, 0.575551, 0.566886, 0.557615, 0.547889, 0.537869, 0.527729,\
        0.517642, 0.507784, 0.498361, 0.489454, 0.480881, 0.472519, 0.464312, 0.456224, 0.448225, 0.440284,\
        0.432374],\
        [0.433484, 0.455673, 0.476169, 0.493675, 0.507017, 0.515847, 0.520830, 0.522776, 0.522484, 0.520695,\
        0.518012, 0.514867, 0.511558, 0.508332, 0.505441, 0.503092, 0.501060, 0.499954, 0.498939, 0.497742,\
        0.496852, 0.496568, 0.496373, 0.496604, 0.497912, 0.499591, 0.503860, 0.508519, 0.513153, 0.517681,\
        0.521483, 0.524347, 0.526089, 0.526439, 0.524982, 0.521255, 0.514989, 0.506366, 0.495443, 0.484311,\
        0.474043, 0.465732, 0.460075, 0.457369, 0.458170, 0.463396, 0.473517, 0.487439, 0.500292, 0.518228,\
        0.543617, 0.570903, 0.595496, 0.612891, 0.631949, 0.656449, 0.685440, 0.713360, 0.737287, 0.756686,\
        0.770983, 0.780807, 0.786540, 0.791789, 0.794881, 0.795237, 0.794981, 0.795454, 0.794995, 0.792834,\
        0.791168, 0.790182, 0.788478, 0.785426, 0.782043, 0.778695, 0.775550, 0.772512, 0.769324, 0.765728,\
        0.761523, 0.756577, 0.750824, 0.744261, 0.736923, 0.728867, 0.720177, 0.710987, 0.701534, 0.692728,\
        0.683697, 0.674460, 0.665321, 0.656430, 0.647906, 0.639862, 0.632390, 0.625516, 0.619162, 0.613203,\
        0.607656, 0.602464, 0.596178, 0.588697, 0.580241, 0.571007, 0.561127, 0.550757, 0.540083, 0.529302,\
        0.518614, 0.508250, 0.498421, 0.489083, 0.480233, 0.471693, 0.463360, 0.455180, 0.447113, 0.439123,\
        0.431180],\
        [0.435031, 0.456071, 0.475458, 0.492184, 0.504870, 0.513176, 0.518194, 0.520515, 0.520842, 0.519843,\
        0.518026, 0.515750, 0.513287, 0.510871, 0.508709, 0.506935, 0.505630, 0.504897, 0.504348, 0.503721,\
        0.503395, 0.503600, 0.503850, 0.504471, 0.506135, 0.508487, 0.512402, 0.517092, 0.521798, 0.526189,\
        0.529888, 0.532580, 0.533989, 0.533804, 0.531677, 0.527323, 0.520686, 0.512039, 0.502365, 0.493025,\
        0.483735, 0.476887, 0.472929, 0.471763, 0.473552, 0.478974, 0.488971, 0.502528, 0.517588, 0.536699,\
        0.562424, 0.590091, 0.614166, 0.632872, 0.653990, 0.680813, 0.710956, 0.737395, 0.759184, 0.776307,\
        0.788441, 0.797510, 0.801787, 0.807065, 0.810688, 0.811207, 0.810596, 0.810125, 0.808658, 0.805815,\
        0.803230, 0.801597, 0.798888, 0.794896, 0.790600, 0.786474, 0.782878, 0.779572, 0.776135, 0.772258,\
        0.767707, 0.762320, 0.756018, 0.748799, 0.740726, 0.731922, 0.722579, 0.713040, 0.703978, 0.695110,\
        0.685764, 0.676775, 0.668077, 0.659695, 0.651699, 0.644156, 0.637111, 0.630559, 0.624442, 0.618672,\
        0.613143, 0.607500, 0.601058, 0.593264, 0.584431, 0.574748, 0.564314, 0.553331, 0.542036, 0.530656,\
        0.519400, 0.508456, 0.498112, 0.488594, 0.479542, 0.470813, 0.462333, 0.454047, 0.445906, 0.437866,\
        0.429889],\
        [0.435115, 0.455116, 0.473449, 0.489139, 0.501168, 0.509619, 0.514880, 0.517501, 0.518383, 0.518205,\
        0.517352, 0.516074, 0.514581, 0.513074, 0.511732, 0.510689, 0.510032, 0.509755, 0.509696, 0.509757,\
        0.510077, 0.510740, 0.511649, 0.513002, 0.515095, 0.518013, 0.521878, 0.526353, 0.530892, 0.535068,\
        0.538518, 0.540902, 0.541891, 0.541137, 0.538305, 0.533174, 0.525803, 0.516749, 0.507242, 0.497948,\
        0.489951, 0.485220, 0.483331, 0.483548, 0.485551, 0.489705, 0.497162, 0.509989, 0.526614, 0.547835,\
        0.573737, 0.600515, 0.623221, 0.644835, 0.670845, 0.702543, 0.734595, 0.761204, 0.781675, 0.797629,\
        0.808398, 0.815368, 0.819669, 0.822769, 0.824997, 0.825285, 0.824343, 0.822821, 0.820514, 0.817431,\
        0.814277, 0.811298, 0.807582, 0.803028, 0.798159, 0.793330, 0.789700, 0.786355, 0.782773, 0.778686,\
        0.773832, 0.768022, 0.761164, 0.753278, 0.744472, 0.734930, 0.724904, 0.714728, 0.704800, 0.695635,\
        0.687017, 0.678548, 0.670325, 0.662428, 0.654919, 0.647837, 0.641193, 0.634954, 0.629048, 0.623354,\
        0.617679, 0.611676, 0.604865, 0.596928, 0.588037, 0.578025, 0.567065, 0.555494, 0.543621, 0.531714,\
        0.520010, 0.508700, 0.498026, 0.488176, 0.478827, 0.469866, 0.461223, 0.452825, 0.444608, 0.436518,\
        0.428508],\
        [0.433840, 0.452929, 0.470415, 0.485439, 0.497195, 0.505925, 0.511169, 0.513762, 0.515141, 0.515881,\
        0.516106, 0.515912, 0.515451, 0.514902, 0.514431, 0.514166, 0.514182, 0.514484, 0.515016, 0.515751,\
        0.516737, 0.518023, 0.519639, 0.521703, 0.524365, 0.527696, 0.531652, 0.535985, 0.540305, 0.544222,\
        0.547383, 0.549447, 0.550063, 0.548848, 0.545427, 0.539539, 0.531216, 0.520953, 0.509771, 0.499488,\
        0.493262, 0.491113, 0.491334, 0.492636, 0.494396, 0.496662, 0.501675, 0.512401, 0.529039, 0.551349,\
        0.578223, 0.605576, 0.629945, 0.655820, 0.690333, 0.731245, 0.763725, 0.788730, 0.806936, 0.821454,\
        0.831059, 0.836101, 0.838232, 0.838989, 0.839012, 0.838126, 0.836417, 0.834075, 0.831121, 0.827647,\
        0.823912, 0.819961, 0.815504, 0.810436, 0.804894, 0.800080, 0.796542, 0.793140, 0.789357, 0.785112,\
        0.779982, 0.773754, 0.766328, 0.757751, 0.748199, 0.737926, 0.727231, 0.716427, 0.705877, 0.696528,\
        0.688128, 0.679970, 0.672101, 0.664601, 0.657512, 0.650838, 0.644553, 0.638594, 0.632857, 0.627190,\
        0.621372, 0.615113, 0.608100, 0.600158, 0.591254, 0.580775, 0.569243, 0.557111, 0.544706, 0.532329,\
        0.520277, 0.508844, 0.498066, 0.487709, 0.477980, 0.468787, 0.459995, 0.451498, 0.443214, 0.435080,\
        0.427041],\
        [0.431328, 0.449639, 0.466561, 0.481492, 0.493957, 0.502816, 0.506948, 0.509132, 0.511190, 0.513077,\
        0.514478, 0.515388, 0.515956, 0.516363, 0.516772, 0.517305, 0.518039, 0.519002, 0.520190, 0.521607,\
        0.523283, 0.525259, 0.527584, 0.530328, 0.533559, 0.537281, 0.541399, 0.545689, 0.549840, 0.553523,\
        0.556427, 0.558248, 0.558650, 0.557225, 0.553512, 0.547129, 0.537987, 0.526498, 0.513758, 0.502071,\
        0.496049, 0.495859, 0.497154, 0.498582, 0.499547, 0.499984, 0.502017, 0.510371, 0.525932, 0.548880,\
        0.578118, 0.609790, 0.640311, 0.673977, 0.723445, 0.774850, 0.798426, 0.819119, 0.834990, 0.847390,\
        0.854987, 0.857162, 0.856143, 0.854172, 0.852061, 0.849765, 0.847117, 0.844044, 0.840528, 0.836614,\
        0.832394, 0.827877, 0.822983, 0.817719, 0.812367, 0.807634, 0.803730, 0.799889, 0.795935, 0.791595,\
        0.786164, 0.779532, 0.771500, 0.762179, 0.751843, 0.740841, 0.729554, 0.718403, 0.707879, 0.698289,\
        0.689372, 0.681106, 0.673390, 0.666176, 0.659429, 0.653104, 0.647135, 0.641424, 0.635837, 0.630197,\
        0.624283, 0.617858, 0.610762, 0.603118, 0.593808, 0.582664, 0.570654, 0.558046, 0.545169, 0.532345,\
        0.519879, 0.508061, 0.497082, 0.486635, 0.476791, 0.467490, 0.458610, 0.450047, 0.441717, 0.433549,\
        0.425489],\
        [0.427693, 0.445280, 0.461679, 0.476379, 0.488744, 0.497525, 0.501279, 0.503547, 0.506984, 0.510212,\
        0.512731, 0.514645, 0.516163, 0.517478, 0.518746, 0.520080, 0.521559, 0.523225, 0.525102, 0.527212,\
        0.529585, 0.532258, 0.535271, 0.538660, 0.542439, 0.546563, 0.550903, 0.555243, 0.559305, 0.562814,\
        0.565526, 0.567217, 0.567622, 0.566347, 0.562815, 0.556430, 0.546944, 0.534770, 0.521190, 0.508592,\
        0.500853, 0.500545, 0.500282, 0.499873, 0.498540, 0.496885, 0.497360, 0.503687, 0.518083, 0.541772,\
        0.575010, 0.615591, 0.659098, 0.705640, 0.769014, 0.817847, 0.837338, 0.849219, 0.861398, 0.872428,\
        0.878240, 0.876295, 0.871612, 0.867198, 0.863430, 0.859983, 0.856504, 0.852793, 0.848778, 0.844468,\
        0.839896, 0.835083, 0.830037, 0.824845, 0.819749, 0.815066, 0.810834, 0.806701, 0.802691, 0.797906,\
        0.792295, 0.785299, 0.776567, 0.766412, 0.755230, 0.743458, 0.731553, 0.719981, 0.709167, 0.699270,\
        0.690072, 0.681700, 0.674075, 0.667079, 0.660615, 0.654590, 0.648901, 0.643417, 0.637979, 0.632391,\
        0.626426, 0.619841, 0.612383, 0.603692, 0.594270, 0.583438, 0.571208, 0.558231, 0.544952, 0.531731,\
        0.518877, 0.506607, 0.495298, 0.485026, 0.475260, 0.465961, 0.457058, 0.448468, 0.440113, 0.431926,\
        0.423852],\
        [0.423091, 0.439948, 0.455742, 0.469919, 0.481756, 0.490325, 0.495261, 0.498965, 0.503658, 0.507756,\
        0.511051, 0.513758, 0.516103, 0.518256, 0.520347, 0.522471, 0.524699, 0.527080, 0.529653, 0.532451,\
        0.535506, 0.538852, 0.542518, 0.546516, 0.550823, 0.555363, 0.559985, 0.564463, 0.568523, 0.571918,\
        0.574488, 0.576133, 0.576737, 0.576015, 0.573254, 0.567483, 0.558194, 0.545822, 0.531890, 0.518843,\
        0.509547, 0.504641, 0.501063, 0.497353, 0.493244, 0.489641, 0.488764, 0.493614, 0.506987, 0.531340,\
        0.568907, 0.620939, 0.680831, 0.737261, 0.795670, 0.842613, 0.863911, 0.873540, 0.882983, 0.892627,\
        0.897686, 0.890296, 0.883144, 0.877483, 0.872868, 0.868720, 0.864612, 0.860357, 0.855904, 0.851260,\
        0.846451, 0.841506, 0.836467, 0.831427, 0.826533, 0.821922, 0.817582, 0.813349, 0.808973, 0.804005,\
        0.798428, 0.790813, 0.781254, 0.770195, 0.758108, 0.745502, 0.732883, 0.720722, 0.709400, 0.699118,\
        0.689852, 0.681527, 0.674031, 0.667235, 0.661017, 0.655255, 0.649818, 0.644552, 0.639274, 0.633773,\
        0.627815, 0.621157, 0.613572, 0.604964, 0.595095, 0.583619, 0.570997, 0.557665, 0.544053, 0.530530,\
        0.517435, 0.505195, 0.493845, 0.483317, 0.473534, 0.464253, 0.455358, 0.446766, 0.438404, 0.430210,\
        0.422130],\
        [0.417758, 0.433909, 0.449121, 0.462833, 0.474435, 0.483456, 0.490126, 0.495757, 0.501095, 0.505593,\
        0.509334, 0.512663, 0.515744, 0.518684, 0.521565, 0.524457, 0.527420, 0.530507, 0.533760, 0.537219,\
        0.540922, 0.544900, 0.549171, 0.553733, 0.558544, 0.563509, 0.568465, 0.573175, 0.577326, 0.580649,\
        0.583096, 0.584711, 0.585557, 0.585693, 0.584362, 0.579786, 0.571111, 0.558655, 0.544250, 0.530294,\
        0.518637, 0.509562, 0.501853, 0.494511, 0.487495, 0.481793, 0.479365, 0.482848, 0.495266, 0.520108,\
        0.561466, 0.624751, 0.697300, 0.753281, 0.809063, 0.862221, 0.886588, 0.893113, 0.901754, 0.907168,\
        0.906954, 0.898775, 0.891563, 0.885631, 0.880773, 0.876197, 0.871539, 0.866765, 0.861898, 0.856967,\
        0.851995, 0.847010, 0.842049, 0.837171, 0.832450, 0.827931, 0.823571, 0.819227, 0.814661, 0.809577,\
        0.803453, 0.795390, 0.785180, 0.773213, 0.760212, 0.746741, 0.733346, 0.720527, 0.708665, 0.698026,\
        0.688776, 0.680556, 0.673209, 0.666599, 0.660595, 0.655063, 0.649856, 0.644800, 0.639695, 0.634313,\
        0.628412, 0.621753, 0.614121, 0.605350, 0.595032, 0.583046, 0.569995, 0.556328, 0.542470, 0.528772,\
        0.515502, 0.502918, 0.491723, 0.481402, 0.471666, 0.462410, 0.453532, 0.444949, 0.436592, 0.428400,\
        0.420320],\
        [0.412007, 0.427538, 0.442307, 0.455758, 0.467427, 0.477111, 0.485070, 0.491945, 0.498092, 0.502991,\
        0.507236, 0.511214, 0.515031, 0.518741, 0.522389, 0.526023, 0.529695, 0.533456, 0.537354, 0.541433,\
        0.545732, 0.550283, 0.555100, 0.560172, 0.565448, 0.570824, 0.576138, 0.581166, 0.585542, 0.588807,\
        0.591137, 0.592749, 0.593668, 0.594522, 0.595308, 0.592130, 0.584526, 0.571723, 0.556605, 0.541555,\
        0.527773, 0.515455, 0.504138, 0.493462, 0.483726, 0.476032, 0.472195, 0.474586, 0.486204, 0.511315,\
        0.556920, 0.631555, 0.711836, 0.763154, 0.829139, 0.893509, 0.917574, 0.920763, 0.923992, 0.923589,\
        0.918218, 0.908974, 0.899847, 0.893088, 0.887968, 0.882728, 0.877342, 0.871982, 0.866698, 0.861509,\
        0.856424, 0.851450, 0.846597, 0.841878, 0.837308, 0.832872, 0.828498, 0.824038, 0.819251, 0.813793,\
        0.807217, 0.799040, 0.787894, 0.775160, 0.761337, 0.747026, 0.732849, 0.719393, 0.707173, 0.696402,\
        0.687028, 0.678852, 0.671617, 0.665152, 0.659319, 0.653980, 0.648978, 0.644126, 0.639204, 0.633960,\
        0.628133, 0.621475, 0.613728, 0.604504, 0.593755, 0.581501, 0.568089, 0.554147, 0.540155, 0.526475,\
        0.513378, 0.501059, 0.489764, 0.479405, 0.469681, 0.460444, 0.451587, 0.443022, 0.434678, 0.426495,\
        0.418420],\
        [0.406204, 0.421255, 0.435738, 0.449064, 0.460879, 0.471084, 0.479841, 0.487446, 0.494031, 0.499620,\
        0.504614, 0.509350, 0.513951, 0.518434, 0.522825, 0.527166, 0.531504, 0.535894, 0.540386, 0.545026,\
        0.549858, 0.554910, 0.560199, 0.565713, 0.571400, 0.577145, 0.582771, 0.588100, 0.592924, 0.596143,\
        0.598341, 0.600251, 0.601317, 0.602414, 0.604133, 0.602558, 0.596343, 0.582997, 0.567719, 0.552237,\
        0.537198, 0.522737, 0.508768, 0.495441, 0.483490, 0.474289, 0.469682, 0.471818, 0.483273, 0.507823,\
        0.552135, 0.629112, 0.716137, 0.772143, 0.854666, 0.928153, 0.949800, 0.951979, 0.948331, 0.941577,\
        0.931854, 0.920396, 0.909437, 0.901074, 0.894887, 0.888177, 0.881857, 0.875867, 0.870176, 0.864767,\
        0.859617, 0.854697, 0.849974, 0.845414, 0.840977, 0.836601, 0.832188, 0.827584, 0.822544, 0.816679,\
        0.809379, 0.799932, 0.789118, 0.775971, 0.761391, 0.746298, 0.731388, 0.717281, 0.704677, 0.693870,\
        0.684570, 0.676449, 0.669273, 0.662887, 0.657163, 0.651968, 0.647142, 0.642487, 0.637757, 0.632663,\
        0.626903, 0.620205, 0.612396, 0.603341, 0.592028, 0.579023, 0.565180, 0.551019, 0.536996, 0.523475,\
        0.510740, 0.498903, 0.487656, 0.477255, 0.467541, 0.458335, 0.449511, 0.440976, 0.432655, 0.424488,\
        0.416425],\
        [0.400841, 0.415564, 0.429790, 0.442965, 0.454840, 0.465353, 0.474586, 0.482682, 0.489759, 0.495969,\
        0.501639, 0.507184, 0.512577, 0.517807, 0.522899, 0.527894, 0.532843, 0.537801, 0.542824, 0.547959,\
        0.553248, 0.558722, 0.564397, 0.570268, 0.576292, 0.582355, 0.588221, 0.593581, 0.598471, 0.602321,\
        0.604705, 0.607041, 0.608775, 0.610266, 0.611138, 0.609454, 0.602904, 0.591162, 0.577301, 0.562596,\
        0.547424, 0.531983, 0.516391, 0.501191, 0.487652, 0.477644, 0.473303, 0.476706, 0.489854, 0.515552,\
        0.559253, 0.633954, 0.720713, 0.788215, 0.874071, 0.942198, 0.970018, 0.974192, 0.968032, 0.957273,\
        0.944248, 0.930661, 0.918230, 0.908128, 0.899492, 0.891821, 0.884747, 0.878194, 0.872162, 0.866606,\
        0.861461, 0.856651, 0.852095, 0.847706, 0.843393, 0.839052, 0.834563, 0.829778, 0.824495, 0.818399,\
        0.811013, 0.801911, 0.790002, 0.775743, 0.760294, 0.744531, 0.729114, 0.714454, 0.701300, 0.690697,\
        0.681541, 0.673404, 0.666182, 0.659784, 0.654096, 0.648988, 0.644303, 0.639835, 0.635314, 0.630397,\
        0.624702, 0.617864, 0.609610, 0.599949, 0.588523, 0.575221, 0.561127, 0.546864, 0.532910, 0.519623,\
        0.507166, 0.495648, 0.484896, 0.474746, 0.465162, 0.456042, 0.447283, 0.438797, 0.430512, 0.422370,\
        0.414325],\
        [0.396778, 0.410925, 0.424681, 0.437542, 0.449324, 0.459961, 0.469483, 0.477975, 0.485545, 0.492357,\
        0.498710, 0.504938, 0.511024, 0.516925, 0.522645, 0.528222, 0.533710, 0.539168, 0.544652, 0.550210,\
        0.555881, 0.561693, 0.567661, 0.573786, 0.580045, 0.586372, 0.592559, 0.597974, 0.602632, 0.606904,\
        0.610305, 0.613165, 0.615442, 0.616988, 0.617175, 0.614670, 0.608147, 0.597974, 0.586608, 0.573456,\
        0.559096, 0.543790, 0.527516, 0.511087, 0.496458, 0.486264, 0.483246, 0.489715, 0.507107, 0.536796,\
        0.584904, 0.657911, 0.736742, 0.810191, 0.893782, 0.952885, 0.980424, 0.986353, 0.979976, 0.967498,\
        0.952516, 0.937336, 0.923563, 0.911932, 0.902161, 0.893547, 0.885718, 0.878721, 0.872485, 0.866907,\
        0.861872, 0.857255, 0.852924, 0.848743, 0.844566, 0.840245, 0.835633, 0.830592, 0.824993, 0.818678,\
        0.811265, 0.801837, 0.789168, 0.774006, 0.757796, 0.741571, 0.726142, 0.711843, 0.698709, 0.687744,\
        0.678181, 0.669726, 0.662304, 0.655800, 0.650077, 0.644995, 0.640406, 0.636118, 0.631836, 0.627151,\
        0.621588, 0.614651, 0.605965, 0.595473, 0.583512, 0.570121, 0.555955, 0.541714, 0.527930, 0.515025,\
        0.503147, 0.492068, 0.481717, 0.471891, 0.462519, 0.453540, 0.444880, 0.436468, 0.428237, 0.420132,\
        0.412112],\
        [0.393578, 0.407211, 0.420276, 0.432687, 0.444261, 0.454899, 0.464602, 0.473429, 0.481476, 0.488890,\
        0.495882, 0.502679, 0.509358, 0.515831, 0.522086, 0.528157, 0.534104, 0.539989, 0.545866, 0.551780,\
        0.557763, 0.563838, 0.570013, 0.576284, 0.582635, 0.589041, 0.595455, 0.601428, 0.606250, 0.610868,\
        0.615004, 0.618523, 0.621287, 0.622987, 0.623021, 0.620541, 0.614957, 0.606861, 0.597462, 0.585399,\
        0.572696, 0.558635, 0.542466, 0.525151, 0.509658, 0.499588, 0.498492, 0.509290, 0.533140, 0.567844,\
        0.613744, 0.684434, 0.766166, 0.839470, 0.910809, 0.960138, 0.984852, 0.990644, 0.984243, 0.971127,\
        0.955165, 0.939001, 0.924373, 0.912050, 0.901707, 0.892564, 0.884374, 0.877231, 0.871022, 0.865600,\
        0.860813, 0.856498, 0.852481, 0.848572, 0.844573, 0.840284, 0.835516, 0.830108, 0.823972, 0.817129,\
        0.809436, 0.799517, 0.786188, 0.770473, 0.753695, 0.737061, 0.721784, 0.708777, 0.696535, 0.684560,\
        0.674127, 0.665188, 0.657510, 0.650872, 0.645076, 0.639960, 0.635406, 0.631275, 0.627274, 0.622891,\
        0.617565, 0.610717, 0.601662, 0.590587, 0.577894, 0.564130, 0.549873, 0.535753, 0.522257, 0.509659,\
        0.498626, 0.488237, 0.478290, 0.468775, 0.459631, 0.450818, 0.442286, 0.433974, 0.425819, 0.417765,\
        0.409776],\
        [0.390484, 0.403570, 0.416107, 0.428173, 0.439510, 0.450080, 0.459899, 0.469014, 0.477501, 0.485470,\
        0.493055, 0.500402, 0.507576, 0.514528, 0.521217, 0.527689, 0.534014, 0.540257, 0.546467, 0.552682,\
        0.558926, 0.565211, 0.571535, 0.577881, 0.584215, 0.590485, 0.596642, 0.602763, 0.608649, 0.614062,\
        0.618957, 0.623186, 0.626544, 0.628704, 0.629196, 0.627512, 0.623410, 0.617138, 0.608714, 0.598581,\
        0.588454, 0.576657, 0.561154, 0.542920, 0.526738, 0.516958, 0.517780, 0.532974, 0.563474, 0.605440,\
        0.649012, 0.714057, 0.798139, 0.870669, 0.927760, 0.965813, 0.985144, 0.989006, 0.981969, 0.968445,\
        0.952015, 0.935343, 0.920321, 0.907951, 0.897653, 0.888517, 0.880554, 0.873671, 0.867763, 0.862695,\
        0.858310, 0.854426, 0.850831, 0.847290, 0.843549, 0.839353, 0.834457, 0.828640, 0.821739, 0.813789,\
        0.805511, 0.795465, 0.781586, 0.765450, 0.748196, 0.730961, 0.715005, 0.702161, 0.691028, 0.679168,\
        0.668554, 0.659428, 0.651645, 0.644958, 0.639110, 0.633905, 0.629278, 0.625244, 0.621594, 0.617576,\
        0.612389, 0.605922, 0.596649, 0.584759, 0.571471, 0.557383, 0.543073, 0.529198, 0.516383, 0.504601,\
        0.493868, 0.484174, 0.474682, 0.465447, 0.456510, 0.447865, 0.439479, 0.431296, 0.423249, 0.415266,\
        0.407314],\
        [0.386976, 0.399695, 0.412130, 0.423830, 0.434896, 0.445362, 0.455255, 0.464618, 0.473506, 0.481986,\
        0.490128, 0.497976, 0.505635, 0.512981, 0.519998, 0.526780, 0.533414, 0.539960, 0.546459, 0.552938,\
        0.559412, 0.565884, 0.572344, 0.578768, 0.585117, 0.591334, 0.597428, 0.603848, 0.610421, 0.616626,\
        0.622297, 0.627288, 0.631390, 0.634319, 0.635722, 0.635274, 0.632798, 0.628304, 0.621912, 0.614285,\
        0.605829, 0.597165, 0.581890, 0.563157, 0.547278, 0.538521, 0.541487, 0.560786, 0.598119, 0.642590,\
        0.694602, 0.762418, 0.830533, 0.896380, 0.939649, 0.967848, 0.981460, 0.982566, 0.974379, 0.960417,\
        0.943826, 0.927053, 0.911898, 0.899939, 0.890314, 0.881905, 0.874565, 0.868237, 0.862848, 0.858301,\
        0.854457, 0.851130, 0.848080, 0.845026, 0.841659, 0.837670, 0.832765, 0.826680, 0.819175, 0.810052,\
        0.799499, 0.789551, 0.775815, 0.759546, 0.741976, 0.724206, 0.707506, 0.693512, 0.682347, 0.671455,\
        0.661265, 0.652313, 0.644660, 0.638095, 0.632289, 0.626968, 0.622087, 0.617954, 0.614782, 0.611420,\
        0.605857, 0.598815, 0.589669, 0.577550, 0.564059, 0.549942, 0.535754, 0.522130, 0.510268, 0.500355,\
        0.490163, 0.480362, 0.470976, 0.461907, 0.453137, 0.444650, 0.436420, 0.428400, 0.420511, 0.412647,\
        0.404735],\
        [0.383569, 0.396276, 0.408146, 0.419420, 0.430223, 0.440576, 0.450523, 0.460103, 0.469351, 0.478302,\
        0.486987, 0.495422, 0.503505, 0.511085, 0.518334, 0.525362, 0.532262, 0.539081, 0.545844, 0.552568,\
        0.559262, 0.565926, 0.572547, 0.579104, 0.585587, 0.592021, 0.598461, 0.605080, 0.611994, 0.618767,\
        0.625131, 0.630894, 0.635855, 0.639772, 0.642375, 0.643420, 0.642745, 0.640307, 0.636207, 0.630698,\
        0.623888, 0.615021, 0.600673, 0.584520, 0.571494, 0.565054, 0.573677, 0.597439, 0.646088, 0.687895,\
        0.737244, 0.800646, 0.858897, 0.907810, 0.943014, 0.964393, 0.973558, 0.972104, 0.962662, 0.948300,\
        0.931902, 0.915712, 0.901212, 0.889843, 0.881079, 0.873541, 0.866967, 0.861308, 0.856538, 0.852602,\
        0.849396, 0.846736, 0.844356, 0.841922, 0.839064, 0.835419, 0.830666, 0.824552, 0.816903, 0.807689,\
        0.796953, 0.784522, 0.770046, 0.753513, 0.735773, 0.717824, 0.700742, 0.686311, 0.673938, 0.662800,\
        0.652746, 0.644002, 0.636632, 0.630396, 0.624817, 0.619459, 0.614159, 0.609396, 0.606696, 0.604516,\
        0.599545, 0.590785, 0.580827, 0.568905, 0.555688, 0.541957, 0.528313, 0.515175, 0.502934, 0.494999,\
        0.485799, 0.476256, 0.467004, 0.458077, 0.449455, 0.441119, 0.433051, 0.425219, 0.417554, 0.409936,\
        0.402118],\
        [0.379684, 0.391949, 0.403599, 0.414712, 0.425313, 0.435564, 0.445562, 0.455336, 0.464897, 0.474256,\
        0.483410, 0.492303, 0.500744, 0.508639, 0.516093, 0.523348, 0.530515, 0.537602, 0.544621, 0.551586,\
        0.558507, 0.565384, 0.572205, 0.578956, 0.585636, 0.592297, 0.599081, 0.606036, 0.613259, 0.620488,\
        0.627451, 0.633959, 0.639828, 0.644854, 0.648814, 0.651499, 0.652743, 0.652452, 0.650590, 0.647072,\
        0.641521, 0.632982, 0.620874, 0.606892, 0.600670, 0.595034, 0.615083, 0.651597, 0.702034, 0.732974,\
        0.767594, 0.814158, 0.863229, 0.904962, 0.936889, 0.955574, 0.962082, 0.958668, 0.948118, 0.933484,\
        0.917681, 0.902803, 0.890026, 0.879599, 0.871273, 0.864325, 0.858378, 0.853325, 0.849145, 0.845826,\
        0.843298, 0.841387, 0.839795, 0.838117, 0.835903, 0.832720, 0.828221, 0.822183, 0.814485, 0.805050,\
        0.793602, 0.780063, 0.764720, 0.747749, 0.729787, 0.712132, 0.695343, 0.679498, 0.665761, 0.653784,\
        0.643421, 0.634701, 0.627651, 0.621965, 0.616872, 0.611694, 0.606327, 0.600519, 0.597536, 0.595251,\
        0.590119, 0.581556, 0.571139, 0.559344, 0.546585, 0.533537, 0.520911, 0.509371, 0.499547, 0.489726,\
        0.480686, 0.471567, 0.462588, 0.453864, 0.445409, 0.437221, 0.429300, 0.421650, 0.414224, 0.406903,\
        0.399614],\
        [0.375416, 0.387368, 0.398831, 0.409719, 0.420045, 0.430208, 0.440276, 0.450223, 0.460042, 0.469729,\
        0.479250, 0.488520, 0.497374, 0.505568, 0.513131, 0.520675, 0.528160, 0.535524, 0.542792, 0.549996,\
        0.557154, 0.564270, 0.571337, 0.578342, 0.585272, 0.592114, 0.598952, 0.606391, 0.614051, 0.621669,\
        0.629136, 0.636326, 0.643092, 0.649259, 0.654631, 0.659003, 0.662191, 0.664045, 0.664438, 0.663152,\
        0.659691, 0.653510, 0.645051, 0.637109, 0.632968, 0.636341, 0.657287, 0.691166, 0.730221, 0.758227,\
        0.784733, 0.819863, 0.861500, 0.899767, 0.928239, 0.944330, 0.948740, 0.943707, 0.932248, 0.917558,\
        0.902671, 0.889744, 0.878559, 0.869136, 0.861318, 0.854788, 0.849284, 0.844688, 0.840984, 0.838204,\
        0.836336, 0.835227, 0.834533, 0.833749, 0.832305, 0.829663, 0.825415, 0.819365, 0.811471, 0.801519,\
        0.789419, 0.775340, 0.759572, 0.742316, 0.723954, 0.705202, 0.688414, 0.672123, 0.657226, 0.644321,\
        0.633456, 0.624546, 0.617711, 0.612793, 0.608433, 0.603491, 0.598209, 0.592852, 0.588369, 0.584399,\
        0.578690, 0.570566, 0.560548, 0.549196, 0.536998, 0.524656, 0.513040, 0.502586, 0.493315, 0.484208,\
        0.475199, 0.466371, 0.457706, 0.449238, 0.440980, 0.432935, 0.425121, 0.417593, 0.410392, 0.403208,\
        0.395837],\
        [0.370844, 0.382480, 0.393731, 0.404213, 0.414267, 0.424481, 0.434652, 0.444734, 0.454744, 0.464675,\
        0.474473, 0.484032, 0.493186, 0.501439, 0.509308, 0.517421, 0.525249, 0.532858, 0.540353, 0.547790,\
        0.555194, 0.562570, 0.569918, 0.577233, 0.584518, 0.591805, 0.599184, 0.606635, 0.614330, 0.622152,\
        0.629990, 0.637764, 0.645362, 0.652629, 0.659378, 0.665398, 0.670469, 0.674385, 0.677012, 0.678266,\
        0.677768, 0.675026, 0.671183, 0.668451, 0.668712, 0.676996, 0.699484, 0.723005, 0.750590, 0.771708,\
        0.790664, 0.820042, 0.857663, 0.893301, 0.919012, 0.932732, 0.935208, 0.928683, 0.916588, 0.902481,\
        0.889133, 0.877590, 0.867514, 0.858814, 0.851433, 0.845227, 0.840030, 0.835737, 0.832348, 0.829961,\
        0.828676, 0.828397, 0.828704, 0.828950, 0.828404, 0.826364, 0.822291, 0.815962, 0.807593, 0.797233,\
        0.784421, 0.770021, 0.754270, 0.737199, 0.718840, 0.700357, 0.681755, 0.664152, 0.648042, 0.634262,\
        0.623018, 0.613843, 0.606806, 0.602706, 0.599058, 0.594259, 0.588948, 0.583611, 0.578450, 0.573182,\
        0.566849, 0.558922, 0.549494, 0.538856, 0.527318, 0.515482, 0.504574, 0.495116, 0.486274, 0.477705,\
        0.469172, 0.460726, 0.452406, 0.444232, 0.436203, 0.428306, 0.420550, 0.413007, 0.405879, 0.399315,\
        0.391821],\
        [0.366010, 0.377213, 0.387898, 0.397919, 0.408209, 0.418567, 0.428777, 0.438896, 0.449013, 0.459119,\
        0.469129, 0.478889, 0.488146, 0.496630, 0.505462, 0.513899, 0.521838, 0.529594, 0.537287, 0.544951,\
        0.552601, 0.560244, 0.567886, 0.575526, 0.583168, 0.590829, 0.598504, 0.606046, 0.613805, 0.621706,\
        0.629773, 0.638010, 0.646332, 0.654593, 0.662607, 0.670152, 0.676967, 0.682750, 0.687299, 0.690869,\
        0.693646, 0.694274, 0.694398, 0.697098, 0.701506, 0.711597, 0.732156, 0.754101, 0.772279, 0.784970,\
        0.796957, 0.820593, 0.853245, 0.885018, 0.908935, 0.921675, 0.922511, 0.914594, 0.902413, 0.889529,\
        0.877525, 0.866721, 0.857187, 0.848891, 0.841803, 0.835842, 0.830879, 0.826787, 0.823543, 0.821334,\
        0.820490, 0.821046, 0.822449, 0.823843, 0.824321, 0.822977, 0.819039, 0.812148, 0.802517, 0.791280,\
        0.778135, 0.763618, 0.748249, 0.731993, 0.714450, 0.695165, 0.675335, 0.655832, 0.638123, 0.623455,\
        0.612285, 0.603436, 0.596084, 0.591810, 0.588149, 0.583703, 0.578653, 0.573294, 0.567720, 0.561750,\
        0.554994, 0.547212, 0.538433, 0.528771, 0.518209, 0.506828, 0.496336, 0.487500, 0.479139, 0.470935,\
        0.462796, 0.454726, 0.446755, 0.438898, 0.431143, 0.423440, 0.415752, 0.408172, 0.400768, 0.394078,\
        0.387122],\
        [0.361055, 0.371881, 0.382278, 0.392357, 0.402505, 0.412758, 0.422733, 0.432731, 0.442887, 0.453132,\
        0.463336, 0.473361, 0.483101, 0.492655, 0.501825, 0.509906, 0.517777, 0.525664, 0.533565, 0.541458,\
        0.549347, 0.557247, 0.565166, 0.573107, 0.581051, 0.588942, 0.596659, 0.604495, 0.612303, 0.620097,\
        0.628239, 0.636815, 0.645729, 0.654821, 0.663901, 0.672754, 0.681099, 0.688532, 0.694492, 0.699143,\
        0.703898, 0.707308, 0.710954, 0.717336, 0.726357, 0.740351, 0.760200, 0.779982, 0.792059, 0.798505,\
        0.805547, 0.822845, 0.848481, 0.875945, 0.898907, 0.911611, 0.910771, 0.901628, 0.890217, 0.878394,\
        0.867099, 0.856747, 0.847493, 0.839405, 0.832505, 0.826760, 0.822048, 0.818155, 0.814951, 0.812641,\
        0.811998, 0.813404, 0.815938, 0.818489, 0.820086, 0.819592, 0.815866, 0.808356, 0.797600, 0.784696,\
        0.770433, 0.755509, 0.740575, 0.725395, 0.708949, 0.689888, 0.668153, 0.646855, 0.627521, 0.611770,\
        0.600991, 0.592856, 0.586121, 0.581167, 0.577042, 0.572683, 0.567807, 0.562451, 0.556637, 0.550285,\
        0.543281, 0.535640, 0.527507, 0.519006, 0.510068, 0.499924, 0.489631, 0.480575, 0.472195, 0.464130,\
        0.456234, 0.448460, 0.440804, 0.433269, 0.425839, 0.418445, 0.410922, 0.403293, 0.396151, 0.389138,\
        0.382138],\
        [0.356099, 0.366656, 0.376930, 0.387065, 0.397246, 0.407043, 0.416378, 0.426218, 0.436429, 0.446805,\
        0.457182, 0.467436, 0.477492, 0.487276, 0.496431, 0.504750, 0.512847, 0.521029, 0.529196, 0.537316,\
        0.545419, 0.553542, 0.561702, 0.569897, 0.578110, 0.586315, 0.594503, 0.602441, 0.609665, 0.617065,\
        0.625186, 0.634017, 0.643377, 0.653079, 0.662938, 0.672745, 0.682243, 0.691064, 0.698436, 0.703262,\
        0.707862, 0.713053, 0.719495, 0.728415, 0.740555, 0.756746, 0.776508, 0.795470, 0.801041, 0.802430,\
        0.807066, 0.819588, 0.840001, 0.864107, 0.886150, 0.899500, 0.898580, 0.889403, 0.878749, 0.867744,\
        0.857110, 0.847198, 0.838195, 0.830267, 0.823532, 0.818021, 0.813675, 0.810129, 0.807067, 0.804460,\
        0.803646, 0.805963, 0.809391, 0.812803, 0.815482, 0.816026, 0.812672, 0.804305, 0.791898, 0.777013,\
        0.760988, 0.745018, 0.730197, 0.716370, 0.700053, 0.680667, 0.659097, 0.637314, 0.617094, 0.600653,\
        0.589819, 0.582221, 0.575973, 0.570842, 0.566340, 0.561840, 0.556971, 0.551590, 0.545638, 0.539075,\
        0.531903, 0.524314, 0.516696, 0.509045, 0.501563, 0.492794, 0.482979, 0.473782, 0.465286, 0.457250,\
        0.449504, 0.441956, 0.434565, 0.427319, 0.420223, 0.413267, 0.406276, 0.398744, 0.391355, 0.384214,\
        0.377081],\
        [0.351180, 0.361483, 0.371561, 0.381507, 0.391307, 0.400283, 0.409487, 0.419452, 0.429783, 0.440273,\
        0.450757, 0.461114, 0.471238, 0.480984, 0.490141, 0.498676, 0.507232, 0.515838, 0.524258, 0.532554,\
        0.540820, 0.549118, 0.557465, 0.565844, 0.574224, 0.582571, 0.590813, 0.598639, 0.605245, 0.612376,\
        0.620585, 0.629634, 0.639263, 0.649299, 0.659572, 0.669859, 0.679862, 0.689173, 0.696970, 0.701671,\
        0.706283, 0.712461, 0.720308, 0.730541, 0.743613, 0.759270, 0.775857, 0.788929, 0.792687, 0.793239,\
        0.797180, 0.807376, 0.824996, 0.846903, 0.867116, 0.879905, 0.881646, 0.875323, 0.866195, 0.856480,\
        0.846920, 0.837737, 0.829111, 0.821392, 0.814889, 0.809603, 0.805753, 0.802796, 0.800071, 0.797747,\
        0.797016, 0.799463, 0.802915, 0.806432, 0.809942, 0.811575, 0.808705, 0.799453, 0.784621, 0.767421,\
        0.749450, 0.731937, 0.716158, 0.704138, 0.689101, 0.669749, 0.648839, 0.627864, 0.608438, 0.592286,\
        0.581060, 0.572884, 0.566419, 0.561002, 0.556163, 0.551449, 0.546500, 0.541080, 0.535067, 0.528443,\
        0.521229, 0.513470, 0.506316, 0.499170, 0.491922, 0.484011, 0.475216, 0.466429, 0.458063, 0.450133,\
        0.442542, 0.435195, 0.428028, 0.421009, 0.414142, 0.407489, 0.401043, 0.393924, 0.386391, 0.379055,\
        0.371831],\
        [0.346315, 0.356338, 0.366110, 0.375630, 0.384787, 0.393561, 0.402834, 0.412814, 0.423198, 0.433721,\
        0.444211, 0.454557, 0.464655, 0.474392, 0.483678, 0.492599, 0.501526, 0.510335, 0.518841, 0.527207,\
        0.535566, 0.543991, 0.552474, 0.560962, 0.569377, 0.577622, 0.585520, 0.592714, 0.599189, 0.606375,\
        0.614813, 0.623932, 0.633559, 0.643599, 0.653918, 0.664233, 0.674123, 0.683018, 0.690143, 0.695224,\
        0.700328, 0.706766, 0.714687, 0.724811, 0.737163, 0.750528, 0.762408, 0.769858, 0.772132, 0.773127,\
        0.777286, 0.787194, 0.804192, 0.825410, 0.844625, 0.857386, 0.861375, 0.858191, 0.851577, 0.844033,\
        0.836284, 0.828326, 0.820264, 0.812757, 0.806715, 0.801733, 0.798103, 0.795795, 0.793421, 0.791785,\
        0.791601, 0.793372, 0.796111, 0.799298, 0.802568, 0.805251, 0.800894, 0.790409, 0.773911, 0.755248,\
        0.735985, 0.717491, 0.700928, 0.687851, 0.674340, 0.657107, 0.637901, 0.618385, 0.600635, 0.585704,\
        0.573779, 0.564764, 0.557662, 0.551748, 0.546516, 0.541560, 0.536524, 0.531106, 0.525106, 0.518543,\
        0.511760, 0.504283, 0.496698, 0.490036, 0.482691, 0.474945, 0.466744, 0.458440, 0.450367, 0.442657,\
        0.435288, 0.428163, 0.421204, 0.414374, 0.407602, 0.401000, 0.394567, 0.387822, 0.380663, 0.373440,\
        0.366265],\
        [0.341546, 0.351290, 0.360760, 0.369950, 0.378864, 0.387679, 0.396818, 0.406614, 0.416910, 0.427326,\
        0.437699, 0.447938, 0.457967, 0.467723, 0.477188, 0.486449, 0.495623, 0.504450, 0.512928, 0.521283,\
        0.529681, 0.538215, 0.546816, 0.555371, 0.563758, 0.571847, 0.579478, 0.586522, 0.593197, 0.600422,\
        0.608605, 0.617302, 0.626541, 0.636227, 0.646313, 0.656400, 0.665923, 0.674318, 0.681163, 0.686642,\
        0.691825, 0.697522, 0.704514, 0.713397, 0.724341, 0.735235, 0.742990, 0.745656, 0.745056, 0.746064,\
        0.751007, 0.761979, 0.780141, 0.801678, 0.821190, 0.834967, 0.840601, 0.839340, 0.835182, 0.830561,\
        0.825412, 0.819271, 0.812056, 0.804503, 0.799033, 0.794627, 0.791149, 0.788702, 0.786790, 0.785614,\
        0.785568, 0.786755, 0.788754, 0.791136, 0.793349, 0.794604, 0.787871, 0.775062, 0.758683, 0.740455,\
        0.721189, 0.702582, 0.686882, 0.673174, 0.659613, 0.644124, 0.626890, 0.609300, 0.592580, 0.578462,\
        0.566671, 0.557129, 0.549395, 0.542920, 0.537250, 0.532046, 0.526975, 0.521655, 0.515810, 0.509184,\
        0.502690, 0.496863, 0.488890, 0.481196, 0.473607, 0.465862, 0.457936, 0.449978, 0.442208, 0.434787,\
        0.427732, 0.420887, 0.414113, 0.407527, 0.400872, 0.394281, 0.387825, 0.381209, 0.374346, 0.367359,\
        0.360352],\
        [0.336925, 0.346428, 0.355666, 0.364664, 0.373492, 0.382322, 0.391410, 0.401013, 0.411051, 0.421182,\
        0.431309, 0.441356, 0.451266, 0.460998, 0.470543, 0.479923, 0.489114, 0.497963, 0.506539, 0.514776,\
        0.523208, 0.531896, 0.540634, 0.549253, 0.557615, 0.565607, 0.573156, 0.580308, 0.587372, 0.594727,\
        0.601675, 0.610146, 0.618432, 0.627439, 0.637211, 0.647026, 0.656165, 0.664154, 0.670796, 0.676305,\
        0.681284, 0.686380, 0.692217, 0.698996, 0.708898, 0.717316, 0.722249, 0.720656, 0.715489, 0.716728,\
        0.721867, 0.733322, 0.753975, 0.777132, 0.797990, 0.814008, 0.820711, 0.819872, 0.817781, 0.816856,\
        0.814734, 0.810790, 0.805096, 0.797892, 0.792073, 0.787685, 0.784207, 0.781632, 0.779810, 0.778736,\
        0.778495, 0.779023, 0.779991, 0.780860, 0.780783, 0.777889, 0.769333, 0.755692, 0.739846, 0.723706,\
        0.706361, 0.688316, 0.672848, 0.659124, 0.645907, 0.631797, 0.616094, 0.599973, 0.584857, 0.571059,\
        0.559259, 0.549401, 0.541202, 0.534230, 0.528100, 0.522620, 0.517600, 0.512463, 0.506984, 0.500895,\
        0.493750, 0.487194, 0.479838, 0.472070, 0.464349, 0.456640, 0.448899, 0.441183, 0.433652, 0.426515,\
        0.419887, 0.413485, 0.406825, 0.400270, 0.394001, 0.387435, 0.380931, 0.374375, 0.367701, 0.360939,\
        0.354146],\
        [0.332490, 0.341798, 0.350867, 0.359746, 0.368526, 0.377354, 0.386417, 0.395848, 0.405488, 0.415247,\
        0.425049, 0.434838, 0.444572, 0.454214, 0.463734, 0.473101, 0.482262, 0.491175, 0.499755, 0.507593,\
        0.516306, 0.525231, 0.534113, 0.542788, 0.551128, 0.559051, 0.566543, 0.573706, 0.580743, 0.587806,\
        0.595258, 0.602359, 0.609134, 0.617479, 0.627162, 0.636745, 0.645455, 0.653014, 0.659350, 0.664635,\
        0.669312, 0.673942, 0.679049, 0.685246, 0.692733, 0.698099, 0.701729, 0.698040, 0.689469, 0.689496,\
        0.693349, 0.705413, 0.729044, 0.753249, 0.775704, 0.793538, 0.801474, 0.801110, 0.801252, 0.804168,\
        0.804260, 0.802196, 0.798005, 0.791542, 0.785332, 0.780482, 0.776793, 0.774074, 0.772151, 0.770895,\
        0.770198, 0.769872, 0.769532, 0.768506, 0.765727, 0.759660, 0.748956, 0.734031, 0.718346, 0.704101,\
        0.690374, 0.673558, 0.658209, 0.644718, 0.632345, 0.620036, 0.606043, 0.590651, 0.576199, 0.562895,\
        0.551174, 0.541171, 0.532732, 0.525431, 0.518813, 0.512896, 0.507902, 0.503339, 0.497272, 0.491792,\
        0.484512, 0.477212, 0.469815, 0.462255, 0.454697, 0.447183, 0.439678, 0.432180, 0.424804, 0.417826,\
        0.411657, 0.406105, 0.399718, 0.393076, 0.386725, 0.380315, 0.373862, 0.367382, 0.360857, 0.354298,\
        0.347730],\
        [0.328253, 0.337402, 0.346338, 0.355121, 0.363838, 0.372606, 0.381543, 0.390699, 0.400019, 0.409438,\
        0.418882, 0.428379, 0.437897, 0.447400, 0.456835, 0.466134, 0.475211, 0.483957, 0.492175, 0.500483,\
        0.509433, 0.518469, 0.527433, 0.536121, 0.544404, 0.552212, 0.559558, 0.566543, 0.573330, 0.580068,\
        0.586761, 0.592981, 0.598828, 0.607148, 0.616941, 0.626064, 0.634077, 0.641081, 0.646982, 0.651836,\
        0.656037, 0.660171, 0.664683, 0.669702, 0.674559, 0.677957, 0.678538, 0.673953, 0.666956, 0.664281,\
        0.668355, 0.682866, 0.705990, 0.729488, 0.752598, 0.770661, 0.780915, 0.784975, 0.788504, 0.791624,\
        0.792839, 0.791970, 0.787933, 0.782580, 0.776945, 0.772137, 0.768399, 0.765611, 0.763549, 0.761969,\
        0.760643, 0.759320, 0.757602, 0.754775, 0.749798, 0.741532, 0.729407, 0.713389, 0.696443, 0.683528,\
        0.670426, 0.656095, 0.641892, 0.629084, 0.617654, 0.606996, 0.595621, 0.580829, 0.566587, 0.553624,\
        0.542091, 0.532093, 0.523674, 0.516419, 0.509357, 0.502706, 0.497355, 0.492620, 0.486818, 0.480508,\
        0.473601, 0.466380, 0.459109, 0.451835, 0.444619, 0.437480, 0.430351, 0.423146, 0.415902, 0.408913,\
        0.402722, 0.398192, 0.392268, 0.385783, 0.379378, 0.373015, 0.366645, 0.360268, 0.353891, 0.347526,\
        0.341189],\
        [0.324209, 0.333221, 0.342039, 0.350720, 0.359342, 0.367989, 0.376739, 0.385626, 0.394640, 0.403670,\
        0.412767, 0.421976, 0.431256, 0.440604, 0.449948, 0.459179, 0.468193, 0.476910, 0.485375, 0.493937,\
        0.502693, 0.511766, 0.520709, 0.529331, 0.537490, 0.545099, 0.552167, 0.558803, 0.565166, 0.571384,\
        0.577469, 0.583398, 0.589740, 0.597789, 0.607288, 0.614984, 0.621900, 0.628313, 0.633714, 0.637925,\
        0.641351, 0.644822, 0.648621, 0.652389, 0.655331, 0.656550, 0.655103, 0.650260, 0.643755, 0.639948,\
        0.643271, 0.657408, 0.680008, 0.706035, 0.729824, 0.748330, 0.760784, 0.768543, 0.773969, 0.777733,\
        0.779483, 0.778880, 0.775838, 0.771351, 0.766502, 0.762151, 0.758667, 0.756006, 0.753897, 0.751986,\
        0.749938, 0.747531, 0.744511, 0.740217, 0.733672, 0.723905, 0.710744, 0.695913, 0.678604, 0.663975,\
        0.650780, 0.637546, 0.624263, 0.612120, 0.601579, 0.591390, 0.580673, 0.568094, 0.555139, 0.542971,\
        0.531869, 0.521943, 0.513581, 0.507084, 0.499842, 0.492592, 0.486418, 0.480752, 0.474769, 0.468347,\
        0.461614, 0.454718, 0.447802, 0.440931, 0.434173, 0.427565, 0.421000, 0.414261, 0.407271, 0.400390,\
        0.394372, 0.389371, 0.384018, 0.378025, 0.371833, 0.365596, 0.359343, 0.353093, 0.346871, 0.340702,\
        0.334601],\
        [0.320342, 0.329231, 0.337932, 0.346499, 0.354995, 0.363484, 0.372016, 0.380615, 0.389264, 0.397926,\
        0.406775, 0.415658, 0.424668, 0.433885, 0.443169, 0.452365, 0.461370, 0.470153, 0.478785, 0.487425,\
        0.496238, 0.505172, 0.513933, 0.522424, 0.530395, 0.537707, 0.544360, 0.550491, 0.556312, 0.562015,\
        0.567741, 0.573705, 0.580414, 0.588436, 0.596674, 0.602418, 0.608504, 0.614536, 0.619453, 0.622880,\
        0.625042, 0.627784, 0.630990, 0.633845, 0.635324, 0.634845, 0.632600, 0.627967, 0.621261, 0.616275,\
        0.617174, 0.630779, 0.655619, 0.683418, 0.708244, 0.727333, 0.741404, 0.751486, 0.758397, 0.762847,\
        0.764952, 0.764664, 0.762235, 0.758448, 0.754281, 0.750521, 0.747528, 0.745223, 0.743264, 0.741145,\
        0.738343, 0.734690, 0.730422, 0.725038, 0.717118, 0.706910, 0.691881, 0.675965, 0.659571, 0.644643,\
        0.631595, 0.619306, 0.606715, 0.594841, 0.584945, 0.575087, 0.564690, 0.553452, 0.541978, 0.531023,\
        0.520709, 0.510938, 0.502229, 0.496002, 0.489345, 0.481827, 0.474923, 0.468469, 0.462066, 0.455569,\
        0.449030, 0.442534, 0.436111, 0.429723, 0.423442, 0.417449, 0.411610, 0.405534, 0.398966, 0.392409,\
        0.386479, 0.381119, 0.375768, 0.370095, 0.364178, 0.358136, 0.352039, 0.345937, 0.339878, 0.333907,\
        0.328051],\
        [0.316637, 0.325408, 0.333990, 0.342432, 0.350786, 0.359098, 0.367405, 0.375727, 0.384076, 0.392503,\
        0.401055, 0.409364, 0.418145, 0.427325, 0.436593, 0.445762, 0.454753, 0.463562, 0.472255, 0.480934,\
        0.489668, 0.498346, 0.507005, 0.515341, 0.523082, 0.530009, 0.536102, 0.541575, 0.546745, 0.551906,\
        0.557266, 0.563027, 0.569419, 0.576384, 0.582952, 0.588429, 0.593956, 0.599311, 0.603787, 0.606706,\
        0.606823, 0.609339, 0.611934, 0.614358, 0.614916, 0.612726, 0.610570, 0.606959, 0.600533, 0.594954,\
        0.594520, 0.608450, 0.635050, 0.662819, 0.688030, 0.706929, 0.723134, 0.734597, 0.742432, 0.747495,\
        0.749990, 0.749977, 0.747831, 0.744399, 0.740683, 0.737486, 0.735138, 0.733385, 0.731816, 0.729811,\
        0.726364, 0.721102, 0.715191, 0.709467, 0.699616, 0.687825, 0.672472, 0.655518, 0.638996, 0.624535,\
        0.612335, 0.601503, 0.590003, 0.578882, 0.568798, 0.558903, 0.548664, 0.538143, 0.527736, 0.518064,\
        0.508987, 0.499573, 0.491027, 0.483894, 0.477093, 0.469890, 0.462756, 0.455844, 0.449089, 0.442509,\
        0.436181, 0.430137, 0.424352, 0.418528, 0.412557, 0.407156, 0.401992, 0.396567, 0.390489, 0.384370,\
        0.378589, 0.373147, 0.367785, 0.362285, 0.356598, 0.350765, 0.344846, 0.338905, 0.333013, 0.327239,\
        0.321635],\
        [0.313077, 0.321732, 0.330194, 0.338506, 0.346713, 0.354850, 0.362943, 0.371008, 0.379059, 0.387103,\
        0.395035, 0.403001, 0.411858, 0.421093, 0.430320, 0.439398, 0.448295, 0.457029, 0.465655, 0.474243,\
        0.482823, 0.491379, 0.499857, 0.507932, 0.515482, 0.521931, 0.527314, 0.531959, 0.536350, 0.540902,\
        0.545784, 0.551046, 0.556709, 0.562569, 0.568117, 0.573096, 0.577791, 0.582110, 0.585851, 0.587946,\
        0.587946, 0.589789, 0.591519, 0.593540, 0.593648, 0.590457, 0.588096, 0.584760, 0.579628, 0.576196,\
        0.579994, 0.596534, 0.619114, 0.644858, 0.669229, 0.690064, 0.706851, 0.718144, 0.726493, 0.732151,\
        0.735149, 0.735371, 0.733186, 0.729722, 0.726175, 0.723389, 0.721843, 0.720697, 0.719606, 0.718322,\
        0.714828, 0.707799, 0.699036, 0.692171, 0.681246, 0.667294, 0.651457, 0.634621, 0.618092, 0.603784,\
        0.593357, 0.583124, 0.573133, 0.563121, 0.553346, 0.543370, 0.533020, 0.522868, 0.513051, 0.504130,\
        0.496293, 0.487624, 0.479211, 0.471649, 0.464511, 0.457375, 0.450223, 0.443108, 0.436101, 0.429456,\
        0.423382, 0.417779, 0.412775, 0.407960, 0.401850, 0.396928, 0.392188, 0.387139, 0.381688, 0.376091,\
        0.370611, 0.365298, 0.360042, 0.354711, 0.349237, 0.343618, 0.337892, 0.332124, 0.326401, 0.320819,\
        0.315471],\
        [0.309643, 0.318186, 0.326527, 0.334712, 0.342779, 0.350755, 0.358658, 0.366499, 0.374282, 0.381994,\
        0.389636, 0.397515, 0.406292, 0.415409, 0.424413, 0.433253, 0.441926, 0.450449, 0.458867, 0.467236,\
        0.475590, 0.483916, 0.492100, 0.500104, 0.507439, 0.513264, 0.517842, 0.521503, 0.524987, 0.528935,\
        0.533300, 0.537901, 0.542724, 0.547650, 0.552325, 0.556463, 0.560052, 0.563153, 0.565646, 0.567108,\
        0.567789, 0.568798, 0.569934, 0.570819, 0.570345, 0.568122, 0.565404, 0.562099, 0.558497, 0.557408,\
        0.563463, 0.579391, 0.602327, 0.628348, 0.652927, 0.674040, 0.690200, 0.702002, 0.710845, 0.717177,\
        0.720922, 0.721423, 0.718804, 0.714933, 0.711487, 0.708634, 0.708288, 0.707476, 0.706296, 0.706307,\
        0.704372, 0.695504, 0.684598, 0.673520, 0.660974, 0.646416, 0.630360, 0.614085, 0.598960, 0.585278,\
        0.574901, 0.565512, 0.556478, 0.547624, 0.538649, 0.528893, 0.518124, 0.507971, 0.499272, 0.490253,\
        0.482541, 0.474596, 0.466647, 0.459084, 0.451863, 0.444799, 0.437767, 0.430671, 0.423472, 0.416692,\
        0.411061, 0.405826, 0.401284, 0.397790, 0.392473, 0.387275, 0.382588, 0.377813, 0.372821, 0.367705,\
        0.362599, 0.357552, 0.352520, 0.347420, 0.342190, 0.336810, 0.331303, 0.325735, 0.320202, 0.314816,\
        0.309722],\
        [0.306318, 0.314748, 0.322970, 0.331038, 0.338982, 0.346819, 0.354564, 0.362231, 0.369840, 0.377421,\
        0.385081, 0.393086, 0.401634, 0.410238, 0.418782, 0.427233, 0.435551, 0.443722, 0.451780, 0.459786,\
        0.467779, 0.475757, 0.483666, 0.491378, 0.498184, 0.503527, 0.507477, 0.510093, 0.512568, 0.516133,\
        0.520064, 0.523872, 0.527808, 0.531947, 0.535856, 0.539030, 0.541327, 0.543094, 0.544487, 0.545487,\
        0.546287, 0.546971, 0.547373, 0.547495, 0.546932, 0.545363, 0.543109, 0.540385, 0.537701, 0.537084,\
        0.542735, 0.558894, 0.584185, 0.611944, 0.637502, 0.658348, 0.674427, 0.686660, 0.695889, 0.702842,\
        0.707668, 0.708521, 0.705169, 0.700611, 0.697503, 0.694788, 0.694660, 0.693791, 0.692516, 0.692144,\
        0.689478, 0.680881, 0.667921, 0.654345, 0.640291, 0.625614, 0.610499, 0.594275, 0.580759, 0.568416,\
        0.557992, 0.548807, 0.540340, 0.532509, 0.524762, 0.515891, 0.505004, 0.494114, 0.485576, 0.477103,\
        0.469107, 0.461357, 0.453737, 0.446354, 0.439252, 0.432397, 0.425690, 0.418977, 0.411899, 0.404498,\
        0.399636, 0.394952, 0.390635, 0.386954, 0.382609, 0.377885, 0.373291, 0.368723, 0.364079, 0.359370,\
        0.354653, 0.349950, 0.345235, 0.340444, 0.335520, 0.330429, 0.325186, 0.319867, 0.314599, 0.309498,\
        0.304657],\
        [0.303079, 0.311390, 0.319499, 0.327472, 0.335315, 0.343035, 0.350649, 0.358182, 0.365674, 0.373189,\
        0.380844, 0.388785, 0.396985, 0.405134, 0.413176, 0.421189, 0.429067, 0.436753, 0.444298, 0.451784,\
        0.459260, 0.466717, 0.474065, 0.481060, 0.487236, 0.492187, 0.495809, 0.497734, 0.499444, 0.502905,\
        0.506464, 0.509199, 0.512241, 0.515754, 0.519092, 0.521389, 0.522165, 0.522656, 0.523136, 0.523632,\
        0.524509, 0.525137, 0.524705, 0.524407, 0.524115, 0.523102, 0.521645, 0.519925, 0.518000, 0.516687,\
        0.519488, 0.536172, 0.564652, 0.595985, 0.622896, 0.642861, 0.658916, 0.671739, 0.681692, 0.689226,\
        0.694997, 0.695532, 0.692196, 0.687617, 0.684073, 0.681579, 0.680258, 0.679123, 0.677664, 0.675502,\
        0.670710, 0.661428, 0.648379, 0.633985, 0.619442, 0.604827, 0.590955, 0.576620, 0.563453, 0.552054,\
        0.542077, 0.532956, 0.524516, 0.517382, 0.511078, 0.503382, 0.493272, 0.482261, 0.472726, 0.464093,\
        0.455991, 0.448289, 0.440884, 0.433711, 0.426795, 0.420261, 0.414021, 0.407950, 0.402106, 0.394829,\
        0.389315, 0.384835, 0.380703, 0.376837, 0.372802, 0.368530, 0.364203, 0.359875, 0.355533, 0.351185,\
        0.346853, 0.342538, 0.338210, 0.333809, 0.329265, 0.324531, 0.319611, 0.314590, 0.309686, 0.305223,\
        0.301037],\
        [0.299882, 0.308055, 0.316096, 0.324016, 0.331777, 0.339388, 0.346877, 0.354280, 0.361648, 0.369048,\
        0.376563, 0.384246, 0.392038, 0.399747, 0.407366, 0.415021, 0.422405, 0.429462, 0.436344, 0.443169,\
        0.449989, 0.456778, 0.463407, 0.469632, 0.475113, 0.479547, 0.482760, 0.484809, 0.486740, 0.489433,\
        0.492297, 0.494175, 0.496598, 0.499332, 0.502231, 0.503953, 0.502951, 0.502561, 0.502375, 0.502295,\
        0.503288, 0.504423, 0.502664, 0.502310, 0.502482, 0.501802, 0.501153, 0.500512, 0.499541, 0.498507,\
        0.497938, 0.516167, 0.546454, 0.579197, 0.605742, 0.625831, 0.642908, 0.656713, 0.667328, 0.675216,\
        0.680211, 0.681083, 0.678441, 0.674333, 0.670419, 0.667377, 0.665243, 0.663477, 0.661302, 0.657738,\
        0.651378, 0.641175, 0.627738, 0.613059, 0.599010, 0.585047, 0.571617, 0.558621, 0.546571, 0.536109,\
        0.526942, 0.518072, 0.509022, 0.501844, 0.496378, 0.489036, 0.479680, 0.469538, 0.459984, 0.451231,\
        0.443084, 0.435459, 0.428324, 0.421421, 0.414619, 0.408437, 0.402711, 0.397098, 0.391646, 0.385681,\
        0.380105, 0.375429, 0.371268, 0.367352, 0.363441, 0.359433, 0.355362, 0.351280, 0.347217, 0.343199,\
        0.339243, 0.335341, 0.331455, 0.327515, 0.323432, 0.319130, 0.314596, 0.309906, 0.305284, 0.301436,\
        0.298753],\
        [0.296590, 0.304713, 0.312819, 0.320707, 0.328372, 0.335857, 0.343202, 0.350448, 0.357645, 0.364852,\
        0.372122, 0.379471, 0.386842, 0.394125, 0.401399, 0.408796, 0.415516, 0.421756, 0.427860, 0.433932,\
        0.440015, 0.446077, 0.451971, 0.457468, 0.462309, 0.466275, 0.469270, 0.471475, 0.473433, 0.475531,\
        0.477550, 0.479211, 0.480991, 0.482980, 0.485010, 0.485730, 0.484538, 0.483515, 0.482905, 0.482701,\
        0.483326, 0.483819, 0.482678, 0.482095, 0.482129, 0.481909, 0.481701, 0.481607, 0.481519, 0.482088,\
        0.486910, 0.503687, 0.530156, 0.559619, 0.585763, 0.607582, 0.626893, 0.641511, 0.652416, 0.660330,\
        0.665001, 0.666036, 0.663932, 0.660052, 0.655798, 0.652098, 0.649273, 0.646954, 0.644181, 0.639802,\
        0.632755, 0.621843, 0.608147, 0.593277, 0.579635, 0.566365, 0.553478, 0.541147, 0.529852, 0.520301,\
        0.512345, 0.504226, 0.495139, 0.487198, 0.480753, 0.473501, 0.464926, 0.455766, 0.446825, 0.438386,\
        0.430371, 0.422838, 0.416144, 0.409839, 0.403027, 0.397030, 0.391778, 0.386543, 0.381361, 0.376097,\
        0.371022, 0.366439, 0.362288, 0.358386, 0.354563, 0.350721, 0.346848, 0.342976, 0.339156, 0.335435,\
        0.331840, 0.328360, 0.324950, 0.321530, 0.317984, 0.314180, 0.310081, 0.305818, 0.301461, 0.297639,\
        0.294526],\
        [0.293650, 0.301784, 0.309851, 0.317585, 0.325092, 0.332416, 0.339583, 0.346625, 0.353589, 0.360527,\
        0.367480, 0.374466, 0.381471, 0.388486, 0.395589, 0.402541, 0.408147, 0.413479, 0.418826, 0.424111,\
        0.429443, 0.434809, 0.440028, 0.444878, 0.449175, 0.452771, 0.455597, 0.457787, 0.459631, 0.461318,\
        0.462804, 0.464057, 0.465245, 0.466433, 0.467367, 0.467407, 0.466460, 0.465352, 0.464586, 0.464294,\
        0.464417, 0.464390, 0.463836, 0.463389, 0.463320, 0.463387, 0.463580, 0.464054, 0.465217, 0.468365,\
        0.476392, 0.492226, 0.515467, 0.542237, 0.568172, 0.591964, 0.612063, 0.625968, 0.636955, 0.645153,\
        0.649928, 0.651084, 0.649089, 0.645040, 0.640229, 0.635754, 0.632306, 0.629725, 0.626743, 0.621887,\
        0.615173, 0.604569, 0.590093, 0.575814, 0.561990, 0.548955, 0.536600, 0.524726, 0.513607, 0.504503,\
        0.497390, 0.489935, 0.481371, 0.473202, 0.465738, 0.458114, 0.449930, 0.441533, 0.433414, 0.425675,\
        0.418064, 0.410478, 0.404122, 0.398938, 0.392504, 0.386568, 0.381351, 0.376378, 0.371498, 0.366685,\
        0.362046, 0.357711, 0.353686, 0.349873, 0.346154, 0.342445, 0.338719, 0.335005, 0.331375, 0.327907,\
        0.324647, 0.321580, 0.318648, 0.315778, 0.312839, 0.309587, 0.305870, 0.301942, 0.298101, 0.294118,\
        0.290367],\
        [0.291137, 0.299411, 0.307112, 0.314568, 0.321896, 0.329039, 0.335990, 0.342776, 0.349441, 0.356033,\
        0.362598, 0.369166, 0.375751, 0.382361, 0.388948, 0.395003, 0.399763, 0.404602, 0.409332, 0.413787,\
        0.418418, 0.423197, 0.427837, 0.432108, 0.435945, 0.439253, 0.441884, 0.443924, 0.445597, 0.447000,\
        0.448087, 0.448857, 0.449429, 0.449882, 0.450084, 0.449749, 0.448849, 0.447818, 0.447096, 0.446776,\
        0.446674, 0.446481, 0.446111, 0.445865, 0.445961, 0.446344, 0.446993, 0.448169, 0.450565, 0.455551,\
        0.465069, 0.480613, 0.502050, 0.526701, 0.552009, 0.576016, 0.595744, 0.608980, 0.621282, 0.630181,\
        0.635331, 0.636570, 0.634348, 0.629774, 0.624154, 0.618722, 0.614523, 0.611984, 0.609126, 0.604034,\
        0.596534, 0.587451, 0.573187, 0.558873, 0.545333, 0.532619, 0.521038, 0.510049, 0.498923, 0.489878,\
        0.482478, 0.475102, 0.467140, 0.459099, 0.451234, 0.443309, 0.435212, 0.427289, 0.420013, 0.413273,\
        0.406475, 0.399261, 0.392750, 0.387613, 0.382091, 0.376584, 0.371483, 0.366685, 0.362068, 0.357597,\
        0.353311, 0.349257, 0.345436, 0.341783, 0.338209, 0.334635, 0.331028, 0.327416, 0.323904, 0.320628,\
        0.317670, 0.314992, 0.312492, 0.310130, 0.307847, 0.305245, 0.301815, 0.297983, 0.294166, 0.290223,\
        0.286153],\
        [0.288485, 0.296522, 0.304082, 0.311535, 0.318763, 0.325717, 0.332414, 0.338886, 0.345180, 0.351350,\
        0.357446, 0.363505, 0.369538, 0.375510, 0.381269, 0.386468, 0.390995, 0.395376, 0.399456, 0.403088,\
        0.407168, 0.411506, 0.415671, 0.419347, 0.422773, 0.425932, 0.428298, 0.430038, 0.431525, 0.432754,\
        0.433529, 0.433798, 0.433762, 0.433625, 0.433414, 0.432932, 0.431980, 0.430951, 0.430391, 0.430219,\
        0.430147, 0.429929, 0.429582, 0.429487, 0.429923, 0.430744, 0.431876, 0.433650, 0.436850, 0.442755,\
        0.452858, 0.468132, 0.488190, 0.511328, 0.535625, 0.558489, 0.577664, 0.593217, 0.606425, 0.615710,\
        0.621451, 0.622721, 0.619958, 0.614771, 0.608182, 0.602102, 0.596853, 0.594043, 0.590658, 0.585230,\
        0.577364, 0.567484, 0.554888, 0.541629, 0.529180, 0.517265, 0.505964, 0.497149, 0.486221, 0.476752,\
        0.468635, 0.460943, 0.453135, 0.445229, 0.437319, 0.429298, 0.421157, 0.413370, 0.406782, 0.401131,\
        0.395041, 0.388633, 0.382498, 0.377115, 0.371945, 0.366871, 0.362029, 0.357439, 0.353061, 0.348874,\
        0.344889, 0.341117, 0.337542, 0.334108, 0.330731, 0.327326, 0.323836, 0.320279, 0.316795, 0.313626,\
        0.310937, 0.308637, 0.306492, 0.304426, 0.302749, 0.300755, 0.297742, 0.293915, 0.290010, 0.285978,\
        0.281744],\
        [0.285758, 0.293640, 0.301211, 0.308677, 0.315750, 0.322468, 0.328862, 0.334960, 0.340811, 0.346478,\
        0.352023, 0.357486, 0.362873, 0.368132, 0.373133, 0.377716, 0.381865, 0.385714, 0.389216, 0.392516,\
        0.396118, 0.399993, 0.403790, 0.406717, 0.409760, 0.413148, 0.414976, 0.416187, 0.417536, 0.418743,\
        0.419307, 0.419053, 0.418439, 0.417852, 0.417469, 0.417184, 0.416116, 0.414829, 0.414607, 0.414716,\
        0.414837, 0.414678, 0.414227, 0.414168, 0.415121, 0.416455, 0.417977, 0.420109, 0.423567, 0.429578,\
        0.439570, 0.454347, 0.473609, 0.496182, 0.519230, 0.541550, 0.561319, 0.578034, 0.591500, 0.601986,\
        0.608231, 0.609548, 0.605634, 0.600417, 0.593134, 0.586530, 0.580880, 0.576413, 0.571747, 0.565587,\
        0.557472, 0.547468, 0.535908, 0.523896, 0.513026, 0.504359, 0.492697, 0.483126, 0.473337, 0.463973,\
        0.455514, 0.447575, 0.439744, 0.431922, 0.424196, 0.416384, 0.408273, 0.400546, 0.394302, 0.389108,\
        0.383710, 0.378036, 0.372471, 0.367260, 0.362316, 0.357550, 0.352976, 0.348613, 0.344463, 0.340526,\
        0.336810, 0.333313, 0.330010, 0.326842, 0.323722, 0.320546, 0.317217, 0.313703, 0.310147, 0.306963,\
        0.304520, 0.302579, 0.300809, 0.298809, 0.297343, 0.295518, 0.292803, 0.289324, 0.285491, 0.281427,\
        0.277113],\
        [0.283191, 0.291188, 0.298867, 0.306066, 0.312864, 0.319305, 0.325354, 0.331019, 0.336352, 0.341439,\
        0.346360, 0.351170, 0.355880, 0.360446, 0.364785, 0.368813, 0.372512, 0.375919, 0.379078, 0.382148,\
        0.385357, 0.388718, 0.391950, 0.394457, 0.397440, 0.401211, 0.401826, 0.402445, 0.403750, 0.405028,\
        0.405509, 0.404700, 0.403659, 0.402792, 0.402326, 0.402627, 0.401605, 0.399698, 0.400022, 0.400270,\
        0.400605, 0.400584, 0.400095, 0.399938, 0.401512, 0.403184, 0.404895, 0.407194, 0.410458, 0.415922,\
        0.425336, 0.439564, 0.458363, 0.480210, 0.503146, 0.525726, 0.545689, 0.562605, 0.576547, 0.587303,\
        0.594728, 0.595709, 0.591375, 0.585445, 0.578470, 0.571408, 0.565018, 0.559210, 0.553240, 0.546280,\
        0.537910, 0.528077, 0.517314, 0.506585, 0.496457, 0.489082, 0.479706, 0.469632, 0.460088, 0.451090,\
        0.442750, 0.434862, 0.427084, 0.419275, 0.411832, 0.404733, 0.396900, 0.389573, 0.383395, 0.378101,\
        0.373000, 0.367854, 0.362771, 0.357878, 0.353192, 0.348686, 0.344355, 0.340211, 0.336270, 0.332553,\
        0.329079, 0.325848, 0.322827, 0.319958, 0.317151, 0.314286, 0.311220, 0.307828, 0.304187, 0.300843,\
        0.298612, 0.296889, 0.295272, 0.293565, 0.291919, 0.290011, 0.287462, 0.284266, 0.280600, 0.276589,\
        0.272263],\
        [0.280693, 0.288750, 0.296342, 0.303404, 0.310043, 0.316244, 0.321928, 0.327095, 0.331831, 0.336260,\
        0.340499, 0.344626, 0.348663, 0.352583, 0.356330, 0.359848, 0.363118, 0.366158, 0.369024, 0.371824,\
        0.374662, 0.377529, 0.380264, 0.382799, 0.385590, 0.388094, 0.388891, 0.389397, 0.390353, 0.391356,\
        0.391627, 0.390766, 0.389676, 0.388821, 0.388433, 0.388567, 0.387866, 0.386648, 0.386547, 0.386830,\
        0.387164, 0.387252, 0.387097, 0.387386, 0.388699, 0.390352, 0.392141, 0.394420, 0.397357, 0.402063,\
        0.410710, 0.424310, 0.442964, 0.465072, 0.488621, 0.511307, 0.530399, 0.546958, 0.560781, 0.571429,\
        0.578056, 0.579394, 0.576073, 0.570212, 0.563221, 0.555951, 0.548950, 0.542212, 0.535295, 0.527692,\
        0.519265, 0.509822, 0.499672, 0.490287, 0.481431, 0.473434, 0.464918, 0.455714, 0.446655, 0.438140,\
        0.430223, 0.422714, 0.415265, 0.407469, 0.399945, 0.393485, 0.386429, 0.379569, 0.373500, 0.368135,\
        0.363139, 0.358294, 0.353567, 0.348988, 0.344571, 0.340308, 0.336199, 0.332251, 0.328491, 0.324958,\
        0.321698, 0.318716, 0.315972, 0.313407, 0.310941, 0.308434, 0.305758, 0.302640, 0.299144, 0.295874,\
        0.293497, 0.291674, 0.290021, 0.288355, 0.286595, 0.284560, 0.282046, 0.278993, 0.275462, 0.271527,\
        0.267228],\
        [0.278187, 0.286298, 0.293825, 0.300795, 0.307363, 0.313365, 0.318647, 0.323230, 0.327268, 0.330958,\
        0.334469, 0.337905, 0.341299, 0.344629, 0.347846, 0.350904, 0.353779, 0.356484, 0.359058, 0.361561,\
        0.364041, 0.366489, 0.368852, 0.371126, 0.373320, 0.375067, 0.376006, 0.376585, 0.377219, 0.377765,\
        0.377805, 0.377207, 0.376365, 0.375671, 0.375310, 0.375156, 0.374748, 0.374222, 0.374113, 0.374349,\
        0.374672, 0.374911, 0.375129, 0.375667, 0.376745, 0.378188, 0.379883, 0.381973, 0.384780, 0.389250,\
        0.396912, 0.409676, 0.428390, 0.450781, 0.474283, 0.495994, 0.514889, 0.531076, 0.544591, 0.554810,\
        0.560977, 0.562524, 0.559884, 0.554401, 0.547451, 0.540023, 0.532635, 0.525344, 0.517901, 0.509863,\
        0.501454, 0.492864, 0.483638, 0.474966, 0.466769, 0.458764, 0.450449, 0.441746, 0.433186, 0.425247,\
        0.417922, 0.410983, 0.404169, 0.396870, 0.389460, 0.382854, 0.376371, 0.370058, 0.364231, 0.358938,\
        0.354039, 0.349394, 0.344931, 0.340627, 0.336466, 0.332436, 0.328533, 0.324763, 0.321149, 0.317753,\
        0.314668, 0.311925, 0.309432, 0.307126, 0.304991, 0.302783, 0.300449, 0.297696, 0.294439, 0.291397,\
        0.288900, 0.286869, 0.285060, 0.283277, 0.281379, 0.279220, 0.276669, 0.273661, 0.270201, 0.266322,\
        0.262055],\
        [0.275644, 0.283925, 0.291555, 0.298518, 0.305018, 0.310804, 0.315579, 0.319441, 0.322656, 0.325524,\
        0.328278, 0.331041, 0.333843, 0.336649, 0.339402, 0.342048, 0.344563, 0.346950, 0.349229, 0.351428,\
        0.353564, 0.355635, 0.357617, 0.359483, 0.361158, 0.362468, 0.363319, 0.363871, 0.364303, 0.364577,\
        0.364533, 0.364136, 0.363574, 0.363085, 0.362793, 0.362644, 0.362499, 0.362400, 0.362510, 0.362831,\
        0.363244, 0.363664, 0.364134, 0.364814, 0.365817, 0.367139, 0.368789, 0.370929, 0.373948, 0.378603,\
        0.386010, 0.397837, 0.416135, 0.437967, 0.460265, 0.481197, 0.499708, 0.515469, 0.528830, 0.538690,\
        0.544694, 0.546325, 0.543885, 0.538469, 0.531406, 0.523811, 0.516190, 0.508580, 0.500865, 0.492830,\
        0.484528, 0.476450, 0.468338, 0.460461, 0.452793, 0.445032, 0.436864, 0.428306, 0.419965, 0.412628,\
        0.405950, 0.399525, 0.393149, 0.386459, 0.379627, 0.373138, 0.366948, 0.361025, 0.355473, 0.350337,\
        0.345573, 0.341103, 0.336864, 0.332806, 0.328889, 0.325083, 0.321378, 0.317779, 0.314293, 0.310971,\
        0.308002, 0.305505, 0.303249, 0.301091, 0.299213, 0.297226, 0.295018, 0.292508, 0.289684, 0.286910,\
        0.284434, 0.282270, 0.280292, 0.278344, 0.276288, 0.273998, 0.271371, 0.268344, 0.264899, 0.261044,\
        0.256794],\
        [0.272974, 0.281571, 0.289531, 0.296765, 0.303248, 0.308623, 0.312720, 0.315686, 0.317938, 0.319916,\
        0.321916, 0.324058, 0.326344, 0.328710, 0.331069, 0.333353, 0.335535, 0.337619, 0.339613, 0.341512,\
        0.343317, 0.345027, 0.346626, 0.348082, 0.349335, 0.350313, 0.350994, 0.351448, 0.351756, 0.351919,\
        0.351889, 0.351665, 0.351347, 0.351076, 0.350943, 0.350957, 0.351085, 0.351336, 0.351738, 0.352272,\
        0.352872, 0.353496, 0.354171, 0.354987, 0.356036, 0.357396, 0.359182, 0.361631, 0.365180, 0.370523,\
        0.378625, 0.390721, 0.407764, 0.427628, 0.448027, 0.467668, 0.485595, 0.501862, 0.514179, 0.523911,\
        0.530088, 0.531622, 0.528874, 0.523057, 0.515543, 0.507661, 0.499884, 0.491970, 0.484033, 0.476062,\
        0.468218, 0.460727, 0.453538, 0.446559, 0.439590, 0.432316, 0.424443, 0.415937, 0.407366, 0.400591,\
        0.394557, 0.388542, 0.382528, 0.376353, 0.370097, 0.363982, 0.358103, 0.352491, 0.347195, 0.342249,\
        0.337645, 0.333353, 0.329326, 0.325508, 0.321835, 0.318249, 0.314733, 0.311320, 0.308003, 0.304712,\
        0.301749, 0.299491, 0.297459, 0.295464, 0.293630, 0.291758, 0.289686, 0.287379, 0.284875, 0.282350,\
        0.279957, 0.277736, 0.275627, 0.273522, 0.271310, 0.268884, 0.266158, 0.263075, 0.259604, 0.255739,\
        0.251483],\
        [0.269973, 0.278976, 0.287401, 0.295044, 0.301532, 0.306611, 0.309927, 0.311825, 0.312991, 0.314062,\
        0.315376, 0.316995, 0.318869, 0.320893, 0.322934, 0.324896, 0.326763, 0.328566, 0.330299, 0.331913,\
        0.333395, 0.334759, 0.335996, 0.337076, 0.337973, 0.338665, 0.339156, 0.339490, 0.339723, 0.339873,\
        0.339922, 0.339872, 0.339774, 0.339723, 0.339803, 0.340057, 0.340491, 0.341096, 0.341843, 0.342677,\
        0.343538, 0.344396, 0.345277, 0.346256, 0.347439, 0.348959, 0.351011, 0.353901, 0.358099, 0.364267,\
        0.373227, 0.385745, 0.401607, 0.419061, 0.437173, 0.455506, 0.473079, 0.488564, 0.500797, 0.511444,\
        0.518040, 0.518982, 0.515335, 0.508715, 0.500619, 0.492214, 0.484062, 0.475786, 0.467587, 0.459673,\
        0.452261, 0.445468, 0.439195, 0.433191, 0.427091, 0.420541, 0.413268, 0.405118, 0.396772, 0.389800,\
        0.383968, 0.378265, 0.372565, 0.366804, 0.361016, 0.355305, 0.349761, 0.344440, 0.339384, 0.334629,\
        0.330193, 0.326079, 0.322265, 0.318697, 0.315283, 0.311922, 0.308572, 0.305322, 0.302311, 0.299148,\
        0.296257, 0.293997, 0.292030, 0.290158, 0.288351, 0.286513, 0.284539, 0.282392, 0.280103, 0.277761,\
        0.275454, 0.273214, 0.271011, 0.268776, 0.266421, 0.263863, 0.261028, 0.257864, 0.254338, 0.250434,\
        0.246150],\
        [0.266374, 0.275759, 0.284646, 0.292755, 0.299597, 0.304526, 0.306862, 0.307609, 0.307657, 0.307920,\
        0.308714, 0.309940, 0.311512, 0.313303, 0.315105, 0.316770, 0.318316, 0.319866, 0.321396, 0.322743,\
        0.323894, 0.324933, 0.325845, 0.326586, 0.327170, 0.327612, 0.327917, 0.328132, 0.328323, 0.328520,\
        0.328701, 0.328838, 0.328945, 0.329105, 0.329418, 0.329947, 0.330716, 0.331712, 0.332860, 0.334058,\
        0.335228, 0.336344, 0.337440, 0.338601, 0.339957, 0.341684, 0.344025, 0.347321, 0.352031, 0.358729,\
        0.368018, 0.380287, 0.395190, 0.410344, 0.426500, 0.444006, 0.461367, 0.477300, 0.491130, 0.502711,\
        0.509042, 0.508290, 0.503270, 0.495689, 0.487052, 0.478058, 0.469123, 0.460280, 0.451711, 0.443742,\
        0.436673, 0.430588, 0.425233, 0.420256, 0.415095, 0.409288, 0.402825, 0.395243, 0.387429, 0.380460,\
        0.374408, 0.368784, 0.363308, 0.357872, 0.352462, 0.347120, 0.341902, 0.336858, 0.332030, 0.327457,\
        0.323178, 0.319228, 0.315622, 0.312316, 0.309191, 0.306077, 0.302889, 0.299698, 0.296877, 0.294025,\
        0.291322, 0.289020, 0.287014, 0.285154, 0.283350, 0.281524, 0.279606, 0.277563, 0.275409, 0.273187,\
        0.270943, 0.268694, 0.266423, 0.264079, 0.261600, 0.258917, 0.255969, 0.252711, 0.249109, 0.245145,\
        0.240811],\
        [0.261921, 0.271545, 0.280751, 0.289308, 0.296771, 0.301242, 0.302794, 0.302760, 0.301881, 0.301641,\
        0.302141, 0.303075, 0.304409, 0.306067, 0.307695, 0.309073, 0.310243, 0.311581, 0.313059, 0.314123,\
        0.314881, 0.315639, 0.316294, 0.316707, 0.317010, 0.317258, 0.317388, 0.317482, 0.317654, 0.317946,\
        0.318309, 0.318656, 0.318947, 0.319289, 0.319839, 0.320650, 0.321772, 0.323206, 0.324816, 0.326424,\
        0.327924, 0.329298, 0.330600, 0.331935, 0.333455, 0.335362, 0.337919, 0.341464, 0.346403, 0.353179,\
        0.362158, 0.373407, 0.386286, 0.399616, 0.415493, 0.432811, 0.450309, 0.467174, 0.482737, 0.495947,\
        0.501268, 0.498311, 0.491816, 0.483532, 0.474240, 0.464635, 0.455036, 0.445588, 0.436550, 0.428393,\
        0.421536, 0.416170, 0.411631, 0.407562, 0.403385, 0.397960, 0.391995, 0.385193, 0.378124, 0.371499,\
        0.365504, 0.359968, 0.354684, 0.349526, 0.344441, 0.339429, 0.334517, 0.329740, 0.325131, 0.320729,\
        0.316587, 0.312770, 0.309348, 0.306296, 0.303463, 0.300612, 0.297593, 0.294595, 0.291793, 0.289131,\
        0.286618, 0.284354, 0.282320, 0.280432, 0.278605, 0.276771, 0.274876, 0.272890, 0.270811, 0.268657,\
        0.266446, 0.264184, 0.261854, 0.259419, 0.256830, 0.254030, 0.250972, 0.247611, 0.243919, 0.239876,\
        0.235474],\
        [0.256460, 0.266092, 0.275283, 0.283708, 0.290689, 0.295055, 0.296956, 0.296991, 0.296149, 0.295724,\
        0.295971, 0.296667, 0.297770, 0.299289, 0.300715, 0.301876, 0.302611, 0.303777, 0.305547, 0.306146,\
        0.306353, 0.306958, 0.307485, 0.307508, 0.307558, 0.307731, 0.307681, 0.307654, 0.307823, 0.308243,\
        0.308833, 0.309423, 0.309864, 0.310327, 0.311143, 0.312218, 0.313667, 0.315608, 0.317743, 0.319773,\
        0.321583, 0.323184, 0.324655, 0.326125, 0.327761, 0.329774, 0.332423, 0.336019, 0.340906, 0.347405,\
        0.355722, 0.365799, 0.377277, 0.390164, 0.405287, 0.421808, 0.439061, 0.456099, 0.471774, 0.484045,\
        0.489076, 0.486601, 0.479794, 0.471105, 0.461495, 0.451586, 0.441670, 0.431841, 0.422333, 0.413951,\
        0.407154, 0.402472, 0.398664, 0.394948, 0.391334, 0.386502, 0.380990, 0.374943, 0.368673, 0.362594,\
        0.356909, 0.351602, 0.346564, 0.341690, 0.336916, 0.332214, 0.327593, 0.323079, 0.318693, 0.314458,\
        0.310446, 0.306737, 0.303434, 0.300619, 0.297956, 0.295303, 0.292502, 0.289709, 0.287031, 0.284492,\
        0.282105, 0.279896, 0.277856, 0.275937, 0.274078, 0.272222, 0.270324, 0.268356, 0.266308, 0.264180,\
        0.261976, 0.259690, 0.257305, 0.254788, 0.252099, 0.249192, 0.246024, 0.242558, 0.238765, 0.234630,\
        0.230144],\
        [0.249997, 0.259427, 0.268336, 0.276338, 0.282845, 0.287282, 0.289619, 0.290309, 0.290134, 0.289938,\
        0.290113, 0.290683, 0.291619, 0.292824, 0.294025, 0.294999, 0.295696, 0.296912, 0.298736, 0.298700,\
        0.298459, 0.298974, 0.299450, 0.299068, 0.298930, 0.299114, 0.298901, 0.298818, 0.299002, 0.299539,\
        0.300330, 0.301173, 0.301757, 0.302256, 0.303483, 0.304739, 0.306414, 0.308998, 0.311684, 0.314075,\
        0.316121, 0.317879, 0.319460, 0.321009, 0.322696, 0.324723, 0.327333, 0.330804, 0.335428, 0.341464,\
        0.349069, 0.358251, 0.368941, 0.381235, 0.395199, 0.410665, 0.427224, 0.443586, 0.458152, 0.468850,\
        0.473611, 0.472185, 0.466288, 0.457911, 0.448434, 0.438689, 0.428974, 0.419287, 0.409561, 0.400915,\
        0.394606, 0.389945, 0.386388, 0.382918, 0.379306, 0.375031, 0.370115, 0.364759, 0.359207, 0.353727,\
        0.348491, 0.343539, 0.338828, 0.334282, 0.329832, 0.325436, 0.321095, 0.316849, 0.312713, 0.308665,\
        0.304759, 0.301357, 0.298030, 0.295330, 0.292757, 0.290206, 0.287607, 0.285007, 0.282477, 0.280056,\
        0.277760, 0.275599, 0.273563, 0.271620, 0.269725, 0.267836, 0.265916, 0.263938, 0.261887, 0.259755,\
        0.257535, 0.255214, 0.252773, 0.250180, 0.247399, 0.244391, 0.241118, 0.237544, 0.233645, 0.229404,\
        0.224818],\
        [0.242693, 0.251806, 0.260358, 0.267983, 0.274247, 0.278812, 0.281650, 0.283073, 0.283631, 0.283899,\
        0.284248, 0.284822, 0.285635, 0.286612, 0.287602, 0.288487, 0.289332, 0.290415, 0.291432, 0.291601,\
        0.291467, 0.291615, 0.291728, 0.291473, 0.291271, 0.291223, 0.291096, 0.291079, 0.291338, 0.291930,\
        0.292780, 0.293729, 0.294529, 0.295407, 0.296875, 0.298290, 0.300304, 0.303558, 0.306624, 0.309195,\
        0.311371, 0.313200, 0.314826, 0.316402, 0.318081, 0.320040, 0.322500, 0.325712, 0.329940, 0.335429,\
        0.342360, 0.350819, 0.360822, 0.372356, 0.385329, 0.399671, 0.415189, 0.430482, 0.443782, 0.453343,\
        0.457841, 0.457062, 0.451980, 0.444118, 0.435003, 0.425820, 0.416836, 0.407794, 0.398982, 0.390273,\
        0.383829, 0.378968, 0.375101, 0.371574, 0.367970, 0.363993, 0.359568, 0.354802, 0.349871, 0.344967,\
        0.340227, 0.335707, 0.331392, 0.327226, 0.323138, 0.319053, 0.314968, 0.310980, 0.307147, 0.303355,\
        0.299614, 0.296244, 0.293232, 0.290458, 0.287897, 0.285411, 0.282942, 0.280492, 0.278095, 0.275778,\
        0.273557, 0.271436, 0.269404, 0.267438, 0.265507, 0.263578, 0.261620, 0.259610, 0.257532, 0.255372,\
        0.253118, 0.250752, 0.248252, 0.245586, 0.242720, 0.239618, 0.236243, 0.232564, 0.228553, 0.224197,\
        0.219497],\
        [0.234795, 0.243571, 0.251812, 0.259194, 0.265383, 0.270150, 0.273475, 0.275569, 0.276808, 0.277596,\
        0.278246, 0.278934, 0.279718, 0.280577, 0.281442, 0.282268, 0.283080, 0.283903, 0.284541, 0.284766,\
        0.284746, 0.284723, 0.284653, 0.284459, 0.284266, 0.284151, 0.284102, 0.284198, 0.284542, 0.285176,\
        0.286055, 0.287076, 0.288161, 0.289441, 0.291079, 0.293015, 0.295648, 0.299104, 0.302186, 0.304816,\
        0.307079, 0.308890, 0.310519, 0.312101, 0.313737, 0.315572, 0.317806, 0.320670, 0.324417, 0.329309,\
        0.335587, 0.343408, 0.352810, 0.363721, 0.375984, 0.389404, 0.403673, 0.417780, 0.429959, 0.438699,\
        0.442963, 0.442501, 0.437913, 0.430365, 0.421417, 0.412870, 0.405232, 0.396732, 0.388457, 0.380643,\
        0.374115, 0.368964, 0.364765, 0.361035, 0.357389, 0.353576, 0.349499, 0.345193, 0.340773, 0.336382,\
        0.332133, 0.328075, 0.324195, 0.320456, 0.316786, 0.313044, 0.309163, 0.305346, 0.301845, 0.298348,\
        0.294818, 0.291550, 0.288592, 0.285867, 0.283321, 0.280882, 0.278499, 0.276157, 0.273863, 0.271631,\
        0.269470, 0.267378, 0.265347, 0.263358, 0.261388, 0.259411, 0.257403, 0.255345, 0.253221, 0.251015,\
        0.248713, 0.246294, 0.243733, 0.240997, 0.238051, 0.234860, 0.231391, 0.227611, 0.223490, 0.219009,\
        0.214177],\
        [0.226581, 0.235059, 0.243085, 0.250354, 0.256583, 0.261592, 0.265360, 0.268029, 0.269857, 0.271140,\
        0.272129, 0.272997, 0.273833, 0.274660, 0.275463, 0.276225, 0.276942, 0.277582, 0.278047, 0.278265,\
        0.278295, 0.278237, 0.278124, 0.277966, 0.277818, 0.277740, 0.277773, 0.277973, 0.278403, 0.279094,\
        0.280025, 0.281156, 0.282473, 0.284046, 0.285965, 0.288314, 0.291224, 0.294551, 0.297667, 0.300657,\
        0.302881, 0.304609, 0.306276, 0.307898, 0.309489, 0.311175, 0.313148, 0.315619, 0.318828, 0.323069,\
        0.328685, 0.335927, 0.344838, 0.355283, 0.367032, 0.379776, 0.393053, 0.406013, 0.417256, 0.425427,\
        0.429525, 0.429204, 0.424932, 0.417565, 0.408437, 0.399997, 0.393452, 0.385950, 0.378240, 0.371046,\
        0.364766, 0.359525, 0.355119, 0.351220, 0.347520, 0.343803, 0.339965, 0.336001, 0.331981, 0.328019,\
        0.324224, 0.320631, 0.317193, 0.313882, 0.310704, 0.307418, 0.303707, 0.299937, 0.296615, 0.293367,\
        0.290107, 0.287004, 0.284126, 0.281455, 0.278947, 0.276554, 0.274236, 0.271973, 0.269757, 0.267591,\
        0.265474, 0.263403, 0.261367, 0.259351, 0.257338, 0.255306, 0.253237, 0.251116, 0.248929, 0.246663,\
        0.244302, 0.241825, 0.239203, 0.236400, 0.233379, 0.230102, 0.226544, 0.222676, 0.218456, 0.213847,\
        0.208861],\
        [0.218339, 0.226588, 0.234508, 0.241768, 0.248101, 0.253353, 0.257501, 0.260640, 0.262954, 0.264664,\
        0.265976, 0.267048, 0.267983, 0.268833, 0.269614, 0.270328, 0.270967, 0.271500, 0.271884, 0.272093,\
        0.272155, 0.272122, 0.272036, 0.271930, 0.271852, 0.271850, 0.271973, 0.272269, 0.272784, 0.273542,\
        0.274545, 0.275785, 0.277273, 0.279052, 0.281183, 0.283718, 0.286662, 0.289859, 0.293044, 0.296025,\
        0.298222, 0.300061, 0.301864, 0.303592, 0.305148, 0.306690, 0.308422, 0.310506, 0.313162, 0.316670,\
        0.321578, 0.328302, 0.336837, 0.346936, 0.358284, 0.370495, 0.383040, 0.395127, 0.405662, 0.413576,\
        0.417655, 0.417327, 0.413376, 0.406522, 0.397630, 0.389321, 0.382292, 0.375308, 0.368233, 0.361546,\
        0.355571, 0.350411, 0.345958, 0.341989, 0.338271, 0.334628, 0.330967, 0.327260, 0.323538, 0.319903,\
        0.316505, 0.313385, 0.310371, 0.307393, 0.304667, 0.302131, 0.298570, 0.294973, 0.291649, 0.288507,\
        0.285449, 0.282516, 0.279750, 0.277154, 0.274703, 0.272364, 0.270104, 0.267903, 0.265748, 0.263631,\
        0.261546, 0.259486, 0.257440, 0.255393, 0.253330, 0.251235, 0.249095, 0.246898, 0.244633, 0.242292,\
        0.239862, 0.237322, 0.234643, 0.231781, 0.228689, 0.225327, 0.221678, 0.217735, 0.213446, 0.208729,\
        0.203563],\
        [0.210370, 0.218488, 0.226401, 0.233706, 0.240151, 0.245608, 0.250052, 0.253551, 0.256241, 0.258294,\
        0.259879, 0.261143, 0.262191, 0.263092, 0.263881, 0.264574, 0.265169, 0.265652, 0.266006, 0.266224,\
        0.266324, 0.266342, 0.266316, 0.266284, 0.266289, 0.266377, 0.266593, 0.266979, 0.267573, 0.268399,\
        0.269472, 0.270798, 0.272395, 0.274292, 0.276518, 0.279087, 0.281958, 0.285005, 0.288036, 0.290819,\
        0.293152, 0.295175, 0.297093, 0.298935, 0.300445, 0.301952, 0.303505, 0.305278, 0.307460, 0.310091,\
        0.314213, 0.320543, 0.328826, 0.338641, 0.349626, 0.361360, 0.373302, 0.384745, 0.394872, 0.402753,\
        0.407136, 0.406535, 0.402516, 0.396735, 0.388282, 0.379949, 0.372388, 0.365293, 0.358503, 0.352179,\
        0.346502, 0.341524, 0.337157, 0.333226, 0.329548, 0.325985, 0.322473, 0.318984, 0.315489, 0.312061,\
        0.308951, 0.306351, 0.303765, 0.301033, 0.298454, 0.296344, 0.293308, 0.289970, 0.286771, 0.283738,\
        0.280833, 0.278053, 0.275412, 0.272910, 0.270532, 0.268255, 0.266054, 0.263908, 0.261802, 0.259723,\
        0.257662, 0.255606, 0.253543, 0.251460, 0.249342, 0.247176, 0.244953, 0.242665, 0.240308, 0.237878,\
        0.235368, 0.232762, 0.230030, 0.227121, 0.223970, 0.220517, 0.216752, 0.212733, 0.208433, 0.203644,\
        0.198364],\
        [0.203056, 0.211166, 0.219094, 0.226404, 0.232904, 0.238490, 0.243132, 0.246877, 0.249829, 0.252127,\
        0.253919, 0.255338, 0.256489, 0.257448, 0.258263, 0.258959, 0.259544, 0.260017, 0.260376, 0.260622,\
        0.260774, 0.260858, 0.260908, 0.260961, 0.261057, 0.261237, 0.261542, 0.262010, 0.262675, 0.263561,\
        0.264687, 0.266066, 0.267715, 0.269648, 0.271877, 0.274386, 0.277121, 0.279976, 0.282802, 0.285439,\
        0.287792, 0.289901, 0.291853, 0.293652, 0.295224, 0.296775, 0.298182, 0.299954, 0.301724, 0.303363,\
        0.306663, 0.312903, 0.320972, 0.330456, 0.341042, 0.352285, 0.363652, 0.374525, 0.384211, 0.391884,\
        0.396827, 0.396421, 0.391855, 0.386110, 0.378586, 0.370568, 0.362880, 0.355686, 0.349007, 0.342933,\
        0.337543, 0.332819, 0.328645, 0.324853, 0.321277, 0.317801, 0.314423, 0.311169, 0.307907, 0.304590,\
        0.301602, 0.299435, 0.297301, 0.294820, 0.292487, 0.290234, 0.287628, 0.284723, 0.281791, 0.278945,\
        0.276207, 0.273582, 0.271073, 0.268678, 0.266387, 0.264181, 0.262041, 0.259948, 0.257886, 0.255839,\
        0.253795, 0.251738, 0.249655, 0.247531, 0.245353, 0.243109, 0.240791, 0.238399, 0.235933, 0.233396,\
        0.230790, 0.228110, 0.225331, 0.222399, 0.219218, 0.215684, 0.211743, 0.207543, 0.203363, 0.198461,\
        0.193033],\
        [0.197176, 0.205102, 0.212836, 0.220011, 0.226468, 0.232089, 0.236824, 0.240698, 0.243795, 0.246235,\
        0.248154, 0.249674, 0.250902, 0.251914, 0.252763, 0.253480, 0.254081, 0.254573, 0.254961, 0.255254,\
        0.255468, 0.255627, 0.255761, 0.255904, 0.256092, 0.256361, 0.256748, 0.257288, 0.258010, 0.258940,\
        0.260096, 0.261494, 0.263144, 0.265053, 0.267217, 0.269609, 0.272174, 0.274824, 0.277447, 0.279937,\
        0.282229, 0.284323, 0.286242, 0.287992, 0.289579, 0.291067, 0.292503, 0.294061, 0.295505, 0.296834,\
        0.299915, 0.306022, 0.313490, 0.322489, 0.332594, 0.343277, 0.353999, 0.364182, 0.373189, 0.380292,\
        0.384561, 0.384570, 0.380856, 0.375246, 0.368335, 0.360805, 0.353299, 0.346165, 0.339597, 0.333744,\
        0.328666, 0.324271, 0.320379, 0.316813, 0.313408, 0.310026, 0.306707, 0.303743, 0.300781, 0.297663,\
        0.294881, 0.292720, 0.290737, 0.288598, 0.286441, 0.284249, 0.281879, 0.279321, 0.276687, 0.274074,\
        0.271529, 0.269069, 0.266701, 0.264424, 0.262228, 0.260102, 0.258027, 0.255988, 0.253967, 0.251949,\
        0.249918, 0.247858, 0.245753, 0.243586, 0.241344, 0.239015, 0.236594, 0.234082, 0.231488, 0.228823,\
        0.226100, 0.223327, 0.220502, 0.217572, 0.214415, 0.210865, 0.206773, 0.202111, 0.197823, 0.192996,\
        0.187450],\
        [0.192586, 0.200239, 0.207589, 0.214527, 0.220875, 0.226447, 0.231169, 0.235055, 0.238180, 0.240657,\
        0.242615, 0.244177, 0.245444, 0.246494, 0.247378, 0.248128, 0.248762, 0.249293, 0.249729, 0.250081,\
        0.250367, 0.250608, 0.250829, 0.251061, 0.251336, 0.251687, 0.252147, 0.252746, 0.253511, 0.254466,\
        0.255629, 0.257013, 0.258625, 0.260466, 0.262523, 0.264767, 0.267146, 0.269591, 0.272017, 0.274349,\
        0.276536, 0.278560, 0.280420, 0.282121, 0.283679, 0.285133, 0.286536, 0.287942, 0.289377, 0.291222,\
        0.294595, 0.299679, 0.306366, 0.314870, 0.324403, 0.334424, 0.344390, 0.353746, 0.361892, 0.368141,\
        0.371695, 0.371910, 0.369050, 0.364063, 0.357743, 0.350729, 0.343556, 0.336607, 0.330179, 0.324552,\
        0.319853, 0.315876, 0.312323, 0.309036, 0.305871, 0.302660, 0.299335, 0.296526, 0.293815, 0.291040,\
        0.288496, 0.286329, 0.284362, 0.282397, 0.280397, 0.278334, 0.276159, 0.273863, 0.271496, 0.269120,\
        0.266777, 0.264491, 0.262271, 0.260118, 0.258026, 0.255984, 0.253977, 0.251993, 0.250013, 0.248023,\
        0.246005, 0.243942, 0.241815, 0.239606, 0.237299, 0.234882, 0.232348, 0.229703, 0.226962, 0.224148,\
        0.221279, 0.218378, 0.215478, 0.212569, 0.209481, 0.205996, 0.201898, 0.197160, 0.192250, 0.187175,\
        0.181573],\
        [0.188642, 0.196027, 0.203078, 0.209889, 0.216117, 0.221565, 0.226175, 0.229959, 0.232993, 0.235397,\
        0.237306, 0.238846, 0.240114, 0.241184, 0.242099, 0.242889, 0.243570, 0.244154, 0.244650, 0.245072,\
        0.245435, 0.245759, 0.246067, 0.246385, 0.246742, 0.247166, 0.247687, 0.248331, 0.249124, 0.250086,\
        0.251234, 0.252578, 0.254123, 0.255865, 0.257790, 0.259869, 0.262059, 0.264303, 0.266539, 0.268710,\
        0.270774, 0.272707, 0.274502, 0.276161, 0.277700, 0.279153, 0.280571, 0.282030, 0.283684, 0.285899,\
        0.289185, 0.293761, 0.299996, 0.307803, 0.316617, 0.325872, 0.334981, 0.343423, 0.350667, 0.356114,\
        0.359162, 0.359448, 0.357116, 0.352743, 0.347009, 0.340526, 0.333773, 0.327095, 0.320802, 0.315344,\
        0.311122, 0.307679, 0.304444, 0.301411, 0.298480, 0.295469, 0.292416, 0.289623, 0.287034, 0.284530,\
        0.282196, 0.280097, 0.278164, 0.276289, 0.274407, 0.272478, 0.270474, 0.268390, 0.266252, 0.264093,\
        0.261948, 0.259835, 0.257765, 0.255739, 0.253753, 0.251798, 0.249862, 0.247932, 0.245994, 0.244032,\
        0.242028, 0.239962, 0.237817, 0.235570, 0.233202, 0.230697, 0.228046, 0.225255, 0.222352, 0.219370,\
        0.216334, 0.213259, 0.210198, 0.207307, 0.204282, 0.200772, 0.196633, 0.191798, 0.186637, 0.181228,\
        0.175458],\
        [0.185238, 0.192516, 0.199471, 0.206155, 0.212167, 0.217407, 0.221815, 0.225387, 0.228210, 0.230431,\
        0.232203, 0.233659, 0.234894, 0.235968, 0.236914, 0.237750, 0.238486, 0.239131, 0.239696, 0.240192,\
        0.240636, 0.241043, 0.241436, 0.241835, 0.242266, 0.242754, 0.243323, 0.244000, 0.244805, 0.245757,\
        0.246871, 0.248155, 0.249612, 0.251238, 0.253017, 0.254926, 0.256928, 0.258980, 0.261035, 0.263050,\
        0.264990, 0.266833, 0.268570, 0.270204, 0.271753, 0.273252, 0.274762, 0.276383, 0.278290, 0.280756,\
        0.284092, 0.288546, 0.294306, 0.301315, 0.309342, 0.317779, 0.325933, 0.333426, 0.339834, 0.344631,\
        0.347313, 0.347594, 0.345555, 0.341616, 0.336371, 0.330406, 0.324157, 0.317882, 0.311755, 0.306245,\
        0.302541, 0.299746, 0.296725, 0.293887, 0.291121, 0.288333, 0.285561, 0.282928, 0.280463, 0.278135,\
        0.275960, 0.273951, 0.272076, 0.270272, 0.268484, 0.266674, 0.264818, 0.262912, 0.260967, 0.259002,\
        0.257039, 0.255091, 0.253168, 0.251269, 0.249389, 0.247520, 0.245654, 0.243777, 0.241879, 0.239944,\
        0.237955, 0.235891, 0.233730, 0.231452, 0.229033, 0.226449, 0.223684, 0.220738, 0.217655, 0.214498,\
        0.211298, 0.208054, 0.204705, 0.201682, 0.198779, 0.195039, 0.190767, 0.185914, 0.180640, 0.175038,\
        0.169102],\
        [0.182317, 0.189715, 0.196746, 0.203187, 0.208876, 0.213899, 0.218043, 0.221289, 0.223774, 0.225698,\
        0.227250, 0.228573, 0.229752, 0.230825, 0.231804, 0.232691, 0.233489, 0.234201, 0.234838, 0.235411,\
        0.235935, 0.236425, 0.236897, 0.237372, 0.237869, 0.238410, 0.239018, 0.239714, 0.240517, 0.241445,\
        0.242511, 0.243720, 0.245077, 0.246575, 0.248204, 0.249942, 0.251763, 0.253634, 0.255521, 0.257390,\
        0.259216, 0.260980, 0.262676, 0.264308, 0.265897, 0.267483, 0.269133, 0.270954, 0.273102, 0.275785,\
        0.279226, 0.283607, 0.289024, 0.295418, 0.302746, 0.310249, 0.317303, 0.323865, 0.329575, 0.333910,\
        0.336349, 0.336591, 0.334672, 0.330959, 0.326037, 0.320536, 0.314875, 0.309204, 0.303597, 0.298144,\
        0.294499, 0.291874, 0.289185, 0.286532, 0.283929, 0.281344, 0.278798, 0.276347, 0.274025, 0.271837,\
        0.269785, 0.267866, 0.266058, 0.264321, 0.262615, 0.260907, 0.259179, 0.257423, 0.255641, 0.253845,\
        0.252046, 0.250252, 0.248468, 0.246690, 0.244913, 0.243128, 0.241326, 0.239499, 0.237638, 0.235730,\
        0.233756, 0.231695, 0.229524, 0.227222, 0.224764, 0.222124, 0.219262, 0.216158, 0.212865, 0.209526,\
        0.206190, 0.202808, 0.199499, 0.195969, 0.192764, 0.188927, 0.184526, 0.179612, 0.174261, 0.168539,\
        0.162473],\
        [0.179673, 0.187219, 0.194183, 0.200378, 0.206083, 0.211002, 0.214805, 0.217584, 0.219588, 0.221104,\
        0.222367, 0.223529, 0.224649, 0.225729, 0.226751, 0.227697, 0.228560, 0.229342, 0.230050, 0.230698,\
        0.231300, 0.231869, 0.232418, 0.232962, 0.233516, 0.234101, 0.234736, 0.235440, 0.236231, 0.237124,\
        0.238130, 0.239256, 0.240505, 0.241872, 0.243348, 0.244920, 0.246568, 0.248270, 0.250002, 0.251741,\
        0.253467, 0.255168, 0.256840, 0.258492, 0.260145, 0.261842, 0.263652, 0.265672, 0.268037, 0.270905,\
        0.274437, 0.278764, 0.283952, 0.289986, 0.296722, 0.302908, 0.309009, 0.314735, 0.319968, 0.324064,\
        0.326392, 0.326592, 0.324665, 0.320978, 0.316166, 0.311014, 0.305995, 0.300979, 0.296001, 0.291249,\
        0.287467, 0.284565, 0.281952, 0.279428, 0.276960, 0.274534, 0.272161, 0.269869, 0.267682, 0.265613,\
        0.263663, 0.261825, 0.260083, 0.258410, 0.256777, 0.255159, 0.253539, 0.251908, 0.250265, 0.248614,\
        0.246960, 0.245307, 0.243652, 0.241989, 0.240309, 0.238601, 0.236856, 0.235071, 0.233240, 0.231354,\
        0.229396, 0.227340, 0.225160, 0.222836, 0.220353, 0.217686, 0.214776, 0.211541, 0.207981, 0.204417,\
        0.201020, 0.197473, 0.193946, 0.190322, 0.186608, 0.182543, 0.178002, 0.172992, 0.167555, 0.161735,\
        0.155562],\
        [0.177103, 0.184893, 0.192006, 0.198331, 0.204101, 0.208740, 0.212008, 0.214134, 0.215510, 0.216520,\
        0.217453, 0.218459, 0.219550, 0.220664, 0.221743, 0.222754, 0.223683, 0.224530, 0.225305, 0.226023,\
        0.226699, 0.227344, 0.227965, 0.228571, 0.229176, 0.229796, 0.230448, 0.231151, 0.231920, 0.232769,\
        0.233709, 0.234747, 0.235884, 0.237119, 0.238447, 0.239859, 0.241344, 0.242889, 0.244480, 0.246103,\
        0.247746, 0.249400, 0.251066, 0.252753, 0.254484, 0.256302, 0.258268, 0.260469, 0.263013, 0.266020,\
        0.269604, 0.273848, 0.278777, 0.284306, 0.290100, 0.295704, 0.301071, 0.305997, 0.311102, 0.315140,\
        0.317478, 0.317648, 0.315630, 0.311848, 0.306975, 0.301951, 0.297561, 0.293099, 0.288649, 0.284449,\
        0.280813, 0.277778, 0.275108, 0.272618, 0.270229, 0.267911, 0.265661, 0.263492, 0.261417, 0.259446,\
        0.257579, 0.255812, 0.254131, 0.252516, 0.250948, 0.249407, 0.247877, 0.246351, 0.244824, 0.243297,\
        0.241770, 0.240242, 0.238706, 0.237151, 0.235561, 0.233921, 0.232222, 0.230464, 0.228651, 0.226782,\
        0.224840, 0.222790, 0.220596, 0.218241, 0.215730, 0.213059, 0.210178, 0.206941, 0.203116, 0.199187,\
        0.195650, 0.191993, 0.188233, 0.184363, 0.180318, 0.175979, 0.171252, 0.166112, 0.160569, 0.154649,\
        0.148377],\
        [0.174381, 0.182548, 0.190118, 0.196960, 0.202698, 0.206858, 0.209392, 0.210705, 0.211349, 0.211805,\
        0.212410, 0.213318, 0.214444, 0.215627, 0.216774, 0.217850, 0.218841, 0.219743, 0.220572, 0.221351,\
        0.222098, 0.222818, 0.223507, 0.224170, 0.224817, 0.225462, 0.226123, 0.226817, 0.227558, 0.228359,\
        0.229230, 0.230177, 0.231205, 0.232312, 0.233496, 0.234755, 0.236085, 0.237484, 0.238948, 0.240470,\
        0.242045, 0.243669, 0.245341, 0.247073, 0.248887, 0.250821, 0.252928, 0.255280, 0.257960, 0.261056,\
        0.264642, 0.268762, 0.273395, 0.278425, 0.283609, 0.288692, 0.293420, 0.298213, 0.303077, 0.307016,\
        0.309486, 0.309617, 0.307459, 0.303612, 0.298848, 0.294050, 0.289793, 0.285682, 0.281629, 0.277789,\
        0.274325, 0.271280, 0.268566, 0.266069, 0.263713, 0.261458, 0.259289, 0.257207, 0.255217, 0.253321,\
        0.251521, 0.249811, 0.248181, 0.246617, 0.245104, 0.243627, 0.242172, 0.240731, 0.239300, 0.237875,\
        0.236457, 0.235040, 0.233614, 0.232161, 0.230656, 0.229075, 0.227405, 0.225652, 0.223837, 0.221975,\
        0.220052, 0.218013, 0.215797, 0.213382, 0.210807, 0.208110, 0.205274, 0.202165, 0.198414, 0.194171,\
        0.190217, 0.186297, 0.182290, 0.178151, 0.173822, 0.169224, 0.164294, 0.158998, 0.153330, 0.147301,\
        0.140926],\
        [0.171203, 0.179735, 0.187721, 0.194959, 0.200928, 0.204709, 0.206501, 0.206972, 0.206913, 0.206865,\
        0.207183, 0.208117, 0.209368, 0.210638, 0.211840, 0.212968, 0.214015, 0.214957, 0.215817, 0.216646,\
        0.217464, 0.218259, 0.219015, 0.219729, 0.220408, 0.221070, 0.221734, 0.222413, 0.223121, 0.223871,\
        0.224673, 0.225533, 0.226455, 0.227440, 0.228489, 0.229601, 0.230785, 0.232047, 0.233396, 0.234831,\
        0.236353, 0.237957, 0.239645, 0.241427, 0.243321, 0.245359, 0.247584, 0.250052, 0.252826, 0.255965,\
        0.259515, 0.263486, 0.267843, 0.272478, 0.277230, 0.281932, 0.286517, 0.291111, 0.295508, 0.299323,\
        0.302013, 0.301942, 0.299689, 0.295928, 0.291480, 0.286966, 0.282734, 0.278738, 0.274906, 0.271287,\
        0.267958, 0.264947, 0.262219, 0.259710, 0.257363, 0.255140, 0.253020, 0.250995, 0.249062, 0.247222,\
        0.245471, 0.243805, 0.242216, 0.240693, 0.239224, 0.237798, 0.236403, 0.235031, 0.233674, 0.232332,\
        0.231002, 0.229681, 0.228355, 0.227000, 0.225579, 0.224051, 0.222392, 0.220612, 0.218761, 0.216886,\
        0.214988, 0.212982, 0.210740, 0.208222, 0.205525, 0.202736, 0.199831, 0.196661, 0.192924, 0.188753,\
        0.184543, 0.180343, 0.176075, 0.171676, 0.167088, 0.162252, 0.157118, 0.151655, 0.145850, 0.139702,\
        0.133220],\
        [0.167280, 0.176026, 0.184202, 0.191572, 0.197603, 0.201418, 0.202763, 0.202619, 0.202117, 0.201801,\
        0.201843, 0.203015, 0.204397, 0.205710, 0.206915, 0.208077, 0.209185, 0.210144, 0.210998, 0.211865,\
        0.212765, 0.213639, 0.214459, 0.215215, 0.215915, 0.216587, 0.217249, 0.217910, 0.218583, 0.219284,\
        0.220020, 0.220798, 0.221623, 0.222496, 0.223418, 0.224392, 0.225433, 0.226566, 0.227809, 0.229171,\
        0.230651, 0.232246, 0.233957, 0.235789, 0.237755, 0.239880, 0.242197, 0.244748, 0.247577, 0.250722,\
        0.254206, 0.258027, 0.262141, 0.266467, 0.270894, 0.275323, 0.279700, 0.283991, 0.288027, 0.291505,\
        0.293690, 0.293622, 0.291614, 0.288245, 0.284219, 0.280025, 0.275947, 0.272062, 0.268372, 0.264896,\
        0.261668, 0.258701, 0.255980, 0.253466, 0.251121, 0.248912, 0.246819, 0.244828, 0.242932, 0.241128,\
        0.239411, 0.237776, 0.236216, 0.234723, 0.233287, 0.231901, 0.230552, 0.229231, 0.227930, 0.226647,\
        0.225384, 0.224141, 0.222904, 0.221646, 0.220314, 0.218844, 0.217181, 0.215331, 0.213388, 0.211465,\
        0.209590, 0.207663, 0.205415, 0.202750, 0.199891, 0.196979, 0.193925, 0.190590, 0.186835, 0.182718,\
        0.178420, 0.174030, 0.169540, 0.164908, 0.160089, 0.155035, 0.149707, 0.144077, 0.138129, 0.131857,\
        0.125263],\
        [0.162434, 0.171207, 0.179321, 0.186502, 0.192295, 0.196130, 0.197630, 0.197589, 0.197136, 0.196871,\
        0.197035, 0.198160, 0.199487, 0.200797, 0.201927, 0.203120, 0.204337, 0.205269, 0.206053, 0.206965,\
        0.207980, 0.208926, 0.209808, 0.210599, 0.211298, 0.211973, 0.212637, 0.213280, 0.213916, 0.214571,\
        0.215251, 0.215954, 0.216691, 0.217469, 0.218279, 0.219120, 0.220019, 0.221026, 0.222175, 0.223475,\
        0.224925, 0.226520, 0.228256, 0.230134, 0.232161, 0.234354, 0.236736, 0.239338, 0.242188, 0.245310,\
        0.248711, 0.252380, 0.256279, 0.260342, 0.264490, 0.268643, 0.272731, 0.276655, 0.280230, 0.283119,\
        0.284781, 0.284761, 0.283131, 0.280300, 0.276776, 0.272970, 0.269147, 0.265435, 0.261887, 0.258533,\
        0.255394, 0.252478, 0.249778, 0.247270, 0.244928, 0.242727, 0.240647, 0.238675, 0.236801, 0.235018,\
        0.233322, 0.231705, 0.230162, 0.228687, 0.227274, 0.225915, 0.224600, 0.223316, 0.222051, 0.220803,\
        0.219582, 0.218395, 0.217233, 0.216067, 0.214837, 0.213445, 0.211782, 0.209812, 0.207704, 0.205688,\
        0.203787, 0.201951, 0.199738, 0.196942, 0.193962, 0.190882, 0.187644, 0.184147, 0.180323, 0.176191,\
        0.171834, 0.167315, 0.162650, 0.157823, 0.152802, 0.147553, 0.142044, 0.136253, 0.130162, 0.123764,\
        0.117056],\
        [0.156629, 0.165288, 0.173193, 0.180077, 0.185583, 0.189349, 0.191280, 0.191867, 0.191888, 0.191924,\
        0.192329, 0.193260, 0.194465, 0.195700, 0.196847, 0.198092, 0.199366, 0.200258, 0.200933, 0.201952,\
        0.203115, 0.204065, 0.205022, 0.205847, 0.206498, 0.207188, 0.207871, 0.208491, 0.209084, 0.209706,\
        0.210348, 0.210979, 0.211638, 0.212345, 0.213070, 0.213783, 0.214530, 0.215413, 0.216483, 0.217735,\
        0.219164, 0.220764, 0.222527, 0.224444, 0.226518, 0.228758, 0.231180, 0.233803, 0.236646, 0.239719,\
        0.243023, 0.246539, 0.250234, 0.254052, 0.257926, 0.261781, 0.265527, 0.269040, 0.272125, 0.274491,\
        0.275790, 0.275775, 0.274469, 0.272125, 0.269098, 0.265710, 0.262200, 0.258716, 0.255340, 0.252118,\
        0.249074, 0.246220, 0.243554, 0.241064, 0.238731, 0.236537, 0.234465, 0.232503, 0.230640, 0.228868,\
        0.227181, 0.225572, 0.224036, 0.222568, 0.221165, 0.219822, 0.218532, 0.217276, 0.216031, 0.214791,\
        0.213580, 0.212422, 0.211309, 0.210218, 0.209095, 0.207812, 0.206194, 0.204090, 0.201732, 0.199647,\
        0.197642, 0.195697, 0.193422, 0.190653, 0.187611, 0.184397, 0.180997, 0.177359, 0.173442, 0.169252,\
        0.164826, 0.160202, 0.155396, 0.150406, 0.145211, 0.139789, 0.134116, 0.128172, 0.121942, 0.115419,\
        0.108597],\
        [0.149926, 0.158404, 0.166065, 0.172684, 0.178009, 0.181846, 0.184217, 0.185454, 0.186074, 0.186546,\
        0.187182, 0.188100, 0.189222, 0.190414, 0.191613, 0.192848, 0.194022, 0.194975, 0.195804, 0.196887,\
        0.198040, 0.198991, 0.200035, 0.200914, 0.201424, 0.202211, 0.202917, 0.203510, 0.204044, 0.204660,\
        0.205300, 0.205843, 0.206431, 0.207106, 0.207793, 0.208384, 0.208948, 0.209716, 0.210739, 0.211952,\
        0.213363, 0.214973, 0.216759, 0.218705, 0.220809, 0.223075, 0.225511, 0.228129, 0.230939, 0.233943,\
        0.237135, 0.240494, 0.243987, 0.247563, 0.251161, 0.254702, 0.258089, 0.261192, 0.263833, 0.265788,\
        0.266830, 0.266814, 0.265753, 0.263807, 0.261219, 0.258235, 0.255059, 0.251835, 0.248658, 0.245586,\
        0.242652, 0.239875, 0.237259, 0.234798, 0.232483, 0.230299, 0.228235, 0.226279, 0.224421, 0.222653,\
        0.220968, 0.219359, 0.217821, 0.216350, 0.214945, 0.213605, 0.212333, 0.211103, 0.209866, 0.208605,\
        0.207371, 0.206219, 0.205121, 0.204059, 0.203011, 0.201815, 0.200264, 0.198132, 0.195676, 0.193421,\
        0.191276, 0.189092, 0.186664, 0.183884, 0.180807, 0.177502, 0.173979, 0.170221, 0.166207, 0.161939,\
        0.157433, 0.152711, 0.147784, 0.142654, 0.137309, 0.131733, 0.125911, 0.119825, 0.113463, 0.106818,\
        0.099883],\
        [0.142436, 0.150723, 0.158172, 0.164607, 0.169867, 0.173858, 0.176639, 0.178451, 0.179647, 0.180576,\
        0.181493, 0.182519, 0.183658, 0.184859, 0.186081, 0.187298, 0.188450, 0.189490, 0.190488, 0.191568,\
        0.192668, 0.193708, 0.194731, 0.195573, 0.196208, 0.196985, 0.197686, 0.198278, 0.198783, 0.199418,\
        0.200063, 0.200512, 0.201043, 0.201725, 0.202427, 0.202925, 0.203246, 0.203951, 0.204980, 0.206132,\
        0.207526, 0.209147, 0.210948, 0.212909, 0.215024, 0.217292, 0.219717, 0.222306, 0.225060, 0.227976,\
        0.231043, 0.234239, 0.237528, 0.240863, 0.244183, 0.247410, 0.250449, 0.253175, 0.255440, 0.257076,\
        0.257930, 0.257914, 0.257038, 0.255407, 0.253190, 0.250572, 0.247720, 0.244765, 0.241802, 0.238897,\
        0.236089, 0.233404, 0.230851, 0.228432, 0.226144, 0.223978, 0.221924, 0.219974, 0.218119, 0.216351,\
        0.214662, 0.213048, 0.211502, 0.210020, 0.208601, 0.207249, 0.205979, 0.204785, 0.203556, 0.202255,\
        0.200966, 0.199814, 0.198702, 0.197615, 0.196538, 0.195297, 0.193696, 0.191631, 0.189279, 0.186920,\
        0.184590, 0.182192, 0.179601, 0.176737, 0.173600, 0.170215, 0.166593, 0.162732, 0.158625, 0.154271,\
        0.149680, 0.144862, 0.139826, 0.134572, 0.129094, 0.123382, 0.117423, 0.111205, 0.104719, 0.097956,\
        0.090911],\
        [0.134270, 0.142391, 0.149681, 0.156015, 0.161291, 0.165473, 0.168628, 0.170934, 0.172642, 0.174009,\
        0.175238, 0.176450, 0.177688, 0.178946, 0.180206, 0.181443, 0.182628, 0.183748, 0.184840, 0.185942,\
        0.187038, 0.188095, 0.189086, 0.189949, 0.190707, 0.191452, 0.192146, 0.192758, 0.193323, 0.193937,\
        0.194516, 0.194998, 0.195524, 0.196168, 0.196815, 0.197276, 0.197540, 0.198235, 0.199232, 0.200284,\
        0.201679, 0.203299, 0.205095, 0.207048, 0.209149, 0.211396, 0.213788, 0.216325, 0.219004, 0.221817,\
        0.224748, 0.227773, 0.230857, 0.233953, 0.237001, 0.239928, 0.242643, 0.245038, 0.246990, 0.248376,\
        0.249090, 0.249075, 0.248339, 0.246956, 0.245046, 0.242748, 0.240195, 0.237501, 0.234757, 0.232028,\
        0.229357, 0.226776, 0.224299, 0.221934, 0.219682, 0.217540, 0.215501, 0.213559, 0.211706, 0.209937,\
        0.208244, 0.206621, 0.205064, 0.203568, 0.202131, 0.200756, 0.199461, 0.198280, 0.197047, 0.195734,\
        0.194437, 0.193230, 0.192073, 0.190921, 0.189723, 0.188356, 0.186687, 0.184670, 0.182396, 0.180002,\
        0.177541, 0.174974, 0.172232, 0.169259, 0.166037, 0.162567, 0.158855, 0.154901, 0.150703, 0.146262,\
        0.141583, 0.136672, 0.131532, 0.126166, 0.120568, 0.114732, 0.108649, 0.102309, 0.095706, 0.088830,\
        0.081679]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=40 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax40vt10
    zgrid_H_Hmax_vmax40vt10=[[0.317227, 0.324049, 0.330232, 0.335782, 0.340713, 0.345047, 0.348799, 0.351984, 0.354618, 0.356719,\
        0.358313, 0.359442, 0.360164, 0.360566, 0.360740, 0.360776, 0.360737, 0.360668, 0.360600, 0.360559,\
        0.360582, 0.360725, 0.361058, 0.361660, 0.362612, 0.363989, 0.365847, 0.368214, 0.371085, 0.374416,\
        0.378124, 0.382089, 0.386167, 0.390202, 0.394014, 0.397306, 0.399546, 0.400761, 0.401370, 0.401565,\
        0.401462, 0.401140, 0.400663, 0.400076, 0.399416, 0.398711, 0.397836, 0.396871, 0.395877, 0.394840,\
        0.393893, 0.393173, 0.392797, 0.392872, 0.393520, 0.395005, 0.397642, 0.401692, 0.406988, 0.413124,\
        0.419864, 0.427036, 0.434504, 0.442148, 0.449870, 0.457599, 0.465309, 0.473014, 0.480714, 0.488275,\
        0.494985, 0.500556, 0.505989, 0.511028, 0.515598, 0.519664, 0.523210, 0.526238, 0.528759, 0.530795,\
        0.532372, 0.533522, 0.534283, 0.534695, 0.534802, 0.534645, 0.534264, 0.533693, 0.532961, 0.532090,\
        0.531095, 0.529986, 0.528765, 0.527428, 0.525967, 0.524367, 0.522611, 0.520678, 0.518545, 0.516193,\
        0.513602, 0.510758, 0.507652, 0.504282, 0.500640, 0.496520, 0.492820, 0.488647, 0.484131, 0.479385,\
        0.474451, 0.469243, 0.463633, 0.457539, 0.450939, 0.443840, 0.436267, 0.428249, 0.419810, 0.410971,\
        0.401750],\
        [0.325163, 0.332038, 0.338245, 0.343796, 0.348716, 0.353035, 0.356773, 0.359943, 0.362553, 0.364614,\
        0.366142, 0.367163, 0.367731, 0.367943, 0.367921, 0.367767, 0.367544, 0.367285, 0.367005, 0.366710,\
        0.366425, 0.366211, 0.366153, 0.366346, 0.366893, 0.367891, 0.369416, 0.371513, 0.374183, 0.377384,\
        0.381024, 0.384957, 0.389003, 0.392987, 0.396761, 0.400034, 0.402240, 0.403409, 0.403961, 0.404087,\
        0.403907, 0.403506, 0.402952, 0.402299, 0.401591, 0.400853, 0.399906, 0.399007, 0.398063, 0.396977,\
        0.396029, 0.395358, 0.395077, 0.395294, 0.396028, 0.397483, 0.400554, 0.404960, 0.410646, 0.417164,\
        0.424264, 0.431775, 0.439559, 0.447492, 0.455457, 0.463359, 0.471173, 0.478965, 0.486821, 0.494747,\
        0.502306, 0.508151, 0.513719, 0.518923, 0.523629, 0.527784, 0.531373, 0.534401, 0.536886, 0.538854,\
        0.540338, 0.541373, 0.542003, 0.542274, 0.542236, 0.541939, 0.541427, 0.540740, 0.539908, 0.538956,\
        0.537899, 0.536746, 0.535499, 0.534151, 0.532692, 0.531104, 0.529364, 0.527445, 0.525320, 0.522961,\
        0.520340, 0.517435, 0.514223, 0.510677, 0.506729, 0.502642, 0.499296, 0.495034, 0.490564, 0.485857,\
        0.481128, 0.476126, 0.470640, 0.464586, 0.457966, 0.450815, 0.443178, 0.435093, 0.426590, 0.417691,\
        0.408412],\
        [0.332786, 0.339724, 0.345957, 0.351505, 0.356410, 0.360721, 0.364473, 0.367672, 0.370311, 0.372396,\
        0.373938, 0.374928, 0.375392, 0.375450, 0.375274, 0.374990, 0.374642, 0.374246, 0.373804, 0.373282,\
        0.372684, 0.372089, 0.371601, 0.371330, 0.371402, 0.371947, 0.373067, 0.374821, 0.377221, 0.380231,\
        0.383752, 0.387609, 0.391555, 0.395333, 0.398813, 0.401874, 0.404067, 0.405354, 0.405962, 0.406088,\
        0.405868, 0.405405, 0.404778, 0.404049, 0.403260, 0.402414, 0.401495, 0.400667, 0.399696, 0.398623,\
        0.397797, 0.397281, 0.397196, 0.397738, 0.398964, 0.400391, 0.403859, 0.408832, 0.414858, 0.421678,\
        0.429064, 0.436846, 0.444895, 0.453088, 0.461284, 0.469325, 0.477137, 0.484852, 0.492688, 0.500607,\
        0.508427, 0.515298, 0.521361, 0.526837, 0.531712, 0.535960, 0.539580, 0.542590, 0.545020, 0.546902,\
        0.548275, 0.549176, 0.549654, 0.549764, 0.549563, 0.549109, 0.548455, 0.547643, 0.546706, 0.545669,\
        0.544549, 0.543353, 0.542082, 0.540729, 0.539280, 0.537713, 0.536000, 0.534110, 0.532006, 0.529653,\
        0.527018, 0.524071, 0.520787, 0.517155, 0.513231, 0.509304, 0.505376, 0.501290, 0.496584, 0.491971,\
        0.487636, 0.482873, 0.477480, 0.471410, 0.464719, 0.457481, 0.449756, 0.441593, 0.433020, 0.424059,\
        0.414722],\
        [0.340065, 0.347077, 0.353332, 0.358857, 0.363722, 0.368018, 0.371810, 0.375090, 0.377815, 0.379997,\
        0.381669, 0.382757, 0.383194, 0.383138, 0.382873, 0.382554, 0.382156, 0.381675, 0.381149, 0.380444,\
        0.379515, 0.378516, 0.377568, 0.376777, 0.376301, 0.376315, 0.376954, 0.378290, 0.380338, 0.383079,\
        0.386429, 0.390194, 0.394060, 0.397604, 0.400730, 0.403513, 0.405612, 0.406932, 0.407579, 0.407706,\
        0.407446, 0.406912, 0.406195, 0.405366, 0.404475, 0.403544, 0.402593, 0.401641, 0.400618, 0.399749,\
        0.399116, 0.398874, 0.399033, 0.399965, 0.401746, 0.404185, 0.407993, 0.413270, 0.419595, 0.426668,\
        0.434271, 0.442248, 0.450495, 0.458920, 0.467383, 0.475631, 0.483420, 0.490927, 0.498799, 0.506995,\
        0.515034, 0.522428, 0.529006, 0.534812, 0.539873, 0.544204, 0.547832, 0.550799, 0.553152, 0.554931,\
        0.556175, 0.556926, 0.557234, 0.557162, 0.556779, 0.556155, 0.555348, 0.554405, 0.553360, 0.552238,\
        0.551054, 0.549818, 0.548528, 0.547176, 0.545746, 0.544213, 0.542543, 0.540698, 0.538635, 0.536312,\
        0.533690, 0.530738, 0.527439, 0.523806, 0.519912, 0.515870, 0.511680, 0.507203, 0.502217, 0.498209,\
        0.494163, 0.489541, 0.484144, 0.477961, 0.471144, 0.463786, 0.455959, 0.447715, 0.439078, 0.430059,\
        0.420668],\
        [0.346969, 0.354072, 0.360342, 0.365804, 0.370567, 0.374802, 0.378650, 0.382089, 0.384961, 0.387287,\
        0.389244, 0.390663, 0.391185, 0.391075, 0.390822, 0.390633, 0.390275, 0.389687, 0.389223, 0.388386,\
        0.387059, 0.385663, 0.384256, 0.382886, 0.381782, 0.381188, 0.381281, 0.382126, 0.383725, 0.386086,\
        0.389185, 0.392829, 0.396639, 0.400178, 0.402919, 0.405341, 0.407221, 0.408447, 0.409049, 0.409121,\
        0.408780, 0.408135, 0.407289, 0.406326, 0.405309, 0.404276, 0.403239, 0.402193, 0.401163, 0.400296,\
        0.399843, 0.400016, 0.400579, 0.402017, 0.404458, 0.407862, 0.412410, 0.418132, 0.424788, 0.432106,\
        0.439875, 0.447959, 0.456294, 0.464871, 0.473646, 0.482330, 0.490292, 0.497571, 0.505471, 0.513947,\
        0.522226, 0.529941, 0.536874, 0.542951, 0.548162, 0.552535, 0.556127, 0.559015, 0.561264, 0.562922,\
        0.564027, 0.564615, 0.564735, 0.564460, 0.563878, 0.563073, 0.562110, 0.561034, 0.559882, 0.558675,\
        0.557431, 0.556156, 0.554852, 0.553508, 0.552107, 0.550620, 0.549010, 0.547231, 0.545233, 0.542967,\
        0.540388, 0.537465, 0.534185, 0.530566, 0.526657, 0.522514, 0.518149, 0.513548, 0.508921, 0.504803,\
        0.500589, 0.496100, 0.490477, 0.484113, 0.477154, 0.469660, 0.461735, 0.453427, 0.444741, 0.435678,\
        0.426239],\
        [0.353459, 0.360687, 0.366976, 0.372320, 0.376864, 0.380907, 0.384775, 0.388516, 0.391641, 0.394071,\
        0.396444, 0.398566, 0.399364, 0.399435, 0.399353, 0.399485, 0.399279, 0.398378, 0.398242, 0.397231,\
        0.395431, 0.393734, 0.391907, 0.389879, 0.388081, 0.386812, 0.386320, 0.386627, 0.387657, 0.389461,\
        0.392171, 0.395662, 0.399242, 0.402542, 0.405242, 0.407412, 0.409057, 0.410111, 0.410577, 0.410516,\
        0.410021, 0.409201, 0.408163, 0.407011, 0.405827, 0.404665, 0.403538, 0.402432, 0.401354, 0.400416,\
        0.399986, 0.400480, 0.401718, 0.403880, 0.407142, 0.411500, 0.416918, 0.423285, 0.430385, 0.437983,\
        0.445887, 0.453980, 0.462231, 0.470727, 0.479655, 0.489026, 0.497702, 0.505100, 0.513037, 0.521526,\
        0.529961, 0.537901, 0.545075, 0.551334, 0.556621, 0.560960, 0.564448, 0.567205, 0.569322, 0.570848,\
        0.571811, 0.572231, 0.572149, 0.571649, 0.570852, 0.569862, 0.568745, 0.567544, 0.566288, 0.565002,\
        0.563699, 0.562389, 0.561072, 0.559741, 0.558377, 0.556951, 0.555418, 0.553727, 0.551818, 0.549633,\
        0.547123, 0.544250, 0.541003, 0.537393, 0.533459, 0.529247, 0.524805, 0.520211, 0.515642, 0.511229,\
        0.506797, 0.501968, 0.496038, 0.489768, 0.482656, 0.475024, 0.467037, 0.458707, 0.450004, 0.440912,\
        0.431432],\
        [0.359478, 0.366897, 0.373248, 0.378434, 0.382587, 0.386169, 0.389817, 0.393990, 0.397772, 0.400271,\
        0.403230, 0.405998, 0.407678, 0.408504, 0.408984, 0.409318, 0.409211, 0.408609, 0.408133, 0.406917,\
        0.404978, 0.402989, 0.400682, 0.397987, 0.395515, 0.393493, 0.392434, 0.392201, 0.392536, 0.393549,\
        0.395560, 0.398919, 0.402318, 0.405308, 0.407842, 0.409829, 0.411250, 0.412084, 0.412337, 0.412058,\
        0.411326, 0.410244, 0.408929, 0.407504, 0.406083, 0.404749, 0.403524, 0.402374, 0.401283, 0.400351,\
        0.399950, 0.400664, 0.402476, 0.405456, 0.409706, 0.415143, 0.421570, 0.428744, 0.436410, 0.444343,\
        0.452373, 0.460397, 0.468375, 0.476400, 0.484861, 0.494192, 0.503769, 0.512125, 0.520641, 0.529433,\
        0.538099, 0.546249, 0.553605, 0.559980, 0.565261, 0.569462, 0.572745, 0.575311, 0.577275, 0.578664,\
        0.579493, 0.579760, 0.579463, 0.578711, 0.577687, 0.576523, 0.575272, 0.573958, 0.572608, 0.571247,\
        0.569890, 0.568545, 0.567215, 0.565897, 0.564576, 0.563219, 0.561780, 0.560196, 0.558397, 0.556316,\
        0.553893, 0.551082, 0.547866, 0.544254, 0.540283, 0.536005, 0.531486, 0.526812, 0.522083, 0.517326,\
        0.512409, 0.507052, 0.501407, 0.494909, 0.487508, 0.479796, 0.471848, 0.463567, 0.454881, 0.445771,\
        0.436249],\
        [0.364928, 0.372636, 0.379176, 0.384252, 0.387855, 0.390645, 0.393553, 0.397397, 0.402228, 0.406075,\
        0.409979, 0.413562, 0.416302, 0.418164, 0.419378, 0.420074, 0.420215, 0.419853, 0.419101, 0.417746,\
        0.415780, 0.413430, 0.410657, 0.407553, 0.404478, 0.401726, 0.400138, 0.399297, 0.398842, 0.399040,\
        0.399864, 0.402824, 0.406016, 0.408786, 0.411070, 0.412807, 0.413962, 0.414519, 0.414487, 0.413906,\
        0.412846, 0.411407, 0.409710, 0.407899, 0.406128, 0.404533, 0.403158, 0.401926, 0.400772, 0.399799,\
        0.399495, 0.400475, 0.402809, 0.406693, 0.412143, 0.418880, 0.426504, 0.434635, 0.442968, 0.451280,\
        0.459438, 0.467383, 0.475046, 0.482323, 0.489618, 0.498299, 0.508278, 0.518297, 0.528007, 0.537471,\
        0.546513, 0.554887, 0.562394, 0.568849, 0.574061, 0.577993, 0.580930, 0.583243, 0.585058, 0.586306,\
        0.587011, 0.587183, 0.586660, 0.585611, 0.584367, 0.583074, 0.581728, 0.580318, 0.578883, 0.577455,\
        0.576046, 0.574662, 0.573312, 0.572001, 0.570720, 0.569440, 0.568106, 0.566644, 0.564973, 0.563013,\
        0.560688, 0.557941, 0.554745, 0.551110, 0.547075, 0.542699, 0.538052, 0.533203, 0.528206, 0.523056,\
        0.517670, 0.511953, 0.505854, 0.499036, 0.491522, 0.483981, 0.476241, 0.468073, 0.459413, 0.450273,\
        0.440694],\
        [0.369642, 0.377733, 0.384683, 0.389908, 0.392911, 0.394749, 0.397099, 0.399961, 0.405703, 0.411594,\
        0.416924, 0.421615, 0.425446, 0.428329, 0.430327, 0.431523, 0.431980, 0.431780, 0.430983, 0.429564,\
        0.427532, 0.424961, 0.421910, 0.418528, 0.415097, 0.412001, 0.409689, 0.408049, 0.406916, 0.406412,\
        0.406781, 0.408575, 0.411001, 0.413285, 0.415168, 0.416543, 0.417349, 0.417558, 0.417168, 0.416208,\
        0.414737, 0.412845, 0.410654, 0.408319, 0.406032, 0.404008, 0.402372, 0.400916, 0.399428, 0.398204,\
        0.397988, 0.399383, 0.402470, 0.407535, 0.414554, 0.422905, 0.431939, 0.441152, 0.450210, 0.458912,\
        0.467165, 0.475042, 0.482618, 0.489508, 0.495273, 0.503288, 0.513962, 0.524960, 0.535640, 0.545777,\
        0.555187, 0.563715, 0.571294, 0.577828, 0.582978, 0.586469, 0.588866, 0.590883, 0.592619, 0.593712,\
        0.594223, 0.594485, 0.593684, 0.592299, 0.590889, 0.589567, 0.588185, 0.586682, 0.585168, 0.583687,\
        0.582227, 0.580793, 0.579407, 0.578085, 0.576834, 0.575628, 0.574406, 0.573074, 0.571540, 0.569711,\
        0.567492, 0.564804, 0.561607, 0.557913, 0.553773, 0.549255, 0.544425, 0.539341, 0.534030, 0.528480,\
        0.522628, 0.516386, 0.509631, 0.502275, 0.494912, 0.487886, 0.480433, 0.472341, 0.463646, 0.454431,\
        0.444763],\
        [0.373407, 0.381844, 0.389338, 0.395284, 0.397896, 0.398727, 0.401611, 0.405678, 0.411440, 0.418084,\
        0.424494, 0.430255, 0.435128, 0.438965, 0.441720, 0.443434, 0.444209, 0.444177, 0.443447, 0.442059,\
        0.440026, 0.437373, 0.434181, 0.430623, 0.426972, 0.423554, 0.420638, 0.418289, 0.416513, 0.415422,\
        0.415217, 0.416007, 0.417429, 0.418962, 0.420267, 0.421158, 0.421527, 0.421316, 0.420502, 0.419096,\
        0.417144, 0.414723, 0.411946, 0.408959, 0.405962, 0.403239, 0.401096, 0.399266, 0.397250, 0.395772,\
        0.395541, 0.397230, 0.401268, 0.408148, 0.417261, 0.427572, 0.438198, 0.448537, 0.458298, 0.467353,\
        0.475579, 0.483190, 0.490846, 0.498223, 0.504026, 0.511374, 0.521659, 0.532851, 0.543992, 0.554556,\
        0.564170, 0.572646, 0.580058, 0.586605, 0.591912, 0.594747, 0.596454, 0.598150, 0.599885, 0.600847,\
        0.601019, 0.601428, 0.600356, 0.598807, 0.597331, 0.596130, 0.594753, 0.593109, 0.591539, 0.590029,\
        0.588509, 0.587006, 0.585555, 0.584191, 0.582945, 0.581805, 0.580693, 0.579489, 0.578083, 0.576388,\
        0.574282, 0.571642, 0.568408, 0.564609, 0.560317, 0.555611, 0.550560, 0.545211, 0.539585, 0.533665,\
        0.527406, 0.520738, 0.513593, 0.506045, 0.498902, 0.492116, 0.484666, 0.476435, 0.467585, 0.458229,\
        0.448436],\
        [0.376088, 0.384636, 0.392293, 0.398635, 0.402066, 0.403763, 0.407211, 0.412302, 0.418599, 0.425660,\
        0.432777, 0.439423, 0.445236, 0.449949, 0.453400, 0.455575, 0.456617, 0.456744, 0.456141, 0.454888,\
        0.452966, 0.450350, 0.447105, 0.443430, 0.439598, 0.435880, 0.432485, 0.429543, 0.427160, 0.425464,\
        0.424575, 0.424493, 0.424989, 0.425714, 0.426353, 0.426682, 0.426551, 0.425866, 0.424578, 0.422680,\
        0.420200, 0.417205, 0.413795, 0.410102, 0.406295, 0.402609, 0.399633, 0.397235, 0.394908, 0.393203,\
        0.392928, 0.394964, 0.400255, 0.409325, 0.420874, 0.433396, 0.445692, 0.457043, 0.467310, 0.476666,\
        0.484821, 0.491572, 0.498812, 0.506456, 0.513366, 0.521152, 0.530859, 0.541797, 0.553051, 0.563844,\
        0.573501, 0.581657, 0.588450, 0.594529, 0.600070, 0.602566, 0.603969, 0.605390, 0.606783, 0.607588,\
        0.607828, 0.607717, 0.606834, 0.605452, 0.604058, 0.602877, 0.601449, 0.599684, 0.598127, 0.596596,\
        0.594982, 0.593385, 0.591832, 0.590381, 0.589097, 0.588001, 0.586995, 0.585889, 0.584569, 0.583002,\
        0.581029, 0.578419, 0.575095, 0.571129, 0.566635, 0.561707, 0.556415, 0.550801, 0.544882, 0.538650,\
        0.532088, 0.525184, 0.517985, 0.510708, 0.503710, 0.496714, 0.488836, 0.480235, 0.471147, 0.461613,\
        0.451675],\
        [0.377784, 0.386255, 0.393752, 0.399977, 0.404852, 0.408825, 0.413451, 0.419295, 0.426172, 0.433709,\
        0.441419, 0.448826, 0.455503, 0.461048, 0.465132, 0.467666, 0.468869, 0.469104, 0.468670, 0.467670,\
        0.466004, 0.463529, 0.460277, 0.456507, 0.452532, 0.448578, 0.444799, 0.441347, 0.438388, 0.436078,\
        0.434510, 0.433659, 0.433357, 0.433338, 0.433323, 0.433077, 0.432422, 0.431238, 0.429450, 0.427033,\
        0.424003, 0.420417, 0.416377, 0.412018, 0.407520, 0.403151, 0.399386, 0.396109, 0.393272, 0.391332,\
        0.391096, 0.393598, 0.400309, 0.411829, 0.426079, 0.440934, 0.454872, 0.466932, 0.477141, 0.486500,\
        0.495218, 0.500707, 0.507380, 0.514907, 0.522518, 0.530809, 0.540382, 0.551102, 0.562389, 0.573421,\
        0.583134, 0.590838, 0.596772, 0.601792, 0.606666, 0.609693, 0.611361, 0.612625, 0.613651, 0.614274,\
        0.614447, 0.614185, 0.613429, 0.612307, 0.611069, 0.609805, 0.608345, 0.606692, 0.605084, 0.603455,\
        0.601739, 0.600028, 0.598341, 0.596750, 0.595362, 0.594266, 0.593358, 0.592282, 0.590925, 0.589478,\
        0.587699, 0.585081, 0.581575, 0.577377, 0.572642, 0.567475, 0.561944, 0.556081, 0.549902, 0.543411,\
        0.536619, 0.529563, 0.522334, 0.515096, 0.507973, 0.500700, 0.492284, 0.483468, 0.474212, 0.464513,\
        0.454428],\
        [0.378780, 0.387234, 0.394815, 0.401557, 0.407641, 0.413336, 0.419322, 0.426058, 0.433569, 0.441630,\
        0.449899, 0.457995, 0.465503, 0.471902, 0.476599, 0.479350, 0.480567, 0.480823, 0.480586, 0.479986,\
        0.478756, 0.476521, 0.473250, 0.469381, 0.465349, 0.461279, 0.457207, 0.453297, 0.449798, 0.446910,\
        0.444728, 0.443230, 0.442275, 0.441633, 0.441042, 0.440262, 0.439103, 0.437424, 0.435134, 0.432190,\
        0.428599, 0.424421, 0.419769, 0.414801, 0.409727, 0.404816, 0.400294, 0.396211, 0.392867, 0.390747,\
        0.390813, 0.394199, 0.402396, 0.416610, 0.433453, 0.450534, 0.466080, 0.478590, 0.487756, 0.496141,\
        0.504570, 0.511076, 0.517244, 0.524231, 0.531822, 0.540191, 0.549619, 0.560102, 0.571336, 0.582713,\
        0.592774, 0.599943, 0.605347, 0.609791, 0.613741, 0.616642, 0.618489, 0.619709, 0.620537, 0.621007,\
        0.621111, 0.620853, 0.620240, 0.619343, 0.618264, 0.617039, 0.615630, 0.614056, 0.612398, 0.610664,\
        0.608854, 0.607014, 0.605189, 0.603461, 0.601914, 0.600729, 0.599813, 0.598652, 0.597041, 0.595707,\
        0.594267, 0.591490, 0.587686, 0.583220, 0.578238, 0.572844, 0.567095, 0.561012, 0.554604, 0.547881,\
        0.540873, 0.533637, 0.526254, 0.518795, 0.511201, 0.503154, 0.494833, 0.486115, 0.476735, 0.466872,\
        0.456647],\
        [0.379350, 0.387964, 0.395861, 0.403291, 0.410437, 0.417411, 0.424568, 0.432199, 0.440368, 0.448948,\
        0.457707, 0.466375, 0.474634, 0.481970, 0.487380, 0.490126, 0.491262, 0.491465, 0.491455, 0.491399,\
        0.490799, 0.488891, 0.485500, 0.481523, 0.477646, 0.473667, 0.469379, 0.465035, 0.461051, 0.457670,\
        0.454987, 0.452988, 0.451546, 0.450434, 0.449384, 0.448151, 0.446535, 0.444390, 0.441612, 0.438143,\
        0.433984, 0.429203, 0.423934, 0.418366, 0.412724, 0.407230, 0.402063, 0.397532, 0.393772, 0.391474,\
        0.392114, 0.397241, 0.408052, 0.424157, 0.443011, 0.461954, 0.479005, 0.492351, 0.499947, 0.506961,\
        0.514506, 0.521172, 0.527393, 0.533983, 0.541242, 0.549341, 0.558420, 0.568468, 0.579266, 0.590495,\
        0.601204, 0.608172, 0.613421, 0.617684, 0.621174, 0.623812, 0.625610, 0.626767, 0.627480, 0.627855,\
        0.627939, 0.627752, 0.627307, 0.626624, 0.625727, 0.624621, 0.623296, 0.621764, 0.620067, 0.618238,\
        0.616313, 0.614343, 0.612392, 0.610538, 0.608873, 0.607478, 0.606250, 0.604823, 0.603106, 0.601750,\
        0.600453, 0.597235, 0.593212, 0.588526, 0.583330, 0.577750, 0.571828, 0.565562, 0.558945, 0.551991,\
        0.544741, 0.537258, 0.529614, 0.521854, 0.513944, 0.505811, 0.497447, 0.488355, 0.478680, 0.468632,\
        0.458287],\
        [0.379619, 0.388598, 0.396976, 0.405087, 0.413070, 0.420994, 0.429090, 0.437521, 0.446310, 0.455351,\
        0.464464, 0.473454, 0.482149, 0.490283, 0.496820, 0.499159, 0.500541, 0.500801, 0.501116, 0.501540,\
        0.501479, 0.499921, 0.496376, 0.492391, 0.489127, 0.485502, 0.480990, 0.476200, 0.471863, 0.468140,\
        0.465098, 0.462765, 0.461026, 0.459624, 0.458255, 0.456660, 0.454648, 0.452079, 0.448840, 0.444854,\
        0.440111, 0.434690, 0.428758, 0.422549, 0.416327, 0.410348, 0.404845, 0.399953, 0.395699, 0.392926,\
        0.393918, 0.400943, 0.414469, 0.433119, 0.454034, 0.474381, 0.492139, 0.505609, 0.513566, 0.519607,\
        0.525789, 0.531785, 0.537673, 0.543863, 0.550691, 0.558354, 0.566939, 0.576418, 0.586485, 0.596634,\
        0.606743, 0.614938, 0.620885, 0.625366, 0.628766, 0.631240, 0.632915, 0.633968, 0.634582, 0.634898,\
        0.635003, 0.634932, 0.634684, 0.634233, 0.633547, 0.632595, 0.631360, 0.629851, 0.628101, 0.626160,\
        0.624090, 0.621963, 0.619855, 0.617841, 0.615984, 0.614302, 0.612700, 0.611011, 0.609226, 0.607497,\
        0.605383, 0.602150, 0.598098, 0.593201, 0.587857, 0.582163, 0.576129, 0.569721, 0.562907, 0.555702,\
        0.548160, 0.540359, 0.532376, 0.524270, 0.516079, 0.507822, 0.499241, 0.489764, 0.479878, 0.469721,\
        0.459317],\
        [0.379519, 0.389038, 0.398120, 0.406766, 0.415287, 0.423914, 0.432778, 0.441917, 0.451275, 0.460712,\
        0.470033, 0.479034, 0.487578, 0.495577, 0.502402, 0.506012, 0.507887, 0.508942, 0.509689, 0.510252,\
        0.510160, 0.508698, 0.505578, 0.502177, 0.499642, 0.496483, 0.491493, 0.486408, 0.482081, 0.478212,\
        0.474943, 0.472455, 0.470633, 0.469143, 0.467596, 0.465719, 0.463366, 0.460420, 0.456755, 0.452262,\
        0.446903, 0.440770, 0.434080, 0.427133, 0.420248, 0.413726, 0.407821, 0.402684, 0.397887, 0.394227,\
        0.395296, 0.403921, 0.420684, 0.442579, 0.465937, 0.487294, 0.504787, 0.517548, 0.525953, 0.531988,\
        0.537384, 0.542681, 0.548070, 0.553808, 0.560196, 0.567434, 0.575525, 0.584525, 0.594319, 0.604062,\
        0.613684, 0.622042, 0.628481, 0.633236, 0.636662, 0.639030, 0.640561, 0.641469, 0.641963, 0.642225,\
        0.642375, 0.642458, 0.642447, 0.642261, 0.641810, 0.641025, 0.639872, 0.638355, 0.636513, 0.634414,\
        0.632147, 0.629810, 0.627488, 0.625248, 0.623129, 0.621120, 0.619154, 0.617141, 0.615032, 0.612753,\
        0.610017, 0.606527, 0.602218, 0.597227, 0.591845, 0.586101, 0.580032, 0.573522, 0.566501, 0.559000,\
        0.551105, 0.542915, 0.534517, 0.525978, 0.517339, 0.508578, 0.499531, 0.490091, 0.480261, 0.470132,\
        0.459746],\
        [0.378817, 0.388826, 0.398571, 0.407673, 0.416670, 0.425945, 0.435526, 0.445336, 0.455244, 0.465065,\
        0.474554, 0.483439, 0.491505, 0.498845, 0.505633, 0.510566, 0.513631, 0.515563, 0.516824, 0.517533,\
        0.517448, 0.516235, 0.513937, 0.511292, 0.508679, 0.505346, 0.499960, 0.495779, 0.491833, 0.487925,\
        0.484466, 0.482036, 0.480358, 0.479002, 0.477392, 0.475261, 0.472603, 0.469327, 0.465285, 0.460299,\
        0.454271, 0.447309, 0.439716, 0.431880, 0.424181, 0.416931, 0.410335, 0.404394, 0.398769, 0.394640,\
        0.395945, 0.406489, 0.426475, 0.452115, 0.478588, 0.500784, 0.517494, 0.529414, 0.537570, 0.543494,\
        0.548553, 0.553463, 0.558489, 0.563833, 0.569850, 0.576890, 0.584705, 0.593133, 0.603116, 0.613071,\
        0.622405, 0.630537, 0.637000, 0.641792, 0.645157, 0.647379, 0.648721, 0.649431, 0.649760, 0.649941,\
        0.650139, 0.650412, 0.650689, 0.650813, 0.650620, 0.649994, 0.648892, 0.647318, 0.645319, 0.642985,\
        0.640443, 0.637822, 0.635220, 0.632693, 0.630257, 0.627892, 0.625539, 0.623124, 0.620565, 0.617740,\
        0.614459, 0.610549, 0.605966, 0.600886, 0.595378, 0.589625, 0.583638, 0.577043, 0.569758, 0.561885,\
        0.553566, 0.544928, 0.536065, 0.527036, 0.517871, 0.508597, 0.499260, 0.489756, 0.480014, 0.469967,\
        0.459637],\
        [0.377329, 0.387538, 0.397577, 0.407208, 0.416894, 0.426949, 0.437289, 0.447774, 0.458243, 0.468495,\
        0.478266, 0.487235, 0.495119, 0.502158, 0.508864, 0.514267, 0.518148, 0.520817, 0.522590, 0.523586,\
        0.523738, 0.522949, 0.521319, 0.519143, 0.516494, 0.513004, 0.508735, 0.504974, 0.501277, 0.497396,\
        0.493672, 0.491707, 0.490244, 0.489290, 0.487629, 0.485196, 0.482272, 0.478698, 0.474344, 0.468897,\
        0.462114, 0.454149, 0.445463, 0.436571, 0.427909, 0.419789, 0.412360, 0.405562, 0.399250, 0.394529,\
        0.395590, 0.407993, 0.431737, 0.461160, 0.490909, 0.514241, 0.530159, 0.541105, 0.548569, 0.554127,\
        0.559028, 0.563920, 0.568929, 0.574051, 0.579713, 0.586839, 0.595415, 0.603466, 0.613388, 0.623537,\
        0.632755, 0.640598, 0.646806, 0.651375, 0.654510, 0.656496, 0.657588, 0.658035, 0.658126, 0.658167,\
        0.658395, 0.658894, 0.659530, 0.660020, 0.660093, 0.659592, 0.658484, 0.656784, 0.654536, 0.651856,\
        0.648934, 0.645946, 0.643000, 0.640135, 0.637344, 0.634592, 0.631822, 0.628962, 0.625920, 0.622582,\
        0.618807, 0.614481, 0.609565, 0.604116, 0.598305, 0.592950, 0.587172, 0.580384, 0.572688, 0.564335,\
        0.555527, 0.546406, 0.537072, 0.527581, 0.517950, 0.508169, 0.498469, 0.489064, 0.479387, 0.469374,\
        0.459074],\
        [0.375151, 0.385317, 0.395329, 0.405443, 0.416066, 0.427024, 0.438138, 0.449272, 0.460288, 0.471012,\
        0.481209, 0.490563, 0.498800, 0.505802, 0.512126, 0.517532, 0.521797, 0.524975, 0.527211, 0.528593,\
        0.529134, 0.528835, 0.527772, 0.526063, 0.523739, 0.520767, 0.517370, 0.513952, 0.510467, 0.506622,\
        0.503482, 0.501971, 0.500409, 0.500122, 0.498089, 0.495459, 0.492339, 0.488413, 0.483844, 0.477998,\
        0.470296, 0.461069, 0.451089, 0.441013, 0.431311, 0.422298, 0.414138, 0.406828, 0.400200, 0.394877,\
        0.395912, 0.410084, 0.437249, 0.470356, 0.501579, 0.526429, 0.542590, 0.552509, 0.558893, 0.563826,\
        0.568650, 0.573929, 0.579494, 0.584790, 0.590126, 0.596938, 0.606651, 0.616151, 0.625989, 0.635861,\
        0.644776, 0.652208, 0.657979, 0.662119, 0.664860, 0.666545, 0.667365, 0.667490, 0.667254, 0.667063,\
        0.667269, 0.668032, 0.669126, 0.670045, 0.670367, 0.669903, 0.668695, 0.666794, 0.664181, 0.660999,\
        0.657563, 0.654128, 0.650794, 0.647557, 0.644379, 0.641211, 0.637998, 0.634673, 0.631154, 0.627341,\
        0.623135, 0.618452, 0.613259, 0.607598, 0.601883, 0.596759, 0.590885, 0.583519, 0.575200, 0.566273,\
        0.556939, 0.547336, 0.537566, 0.527701, 0.517798, 0.507921, 0.498142, 0.488433, 0.478546, 0.468442,\
        0.458118],\
        [0.372610, 0.382777, 0.392819, 0.403347, 0.414766, 0.426486, 0.438230, 0.449886, 0.461353, 0.472496,\
        0.483134, 0.492995, 0.501465, 0.508545, 0.514740, 0.520140, 0.524630, 0.528182, 0.530842, 0.532665,\
        0.533681, 0.533923, 0.533443, 0.532311, 0.530576, 0.528308, 0.525670, 0.522840, 0.519873, 0.516890,\
        0.514512, 0.512969, 0.511735, 0.510812, 0.508979, 0.506292, 0.502784, 0.498393, 0.493700, 0.487509,\
        0.478509, 0.467701, 0.456318, 0.445045, 0.434319, 0.424460, 0.415687, 0.408089, 0.401568, 0.395732,\
        0.396258, 0.412195, 0.443521, 0.481785, 0.516507, 0.541292, 0.556058, 0.563910, 0.568543, 0.572494,\
        0.577210, 0.583193, 0.590072, 0.596397, 0.602045, 0.609617, 0.619360, 0.629769, 0.640020, 0.649764,\
        0.658306, 0.665188, 0.670418, 0.674007, 0.676190, 0.677625, 0.678267, 0.678043, 0.677408, 0.676881,\
        0.676940, 0.677987, 0.679693, 0.681074, 0.681588, 0.680968, 0.679523, 0.677387, 0.674268, 0.670346,\
        0.666250, 0.662325, 0.658591, 0.654964, 0.651367, 0.647750, 0.644068, 0.640266, 0.636277, 0.632024,\
        0.627436, 0.622470, 0.617144, 0.611602, 0.606186, 0.601094, 0.594316, 0.586047, 0.577059, 0.567571,\
        0.557733, 0.547676, 0.537515, 0.527343, 0.517239, 0.507326, 0.497628, 0.487519, 0.477383, 0.467156,\
        0.456783],\
        [0.369976, 0.380411, 0.390816, 0.401637, 0.413529, 0.425622, 0.437690, 0.449647, 0.461393, 0.472793,\
        0.483664, 0.493742, 0.502490, 0.509810, 0.516223, 0.521809, 0.526559, 0.530466, 0.533552, 0.535855,\
        0.537409, 0.538248, 0.538416, 0.537967, 0.536963, 0.535491, 0.533673, 0.531640, 0.529512, 0.527478,\
        0.525814, 0.524583, 0.523548, 0.522359, 0.520459, 0.517591, 0.513703, 0.508956, 0.503691, 0.496927,\
        0.485864, 0.473494, 0.460905, 0.448599, 0.436959, 0.426319, 0.416977, 0.409133, 0.402640, 0.397253,\
        0.396685, 0.414218, 0.450707, 0.495297, 0.534991, 0.560114, 0.571345, 0.575553, 0.577675, 0.580198,\
        0.584590, 0.591238, 0.599503, 0.607657, 0.615242, 0.623988, 0.634028, 0.644582, 0.654976, 0.664728,\
        0.672977, 0.679117, 0.683811, 0.686978, 0.688270, 0.689786, 0.690521, 0.689962, 0.688966, 0.688095,\
        0.687771, 0.688973, 0.691576, 0.693202, 0.693887, 0.692748, 0.690896, 0.688624, 0.684769, 0.679728,\
        0.674894, 0.670531, 0.666429, 0.662389, 0.658324, 0.654206, 0.650017, 0.645719, 0.641259, 0.636577,\
        0.631622, 0.626373, 0.620864, 0.615202, 0.609496, 0.603444, 0.596091, 0.587468, 0.578064, 0.568138,\
        0.557859, 0.547386, 0.536867, 0.526424, 0.516107, 0.505857, 0.495804, 0.485894, 0.475755, 0.465467,\
        0.455058],\
        [0.367247, 0.378165, 0.389279, 0.400756, 0.412449, 0.424406, 0.436496, 0.448534, 0.460377, 0.471876,\
        0.482812, 0.492831, 0.501712, 0.509678, 0.516497, 0.522417, 0.527512, 0.531810, 0.535353, 0.538182,\
        0.540328, 0.541824, 0.542707, 0.543023, 0.542835, 0.542228, 0.541305, 0.540186, 0.538998, 0.537888,\
        0.536976, 0.536240, 0.535468, 0.534297, 0.532302, 0.529170, 0.524778, 0.519178, 0.512328, 0.503099,\
        0.491227, 0.478198, 0.464871, 0.451822, 0.439437, 0.428076, 0.418145, 0.410086, 0.404275, 0.400207,\
        0.397651, 0.417602, 0.460252, 0.511888, 0.558944, 0.583265, 0.587936, 0.587585, 0.586948, 0.587539,\
        0.590966, 0.598439, 0.607770, 0.618042, 0.628271, 0.638726, 0.649443, 0.660027, 0.670210, 0.679998,\
        0.688358, 0.693355, 0.697438, 0.700875, 0.701031, 0.703045, 0.704072, 0.703529, 0.702477, 0.701509,\
        0.700820, 0.701610, 0.704888, 0.706472, 0.706900, 0.705350, 0.702986, 0.700421, 0.695244, 0.688866,\
        0.683511, 0.678881, 0.674425, 0.669891, 0.665257, 0.660559, 0.655811, 0.650990, 0.646045, 0.640919,\
        0.635569, 0.629971, 0.624125, 0.618035, 0.611622, 0.604640, 0.596762, 0.587912, 0.578273, 0.568008,\
        0.557331, 0.546453, 0.535578, 0.524890, 0.514503, 0.504316, 0.494141, 0.483964, 0.473707, 0.463362,\
        0.452936],\
        [0.364198, 0.375543, 0.387162, 0.398944, 0.410578, 0.422465, 0.434498, 0.446479, 0.458285, 0.469793,\
        0.480826, 0.491183, 0.500605, 0.508511, 0.515635, 0.521971, 0.527468, 0.532194, 0.536232, 0.539635,\
        0.542432, 0.544644, 0.546300, 0.547439, 0.548119, 0.548417, 0.548427, 0.548256, 0.548018, 0.547812,\
        0.547682, 0.547556, 0.547203, 0.546249, 0.544255, 0.540826, 0.535715, 0.528806, 0.519937, 0.508865,\
        0.496087, 0.482428, 0.468654, 0.455133, 0.442188, 0.430166, 0.419565, 0.411022, 0.405143, 0.402034,\
        0.400699, 0.427824, 0.477884, 0.532106, 0.584076, 0.606990, 0.605563, 0.601405, 0.598206, 0.596499,\
        0.596860, 0.604908, 0.616621, 0.629069, 0.641526, 0.653613, 0.665085, 0.675616, 0.684959, 0.694251,\
        0.702931, 0.707661, 0.711388, 0.714418, 0.715987, 0.717655, 0.718836, 0.718890, 0.718287, 0.717540,\
        0.717111, 0.717865, 0.719787, 0.721017, 0.720825, 0.719028, 0.716006, 0.711784, 0.705307, 0.698276,\
        0.692563, 0.687731, 0.682735, 0.677501, 0.672133, 0.666749, 0.661389, 0.656015, 0.650561, 0.644963,\
        0.639173, 0.633153, 0.626867, 0.620242, 0.613130, 0.605299, 0.596846, 0.587858, 0.577953, 0.567321,\
        0.556222, 0.544911, 0.533632, 0.522626, 0.512147, 0.502280, 0.492003, 0.481588, 0.471214, 0.460834,\
        0.450416],\
        [0.360617, 0.372199, 0.383919, 0.395649, 0.407549, 0.419621, 0.431598, 0.443414, 0.455076, 0.466523,\
        0.477600, 0.488060, 0.497448, 0.505886, 0.513714, 0.520540, 0.526432, 0.531592, 0.536163, 0.540196,\
        0.543702, 0.546687, 0.549164, 0.551165, 0.552739, 0.553954, 0.554894, 0.555654, 0.556330, 0.556994,\
        0.557659, 0.558227, 0.558451, 0.557921, 0.556108, 0.552478, 0.546641, 0.538437, 0.527894, 0.515265,\
        0.501211, 0.487029, 0.472892, 0.459121, 0.445828, 0.433243, 0.421967, 0.412897, 0.407096, 0.406030,\
        0.414523, 0.445687, 0.498649, 0.550883, 0.595342, 0.619586, 0.622768, 0.618288, 0.613131, 0.609528,\
        0.609081, 0.615828, 0.627909, 0.641578, 0.655456, 0.668676, 0.680570, 0.691209, 0.699378, 0.707528,\
        0.715489, 0.721260, 0.725511, 0.728880, 0.731423, 0.733490, 0.734999, 0.735698, 0.735709, 0.735416,\
        0.735300, 0.735749, 0.736552, 0.736844, 0.735925, 0.733510, 0.729589, 0.724084, 0.717025, 0.709511,\
        0.702992, 0.697339, 0.691335, 0.685117, 0.678824, 0.672668, 0.666668, 0.660725, 0.654738, 0.648631,\
        0.642355, 0.635864, 0.629105, 0.621994, 0.614404, 0.606198, 0.597358, 0.587765, 0.577329, 0.566206,\
        0.554628, 0.542844, 0.531112, 0.519683, 0.508690, 0.498549, 0.488753, 0.478525, 0.468215, 0.457877,\
        0.447511],\
        [0.356428, 0.368179, 0.380052, 0.391946, 0.403968, 0.416069, 0.427789, 0.439263, 0.450686, 0.462045,\
        0.473166, 0.483758, 0.493526, 0.502682, 0.511081, 0.518215, 0.524369, 0.529948, 0.535107, 0.539844,\
        0.544125, 0.547933, 0.551269, 0.554155, 0.556629, 0.558749, 0.560585, 0.562223, 0.563746, 0.565218,\
        0.566648, 0.567951, 0.568886, 0.569010, 0.567662, 0.564085, 0.557713, 0.548423, 0.536509, 0.522586,\
        0.507618, 0.492548, 0.478053, 0.464401, 0.451043, 0.437997, 0.426122, 0.416777, 0.411742, 0.413695,\
        0.427264, 0.457670, 0.505379, 0.561379, 0.605717, 0.629925, 0.636999, 0.634592, 0.629471, 0.625564,\
        0.625509, 0.631243, 0.642050, 0.655531, 0.669812, 0.683697, 0.695235, 0.705271, 0.713656, 0.721109,\
        0.728350, 0.734510, 0.739544, 0.743716, 0.747138, 0.749897, 0.752015, 0.753398, 0.753976, 0.754033,\
        0.754055, 0.754150, 0.754101, 0.753480, 0.751757, 0.748590, 0.743846, 0.737517, 0.729916, 0.721862,\
        0.714235, 0.707042, 0.699878, 0.692437, 0.685096, 0.678177, 0.671571, 0.665062, 0.658511, 0.651851,\
        0.645041, 0.638043, 0.630804, 0.623252, 0.615306, 0.606887, 0.597726, 0.587424, 0.576361, 0.564691,\
        0.552616, 0.540364, 0.528183, 0.516410, 0.505473, 0.495014, 0.485027, 0.474971, 0.464786, 0.454538,\
        0.444256],\
        [0.351598, 0.363495, 0.375609, 0.387850, 0.400092, 0.411903, 0.422938, 0.433874, 0.445029, 0.456350,\
        0.467632, 0.478639, 0.489184, 0.499084, 0.507819, 0.514864, 0.521101, 0.527168, 0.533042, 0.538580,\
        0.543702, 0.548375, 0.552596, 0.556377, 0.559743, 0.562739, 0.565420, 0.567859, 0.570131, 0.572306,\
        0.574413, 0.576411, 0.578120, 0.579119, 0.578620, 0.575498, 0.568876, 0.558736, 0.545717, 0.530689,\
        0.514625, 0.498625, 0.484423, 0.471627, 0.458453, 0.444874, 0.432370, 0.422825, 0.418544, 0.422561,\
        0.438542, 0.469327, 0.516343, 0.574278, 0.618138, 0.641731, 0.650185, 0.649347, 0.645063, 0.641652,\
        0.641906, 0.647181, 0.657093, 0.669990, 0.683578, 0.697127, 0.708672, 0.718100, 0.726571, 0.734216,\
        0.741332, 0.747788, 0.753541, 0.758639, 0.762974, 0.766448, 0.769231, 0.771363, 0.772351, 0.772404,\
        0.772512, 0.772280, 0.771480, 0.770139, 0.767686, 0.763803, 0.758378, 0.751336, 0.742996, 0.734043,\
        0.725097, 0.716352, 0.707697, 0.699007, 0.690724, 0.683211, 0.676087, 0.668992, 0.661819, 0.654544,\
        0.647152, 0.639611, 0.631870, 0.623865, 0.615510, 0.606634, 0.597002, 0.586420, 0.574878, 0.562716,\
        0.550206, 0.537577, 0.525032, 0.512782, 0.501522, 0.491498, 0.481323, 0.471208, 0.461069, 0.450897,\
        0.440703],\
        [0.346077, 0.358019, 0.370271, 0.382767, 0.395299, 0.406460, 0.416562, 0.427024, 0.438048, 0.449429,\
        0.460920, 0.472336, 0.483526, 0.494217, 0.503324, 0.509874, 0.516328, 0.523230, 0.530018, 0.536448,\
        0.542453, 0.548018, 0.553138, 0.557815, 0.562061, 0.565898, 0.569363, 0.572513, 0.575417, 0.578149,\
        0.580772, 0.583309, 0.585693, 0.587655, 0.588429, 0.586324, 0.579733, 0.568966, 0.555225, 0.539556,\
        0.522896, 0.506490, 0.492706, 0.481661, 0.468312, 0.453773, 0.440517, 0.430604, 0.426500, 0.431371,\
        0.448833, 0.481853, 0.530225, 0.585796, 0.629133, 0.653838, 0.662963, 0.662646, 0.658967, 0.656124,\
        0.656766, 0.662021, 0.671461, 0.683957, 0.697349, 0.708492, 0.720078, 0.730032, 0.738845, 0.746871,\
        0.754263, 0.761087, 0.767486, 0.773545, 0.778843, 0.782822, 0.786000, 0.788912, 0.790208, 0.789445,\
        0.790002, 0.789379, 0.787717, 0.786036, 0.782901, 0.778400, 0.772515, 0.764747, 0.755447, 0.745381,\
        0.735127, 0.724939, 0.714849, 0.704922, 0.695779, 0.687950, 0.680288, 0.672479, 0.664576, 0.656616,\
        0.648592, 0.640473, 0.632206, 0.623715, 0.614939, 0.605836, 0.595928, 0.584814, 0.572754, 0.560173,\
        0.547350, 0.534510, 0.521881, 0.509716, 0.498172, 0.487692, 0.477480, 0.467296, 0.457146, 0.447017,\
        0.436901],\
        [0.339858, 0.351666, 0.363764, 0.375970, 0.387824, 0.398466, 0.408310, 0.418738, 0.429901, 0.441402,\
        0.453000, 0.464569, 0.475955, 0.486768, 0.495925, 0.502427, 0.510217, 0.518423, 0.526201, 0.533524,\
        0.540413, 0.546874, 0.552898, 0.558473, 0.563586, 0.568235, 0.572431, 0.576207, 0.579619, 0.582743,\
        0.585666, 0.588468, 0.591195, 0.593806, 0.595993, 0.595685, 0.589278, 0.578235, 0.564480, 0.549048,\
        0.532866, 0.517197, 0.504413, 0.494790, 0.479334, 0.463868, 0.450102, 0.439688, 0.435036, 0.439365,\
        0.456572, 0.490491, 0.540415, 0.592618, 0.640587, 0.668587, 0.676025, 0.674755, 0.671178, 0.668691,\
        0.669767, 0.675483, 0.684919, 0.696083, 0.709375, 0.720996, 0.731799, 0.741811, 0.751010, 0.759541,\
        0.767374, 0.774552, 0.781384, 0.788314, 0.794732, 0.798888, 0.802063, 0.805068, 0.806352, 0.805037,\
        0.805627, 0.804435, 0.802140, 0.800418, 0.796309, 0.791464, 0.785583, 0.776919, 0.766493, 0.755332,\
        0.743957, 0.732640, 0.721531, 0.710845, 0.701198, 0.692785, 0.684132, 0.675367, 0.666636, 0.657941,\
        0.649253, 0.640538, 0.631735, 0.622747, 0.613422, 0.603779, 0.593996, 0.582262, 0.569782, 0.556919,\
        0.543931, 0.531027, 0.518432, 0.506484, 0.495318, 0.484192, 0.473571, 0.463232, 0.453038, 0.442934,\
        0.432889],\
        [0.333051, 0.344570, 0.356303, 0.367960, 0.379096, 0.389334, 0.399059, 0.409737, 0.421117, 0.432605,\
        0.444065, 0.455473, 0.466708, 0.477341, 0.486667, 0.494972, 0.504214, 0.513203, 0.521730, 0.529853,\
        0.537595, 0.544948, 0.551883, 0.558361, 0.564342, 0.569792, 0.574686, 0.579027, 0.582846, 0.586216,\
        0.589232, 0.592004, 0.594624, 0.597139, 0.599535, 0.600871, 0.595474, 0.585416, 0.572793, 0.558540,\
        0.543606, 0.529216, 0.516575, 0.503678, 0.489007, 0.474495, 0.460894, 0.450034, 0.444314, 0.446920,\
        0.461838, 0.494041, 0.547901, 0.611490, 0.665693, 0.688972, 0.689114, 0.686233, 0.682608, 0.680084,\
        0.681469, 0.688008, 0.698649, 0.709686, 0.721771, 0.733537, 0.744225, 0.754102, 0.763550, 0.772720,\
        0.781137, 0.788689, 0.795680, 0.803030, 0.809959, 0.814671, 0.817864, 0.820116, 0.820812, 0.820102,\
        0.819234, 0.817478, 0.814864, 0.811754, 0.807327, 0.802465, 0.796638, 0.786625, 0.775317, 0.763379,\
        0.751222, 0.739145, 0.727397, 0.716321, 0.706372, 0.696953, 0.687020, 0.677320, 0.667808, 0.658391,\
        0.649033, 0.639708, 0.630377, 0.620961, 0.611296, 0.601071, 0.590232, 0.578442, 0.565806, 0.552831,\
        0.539832, 0.526995, 0.514421, 0.502299, 0.490964, 0.480051, 0.469374, 0.458954, 0.448735, 0.438661,\
        0.428690],\
        [0.325881, 0.337059, 0.348420, 0.359650, 0.370422, 0.380608, 0.390602, 0.401250, 0.412345, 0.423480,\
        0.434446, 0.445441, 0.456494, 0.467296, 0.477561, 0.487581, 0.497703, 0.507315, 0.516499, 0.525389,\
        0.533981, 0.542235, 0.550094, 0.557493, 0.564362, 0.570629, 0.576227, 0.581112, 0.585289, 0.588817,\
        0.591803, 0.594376, 0.596662, 0.598767, 0.600757, 0.601426, 0.597961, 0.590401, 0.579893, 0.567410,\
        0.553892, 0.540274, 0.526953, 0.513331, 0.499813, 0.486165, 0.473006, 0.461884, 0.454980, 0.455587,\
        0.467980, 0.498009, 0.551560, 0.618989, 0.683573, 0.707125, 0.702708, 0.698904, 0.695121, 0.691893,\
        0.693578, 0.701045, 0.712394, 0.724459, 0.736307, 0.747591, 0.757909, 0.767359, 0.776669, 0.786790,\
        0.795988, 0.804107, 0.811601, 0.818845, 0.825287, 0.829970, 0.832916, 0.834403, 0.834455, 0.833314,\
        0.831402, 0.828711, 0.825239, 0.821012, 0.815937, 0.810166, 0.802611, 0.792761, 0.781455, 0.769191,\
        0.756628, 0.744100, 0.731853, 0.720136, 0.709118, 0.698587, 0.688266, 0.678102, 0.667980, 0.657887,\
        0.647853, 0.637899, 0.628012, 0.618137, 0.608216, 0.597881, 0.586094, 0.573701, 0.560868, 0.547860,\
        0.534966, 0.522379, 0.510064, 0.497994, 0.486560, 0.475557, 0.464858, 0.454427, 0.444225, 0.434205,\
        0.424325],\
        [0.318637, 0.329514, 0.340612, 0.351596, 0.362237, 0.372531, 0.382754, 0.393205, 0.403886, 0.414383,\
        0.424369, 0.434790, 0.445782, 0.456961, 0.468059, 0.479058, 0.489908, 0.500346, 0.510349, 0.520071,\
        0.529546, 0.538724, 0.547528, 0.555875, 0.563672, 0.570808, 0.577161, 0.582631, 0.587177, 0.590857,\
        0.593806, 0.596207, 0.598261, 0.600180, 0.601988, 0.602330, 0.599887, 0.594351, 0.586155, 0.575490,\
        0.563291, 0.550421, 0.537354, 0.524267, 0.511384, 0.498586, 0.486273, 0.475265, 0.467274, 0.466035,\
        0.476964, 0.505834, 0.556669, 0.621660, 0.683057, 0.713349, 0.716792, 0.713472, 0.709620, 0.706896,\
        0.709468, 0.717612, 0.728840, 0.741026, 0.752761, 0.763587, 0.773479, 0.782391, 0.790502, 0.801375,\
        0.811897, 0.820771, 0.828571, 0.835407, 0.841027, 0.844996, 0.847198, 0.847766, 0.846888, 0.844822,\
        0.841829, 0.838030, 0.833453, 0.828077, 0.821828, 0.814579, 0.806155, 0.796644, 0.784995, 0.772710,\
        0.760072, 0.747338, 0.734731, 0.722385, 0.710296, 0.698748, 0.688440, 0.677932, 0.667200, 0.656418,\
        0.645689, 0.635076, 0.624591, 0.614131, 0.603457, 0.592472, 0.580608, 0.568002, 0.555042, 0.542017,\
        0.529226, 0.516972, 0.505313, 0.493492, 0.481863, 0.470755, 0.460047, 0.449652, 0.439513, 0.429582,\
        0.419819],\
        [0.311642, 0.322316, 0.333259, 0.344071, 0.354594, 0.364874, 0.375093, 0.385428, 0.395854, 0.405446,\
        0.413679, 0.423747, 0.434883, 0.446472, 0.458183, 0.469846, 0.481330, 0.492501, 0.503335, 0.513919,\
        0.524302, 0.534416, 0.544173, 0.553489, 0.562268, 0.570362, 0.577582, 0.583740, 0.588730, 0.592622,\
        0.595628, 0.598000, 0.600135, 0.602180, 0.603770, 0.603990, 0.602027, 0.598508, 0.592100, 0.582823,\
        0.571749, 0.559717, 0.547287, 0.534836, 0.522635, 0.510880, 0.500008, 0.489514, 0.480526, 0.476759,\
        0.484887, 0.512836, 0.561803, 0.623347, 0.680730, 0.713852, 0.724255, 0.725592, 0.724680, 0.724984,\
        0.728967, 0.737005, 0.747771, 0.759740, 0.771167, 0.781392, 0.791123, 0.800127, 0.808272, 0.817970,\
        0.829033, 0.838304, 0.845881, 0.851996, 0.856596, 0.859472, 0.860538, 0.859907, 0.857818, 0.854551,\
        0.850350, 0.845360, 0.839619, 0.833078, 0.825603, 0.817005, 0.807222, 0.796821, 0.786034, 0.774184,\
        0.761663, 0.748879, 0.736123, 0.723562, 0.711354, 0.699941, 0.688531, 0.677044, 0.665513, 0.653997,\
        0.642563, 0.631278, 0.620204, 0.609262, 0.598084, 0.586442, 0.574250, 0.561509, 0.548481, 0.535435,\
        0.522623, 0.510402, 0.499078, 0.487776, 0.476478, 0.465524, 0.454931, 0.444648, 0.434626, 0.424823,\
        0.415203],\
        [0.305293, 0.315905, 0.326683, 0.337219, 0.347468, 0.357497, 0.367418, 0.377301, 0.386941, 0.394910,\
        0.402696, 0.412931, 0.424233, 0.436116, 0.448231, 0.460350, 0.472336, 0.484107, 0.495645, 0.507045,\
        0.518302, 0.529325, 0.539996, 0.550276, 0.560089, 0.569262, 0.577525, 0.584559, 0.590124, 0.594286,\
        0.597502, 0.600059, 0.602436, 0.604591, 0.606108, 0.606489, 0.605427, 0.602803, 0.597457, 0.589195,\
        0.579137, 0.567991, 0.556311, 0.544537, 0.533029, 0.522122, 0.512083, 0.502671, 0.493682, 0.487172,\
        0.488352, 0.512913, 0.565338, 0.625273, 0.676166, 0.709128, 0.726403, 0.733869, 0.738109, 0.742514,\
        0.748824, 0.757357, 0.767650, 0.779501, 0.791174, 0.800300, 0.810304, 0.819763, 0.828916, 0.838492,\
        0.848143, 0.856412, 0.862947, 0.867922, 0.871294, 0.872860, 0.872528, 0.870447, 0.866926, 0.862295,\
        0.856813, 0.850629, 0.843793, 0.836255, 0.827867, 0.818414, 0.807772, 0.796826, 0.786099, 0.774209,\
        0.761589, 0.748727, 0.735948, 0.723426, 0.711222, 0.699247, 0.687362, 0.675079, 0.662793, 0.650613,\
        0.638559, 0.626628, 0.614961, 0.603655, 0.592116, 0.580036, 0.567461, 0.554516, 0.541429, 0.528423,\
        0.515633, 0.503170, 0.491970, 0.481239, 0.470500, 0.459910, 0.449564, 0.439471, 0.429616, 0.419975,\
        0.410525],\
        [0.300457, 0.310852, 0.321091, 0.331082, 0.340835, 0.350379, 0.359738, 0.368848, 0.377428, 0.385210,\
        0.393371, 0.403129, 0.414272, 0.426208, 0.438492, 0.450843, 0.463150, 0.475373, 0.487506, 0.499560,\
        0.511630, 0.523461, 0.534919, 0.546104, 0.556988, 0.567373, 0.576913, 0.585132, 0.591515, 0.595930,\
        0.599237, 0.602302, 0.604926, 0.607110, 0.608594, 0.609084, 0.608336, 0.605996, 0.601384, 0.594284,\
        0.585312, 0.575097, 0.564218, 0.553131, 0.542188, 0.531693, 0.521890, 0.512923, 0.505162, 0.499275,\
        0.494195, 0.515064, 0.570153, 0.627411, 0.674023, 0.707331, 0.726023, 0.739253, 0.749527, 0.758656,\
        0.767979, 0.777588, 0.787381, 0.797842, 0.810339, 0.820083, 0.830009, 0.839966, 0.849724, 0.859184,\
        0.867589, 0.874034, 0.878763, 0.882260, 0.884303, 0.884477, 0.882605, 0.878905, 0.873829, 0.867779,\
        0.861025, 0.853722, 0.845932, 0.837646, 0.828784, 0.819198, 0.808833, 0.797915, 0.785747, 0.772965,\
        0.759818, 0.746723, 0.733935, 0.721502, 0.709448, 0.697359, 0.684596, 0.671688, 0.658848, 0.646224,\
        0.633830, 0.621494, 0.609063, 0.597529, 0.585796, 0.573369, 0.560415, 0.547216, 0.534070, 0.521227,\
        0.508798, 0.496785, 0.485441, 0.474794, 0.464388, 0.454139, 0.444076, 0.434217, 0.424561, 0.415107,\
        0.405845],\
        [0.297459, 0.306925, 0.316249, 0.325522, 0.334648, 0.343590, 0.352317, 0.360761, 0.368840, 0.376707,\
        0.385022, 0.394452, 0.405160, 0.416948, 0.429143, 0.441462, 0.453869, 0.466370, 0.478965, 0.491656,\
        0.504415, 0.516755, 0.528767, 0.540773, 0.552737, 0.564426, 0.575459, 0.585276, 0.593041, 0.597902,\
        0.601192, 0.604459, 0.607248, 0.609413, 0.610794, 0.611202, 0.610432, 0.608208, 0.604178, 0.598159,\
        0.590250, 0.580976, 0.570976, 0.560646, 0.550245, 0.539999, 0.530182, 0.521224, 0.513883, 0.508989,\
        0.508072, 0.529386, 0.577985, 0.627896, 0.668128, 0.698899, 0.723122, 0.743880, 0.760578, 0.774044,\
        0.786401, 0.797333, 0.806963, 0.816781, 0.827894, 0.838603, 0.848938, 0.859099, 0.868929, 0.878131,\
        0.885469, 0.889487, 0.891840, 0.893823, 0.894614, 0.893502, 0.890077, 0.884703, 0.878097, 0.870706,\
        0.862787, 0.854486, 0.845886, 0.837031, 0.827961, 0.818624, 0.808425, 0.796742, 0.783836, 0.769942,\
        0.755978, 0.742550, 0.729812, 0.717476, 0.705147, 0.693004, 0.679985, 0.666688, 0.653527, 0.640712,\
        0.628389, 0.616528, 0.604345, 0.592031, 0.579715, 0.566664, 0.553227, 0.539716, 0.526471, 0.513819,\
        0.501846, 0.490243, 0.479153, 0.468604, 0.458402, 0.448419, 0.438622, 0.429003, 0.419561, 0.410307,\
        0.401248],\
        [0.293826, 0.302843, 0.311646, 0.320315, 0.328833, 0.337185, 0.345348, 0.353305, 0.361105, 0.368973,\
        0.377332, 0.386590, 0.396981, 0.408412, 0.420190, 0.432192, 0.444471, 0.457055, 0.469925, 0.483051,\
        0.496283, 0.508796, 0.521285, 0.534083, 0.547096, 0.560053, 0.572586, 0.584208, 0.594134, 0.600494,\
        0.603469, 0.606389, 0.609123, 0.611212, 0.612426, 0.612620, 0.611675, 0.609471, 0.605900, 0.600847,\
        0.593850, 0.585594, 0.576677, 0.567326, 0.557662, 0.547807, 0.538036, 0.528970, 0.521807, 0.518670,\
        0.524264, 0.547172, 0.587035, 0.628968, 0.664287, 0.692574, 0.723625, 0.752667, 0.774546, 0.790432,\
        0.803810, 0.815842, 0.825609, 0.835148, 0.845248, 0.855654, 0.866006, 0.875896, 0.884943, 0.893242,\
        0.899543, 0.900822, 0.900798, 0.901292, 0.900838, 0.898949, 0.894058, 0.887207, 0.879343, 0.870849,\
        0.861953, 0.852804, 0.843501, 0.834115, 0.824757, 0.815785, 0.806809, 0.794654, 0.779914, 0.764464,\
        0.749532, 0.735808, 0.723353, 0.711633, 0.699245, 0.686519, 0.673416, 0.660039, 0.646769, 0.633891,\
        0.621693, 0.610581, 0.600817, 0.587557, 0.573658, 0.559805, 0.545923, 0.532131, 0.518736, 0.506184,\
        0.494617, 0.483568, 0.472901, 0.462612, 0.452627, 0.442874, 0.433319, 0.423945, 0.414722, 0.405652,\
        0.396827],\
        [0.290030, 0.298691, 0.307150, 0.315333, 0.323316, 0.331160, 0.338859, 0.346433, 0.353993, 0.361764,\
        0.370058, 0.379179, 0.389300, 0.400229, 0.411387, 0.422891, 0.434886, 0.447388, 0.460281, 0.473422,\
        0.486537, 0.499404, 0.512468, 0.526023, 0.539973, 0.553976, 0.567671, 0.580598, 0.591943, 0.599698,\
        0.603419, 0.607106, 0.610150, 0.612280, 0.613343, 0.613269, 0.612047, 0.609702, 0.606270, 0.601711,\
        0.595837, 0.588981, 0.581501, 0.573501, 0.564972, 0.555909, 0.546515, 0.537524, 0.530613, 0.528907,\
        0.537265, 0.560225, 0.596040, 0.634602, 0.671388, 0.704390, 0.738989, 0.770790, 0.794212, 0.810320,\
        0.823008, 0.833936, 0.843298, 0.852143, 0.861255, 0.870814, 0.880619, 0.889761, 0.896973, 0.902752,\
        0.906651, 0.906911, 0.905575, 0.904081, 0.901846, 0.898868, 0.893539, 0.886056, 0.877445, 0.868172,\
        0.858520, 0.848672, 0.838750, 0.828834, 0.818890, 0.809004, 0.801494, 0.788976, 0.772521, 0.755799,\
        0.740056, 0.726195, 0.714294, 0.703106, 0.691827, 0.678662, 0.665324, 0.651950, 0.638720, 0.625826,\
        0.613475, 0.602047, 0.591628, 0.579786, 0.566289, 0.552407, 0.538512, 0.524721, 0.511214, 0.498867,\
        0.487717, 0.477152, 0.466888, 0.456888, 0.447126, 0.437577, 0.428241, 0.419125, 0.410174, 0.401304,\
        0.392656],\
        [0.286282, 0.294679, 0.302808, 0.310491, 0.318032, 0.325485, 0.332801, 0.340027, 0.347309, 0.354870,\
        0.362961, 0.371793, 0.381446, 0.391760, 0.402466, 0.413400, 0.425114, 0.437478, 0.450242, 0.463257,\
        0.476333, 0.489406, 0.502738, 0.516910, 0.531562, 0.546224, 0.560583, 0.574169, 0.586031, 0.594888,\
        0.601403, 0.606417, 0.610127, 0.612508, 0.613509, 0.613181, 0.611635, 0.609029, 0.605545, 0.601364,\
        0.596656, 0.591428, 0.585692, 0.579443, 0.572566, 0.564888, 0.556414, 0.547778, 0.540790, 0.538920,\
        0.547159, 0.569680, 0.604731, 0.645303, 0.686548, 0.728451, 0.765935, 0.796471, 0.817857, 0.832469,\
        0.843520, 0.852524, 0.860398, 0.867931, 0.875642, 0.883719, 0.892146, 0.900341, 0.905216, 0.908348,\
        0.909583, 0.908578, 0.906219, 0.903423, 0.899832, 0.895392, 0.889504, 0.881767, 0.872727, 0.862908,\
        0.852670, 0.842241, 0.831747, 0.821339, 0.811180, 0.800867, 0.789907, 0.776946, 0.760754, 0.743923,\
        0.727845, 0.713736, 0.703155, 0.692349, 0.681075, 0.668771, 0.655813, 0.642690, 0.629697, 0.617000,\
        0.604629, 0.593042, 0.582127, 0.570525, 0.557797, 0.544467, 0.531058, 0.517880, 0.504852, 0.492615,\
        0.481716, 0.471348, 0.461320, 0.451545, 0.441974, 0.432587, 0.423407, 0.414523, 0.405954, 0.397366,\
        0.388834],\
        [0.282533, 0.290609, 0.298330, 0.305625, 0.312972, 0.320158, 0.327125, 0.333978, 0.340911, 0.348152,\
        0.355912, 0.364339, 0.373464, 0.383174, 0.393256, 0.403728, 0.415436, 0.427601, 0.440138, 0.453046,\
        0.466154, 0.479399, 0.492987, 0.507369, 0.522304, 0.537095, 0.551722, 0.565836, 0.578618, 0.589350,\
        0.597926, 0.604359, 0.609085, 0.611920, 0.612981, 0.612472, 0.610632, 0.607771, 0.604234, 0.600493,\
        0.596946, 0.593307, 0.589413, 0.585195, 0.580481, 0.574905, 0.568105, 0.560333, 0.553151, 0.549936,\
        0.556242, 0.579094, 0.619061, 0.664972, 0.710244, 0.756477, 0.796307, 0.823220, 0.841590, 0.854701,\
        0.864201, 0.871106, 0.877097, 0.882925, 0.888928, 0.894829, 0.901211, 0.906717, 0.910114, 0.911116,\
        0.910146, 0.907513, 0.903948, 0.900141, 0.895973, 0.890454, 0.883636, 0.875351, 0.865835, 0.855503,\
        0.844744, 0.833822, 0.822782, 0.811622, 0.800847, 0.789760, 0.776873, 0.762278, 0.746152, 0.730002,\
        0.714783, 0.700203, 0.689979, 0.680257, 0.669459, 0.657762, 0.645361, 0.632639, 0.620013, 0.607794,\
        0.595827, 0.584207, 0.572920, 0.561321, 0.549058, 0.536303, 0.523495, 0.511174, 0.499700, 0.487641,\
        0.476496, 0.466136, 0.456240, 0.446639, 0.437237, 0.427979, 0.418863, 0.410004, 0.401697, 0.393493,\
        0.385157],\
        [0.278805, 0.286535, 0.293887, 0.301080, 0.308324, 0.315228, 0.321791, 0.328195, 0.334694, 0.341529,\
        0.348891, 0.356888, 0.365536, 0.374764, 0.384498, 0.394985, 0.406441, 0.417929, 0.430188, 0.443064,\
        0.456246, 0.469676, 0.483505, 0.497930, 0.512608, 0.526786, 0.541521, 0.556309, 0.570245, 0.582623,\
        0.593014, 0.601328, 0.607238, 0.610579, 0.611857, 0.611301, 0.609255, 0.606244, 0.602837, 0.599594,\
        0.597082, 0.594813, 0.592604, 0.590435, 0.588201, 0.585429, 0.581211, 0.575229, 0.568693, 0.564604,\
        0.568921, 0.591014, 0.642134, 0.696595, 0.744890, 0.787236, 0.823691, 0.847710, 0.863903, 0.876191,\
        0.884984, 0.889472, 0.893847, 0.897876, 0.902046, 0.905918, 0.909568, 0.912462, 0.913492, 0.912403,\
        0.909462, 0.905162, 0.900341, 0.895743, 0.890629, 0.884313, 0.876628, 0.867603, 0.857439, 0.846490,\
        0.835157, 0.823769, 0.812547, 0.800380, 0.788565, 0.776160, 0.762294, 0.746857, 0.730464, 0.714969,\
        0.702130, 0.689355, 0.678177, 0.668212, 0.657810, 0.646590, 0.634664, 0.622337, 0.609922, 0.598234,\
        0.586720, 0.575314, 0.563996, 0.552490, 0.540579, 0.528285, 0.515901, 0.503924, 0.492862, 0.482036,\
        0.471401, 0.461263, 0.451556, 0.442153, 0.432951, 0.423866, 0.414837, 0.405880, 0.397408, 0.389592,\
        0.381617],\
        [0.275196, 0.282738, 0.290040, 0.297197, 0.304152, 0.310672, 0.316716, 0.322575, 0.328570, 0.334959,\
        0.341913, 0.349519, 0.357792, 0.366711, 0.376293, 0.386650, 0.397397, 0.408239, 0.420610, 0.433528,\
        0.446698, 0.460106, 0.473873, 0.488069, 0.501588, 0.515181, 0.530415, 0.546070, 0.561160, 0.575022,\
        0.587247, 0.597416, 0.604321, 0.608400, 0.610269, 0.609851, 0.607663, 0.604578, 0.601492, 0.599039,\
        0.597222, 0.595928, 0.595006, 0.594506, 0.594540, 0.594892, 0.593966, 0.590818, 0.586709, 0.584636,\
        0.589685, 0.621843, 0.679446, 0.735270, 0.783270, 0.820263, 0.850187, 0.871326, 0.885200, 0.896942,\
        0.905689, 0.908726, 0.911470, 0.913780, 0.915707, 0.917255, 0.918189, 0.918175, 0.916640, 0.913356,\
        0.908540, 0.902876, 0.897031, 0.891327, 0.885050, 0.877671, 0.869011, 0.859108, 0.848139, 0.836414,\
        0.824401, 0.812378, 0.800613, 0.788859, 0.775669, 0.762131, 0.747677, 0.732099, 0.715538, 0.700142,\
        0.688361, 0.677552, 0.667040, 0.657014, 0.646783, 0.635876, 0.624275, 0.612414, 0.600461, 0.588781,\
        0.577636, 0.566501, 0.555351, 0.544089, 0.532599, 0.520804, 0.508796, 0.497317, 0.486660, 0.476405,\
        0.466346, 0.456586, 0.447159, 0.438017, 0.429086, 0.420286, 0.411544, 0.402837, 0.394414, 0.386401,\
        0.378448],\
        [0.271699, 0.279119, 0.286362, 0.293439, 0.300250, 0.306286, 0.311721, 0.316981, 0.322456, 0.328414,\
        0.335004, 0.342289, 0.350275, 0.358950, 0.368292, 0.378214, 0.388472, 0.399576, 0.411910, 0.424658,\
        0.437587, 0.450665, 0.463878, 0.477023, 0.489747, 0.503538, 0.519311, 0.535678, 0.551626, 0.566566,\
        0.580048, 0.591407, 0.599643, 0.605556, 0.608518, 0.608327, 0.605956, 0.602781, 0.600005, 0.598223,\
        0.597139, 0.596547, 0.596410, 0.596836, 0.598095, 0.600377, 0.602272, 0.602453, 0.602213, 0.605385,\
        0.616632, 0.654653, 0.711741, 0.764532, 0.813103, 0.851185, 0.878690, 0.897470, 0.909226, 0.919177,\
        0.926198, 0.929098, 0.930019, 0.930118, 0.929600, 0.928514, 0.926722, 0.923972, 0.919968, 0.914589,\
        0.908003, 0.900753, 0.894208, 0.887277, 0.879633, 0.870984, 0.861195, 0.850309, 0.838475, 0.825854,\
        0.812975, 0.800632, 0.788061, 0.775481, 0.762124, 0.748159, 0.733676, 0.718837, 0.703812, 0.688828,\
        0.676790, 0.666281, 0.656314, 0.646564, 0.636601, 0.625999, 0.614568, 0.602748, 0.591524, 0.580121,\
        0.568968, 0.557960, 0.547005, 0.536065, 0.525122, 0.514098, 0.502762, 0.491620, 0.481207, 0.471209,\
        0.461506, 0.452089, 0.442969, 0.434126, 0.425514, 0.417074, 0.408739, 0.400412, 0.391999, 0.383755,\
        0.375645],\
        [0.268246, 0.275511, 0.282592, 0.289459, 0.295940, 0.301660, 0.306567, 0.311278, 0.316303, 0.321901,\
        0.328207, 0.335252, 0.343025, 0.351507, 0.360666, 0.370444, 0.380846, 0.392105, 0.404141, 0.416487,\
        0.428999, 0.441602, 0.454251, 0.466890, 0.479704, 0.493511, 0.509118, 0.525633, 0.541845, 0.557344,\
        0.571736, 0.584460, 0.594996, 0.602927, 0.606863, 0.606827, 0.604179, 0.600938, 0.598719, 0.597511,\
        0.596919, 0.596748, 0.596959, 0.597616, 0.598888, 0.601386, 0.604501, 0.607138, 0.610780, 0.619329,\
        0.639033, 0.677216, 0.729580, 0.784656, 0.838829, 0.880251, 0.907811, 0.925955, 0.936815, 0.943958,\
        0.948118, 0.949235, 0.948212, 0.945965, 0.942925, 0.939233, 0.934846, 0.929621, 0.923408, 0.916192,\
        0.908278, 0.900389, 0.892065, 0.883559, 0.874427, 0.864396, 0.853374, 0.841437, 0.828818, 0.815708,\
        0.801438, 0.788576, 0.775491, 0.762127, 0.748456, 0.734450, 0.720182, 0.706069, 0.692421, 0.678826,\
        0.666665, 0.656007, 0.646205, 0.636764, 0.627199, 0.617061, 0.606059, 0.594338, 0.582943, 0.571729,\
        0.560634, 0.549704, 0.538913, 0.528254, 0.517813, 0.507654, 0.497328, 0.486498, 0.476159, 0.466327,\
        0.456874, 0.447740, 0.438907, 0.430360, 0.422060, 0.413952, 0.405976, 0.397877, 0.389548, 0.381233,\
        0.373006],\
        [0.264797, 0.271846, 0.278657, 0.285158, 0.291194, 0.296549, 0.301102, 0.305457, 0.310165, 0.315501,\
        0.321623, 0.328503, 0.336114, 0.344454, 0.353517, 0.363284, 0.373772, 0.385004, 0.396811, 0.408848,\
        0.420947, 0.433105, 0.445364, 0.457832, 0.470794, 0.484760, 0.500082, 0.515947, 0.531758, 0.547337,\
        0.562367, 0.576469, 0.589139, 0.599297, 0.604687, 0.605120, 0.602313, 0.599344, 0.597753, 0.596995,\
        0.596713, 0.596759, 0.597095, 0.597770, 0.598968, 0.601217, 0.604418, 0.608363, 0.614860, 0.627764,\
        0.652165, 0.692258, 0.746454, 0.807938, 0.870710, 0.913324, 0.938382, 0.954302, 0.963527, 0.968228,\
        0.969618, 0.968314, 0.965011, 0.960397, 0.954926, 0.948810, 0.942083, 0.934694, 0.926587, 0.917768,\
        0.908345, 0.898851, 0.889715, 0.879868, 0.869271, 0.857845, 0.845556, 0.832499, 0.818934, 0.805637,\
        0.792042, 0.777177, 0.762989, 0.748949, 0.735026, 0.721282, 0.707305, 0.693774, 0.680891, 0.668421,\
        0.656766, 0.646176, 0.636471, 0.627335, 0.618276, 0.608529, 0.598150, 0.586530, 0.574926, 0.563590,\
        0.552501, 0.541654, 0.531026, 0.520561, 0.510315, 0.500661, 0.490949, 0.480933, 0.471055, 0.461526,\
        0.452342, 0.443469, 0.434892, 0.426609, 0.418594, 0.410731, 0.402904, 0.395049, 0.386876, 0.378610,\
        0.370355],\
        [0.261370, 0.268171, 0.274668, 0.280775, 0.286368, 0.291320, 0.295661, 0.299837, 0.304252, 0.309401,\
        0.315441, 0.322177, 0.329626, 0.337837, 0.346827, 0.356576, 0.367051, 0.378178, 0.389787, 0.401598,\
        0.413363, 0.425150, 0.437119, 0.449449, 0.462380, 0.476188, 0.490904, 0.505898, 0.521069, 0.536362,\
        0.551599, 0.566541, 0.580613, 0.592577, 0.600368, 0.602675, 0.600171, 0.598096, 0.597046, 0.596621,\
        0.596567, 0.596760, 0.597181, 0.597909, 0.599130, 0.601071, 0.603879, 0.608121, 0.615943, 0.631560,\
        0.660112, 0.705901, 0.767954, 0.835764, 0.901461, 0.940712, 0.964336, 0.978906, 0.986764, 0.989574,\
        0.988661, 0.985005, 0.979410, 0.972537, 0.964848, 0.956593, 0.947855, 0.938636, 0.928959, 0.918919,\
        0.908566, 0.897953, 0.887267, 0.875930, 0.863882, 0.851135, 0.837655, 0.823486, 0.808992, 0.794294,\
        0.780126, 0.765258, 0.750457, 0.735942, 0.721738, 0.708618, 0.695640, 0.682410, 0.669972, 0.658101,\
        0.646871, 0.636413, 0.626748, 0.617796, 0.609435, 0.600149, 0.589471, 0.578244, 0.566767, 0.555454,\
        0.544422, 0.533711, 0.523334, 0.513191, 0.503046, 0.493514, 0.484393, 0.475094, 0.465802, 0.456693,\
        0.447831, 0.439221, 0.430865, 0.422789, 0.415036, 0.407545, 0.399840, 0.392033, 0.384006, 0.375823,\
        0.367597],\
        [0.258024, 0.264587, 0.270797, 0.276576, 0.281839, 0.286540, 0.290768, 0.294789, 0.298888, 0.304038,\
        0.309940, 0.316419, 0.323635, 0.331685, 0.340589, 0.350289, 0.360689, 0.371666, 0.383052, 0.394631,\
        0.406144, 0.417620, 0.429329, 0.441454, 0.454116, 0.467397, 0.481213, 0.495332, 0.509764, 0.524393,\
        0.539312, 0.554507, 0.569342, 0.582524, 0.592499, 0.597500, 0.597360, 0.596454, 0.596099, 0.596145,\
        0.596411, 0.596805, 0.597358, 0.598185, 0.599385, 0.601135, 0.603558, 0.607263, 0.614454, 0.630283,\
        0.662934, 0.717210, 0.786364, 0.851550, 0.909877, 0.953157, 0.980719, 0.996941, 1.004754, 1.006430,\
        1.003755, 0.998082, 0.990432, 0.981557, 0.971966, 0.961939, 0.951570, 0.940853, 0.929820, 0.918732,\
        0.908012, 0.896799, 0.884224, 0.871230, 0.857810, 0.843926, 0.829555, 0.814316, 0.798883, 0.783403,\
        0.767938, 0.752674, 0.737754, 0.723267, 0.709015, 0.695484, 0.683654, 0.671343, 0.659409, 0.647994,\
        0.637063, 0.626657, 0.616913, 0.607824, 0.599387, 0.590936, 0.580480, 0.569447, 0.558250, 0.547143,\
        0.536284, 0.525762, 0.515687, 0.506165, 0.496628, 0.487176, 0.478185, 0.469324, 0.460526, 0.451848,\
        0.443336, 0.434998, 0.426828, 0.418855, 0.411210, 0.404011, 0.396891, 0.388993, 0.380969, 0.372861,\
        0.364696],\
        [0.254829, 0.261200, 0.267194, 0.272751, 0.277822, 0.282407, 0.286606, 0.290638, 0.294898, 0.299854,\
        0.305219, 0.311299, 0.318177, 0.326032, 0.334848, 0.344481, 0.354773, 0.365564, 0.376687, 0.387963,\
        0.399224, 0.410446, 0.421930, 0.433814, 0.446074, 0.458691, 0.471614, 0.484786, 0.498206, 0.511698,\
        0.525829, 0.540907, 0.556231, 0.570261, 0.581462, 0.588617, 0.591915, 0.593355, 0.594363, 0.595290,\
        0.596134, 0.596883, 0.597653, 0.598681, 0.600109, 0.601844, 0.603924, 0.606555, 0.610944, 0.621535,\
        0.652177, 0.714488, 0.784234, 0.850696, 0.909007, 0.956928, 0.989817, 1.008670, 1.017023, 1.017999,\
        1.013957, 1.006628, 0.997258, 0.986729, 0.975623, 0.964255, 0.952705, 0.940874, 0.928631, 0.916090,\
        0.904002, 0.892213, 0.878941, 0.864961, 0.850508, 0.835646, 0.820782, 0.806012, 0.788856, 0.771896,\
        0.755327, 0.739615, 0.724875, 0.710715, 0.697485, 0.684190, 0.672016, 0.660322, 0.648993, 0.638092,\
        0.627482, 0.617049, 0.607133, 0.598011, 0.589288, 0.580502, 0.570718, 0.560204, 0.549423, 0.538658,\
        0.528087, 0.517807, 0.507903, 0.498691, 0.489849, 0.480917, 0.472215, 0.463705, 0.455315, 0.447039,\
        0.438886, 0.430848, 0.422885, 0.414951, 0.407145, 0.399926, 0.392903, 0.385487, 0.377696, 0.369719,\
        0.361643],\
        [0.251844, 0.258088, 0.263955, 0.269400, 0.274401, 0.278978, 0.283228, 0.287345, 0.291602, 0.296188,\
        0.301248, 0.306824, 0.313235, 0.320937, 0.329709, 0.339279, 0.349441, 0.360024, 0.370867, 0.381813,\
        0.392746, 0.403714, 0.415047, 0.426714, 0.438512, 0.450426, 0.462452, 0.474558, 0.486701, 0.499039,\
        0.511879, 0.526875, 0.542699, 0.557387, 0.569681, 0.578806, 0.584874, 0.588853, 0.591714, 0.593949,\
        0.595702, 0.597016, 0.598086, 0.599381, 0.601187, 0.603179, 0.605095, 0.606785, 0.607991, 0.608131,\
        0.618981, 0.693128, 0.776100, 0.848461, 0.908785, 0.961008, 0.996030, 1.015656, 1.023828, 1.023974,\
        1.018726, 1.010029, 0.999285, 0.987476, 0.975264, 0.963022, 0.950816, 0.938420, 0.925478, 0.911587,\
        0.897548, 0.884871, 0.871262, 0.856744, 0.841632, 0.825944, 0.809520, 0.794646, 0.777580, 0.759336,\
        0.741794, 0.725707, 0.711922, 0.698217, 0.685258, 0.672788, 0.660835, 0.649462, 0.638631, 0.628340,\
        0.618298, 0.607948, 0.597702, 0.588478, 0.579516, 0.570431, 0.560856, 0.550784, 0.540455, 0.530104,\
        0.519926, 0.510051, 0.500466, 0.491477, 0.482996, 0.474609, 0.466332, 0.458195, 0.450181, 0.442280,\
        0.434489, 0.426800, 0.419170, 0.411485, 0.403568, 0.396155, 0.389226, 0.381929, 0.374291, 0.366434,\
        0.358450],\
        [0.249101, 0.255290, 0.261117, 0.266552, 0.271584, 0.276235, 0.280583, 0.284760, 0.288939, 0.293301,\
        0.298018, 0.302682, 0.308816, 0.316580, 0.325360, 0.334845, 0.344841, 0.355197, 0.365775, 0.376447,\
        0.387116, 0.397793, 0.409096, 0.420446, 0.431681, 0.442899, 0.454112, 0.465287, 0.476397, 0.487487,\
        0.499055, 0.514513, 0.530348, 0.545340, 0.558525, 0.569244, 0.577451, 0.583631, 0.588399, 0.592183,\
        0.595154, 0.597312, 0.598800, 0.600410, 0.602616, 0.604776, 0.606506, 0.607369, 0.606711, 0.600597,\
        0.591291, 0.653271, 0.751172, 0.841814, 0.915798, 0.967174, 1.000033, 1.018155, 1.025145, 1.024150,\
        1.017749, 1.007930, 0.996147, 0.983418, 0.970468, 0.957783, 0.945473, 0.933107, 0.920219, 0.906636,\
        0.891497, 0.876495, 0.861641, 0.846443, 0.830825, 0.815293, 0.798546, 0.780945, 0.763377, 0.745167,\
        0.727371, 0.710788, 0.697452, 0.685725, 0.673189, 0.661214, 0.649681, 0.638613, 0.628132, 0.618417,\
        0.609357, 0.599533, 0.589150, 0.579380, 0.570029, 0.560724, 0.551220, 0.541457, 0.531544, 0.521624,\
        0.511854, 0.502479, 0.493440, 0.484748, 0.476482, 0.468445, 0.460549, 0.452775, 0.445112, 0.437551,\
        0.430100, 0.422769, 0.415566, 0.408464, 0.401309, 0.393578, 0.386035, 0.378516, 0.370855, 0.363048,\
        0.355134],\
        [0.246610, 0.252810, 0.258678, 0.264190, 0.269338, 0.274139, 0.278634, 0.282895, 0.287007, 0.291048,\
        0.295031, 0.299206, 0.305568, 0.313378, 0.322043, 0.331332, 0.341083, 0.351171, 0.361494, 0.371973,\
        0.382583, 0.393424, 0.404415, 0.415063, 0.425660, 0.436281, 0.446895, 0.457481, 0.468087, 0.478987,\
        0.491187, 0.505185, 0.520003, 0.534929, 0.548714, 0.560565, 0.570319, 0.578225, 0.584678, 0.590015,\
        0.594412, 0.597813, 0.600040, 0.601988, 0.604128, 0.606145, 0.607445, 0.606820, 0.602075, 0.590265,\
        0.582497, 0.631059, 0.722444, 0.818164, 0.911698, 0.966802, 0.998311, 1.014881, 1.020514, 1.018324,\
        1.010888, 1.000213, 0.987745, 0.974444, 0.961014, 0.948152, 0.936277, 0.924480, 0.911381, 0.897464,\
        0.881721, 0.865381, 0.849438, 0.833641, 0.817614, 0.800591, 0.784991, 0.766369, 0.747901, 0.729938,\
        0.712685, 0.696867, 0.683417, 0.672012, 0.660753, 0.649595, 0.638566, 0.627746, 0.617428, 0.607763,\
        0.599507, 0.590314, 0.580162, 0.570241, 0.560669, 0.551268, 0.541855, 0.532354, 0.522834, 0.513390,\
        0.504010, 0.495023, 0.486469, 0.478187, 0.470204, 0.462461, 0.454888, 0.447439, 0.440089, 0.432823,\
        0.425648, 0.418590, 0.411685, 0.404932, 0.398029, 0.390489, 0.382757, 0.375065, 0.367345, 0.359556,\
        0.351699],\
        [0.244352, 0.250624, 0.256599, 0.262261, 0.267602, 0.272628, 0.277351, 0.281793, 0.285976, 0.289940,\
        0.293855, 0.298415, 0.304301, 0.311606, 0.319883, 0.328798, 0.338183, 0.347930, 0.357958, 0.368217,\
        0.378691, 0.389375, 0.399957, 0.410117, 0.420325, 0.430578, 0.440862, 0.451213, 0.461751, 0.472759,\
        0.484637, 0.497492, 0.511582, 0.526554, 0.540623, 0.553078, 0.563793, 0.572903, 0.580656, 0.587297,\
        0.592998, 0.597784, 0.601050, 0.603028, 0.604929, 0.606750, 0.607850, 0.606599, 0.599110, 0.581006,\
        0.573057, 0.624428, 0.723096, 0.810694, 0.898591, 0.956137, 0.988960, 1.005149, 1.009785, 1.006545,\
        0.998251, 0.987016, 0.974277, 0.960824, 0.947066, 0.933872, 0.922717, 0.913090, 0.898815, 0.883496,\
        0.866926, 0.850085, 0.833788, 0.817542, 0.801657, 0.784505, 0.766891, 0.748889, 0.730877, 0.713825,\
        0.697805, 0.682958, 0.670066, 0.658828, 0.648377, 0.638132, 0.627713, 0.617030, 0.606812, 0.597378,\
        0.588589, 0.579851, 0.570364, 0.560727, 0.551246, 0.541950, 0.532755, 0.523533, 0.514346, 0.505522,\
        0.496663, 0.488008, 0.479742, 0.471777, 0.464079, 0.456614, 0.449331, 0.442177, 0.435108, 0.428087,\
        0.421107, 0.414196, 0.407387, 0.400638, 0.393822, 0.386571, 0.379024, 0.371374, 0.363675, 0.355929,\
        0.348137],\
        [0.242293, 0.248681, 0.254815, 0.260684, 0.266282, 0.271607, 0.276657, 0.281424, 0.285897, 0.290086,\
        0.294132, 0.298469, 0.304137, 0.311043, 0.318778, 0.327158, 0.336054, 0.345369, 0.355026, 0.364963,\
        0.375114, 0.385356, 0.395458, 0.405521, 0.415613, 0.425701, 0.435861, 0.446205, 0.456887, 0.468090,\
        0.479922, 0.492298, 0.506054, 0.520650, 0.534329, 0.546788, 0.557913, 0.567730, 0.576341, 0.583862,\
        0.590362, 0.595761, 0.599681, 0.602252, 0.604148, 0.606099, 0.607670, 0.607621, 0.602935, 0.588990,\
        0.576935, 0.639623, 0.739340, 0.816286, 0.888053, 0.942611, 0.974878, 0.990176, 0.993647, 0.989320,\
        0.980279, 0.968738, 0.956185, 0.943253, 0.929529, 0.915439, 0.903983, 0.896724, 0.883685, 0.865623,\
        0.847863, 0.830538, 0.814256, 0.798444, 0.781172, 0.764250, 0.747053, 0.729826, 0.712697, 0.696772,\
        0.682232, 0.668616, 0.656460, 0.645778, 0.636128, 0.626922, 0.617422, 0.606910, 0.596420, 0.587181,\
        0.578237, 0.569366, 0.560238, 0.550954, 0.541726, 0.532699, 0.523923, 0.515185, 0.506081, 0.497667,\
        0.489459, 0.481206, 0.473209, 0.465499, 0.458057, 0.450854, 0.443844, 0.436971, 0.430169, 0.423369,\
        0.416526, 0.409667, 0.402891, 0.396169, 0.389364, 0.382287, 0.374938, 0.367421, 0.359814, 0.352149,\
        0.344442],\
        [0.240383, 0.246916, 0.253242, 0.259354, 0.265252, 0.270932, 0.276390, 0.281610, 0.286564, 0.291225,\
        0.295614, 0.300027, 0.305223, 0.311436, 0.318471, 0.326200, 0.334520, 0.343334, 0.352555, 0.362102,\
        0.371880, 0.381769, 0.391666, 0.401574, 0.411509, 0.421495, 0.431609, 0.442038, 0.452964, 0.464533,\
        0.476860, 0.490073, 0.503743, 0.516658, 0.529326, 0.541368, 0.552514, 0.562661, 0.571770, 0.579793,\
        0.586652, 0.592222, 0.596424, 0.599297, 0.601342, 0.603902, 0.606547, 0.609120, 0.611373, 0.611232,\
        0.605406, 0.688068, 0.766244, 0.825965, 0.886986, 0.931723, 0.959130, 0.971855, 0.973311, 0.967560,\
        0.957789, 0.946090, 0.933968, 0.922392, 0.910025, 0.895159, 0.882901, 0.872774, 0.859595, 0.842405,\
        0.824860, 0.807951, 0.790992, 0.775300, 0.758550, 0.741833, 0.725971, 0.710606, 0.694744, 0.680101,\
        0.666562, 0.653948, 0.642527, 0.632501, 0.623666, 0.615522, 0.607481, 0.597589, 0.586987, 0.577220,\
        0.568036, 0.559087, 0.550145, 0.541151, 0.532190, 0.523444, 0.515175, 0.507523, 0.499123, 0.490481,\
        0.482431, 0.474503, 0.466770, 0.459295, 0.452087, 0.445129, 0.438383, 0.431790, 0.425268, 0.418715,\
        0.412017, 0.405146, 0.398367, 0.391687, 0.384890, 0.377887, 0.370666, 0.363279, 0.355785, 0.348225,\
        0.340622],\
        [0.238564, 0.245256, 0.251784, 0.258151, 0.264359, 0.270412, 0.276309, 0.282036, 0.287563, 0.292833,\
        0.297729, 0.302181, 0.306852, 0.312292, 0.318573, 0.325625, 0.333349, 0.341641, 0.350404, 0.359545,\
        0.368970, 0.378586, 0.388304, 0.398049, 0.407890, 0.417729, 0.427742, 0.438243, 0.449405, 0.461309,\
        0.473972, 0.487240, 0.500312, 0.512654, 0.524692, 0.536311, 0.547314, 0.557610, 0.567048, 0.575409,\
        0.582453, 0.587988, 0.591972, 0.594640, 0.596899, 0.599982, 0.603911, 0.609606, 0.620410, 0.636378,\
        0.666365, 0.729076, 0.784198, 0.832653, 0.880285, 0.918394, 0.942509, 0.951637, 0.949940, 0.942363,\
        0.932018, 0.920417, 0.908487, 0.898064, 0.887205, 0.873345, 0.859828, 0.846976, 0.832712, 0.816102,\
        0.799043, 0.783500, 0.767035, 0.751114, 0.735284, 0.719027, 0.704630, 0.691561, 0.677679, 0.664171,\
        0.651430, 0.639424, 0.628442, 0.618912, 0.610759, 0.603193, 0.595766, 0.586940, 0.576798, 0.567003,\
        0.557763, 0.548893, 0.540187, 0.531501, 0.522801, 0.514237, 0.506152, 0.499102, 0.491973, 0.483674,\
        0.475635, 0.467884, 0.460372, 0.453113, 0.446120, 0.439390, 0.432897, 0.426583, 0.420360, 0.414119,\
        0.407689, 0.400891, 0.394050, 0.387291, 0.380446, 0.373454, 0.366303, 0.359015, 0.351631, 0.344183,\
        0.336695],\
        [0.236777, 0.243624, 0.250346, 0.256951, 0.263447, 0.269842, 0.276138, 0.282325, 0.288363, 0.294151,\
        0.299488, 0.303846, 0.308119, 0.312978, 0.318642, 0.325112, 0.332303, 0.340112, 0.348434, 0.357177,\
        0.366259, 0.375615, 0.385193, 0.394928, 0.404631, 0.414038, 0.423838, 0.434382, 0.445720, 0.457812,\
        0.470515, 0.483450, 0.495972, 0.508053, 0.519979, 0.531193, 0.542005, 0.552467, 0.562307, 0.571107,\
        0.578432, 0.583967, 0.587646, 0.589822, 0.591411, 0.593777, 0.598179, 0.607055, 0.628247, 0.659191,\
        0.699665, 0.746880, 0.790746, 0.833308, 0.873557, 0.906583, 0.926888, 0.930383, 0.924306, 0.914956,\
        0.904600, 0.893778, 0.882589, 0.871843, 0.860711, 0.848065, 0.834359, 0.820316, 0.805778, 0.790359,\
        0.774426, 0.758970, 0.743048, 0.727587, 0.714453, 0.698899, 0.685350, 0.673194, 0.661018, 0.648912,\
        0.637069, 0.625492, 0.614618, 0.605224, 0.597622, 0.590317, 0.582842, 0.574540, 0.565383, 0.556132,\
        0.547212, 0.538665, 0.530369, 0.522135, 0.513796, 0.505423, 0.497608, 0.490601, 0.483761, 0.476340,\
        0.468744, 0.461266, 0.453980, 0.446923, 0.440123, 0.433596, 0.427335, 0.421285, 0.415343, 0.409409,\
        0.403375, 0.396772, 0.389841, 0.382932, 0.376004, 0.368996, 0.361884, 0.354674, 0.347388, 0.340052,\
        0.332686],\
        [0.234971, 0.241954, 0.248843, 0.255648, 0.262377, 0.269039, 0.275636, 0.282153, 0.288532, 0.294594,\
        0.299912, 0.304397, 0.308501, 0.313044, 0.318332, 0.324403, 0.331190, 0.338597, 0.346525, 0.354885,\
        0.363605, 0.372632, 0.381936, 0.391481, 0.400827, 0.409775, 0.419517, 0.430171, 0.441634, 0.453796,\
        0.466482, 0.479370, 0.491984, 0.504216, 0.515199, 0.525564, 0.536220, 0.547088, 0.557650, 0.567246,\
        0.575221, 0.581082, 0.584687, 0.586378, 0.586983, 0.587780, 0.591252, 0.605029, 0.634363, 0.672094,\
        0.713619, 0.755216, 0.794545, 0.829842, 0.864540, 0.894972, 0.910907, 0.907034, 0.897169, 0.887189,\
        0.877236, 0.867210, 0.856434, 0.844776, 0.832828, 0.821025, 0.807428, 0.793335, 0.779447, 0.765516,\
        0.751234, 0.736524, 0.720994, 0.705607, 0.694558, 0.682690, 0.668840, 0.656669, 0.645341, 0.634493,\
        0.623768, 0.612689, 0.601842, 0.592572, 0.584837, 0.577515, 0.569982, 0.561943, 0.553425, 0.544795,\
        0.536393, 0.528355, 0.520644, 0.513072, 0.505308, 0.497334, 0.489758, 0.482768, 0.475956, 0.468959,\
        0.461802, 0.454647, 0.447602, 0.440734, 0.434094, 0.427729, 0.421660, 0.415855, 0.410160, 0.404346,\
        0.398458, 0.392112, 0.385326, 0.378428, 0.371486, 0.364487, 0.357417, 0.350279, 0.343087, 0.335862,\
        0.328620],\
        [0.233102, 0.240194, 0.247213, 0.254165, 0.261055, 0.267886, 0.274649, 0.281321, 0.287839, 0.294069,\
        0.299689, 0.303801, 0.307795, 0.312298, 0.317478, 0.323363, 0.329901, 0.337011, 0.344606, 0.352605,\
        0.360938, 0.369540, 0.378336, 0.387185, 0.395859, 0.404799, 0.414802, 0.425604, 0.437087, 0.449164,\
        0.461707, 0.474557, 0.487548, 0.499619, 0.508948, 0.518620, 0.529570, 0.541327, 0.553110, 0.564059,\
        0.573277, 0.580039, 0.584103, 0.585910, 0.586512, 0.587430, 0.591721, 0.608463, 0.642278, 0.682291,\
        0.720130, 0.755564, 0.787418, 0.817291, 0.846528, 0.870482, 0.882907, 0.879057, 0.869410, 0.859166,\
        0.849114, 0.839845, 0.829276, 0.816713, 0.803170, 0.792511, 0.779690, 0.766320, 0.753646, 0.741655,\
        0.729257, 0.716145, 0.701929, 0.687534, 0.675729, 0.664611, 0.652603, 0.641076, 0.630423, 0.620747,\
        0.611504, 0.601346, 0.590694, 0.581323, 0.573089, 0.565366, 0.557638, 0.549678, 0.541518, 0.533373,\
        0.525489, 0.518008, 0.510915, 0.504113, 0.496994, 0.489522, 0.482250, 0.475331, 0.468592, 0.461828,\
        0.454985, 0.448117, 0.441299, 0.434595, 0.428072, 0.421809, 0.415871, 0.410258, 0.404909, 0.399182,\
        0.393246, 0.387041, 0.380483, 0.373720, 0.366853, 0.359912, 0.352908, 0.345850, 0.338754, 0.331641,\
        0.324527],\
        [0.231144, 0.238313, 0.245419, 0.252463, 0.259442, 0.266343, 0.273144, 0.279797, 0.286213, 0.292218,\
        0.297416, 0.301698, 0.306066, 0.310810, 0.316108, 0.321993, 0.328423, 0.335337, 0.342667, 0.350344,\
        0.358301, 0.366464, 0.374746, 0.383048, 0.391375, 0.400263, 0.410228, 0.420908, 0.432187, 0.443931,\
        0.455992, 0.468175, 0.480084, 0.490797, 0.499978, 0.510027, 0.521993, 0.535114, 0.548601, 0.561530,\
        0.572740, 0.581140, 0.586306, 0.588979, 0.590881, 0.594420, 0.602198, 0.621331, 0.652532, 0.688527,\
        0.722009, 0.750930, 0.776226, 0.799810, 0.820416, 0.835841, 0.842524, 0.842609, 0.838047, 0.828882,\
        0.817991, 0.810143, 0.799627, 0.786723, 0.773660, 0.762569, 0.751298, 0.739294, 0.728033, 0.718745,\
        0.708381, 0.696437, 0.683942, 0.670775, 0.658761, 0.647583, 0.636670, 0.626108, 0.615906, 0.607090,\
        0.599148, 0.589989, 0.580017, 0.570663, 0.562104, 0.554004, 0.546020, 0.538010, 0.530034, 0.522228,\
        0.514772, 0.507821, 0.501252, 0.494942, 0.488459, 0.481636, 0.474809, 0.468129, 0.461555, 0.454993,\
        0.448396, 0.441772, 0.435160, 0.428602, 0.422151, 0.415918, 0.410052, 0.404486, 0.399272, 0.393923,\
        0.388017, 0.381876, 0.375480, 0.368875, 0.362130, 0.355289, 0.348377, 0.341414, 0.334421, 0.327422,\
        0.320440],\
        [0.229082, 0.236299, 0.243455, 0.250544, 0.257552, 0.264453, 0.271205, 0.277737, 0.283935, 0.289625,\
        0.294632, 0.299185, 0.303837, 0.308869, 0.314413, 0.320412, 0.326826, 0.333618, 0.340743, 0.348152,\
        0.355784, 0.363581, 0.371485, 0.379480, 0.387693, 0.396465, 0.405987, 0.416279, 0.427136, 0.438305,\
        0.449587, 0.460748, 0.471434, 0.481269, 0.490559, 0.501005, 0.514005, 0.528391, 0.543792, 0.559248,\
        0.573273, 0.584156, 0.590905, 0.594825, 0.598109, 0.604616, 0.615841, 0.635145, 0.662406, 0.692573,\
        0.720121, 0.743080, 0.761852, 0.777673, 0.791225, 0.798843, 0.800413, 0.802942, 0.803438, 0.796698,\
        0.784752, 0.775832, 0.765589, 0.753896, 0.742834, 0.733595, 0.724074, 0.713560, 0.703106, 0.696304,\
        0.688322, 0.677433, 0.665970, 0.654138, 0.642624, 0.631666, 0.621482, 0.612334, 0.602460, 0.593803,\
        0.586255, 0.577963, 0.569017, 0.560142, 0.551633, 0.543387, 0.535221, 0.527103, 0.519235, 0.511747,\
        0.504553, 0.498151, 0.492070, 0.486134, 0.480096, 0.473842, 0.467483, 0.461127, 0.454791, 0.448448,\
        0.442075, 0.435675, 0.429270, 0.422878, 0.416501, 0.410198, 0.404363, 0.399007, 0.393687, 0.388376,\
        0.382691, 0.376690, 0.370439, 0.363985, 0.357378, 0.350661, 0.343863, 0.337010, 0.330127, 0.323245,\
        0.316396],\
        [0.226918, 0.234158, 0.241336, 0.248438, 0.255441, 0.262308, 0.268983, 0.275390, 0.281427, 0.286983,\
        0.292025, 0.296728, 0.301477, 0.306842, 0.312670, 0.318794, 0.325215, 0.331923, 0.338891, 0.346083,\
        0.353461, 0.360984, 0.368632, 0.376424, 0.384465, 0.392918, 0.401981, 0.411888, 0.422193, 0.432628,\
        0.443008, 0.453156, 0.462909, 0.472282, 0.481827, 0.492745, 0.505690, 0.520776, 0.537994, 0.556232,\
        0.573785, 0.588223, 0.597000, 0.601726, 0.606546, 0.614020, 0.626450, 0.645045, 0.669086, 0.694724,\
        0.716513, 0.732246, 0.743941, 0.753148, 0.760119, 0.763427, 0.764232, 0.765424, 0.765454, 0.761072,\
        0.751984, 0.740974, 0.731238, 0.721739, 0.712410, 0.706556, 0.699237, 0.690527, 0.682043, 0.675231,\
        0.668069, 0.658958, 0.648726, 0.638014, 0.627313, 0.616813, 0.606796, 0.599072, 0.590882, 0.582083,\
        0.574216, 0.566304, 0.558095, 0.549823, 0.541655, 0.533542, 0.525357, 0.517117, 0.509229, 0.502367,\
        0.495498, 0.489340, 0.483581, 0.477924, 0.472213, 0.466366, 0.460403, 0.454372, 0.448300, 0.442187,\
        0.436034, 0.429851, 0.423666, 0.417510, 0.411366, 0.405035, 0.398940, 0.393749, 0.388482, 0.383090,\
        0.377463, 0.371571, 0.365445, 0.359128, 0.352661, 0.346080, 0.339415, 0.332690, 0.325927, 0.319162,\
        0.312446],\
        [0.224663, 0.231909, 0.239092, 0.246192, 0.253180, 0.260012, 0.266631, 0.272970, 0.278953, 0.284529,\
        0.289719, 0.294652, 0.299625, 0.305252, 0.311135, 0.317281, 0.323682, 0.330314, 0.337152, 0.344174,\
        0.351355, 0.358680, 0.366148, 0.373790, 0.381674, 0.389904, 0.398648, 0.407992, 0.417602, 0.427209,\
        0.436673, 0.445911, 0.454916, 0.463874, 0.473298, 0.483940, 0.496447, 0.511754, 0.530436, 0.550947,\
        0.571914, 0.590899, 0.602649, 0.607494, 0.612301, 0.619750, 0.631619, 0.648505, 0.669420, 0.691477,\
        0.708489, 0.716893, 0.722589, 0.726657, 0.728949, 0.730450, 0.731498, 0.730824, 0.728745, 0.726810,\
        0.722445, 0.710988, 0.701913, 0.695160, 0.686939, 0.682184, 0.676750, 0.669940, 0.663046, 0.656512,\
        0.649608, 0.641543, 0.632444, 0.622745, 0.612963, 0.603515, 0.593983, 0.586102, 0.578635, 0.570725,\
        0.562964, 0.555295, 0.547585, 0.539886, 0.532239, 0.524533, 0.516598, 0.508399, 0.500364, 0.493667,\
        0.487469, 0.481496, 0.475856, 0.470364, 0.464877, 0.459317, 0.453661, 0.447918, 0.442098, 0.436210,\
        0.430266, 0.424291, 0.418314, 0.412397, 0.406720, 0.401021, 0.394783, 0.389092, 0.383616, 0.378101,\
        0.372443, 0.366597, 0.360563, 0.354365, 0.348032, 0.341597, 0.335083, 0.328509, 0.321888, 0.315246,\
        0.308653],\
        [0.222336, 0.229578, 0.236759, 0.243856, 0.250836, 0.257655, 0.264261, 0.270599, 0.276622, 0.282320,\
        0.287748, 0.293078, 0.298570, 0.304024, 0.309790, 0.315906, 0.322269, 0.328825, 0.335548, 0.342425,\
        0.349450, 0.356622, 0.363955, 0.371481, 0.379255, 0.387346, 0.395809, 0.404591, 0.413451, 0.422204,\
        0.430793, 0.439214, 0.447543, 0.456005, 0.465023, 0.475165, 0.487118, 0.502276, 0.520816, 0.541928,\
        0.564480, 0.585608, 0.599716, 0.606538, 0.612156, 0.619217, 0.629884, 0.644342, 0.661411, 0.678687,\
        0.690943, 0.695447, 0.698132, 0.699631, 0.699433, 0.700276, 0.702260, 0.700533, 0.696946, 0.697343,\
        0.698066, 0.691567, 0.680979, 0.673720, 0.666968, 0.661754, 0.656866, 0.651438, 0.645646, 0.639608,\
        0.633046, 0.625602, 0.617344, 0.608597, 0.599405, 0.590913, 0.582418, 0.574549, 0.567138, 0.559706,\
        0.552264, 0.544869, 0.537547, 0.530370, 0.523365, 0.516318, 0.508895, 0.501174, 0.493358, 0.486449,\
        0.480235, 0.474382, 0.468807, 0.463406, 0.458067, 0.452709, 0.447290, 0.441788, 0.436196, 0.430512,\
        0.424755, 0.418965, 0.413185, 0.407380, 0.401695, 0.396579, 0.390642, 0.384719, 0.379005, 0.373347,\
        0.367630, 0.361796, 0.355828, 0.349732, 0.343529, 0.337245, 0.330902, 0.324518, 0.318092, 0.311606,\
        0.305119],\
        [0.219960, 0.227193, 0.234374, 0.241477, 0.248467, 0.255303, 0.261940, 0.268335, 0.274460, 0.280318,\
        0.285960, 0.291483, 0.296968, 0.302568, 0.308497, 0.314654, 0.320987, 0.327461, 0.334069, 0.340812,\
        0.347697, 0.354737, 0.361950, 0.369368, 0.377031, 0.384972, 0.393183, 0.401511, 0.409656, 0.417603,\
        0.425407, 0.433108, 0.440811, 0.448735, 0.457240, 0.466834, 0.478231, 0.492200, 0.508675, 0.529122,\
        0.551286, 0.571783, 0.587250, 0.597666, 0.605041, 0.611665, 0.621446, 0.633232, 0.645743, 0.657547,\
        0.665944, 0.670030, 0.672008, 0.673012, 0.673407, 0.674441, 0.675698, 0.674806, 0.673122, 0.673508,\
        0.673886, 0.669976, 0.662648, 0.655543, 0.649412, 0.644255, 0.639590, 0.634889, 0.629834, 0.624242,\
        0.618075, 0.611106, 0.603334, 0.595459, 0.587174, 0.579082, 0.571264, 0.563749, 0.556495, 0.549316,\
        0.542136, 0.534982, 0.527940, 0.521164, 0.514814, 0.508709, 0.501792, 0.494415, 0.486997, 0.480058,\
        0.473731, 0.467855, 0.462297, 0.456947, 0.451708, 0.446502, 0.441270, 0.435974, 0.430584, 0.425083,\
        0.419476, 0.413820, 0.408244, 0.402762, 0.397010, 0.391594, 0.386001, 0.380220, 0.374461, 0.368732,\
        0.362981, 0.357160, 0.351250, 0.345249, 0.339171, 0.333040, 0.326882, 0.320725, 0.314585, 0.308421,\
        0.302058],\
        [0.217557, 0.224783, 0.231970, 0.239092, 0.246113, 0.252996, 0.259700, 0.266193, 0.272457, 0.278493,\
        0.284330, 0.290013, 0.295618, 0.301358, 0.307369, 0.313553, 0.319833, 0.326205, 0.332684, 0.339288,\
        0.346034, 0.352940, 0.360026, 0.367317, 0.374831, 0.382573, 0.390496, 0.398422, 0.405982, 0.413289,\
        0.420463, 0.427570, 0.434721, 0.442116, 0.450060, 0.458955, 0.469259, 0.481341, 0.496000, 0.514724,\
        0.535480, 0.554976, 0.570941, 0.582996, 0.591632, 0.598697, 0.608708, 0.617493, 0.625564, 0.633246,\
        0.639284, 0.643111, 0.645599, 0.647748, 0.649820, 0.651631, 0.652721, 0.652712, 0.652404, 0.652658,\
        0.652356, 0.649613, 0.644525, 0.638753, 0.633371, 0.628554, 0.624109, 0.619930, 0.615460, 0.610098,\
        0.604242, 0.597854, 0.590487, 0.583056, 0.575515, 0.568007, 0.560677, 0.553539, 0.546544, 0.539595,\
        0.532629, 0.525662, 0.518781, 0.512185, 0.506181, 0.501066, 0.494967, 0.487764, 0.480655, 0.473888,\
        0.467587, 0.461711, 0.456168, 0.450861, 0.445704, 0.440620, 0.435547, 0.430435, 0.425235, 0.419907,\
        0.414425, 0.408812, 0.403239, 0.397979, 0.392483, 0.386907, 0.381315, 0.375636, 0.369921, 0.364198,\
        0.358456, 0.352671, 0.346826, 0.340921, 0.334966, 0.328984, 0.323007, 0.317074, 0.311239, 0.305593,\
        0.300104],\
        [0.215154, 0.222375, 0.229576, 0.236731, 0.243803, 0.250754, 0.257551, 0.264168, 0.270591, 0.276823,\
        0.282879, 0.288786, 0.294584, 0.300402, 0.306484, 0.312620, 0.318782, 0.325011, 0.331337, 0.337789,\
        0.344386, 0.351148, 0.358091, 0.365224, 0.372548, 0.380034, 0.387603, 0.395084, 0.402275, 0.409168,\
        0.415900, 0.422558, 0.429254, 0.436172, 0.443574, 0.451779, 0.461142, 0.472071, 0.485221, 0.501362,\
        0.519998, 0.537868, 0.553186, 0.565636, 0.575562, 0.584440, 0.593401, 0.599170, 0.603979, 0.609091,\
        0.613745, 0.617380, 0.620588, 0.624443, 0.628693, 0.631577, 0.632849, 0.633258, 0.633565, 0.633905,\
        0.633510, 0.631370, 0.627470, 0.622835, 0.618342, 0.614084, 0.609725, 0.605953, 0.602229, 0.596882,\
        0.591147, 0.585037, 0.578344, 0.571407, 0.564416, 0.557460, 0.550608, 0.543866, 0.537190, 0.530508,\
        0.523761, 0.516974, 0.510226, 0.503676, 0.497834, 0.492720, 0.487226, 0.480804, 0.474168, 0.467702,\
        0.461559, 0.455762, 0.450272, 0.445024, 0.439947, 0.434974, 0.430044, 0.425102, 0.420092, 0.414950,\
        0.409616, 0.404077, 0.398438, 0.393088, 0.387724, 0.382229, 0.376668, 0.371053, 0.365401, 0.359729,\
        0.354036, 0.348314, 0.342552, 0.336751, 0.330918, 0.325078, 0.319265, 0.313515, 0.307847, 0.302354,\
        0.297007],\
        [0.212776, 0.219995, 0.227221, 0.234420, 0.241555, 0.248589, 0.255490, 0.262239, 0.268826, 0.275258,\
        0.281550, 0.287732, 0.293834, 0.299875, 0.305852, 0.311780, 0.317751, 0.323800, 0.329955, 0.336242,\
        0.342683, 0.349291, 0.356074, 0.363029, 0.370139, 0.377357, 0.384593, 0.391717, 0.398601, 0.405230,\
        0.411692, 0.418034, 0.424373, 0.430890, 0.437824, 0.445461, 0.454125, 0.464210, 0.476180, 0.490415,\
        0.506466, 0.521756, 0.535598, 0.547911, 0.558598, 0.567887, 0.575530, 0.580734, 0.584507, 0.588142,\
        0.591860, 0.595271, 0.598981, 0.604188, 0.610856, 0.614230, 0.615266, 0.615668, 0.616325, 0.616926,\
        0.616826, 0.615107, 0.611693, 0.607751, 0.604046, 0.600562, 0.596348, 0.592583, 0.588957, 0.584134,\
        0.578672, 0.572843, 0.566656, 0.560251, 0.553784, 0.547346, 0.540974, 0.534654, 0.528346, 0.521997,\
        0.515509, 0.508906, 0.502441, 0.496066, 0.490295, 0.484995, 0.479572, 0.473692, 0.467559, 0.461454,\
        0.455543, 0.449890, 0.444497, 0.439330, 0.434340, 0.429473, 0.424672, 0.419883, 0.415058, 0.410119,\
        0.404946, 0.399540, 0.393956, 0.388460, 0.383030, 0.377564, 0.372050, 0.366498, 0.360920, 0.355325,\
        0.349714, 0.344084, 0.338428, 0.332746, 0.327043, 0.321338, 0.315669, 0.310093, 0.304629, 0.299120,\
        0.293857],\
        [0.210452, 0.217674, 0.224931, 0.232184, 0.239388, 0.246505, 0.253507, 0.260378, 0.267113, 0.273719,\
        0.280216, 0.286632, 0.293004, 0.299308, 0.305071, 0.310802, 0.316599, 0.322473, 0.328454, 0.334575,\
        0.340856, 0.347307, 0.353925, 0.360700, 0.367600, 0.374574, 0.381541, 0.388401, 0.395071, 0.401544,\
        0.407835, 0.413951, 0.420010, 0.426198, 0.432743, 0.439908, 0.447990, 0.457308, 0.468162, 0.480642,\
        0.494024, 0.506661, 0.518882, 0.530890, 0.541843, 0.551197, 0.558548, 0.563794, 0.567612, 0.570944,\
        0.574313, 0.577942, 0.582364, 0.588528, 0.595350, 0.598474, 0.599277, 0.599609, 0.600592, 0.601365,\
        0.601670, 0.600339, 0.597159, 0.593692, 0.590478, 0.587255, 0.583633, 0.579934, 0.576108, 0.571651,\
        0.566600, 0.561148, 0.555406, 0.549487, 0.543515, 0.537579, 0.531695, 0.525816, 0.519890, 0.513941,\
        0.507853, 0.501315, 0.495101, 0.489017, 0.483256, 0.477802, 0.472365, 0.466746, 0.460973, 0.455190,\
        0.449523, 0.444043, 0.438772, 0.433702, 0.428802, 0.424034, 0.419346, 0.414675, 0.409988, 0.405266,\
        0.400231, 0.394855, 0.389347, 0.383841, 0.378378, 0.372924, 0.367459, 0.361979, 0.356488, 0.350990,\
        0.345488, 0.339978, 0.334456, 0.328918, 0.323363, 0.317799, 0.312253, 0.306800, 0.301577, 0.296397,\
        0.291062],\
        [0.208217, 0.215444, 0.222741, 0.230051, 0.237320, 0.244508, 0.251591, 0.258556, 0.265401, 0.272133,\
        0.278761, 0.285289, 0.291685, 0.297830, 0.303733, 0.309509, 0.315213, 0.320936, 0.326756, 0.332720,\
        0.338850, 0.345150, 0.351611, 0.358215, 0.364927, 0.371699, 0.378464, 0.385153, 0.391708, 0.398100,\
        0.404280, 0.410216, 0.416054, 0.421972, 0.428183, 0.434931, 0.442459, 0.450990, 0.460651, 0.471302,\
        0.482323, 0.492965, 0.503959, 0.515623, 0.526591, 0.535992, 0.543366, 0.548784, 0.552845, 0.556332,\
        0.559850, 0.563818, 0.568629, 0.574381, 0.579847, 0.583201, 0.584620, 0.585389, 0.586273, 0.587003,\
        0.587168, 0.586041, 0.583566, 0.580605, 0.577628, 0.574565, 0.571257, 0.567721, 0.563909, 0.559660,\
        0.554954, 0.549886, 0.544557, 0.539066, 0.533537, 0.528076, 0.522695, 0.517283, 0.511681, 0.506021,\
        0.500568, 0.494338, 0.488113, 0.482165, 0.476457, 0.470952, 0.465518, 0.460039, 0.454501, 0.448967,\
        0.443515, 0.438203, 0.433061, 0.428091, 0.423276, 0.418595, 0.414010, 0.409427, 0.404739, 0.400126,\
        0.395242, 0.389965, 0.384555, 0.379115, 0.373690, 0.368282, 0.362883, 0.357490, 0.352102, 0.346723,\
        0.341354, 0.335994, 0.330638, 0.325278, 0.319907, 0.314522, 0.309119, 0.303735, 0.298560, 0.293601,\
        0.288453],\
        [0.206115, 0.213356, 0.220699, 0.228060, 0.235374, 0.242606, 0.249735, 0.256752, 0.263658, 0.270455,\
        0.277142, 0.283705, 0.290096, 0.296243, 0.302187, 0.307965, 0.313553, 0.319121, 0.324793, 0.330622,\
        0.336621, 0.342786, 0.349106, 0.355557, 0.362107, 0.368714, 0.375328, 0.381896, 0.388375, 0.394721,\
        0.400836, 0.406672, 0.412359, 0.418053, 0.423974, 0.430341, 0.437344, 0.445125, 0.453711, 0.462946,\
        0.472485, 0.482124, 0.492230, 0.502991, 0.513653, 0.522945, 0.530245, 0.535653, 0.539838, 0.543537,\
        0.547289, 0.551439, 0.556150, 0.561202, 0.565796, 0.569077, 0.570997, 0.572147, 0.572986, 0.573508,\
        0.573434, 0.572431, 0.570502, 0.568028, 0.565323, 0.562444, 0.559339, 0.555962, 0.552264, 0.548193,\
        0.543764, 0.539047, 0.534099, 0.528973, 0.523796, 0.518755, 0.513875, 0.508997, 0.503766, 0.498121,\
        0.492807, 0.487118, 0.481212, 0.475414, 0.469788, 0.464310, 0.458914, 0.453539, 0.448164, 0.442815,\
        0.437537, 0.432373, 0.427351, 0.422474, 0.417727, 0.413101, 0.408605, 0.404183, 0.399478, 0.394747,\
        0.389936, 0.384844, 0.379587, 0.374266, 0.368935, 0.363614, 0.358310, 0.353023, 0.347758, 0.342517,\
        0.337305, 0.332122, 0.326964, 0.321822, 0.316683, 0.311547, 0.306412, 0.301197, 0.296163, 0.291201,\
        0.286088],\
        [0.204215, 0.211482, 0.218871, 0.226255, 0.233575, 0.240806, 0.247934, 0.254954, 0.261865, 0.268666,\
        0.275356, 0.281920, 0.288333, 0.294565, 0.300573, 0.306188, 0.311556, 0.316960, 0.322516, 0.328245,\
        0.334142, 0.340196, 0.346392, 0.352709, 0.359120, 0.365588, 0.372076, 0.378542, 0.384945, 0.391232,\
        0.397324, 0.403174, 0.408764, 0.414269, 0.419945, 0.425983, 0.432527, 0.439669, 0.447421, 0.455700,\
        0.464375, 0.473419, 0.482969, 0.493025, 0.503040, 0.512077, 0.519005, 0.524031, 0.528118, 0.531956,\
        0.535894, 0.540091, 0.544566, 0.549078, 0.553121, 0.556231, 0.558320, 0.559633, 0.560426, 0.560760,\
        0.560526, 0.559597, 0.557988, 0.555880, 0.553456, 0.550794, 0.547884, 0.544679, 0.541127, 0.537216,\
        0.533013, 0.528618, 0.524058, 0.519253, 0.514279, 0.509565, 0.505120, 0.500699, 0.495922, 0.490585,\
        0.485254, 0.479821, 0.474250, 0.468677, 0.463188, 0.457793, 0.452472, 0.447194, 0.441949, 0.436743,\
        0.431601, 0.426558, 0.421640, 0.416853, 0.412162, 0.407545, 0.403039, 0.398704, 0.394167, 0.389408,\
        0.384580, 0.379604, 0.374498, 0.369319, 0.364116, 0.358915, 0.353732, 0.348575, 0.343450, 0.338365,\
        0.333326, 0.328342, 0.323415, 0.318532, 0.313666, 0.308804, 0.304049, 0.299318, 0.294241, 0.289164,\
        0.283971],\
        [0.202636, 0.209952, 0.217341, 0.224679, 0.231940, 0.239112, 0.246184, 0.253152, 0.260012, 0.266761,\
        0.273394, 0.279905, 0.286283, 0.292516, 0.298523, 0.303924, 0.309070, 0.314393, 0.319907, 0.325584,\
        0.331409, 0.337372, 0.343462, 0.349660, 0.355945, 0.362289, 0.368661, 0.375026, 0.381347, 0.387582,\
        0.393680, 0.399577, 0.405069, 0.410437, 0.415941, 0.421728, 0.427903, 0.434539, 0.441670, 0.449291,\
        0.457379, 0.465949, 0.475033, 0.484535, 0.494078, 0.502802, 0.508968, 0.513246, 0.517151, 0.521144,\
        0.525262, 0.529447, 0.533709, 0.537870, 0.541556, 0.544459, 0.546499, 0.547790, 0.548484, 0.548660,\
        0.548316, 0.547424, 0.546008, 0.544159, 0.541985, 0.539550, 0.536862, 0.533866, 0.530483, 0.526699,\
        0.522648, 0.518544, 0.514451, 0.510062, 0.505095, 0.500577, 0.496489, 0.492284, 0.487784, 0.482888,\
        0.477797, 0.472592, 0.467291, 0.461952, 0.456633, 0.451360, 0.446139, 0.440965, 0.435835, 0.430750,\
        0.425717, 0.420762, 0.415928, 0.411238, 0.406617, 0.402020, 0.397473, 0.393081, 0.388608, 0.383960,\
        0.379193, 0.374317, 0.369346, 0.364314, 0.359254, 0.354196, 0.349155, 0.344145, 0.339176, 0.334257,\
        0.329402, 0.324627, 0.319949, 0.315370, 0.310847, 0.306255, 0.301565, 0.297173, 0.292364, 0.287261,\
        0.281985],\
        [0.201722, 0.208944, 0.216156, 0.223333, 0.230459, 0.237513, 0.244477, 0.251342, 0.258099, 0.264741,\
        0.271259, 0.277642, 0.283871, 0.289903, 0.295624, 0.300832, 0.306042, 0.311463, 0.317012, 0.322672,\
        0.328443, 0.334326, 0.340316, 0.346404, 0.352570, 0.358794, 0.365049, 0.371305, 0.377531, 0.383685,\
        0.389718, 0.395537, 0.401040, 0.406411, 0.411843, 0.417470, 0.423365, 0.429597, 0.436243, 0.443353,\
        0.450949, 0.459036, 0.467585, 0.476457, 0.485291, 0.493254, 0.498775, 0.502631, 0.506590, 0.510868,\
        0.515225, 0.519290, 0.523397, 0.527388, 0.530845, 0.533521, 0.535391, 0.536541, 0.537079, 0.537100,\
        0.536665, 0.535793, 0.534501, 0.532841, 0.530875, 0.528659, 0.526220, 0.523502, 0.520338, 0.516643,\
        0.512639, 0.508707, 0.505152, 0.501359, 0.496726, 0.492152, 0.488122, 0.484048, 0.479748, 0.475167,\
        0.470369, 0.465425, 0.460377, 0.455265, 0.450127, 0.444994, 0.439886, 0.434822, 0.429808, 0.424840,\
        0.419904, 0.415001, 0.410198, 0.405620, 0.401099, 0.396524, 0.391999, 0.387536, 0.383052, 0.378470,\
        0.373787, 0.369015, 0.364173, 0.359285, 0.354377, 0.349473, 0.344589, 0.339740, 0.334937, 0.330191,\
        0.325519, 0.320944, 0.316505, 0.312245, 0.308163, 0.304178, 0.299953, 0.295344, 0.290480, 0.285337,\
        0.280003],\
        [0.201551, 0.208312, 0.215195, 0.222145, 0.229088, 0.235982, 0.242799, 0.249520, 0.256132, 0.262622,\
        0.268978, 0.275180, 0.281194, 0.286965, 0.292407, 0.297552, 0.302878, 0.308370, 0.313938, 0.319570,\
        0.325277, 0.331074, 0.336965, 0.342941, 0.348989, 0.355090, 0.361222, 0.367360, 0.373472, 0.379520,\
        0.385451, 0.391200, 0.396743, 0.402164, 0.407587, 0.413142, 0.418814, 0.424686, 0.430907, 0.437570,\
        0.444692, 0.452246, 0.460147, 0.468203, 0.476025, 0.482928, 0.488238, 0.492432, 0.496568, 0.501178,\
        0.505723, 0.509315, 0.513447, 0.517468, 0.520767, 0.523179, 0.524806, 0.525768, 0.526113, 0.525962,\
        0.525452, 0.524602, 0.523401, 0.521888, 0.520091, 0.518059, 0.515868, 0.513518, 0.510706, 0.507074,\
        0.503094, 0.499185, 0.495932, 0.492486, 0.488395, 0.484142, 0.480083, 0.476059, 0.471903, 0.467551,\
        0.463014, 0.458329, 0.453528, 0.448640, 0.443688, 0.438696, 0.433701, 0.428744, 0.423851, 0.419015,\
        0.414191, 0.409340, 0.404472, 0.399907, 0.395526, 0.391010, 0.386507, 0.382030, 0.377543, 0.373003,\
        0.368395, 0.363725, 0.359008, 0.354262, 0.349507, 0.344764, 0.340048, 0.335371, 0.330743, 0.326174,\
        0.321678, 0.317277, 0.313019, 0.309000, 0.305293, 0.301763, 0.298142, 0.293479, 0.288460, 0.283274,\
        0.277921],\
        [0.200801, 0.207496, 0.214233, 0.221005, 0.227770, 0.234489, 0.241134, 0.247684, 0.254121, 0.260432,\
        0.266599, 0.272602, 0.278412, 0.283993, 0.289338, 0.294552, 0.299834, 0.305262, 0.310782, 0.316330,\
        0.321939, 0.327636, 0.333420, 0.339281, 0.345205, 0.351176, 0.357177, 0.363186, 0.369175, 0.375111,\
        0.380952, 0.386662, 0.392231, 0.397685, 0.403152, 0.408732, 0.414156, 0.419639, 0.425461, 0.431717,\
        0.438382, 0.445373, 0.452567, 0.459766, 0.466651, 0.472820, 0.478038, 0.482583, 0.487031, 0.491771,\
        0.495935, 0.499345, 0.503841, 0.507938, 0.511071, 0.513145, 0.514507, 0.515341, 0.515485, 0.515119,\
        0.514577, 0.513781, 0.512643, 0.511264, 0.509615, 0.507708, 0.505698, 0.503700, 0.501501, 0.497928,\
        0.494085, 0.490435, 0.487092, 0.483718, 0.480017, 0.476119, 0.472202, 0.468267, 0.464240, 0.460073,\
        0.455761, 0.451319, 0.446763, 0.442101, 0.437337, 0.432484, 0.427584, 0.422711, 0.417938, 0.413258,\
        0.408585, 0.403851, 0.399030, 0.394348, 0.389924, 0.385480, 0.381016, 0.376552, 0.372080, 0.367581,\
        0.363043, 0.358470, 0.353873, 0.349266, 0.344666, 0.340090, 0.335548, 0.331051, 0.326606, 0.322223,\
        0.317907, 0.313662, 0.309509, 0.305560, 0.302068, 0.298721, 0.295070, 0.290818, 0.286040, 0.280965,\
        0.275673],\
        [0.199737, 0.206520, 0.213218, 0.219861, 0.226464, 0.233008, 0.239470, 0.245831, 0.252077, 0.258193,\
        0.264165, 0.269976, 0.275611, 0.281063, 0.286359, 0.291579, 0.296824, 0.302180, 0.307584, 0.312961,\
        0.318440, 0.324027, 0.329698, 0.335435, 0.341225, 0.347058, 0.352919, 0.358790, 0.364650, 0.370473,\
        0.376234, 0.381915, 0.387517, 0.393072, 0.398696, 0.404241, 0.409218, 0.414262, 0.419746, 0.425674,\
        0.431927, 0.438370, 0.444869, 0.451271, 0.457381, 0.463007, 0.468079, 0.472757, 0.477310, 0.481812,\
        0.485959, 0.489952, 0.494297, 0.498314, 0.501330, 0.503057, 0.504218, 0.505157, 0.505104, 0.504454,\
        0.503986, 0.503300, 0.502167, 0.500922, 0.499430, 0.497638, 0.495783, 0.493939, 0.491832, 0.488796,\
        0.485322, 0.481886, 0.478577, 0.475258, 0.471780, 0.468144, 0.464414, 0.460612, 0.456721, 0.452721,\
        0.448611, 0.444398, 0.440086, 0.435663, 0.431106, 0.426393, 0.421555, 0.416712, 0.412028, 0.407528,\
        0.403021, 0.398377, 0.393659, 0.388989, 0.384467, 0.380008, 0.375560, 0.371116, 0.366671, 0.362216,\
        0.357747, 0.353267, 0.348786, 0.344316, 0.339871, 0.335464, 0.331103, 0.326795, 0.322543, 0.318355,\
        0.314241, 0.310188, 0.306149, 0.302162, 0.298613, 0.295436, 0.291880, 0.287835, 0.283313, 0.278424,\
        0.273256],\
        [0.199147, 0.205697, 0.212201, 0.218705, 0.225156, 0.231526, 0.237799, 0.243963, 0.250007, 0.255923,\
        0.261700, 0.267329, 0.272808, 0.278148, 0.283383, 0.288577, 0.293810, 0.299151, 0.304251, 0.309403,\
        0.314778, 0.320266, 0.325817, 0.331418, 0.337063, 0.342746, 0.348456, 0.354180, 0.359902, 0.365606,\
        0.371277, 0.376910, 0.382517, 0.388135, 0.393788, 0.399134, 0.403609, 0.408385, 0.413700, 0.419426,\
        0.425342, 0.431272, 0.437108, 0.442780, 0.448222, 0.453372, 0.458223, 0.462853, 0.467363, 0.471766,\
        0.476008, 0.480172, 0.484306, 0.488056, 0.490919, 0.492718, 0.493931, 0.494873, 0.494811, 0.494164,\
        0.493679, 0.493026, 0.491999, 0.490820, 0.489444, 0.487840, 0.486128, 0.484317, 0.482204, 0.479558,\
        0.476523, 0.473370, 0.470210, 0.467012, 0.463707, 0.460272, 0.456721, 0.453066, 0.449314, 0.445469,\
        0.441542, 0.437546, 0.433484, 0.429330, 0.425022, 0.420483, 0.415683, 0.410767, 0.406091, 0.401754,\
        0.397400, 0.392843, 0.388217, 0.383610, 0.379076, 0.374602, 0.370160, 0.365736, 0.361323, 0.356917,\
        0.352517, 0.348128, 0.343760, 0.339424, 0.335134, 0.330900, 0.326727, 0.322617, 0.318564, 0.314575,\
        0.310677, 0.306898, 0.303131, 0.299265, 0.295675, 0.292305, 0.288750, 0.284812, 0.280462, 0.275742,\
        0.270709],\
        [0.198218, 0.204662, 0.211132, 0.217543, 0.223847, 0.230037, 0.236114, 0.242075, 0.247914, 0.253627,\
        0.259210, 0.264663, 0.269990, 0.275208, 0.280348, 0.285455, 0.290564, 0.295624, 0.300493, 0.305650,\
        0.311007, 0.316389, 0.321799, 0.327246, 0.332731, 0.338252, 0.343799, 0.349365, 0.354937, 0.360505,\
        0.366061, 0.371599, 0.377119, 0.382608, 0.387971, 0.392922, 0.397417, 0.402132, 0.407441, 0.413070,\
        0.418702, 0.424138, 0.429319, 0.434293, 0.439116, 0.443814, 0.448398, 0.452889, 0.457308, 0.461649,\
        0.465892, 0.470024, 0.473960, 0.477460, 0.480236, 0.482205, 0.483549, 0.484361, 0.484483, 0.484114,\
        0.483605, 0.482937, 0.482030, 0.480931, 0.479653, 0.478198, 0.476597, 0.474831, 0.472798, 0.470413,\
        0.467726, 0.464867, 0.461922, 0.458899, 0.455772, 0.452518, 0.449133, 0.445622, 0.442002, 0.438293,\
        0.434525, 0.430728, 0.426917, 0.423066, 0.419081, 0.414814, 0.410126, 0.405022, 0.400259, 0.395972,\
        0.391703, 0.387272, 0.382749, 0.378212, 0.373706, 0.369244, 0.364818, 0.360420, 0.356045, 0.351692,\
        0.347363, 0.343065, 0.338807, 0.334603, 0.330466, 0.326407, 0.322429, 0.318528, 0.314688, 0.310889,\
        0.307154, 0.303654, 0.300243, 0.296583, 0.292945, 0.289407, 0.285767, 0.281855, 0.277601, 0.273004,\
        0.268089],\
        [0.197171, 0.203635, 0.210132, 0.216415, 0.222542, 0.228538, 0.234410, 0.240162, 0.245794, 0.251304,\
        0.256694, 0.261969, 0.267138, 0.272218, 0.277232, 0.282204, 0.287138, 0.292004, 0.296861, 0.302006,\
        0.307225, 0.312423, 0.317656, 0.322930, 0.328242, 0.333588, 0.338962, 0.344357, 0.349765, 0.355177,\
        0.360587, 0.365983, 0.371349, 0.376638, 0.381754, 0.386585, 0.391206, 0.395973, 0.401231, 0.406718,\
        0.412088, 0.417004, 0.421496, 0.425772, 0.430003, 0.434275, 0.438590, 0.442911, 0.447213, 0.451471,\
        0.455649, 0.459686, 0.463467, 0.466817, 0.469562, 0.471634, 0.473078, 0.473936, 0.474225, 0.474074,\
        0.473656, 0.473036, 0.472215, 0.471209, 0.470030, 0.468687, 0.467183, 0.465499, 0.463585, 0.461409,\
        0.458995, 0.456407, 0.453697, 0.450883, 0.447951, 0.444880, 0.441655, 0.438280, 0.434775, 0.431176,\
        0.427534, 0.423899, 0.420315, 0.416784, 0.413198, 0.409322, 0.404971, 0.399990, 0.394959, 0.390512,\
        0.386163, 0.381762, 0.377303, 0.372825, 0.368362, 0.363930, 0.359534, 0.355172, 0.350843, 0.346549,\
        0.342295, 0.338088, 0.333939, 0.329864, 0.325876, 0.321990, 0.318212, 0.314535, 0.310936, 0.307365,\
        0.303721, 0.300221, 0.297126, 0.293700, 0.290161, 0.286584, 0.282891, 0.278978, 0.274776, 0.270261,\
        0.265437],\
        [0.196774, 0.203161, 0.209290, 0.215315, 0.221224, 0.227010, 0.232674, 0.238216, 0.243638, 0.248945,\
        0.254141, 0.259236, 0.264240, 0.269169, 0.274042, 0.278874, 0.283677, 0.288475, 0.293352, 0.298384,\
        0.303349, 0.308334, 0.313383, 0.318477, 0.323608, 0.328770, 0.333961, 0.339175, 0.344404, 0.349643,\
        0.354884, 0.360114, 0.365308, 0.370424, 0.375406, 0.380230, 0.384984, 0.389872, 0.395029, 0.400330,\
        0.405501, 0.409807, 0.413564, 0.417162, 0.420840, 0.424751, 0.428838, 0.432987, 0.437150, 0.441301,\
        0.445387, 0.449324, 0.452997, 0.456268, 0.459010, 0.461154, 0.462692, 0.463653, 0.464090, 0.464102,\
        0.463804, 0.463272, 0.462541, 0.461630, 0.460550, 0.459308, 0.457902, 0.456318, 0.454536, 0.452543,\
        0.450357, 0.448012, 0.445539, 0.442950, 0.440232, 0.437356, 0.434296, 0.431047, 0.427636, 0.424118,\
        0.420560, 0.417032, 0.413608, 0.410351, 0.407196, 0.403718, 0.399629, 0.394937, 0.389969, 0.385278,\
        0.380783, 0.376343, 0.371905, 0.367471, 0.363053, 0.358665, 0.354312, 0.349998, 0.345725, 0.341498,\
        0.337323, 0.333210, 0.329171, 0.325220, 0.321375, 0.317656, 0.314073, 0.310625, 0.307283, 0.304020,\
        0.300757, 0.297238, 0.294023, 0.290746, 0.287319, 0.283771, 0.280073, 0.276165, 0.271997, 0.267538,\
        0.262780],\
        [0.196056, 0.202359, 0.208307, 0.214132, 0.219844, 0.225430, 0.230889, 0.236223, 0.241438, 0.246542,\
        0.251543, 0.256454, 0.261287, 0.266058, 0.270784, 0.275480, 0.280167, 0.284874, 0.289641, 0.294457,\
        0.299255, 0.304091, 0.308981, 0.313898, 0.318841, 0.323814, 0.328816, 0.333841, 0.338881, 0.343933,\
        0.348993, 0.354049, 0.359083, 0.364063, 0.368964, 0.373791, 0.378610, 0.383523, 0.388585, 0.393731,\
        0.398552, 0.402256, 0.405417, 0.408456, 0.411635, 0.415319, 0.419251, 0.423214, 0.427211, 0.431237,\
        0.435227, 0.439077, 0.442677, 0.445911, 0.448671, 0.450878, 0.452503, 0.453565, 0.454120, 0.454255,\
        0.454068, 0.453635, 0.453002, 0.452189, 0.451208, 0.450064, 0.448759, 0.447285, 0.445635, 0.443810,\
        0.441821, 0.439693, 0.437448, 0.435092, 0.432607, 0.429949, 0.427064, 0.423934, 0.420598, 0.417134,\
        0.413625, 0.410156, 0.406803, 0.403664, 0.400861, 0.397667, 0.393728, 0.389289, 0.384602, 0.379952,\
        0.375419, 0.370963, 0.366546, 0.362153, 0.357786, 0.353453, 0.349158, 0.344906, 0.340701, 0.336550,\
        0.332463, 0.328448, 0.324520, 0.320691, 0.316982, 0.313415, 0.310014, 0.306782, 0.303673, 0.300616,\
        0.297593, 0.294361, 0.291109, 0.287847, 0.284480, 0.280973, 0.277293, 0.273401, 0.269260, 0.264841,\
        0.260132],\
        [0.194928, 0.201104, 0.207055, 0.212804, 0.218370, 0.223777, 0.229040, 0.234172, 0.239182, 0.244084,\
        0.248890, 0.253615, 0.258273, 0.262881, 0.267455, 0.272012, 0.276570, 0.281150, 0.285765, 0.290405,\
        0.295057, 0.299745, 0.304474, 0.309207, 0.313956, 0.318739, 0.323553, 0.328385, 0.333228, 0.338083,\
        0.342954, 0.347839, 0.352724, 0.357588, 0.362416, 0.367213, 0.372011, 0.376844, 0.381700, 0.386443,\
        0.390721, 0.394193, 0.397120, 0.399794, 0.402514, 0.406241, 0.410001, 0.413703, 0.417493, 0.421389,\
        0.425292, 0.429062, 0.432605, 0.435830, 0.438626, 0.440893, 0.442588, 0.443724, 0.444357, 0.444573,\
        0.444473, 0.444134, 0.443602, 0.442891, 0.442010, 0.440965, 0.439761, 0.438399, 0.436877, 0.435201,\
        0.433386, 0.431451, 0.429417, 0.427292, 0.425060, 0.422657, 0.419972, 0.416952, 0.413673, 0.410251,\
        0.406780, 0.403367, 0.400080, 0.397001, 0.394183, 0.391102, 0.387430, 0.383295, 0.378897, 0.374422,\
        0.369971, 0.365566, 0.361199, 0.356865, 0.352565, 0.348301, 0.344079, 0.339904, 0.335781, 0.331718,\
        0.327727, 0.323821, 0.320012, 0.316309, 0.312731, 0.309301, 0.306053, 0.303015, 0.300136, 0.297246,\
        0.294338, 0.291318, 0.288193, 0.284988, 0.281670, 0.278201, 0.274547, 0.270678, 0.266561, 0.262174,\
        0.257500],\
        [0.193709, 0.199800, 0.205733, 0.211385, 0.216807, 0.222045, 0.227122, 0.232056, 0.236864, 0.241564,\
        0.246174, 0.250710, 0.255190, 0.259631, 0.264049, 0.268460, 0.272878, 0.277316, 0.281779, 0.286264,\
        0.290772, 0.295311, 0.299873, 0.304412, 0.308967, 0.313570, 0.318205, 0.322843, 0.327478, 0.332126,\
        0.336805, 0.341522, 0.346266, 0.351020, 0.355765, 0.360493, 0.365202, 0.369878, 0.374462, 0.378814,\
        0.382717, 0.386034, 0.388877, 0.391466, 0.394243, 0.397776, 0.401125, 0.404548, 0.408089, 0.411884,\
        0.415712, 0.419377, 0.422855, 0.426094, 0.428948, 0.431266, 0.433001, 0.434176, 0.434840, 0.435087,\
        0.435038, 0.434786, 0.434359, 0.433754, 0.432970, 0.432020, 0.430916, 0.429663, 0.428262, 0.426720,\
        0.425053, 0.423286, 0.421438, 0.419525, 0.417554, 0.415462, 0.413032, 0.410099, 0.406848, 0.403492,\
        0.400075, 0.396753, 0.393564, 0.390535, 0.387597, 0.384486, 0.380990, 0.377119, 0.372989, 0.368728,\
        0.364426, 0.360128, 0.355853, 0.351605, 0.347390, 0.343214, 0.339083, 0.335002, 0.330976, 0.327015,\
        0.323133, 0.319349, 0.315676, 0.312114, 0.308676, 0.305387, 0.302257, 0.299387, 0.296672, 0.293954,\
        0.291170, 0.288285, 0.285281, 0.282155, 0.278891, 0.275462, 0.271838, 0.267992, 0.263897, 0.259534,\
        0.254886],\
        [0.192534, 0.198681, 0.204425, 0.209894, 0.215164, 0.220237, 0.225132, 0.229871, 0.234478, 0.238976,\
        0.243387, 0.247732, 0.252032, 0.256304, 0.260563, 0.264824, 0.269097, 0.273389, 0.277704, 0.282040,\
        0.286398, 0.290775, 0.295155, 0.299518, 0.303895, 0.308343, 0.312815, 0.317256, 0.321668, 0.326094,\
        0.330577, 0.335131, 0.339744, 0.344391, 0.349044, 0.353677, 0.358262, 0.362757, 0.367090, 0.371151,\
        0.374819, 0.378041, 0.380897, 0.383589, 0.386424, 0.389536, 0.392680, 0.395871, 0.399150, 0.402911,\
        0.406603, 0.410094, 0.413473, 0.416762, 0.419710, 0.422039, 0.423769, 0.424953, 0.425603, 0.425814,\
        0.425774, 0.425606, 0.425301, 0.424806, 0.424108, 0.423243, 0.422235, 0.421088, 0.419798, 0.418372,\
        0.416831, 0.415203, 0.413511, 0.411767, 0.410014, 0.408295, 0.406242, 0.403347, 0.400078, 0.396827,\
        0.393524, 0.390283, 0.387152, 0.384120, 0.381108, 0.377967, 0.374572, 0.370895, 0.366992, 0.362941,\
        0.358816, 0.354663, 0.350510, 0.346374, 0.342267, 0.338200, 0.334179, 0.330212, 0.326302, 0.322456,\
        0.318692, 0.315043, 0.311538, 0.308158, 0.304853, 0.301810, 0.298790, 0.296042, 0.293416, 0.290796,\
        0.288111, 0.285325, 0.282413, 0.279361, 0.276151, 0.272760, 0.269165, 0.265341, 0.261265, 0.256919,\
        0.252287],\
        [0.191224, 0.197234, 0.202889, 0.208296, 0.213444, 0.218359, 0.223073, 0.227617, 0.232021, 0.236314,\
        0.240524, 0.244677, 0.248794, 0.252895, 0.256995, 0.261105, 0.265233, 0.269382, 0.273554, 0.277745,\
        0.281954, 0.286171, 0.290385, 0.294589, 0.298814, 0.303115, 0.307439, 0.311664, 0.315823, 0.320013,\
        0.324302, 0.328704, 0.333199, 0.337747, 0.342307, 0.346837, 0.351292, 0.355619, 0.359751, 0.363611,\
        0.367134, 0.370309, 0.373206, 0.375974, 0.378781, 0.381699, 0.384688, 0.387753, 0.390991, 0.394501,\
        0.397945, 0.401269, 0.404481, 0.407919, 0.410997, 0.413195, 0.414867, 0.416076, 0.416686, 0.416760,\
        0.416676, 0.416616, 0.416472, 0.416086, 0.415442, 0.414645, 0.413731, 0.412685, 0.411500, 0.410174,\
        0.408734, 0.407221, 0.405666, 0.404050, 0.402389, 0.400954, 0.399410, 0.396573, 0.393384, 0.390192,\
        0.387000, 0.383848, 0.380764, 0.377732, 0.374692, 0.371550, 0.368230, 0.364701, 0.360985, 0.357130,\
        0.353186, 0.349195, 0.345187, 0.341185, 0.337205, 0.333264, 0.329375, 0.325545, 0.321773, 0.318059,\
        0.314419, 0.310902, 0.307586, 0.304507, 0.301387, 0.298421, 0.295672, 0.292968, 0.290374, 0.287801,\
        0.285177, 0.282459, 0.279613, 0.276619, 0.273454, 0.270098, 0.266528, 0.262723, 0.258661, 0.254325,\
        0.249702],\
        [0.189820, 0.195747, 0.201345, 0.206664, 0.211676, 0.216425, 0.220953, 0.225297, 0.229493, 0.233577,\
        0.237582, 0.241538, 0.245472, 0.249403, 0.253346, 0.257308, 0.261295, 0.265307, 0.269343, 0.273398,\
        0.277466, 0.281539, 0.285613, 0.289689, 0.293790, 0.297949, 0.302112, 0.306077, 0.309954, 0.313906,\
        0.318016, 0.322292, 0.326689, 0.331151, 0.335623, 0.340053, 0.344389, 0.348574, 0.352552, 0.356272,\
        0.359703, 0.362853, 0.365782, 0.368588, 0.371375, 0.374205, 0.377100, 0.380089, 0.383220, 0.386485,\
        0.389754, 0.392956, 0.396108, 0.399780, 0.402731, 0.404580, 0.406208, 0.407543, 0.408156, 0.407916,\
        0.407749, 0.407857, 0.407928, 0.407642, 0.406975, 0.406239, 0.405421, 0.404466, 0.403381, 0.402142,\
        0.400787, 0.399373, 0.397935, 0.396446, 0.394909, 0.393494, 0.391852, 0.389434, 0.386535, 0.383498,\
        0.380434, 0.377383, 0.374363, 0.371360, 0.368332, 0.365220, 0.361974, 0.358572, 0.355019, 0.351343,\
        0.347578, 0.343758, 0.339909, 0.336053, 0.332214, 0.328415, 0.324675, 0.321007, 0.317405, 0.313850,\
        0.310340, 0.306931, 0.303751, 0.300933, 0.298351, 0.295404, 0.292669, 0.290043, 0.287485, 0.284944,\
        0.282359, 0.279686, 0.276887, 0.273935, 0.270806, 0.267477, 0.263927, 0.260134, 0.256080, 0.251749,\
        0.247127],\
        [0.188394, 0.194313, 0.199856, 0.205024, 0.209874, 0.214443, 0.218776, 0.222914, 0.226895, 0.230761,\
        0.234554, 0.238312, 0.242062, 0.245826, 0.249617, 0.253439, 0.257293, 0.261178, 0.265088, 0.269017,\
        0.272957, 0.276903, 0.280851, 0.284804, 0.288776, 0.292767, 0.296706, 0.300446, 0.304070, 0.307814,\
        0.311781, 0.315965, 0.320289, 0.324674, 0.329065, 0.333407, 0.337638, 0.341704, 0.345560, 0.349176,\
        0.352541, 0.355671, 0.358614, 0.361438, 0.364214, 0.366998, 0.369828, 0.372736, 0.375745, 0.378842,\
        0.381976, 0.385123, 0.388354, 0.391693, 0.394318, 0.396142, 0.397768, 0.399220, 0.399972, 0.399369,\
        0.399183, 0.399419, 0.399657, 0.399451, 0.398731, 0.398064, 0.397324, 0.396441, 0.395433, 0.394280,\
        0.393018, 0.391696, 0.390337, 0.388934, 0.387493, 0.386008, 0.384279, 0.382085, 0.379491, 0.376684,\
        0.373791, 0.370869, 0.367940, 0.364999, 0.362020, 0.358966, 0.355806, 0.352523, 0.349120, 0.345613,\
        0.342025, 0.338380, 0.334699, 0.331001, 0.327310, 0.323657, 0.320078, 0.316595, 0.313205, 0.309860,\
        0.306509, 0.303207, 0.300112, 0.297444, 0.294958, 0.292358, 0.289749, 0.287201, 0.284694, 0.282187,\
        0.279634, 0.276993, 0.274227, 0.271305, 0.268202, 0.264892, 0.261355, 0.257570, 0.253519, 0.249185,\
        0.244555],\
        [0.186921, 0.192823, 0.198286, 0.203347, 0.208035, 0.212416, 0.216552, 0.220476, 0.224229, 0.227865,\
        0.231437, 0.234991, 0.238560, 0.242164, 0.245811, 0.249503, 0.253237, 0.257007, 0.260804, 0.264619,\
        0.268445, 0.272274, 0.276103, 0.279930, 0.283751, 0.287544, 0.291250, 0.294808, 0.298273, 0.301856,\
        0.305707, 0.309834, 0.314092, 0.318395, 0.322705, 0.326967, 0.331106, 0.335066, 0.338814, 0.342341,\
        0.345648, 0.348755, 0.351698, 0.354524, 0.357287, 0.360034, 0.362803, 0.365626, 0.368518, 0.371476,\
        0.374481, 0.377519, 0.380575, 0.383500, 0.385960, 0.387895, 0.389524, 0.390849, 0.391487, 0.391324,\
        0.391211, 0.391362, 0.391511, 0.391334, 0.390790, 0.390154, 0.389442, 0.388613, 0.387658, 0.386580,\
        0.385400, 0.384150, 0.382847, 0.381492, 0.380074, 0.378548, 0.376803, 0.374740, 0.372372, 0.369791,\
        0.367086, 0.364315, 0.361504, 0.358655, 0.355756, 0.352784, 0.349723, 0.346562, 0.343305, 0.339962,\
        0.336551, 0.333087, 0.329582, 0.326050, 0.322510, 0.318999, 0.315574, 0.312286, 0.309147, 0.306093,\
        0.302956, 0.299808, 0.296874, 0.294242, 0.291782, 0.289330, 0.286865, 0.284410, 0.281963, 0.279497,\
        0.276976, 0.274361, 0.271620, 0.268720, 0.265635, 0.262338, 0.258808, 0.255024, 0.250968, 0.246625,\
        0.241980],\
        [0.185377, 0.191282, 0.196715, 0.201674, 0.206144, 0.210347, 0.214295, 0.217995, 0.221498, 0.224883,\
        0.228222, 0.231570, 0.234962, 0.238415, 0.241932, 0.245510, 0.249139, 0.252808, 0.256506, 0.260222,\
        0.263946, 0.267670, 0.271387, 0.275091, 0.278768, 0.282394, 0.285935, 0.289374, 0.292768, 0.296246,\
        0.299985, 0.304036, 0.308194, 0.312375, 0.316600, 0.320798, 0.324850, 0.328698, 0.332335, 0.335772,\
        0.339020, 0.342096, 0.345025, 0.347840, 0.350579, 0.353282, 0.355985, 0.358714, 0.361487, 0.364305,\
        0.367157, 0.370021, 0.372838, 0.375483, 0.377798, 0.379728, 0.381312, 0.382510, 0.383185, 0.383382,\
        0.383443, 0.383541, 0.383587, 0.383420, 0.383009, 0.382442, 0.381764, 0.380972, 0.380062, 0.379038,\
        0.377916, 0.376715, 0.375449, 0.374115, 0.372699, 0.371158, 0.369431, 0.367467, 0.365269, 0.362880,\
        0.360359, 0.357749, 0.355075, 0.352343, 0.349548, 0.346679, 0.343729, 0.340695, 0.337582, 0.334402,\
        0.331169, 0.327895, 0.324582, 0.321230, 0.317843, 0.314458, 0.311162, 0.308046, 0.305145, 0.302464,\
        0.299623, 0.296632, 0.293820, 0.291225, 0.288784, 0.286403, 0.284032, 0.281660, 0.279274, 0.276851,\
        0.274361, 0.271772, 0.269051, 0.266168, 0.263094, 0.259804, 0.256275, 0.252487, 0.248420, 0.244060,\
        0.239394],\
        [0.183744, 0.189668, 0.195037, 0.199798, 0.204157, 0.208283, 0.212052, 0.215491, 0.218702, 0.221805,\
        0.224896, 0.228038, 0.231263, 0.234581, 0.237987, 0.241469, 0.245011, 0.248596, 0.252211, 0.255841,\
        0.259476, 0.263108, 0.266726, 0.270322, 0.273883, 0.277391, 0.280833, 0.284216, 0.287593, 0.291062,\
        0.294737, 0.298644, 0.302645, 0.306632, 0.310802, 0.314968, 0.318910, 0.322616, 0.326123, 0.329459,\
        0.332644, 0.335683, 0.338588, 0.341376, 0.344077, 0.346724, 0.349346, 0.351970, 0.354612, 0.357279,\
        0.359959, 0.362625, 0.365218, 0.367649, 0.369823, 0.371685, 0.373213, 0.374373, 0.375129, 0.375537,\
        0.375752, 0.375874, 0.375891, 0.375741, 0.375402, 0.374901, 0.374267, 0.373510, 0.372634, 0.371647,\
        0.370562, 0.369391, 0.368144, 0.366818, 0.365401, 0.363863, 0.362169, 0.360291, 0.358228, 0.356004,\
        0.353654, 0.351206, 0.348680, 0.346082, 0.343410, 0.340660, 0.337830, 0.334926, 0.331958, 0.328939,\
        0.325887, 0.322812, 0.319713, 0.316570, 0.313354, 0.310075, 0.306862, 0.303898, 0.301158, 0.298683,\
        0.296178, 0.293447, 0.290787, 0.288272, 0.285878, 0.283551, 0.281248, 0.278941, 0.276609, 0.274228,\
        0.271770, 0.269205, 0.266502, 0.263631, 0.260565, 0.257278, 0.253745, 0.249947, 0.245865, 0.241482,\
        0.236787],\
        [0.182007, 0.188007, 0.193355, 0.198040, 0.202380, 0.206375, 0.209887, 0.212973, 0.215820, 0.218603,\
        0.221437, 0.224383, 0.227461, 0.230668, 0.233987, 0.237396, 0.240870, 0.244389, 0.247935, 0.251494,\
        0.255054, 0.258606, 0.262142, 0.265653, 0.269129, 0.272562, 0.275951, 0.279315, 0.282699, 0.286170,\
        0.289791, 0.293560, 0.297378, 0.301144, 0.305442, 0.309549, 0.313288, 0.316804, 0.320154, 0.323382,\
        0.326503, 0.329506, 0.332378, 0.335125, 0.337770, 0.340342, 0.342867, 0.345371, 0.347871, 0.350374,\
        0.352874, 0.355345, 0.357739, 0.359990, 0.362032, 0.363812, 0.365295, 0.366456, 0.367289, 0.367833,\
        0.368166, 0.368348, 0.368388, 0.368269, 0.367981, 0.367533, 0.366941, 0.366216, 0.365368, 0.364405,\
        0.363340, 0.362183, 0.360943, 0.359619, 0.358204, 0.356679, 0.355024, 0.353224, 0.351278, 0.349199,\
        0.347006, 0.344718, 0.342348, 0.339897, 0.337362, 0.334741, 0.332037, 0.329263, 0.326437, 0.323576,\
        0.320701, 0.317831, 0.314970, 0.312084, 0.309099, 0.305943, 0.302724, 0.299931, 0.297384, 0.294995,\
        0.292632, 0.290175, 0.287711, 0.285315, 0.282997, 0.280732, 0.278487, 0.276236, 0.273953, 0.271611,\
        0.269184, 0.266642, 0.263955, 0.261095, 0.258034, 0.254745, 0.251205, 0.247393, 0.243289, 0.238878,\
        0.234147],\
        [0.180122, 0.186320, 0.191874, 0.196715, 0.200928, 0.204742, 0.207813, 0.210390, 0.212793, 0.215230,\
        0.217817, 0.220594, 0.223559, 0.226688, 0.229949, 0.233309, 0.236737, 0.240206, 0.243698, 0.247197,\
        0.250694, 0.254180, 0.257650, 0.261096, 0.264514, 0.267902, 0.271266, 0.274626, 0.278019, 0.281488,\
        0.285065, 0.288746, 0.292495, 0.296417, 0.300735, 0.304450, 0.307895, 0.311210, 0.314388, 0.317507,\
        0.320585, 0.323560, 0.326389, 0.329076, 0.331647, 0.334126, 0.336538, 0.338906, 0.341251, 0.343583,\
        0.345902, 0.348187, 0.350403, 0.352501, 0.354426, 0.356133, 0.357584, 0.358758, 0.359653, 0.360294,\
        0.360722, 0.360972, 0.361060, 0.360985, 0.360744, 0.360342, 0.359787, 0.359090, 0.358262, 0.357313,\
        0.356255, 0.355101, 0.353860, 0.352535, 0.351123, 0.349617, 0.348005, 0.346279, 0.344437, 0.342488,\
        0.340443, 0.338313, 0.336104, 0.333811, 0.331425, 0.328939, 0.326362, 0.323716, 0.321028, 0.318318,\
        0.315607, 0.312929, 0.310315, 0.307732, 0.305072, 0.302177, 0.299032, 0.296254, 0.293808, 0.291487,\
        0.289208, 0.286911, 0.284609, 0.282336, 0.280107, 0.277909, 0.275723, 0.273523, 0.271284, 0.268979,\
        0.266582, 0.264063, 0.261392, 0.258542, 0.255484, 0.252192, 0.248642, 0.244811, 0.240681, 0.236236,\
        0.231463],\
        [0.177959, 0.184441, 0.190361, 0.195619, 0.200035, 0.203365, 0.205645, 0.207582, 0.209517, 0.211632,\
        0.214016, 0.216676, 0.219574, 0.222663, 0.225897, 0.229234, 0.232635, 0.236070, 0.239518, 0.242966,\
        0.246408, 0.249840, 0.253256, 0.256654, 0.260033, 0.263394, 0.266745, 0.270106, 0.273503, 0.276966,\
        0.280518, 0.284168, 0.287933, 0.291868, 0.295820, 0.299302, 0.302616, 0.305800, 0.308774, 0.311804,\
        0.314897, 0.317854, 0.320616, 0.323218, 0.325696, 0.328068, 0.330350, 0.332567, 0.334745, 0.336901,\
        0.339041, 0.341154, 0.343213, 0.345179, 0.347008, 0.348655, 0.350084, 0.351274, 0.352219, 0.352934,\
        0.353439, 0.353759, 0.353904, 0.353880, 0.353687, 0.353327, 0.352807, 0.352134, 0.351319, 0.350375,\
        0.349316, 0.348157, 0.346908, 0.345578, 0.344171, 0.342686, 0.341119, 0.339465, 0.337721, 0.335891,\
        0.333985, 0.332010, 0.329969, 0.327848, 0.325623, 0.323278, 0.320822, 0.318294, 0.315740, 0.313179,\
        0.310612, 0.308086, 0.305692, 0.303401, 0.301083, 0.298463, 0.295591, 0.292861, 0.290402, 0.288106,\
        0.285884, 0.283686, 0.281499, 0.279332, 0.277187, 0.275058, 0.272929, 0.270778, 0.268582, 0.266313,\
        0.263945, 0.261449, 0.258795, 0.255954, 0.252898, 0.249601, 0.246038, 0.242186, 0.238026, 0.233542,\
        0.228721],\
        [0.175325, 0.182049, 0.188292, 0.193994, 0.198900, 0.201370, 0.202882, 0.204325, 0.205901, 0.207780,\
        0.210050, 0.212665, 0.215546, 0.218631, 0.221869, 0.225206, 0.228595, 0.232003, 0.235411, 0.238813,\
        0.242205, 0.245587, 0.248960, 0.252321, 0.255672, 0.259016, 0.262359, 0.265718, 0.269112, 0.272563,\
        0.276087, 0.279695, 0.283392, 0.287145, 0.290807, 0.294226, 0.297478, 0.300569, 0.303232, 0.306293,\
        0.309502, 0.312405, 0.315037, 0.317524, 0.319900, 0.322157, 0.324296, 0.326347, 0.328347, 0.330325,\
        0.332292, 0.334245, 0.336166, 0.338024, 0.339776, 0.341380, 0.342797, 0.344003, 0.344989, 0.345761,\
        0.346332, 0.346716, 0.346922, 0.346954, 0.346811, 0.346494, 0.346005, 0.345352, 0.344546, 0.343600,\
        0.342533, 0.341361, 0.340100, 0.338763, 0.337360, 0.335898, 0.334378, 0.332794, 0.331143, 0.329424,\
        0.327646, 0.325823, 0.323956, 0.322024, 0.319982, 0.317786, 0.315436, 0.313004, 0.310583, 0.308187,\
        0.305766, 0.303320, 0.301095, 0.299026, 0.296888, 0.294523, 0.291961, 0.289425, 0.287034, 0.284778,\
        0.282606, 0.280479, 0.278378, 0.276294, 0.274224, 0.272159, 0.270085, 0.267981, 0.265826, 0.263592,\
        0.261254, 0.258781, 0.256143, 0.253313, 0.250259, 0.246956, 0.243378, 0.239502, 0.235309, 0.230781,\
        0.225907],\
        [0.172055, 0.178873, 0.185166, 0.190763, 0.195162, 0.197729, 0.199275, 0.200578, 0.201986, 0.203733,\
        0.206003, 0.208641, 0.211536, 0.214642, 0.217906, 0.221262, 0.224647, 0.228029, 0.231394, 0.234744,\
        0.238085, 0.241421, 0.244755, 0.248087, 0.251417, 0.254746, 0.258081, 0.261432, 0.264812, 0.268237,\
        0.271717, 0.275255, 0.278838, 0.282418, 0.285907, 0.289235, 0.292394, 0.295304, 0.297968, 0.301231,\
        0.304499, 0.307167, 0.309583, 0.311952, 0.314243, 0.316388, 0.318370, 0.320237, 0.322052, 0.323853,\
        0.325654, 0.327460, 0.329262, 0.331033, 0.332732, 0.334310, 0.335724, 0.336944, 0.337962, 0.338779,\
        0.339406, 0.339850, 0.340116, 0.340206, 0.340116, 0.339843, 0.339387, 0.338752, 0.337949, 0.336996,\
        0.335915, 0.334726, 0.333449, 0.332101, 0.330701, 0.329263, 0.327792, 0.326282, 0.324722, 0.323104,\
        0.321439, 0.319754, 0.318068, 0.316350, 0.314522, 0.312497, 0.310234, 0.307847, 0.305556, 0.303352,\
        0.301142, 0.298795, 0.296661, 0.294698, 0.292680, 0.290511, 0.288209, 0.285881, 0.283615, 0.281433,\
        0.279322, 0.277258, 0.275223, 0.273207, 0.271201, 0.269193, 0.267168, 0.265110, 0.262994, 0.260796,\
        0.258487, 0.256038, 0.253419, 0.250598, 0.247547, 0.244238, 0.240644, 0.236742, 0.232513, 0.227937,\
        0.223003],\
        [0.168097, 0.174867, 0.181013, 0.186321, 0.190461, 0.193274, 0.195136, 0.196589, 0.198010, 0.199735,\
        0.202064, 0.204721, 0.207614, 0.210747, 0.214060, 0.217446, 0.220825, 0.224167, 0.227474, 0.230761,\
        0.234045, 0.237334, 0.240633, 0.243939, 0.247251, 0.250566, 0.253888, 0.257223, 0.260578, 0.263963,\
        0.267383, 0.270835, 0.274300, 0.277737, 0.281095, 0.284329, 0.287415, 0.290352, 0.293298, 0.296525,\
        0.299655, 0.301898, 0.304157, 0.306465, 0.308717, 0.310762, 0.312557, 0.314221, 0.315853, 0.317486,\
        0.319131, 0.320801, 0.322501, 0.324209, 0.325878, 0.327449, 0.328868, 0.330100, 0.331138, 0.331988,\
        0.332661, 0.333160, 0.333486, 0.333636, 0.333603, 0.333379, 0.332957, 0.332339, 0.331537, 0.330571,\
        0.329471, 0.328265, 0.326971, 0.325609, 0.324205, 0.322791, 0.321374, 0.319944, 0.318481, 0.316960,\
        0.315379, 0.313804, 0.312295, 0.310812, 0.309244, 0.307442, 0.305276, 0.302838, 0.300672, 0.298649,\
        0.296593, 0.294476, 0.292434, 0.290498, 0.288554, 0.286525, 0.284410, 0.282258, 0.280123, 0.278033,\
        0.275991, 0.273988, 0.272011, 0.270052, 0.268098, 0.266139, 0.264160, 0.262143, 0.260066, 0.257902,\
        0.255624, 0.253201, 0.250601, 0.247793, 0.244745, 0.241428, 0.237818, 0.233888, 0.229619, 0.224992,\
        0.219993],\
        [0.163511, 0.170171, 0.176148, 0.181272, 0.185374, 0.188433, 0.190677, 0.192469, 0.194157, 0.196060,\
        0.198366, 0.200974, 0.203821, 0.206998, 0.210388, 0.213805, 0.217156, 0.220428, 0.223650, 0.226856,\
        0.230073, 0.233314, 0.236579, 0.239864, 0.243160, 0.246461, 0.249765, 0.253073, 0.256392, 0.259725,\
        0.263074, 0.266434, 0.269788, 0.273107, 0.276358, 0.279514, 0.282566, 0.285545, 0.288535, 0.291554,\
        0.294319, 0.296599, 0.298809, 0.301067, 0.303324, 0.305270, 0.306817, 0.308274, 0.309748, 0.311239,\
        0.312736, 0.314277, 0.315893, 0.317560, 0.319224, 0.320809, 0.322238, 0.323472, 0.324513, 0.325383,\
        0.326094, 0.326643, 0.327026, 0.327241, 0.327272, 0.327101, 0.326718, 0.326121, 0.325318, 0.324332,\
        0.323209, 0.321989, 0.320686, 0.319305, 0.317887, 0.316494, 0.315140, 0.313797, 0.312445, 0.311034,\
        0.309500, 0.307970, 0.306631, 0.305379, 0.304083, 0.302560, 0.300579, 0.298191, 0.296031, 0.294092,\
        0.292153, 0.290198, 0.288271, 0.286388, 0.284504, 0.282577, 0.280594, 0.278579, 0.276560, 0.274561,\
        0.272589, 0.270643, 0.268718, 0.266805, 0.264896, 0.262979, 0.261040, 0.259060, 0.257019, 0.254891,\
        0.252645, 0.250249, 0.247671, 0.244875, 0.241831, 0.238507, 0.234878, 0.230920, 0.226609, 0.221927,\
        0.216858],\
        [0.158403, 0.164962, 0.170827, 0.175889, 0.180069, 0.183393, 0.186026, 0.188229, 0.190281, 0.192424,\
        0.194794, 0.197383, 0.200208, 0.203476, 0.206960, 0.210379, 0.213652, 0.216802, 0.219901, 0.223007,\
        0.226151, 0.229345, 0.232582, 0.235851, 0.239134, 0.242420, 0.245699, 0.248972, 0.252242, 0.255511,\
        0.258782, 0.262049, 0.265300, 0.268515, 0.271672, 0.274757, 0.277766, 0.280716, 0.283622, 0.286441,\
        0.289039, 0.291369, 0.293581, 0.295787, 0.297983, 0.299767, 0.301075, 0.302393, 0.303755, 0.305148,\
        0.306494, 0.307906, 0.309460, 0.311102, 0.312782, 0.314405, 0.315846, 0.317058, 0.318075, 0.318953,\
        0.319700, 0.320294, 0.320729, 0.321012, 0.321117, 0.321008, 0.320670, 0.320101, 0.319299, 0.318282,\
        0.317127, 0.315908, 0.314626, 0.313226, 0.311758, 0.310389, 0.309115, 0.307853, 0.306619, 0.305387,\
        0.303882, 0.302273, 0.301128, 0.300038, 0.298921, 0.297578, 0.295784, 0.293682, 0.291615, 0.289689,\
        0.287822, 0.285970, 0.284136, 0.282318, 0.280497, 0.278650, 0.276766, 0.274853, 0.272929, 0.271008,\
        0.269100, 0.267205, 0.265323, 0.263449, 0.261575, 0.259692, 0.257786, 0.255840, 0.253833, 0.251739,\
        0.249527, 0.247162, 0.244607, 0.241827, 0.238786, 0.235454, 0.231805, 0.227815, 0.223461, 0.218721,\
        0.213577],\
        [0.152877, 0.159373, 0.165199, 0.170288, 0.174613, 0.178219, 0.181234, 0.183852, 0.186287, 0.188724,\
        0.191280, 0.194008, 0.196977, 0.200323, 0.203810, 0.207163, 0.210284, 0.213244, 0.216185, 0.219182,\
        0.222256, 0.225409, 0.228630, 0.231892, 0.235168, 0.238435, 0.241683, 0.244910, 0.248119, 0.251314,\
        0.254500, 0.257672, 0.260824, 0.263942, 0.267011, 0.270021, 0.272965, 0.275843, 0.278642, 0.281322,\
        0.283827, 0.286148, 0.288344, 0.290463, 0.292449, 0.294093, 0.295406, 0.296651, 0.297930, 0.299246,\
        0.300439, 0.301738, 0.303260, 0.304863, 0.306568, 0.308262, 0.309699, 0.310847, 0.311800, 0.312679,\
        0.313480, 0.314109, 0.314575, 0.314934, 0.315133, 0.315091, 0.314796, 0.314276, 0.313496, 0.312426,\
        0.311207, 0.310011, 0.308829, 0.307448, 0.305832, 0.304504, 0.303345, 0.302137, 0.300965, 0.299957,\
        0.298579, 0.297006, 0.295861, 0.294843, 0.293764, 0.292485, 0.290894, 0.289068, 0.287183, 0.285341,\
        0.283545, 0.281772, 0.280013, 0.278260, 0.276500, 0.274720, 0.272911, 0.271077, 0.269227, 0.267369,\
        0.265512, 0.263659, 0.261811, 0.259965, 0.258116, 0.256258, 0.254377, 0.252460, 0.250485, 0.248426,\
        0.246249, 0.243918, 0.241391, 0.238627, 0.235590, 0.232247, 0.228574, 0.224549, 0.220151, 0.215351,\
        0.210127],\
        [0.147014, 0.153492, 0.159339, 0.164518, 0.169029, 0.172922, 0.176303, 0.179320, 0.182139, 0.184906,\
        0.187736, 0.190709, 0.193897, 0.197317, 0.200779, 0.204075, 0.206965, 0.209661, 0.212447, 0.215351,\
        0.218365, 0.221494, 0.224717, 0.227987, 0.231260, 0.234506, 0.237712, 0.240879, 0.244013, 0.247124,\
        0.250216, 0.253291, 0.256345, 0.259369, 0.262352, 0.265281, 0.268147, 0.270939, 0.273638, 0.276215,\
        0.278643, 0.280917, 0.283056, 0.285063, 0.286891, 0.288465, 0.289810, 0.291049, 0.292270, 0.293479,\
        0.294648, 0.295919, 0.297363, 0.298897, 0.300607, 0.302381, 0.303762, 0.304808, 0.305649, 0.306540,\
        0.307463, 0.308090, 0.308525, 0.308988, 0.309315, 0.309331, 0.309049, 0.308614, 0.307940, 0.306777,\
        0.305426, 0.304264, 0.303269, 0.302112, 0.300206, 0.298913, 0.297836, 0.296687, 0.295577, 0.294544,\
        0.293335, 0.292006, 0.290839, 0.289786, 0.288700, 0.287466, 0.286024, 0.284404, 0.282697, 0.280977,\
        0.279268, 0.277572, 0.275882, 0.274190, 0.272487, 0.270764, 0.269015, 0.267241, 0.265445, 0.263635,\
        0.261816, 0.259992, 0.258165, 0.256336, 0.254502, 0.252658, 0.250795, 0.248899, 0.246952, 0.244927,\
        0.242789, 0.240497, 0.238002, 0.235259, 0.232224, 0.228866, 0.225160, 0.221092, 0.216648, 0.211792,\
        0.206483],\
        [0.140869, 0.147372, 0.153285, 0.158594, 0.163313, 0.167492, 0.171218, 0.174611, 0.177798, 0.180904,\
        0.184030, 0.187256, 0.190630, 0.194136, 0.197663, 0.200966, 0.203500, 0.205917, 0.208660, 0.211504,\
        0.214466, 0.217594, 0.220847, 0.224144, 0.227419, 0.230634, 0.233782, 0.236871, 0.239914, 0.242926,\
        0.245916, 0.248890, 0.251847, 0.254779, 0.257675, 0.260523, 0.263308, 0.266014, 0.268623, 0.271114,\
        0.273467, 0.275676, 0.277742, 0.279655, 0.281389, 0.282923, 0.284281, 0.285529, 0.286727, 0.287905,\
        0.289090, 0.290356, 0.291745, 0.293255, 0.294914, 0.296559, 0.297889, 0.298879, 0.299641, 0.300556,\
        0.301702, 0.302225, 0.302525, 0.303170, 0.303643, 0.303707, 0.303345, 0.303015, 0.302657, 0.301376,\
        0.299820, 0.298727, 0.297796, 0.296784, 0.295132, 0.293702, 0.292548, 0.291440, 0.290366, 0.289309,\
        0.288189, 0.287022, 0.285901, 0.284831, 0.283736, 0.282540, 0.281202, 0.279731, 0.278173, 0.276574,\
        0.274960, 0.273342, 0.271719, 0.270085, 0.268434, 0.266761, 0.265060, 0.263330, 0.261575, 0.259797,\
        0.258001, 0.256193, 0.254375, 0.252550, 0.250718, 0.248876, 0.247019, 0.245136, 0.243211, 0.241218,\
        0.239123, 0.236876, 0.234422, 0.231704, 0.228672, 0.225290, 0.221537, 0.217407, 0.212911, 0.208017,\
        0.202614],\
        [0.134477, 0.141037, 0.147050, 0.152514, 0.157450, 0.161906, 0.165956, 0.169696, 0.173228, 0.176654,\
        0.180059, 0.183507, 0.187026, 0.190591, 0.194098, 0.197270, 0.199591, 0.202041, 0.204940, 0.207650,\
        0.210546, 0.213715, 0.217045, 0.220389, 0.223659, 0.226823, 0.229887, 0.232873, 0.235804, 0.238703,\
        0.241582, 0.244452, 0.247311, 0.250154, 0.252967, 0.255736, 0.258443, 0.261070, 0.263598, 0.266011,\
        0.268292, 0.270433, 0.272429, 0.274270, 0.275946, 0.277455, 0.278818, 0.280079, 0.281283, 0.282466,\
        0.283667, 0.284930, 0.286288, 0.287747, 0.289275, 0.290750, 0.292016, 0.293028, 0.293896, 0.294861,\
        0.295893, 0.296429, 0.296794, 0.297457, 0.297966, 0.298079, 0.297722, 0.297442, 0.297184, 0.296062,\
        0.294624, 0.293450, 0.292431, 0.291329, 0.289982, 0.288640, 0.287441, 0.286330, 0.285264, 0.284214,\
        0.283146, 0.282064, 0.280992, 0.279930, 0.278842, 0.277681, 0.276420, 0.275058, 0.273618, 0.272127,\
        0.270605, 0.269063, 0.267504, 0.265926, 0.264324, 0.262693, 0.261031, 0.259334, 0.257605, 0.255846,\
        0.254060, 0.252253, 0.250431, 0.248595, 0.246751, 0.244896, 0.243031, 0.241148, 0.239236, 0.237274,\
        0.235224, 0.233034, 0.230633, 0.227948, 0.224919, 0.221506, 0.217691, 0.213462, 0.208857, 0.203996,\
        0.198493],\
        [0.127858, 0.134501, 0.140635, 0.146268, 0.151423, 0.156144, 0.160494, 0.164551, 0.168397, 0.172113,\
        0.175767, 0.179403, 0.183033, 0.186617, 0.190043, 0.193109, 0.195759, 0.198606, 0.201294, 0.203713,\
        0.206603, 0.209904, 0.213366, 0.216762, 0.219997, 0.223068, 0.226010, 0.228864, 0.231662, 0.234432,\
        0.237192, 0.239955, 0.242719, 0.245477, 0.248214, 0.250911, 0.253547, 0.256103, 0.258559, 0.260901,\
        0.263115, 0.265193, 0.267127, 0.268914, 0.270550, 0.272042, 0.273410, 0.274685, 0.275904, 0.277101,\
        0.278313, 0.279569, 0.280891, 0.282275, 0.283682, 0.285031, 0.286239, 0.287286, 0.288235, 0.289175,\
        0.290045, 0.290682, 0.291199, 0.291749, 0.292176, 0.292302, 0.292131, 0.291857, 0.291417, 0.290530,\
        0.289358, 0.288210, 0.287127, 0.286007, 0.284794, 0.283563, 0.282393, 0.281290, 0.280233, 0.279197,\
        0.278165, 0.277130, 0.276095, 0.275054, 0.273984, 0.272858, 0.271657, 0.270379, 0.269031, 0.267631,\
        0.266191, 0.264720, 0.263222, 0.261695, 0.260137, 0.258544, 0.256913, 0.255240, 0.253526, 0.251773,\
        0.249984, 0.248166, 0.246323, 0.244463, 0.242590, 0.240708, 0.238817, 0.236918, 0.235005, 0.233065,\
        0.231064, 0.228943, 0.226615, 0.223979, 0.220952, 0.217497, 0.213617, 0.209298, 0.204424, 0.199400,\
        0.193970],\
        [0.121021, 0.127764, 0.134034, 0.139842, 0.145211, 0.150183, 0.154810, 0.159155, 0.163285, 0.167262,\
        0.171137, 0.174941, 0.178675, 0.182299, 0.185737, 0.188911, 0.191863, 0.194729, 0.197300, 0.199751,\
        0.202763, 0.206286, 0.209907, 0.213302, 0.216430, 0.219344, 0.222119, 0.224810, 0.227456, 0.230085,\
        0.232720, 0.235375, 0.238051, 0.240734, 0.243406, 0.246040, 0.248615, 0.251108, 0.253501, 0.255780,\
        0.257932, 0.259952, 0.261835, 0.263578, 0.265185, 0.266665, 0.268035, 0.269323, 0.270557, 0.271767,\
        0.272982, 0.274224, 0.275505, 0.276816, 0.278126, 0.279385, 0.280547, 0.281599, 0.282565, 0.283469,\
        0.284280, 0.284953, 0.285519, 0.286013, 0.286370, 0.286509, 0.286421, 0.286154, 0.285676, 0.284916,\
        0.283940, 0.282890, 0.281827, 0.280735, 0.279603, 0.278458, 0.277340, 0.276265, 0.275226, 0.274210,\
        0.273205, 0.272202, 0.271196, 0.270177, 0.269131, 0.268039, 0.266888, 0.265675, 0.264401, 0.263076,\
        0.261707, 0.260300, 0.258857, 0.257378, 0.255861, 0.254300, 0.252693, 0.251037, 0.249330, 0.247572,\
        0.245768, 0.243924, 0.242047, 0.240147, 0.238231, 0.236304, 0.234369, 0.232431, 0.230495, 0.228561,\
        0.226607, 0.224573, 0.222347, 0.219785, 0.216761, 0.213233, 0.209262, 0.204930, 0.200137, 0.194625,\
        0.188957],\
        [0.113968, 0.120824, 0.127238, 0.133221, 0.138799, 0.144006, 0.148888, 0.153495, 0.157881, 0.162094,\
        0.166174, 0.170141, 0.173993, 0.177707, 0.181239, 0.184565, 0.187699, 0.190660, 0.193438, 0.196213,\
        0.199312, 0.203104, 0.206739, 0.209994, 0.212906, 0.215586, 0.218155, 0.220668, 0.223149, 0.225628,\
        0.228136, 0.230690, 0.233288, 0.235911, 0.238531, 0.241117, 0.243641, 0.246081, 0.248417, 0.250639,\
        0.252736, 0.254704, 0.256541, 0.258248, 0.259831, 0.261300, 0.262672, 0.263968, 0.265212, 0.266430,\
        0.267642, 0.268865, 0.270104, 0.271353, 0.272588, 0.273778, 0.274895, 0.275928, 0.276881, 0.277756,\
        0.278537, 0.279210, 0.279777, 0.280235, 0.280553, 0.280691, 0.280634, 0.280384, 0.279932, 0.279266,\
        0.278426, 0.277482, 0.276484, 0.275450, 0.274387, 0.273314, 0.272252, 0.271216, 0.270205, 0.269213,\
        0.268233, 0.267255, 0.266273, 0.265277, 0.264256, 0.263196, 0.262088, 0.260927, 0.259713, 0.258450,\
        0.257141, 0.255790, 0.254397, 0.252961, 0.251479, 0.249947, 0.248360, 0.246715, 0.245007, 0.243237,\
        0.241407, 0.239524, 0.237600, 0.235646, 0.233673, 0.231685, 0.229686, 0.227683, 0.225690, 0.223731,\
        0.221810, 0.219877, 0.217796, 0.215357, 0.212349, 0.208694, 0.204537, 0.200052, 0.195177, 0.189652,\
        0.183695],\
        [0.106699, 0.113676, 0.120236, 0.126394, 0.132171, 0.137599, 0.142716, 0.147563, 0.152183, 0.156615,\
        0.160889, 0.165022, 0.169015, 0.172857, 0.176531, 0.180030, 0.183365, 0.186563, 0.189689, 0.192905,\
        0.196457, 0.200301, 0.203647, 0.206707, 0.209293, 0.211677, 0.214043, 0.216387, 0.218704, 0.221028,\
        0.223410, 0.225873, 0.228412, 0.230995, 0.233582, 0.236135, 0.238620, 0.241013, 0.243300, 0.245468,\
        0.247514, 0.249435, 0.251231, 0.252907, 0.254468, 0.255927, 0.257297, 0.258596, 0.259845, 0.261064,\
        0.262268, 0.263469, 0.264671, 0.265867, 0.267041, 0.268175, 0.269248, 0.270251, 0.271178, 0.272024,\
        0.272780, 0.273435, 0.273984, 0.274416, 0.274709, 0.274841, 0.274798, 0.274573, 0.274164, 0.273577,\
        0.272840, 0.271994, 0.271077, 0.270114, 0.269121, 0.268114, 0.267108, 0.266115, 0.265138, 0.264174,\
        0.263219, 0.262265, 0.261305, 0.260332, 0.259334, 0.258303, 0.257232, 0.256114, 0.254949, 0.253737,\
        0.252479, 0.251175, 0.249825, 0.248427, 0.246978, 0.245471, 0.243902, 0.242264, 0.240552, 0.238762,\
        0.236898, 0.234965, 0.232980, 0.230960, 0.228918, 0.226860, 0.224782, 0.222687, 0.220594, 0.218554,\
        0.216627, 0.214795, 0.212900, 0.210662, 0.207731, 0.203900, 0.199504, 0.194819, 0.189736, 0.184133,\
        0.178047],\
        [0.099210, 0.106313, 0.113020, 0.119348, 0.125315, 0.130951, 0.136285, 0.141354, 0.146192, 0.150829,\
        0.155292, 0.159595, 0.163746, 0.167742, 0.171584, 0.175275, 0.178837, 0.182311, 0.185774, 0.189338,\
        0.193078, 0.196801, 0.200279, 0.203272, 0.205355, 0.207467, 0.209717, 0.211935, 0.214090, 0.216251,\
        0.218511, 0.220906, 0.223414, 0.225983, 0.228558, 0.231091, 0.233546, 0.235899, 0.238138, 0.240257,\
        0.242254, 0.244130, 0.245888, 0.247534, 0.249075, 0.250521, 0.251886, 0.253185, 0.254434, 0.255648,\
        0.256840, 0.258019, 0.259187, 0.260338, 0.261463, 0.262548, 0.263579, 0.264546, 0.265441, 0.266257,\
        0.266986, 0.267617, 0.268143, 0.268552, 0.268827, 0.268954, 0.268921, 0.268723, 0.268360, 0.267842,\
        0.267190, 0.266431, 0.265596, 0.264707, 0.263783, 0.262839, 0.261889, 0.260941, 0.260001, 0.259067,\
        0.258138, 0.257207, 0.256269, 0.255317, 0.254342, 0.253338, 0.252298, 0.251216, 0.250091, 0.248920,\
        0.247704, 0.246440, 0.245128, 0.243764, 0.242343, 0.240859, 0.239306, 0.237674, 0.235956, 0.234146,\
        0.232240, 0.230247, 0.228186, 0.226087, 0.223972, 0.221842, 0.219682, 0.217480, 0.215242, 0.213039,\
        0.211029, 0.209264, 0.207552, 0.205546, 0.202847, 0.198833, 0.194216, 0.189270, 0.183951, 0.178192,\
        0.171984],\
        [0.091497, 0.098726, 0.105581, 0.112073, 0.118222, 0.124052, 0.129590, 0.134864, 0.139905, 0.144738,\
        0.149385, 0.153862, 0.158179, 0.162343, 0.166362, 0.170250, 0.174033, 0.177752, 0.181466, 0.185230,\
        0.189044, 0.192804, 0.196372, 0.199219, 0.200763, 0.202925, 0.205170, 0.207301, 0.209281, 0.211260,\
        0.213414, 0.215779, 0.218294, 0.220878, 0.223461, 0.225986, 0.228415, 0.230728, 0.232919, 0.234987,\
        0.236936, 0.238769, 0.240491, 0.242108, 0.243628, 0.245061, 0.246419, 0.247713, 0.248957, 0.250164,\
        0.251342, 0.252499, 0.253636, 0.254749, 0.255832, 0.256874, 0.257864, 0.258795, 0.259656, 0.260441,\
        0.261141, 0.261747, 0.262249, 0.262637, 0.262898, 0.263022, 0.263000, 0.262828, 0.262509, 0.262051,\
        0.261471, 0.260790, 0.260031, 0.259215, 0.258358, 0.257475, 0.256578, 0.255676, 0.254773, 0.253871,\
        0.252968, 0.252061, 0.251144, 0.250212, 0.249259, 0.248278, 0.247264, 0.246212, 0.245119, 0.243982,\
        0.242800, 0.241569, 0.240288, 0.238953, 0.237557, 0.236094, 0.234556, 0.232933, 0.231212, 0.229383,\
        0.227437, 0.225374, 0.223220, 0.221024, 0.218836, 0.216647, 0.214415, 0.212119, 0.209733, 0.207265,\
        0.205057, 0.203288, 0.201674, 0.199688, 0.197029, 0.193104, 0.188382, 0.183252, 0.177746, 0.171836,\
        0.165508],\
        [0.083554, 0.090911, 0.097909, 0.104560, 0.110882, 0.116896, 0.122624, 0.128092, 0.133324, 0.138343,\
        0.143170, 0.147819, 0.152304, 0.156637, 0.160830, 0.164902, 0.168876, 0.172786, 0.176665, 0.180531,\
        0.184356, 0.188044, 0.191397, 0.194028, 0.196011, 0.198177, 0.200374, 0.202470, 0.204236, 0.206025,\
        0.208119, 0.210512, 0.213074, 0.215691, 0.218296, 0.220819, 0.223220, 0.225487, 0.227625, 0.229641,\
        0.231541, 0.233332, 0.235018, 0.236608, 0.238107, 0.239526, 0.240874, 0.242161, 0.243398, 0.244594,\
        0.245758, 0.246894, 0.248003, 0.249083, 0.250128, 0.251131, 0.252085, 0.252979, 0.253808, 0.254561,\
        0.255232, 0.255812, 0.256292, 0.256662, 0.256912, 0.257035, 0.257024, 0.256877, 0.256598, 0.256193,\
        0.255677, 0.255064, 0.254375, 0.253625, 0.252832, 0.252007, 0.251161, 0.250304, 0.249438, 0.248568,\
        0.247691, 0.246807, 0.245910, 0.244998, 0.244064, 0.243103, 0.242111, 0.241082, 0.240015, 0.238904,\
        0.237749, 0.236545, 0.235289, 0.233977, 0.232603, 0.231159, 0.229636, 0.228024, 0.226307, 0.224468,\
        0.222490, 0.220359, 0.218089, 0.215763, 0.213503, 0.211288, 0.208993, 0.206625, 0.204197, 0.201484,\
        0.198972, 0.197098, 0.195304, 0.193139, 0.190258, 0.186440, 0.181822, 0.176657, 0.171054, 0.165039,\
        0.158613],\
        [0.075376, 0.082860, 0.089999, 0.096803, 0.103290, 0.109476, 0.115384, 0.121033, 0.126446, 0.131643,\
        0.136642, 0.141459, 0.146108, 0.150604, 0.154960, 0.159193, 0.163324, 0.167374, 0.171355, 0.175262,\
        0.179051, 0.182627, 0.185835, 0.188532, 0.190835, 0.193045, 0.195217, 0.197236, 0.198901, 0.200593,\
        0.202701, 0.205175, 0.207796, 0.210429, 0.213070, 0.215587, 0.217942, 0.220151, 0.222230, 0.224192,\
        0.226044, 0.227794, 0.229447, 0.231011, 0.232490, 0.233895, 0.235231, 0.236510, 0.237738, 0.238923,\
        0.240072, 0.241188, 0.242272, 0.243323, 0.244335, 0.245305, 0.246224, 0.247085, 0.247881, 0.248605,\
        0.249248, 0.249803, 0.250262, 0.250616, 0.250858, 0.250982, 0.250983, 0.250860, 0.250616, 0.250258,\
        0.249796, 0.249244, 0.248617, 0.247929, 0.247193, 0.246422, 0.245625, 0.244810, 0.243980, 0.243140,\
        0.242290, 0.241427, 0.240550, 0.239655, 0.238738, 0.237793, 0.236818, 0.235808, 0.234760, 0.233670,\
        0.232534, 0.231350, 0.230113, 0.228818, 0.227460, 0.226032, 0.224524, 0.222924, 0.221217, 0.219385,\
        0.217398, 0.215221, 0.212824, 0.210292, 0.207947, 0.205796, 0.203413, 0.200935, 0.198476, 0.195786,\
        0.193127, 0.190871, 0.188680, 0.186178, 0.183074, 0.179225, 0.174661, 0.169511, 0.163872, 0.157794,\
        0.151298],\
        [0.066959, 0.074569, 0.081844, 0.088796, 0.095439, 0.101790, 0.107866, 0.113686, 0.119270, 0.124635,\
        0.129798, 0.134776, 0.139581, 0.144228, 0.148732, 0.153105, 0.157362, 0.161513, 0.165558, 0.169480,\
        0.173232, 0.176740, 0.179913, 0.182705, 0.185184, 0.187480, 0.189641, 0.191618, 0.193403, 0.195228,\
        0.197345, 0.199897, 0.202459, 0.205076, 0.207796, 0.210267, 0.212548, 0.214685, 0.216702, 0.218611,\
        0.220419, 0.222132, 0.223755, 0.225295, 0.226757, 0.228147, 0.229474, 0.230743, 0.231961, 0.233136,\
        0.234271, 0.235368, 0.236430, 0.237455, 0.238439, 0.239377, 0.240265, 0.241096, 0.241863, 0.242558,\
        0.243176, 0.243709, 0.244149, 0.244490, 0.244726, 0.244851, 0.244863, 0.244762, 0.244549, 0.244232,\
        0.243819, 0.243320, 0.242748, 0.242114, 0.241431, 0.240710, 0.239957, 0.239181, 0.238386, 0.237575,\
        0.236748, 0.235906, 0.235047, 0.234167, 0.233263, 0.232332, 0.231370, 0.230373, 0.229338, 0.228261,\
        0.227138, 0.225966, 0.224741, 0.223457, 0.222110, 0.220691, 0.219193, 0.217604, 0.215911, 0.214096,\
        0.212130, 0.209968, 0.207520, 0.204671, 0.202101, 0.200160, 0.197674, 0.195069, 0.192456, 0.189747,\
        0.187033, 0.184442, 0.181838, 0.178968, 0.175613, 0.171649, 0.167061, 0.161903, 0.156242, 0.150122,\
        0.143576],\
        [0.058298, 0.066031, 0.073438, 0.080532, 0.087325, 0.093831, 0.100067, 0.106049, 0.111794, 0.117318,\
        0.122637, 0.127766, 0.132717, 0.137505, 0.142140, 0.146633, 0.150993, 0.155223, 0.159316, 0.163250,\
        0.166987, 0.170476, 0.173667, 0.176545, 0.179148, 0.181539, 0.183757, 0.185818, 0.187778, 0.189787,\
        0.192011, 0.194488, 0.196976, 0.199754, 0.202427, 0.204779, 0.206972, 0.209042, 0.211005, 0.212868,\
        0.214638, 0.216321, 0.217920, 0.219440, 0.220887, 0.222266, 0.223583, 0.224843, 0.226053, 0.227217,\
        0.228339, 0.229421, 0.230463, 0.231465, 0.232424, 0.233336, 0.234196, 0.234999, 0.235739, 0.236409,\
        0.237004, 0.237516, 0.237941, 0.238270, 0.238501, 0.238629, 0.238652, 0.238570, 0.238386, 0.238104,\
        0.237733, 0.237280, 0.236756, 0.236172, 0.235536, 0.234858, 0.234146, 0.233406, 0.232642, 0.231857,\
        0.231053, 0.230229, 0.229385, 0.228517, 0.227625, 0.226703, 0.225749, 0.224760, 0.223732, 0.222661,\
        0.221544, 0.220377, 0.219156, 0.217875, 0.216530, 0.215113, 0.213616, 0.212030, 0.210344, 0.208542,\
        0.206603, 0.204495, 0.202155, 0.199301, 0.196341, 0.194094, 0.191624, 0.188947, 0.186200, 0.183390,\
        0.180546, 0.177688, 0.174734, 0.171527, 0.167912, 0.163787, 0.159116, 0.153914, 0.148217, 0.142059,\
        0.135468]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=50 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax50vt0
    zgrid_H_Hmax_vmax50vt0=[[0.57463, 0.58281, 0.59056, 0.59791, 0.60485, 0.61140, 0.61757, 0.62337, 0.62882, 0.63391, 0.63866,\
        0.64304, 0.64702, 0.65059, 0.65369, 0.65627, 0.65832, 0.65985, 0.66090, 0.66156, 0.66192, 0.66206,\
        0.66203, 0.66188, 0.66163, 0.66135, 0.66110, 0.66098, 0.66110, 0.66158, 0.66252, 0.66401, 0.66611,\
        0.66887, 0.67233, 0.67652, 0.68143, 0.68703, 0.69308, 0.69897, 0.70416, 0.70890, 0.71324, 0.71710,\
        0.72040, 0.72308, 0.72511, 0.72651, 0.72730, 0.72753, 0.72727, 0.72657, 0.72548, 0.72405, 0.72231,\
        0.72029, 0.71799, 0.71544, 0.71264, 0.70960, 0.70633, 0.70284, 0.69914, 0.69526, 0.69118, 0.68705,\
        0.68314, 0.67941, 0.67567, 0.67191, 0.66820, 0.66455, 0.66095, 0.65741, 0.65393, 0.65052, 0.64719,\
        0.64395, 0.64079, 0.63772, 0.63469, 0.63169, 0.62869, 0.62566, 0.62257, 0.61941, 0.61616, 0.61281,\
        0.60934, 0.60576, 0.60206, 0.59824, 0.59427, 0.59018, 0.58615, 0.58222, 0.57832, 0.57440, 0.57037,\
        0.56617, 0.56174, 0.55708, 0.55221, 0.54718, 0.54204, 0.53684, 0.53160, 0.52635, 0.52111, 0.51588,\
        0.51066, 0.50544, 0.50019, 0.49474, 0.48933, 0.48414, 0.47912, 0.47412, 0.46906, 0.46386, 0.45841],\
        [0.58381, 0.59190, 0.59957, 0.60681, 0.61364, 0.62007, 0.62611, 0.63180, 0.63714, 0.64215, 0.64683,\
        0.65116, 0.65512, 0.65867, 0.66175, 0.66428, 0.66621, 0.66754, 0.66834, 0.66872, 0.66881, 0.66869,\
        0.66842, 0.66802, 0.66750, 0.66689, 0.66627, 0.66577, 0.66550, 0.66563, 0.66626, 0.66749, 0.66939,\
        0.67200, 0.67536, 0.67950, 0.68446, 0.69022, 0.69672, 0.70323, 0.70914, 0.71461, 0.71962, 0.72404,\
        0.72779, 0.73079, 0.73304, 0.73457, 0.73542, 0.73567, 0.73540, 0.73469, 0.73360, 0.73218, 0.73048,\
        0.72850, 0.72626, 0.72378, 0.72105, 0.71807, 0.71486, 0.71141, 0.70773, 0.70384, 0.69970, 0.69538,\
        0.69159, 0.68799, 0.68418, 0.68030, 0.67650, 0.67277, 0.66909, 0.66544, 0.66184, 0.65830, 0.65486,\
        0.65153, 0.64831, 0.64519, 0.64213, 0.63910, 0.63606, 0.63299, 0.62986, 0.62664, 0.62333, 0.61989,\
        0.61634, 0.61265, 0.60883, 0.60488, 0.60075, 0.59650, 0.59240, 0.58846, 0.58459, 0.58069, 0.57670,\
        0.57251, 0.56802, 0.56324, 0.55823, 0.55307, 0.54781, 0.54252, 0.53723, 0.53195, 0.52669, 0.52146,\
        0.51625, 0.51103, 0.50577, 0.50039, 0.49517, 0.49034, 0.48573, 0.48106, 0.47635, 0.47154, 0.46648],\
        [0.59266, 0.60067, 0.60824, 0.61537, 0.62207, 0.62836, 0.63427, 0.63983, 0.64505, 0.64995, 0.65456,\
        0.65884, 0.66279, 0.66636, 0.66947, 0.67200, 0.67385, 0.67498, 0.67551, 0.67561, 0.67543, 0.67509,\
        0.67463, 0.67403, 0.67325, 0.67232, 0.67132, 0.67039, 0.66971, 0.66944, 0.66975, 0.67073, 0.67243,\
        0.67489, 0.67811, 0.68215, 0.68705, 0.69277, 0.69926, 0.70656, 0.71356, 0.72002, 0.72583, 0.73090,\
        0.73513, 0.73846, 0.74091, 0.74252, 0.74338, 0.74361, 0.74330, 0.74256, 0.74147, 0.74008, 0.73842,\
        0.73651, 0.73437, 0.73198, 0.72935, 0.72648, 0.72337, 0.72000, 0.71639, 0.71252, 0.70838, 0.70414,\
        0.70054, 0.69698, 0.69293, 0.68890, 0.68501, 0.68120, 0.67742, 0.67363, 0.66986, 0.66615, 0.66257,\
        0.65913, 0.65582, 0.65264, 0.64953, 0.64646, 0.64339, 0.64027, 0.63708, 0.63379, 0.63039, 0.62687,\
        0.62320, 0.61939, 0.61543, 0.61130, 0.60701, 0.60266, 0.59842, 0.59452, 0.59059, 0.58668, 0.58278,\
        0.57859, 0.57399, 0.56903, 0.56384, 0.55852, 0.55314, 0.54776, 0.54241, 0.53710, 0.53184, 0.52662,\
        0.52142, 0.51623, 0.51099, 0.50572, 0.50045, 0.49644, 0.49210, 0.48767, 0.48331, 0.47893, 0.47430],\
        [0.60118, 0.60911, 0.61658, 0.62359, 0.63015, 0.63629, 0.64204, 0.64743, 0.65251, 0.65729, 0.66181,\
        0.66604, 0.66998, 0.67359, 0.67678, 0.67939, 0.68122, 0.68216, 0.68239, 0.68219, 0.68176, 0.68124,\
        0.68067, 0.67995, 0.67898, 0.67773, 0.67631, 0.67492, 0.67377, 0.67310, 0.67308, 0.67384, 0.67540,\
        0.67776, 0.68087, 0.68476, 0.68964, 0.69562, 0.70248, 0.71026, 0.71814, 0.72552, 0.73214, 0.73786,\
        0.74255, 0.74618, 0.74876, 0.75039, 0.75119, 0.75133, 0.75095, 0.75016, 0.74905, 0.74769, 0.74611,\
        0.74430, 0.74228, 0.74003, 0.73755, 0.73482, 0.73186, 0.72864, 0.72516, 0.72144, 0.71751, 0.71359,\
        0.70989, 0.70611, 0.70194, 0.69782, 0.69384, 0.68992, 0.68601, 0.68203, 0.67802, 0.67409, 0.67033,\
        0.66675, 0.66335, 0.66009, 0.65693, 0.65381, 0.65068, 0.64750, 0.64423, 0.64086, 0.63737, 0.63374,\
        0.62996, 0.62601, 0.62190, 0.61763, 0.61321, 0.60876, 0.60443, 0.60049, 0.59616, 0.59234, 0.58865,\
        0.58444, 0.57962, 0.57440, 0.56898, 0.56348, 0.55798, 0.55252, 0.54713, 0.54182, 0.53659, 0.53141,\
        0.52629, 0.52119, 0.51610, 0.51110, 0.50647, 0.50244, 0.49813, 0.49394, 0.48996, 0.48603, 0.48190],\
        [0.60937, 0.61723, 0.62460, 0.63148, 0.63788, 0.64384, 0.64940, 0.65460, 0.65951, 0.66415, 0.66855,\
        0.67271, 0.67661, 0.68024, 0.68356, 0.68638, 0.68829, 0.68901, 0.68891, 0.68844, 0.68781, 0.68717,\
        0.68659, 0.68589, 0.68477, 0.68319, 0.68132, 0.67940, 0.67775, 0.67665, 0.67634, 0.67692, 0.67845,\
        0.68083, 0.68394, 0.68767, 0.69240, 0.69882, 0.70644, 0.71479, 0.72334, 0.73148, 0.73881, 0.74509,\
        0.75018, 0.75401, 0.75663, 0.75817, 0.75882, 0.75879, 0.75828, 0.75741, 0.75630, 0.75499, 0.75351,\
        0.75184, 0.74998, 0.74790, 0.74561, 0.74308, 0.74031, 0.73730, 0.73404, 0.73056, 0.72691, 0.72318,\
        0.71944, 0.71554, 0.71142, 0.70724, 0.70311, 0.69902, 0.69494, 0.69066, 0.68634, 0.68214, 0.67817,\
        0.67443, 0.67090, 0.66755, 0.66433, 0.66115, 0.65796, 0.65469, 0.65133, 0.64786, 0.64426, 0.64052,\
        0.63661, 0.63254, 0.62829, 0.62388, 0.61935, 0.61478, 0.61030, 0.60585, 0.60139, 0.59789, 0.59438,\
        0.58998, 0.58476, 0.57921, 0.57356, 0.56790, 0.56231, 0.55680, 0.55141, 0.54613, 0.54097, 0.53589,\
        0.53090, 0.52598, 0.52116, 0.51650, 0.51215, 0.50804, 0.50395, 0.50003, 0.49639, 0.49285, 0.48932],\
        [0.61725, 0.62504, 0.63230, 0.63905, 0.64528, 0.65104, 0.65637, 0.66135, 0.66603, 0.67049, 0.67476,\
        0.67883, 0.68266, 0.68624, 0.68960, 0.69274, 0.69499, 0.69542, 0.69501, 0.69442, 0.69366, 0.69289,\
        0.69245, 0.69194, 0.69069, 0.68875, 0.68638, 0.68389, 0.68169, 0.68016, 0.67957, 0.68007, 0.68168,\
        0.68431, 0.68773, 0.69146, 0.69565, 0.70226, 0.71088, 0.72009, 0.72931, 0.73807, 0.74596, 0.75271,\
        0.75808, 0.76200, 0.76451, 0.76582, 0.76619, 0.76591, 0.76522, 0.76427, 0.76316, 0.76193, 0.76058,\
        0.75910, 0.75744, 0.75558, 0.75351, 0.75121, 0.74868, 0.74593, 0.74295, 0.73976, 0.73638, 0.73286,\
        0.72919, 0.72532, 0.72126, 0.71707, 0.71281, 0.70851, 0.70414, 0.69950, 0.69483, 0.69035, 0.68613,\
        0.68219, 0.67851, 0.67506, 0.67176, 0.66851, 0.66522, 0.66185, 0.65837, 0.65478, 0.65107, 0.64721,\
        0.64318, 0.63897, 0.63459, 0.63003, 0.62537, 0.62065, 0.61595, 0.61135, 0.60705, 0.60332, 0.59983,\
        0.59480, 0.58910, 0.58330, 0.57749, 0.57174, 0.56610, 0.56060, 0.55525, 0.55005, 0.54499, 0.54007,\
        0.53527, 0.53058, 0.52604, 0.52168, 0.51755, 0.51361, 0.50979, 0.50614, 0.50268, 0.49935, 0.49665],\
        [0.62479, 0.63253, 0.63970, 0.64631, 0.65236, 0.65789, 0.66296, 0.66767, 0.67209, 0.67633, 0.68044,\
        0.68441, 0.68813, 0.69157, 0.69477, 0.69790, 0.70079, 0.70113, 0.70076, 0.70027, 0.69954, 0.69854,\
        0.69835, 0.69819, 0.69669, 0.69442, 0.69154, 0.68841, 0.68564, 0.68369, 0.68288, 0.68337, 0.68517,\
        0.68819, 0.69229, 0.69697, 0.70075, 0.70769, 0.71674, 0.72645, 0.73613, 0.74533, 0.75364, 0.76071,\
        0.76626, 0.77012, 0.77235, 0.77324, 0.77321, 0.77259, 0.77168, 0.77064, 0.76956, 0.76846, 0.76731,\
        0.76606, 0.76464, 0.76303, 0.76121, 0.75917, 0.75692, 0.75447, 0.75180, 0.74893, 0.74584, 0.74255,\
        0.73903, 0.73528, 0.73129, 0.72710, 0.72274, 0.71822, 0.71352, 0.70860, 0.70361, 0.69880, 0.69430,\
        0.69010, 0.68621, 0.68263, 0.67925, 0.67590, 0.67248, 0.66895, 0.66533, 0.66160, 0.65776, 0.65378,\
        0.64964, 0.64531, 0.64078, 0.63607, 0.63123, 0.62632, 0.62142, 0.61665, 0.61214, 0.60792, 0.60351,\
        0.59817, 0.59246, 0.58662, 0.58077, 0.57499, 0.56937, 0.56392, 0.55866, 0.55359, 0.54869, 0.54395,\
        0.53938, 0.53495, 0.53070, 0.52662, 0.52274, 0.51904, 0.51549, 0.51211, 0.50890, 0.50596, 0.50323],\
        [0.63200, 0.63970, 0.64681, 0.65329, 0.65915, 0.66443, 0.66921, 0.67359, 0.67768, 0.68163, 0.68556,\
        0.68949, 0.69314, 0.69643, 0.69953, 0.70237, 0.70505, 0.70628, 0.70648, 0.70625, 0.70567, 0.70491,\
        0.70462, 0.70418, 0.70258, 0.70014, 0.69678, 0.69303, 0.68970, 0.68739, 0.68639, 0.68695, 0.68901,\
        0.69244, 0.69703, 0.70257, 0.70812, 0.71524, 0.72419, 0.73394, 0.74376, 0.75315, 0.76169, 0.76899,\
        0.77462, 0.77827, 0.78000, 0.78029, 0.77970, 0.77869, 0.77755, 0.77645, 0.77547, 0.77457, 0.77368,\
        0.77271, 0.77158, 0.77024, 0.76867, 0.76690, 0.76494, 0.76281, 0.76049, 0.75796, 0.75519, 0.75216,\
        0.74885, 0.74526, 0.74138, 0.73721, 0.73277, 0.72808, 0.72313, 0.71798, 0.71275, 0.70763, 0.70275,\
        0.69821, 0.69403, 0.69026, 0.68679, 0.68332, 0.67969, 0.67596, 0.67215, 0.66828, 0.66432, 0.66023,\
        0.65598, 0.65152, 0.64684, 0.64195, 0.63689, 0.63174, 0.62657, 0.62148, 0.61651, 0.61160, 0.60644,\
        0.60086, 0.59517, 0.58931, 0.58345, 0.57770, 0.57215, 0.56680, 0.56168, 0.55677, 0.55207, 0.54755,\
        0.54322, 0.53907, 0.53510, 0.53130, 0.52769, 0.52424, 0.52094, 0.51779, 0.51478, 0.51187, 0.50868],\
        [0.63887, 0.64657, 0.65362, 0.65999, 0.66566, 0.67068, 0.67514, 0.67917, 0.68288, 0.68641, 0.69003,\
        0.69403, 0.69780, 0.70097, 0.70416, 0.70714, 0.70970, 0.71140, 0.71223, 0.71244, 0.71223, 0.71179,\
        0.71127, 0.71036, 0.70855, 0.70579, 0.70212, 0.69784, 0.69414, 0.69158, 0.69028, 0.69106, 0.69355,\
        0.69713, 0.70260, 0.70876, 0.71571, 0.72360, 0.73259, 0.74222, 0.75193, 0.76125, 0.76981, 0.77722,\
        0.78291, 0.78623, 0.78720, 0.78668, 0.78547, 0.78406, 0.78274, 0.78166, 0.78085, 0.78023, 0.77968,\
        0.77905, 0.77824, 0.77718, 0.77586, 0.77434, 0.77267, 0.77087, 0.76892, 0.76675, 0.76432, 0.76158,\
        0.75852, 0.75512, 0.75137, 0.74726, 0.74279, 0.73798, 0.73285, 0.72750, 0.72205, 0.71666, 0.71146,\
        0.70656, 0.70201, 0.69799, 0.69442, 0.69072, 0.68677, 0.68278, 0.67878, 0.67476, 0.67068, 0.66652,\
        0.66218, 0.65760, 0.65276, 0.64765, 0.64234, 0.63688, 0.63135, 0.62583, 0.62033, 0.61481, 0.60916,\
        0.60339, 0.59750, 0.59150, 0.58563, 0.57995, 0.57450, 0.56930, 0.56435, 0.55963, 0.55515, 0.55088,\
        0.54681, 0.54293, 0.53923, 0.53571, 0.53235, 0.52913, 0.52604, 0.52305, 0.52010, 0.51705, 0.51363],\
        [0.64536, 0.65309, 0.66015, 0.66645, 0.67194, 0.67667, 0.68080, 0.68450, 0.68785, 0.69085, 0.69379,\
        0.69745, 0.70181, 0.70530, 0.70877, 0.71200, 0.71476, 0.71684, 0.71819, 0.71890, 0.71910, 0.71890,\
        0.71830, 0.71710, 0.71504, 0.71199, 0.70802, 0.70345, 0.69980, 0.69682, 0.69473, 0.69591, 0.69950,\
        0.70324, 0.70910, 0.71614, 0.72389, 0.73236, 0.74147, 0.75092, 0.76031, 0.76928, 0.77756, 0.78487,\
        0.79065, 0.79360, 0.79344, 0.79207, 0.79035, 0.78863, 0.78721, 0.78626, 0.78574, 0.78550, 0.78534,\
        0.78512, 0.78465, 0.78383, 0.78272, 0.78141, 0.78000, 0.77854, 0.77698, 0.77520, 0.77311, 0.77067,\
        0.76788, 0.76471, 0.76113, 0.75712, 0.75267, 0.74778, 0.74252, 0.73699, 0.73133, 0.72569, 0.72021,\
        0.71501, 0.71018, 0.70587, 0.70199, 0.69786, 0.69354, 0.68933, 0.68515, 0.68097, 0.67682, 0.67261,\
        0.66822, 0.66354, 0.65851, 0.65317, 0.64754, 0.64172, 0.63579, 0.62979, 0.62376, 0.61770, 0.61159,\
        0.60543, 0.59926, 0.59324, 0.58741, 0.58182, 0.57651, 0.57147, 0.56672, 0.56223, 0.55798, 0.55396,\
        0.55016, 0.54655, 0.54311, 0.53984, 0.53671, 0.53369, 0.53076, 0.52785, 0.52490, 0.52175, 0.51819],\
        [0.65141, 0.65923, 0.66636, 0.67266, 0.67800, 0.68241, 0.68617, 0.68959, 0.69278, 0.69545, 0.69760,\
        0.70043, 0.70501, 0.70951, 0.71353, 0.71712, 0.72021, 0.72267, 0.72448, 0.72566, 0.72627, 0.72633,\
        0.72581, 0.72456, 0.72241, 0.71930, 0.71536, 0.71102, 0.70708, 0.70345, 0.70057, 0.70204, 0.70627,\
        0.71114, 0.71723, 0.72456, 0.73271, 0.74146, 0.75058, 0.75976, 0.76863, 0.77693, 0.78448, 0.79116,\
        0.79669, 0.79942, 0.79787, 0.79624, 0.79432, 0.79244, 0.79105, 0.79033, 0.79023, 0.79045, 0.79072,\
        0.79092, 0.79081, 0.79019, 0.78920, 0.78804, 0.78685, 0.78572, 0.78457, 0.78319, 0.78144, 0.77928,\
        0.77676, 0.77385, 0.77050, 0.76664, 0.76224, 0.75732, 0.75195, 0.74625, 0.74038, 0.73450, 0.72877,\
        0.72329, 0.71818, 0.71346, 0.70900, 0.70447, 0.69993, 0.69558, 0.69117, 0.68683, 0.68266, 0.67848,\
        0.67409, 0.66931, 0.66410, 0.65847, 0.65250, 0.64629, 0.63992, 0.63345, 0.62692, 0.62036, 0.61379,\
        0.60725, 0.60086, 0.59475, 0.58892, 0.58343, 0.57827, 0.57342, 0.56887, 0.56460, 0.56060, 0.55684,\
        0.55329, 0.54994, 0.54675, 0.54371, 0.54078, 0.53793, 0.53511, 0.53226, 0.52930, 0.52607, 0.52245],\
        [0.65697, 0.66490, 0.67218, 0.67859, 0.68386, 0.68790, 0.69120, 0.69436, 0.69760, 0.70054, 0.70262,\
        0.70563, 0.70982, 0.71441, 0.71875, 0.72266, 0.72606, 0.72888, 0.73111, 0.73272, 0.73375, 0.73415,\
        0.73387, 0.73278, 0.73079, 0.72787, 0.72417, 0.72007, 0.71603, 0.71243, 0.71026, 0.71125, 0.71495,\
        0.72008, 0.72637, 0.73376, 0.74200, 0.75079, 0.75978, 0.76859, 0.77680, 0.78415, 0.79054, 0.79591,\
        0.79999, 0.80179, 0.80067, 0.79972, 0.79765, 0.79580, 0.79451, 0.79410, 0.79454, 0.79526, 0.79587,\
        0.79646, 0.79675, 0.79619, 0.79525, 0.79420, 0.79313, 0.79230, 0.79160, 0.79063, 0.78912, 0.78720,\
        0.78496, 0.78236, 0.77927, 0.77563, 0.77135, 0.76642, 0.76094, 0.75507, 0.74900, 0.74290, 0.73693,\
        0.73118, 0.72573, 0.72058, 0.71563, 0.71077, 0.70601, 0.70143, 0.69668, 0.69225, 0.68818, 0.68413,\
        0.67979, 0.67493, 0.66949, 0.66354, 0.65720, 0.65058, 0.64379, 0.63687, 0.62989, 0.62287, 0.61588,\
        0.60901, 0.60240, 0.59612, 0.59030, 0.58491, 0.57989, 0.57522, 0.57088, 0.56684, 0.56307, 0.55955,\
        0.55625, 0.55314, 0.55018, 0.54735, 0.54459, 0.54188, 0.53916, 0.53635, 0.53337, 0.53011, 0.52645],\
        [0.66192, 0.66997, 0.67745, 0.68412, 0.68954, 0.69316, 0.69579, 0.69881, 0.70212, 0.70538, 0.70831,\
        0.71165, 0.71572, 0.72018, 0.72458, 0.72866, 0.73231, 0.73544, 0.73803, 0.74006, 0.74151, 0.74233,\
        0.74244, 0.74173, 0.74010, 0.73756, 0.73424, 0.73046, 0.72668, 0.72349, 0.72179, 0.72241, 0.72536,\
        0.73006, 0.73614, 0.74336, 0.75146, 0.76011, 0.76889, 0.77730, 0.78482, 0.79109, 0.79617, 0.80018,\
        0.80316, 0.80443, 0.80406, 0.80288, 0.80105, 0.79940, 0.79814, 0.79797, 0.79897, 0.80014, 0.80092,\
        0.80169, 0.80218, 0.80171, 0.80087, 0.79994, 0.79883, 0.79827, 0.79797, 0.79734, 0.79591, 0.79417,\
        0.79226, 0.78999, 0.78723, 0.78388, 0.77977, 0.77486, 0.76927, 0.76324, 0.75699, 0.75070, 0.74451,\
        0.73851, 0.73275, 0.72722, 0.72188, 0.71668, 0.71161, 0.70662, 0.70170, 0.69733, 0.69344, 0.68958,\
        0.68533, 0.68034, 0.67461, 0.66830, 0.66158, 0.65459, 0.64741, 0.64011, 0.63274, 0.62535, 0.61800,\
        0.61077, 0.60378, 0.59739, 0.59172, 0.58641, 0.58150, 0.57698, 0.57283, 0.56899, 0.56545, 0.56216,\
        0.55909, 0.55620, 0.55344, 0.55079, 0.54819, 0.54560, 0.54295, 0.54018, 0.53720, 0.53392, 0.53024],\
        [0.66619, 0.67427, 0.68187, 0.68886, 0.69482, 0.69829, 0.69985, 0.70323, 0.70686, 0.71051, 0.71412,\
        0.71793, 0.72211, 0.72652, 0.73092, 0.73509, 0.73889, 0.74225, 0.74515, 0.74756, 0.74945, 0.75077,\
        0.75140, 0.75123, 0.75015, 0.74812, 0.74526, 0.74187, 0.73843, 0.73557, 0.73399, 0.73427, 0.73653,\
        0.74057, 0.74612, 0.75293, 0.76072, 0.76911, 0.77762, 0.78568, 0.79261, 0.79785, 0.80181, 0.80491,\
        0.80704, 0.80801, 0.80783, 0.80684, 0.80539, 0.80395, 0.80287, 0.80285, 0.80388, 0.80511, 0.80606,\
        0.80680, 0.80714, 0.80686, 0.80617, 0.80527, 0.80432, 0.80377, 0.80346, 0.80285, 0.80153, 0.79999,\
        0.79842, 0.79649, 0.79409, 0.79115, 0.78731, 0.78242, 0.77672, 0.77054, 0.76416, 0.75775, 0.75141,\
        0.74522, 0.73921, 0.73338, 0.72770, 0.72218, 0.71681, 0.71161, 0.70669, 0.70232, 0.69844, 0.69477,\
        0.69065, 0.68545, 0.67933, 0.67264, 0.66557, 0.65827, 0.65080, 0.64321, 0.63555, 0.62787, 0.62025,\
        0.61280, 0.60574, 0.59937, 0.59338, 0.58801, 0.58319, 0.57880, 0.57480, 0.57115, 0.56780, 0.56471,\
        0.56185, 0.55915, 0.55659, 0.55409, 0.55163, 0.54913, 0.54655, 0.54380, 0.54083, 0.53754, 0.53387],\
        [0.66975, 0.67773, 0.68524, 0.69218, 0.69833, 0.70239, 0.70462, 0.70795, 0.71188, 0.71596, 0.72008,\
        0.72431, 0.72870, 0.73317, 0.73758, 0.74178, 0.74567, 0.74920, 0.75234, 0.75508, 0.75741, 0.75926,\
        0.76052, 0.76103, 0.76063, 0.75923, 0.75687, 0.75384, 0.75065, 0.74791, 0.74625, 0.74611, 0.74771,\
        0.75100, 0.75586, 0.76208, 0.76939, 0.77740, 0.78557, 0.79322, 0.79968, 0.80415, 0.80734, 0.80977,\
        0.81142, 0.81223, 0.81221, 0.81154, 0.81052, 0.80949, 0.80881, 0.80885, 0.80957, 0.81054, 0.81138,\
        0.81195, 0.81214, 0.81187, 0.81122, 0.81036, 0.80948, 0.80875, 0.80810, 0.80723, 0.80600, 0.80460,\
        0.80316, 0.80149, 0.79950, 0.79713, 0.79368, 0.78877, 0.78297, 0.77674, 0.77036, 0.76394, 0.75757,\
        0.75129, 0.74511, 0.73905, 0.73311, 0.72730, 0.72167, 0.71629, 0.71132, 0.70691, 0.70304, 0.69962,\
        0.69555, 0.68991, 0.68337, 0.67637, 0.66908, 0.66160, 0.65396, 0.64620, 0.63836, 0.63051, 0.62271,\
        0.61510, 0.60783, 0.60105, 0.59506, 0.58981, 0.58506, 0.58076, 0.57687, 0.57337, 0.57018, 0.56728,\
        0.56459, 0.56207, 0.55966, 0.55731, 0.55495, 0.55254, 0.55000, 0.54729, 0.54432, 0.54103, 0.53736],\
        [0.67270, 0.68049, 0.68774, 0.69425, 0.70003, 0.70490, 0.70881, 0.71278, 0.71709, 0.72158, 0.72614,\
        0.73075, 0.73537, 0.73995, 0.74439, 0.74860, 0.75251, 0.75611, 0.75941, 0.76242, 0.76515, 0.76754,\
        0.76948, 0.77078, 0.77119, 0.77050, 0.76864, 0.76588, 0.76278, 0.76000, 0.75809, 0.75748, 0.75840,\
        0.76090, 0.76495, 0.77041, 0.77708, 0.78459, 0.79235, 0.79940, 0.80510, 0.80926, 0.81225, 0.81445,\
        0.81598, 0.81685, 0.81709, 0.81683, 0.81627, 0.81568, 0.81532, 0.81537, 0.81583, 0.81645, 0.81699,\
        0.81730, 0.81727, 0.81687, 0.81614, 0.81521, 0.81418, 0.81316, 0.81209, 0.81088, 0.80946, 0.80794,\
        0.80640, 0.80482, 0.80317, 0.80134, 0.79834, 0.79341, 0.78771, 0.78167, 0.77548, 0.76923, 0.76298,\
        0.75672, 0.75049, 0.74428, 0.73811, 0.73204, 0.72613, 0.72050, 0.71531, 0.71071, 0.70677, 0.70330,\
        0.69907, 0.69308, 0.68637, 0.67932, 0.67204, 0.66457, 0.65692, 0.64913, 0.64125, 0.63332, 0.62544,\
        0.61771, 0.61027, 0.60332, 0.59735, 0.59202, 0.58722, 0.58294, 0.57912, 0.57572, 0.57267, 0.56990,\
        0.56737, 0.56499, 0.56271, 0.56047, 0.55821, 0.55586, 0.55336, 0.55067, 0.54770, 0.54441, 0.54074],\
        [0.67518, 0.68281, 0.68995, 0.69628, 0.70193, 0.70751, 0.71254, 0.71737, 0.72226, 0.72723, 0.73222,\
        0.73718, 0.74204, 0.74675, 0.75122, 0.75540, 0.75927, 0.76284, 0.76618, 0.76935, 0.77238, 0.77527,\
        0.77789, 0.78004, 0.78137, 0.78149, 0.78010, 0.77746, 0.77431, 0.77139, 0.76918, 0.76807, 0.76829,\
        0.76996, 0.77310, 0.77768, 0.78355, 0.79037, 0.79759, 0.80408, 0.80927, 0.81336, 0.81648, 0.81883,\
        0.82055, 0.82169, 0.82232, 0.82252, 0.82244, 0.82226, 0.82214, 0.82221, 0.82244, 0.82272, 0.82290,\
        0.82287, 0.82255, 0.82191, 0.82099, 0.81984, 0.81854, 0.81713, 0.81561, 0.81393, 0.81211, 0.81021,\
        0.80833, 0.80653, 0.80482, 0.80297, 0.80015, 0.79589, 0.79078, 0.78528, 0.77956, 0.77369, 0.76771,\
        0.76161, 0.75542, 0.74913, 0.74279, 0.73645, 0.73020, 0.72417, 0.71852, 0.71343, 0.70893, 0.70481,\
        0.70025, 0.69455, 0.68818, 0.68147, 0.67448, 0.66724, 0.65975, 0.65207, 0.64425, 0.63636, 0.62850,\
        0.62077, 0.61334, 0.60645, 0.60028, 0.59464, 0.58971, 0.58540, 0.58160, 0.57826, 0.57530, 0.57265,\
        0.57023, 0.56797, 0.56580, 0.56364, 0.56144, 0.55914, 0.55667, 0.55398, 0.55102, 0.54772, 0.54404],\
        [0.67732, 0.68482, 0.69206, 0.69894, 0.70479, 0.71056, 0.71624, 0.72180, 0.72731, 0.73280, 0.73823,\
        0.74354, 0.74865, 0.75348, 0.75797, 0.76208, 0.76581, 0.76924, 0.77248, 0.77564, 0.77883, 0.78208,\
        0.78532, 0.78827, 0.79054, 0.79161, 0.79068, 0.78798, 0.78477, 0.78175, 0.77928, 0.77770, 0.77726,\
        0.77806, 0.78022, 0.78381, 0.78878, 0.79480, 0.80124, 0.80735, 0.81255, 0.81682, 0.82025, 0.82297,\
        0.82509, 0.82667, 0.82779, 0.82850, 0.82890, 0.82910, 0.82920, 0.82926, 0.82929, 0.82925, 0.82906,\
        0.82865, 0.82798, 0.82704, 0.82582, 0.82437, 0.82271, 0.82085, 0.81881, 0.81658, 0.81418, 0.81170,\
        0.80925, 0.80692, 0.80476, 0.80260, 0.80013, 0.79667, 0.79248, 0.78780, 0.78278, 0.77747, 0.77191,\
        0.76609, 0.76003, 0.75374, 0.74726, 0.74064, 0.73398, 0.72742, 0.72111, 0.71520, 0.70981, 0.70486,\
        0.70001, 0.69474, 0.68908, 0.68305, 0.67660, 0.66975, 0.66256, 0.65509, 0.64743, 0.63966, 0.63189,\
        0.62423, 0.61683, 0.60985, 0.60338, 0.59752, 0.59258, 0.58821, 0.58437, 0.58102, 0.57812, 0.57555,\
        0.57323, 0.57105, 0.56896, 0.56686, 0.56470, 0.56242, 0.55996, 0.55726, 0.55428, 0.55097, 0.54728],\
        [0.67926, 0.68655, 0.69365, 0.70061, 0.70725, 0.71363, 0.71993, 0.72613, 0.73223, 0.73824, 0.74411,\
        0.74977, 0.75514, 0.76010, 0.76459, 0.76856, 0.77206, 0.77520, 0.77816, 0.78112, 0.78426, 0.78766,\
        0.79133, 0.79493, 0.79787, 0.79990, 0.79945, 0.79671, 0.79379, 0.79094, 0.78829, 0.78634, 0.78532,\
        0.78528, 0.78634, 0.78887, 0.79301, 0.79837, 0.80433, 0.81018, 0.81546, 0.82001, 0.82384, 0.82702,\
        0.82965, 0.83178, 0.83345, 0.83471, 0.83560, 0.83615, 0.83644, 0.83649, 0.83634, 0.83599, 0.83542,\
        0.83461, 0.83356, 0.83226, 0.83070, 0.82890, 0.82683, 0.82451, 0.82192, 0.81908, 0.81601, 0.81280,\
        0.80959, 0.80654, 0.80379, 0.80131, 0.79903, 0.79654, 0.79337, 0.78964, 0.78544, 0.78081, 0.77578,\
        0.77033, 0.76448, 0.75825, 0.75167, 0.74479, 0.73771, 0.73053, 0.72341, 0.71653, 0.71009, 0.70421,\
        0.69901, 0.69441, 0.68967, 0.68449, 0.67870, 0.67232, 0.66547, 0.65828, 0.65084, 0.64326, 0.63564,\
        0.62809, 0.62072, 0.61367, 0.60706, 0.60114, 0.59599, 0.59141, 0.58743, 0.58405, 0.58118, 0.57867,\
        0.57640, 0.57428, 0.57222, 0.57015, 0.56800, 0.56572, 0.56325, 0.56053, 0.55752, 0.55418, 0.55046],\
        [0.68123, 0.68829, 0.69520, 0.70226, 0.70954, 0.71665, 0.72358, 0.73037, 0.73702, 0.74351, 0.74980,\
        0.75582, 0.76144, 0.76655, 0.77103, 0.77481, 0.77796, 0.78066, 0.78316, 0.78572, 0.78857, 0.79181,\
        0.79556, 0.79965, 0.80288, 0.80507, 0.80528, 0.80354, 0.80145, 0.79896, 0.79628, 0.79413, 0.79268,\
        0.79184, 0.79160, 0.79300, 0.79667, 0.80163, 0.80737, 0.81314, 0.81852, 0.82332, 0.82752, 0.83117,\
        0.83433, 0.83703, 0.83930, 0.84113, 0.84250, 0.84341, 0.84386, 0.84390, 0.84355, 0.84288, 0.84192,\
        0.84071, 0.83927, 0.83760, 0.83570, 0.83353, 0.83107, 0.82830, 0.82518, 0.82172, 0.81796, 0.81397,\
        0.80991, 0.80603, 0.80269, 0.80015, 0.79827, 0.79643, 0.79413, 0.79127, 0.78790, 0.78400, 0.77955,\
        0.77453, 0.76896, 0.76284, 0.75622, 0.74914, 0.74166, 0.73388, 0.72595, 0.71806, 0.71053, 0.70387,\
        0.69851, 0.69436, 0.69065, 0.68634, 0.68112, 0.67514, 0.66862, 0.66171, 0.65453, 0.64718, 0.63974,\
        0.63233, 0.62503, 0.61800, 0.61136, 0.60530, 0.59985, 0.59497, 0.59079, 0.58737, 0.58452, 0.58203,\
        0.57978, 0.57766, 0.57561, 0.57354, 0.57137, 0.56906, 0.56655, 0.56379, 0.56074, 0.55735, 0.55360],\
        [0.68348, 0.69044, 0.69736, 0.70467, 0.71227, 0.71985, 0.72728, 0.73454, 0.74165, 0.74856, 0.75524,\
        0.76160, 0.76750, 0.77278, 0.77726, 0.78081, 0.78350, 0.78559, 0.78746, 0.78947, 0.79188, 0.79475,\
        0.79809, 0.80214, 0.80593, 0.80854, 0.80959, 0.80916, 0.80788, 0.80594, 0.80365, 0.80146, 0.79967,\
        0.79811, 0.79614, 0.79613, 0.80010, 0.80531, 0.81100, 0.81673, 0.82213, 0.82707, 0.83153, 0.83556,\
        0.83921, 0.84248, 0.84535, 0.84776, 0.84962, 0.85088, 0.85149, 0.85147, 0.85090, 0.84988, 0.84852,\
        0.84690, 0.84508, 0.84306, 0.84084, 0.83836, 0.83556, 0.83240, 0.82882, 0.82481, 0.82041, 0.81570,\
        0.81082, 0.80607, 0.80199, 0.79942, 0.79822, 0.79697, 0.79526, 0.79312, 0.79050, 0.78730, 0.78344,\
        0.77887, 0.77360, 0.76766, 0.76108, 0.75389, 0.74613, 0.73788, 0.72925, 0.72046, 0.71184, 0.70403,\
        0.69827, 0.69521, 0.69274, 0.68904, 0.68410, 0.67835, 0.67208, 0.66544, 0.65853, 0.65143, 0.64421,\
        0.63695, 0.62976, 0.62275, 0.61605, 0.60979, 0.60403, 0.59881, 0.59442, 0.59104, 0.58822, 0.58569,\
        0.58338, 0.58123, 0.57914, 0.57703, 0.57481, 0.57244, 0.56987, 0.56705, 0.56394, 0.56050, 0.55669],\
        [0.68613, 0.69318, 0.70024, 0.70772, 0.71550, 0.72331, 0.73105, 0.73866, 0.74610, 0.75335, 0.76035,\
        0.76701, 0.77320, 0.77869, 0.78323, 0.78655, 0.78868, 0.79001, 0.79110, 0.79246, 0.79444, 0.79713,\
        0.80024, 0.80423, 0.80827, 0.81144, 0.81337, 0.81406, 0.81373, 0.81261, 0.81092, 0.80892, 0.80671,\
        0.80401, 0.79997, 0.79940, 0.80418, 0.81002, 0.81583, 0.82140, 0.82662, 0.83148, 0.83603, 0.84031,\
        0.84436, 0.84815, 0.85161, 0.85461, 0.85699, 0.85859, 0.85933, 0.85922, 0.85837, 0.85696, 0.85516,\
        0.85313, 0.85095, 0.84864, 0.84617, 0.84346, 0.84042, 0.83696, 0.83303, 0.82860, 0.82370, 0.81843,\
        0.81294, 0.80750, 0.80260, 0.79973, 0.79954, 0.79846, 0.79706, 0.79547, 0.79349, 0.79092, 0.78760,\
        0.78345, 0.77851, 0.77282, 0.76638, 0.75922, 0.75135, 0.74283, 0.73379, 0.72443, 0.71510, 0.70640,\
        0.70004, 0.69805, 0.69653, 0.69268, 0.68760, 0.68191, 0.67585, 0.66948, 0.66286, 0.65602, 0.64902,\
        0.64194, 0.63486, 0.62788, 0.62112, 0.61468, 0.60863, 0.60308, 0.59860, 0.59527, 0.59234, 0.58963,\
        0.58719, 0.58497, 0.58281, 0.58061, 0.57831, 0.57585, 0.57320, 0.57031, 0.56712, 0.56360, 0.55973],\
        [0.68915, 0.69640, 0.70371, 0.71126, 0.71906, 0.72697, 0.73486, 0.74266, 0.75033, 0.75780, 0.76503,\
        0.77193, 0.77836, 0.78412, 0.78883, 0.79198, 0.79345, 0.79392, 0.79416, 0.79478, 0.79630, 0.79908,\
        0.80251, 0.80652, 0.81075, 0.81444, 0.81719, 0.81887, 0.81957, 0.81942, 0.81856, 0.81706, 0.81488,\
        0.81184, 0.80855, 0.80843, 0.81221, 0.81738, 0.82258, 0.82753, 0.83221, 0.83671, 0.84111, 0.84546,\
        0.84979, 0.85404, 0.85806, 0.86167, 0.86459, 0.86655, 0.86740, 0.86713, 0.86592, 0.86404, 0.86177,\
        0.85933, 0.85683, 0.85431, 0.85170, 0.84888, 0.84572, 0.84210, 0.83795, 0.83324, 0.82803, 0.82242,\
        0.81663, 0.81101, 0.80618, 0.80325, 0.80214, 0.80102, 0.79972, 0.79851, 0.79702, 0.79497, 0.79210,\
        0.78831, 0.78370, 0.77832, 0.77215, 0.76519, 0.75742, 0.74890, 0.73977, 0.73030, 0.72094, 0.71248,\
        0.70629, 0.70334, 0.70106, 0.69658, 0.69137, 0.68575, 0.67992, 0.67385, 0.66752, 0.66095, 0.65418,\
        0.64727, 0.64030, 0.63337, 0.62657, 0.62004, 0.61388, 0.60832, 0.60373, 0.60009, 0.59685, 0.59374,\
        0.59114, 0.58885, 0.58660, 0.58427, 0.58183, 0.57927, 0.57652, 0.57354, 0.57026, 0.56666, 0.56270],\
        [0.69236, 0.69981, 0.70730, 0.71493, 0.72275, 0.73068, 0.73862, 0.74651, 0.75427, 0.76185, 0.76919,\
        0.77620, 0.78278, 0.78875, 0.79376, 0.79696, 0.79763, 0.79732, 0.79683, 0.79676, 0.79767, 0.80029,\
        0.80438, 0.80900, 0.81366, 0.81790, 0.82139, 0.82400, 0.82574, 0.82668, 0.82689, 0.82641, 0.82519,\
        0.82338, 0.82178, 0.82194, 0.82425, 0.82773, 0.83150, 0.83523, 0.83894, 0.84274, 0.84674, 0.85099,\
        0.85548, 0.86010, 0.86466, 0.86889, 0.87240, 0.87477, 0.87568, 0.87515, 0.87348, 0.87105, 0.86825,\
        0.86541, 0.86267, 0.86005, 0.85743, 0.85465, 0.85152, 0.84789, 0.84365, 0.83880, 0.83342, 0.82766,\
        0.82182, 0.81628, 0.81161, 0.80834, 0.80633, 0.80485, 0.80354, 0.80237, 0.80108, 0.79937, 0.79685,\
        0.79333, 0.78907, 0.78410, 0.77835, 0.77176, 0.76429, 0.75599, 0.74705, 0.73779, 0.72871, 0.72055,\
        0.71412, 0.70953, 0.70542, 0.70078, 0.69527, 0.68984, 0.68432, 0.67855, 0.67249, 0.66618, 0.65964,\
        0.65290, 0.64604, 0.63915, 0.63235, 0.62577, 0.61958, 0.61398, 0.60915, 0.60506, 0.60128, 0.59789,\
        0.59521, 0.59288, 0.59048, 0.58794, 0.58532, 0.58263, 0.57979, 0.57672, 0.57335, 0.56964, 0.56559],\
        [0.69565, 0.70323, 0.71085, 0.71856, 0.72640, 0.73431, 0.74225, 0.75013, 0.75789, 0.76546, 0.77278,\
        0.77975, 0.78626, 0.79218, 0.79722, 0.80062, 0.80089, 0.80035, 0.79948, 0.79920, 0.79934, 0.80229,\
        0.80698, 0.81214, 0.81728, 0.82204, 0.82619, 0.82964, 0.83238, 0.83446, 0.83597, 0.83691, 0.83730,\
        0.83724, 0.83711, 0.83749, 0.83862, 0.84027, 0.84215, 0.84423, 0.84662, 0.84946, 0.85286, 0.85685,\
        0.86137, 0.86628, 0.87132, 0.87615, 0.88033, 0.88318, 0.88410, 0.88319, 0.88094, 0.87785, 0.87449,\
        0.87128, 0.86840, 0.86582, 0.86337, 0.86080, 0.85786, 0.85433, 0.85012, 0.84523, 0.83976, 0.83394,\
        0.82809, 0.82261, 0.81790, 0.81427, 0.81165, 0.80970, 0.80815, 0.80682, 0.80547, 0.80382, 0.80149,\
        0.79828, 0.79447, 0.79000, 0.78480, 0.77876, 0.77176, 0.76385, 0.75524, 0.74629, 0.73747, 0.72933,\
        0.72231, 0.71639, 0.71091, 0.70525, 0.69958, 0.69447, 0.68918, 0.68360, 0.67776, 0.67168, 0.66534,\
        0.65876, 0.65199, 0.64513, 0.63832, 0.63170, 0.62544, 0.61970, 0.61457, 0.61001, 0.60589, 0.60230,\
        0.59944, 0.59706, 0.59433, 0.59148, 0.58869, 0.58591, 0.58299, 0.57983, 0.57636, 0.57254, 0.56836],\
        [0.69891, 0.70660, 0.71430, 0.72206, 0.72990, 0.73778, 0.74567, 0.75349, 0.76118, 0.76866, 0.77584,\
        0.78263, 0.78890, 0.79446, 0.79902, 0.80215, 0.80330, 0.80323, 0.80280, 0.80276, 0.80345, 0.80634,\
        0.81107, 0.81642, 0.82183, 0.82696, 0.83164, 0.83579, 0.83942, 0.84261, 0.84546, 0.84802, 0.85028,\
        0.85211, 0.85335, 0.85396, 0.85408, 0.85394, 0.85382, 0.85405, 0.85493, 0.85667, 0.85936, 0.86299,\
        0.86745, 0.87255, 0.87799, 0.88330, 0.88809, 0.89162, 0.89240, 0.89102, 0.88815, 0.88432, 0.88038,\
        0.87688, 0.87399, 0.87164, 0.86953, 0.86734, 0.86472, 0.86142, 0.85730, 0.85238, 0.84684, 0.84093,\
        0.83503, 0.82948, 0.82462, 0.82064, 0.81753, 0.81512, 0.81320, 0.81155, 0.80997, 0.80818, 0.80591,\
        0.80308, 0.79973, 0.79582, 0.79128, 0.78595, 0.77953, 0.77207, 0.76385, 0.75520, 0.74652, 0.73822,\
        0.73061, 0.72370, 0.71726, 0.71104, 0.70521, 0.69981, 0.69449, 0.68890, 0.68322, 0.67735, 0.67121,\
        0.66476, 0.65805, 0.65119, 0.64433, 0.63763, 0.63126, 0.62533, 0.61992, 0.61503, 0.61066, 0.60688,\
        0.60376, 0.60098, 0.59784, 0.59471, 0.59183, 0.58905, 0.58611, 0.58286, 0.57925, 0.57530, 0.57100],\
        [0.70208, 0.70985, 0.71760, 0.72539, 0.73321, 0.74104, 0.74886, 0.75658, 0.76416, 0.77149, 0.77848,\
        0.78503, 0.79100, 0.79623, 0.80054, 0.80370, 0.80552, 0.80633, 0.80677, 0.80748, 0.80906, 0.81214,\
        0.81663, 0.82187, 0.82733, 0.83266, 0.83768, 0.84234, 0.84668, 0.85080, 0.85486, 0.85896, 0.86308,\
        0.86688, 0.86958, 0.87046, 0.86964, 0.86779, 0.86569, 0.86408, 0.86349, 0.86417, 0.86619, 0.86947,\
        0.87382, 0.87900, 0.88474, 0.89030, 0.89506, 0.89955, 0.89993, 0.89842, 0.89497, 0.89035, 0.88590,\
        0.88224, 0.87949, 0.87753, 0.87595, 0.87429, 0.87211, 0.86909, 0.86507, 0.86009, 0.85439, 0.84832,\
        0.84226, 0.83655, 0.83146, 0.82715, 0.82362, 0.82077, 0.81842, 0.81639, 0.81448, 0.81248, 0.81022,\
        0.80762, 0.80466, 0.80127, 0.79748, 0.79301, 0.78717, 0.78017, 0.77237, 0.76401, 0.75539, 0.74687,\
        0.73875, 0.73114, 0.72402, 0.71735, 0.71116, 0.70544, 0.69985, 0.69417, 0.68874, 0.68308, 0.67712,\
        0.67078, 0.66408, 0.65717, 0.65022, 0.64340, 0.63688, 0.63075, 0.62510, 0.61994, 0.61531, 0.61122,\
        0.60763, 0.60426, 0.60081, 0.59755, 0.59479, 0.59211, 0.58914, 0.58576, 0.58199, 0.57788, 0.57344],\
        [0.70513, 0.71294, 0.72073, 0.72851, 0.73630, 0.74407, 0.75180, 0.75942, 0.76687, 0.77405, 0.78086,\
        0.78718, 0.79291, 0.79796, 0.80225, 0.80565, 0.80809, 0.80979, 0.81121, 0.81285, 0.81522, 0.81863,\
        0.82308, 0.82821, 0.83362, 0.83901, 0.84420, 0.84915, 0.85393, 0.85868, 0.86360, 0.86887, 0.87455,\
        0.88033, 0.88476, 0.88589, 0.88426, 0.88087, 0.87702, 0.87385, 0.87205, 0.87193, 0.87347, 0.87653,\
        0.88080, 0.88586, 0.89171, 0.89784, 0.90187, 0.90556, 0.90665, 0.90506, 0.90127, 0.89597, 0.89132,\
        0.88760, 0.88507, 0.88364, 0.88271, 0.88164, 0.87995, 0.87724, 0.87327, 0.86812, 0.86213, 0.85577,\
        0.84945, 0.84352, 0.83818, 0.83357, 0.82967, 0.82640, 0.82363, 0.82119, 0.81891, 0.81665, 0.81428,\
        0.81175, 0.80904, 0.80614, 0.80303, 0.79944, 0.79398, 0.78755, 0.78036, 0.77232, 0.76370, 0.75494,\
        0.74640, 0.73828, 0.73065, 0.72354, 0.71695, 0.71081, 0.70490, 0.69947, 0.69420, 0.68868, 0.68293,\
        0.67667, 0.66992, 0.66289, 0.65580, 0.64886, 0.64218, 0.63587, 0.62999, 0.62458, 0.61966, 0.61521,\
        0.61116, 0.60735, 0.60368, 0.60041, 0.59774, 0.59515, 0.59206, 0.58847, 0.58449, 0.58021, 0.57564],\
        [0.70803, 0.71586, 0.72366, 0.73142, 0.73916, 0.74687, 0.75450, 0.76202, 0.76935, 0.77641, 0.78310,\
        0.78928, 0.79487, 0.79987, 0.80430, 0.80804, 0.81108, 0.81361, 0.81595, 0.81849, 0.82157, 0.82540,\
        0.82998, 0.83509, 0.84046, 0.84582, 0.85103, 0.85607, 0.86100, 0.86600, 0.87130, 0.87710, 0.88356,\
        0.89064, 0.89727, 0.89836, 0.89667, 0.89223, 0.88727, 0.88312, 0.88059, 0.88016, 0.88153, 0.88463,\
        0.88907, 0.89402, 0.89901, 0.90553, 0.91011, 0.91301, 0.91375, 0.91192, 0.90779, 0.90242, 0.89761,\
        0.89353, 0.89102, 0.89018, 0.88993, 0.88934, 0.88813, 0.88570, 0.88165, 0.87614, 0.86970, 0.86295,\
        0.85633, 0.85015, 0.84459, 0.83972, 0.83552, 0.83189, 0.82872, 0.82586, 0.82318, 0.82058, 0.81798,\
        0.81534, 0.81268, 0.80999, 0.80718, 0.80379, 0.79911, 0.79381, 0.78751, 0.77975, 0.77108, 0.76210,\
        0.75324, 0.74474, 0.73673, 0.72925, 0.72232, 0.71589, 0.70994, 0.70449, 0.69911, 0.69387, 0.68851,\
        0.68226, 0.67533, 0.66813, 0.66091, 0.65385, 0.64705, 0.64058, 0.63451, 0.62886, 0.62364, 0.61886,\
        0.61446, 0.61039, 0.60667, 0.60343, 0.60075, 0.59810, 0.59470, 0.59082, 0.58664, 0.58220, 0.57753],\
        [0.71078, 0.71862, 0.72639, 0.73412, 0.74181, 0.74943, 0.75698, 0.76440, 0.77164, 0.77863, 0.78527,\
        0.79145, 0.79703, 0.80208, 0.80670, 0.81080, 0.81440, 0.81767, 0.82084, 0.82417, 0.82792, 0.83221,\
        0.83703, 0.84224, 0.84761, 0.85294, 0.85807, 0.86300, 0.86781, 0.87272, 0.87794, 0.88364, 0.88991,\
        0.89661, 0.90312, 0.90616, 0.90528, 0.90126, 0.89660, 0.89213, 0.88940, 0.88956, 0.89098, 0.89441,\
        0.89946, 0.90470, 0.90988, 0.91533, 0.91973, 0.92218, 0.92241, 0.92032, 0.91622, 0.91100, 0.90564,\
        0.90096, 0.89785, 0.89748, 0.89761, 0.89718, 0.89634, 0.89410, 0.88979, 0.88368, 0.87667, 0.86949,\
        0.86260, 0.85625, 0.85055, 0.84552, 0.84109, 0.83717, 0.83363, 0.83036, 0.82725, 0.82422, 0.82123,\
        0.81829, 0.81542, 0.81264, 0.80983, 0.80671, 0.80305, 0.79909, 0.79360, 0.78587, 0.77707, 0.76797,\
        0.75895, 0.75025, 0.74199, 0.73424, 0.72704, 0.72040, 0.71431, 0.70869, 0.70337, 0.69871, 0.69376,\
        0.68718, 0.67998, 0.67264, 0.66536, 0.65825, 0.65139, 0.64483, 0.63860, 0.63273, 0.62725, 0.62216,\
        0.61747, 0.61318, 0.60935, 0.60604, 0.60320, 0.60029, 0.59666, 0.59260, 0.58829, 0.58378, 0.57907],\
        [0.71338, 0.72119, 0.72894, 0.73662, 0.74423, 0.75178, 0.75924, 0.76657, 0.77374, 0.78069, 0.78736,\
        0.79366, 0.79942, 0.80458, 0.80937, 0.81380, 0.81792, 0.82184, 0.82573, 0.82977, 0.83413, 0.83889,\
        0.84403, 0.84943, 0.85490, 0.86022, 0.86524, 0.86994, 0.87444, 0.87900, 0.88388, 0.88922, 0.89495,\
        0.90124, 0.90742, 0.91128, 0.91177, 0.90950, 0.90591, 0.90186, 0.89934, 0.90118, 0.90302, 0.90688,\
        0.91239, 0.91791, 0.92311, 0.92780, 0.93137, 0.93316, 0.93288, 0.93055, 0.92649, 0.92133, 0.91585,\
        0.91085, 0.90700, 0.90586, 0.90570, 0.90505, 0.90395, 0.90159, 0.89690, 0.89013, 0.88257, 0.87510,\
        0.86809, 0.86171, 0.85601, 0.85092, 0.84637, 0.84224, 0.83839, 0.83472, 0.83113, 0.82759, 0.82407,\
        0.82062, 0.81729, 0.81415, 0.81120, 0.80837, 0.80560, 0.80261, 0.79756, 0.78988, 0.78124, 0.77228,\
        0.76334, 0.75462, 0.74627, 0.73836, 0.73096, 0.72411, 0.71784, 0.71215, 0.70711, 0.70282, 0.69758,\
        0.69068, 0.68345, 0.67619, 0.66902, 0.66201, 0.65518, 0.64858, 0.64224, 0.63619, 0.63046, 0.62509,\
        0.62010, 0.61554, 0.61145, 0.60785, 0.60463, 0.60139, 0.59775, 0.59370, 0.58939, 0.58491, 0.58024],\
        [0.71583, 0.72361, 0.73130, 0.73892, 0.74646, 0.75392, 0.76129, 0.76854, 0.77564, 0.78255, 0.78923,\
        0.79564, 0.80163, 0.80701, 0.81206, 0.81687, 0.82148, 0.82598, 0.83049, 0.83515, 0.84006, 0.84529,\
        0.85080, 0.85649, 0.86216, 0.86756, 0.87250, 0.87692, 0.88099, 0.88508, 0.88959, 0.89476, 0.90040,\
        0.90653, 0.91249, 0.91671, 0.91846, 0.91795, 0.91605, 0.91380, 0.91305, 0.91502, 0.91815, 0.92245,\
        0.92777, 0.93315, 0.93800, 0.94198, 0.94467, 0.94570, 0.94489, 0.94228, 0.93819, 0.93311, 0.92768,\
        0.92258, 0.91851, 0.91604, 0.91447, 0.91280, 0.91056, 0.90715, 0.90199, 0.89483, 0.88707, 0.87962,\
        0.87274, 0.86654, 0.86099, 0.85600, 0.85144, 0.84716, 0.84305, 0.83899, 0.83491, 0.83077, 0.82660,\
        0.82245, 0.81842, 0.81465, 0.81127, 0.80836, 0.80576, 0.80278, 0.79809, 0.79134, 0.78342, 0.77498,\
        0.76638, 0.75785, 0.74954, 0.74156, 0.73400, 0.72693, 0.72040, 0.71448, 0.70918, 0.70420, 0.69867,\
        0.69234, 0.68560, 0.67875, 0.67192, 0.66513, 0.65844, 0.65186, 0.64545, 0.63925, 0.63330, 0.62764,\
        0.62233, 0.61741, 0.61293, 0.60891, 0.60527, 0.60181, 0.59816, 0.59415, 0.58997, 0.58563, 0.58109],\
        [0.71813, 0.72587, 0.73350, 0.74105, 0.74850, 0.75588, 0.76316, 0.77032, 0.77735, 0.78421, 0.79086,\
        0.79728, 0.80341, 0.80918, 0.81463, 0.81986, 0.82495, 0.82998, 0.83503, 0.84022, 0.84562, 0.85127,\
        0.85717, 0.86321, 0.86919, 0.87482, 0.87979, 0.88397, 0.88759, 0.89122, 0.89539, 0.90064, 0.90661,\
        0.91278, 0.91858, 0.92317, 0.92600, 0.92717, 0.92728, 0.92726, 0.92827, 0.93090, 0.93474, 0.93943,\
        0.94458, 0.94956, 0.95385, 0.95706, 0.95890, 0.95916, 0.95779, 0.95489, 0.95072, 0.94570, 0.94031,\
        0.93511, 0.93056, 0.92687, 0.92370, 0.92045, 0.91659, 0.91165, 0.90527, 0.89779, 0.89042, 0.88326,\
        0.87670, 0.87086, 0.86563, 0.86087, 0.85640, 0.85207, 0.84774, 0.84331, 0.83871, 0.83393, 0.82899,\
        0.82400, 0.81908, 0.81441, 0.81024, 0.80679, 0.80399, 0.80106, 0.79674, 0.79094, 0.78399, 0.77632,\
        0.76826, 0.76007, 0.75191, 0.74392, 0.73623, 0.72891, 0.72203, 0.71567, 0.70978, 0.70423, 0.69866,\
        0.69280, 0.68674, 0.68052, 0.67418, 0.66775, 0.66126, 0.65475, 0.64830, 0.64196, 0.63579, 0.62984,\
        0.62418, 0.61885, 0.61391, 0.60939, 0.60529, 0.60150, 0.59782, 0.59406, 0.59014, 0.58604, 0.58170],\
        [0.72030, 0.72798, 0.73555, 0.74301, 0.75038, 0.75766, 0.76485, 0.77194, 0.77891, 0.78573, 0.79237,\
        0.79881, 0.80512, 0.81121, 0.81706, 0.82272, 0.82827, 0.83376, 0.83928, 0.84489, 0.85068, 0.85669,\
        0.86293, 0.86933, 0.87571, 0.88176, 0.88698, 0.89105, 0.89428, 0.89762, 0.90183, 0.90717, 0.91331,\
        0.91958, 0.92547, 0.93047, 0.93427, 0.93693, 0.93888, 0.94081, 0.94340, 0.94701, 0.95148, 0.95647,\
        0.96151, 0.96608, 0.96973, 0.97214, 0.97313, 0.97265, 0.97074, 0.96755, 0.96331, 0.95833, 0.95296,\
        0.94758, 0.94247, 0.93769, 0.93305, 0.92816, 0.92262, 0.91615, 0.90875, 0.90101, 0.89351, 0.88638,\
        0.88026, 0.87496, 0.87019, 0.86574, 0.86143, 0.85710, 0.85260, 0.84782, 0.84269, 0.83723, 0.83150,\
        0.82558, 0.81963, 0.81387, 0.80859, 0.80423, 0.80112, 0.79823, 0.79453, 0.78962, 0.78361, 0.77675,\
        0.76931, 0.76152, 0.75356, 0.74562, 0.73780, 0.73023, 0.72297, 0.71610, 0.70962, 0.70355, 0.69796,\
        0.69270, 0.68739, 0.68187, 0.67609, 0.67004, 0.66377, 0.65735, 0.65086, 0.64439, 0.63800, 0.63177,\
        0.62576, 0.62001, 0.61458, 0.60951, 0.60491, 0.60097, 0.59730, 0.59374, 0.59013, 0.58631, 0.58218],\
        [0.72235, 0.72997, 0.73747, 0.74484, 0.75211, 0.75929, 0.76638, 0.77341, 0.78035, 0.78718, 0.79384,\
        0.80036, 0.80684, 0.81319, 0.81938, 0.82544, 0.83139, 0.83729, 0.84318, 0.84912, 0.85518, 0.86143,\
        0.86789, 0.87455, 0.88131, 0.88791, 0.89368, 0.89780, 0.90093, 0.90429, 0.90877, 0.91422, 0.92036,\
        0.92668, 0.93272, 0.93813, 0.94273, 0.94658, 0.95000, 0.95345, 0.95736, 0.96194, 0.96705, 0.97234,\
        0.97733, 0.98152, 0.98452, 0.98612, 0.98629, 0.98511, 0.98273, 0.97933, 0.97510, 0.97023, 0.96494,\
        0.95944, 0.95385, 0.94820, 0.94234, 0.93603, 0.92902, 0.92120, 0.91276, 0.90430, 0.89608, 0.88923,\
        0.88388, 0.87923, 0.87494, 0.87083, 0.86671, 0.86242, 0.85777, 0.85265, 0.84701, 0.84088, 0.83434,\
        0.82752, 0.82057, 0.81370, 0.80724, 0.80171, 0.79814, 0.79549, 0.79232, 0.78817, 0.78297, 0.77683,\
        0.76994, 0.76251, 0.75476, 0.74686, 0.73896, 0.73117, 0.72358, 0.71626, 0.70931, 0.70291, 0.69741,\
        0.69268, 0.68808, 0.68321, 0.67792, 0.67220, 0.66611, 0.65975, 0.65321, 0.64660, 0.64001, 0.63352,\
        0.62718, 0.62106, 0.61520, 0.60964, 0.60449, 0.59996, 0.59664, 0.59348, 0.59020, 0.58663, 0.58268],\
        [0.72430, 0.73186, 0.73928, 0.74656, 0.75372, 0.76079, 0.76778, 0.77472, 0.78164, 0.78853, 0.79534,\
        0.80198, 0.80857, 0.81511, 0.82160, 0.82801, 0.83433, 0.84057, 0.84674, 0.85290, 0.85910, 0.86541,\
        0.87189, 0.87859, 0.88547, 0.89241, 0.89891, 0.90321, 0.90669, 0.91060, 0.91542, 0.92110, 0.92733,\
        0.93372, 0.93989, 0.94560, 0.95072, 0.95536, 0.95976, 0.96425, 0.96912, 0.97448, 0.98016, 0.98577,\
        0.99076, 0.99461, 0.99697, 0.99775, 0.99712, 0.99533, 0.99264, 0.98923, 0.98522, 0.98067, 0.97565,\
        0.97022, 0.96441, 0.95819, 0.95148, 0.94413, 0.93600, 0.92706, 0.91748, 0.90761, 0.89865, 0.89276,\
        0.88835, 0.88416, 0.88019, 0.87633, 0.87238, 0.86813, 0.86337, 0.85793, 0.85178, 0.84500, 0.83771,\
        0.83008, 0.82229, 0.81456, 0.80721, 0.80090, 0.79680, 0.79395, 0.79100, 0.78732, 0.78266, 0.77701,\
        0.77052, 0.76337, 0.75577, 0.74790, 0.73993, 0.73199, 0.72419, 0.71661, 0.70938, 0.70276, 0.69742,\
        0.69328, 0.68926, 0.68487, 0.67992, 0.67441, 0.66841, 0.66203, 0.65540, 0.64864, 0.64186, 0.63514,\
        0.62855, 0.62216, 0.61600, 0.61016, 0.60477, 0.60022, 0.59669, 0.59369, 0.59061, 0.58718, 0.58331],\
        [0.72616, 0.73366, 0.74100, 0.74819, 0.75525, 0.76219, 0.76905, 0.77587, 0.78272, 0.78965, 0.79667,\
        0.80347, 0.81013, 0.81689, 0.82368, 0.83044, 0.83711, 0.84364, 0.85003, 0.85630, 0.86249, 0.86868,\
        0.87495, 0.88139, 0.88798, 0.89458, 0.90080, 0.90613, 0.91079, 0.91569, 0.92123, 0.92736, 0.93382,\
        0.94031, 0.94653, 0.95231, 0.95761, 0.96254, 0.96735, 0.97230, 0.97765, 0.98345, 0.98952, 0.99541,\
        1.00044, 1.00397, 1.00566, 1.00561, 1.00422, 1.00197, 0.99923, 0.99617, 0.99278, 0.98894, 0.98455,\
        0.97953, 0.97385, 0.96747, 0.96035, 0.95242, 0.94363, 0.93405, 0.92390, 0.91367, 0.90463, 0.89889,\
        0.89446, 0.88997, 0.88609, 0.88233, 0.87848, 0.87431, 0.86945, 0.86368, 0.85701, 0.84961, 0.84166,\
        0.83339, 0.82500, 0.81674, 0.80899, 0.80237, 0.79763, 0.79416, 0.79106, 0.78751, 0.78308, 0.77766,\
        0.77135, 0.76432, 0.75678, 0.74891, 0.74089, 0.73288, 0.72502, 0.71742, 0.71025, 0.70378, 0.69866,\
        0.69492, 0.69120, 0.68702, 0.68222, 0.67676, 0.67071, 0.66423, 0.65746, 0.65054, 0.64358, 0.63667,\
        0.62991, 0.62336, 0.61708, 0.61116, 0.60574, 0.60103, 0.59759, 0.59458, 0.59150, 0.58805, 0.58411],\
        [0.72796, 0.73541, 0.74268, 0.74979, 0.75673, 0.76353, 0.77023, 0.77688, 0.78357, 0.79038, 0.79733,\
        0.80432, 0.81133, 0.81846, 0.82564, 0.83278, 0.83979, 0.84659, 0.85315, 0.85944, 0.86550, 0.87140,\
        0.87725, 0.88317, 0.88926, 0.89541, 0.90149, 0.90752, 0.91342, 0.91955, 0.92606, 0.93283, 0.93962,\
        0.94617, 0.95226, 0.95780, 0.96282, 0.96747, 0.97200, 0.97671, 0.98186, 0.98758, 0.99371, 0.99974,\
        1.00482, 1.00802, 1.00897, 1.00800, 1.00591, 1.00348, 1.00117, 0.99906, 0.99692, 0.99439, 0.99115,\
        0.98701, 0.98190, 0.97583, 0.96878, 0.96077, 0.95185, 0.94217, 0.93208, 0.92219, 0.91357, 0.90707,\
        0.90144, 0.89648, 0.89274, 0.88874, 0.88494, 0.88093, 0.87599, 0.86984, 0.86261, 0.85460, 0.84609,\
        0.83734, 0.82859, 0.82010, 0.81223, 0.80543, 0.79998, 0.79588, 0.79258, 0.78894, 0.78442, 0.77895,\
        0.77259, 0.76552, 0.75792, 0.74999, 0.74192, 0.73390, 0.72610, 0.71868, 0.71187, 0.70594, 0.70127,\
        0.69760, 0.69378, 0.68961, 0.68478, 0.67919, 0.67298, 0.66631, 0.65935, 0.65225, 0.64513, 0.63810,\
        0.63123, 0.62463, 0.61836, 0.61254, 0.60730, 0.60285, 0.59921, 0.59605, 0.59287, 0.58921, 0.58506],\
        [0.72969, 0.73711, 0.74433, 0.75137, 0.75822, 0.76490, 0.77143, 0.77787, 0.78430, 0.79087, 0.79771,\
        0.80487, 0.81230, 0.81989, 0.82753, 0.83509, 0.84245, 0.84952, 0.85621, 0.86249, 0.86835, 0.87383,\
        0.87907, 0.88430, 0.88975, 0.89568, 0.90187, 0.90817, 0.91511, 0.92252, 0.93013, 0.93762, 0.94471,\
        0.95118, 0.95688, 0.96177, 0.96594, 0.96960, 0.97304, 0.97663, 0.98072, 0.98555, 0.99111, 0.99698,\
        1.00206, 1.00487, 1.00486, 1.00286, 1.00029, 0.99820, 0.99709, 0.99685, 0.99689, 0.99647, 0.99503,\
        0.99231, 0.98828, 0.98300, 0.97653, 0.96893, 0.96035, 0.95101, 0.94130, 0.93181, 0.92322, 0.91586,\
        0.90941, 0.90419, 0.89985, 0.89510, 0.89164, 0.88793, 0.88288, 0.87619, 0.86830, 0.85970, 0.85074,\
        0.84169, 0.83277, 0.82422, 0.81634, 0.80938, 0.80351, 0.79904, 0.79556, 0.79153, 0.78665, 0.78088,\
        0.77427, 0.76699, 0.75922, 0.75117, 0.74303, 0.73501, 0.72730, 0.72012, 0.71371, 0.70830, 0.70397,\
        0.70022, 0.69649, 0.69241, 0.68742, 0.68157, 0.67507, 0.66815, 0.66098, 0.65372, 0.64647, 0.63934,\
        0.63243, 0.62584, 0.61966, 0.61402, 0.60902, 0.60472, 0.60106, 0.59797, 0.59452, 0.59048, 0.58598],\
        [0.73138, 0.73878, 0.74598, 0.75298, 0.75978, 0.76636, 0.77276, 0.77899, 0.78514, 0.79140, 0.79812,\
        0.80544, 0.81328, 0.82137, 0.82948, 0.83745, 0.84517, 0.85252, 0.85937, 0.86565, 0.87128, 0.87630,\
        0.88084, 0.88518, 0.88970, 0.89488, 0.90093, 0.90802, 0.91633, 0.92516, 0.93387, 0.94201, 0.94926,\
        0.95540, 0.96031, 0.96403, 0.96670, 0.96857, 0.97000, 0.97140, 0.97329, 0.97606, 0.97990, 0.98486,\
        0.98982, 0.99204, 0.99061, 0.98757, 0.98509, 0.98436, 0.98565, 0.98859, 0.99210, 0.99479, 0.99586,\
        0.99513, 0.99272, 0.98875, 0.98335, 0.97663, 0.96878, 0.96006, 0.95086, 0.94169, 0.93303, 0.92517,\
        0.91818, 0.91214, 0.90661, 0.90177, 0.89861, 0.89508, 0.88972, 0.88220, 0.87360, 0.86452, 0.85528,\
        0.84610, 0.83717, 0.82871, 0.82093, 0.81403, 0.80820, 0.80356, 0.79956, 0.79488, 0.78951, 0.78329,\
        0.77629, 0.76868, 0.76066, 0.75242, 0.74415, 0.73607, 0.72840, 0.72139, 0.71524, 0.71013, 0.70600,\
        0.70247, 0.69916, 0.69516, 0.68985, 0.68359, 0.67676, 0.66959, 0.66225, 0.65485, 0.64751, 0.64033,\
        0.63341, 0.62685, 0.62077, 0.61528, 0.61049, 0.60643, 0.60305, 0.59991, 0.59602, 0.59150, 0.58663],\
        [0.73303, 0.74043, 0.74763, 0.75463, 0.76142, 0.76799, 0.77434, 0.78045, 0.78640, 0.79232, 0.79863,\
        0.80628, 0.81460, 0.82311, 0.83160, 0.83994, 0.84801, 0.85565, 0.86273, 0.86907, 0.87457, 0.87919,\
        0.88304, 0.88644, 0.88987, 0.89392, 0.89952, 0.90794, 0.91797, 0.92820, 0.93785, 0.94643, 0.95355,\
        0.95899, 0.96265, 0.96459, 0.96501, 0.96424, 0.96263, 0.96063, 0.95895, 0.95817, 0.95834, 0.96035,\
        0.96481, 0.96583, 0.96202, 0.95867, 0.95796, 0.96047, 0.96576, 0.97364, 0.98232, 0.98917, 0.99339,\
        0.99520, 0.99493, 0.99282, 0.98900, 0.98360, 0.97679, 0.96888, 0.96027, 0.95138, 0.94267, 0.93444,\
        0.92689, 0.92009, 0.91404, 0.90904, 0.90517, 0.90146, 0.89539, 0.88695, 0.87793, 0.86867, 0.85938,\
        0.85025, 0.84146, 0.83318, 0.82559, 0.81883, 0.81301, 0.80806, 0.80343, 0.79841, 0.79267, 0.78593,\
        0.77845, 0.77047, 0.76215, 0.75366, 0.74520, 0.73698, 0.72924, 0.72222, 0.71614, 0.71115, 0.70722,\
        0.70412, 0.70132, 0.69726, 0.69149, 0.68487, 0.67780, 0.67049, 0.66304, 0.65558, 0.64820, 0.64099,\
        0.63406, 0.62752, 0.62149, 0.61608, 0.61138, 0.60742, 0.60403, 0.60073, 0.59659, 0.59183, 0.58677],\
        [0.73462, 0.74204, 0.74928, 0.75632, 0.76316, 0.76980, 0.77621, 0.78239, 0.78836, 0.79426, 0.80063,\
        0.80820, 0.81656, 0.82521, 0.83392, 0.84254, 0.85093, 0.85891, 0.86629, 0.87286, 0.87840, 0.88280,\
        0.88614, 0.88880, 0.89145, 0.89503, 0.90048, 0.91006, 0.92128, 0.93243, 0.94264, 0.95129, 0.95792,\
        0.96223, 0.96410, 0.96359, 0.96097, 0.95663, 0.95098, 0.94433, 0.93750, 0.93192, 0.92613, 0.91981,\
        0.92201, 0.91875, 0.91156, 0.91233, 0.91686, 0.92638, 0.93677, 0.95198, 0.96783, 0.97947, 0.98730,\
        0.99219, 0.99471, 0.99509, 0.99338, 0.98968, 0.98419, 0.97723, 0.96922, 0.96060, 0.95182, 0.94324,\
        0.93513, 0.92766, 0.92098, 0.91519, 0.91008, 0.90478, 0.89790, 0.88975, 0.88096, 0.87189, 0.86280,\
        0.85389, 0.84535, 0.83731, 0.82993, 0.82329, 0.81741, 0.81213, 0.80711, 0.80191, 0.79576, 0.78843,\
        0.78053, 0.77222, 0.76361, 0.75485, 0.74614, 0.73767, 0.72971, 0.72249, 0.71625, 0.71115, 0.70721,\
        0.70416, 0.70136, 0.69745, 0.69164, 0.68504, 0.67800, 0.67072, 0.66330, 0.65586, 0.64849, 0.64128,\
        0.63435, 0.62779, 0.62173, 0.61627, 0.61150, 0.60742, 0.60384, 0.60019, 0.59603, 0.59133, 0.58628],\
        [0.73613, 0.74359, 0.75088, 0.75799, 0.76494, 0.77171, 0.77831, 0.78473, 0.79096, 0.79712, 0.80362,\
        0.81096, 0.81904, 0.82758, 0.83634, 0.84515, 0.85382, 0.86217, 0.86996, 0.87694, 0.88278, 0.88725,\
        0.89038, 0.89270, 0.89511, 0.89890, 0.90590, 0.91555, 0.92689, 0.93828, 0.94857, 0.95692, 0.96270,\
        0.96546, 0.96500, 0.96137, 0.95485, 0.94595, 0.93529, 0.92305, 0.90923, 0.89836, 0.88531, 0.86672,\
        0.85045, 0.83654, 0.83136, 0.84752, 0.86445, 0.88270, 0.89815, 0.92637, 0.94859, 0.96501, 0.97702,\
        0.98579, 0.99199, 0.99559, 0.99653, 0.99489, 0.99091, 0.98497, 0.97754, 0.96914, 0.96024, 0.95126,\
        0.94255, 0.93435, 0.92682, 0.91994, 0.91342, 0.90662, 0.89911, 0.89131, 0.88293, 0.87422, 0.86546,\
        0.85688, 0.84865, 0.84090, 0.83373, 0.82717, 0.82119, 0.81563, 0.81020, 0.80451, 0.79791, 0.79043,\
        0.78237, 0.77386, 0.76501, 0.75598, 0.74696, 0.73816, 0.72982, 0.72222, 0.71559, 0.71012, 0.70589,\
        0.70268, 0.69942, 0.69548, 0.69027, 0.68410, 0.67737, 0.67031, 0.66305, 0.65571, 0.64840, 0.64123,\
        0.63428, 0.62766, 0.62149, 0.61586, 0.61085, 0.60649, 0.60276, 0.59906, 0.59478, 0.59016, 0.58523],\
        [0.73755, 0.74504, 0.75238, 0.75957, 0.76664, 0.77358, 0.78043, 0.78715, 0.79375, 0.80024, 0.80673,\
        0.81380, 0.82157, 0.82991, 0.83863, 0.84753, 0.85644, 0.86516, 0.87345, 0.88101, 0.88745, 0.89236,\
        0.89558, 0.89792, 0.90088, 0.90543, 0.91304, 0.92303, 0.93432, 0.94561, 0.95567, 0.96347, 0.96814,\
        0.96904, 0.96580, 0.95841, 0.94720, 0.93261, 0.91538, 0.89646, 0.87542, 0.85692, 0.83451, 0.79958,\
        0.76080, 0.73601, 0.74479, 0.77500, 0.80358, 0.82870, 0.85360, 0.89123, 0.91972, 0.94449, 0.96191,\
        0.97603, 0.98711, 0.99473, 0.99879, 0.99944, 0.99704, 0.99209, 0.98518, 0.97688, 0.96776, 0.95830,\
        0.94892, 0.93989, 0.93140, 0.92345, 0.91583, 0.90822, 0.90044, 0.89248, 0.88420, 0.87577, 0.86736,\
        0.85915, 0.85127, 0.84383, 0.83687, 0.83039, 0.82433, 0.81851, 0.81266, 0.80641, 0.79950, 0.79212,\
        0.78407, 0.77544, 0.76640, 0.75710, 0.74774, 0.73853, 0.72972, 0.72158, 0.71437, 0.70835, 0.70355,\
        0.70010, 0.69695, 0.69293, 0.68810, 0.68244, 0.67615, 0.66941, 0.66240, 0.65523, 0.64802, 0.64088,\
        0.63391, 0.62721, 0.62088, 0.61500, 0.60966, 0.60488, 0.60066, 0.59702, 0.59294, 0.58851, 0.58378],\
        [0.73884, 0.74636, 0.75373, 0.76098, 0.76813, 0.77522, 0.78227, 0.78926, 0.79612, 0.80272, 0.80922,\
        0.81613, 0.82367, 0.83184, 0.84048, 0.84943, 0.85852, 0.86756, 0.87634, 0.88454, 0.89175, 0.89743,\
        0.90096, 0.90368, 0.90738, 0.91300, 0.92111, 0.93134, 0.94267, 0.95389, 0.96371, 0.97092, 0.97443,\
        0.97332, 0.96700, 0.95537, 0.93891, 0.91814, 0.89287, 0.86326, 0.83121, 0.80106, 0.76140, 0.71130,\
        0.65521, 0.63195, 0.65383, 0.70151, 0.73065, 0.76648, 0.79469, 0.83325, 0.88115, 0.91736, 0.94226,\
        0.96417, 0.98125, 0.99339, 1.00075, 1.00370, 1.00278, 0.99868, 0.99210, 0.98376, 0.97428, 0.96423,\
        0.95407, 0.94416, 0.93472, 0.92582, 0.91737, 0.90922, 0.90120, 0.89303, 0.88479, 0.87660, 0.86853,\
        0.86070, 0.85319, 0.84606, 0.83933, 0.83297, 0.82688, 0.82091, 0.81482, 0.80838, 0.80146, 0.79395,\
        0.78580, 0.77706, 0.76785, 0.75829, 0.74859, 0.73895, 0.72961, 0.72084, 0.71296, 0.70632, 0.70112,\
        0.69721, 0.69395, 0.69022, 0.68576, 0.68053, 0.67465, 0.66826, 0.66151, 0.65454, 0.64745, 0.64036,\
        0.63338, 0.62658, 0.62007, 0.61393, 0.60826, 0.60318, 0.59872, 0.59479, 0.59085, 0.58663, 0.58212],\
        [0.74000, 0.74752, 0.75489, 0.76214, 0.76931, 0.77645, 0.78359, 0.79074, 0.79775, 0.80432, 0.81076,\
        0.81759, 0.82503, 0.83309, 0.84166, 0.85061, 0.85978, 0.86901, 0.87812, 0.88684, 0.89472, 0.90085,\
        0.90495, 0.90847, 0.91313, 0.91979, 0.92870, 0.93944, 0.95110, 0.96247, 0.97224, 0.97906, 0.98160,\
        0.97863, 0.96918, 0.95293, 0.93066, 0.90412, 0.87194, 0.83081, 0.78359, 0.73813, 0.68444, 0.62645,\
        0.56386, 0.53674, 0.56995, 0.62237, 0.66373, 0.70868, 0.74403, 0.78694, 0.84303, 0.88770, 0.92418,\
        0.95359, 0.97624, 0.99272, 1.00317, 1.00812, 1.00837, 1.00482, 0.99833, 0.98973, 0.97973, 0.96896,\
        0.95795, 0.94710, 0.93673, 0.92698, 0.91786, 0.90921, 0.90084, 0.89268, 0.88462, 0.87670, 0.86898,\
        0.86154, 0.85441, 0.84762, 0.84114, 0.83494, 0.82892, 0.82292, 0.81678, 0.81030, 0.80336, 0.79581,\
        0.78761, 0.77880, 0.76943, 0.75964, 0.74963, 0.73960, 0.72978, 0.72040, 0.71178, 0.70442, 0.69891,\
        0.69491, 0.69152, 0.68792, 0.68370, 0.67878, 0.67321, 0.66710, 0.66059, 0.65379, 0.64682, 0.63979,\
        0.63279, 0.62592, 0.61927, 0.61291, 0.60694, 0.60155, 0.59699, 0.59292, 0.58894, 0.58481, 0.58044],\
        [0.74102, 0.74852, 0.75584, 0.76303, 0.77011, 0.77716, 0.78421, 0.79130, 0.79831, 0.80479, 0.81119,\
        0.81805, 0.82550, 0.83353, 0.84206, 0.85094, 0.86007, 0.86928, 0.87843, 0.88729, 0.89547, 0.90198,\
        0.90659, 0.91112, 0.91697, 0.92489, 0.93492, 0.94655, 0.95887, 0.97071, 0.98075, 0.98752, 0.98950,\
        0.98511, 0.97288, 0.95195, 0.92305, 0.89024, 0.85217, 0.79993, 0.73952, 0.69188, 0.63432, 0.57217,\
        0.51382, 0.47607, 0.50330, 0.55404, 0.59665, 0.64989, 0.70676, 0.75879, 0.81919, 0.87246, 0.91374,\
        0.94605, 0.97360, 0.99385, 1.00671, 1.01304, 1.01395, 1.01054, 1.00384, 0.99475, 0.98406, 0.97246,\
        0.96051, 0.94870, 0.93739, 0.92686, 0.91720, 0.90831, 0.89980, 0.89164, 0.88373, 0.87609, 0.86875,\
        0.86172, 0.85499, 0.84855, 0.84236, 0.83637, 0.83049, 0.82458, 0.81849, 0.81207, 0.80515, 0.79764,\
        0.78951, 0.78068, 0.77118, 0.76120, 0.75094, 0.74063, 0.73047, 0.72067, 0.71150, 0.70338, 0.69750,\
        0.69362, 0.69015, 0.68646, 0.68227, 0.67746, 0.67206, 0.66613, 0.65979, 0.65313, 0.64626, 0.63928,\
        0.63228, 0.62537, 0.61862, 0.61214, 0.60601, 0.60041, 0.59568, 0.59146, 0.58737, 0.58323, 0.57891],\
        [0.74191, 0.74936, 0.75660, 0.76366, 0.77057, 0.77738, 0.78413, 0.79086, 0.79752, 0.80392, 0.81050,\
        0.81756, 0.82514, 0.83321, 0.84168, 0.85045, 0.85938, 0.86835, 0.87720, 0.88574, 0.89361, 0.90031,\
        0.90567, 0.91119, 0.91843, 0.92787, 0.93932, 0.95214, 0.96542, 0.97802, 0.98861, 0.99570, 0.99762,\
        0.99250, 0.97828, 0.95321, 0.91739, 0.87643, 0.83040, 0.76183, 0.69466, 0.64760, 0.57765, 0.50266,\
        0.43141, 0.41117, 0.43573, 0.49269, 0.54986, 0.60296, 0.66164, 0.72173, 0.79558, 0.85800, 0.90105,\
        0.94224, 0.97467, 0.99752, 1.01169, 1.01853, 1.01946, 1.01576, 1.00853, 0.99875, 0.98724, 0.97471,\
        0.96178, 0.94897, 0.93671, 0.92537, 0.91522, 0.90632, 0.89806, 0.88983, 0.88210, 0.87481, 0.86790,\
        0.86131, 0.85500, 0.84893, 0.84305, 0.83731, 0.83162, 0.82587, 0.81994, 0.81368, 0.80689, 0.79949,\
        0.79155, 0.78271, 0.77307, 0.76293, 0.75252, 0.74208, 0.73181, 0.72192, 0.71271, 0.70463, 0.69851,\
        0.69405, 0.69008, 0.68603, 0.68163, 0.67674, 0.67135, 0.66549, 0.65924, 0.65266, 0.64586, 0.63892,\
        0.63194, 0.62500, 0.61822, 0.61170, 0.60557, 0.59999, 0.59497, 0.59045, 0.58618, 0.58194, 0.57760],\
        [0.74270, 0.75010, 0.75723, 0.76412, 0.77078, 0.77727, 0.78360, 0.78981, 0.79592, 0.80222, 0.80911,\
        0.81643, 0.82416, 0.83228, 0.84069, 0.84928, 0.85792, 0.86647, 0.87478, 0.88270, 0.89002, 0.89658,\
        0.90220, 0.90878, 0.91758, 0.92876, 0.94181, 0.95600, 0.97038, 0.98387, 0.99517, 1.00280, 1.00505,\
        0.99985, 0.98460, 0.95610, 0.91160, 0.85333, 0.78982, 0.70906, 0.64439, 0.58290, 0.48946, 0.41004,\
        0.35136, 0.33919, 0.35733, 0.40977, 0.48791, 0.54639, 0.61057, 0.68222, 0.77067, 0.84810, 0.90371,\
        0.94582, 0.97991, 1.00357, 1.01776, 1.02423, 1.02459, 1.02021, 1.01224, 1.00164, 0.98923, 0.97575,\
        0.96183, 0.94803, 0.93482, 0.92262, 0.91180, 0.90266, 0.89460, 0.88693, 0.87971, 0.87294, 0.86654,\
        0.86042, 0.85454, 0.84884, 0.84328, 0.83780, 0.83232, 0.82678, 0.82107, 0.81507, 0.80866, 0.80169,\
        0.79385, 0.78475, 0.77493, 0.76469, 0.75427, 0.74387, 0.73370, 0.72400, 0.71507, 0.70733, 0.70108,\
        0.69586, 0.69115, 0.68655, 0.68176, 0.67664, 0.67114, 0.66525, 0.65901, 0.65247, 0.64571, 0.63880,\
        0.63182, 0.62488, 0.61806, 0.61151, 0.60535, 0.59969, 0.59451, 0.58978, 0.58533, 0.58097, 0.57656],\
        [0.74342, 0.75077, 0.75781, 0.76452, 0.77093, 0.77708, 0.78299, 0.78877, 0.79456, 0.80075, 0.80762,\
        0.81503, 0.82287, 0.83101, 0.83934, 0.84771, 0.85600, 0.86406, 0.87174, 0.87891, 0.88547, 0.89140,\
        0.89726, 0.90465, 0.91509, 0.92803, 0.94265, 0.95814, 0.97357, 0.98787, 0.99981, 1.00794, 1.01056,\
        1.00551, 0.98994, 0.95941, 0.90559, 0.82531, 0.74018, 0.65567, 0.57671, 0.50114, 0.42456, 0.35814,\
        0.31255, 0.29419, 0.30457, 0.35148, 0.41525, 0.47868, 0.54946, 0.63736, 0.73321, 0.82568, 0.89536,\
        0.94998, 0.98710, 1.01055, 1.02382, 1.02930, 1.02876, 1.02354, 1.01474, 1.00330, 0.99003, 0.97564,\
        0.96080, 0.94609, 0.93202, 0.91903, 0.90748, 0.89793, 0.89031, 0.88336, 0.87685, 0.87069, 0.86483,\
        0.85920, 0.85373, 0.84839, 0.84312, 0.83788, 0.83261, 0.82726, 0.82174, 0.81599, 0.80993, 0.80341,\
        0.79552, 0.78628, 0.77645, 0.76629, 0.75600, 0.74579, 0.73587, 0.72647, 0.71787, 0.71030, 0.70384,\
        0.69814, 0.69287, 0.68773, 0.68249, 0.67707, 0.67138, 0.66540, 0.65912, 0.65258, 0.64583, 0.63893,\
        0.63196, 0.62500, 0.61816, 0.61153, 0.60526, 0.59947, 0.59421, 0.58937, 0.58478, 0.58030, 0.57578],\
        [0.74411, 0.75145, 0.75841, 0.76499, 0.77118, 0.77703, 0.78260, 0.78803, 0.79355, 0.79953, 0.80627,\
        0.81369, 0.82157, 0.82969, 0.83791, 0.84608, 0.85403, 0.86163, 0.86873, 0.87524, 0.88116, 0.88671,\
        0.89261, 0.90050, 0.91219, 0.92648, 0.94231, 0.95879, 0.97497, 0.98980, 1.00209, 1.01042, 1.01304,\
        1.00773, 0.99145, 0.95963, 0.90243, 0.80378, 0.71454, 0.61463, 0.53222, 0.46050, 0.39261, 0.32989,\
        0.28805, 0.27012, 0.28091, 0.32132, 0.38144, 0.44169, 0.51137, 0.59359, 0.70382, 0.80100, 0.89418,\
        0.95663, 0.99400, 1.01626, 1.02828, 1.03273, 1.03132, 1.02533, 1.01582, 1.00366, 0.98965, 0.97449,\
        0.95886, 0.94339, 0.92867, 0.91522, 0.90345, 0.89390, 0.88636, 0.87988, 0.87395, 0.86836, 0.86300,\
        0.85781, 0.85271, 0.84768, 0.84266, 0.83762, 0.83251, 0.82729, 0.82188, 0.81623, 0.81018, 0.80348,\
        0.79575, 0.78695, 0.77744, 0.76755, 0.75754, 0.74763, 0.73803, 0.72895, 0.72060, 0.71313, 0.70652,\
        0.70055, 0.69491, 0.68926, 0.68360, 0.67788, 0.67200, 0.66590, 0.65956, 0.65299, 0.64623, 0.63933,\
        0.63236, 0.62540, 0.61852, 0.61182, 0.60542, 0.59949, 0.59417, 0.58925, 0.58454, 0.57991, 0.57527],\
        [0.74480, 0.75217, 0.75912, 0.76562, 0.77166, 0.77728, 0.78258, 0.78772, 0.79294, 0.79862, 0.80523,\
        0.81264, 0.82049, 0.82856, 0.83667, 0.84466, 0.85236, 0.85961, 0.86630, 0.87238, 0.87795, 0.88339,\
        0.88958, 0.89797, 0.90998, 0.92484, 0.94129, 0.95826, 0.97474, 0.98969, 1.00191, 1.00999, 1.01214,\
        1.00601, 0.98818, 0.95312, 0.89303, 0.81109, 0.72498, 0.60981, 0.53035, 0.46094, 0.39524, 0.33126,\
        0.28348, 0.26918, 0.28729, 0.33217, 0.38761, 0.44426, 0.50910, 0.59944, 0.70160, 0.79766, 0.89768,\
        0.96118, 0.99714, 1.01839, 1.02972, 1.03364, 1.03176, 1.02533, 1.01536, 1.00272, 0.98818, 0.97246,\
        0.95623, 0.94017, 0.92500, 0.91143, 0.89988, 0.89053, 0.88310, 0.87690, 0.87138, 0.86622, 0.86127,\
        0.85642, 0.85162, 0.84682, 0.84199, 0.83708, 0.83207, 0.82690, 0.82151, 0.81584, 0.80973, 0.80293,\
        0.79540, 0.78703, 0.77793, 0.76842, 0.75878, 0.74923, 0.73997, 0.73119, 0.72304, 0.71562, 0.70893,\
        0.70285, 0.69688, 0.69079, 0.68484, 0.67892, 0.67289, 0.66669, 0.66028, 0.65368, 0.64690, 0.63999,\
        0.63302, 0.62605, 0.61917, 0.61244, 0.60597, 0.59993, 0.59452, 0.58948, 0.58461, 0.57980, 0.57499],\
        [0.74548, 0.75297, 0.75998, 0.76648, 0.77246, 0.77795, 0.78305, 0.78796, 0.79294, 0.79836, 0.80480,\
        0.81208, 0.81981, 0.82778, 0.83579, 0.84367, 0.85124, 0.85833, 0.86483, 0.87073, 0.87620, 0.88168,\
        0.88806, 0.89663, 0.90857, 0.92346, 0.93994, 0.95685, 0.97314, 0.98774, 0.99947, 1.00692, 1.00835,\
        1.00148, 0.98319, 0.94923, 0.89513, 0.82283, 0.75185, 0.66199, 0.57616, 0.48263, 0.41589, 0.37484,\
        0.32127, 0.30579, 0.32616, 0.37662, 0.42861, 0.48578, 0.54784, 0.63728, 0.73810, 0.81802, 0.90206,\
        0.95855, 0.99410, 1.01587, 1.02763, 1.03177, 1.02996, 1.02348, 1.01339, 1.00056, 0.98579, 0.96977,\
        0.95319, 0.93674, 0.92121, 0.90753, 0.89643, 0.88755, 0.88048, 0.87458, 0.86936, 0.86450, 0.85981,\
        0.85519, 0.85057, 0.84591, 0.84118, 0.83634, 0.83135, 0.82617, 0.82074, 0.81500, 0.80892, 0.80238,\
        0.79500, 0.78678, 0.77801, 0.76889, 0.75964, 0.75047, 0.74156, 0.73305, 0.72507, 0.71768, 0.71083,\
        0.70438, 0.69812, 0.69208, 0.68610, 0.68010, 0.67399, 0.66771, 0.66125, 0.65459, 0.64778, 0.64086,\
        0.63389, 0.62694, 0.62007, 0.61338, 0.60695, 0.60090, 0.59533, 0.59005, 0.58493, 0.57990, 0.57490],\
        [0.74616, 0.75383, 0.76100, 0.76761, 0.77362, 0.77905, 0.78402, 0.78875, 0.79359, 0.79895, 0.80509,\
        0.81204, 0.81957, 0.82742, 0.83537, 0.84323, 0.85080, 0.85793, 0.86447, 0.87044, 0.87599, 0.88157,\
        0.88795, 0.89623, 0.90792, 0.92252, 0.93852, 0.95485, 0.97046, 0.98432, 0.99523, 1.00185, 1.00254,\
        0.99535, 0.97797, 0.94784, 0.90154, 0.83596, 0.77359, 0.70881, 0.65155, 0.58122, 0.47114, 0.42022,\
        0.38608, 0.36418, 0.38139, 0.42338, 0.47397, 0.54818, 0.62023, 0.70050, 0.77961, 0.84242, 0.89973,\
        0.95086, 0.98715, 1.01015, 1.02287, 1.02764, 1.02624, 1.02003, 1.01010, 0.99739, 0.98267, 0.96667,\
        0.95007, 0.93354, 0.91782, 0.90386, 0.89327, 0.88524, 0.87867, 0.87306, 0.86803, 0.86331, 0.85873,\
        0.85420, 0.84965, 0.84503, 0.84032, 0.83547, 0.83045, 0.82520, 0.81968, 0.81383, 0.80760, 0.80111,\
        0.79404, 0.78611, 0.77770, 0.76897, 0.76012, 0.75132, 0.74274, 0.73450, 0.72668, 0.71932, 0.71238,\
        0.70575, 0.69942, 0.69338, 0.68741, 0.68140, 0.67525, 0.66892, 0.66240, 0.65569, 0.64883, 0.64188,\
        0.63490, 0.62796, 0.62114, 0.61450, 0.60815, 0.60215, 0.59638, 0.59081, 0.58541, 0.58014, 0.57493],\
        [0.74676, 0.75472, 0.76215, 0.76898, 0.77514, 0.78061, 0.78549, 0.79001, 0.79460, 0.79976, 0.80565,\
        0.81234, 0.81967, 0.82741, 0.83535, 0.84329, 0.85104, 0.85840, 0.86522, 0.87147, 0.87725, 0.88298,\
        0.88937, 0.89750, 0.90855, 0.92220, 0.93718, 0.95245, 0.96701, 0.97982, 0.98974, 0.99549, 0.99560,\
        0.98843, 0.97225, 0.94536, 0.90632, 0.85731, 0.80658, 0.74690, 0.69472, 0.65895, 0.57996, 0.47293,\
        0.43885, 0.41974, 0.43415, 0.47667, 0.54218, 0.61950, 0.67823, 0.74289, 0.80766, 0.86056, 0.90614,\
        0.94809, 0.98157, 1.00415, 1.01710, 1.02225, 1.02122, 1.01537, 1.00579, 0.99342, 0.97905, 0.96340,\
        0.94715, 0.93100, 0.91573, 0.90231, 0.89206, 0.88429, 0.87795, 0.87246, 0.86745, 0.86270, 0.85808,\
        0.85349, 0.84889, 0.84422, 0.83945, 0.83453, 0.82943, 0.82410, 0.81848, 0.81254, 0.80617, 0.79939,\
        0.79255, 0.78507, 0.77704, 0.76869, 0.76021, 0.75177, 0.74350, 0.73553, 0.72790, 0.72065, 0.71374,\
        0.70714, 0.70085, 0.69477, 0.68881, 0.68282, 0.67665, 0.67027, 0.66368, 0.65689, 0.64997, 0.64297,\
        0.63597, 0.62903, 0.62221, 0.61561, 0.60927, 0.60320, 0.59730, 0.59152, 0.58591, 0.58042, 0.57502],\
        [0.74722, 0.75555, 0.76335, 0.77053, 0.77696, 0.78259, 0.78744, 0.79169, 0.79581, 0.80056, 0.80628,\
        0.81280, 0.81996, 0.82761, 0.83559, 0.84371, 0.85177, 0.85957, 0.86689, 0.87362, 0.87979, 0.88567,\
        0.89195, 0.89984, 0.90999, 0.92228, 0.93586, 0.94975, 0.96300, 0.97462, 0.98353, 0.98854, 0.98837,\
        0.98165, 0.96707, 0.94367, 0.91184, 0.87562, 0.83666, 0.78689, 0.72721, 0.68845, 0.64002, 0.56660,\
        0.51087, 0.47992, 0.50333, 0.56174, 0.63083, 0.68524, 0.72720, 0.77304, 0.82820, 0.87532, 0.91515,\
        0.95123, 0.97986, 0.99990, 1.01173, 1.01651, 1.01552, 1.00993, 1.00075, 0.98888, 0.97509, 0.96009,\
        0.94455, 0.92916, 0.91476, 0.90244, 0.89244, 0.88470, 0.87837, 0.87278, 0.86759, 0.86263, 0.85781,\
        0.85305, 0.84829, 0.84349, 0.83859, 0.83356, 0.82835, 0.82291, 0.81721, 0.81124, 0.80503, 0.79846,\
        0.79136, 0.78392, 0.77613, 0.76809, 0.75993, 0.75181, 0.74385, 0.73615, 0.72875, 0.72168, 0.71491,\
        0.70840, 0.70214, 0.69613, 0.69029, 0.68436, 0.67817, 0.67171, 0.66501, 0.65813, 0.65112, 0.64405,\
        0.63700, 0.63002, 0.62319, 0.61656, 0.61017, 0.60401, 0.59800, 0.59207, 0.58630, 0.58066, 0.57510],\
        [0.74742, 0.75618, 0.76447, 0.77212, 0.77898, 0.78492, 0.78987, 0.79393, 0.79749, 0.80167, 0.80713,\
        0.81337, 0.82028, 0.82781, 0.83583, 0.84421, 0.85272, 0.86113, 0.86916, 0.87658, 0.88328, 0.88935,\
        0.89514, 0.90211, 0.91140, 0.92233, 0.93434, 0.94670, 0.95856, 0.96901, 0.97703, 0.98158, 0.98156,\
        0.97591, 0.96372, 0.94462, 0.91947, 0.89202, 0.86401, 0.82789, 0.78194, 0.73736, 0.69537, 0.65343,\
        0.59423, 0.55280, 0.58340, 0.64699, 0.69716, 0.73763, 0.76650, 0.79924, 0.85280, 0.89828, 0.93064,\
        0.95895, 0.98160, 0.99769, 1.00727, 1.01094, 1.00955, 1.00401, 0.99520, 0.98391, 0.97087, 0.95675,\
        0.94220, 0.92785, 0.91430, 0.90225, 0.89333, 0.88616, 0.87976, 0.87385, 0.86829, 0.86298, 0.85783,\
        0.85279, 0.84780, 0.84279, 0.83773, 0.83256, 0.82722, 0.82167, 0.81586, 0.80981, 0.80352, 0.79696,\
        0.78997, 0.78263, 0.77502, 0.76720, 0.75931, 0.75146, 0.74378, 0.73635, 0.72922, 0.72240, 0.71587,\
        0.70956, 0.70339, 0.69756, 0.69196, 0.68602, 0.67974, 0.67315, 0.66631, 0.65930, 0.65217, 0.64502,\
        0.63789, 0.63087, 0.62399, 0.61729, 0.61080, 0.60450, 0.59837, 0.59240, 0.58655, 0.58080, 0.57513],\
        [0.74719, 0.75645, 0.76530, 0.77355, 0.78099, 0.78739, 0.79265, 0.79677, 0.79998, 0.80360, 0.80827,\
        0.81389, 0.82038, 0.82770, 0.83576, 0.84441, 0.85347, 0.86263, 0.87156, 0.87985, 0.88723, 0.89367,\
        0.89943, 0.90508, 0.91277, 0.92204, 0.93238, 0.94320, 0.95372, 0.96310, 0.97047, 0.97490, 0.97552,\
        0.97156, 0.96252, 0.94848, 0.93052, 0.91121, 0.89119, 0.86533, 0.83339, 0.80147, 0.77089, 0.73822,\
        0.70183, 0.66520, 0.68428, 0.73032, 0.76458, 0.79370, 0.81411, 0.84272, 0.88634, 0.92340, 0.94848,\
        0.96870, 0.98498, 0.99665, 1.00343, 1.00552, 1.00342, 0.99776, 0.98924, 0.97855, 0.96635, 0.95328,\
        0.93995, 0.92695, 0.91484, 0.90432, 0.89591, 0.88861, 0.88179, 0.87537, 0.86929, 0.86351, 0.85797,\
        0.85259, 0.84731, 0.84208, 0.83683, 0.83150, 0.82604, 0.82038, 0.81447, 0.80827, 0.80184, 0.79525,\
        0.78839, 0.78117, 0.77369, 0.76604, 0.75834, 0.75072, 0.74328, 0.73611, 0.72927, 0.72277, 0.71660,\
        0.71069, 0.70500, 0.69955, 0.69381, 0.68770, 0.68125, 0.67448, 0.66747, 0.66030, 0.65304, 0.64578,\
        0.63858, 0.63148, 0.62454, 0.61778, 0.61121, 0.60480, 0.59858, 0.59256, 0.58666, 0.58084, 0.57507],\
        [0.74636, 0.75613, 0.76560, 0.77453, 0.78266, 0.78965, 0.79527, 0.79953, 0.80252, 0.80538, 0.80911,\
        0.81395, 0.81991, 0.82694, 0.83497, 0.84389, 0.85351, 0.86354, 0.87352, 0.88287, 0.89097, 0.89752,\
        0.90281, 0.90761, 0.91349, 0.92096, 0.92969, 0.93907, 0.94840, 0.95693, 0.96389, 0.96853, 0.97017,\
        0.96829, 0.96271, 0.95371, 0.94226, 0.92963, 0.91554, 0.89738, 0.87542, 0.85533, 0.83524, 0.80708,\
        0.78158, 0.76330, 0.77809, 0.80427, 0.82333, 0.84272, 0.86140, 0.88735, 0.91652, 0.94241, 0.96171,\
        0.97641, 0.98766, 0.99543, 0.99951, 0.99995, 0.99701, 0.99114, 0.98287, 0.97277, 0.96144, 0.94946,\
        0.93744, 0.92596, 0.91557, 0.90666, 0.89887, 0.89127, 0.88384, 0.87681, 0.87020, 0.86395, 0.85800,\
        0.85227, 0.84671, 0.84124, 0.83581, 0.83035, 0.82480, 0.81908, 0.81310, 0.80681, 0.80031, 0.79367,\
        0.78679, 0.77960, 0.77216, 0.76459, 0.75702, 0.74956, 0.74231, 0.73538, 0.72882, 0.72266, 0.71690,\
        0.71148, 0.70628, 0.70105, 0.69533, 0.68913, 0.68251, 0.67555, 0.66835, 0.66101, 0.65362, 0.64625,\
        0.63897, 0.63182, 0.62483, 0.61802, 0.61141, 0.60496, 0.59870, 0.59262, 0.58666, 0.58076, 0.57491],\
        [0.74474, 0.75498, 0.76504, 0.77469, 0.78360, 0.79129, 0.79721, 0.80128, 0.80387, 0.80605, 0.80895,\
        0.81307, 0.81850, 0.82517, 0.83308, 0.84217, 0.85230, 0.86319, 0.87433, 0.88495, 0.89399, 0.90049,\
        0.90480, 0.90825, 0.91259, 0.91859, 0.92597, 0.93415, 0.94252, 0.95042, 0.95721, 0.96228, 0.96512,\
        0.96540, 0.96308, 0.95844, 0.95207, 0.94441, 0.93508, 0.92301, 0.90864, 0.89674, 0.88687, 0.87579,\
        0.86629, 0.85704, 0.86129, 0.86801, 0.87305, 0.88166, 0.89678, 0.91718, 0.93785, 0.95576, 0.96995,\
        0.98059, 0.98822, 0.99298, 0.99485, 0.99385, 0.99014, 0.98406, 0.97601, 0.96648, 0.95598, 0.94506,\
        0.93425, 0.92411, 0.91514, 0.90749, 0.90055, 0.89284, 0.88506, 0.87763, 0.87061, 0.86399, 0.85770,\
        0.85169, 0.84588, 0.84020, 0.83460, 0.82903, 0.82342, 0.81771, 0.81179, 0.80548, 0.79887, 0.79221,\
        0.78521, 0.77790, 0.77041, 0.76285, 0.75533, 0.74796, 0.74086, 0.73410, 0.72778, 0.72192, 0.71655,\
        0.71159, 0.70682, 0.70181, 0.69628, 0.69010, 0.68334, 0.67618, 0.66879, 0.66130, 0.65380, 0.64634,\
        0.63900, 0.63181, 0.62479, 0.61798, 0.61136, 0.60495, 0.59869, 0.59254, 0.58650, 0.58055, 0.57464],\
        [0.74217, 0.75278, 0.76332, 0.77359, 0.78327, 0.79186, 0.79842, 0.80208, 0.80378, 0.80512, 0.80731,\
        0.81092, 0.81588, 0.82213, 0.82977, 0.83885, 0.84933, 0.86094, 0.87313, 0.88511, 0.89556, 0.90230,\
        0.90495, 0.90658, 0.90960, 0.91467, 0.92110, 0.92835, 0.93598, 0.94346, 0.95025, 0.95587, 0.95992,\
        0.96221, 0.96271, 0.96161, 0.95921, 0.95565, 0.95078, 0.94447, 0.93780, 0.93343, 0.93239, 0.93400,\
        0.93694, 0.93533, 0.92815, 0.91957, 0.91350, 0.91374, 0.92315, 0.93662, 0.95089, 0.96354, 0.97370,\
        0.98122, 0.98623, 0.98882, 0.98904, 0.98693, 0.98264, 0.97641, 0.96860, 0.95961, 0.94989, 0.93989,\
        0.93009, 0.92095, 0.91293, 0.90620, 0.89967, 0.89248, 0.88488, 0.87739, 0.87022, 0.86341, 0.85692,\
        0.85070, 0.84470, 0.83885, 0.83311, 0.82744, 0.82179, 0.81614, 0.81043, 0.80447, 0.79791, 0.79087,\
        0.78353, 0.77600, 0.76839, 0.76077, 0.75324, 0.74592, 0.73888, 0.73224, 0.72607, 0.72044, 0.71537,\
        0.71083, 0.70662, 0.70216, 0.69681, 0.69049, 0.68352, 0.67618, 0.66866, 0.66106, 0.65349, 0.64599,\
        0.63862, 0.63141, 0.62440, 0.61759, 0.61100, 0.60462, 0.59840, 0.59222, 0.58616, 0.58017, 0.57422],\
        [0.73855, 0.74935, 0.76017, 0.77082, 0.78097, 0.79020, 0.79766, 0.80076, 0.80170, 0.80237, 0.80409,\
        0.80748, 0.81202, 0.81773, 0.82486, 0.83368, 0.84424, 0.85626, 0.86911, 0.88193, 0.89344, 0.90062,\
        0.90193, 0.90229, 0.90465, 0.90951, 0.91516, 0.92166, 0.92876, 0.93599, 0.94288, 0.94905, 0.95420,\
        0.95820, 0.96102, 0.96276, 0.96356, 0.96352, 0.96272, 0.96147, 0.96076, 0.96199, 0.96577, 0.97116,\
        0.97531, 0.97399, 0.96578, 0.95419, 0.94418, 0.93978, 0.94236, 0.94944, 0.95820, 0.96661, 0.97359,\
        0.97870, 0.98181, 0.98289, 0.98197, 0.97910, 0.97442, 0.96816, 0.96062, 0.95215, 0.94313, 0.93396,\
        0.92500, 0.91655, 0.90892, 0.90303, 0.89740, 0.89049, 0.88323, 0.87597, 0.86889, 0.86208, 0.85553,\
        0.84922, 0.84310, 0.83713, 0.83127, 0.82550, 0.81978, 0.81412, 0.80852, 0.80291, 0.79652, 0.78910,\
        0.78147, 0.77376, 0.76602, 0.75833, 0.75077, 0.74343, 0.73640, 0.72979, 0.72369, 0.71815, 0.71323,\
        0.70894, 0.70527, 0.70172, 0.69649, 0.68993, 0.68281, 0.67539, 0.66782, 0.66021, 0.65262, 0.64513,\
        0.63778, 0.63061, 0.62364, 0.61686, 0.61028, 0.60388, 0.59769, 0.59161, 0.58560, 0.57962, 0.57366],\
        [0.73388, 0.74467, 0.75553, 0.76622, 0.77634, 0.78533, 0.79219, 0.79585, 0.79735, 0.79813, 0.79987,\
        0.80311, 0.80710, 0.81203, 0.81837, 0.82665, 0.83702, 0.84910, 0.86207, 0.87479, 0.88573, 0.89267,\
        0.89538, 0.89650, 0.89936, 0.90359, 0.90822, 0.91415, 0.92089, 0.92798, 0.93500, 0.94165, 0.94771,\
        0.95309, 0.95774, 0.96172, 0.96508, 0.96793, 0.97042, 0.97290, 0.97599, 0.98029, 0.98574, 0.99112,\
        0.99402, 0.99188, 0.98410, 0.97303, 0.96245, 0.95563, 0.95381, 0.95607, 0.96059, 0.96569, 0.97024,\
        0.97356, 0.97533, 0.97539, 0.97373, 0.97040, 0.96554, 0.95935, 0.95211, 0.94414, 0.93578, 0.92737,\
        0.91921, 0.91153, 0.90423, 0.89795, 0.89313, 0.88699, 0.88028, 0.87343, 0.86663, 0.85998, 0.85352,\
        0.84722, 0.84107, 0.83503, 0.82908, 0.82319, 0.81734, 0.81152, 0.80571, 0.79986, 0.79351, 0.78637,\
        0.77881, 0.77107, 0.76328, 0.75554, 0.74792, 0.74053, 0.73346, 0.72682, 0.72069, 0.71513, 0.71019,\
        0.70586, 0.70211, 0.69911, 0.69424, 0.68793, 0.68098, 0.67369, 0.66623, 0.65870, 0.65119, 0.64376,\
        0.63647, 0.62937, 0.62249, 0.61582, 0.60931, 0.60291, 0.59677, 0.59082, 0.58487, 0.57891, 0.57295],\
        [0.72827, 0.73888, 0.74959, 0.76007, 0.76986, 0.77837, 0.78491, 0.78909, 0.79143, 0.79302, 0.79503,\
        0.79784, 0.80127, 0.80526, 0.81057, 0.81806, 0.82809, 0.84007, 0.85281, 0.86500, 0.87528, 0.88248,\
        0.88670, 0.88952, 0.89274, 0.89627, 0.90043, 0.90605, 0.91254, 0.91952, 0.92663, 0.93361, 0.94034,\
        0.94676, 0.95281, 0.95849, 0.96381, 0.96882, 0.97365, 0.97851, 0.98365, 0.98915, 0.99458, 0.99879,\
        1.00011, 0.99711, 0.98971, 0.97956, 0.96935, 0.96153, 0.95737, 0.95670, 0.95844, 0.96132, 0.96421,\
        0.96633, 0.96723, 0.96667, 0.96459, 0.96103, 0.95614, 0.95010, 0.94318, 0.93568, 0.92796, 0.92029,\
        0.91286, 0.90614, 0.89990, 0.89376, 0.88823, 0.88249, 0.87634, 0.86997, 0.86356, 0.85719, 0.85092,\
        0.84473, 0.83863, 0.83258, 0.82657, 0.82057, 0.81457, 0.80853, 0.80245, 0.79635, 0.78999, 0.78305,\
        0.77565, 0.76798, 0.76020, 0.75243, 0.74476, 0.73730, 0.73015, 0.72342, 0.71722, 0.71159, 0.70653,\
        0.70213, 0.69824, 0.69469, 0.69028, 0.68460, 0.67810, 0.67113, 0.66391, 0.65656, 0.64920, 0.64189,\
        0.63470, 0.62771, 0.62095, 0.61447, 0.60821, 0.60203, 0.59595, 0.58997, 0.58400, 0.57804, 0.57208],\
        [0.72193, 0.73227, 0.74273, 0.75291, 0.76233, 0.77049, 0.77695, 0.78157, 0.78468, 0.78699, 0.78923,\
        0.79175, 0.79460, 0.79783, 0.80212, 0.80864, 0.81839, 0.83030, 0.84270, 0.85433, 0.86418, 0.87166,\
        0.87693, 0.88089, 0.88449, 0.88818, 0.89245, 0.89774, 0.90392, 0.91078, 0.91786, 0.92496, 0.93210,\
        0.93924, 0.94629, 0.95319, 0.95988, 0.96637, 0.97268, 0.97883, 0.98482, 0.99042, 0.99508, 0.99787,\
        0.99772, 0.99390, 0.98659, 0.97707, 0.96730, 0.95921, 0.95404, 0.95197, 0.95234, 0.95408, 0.95611,\
        0.95761, 0.95805, 0.95717, 0.95489, 0.95126, 0.94643, 0.94060, 0.93400, 0.92693, 0.91977, 0.91294,\
        0.90616, 0.89976, 0.89441, 0.88866, 0.88306, 0.87748, 0.87172, 0.86582, 0.85983, 0.85382, 0.84782,\
        0.84183, 0.83585, 0.82984, 0.82380, 0.81774, 0.81164, 0.80547, 0.79920, 0.79296, 0.78646, 0.77955,\
        0.77224, 0.76464, 0.75689, 0.74911, 0.74139, 0.73385, 0.72659, 0.71973, 0.71343, 0.70779, 0.70267,\
        0.69810, 0.69419, 0.69028, 0.68585, 0.68055, 0.67450, 0.66791, 0.66100, 0.65390, 0.64672, 0.63957,\
        0.63251, 0.62562, 0.61899, 0.61271, 0.60682, 0.60106, 0.59503, 0.58895, 0.58293, 0.57697, 0.57102],\
        [0.71520, 0.72523, 0.73544, 0.74534, 0.75446, 0.76241, 0.76890, 0.77388, 0.77759, 0.78047, 0.78298,\
        0.78542, 0.78795, 0.79077, 0.79438, 0.79992, 0.80956, 0.82125, 0.83306, 0.84400, 0.85341, 0.86096,\
        0.86679, 0.87145, 0.87556, 0.87964, 0.88413, 0.88932, 0.89528, 0.90192, 0.90880, 0.91578, 0.92309,\
        0.93069, 0.93841, 0.94610, 0.95366, 0.96102, 0.96811, 0.97480, 0.98090, 0.98604, 0.98967, 0.99111,\
        0.98968, 0.98506, 0.97759, 0.96831, 0.95880, 0.95073, 0.94534, 0.94302, 0.94321, 0.94480, 0.94669,\
        0.94805, 0.94836, 0.94736, 0.94501, 0.94139, 0.93667, 0.93108, 0.92482, 0.91811, 0.91130, 0.90525,\
        0.90012, 0.89366, 0.88811, 0.88271, 0.87735, 0.87202, 0.86663, 0.86116, 0.85561, 0.85000, 0.84434,\
        0.83861, 0.83281, 0.82691, 0.82090, 0.81478, 0.80864, 0.80252, 0.79618, 0.78972, 0.78306, 0.77609,\
        0.76878, 0.76121, 0.75347, 0.74567, 0.73792, 0.73031, 0.72295, 0.71593, 0.70945, 0.70382, 0.69894,\
        0.69428, 0.69008, 0.68592, 0.68139, 0.67627, 0.67053, 0.66428, 0.65767, 0.65084, 0.64388, 0.63689,\
        0.62997, 0.62318, 0.61660, 0.61037, 0.60470, 0.59935, 0.59346, 0.58749, 0.58156, 0.57566, 0.56977],\
        [0.70848, 0.71824, 0.72825, 0.73790, 0.74677, 0.75457, 0.76111, 0.76640, 0.77057, 0.77394, 0.77681,\
        0.77946, 0.78211, 0.78508, 0.78890, 0.79464, 0.80348, 0.81401, 0.82472, 0.83473, 0.84349, 0.85081,\
        0.85681, 0.86184, 0.86637, 0.87081, 0.87553, 0.88076, 0.88659, 0.89294, 0.89947, 0.90619, 0.91356,\
        0.92144, 0.92951, 0.93761, 0.94560, 0.95337, 0.96076, 0.96751, 0.97333, 0.97778, 0.98038, 0.98063,\
        0.97812, 0.97278, 0.96501, 0.95572, 0.94627, 0.93822, 0.93305, 0.93135, 0.93233, 0.93456, 0.93684,\
        0.93839, 0.93876, 0.93775, 0.93536, 0.93174, 0.92712, 0.92177, 0.91590, 0.90961, 0.90303, 0.89709,\
        0.89237, 0.88696, 0.88150, 0.87627, 0.87118, 0.86616, 0.86116, 0.85612, 0.85103, 0.84586, 0.84059,\
        0.83518, 0.82963, 0.82389, 0.81797, 0.81183, 0.80554, 0.79939, 0.79312, 0.78658, 0.77980, 0.77273,\
        0.76538, 0.75779, 0.75004, 0.74223, 0.73446, 0.72682, 0.71940, 0.71227, 0.70555, 0.69964, 0.69490,\
        0.69038, 0.68603, 0.68167, 0.67705, 0.67199, 0.66645, 0.66046, 0.65411, 0.64752, 0.64078, 0.63398,\
        0.62720, 0.62051, 0.61397, 0.60761, 0.60167, 0.59655, 0.59121, 0.58559, 0.57988, 0.57412, 0.56834],\
        [0.70230, 0.71194, 0.72171, 0.73103, 0.73962, 0.74726, 0.75385, 0.75937, 0.76394, 0.76777, 0.77109,\
        0.77418, 0.77729, 0.78079, 0.78524, 0.79133, 0.79933, 0.80848, 0.81786, 0.82670, 0.83460, 0.84142,\
        0.84728, 0.85242, 0.85719, 0.86190, 0.86682, 0.87211, 0.87784, 0.88391, 0.89018, 0.89671, 0.90404,\
        0.91192, 0.92005, 0.92820, 0.93626, 0.94410, 0.95148, 0.95805, 0.96342, 0.96716, 0.96884, 0.96812,\
        0.96478, 0.95887, 0.95083, 0.94151, 0.93206, 0.92395, 0.91912, 0.91879, 0.92134, 0.92465, 0.92754,\
        0.92939, 0.92986, 0.92882, 0.92634, 0.92263, 0.91802, 0.91285, 0.90732, 0.90166, 0.89544, 0.88968,\
        0.88457, 0.87951, 0.87441, 0.86944, 0.86465, 0.85998, 0.85539, 0.85083, 0.84622, 0.84152, 0.83668,\
        0.83164, 0.82639, 0.82089, 0.81513, 0.80912, 0.80280, 0.79644, 0.79016, 0.78357, 0.77668, 0.76951,\
        0.76208, 0.75445, 0.74669, 0.73887, 0.73109, 0.72346, 0.71607, 0.70902, 0.70233, 0.69634, 0.69128,\
        0.68665, 0.68214, 0.67761, 0.67289, 0.66784, 0.66240, 0.65659, 0.65046, 0.64409, 0.63755, 0.63094,\
        0.62432, 0.61779, 0.61140, 0.60519, 0.59920, 0.59394, 0.58881, 0.58347, 0.57799, 0.57241, 0.56675],\
        [0.69789, 0.70701, 0.71615, 0.72496, 0.73319, 0.74065, 0.74724, 0.75295, 0.75787, 0.76214, 0.76598,\
        0.76961, 0.77331, 0.77744, 0.78242, 0.78860, 0.79597, 0.80409, 0.81224, 0.81984, 0.82671, 0.83285,\
        0.83834, 0.84338, 0.84823, 0.85311, 0.85819, 0.86357, 0.86927, 0.87524, 0.88144, 0.88797, 0.89503,\
        0.90259, 0.91050, 0.91837, 0.92620, 0.93389, 0.94112, 0.94743, 0.95233, 0.95542, 0.95636, 0.95493,\
        0.95103, 0.94482, 0.93675, 0.92764, 0.91860, 0.91065, 0.90592, 0.90782, 0.91214, 0.91633, 0.91965,\
        0.92167, 0.92213, 0.92098, 0.91830, 0.91436, 0.90961, 0.90453, 0.89916, 0.89370, 0.88796, 0.88230,\
        0.87701, 0.87195, 0.86705, 0.86234, 0.85785, 0.85357, 0.84943, 0.84535, 0.84126, 0.83707, 0.83270,\
        0.82808, 0.82317, 0.81794, 0.81240, 0.80663, 0.80067, 0.79417, 0.78755, 0.78076, 0.77372, 0.76644,\
        0.75893, 0.75125, 0.74345, 0.73562, 0.72784, 0.72022, 0.71291, 0.70607, 0.69965, 0.69356, 0.68815,\
        0.68322, 0.67849, 0.67379, 0.66895, 0.66387, 0.65849, 0.65281, 0.64684, 0.64065, 0.63430, 0.62785,\
        0.62141, 0.61505, 0.60890, 0.60308, 0.59739, 0.59187, 0.58661, 0.58136, 0.57602, 0.57058, 0.56505],\
        [0.69456, 0.70300, 0.71143, 0.71970, 0.72754, 0.73479, 0.74135, 0.74722, 0.75244, 0.75713, 0.76147,\
        0.76566, 0.76995, 0.77462, 0.77996, 0.78616, 0.79316, 0.80056, 0.80750, 0.81384, 0.81968, 0.82504,\
        0.83003, 0.83483, 0.83965, 0.84462, 0.84984, 0.85534, 0.86111, 0.86709, 0.87331, 0.87980, 0.88664,\
        0.89385, 0.90131, 0.90853, 0.91594, 0.92341, 0.93048, 0.93653, 0.94102, 0.94357, 0.94395, 0.94206,\
        0.93793, 0.93177, 0.92404, 0.91558, 0.90787, 0.90178, 0.89836, 0.90164, 0.90612, 0.91033, 0.91363,\
        0.91557, 0.91589, 0.91450, 0.91154, 0.90729, 0.90214, 0.89710, 0.89178, 0.88625, 0.88058, 0.87494,\
        0.86953, 0.86441, 0.85958, 0.85507, 0.85089, 0.84700, 0.84333, 0.83979, 0.83625, 0.83260, 0.82874,\
        0.82458, 0.82004, 0.81508, 0.80971, 0.80402, 0.79807, 0.79174, 0.78504, 0.77810, 0.77091, 0.76351,\
        0.75591, 0.74817, 0.74035, 0.73251, 0.72473, 0.71712, 0.70979, 0.70296, 0.69669, 0.69072, 0.68518,\
        0.68002, 0.67509, 0.67020, 0.66524, 0.66011, 0.65476, 0.64915, 0.64332, 0.63728, 0.63109, 0.62481,\
        0.61852, 0.61230, 0.60627, 0.60053, 0.59511, 0.58974, 0.58449, 0.57929, 0.57404, 0.56871, 0.56329],\
        [0.69137, 0.69931, 0.70728, 0.71511, 0.72260, 0.72964, 0.73616, 0.74215, 0.74763, 0.75271, 0.75750,\
        0.76220, 0.76698, 0.77207, 0.77764, 0.78377, 0.79037, 0.79707, 0.80307, 0.80838, 0.81333, 0.81794,\
        0.82236, 0.82683, 0.83155, 0.83660, 0.84198, 0.84763, 0.85350, 0.85956, 0.86580, 0.87223, 0.87887,\
        0.88568, 0.89247, 0.89912, 0.90609, 0.91334, 0.92029, 0.92612, 0.93025, 0.93234, 0.93228, 0.93017,\
        0.92616, 0.92043, 0.91350, 0.90629, 0.89984, 0.89713, 0.89613, 0.89878, 0.90266, 0.90641, 0.90941,\
        0.91112, 0.91119, 0.90949, 0.90615, 0.90171, 0.89635, 0.89083, 0.88521, 0.87941, 0.87353, 0.86772,\
        0.86214, 0.85691, 0.85209, 0.84772, 0.84383, 0.84035, 0.83719, 0.83421, 0.83126, 0.82818, 0.82486,\
        0.82119, 0.81707, 0.81241, 0.80718, 0.80151, 0.79557, 0.78928, 0.78256, 0.77551, 0.76820, 0.76069,\
        0.75301, 0.74522, 0.73738, 0.72955, 0.72181, 0.71421, 0.70685, 0.69988, 0.69368, 0.68785, 0.68229,\
        0.67700, 0.67189, 0.66685, 0.66177, 0.65658, 0.65123, 0.64568, 0.63994, 0.63404, 0.62800, 0.62187,\
        0.61572, 0.60963, 0.60369, 0.59804, 0.59273, 0.58753, 0.58238, 0.57725, 0.57207, 0.56683, 0.56149],\
        [0.68852, 0.69609, 0.70370, 0.71113, 0.71829, 0.72513, 0.73161, 0.73769, 0.74341, 0.74881, 0.75400,\
        0.75912, 0.76430, 0.76968, 0.77536, 0.78133, 0.78745, 0.79338, 0.79867, 0.80335, 0.80767, 0.81159,\
        0.81536, 0.81944, 0.82407, 0.82921, 0.83476, 0.84058, 0.84658, 0.85272, 0.85897, 0.86532, 0.87177,\
        0.87824, 0.88464, 0.89097, 0.89744, 0.90442, 0.91119, 0.91677, 0.92055, 0.92223, 0.92175, 0.91954,\
        0.91603, 0.91109, 0.90510, 0.89990, 0.89560, 0.89368, 0.89411, 0.89668, 0.90030, 0.90378, 0.90656,\
        0.90803, 0.90783, 0.90585, 0.90207, 0.89713, 0.89157, 0.88555, 0.87938, 0.87312, 0.86687, 0.86075,\
        0.85491, 0.84949, 0.84462, 0.84036, 0.83674, 0.83369, 0.83107, 0.82870, 0.82636, 0.82388, 0.82111,\
        0.81794, 0.81427, 0.81001, 0.80500, 0.79930, 0.79338, 0.78698, 0.78015, 0.77297, 0.76555, 0.75793,\
        0.75018, 0.74235, 0.73452, 0.72673, 0.71906, 0.71156, 0.70429, 0.69742, 0.69109, 0.68517, 0.67954,\
        0.67413, 0.66888, 0.66371, 0.65852, 0.65327, 0.64790, 0.64239, 0.63674, 0.63095, 0.62505, 0.61907,\
        0.61306, 0.60710, 0.60132, 0.59580, 0.59053, 0.58540, 0.58032, 0.57524, 0.57012, 0.56494, 0.55967],\
        [0.68602, 0.69333, 0.70062, 0.70761, 0.71446, 0.72115, 0.72761, 0.73379, 0.73970, 0.74537, 0.75089,\
        0.75634, 0.76183, 0.76741, 0.77313, 0.77893, 0.78466, 0.79004, 0.79486, 0.79911, 0.80292, 0.80611,\
        0.80906, 0.81274, 0.81734, 0.82265, 0.82839, 0.83436, 0.84047, 0.84665, 0.85288, 0.85915, 0.86544,\
        0.87171, 0.87794, 0.88417, 0.89053, 0.89711, 0.90355, 0.90870, 0.91217, 0.91354, 0.91251, 0.91006,\
        0.90742, 0.90376, 0.89870, 0.89456, 0.89191, 0.89097, 0.89201, 0.89468, 0.89821, 0.90173, 0.90451,\
        0.90584, 0.90532, 0.90309, 0.89906, 0.89352, 0.88733, 0.88077, 0.87404, 0.86727, 0.86056, 0.85404,\
        0.84787, 0.84221, 0.83722, 0.83303, 0.82969, 0.82710, 0.82507, 0.82334, 0.82165, 0.81975, 0.81750,\
        0.81480, 0.81157, 0.80778, 0.80327, 0.79772, 0.79145, 0.78477, 0.77774, 0.77043, 0.76289, 0.75519,\
        0.74738, 0.73954, 0.73172, 0.72399, 0.71643, 0.70910, 0.70199, 0.69516, 0.68874, 0.68269, 0.67695,\
        0.67143, 0.66605, 0.66076, 0.65548, 0.65016, 0.64477, 0.63929, 0.63371, 0.62804, 0.62228, 0.61645,\
        0.61058, 0.60474, 0.59910, 0.59369, 0.58847, 0.58336, 0.57831, 0.57326, 0.56819, 0.56306, 0.55784],\
        [0.68375, 0.69075, 0.69762, 0.70434, 0.71104, 0.71765, 0.72411, 0.73036, 0.73643, 0.74232, 0.74810,\
        0.75382, 0.75954, 0.76528, 0.77104, 0.77673, 0.78221, 0.78730, 0.79185, 0.79583, 0.79931, 0.80157,\
        0.80352, 0.80694, 0.81167, 0.81718, 0.82306, 0.82912, 0.83525, 0.84140, 0.84757, 0.85373, 0.85988,\
        0.86601, 0.87214, 0.87831, 0.88459, 0.89095, 0.89705, 0.90144, 0.90515, 0.90635, 0.90479, 0.90140,\
        0.89972, 0.89713, 0.89335, 0.89007, 0.88823, 0.88803, 0.88950, 0.89234, 0.89600, 0.89976, 0.90272,\
        0.90409, 0.90306, 0.90008, 0.89577, 0.88987, 0.88320, 0.87616, 0.86895, 0.86171, 0.85456, 0.84764,\
        0.84109, 0.83513, 0.82997, 0.82581, 0.82274, 0.82066, 0.81930, 0.81826, 0.81721, 0.81587, 0.81406,\
        0.81170, 0.80877, 0.80526, 0.80110, 0.79570, 0.78927, 0.78240, 0.77522, 0.76779, 0.76017, 0.75241,\
        0.74458, 0.73673, 0.72893, 0.72126, 0.71379, 0.70662, 0.69974, 0.69292, 0.68645, 0.68032, 0.67448,\
        0.66886, 0.66337, 0.65797, 0.65261, 0.64723, 0.64181, 0.63635, 0.63083, 0.62526, 0.61966, 0.61401,\
        0.60831, 0.60255, 0.59702, 0.59170, 0.58651, 0.58140, 0.57635, 0.57131, 0.56627, 0.56117, 0.55600],\
        [0.68173, 0.68842, 0.69496, 0.70146, 0.70806, 0.71461, 0.72105, 0.72736, 0.73353, 0.73958, 0.74556,\
        0.75149, 0.75740, 0.76330, 0.76913, 0.77480, 0.78020, 0.78515, 0.78954, 0.79327, 0.79620, 0.79790,\
        0.79940, 0.80263, 0.80753, 0.81311, 0.81897, 0.82495, 0.83097, 0.83700, 0.84300, 0.84900, 0.85497,\
        0.86094, 0.86693, 0.87295, 0.87901, 0.88499, 0.89050, 0.89515, 0.89922, 0.89957, 0.89823, 0.89486,\
        0.89247, 0.89016, 0.88745, 0.88517, 0.88404, 0.88437, 0.88617, 0.88921, 0.89304, 0.89701, 0.90041,\
        0.90224, 0.90084, 0.89692, 0.89187, 0.88572, 0.87881, 0.87148, 0.86395, 0.85637, 0.84886, 0.84158,\
        0.83468, 0.82839, 0.82298, 0.81878, 0.81600, 0.81452, 0.81389, 0.81359, 0.81317, 0.81231, 0.81084,\
        0.80865, 0.80578, 0.80224, 0.79799, 0.79273, 0.78649, 0.77966, 0.77245, 0.76498, 0.75733, 0.74955,\
        0.74172, 0.73390, 0.72613, 0.71850, 0.71106, 0.70391, 0.69711, 0.69050, 0.68412, 0.67799, 0.67210,\
        0.66640, 0.66083, 0.65534, 0.64989, 0.64445, 0.63900, 0.63353, 0.62805, 0.62257, 0.61711, 0.61169,\
        0.60626, 0.60074, 0.59521, 0.58983, 0.58459, 0.57945, 0.57439, 0.56936, 0.56433, 0.55926, 0.55413],\
        [0.68001, 0.68650, 0.69285, 0.69917, 0.70558, 0.71202, 0.71841, 0.72472, 0.73094, 0.73710, 0.74321,\
        0.74931, 0.75540, 0.76145, 0.76740, 0.77316, 0.77860, 0.78357, 0.78792, 0.79152, 0.79421, 0.79605,\
        0.79780, 0.80081, 0.80537, 0.81062, 0.81618, 0.82187, 0.82762, 0.83338, 0.83913, 0.84487, 0.85060,\
        0.85633, 0.86206, 0.86779, 0.87347, 0.87897, 0.88402, 0.88838, 0.89136, 0.89191, 0.89057, 0.88803,\
        0.88547, 0.88316, 0.88104, 0.87947, 0.87892, 0.87965, 0.88169, 0.88484, 0.88876, 0.89292, 0.89676,\
        0.89868, 0.89666, 0.89263, 0.88724, 0.88095, 0.87399, 0.86659, 0.85894, 0.85119, 0.84349, 0.83596,\
        0.82878, 0.82218, 0.81647, 0.81214, 0.80964, 0.80888, 0.80907, 0.80948, 0.80964, 0.80918, 0.80790,\
        0.80572, 0.80273, 0.79903, 0.79463, 0.78937, 0.78328, 0.77656, 0.76941, 0.76197, 0.75433, 0.74659,\
        0.73879, 0.73102, 0.72332, 0.71575, 0.70836, 0.70120, 0.69448, 0.68804, 0.68177, 0.67569, 0.66979,\
        0.66404, 0.65841, 0.65284, 0.64732, 0.64182, 0.63632, 0.63082, 0.62535, 0.61991, 0.61455, 0.60928,\
        0.60411, 0.59892, 0.59335, 0.58790, 0.58261, 0.57745, 0.57239, 0.56737, 0.56236, 0.55732, 0.55223],\
        [0.67859, 0.68498, 0.69121, 0.69736, 0.70357, 0.70986, 0.71614, 0.72238, 0.72858, 0.73477, 0.74098,\
        0.74721, 0.75346, 0.75968, 0.76581, 0.77175, 0.77736, 0.78251, 0.78702, 0.79076, 0.79366, 0.79592,\
        0.79815, 0.80106, 0.80495, 0.80955, 0.81455, 0.81975, 0.82507, 0.83044, 0.83582, 0.84122, 0.84661,\
        0.85200, 0.85737, 0.86268, 0.86786, 0.87276, 0.87716, 0.88073, 0.88295, 0.88342, 0.88235, 0.88030,\
        0.87798, 0.87582, 0.87404, 0.87292, 0.87275, 0.87372, 0.87586, 0.87901, 0.88282, 0.88680, 0.89015,\
        0.89155, 0.89011, 0.88652, 0.88148, 0.87544, 0.86869, 0.86145, 0.85391, 0.84621, 0.83849, 0.83091,\
        0.82361, 0.81681, 0.81083, 0.80623, 0.80396, 0.80411, 0.80509, 0.80607, 0.80668, 0.80653, 0.80530,\
        0.80298, 0.79977, 0.79585, 0.79123, 0.78587, 0.77981, 0.77317, 0.76611, 0.75874, 0.75117, 0.74350,\
        0.73578, 0.72809, 0.72049, 0.71303, 0.70577, 0.69877, 0.69208, 0.68568, 0.67949, 0.67345, 0.66756,\
        0.66178, 0.65609, 0.65047, 0.64487, 0.63930, 0.63373, 0.62819, 0.62268, 0.61723, 0.61189, 0.60665,\
        0.60150, 0.59635, 0.59105, 0.58572, 0.58048, 0.57534, 0.57030, 0.56531, 0.56033, 0.55533, 0.55028],\
        [0.67742, 0.68379, 0.68996, 0.69597, 0.70196, 0.70805, 0.71416, 0.72025, 0.72635, 0.73251, 0.73876,\
        0.74510, 0.75151, 0.75792, 0.76427, 0.77046, 0.77636, 0.78183, 0.78672, 0.79085, 0.79416, 0.79682,\
        0.79937, 0.80217, 0.80550, 0.80940, 0.81374, 0.81835, 0.82313, 0.82801, 0.83295, 0.83792, 0.84289,\
        0.84785, 0.85276, 0.85756, 0.86215, 0.86638, 0.87003, 0.87282, 0.87444, 0.87471, 0.87377, 0.87201,\
        0.86994, 0.86795, 0.86638, 0.86549, 0.86551, 0.86658, 0.86870, 0.87172, 0.87527, 0.87884, 0.88167,\
        0.88287, 0.88195, 0.87909, 0.87472, 0.86924, 0.86297, 0.85612, 0.84888, 0.84143, 0.83392, 0.82650,\
        0.81934, 0.81262, 0.80664, 0.80190, 0.79976, 0.80082, 0.80212, 0.80328, 0.80422, 0.80430, 0.80300,\
        0.80035, 0.79684, 0.79262, 0.78774, 0.78224, 0.77616, 0.76957, 0.76259, 0.75532, 0.74786, 0.74029,\
        0.73268, 0.72511, 0.71764, 0.71033, 0.70324, 0.69638, 0.68979, 0.68344, 0.67729, 0.67129, 0.66540,\
        0.65961, 0.65389, 0.64820, 0.64254, 0.63689, 0.63125, 0.62563, 0.62004, 0.61453, 0.60914, 0.60389,\
        0.59873, 0.59362, 0.58847, 0.58329, 0.57815, 0.57309, 0.56810, 0.56315, 0.55822, 0.55327, 0.54827],\
        [0.67639, 0.68284, 0.68903, 0.69494, 0.70069, 0.70657, 0.71241, 0.71823, 0.72413, 0.73020, 0.73646,\
        0.74289, 0.74945, 0.75607, 0.76267, 0.76916, 0.77542, 0.78133, 0.78672, 0.79145, 0.79530, 0.79822,\
        0.80079, 0.80346, 0.80636, 0.80966, 0.81336, 0.81735, 0.82156, 0.82591, 0.83035, 0.83483, 0.83933,\
        0.84380, 0.84819, 0.85242, 0.85638, 0.85992, 0.86285, 0.86495, 0.86603, 0.86602, 0.86505, 0.86340,\
        0.86146, 0.85958, 0.85809, 0.85726, 0.85731, 0.85836, 0.86037, 0.86319, 0.86647, 0.86970, 0.87224,\
        0.87348, 0.87303, 0.87090, 0.86732, 0.86258, 0.85695, 0.85066, 0.84392, 0.83691, 0.82980, 0.82276,\
        0.81599, 0.80971, 0.80421, 0.80008, 0.79828, 0.79877, 0.79976, 0.80077, 0.80197, 0.80219, 0.80074,\
        0.79765, 0.79372, 0.78917, 0.78406, 0.77843, 0.77231, 0.76577, 0.75888, 0.75173, 0.74439, 0.73696,\
        0.72950, 0.72207, 0.71476, 0.70761, 0.70068, 0.69400, 0.68754, 0.68127, 0.67518, 0.66921, 0.66334,\
        0.65753, 0.65178, 0.64605, 0.64032, 0.63460, 0.62887, 0.62315, 0.61747, 0.61185, 0.60637, 0.60107,\
        0.59591, 0.59083, 0.58576, 0.58070, 0.57567, 0.57069, 0.56577, 0.56089, 0.55602, 0.55113, 0.54620],\
        [0.67534, 0.68195, 0.68831, 0.69430, 0.69987, 0.70540, 0.71074, 0.71612, 0.72175, 0.72770, 0.73395,\
        0.74046, 0.74717, 0.75400, 0.76087, 0.76767, 0.77431, 0.78067, 0.78663, 0.79203, 0.79664, 0.79991,\
        0.80234, 0.80466, 0.80711, 0.80989, 0.81303, 0.81647, 0.82013, 0.82394, 0.82785, 0.83183, 0.83582,\
        0.83977, 0.84363, 0.84729, 0.85063, 0.85351, 0.85577, 0.85725, 0.85784, 0.85752, 0.85639, 0.85469,\
        0.85273, 0.85083, 0.84930, 0.84841, 0.84837, 0.84928, 0.85115, 0.85379, 0.85688, 0.85994, 0.86243,\
        0.86385, 0.86388, 0.86245, 0.85967, 0.85571, 0.85083, 0.84522, 0.83909, 0.83263, 0.82606, 0.81956,\
        0.81336, 0.80774, 0.80300, 0.79950, 0.79759, 0.79722, 0.79764, 0.79844, 0.79941, 0.79928, 0.79755,\
        0.79424, 0.79006, 0.78530, 0.78005, 0.77435, 0.76823, 0.76176, 0.75498, 0.74797, 0.74080, 0.73353,\
        0.72624, 0.71899, 0.71185, 0.70488, 0.69815, 0.69164, 0.68532, 0.67916, 0.67313, 0.66720, 0.66135,\
        0.65554, 0.64976, 0.64399, 0.63822, 0.63242, 0.62661, 0.62079, 0.61499, 0.60925, 0.60362, 0.59825,\
        0.59311, 0.58804, 0.58302, 0.57802, 0.57307, 0.56817, 0.56333, 0.55852, 0.55372, 0.54891, 0.54405],\
        [0.67405, 0.68082, 0.68746, 0.69384, 0.69958, 0.70438, 0.70882, 0.71366, 0.71902, 0.72486, 0.73113,\
        0.73773, 0.74460, 0.75163, 0.75873, 0.76582, 0.77279, 0.77954, 0.78595, 0.79190, 0.79714, 0.80099,\
        0.80329, 0.80527, 0.80735, 0.80973, 0.81245, 0.81544, 0.81861, 0.82192, 0.82532, 0.82879, 0.83228,\
        0.83573, 0.83906, 0.84218, 0.84495, 0.84724, 0.84892, 0.84985, 0.84997, 0.84930, 0.84793, 0.84606,\
        0.84396, 0.84192, 0.84023, 0.83916, 0.83892, 0.83965, 0.84136, 0.84389, 0.84692, 0.85000, 0.85261,\
        0.85434, 0.85488, 0.85411, 0.85208, 0.84890, 0.84478, 0.83988, 0.83442, 0.82860, 0.82261, 0.81671,\
        0.81113, 0.80614, 0.80198, 0.79885, 0.79673, 0.79571, 0.79548, 0.79568, 0.79581, 0.79510, 0.79310,\
        0.78984, 0.78568, 0.78090, 0.77563, 0.76994, 0.76390, 0.75753, 0.75090, 0.74407, 0.73709, 0.73003,\
        0.72293, 0.71588, 0.70894, 0.70218, 0.69565, 0.68933, 0.68316, 0.67710, 0.67114, 0.66526, 0.65942,\
        0.65362, 0.64783, 0.64203, 0.63621, 0.63035, 0.62446, 0.61855, 0.61265, 0.60681, 0.60111, 0.59562,\
        0.59039, 0.58528, 0.58025, 0.57528, 0.57038, 0.56554, 0.56076, 0.55603, 0.55132, 0.54659, 0.54181],\
        [0.67227, 0.67909, 0.68582, 0.69239, 0.69835, 0.70215, 0.70596, 0.71052, 0.71576, 0.72160, 0.72792,\
        0.73463, 0.74165, 0.74886, 0.75618, 0.76349, 0.77070, 0.77770, 0.78437, 0.79055, 0.79593, 0.80005,\
        0.80284, 0.80479, 0.80669, 0.80891, 0.81140, 0.81407, 0.81685, 0.81970, 0.82262, 0.82561, 0.82863,\
        0.83161, 0.83448, 0.83712, 0.83939, 0.84118, 0.84235, 0.84281, 0.84251, 0.84147, 0.83981, 0.83770,\
        0.83538, 0.83310, 0.83115, 0.82980, 0.82930, 0.82980, 0.83136, 0.83387, 0.83699, 0.84026, 0.84315,\
        0.84526, 0.84630, 0.84614, 0.84480, 0.84235, 0.83895, 0.83476, 0.82997, 0.82477, 0.81938, 0.81402,\
        0.80898, 0.80452, 0.80074, 0.79762, 0.79530, 0.79377, 0.79286, 0.79227, 0.79156, 0.79024, 0.78798,\
        0.78473, 0.78066, 0.77596, 0.77079, 0.76521, 0.75929, 0.75309, 0.74665, 0.74004, 0.73329, 0.72647,\
        0.71961, 0.71278, 0.70606, 0.69955, 0.69324, 0.68710, 0.68106, 0.67510, 0.66920, 0.66336, 0.65755,\
        0.65175, 0.64596, 0.64015, 0.63429, 0.62839, 0.62243, 0.61643, 0.61042, 0.60450, 0.59874, 0.59315,\
        0.58778, 0.58256, 0.57747, 0.57248, 0.56759, 0.56280, 0.55808, 0.55343, 0.54881, 0.54418, 0.53950],\
        [0.66988, 0.67652, 0.68299, 0.68906, 0.69421, 0.69812, 0.70208, 0.70667, 0.71199, 0.71790, 0.72430,\
        0.73114, 0.73829, 0.74567, 0.75314, 0.76061, 0.76795, 0.77507, 0.78182, 0.78803, 0.79347, 0.79789,\
        0.80103, 0.80296, 0.80495, 0.80730, 0.80978, 0.81225, 0.81470, 0.81715, 0.81964, 0.82219, 0.82479,\
        0.82738, 0.82986, 0.83211, 0.83399, 0.83537, 0.83613, 0.83619, 0.83552, 0.83414, 0.83217, 0.82978,\
        0.82719, 0.82463, 0.82237, 0.82069, 0.81988, 0.82013, 0.82156, 0.82413, 0.82751, 0.83111, 0.83436,\
        0.83687, 0.83837, 0.83875, 0.83800, 0.83619, 0.83345, 0.82991, 0.82576, 0.82116, 0.81628, 0.81137,\
        0.80669, 0.80249, 0.79911, 0.79591, 0.79327, 0.79123, 0.78966, 0.78832, 0.78688, 0.78500, 0.78244,\
        0.77914, 0.77515, 0.77059, 0.76557, 0.76016, 0.75444, 0.74844, 0.74224, 0.73588, 0.72941, 0.72289,\
        0.71632, 0.70973, 0.70325, 0.69701, 0.69093, 0.68494, 0.67901, 0.67313, 0.66729, 0.66148, 0.65569,\
        0.64991, 0.64412, 0.63831, 0.63244, 0.62652, 0.62051, 0.61443, 0.60831, 0.60224, 0.59637, 0.59070,\
        0.58518, 0.57984, 0.57465, 0.56960, 0.56470, 0.55993, 0.55528, 0.55071, 0.54618, 0.54166, 0.53710],\
        [0.66687, 0.67319, 0.67923, 0.68477, 0.68957, 0.69370, 0.69776, 0.70242, 0.70786, 0.71384, 0.72032,\
        0.72726, 0.73455, 0.74205, 0.74963, 0.75717, 0.76456, 0.77167, 0.77839, 0.78454, 0.78994, 0.79439,\
        0.79765, 0.79984, 0.80226, 0.80500, 0.80756, 0.80991, 0.81208, 0.81416, 0.81626, 0.81845, 0.82072,\
        0.82300, 0.82520, 0.82716, 0.82875, 0.82984, 0.83030, 0.83005, 0.82907, 0.82739, 0.82513, 0.82246,\
        0.81962, 0.81680, 0.81423, 0.81222, 0.81112, 0.81111, 0.81239, 0.81513, 0.81893, 0.82295, 0.82656,\
        0.82941, 0.83128, 0.83208, 0.83180, 0.83051, 0.82833, 0.82537, 0.82180, 0.81774, 0.81332, 0.80874,\
        0.80431, 0.80023, 0.79665, 0.79347, 0.79059, 0.78808, 0.78590, 0.78389, 0.78182, 0.77945, 0.77662,\
        0.77323, 0.76931, 0.76489, 0.76006, 0.75487, 0.74937, 0.74362, 0.73768, 0.73160, 0.72545, 0.71928,\
        0.71312, 0.70688, 0.70056, 0.69462, 0.68873, 0.68285, 0.67699, 0.67117, 0.66537, 0.65959, 0.65383,\
        0.64806, 0.64228, 0.63648, 0.63062, 0.62469, 0.61867, 0.61254, 0.60634, 0.60014, 0.59410, 0.58825,\
        0.58256, 0.57705, 0.57173, 0.56661, 0.56167, 0.55692, 0.55232, 0.54784, 0.54344, 0.53904, 0.53461],\
        [0.66337, 0.66932, 0.67496, 0.68014, 0.68481, 0.68910, 0.69339, 0.69817, 0.70356, 0.70950, 0.71600,\
        0.72304, 0.73045, 0.73805, 0.74568, 0.75323, 0.76057, 0.76761, 0.77420, 0.78023, 0.78553, 0.78997,\
        0.79346, 0.79628, 0.79919, 0.80212, 0.80473, 0.80699, 0.80888, 0.81062, 0.81241, 0.81431, 0.81634,\
        0.81843, 0.82046, 0.82226, 0.82368, 0.82459, 0.82486, 0.82441, 0.82320, 0.82127, 0.81876, 0.81586,\
        0.81283, 0.80990, 0.80713, 0.80481, 0.80362, 0.80342, 0.80444, 0.80746, 0.81182, 0.81618, 0.82000,\
        0.82303, 0.82512, 0.82620, 0.82625, 0.82535, 0.82359, 0.82112, 0.81806, 0.81452, 0.81054, 0.80616,\
        0.80179, 0.79784, 0.79409, 0.79060, 0.78736, 0.78438, 0.78164, 0.77904, 0.77643, 0.77364, 0.77055,\
        0.76709, 0.76321, 0.75895, 0.75432, 0.74937, 0.74414, 0.73866, 0.73299, 0.72721, 0.72138, 0.71559,\
        0.70992, 0.70438, 0.69851, 0.69254, 0.68664, 0.68078, 0.67496, 0.66917, 0.66340, 0.65765, 0.65191,\
        0.64616, 0.64040, 0.63460, 0.62877, 0.62286, 0.61685, 0.61070, 0.60443, 0.59812, 0.59187, 0.58577,\
        0.57984, 0.57413, 0.56866, 0.56344, 0.55847, 0.55374, 0.54920, 0.54483, 0.54055, 0.53631, 0.53202],\
        [0.65951, 0.66513, 0.67044, 0.67539, 0.68001, 0.68442, 0.68890, 0.69373, 0.69905, 0.70490, 0.71140,\
        0.71856, 0.72608, 0.73373, 0.74136, 0.74884, 0.75608, 0.76297, 0.76940, 0.77529, 0.78050, 0.78499,\
        0.78878, 0.79212, 0.79532, 0.79835, 0.80110, 0.80337, 0.80498, 0.80643, 0.80801, 0.80973, 0.81162,\
        0.81365, 0.81565, 0.81739, 0.81876, 0.81961, 0.81982, 0.81927, 0.81792, 0.81583, 0.81314, 0.81004,\
        0.80690, 0.80409, 0.80149, 0.79903, 0.79796, 0.79785, 0.79880, 0.80222, 0.80679, 0.81104, 0.81477,\
        0.81778, 0.81992, 0.82111, 0.82134, 0.82067, 0.81920, 0.81707, 0.81443, 0.81142, 0.80799, 0.80393,\
        0.79937, 0.79528, 0.79127, 0.78738, 0.78367, 0.78019, 0.77691, 0.77379, 0.77071, 0.76758, 0.76428,\
        0.76075, 0.75694, 0.75283, 0.74843, 0.74374, 0.73879, 0.73360, 0.72822, 0.72271, 0.71718, 0.71171,\
        0.70636, 0.70143, 0.69624, 0.69030, 0.68442, 0.67860, 0.67282, 0.66708, 0.66135, 0.65563, 0.64990,\
        0.64416, 0.63841, 0.63263, 0.62681, 0.62094, 0.61497, 0.60881, 0.60246, 0.59599, 0.58951, 0.58313,\
        0.57695, 0.57102, 0.56539, 0.56007, 0.55506, 0.55035, 0.54589, 0.54164, 0.53753, 0.53346, 0.52935],\
        [0.65542, 0.66073, 0.66579, 0.67058, 0.67516, 0.67965, 0.68423, 0.68909, 0.69434, 0.70009, 0.70662,\
        0.71396, 0.72154, 0.72918, 0.73673, 0.74408, 0.75114, 0.75784, 0.76409, 0.76982, 0.77496, 0.77950,\
        0.78350, 0.78710, 0.79045, 0.79358, 0.79642, 0.79872, 0.80016, 0.80154, 0.80310, 0.80473, 0.80656,\
        0.80869, 0.81077, 0.81253, 0.81392, 0.81484, 0.81511, 0.81458, 0.81319, 0.81103, 0.80828, 0.80507,\
        0.80182, 0.79925, 0.79704, 0.79515, 0.79427, 0.79461, 0.79639, 0.79993, 0.80366, 0.80734, 0.81074,\
        0.81357, 0.81561, 0.81676, 0.81700, 0.81640, 0.81506, 0.81311, 0.81073, 0.80815, 0.80541, 0.80207,\
        0.79757, 0.79277, 0.78816, 0.78374, 0.77952, 0.77553, 0.77175, 0.76816, 0.76469, 0.76126, 0.75780,\
        0.75424, 0.75051, 0.74658, 0.74242, 0.73802, 0.73338, 0.72850, 0.72340, 0.71816, 0.71289, 0.70774,\
        0.70264, 0.69762, 0.69283, 0.68747, 0.68186, 0.67620, 0.67052, 0.66485, 0.65917, 0.65348, 0.64777,\
        0.64203, 0.63627, 0.63048, 0.62466, 0.61882, 0.61292, 0.60680, 0.60032, 0.59364, 0.58689, 0.58023,\
        0.57378, 0.56764, 0.56184, 0.55642, 0.55138, 0.54671, 0.54237, 0.53827, 0.53435, 0.53049, 0.52660],\
        [0.65118, 0.65622, 0.66105, 0.66571, 0.67025, 0.67479, 0.67945, 0.68437, 0.68965, 0.69542, 0.70212,\
        0.70944, 0.71692, 0.72443, 0.73183, 0.73898, 0.74582, 0.75229, 0.75833, 0.76390, 0.76895, 0.77349,\
        0.77756, 0.78127, 0.78468, 0.78782, 0.79061, 0.79287, 0.79458, 0.79611, 0.79773, 0.79940, 0.80118,\
        0.80366, 0.80583, 0.80758, 0.80907, 0.81017, 0.81066, 0.81026, 0.80888, 0.80671, 0.80409, 0.80102,\
        0.79787, 0.79539, 0.79350, 0.79219, 0.79179, 0.79259, 0.79469, 0.79784, 0.80117, 0.80448, 0.80759,\
        0.81021, 0.81205, 0.81301, 0.81311, 0.81245, 0.81110, 0.80915, 0.80677, 0.80428, 0.80200, 0.79916,\
        0.79466, 0.78955, 0.78448, 0.77957, 0.77487, 0.77040, 0.76617, 0.76217, 0.75836, 0.75471, 0.75113,\
        0.74758, 0.74397, 0.74024, 0.73635, 0.73227, 0.72796, 0.72341, 0.71862, 0.71363, 0.70854, 0.70361,\
        0.69918, 0.69427, 0.68940, 0.68433, 0.67901, 0.67356, 0.66803, 0.66245, 0.65684, 0.65118, 0.64548,\
        0.63974, 0.63395, 0.62811, 0.62224, 0.61636, 0.61052, 0.60457, 0.59798, 0.59098, 0.58390, 0.57695,\
        0.57026, 0.56391, 0.55797, 0.55246, 0.54742, 0.54281, 0.53860, 0.53470, 0.53102, 0.52742, 0.52377],\
        [0.64685, 0.65163, 0.65626, 0.66079, 0.66528, 0.66983, 0.67456, 0.67959, 0.68503, 0.69107, 0.69782,\
        0.70483, 0.71212, 0.71949, 0.72668, 0.73356, 0.74012, 0.74633, 0.75216, 0.75755, 0.76247, 0.76695,\
        0.77102, 0.77473, 0.77814, 0.78125, 0.78402, 0.78637, 0.78836, 0.79017, 0.79197, 0.79385, 0.79598,\
        0.79859, 0.80064, 0.80242, 0.80407, 0.80546, 0.80630, 0.80613, 0.80482, 0.80257, 0.80008, 0.79745,\
        0.79458, 0.79226, 0.79062, 0.78974, 0.78977, 0.79087, 0.79300, 0.79585, 0.79898, 0.80208, 0.80509,\
        0.80754, 0.80907, 0.80968, 0.80951, 0.80868, 0.80723, 0.80519, 0.80265, 0.79985, 0.79747, 0.79448,\
        0.79027, 0.78527, 0.78002, 0.77479, 0.76970, 0.76483, 0.76019, 0.75583, 0.75175, 0.74791, 0.74428,\
        0.74078, 0.73733, 0.73384, 0.73024, 0.72650, 0.72257, 0.71839, 0.71393, 0.70923, 0.70438, 0.69945,\
        0.69512, 0.69067, 0.68597, 0.68108, 0.67599, 0.67073, 0.66536, 0.65989, 0.65434, 0.64873, 0.64304,\
        0.63728, 0.63144, 0.62551, 0.61950, 0.61343, 0.60741, 0.60154, 0.59498, 0.58771, 0.58040, 0.57323,\
        0.56634, 0.55981, 0.55374, 0.54817, 0.54313, 0.53862, 0.53458, 0.53093, 0.52753, 0.52423, 0.52088],\
        [0.64245, 0.64699, 0.65142, 0.65579, 0.66020, 0.66474, 0.66951, 0.67463, 0.68020, 0.68631, 0.69290,\
        0.69981, 0.70707, 0.71432, 0.72124, 0.72778, 0.73401, 0.73996, 0.74557, 0.75078, 0.75556, 0.75993,\
        0.76392, 0.76759, 0.77096, 0.77406, 0.77686, 0.77937, 0.78162, 0.78374, 0.78584, 0.78802, 0.79040,\
        0.79288, 0.79508, 0.79703, 0.79885, 0.80053, 0.80179, 0.80187, 0.80064, 0.79855, 0.79605, 0.79364,\
        0.79125, 0.78926, 0.78793, 0.78740, 0.78776, 0.78907, 0.79124, 0.79402, 0.79704, 0.80006, 0.80319,\
        0.80541, 0.80646, 0.80655, 0.80597, 0.80490, 0.80333, 0.80116, 0.79851, 0.79558, 0.79265, 0.78928,\
        0.78508, 0.78017, 0.77487, 0.76944, 0.76406, 0.75884, 0.75386, 0.74919, 0.74486, 0.74090, 0.73726,\
        0.73387, 0.73062, 0.72740, 0.72411, 0.72073, 0.71722, 0.71346, 0.70935, 0.70495, 0.70045, 0.69587,\
        0.69142, 0.68703, 0.68249, 0.67777, 0.67285, 0.66776, 0.66253, 0.65717, 0.65170, 0.64612, 0.64044,\
        0.63465, 0.62875, 0.62273, 0.61656, 0.61020, 0.60363, 0.59735, 0.59084, 0.58371, 0.57636, 0.56906,\
        0.56201, 0.55534, 0.54915, 0.54353, 0.53851, 0.53412, 0.53030, 0.52694, 0.52389, 0.52097, 0.51795],\
        [0.63803, 0.64231, 0.64652, 0.65072, 0.65501, 0.65949, 0.66425, 0.66939, 0.67499, 0.68110, 0.68765,\
        0.69458, 0.70180, 0.70890, 0.71544, 0.72152, 0.72743, 0.73317, 0.73860, 0.74362, 0.74824, 0.75246,\
        0.75634, 0.75993, 0.76326, 0.76637, 0.76925, 0.77193, 0.77444, 0.77687, 0.77928, 0.78175, 0.78429,\
        0.78682, 0.78919, 0.79137, 0.79338, 0.79520, 0.79655, 0.79679, 0.79576, 0.79400, 0.79183, 0.78966,\
        0.78766, 0.78605, 0.78505, 0.78482, 0.78544, 0.78693, 0.78921, 0.79210, 0.79534, 0.79874, 0.80180,\
        0.80357, 0.80387, 0.80330, 0.80225, 0.80088, 0.79917, 0.79686, 0.79407, 0.79099, 0.78765, 0.78391,\
        0.77956, 0.77462, 0.76926, 0.76368, 0.75807, 0.75255, 0.74726, 0.74230, 0.73776, 0.73370, 0.73011,\
        0.72690, 0.72390, 0.72097, 0.71798, 0.71493, 0.71185, 0.70860, 0.70487, 0.70070, 0.69642, 0.69210,\
        0.68777, 0.68343, 0.67899, 0.67440, 0.66963, 0.66469, 0.65958, 0.65432, 0.64891, 0.64337, 0.63769,\
        0.63188, 0.62591, 0.61979, 0.61351, 0.60705, 0.60029, 0.59341, 0.58654, 0.57934, 0.57193, 0.56453,\
        0.55735, 0.55055, 0.54425, 0.53857, 0.53359, 0.52933, 0.52576, 0.52276, 0.52013, 0.51764, 0.51502],\
        [0.63361, 0.63761, 0.64157, 0.64556, 0.64968, 0.65403, 0.65871, 0.66379, 0.66935, 0.67541, 0.68194,\
        0.68887, 0.69606, 0.70307, 0.70901, 0.71458, 0.72029, 0.72595, 0.73126, 0.73611, 0.74053, 0.74459,\
        0.74834, 0.75184, 0.75514, 0.75826, 0.76125, 0.76412, 0.76689, 0.76961, 0.77232, 0.77505, 0.77778,\
        0.78046, 0.78299, 0.78535, 0.78748, 0.78929, 0.79053, 0.79087, 0.79022, 0.78886, 0.78713, 0.78532,\
        0.78368, 0.78243, 0.78174, 0.78177, 0.78259, 0.78422, 0.78662, 0.78965, 0.79312, 0.79677, 0.80000,\
        0.80133, 0.80075, 0.79963, 0.79820, 0.79655, 0.79459, 0.79216, 0.78928, 0.78604, 0.78245, 0.77843,\
        0.77390, 0.76886, 0.76341, 0.75769, 0.75187, 0.74609, 0.74050, 0.73527, 0.73052, 0.72638, 0.72287,\
        0.71989, 0.71722, 0.71461, 0.71188, 0.70904, 0.70635, 0.70373, 0.70039, 0.69644, 0.69235, 0.68820,\
        0.68402, 0.67978, 0.67545, 0.67098, 0.66634, 0.66152, 0.65652, 0.65135, 0.64600, 0.64049, 0.63482,\
        0.62897, 0.62293, 0.61671, 0.61030, 0.60374, 0.59691, 0.58966, 0.58233, 0.57487, 0.56731, 0.55979,\
        0.55247, 0.54552, 0.53910, 0.53335, 0.52838, 0.52427, 0.52099, 0.51840, 0.51627, 0.51430, 0.51213],\
        [0.62922, 0.63292, 0.63659, 0.64031, 0.64420, 0.64836, 0.65287, 0.65782, 0.66325, 0.66918, 0.67557,\
        0.68235, 0.68932, 0.69598, 0.70145, 0.70678, 0.71265, 0.71842, 0.72362, 0.72826, 0.73247, 0.73636,\
        0.73998, 0.74339, 0.74665, 0.74982, 0.75293, 0.75599, 0.75902, 0.76203, 0.76502, 0.76800, 0.77094,\
        0.77379, 0.77649, 0.77897, 0.78115, 0.78294, 0.78414, 0.78458, 0.78424, 0.78328, 0.78196, 0.78054,\
        0.77924, 0.77829, 0.77786, 0.77807, 0.77902, 0.78073, 0.78316, 0.78620, 0.78965, 0.79321, 0.79624,\
        0.79729, 0.79656, 0.79527, 0.79366, 0.79181, 0.78965, 0.78711, 0.78416, 0.78081, 0.77706, 0.77286,\
        0.76819, 0.76305, 0.75750, 0.75166, 0.74564, 0.73962, 0.73375, 0.72823, 0.72325, 0.71901, 0.71562,\
        0.71293, 0.71065, 0.70842, 0.70594, 0.70315, 0.70057, 0.69851, 0.69553, 0.69191, 0.68807, 0.68413,\
        0.68012, 0.67603, 0.67183, 0.66749, 0.66298, 0.65828, 0.65338, 0.64828, 0.64299, 0.63750, 0.63182,\
        0.62594, 0.61984, 0.61349, 0.60690, 0.60007, 0.59303, 0.58567, 0.57809, 0.57040, 0.56266, 0.55497,\
        0.54749, 0.54038, 0.53381, 0.52795, 0.52298, 0.51901, 0.51603, 0.51390, 0.51235, 0.51098, 0.50933],\
        [0.62491, 0.62827, 0.63161, 0.63501, 0.63860, 0.64249, 0.64675, 0.65148, 0.65669, 0.66238, 0.66851,\
        0.67495, 0.68146, 0.68762, 0.69314, 0.69859, 0.70486, 0.71073, 0.71568, 0.72004, 0.72405, 0.72779,\
        0.73130, 0.73463, 0.73786, 0.74109, 0.74434, 0.74762, 0.75091, 0.75420, 0.75747, 0.76071, 0.76386,\
        0.76690, 0.76974, 0.77232, 0.77455, 0.77633, 0.77755, 0.77811, 0.77801, 0.77738, 0.77641, 0.77531,\
        0.77431, 0.77359, 0.77333, 0.77366, 0.77465, 0.77633, 0.77867, 0.78155, 0.78474, 0.78787, 0.79031,\
        0.79136, 0.79105, 0.78995, 0.78840, 0.78653, 0.78432, 0.78174, 0.77875, 0.77536, 0.77153, 0.76725,\
        0.76251, 0.75731, 0.75169, 0.74574, 0.73958, 0.73334, 0.72721, 0.72140, 0.71615, 0.71174, 0.70845,\
        0.70613, 0.70420, 0.70239, 0.70018, 0.69757, 0.69507, 0.69286, 0.69021, 0.68702, 0.68352, 0.67984,\
        0.67605, 0.67215, 0.66811, 0.66392, 0.65954, 0.65496, 0.65016, 0.64514, 0.63989, 0.63441, 0.62873,\
        0.62283, 0.61668, 0.61023, 0.60345, 0.59642, 0.58914, 0.58160, 0.57384, 0.56596, 0.55805, 0.55021,\
        0.54257, 0.53529, 0.52854, 0.52254, 0.51751, 0.51364, 0.51097, 0.50933, 0.50843, 0.50776, 0.50667],\
        [0.62072, 0.62371, 0.62667, 0.62969, 0.63292, 0.63646, 0.64041, 0.64483, 0.64974, 0.65513, 0.66091,\
        0.66696, 0.67307, 0.67902, 0.68477, 0.69067, 0.69698, 0.70272, 0.70724, 0.71137, 0.71527, 0.71895,\
        0.72238, 0.72564, 0.72885, 0.73216, 0.73559, 0.73911, 0.74267, 0.74623, 0.74977, 0.75325, 0.75663,\
        0.75985, 0.76284, 0.76552, 0.76781, 0.76962, 0.77088, 0.77155, 0.77164, 0.77126, 0.77055, 0.76971,\
        0.76891, 0.76835, 0.76817, 0.76851, 0.76945, 0.77101, 0.77316, 0.77576, 0.77858, 0.78126, 0.78335,\
        0.78443, 0.78448, 0.78373, 0.78243, 0.78069, 0.77857, 0.77605, 0.77311, 0.76974, 0.76592, 0.76165,\
        0.75691, 0.75171, 0.74608, 0.74009, 0.73383, 0.72744, 0.72109, 0.71503, 0.70954, 0.70489, 0.70160,\
        0.69967, 0.69794, 0.69627, 0.69430, 0.69199, 0.68964, 0.68734, 0.68481, 0.68193, 0.67875, 0.67537,\
        0.67182, 0.66813, 0.66428, 0.66026, 0.65603, 0.65158, 0.64688, 0.64193, 0.63671, 0.63124, 0.62553,\
        0.61961, 0.61346, 0.60699, 0.60006, 0.59281, 0.58531, 0.57757, 0.56965, 0.56163, 0.55359, 0.54563,\
        0.53786, 0.53042, 0.52349, 0.51731, 0.51215, 0.50833, 0.50600, 0.50483, 0.50456, 0.50467, 0.50418],\
        [0.61671, 0.61931, 0.62184, 0.62443, 0.62722, 0.63035, 0.63392, 0.63798, 0.64254, 0.64757, 0.65298,\
        0.65865, 0.66445, 0.67027, 0.67612, 0.68210, 0.68813, 0.69356, 0.69812, 0.70225, 0.70619, 0.70992,\
        0.71334, 0.71651, 0.71970, 0.72313, 0.72678, 0.73056, 0.73440, 0.73823, 0.74202, 0.74574, 0.74934,\
        0.75275, 0.75589, 0.75868, 0.76104, 0.76291, 0.76423, 0.76499, 0.76522, 0.76501, 0.76448, 0.76379,\
        0.76311, 0.76261, 0.76243, 0.76269, 0.76349, 0.76485, 0.76674, 0.76902, 0.77149, 0.77384, 0.77573,\
        0.77688, 0.77723, 0.77685, 0.77586, 0.77438, 0.77245, 0.77008, 0.76727, 0.76401, 0.76028, 0.75609,\
        0.75143, 0.74631, 0.74076, 0.73482, 0.72856, 0.72210, 0.71560, 0.70939, 0.70383, 0.69914, 0.69583,\
        0.69374, 0.69202, 0.69031, 0.68844, 0.68634, 0.68414, 0.68187, 0.67944, 0.67677, 0.67386, 0.67075,\
        0.66746, 0.66400, 0.66036, 0.65652, 0.65246, 0.64815, 0.64357, 0.63868, 0.63350, 0.62802, 0.62225,\
        0.61625, 0.61009, 0.60374, 0.59674, 0.58925, 0.58153, 0.57362, 0.56558, 0.55746, 0.54935, 0.54132,\
        0.53349, 0.52596, 0.51890, 0.51255, 0.50725, 0.50338, 0.50151, 0.50069, 0.50092, 0.50166, 0.50177],\
        [0.61295, 0.61514, 0.61720, 0.61930, 0.62161, 0.62427, 0.62740, 0.63107, 0.63525, 0.63989, 0.64492,\
        0.65023, 0.65573, 0.66137, 0.66710, 0.67291, 0.67862, 0.68389, 0.68859, 0.69289, 0.69695, 0.70082,\
        0.70427, 0.70733, 0.71053, 0.71414, 0.71807, 0.72215, 0.72624, 0.73031, 0.73434, 0.73829, 0.74211,\
        0.74572, 0.74901, 0.75192, 0.75436, 0.75628, 0.75765, 0.75848, 0.75880, 0.75870, 0.75827, 0.75766,\
        0.75701, 0.75647, 0.75619, 0.75630, 0.75688, 0.75798, 0.75957, 0.76156, 0.76375, 0.76588, 0.76769,\
        0.76895, 0.76957, 0.76953, 0.76890, 0.76772, 0.76605, 0.76391, 0.76129, 0.75820, 0.75464, 0.75060,\
        0.74610, 0.74115, 0.73576, 0.72998, 0.72385, 0.71746, 0.71094, 0.70460, 0.69915, 0.69465, 0.69120,\
        0.68867, 0.68663, 0.68474, 0.68280, 0.68076, 0.67864, 0.67642, 0.67409, 0.67158, 0.66889, 0.66602,\
        0.66298, 0.65976, 0.65634, 0.65272, 0.64884, 0.64469, 0.64023, 0.63544, 0.63029, 0.62479, 0.61893,\
        0.61277, 0.60639, 0.59995, 0.59303, 0.58553, 0.57773, 0.56974, 0.56163, 0.55348, 0.54536, 0.53735,\
        0.52956, 0.52207, 0.51501, 0.50859, 0.50326, 0.49958, 0.49786, 0.49740, 0.49774, 0.49847, 0.49870],\
        [0.60954, 0.61130, 0.61287, 0.61444, 0.61619, 0.61833, 0.62100, 0.62425, 0.62804, 0.63230, 0.63693,\
        0.64186, 0.64704, 0.65240, 0.65791, 0.66348, 0.66897, 0.67417, 0.67899, 0.68347, 0.68769, 0.69165,\
        0.69517, 0.69830, 0.70160, 0.70547, 0.70970, 0.71404, 0.71835, 0.72260, 0.72681, 0.73098, 0.73503,\
        0.73885, 0.74231, 0.74533, 0.74784, 0.74982, 0.75123, 0.75210, 0.75247, 0.75241, 0.75201, 0.75140,\
        0.75070, 0.75005, 0.74959, 0.74946, 0.74975, 0.75055, 0.75185, 0.75358, 0.75557, 0.75759, 0.75941,\
        0.76081, 0.76169, 0.76198, 0.76169, 0.76085, 0.75948, 0.75759, 0.75522, 0.75235, 0.74901, 0.74519,\
        0.74091, 0.73620, 0.73107, 0.72556, 0.71970, 0.71355, 0.70725, 0.70106, 0.69550, 0.69096, 0.68724,\
        0.68426, 0.68177, 0.67955, 0.67743, 0.67532, 0.67318, 0.67100, 0.66874, 0.66638, 0.66387, 0.66122,\
        0.65841, 0.65543, 0.65225, 0.64885, 0.64518, 0.64121, 0.63691, 0.63222, 0.62714, 0.62163, 0.61570,\
        0.60938, 0.60271, 0.59601, 0.58912, 0.58172, 0.57394, 0.56594, 0.55782, 0.54969, 0.54163, 0.53374,\
        0.52610, 0.51882, 0.51199, 0.50575, 0.50044, 0.49712, 0.49534, 0.49465, 0.49462, 0.49474, 0.49413],\
        [0.60656, 0.60790, 0.60899, 0.60998, 0.61112, 0.61269, 0.61489, 0.61774, 0.62115, 0.62501, 0.62922,\
        0.63374, 0.63852, 0.64355, 0.64877, 0.65411, 0.65944, 0.66461, 0.66953, 0.67418, 0.67855, 0.68263,\
        0.68635, 0.68984, 0.69344, 0.69747, 0.70189, 0.70642, 0.71083, 0.71517, 0.71953, 0.72393, 0.72822,\
        0.73225, 0.73587, 0.73900, 0.74157, 0.74358, 0.74501, 0.74589, 0.74627, 0.74621, 0.74580, 0.74513,\
        0.74432, 0.74349, 0.74278, 0.74234, 0.74229, 0.74275, 0.74376, 0.74527, 0.74715, 0.74915, 0.75105,\
        0.75264, 0.75377, 0.75437, 0.75441, 0.75388, 0.75281, 0.75121, 0.74910, 0.74649, 0.74340, 0.73984,\
        0.73583, 0.73141, 0.72661, 0.72147, 0.71600, 0.71020, 0.70422, 0.69830, 0.69270, 0.68782, 0.68369,\
        0.68022, 0.67726, 0.67465, 0.67226, 0.66999, 0.66779, 0.66560, 0.66340, 0.66114, 0.65880, 0.65634,\
        0.65376, 0.65101, 0.64808, 0.64491, 0.64148, 0.63772, 0.63359, 0.62905, 0.62407, 0.61861, 0.61267,\
        0.60629, 0.59956, 0.59264, 0.58552, 0.57805, 0.57026, 0.56225, 0.55415, 0.54607, 0.53812, 0.53040,\
        0.52303, 0.51610, 0.50972, 0.50407, 0.49932, 0.49570, 0.49341, 0.49211, 0.49135, 0.49061, 0.48924],\
        [0.60408, 0.60507, 0.60570, 0.60611, 0.60659, 0.60754, 0.60928, 0.61177, 0.61482, 0.61826, 0.62201,\
        0.62605, 0.63037, 0.63500, 0.63991, 0.64503, 0.65024, 0.65540, 0.66042, 0.66523, 0.66978, 0.67407,\
        0.67810, 0.68199, 0.68596, 0.69020, 0.69471, 0.69929, 0.70373, 0.70808, 0.71257, 0.71722, 0.72178,\
        0.72602, 0.72979, 0.73299, 0.73560, 0.73761, 0.73903, 0.73990, 0.74025, 0.74016, 0.73970, 0.73894,\
        0.73799, 0.73694, 0.73593, 0.73512, 0.73466, 0.73474, 0.73547, 0.73682, 0.73866, 0.74074, 0.74278,\
        0.74457, 0.74595, 0.74683, 0.74716, 0.74693, 0.74615, 0.74483, 0.74298, 0.74064, 0.73781, 0.73453,\
        0.73081, 0.72669, 0.72225, 0.71755, 0.71257, 0.70719, 0.70144, 0.69563, 0.68999, 0.68482, 0.68025,\
        0.67630, 0.67288, 0.66988, 0.66719, 0.66473, 0.66241, 0.66019, 0.65802, 0.65585, 0.65365, 0.65139,\
        0.64902, 0.64651, 0.64382, 0.64091, 0.63773, 0.63421, 0.63030, 0.62594, 0.62109, 0.61573, 0.60985,\
        0.60346, 0.59667, 0.58958, 0.58223, 0.57460, 0.56672, 0.55868, 0.55059, 0.54257, 0.53474, 0.52723,\
        0.52016, 0.51364, 0.50776, 0.50252, 0.49810, 0.49437, 0.49153, 0.48949, 0.48789, 0.48630, 0.48422],\
        [0.60217, 0.60289, 0.60316, 0.60308, 0.60287, 0.60314, 0.60448, 0.60666, 0.60933, 0.61231, 0.61552,\
        0.61899, 0.62278, 0.62695, 0.63151, 0.63642, 0.64153, 0.64671, 0.65183, 0.65679, 0.66155, 0.66609,\
        0.67044, 0.67470, 0.67899, 0.68342, 0.68799, 0.69258, 0.69705, 0.70140, 0.70604, 0.71102, 0.71586,\
        0.72027, 0.72412, 0.72735, 0.72995, 0.73193, 0.73331, 0.73414, 0.73445, 0.73432, 0.73380, 0.73294,\
        0.73184, 0.73056, 0.72923, 0.72801, 0.72709, 0.72673, 0.72716, 0.72842, 0.73033, 0.73255, 0.73477,\
        0.73676, 0.73836, 0.73947, 0.74005, 0.74008, 0.73955, 0.73848, 0.73688, 0.73479, 0.73222, 0.72921,\
        0.72577, 0.72194, 0.71781, 0.71354, 0.70918, 0.70437, 0.69879, 0.69292, 0.68710, 0.68162, 0.67666,\
        0.67227, 0.66844, 0.66507, 0.66210, 0.65943, 0.65699, 0.65472, 0.65257, 0.65049, 0.64842, 0.64633,\
        0.64417, 0.64190, 0.63947, 0.63682, 0.63391, 0.63066, 0.62700, 0.62286, 0.61817, 0.61293, 0.60716,\
        0.60081, 0.59392, 0.58667, 0.57912, 0.57132, 0.56331, 0.55520, 0.54710, 0.53912, 0.53141, 0.52408,\
        0.51726, 0.51109, 0.50573, 0.50106, 0.49653, 0.49257, 0.48927, 0.48656, 0.48420, 0.48183, 0.47911],\
        [0.60082, 0.60138, 0.60146, 0.60111, 0.60040, 0.59997, 0.60097, 0.60276, 0.60497, 0.60739, 0.60996,\
        0.61276, 0.61590, 0.61954, 0.62374, 0.62844, 0.63349, 0.63869, 0.64390, 0.64901, 0.65396, 0.65875,\
        0.66339, 0.66794, 0.67250, 0.67711, 0.68177, 0.68642, 0.69097, 0.69547, 0.70033, 0.70554, 0.71055,\
        0.71503, 0.71888, 0.72207, 0.72462, 0.72653, 0.72785, 0.72861, 0.72888, 0.72871, 0.72814, 0.72721,\
        0.72599, 0.72452, 0.72290, 0.72125, 0.71982, 0.71895, 0.71906, 0.72030, 0.72237, 0.72480, 0.72721,\
        0.72937, 0.73112, 0.73240, 0.73317, 0.73339, 0.73307, 0.73220, 0.73080, 0.72893, 0.72661, 0.72386,\
        0.72067, 0.71706, 0.71314, 0.70910, 0.70535, 0.70155, 0.69598, 0.68983, 0.68373, 0.67796, 0.67267,\
        0.66794, 0.66376, 0.66009, 0.65686, 0.65401, 0.65146, 0.64916, 0.64703, 0.64502, 0.64307, 0.64115,\
        0.63919, 0.63716, 0.63498, 0.63262, 0.63000, 0.62705, 0.62369, 0.61979, 0.61529, 0.61014, 0.60442,\
        0.59814, 0.59118, 0.58379, 0.57608, 0.56811, 0.55998, 0.55177, 0.54362, 0.53567, 0.52804, 0.52085,\
        0.51420, 0.50818, 0.50301, 0.49868, 0.49431, 0.49012, 0.48645, 0.48320, 0.48019, 0.47717, 0.47387],\
        [0.59999, 0.60048, 0.60049, 0.60011, 0.59933, 0.59874, 0.59913, 0.60031, 0.60193, 0.60368, 0.60552,\
        0.60754, 0.60991, 0.61293, 0.61675, 0.62127, 0.62626, 0.63148, 0.63675, 0.64197, 0.64708, 0.65206,\
        0.65693, 0.66173, 0.66650, 0.67128, 0.67607, 0.68086, 0.68562, 0.69045, 0.69553, 0.70080, 0.70584,\
        0.71024, 0.71399, 0.71710, 0.71956, 0.72138, 0.72260, 0.72329, 0.72352, 0.72332, 0.72274, 0.72180,\
        0.72053, 0.71895, 0.71713, 0.71515, 0.71319, 0.71169, 0.71142, 0.71276, 0.71512, 0.71776, 0.72028,\
        0.72250, 0.72431, 0.72566, 0.72653, 0.72690, 0.72673, 0.72600, 0.72474, 0.72303, 0.72091, 0.71842,\
        0.71547, 0.71204, 0.70821, 0.70418, 0.70019, 0.69729, 0.69197, 0.68578, 0.67956, 0.67362, 0.66812,\
        0.66314, 0.65871, 0.65480, 0.65139, 0.64840, 0.64577, 0.64344, 0.64134, 0.63941, 0.63759, 0.63583,\
        0.63407, 0.63226, 0.63034, 0.62825, 0.62594, 0.62332, 0.62030, 0.61672, 0.61243, 0.60735, 0.60156,\
        0.59524, 0.58832, 0.58087, 0.57303, 0.56493, 0.55666, 0.54834, 0.54012, 0.53213, 0.52455, 0.51748,\
        0.51099, 0.50500, 0.49963, 0.49528, 0.49107, 0.48692, 0.48302, 0.47937, 0.47585, 0.47228, 0.46846],\
        [0.59960, 0.60003, 0.60004, 0.59969, 0.59905, 0.59853, 0.59855, 0.59917, 0.60014, 0.60122, 0.60233,\
        0.60351, 0.60500, 0.60729, 0.61073, 0.61510, 0.62002, 0.62521, 0.63049, 0.63575, 0.64095, 0.64605,\
        0.65107, 0.65604, 0.66097, 0.66591, 0.67085, 0.67580, 0.68079, 0.68588, 0.69112, 0.69644, 0.70143,\
        0.70565, 0.70929, 0.71235, 0.71471, 0.71640, 0.71750, 0.71813, 0.71832, 0.71814, 0.71760, 0.71672,\
        0.71549, 0.71393, 0.71207, 0.70996, 0.70768, 0.70549, 0.70465, 0.70635, 0.70899, 0.71165, 0.71409,\
        0.71622, 0.71796, 0.71927, 0.72016, 0.72061, 0.72055, 0.71990, 0.71869, 0.71703, 0.71507, 0.71283,\
        0.71014, 0.70684, 0.70303, 0.69908, 0.69510, 0.69122, 0.68633, 0.68053, 0.67446, 0.66852, 0.66293,\
        0.65780, 0.65321, 0.64915, 0.64561, 0.64254, 0.63987, 0.63754, 0.63548, 0.63363, 0.63194, 0.63034,\
        0.62878, 0.62719, 0.62551, 0.62368, 0.62166, 0.61939, 0.61677, 0.61360, 0.60963, 0.60467, 0.59884,\
        0.59243, 0.58542, 0.57787, 0.56992, 0.56170, 0.55331, 0.54488, 0.53655, 0.52848, 0.52088, 0.51392,\
        0.50764, 0.50197, 0.49660, 0.49182, 0.48743, 0.48319, 0.47908, 0.47510, 0.47116, 0.46713, 0.46287],\
        [0.59957, 0.59995, 0.60002, 0.59977, 0.59929, 0.59884, 0.59869, 0.59891, 0.59939, 0.59995, 0.60048,\
        0.60093, 0.60148, 0.60286, 0.60596, 0.61017, 0.61493, 0.61998, 0.62516, 0.63037, 0.63556, 0.64070,\
        0.64579, 0.65086, 0.65590, 0.66094, 0.66600, 0.67109, 0.67623, 0.68145, 0.68673, 0.69195, 0.69679,\
        0.70099, 0.70463, 0.70772, 0.70998, 0.71149, 0.71248, 0.71305, 0.71324, 0.71309, 0.71265, 0.71191,\
        0.71082, 0.70941, 0.70771, 0.70578, 0.70366, 0.70143, 0.69987, 0.70196, 0.70426, 0.70652, 0.70864,\
        0.71051, 0.71203, 0.71318, 0.71400, 0.71449, 0.71453, 0.71392, 0.71264, 0.71089, 0.70898, 0.70696,\
        0.70458, 0.70139, 0.69752, 0.69347, 0.68931, 0.68492, 0.67996, 0.67439, 0.66852, 0.66267, 0.65709,\
        0.65191, 0.64724, 0.64311, 0.63951, 0.63640, 0.63373, 0.63144, 0.62944, 0.62769, 0.62612, 0.62467,\
        0.62329, 0.62192, 0.62046, 0.61886, 0.61709, 0.61513, 0.61294, 0.61033, 0.60682, 0.60205, 0.59621,\
        0.58962, 0.58243, 0.57476, 0.56670, 0.55838, 0.54990, 0.54137, 0.53292, 0.52472, 0.51699, 0.51004,\
        0.50399, 0.49835, 0.49312, 0.48817, 0.48354, 0.47910, 0.47476, 0.47047, 0.46616, 0.46174, 0.45709],\
        [0.59988, 0.60027, 0.60038, 0.60020, 0.59983, 0.59943, 0.59919, 0.59918, 0.59938, 0.59965, 0.59988,\
        0.59998, 0.60000, 0.60028, 0.60294, 0.60671, 0.61108, 0.61582, 0.62075, 0.62579, 0.63088, 0.63597,\
        0.64105, 0.64614, 0.65122, 0.65632, 0.66144, 0.66660, 0.67180, 0.67703, 0.68224, 0.68730, 0.69202,\
        0.69622, 0.69995, 0.70318, 0.70523, 0.70650, 0.70743, 0.70799, 0.70819, 0.70811, 0.70782, 0.70728,\
        0.70641, 0.70520, 0.70378, 0.70223, 0.70060, 0.69906, 0.69819, 0.69905, 0.70046, 0.70206, 0.70371,\
        0.70522, 0.70642, 0.70731, 0.70797, 0.70847, 0.70865, 0.70809, 0.70661, 0.70460, 0.70260, 0.70070,\
        0.69846, 0.69534, 0.69149, 0.68733, 0.68295, 0.67828, 0.67320, 0.66771, 0.66197, 0.65623, 0.65068,\
        0.64551, 0.64082, 0.63667, 0.63307, 0.62999, 0.62737, 0.62514, 0.62323, 0.62158, 0.62012, 0.61881,\
        0.61761, 0.61644, 0.61519, 0.61377, 0.61216, 0.61041, 0.60859, 0.60663, 0.60384, 0.59921, 0.59326,\
        0.58650, 0.57917, 0.57140, 0.56329, 0.55492, 0.54640, 0.53781, 0.52929, 0.52095, 0.51299, 0.50574,\
        0.49995, 0.49447, 0.48920, 0.48419, 0.47938, 0.47472, 0.47012, 0.46553, 0.46089, 0.45610, 0.45109],\
        [0.60000, 0.60056, 0.60074, 0.60068, 0.60042, 0.60008, 0.59979, 0.59966, 0.59972, 0.59986, 0.59998,\
        0.59999, 0.59991, 0.60011, 0.60167, 0.60460, 0.60835, 0.61261, 0.61718, 0.62195, 0.62683, 0.63179,\
        0.63679, 0.64182, 0.64688, 0.65196, 0.65708, 0.66223, 0.66740, 0.67256, 0.67766, 0.68257, 0.68718,\
        0.69140, 0.69520, 0.69841, 0.70018, 0.70128, 0.70230, 0.70292, 0.70311, 0.70311, 0.70300, 0.70271,\
        0.70207, 0.70104, 0.69989, 0.69866, 0.69737, 0.69624, 0.69565, 0.69591, 0.69672, 0.69780, 0.69900,\
        0.70013, 0.70100, 0.70156, 0.70199, 0.70243, 0.70282, 0.70234, 0.70062, 0.69836, 0.69620, 0.69414,\
        0.69171, 0.68861, 0.68486, 0.68067, 0.67615, 0.67132, 0.66615, 0.66067, 0.65501, 0.64933, 0.64383,\
        0.63867, 0.63399, 0.62988, 0.62635, 0.62334, 0.62081, 0.61867, 0.61687, 0.61532, 0.61397, 0.61277,\
        0.61173, 0.61075, 0.60970, 0.60843, 0.60688, 0.60519, 0.60351, 0.60198, 0.60009, 0.59555, 0.58954,\
        0.58276, 0.57542, 0.56767, 0.55959, 0.55126, 0.54277, 0.53422, 0.52571, 0.51737, 0.50930, 0.50176,\
        0.49557, 0.49029, 0.48507, 0.47997, 0.47499, 0.47009, 0.46522, 0.46034, 0.45536, 0.45024, 0.44490],\
        [0.59876, 0.60003, 0.60064, 0.60087, 0.60082, 0.60057, 0.60025, 0.60004, 0.59999, 0.60000, 0.59993,\
        0.59976, 0.59964, 0.59989, 0.60106, 0.60331, 0.60642, 0.61014, 0.61428, 0.61870, 0.62332, 0.62807,\
        0.63291, 0.63782, 0.64279, 0.64781, 0.65286, 0.65793, 0.66301, 0.66804, 0.67298, 0.67774, 0.68221,\
        0.68631, 0.68994, 0.69284, 0.69475, 0.69602, 0.69708, 0.69774, 0.69801, 0.69811, 0.69812, 0.69800,\
        0.69755, 0.69673, 0.69577, 0.69475, 0.69374, 0.69287, 0.69237, 0.69237, 0.69278, 0.69345, 0.69424,\
        0.69499, 0.69556, 0.69591, 0.69616, 0.69645, 0.69668, 0.69611, 0.69440, 0.69215, 0.68983, 0.68744,\
        0.68475, 0.68155, 0.67781, 0.67360, 0.66902, 0.66410, 0.65888, 0.65340, 0.64777, 0.64213, 0.63666,\
        0.63151, 0.62686, 0.62281, 0.61940, 0.61652, 0.61409, 0.61207, 0.61040, 0.60896, 0.60769, 0.60657,\
        0.60565, 0.60485, 0.60399, 0.60288, 0.60131, 0.59963, 0.59800, 0.59643, 0.59437, 0.59032, 0.58472,\
        0.57819, 0.57105, 0.56346, 0.55552, 0.54734, 0.53899, 0.53056, 0.52216, 0.51397, 0.50617, 0.49876,\
        0.49219, 0.48640, 0.48093, 0.47562, 0.47040, 0.46524, 0.46009, 0.45490, 0.44961, 0.44417, 0.43851],\
        [0.59675, 0.59866, 0.59986, 0.60056, 0.60085, 0.60075, 0.60039, 0.60009, 0.59993, 0.59977, 0.59956,\
        0.59932, 0.59922, 0.59950, 0.60048, 0.60230, 0.60490, 0.60813, 0.61183, 0.61588, 0.62019, 0.62469,\
        0.62933, 0.63408, 0.63890, 0.64378, 0.64871, 0.65365, 0.65858, 0.66345, 0.66821, 0.67277, 0.67705,\
        0.68096, 0.68437, 0.68712, 0.68914, 0.69060, 0.69169, 0.69242, 0.69284, 0.69306, 0.69315, 0.69308,\
        0.69275, 0.69214, 0.69138, 0.69054, 0.68972, 0.68901, 0.68856, 0.68841, 0.68855, 0.68890, 0.68934,\
        0.68977, 0.69008, 0.69026, 0.69035, 0.69038, 0.69019, 0.68937, 0.68776, 0.68561, 0.68320, 0.68058,\
        0.67766, 0.67432, 0.67052, 0.66628, 0.66166, 0.65671, 0.65147, 0.64600, 0.64039, 0.63478, 0.62931,\
        0.62415, 0.61950, 0.61555, 0.61232, 0.60962, 0.60731, 0.60541, 0.60387, 0.60256, 0.60136, 0.60026,\
        0.59943, 0.59875, 0.59798, 0.59695, 0.59551, 0.59387, 0.59221, 0.59038, 0.58788, 0.58408, 0.57895,\
        0.57284, 0.56603, 0.55872, 0.55105, 0.54311, 0.53499, 0.52676, 0.51852, 0.51044, 0.50292, 0.49578,\
        0.48890, 0.48265, 0.47680, 0.47116, 0.46565, 0.46020, 0.45474, 0.44924, 0.44363, 0.43788, 0.43191],\
        [0.59408, 0.59655, 0.59833, 0.59959, 0.60037, 0.60059, 0.60015, 0.59978, 0.59950, 0.59921, 0.59890,\
        0.59865, 0.59858, 0.59888, 0.59975, 0.60130, 0.60352, 0.60634, 0.60964, 0.61334, 0.61733, 0.62155,\
        0.62595, 0.63049, 0.63513, 0.63984, 0.64459, 0.64936, 0.65411, 0.65878, 0.66333, 0.66768, 0.67174,\
        0.67544, 0.67866, 0.68133, 0.68341, 0.68499, 0.68616, 0.68699, 0.68753, 0.68786, 0.68801, 0.68799,\
        0.68775, 0.68730, 0.68670, 0.68602, 0.68534, 0.68474, 0.68430, 0.68405, 0.68400, 0.68410, 0.68426,\
        0.68442, 0.68450, 0.68448, 0.68436, 0.68409, 0.68354, 0.68249, 0.68087, 0.67875, 0.67628, 0.67352,\
        0.67043, 0.66697, 0.66309, 0.65882, 0.65418, 0.64922, 0.64399, 0.63855, 0.63299, 0.62742, 0.62196,\
        0.61678, 0.61209, 0.60820, 0.60525, 0.60277, 0.60057, 0.59877, 0.59734, 0.59611, 0.59500, 0.59400,\
        0.59318, 0.59247, 0.59168, 0.59065, 0.58932, 0.58774, 0.58597, 0.58387, 0.58111, 0.57737, 0.57257,\
        0.56686, 0.56044, 0.55348, 0.54614, 0.53852, 0.53070, 0.52277, 0.51477, 0.50670, 0.49895, 0.49201,\
        0.48520, 0.47870, 0.47251, 0.46654, 0.46070, 0.45493, 0.44916, 0.44335, 0.43743, 0.43136, 0.42510],\
        [0.59076, 0.59370, 0.59601, 0.59779, 0.59910, 0.59989, 0.59969, 0.59910, 0.59865, 0.59825, 0.59790,\
        0.59767, 0.59764, 0.59796, 0.59876, 0.60013, 0.60208, 0.60457, 0.60754, 0.61090, 0.61459, 0.61853,\
        0.62268, 0.62698, 0.63140, 0.63591, 0.64046, 0.64503, 0.64957, 0.65403, 0.65835, 0.66247, 0.66632,\
        0.66982, 0.67289, 0.67548, 0.67758, 0.67922, 0.68048, 0.68140, 0.68204, 0.68246, 0.68268, 0.68272,\
        0.68256, 0.68223, 0.68176, 0.68120, 0.68063, 0.68010, 0.67965, 0.67934, 0.67914, 0.67903, 0.67897,\
        0.67889, 0.67876, 0.67852, 0.67816, 0.67761, 0.67677, 0.67553, 0.67382, 0.67167, 0.66914, 0.66628,\
        0.66308, 0.65952, 0.65559, 0.65129, 0.64665, 0.64171, 0.63652, 0.63114, 0.62566, 0.62016, 0.61476,\
        0.60961, 0.60489, 0.60092, 0.59831, 0.59602, 0.59397, 0.59226, 0.59086, 0.58967, 0.58861, 0.58766,\
        0.58683, 0.58604, 0.58517, 0.58411, 0.58279, 0.58120, 0.57932, 0.57701, 0.57410, 0.57038, 0.56580,\
        0.56042, 0.55436, 0.54777, 0.54078, 0.53351, 0.52604, 0.51847, 0.51087, 0.50328, 0.49550, 0.48815,\
        0.48116, 0.47443, 0.46794, 0.46166, 0.45550, 0.44942, 0.44334, 0.43721, 0.43098, 0.42462, 0.41806]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=50 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax50vt2.5
    zgrid_H_Hmax_vmax50vt2_5=[[0.43371, 0.44029, 0.44653, 0.45244, 0.45801, 0.46324, 0.46813, 0.47271, 0.47697, 0.48096, 0.48470,\
        0.48824, 0.49163, 0.49488, 0.49792, 0.50025, 0.50190, 0.50370, 0.50576, 0.50809, 0.51066, 0.51341,\
        0.51633, 0.51936, 0.52248, 0.52567, 0.52889, 0.53215, 0.53541, 0.53867, 0.54191, 0.54513, 0.54832,\
        0.55147, 0.55459, 0.55769, 0.56077, 0.56384, 0.56693, 0.57004, 0.57320, 0.57640, 0.57960, 0.58274,\
        0.58566, 0.58821, 0.59045, 0.59252, 0.59448, 0.59636, 0.59809, 0.59980, 0.60149, 0.60309, 0.60455,\
        0.60585, 0.60697, 0.60789, 0.60861, 0.60911, 0.60939, 0.60945, 0.60930, 0.60894, 0.60839, 0.60765,\
        0.60674, 0.60569, 0.60450, 0.60321, 0.60182, 0.60039, 0.59896, 0.59771, 0.59665, 0.59537, 0.59391,\
        0.59210, 0.59022, 0.58845, 0.58618, 0.58289, 0.57870, 0.57410, 0.56949, 0.56509, 0.56100, 0.55727,\
        0.55386, 0.55073, 0.54782, 0.54504, 0.54235, 0.53968, 0.53698, 0.53420, 0.53131, 0.52828, 0.52511,\
        0.52179, 0.51833, 0.51475, 0.51106, 0.50727, 0.50342, 0.49947, 0.49523, 0.49163, 0.48813, 0.48476,\
        0.48154, 0.47839, 0.47516, 0.47170, 0.46786, 0.46350, 0.45852, 0.45288, 0.44659, 0.43970, 0.43230],\
        [0.43917, 0.44560, 0.45168, 0.45740, 0.46275, 0.46774, 0.47237, 0.47663, 0.48055, 0.48415, 0.48745,\
        0.49050, 0.49336, 0.49606, 0.49854, 0.50049, 0.50233, 0.50440, 0.50676, 0.50939, 0.51224, 0.51527,\
        0.51842, 0.52168, 0.52501, 0.52839, 0.53179, 0.53520, 0.53860, 0.54199, 0.54535, 0.54867, 0.55194,\
        0.55517, 0.55836, 0.56150, 0.56461, 0.56771, 0.57082, 0.57397, 0.57718, 0.58047, 0.58385, 0.58725,\
        0.59049, 0.59319, 0.59544, 0.59766, 0.59979, 0.60191, 0.60387, 0.60578, 0.60766, 0.60943, 0.61106,\
        0.61252, 0.61379, 0.61484, 0.61568, 0.61628, 0.61664, 0.61678, 0.61668, 0.61637, 0.61585, 0.61514,\
        0.61426, 0.61322, 0.61206, 0.61078, 0.60942, 0.60801, 0.60661, 0.60536, 0.60422, 0.60289, 0.60137,\
        0.59930, 0.59729, 0.59587, 0.59378, 0.59010, 0.58526, 0.58014, 0.57520, 0.57062, 0.56648, 0.56278,\
        0.55946, 0.55645, 0.55366, 0.55101, 0.54844, 0.54587, 0.54324, 0.54050, 0.53762, 0.53457, 0.53134,\
        0.52794, 0.52437, 0.52065, 0.51679, 0.51279, 0.50862, 0.50418, 0.50043, 0.49707, 0.49361, 0.49035,\
        0.48738, 0.48452, 0.48159, 0.47844, 0.47490, 0.47077, 0.46592, 0.46029, 0.45396, 0.44702, 0.43955],\
        [0.44437, 0.45065, 0.45657, 0.46210, 0.46725, 0.47199, 0.47634, 0.48029, 0.48384, 0.48701, 0.48982,\
        0.49230, 0.49452, 0.49652, 0.49831, 0.49995, 0.50209, 0.50460, 0.50739, 0.51040, 0.51359, 0.51693,\
        0.52036, 0.52386, 0.52741, 0.53098, 0.53456, 0.53813, 0.54168, 0.54520, 0.54868, 0.55210, 0.55546,\
        0.55876, 0.56200, 0.56518, 0.56831, 0.57141, 0.57452, 0.57765, 0.58085, 0.58416, 0.58763, 0.59124,\
        0.59490, 0.59791, 0.59996, 0.60250, 0.60492, 0.60728, 0.60954, 0.61170, 0.61379, 0.61575, 0.61756,\
        0.61919, 0.62061, 0.62180, 0.62275, 0.62346, 0.62392, 0.62412, 0.62409, 0.62382, 0.62333, 0.62266,\
        0.62181, 0.62080, 0.61967, 0.61844, 0.61713, 0.61579, 0.61445, 0.61317, 0.61188, 0.61045, 0.60874,\
        0.60665, 0.60469, 0.60314, 0.60097, 0.59668, 0.59118, 0.58569, 0.58055, 0.57590, 0.57179, 0.56819,\
        0.56501, 0.56215, 0.55952, 0.55701, 0.55456, 0.55208, 0.54953, 0.54682, 0.54394, 0.54085, 0.53755,\
        0.53405, 0.53036, 0.52649, 0.52247, 0.51830, 0.51402, 0.50976, 0.50596, 0.50245, 0.49890, 0.49573,\
        0.49301, 0.49040, 0.48772, 0.48486, 0.48164, 0.47777, 0.47300, 0.46734, 0.46091, 0.45388, 0.44633],\
        [0.44932, 0.45547, 0.46123, 0.46659, 0.47152, 0.47602, 0.48010, 0.48374, 0.48694, 0.48969, 0.49199,\
        0.49390, 0.49549, 0.49685, 0.49815, 0.49972, 0.50196, 0.50479, 0.50796, 0.51135, 0.51490, 0.51854,\
        0.52225, 0.52600, 0.52976, 0.53352, 0.53727, 0.54100, 0.54469, 0.54834, 0.55193, 0.55546, 0.55891,\
        0.56228, 0.56557, 0.56877, 0.57191, 0.57500, 0.57808, 0.58116, 0.58427, 0.58749, 0.59092, 0.59455,\
        0.59821, 0.60183, 0.60438, 0.60712, 0.60988, 0.61256, 0.61512, 0.61757, 0.61988, 0.62206, 0.62406,\
        0.62586, 0.62743, 0.62877, 0.62985, 0.63067, 0.63121, 0.63149, 0.63151, 0.63129, 0.63084, 0.63019,\
        0.62938, 0.62841, 0.62733, 0.62615, 0.62491, 0.62364, 0.62234, 0.62103, 0.61965, 0.61810, 0.61630,\
        0.61425, 0.61214, 0.60991, 0.60684, 0.60216, 0.59646, 0.59091, 0.58574, 0.58110, 0.57709, 0.57366,\
        0.57066, 0.56797, 0.56548, 0.56310, 0.56076, 0.55838, 0.55588, 0.55320, 0.55029, 0.54714, 0.54375,\
        0.54013, 0.53632, 0.53232, 0.52818, 0.52393, 0.51965, 0.51547, 0.51155, 0.50784, 0.50428, 0.50120,\
        0.49856, 0.49605, 0.49350, 0.49085, 0.48803, 0.48446, 0.47970, 0.47389, 0.46733, 0.46018, 0.45257],\
        [0.45402, 0.46005, 0.46568, 0.47087, 0.47560, 0.47988, 0.48369, 0.48704, 0.48993, 0.49231, 0.49416,\
        0.49554, 0.49659, 0.49744, 0.49835, 0.49983, 0.50221, 0.50528, 0.50877, 0.51249, 0.51634, 0.52026,\
        0.52421, 0.52818, 0.53213, 0.53607, 0.53997, 0.54384, 0.54766, 0.55143, 0.55514, 0.55877, 0.56232,\
        0.56576, 0.56910, 0.57233, 0.57546, 0.57854, 0.58158, 0.58460, 0.58760, 0.59062, 0.59386, 0.59760,\
        0.60115, 0.60491, 0.60838, 0.61163, 0.61477, 0.61780, 0.62068, 0.62340, 0.62597, 0.62836, 0.63055,\
        0.63253, 0.63427, 0.63576, 0.63697, 0.63790, 0.63853, 0.63888, 0.63896, 0.63878, 0.63836, 0.63774,\
        0.63695, 0.63602, 0.63499, 0.63388, 0.63271, 0.63150, 0.63025, 0.62894, 0.62751, 0.62589, 0.62401,\
        0.62186, 0.61942, 0.61651, 0.61270, 0.60771, 0.60194, 0.59642, 0.59114, 0.58647, 0.58262, 0.57939,\
        0.57657, 0.57402, 0.57163, 0.56935, 0.56709, 0.56478, 0.56233, 0.55966, 0.55671, 0.55347, 0.54996,\
        0.54621, 0.54226, 0.53815, 0.53392, 0.52962, 0.52533, 0.52115, 0.51717, 0.51339, 0.50989, 0.50677,\
        0.50400, 0.50144, 0.49881, 0.49618, 0.49399, 0.49078, 0.48582, 0.47978, 0.47306, 0.46583, 0.45820],\
        [0.45849, 0.46442, 0.46993, 0.47497, 0.47953, 0.48358, 0.48715, 0.49024, 0.49288, 0.49499, 0.49646,\
        0.49738, 0.49804, 0.49853, 0.49896, 0.50033, 0.50294, 0.50628, 0.51003, 0.51400, 0.51808, 0.52221,\
        0.52636, 0.53049, 0.53461, 0.53868, 0.54272, 0.54671, 0.55064, 0.55452, 0.55834, 0.56207, 0.56571,\
        0.56924, 0.57263, 0.57589, 0.57902, 0.58208, 0.58511, 0.58812, 0.59107, 0.59387, 0.59664, 0.60013,\
        0.60438, 0.60846, 0.61242, 0.61615, 0.61970, 0.62307, 0.62625, 0.62925, 0.63206, 0.63467, 0.63706,\
        0.63922, 0.64113, 0.64277, 0.64411, 0.64515, 0.64588, 0.64630, 0.64642, 0.64627, 0.64588, 0.64528,\
        0.64451, 0.64361, 0.64262, 0.64157, 0.64047, 0.63934, 0.63814, 0.63686, 0.63543, 0.63377, 0.63183,\
        0.62954, 0.62680, 0.62345, 0.61926, 0.61417, 0.60850, 0.60275, 0.59709, 0.59233, 0.58873, 0.58566,\
        0.58293, 0.58043, 0.57807, 0.57581, 0.57358, 0.57132, 0.56891, 0.56621, 0.56318, 0.55981, 0.55615,\
        0.55225, 0.54817, 0.54394, 0.53963, 0.53528, 0.53097, 0.52677, 0.52275, 0.51894, 0.51541, 0.51218,\
        0.50923, 0.50644, 0.50361, 0.50084, 0.49950, 0.49630, 0.49093, 0.48473, 0.47798, 0.47077, 0.46318],\
        [0.46273, 0.46858, 0.47399, 0.47892, 0.48331, 0.48716, 0.49049, 0.49333, 0.49576, 0.49774, 0.49898,\
        0.49950, 0.49995, 0.50031, 0.50050, 0.50169, 0.50454, 0.50804, 0.51192, 0.51601, 0.52022, 0.52448,\
        0.52875, 0.53301, 0.53723, 0.54141, 0.54554, 0.54962, 0.55365, 0.55762, 0.56153, 0.56537, 0.56912,\
        0.57274, 0.57620, 0.57949, 0.58263, 0.58566, 0.58869, 0.59177, 0.59487, 0.59776, 0.60031, 0.60370,\
        0.60805, 0.61250, 0.61680, 0.62089, 0.62477, 0.62844, 0.63190, 0.63515, 0.63818, 0.64100, 0.64359,\
        0.64593, 0.64801, 0.64980, 0.65128, 0.65243, 0.65325, 0.65374, 0.65390, 0.65377, 0.65338, 0.65279,\
        0.65203, 0.65115, 0.65019, 0.64919, 0.64817, 0.64711, 0.64600, 0.64478, 0.64339, 0.64174, 0.63977,\
        0.63739, 0.63450, 0.63095, 0.62664, 0.62158, 0.61595, 0.61003, 0.60413, 0.59942, 0.59588, 0.59271,\
        0.58989, 0.58730, 0.58484, 0.58248, 0.58022, 0.57800, 0.57561, 0.57286, 0.56969, 0.56614, 0.56230,\
        0.55822, 0.55399, 0.54965, 0.54525, 0.54085, 0.53651, 0.53227, 0.52820, 0.52434, 0.52073, 0.51736,\
        0.51421, 0.51120, 0.50828, 0.50565, 0.50347, 0.49984, 0.49469, 0.48866, 0.48205, 0.47498, 0.46753],\
        [0.46671, 0.47251, 0.47787, 0.48271, 0.48698, 0.49064, 0.49373, 0.49633, 0.49851, 0.50035, 0.50153,\
        0.50201, 0.50240, 0.50277, 0.50330, 0.50469, 0.50729, 0.51067, 0.51450, 0.51859, 0.52283, 0.52713,\
        0.53146, 0.53577, 0.54005, 0.54428, 0.54847, 0.55260, 0.55669, 0.56073, 0.56472, 0.56866, 0.57252,\
        0.57627, 0.57984, 0.58318, 0.58632, 0.58932, 0.59234, 0.59553, 0.59884, 0.60206, 0.60503, 0.60851,\
        0.61269, 0.61717, 0.62163, 0.62594, 0.63006, 0.63397, 0.63765, 0.64112, 0.64435, 0.64736, 0.65013,\
        0.65264, 0.65489, 0.65684, 0.65846, 0.65974, 0.66064, 0.66118, 0.66137, 0.66125, 0.66086, 0.66025,\
        0.65948, 0.65860, 0.65766, 0.65671, 0.65576, 0.65480, 0.65379, 0.65268, 0.65137, 0.64979, 0.64785,\
        0.64548, 0.64257, 0.63904, 0.63480, 0.62988, 0.62442, 0.61865, 0.61303, 0.60820, 0.60419, 0.60065,\
        0.59755, 0.59466, 0.59195, 0.58935, 0.58694, 0.58477, 0.58242, 0.57957, 0.57618, 0.57238, 0.56831,\
        0.56405, 0.55966, 0.55521, 0.55073, 0.54627, 0.54187, 0.53758, 0.53346, 0.52952, 0.52579, 0.52227,\
        0.51892, 0.51572, 0.51263, 0.50966, 0.50654, 0.50255, 0.49756, 0.49177, 0.48540, 0.47856, 0.47131],\
        [0.47041, 0.47619, 0.48154, 0.48635, 0.49053, 0.49404, 0.49693, 0.49931, 0.50130, 0.50294, 0.50410,\
        0.50480, 0.50533, 0.50591, 0.50681, 0.50839, 0.51085, 0.51404, 0.51772, 0.52171, 0.52589, 0.53016,\
        0.53448, 0.53879, 0.54307, 0.54732, 0.55152, 0.55567, 0.55978, 0.56386, 0.56791, 0.57193, 0.57591,\
        0.57980, 0.58353, 0.58698, 0.59012, 0.59310, 0.59610, 0.59934, 0.60291, 0.60643, 0.60994, 0.61372,\
        0.61791, 0.62237, 0.62689, 0.63133, 0.63560, 0.63968, 0.64354, 0.64717, 0.65058, 0.65375, 0.65668,\
        0.65936, 0.66176, 0.66387, 0.66564, 0.66704, 0.66804, 0.66863, 0.66884, 0.66871, 0.66830, 0.66766,\
        0.66685, 0.66595, 0.66500, 0.66408, 0.66320, 0.66235, 0.66149, 0.66052, 0.65934, 0.65787, 0.65604,\
        0.65378, 0.65100, 0.64762, 0.64361, 0.63896, 0.63379, 0.62831, 0.62288, 0.61788, 0.61342, 0.60942,\
        0.60581, 0.60247, 0.59936, 0.59633, 0.59362, 0.59162, 0.58933, 0.58625, 0.58251, 0.57841, 0.57409,\
        0.56964, 0.56512, 0.56056, 0.55600, 0.55147, 0.54701, 0.54267, 0.53846, 0.53442, 0.53056, 0.52688,\
        0.52333, 0.51990, 0.51652, 0.51309, 0.50938, 0.50506, 0.50002, 0.49438, 0.48824, 0.48165, 0.47463],\
        [0.47380, 0.47957, 0.48495, 0.48980, 0.49394, 0.49732, 0.50007, 0.50230, 0.50417, 0.50572, 0.50692,\
        0.50782, 0.50861, 0.50949, 0.51070, 0.51246, 0.51488, 0.51792, 0.52144, 0.52529, 0.52936, 0.53355,\
        0.53781, 0.54207, 0.54632, 0.55053, 0.55470, 0.55882, 0.56292, 0.56699, 0.57106, 0.57513, 0.57922,\
        0.58328, 0.58724, 0.59090, 0.59406, 0.59702, 0.60015, 0.60352, 0.60726, 0.61110, 0.61501, 0.61910,\
        0.62343, 0.62794, 0.63250, 0.63700, 0.64137, 0.64556, 0.64955, 0.65331, 0.65685, 0.66016, 0.66323,\
        0.66605, 0.66861, 0.67088, 0.67281, 0.67435, 0.67544, 0.67607, 0.67628, 0.67613, 0.67568, 0.67499,\
        0.67413, 0.67317, 0.67219, 0.67126, 0.67045, 0.66973, 0.66905, 0.66826, 0.66726, 0.66594, 0.66427,\
        0.66219, 0.65964, 0.65656, 0.65288, 0.64860, 0.64377, 0.63854, 0.63320, 0.62801, 0.62314, 0.61863,\
        0.61446, 0.61057, 0.60692, 0.60329, 0.60044, 0.59852, 0.59619, 0.59262, 0.58845, 0.58406, 0.57953,\
        0.57493, 0.57030, 0.56565, 0.56102, 0.55643, 0.55190, 0.54748, 0.54319, 0.53904, 0.53504, 0.53117,\
        0.52742, 0.52373, 0.52003, 0.51620, 0.51208, 0.50748, 0.50233, 0.49676, 0.49080, 0.48442, 0.47762],\
        [0.47683, 0.48258, 0.48801, 0.49296, 0.49714, 0.50041, 0.50305, 0.50522, 0.50706, 0.50863, 0.50994,\
        0.51107, 0.51214, 0.51333, 0.51480, 0.51672, 0.51917, 0.52213, 0.52552, 0.52923, 0.53318, 0.53726,\
        0.54142, 0.54560, 0.54977, 0.55391, 0.55801, 0.56207, 0.56610, 0.57012, 0.57416, 0.57823, 0.58237,\
        0.58657, 0.59078, 0.59486, 0.59815, 0.60098, 0.60438, 0.60802, 0.61193, 0.61603, 0.62025, 0.62461,\
        0.62912, 0.63372, 0.63834, 0.64290, 0.64732, 0.65158, 0.65565, 0.65951, 0.66315, 0.66657, 0.66974,\
        0.67269, 0.67540, 0.67783, 0.67992, 0.68161, 0.68282, 0.68348, 0.68366, 0.68348, 0.68300, 0.68225,\
        0.68130, 0.68026, 0.67921, 0.67824, 0.67746, 0.67688, 0.67642, 0.67587, 0.67505, 0.67391, 0.67243,\
        0.67059, 0.66834, 0.66563, 0.66239, 0.65854, 0.65405, 0.64902, 0.64367, 0.63826, 0.63299, 0.62797,\
        0.62324, 0.61880, 0.61463, 0.61076, 0.60760, 0.60506, 0.60223, 0.59809, 0.59379, 0.58923, 0.58454,\
        0.57985, 0.57515, 0.57045, 0.56576, 0.56111, 0.55651, 0.55201, 0.54762, 0.54335, 0.53921, 0.53517,\
        0.53121, 0.52726, 0.52327, 0.51912, 0.51470, 0.50990, 0.50468, 0.49911, 0.49325, 0.48702, 0.48037],\
        [0.47947, 0.48512, 0.49055, 0.49562, 0.49995, 0.50313, 0.50573, 0.50795, 0.50988, 0.51158, 0.51308,\
        0.51446, 0.51583, 0.51731, 0.51903, 0.52111, 0.52361, 0.52653, 0.52983, 0.53343, 0.53725, 0.54122,\
        0.54527, 0.54935, 0.55342, 0.55746, 0.56146, 0.56542, 0.56935, 0.57327, 0.57721, 0.58121, 0.58531,\
        0.58954, 0.59385, 0.59817, 0.60204, 0.60514, 0.60869, 0.61259, 0.61675, 0.62111, 0.62562, 0.63023,\
        0.63492, 0.63965, 0.64434, 0.64894, 0.65340, 0.65770, 0.66182, 0.66575, 0.66947, 0.67296, 0.67621,\
        0.67925, 0.68209, 0.68469, 0.68692, 0.68877, 0.69016, 0.69081, 0.69093, 0.69076, 0.69026, 0.68943,\
        0.68840, 0.68725, 0.68607, 0.68500, 0.68419, 0.68375, 0.68357, 0.68331, 0.68266, 0.68164, 0.68036,\
        0.67879, 0.67686, 0.67458, 0.67185, 0.66850, 0.66436, 0.65945, 0.65400, 0.64831, 0.64264, 0.63715,\
        0.63191, 0.62697, 0.62235, 0.61812, 0.61438, 0.61094, 0.60723, 0.60300, 0.59869, 0.59379, 0.58910,\
        0.58440, 0.57968, 0.57495, 0.57022, 0.56550, 0.56083, 0.55624, 0.55175, 0.54736, 0.54308, 0.53888,\
        0.53472, 0.53056, 0.52631, 0.52192, 0.51728, 0.51234, 0.50707, 0.50150, 0.49569, 0.48952, 0.48296],\
        [0.48174, 0.48718, 0.49242, 0.49742, 0.50178, 0.50517, 0.50799, 0.51044, 0.51260, 0.51453, 0.51629,\
        0.51795, 0.51961, 0.52136, 0.52331, 0.52555, 0.52813, 0.53105, 0.53429, 0.53780, 0.54152, 0.54537,\
        0.54931, 0.55328, 0.55725, 0.56117, 0.56504, 0.56887, 0.57266, 0.57644, 0.58023, 0.58409, 0.58805,\
        0.59220, 0.59652, 0.60078, 0.60500, 0.60891, 0.61290, 0.61714, 0.62161, 0.62626, 0.63104, 0.63590,\
        0.64079, 0.64566, 0.65043, 0.65507, 0.65955, 0.66386, 0.66801, 0.67198, 0.67577, 0.67932, 0.68262,\
        0.68569, 0.68862, 0.69142, 0.69378, 0.69567, 0.69740, 0.69799, 0.69804, 0.69797, 0.69746, 0.69657,\
        0.69545, 0.69420, 0.69285, 0.69159, 0.69062, 0.69026, 0.69042, 0.69054, 0.68993, 0.68897, 0.68792,\
        0.68663, 0.68498, 0.68311, 0.68098, 0.67823, 0.67443, 0.66955, 0.66389, 0.65788, 0.65182, 0.64590,\
        0.64022, 0.63482, 0.62976, 0.62505, 0.62067, 0.61645, 0.61211, 0.60751, 0.60258, 0.59781, 0.59334,\
        0.58860, 0.58388, 0.57916, 0.57439, 0.56960, 0.56485, 0.56017, 0.55558, 0.55109, 0.54668, 0.54233,\
        0.53800, 0.53365, 0.52920, 0.52461, 0.51981, 0.51477, 0.50947, 0.50394, 0.49817, 0.49194, 0.48540],\
        [0.48377, 0.48892, 0.49381, 0.49845, 0.50285, 0.50669, 0.50995, 0.51276, 0.51524, 0.51748, 0.51953,\
        0.52149, 0.52343, 0.52545, 0.52762, 0.53001, 0.53268, 0.53562, 0.53883, 0.54228, 0.54591, 0.54967,\
        0.55351, 0.55737, 0.56122, 0.56502, 0.56876, 0.57243, 0.57606, 0.57967, 0.58330, 0.58696, 0.59069,\
        0.59463, 0.59903, 0.60347, 0.60795, 0.61242, 0.61694, 0.62159, 0.62642, 0.63139, 0.63647, 0.64159,\
        0.64669, 0.65170, 0.65657, 0.66124, 0.66572, 0.67001, 0.67416, 0.67816, 0.68202, 0.68566, 0.68899,\
        0.69201, 0.69491, 0.69796, 0.70049, 0.70236, 0.70410, 0.70492, 0.70511, 0.70506, 0.70457, 0.70368,\
        0.70252, 0.70117, 0.69966, 0.69819, 0.69686, 0.69646, 0.69671, 0.69752, 0.69662, 0.69574, 0.69501,\
        0.69399, 0.69244, 0.69089, 0.68945, 0.68743, 0.68397, 0.67893, 0.67297, 0.66665, 0.66027, 0.65401,\
        0.64796, 0.64217, 0.63668, 0.63148, 0.62652, 0.62169, 0.61681, 0.61181, 0.60675, 0.60208, 0.59724,\
        0.59242, 0.58783, 0.58313, 0.57829, 0.57340, 0.56856, 0.56379, 0.55912, 0.55454, 0.55003, 0.54555,\
        0.54108, 0.53657, 0.53196, 0.52722, 0.52229, 0.51714, 0.51178, 0.50621, 0.50036, 0.49412, 0.48766],\
        [0.48575, 0.49066, 0.49527, 0.49965, 0.50411, 0.50825, 0.51189, 0.51507, 0.51790, 0.52046, 0.52282,\
        0.52507, 0.52729, 0.52955, 0.53192, 0.53447, 0.53722, 0.54020, 0.54340, 0.54680, 0.55037, 0.55405,\
        0.55780, 0.56157, 0.56532, 0.56900, 0.57259, 0.57610, 0.57956, 0.58300, 0.58648, 0.59000, 0.59349,\
        0.59696, 0.60121, 0.60606, 0.61098, 0.61592, 0.62089, 0.62596, 0.63115, 0.63645, 0.64183, 0.64722,\
        0.65255, 0.65775, 0.66272, 0.66742, 0.67186, 0.67610, 0.68022, 0.68424, 0.68816, 0.69194, 0.69537,\
        0.69834, 0.70120, 0.70421, 0.70693, 0.70902, 0.71067, 0.71169, 0.71212, 0.71211, 0.71167, 0.71082,\
        0.70965, 0.70823, 0.70663, 0.70499, 0.70347, 0.70286, 0.70270, 0.70327, 0.70276, 0.70208, 0.70157,\
        0.70070, 0.69899, 0.69771, 0.69692, 0.69571, 0.69249, 0.68703, 0.68081, 0.67431, 0.66776, 0.66128,\
        0.65497, 0.64888, 0.64303, 0.63741, 0.63198, 0.62666, 0.62137, 0.61608, 0.61083, 0.60566, 0.60042,\
        0.59608, 0.59167, 0.58689, 0.58190, 0.57689, 0.57195, 0.56712, 0.56238, 0.55774, 0.55314, 0.54857,\
        0.54399, 0.53935, 0.53462, 0.52975, 0.52470, 0.51946, 0.51399, 0.50827, 0.50226, 0.49609, 0.48984],\
        [0.48785, 0.49264, 0.49723, 0.50157, 0.50597, 0.51019, 0.51406, 0.51753, 0.52066, 0.52352, 0.52618,\
        0.52871, 0.53118, 0.53366, 0.53621, 0.53889, 0.54173, 0.54475, 0.54796, 0.55133, 0.55484, 0.55847,\
        0.56215, 0.56585, 0.56951, 0.57309, 0.57655, 0.57989, 0.58316, 0.58643, 0.58979, 0.59327, 0.59683,\
        0.60018, 0.60411, 0.60909, 0.61428, 0.61954, 0.62486, 0.63027, 0.63579, 0.64140, 0.64707, 0.65275,\
        0.65834, 0.66374, 0.66884, 0.67356, 0.67792, 0.68206, 0.68613, 0.69015, 0.69409, 0.69801, 0.70167,\
        0.70473, 0.70769, 0.71063, 0.71333, 0.71557, 0.71730, 0.71846, 0.71907, 0.71918, 0.71882, 0.71804,\
        0.71690, 0.71547, 0.71387, 0.71221, 0.71076, 0.70984, 0.70941, 0.70926, 0.70889, 0.70829, 0.70760,\
        0.70655, 0.70503, 0.70388, 0.70309, 0.70204, 0.69861, 0.69310, 0.68710, 0.68069, 0.67416, 0.66763,\
        0.66118, 0.65488, 0.64876, 0.64282, 0.63703, 0.63135, 0.62575, 0.62021, 0.61475, 0.60943, 0.60446,\
        0.60006, 0.59548, 0.59038, 0.58515, 0.58001, 0.57501, 0.57015, 0.56539, 0.56071, 0.55607, 0.55143,\
        0.54676, 0.54203, 0.53720, 0.53223, 0.52709, 0.52175, 0.51620, 0.51040, 0.50437, 0.49829, 0.49210],\
        [0.49014, 0.49485, 0.49948, 0.50397, 0.50833, 0.51256, 0.51655, 0.52023, 0.52360, 0.52672, 0.52963,\
        0.53240, 0.53509, 0.53777, 0.54048, 0.54328, 0.54619, 0.54925, 0.55246, 0.55581, 0.55929, 0.56286,\
        0.56650, 0.57015, 0.57376, 0.57727, 0.58061, 0.58379, 0.58686, 0.58994, 0.59317, 0.59666, 0.60031,\
        0.60401, 0.60801, 0.61275, 0.61794, 0.62334, 0.62887, 0.63453, 0.64030, 0.64618, 0.65213, 0.65809,\
        0.66396, 0.66962, 0.67490, 0.67967, 0.68390, 0.68783, 0.69184, 0.69593, 0.69985, 0.70382, 0.70759,\
        0.71097, 0.71411, 0.71709, 0.71981, 0.72213, 0.72397, 0.72528, 0.72605, 0.72630, 0.72606, 0.72537,\
        0.72429, 0.72291, 0.72134, 0.71973, 0.71827, 0.71713, 0.71633, 0.71573, 0.71508, 0.71429, 0.71332,\
        0.71209, 0.71065, 0.70927, 0.70786, 0.70573, 0.70198, 0.69754, 0.69193, 0.68581, 0.67948, 0.67304,\
        0.66658, 0.66018, 0.65387, 0.64770, 0.64166, 0.63573, 0.62989, 0.62416, 0.61856, 0.61320, 0.60820,\
        0.60357, 0.59875, 0.59327, 0.58789, 0.58272, 0.57775, 0.57292, 0.56818, 0.56349, 0.55883, 0.55415,\
        0.54943, 0.54462, 0.53972, 0.53467, 0.52947, 0.52408, 0.51849, 0.51270, 0.50675, 0.50070, 0.49449],\
        [0.49266, 0.49736, 0.50194, 0.50653, 0.51099, 0.51529, 0.51937, 0.52319, 0.52675, 0.53008, 0.53320,\
        0.53617, 0.53905, 0.54188, 0.54472, 0.54761, 0.55059, 0.55367, 0.55688, 0.56020, 0.56364, 0.56718,\
        0.57078, 0.57440, 0.57800, 0.58149, 0.58477, 0.58779, 0.59065, 0.59352, 0.59658, 0.60007, 0.60382,\
        0.60775, 0.61198, 0.61670, 0.62183, 0.62726, 0.63290, 0.63870, 0.64465, 0.65073, 0.65691, 0.66314,\
        0.66931, 0.67528, 0.68084, 0.68575, 0.68983, 0.69335, 0.69720, 0.70152, 0.70557, 0.70958, 0.71345,\
        0.71705, 0.72039, 0.72349, 0.72630, 0.72872, 0.73070, 0.73216, 0.73309, 0.73350, 0.73339, 0.73281,\
        0.73182, 0.73050, 0.72897, 0.72736, 0.72583, 0.72448, 0.72334, 0.72231, 0.72128, 0.72014, 0.71882,\
        0.71730, 0.71561, 0.71377, 0.71166, 0.70890, 0.70528, 0.70084, 0.69551, 0.68984, 0.68387, 0.67761,\
        0.67121, 0.66477, 0.65838, 0.65207, 0.64587, 0.63977, 0.63376, 0.62787, 0.62213, 0.61659, 0.61129,\
        0.60613, 0.60084, 0.59538, 0.59012, 0.58509, 0.58024, 0.57549, 0.57081, 0.56615, 0.56148, 0.55677,\
        0.55201, 0.54716, 0.54220, 0.53711, 0.53186, 0.52644, 0.52084, 0.51507, 0.50915, 0.50311, 0.49688],\
        [0.49541, 0.50021, 0.50485, 0.50945, 0.51396, 0.51832, 0.52249, 0.52643, 0.53013, 0.53361, 0.53689,\
        0.54002, 0.54304, 0.54599, 0.54893, 0.55189, 0.55490, 0.55799, 0.56118, 0.56447, 0.56787, 0.57135,\
        0.57492, 0.57853, 0.58214, 0.58567, 0.58897, 0.59188, 0.59449, 0.59724, 0.60017, 0.60359, 0.60745,\
        0.61155, 0.61593, 0.62068, 0.62580, 0.63121, 0.63687, 0.64273, 0.64877, 0.65497, 0.66132, 0.66776,\
        0.67421, 0.68052, 0.68647, 0.69171, 0.69580, 0.69894, 0.70240, 0.70680, 0.71111, 0.71526, 0.71927,\
        0.72305, 0.72657, 0.72983, 0.73277, 0.73534, 0.73747, 0.73910, 0.74021, 0.74078, 0.74081, 0.74034,\
        0.73943, 0.73816, 0.73664, 0.73500, 0.73335, 0.73178, 0.73030, 0.72887, 0.72741, 0.72584, 0.72411,\
        0.72217, 0.72002, 0.71765, 0.71493, 0.71170, 0.70782, 0.70322, 0.69822, 0.69319, 0.68755, 0.68146,\
        0.67513, 0.66871, 0.66230, 0.65595, 0.64967, 0.64346, 0.63735, 0.63133, 0.62542, 0.61964, 0.61396,\
        0.60831, 0.60259, 0.59713, 0.59212, 0.58730, 0.58261, 0.57797, 0.57335, 0.56872, 0.56405, 0.55933,\
        0.55454, 0.54966, 0.54467, 0.53954, 0.53427, 0.52883, 0.52323, 0.51747, 0.51155, 0.50547, 0.49923],\
        [0.49823, 0.50317, 0.50797, 0.51267, 0.51725, 0.52167, 0.52591, 0.52992, 0.53372, 0.53731, 0.54071,\
        0.54395, 0.54707, 0.55011, 0.55310, 0.55609, 0.55911, 0.56219, 0.56534, 0.56858, 0.57192, 0.57534,\
        0.57884, 0.58241, 0.58604, 0.58967, 0.59313, 0.59604, 0.59828, 0.60097, 0.60401, 0.60741, 0.61124,\
        0.61539, 0.61985, 0.62462, 0.62971, 0.63509, 0.64071, 0.64656, 0.65260, 0.65884, 0.66526, 0.67182,\
        0.67847, 0.68506, 0.69138, 0.69713, 0.70141, 0.70467, 0.70813, 0.71225, 0.71658, 0.72087, 0.72501,\
        0.72896, 0.73267, 0.73610, 0.73921, 0.74195, 0.74426, 0.74608, 0.74738, 0.74813, 0.74831, 0.74794,\
        0.74709, 0.74584, 0.74429, 0.74256, 0.74075, 0.73894, 0.73714, 0.73532, 0.73343, 0.73140, 0.72918,\
        0.72674, 0.72405, 0.72110, 0.71782, 0.71417, 0.71009, 0.70566, 0.70114, 0.69621, 0.69065, 0.68463,\
        0.67837, 0.67203, 0.66568, 0.65935, 0.65307, 0.64685, 0.64067, 0.63456, 0.62851, 0.62253, 0.61658,\
        0.61065, 0.60479, 0.59931, 0.59430, 0.58960, 0.58502, 0.58046, 0.57588, 0.57126, 0.56659, 0.56186,\
        0.55705, 0.55214, 0.54712, 0.54198, 0.53670, 0.53126, 0.52567, 0.51991, 0.51399, 0.50790, 0.50167],\
        [0.50156, 0.50650, 0.51145, 0.51624, 0.52087, 0.52534, 0.52960, 0.53367, 0.53752, 0.54118, 0.54465,\
        0.54796, 0.55113, 0.55421, 0.55723, 0.56022, 0.56322, 0.56625, 0.56934, 0.57251, 0.57576, 0.57909,\
        0.58250, 0.58598, 0.58954, 0.59318, 0.59688, 0.60003, 0.60222, 0.60473, 0.60780, 0.61125, 0.61507,\
        0.61923, 0.62370, 0.62847, 0.63352, 0.63884, 0.64438, 0.65015, 0.65611, 0.66228, 0.66866, 0.67522,\
        0.68194, 0.68868, 0.69515, 0.70096, 0.70571, 0.70957, 0.71341, 0.71757, 0.72196, 0.72637, 0.73067,\
        0.73480, 0.73869, 0.74230, 0.74560, 0.74853, 0.75104, 0.75308, 0.75459, 0.75552, 0.75584, 0.75557,\
        0.75475, 0.75347, 0.75185, 0.74999, 0.74800, 0.74595, 0.74383, 0.74164, 0.73932, 0.73682, 0.73408,\
        0.73108, 0.72781, 0.72425, 0.72042, 0.71633, 0.71205, 0.70771, 0.70335, 0.69860, 0.69304, 0.68708,\
        0.68096, 0.67476, 0.66855, 0.66234, 0.65614, 0.64995, 0.64378, 0.63763, 0.63150, 0.62540, 0.61932,\
        0.61330, 0.60742, 0.60184, 0.59677, 0.59215, 0.58759, 0.58303, 0.57845, 0.57382, 0.56913, 0.56437,\
        0.55953, 0.55460, 0.54957, 0.54442, 0.53914, 0.53371, 0.52813, 0.52239, 0.51649, 0.51044, 0.50424],\
        [0.50590, 0.51063, 0.51547, 0.52023, 0.52485, 0.52930, 0.53357, 0.53764, 0.54152, 0.54520, 0.54870,\
        0.55203, 0.55523, 0.55831, 0.56131, 0.56426, 0.56720, 0.57016, 0.57316, 0.57622, 0.57937, 0.58258,\
        0.58586, 0.58920, 0.59259, 0.59603, 0.59955, 0.60293, 0.60563, 0.60836, 0.61147, 0.61496, 0.61880,\
        0.62298, 0.62745, 0.63219, 0.63719, 0.64242, 0.64786, 0.65348, 0.65929, 0.66530, 0.67150, 0.67793,\
        0.68456, 0.69131, 0.69790, 0.70377, 0.70889, 0.71349, 0.71794, 0.72247, 0.72710, 0.73171, 0.73622,\
        0.74054, 0.74462, 0.74842, 0.75191, 0.75504, 0.75777, 0.76004, 0.76178, 0.76292, 0.76340, 0.76320,\
        0.76237, 0.76102, 0.75926, 0.75724, 0.75505, 0.75276, 0.75036, 0.74783, 0.74510, 0.74213, 0.73887,\
        0.73530, 0.73141, 0.72721, 0.72277, 0.71816, 0.71350, 0.70895, 0.70452, 0.69982, 0.69448, 0.68876,\
        0.68290, 0.67697, 0.67099, 0.66498, 0.65893, 0.65284, 0.64673, 0.64059, 0.63445, 0.62831, 0.62221,\
        0.61619, 0.61034, 0.60478, 0.59976, 0.59501, 0.59033, 0.58571, 0.58107, 0.57639, 0.57166, 0.56686,\
        0.56200, 0.55706, 0.55202, 0.54687, 0.54160, 0.53619, 0.53063, 0.52491, 0.51904, 0.51302, 0.50686],\
        [0.51074, 0.51532, 0.51999, 0.52463, 0.52917, 0.53356, 0.53778, 0.54182, 0.54567, 0.54934, 0.55283,\
        0.55616, 0.55933, 0.56238, 0.56533, 0.56821, 0.57105, 0.57389, 0.57677, 0.57971, 0.58273, 0.58582,\
        0.58895, 0.59212, 0.59533, 0.59856, 0.60189, 0.60530, 0.60846, 0.61155, 0.61485, 0.61845, 0.62236,\
        0.62656, 0.63104, 0.63576, 0.64070, 0.64583, 0.65113, 0.65659, 0.66219, 0.66794, 0.67388, 0.68001,\
        0.68638, 0.69296, 0.69958, 0.70567, 0.71134, 0.71670, 0.72187, 0.72695, 0.73198, 0.73689, 0.74164,\
        0.74618, 0.75045, 0.75445, 0.75813, 0.76147, 0.76442, 0.76694, 0.76893, 0.77029, 0.77093, 0.77078,\
        0.76990, 0.76841, 0.76648, 0.76426, 0.76186, 0.75935, 0.75671, 0.75388, 0.75080, 0.74739, 0.74362,\
        0.73947, 0.73495, 0.73010, 0.72498, 0.71972, 0.71447, 0.70943, 0.70474, 0.70013, 0.69509, 0.68977,\
        0.68432, 0.67876, 0.67312, 0.66737, 0.66152, 0.65559, 0.64958, 0.64350, 0.63739, 0.63128, 0.62521,\
        0.61923, 0.61345, 0.60798, 0.60286, 0.59791, 0.59316, 0.58846, 0.58373, 0.57897, 0.57417, 0.56933,\
        0.56444, 0.55949, 0.55446, 0.54932, 0.54407, 0.53868, 0.53314, 0.52746, 0.52162, 0.51563, 0.50949],\
        [0.51591, 0.52037, 0.52487, 0.52936, 0.53377, 0.53806, 0.54219, 0.54616, 0.54996, 0.55358, 0.55703,\
        0.56031, 0.56344, 0.56642, 0.56928, 0.57204, 0.57475, 0.57745, 0.58017, 0.58295, 0.58582, 0.58878,\
        0.59179, 0.59478, 0.59785, 0.60102, 0.60428, 0.60765, 0.61101, 0.61440, 0.61792, 0.62167, 0.62567,\
        0.62993, 0.63443, 0.63914, 0.64403, 0.64907, 0.65424, 0.65951, 0.66487, 0.67034, 0.67592, 0.68167,\
        0.68763, 0.69384, 0.70027, 0.70690, 0.71334, 0.71951, 0.72544, 0.73115, 0.73666, 0.74194, 0.74698,\
        0.75174, 0.75621, 0.76038, 0.76424, 0.76776, 0.77093, 0.77370, 0.77596, 0.77758, 0.77839, 0.77828,\
        0.77729, 0.77559, 0.77342, 0.77098, 0.76839, 0.76571, 0.76289, 0.75983, 0.75645, 0.75265, 0.74840,\
        0.74370, 0.73856, 0.73304, 0.72723, 0.72123, 0.71521, 0.70942, 0.70424, 0.69974, 0.69512, 0.69036,\
        0.68544, 0.68034, 0.67507, 0.66963, 0.66402, 0.65826, 0.65238, 0.64639, 0.64034, 0.63427, 0.62824,\
        0.62232, 0.61658, 0.61109, 0.60585, 0.60084, 0.59606, 0.59122, 0.58637, 0.58151, 0.57664, 0.57176,\
        0.56685, 0.56190, 0.55688, 0.55176, 0.54653, 0.54117, 0.53567, 0.53002, 0.52421, 0.51824, 0.51211],\
        [0.52130, 0.52564, 0.53000, 0.53433, 0.53859, 0.54274, 0.54676, 0.55063, 0.55434, 0.55789, 0.56127,\
        0.56448, 0.56752, 0.57041, 0.57315, 0.57577, 0.57830, 0.58080, 0.58332, 0.58592, 0.58862, 0.59145,\
        0.59438, 0.59723, 0.60007, 0.60321, 0.60649, 0.60990, 0.61340, 0.61698, 0.62070, 0.62461, 0.62873,\
        0.63307, 0.63760, 0.64232, 0.64718, 0.65215, 0.65720, 0.66230, 0.66745, 0.67263, 0.67787, 0.68322,\
        0.68877, 0.69469, 0.70117, 0.70820, 0.71534, 0.72229, 0.72895, 0.73528, 0.74129, 0.74695, 0.75227,\
        0.75725, 0.76190, 0.76622, 0.77023, 0.77391, 0.77726, 0.78026, 0.78280, 0.78472, 0.78573, 0.78563,\
        0.78445, 0.78247, 0.78001, 0.77734, 0.77460, 0.77182, 0.76891, 0.76572, 0.76211, 0.75797, 0.75329,\
        0.74806, 0.74233, 0.73619, 0.72972, 0.72303, 0.71624, 0.70956, 0.70336, 0.69881, 0.69504, 0.69096,\
        0.68658, 0.68194, 0.67704, 0.67190, 0.66653, 0.66094, 0.65518, 0.64928, 0.64329, 0.63727, 0.63127,\
        0.62537, 0.61963, 0.61411, 0.60882, 0.60378, 0.59882, 0.59387, 0.58892, 0.58396, 0.57902, 0.57411,\
        0.56920, 0.56427, 0.55927, 0.55419, 0.54900, 0.54367, 0.53820, 0.53258, 0.52679, 0.52084, 0.51473],\
        [0.52682, 0.53106, 0.53528, 0.53945, 0.54354, 0.54754, 0.55141, 0.55516, 0.55876, 0.56221, 0.56550,\
        0.56862, 0.57156, 0.57433, 0.57693, 0.57937, 0.58170, 0.58396, 0.58624, 0.58859, 0.59107, 0.59373,\
        0.59660, 0.59955, 0.60219, 0.60521, 0.60851, 0.61198, 0.61559, 0.61933, 0.62322, 0.62727, 0.63151,\
        0.63594, 0.64053, 0.64528, 0.65013, 0.65506, 0.66004, 0.66502, 0.67000, 0.67496, 0.67993, 0.68497,\
        0.69019, 0.69585, 0.70250, 0.71000, 0.71775, 0.72535, 0.73264, 0.73953, 0.74600, 0.75202, 0.75760,\
        0.76277, 0.76755, 0.77200, 0.77612, 0.77990, 0.78336, 0.78654, 0.78934, 0.79157, 0.79284, 0.79272,\
        0.79124, 0.78895, 0.78617, 0.78326, 0.78044, 0.77768, 0.77481, 0.77159, 0.76782, 0.76340, 0.75831,\
        0.75260, 0.74635, 0.73966, 0.73265, 0.72543, 0.71813, 0.71096, 0.70436, 0.69968, 0.69599, 0.69215,\
        0.68809, 0.68378, 0.67920, 0.67430, 0.66912, 0.66367, 0.65801, 0.65218, 0.64624, 0.64024, 0.63425,\
        0.62834, 0.62256, 0.61697, 0.61159, 0.60640, 0.60133, 0.59637, 0.59130, 0.58624, 0.58127, 0.57636,\
        0.57148, 0.56659, 0.56164, 0.55660, 0.55145, 0.54616, 0.54072, 0.53513, 0.52936, 0.52343, 0.51732],\
        [0.53242, 0.53655, 0.54063, 0.54464, 0.54856, 0.55239, 0.55610, 0.55970, 0.56317, 0.56650, 0.56968,\
        0.57269, 0.57553, 0.57818, 0.58061, 0.58286, 0.58493, 0.58692, 0.58890, 0.59097, 0.59319, 0.59555,\
        0.59808, 0.60096, 0.60386, 0.60695, 0.61031, 0.61387, 0.61759, 0.62146, 0.62548, 0.62966, 0.63401,\
        0.63853, 0.64320, 0.64799, 0.65287, 0.65781, 0.66276, 0.66769, 0.67258, 0.67743, 0.68227, 0.68717,\
        0.69230, 0.69791, 0.70465, 0.71260, 0.72080, 0.72890, 0.73669, 0.74405, 0.75091, 0.75725, 0.76305,\
        0.76834, 0.77321, 0.77775, 0.78197, 0.78579, 0.78925, 0.79252, 0.79548, 0.79795, 0.79943, 0.79927,\
        0.79745, 0.79495, 0.79180, 0.78874, 0.78593, 0.78332, 0.78063, 0.77750, 0.77365, 0.76897, 0.76349,\
        0.75732, 0.75061, 0.74348, 0.73607, 0.72853, 0.72103, 0.71387, 0.70752, 0.70246, 0.69818, 0.69411,\
        0.69013, 0.68604, 0.68165, 0.67692, 0.67185, 0.66650, 0.66089, 0.65510, 0.64918, 0.64318, 0.63716,\
        0.63120, 0.62534, 0.61964, 0.61412, 0.60880, 0.60368, 0.59864, 0.59337, 0.58830, 0.58336, 0.57851,\
        0.57370, 0.56887, 0.56398, 0.55900, 0.55389, 0.54864, 0.54324, 0.53766, 0.53191, 0.52598, 0.51987],\
        [0.53804, 0.54207, 0.54600, 0.54984, 0.55358, 0.55723, 0.56076, 0.56419, 0.56751, 0.57070, 0.57376,\
        0.57666, 0.57939, 0.58191, 0.58419, 0.58622, 0.58802, 0.58969, 0.59133, 0.59308, 0.59505, 0.59719,\
        0.59929, 0.60196, 0.60514, 0.60844, 0.61192, 0.61558, 0.61939, 0.62336, 0.62748, 0.63176, 0.63620,\
        0.64080, 0.64555, 0.65041, 0.65535, 0.66033, 0.66532, 0.67028, 0.67519, 0.68005, 0.68491, 0.68984,\
        0.69514, 0.70119, 0.70818, 0.71614, 0.72456, 0.73299, 0.74116, 0.74890, 0.75610, 0.76271, 0.76869,\
        0.77404, 0.77890, 0.78347, 0.78784, 0.79172, 0.79499, 0.79833, 0.80132, 0.80375, 0.80509, 0.80482,\
        0.80311, 0.80035, 0.79696, 0.79387, 0.79114, 0.78883, 0.78648, 0.78351, 0.77959, 0.77463, 0.76876,\
        0.76216, 0.75504, 0.74758, 0.73991, 0.73222, 0.72469, 0.71760, 0.71125, 0.70580, 0.70102, 0.69668,\
        0.69275, 0.68878, 0.68448, 0.67979, 0.67476, 0.66942, 0.66383, 0.65804, 0.65210, 0.64606, 0.63999,\
        0.63394, 0.62798, 0.62213, 0.61644, 0.61090, 0.60548, 0.60009, 0.59508, 0.59014, 0.58531, 0.58057,\
        0.57586, 0.57112, 0.56631, 0.56138, 0.55633, 0.55111, 0.54573, 0.54017, 0.53443, 0.52850, 0.52238],\
        [0.54364, 0.54755, 0.55133, 0.55499, 0.55854, 0.56199, 0.56534, 0.56858, 0.57172, 0.57475, 0.57767,\
        0.58046, 0.58309, 0.58550, 0.58764, 0.58946, 0.59098, 0.59228, 0.59354, 0.59493, 0.59664, 0.59876,\
        0.60101, 0.60345, 0.60650, 0.60986, 0.61341, 0.61713, 0.62101, 0.62503, 0.62921, 0.63355, 0.63806,\
        0.64272, 0.64754, 0.65247, 0.65750, 0.66257, 0.66765, 0.67270, 0.67771, 0.68271, 0.68775, 0.69285,\
        0.69827, 0.70471, 0.71207, 0.72021, 0.72882, 0.73752, 0.74601, 0.75407, 0.76157, 0.76844, 0.77460,\
        0.77997, 0.78467, 0.78914, 0.79370, 0.79802, 0.80086, 0.80421, 0.80720, 0.80939, 0.81046, 0.81011,\
        0.80840, 0.80558, 0.80222, 0.79909, 0.79622, 0.79437, 0.79242, 0.78961, 0.78558, 0.78028, 0.77396,\
        0.76696, 0.75951, 0.75180, 0.74399, 0.73625, 0.72875, 0.72170, 0.71528, 0.70954, 0.70440, 0.69981,\
        0.69602, 0.69202, 0.68763, 0.68287, 0.67778, 0.67242, 0.66681, 0.66098, 0.65499, 0.64888, 0.64272,\
        0.63657, 0.63047, 0.62447, 0.61860, 0.61285, 0.60726, 0.60191, 0.59679, 0.59189, 0.58720, 0.58259,\
        0.57800, 0.57336, 0.56863, 0.56377, 0.55876, 0.55357, 0.54821, 0.54266, 0.53691, 0.53097, 0.52484],\
        [0.54918, 0.55295, 0.55657, 0.56005, 0.56340, 0.56663, 0.56976, 0.57279, 0.57574, 0.57860, 0.58136,\
        0.58403, 0.58656, 0.58889, 0.59093, 0.59257, 0.59380, 0.59472, 0.59556, 0.59656, 0.59795, 0.59983,\
        0.60224, 0.60490, 0.60793, 0.61127, 0.61483, 0.61856, 0.62245, 0.62648, 0.63067, 0.63501, 0.63953,\
        0.64423, 0.64909, 0.65410, 0.65922, 0.66441, 0.66962, 0.67480, 0.67997, 0.68517, 0.69052, 0.69606,\
        0.70176, 0.70836, 0.71598, 0.72438, 0.73327, 0.74229, 0.75111, 0.75948, 0.76726, 0.77439, 0.78082,\
        0.78628, 0.79065, 0.79483, 0.79942, 0.80396, 0.80749, 0.81062, 0.81337, 0.81529, 0.81611, 0.81565,\
        0.81396, 0.81127, 0.80802, 0.80467, 0.80179, 0.80020, 0.79844, 0.79563, 0.79143, 0.78564, 0.77885,\
        0.77150, 0.76381, 0.75598, 0.74813, 0.74041, 0.73296, 0.72595, 0.71948, 0.71362, 0.70837, 0.70381,\
        0.69981, 0.69553, 0.69095, 0.68603, 0.68084, 0.67543, 0.66978, 0.66389, 0.65782, 0.65162, 0.64536,\
        0.63908, 0.63286, 0.62671, 0.62067, 0.61477, 0.60904, 0.60354, 0.59837, 0.59362, 0.58910, 0.58464,\
        0.58017, 0.57562, 0.57096, 0.56616, 0.56118, 0.55602, 0.55066, 0.54510, 0.53935, 0.53339, 0.52724],\
        [0.55461, 0.55825, 0.56170, 0.56497, 0.56810, 0.57110, 0.57400, 0.57679, 0.57951, 0.58217, 0.58476,\
        0.58728, 0.58970, 0.59197, 0.59397, 0.59551, 0.59646, 0.59700, 0.59748, 0.59811, 0.59927, 0.60112,\
        0.60353, 0.60631, 0.60937, 0.61268, 0.61621, 0.61990, 0.62373, 0.62770, 0.63182, 0.63611, 0.64058,\
        0.64524, 0.65010, 0.65516, 0.66037, 0.66569, 0.67106, 0.67642, 0.68174, 0.68709, 0.69264, 0.69863,\
        0.70474, 0.71159, 0.71951, 0.72829, 0.73760, 0.74706, 0.75631, 0.76502, 0.77304, 0.78039, 0.78720,\
        0.79319, 0.79720, 0.80102, 0.80580, 0.81037, 0.81425, 0.81745, 0.81998, 0.82164, 0.82224, 0.82168,\
        0.82000, 0.81742, 0.81429, 0.81103, 0.80822, 0.80607, 0.80392, 0.80107, 0.79660, 0.79023, 0.78310,\
        0.77555, 0.76779, 0.75995, 0.75215, 0.74451, 0.73716, 0.73020, 0.72372, 0.71779, 0.71245, 0.70771,\
        0.70333, 0.69897, 0.69418, 0.68905, 0.68381, 0.67840, 0.67271, 0.66675, 0.66057, 0.65425, 0.64787,\
        0.64147, 0.63513, 0.62887, 0.62272, 0.61671, 0.61088, 0.60526, 0.60000, 0.59554, 0.59111, 0.58677,\
        0.58239, 0.57793, 0.57332, 0.56855, 0.56360, 0.55844, 0.55308, 0.54750, 0.54172, 0.53574, 0.52955],\
        [0.55992, 0.56340, 0.56666, 0.56972, 0.57261, 0.57536, 0.57800, 0.58053, 0.58300, 0.58541, 0.58778,\
        0.59012, 0.59241, 0.59460, 0.59659, 0.59815, 0.59886, 0.59906, 0.59936, 0.59980, 0.60082, 0.60269,\
        0.60507, 0.60782, 0.61086, 0.61411, 0.61756, 0.62114, 0.62486, 0.62870, 0.63267, 0.63681, 0.64113,\
        0.64568, 0.65047, 0.65551, 0.66079, 0.66624, 0.67179, 0.67735, 0.68284, 0.68826, 0.69373, 0.69959,\
        0.70628, 0.71380, 0.72226, 0.73159, 0.74149, 0.75157, 0.76142, 0.77061, 0.77885, 0.78621, 0.79319,\
        0.79989, 0.80462, 0.80862, 0.81318, 0.81763, 0.82152, 0.82469, 0.82705, 0.82851, 0.82892, 0.82824,\
        0.82653, 0.82397, 0.82085, 0.81755, 0.81441, 0.81152, 0.80855, 0.80498, 0.79995, 0.79361, 0.78649,\
        0.77899, 0.77132, 0.76361, 0.75596, 0.74847, 0.74123, 0.73433, 0.72785, 0.72184, 0.71632, 0.71125,\
        0.70650, 0.70189, 0.69682, 0.69171, 0.68665, 0.68131, 0.67558, 0.66951, 0.66319, 0.65674, 0.65022,\
        0.64372, 0.63728, 0.63094, 0.62473, 0.61870, 0.61287, 0.60735, 0.60230, 0.59760, 0.59326, 0.58900,\
        0.58469, 0.58027, 0.57569, 0.57095, 0.56599, 0.56083, 0.55545, 0.54984, 0.54403, 0.53800, 0.53177],\
        [0.56507, 0.56838, 0.57144, 0.57426, 0.57690, 0.57938, 0.58173, 0.58398, 0.58617, 0.58830, 0.59040,\
        0.59249, 0.59457, 0.59661, 0.59849, 0.60003, 0.60067, 0.60083, 0.60115, 0.60170, 0.60278, 0.60453,\
        0.60682, 0.60948, 0.61243, 0.61558, 0.61890, 0.62233, 0.62586, 0.62948, 0.63321, 0.63708, 0.64114,\
        0.64544, 0.65005, 0.65498, 0.66024, 0.66578, 0.67153, 0.67736, 0.68314, 0.68873, 0.69416, 0.69993,\
        0.70695, 0.71497, 0.72397, 0.73395, 0.74462, 0.75554, 0.76622, 0.77616, 0.78483, 0.79207, 0.79873,\
        0.80579, 0.81170, 0.81660, 0.82123, 0.82556, 0.82934, 0.83239, 0.83460, 0.83587, 0.83611, 0.83530,\
        0.83348, 0.83080, 0.82751, 0.82389, 0.82016, 0.81637, 0.81236, 0.80780, 0.80235, 0.79603, 0.78911,\
        0.78183, 0.77440, 0.76694, 0.75952, 0.75221, 0.74511, 0.73828, 0.73179, 0.72566, 0.71992, 0.71449,\
        0.70923, 0.70393, 0.69869, 0.69421, 0.68949, 0.68421, 0.67837, 0.67212, 0.66563, 0.65902, 0.65238,\
        0.64578, 0.63927, 0.63289, 0.62668, 0.62068, 0.61494, 0.60951, 0.60443, 0.59984, 0.59558, 0.59134,\
        0.58703, 0.58262, 0.57806, 0.57332, 0.56836, 0.56317, 0.55775, 0.55210, 0.54623, 0.54016, 0.53388],\
        [0.57003, 0.57316, 0.57599, 0.57857, 0.58093, 0.58312, 0.58517, 0.58712, 0.58900, 0.59082, 0.59262,\
        0.59440, 0.59617, 0.59795, 0.59967, 0.60091, 0.60174, 0.60224, 0.60278, 0.60357, 0.60478, 0.60651,\
        0.60870, 0.61126, 0.61409, 0.61711, 0.62026, 0.62348, 0.62676, 0.63008, 0.63345, 0.63692, 0.64056,\
        0.64445, 0.64870, 0.65337, 0.65848, 0.66401, 0.66990, 0.67606, 0.68232, 0.68845, 0.69426, 0.70014,\
        0.70715, 0.71508, 0.72435, 0.73502, 0.74662, 0.75857, 0.77033, 0.78140, 0.79112, 0.79857, 0.80553,\
        0.81262, 0.81909, 0.82467, 0.82958, 0.83392, 0.83760, 0.84052, 0.84257, 0.84368, 0.84375, 0.84277,\
        0.84075, 0.83782, 0.83417, 0.83002, 0.82554, 0.82079, 0.81571, 0.81020, 0.80424, 0.79792, 0.79119,\
        0.78420, 0.77712, 0.76999, 0.76285, 0.75576, 0.74880, 0.74203, 0.73551, 0.72928, 0.72332, 0.71760,\
        0.71204, 0.70658, 0.70158, 0.69718, 0.69256, 0.68711, 0.68098, 0.67447, 0.66778, 0.66102, 0.65428,\
        0.64761, 0.64106, 0.63469, 0.62851, 0.62258, 0.61694, 0.61165, 0.60674, 0.60224, 0.59796, 0.59367,\
        0.58934, 0.58494, 0.58040, 0.57565, 0.57066, 0.56543, 0.55995, 0.55425, 0.54832, 0.54219, 0.53586],\
        [0.57477, 0.57771, 0.58030, 0.58261, 0.58468, 0.58656, 0.58830, 0.58994, 0.59150, 0.59301, 0.59448,\
        0.59592, 0.59732, 0.59868, 0.60006, 0.60138, 0.60247, 0.60336, 0.60425, 0.60532, 0.60671, 0.60850,\
        0.61066, 0.61314, 0.61585, 0.61871, 0.62166, 0.62465, 0.62761, 0.63054, 0.63344, 0.63636, 0.63938,\
        0.64265, 0.64630, 0.65046, 0.65520, 0.66052, 0.66638, 0.67277, 0.67962, 0.68677, 0.69378, 0.70028,\
        0.70675, 0.71357, 0.72285, 0.73438, 0.74711, 0.76025, 0.77320, 0.78544, 0.79666, 0.80535, 0.81282,\
        0.82015, 0.82694, 0.83294, 0.83813, 0.84254, 0.84616, 0.84896, 0.85088, 0.85184, 0.85176, 0.85058,\
        0.84829, 0.84498, 0.84083, 0.83603, 0.83072, 0.82500, 0.81890, 0.81244, 0.80583, 0.79955, 0.79283,\
        0.78629, 0.77966, 0.77290, 0.76604, 0.75916, 0.75232, 0.74560, 0.73904, 0.73269, 0.72657, 0.72067,\
        0.71502, 0.70968, 0.70484, 0.70043, 0.69574, 0.68973, 0.68318, 0.67640, 0.66953, 0.66266, 0.65585,\
        0.64915, 0.64260, 0.63625, 0.63014, 0.62430, 0.61878, 0.61361, 0.60882, 0.60437, 0.60010, 0.59581,\
        0.59151, 0.58716, 0.58265, 0.57790, 0.57288, 0.56758, 0.56204, 0.55626, 0.55026, 0.54406, 0.53768],\
        [0.57928, 0.58200, 0.58434, 0.58635, 0.58811, 0.58968, 0.59111, 0.59244, 0.59369, 0.59489, 0.59605,\
        0.59718, 0.59828, 0.59931, 0.60039, 0.60183, 0.60318, 0.60441, 0.60565, 0.60701, 0.60861, 0.61049,\
        0.61267, 0.61509, 0.61770, 0.62041, 0.62316, 0.62588, 0.62849, 0.63096, 0.63327, 0.63547, 0.63767,\
        0.64003, 0.64277, 0.64610, 0.65014, 0.65488, 0.66033, 0.66652, 0.67364, 0.68187, 0.69115, 0.70002,\
        0.70483, 0.70891, 0.71883, 0.73161, 0.74567, 0.76017, 0.77444, 0.78776, 0.79945, 0.81021, 0.81938,\
        0.82757, 0.83492, 0.84133, 0.84680, 0.85132, 0.85492, 0.85762, 0.85943, 0.86028, 0.86007, 0.85867,\
        0.85604, 0.85226, 0.84750, 0.84196, 0.83584, 0.82924, 0.82225, 0.81491, 0.80735, 0.80004, 0.79440,\
        0.78849, 0.78228, 0.77581, 0.76918, 0.76246, 0.75571, 0.74900, 0.74238, 0.73591, 0.72962, 0.72357,\
        0.71781, 0.71244, 0.70757, 0.70309, 0.69786, 0.69145, 0.68466, 0.67774, 0.67079, 0.66388, 0.65705,\
        0.65036, 0.64385, 0.63755, 0.63151, 0.62576, 0.62035, 0.61528, 0.61055, 0.60612, 0.60186, 0.59766,\
        0.59348, 0.58926, 0.58481, 0.58005, 0.57497, 0.56959, 0.56395, 0.55809, 0.55202, 0.54576, 0.53932],\
        [0.58351, 0.58600, 0.58806, 0.58976, 0.59119, 0.59244, 0.59357, 0.59461, 0.59558, 0.59650, 0.59739,\
        0.59829, 0.59922, 0.60020, 0.60127, 0.60261, 0.60408, 0.60557, 0.60709, 0.60872, 0.61052, 0.61252,\
        0.61474, 0.61714, 0.61967, 0.62225, 0.62481, 0.62725, 0.62950, 0.63146, 0.63310, 0.63443, 0.63555,\
        0.63667, 0.63810, 0.64018, 0.64309, 0.64677, 0.65114, 0.65632, 0.66261, 0.67053, 0.68081, 0.69257,\
        0.69662, 0.70060, 0.71215, 0.72624, 0.74178, 0.75789, 0.77380, 0.78901, 0.80266, 0.81445, 0.82517,\
        0.83460, 0.84276, 0.84973, 0.85555, 0.86022, 0.86381, 0.86640, 0.86811, 0.86890, 0.86860, 0.86698,\
        0.86395, 0.85960, 0.85414, 0.84785, 0.84094, 0.83361, 0.82596, 0.81814, 0.81038, 0.80330, 0.79714,\
        0.79130, 0.78520, 0.77886, 0.77235, 0.76571, 0.75900, 0.75226, 0.74555, 0.73892, 0.73243, 0.72615,\
        0.72015, 0.71449, 0.70917, 0.70395, 0.69831, 0.69199, 0.68529, 0.67843, 0.67153, 0.66465, 0.65787,\
        0.65123, 0.64477, 0.63854, 0.63257, 0.62692, 0.62159, 0.61661, 0.61195, 0.60755, 0.60331, 0.59919,\
        0.59527, 0.59127, 0.58688, 0.58206, 0.57687, 0.57139, 0.56565, 0.55970, 0.55356, 0.54724, 0.54075],\
        [0.58744, 0.58968, 0.59142, 0.59278, 0.59388, 0.59481, 0.59565, 0.59644, 0.59718, 0.59787, 0.59854,\
        0.59926, 0.60010, 0.60109, 0.60225, 0.60364, 0.60522, 0.60690, 0.60867, 0.61053, 0.61252, 0.61465,\
        0.61693, 0.61933, 0.62180, 0.62427, 0.62666, 0.62887, 0.63076, 0.63223, 0.63315, 0.63349, 0.63331,\
        0.63281, 0.63244, 0.63273, 0.63403, 0.63609, 0.63852, 0.64153, 0.64547, 0.65060, 0.65771, 0.66753,\
        0.67735, 0.68801, 0.70158, 0.71721, 0.73475, 0.75286, 0.77057, 0.78754, 0.80327, 0.81757, 0.83031,\
        0.84124, 0.85042, 0.85806, 0.86435, 0.86927, 0.87281, 0.87523, 0.87682, 0.87760, 0.87727, 0.87544,\
        0.87193, 0.86689, 0.86066, 0.85359, 0.84598, 0.83806, 0.82999, 0.82197, 0.81425, 0.80717, 0.80086,\
        0.79471, 0.78843, 0.78205, 0.77556, 0.76895, 0.76222, 0.75540, 0.74854, 0.74170, 0.73496, 0.72837,\
        0.72199, 0.71586, 0.70992, 0.70403, 0.69801, 0.69180, 0.68529, 0.67858, 0.67180, 0.66503, 0.65833,\
        0.65176, 0.64537, 0.63921, 0.63331, 0.62773, 0.62249, 0.61760, 0.61304, 0.60874, 0.60463, 0.60066,\
        0.59707, 0.59326, 0.58882, 0.58385, 0.57852, 0.57291, 0.56707, 0.56104, 0.55484, 0.54847, 0.54195],\
        [0.59104, 0.59299, 0.59438, 0.59538, 0.59612, 0.59674, 0.59731, 0.59788, 0.59845, 0.59898, 0.59949,\
        0.60009, 0.60091, 0.60197, 0.60327, 0.60481, 0.60656, 0.60846, 0.61045, 0.61254, 0.61470, 0.61696,\
        0.61929, 0.62170, 0.62413, 0.62653, 0.62879, 0.63081, 0.63241, 0.63344, 0.63369, 0.63303, 0.63139,\
        0.62894, 0.62616, 0.62394, 0.62311, 0.62308, 0.62245, 0.62206, 0.62245, 0.62345, 0.62472, 0.63334,\
        0.64929, 0.66697, 0.68469, 0.70323, 0.72405, 0.74463, 0.76438, 0.78351, 0.80200, 0.81972, 0.83512,\
        0.84773, 0.85794, 0.86633, 0.87326, 0.87858, 0.88203, 0.88410, 0.88549, 0.88629, 0.88604, 0.88396,\
        0.87983, 0.87395, 0.86683, 0.85899, 0.85076, 0.84239, 0.83405, 0.82593, 0.81822, 0.81110, 0.80453,\
        0.79814, 0.79169, 0.78528, 0.77879, 0.77216, 0.76537, 0.75842, 0.75137, 0.74428, 0.73722, 0.73025,\
        0.72344, 0.71678, 0.71027, 0.70390, 0.69765, 0.69136, 0.68496, 0.67839, 0.67174, 0.66508, 0.65848,\
        0.65199, 0.64567, 0.63957, 0.63373, 0.62820, 0.62302, 0.61822, 0.61378, 0.60970, 0.60591, 0.60238,\
        0.59909, 0.59519, 0.59045, 0.58526, 0.57978, 0.57406, 0.56815, 0.56207, 0.55583, 0.54944, 0.54290],\
        [0.59423, 0.59587, 0.59687, 0.59747, 0.59785, 0.59816, 0.59849, 0.59890, 0.59936, 0.59980, 0.60023,\
        0.60079, 0.60165, 0.60284, 0.60434, 0.60612, 0.60812, 0.61027, 0.61251, 0.61481, 0.61714, 0.61951,\
        0.62190, 0.62432, 0.62672, 0.62906, 0.63124, 0.63314, 0.63456, 0.63528, 0.63501, 0.63347, 0.63043,\
        0.62585, 0.62012, 0.61436, 0.61061, 0.60845, 0.60301, 0.59798, 0.59364, 0.59085, 0.58726, 0.59392,\
        0.61318, 0.63611, 0.65934, 0.68433, 0.70928, 0.73245, 0.75464, 0.77774, 0.80072, 0.82225, 0.84012,\
        0.85433, 0.86549, 0.87454, 0.88229, 0.88843, 0.89167, 0.89313, 0.89424, 0.89492, 0.89478, 0.89227,\
        0.88731, 0.88040, 0.87235, 0.86379, 0.85506, 0.84637, 0.83787, 0.82969, 0.82194, 0.81470, 0.80787,\
        0.80125, 0.79483, 0.78846, 0.78199, 0.77535, 0.76846, 0.76134, 0.75405, 0.74665, 0.73924, 0.73188,\
        0.72462, 0.71749, 0.71050, 0.70362, 0.69712, 0.69090, 0.68452, 0.67803, 0.67147, 0.66491, 0.65840,\
        0.65198, 0.64571, 0.63964, 0.63383, 0.62833, 0.62318, 0.61842, 0.61410, 0.61021, 0.60673, 0.60361,\
        0.60053, 0.59632, 0.59138, 0.58608, 0.58052, 0.57476, 0.56883, 0.56274, 0.55650, 0.55012, 0.54358],\
        [0.59696, 0.59823, 0.59881, 0.59900, 0.59903, 0.59904, 0.59914, 0.59940, 0.59980, 0.60020, 0.60063,\
        0.60129, 0.60229, 0.60371, 0.60551, 0.60763, 0.60996, 0.61242, 0.61493, 0.61744, 0.61993, 0.62239,\
        0.62483, 0.62724, 0.62962, 0.63191, 0.63406, 0.63591, 0.63728, 0.63787, 0.63733, 0.63522, 0.63109,\
        0.62462, 0.61591, 0.60594, 0.59766, 0.59141, 0.58095, 0.56918, 0.55967, 0.55278, 0.54842, 0.55032,\
        0.56225, 0.58597, 0.61797, 0.65621, 0.68845, 0.71438, 0.73896, 0.76892, 0.79974, 0.82469, 0.84514,\
        0.86126, 0.87341, 0.88283, 0.89130, 0.89877, 0.90210, 0.90305, 0.90369, 0.90369, 0.90289, 0.89955,\
        0.89384, 0.88572, 0.87683, 0.86773, 0.85868, 0.84981, 0.84124, 0.83304, 0.82527, 0.81794, 0.81098,\
        0.80433, 0.79799, 0.79156, 0.78516, 0.77850, 0.77149, 0.76414, 0.75656, 0.74883, 0.74106, 0.73330,\
        0.72563, 0.71810, 0.71080, 0.70390, 0.69728, 0.69061, 0.68409, 0.67760, 0.67110, 0.66461, 0.65815,\
        0.65178, 0.64554, 0.63948, 0.63366, 0.62813, 0.62295, 0.61818, 0.61387, 0.61005, 0.60670, 0.60361,\
        0.60025, 0.59619, 0.59140, 0.58618, 0.58069, 0.57498, 0.56910, 0.56305, 0.55685, 0.55050, 0.54400],\
        [0.59908, 0.59992, 0.60007, 0.59992, 0.59964, 0.59939, 0.59927, 0.59936, 0.59967, 0.60012, 0.60070,\
        0.60156, 0.60283, 0.60461, 0.60684, 0.60941, 0.61217, 0.61500, 0.61781, 0.62055, 0.62318, 0.62572,\
        0.62816, 0.63054, 0.63287, 0.63512, 0.63724, 0.63911, 0.64054, 0.64123, 0.64072, 0.63848, 0.63384,\
        0.62614, 0.61507, 0.60150, 0.58822, 0.57422, 0.55650, 0.53712, 0.52217, 0.51237, 0.50548, 0.49950,\
        0.49733, 0.51244, 0.55542, 0.61268, 0.65642, 0.68624, 0.71549, 0.75516, 0.79266, 0.82427, 0.84961,\
        0.86887, 0.88272, 0.89225, 0.90105, 0.90918, 0.91341, 0.91452, 0.91414, 0.91262, 0.90980, 0.90517,\
        0.89812, 0.88930, 0.88003, 0.87070, 0.86154, 0.85264, 0.84408, 0.83592, 0.82818, 0.82085, 0.81387,\
        0.80717, 0.80058, 0.79448, 0.78834, 0.78163, 0.77441, 0.76678, 0.75887, 0.75080, 0.74267, 0.73455,\
        0.72652, 0.71863, 0.71096, 0.70363, 0.69680, 0.69018, 0.68366, 0.67718, 0.67071, 0.66425, 0.65783,\
        0.65147, 0.64523, 0.63914, 0.63327, 0.62766, 0.62239, 0.61750, 0.61307, 0.60916, 0.60576, 0.60265,\
        0.59928, 0.59524, 0.59064, 0.58562, 0.58030, 0.57473, 0.56896, 0.56300, 0.55688, 0.55060, 0.54416],\
        [0.60027, 0.60067, 0.60060, 0.60025, 0.59976, 0.59929, 0.59898, 0.59892, 0.59916, 0.59972, 0.60054,\
        0.60166, 0.60332, 0.60563, 0.60846, 0.61161, 0.61489, 0.61815, 0.62128, 0.62424, 0.62701, 0.62958,\
        0.63200, 0.63430, 0.63653, 0.63870, 0.64078, 0.64270, 0.64428, 0.64520, 0.64502, 0.64309, 0.63857,\
        0.63047, 0.61788, 0.60081, 0.58097, 0.55884, 0.53242, 0.50340, 0.48309, 0.47114, 0.45970, 0.44421,\
        0.43452, 0.44109, 0.47958, 0.54324, 0.60796, 0.64949, 0.69233, 0.73893, 0.78252, 0.82168, 0.85414,\
        0.87767, 0.89481, 0.90572, 0.91503, 0.92243, 0.92631, 0.92688, 0.92507, 0.92145, 0.91619, 0.90919,\
        0.90058, 0.89156, 0.88219, 0.87284, 0.86371, 0.85489, 0.84641, 0.83833, 0.83066, 0.82340, 0.81652,\
        0.80991, 0.80358, 0.79789, 0.79160, 0.78467, 0.77713, 0.76916, 0.76090, 0.75250, 0.74405, 0.73564,\
        0.72732, 0.71914, 0.71116, 0.70346, 0.69640, 0.68983, 0.68332, 0.67684, 0.67036, 0.66391, 0.65748,\
        0.65112, 0.64484, 0.63870, 0.63274, 0.62702, 0.62158, 0.61649, 0.61183, 0.60767, 0.60404, 0.60086,\
        0.59752, 0.59365, 0.58931, 0.58455, 0.57946, 0.57408, 0.56846, 0.56264, 0.55663, 0.55044, 0.54408],\
        [0.59988, 0.60045, 0.60051, 0.60014, 0.59955, 0.59894, 0.59848, 0.59829, 0.59846, 0.59911, 0.60023,\
        0.60165, 0.60386, 0.60695, 0.61057, 0.61443, 0.61830, 0.62202, 0.62549, 0.62867, 0.63153, 0.63411,\
        0.63645, 0.63862, 0.64069, 0.64272, 0.64471, 0.64664, 0.64835, 0.64958, 0.64987, 0.64853, 0.64459,\
        0.63672, 0.62318, 0.60192, 0.57300, 0.54190, 0.50699, 0.46691, 0.44064, 0.42301, 0.40835, 0.38009,\
        0.36656, 0.36840, 0.40660, 0.46901, 0.54953, 0.60448, 0.66142, 0.71738, 0.76800, 0.81624, 0.86063,\
        0.88899, 0.90850, 0.92304, 0.93255, 0.93858, 0.94102, 0.94009, 0.93638, 0.93044, 0.92264, 0.91338,\
        0.90353, 0.89356, 0.88379, 0.87446, 0.86543, 0.85670, 0.84830, 0.84027, 0.83267, 0.82552, 0.81882,\
        0.81256, 0.80673, 0.80096, 0.79457, 0.78736, 0.77945, 0.77111, 0.76253, 0.75385, 0.74516, 0.73653,\
        0.72802, 0.71970, 0.71162, 0.70393, 0.69669, 0.68984, 0.68319, 0.67663, 0.67012, 0.66363, 0.65718,\
        0.65078, 0.64445, 0.63824, 0.63217, 0.62629, 0.62067, 0.61534, 0.61037, 0.60586, 0.60191, 0.59855,\
        0.59533, 0.59172, 0.58765, 0.58316, 0.57831, 0.57313, 0.56769, 0.56201, 0.55612, 0.55004, 0.54378],\
        [0.59928, 0.60003, 0.60021, 0.59990, 0.59928, 0.59859, 0.59807, 0.59782, 0.59789, 0.59842, 0.59962,\
        0.60148, 0.60472, 0.60891, 0.61348, 0.61811, 0.62259, 0.62677, 0.63058, 0.63395, 0.63690, 0.63944,\
        0.64166, 0.64364, 0.64548, 0.64728, 0.64909, 0.65092, 0.65268, 0.65415, 0.65487, 0.65413, 0.65083,\
        0.64335, 0.62924, 0.60416, 0.56233, 0.51490, 0.47143, 0.41936, 0.39107, 0.36178, 0.33546, 0.31033,\
        0.30511, 0.29612, 0.32433, 0.40451, 0.49221, 0.55861, 0.62489, 0.68984, 0.74890, 0.80512, 0.86826,\
        0.90983, 0.92773, 0.94204, 0.95125, 0.95598, 0.95670, 0.95386, 0.94802, 0.93974, 0.92955, 0.91800,\
        0.90582, 0.89494, 0.88533, 0.87601, 0.86700, 0.85827, 0.84985, 0.84179, 0.83417, 0.82705, 0.82048,\
        0.81445, 0.80885, 0.80326, 0.79691, 0.78935, 0.78106, 0.77242, 0.76362, 0.75477, 0.74594, 0.73719,\
        0.72859, 0.72021, 0.71213, 0.70440, 0.69706, 0.69005, 0.68325, 0.67658, 0.66999, 0.66346, 0.65696,\
        0.65051, 0.64412, 0.63781, 0.63163, 0.62562, 0.61980, 0.61425, 0.60901, 0.60418, 0.59994, 0.59653,\
        0.59329, 0.58981, 0.58594, 0.58165, 0.57700, 0.57200, 0.56672, 0.56119, 0.55542, 0.54945, 0.54329],\
        [0.59881, 0.59970, 0.60001, 0.59980, 0.59921, 0.59854, 0.59811, 0.59799, 0.59810, 0.59845, 0.59926,\
        0.60198, 0.60677, 0.61211, 0.61758, 0.62292, 0.62795, 0.63256, 0.63667, 0.64022, 0.64323, 0.64571,\
        0.64777, 0.64952, 0.65108, 0.65257, 0.65409, 0.65568, 0.65732, 0.65884, 0.65979, 0.65943, 0.65652,\
        0.64905, 0.63379, 0.60525, 0.55554, 0.49577, 0.44080, 0.38911, 0.35187, 0.31818, 0.29277, 0.28050,\
        0.27016, 0.26578, 0.29828, 0.37846, 0.45587, 0.51976, 0.58933, 0.66298, 0.73194, 0.79236, 0.86301,\
        0.91643, 0.94311, 0.95943, 0.96907, 0.97308, 0.97233, 0.96762, 0.95971, 0.94930, 0.93706, 0.92368,\
        0.91022, 0.89811, 0.88758, 0.87794, 0.86870, 0.85977, 0.85116, 0.84292, 0.83513, 0.82788, 0.82123,\
        0.81518, 0.80963, 0.80424, 0.79797, 0.79011, 0.78164, 0.77291, 0.76407, 0.75519, 0.74634, 0.73757,\
        0.72897, 0.72059, 0.71253, 0.70484, 0.69747, 0.69033, 0.68340, 0.67664, 0.66999, 0.66340, 0.65685,\
        0.65034, 0.64388, 0.63750, 0.63122, 0.62507, 0.61911, 0.61339, 0.60800, 0.60307, 0.59874, 0.59511,\
        0.59172, 0.58819, 0.58437, 0.58018, 0.57565, 0.57079, 0.56564, 0.56023, 0.55458, 0.54871, 0.54264],\
        [0.59845, 0.59952, 0.60003, 0.60004, 0.59960, 0.59906, 0.59896, 0.59931, 0.59986, 0.60069, 0.60236,\
        0.60586, 0.61112, 0.61708, 0.62315, 0.62901, 0.63448, 0.63945, 0.64382, 0.64756, 0.65062, 0.65305,\
        0.65495, 0.65646, 0.65775, 0.65891, 0.66003, 0.66124, 0.66258, 0.66392, 0.66482, 0.66451, 0.66173,\
        0.65420, 0.63787, 0.60668, 0.55799, 0.49789, 0.43742, 0.38455, 0.34257, 0.31467, 0.29992, 0.28500,\
        0.26797, 0.26361, 0.29928, 0.38041, 0.44559, 0.49759, 0.56171, 0.63928, 0.71503, 0.78256, 0.85278,\
        0.91174, 0.95042, 0.97303, 0.98486, 0.98891, 0.98707, 0.98073, 0.97100, 0.95878, 0.94490, 0.93020,\
        0.91563, 0.90229, 0.89087, 0.88044, 0.87066, 0.86130, 0.85230, 0.84370, 0.83556, 0.82797, 0.82100,\
        0.81466, 0.80880, 0.80307, 0.79673, 0.78922, 0.78103, 0.77252, 0.76384, 0.75509, 0.74635, 0.73767,\
        0.72912, 0.72077, 0.71271, 0.70508, 0.69779, 0.69052, 0.68356, 0.67678, 0.67011, 0.66348, 0.65688,\
        0.65031, 0.64378, 0.63732, 0.63096, 0.62474, 0.61868, 0.61285, 0.60736, 0.60242, 0.59811, 0.59424,\
        0.59062, 0.58694, 0.58303, 0.57884, 0.57435, 0.56957, 0.56450, 0.55919, 0.55363, 0.54785, 0.54187],\
        [0.59828, 0.59959, 0.60037, 0.60071, 0.60067, 0.60057, 0.60106, 0.60206, 0.60339, 0.60520, 0.60797,\
        0.61214, 0.61762, 0.62383, 0.63021, 0.63639, 0.64217, 0.64740, 0.65202, 0.65595, 0.65913, 0.66156,\
        0.66334, 0.66470, 0.66581, 0.66672, 0.66745, 0.66817, 0.66907, 0.67007, 0.67067, 0.67010, 0.66736,\
        0.66042, 0.64529, 0.61632, 0.56720, 0.50484, 0.44409, 0.38945, 0.34572, 0.32006, 0.30928, 0.30067,\
        0.28336, 0.27713, 0.31515, 0.39214, 0.44296, 0.48443, 0.55166, 0.63172, 0.70269, 0.77668, 0.85040,\
        0.91373, 0.95843, 0.98550, 0.99916, 1.00323, 1.00040, 0.99262, 0.98131, 0.96759, 0.95243, 0.93673,\
        0.92138, 0.90727, 0.89461, 0.88325, 0.87280, 0.86284, 0.85329, 0.84415, 0.83550, 0.82739, 0.81990,\
        0.81306, 0.80674, 0.80056, 0.79417, 0.78714, 0.77947, 0.77138, 0.76304, 0.75456, 0.74603, 0.73752,\
        0.72908, 0.72079, 0.71269, 0.70483, 0.69744, 0.69054, 0.68375, 0.67703, 0.67035, 0.66369, 0.65704,\
        0.65041, 0.64382, 0.63730, 0.63088, 0.62461, 0.61853, 0.61267, 0.60709, 0.60187, 0.59759, 0.59377,\
        0.58993, 0.58601, 0.58194, 0.57766, 0.57314, 0.56837, 0.56335, 0.55810, 0.55261, 0.54691, 0.54101],\
        [0.59835, 0.59997, 0.60116, 0.60198, 0.60260, 0.60330, 0.60446, 0.60615, 0.60836, 0.61120, 0.61493,\
        0.61975, 0.62555, 0.63195, 0.63851, 0.64488, 0.65083, 0.65625, 0.66111, 0.66532, 0.66873, 0.67127,\
        0.67306, 0.67443, 0.67565, 0.67663, 0.67709, 0.67731, 0.67775, 0.67841, 0.67867, 0.67749, 0.67463,\
        0.66899, 0.65647, 0.62893, 0.57986, 0.51788, 0.45802, 0.39742, 0.35101, 0.32462, 0.31431, 0.30966,\
        0.30135, 0.29327, 0.33765, 0.40467, 0.44317, 0.47815, 0.54401, 0.62691, 0.70357, 0.78061, 0.85985,\
        0.92613, 0.97204, 0.99948, 1.01285, 1.01602, 1.01194, 1.00274, 0.99003, 0.97507, 0.95890, 0.94243,\
        0.92642, 0.91143, 0.89790, 0.88609, 0.87494, 0.86428, 0.85407, 0.84432, 0.83506, 0.82633, 0.81819,\
        0.81076, 0.80406, 0.79772, 0.79131, 0.78459, 0.77739, 0.76977, 0.76184, 0.75371, 0.74547, 0.73719,\
        0.72894, 0.72079, 0.71278, 0.70501, 0.69771, 0.69084, 0.68412, 0.67743, 0.67074, 0.66404, 0.65733,\
        0.65063, 0.64398, 0.63741, 0.63095, 0.62467, 0.61861, 0.61282, 0.60739, 0.60246, 0.59808, 0.59376,\
        0.58953, 0.58531, 0.58103, 0.57661, 0.57202, 0.56722, 0.56222, 0.55699, 0.55156, 0.54591, 0.54007],\
        [0.59868, 0.60071, 0.60239, 0.60384, 0.60522, 0.60677, 0.60872, 0.61121, 0.61429, 0.61807, 0.62266,\
        0.62812, 0.63431, 0.64095, 0.64767, 0.65414, 0.66017, 0.66570, 0.67077, 0.67538, 0.67926, 0.68216,\
        0.68416, 0.68580, 0.68763, 0.68940, 0.68991, 0.68987, 0.69005, 0.69045, 0.69078, 0.68850, 0.68539,\
        0.68138, 0.67416, 0.64889, 0.59616, 0.53797, 0.48240, 0.41494, 0.36139, 0.33303, 0.32307, 0.32130,\
        0.31969, 0.32365, 0.36263, 0.41546, 0.44781, 0.48432, 0.54987, 0.63010, 0.71256, 0.79745, 0.88718,\
        0.95038, 0.99110, 1.01484, 1.02567, 1.02690, 1.02116, 1.01051, 0.99655, 0.98058, 0.96364, 0.94661,\
        0.93022, 0.91503, 0.90138, 0.88874, 0.87680, 0.86539, 0.85454, 0.84422, 0.83439, 0.82505, 0.81624,\
        0.80810, 0.80095, 0.79464, 0.78844, 0.78201, 0.77519, 0.76798, 0.76045, 0.75268, 0.74476, 0.73677,\
        0.72878, 0.72084, 0.71305, 0.70553, 0.69835, 0.69145, 0.68472, 0.67802, 0.67128, 0.66451, 0.65772,\
        0.65095, 0.64423, 0.63760, 0.63111, 0.62483, 0.61880, 0.61311, 0.60784, 0.60303, 0.59842, 0.59378,\
        0.58920, 0.58470, 0.58020, 0.57564, 0.57096, 0.56612, 0.56110, 0.55588, 0.55048, 0.54487, 0.53908],\
        [0.59925, 0.60173, 0.60398, 0.60611, 0.60828, 0.61066, 0.61345, 0.61676, 0.62070, 0.62532, 0.63069,\
        0.63676, 0.64340, 0.65034, 0.65725, 0.66381, 0.66983, 0.67530, 0.68048, 0.68562, 0.69031, 0.69391,\
        0.69659, 0.69902, 0.70213, 0.70545, 0.70701, 0.70759, 0.70800, 0.70828, 0.70842, 0.70572, 0.70333,\
        0.70073, 0.70230, 0.68538, 0.62488, 0.55532, 0.50597, 0.44153, 0.38458, 0.35554, 0.34606, 0.34723,\
        0.34976, 0.35536, 0.38065, 0.42161, 0.45966, 0.50416, 0.56905, 0.64342, 0.72877, 0.82728, 0.92075,\
        0.97676, 1.01019, 1.02881, 1.03616, 1.03495, 1.02740, 1.01536, 1.00035, 0.98359, 0.96610, 0.94873,\
        0.93216, 0.91696, 0.90341, 0.89049, 0.87779, 0.86584, 0.85456, 0.84384, 0.83362, 0.82385, 0.81455,\
        0.80581, 0.79820, 0.79198, 0.78594, 0.77971, 0.77315, 0.76624, 0.75903, 0.75159, 0.74399, 0.73631,\
        0.72860, 0.72094, 0.71339, 0.70605, 0.69900, 0.69220, 0.68550, 0.67876, 0.67195, 0.66508, 0.65819,\
        0.65132, 0.64451, 0.63782, 0.63129, 0.62498, 0.61894, 0.61326, 0.60799, 0.60314, 0.59839, 0.59352,\
        0.58874, 0.58405, 0.57939, 0.57470, 0.56993, 0.56504, 0.55999, 0.55478, 0.54938, 0.54381, 0.53805],\
        [0.60000, 0.60297, 0.60581, 0.60862, 0.61154, 0.61473, 0.61833, 0.62245, 0.62718, 0.63256, 0.63862,\
        0.64528, 0.65238, 0.65967, 0.66682, 0.67348, 0.67943, 0.68465, 0.68958, 0.69514, 0.70093, 0.70561,\
        0.70981, 0.71420, 0.71923, 0.72426, 0.72803, 0.73026, 0.73136, 0.73173, 0.73142, 0.73038, 0.73040,\
        0.73283, 0.73912, 0.73350, 0.69234, 0.60332, 0.53549, 0.48512, 0.43740, 0.40631, 0.39340, 0.39293,\
        0.39358, 0.39104, 0.39719, 0.43979, 0.48769, 0.54300, 0.59709, 0.66688, 0.76708, 0.86589, 0.94465,\
        0.99477, 1.02371, 1.03835, 1.04264, 1.03919, 1.03003, 1.01683, 1.00101, 0.98373, 0.96595, 0.94842,\
        0.93175, 0.91637, 0.90256, 0.88981, 0.87729, 0.86536, 0.85401, 0.84319, 0.83284, 0.82293, 0.81352,\
        0.80477, 0.79705, 0.79041, 0.78417, 0.77789, 0.77141, 0.76467, 0.75769, 0.75051, 0.74320, 0.73582,\
        0.72842, 0.72106, 0.71377, 0.70659, 0.69963, 0.69311, 0.68645, 0.67961, 0.67267, 0.66567, 0.65865,\
        0.65167, 0.64478, 0.63801, 0.63141, 0.62504, 0.61895, 0.61317, 0.60774, 0.60265, 0.59777, 0.59289,\
        0.58806, 0.58329, 0.57854, 0.57377, 0.56893, 0.56398, 0.55891, 0.55368, 0.54829, 0.54272, 0.53699],\
        [0.60084, 0.60430, 0.60772, 0.61119, 0.61483, 0.61876, 0.62310, 0.62796, 0.63340, 0.63945, 0.64610,\
        0.65328, 0.66082, 0.66846, 0.67588, 0.68268, 0.68855, 0.69347, 0.69784, 0.70330, 0.70987, 0.71616,\
        0.72247, 0.72929, 0.73673, 0.74420, 0.75049, 0.75459, 0.75648, 0.75698, 0.75685, 0.75694, 0.75896,\
        0.76476, 0.77353, 0.77665, 0.75868, 0.69215, 0.60262, 0.54391, 0.50438, 0.48357, 0.46903, 0.45294,\
        0.44295, 0.43740, 0.44672, 0.48710, 0.53842, 0.59999, 0.65232, 0.72879, 0.82787, 0.90374, 0.96441,\
        1.00604, 1.03061, 1.04220, 1.04417, 1.03901, 1.02864, 1.01463, 0.99832, 0.98083, 0.96306, 0.94568,\
        0.92913, 0.91371, 0.89966, 0.88730, 0.87549, 0.86400, 0.85292, 0.84227, 0.83204, 0.82225, 0.81297,\
        0.80443, 0.79671, 0.78965, 0.78305, 0.77658, 0.77003, 0.76332, 0.75646, 0.74947, 0.74239, 0.73528,\
        0.72819, 0.72115, 0.71421, 0.70742, 0.70097, 0.69433, 0.68744, 0.68044, 0.67334, 0.66619, 0.65904,\
        0.65195, 0.64496, 0.63811, 0.63144, 0.62500, 0.61881, 0.61289, 0.60725, 0.60189, 0.59687, 0.59204,\
        0.58724, 0.58245, 0.57765, 0.57282, 0.56793, 0.56294, 0.55783, 0.55259, 0.54719, 0.54163, 0.53591],\
        [0.60169, 0.60564, 0.60960, 0.61367, 0.61796, 0.62256, 0.62757, 0.63307, 0.63910, 0.64569, 0.65281,\
        0.66039, 0.66827, 0.67620, 0.68384, 0.69077, 0.69651, 0.70110, 0.70537, 0.71073, 0.71753, 0.72510,\
        0.73334, 0.74241, 0.75229, 0.76244, 0.77152, 0.77739, 0.77962, 0.78011, 0.78008, 0.78005, 0.78153,\
        0.78634, 0.79380, 0.79828, 0.78789, 0.74930, 0.68296, 0.61198, 0.56622, 0.54570, 0.52750, 0.50560,\
        0.49131, 0.48948, 0.50107, 0.53831, 0.59596, 0.65947, 0.72232, 0.80421, 0.87606, 0.93352, 0.97935,\
        1.01206, 1.03180, 1.04057, 1.04072, 1.03436, 1.02323, 1.00879, 0.99232, 0.97491, 0.95750, 0.94070,\
        0.92480, 0.90991, 0.89672, 0.88451, 0.87306, 0.86202, 0.85136, 0.84108, 0.83118, 0.82168, 0.81263,\
        0.80407, 0.79634, 0.78925, 0.78237, 0.77563, 0.76891, 0.76214, 0.75530, 0.74842, 0.74152, 0.73464,\
        0.72782, 0.72110, 0.71450, 0.70805, 0.70164, 0.69508, 0.68820, 0.68107, 0.67383, 0.66654, 0.65927,\
        0.65208, 0.64500, 0.63807, 0.63134, 0.62482, 0.61855, 0.61253, 0.60676, 0.60125, 0.59612, 0.59123,\
        0.58640, 0.58159, 0.57675, 0.57188, 0.56694, 0.56191, 0.55677, 0.55150, 0.54609, 0.54053, 0.53481],\
        [0.60248, 0.60687, 0.61134, 0.61596, 0.62082, 0.62600, 0.63157, 0.63760, 0.64410, 0.65108, 0.65851,\
        0.66632, 0.67436, 0.68240, 0.69006, 0.69679, 0.70218, 0.70628, 0.71064, 0.71622, 0.72335, 0.73184,\
        0.74148, 0.75220, 0.76386, 0.77603, 0.78763, 0.79512, 0.79671, 0.79721, 0.79738, 0.79648, 0.79562,\
        0.79682, 0.80004, 0.80273, 0.79788, 0.77733, 0.73896, 0.68673, 0.63556, 0.60340, 0.56896, 0.54487,\
        0.53528, 0.54174, 0.56264, 0.61197, 0.68130, 0.73754, 0.79243, 0.85407, 0.90763, 0.95202, 0.98752,\
        1.01282, 1.02801, 1.03415, 1.03281, 1.02564, 1.01413, 0.99961, 0.98326, 0.96620, 0.94942, 0.93366,\
        0.91917, 0.90560, 0.89283, 0.88122, 0.87012, 0.85953, 0.84937, 0.83961, 0.83019, 0.82111, 0.81242,\
        0.80429, 0.79661, 0.78907, 0.78188, 0.77486, 0.76792, 0.76101, 0.75413, 0.74728, 0.74050, 0.73380,\
        0.72721, 0.72075, 0.71443, 0.70822, 0.70195, 0.69550, 0.68860, 0.68138, 0.67401, 0.66661, 0.65925,\
        0.65198, 0.64484, 0.63786, 0.63108, 0.62452, 0.61820, 0.61213, 0.60632, 0.60079, 0.59555, 0.59053,\
        0.58562, 0.58075, 0.57587, 0.57096, 0.56598, 0.56091, 0.55573, 0.55044, 0.54500, 0.53943, 0.53370],\
        [0.60314, 0.60793, 0.61284, 0.61794, 0.62330, 0.62897, 0.63501, 0.64144, 0.64827, 0.65550, 0.66306,\
        0.67087, 0.67881, 0.68667, 0.69409, 0.70024, 0.70464, 0.70854, 0.71309, 0.71905, 0.72669, 0.73587,\
        0.74637, 0.75794, 0.77025, 0.78270, 0.79413, 0.80163, 0.80410, 0.80548, 0.80601, 0.80498, 0.80294,\
        0.80115, 0.80010, 0.80017, 0.79877, 0.78956, 0.76926, 0.73870, 0.70505, 0.67251, 0.62978, 0.60165,\
        0.58796, 0.60062, 0.63540, 0.70754, 0.76150, 0.80757, 0.84733, 0.88894, 0.92849, 0.96213, 0.98925,\
        1.00861, 1.01992, 1.02376, 1.02120, 1.01352, 1.00197, 0.98768, 0.97170, 0.95509, 0.93905, 0.92455,\
        0.91173, 0.90042, 0.88868, 0.87713, 0.86645, 0.85642, 0.84691, 0.83780, 0.82899, 0.82043, 0.81209,\
        0.80396, 0.79622, 0.78878, 0.78140, 0.77411, 0.76692, 0.75982, 0.75284, 0.74597, 0.73925, 0.73267,\
        0.72625, 0.71998, 0.71388, 0.70790, 0.70202, 0.69560, 0.68852, 0.68121, 0.67377, 0.66632, 0.65892,\
        0.65161, 0.64444, 0.63744, 0.63064, 0.62406, 0.61773, 0.61166, 0.60587, 0.60034, 0.59502, 0.58989,\
        0.58490, 0.57996, 0.57503, 0.57007, 0.56504, 0.55993, 0.55472, 0.54938, 0.54392, 0.53833, 0.53259],\
        [0.60360, 0.60874, 0.61404, 0.61956, 0.62533, 0.63141, 0.63782, 0.64455, 0.65160, 0.65891, 0.66643,\
        0.67403, 0.68157, 0.68882, 0.69544, 0.70070, 0.70436, 0.70801, 0.71282, 0.71926, 0.72743, 0.73715,\
        0.74808, 0.75981, 0.77180, 0.78323, 0.79279, 0.79908, 0.80343, 0.80655, 0.80818, 0.80815, 0.80686,\
        0.80509, 0.80356, 0.80274, 0.80174, 0.79783, 0.78851, 0.77181, 0.75198, 0.73326, 0.71067, 0.68270,\
        0.66391, 0.66178, 0.69164, 0.76302, 0.80899, 0.84534, 0.87612, 0.90663, 0.93743, 0.96419, 0.98540,\
        1.00021, 1.00837, 1.01026, 1.00673, 0.99880, 0.98752, 0.97382, 0.95849, 0.94239, 0.92684, 0.91363,\
        0.90242, 0.89194, 0.88178, 0.87155, 0.86178, 0.85259, 0.84390, 0.83558, 0.82751, 0.81958, 0.81170,\
        0.80386, 0.79612, 0.78847, 0.78083, 0.77324, 0.76576, 0.75844, 0.75130, 0.74437, 0.73767, 0.73118,\
        0.72487, 0.71872, 0.71272, 0.70688, 0.70098, 0.69465, 0.68771, 0.68044, 0.67303, 0.66560, 0.65822,\
        0.65093, 0.64378, 0.63679, 0.63001, 0.62344, 0.61711, 0.61105, 0.60529, 0.59982, 0.59443, 0.58924,\
        0.58419, 0.57921, 0.57423, 0.56922, 0.56415, 0.55899, 0.55373, 0.54835, 0.54285, 0.53723, 0.53147],\
        [0.60382, 0.60927, 0.61490, 0.62076, 0.62689, 0.63330, 0.63999, 0.64694, 0.65410, 0.66140, 0.66874,\
        0.67596, 0.68289, 0.68927, 0.69476, 0.69900, 0.70174, 0.70537, 0.71045, 0.71734, 0.72602, 0.73613,\
        0.74719, 0.75870, 0.77004, 0.78040, 0.78890, 0.79512, 0.80001, 0.80424, 0.80727, 0.80888, 0.80946,\
        0.80961, 0.80999, 0.81094, 0.81195, 0.81149, 0.80792, 0.79714, 0.78314, 0.77036, 0.75551, 0.74015,\
        0.73018, 0.73068, 0.76586, 0.82446, 0.85576, 0.87652, 0.89319, 0.91495, 0.93949, 0.96095, 0.97762,\
        0.98881, 0.99437, 0.99462, 0.99028, 0.98230, 0.97160, 0.95894, 0.94475, 0.92950, 0.91408, 0.90156,\
        0.89227, 0.88291, 0.87378, 0.86481, 0.85618, 0.84802, 0.84031, 0.83293, 0.82571, 0.81849, 0.81117,\
        0.80370, 0.79597, 0.78803, 0.78005, 0.77211, 0.76430, 0.75671, 0.74939, 0.74238, 0.73568, 0.72926,\
        0.72306, 0.71698, 0.71095, 0.70502, 0.69930, 0.69303, 0.68619, 0.67906, 0.67177, 0.66443, 0.65712,\
        0.64991, 0.64283, 0.63591, 0.62918, 0.62265, 0.61634, 0.61028, 0.60449, 0.59898, 0.59369, 0.58855,\
        0.58350, 0.57849, 0.57347, 0.56841, 0.56329, 0.55808, 0.55276, 0.54734, 0.54179, 0.53613, 0.53034],\
        [0.60378, 0.60947, 0.61537, 0.62153, 0.62795, 0.63464, 0.64156, 0.64866, 0.65588, 0.66311, 0.67022,\
        0.67702, 0.68330, 0.68880, 0.69322, 0.69635, 0.69871, 0.70197, 0.70688, 0.71412, 0.72325, 0.73359,\
        0.74461, 0.75576, 0.76648, 0.77616, 0.78431, 0.79083, 0.79624, 0.80115, 0.80520, 0.80857, 0.81146,\
        0.81424, 0.81741, 0.82127, 0.82561, 0.82954, 0.83124, 0.82785, 0.82067, 0.81031, 0.79738, 0.78868,\
        0.78803, 0.79864, 0.83439, 0.87171, 0.89163, 0.90243, 0.91023, 0.92294, 0.93955, 0.95530, 0.96773,\
        0.97574, 0.97901, 0.97777, 0.97270, 0.96475, 0.95489, 0.94377, 0.93150, 0.91800, 0.90399, 0.89161,\
        0.88240, 0.87381, 0.86548, 0.85746, 0.84988, 0.84279, 0.83615, 0.82980, 0.82352, 0.81708, 0.81030,\
        0.80306, 0.79534, 0.78720, 0.77888, 0.77056, 0.76240, 0.75452, 0.74700, 0.73990, 0.73320, 0.72687,\
        0.72081, 0.71486, 0.70882, 0.70264, 0.69668, 0.69065, 0.68408, 0.67714, 0.67001, 0.66281, 0.65564,\
        0.64855, 0.64160, 0.63479, 0.62816, 0.62172, 0.61547, 0.60943, 0.60362, 0.59813, 0.59294, 0.58787,\
        0.58284, 0.57782, 0.57277, 0.56766, 0.56248, 0.55720, 0.55182, 0.54634, 0.54074, 0.53503, 0.52920],\
        [0.60344, 0.60933, 0.61546, 0.62186, 0.62854, 0.63546, 0.64257, 0.64980, 0.65705, 0.66421, 0.67112,\
        0.67758, 0.68335, 0.68820, 0.69191, 0.69448, 0.69644, 0.69876, 0.70324, 0.71075, 0.72013, 0.73047,\
        0.74123, 0.75195, 0.76216, 0.77145, 0.77953, 0.78640, 0.79236, 0.79766, 0.80268, 0.80777, 0.81293,\
        0.81833, 0.82429, 0.83112, 0.83890, 0.84735, 0.85585, 0.86354, 0.86785, 0.86621, 0.86223, 0.86161,\
        0.86726, 0.87849, 0.89666, 0.91026, 0.91626, 0.91891, 0.92165, 0.92799, 0.93785, 0.94835, 0.95692,\
        0.96209, 0.96329, 0.96066, 0.95484, 0.94682, 0.93783, 0.92870, 0.91907, 0.90791, 0.89565, 0.88396,\
        0.87395, 0.86519, 0.85718, 0.84980, 0.84308, 0.83700, 0.83144, 0.82618, 0.82091, 0.81531, 0.80907,\
        0.80197, 0.79419, 0.78585, 0.77718, 0.76846, 0.75993, 0.75174, 0.74402, 0.73682, 0.73014, 0.72395,\
        0.71811, 0.71245, 0.70668, 0.70052, 0.69439, 0.68817, 0.68162, 0.67480, 0.66782, 0.66079, 0.65379,\
        0.64688, 0.64009, 0.63346, 0.62698, 0.62069, 0.61457, 0.60863, 0.60288, 0.59745, 0.59233, 0.58729,\
        0.58227, 0.57722, 0.57213, 0.56696, 0.56171, 0.55636, 0.55091, 0.54536, 0.53970, 0.53393, 0.52806],\
        [0.60281, 0.60884, 0.61515, 0.62177, 0.62868, 0.63581, 0.64308, 0.65043, 0.65773, 0.66486, 0.67165,\
        0.67792, 0.68343, 0.68798, 0.69141, 0.69380, 0.69556, 0.69746, 0.70115, 0.70860, 0.71767, 0.72755,\
        0.73778, 0.74795, 0.75771, 0.76674, 0.77485, 0.78203, 0.78843, 0.79432, 0.80020, 0.80670, 0.81380,\
        0.82134, 0.82950, 0.83858, 0.84882, 0.86030, 0.87311, 0.88738, 0.90060, 0.90500, 0.90962, 0.91592,\
        0.92271, 0.92758, 0.93069, 0.93068, 0.92790, 0.92503, 0.92432, 0.92719, 0.93315, 0.94007, 0.94575,\
        0.94865, 0.94808, 0.94415, 0.93757, 0.92929, 0.92078, 0.91387, 0.90688, 0.89898, 0.88730, 0.87601,\
        0.86582, 0.85682, 0.84888, 0.84190, 0.83587, 0.83071, 0.82621, 0.82205, 0.81782, 0.81310, 0.80753,\
        0.80074, 0.79272, 0.78394, 0.77483, 0.76569, 0.75679, 0.74830, 0.74037, 0.73308, 0.72644, 0.72040,\
        0.71481, 0.70953, 0.70435, 0.69855, 0.69207, 0.68551, 0.67886, 0.67209, 0.66524, 0.65839, 0.65159,\
        0.64490, 0.63833, 0.63191, 0.62566, 0.61957, 0.61365, 0.60792, 0.60238, 0.59706, 0.59192, 0.58685,\
        0.58180, 0.57671, 0.57156, 0.56632, 0.56099, 0.55555, 0.55002, 0.54438, 0.53865, 0.53282, 0.52689],\
        [0.60187, 0.60798, 0.61446, 0.62131, 0.62843, 0.63575, 0.64317, 0.65062, 0.65799, 0.66514, 0.67194,\
        0.67820, 0.68371, 0.68829, 0.69183, 0.69439, 0.69640, 0.69873, 0.70239, 0.70839, 0.71629, 0.72521,\
        0.73463, 0.74415, 0.75345, 0.76226, 0.77043, 0.77793, 0.78485, 0.79139, 0.79796, 0.80563, 0.81410,\
        0.82296, 0.83238, 0.84259, 0.85374, 0.86579, 0.87838, 0.89028, 0.89904, 0.90533, 0.91408, 0.92491,\
        0.93354, 0.93747, 0.93662, 0.93200, 0.92594, 0.92102, 0.91904, 0.92086, 0.92541, 0.93064, 0.93465,\
        0.93603, 0.93408, 0.92900, 0.92186, 0.91354, 0.90468, 0.90013, 0.89435, 0.88725, 0.87767, 0.86725,\
        0.85733, 0.84834, 0.84044, 0.83374, 0.82828, 0.82395, 0.82046, 0.81737, 0.81416, 0.81025, 0.80518,\
        0.79875, 0.79040, 0.78119, 0.77168, 0.76217, 0.75292, 0.74416, 0.73603, 0.72866, 0.72206, 0.71618,\
        0.71081, 0.70569, 0.70078, 0.69531, 0.68892, 0.68232, 0.67566, 0.66897, 0.66228, 0.65563, 0.64907,\
        0.64262, 0.63633, 0.63018, 0.62419, 0.61836, 0.61268, 0.60720, 0.60197, 0.59677, 0.59159, 0.58650,\
        0.58141, 0.57627, 0.57105, 0.56573, 0.56030, 0.55477, 0.54914, 0.54341, 0.53759, 0.53169, 0.52570],\
        [0.60053, 0.60675, 0.61347, 0.62054, 0.62786, 0.63534, 0.64289, 0.65043, 0.65788, 0.66511, 0.67200,\
        0.67840, 0.68413, 0.68902, 0.69294, 0.69588, 0.69819, 0.70060, 0.70405, 0.70903, 0.71561, 0.72336,\
        0.73183, 0.74063, 0.74945, 0.75807, 0.76631, 0.77416, 0.78167, 0.78904, 0.79665, 0.80493, 0.81379,\
        0.82306, 0.83277, 0.84302, 0.85375, 0.86466, 0.87498, 0.88341, 0.88884, 0.89259, 0.90020, 0.91197,\
        0.92186, 0.92664, 0.92601, 0.92063, 0.91435, 0.90999, 0.90832, 0.91100, 0.91584, 0.92070, 0.92407,\
        0.92472, 0.92188, 0.91576, 0.90811, 0.90090, 0.89304, 0.88745, 0.88223, 0.87547, 0.86714, 0.85785,\
        0.84851, 0.83972, 0.83189, 0.82537, 0.82038, 0.81681, 0.81425, 0.81215, 0.80989, 0.80666, 0.80166,\
        0.79493, 0.78669, 0.77740, 0.76766, 0.75788, 0.74836, 0.73935, 0.73106, 0.72360, 0.71703, 0.71135,\
        0.70627, 0.70117, 0.69611, 0.69078, 0.68480, 0.67844, 0.67195, 0.66543, 0.65894, 0.65253, 0.64623,\
        0.64009, 0.63410, 0.62828, 0.62261, 0.61707, 0.61166, 0.60635, 0.60118, 0.59622, 0.59124, 0.58620,\
        0.58110, 0.57590, 0.57059, 0.56517, 0.55964, 0.55400, 0.54826, 0.54243, 0.53652, 0.53053, 0.52448],\
        [0.59897, 0.60540, 0.61236, 0.61962, 0.62708, 0.63466, 0.64228, 0.64988, 0.65740, 0.66472, 0.67176,\
        0.67838, 0.68445, 0.68982, 0.69431, 0.69771, 0.70020, 0.70251, 0.70545, 0.70956, 0.71502, 0.72167,\
        0.72921, 0.73731, 0.74569, 0.75410, 0.76241, 0.77057, 0.77862, 0.78672, 0.79506, 0.80367, 0.81249,\
        0.82151, 0.83081, 0.84037, 0.85000, 0.85926, 0.86742, 0.87370, 0.87791, 0.88148, 0.88714, 0.89576,\
        0.90345, 0.90850, 0.90972, 0.90453, 0.89940, 0.89732, 0.89685, 0.90075, 0.90601, 0.91092, 0.91426,\
        0.91485, 0.91176, 0.90514, 0.89669, 0.88986, 0.88300, 0.87691, 0.87113, 0.86459, 0.85698, 0.84856,\
        0.83983, 0.83131, 0.82350, 0.81701, 0.81238, 0.80955, 0.80775, 0.80640, 0.80492, 0.80224, 0.79735,\
        0.79031, 0.78198, 0.77270, 0.76289, 0.75294, 0.74321, 0.73400, 0.72559, 0.71810, 0.71151, 0.70591,\
        0.70152, 0.69645, 0.69127, 0.68588, 0.68012, 0.67404, 0.66779, 0.66150, 0.65526, 0.64911, 0.64312,\
        0.63730, 0.63166, 0.62621, 0.62091, 0.61574, 0.61065, 0.60560, 0.60060, 0.59578, 0.59094, 0.58596,\
        0.58084, 0.57558, 0.57018, 0.56465, 0.55900, 0.55323, 0.54737, 0.54143, 0.53541, 0.52934, 0.52321],\
        [0.59796, 0.60437, 0.61136, 0.61867, 0.62616, 0.63374, 0.64136, 0.64897, 0.65651, 0.66391, 0.67109,\
        0.67794, 0.68435, 0.69021, 0.69533, 0.69939, 0.70182, 0.70382, 0.70620, 0.70954, 0.71407, 0.71981,\
        0.72656, 0.73405, 0.74202, 0.75025, 0.75857, 0.76692, 0.77533, 0.78387, 0.79258, 0.80135, 0.80987,\
        0.81830, 0.82682, 0.83540, 0.84378, 0.85154, 0.85819, 0.86338, 0.86730, 0.87098, 0.87573, 0.88173,\
        0.88725, 0.89183, 0.89413, 0.89098, 0.88771, 0.88681, 0.88804, 0.89176, 0.89673, 0.90148, 0.90481,\
        0.90581, 0.90280, 0.89616, 0.88805, 0.88049, 0.87366, 0.86736, 0.86128, 0.85485, 0.84777, 0.84003,\
        0.83189, 0.82369, 0.81588, 0.80918, 0.80474, 0.80267, 0.80139, 0.80029, 0.79918, 0.79641, 0.79157,\
        0.78475, 0.77654, 0.76737, 0.75760, 0.74759, 0.73770, 0.72833, 0.71985, 0.71247, 0.70603, 0.70023,\
        0.69586, 0.69128, 0.68618, 0.68082, 0.67519, 0.66932, 0.66331, 0.65726, 0.65128, 0.64542, 0.63974,\
        0.63428, 0.62903, 0.62399, 0.61912, 0.61437, 0.60968, 0.60503, 0.60041, 0.59564, 0.59078, 0.58580,\
        0.58064, 0.57530, 0.56980, 0.56414, 0.55835, 0.55245, 0.54645, 0.54039, 0.53426, 0.52810, 0.52189],\
        [0.59727, 0.60361, 0.61054, 0.61777, 0.62515, 0.63261, 0.64013, 0.64767, 0.65518, 0.66259, 0.66985,\
        0.67685, 0.68350, 0.68968, 0.69520, 0.69965, 0.70223, 0.70403, 0.70596, 0.70867, 0.71249, 0.71753,\
        0.72368, 0.73070, 0.73835, 0.74638, 0.75464, 0.76305, 0.77159, 0.78024, 0.78899, 0.79765, 0.80579,\
        0.81351, 0.82116, 0.82879, 0.83609, 0.84272, 0.84840, 0.85306, 0.85696, 0.86072, 0.86495, 0.86966,\
        0.87424, 0.87810, 0.88008, 0.87945, 0.87825, 0.87833, 0.88014, 0.88355, 0.88784, 0.89195, 0.89490,\
        0.89577, 0.89265, 0.88653, 0.87903, 0.87161, 0.86477, 0.85846, 0.85241, 0.84624, 0.83969, 0.83264,\
        0.82517, 0.81750, 0.80991, 0.80304, 0.79857, 0.79690, 0.79572, 0.79452, 0.79289, 0.78991, 0.78512,\
        0.77860, 0.77069, 0.76175, 0.75214, 0.74217, 0.73219, 0.72262, 0.71407, 0.70697, 0.70096, 0.69554,\
        0.69068, 0.68599, 0.68100, 0.67572, 0.67018, 0.66446, 0.65864, 0.65281, 0.64707, 0.64149, 0.63614,\
        0.63105, 0.62623, 0.62164, 0.61724, 0.61295, 0.60870, 0.60441, 0.60002, 0.59546, 0.59068, 0.58569,\
        0.58047, 0.57504, 0.56942, 0.56362, 0.55768, 0.55162, 0.54548, 0.53929, 0.53305, 0.52679, 0.52051],\
        [0.59692, 0.60317, 0.60993, 0.61691, 0.62402, 0.63124, 0.63855, 0.64593, 0.65333, 0.66070, 0.66795,\
        0.67499, 0.68171, 0.68797, 0.69353, 0.69796, 0.70094, 0.70289, 0.70456, 0.70683, 0.71016, 0.71474,\
        0.72050, 0.72720, 0.73460, 0.74244, 0.75057, 0.75887, 0.76730, 0.77580, 0.78428, 0.79255, 0.80030,\
        0.80729, 0.81424, 0.82119, 0.82767, 0.83345, 0.83849, 0.84289, 0.84688, 0.85075, 0.85479, 0.85898,\
        0.86301, 0.86639, 0.86854, 0.86935, 0.86971, 0.87065, 0.87265, 0.87562, 0.87906, 0.88220, 0.88422,\
        0.88416, 0.88124, 0.87588, 0.86926, 0.86242, 0.85595, 0.84995, 0.84426, 0.83861, 0.83274, 0.82650,\
        0.81988, 0.81302, 0.80625, 0.80005, 0.79549, 0.79290, 0.79110, 0.78931, 0.78700, 0.78359, 0.77874,\
        0.77241, 0.76479, 0.75617, 0.74683, 0.73705, 0.72713, 0.71740, 0.70851, 0.70188, 0.69631, 0.69108,\
        0.68607, 0.68113, 0.67603, 0.67073, 0.66522, 0.65959, 0.65389, 0.64823, 0.64270, 0.63738, 0.63235,\
        0.62764, 0.62325, 0.61916, 0.61527, 0.61148, 0.60769, 0.60378, 0.59966, 0.59528, 0.59060, 0.58561,\
        0.58033, 0.57478, 0.56901, 0.56306, 0.55695, 0.55074, 0.54444, 0.53811, 0.53175, 0.52539, 0.51905],\
        [0.59689, 0.60301, 0.60944, 0.61598, 0.62267, 0.62953, 0.63656, 0.64372, 0.65095, 0.65820, 0.66536,\
        0.67235, 0.67902, 0.68520, 0.69065, 0.69509, 0.69841, 0.70047, 0.70207, 0.70411, 0.70718, 0.71151,\
        0.71707, 0.72361, 0.73082, 0.73846, 0.74635, 0.75441, 0.76254, 0.77068, 0.77871, 0.78643, 0.79359,\
        0.79990, 0.80673, 0.81321, 0.81908, 0.82415, 0.82870, 0.83299, 0.83713, 0.84119, 0.84525, 0.84929,\
        0.85310, 0.85638, 0.85884, 0.86051, 0.86182, 0.86333, 0.86533, 0.86779, 0.87034, 0.87241, 0.87335,\
        0.87252, 0.86958, 0.86480, 0.85897, 0.85287, 0.84705, 0.84168, 0.83668, 0.83181, 0.82681, 0.82151,\
        0.81585, 0.80994, 0.80404, 0.79856, 0.79383, 0.79027, 0.78747, 0.78480, 0.78172, 0.77781, 0.77277,\
        0.76653, 0.75918, 0.75091, 0.74195, 0.73256, 0.72300, 0.71355, 0.70451, 0.69756, 0.69250, 0.68723,\
        0.68197, 0.67673, 0.67141, 0.66597, 0.66042, 0.65479, 0.64916, 0.64362, 0.63825, 0.63315, 0.62840,\
        0.62406, 0.62013, 0.61655, 0.61322, 0.60998, 0.60669, 0.60319, 0.59937, 0.59516, 0.59054, 0.58553,\
        0.58017, 0.57449, 0.56856, 0.56243, 0.55615, 0.54976, 0.54331, 0.53683, 0.53036, 0.52391, 0.51749],\
        [0.59707, 0.60302, 0.60880, 0.61475, 0.62093, 0.62738, 0.63408, 0.64097, 0.64800, 0.65509, 0.66213,\
        0.66901, 0.67557, 0.68164, 0.68700, 0.69144, 0.69483, 0.69715, 0.69894, 0.70089, 0.70385, 0.70806,\
        0.71361, 0.72009, 0.72714, 0.73451, 0.74207, 0.74974, 0.75745, 0.76511, 0.77262, 0.77982, 0.78656,\
        0.79296, 0.79931, 0.80536, 0.81077, 0.81503, 0.81912, 0.82348, 0.82787, 0.83216, 0.83636, 0.84045,\
        0.84428, 0.84767, 0.85045, 0.85265, 0.85450, 0.85628, 0.85815, 0.86007, 0.86178, 0.86286, 0.86285,\
        0.86136, 0.85825, 0.85379, 0.84853, 0.84315, 0.83811, 0.83362, 0.82957, 0.82571, 0.82174, 0.81746,\
        0.81278, 0.80776, 0.80250, 0.79726, 0.79240, 0.78815, 0.78441, 0.78084, 0.77702, 0.77260, 0.76734,\
        0.76113, 0.75401, 0.74610, 0.73759, 0.72873, 0.71982, 0.71125, 0.70352, 0.69649, 0.69002, 0.68410,\
        0.67839, 0.67277, 0.66716, 0.66151, 0.65583, 0.65015, 0.64452, 0.63903, 0.63377, 0.62883, 0.62433,\
        0.62034, 0.61687, 0.61384, 0.61111, 0.60847, 0.60571, 0.60264, 0.59912, 0.59507, 0.59050, 0.58544,\
        0.57996, 0.57413, 0.56802, 0.56170, 0.55523, 0.54867, 0.54206, 0.53544, 0.52885, 0.52232, 0.51584],\
        [0.59629, 0.60208, 0.60748, 0.61288, 0.61860, 0.62467, 0.63105, 0.63768, 0.64450, 0.65141, 0.65832,\
        0.66508, 0.67156, 0.67756, 0.68291, 0.68742, 0.69100, 0.69370, 0.69583, 0.69796, 0.70069, 0.70482,\
        0.71053, 0.71694, 0.72372, 0.73071, 0.73781, 0.74498, 0.75216, 0.75929, 0.76629, 0.77306, 0.77957,\
        0.78588, 0.79201, 0.79801, 0.80300, 0.80610, 0.80989, 0.81464, 0.81929, 0.82376, 0.82813, 0.83237,\
        0.83637, 0.83995, 0.84301, 0.84554, 0.84765, 0.84950, 0.85115, 0.85256, 0.85354, 0.85378, 0.85299,\
        0.85092, 0.84757, 0.84318, 0.83828, 0.83348, 0.82926, 0.82580, 0.82292, 0.82025, 0.81741, 0.81413,\
        0.81032, 0.80603, 0.80131, 0.79610, 0.79088, 0.78602, 0.78153, 0.77719, 0.77273, 0.76787, 0.76241,\
        0.75622, 0.74932, 0.74177, 0.73372, 0.72541, 0.71711, 0.70919, 0.70192, 0.69484, 0.68791, 0.68141,\
        0.67520, 0.66917, 0.66323, 0.65733, 0.65148, 0.64570, 0.64003, 0.63455, 0.62934, 0.62451, 0.62019,\
        0.61650, 0.61348, 0.61105, 0.60898, 0.60699, 0.60478, 0.60214, 0.59890, 0.59499, 0.59043, 0.58529,\
        0.57966, 0.57365, 0.56735, 0.56083, 0.55417, 0.54743, 0.54067, 0.53391, 0.52722, 0.52061, 0.51409],\
        [0.59444, 0.60022, 0.60515, 0.61013, 0.61556, 0.62134, 0.62745, 0.63384, 0.64046, 0.64721, 0.65401,\
        0.66070, 0.66713, 0.67316, 0.67862, 0.68336, 0.68732, 0.69053, 0.69321, 0.69571, 0.69852, 0.70281,\
        0.70834, 0.71442, 0.72073, 0.72716, 0.73365, 0.74020, 0.74677, 0.75333, 0.75982, 0.76621, 0.77247,\
        0.77861, 0.78462, 0.79032, 0.79470, 0.79776, 0.80171, 0.80704, 0.81151, 0.81602, 0.82052, 0.82496,\
        0.82920, 0.83302, 0.83630, 0.83901, 0.84121, 0.84298, 0.84438, 0.84533, 0.84571, 0.84530, 0.84390,\
        0.84137, 0.83775, 0.83329, 0.82851, 0.82411, 0.82068, 0.81837, 0.81682, 0.81545, 0.81374, 0.81137,\
        0.80818, 0.80423, 0.79970, 0.79443, 0.78892, 0.78358, 0.77850, 0.77359, 0.76864, 0.76346, 0.75786,\
        0.75173, 0.74504, 0.73784, 0.73024, 0.72239, 0.71451, 0.70679, 0.69955, 0.69254, 0.68558, 0.67880,\
        0.67223, 0.66583, 0.65957, 0.65341, 0.64737, 0.64147, 0.63574, 0.63024, 0.62504, 0.62025, 0.61604,\
        0.61259, 0.61001, 0.60821, 0.60686, 0.60556, 0.60395, 0.60172, 0.59872, 0.59491, 0.59030, 0.58501,\
        0.57920, 0.57299, 0.56649, 0.55978, 0.55294, 0.54603, 0.53911, 0.53224, 0.52545, 0.51877, 0.51223],\
        [0.59155, 0.59692, 0.60149, 0.60651, 0.61187, 0.61744, 0.62330, 0.62948, 0.63593, 0.64257, 0.64928,\
        0.65595, 0.66242, 0.66857, 0.67426, 0.67938, 0.68387, 0.68777, 0.69120, 0.69441, 0.69784, 0.70205,\
        0.70711, 0.71257, 0.71817, 0.72385, 0.72960, 0.73542, 0.74132, 0.74727, 0.75325, 0.75923, 0.76519,\
        0.77109, 0.77683, 0.78216, 0.78669, 0.79076, 0.79564, 0.80023, 0.80426, 0.80893, 0.81345, 0.81813,\
        0.82267, 0.82673, 0.83014, 0.83291, 0.83507, 0.83670, 0.83783, 0.83841, 0.83834, 0.83748, 0.83567,\
        0.83284, 0.82902, 0.82443, 0.81958, 0.81533, 0.81259, 0.81156, 0.81146, 0.81141, 0.81075, 0.80907,\
        0.80624, 0.80227, 0.79741, 0.79201, 0.78628, 0.78059, 0.77511, 0.76980, 0.76455, 0.75919, 0.75356,\
        0.74754, 0.74110, 0.73425, 0.72705, 0.71962, 0.71209, 0.70458, 0.69723, 0.69022, 0.68320, 0.67623,\
        0.66937, 0.66266, 0.65611, 0.64971, 0.64349, 0.63747, 0.63169, 0.62617, 0.62099, 0.61621, 0.61201,\
        0.60868, 0.60651, 0.60539, 0.60481, 0.60422, 0.60323, 0.60146, 0.59864, 0.59480, 0.59002, 0.58452,\
        0.57849, 0.57207, 0.56538, 0.55849, 0.55149, 0.54444, 0.53739, 0.53041, 0.52354, 0.51682, 0.51027],\
        [0.58786, 0.59285, 0.59753, 0.60262, 0.60776, 0.61304, 0.61867, 0.62467, 0.63099, 0.63754, 0.64422,\
        0.65091, 0.65749, 0.66384, 0.66985, 0.67546, 0.68060, 0.68531, 0.68964, 0.69371, 0.69767, 0.70189,\
        0.70653, 0.71121, 0.71590, 0.72067, 0.72557, 0.73061, 0.73580, 0.74112, 0.74656, 0.75212, 0.75774,\
        0.76339, 0.76893, 0.77419, 0.77909, 0.78384, 0.78871, 0.79324, 0.79782, 0.80241, 0.80669, 0.81184,\
        0.81672, 0.82092, 0.82436, 0.82707, 0.82911, 0.83055, 0.83144, 0.83174, 0.83139, 0.83028, 0.82830,\
        0.82539, 0.82157, 0.81699, 0.81203, 0.80761, 0.80541, 0.80580, 0.80714, 0.80825, 0.80841, 0.80716,\
        0.80437, 0.80017, 0.79484, 0.78895, 0.78288, 0.77690, 0.77116, 0.76565, 0.76028, 0.75490, 0.74936,\
        0.74355, 0.73739, 0.73088, 0.72407, 0.71704, 0.70985, 0.70258, 0.69539, 0.68813, 0.68091, 0.67371,\
        0.66660, 0.65962, 0.65280, 0.64618, 0.63980, 0.63369, 0.62788, 0.62241, 0.61729, 0.61256, 0.60834,\
        0.60496, 0.60311, 0.60276, 0.60288, 0.60288, 0.60252, 0.60127, 0.59855, 0.59446, 0.58940, 0.58365,\
        0.57741, 0.57081, 0.56396, 0.55693, 0.54981, 0.54264, 0.53550, 0.52843, 0.52150, 0.51476, 0.50823],\
        [0.58389, 0.58879, 0.59359, 0.59853, 0.60321, 0.60821, 0.61364, 0.61949, 0.62572, 0.63222, 0.63890,\
        0.64565, 0.65236, 0.65895, 0.66534, 0.67147, 0.67732, 0.68287, 0.68816, 0.69319, 0.69797, 0.70233,\
        0.70627, 0.70997, 0.71362, 0.71742, 0.72145, 0.72570, 0.73017, 0.73485, 0.73976, 0.74488, 0.75019,\
        0.75562, 0.76106, 0.76639, 0.77156, 0.77665, 0.78166, 0.78654, 0.79136, 0.79582, 0.80060, 0.80626,\
        0.81122, 0.81538, 0.81873, 0.82127, 0.82312, 0.82437, 0.82506, 0.82519, 0.82473, 0.82358, 0.82167,\
        0.81895, 0.81546, 0.81129, 0.80665, 0.80210, 0.80019, 0.80190, 0.80407, 0.80592, 0.80661, 0.80540,\
        0.80223, 0.79737, 0.79141, 0.78499, 0.77854, 0.77235, 0.76651, 0.76099, 0.75571, 0.75048, 0.74517,\
        0.73964, 0.73381, 0.72765, 0.72120, 0.71455, 0.70776, 0.70070, 0.69346, 0.68610, 0.67866, 0.67124,\
        0.66388, 0.65665, 0.64959, 0.64278, 0.63626, 0.63008, 0.62430, 0.61895, 0.61403, 0.60952, 0.60543,\
        0.60194, 0.60023, 0.60058, 0.60113, 0.60131, 0.60139, 0.60078, 0.59791, 0.59349, 0.58816, 0.58221,\
        0.57583, 0.56912, 0.56218, 0.55508, 0.54788, 0.54066, 0.53345, 0.52633, 0.51937, 0.51262, 0.50614],\
        [0.57978, 0.58455, 0.58921, 0.59377, 0.59828, 0.60310, 0.60837, 0.61409, 0.62024, 0.62671, 0.63340,\
        0.64020, 0.64704, 0.65387, 0.66062, 0.66726, 0.67376, 0.68010, 0.68627, 0.69222, 0.69772, 0.70228,\
        0.70565, 0.70832, 0.71095, 0.71384, 0.71708, 0.72061, 0.72440, 0.72846, 0.73284, 0.73755, 0.74258,\
        0.74784, 0.75325, 0.75868, 0.76407, 0.76939, 0.77464, 0.77979, 0.78486, 0.78990, 0.79536, 0.80099,\
        0.80569, 0.80974, 0.81295, 0.81522, 0.81685, 0.81793, 0.81848, 0.81855, 0.81813, 0.81714, 0.81552,\
        0.81322, 0.81037, 0.80714, 0.80365, 0.80037, 0.79882, 0.79981, 0.80191, 0.80395, 0.80500, 0.80343,\
        0.79923, 0.79347, 0.78675, 0.77980, 0.77307, 0.76679, 0.76103, 0.75573, 0.75072, 0.74584, 0.74091,\
        0.73576, 0.73031, 0.72450, 0.71834, 0.71197, 0.70558, 0.69890, 0.69157, 0.68401, 0.67638, 0.66875,\
        0.66117, 0.65371, 0.64644, 0.63945, 0.63280, 0.62658, 0.62087, 0.61571, 0.61112, 0.60706, 0.60351,\
        0.60055, 0.59886, 0.59883, 0.59928, 0.59929, 0.59936, 0.59890, 0.59594, 0.59142, 0.58603, 0.58005,\
        0.57365, 0.56694, 0.56000, 0.55291, 0.54572, 0.53850, 0.53128, 0.52415, 0.51717, 0.51044, 0.50403],\
        [0.57557, 0.58018, 0.58464, 0.58901, 0.59338, 0.59800, 0.60304, 0.60860, 0.61468, 0.62112, 0.62778,\
        0.63460, 0.64153, 0.64854, 0.65560, 0.66266, 0.66969, 0.67666, 0.68352, 0.69016, 0.69631, 0.70123,\
        0.70394, 0.70570, 0.70750, 0.70972, 0.71240, 0.71533, 0.71849, 0.72196, 0.72583, 0.73017, 0.73495,\
        0.74011, 0.74552, 0.75106, 0.75663, 0.76215, 0.76759, 0.77296, 0.77830, 0.78370, 0.78927, 0.79469,\
        0.79931, 0.80362, 0.80662, 0.80850, 0.80999, 0.81098, 0.81145, 0.81156, 0.81131, 0.81063, 0.80947,\
        0.80768, 0.80556, 0.80351, 0.80136, 0.79921, 0.79776, 0.79804, 0.79953, 0.80115, 0.80273, 0.80093,\
        0.79507, 0.78800, 0.78049, 0.77311, 0.76625, 0.76011, 0.75466, 0.74980, 0.74530, 0.74095, 0.73653,\
        0.73187, 0.72686, 0.72142, 0.71552, 0.70919, 0.70269, 0.69637, 0.68924, 0.68171, 0.67399, 0.66619,\
        0.65843, 0.65077, 0.64331, 0.63614, 0.62937, 0.62310, 0.61745, 0.61251, 0.60830, 0.60476, 0.60190,\
        0.59946, 0.59767, 0.59695, 0.59680, 0.59663, 0.59625, 0.59510, 0.59233, 0.58811, 0.58294, 0.57712,\
        0.57085, 0.56426, 0.55744, 0.55045, 0.54335, 0.53621, 0.52906, 0.52196, 0.51500, 0.50830, 0.50199],\
        [0.57135, 0.57580, 0.58011, 0.58435, 0.58862, 0.59307, 0.59787, 0.60324, 0.60923, 0.61557, 0.62214,\
        0.62890, 0.63583, 0.64294, 0.65021, 0.65757, 0.66497, 0.67233, 0.67958, 0.68654, 0.69291, 0.69792,\
        0.70027, 0.70173, 0.70311, 0.70500, 0.70745, 0.70989, 0.71246, 0.71538, 0.71878, 0.72278, 0.72737,\
        0.73248, 0.73795, 0.74360, 0.74930, 0.75495, 0.76051, 0.76598, 0.77141, 0.77683, 0.78222, 0.78738,\
        0.79213, 0.79646, 0.79905, 0.80069, 0.80226, 0.80326, 0.80369, 0.80394, 0.80398, 0.80365, 0.80313,\
        0.80179, 0.80020, 0.79901, 0.79783, 0.79610, 0.79494, 0.79480, 0.79549, 0.79659, 0.79735, 0.79472,\
        0.78822, 0.78033, 0.77230, 0.76473, 0.75800, 0.75224, 0.74740, 0.74323, 0.73946, 0.73581, 0.73203,\
        0.72796, 0.72346, 0.71846, 0.71291, 0.70676, 0.70013, 0.69362, 0.68670, 0.67923, 0.67146, 0.66355,\
        0.65563, 0.64781, 0.64018, 0.63284, 0.62592, 0.61956, 0.61393, 0.60916, 0.60528, 0.60209, 0.59954,\
        0.59738, 0.59557, 0.59442, 0.59373, 0.59311, 0.59221, 0.59058, 0.58780, 0.58387, 0.57901, 0.57348,\
        0.56748, 0.56112, 0.55451, 0.54772, 0.54081, 0.53383, 0.52683, 0.51987, 0.51300, 0.50631, 0.50012],\
        [0.56717, 0.57150, 0.57570, 0.57985, 0.58404, 0.58841, 0.59309, 0.59829, 0.60403, 0.61016, 0.61654,\
        0.62311, 0.62994, 0.63706, 0.64443, 0.65196, 0.65955, 0.66709, 0.67442, 0.68133, 0.68743, 0.69208,\
        0.69491, 0.69666, 0.69808, 0.70000, 0.70247, 0.70432, 0.70638, 0.70883, 0.71179, 0.71549, 0.71995,\
        0.72507, 0.73063, 0.73639, 0.74217, 0.74786, 0.75341, 0.75882, 0.76412, 0.76932, 0.77439, 0.77922,\
        0.78364, 0.78739, 0.79001, 0.79184, 0.79338, 0.79444, 0.79504, 0.79551, 0.79585, 0.79589, 0.79581,\
        0.79506, 0.79397, 0.79310, 0.79216, 0.79099, 0.79001, 0.78951, 0.78942, 0.78931, 0.78818, 0.78451,\
        0.77808, 0.77018, 0.76210, 0.75466, 0.74834, 0.74329, 0.73933, 0.73613, 0.73330, 0.73050, 0.72745,\
        0.72401, 0.72006, 0.71554, 0.71044, 0.70475, 0.69849, 0.69150, 0.68426, 0.67667, 0.66881, 0.66080,\
        0.65277, 0.64481, 0.63703, 0.62955, 0.62247, 0.61597, 0.61024, 0.60551, 0.60188, 0.59894, 0.59639,\
        0.59427, 0.59249, 0.59110, 0.58999, 0.58890, 0.58754, 0.58558, 0.58276, 0.57902, 0.57448, 0.56930,\
        0.56362, 0.55758, 0.55128, 0.54477, 0.53813, 0.53142, 0.52467, 0.51796, 0.51135, 0.50488, 0.49876],\
        [0.56308, 0.56732, 0.57145, 0.57554, 0.57968, 0.58399, 0.58860, 0.59362, 0.59908, 0.60495, 0.61103,\
        0.61724, 0.62385, 0.63091, 0.63832, 0.64592, 0.65355, 0.66108, 0.66831, 0.67501, 0.68083, 0.68541,\
        0.68865, 0.69093, 0.69282, 0.69490, 0.69705, 0.69857, 0.70042, 0.70247, 0.70506, 0.70847, 0.71286,\
        0.71805, 0.72370, 0.72953, 0.73533, 0.74095, 0.74634, 0.75152, 0.75650, 0.76131, 0.76592, 0.77025,\
        0.77415, 0.77744, 0.77998, 0.78190, 0.78343, 0.78461, 0.78549, 0.78623, 0.78684, 0.78725, 0.78742,\
        0.78716, 0.78659, 0.78592, 0.78513, 0.78418, 0.78320, 0.78231, 0.78136, 0.77993, 0.77727, 0.77266,\
        0.76605, 0.75823, 0.75028, 0.74317, 0.73752, 0.73346, 0.73070, 0.72872, 0.72700, 0.72513, 0.72284,\
        0.72000, 0.71656, 0.71250, 0.70780, 0.70234, 0.69608, 0.68912, 0.68169, 0.67396, 0.66601, 0.65792,\
        0.64981, 0.64176, 0.63389, 0.62628, 0.61908, 0.61241, 0.60648, 0.60169, 0.59805, 0.59522, 0.59272,\
        0.59055, 0.58869, 0.58708, 0.58563, 0.58417, 0.58245, 0.58025, 0.57739, 0.57381, 0.56955, 0.56472,\
        0.55941, 0.55375, 0.54781, 0.54166, 0.53537, 0.52899, 0.52258, 0.51622, 0.51000, 0.50410, 0.49844],\
        [0.55911, 0.56328, 0.56735, 0.57140, 0.57550, 0.57976, 0.58427, 0.58913, 0.59437, 0.60001, 0.60559,\
        0.61122, 0.61758, 0.62459, 0.63200, 0.63957, 0.64715, 0.65455, 0.66161, 0.66809, 0.67376, 0.67840,\
        0.68199, 0.68476, 0.68709, 0.68926, 0.69123, 0.69293, 0.69467, 0.69657, 0.69910, 0.70196, 0.70642,\
        0.71164, 0.71735, 0.72319, 0.72890, 0.73430, 0.73936, 0.74413, 0.74864, 0.75293, 0.75697, 0.76071,\
        0.76407, 0.76694, 0.76927, 0.77115, 0.77270, 0.77402, 0.77515, 0.77618, 0.77709, 0.77783, 0.77830,\
        0.77842, 0.77820, 0.77772, 0.77702, 0.77609, 0.77497, 0.77364, 0.77192, 0.76944, 0.76570, 0.76031,\
        0.75336, 0.74546, 0.73759, 0.73081, 0.72596, 0.72318, 0.72189, 0.72131, 0.72079, 0.71985, 0.71825,\
        0.71591, 0.71287, 0.70914, 0.70466, 0.69956, 0.69322, 0.68621, 0.67874, 0.67097, 0.66299, 0.65489,\
        0.64674, 0.63866, 0.63073, 0.62308, 0.61581, 0.60909, 0.60302, 0.59807, 0.59437, 0.59134, 0.58872,\
        0.58642, 0.58438, 0.58254, 0.58081, 0.57903, 0.57705, 0.57468, 0.57181, 0.56837, 0.56437, 0.55987,\
        0.55496, 0.54970, 0.54417, 0.53844, 0.53255, 0.52657, 0.52055, 0.51455, 0.50865, 0.50298, 0.49757],\
        [0.55527, 0.55938, 0.56341, 0.56742, 0.57147, 0.57566, 0.58005, 0.58472, 0.58969, 0.59489, 0.59982,\
        0.60499, 0.61130, 0.61831, 0.62567, 0.63314, 0.64055, 0.64777, 0.65462, 0.66093, 0.66652, 0.67126,\
        0.67515, 0.67833, 0.68102, 0.68338, 0.68548, 0.68739, 0.68928, 0.69133, 0.69375, 0.69683, 0.70095,\
        0.70606, 0.71174, 0.71751, 0.72297, 0.72797, 0.73254, 0.73674, 0.74065, 0.74430, 0.74771, 0.75085,\
        0.75365, 0.75608, 0.75814, 0.75990, 0.76145, 0.76288, 0.76424, 0.76557, 0.76681, 0.76788, 0.76867,\
        0.76909, 0.76911, 0.76877, 0.76809, 0.76709, 0.76578, 0.76408, 0.76179, 0.75861, 0.75418, 0.74827,\
        0.74100, 0.73291, 0.72496, 0.71834, 0.71433, 0.71306, 0.71342, 0.71431, 0.71494, 0.71481, 0.71373,\
        0.71170, 0.70886, 0.70541, 0.70101, 0.69569, 0.68959, 0.68271, 0.67534, 0.66763, 0.65970, 0.65164,\
        0.64354, 0.63547, 0.62755, 0.61991, 0.61267, 0.60596, 0.60019, 0.59508, 0.59088, 0.58746, 0.58455,\
        0.58200, 0.57972, 0.57764, 0.57564, 0.57362, 0.57143, 0.56896, 0.56609, 0.56279, 0.55904, 0.55488,\
        0.55035, 0.54551, 0.54043, 0.53515, 0.52972, 0.52418, 0.51858, 0.51296, 0.50738, 0.50188, 0.49668],\
        [0.55156, 0.55564, 0.55963, 0.56359, 0.56759, 0.57169, 0.57593, 0.58036, 0.58496, 0.58961, 0.59419,\
        0.59926, 0.60543, 0.61237, 0.61959, 0.62682, 0.63397, 0.64094, 0.64758, 0.65376, 0.65931, 0.66417,\
        0.66831, 0.67183, 0.67484, 0.67749, 0.67986, 0.68206, 0.68424, 0.68658, 0.68927, 0.69254, 0.69657,\
        0.70147, 0.70697, 0.71257, 0.71761, 0.72198, 0.72586, 0.72936, 0.73258, 0.73555, 0.73831, 0.74083,\
        0.74308, 0.74507, 0.74681, 0.74840, 0.74991, 0.75142, 0.75299, 0.75462, 0.75622, 0.75765, 0.75876,\
        0.75945, 0.75964, 0.75936, 0.75864, 0.75753, 0.75603, 0.75408, 0.75149, 0.74799, 0.74328, 0.73718,\
        0.72978, 0.72158, 0.71356, 0.70694, 0.70365, 0.70404, 0.70593, 0.70809, 0.70963, 0.71008, 0.70928,\
        0.70733, 0.70440, 0.70095, 0.69687, 0.69148, 0.68538, 0.67866, 0.67145, 0.66390, 0.65611, 0.64817,\
        0.64016, 0.63217, 0.62430, 0.61669, 0.60955, 0.60300, 0.59697, 0.59181, 0.58732, 0.58350, 0.58023,\
        0.57737, 0.57482, 0.57247, 0.57024, 0.56801, 0.56567, 0.56313, 0.56030, 0.55714, 0.55364, 0.54980,\
        0.54566, 0.54126, 0.53664, 0.53184, 0.52689, 0.52183, 0.51670, 0.51151, 0.50633, 0.50121, 0.49622],\
        [0.54800, 0.55204, 0.55600, 0.55992, 0.56386, 0.56786, 0.57196, 0.57618, 0.58050, 0.58490, 0.58945,\
        0.59448, 0.60035, 0.60706, 0.61392, 0.62074, 0.62753, 0.63421, 0.64066, 0.64671, 0.65226, 0.65722,\
        0.66158, 0.66538, 0.66872, 0.67168, 0.67438, 0.67693, 0.67948, 0.68219, 0.68524, 0.68880, 0.69298,\
        0.69769, 0.70317, 0.70843, 0.71269, 0.71620, 0.71924, 0.72199, 0.72448, 0.72676, 0.72888, 0.73083,\
        0.73256, 0.73410, 0.73550, 0.73688, 0.73831, 0.73988, 0.74164, 0.74359, 0.74558, 0.74742, 0.74887,\
        0.74978, 0.75008, 0.74978, 0.74895, 0.74769, 0.74605, 0.74399, 0.74137, 0.73793, 0.73338, 0.72750,\
        0.72035, 0.71233, 0.70455, 0.69876, 0.69613, 0.69716, 0.69984, 0.70269, 0.70479, 0.70553, 0.70469,\
        0.70279, 0.69959, 0.69593, 0.69172, 0.68656, 0.68063, 0.67410, 0.66711, 0.65978, 0.65220, 0.64446,\
        0.63661, 0.62874, 0.62095, 0.61335, 0.60620, 0.59971, 0.59374, 0.58830, 0.58352, 0.57936, 0.57574,\
        0.57256, 0.56971, 0.56712, 0.56467, 0.56227, 0.55983, 0.55726, 0.55449, 0.55148, 0.54822, 0.54471,\
        0.54096, 0.53700, 0.53285, 0.52854, 0.52410, 0.51955, 0.51490, 0.51019, 0.50544, 0.50068, 0.49593],\
        [0.54456, 0.54858, 0.55251, 0.55640, 0.56028, 0.56420, 0.56818, 0.57225, 0.57643, 0.58074, 0.58532,\
        0.59037, 0.59611, 0.60240, 0.60862, 0.61488, 0.62127, 0.62768, 0.63395, 0.63991, 0.64545, 0.65050,\
        0.65505, 0.65911, 0.66273, 0.66601, 0.66904, 0.67194, 0.67485, 0.67793, 0.68134, 0.68522, 0.68968,\
        0.69475, 0.70020, 0.70463, 0.70780, 0.71031, 0.71251, 0.71458, 0.71640, 0.71800, 0.71955, 0.72101,\
        0.72227, 0.72336, 0.72443, 0.72560, 0.72693, 0.72850, 0.73044, 0.73274, 0.73520, 0.73747, 0.73927,\
        0.74038, 0.74071, 0.74030, 0.73927, 0.73780, 0.73604, 0.73401, 0.73159, 0.72855, 0.72458, 0.71938,\
        0.71292, 0.70573, 0.69854, 0.69371, 0.69161, 0.69228, 0.69466, 0.69753, 0.69984, 0.70076, 0.69947,\
        0.69755, 0.69443, 0.69057, 0.68618, 0.68111, 0.67536, 0.66907, 0.66234, 0.65528, 0.64797, 0.64048,\
        0.63286, 0.62517, 0.61751, 0.60993, 0.60263, 0.59605, 0.59003, 0.58447, 0.57943, 0.57496, 0.57103,\
        0.56754, 0.56444, 0.56162, 0.55899, 0.55646, 0.55395, 0.55138, 0.54869, 0.54585, 0.54284, 0.53964,\
        0.53628, 0.53275, 0.52909, 0.52529, 0.52136, 0.51733, 0.51319, 0.50895, 0.50463, 0.50022, 0.49574],\
        [0.54127, 0.54527, 0.54918, 0.55304, 0.55688, 0.56073, 0.56463, 0.56861, 0.57270, 0.57697, 0.58153,\
        0.58651, 0.59199, 0.59778, 0.60337, 0.60914, 0.61522, 0.62142, 0.62754, 0.63342, 0.63895, 0.64408,\
        0.64877, 0.65305, 0.65693, 0.66049, 0.66381, 0.66701, 0.67022, 0.67359, 0.67725, 0.68134, 0.68592,\
        0.69096, 0.69602, 0.69975, 0.70226, 0.70393, 0.70546, 0.70712, 0.70842, 0.70928, 0.71043, 0.71155,\
        0.71240, 0.71306, 0.71382, 0.71483, 0.71607, 0.71760, 0.71966, 0.72238, 0.72537, 0.72812, 0.73026,\
        0.73152, 0.73180, 0.73116, 0.72981, 0.72804, 0.72614, 0.72422, 0.72217, 0.71977, 0.71672, 0.71258,\
        0.70702, 0.70070, 0.69488, 0.69027, 0.68806, 0.68813, 0.68976, 0.69201, 0.69398, 0.69461, 0.69348,\
        0.69139, 0.68840, 0.68462, 0.68023, 0.67524, 0.66967, 0.66361, 0.65716, 0.65040, 0.64341, 0.63622,\
        0.62887, 0.62142, 0.61397, 0.60665, 0.59927, 0.59247, 0.58618, 0.58035, 0.57505, 0.57030, 0.56608,\
        0.56234, 0.55901, 0.55600, 0.55324, 0.55062, 0.54808, 0.54554, 0.54296, 0.54029, 0.53752, 0.53464,\
        0.53166, 0.52857, 0.52539, 0.52210, 0.51870, 0.51518, 0.51154, 0.50778, 0.50390, 0.49986, 0.49561],\
        [0.53810, 0.54209, 0.54600, 0.54983, 0.55364, 0.55745, 0.56129, 0.56521, 0.56925, 0.57346, 0.57792,\
        0.58269, 0.58777, 0.59297, 0.59815, 0.60361, 0.60949, 0.61555, 0.62153, 0.62730, 0.63280, 0.63798,\
        0.64280, 0.64724, 0.65133, 0.65511, 0.65866, 0.66207, 0.66548, 0.66900, 0.67276, 0.67683, 0.68123,\
        0.68579, 0.69006, 0.69337, 0.69561, 0.69696, 0.69817, 0.69961, 0.70065, 0.70063, 0.70172, 0.70256,\
        0.70313, 0.70339, 0.70394, 0.70492, 0.70614, 0.70757, 0.70962, 0.71290, 0.71643, 0.71964, 0.72207,\
        0.72341, 0.72356, 0.72258, 0.72076, 0.71853, 0.71643, 0.71468, 0.71303, 0.71127, 0.70933, 0.70653,\
        0.70200, 0.69618, 0.69086, 0.68674, 0.68439, 0.68386, 0.68466, 0.68607, 0.68728, 0.68756, 0.68661,\
        0.68461, 0.68174, 0.67812, 0.67384, 0.66898, 0.66360, 0.65778, 0.65160, 0.64516, 0.63851, 0.63167,\
        0.62464, 0.61741, 0.61007, 0.60299, 0.59604, 0.58876, 0.58206, 0.57592, 0.57035, 0.56534, 0.56088,\
        0.55694, 0.55344, 0.55030, 0.54745, 0.54480, 0.54226, 0.53979, 0.53732, 0.53483, 0.53231, 0.52974,\
        0.52714, 0.52448, 0.52178, 0.51899, 0.51611, 0.51311, 0.50996, 0.50664, 0.50314, 0.49944, 0.49546],\
        [0.53505, 0.53905, 0.54295, 0.54678, 0.55056, 0.55434, 0.55815, 0.56202, 0.56599, 0.57012, 0.57445,\
        0.57899, 0.58372, 0.58855, 0.59346, 0.59862, 0.60433, 0.61020, 0.61597, 0.62159, 0.62703, 0.63223,\
        0.63714, 0.64172, 0.64596, 0.64988, 0.65356, 0.65708, 0.66055, 0.66408, 0.66777, 0.67164, 0.67566,\
        0.67964, 0.68325, 0.68615, 0.68821, 0.68961, 0.69076, 0.69190, 0.69257, 0.69276, 0.69340, 0.69402,\
        0.69443, 0.69465, 0.69519, 0.69619, 0.69749, 0.69898, 0.70097, 0.70492, 0.70867, 0.71224, 0.71481,\
        0.71618, 0.71613, 0.71472, 0.71233, 0.70949, 0.70702, 0.70553, 0.70420, 0.70274, 0.70172, 0.69994,\
        0.69627, 0.69112, 0.68626, 0.68244, 0.68002, 0.67902, 0.67910, 0.67973, 0.68028, 0.68022, 0.67926,\
        0.67738, 0.67467, 0.67121, 0.66710, 0.66241, 0.65722, 0.65162, 0.64569, 0.63955, 0.63325, 0.62680,\
        0.62012, 0.61314, 0.60586, 0.59839, 0.59140, 0.58430, 0.57744, 0.57108, 0.56528, 0.56007, 0.55544,\
        0.55135, 0.54774, 0.54455, 0.54167, 0.53903, 0.53655, 0.53416, 0.53183, 0.52953, 0.52724, 0.52498,\
        0.52274, 0.52052, 0.51828, 0.51600, 0.51364, 0.51115, 0.50848, 0.50559, 0.50242, 0.49898, 0.49532],\
        [0.53212, 0.53613, 0.54004, 0.54386, 0.54764, 0.55140, 0.55517, 0.55900, 0.56291, 0.56695, 0.57114,\
        0.57549, 0.57998, 0.58460, 0.58937, 0.59442, 0.59986, 0.60538, 0.61085, 0.61627, 0.62162, 0.62683,\
        0.63182, 0.63649, 0.64081, 0.64480, 0.64850, 0.65202, 0.65543, 0.65884, 0.66231, 0.66586, 0.66943,\
        0.67287, 0.67597, 0.67852, 0.68046, 0.68189, 0.68301, 0.68393, 0.68454, 0.68494, 0.68542, 0.68591,\
        0.68635, 0.68679, 0.68750, 0.68861, 0.69012, 0.69202, 0.69490, 0.69820, 0.70233, 0.70585, 0.70831,\
        0.70974, 0.70943, 0.70757, 0.70462, 0.70142, 0.69841, 0.69700, 0.69577, 0.69448, 0.69345, 0.69179,\
        0.68876, 0.68466, 0.68053, 0.67710, 0.67472, 0.67343, 0.67301, 0.67306, 0.67311, 0.67275, 0.67173,\
        0.66993, 0.66737, 0.66408, 0.66014, 0.65562, 0.65061, 0.64519, 0.63946, 0.63356, 0.62758, 0.62154,\
        0.61528, 0.60855, 0.60142, 0.59396, 0.58650, 0.57925, 0.57229, 0.56578, 0.55983, 0.55449, 0.54975,\
        0.54559, 0.54196, 0.53877, 0.53594, 0.53337, 0.53099, 0.52873, 0.52654, 0.52442, 0.52237, 0.52040,\
        0.51851, 0.51670, 0.51493, 0.51315, 0.51130, 0.50931, 0.50712, 0.50468, 0.50194, 0.49882, 0.49531],\
        [0.52930, 0.53332, 0.53724, 0.54107, 0.54484, 0.54859, 0.55234, 0.55613, 0.55998, 0.56393, 0.56800,\
        0.57220, 0.57654, 0.58102, 0.58567, 0.59058, 0.59571, 0.60092, 0.60610, 0.61130, 0.61656, 0.62181,\
        0.62687, 0.63159, 0.63592, 0.63987, 0.64350, 0.64689, 0.65013, 0.65330, 0.65646, 0.65962, 0.66275,\
        0.66572, 0.66840, 0.67068, 0.67250, 0.67393, 0.67506, 0.67594, 0.67662, 0.67716, 0.67769, 0.67825,\
        0.67883, 0.67954, 0.68050, 0.68184, 0.68364, 0.68598, 0.68901, 0.69260, 0.69659, 0.69972, 0.70212,\
        0.70378, 0.70302, 0.70090, 0.69752, 0.69419, 0.69121, 0.68919, 0.68768, 0.68631, 0.68493, 0.68311,\
        0.68046, 0.67714, 0.67371, 0.67074, 0.66851, 0.66711, 0.66640, 0.66608, 0.66580, 0.66525, 0.66418,\
        0.66245, 0.66001, 0.65688, 0.65310, 0.64873, 0.64387, 0.63859, 0.63299, 0.62721, 0.62143, 0.61579,\
        0.61006, 0.60357, 0.59629, 0.58874, 0.58113, 0.57371, 0.56664, 0.56004, 0.55401, 0.54861, 0.54384,\
        0.53970, 0.53612, 0.53303, 0.53032, 0.52789, 0.52566, 0.52355, 0.52153, 0.51958, 0.51774, 0.51603,\
        0.51448, 0.51307, 0.51176, 0.51047, 0.50912, 0.50761, 0.50588, 0.50386, 0.50150, 0.49868, 0.49533],\
        [0.52657, 0.53062, 0.53455, 0.53839, 0.54217, 0.54591, 0.54963, 0.55338, 0.55718, 0.56105, 0.56502,\
        0.56911, 0.57333, 0.57769, 0.58222, 0.58694, 0.59180, 0.59673, 0.60167, 0.60663, 0.61187, 0.61721,\
        0.62233, 0.62703, 0.63129, 0.63511, 0.63857, 0.64172, 0.64468, 0.64753, 0.65031, 0.65307, 0.65576,\
        0.65833, 0.66067, 0.66273, 0.66445, 0.66587, 0.66704, 0.66800, 0.66880, 0.66951, 0.67020, 0.67093,\
        0.67174, 0.67272, 0.67396, 0.67556, 0.67759, 0.68014, 0.68322, 0.68670, 0.69024, 0.69323, 0.69577,\
        0.69711, 0.69610, 0.69372, 0.69055, 0.68718, 0.68417, 0.68176, 0.67982, 0.67810, 0.67637, 0.67435,\
        0.67186, 0.66901, 0.66612, 0.66355, 0.66154, 0.66017, 0.65934, 0.65886, 0.65843, 0.65780, 0.65673,\
        0.65508, 0.65276, 0.64977, 0.64612, 0.64188, 0.63714, 0.63196, 0.62642, 0.62061, 0.61476, 0.60932,\
        0.60417, 0.59808, 0.59060, 0.58287, 0.57517, 0.56767, 0.56054, 0.55391, 0.54787, 0.54248, 0.53776,\
        0.53372, 0.53028, 0.52737, 0.52487, 0.52266, 0.52064, 0.51871, 0.51686, 0.51508, 0.51342, 0.51194,\
        0.51069, 0.50968, 0.50883, 0.50802, 0.50714, 0.50608, 0.50476, 0.50308, 0.50097, 0.49839, 0.49529],\
        [0.52393, 0.52800, 0.53195, 0.53581, 0.53959, 0.54332, 0.54703, 0.55074, 0.55448, 0.55828, 0.56217,\
        0.56616, 0.57027, 0.57453, 0.57894, 0.58349, 0.58815, 0.59285, 0.59750, 0.60228, 0.60764, 0.61315,\
        0.61826, 0.62285, 0.62692, 0.63052, 0.63369, 0.63653, 0.63912, 0.64157, 0.64394, 0.64628, 0.64858,\
        0.65079, 0.65286, 0.65473, 0.65638, 0.65781, 0.65905, 0.66014, 0.66111, 0.66202, 0.66293, 0.66389,\
        0.66496, 0.66620, 0.66768, 0.66950, 0.67169, 0.67429, 0.67725, 0.68044, 0.68355, 0.68626, 0.68830,\
        0.68911, 0.68823, 0.68604, 0.68311, 0.67994, 0.67693, 0.67429, 0.67200, 0.66991, 0.66785, 0.66563,\
        0.66319, 0.66059, 0.65804, 0.65578, 0.65400, 0.65275, 0.65197, 0.65149, 0.65107, 0.65047, 0.64948,\
        0.64795, 0.64576, 0.64289, 0.63935, 0.63520, 0.63054, 0.62546, 0.61998, 0.61410, 0.60788, 0.60206,\
        0.59712, 0.59111, 0.58395, 0.57632, 0.56865, 0.56117, 0.55407, 0.54747, 0.54147, 0.53616, 0.53157,\
        0.52771, 0.52451, 0.52189, 0.51969, 0.51778, 0.51602, 0.51432, 0.51264, 0.51101, 0.50949, 0.50819,\
        0.50722, 0.50660, 0.50620, 0.50585, 0.50541, 0.50476, 0.50380, 0.50244, 0.50059, 0.49819, 0.49525],\
        [0.52135, 0.52546, 0.52944, 0.53331, 0.53709, 0.54081, 0.54450, 0.54817, 0.55186, 0.55559, 0.55939,\
        0.56329, 0.56731, 0.57147, 0.57578, 0.58023, 0.58478, 0.58938, 0.59399, 0.59871, 0.60431, 0.60979,\
        0.61467, 0.61898, 0.62276, 0.62605, 0.62888, 0.63131, 0.63347, 0.63547, 0.63741, 0.63934, 0.64127,\
        0.64318, 0.64502, 0.64676, 0.64836, 0.64982, 0.65116, 0.65240, 0.65356, 0.65470, 0.65585, 0.65707,\
        0.65839, 0.65987, 0.66157, 0.66353, 0.66578, 0.66832, 0.67107, 0.67390, 0.67657, 0.67881, 0.68031,\
        0.68074, 0.67989, 0.67796, 0.67533, 0.67238, 0.66944, 0.66669, 0.66416, 0.66177, 0.65943, 0.65703,\
        0.65455, 0.65205, 0.64970, 0.64768, 0.64611, 0.64505, 0.64443, 0.64409, 0.64381, 0.64337, 0.64254,\
        0.64117, 0.63913, 0.63638, 0.63292, 0.62882, 0.62421, 0.61920, 0.61388, 0.60809, 0.60177, 0.59543,\
        0.58980, 0.58366, 0.57670, 0.56926, 0.56172, 0.55434, 0.54732, 0.54081, 0.53493, 0.52976, 0.52536,\
        0.52175, 0.51889, 0.51665, 0.51487, 0.51334, 0.51191, 0.51047, 0.50899, 0.50750, 0.50608, 0.50489,\
        0.50415, 0.50392, 0.50396, 0.50404, 0.50398, 0.50368, 0.50303, 0.50195, 0.50033, 0.49809, 0.49524],\
        [0.51885, 0.52298, 0.52699, 0.53088, 0.53466, 0.53837, 0.54203, 0.54566, 0.54928, 0.55294, 0.55665,\
        0.56046, 0.56438, 0.56844, 0.57267, 0.57705, 0.58160, 0.58630, 0.59120, 0.59647, 0.60198, 0.60693,\
        0.61133, 0.61524, 0.61869, 0.62166, 0.62409, 0.62607, 0.62772, 0.62924, 0.63074, 0.63229, 0.63390,\
        0.63555, 0.63721, 0.63886, 0.64044, 0.64196, 0.64342, 0.64482, 0.64619, 0.64756, 0.64897, 0.65044,\
        0.65200, 0.65370, 0.65556, 0.65761, 0.65985, 0.66225, 0.66474, 0.66719, 0.66940, 0.67115, 0.67220,\
        0.67234, 0.67147, 0.66971, 0.66732, 0.66459, 0.66175, 0.65898, 0.65631, 0.65373, 0.65118, 0.64862,\
        0.64606, 0.64357, 0.64131, 0.63943, 0.63806, 0.63723, 0.63687, 0.63679, 0.63678, 0.63660, 0.63602,\
        0.63487, 0.63302, 0.63039, 0.62698, 0.62287, 0.61823, 0.61322, 0.60807, 0.60262, 0.59621, 0.58958,\
        0.58309, 0.57644, 0.56938, 0.56201, 0.55458, 0.54733, 0.54045, 0.53409, 0.52837, 0.52339, 0.51924,\
        0.51595, 0.51351, 0.51178, 0.51052, 0.50946, 0.50841, 0.50726, 0.50601, 0.50468, 0.50336, 0.50221,\
        0.50163, 0.50182, 0.50227, 0.50266, 0.50289, 0.50284, 0.50244, 0.50156, 0.50011, 0.49799, 0.49522],\
        [0.51639, 0.52056, 0.52459, 0.52850, 0.53229, 0.53599, 0.53961, 0.54318, 0.54673, 0.55029, 0.55390,\
        0.55759, 0.56140, 0.56536, 0.56950, 0.57383, 0.57839, 0.58320, 0.58832, 0.59375, 0.59914, 0.60381,\
        0.60775, 0.61132, 0.61455, 0.61726, 0.61932, 0.62078, 0.62188, 0.62289, 0.62397, 0.62518, 0.62651,\
        0.62795, 0.62949, 0.63108, 0.63269, 0.63428, 0.63586, 0.63744, 0.63902, 0.64063, 0.64228, 0.64399,\
        0.64578, 0.64766, 0.64964, 0.65172, 0.65390, 0.65613, 0.65834, 0.66042, 0.66220, 0.66351, 0.66417,\
        0.66406, 0.66314, 0.66148, 0.65926, 0.65671, 0.65400, 0.65126, 0.64855, 0.64587, 0.64320, 0.64051,\
        0.63784, 0.63529, 0.63303, 0.63123, 0.63003, 0.62947, 0.62944, 0.62974, 0.63010, 0.63027, 0.63002,\
        0.62915, 0.62753, 0.62505, 0.62168, 0.61751, 0.61276, 0.60763, 0.60218, 0.59704, 0.59063, 0.58370,\
        0.57668, 0.56954, 0.56223, 0.55481, 0.54744, 0.54033, 0.53363, 0.52747, 0.52197, 0.51722, 0.51335,\
        0.51044, 0.50849, 0.50739, 0.50677, 0.50624, 0.50559, 0.50475, 0.50374, 0.50263, 0.50150, 0.50050,\
        0.50001, 0.50059, 0.50121, 0.50175, 0.50213, 0.50224, 0.50199, 0.50126, 0.49992, 0.49787, 0.49517],\
        [0.51397, 0.51819, 0.52225, 0.52617, 0.52996, 0.53364, 0.53721, 0.54071, 0.54417, 0.54762, 0.55110,\
        0.55465, 0.55832, 0.56214, 0.56615, 0.57039, 0.57489, 0.57968, 0.58476, 0.59004, 0.59520, 0.59972,\
        0.60348, 0.60697, 0.61023, 0.61284, 0.61453, 0.61544, 0.61592, 0.61641, 0.61715, 0.61807, 0.61917,\
        0.62046, 0.62193, 0.62351, 0.62514, 0.62682, 0.62854, 0.63029, 0.63208, 0.63391, 0.63580, 0.63774,\
        0.63973, 0.64176, 0.64383, 0.64591, 0.64799, 0.65003, 0.65197, 0.65369, 0.65508, 0.65601, 0.65635,\
        0.65602, 0.65501, 0.65339, 0.65129, 0.64888, 0.64630, 0.64365, 0.64099, 0.63831, 0.63559, 0.63282,\
        0.63005, 0.62740, 0.62505, 0.62326, 0.62221, 0.62195, 0.62234, 0.62309, 0.62390, 0.62449, 0.62462,\
        0.62409, 0.62276, 0.62048, 0.61719, 0.61291, 0.60793, 0.60295, 0.59736, 0.59145, 0.58494, 0.57781,\
        0.57041, 0.56288, 0.55532, 0.54780, 0.54047, 0.53350, 0.52702, 0.52113, 0.51592, 0.51146, 0.50792,\
        0.50541, 0.50399, 0.50364, 0.50375, 0.50374, 0.50347, 0.50292, 0.50216, 0.50130, 0.50048, 0.49986,\
        0.49975, 0.50016, 0.50068, 0.50121, 0.50164, 0.50183, 0.50167, 0.50101, 0.49973, 0.49773, 0.49506],\
        [0.51159, 0.51585, 0.51994, 0.52388, 0.52767, 0.53132, 0.53484, 0.53825, 0.54159, 0.54489, 0.54821,\
        0.55159, 0.55507, 0.55870, 0.56253, 0.56660, 0.57094, 0.57556, 0.58042, 0.58540, 0.59025, 0.59463,\
        0.59846, 0.60213, 0.60571, 0.60838, 0.60970, 0.61000, 0.60978, 0.60979, 0.61036, 0.61107, 0.61196,\
        0.61315, 0.61460, 0.61621, 0.61789, 0.61964, 0.62150, 0.62342, 0.62540, 0.62745, 0.62956, 0.63172,\
        0.63389, 0.63605, 0.63817, 0.64022, 0.64219, 0.64403, 0.64570, 0.64711, 0.64816, 0.64877, 0.64883,\
        0.64830, 0.64719, 0.64556, 0.64352, 0.64122, 0.63878, 0.63628, 0.63374, 0.63116, 0.62850, 0.62572,\
        0.62288, 0.62009, 0.61760, 0.61572, 0.61477, 0.61485, 0.61574, 0.61702, 0.61832, 0.61933, 0.61986,\
        0.61971, 0.61872, 0.61675, 0.61366, 0.60940, 0.60399, 0.59857, 0.59309, 0.58663, 0.57959, 0.57208,\
        0.56432, 0.55647, 0.54868, 0.54107, 0.53377, 0.52696, 0.52077, 0.51527, 0.51045, 0.50634, 0.50321,\
        0.50119, 0.50032, 0.50080, 0.50157, 0.50194, 0.50198, 0.50167, 0.50111, 0.50048, 0.49995, 0.49962,\
        0.49963, 0.49997, 0.50042, 0.50091, 0.50133, 0.50154, 0.50141, 0.50080, 0.49953, 0.49755, 0.49489],\
        [0.50924, 0.51354, 0.51767, 0.52163, 0.52542, 0.52903, 0.53247, 0.53577, 0.53896, 0.54209, 0.54521,\
        0.54836, 0.55161, 0.55500, 0.55858, 0.56239, 0.56647, 0.57080, 0.57536, 0.58002, 0.58461, 0.58896,\
        0.59305, 0.59705, 0.60098, 0.60373, 0.60454, 0.60441, 0.60349, 0.60306, 0.60386, 0.60429, 0.60496,\
        0.60612, 0.60760, 0.60929, 0.61098, 0.61279, 0.61480, 0.61688, 0.61903, 0.62126, 0.62359, 0.62596,\
        0.62831, 0.63058, 0.63272, 0.63472, 0.63655, 0.63819, 0.63961, 0.64074, 0.64151, 0.64184, 0.64167,\
        0.64096, 0.63973, 0.63805, 0.63605, 0.63384, 0.63154, 0.62923, 0.62691, 0.62454, 0.62205, 0.61939,\
        0.61654, 0.61364, 0.61094, 0.60888, 0.60798, 0.60845, 0.60991, 0.61173, 0.61346, 0.61484, 0.61571,\
        0.61593, 0.61533, 0.61375, 0.61102, 0.60709, 0.60209, 0.59599, 0.58940, 0.58225, 0.57459, 0.56660,\
        0.55845, 0.55031, 0.54234, 0.53467, 0.52743, 0.52080, 0.51497, 0.51001, 0.50581, 0.50217, 0.49951,\
        0.49818, 0.49797, 0.49885, 0.49994, 0.50056, 0.50093, 0.50084, 0.50042, 0.49996, 0.49961, 0.49946,\
        0.49954, 0.49984, 0.50026, 0.50071, 0.50111, 0.50132, 0.50119, 0.50057, 0.49930, 0.49730, 0.49464],\
        [0.50692, 0.51127, 0.51544, 0.51943, 0.52321, 0.52678, 0.53012, 0.53328, 0.53629, 0.53920, 0.54207,\
        0.54495, 0.54791, 0.55099, 0.55426, 0.55774, 0.56146, 0.56544, 0.56963, 0.57397, 0.57835, 0.58269,\
        0.58698, 0.59126, 0.59539, 0.59779, 0.59844, 0.59836, 0.59732, 0.59686, 0.59759, 0.59780, 0.59840,\
        0.59953, 0.60102, 0.60291, 0.60445, 0.60631, 0.60852, 0.61074, 0.61299, 0.61538, 0.61792, 0.62051,\
        0.62303, 0.62540, 0.62754, 0.62945, 0.63113, 0.63257, 0.63376, 0.63465, 0.63519, 0.63530, 0.63493,\
        0.63405, 0.63270, 0.63096, 0.62894, 0.62679, 0.62465, 0.62258, 0.62057, 0.61854, 0.61637, 0.61395,\
        0.61124, 0.60833, 0.60546, 0.60317, 0.60224, 0.60314, 0.60519, 0.60742, 0.60940, 0.61098, 0.61207,\
        0.61259, 0.61236, 0.61120, 0.60884, 0.60513, 0.60005, 0.59353, 0.58608, 0.57812, 0.56981, 0.56131,\
        0.55279, 0.54442, 0.53634, 0.52868, 0.52155, 0.51512, 0.50966, 0.50537, 0.50200, 0.49925, 0.49706,\
        0.49612, 0.49625, 0.49717, 0.49834, 0.49935, 0.50008, 0.50019, 0.49986, 0.49958, 0.49939, 0.49934,\
        0.49946, 0.49973, 0.50010, 0.50053, 0.50093, 0.50113, 0.50099, 0.50032, 0.49902, 0.49700, 0.49431],\
        [0.50462, 0.50903, 0.51326, 0.51729, 0.52107, 0.52457, 0.52779, 0.53078, 0.53356, 0.53621, 0.53879,\
        0.54136, 0.54397, 0.54669, 0.54957, 0.55265, 0.55596, 0.55952, 0.56332, 0.56731, 0.57144, 0.57566,\
        0.57990, 0.58406, 0.58772, 0.59011, 0.59117, 0.59134, 0.59097, 0.59082, 0.59118, 0.59162, 0.59233,\
        0.59348, 0.59505, 0.59686, 0.59835, 0.60042, 0.60279, 0.60501, 0.60731, 0.60984, 0.61261, 0.61544,\
        0.61813, 0.62056, 0.62267, 0.62445, 0.62595, 0.62718, 0.62817, 0.62887, 0.62923, 0.62918, 0.62866,\
        0.62765, 0.62618, 0.62434, 0.62227, 0.62015, 0.61815, 0.61636, 0.61476, 0.61320, 0.61148, 0.60947,\
        0.60707, 0.60435, 0.60157, 0.59929, 0.59852, 0.59962, 0.60191, 0.60417, 0.60608, 0.60760, 0.60875,\
        0.60944, 0.60952, 0.60874, 0.60670, 0.60296, 0.59749, 0.59051, 0.58250, 0.57392, 0.56506, 0.55612,\
        0.54729, 0.53876, 0.53068, 0.52316, 0.51626, 0.51007, 0.50489, 0.50135, 0.49873, 0.49665, 0.49516,\
        0.49455, 0.49482, 0.49573, 0.49691, 0.49807, 0.49898, 0.49936, 0.49931, 0.49921, 0.49916, 0.49920,\
        0.49935, 0.49961, 0.49994, 0.50035, 0.50075, 0.50096, 0.50080, 0.50010, 0.49873, 0.49666, 0.49389],\
        [0.50236, 0.50683, 0.51115, 0.51524, 0.51901, 0.52242, 0.52548, 0.52825, 0.53077, 0.53312, 0.53536,\
        0.53757, 0.53980, 0.54211, 0.54456, 0.54718, 0.55002, 0.55311, 0.55646, 0.56005, 0.56387, 0.56785,\
        0.57188, 0.57576, 0.57913, 0.58160, 0.58309, 0.58381, 0.58409, 0.58438, 0.58487, 0.58555, 0.58648,\
        0.58775, 0.58934, 0.59110, 0.59292, 0.59508, 0.59743, 0.59967, 0.60200, 0.60469, 0.60773, 0.61083,\
        0.61367, 0.61612, 0.61813, 0.61974, 0.62102, 0.62204, 0.62284, 0.62340, 0.62366, 0.62353, 0.62292,\
        0.62181, 0.62023, 0.61827, 0.61612, 0.61398, 0.61208, 0.61059, 0.60948, 0.60849, 0.60735, 0.60586,\
        0.60391, 0.60157, 0.59918, 0.59731, 0.59677, 0.59777, 0.59970, 0.60173, 0.60325, 0.60446, 0.60551,\
        0.60622, 0.60643, 0.60597, 0.60432, 0.60044, 0.59445, 0.58697, 0.57850, 0.56948, 0.56020, 0.55093,\
        0.54189, 0.53330, 0.52533, 0.51813, 0.51174, 0.50610, 0.50115, 0.49817, 0.49612, 0.49453, 0.49352,\
        0.49321, 0.49357, 0.49446, 0.49561, 0.49678, 0.49774, 0.49834, 0.49862, 0.49876, 0.49888, 0.49904,\
        0.49924, 0.49949, 0.49980, 0.50019, 0.50059, 0.50080, 0.50063, 0.49989, 0.49843, 0.49625, 0.49339],\
        [0.50016, 0.50471, 0.50917, 0.51332, 0.51705, 0.52034, 0.52320, 0.52570, 0.52791, 0.52991, 0.53178,\
        0.53359, 0.53541, 0.53728, 0.53926, 0.54138, 0.54371, 0.54628, 0.54913, 0.55228, 0.55573, 0.55940,\
        0.56317, 0.56683, 0.57009, 0.57272, 0.57463, 0.57593, 0.57685, 0.57765, 0.57851, 0.57951, 0.58072,\
        0.58218, 0.58388, 0.58576, 0.58780, 0.59005, 0.59241, 0.59480, 0.59728, 0.60011, 0.60343, 0.60677,\
        0.60968, 0.61207, 0.61393, 0.61531, 0.61633, 0.61711, 0.61776, 0.61825, 0.61850, 0.61836, 0.61774,\
        0.61658, 0.61491, 0.61284, 0.61058, 0.60838, 0.60651, 0.60532, 0.60476, 0.60434, 0.60382, 0.60285,\
        0.60143, 0.59944, 0.59744, 0.59599, 0.59558, 0.59632, 0.59787, 0.59957, 0.60046, 0.60129, 0.60219,\
        0.60274, 0.60276, 0.60240, 0.60098, 0.59721, 0.59064, 0.58275, 0.57399, 0.56470, 0.55519, 0.54572,\
        0.53656, 0.52797, 0.52021, 0.51348, 0.50785, 0.50320, 0.49960, 0.49641, 0.49426, 0.49288, 0.49214,\
        0.49202, 0.49246, 0.49333, 0.49443, 0.49557, 0.49658, 0.49735, 0.49788, 0.49825, 0.49855, 0.49883,\
        0.49911, 0.49939, 0.49970, 0.50005, 0.50043, 0.50065, 0.50046, 0.49964, 0.49807, 0.49576, 0.49276],\
        [0.49809, 0.50279, 0.50741, 0.51159, 0.51523, 0.51832, 0.52091, 0.52309, 0.52495, 0.52657, 0.52805,\
        0.52944, 0.53082, 0.53224, 0.53373, 0.53534, 0.53712, 0.53912, 0.54143, 0.54409, 0.54714, 0.55051,\
        0.55406, 0.55759, 0.56087, 0.56372, 0.56605, 0.56790, 0.56942, 0.57077, 0.57208, 0.57348, 0.57500,\
        0.57671, 0.57861, 0.58067, 0.58290, 0.58529, 0.58781, 0.59042, 0.59321, 0.59631, 0.59982, 0.60320,\
        0.60605, 0.60836, 0.61001, 0.61110, 0.61181, 0.61235, 0.61288, 0.61339, 0.61372, 0.61369, 0.61315,\
        0.61200, 0.61028, 0.60812, 0.60575, 0.60354, 0.60168, 0.60070, 0.60065, 0.60062, 0.60067, 0.59996,\
        0.59892, 0.59736, 0.59575, 0.59462, 0.59426, 0.59472, 0.59575, 0.59682, 0.59752, 0.59810, 0.59871,\
        0.59895, 0.59855, 0.59785, 0.59590, 0.59192, 0.58560, 0.57776, 0.56898, 0.55963, 0.55004, 0.54050,\
        0.53131, 0.52277, 0.51523, 0.50896, 0.50410, 0.50061, 0.49733, 0.49467, 0.49273, 0.49152, 0.49097,\
        0.49098, 0.49147, 0.49232, 0.49338, 0.49450, 0.49554, 0.49643, 0.49714, 0.49771, 0.49818, 0.49859,\
        0.49896, 0.49929, 0.49961, 0.49993, 0.50029, 0.50050, 0.50027, 0.49934, 0.49762, 0.49515, 0.49197],\
        [0.49649, 0.50135, 0.50598, 0.51005, 0.51349, 0.51631, 0.51858, 0.52040, 0.52187, 0.52309, 0.52415,\
        0.52512, 0.52607, 0.52704, 0.52806, 0.52915, 0.53036, 0.53176, 0.53348, 0.53563, 0.53828, 0.54138,\
        0.54478, 0.54828, 0.55167, 0.55478, 0.55752, 0.55990, 0.56197, 0.56386, 0.56566, 0.56746, 0.56934,\
        0.57133, 0.57347, 0.57576, 0.57819, 0.58077, 0.58349, 0.58636, 0.58944, 0.59279, 0.59635, 0.59966,\
        0.60256, 0.60485, 0.60626, 0.60704, 0.60742, 0.60768, 0.60817, 0.60880, 0.60931, 0.60948, 0.60911,\
        0.60805, 0.60633, 0.60414, 0.60174, 0.59957, 0.59801, 0.59715, 0.59710, 0.59724, 0.59737, 0.59696,\
        0.59610, 0.59496, 0.59378, 0.59290, 0.59256, 0.59276, 0.59332, 0.59395, 0.59442, 0.59476, 0.59496,\
        0.59478, 0.59404, 0.59266, 0.59010, 0.58584, 0.57975, 0.57218, 0.56360, 0.55439, 0.54489, 0.53541,\
        0.52626, 0.51779, 0.51041, 0.50451, 0.50024, 0.49727, 0.49485, 0.49277, 0.49124, 0.49031, 0.48994,\
        0.49006, 0.49059, 0.49143, 0.49245, 0.49355, 0.49461, 0.49558, 0.49642, 0.49714, 0.49776, 0.49830,\
        0.49878, 0.49918, 0.49953, 0.49984, 0.50015, 0.50035, 0.50005, 0.49897, 0.49705, 0.49436, 0.49097],\
        [0.49506, 0.50013, 0.50467, 0.50855, 0.51173, 0.51423, 0.51612, 0.51754, 0.51860, 0.51942, 0.52007,\
        0.52064, 0.52118, 0.52174, 0.52233, 0.52293, 0.52357, 0.52435, 0.52544, 0.52705, 0.52931, 0.53220,\
        0.53554, 0.53909, 0.54266, 0.54606, 0.54921, 0.55206, 0.55464, 0.55703, 0.55929, 0.56150, 0.56373,\
        0.56603, 0.56843, 0.57095, 0.57359, 0.57637, 0.57930, 0.58239, 0.58566, 0.58911, 0.59263, 0.59597,\
        0.59900, 0.60134, 0.60242, 0.60303, 0.60317, 0.60307, 0.60366, 0.60449, 0.60522, 0.60565, 0.60554,\
        0.60465, 0.60292, 0.60079, 0.59854, 0.59652, 0.59508, 0.59427, 0.59403, 0.59406, 0.59407, 0.59378,\
        0.59316, 0.59233, 0.59147, 0.59079, 0.59044, 0.59043, 0.59065, 0.59093, 0.59111, 0.59114, 0.59093,\
        0.59032, 0.58914, 0.58718, 0.58412, 0.57965, 0.57368, 0.56639, 0.55812, 0.54919, 0.53992, 0.53063,\
        0.52162, 0.51328, 0.50604, 0.50045, 0.49677, 0.49429, 0.49242, 0.49094, 0.48986, 0.48923, 0.48904,\
        0.48926, 0.48982, 0.49064, 0.49164, 0.49271, 0.49378, 0.49479, 0.49572, 0.49655, 0.49729, 0.49795,\
        0.49853, 0.49903, 0.49944, 0.49976, 0.50003, 0.50022, 0.49981, 0.49848, 0.49628, 0.49333, 0.48970],\
        [0.49326, 0.49850, 0.50306, 0.50682, 0.50976, 0.51191, 0.51341, 0.51443, 0.51510, 0.51553, 0.51580,\
        0.51599, 0.51616, 0.51638, 0.51662, 0.51681, 0.51693, 0.51708, 0.51748, 0.51851, 0.52042, 0.52319,\
        0.52656, 0.53023, 0.53400, 0.53770, 0.54122, 0.54451, 0.54754, 0.55037, 0.55305, 0.55564, 0.55821,\
        0.56080, 0.56345, 0.56620, 0.56905, 0.57201, 0.57511, 0.57835, 0.58173, 0.58520, 0.58867, 0.59198,\
        0.59494, 0.59712, 0.59824, 0.59893, 0.59910, 0.59884, 0.59956, 0.60047, 0.60134, 0.60200, 0.60221,\
        0.60158, 0.59984, 0.59776, 0.59570, 0.59386, 0.59248, 0.59161, 0.59120, 0.59103, 0.59091, 0.59063,\
        0.59015, 0.58953, 0.58887, 0.58832, 0.58795, 0.58778, 0.58773, 0.58771, 0.58759, 0.58728, 0.58669,\
        0.58566, 0.58404, 0.58163, 0.57821, 0.57359, 0.56772, 0.56072, 0.55280, 0.54424, 0.53533, 0.52636,\
        0.51766, 0.50959, 0.50262, 0.49749, 0.49422, 0.49208, 0.49057, 0.48948, 0.48874, 0.48834, 0.48830,\
        0.48859, 0.48916, 0.48995, 0.49090, 0.49194, 0.49299, 0.49402, 0.49500, 0.49590, 0.49673, 0.49748,\
        0.49815, 0.49875, 0.49925, 0.49965, 0.49993, 0.50012, 0.49946, 0.49775, 0.49521, 0.49196, 0.48807],\
        [0.49081, 0.49619, 0.50088, 0.50462, 0.50738, 0.50921, 0.51032, 0.51095, 0.51127, 0.51138, 0.51133,\
        0.51118, 0.51103, 0.51099, 0.51100, 0.51090, 0.51058, 0.51014, 0.50984, 0.51022, 0.51186, 0.51464,\
        0.51809, 0.52190, 0.52586, 0.52983, 0.53369, 0.53734, 0.54076, 0.54397, 0.54700, 0.54992, 0.55278,\
        0.55564, 0.55852, 0.56147, 0.56450, 0.56762, 0.57085, 0.57418, 0.57759, 0.58104, 0.58441, 0.58758,\
        0.59034, 0.59243, 0.59377, 0.59458, 0.59495, 0.59516, 0.59577, 0.59664, 0.59754, 0.59827, 0.59862,\
        0.59818, 0.59669, 0.59479, 0.59291, 0.59124, 0.58991, 0.58900, 0.58843, 0.58810, 0.58783, 0.58752,\
        0.58710, 0.58659, 0.58604, 0.58554, 0.58514, 0.58482, 0.58456, 0.58427, 0.58386, 0.58323, 0.58229,\
        0.58089, 0.57890, 0.57615, 0.57249, 0.56780, 0.56206, 0.55535, 0.54782, 0.53971, 0.53127, 0.52278,\
        0.51455, 0.50697, 0.50054, 0.49587, 0.49280, 0.49080, 0.48946, 0.48855, 0.48798, 0.48771, 0.48774,\
        0.48804, 0.48859, 0.48934, 0.49022, 0.49120, 0.49222, 0.49323, 0.49421, 0.49514, 0.49601, 0.49682,\
        0.49756, 0.49823, 0.49881, 0.49932, 0.49973, 0.49995, 0.49871, 0.49655, 0.49366, 0.49013, 0.48601],\
        [0.48762, 0.49308, 0.49786, 0.50170, 0.50439, 0.50591, 0.50667, 0.50700, 0.50707, 0.50696, 0.50668,\
        0.50624, 0.50579, 0.50558, 0.50552, 0.50527, 0.50463, 0.50376, 0.50285, 0.50253, 0.50404, 0.50690,\
        0.51042, 0.51426, 0.51835, 0.52255, 0.52670, 0.53065, 0.53437, 0.53786, 0.54117, 0.54435, 0.54746,\
        0.55054, 0.55362, 0.55674, 0.55991, 0.56316, 0.56647, 0.56985, 0.57325, 0.57663, 0.57989, 0.58290,\
        0.58552, 0.58759, 0.58908, 0.59009, 0.59075, 0.59129, 0.59197, 0.59280, 0.59364, 0.59430, 0.59458,\
        0.59420, 0.59309, 0.59156, 0.58994, 0.58845, 0.58722, 0.58628, 0.58562, 0.58515, 0.58478, 0.58440,\
        0.58398, 0.58351, 0.58301, 0.58252, 0.58205, 0.58161, 0.58116, 0.58064, 0.57996, 0.57905, 0.57779,\
        0.57609, 0.57381, 0.57083, 0.56703, 0.56236, 0.55679, 0.55039, 0.54330, 0.53570, 0.52783, 0.51993,\
        0.51232, 0.50538, 0.49956, 0.49531, 0.49238, 0.49040, 0.48907, 0.48817, 0.48762, 0.48735, 0.48736,\
        0.48762, 0.48810, 0.48876, 0.48956, 0.49046, 0.49140, 0.49236, 0.49331, 0.49423, 0.49510, 0.49591,\
        0.49666, 0.49733, 0.49791, 0.49839, 0.49867, 0.49839, 0.49697, 0.49459, 0.49148, 0.48775, 0.48345],\
        [0.48361, 0.48907, 0.49386, 0.49779, 0.50049, 0.50179, 0.50233, 0.50251, 0.50246, 0.50223, 0.50186,\
        0.50124, 0.50051, 0.50020, 0.50013, 0.49983, 0.49896, 0.49790, 0.49683, 0.49639, 0.49766, 0.50038,\
        0.50372, 0.50743, 0.51158, 0.51597, 0.52033, 0.52448, 0.52839, 0.53208, 0.53558, 0.53896, 0.54225,\
        0.54550, 0.54873, 0.55199, 0.55527, 0.55860, 0.56196, 0.56534, 0.56871, 0.57201, 0.57516, 0.57805,\
        0.58058, 0.58266, 0.58428, 0.58549, 0.58641, 0.58719, 0.58798, 0.58879, 0.58955, 0.59012, 0.59033,\
        0.59004, 0.58922, 0.58803, 0.58670, 0.58542, 0.58430, 0.58339, 0.58268, 0.58213, 0.58167, 0.58123,\
        0.58078, 0.58030, 0.57980, 0.57928, 0.57874, 0.57819, 0.57757, 0.57684, 0.57593, 0.57477, 0.57327,\
        0.57132, 0.56883, 0.56571, 0.56187, 0.55727, 0.55193, 0.54589, 0.53927, 0.53224, 0.52499, 0.51776,\
        0.51083, 0.50458, 0.49943, 0.49550, 0.49271, 0.49071, 0.48927, 0.48825, 0.48759, 0.48722, 0.48713,\
        0.48728, 0.48765, 0.48819, 0.48888, 0.48966, 0.49051, 0.49138, 0.49225, 0.49311, 0.49392, 0.49468,\
        0.49536, 0.49595, 0.49642, 0.49668, 0.49661, 0.49591, 0.49433, 0.49185, 0.48864, 0.48479, 0.48037],\
        [0.47879, 0.48417, 0.48886, 0.49266, 0.49531, 0.49673, 0.49734, 0.49754, 0.49748, 0.49722, 0.49680,\
        0.49615, 0.49544, 0.49499, 0.49471, 0.49425, 0.49343, 0.49245, 0.49163, 0.49151, 0.49263, 0.49494,\
        0.49795, 0.50142, 0.50561, 0.51016, 0.51464, 0.51886, 0.52283, 0.52660, 0.53021, 0.53371, 0.53713,\
        0.54050, 0.54385, 0.54720, 0.55056, 0.55394, 0.55732, 0.56069, 0.56401, 0.56723, 0.57028, 0.57309,\
        0.57557, 0.57767, 0.57940, 0.58078, 0.58190, 0.58286, 0.58374, 0.58456, 0.58527, 0.58578, 0.58597,\
        0.58577, 0.58516, 0.58426, 0.58321, 0.58213, 0.58114, 0.58028, 0.57956, 0.57897, 0.57844, 0.57795,\
        0.57747, 0.57696, 0.57642, 0.57585, 0.57524, 0.57457, 0.57380, 0.57290, 0.57180, 0.57044, 0.56874,\
        0.56661, 0.56399, 0.56080, 0.55699, 0.55254, 0.54746, 0.54181, 0.53569, 0.52926, 0.52268, 0.51617,\
        0.50996, 0.50430, 0.49956, 0.49624, 0.49362, 0.49152, 0.48988, 0.48865, 0.48778, 0.48724, 0.48698,\
        0.48698, 0.48719, 0.48758, 0.48812, 0.48876, 0.48948, 0.49023, 0.49099, 0.49174, 0.49244, 0.49308,\
        0.49364, 0.49407, 0.49432, 0.49430, 0.49387, 0.49283, 0.49105, 0.48848, 0.48519, 0.48126, 0.47676],\
        [0.47323, 0.47848, 0.48302, 0.48668, 0.48932, 0.49094, 0.49179, 0.49213, 0.49215, 0.49194, 0.49152,\
        0.49095, 0.49034, 0.48982, 0.48936, 0.48883, 0.48813, 0.48741, 0.48695, 0.48713, 0.48824, 0.49029,\
        0.49307, 0.49647, 0.50056, 0.50521, 0.50960, 0.51370, 0.51762, 0.52138, 0.52503, 0.52859, 0.53209,\
        0.53554, 0.53896, 0.54237, 0.54578, 0.54918, 0.55256, 0.55590, 0.55917, 0.56232, 0.56529, 0.56803,\
        0.57049, 0.57262, 0.57443, 0.57595, 0.57722, 0.57832, 0.57928, 0.58012, 0.58082, 0.58130, 0.58151,\
        0.58140, 0.58097, 0.58030, 0.57947, 0.57859, 0.57773, 0.57694, 0.57625, 0.57563, 0.57507, 0.57454,\
        0.57401, 0.57346, 0.57288, 0.57225, 0.57156, 0.57078, 0.56989, 0.56885, 0.56759, 0.56608, 0.56423,\
        0.56199, 0.55930, 0.55611, 0.55238, 0.54811, 0.54333, 0.53809, 0.53249, 0.52667, 0.52078, 0.51502,\
        0.50958, 0.50472, 0.50071, 0.49757, 0.49491, 0.49258, 0.49066, 0.48916, 0.48805, 0.48729, 0.48683,\
        0.48663, 0.48666, 0.48688, 0.48724, 0.48772, 0.48827, 0.48887, 0.48949, 0.49008, 0.49063, 0.49111,\
        0.49148, 0.49169, 0.49167, 0.49135, 0.49059, 0.48927, 0.48728, 0.48459, 0.48122, 0.47723, 0.47266],\
        [0.46702, 0.47215, 0.47655, 0.48014, 0.48282, 0.48464, 0.48573, 0.48629, 0.48646, 0.48636, 0.48606,\
        0.48560, 0.48509, 0.48460, 0.48412, 0.48362, 0.48310, 0.48268, 0.48256, 0.48300, 0.48419, 0.48618,\
        0.48890, 0.49233, 0.49645, 0.50091, 0.50494, 0.50883, 0.51263, 0.51634, 0.51998, 0.52356, 0.52709,\
        0.53058, 0.53405, 0.53749, 0.54092, 0.54432, 0.54769, 0.55099, 0.55421, 0.55729, 0.56021, 0.56290,\
        0.56534, 0.56750, 0.56938, 0.57100, 0.57238, 0.57358, 0.57461, 0.57549, 0.57619, 0.57669, 0.57694,\
        0.57692, 0.57664, 0.57615, 0.57552, 0.57480, 0.57407, 0.57337, 0.57271, 0.57210, 0.57153, 0.57097,\
        0.57040, 0.56981, 0.56918, 0.56849, 0.56772, 0.56686, 0.56586, 0.56470, 0.56332, 0.56169, 0.55975,\
        0.55744, 0.55474, 0.55159, 0.54799, 0.54394, 0.53947, 0.53465, 0.52956, 0.52433, 0.51910, 0.51406,\
        0.50939, 0.50529, 0.50191, 0.49912, 0.49616, 0.49353, 0.49134, 0.48959, 0.48823, 0.48724, 0.48657,\
        0.48616, 0.48599, 0.48601, 0.48619, 0.48648, 0.48686, 0.48727, 0.48771, 0.48812, 0.48848, 0.48876,\
        0.48890, 0.48885, 0.48856, 0.48793, 0.48687, 0.48528, 0.48309, 0.48026, 0.47680, 0.47274, 0.46811]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=50 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax50vt5
    zgrid_H_Hmax_vmax50vt5=[[0.30774, 0.31496, 0.32187, 0.32848, 0.33480, 0.34084, 0.34661, 0.35211, 0.35735, 0.36234, 0.36709,\
        0.37161, 0.37589, 0.37994, 0.38375, 0.38732, 0.39063, 0.39365, 0.39632, 0.39853, 0.39990, 0.40003,\
        0.40027, 0.40080, 0.40173, 0.40318, 0.40520, 0.40783, 0.41103, 0.41476, 0.41894, 0.42350, 0.42836,\
        0.43346, 0.43872, 0.44408, 0.44950, 0.45493, 0.46032, 0.46563, 0.47079, 0.47578, 0.48053, 0.48499,\
        0.48915, 0.49298, 0.49665, 0.50048, 0.50382, 0.50698, 0.50998, 0.51286, 0.51564, 0.51832, 0.52092,\
        0.52345, 0.52589, 0.52826, 0.53056, 0.53277, 0.53490, 0.53693, 0.53887, 0.54070, 0.54240, 0.54397,\
        0.54540, 0.54667, 0.54778, 0.54871, 0.54946, 0.55004, 0.55045, 0.55069, 0.55079, 0.55075, 0.55059,\
        0.55031, 0.54989, 0.54935, 0.54868, 0.54785, 0.54687, 0.54571, 0.54437, 0.54286, 0.54116, 0.53928,\
        0.53725, 0.53507, 0.53274, 0.53028, 0.52768, 0.52495, 0.52207, 0.51905, 0.51587, 0.51254, 0.50905,\
        0.50537, 0.50149, 0.49730, 0.49262, 0.48810, 0.48355, 0.47866, 0.47350, 0.46822, 0.46288, 0.45751,\
        0.45208, 0.44656, 0.44088, 0.43501, 0.42892, 0.42258, 0.41596, 0.40905, 0.40183, 0.39429, 0.38644],\
        [0.31388, 0.32089, 0.32758, 0.33397, 0.34006, 0.34586, 0.35137, 0.35662, 0.36159, 0.36631, 0.37079,\
        0.37502, 0.37900, 0.38275, 0.38624, 0.38948, 0.39244, 0.39509, 0.39735, 0.39907, 0.39991, 0.39997,\
        0.40017, 0.40061, 0.40149, 0.40295, 0.40505, 0.40783, 0.41126, 0.41526, 0.41974, 0.42459, 0.42973,\
        0.43509, 0.44060, 0.44620, 0.45184, 0.45747, 0.46305, 0.46854, 0.47388, 0.47903, 0.48391, 0.48847,\
        0.49268, 0.49645, 0.49983, 0.50342, 0.50714, 0.51054, 0.51374, 0.51679, 0.51972, 0.52256, 0.52531,\
        0.52798, 0.53058, 0.53311, 0.53556, 0.53793, 0.54021, 0.54240, 0.54448, 0.54646, 0.54831, 0.55002,\
        0.55158, 0.55297, 0.55418, 0.55520, 0.55603, 0.55666, 0.55710, 0.55737, 0.55749, 0.55747, 0.55733,\
        0.55707, 0.55669, 0.55618, 0.55553, 0.55473, 0.55376, 0.55262, 0.55128, 0.54973, 0.54800, 0.54608,\
        0.54399, 0.54176, 0.53938, 0.53688, 0.53424, 0.53146, 0.52854, 0.52547, 0.52224, 0.51884, 0.51527,\
        0.51150, 0.50748, 0.50310, 0.49823, 0.49437, 0.49005, 0.48512, 0.47990, 0.47464, 0.46941, 0.46422,\
        0.45900, 0.45368, 0.44821, 0.44254, 0.43662, 0.43044, 0.42396, 0.41718, 0.41007, 0.40264, 0.39488],\
        [0.31975, 0.32656, 0.33303, 0.33919, 0.34504, 0.35059, 0.35584, 0.36081, 0.36551, 0.36995, 0.37413,\
        0.37805, 0.38172, 0.38514, 0.38829, 0.39115, 0.39371, 0.39592, 0.39770, 0.39893, 0.39949, 0.39961,\
        0.39971, 0.40004, 0.40090, 0.40238, 0.40458, 0.40758, 0.41130, 0.41563, 0.42043, 0.42559, 0.43102,\
        0.43664, 0.44239, 0.44821, 0.45404, 0.45986, 0.46562, 0.47127, 0.47678, 0.48208, 0.48710, 0.49177,\
        0.49604, 0.49999, 0.50336, 0.50685, 0.51047, 0.51398, 0.51733, 0.52054, 0.52363, 0.52663, 0.52955,\
        0.53238, 0.53515, 0.53784, 0.54045, 0.54299, 0.54544, 0.54779, 0.55004, 0.55218, 0.55419, 0.55605,\
        0.55774, 0.55926, 0.56059, 0.56171, 0.56261, 0.56329, 0.56375, 0.56403, 0.56416, 0.56416, 0.56404,\
        0.56380, 0.56344, 0.56296, 0.56234, 0.56156, 0.56062, 0.55948, 0.55812, 0.55655, 0.55475, 0.55277,\
        0.55061, 0.54831, 0.54588, 0.54332, 0.54063, 0.53780, 0.53483, 0.53170, 0.52840, 0.52492, 0.52124,\
        0.51735, 0.51318, 0.50871, 0.50438, 0.50068, 0.49627, 0.49114, 0.48583, 0.48062, 0.47558, 0.47062,\
        0.46566, 0.46058, 0.45533, 0.44985, 0.44412, 0.43809, 0.43175, 0.42509, 0.41810, 0.41076, 0.40308],\
        [0.32539, 0.33198, 0.33824, 0.34416, 0.34976, 0.35505, 0.36003, 0.36472, 0.36913, 0.37327, 0.37714,\
        0.38076, 0.38411, 0.38718, 0.38997, 0.39245, 0.39460, 0.39638, 0.39773, 0.39860, 0.39900, 0.39908,\
        0.39911, 0.39938, 0.40019, 0.40161, 0.40393, 0.40722, 0.41130, 0.41599, 0.42113, 0.42661, 0.43232,\
        0.43819, 0.44415, 0.45016, 0.45616, 0.46213, 0.46803, 0.47382, 0.47947, 0.48491, 0.49007, 0.49488,\
        0.49914, 0.50306, 0.50670, 0.51024, 0.51382, 0.51737, 0.52082, 0.52417, 0.52742, 0.53059, 0.53367,\
        0.53668, 0.53961, 0.54248, 0.54528, 0.54799, 0.55061, 0.55314, 0.55556, 0.55787, 0.56005, 0.56207,\
        0.56392, 0.56557, 0.56702, 0.56825, 0.56923, 0.56996, 0.57043, 0.57070, 0.57083, 0.57083, 0.57073,\
        0.57051, 0.57017, 0.56971, 0.56912, 0.56837, 0.56745, 0.56631, 0.56494, 0.56332, 0.56145, 0.55937,\
        0.55712, 0.55474, 0.55224, 0.54963, 0.54689, 0.54400, 0.54097, 0.53777, 0.53439, 0.53083, 0.52707,\
        0.52309, 0.51890, 0.51459, 0.51040, 0.50632, 0.50177, 0.49654, 0.49123, 0.48621, 0.48145, 0.47679,\
        0.47212, 0.46730, 0.46228, 0.45699, 0.45142, 0.44555, 0.43935, 0.43281, 0.42591, 0.41866, 0.41105],\
        [0.33080, 0.33718, 0.34321, 0.34890, 0.35425, 0.35927, 0.36397, 0.36837, 0.37248, 0.37632, 0.37989,\
        0.38319, 0.38622, 0.38896, 0.39140, 0.39351, 0.39528, 0.39668, 0.39769, 0.39831, 0.39858, 0.39861,\
        0.39858, 0.39873, 0.39935, 0.40068, 0.40328, 0.40697, 0.41146, 0.41652, 0.42199, 0.42776, 0.43372,\
        0.43981, 0.44596, 0.45212, 0.45825, 0.46432, 0.47031, 0.47620, 0.48195, 0.48750, 0.49274, 0.49763,\
        0.50208, 0.50599, 0.50974, 0.51341, 0.51705, 0.52067, 0.52423, 0.52772, 0.53112, 0.53445, 0.53770,\
        0.54089, 0.54401, 0.54707, 0.55005, 0.55295, 0.55576, 0.55847, 0.56108, 0.56357, 0.56593, 0.56813,\
        0.57014, 0.57193, 0.57350, 0.57485, 0.57594, 0.57671, 0.57718, 0.57741, 0.57751, 0.57753, 0.57745,\
        0.57723, 0.57690, 0.57646, 0.57590, 0.57518, 0.57427, 0.57315, 0.57176, 0.57007, 0.56809, 0.56588,\
        0.56352, 0.56105, 0.55849, 0.55582, 0.55303, 0.55010, 0.54700, 0.54373, 0.54028, 0.53664, 0.53281,\
        0.52878, 0.52458, 0.52029, 0.51597, 0.51152, 0.50670, 0.50148, 0.49620, 0.49156, 0.48716, 0.48284,\
        0.47847, 0.47391, 0.46908, 0.46396, 0.45855, 0.45282, 0.44675, 0.44032, 0.43351, 0.42634, 0.41879],\
        [0.33600, 0.34217, 0.34798, 0.35343, 0.35852, 0.36327, 0.36768, 0.37179, 0.37560, 0.37913, 0.38240,\
        0.38540, 0.38812, 0.39055, 0.39267, 0.39445, 0.39589, 0.39697, 0.39771, 0.39816, 0.39837, 0.39840,\
        0.39836, 0.39840, 0.39876, 0.40007, 0.40304, 0.40717, 0.41204, 0.41741, 0.42315, 0.42915, 0.43532,\
        0.44158, 0.44787, 0.45413, 0.46034, 0.46645, 0.47247, 0.47838, 0.48419, 0.48983, 0.49516, 0.49992,\
        0.50444, 0.50859, 0.51252, 0.51634, 0.52013, 0.52387, 0.52756, 0.53119, 0.53475, 0.53825, 0.54168,\
        0.54505, 0.54836, 0.55162, 0.55480, 0.55790, 0.56091, 0.56381, 0.56660, 0.56929, 0.57186, 0.57426,\
        0.57643, 0.57835, 0.58004, 0.58154, 0.58277, 0.58360, 0.58401, 0.58415, 0.58421, 0.58428, 0.58424,\
        0.58400, 0.58365, 0.58324, 0.58271, 0.58199, 0.58111, 0.58002, 0.57861, 0.57683, 0.57469, 0.57232,\
        0.56982, 0.56726, 0.56464, 0.56194, 0.55910, 0.55611, 0.55296, 0.54962, 0.54610, 0.54239, 0.53849,\
        0.53441, 0.53018, 0.52583, 0.52135, 0.51667, 0.51170, 0.50645, 0.50135, 0.49716, 0.49287, 0.48887,\
        0.48480, 0.48045, 0.47577, 0.47078, 0.46549, 0.45990, 0.45395, 0.44762, 0.44090, 0.43378, 0.42628],\
        [0.34101, 0.34697, 0.35256, 0.35777, 0.36260, 0.36707, 0.37120, 0.37500, 0.37851, 0.38174, 0.38471,\
        0.38742, 0.38987, 0.39203, 0.39387, 0.39538, 0.39653, 0.39736, 0.39790, 0.39825, 0.39847, 0.39862,\
        0.39871, 0.39884, 0.39925, 0.40056, 0.40379, 0.40816, 0.41323, 0.41879, 0.42471, 0.43087, 0.43718,\
        0.44356, 0.44994, 0.45627, 0.46249, 0.46859, 0.47455, 0.48039, 0.48615, 0.49182, 0.49735, 0.50223,\
        0.50665, 0.51092, 0.51506, 0.51910, 0.52307, 0.52698, 0.53083, 0.53462, 0.53835, 0.54202, 0.54563,\
        0.54919, 0.55270, 0.55617, 0.55957, 0.56289, 0.56610, 0.56919, 0.57217, 0.57506, 0.57785, 0.58049,\
        0.58285, 0.58487, 0.58665, 0.58830, 0.58974, 0.59067, 0.59099, 0.59098, 0.59095, 0.59112, 0.59120,\
        0.59085, 0.59041, 0.59010, 0.58959, 0.58883, 0.58796, 0.58693, 0.58552, 0.58361, 0.58126, 0.57866,\
        0.57601, 0.57338, 0.57073, 0.56800, 0.56513, 0.56209, 0.55888, 0.55547, 0.55188, 0.54810, 0.54414,\
        0.54001, 0.53573, 0.53131, 0.52674, 0.52198, 0.51704, 0.51202, 0.50727, 0.50285, 0.49860, 0.49506,\
        0.49118, 0.48693, 0.48232, 0.47740, 0.47223, 0.46678, 0.46096, 0.45473, 0.44807, 0.44099, 0.43352],\
        [0.34584, 0.35160, 0.35697, 0.36195, 0.36652, 0.37071, 0.37454, 0.37804, 0.38123, 0.38416, 0.38685,\
        0.38930, 0.39151, 0.39344, 0.39507, 0.39636, 0.39729, 0.39792, 0.39832, 0.39862, 0.39893, 0.39931,\
        0.39967, 0.40011, 0.40093, 0.40263, 0.40575, 0.41004, 0.41511, 0.42072, 0.42671, 0.43294, 0.43933,\
        0.44579, 0.45222, 0.45857, 0.46478, 0.47080, 0.47663, 0.48229, 0.48784, 0.49335, 0.49873, 0.50381,\
        0.50851, 0.51304, 0.51744, 0.52173, 0.52592, 0.53004, 0.53407, 0.53804, 0.54194, 0.54579, 0.54958,\
        0.55333, 0.55705, 0.56074, 0.56438, 0.56793, 0.57137, 0.57466, 0.57781, 0.58089, 0.58392, 0.58685,\
        0.58946, 0.59155, 0.59333, 0.59511, 0.59685, 0.59790, 0.59811, 0.59796, 0.59779, 0.59810, 0.59842,\
        0.59779, 0.59717, 0.59715, 0.59657, 0.59571, 0.59483, 0.59389, 0.59252, 0.59042, 0.58775, 0.58488,\
        0.58208, 0.57942, 0.57680, 0.57407, 0.57116, 0.56807, 0.56478, 0.56131, 0.55764, 0.55379, 0.54977,\
        0.54559, 0.54127, 0.53681, 0.53222, 0.52749, 0.52267, 0.51787, 0.51325, 0.50887, 0.50493, 0.50137,\
        0.49749, 0.49328, 0.48866, 0.48378, 0.47874, 0.47346, 0.46778, 0.46163, 0.45500, 0.44793, 0.44047],\
        [0.35049, 0.35607, 0.36124, 0.36599, 0.37031, 0.37422, 0.37774, 0.38092, 0.38380, 0.38643, 0.38884,\
        0.39105, 0.39306, 0.39482, 0.39629, 0.39744, 0.39824, 0.39872, 0.39904, 0.39931, 0.39974, 0.40039,\
        0.40107, 0.40191, 0.40319, 0.40528, 0.40843, 0.41261, 0.41759, 0.42315, 0.42912, 0.43537, 0.44179,\
        0.44828, 0.45474, 0.46110, 0.46726, 0.47318, 0.47882, 0.48420, 0.48942, 0.49464, 0.49992, 0.50512,\
        0.51017, 0.51505, 0.51976, 0.52434, 0.52878, 0.53311, 0.53734, 0.54149, 0.54557, 0.54959, 0.55357,\
        0.55751, 0.56143, 0.56535, 0.56924, 0.57308, 0.57678, 0.58027, 0.58358, 0.58680, 0.59005, 0.59333,\
        0.59633, 0.59847, 0.60022, 0.60211, 0.60390, 0.60506, 0.60538, 0.60530, 0.60518, 0.60540, 0.60553,\
        0.60498, 0.60440, 0.60427, 0.60365, 0.60274, 0.60182, 0.60090, 0.59950, 0.59717, 0.59411, 0.59094,\
        0.58806, 0.58546, 0.58292, 0.58020, 0.57723, 0.57406, 0.57070, 0.56715, 0.56340, 0.55947, 0.55539,\
        0.55117, 0.54682, 0.54236, 0.53779, 0.53314, 0.52846, 0.52382, 0.51934, 0.51511, 0.51119, 0.50744,\
        0.50356, 0.49936, 0.49467, 0.48980, 0.48499, 0.47995, 0.47442, 0.46830, 0.46166, 0.45458, 0.44712],\
        [0.35499, 0.36040, 0.36538, 0.36991, 0.37398, 0.37761, 0.38082, 0.38367, 0.38622, 0.38854, 0.39068,\
        0.39268, 0.39452, 0.39617, 0.39756, 0.39865, 0.39938, 0.39979, 0.40012, 0.40048, 0.40105, 0.40188,\
        0.40287, 0.40411, 0.40581, 0.40821, 0.41147, 0.41561, 0.42049, 0.42597, 0.43188, 0.43811, 0.44452,\
        0.45102, 0.45750, 0.46386, 0.47000, 0.47584, 0.48129, 0.48635, 0.49114, 0.49597, 0.50115, 0.50657,\
        0.51194, 0.51716, 0.52219, 0.52704, 0.53172, 0.53626, 0.54068, 0.54501, 0.54927, 0.55347, 0.55762,\
        0.56175, 0.56587, 0.57000, 0.57418, 0.57836, 0.58240, 0.58613, 0.58956, 0.59290, 0.59630, 0.59986,\
        0.60318, 0.60564, 0.60763, 0.60951, 0.61118, 0.61233, 0.61286, 0.61299, 0.61299, 0.61303, 0.61293,\
        0.61253, 0.61204, 0.61159, 0.61093, 0.61003, 0.60901, 0.60781, 0.60612, 0.60361, 0.60030, 0.59696,\
        0.59410, 0.59160, 0.58923, 0.58647, 0.58334, 0.58006, 0.57664, 0.57301, 0.56917, 0.56515, 0.56100,\
        0.55674, 0.55238, 0.54794, 0.54342, 0.53886, 0.53429, 0.52978, 0.52541, 0.52122, 0.51722, 0.51331,\
        0.50928, 0.50496, 0.50018, 0.49550, 0.49105, 0.48632, 0.48088, 0.47471, 0.46800, 0.46088, 0.45342],\
        [0.35933, 0.36459, 0.36940, 0.37373, 0.37757, 0.38092, 0.38381, 0.38631, 0.38851, 0.39051, 0.39239,\
        0.39418, 0.39589, 0.39749, 0.39884, 0.39987, 0.40065, 0.40112, 0.40157, 0.40210, 0.40283, 0.40382,\
        0.40507, 0.40664, 0.40867, 0.41132, 0.41470, 0.41884, 0.42365, 0.42904, 0.43489, 0.44106, 0.44746,\
        0.45396, 0.46046, 0.46685, 0.47301, 0.47883, 0.48420, 0.48902, 0.49332, 0.49772, 0.50286, 0.50847,\
        0.51411, 0.51960, 0.52488, 0.52996, 0.53485, 0.53957, 0.54416, 0.54865, 0.55308, 0.55745, 0.56179,\
        0.56610, 0.57039, 0.57471, 0.57913, 0.58371, 0.58826, 0.59230, 0.59588, 0.59941, 0.60303, 0.60675,\
        0.61018, 0.61299, 0.61528, 0.61723, 0.61883, 0.61998, 0.62064, 0.62092, 0.62099, 0.62094, 0.62074,\
        0.62035, 0.61983, 0.61922, 0.61844, 0.61745, 0.61625, 0.61475, 0.61275, 0.61007, 0.60681, 0.60349,\
        0.60048, 0.59805, 0.59586, 0.59284, 0.58943, 0.58608, 0.58263, 0.57890, 0.57493, 0.57081, 0.56659,\
        0.56230, 0.55794, 0.55353, 0.54906, 0.54457, 0.54009, 0.53566, 0.53134, 0.52714, 0.52305, 0.51898,\
        0.51480, 0.51038, 0.50573, 0.50125, 0.49705, 0.49263, 0.48710, 0.48073, 0.47392, 0.46678, 0.45934],\
        [0.36351, 0.36864, 0.37331, 0.37747, 0.38110, 0.38417, 0.38673, 0.38886, 0.39069, 0.39235, 0.39395,\
        0.39555, 0.39713, 0.39870, 0.40012, 0.40114, 0.40197, 0.40264, 0.40328, 0.40402, 0.40494, 0.40611,\
        0.40758, 0.40941, 0.41170, 0.41455, 0.41803, 0.42218, 0.42695, 0.43226, 0.43803, 0.44416, 0.45052,\
        0.45703, 0.46357, 0.47001, 0.47624, 0.48213, 0.48758, 0.49244, 0.49638, 0.50029, 0.50548, 0.51115,\
        0.51689, 0.52252, 0.52796, 0.53319, 0.53822, 0.54309, 0.54782, 0.55246, 0.55704, 0.56159, 0.56612,\
        0.57061, 0.57506, 0.57950, 0.58407, 0.58898, 0.59424, 0.59886, 0.60263, 0.60651, 0.61040, 0.61421,\
        0.61772, 0.62072, 0.62320, 0.62523, 0.62683, 0.62799, 0.62871, 0.62907, 0.62917, 0.62908, 0.62881,\
        0.62837, 0.62778, 0.62704, 0.62614, 0.62501, 0.62363, 0.62189, 0.61969, 0.61697, 0.61384, 0.61061,\
        0.60758, 0.60499, 0.60252, 0.59901, 0.59540, 0.59217, 0.58872, 0.58482, 0.58067, 0.57642, 0.57214,\
        0.56782, 0.56348, 0.55909, 0.55467, 0.55024, 0.54581, 0.54143, 0.53711, 0.53287, 0.52869, 0.52450,\
        0.52023, 0.51584, 0.51139, 0.50707, 0.50299, 0.49868, 0.49266, 0.48611, 0.47930, 0.47221, 0.46485],\
        [0.36752, 0.37254, 0.37710, 0.38114, 0.38458, 0.38740, 0.38961, 0.39135, 0.39277, 0.39407, 0.39541,\
        0.39684, 0.39830, 0.39975, 0.40122, 0.40243, 0.40343, 0.40432, 0.40520, 0.40616, 0.40728, 0.40866,\
        0.41033, 0.41236, 0.41484, 0.41783, 0.42140, 0.42555, 0.43028, 0.43552, 0.44122, 0.44728, 0.45361,\
        0.46013, 0.46671, 0.47324, 0.47958, 0.48560, 0.49119, 0.49634, 0.50072, 0.50429, 0.50918, 0.51466,\
        0.52032, 0.52596, 0.53145, 0.53677, 0.54190, 0.54686, 0.55169, 0.55644, 0.56116, 0.56589, 0.57064,\
        0.57536, 0.58001, 0.58455, 0.58907, 0.59397, 0.59972, 0.60517, 0.60968, 0.61400, 0.61819, 0.62217,\
        0.62577, 0.62888, 0.63147, 0.63354, 0.63515, 0.63630, 0.63705, 0.63743, 0.63753, 0.63740, 0.63707,\
        0.63655, 0.63587, 0.63502, 0.63399, 0.63273, 0.63119, 0.62930, 0.62700, 0.62428, 0.62124, 0.61807,\
        0.61497, 0.61201, 0.60889, 0.60526, 0.60165, 0.59850, 0.59494, 0.59069, 0.58631, 0.58195, 0.57762,\
        0.57330, 0.56897, 0.56462, 0.56023, 0.55582, 0.55142, 0.54704, 0.54271, 0.53841, 0.53414, 0.52986,\
        0.52554, 0.52115, 0.51676, 0.51242, 0.50805, 0.50307, 0.49706, 0.49079, 0.48414, 0.47720, 0.46998],\
        [0.37134, 0.37627, 0.38076, 0.38472, 0.38804, 0.39063, 0.39249, 0.39381, 0.39479, 0.39568, 0.39677,\
        0.39814, 0.39961, 0.40101, 0.40249, 0.40386, 0.40508, 0.40620, 0.40730, 0.40848, 0.40982, 0.41138,\
        0.41322, 0.41542, 0.41804, 0.42113, 0.42475, 0.42889, 0.43357, 0.43874, 0.44435, 0.45034, 0.45663,\
        0.46313, 0.46976, 0.47640, 0.48288, 0.48905, 0.49471, 0.49986, 0.50452, 0.50878, 0.51350, 0.51875,\
        0.52428, 0.52986, 0.53536, 0.54071, 0.54589, 0.55090, 0.55578, 0.56060, 0.56544, 0.57034, 0.57535,\
        0.58040, 0.58536, 0.59010, 0.59468, 0.59932, 0.60528, 0.61139, 0.61683, 0.62177, 0.62635, 0.63054,\
        0.63427, 0.63745, 0.64008, 0.64217, 0.64375, 0.64489, 0.64561, 0.64599, 0.64606, 0.64588, 0.64547,\
        0.64487, 0.64409, 0.64313, 0.64198, 0.64060, 0.63896, 0.63698, 0.63463, 0.63191, 0.62890, 0.62571,\
        0.62245, 0.61913, 0.61565, 0.61196, 0.60832, 0.60482, 0.60086, 0.59632, 0.59182, 0.58740, 0.58305,\
        0.57874, 0.57444, 0.57009, 0.56571, 0.56131, 0.55690, 0.55250, 0.54811, 0.54375, 0.53940, 0.53503,\
        0.53063, 0.52620, 0.52172, 0.51716, 0.51237, 0.50704, 0.50124, 0.49507, 0.48855, 0.48183, 0.47479],\
        [0.37495, 0.37979, 0.38422, 0.38816, 0.39144, 0.39386, 0.39537, 0.39627, 0.39682, 0.39724, 0.39804,\
        0.39944, 0.40103, 0.40256, 0.40409, 0.40558, 0.40697, 0.40829, 0.40961, 0.41099, 0.41251, 0.41423,\
        0.41623, 0.41855, 0.42127, 0.42442, 0.42804, 0.43216, 0.43676, 0.44184, 0.44735, 0.45324, 0.45946,\
        0.46594, 0.47259, 0.47932, 0.48598, 0.49235, 0.49818, 0.50336, 0.50821, 0.51292, 0.51782, 0.52305,\
        0.52853, 0.53409, 0.53960, 0.54498, 0.55019, 0.55522, 0.56012, 0.56497, 0.56986, 0.57492, 0.58020,\
        0.58567, 0.59113, 0.59626, 0.60124, 0.60637, 0.61226, 0.61851, 0.62445, 0.62990, 0.63486, 0.63930,\
        0.64315, 0.64639, 0.64902, 0.65107, 0.65261, 0.65369, 0.65437, 0.65470, 0.65472, 0.65447, 0.65398,\
        0.65328, 0.65239, 0.65132, 0.65008, 0.64862, 0.64690, 0.64487, 0.64249, 0.63976, 0.63672, 0.63345,\
        0.63000, 0.62641, 0.62266, 0.61879, 0.61488, 0.61087, 0.60653, 0.60190, 0.59730, 0.59282, 0.58845,\
        0.58417, 0.57987, 0.57551, 0.57111, 0.56668, 0.56224, 0.55778, 0.55333, 0.54888, 0.54443, 0.53997,\
        0.53548, 0.53093, 0.52631, 0.52153, 0.51648, 0.51104, 0.50520, 0.49895, 0.49270, 0.48623, 0.47937],\
        [0.37830, 0.38304, 0.38742, 0.39135, 0.39467, 0.39707, 0.39816, 0.39870, 0.39902, 0.39888, 0.39941,\
        0.40089, 0.40264, 0.40436, 0.40602, 0.40763, 0.40916, 0.41065, 0.41214, 0.41368, 0.41535, 0.41720,\
        0.41931, 0.42172, 0.42448, 0.42765, 0.43125, 0.43530, 0.43981, 0.44477, 0.45015, 0.45592, 0.46203,\
        0.46843, 0.47506, 0.48183, 0.48863, 0.49523, 0.50127, 0.50660, 0.51167, 0.51672, 0.52191, 0.52731,\
        0.53288, 0.53851, 0.54409, 0.54953, 0.55477, 0.55982, 0.56471, 0.56953, 0.57442, 0.57955, 0.58506,\
        0.59097, 0.59713, 0.60279, 0.60826, 0.61397, 0.62008, 0.62644, 0.63266, 0.63848, 0.64377, 0.64843,\
        0.65240, 0.65566, 0.65823, 0.66021, 0.66166, 0.66267, 0.66328, 0.66354, 0.66349, 0.66317, 0.66258,\
        0.66176, 0.66075, 0.65958, 0.65825, 0.65674, 0.65498, 0.65292, 0.65053, 0.64777, 0.64467, 0.64126,\
        0.63761, 0.63376, 0.62974, 0.62560, 0.62136, 0.61697, 0.61238, 0.60764, 0.60290, 0.59828, 0.59389,\
        0.58962, 0.58528, 0.58086, 0.57640, 0.57192, 0.56741, 0.56288, 0.55834, 0.55379, 0.54923, 0.54466,\
        0.54005, 0.53538, 0.53061, 0.52567, 0.52047, 0.51496, 0.50910, 0.50300, 0.49691, 0.49056, 0.48378],\
        [0.38139, 0.38599, 0.39028, 0.39415, 0.39744, 0.39996, 0.40079, 0.40114, 0.40133, 0.40113, 0.40162,\
        0.40304, 0.40479, 0.40658, 0.40834, 0.41003, 0.41167, 0.41328, 0.41489, 0.41655, 0.41833, 0.42027,\
        0.42245, 0.42489, 0.42767, 0.43081, 0.43435, 0.43831, 0.44269, 0.44750, 0.45272, 0.45832, 0.46427,\
        0.47053, 0.47706, 0.48378, 0.49060, 0.49735, 0.50354, 0.50917, 0.51463, 0.52010, 0.52567, 0.53138,\
        0.53719, 0.54301, 0.54874, 0.55430, 0.55962, 0.56470, 0.56957, 0.57434, 0.57915, 0.58424, 0.58988,\
        0.59612, 0.60269, 0.60915, 0.61539, 0.62172, 0.62824, 0.63485, 0.64133, 0.64745, 0.65302, 0.65789,\
        0.66195, 0.66519, 0.66766, 0.66951, 0.67085, 0.67176, 0.67229, 0.67246, 0.67235, 0.67195, 0.67125,\
        0.67030, 0.66915, 0.66787, 0.66647, 0.66491, 0.66314, 0.66109, 0.65868, 0.65588, 0.65268, 0.64910,\
        0.64522, 0.64110, 0.63680, 0.63236, 0.62781, 0.62314, 0.61835, 0.61348, 0.60863, 0.60390, 0.59943,\
        0.59512, 0.59066, 0.58609, 0.58155, 0.57701, 0.57241, 0.56778, 0.56312, 0.55846, 0.55379, 0.54910,\
        0.54438, 0.53958, 0.53468, 0.52962, 0.52434, 0.51883, 0.51307, 0.50716, 0.50113, 0.49482, 0.48802],\
        [0.38426, 0.38864, 0.39279, 0.39657, 0.39966, 0.40195, 0.40317, 0.40369, 0.40395, 0.40413, 0.40475,\
        0.40598, 0.40759, 0.40933, 0.41109, 0.41282, 0.41451, 0.41619, 0.41787, 0.41960, 0.42144, 0.42343,\
        0.42563, 0.42807, 0.43082, 0.43389, 0.43733, 0.44116, 0.44538, 0.45001, 0.45502, 0.46042, 0.46616,\
        0.47222, 0.47856, 0.48511, 0.49178, 0.49843, 0.50487, 0.51101, 0.51702, 0.52302, 0.52908, 0.53521,\
        0.54137, 0.54749, 0.55348, 0.55924, 0.56471, 0.56987, 0.57475, 0.57946, 0.58417, 0.58912, 0.59476,\
        0.60135, 0.60842, 0.61555, 0.62257, 0.62955, 0.63654, 0.64349, 0.65026, 0.65667, 0.66252, 0.66760,\
        0.67174, 0.67492, 0.67723, 0.67889, 0.68010, 0.68093, 0.68136, 0.68143, 0.68125, 0.68080, 0.67999,\
        0.67885, 0.67753, 0.67614, 0.67469, 0.67311, 0.67135, 0.66931, 0.66689, 0.66404, 0.66071, 0.65693,\
        0.65278, 0.64837, 0.64377, 0.63905, 0.63422, 0.62932, 0.62434, 0.61935, 0.61441, 0.60961, 0.60502,\
        0.60059, 0.59584, 0.59111, 0.58654, 0.58194, 0.57723, 0.57246, 0.56766, 0.56287, 0.55809, 0.55328,\
        0.54845, 0.54354, 0.53853, 0.53338, 0.52806, 0.52256, 0.51689, 0.51107, 0.50511, 0.49882, 0.49194],\
        [0.38696, 0.39106, 0.39498, 0.39867, 0.40174, 0.40396, 0.40544, 0.40634, 0.40692, 0.40746, 0.40826,\
        0.40944, 0.41092, 0.41256, 0.41427, 0.41598, 0.41767, 0.41936, 0.42107, 0.42283, 0.42468, 0.42667,\
        0.42884, 0.43124, 0.43391, 0.43689, 0.44019, 0.44385, 0.44788, 0.45229, 0.45707, 0.46221, 0.46771,\
        0.47353, 0.47963, 0.48597, 0.49244, 0.49899, 0.50568, 0.51233, 0.51891, 0.52549, 0.53210, 0.53874,\
        0.54536, 0.55188, 0.55822, 0.56429, 0.57001, 0.57531, 0.58025, 0.58497, 0.58968, 0.59454, 0.59999,\
        0.60703, 0.61458, 0.62227, 0.62991, 0.63747, 0.64491, 0.65220, 0.65925, 0.66596, 0.67212, 0.67744,\
        0.68166, 0.68475, 0.68683, 0.68823, 0.68933, 0.69016, 0.69047, 0.69040, 0.69017, 0.68974, 0.68880,\
        0.68740, 0.68589, 0.68439, 0.68288, 0.68130, 0.67955, 0.67752, 0.67511, 0.67219, 0.66869, 0.66466,\
        0.66021, 0.65548, 0.65058, 0.64559, 0.64052, 0.63541, 0.63028, 0.62516, 0.62011, 0.61517, 0.61037,\
        0.60558, 0.60063, 0.59590, 0.59141, 0.58671, 0.58182, 0.57688, 0.57194, 0.56702, 0.56212, 0.55721,\
        0.55227, 0.54726, 0.54216, 0.53694, 0.53158, 0.52606, 0.52040, 0.51459, 0.50858, 0.50224, 0.49539],\
        [0.38967, 0.39342, 0.39705, 0.40051, 0.40365, 0.40610, 0.40790, 0.40917, 0.41014, 0.41101, 0.41200,\
        0.41321, 0.41462, 0.41618, 0.41781, 0.41947, 0.42113, 0.42279, 0.42448, 0.42622, 0.42804, 0.42998,\
        0.43209, 0.43440, 0.43696, 0.43979, 0.44292, 0.44638, 0.45019, 0.45434, 0.45886, 0.46373, 0.46894,\
        0.47449, 0.48035, 0.48649, 0.49286, 0.49943, 0.50630, 0.51331, 0.52041, 0.52757, 0.53476, 0.54197,\
        0.54911, 0.55611, 0.56290, 0.56937, 0.57543, 0.58099, 0.58601, 0.59079, 0.59570, 0.60082, 0.60658,\
        0.61356, 0.62131, 0.62937, 0.63748, 0.64550, 0.65333, 0.66089, 0.66814, 0.67509, 0.68160, 0.68724,\
        0.69151, 0.69450, 0.69637, 0.69747, 0.69850, 0.69939, 0.69960, 0.69937, 0.69910, 0.69872, 0.69764,\
        0.69590, 0.69422, 0.69265, 0.69106, 0.68944, 0.68768, 0.68567, 0.68325, 0.68026, 0.67656, 0.67220,\
        0.66738, 0.66232, 0.65715, 0.65191, 0.64665, 0.64136, 0.63608, 0.63082, 0.62563, 0.62052, 0.61549,\
        0.61047, 0.60547, 0.60078, 0.59626, 0.59128, 0.58612, 0.58099, 0.57591, 0.57089, 0.56589, 0.56089,\
        0.55585, 0.55075, 0.54556, 0.54027, 0.53485, 0.52929, 0.52358, 0.51769, 0.51158, 0.50514, 0.49825],\
        [0.39258, 0.39600, 0.39935, 0.40281, 0.40595, 0.40859, 0.41067, 0.41228, 0.41359, 0.41476, 0.41593,\
        0.41721, 0.41861, 0.42011, 0.42166, 0.42325, 0.42484, 0.42645, 0.42808, 0.42975, 0.43150, 0.43336,\
        0.43536, 0.43755, 0.43995, 0.44260, 0.44553, 0.44876, 0.45231, 0.45619, 0.46041, 0.46498, 0.46990,\
        0.47516, 0.48076, 0.48671, 0.49304, 0.49977, 0.50670, 0.51400, 0.52156, 0.52927, 0.53707, 0.54486,\
        0.55256, 0.56010, 0.56738, 0.57432, 0.58084, 0.58680, 0.59197, 0.59668, 0.60185, 0.60737, 0.61350,\
        0.62056, 0.62842, 0.63674, 0.64522, 0.65363, 0.66178, 0.66952, 0.67683, 0.68382, 0.69060, 0.69666,\
        0.70092, 0.70382, 0.70566, 0.70677, 0.70773, 0.70847, 0.70870, 0.70851, 0.70815, 0.70755, 0.70631,\
        0.70451, 0.70272, 0.70101, 0.69927, 0.69756, 0.69571, 0.69365, 0.69121, 0.68816, 0.68419, 0.67939,\
        0.67416, 0.66879, 0.66339, 0.65797, 0.65254, 0.64711, 0.64168, 0.63628, 0.63094, 0.62566, 0.62044,\
        0.61530, 0.61028, 0.60549, 0.60067, 0.59535, 0.58999, 0.58474, 0.57958, 0.57448, 0.56941, 0.56432,\
        0.55919, 0.55400, 0.54873, 0.54336, 0.53787, 0.53224, 0.52645, 0.52047, 0.51423, 0.50766, 0.50076],\
        [0.39588, 0.39915, 0.40236, 0.40570, 0.40883, 0.41156, 0.41384, 0.41571, 0.41729, 0.41870, 0.42005,\
        0.42141, 0.42282, 0.42427, 0.42576, 0.42726, 0.42877, 0.43029, 0.43183, 0.43341, 0.43505, 0.43678,\
        0.43865, 0.44067, 0.44289, 0.44533, 0.44802, 0.45099, 0.45426, 0.45784, 0.46175, 0.46600, 0.47060,\
        0.47556, 0.48089, 0.48662, 0.49280, 0.49945, 0.50664, 0.51431, 0.52235, 0.53062, 0.53899, 0.54738,\
        0.55567, 0.56376, 0.57155, 0.57896, 0.58593, 0.59246, 0.59812, 0.60270, 0.60795, 0.61376, 0.62021,\
        0.62749, 0.63554, 0.64413, 0.65298, 0.66179, 0.67026, 0.67815, 0.68536, 0.69208, 0.69875, 0.70498,\
        0.70945, 0.71250, 0.71454, 0.71590, 0.71688, 0.71754, 0.71779, 0.71765, 0.71719, 0.71637, 0.71505,\
        0.71331, 0.71142, 0.70951, 0.70758, 0.70564, 0.70358, 0.70134, 0.69879, 0.69570, 0.69137, 0.68596,\
        0.68034, 0.67477, 0.66924, 0.66372, 0.65818, 0.65262, 0.64706, 0.64152, 0.63602, 0.63057, 0.62519,\
        0.61989, 0.61468, 0.60955, 0.60431, 0.59885, 0.59344, 0.58816, 0.58297, 0.57783, 0.57269, 0.56752,\
        0.56231, 0.55703, 0.55168, 0.54622, 0.54066, 0.53495, 0.52909, 0.52302, 0.51671, 0.51013, 0.50332],\
        [0.39978, 0.40282, 0.40607, 0.40926, 0.41230, 0.41505, 0.41744, 0.41949, 0.42127, 0.42286, 0.42435,\
        0.42578, 0.42721, 0.42863, 0.43006, 0.43148, 0.43289, 0.43430, 0.43573, 0.43718, 0.43868, 0.44025,\
        0.44194, 0.44377, 0.44577, 0.44797, 0.45040, 0.45308, 0.45605, 0.45931, 0.46290, 0.46681, 0.47109,\
        0.47573, 0.48078, 0.48627, 0.49225, 0.49884, 0.50623, 0.51428, 0.52279, 0.53158, 0.54052, 0.54948,\
        0.55834, 0.56699, 0.57530, 0.58313, 0.59039, 0.59716, 0.60333, 0.60849, 0.61383, 0.61980, 0.62650,\
        0.63402, 0.64234, 0.65128, 0.66056, 0.66985, 0.67874, 0.68686, 0.69400, 0.70044, 0.70681, 0.71275,\
        0.71741, 0.72077, 0.72310, 0.72472, 0.72583, 0.72654, 0.72683, 0.72672, 0.72620, 0.72526, 0.72387,\
        0.72211, 0.72011, 0.71800, 0.71581, 0.71355, 0.71116, 0.70858, 0.70571, 0.70235, 0.69750, 0.69153,\
        0.68582, 0.68027, 0.67474, 0.66917, 0.66355, 0.65789, 0.65221, 0.64653, 0.64087, 0.63525, 0.62969,\
        0.62418, 0.61873, 0.61328, 0.60774, 0.60213, 0.59666, 0.59136, 0.58616, 0.58097, 0.57576, 0.57051,\
        0.56521, 0.55985, 0.55440, 0.54887, 0.54323, 0.53745, 0.53152, 0.52541, 0.51908, 0.51253, 0.50577],\
        [0.40398, 0.40710, 0.41023, 0.41332, 0.41627, 0.41899, 0.42144, 0.42360, 0.42552, 0.42724, 0.42883,\
        0.43032, 0.43176, 0.43315, 0.43452, 0.43585, 0.43716, 0.43845, 0.43973, 0.44103, 0.44236, 0.44376,\
        0.44524, 0.44685, 0.44860, 0.45053, 0.45267, 0.45505, 0.45769, 0.46061, 0.46385, 0.46743, 0.47138,\
        0.47571, 0.48048, 0.48573, 0.49156, 0.49814, 0.50563, 0.51396, 0.52287, 0.53214, 0.54159, 0.55109,\
        0.56049, 0.56968, 0.57853, 0.58684, 0.59434, 0.60106, 0.60747, 0.61329, 0.61903, 0.62523, 0.63216,\
        0.63994, 0.64859, 0.65796, 0.66775, 0.67759, 0.68705, 0.69558, 0.70282, 0.70921, 0.71527, 0.72080,\
        0.72539, 0.72890, 0.73147, 0.73331, 0.73460, 0.73542, 0.73580, 0.73573, 0.73519, 0.73417, 0.73270,\
        0.73084, 0.72868, 0.72633, 0.72383, 0.72118, 0.71835, 0.71526, 0.71175, 0.70750, 0.70198, 0.69619,\
        0.69083, 0.68544, 0.67997, 0.67439, 0.66870, 0.66294, 0.65712, 0.65130, 0.64548, 0.63968, 0.63393,\
        0.62822, 0.62254, 0.61685, 0.61113, 0.60542, 0.59984, 0.59447, 0.58922, 0.58397, 0.57867, 0.57332,\
        0.56791, 0.56245, 0.55692, 0.55131, 0.54559, 0.53976, 0.53378, 0.52764, 0.52131, 0.51479, 0.50809],\
        [0.40837, 0.41158, 0.41469, 0.41771, 0.42060, 0.42330, 0.42577, 0.42800, 0.43000, 0.43182, 0.43347,\
        0.43501, 0.43645, 0.43782, 0.43912, 0.44036, 0.44155, 0.44270, 0.44383, 0.44496, 0.44610, 0.44728,\
        0.44854, 0.44989, 0.45137, 0.45301, 0.45484, 0.45688, 0.45917, 0.46175, 0.46463, 0.46786, 0.47147,\
        0.47550, 0.48000, 0.48504, 0.49072, 0.49720, 0.50476, 0.51334, 0.52259, 0.53225, 0.54215, 0.55212,\
        0.56202, 0.57172, 0.58108, 0.58997, 0.59810, 0.60495, 0.61133, 0.61742, 0.62349, 0.62993, 0.63706,\
        0.64510, 0.65411, 0.66398, 0.67435, 0.68477, 0.69478, 0.70375, 0.71126, 0.71784, 0.72380, 0.72910,\
        0.73355, 0.73708, 0.73977, 0.74178, 0.74323, 0.74420, 0.74470, 0.74469, 0.74414, 0.74306, 0.74147,\
        0.73945, 0.73707, 0.73443, 0.73157, 0.72850, 0.72517, 0.72150, 0.71734, 0.71246, 0.70685, 0.70123,\
        0.69586, 0.69052, 0.68506, 0.67943, 0.67365, 0.66776, 0.66180, 0.65582, 0.64984, 0.64387, 0.63794,\
        0.63204, 0.62616, 0.62030, 0.61446, 0.60868, 0.60301, 0.59755, 0.59221, 0.58683, 0.58140, 0.57593,\
        0.57042, 0.56486, 0.55924, 0.55355, 0.54777, 0.54188, 0.53587, 0.52971, 0.52340, 0.51691, 0.51026],\
        [0.41290, 0.41620, 0.41933, 0.42233, 0.42519, 0.42788, 0.43036, 0.43263, 0.43469, 0.43656, 0.43827,\
        0.43983, 0.44127, 0.44259, 0.44382, 0.44497, 0.44603, 0.44704, 0.44800, 0.44894, 0.44987, 0.45082,\
        0.45183, 0.45291, 0.45409, 0.45541, 0.45690, 0.45859, 0.46052, 0.46271, 0.46522, 0.46808, 0.47135,\
        0.47507, 0.47932, 0.48417, 0.48972, 0.49611, 0.50368, 0.51241, 0.52186, 0.53182, 0.54207, 0.55245,\
        0.56279, 0.57294, 0.58275, 0.59205, 0.60059, 0.60799, 0.61466, 0.62099, 0.62729, 0.63390, 0.64119,\
        0.64943, 0.65880, 0.66922, 0.68031, 0.69135, 0.70171, 0.71112, 0.71921, 0.72621, 0.73232, 0.73757,\
        0.74193, 0.74542, 0.74813, 0.75021, 0.75180, 0.75292, 0.75354, 0.75361, 0.75306, 0.75188, 0.75013,\
        0.74789, 0.74524, 0.74227, 0.73902, 0.73551, 0.73170, 0.72753, 0.72290, 0.71774, 0.71219, 0.70657,\
        0.70104, 0.69564, 0.69011, 0.68433, 0.67839, 0.67235, 0.66625, 0.66011, 0.65396, 0.64782, 0.64171,\
        0.63564, 0.62961, 0.62362, 0.61768, 0.61184, 0.60613, 0.60059, 0.59511, 0.58955, 0.58395, 0.57834,\
        0.57271, 0.56705, 0.56135, 0.55559, 0.54976, 0.54383, 0.53780, 0.53164, 0.52534, 0.51889, 0.51228],\
        [0.41754, 0.42091, 0.42408, 0.42710, 0.42996, 0.43264, 0.43514, 0.43744, 0.43954, 0.44145, 0.44319,\
        0.44476, 0.44618, 0.44746, 0.44862, 0.44966, 0.45059, 0.45144, 0.45222, 0.45295, 0.45366, 0.45436,\
        0.45510, 0.45589, 0.45676, 0.45774, 0.45887, 0.46018, 0.46171, 0.46350, 0.46560, 0.46808, 0.47098,\
        0.47437, 0.47835, 0.48301, 0.48847, 0.49489, 0.50244, 0.51104, 0.52052, 0.53067, 0.54121, 0.55193,\
        0.56266, 0.57325, 0.58351, 0.59319, 0.60208, 0.61014, 0.61737, 0.62407, 0.63059, 0.63731, 0.64464,\
        0.65298, 0.66262, 0.67360, 0.68557, 0.69764, 0.70849, 0.71835, 0.72706, 0.73458, 0.74099, 0.74632,\
        0.75062, 0.75399, 0.75661, 0.75867, 0.76033, 0.76159, 0.76237, 0.76251, 0.76193, 0.76062, 0.75864,\
        0.75612, 0.75314, 0.74981, 0.74617, 0.74224, 0.73801, 0.73343, 0.72847, 0.72315, 0.71756, 0.71191,\
        0.70630, 0.70078, 0.69503, 0.68902, 0.68291, 0.67672, 0.67045, 0.66414, 0.65783, 0.65153, 0.64526,\
        0.63903, 0.63286, 0.62674, 0.62072, 0.61480, 0.60903, 0.60341, 0.59777, 0.59199, 0.58624, 0.58051,\
        0.57479, 0.56904, 0.56326, 0.55744, 0.55156, 0.54561, 0.53956, 0.53341, 0.52714, 0.52072, 0.51416],\
        [0.42225, 0.42567, 0.42890, 0.43194, 0.43482, 0.43752, 0.44004, 0.44236, 0.44450, 0.44644, 0.44819,\
        0.44976, 0.45116, 0.45240, 0.45348, 0.45441, 0.45521, 0.45589, 0.45648, 0.45699, 0.45746, 0.45790,\
        0.45835, 0.45883, 0.45937, 0.45999, 0.46074, 0.46164, 0.46274, 0.46410, 0.46576, 0.46780, 0.47029,\
        0.47331, 0.47697, 0.48137, 0.48666, 0.49300, 0.50041, 0.50875, 0.51828, 0.52859, 0.53934, 0.55036,\
        0.56148, 0.57254, 0.58335, 0.59369, 0.60326, 0.61189, 0.61970, 0.62684, 0.63361, 0.64040, 0.64768,\
        0.65597, 0.66572, 0.67717, 0.68998, 0.70299, 0.71498, 0.72574, 0.73523, 0.74334, 0.75010, 0.75556,\
        0.75978, 0.76293, 0.76529, 0.76720, 0.76886, 0.77026, 0.77120, 0.77143, 0.77077, 0.76923, 0.76695,\
        0.76408, 0.76075, 0.75704, 0.75301, 0.74870, 0.74410, 0.73919, 0.73397, 0.72849, 0.72282, 0.71707,\
        0.71133, 0.70556, 0.69957, 0.69340, 0.68717, 0.68083, 0.67439, 0.66792, 0.66144, 0.65499, 0.64857,\
        0.64220, 0.63588, 0.62965, 0.62351, 0.61748, 0.61157, 0.60575, 0.59991, 0.59405, 0.58825, 0.58245,\
        0.57664, 0.57081, 0.56496, 0.55909, 0.55318, 0.54721, 0.54117, 0.53503, 0.52879, 0.52242, 0.51591],\
        [0.42699, 0.43047, 0.43374, 0.43683, 0.43973, 0.44246, 0.44500, 0.44736, 0.44952, 0.45148, 0.45325,\
        0.45482, 0.45620, 0.45738, 0.45838, 0.45920, 0.45986, 0.46037, 0.46075, 0.46104, 0.46125, 0.46143,\
        0.46158, 0.46174, 0.46193, 0.46217, 0.46251, 0.46298, 0.46362, 0.46449, 0.46567, 0.46722, 0.46923,\
        0.47180, 0.47503, 0.47906, 0.48399, 0.48993, 0.49688, 0.50522, 0.51495, 0.52534, 0.53619, 0.54745,\
        0.55899, 0.57063, 0.58213, 0.59322, 0.60366, 0.61329, 0.62190, 0.62960, 0.63668, 0.64357, 0.65077,\
        0.65888, 0.66853, 0.68028, 0.69406, 0.70831, 0.72173, 0.73379, 0.74420, 0.75292, 0.76003, 0.76560,\
        0.76966, 0.77241, 0.77429, 0.77585, 0.77743, 0.77896, 0.78008, 0.78037, 0.77953, 0.77765, 0.77498,\
        0.77171, 0.76798, 0.76390, 0.75951, 0.75486, 0.74994, 0.74475, 0.73932, 0.73366, 0.72785, 0.72195,\
        0.71600, 0.70997, 0.70378, 0.69753, 0.69120, 0.68467, 0.67805, 0.67142, 0.66480, 0.65821, 0.65164,\
        0.64513, 0.63868, 0.63231, 0.62604, 0.61987, 0.61380, 0.60778, 0.60176, 0.59588, 0.59006, 0.58418,\
        0.57827, 0.57237, 0.56646, 0.56055, 0.55462, 0.54865, 0.54262, 0.53651, 0.53030, 0.52397, 0.51752],\
        [0.43172, 0.43525, 0.43857, 0.44170, 0.44464, 0.44740, 0.44998, 0.45236, 0.45455, 0.45654, 0.45832,\
        0.45989, 0.46124, 0.46238, 0.46330, 0.46401, 0.46452, 0.46485, 0.46503, 0.46508, 0.46504, 0.46493,\
        0.46478, 0.46462, 0.46445, 0.46430, 0.46420, 0.46420, 0.46434, 0.46469, 0.46531, 0.46630, 0.46774,\
        0.46975, 0.47244, 0.47594, 0.48039, 0.48589, 0.49259, 0.50076, 0.51050, 0.52056, 0.53130, 0.54279,\
        0.55482, 0.56722, 0.57975, 0.59197, 0.60366, 0.61458, 0.62417, 0.63258, 0.64016, 0.64733, 0.65457,\
        0.66252, 0.67197, 0.68377, 0.69848, 0.71466, 0.72996, 0.74338, 0.75462, 0.76377, 0.77109, 0.77673,\
        0.78058, 0.78269, 0.78379, 0.78470, 0.78610, 0.78772, 0.78897, 0.78931, 0.78810, 0.78571, 0.78260,\
        0.77891, 0.77478, 0.77034, 0.76563, 0.76069, 0.75550, 0.75008, 0.74443, 0.73860, 0.73262, 0.72653,\
        0.72037, 0.71413, 0.70783, 0.70150, 0.69495, 0.68818, 0.68140, 0.67465, 0.66791, 0.66118, 0.65448,\
        0.64783, 0.64124, 0.63473, 0.62832, 0.62201, 0.61580, 0.60967, 0.60362, 0.59768, 0.59172, 0.58569,\
        0.57968, 0.57370, 0.56775, 0.56182, 0.55589, 0.54993, 0.54392, 0.53784, 0.53168, 0.52540, 0.51900],\
        [0.43642, 0.43999, 0.44335, 0.44652, 0.44950, 0.45230, 0.45492, 0.45734, 0.45956, 0.46157, 0.46336,\
        0.46494, 0.46627, 0.46737, 0.46822, 0.46882, 0.46918, 0.46932, 0.46929, 0.46910, 0.46880, 0.46841,\
        0.46796, 0.46747, 0.46693, 0.46638, 0.46583, 0.46533, 0.46492, 0.46468, 0.46469, 0.46503, 0.46580,\
        0.46712, 0.46912, 0.47192, 0.47571, 0.48069, 0.48711, 0.49527, 0.50445, 0.51332, 0.52401, 0.53585,\
        0.54842, 0.56181, 0.57590, 0.59010, 0.60365, 0.61593, 0.62654, 0.63585, 0.64428, 0.65215, 0.65986,\
        0.66803, 0.67765, 0.68979, 0.70581, 0.72418, 0.74110, 0.75545, 0.76709, 0.77628, 0.78354, 0.78918,\
        0.79281, 0.79402, 0.79415, 0.79400, 0.79513, 0.79666, 0.79772, 0.79810, 0.79613, 0.79320, 0.78969,\
        0.78556, 0.78105, 0.77630, 0.77134, 0.76616, 0.76075, 0.75512, 0.74928, 0.74327, 0.73710, 0.73082,\
        0.72445, 0.71799, 0.71149, 0.70492, 0.69817, 0.69126, 0.68443, 0.67762, 0.67079, 0.66394, 0.65711,\
        0.65031, 0.64358, 0.63693, 0.63036, 0.62390, 0.61756, 0.61134, 0.60529, 0.59933, 0.59310, 0.58691,\
        0.58082, 0.57480, 0.56884, 0.56291, 0.55699, 0.55106, 0.54508, 0.53905, 0.53293, 0.52671, 0.52037],\
        [0.44103, 0.44464, 0.44804, 0.45125, 0.45428, 0.45712, 0.45977, 0.46223, 0.46448, 0.46652, 0.46834,\
        0.46992, 0.47125, 0.47232, 0.47310, 0.47360, 0.47381, 0.47377, 0.47351, 0.47308, 0.47252, 0.47186,\
        0.47112, 0.47031, 0.46941, 0.46845, 0.46743, 0.46639, 0.46539, 0.46451, 0.46381, 0.46342, 0.46341,\
        0.46391, 0.46503, 0.46691, 0.46975, 0.47378, 0.47927, 0.48633, 0.49404, 0.50248, 0.51386, 0.52601,\
        0.53901, 0.55354, 0.56970, 0.58680, 0.60314, 0.61677, 0.62858, 0.63922, 0.64907, 0.65839, 0.66734,\
        0.67647, 0.68731, 0.70146, 0.71965, 0.73892, 0.75618, 0.77056, 0.78204, 0.79076, 0.79759, 0.80293,\
        0.80615, 0.80668, 0.80596, 0.80484, 0.80512, 0.80595, 0.80623, 0.80566, 0.80326, 0.80005, 0.79617,\
        0.79155, 0.78671, 0.78175, 0.77661, 0.77125, 0.76566, 0.75986, 0.75385, 0.74766, 0.74132, 0.73484,\
        0.72825, 0.72156, 0.71477, 0.70787, 0.70088, 0.69396, 0.68724, 0.68041, 0.67347, 0.66650, 0.65953,\
        0.65260, 0.64572, 0.63891, 0.63219, 0.62556, 0.61906, 0.61268, 0.60642, 0.60020, 0.59398, 0.58780,\
        0.58170, 0.57569, 0.56974, 0.56384, 0.55795, 0.55205, 0.54613, 0.54014, 0.53408, 0.52792, 0.52164],\
        [0.44555, 0.44918, 0.45261, 0.45586, 0.45892, 0.46180, 0.46449, 0.46699, 0.46928, 0.47136, 0.47320,\
        0.47480, 0.47614, 0.47719, 0.47792, 0.47833, 0.47840, 0.47817, 0.47769, 0.47702, 0.47620, 0.47528,\
        0.47427, 0.47315, 0.47191, 0.47054, 0.46904, 0.46744, 0.46580, 0.46420, 0.46274, 0.46151, 0.46061,\
        0.46014, 0.46021, 0.46093, 0.46246, 0.46500, 0.46876, 0.47385, 0.48027, 0.48888, 0.50038, 0.51223,\
        0.52560, 0.54135, 0.55992, 0.58020, 0.59965, 0.61574, 0.62957, 0.64224, 0.65426, 0.66599, 0.67736,\
        0.68830, 0.70153, 0.71896, 0.73904, 0.75851, 0.77486, 0.78828, 0.79928, 0.80743, 0.81369, 0.81821,\
        0.82052, 0.82057, 0.81921, 0.81748, 0.81634, 0.81562, 0.81460, 0.81280, 0.80991, 0.80619, 0.80178,\
        0.79671, 0.79175, 0.78672, 0.78147, 0.77598, 0.77025, 0.76430, 0.75814, 0.75179, 0.74528, 0.73861,\
        0.73182, 0.72491, 0.71788, 0.71074, 0.70351, 0.69671, 0.69010, 0.68311, 0.67599, 0.66887, 0.66177,\
        0.65471, 0.64769, 0.64072, 0.63383, 0.62703, 0.62032, 0.61373, 0.60723, 0.60081, 0.59457, 0.58843,\
        0.58237, 0.57639, 0.57048, 0.56462, 0.55878, 0.55294, 0.54706, 0.54113, 0.53513, 0.52903, 0.52282],\
        [0.44992, 0.45357, 0.45703, 0.46029, 0.46339, 0.46630, 0.46904, 0.47158, 0.47391, 0.47601, 0.47789,\
        0.47951, 0.48088, 0.48194, 0.48265, 0.48299, 0.48293, 0.48250, 0.48179, 0.48087, 0.47982, 0.47866,\
        0.47741, 0.47603, 0.47448, 0.47271, 0.47072, 0.46852, 0.46619, 0.46382, 0.46152, 0.45938, 0.45749,\
        0.45595, 0.45481, 0.45415, 0.45407, 0.45470, 0.45626, 0.45906, 0.46365, 0.47090, 0.48102, 0.49315,\
        0.50744, 0.52460, 0.54622, 0.57011, 0.59285, 0.61227, 0.62918, 0.64457, 0.65932, 0.67415, 0.68934,\
        0.70358, 0.71935, 0.73906, 0.76097, 0.78096, 0.79548, 0.80740, 0.81751, 0.82535, 0.83106, 0.83465,\
        0.83597, 0.83520, 0.83304, 0.83036, 0.82780, 0.82543, 0.82291, 0.81987, 0.81613, 0.81171, 0.80669,\
        0.80138, 0.79640, 0.79127, 0.78593, 0.78035, 0.77451, 0.76844, 0.76216, 0.75567, 0.74900, 0.74218,\
        0.73521, 0.72812, 0.72095, 0.71376, 0.70673, 0.70012, 0.69299, 0.68560, 0.67830, 0.67106, 0.66386,\
        0.65668, 0.64952, 0.64240, 0.63533, 0.62834, 0.62144, 0.61464, 0.60796, 0.60143, 0.59511, 0.58895,\
        0.58291, 0.57696, 0.57110, 0.56530, 0.55952, 0.55373, 0.54792, 0.54205, 0.53610, 0.53007, 0.52392],\
        [0.45414, 0.45779, 0.46124, 0.46452, 0.46764, 0.47059, 0.47336, 0.47594, 0.47831, 0.48045, 0.48235,\
        0.48401, 0.48541, 0.48651, 0.48724, 0.48754, 0.48736, 0.48674, 0.48579, 0.48463, 0.48335, 0.48200,\
        0.48055, 0.47897, 0.47715, 0.47502, 0.47253, 0.46972, 0.46665, 0.46345, 0.46024, 0.45712, 0.45417,\
        0.45147, 0.44904, 0.44688, 0.44500, 0.44349, 0.44254, 0.44257, 0.44441, 0.44917, 0.45753, 0.46940,\
        0.48455, 0.50378, 0.52998, 0.55837, 0.58472, 0.60775, 0.62797, 0.64628, 0.66381, 0.68160, 0.70028,\
        0.71872, 0.73737, 0.75807, 0.78085, 0.80227, 0.81544, 0.82623, 0.83562, 0.84319, 0.84855, 0.85141,\
        0.85171, 0.84985, 0.84660, 0.84274, 0.83879, 0.83492, 0.83098, 0.82675, 0.82209, 0.81696, 0.81146,\
        0.80589, 0.80060, 0.79523, 0.79002, 0.78438, 0.77846, 0.77230, 0.76591, 0.75931, 0.75251, 0.74555,\
        0.73844, 0.73122, 0.72393, 0.71665, 0.70950, 0.70244, 0.69506, 0.68768, 0.68038, 0.67311, 0.66584,\
        0.65856, 0.65127, 0.64400, 0.63676, 0.62957, 0.62247, 0.61549, 0.60867, 0.60208, 0.59566, 0.58941,\
        0.58336, 0.57746, 0.57165, 0.56591, 0.56019, 0.55447, 0.54871, 0.54290, 0.53702, 0.53104, 0.52495],\
        [0.45817, 0.46180, 0.46524, 0.46851, 0.47163, 0.47460, 0.47741, 0.48003, 0.48244, 0.48461, 0.48652,\
        0.48820, 0.48965, 0.49083, 0.49164, 0.49194, 0.49166, 0.49085, 0.48966, 0.48828, 0.48681, 0.48529,\
        0.48373, 0.48202, 0.48001, 0.47756, 0.47458, 0.47111, 0.46726, 0.46318, 0.45901, 0.45486, 0.45081,\
        0.44691, 0.44314, 0.43946, 0.43576, 0.43201, 0.42842, 0.42537, 0.42378, 0.42549, 0.43194, 0.44345,\
        0.45979, 0.48266, 0.51348, 0.54675, 0.57719, 0.60344, 0.62658, 0.64776, 0.66799, 0.68857, 0.71056,\
        0.73284, 0.75399, 0.77451, 0.79495, 0.81551, 0.83093, 0.84254, 0.85214, 0.85988, 0.86526, 0.86761,\
        0.86685, 0.86370, 0.85917, 0.85406, 0.84884, 0.84368, 0.83854, 0.83328, 0.82778, 0.82202, 0.81605,\
        0.80999, 0.80396, 0.79927, 0.79382, 0.78807, 0.78212, 0.77589, 0.76942, 0.76272, 0.75582, 0.74875,\
        0.74153, 0.73419, 0.72676, 0.71929, 0.71178, 0.70416, 0.69670, 0.68956, 0.68237, 0.67511, 0.66779,\
        0.66041, 0.65300, 0.64557, 0.63815, 0.63077, 0.62347, 0.61629, 0.60929, 0.60253, 0.59602, 0.58978,\
        0.58378, 0.57793, 0.57218, 0.56650, 0.56084, 0.55517, 0.54947, 0.54372, 0.53789, 0.53196, 0.52593],\
        [0.46199, 0.46557, 0.46897, 0.47222, 0.47533, 0.47830, 0.48114, 0.48380, 0.48625, 0.48844, 0.49036,\
        0.49202, 0.49351, 0.49480, 0.49574, 0.49613, 0.49579, 0.49477, 0.49337, 0.49181, 0.49018, 0.48856,\
        0.48695, 0.48522, 0.48314, 0.48043, 0.47696, 0.47279, 0.46811, 0.46310, 0.45794, 0.45274, 0.44756,\
        0.44245, 0.43737, 0.43223, 0.42679, 0.42083, 0.41462, 0.40862, 0.40342, 0.40185, 0.40610, 0.41727,\
        0.43346, 0.45889, 0.49488, 0.53466, 0.56971, 0.59892, 0.62524, 0.64945, 0.67259, 0.69643, 0.72241,\
        0.74805, 0.77090, 0.79080, 0.80979, 0.82811, 0.84369, 0.85606, 0.86620, 0.87447, 0.88030, 0.88239,\
        0.88042, 0.87583, 0.87005, 0.86384, 0.85760, 0.85144, 0.84536, 0.83928, 0.83312, 0.82685, 0.82050,\
        0.81419, 0.80817, 0.80266, 0.79709, 0.79146, 0.78553, 0.77925, 0.77269, 0.76591, 0.75893, 0.75177,\
        0.74447, 0.73704, 0.72951, 0.72189, 0.71419, 0.70647, 0.69898, 0.69172, 0.68449, 0.67718, 0.66978,\
        0.66229, 0.65474, 0.64716, 0.63956, 0.63200, 0.62451, 0.61713, 0.60991, 0.60289, 0.59634, 0.59018,\
        0.58423, 0.57843, 0.57273, 0.56710, 0.56150, 0.55588, 0.55023, 0.54451, 0.53873, 0.53285, 0.52686],\
        [0.46560, 0.46910, 0.47243, 0.47561, 0.47868, 0.48164, 0.48449, 0.48719, 0.48970, 0.49192, 0.49380,\
        0.49540, 0.49689, 0.49831, 0.49939, 0.49991, 0.49961, 0.49845, 0.49694, 0.49528, 0.49355, 0.49183,\
        0.49023, 0.48863, 0.48663, 0.48374, 0.47975, 0.47484, 0.46927, 0.46332, 0.45717, 0.45094, 0.44465,\
        0.43831, 0.43190, 0.42537, 0.41845, 0.41035, 0.40136, 0.39265, 0.38487, 0.38074, 0.38247, 0.39139,\
        0.40356, 0.42939, 0.46877, 0.51959, 0.56060, 0.59392, 0.62385, 0.65138, 0.67798, 0.70589, 0.73605,\
        0.76497, 0.78927, 0.80858, 0.82574, 0.84167, 0.85565, 0.86743, 0.87747, 0.88601, 0.89253, 0.89462,\
        0.89114, 0.88530, 0.87864, 0.87173, 0.86483, 0.85802, 0.85130, 0.84464, 0.83799, 0.83135, 0.82476,\
        0.81830, 0.81210, 0.80616, 0.80040, 0.79483, 0.78877, 0.78237, 0.77572, 0.76887, 0.76183, 0.75463,\
        0.74728, 0.73980, 0.73222, 0.72455, 0.71683, 0.70913, 0.70156, 0.69416, 0.68681, 0.67939, 0.67187,\
        0.66424, 0.65655, 0.64880, 0.64104, 0.63331, 0.62564, 0.61810, 0.61074, 0.60368, 0.59703, 0.59077,\
        0.58480, 0.57902, 0.57336, 0.56776, 0.56219, 0.55661, 0.55098, 0.54531, 0.53955, 0.53371, 0.52776],\
        [0.46898, 0.47237, 0.47559, 0.47867, 0.48165, 0.48456, 0.48739, 0.49013, 0.49270, 0.49499, 0.49685,\
        0.49833, 0.49970, 0.50127, 0.50245, 0.50307, 0.50290, 0.50193, 0.50053, 0.49888, 0.49706, 0.49521,\
        0.49361, 0.49228, 0.49057, 0.48757, 0.48301, 0.47727, 0.47080, 0.46394, 0.45688, 0.44970, 0.44236,\
        0.43479, 0.42693, 0.41885, 0.41064, 0.40105, 0.38900, 0.37756, 0.36786, 0.36162, 0.36037, 0.36464,\
        0.37541, 0.39430, 0.43121, 0.49477, 0.54553, 0.58559, 0.62054, 0.65257, 0.68378, 0.71640, 0.75028,\
        0.78217, 0.80755, 0.82587, 0.84124, 0.85491, 0.86684, 0.87699, 0.88592, 0.89378, 0.89978, 0.90162,\
        0.89760, 0.89149, 0.88473, 0.87768, 0.87053, 0.86338, 0.85630, 0.84927, 0.84231, 0.83543, 0.82866,\
        0.82208, 0.81574, 0.80969, 0.80391, 0.79801, 0.79173, 0.78520, 0.77847, 0.77158, 0.76452, 0.75730,\
        0.74994, 0.74246, 0.73488, 0.72723, 0.71954, 0.71188, 0.70429, 0.69680, 0.68931, 0.68172, 0.67404,\
        0.66627, 0.65842, 0.65052, 0.64261, 0.63471, 0.62690, 0.61922, 0.61176, 0.60463, 0.59789, 0.59153,\
        0.58551, 0.57972, 0.57408, 0.56850, 0.56294, 0.55737, 0.55177, 0.54610, 0.54037, 0.53454, 0.52862],\
        [0.47213, 0.47538, 0.47844, 0.48137, 0.48421, 0.48701, 0.48978, 0.49250, 0.49513, 0.49753, 0.49942,\
        0.50088, 0.50228, 0.50382, 0.50514, 0.50595, 0.50607, 0.50551, 0.50440, 0.50287, 0.50100, 0.49894,\
        0.49718, 0.49614, 0.49498, 0.49192, 0.48664, 0.48007, 0.47275, 0.46508, 0.45727, 0.44934, 0.44113,\
        0.43240, 0.42299, 0.41282, 0.40231, 0.39088, 0.37716, 0.36381, 0.35255, 0.34455, 0.34071, 0.34171,\
        0.34882, 0.36466, 0.40227, 0.46484, 0.52252, 0.57011, 0.61293, 0.65197, 0.68865, 0.72644, 0.76376,\
        0.79646, 0.82225, 0.84072, 0.85517, 0.86732, 0.87739, 0.88524, 0.89207, 0.89877, 0.90316, 0.90382,\
        0.90028, 0.89503, 0.88884, 0.88202, 0.87490, 0.86766, 0.86040, 0.85318, 0.84603, 0.83898, 0.83209,\
        0.82540, 0.81895, 0.81275, 0.80671, 0.80062, 0.79425, 0.78764, 0.78091, 0.77402, 0.76697, 0.75977,\
        0.75243, 0.74498, 0.73744, 0.72984, 0.72222, 0.71462, 0.70706, 0.69952, 0.69186, 0.68411, 0.67628,\
        0.66836, 0.66036, 0.65232, 0.64426, 0.63623, 0.62827, 0.62047, 0.61289, 0.60565, 0.59882, 0.59240,\
        0.58635, 0.58057, 0.57492, 0.56934, 0.56377, 0.55819, 0.55258, 0.54691, 0.54117, 0.53536, 0.52946],\
        [0.47508, 0.47814, 0.48100, 0.48372, 0.48636, 0.48897, 0.49159, 0.49423, 0.49683, 0.49917, 0.50118,\
        0.50279, 0.50444, 0.50614, 0.50770, 0.50886, 0.50944, 0.50939, 0.50873, 0.50752, 0.50579, 0.50368,\
        0.50156, 0.50019, 0.49945, 0.49625, 0.49043, 0.48322, 0.47517, 0.46689, 0.45860, 0.45025, 0.44152,\
        0.43194, 0.42111, 0.40902, 0.39574, 0.38136, 0.36620, 0.35171, 0.33948, 0.33051, 0.32527, 0.32439,\
        0.33010, 0.34808, 0.38603, 0.44080, 0.49650, 0.54934, 0.60288, 0.65036, 0.69205, 0.73571, 0.77765,\
        0.81094, 0.83576, 0.85388, 0.86783, 0.87928, 0.88830, 0.89369, 0.89710, 0.90258, 0.90604, 0.90599,\
        0.90272, 0.89793, 0.89196, 0.88532, 0.87832, 0.87108, 0.86374, 0.85640, 0.84913, 0.84197, 0.83498,\
        0.82819, 0.82162, 0.81524, 0.80896, 0.80266, 0.79619, 0.78968, 0.78304, 0.77620, 0.76919, 0.76201,\
        0.75472, 0.74732, 0.73984, 0.73232, 0.72477, 0.71722, 0.70966, 0.70205, 0.69432, 0.68648, 0.67853,\
        0.67048, 0.66235, 0.65418, 0.64599, 0.63784, 0.62977, 0.62184, 0.61413, 0.60675, 0.59980, 0.59335,\
        0.58734, 0.58158, 0.57592, 0.57030, 0.56468, 0.55906, 0.55341, 0.54772, 0.54197, 0.53616, 0.53026],\
        [0.47784, 0.48068, 0.48330, 0.48577, 0.48814, 0.49048, 0.49285, 0.49528, 0.49777, 0.50011, 0.50201,\
        0.50402, 0.50612, 0.50827, 0.51028, 0.51196, 0.51314, 0.51374, 0.51371, 0.51305, 0.51177, 0.50996,\
        0.50783, 0.50554, 0.50349, 0.50025, 0.49462, 0.48688, 0.47821, 0.46956, 0.46115, 0.45286, 0.44419,\
        0.43433, 0.42248, 0.40811, 0.39216, 0.37506, 0.35772, 0.34194, 0.32913, 0.32004, 0.31469, 0.31296,\
        0.31677, 0.33361, 0.37220, 0.42150, 0.46882, 0.52209, 0.58964, 0.64875, 0.69440, 0.74376, 0.79280,\
        0.82625, 0.84948, 0.86683, 0.88046, 0.89172, 0.90046, 0.90470, 0.90600, 0.90906, 0.91089, 0.90986,\
        0.90622, 0.90081, 0.89472, 0.88822, 0.88121, 0.87391, 0.86648, 0.85904, 0.85165, 0.84439, 0.83731,\
        0.83043, 0.82375, 0.81723, 0.81077, 0.80425, 0.79782, 0.79149, 0.78492, 0.77812, 0.77114, 0.76400,\
        0.75676, 0.74943, 0.74203, 0.73459, 0.72711, 0.71961, 0.71207, 0.70444, 0.69668, 0.68878, 0.68074,\
        0.67258, 0.66435, 0.65607, 0.64779, 0.63954, 0.63138, 0.62337, 0.61554, 0.60799, 0.60086, 0.59447,\
        0.58858, 0.58280, 0.57707, 0.57135, 0.56566, 0.55996, 0.55426, 0.54853, 0.54275, 0.53692, 0.53102],\
        [0.48045, 0.48305, 0.48540, 0.48756, 0.48960, 0.49160, 0.49363, 0.49571, 0.49786, 0.49997, 0.50225,\
        0.50481, 0.50753, 0.51031, 0.51296, 0.51533, 0.51726, 0.51864, 0.51941, 0.51953, 0.51897, 0.51775,\
        0.51590, 0.51348, 0.51044, 0.50614, 0.50012, 0.49116, 0.48213, 0.47340, 0.46522, 0.45755, 0.44972,\
        0.44053, 0.42859, 0.41299, 0.39395, 0.37286, 0.35260, 0.33501, 0.32155, 0.31290, 0.30853, 0.30684,\
        0.30682, 0.31504, 0.35354, 0.40507, 0.44379, 0.48766, 0.57098, 0.64783, 0.69587, 0.74871, 0.80118,\
        0.83868, 0.86325, 0.88128, 0.89537, 0.90681, 0.91500, 0.91876, 0.91962, 0.91978, 0.91886, 0.91597,\
        0.91108, 0.90482, 0.89831, 0.89123, 0.88390, 0.87638, 0.86877, 0.86116, 0.85363, 0.84626, 0.83908,\
        0.83211, 0.82536, 0.81879, 0.81234, 0.80595, 0.79962, 0.79321, 0.78658, 0.77976, 0.77279, 0.76570,\
        0.75852, 0.75127, 0.74396, 0.73661, 0.72921, 0.72178, 0.71427, 0.70667, 0.69892, 0.69098, 0.68287,\
        0.67463, 0.66632, 0.65796, 0.64961, 0.64130, 0.63310, 0.62505, 0.61721, 0.60965, 0.60250, 0.59619,\
        0.59021, 0.58425, 0.57834, 0.57248, 0.56666, 0.56087, 0.55509, 0.54931, 0.54350, 0.53764, 0.53173],\
        [0.48296, 0.48530, 0.48735, 0.48918, 0.49087, 0.49251, 0.49415, 0.49586, 0.49768, 0.49967, 0.50234,\
        0.50550, 0.50893, 0.51244, 0.51585, 0.51901, 0.52179, 0.52407, 0.52577, 0.52684, 0.52721, 0.52681,\
        0.52553, 0.52321, 0.51954, 0.51400, 0.50599, 0.49669, 0.48777, 0.47886, 0.47105, 0.46456, 0.45847,\
        0.45113, 0.44036, 0.42376, 0.40085, 0.37553, 0.35156, 0.33127, 0.31646, 0.30806, 0.30518, 0.30430,\
        0.30141, 0.29776, 0.33800, 0.39587, 0.42713, 0.45833, 0.54516, 0.64570, 0.69753, 0.75335, 0.81101,\
        0.85345, 0.88010, 0.90074, 0.91614, 0.92713, 0.93366, 0.93600, 0.93532, 0.93292, 0.92920, 0.92404,\
        0.91753, 0.91013, 0.90230, 0.89440, 0.88656, 0.87862, 0.87068, 0.86282, 0.85511, 0.84758, 0.84028,\
        0.83324, 0.82644, 0.81987, 0.81349, 0.80726, 0.80108, 0.79460, 0.78788, 0.78103, 0.77408, 0.76705,\
        0.75995, 0.75279, 0.74558, 0.73833, 0.73104, 0.72368, 0.71624, 0.70868, 0.70097, 0.69301, 0.68486,\
        0.67658, 0.66822, 0.65981, 0.65141, 0.64307, 0.63485, 0.62682, 0.61907, 0.61170, 0.60486, 0.59850,\
        0.59207, 0.58578, 0.57962, 0.57358, 0.56763, 0.56174, 0.55589, 0.55004, 0.54418, 0.53830, 0.53237],\
        [0.48540, 0.48748, 0.48923, 0.49073, 0.49208, 0.49337, 0.49467, 0.49607, 0.49770, 0.49991, 0.50272,\
        0.50640, 0.51054, 0.51482, 0.51904, 0.52304, 0.52670, 0.52993, 0.53266, 0.53479, 0.53624, 0.53686,\
        0.53643, 0.53457, 0.53081, 0.52464, 0.51605, 0.50625, 0.49613, 0.48629, 0.47878, 0.47403, 0.47048,\
        0.46617, 0.45853, 0.44266, 0.41610, 0.38470, 0.35579, 0.33143, 0.31393, 0.30450, 0.30237, 0.30104,\
        0.29240, 0.29114, 0.32878, 0.38738, 0.41894, 0.44184, 0.51039, 0.62616, 0.69664, 0.75847, 0.82592,\
        0.88043, 0.90741, 0.92882, 0.94342, 0.95195, 0.95565, 0.95540, 0.95225, 0.94717, 0.94073, 0.93318,\
        0.92478, 0.91584, 0.90674, 0.89790, 0.88920, 0.88060, 0.87221, 0.86404, 0.85608, 0.84837, 0.84093,\
        0.83378, 0.82693, 0.82036, 0.81403, 0.80788, 0.80175, 0.79531, 0.78862, 0.78182, 0.77494, 0.76800,\
        0.76101, 0.75397, 0.74688, 0.73975, 0.73256, 0.72529, 0.71793, 0.71043, 0.70275, 0.69482, 0.68668,\
        0.67839, 0.67000, 0.66156, 0.65313, 0.64477, 0.63655, 0.62854, 0.62085, 0.61357, 0.60676, 0.60021,\
        0.59362, 0.58711, 0.58076, 0.57458, 0.56851, 0.56253, 0.55660, 0.55070, 0.54480, 0.53888, 0.53294],\
        [0.48781, 0.48964, 0.49110, 0.49230, 0.49334, 0.49434, 0.49540, 0.49661, 0.49799, 0.49983, 0.50332,\
        0.50774, 0.51259, 0.51759, 0.52256, 0.52737, 0.53192, 0.53610, 0.53984, 0.54308, 0.54569, 0.54752,\
        0.54825, 0.54737, 0.54420, 0.53819, 0.52939, 0.51866, 0.50701, 0.49597, 0.48864, 0.48629, 0.48563,\
        0.48465, 0.48253, 0.47039, 0.44108, 0.40302, 0.36655, 0.33670, 0.31597, 0.30323, 0.30008, 0.29724,\
        0.28994, 0.29221, 0.32894, 0.38151, 0.42021, 0.45095, 0.50554, 0.59896, 0.69552, 0.76443, 0.84651,\
        0.91954, 0.94738, 0.96379, 0.97399, 0.97867, 0.97879, 0.97534, 0.96926, 0.96138, 0.95230, 0.94242,\
        0.93206, 0.92151, 0.91112, 0.90116, 0.89138, 0.88216, 0.87334, 0.86481, 0.85657, 0.84862, 0.84100,\
        0.83373, 0.82681, 0.82021, 0.81388, 0.80772, 0.80158, 0.79526, 0.78874, 0.78209, 0.77535, 0.76855,\
        0.76169, 0.75478, 0.74783, 0.74083, 0.73376, 0.72660, 0.71933, 0.71190, 0.70426, 0.69639, 0.68830,\
        0.68003, 0.67164, 0.66319, 0.65474, 0.64635, 0.63810, 0.63007, 0.62235, 0.61501, 0.60806, 0.60137,\
        0.59469, 0.58808, 0.58166, 0.57540, 0.56926, 0.56320, 0.55721, 0.55125, 0.54531, 0.53937, 0.53342],\
        [0.49021, 0.49181, 0.49300, 0.49392, 0.49471, 0.49551, 0.49645, 0.49770, 0.49935, 0.50134, 0.50502,\
        0.50986, 0.51522, 0.52080, 0.52643, 0.53197, 0.53731, 0.54238, 0.54709, 0.55137, 0.55514, 0.55826,\
        0.56042, 0.56105, 0.55919, 0.55392, 0.54511, 0.53368, 0.52114, 0.50951, 0.50199, 0.50209, 0.50381,\
        0.50588, 0.50840, 0.50123, 0.47195, 0.42737, 0.38111, 0.34865, 0.32811, 0.31378, 0.30793, 0.30866,\
        0.30936, 0.32069, 0.35455, 0.39837, 0.44161, 0.47990, 0.52230, 0.59122, 0.69088, 0.77081, 0.86143,\
        0.93950, 0.97714, 0.99459, 1.00228, 1.00368, 1.00049, 0.99391, 0.98495, 0.97439, 0.96287, 0.95084,\
        0.93864, 0.92653, 0.91471, 0.90333, 0.89291, 0.88329, 0.87405, 0.86512, 0.85653, 0.84831, 0.84049,\
        0.83308, 0.82608, 0.81944, 0.81309, 0.80694, 0.80081, 0.79466, 0.78835, 0.78189, 0.77532, 0.76867,\
        0.76197, 0.75522, 0.74842, 0.74157, 0.73464, 0.72761, 0.72045, 0.71311, 0.70555, 0.69775, 0.68973,\
        0.68150, 0.67312, 0.66466, 0.65618, 0.64775, 0.63945, 0.63135, 0.62352, 0.61601, 0.60884, 0.60199,\
        0.59525, 0.58867, 0.58228, 0.57601, 0.56983, 0.56373, 0.55768, 0.55167, 0.54570, 0.53974, 0.53378],\
        [0.49260, 0.49398, 0.49493, 0.49563, 0.49623, 0.49691, 0.49785, 0.49922, 0.50129, 0.50400, 0.50783,\
        0.51277, 0.51840, 0.52440, 0.53055, 0.53671, 0.54275, 0.54858, 0.55413, 0.55933, 0.56412, 0.56843,\
        0.57209, 0.57464, 0.57482, 0.57086, 0.56231, 0.55065, 0.53811, 0.52720, 0.52085, 0.52054, 0.52386,\
        0.52865, 0.53226, 0.52670, 0.50211, 0.45431, 0.40002, 0.37164, 0.35489, 0.34392, 0.34072, 0.34330,\
        0.35154, 0.37006, 0.40376, 0.44350, 0.48091, 0.50917, 0.53784, 0.59624, 0.69077, 0.78035, 0.87349,\
        0.95040, 0.99479, 1.01630, 1.02427, 1.02404, 1.01849, 1.00940, 0.99798, 0.98512, 0.97151, 0.95765,\
        0.94389, 0.93050, 0.91765, 0.90548, 0.89439, 0.88413, 0.87432, 0.86492, 0.85594, 0.84741, 0.83937,\
        0.83183, 0.82475, 0.81809, 0.81177, 0.80572, 0.79979, 0.79373, 0.78756, 0.78127, 0.77487, 0.76840,\
        0.76187, 0.75529, 0.74866, 0.74197, 0.73520, 0.72832, 0.72130, 0.71408, 0.70664, 0.69894, 0.69098,\
        0.68279, 0.67443, 0.66595, 0.65744, 0.64895, 0.64057, 0.63237, 0.62439, 0.61668, 0.60923, 0.60208,\
        0.59538, 0.58900, 0.58270, 0.57645, 0.57024, 0.56409, 0.55800, 0.55195, 0.54595, 0.53998, 0.53402],\
        [0.49496, 0.49614, 0.49685, 0.49738, 0.49788, 0.49851, 0.49960, 0.50119, 0.50360, 0.50683, 0.51100,\
        0.51610, 0.52194, 0.52825, 0.53481, 0.54146, 0.54807, 0.55453, 0.56075, 0.56667, 0.57222, 0.57735,\
        0.58213, 0.58656, 0.58949, 0.58744, 0.57926, 0.56776, 0.55572, 0.54568, 0.53990, 0.53937, 0.54315,\
        0.54905, 0.55296, 0.54811, 0.52751, 0.48619, 0.43662, 0.40744, 0.39163, 0.38442, 0.38694, 0.39215,\
        0.40514, 0.42609, 0.45708, 0.48923, 0.51052, 0.52390, 0.54611, 0.60332, 0.69882, 0.79197, 0.88546,\
        0.96031, 1.00657, 1.03037, 1.03914, 1.03833, 1.03140, 1.02058, 1.00733, 0.99271, 0.97746, 0.96217,\
        0.94723, 0.93296, 0.91953, 0.90704, 0.89540, 0.88443, 0.87402, 0.86410, 0.85472, 0.84589, 0.83764,\
        0.82998, 0.82288, 0.81626, 0.80997, 0.80391, 0.79824, 0.79239, 0.78640, 0.78028, 0.77406, 0.76775,\
        0.76139, 0.75498, 0.74853, 0.74202, 0.73544, 0.72874, 0.72188, 0.71482, 0.70752, 0.69993, 0.69205,\
        0.68390, 0.67555, 0.66705, 0.65849, 0.64993, 0.64146, 0.63315, 0.62506, 0.61721, 0.60960, 0.60228,\
        0.59562, 0.58934, 0.58306, 0.57676, 0.57049, 0.56428, 0.55814, 0.55206, 0.54605, 0.54007, 0.53411],\
        [0.49724, 0.49818, 0.49867, 0.49912, 0.49963, 0.50025, 0.50155, 0.50348, 0.50618, 0.50975, 0.51421,\
        0.51953, 0.52557, 0.53214, 0.53904, 0.54610, 0.55316, 0.56009, 0.56681, 0.57322, 0.57924, 0.58472,\
        0.58967, 0.59479, 0.60016, 0.60040, 0.59332, 0.58280, 0.57173, 0.56249, 0.55704, 0.55636, 0.56015,\
        0.56672, 0.57167, 0.56773, 0.54850, 0.51226, 0.46780, 0.43957, 0.42718, 0.42742, 0.43586, 0.44558,\
        0.45773, 0.47506, 0.49538, 0.51345, 0.52287, 0.53187, 0.55381, 0.61288, 0.71279, 0.80417, 0.90101,\
        0.97183, 1.01548, 1.03876, 1.04749, 1.04635, 1.03867, 1.02681, 1.01238, 0.99657, 0.98020, 0.96392,\
        0.94819, 0.93337, 0.91971, 0.90710, 0.89517, 0.88382, 0.87287, 0.86251, 0.85278, 0.84369, 0.83528,\
        0.82755, 0.82049, 0.81403, 0.80797, 0.80202, 0.79632, 0.79074, 0.78494, 0.77898, 0.77291, 0.76676,\
        0.76056, 0.75432, 0.74805, 0.74174, 0.73535, 0.72886, 0.72220, 0.71532, 0.70818, 0.70071, 0.69292,\
        0.68483, 0.67648, 0.66795, 0.65933, 0.65069, 0.64212, 0.63371, 0.62553, 0.61765, 0.61013, 0.60300,\
        0.59633, 0.58986, 0.58338, 0.57693, 0.57055, 0.56427, 0.55808, 0.55198, 0.54596, 0.53999, 0.53406],\
        [0.49928, 0.50002, 0.50029, 0.50073, 0.50131, 0.50215, 0.50358, 0.50576, 0.50873, 0.51257, 0.51728,\
        0.52282, 0.52908, 0.53589, 0.54309, 0.55048, 0.55789, 0.56519, 0.57224, 0.57895, 0.58526, 0.59092,\
        0.59534, 0.59925, 0.60478, 0.60665, 0.60258, 0.59449, 0.58507, 0.57682, 0.57185, 0.57127, 0.57504,\
        0.58261, 0.59057, 0.58943, 0.57284, 0.53752, 0.49590, 0.46958, 0.46213, 0.46913, 0.47914, 0.49050,\
        0.49472, 0.50339, 0.51027, 0.51813, 0.52755, 0.54306, 0.57432, 0.63875, 0.73097, 0.82351, 0.91949,\
        0.98233, 1.02100, 1.04205, 1.04995, 1.04841, 1.04035, 1.02797, 1.01293, 0.99647, 0.97949, 0.96266,\
        0.94647, 0.93131, 0.91758, 0.90566, 0.89389, 0.88192, 0.87061, 0.86000, 0.85007, 0.84083, 0.83230,\
        0.82454, 0.81758, 0.81139, 0.80582, 0.80038, 0.79472, 0.78903, 0.78328, 0.77743, 0.77147, 0.76545,\
        0.75939, 0.75331, 0.74722, 0.74110, 0.73493, 0.72866, 0.72224, 0.71558, 0.70862, 0.70130, 0.69361,\
        0.68557, 0.67722, 0.66864, 0.65994, 0.65121, 0.64254, 0.63402, 0.62576, 0.61787, 0.61048, 0.60368,\
        0.59706, 0.59025, 0.58347, 0.57683, 0.57035, 0.56401, 0.55779, 0.55169, 0.54567, 0.53973, 0.53384],\
        [0.50055, 0.50134, 0.50170, 0.50213, 0.50280, 0.50384, 0.50548, 0.50786, 0.51106, 0.51513, 0.52007,\
        0.52583, 0.53230, 0.53935, 0.54681, 0.55449, 0.56222, 0.56979, 0.57704, 0.58385, 0.59026, 0.59631,\
        0.60092, 0.60444, 0.60864, 0.61066, 0.60864, 0.60316, 0.59592, 0.58890, 0.58498, 0.58563, 0.59009,\
        0.59888, 0.61076, 0.61476, 0.60605, 0.58175, 0.54311, 0.50204, 0.49857, 0.50641, 0.50970, 0.51392,\
        0.51089, 0.50871, 0.50915, 0.51624, 0.53314, 0.56174, 0.60708, 0.67981, 0.77333, 0.85829, 0.93509,\
        0.98828, 1.02187, 1.04029, 1.04704, 1.04506, 1.03682, 1.02431, 1.00912, 0.99248, 0.97535, 0.95840,\
        0.94212, 0.92681, 0.91268, 0.90030, 0.88948, 0.87808, 0.86702, 0.85652, 0.84663, 0.83738, 0.82882,\
        0.82104, 0.81415, 0.80817, 0.80299, 0.79821, 0.79271, 0.78710, 0.78142, 0.77564, 0.76977, 0.76384,\
        0.75789, 0.75195, 0.74602, 0.74011, 0.73417, 0.72815, 0.72199, 0.71559, 0.70887, 0.70173, 0.69415,\
        0.68612, 0.67774, 0.66910, 0.66032, 0.65148, 0.64269, 0.63404, 0.62565, 0.61763, 0.61013, 0.60343,\
        0.59702, 0.58995, 0.58304, 0.57634, 0.56981, 0.56346, 0.55725, 0.55116, 0.54518, 0.53929, 0.53345],\
        [0.50096, 0.50199, 0.50258, 0.50315, 0.50396, 0.50519, 0.50703, 0.50961, 0.51302, 0.51730, 0.52245,\
        0.52842, 0.53511, 0.54238, 0.55010, 0.55806, 0.56608, 0.57391, 0.58132, 0.58810, 0.59424, 0.60005,\
        0.60532, 0.60941, 0.61287, 0.61471, 0.61383, 0.61033, 0.60515, 0.59965, 0.59714, 0.60188, 0.60926,\
        0.62120, 0.63432, 0.64180, 0.63948, 0.62522, 0.59587, 0.55816, 0.54065, 0.53764, 0.53376, 0.52711,\
        0.51702, 0.50744, 0.50481, 0.51658, 0.54649, 0.59009, 0.64753, 0.73762, 0.82958, 0.89115, 0.94676,\
        0.98997, 1.01836, 1.03401, 1.03944, 1.03700, 1.02873, 1.01636, 1.00135, 0.98492, 0.96804, 0.95143,\
        0.93551, 0.92055, 0.90667, 0.89418, 0.88334, 0.87279, 0.86236, 0.85227, 0.84264, 0.83353, 0.82502,\
        0.81724, 0.81036, 0.80459, 0.79958, 0.79500, 0.79013, 0.78487, 0.77936, 0.77366, 0.76784, 0.76195,\
        0.75607, 0.75023, 0.74446, 0.73874, 0.73305, 0.72730, 0.72142, 0.71532, 0.70887, 0.70197, 0.69448,\
        0.68645, 0.67803, 0.66932, 0.66044, 0.65148, 0.64256, 0.63378, 0.62523, 0.61700, 0.60913, 0.60162,\
        0.59520, 0.58870, 0.58202, 0.57540, 0.56892, 0.56260, 0.55643, 0.55040, 0.54448, 0.53865, 0.53288],\
        [0.50096, 0.50215, 0.50297, 0.50373, 0.50471, 0.50611, 0.50813, 0.51090, 0.51450, 0.51899, 0.52434,\
        0.53050, 0.53739, 0.54489, 0.55285, 0.56108, 0.56940, 0.57755, 0.58523, 0.59210, 0.59802, 0.60369,\
        0.60909, 0.61361, 0.61713, 0.61920, 0.61935, 0.61759, 0.61450, 0.61120, 0.61208, 0.62020, 0.63167,\
        0.64593, 0.65966, 0.66762, 0.66663, 0.65600, 0.63494, 0.60558, 0.58323, 0.57070, 0.55785, 0.54281,\
        0.52622, 0.51236, 0.50950, 0.52998, 0.57787, 0.63753, 0.70451, 0.80570, 0.87118, 0.91506, 0.95612,\
        0.98919, 1.01168, 1.02411, 1.02803, 1.02511, 1.01691, 1.00486, 0.99027, 0.97430, 0.95802, 0.94215,\
        0.92703, 0.91288, 0.89999, 0.88799, 0.87714, 0.86696, 0.85711, 0.84757, 0.83838, 0.82959, 0.82126,\
        0.81352, 0.80658, 0.80097, 0.79648, 0.79215, 0.78761, 0.78262, 0.77724, 0.77156, 0.76571, 0.75980,\
        0.75392, 0.74815, 0.74251, 0.73700, 0.73155, 0.72609, 0.72051, 0.71471, 0.70855, 0.70188, 0.69450,\
        0.68649, 0.67804, 0.66927, 0.66030, 0.65123, 0.64217, 0.63325, 0.62454, 0.61616, 0.60820, 0.60081,\
        0.59383, 0.58718, 0.58060, 0.57408, 0.56769, 0.56144, 0.55535, 0.54939, 0.54356, 0.53782, 0.53215],\
        [0.50072, 0.50200, 0.50297, 0.50389, 0.50501, 0.50657, 0.50874, 0.51167, 0.51546, 0.52012, 0.52566,\
        0.53202, 0.53910, 0.54680, 0.55497, 0.56346, 0.57207, 0.58059, 0.58871, 0.59598, 0.60209, 0.60760,\
        0.61286, 0.61756, 0.62143, 0.62419, 0.62564, 0.62589, 0.62557, 0.62619, 0.63045, 0.63980, 0.65312,\
        0.66883, 0.68316, 0.69030, 0.68788, 0.67867, 0.66329, 0.64280, 0.62357, 0.60627, 0.58643, 0.56623,\
        0.54629, 0.53383, 0.53652, 0.57423, 0.64995, 0.71604, 0.78289, 0.85354, 0.89978, 0.93322, 0.96261,\
        0.98616, 1.00246, 1.01139, 1.01366, 1.01027, 1.00227, 0.99070, 0.97666, 0.96130, 0.94580, 0.93112,\
        0.91728, 0.90401, 0.89202, 0.88092, 0.87057, 0.86087, 0.85163, 0.84273, 0.83414, 0.82585, 0.81791,\
        0.81044, 0.80370, 0.79816, 0.79401, 0.79000, 0.78560, 0.78068, 0.77525, 0.76944, 0.76344, 0.75739,\
        0.75145, 0.74569, 0.74017, 0.73485, 0.72967, 0.72451, 0.71925, 0.71374, 0.70783, 0.70136, 0.69412,\
        0.68619, 0.67776, 0.66895, 0.65989, 0.65071, 0.64152, 0.63245, 0.62362, 0.61512, 0.60703, 0.59946,\
        0.59239, 0.58558, 0.57896, 0.57249, 0.56618, 0.56002, 0.55402, 0.54816, 0.54243, 0.53680, 0.53124],\
        [0.50032, 0.50163, 0.50268, 0.50369, 0.50490, 0.50656, 0.50885, 0.51192, 0.51586, 0.52070, 0.52641,\
        0.53295, 0.54021, 0.54808, 0.55642, 0.56509, 0.57392, 0.58273, 0.59128, 0.59910, 0.60536, 0.61091,\
        0.61625, 0.62126, 0.62574, 0.62951, 0.63251, 0.63498, 0.63755, 0.64137, 0.64791, 0.65790, 0.67104,\
        0.68654, 0.70104, 0.70716, 0.70465, 0.69759, 0.68864, 0.67634, 0.66182, 0.64348, 0.62123, 0.59804,\
        0.57627, 0.57455, 0.59516, 0.67282, 0.74125, 0.78506, 0.83692, 0.88376, 0.91973, 0.94455, 0.96446,\
        0.97996, 0.99061, 0.99624, 0.99704, 0.99338, 0.98576, 0.97486, 0.96152, 0.94677, 0.93195, 0.91877,\
        0.90723, 0.89505, 0.88373, 0.87329, 0.86359, 0.85456, 0.84605, 0.83794, 0.83013, 0.82256, 0.81525,\
        0.80831, 0.80209, 0.79704, 0.79291, 0.78887, 0.78436, 0.77922, 0.77350, 0.76736, 0.76103, 0.75472,\
        0.74861, 0.74282, 0.73739, 0.73229, 0.72740, 0.72258, 0.71766, 0.71245, 0.70677, 0.70047, 0.69339,\
        0.68559, 0.67721, 0.66838, 0.65924, 0.64993, 0.64059, 0.63137, 0.62241, 0.61383, 0.60571, 0.59797,\
        0.59073, 0.58380, 0.57713, 0.57068, 0.56443, 0.55837, 0.55247, 0.54673, 0.54111, 0.53561, 0.53018],\
        [0.49983, 0.50113, 0.50218, 0.50320, 0.50443, 0.50612, 0.50847, 0.51164, 0.51572, 0.52072, 0.52661,\
        0.53332, 0.54074, 0.54874, 0.55720, 0.56595, 0.57486, 0.58373, 0.59236, 0.60031, 0.60700, 0.61306,\
        0.61888, 0.62450, 0.62983, 0.63480, 0.63946, 0.64406, 0.64910, 0.65528, 0.66326, 0.67325, 0.68485,\
        0.69747, 0.71065, 0.71807, 0.71909, 0.71650, 0.71251, 0.70650, 0.69797, 0.68199, 0.65922, 0.63481,\
        0.61102, 0.61488, 0.65641, 0.72677, 0.77977, 0.82062, 0.86621, 0.90178, 0.92896, 0.94772, 0.96067,\
        0.96986, 0.97593, 0.97895, 0.97879, 0.97527, 0.96837, 0.95839, 0.94597, 0.93193, 0.91752, 0.90476,\
        0.89691, 0.88580, 0.87504, 0.86521, 0.85625, 0.84804, 0.84043, 0.83327, 0.82644, 0.81983, 0.81342,\
        0.80725, 0.80154, 0.79705, 0.79309, 0.78884, 0.78394, 0.77830, 0.77201, 0.76530, 0.75844, 0.75173,\
        0.74536, 0.73950, 0.73416, 0.72931, 0.72477, 0.72036, 0.71583, 0.71094, 0.70551, 0.69938, 0.69248,\
        0.68481, 0.67647, 0.66761, 0.65837, 0.64892, 0.63942, 0.63001, 0.62087, 0.61215, 0.60400, 0.59632,\
        0.58883, 0.58177, 0.57507, 0.56865, 0.56247, 0.55650, 0.55072, 0.54510, 0.53962, 0.53425, 0.52897],\
        [0.49932, 0.50057, 0.50157, 0.50252, 0.50368, 0.50531, 0.50766, 0.51090, 0.51510, 0.52026, 0.52632,\
        0.53319, 0.54075, 0.54886, 0.55738, 0.56615, 0.57499, 0.58372, 0.59212, 0.59997, 0.60724, 0.61409,\
        0.62067, 0.62711, 0.63342, 0.63963, 0.64587, 0.65233, 0.65936, 0.66728, 0.67622, 0.68604, 0.69651,\
        0.70723, 0.71831, 0.72669, 0.73116, 0.73297, 0.73333, 0.73220, 0.72815, 0.71710, 0.69987, 0.68086,\
        0.66202, 0.66046, 0.69792, 0.76557, 0.81353, 0.85544, 0.89178, 0.91737, 0.93435, 0.94530, 0.95193,\
        0.95599, 0.95856, 0.95991, 0.95959, 0.95682, 0.95103, 0.94221, 0.93101, 0.91816, 0.90447, 0.89280,\
        0.88404, 0.87494, 0.86545, 0.85656, 0.84852, 0.84130, 0.83476, 0.82873, 0.82305, 0.81760, 0.81233,\
        0.80728, 0.80263, 0.79857, 0.79439, 0.78973, 0.78423, 0.77782, 0.77070, 0.76317, 0.75560, 0.74833,\
        0.74163, 0.73565, 0.73044, 0.72590, 0.72182, 0.71790, 0.71386, 0.70938, 0.70424, 0.69831, 0.69156,\
        0.68396, 0.67562, 0.66668, 0.65732, 0.64770, 0.63800, 0.62839, 0.61901, 0.61005, 0.60162, 0.59384,\
        0.58648, 0.57946, 0.57279, 0.56641, 0.56031, 0.55444, 0.54878, 0.54330, 0.53796, 0.53275, 0.52762],\
        [0.49886, 0.50004, 0.50094, 0.50174, 0.50272, 0.50421, 0.50649, 0.50977, 0.51408, 0.51939, 0.52562,\
        0.53265, 0.54035, 0.54855, 0.55711, 0.56586, 0.57461, 0.58318, 0.59137, 0.59912, 0.60686, 0.61440,\
        0.62174, 0.62901, 0.63629, 0.64365, 0.65122, 0.65918, 0.66772, 0.67694, 0.68664, 0.69628, 0.70576,\
        0.71548, 0.72518, 0.73379, 0.74049, 0.74555, 0.74950, 0.75229, 0.75294, 0.74983, 0.74252, 0.73125,\
        0.72066, 0.72632, 0.76914, 0.82614, 0.86163, 0.89166, 0.91356, 0.92761, 0.93530, 0.93870, 0.93943,\
        0.93916, 0.93918, 0.93984, 0.94032, 0.93900, 0.93466, 0.92699, 0.91697, 0.90613, 0.89370, 0.88227,\
        0.87269, 0.86384, 0.85531, 0.84743, 0.84044, 0.83435, 0.82901, 0.82423, 0.81983, 0.81565, 0.81165,\
        0.80781, 0.80412, 0.80033, 0.79624, 0.79121, 0.78496, 0.77756, 0.76936, 0.76080, 0.75235, 0.74442,\
        0.73732, 0.73124, 0.72621, 0.72209, 0.71858, 0.71530, 0.71185, 0.70788, 0.70310, 0.69739, 0.69072,\
        0.68311, 0.67469, 0.66563, 0.65611, 0.64630, 0.63639, 0.62655, 0.61694, 0.60773, 0.59914, 0.59126,\
        0.58390, 0.57693, 0.57030, 0.56399, 0.55797, 0.55221, 0.54667, 0.54133, 0.53616, 0.53111, 0.52616],\
        [0.49850, 0.49962, 0.50039, 0.50099, 0.50169, 0.50290, 0.50505, 0.50834, 0.51276, 0.51824, 0.52465,\
        0.53185, 0.53968, 0.54798, 0.55658, 0.56530, 0.57399, 0.58250, 0.59072, 0.59863, 0.60650, 0.61433,\
        0.62222, 0.63022, 0.63833, 0.64662, 0.65519, 0.66418, 0.67371, 0.68376, 0.69397, 0.70337, 0.71193,\
        0.72075, 0.72983, 0.73865, 0.74672, 0.75396, 0.76052, 0.76648, 0.77181, 0.77665, 0.77995, 0.77578,\
        0.77176, 0.78815, 0.82736, 0.86959, 0.89801, 0.91670, 0.92663, 0.93108, 0.93128, 0.92863, 0.92453,\
        0.92080, 0.91911, 0.91994, 0.92217, 0.92278, 0.92016, 0.91335, 0.90364, 0.89379, 0.88303, 0.87199,\
        0.86204, 0.85312, 0.84509, 0.83806, 0.83211, 0.82721, 0.82315, 0.81969, 0.81659, 0.81368, 0.81090,\
        0.80821, 0.80545, 0.80233, 0.79838, 0.79291, 0.78580, 0.77719, 0.76769, 0.75796, 0.74853, 0.73988,\
        0.73235, 0.72620, 0.72146, 0.71791, 0.71513, 0.71262, 0.70989, 0.70653, 0.70219, 0.69666, 0.68997,\
        0.68226, 0.67369, 0.66446, 0.65474, 0.64473, 0.63461, 0.62455, 0.61474, 0.60536, 0.59663, 0.58866,\
        0.58124, 0.57426, 0.56766, 0.56141, 0.55547, 0.54982, 0.54442, 0.53923, 0.53422, 0.52935, 0.52459],\
        [0.49828, 0.49936, 0.50003, 0.50042, 0.50074, 0.50152, 0.50348, 0.50678, 0.51130, 0.51694, 0.52353,\
        0.53091, 0.53889, 0.54730, 0.55595, 0.56468, 0.57333, 0.58180, 0.59010, 0.59835, 0.60596, 0.61385,\
        0.62220, 0.63079, 0.63954, 0.64846, 0.65765, 0.66717, 0.67707, 0.68729, 0.69746, 0.70627, 0.71434,\
        0.72273, 0.73164, 0.74079, 0.74977, 0.75837, 0.76655, 0.77443, 0.78235, 0.79108, 0.80164, 0.80465,\
        0.80604, 0.82483, 0.85782, 0.89186, 0.91330, 0.92463, 0.92872, 0.92744, 0.92281, 0.91642, 0.90931,\
        0.90325, 0.90112, 0.90241, 0.90654, 0.90861, 0.90796, 0.90186, 0.89240, 0.88246, 0.87212, 0.86159,\
        0.85163, 0.84267, 0.83494, 0.82860, 0.82365, 0.81993, 0.81717, 0.81502, 0.81314, 0.81136, 0.80964,\
        0.80797, 0.80624, 0.80394, 0.80014, 0.79453, 0.78631, 0.77619, 0.76528, 0.75435, 0.74399, 0.73464,\
        0.72672, 0.72056, 0.71624, 0.71346, 0.71160, 0.70999, 0.70804, 0.70529, 0.70138, 0.69598, 0.68921,\
        0.68134, 0.67258, 0.66313, 0.65320, 0.64298, 0.63265, 0.62240, 0.61244, 0.60298, 0.59415, 0.58603,\
        0.57851, 0.57149, 0.56489, 0.55868, 0.55283, 0.54728, 0.54202, 0.53699, 0.53216, 0.52749, 0.52292],\
        [0.49820, 0.49928, 0.49991, 0.50017, 0.50017, 0.50029, 0.50204, 0.50527, 0.50986, 0.51565, 0.52243,\
        0.53000, 0.53815, 0.54668, 0.55540, 0.56415, 0.57276, 0.58111, 0.58912, 0.59673, 0.60443, 0.61299,\
        0.62186, 0.63088, 0.64001, 0.64927, 0.65869, 0.66827, 0.67799, 0.68768, 0.69691, 0.70534, 0.71324,\
        0.72158, 0.73069, 0.74030, 0.74996, 0.75937, 0.76838, 0.77699, 0.78542, 0.79414, 0.80471, 0.81511,\
        0.82551, 0.84375, 0.86905, 0.89317, 0.91091, 0.92027, 0.92217, 0.91817, 0.91117, 0.90373, 0.89625,\
        0.89044, 0.88834, 0.88998, 0.89340, 0.89600, 0.89642, 0.89076, 0.88201, 0.87214, 0.86184, 0.85145,\
        0.84149, 0.83248, 0.82495, 0.81921, 0.81520, 0.81261, 0.81111, 0.81015, 0.80932, 0.80841, 0.80742,\
        0.80648, 0.80565, 0.80452, 0.80196, 0.79573, 0.78556, 0.77379, 0.76161, 0.74973, 0.73864, 0.72877,\
        0.72054, 0.71444, 0.71073, 0.70898, 0.70821, 0.70754, 0.70635, 0.70414, 0.70050, 0.69517, 0.68831,\
        0.68025, 0.67125, 0.66158, 0.65144, 0.64102, 0.63050, 0.62008, 0.60997, 0.60041, 0.59152, 0.58330,\
        0.57568, 0.56860, 0.56199, 0.55581, 0.55002, 0.54459, 0.53948, 0.53463, 0.53000, 0.52553, 0.52118],\
        [0.49823, 0.49934, 0.49996, 0.50015, 0.50002, 0.49979, 0.50101, 0.50399, 0.50859, 0.51452, 0.52149,\
        0.52926, 0.53758, 0.54625, 0.55506, 0.56386, 0.57249, 0.58078, 0.58858, 0.59591, 0.60373, 0.61250,\
        0.62154, 0.63069, 0.63994, 0.64926, 0.65861, 0.66794, 0.67713, 0.68602, 0.69443, 0.70215, 0.70951,\
        0.71799, 0.72761, 0.73779, 0.74803, 0.75796, 0.76735, 0.77616, 0.78455, 0.79319, 0.80396, 0.81726,\
        0.83193, 0.84952, 0.86928, 0.88754, 0.90129, 0.90916, 0.91110, 0.90615, 0.89838, 0.89168, 0.88541,\
        0.88074, 0.87888, 0.87981, 0.88213, 0.88405, 0.88357, 0.87902, 0.87142, 0.86218, 0.85217, 0.84192,\
        0.83193, 0.82280, 0.81531, 0.81015, 0.80709, 0.80538, 0.80507, 0.80506, 0.80496, 0.80459, 0.80389,\
        0.80315, 0.80265, 0.80213, 0.79997, 0.79301, 0.78172, 0.76906, 0.75623, 0.74392, 0.73259, 0.72256,\
        0.71420, 0.70818, 0.70530, 0.70489, 0.70527, 0.70546, 0.70491, 0.70313, 0.69963, 0.69423, 0.68720,\
        0.67888, 0.66962, 0.65971, 0.64937, 0.63879, 0.62814, 0.61757, 0.60730, 0.59763, 0.58870, 0.58041,\
        0.57270, 0.56555, 0.55892, 0.55277, 0.54706, 0.54176, 0.53680, 0.53214, 0.52772, 0.52349, 0.51937],\
        [0.49832, 0.49946, 0.50003, 0.50008, 0.49966, 0.49935, 0.50015, 0.50292, 0.50758, 0.51368, 0.52086,\
        0.52883, 0.53733, 0.54613, 0.55505, 0.56392, 0.57262, 0.58104, 0.58906, 0.59661, 0.60436, 0.61270,\
        0.62140, 0.63037, 0.63950, 0.64868, 0.65782, 0.66675, 0.67532, 0.68334, 0.69067, 0.69731, 0.70421,\
        0.71317, 0.72347, 0.73423, 0.74492, 0.75519, 0.76483, 0.77380, 0.78231, 0.79109, 0.80182, 0.81568,\
        0.83100, 0.84712, 0.86323, 0.87758, 0.88845, 0.89525, 0.89819, 0.89377, 0.88697, 0.88080, 0.87563,\
        0.87197, 0.87038, 0.87070, 0.87197, 0.87279, 0.87166, 0.86769, 0.86109, 0.85268, 0.84321, 0.83328,\
        0.82341, 0.81417, 0.80652, 0.80182, 0.79986, 0.79865, 0.79914, 0.79957, 0.79982, 0.79964, 0.79890,\
        0.79794, 0.79705, 0.79570, 0.79219, 0.78488, 0.77411, 0.76170, 0.74904, 0.73701, 0.72610, 0.71654,\
        0.70853, 0.70264, 0.70080, 0.70190, 0.70309, 0.70383, 0.70372, 0.70221, 0.69876, 0.69311, 0.68576,\
        0.67712, 0.66756, 0.65740, 0.64689, 0.63623, 0.62553, 0.61489, 0.60452, 0.59478, 0.58575, 0.57734,\
        0.56952, 0.56230, 0.55564, 0.54953, 0.54392, 0.53875, 0.53399, 0.52954, 0.52536, 0.52137, 0.51751],\
        [0.49842, 0.49957, 0.50007, 0.49989, 0.49923, 0.49880, 0.49947, 0.50224, 0.50699, 0.51325, 0.52064,\
        0.52882, 0.53750, 0.54643, 0.55541, 0.56432, 0.57306, 0.58164, 0.59005, 0.59820, 0.60547, 0.61305,\
        0.62124, 0.62989, 0.63880, 0.64780, 0.65669, 0.66528, 0.67334, 0.68070, 0.68724, 0.69318, 0.69977,\
        0.70897, 0.71953, 0.73059, 0.74152, 0.75198, 0.76180, 0.77100, 0.77983, 0.78891, 0.79940, 0.81245,\
        0.82646, 0.84029, 0.85337, 0.86486, 0.87385, 0.87984, 0.88213, 0.87983, 0.87502, 0.87010, 0.86604,\
        0.86328, 0.86203, 0.86205, 0.86259, 0.86262, 0.86112, 0.85745, 0.85160, 0.84400, 0.83521, 0.82581,\
        0.81638, 0.80746, 0.79980, 0.79546, 0.79363, 0.79293, 0.79311, 0.79349, 0.79365, 0.79335, 0.79248,\
        0.79117, 0.78944, 0.78672, 0.78196, 0.77432, 0.76406, 0.75232, 0.74039, 0.72924, 0.71943, 0.71118,\
        0.70454, 0.69986, 0.69889, 0.70028, 0.70161, 0.70251, 0.70268, 0.70123, 0.69757, 0.69163, 0.68387,\
        0.67482, 0.66491, 0.65450, 0.64386, 0.63321, 0.62261, 0.61205, 0.60163, 0.59182, 0.58263, 0.57404,\
        0.56607, 0.55877, 0.55210, 0.54605, 0.54057, 0.53558, 0.53103, 0.52683, 0.52292, 0.51921, 0.51562],\
        [0.49848, 0.49964, 0.50005, 0.49966, 0.49887, 0.49845, 0.49929, 0.50202, 0.50684, 0.51331, 0.52094,\
        0.52935, 0.53819, 0.54721, 0.55621, 0.56505, 0.57368, 0.58211, 0.59037, 0.59826, 0.60550, 0.61284,\
        0.62074, 0.62916, 0.63792, 0.64679, 0.65555, 0.66396, 0.67180, 0.67891, 0.68524, 0.69114, 0.69771,\
        0.70626, 0.71666, 0.72764, 0.73848, 0.74889, 0.75876, 0.76815, 0.77733, 0.78676, 0.79704, 0.80864,\
        0.82017, 0.83102, 0.84120, 0.85035, 0.85785, 0.86304, 0.86535, 0.86464, 0.86194, 0.85874, 0.85601,\
        0.85424, 0.85352, 0.85357, 0.85382, 0.85349, 0.85186, 0.84847, 0.84323, 0.83637, 0.82832, 0.81960,\
        0.81084, 0.80285, 0.79600, 0.79130, 0.78881, 0.78767, 0.78730, 0.78718, 0.78691, 0.78622, 0.78498,\
        0.78311, 0.78046, 0.77659, 0.77088, 0.76295, 0.75302, 0.74193, 0.73085, 0.72092, 0.71273, 0.70636,\
        0.70191, 0.69896, 0.69816, 0.69917, 0.70018, 0.70110, 0.70168, 0.70044, 0.69624, 0.68966, 0.68134,\
        0.67183, 0.66155, 0.65086, 0.64009, 0.62949, 0.61919, 0.60902, 0.59865, 0.58867, 0.57918, 0.57034,\
        0.56224, 0.55489, 0.54826, 0.54231, 0.53699, 0.53222, 0.52793, 0.52402, 0.52042, 0.51701, 0.51373],\
        [0.49849, 0.49967, 0.50001, 0.49946, 0.49858, 0.49820, 0.49911, 0.50203, 0.50713, 0.51391, 0.52183,\
        0.53048, 0.53950, 0.54858, 0.55752, 0.56616, 0.57445, 0.58240, 0.59003, 0.59730, 0.60450, 0.61191,\
        0.61978, 0.62816, 0.63690, 0.64577, 0.65456, 0.66302, 0.67097, 0.67825, 0.68486, 0.69102, 0.69722,\
        0.70535, 0.71534, 0.72575, 0.73610, 0.74614, 0.75580, 0.76517, 0.77447, 0.78404, 0.79417, 0.80411,\
        0.81260, 0.82010, 0.82754, 0.83478, 0.84113, 0.84589, 0.84854, 0.84910, 0.84815, 0.84664, 0.84536,\
        0.84472, 0.84474, 0.84517, 0.84552, 0.84521, 0.84371, 0.84067, 0.83600, 0.82984, 0.82252, 0.81452,\
        0.80634, 0.79887, 0.79274, 0.78797, 0.78480, 0.78288, 0.78172, 0.78088, 0.77996, 0.77869, 0.77687,\
        0.77434, 0.77089, 0.76617, 0.75986, 0.75176, 0.74209, 0.73151, 0.72114, 0.71245, 0.70618, 0.70160,\
        0.69892, 0.69738, 0.69693, 0.69755, 0.69846, 0.69935, 0.70019, 0.69928, 0.69415, 0.68683, 0.67798,\
        0.66804, 0.65739, 0.64639, 0.63540, 0.62475, 0.61472, 0.60535, 0.59536, 0.58486, 0.57505, 0.56605,\
        0.55790, 0.55057, 0.54405, 0.53827, 0.53318, 0.52869, 0.52471, 0.52114, 0.51787, 0.51481, 0.51186],\
        [0.49843, 0.49967, 0.50000, 0.49935, 0.49839, 0.49804, 0.49908, 0.50236, 0.50793, 0.51512, 0.52340,\
        0.53231, 0.54149, 0.55062, 0.55944, 0.56780, 0.57560, 0.58287, 0.58975, 0.59650, 0.60335, 0.61060,\
        0.61851, 0.62698, 0.63583, 0.64482, 0.65376, 0.66246, 0.67076, 0.67853, 0.68575, 0.69262, 0.69961,\
        0.70693, 0.71555, 0.72483, 0.73427, 0.74361, 0.75276, 0.76176, 0.77072, 0.77980, 0.78893, 0.79738,\
        0.80339, 0.80775, 0.81311, 0.81892, 0.82453, 0.82909, 0.83216, 0.83373, 0.83414, 0.83407, 0.83418,\
        0.83475, 0.83573, 0.83682, 0.83759, 0.83758, 0.83641, 0.83381, 0.82973, 0.82426, 0.81765, 0.81037,\
        0.80311, 0.79598, 0.78980, 0.78481, 0.78103, 0.77830, 0.77630, 0.77463, 0.77298, 0.77104, 0.76858,\
        0.76541, 0.76132, 0.75606, 0.74945, 0.74142, 0.73216, 0.72213, 0.71221, 0.70432, 0.70019, 0.69739,\
        0.69566, 0.69496, 0.69496, 0.69552, 0.69631, 0.69702, 0.69728, 0.69545, 0.69029, 0.68278, 0.67367,\
        0.66343, 0.65245, 0.64112, 0.62981, 0.61887, 0.60859, 0.59920, 0.58965, 0.57958, 0.56991, 0.56100,\
        0.55295, 0.54577, 0.53945, 0.53393, 0.52914, 0.52499, 0.52138, 0.51820, 0.51532, 0.51264, 0.51005],\
        [0.49828, 0.49962, 0.50002, 0.49934, 0.49831, 0.49806, 0.49960, 0.50335, 0.50940, 0.51703, 0.52568,\
        0.53488, 0.54424, 0.55341, 0.56211, 0.57014, 0.57738, 0.58387, 0.58981, 0.59566, 0.60194, 0.60913,\
        0.61717, 0.62580, 0.63479, 0.64394, 0.65309, 0.66211, 0.67087, 0.67926, 0.68720, 0.69459, 0.70162,\
        0.70872, 0.71623, 0.72427, 0.73262, 0.74104, 0.74941, 0.75769, 0.76586, 0.77390, 0.78152, 0.78802,\
        0.79230, 0.79532, 0.79915, 0.80381, 0.80889, 0.81322, 0.81658, 0.81893, 0.82029, 0.82129, 0.82268,\
        0.82449, 0.82654, 0.82848, 0.82989, 0.83039, 0.82968, 0.82760, 0.82415, 0.81937, 0.81341, 0.80659,\
        0.79966, 0.79311, 0.78688, 0.78152, 0.77715, 0.77368, 0.77088, 0.76844, 0.76605, 0.76345, 0.76040,\
        0.75669, 0.75214, 0.74661, 0.74000, 0.73236, 0.72386, 0.71485, 0.70590, 0.69861, 0.69549, 0.69379,\
        0.69277, 0.69241, 0.69256, 0.69302, 0.69351, 0.69367, 0.69292, 0.69027, 0.68509, 0.67767, 0.66854,\
        0.65815, 0.64692, 0.63526, 0.62358, 0.61228, 0.60162, 0.59186, 0.58251, 0.57303, 0.56381, 0.55520,\
        0.54742, 0.54052, 0.53450, 0.52932, 0.52491, 0.52117, 0.51799, 0.51525, 0.51281, 0.51054, 0.50834],\
        [0.49804, 0.49950, 0.50005, 0.49943, 0.49835, 0.49822, 0.50028, 0.50491, 0.51157, 0.51967, 0.52870,\
        0.53818, 0.54772, 0.55697, 0.56559, 0.57334, 0.58005, 0.58573, 0.59062, 0.59532, 0.60084, 0.60799,\
        0.61613, 0.62483, 0.63387, 0.64309, 0.65239, 0.66167, 0.67084, 0.67982, 0.68847, 0.69651, 0.70344,\
        0.70986, 0.71641, 0.72337, 0.73067, 0.73812, 0.74556, 0.75290, 0.76003, 0.76681, 0.77291, 0.77785,\
        0.78132, 0.78398, 0.78692, 0.79060, 0.79482, 0.79868, 0.80208, 0.80506, 0.80692, 0.80853, 0.81115,\
        0.81412, 0.81724, 0.82012, 0.82229, 0.82338, 0.82319, 0.82164, 0.81882, 0.81482, 0.80963, 0.80327,\
        0.79648, 0.78987, 0.78351, 0.77779, 0.77290, 0.76881, 0.76534, 0.76224, 0.75923, 0.75606, 0.75251,\
        0.74839, 0.74358, 0.73798, 0.73161, 0.72459, 0.71719, 0.70981, 0.70284, 0.69699, 0.69331, 0.69135,\
        0.69035, 0.68996, 0.68996, 0.69012, 0.69012, 0.68960, 0.68800, 0.68472, 0.67939, 0.67203, 0.66296,\
        0.65253, 0.64113, 0.62915, 0.61712, 0.60553, 0.59473, 0.58473, 0.57522, 0.56598, 0.55711, 0.54886,\
        0.54142, 0.53489, 0.52926, 0.52450, 0.52054, 0.51728, 0.51460, 0.51235, 0.51039, 0.50857, 0.50677],\
        [0.49774, 0.49930, 0.50002, 0.49967, 0.49872, 0.49888, 0.50162, 0.50717, 0.51448, 0.52303, 0.53239,\
        0.54213, 0.55187, 0.56122, 0.56985, 0.57743, 0.58374, 0.58874, 0.59268, 0.59625, 0.60100, 0.60784,\
        0.61577, 0.62426, 0.63312, 0.64221, 0.65146, 0.66082, 0.67020, 0.67953, 0.68863, 0.69705, 0.70397,\
        0.70970, 0.71545, 0.72159, 0.72803, 0.73459, 0.74111, 0.74745, 0.75351, 0.75909, 0.76397, 0.76789,\
        0.77085, 0.77327, 0.77580, 0.77880, 0.78218, 0.78559, 0.78888, 0.79195, 0.79418, 0.79646, 0.79999,\
        0.80371, 0.80785, 0.81163, 0.81456, 0.81624, 0.81655, 0.81545, 0.81318, 0.81002, 0.80569, 0.80050,\
        0.79341, 0.78621, 0.77946, 0.77338, 0.76808, 0.76353, 0.75958, 0.75599, 0.75252, 0.74892, 0.74501,\
        0.74063, 0.73570, 0.73019, 0.72417, 0.71784, 0.71156, 0.70576, 0.70097, 0.69603, 0.69207, 0.68963,\
        0.68828, 0.68758, 0.68724, 0.68695, 0.68639, 0.68519, 0.68292, 0.67916, 0.67363, 0.66631, 0.65733,\
        0.64695, 0.63545, 0.62323, 0.61083, 0.59900, 0.58809, 0.57786, 0.56814, 0.55888, 0.55017, 0.54220,\
        0.53513, 0.52902, 0.52385, 0.51958, 0.51613, 0.51341, 0.51128, 0.50958, 0.50814, 0.50679, 0.50540],\
        [0.49749, 0.49917, 0.50006, 0.50021, 0.49969, 0.50073, 0.50448, 0.51050, 0.51818, 0.52702, 0.53661,\
        0.54655, 0.55647, 0.56598, 0.57469, 0.58225, 0.58836, 0.59289, 0.59620, 0.59901, 0.60296, 0.60902,\
        0.61623, 0.62412, 0.63247, 0.64116, 0.65011, 0.65927, 0.66856, 0.67788, 0.68700, 0.69553, 0.70240,\
        0.70747, 0.71288, 0.71863, 0.72451, 0.73035, 0.73602, 0.74144, 0.74652, 0.75111, 0.75506, 0.75828,\
        0.76084, 0.76305, 0.76529, 0.76781, 0.77063, 0.77362, 0.77663, 0.77955, 0.78226, 0.78528, 0.78907,\
        0.79341, 0.79821, 0.80267, 0.80636, 0.80850, 0.80937, 0.80853, 0.80653, 0.80410, 0.80103, 0.79521,\
        0.78849, 0.78124, 0.77431, 0.76804, 0.76252, 0.75774, 0.75352, 0.74966, 0.74592, 0.74207, 0.73794,\
        0.73344, 0.72852, 0.72319, 0.71756, 0.71182, 0.70645, 0.70159, 0.69753, 0.69379, 0.69037, 0.68787,\
        0.68623, 0.68517, 0.68439, 0.68360, 0.68248, 0.68068, 0.67788, 0.67374, 0.66806, 0.66078, 0.65194,\
        0.64170, 0.63027, 0.61795, 0.60525, 0.59322, 0.58206, 0.57146, 0.56139, 0.55193, 0.54322, 0.53545,\
        0.52872, 0.52307, 0.51842, 0.51469, 0.51180, 0.50966, 0.50814, 0.50704, 0.50614, 0.50527, 0.50428],\
        [0.49743, 0.49933, 0.50058, 0.50130, 0.50193, 0.50402, 0.50838, 0.51466, 0.52248, 0.53142, 0.54111,\
        0.55117, 0.56123, 0.57089, 0.57974, 0.58738, 0.59347, 0.59768, 0.60046, 0.60298, 0.60626, 0.61111,\
        0.61722, 0.62418, 0.63175, 0.63977, 0.64817, 0.65685, 0.66573, 0.67464, 0.68334, 0.69133, 0.69762,\
        0.70300, 0.70877, 0.71452, 0.72008, 0.72536, 0.73031, 0.73494, 0.73921, 0.74303, 0.74630, 0.74899,\
        0.75123, 0.75322, 0.75522, 0.75741, 0.75983, 0.76244, 0.76516, 0.76796, 0.77089, 0.77424, 0.77823,\
        0.78284, 0.78784, 0.79276, 0.79699, 0.79948, 0.80129, 0.80053, 0.79852, 0.79639, 0.79331, 0.78813,\
        0.78158, 0.77461, 0.76783, 0.76162, 0.75614, 0.75136, 0.74714, 0.74325, 0.73945, 0.73552, 0.73132,\
        0.72680, 0.72198, 0.71693, 0.71173, 0.70644, 0.70159, 0.69779, 0.69416, 0.69096, 0.68809, 0.68575,\
        0.68397, 0.68260, 0.68141, 0.68014, 0.67849, 0.67619, 0.67295, 0.66853, 0.66275, 0.65554, 0.64692,\
        0.63696, 0.62578, 0.61363, 0.60099, 0.58860, 0.57684, 0.56562, 0.55503, 0.54522, 0.53638, 0.52871,\
        0.52233, 0.51720, 0.51315, 0.51001, 0.50769, 0.50616, 0.50530, 0.50484, 0.50450, 0.50408, 0.50347],\
        [0.49766, 0.49992, 0.50170, 0.50323, 0.50507, 0.50808, 0.51281, 0.51921, 0.52702, 0.53592, 0.54557,\
        0.55563, 0.56574, 0.57551, 0.58445, 0.59210, 0.59813, 0.60204, 0.60435, 0.60648, 0.60925, 0.61315,\
        0.61816, 0.62408, 0.63069, 0.63787, 0.64549, 0.65348, 0.66170, 0.66997, 0.67800, 0.68536, 0.69172,\
        0.69764, 0.70365, 0.70945, 0.71486, 0.71967, 0.72398, 0.72798, 0.73165, 0.73491, 0.73768, 0.73998,\
        0.74194, 0.74374, 0.74556, 0.74752, 0.74964, 0.75193, 0.75436, 0.75697, 0.75986, 0.76321, 0.76718,\
        0.77175, 0.77671, 0.78165, 0.78595, 0.78914, 0.79114, 0.79103, 0.78948, 0.78723, 0.78390, 0.77908,\
        0.77305, 0.76651, 0.76007, 0.75414, 0.74893, 0.74442, 0.74046, 0.73679, 0.73314, 0.72930, 0.72515,\
        0.72068, 0.71599, 0.71125, 0.70666, 0.70212, 0.69755, 0.69403, 0.69090, 0.68805, 0.68550, 0.68330,\
        0.68144, 0.67982, 0.67828, 0.67658, 0.67448, 0.67175, 0.66816, 0.66352, 0.65768, 0.65058, 0.64223,\
        0.63265, 0.62189, 0.61002, 0.59740, 0.58468, 0.57220, 0.56025, 0.54903, 0.53876, 0.52969, 0.52207,\
        0.51608, 0.51159, 0.50825, 0.50575, 0.50400, 0.50307, 0.50293, 0.50312, 0.50329, 0.50328, 0.50299],\
        [0.49819, 0.50091, 0.50331, 0.50566, 0.50843, 0.51215, 0.51721, 0.52367, 0.53139, 0.54012, 0.54960,\
        0.55953, 0.56956, 0.57933, 0.58832, 0.59582, 0.60126, 0.60478, 0.60695, 0.60886, 0.61123, 0.61442,\
        0.61852, 0.62344, 0.62905, 0.63527, 0.64201, 0.64916, 0.65661, 0.66417, 0.67159, 0.67862, 0.68516,\
        0.69144, 0.69758, 0.70353, 0.70894, 0.71326, 0.71700, 0.72056, 0.72388, 0.72681, 0.72924, 0.73123,\
        0.73295, 0.73460, 0.73631, 0.73812, 0.74003, 0.74204, 0.74417, 0.74648, 0.74911, 0.75221, 0.75593,\
        0.76025, 0.76499, 0.76979, 0.77415, 0.77760, 0.77976, 0.78026, 0.77926, 0.77710, 0.77374, 0.76911,\
        0.76347, 0.75733, 0.75128, 0.74575, 0.74099, 0.73699, 0.73354, 0.73033, 0.72704, 0.72344, 0.71942,\
        0.71505, 0.71050, 0.70598, 0.70183, 0.69814, 0.69420, 0.69073, 0.68776, 0.68512, 0.68274, 0.68061,\
        0.67867, 0.67685, 0.67500, 0.67293, 0.67045, 0.66736, 0.66348, 0.65864, 0.65276, 0.64577, 0.63767,\
        0.62850, 0.61828, 0.60685, 0.59418, 0.58093, 0.56777, 0.55513, 0.54331, 0.53258, 0.52323, 0.51560,\
        0.51003, 0.50643, 0.50396, 0.50215, 0.50106, 0.50063, 0.50129, 0.50202, 0.50259, 0.50289, 0.50284],\
        [0.49893, 0.50216, 0.50515, 0.50820, 0.51166, 0.51592, 0.52123, 0.52769, 0.53524, 0.54371, 0.55288,\
        0.56249, 0.57225, 0.58181, 0.59071, 0.59827, 0.60303, 0.60587, 0.60786, 0.60971, 0.61187, 0.61459,\
        0.61796, 0.62198, 0.62663, 0.63187, 0.63766, 0.64394, 0.65059, 0.65746, 0.66437, 0.67116, 0.67776,\
        0.68423, 0.69059, 0.69679, 0.70225, 0.70591, 0.70927, 0.71272, 0.71597, 0.71877, 0.72099, 0.72272,\
        0.72425, 0.72581, 0.72749, 0.72926, 0.73104, 0.73282, 0.73461, 0.73652, 0.73871, 0.74137, 0.74464,\
        0.74857, 0.75302, 0.75768, 0.76206, 0.76567, 0.76810, 0.76905, 0.76850, 0.76656, 0.76330, 0.75882,\
        0.75340, 0.74753, 0.74177, 0.73666, 0.73247, 0.72918, 0.72649, 0.72397, 0.72123, 0.71799, 0.71416,\
        0.70990, 0.70547, 0.70115, 0.69728, 0.69389, 0.69057, 0.68744, 0.68463, 0.68212, 0.67983, 0.67772,\
        0.67570, 0.67369, 0.67157, 0.66919, 0.66638, 0.66298, 0.65884, 0.65383, 0.64788, 0.64094, 0.63300,\
        0.62410, 0.61430, 0.60352, 0.59068, 0.57688, 0.56319, 0.55006, 0.53781, 0.52673, 0.51715, 0.50946,\
        0.50424, 0.50193, 0.50034, 0.49943, 0.49907, 0.49945, 0.50051, 0.50152, 0.50233, 0.50285, 0.50298],\
        [0.49977, 0.50348, 0.50700, 0.51060, 0.51456, 0.51917, 0.52463, 0.53102, 0.53832, 0.54642, 0.55514,\
        0.56426, 0.57350, 0.58253, 0.59089, 0.59783, 0.60238, 0.60491, 0.60695, 0.60895, 0.61110, 0.61352,\
        0.61632, 0.61956, 0.62331, 0.62760, 0.63246, 0.63786, 0.64374, 0.64998, 0.65643, 0.66298, 0.66954,\
        0.67606, 0.68248, 0.68861, 0.69380, 0.69748, 0.70096, 0.70461, 0.70802, 0.71092, 0.71298, 0.71445,\
        0.71582, 0.71741, 0.71921, 0.72104, 0.72278, 0.72435, 0.72580, 0.72723, 0.72885, 0.73089, 0.73357,\
        0.73702, 0.74118, 0.74574, 0.75019, 0.75401, 0.75673, 0.75805, 0.75786, 0.75617, 0.75306, 0.74869,\
        0.74336, 0.73756, 0.73192, 0.72712, 0.72357, 0.72116, 0.71941, 0.71778, 0.71576, 0.71297, 0.70934,\
        0.70516, 0.70089, 0.69685, 0.69321, 0.68994, 0.68690, 0.68405, 0.68142, 0.67902, 0.67678, 0.67465,\
        0.67254, 0.67037, 0.66802, 0.66536, 0.66226, 0.65858, 0.65420, 0.64902, 0.64295, 0.63596, 0.62803,\
        0.61913, 0.60922, 0.59825, 0.58572, 0.57200, 0.55820, 0.54491, 0.53251, 0.52135, 0.51175, 0.50419,\
        0.49935, 0.49784, 0.49726, 0.49713, 0.49758, 0.49866, 0.50007, 0.50134, 0.50237, 0.50306, 0.50333],\
        [0.50058, 0.50471, 0.50867, 0.51268, 0.51698, 0.52180, 0.52729, 0.53352, 0.54050, 0.54813, 0.55627,\
        0.56472, 0.57323, 0.58145, 0.58896, 0.59517, 0.59933, 0.60187, 0.60434, 0.60672, 0.60899, 0.61124,\
        0.61359, 0.61615, 0.61905, 0.62244, 0.62642, 0.63101, 0.63617, 0.64183, 0.64786, 0.65416, 0.66059,\
        0.66705, 0.67338, 0.67931, 0.68444, 0.68868, 0.69259, 0.69644, 0.70008, 0.70330, 0.70521, 0.70644,\
        0.70771, 0.70951, 0.71159, 0.71361, 0.71537, 0.71679, 0.71789, 0.71880, 0.71976, 0.72108, 0.72305,\
        0.72596, 0.72986, 0.73443, 0.73906, 0.74314, 0.74615, 0.74779, 0.74789, 0.74643, 0.74347, 0.73918,\
        0.73385, 0.72795, 0.72217, 0.71746, 0.71453, 0.71313, 0.71243, 0.71177, 0.71060, 0.70833, 0.70485,\
        0.70070, 0.69662, 0.69278, 0.68928, 0.68612, 0.68323, 0.68056, 0.67808, 0.67577, 0.67357, 0.67141,\
        0.66922, 0.66690, 0.66434, 0.66144, 0.65807, 0.65414, 0.64953, 0.64415, 0.63792, 0.63079, 0.62277,\
        0.61376, 0.60350, 0.59219, 0.57976, 0.56640, 0.55282, 0.53967, 0.52744, 0.51655, 0.50732, 0.50012,\
        0.49607, 0.49454, 0.49443, 0.49504, 0.49622, 0.49786, 0.49965, 0.50127, 0.50254, 0.50339, 0.50377],\
        [0.50122, 0.50571, 0.50999, 0.51430, 0.51882, 0.52373, 0.52915, 0.53514, 0.54171, 0.54879, 0.55626,\
        0.56393, 0.57158, 0.57889, 0.58548, 0.59091, 0.59484, 0.59777, 0.60056, 0.60329, 0.60567, 0.60780,\
        0.60979, 0.61176, 0.61388, 0.61644, 0.61962, 0.62348, 0.62801, 0.63315, 0.63880, 0.64485, 0.65113,\
        0.65749, 0.66372, 0.66961, 0.67492, 0.67967, 0.68404, 0.68820, 0.69211, 0.69540, 0.69748, 0.69883,\
        0.70015, 0.70240, 0.70482, 0.70709, 0.70894, 0.71027, 0.71108, 0.71149, 0.71179, 0.71236, 0.71354,\
        0.71580, 0.71951, 0.72425, 0.72916, 0.73350, 0.73677, 0.73864, 0.73897, 0.73770, 0.73489, 0.73070,\
        0.72537, 0.71930, 0.71313, 0.70813, 0.70579, 0.70543, 0.70561, 0.70577, 0.70550, 0.70380, 0.70039,\
        0.69633, 0.69235, 0.68864, 0.68527, 0.68222, 0.67946, 0.67693, 0.67457, 0.67234, 0.67017, 0.66800,\
        0.66573, 0.66328, 0.66054, 0.65741, 0.65380, 0.64963, 0.64479, 0.63921, 0.63279, 0.62545, 0.61721,\
        0.60827, 0.59794, 0.58624, 0.57367, 0.56049, 0.54721, 0.53439, 0.52255, 0.51218, 0.50380, 0.49752,\
        0.49364, 0.49211, 0.49217, 0.49322, 0.49493, 0.49704, 0.49926, 0.50120, 0.50268, 0.50369, 0.50419],\
        [0.50157, 0.50633, 0.51086, 0.51538, 0.52002, 0.52493, 0.53020, 0.53588, 0.54198, 0.54844, 0.55518,\
        0.56203, 0.56879, 0.57521, 0.58098, 0.58585, 0.58974, 0.59295, 0.59592, 0.59874, 0.60116, 0.60319,\
        0.60497, 0.60645, 0.60785, 0.60969, 0.61221, 0.61546, 0.61943, 0.62412, 0.62944, 0.63527, 0.64142,\
        0.64769, 0.65390, 0.65985, 0.66541, 0.67057, 0.67537, 0.67986, 0.68394, 0.68737, 0.68993, 0.69189,\
        0.69385, 0.69630, 0.69896, 0.70148, 0.70346, 0.70481, 0.70551, 0.70553, 0.70526, 0.70527, 0.70566,\
        0.70711, 0.71077, 0.71584, 0.72100, 0.72550, 0.72888, 0.73088, 0.73133, 0.73020, 0.72753, 0.72347,\
        0.71827, 0.71224, 0.70586, 0.70026, 0.69826, 0.69837, 0.69886, 0.69936, 0.69970, 0.69850, 0.69522,\
        0.69141, 0.68764, 0.68413, 0.68094, 0.67807, 0.67546, 0.67307, 0.67084, 0.66870, 0.66658, 0.66441,\
        0.66208, 0.65952, 0.65662, 0.65329, 0.64945, 0.64503, 0.63997, 0.63421, 0.62764, 0.62009, 0.61136,\
        0.60194, 0.59193, 0.58013, 0.56751, 0.55451, 0.54156, 0.52916, 0.51781, 0.50799, 0.50016, 0.49480,\
        0.49146, 0.49009, 0.49029, 0.49159, 0.49362, 0.49609, 0.49868, 0.50094, 0.50262, 0.50382, 0.50449],\
        [0.50143, 0.50641, 0.51119, 0.51588, 0.52060, 0.52544, 0.53049, 0.53579, 0.54136, 0.54718, 0.55315,\
        0.55918, 0.56508, 0.57068, 0.57576, 0.58019, 0.58396, 0.58722, 0.59020, 0.59294, 0.59533, 0.59734,\
        0.59902, 0.60022, 0.60105, 0.60241, 0.60444, 0.60719, 0.61071, 0.61498, 0.62003, 0.62570, 0.63175,\
        0.63797, 0.64418, 0.65024, 0.65605, 0.66156, 0.66674, 0.67157, 0.67594, 0.67972, 0.68286, 0.68556,\
        0.68817, 0.69095, 0.69382, 0.69652, 0.69871, 0.70028, 0.70115, 0.70112, 0.70048, 0.70046, 0.70042,\
        0.70082, 0.70472, 0.70993, 0.71499, 0.71934, 0.72262, 0.72457, 0.72505, 0.72397, 0.72140, 0.71749,\
        0.71252, 0.70681, 0.70090, 0.69558, 0.69271, 0.69195, 0.69203, 0.69233, 0.69238, 0.69126, 0.68870,\
        0.68549, 0.68219, 0.67905, 0.67616, 0.67354, 0.67115, 0.66894, 0.66686, 0.66483, 0.66278, 0.66063,\
        0.65828, 0.65564, 0.65260, 0.64909, 0.64501, 0.64034, 0.63504, 0.62909, 0.62245, 0.61495, 0.60621,\
        0.59601, 0.58546, 0.57379, 0.56131, 0.54854, 0.53595, 0.52402, 0.51323, 0.50414, 0.49701, 0.49215,\
        0.48929, 0.48818, 0.48852, 0.48998, 0.49220, 0.49488, 0.49769, 0.50020, 0.50217, 0.50367, 0.50457],\
        [0.50052, 0.50592, 0.51100, 0.51586, 0.52060, 0.52532, 0.53008, 0.53496, 0.53997, 0.54511, 0.55032,\
        0.55554, 0.56063, 0.56548, 0.56994, 0.57395, 0.57750, 0.58067, 0.58354, 0.58615, 0.58843, 0.59036,\
        0.59190, 0.59300, 0.59383, 0.59498, 0.59668, 0.59906, 0.60220, 0.60605, 0.61088, 0.61647, 0.62241,\
        0.62856, 0.63478, 0.64096, 0.64699, 0.65282, 0.65837, 0.66356, 0.66832, 0.67256, 0.67629, 0.67966,\
        0.68286, 0.68603, 0.68914, 0.69203, 0.69449, 0.69639, 0.69761, 0.69801, 0.69788, 0.69807, 0.69848,\
        0.69968, 0.70263, 0.70681, 0.71114, 0.71496, 0.71787, 0.71961, 0.71999, 0.71890, 0.71632, 0.71246,\
        0.70768, 0.70227, 0.69650, 0.69135, 0.68778, 0.68590, 0.68509, 0.68473, 0.68426, 0.68312, 0.68115,\
        0.67864, 0.67595, 0.67332, 0.67086, 0.66858, 0.66648, 0.66450, 0.66260, 0.66070, 0.65875, 0.65666,\
        0.65432, 0.65163, 0.64850, 0.64482, 0.64052, 0.63556, 0.62995, 0.62376, 0.61701, 0.60959, 0.60129,\
        0.59092, 0.57950, 0.56753, 0.55514, 0.54264, 0.53047, 0.51905, 0.50879, 0.50031, 0.49403, 0.48966,\
        0.48714, 0.48623, 0.48670, 0.48825, 0.49057, 0.49336, 0.49627, 0.49897, 0.50131, 0.50321, 0.50441],\
        [0.49945, 0.50514, 0.51045, 0.51541, 0.52011, 0.52465, 0.52908, 0.53349, 0.53791, 0.54235, 0.54682,\
        0.55125, 0.55558, 0.55973, 0.56362, 0.56721, 0.57047, 0.57345, 0.57617, 0.57864, 0.58082, 0.58267,\
        0.58417, 0.58534, 0.58637, 0.58757, 0.58920, 0.59141, 0.59426, 0.59778, 0.60249, 0.60793, 0.61365,\
        0.61969, 0.62591, 0.63216, 0.63838, 0.64450, 0.65041, 0.65600, 0.66118, 0.66590, 0.67019, 0.67412,\
        0.67784, 0.68140, 0.68480, 0.68796, 0.69073, 0.69301, 0.69472, 0.69583, 0.69658, 0.69739, 0.69853,\
        0.70029, 0.70257, 0.70559, 0.70889, 0.71192, 0.71428, 0.71568, 0.71588, 0.71472, 0.71205, 0.70800,\
        0.70305, 0.69772, 0.69191, 0.68657, 0.68241, 0.67959, 0.67785, 0.67672, 0.67576, 0.67458, 0.67300,\
        0.67111, 0.66907, 0.66704, 0.66508, 0.66322, 0.66144, 0.65973, 0.65804, 0.65632, 0.65450, 0.65249,\
        0.65020, 0.64752, 0.64433, 0.64053, 0.63602, 0.63075, 0.62475, 0.61816, 0.61113, 0.60350, 0.59478,\
        0.58464, 0.57328, 0.56129, 0.54903, 0.53685, 0.52515, 0.51435, 0.50480, 0.49691, 0.49113, 0.48719,\
        0.48492, 0.48416, 0.48469, 0.48628, 0.48866, 0.49152, 0.49457, 0.49752, 0.50021, 0.50250, 0.50398],\
        [0.49837, 0.50424, 0.50965, 0.51464, 0.51924, 0.52354, 0.52759, 0.53149, 0.53529, 0.53904, 0.54276,\
        0.54644, 0.55005, 0.55355, 0.55689, 0.56005, 0.56301, 0.56577, 0.56833, 0.57068, 0.57279, 0.57463,\
        0.57621, 0.57757, 0.57885, 0.58027, 0.58203, 0.58427, 0.58710, 0.59066, 0.59514, 0.60021, 0.60560,\
        0.61157, 0.61774, 0.62397, 0.63034, 0.63675, 0.64302, 0.64899, 0.65458, 0.65974, 0.66450, 0.66892,\
        0.67307, 0.67700, 0.68072, 0.68417, 0.68730, 0.69004, 0.69234, 0.69424, 0.69586, 0.69738, 0.69893,\
        0.70073, 0.70277, 0.70502, 0.70738, 0.70958, 0.71130, 0.71227, 0.71226, 0.71108, 0.70837, 0.70389,\
        0.69850, 0.69266, 0.68665, 0.68099, 0.67626, 0.67266, 0.67011, 0.66833, 0.66697, 0.66575, 0.66449,\
        0.66315, 0.66175, 0.66032, 0.65891, 0.65750, 0.65609, 0.65467, 0.65321, 0.65167, 0.65000, 0.64811,\
        0.64592, 0.64329, 0.64011, 0.63626, 0.63160, 0.62605, 0.61959, 0.61241, 0.60493, 0.59704, 0.58809,\
        0.57796, 0.56680, 0.55501, 0.54300, 0.53115, 0.51995, 0.50984, 0.50106, 0.49385, 0.48841, 0.48469,\
        0.48256, 0.48186, 0.48241, 0.48399, 0.48638, 0.48932, 0.49253, 0.49578, 0.49884, 0.50146, 0.50323],\
        [0.49714, 0.50317, 0.50865, 0.51363, 0.51810, 0.52210, 0.52573, 0.52908, 0.53224, 0.53528, 0.53827,\
        0.54122, 0.54414, 0.54702, 0.54985, 0.55259, 0.55524, 0.55779, 0.56021, 0.56248, 0.56458, 0.56650,\
        0.56823, 0.56984, 0.57144, 0.57317, 0.57519, 0.57762, 0.58060, 0.58422, 0.58852, 0.59332, 0.59848,\
        0.60457, 0.61040, 0.61642, 0.62296, 0.62969, 0.63630, 0.64261, 0.64854, 0.65407, 0.65922, 0.66401,\
        0.66851, 0.67276, 0.67678, 0.68054, 0.68403, 0.68721, 0.69011, 0.69275, 0.69521, 0.69749, 0.69942,\
        0.70116, 0.70277, 0.70429, 0.70580, 0.70721, 0.70830, 0.70878, 0.70846, 0.70734, 0.70509, 0.70024,\
        0.69398, 0.68731, 0.68065, 0.67448, 0.66918, 0.66495, 0.66178, 0.65950, 0.65788, 0.65669, 0.65576,\
        0.65494, 0.65415, 0.65335, 0.65248, 0.65154, 0.65050, 0.64936, 0.64812, 0.64677, 0.64525, 0.64351,\
        0.64145, 0.63893, 0.63583, 0.63201, 0.62733, 0.62165, 0.61484, 0.60698, 0.59882, 0.59068, 0.58156,\
        0.57138, 0.56036, 0.54883, 0.53712, 0.52561, 0.51477, 0.50537, 0.49738, 0.49070, 0.48559, 0.48204,\
        0.47997, 0.47925, 0.47974, 0.48126, 0.48362, 0.48659, 0.48996, 0.49348, 0.49689, 0.49989, 0.50202],\
        [0.49559, 0.50180, 0.50750, 0.51248, 0.51676, 0.52043, 0.52359, 0.52636, 0.52885, 0.53119, 0.53345,\
        0.53570, 0.53796, 0.54026, 0.54260, 0.54495, 0.54732, 0.54967, 0.55198, 0.55423, 0.55638, 0.55843,\
        0.56039, 0.56230, 0.56425, 0.56634, 0.56869, 0.57142, 0.57462, 0.57835, 0.58263, 0.58742, 0.59275,\
        0.59852, 0.60359, 0.60954, 0.61643, 0.62348, 0.63036, 0.63692, 0.64310, 0.64889, 0.65430, 0.65936,\
        0.66413, 0.66862, 0.67287, 0.67689, 0.68068, 0.68426, 0.68763, 0.69086, 0.69399, 0.69702, 0.69973,\
        0.70113, 0.70197, 0.70275, 0.70350, 0.70416, 0.70467, 0.70467, 0.70374, 0.70234, 0.70066, 0.69616,\
        0.68867, 0.68100, 0.67363, 0.66691, 0.66113, 0.65643, 0.65282, 0.65023, 0.64849, 0.64744, 0.64688,\
        0.64662, 0.64647, 0.64628, 0.64596, 0.64545, 0.64474, 0.64385, 0.64280, 0.64160, 0.64024, 0.63866,\
        0.63675, 0.63438, 0.63141, 0.62771, 0.62314, 0.61761, 0.61091, 0.60267, 0.59405, 0.58514, 0.57548,\
        0.56507, 0.55411, 0.54284, 0.53152, 0.52043, 0.50985, 0.50043, 0.49344, 0.48738, 0.48257, 0.47913,\
        0.47704, 0.47623, 0.47660, 0.47799, 0.48024, 0.48318, 0.48659, 0.49027, 0.49397, 0.49743, 0.49963],\
        [0.49407, 0.50060, 0.50638, 0.51128, 0.51533, 0.51862, 0.52127, 0.52342, 0.52524, 0.52686, 0.52841,\
        0.52998, 0.53162, 0.53337, 0.53526, 0.53727, 0.53938, 0.54157, 0.54382, 0.54608, 0.54833, 0.55057,\
        0.55279, 0.55503, 0.55735, 0.55984, 0.56257, 0.56563, 0.56909, 0.57301, 0.57739, 0.58220, 0.58737,\
        0.59257, 0.59757, 0.60391, 0.61110, 0.61830, 0.62527, 0.63192, 0.63821, 0.64414, 0.64970, 0.65492,\
        0.65984, 0.66449, 0.66889, 0.67308, 0.67707, 0.68089, 0.68457, 0.68813, 0.69157, 0.69483, 0.69753,\
        0.69893, 0.69949, 0.69978, 0.69992, 0.69992, 0.69986, 0.69942, 0.69774, 0.69566, 0.69299, 0.68819,\
        0.68099, 0.67303, 0.66529, 0.65822, 0.65211, 0.64712, 0.64328, 0.64055, 0.63885, 0.63806, 0.63798,\
        0.63834, 0.63887, 0.63931, 0.63950, 0.63937, 0.63892, 0.63820, 0.63727, 0.63618, 0.63495, 0.63351,\
        0.63177, 0.62956, 0.62675, 0.62317, 0.61873, 0.61344, 0.60749, 0.60003, 0.59028, 0.58010, 0.56965,\
        0.55892, 0.54800, 0.53703, 0.52621, 0.51579, 0.50605, 0.49721, 0.48993, 0.48406, 0.47938, 0.47593,\
        0.47374, 0.47276, 0.47291, 0.47409, 0.47615, 0.47894, 0.48225, 0.48587, 0.48951, 0.49272, 0.49462],\
        [0.49247, 0.49939, 0.50524, 0.51004, 0.51385, 0.51673, 0.51884, 0.52035, 0.52148, 0.52240, 0.52326,\
        0.52417, 0.52523, 0.52648, 0.52796, 0.52966, 0.53156, 0.53364, 0.53585, 0.53817, 0.54055, 0.54300,\
        0.54551, 0.54811, 0.55082, 0.55371, 0.55683, 0.56024, 0.56401, 0.56816, 0.57269, 0.57757, 0.58272,\
        0.58802, 0.59363, 0.60008, 0.60709, 0.61412, 0.62096, 0.62755, 0.63383, 0.63977, 0.64537, 0.65063,\
        0.65559, 0.66028, 0.66473, 0.66898, 0.67304, 0.67694, 0.68070, 0.68431, 0.68771, 0.69075, 0.69311,\
        0.69447, 0.69500, 0.69504, 0.69478, 0.69429, 0.69359, 0.69238, 0.69029, 0.68744, 0.68364, 0.67827,\
        0.67126, 0.66342, 0.65562, 0.64844, 0.64224, 0.63718, 0.63330, 0.63060, 0.62907, 0.62866, 0.62918,\
        0.63029, 0.63156, 0.63263, 0.63328, 0.63345, 0.63315, 0.63249, 0.63157, 0.63050, 0.62934, 0.62804,\
        0.62645, 0.62439, 0.62169, 0.61823, 0.61379, 0.60825, 0.60214, 0.59523, 0.58524, 0.57441, 0.56348,\
        0.55259, 0.54183, 0.53126, 0.52100, 0.51125, 0.50227, 0.49406, 0.48677, 0.48077, 0.47599, 0.47241,\
        0.47001, 0.46877, 0.46864, 0.46952, 0.47130, 0.47381, 0.47688, 0.48024, 0.48357, 0.48639, 0.48814],\
        [0.49051, 0.49780, 0.50389, 0.50872, 0.51233, 0.51481, 0.51636, 0.51723, 0.51766, 0.51789, 0.51809,\
        0.51840, 0.51892, 0.51973, 0.52084, 0.52228, 0.52401, 0.52601, 0.52823, 0.53062, 0.53316, 0.53583,\
        0.53865, 0.54159, 0.54470, 0.54798, 0.55150, 0.55528, 0.55936, 0.56377, 0.56851, 0.57356, 0.57889,\
        0.58451, 0.59055, 0.59713, 0.60395, 0.61067, 0.61725, 0.62366, 0.62983, 0.63569, 0.64121, 0.64641,\
        0.65130, 0.65592, 0.66030, 0.66448, 0.66848, 0.67231, 0.67596, 0.67941, 0.68255, 0.68524, 0.68725,\
        0.68843, 0.68885, 0.68869, 0.68811, 0.68719, 0.68590, 0.68406, 0.68144, 0.67793, 0.67335, 0.66748,\
        0.66041, 0.65266, 0.64493, 0.63782, 0.63175, 0.62685, 0.62314, 0.62061, 0.61934, 0.61941, 0.62070,\
        0.62271, 0.62478, 0.62645, 0.62748, 0.62781, 0.62753, 0.62677, 0.62572, 0.62454, 0.62338, 0.62221,\
        0.62078, 0.61879, 0.61611, 0.61269, 0.60825, 0.60238, 0.59524, 0.58752, 0.57802, 0.56740, 0.55652,\
        0.54580, 0.53540, 0.52538, 0.51574, 0.50655, 0.49804, 0.49026, 0.48326, 0.47724, 0.47232, 0.46852,\
        0.46584, 0.46427, 0.46377, 0.46427, 0.46568, 0.46785, 0.47057, 0.47359, 0.47657, 0.47910, 0.48078],\
        [0.48813, 0.49590, 0.50235, 0.50733, 0.51081, 0.51292, 0.51390, 0.51410, 0.51385, 0.51344, 0.51305,\
        0.51282, 0.51286, 0.51327, 0.51409, 0.51529, 0.51688, 0.51884, 0.52108, 0.52355, 0.52624, 0.52915,\
        0.53226, 0.53555, 0.53903, 0.54270, 0.54659, 0.55072, 0.55512, 0.55981, 0.56479, 0.57006, 0.57562,\
        0.58151, 0.58777, 0.59440, 0.60111, 0.60751, 0.61384, 0.62008, 0.62609, 0.63180, 0.63716, 0.64218,\
        0.64690, 0.65134, 0.65554, 0.65953, 0.66334, 0.66696, 0.67037, 0.67353, 0.67632, 0.67861, 0.68026,\
        0.68118, 0.68138, 0.68098, 0.68010, 0.67879, 0.67704, 0.67472, 0.67164, 0.66763, 0.66254, 0.65631,\
        0.64909, 0.64133, 0.63365, 0.62671, 0.62094, 0.61648, 0.61317, 0.61092, 0.60998, 0.61060, 0.61283,\
        0.61590, 0.61881, 0.62098, 0.62222, 0.62255, 0.62211, 0.62111, 0.61976, 0.61832, 0.61705, 0.61601,\
        0.61479, 0.61277, 0.60984, 0.60631, 0.60176, 0.59556, 0.58787, 0.57928, 0.56969, 0.55931, 0.54873,\
        0.53841, 0.52861, 0.51935, 0.51049, 0.50185, 0.49377, 0.48628, 0.47943, 0.47339, 0.46831, 0.46424,\
        0.46122, 0.45926, 0.45833, 0.45840, 0.45938, 0.46114, 0.46349, 0.46616, 0.46884, 0.47118, 0.47285],\
        [0.48529, 0.49369, 0.50064, 0.50594, 0.50937, 0.51108, 0.51148, 0.51102, 0.51013, 0.50915, 0.50828,\
        0.50761, 0.50721, 0.50732, 0.50791, 0.50889, 0.51037, 0.51232, 0.51458, 0.51709, 0.51990, 0.52302,\
        0.52641, 0.53004, 0.53387, 0.53789, 0.54213, 0.54658, 0.55128, 0.55624, 0.56145, 0.56694, 0.57269,\
        0.57873, 0.58505, 0.59155, 0.59802, 0.60429, 0.61056, 0.61668, 0.62253, 0.62802, 0.63313, 0.63788,\
        0.64231, 0.64647, 0.65039, 0.65409, 0.65761, 0.66092, 0.66400, 0.66679, 0.66919, 0.67109, 0.67236,\
        0.67294, 0.67286, 0.67217, 0.67097, 0.66933, 0.66724, 0.66459, 0.66122, 0.65693, 0.65156, 0.64509,\
        0.63774, 0.62992, 0.62227, 0.61550, 0.61021, 0.60645, 0.60381, 0.60202, 0.60154, 0.60269, 0.60607,\
        0.61030, 0.61386, 0.61632, 0.61757, 0.61770, 0.61694, 0.61557, 0.61380, 0.61189, 0.61032, 0.60944,\
        0.60860, 0.60652, 0.60284, 0.59888, 0.59391, 0.58735, 0.57943, 0.57051, 0.56080, 0.55058, 0.54031,\
        0.53048, 0.52142, 0.51310, 0.50540, 0.49746, 0.48960, 0.48217, 0.47533, 0.46921, 0.46394, 0.45958,\
        0.45619, 0.45379, 0.45239, 0.45198, 0.45250, 0.45383, 0.45581, 0.45818, 0.46064, 0.46287, 0.46458],\
        [0.48188, 0.49117, 0.49892, 0.50466, 0.50801, 0.50930, 0.50910, 0.50802, 0.50657, 0.50513, 0.50397,\
        0.50303, 0.50225, 0.50215, 0.50262, 0.50330, 0.50467, 0.50667, 0.50887, 0.51132, 0.51420, 0.51752,\
        0.52118, 0.52510, 0.52924, 0.53357, 0.53811, 0.54285, 0.54781, 0.55299, 0.55841, 0.56407, 0.56996,\
        0.57608, 0.58236, 0.58871, 0.59497, 0.60117, 0.60743, 0.61343, 0.61907, 0.62428, 0.62905, 0.63343,\
        0.63748, 0.64126, 0.64480, 0.64814, 0.65128, 0.65421, 0.65691, 0.65930, 0.66131, 0.66281, 0.66370,\
        0.66392, 0.66349, 0.66247, 0.66094, 0.65900, 0.65667, 0.65387, 0.65044, 0.64612, 0.64070, 0.63416,\
        0.62675, 0.61892, 0.61130, 0.60474, 0.60001, 0.59709, 0.59526, 0.59436, 0.59478, 0.59689, 0.60133,\
        0.60611, 0.60988, 0.61237, 0.61344, 0.61317, 0.61194, 0.61016, 0.60801, 0.60550, 0.60335, 0.60252,\
        0.60214, 0.60009, 0.59577, 0.59090, 0.58528, 0.57841, 0.57036, 0.56137, 0.55168, 0.54159, 0.53154,\
        0.52215, 0.51393, 0.50655, 0.49982, 0.49285, 0.48514, 0.47778, 0.47093, 0.46473, 0.45926, 0.45460,\
        0.45081, 0.44795, 0.44604, 0.44512, 0.44515, 0.44607, 0.44771, 0.44985, 0.45217, 0.45437, 0.45616],\
        [0.47759, 0.48797, 0.49691, 0.50321, 0.50653, 0.50749, 0.50673, 0.50513, 0.50330, 0.50154, 0.50030,\
        0.49934, 0.49842, 0.49814, 0.49842, 0.49885, 0.50018, 0.50211, 0.50407, 0.50630, 0.50923, 0.51275,\
        0.51664, 0.52080, 0.52518, 0.52976, 0.53453, 0.53950, 0.54467, 0.55004, 0.55561, 0.56140, 0.56737,\
        0.57350, 0.57975, 0.58602, 0.59225, 0.59845, 0.60450, 0.61028, 0.61565, 0.62050, 0.62483, 0.62874,\
        0.63232, 0.63565, 0.63876, 0.64166, 0.64437, 0.64687, 0.64915, 0.65115, 0.65277, 0.65390, 0.65442,\
        0.65427, 0.65346, 0.65206, 0.65018, 0.64796, 0.64547, 0.64270, 0.63946, 0.63540, 0.63017, 0.62371,\
        0.61637, 0.60871, 0.60135, 0.59526, 0.59109, 0.58865, 0.58752, 0.58764, 0.58922, 0.59258, 0.59754,\
        0.60255, 0.60620, 0.60871, 0.60958, 0.60875, 0.60691, 0.60477, 0.60250, 0.59966, 0.59703, 0.59575,\
        0.59477, 0.59226, 0.58797, 0.58259, 0.57639, 0.56922, 0.56110, 0.55218, 0.54266, 0.53279, 0.52291,\
        0.51367, 0.50623, 0.50053, 0.49391, 0.48721, 0.48009, 0.47298, 0.46622, 0.45997, 0.45432, 0.44938,\
        0.44520, 0.44186, 0.43942, 0.43796, 0.43750, 0.43801, 0.43937, 0.44135, 0.44363, 0.44587, 0.44778],\
        [0.47205, 0.48344, 0.49337, 0.50066, 0.50467, 0.50560, 0.50436, 0.50238, 0.50052, 0.49882, 0.49751,\
        0.49651, 0.49569, 0.49528, 0.49533, 0.49580, 0.49692, 0.49853, 0.50016, 0.50205, 0.50514, 0.50882,\
        0.51286, 0.51716, 0.52170, 0.52644, 0.53139, 0.53652, 0.54183, 0.54732, 0.55300, 0.55884, 0.56485,\
        0.57097, 0.57718, 0.58341, 0.58959, 0.59565, 0.60150, 0.60714, 0.61222, 0.61660, 0.62038, 0.62371,\
        0.62676, 0.62959, 0.63223, 0.63466, 0.63688, 0.63892, 0.64077, 0.64239, 0.64367, 0.64447, 0.64466,\
        0.64414, 0.64293, 0.64112, 0.63886, 0.63633, 0.63371, 0.63110, 0.62832, 0.62487, 0.62010, 0.61380,\
        0.60660, 0.59934, 0.59255, 0.58700, 0.58315, 0.58100, 0.58037, 0.58121, 0.58361, 0.58758, 0.59275,\
        0.59784, 0.60168, 0.60478, 0.60564, 0.60418, 0.60163, 0.59922, 0.59677, 0.59399, 0.59125, 0.58912,\
        0.58700, 0.58389, 0.57945, 0.57387, 0.56737, 0.56003, 0.55194, 0.54323, 0.53408, 0.52470, 0.51526,\
        0.50597, 0.49877, 0.49359, 0.48786, 0.48144, 0.47470, 0.46790, 0.46128, 0.45502, 0.44925, 0.44405,\
        0.43951, 0.43569, 0.43271, 0.43067, 0.42970, 0.42983, 0.43098, 0.43288, 0.43520, 0.43755, 0.43960],\
        [0.46502, 0.47738, 0.48846, 0.49723, 0.50256, 0.50377, 0.50207, 0.49990, 0.49829, 0.49684, 0.49556,\
        0.49452, 0.49372, 0.49327, 0.49321, 0.49361, 0.49449, 0.49574, 0.49718, 0.49915, 0.50223, 0.50583,\
        0.50986, 0.51419, 0.51879, 0.52362, 0.52865, 0.53387, 0.53926, 0.54481, 0.55051, 0.55636, 0.56234,\
        0.56841, 0.57455, 0.58070, 0.58681, 0.59278, 0.59855, 0.60403, 0.60864, 0.61243, 0.61555, 0.61822,\
        0.62069, 0.62304, 0.62521, 0.62713, 0.62884, 0.63039, 0.63180, 0.63306, 0.63405, 0.63460, 0.63450,\
        0.63364, 0.63202, 0.62978, 0.62713, 0.62426, 0.62147, 0.61905, 0.61693, 0.61444, 0.61046, 0.60431,\
        0.59728, 0.59043, 0.58421, 0.57912, 0.57555, 0.57362, 0.57328, 0.57451, 0.57726, 0.58138, 0.58638,\
        0.59145, 0.59607, 0.59958, 0.60071, 0.59926, 0.59619, 0.59350, 0.59079, 0.58794, 0.58506, 0.58227,\
        0.57927, 0.57554, 0.57079, 0.56503, 0.55838, 0.55100, 0.54302, 0.53465, 0.52607, 0.51749, 0.50927,\
        0.50130, 0.49391, 0.48792, 0.48204, 0.47580, 0.46929, 0.46271, 0.45624, 0.45002, 0.44417, 0.43878,\
        0.43390, 0.42964, 0.42611, 0.42348, 0.42196, 0.42172, 0.42273, 0.42466, 0.42709, 0.42960, 0.43181],\
        [0.45644, 0.46971, 0.48211, 0.49266, 0.50003, 0.50205, 0.50017, 0.49826, 0.49674, 0.49535, 0.49411,\
        0.49308, 0.49230, 0.49184, 0.49175, 0.49206, 0.49279, 0.49388, 0.49535, 0.49744, 0.50024, 0.50367,\
        0.50757, 0.51183, 0.51640, 0.52123, 0.52628, 0.53152, 0.53691, 0.54245, 0.54811, 0.55390, 0.55978,\
        0.56574, 0.57175, 0.57777, 0.58375, 0.58964, 0.59539, 0.60058, 0.60451, 0.60777, 0.61020, 0.61212,\
        0.61404, 0.61598, 0.61771, 0.61911, 0.62027, 0.62128, 0.62225, 0.62319, 0.62395, 0.62433, 0.62404,\
        0.62286, 0.62082, 0.61817, 0.61518, 0.61200, 0.60892, 0.60663, 0.60511, 0.60360, 0.60075, 0.59481,\
        0.58808, 0.58161, 0.57583, 0.57115, 0.56784, 0.56608, 0.56588, 0.56721, 0.56998, 0.57396, 0.57873,\
        0.58370, 0.58831, 0.59171, 0.59320, 0.59241, 0.58998, 0.58725, 0.58444, 0.58151, 0.57843, 0.57520,\
        0.57160, 0.56736, 0.56228, 0.55634, 0.54960, 0.54223, 0.53442, 0.52644, 0.51854, 0.51080, 0.50350,\
        0.49749, 0.49006, 0.48317, 0.47677, 0.47042, 0.46401, 0.45758, 0.45124, 0.44509, 0.43922, 0.43370,\
        0.42859, 0.42396, 0.41992, 0.41668, 0.41455, 0.41394, 0.41490, 0.41696, 0.41956, 0.42222, 0.42457],\
        [0.44636, 0.46038, 0.47390, 0.48592, 0.49509, 0.49966, 0.49873, 0.49689, 0.49532, 0.49396, 0.49280,\
        0.49186, 0.49117, 0.49078, 0.49072, 0.49102, 0.49169, 0.49274, 0.49423, 0.49628, 0.49895, 0.50218,\
        0.50586, 0.50997, 0.51446, 0.51924, 0.52424, 0.52943, 0.53475, 0.54020, 0.54575, 0.55140, 0.55712,\
        0.56289, 0.56868, 0.57447, 0.58020, 0.58580, 0.59111, 0.59572, 0.59948, 0.60244, 0.60419, 0.60527,\
        0.60676, 0.60844, 0.60977, 0.61061, 0.61119, 0.61165, 0.61217, 0.61280, 0.61337, 0.61366, 0.61328,\
        0.61182, 0.60937, 0.60634, 0.60314, 0.59987, 0.59665, 0.59444, 0.59309, 0.59174, 0.58907, 0.58422,\
        0.57830, 0.57242, 0.56714, 0.56283, 0.55977, 0.55812, 0.55794, 0.55920, 0.56180, 0.56551, 0.56994,\
        0.57461, 0.57893, 0.58222, 0.58394, 0.58387, 0.58238, 0.58015, 0.57755, 0.57465, 0.57148, 0.56797,\
        0.56400, 0.55940, 0.55407, 0.54797, 0.54119, 0.53386, 0.52620, 0.51853, 0.51133, 0.50470, 0.49770,\
        0.49151, 0.48496, 0.47822, 0.47166, 0.46525, 0.45889, 0.45258, 0.44637, 0.44033, 0.43451, 0.42897,\
        0.42375, 0.41889, 0.41447, 0.41067, 0.40786, 0.40682, 0.40788, 0.41013, 0.41283, 0.41560, 0.41801],\
        [0.43504, 0.44962, 0.46392, 0.47678, 0.48695, 0.49314, 0.49476, 0.49414, 0.49312, 0.49212, 0.49125,\
        0.49055, 0.49007, 0.48984, 0.48991, 0.49029, 0.49100, 0.49207, 0.49355, 0.49552, 0.49805, 0.50106,\
        0.50451, 0.50849, 0.51288, 0.51757, 0.52248, 0.52755, 0.53274, 0.53803, 0.54339, 0.54882, 0.55429,\
        0.55979, 0.56528, 0.57072, 0.57606, 0.58118, 0.58593, 0.59007, 0.59347, 0.59603, 0.59732, 0.59787,\
        0.59904, 0.60045, 0.60134, 0.60160, 0.60166, 0.60160, 0.60166, 0.60200, 0.60231, 0.60249, 0.60205,\
        0.60034, 0.59756, 0.59432, 0.59104, 0.58783, 0.58492, 0.58272, 0.58113, 0.57945, 0.57681, 0.57280,\
        0.56786, 0.56274, 0.55803, 0.55411, 0.55128, 0.54970, 0.54946, 0.55054, 0.55287, 0.55622, 0.56029,\
        0.56462, 0.56869, 0.57195, 0.57396, 0.57456, 0.57391, 0.57234, 0.57015, 0.56746, 0.56432, 0.56070,\
        0.55653, 0.55173, 0.54624, 0.54005, 0.53326, 0.52600, 0.51848, 0.51094, 0.50397, 0.49843, 0.49227,\
        0.48581, 0.47941, 0.47292, 0.46648, 0.46014, 0.45388, 0.44771, 0.44167, 0.43578, 0.43011, 0.42467,\
        0.41950, 0.41462, 0.41007, 0.40598, 0.40263, 0.40099, 0.40234, 0.40455, 0.40708, 0.40987, 0.41225],\
        [0.42309, 0.43812, 0.45295, 0.46626, 0.47699, 0.48429, 0.48806, 0.48941, 0.48963, 0.48944, 0.48914,\
        0.48889, 0.48878, 0.48885, 0.48916, 0.48972, 0.49055, 0.49169, 0.49316, 0.49502, 0.49731, 0.50007,\
        0.50342, 0.50734, 0.51164, 0.51620, 0.52096, 0.52585, 0.53083, 0.53589, 0.54099, 0.54613, 0.55128,\
        0.55642, 0.56152, 0.56653, 0.57137, 0.57594, 0.58011, 0.58370, 0.58659, 0.58862, 0.58972, 0.59030,\
        0.59101, 0.59180, 0.59218, 0.59207, 0.59172, 0.59131, 0.59102, 0.59095, 0.59091, 0.59070, 0.58991,\
        0.58804, 0.58527, 0.58208, 0.57886, 0.57582, 0.57315, 0.57101, 0.56922, 0.56733, 0.56479, 0.56133,\
        0.55718, 0.55277, 0.54861, 0.54508, 0.54245, 0.54091, 0.54055, 0.54140, 0.54340, 0.54639, 0.55011,\
        0.55417, 0.55810, 0.56143, 0.56380, 0.56500, 0.56507, 0.56416, 0.56246, 0.56009, 0.55710, 0.55350,\
        0.54928, 0.54440, 0.53884, 0.53265, 0.52589, 0.51874, 0.51151, 0.50418, 0.49733, 0.49164, 0.48599,\
        0.47993, 0.47373, 0.46748, 0.46124, 0.45506, 0.44896, 0.44297, 0.43712, 0.43145, 0.42599, 0.42079,\
        0.41585, 0.41118, 0.40680, 0.40286, 0.39974, 0.39802, 0.39871, 0.40029, 0.40232, 0.40511, 0.40729],\
        [0.41159, 0.42712, 0.44211, 0.45551, 0.46660, 0.47484, 0.48021, 0.48330, 0.48496, 0.48584, 0.48635,\
        0.48674, 0.48714, 0.48765, 0.48831, 0.48916, 0.49021, 0.49149, 0.49300, 0.49478, 0.49690, 0.49947,\
        0.50273, 0.50653, 0.51070, 0.51509, 0.51963, 0.52428, 0.52899, 0.53374, 0.53851, 0.54330, 0.54806,\
        0.55278, 0.55742, 0.56193, 0.56622, 0.57022, 0.57379, 0.57681, 0.57917, 0.58078, 0.58169, 0.58215,\
        0.58245, 0.58264, 0.58254, 0.58210, 0.58146, 0.58077, 0.58018, 0.57970, 0.57922, 0.57853, 0.57732,\
        0.57533, 0.57267, 0.56964, 0.56658, 0.56375, 0.56131, 0.55929, 0.55751, 0.55562, 0.55326, 0.55023,\
        0.54665, 0.54282, 0.53913, 0.53592, 0.53348, 0.53195, 0.53145, 0.53203, 0.53368, 0.53633, 0.53976,\
        0.54365, 0.54757, 0.55108, 0.55382, 0.55557, 0.55627, 0.55596, 0.55475, 0.55274, 0.54997, 0.54649,\
        0.54231, 0.53744, 0.53192, 0.52579, 0.51914, 0.51207, 0.50511, 0.49861, 0.49178, 0.48570, 0.47991,\
        0.47402, 0.46803, 0.46199, 0.45597, 0.44999, 0.44411, 0.43833, 0.43271, 0.42728, 0.42210, 0.41721,\
        0.41264, 0.40839, 0.40442, 0.40082, 0.39803, 0.39631, 0.39615, 0.39701, 0.39879, 0.40122, 0.40288],\
        [0.40304, 0.41803, 0.43220, 0.44529, 0.45656, 0.46549, 0.47202, 0.47649, 0.47945, 0.48145, 0.48289,\
        0.48403, 0.48508, 0.48612, 0.48725, 0.48849, 0.48986, 0.49137, 0.49302, 0.49483, 0.49688, 0.49937,\
        0.50244, 0.50608, 0.51005, 0.51420, 0.51846, 0.52279, 0.52716, 0.53154, 0.53593, 0.54030, 0.54462,\
        0.54887, 0.55301, 0.55698, 0.56071, 0.56412, 0.56711, 0.56958, 0.57144, 0.57267, 0.57331, 0.57352,\
        0.57345, 0.57316, 0.57263, 0.57186, 0.57094, 0.56999, 0.56909, 0.56826, 0.56738, 0.56629, 0.56476,\
        0.56266, 0.56005, 0.55716, 0.55429, 0.55168, 0.54951, 0.54774, 0.54616, 0.54448, 0.54239, 0.53975,\
        0.53661, 0.53320, 0.52987, 0.52693, 0.52464, 0.52314, 0.52249, 0.52276, 0.52406, 0.52638, 0.52960,\
        0.53343, 0.53746, 0.54123, 0.54436, 0.54660, 0.54783, 0.54804, 0.54728, 0.54561, 0.54309, 0.53977,\
        0.53569, 0.53090, 0.52545, 0.51947, 0.51304, 0.50622, 0.49901, 0.49274, 0.48632, 0.48010, 0.47413,\
        0.46825, 0.46238, 0.45652, 0.45071, 0.44496, 0.43930, 0.43376, 0.42837, 0.42319, 0.41830, 0.41376,\
        0.40961, 0.40586, 0.40251, 0.39929, 0.39646, 0.39462, 0.39394, 0.39432, 0.39556, 0.39725, 0.39812],\
        [0.39714, 0.41002, 0.42306, 0.43584, 0.44720, 0.45658, 0.46390, 0.46939, 0.47344, 0.47646, 0.47882,\
        0.48077, 0.48252, 0.48419, 0.48587, 0.48760, 0.48939, 0.49126, 0.49317, 0.49514, 0.49718, 0.49943,\
        0.50245, 0.50595, 0.50965, 0.51346, 0.51736, 0.52131, 0.52528, 0.52925, 0.53321, 0.53712, 0.54096,\
        0.54470, 0.54831, 0.55173, 0.55490, 0.55775, 0.56018, 0.56213, 0.56353, 0.56437, 0.56470, 0.56460,\
        0.56417, 0.56348, 0.56256, 0.56147, 0.56027, 0.55905, 0.55788, 0.55675, 0.55558, 0.55421, 0.55248,\
        0.55030, 0.54771, 0.54490, 0.54215, 0.53976, 0.53791, 0.53653, 0.53536, 0.53406, 0.53235, 0.53008,\
        0.52729, 0.52418, 0.52109, 0.51837, 0.51626, 0.51482, 0.51404, 0.51399, 0.51491, 0.51694, 0.52000,\
        0.52388, 0.52813, 0.53221, 0.53571, 0.53836, 0.54002, 0.54064, 0.54025, 0.53888, 0.53659, 0.53343,\
        0.52947, 0.52476, 0.51940, 0.51354, 0.50738, 0.50089, 0.49411, 0.48735, 0.48086, 0.47458, 0.46850,\
        0.46260, 0.45680, 0.45109, 0.44547, 0.43995, 0.43453, 0.42923, 0.42407, 0.41912, 0.41447, 0.41022,\
        0.40645, 0.40307, 0.40007, 0.39722, 0.39448, 0.39251, 0.39147, 0.39131, 0.39180, 0.39246, 0.39257],\
        [0.38914, 0.40129, 0.41450, 0.42731, 0.43866, 0.44824, 0.45602, 0.46222, 0.46711, 0.47101, 0.47422,\
        0.47697, 0.47945, 0.48179, 0.48408, 0.48637, 0.48869, 0.49102, 0.49334, 0.49561, 0.49786, 0.50024,\
        0.50299, 0.50610, 0.50938, 0.51277, 0.51624, 0.51975, 0.52329, 0.52681, 0.53030, 0.53373, 0.53707,\
        0.54029, 0.54336, 0.54624, 0.54886, 0.55117, 0.55308, 0.55455, 0.55551, 0.55597, 0.55596, 0.55553,\
        0.55476, 0.55371, 0.55244, 0.55103, 0.54955, 0.54808, 0.54668, 0.54534, 0.54400, 0.54250, 0.54068,\
        0.53846, 0.53587, 0.53309, 0.53040, 0.52817, 0.52670, 0.52589, 0.52531, 0.52454, 0.52329, 0.52142,\
        0.51892, 0.51602, 0.51307, 0.51052, 0.50864, 0.50741, 0.50659, 0.50617, 0.50673, 0.50849, 0.51139,\
        0.51541, 0.51995, 0.52433, 0.52813, 0.53107, 0.53303, 0.53393, 0.53380, 0.53265, 0.53054, 0.52752,\
        0.52366, 0.51903, 0.51373, 0.50783, 0.50184, 0.49549, 0.48878, 0.48202, 0.47543, 0.46905, 0.46290,\
        0.45697, 0.45123, 0.44566, 0.44023, 0.43494, 0.42978, 0.42472, 0.41978, 0.41501, 0.41051, 0.40645,\
        0.40297, 0.39988, 0.39691, 0.39419, 0.39170, 0.38970, 0.38837, 0.38768, 0.38742, 0.38721, 0.38663],\
        [0.38200, 0.39446, 0.40750, 0.41990, 0.43093, 0.44040, 0.44837, 0.45502, 0.46054, 0.46517, 0.46914,\
        0.47264, 0.47584, 0.47886, 0.48179, 0.48469, 0.48758, 0.49046, 0.49330, 0.49604, 0.49863, 0.50111,\
        0.50363, 0.50627, 0.50905, 0.51195, 0.51495, 0.51801, 0.52109, 0.52416, 0.52717, 0.53010, 0.53293,\
        0.53563, 0.53818, 0.54053, 0.54264, 0.54444, 0.54588, 0.54689, 0.54746, 0.54755, 0.54720, 0.54644,\
        0.54534, 0.54396, 0.54238, 0.54067, 0.53892, 0.53722, 0.53562, 0.53416, 0.53277, 0.53130, 0.52953,\
        0.52734, 0.52479, 0.52203, 0.51930, 0.51712, 0.51609, 0.51605, 0.51621, 0.51603, 0.51532, 0.51389,\
        0.51168, 0.50892, 0.50607, 0.50366, 0.50209, 0.50121, 0.50054, 0.49989, 0.50021, 0.50166, 0.50425,\
        0.50854, 0.51334, 0.51790, 0.52182, 0.52489, 0.52698, 0.52802, 0.52802, 0.52700, 0.52500, 0.52207,\
        0.51826, 0.51368, 0.50849, 0.50268, 0.49628, 0.48999, 0.48333, 0.47657, 0.46990, 0.46344, 0.45724,\
        0.45131, 0.44563, 0.44019, 0.43496, 0.42991, 0.42500, 0.42021, 0.41550, 0.41087, 0.40643, 0.40243,\
        0.39904, 0.39612, 0.39328, 0.39062, 0.38821, 0.38616, 0.38457, 0.38340, 0.38250, 0.38159, 0.38040],\
        [0.37572, 0.38857, 0.40136, 0.41319, 0.42365, 0.43280, 0.44079, 0.44772, 0.45372, 0.45896, 0.46359,\
        0.46778, 0.47165, 0.47533, 0.47890, 0.48241, 0.48590, 0.48937, 0.49279, 0.49607, 0.49903, 0.50164,\
        0.50389, 0.50606, 0.50834, 0.51077, 0.51332, 0.51595, 0.51860, 0.52122, 0.52377, 0.52621, 0.52854,\
        0.53073, 0.53277, 0.53462, 0.53625, 0.53759, 0.53860, 0.53923, 0.53942, 0.53918, 0.53850, 0.53744,\
        0.53604, 0.53436, 0.53248, 0.53051, 0.52852, 0.52661, 0.52485, 0.52332, 0.52204, 0.52079, 0.51917,\
        0.51708, 0.51465, 0.51201, 0.50927, 0.50694, 0.50638, 0.50737, 0.50824, 0.50855, 0.50845, 0.50756,\
        0.50561, 0.50307, 0.50039, 0.49825, 0.49700, 0.49636, 0.49591, 0.49561, 0.49597, 0.49717, 0.49965,\
        0.50397, 0.50867, 0.51308, 0.51689, 0.51988, 0.52192, 0.52294, 0.52294, 0.52193, 0.51995, 0.51705,\
        0.51324, 0.50859, 0.50339, 0.49769, 0.49129, 0.48462, 0.47783, 0.47099, 0.46424, 0.45771, 0.45146,\
        0.44553, 0.43993, 0.43462, 0.42959, 0.42479, 0.42017, 0.41567, 0.41122, 0.40675, 0.40235, 0.39836,\
        0.39499, 0.39200, 0.38920, 0.38657, 0.38414, 0.38198, 0.38013, 0.37854, 0.37709, 0.37561, 0.37388],\
        [0.36928, 0.38221, 0.39486, 0.40635, 0.41620, 0.42505, 0.43303, 0.44019, 0.44660, 0.45235, 0.45757,\
        0.46237, 0.46688, 0.47117, 0.47533, 0.47942, 0.48346, 0.48747, 0.49143, 0.49528, 0.49888, 0.50147,\
        0.50328, 0.50506, 0.50696, 0.50901, 0.51119, 0.51346, 0.51573, 0.51794, 0.52005, 0.52202, 0.52386,\
        0.52557, 0.52713, 0.52853, 0.52972, 0.53066, 0.53130, 0.53157, 0.53145, 0.53090, 0.52994, 0.52860,\
        0.52694, 0.52500, 0.52287, 0.52066, 0.51849, 0.51642, 0.51453, 0.51294, 0.51186, 0.51108, 0.50973,\
        0.50771, 0.50550, 0.50321, 0.50075, 0.49855, 0.49841, 0.50006, 0.50137, 0.50188, 0.50245, 0.50233,\
        0.50052, 0.49834, 0.49614, 0.49443, 0.49341, 0.49295, 0.49282, 0.49301, 0.49375, 0.49529, 0.49797,\
        0.50174, 0.50586, 0.50981, 0.51326, 0.51597, 0.51780, 0.51865, 0.51850, 0.51738, 0.51533, 0.51241,\
        0.50862, 0.50380, 0.49827, 0.49246, 0.48603, 0.47922, 0.47226, 0.46529, 0.45843, 0.45181, 0.44551,\
        0.43959, 0.43407, 0.42892, 0.42408, 0.41952, 0.41519, 0.41101, 0.40687, 0.40264, 0.39831, 0.39434,\
        0.39083, 0.38769, 0.38480, 0.38209, 0.37956, 0.37724, 0.37512, 0.37315, 0.37125, 0.36927, 0.36708],\
        [0.36221, 0.37497, 0.38727, 0.39835, 0.40801, 0.41683, 0.42491, 0.43234, 0.43913, 0.44533, 0.45106,\
        0.45642, 0.46148, 0.46633, 0.47103, 0.47562, 0.48012, 0.48456, 0.48889, 0.49304, 0.49675, 0.49943,\
        0.50129, 0.50296, 0.50469, 0.50651, 0.50843, 0.51043, 0.51240, 0.51426, 0.51597, 0.51751, 0.51889,\
        0.52014, 0.52126, 0.52225, 0.52307, 0.52366, 0.52398, 0.52397, 0.52359, 0.52277, 0.52155, 0.51999,\
        0.51813, 0.51599, 0.51365, 0.51127, 0.50899, 0.50685, 0.50491, 0.50324, 0.50232, 0.50222, 0.50119,\
        0.49917, 0.49725, 0.49537, 0.49349, 0.49214, 0.49224, 0.49361, 0.49498, 0.49595, 0.49692, 0.49727,\
        0.49601, 0.49432, 0.49273, 0.49153, 0.49087, 0.49074, 0.49103, 0.49174, 0.49297, 0.49488, 0.49760,\
        0.50091, 0.50438, 0.50774, 0.51069, 0.51300, 0.51449, 0.51504, 0.51461, 0.51325, 0.51101, 0.50795,\
        0.50415, 0.49956, 0.49362, 0.48735, 0.48069, 0.47372, 0.46659, 0.45946, 0.45246, 0.44572, 0.43936,\
        0.43346, 0.42803, 0.42304, 0.41839, 0.41404, 0.41000, 0.40611, 0.40218, 0.39827, 0.39403, 0.39003,\
        0.38640, 0.38310, 0.38006, 0.37721, 0.37453, 0.37201, 0.36962, 0.36730, 0.36500, 0.36260, 0.36000],\
        [0.35440, 0.36683, 0.37865, 0.38945, 0.39920, 0.40814, 0.41640, 0.42415, 0.43129, 0.43788, 0.44406,\
        0.44990, 0.45547, 0.46080, 0.46596, 0.47096, 0.47583, 0.48056, 0.48509, 0.48929, 0.49294, 0.49578,\
        0.49792, 0.49973, 0.50146, 0.50317, 0.50496, 0.50680, 0.50856, 0.51013, 0.51149, 0.51264, 0.51360,\
        0.51442, 0.51515, 0.51580, 0.51631, 0.51660, 0.51664, 0.51644, 0.51586, 0.51481, 0.51337, 0.51164,\
        0.50966, 0.50741, 0.50492, 0.50242, 0.50016, 0.49806, 0.49621, 0.49470, 0.49390, 0.49374, 0.49296,\
        0.49140, 0.48980, 0.48832, 0.48707, 0.48640, 0.48669, 0.48779, 0.48915, 0.49041, 0.49148, 0.49196,\
        0.49153, 0.49062, 0.48973, 0.48913, 0.48898, 0.48931, 0.49009, 0.49132, 0.49301, 0.49518, 0.49783,\
        0.50074, 0.50366, 0.50644, 0.50886, 0.51071, 0.51179, 0.51197, 0.51116, 0.50941, 0.50686, 0.50353,\
        0.49943, 0.49456, 0.48869, 0.48218, 0.47528, 0.46812, 0.46082, 0.45351, 0.44634, 0.43946, 0.43301,\
        0.42711, 0.42179, 0.41698, 0.41253, 0.40832, 0.40443, 0.40103, 0.39706, 0.39316, 0.38916, 0.38524,\
        0.38156, 0.37814, 0.37495, 0.37194, 0.36908, 0.36633, 0.36367, 0.36104, 0.35838, 0.35560, 0.35263],\
        [0.34597, 0.35809, 0.36959, 0.38026, 0.39007, 0.39903, 0.40758, 0.41571, 0.42309, 0.42999, 0.43656,\
        0.44284, 0.44884, 0.45460, 0.46014, 0.46548, 0.47062, 0.47554, 0.48017, 0.48440, 0.48807, 0.49107,\
        0.49348, 0.49550, 0.49731, 0.49902, 0.50076, 0.50254, 0.50417, 0.50548, 0.50655, 0.50740, 0.50798,\
        0.50841, 0.50879, 0.50917, 0.50949, 0.50951, 0.50927, 0.50897, 0.50830, 0.50704, 0.50538, 0.50353,\
        0.50152, 0.49926, 0.49676, 0.49430, 0.49208, 0.49009, 0.48838, 0.48708, 0.48629, 0.48582, 0.48514,\
        0.48408, 0.48292, 0.48189, 0.48116, 0.48096, 0.48141, 0.48244, 0.48377, 0.48511, 0.48627, 0.48700,\
        0.48720, 0.48706, 0.48690, 0.48695, 0.48738, 0.48825, 0.48955, 0.49129, 0.49339, 0.49577, 0.49827,\
        0.50080, 0.50325, 0.50551, 0.50741, 0.50879, 0.50947, 0.50928, 0.50804, 0.50575, 0.50277, 0.49915,\
        0.49468, 0.48945, 0.48345, 0.47682, 0.46976, 0.46242, 0.45495, 0.44746, 0.44012, 0.43305, 0.42646,\
        0.42052, 0.41535, 0.41077, 0.40653, 0.40259, 0.39851, 0.39524, 0.39153, 0.38766, 0.38377, 0.37994,\
        0.37626, 0.37276, 0.36943, 0.36626, 0.36321, 0.36024, 0.35731, 0.35439, 0.35140, 0.34829, 0.34498],\
        [0.33705, 0.34893, 0.36025, 0.37085, 0.38072, 0.38994, 0.39889, 0.40709, 0.41447, 0.42165, 0.42860,\
        0.43526, 0.44164, 0.44775, 0.45360, 0.45921, 0.46456, 0.46962, 0.47433, 0.47860, 0.48234, 0.48551,\
        0.48814, 0.49038, 0.49234, 0.49414, 0.49589, 0.49760, 0.49913, 0.50022, 0.50110, 0.50175, 0.50199,\
        0.50213, 0.50224, 0.50238, 0.50265, 0.50242, 0.50182, 0.50153, 0.50087, 0.49941, 0.49761, 0.49570,\
        0.49366, 0.49144, 0.48907, 0.48674, 0.48461, 0.48274, 0.48118, 0.47998, 0.47916, 0.47857, 0.47796,\
        0.47723, 0.47648, 0.47588, 0.47557, 0.47570, 0.47632, 0.47737, 0.47869, 0.48007, 0.48135, 0.48236,\
        0.48307, 0.48359, 0.48410, 0.48479, 0.48580, 0.48720, 0.48902, 0.49123, 0.49375, 0.49637, 0.49872,\
        0.50080, 0.50286, 0.50461, 0.50599, 0.50691, 0.50722, 0.50675, 0.50521, 0.50228, 0.49869, 0.49467,\
        0.48986, 0.48430, 0.47808, 0.47130, 0.46411, 0.45663, 0.44902, 0.44138, 0.43386, 0.42660, 0.41980,\
        0.41372, 0.40870, 0.40447, 0.40033, 0.39653, 0.39275, 0.38913, 0.38551, 0.38176, 0.37796, 0.37419,\
        0.37051, 0.36695, 0.36352, 0.36019, 0.35695, 0.35376, 0.35058, 0.34738, 0.34409, 0.34067, 0.33705],\
        [0.32773, 0.33946, 0.35066, 0.36126, 0.37124, 0.38070, 0.38969, 0.39780, 0.40536, 0.41292, 0.42022,\
        0.42722, 0.43391, 0.44031, 0.44642, 0.45224, 0.45776, 0.46293, 0.46772, 0.47206, 0.47590, 0.47921,\
        0.48205, 0.48448, 0.48660, 0.48851, 0.49028, 0.49191, 0.49328, 0.49431, 0.49507, 0.49555, 0.49567,\
        0.49565, 0.49559, 0.49557, 0.49564, 0.49528, 0.49460, 0.49407, 0.49324, 0.49181, 0.49004, 0.48811,\
        0.48608, 0.48393, 0.48171, 0.47956, 0.47757, 0.47583, 0.47438, 0.47326, 0.47244, 0.47183, 0.47131,\
        0.47082, 0.47041, 0.47016, 0.47018, 0.47057, 0.47134, 0.47246, 0.47381, 0.47525, 0.47667, 0.47795,\
        0.47909, 0.48014, 0.48122, 0.48248, 0.48401, 0.48589, 0.48815, 0.49076, 0.49364, 0.49663, 0.49934,\
        0.50095, 0.50226, 0.50338, 0.50422, 0.50469, 0.50465, 0.50396, 0.50237, 0.49927, 0.49490, 0.49020,\
        0.48493, 0.47905, 0.47260, 0.46567, 0.45835, 0.45078, 0.44306, 0.43532, 0.42767, 0.42026, 0.41325,\
        0.40688, 0.40175, 0.39804, 0.39404, 0.39022, 0.38652, 0.38287, 0.37923, 0.37553, 0.37179, 0.36806,\
        0.36437, 0.36075, 0.35721, 0.35374, 0.35031, 0.34691, 0.34349, 0.34002, 0.33646, 0.33275, 0.32885]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=50 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax50vt7.5
    zgrid_H_Hmax_vmax50vt7_5=[[0.215921, 0.225180, 0.233912, 0.242140, 0.249887, 0.257176, 0.264033, 0.270482, 0.276549, 0.282258,\
        0.287637, 0.292712, 0.297510, 0.302059, 0.306385, 0.310517, 0.314481, 0.318303, 0.322007, 0.325615,\
        0.329150, 0.332629, 0.336071, 0.339490, 0.342901, 0.346317, 0.349747, 0.353204, 0.356696, 0.360231,\
        0.363818, 0.367464, 0.371174, 0.374954, 0.378807, 0.382734, 0.386733, 0.390791, 0.394872, 0.398908,\
        0.403043, 0.407333, 0.411771, 0.416353, 0.421059, 0.425854, 0.430690, 0.435510, 0.440245, 0.444821,\
        0.449161, 0.453191, 0.456847, 0.460085, 0.462891, 0.465287, 0.467329, 0.469104, 0.470707, 0.472230,\
        0.473746, 0.475301, 0.476933, 0.478689, 0.480643, 0.482880, 0.485483, 0.488493, 0.491841, 0.495438,\
        0.499133, 0.502774, 0.506374, 0.509903, 0.513321, 0.516591, 0.519677, 0.522548, 0.525178, 0.527547,\
        0.529640, 0.531451, 0.532982, 0.534239, 0.535231, 0.535965, 0.536448, 0.536683, 0.536674, 0.536423,\
        0.535932, 0.535204, 0.534236, 0.533027, 0.531573, 0.529872, 0.527923, 0.525725, 0.523280, 0.520588,\
        0.517649, 0.514459, 0.511012, 0.507297, 0.503295, 0.498971, 0.494232, 0.489127, 0.483882, 0.478286,\
        0.472244, 0.465763, 0.458858, 0.451541, 0.443815, 0.435680, 0.427138, 0.418192, 0.408845, 0.399098,\
        0.388957],\
        [0.227088, 0.236133, 0.244637, 0.252623, 0.260117, 0.267144, 0.273730, 0.279901, 0.285683, 0.291104,\
        0.296189, 0.300966, 0.305465, 0.309713, 0.313741, 0.317576, 0.321249, 0.324785, 0.328211, 0.331551,\
        0.334825, 0.338054, 0.341255, 0.344443, 0.347632, 0.350835, 0.354061, 0.357321, 0.360624, 0.363979,\
        0.367394, 0.370877, 0.374435, 0.378073, 0.381796, 0.385607, 0.389513, 0.393516, 0.397580, 0.401620,\
        0.405671, 0.409936, 0.414421, 0.419108, 0.423970, 0.428967, 0.434045, 0.439135, 0.444161, 0.449034,\
        0.453659, 0.457943, 0.461800, 0.465168, 0.468019, 0.470377, 0.472317, 0.473955, 0.475425, 0.476849,\
        0.478312, 0.479851, 0.481474, 0.483210, 0.485149, 0.487406, 0.490079, 0.493297, 0.496961, 0.500807,\
        0.504867, 0.508851, 0.512768, 0.516589, 0.520274, 0.523784, 0.527084, 0.530141, 0.532928, 0.535424,\
        0.537614, 0.539494, 0.541070, 0.542355, 0.543364, 0.544109, 0.544597, 0.544832, 0.544817, 0.544557,\
        0.544054, 0.543313, 0.542331, 0.541105, 0.539632, 0.537908, 0.535932, 0.533705, 0.531231, 0.528513,\
        0.525552, 0.522348, 0.518894, 0.515180, 0.511184, 0.506869, 0.502104, 0.497113, 0.492129, 0.486648,\
        0.480665, 0.474242, 0.467406, 0.460165, 0.452519, 0.444463, 0.435998, 0.427127, 0.417855, 0.408183,\
        0.398113],\
        [0.238043, 0.246870, 0.255140, 0.262879, 0.270114, 0.276873, 0.283182, 0.289070, 0.294563, 0.299688,\
        0.304472, 0.308945, 0.313137, 0.317077, 0.320797, 0.324329, 0.327702, 0.330946, 0.334087, 0.337152,\
        0.340161, 0.343135, 0.346091, 0.349044, 0.352007, 0.354991, 0.358007, 0.361064, 0.364170, 0.367334,\
        0.370563, 0.373868, 0.377254, 0.380729, 0.384296, 0.387955, 0.391710, 0.395578, 0.399586, 0.403537,\
        0.407606, 0.411901, 0.416459, 0.421273, 0.426314, 0.431539, 0.436887, 0.442285, 0.447644, 0.452863,\
        0.457828, 0.462423, 0.466531, 0.470062, 0.472969, 0.475271, 0.477065, 0.478514, 0.479806, 0.481112,\
        0.482545, 0.484125, 0.485799, 0.487537, 0.489467, 0.491781, 0.494522, 0.497786, 0.501974, 0.506176,\
        0.510545, 0.514898, 0.519157, 0.523286, 0.527245, 0.530999, 0.534513, 0.537755, 0.540697, 0.543314,\
        0.545589, 0.547520, 0.549119, 0.550411, 0.551420, 0.552161, 0.552641, 0.552862, 0.552826, 0.552540,\
        0.552011, 0.551242, 0.550233, 0.548978, 0.547471, 0.545706, 0.543683, 0.541405, 0.538879, 0.536112,\
        0.533108, 0.529866, 0.526382, 0.522645, 0.518632, 0.514307, 0.509656, 0.504899, 0.499992, 0.494543,\
        0.488583, 0.482228, 0.475469, 0.468322, 0.460772, 0.452805, 0.444423, 0.435635, 0.426445, 0.416853,\
        0.406861],\
        [0.248792, 0.257396, 0.265427, 0.272912, 0.279883, 0.286368, 0.292396, 0.297996, 0.303194, 0.308018,\
        0.312496, 0.316658, 0.320535, 0.324158, 0.327563, 0.330782, 0.333848, 0.336792, 0.339643, 0.342427,\
        0.345167, 0.347883, 0.350591, 0.353306, 0.356040, 0.358803, 0.361605, 0.364452, 0.367353, 0.370315,\
        0.373346, 0.376455, 0.379651, 0.382940, 0.386328, 0.389810, 0.393368, 0.396993, 0.400764, 0.404730,\
        0.408848, 0.413210, 0.417869, 0.422832, 0.428073, 0.433547, 0.439193, 0.444931, 0.450665, 0.456280,\
        0.461645, 0.466616, 0.471041, 0.474785, 0.477765, 0.479988, 0.481579, 0.482760, 0.483803, 0.484954,\
        0.486378, 0.488093, 0.489945, 0.491724, 0.493613, 0.496123, 0.499123, 0.502634, 0.506948, 0.511629,\
        0.516359, 0.521054, 0.525639, 0.530066, 0.534294, 0.538286, 0.542008, 0.545428, 0.548517, 0.551248,\
        0.553596, 0.555556, 0.557150, 0.558422, 0.559411, 0.560137, 0.560600, 0.560794, 0.560721, 0.560391,\
        0.559817, 0.559006, 0.557957, 0.556661, 0.555106, 0.553283, 0.551191, 0.548838, 0.546236, 0.543397,\
        0.540327, 0.537028, 0.533497, 0.529724, 0.525696, 0.521403, 0.516873, 0.512168, 0.507215, 0.501791,\
        0.495929, 0.489686, 0.483008, 0.475999, 0.468571, 0.460697, 0.452405, 0.443709, 0.434611, 0.425107,\
        0.415199],\
        [0.259342, 0.267718, 0.275502, 0.282728, 0.289427, 0.295633, 0.301375, 0.306683, 0.311583, 0.316103,\
        0.320269, 0.324113, 0.327667, 0.330966, 0.334046, 0.336943, 0.339694, 0.342332, 0.344888, 0.347388,\
        0.349856, 0.352311, 0.354769, 0.357245, 0.359749, 0.362290, 0.364874, 0.367508, 0.370198, 0.372950,\
        0.375770, 0.378667, 0.381651, 0.384733, 0.387921, 0.391219, 0.394604, 0.398016, 0.401562, 0.405396,\
        0.409486, 0.413896, 0.418661, 0.423784, 0.429237, 0.434977, 0.440940, 0.447043, 0.453185, 0.459243,\
        0.465069, 0.470490, 0.475314, 0.479346, 0.482437, 0.484559, 0.485868, 0.486679, 0.487369, 0.488290,\
        0.489690, 0.491637, 0.493909, 0.495879, 0.497557, 0.500344, 0.503912, 0.507901, 0.512464, 0.517393,\
        0.522428, 0.527425, 0.532301, 0.537000, 0.541478, 0.545692, 0.549606, 0.553190, 0.556418, 0.559258,\
        0.561669, 0.563631, 0.565182, 0.566399, 0.567351, 0.568056, 0.568496, 0.568651, 0.568521, 0.568126,\
        0.567488, 0.566619, 0.565519, 0.564173, 0.562558, 0.560657, 0.558471, 0.556017, 0.553314, 0.550379,\
        0.547223, 0.543849, 0.540256, 0.536438, 0.532390, 0.528110, 0.523608, 0.518883, 0.513871, 0.508484,\
        0.502752, 0.496550, 0.489968, 0.483229, 0.475918, 0.468124, 0.459931, 0.451344, 0.442353, 0.432948,\
        0.423128],\
        [0.269701, 0.277842, 0.285372, 0.292329, 0.298750, 0.304672, 0.310125, 0.315138, 0.319738, 0.323950,\
        0.327801, 0.331321, 0.334544, 0.337509, 0.340254, 0.342822, 0.345250, 0.347576, 0.349831, 0.352044,\
        0.354238, 0.356432, 0.358642, 0.360880, 0.363155, 0.365473, 0.367841, 0.370260, 0.372736, 0.375270,\
        0.377869, 0.380540, 0.383291, 0.386137, 0.389094, 0.392182, 0.395420, 0.398786, 0.402056, 0.405581,\
        0.409570, 0.414002, 0.418866, 0.424147, 0.429814, 0.435822, 0.442108, 0.448590, 0.455164, 0.461699,\
        0.468037, 0.473983, 0.479305, 0.483731, 0.487004, 0.489014, 0.489954, 0.490275, 0.490491, 0.491062,\
        0.492348, 0.494498, 0.497344, 0.500005, 0.501757, 0.504815, 0.508855, 0.513382, 0.518315, 0.523522,\
        0.528823, 0.534081, 0.539208, 0.544145, 0.548845, 0.553259, 0.557341, 0.561064, 0.564415, 0.567366,\
        0.569843, 0.571777, 0.573226, 0.574344, 0.575244, 0.575937, 0.576362, 0.576464, 0.576249, 0.575765,\
        0.575042, 0.574096, 0.572935, 0.571535, 0.569851, 0.567849, 0.565539, 0.562953, 0.560123, 0.557072,\
        0.553811, 0.550346, 0.546676, 0.542800, 0.538714, 0.534413, 0.529892, 0.525131, 0.520082, 0.514693,\
        0.508916, 0.502592, 0.496603, 0.490117, 0.482774, 0.475053, 0.466992, 0.458545, 0.449678, 0.440378,\
        0.430650],\
        [0.279877, 0.287773, 0.295039, 0.301719, 0.307855, 0.313487, 0.318648, 0.323367, 0.327666, 0.331569,\
        0.335101, 0.338291, 0.341176, 0.343797, 0.346198, 0.348425, 0.350523, 0.352530, 0.354482, 0.356406,\
        0.358326, 0.360261, 0.362225, 0.364229, 0.366278, 0.368379, 0.370532, 0.372739, 0.375000, 0.377316,\
        0.379688, 0.382119, 0.384619, 0.387199, 0.389876, 0.392673, 0.395610, 0.398646, 0.401694, 0.405160,\
        0.409113, 0.413571, 0.418526, 0.423953, 0.429821, 0.436086, 0.442687, 0.449546, 0.456556, 0.463584,\
        0.470465, 0.476993, 0.482908, 0.487868, 0.491456, 0.493370, 0.493850, 0.493592, 0.493246, 0.493339,\
        0.494365, 0.496592, 0.499691, 0.503000, 0.506048, 0.509705, 0.514167, 0.519161, 0.524494, 0.530015,\
        0.535578, 0.541066, 0.546401, 0.551537, 0.556432, 0.561025, 0.565247, 0.569062, 0.572497, 0.575571,\
        0.578155, 0.580031, 0.581285, 0.582248, 0.583099, 0.583808, 0.584240, 0.584267, 0.583926, 0.583327,\
        0.582498, 0.581451, 0.580221, 0.578775, 0.577015, 0.574882, 0.572407, 0.569653, 0.566673, 0.563487,\
        0.560106, 0.556534, 0.552773, 0.548824, 0.544682, 0.540338, 0.535780, 0.530983, 0.525910, 0.520525,\
        0.514811, 0.508894, 0.503094, 0.496297, 0.488985, 0.481460, 0.473605, 0.465334, 0.456603, 0.447408,\
        0.437768],\
        [0.289878, 0.297519, 0.304508, 0.310898, 0.316740, 0.322078, 0.326947, 0.331373, 0.335375, 0.338971,\
        0.342182, 0.345037, 0.347574, 0.349839, 0.351884, 0.353762, 0.355521, 0.357205, 0.358851, 0.360487,\
        0.362136, 0.363816, 0.365539, 0.367313, 0.369143, 0.371032, 0.372978, 0.374979, 0.377031, 0.379131,\
        0.381276, 0.383465, 0.385703, 0.387996, 0.390357, 0.392800, 0.395332, 0.397959, 0.400879, 0.404281,\
        0.408202, 0.412681, 0.417705, 0.423252, 0.429291, 0.435784, 0.442677, 0.449893, 0.457328, 0.464843,\
        0.472265, 0.479384, 0.485935, 0.491551, 0.495668, 0.497552, 0.497522, 0.496735, 0.495891, 0.495432,\
        0.496050, 0.498348, 0.501935, 0.505956, 0.510159, 0.514724, 0.519799, 0.525292, 0.531044, 0.536900,\
        0.542722, 0.548409, 0.553906, 0.559195, 0.564260, 0.569026, 0.573367, 0.577205, 0.580622, 0.583807,\
        0.586637, 0.588439, 0.589340, 0.590114, 0.590941, 0.591700, 0.592187, 0.592090, 0.591555, 0.590846,\
        0.589897, 0.588697, 0.587388, 0.585924, 0.584087, 0.581771, 0.579071, 0.576118, 0.572971, 0.569640,\
        0.566125, 0.562432, 0.558566, 0.554530, 0.550318, 0.545919, 0.541318, 0.536490, 0.531409, 0.526054,\
        0.520433, 0.514591, 0.508428, 0.501650, 0.494660, 0.487440, 0.479842, 0.471755, 0.463147, 0.454043,\
        0.444481],\
        [0.299714, 0.307085, 0.313781, 0.319868, 0.325405, 0.330444, 0.335022, 0.339160, 0.342872, 0.346167,\
        0.349058, 0.351572, 0.353750, 0.355646, 0.357321, 0.358838, 0.360251, 0.361610, 0.362950, 0.364301,\
        0.365683, 0.367114, 0.368602, 0.370155, 0.371775, 0.373462, 0.375211, 0.377016, 0.378870, 0.380764,\
        0.382690, 0.384643, 0.386622, 0.388628, 0.390670, 0.392763, 0.394939, 0.397286, 0.399953, 0.403143,\
        0.406978, 0.411440, 0.416493, 0.422111, 0.428269, 0.434943, 0.442088, 0.449632, 0.457468, 0.465447,\
        0.473380, 0.481037, 0.488154, 0.494383, 0.499089, 0.501121, 0.500928, 0.499951, 0.498924, 0.497981,\
        0.498102, 0.500613, 0.504799, 0.509583, 0.514663, 0.520050, 0.525787, 0.531817, 0.538010, 0.544215,\
        0.550294, 0.556149, 0.561744, 0.567118, 0.572321, 0.577288, 0.581772, 0.585564, 0.588766, 0.591859,\
        0.595211, 0.596978, 0.597392, 0.598065, 0.598892, 0.599652, 0.600200, 0.599914, 0.599152, 0.598381,\
        0.597307, 0.595854, 0.594459, 0.593027, 0.591103, 0.588498, 0.585504, 0.582339, 0.579032, 0.575553,\
        0.571891, 0.568060, 0.564075, 0.559940, 0.555649, 0.551189, 0.546542, 0.541687, 0.536601, 0.531268,\
        0.525678, 0.519812, 0.513594, 0.506993, 0.500159, 0.493168, 0.485795, 0.477848, 0.469318, 0.460273,\
        0.450779],\
        [0.309397, 0.316480, 0.322862, 0.328626, 0.333845, 0.338581, 0.342871, 0.346733, 0.350167, 0.353170,\
        0.355745, 0.357911, 0.359716, 0.361225, 0.362514, 0.363658, 0.364720, 0.365752, 0.366790, 0.367862,\
        0.368986, 0.370175, 0.371438, 0.372779, 0.374200, 0.375697, 0.377263, 0.378887, 0.380558, 0.382262,\
        0.383986, 0.385720, 0.387459, 0.389199, 0.390948, 0.392724, 0.394573, 0.396586, 0.398930, 0.401892,\
        0.405595, 0.409982, 0.414994, 0.420606, 0.426810, 0.433599, 0.440943, 0.448777, 0.456992, 0.465422,\
        0.473841, 0.481970, 0.489492, 0.496085, 0.501094, 0.503628, 0.504198, 0.503701, 0.502816, 0.501855,\
        0.501963, 0.504787, 0.509143, 0.514301, 0.519926, 0.525920, 0.532238, 0.538791, 0.545435, 0.552002,\
        0.558341, 0.564336, 0.569952, 0.575296, 0.580558, 0.585788, 0.590530, 0.594277, 0.597199, 0.599722,\
        0.603199, 0.605240, 0.605831, 0.606462, 0.607182, 0.607777, 0.608054, 0.607723, 0.606935, 0.605971,\
        0.604695, 0.603097, 0.601558, 0.600095, 0.598005, 0.594963, 0.591660, 0.588332, 0.584897, 0.581262,\
        0.577428, 0.573439, 0.569323, 0.565080, 0.560703, 0.556176, 0.551483, 0.546600, 0.541507, 0.536183,\
        0.530609, 0.524762, 0.518617, 0.512191, 0.505550, 0.498741, 0.491492, 0.483598, 0.475076, 0.466063,\
        0.456639],\
        [0.318943, 0.325714, 0.331754, 0.337169, 0.342054, 0.346480, 0.350489, 0.354091, 0.357269, 0.359998,\
        0.362261, 0.364072, 0.365484, 0.366583, 0.367467, 0.368226, 0.368933, 0.369642, 0.370386, 0.371189,\
        0.372064, 0.373022, 0.374070, 0.375211, 0.376444, 0.377766, 0.379165, 0.380627, 0.382135, 0.383671,\
        0.385217, 0.386759, 0.388287, 0.389798, 0.391297, 0.392804, 0.394366, 0.396075, 0.398102, 0.400737,\
        0.404231, 0.408456, 0.413320, 0.418816, 0.424966, 0.431786, 0.439268, 0.447358, 0.455943, 0.464842,\
        0.473790, 0.482430, 0.490311, 0.497006, 0.502428, 0.506025, 0.507738, 0.508134, 0.507857, 0.507568,\
        0.508318, 0.510925, 0.515062, 0.520215, 0.526062, 0.532433, 0.539211, 0.546246, 0.553339, 0.560279,\
        0.566898, 0.573046, 0.578614, 0.583777, 0.588904, 0.594342, 0.599565, 0.603281, 0.606204, 0.608780,\
        0.611501, 0.613521, 0.614569, 0.615236, 0.615766, 0.616098, 0.616095, 0.615641, 0.614785, 0.613633,\
        0.612193, 0.610523, 0.608773, 0.606883, 0.604399, 0.601091, 0.597606, 0.594185, 0.590645, 0.586810,\
        0.582754, 0.578586, 0.574332, 0.569979, 0.565509, 0.560911, 0.556168, 0.551257, 0.546152, 0.540830,\
        0.535271, 0.529460, 0.523396, 0.517102, 0.510622, 0.503930, 0.496793, 0.488886, 0.480310, 0.471355,\
        0.462045],\
        [0.328372, 0.334801, 0.340463, 0.345496, 0.350023, 0.354132, 0.357870, 0.361235, 0.364188, 0.366669,\
        0.368630, 0.370073, 0.371063, 0.371720, 0.372175, 0.372540, 0.372896, 0.373292, 0.373757, 0.374305,\
        0.374945, 0.375683, 0.376525, 0.377476, 0.378535, 0.379696, 0.380947, 0.382268, 0.383637, 0.385030,\
        0.386426, 0.387807, 0.389162, 0.390486, 0.391785, 0.393081, 0.394419, 0.395884, 0.397623, 0.399907,\
        0.403090, 0.407001, 0.411562, 0.416801, 0.422776, 0.429533, 0.437087, 0.445400, 0.454367, 0.463797,\
        0.473399, 0.482778, 0.491417, 0.498662, 0.504680, 0.509302, 0.512118, 0.513505, 0.514072, 0.514552,\
        0.515764, 0.518277, 0.522133, 0.527107, 0.532967, 0.539540, 0.546669, 0.554154, 0.561708, 0.569017,\
        0.575927, 0.582343, 0.587871, 0.592757, 0.597671, 0.602962, 0.608300, 0.612410, 0.615590, 0.618260,\
        0.620604, 0.622405, 0.623536, 0.624190, 0.624537, 0.624604, 0.624340, 0.623700, 0.622698, 0.621378,\
        0.619773, 0.617929, 0.615876, 0.613528, 0.610674, 0.607279, 0.603655, 0.600050, 0.596358, 0.592195,\
        0.587859, 0.583516, 0.579138, 0.574669, 0.570097, 0.565422, 0.560627, 0.555684, 0.550563, 0.545235,\
        0.539682, 0.533890, 0.527859, 0.521603, 0.515135, 0.508439, 0.501431, 0.493369, 0.484855, 0.476126,\
        0.467013],\
        [0.337713, 0.343761, 0.348998, 0.353606, 0.357745, 0.361528, 0.365003, 0.368160, 0.370929, 0.373202,\
        0.374878, 0.375931, 0.376455, 0.376627, 0.376628, 0.376597, 0.376615, 0.376722, 0.376930, 0.377242,\
        0.377660, 0.378188, 0.378833, 0.379603, 0.380498, 0.381514, 0.382635, 0.383838, 0.385093, 0.386372,\
        0.387647, 0.388899, 0.390116, 0.391296, 0.392446, 0.393587, 0.394764, 0.396055, 0.397594, 0.399584,\
        0.402292, 0.405684, 0.409761, 0.414591, 0.420262, 0.426853, 0.434411, 0.442922, 0.452294, 0.462340,\
        0.472760, 0.483141, 0.492979, 0.501662, 0.508526, 0.513915, 0.517658, 0.519932, 0.521254, 0.522315,\
        0.523817, 0.526277, 0.529900, 0.534657, 0.540430, 0.547087, 0.554485, 0.562415, 0.570498, 0.578171,\
        0.585197, 0.592094, 0.597829, 0.602509, 0.607507, 0.612654, 0.617675, 0.621927, 0.625318, 0.628035,\
        0.630197, 0.631786, 0.632795, 0.633313, 0.633450, 0.633256, 0.632734, 0.631874, 0.630682, 0.629176,\
        0.627376, 0.625298, 0.622934, 0.620223, 0.617088, 0.613555, 0.609786, 0.605926, 0.601842, 0.597326,\
        0.592717, 0.588280, 0.583801, 0.579190, 0.574496, 0.569736, 0.564889, 0.559914, 0.554770, 0.549426,\
        0.543859, 0.538055, 0.532006, 0.525702, 0.519111, 0.512149, 0.504591, 0.496628, 0.488857, 0.480520,\
        0.471625],\
        [0.347002, 0.352626, 0.357377, 0.361505, 0.365221, 0.368663, 0.371883, 0.374857, 0.377491, 0.379616,\
        0.381031, 0.381657, 0.381644, 0.381275, 0.380807, 0.380394, 0.380106, 0.379958, 0.379941, 0.380041,\
        0.380252, 0.380576, 0.381028, 0.381620, 0.382359, 0.383243, 0.384254, 0.385362, 0.386529, 0.387719,\
        0.388899, 0.390050, 0.391161, 0.392233, 0.393275, 0.394306, 0.395364, 0.396514, 0.397873, 0.399598,\
        0.401714, 0.404439, 0.407895, 0.412178, 0.417418, 0.423743, 0.431239, 0.439925, 0.449733, 0.460486,\
        0.471883, 0.483480, 0.494645, 0.504720, 0.513152, 0.519707, 0.524378, 0.527384, 0.529237, 0.530624,\
        0.532236, 0.534607, 0.538031, 0.542592, 0.548245, 0.554907, 0.562483, 0.570824, 0.579596, 0.587852,\
        0.594530, 0.601374, 0.607900, 0.613054, 0.618107, 0.623130, 0.627888, 0.632054, 0.635471, 0.638152,\
        0.640155, 0.641522, 0.642305, 0.642592, 0.642471, 0.642005, 0.641219, 0.640123, 0.638717, 0.637006,\
        0.634989, 0.632663, 0.630010, 0.626997, 0.623603, 0.619860, 0.615845, 0.611607, 0.607083, 0.602233,\
        0.597405, 0.593060, 0.588398, 0.583563, 0.578723, 0.573883, 0.568990, 0.563983, 0.558808, 0.553432,\
        0.547831, 0.541990, 0.535893, 0.529516, 0.522811, 0.515699, 0.508154, 0.500594, 0.492980, 0.484767,\
        0.475966],\
        [0.356290, 0.361436, 0.365631, 0.369217, 0.372473, 0.375560, 0.378523, 0.381319, 0.383852, 0.385911,\
        0.387112, 0.387240, 0.386578, 0.385613, 0.384685, 0.383939, 0.383402, 0.383051, 0.382849, 0.382761,\
        0.382773, 0.382893, 0.383145, 0.383555, 0.384140, 0.384902, 0.385822, 0.386859, 0.387965, 0.389090,\
        0.390197, 0.391263, 0.392287, 0.393274, 0.394236, 0.395189, 0.396150, 0.397151, 0.398248, 0.399506,\
        0.401075, 0.403166, 0.405921, 0.409538, 0.414229, 0.420189, 0.427560, 0.436405, 0.446682, 0.458231,\
        0.470756, 0.483830, 0.496767, 0.508594, 0.518708, 0.526644, 0.532250, 0.535793, 0.537885, 0.539312,\
        0.540837, 0.543053, 0.546314, 0.550739, 0.556299, 0.562921, 0.570557, 0.579105, 0.588463, 0.597938,\
        0.604876, 0.611353, 0.617991, 0.623811, 0.629109, 0.634091, 0.638688, 0.642725, 0.646057, 0.648620,\
        0.650420, 0.651514, 0.651999, 0.651982, 0.651559, 0.650799, 0.649742, 0.648401, 0.646774, 0.644851,\
        0.642614, 0.640045, 0.637125, 0.633838, 0.630184, 0.626186, 0.621885, 0.617305, 0.612450, 0.607398,\
        0.602575, 0.598047, 0.592859, 0.587760, 0.582796, 0.577899, 0.572975, 0.567935, 0.562717, 0.557289,\
        0.551633, 0.545736, 0.539585, 0.533159, 0.526430, 0.519391, 0.512125, 0.504804, 0.497200, 0.488805,\
        0.480022],\
        [0.365639, 0.370250, 0.373807, 0.376793, 0.379565, 0.382295, 0.384993, 0.387563, 0.389940, 0.392027,\
        0.393108, 0.392598, 0.391143, 0.389569, 0.388260, 0.387274, 0.386567, 0.386076, 0.385733, 0.385476,\
        0.385288, 0.385190, 0.385225, 0.385438, 0.385863, 0.386510, 0.387356, 0.388348, 0.389418, 0.390499,\
        0.391542, 0.392530, 0.393469, 0.394378, 0.395276, 0.396171, 0.397055, 0.397917, 0.398746, 0.399558,\
        0.400533, 0.401890, 0.403825, 0.406650, 0.410676, 0.416179, 0.423367, 0.432358, 0.443148, 0.455595,\
        0.469383, 0.484041, 0.499046, 0.513173, 0.525404, 0.534878, 0.541314, 0.545095, 0.547057, 0.548196,\
        0.549425, 0.551429, 0.554597, 0.559031, 0.564621, 0.571230, 0.578878, 0.587504, 0.596493, 0.606494,\
        0.614963, 0.622168, 0.628871, 0.634946, 0.640416, 0.645389, 0.649875, 0.653790, 0.657005, 0.659398,\
        0.660930, 0.661682, 0.661799, 0.661420, 0.660655, 0.659583, 0.658248, 0.656665, 0.654823, 0.652696,\
        0.650248, 0.647447, 0.644276, 0.640730, 0.636817, 0.632556, 0.627977, 0.623114, 0.618019, 0.612819,\
        0.607702, 0.602456, 0.597016, 0.591804, 0.586766, 0.581851, 0.576909, 0.571822, 0.566533, 0.561024,\
        0.555287, 0.549321, 0.543115, 0.536659, 0.529945, 0.522986, 0.515825, 0.508461, 0.500652, 0.492401,\
        0.483751],\
        [0.375116, 0.379133, 0.381982, 0.384336, 0.386646, 0.389043, 0.391478, 0.393749, 0.395671, 0.397704,\
        0.398809, 0.397452, 0.395165, 0.393124, 0.391614, 0.390516, 0.389712, 0.389141, 0.388697, 0.388280,\
        0.387871, 0.387523, 0.387309, 0.387298, 0.387546, 0.388078, 0.388869, 0.389846, 0.390907, 0.391958,\
        0.392937, 0.393831, 0.394667, 0.395487, 0.396323, 0.397176, 0.398013, 0.398774, 0.399393, 0.399851,\
        0.400173, 0.400619, 0.401590, 0.403495, 0.406745, 0.411706, 0.418662, 0.427794, 0.439154, 0.452639,\
        0.467947, 0.484475, 0.501498, 0.518759, 0.533576, 0.544642, 0.551634, 0.555190, 0.556546, 0.557033,\
        0.557765, 0.559529, 0.562746, 0.567463, 0.573392, 0.580151, 0.587816, 0.596933, 0.606529, 0.616275,\
        0.625447, 0.633380, 0.640353, 0.646535, 0.652028, 0.656928, 0.661292, 0.665092, 0.668201, 0.670413,\
        0.671607, 0.671927, 0.671616, 0.670832, 0.669697, 0.668301, 0.666689, 0.664871, 0.662833, 0.660525,\
        0.657884, 0.654863, 0.651452, 0.647656, 0.643489, 0.638972, 0.634135, 0.629016, 0.623677, 0.618202,\
        0.612645, 0.606969, 0.601313, 0.595860, 0.590749, 0.585846, 0.580871, 0.575694, 0.570284, 0.564651,\
        0.558805, 0.552752, 0.546486, 0.540000, 0.533288, 0.526352, 0.519186, 0.511735, 0.503912, 0.495751,\
        0.487223],\
        [0.384772, 0.388130, 0.390295, 0.392059, 0.394024, 0.396148, 0.398276, 0.400270, 0.401484, 0.402804,\
        0.403221, 0.401464, 0.398718, 0.396508, 0.395025, 0.393885, 0.392985, 0.392378, 0.391869, 0.391275,\
        0.390598, 0.389950, 0.389443, 0.389165, 0.389204, 0.389615, 0.390372, 0.391368, 0.392452, 0.393483,\
        0.394381, 0.395143, 0.395833, 0.396531, 0.397290, 0.398110, 0.398933, 0.399643, 0.400085, 0.400121,\
        0.399758, 0.399276, 0.399181, 0.400056, 0.402438, 0.406789, 0.413477, 0.422750, 0.434725, 0.449378,\
        0.466530, 0.485781, 0.506190, 0.526187, 0.543584, 0.556131, 0.563180, 0.565840, 0.566026, 0.565501,\
        0.565575, 0.567119, 0.570582, 0.575985, 0.582837, 0.590278, 0.598016, 0.607704, 0.618088, 0.628072,\
        0.637278, 0.645387, 0.652426, 0.658551, 0.663912, 0.668622, 0.672790, 0.676450, 0.679496, 0.681579,\
        0.682330, 0.682114, 0.681356, 0.680149, 0.678619, 0.676904, 0.675021, 0.672975, 0.670773, 0.668326,\
        0.665513, 0.662279, 0.658629, 0.654588, 0.650178, 0.645420, 0.640339, 0.634974, 0.629378, 0.623614,\
        0.617728, 0.611764, 0.605821, 0.600049, 0.594935, 0.590021, 0.584930, 0.579571, 0.573966, 0.568160,\
        0.562176, 0.556021, 0.549686, 0.543159, 0.536430, 0.529487, 0.522306, 0.514849, 0.507073, 0.498970,\
        0.490507],\
        [0.394588, 0.397171, 0.399045, 0.400455, 0.402277, 0.404178, 0.405895, 0.407273, 0.408016, 0.408268,\
        0.407609, 0.405585, 0.402818, 0.400402, 0.398977, 0.397647, 0.396514, 0.395937, 0.395382, 0.394552,\
        0.393523, 0.392526, 0.391686, 0.391080, 0.390854, 0.391127, 0.391876, 0.392938, 0.394079, 0.395094,\
        0.395874, 0.396436, 0.396907, 0.397428, 0.398081, 0.398869, 0.399704, 0.400416, 0.400762, 0.400462,\
        0.399373, 0.397827, 0.396563, 0.396335, 0.397791, 0.401491, 0.407894, 0.417322, 0.429940, 0.445772,\
        0.464758, 0.486692, 0.510676, 0.534585, 0.555248, 0.569302, 0.575649, 0.576527, 0.575007, 0.573229,\
        0.572588, 0.573998, 0.577904, 0.584346, 0.592851, 0.601743, 0.610766, 0.620815, 0.631379, 0.641383,\
        0.650392, 0.658284, 0.665084, 0.670920, 0.676006, 0.680382, 0.684239, 0.687673, 0.690626, 0.692780,\
        0.692871, 0.692066, 0.690968, 0.689327, 0.687363, 0.685372, 0.683218, 0.680921, 0.678616, 0.676097,\
        0.673128, 0.669664, 0.665766, 0.661488, 0.656850, 0.651866, 0.646556, 0.640953, 0.635110, 0.629089,\
        0.622954, 0.616780, 0.610681, 0.604862, 0.599571, 0.594435, 0.589072, 0.583405, 0.577527, 0.571509,\
        0.565372, 0.559108, 0.552700, 0.546126, 0.539367, 0.532405, 0.525215, 0.517774, 0.510055, 0.502026,\
        0.493601],\
        [0.403365, 0.406411, 0.408475, 0.410147, 0.411770, 0.413285, 0.414444, 0.415085, 0.415087, 0.414413,\
        0.412937, 0.410634, 0.407918, 0.405469, 0.403607, 0.401938, 0.400524, 0.399980, 0.399269, 0.398142,\
        0.396650, 0.395319, 0.394140, 0.393101, 0.392518, 0.392626, 0.393398, 0.394584, 0.395815, 0.396802,\
        0.397409, 0.397667, 0.397826, 0.398105, 0.398610, 0.399346, 0.400200, 0.400942, 0.401244, 0.400726,\
        0.398883, 0.396145, 0.393706, 0.392381, 0.392903, 0.395943, 0.402075, 0.411711, 0.425041, 0.442039,\
        0.462623, 0.486836, 0.514449, 0.543015, 0.567887, 0.583735, 0.588135, 0.586308, 0.582922, 0.579939,\
        0.578672, 0.580151, 0.584755, 0.592308, 0.602502, 0.613522, 0.624392, 0.635408, 0.646167, 0.655960,\
        0.664491, 0.671878, 0.678214, 0.683437, 0.688237, 0.692136, 0.695574, 0.698734, 0.701271, 0.703323,\
        0.702945, 0.701804, 0.700461, 0.698359, 0.695877, 0.693777, 0.691260, 0.688613, 0.686358, 0.683870,\
        0.680708, 0.676958, 0.672801, 0.668302, 0.663462, 0.658273, 0.652747, 0.646915, 0.640835, 0.634580,\
        0.628235, 0.621900, 0.615705, 0.609811, 0.604281, 0.598862, 0.593120, 0.587042, 0.580861, 0.574637,\
        0.568360, 0.561998, 0.555521, 0.548898, 0.542103, 0.535114, 0.527908, 0.520464, 0.512766, 0.504794,\
        0.496390],\
        [0.410371, 0.415016, 0.418039, 0.420245, 0.421929, 0.423115, 0.423697, 0.423593, 0.422778, 0.421279,\
        0.419145, 0.416515, 0.413708, 0.411105, 0.408884, 0.406978, 0.405478, 0.404449, 0.403374, 0.401904,\
        0.399997, 0.398507, 0.396961, 0.395299, 0.394248, 0.394162, 0.394985, 0.396348, 0.397675, 0.398564,\
        0.398949, 0.398764, 0.398545, 0.398526, 0.398832, 0.399478, 0.400329, 0.401078, 0.401266, 0.400291,\
        0.397660, 0.394081, 0.390671, 0.388355, 0.387980, 0.390366, 0.396258, 0.406220, 0.420489, 0.438882,\
        0.460981, 0.486975, 0.517856, 0.550701, 0.579536, 0.597777, 0.598519, 0.594134, 0.589563, 0.585710,\
        0.583926, 0.585836, 0.591753, 0.600974, 0.612916, 0.625870, 0.638561, 0.650732, 0.661920, 0.671414,\
        0.679166, 0.685762, 0.691688, 0.695727, 0.700325, 0.703928, 0.706971, 0.709745, 0.711853, 0.713055,\
        0.712786, 0.711534, 0.709727, 0.707285, 0.704495, 0.702194, 0.698886, 0.696035, 0.694101, 0.691690,\
        0.688155, 0.684040, 0.679647, 0.674975, 0.669969, 0.664600, 0.658867, 0.652807, 0.646488, 0.639998,\
        0.633439, 0.626919, 0.620553, 0.614439, 0.608587, 0.602831, 0.596682, 0.590255, 0.583859, 0.577495,\
        0.571119, 0.564685, 0.558151, 0.551483, 0.544651, 0.537628, 0.530388, 0.522909, 0.515161, 0.507094,\
        0.498643],\
        [0.416861, 0.422929, 0.427242, 0.430280, 0.432289, 0.433338, 0.433436, 0.432626, 0.431010, 0.428739,\
        0.425993, 0.422985, 0.419971, 0.417184, 0.414745, 0.412665, 0.410938, 0.409461, 0.407944, 0.406145,\
        0.404135, 0.402252, 0.400106, 0.397790, 0.396217, 0.395930, 0.396773, 0.398250, 0.399662, 0.400198,\
        0.400259, 0.399661, 0.399113, 0.398750, 0.398790, 0.399293, 0.400098, 0.400810, 0.400831, 0.399321,\
        0.396067, 0.391899, 0.387763, 0.384601, 0.383378, 0.385097, 0.390747, 0.401159, 0.416772, 0.437324,\
        0.461753, 0.489090, 0.522437, 0.557738, 0.587123, 0.604081, 0.605510, 0.600705, 0.595644, 0.591210,\
        0.588703, 0.591414, 0.599614, 0.611322, 0.625191, 0.639580, 0.653376, 0.666365, 0.678148, 0.687256,\
        0.694005, 0.699562, 0.704771, 0.708607, 0.712120, 0.715700, 0.718507, 0.720776, 0.722342, 0.722977,\
        0.722512, 0.721104, 0.718991, 0.716322, 0.713353, 0.710275, 0.706830, 0.703892, 0.701925, 0.699301,\
        0.695162, 0.690751, 0.686244, 0.681468, 0.676336, 0.670807, 0.664869, 0.658566, 0.651988, 0.645244,\
        0.638441, 0.631683, 0.625056, 0.618607, 0.612308, 0.606004, 0.599505, 0.592972, 0.586510, 0.580090,\
        0.573665, 0.567185, 0.560609, 0.553901, 0.547030, 0.539969, 0.532691, 0.525166, 0.517360, 0.509234,\
        0.500816],\
        [0.422970, 0.430406, 0.436065, 0.440120, 0.442671, 0.443764, 0.443484, 0.442010, 0.439608, 0.436583,\
        0.433231, 0.429818, 0.426564, 0.423619, 0.421047, 0.418831, 0.416899, 0.415109, 0.413266, 0.411218,\
        0.408968, 0.406556, 0.403868, 0.401120, 0.398934, 0.398476, 0.399219, 0.400327, 0.401510, 0.401824,\
        0.401482, 0.400688, 0.399755, 0.398977, 0.398668, 0.398958, 0.399677, 0.400300, 0.400071, 0.398191,\
        0.394652, 0.390165, 0.385529, 0.381670, 0.379643, 0.380639, 0.385935, 0.396760, 0.414011, 0.437741,\
        0.466513, 0.498001, 0.531981, 0.565825, 0.592772, 0.607809, 0.610825, 0.607206, 0.602624, 0.596974,\
        0.594292, 0.597194, 0.608737, 0.623493, 0.639373, 0.654813, 0.668744, 0.681363, 0.693859, 0.702540,\
        0.708518, 0.713453, 0.717845, 0.721490, 0.724662, 0.727599, 0.730013, 0.731800, 0.732848, 0.733019,\
        0.732246, 0.730606, 0.728256, 0.725370, 0.722140, 0.718711, 0.715233, 0.712083, 0.709277, 0.705941,\
        0.701615, 0.697134, 0.692641, 0.687770, 0.682530, 0.676857, 0.670698, 0.664119, 0.657250, 0.650222,\
        0.643148, 0.636115, 0.629181, 0.622365, 0.615626, 0.608874, 0.602079, 0.595429, 0.588937, 0.582498,\
        0.576045, 0.569532, 0.562920, 0.556175, 0.549267, 0.542170, 0.534857, 0.527303, 0.519482, 0.511385,\
        0.503042],\
        [0.428669, 0.437492, 0.444550, 0.449766, 0.453036, 0.454311, 0.453715, 0.451579, 0.448371, 0.444579,\
        0.440626, 0.436822, 0.433360, 0.430325, 0.427712, 0.425448, 0.423414, 0.421458, 0.419420, 0.417182,\
        0.414696, 0.411955, 0.408992, 0.406035, 0.403682, 0.402783, 0.402997, 0.403548, 0.404031, 0.403982,\
        0.403321, 0.402206, 0.400866, 0.399599, 0.398849, 0.398830, 0.399435, 0.399989, 0.399503, 0.397524,\
        0.394054, 0.389532, 0.384643, 0.380271, 0.377525, 0.377745, 0.382473, 0.393367, 0.411940, 0.438943,\
        0.473253, 0.509034, 0.542723, 0.574758, 0.599869, 0.612555, 0.616098, 0.614090, 0.609806, 0.605296,\
        0.604507, 0.609523, 0.621940, 0.638467, 0.655655, 0.671634, 0.684982, 0.695116, 0.706316, 0.715676,\
        0.721933, 0.726770, 0.730790, 0.734159, 0.737026, 0.739480, 0.741430, 0.742753, 0.743334, 0.743083,\
        0.741968, 0.740048, 0.737447, 0.734324, 0.730846, 0.727174, 0.723477, 0.719903, 0.716357, 0.712463,\
        0.708073, 0.703517, 0.698898, 0.693808, 0.688519, 0.682715, 0.676281, 0.669370, 0.662176, 0.654851,\
        0.647499, 0.640187, 0.632956, 0.625812, 0.618729, 0.611674, 0.604668, 0.597855, 0.591298, 0.584812,\
        0.578319, 0.571768, 0.565116, 0.558330, 0.551384, 0.544253, 0.536915, 0.529349, 0.521540, 0.513486,\
        0.505199],\
        [0.433861, 0.444115, 0.452666, 0.459212, 0.463384, 0.464947, 0.464033, 0.461168, 0.457085, 0.452499,\
        0.447966, 0.443834, 0.440258, 0.437247, 0.434718, 0.432532, 0.430529, 0.428547, 0.426443, 0.424111,\
        0.421503, 0.418632, 0.415599, 0.412667, 0.410290, 0.408868, 0.408274, 0.408048, 0.407787, 0.407182,\
        0.406116, 0.404648, 0.402942, 0.401248, 0.400055, 0.399508, 0.399941, 0.400406, 0.399898, 0.398070,\
        0.394888, 0.390611, 0.385767, 0.381167, 0.377934, 0.377499, 0.381572, 0.392096, 0.411115, 0.440011,\
        0.477167, 0.516430, 0.551756, 0.582337, 0.605214, 0.617364, 0.621789, 0.621581, 0.619408, 0.617855,\
        0.619733, 0.626902, 0.639673, 0.656138, 0.673458, 0.689339, 0.701823, 0.710856, 0.719525, 0.727936,\
        0.734407, 0.739336, 0.743241, 0.746405, 0.748993, 0.751079, 0.752627, 0.753539, 0.753708, 0.753063,\
        0.751592, 0.749357, 0.746473, 0.743082, 0.739339, 0.735392, 0.731371, 0.727344, 0.723252, 0.718947,\
        0.714383, 0.709651, 0.704710, 0.699528, 0.694325, 0.688315, 0.681488, 0.674183, 0.666659, 0.659056,\
        0.651450, 0.643891, 0.636408, 0.629007, 0.621682, 0.614434, 0.607299, 0.600375, 0.593676, 0.587089,\
        0.580529, 0.573923, 0.567220, 0.560387, 0.553399, 0.546235, 0.538876, 0.531307, 0.523517, 0.515501,\
        0.507260],\
        [0.438406, 0.450119, 0.460271, 0.468366, 0.473686, 0.475634, 0.474327, 0.470595, 0.465532, 0.460130,\
        0.455082, 0.450744, 0.447206, 0.444381, 0.442089, 0.440123, 0.438284, 0.436402, 0.434344, 0.432023,\
        0.429402, 0.426518, 0.423496, 0.420574, 0.418055, 0.416152, 0.414818, 0.413798, 0.412798, 0.411592,\
        0.410074, 0.408266, 0.406298, 0.404403, 0.402886, 0.402004, 0.401981, 0.402144, 0.401663, 0.400149,\
        0.397483, 0.393765, 0.389359, 0.384980, 0.381763, 0.381234, 0.385178, 0.395564, 0.414676, 0.445041,\
        0.486033, 0.527662, 0.563456, 0.590391, 0.610509, 0.622712, 0.628404, 0.630202, 0.630617, 0.632016,\
        0.636481, 0.645299, 0.658485, 0.674561, 0.691113, 0.705766, 0.717083, 0.725729, 0.733361, 0.740458,\
        0.746487, 0.751316, 0.755118, 0.758102, 0.760444, 0.762238, 0.763469, 0.764044, 0.763857, 0.762843,\
        0.761009, 0.758430, 0.755226, 0.751533, 0.747489, 0.743225, 0.738844, 0.734395, 0.729861, 0.725189,\
        0.720358, 0.715387, 0.710286, 0.705133, 0.699784, 0.693398, 0.686053, 0.678377, 0.670595, 0.662782,\
        0.654980, 0.647228, 0.639551, 0.631964, 0.624476, 0.617103, 0.609878, 0.602844, 0.596004, 0.589323,\
        0.582689, 0.576013, 0.569244, 0.562352, 0.555315, 0.548117, 0.540740, 0.533170, 0.525397, 0.517413,\
        0.509212],\
        [0.442163, 0.455294, 0.467086, 0.476958, 0.483800, 0.486237, 0.484406, 0.479645, 0.473508, 0.467316,\
        0.461881, 0.457528, 0.454233, 0.451784, 0.449894, 0.448280, 0.446714, 0.445026, 0.443102, 0.440870,\
        0.438305, 0.435450, 0.432431, 0.429444, 0.426706, 0.424354, 0.422382, 0.420649, 0.418967, 0.417180,\
        0.415209, 0.413070, 0.410874, 0.408809, 0.407109, 0.405985, 0.405481, 0.405241, 0.404752, 0.403648,\
        0.401727, 0.398933, 0.395450, 0.391896, 0.389501, 0.390051, 0.395538, 0.407811, 0.428891, 0.461950,\
        0.510955, 0.554586, 0.585014, 0.605901, 0.620749, 0.631019, 0.636864, 0.640054, 0.642591, 0.646378,\
        0.652901, 0.662929, 0.676355, 0.691900, 0.707033, 0.720099, 0.730642, 0.739063, 0.746222, 0.752600,\
        0.758166, 0.762791, 0.766437, 0.769203, 0.771292, 0.772844, 0.773839, 0.774157, 0.773669, 0.772304,\
        0.770089, 0.767140, 0.763591, 0.759568, 0.755195, 0.750590, 0.745844, 0.741005, 0.736080, 0.731049,\
        0.725901, 0.720639, 0.715276, 0.709809, 0.704055, 0.697278, 0.689614, 0.681820, 0.673946, 0.666024,\
        0.658096, 0.650207, 0.642392, 0.634677, 0.627081, 0.619623, 0.612322, 0.605190, 0.598251, 0.591513,\
        0.584798, 0.578035, 0.571184, 0.564222, 0.557130, 0.549894, 0.542498, 0.534927, 0.527170, 0.519215,\
        0.511050],\
        [0.445070, 0.459479, 0.472748, 0.484372, 0.493246, 0.496235, 0.493923, 0.488099, 0.480903, 0.474043,\
        0.468435, 0.464306, 0.461481, 0.459597, 0.458248, 0.457082, 0.455847, 0.454398, 0.452654, 0.450556,\
        0.448075, 0.445246, 0.442185, 0.439066, 0.436065, 0.433295, 0.430773, 0.428427, 0.426141, 0.423817,\
        0.421405, 0.418924, 0.416464, 0.414171, 0.412217, 0.410748, 0.409785, 0.409169, 0.408644, 0.407980,\
        0.406981, 0.405465, 0.403401, 0.401296, 0.400634, 0.403881, 0.413799, 0.432404, 0.461517, 0.506864,\
        0.568032, 0.597788, 0.614720, 0.627656, 0.636347, 0.642776, 0.647430, 0.651070, 0.654884, 0.660308,\
        0.668327, 0.679080, 0.692371, 0.707087, 0.721024, 0.733105, 0.743073, 0.751194, 0.758022, 0.763990,\
        0.769256, 0.773745, 0.777222, 0.779680, 0.781466, 0.782807, 0.783641, 0.783768, 0.783024, 0.781312,\
        0.778686, 0.775349, 0.771451, 0.767087, 0.762371, 0.757419, 0.752320, 0.747123, 0.741839, 0.736457,\
        0.730961, 0.725332, 0.719540, 0.713491, 0.706950, 0.699713, 0.692282, 0.684638, 0.676804, 0.668843,\
        0.660836, 0.652850, 0.644938, 0.637136, 0.629469, 0.621959, 0.614622, 0.607466, 0.600503, 0.593664,\
        0.586836, 0.579968, 0.573021, 0.565978, 0.558828, 0.551553, 0.544140, 0.536569, 0.528827, 0.520898,\
        0.512769],\
        [0.447243, 0.462785, 0.477223, 0.489974, 0.500042, 0.504415, 0.502407, 0.496024, 0.487964, 0.480641,\
        0.475100, 0.471413, 0.469234, 0.468042, 0.467305, 0.466605, 0.465684, 0.464450, 0.462888, 0.460947,\
        0.458559, 0.455725, 0.452566, 0.449260, 0.445974, 0.442820, 0.439829, 0.436964, 0.434155, 0.431341,\
        0.428498, 0.425648, 0.422856, 0.420226, 0.417877, 0.415914, 0.414386, 0.413276, 0.412537, 0.412146,\
        0.412039, 0.411989, 0.411682, 0.411394, 0.413054, 0.420426, 0.438001, 0.468825, 0.512775, 0.570727,\
        0.626248, 0.644358, 0.647834, 0.651052, 0.653709, 0.656405, 0.659562, 0.663112, 0.667327, 0.673507,\
        0.682807, 0.693906, 0.707015, 0.720960, 0.734120, 0.745516, 0.754911, 0.762577, 0.769009, 0.774608,\
        0.779633, 0.784146, 0.787529, 0.789499, 0.790907, 0.792074, 0.792793, 0.792750, 0.791773, 0.789710,\
        0.786606, 0.782908, 0.778703, 0.774005, 0.768942, 0.763654, 0.758232, 0.752718, 0.747113, 0.741402,\
        0.735556, 0.729537, 0.723288, 0.716708, 0.709675, 0.702232, 0.694756, 0.687185, 0.679357, 0.671342,\
        0.663257, 0.655192, 0.647206, 0.639339, 0.631621, 0.624077, 0.616724, 0.609566, 0.602570, 0.595651,\
        0.588745, 0.581768, 0.574716, 0.567593, 0.560387, 0.553080, 0.545653, 0.538086, 0.530361, 0.522460,\
        0.514367],\
        [0.448995, 0.465745, 0.481399, 0.495158, 0.506205, 0.511993, 0.511021, 0.504606, 0.495849, 0.488040,\
        0.482685, 0.479423, 0.477901, 0.477385, 0.477204, 0.476896, 0.476165, 0.475042, 0.473635, 0.471877,\
        0.469585, 0.466696, 0.463375, 0.459843, 0.456270, 0.452775, 0.449398, 0.446108, 0.442852, 0.439593,\
        0.436328, 0.433079, 0.429886, 0.426802, 0.423894, 0.421240, 0.418913, 0.416988, 0.415578, 0.414889,\
        0.415165, 0.416317, 0.417664, 0.418879, 0.422558, 0.434911, 0.461994, 0.510466, 0.571927, 0.626876,\
        0.666517, 0.676385, 0.675467, 0.672934, 0.670707, 0.670364, 0.672689, 0.676379, 0.680328, 0.685585,\
        0.696218, 0.708223, 0.721338, 0.734858, 0.747357, 0.758044, 0.766690, 0.773637, 0.779526, 0.784669,\
        0.789215, 0.793814, 0.797441, 0.798644, 0.799724, 0.800701, 0.801206, 0.800905, 0.799667, 0.797240,\
        0.793579, 0.789710, 0.785286, 0.780244, 0.774835, 0.769245, 0.763555, 0.757782, 0.751914, 0.745918,\
        0.739758, 0.733389, 0.726758, 0.719809, 0.712520, 0.704977, 0.697389, 0.689727, 0.681749, 0.673597,\
        0.665412, 0.657269, 0.649217, 0.641294, 0.633529, 0.625949, 0.618570, 0.611398, 0.604389, 0.597465,\
        0.590478, 0.583370, 0.576219, 0.569030, 0.561784, 0.554457, 0.547027, 0.539470, 0.531767, 0.523897,\
        0.515841],\
        [0.450686, 0.469029, 0.486637, 0.502535, 0.515447, 0.522573, 0.522530, 0.516363, 0.506848, 0.498167,\
        0.492490, 0.489022, 0.487944, 0.487839, 0.487952, 0.487903, 0.487119, 0.485933, 0.484651, 0.483137,\
        0.480965, 0.477922, 0.474386, 0.470624, 0.466777, 0.463006, 0.459342, 0.455727, 0.452098, 0.448434,\
        0.444759, 0.441101, 0.437460, 0.433827, 0.430204, 0.426633, 0.423190, 0.419984, 0.417171, 0.415072,\
        0.414363, 0.415463, 0.417344, 0.418372, 0.421593, 0.440204, 0.482463, 0.545359, 0.613710, 0.670147,\
        0.704600, 0.703340, 0.698912, 0.692817, 0.686602, 0.683779, 0.686719, 0.691459, 0.695475, 0.698977,\
        0.707507, 0.722163, 0.736267, 0.749515, 0.761330, 0.771221, 0.778885, 0.784714, 0.789942, 0.794690,\
        0.798444, 0.802451, 0.806040, 0.807468, 0.808300, 0.808829, 0.808841, 0.808065, 0.806356, 0.803389,\
        0.799599, 0.795848, 0.791156, 0.785691, 0.779970, 0.774157, 0.768286, 0.762341, 0.756283, 0.750069,\
        0.743656, 0.737004, 0.730077, 0.722863, 0.715404, 0.707817, 0.700201, 0.692270, 0.683948, 0.675621,\
        0.667336, 0.659121, 0.651008, 0.643026, 0.635204, 0.627565, 0.620128, 0.612911, 0.605939, 0.599111,\
        0.591894, 0.584674, 0.577478, 0.570263, 0.563004, 0.555677, 0.548256, 0.540719, 0.533043, 0.525208,\
        0.517193],\
        [0.452514, 0.472830, 0.493541, 0.512830, 0.528411, 0.537240, 0.537967, 0.531945, 0.522336, 0.512847,\
        0.505467, 0.500906, 0.499688, 0.499346, 0.499219, 0.499333, 0.498174, 0.496864, 0.495662, 0.494448,\
        0.492463, 0.489038, 0.485348, 0.481424, 0.477301, 0.473366, 0.469549, 0.465722, 0.461793, 0.457753,\
        0.453691, 0.449650, 0.445570, 0.441345, 0.436887, 0.432181, 0.427295, 0.422339, 0.417387, 0.412471,\
        0.408274, 0.406385, 0.404791, 0.401103, 0.396776, 0.418009, 0.494944, 0.569384, 0.622609, 0.684827,\
        0.734861, 0.735229, 0.723030, 0.712107, 0.701781, 0.697502, 0.702208, 0.707969, 0.712903, 0.717976,\
        0.726623, 0.739411, 0.752966, 0.765393, 0.776181, 0.785233, 0.791890, 0.796177, 0.800592, 0.805024,\
        0.808482, 0.811645, 0.814390, 0.815894, 0.816494, 0.816496, 0.815873, 0.814480, 0.812190, 0.808970,\
        0.805196, 0.801247, 0.795964, 0.790175, 0.784293, 0.778398, 0.772469, 0.766451, 0.760288, 0.753931,\
        0.747337, 0.740470, 0.733311, 0.725863, 0.718179, 0.710359, 0.702474, 0.694175, 0.685800, 0.677424,\
        0.669083, 0.660809, 0.652632, 0.644580, 0.636681, 0.628955, 0.621413, 0.614051, 0.606827, 0.599674,\
        0.592709, 0.585637, 0.578486, 0.571290, 0.564047, 0.556737, 0.549340, 0.541832, 0.534191, 0.526396,\
        0.518424],\
        [0.454611, 0.476870, 0.500719, 0.524831, 0.544425, 0.555445, 0.556453, 0.549891, 0.539767, 0.529461,\
        0.520981, 0.515383, 0.512632, 0.511353, 0.510693, 0.510180, 0.509089, 0.507806, 0.506601, 0.505374,\
        0.503311, 0.499602, 0.496136, 0.492080, 0.487592, 0.483782, 0.479929, 0.476027, 0.471870, 0.467450,\
        0.463042, 0.458714, 0.454290, 0.449515, 0.444171, 0.438164, 0.431551, 0.424491, 0.417050, 0.408842,\
        0.398971, 0.389276, 0.376626, 0.359470, 0.344816, 0.361165, 0.485144, 0.580157, 0.619312, 0.707281,\
        0.791391, 0.770932, 0.749502, 0.734099, 0.722302, 0.717466, 0.720430, 0.726129, 0.732332, 0.739200,\
        0.748127, 0.759410, 0.771500, 0.782425, 0.791398, 0.799580, 0.805282, 0.808880, 0.812291, 0.815880,\
        0.818869, 0.821324, 0.823183, 0.824177, 0.824318, 0.823728, 0.822432, 0.820383, 0.817538, 0.813949,\
        0.809787, 0.805040, 0.799534, 0.793728, 0.787879, 0.782061, 0.776198, 0.770204, 0.764016, 0.757589,\
        0.750884, 0.743869, 0.736527, 0.728862, 0.720899, 0.712664, 0.704165, 0.695846, 0.687538, 0.679144,\
        0.670755, 0.662416, 0.654160, 0.646019, 0.638020, 0.630183, 0.622518, 0.615015, 0.607642, 0.600461,\
        0.593484, 0.586425, 0.579301, 0.572138, 0.564927, 0.557650, 0.550287, 0.542816, 0.535215, 0.527463,\
        0.519538],\
        [0.457304, 0.481588, 0.508991, 0.537899, 0.562159, 0.575569, 0.575579, 0.567421, 0.556449, 0.545601,\
        0.536512, 0.529958, 0.525859, 0.523468, 0.522025, 0.520940, 0.519797, 0.518565, 0.517284, 0.515740,\
        0.513404, 0.510131, 0.506622, 0.502090, 0.497628, 0.494423, 0.490296, 0.486638, 0.482281, 0.477386,\
        0.472741, 0.468337, 0.463776, 0.458602, 0.452429, 0.445046, 0.436477, 0.426999, 0.416951, 0.405865,\
        0.391556, 0.372914, 0.349477, 0.322858, 0.305517, 0.330018, 0.456967, 0.572189, 0.627917, 0.762625,\
        0.856764, 0.818344, 0.782742, 0.762340, 0.748208, 0.741204, 0.741529, 0.746405, 0.753196, 0.760851,\
        0.769607, 0.779669, 0.790311, 0.799920, 0.806779, 0.813285, 0.818297, 0.821756, 0.824606, 0.827243,\
        0.829460, 0.831122, 0.832139, 0.832400, 0.831876, 0.830602, 0.828605, 0.825884, 0.822447, 0.818341,\
        0.813632, 0.808345, 0.802579, 0.796666, 0.790956, 0.785335, 0.779621, 0.773713, 0.767560, 0.761125,\
        0.754375, 0.747280, 0.739818, 0.731984, 0.723798, 0.715321, 0.706701, 0.698096, 0.689498, 0.680968,\
        0.672487, 0.664047, 0.655677, 0.647415, 0.639289, 0.631323, 0.623531, 0.615928, 0.608558, 0.601445,\
        0.594255, 0.587088, 0.579969, 0.572841, 0.565669, 0.558432, 0.551108, 0.543679, 0.536122, 0.528416,\
        0.520540],\
        [0.460922, 0.487098, 0.517822, 0.550296, 0.578360, 0.594229, 0.590878, 0.581274, 0.570190, 0.559484,\
        0.550333, 0.543307, 0.538396, 0.535166, 0.533049, 0.531532, 0.530246, 0.528995, 0.527631, 0.525920,\
        0.523569, 0.520501, 0.516817, 0.512493, 0.508498, 0.504854, 0.500447, 0.497725, 0.492757, 0.487356,\
        0.482796, 0.478650, 0.474241, 0.468930, 0.462125, 0.453464, 0.442855, 0.430582, 0.417482, 0.404276,\
        0.387445, 0.365703, 0.338071, 0.308482, 0.293686, 0.325708, 0.434345, 0.556468, 0.652701, 0.793828,\
        0.881615, 0.860433, 0.820674, 0.793265, 0.773812, 0.763942, 0.762417, 0.766866, 0.774019, 0.781686,\
        0.789701, 0.798423, 0.807402, 0.815408, 0.821645, 0.826864, 0.831089, 0.834242, 0.836663, 0.838595,\
        0.840026, 0.840867, 0.841040, 0.840487, 0.839190, 0.837166, 0.834440, 0.831036, 0.826986, 0.822334,\
        0.817136, 0.811453, 0.805420, 0.799398, 0.793876, 0.788467, 0.782898, 0.777082, 0.770993, 0.764604,\
        0.757878, 0.750778, 0.743271, 0.735343, 0.727011, 0.718342, 0.709463, 0.700541, 0.691692, 0.683028,\
        0.674410, 0.665805, 0.657265, 0.648838, 0.640557, 0.632438, 0.624495, 0.616743, 0.609201, 0.601865,\
        0.594670, 0.587579, 0.580519, 0.573436, 0.566303, 0.559103, 0.551819, 0.544432, 0.536920, 0.529262,\
        0.521436],\
        [0.466124, 0.493070, 0.526074, 0.559866, 0.587421, 0.600865, 0.597615, 0.589774, 0.580051, 0.570350,\
        0.561761, 0.554831, 0.549654, 0.546001, 0.543487, 0.541712, 0.540333, 0.539086, 0.537740, 0.536052,\
        0.533810, 0.530917, 0.527411, 0.523485, 0.519536, 0.515578, 0.511697, 0.508067, 0.502865, 0.497555,\
        0.493552, 0.489887, 0.485864, 0.480754, 0.473659, 0.464125, 0.451845, 0.436795, 0.419756, 0.402819,\
        0.388799, 0.371999, 0.346762, 0.316970, 0.299879, 0.327617, 0.427395, 0.557722, 0.683724, 0.811013,\
        0.898134, 0.877514, 0.845201, 0.815461, 0.793957, 0.782078, 0.779915, 0.784874, 0.792956, 0.800110,\
        0.807313, 0.814831, 0.822452, 0.829346, 0.835062, 0.839709, 0.843406, 0.846203, 0.848230, 0.849601,\
        0.850331, 0.850385, 0.849726, 0.848336, 0.846224, 0.843415, 0.839946, 0.835862, 0.831209, 0.826049,\
        0.820454, 0.814526, 0.808430, 0.802455, 0.796955, 0.791642, 0.786114, 0.780356, 0.774351, 0.768060,\
        0.761434, 0.754416, 0.746956, 0.739023, 0.730619, 0.721792, 0.712640, 0.703326, 0.694281, 0.685510,\
        0.676644, 0.667764, 0.658983, 0.650349, 0.641881, 0.633586, 0.625471, 0.617539, 0.609787, 0.602230,\
        0.595034, 0.588040, 0.581029, 0.573969, 0.566856, 0.559683, 0.552433, 0.545085, 0.537618, 0.530008,\
        0.522232],\
        [0.473513, 0.503082, 0.535574, 0.567530, 0.591779, 0.602397, 0.601193, 0.594755, 0.586767, 0.578452,\
        0.570823, 0.564419, 0.559411, 0.555719, 0.553110, 0.551286, 0.549948, 0.548816, 0.547628, 0.546129,\
        0.544116, 0.541483, 0.538260, 0.534614, 0.530767, 0.526843, 0.522900, 0.518737, 0.513976, 0.509292,\
        0.505590, 0.502217, 0.498618, 0.494065, 0.487005, 0.477432, 0.464598, 0.448072, 0.429140, 0.411157,\
        0.401016, 0.392557, 0.373335, 0.345477, 0.320628, 0.329291, 0.433924, 0.570908, 0.705795, 0.810921,\
        0.900828, 0.881563, 0.856852, 0.829270, 0.805837, 0.792738, 0.792860, 0.797831, 0.806941, 0.814489,\
        0.821683, 0.828816, 0.835726, 0.841986, 0.847298, 0.851619, 0.855003, 0.857509, 0.859195, 0.860097,\
        0.860212, 0.859532, 0.858066, 0.855844, 0.852908, 0.849309, 0.845102, 0.840348, 0.835118, 0.829492,\
        0.823573, 0.817492, 0.811432, 0.805620, 0.800226, 0.794773, 0.789185, 0.783485, 0.777607, 0.771488,\
        0.765054, 0.758223, 0.750919, 0.743084, 0.734700, 0.725805, 0.716506, 0.707006, 0.697756, 0.688626,\
        0.679212, 0.669935, 0.660863, 0.651993, 0.643314, 0.634823, 0.626525, 0.618435, 0.610560, 0.602905,\
        0.595678, 0.588645, 0.581564, 0.574463, 0.567341, 0.560181, 0.552958, 0.545646, 0.538222, 0.530659,\
        0.522932],\
        [0.479623, 0.511391, 0.543949, 0.574442, 0.596841, 0.606008, 0.604701, 0.598760, 0.591717, 0.584535,\
        0.577949, 0.572272, 0.567697, 0.564245, 0.561790, 0.560121, 0.558987, 0.558121, 0.557248, 0.556095,\
        0.554433, 0.552130, 0.549201, 0.545792, 0.542100, 0.538274, 0.534360, 0.530292, 0.526076, 0.522069,\
        0.518601, 0.515453, 0.512146, 0.507924, 0.501363, 0.493081, 0.481027, 0.464653, 0.447005, 0.432054,\
        0.422825, 0.416513, 0.407403, 0.386706, 0.366109, 0.344878, 0.449908, 0.589133, 0.726892, 0.822712,\
        0.902819, 0.886435, 0.865821, 0.839507, 0.813395, 0.798741, 0.802776, 0.809156, 0.817330, 0.825747,\
        0.833504, 0.840762, 0.847477, 0.853447, 0.858506, 0.862612, 0.865790, 0.868075, 0.869485, 0.869996,\
        0.869564, 0.868193, 0.865946, 0.862908, 0.859162, 0.854786, 0.849856, 0.844453, 0.838664, 0.832596,\
        0.826371, 0.820138, 0.814067, 0.808308, 0.802841, 0.797382, 0.791908, 0.786359, 0.780693, 0.774847,\
        0.768723, 0.762205, 0.755181, 0.747561, 0.739298, 0.730414, 0.721039, 0.711461, 0.702057, 0.691893,\
        0.681912, 0.672282, 0.662928, 0.653810, 0.644899, 0.636186, 0.627684, 0.619432, 0.611507, 0.604011,\
        0.596740, 0.589336, 0.582070, 0.574896, 0.567753, 0.560599, 0.553398, 0.546121, 0.538738, 0.531220,\
        0.523542],\
        [0.483107, 0.515648, 0.549869, 0.579961, 0.601072, 0.609403, 0.608352, 0.602476, 0.595466, 0.589351,\
        0.583697, 0.578721, 0.574670, 0.571617, 0.569492, 0.568143, 0.567363, 0.566911, 0.566509, 0.565857,\
        0.564666, 0.562749, 0.560105, 0.556909, 0.553396, 0.549750, 0.546053, 0.542319, 0.538609, 0.535096,\
        0.531907, 0.528920, 0.525750, 0.521754, 0.516321, 0.509277, 0.498425, 0.483233, 0.467455, 0.454356,\
        0.445921, 0.440961, 0.435555, 0.428503, 0.420596, 0.413091, 0.501664, 0.629741, 0.737490, 0.814083,\
        0.898905, 0.892594, 0.875601, 0.851726, 0.827932, 0.814223, 0.814283, 0.820107, 0.827761, 0.835988,\
        0.843956, 0.851375, 0.858088, 0.863932, 0.868813, 0.872743, 0.875749, 0.877843, 0.879037, 0.879239,\
        0.878309, 0.876268, 0.873257, 0.869426, 0.864899, 0.859775, 0.854148, 0.848113, 0.841778, 0.835265,\
        0.828713, 0.822272, 0.816089, 0.810262, 0.804761, 0.799454, 0.794182, 0.788862, 0.783519, 0.778078,\
        0.772404, 0.766345, 0.759746, 0.752471, 0.744428, 0.735597, 0.726056, 0.715960, 0.705390, 0.694864,\
        0.684759, 0.674865, 0.665237, 0.655855, 0.646679, 0.637698, 0.628936, 0.620440, 0.612282, 0.604562,\
        0.597122, 0.589700, 0.582421, 0.575228, 0.568078, 0.560933, 0.553756, 0.546513, 0.539170, 0.531697,\
        0.524066],\
        [0.487813, 0.520815, 0.555142, 0.584782, 0.603995, 0.612186, 0.611498, 0.605744, 0.599265, 0.593745,\
        0.588526, 0.584049, 0.580509, 0.577931, 0.576242, 0.575319, 0.575004, 0.575081, 0.575284, 0.575285,\
        0.574699, 0.573216, 0.570828, 0.567809, 0.564493, 0.561118, 0.557777, 0.554469, 0.551210, 0.548066,\
        0.545090, 0.542207, 0.539133, 0.535385, 0.530416, 0.523517, 0.513518, 0.500444, 0.486461, 0.474380,\
        0.466526, 0.463307, 0.462729, 0.461808, 0.455235, 0.462377, 0.545989, 0.652756, 0.732581, 0.806105,\
        0.873392, 0.891422, 0.882467, 0.863707, 0.844484, 0.832063, 0.828924, 0.832163, 0.838505, 0.846076,\
        0.853842, 0.861249, 0.867956, 0.873689, 0.878344, 0.882060, 0.884893, 0.886753, 0.887771, 0.887775,\
        0.886362, 0.883639, 0.879880, 0.875297, 0.870036, 0.864208, 0.857916, 0.851269, 0.844392, 0.837421,\
        0.830503, 0.823788, 0.817413, 0.811475, 0.806007, 0.800923, 0.795848, 0.790861, 0.785993, 0.781113,\
        0.776050, 0.770611, 0.764597, 0.757816, 0.750115, 0.741406, 0.731702, 0.721127, 0.709967, 0.698843,\
        0.688196, 0.677837, 0.667884, 0.658194, 0.648693, 0.639379, 0.630288, 0.621466, 0.612942, 0.604670,\
        0.596994, 0.589866, 0.582651, 0.575461, 0.568314, 0.561183, 0.554032, 0.546824, 0.539521, 0.532093,\
        0.524507],\
        [0.492958, 0.525931, 0.559217, 0.588579, 0.607337, 0.614526, 0.614322, 0.609609, 0.603527, 0.597632,\
        0.592494, 0.588378, 0.585362, 0.583313, 0.582108, 0.581657, 0.581854, 0.582519, 0.583403, 0.584190,\
        0.584373, 0.583368, 0.581172, 0.578297, 0.575205, 0.572209, 0.569350, 0.566539, 0.563702, 0.560835,\
        0.557982, 0.555150, 0.552152, 0.548545, 0.543691, 0.536824, 0.527301, 0.515375, 0.502523, 0.490766,\
        0.483168, 0.482365, 0.485698, 0.492894, 0.492350, 0.497774, 0.585265, 0.664807, 0.732781, 0.806294,\
        0.861309, 0.883964, 0.882757, 0.870368, 0.856093, 0.845931, 0.842193, 0.843965, 0.849170, 0.856043,\
        0.863436, 0.870696, 0.877369, 0.882969, 0.887210, 0.890559, 0.893297, 0.894731, 0.895528, 0.895537,\
        0.893549, 0.890143, 0.885686, 0.880426, 0.874506, 0.868033, 0.861115, 0.853876, 0.846458, 0.839014,\
        0.831698, 0.824659, 0.818030, 0.811915, 0.806375, 0.801360, 0.796682, 0.792268, 0.788060, 0.783906,\
        0.779609, 0.774952, 0.769694, 0.763582, 0.756383, 0.747927, 0.738173, 0.727256, 0.715560, 0.703706,\
        0.692033, 0.681276, 0.670973, 0.660886, 0.650960, 0.641222, 0.631736, 0.622570, 0.613801, 0.605573,\
        0.597889, 0.590235, 0.582847, 0.575615, 0.568466, 0.561353, 0.554230, 0.547056, 0.539794, 0.532409,\
        0.524867],\
        [0.494620, 0.527866, 0.560497, 0.588151, 0.606718, 0.615342, 0.616214, 0.612347, 0.606436, 0.600455,\
        0.595349, 0.591749, 0.589401, 0.587940, 0.587212, 0.587202, 0.587902, 0.589147, 0.590657, 0.592271,\
        0.593414, 0.592920, 0.590843, 0.588165, 0.585355, 0.582908, 0.580689, 0.578452, 0.576045, 0.573388,\
        0.570558, 0.567736, 0.564856, 0.561450, 0.556795, 0.550125, 0.540858, 0.529098, 0.515962, 0.503199,\
        0.495862, 0.499278, 0.504792, 0.515513, 0.524158, 0.544581, 0.605206, 0.662181, 0.720801, 0.796347,\
        0.846868, 0.870044, 0.874676, 0.868827, 0.860176, 0.853792, 0.851776, 0.853961, 0.859073, 0.865684,\
        0.872757, 0.879783, 0.886434, 0.892002, 0.895620, 0.898084, 0.900973, 0.901906, 0.902281, 0.902013,\
        0.899564, 0.895596, 0.890572, 0.884760, 0.878279, 0.871228, 0.863721, 0.855908, 0.847953, 0.840028,\
        0.832298, 0.824916, 0.818009, 0.811678, 0.806005, 0.801104, 0.796880, 0.793164, 0.789747, 0.786433,\
        0.783025, 0.779291, 0.774959, 0.769710, 0.763211, 0.755184, 0.745511, 0.734312, 0.721959, 0.709015,\
        0.696705, 0.685448, 0.674605, 0.663939, 0.653435, 0.643164, 0.633215, 0.623668, 0.614589, 0.606045,\
        0.598018, 0.590360, 0.582936, 0.575685, 0.568539, 0.561444, 0.554350, 0.547213, 0.539990, 0.532647,\
        0.525148],\
        [0.494973, 0.527983, 0.560253, 0.587505, 0.606194, 0.615435, 0.617124, 0.613838, 0.607997, 0.601731,\
        0.597021, 0.594408, 0.592932, 0.592062, 0.591734, 0.592054, 0.593177, 0.595042, 0.596861, 0.599260,\
        0.601045, 0.601098, 0.599569, 0.597244, 0.594922, 0.593258, 0.591822, 0.590194, 0.588272, 0.585805,\
        0.582892, 0.580073, 0.577447, 0.574451, 0.570228, 0.564066, 0.555204, 0.543432, 0.529871, 0.517065,\
        0.511605, 0.515698, 0.523666, 0.534892, 0.549203, 0.573313, 0.609762, 0.645529, 0.687474, 0.753888,\
        0.812339, 0.844615, 0.857034, 0.858364, 0.856449, 0.855584, 0.857330, 0.861720, 0.867997, 0.875029,\
        0.881913, 0.888477, 0.895032, 0.900518, 0.903827, 0.905730, 0.907643, 0.908486, 0.908342, 0.907126,\
        0.904300, 0.899944, 0.894564, 0.888325, 0.881383, 0.873809, 0.865737, 0.857356, 0.848867, 0.840465,\
        0.832331, 0.824624, 0.817476, 0.810987, 0.805222, 0.800307, 0.796707, 0.793747, 0.791133, 0.788686,\
        0.786220, 0.783506, 0.780250, 0.776069, 0.770501, 0.763107, 0.753646, 0.742234, 0.729363, 0.715843,\
        0.702679, 0.690439, 0.678701, 0.667219, 0.655965, 0.645053, 0.634592, 0.624644, 0.615227, 0.606315,\
        0.598009, 0.590357, 0.582934, 0.575675, 0.568535, 0.561459, 0.554395, 0.547293, 0.540110, 0.532806,\
        0.525347],\
        [0.494342, 0.526993, 0.559275, 0.587017, 0.605899, 0.615187, 0.617318, 0.614553, 0.608958, 0.602648,\
        0.598433, 0.597038, 0.596353, 0.595930, 0.595904, 0.596429, 0.597787, 0.600237, 0.602573, 0.605106,\
        0.607265, 0.607995, 0.607407, 0.605888, 0.604460, 0.603524, 0.602793, 0.601712, 0.600350, 0.598130,\
        0.595182, 0.592446, 0.590267, 0.587914, 0.584177, 0.578909, 0.570909, 0.559353, 0.546194, 0.535174,\
        0.530566, 0.533462, 0.541446, 0.552790, 0.567192, 0.585637, 0.607335, 0.630538, 0.660503, 0.708195,\
        0.766508, 0.810695, 0.831043, 0.840195, 0.846513, 0.853007, 0.860111, 0.867939, 0.876405, 0.884556,\
        0.891441, 0.897160, 0.903127, 0.908258, 0.911430, 0.913213, 0.914260, 0.914451, 0.913624, 0.911634,\
        0.908272, 0.903582, 0.897791, 0.891232, 0.883913, 0.875822, 0.867172, 0.858216, 0.849196, 0.840335,\
        0.831827, 0.823845, 0.816539, 0.810044, 0.804472, 0.799976, 0.796658, 0.794213, 0.792278, 0.790631,\
        0.789082, 0.787415, 0.785338, 0.782423, 0.778053, 0.771522, 0.762370, 0.750718, 0.737252, 0.722975,\
        0.708898, 0.695528, 0.682845, 0.670392, 0.658247, 0.646647, 0.635686, 0.625384, 0.615741, 0.606761,\
        0.598399, 0.590446, 0.582890, 0.575591, 0.568451, 0.561396, 0.554361, 0.547295, 0.540149, 0.532885,\
        0.525464],\
        [0.492511, 0.524584, 0.556809, 0.585421, 0.604648, 0.613858, 0.616846, 0.614863, 0.610003, 0.604488,\
        0.600737, 0.600091, 0.599702, 0.599573, 0.599910, 0.600741, 0.602462, 0.605107, 0.608039, 0.610836,\
        0.613188, 0.614569, 0.614922, 0.614578, 0.614101, 0.613797, 0.613528, 0.613003, 0.612051, 0.610329,\
        0.607981, 0.605711, 0.603966, 0.602104, 0.598518, 0.594249, 0.587517, 0.575787, 0.562700, 0.552464,\
        0.548303, 0.550717, 0.558079, 0.568533, 0.580883, 0.593808, 0.606793, 0.622578, 0.645281, 0.679745,\
        0.727969, 0.772841, 0.800006, 0.817765, 0.834089, 0.849697, 0.862868, 0.874153, 0.885113, 0.894936,\
        0.901926, 0.907302, 0.912125, 0.916204, 0.918826, 0.920135, 0.920459, 0.919876, 0.918312, 0.915647,\
        0.911775, 0.906673, 0.900508, 0.893790, 0.885993, 0.877288, 0.868011, 0.858465, 0.848929, 0.839641,\
        0.830809, 0.822613, 0.815225, 0.808825, 0.803584, 0.799548, 0.796571, 0.794533, 0.793157, 0.792201,\
        0.791471, 0.790782, 0.789891, 0.788393, 0.785528, 0.780147, 0.771338, 0.759248, 0.744886, 0.729521,\
        0.714330, 0.700098, 0.686613, 0.672882, 0.659836, 0.647646, 0.636290, 0.625726, 0.615915, 0.606824,\
        0.598347, 0.590331, 0.582720, 0.575406, 0.568278, 0.561247, 0.554244, 0.547213, 0.540105, 0.532878,\
        0.525493],\
        [0.489251, 0.520491, 0.552450, 0.580439, 0.600225, 0.611799, 0.615906, 0.614736, 0.610824, 0.606449,\
        0.603505, 0.602521, 0.602431, 0.602812, 0.603701, 0.605160, 0.607355, 0.610229, 0.613410, 0.616499,\
        0.619173, 0.621154, 0.622387, 0.623062, 0.623480, 0.623829, 0.624076, 0.624072, 0.623660, 0.622729,\
        0.621414, 0.620064, 0.618780, 0.616916, 0.613669, 0.609108, 0.601141, 0.590615, 0.576775, 0.566825,\
        0.563311, 0.565968, 0.572888, 0.581896, 0.591795, 0.601266, 0.608983, 0.620361, 0.639191, 0.668173,\
        0.706292, 0.743367, 0.772443, 0.797358, 0.825543, 0.851227, 0.869785, 0.882614, 0.894534, 0.905891,\
        0.913160, 0.918170, 0.921899, 0.924599, 0.926152, 0.926600, 0.926093, 0.924704, 0.922409, 0.919137,\
        0.914834, 0.909514, 0.903354, 0.896096, 0.887534, 0.878130, 0.868188, 0.858056, 0.848035, 0.838377,\
        0.829292, 0.820957, 0.813539, 0.807229, 0.802267, 0.798659, 0.796161, 0.794586, 0.793711, 0.793327,\
        0.793262, 0.793371, 0.793505, 0.793407, 0.792373, 0.788575, 0.780041, 0.767176, 0.751571, 0.734750,\
        0.718125, 0.703189, 0.688298, 0.673756, 0.660281, 0.647803, 0.636251, 0.625550, 0.615633, 0.606441,\
        0.597910, 0.589938, 0.582375, 0.575097, 0.568001, 0.561003, 0.554036, 0.547041, 0.539969, 0.532778,\
        0.525427],\
        [0.485832, 0.516141, 0.547958, 0.576416, 0.597981, 0.610260, 0.614514, 0.613730, 0.610639, 0.607297,\
        0.605050, 0.604284, 0.604609, 0.605638, 0.607237, 0.609392, 0.612119, 0.615327, 0.618773, 0.622159,\
        0.625221, 0.627775, 0.629771, 0.631299, 0.632513, 0.633525, 0.634344, 0.634914, 0.635177, 0.635134,\
        0.634885, 0.634536, 0.633926, 0.632431, 0.629193, 0.623337, 0.614113, 0.601487, 0.587197, 0.577966,\
        0.575438, 0.578860, 0.585638, 0.592700, 0.598808, 0.604573, 0.610443, 0.620162, 0.637258, 0.663291,\
        0.695592, 0.726513, 0.754747, 0.786536, 0.827319, 0.863874, 0.885810, 0.897493, 0.908015, 0.918344,\
        0.925290, 0.929413, 0.931792, 0.932981, 0.933185, 0.932510, 0.931043, 0.928817, 0.925807, 0.921956,\
        0.917227, 0.911620, 0.905048, 0.897275, 0.888288, 0.878186, 0.867593, 0.856919, 0.846479, 0.836534,\
        0.827297, 0.818933, 0.811575, 0.805357, 0.800559, 0.797451, 0.795484, 0.794375, 0.793937, 0.794002,\
        0.794421, 0.795074, 0.795874, 0.796769, 0.797570, 0.796136, 0.787555, 0.773671, 0.756738, 0.738323,\
        0.719690, 0.701876, 0.686939, 0.672900, 0.659535, 0.647072, 0.635532, 0.624850, 0.614939, 0.605734,\
        0.597234, 0.589358, 0.581882, 0.574665, 0.567614, 0.560655, 0.553723, 0.546765, 0.539730, 0.532574,\
        0.525256],\
        [0.484386, 0.513572, 0.544186, 0.572598, 0.595202, 0.607916, 0.611793, 0.611146, 0.608870, 0.606587,\
        0.605284, 0.605267, 0.606321, 0.608133, 0.610513, 0.613375, 0.616666, 0.620286, 0.624063, 0.627785,\
        0.631260, 0.634351, 0.637006, 0.639256, 0.641173, 0.642821, 0.644224, 0.645388, 0.646337, 0.647137,\
        0.647881, 0.648571, 0.648928, 0.648247, 0.645392, 0.639000, 0.628067, 0.612796, 0.597271, 0.588206,\
        0.585803, 0.589769, 0.596265, 0.601065, 0.603589, 0.606327, 0.610844, 0.619669, 0.635539, 0.659720,\
        0.689965, 0.718863, 0.748017, 0.787721, 0.845371, 0.891604, 0.912469, 0.922198, 0.928896, 0.934844,\
        0.938902, 0.940909, 0.941394, 0.940841, 0.939532, 0.937605, 0.935116, 0.932054, 0.928355, 0.923931,\
        0.918714, 0.912687, 0.905916, 0.898066, 0.888062, 0.877257, 0.866112, 0.854993, 0.844230, 0.834102,\
        0.824837, 0.816601, 0.809497, 0.803614, 0.799150, 0.796358, 0.794756, 0.793999, 0.793898, 0.794299,\
        0.795057, 0.796039, 0.797151, 0.798375, 0.799833, 0.800291, 0.792165, 0.777917, 0.760045, 0.740403,\
        0.720639, 0.702196, 0.685916, 0.671338, 0.657937, 0.645577, 0.634192, 0.623683, 0.613929, 0.604842,\
        0.596453, 0.588683, 0.581283, 0.574119, 0.567109, 0.560185, 0.553290, 0.546369, 0.539372, 0.532252,\
        0.524967],\
        [0.483509, 0.511277, 0.540269, 0.567672, 0.590070, 0.603343, 0.606628, 0.606581, 0.605552, 0.604522,\
        0.604472, 0.605614, 0.607682, 0.610401, 0.613590, 0.617148, 0.621003, 0.625068, 0.629217, 0.633304,\
        0.637192, 0.640777, 0.644007, 0.646879, 0.649417, 0.651648, 0.653599, 0.655306, 0.656848, 0.658351,\
        0.659951, 0.661671, 0.663245, 0.663962, 0.662489, 0.656912, 0.645790, 0.629961, 0.613583, 0.601182,\
        0.596717, 0.599463, 0.603952, 0.606234, 0.606855, 0.607821, 0.611068, 0.618730, 0.632966, 0.655088,\
        0.684379, 0.715700, 0.752149, 0.803364, 0.876378, 0.923611, 0.941220, 0.947952, 0.950874, 0.952397,\
        0.952749, 0.951900, 0.950085, 0.947617, 0.944729, 0.941539, 0.938063, 0.934232, 0.929918, 0.924965,\
        0.919222, 0.912564, 0.904915, 0.896290, 0.886305, 0.875222, 0.863716, 0.852273, 0.841286, 0.831077,\
        0.821905, 0.813959, 0.807324, 0.801984, 0.798056, 0.795482, 0.794065, 0.793529, 0.793669, 0.794325,\
        0.795360, 0.796634, 0.798012, 0.799622, 0.801510, 0.801130, 0.793993, 0.779980, 0.761445, 0.740816,\
        0.720185, 0.701371, 0.684255, 0.669160, 0.655655, 0.643425, 0.632294, 0.622101, 0.612670, 0.603844,\
        0.595631, 0.587943, 0.580579, 0.573442, 0.566460, 0.559570, 0.552712, 0.545832, 0.538876, 0.531795,\
        0.524546],\
        [0.482544, 0.508489, 0.535837, 0.561422, 0.582183, 0.594796, 0.599317, 0.600954, 0.601645, 0.601794,\
        0.603240, 0.605771, 0.608967, 0.612612, 0.616575, 0.620777, 0.625160, 0.629656, 0.634180, 0.638631,\
        0.642913, 0.646946, 0.650681, 0.654095, 0.657179, 0.659933, 0.662364, 0.664510, 0.666478, 0.668457,\
        0.670683, 0.673295, 0.676149, 0.678679, 0.679680, 0.676544, 0.666648, 0.651038, 0.633948, 0.619673,\
        0.612142, 0.610767, 0.611241, 0.611139, 0.610350, 0.610234, 0.612501, 0.618912, 0.631260, 0.651151,\
        0.678946, 0.715694, 0.765931, 0.827182, 0.894600, 0.940104, 0.960290, 0.967508, 0.968789, 0.967430,\
        0.964709, 0.961131, 0.957027, 0.952679, 0.948289, 0.943946, 0.939621, 0.935177, 0.930404, 0.925056,\
        0.918897, 0.911717, 0.903285, 0.894007, 0.883691, 0.872344, 0.860541, 0.848834, 0.837691, 0.827482,\
        0.818483, 0.810926, 0.804984, 0.800306, 0.796842, 0.794555, 0.793322, 0.792961, 0.793284, 0.794146,\
        0.795444, 0.797117, 0.799052, 0.801312, 0.803566, 0.802715, 0.795362, 0.780679, 0.761106, 0.739440,\
        0.717779, 0.697764, 0.681127, 0.666109, 0.652661, 0.640640, 0.629847, 0.620087, 0.611139, 0.602736,\
        0.594721, 0.587060, 0.579699, 0.572576, 0.565621, 0.558770, 0.551958, 0.545126, 0.538218, 0.531184,\
        0.523975],\
        [0.482595, 0.506779, 0.531934, 0.555196, 0.574004, 0.586529, 0.593174, 0.596588, 0.598448, 0.599789,\
        0.602620, 0.606340, 0.610517, 0.614960, 0.619578, 0.624323, 0.629155, 0.634033, 0.638898, 0.643683,\
        0.648321, 0.652750, 0.656926, 0.660811, 0.664378, 0.667596, 0.670438, 0.672908, 0.675099, 0.677256,\
        0.679762, 0.682965, 0.686881, 0.691007, 0.694765, 0.695344, 0.687421, 0.672130, 0.655210, 0.640423,\
        0.630154, 0.624342, 0.620911, 0.618218, 0.616015, 0.615195, 0.617067, 0.622878, 0.633966, 0.652697,\
        0.683129, 0.726959, 0.782192, 0.845793, 0.904250, 0.947266, 0.970495, 0.979580, 0.980799, 0.977996,\
        0.973204, 0.967478, 0.961425, 0.955440, 0.949765, 0.944489, 0.939550, 0.934741, 0.929753, 0.924234,\
        0.917867, 0.910456, 0.902019, 0.891967, 0.880832, 0.868979, 0.856797, 0.844808, 0.833532, 0.823392,\
        0.814636, 0.807392, 0.802154, 0.798549, 0.795424, 0.793425, 0.792427, 0.792257, 0.792748, 0.793773,\
        0.795263, 0.797234, 0.799789, 0.802842, 0.805347, 0.804423, 0.796426, 0.780189, 0.759001, 0.736217,\
        0.714655, 0.695009, 0.677479, 0.662245, 0.648938, 0.637222, 0.626827, 0.617544, 0.609152, 0.601298,\
        0.593465, 0.585843, 0.578518, 0.571438, 0.564535, 0.557740, 0.550990, 0.544221, 0.537375, 0.530396,\
        0.523236],\
        [0.483999, 0.506587, 0.529403, 0.550356, 0.567719, 0.580433, 0.588674, 0.593672, 0.596767, 0.599480,\
        0.603201, 0.607712, 0.612564, 0.617570, 0.622660, 0.627803, 0.632978, 0.638158, 0.643305, 0.648372,\
        0.653313, 0.658079, 0.662628, 0.666922, 0.670916, 0.674554, 0.677765, 0.680480, 0.682715, 0.684728,\
        0.687094, 0.690446, 0.695130, 0.700296, 0.705329, 0.707596, 0.702081, 0.689032, 0.674161, 0.659680,\
        0.647861, 0.639235, 0.632990, 0.628230, 0.624906, 0.623931, 0.626494, 0.633403, 0.644974, 0.663488,\
        0.695559, 0.746672, 0.805049, 0.867497, 0.919844, 0.955767, 0.976914, 0.986088, 0.987242, 0.983662,\
        0.977552, 0.970290, 0.962743, 0.955471, 0.948814, 0.942906, 0.937667, 0.932817, 0.927935, 0.922536,\
        0.916166, 0.908517, 0.899516, 0.889214, 0.877646, 0.865297, 0.852675, 0.840345, 0.828907, 0.818904,\
        0.810614, 0.803858, 0.798705, 0.795783, 0.793501, 0.792000, 0.791336, 0.791404, 0.792071, 0.793224,\
        0.794805, 0.796835, 0.799506, 0.803157, 0.806100, 0.805105, 0.796201, 0.778298, 0.754957, 0.730349,\
        0.708088, 0.689607, 0.672283, 0.657367, 0.644503, 0.633239, 0.623266, 0.614397, 0.606439, 0.598997,\
        0.591495, 0.584099, 0.576923, 0.569953, 0.563144, 0.556438, 0.549774, 0.543088, 0.536319, 0.529411,\
        0.522309],\
        [0.486208, 0.507525, 0.528057, 0.547039, 0.563406, 0.576381, 0.585878, 0.592369, 0.596680, 0.600357,\
        0.604907, 0.609915, 0.615126, 0.620433, 0.625790, 0.631177, 0.636576, 0.641966, 0.647321, 0.652606,\
        0.657784, 0.662819, 0.667672, 0.672304, 0.676667, 0.680692, 0.684267, 0.687238, 0.689449, 0.691051,\
        0.692920, 0.696041, 0.700900, 0.706966, 0.712042, 0.713974, 0.710171, 0.701362, 0.689590, 0.676050,\
        0.664170, 0.654465, 0.646966, 0.641139, 0.637220, 0.636777, 0.641375, 0.651948, 0.667641, 0.688156,\
        0.725775, 0.779120, 0.835992, 0.890852, 0.934150, 0.963298, 0.980798, 0.988553, 0.989058, 0.984794,\
        0.977762, 0.969420, 0.960789, 0.952581, 0.945262, 0.939049, 0.933867, 0.929354, 0.924952, 0.920026,\
        0.913959, 0.906254, 0.896936, 0.886241, 0.874257, 0.861450, 0.848376, 0.835641, 0.823933, 0.814014,\
        0.806392, 0.800818, 0.796246, 0.793230, 0.791393, 0.790380, 0.790090, 0.790426, 0.791278, 0.792548,\
        0.794179, 0.796205, 0.798836, 0.802463, 0.805609, 0.804759, 0.795257, 0.775238, 0.749487, 0.722472,\
        0.698949, 0.680679, 0.665271, 0.651622, 0.639599, 0.628935, 0.619367, 0.610771, 0.603124, 0.595957,\
        0.588880, 0.581833, 0.574891, 0.568089, 0.561414, 0.554828, 0.548277, 0.541698, 0.535028, 0.528205,\
        0.521175],\
        [0.487823, 0.508095, 0.526856, 0.544594, 0.560461, 0.573826, 0.584503, 0.592600, 0.598222, 0.602753,\
        0.607602, 0.612718, 0.618001, 0.623389, 0.628842, 0.634337, 0.639853, 0.645367, 0.650855, 0.656288,\
        0.661635, 0.666863, 0.671940, 0.676828, 0.681482, 0.685835, 0.689772, 0.693082, 0.695426, 0.696512,\
        0.697721, 0.700960, 0.705944, 0.711752, 0.716503, 0.718404, 0.716171, 0.709914, 0.699663, 0.689474,\
        0.678872, 0.669388, 0.662432, 0.656784, 0.652530, 0.653218, 0.661091, 0.676124, 0.701334, 0.726918,\
        0.766032, 0.814488, 0.863269, 0.907965, 0.943453, 0.967610, 0.981770, 0.987622, 0.987059, 0.982018,\
        0.974232, 0.965101, 0.955700, 0.946843, 0.939131, 0.932919, 0.928159, 0.924366, 0.920815, 0.916722,\
        0.911338, 0.904082, 0.894546, 0.883290, 0.870827, 0.857609, 0.844147, 0.831017, 0.818903, 0.808743,\
        0.801686, 0.796825, 0.793236, 0.790758, 0.789327, 0.788715, 0.788765, 0.789360, 0.790395, 0.791777,\
        0.793445, 0.795408, 0.797807, 0.800796, 0.804097, 0.803303, 0.792681, 0.771507, 0.743997, 0.715070,\
        0.690778, 0.673159, 0.658595, 0.645936, 0.634742, 0.624717, 0.615562, 0.607039, 0.599558, 0.592762,\
        0.585994, 0.579209, 0.572477, 0.565849, 0.559329, 0.552887, 0.546473, 0.540026, 0.533477, 0.526760,\
        0.519815],\
        [0.487321, 0.506447, 0.525015, 0.542443, 0.558244, 0.572053, 0.583731, 0.593351, 0.600759, 0.605651,\
        0.610532, 0.615593, 0.620815, 0.626166, 0.631613, 0.637127, 0.642682, 0.648250, 0.653806, 0.659322,\
        0.664769, 0.670117, 0.675332, 0.680378, 0.685212, 0.689774, 0.693971, 0.697584, 0.700200, 0.701193,\
        0.702384, 0.705639, 0.710455, 0.715717, 0.720079, 0.722182, 0.721080, 0.716505, 0.709355, 0.701689,\
        0.692083, 0.683210, 0.679178, 0.675328, 0.669799, 0.672184, 0.685621, 0.706471, 0.736402, 0.767593,\
        0.803576, 0.845032, 0.885710, 0.921464, 0.949601, 0.968980, 0.979970, 0.983784, 0.981936, 0.976024,\
        0.967556, 0.957832, 0.947902, 0.938611, 0.930681, 0.924698, 0.920712, 0.917978, 0.915545, 0.912527,\
        0.907961, 0.901088, 0.891567, 0.880024, 0.867300, 0.853883, 0.840250, 0.826941, 0.814589, 0.803992,\
        0.797001, 0.793024, 0.790227, 0.788378, 0.787377, 0.787098, 0.787414, 0.788217, 0.789403, 0.790882,\
        0.792578, 0.794447, 0.796476, 0.798840, 0.802476, 0.802485, 0.790731, 0.767764, 0.739316, 0.710652,\
        0.686458, 0.667990, 0.653314, 0.641014, 0.630358, 0.620901, 0.612318, 0.604336, 0.596754, 0.589853,\
        0.583058, 0.576339, 0.569729, 0.563244, 0.556876, 0.550593, 0.544339, 0.538049, 0.531646, 0.525056,\
        0.518211],\
        [0.486680, 0.504919, 0.523273, 0.540554, 0.556356, 0.570463, 0.582731, 0.593001, 0.601060, 0.607379,\
        0.612756, 0.617924, 0.623135, 0.628450, 0.633870, 0.639371, 0.644924, 0.650501, 0.656076, 0.661622,\
        0.667110, 0.672507, 0.677779, 0.682883, 0.687768, 0.692367, 0.696569, 0.700219, 0.702818, 0.704544,\
        0.706495, 0.709747, 0.714210, 0.719078, 0.723316, 0.725909, 0.726116, 0.723695, 0.719036, 0.712444,\
        0.703070, 0.696232, 0.696510, 0.696521, 0.687193, 0.693640, 0.713534, 0.741486, 0.776291, 0.807877,\
        0.835674, 0.870002, 0.902975, 0.931361, 0.953257, 0.968057, 0.975899, 0.977598, 0.974352, 0.967512,\
        0.958420, 0.948277, 0.938067, 0.928563, 0.920484, 0.914804, 0.911969, 0.910500, 0.909191, 0.907360,\
        0.903518, 0.896842, 0.887429, 0.876119, 0.863570, 0.850316, 0.836853, 0.823747, 0.811713, 0.801689,\
        0.794668, 0.790502, 0.787947, 0.786407, 0.785661, 0.785577, 0.786046, 0.786963, 0.788229, 0.789752,\
        0.791452, 0.793274, 0.795219, 0.797456, 0.800399, 0.801806, 0.787676, 0.762703, 0.733937, 0.706637,\
        0.682985, 0.664126, 0.649171, 0.636894, 0.626488, 0.617415, 0.609269, 0.601836, 0.594522, 0.587039,\
        0.579967, 0.573187, 0.566629, 0.560257, 0.554038, 0.547923, 0.541851, 0.535743, 0.529513, 0.523073,\
        0.516347],\
        [0.486983, 0.504669, 0.522228, 0.538983, 0.554558, 0.568726, 0.581309, 0.592105, 0.600932, 0.607991,\
        0.613869, 0.619288, 0.624608, 0.629968, 0.635402, 0.640903, 0.646449, 0.652015, 0.657578, 0.663115,\
        0.668597, 0.673991, 0.679255, 0.684336, 0.689170, 0.693680, 0.697757, 0.701196, 0.704000, 0.706431,\
        0.709105, 0.712605, 0.716985, 0.721779, 0.726271, 0.729718, 0.731542, 0.731446, 0.729351, 0.725120,\
        0.719168, 0.716153, 0.717309, 0.716046, 0.710870, 0.722920, 0.746065, 0.773262, 0.805277, 0.836261,\
        0.862824, 0.890256, 0.916102, 0.937915, 0.954500, 0.965188, 0.970032, 0.969630, 0.964950, 0.957171,\
        0.947529, 0.937172, 0.927020, 0.917711, 0.909696, 0.904054, 0.902853, 0.902450, 0.901822, 0.901167,\
        0.897795, 0.891253, 0.882337, 0.871608, 0.859632, 0.846921, 0.833980, 0.821382, 0.809877, 0.800456,\
        0.793638, 0.789261, 0.786561, 0.784977, 0.784225, 0.784143, 0.784610, 0.785513, 0.786744, 0.788200,\
        0.789792, 0.791451, 0.793146, 0.794865, 0.796353, 0.793875, 0.779139, 0.754640, 0.726339, 0.699482,\
        0.678258, 0.660380, 0.645560, 0.633244, 0.622883, 0.613975, 0.605978, 0.598300, 0.590905, 0.583541,\
        0.576419, 0.569621, 0.563114, 0.556852, 0.550784, 0.544851, 0.538981, 0.533084, 0.527057, 0.520794,\
        0.514203],\
        [0.487673, 0.504906, 0.521441, 0.537444, 0.552580, 0.566592, 0.579300, 0.590560, 0.600136, 0.607450,\
        0.613703, 0.619466, 0.625021, 0.630529, 0.636053, 0.641598, 0.647154, 0.652706, 0.658241, 0.663744,\
        0.669191, 0.674549, 0.679771, 0.684791, 0.689527, 0.693879, 0.697758, 0.701069, 0.704104, 0.707073,\
        0.710267, 0.714066, 0.718601, 0.723644, 0.728733, 0.733351, 0.737076, 0.739651, 0.740980, 0.741162,\
        0.740940, 0.741898, 0.743868, 0.744787, 0.747580, 0.760449, 0.782382, 0.802239, 0.829430, 0.857077,\
        0.881441, 0.903940, 0.924563, 0.941331, 0.953493, 0.960623, 0.962761, 0.960391, 0.954339, 0.945665,\
        0.935567, 0.925223, 0.915531, 0.907042, 0.900035, 0.895138, 0.894136, 0.894187, 0.894016, 0.893370,\
        0.890474, 0.884594, 0.876485, 0.866642, 0.855549, 0.843694, 0.831576, 0.819730, 0.808763, 0.799557,\
        0.793268, 0.788861, 0.785860, 0.783974, 0.782979, 0.782706, 0.783009, 0.783753, 0.784803, 0.786026,\
        0.787292, 0.788464, 0.789363, 0.789609, 0.788042, 0.781609, 0.766942, 0.744834, 0.719024, 0.694200,\
        0.674105, 0.656947, 0.642221, 0.629727, 0.619200, 0.610243, 0.602243, 0.594459, 0.586763, 0.579369,\
        0.572289, 0.565547, 0.559129, 0.552991, 0.547083, 0.541343, 0.535696, 0.530042, 0.524255, 0.518200,\
        0.511761],\
        [0.487521, 0.504063, 0.520099, 0.535512, 0.550136, 0.563825, 0.576408, 0.587680, 0.597298, 0.605258,\
        0.612169, 0.618401, 0.624290, 0.630047, 0.635742, 0.641384, 0.646973, 0.652514, 0.658013, 0.663468,\
        0.668867, 0.674180, 0.679357, 0.684330, 0.689001, 0.693246, 0.696932, 0.700166, 0.703473, 0.706766,\
        0.710183, 0.714175, 0.718967, 0.724463, 0.730367, 0.736330, 0.742068, 0.747449, 0.752516, 0.757499,\
        0.762671, 0.767798, 0.772338, 0.776084, 0.781842, 0.793608, 0.811289, 0.829623, 0.850638, 0.874103,\
        0.895689, 0.913269, 0.929165, 0.941808, 0.950395, 0.954579, 0.954412, 0.950338, 0.943097, 0.933658,\
        0.923208, 0.913087, 0.904234, 0.896932, 0.891299, 0.887613, 0.886236, 0.886068, 0.885923, 0.884999,\
        0.882295, 0.877281, 0.870181, 0.861404, 0.851387, 0.840594, 0.829521, 0.818703, 0.808718, 0.800254,\
        0.793888, 0.789031, 0.785500, 0.783126, 0.781721, 0.781105, 0.781107, 0.781559, 0.782284, 0.783101,\
        0.783806, 0.784156, 0.783798, 0.782097, 0.777844, 0.769197, 0.754766, 0.735165, 0.713129, 0.691566,\
        0.671371, 0.654037, 0.639027, 0.626098, 0.615123, 0.605932, 0.597642, 0.589665, 0.581964, 0.574600,\
        0.567597, 0.560954, 0.554653, 0.548655, 0.542912, 0.537371, 0.531961, 0.526579, 0.521076, 0.515269,\
        0.509000],\
        [0.486791, 0.502727, 0.518254, 0.532980, 0.547023, 0.560300, 0.572629, 0.583778, 0.593536, 0.602028,\
        0.609529, 0.616174, 0.622430, 0.628523, 0.634464, 0.640243, 0.645877, 0.651404, 0.656858, 0.662260,\
        0.667608, 0.672880, 0.678034, 0.683006, 0.687713, 0.692058, 0.695942, 0.699349, 0.702709, 0.705779,\
        0.708969, 0.712966, 0.718023, 0.724055, 0.730837, 0.738111, 0.745690, 0.753527, 0.761747, 0.770608,\
        0.780187, 0.789029, 0.795015, 0.800650, 0.805979, 0.816296, 0.830017, 0.846470, 0.864862, 0.884689,\
        0.903333, 0.917932, 0.930083, 0.939482, 0.945335, 0.947237, 0.945250, 0.939849, 0.931760, 0.921847,\
        0.911239, 0.901507, 0.893838, 0.887720, 0.883120, 0.880129, 0.878688, 0.878191, 0.877787, 0.876681,\
        0.874170, 0.869853, 0.863735, 0.856061, 0.847182, 0.837517, 0.827551, 0.817833, 0.808960, 0.801422,\
        0.794678, 0.789152, 0.785006, 0.782079, 0.780187, 0.779145, 0.778759, 0.778827, 0.779127, 0.779412,\
        0.779402, 0.778757, 0.777019, 0.773531, 0.767370, 0.757536, 0.743552, 0.726030, 0.706314, 0.686763,\
        0.668194, 0.651069, 0.635773, 0.622276, 0.610547, 0.600709, 0.592306, 0.584268, 0.576606, 0.569315,\
        0.562407, 0.555878, 0.549707, 0.543851, 0.538266, 0.532908, 0.527730, 0.522645, 0.517481, 0.511978,\
        0.505892],\
        [0.485774, 0.501637, 0.515760, 0.529632, 0.543108, 0.555997, 0.568098, 0.579222, 0.589260, 0.598233,\
        0.606021, 0.612836, 0.619508, 0.626047, 0.632288, 0.638205, 0.643866, 0.649359, 0.654755, 0.660098,\
        0.665399, 0.670642, 0.675795, 0.680807, 0.685624, 0.690198, 0.694502, 0.698456, 0.701695, 0.703982,\
        0.706533, 0.710450, 0.715774, 0.722330, 0.729912, 0.738282, 0.747268, 0.756814, 0.766974, 0.777831,\
        0.789236, 0.799891, 0.805616, 0.811710, 0.818283, 0.827674, 0.839931, 0.854554, 0.870823, 0.887519,\
        0.902927, 0.916417, 0.926782, 0.934284, 0.938401, 0.938755, 0.935453, 0.929157, 0.920723, 0.910861,\
        0.900687, 0.891757, 0.884790, 0.879366, 0.875336, 0.872662, 0.871169, 0.870409, 0.869753, 0.868555,\
        0.866267, 0.862554, 0.857330, 0.850708, 0.842929, 0.834333, 0.825353, 0.816500, 0.808276, 0.800888,\
        0.794233, 0.788483, 0.783904, 0.780489, 0.778119, 0.776633, 0.775832, 0.775484, 0.775323, 0.775041,\
        0.774288, 0.772660, 0.769674, 0.764764, 0.757315, 0.746836, 0.733274, 0.717376, 0.700292, 0.682201,\
        0.664434, 0.647730, 0.632359, 0.618434, 0.606061, 0.595573, 0.586732, 0.578608, 0.570920, 0.563651,\
        0.556808, 0.550380, 0.544334, 0.538615, 0.533164, 0.527947, 0.522957, 0.518160, 0.513392, 0.508303,\
        0.502400],\
        [0.482976, 0.497696, 0.511737, 0.525252, 0.538342, 0.550944, 0.562875, 0.573986, 0.584194, 0.593454,\
        0.601412, 0.608278, 0.615732, 0.622826, 0.629336, 0.635323, 0.640945, 0.646362, 0.651682, 0.656965,\
        0.662228, 0.667458, 0.672622, 0.677680, 0.682593, 0.687326, 0.691846, 0.696049, 0.699364, 0.700791,\
        0.702775, 0.706824, 0.712373, 0.719351, 0.727565, 0.736706, 0.746552, 0.756964, 0.767813, 0.778851,\
        0.789439, 0.798401, 0.805724, 0.812679, 0.820424, 0.829994, 0.841564, 0.854915, 0.869662, 0.885046,\
        0.899865, 0.910358, 0.919532, 0.926314, 0.929716, 0.929307, 0.925127, 0.918255, 0.910038, 0.900940,\
        0.891614, 0.883434, 0.876771, 0.871578, 0.867730, 0.865115, 0.863520, 0.862568, 0.861753, 0.860554,\
        0.858528, 0.855383, 0.850987, 0.845353, 0.838601, 0.830960, 0.822769, 0.814470, 0.806546, 0.799304,\
        0.792639, 0.786792, 0.781925, 0.778109, 0.775310, 0.773409, 0.772218, 0.771486, 0.770903, 0.770110,\
        0.768703, 0.766239, 0.762251, 0.756284, 0.747976, 0.737168, 0.723924, 0.708726, 0.693084, 0.676772,\
        0.660157, 0.644004, 0.628777, 0.614747, 0.602042, 0.591055, 0.581566, 0.573005, 0.565089, 0.557722,\
        0.550876, 0.544519, 0.538592, 0.533009, 0.527671, 0.522525, 0.517617, 0.513035, 0.508632, 0.504120,\
        0.498526],\
        [0.479317, 0.493514, 0.507050, 0.520006, 0.532843, 0.545244, 0.557021, 0.568070, 0.578293, 0.587519,\
        0.595425, 0.603444, 0.611679, 0.619128, 0.625719, 0.631613, 0.637092, 0.642383, 0.647617, 0.652852,\
        0.658098, 0.663332, 0.668517, 0.673607, 0.678553, 0.683300, 0.687754, 0.691692, 0.694613, 0.696386,\
        0.698614, 0.702702, 0.708081, 0.715362, 0.724005, 0.733548, 0.743739, 0.754351, 0.765097, 0.775543,\
        0.785071, 0.793157, 0.800261, 0.807442, 0.815765, 0.825680, 0.836658, 0.848724, 0.861991, 0.875990,\
        0.888906, 0.898899, 0.908861, 0.915639, 0.919350, 0.919164, 0.914381, 0.907062, 0.899219, 0.891309,\
        0.883023, 0.875444, 0.869024, 0.863876, 0.859986, 0.857276, 0.855555, 0.854482, 0.853611, 0.852496,\
        0.850772, 0.848189, 0.844598, 0.839923, 0.834157, 0.827387, 0.819836, 0.811873, 0.803976, 0.796733,\
        0.790124, 0.784147, 0.778997, 0.774811, 0.771622, 0.769355, 0.767838, 0.766803, 0.765905, 0.764736,\
        0.762847, 0.759772, 0.755064, 0.748348, 0.739437, 0.728509, 0.716053, 0.701761, 0.686802, 0.671376,\
        0.655552, 0.639912, 0.624914, 0.610931, 0.598213, 0.586746, 0.576559, 0.567435, 0.559142, 0.551565,\
        0.544645, 0.538327, 0.532526, 0.527111, 0.521904, 0.516769, 0.511770, 0.507239, 0.503167, 0.498606,\
        0.493463],\
        [0.475215, 0.488668, 0.501213, 0.514175, 0.526917, 0.539084, 0.550646, 0.561587, 0.571851, 0.581378,\
        0.590316, 0.599173, 0.607479, 0.614997, 0.621412, 0.626995, 0.632223, 0.637370, 0.642538, 0.647760,\
        0.653029, 0.658303, 0.663527, 0.668642, 0.673585, 0.678273, 0.682581, 0.686310, 0.689267, 0.691670,\
        0.694414, 0.698077, 0.703231, 0.710896, 0.719635, 0.729173, 0.739299, 0.749714, 0.760038, 0.769815,\
        0.778590, 0.786146, 0.792784, 0.799153, 0.807656, 0.817549, 0.827268, 0.837442, 0.848837, 0.861094,\
        0.873000, 0.884137, 0.894654, 0.901656, 0.907097, 0.908646, 0.903321, 0.896022, 0.888795, 0.881521,\
        0.874164, 0.867180, 0.860985, 0.855802, 0.851749, 0.848864, 0.847030, 0.845924, 0.845098, 0.844133,\
        0.842736, 0.840723, 0.837950, 0.834267, 0.829523, 0.823624, 0.816658, 0.809028, 0.801176, 0.793796,\
        0.786948, 0.780650, 0.775118, 0.770528, 0.766962, 0.764382, 0.762627, 0.761415, 0.760358, 0.759000,\
        0.756848, 0.753418, 0.748276, 0.741086, 0.731696, 0.720373, 0.708472, 0.696582, 0.681491, 0.666066,\
        0.650641, 0.635423, 0.620631, 0.606636, 0.593782, 0.582028, 0.571355, 0.561702, 0.552987, 0.545140,\
        0.538100, 0.531801, 0.526146, 0.520977, 0.516020, 0.510935, 0.505741, 0.500839, 0.497168, 0.492576,\
        0.487254],\
        [0.470912, 0.483746, 0.496143, 0.508709, 0.520950, 0.532614, 0.543821, 0.554596, 0.564918, 0.574820,\
        0.584422, 0.593743, 0.602534, 0.610217, 0.616188, 0.621228, 0.626204, 0.631272, 0.636437, 0.641714,\
        0.647073, 0.652441, 0.657743, 0.662909, 0.667868, 0.672535, 0.676801, 0.680556, 0.683781, 0.686720,\
        0.689878, 0.693791, 0.699351, 0.706534, 0.714787, 0.723949, 0.733754, 0.743801, 0.753648, 0.762877,\
        0.771164, 0.778415, 0.784890, 0.791392, 0.799466, 0.807981, 0.815246, 0.822681, 0.832094, 0.842942,\
        0.854067, 0.865141, 0.875694, 0.884688, 0.892333, 0.895915, 0.891461, 0.884952, 0.878258, 0.871532,\
        0.864824, 0.858331, 0.852315, 0.847019, 0.842694, 0.839565, 0.837664, 0.836647, 0.835962, 0.835177,\
        0.834093, 0.832645, 0.830732, 0.828135, 0.824545, 0.819644, 0.813264, 0.805729, 0.798269, 0.790552,\
        0.783144, 0.776311, 0.770266, 0.765211, 0.761265, 0.758425, 0.756537, 0.755298, 0.754272, 0.752936,\
        0.750747, 0.747205, 0.741893, 0.734520, 0.724947, 0.713170, 0.699851, 0.688302, 0.674790, 0.660028,\
        0.645087, 0.630409, 0.616021, 0.601799, 0.588918, 0.576901, 0.565767, 0.555628, 0.546504, 0.538378,\
        0.531206, 0.524912, 0.519407, 0.514556, 0.510074, 0.505333, 0.499925, 0.495084, 0.490694, 0.486039,\
        0.480666],\
        [0.466646, 0.479250, 0.491564, 0.503615, 0.514887, 0.525768, 0.536523, 0.547047, 0.557301, 0.567357,\
        0.577299, 0.587091, 0.596459, 0.604394, 0.609526, 0.613926, 0.618956, 0.624096, 0.629339, 0.634771,\
        0.640317, 0.645856, 0.651292, 0.656559, 0.661596, 0.666336, 0.670705, 0.674659, 0.678248, 0.681686,\
        0.685350, 0.689726, 0.695266, 0.701905, 0.709477, 0.718137, 0.727563, 0.737217, 0.746628, 0.755428,\
        0.763389, 0.770494, 0.777015, 0.783555, 0.790600, 0.797106, 0.801921, 0.806057, 0.814001, 0.823580,\
        0.833337, 0.843623, 0.854580, 0.865062, 0.873621, 0.877890, 0.876600, 0.872171, 0.866644, 0.860751,\
        0.854728, 0.848705, 0.842831, 0.837332, 0.832571, 0.829063, 0.827157, 0.826408, 0.825949, 0.825307,\
        0.824456, 0.823541, 0.822538, 0.821153, 0.818916, 0.815266, 0.809788, 0.802332, 0.794335, 0.786426,\
        0.778498, 0.771039, 0.764392, 0.758817, 0.754485, 0.751434, 0.749528, 0.748434, 0.747644, 0.746542,\
        0.744512, 0.741026, 0.735705, 0.728331, 0.718918, 0.707767, 0.694859, 0.681276, 0.667450, 0.653018,\
        0.638477, 0.624418, 0.611109, 0.597531, 0.583916, 0.571262, 0.559636, 0.549072, 0.539596, 0.531230,\
        0.523954, 0.517663, 0.512253, 0.507668, 0.503679, 0.499671, 0.494405, 0.489200, 0.484312, 0.479306,\
        0.473821],\
        [0.462252, 0.474517, 0.486451, 0.497846, 0.508077, 0.518355, 0.528756, 0.538918, 0.548857, 0.558748,\
        0.568677, 0.578552, 0.588004, 0.595980, 0.600523, 0.605069, 0.610727, 0.615918, 0.621297, 0.627034,\
        0.632895, 0.638683, 0.644309, 0.649735, 0.654928, 0.659844, 0.664440, 0.668703, 0.672700, 0.676606,\
        0.680698, 0.685299, 0.690646, 0.696748, 0.703705, 0.712041, 0.721148, 0.730425, 0.739426, 0.747845,\
        0.755518, 0.762441, 0.768774, 0.774754, 0.780357, 0.784998, 0.788179, 0.791271, 0.797209, 0.805210,\
        0.812401, 0.821375, 0.832719, 0.844175, 0.853379, 0.858727, 0.859751, 0.857544, 0.853586, 0.848821,\
        0.843645, 0.838170, 0.832447, 0.826681, 0.821285, 0.817073, 0.815226, 0.814991, 0.814757, 0.814150,\
        0.813396, 0.812969, 0.812929, 0.812821, 0.812098, 0.809826, 0.805584, 0.798972, 0.790314, 0.781486,\
        0.772865, 0.764743, 0.757461, 0.751336, 0.746610, 0.743393, 0.741585, 0.740820, 0.740483, 0.739816,\
        0.738071, 0.734678, 0.729314, 0.721820, 0.712187, 0.700842, 0.687840, 0.673718, 0.659433, 0.644986,\
        0.630515, 0.616554, 0.603838, 0.591154, 0.577529, 0.564625, 0.552754, 0.541944, 0.532229, 0.523697,\
        0.516405, 0.510182, 0.504783, 0.500329, 0.496482, 0.492544, 0.487836, 0.482749, 0.477638, 0.472385,\
        0.466773],\
        [0.457586, 0.469315, 0.480496, 0.490881, 0.500538, 0.510653, 0.520795, 0.530294, 0.539562, 0.548953,\
        0.558521, 0.568045, 0.577016, 0.584569, 0.590167, 0.595865, 0.601663, 0.606704, 0.612408, 0.618702,\
        0.624989, 0.631073, 0.636923, 0.642555, 0.647977, 0.653165, 0.658084, 0.662727, 0.667149, 0.671485,\
        0.675927, 0.680682, 0.685897, 0.691653, 0.698250, 0.706182, 0.714913, 0.723753, 0.732303, 0.740308,\
        0.747646, 0.754284, 0.760239, 0.765513, 0.769975, 0.773386, 0.775858, 0.778491, 0.782641, 0.788053,\
        0.793117, 0.800777, 0.812382, 0.824300, 0.833812, 0.839993, 0.842514, 0.841896, 0.839312, 0.835707,\
        0.831468, 0.826640, 0.821087, 0.815044, 0.809035, 0.803650, 0.801819, 0.802019, 0.801772, 0.801170,\
        0.800539, 0.800648, 0.801597, 0.802602, 0.803458, 0.802167, 0.798684, 0.792503, 0.784205, 0.775155,\
        0.766080, 0.757404, 0.749526, 0.742843, 0.737708, 0.734346, 0.732739, 0.732490, 0.732839, 0.732799,\
        0.731389, 0.727938, 0.722252, 0.714343, 0.703977, 0.691574, 0.678178, 0.664201, 0.650084, 0.635870,\
        0.621434, 0.607072, 0.594025, 0.581787, 0.569139, 0.556711, 0.545039, 0.534289, 0.524499, 0.515854,\
        0.508668, 0.502710, 0.497443, 0.492972, 0.489033, 0.485018, 0.480555, 0.475686, 0.470571, 0.465215,\
        0.459527],\
        [0.452685, 0.463852, 0.474426, 0.484345, 0.493860, 0.503846, 0.513015, 0.521212, 0.529449, 0.538138,\
        0.547183, 0.556223, 0.564774, 0.572352, 0.578944, 0.585209, 0.591143, 0.596695, 0.603139, 0.610096,\
        0.616787, 0.623154, 0.629229, 0.635100, 0.640821, 0.646368, 0.651688, 0.656755, 0.661609, 0.666349,\
        0.671117, 0.676066, 0.681331, 0.687064, 0.693518, 0.700963, 0.709077, 0.717361, 0.725363, 0.732863,\
        0.739798, 0.746097, 0.751656, 0.756351, 0.760052, 0.762721, 0.764641, 0.766552, 0.769223, 0.772824,\
        0.777539, 0.784832, 0.795540, 0.806590, 0.815803, 0.822419, 0.825837, 0.825944, 0.824193, 0.821562,\
        0.818130, 0.813999, 0.808541, 0.802043, 0.795643, 0.789951, 0.787131, 0.786501, 0.786125, 0.785771,\
        0.785788, 0.786674, 0.788416, 0.790437, 0.792058, 0.791608, 0.788738, 0.783385, 0.775957, 0.767308,\
        0.758186, 0.749170, 0.740790, 0.733565, 0.727987, 0.724452, 0.723112, 0.723555, 0.724817, 0.725622,\
        0.724569, 0.720692, 0.714042, 0.705079, 0.693614, 0.680080, 0.666234, 0.652571, 0.639089, 0.625635,\
        0.611896, 0.597576, 0.584297, 0.571868, 0.559669, 0.547813, 0.536640, 0.526320, 0.516745, 0.507935,\
        0.500906, 0.495271, 0.490245, 0.485748, 0.481609, 0.477465, 0.473046, 0.468280, 0.463200, 0.457817,\
        0.452097],\
        [0.447635, 0.458300, 0.468442, 0.478160, 0.487742, 0.497199, 0.504786, 0.511437, 0.518606, 0.526662,\
        0.535235, 0.543849, 0.552136, 0.559824, 0.566921, 0.573662, 0.580185, 0.586756, 0.593909, 0.601281,\
        0.608344, 0.614986, 0.621275, 0.627414, 0.633518, 0.639517, 0.645298, 0.650813, 0.656088, 0.661209,\
        0.666296, 0.671479, 0.676894, 0.682694, 0.689051, 0.696030, 0.703528, 0.711239, 0.718558, 0.725418,\
        0.731911, 0.737906, 0.743158, 0.747459, 0.750682, 0.752839, 0.754199, 0.755321, 0.756879, 0.759501,\
        0.763922, 0.770951, 0.780432, 0.790517, 0.799290, 0.806293, 0.810388, 0.810099, 0.808584, 0.806760,\
        0.803550, 0.799778, 0.794303, 0.787198, 0.780328, 0.774594, 0.770958, 0.769241, 0.768549, 0.768524,\
        0.769300, 0.771063, 0.773657, 0.776494, 0.778661, 0.779080, 0.777176, 0.772866, 0.766435, 0.758451,\
        0.749561, 0.740411, 0.731652, 0.723937, 0.717897, 0.714085, 0.713011, 0.714248, 0.716547, 0.718463,\
        0.717916, 0.713165, 0.704531, 0.693442, 0.680126, 0.665718, 0.651756, 0.638792, 0.626280, 0.613781,\
        0.601318, 0.587673, 0.574226, 0.561623, 0.549640, 0.538266, 0.527705, 0.518185, 0.509594, 0.500902,\
        0.493801, 0.488167, 0.483151, 0.478571, 0.474255, 0.469964, 0.465492, 0.460733, 0.455657, 0.450258,\
        0.444524],\
        [0.442481, 0.452653, 0.462333, 0.471599, 0.480490, 0.488644, 0.495279, 0.501106, 0.507434, 0.515193,\
        0.523353, 0.531552, 0.539622, 0.547331, 0.554693, 0.561883, 0.569083, 0.576485, 0.584209, 0.592035,\
        0.599571, 0.606581, 0.613069, 0.619527, 0.626150, 0.632700, 0.638975, 0.644920, 0.650580, 0.656042,\
        0.661420, 0.666830, 0.672398, 0.678247, 0.684486, 0.691148, 0.698177, 0.705285, 0.711651, 0.717717,\
        0.723836, 0.729646, 0.734708, 0.738769, 0.741701, 0.743476, 0.744286, 0.744610, 0.745169, 0.746889,\
        0.750769, 0.757370, 0.766176, 0.775677, 0.784324, 0.791525, 0.795729, 0.794885, 0.793343, 0.791565,\
        0.788317, 0.783868, 0.777886, 0.770611, 0.763414, 0.757441, 0.753331, 0.751013, 0.750057, 0.750260,\
        0.751677, 0.754314, 0.757860, 0.761598, 0.764585, 0.765935, 0.765093, 0.761892, 0.756490, 0.749276,\
        0.740778, 0.731648, 0.722671, 0.714608, 0.708244, 0.704097, 0.703182, 0.705034, 0.708056, 0.711305,\
        0.711578, 0.706213, 0.694530, 0.680238, 0.664469, 0.648735, 0.634695, 0.622999, 0.611864, 0.599967,\
        0.588140, 0.575663, 0.562970, 0.550733, 0.539122, 0.528234, 0.518278, 0.509451, 0.502574, 0.495153,\
        0.487600, 0.481443, 0.476176, 0.471427, 0.466946, 0.462516, 0.457957, 0.453152, 0.448042, 0.442611,\
        0.436857],\
        [0.437251, 0.446902, 0.456025, 0.464634, 0.472649, 0.479806, 0.485908, 0.491372, 0.497329, 0.504782,\
        0.512118, 0.519837, 0.527694, 0.535247, 0.542658, 0.550171, 0.557930, 0.565981, 0.574248, 0.582509,\
        0.590482, 0.597902, 0.604616, 0.611551, 0.618912, 0.626059, 0.632782, 0.639084, 0.645051, 0.650787,\
        0.656400, 0.661991, 0.667657, 0.673491, 0.679569, 0.685921, 0.692498, 0.698886, 0.704094, 0.709421,\
        0.715439, 0.721206, 0.726112, 0.730001, 0.732787, 0.734291, 0.734577, 0.734092, 0.733565, 0.734134,\
        0.737250, 0.743622, 0.752391, 0.761794, 0.770270, 0.776790, 0.780212, 0.780265, 0.778706, 0.776289,\
        0.772634, 0.767497, 0.760812, 0.753043, 0.745413, 0.739189, 0.734945, 0.732473, 0.731410, 0.731776,\
        0.733729, 0.737223, 0.741864, 0.746702, 0.750698, 0.753055, 0.753272, 0.751154, 0.746773, 0.740392,\
        0.732409, 0.723409, 0.714380, 0.706478, 0.699942, 0.696148, 0.695060, 0.696497, 0.699522, 0.703383,\
        0.704952, 0.696983, 0.682950, 0.665337, 0.647117, 0.629863, 0.615367, 0.605415, 0.596055, 0.585115,\
        0.573888, 0.562388, 0.550694, 0.539202, 0.528222, 0.517883, 0.508545, 0.500485, 0.493584, 0.487308,\
        0.480564, 0.474451, 0.469079, 0.464218, 0.459627, 0.455098, 0.450461, 0.445594, 0.440427, 0.434939,\
        0.429147],\
        [0.431994, 0.441107, 0.449645, 0.457606, 0.464924, 0.471499, 0.477366, 0.482912, 0.488864, 0.495415,\
        0.501640, 0.509305, 0.516819, 0.523846, 0.531062, 0.538749, 0.546961, 0.555577, 0.564368, 0.573068,\
        0.581413, 0.589176, 0.596325, 0.604096, 0.612161, 0.619746, 0.626746, 0.633264, 0.639429, 0.645355,\
        0.651131, 0.656831, 0.662518, 0.668246, 0.674052, 0.679928, 0.685749, 0.691132, 0.695724, 0.700749,\
        0.706774, 0.712438, 0.716986, 0.720637, 0.723453, 0.724806, 0.724567, 0.723354, 0.721713, 0.720761,\
        0.722909, 0.729561, 0.739011, 0.748657, 0.756750, 0.762415, 0.765234, 0.765403, 0.763778, 0.760957,\
        0.756907, 0.751315, 0.744004, 0.735445, 0.727100, 0.720666, 0.716697, 0.714508, 0.713484, 0.713989,\
        0.716457, 0.720729, 0.726666, 0.732714, 0.737724, 0.741041, 0.742224, 0.741081, 0.737661, 0.732169,\
        0.724877, 0.716173, 0.706920, 0.699721, 0.693870, 0.689648, 0.687933, 0.688405, 0.690488, 0.693005,\
        0.693776, 0.683523, 0.666979, 0.647989, 0.628649, 0.610791, 0.596309, 0.586828, 0.578631, 0.569185,\
        0.559080, 0.548636, 0.537956, 0.527305, 0.517222, 0.507567, 0.498697, 0.491289, 0.484675, 0.478562,\
        0.472552, 0.466865, 0.461655, 0.456837, 0.452238, 0.447678, 0.443003, 0.438093, 0.432865, 0.427297,\
        0.421438],\
        [0.426774, 0.435364, 0.443358, 0.450776, 0.457610, 0.463863, 0.469643, 0.475238, 0.480995, 0.486985,\
        0.493346, 0.500806, 0.507068, 0.513183, 0.520090, 0.527855, 0.536486, 0.545648, 0.554948, 0.564093,\
        0.572881, 0.581230, 0.589338, 0.597697, 0.606027, 0.613732, 0.620768, 0.627344, 0.633601, 0.639632,\
        0.645501, 0.651244, 0.656874, 0.662403, 0.667835, 0.673141, 0.678219, 0.682922, 0.687352, 0.692179,\
        0.697654, 0.702927, 0.706638, 0.709875, 0.713064, 0.714347, 0.713343, 0.711731, 0.709379, 0.706807,\
        0.707799, 0.715604, 0.726380, 0.736324, 0.743765, 0.748415, 0.750450, 0.750236, 0.748344, 0.745316,\
        0.741231, 0.735656, 0.728113, 0.718721, 0.709461, 0.702905, 0.699582, 0.697984, 0.697323, 0.698177,\
        0.701215, 0.705966, 0.713492, 0.720431, 0.726182, 0.730273, 0.732213, 0.731813, 0.729188, 0.724575,\
        0.718198, 0.710255, 0.701244, 0.693415, 0.687469, 0.682878, 0.680119, 0.679010, 0.678832, 0.678212,\
        0.674306, 0.663590, 0.647397, 0.628852, 0.610025, 0.593367, 0.580112, 0.570272, 0.561898, 0.553280,\
        0.544217, 0.534906, 0.525332, 0.515363, 0.506120, 0.497917, 0.489333, 0.482027, 0.475568, 0.469641,\
        0.464053, 0.458792, 0.453879, 0.449242, 0.444742, 0.440229, 0.435576, 0.430673, 0.425414, 0.419746,\
        0.413777],\
        [0.421651, 0.429749, 0.437263, 0.444260, 0.450774, 0.456850, 0.462594, 0.468195, 0.473858, 0.479737,\
        0.486010, 0.492445, 0.497743, 0.503475, 0.510046, 0.517795, 0.526909, 0.536592, 0.546314, 0.555833,\
        0.565042, 0.573952, 0.582720, 0.591496, 0.600057, 0.607661, 0.614586, 0.621147, 0.627430, 0.633500,\
        0.639410, 0.645144, 0.650655, 0.655911, 0.660906, 0.665626, 0.670049, 0.674193, 0.678249, 0.682541,\
        0.687114, 0.691310, 0.694347, 0.697134, 0.700659, 0.701795, 0.699472, 0.697777, 0.695334, 0.692894,\
        0.694179, 0.702996, 0.714773, 0.724539, 0.730779, 0.734161, 0.735305, 0.734544, 0.732221, 0.729055,\
        0.725364, 0.720314, 0.713392, 0.703687, 0.694038, 0.687637, 0.684429, 0.683266, 0.683535, 0.685457,\
        0.689233, 0.695398, 0.703088, 0.710140, 0.716265, 0.720940, 0.723293, 0.723213, 0.721031, 0.717077,\
        0.711561, 0.704647, 0.696161, 0.687485, 0.680385, 0.674838, 0.670725, 0.667612, 0.664608, 0.660371,\
        0.653062, 0.641269, 0.625886, 0.609208, 0.592385, 0.577446, 0.565234, 0.555349, 0.546683, 0.538206,\
        0.529727, 0.521469, 0.513219, 0.504523, 0.495021, 0.487467, 0.479752, 0.472632, 0.466275, 0.460530,\
        0.455256, 0.450373, 0.445806, 0.441442, 0.437131, 0.432730, 0.428150, 0.423332, 0.418149, 0.412387,\
        0.406231],\
        [0.416659, 0.424293, 0.431395, 0.438082, 0.444404, 0.450397, 0.456138, 0.461747, 0.467359, 0.473081,\
        0.478915, 0.484593, 0.489883, 0.495389, 0.501367, 0.509114, 0.518821, 0.528789, 0.538674, 0.548353,\
        0.557786, 0.566993, 0.576039, 0.584919, 0.593384, 0.600998, 0.607921, 0.614528, 0.620801, 0.626842,\
        0.632770, 0.638477, 0.643819, 0.648738, 0.653241, 0.657355, 0.661112, 0.664588, 0.667948, 0.671360,\
        0.674761, 0.677769, 0.680137, 0.682293, 0.684336, 0.684511, 0.682411, 0.680168, 0.678203, 0.677617,\
        0.681127, 0.690413, 0.702497, 0.711843, 0.716305, 0.718653, 0.719112, 0.718063, 0.715384, 0.711725,\
        0.709015, 0.704133, 0.698271, 0.689348, 0.680353, 0.674188, 0.671098, 0.670465, 0.671777, 0.674920,\
        0.679941, 0.686775, 0.694433, 0.701476, 0.707979, 0.713195, 0.715322, 0.714896, 0.712645, 0.708846,\
        0.703740, 0.696789, 0.688426, 0.679570, 0.671632, 0.665013, 0.659511, 0.654462, 0.648822, 0.641545,\
        0.631925, 0.619190, 0.604337, 0.589636, 0.575348, 0.562218, 0.551008, 0.541459, 0.532750, 0.524135,\
        0.515646, 0.508327, 0.500932, 0.493677, 0.485080, 0.477090, 0.469719, 0.462891, 0.456714, 0.451173,\
        0.446190, 0.441671, 0.437487, 0.433469, 0.429416, 0.425163, 0.420657, 0.415967, 0.411095, 0.405459,\
        0.398973],\
        [0.411795, 0.418969, 0.425730, 0.432230, 0.438477, 0.444467, 0.450238, 0.455866, 0.461440, 0.467019,\
        0.472588, 0.478061, 0.483461, 0.489056, 0.495239, 0.503412, 0.512742, 0.522379, 0.532025, 0.541566,\
        0.550955, 0.560168, 0.569185, 0.577924, 0.586190, 0.593775, 0.600792, 0.607520, 0.613645, 0.619532,\
        0.625527, 0.631217, 0.636329, 0.640829, 0.644775, 0.648240, 0.651271, 0.653929, 0.656353, 0.658698,\
        0.660924, 0.662760, 0.664014, 0.664780, 0.665002, 0.664145, 0.662166, 0.659964, 0.658591, 0.659380,\
        0.664052, 0.673312, 0.684965, 0.694758, 0.698123, 0.701337, 0.701157, 0.700277, 0.698395, 0.693847,\
        0.691517, 0.686509, 0.680498, 0.673137, 0.665962, 0.660922, 0.658647, 0.658915, 0.661333, 0.665643,\
        0.671668, 0.679069, 0.687077, 0.694732, 0.702312, 0.707027, 0.707753, 0.706225, 0.703565, 0.699273,\
        0.693859, 0.686639, 0.678105, 0.669124, 0.660621, 0.653116, 0.646503, 0.640027, 0.632448, 0.622608,\
        0.611714, 0.598761, 0.584692, 0.571273, 0.558665, 0.547116, 0.537013, 0.528174, 0.519804, 0.511197,\
        0.502336, 0.495128, 0.488623, 0.481506, 0.473984, 0.466476, 0.459385, 0.452816, 0.446852, 0.441536,\
        0.436847, 0.432701, 0.428943, 0.425342, 0.421624, 0.417540, 0.413035, 0.408325, 0.403799, 0.398937,\
        0.392324],\
        [0.406993, 0.413679, 0.420238, 0.426717, 0.433008, 0.439059, 0.444887, 0.450547, 0.456108, 0.461627,\
        0.467131, 0.472652, 0.478308, 0.484375, 0.491316, 0.499378, 0.508027, 0.517002, 0.526104, 0.535263,\
        0.544385, 0.553381, 0.562171, 0.570662, 0.578732, 0.586304, 0.593451, 0.600273, 0.605745, 0.611448,\
        0.617736, 0.623371, 0.628119, 0.632082, 0.635394, 0.638176, 0.640462, 0.642201, 0.643505, 0.644702,\
        0.645843, 0.646545, 0.646443, 0.645533, 0.644098, 0.642331, 0.640215, 0.637995, 0.636894, 0.638055,\
        0.642578, 0.651247, 0.662025, 0.671314, 0.676966, 0.680416, 0.681376, 0.680941, 0.679207, 0.675936,\
        0.672406, 0.667724, 0.662046, 0.655977, 0.650654, 0.647302, 0.646458, 0.648007, 0.651604, 0.656940,\
        0.663749, 0.671685, 0.680273, 0.689082, 0.697274, 0.700056, 0.699413, 0.696719, 0.693339, 0.688574,\
        0.682368, 0.674736, 0.665921, 0.656566, 0.647422, 0.639113, 0.631719, 0.624431, 0.616063, 0.604622,\
        0.591684, 0.579459, 0.566251, 0.553843, 0.542437, 0.532003, 0.522855, 0.515011, 0.507229, 0.498966,\
        0.490320, 0.482746, 0.476072, 0.469343, 0.462430, 0.455497, 0.448788, 0.442481, 0.436726, 0.431634,\
        0.427234, 0.423466, 0.420166, 0.417045, 0.413757, 0.409906, 0.405326, 0.400516, 0.395823, 0.391005,\
        0.385038],\
        [0.401961, 0.408300, 0.414996, 0.421646, 0.428066, 0.434207, 0.440094, 0.445785, 0.451352, 0.456867,\
        0.462402, 0.468050, 0.473966, 0.480383, 0.487543, 0.495444, 0.503748, 0.512037, 0.520487, 0.529176,\
        0.537915, 0.546548, 0.554979, 0.563134, 0.570945, 0.578374, 0.585422, 0.591882, 0.596856, 0.603091,\
        0.609575, 0.614844, 0.619003, 0.622320, 0.624924, 0.627023, 0.628679, 0.629520, 0.629478, 0.629456,\
        0.629754, 0.629534, 0.628135, 0.625497, 0.622481, 0.619983, 0.617943, 0.615257, 0.614078, 0.615052,\
        0.618216, 0.626502, 0.636855, 0.646105, 0.653167, 0.657788, 0.659914, 0.660220, 0.659101, 0.656661,\
        0.653211, 0.648792, 0.643693, 0.638729, 0.635039, 0.633543, 0.634478, 0.637530, 0.642289, 0.648448,\
        0.655742, 0.663855, 0.672368, 0.680626, 0.687143, 0.689766, 0.688974, 0.686123, 0.682045, 0.676676,\
        0.669879, 0.661722, 0.652456, 0.642517, 0.632482, 0.623171, 0.615148, 0.607225, 0.598035, 0.586588,\
        0.572929, 0.560068, 0.547841, 0.536641, 0.526490, 0.516898, 0.508067, 0.501340, 0.493944, 0.485936,\
        0.477902, 0.470486, 0.463750, 0.457282, 0.450841, 0.444410, 0.438084, 0.432020, 0.426439, 0.421551,\
        0.417412, 0.413990, 0.411149, 0.408493, 0.405658, 0.402211, 0.397580, 0.392734, 0.387903, 0.382841,\
        0.377162],\
        [0.396735, 0.403369, 0.410381, 0.417234, 0.423757, 0.429951, 0.435861, 0.441554, 0.447113, 0.452627,\
        0.458194, 0.463931, 0.469984, 0.476515, 0.483643, 0.491340, 0.499337, 0.506806, 0.514775, 0.523097,\
        0.531417, 0.539582, 0.547541, 0.555261, 0.562695, 0.569793, 0.576480, 0.582600, 0.588303, 0.594520,\
        0.600523, 0.605151, 0.608581, 0.611305, 0.613144, 0.614547, 0.615894, 0.616194, 0.614378, 0.613003,\
        0.612888, 0.612012, 0.609643, 0.605487, 0.601141, 0.597988, 0.596186, 0.593128, 0.591621, 0.592193,\
        0.594159, 0.602292, 0.612126, 0.621012, 0.629228, 0.635113, 0.637984, 0.639118, 0.638969, 0.637421,\
        0.634565, 0.630708, 0.626223, 0.622138, 0.619848, 0.620237, 0.623116, 0.627673, 0.633397, 0.640068,\
        0.647465, 0.655267, 0.663018, 0.669990, 0.675077, 0.677291, 0.676685, 0.673972, 0.669688, 0.663936,\
        0.656674, 0.648020, 0.638272, 0.627741, 0.616705, 0.605911, 0.597078, 0.588212, 0.577883, 0.566015,\
        0.553134, 0.540666, 0.529297, 0.519347, 0.510715, 0.502192, 0.493490, 0.486506, 0.479565, 0.472167,\
        0.464820, 0.457898, 0.451471, 0.445361, 0.439384, 0.433442, 0.427495, 0.421634, 0.416155, 0.411454,\
        0.407581, 0.404371, 0.401957, 0.399600, 0.396978, 0.393718, 0.389443, 0.384701, 0.379799, 0.374645,\
        0.369074],\
        [0.392994, 0.399798, 0.406845, 0.413656, 0.420145, 0.426301, 0.432162, 0.437800, 0.443303, 0.448771,\
        0.454313, 0.460044, 0.466084, 0.472533, 0.479426, 0.486655, 0.493883, 0.501047, 0.508946, 0.517002,\
        0.524819, 0.532389, 0.539764, 0.546953, 0.553913, 0.560569, 0.566855, 0.572759, 0.578457, 0.584142,\
        0.589376, 0.593486, 0.596439, 0.598772, 0.599856, 0.600562, 0.601764, 0.602403, 0.598748, 0.595976,\
        0.595260, 0.593642, 0.590525, 0.585992, 0.581264, 0.577616, 0.574794, 0.572028, 0.570297, 0.570449,\
        0.573206, 0.579883, 0.588687, 0.597169, 0.607316, 0.613639, 0.616518, 0.618545, 0.619719, 0.619272,\
        0.617191, 0.614368, 0.610563, 0.607198, 0.606105, 0.608319, 0.613162, 0.618862, 0.625100, 0.631899,\
        0.639029, 0.646136, 0.652787, 0.658399, 0.662274, 0.663894, 0.663249, 0.660685, 0.656486, 0.650656,\
        0.643089, 0.633960, 0.623774, 0.612870, 0.601272, 0.589490, 0.578974, 0.568644, 0.557354, 0.545190,\
        0.532956, 0.521373, 0.510936, 0.502083, 0.494625, 0.487102, 0.479238, 0.471943, 0.464996, 0.458090,\
        0.451396, 0.445099, 0.439202, 0.433581, 0.428126, 0.422740, 0.417244, 0.411578, 0.406143, 0.401561,\
        0.398114, 0.395050, 0.392733, 0.390462, 0.387871, 0.384678, 0.380748, 0.376273, 0.371463, 0.366347,\
        0.360876],\
        [0.390249, 0.397338, 0.404200, 0.410821, 0.417176, 0.423204, 0.428936, 0.434444, 0.439821, 0.445171,\
        0.450603, 0.456221, 0.462121, 0.468362, 0.474939, 0.481753, 0.488661, 0.495749, 0.503371, 0.510967,\
        0.518015, 0.524841, 0.531538, 0.538136, 0.544566, 0.550714, 0.556511, 0.561974, 0.567179, 0.572110,\
        0.576505, 0.580031, 0.582623, 0.584365, 0.585127, 0.585454, 0.585834, 0.585332, 0.582277, 0.579019,\
        0.576787, 0.574250, 0.570752, 0.566453, 0.562043, 0.558127, 0.554703, 0.551765, 0.549943, 0.550028,\
        0.552726, 0.558538, 0.566781, 0.576404, 0.586864, 0.592955, 0.596417, 0.599492, 0.602162, 0.603001,\
        0.601465, 0.600502, 0.597599, 0.595499, 0.595742, 0.599379, 0.605531, 0.611329, 0.617491, 0.624118,\
        0.630710, 0.636862, 0.642258, 0.646531, 0.649256, 0.650140, 0.649209, 0.646705, 0.642773, 0.637205,\
        0.629540, 0.619830, 0.609177, 0.597951, 0.585992, 0.573726, 0.561885, 0.550177, 0.537940, 0.525300,\
        0.513629, 0.503237, 0.493829, 0.485947, 0.478978, 0.472058, 0.464859, 0.457695, 0.450747, 0.444121,\
        0.437972, 0.432338, 0.427078, 0.421994, 0.417059, 0.412314, 0.407463, 0.402045, 0.396822, 0.392445,\
        0.389047, 0.386193, 0.383749, 0.381380, 0.378740, 0.375600, 0.371870, 0.367619, 0.362963, 0.357959,\
        0.352611],\
        [0.387969, 0.395133, 0.401938, 0.408526, 0.414730, 0.420563, 0.426090, 0.431392, 0.436564, 0.441713,\
        0.446943, 0.452351, 0.458016, 0.463979, 0.470237, 0.476749, 0.483496, 0.490568, 0.498066, 0.504556,\
        0.510717, 0.516770, 0.522742, 0.528750, 0.534652, 0.540239, 0.545420, 0.550228, 0.554713, 0.558824,\
        0.562406, 0.565303, 0.567419, 0.568674, 0.569057, 0.568829, 0.568151, 0.566615, 0.563905, 0.560789,\
        0.557811, 0.554610, 0.550908, 0.546931, 0.543026, 0.539283, 0.535627, 0.532373, 0.530375, 0.530373,\
        0.532665, 0.537925, 0.546176, 0.556186, 0.566069, 0.573512, 0.578710, 0.582928, 0.586393, 0.588196,\
        0.588259, 0.588091, 0.587269, 0.587013, 0.588842, 0.593405, 0.599437, 0.604427, 0.610527, 0.616960,\
        0.622832, 0.627814, 0.631828, 0.634775, 0.636368, 0.636373, 0.634899, 0.632291, 0.628724, 0.623852,\
        0.616596, 0.606129, 0.594826, 0.583250, 0.570948, 0.558303, 0.545749, 0.533375, 0.520774, 0.507625,\
        0.496145, 0.487095, 0.478737, 0.471403, 0.464705, 0.458068, 0.451175, 0.444082, 0.437065, 0.430535,\
        0.424829, 0.419868, 0.415305, 0.410740, 0.406140, 0.401960, 0.397784, 0.392867, 0.388025, 0.383871,\
        0.380462, 0.377598, 0.375045, 0.372540, 0.369816, 0.366681, 0.363052, 0.358945, 0.354428, 0.349552,\
        0.344339],\
        [0.386014, 0.393362, 0.400296, 0.406715, 0.412670, 0.418250, 0.423513, 0.428540, 0.433432, 0.438296,\
        0.443239, 0.448353, 0.453707, 0.459338, 0.465250, 0.471427, 0.477861, 0.484519, 0.491107, 0.497002,\
        0.502821, 0.508027, 0.513282, 0.518791, 0.524230, 0.529205, 0.533633, 0.537638, 0.541315, 0.544596,\
        0.547360, 0.549609, 0.551274, 0.552109, 0.551974, 0.551024, 0.549445, 0.547217, 0.544505, 0.541610,\
        0.538493, 0.535012, 0.531140, 0.527471, 0.524226, 0.521017, 0.517328, 0.513687, 0.511399, 0.511566,\
        0.513208, 0.517952, 0.526630, 0.537042, 0.547284, 0.555944, 0.562737, 0.568101, 0.572244, 0.574994,\
        0.576539, 0.577523, 0.578372, 0.579850, 0.582828, 0.587495, 0.592882, 0.598193, 0.604613, 0.610728,\
        0.615688, 0.619247, 0.621703, 0.623375, 0.623916, 0.622909, 0.620591, 0.617591, 0.614258, 0.610245,\
        0.604254, 0.593715, 0.581550, 0.569264, 0.556570, 0.543608, 0.530663, 0.518124, 0.506338, 0.493671,\
        0.482346, 0.473245, 0.465404, 0.458428, 0.451946, 0.445500, 0.438742, 0.431586, 0.424298, 0.417586,\
        0.412231, 0.407935, 0.403986, 0.400097, 0.395606, 0.391714, 0.387785, 0.383537, 0.379276, 0.375431,\
        0.372120, 0.369246, 0.366617, 0.364008, 0.361204, 0.358043, 0.354446, 0.350410, 0.345983, 0.341216,\
        0.336132],\
        [0.384241, 0.391789, 0.398784, 0.405049, 0.410805, 0.416133, 0.421098, 0.425795, 0.430336, 0.434839,\
        0.439417, 0.444163, 0.449141, 0.454383, 0.459889, 0.465635, 0.471571, 0.477579, 0.483422, 0.488918,\
        0.494056, 0.498565, 0.503231, 0.508365, 0.513431, 0.517691, 0.521214, 0.524315, 0.527204, 0.529724,\
        0.531595, 0.533169, 0.534532, 0.535052, 0.534323, 0.532587, 0.530314, 0.527489, 0.524632, 0.522191,\
        0.519149, 0.515772, 0.511563, 0.508151, 0.505538, 0.503072, 0.499417, 0.495604, 0.493195, 0.493474,\
        0.494826, 0.499515, 0.508906, 0.519776, 0.530728, 0.540435, 0.548399, 0.554719, 0.559617, 0.563270,\
        0.565966, 0.568158, 0.570341, 0.573078, 0.576852, 0.581724, 0.587299, 0.593339, 0.599737, 0.605374,\
        0.609452, 0.611322, 0.611887, 0.612458, 0.612224, 0.610090, 0.606727, 0.602995, 0.599604, 0.595588,\
        0.589819, 0.580141, 0.568293, 0.555818, 0.542980, 0.529947, 0.516919, 0.504035, 0.492630, 0.481251,\
        0.470446, 0.461310, 0.453596, 0.446846, 0.440606, 0.434409, 0.427840, 0.420656, 0.412995, 0.405730,\
        0.400570, 0.396817, 0.393178, 0.389543, 0.385629, 0.381832, 0.378094, 0.374286, 0.370528, 0.367036,\
        0.363912, 0.361105, 0.358470, 0.355820, 0.352969, 0.349775, 0.346164, 0.342135, 0.337740, 0.333044,\
        0.328072],\
        [0.382492, 0.390141, 0.397086, 0.403366, 0.409028, 0.414121, 0.418762, 0.423078, 0.427204, 0.431275,\
        0.435417, 0.439732, 0.444280, 0.449084, 0.454130, 0.459373, 0.464734, 0.470088, 0.475275, 0.480167,\
        0.484674, 0.488844, 0.493074, 0.497676, 0.502320, 0.505626, 0.508196, 0.510344, 0.512557, 0.514612,\
        0.515321, 0.516114, 0.517494, 0.517856, 0.516497, 0.513865, 0.511247, 0.508109, 0.504598, 0.503271,\
        0.499944, 0.496935, 0.492862, 0.489432, 0.486963, 0.484448, 0.481194, 0.477855, 0.475819, 0.475889,\
        0.478184, 0.483851, 0.493150, 0.504453, 0.516229, 0.526775, 0.535569, 0.542601, 0.548169, 0.552610,\
        0.556249, 0.559459, 0.562665, 0.566297, 0.570676, 0.575885, 0.581788, 0.588176, 0.594610, 0.600032,\
        0.604319, 0.604245, 0.602423, 0.602037, 0.601508, 0.598047, 0.593925, 0.589632, 0.585554, 0.580834,\
        0.574397, 0.565410, 0.554496, 0.542658, 0.530233, 0.517432, 0.505014, 0.492380, 0.480536, 0.469475,\
        0.459242, 0.450343, 0.442808, 0.436312, 0.430394, 0.424586, 0.418425, 0.411489, 0.403780, 0.396299,\
        0.390821, 0.386803, 0.383163, 0.379581, 0.375946, 0.372336, 0.368785, 0.365284, 0.361897, 0.358730,\
        0.355836, 0.353173, 0.350620, 0.348009, 0.345166, 0.341956, 0.338308, 0.334229, 0.329797, 0.325121,\
        0.320244],\
        [0.380717, 0.388520, 0.395565, 0.401834, 0.407343, 0.412167, 0.416449, 0.420336, 0.423982, 0.427551,\
        0.431195, 0.435027, 0.439107, 0.443442, 0.448000, 0.452714, 0.457492, 0.462221, 0.466781, 0.471080,\
        0.475084, 0.478878, 0.482671, 0.486575, 0.490175, 0.492771, 0.494616, 0.495994, 0.497591, 0.499563,\
        0.498896, 0.498923, 0.500310, 0.500433, 0.498620, 0.495295, 0.492427, 0.489441, 0.486085, 0.484083,\
        0.481229, 0.478121, 0.474715, 0.471517, 0.468838, 0.466167, 0.463209, 0.460417, 0.458787, 0.459228,\
        0.462418, 0.468938, 0.478679, 0.490650, 0.503102, 0.514433, 0.523860, 0.531363, 0.537470, 0.542602,\
        0.547032, 0.551043, 0.554992, 0.559234, 0.564035, 0.569497, 0.575559, 0.582011, 0.588452, 0.594451,\
        0.598899, 0.597911, 0.594250, 0.592186, 0.590139, 0.586364, 0.581780, 0.577010, 0.572089, 0.566495,\
        0.559512, 0.550751, 0.540585, 0.529838, 0.518730, 0.506138, 0.494333, 0.481970, 0.469758, 0.458501,\
        0.448445, 0.439792, 0.432538, 0.426393, 0.420871, 0.415524, 0.409974, 0.403541, 0.396066, 0.388737,\
        0.382757, 0.378070, 0.374068, 0.370355, 0.366762, 0.363260, 0.359858, 0.356576, 0.353462, 0.350572,\
        0.347926, 0.345473, 0.343091, 0.340610, 0.337852, 0.334670, 0.330987, 0.326813, 0.322263, 0.317537,\
        0.312746],\
        [0.378862, 0.386879, 0.394076, 0.400361, 0.405709, 0.410225, 0.414130, 0.417543, 0.420639, 0.423632,\
        0.426718, 0.430032, 0.433626, 0.437487, 0.441557, 0.445750, 0.449964, 0.454097, 0.458058, 0.461788,\
        0.465276, 0.468577, 0.471765, 0.474806, 0.477431, 0.479361, 0.480654, 0.481593, 0.482544, 0.483258,\
        0.482762, 0.482382, 0.482669, 0.482164, 0.480176, 0.477160, 0.474076, 0.471046, 0.468114, 0.465557,\
        0.462863, 0.459945, 0.456957, 0.454125, 0.451516, 0.448895, 0.446146, 0.443653, 0.442285, 0.443084,\
        0.446921, 0.454195, 0.464585, 0.477095, 0.490278, 0.502647, 0.512600, 0.520426, 0.527052, 0.532897,\
        0.538032, 0.542645, 0.547082, 0.551686, 0.556701, 0.562232, 0.568227, 0.574473, 0.580585, 0.585924,\
        0.589086, 0.588425, 0.585258, 0.581950, 0.578529, 0.574362, 0.569568, 0.564435, 0.558929, 0.552711,\
        0.545361, 0.536646, 0.526786, 0.516967, 0.507853, 0.498584, 0.485056, 0.471774, 0.459189, 0.447745,\
        0.437748, 0.429314, 0.422389, 0.416706, 0.411592, 0.406535, 0.401400, 0.395456, 0.388318, 0.381318,\
        0.375198, 0.370082, 0.365705, 0.361774, 0.358107, 0.354624, 0.351307, 0.348171, 0.345255, 0.342599,\
        0.340209, 0.338023, 0.335904, 0.333659, 0.331082, 0.328001, 0.324320, 0.320046, 0.315301, 0.310387,\
        0.305712],\
        [0.376836, 0.385100, 0.392471, 0.398856, 0.403942, 0.408252, 0.411821, 0.414707, 0.417161, 0.419491,\
        0.421965, 0.424743, 0.427861, 0.431268, 0.434873, 0.438571, 0.442250, 0.445814, 0.449193, 0.452349,\
        0.455280, 0.457998, 0.460507, 0.462736, 0.464535, 0.465790, 0.466545, 0.466968, 0.467183, 0.467043,\
        0.466393, 0.465664, 0.465029, 0.463919, 0.461898, 0.459137, 0.456140, 0.453215, 0.450471, 0.447889,\
        0.445271, 0.442556, 0.439906, 0.437482, 0.435228, 0.432900, 0.430359, 0.427941, 0.426531, 0.427385,\
        0.431612, 0.439555, 0.450596, 0.463583, 0.477220, 0.490141, 0.500777, 0.509196, 0.516555, 0.523335,\
        0.529101, 0.534093, 0.538786, 0.543534, 0.548565, 0.553979, 0.559705, 0.565492, 0.570885, 0.575171,\
        0.577372, 0.576846, 0.574198, 0.570594, 0.566533, 0.562017, 0.557075, 0.551757, 0.545951, 0.539407,\
        0.531922, 0.523437, 0.513624, 0.503457, 0.495454, 0.485913, 0.473900, 0.460792, 0.448181, 0.436742,\
        0.426895, 0.418736, 0.412084, 0.406980, 0.402268, 0.397286, 0.392147, 0.386446, 0.380032, 0.373564,\
        0.367611, 0.362373, 0.357783, 0.353677, 0.349913, 0.346405, 0.343120, 0.340066, 0.337284, 0.334821,\
        0.332689, 0.330823, 0.329062, 0.327170, 0.324892, 0.322011, 0.318408, 0.314100, 0.309251, 0.303908,\
        0.299365],\
        [0.374547, 0.383108, 0.390637, 0.396877, 0.402007, 0.406423, 0.409631, 0.411863, 0.413542, 0.415107,\
        0.416921, 0.419177, 0.421859, 0.424849, 0.428027, 0.431268, 0.434445, 0.437456, 0.440254, 0.442828,\
        0.445173, 0.447280, 0.449119, 0.450620, 0.451688, 0.452283, 0.452463, 0.452326, 0.451908, 0.451156,\
        0.450098, 0.448938, 0.447759, 0.446313, 0.444277, 0.441642, 0.438772, 0.436041, 0.433546, 0.431173,\
        0.428717, 0.426146, 0.423766, 0.421820, 0.420134, 0.418352, 0.416084, 0.413647, 0.411913, 0.412467,\
        0.416854, 0.425458, 0.437117, 0.450390, 0.464076, 0.477037, 0.488306, 0.497581, 0.506164, 0.514100,\
        0.520169, 0.525252, 0.530027, 0.534757, 0.539648, 0.544815, 0.550169, 0.555405, 0.560041, 0.563452,\
        0.565002, 0.564391, 0.561928, 0.558277, 0.553971, 0.549275, 0.544281, 0.538947, 0.533094, 0.526451,\
        0.518924, 0.510896, 0.502725, 0.491916, 0.482596, 0.472655, 0.461188, 0.448823, 0.436635, 0.425411,\
        0.415833, 0.408239, 0.401731, 0.396904, 0.392581, 0.387772, 0.382717, 0.377275, 0.371438, 0.365520,\
        0.359885, 0.354736, 0.350103, 0.345919, 0.342096, 0.338562, 0.335281, 0.332259, 0.329550, 0.327229,\
        0.325346, 0.323849, 0.322543, 0.321127, 0.319271, 0.316700, 0.313266, 0.308982, 0.304192, 0.299694,\
        0.294003],\
        [0.371886, 0.380874, 0.388833, 0.395535, 0.400998, 0.405091, 0.407645, 0.409002, 0.409763, 0.410463,\
        0.411587, 0.413386, 0.415705, 0.418319, 0.421101, 0.423932, 0.426639, 0.429101, 0.431308, 0.433294,\
        0.435055, 0.436565, 0.437787, 0.438648, 0.439060, 0.439013, 0.438612, 0.437963, 0.437044, 0.435778,\
        0.434237, 0.432634, 0.431142, 0.429649, 0.427719, 0.425096, 0.422253, 0.419769, 0.417622, 0.415630,\
        0.413457, 0.410950, 0.408717, 0.407329, 0.406239, 0.405245, 0.403281, 0.401012, 0.399075, 0.399348,\
        0.403659, 0.412925, 0.424942, 0.438135, 0.451595, 0.464516, 0.476281, 0.486802, 0.496822, 0.505158,\
        0.510910, 0.515965, 0.520843, 0.525447, 0.530080, 0.534960, 0.539961, 0.544717, 0.548745, 0.551515,\
        0.552580, 0.551755, 0.549229, 0.545458, 0.540971, 0.536176, 0.531210, 0.525985, 0.520291, 0.513753,\
        0.506107, 0.498045, 0.490153, 0.480543, 0.470301, 0.459740, 0.448409, 0.436605, 0.425020, 0.414191,\
        0.404713, 0.397912, 0.392071, 0.387122, 0.382742, 0.378188, 0.373349, 0.368209, 0.362820, 0.357370,\
        0.352086, 0.347133, 0.342577, 0.338410, 0.334589, 0.331062, 0.327786, 0.324763, 0.322063, 0.319815,\
        0.318148, 0.317059, 0.316299, 0.315466, 0.314144, 0.311962, 0.308792, 0.304583, 0.299579, 0.294591,\
        0.288984],\
        [0.368665, 0.378190, 0.386818, 0.394271, 0.400154, 0.403992, 0.405668, 0.405968, 0.405794, 0.405597,\
        0.406008, 0.407507, 0.409546, 0.411770, 0.414160, 0.416650, 0.418930, 0.420814, 0.422396, 0.423810,\
        0.425022, 0.425965, 0.426645, 0.426985, 0.426822, 0.426147, 0.425172, 0.424122, 0.422886, 0.421196,\
        0.419177, 0.417135, 0.415438, 0.414178, 0.412634, 0.409913, 0.406899, 0.404780, 0.402976, 0.401393,\
        0.399648, 0.397289, 0.395261, 0.394250, 0.393535, 0.392911, 0.391479, 0.389712, 0.388607, 0.389548,\
        0.394159, 0.403260, 0.414696, 0.427259, 0.440200, 0.452931, 0.465010, 0.476272, 0.486609, 0.495077,\
        0.500695, 0.506381, 0.511477, 0.515791, 0.520066, 0.524734, 0.529514, 0.533940, 0.537557, 0.539896,\
        0.540580, 0.539473, 0.536710, 0.532634, 0.527818, 0.522878, 0.517954, 0.512834, 0.507301, 0.501131,\
        0.493439, 0.485395, 0.477114, 0.467960, 0.457932, 0.447290, 0.436095, 0.424719, 0.413866, 0.403961,\
        0.394998, 0.388199, 0.382719, 0.377869, 0.373391, 0.368915, 0.364257, 0.359382, 0.354346, 0.349274,\
        0.344317, 0.339597, 0.335185, 0.331105, 0.327351, 0.323885, 0.320650, 0.317616, 0.314870, 0.312610,\
        0.311077, 0.310428, 0.310271, 0.310058, 0.309366, 0.307555, 0.304558, 0.300787, 0.295442, 0.289732,\
        0.283618],\
        [0.364619, 0.374648, 0.383939, 0.392188, 0.398803, 0.402547, 0.403190, 0.402412, 0.401629, 0.400759,\
        0.400441, 0.401850, 0.403542, 0.405273, 0.407211, 0.409511, 0.411413, 0.412630, 0.413513, 0.414434,\
        0.415179, 0.415550, 0.415771, 0.415770, 0.415149, 0.413863, 0.412280, 0.410981, 0.409724, 0.407695,\
        0.405293, 0.402941, 0.401045, 0.400057, 0.399078, 0.396421, 0.393477, 0.391511, 0.389937, 0.388531,\
        0.386999, 0.385191, 0.383632, 0.382671, 0.382084, 0.381534, 0.380717, 0.379927, 0.380011, 0.382081,\
        0.387215, 0.395593, 0.406092, 0.417698, 0.429887, 0.442239, 0.454278, 0.465600, 0.475773, 0.484201,\
        0.490882, 0.497017, 0.502172, 0.505955, 0.509901, 0.514596, 0.519322, 0.523547, 0.526945, 0.529046,\
        0.529428, 0.527993, 0.524921, 0.520375, 0.514899, 0.509671, 0.504734, 0.499559, 0.493916, 0.487654,\
        0.480380, 0.472508, 0.464250, 0.455371, 0.445742, 0.435441, 0.424582, 0.413501, 0.403364, 0.394407,\
        0.386400, 0.379672, 0.374099, 0.369199, 0.364636, 0.360152, 0.355587, 0.350897, 0.346120, 0.341344,\
        0.336671, 0.332183, 0.327939, 0.323982, 0.320341, 0.317009, 0.313897, 0.310887, 0.308065, 0.305759,\
        0.304198, 0.304024, 0.304464, 0.304640, 0.304667, 0.303318, 0.299768, 0.296015, 0.290639, 0.284456,\
        0.277861],\
        [0.359506, 0.369849, 0.379542, 0.388277, 0.395464, 0.399020, 0.399438, 0.398225, 0.397133, 0.396118,\
        0.395699, 0.396441, 0.397632, 0.398843, 0.400301, 0.402607, 0.404065, 0.404528, 0.404600, 0.405238,\
        0.405660, 0.405354, 0.405176, 0.405082, 0.404119, 0.402354, 0.400212, 0.398742, 0.397563, 0.395403,\
        0.392897, 0.390580, 0.388734, 0.387569, 0.386352, 0.384294, 0.381929, 0.380005, 0.378483, 0.377140,\
        0.375794, 0.374422, 0.373242, 0.372432, 0.371939, 0.371604, 0.371375, 0.371531, 0.372695, 0.375642,\
        0.380990, 0.388793, 0.398454, 0.409055, 0.420458, 0.432380, 0.444188, 0.455316, 0.465330, 0.473934,\
        0.481213, 0.487488, 0.492637, 0.496343, 0.500396, 0.505216, 0.509909, 0.513897, 0.517293, 0.519388,\
        0.519516, 0.517623, 0.514277, 0.509474, 0.502852, 0.497113, 0.491892, 0.486356, 0.480434, 0.473992,\
        0.466943, 0.459418, 0.451513, 0.443100, 0.434028, 0.424347, 0.414289, 0.403683, 0.393955, 0.385719,\
        0.378295, 0.371795, 0.366165, 0.361152, 0.356483, 0.351942, 0.347397, 0.342804, 0.338196, 0.333643,\
        0.329209, 0.324937, 0.320861, 0.317028, 0.313506, 0.310364, 0.307517, 0.304626, 0.301747, 0.299517,\
        0.298048, 0.298119, 0.298851, 0.298969, 0.299325, 0.298171, 0.294761, 0.290130, 0.284633, 0.278320,\
        0.271464],\
        [0.353222, 0.363619, 0.373336, 0.381951, 0.388696, 0.392511, 0.393611, 0.393116, 0.392130, 0.391220,\
        0.390761, 0.390972, 0.391632, 0.392508, 0.393769, 0.395457, 0.396360, 0.396356, 0.395987, 0.396265,\
        0.396310, 0.395572, 0.395043, 0.394658, 0.393486, 0.391584, 0.389480, 0.387729, 0.386155, 0.384152,\
        0.381874, 0.379729, 0.377952, 0.376550, 0.375156, 0.373443, 0.371566, 0.369861, 0.368418, 0.367156,\
        0.365984, 0.364902, 0.364007, 0.363393, 0.363077, 0.363040, 0.363352, 0.364272, 0.366241, 0.369759,\
        0.375172, 0.382468, 0.391246, 0.400998, 0.411833, 0.423460, 0.434991, 0.445820, 0.455644, 0.464309,\
        0.471816, 0.478234, 0.483548, 0.487973, 0.492356, 0.497198, 0.501469, 0.505169, 0.508924, 0.511315,\
        0.511279, 0.508536, 0.504540, 0.500381, 0.493028, 0.486041, 0.479743, 0.473523, 0.467139, 0.460473,\
        0.453502, 0.446312, 0.438948, 0.431239, 0.422872, 0.413839, 0.404685, 0.395717, 0.386218, 0.377993,\
        0.370776, 0.364402, 0.358754, 0.353650, 0.348881, 0.344265, 0.339687, 0.335112, 0.330587, 0.326190,\
        0.321966, 0.317899, 0.313979, 0.310250, 0.306802, 0.303788, 0.301334, 0.298660, 0.295940, 0.293833,\
        0.292619, 0.292426, 0.292702, 0.292845, 0.292593, 0.291064, 0.287842, 0.283227, 0.277643, 0.271282,\
        0.264315],\
        [0.345856, 0.356109, 0.365624, 0.373933, 0.380411, 0.384544, 0.386437, 0.386766, 0.386316, 0.385684,\
        0.385245, 0.385182, 0.385469, 0.386034, 0.386863, 0.387753, 0.388184, 0.388020, 0.387614, 0.387342,\
        0.386925, 0.386159, 0.385360, 0.384496, 0.383193, 0.381415, 0.379467, 0.377622, 0.375843, 0.373928,\
        0.371905, 0.369974, 0.368277, 0.366802, 0.365383, 0.363877, 0.362322, 0.360857, 0.359557, 0.358416,\
        0.357407, 0.356540, 0.355869, 0.355461, 0.355366, 0.355642, 0.356411, 0.357900, 0.360426, 0.364307,\
        0.369737, 0.376681, 0.384872, 0.394024, 0.404358, 0.415819, 0.426899, 0.437258, 0.446785, 0.455387,\
        0.463005, 0.469631, 0.475328, 0.480346, 0.485117, 0.489803, 0.493933, 0.497913, 0.502083, 0.504857,\
        0.505294, 0.501191, 0.496008, 0.490474, 0.483260, 0.475479, 0.468103, 0.461070, 0.454168, 0.447231,\
        0.440224, 0.433286, 0.426543, 0.419793, 0.412340, 0.403817, 0.395223, 0.386870, 0.378490, 0.370650,\
        0.363622, 0.357352, 0.351722, 0.346579, 0.341747, 0.337068, 0.332431, 0.327807, 0.323272, 0.318966,\
        0.314945, 0.311086, 0.307311, 0.303679, 0.300298, 0.297345, 0.294930, 0.292516, 0.290130, 0.288204,\
        0.286972, 0.286411, 0.286186, 0.285856, 0.284984, 0.283061, 0.279804, 0.275289, 0.269759, 0.263419,\
        0.256426],\
        [0.337655, 0.347684, 0.356976, 0.365076, 0.371505, 0.375958, 0.378505, 0.379569, 0.379710, 0.379435,\
        0.379106, 0.378927, 0.378961, 0.379195, 0.379561, 0.379882, 0.379920, 0.379600, 0.379069, 0.378474,\
        0.377765, 0.376890, 0.375911, 0.374803, 0.373433, 0.371787, 0.370001, 0.368213, 0.366444, 0.364648,\
        0.362845, 0.361122, 0.359552, 0.358131, 0.356785, 0.355450, 0.354130, 0.352882, 0.351753, 0.350759,\
        0.349908, 0.349219, 0.348736, 0.348522, 0.348645, 0.349198, 0.350322, 0.352213, 0.355107, 0.359222,\
        0.364688, 0.371493, 0.379513, 0.388641, 0.398982, 0.409642, 0.419832, 0.429517, 0.438673, 0.447172,\
        0.454874, 0.461718, 0.467757, 0.473171, 0.478193, 0.482927, 0.487366, 0.491716, 0.495884, 0.499114,\
        0.501002, 0.495860, 0.488609, 0.481116, 0.473027, 0.464634, 0.456501, 0.448837, 0.441544, 0.434389,\
        0.427252, 0.420386, 0.414167, 0.408485, 0.402375, 0.394384, 0.386239, 0.378322, 0.370613, 0.363301,\
        0.356578, 0.350475, 0.344927, 0.339813, 0.334985, 0.330289, 0.325604, 0.320884, 0.316226, 0.311905,\
        0.308121, 0.304487, 0.300821, 0.297290, 0.294009, 0.291110, 0.288585, 0.286248, 0.284073, 0.282242,\
        0.280880, 0.279940, 0.279185, 0.278273, 0.276832, 0.274512, 0.271106, 0.266605, 0.261132, 0.254839,\
        0.247860],\
        [0.328954, 0.338780, 0.347937, 0.355968, 0.362465, 0.367209, 0.370251, 0.371881, 0.372519, 0.372580,\
        0.372395, 0.372181, 0.372042, 0.371995, 0.371988, 0.371910, 0.371639, 0.371136, 0.370455, 0.369665,\
        0.368779, 0.367794, 0.366717, 0.365526, 0.364176, 0.362665, 0.361053, 0.359412, 0.357778, 0.356156,\
        0.354568, 0.353057, 0.351658, 0.350370, 0.349160, 0.347997, 0.346879, 0.345829, 0.344876, 0.344040,\
        0.343342, 0.342810, 0.342488, 0.342440, 0.342747, 0.343515, 0.344887, 0.347036, 0.350154, 0.354409,\
        0.359908, 0.366674, 0.374668, 0.383826, 0.393891, 0.403905, 0.413226, 0.422255, 0.431140, 0.439602,\
        0.447393, 0.454404, 0.460656, 0.466277, 0.471423, 0.476216, 0.480732, 0.485018, 0.488886, 0.491898,\
        0.492383, 0.487651, 0.479927, 0.471355, 0.462428, 0.453425, 0.444732, 0.436682, 0.429290, 0.422151,\
        0.414896, 0.407821, 0.401946, 0.396849, 0.391534, 0.384647, 0.377208, 0.369839, 0.362688, 0.355872,\
        0.349504, 0.343633, 0.338234, 0.333225, 0.328481, 0.323849, 0.319182, 0.314382, 0.309507, 0.304935,\
        0.301399, 0.297991, 0.294402, 0.290951, 0.287742, 0.284841, 0.282243, 0.279881, 0.277738, 0.275867,\
        0.274302, 0.272977, 0.271711, 0.270241, 0.268280, 0.265572, 0.261955, 0.257387, 0.251923, 0.245660,\
        0.238701],\
        [0.320129, 0.329840, 0.338990, 0.347060, 0.353660, 0.358618, 0.361983, 0.363988, 0.364969, 0.365279,\
        0.365217, 0.364998, 0.364743, 0.364491, 0.364217, 0.363858, 0.363350, 0.362673, 0.361852, 0.360931,\
        0.359939, 0.358890, 0.357787, 0.356616, 0.355357, 0.354006, 0.352591, 0.351152, 0.349719, 0.348314,\
        0.346957, 0.345674, 0.344480, 0.343375, 0.342343, 0.341366, 0.340442, 0.339581, 0.338802, 0.338125,\
        0.337574, 0.337181, 0.336993, 0.337076, 0.337516, 0.338423, 0.339936, 0.342218, 0.345434, 0.349732,\
        0.355211, 0.361902, 0.369761, 0.378645, 0.388198, 0.397766, 0.406373, 0.415166, 0.424093, 0.432645,\
        0.440521, 0.447602, 0.453903, 0.459526, 0.464605, 0.469249, 0.473505, 0.477310, 0.480382, 0.482065,\
        0.481115, 0.476574, 0.469255, 0.460557, 0.451306, 0.441935, 0.432814, 0.424525, 0.417428, 0.410686,\
        0.403662, 0.396457, 0.390659, 0.385628, 0.380436, 0.374458, 0.367917, 0.361243, 0.354672, 0.348338,\
        0.342337, 0.336728, 0.331524, 0.326682, 0.322097, 0.317619, 0.313088, 0.308336, 0.303329, 0.298525,\
        0.294746, 0.291394, 0.287933, 0.284555, 0.281374, 0.278445, 0.275771, 0.273328, 0.271099, 0.269082,\
        0.267259, 0.265550, 0.263807, 0.261829, 0.259395, 0.256315, 0.252454, 0.247762, 0.242253, 0.235985,\
        0.229032],\
        [0.311607, 0.321347, 0.330608, 0.338747, 0.345395, 0.350430, 0.353925, 0.356098, 0.357247, 0.357685,\
        0.357690, 0.357466, 0.357137, 0.356751, 0.356302, 0.355752, 0.355069, 0.354246, 0.353307, 0.352292,\
        0.351241, 0.350179, 0.349112, 0.348032, 0.346919, 0.345763, 0.344572, 0.343367, 0.342173, 0.341012,\
        0.339906, 0.338869, 0.337909, 0.337020, 0.336193, 0.335417, 0.334689, 0.334015, 0.333408, 0.332886,\
        0.332473, 0.332202, 0.332121, 0.332295, 0.332809, 0.333775, 0.335328, 0.337624, 0.340821, 0.345060,\
        0.350437, 0.356974, 0.364599, 0.373100, 0.382077, 0.390923, 0.399390, 0.408503, 0.417696, 0.426369,\
        0.434255, 0.441259, 0.447410, 0.452812, 0.457594, 0.461846, 0.465570, 0.468623, 0.470657, 0.471069,\
        0.469116, 0.464415, 0.457392, 0.448930, 0.439803, 0.430479, 0.421118, 0.412347, 0.405941, 0.399807,\
        0.393046, 0.386300, 0.380332, 0.375065, 0.369893, 0.364403, 0.358566, 0.352562, 0.346565, 0.340697,\
        0.335050, 0.329697, 0.324693, 0.320045, 0.315666, 0.311392, 0.307065, 0.302567, 0.297554, 0.292746,\
        0.288596, 0.284931, 0.281429, 0.278052, 0.274851, 0.271861, 0.269088, 0.266519, 0.264133, 0.261903,\
        0.259787, 0.257699, 0.255515, 0.253076, 0.250213, 0.246776, 0.242658, 0.237803, 0.232206, 0.225896,\
        0.218923],\
        [0.303989, 0.313909, 0.323266, 0.331363, 0.337897, 0.342814, 0.346224, 0.348351, 0.349482, 0.349917,\
        0.349915, 0.349670, 0.349297, 0.348841, 0.348295, 0.347630, 0.346826, 0.345888, 0.344851, 0.343767,\
        0.342687, 0.341652, 0.340674, 0.339740, 0.338822, 0.337893, 0.336947, 0.335994, 0.335057, 0.334158,\
        0.333318, 0.332544, 0.331836, 0.331187, 0.330585, 0.330021, 0.329492, 0.329004, 0.328566, 0.328195,\
        0.327911, 0.327746, 0.327744, 0.327968, 0.328500, 0.329447, 0.330942, 0.333141, 0.336205, 0.340284,\
        0.345481, 0.351820, 0.359215, 0.367441, 0.376138, 0.384921, 0.393736, 0.402956, 0.412223, 0.420857,\
        0.428596, 0.435332, 0.441110, 0.446060, 0.450325, 0.453991, 0.457029, 0.459257, 0.460319, 0.459729,\
        0.457024, 0.452049, 0.445135, 0.436946, 0.428181, 0.419424, 0.410742, 0.401174, 0.394795, 0.388993,\
        0.382565, 0.376211, 0.370334, 0.364967, 0.359829, 0.354649, 0.349326, 0.343889, 0.338413, 0.332974,\
        0.327647, 0.322510, 0.317656, 0.313187, 0.309023, 0.304934, 0.300708, 0.296283, 0.291505, 0.286759,\
        0.282434, 0.278517, 0.274867, 0.271409, 0.268132, 0.265044, 0.262142, 0.259411, 0.256824, 0.254345,\
        0.251917, 0.249461, 0.246867, 0.244008, 0.240750, 0.236976, 0.232595, 0.227553, 0.221834, 0.215450,\
        0.208430],\
        [0.298413, 0.308138, 0.317314, 0.325094, 0.331269, 0.335839, 0.338940, 0.340810, 0.341746, 0.342045,\
        0.341964, 0.341680, 0.341293, 0.340827, 0.340258, 0.339543, 0.338662, 0.337635, 0.336514, 0.335370,\
        0.334279, 0.333301, 0.332454, 0.331716, 0.331034, 0.330360, 0.329671, 0.328975, 0.328298, 0.327666,\
        0.327098, 0.326600, 0.326161, 0.325766, 0.325400, 0.325053, 0.324725, 0.324421, 0.324150, 0.323924,\
        0.323760, 0.323686, 0.323740, 0.323979, 0.324477, 0.325334, 0.326681, 0.328676, 0.331497, 0.335318,\
        0.340271, 0.346404, 0.353647, 0.361801, 0.370573, 0.379682, 0.388998, 0.398455, 0.407592, 0.416065,\
        0.423503, 0.429761, 0.434932, 0.439207, 0.442772, 0.445728, 0.448043, 0.449518, 0.449808, 0.448496,\
        0.445240, 0.439966, 0.433021, 0.425041, 0.416603, 0.408435, 0.401160, 0.393107, 0.385186, 0.378621,\
        0.372291, 0.366170, 0.360435, 0.355114, 0.350076, 0.345153, 0.340237, 0.335279, 0.330263, 0.325205,\
        0.320159, 0.315194, 0.310382, 0.306019, 0.302053, 0.298049, 0.293869, 0.289467, 0.284888, 0.280314,\
        0.275970, 0.271924, 0.268139, 0.264564, 0.261174, 0.257958, 0.254900, 0.251980, 0.249167, 0.246418,\
        0.243676, 0.240863, 0.237887, 0.234641, 0.231019, 0.226924, 0.222279, 0.217037, 0.211172, 0.204686,\
        0.197596],\
        [0.294569, 0.304120, 0.312699, 0.319830, 0.325426, 0.329450, 0.332044, 0.333470, 0.334050, 0.334101,\
        0.333880, 0.333550, 0.333184, 0.332774, 0.332255, 0.331550, 0.330628, 0.329527, 0.328327, 0.327123,\
        0.326021, 0.325117, 0.324440, 0.323943, 0.323537, 0.323136, 0.322701, 0.322250, 0.321823, 0.321453,\
        0.321159, 0.320943, 0.320784, 0.320651, 0.320523, 0.320393, 0.320262, 0.320137, 0.320028, 0.319943,\
        0.319893, 0.319897, 0.319989, 0.320216, 0.320640, 0.321349, 0.322467, 0.324159, 0.326627, 0.330090,\
        0.334738, 0.340669, 0.347846, 0.356099, 0.365162, 0.374744, 0.384564, 0.394264, 0.403461, 0.411882,\
        0.418878, 0.424444, 0.428775, 0.432176, 0.434903, 0.437095, 0.438734, 0.439617, 0.439391, 0.437650,\
        0.434072, 0.428542, 0.421411, 0.413647, 0.405496, 0.397551, 0.390497, 0.383176, 0.375638, 0.368681,\
        0.362242, 0.356214, 0.350603, 0.345394, 0.340504, 0.335826, 0.331266, 0.326738, 0.322138, 0.317412,\
        0.312603, 0.307832, 0.303043, 0.298552, 0.294711, 0.290733, 0.286596, 0.282285, 0.277855, 0.273422,\
        0.269121, 0.265025, 0.261141, 0.257452, 0.253934, 0.250569, 0.247337, 0.244212, 0.241159, 0.238134,\
        0.235080, 0.231926, 0.228590, 0.224986, 0.221023, 0.216623, 0.211719, 0.206267, 0.200240, 0.193632,\
        0.186450],\
        [0.291388, 0.300954, 0.308617, 0.315142, 0.320127, 0.323486, 0.325429, 0.326269, 0.326364, 0.326080,\
        0.325687, 0.325321, 0.325018, 0.324739, 0.324361, 0.323725, 0.322776, 0.321604, 0.320332, 0.319058,\
        0.317926, 0.317102, 0.316628, 0.316415, 0.316322, 0.316202, 0.315996, 0.315759, 0.315563, 0.315439,\
        0.315409, 0.315480, 0.315609, 0.315737, 0.315840, 0.315918, 0.315974, 0.316019, 0.316067, 0.316121,\
        0.316178, 0.316251, 0.316369, 0.316569, 0.316897, 0.317418, 0.318244, 0.319542, 0.321545, 0.324539,\
        0.328806, 0.334525, 0.341697, 0.350155, 0.359618, 0.369749, 0.380165, 0.390430, 0.400112, 0.408120,\
        0.414553, 0.419213, 0.422480, 0.424858, 0.426668, 0.428094, 0.429162, 0.429661, 0.429194, 0.427318,\
        0.423721, 0.418186, 0.410644, 0.402871, 0.395542, 0.387667, 0.380303, 0.373008, 0.365718, 0.358759,\
        0.352293, 0.346328, 0.340834, 0.335760, 0.331040, 0.326588, 0.322341, 0.318235, 0.314032, 0.309583,\
        0.304944, 0.300293, 0.295772, 0.291261, 0.287176, 0.283156, 0.279037, 0.274797, 0.270472, 0.266141,\
        0.261888, 0.257772, 0.253818, 0.250024, 0.246375, 0.242854, 0.239438, 0.236099, 0.232802, 0.229500,\
        0.226142, 0.222662, 0.218989, 0.215049, 0.210767, 0.206076, 0.200918, 0.195252, 0.189052, 0.182306,\
        0.175014],\
        [0.287280, 0.296397, 0.304198, 0.310670, 0.315077, 0.317703, 0.318937, 0.319117, 0.318642, 0.317974,\
        0.317419, 0.317047, 0.316845, 0.316774, 0.316651, 0.316145, 0.315144, 0.313886, 0.312581, 0.311238,\
        0.310029, 0.309284, 0.309045, 0.309138, 0.309387, 0.309542, 0.309501, 0.309430, 0.309448, 0.309549,\
        0.309757, 0.310118, 0.310551, 0.310922, 0.311230, 0.311504, 0.311730, 0.311925, 0.312124, 0.312319,\
        0.312480, 0.312610, 0.312745, 0.312920, 0.313151, 0.313469, 0.313966, 0.314803, 0.316231, 0.318622,\
        0.322410, 0.327891, 0.335088, 0.343798, 0.353696, 0.364389, 0.375430, 0.386264, 0.396057, 0.404275,\
        0.410363, 0.413790, 0.415795, 0.417129, 0.418022, 0.418717, 0.419342, 0.419674, 0.419224, 0.417425,\
        0.414031, 0.409151, 0.401889, 0.393436, 0.385970, 0.378261, 0.370614, 0.363106, 0.355777, 0.348812,\
        0.342370, 0.336496, 0.331134, 0.326203, 0.321661, 0.317408, 0.313359, 0.309692, 0.305914, 0.301597,\
        0.297165, 0.292574, 0.288106, 0.283721, 0.279512, 0.275394, 0.271251, 0.267042, 0.262780, 0.258510,\
        0.254289, 0.250159, 0.246145, 0.242253, 0.238475, 0.234795, 0.231192, 0.227637, 0.224095, 0.220523,\
        0.216871, 0.213080, 0.209090, 0.204835, 0.200251, 0.195281, 0.189875, 0.183994, 0.177613, 0.170717,\
        0.163303],\
        [0.282425, 0.291842, 0.300102, 0.306265, 0.309877, 0.311760, 0.312378, 0.311947, 0.310862, 0.309805,\
        0.309173, 0.308839, 0.308730, 0.308896, 0.309192, 0.308885, 0.307730, 0.306336, 0.305128, 0.303778,\
        0.302427, 0.301784, 0.301784, 0.302113, 0.302693, 0.303110, 0.303125, 0.303182, 0.303429, 0.303729,\
        0.304115, 0.304770, 0.305538, 0.306083, 0.306551, 0.307026, 0.307396, 0.307700, 0.308052, 0.308401,\
        0.308656, 0.308821, 0.308963, 0.309129, 0.309292, 0.309415, 0.309580, 0.309940, 0.310709, 0.312343,\
        0.315528, 0.320746, 0.327984, 0.336942, 0.347227, 0.358400, 0.369973, 0.381385, 0.391988, 0.401065,\
        0.405966, 0.407578, 0.408381, 0.408951, 0.409013, 0.409026, 0.409331, 0.409654, 0.409394, 0.407811,\
        0.404284, 0.400049, 0.393692, 0.385332, 0.377155, 0.369150, 0.361229, 0.353454, 0.345943, 0.338887,\
        0.332466, 0.326725, 0.321551, 0.316744, 0.312364, 0.308395, 0.304278, 0.300887, 0.297625, 0.293363,\
        0.289010, 0.284578, 0.280172, 0.275839, 0.271597, 0.267417, 0.263240, 0.259033, 0.254796, 0.250554,\
        0.246342, 0.242189, 0.238115, 0.234127, 0.230221, 0.226383, 0.222593, 0.218824, 0.215042, 0.211207,\
        0.207273, 0.203188, 0.198897, 0.194345, 0.189475, 0.184237, 0.178589, 0.172494, 0.165928, 0.158874,\
        0.151326],\
        [0.276669, 0.286437, 0.294980, 0.300539, 0.303974, 0.305177, 0.305595, 0.304747, 0.303126, 0.301726,\
        0.301178, 0.300888, 0.300837, 0.301103, 0.301968, 0.301856, 0.300492, 0.298951, 0.297897, 0.296674,\
        0.295449, 0.294882, 0.294946, 0.295388, 0.296073, 0.296617, 0.296790, 0.297016, 0.297442, 0.297923,\
        0.298473, 0.299389, 0.300464, 0.301044, 0.301645, 0.302365, 0.302836, 0.303174, 0.303705, 0.304226,\
        0.304550, 0.304714, 0.304842, 0.305022, 0.305184, 0.305115, 0.304977, 0.304939, 0.305057, 0.305792,\
        0.308253, 0.313233, 0.320509, 0.329636, 0.340172, 0.351628, 0.363474, 0.375165, 0.386075, 0.394985,\
        0.399522, 0.399835, 0.400128, 0.400459, 0.399883, 0.399382, 0.399403, 0.399683, 0.399405, 0.398175,\
        0.394473, 0.389941, 0.383960, 0.376393, 0.368303, 0.360152, 0.352028, 0.344006, 0.336246, 0.329019,\
        0.322596, 0.317030, 0.312179, 0.307522, 0.303049, 0.299575, 0.295591, 0.291987, 0.288617, 0.284722,\
        0.280535, 0.276243, 0.271929, 0.267642, 0.263399, 0.259188, 0.254982, 0.250763, 0.246527, 0.242286,\
        0.238060, 0.233870, 0.229730, 0.225645, 0.221611, 0.217616, 0.213641, 0.209662, 0.205646, 0.201558,\
        0.197353, 0.192988, 0.188413, 0.183578, 0.178436, 0.172942, 0.167058, 0.160751, 0.153997, 0.146779,\
        0.139089],\
        [0.269521, 0.279137, 0.287207, 0.293032, 0.296815, 0.297672, 0.298358, 0.297361, 0.295681, 0.294301,\
        0.293635, 0.293356, 0.293362, 0.293743, 0.294418, 0.294337, 0.293255, 0.291912, 0.290788, 0.289732,\
        0.288823, 0.288360, 0.288405, 0.288827, 0.289441, 0.290009, 0.290432, 0.290866, 0.291414, 0.292062,\
        0.292843, 0.293851, 0.294900, 0.295695, 0.296462, 0.297294, 0.297860, 0.298284, 0.298963, 0.299599,\
        0.299961, 0.300110, 0.300214, 0.300393, 0.300628, 0.300329, 0.299942, 0.299629, 0.299283, 0.299278,\
        0.301046, 0.305807, 0.312995, 0.322083, 0.332649, 0.344106, 0.355827, 0.367190, 0.377431, 0.385397,\
        0.389800, 0.391079, 0.391397, 0.391408, 0.390860, 0.390237, 0.389952, 0.389784, 0.389184, 0.387512,\
        0.384198, 0.379592, 0.373772, 0.366791, 0.359102, 0.351095, 0.342955, 0.334788, 0.326765, 0.319265,\
        0.312817, 0.307440, 0.302898, 0.298832, 0.294198, 0.290494, 0.286806, 0.283163, 0.279585, 0.275795,\
        0.271770, 0.267608, 0.263384, 0.259146, 0.254914, 0.250690, 0.246463, 0.242223, 0.237970, 0.233709,\
        0.229452, 0.225212, 0.220997, 0.216809, 0.212645, 0.208494, 0.204338, 0.200153, 0.195911, 0.191578,\
        0.187116, 0.182483, 0.177638, 0.172535, 0.167133, 0.161393, 0.155279, 0.148762, 0.141820, 0.134434,\
        0.126595],\
        [0.260946, 0.270236, 0.277987, 0.283835, 0.287573, 0.289301, 0.289880, 0.289340, 0.288141, 0.287018,\
        0.286317, 0.286011, 0.286023, 0.286303, 0.286616, 0.286486, 0.285756, 0.284742, 0.283753, 0.282877,\
        0.282204, 0.281869, 0.281920, 0.282289, 0.282840, 0.283433, 0.284006, 0.284605, 0.285286, 0.286071,\
        0.286980, 0.288009, 0.289050, 0.289993, 0.290880, 0.291717, 0.292409, 0.293022, 0.293690, 0.294297,\
        0.294693, 0.294895, 0.295013, 0.295135, 0.295178, 0.294843, 0.294314, 0.293784, 0.293313, 0.293362,\
        0.295015, 0.299205, 0.305867, 0.314560, 0.324937, 0.336104, 0.347320, 0.357952, 0.367282, 0.374488,\
        0.378989, 0.381113, 0.381878, 0.381945, 0.381548, 0.380970, 0.380425, 0.379794, 0.378701, 0.376686,\
        0.373439, 0.368979, 0.363412, 0.356878, 0.349639, 0.341962, 0.334023, 0.325910, 0.317724, 0.309825,\
        0.303269, 0.298223, 0.293723, 0.289710, 0.285531, 0.281619, 0.277894, 0.274223, 0.270544, 0.266751,\
        0.262802, 0.258726, 0.254571, 0.250372, 0.246153, 0.241920, 0.237672, 0.233407, 0.229124, 0.224827,\
        0.220524, 0.216220, 0.211920, 0.207625, 0.203329, 0.199021, 0.194686, 0.190301, 0.185840, 0.181272,\
        0.176563, 0.171676, 0.166573, 0.161215, 0.155564, 0.149586, 0.143249, 0.136525, 0.129394, 0.121839,\
        0.113847],\
        [0.251163, 0.260144, 0.267653, 0.273442, 0.277396, 0.279668, 0.280644, 0.280634, 0.280031, 0.279309,\
        0.278771, 0.278505, 0.278488, 0.278631, 0.278745, 0.278594, 0.278094, 0.277373, 0.276616, 0.275950,\
        0.275470, 0.275258, 0.275343, 0.275688, 0.276210, 0.276825, 0.277486, 0.278199, 0.278987, 0.279867,\
        0.280840, 0.281884, 0.282938, 0.283944, 0.284891, 0.285763, 0.286538, 0.287241, 0.287901, 0.288468,\
        0.288873, 0.289113, 0.289238, 0.289277, 0.289170, 0.288829, 0.288328, 0.287829, 0.287555, 0.287962,\
        0.289757, 0.293510, 0.299267, 0.307406, 0.317475, 0.328019, 0.338411, 0.348119, 0.356566, 0.363203,\
        0.367739, 0.370361, 0.371602, 0.371960, 0.371762, 0.371243, 0.370527, 0.369544, 0.368059, 0.365783,\
        0.362512, 0.358201, 0.352915, 0.346786, 0.340001, 0.332759, 0.325214, 0.317447, 0.309436, 0.301431,\
        0.294603, 0.289474, 0.284961, 0.280780, 0.276711, 0.272771, 0.268960, 0.265208, 0.261449, 0.257618,\
        0.253682, 0.249642, 0.245521, 0.241344, 0.237128, 0.232881, 0.228608, 0.224310, 0.219987, 0.215641,\
        0.211278, 0.206898, 0.202506, 0.198097, 0.193666, 0.189203, 0.184691, 0.180110, 0.175437, 0.170643,\
        0.165697, 0.160567, 0.155218, 0.149615, 0.143726, 0.137518, 0.130964, 0.124037, 0.116719, 0.108991,\
        0.100843],\
        [0.240397, 0.249131, 0.256478, 0.262277, 0.266481, 0.269207, 0.270707, 0.271280, 0.271265, 0.271002,\
        0.270743, 0.270608, 0.270613, 0.270695, 0.270738, 0.270617, 0.270288, 0.269809, 0.269291, 0.268839,\
        0.268538, 0.268448, 0.268590, 0.268946, 0.269469, 0.270107, 0.270824, 0.271612, 0.272473, 0.273413,\
        0.274422, 0.275477, 0.276539, 0.277570, 0.278545, 0.279448, 0.280267, 0.281004, 0.281659, 0.282205,\
        0.282610, 0.282867, 0.282993, 0.282995, 0.282855, 0.282568, 0.282207, 0.281934, 0.282005, 0.282813,\
        0.284831, 0.288438, 0.293845, 0.301547, 0.310682, 0.320107, 0.329418, 0.338120, 0.345731, 0.351859,\
        0.356316, 0.359193, 0.360791, 0.361452, 0.361460, 0.361011, 0.360197, 0.358986, 0.357241, 0.354788,\
        0.351496, 0.347320, 0.342301, 0.336544, 0.330194, 0.323415, 0.316354, 0.309126, 0.301759, 0.293965,\
        0.287064, 0.281400, 0.276544, 0.272125, 0.267941, 0.263910, 0.259993, 0.256136, 0.252284, 0.248390,\
        0.244424, 0.240378, 0.236258, 0.232078, 0.227849, 0.223580, 0.219273, 0.214932, 0.210558, 0.206153,\
        0.201717, 0.197253, 0.192759, 0.188231, 0.183663, 0.179043, 0.174357, 0.169585, 0.164706, 0.159694,\
        0.154521, 0.149157, 0.143573, 0.137736, 0.131616, 0.125187, 0.118420, 0.111294, 0.103790, 0.095890,\
        0.087583]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=50 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax50vt10
    zgrid_H_Hmax_vmax50vt10=[[0.29917, 0.29855, 0.29706, 0.29547, 0.29400, 0.29293, 0.29220, 0.29152, 0.29083, 0.29014, 0.28942,\
        0.28874, 0.28824, 0.28792, 0.28771, 0.28756, 0.28745, 0.28740, 0.28741, 0.28738, 0.28716, 0.28668,\
        0.28607, 0.28556, 0.28539, 0.28578, 0.28686, 0.28872, 0.29137, 0.29473, 0.29875, 0.30356, 0.30914,\
        0.31537, 0.32211, 0.32924, 0.33661, 0.34407, 0.35146, 0.35863, 0.36540, 0.37162, 0.37715, 0.38193,\
        0.38602, 0.38959, 0.39283, 0.39588, 0.39882, 0.40243, 0.40667, 0.41138, 0.41653, 0.42207, 0.42797,\
        0.43418, 0.44068, 0.44743, 0.45441, 0.46163, 0.46908, 0.47674, 0.48449, 0.49191, 0.49856, 0.50582,\
        0.51326, 0.52055, 0.52751, 0.53403, 0.54003, 0.54549, 0.55044, 0.55494, 0.55905, 0.56283, 0.56633,\
        0.56960, 0.57261, 0.57532, 0.57775, 0.57996, 0.58197, 0.58383, 0.58565, 0.58736, 0.58885, 0.59029,\
        0.59171, 0.59286, 0.59293, 0.59257, 0.59227, 0.59153, 0.59085, 0.59021, 0.58971, 0.58931, 0.58835,\
        0.58671, 0.58445, 0.58140, 0.57756, 0.57310, 0.56813, 0.56267, 0.55673, 0.55033, 0.54351, 0.53630,\
        0.52874, 0.52085, 0.51267, 0.50420, 0.49546, 0.48645, 0.47717, 0.46765, 0.45786, 0.44782, 0.43753],\
        [0.30527, 0.30478, 0.30335, 0.30172, 0.30014, 0.29911, 0.29852, 0.29781, 0.29706, 0.29632, 0.29545,\
        0.29455, 0.29395, 0.29354, 0.29321, 0.29289, 0.29255, 0.29227, 0.29212, 0.29199, 0.29154, 0.29060,\
        0.28940, 0.28829, 0.28760, 0.28756, 0.28833, 0.28998, 0.29257, 0.29604, 0.30007, 0.30486, 0.31060,\
        0.31706, 0.32409, 0.33155, 0.33926, 0.34707, 0.35482, 0.36233, 0.36943, 0.37590, 0.38156, 0.38630,\
        0.39018, 0.39343, 0.39641, 0.39939, 0.40225, 0.40566, 0.40984, 0.41460, 0.41988, 0.42561, 0.43173,\
        0.43820, 0.44495, 0.45196, 0.45920, 0.46667, 0.47441, 0.48247, 0.49080, 0.49914, 0.50636, 0.51371,\
        0.52138, 0.52893, 0.53613, 0.54282, 0.54888, 0.55430, 0.55915, 0.56353, 0.56751, 0.57115, 0.57453,\
        0.57775, 0.58076, 0.58340, 0.58570, 0.58781, 0.58973, 0.59138, 0.59317, 0.59493, 0.59619, 0.59752,\
        0.59887, 0.60029, 0.59996, 0.59936, 0.59941, 0.59828, 0.59771, 0.59693, 0.59635, 0.59652, 0.59558,\
        0.59387, 0.59176, 0.58865, 0.58460, 0.58003, 0.57501, 0.56951, 0.56351, 0.55702, 0.55010, 0.54280,\
        0.53516, 0.52722, 0.51899, 0.51049, 0.50172, 0.49270, 0.48342, 0.47389, 0.46410, 0.45407, 0.44378],\
        [0.31091, 0.31073, 0.30966, 0.30824, 0.30683, 0.30582, 0.30513, 0.30439, 0.30359, 0.30280, 0.30179,\
        0.30071, 0.30005, 0.29950, 0.29903, 0.29848, 0.29787, 0.29725, 0.29691, 0.29670, 0.29604, 0.29451,\
        0.29262, 0.29088, 0.28966, 0.28920, 0.28963, 0.29102, 0.29346, 0.29698, 0.30127, 0.30608, 0.31191,\
        0.31861, 0.32592, 0.33367, 0.34168, 0.34980, 0.35786, 0.36571, 0.37313, 0.37989, 0.38571, 0.39039,\
        0.39395, 0.39673, 0.39932, 0.40208, 0.40505, 0.40852, 0.41270, 0.41756, 0.42300, 0.42896, 0.43536,\
        0.44212, 0.44917, 0.45646, 0.46395, 0.47164, 0.47958, 0.48786, 0.49652, 0.50529, 0.51359, 0.52161,\
        0.52963, 0.53750, 0.54498, 0.55184, 0.55795, 0.56327, 0.56796, 0.57217, 0.57601, 0.57946, 0.58261,\
        0.58581, 0.58893, 0.59152, 0.59362, 0.59565, 0.59763, 0.59894, 0.60064, 0.60250, 0.60364, 0.60485,\
        0.60604, 0.60695, 0.60688, 0.60648, 0.60617, 0.60532, 0.60464, 0.60373, 0.60308, 0.60327, 0.60227,\
        0.60046, 0.59871, 0.59524, 0.59099, 0.58643, 0.58144, 0.57595, 0.56989, 0.56330, 0.55628, 0.54888,\
        0.54118, 0.53318, 0.52492, 0.51639, 0.50761, 0.49858, 0.48931, 0.47978, 0.47001, 0.45998, 0.44971],\
        [0.31655, 0.31666, 0.31601, 0.31495, 0.31382, 0.31287, 0.31207, 0.31130, 0.31048, 0.30959, 0.30857,\
        0.30753, 0.30668, 0.30594, 0.30522, 0.30441, 0.30349, 0.30255, 0.30187, 0.30134, 0.30038, 0.29815,\
        0.29565, 0.29335, 0.29166, 0.29085, 0.29097, 0.29205, 0.29411, 0.29730, 0.30162, 0.30687, 0.31309,\
        0.32010, 0.32769, 0.33567, 0.34389, 0.35222, 0.36052, 0.36861, 0.37633, 0.38339, 0.38945, 0.39408,\
        0.39722, 0.39941, 0.40151, 0.40408, 0.40713, 0.41076, 0.41510, 0.42015, 0.42585, 0.43212, 0.43885,\
        0.44597, 0.45338, 0.46101, 0.46879, 0.47670, 0.48480, 0.49323, 0.50216, 0.51145, 0.52053, 0.52931,\
        0.53787, 0.54615, 0.55398, 0.56108, 0.56723, 0.57239, 0.57680, 0.58080, 0.58457, 0.58785, 0.59052,\
        0.59363, 0.59708, 0.59970, 0.60161, 0.60355, 0.60545, 0.60689, 0.60840, 0.61000, 0.61127, 0.61237,\
        0.61332, 0.61394, 0.61402, 0.61376, 0.61330, 0.61261, 0.61183, 0.61098, 0.61030, 0.60976, 0.60864,\
        0.60686, 0.60452, 0.60071, 0.59686, 0.59232, 0.58749, 0.58205, 0.57588, 0.56915, 0.56200, 0.55452,\
        0.54676, 0.53873, 0.53045, 0.52191, 0.51314, 0.50412, 0.49485, 0.48533, 0.47558, 0.46557, 0.45532],\
        [0.32229, 0.32271, 0.32244, 0.32177, 0.32093, 0.32010, 0.31932, 0.31855, 0.31772, 0.31682, 0.31582,\
        0.31479, 0.31380, 0.31284, 0.31185, 0.31074, 0.30953, 0.30827, 0.30707, 0.30580, 0.30403, 0.30146,\
        0.29861, 0.29581, 0.29379, 0.29278, 0.29273, 0.29357, 0.29516, 0.29769, 0.30197, 0.30779, 0.31447,\
        0.32177, 0.32953, 0.33762, 0.34593, 0.35432, 0.36269, 0.37090, 0.37877, 0.38608, 0.39244, 0.39714,\
        0.39955, 0.40115, 0.40291, 0.40536, 0.40851, 0.41238, 0.41699, 0.42236, 0.42841, 0.43506, 0.44222,\
        0.44977, 0.45762, 0.46568, 0.47384, 0.48203, 0.49034, 0.49902, 0.50836, 0.51802, 0.52762, 0.53697,\
        0.54604, 0.55476, 0.56300, 0.57048, 0.57674, 0.58162, 0.58563, 0.58934, 0.59307, 0.59657, 0.59850,\
        0.60114, 0.60487, 0.60763, 0.60975, 0.61165, 0.61342, 0.61496, 0.61641, 0.61780, 0.61900, 0.62001,\
        0.62080, 0.62128, 0.62139, 0.62118, 0.62072, 0.62006, 0.61926, 0.61838, 0.61747, 0.61643, 0.61496,\
        0.61288, 0.61006, 0.60647, 0.60235, 0.59784, 0.59334, 0.58784, 0.58144, 0.57452, 0.56725, 0.55970,\
        0.55191, 0.54387, 0.53558, 0.52706, 0.51830, 0.50930, 0.50005, 0.49055, 0.48082, 0.47083, 0.46061],\
        [0.32819, 0.32890, 0.32900, 0.32868, 0.32813, 0.32749, 0.32682, 0.32612, 0.32534, 0.32448, 0.32351,\
        0.32247, 0.32138, 0.32022, 0.31897, 0.31757, 0.31601, 0.31433, 0.31253, 0.31049, 0.30799, 0.30497,\
        0.30165, 0.29835, 0.29650, 0.29543, 0.29534, 0.29618, 0.29758, 0.29966, 0.30381, 0.30976, 0.31654,\
        0.32388, 0.33162, 0.33963, 0.34783, 0.35610, 0.36433, 0.37242, 0.38021, 0.38750, 0.39397, 0.39897,\
        0.40058, 0.40142, 0.40322, 0.40583, 0.40922, 0.41339, 0.41838, 0.42417, 0.43068, 0.43782, 0.44547,\
        0.45353, 0.46190, 0.47048, 0.47916, 0.48778, 0.49643, 0.50548, 0.51511, 0.52505, 0.53498, 0.54472,\
        0.55415, 0.56320, 0.57184, 0.57985, 0.58644, 0.59093, 0.59444, 0.59787, 0.60140, 0.60495, 0.60723,\
        0.60971, 0.61287, 0.61567, 0.61795, 0.61989, 0.62162, 0.62315, 0.62454, 0.62580, 0.62690, 0.62779,\
        0.62846, 0.62885, 0.62894, 0.62875, 0.62831, 0.62764, 0.62679, 0.62578, 0.62460, 0.62315, 0.62128,\
        0.61885, 0.61577, 0.61208, 0.60790, 0.60356, 0.59912, 0.59311, 0.58639, 0.57932, 0.57198, 0.56442,\
        0.55663, 0.54861, 0.54035, 0.53185, 0.52312, 0.51414, 0.50492, 0.49545, 0.48574, 0.47578, 0.46558],\
        [0.33427, 0.33527, 0.33569, 0.33571, 0.33546, 0.33506, 0.33456, 0.33399, 0.33333, 0.33255, 0.33164,\
        0.33059, 0.32942, 0.32810, 0.32661, 0.32492, 0.32301, 0.32088, 0.31850, 0.31582, 0.31274, 0.30927,\
        0.30556, 0.30237, 0.30055, 0.29923, 0.29915, 0.30018, 0.30166, 0.30395, 0.30777, 0.31318, 0.31960,\
        0.32663, 0.33408, 0.34179, 0.34966, 0.35759, 0.36546, 0.37316, 0.38054, 0.38739, 0.39334, 0.39766,\
        0.39930, 0.40044, 0.40264, 0.40559, 0.40929, 0.41386, 0.41932, 0.42563, 0.43270, 0.44041, 0.44863,\
        0.45725, 0.46617, 0.47535, 0.48469, 0.49391, 0.50298, 0.51240, 0.52226, 0.53241, 0.54259, 0.55259,\
        0.56223, 0.57140, 0.58012, 0.58859, 0.59617, 0.60010, 0.60346, 0.60683, 0.61026, 0.61350, 0.61621,\
        0.61880, 0.62154, 0.62413, 0.62638, 0.62831, 0.63000, 0.63149, 0.63280, 0.63396, 0.63494, 0.63573,\
        0.63629, 0.63660, 0.63666, 0.63645, 0.63600, 0.63530, 0.63436, 0.63317, 0.63171, 0.62990, 0.62764,\
        0.62486, 0.62151, 0.61763, 0.61333, 0.60875, 0.60361, 0.59739, 0.59063, 0.58354, 0.57622, 0.56870,\
        0.56096, 0.55298, 0.54476, 0.53631, 0.52761, 0.51866, 0.50947, 0.50004, 0.49035, 0.48043, 0.47026],\
        [0.34054, 0.34182, 0.34257, 0.34290, 0.34295, 0.34281, 0.34255, 0.34218, 0.34168, 0.34103, 0.34020,\
        0.33918, 0.33795, 0.33651, 0.33482, 0.33286, 0.33061, 0.32806, 0.32519, 0.32202, 0.31855, 0.31487,\
        0.31126, 0.30823, 0.30610, 0.30476, 0.30457, 0.30544, 0.30698, 0.30936, 0.31294, 0.31777, 0.32358,\
        0.33006, 0.33697, 0.34417, 0.35150, 0.35888, 0.36617, 0.37325, 0.37996, 0.38607, 0.39122, 0.39496,\
        0.39719, 0.39904, 0.40158, 0.40480, 0.40882, 0.41384, 0.41987, 0.42682, 0.43455, 0.44292, 0.45177,\
        0.46097, 0.47041, 0.48007, 0.49006, 0.50018, 0.50961, 0.51939, 0.52956, 0.53995, 0.55038, 0.56060,\
        0.57040, 0.57952, 0.58785, 0.59577, 0.60363, 0.60883, 0.61262, 0.61613, 0.61949, 0.62262, 0.62545,\
        0.62808, 0.63062, 0.63299, 0.63511, 0.63695, 0.63856, 0.63997, 0.64121, 0.64226, 0.64313, 0.64379,\
        0.64425, 0.64448, 0.64448, 0.64424, 0.64375, 0.64298, 0.64193, 0.64055, 0.63881, 0.63665, 0.63401,\
        0.63086, 0.62718, 0.62300, 0.61836, 0.61327, 0.60756, 0.60119, 0.59436, 0.58728, 0.58007, 0.57263,\
        0.56496, 0.55703, 0.54886, 0.54045, 0.53179, 0.52288, 0.51372, 0.50432, 0.49467, 0.48478, 0.47463],\
        [0.34703, 0.34860, 0.34964, 0.35029, 0.35064, 0.35079, 0.35080, 0.35069, 0.35042, 0.34995, 0.34923,\
        0.34826, 0.34701, 0.34547, 0.34363, 0.34145, 0.33891, 0.33598, 0.33271, 0.32917, 0.32546, 0.32173,\
        0.31824, 0.31530, 0.31311, 0.31178, 0.31142, 0.31201, 0.31340, 0.31566, 0.31891, 0.32317, 0.32829,\
        0.33407, 0.34029, 0.34679, 0.35343, 0.36008, 0.36662, 0.37292, 0.37882, 0.38413, 0.38860, 0.39206,\
        0.39470, 0.39716, 0.40004, 0.40347, 0.40784, 0.41343, 0.42017, 0.42788, 0.43635, 0.44543, 0.45496,\
        0.46480, 0.47474, 0.48461, 0.49472, 0.50531, 0.51575, 0.52623, 0.53687, 0.54760, 0.55831, 0.56878,\
        0.57881, 0.58808, 0.59593, 0.60349, 0.61091, 0.61702, 0.62165, 0.62551, 0.62896, 0.63210, 0.63495,\
        0.63756, 0.63997, 0.64216, 0.64410, 0.64581, 0.64731, 0.64862, 0.64975, 0.65069, 0.65143, 0.65197,\
        0.65231, 0.65244, 0.65236, 0.65207, 0.65153, 0.65069, 0.64950, 0.64792, 0.64589, 0.64338, 0.64037,\
        0.63683, 0.63277, 0.62820, 0.62314, 0.61756, 0.61141, 0.60474, 0.59773, 0.59075, 0.58368, 0.57633,\
        0.56870, 0.56082, 0.55269, 0.54431, 0.53569, 0.52681, 0.51769, 0.50832, 0.49870, 0.48883, 0.47872],\
        [0.35376, 0.35562, 0.35695, 0.35790, 0.35856, 0.35903, 0.35935, 0.35954, 0.35957, 0.35932, 0.35876,\
        0.35786, 0.35661, 0.35501, 0.35307, 0.35074, 0.34795, 0.34469, 0.34107, 0.33723, 0.33335, 0.32962,\
        0.32624, 0.32340, 0.32124, 0.31986, 0.31933, 0.31964, 0.32076, 0.32270, 0.32551, 0.32917, 0.33358,\
        0.33858, 0.34401, 0.34970, 0.35551, 0.36131, 0.36700, 0.37244, 0.37750, 0.38204, 0.38594, 0.38918,\
        0.39196, 0.39469, 0.39780, 0.40153, 0.40640, 0.41280, 0.42041, 0.42897, 0.43824, 0.44806, 0.45831,\
        0.46885, 0.47948, 0.48973, 0.49907, 0.51040, 0.52185, 0.53311, 0.54427, 0.55536, 0.56635, 0.57710,\
        0.58734, 0.59705, 0.60532, 0.61265, 0.61973, 0.62589, 0.63090, 0.63505, 0.63862, 0.64180, 0.64467,\
        0.64725, 0.64955, 0.65157, 0.65333, 0.65486, 0.65622, 0.65741, 0.65843, 0.65925, 0.65986, 0.66025,\
        0.66044, 0.66044, 0.66026, 0.65991, 0.65931, 0.65840, 0.65707, 0.65527, 0.65295, 0.65009, 0.64669,\
        0.64274, 0.63827, 0.63329, 0.62780, 0.62181, 0.61531, 0.60837, 0.60118, 0.59434, 0.58727, 0.57987,\
        0.57224, 0.56437, 0.55627, 0.54792, 0.53933, 0.53048, 0.52138, 0.51204, 0.50245, 0.49261, 0.48253],\
        [0.36076, 0.36291, 0.36452, 0.36577, 0.36677, 0.36756, 0.36821, 0.36878, 0.36918, 0.36921, 0.36882,\
        0.36803, 0.36679, 0.36514, 0.36315, 0.36075, 0.35777, 0.35420, 0.35022, 0.34608, 0.34205, 0.33833,\
        0.33506, 0.33234, 0.33023, 0.32879, 0.32807, 0.32809, 0.32885, 0.33036, 0.33264, 0.33565, 0.33932,\
        0.34351, 0.34808, 0.35289, 0.35780, 0.36269, 0.36747, 0.37201, 0.37625, 0.38008, 0.38347, 0.38647,\
        0.38925, 0.39208, 0.39526, 0.39918, 0.40490, 0.41234, 0.42091, 0.43033, 0.44040, 0.45095, 0.46186,\
        0.47308, 0.48455, 0.49601, 0.50635, 0.51709, 0.52873, 0.54043, 0.55196, 0.56328, 0.57449, 0.58563,\
        0.59601, 0.60578, 0.61471, 0.62250, 0.62949, 0.63555, 0.64061, 0.64483, 0.64846, 0.65167, 0.65457,\
        0.65714, 0.65935, 0.66120, 0.66274, 0.66409, 0.66529, 0.66636, 0.66726, 0.66796, 0.66841, 0.66862,\
        0.66862, 0.66844, 0.66814, 0.66772, 0.66708, 0.66610, 0.66463, 0.66260, 0.65997, 0.65674, 0.65294,\
        0.64858, 0.64369, 0.63830, 0.63243, 0.62609, 0.61934, 0.61231, 0.60524, 0.59823, 0.59082, 0.58326,\
        0.57558, 0.56772, 0.55964, 0.55131, 0.54273, 0.53390, 0.52482, 0.51550, 0.50593, 0.49612, 0.48607],\
        [0.36806, 0.37050, 0.37238, 0.37394, 0.37533, 0.37648, 0.37744, 0.37841, 0.37934, 0.37967, 0.37940,\
        0.37880, 0.37758, 0.37583, 0.37385, 0.37153, 0.36841, 0.36446, 0.36004, 0.35558, 0.35140, 0.34772,\
        0.34458, 0.34199, 0.33993, 0.33841, 0.33748, 0.33716, 0.33750, 0.33850, 0.34019, 0.34252, 0.34542,\
        0.34879, 0.35249, 0.35639, 0.36036, 0.36431, 0.36815, 0.37180, 0.37523, 0.37840, 0.38133, 0.38411,\
        0.38688, 0.38987, 0.39337, 0.39791, 0.40435, 0.41258, 0.42198, 0.43221, 0.44303, 0.45425, 0.46572,\
        0.47739, 0.48935, 0.50149, 0.51296, 0.52442, 0.53630, 0.54831, 0.56011, 0.57154, 0.58261, 0.59405,\
        0.60522, 0.61520, 0.62452, 0.63275, 0.63985, 0.64583, 0.65076, 0.65488, 0.65845, 0.66167, 0.66461,\
        0.66722, 0.66937, 0.67103, 0.67233, 0.67345, 0.67449, 0.67545, 0.67625, 0.67681, 0.67709, 0.67709,\
        0.67683, 0.67641, 0.67594, 0.67545, 0.67481, 0.67378, 0.67218, 0.66990, 0.66693, 0.66331, 0.65909,\
        0.65431, 0.64901, 0.64324, 0.63701, 0.63037, 0.62340, 0.61622, 0.60896, 0.60161, 0.59403, 0.58641,\
        0.57874, 0.57089, 0.56281, 0.55448, 0.54590, 0.53708, 0.52801, 0.51870, 0.50915, 0.49937, 0.48934],\
        [0.37570, 0.37846, 0.38052, 0.38241, 0.38435, 0.38593, 0.38713, 0.38841, 0.39018, 0.39077, 0.39040,\
        0.39031, 0.38903, 0.38700, 0.38511, 0.38315, 0.37986, 0.37537, 0.37042, 0.36556, 0.36124, 0.35764,\
        0.35469, 0.35227, 0.35024, 0.34861, 0.34741, 0.34670, 0.34656, 0.34700, 0.34805, 0.34968, 0.35182,\
        0.35437, 0.35721, 0.36019, 0.36323, 0.36622, 0.36911, 0.37188, 0.37453, 0.37707, 0.37959, 0.38219,\
        0.38503, 0.38834, 0.39239, 0.39766, 0.40479, 0.41369, 0.42380, 0.43476, 0.44629, 0.45817, 0.47017,\
        0.48211, 0.49406, 0.50667, 0.51931, 0.53176, 0.54425, 0.55671, 0.56887, 0.58050, 0.59131, 0.60187,\
        0.61397, 0.62498, 0.63488, 0.64353, 0.65077, 0.65661, 0.66128, 0.66514, 0.66854, 0.67171, 0.67473,\
        0.67748, 0.67964, 0.68108, 0.68206, 0.68295, 0.68385, 0.68471, 0.68543, 0.68585, 0.68589, 0.68566,\
        0.68510, 0.68431, 0.68362, 0.68308, 0.68248, 0.68145, 0.67968, 0.67711, 0.67376, 0.66972, 0.66507,\
        0.65988, 0.65420, 0.64806, 0.64150, 0.63457, 0.62735, 0.61992, 0.61234, 0.60461, 0.59686, 0.58937,\
        0.58178, 0.57393, 0.56581, 0.55745, 0.54885, 0.54002, 0.53095, 0.52165, 0.51212, 0.50235, 0.49234],\
        [0.38371, 0.38688, 0.38899, 0.39108, 0.39400, 0.39614, 0.39764, 0.39890, 0.40149, 0.40245, 0.40199,\
        0.40252, 0.40106, 0.39880, 0.39692, 0.39570, 0.39189, 0.38673, 0.38120, 0.37586, 0.37144, 0.36803,\
        0.36537, 0.36313, 0.36111, 0.35927, 0.35773, 0.35658, 0.35590, 0.35575, 0.35614, 0.35706, 0.35846,\
        0.36023, 0.36222, 0.36431, 0.36641, 0.36845, 0.37040, 0.37229, 0.37416, 0.37610, 0.37822, 0.38067,\
        0.38365, 0.38741, 0.39217, 0.39817, 0.40603, 0.41558, 0.42636, 0.43801, 0.45024, 0.46284, 0.47557,\
        0.48803, 0.50029, 0.51314, 0.52632, 0.53942, 0.55250, 0.56553, 0.57819, 0.59015, 0.60168, 0.61237,\
        0.62398, 0.63539, 0.64581, 0.65485, 0.66222, 0.66784, 0.67206, 0.67551, 0.67867, 0.68174, 0.68486,\
        0.68788, 0.69019, 0.69134, 0.69193, 0.69263, 0.69342, 0.69418, 0.69484, 0.69507, 0.69480, 0.69436,\
        0.69345, 0.69211, 0.69116, 0.69059, 0.69007, 0.68907, 0.68707, 0.68414, 0.68037, 0.67589, 0.67081,\
        0.66523, 0.65919, 0.65272, 0.64586, 0.63865, 0.63117, 0.62348, 0.61564, 0.60771, 0.59992, 0.59247,\
        0.58480, 0.57683, 0.56863, 0.56021, 0.55157, 0.54272, 0.53365, 0.52435, 0.51483, 0.50508, 0.49509],\
        [0.39198, 0.39581, 0.39814, 0.39995, 0.40411, 0.40714, 0.40936, 0.41131, 0.41355, 0.41490, 0.41520,\
        0.41515, 0.41396, 0.41192, 0.41002, 0.40811, 0.40394, 0.39842, 0.39239, 0.38639, 0.38204, 0.37896,\
        0.37662, 0.37459, 0.37246, 0.37029, 0.36832, 0.36666, 0.36541, 0.36462, 0.36434, 0.36458, 0.36527,\
        0.36630, 0.36751, 0.36874, 0.36992, 0.37100, 0.37201, 0.37301, 0.37411, 0.37543, 0.37714, 0.37943,\
        0.38254, 0.38674, 0.39233, 0.39954, 0.40789, 0.41803, 0.42953, 0.44190, 0.45482, 0.46815, 0.48185,\
        0.49544, 0.50815, 0.52111, 0.53430, 0.54757, 0.56096, 0.57452, 0.58789, 0.60010, 0.61178, 0.62326,\
        0.63485, 0.64635, 0.65714, 0.66662, 0.67415, 0.67945, 0.68300, 0.68589, 0.68888, 0.69178, 0.69487,\
        0.69825, 0.70077, 0.70174, 0.70211, 0.70272, 0.70338, 0.70397, 0.70442, 0.70441, 0.70385, 0.70312,\
        0.70181, 0.69995, 0.69876, 0.69807, 0.69751, 0.69656, 0.69413, 0.69082, 0.68662, 0.68168, 0.67621,\
        0.67030, 0.66395, 0.65718, 0.65004, 0.64256, 0.63483, 0.62693, 0.61895, 0.61102, 0.60332, 0.59571,\
        0.58769, 0.57953, 0.57121, 0.56272, 0.55405, 0.54517, 0.53609, 0.52680, 0.51729, 0.50755, 0.49758],\
        [0.39953, 0.40469, 0.40802, 0.41115, 0.41519, 0.41904, 0.42215, 0.42478, 0.42703, 0.42854, 0.42915,\
        0.42895, 0.42787, 0.42606, 0.42384, 0.42089, 0.41645, 0.41070, 0.40398, 0.39744, 0.39352, 0.39060,\
        0.38847, 0.38664, 0.38413, 0.38145, 0.37899, 0.37682, 0.37495, 0.37350, 0.37255, 0.37213, 0.37219,\
        0.37256, 0.37305, 0.37348, 0.37375, 0.37386, 0.37390, 0.37399, 0.37428, 0.37496, 0.37621, 0.37829,\
        0.38143, 0.38590, 0.39196, 0.39974, 0.40934, 0.42060, 0.43313, 0.44640, 0.46002, 0.47389, 0.48819,\
        0.50285, 0.51660, 0.52989, 0.54304, 0.55615, 0.56948, 0.58317, 0.59715, 0.60998, 0.62191, 0.63377,\
        0.64560, 0.65727, 0.66839, 0.67840, 0.68631, 0.69124, 0.69403, 0.69621, 0.69930, 0.70220, 0.70513,\
        0.70842, 0.71090, 0.71218, 0.71279, 0.71331, 0.71378, 0.71412, 0.71421, 0.71387, 0.71307, 0.71190,\
        0.71025, 0.70837, 0.70683, 0.70571, 0.70473, 0.70329, 0.70050, 0.69701, 0.69232, 0.68694, 0.68119,\
        0.67503, 0.66843, 0.66141, 0.65400, 0.64627, 0.63830, 0.63018, 0.62204, 0.61398, 0.60611, 0.59817,\
        0.59009, 0.58189, 0.57351, 0.56497, 0.55626, 0.54737, 0.53829, 0.52900, 0.51950, 0.50978, 0.49983],\
        [0.40713, 0.41328, 0.41831, 0.42288, 0.42749, 0.43188, 0.43568, 0.43886, 0.44138, 0.44305, 0.44376,\
        0.44354, 0.44250, 0.44072, 0.43821, 0.43472, 0.42992, 0.42389, 0.41709, 0.41087, 0.40648, 0.40313,\
        0.40083, 0.39905, 0.39571, 0.39251, 0.38960, 0.38691, 0.38441, 0.38227, 0.38066, 0.37964, 0.37915,\
        0.37898, 0.37885, 0.37851, 0.37787, 0.37699, 0.37603, 0.37517, 0.37461, 0.37458, 0.37532, 0.37709,\
        0.38016, 0.38479, 0.39118, 0.39961, 0.41044, 0.42313, 0.43706, 0.45156, 0.46600, 0.48028, 0.49494,\
        0.51029, 0.52536, 0.53941, 0.55257, 0.56524, 0.57817, 0.59163, 0.60529, 0.61887, 0.63158, 0.64386,\
        0.65590, 0.66765, 0.67890, 0.68929, 0.69798, 0.70251, 0.70510, 0.70713, 0.71003, 0.71302, 0.71596,\
        0.71886, 0.72120, 0.72271, 0.72358, 0.72411, 0.72441, 0.72446, 0.72418, 0.72349, 0.72236, 0.72083,\
        0.71895, 0.71691, 0.71501, 0.71329, 0.71154, 0.70932, 0.70624, 0.70230, 0.69714, 0.69157, 0.68573,\
        0.67942, 0.67263, 0.66538, 0.65772, 0.64974, 0.64151, 0.63316, 0.62478, 0.61646, 0.60822, 0.60007,\
        0.59216, 0.58391, 0.57550, 0.56693, 0.55820, 0.54931, 0.54023, 0.53095, 0.52146, 0.51175, 0.50182],\
        [0.41563, 0.42252, 0.42884, 0.43468, 0.44017, 0.44519, 0.44957, 0.45324, 0.45613, 0.45803, 0.45878,\
        0.45853, 0.45751, 0.45577, 0.45316, 0.44946, 0.44448, 0.43837, 0.43171, 0.42552, 0.42048, 0.41652,\
        0.41347, 0.41058, 0.40696, 0.40332, 0.39998, 0.39679, 0.39365, 0.39084, 0.38861, 0.38705, 0.38609,\
        0.38552, 0.38490, 0.38382, 0.38223, 0.38032, 0.37831, 0.37645, 0.37499, 0.37419, 0.37433, 0.37572,\
        0.37863, 0.38337, 0.39021, 0.39949, 0.41136, 0.42552, 0.44118, 0.45740, 0.47317, 0.48798, 0.50322,\
        0.51952, 0.53546, 0.55020, 0.56331, 0.57517, 0.58732, 0.60087, 0.61458, 0.62815, 0.64119, 0.65369,\
        0.66574, 0.67735, 0.68837, 0.69842, 0.70690, 0.71231, 0.71559, 0.71822, 0.72103, 0.72397, 0.72687,\
        0.72956, 0.73178, 0.73338, 0.73438, 0.73492, 0.73507, 0.73486, 0.73424, 0.73319, 0.73170, 0.72983,\
        0.72765, 0.72530, 0.72293, 0.72055, 0.71801, 0.71501, 0.71131, 0.70672, 0.70129, 0.69586, 0.68998,\
        0.68352, 0.67656, 0.66909, 0.66118, 0.65292, 0.64444, 0.63583, 0.62719, 0.61860, 0.61015, 0.60214,\
        0.59388, 0.58557, 0.57717, 0.56860, 0.55987, 0.55098, 0.54191, 0.53264, 0.52317, 0.51348, 0.50356],\
        [0.42426, 0.43199, 0.43936, 0.44630, 0.45274, 0.45848, 0.46336, 0.46744, 0.47081, 0.47309, 0.47385,\
        0.47350, 0.47257, 0.47103, 0.46855, 0.46489, 0.45989, 0.45367, 0.44691, 0.44045, 0.43484, 0.43014,\
        0.42607, 0.42214, 0.41801, 0.41388, 0.40996, 0.40620, 0.40255, 0.39916, 0.39642, 0.39437, 0.39298,\
        0.39217, 0.39118, 0.38934, 0.38673, 0.38374, 0.38067, 0.37777, 0.37534, 0.37369, 0.37313, 0.37402,\
        0.37668, 0.38145, 0.38860, 0.39843, 0.41147, 0.42728, 0.44493, 0.46349, 0.48166, 0.49777, 0.51363,\
        0.53033, 0.54689, 0.56247, 0.57590, 0.58679, 0.59738, 0.61103, 0.62496, 0.63838, 0.65124, 0.66359,\
        0.67541, 0.68659, 0.69725, 0.70687, 0.71498, 0.72101, 0.72527, 0.72864, 0.73174, 0.73478, 0.73767,\
        0.74029, 0.74247, 0.74409, 0.74513, 0.74565, 0.74567, 0.74521, 0.74427, 0.74285, 0.74099, 0.73874,\
        0.73618, 0.73342, 0.73052, 0.72747, 0.72417, 0.72044, 0.71613, 0.71116, 0.70572, 0.70020, 0.69387,\
        0.68732, 0.68023, 0.67252, 0.66433, 0.65579, 0.64705, 0.63819, 0.62931, 0.62047, 0.61170, 0.60295,\
        0.59499, 0.58696, 0.57857, 0.57000, 0.56128, 0.55240, 0.54335, 0.53410, 0.52464, 0.51497, 0.50507],\
        [0.43258, 0.44117, 0.44949, 0.45741, 0.46480, 0.47133, 0.47660, 0.48086, 0.48484, 0.48789, 0.48853,\
        0.48805, 0.48739, 0.48630, 0.48409, 0.48067, 0.47575, 0.46927, 0.46208, 0.45520, 0.44910, 0.44370,\
        0.43872, 0.43383, 0.42894, 0.42414, 0.41957, 0.41524, 0.41115, 0.40742, 0.40427, 0.40175, 0.39987,\
        0.39889, 0.39758, 0.39485, 0.39116, 0.38715, 0.38302, 0.37906, 0.37560, 0.37300, 0.37161, 0.37185,\
        0.37412, 0.37881, 0.38626, 0.39671, 0.41068, 0.42772, 0.44730, 0.46835, 0.48949, 0.50820, 0.52458,\
        0.54134, 0.55843, 0.57536, 0.59039, 0.60113, 0.61165, 0.62408, 0.63716, 0.64980, 0.66197, 0.67381,\
        0.68528, 0.69587, 0.70586, 0.71520, 0.72312, 0.72945, 0.73439, 0.73841, 0.74194, 0.74521, 0.74822,\
        0.75088, 0.75307, 0.75470, 0.75575, 0.75622, 0.75610, 0.75541, 0.75414, 0.75235, 0.75009, 0.74743,\
        0.74444, 0.74120, 0.73773, 0.73404, 0.73005, 0.72567, 0.72081, 0.71548, 0.70975, 0.70362, 0.69735,\
        0.69093, 0.68366, 0.67563, 0.66711, 0.65829, 0.64931, 0.64024, 0.63117, 0.62218, 0.61336, 0.60494,\
        0.59675, 0.58826, 0.57974, 0.57116, 0.56244, 0.55358, 0.54454, 0.53532, 0.52588, 0.51622, 0.50633],\
        [0.44024, 0.44964, 0.45881, 0.46758, 0.47585, 0.48335, 0.48902, 0.49288, 0.49733, 0.50156, 0.50227,\
        0.50210, 0.50189, 0.50138, 0.49938, 0.49635, 0.49161, 0.48459, 0.47675, 0.46950, 0.46306, 0.45716,\
        0.45137, 0.44552, 0.43972, 0.43416, 0.42894, 0.42410, 0.41966, 0.41566, 0.41222, 0.40936, 0.40713,\
        0.40543, 0.40335, 0.39975, 0.39539, 0.39053, 0.38536, 0.38031, 0.37576, 0.37209, 0.36973, 0.36912,\
        0.37074, 0.37508, 0.38263, 0.39380, 0.40781, 0.42575, 0.44713, 0.47054, 0.49363, 0.51471, 0.53322,\
        0.55077, 0.56838, 0.58636, 0.60357, 0.61632, 0.62769, 0.63942, 0.65114, 0.66240, 0.67337, 0.68429,\
        0.69529, 0.70568, 0.71502, 0.72376, 0.73143, 0.73788, 0.74320, 0.74768, 0.75162, 0.75519, 0.75841,\
        0.76119, 0.76343, 0.76507, 0.76610, 0.76652, 0.76627, 0.76534, 0.76374, 0.76156, 0.75888, 0.75578,\
        0.75234, 0.74859, 0.74456, 0.74025, 0.73563, 0.73064, 0.72528, 0.71955, 0.71351, 0.70727, 0.70103,\
        0.69453, 0.68672, 0.67824, 0.66941, 0.66036, 0.65119, 0.64196, 0.63275, 0.62363, 0.61472, 0.60611,\
        0.59766, 0.58913, 0.58063, 0.57206, 0.56338, 0.55453, 0.54552, 0.53631, 0.52688, 0.51724, 0.50736],\
        [0.44699, 0.45710, 0.46700, 0.47642, 0.48518, 0.49353, 0.50028, 0.50393, 0.50856, 0.51283, 0.51503,\
        0.51589, 0.51622, 0.51585, 0.51422, 0.51127, 0.50643, 0.49892, 0.49067, 0.48334, 0.47680, 0.47061,\
        0.46402, 0.45708, 0.45028, 0.44392, 0.43811, 0.43284, 0.42808, 0.42384, 0.42016, 0.41700, 0.41430,\
        0.41174, 0.40867, 0.40459, 0.39974, 0.39392, 0.38770, 0.38157, 0.37587, 0.37101, 0.36749, 0.36579,\
        0.36644, 0.36996, 0.37681, 0.38723, 0.40164, 0.42060, 0.44344, 0.46965, 0.49647, 0.51928, 0.53942,\
        0.55824, 0.57649, 0.59473, 0.61352, 0.62984, 0.64347, 0.65549, 0.66622, 0.67588, 0.68529, 0.69522,\
        0.70522, 0.71521, 0.72430, 0.73258, 0.73994, 0.74632, 0.75179, 0.75655, 0.76080, 0.76467, 0.76816,\
        0.77112, 0.77343, 0.77505, 0.77606, 0.77643, 0.77608, 0.77490, 0.77294, 0.77034, 0.76723, 0.76370,\
        0.75980, 0.75556, 0.75099, 0.74610, 0.74088, 0.73532, 0.72944, 0.72328, 0.71691, 0.71047, 0.70409,\
        0.69706, 0.68883, 0.68012, 0.67112, 0.66195, 0.65267, 0.64334, 0.63403, 0.62479, 0.61567, 0.60668,\
        0.59797, 0.58966, 0.58129, 0.57277, 0.56410, 0.55528, 0.54628, 0.53708, 0.52767, 0.51803, 0.50815],\
        [0.45257, 0.46330, 0.47387, 0.48390, 0.49278, 0.50070, 0.50834, 0.51395, 0.51902, 0.52356, 0.52683,\
        0.52883, 0.52983, 0.52977, 0.52834, 0.52526, 0.52012, 0.51281, 0.50473, 0.49717, 0.49052, 0.48427,\
        0.47657, 0.46828, 0.46045, 0.45340, 0.44710, 0.44146, 0.43643, 0.43197, 0.42804, 0.42458, 0.42140,\
        0.41815, 0.41436, 0.40968, 0.40398, 0.39729, 0.39020, 0.38301, 0.37606, 0.36989, 0.36502, 0.36200,\
        0.36138, 0.36367, 0.36936, 0.37900, 0.39327, 0.41175, 0.43533, 0.46412, 0.49398, 0.52054, 0.54353,\
        0.56439, 0.58454, 0.60441, 0.62495, 0.64364, 0.65918, 0.67184, 0.68195, 0.69007, 0.69764, 0.70672,\
        0.71603, 0.72515, 0.73374, 0.74158, 0.74860, 0.75478, 0.76021, 0.76505, 0.76947, 0.77360, 0.77740,\
        0.78061, 0.78296, 0.78450, 0.78544, 0.78582, 0.78540, 0.78396, 0.78160, 0.77857, 0.77504, 0.77111,\
        0.76679, 0.76208, 0.75699, 0.75155, 0.74576, 0.73965, 0.73324, 0.72658, 0.71974, 0.71276, 0.70560,\
        0.69808, 0.68992, 0.68124, 0.67225, 0.66306, 0.65376, 0.64440, 0.63504, 0.62573, 0.61653, 0.60748,\
        0.59875, 0.59036, 0.58189, 0.57333, 0.56465, 0.55583, 0.54684, 0.53764, 0.52823, 0.51859, 0.50871],\
        [0.45685, 0.46799, 0.47911, 0.48996, 0.49957, 0.50754, 0.51527, 0.52215, 0.52823, 0.53353, 0.53776,\
        0.54079, 0.54259, 0.54303, 0.54188, 0.53890, 0.53392, 0.52716, 0.51944, 0.51163, 0.50466, 0.49805,\
        0.48840, 0.47875, 0.47012, 0.46260, 0.45597, 0.45005, 0.44476, 0.44006, 0.43590, 0.43218, 0.42864,\
        0.42492, 0.42058, 0.41526, 0.40880, 0.40130, 0.39327, 0.38487, 0.37653, 0.36887, 0.36251, 0.35803,\
        0.35595, 0.35678, 0.36098, 0.36904, 0.38142, 0.39853, 0.42324, 0.45534, 0.48948, 0.52042, 0.54662,\
        0.56984, 0.59242, 0.61567, 0.63822, 0.65848, 0.67502, 0.68798, 0.69771, 0.70472, 0.71131, 0.71895,\
        0.72722, 0.73548, 0.74339, 0.75072, 0.75734, 0.76321, 0.76845, 0.77320, 0.77763, 0.78189, 0.78602,\
        0.78960, 0.79195, 0.79325, 0.79409, 0.79450, 0.79409, 0.79232, 0.78950, 0.78608, 0.78219, 0.77794,\
        0.77326, 0.76813, 0.76256, 0.75659, 0.75026, 0.74361, 0.73666, 0.72946, 0.72205, 0.71446, 0.70667,\
        0.69864, 0.69044, 0.68183, 0.67292, 0.66379, 0.65452, 0.64517, 0.63581, 0.62648, 0.61730, 0.60840,\
        0.59983, 0.59103, 0.58236, 0.57373, 0.56503, 0.55620, 0.54719, 0.53799, 0.52856, 0.51891, 0.50902],\
        [0.45976, 0.47106, 0.48233, 0.49343, 0.50376, 0.51269, 0.52110, 0.52898, 0.53613, 0.54244, 0.54771,\
        0.55177, 0.55445, 0.55556, 0.55490, 0.55232, 0.54780, 0.54157, 0.53419, 0.52628, 0.51829, 0.50946,\
        0.49885, 0.48840, 0.47946, 0.47178, 0.46490, 0.45872, 0.45315, 0.44818, 0.44380, 0.43990, 0.43618,\
        0.43222, 0.42754, 0.42174, 0.41466, 0.40639, 0.39723, 0.38734, 0.37739, 0.36812, 0.36019, 0.35420,\
        0.35066, 0.35004, 0.35275, 0.35917, 0.36973, 0.38555, 0.41056, 0.44606, 0.48521, 0.52027, 0.54983,\
        0.57607, 0.60059, 0.62711, 0.65212, 0.67371, 0.69036, 0.70251, 0.71166, 0.71836, 0.72451, 0.73117,\
        0.73839, 0.74581, 0.75309, 0.75992, 0.76611, 0.77160, 0.77651, 0.78105, 0.78532, 0.78948, 0.79379,\
        0.79791, 0.80024, 0.80129, 0.80200, 0.80230, 0.80178, 0.79961, 0.79646, 0.79275, 0.78857, 0.78418,\
        0.77924, 0.77371, 0.76767, 0.76121, 0.75437, 0.74721, 0.73974, 0.73200, 0.72402, 0.71585, 0.70756,\
        0.69927, 0.69080, 0.68215, 0.67330, 0.66425, 0.65504, 0.64572, 0.63636, 0.62702, 0.61778, 0.60871,\
        0.59982, 0.59117, 0.58257, 0.57395, 0.56524, 0.55638, 0.54735, 0.53812, 0.52867, 0.51900, 0.50908],\
        [0.46144, 0.47271, 0.48385, 0.49470, 0.50545, 0.51581, 0.52547, 0.53447, 0.54275, 0.55016, 0.55654,\
        0.56167, 0.56533, 0.56728, 0.56730, 0.56531, 0.56138, 0.55568, 0.54853, 0.54030, 0.53115, 0.52084,\
        0.50938, 0.49809, 0.48929, 0.48139, 0.47415, 0.46765, 0.46170, 0.45637, 0.45178, 0.44781, 0.44413,\
        0.44019, 0.43540, 0.42927, 0.42155, 0.41233, 0.40183, 0.39032, 0.37867, 0.36774, 0.35826, 0.35084,\
        0.34600, 0.34420, 0.34587, 0.35142, 0.36147, 0.37739, 0.40247, 0.43781, 0.47823, 0.51751, 0.55367,\
        0.58548, 0.61335, 0.64025, 0.66561, 0.68782, 0.70389, 0.71475, 0.72310, 0.72987, 0.73603, 0.74231,\
        0.74892, 0.75576, 0.76256, 0.76904, 0.77489, 0.77994, 0.78442, 0.78872, 0.79280, 0.79666, 0.80061,\
        0.80485, 0.80744, 0.80870, 0.80927, 0.80919, 0.80815, 0.80579, 0.80253, 0.79832, 0.79421, 0.78995,\
        0.78475, 0.77881, 0.77232, 0.76540, 0.75811, 0.75048, 0.74253, 0.73430, 0.72580, 0.71709, 0.70827,\
        0.69956, 0.69098, 0.68237, 0.67356, 0.66456, 0.65538, 0.64609, 0.63674, 0.62739, 0.61807, 0.60882,\
        0.59978, 0.59117, 0.58264, 0.57402, 0.56526, 0.55636, 0.54728, 0.53801, 0.52853, 0.51882, 0.50888],\
        [0.46213, 0.47338, 0.48451, 0.49553, 0.50672, 0.51791, 0.52863, 0.53872, 0.54810, 0.55662, 0.56411,\
        0.57036, 0.57509, 0.57798, 0.57881, 0.57750, 0.57425, 0.56915, 0.56225, 0.55379, 0.54402, 0.53307,\
        0.52133, 0.51005, 0.50059, 0.49159, 0.48383, 0.47704, 0.47047, 0.46462, 0.45983, 0.45597, 0.45258,\
        0.44896, 0.44431, 0.43796, 0.42952, 0.41906, 0.40695, 0.39375, 0.38036, 0.36779, 0.35686, 0.34819,\
        0.34231, 0.33972, 0.34093, 0.34652, 0.35724, 0.37408, 0.39830, 0.43322, 0.47323, 0.51603, 0.55946,\
        0.59712, 0.62690, 0.65256, 0.67604, 0.69700, 0.71236, 0.72316, 0.73176, 0.73904, 0.74562, 0.75199,\
        0.75842, 0.76497, 0.77154, 0.77791, 0.78366, 0.78827, 0.79217, 0.79631, 0.80041, 0.80412, 0.80788,\
        0.81146, 0.81405, 0.81544, 0.81588, 0.81543, 0.81396, 0.81138, 0.80776, 0.80336, 0.79988, 0.79538,\
        0.78976, 0.78338, 0.77646, 0.76915, 0.76147, 0.75345, 0.74510, 0.73645, 0.72753, 0.71838, 0.70908,\
        0.69987, 0.69132, 0.68267, 0.67382, 0.66479, 0.65560, 0.64631, 0.63698, 0.62764, 0.61833, 0.60909,\
        0.60001, 0.59134, 0.58269, 0.57396, 0.56511, 0.55612, 0.54697, 0.53764, 0.52812, 0.51837, 0.50839],\
        [0.46205, 0.47335, 0.48464, 0.49602, 0.50762, 0.51934, 0.53083, 0.54184, 0.55218, 0.56172, 0.57026,\
        0.57760, 0.58343, 0.58735, 0.58904, 0.58839, 0.58594, 0.58167, 0.57515, 0.56671, 0.55686, 0.54591,\
        0.53431, 0.52290, 0.51221, 0.50194, 0.49436, 0.48707, 0.47935, 0.47284, 0.46792, 0.46438, 0.46158,\
        0.45863, 0.45443, 0.44798, 0.43874, 0.42679, 0.41278, 0.39764, 0.38243, 0.36830, 0.35608, 0.34637,\
        0.33971, 0.33665, 0.33782, 0.34398, 0.35602, 0.37502, 0.40161, 0.43449, 0.47439, 0.51991, 0.56705,\
        0.60777, 0.63770, 0.66158, 0.68211, 0.70005, 0.71554, 0.72780, 0.73772, 0.74611, 0.75350, 0.76030,\
        0.76683, 0.77331, 0.77981, 0.78622, 0.79232, 0.79666, 0.80006, 0.80394, 0.80811, 0.81184, 0.81531,\
        0.81837, 0.82062, 0.82185, 0.82207, 0.82130, 0.81952, 0.81673, 0.81308, 0.80899, 0.80491, 0.80006,\
        0.79411, 0.78730, 0.78006, 0.77246, 0.76449, 0.75615, 0.74747, 0.73849, 0.72927, 0.71987, 0.71045,\
        0.70123, 0.69216, 0.68317, 0.67413, 0.66497, 0.65571, 0.64638, 0.63705, 0.62774, 0.61850, 0.60940,\
        0.60055, 0.59155, 0.58265, 0.57373, 0.56473, 0.55562, 0.54637, 0.53698, 0.52740, 0.51761, 0.50759],\
        [0.46132, 0.47273, 0.48421, 0.49584, 0.50783, 0.52003, 0.53212, 0.54386, 0.55501, 0.56538, 0.57481,\
        0.58311, 0.58999, 0.59486, 0.59749, 0.59728, 0.59586, 0.59286, 0.58675, 0.57861, 0.56923, 0.55873,\
        0.54740, 0.53583, 0.52454, 0.51417, 0.50579, 0.49678, 0.48794, 0.48099, 0.47616, 0.47314, 0.47122,\
        0.46929, 0.46588, 0.45953, 0.44938, 0.43562, 0.41934, 0.40197, 0.38486, 0.36928, 0.35599, 0.34545,\
        0.33819, 0.33483, 0.33609, 0.34279, 0.35587, 0.37616, 0.40361, 0.43725, 0.47779, 0.52507, 0.57397,\
        0.61513, 0.64493, 0.66732, 0.68552, 0.70173, 0.71701, 0.73051, 0.74200, 0.75178, 0.76021, 0.76763,\
        0.77436, 0.78083, 0.78737, 0.79362, 0.79977, 0.80480, 0.80840, 0.81220, 0.81608, 0.81966, 0.82282,\
        0.82541, 0.82724, 0.82814, 0.82806, 0.82699, 0.82493, 0.82196, 0.81822, 0.81396, 0.80927, 0.80392,\
        0.79756, 0.79051, 0.78315, 0.77539, 0.76721, 0.75861, 0.74965, 0.74041, 0.73095, 0.72138, 0.71181,\
        0.70232, 0.69299, 0.68371, 0.67441, 0.66504, 0.65565, 0.64625, 0.63688, 0.62759, 0.61840, 0.60930,\
        0.60025, 0.59132, 0.58231, 0.57321, 0.56403, 0.55477, 0.54542, 0.53595, 0.52633, 0.51651, 0.50645],\
        [0.46000, 0.47152, 0.48319, 0.49515, 0.50740, 0.51991, 0.53250, 0.54481, 0.55658, 0.56758, 0.57763,\
        0.58655, 0.59417, 0.60002, 0.60299, 0.60394, 0.60362, 0.60167, 0.59612, 0.58886, 0.58070, 0.57103,\
        0.56012, 0.54860, 0.53713, 0.52630, 0.51619, 0.50607, 0.49675, 0.48953, 0.48499, 0.48255, 0.48157,\
        0.48094, 0.47869, 0.47265, 0.46143, 0.44549, 0.42653, 0.40652, 0.38751, 0.37087, 0.35674, 0.34550,\
        0.33773, 0.33408, 0.33531, 0.34226, 0.35576, 0.37644, 0.40452, 0.43981, 0.48192, 0.53050, 0.57995,\
        0.62059, 0.64981, 0.67113, 0.68779, 0.70303, 0.71847, 0.73298, 0.74585, 0.75698, 0.76651, 0.77461,\
        0.78153, 0.78778, 0.79435, 0.80106, 0.80674, 0.81222, 0.81665, 0.82060, 0.82429, 0.82759, 0.83038,\
        0.83254, 0.83392, 0.83441, 0.83396, 0.83255, 0.83021, 0.82700, 0.82302, 0.81840, 0.81313, 0.80710,\
        0.80028, 0.79329, 0.78594, 0.77808, 0.76970, 0.76085, 0.75163, 0.74212, 0.73245, 0.72270, 0.71299,\
        0.70331, 0.69373, 0.68414, 0.67452, 0.66492, 0.65534, 0.64582, 0.63639, 0.62708, 0.61794, 0.60892,\
        0.59988, 0.59083, 0.58161, 0.57228, 0.56291, 0.55350, 0.54405, 0.53452, 0.52486, 0.51501, 0.50493],\
        [0.45811, 0.46974, 0.48151, 0.49354, 0.50598, 0.51893, 0.53199, 0.54474, 0.55692, 0.56833, 0.57878,\
        0.58798, 0.59558, 0.60152, 0.60595, 0.60828, 0.60881, 0.60720, 0.60284, 0.59750, 0.59113, 0.58239,\
        0.57200, 0.56071, 0.54924, 0.53803, 0.52721, 0.51674, 0.50715, 0.49970, 0.49544, 0.49317, 0.49284,\
        0.49348, 0.49248, 0.48699, 0.47440, 0.45597, 0.43433, 0.41180, 0.39116, 0.37355, 0.35856, 0.34661,\
        0.33830, 0.33428, 0.33532, 0.34224, 0.35585, 0.37678, 0.40544, 0.44273, 0.48701, 0.53633, 0.58501,\
        0.62483, 0.65391, 0.67481, 0.69028, 0.70505, 0.72104, 0.73640, 0.75029, 0.76252, 0.77307, 0.78193,\
        0.78912, 0.79509, 0.80086, 0.80818, 0.81455, 0.82015, 0.82494, 0.82904, 0.83259, 0.83560, 0.83799,\
        0.83969, 0.84060, 0.84065, 0.83980, 0.83802, 0.83535, 0.83183, 0.82754, 0.82254, 0.81683, 0.81041,\
        0.80341, 0.79620, 0.78870, 0.78065, 0.77201, 0.76287, 0.75334, 0.74355, 0.73363, 0.72368, 0.71382,\
        0.70411, 0.69425, 0.68424, 0.67432, 0.66446, 0.65470, 0.64502, 0.63548, 0.62608, 0.61692, 0.60809,\
        0.59935, 0.58992, 0.58035, 0.57080, 0.56126, 0.55172, 0.54217, 0.53259, 0.52292, 0.51308, 0.50299],\
        [0.45570, 0.46746, 0.47932, 0.49133, 0.50392, 0.51736, 0.53075, 0.54371, 0.55608, 0.56772, 0.57844,\
        0.58800, 0.59607, 0.60244, 0.60738, 0.61061, 0.61192, 0.61118, 0.60856, 0.60498, 0.59985, 0.59216,\
        0.58247, 0.57166, 0.56053, 0.54953, 0.53889, 0.52883, 0.51986, 0.51290, 0.50846, 0.50609, 0.50579,\
        0.50663, 0.50595, 0.50068, 0.48665, 0.46587, 0.44234, 0.41854, 0.39660, 0.37753, 0.36148, 0.34877,\
        0.33989, 0.33547, 0.33620, 0.34289, 0.35645, 0.37798, 0.40811, 0.44603, 0.49149, 0.54186, 0.59012,\
        0.62924, 0.65843, 0.68015, 0.69588, 0.71008, 0.72601, 0.74153, 0.75591, 0.76885, 0.78023, 0.78996,\
        0.79766, 0.80392, 0.80995, 0.81662, 0.82304, 0.82870, 0.83353, 0.83759, 0.84095, 0.84362, 0.84560,\
        0.84683, 0.84726, 0.84684, 0.84554, 0.84336, 0.84032, 0.83646, 0.83184, 0.82649, 0.82048, 0.81385,\
        0.80673, 0.79928, 0.79154, 0.78314, 0.77413, 0.76462, 0.75473, 0.74461, 0.73438, 0.72417, 0.71402,\
        0.70389, 0.69386, 0.68377, 0.67365, 0.66359, 0.65363, 0.64380, 0.63410, 0.62454, 0.61512, 0.60591,\
        0.59706, 0.58785, 0.57829, 0.56866, 0.55901, 0.54937, 0.53973, 0.53010, 0.52045, 0.51067, 0.50058],\
        [0.45274, 0.46475, 0.47692, 0.48929, 0.50214, 0.51558, 0.52887, 0.54172, 0.55405, 0.56576, 0.57668,\
        0.58658, 0.59521, 0.60206, 0.60747, 0.61138, 0.61343, 0.61366, 0.61234, 0.60981, 0.60574, 0.59953,\
        0.59087, 0.58095, 0.57070, 0.56056, 0.55079, 0.54170, 0.53377, 0.52754, 0.52332, 0.52098, 0.52015,\
        0.51965, 0.51728, 0.51015, 0.49507, 0.47371, 0.44944, 0.42500, 0.40222, 0.38218, 0.36524, 0.35185,\
        0.34248, 0.33770, 0.33813, 0.34451, 0.35770, 0.37888, 0.40954, 0.44982, 0.49706, 0.54854, 0.59667,\
        0.63459, 0.66335, 0.68600, 0.70420, 0.71842, 0.73336, 0.74842, 0.76281, 0.77609, 0.78788, 0.79829,\
        0.80673, 0.81356, 0.81996, 0.82632, 0.83239, 0.83782, 0.84246, 0.84629, 0.84933, 0.85162, 0.85314,\
        0.85388, 0.85380, 0.85290, 0.85114, 0.84853, 0.84510, 0.84087, 0.83590, 0.83024, 0.82397, 0.81716,\
        0.80995, 0.80243, 0.79432, 0.78540, 0.77595, 0.76602, 0.75574, 0.74523, 0.73465, 0.72411, 0.71371,\
        0.70342, 0.69312, 0.68280, 0.67250, 0.66225, 0.65212, 0.64211, 0.63227, 0.62255, 0.61290, 0.60329,\
        0.59405, 0.58495, 0.57552, 0.56589, 0.55617, 0.54642, 0.53668, 0.52698, 0.51734, 0.50766, 0.49765],\
        [0.44913, 0.46143, 0.47406, 0.48700, 0.50017, 0.51339, 0.52615, 0.53857, 0.55070, 0.56239, 0.57344,\
        0.58357, 0.59244, 0.59987, 0.60623, 0.61078, 0.61340, 0.61430, 0.61380, 0.61205, 0.60885, 0.60382,\
        0.59665, 0.58828, 0.57953, 0.57079, 0.56237, 0.55458, 0.54781, 0.54242, 0.53856, 0.53606, 0.53433,\
        0.53207, 0.52724, 0.51755, 0.50162, 0.47982, 0.45520, 0.43053, 0.40755, 0.38697, 0.36945, 0.35566,\
        0.34599, 0.34101, 0.34134, 0.34764, 0.36061, 0.38142, 0.41290, 0.45529, 0.50481, 0.55728, 0.60463,\
        0.64125, 0.66864, 0.69033, 0.70932, 0.72625, 0.74183, 0.75664, 0.77075, 0.78426, 0.79607, 0.80652,\
        0.81570, 0.82340, 0.83023, 0.83653, 0.84228, 0.84734, 0.85162, 0.85506, 0.85769, 0.85951, 0.86053,\
        0.86075, 0.86016, 0.85875, 0.85653, 0.85348, 0.84963, 0.84502, 0.83968, 0.83369, 0.82713, 0.82009,\
        0.81265, 0.80479, 0.79628, 0.78714, 0.77734, 0.76699, 0.75629, 0.74537, 0.73439, 0.72348, 0.71279,\
        0.70237, 0.69188, 0.68134, 0.67086, 0.66044, 0.65012, 0.63995, 0.62997, 0.62020, 0.61056, 0.60090,\
        0.59134, 0.58188, 0.57231, 0.56260, 0.55279, 0.54293, 0.53304, 0.52318, 0.51345, 0.50385, 0.49387],\
        [0.44465, 0.45717, 0.47018, 0.48366, 0.49740, 0.51007, 0.52197, 0.53392, 0.54585, 0.55752, 0.56872,\
        0.57918, 0.58867, 0.59705, 0.60391, 0.60887, 0.61172, 0.61300, 0.61310, 0.61206, 0.60970, 0.60574,\
        0.60021, 0.59385, 0.58696, 0.57998, 0.57319, 0.56686, 0.56131, 0.55676, 0.55327, 0.55061, 0.54806,\
        0.54432, 0.53748, 0.52561, 0.50785, 0.48503, 0.46009, 0.43527, 0.41197, 0.39123, 0.37392, 0.36008,\
        0.35030, 0.34534, 0.34587, 0.35259, 0.36620, 0.38730, 0.41901, 0.46199, 0.51218, 0.56597, 0.61495,\
        0.65009, 0.67598, 0.69756, 0.71722, 0.73540, 0.75169, 0.76629, 0.77941, 0.79242, 0.80519, 0.81564,\
        0.82524, 0.83355, 0.84071, 0.84697, 0.85241, 0.85703, 0.86083, 0.86378, 0.86590, 0.86719, 0.86767,\
        0.86734, 0.86623, 0.86432, 0.86162, 0.85813, 0.85386, 0.84884, 0.84311, 0.83674, 0.82983, 0.82246,\
        0.81469, 0.80646, 0.79774, 0.78843, 0.77822, 0.76745, 0.75634, 0.74503, 0.73364, 0.72230, 0.71118,\
        0.70040, 0.68987, 0.67935, 0.66877, 0.65819, 0.64766, 0.63727, 0.62712, 0.61730, 0.60778, 0.59818,\
        0.58833, 0.57850, 0.56870, 0.55886, 0.54895, 0.53898, 0.52895, 0.51886, 0.50876, 0.49904, 0.48929],\
        [0.43913, 0.45162, 0.46466, 0.47815, 0.49166, 0.50392, 0.51558, 0.52747, 0.53939, 0.55111, 0.56247,\
        0.57331, 0.58341, 0.59250, 0.60017, 0.60549, 0.60818, 0.60975, 0.61048, 0.61029, 0.60897, 0.60637,\
        0.60263, 0.59808, 0.59304, 0.58802, 0.58299, 0.57816, 0.57383, 0.57015, 0.56709, 0.56440, 0.56145,\
        0.55705, 0.54922, 0.53574, 0.51554, 0.49091, 0.46548, 0.44021, 0.41669, 0.39623, 0.37916, 0.36508,\
        0.35521, 0.35041, 0.35137, 0.35879, 0.37365, 0.39709, 0.42808, 0.46891, 0.51710, 0.56987, 0.62338,\
        0.66343, 0.68716, 0.70892, 0.72929, 0.74755, 0.76388, 0.77829, 0.79010, 0.80063, 0.81367, 0.82556,\
        0.83566, 0.84423, 0.85146, 0.85753, 0.86256, 0.86665, 0.86987, 0.87225, 0.87379, 0.87450, 0.87440,\
        0.87352, 0.87188, 0.86948, 0.86632, 0.86240, 0.85771, 0.85227, 0.84613, 0.83934, 0.83200, 0.82421,\
        0.81608, 0.80775, 0.79928, 0.78911, 0.77842, 0.76731, 0.75588, 0.74423, 0.73249, 0.72079, 0.70925,\
        0.69815, 0.68766, 0.67707, 0.66634, 0.65555, 0.64478, 0.63411, 0.62364, 0.61358, 0.60414, 0.59449,\
        0.58453, 0.57457, 0.56463, 0.55469, 0.54470, 0.53466, 0.52455, 0.51436, 0.50410, 0.49404, 0.48433],\
        [0.43252, 0.44472, 0.45741, 0.47040, 0.48319, 0.49527, 0.50724, 0.51944, 0.53145, 0.54318, 0.55466,\
        0.56578, 0.57631, 0.58598, 0.59433, 0.59956, 0.60246, 0.60469, 0.60626, 0.60716, 0.60708, 0.60590,\
        0.60378, 0.60090, 0.59773, 0.59494, 0.59162, 0.58822, 0.58516, 0.58240, 0.57986, 0.57737, 0.57453,\
        0.57052, 0.56334, 0.54973, 0.52811, 0.50112, 0.47256, 0.44608, 0.42269, 0.40277, 0.38477, 0.37020,\
        0.36035, 0.35589, 0.35736, 0.36530, 0.38036, 0.40349, 0.43567, 0.47633, 0.52385, 0.57795, 0.64185,\
        0.69024, 0.70822, 0.72725, 0.74585, 0.76287, 0.77840, 0.79267, 0.80473, 0.81476, 0.82616, 0.83739,\
        0.84722, 0.85555, 0.86247, 0.86809, 0.87252, 0.87594, 0.87850, 0.88024, 0.88116, 0.88127, 0.88057,\
        0.87913, 0.87698, 0.87412, 0.87054, 0.86622, 0.86114, 0.85529, 0.84871, 0.84145, 0.83360, 0.82529,\
        0.81664, 0.80763, 0.79820, 0.78846, 0.77776, 0.76651, 0.75490, 0.74304, 0.73104, 0.71912, 0.70769,\
        0.69665, 0.68562, 0.67465, 0.66362, 0.65257, 0.64153, 0.63056, 0.61970, 0.60905, 0.59911, 0.58975,\
        0.57997, 0.57004, 0.56007, 0.55008, 0.54007, 0.53002, 0.51990, 0.50975, 0.49962, 0.48945, 0.47943],\
        [0.42494, 0.43662, 0.44881, 0.46131, 0.47375, 0.48592, 0.49808, 0.51042, 0.52226, 0.53383, 0.54535,\
        0.55662, 0.56734, 0.57712, 0.58533, 0.59121, 0.59523, 0.59837, 0.60092, 0.60309, 0.60431, 0.60452,\
        0.60398, 0.60282, 0.60153, 0.60074, 0.59880, 0.59684, 0.59517, 0.59343, 0.59157, 0.58955, 0.58714,\
        0.58432, 0.57987, 0.56787, 0.54485, 0.51324, 0.48095, 0.45268, 0.42821, 0.40735, 0.38927, 0.37480,\
        0.36540, 0.36154, 0.36361, 0.37203, 0.38738, 0.41027, 0.44241, 0.48417, 0.53402, 0.59398, 0.66564,\
        0.71835, 0.73742, 0.75099, 0.76561, 0.78041, 0.79465, 0.80804, 0.82022, 0.83087, 0.84122, 0.85107,\
        0.85981, 0.86731, 0.87355, 0.87844, 0.88202, 0.88460, 0.88640, 0.88748, 0.88779, 0.88727, 0.88598,\
        0.88400, 0.88138, 0.87811, 0.87417, 0.86951, 0.86409, 0.85787, 0.85086, 0.84310, 0.83467, 0.82573,\
        0.81649, 0.80716, 0.79758, 0.78721, 0.77639, 0.76512, 0.75347, 0.74151, 0.72936, 0.71713, 0.70520,\
        0.69440, 0.68325, 0.67193, 0.66057, 0.64922, 0.63794, 0.62677, 0.61571, 0.60482, 0.59444, 0.58474,\
        0.57498, 0.56505, 0.55506, 0.54506, 0.53507, 0.52504, 0.51494, 0.50480, 0.49467, 0.48451, 0.47438],\
        [0.41659, 0.42764, 0.43934, 0.45151, 0.46381, 0.47608, 0.48840, 0.50067, 0.51182, 0.52320, 0.53476,\
        0.54613, 0.55690, 0.56670, 0.57508, 0.58177, 0.58701, 0.59131, 0.59507, 0.59840, 0.60084, 0.60236,\
        0.60353, 0.60413, 0.60454, 0.60480, 0.60449, 0.60405, 0.60370, 0.60311, 0.60223, 0.60106, 0.59944,\
        0.59747, 0.59781, 0.58870, 0.56467, 0.52854, 0.49176, 0.46034, 0.43378, 0.41132, 0.39275, 0.37906,\
        0.37047, 0.36737, 0.37009, 0.37882, 0.39430, 0.41723, 0.44942, 0.49197, 0.54575, 0.61081, 0.68281,\
        0.73337, 0.75750, 0.77171, 0.78505, 0.79848, 0.81191, 0.82471, 0.83664, 0.84740, 0.85708, 0.86558,\
        0.87277, 0.87895, 0.88431, 0.88826, 0.89065, 0.89225, 0.89323, 0.89365, 0.89340, 0.89226, 0.89037,\
        0.88791, 0.88491, 0.88134, 0.87713, 0.87222, 0.86653, 0.86001, 0.85261, 0.84437, 0.83533, 0.82566,\
        0.81564, 0.80560, 0.79563, 0.78525, 0.77445, 0.76323, 0.75161, 0.73967, 0.72753, 0.71533, 0.70318,\
        0.69177, 0.68041, 0.66875, 0.65703, 0.64540, 0.63392, 0.62267, 0.61165, 0.60075, 0.59016, 0.57992,\
        0.56983, 0.55973, 0.54964, 0.53961, 0.52967, 0.51974, 0.50968, 0.49951, 0.48936, 0.47920, 0.46904],\
        [0.40775, 0.41804, 0.42938, 0.44138, 0.45355, 0.46569, 0.47771, 0.48928, 0.50025, 0.51171, 0.52337,\
        0.53480, 0.54559, 0.55546, 0.56418, 0.57165, 0.57800, 0.58355, 0.58853, 0.59296, 0.59665, 0.59978,\
        0.60247, 0.60451, 0.60625, 0.60769, 0.60880, 0.60974, 0.61061, 0.61129, 0.61171, 0.61187, 0.61175,\
        0.61202, 0.61281, 0.60683, 0.58192, 0.54316, 0.50294, 0.46877, 0.44080, 0.41685, 0.39773, 0.38428,\
        0.37602, 0.37352, 0.37698, 0.38602, 0.40023, 0.42353, 0.45796, 0.50497, 0.56288, 0.62886, 0.69988,\
        0.74897, 0.77297, 0.78833, 0.80277, 0.81625, 0.82930, 0.84180, 0.85344, 0.86396, 0.87297, 0.88010,\
        0.88530, 0.88961, 0.89409, 0.89710, 0.89787, 0.89852, 0.89861, 0.89837, 0.89772, 0.89590, 0.89350,\
        0.89069, 0.88744, 0.88368, 0.87932, 0.87427, 0.86842, 0.86168, 0.85400, 0.84535, 0.83576, 0.82534,\
        0.81437, 0.80338, 0.79300, 0.78274, 0.77206, 0.76090, 0.74931, 0.73739, 0.72530, 0.71332, 0.70193,\
        0.68961, 0.67715, 0.66491, 0.65284, 0.64095, 0.62928, 0.61794, 0.60721, 0.59637, 0.58549, 0.57485,\
        0.56443, 0.55409, 0.54382, 0.53369, 0.52380, 0.51408, 0.50416, 0.49396, 0.48374, 0.47353, 0.46336],\
        [0.39853, 0.40811, 0.41950, 0.43139, 0.44330, 0.45507, 0.46664, 0.47786, 0.48888, 0.50014, 0.51178,\
        0.52314, 0.53387, 0.54383, 0.55290, 0.56107, 0.56842, 0.57511, 0.58127, 0.58690, 0.59197, 0.59654,\
        0.60042, 0.60381, 0.60678, 0.60933, 0.61163, 0.61380, 0.61587, 0.61779, 0.61951, 0.62099, 0.62226,\
        0.62322, 0.62217, 0.61354, 0.59044, 0.55416, 0.51387, 0.47974, 0.45056, 0.42562, 0.40564, 0.39099,\
        0.38225, 0.37982, 0.38408, 0.39410, 0.40818, 0.43028, 0.46629, 0.51948, 0.58427, 0.64955, 0.72493,\
        0.77588, 0.79131, 0.80599, 0.82045, 0.83396, 0.84678, 0.85883, 0.86991, 0.87995, 0.88819, 0.89400,\
        0.89692, 0.89877, 0.90181, 0.90378, 0.90358, 0.90309, 0.90228, 0.90130, 0.89999, 0.89779, 0.89517,\
        0.89221, 0.88886, 0.88504, 0.88066, 0.87558, 0.86969, 0.86288, 0.85504, 0.84612, 0.83612, 0.82515,\
        0.81342, 0.80149, 0.79077, 0.78029, 0.76945, 0.75817, 0.74648, 0.73445, 0.72222, 0.70989, 0.69751,\
        0.68529, 0.67265, 0.66009, 0.64777, 0.63572, 0.62392, 0.61225, 0.60119, 0.59088, 0.57998, 0.56922,\
        0.55861, 0.54810, 0.53761, 0.52725, 0.51728, 0.50790, 0.49821, 0.48798, 0.47769, 0.46747, 0.45732],\
        [0.38988, 0.39943, 0.41060, 0.42202, 0.43338, 0.44463, 0.45579, 0.46682, 0.47782, 0.48900, 0.50041,\
        0.51147, 0.52203, 0.53204, 0.54142, 0.55018, 0.55840, 0.56615, 0.57344, 0.58025, 0.58655, 0.59231,\
        0.59749, 0.60220, 0.60611, 0.60958, 0.61292, 0.61618, 0.61933, 0.62233, 0.62508, 0.62743, 0.62910,\
        0.62931, 0.62608, 0.61599, 0.59521, 0.56255, 0.52659, 0.49262, 0.46216, 0.43606, 0.41482, 0.39917,\
        0.38890, 0.38583, 0.39077, 0.40111, 0.41516, 0.43744, 0.47508, 0.53415, 0.60406, 0.67221, 0.76478,\
        0.81776, 0.82314, 0.83045, 0.84098, 0.85262, 0.86456, 0.87579, 0.88525, 0.89460, 0.90161, 0.90587,\
        0.90732, 0.90761, 0.90835, 0.90862, 0.90767, 0.90613, 0.90435, 0.90243, 0.90030, 0.89801, 0.89541,\
        0.89246, 0.88914, 0.88538, 0.88106, 0.87608, 0.87027, 0.86351, 0.85565, 0.84662, 0.83642, 0.82517,\
        0.81321, 0.80114, 0.78940, 0.77798, 0.76655, 0.75491, 0.74297, 0.73068, 0.71812, 0.70558, 0.69271,\
        0.67974, 0.66682, 0.65406, 0.64164, 0.62958, 0.61786, 0.60646, 0.59493, 0.58420, 0.57354, 0.56289,\
        0.55233, 0.54179, 0.53116, 0.52046, 0.50999, 0.50053, 0.49126, 0.48125, 0.47110, 0.46097, 0.45091],\
        [0.38273, 0.39229, 0.40271, 0.41334, 0.42389, 0.43451, 0.44528, 0.45612, 0.46701, 0.47802, 0.48905,\
        0.49982, 0.51023, 0.52027, 0.52989, 0.53913, 0.54813, 0.55684, 0.56518, 0.57306, 0.58046, 0.58733,\
        0.59369, 0.59940, 0.60396, 0.60834, 0.61265, 0.61686, 0.62094, 0.62479, 0.62823, 0.63097, 0.63244,\
        0.63159, 0.62659, 0.61496, 0.59489, 0.56759, 0.53592, 0.50397, 0.47410, 0.44758, 0.42551, 0.40813,\
        0.39563, 0.39134, 0.39798, 0.40928, 0.42424, 0.44722, 0.48701, 0.55196, 0.62506, 0.69904, 0.78918,\
        0.84076, 0.85264, 0.85631, 0.86268, 0.87220, 0.88251, 0.89334, 0.89896, 0.90682, 0.91272, 0.91564,\
        0.91617, 0.91539, 0.91423, 0.91267, 0.91047, 0.90784, 0.90509, 0.90239, 0.89969, 0.89716, 0.89449,\
        0.89156, 0.88829, 0.88463, 0.88046, 0.87565, 0.87003, 0.86342, 0.85568, 0.84668, 0.83640, 0.82497,\
        0.81270, 0.80008, 0.78752, 0.77511, 0.76291, 0.75080, 0.73856, 0.72598, 0.71286, 0.69951, 0.68636,\
        0.67298, 0.65971, 0.64676, 0.63431, 0.62231, 0.61062, 0.59941, 0.58823, 0.57708, 0.56637, 0.55589,\
        0.54556, 0.53524, 0.52472, 0.51383, 0.50294, 0.49295, 0.48365, 0.47396, 0.46402, 0.45406, 0.44414],\
        [0.37604, 0.38530, 0.39515, 0.40510, 0.41462, 0.42467, 0.43515, 0.44578, 0.45646, 0.46717, 0.47785,\
        0.48836, 0.49864, 0.50873, 0.51845, 0.52805, 0.53778, 0.54740, 0.55668, 0.56551, 0.57381, 0.58155,\
        0.58865, 0.59490, 0.60038, 0.60579, 0.61095, 0.61595, 0.62078, 0.62527, 0.62917, 0.63205, 0.63325,\
        0.63169, 0.62591, 0.61442, 0.59634, 0.57210, 0.54394, 0.51463, 0.48608, 0.46041, 0.43865, 0.41970,\
        0.40432, 0.39798, 0.40818, 0.42278, 0.43960, 0.46229, 0.50329, 0.57108, 0.64869, 0.72619, 0.80353,\
        0.85310, 0.87219, 0.87808, 0.88388, 0.89333, 0.90148, 0.90971, 0.91525, 0.91968, 0.92330, 0.92462,\
        0.92390, 0.92185, 0.91910, 0.91589, 0.91229, 0.90848, 0.90473, 0.90136, 0.89842, 0.89548, 0.89259,\
        0.88957, 0.88631, 0.88274, 0.87874, 0.87415, 0.86878, 0.86243, 0.85488, 0.84597, 0.83563, 0.82396,\
        0.81123, 0.79771, 0.78419, 0.77086, 0.75793, 0.74535, 0.73289, 0.72024, 0.70680, 0.69275, 0.67882,\
        0.66494, 0.65129, 0.63811, 0.62564, 0.61389, 0.60225, 0.59079, 0.57975, 0.56888, 0.55837, 0.54818,\
        0.53824, 0.52839, 0.51842, 0.50786, 0.49676, 0.48637, 0.47642, 0.46655, 0.45666, 0.44682, 0.43708],\
        [0.36944, 0.37826, 0.38743, 0.39641, 0.40530, 0.41519, 0.42551, 0.43590, 0.44626, 0.45663, 0.46700,\
        0.47731, 0.48752, 0.49764, 0.50713, 0.51708, 0.52764, 0.53809, 0.54816, 0.55776, 0.56681, 0.57523,\
        0.58294, 0.58990, 0.59628, 0.60231, 0.60801, 0.61366, 0.61909, 0.62406, 0.62829, 0.63133, 0.63250,\
        0.63084, 0.62524, 0.61467, 0.59861, 0.57758, 0.55241, 0.52490, 0.49768, 0.47644, 0.45726, 0.43840,\
        0.42282, 0.41791, 0.42788, 0.44493, 0.46399, 0.48414, 0.52112, 0.58769, 0.66863, 0.74743, 0.82030,\
        0.87124, 0.89328, 0.90256, 0.90903, 0.91765, 0.92429, 0.92906, 0.93223, 0.93390, 0.93440, 0.93337,\
        0.93086, 0.92725, 0.92296, 0.91825, 0.91329, 0.90830, 0.90355, 0.89939, 0.89603, 0.89285, 0.88971,\
        0.88651, 0.88318, 0.87964, 0.87577, 0.87140, 0.86633, 0.86027, 0.85296, 0.84415, 0.83367, 0.82156,\
        0.80822, 0.79396, 0.77900, 0.76461, 0.75099, 0.73804, 0.72538, 0.71274, 0.69983, 0.68485, 0.67004,\
        0.65563, 0.64162, 0.62813, 0.61549, 0.60396, 0.59272, 0.58124, 0.57022, 0.55965, 0.54949, 0.53970,\
        0.53021, 0.52091, 0.51168, 0.50209, 0.49097, 0.47986, 0.46928, 0.45905, 0.44908, 0.43933, 0.42978],\
        [0.36291, 0.37123, 0.37971, 0.38813, 0.39689, 0.40649, 0.41659, 0.42661, 0.43652, 0.44649, 0.45659,\
        0.46674, 0.47685, 0.48672, 0.49622, 0.50691, 0.51820, 0.52921, 0.53984, 0.55002, 0.55967, 0.56866,\
        0.57695, 0.58452, 0.59147, 0.59787, 0.60407, 0.61033, 0.61620, 0.62150, 0.62604, 0.62937, 0.63087,\
        0.62973, 0.62512, 0.61631, 0.60301, 0.58545, 0.56347, 0.53909, 0.51614, 0.49955, 0.48304, 0.46564,\
        0.45060, 0.44670, 0.45715, 0.47368, 0.49319, 0.51245, 0.54362, 0.60405, 0.68561, 0.76491, 0.84640,\
        0.90559, 0.92741, 0.93746, 0.94263, 0.94684, 0.94952, 0.95043, 0.94991, 0.94819, 0.94545, 0.94171,\
        0.93705, 0.93169, 0.92587, 0.91978, 0.91358, 0.90748, 0.90195, 0.89717, 0.89313, 0.88945, 0.88589,\
        0.88237, 0.87883, 0.87521, 0.87140, 0.86722, 0.86241, 0.85665, 0.84957, 0.84083, 0.83013, 0.81728,\
        0.80249, 0.78686, 0.77100, 0.75575, 0.74160, 0.72848, 0.71569, 0.70236, 0.68891, 0.67433, 0.65946,\
        0.64497, 0.63091, 0.61720, 0.60417, 0.59258, 0.58154, 0.57050, 0.55977, 0.54956, 0.53984, 0.53050,\
        0.52140, 0.51248, 0.50359, 0.49422, 0.48365, 0.47259, 0.46175, 0.45129, 0.44125, 0.43160, 0.42228],\
        [0.35656, 0.36446, 0.37254, 0.38080, 0.38947, 0.39879, 0.40853, 0.41791, 0.42718, 0.43674, 0.44666,\
        0.45677, 0.46691, 0.47698, 0.48725, 0.49832, 0.50969, 0.52084, 0.53178, 0.54242, 0.55255, 0.56204,\
        0.57081, 0.57887, 0.58629, 0.59317, 0.59984, 0.60642, 0.61244, 0.61790, 0.62278, 0.62661, 0.62876,\
        0.62860, 0.62553, 0.61915, 0.60930, 0.59533, 0.57712, 0.55744, 0.53974, 0.52577, 0.51318, 0.49917,\
        0.48409, 0.47918, 0.49217, 0.50371, 0.51615, 0.53218, 0.56221, 0.62020, 0.70116, 0.77828, 0.86754,\
        0.93534, 0.96353, 0.97402, 0.97709, 0.97698, 0.97499, 0.97154, 0.96698, 0.96159, 0.95558, 0.94907,\
        0.94216, 0.93501, 0.92774, 0.92047, 0.91327, 0.90618, 0.89963, 0.89444, 0.88970, 0.88531, 0.88114,\
        0.87712, 0.87320, 0.86934, 0.86546, 0.86138, 0.85680, 0.85128, 0.84436, 0.83558, 0.82462, 0.81100,\
        0.79466, 0.77720, 0.76008, 0.74408, 0.72949, 0.71640, 0.70406, 0.69034, 0.67614, 0.66162, 0.64708,\
        0.63303, 0.61945, 0.60604, 0.59297, 0.58112, 0.56996, 0.55909, 0.54863, 0.53880, 0.52964, 0.52081,\
        0.51201, 0.50332, 0.49454, 0.48526, 0.47515, 0.46446, 0.45369, 0.44318, 0.43312, 0.42364, 0.41465],\
        [0.35043, 0.35802, 0.36583, 0.37395, 0.38252, 0.39163, 0.40082, 0.40949, 0.41802, 0.42732, 0.43726,\
        0.44749, 0.45781, 0.46825, 0.47901, 0.49024, 0.50164, 0.51275, 0.52396, 0.53502, 0.54557, 0.55545,\
        0.56461, 0.57308, 0.58095, 0.58834, 0.59544, 0.60223, 0.60788, 0.61343, 0.61881, 0.62332, 0.62635,\
        0.62738, 0.62601, 0.62194, 0.61495, 0.60465, 0.59099, 0.57580, 0.56242, 0.55225, 0.54429, 0.53623,\
        0.52734, 0.52394, 0.53069, 0.53440, 0.53715, 0.54666, 0.57294, 0.62857, 0.70575, 0.78304, 0.87886,\
        0.95308, 0.98977, 1.00379, 1.00640, 1.00345, 0.99764, 0.99021, 0.98184, 0.97297, 0.96387, 0.95475,\
        0.94573, 0.93691, 0.92836, 0.92013, 0.91228, 0.90485, 0.89776, 0.89137, 0.88566, 0.88038, 0.87542,\
        0.87071, 0.86622, 0.86193, 0.85780, 0.85369, 0.84923, 0.84388, 0.83697, 0.82784, 0.81624, 0.80201,\
        0.78371, 0.76448, 0.74627, 0.72985, 0.71500, 0.70165, 0.68958, 0.67630, 0.66211, 0.64760, 0.63335,\
        0.62000, 0.60733, 0.59450, 0.58181, 0.56980, 0.55844, 0.54752, 0.53714, 0.52759, 0.51911, 0.51109,\
        0.50250, 0.49398, 0.48533, 0.47617, 0.46634, 0.45596, 0.44536, 0.43485, 0.42478, 0.41544, 0.40692],\
        [0.34452, 0.35183, 0.35939, 0.36727, 0.37554, 0.38415, 0.39280, 0.40080, 0.40880, 0.41835, 0.42853,\
        0.43893, 0.44943, 0.46009, 0.47103, 0.48224, 0.49349, 0.50471, 0.51644, 0.52793, 0.53879, 0.54893,\
        0.55836, 0.56713, 0.57533, 0.58305, 0.59035, 0.59688, 0.60224, 0.60837, 0.61449, 0.61977, 0.62373,\
        0.62599, 0.62623, 0.62419, 0.61968, 0.61259, 0.60301, 0.59193, 0.58282, 0.57695, 0.57475, 0.57473,\
        0.57635, 0.57895, 0.57368, 0.56475, 0.55779, 0.55905, 0.57844, 0.62748, 0.70102, 0.78630, 0.89173,\
        0.96801, 1.00794, 1.02468, 1.02785, 1.02350, 1.01511, 1.00462, 0.99311, 0.98126, 0.96950, 0.95812,\
        0.94727, 0.93705, 0.92745, 0.91845, 0.91003, 0.90235, 0.89481, 0.88747, 0.88074, 0.87451, 0.86867,\
        0.86314, 0.85789, 0.85293, 0.84832, 0.84399, 0.83955, 0.83424, 0.82722, 0.81737, 0.80407, 0.78793,\
        0.76870, 0.74846, 0.72969, 0.71363, 0.69950, 0.68634, 0.67410, 0.66123, 0.64743, 0.63312, 0.61883,\
        0.60608, 0.59436, 0.58230, 0.57021, 0.55847, 0.54713, 0.53615, 0.52569, 0.51615, 0.50817, 0.50174,\
        0.49343, 0.48494, 0.47633, 0.46729, 0.45769, 0.44758, 0.43714, 0.42666, 0.41651, 0.40708, 0.39908],\
        [0.33881, 0.34583, 0.35309, 0.36065, 0.36852, 0.37658, 0.38457, 0.39219, 0.40050, 0.41034, 0.42066,\
        0.43108, 0.44159, 0.45230, 0.46330, 0.47454, 0.48590, 0.49752, 0.50952, 0.52121, 0.53217, 0.54241,\
        0.55199, 0.56096, 0.56936, 0.57723, 0.58450, 0.59096, 0.59696, 0.60359, 0.61026, 0.61615, 0.62096,\
        0.62438, 0.62611, 0.62591, 0.62361, 0.61921, 0.61294, 0.60612, 0.60074, 0.59849, 0.60107, 0.60640,\
        0.61257, 0.61563, 0.60559, 0.59028, 0.57739, 0.57121, 0.58396, 0.62937, 0.70143, 0.79435, 0.90706,\
        0.97927, 1.01906, 1.03708, 1.04090, 1.03603, 1.02616, 1.01360, 0.99979, 0.98567, 0.97182, 0.95866,\
        0.94640, 0.93513, 0.92479, 0.91525, 0.90629, 0.89783, 0.88997, 0.88219, 0.87471, 0.86761, 0.86086,\
        0.85443, 0.84829, 0.84243, 0.83701, 0.83224, 0.82771, 0.82218, 0.81484, 0.80453, 0.78925, 0.77116,\
        0.75100, 0.73022, 0.71125, 0.69606, 0.68328, 0.67112, 0.65914, 0.64654, 0.63307, 0.61930, 0.60526,\
        0.59229, 0.58120, 0.56991, 0.55853, 0.54731, 0.53628, 0.52540, 0.51487, 0.50527, 0.49726, 0.49116,\
        0.48402, 0.47600, 0.46761, 0.45880, 0.44949, 0.43967, 0.42946, 0.41907, 0.40893, 0.39964, 0.39151],\
        [0.33328, 0.33999, 0.34692, 0.35415, 0.36165, 0.36934, 0.37713, 0.38509, 0.39380, 0.40350, 0.41364,\
        0.42381, 0.43410, 0.44472, 0.45574, 0.46712, 0.47883, 0.49086, 0.50307, 0.51471, 0.52550, 0.53571,\
        0.54538, 0.55450, 0.56307, 0.57111, 0.57858, 0.58554, 0.59232, 0.59935, 0.60620, 0.61244, 0.61804,\
        0.62257, 0.62569, 0.62719, 0.62697, 0.62513, 0.62208, 0.61885, 0.61673, 0.61736, 0.62146, 0.62725,\
        0.63197, 0.63144, 0.62229, 0.60822, 0.59678, 0.58559, 0.58823, 0.64286, 0.73269, 0.83008, 0.91803,\
        0.98537, 1.02399, 1.04206, 1.04614, 1.04111, 1.03049, 1.01672, 1.00144, 0.98578, 0.97049, 0.95609,\
        0.94291, 0.93102, 0.92030, 0.91051, 0.90143, 0.89255, 0.88403, 0.87572, 0.86758, 0.85967, 0.85205,\
        0.84474, 0.83768, 0.83074, 0.82410, 0.81849, 0.81396, 0.80785, 0.79913, 0.78786, 0.77189, 0.75301,\
        0.73258, 0.71195, 0.69355, 0.67932, 0.66741, 0.65627, 0.64505, 0.63295, 0.61957, 0.60612, 0.59362,\
        0.58071, 0.56909, 0.55802, 0.54715, 0.53652, 0.52611, 0.51570, 0.50524, 0.49594, 0.48816, 0.48147,\
        0.47465, 0.46720, 0.45924, 0.45080, 0.44186, 0.43244, 0.42262, 0.41249, 0.40246, 0.39346, 0.38536],\
        [0.32792, 0.33429, 0.34089, 0.34780, 0.35503, 0.36255, 0.37037, 0.37863, 0.38758, 0.39725, 0.40707,\
        0.41678, 0.42668, 0.43713, 0.44815, 0.45964, 0.47157, 0.48389, 0.49636, 0.50771, 0.51832, 0.52860,\
        0.53840, 0.54769, 0.55646, 0.56475, 0.57257, 0.58006, 0.58743, 0.59479, 0.60167, 0.60845, 0.61494,\
        0.62055, 0.62498, 0.62808, 0.62982, 0.63035, 0.63007, 0.62971, 0.63018, 0.63235, 0.63625, 0.64050,\
        0.64276, 0.64038, 0.63232, 0.62128, 0.61178, 0.60391, 0.60591, 0.67522, 0.77524, 0.86895, 0.94044,\
        0.99204, 1.02487, 1.04101, 1.04456, 1.03934, 1.02841, 1.01409, 0.99808, 0.98159, 0.96550, 0.95042,\
        0.93681, 0.92480, 0.91412, 0.90429, 0.89507, 0.88607, 0.87713, 0.86829, 0.85951, 0.85082, 0.84232,\
        0.83418, 0.82640, 0.81855, 0.81037, 0.80333, 0.79846, 0.79187, 0.78147, 0.76853, 0.75249, 0.73406,\
        0.71443, 0.69507, 0.67801, 0.66412, 0.65255, 0.64215, 0.63193, 0.62080, 0.60789, 0.59401, 0.58162,\
        0.56941, 0.55773, 0.54669, 0.53614, 0.52605, 0.51647, 0.50729, 0.49728, 0.48801, 0.48004, 0.47292,\
        0.46602, 0.45887, 0.45132, 0.44332, 0.43484, 0.42591, 0.41667, 0.40719, 0.39752, 0.38850, 0.38018],\
        [0.32273, 0.32873, 0.33500, 0.34163, 0.34865, 0.35607, 0.36391, 0.37228, 0.38124, 0.39074, 0.40040,\
        0.40949, 0.41896, 0.42935, 0.44034, 0.45179, 0.46368, 0.47587, 0.48797, 0.49931, 0.51034, 0.52094,\
        0.53094, 0.54044, 0.54947, 0.55805, 0.56627, 0.57422, 0.58199, 0.58957, 0.59689, 0.60439, 0.61172,\
        0.61830, 0.62391, 0.62848, 0.63202, 0.63466, 0.63669, 0.63854, 0.64073, 0.64360, 0.64692, 0.64971,\
        0.65044, 0.64771, 0.64119, 0.63250, 0.62493, 0.62458, 0.65043, 0.72296, 0.81435, 0.89582, 0.95492,\
        0.99555, 1.02157, 1.03464, 1.03708, 1.03156, 1.02058, 1.00620, 0.99008, 0.97346, 0.95720, 0.94197,\
        0.92839, 0.91680, 0.90670, 0.89715, 0.88786, 0.87863, 0.86935, 0.86003, 0.85066, 0.84121, 0.83177,\
        0.82277, 0.81458, 0.80635, 0.79748, 0.78911, 0.78229, 0.77408, 0.76273, 0.74859, 0.73243, 0.71485,\
        0.69650, 0.67875, 0.66290, 0.64947, 0.63816, 0.62829, 0.61910, 0.60937, 0.59725, 0.58362, 0.57056,\
        0.55827, 0.54663, 0.53568, 0.52539, 0.51570, 0.50660, 0.49828, 0.48954, 0.48044, 0.47232, 0.46498,\
        0.45802, 0.45107, 0.44391, 0.43637, 0.42837, 0.41990, 0.41119, 0.40250, 0.39332, 0.38419, 0.37560],\
        [0.31770, 0.32330, 0.32923, 0.33560, 0.34245, 0.34977, 0.35757, 0.36586, 0.37462, 0.38372, 0.39280,\
        0.40133, 0.41092, 0.42143, 0.43229, 0.44349, 0.45507, 0.46694, 0.47880, 0.49043, 0.50191, 0.51266,\
        0.52287, 0.53266, 0.54199, 0.55090, 0.55949, 0.56788, 0.57611, 0.58422, 0.59226, 0.60036, 0.60837,\
        0.61569, 0.62231, 0.62821, 0.63338, 0.63788, 0.64185, 0.64547, 0.64889, 0.65219, 0.65513, 0.65718,\
        0.65769, 0.65618, 0.65278, 0.64860, 0.64696, 0.65678, 0.70168, 0.78208, 0.85385, 0.91326, 0.95884,\
        0.99178, 1.01285, 1.02319, 1.02446, 1.01866, 1.00782, 0.99378, 0.97811, 0.96201, 0.94633, 0.93155,\
        0.91831, 0.90765, 0.89850, 0.88943, 0.88007, 0.87050, 0.86079, 0.85101, 0.84115, 0.83109, 0.82067,\
        0.81041, 0.80178, 0.79341, 0.78421, 0.77522, 0.76650, 0.75655, 0.74402, 0.72865, 0.71209, 0.69557,\
        0.67841, 0.66212, 0.64747, 0.63469, 0.62361, 0.61400, 0.60529, 0.59644, 0.58502, 0.57210, 0.55929,\
        0.54711, 0.53560, 0.52480, 0.51480, 0.50550, 0.49676, 0.48862, 0.48055, 0.47232, 0.46450, 0.45724,\
        0.45037, 0.44366, 0.43690, 0.42988, 0.42241, 0.41433, 0.40575, 0.39729, 0.38867, 0.37985, 0.37121],\
        [0.31284, 0.31798, 0.32356, 0.32971, 0.33642, 0.34363, 0.35131, 0.35941, 0.36786, 0.37650, 0.38509,\
        0.39374, 0.40345, 0.41369, 0.42407, 0.43475, 0.44588, 0.45740, 0.46913, 0.48091, 0.49251, 0.50346,\
        0.51412, 0.52431, 0.53393, 0.54315, 0.55212, 0.56096, 0.56970, 0.57842, 0.58717, 0.59603, 0.60446,\
        0.61234, 0.61989, 0.62702, 0.63369, 0.63990, 0.64563, 0.65080, 0.65534, 0.65914, 0.66208, 0.66412,\
        0.66542, 0.66649, 0.66823, 0.67172, 0.68067, 0.70481, 0.75988, 0.82723, 0.87547, 0.91665, 0.95374,\
        0.98146, 0.99901, 1.00721, 1.00750, 1.00154, 0.99105, 0.97766, 0.96289, 0.94801, 0.93387, 0.92068,\
        0.90804, 0.89864, 0.89046, 0.88147, 0.87179, 0.86170, 0.85143, 0.84114, 0.83094, 0.82064, 0.80956,\
        0.79816, 0.78820, 0.77912, 0.76977, 0.76036, 0.75059, 0.73949, 0.72618, 0.71008, 0.69214, 0.67596,\
        0.65997, 0.64504, 0.63163, 0.61967, 0.60885, 0.59926, 0.59059, 0.58158, 0.57106, 0.55930, 0.54732,\
        0.53570, 0.52458, 0.51400, 0.50433, 0.49550, 0.48716, 0.47926, 0.47154, 0.46390, 0.45651, 0.44950,\
        0.44284, 0.43643, 0.43011, 0.42367, 0.41688, 0.40941, 0.40095, 0.39252, 0.38405, 0.37541, 0.36680],\
        [0.30814, 0.31272, 0.31800, 0.32400, 0.33060, 0.33769, 0.34521, 0.35309, 0.36124, 0.36957, 0.37805,\
        0.38688, 0.39634, 0.40605, 0.41560, 0.42558, 0.43619, 0.44734, 0.45888, 0.47061, 0.48225, 0.49363,\
        0.50484, 0.51536, 0.52518, 0.53468, 0.54404, 0.55332, 0.56256, 0.57179, 0.58106, 0.59029, 0.59919,\
        0.60788, 0.61640, 0.62465, 0.63272, 0.64060, 0.64807, 0.65486, 0.66065, 0.66520, 0.66851, 0.67093,\
        0.67334, 0.67725, 0.68471, 0.69691, 0.71633, 0.75336, 0.80656, 0.84926, 0.88269, 0.91347, 0.94401,\
        0.96706, 0.98142, 0.98768, 0.98710, 0.98114, 0.97121, 0.95872, 0.94516, 0.93199, 0.92034, 0.91064,\
        0.90244, 0.89266, 0.88336, 0.87349, 0.86300, 0.85213, 0.84113, 0.83023, 0.81960, 0.80939, 0.79842,\
        0.78604, 0.77463, 0.76416, 0.75417, 0.74437, 0.73412, 0.72232, 0.70831, 0.69240, 0.67423, 0.65704,\
        0.64138, 0.62763, 0.61557, 0.60448, 0.59432, 0.58477, 0.57591, 0.56676, 0.55672, 0.54589, 0.53485,\
        0.52410, 0.51374, 0.50357, 0.49415, 0.48575, 0.47779, 0.47016, 0.46276, 0.45551, 0.44846, 0.44171,\
        0.43530, 0.42919, 0.42330, 0.41744, 0.41136, 0.40483, 0.39705, 0.38827, 0.37956, 0.37089, 0.36229],\
        [0.30359, 0.30747, 0.31268, 0.31862, 0.32511, 0.33207, 0.33940, 0.34705, 0.35492, 0.36300, 0.37129,\
        0.37990, 0.38888, 0.39790, 0.40660, 0.41601, 0.42614, 0.43691, 0.44821, 0.45983, 0.47158, 0.48330,\
        0.49485, 0.50560, 0.51555, 0.52537, 0.53517, 0.54490, 0.55457, 0.56420, 0.57383, 0.58342, 0.59288,\
        0.60232, 0.61168, 0.62080, 0.63021, 0.63979, 0.64915, 0.65781, 0.66521, 0.67089, 0.67483, 0.67774,\
        0.68099, 0.68665, 0.69743, 0.71511, 0.73936, 0.77322, 0.81438, 0.85296, 0.88343, 0.90971, 0.93273,\
        0.95027, 0.96120, 0.96545, 0.96407, 0.95839, 0.94930, 0.93794, 0.92578, 0.91438, 0.90554, 0.89907,\
        0.89520, 0.88628, 0.87566, 0.86476, 0.85333, 0.84155, 0.82979, 0.81818, 0.80685, 0.79602, 0.78489,\
        0.77240, 0.75993, 0.74815, 0.73726, 0.72702, 0.71659, 0.70447, 0.68949, 0.67296, 0.65541, 0.63829,\
        0.62275, 0.60985, 0.59967, 0.58927, 0.57971, 0.57058, 0.56168, 0.55251, 0.54275, 0.53250, 0.52224,\
        0.51242, 0.50303, 0.49371, 0.48470, 0.47647, 0.46876, 0.46139, 0.45425, 0.44728, 0.44049, 0.43394,\
        0.42769, 0.42181, 0.41627, 0.41087, 0.40523, 0.39903, 0.39173, 0.38337, 0.37474, 0.36612, 0.35758],\
        [0.29892, 0.30279, 0.30800, 0.31374, 0.32008, 0.32687, 0.33400, 0.34139, 0.34898, 0.35674, 0.36468,\
        0.37281, 0.38107, 0.38925, 0.39750, 0.40639, 0.41600, 0.42634, 0.43733, 0.44880, 0.46050, 0.47223,\
        0.48374, 0.49464, 0.50493, 0.51528, 0.52558, 0.53572, 0.54572, 0.55564, 0.56554, 0.57547, 0.58548,\
        0.59568, 0.60539, 0.61501, 0.62588, 0.63722, 0.64863, 0.65954, 0.66912, 0.67645, 0.68134, 0.68477,\
        0.68859, 0.69510, 0.70756, 0.72656, 0.75073, 0.77995, 0.81347, 0.84795, 0.87748, 0.90102, 0.91772,\
        0.93061, 0.93866, 0.94102, 0.93895, 0.93420, 0.92637, 0.91633, 0.90593, 0.89642, 0.88972, 0.88522,\
        0.88110, 0.87443, 0.86515, 0.85436, 0.84237, 0.82969, 0.81731, 0.80518, 0.79331, 0.78175, 0.76988,\
        0.75714, 0.74392, 0.73085, 0.71887, 0.70811, 0.69724, 0.68472, 0.66968, 0.65321, 0.63628, 0.61987,\
        0.60497, 0.59264, 0.58328, 0.57424, 0.56541, 0.55678, 0.54811, 0.53909, 0.52954, 0.51963, 0.50988,\
        0.50081, 0.49226, 0.48375, 0.47543, 0.46755, 0.46010, 0.45297, 0.44607, 0.43932, 0.43272, 0.42627,\
        0.42008, 0.41428, 0.40894, 0.40387, 0.39850, 0.39236, 0.38539, 0.37759, 0.36936, 0.36098, 0.35259],\
        [0.29534, 0.29964, 0.30395, 0.30946, 0.31562, 0.32220, 0.32909, 0.33620, 0.34347, 0.35087, 0.35836,\
        0.36594, 0.37356, 0.38117, 0.38894, 0.39718, 0.40613, 0.41593, 0.42655, 0.43777, 0.44923, 0.46074,\
        0.47211, 0.48313, 0.49389, 0.50470, 0.51541, 0.52587, 0.53607, 0.54613, 0.55618, 0.56630, 0.57653,\
        0.58676, 0.59638, 0.60736, 0.61965, 0.63247, 0.64591, 0.65948, 0.67205, 0.68191, 0.68811, 0.69217,\
        0.69663, 0.70426, 0.71670, 0.73415, 0.75575, 0.78099, 0.80923, 0.83939, 0.86522, 0.88403, 0.89689,\
        0.90704, 0.91359, 0.91468, 0.91198, 0.90935, 0.90357, 0.89474, 0.88649, 0.87893, 0.87321, 0.86906,\
        0.86509, 0.85966, 0.85203, 0.84221, 0.83016, 0.81646, 0.80338, 0.79131, 0.77901, 0.76683, 0.75428,\
        0.74104, 0.72715, 0.71284, 0.69942, 0.68812, 0.67658, 0.66380, 0.64930, 0.63356, 0.61766, 0.60245,\
        0.58893, 0.57762, 0.56832, 0.55991, 0.55176, 0.54366, 0.53539, 0.52670, 0.51742, 0.50775, 0.49844,\
        0.48995, 0.48197, 0.47411, 0.46639, 0.45894, 0.45179, 0.44491, 0.43824, 0.43172, 0.42527, 0.41889,\
        0.41265, 0.40676, 0.40145, 0.39656, 0.39129, 0.38536, 0.37868, 0.37134, 0.36354, 0.35548, 0.34734],\
        [0.29191, 0.29601, 0.30051, 0.30589, 0.31179, 0.31812, 0.32475, 0.33155, 0.33847, 0.34546, 0.35250,\
        0.35956, 0.36663, 0.37375, 0.38105, 0.38871, 0.39698, 0.40603, 0.41625, 0.42711, 0.43810, 0.44924,\
        0.46044, 0.47159, 0.48268, 0.49379, 0.50478, 0.51543, 0.52568, 0.53575, 0.54581, 0.55594, 0.56618,\
        0.57647, 0.58706, 0.59894, 0.61124, 0.62487, 0.63997, 0.65629, 0.67286, 0.68685, 0.69498, 0.69974,\
        0.70509, 0.71243, 0.72320, 0.73771, 0.75579, 0.77710, 0.80171, 0.82848, 0.84897, 0.86236, 0.87115,\
        0.87866, 0.88461, 0.88648, 0.88511, 0.88395, 0.88036, 0.87372, 0.86688, 0.86053, 0.85521, 0.85106,\
        0.84739, 0.84296, 0.83682, 0.82849, 0.81708, 0.80282, 0.78865, 0.77605, 0.76358, 0.75104, 0.73798,\
        0.72418, 0.70996, 0.69497, 0.68064, 0.66809, 0.65599, 0.64323, 0.62945, 0.61470, 0.59995, 0.58645,\
        0.57422, 0.56373, 0.55475, 0.54667, 0.53898, 0.53137, 0.52361, 0.51541, 0.50659, 0.49731, 0.48842,\
        0.48024, 0.47252, 0.46505, 0.45776, 0.45068, 0.44384, 0.43721, 0.43079, 0.42452, 0.41828, 0.41197,\
        0.40569, 0.39972, 0.39440, 0.38939, 0.38414, 0.37833, 0.37190, 0.36491, 0.35747, 0.34975, 0.34187],\
        [0.28879, 0.29313, 0.29780, 0.30291, 0.30854, 0.31464, 0.32100, 0.32748, 0.33403, 0.34061, 0.34719,\
        0.35378, 0.36039, 0.36707, 0.37394, 0.38114, 0.38883, 0.39719, 0.40714, 0.41720, 0.42731, 0.43794,\
        0.44892, 0.46006, 0.47129, 0.48253, 0.49369, 0.50440, 0.51456, 0.52458, 0.53456, 0.54462, 0.55483,\
        0.56526, 0.57614, 0.58766, 0.59985, 0.61387, 0.62957, 0.64765, 0.66847, 0.68923, 0.70165, 0.70645,\
        0.71063, 0.71631, 0.72454, 0.73567, 0.74998, 0.76838, 0.79162, 0.81317, 0.82802, 0.83598, 0.84002,\
        0.84560, 0.85222, 0.85674, 0.85835, 0.85821, 0.85607, 0.85178, 0.84637, 0.84076, 0.83541, 0.83109,\
        0.82781, 0.82424, 0.81914, 0.81241, 0.80251, 0.78801, 0.77335, 0.75973, 0.74678, 0.73411, 0.72094,\
        0.70595, 0.69143, 0.67608, 0.66142, 0.64824, 0.63599, 0.62352, 0.61046, 0.59761, 0.58384, 0.57150,\
        0.56042, 0.55070, 0.54218, 0.53443, 0.52705, 0.51984, 0.51260, 0.50496, 0.49661, 0.48788, 0.47937,\
        0.47137, 0.46384, 0.45662, 0.44963, 0.44285, 0.43626, 0.42987, 0.42368, 0.41770, 0.41179, 0.40568,\
        0.39941, 0.39349, 0.38800, 0.38273, 0.37732, 0.37153, 0.36524, 0.35848, 0.35133, 0.34390, 0.33629],\
        [0.28612, 0.29067, 0.29534, 0.30028, 0.30586, 0.31179, 0.31787, 0.32402, 0.33019, 0.33635, 0.34250,\
        0.34864, 0.35483, 0.36111, 0.36760, 0.37445, 0.38187, 0.39019, 0.39958, 0.40766, 0.41679, 0.42697,\
        0.43769, 0.44868, 0.45982, 0.47099, 0.48203, 0.49264, 0.50284, 0.51281, 0.52264, 0.53258, 0.54269,\
        0.55302, 0.56365, 0.57467, 0.58646, 0.59974, 0.61381, 0.63130, 0.65395, 0.67872, 0.69632, 0.70305,\
        0.70723, 0.71196, 0.71824, 0.72618, 0.73592, 0.74881, 0.76603, 0.78432, 0.79837, 0.80421, 0.80472,\
        0.81053, 0.81951, 0.82690, 0.83084, 0.83198, 0.83137, 0.82892, 0.82454, 0.81966, 0.81398, 0.80910,\
        0.80623, 0.80323, 0.79839, 0.79192, 0.78273, 0.76945, 0.75523, 0.74149, 0.72820, 0.71544, 0.70267,\
        0.68750, 0.67141, 0.65590, 0.64131, 0.62817, 0.61659, 0.60494, 0.59258, 0.58061, 0.56870, 0.55743,\
        0.54729, 0.53832, 0.53036, 0.52302, 0.51585, 0.50886, 0.50205, 0.49482, 0.48696, 0.47880, 0.47075,\
        0.46305, 0.45572, 0.44871, 0.44197, 0.43544, 0.42908, 0.42286, 0.41683, 0.41115, 0.40565, 0.39988,\
        0.39371, 0.38773, 0.38204, 0.37651, 0.37091, 0.36504, 0.35880, 0.35218, 0.34524, 0.33804, 0.33067],\
        [0.28388, 0.28865, 0.29345, 0.29846, 0.30390, 0.30958, 0.31535, 0.32114, 0.32692, 0.33267, 0.33840,\
        0.34413, 0.34990, 0.35580, 0.36192, 0.36840, 0.37540, 0.38300, 0.39073, 0.39807, 0.40681, 0.41660,\
        0.42695, 0.43762, 0.44850, 0.45944, 0.47024, 0.48073, 0.49087, 0.50067, 0.51030, 0.52008, 0.53003,\
        0.54012, 0.55029, 0.56057, 0.57116, 0.58220, 0.59344, 0.60840, 0.62928, 0.65283, 0.67118, 0.68260,\
        0.69037, 0.69660, 0.70264, 0.70837, 0.71358, 0.71943, 0.72943, 0.74423, 0.75863, 0.76709, 0.77151,\
        0.77842, 0.78846, 0.79721, 0.80301, 0.80515, 0.80607, 0.80541, 0.80114, 0.79691, 0.79154, 0.78616,\
        0.78291, 0.77957, 0.77482, 0.76815, 0.75899, 0.74706, 0.73406, 0.72126, 0.70788, 0.69494, 0.68176,\
        0.66684, 0.65070, 0.63512, 0.62103, 0.60799, 0.59741, 0.58702, 0.57587, 0.56485, 0.55412, 0.54394,\
        0.53465, 0.52642, 0.51911, 0.51236, 0.50541, 0.49836, 0.49179, 0.48489, 0.47756, 0.46999, 0.46243,\
        0.45508, 0.44801, 0.44122, 0.43471, 0.42844, 0.42233, 0.41628, 0.41023, 0.40462, 0.39956, 0.39396,\
        0.38803, 0.38211, 0.37632, 0.37061, 0.36483, 0.35887, 0.35263, 0.34609, 0.33928, 0.33226, 0.32509],\
        [0.28200, 0.28703, 0.29207, 0.29722, 0.30254, 0.30794, 0.31336, 0.31878, 0.32417, 0.32953, 0.33485,\
        0.34017, 0.34555, 0.35105, 0.35677, 0.36279, 0.36920, 0.37594, 0.38280, 0.38993, 0.39799, 0.40720,\
        0.41693, 0.42706, 0.43755, 0.44818, 0.45872, 0.46899, 0.47891, 0.48850, 0.49794, 0.50750, 0.51722,\
        0.52701, 0.53665, 0.54599, 0.55507, 0.56403, 0.57351, 0.58544, 0.60220, 0.62342, 0.63877, 0.65163,\
        0.66293, 0.67134, 0.67762, 0.68220, 0.68505, 0.68695, 0.69197, 0.70404, 0.71864, 0.73054, 0.73980,\
        0.74859, 0.75805, 0.76714, 0.77436, 0.77858, 0.78066, 0.78020, 0.77654, 0.77224, 0.76735, 0.76231,\
        0.75835, 0.75446, 0.74938, 0.74280, 0.73400, 0.72272, 0.71071, 0.69932, 0.68712, 0.67399, 0.66052,\
        0.64576, 0.62981, 0.61429, 0.60145, 0.58959, 0.57920, 0.56968, 0.55990, 0.55004, 0.54034, 0.53103,\
        0.52246, 0.51489, 0.50816, 0.50203, 0.49565, 0.48873, 0.48211, 0.47544, 0.46854, 0.46147, 0.45439,\
        0.44741, 0.44062, 0.43407, 0.42778, 0.42177, 0.41599, 0.41027, 0.40429, 0.39831, 0.39334, 0.38801,\
        0.38234, 0.37654, 0.37074, 0.36494, 0.35905, 0.35301, 0.34674, 0.34023, 0.33351, 0.32662, 0.31961],\
        [0.28038, 0.28569, 0.29096, 0.29626, 0.30153, 0.30664, 0.31175, 0.31685, 0.32188, 0.32685, 0.33177,\
        0.33670, 0.34168, 0.34679, 0.35208, 0.35763, 0.36347, 0.36958, 0.37595, 0.38276, 0.39034, 0.39877,\
        0.40763, 0.41713, 0.42718, 0.43748, 0.44771, 0.45770, 0.46736, 0.47674, 0.48598, 0.49524, 0.50467,\
        0.51414, 0.52331, 0.53178, 0.53959, 0.54725, 0.55569, 0.56634, 0.58094, 0.59693, 0.60529, 0.61883,\
        0.63259, 0.64158, 0.64756, 0.65170, 0.65468, 0.65743, 0.66242, 0.67210, 0.68495, 0.69815, 0.70999,\
        0.71970, 0.72841, 0.73757, 0.74641, 0.75260, 0.75535, 0.75430, 0.75056, 0.74669, 0.74238, 0.73712,\
        0.73304, 0.72913, 0.72329, 0.71702, 0.70911, 0.69823, 0.68708, 0.67648, 0.66526, 0.65297, 0.63985,\
        0.62578, 0.61039, 0.59524, 0.58292, 0.57232, 0.56267, 0.55379, 0.54506, 0.53626, 0.52749, 0.51890,\
        0.51076, 0.50379, 0.49755, 0.49165, 0.48566, 0.47930, 0.47290, 0.46648, 0.45994, 0.45331, 0.44665,\
        0.44003, 0.43354, 0.42720, 0.42109, 0.41531, 0.40988, 0.40462, 0.39921, 0.39323, 0.38772, 0.38232,\
        0.37677, 0.37108, 0.36531, 0.35948, 0.35354, 0.34744, 0.34114, 0.33464, 0.32797, 0.32116, 0.31428],\
        [0.27891, 0.28447, 0.28992, 0.29526, 0.30042, 0.30545, 0.31041, 0.31524, 0.31994, 0.32454, 0.32908,\
        0.33363, 0.33823, 0.34294, 0.34783, 0.35294, 0.35831, 0.36394, 0.36990, 0.37632, 0.38333, 0.39093,\
        0.39904, 0.40796, 0.41761, 0.42756, 0.43743, 0.44704, 0.45638, 0.46550, 0.47451, 0.48354, 0.49265,\
        0.50179, 0.51063, 0.51832, 0.52504, 0.53175, 0.53933, 0.54867, 0.55996, 0.57064, 0.57923, 0.59204,\
        0.60473, 0.61192, 0.61741, 0.62168, 0.62640, 0.63139, 0.63734, 0.64601, 0.65743, 0.67010, 0.68215,\
        0.69217, 0.70007, 0.70969, 0.72014, 0.72763, 0.73103, 0.72921, 0.72380, 0.72106, 0.71830, 0.71148,\
        0.70756, 0.70495, 0.69757, 0.69140, 0.68410, 0.67444, 0.66415, 0.65403, 0.64352, 0.63217, 0.61975,\
        0.60670, 0.59304, 0.57873, 0.56662, 0.55638, 0.54740, 0.53928, 0.53150, 0.52362, 0.51565, 0.50785,\
        0.50000, 0.49335, 0.48745, 0.48174, 0.47600, 0.47008, 0.46404, 0.45793, 0.45175, 0.44551, 0.43923,\
        0.43297, 0.42677, 0.42065, 0.41464, 0.40893, 0.40379, 0.39876, 0.39356, 0.38798, 0.38241, 0.37692,\
        0.37139, 0.36577, 0.36006, 0.35424, 0.34828, 0.34216, 0.33585, 0.32935, 0.32269, 0.31594, 0.30916],\
        [0.27747, 0.28330, 0.28893, 0.29429, 0.29939, 0.30444, 0.30930, 0.31389, 0.31827, 0.32251, 0.32668,\
        0.33086, 0.33510, 0.33946, 0.34398, 0.34870, 0.35367, 0.35892, 0.36451, 0.37052, 0.37703, 0.38404,\
        0.39159, 0.39985, 0.40907, 0.41862, 0.42800, 0.43710, 0.44598, 0.45475, 0.46348, 0.47224, 0.48107,\
        0.48989, 0.49843, 0.50529, 0.51107, 0.51705, 0.52376, 0.53155, 0.54030, 0.54914, 0.55822, 0.56883,\
        0.57859, 0.58538, 0.59067, 0.59534, 0.60210, 0.60887, 0.61525, 0.62337, 0.63376, 0.64550, 0.65695,\
        0.66675, 0.67527, 0.68505, 0.69521, 0.70315, 0.70734, 0.70584, 0.69791, 0.69604, 0.69511, 0.68775,\
        0.68301, 0.67981, 0.67364, 0.66693, 0.65949, 0.65076, 0.64145, 0.63212, 0.62237, 0.61194, 0.60040,\
        0.58787, 0.57532, 0.56270, 0.55134, 0.54158, 0.53317, 0.52592, 0.51922, 0.51215, 0.50472, 0.49754,\
        0.49054, 0.48394, 0.47805, 0.47244, 0.46688, 0.46125, 0.45554, 0.44977, 0.44394, 0.43806, 0.43215,\
        0.42622, 0.42032, 0.41448, 0.40862, 0.40270, 0.39773, 0.39289, 0.38782, 0.38252, 0.37711, 0.37168,\
        0.36620, 0.36065, 0.35499, 0.34921, 0.34329, 0.33719, 0.33090, 0.32440, 0.31773, 0.31099, 0.30430],\
        [0.27597, 0.28210, 0.28800, 0.29357, 0.29877, 0.30377, 0.30844, 0.31273, 0.31675, 0.32063, 0.32446,\
        0.32831, 0.33223, 0.33626, 0.34046, 0.34486, 0.34949, 0.35442, 0.35968, 0.36533, 0.37145, 0.37806,\
        0.38523, 0.39311, 0.40178, 0.41070, 0.41937, 0.42780, 0.43611, 0.44443, 0.45279, 0.46120, 0.46962,\
        0.47789, 0.48559, 0.49198, 0.49741, 0.50300, 0.50898, 0.51554, 0.52283, 0.53072, 0.53916, 0.54799,\
        0.55620, 0.56299, 0.56879, 0.57464, 0.58165, 0.58889, 0.59555, 0.60355, 0.61320, 0.62436, 0.63537,\
        0.64449, 0.65280, 0.66157, 0.67036, 0.67753, 0.68154, 0.68047, 0.67490, 0.67149, 0.66883, 0.66371,\
        0.65889, 0.65490, 0.64990, 0.64358, 0.63617, 0.62790, 0.61931, 0.61090, 0.60215, 0.59212, 0.58152,\
        0.56987, 0.55814, 0.54683, 0.53657, 0.52761, 0.51973, 0.51330, 0.50812, 0.50169, 0.49472, 0.48789,\
        0.48133, 0.47508, 0.46926, 0.46372, 0.45829, 0.45288, 0.44744, 0.44197, 0.43647, 0.43094, 0.42539,\
        0.41977, 0.41413, 0.40862, 0.40328, 0.39750, 0.39218, 0.38725, 0.38226, 0.37712, 0.37186, 0.36654,\
        0.36115, 0.35568, 0.35011, 0.34442, 0.33856, 0.33253, 0.32629, 0.31984, 0.31318, 0.30641, 0.29979],\
        [0.27428, 0.28074, 0.28700, 0.29296, 0.29842, 0.30337, 0.30769, 0.31156, 0.31521, 0.31876, 0.32229,\
        0.32586, 0.32951, 0.33328, 0.33721, 0.34133, 0.34569, 0.35033, 0.35530, 0.36066, 0.36646, 0.37276,\
        0.37963, 0.38716, 0.39527, 0.40349, 0.41134, 0.41896, 0.42664, 0.43448, 0.44242, 0.45040, 0.45829,\
        0.46589, 0.47287, 0.47896, 0.48438, 0.48969, 0.49516, 0.50093, 0.50734, 0.51461, 0.52237, 0.52998,\
        0.53746, 0.54437, 0.55073, 0.55716, 0.56411, 0.57132, 0.57863, 0.58676, 0.59588, 0.60729, 0.61758,\
        0.62457, 0.63145, 0.63865, 0.64566, 0.65147, 0.65450, 0.65352, 0.64989, 0.64653, 0.64326, 0.63893,\
        0.63455, 0.63119, 0.62730, 0.62156, 0.61441, 0.60650, 0.59855, 0.59079, 0.58279, 0.57347, 0.56333,\
        0.55256, 0.54174, 0.53148, 0.52233, 0.51456, 0.50739, 0.50089, 0.49744, 0.49168, 0.48517, 0.47873,\
        0.47251, 0.46656, 0.46090, 0.45547, 0.45018, 0.44494, 0.43973, 0.43451, 0.42931, 0.42411, 0.41891,\
        0.41362, 0.40815, 0.40273, 0.39771, 0.39240, 0.38703, 0.38193, 0.37691, 0.37185, 0.36671, 0.36151,\
        0.35623, 0.35088, 0.34542, 0.33983, 0.33409, 0.32817, 0.32205, 0.31571, 0.30914, 0.30240, 0.29580],\
        [0.27225, 0.27900, 0.28560, 0.29198, 0.29799, 0.30277, 0.30659, 0.31008, 0.31343, 0.31674, 0.32006,\
        0.32343, 0.32687, 0.33043, 0.33414, 0.33804, 0.34217, 0.34657, 0.35129, 0.35638, 0.36189, 0.36788,\
        0.37439, 0.38143, 0.38889, 0.39644, 0.40348, 0.41038, 0.41750, 0.42489, 0.43242, 0.43995, 0.44732,\
        0.45434, 0.46082, 0.46668, 0.47205, 0.47723, 0.48247, 0.48796, 0.49395, 0.50093, 0.50818, 0.51450,\
        0.52188, 0.52892, 0.53549, 0.54209, 0.54900, 0.55624, 0.56393, 0.57249, 0.58240, 0.59409, 0.60171,\
        0.60572, 0.61126, 0.61678, 0.62197, 0.62662, 0.62908, 0.62773, 0.62447, 0.62209, 0.61934, 0.61533,\
        0.61084, 0.60896, 0.60642, 0.60115, 0.59449, 0.58727, 0.57998, 0.57265, 0.56485, 0.55612, 0.54651,\
        0.53639, 0.52631, 0.51695, 0.50871, 0.50219, 0.49654, 0.49059, 0.48624, 0.48136, 0.47568, 0.46980,\
        0.46400, 0.45836, 0.45291, 0.44763, 0.44248, 0.43740, 0.43237, 0.42737, 0.42241, 0.41750, 0.41264,\
        0.40776, 0.40255, 0.39711, 0.39204, 0.38696, 0.38180, 0.37672, 0.37171, 0.36670, 0.36167, 0.35658,\
        0.35144, 0.34621, 0.34089, 0.33544, 0.32985, 0.32408, 0.31811, 0.31196, 0.30561, 0.29905, 0.29245],\
        [0.26975, 0.27668, 0.28345, 0.28996, 0.29591, 0.30078, 0.30460, 0.30797, 0.31124, 0.31446, 0.31767,\
        0.32092, 0.32423, 0.32764, 0.33119, 0.33492, 0.33886, 0.34305, 0.34754, 0.35238, 0.35760, 0.36324,\
        0.36932, 0.37580, 0.38254, 0.38926, 0.39569, 0.40206, 0.40874, 0.41577, 0.42292, 0.43003, 0.43694,\
        0.44349, 0.44959, 0.45524, 0.46056, 0.46575, 0.47099, 0.47650, 0.48255, 0.48934, 0.49572, 0.50136,\
        0.50944, 0.51599, 0.52235, 0.52890, 0.53572, 0.54287, 0.55060, 0.55920, 0.56878, 0.57832, 0.58465,\
        0.58848, 0.59283, 0.59675, 0.60036, 0.60423, 0.60690, 0.60475, 0.60073, 0.59977, 0.59774, 0.59425,\
        0.59027, 0.58860, 0.58653, 0.58205, 0.57627, 0.56989, 0.56324, 0.55629, 0.54878, 0.54045, 0.53132,\
        0.52171, 0.51219, 0.50371, 0.49642, 0.49052, 0.48553, 0.48060, 0.47606, 0.47144, 0.46641, 0.46112,\
        0.45577, 0.45046, 0.44524, 0.44014, 0.43514, 0.43022, 0.42534, 0.42051, 0.41573, 0.41102, 0.40639,\
        0.40189, 0.39708, 0.39178, 0.38662, 0.38157, 0.37653, 0.37153, 0.36658, 0.36165, 0.35672, 0.35175,\
        0.34674, 0.34166, 0.33650, 0.33121, 0.32579, 0.32019, 0.31439, 0.30842, 0.30231, 0.29591, 0.28934],\
        [0.26670, 0.27369, 0.28047, 0.28690, 0.29273, 0.29767, 0.30156, 0.30520, 0.30860, 0.31186, 0.31508,\
        0.31829, 0.32154, 0.32487, 0.32831, 0.33191, 0.33569, 0.33970, 0.34398, 0.34856, 0.35349, 0.35877,\
        0.36440, 0.37032, 0.37640, 0.38247, 0.38843, 0.39439, 0.40067, 0.40731, 0.41407, 0.42079, 0.42728,\
        0.43342, 0.43919, 0.44466, 0.44995, 0.45518, 0.46051, 0.46610, 0.47211, 0.47850, 0.48489, 0.49190,\
        0.49860, 0.50451, 0.51055, 0.51706, 0.52370, 0.53049, 0.53782, 0.54587, 0.55434, 0.56221, 0.56816,\
        0.57243, 0.57609, 0.57930, 0.58223, 0.58519, 0.58692, 0.58511, 0.58211, 0.58039, 0.57849, 0.57561,\
        0.57261, 0.57034, 0.56789, 0.56419, 0.55937, 0.55387, 0.54791, 0.54150, 0.53445, 0.52655, 0.51789,\
        0.50884, 0.49991, 0.49211, 0.48564, 0.48016, 0.47540, 0.47092, 0.46659, 0.46221, 0.45762, 0.45281,\
        0.44787, 0.44289, 0.43791, 0.43299, 0.42815, 0.42339, 0.41864, 0.41393, 0.40928, 0.40470, 0.40017,\
        0.39575, 0.39116, 0.38621, 0.38118, 0.37620, 0.37126, 0.36636, 0.36149, 0.35666, 0.35183, 0.34699,\
        0.34213, 0.33720, 0.33220, 0.32710, 0.32186, 0.31645, 0.31082, 0.30495, 0.29891, 0.29270, 0.28626],\
        [0.26313, 0.27009, 0.27681, 0.28313, 0.28887, 0.29384, 0.29811, 0.30209, 0.30556, 0.30898, 0.31228,\
        0.31552, 0.31877, 0.32207, 0.32545, 0.32895, 0.33261, 0.33646, 0.34054, 0.34488, 0.34952, 0.35444,\
        0.35965, 0.36508, 0.37063, 0.37623, 0.38183, 0.38751, 0.39343, 0.39961, 0.40598, 0.41232, 0.41844,\
        0.42418, 0.42960, 0.43488, 0.44014, 0.44543, 0.45082, 0.45642, 0.46232, 0.46851, 0.47492, 0.48152,\
        0.48780, 0.49363, 0.49961, 0.50620, 0.51246, 0.51856, 0.52524, 0.53256, 0.54009, 0.54708, 0.55285,\
        0.55734, 0.56092, 0.56387, 0.56638, 0.56840, 0.56916, 0.56800, 0.56586, 0.56381, 0.56167, 0.55910,\
        0.55638, 0.55380, 0.55109, 0.54778, 0.54374, 0.53902, 0.53378, 0.52804, 0.52161, 0.51423, 0.50607,\
        0.49769, 0.48955, 0.48229, 0.47613, 0.47086, 0.46622, 0.46194, 0.45780, 0.45366, 0.44939, 0.44497,\
        0.44039, 0.43569, 0.43093, 0.42617, 0.42149, 0.41691, 0.41231, 0.40764, 0.40310, 0.39862, 0.39418,\
        0.38976, 0.38524, 0.38051, 0.37567, 0.37082, 0.36598, 0.36118, 0.35642, 0.35169, 0.34699, 0.34228,\
        0.33757, 0.33281, 0.32798, 0.32306, 0.31802, 0.31283, 0.30739, 0.30163, 0.29567, 0.28953, 0.28319],\
        [0.25910, 0.26601, 0.27265, 0.27890, 0.28464, 0.28979, 0.29441, 0.29846, 0.30221, 0.30590, 0.30930,\
        0.31261, 0.31591, 0.31922, 0.32258, 0.32602, 0.32958, 0.33329, 0.33718, 0.34130, 0.34566, 0.35026,\
        0.35509, 0.36010, 0.36525, 0.37048, 0.37579, 0.38122, 0.38685, 0.39268, 0.39866, 0.40466, 0.41044,\
        0.41571, 0.42070, 0.42581, 0.43107, 0.43639, 0.44179, 0.44733, 0.45307, 0.45904, 0.46518, 0.47137,\
        0.47740, 0.48328, 0.48934, 0.49571, 0.50136, 0.50671, 0.51283, 0.51955, 0.52648, 0.53305, 0.53875,\
        0.54340, 0.54706, 0.54989, 0.55205, 0.55346, 0.55381, 0.55292, 0.55121, 0.54916, 0.54688, 0.54432,\
        0.54153, 0.53867, 0.53579, 0.53277, 0.52932, 0.52523, 0.52059, 0.51553, 0.50992, 0.50318, 0.49547,\
        0.48771, 0.48027, 0.47350, 0.46759, 0.46244, 0.45787, 0.45367, 0.44968, 0.44574, 0.44174, 0.43763,\
        0.43336, 0.42893, 0.42434, 0.41967, 0.41509, 0.41078, 0.40643, 0.40171, 0.39719, 0.39279, 0.38838,\
        0.38396, 0.37946, 0.37485, 0.37016, 0.36543, 0.36071, 0.35602, 0.35136, 0.34675, 0.34217, 0.33760,\
        0.33305, 0.32846, 0.32381, 0.31906, 0.31420, 0.30925, 0.30409, 0.29846, 0.29255, 0.28644, 0.28015],\
        [0.25470, 0.26154, 0.26813, 0.27437, 0.28017, 0.28549, 0.29032, 0.29472, 0.29893, 0.30263, 0.30610,\
        0.30954, 0.31294, 0.31632, 0.31969, 0.32310, 0.32657, 0.33015, 0.33388, 0.33779, 0.34189, 0.34620,\
        0.35071, 0.35539, 0.36021, 0.36514, 0.37019, 0.37539, 0.38077, 0.38633, 0.39201, 0.39772, 0.40325,\
        0.40790, 0.41230, 0.41737, 0.42271, 0.42803, 0.43334, 0.43873, 0.44427, 0.44996, 0.45577, 0.46162,\
        0.46743, 0.47319, 0.47902, 0.48478, 0.49004, 0.49517, 0.50085, 0.50711, 0.51368, 0.52005, 0.52579,\
        0.53059, 0.53430, 0.53703, 0.53895, 0.54005, 0.54021, 0.53943, 0.53790, 0.53591, 0.53360, 0.53095,\
        0.52796, 0.52473, 0.52161, 0.51892, 0.51605, 0.51235, 0.50809, 0.50353, 0.49863, 0.49259, 0.48557,\
        0.47845, 0.47162, 0.46532, 0.45967, 0.45465, 0.45015, 0.44603, 0.44214, 0.43837, 0.43461, 0.43077,\
        0.42679, 0.42262, 0.41821, 0.41356, 0.40888, 0.40481, 0.40110, 0.39635, 0.39169, 0.38720, 0.38276,\
        0.37830, 0.37382, 0.36927, 0.36468, 0.36006, 0.35545, 0.35086, 0.34632, 0.34181, 0.33735, 0.33294,\
        0.32854, 0.32414, 0.31967, 0.31506, 0.31031, 0.30548, 0.30068, 0.29524, 0.28937, 0.28331, 0.27708],\
        [0.25001, 0.25680, 0.26336, 0.26962, 0.27551, 0.28099, 0.28606, 0.29075, 0.29509, 0.29899, 0.30271,\
        0.30637, 0.30991, 0.31337, 0.31678, 0.32017, 0.32357, 0.32704, 0.33061, 0.33432, 0.33820, 0.34226,\
        0.34649, 0.35089, 0.35543, 0.36011, 0.36493, 0.36991, 0.37505, 0.38034, 0.38576, 0.39121, 0.39647,\
        0.40028, 0.40433, 0.40973, 0.41512, 0.42031, 0.42541, 0.43057, 0.43584, 0.44123, 0.44672, 0.45225,\
        0.45778, 0.46329, 0.46876, 0.47408, 0.47917, 0.48424, 0.48964, 0.49553, 0.50174, 0.50797, 0.51381,\
        0.51874, 0.52242, 0.52500, 0.52676, 0.52774, 0.52785, 0.52709, 0.52563, 0.52369, 0.52140, 0.51872,\
        0.51556, 0.51194, 0.50828, 0.50599, 0.50386, 0.50022, 0.49623, 0.49198, 0.48736, 0.48197, 0.47587,\
        0.46953, 0.46334, 0.45752, 0.45218, 0.44733, 0.44292, 0.43887, 0.43508, 0.43146, 0.42791, 0.42432,\
        0.42063, 0.41676, 0.41259, 0.40803, 0.40313, 0.39877, 0.39528, 0.39095, 0.38633, 0.38177, 0.37726,\
        0.37278, 0.36829, 0.36378, 0.35926, 0.35473, 0.35021, 0.34573, 0.34128, 0.33689, 0.33255, 0.32826,\
        0.32403, 0.31982, 0.31557, 0.31114, 0.30644, 0.30153, 0.29674, 0.29162, 0.28601, 0.28009, 0.27396],\
        [0.24510, 0.25185, 0.25841, 0.26471, 0.27071, 0.27635, 0.28163, 0.28655, 0.29110, 0.29530, 0.29932,\
        0.30315, 0.30682, 0.31041, 0.31386, 0.31723, 0.32057, 0.32392, 0.32735, 0.33088, 0.33456, 0.33840,\
        0.34239, 0.34655, 0.35086, 0.35531, 0.35991, 0.36466, 0.36955, 0.37458, 0.37968, 0.38471, 0.38936,\
        0.39333, 0.39782, 0.40308, 0.40830, 0.41312, 0.41788, 0.42275, 0.42774, 0.43282, 0.43799, 0.44320,\
        0.44844, 0.45366, 0.45881, 0.46384, 0.46879, 0.47378, 0.47904, 0.48467, 0.49060, 0.49665, 0.50252,\
        0.50762, 0.51109, 0.51343, 0.51513, 0.51615, 0.51628, 0.51552, 0.51405, 0.51212, 0.50990, 0.50728,\
        0.50408, 0.50035, 0.49631, 0.49391, 0.49186, 0.48862, 0.48489, 0.48090, 0.47657, 0.47172, 0.46640,\
        0.46083, 0.45530, 0.44999, 0.44499, 0.44034, 0.43604, 0.43206, 0.42837, 0.42489, 0.42153, 0.41818,\
        0.41474, 0.41117, 0.40739, 0.40307, 0.39831, 0.39371, 0.38964, 0.38539, 0.38091, 0.37637, 0.37184,\
        0.36734, 0.36285, 0.35837, 0.35390, 0.34944, 0.34501, 0.34062, 0.33628, 0.33198, 0.32774, 0.32357,\
        0.31948, 0.31548, 0.31151, 0.30737, 0.30280, 0.29796, 0.29308, 0.28800, 0.28256, 0.27679, 0.27077],\
        [0.24004, 0.24677, 0.25334, 0.25971, 0.26582, 0.27163, 0.27712, 0.28227, 0.28708, 0.29159, 0.29585,\
        0.29982, 0.30374, 0.30745, 0.31092, 0.31427, 0.31754, 0.32078, 0.32406, 0.32744, 0.33094, 0.33458,\
        0.33838, 0.34233, 0.34643, 0.35067, 0.35506, 0.35958, 0.36423, 0.36898, 0.37375, 0.37844, 0.38291,\
        0.38723, 0.39188, 0.39699, 0.40186, 0.40613, 0.41058, 0.41521, 0.41993, 0.42470, 0.42955, 0.43447,\
        0.43942, 0.44436, 0.44924, 0.45404, 0.45881, 0.46368, 0.46878, 0.47419, 0.47989, 0.48573, 0.49148,\
        0.49646, 0.49972, 0.50192, 0.50374, 0.50491, 0.50511, 0.50433, 0.50283, 0.50087, 0.49868, 0.49611,\
        0.49290, 0.48927, 0.48564, 0.48276, 0.48023, 0.47728, 0.47390, 0.47022, 0.46624, 0.46191, 0.45725,\
        0.45239, 0.44749, 0.44268, 0.43804, 0.43361, 0.42942, 0.42550, 0.42189, 0.41854, 0.41538, 0.41223,\
        0.40895, 0.40557, 0.40221, 0.39816, 0.39346, 0.38880, 0.38436, 0.37997, 0.37550, 0.37098, 0.36646,\
        0.36196, 0.35748, 0.35303, 0.34861, 0.34422, 0.33987, 0.33557, 0.33132, 0.32711, 0.32296, 0.31887,\
        0.31488, 0.31103, 0.30736, 0.30366, 0.29937, 0.29454, 0.28959, 0.28448, 0.27910, 0.27345, 0.26753],\
        [0.23489, 0.24161, 0.24822, 0.25467, 0.26090, 0.26688, 0.27258, 0.27796, 0.28304, 0.28783, 0.29235,\
        0.29665, 0.30081, 0.30446, 0.30794, 0.31129, 0.31448, 0.31760, 0.32074, 0.32396, 0.32731, 0.33079,\
        0.33442, 0.33820, 0.34211, 0.34616, 0.35034, 0.35464, 0.35905, 0.36353, 0.36804, 0.37250, 0.37689,\
        0.38130, 0.38588, 0.39062, 0.39505, 0.39914, 0.40347, 0.40793, 0.41238, 0.41684, 0.42139, 0.42603,\
        0.43074, 0.43543, 0.44005, 0.44461, 0.44917, 0.45384, 0.45872, 0.46387, 0.46928, 0.47480, 0.48012,\
        0.48469, 0.48808, 0.49050, 0.49238, 0.49360, 0.49387, 0.49316, 0.49172, 0.48981, 0.48756, 0.48493,\
        0.48183, 0.47843, 0.47507, 0.47204, 0.46925, 0.46637, 0.46326, 0.45991, 0.45633, 0.45250, 0.44843,\
        0.44420, 0.43990, 0.43559, 0.43132, 0.42712, 0.42303, 0.41913, 0.41555, 0.41232, 0.40933, 0.40641,\
        0.40323, 0.39980, 0.39644, 0.39264, 0.38821, 0.38363, 0.37911, 0.37462, 0.37012, 0.36561, 0.36111,\
        0.35662, 0.35217, 0.34775, 0.34338, 0.33905, 0.33478, 0.33057, 0.32642, 0.32232, 0.31824, 0.31421,\
        0.31025, 0.30645, 0.30291, 0.29955, 0.29555, 0.29092, 0.28604, 0.28096, 0.27566, 0.27009, 0.26427],\
        [0.22971, 0.23643, 0.24309, 0.24963, 0.25600, 0.26215, 0.26804, 0.27365, 0.27897, 0.28402, 0.28881,\
        0.29335, 0.29756, 0.30133, 0.30490, 0.30829, 0.31137, 0.31433, 0.31732, 0.32042, 0.32364, 0.32700,\
        0.33049, 0.33411, 0.33786, 0.34173, 0.34572, 0.34981, 0.35399, 0.35824, 0.36252, 0.36680, 0.37107,\
        0.37539, 0.37977, 0.38415, 0.38836, 0.39244, 0.39662, 0.40087, 0.40505, 0.40920, 0.41346, 0.41789,\
        0.42240, 0.42687, 0.43123, 0.43551, 0.43980, 0.44418, 0.44876, 0.45359, 0.45865, 0.46378, 0.46870,\
        0.47301, 0.47646, 0.47907, 0.48100, 0.48218, 0.48250, 0.48194, 0.48066, 0.47883, 0.47659, 0.47396,\
        0.47097, 0.46777, 0.46457, 0.46154, 0.45869, 0.45588, 0.45299, 0.44998, 0.44680, 0.44344, 0.43992,\
        0.43627, 0.43253, 0.42873, 0.42485, 0.42088, 0.41687, 0.41292, 0.40930, 0.40616, 0.40330, 0.40055,\
        0.39750, 0.39409, 0.39059, 0.38681, 0.38262, 0.37821, 0.37374, 0.36925, 0.36476, 0.36027, 0.35579,\
        0.35133, 0.34691, 0.34253, 0.33821, 0.33395, 0.32977, 0.32566, 0.32161, 0.31763, 0.31366, 0.30968,\
        0.30573, 0.30194, 0.29839, 0.29507, 0.29132, 0.28703, 0.28235, 0.27739, 0.27218, 0.26671, 0.26098],\
        [0.22457, 0.23130, 0.23802, 0.24467, 0.25117, 0.25748, 0.26354, 0.26935, 0.27488, 0.28014, 0.28514,\
        0.28986, 0.29424, 0.29818, 0.30210, 0.30531, 0.30811, 0.31090, 0.31378, 0.31678, 0.31992, 0.32318,\
        0.32656, 0.33005, 0.33365, 0.33736, 0.34117, 0.34507, 0.34905, 0.35308, 0.35715, 0.36125, 0.36537,\
        0.36952, 0.37369, 0.37783, 0.38188, 0.38588, 0.38990, 0.39392, 0.39786, 0.40177, 0.40577, 0.41006,\
        0.41443, 0.41869, 0.42275, 0.42671, 0.43066, 0.43469, 0.43890, 0.44335, 0.44805, 0.45283, 0.45745,\
        0.46158, 0.46503, 0.46773, 0.46969, 0.47089, 0.47126, 0.47084, 0.46970, 0.46799, 0.46582, 0.46324,\
        0.46036, 0.45729, 0.45420, 0.45124, 0.44843, 0.44574, 0.44309, 0.44040, 0.43762, 0.43471, 0.43168,\
        0.42856, 0.42537, 0.42207, 0.41861, 0.41493, 0.41101, 0.40695, 0.40317, 0.40011, 0.39731, 0.39453,\
        0.39154, 0.38823, 0.38471, 0.38095, 0.37690, 0.37265, 0.36827, 0.36384, 0.35939, 0.35494, 0.35049,\
        0.34608, 0.34170, 0.33738, 0.33312, 0.32893, 0.32483, 0.32082, 0.31690, 0.31307, 0.30927, 0.30542,\
        0.30147, 0.29776, 0.29427, 0.29084, 0.28716, 0.28306, 0.27859, 0.27378, 0.26869, 0.26332, 0.25768],\
        [0.21953, 0.22628, 0.23307, 0.23983, 0.24647, 0.25291, 0.25912, 0.26508, 0.27077, 0.27620, 0.28135,\
        0.28624, 0.29084, 0.29524, 0.29924, 0.30193, 0.30455, 0.30724, 0.31007, 0.31305, 0.31615, 0.31934,\
        0.32263, 0.32601, 0.32948, 0.33305, 0.33669, 0.34041, 0.34419, 0.34803, 0.35191, 0.35582, 0.35977,\
        0.36373, 0.36771, 0.37166, 0.37557, 0.37944, 0.38329, 0.38712, 0.39088, 0.39461, 0.39845, 0.40261,\
        0.40687, 0.41090, 0.41459, 0.41818, 0.42177, 0.42538, 0.42914, 0.43319, 0.43755, 0.44207, 0.44649,\
        0.45051, 0.45393, 0.45665, 0.45864, 0.45986, 0.46029, 0.45998, 0.45897, 0.45738, 0.45530, 0.45281,\
        0.45001, 0.44704, 0.44403, 0.44114, 0.43843, 0.43590, 0.43351, 0.43115, 0.42876, 0.42626, 0.42366,\
        0.42101, 0.41835, 0.41557, 0.41255, 0.40919, 0.40549, 0.40151, 0.39749, 0.39434, 0.39147, 0.38859,\
        0.38557, 0.38230, 0.37881, 0.37510, 0.37115, 0.36702, 0.36275, 0.35841, 0.35402, 0.34963, 0.34524,\
        0.34088, 0.33656, 0.33230, 0.32810, 0.32399, 0.31997, 0.31607, 0.31229, 0.30861, 0.30503, 0.30145,\
        0.29764, 0.29393, 0.29041, 0.28689, 0.28320, 0.27919, 0.27485, 0.27018, 0.26521, 0.25994, 0.25438],\
        [0.21467, 0.22145, 0.22834, 0.23521, 0.24196, 0.24851, 0.25482, 0.26088, 0.26667, 0.27218, 0.27743,\
        0.28240, 0.28707, 0.29140, 0.29508, 0.29799, 0.30067, 0.30337, 0.30623, 0.30925, 0.31235, 0.31549,\
        0.31869, 0.32198, 0.32534, 0.32878, 0.33227, 0.33582, 0.33941, 0.34306, 0.34675, 0.35049, 0.35426,\
        0.35805, 0.36185, 0.36565, 0.36941, 0.37315, 0.37686, 0.38053, 0.38418, 0.38783, 0.39161, 0.39558,\
        0.39970, 0.40342, 0.40661, 0.40989, 0.41314, 0.41631, 0.41956, 0.42317, 0.42725, 0.43163, 0.43595,\
        0.43990, 0.44327, 0.44596, 0.44795, 0.44920, 0.44969, 0.44945, 0.44856, 0.44708, 0.44511, 0.44271,\
        0.43999, 0.43707, 0.43412, 0.43128, 0.42868, 0.42635, 0.42424, 0.42224, 0.42023, 0.41810, 0.41584,\
        0.41355, 0.41137, 0.40912, 0.40654, 0.40349, 0.39997, 0.39619, 0.39233, 0.38891, 0.38581, 0.38278,\
        0.37968, 0.37641, 0.37294, 0.36928, 0.36542, 0.36139, 0.35723, 0.35298, 0.34867, 0.34435, 0.34003,\
        0.33574, 0.33150, 0.32731, 0.32319, 0.31915, 0.31522, 0.31142, 0.30777, 0.30423, 0.30078, 0.29743,\
        0.29385, 0.29025, 0.28671, 0.28314, 0.27943, 0.27547, 0.27121, 0.26663, 0.26175, 0.25657, 0.25109],\
        [0.21009, 0.21694, 0.22394, 0.23090, 0.23772, 0.24432, 0.25068, 0.25678, 0.26259, 0.26813, 0.27338,\
        0.27833, 0.28295, 0.28714, 0.29077, 0.29386, 0.29666, 0.29940, 0.30235, 0.30544, 0.30855, 0.31163,\
        0.31475, 0.31795, 0.32123, 0.32455, 0.32790, 0.33128, 0.33470, 0.33816, 0.34167, 0.34523, 0.34884,\
        0.35247, 0.35612, 0.35977, 0.36341, 0.36702, 0.37061, 0.37417, 0.37773, 0.38131, 0.38497, 0.38875,\
        0.39252, 0.39585, 0.39878, 0.40189, 0.40485, 0.40755, 0.41022, 0.41339, 0.41730, 0.42167, 0.42598,\
        0.42985, 0.43312, 0.43575, 0.43772, 0.43898, 0.43951, 0.43933, 0.43852, 0.43716, 0.43530, 0.43301,\
        0.43036, 0.42747, 0.42452, 0.42172, 0.41921, 0.41711, 0.41532, 0.41365, 0.41203, 0.41026, 0.40822,\
        0.40610, 0.40437, 0.40257, 0.40033, 0.39751, 0.39416, 0.39057, 0.38693, 0.38347, 0.38023, 0.37707,\
        0.37388, 0.37058, 0.36713, 0.36351, 0.35973, 0.35579, 0.35174, 0.34758, 0.34337, 0.33913, 0.33489,\
        0.33068, 0.32653, 0.32243, 0.31841, 0.31445, 0.31060, 0.30690, 0.30340, 0.30000, 0.29669, 0.29341,\
        0.29005, 0.28659, 0.28310, 0.27953, 0.27581, 0.27187, 0.26766, 0.26316, 0.25835, 0.25324, 0.24782],\
        [0.20595, 0.21291, 0.22000, 0.22700, 0.23382, 0.24041, 0.24674, 0.25280, 0.25858, 0.26408, 0.26927,\
        0.27415, 0.27866, 0.28277, 0.28642, 0.28965, 0.29263, 0.29554, 0.29856, 0.30168, 0.30476, 0.30777,\
        0.31079, 0.31393, 0.31715, 0.32037, 0.32358, 0.32680, 0.33003, 0.33332, 0.33666, 0.34005, 0.34350,\
        0.34699, 0.35050, 0.35403, 0.35755, 0.36106, 0.36454, 0.36801, 0.37146, 0.37493, 0.37843, 0.38194,\
        0.38533, 0.38843, 0.39130, 0.39414, 0.39680, 0.39913, 0.40131, 0.40408, 0.40796, 0.41244, 0.41672,\
        0.42045, 0.42353, 0.42604, 0.42798, 0.42925, 0.42979, 0.42964, 0.42889, 0.42764, 0.42592, 0.42376,\
        0.42121, 0.41833, 0.41535, 0.41256, 0.41011, 0.40823, 0.40682, 0.40539, 0.40413, 0.40273, 0.40089,\
        0.39881, 0.39732, 0.39576, 0.39368, 0.39109, 0.38802, 0.38469, 0.38126, 0.37789, 0.37461, 0.37138,\
        0.36814, 0.36483, 0.36139, 0.35782, 0.35411, 0.35027, 0.34631, 0.34226, 0.33814, 0.33399, 0.32984,\
        0.32572, 0.32166, 0.31769, 0.31378, 0.30995, 0.30616, 0.30253, 0.29920, 0.29596, 0.29278, 0.28959,\
        0.28634, 0.28301, 0.27958, 0.27603, 0.27232, 0.26840, 0.26423, 0.25977, 0.25501, 0.24995, 0.24458],\
        [0.20254, 0.20963, 0.21668, 0.22359, 0.23030, 0.23679, 0.24302, 0.24898, 0.25467, 0.26006, 0.26515,\
        0.26992, 0.27435, 0.27840, 0.28209, 0.28545, 0.28859, 0.29166, 0.29476, 0.29790, 0.30096, 0.30386,\
        0.30677, 0.30992, 0.31313, 0.31626, 0.31931, 0.32234, 0.32540, 0.32851, 0.33168, 0.33493, 0.33823,\
        0.34159, 0.34499, 0.34841, 0.35183, 0.35525, 0.35864, 0.36201, 0.36536, 0.36869, 0.37201, 0.37526,\
        0.37838, 0.38129, 0.38402, 0.38661, 0.38900, 0.39116, 0.39328, 0.39590, 0.39975, 0.40413, 0.40827,\
        0.41172, 0.41446, 0.41679, 0.41874, 0.42006, 0.42058, 0.42037, 0.41965, 0.41854, 0.41699, 0.41499,\
        0.41260, 0.40976, 0.40670, 0.40400, 0.40160, 0.39985, 0.39883, 0.39754, 0.39646, 0.39528, 0.39369,\
        0.39195, 0.39040, 0.38881, 0.38686, 0.38446, 0.38167, 0.37860, 0.37539, 0.37214, 0.36890, 0.36568,\
        0.36243, 0.35912, 0.35572, 0.35221, 0.34858, 0.34483, 0.34097, 0.33703, 0.33301, 0.32895, 0.32489,\
        0.32087, 0.31692, 0.31308, 0.30934, 0.30567, 0.30203, 0.29848, 0.29523, 0.29213, 0.28904, 0.28594,\
        0.28278, 0.27952, 0.27614, 0.27263, 0.26894, 0.26503, 0.26088, 0.25645, 0.25173, 0.24671, 0.24137],\
        [0.20055, 0.20726, 0.21398, 0.22064, 0.22715, 0.23346, 0.23953, 0.24534, 0.25087, 0.25612, 0.26107,\
        0.26572, 0.27006, 0.27407, 0.27778, 0.28123, 0.28450, 0.28768, 0.29084, 0.29397, 0.29702, 0.29987,\
        0.30269, 0.30605, 0.30925, 0.31221, 0.31507, 0.31790, 0.32077, 0.32371, 0.32674, 0.32984, 0.33303,\
        0.33628, 0.33958, 0.34291, 0.34625, 0.34958, 0.35289, 0.35616, 0.35940, 0.36259, 0.36573, 0.36878,\
        0.37169, 0.37444, 0.37701, 0.37944, 0.38170, 0.38387, 0.38615, 0.38893, 0.39251, 0.39652, 0.40046,\
        0.40349, 0.40575, 0.40793, 0.41003, 0.41142, 0.41187, 0.41150, 0.41076, 0.40985, 0.40851, 0.40663,\
        0.40450, 0.40190, 0.39873, 0.39618, 0.39400, 0.39235, 0.39122, 0.39013, 0.38908, 0.38795, 0.38659,\
        0.38509, 0.38357, 0.38194, 0.38005, 0.37781, 0.37523, 0.37239, 0.36938, 0.36627, 0.36313, 0.35995,\
        0.35674, 0.35346, 0.35011, 0.34667, 0.34313, 0.33948, 0.33574, 0.33190, 0.32800, 0.32404, 0.32006,\
        0.31613, 0.31230, 0.30860, 0.30505, 0.30159, 0.29817, 0.29478, 0.29158, 0.28852, 0.28550, 0.28246,\
        0.27935, 0.27615, 0.27281, 0.26933, 0.26565, 0.26176, 0.25762, 0.25321, 0.24851, 0.24351, 0.23819],\
        [0.19884, 0.20527, 0.21168, 0.21806, 0.22433, 0.23041, 0.23627, 0.24187, 0.24721, 0.25228, 0.25707,\
        0.26158, 0.26581, 0.26977, 0.27348, 0.27699, 0.28034, 0.28360, 0.28680, 0.28995, 0.29300, 0.29592,\
        0.29903, 0.30254, 0.30545, 0.30815, 0.31080, 0.31343, 0.31611, 0.31891, 0.32181, 0.32480, 0.32788,\
        0.33104, 0.33426, 0.33753, 0.34080, 0.34405, 0.34728, 0.35046, 0.35359, 0.35665, 0.35964, 0.36252,\
        0.36527, 0.36787, 0.37033, 0.37266, 0.37490, 0.37714, 0.37955, 0.38236, 0.38565, 0.38924, 0.39267,\
        0.39541, 0.39744, 0.39959, 0.40182, 0.40326, 0.40362, 0.40300, 0.40218, 0.40152, 0.40045, 0.39862,\
        0.39665, 0.39434, 0.39156, 0.38908, 0.38702, 0.38540, 0.38415, 0.38306, 0.38201, 0.38090, 0.37965,\
        0.37828, 0.37680, 0.37518, 0.37333, 0.37120, 0.36879, 0.36613, 0.36330, 0.36034, 0.35730, 0.35420,\
        0.35105, 0.34784, 0.34456, 0.34120, 0.33776, 0.33424, 0.33062, 0.32691, 0.32313, 0.31927, 0.31539,\
        0.31154, 0.30781, 0.30426, 0.30090, 0.29761, 0.29439, 0.29120, 0.28810, 0.28510, 0.28212, 0.27913,\
        0.27606, 0.27289, 0.26958, 0.26611, 0.26245, 0.25856, 0.25443, 0.25003, 0.24534, 0.24034, 0.23503],\
        [0.19702, 0.20331, 0.20954, 0.21572, 0.22176, 0.22759, 0.23320, 0.23856, 0.24367, 0.24853, 0.25313,\
        0.25749, 0.26161, 0.26550, 0.26919, 0.27271, 0.27611, 0.27942, 0.28267, 0.28586, 0.28901, 0.29218,\
        0.29554, 0.29884, 0.30139, 0.30397, 0.30647, 0.30887, 0.31140, 0.31410, 0.31690, 0.31979, 0.32278,\
        0.32587, 0.32903, 0.33225, 0.33547, 0.33867, 0.34182, 0.34492, 0.34794, 0.35088, 0.35373, 0.35647,\
        0.35909, 0.36159, 0.36396, 0.36625, 0.36849, 0.37077, 0.37322, 0.37594, 0.37896, 0.38210, 0.38508,\
        0.38762, 0.38976, 0.39185, 0.39383, 0.39517, 0.39550, 0.39495, 0.39421, 0.39354, 0.39253, 0.39095,\
        0.38909, 0.38696, 0.38461, 0.38236, 0.38041, 0.37880, 0.37747, 0.37632, 0.37523, 0.37411, 0.37290,\
        0.37158, 0.37013, 0.36853, 0.36672, 0.36468, 0.36239, 0.35989, 0.35720, 0.35438, 0.35145, 0.34844,\
        0.34537, 0.34224, 0.33906, 0.33580, 0.33248, 0.32909, 0.32561, 0.32206, 0.31842, 0.31469, 0.31091,\
        0.30714, 0.30350, 0.30008, 0.29690, 0.29380, 0.29074, 0.28772, 0.28474, 0.28181, 0.27888, 0.27592,\
        0.27288, 0.26973, 0.26644, 0.26298, 0.25932, 0.25543, 0.25130, 0.24690, 0.24220, 0.23721, 0.23189],\
        [0.19526, 0.20141, 0.20759, 0.21361, 0.21940, 0.22496, 0.23030, 0.23540, 0.24025, 0.24487, 0.24927,\
        0.25345, 0.25743, 0.26123, 0.26488, 0.26839, 0.27181, 0.27515, 0.27844, 0.28168, 0.28491, 0.28815,\
        0.29140, 0.29441, 0.29708, 0.29965, 0.30204, 0.30417, 0.30664, 0.30931, 0.31202, 0.31481, 0.31772,\
        0.32076, 0.32389, 0.32708, 0.33026, 0.33341, 0.33650, 0.33952, 0.34245, 0.34529, 0.34802, 0.35064,\
        0.35316, 0.35556, 0.35787, 0.36012, 0.36235, 0.36464, 0.36705, 0.36964, 0.37238, 0.37518, 0.37784,\
        0.38023, 0.38235, 0.38430, 0.38598, 0.38711, 0.38749, 0.38722, 0.38663, 0.38590, 0.38489, 0.38350,\
        0.38180, 0.37988, 0.37783, 0.37582, 0.37400, 0.37242, 0.37105, 0.36984, 0.36869, 0.36754, 0.36633,\
        0.36502, 0.36359, 0.36201, 0.36024, 0.35827, 0.35608, 0.35369, 0.35113, 0.34843, 0.34561, 0.34270,\
        0.33972, 0.33668, 0.33360, 0.33046, 0.32727, 0.32403, 0.32071, 0.31733, 0.31387, 0.31031, 0.30665,\
        0.30298, 0.29947, 0.29620, 0.29314, 0.29017, 0.28725, 0.28436, 0.28148, 0.27862, 0.27574, 0.27281,\
        0.26979, 0.26666, 0.26337, 0.25991, 0.25624, 0.25235, 0.24821, 0.24380, 0.23909, 0.23408, 0.22875],\
        [0.19384, 0.19994, 0.20597, 0.21171, 0.21721, 0.22249, 0.22754, 0.23235, 0.23693, 0.24129, 0.24545,\
        0.24943, 0.25326, 0.25696, 0.26054, 0.26402, 0.26743, 0.27078, 0.27409, 0.27736, 0.28060, 0.28381,\
        0.28693, 0.28986, 0.29256, 0.29510, 0.29733, 0.29939, 0.30197, 0.30458, 0.30720, 0.30987, 0.31270,\
        0.31571, 0.31885, 0.32203, 0.32519, 0.32830, 0.33133, 0.33427, 0.33711, 0.33985, 0.34248, 0.34500,\
        0.34742, 0.34975, 0.35200, 0.35421, 0.35641, 0.35866, 0.36099, 0.36343, 0.36594, 0.36846, 0.37088,\
        0.37310, 0.37511, 0.37689, 0.37835, 0.37934, 0.37976, 0.37967, 0.37923, 0.37853, 0.37756, 0.37630,\
        0.37478, 0.37306, 0.37124, 0.36943, 0.36773, 0.36619, 0.36482, 0.36355, 0.36236, 0.36117, 0.35993,\
        0.35861, 0.35718, 0.35562, 0.35388, 0.35196, 0.34986, 0.34757, 0.34511, 0.34252, 0.33980, 0.33699,\
        0.33410, 0.33116, 0.32818, 0.32517, 0.32213, 0.31904, 0.31590, 0.31270, 0.30946, 0.30613, 0.30264,\
        0.29910, 0.29574, 0.29258, 0.28960, 0.28672, 0.28390, 0.28110, 0.27831, 0.27551, 0.27268, 0.26977,\
        0.26677, 0.26364, 0.26036, 0.25689, 0.25321, 0.24930, 0.24514, 0.24071, 0.23599, 0.23096, 0.22561],\
        [0.19262, 0.19883, 0.20453, 0.20989, 0.21511, 0.22011, 0.22487, 0.22939, 0.23367, 0.23775, 0.24166,\
        0.24543, 0.24909, 0.25266, 0.25615, 0.25959, 0.26298, 0.26632, 0.26963, 0.27290, 0.27612, 0.27927,\
        0.28231, 0.28519, 0.28787, 0.29035, 0.29263, 0.29489, 0.29737, 0.29990, 0.30243, 0.30494, 0.30772,\
        0.31076, 0.31392, 0.31711, 0.32025, 0.32331, 0.32628, 0.32914, 0.33190, 0.33455, 0.33708, 0.33952,\
        0.34185, 0.34411, 0.34630, 0.34846, 0.35061, 0.35279, 0.35502, 0.35731, 0.35963, 0.36193, 0.36413,\
        0.36619, 0.36804, 0.36966, 0.37096, 0.37186, 0.37232, 0.37234, 0.37201, 0.37139, 0.37050, 0.36935,\
        0.36798, 0.36644, 0.36480, 0.36316, 0.36157, 0.36008, 0.35871, 0.35742, 0.35618, 0.35495, 0.35368,\
        0.35235, 0.35091, 0.34935, 0.34765, 0.34578, 0.34374, 0.34153, 0.33917, 0.33666, 0.33404, 0.33132,\
        0.32852, 0.32567, 0.32280, 0.31992, 0.31703, 0.31413, 0.31115, 0.30811, 0.30510, 0.30206, 0.29878,\
        0.29541, 0.29218, 0.28911, 0.28620, 0.28339, 0.28065, 0.27793, 0.27521, 0.27247, 0.26967, 0.26679,\
        0.26381, 0.26068, 0.25738, 0.25390, 0.25020, 0.24626, 0.24208, 0.23762, 0.23288, 0.22783, 0.22244],\
        [0.19116, 0.19722, 0.20269, 0.20798, 0.21304, 0.21782, 0.22229, 0.22649, 0.23046, 0.23424, 0.23787,\
        0.24141, 0.24488, 0.24831, 0.25171, 0.25509, 0.25845, 0.26177, 0.26507, 0.26832, 0.27151, 0.27461,\
        0.27760, 0.28044, 0.28310, 0.28560, 0.28797, 0.29032, 0.29274, 0.29521, 0.29763, 0.30005, 0.30284,\
        0.30595, 0.30915, 0.31232, 0.31543, 0.31844, 0.32134, 0.32413, 0.32680, 0.32936, 0.33181, 0.33416,\
        0.33642, 0.33860, 0.34073, 0.34283, 0.34491, 0.34701, 0.34913, 0.35128, 0.35342, 0.35553, 0.35755,\
        0.35944, 0.36114, 0.36262, 0.36381, 0.36466, 0.36513, 0.36523, 0.36499, 0.36446, 0.36366, 0.36263,\
        0.36139, 0.35999, 0.35850, 0.35699, 0.35549, 0.35406, 0.35269, 0.35139, 0.35013, 0.34886, 0.34757,\
        0.34621, 0.34477, 0.34322, 0.34153, 0.33971, 0.33773, 0.33559, 0.33331, 0.33088, 0.32834, 0.32570,\
        0.32299, 0.32022, 0.31745, 0.31469, 0.31197, 0.30927, 0.30649, 0.30354, 0.30066, 0.29787, 0.29484,\
        0.29170, 0.28863, 0.28569, 0.28286, 0.28013, 0.27745, 0.27480, 0.27215, 0.26945, 0.26670, 0.26384,\
        0.26087, 0.25774, 0.25443, 0.25092, 0.24719, 0.24322, 0.23900, 0.23451, 0.22974, 0.22465, 0.21923],\
        [0.18953, 0.19536, 0.20081, 0.20614, 0.21107, 0.21560, 0.21978, 0.22365, 0.22727, 0.23072, 0.23406,\
        0.23734, 0.24062, 0.24391, 0.24721, 0.25053, 0.25384, 0.25715, 0.26042, 0.26365, 0.26681, 0.26989,\
        0.27284, 0.27566, 0.27833, 0.28086, 0.28330, 0.28570, 0.28811, 0.29053, 0.29294, 0.29544, 0.29825,\
        0.30135, 0.30452, 0.30766, 0.31071, 0.31366, 0.31650, 0.31920, 0.32179, 0.32426, 0.32662, 0.32889,\
        0.33107, 0.33318, 0.33524, 0.33727, 0.33929, 0.34130, 0.34331, 0.34532, 0.34730, 0.34924, 0.35110,\
        0.35283, 0.35441, 0.35577, 0.35688, 0.35769, 0.35817, 0.35833, 0.35817, 0.35773, 0.35703, 0.35609,\
        0.35496, 0.35369, 0.35232, 0.35090, 0.34949, 0.34810, 0.34676, 0.34545, 0.34417, 0.34288, 0.34157,\
        0.34019, 0.33874, 0.33719, 0.33553, 0.33374, 0.33182, 0.32975, 0.32753, 0.32519, 0.32272, 0.32016,\
        0.31752, 0.31483, 0.31213, 0.30948, 0.30691, 0.30444, 0.30195, 0.29909, 0.29630, 0.29360, 0.29079,\
        0.28790, 0.28503, 0.28223, 0.27951, 0.27687, 0.27427, 0.27169, 0.26909, 0.26645, 0.26373, 0.26090,\
        0.25794, 0.25481, 0.25148, 0.24794, 0.24417, 0.24015, 0.23588, 0.23135, 0.22654, 0.22143, 0.21597],\
        [0.18794, 0.19386, 0.19941, 0.20453, 0.20921, 0.21347, 0.21733, 0.22084, 0.22408, 0.22716, 0.23018,\
        0.23321, 0.23628, 0.23943, 0.24264, 0.24590, 0.24918, 0.25246, 0.25572, 0.25893, 0.26207, 0.26512,\
        0.26806, 0.27088, 0.27356, 0.27613, 0.27861, 0.28105, 0.28348, 0.28592, 0.28840, 0.29100, 0.29382,\
        0.29685, 0.29997, 0.30307, 0.30604, 0.30891, 0.31170, 0.31432, 0.31681, 0.31920, 0.32149, 0.32368,\
        0.32577, 0.32781, 0.32981, 0.33177, 0.33371, 0.33563, 0.33753, 0.33941, 0.34126, 0.34305, 0.34476,\
        0.34636, 0.34781, 0.34908, 0.35013, 0.35093, 0.35143, 0.35164, 0.35155, 0.35118, 0.35056, 0.34972,\
        0.34869, 0.34751, 0.34623, 0.34490, 0.34355, 0.34220, 0.34088, 0.33957, 0.33828, 0.33699, 0.33565,\
        0.33427, 0.33280, 0.33126, 0.32962, 0.32787, 0.32600, 0.32400, 0.32185, 0.31957, 0.31718, 0.31470,\
        0.31214, 0.30952, 0.30688, 0.30431, 0.30186, 0.29954, 0.29725, 0.29467, 0.29200, 0.28937, 0.28672,\
        0.28404, 0.28135, 0.27870, 0.27611, 0.27357, 0.27106, 0.26855, 0.26602, 0.26343, 0.26075, 0.25795,\
        0.25500, 0.25186, 0.24852, 0.24494, 0.24112, 0.23704, 0.23271, 0.22812, 0.22326, 0.21812, 0.21262],\
        [0.18634, 0.19237, 0.19795, 0.20296, 0.20740, 0.21139, 0.21492, 0.21804, 0.22086, 0.22354, 0.22621,\
        0.22897, 0.23185, 0.23487, 0.23800, 0.24122, 0.24447, 0.24774, 0.25099, 0.25419, 0.25731, 0.26035,\
        0.26328, 0.26610, 0.26880, 0.27140, 0.27393, 0.27641, 0.27887, 0.28135, 0.28389, 0.28655, 0.28937,\
        0.29234, 0.29540, 0.29846, 0.30130, 0.30416, 0.30693, 0.30942, 0.31182, 0.31416, 0.31638, 0.31848,\
        0.32049, 0.32246, 0.32439, 0.32629, 0.32816, 0.32999, 0.33179, 0.33355, 0.33527, 0.33692, 0.33850,\
        0.33998, 0.34134, 0.34254, 0.34355, 0.34434, 0.34488, 0.34513, 0.34510, 0.34481, 0.34425, 0.34348,\
        0.34253, 0.34143, 0.34023, 0.33896, 0.33765, 0.33634, 0.33504, 0.33375, 0.33246, 0.33116, 0.32982,\
        0.32842, 0.32695, 0.32540, 0.32378, 0.32208, 0.32028, 0.31835, 0.31626, 0.31404, 0.31171, 0.30931,\
        0.30685, 0.30433, 0.30173, 0.29924, 0.29692, 0.29469, 0.29246, 0.29010, 0.28763, 0.28513, 0.28263,\
        0.28011, 0.27759, 0.27509, 0.27263, 0.27019, 0.26778, 0.26535, 0.26289, 0.26036, 0.25773, 0.25496,\
        0.25203, 0.24889, 0.24553, 0.24192, 0.23803, 0.23387, 0.22945, 0.22478, 0.21987, 0.21469, 0.20917],\
        [0.18462, 0.19073, 0.19629, 0.20118, 0.20556, 0.20937, 0.21257, 0.21524, 0.21758, 0.21981, 0.22212,\
        0.22461, 0.22731, 0.23023, 0.23331, 0.23650, 0.23975, 0.24301, 0.24625, 0.24943, 0.25255, 0.25558,\
        0.25851, 0.26133, 0.26405, 0.26668, 0.26924, 0.27175, 0.27425, 0.27678, 0.27936, 0.28203, 0.28483,\
        0.28774, 0.29071, 0.29365, 0.29650, 0.29940, 0.30215, 0.30441, 0.30675, 0.30911, 0.31127, 0.31327,\
        0.31519, 0.31709, 0.31897, 0.32082, 0.32262, 0.32437, 0.32607, 0.32772, 0.32932, 0.33084, 0.33230,\
        0.33368, 0.33496, 0.33611, 0.33711, 0.33791, 0.33848, 0.33879, 0.33882, 0.33858, 0.33808, 0.33737,\
        0.33647, 0.33543, 0.33428, 0.33306, 0.33180, 0.33051, 0.32923, 0.32795, 0.32667, 0.32538, 0.32405,\
        0.32265, 0.32117, 0.31961, 0.31800, 0.31635, 0.31463, 0.31278, 0.31075, 0.30858, 0.30631, 0.30398,\
        0.30163, 0.29923, 0.29675, 0.29435, 0.29209, 0.28991, 0.28773, 0.28549, 0.28318, 0.28084, 0.27847,\
        0.27610, 0.27373, 0.27137, 0.26904, 0.26672, 0.26440, 0.26206, 0.25968, 0.25721, 0.25464, 0.25191,\
        0.24901, 0.24588, 0.24251, 0.23886, 0.23491, 0.23065, 0.22611, 0.22131, 0.21630, 0.21111, 0.20558],\
        [0.18270, 0.18893, 0.19451, 0.19942, 0.20381, 0.20747, 0.21027, 0.21241, 0.21419, 0.21593, 0.21786,\
        0.22010, 0.22267, 0.22553, 0.22859, 0.23178, 0.23503, 0.23829, 0.24152, 0.24470, 0.24780, 0.25082,\
        0.25375, 0.25658, 0.25931, 0.26195, 0.26453, 0.26707, 0.26960, 0.27214, 0.27474, 0.27741, 0.28017,\
        0.28301, 0.28590, 0.28878, 0.29164, 0.29447, 0.29703, 0.29920, 0.30166, 0.30407, 0.30610, 0.30797,\
        0.30981, 0.31167, 0.31352, 0.31534, 0.31708, 0.31875, 0.32036, 0.32191, 0.32339, 0.32480, 0.32615,\
        0.32743, 0.32864, 0.32977, 0.33077, 0.33161, 0.33223, 0.33260, 0.33268, 0.33248, 0.33203, 0.33135,\
        0.33049, 0.32949, 0.32839, 0.32720, 0.32596, 0.32470, 0.32343, 0.32217, 0.32091, 0.31965, 0.31834,\
        0.31695, 0.31546, 0.31386, 0.31224, 0.31065, 0.30904, 0.30728, 0.30531, 0.30318, 0.30096, 0.29871,\
        0.29644, 0.29414, 0.29181, 0.28951, 0.28730, 0.28516, 0.28303, 0.28088, 0.27869, 0.27647, 0.27424,\
        0.27200, 0.26976, 0.26754, 0.26532, 0.26311, 0.26090, 0.25866, 0.25635, 0.25397, 0.25145, 0.24878,\
        0.24592, 0.24281, 0.23944, 0.23576, 0.23175, 0.22740, 0.22270, 0.21771, 0.21252, 0.20728, 0.20180],\
        [0.18052, 0.18696, 0.19279, 0.19792, 0.20235, 0.20573, 0.20801, 0.20949, 0.21064, 0.21185, 0.21341,\
        0.21545, 0.21795, 0.22079, 0.22387, 0.22708, 0.23034, 0.23360, 0.23682, 0.23999, 0.24308, 0.24609,\
        0.24900, 0.25183, 0.25456, 0.25721, 0.25980, 0.26235, 0.26488, 0.26743, 0.27002, 0.27266, 0.27538,\
        0.27815, 0.28096, 0.28377, 0.28655, 0.28924, 0.29174, 0.29409, 0.29655, 0.29886, 0.30077, 0.30256,\
        0.30435, 0.30618, 0.30803, 0.30985, 0.31154, 0.31310, 0.31464, 0.31612, 0.31749, 0.31878, 0.32002,\
        0.32122, 0.32238, 0.32349, 0.32452, 0.32541, 0.32611, 0.32654, 0.32667, 0.32650, 0.32606, 0.32540,\
        0.32456, 0.32359, 0.32252, 0.32136, 0.32015, 0.31890, 0.31763, 0.31639, 0.31516, 0.31393, 0.31266,\
        0.31131, 0.30982, 0.30816, 0.30649, 0.30496, 0.30348, 0.30182, 0.29989, 0.29782, 0.29566, 0.29348,\
        0.29129, 0.28907, 0.28685, 0.28465, 0.28251, 0.28041, 0.27833, 0.27625, 0.27415, 0.27203, 0.26991,\
        0.26778, 0.26567, 0.26356, 0.26146, 0.25936, 0.25725, 0.25510, 0.25289, 0.25059, 0.24815, 0.24554,\
        0.24273, 0.23966, 0.23631, 0.23262, 0.22858, 0.22414, 0.21929, 0.21406, 0.20855, 0.20303, 0.19765],\
        [0.17792, 0.18464, 0.19080, 0.19634, 0.20083, 0.20391, 0.20563, 0.20636, 0.20686, 0.20756, 0.20879,\
        0.21072, 0.21322, 0.21610, 0.21922, 0.22247, 0.22574, 0.22898, 0.23218, 0.23531, 0.23838, 0.24137,\
        0.24427, 0.24708, 0.24981, 0.25245, 0.25503, 0.25757, 0.26009, 0.26262, 0.26518, 0.26778, 0.27044,\
        0.27314, 0.27587, 0.27859, 0.28129, 0.28389, 0.28638, 0.28878, 0.29114, 0.29333, 0.29526, 0.29704,\
        0.29882, 0.30063, 0.30251, 0.30436, 0.30596, 0.30739, 0.30890, 0.31034, 0.31160, 0.31276, 0.31390,\
        0.31502, 0.31614, 0.31724, 0.31832, 0.31930, 0.32008, 0.32060, 0.32077, 0.32061, 0.32016, 0.31949,\
        0.31866, 0.31772, 0.31667, 0.31554, 0.31435, 0.31309, 0.31182, 0.31059, 0.30940, 0.30822, 0.30700,\
        0.30571, 0.30427, 0.30255, 0.30076, 0.29927, 0.29787, 0.29628, 0.29443, 0.29244, 0.29037, 0.28826,\
        0.28614, 0.28400, 0.28187, 0.27975, 0.27766, 0.27561, 0.27358, 0.27155, 0.26953, 0.26750, 0.26546,\
        0.26344, 0.26143, 0.25942, 0.25743, 0.25544, 0.25343, 0.25138, 0.24927, 0.24705, 0.24469, 0.24216,\
        0.23941, 0.23640, 0.23309, 0.22942, 0.22537, 0.22091, 0.21598, 0.21056, 0.20472, 0.19887, 0.19320],\
        [0.17475, 0.18167, 0.18805, 0.19373, 0.19838, 0.20172, 0.20284, 0.20282, 0.20286, 0.20316, 0.20409,\
        0.20605, 0.20860, 0.21153, 0.21471, 0.21798, 0.22124, 0.22445, 0.22759, 0.23067, 0.23370, 0.23666,\
        0.23954, 0.24233, 0.24503, 0.24766, 0.25021, 0.25272, 0.25521, 0.25770, 0.26021, 0.26276, 0.26535,\
        0.26798, 0.27062, 0.27327, 0.27588, 0.27843, 0.28089, 0.28326, 0.28554, 0.28767, 0.28962, 0.29146,\
        0.29326, 0.29507, 0.29694, 0.29878, 0.30031, 0.30154, 0.30316, 0.30460, 0.30568, 0.30673, 0.30779,\
        0.30884, 0.30991, 0.31102, 0.31216, 0.31324, 0.31414, 0.31475, 0.31497, 0.31478, 0.31427, 0.31358,\
        0.31276, 0.31184, 0.31081, 0.30972, 0.30855, 0.30730, 0.30600, 0.30478, 0.30363, 0.30249, 0.30130,\
        0.30006, 0.29870, 0.29701, 0.29526, 0.29371, 0.29225, 0.29067, 0.28891, 0.28701, 0.28503, 0.28301,\
        0.28095, 0.27889, 0.27683, 0.27478, 0.27275, 0.27075, 0.26876, 0.26678, 0.26481, 0.26285, 0.26089,\
        0.25895, 0.25702, 0.25511, 0.25322, 0.25132, 0.24941, 0.24746, 0.24544, 0.24332, 0.24105, 0.23860,\
        0.23593, 0.23299, 0.22974, 0.22612, 0.22210, 0.21766, 0.21278, 0.20738, 0.20134, 0.19525, 0.18917],\
        [0.17086, 0.17787, 0.18433, 0.19007, 0.19488, 0.19814, 0.19911, 0.19886, 0.19881, 0.19895, 0.19974,\
        0.20171, 0.20423, 0.20718, 0.21040, 0.21368, 0.21688, 0.21999, 0.22304, 0.22605, 0.22903, 0.23196,\
        0.23480, 0.23756, 0.24023, 0.24282, 0.24533, 0.24779, 0.25023, 0.25266, 0.25511, 0.25759, 0.26011,\
        0.26266, 0.26523, 0.26780, 0.27034, 0.27283, 0.27525, 0.27758, 0.27981, 0.28192, 0.28389, 0.28577,\
        0.28760, 0.28942, 0.29124, 0.29297, 0.29446, 0.29581, 0.29742, 0.29876, 0.29969, 0.30068, 0.30172,\
        0.30270, 0.30370, 0.30482, 0.30603, 0.30722, 0.30825, 0.30897, 0.30922, 0.30896, 0.30836, 0.30763,\
        0.30684, 0.30595, 0.30493, 0.30387, 0.30277, 0.30152, 0.30019, 0.29898, 0.29786, 0.29674, 0.29557,\
        0.29433, 0.29296, 0.29139, 0.28976, 0.28819, 0.28666, 0.28506, 0.28336, 0.28154, 0.27964, 0.27769,\
        0.27571, 0.27372, 0.27172, 0.26973, 0.26775, 0.26579, 0.26384, 0.26190, 0.25998, 0.25807, 0.25617,\
        0.25429, 0.25243, 0.25060, 0.24879, 0.24698, 0.24517, 0.24331, 0.24139, 0.23936, 0.23718, 0.23482,\
        0.23224, 0.22938, 0.22621, 0.22265, 0.21867, 0.21426, 0.20947, 0.20427, 0.19842, 0.19189, 0.18527],\
        [0.16620, 0.17321, 0.17960, 0.18522, 0.18979, 0.19287, 0.19427, 0.19459, 0.19473, 0.19507, 0.19601,\
        0.19779, 0.20014, 0.20309, 0.20633, 0.20957, 0.21264, 0.21558, 0.21848, 0.22142, 0.22435, 0.22724,\
        0.23005, 0.23277, 0.23540, 0.23793, 0.24038, 0.24277, 0.24513, 0.24749, 0.24986, 0.25227, 0.25471,\
        0.25719, 0.25968, 0.26218, 0.26466, 0.26710, 0.26948, 0.27177, 0.27397, 0.27607, 0.27806, 0.27996,\
        0.28181, 0.28362, 0.28538, 0.28704, 0.28856, 0.29001, 0.29145, 0.29269, 0.29370, 0.29468, 0.29567,\
        0.29662, 0.29761, 0.29872, 0.29996, 0.30121, 0.30231, 0.30316, 0.30346, 0.30309, 0.30237, 0.30163,\
        0.30087, 0.30003, 0.29902, 0.29798, 0.29691, 0.29571, 0.29444, 0.29323, 0.29210, 0.29096, 0.28979,\
        0.28854, 0.28718, 0.28571, 0.28416, 0.28261, 0.28105, 0.27945, 0.27778, 0.27602, 0.27419, 0.27231,\
        0.27040, 0.26846, 0.26652, 0.26458, 0.26265, 0.26072, 0.25881, 0.25690, 0.25501, 0.25313, 0.25127,\
        0.24944, 0.24764, 0.24587, 0.24413, 0.24241, 0.24068, 0.23892, 0.23708, 0.23514, 0.23305, 0.23078,\
        0.22828, 0.22551, 0.22243, 0.21897, 0.21501, 0.21053, 0.20569, 0.20050, 0.19461, 0.18801, 0.18109],\
        [0.16080, 0.16773, 0.17399, 0.17944, 0.18383, 0.18695, 0.18880, 0.18976, 0.19040, 0.19114, 0.19231,\
        0.19407, 0.19641, 0.19929, 0.20246, 0.20562, 0.20847, 0.21112, 0.21387, 0.21674, 0.21965, 0.22250,\
        0.22526, 0.22793, 0.23050, 0.23297, 0.23534, 0.23764, 0.23991, 0.24217, 0.24446, 0.24678, 0.24915,\
        0.25156, 0.25399, 0.25643, 0.25886, 0.26125, 0.26358, 0.26585, 0.26803, 0.27012, 0.27212, 0.27403,\
        0.27589, 0.27768, 0.27941, 0.28104, 0.28257, 0.28401, 0.28537, 0.28658, 0.28765, 0.28865, 0.28963,\
        0.29061, 0.29163, 0.29274, 0.29394, 0.29515, 0.29625, 0.29710, 0.29741, 0.29704, 0.29634, 0.29560,\
        0.29485, 0.29402, 0.29307, 0.29206, 0.29099, 0.28984, 0.28864, 0.28746, 0.28631, 0.28515, 0.28397,\
        0.28272, 0.28138, 0.27996, 0.27847, 0.27695, 0.27540, 0.27381, 0.27216, 0.27044, 0.26867, 0.26684,\
        0.26499, 0.26311, 0.26122, 0.25932, 0.25742, 0.25553, 0.25364, 0.25176, 0.24989, 0.24804, 0.24620,\
        0.24439, 0.24262, 0.24090, 0.23922, 0.23756, 0.23592, 0.23424, 0.23249, 0.23063, 0.22862, 0.22642,\
        0.22399, 0.22130, 0.21832, 0.21500, 0.21115, 0.20651, 0.20135, 0.19607, 0.19010, 0.18349, 0.17641],\
        [0.15473, 0.16155, 0.16769, 0.17301, 0.17736, 0.18065, 0.18293, 0.18448, 0.18568, 0.18687, 0.18835,\
        0.19027, 0.19267, 0.19550, 0.19862, 0.20171, 0.20422, 0.20647, 0.20912, 0.21201, 0.21491, 0.21771,\
        0.22042, 0.22304, 0.22554, 0.22792, 0.23020, 0.23239, 0.23455, 0.23671, 0.23889, 0.24113, 0.24343,\
        0.24577, 0.24815, 0.25054, 0.25292, 0.25527, 0.25757, 0.25981, 0.26198, 0.26406, 0.26606, 0.26798,\
        0.26984, 0.27162, 0.27332, 0.27494, 0.27646, 0.27788, 0.27921, 0.28043, 0.28153, 0.28257, 0.28358,\
        0.28459, 0.28563, 0.28673, 0.28787, 0.28899, 0.28999, 0.29071, 0.29098, 0.29075, 0.29019, 0.28951,\
        0.28877, 0.28795, 0.28704, 0.28606, 0.28501, 0.28391, 0.28276, 0.28161, 0.28046, 0.27929, 0.27809,\
        0.27684, 0.27553, 0.27414, 0.27269, 0.27120, 0.26967, 0.26809, 0.26647, 0.26479, 0.26306, 0.26128,\
        0.25948, 0.25764, 0.25579, 0.25393, 0.25207, 0.25020, 0.24834, 0.24647, 0.24461, 0.24276, 0.24093,\
        0.23913, 0.23737, 0.23567, 0.23403, 0.23244, 0.23087, 0.22927, 0.22760, 0.22580, 0.22385, 0.22170,\
        0.21932, 0.21667, 0.21375, 0.21059, 0.20709, 0.20261, 0.19701, 0.19134, 0.18516, 0.17843, 0.17123],\
        [0.14808, 0.15481, 0.16086, 0.16613, 0.17053, 0.17404, 0.17673, 0.17880, 0.18053, 0.18218, 0.18400,\
        0.18611, 0.18859, 0.19139, 0.19440, 0.19734, 0.19952, 0.20153, 0.20427, 0.20728, 0.21016, 0.21285,\
        0.21548, 0.21805, 0.22049, 0.22278, 0.22494, 0.22701, 0.22903, 0.23107, 0.23315, 0.23531, 0.23754,\
        0.23983, 0.24216, 0.24451, 0.24685, 0.24917, 0.25145, 0.25367, 0.25582, 0.25789, 0.25989, 0.26181,\
        0.26366, 0.26543, 0.26712, 0.26873, 0.27024, 0.27166, 0.27298, 0.27421, 0.27534, 0.27642, 0.27745,\
        0.27849, 0.27953, 0.28060, 0.28167, 0.28268, 0.28355, 0.28416, 0.28441, 0.28428, 0.28386, 0.28327,\
        0.28257, 0.28178, 0.28091, 0.27996, 0.27895, 0.27789, 0.27679, 0.27566, 0.27451, 0.27335, 0.27215,\
        0.27090, 0.26960, 0.26824, 0.26682, 0.26536, 0.26385, 0.26229, 0.26069, 0.25905, 0.25735, 0.25562,\
        0.25385, 0.25206, 0.25024, 0.24841, 0.24658, 0.24473, 0.24288, 0.24102, 0.23916, 0.23731, 0.23547,\
        0.23365, 0.23188, 0.23018, 0.22856, 0.22701, 0.22551, 0.22399, 0.22239, 0.22063, 0.21871, 0.21659,\
        0.21425, 0.21158, 0.20858, 0.20541, 0.20223, 0.19821, 0.19249, 0.18630, 0.17979, 0.17287, 0.16552],\
        [0.14093, 0.14760, 0.15361, 0.15889, 0.16340, 0.16715, 0.17021, 0.17274, 0.17494, 0.17702, 0.17915,\
        0.18147, 0.18402, 0.18678, 0.18961, 0.19228, 0.19453, 0.19678, 0.19951, 0.20266, 0.20535, 0.20782,\
        0.21039, 0.21296, 0.21534, 0.21752, 0.21954, 0.22146, 0.22334, 0.22525, 0.22723, 0.22931, 0.23149,\
        0.23374, 0.23603, 0.23835, 0.24066, 0.24296, 0.24521, 0.24741, 0.24955, 0.25162, 0.25361, 0.25553,\
        0.25737, 0.25913, 0.26082, 0.26241, 0.26392, 0.26534, 0.26666, 0.26790, 0.26906, 0.27016, 0.27123,\
        0.27227, 0.27331, 0.27433, 0.27532, 0.27624, 0.27699, 0.27752, 0.27776, 0.27770, 0.27738, 0.27687,\
        0.27623, 0.27549, 0.27467, 0.27376, 0.27279, 0.27177, 0.27070, 0.26960, 0.26847, 0.26731, 0.26612,\
        0.26488, 0.26359, 0.26225, 0.26085, 0.25941, 0.25793, 0.25639, 0.25482, 0.25320, 0.25154, 0.24984,\
        0.24810, 0.24634, 0.24455, 0.24275, 0.24093, 0.23910, 0.23726, 0.23540, 0.23354, 0.23167, 0.22981,\
        0.22795, 0.22614, 0.22440, 0.22278, 0.22127, 0.21984, 0.21839, 0.21684, 0.21512, 0.21316, 0.21104,\
        0.20875, 0.20605, 0.20288, 0.19949, 0.19609, 0.19201, 0.18664, 0.18046, 0.17381, 0.16674, 0.15928],\
        [0.13335, 0.13998, 0.14599, 0.15134, 0.15599, 0.15998, 0.16338, 0.16631, 0.16892, 0.17137, 0.17380,\
        0.17631, 0.17894, 0.18167, 0.18440, 0.18701, 0.18946, 0.19197, 0.19479, 0.19780, 0.20019, 0.20249,\
        0.20514, 0.20776, 0.21005, 0.21210, 0.21397, 0.21574, 0.21746, 0.21922, 0.22111, 0.22313, 0.22527,\
        0.22750, 0.22977, 0.23207, 0.23436, 0.23663, 0.23887, 0.24105, 0.24317, 0.24523, 0.24722, 0.24913,\
        0.25097, 0.25272, 0.25440, 0.25599, 0.25749, 0.25891, 0.26024, 0.26149, 0.26268, 0.26380, 0.26487,\
        0.26592, 0.26693, 0.26791, 0.26883, 0.26966, 0.27033, 0.27080, 0.27103, 0.27101, 0.27076, 0.27033,\
        0.26976, 0.26908, 0.26830, 0.26744, 0.26651, 0.26553, 0.26449, 0.26342, 0.26231, 0.26116, 0.25998,\
        0.25875, 0.25747, 0.25615, 0.25478, 0.25336, 0.25189, 0.25038, 0.24883, 0.24723, 0.24559, 0.24392,\
        0.24221, 0.24047, 0.23871, 0.23693, 0.23513, 0.23330, 0.23147, 0.22961, 0.22774, 0.22586, 0.22395,\
        0.22204, 0.22016, 0.21835, 0.21670, 0.21521, 0.21384, 0.21246, 0.21097, 0.20926, 0.20721, 0.20497,\
        0.20276, 0.20009, 0.19676, 0.19321, 0.18944, 0.18508, 0.17983, 0.17377, 0.16713, 0.16002, 0.15249],\
        [0.12538, 0.13201, 0.13806, 0.14349, 0.14831, 0.15253, 0.15624, 0.15951, 0.16247, 0.16525, 0.16795,\
        0.17065, 0.17337, 0.17613, 0.17885, 0.18151, 0.18411, 0.18676, 0.18953, 0.19225, 0.19468, 0.19706,\
        0.19980, 0.20241, 0.20452, 0.20643, 0.20819, 0.20981, 0.21136, 0.21298, 0.21479, 0.21679, 0.21892,\
        0.22113, 0.22339, 0.22567, 0.22794, 0.23019, 0.23241, 0.23458, 0.23669, 0.23874, 0.24071, 0.24262,\
        0.24444, 0.24619, 0.24786, 0.24945, 0.25095, 0.25237, 0.25371, 0.25498, 0.25617, 0.25730, 0.25838,\
        0.25942, 0.26041, 0.26134, 0.26220, 0.26296, 0.26356, 0.26399, 0.26421, 0.26422, 0.26403, 0.26366,\
        0.26315, 0.26252, 0.26180, 0.26098, 0.26010, 0.25915, 0.25815, 0.25710, 0.25602, 0.25489, 0.25372,\
        0.25250, 0.25124, 0.24993, 0.24857, 0.24717, 0.24573, 0.24423, 0.24270, 0.24112, 0.23951, 0.23786,\
        0.23617, 0.23446, 0.23271, 0.23094, 0.22915, 0.22734, 0.22550, 0.22364, 0.22177, 0.21986, 0.21793,\
        0.21596, 0.21397, 0.21204, 0.21033, 0.20884, 0.20752, 0.20620, 0.20473, 0.20306, 0.20090, 0.19845,\
        0.19613, 0.19342, 0.19008, 0.18638, 0.18231, 0.17772, 0.17242, 0.16641, 0.15981, 0.15270, 0.14514],\
        [0.11706, 0.12372, 0.12982, 0.13536, 0.14034, 0.14479, 0.14877, 0.15235, 0.15562, 0.15869, 0.16163,\
        0.16450, 0.16735, 0.17017, 0.17296, 0.17569, 0.17838, 0.18109, 0.18380, 0.18644, 0.18896, 0.19148,\
        0.19412, 0.19656, 0.19860, 0.20046, 0.20215, 0.20365, 0.20507, 0.20653, 0.20831, 0.21032, 0.21245,\
        0.21466, 0.21690, 0.21916, 0.22142, 0.22365, 0.22585, 0.22800, 0.23009, 0.23213, 0.23409, 0.23598,\
        0.23780, 0.23955, 0.24121, 0.24279, 0.24430, 0.24572, 0.24706, 0.24833, 0.24953, 0.25067, 0.25175,\
        0.25277, 0.25373, 0.25463, 0.25543, 0.25613, 0.25669, 0.25708, 0.25730, 0.25733, 0.25717, 0.25686,\
        0.25641, 0.25583, 0.25515, 0.25439, 0.25355, 0.25264, 0.25167, 0.25065, 0.24959, 0.24848, 0.24732,\
        0.24612, 0.24487, 0.24358, 0.24224, 0.24085, 0.23942, 0.23795, 0.23643, 0.23487, 0.23328, 0.23164,\
        0.22997, 0.22827, 0.22654, 0.22478, 0.22299, 0.22118, 0.21935, 0.21748, 0.21560, 0.21368, 0.21173,\
        0.20973, 0.20764, 0.20552, 0.20372, 0.20222, 0.20090, 0.19959, 0.19807, 0.19633, 0.19416, 0.19167,\
        0.18907, 0.18616, 0.18278, 0.17896, 0.17470, 0.16992, 0.16453, 0.15853, 0.15194, 0.14484, 0.13728],\
        [0.10842, 0.11512, 0.12130, 0.12696, 0.13210, 0.13676, 0.14098, 0.14483, 0.14837, 0.15170, 0.15486,\
        0.15792, 0.16090, 0.16382, 0.16668, 0.16949, 0.17226, 0.17500, 0.17770, 0.18034, 0.18291, 0.18544,\
        0.18794, 0.19025, 0.19230, 0.19414, 0.19578, 0.19725, 0.19860, 0.19995, 0.20176, 0.20378, 0.20590,\
        0.20809, 0.21032, 0.21256, 0.21479, 0.21700, 0.21917, 0.22130, 0.22338, 0.22540, 0.22735, 0.22923,\
        0.23104, 0.23278, 0.23443, 0.23601, 0.23751, 0.23894, 0.24028, 0.24155, 0.24276, 0.24389, 0.24496,\
        0.24597, 0.24690, 0.24776, 0.24853, 0.24918, 0.24970, 0.25007, 0.25028, 0.25032, 0.25020, 0.24993,\
        0.24952, 0.24900, 0.24837, 0.24765, 0.24685, 0.24597, 0.24504, 0.24405, 0.24301, 0.24192, 0.24078,\
        0.23959, 0.23836, 0.23708, 0.23576, 0.23439, 0.23297, 0.23151, 0.23001, 0.22846, 0.22688, 0.22526,\
        0.22360, 0.22190, 0.22018, 0.21843, 0.21664, 0.21483, 0.21299, 0.21112, 0.20923, 0.20730, 0.20534,\
        0.20333, 0.20124, 0.19893, 0.19705, 0.19549, 0.19405, 0.19261, 0.19100, 0.18912, 0.18691, 0.18438,\
        0.18159, 0.17849, 0.17499, 0.17104, 0.16662, 0.16170, 0.15623, 0.15019, 0.14360, 0.13650, 0.12894]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=60 (m/s) and translational velocity of Vt=0 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax60vt0
    zgrid_H_Hmax_vmax60vt0=[[0.50364, 0.51406, 0.52405, 0.53359, 0.54268, 0.55132, 0.55950, 0.56724, 0.57454, 0.58143, 0.58793,\
        0.59407, 0.59988, 0.60539, 0.61061, 0.61555, 0.62022, 0.62460, 0.62866, 0.63239, 0.63580, 0.63889,\
        0.64174, 0.64441, 0.64699, 0.64961, 0.65238, 0.65542, 0.65882, 0.66262, 0.66678, 0.67122, 0.67585,\
        0.68071, 0.68585, 0.69090, 0.69504, 0.69875, 0.70223, 0.70544, 0.70838, 0.71108, 0.71353, 0.71577,\
        0.71779, 0.71964, 0.72131, 0.72281, 0.72416, 0.72534, 0.72634, 0.72716, 0.72778, 0.72820, 0.72838,\
        0.72833, 0.72803, 0.72746, 0.72661, 0.72546, 0.72401, 0.72225, 0.72017, 0.71777, 0.71507, 0.71208,\
        0.70881, 0.70531, 0.70158, 0.69767, 0.69359, 0.68938, 0.68511, 0.68097, 0.67707, 0.67338, 0.66981,\
        0.66632, 0.66289, 0.65950, 0.65612, 0.65269, 0.64917, 0.64550, 0.64162, 0.63750, 0.63311, 0.62841,\
        0.62339, 0.61805, 0.61241, 0.60647, 0.60011, 0.59286, 0.58476, 0.57681, 0.56940, 0.56258, 0.55638,\
        0.55073, 0.54554, 0.54070, 0.53609, 0.53160, 0.52712, 0.52254, 0.51777, 0.51274, 0.50738, 0.50164,\
        0.49551, 0.48896, 0.48201, 0.47465, 0.46691, 0.45879, 0.45030, 0.44147, 0.43230, 0.42279, 0.41295],\
        [0.51561, 0.52597, 0.53588, 0.54533, 0.55433, 0.56286, 0.57092, 0.57852, 0.58567, 0.59239, 0.59872,\
        0.60468, 0.61033, 0.61569, 0.62079, 0.62565, 0.63027, 0.63461, 0.63863, 0.64230, 0.64559, 0.64853,\
        0.65116, 0.65358, 0.65589, 0.65820, 0.66068, 0.66348, 0.66673, 0.67048, 0.67468, 0.67918, 0.68378,\
        0.68847, 0.69362, 0.69937, 0.70358, 0.70731, 0.71083, 0.71408, 0.71703, 0.71971, 0.72213, 0.72432,\
        0.72630, 0.72811, 0.72975, 0.73123, 0.73257, 0.73375, 0.73477, 0.73561, 0.73626, 0.73670, 0.73691,\
        0.73689, 0.73661, 0.73606, 0.73523, 0.73409, 0.73263, 0.73084, 0.72872, 0.72626, 0.72347, 0.72037,\
        0.71698, 0.71332, 0.70942, 0.70531, 0.70103, 0.69658, 0.69199, 0.68766, 0.68372, 0.68004, 0.67649,\
        0.67299, 0.66957, 0.66624, 0.66294, 0.65961, 0.65617, 0.65256, 0.64871, 0.64458, 0.64012, 0.63529,\
        0.63006, 0.62441, 0.61837, 0.61195, 0.60517, 0.59754, 0.58910, 0.58133, 0.57415, 0.56764, 0.56181,\
        0.55659, 0.55187, 0.54749, 0.54335, 0.53931, 0.53525, 0.53107, 0.52665, 0.52193, 0.51683, 0.51131,\
        0.50534, 0.49893, 0.49209, 0.48482, 0.47717, 0.46913, 0.46074, 0.45201, 0.44294, 0.43354, 0.42382],\
        [0.52720, 0.53748, 0.54732, 0.55670, 0.56561, 0.57404, 0.58197, 0.58943, 0.59641, 0.60295, 0.60908,\
        0.61485, 0.62031, 0.62551, 0.63050, 0.63529, 0.63988, 0.64424, 0.64829, 0.65196, 0.65519, 0.65801,\
        0.66046, 0.66265, 0.66469, 0.66670, 0.66884, 0.67136, 0.67445, 0.67817, 0.68247, 0.68713, 0.69178,\
        0.69623, 0.70089, 0.70657, 0.71147, 0.71554, 0.71918, 0.72248, 0.72545, 0.72810, 0.73048, 0.73261,\
        0.73453, 0.73628, 0.73788, 0.73935, 0.74068, 0.74187, 0.74291, 0.74377, 0.74445, 0.74492, 0.74516,\
        0.74517, 0.74492, 0.74440, 0.74359, 0.74247, 0.74102, 0.73922, 0.73707, 0.73455, 0.73168, 0.72848,\
        0.72496, 0.72115, 0.71707, 0.71274, 0.70820, 0.70345, 0.69854, 0.69407, 0.69013, 0.68649, 0.68291,\
        0.67934, 0.67591, 0.67267, 0.66951, 0.66631, 0.66298, 0.65945, 0.65565, 0.65152, 0.64700, 0.64203,\
        0.63656, 0.63056, 0.62398, 0.61677, 0.60882, 0.60012, 0.59268, 0.58546, 0.57857, 0.57241, 0.56704,\
        0.56232, 0.55809, 0.55420, 0.55052, 0.54692, 0.54329, 0.53949, 0.53542, 0.53099, 0.52613, 0.52078,\
        0.51495, 0.50864, 0.50187, 0.49467, 0.48708, 0.47911, 0.47080, 0.46215, 0.45319, 0.44391, 0.43431],\
        [0.53837, 0.54860, 0.55838, 0.56769, 0.57651, 0.58484, 0.59265, 0.59996, 0.60677, 0.61311, 0.61902,\
        0.62456, 0.62980, 0.63482, 0.63967, 0.64440, 0.64900, 0.65344, 0.65760, 0.66136, 0.66460, 0.66733,\
        0.66965, 0.67168, 0.67351, 0.67522, 0.67700, 0.67918, 0.68210, 0.68582, 0.69020, 0.69515, 0.70000,\
        0.70450, 0.70917, 0.71431, 0.71924, 0.72356, 0.72736, 0.73072, 0.73369, 0.73630, 0.73861, 0.74065,\
        0.74249, 0.74417, 0.74572, 0.74716, 0.74849, 0.74969, 0.75075, 0.75165, 0.75235, 0.75285, 0.75313,\
        0.75317, 0.75295, 0.75247, 0.75170, 0.75061, 0.74919, 0.74739, 0.74522, 0.74266, 0.73973, 0.73643,\
        0.73280, 0.72885, 0.72460, 0.72006, 0.71526, 0.71023, 0.70515, 0.70048, 0.69641, 0.69278, 0.68908,\
        0.68529, 0.68187, 0.67882, 0.67589, 0.67286, 0.66966, 0.66622, 0.66249, 0.65840, 0.65387, 0.64880,\
        0.64314, 0.63682, 0.62979, 0.62202, 0.61357, 0.60509, 0.59749, 0.58993, 0.58304, 0.57723, 0.57236,\
        0.56815, 0.56440, 0.56096, 0.55772, 0.55453, 0.55129, 0.54785, 0.54411, 0.53994, 0.53528, 0.53007,\
        0.52433, 0.51807, 0.51133, 0.50417, 0.49661, 0.48870, 0.48045, 0.47189, 0.46303, 0.45386, 0.44439],\
        [0.54914, 0.55931, 0.56903, 0.57828, 0.58703, 0.59526, 0.60296, 0.61012, 0.61674, 0.62286, 0.62851,\
        0.63379, 0.63876, 0.64355, 0.64823, 0.65288, 0.65752, 0.66210, 0.66649, 0.67046, 0.67381, 0.67651,\
        0.67873, 0.68068, 0.68243, 0.68396, 0.68535, 0.68710, 0.68989, 0.69368, 0.69803, 0.70310, 0.70816,\
        0.71290, 0.71766, 0.72254, 0.72729, 0.73163, 0.73548, 0.73886, 0.74180, 0.74434, 0.74654, 0.74846,\
        0.75018, 0.75177, 0.75326, 0.75467, 0.75600, 0.75722, 0.75831, 0.75924, 0.75998, 0.76051, 0.76081,\
        0.76088, 0.76069, 0.76025, 0.75953, 0.75850, 0.75712, 0.75534, 0.75317, 0.75058, 0.74759, 0.74424,\
        0.74053, 0.73647, 0.73209, 0.72740, 0.72241, 0.71722, 0.71201, 0.70706, 0.70266, 0.69880, 0.69481,\
        0.69056, 0.68738, 0.68477, 0.68215, 0.67931, 0.67624, 0.67292, 0.66931, 0.66531, 0.66082, 0.65575,\
        0.65000, 0.64351, 0.63625, 0.62825, 0.61974, 0.61119, 0.60293, 0.59476, 0.58782, 0.58248, 0.57815,\
        0.57439, 0.57101, 0.56793, 0.56503, 0.56220, 0.55929, 0.55618, 0.55273, 0.54879, 0.54429, 0.53917,\
        0.53345, 0.52719, 0.52044, 0.51327, 0.50573, 0.49785, 0.48967, 0.48119, 0.47243, 0.46339, 0.45406],\
        [0.55947, 0.56959, 0.57926, 0.58846, 0.59714, 0.60530, 0.61289, 0.61990, 0.62633, 0.63220, 0.63757,\
        0.64252, 0.64717, 0.65166, 0.65611, 0.66065, 0.66531, 0.67008, 0.67483, 0.67921, 0.68279, 0.68547,\
        0.68765, 0.68966, 0.69154, 0.69310, 0.69418, 0.69541, 0.69814, 0.70214, 0.70659, 0.71147, 0.71651,\
        0.72140, 0.72621, 0.73097, 0.73557, 0.73982, 0.74363, 0.74697, 0.74983, 0.75225, 0.75429, 0.75605,\
        0.75761, 0.75908, 0.76049, 0.76188, 0.76322, 0.76447, 0.76559, 0.76655, 0.76732, 0.76788, 0.76820,\
        0.76828, 0.76813, 0.76773, 0.76707, 0.76611, 0.76479, 0.76307, 0.76090, 0.75830, 0.75528, 0.75189,\
        0.74814, 0.74404, 0.73959, 0.73482, 0.72974, 0.72442, 0.71902, 0.71372, 0.70869, 0.70397, 0.69880,\
        0.69501, 0.69264, 0.69078, 0.68844, 0.68571, 0.68276, 0.67958, 0.67612, 0.67228, 0.66793, 0.66295,\
        0.65726, 0.65078, 0.64350, 0.63547, 0.62689, 0.61800, 0.60900, 0.60026, 0.59358, 0.58879, 0.58487,\
        0.58132, 0.57809, 0.57521, 0.57255, 0.56996, 0.56731, 0.56448, 0.56128, 0.55754, 0.55313, 0.54803,\
        0.54226, 0.53593, 0.52913, 0.52192, 0.51438, 0.50654, 0.49842, 0.49003, 0.48139, 0.47248, 0.46330],\
        [0.56936, 0.57943, 0.58906, 0.59821, 0.60685, 0.61494, 0.62244, 0.62932, 0.63555, 0.64116, 0.64619,\
        0.65076, 0.65500, 0.65911, 0.66325, 0.66759, 0.67222, 0.67717, 0.68235, 0.68741, 0.69147, 0.69414,\
        0.69637, 0.69860, 0.70081, 0.70257, 0.70375, 0.70495, 0.70761, 0.71142, 0.71578, 0.72045, 0.72528,\
        0.73010, 0.73486, 0.73951, 0.74396, 0.74811, 0.75183, 0.75508, 0.75782, 0.76007, 0.76190, 0.76342,\
        0.76478, 0.76608, 0.76742, 0.76879, 0.77015, 0.77144, 0.77260, 0.77359, 0.77439, 0.77497, 0.77530,\
        0.77538, 0.77522, 0.77486, 0.77427, 0.77341, 0.77220, 0.77054, 0.76840, 0.76580, 0.76277, 0.75937,\
        0.75563, 0.75155, 0.74713, 0.74236, 0.73725, 0.73186, 0.72627, 0.72059, 0.71488, 0.70908, 0.70362,\
        0.70020, 0.69823, 0.69723, 0.69473, 0.69204, 0.68922, 0.68619, 0.68293, 0.67933, 0.67520, 0.67043,\
        0.66493, 0.65863, 0.65151, 0.64363, 0.63511, 0.62617, 0.61707, 0.60846, 0.60169, 0.59687, 0.59294,\
        0.58912, 0.58572, 0.58287, 0.58031, 0.57781, 0.57531, 0.57270, 0.56974, 0.56615, 0.56177, 0.55658,\
        0.55069, 0.54423, 0.53732, 0.53006, 0.52251, 0.51470, 0.50666, 0.49838, 0.48987, 0.48112, 0.47210],\
        [0.57878, 0.58880, 0.59839, 0.60751, 0.61612, 0.62417, 0.63160, 0.63836, 0.64441, 0.64974, 0.65440,\
        0.65851, 0.66226, 0.66588, 0.66961, 0.67366, 0.67816, 0.68316, 0.68866, 0.69452, 0.69955, 0.70239,\
        0.70502, 0.70767, 0.71016, 0.71224, 0.71388, 0.71561, 0.71814, 0.72155, 0.72553, 0.72986, 0.73439,\
        0.73899, 0.74359, 0.74809, 0.75240, 0.75643, 0.76005, 0.76319, 0.76578, 0.76781, 0.76936, 0.77058,\
        0.77166, 0.77278, 0.77402, 0.77540, 0.77682, 0.77817, 0.77937, 0.78038, 0.78120, 0.78180, 0.78212,\
        0.78215, 0.78196, 0.78160, 0.78109, 0.78036, 0.77928, 0.77773, 0.77563, 0.77302, 0.76999, 0.76663,\
        0.76298, 0.75900, 0.75469, 0.75001, 0.74496, 0.73957, 0.73388, 0.72799, 0.72199, 0.71610, 0.71093,\
        0.70718, 0.70494, 0.70339, 0.70090, 0.69834, 0.69564, 0.69271, 0.68971, 0.68642, 0.68260, 0.67812,\
        0.67292, 0.66696, 0.66018, 0.65263, 0.64442, 0.63578, 0.62708, 0.61895, 0.61209, 0.60671, 0.60220,\
        0.59767, 0.59389, 0.59098, 0.58833, 0.58569, 0.58317, 0.58075, 0.57804, 0.57457, 0.57012, 0.56472,\
        0.55860, 0.55194, 0.54491, 0.53760, 0.53005, 0.52230, 0.51436, 0.50622, 0.49787, 0.48929, 0.48047],\
        [0.58772, 0.59769, 0.60723, 0.61633, 0.62492, 0.63295, 0.64034, 0.64703, 0.65292, 0.65798, 0.66224,\
        0.66582, 0.66897, 0.67199, 0.67520, 0.67887, 0.68316, 0.68810, 0.69362, 0.69964, 0.70554, 0.70986,\
        0.71346, 0.71678, 0.71976, 0.72229, 0.72444, 0.72655, 0.72905, 0.73209, 0.73561, 0.73951, 0.74366,\
        0.74796, 0.75232, 0.75663, 0.76080, 0.76470, 0.76822, 0.77126, 0.77370, 0.77549, 0.77669, 0.77752,\
        0.77826, 0.77914, 0.78030, 0.78172, 0.78326, 0.78470, 0.78592, 0.78692, 0.78775, 0.78837, 0.78866,\
        0.78857, 0.78828, 0.78791, 0.78744, 0.78689, 0.78601, 0.78458, 0.78253, 0.77989, 0.77688, 0.77362,\
        0.77013, 0.76635, 0.76224, 0.75775, 0.75286, 0.74757, 0.74191, 0.73598, 0.72997, 0.72419, 0.71910,\
        0.71510, 0.71219, 0.70980, 0.70729, 0.70470, 0.70198, 0.69911, 0.69644, 0.69348, 0.68997, 0.68582,\
        0.68106, 0.67558, 0.66930, 0.66223, 0.65451, 0.64639, 0.63821, 0.63044, 0.62349, 0.61748, 0.61209,\
        0.60704, 0.60283, 0.59953, 0.59655, 0.59342, 0.59070, 0.58850, 0.58607, 0.58269, 0.57801, 0.57225,\
        0.56580, 0.55893, 0.55179, 0.54445, 0.53695, 0.52930, 0.52150, 0.51353, 0.50537, 0.49700, 0.48839],\
        [0.59615, 0.60605, 0.61555, 0.62462, 0.63320, 0.64124, 0.64864, 0.65529, 0.66108, 0.66590, 0.66974,\
        0.67273, 0.67516, 0.67745, 0.68004, 0.68329, 0.68740, 0.69235, 0.69799, 0.70439, 0.71092, 0.71666,\
        0.72156, 0.72584, 0.72953, 0.73262, 0.73521, 0.73757, 0.73998, 0.74268, 0.74574, 0.74916, 0.75288,\
        0.75683, 0.76092, 0.76503, 0.76904, 0.77282, 0.77625, 0.77921, 0.78155, 0.78310, 0.78388, 0.78424,\
        0.78458, 0.78519, 0.78626, 0.78779, 0.78952, 0.79108, 0.79229, 0.79322, 0.79403, 0.79468, 0.79499,\
        0.79463, 0.79419, 0.79377, 0.79321, 0.79292, 0.79226, 0.79100, 0.78900, 0.78627, 0.78330, 0.78025,\
        0.77703, 0.77355, 0.76973, 0.76553, 0.76089, 0.75580, 0.75026, 0.74439, 0.73843, 0.73269, 0.72753,\
        0.72318, 0.71965, 0.71664, 0.71382, 0.71105, 0.70828, 0.70557, 0.70302, 0.70028, 0.69706, 0.69327,\
        0.68905, 0.68421, 0.67853, 0.67204, 0.66493, 0.65741, 0.64975, 0.64225, 0.63518, 0.62866, 0.62263,\
        0.61711, 0.61232, 0.60825, 0.60455, 0.60078, 0.59781, 0.59584, 0.59367, 0.59028, 0.58511, 0.57884,\
        0.57206, 0.56502, 0.55784, 0.55055, 0.54316, 0.53567, 0.52805, 0.52030, 0.51237, 0.50424, 0.49588],\
        [0.60406, 0.61388, 0.62331, 0.63234, 0.64091, 0.64896, 0.65640, 0.66309, 0.66886, 0.67352, 0.67695,\
        0.67928, 0.68089, 0.68232, 0.68418, 0.68696, 0.69101, 0.69627, 0.70242, 0.70938, 0.71661, 0.72346,\
        0.72959, 0.73493, 0.73943, 0.74311, 0.74606, 0.74852, 0.75077, 0.75309, 0.75566, 0.75859, 0.76186,\
        0.76544, 0.76925, 0.77315, 0.77700, 0.78066, 0.78397, 0.78689, 0.78925, 0.79062, 0.79090, 0.79075,\
        0.79070, 0.79099, 0.79194, 0.79363, 0.79565, 0.79734, 0.79851, 0.79932, 0.80008, 0.80071, 0.80102,\
        0.80029, 0.79978, 0.79928, 0.79820, 0.79843, 0.79774, 0.79682, 0.79490, 0.79197, 0.78918, 0.78651,\
        0.78368, 0.78055, 0.77708, 0.77324, 0.76896, 0.76414, 0.75876, 0.75296, 0.74700, 0.74119, 0.73584,\
        0.73114, 0.72709, 0.72355, 0.72034, 0.71733, 0.71446, 0.71174, 0.70914, 0.70647, 0.70348, 0.70011,\
        0.69656, 0.69249, 0.68744, 0.68159, 0.67516, 0.66830, 0.66115, 0.65390, 0.64675, 0.63983, 0.63327,\
        0.62717, 0.62167, 0.61676, 0.61226, 0.60813, 0.60486, 0.60253, 0.60031, 0.59678, 0.59079, 0.58407,\
        0.57712, 0.57008, 0.56299, 0.55585, 0.54866, 0.54139, 0.53403, 0.52654, 0.51888, 0.51103, 0.50294],\
        [0.61144, 0.62114, 0.63048, 0.63944, 0.64798, 0.65604, 0.66353, 0.67032, 0.67617, 0.68079, 0.68388,\
        0.68550, 0.68619, 0.68667, 0.68770, 0.68996, 0.69396, 0.69977, 0.70671, 0.71448, 0.72261, 0.73052,\
        0.73778, 0.74415, 0.74948, 0.75370, 0.75690, 0.75930, 0.76125, 0.76310, 0.76516, 0.76758, 0.77040,\
        0.77362, 0.77715, 0.78084, 0.78456, 0.78810, 0.79124, 0.79405, 0.79654, 0.79792, 0.79763, 0.79711,\
        0.79682, 0.79681, 0.79759, 0.79937, 0.80159, 0.80335, 0.80454, 0.80538, 0.80606, 0.80654, 0.80659,\
        0.80603, 0.80536, 0.80450, 0.80334, 0.80314, 0.80244, 0.80175, 0.79967, 0.79686, 0.79457, 0.79249,\
        0.79009, 0.78730, 0.78418, 0.78078, 0.77694, 0.77245, 0.76723, 0.76143, 0.75537, 0.74941, 0.74381,\
        0.73876, 0.73427, 0.73029, 0.72669, 0.72338, 0.72030, 0.71742, 0.71468, 0.71196, 0.70915, 0.70622,\
        0.70324, 0.69985, 0.69542, 0.69028, 0.68468, 0.67856, 0.67197, 0.66500, 0.65779, 0.65054, 0.64346,\
        0.63675, 0.63053, 0.62485, 0.61968, 0.61507, 0.61119, 0.60796, 0.60477, 0.60031, 0.59424, 0.58766,\
        0.58091, 0.57410, 0.56725, 0.56038, 0.55348, 0.54651, 0.53945, 0.53227, 0.52493, 0.51739, 0.50961],\
        [0.61829, 0.62784, 0.63705, 0.64589, 0.65435, 0.66237, 0.66988, 0.67676, 0.68280, 0.68759, 0.69048,\
        0.69136, 0.69110, 0.69064, 0.69079, 0.69247, 0.69644, 0.70278, 0.71085, 0.71974, 0.72891, 0.73788,\
        0.74620, 0.75355, 0.75964, 0.76433, 0.76764, 0.76981, 0.77127, 0.77253, 0.77402, 0.77596, 0.77835,\
        0.78120, 0.78450, 0.78800, 0.79150, 0.79510, 0.79796, 0.80056, 0.80294, 0.80426, 0.80409, 0.80359,\
        0.80327, 0.80319, 0.80389, 0.80552, 0.80749, 0.80921, 0.81048, 0.81139, 0.81203, 0.81238, 0.81233,\
        0.81186, 0.81110, 0.81013, 0.80910, 0.80831, 0.80742, 0.80618, 0.80415, 0.80172, 0.79996, 0.79836,\
        0.79627, 0.79369, 0.79088, 0.78796, 0.78467, 0.78057, 0.77544, 0.76951, 0.76326, 0.75708, 0.75124,\
        0.74588, 0.74103, 0.73666, 0.73270, 0.72906, 0.72569, 0.72254, 0.71958, 0.71673, 0.71395, 0.71122,\
        0.70849, 0.70547, 0.70178, 0.69759, 0.69304, 0.68772, 0.68178, 0.67518, 0.66795, 0.66037, 0.65280,\
        0.64549, 0.63861, 0.63224, 0.62639, 0.62110, 0.61636, 0.61196, 0.60738, 0.60199, 0.59610, 0.58998,\
        0.58365, 0.57721, 0.57074, 0.56423, 0.55768, 0.55106, 0.54436, 0.53754, 0.53055, 0.52335, 0.51591],\
        [0.62462, 0.63398, 0.64300, 0.65168, 0.65999, 0.66789, 0.67532, 0.68217, 0.68833, 0.69349, 0.69647,\
        0.69655, 0.69560, 0.69452, 0.69389, 0.69494, 0.69924, 0.70644, 0.71550, 0.72543, 0.73559, 0.74552,\
        0.75480, 0.76304, 0.76985, 0.77491, 0.77815, 0.77990, 0.78067, 0.78116, 0.78206, 0.78362, 0.78563,\
        0.78809, 0.79118, 0.79472, 0.79751, 0.80125, 0.80421, 0.80673, 0.80885, 0.81012, 0.81042, 0.81026,\
        0.81010, 0.81020, 0.81088, 0.81217, 0.81377, 0.81530, 0.81654, 0.81746, 0.81808, 0.81838, 0.81831,\
        0.81786, 0.81710, 0.81612, 0.81505, 0.81395, 0.81274, 0.81122, 0.80930, 0.80728, 0.80559, 0.80399,\
        0.80206, 0.79958, 0.79704, 0.79463, 0.79197, 0.78831, 0.78310, 0.77688, 0.77039, 0.76401, 0.75797,\
        0.75238, 0.74726, 0.74257, 0.73827, 0.73428, 0.73057, 0.72708, 0.72380, 0.72069, 0.71775, 0.71496,\
        0.71227, 0.70949, 0.70647, 0.70329, 0.69973, 0.69523, 0.69022, 0.68410, 0.67681, 0.66891, 0.66090,\
        0.65309, 0.64566, 0.63868, 0.63219, 0.62616, 0.62053, 0.61507, 0.60949, 0.60360, 0.59763, 0.59168,\
        0.58570, 0.57968, 0.57361, 0.56750, 0.56135, 0.55514, 0.54883, 0.54240, 0.53579, 0.52897, 0.52188],\
        [0.63048, 0.63959, 0.64838, 0.65683, 0.66493, 0.67261, 0.67981, 0.68640, 0.69227, 0.69737, 0.70072,\
        0.70055, 0.69972, 0.69875, 0.69781, 0.69816, 0.70297, 0.71128, 0.72117, 0.73185, 0.74274, 0.75339,\
        0.76341, 0.77240, 0.77987, 0.78521, 0.78822, 0.78936, 0.78925, 0.78872, 0.78904, 0.79055, 0.79229,\
        0.79434, 0.79698, 0.80084, 0.80370, 0.80684, 0.81008, 0.81272, 0.81478, 0.81614, 0.81680, 0.81703,\
        0.81716, 0.81745, 0.81810, 0.81914, 0.82041, 0.82169, 0.82282, 0.82371, 0.82431, 0.82461, 0.82454,\
        0.82413, 0.82338, 0.82238, 0.82120, 0.81989, 0.81843, 0.81674, 0.81487, 0.81297, 0.81118, 0.80941,\
        0.80741, 0.80510, 0.80279, 0.80061, 0.79851, 0.79532, 0.78965, 0.78310, 0.77646, 0.77002, 0.76391,\
        0.75820, 0.75290, 0.74798, 0.74338, 0.73905, 0.73496, 0.73106, 0.72736, 0.72386, 0.72058, 0.71753,\
        0.71471, 0.71204, 0.70947, 0.70690, 0.70405, 0.70074, 0.69703, 0.69135, 0.68390, 0.67575, 0.66746,\
        0.65932, 0.65149, 0.64405, 0.63701, 0.63034, 0.62396, 0.61771, 0.61144, 0.60514, 0.59898, 0.59308,\
        0.58740, 0.58174, 0.57606, 0.57035, 0.56461, 0.55881, 0.55292, 0.54690, 0.54070, 0.53427, 0.52756],\
        [0.63590, 0.64474, 0.65324, 0.66141, 0.66923, 0.67665, 0.68356, 0.68975, 0.69496, 0.69903, 0.70225,\
        0.70362, 0.70374, 0.70350, 0.70333, 0.70462, 0.70962, 0.71803, 0.72816, 0.73911, 0.75032, 0.76134,\
        0.77177, 0.78122, 0.78923, 0.79482, 0.79734, 0.79783, 0.79690, 0.79503, 0.79464, 0.79678, 0.79838,\
        0.80037, 0.80297, 0.80644, 0.80991, 0.81312, 0.81619, 0.81883, 0.82092, 0.82242, 0.82337, 0.82394,\
        0.82434, 0.82479, 0.82542, 0.82628, 0.82730, 0.82836, 0.82935, 0.83018, 0.83078, 0.83109, 0.83106,\
        0.83067, 0.82993, 0.82888, 0.82759, 0.82609, 0.82441, 0.82257, 0.82062, 0.81863, 0.81668, 0.81469,\
        0.81258, 0.81031, 0.80802, 0.80579, 0.80351, 0.80004, 0.79415, 0.78780, 0.78138, 0.77509, 0.76906,\
        0.76336, 0.75799, 0.75291, 0.74807, 0.74342, 0.73893, 0.73457, 0.73035, 0.72632, 0.72252, 0.71903,\
        0.71590, 0.71315, 0.71073, 0.70853, 0.70637, 0.70412, 0.70138, 0.69596, 0.68857, 0.68051, 0.67225,\
        0.66404, 0.65604, 0.64832, 0.64091, 0.63377, 0.62684, 0.62004, 0.61329, 0.60660, 0.60014, 0.59446,\
        0.58903, 0.58363, 0.57826, 0.57291, 0.56756, 0.56218, 0.55672, 0.55112, 0.54534, 0.53931, 0.53299],\
        [0.64096, 0.64949, 0.65768, 0.66553, 0.67303, 0.68017, 0.68683, 0.69280, 0.69771, 0.70150, 0.70469,\
        0.70687, 0.70812, 0.70901, 0.71027, 0.71302, 0.71842, 0.72648, 0.73630, 0.74707, 0.75819, 0.76917,\
        0.77958, 0.78897, 0.79700, 0.80264, 0.80478, 0.80484, 0.80330, 0.80076, 0.79957, 0.80161, 0.80432,\
        0.80670, 0.80973, 0.81309, 0.81652, 0.81975, 0.82271, 0.82528, 0.82739, 0.82901, 0.83019, 0.83102,\
        0.83165, 0.83222, 0.83283, 0.83355, 0.83437, 0.83524, 0.83611, 0.83689, 0.83750, 0.83785, 0.83788,\
        0.83752, 0.83676, 0.83564, 0.83420, 0.83250, 0.83061, 0.82857, 0.82643, 0.82426, 0.82208, 0.81986,\
        0.81754, 0.81510, 0.81257, 0.80986, 0.80665, 0.80221, 0.79692, 0.79132, 0.78533, 0.77933, 0.77352,\
        0.76795, 0.76262, 0.75747, 0.75245, 0.74750, 0.74260, 0.73773, 0.73291, 0.72823, 0.72376, 0.71964,\
        0.71600, 0.71293, 0.71043, 0.70836, 0.70649, 0.70454, 0.70178, 0.69708, 0.69059, 0.68313, 0.67526,\
        0.66728, 0.65935, 0.65157, 0.64397, 0.63656, 0.62930, 0.62218, 0.61518, 0.60838, 0.60201, 0.59626,\
        0.59077, 0.58546, 0.58030, 0.57526, 0.57029, 0.56532, 0.56028, 0.55512, 0.54976, 0.54415, 0.53822],\
        [0.64575, 0.65396, 0.66180, 0.66930, 0.67647, 0.68332, 0.68980, 0.69575, 0.70086, 0.70480, 0.70815,\
        0.71092, 0.71319, 0.71534, 0.71800, 0.72195, 0.72783, 0.73571, 0.74512, 0.75543, 0.76611, 0.77672,\
        0.78679, 0.79564, 0.80291, 0.80810, 0.81054, 0.81077, 0.80949, 0.80767, 0.80688, 0.80834, 0.81108,\
        0.81401, 0.81718, 0.82049, 0.82377, 0.82685, 0.82965, 0.83211, 0.83419, 0.83590, 0.83725, 0.83829,\
        0.83911, 0.83976, 0.84034, 0.84092, 0.84156, 0.84227, 0.84304, 0.84381, 0.84447, 0.84491, 0.84500,\
        0.84468, 0.84389, 0.84265, 0.84103, 0.83911, 0.83696, 0.83466, 0.83228, 0.82986, 0.82741, 0.82490,\
        0.82230, 0.81956, 0.81662, 0.81334, 0.80947, 0.80485, 0.79989, 0.79440, 0.78868, 0.78302, 0.77750,\
        0.77215, 0.76694, 0.76180, 0.75665, 0.75145, 0.74614, 0.74073, 0.73526, 0.72981, 0.72453, 0.71958,\
        0.71521, 0.71161, 0.70886, 0.70673, 0.70479, 0.70276, 0.70036, 0.69618, 0.69059, 0.68397, 0.67675,\
        0.66923, 0.66158, 0.65393, 0.64634, 0.63883, 0.63143, 0.62416, 0.61707, 0.61030, 0.60403, 0.59818,\
        0.59247, 0.58718, 0.58222, 0.57748, 0.57287, 0.56830, 0.56368, 0.55895, 0.55402, 0.54883, 0.54329],\
        [0.65037, 0.65825, 0.66574, 0.67288, 0.67969, 0.68624, 0.69255, 0.69850, 0.70378, 0.70826, 0.71217,\
        0.71566, 0.71889, 0.72220, 0.72603, 0.73093, 0.73727, 0.74509, 0.75411, 0.76386, 0.77386, 0.78378,\
        0.79338, 0.80169, 0.80821, 0.81297, 0.81565, 0.81649, 0.81610, 0.81539, 0.81539, 0.81678, 0.81925,\
        0.82221, 0.82537, 0.82854, 0.83160, 0.83444, 0.83701, 0.83929, 0.84130, 0.84304, 0.84453, 0.84576,\
        0.84673, 0.84745, 0.84797, 0.84840, 0.84885, 0.84941, 0.85010, 0.85090, 0.85167, 0.85225, 0.85246,\
        0.85218, 0.85133, 0.84994, 0.84809, 0.84588, 0.84342, 0.84082, 0.83815, 0.83543, 0.83269, 0.82988,\
        0.82695, 0.82386, 0.82050, 0.81676, 0.81252, 0.80779, 0.80265, 0.79717, 0.79172, 0.78642, 0.78123,\
        0.77616, 0.77114, 0.76608, 0.76087, 0.75545, 0.74975, 0.74380, 0.73763, 0.73136, 0.72514, 0.71921,\
        0.71387, 0.70949, 0.70639, 0.70421, 0.70212, 0.69987, 0.69754, 0.69411, 0.68934, 0.68358, 0.67712,\
        0.67019, 0.66298, 0.65561, 0.64817, 0.64072, 0.63330, 0.62599, 0.61886, 0.61202, 0.60557, 0.59950,\
        0.59389, 0.58877, 0.58406, 0.57962, 0.57536, 0.57118, 0.56698, 0.56268, 0.55817, 0.55339, 0.54825],\
        [0.65492, 0.66249, 0.66966, 0.67644, 0.68290, 0.68911, 0.69524, 0.70127, 0.70690, 0.71198, 0.71659,\
        0.72087, 0.72502, 0.72932, 0.73411, 0.73974, 0.74645, 0.75426, 0.76298, 0.77218, 0.78138, 0.79025,\
        0.79890, 0.80681, 0.81306, 0.81771, 0.82073, 0.82231, 0.82296, 0.82334, 0.82416, 0.82582, 0.82826,\
        0.83115, 0.83419, 0.83717, 0.83995, 0.84246, 0.84471, 0.84674, 0.84861, 0.85036, 0.85199, 0.85342,\
        0.85455, 0.85533, 0.85576, 0.85599, 0.85621, 0.85660, 0.85725, 0.85813, 0.85909, 0.85989, 0.86028,\
        0.86005, 0.85912, 0.85752, 0.85536, 0.85279, 0.84998, 0.84704, 0.84404, 0.84102, 0.83797, 0.83485,\
        0.83162, 0.82818, 0.82446, 0.82037, 0.81584, 0.81087, 0.80552, 0.80000, 0.79486, 0.78986, 0.78496,\
        0.78019, 0.77542, 0.77049, 0.76528, 0.75969, 0.75365, 0.74716, 0.74031, 0.73319, 0.72599, 0.71896,\
        0.71246, 0.70698, 0.70333, 0.70145, 0.69925, 0.69698, 0.69459, 0.69155, 0.68751, 0.68254, 0.67682,\
        0.67053, 0.66382, 0.65683, 0.64964, 0.64235, 0.63502, 0.62773, 0.62056, 0.61362, 0.60698, 0.60076,\
        0.59527, 0.59036, 0.58589, 0.58175, 0.57784, 0.57403, 0.57023, 0.56634, 0.56225, 0.55788, 0.55313],\
        [0.65948, 0.66680, 0.67369, 0.68019, 0.68634, 0.69223, 0.69817, 0.70438, 0.71045, 0.71612, 0.72141,\
        0.72645, 0.73141, 0.73653, 0.74206, 0.74824, 0.75522, 0.76301, 0.77148, 0.78030, 0.78885, 0.79666,\
        0.80423, 0.81160, 0.81775, 0.82252, 0.82596, 0.82829, 0.82990, 0.83128, 0.83290, 0.83501, 0.83763,\
        0.84053, 0.84348, 0.84624, 0.84869, 0.85080, 0.85263, 0.85432, 0.85599, 0.85773, 0.85951, 0.86121,\
        0.86259, 0.86345, 0.86376, 0.86372, 0.86364, 0.86382, 0.86444, 0.86547, 0.86673, 0.86786, 0.86849,\
        0.86833, 0.86727, 0.86537, 0.86282, 0.85984, 0.85661, 0.85329, 0.84996, 0.84664, 0.84331, 0.83992,\
        0.83640, 0.83268, 0.82867, 0.82431, 0.81956, 0.81444, 0.80903, 0.80364, 0.79862, 0.79353, 0.78886,\
        0.78444, 0.77996, 0.77522, 0.77006, 0.76435, 0.75800, 0.75104, 0.74353, 0.73561, 0.72748, 0.71939,\
        0.71171, 0.70496, 0.70007, 0.69832, 0.69632, 0.69416, 0.69188, 0.68913, 0.68564, 0.68134, 0.67628,\
        0.67060, 0.66441, 0.65782, 0.65094, 0.64387, 0.63668, 0.62947, 0.62233, 0.61536, 0.60867, 0.60244,\
        0.59694, 0.59212, 0.58783, 0.58396, 0.58035, 0.57690, 0.57348, 0.56998, 0.56630, 0.56233, 0.55795],\
        [0.66415, 0.67126, 0.67793, 0.68426, 0.69032, 0.69603, 0.70186, 0.70827, 0.71466, 0.72079, 0.72665,\
        0.73232, 0.73794, 0.74370, 0.74978, 0.75634, 0.76348, 0.77119, 0.77936, 0.78788, 0.79638, 0.80335,\
        0.81018, 0.81678, 0.82265, 0.82751, 0.83135, 0.83435, 0.83681, 0.83907, 0.84144, 0.84411, 0.84706,\
        0.85012, 0.85306, 0.85564, 0.85774, 0.85936, 0.86066, 0.86189, 0.86327, 0.86497, 0.86697, 0.86907,\
        0.87086, 0.87189, 0.87204, 0.87161, 0.87113, 0.87105, 0.87164, 0.87291, 0.87458, 0.87617, 0.87713,\
        0.87704, 0.87577, 0.87348, 0.87045, 0.86696, 0.86329, 0.85959, 0.85594, 0.85235, 0.84877, 0.84515,\
        0.84141, 0.83746, 0.83323, 0.82868, 0.82379, 0.81859, 0.81319, 0.80780, 0.80247, 0.79733, 0.79317,\
        0.78912, 0.78490, 0.78039, 0.77534, 0.76955, 0.76295, 0.75556, 0.74749, 0.73888, 0.72993, 0.72094,\
        0.71232, 0.70471, 0.69909, 0.69589, 0.69375, 0.69178, 0.68969, 0.68722, 0.68415, 0.68036, 0.67586,\
        0.67071, 0.66499, 0.65881, 0.65226, 0.64544, 0.63843, 0.63135, 0.62428, 0.61735, 0.61068, 0.60449,\
        0.59895, 0.59410, 0.58995, 0.58630, 0.58296, 0.57981, 0.57674, 0.57362, 0.57034, 0.56675, 0.56274],\
        [0.66897, 0.67594, 0.68242, 0.68862, 0.69483, 0.70081, 0.70670, 0.71306, 0.71957, 0.72600, 0.73227,\
        0.73842, 0.74453, 0.75075, 0.75721, 0.76401, 0.77121, 0.77876, 0.78650, 0.79424, 0.80225, 0.80945,\
        0.81592, 0.82209, 0.82774, 0.83267, 0.83687, 0.84044, 0.84360, 0.84661, 0.84969, 0.85295, 0.85635,\
        0.85971, 0.86277, 0.86524, 0.86698, 0.86803, 0.86867, 0.86928, 0.87026, 0.87187, 0.87416, 0.87686,\
        0.87934, 0.88072, 0.88066, 0.87971, 0.87874, 0.87836, 0.87892, 0.88050, 0.88269, 0.88483, 0.88622,\
        0.88616, 0.88456, 0.88179, 0.87817, 0.87413, 0.87000, 0.86594, 0.86201, 0.85820, 0.85443, 0.85063,\
        0.84671, 0.84260, 0.83822, 0.83354, 0.82855, 0.82332, 0.81792, 0.81246, 0.80707, 0.80223, 0.79832,\
        0.79435, 0.79027, 0.78606, 0.78118, 0.77536, 0.76854, 0.76081, 0.75228, 0.74312, 0.73353, 0.72384,\
        0.71447, 0.70604, 0.69940, 0.69518, 0.69248, 0.69041, 0.68840, 0.68611, 0.68332, 0.67991, 0.67584,\
        0.67112, 0.66581, 0.65999, 0.65375, 0.64717, 0.64037, 0.63344, 0.62648, 0.61963, 0.61301, 0.60679,\
        0.60112, 0.59630, 0.59232, 0.58885, 0.58571, 0.58280, 0.58003, 0.57727, 0.57437, 0.57117, 0.56752],\
        [0.67397, 0.68091, 0.68723, 0.69310, 0.69934, 0.70570, 0.71195, 0.71840, 0.72501, 0.73163, 0.73818,\
        0.74465, 0.75111, 0.75763, 0.76431, 0.77122, 0.77840, 0.78580, 0.79319, 0.80043, 0.80759, 0.81470,\
        0.82123, 0.82729, 0.83288, 0.83792, 0.84242, 0.84646, 0.85021, 0.85384, 0.85755, 0.86141, 0.86534,\
        0.86913, 0.87245, 0.87493, 0.87632, 0.87670, 0.87653, 0.87638, 0.87681, 0.87825, 0.88083, 0.88430,\
        0.88787, 0.88995, 0.88959, 0.88805, 0.88667, 0.88600, 0.88647, 0.88843, 0.89118, 0.89380, 0.89564,\
        0.89550, 0.89343, 0.89019, 0.88590, 0.88129, 0.87676, 0.87240, 0.86825, 0.86426, 0.86036, 0.85643,\
        0.85239, 0.84815, 0.84366, 0.83888, 0.83384, 0.82859, 0.82322, 0.81786, 0.81271, 0.80808, 0.80400,\
        0.79982, 0.79601, 0.79225, 0.78756, 0.78169, 0.77469, 0.76670, 0.75787, 0.74833, 0.73829, 0.72809,\
        0.71815, 0.70903, 0.70148, 0.69622, 0.69284, 0.69045, 0.68830, 0.68602, 0.68336, 0.68018, 0.67639,\
        0.67200, 0.66701, 0.66149, 0.65552, 0.64918, 0.64258, 0.63581, 0.62898, 0.62223, 0.61568, 0.60948,\
        0.60381, 0.59900, 0.59507, 0.59164, 0.58858, 0.58583, 0.58332, 0.58090, 0.57839, 0.57558, 0.57228],\
        [0.67910, 0.68618, 0.69262, 0.69808, 0.70404, 0.71073, 0.71736, 0.72402, 0.73076, 0.73752, 0.74426,\
        0.75094, 0.75760, 0.76428, 0.77104, 0.77792, 0.78499, 0.79227, 0.79950, 0.80634, 0.81315, 0.81988,\
        0.82631, 0.83235, 0.83798, 0.84318, 0.84796, 0.85238, 0.85657, 0.86070, 0.86492, 0.86933, 0.87384,\
        0.87816, 0.88189, 0.88457, 0.88565, 0.88527, 0.88420, 0.88313, 0.88285, 0.88405, 0.88686, 0.89100,\
        0.89586, 0.89915, 0.89859, 0.89679, 0.89546, 0.89465, 0.89482, 0.89720, 0.90040, 0.90303, 0.90481,\
        0.90448, 0.90215, 0.89864, 0.89349, 0.88851, 0.88369, 0.87906, 0.87475, 0.87065, 0.86664, 0.86262,\
        0.85848, 0.85415, 0.84956, 0.84470, 0.83960, 0.83433, 0.82900, 0.82375, 0.81876, 0.81415, 0.80989,\
        0.80583, 0.80234, 0.79878, 0.79423, 0.78828, 0.78114, 0.77307, 0.76411, 0.75438, 0.74409, 0.73356,\
        0.72323, 0.71364, 0.70538, 0.69907, 0.69493, 0.69207, 0.68955, 0.68705, 0.68436, 0.68126, 0.67765,\
        0.67346, 0.66870, 0.66342, 0.65766, 0.65153, 0.64510, 0.63849, 0.63180, 0.62517, 0.61873, 0.61262,\
        0.60706, 0.60224, 0.59819, 0.59464, 0.59150, 0.58884, 0.58657, 0.58449, 0.58237, 0.57996, 0.57702],\
        [0.68417, 0.69144, 0.69832, 0.70415, 0.70990, 0.71635, 0.72304, 0.72982, 0.73667, 0.74354, 0.75039,\
        0.75720, 0.76395, 0.77067, 0.77739, 0.78415, 0.79093, 0.79789, 0.80496, 0.81172, 0.81836, 0.82491,\
        0.83125, 0.83731, 0.84303, 0.84840, 0.85343, 0.85816, 0.86268, 0.86714, 0.87173, 0.87660, 0.88168,\
        0.88656, 0.89073, 0.89384, 0.89473, 0.89351, 0.89170, 0.88963, 0.88847, 0.88954, 0.89260, 0.89726,\
        0.90260, 0.90675, 0.90761, 0.90681, 0.90599, 0.90542, 0.90561, 0.90777, 0.91069, 0.91297, 0.91406,\
        0.91336, 0.91083, 0.90665, 0.90125, 0.89626, 0.89102, 0.88610, 0.88168, 0.87750, 0.87339, 0.86926,\
        0.86503, 0.86060, 0.85591, 0.85094, 0.84574, 0.84042, 0.83509, 0.82988, 0.82492, 0.82030, 0.81604,\
        0.81214, 0.80858, 0.80497, 0.80054, 0.79458, 0.78752, 0.77964, 0.77081, 0.76107, 0.75067, 0.73997,\
        0.72941, 0.71951, 0.71078, 0.70372, 0.69883, 0.69523, 0.69201, 0.68916, 0.68632, 0.68319, 0.67963,\
        0.67555, 0.67093, 0.66580, 0.66021, 0.65423, 0.64795, 0.64148, 0.63493, 0.62841, 0.62207, 0.61605,\
        0.61053, 0.60565, 0.60144, 0.59764, 0.59434, 0.59175, 0.58972, 0.58799, 0.58628, 0.58430, 0.58171],\
        [0.68908, 0.69640, 0.70330, 0.70969, 0.71582, 0.72222, 0.72889, 0.73571, 0.74261, 0.74954, 0.75646,\
        0.76332, 0.77008, 0.77673, 0.78333, 0.78996, 0.79643, 0.80295, 0.80981, 0.81657, 0.82320, 0.82970,\
        0.83604, 0.84215, 0.84801, 0.85358, 0.85886, 0.86385, 0.86860, 0.87323, 0.87796, 0.88307, 0.88868,\
        0.89419, 0.89852, 0.90185, 0.90275, 0.90129, 0.89931, 0.89623, 0.89393, 0.89546, 0.89915, 0.90460,\
        0.91053, 0.91521, 0.91765, 0.91840, 0.91848, 0.91851, 0.91906, 0.92058, 0.92253, 0.92395, 0.92422,\
        0.92294, 0.92001, 0.91561, 0.91031, 0.90478, 0.89892, 0.89380, 0.88932, 0.88497, 0.88066, 0.87639,\
        0.87205, 0.86750, 0.86266, 0.85752, 0.85217, 0.84671, 0.84130, 0.83603, 0.83102, 0.82634, 0.82201,\
        0.81802, 0.81424, 0.81034, 0.80577, 0.80004, 0.79346, 0.78624, 0.77772, 0.76811, 0.75772, 0.74694,\
        0.73626, 0.72618, 0.71718, 0.70970, 0.70396, 0.69930, 0.69528, 0.69220, 0.68917, 0.68590, 0.68229,\
        0.67824, 0.67369, 0.66865, 0.66315, 0.65728, 0.65112, 0.64476, 0.63831, 0.63189, 0.62562, 0.61964,\
        0.61407, 0.60901, 0.60448, 0.60040, 0.59697, 0.59454, 0.59276, 0.59136, 0.59007, 0.58853, 0.58632],\
        [0.69381, 0.70123, 0.70823, 0.71491, 0.72140, 0.72796, 0.73467, 0.74152, 0.74846, 0.75542, 0.76237,\
        0.76925, 0.77597, 0.78246, 0.78874, 0.79515, 0.80170, 0.80788, 0.81444, 0.82111, 0.82774, 0.83428,\
        0.84068, 0.84690, 0.85292, 0.85871, 0.86426, 0.86954, 0.87450, 0.87916, 0.88377, 0.88880, 0.89461,\
        0.90085, 0.90545, 0.90863, 0.90983, 0.90904, 0.90698, 0.90380, 0.90081, 0.90220, 0.90818, 0.91441,\
        0.92075, 0.92599, 0.92952, 0.93150, 0.93251, 0.93312, 0.93380, 0.93474, 0.93569, 0.93610, 0.93547,\
        0.93352, 0.93017, 0.92558, 0.92013, 0.91421, 0.90817, 0.90274, 0.89789, 0.89314, 0.88848, 0.88403,\
        0.87953, 0.87482, 0.86977, 0.86436, 0.85874, 0.85307, 0.84748, 0.84207, 0.83692, 0.83208, 0.82756,\
        0.82333, 0.81924, 0.81503, 0.81034, 0.80491, 0.79929, 0.79274, 0.78448, 0.77513, 0.76483, 0.75403,\
        0.74328, 0.73309, 0.72388, 0.71594, 0.70932, 0.70365, 0.69947, 0.69613, 0.69275, 0.68924, 0.68553,\
        0.68146, 0.67691, 0.67189, 0.66644, 0.66063, 0.65455, 0.64828, 0.64192, 0.63556, 0.62932, 0.62331,\
        0.61765, 0.61239, 0.60757, 0.60322, 0.59969, 0.59740, 0.59573, 0.59454, 0.59364, 0.59258, 0.59076],\
        [0.69837, 0.70587, 0.71304, 0.71995, 0.72670, 0.73343, 0.74024, 0.74713, 0.75409, 0.76107, 0.76802,\
        0.77488, 0.78158, 0.78791, 0.79360, 0.79940, 0.80577, 0.81223, 0.81878, 0.82542, 0.83208, 0.83869,\
        0.84520, 0.85157, 0.85777, 0.86380, 0.86966, 0.87530, 0.88058, 0.88530, 0.88964, 0.89434, 0.90006,\
        0.90648, 0.91185, 0.91551, 0.91732, 0.91739, 0.91609, 0.91403, 0.91268, 0.91469, 0.92034, 0.92703,\
        0.93355, 0.93913, 0.94330, 0.94606, 0.94772, 0.94869, 0.94930, 0.94968, 0.94974, 0.94917, 0.94766,\
        0.94503, 0.94122, 0.93637, 0.93075, 0.92467, 0.91853, 0.91272, 0.90727, 0.90193, 0.89681, 0.89218,\
        0.88738, 0.88250, 0.87714, 0.87131, 0.86531, 0.85933, 0.85351, 0.84788, 0.84251, 0.83741, 0.83260,\
        0.82803, 0.82360, 0.81915, 0.81450, 0.80959, 0.80444, 0.79798, 0.79068, 0.78177, 0.77156, 0.76076,\
        0.75000, 0.73976, 0.73040, 0.72214, 0.71504, 0.70911, 0.70445, 0.70053, 0.69676, 0.69294, 0.68919,\
        0.68510, 0.68050, 0.67544, 0.66999, 0.66422, 0.65820, 0.65199, 0.64568, 0.63935, 0.63311, 0.62706,\
        0.62130, 0.61589, 0.61092, 0.60652, 0.60298, 0.60046, 0.59857, 0.59743, 0.59683, 0.59627, 0.59491],\
        [0.70289, 0.71043, 0.71772, 0.72480, 0.73173, 0.73861, 0.74551, 0.75245, 0.75943, 0.76639, 0.77330,\
        0.78010, 0.78675, 0.79319, 0.79855, 0.80336, 0.80972, 0.81626, 0.82287, 0.82955, 0.83626, 0.84297,\
        0.84962, 0.85617, 0.86257, 0.86883, 0.87499, 0.88111, 0.88698, 0.89205, 0.89625, 0.90086, 0.90683,\
        0.91325, 0.91892, 0.92318, 0.92583, 0.92702, 0.92713, 0.92690, 0.92757, 0.93047, 0.93573, 0.94216,\
        0.94858, 0.95422, 0.95866, 0.96179, 0.96376, 0.96480, 0.96517, 0.96499, 0.96424, 0.96279, 0.96047,\
        0.95717, 0.95290, 0.94775, 0.94194, 0.93572, 0.92939, 0.92319, 0.91720, 0.91136, 0.90589, 0.90064,\
        0.89542, 0.89051, 0.88461, 0.87815, 0.87168, 0.86539, 0.85930, 0.85341, 0.84774, 0.84230, 0.83710,\
        0.83212, 0.82730, 0.82258, 0.81789, 0.81317, 0.80819, 0.80275, 0.79635, 0.78758, 0.77737, 0.76663,\
        0.75596, 0.74580, 0.73643, 0.72804, 0.72073, 0.71453, 0.70937, 0.70490, 0.70064, 0.69677, 0.69318,\
        0.68904, 0.68430, 0.67915, 0.67368, 0.66794, 0.66197, 0.65581, 0.64954, 0.64323, 0.63697, 0.63086,\
        0.62500, 0.61949, 0.61443, 0.60998, 0.60630, 0.60343, 0.60128, 0.60005, 0.59948, 0.59927, 0.59847],\
        [0.70730, 0.71489, 0.72226, 0.72944, 0.73650, 0.74348, 0.75045, 0.75743, 0.76441, 0.77135, 0.77818,\
        0.78483, 0.79124, 0.79738, 0.80312, 0.80811, 0.81394, 0.82029, 0.82685, 0.83354, 0.84033, 0.84716,\
        0.85399, 0.86074, 0.86735, 0.87378, 0.88015, 0.88669, 0.89337, 0.89947, 0.90393, 0.90905, 0.91504,\
        0.92136, 0.92718, 0.93195, 0.93548, 0.93789, 0.93957, 0.94117, 0.94353, 0.94735, 0.95268, 0.95886,\
        0.96507, 0.97060, 0.97504, 0.97822, 0.98016, 0.98104, 0.98102, 0.98024, 0.97875, 0.97651, 0.97346,\
        0.96957, 0.96485, 0.95940, 0.95337, 0.94698, 0.94042, 0.93389, 0.92745, 0.92119, 0.91516, 0.90929,\
        0.90383, 0.89865, 0.89165, 0.88452, 0.87770, 0.87115, 0.86482, 0.85864, 0.85261, 0.84676, 0.84109,\
        0.83561, 0.83031, 0.82519, 0.82026, 0.81551, 0.81087, 0.80611, 0.80023, 0.79165, 0.78163, 0.77118,\
        0.76083, 0.75092, 0.74171, 0.73336, 0.72597, 0.71956, 0.71403, 0.70916, 0.70474, 0.70100, 0.69740,\
        0.69303, 0.68806, 0.68284, 0.67739, 0.67171, 0.66580, 0.65969, 0.65344, 0.64712, 0.64082, 0.63464,\
        0.62868, 0.62306, 0.61790, 0.61332, 0.60944, 0.60631, 0.60394, 0.60240, 0.60153, 0.60116, 0.60020],\
        [0.71158, 0.71921, 0.72662, 0.73387, 0.74099, 0.74803, 0.75504, 0.76203, 0.76899, 0.77591, 0.78268,\
        0.78919, 0.79533, 0.80110, 0.80681, 0.81234, 0.81811, 0.82428, 0.83074, 0.83742, 0.84427, 0.85125,\
        0.85831, 0.86534, 0.87222, 0.87880, 0.88511, 0.89175, 0.89906, 0.90589, 0.91183, 0.91775, 0.92404,\
        0.93042, 0.93641, 0.94164, 0.94597, 0.94952, 0.95262, 0.95577, 0.95951, 0.96422, 0.96988, 0.97603,\
        0.98208, 0.98744, 0.99171, 0.99469, 0.99638, 0.99689, 0.99637, 0.99498, 0.99279, 0.98986, 0.98618,\
        0.98177, 0.97666, 0.97094, 0.96471, 0.95814, 0.95137, 0.94453, 0.93772, 0.93101, 0.92442, 0.91801,\
        0.91178, 0.90519, 0.89766, 0.89032, 0.88336, 0.87668, 0.87012, 0.86361, 0.85717, 0.85084, 0.84463,\
        0.83856, 0.83265, 0.82695, 0.82151, 0.81640, 0.81161, 0.80681, 0.80103, 0.79319, 0.78394, 0.77419,\
        0.76442, 0.75498, 0.74609, 0.73792, 0.73056, 0.72405, 0.71832, 0.71328, 0.70883, 0.70494, 0.70117,\
        0.69647, 0.69145, 0.68634, 0.68102, 0.67545, 0.66963, 0.66357, 0.65734, 0.65099, 0.64462, 0.63833,\
        0.63225, 0.62649, 0.62118, 0.61642, 0.61230, 0.60888, 0.60618, 0.60416, 0.60272, 0.60161, 0.60005],\
        [0.71571, 0.72335, 0.73079, 0.73807, 0.74521, 0.75226, 0.75926, 0.76622, 0.77317, 0.78005, 0.78680,\
        0.79328, 0.79933, 0.80500, 0.81058, 0.81620, 0.82201, 0.82810, 0.83450, 0.84118, 0.84809, 0.85523,\
        0.86255, 0.86997, 0.87731, 0.88423, 0.89036, 0.89638, 0.90422, 0.91219, 0.91954, 0.92650, 0.93334,\
        0.93998, 0.94620, 0.95182, 0.95679, 0.96125, 0.96547, 0.96984, 0.97469, 0.98021, 0.98631, 0.99262,\
        0.99863, 1.00383, 1.00785, 1.01050, 1.01176, 1.01176, 1.01066, 1.00864, 1.00583, 1.00231, 0.99813,\
        0.99332, 0.98792, 0.98200, 0.97563, 0.96892, 0.96198, 0.95489, 0.94775, 0.94059, 0.93346, 0.92634,\
        0.91913, 0.91153, 0.90358, 0.89596, 0.88887, 0.88213, 0.87531, 0.86839, 0.86149, 0.85463, 0.84783,\
        0.84110, 0.83446, 0.82799, 0.82178, 0.81594, 0.81058, 0.80548, 0.79976, 0.79273, 0.78456, 0.77578,\
        0.76683, 0.75800, 0.74955, 0.74165, 0.73442, 0.72790, 0.72209, 0.71692, 0.71230, 0.70804, 0.70369,\
        0.69896, 0.69444, 0.68965, 0.68453, 0.67912, 0.67340, 0.66741, 0.66117, 0.65476, 0.64828, 0.64186,\
        0.63561, 0.62967, 0.62415, 0.61916, 0.61477, 0.61102, 0.60789, 0.60532, 0.60312, 0.60106, 0.59902],\
        [0.71966, 0.72731, 0.73475, 0.74202, 0.74915, 0.75617, 0.76312, 0.77002, 0.77689, 0.78373, 0.79047,\
        0.79695, 0.80296, 0.80860, 0.81416, 0.81980, 0.82563, 0.83172, 0.83809, 0.84477, 0.85176, 0.85906,\
        0.86666, 0.87451, 0.88250, 0.89031, 0.89705, 0.90272, 0.91064, 0.91933, 0.92762, 0.93540, 0.94276,\
        0.94968, 0.95610, 0.96197, 0.96734, 0.97238, 0.97736, 0.98256, 0.98824, 0.99446, 1.00105, 1.00765,\
        1.01374, 1.01884, 1.02259, 1.02481, 1.02554, 1.02494, 1.02323, 1.02061, 1.01727, 1.01331, 1.00879,\
        1.00375, 0.99821, 0.99222, 0.98581, 0.97906, 0.97202, 0.96477, 0.95736, 0.94982, 0.94218, 0.93442,\
        0.92645, 0.91823, 0.90988, 0.90175, 0.89456, 0.88777, 0.88044, 0.87302, 0.86563, 0.85824, 0.85082,\
        0.84339, 0.83596, 0.82861, 0.82144, 0.81460, 0.80830, 0.80289, 0.79757, 0.79124, 0.78412, 0.77639,\
        0.76830, 0.76015, 0.75218, 0.74459, 0.73753, 0.73107, 0.72523, 0.71997, 0.71518, 0.71068, 0.70621,\
        0.70180, 0.69746, 0.69285, 0.68793, 0.68269, 0.67709, 0.67115, 0.66489, 0.65838, 0.65175, 0.64513,\
        0.63867, 0.63249, 0.62673, 0.62147, 0.61678, 0.61267, 0.60911, 0.60602, 0.60323, 0.60055, 0.59796],\
        [0.72343, 0.73107, 0.73850, 0.74575, 0.75283, 0.75978, 0.76663, 0.77341, 0.78015, 0.78689, 0.79358,\
        0.80004, 0.80603, 0.81174, 0.81740, 0.82313, 0.82902, 0.83513, 0.84151, 0.84820, 0.85525, 0.86269,\
        0.87053, 0.87877, 0.88736, 0.89608, 0.90409, 0.91101, 0.91889, 0.92762, 0.93630, 0.94453, 0.95219,\
        0.95925, 0.96569, 0.97157, 0.97701, 0.98222, 0.98751, 0.99315, 0.99933, 1.00607, 1.01316, 1.02013,\
        1.02644, 1.03152, 1.03499, 1.03674, 1.03686, 1.03561, 1.03330, 1.03019, 1.02648, 1.02228, 1.01766,\
        1.01261, 1.00714, 1.00125, 0.99497, 0.98831, 0.98131, 0.97400, 0.96642, 0.95860, 0.95056, 0.94230,\
        0.93383, 0.92518, 0.91653, 0.90824, 0.90115, 0.89343, 0.88533, 0.87746, 0.86964, 0.86176, 0.85376,\
        0.84564, 0.83744, 0.82922, 0.82109, 0.81319, 0.80580, 0.79951, 0.79472, 0.78942, 0.78332, 0.77652,\
        0.76920, 0.76165, 0.75411, 0.74681, 0.73992, 0.73353, 0.72770, 0.72240, 0.71756, 0.71303, 0.70871,\
        0.70452, 0.70032, 0.69590, 0.69118, 0.68613, 0.68067, 0.67478, 0.66846, 0.66180, 0.65496, 0.64808,\
        0.64134, 0.63488, 0.62883, 0.62328, 0.61828, 0.61383, 0.60988, 0.60637, 0.60319, 0.60019, 0.59712],\
        [0.72703, 0.73465, 0.74206, 0.74926, 0.75628, 0.76314, 0.76985, 0.77646, 0.78299, 0.78952, 0.79609,\
        0.80250, 0.80857, 0.81448, 0.82033, 0.82622, 0.83222, 0.83840, 0.84481, 0.85152, 0.85861, 0.86616,\
        0.87419, 0.88271, 0.89168, 0.90103, 0.91011, 0.91872, 0.92747, 0.93649, 0.94539, 0.95380, 0.96151,\
        0.96845, 0.97464, 0.98016, 0.98523, 0.99012, 0.99518, 1.00075, 1.00705, 1.01409, 1.02161, 1.02905,\
        1.03569, 1.04084, 1.04406, 1.04528, 1.04474, 1.04284, 1.04001, 1.03658, 1.03277, 1.02864, 1.02421,\
        1.01945, 1.01433, 1.00881, 1.00286, 0.99649, 0.98968, 0.98245, 0.97484, 0.96685, 0.95855, 0.94995,\
        0.94112, 0.93217, 0.92328, 0.91474, 0.90653, 0.89804, 0.88980, 0.88174, 0.87363, 0.86532, 0.85678,\
        0.84804, 0.83915, 0.83020, 0.82129, 0.81260, 0.80453, 0.79792, 0.79299, 0.78824, 0.78284, 0.77667,\
        0.76988, 0.76274, 0.75551, 0.74841, 0.74164, 0.73532, 0.72952, 0.72423, 0.71942, 0.71499, 0.71084,\
        0.70685, 0.70286, 0.69869, 0.69420, 0.68942, 0.68413, 0.67826, 0.67183, 0.66496, 0.65782, 0.65061,\
        0.64353, 0.63675, 0.63039, 0.62455, 0.61925, 0.61449, 0.61022, 0.60636, 0.60282, 0.59949, 0.59614],\
        [0.73045, 0.73805, 0.74543, 0.75258, 0.75953, 0.76629, 0.77287, 0.77928, 0.78554, 0.79173, 0.79802,\
        0.80444, 0.81076, 0.81695, 0.82307, 0.82918, 0.83534, 0.84161, 0.84807, 0.85480, 0.86192, 0.86955,\
        0.87779, 0.88661, 0.89593, 0.90592, 0.91616, 0.92615, 0.93591, 0.94547, 0.95459, 0.96301, 0.97053,\
        0.97706, 0.98263, 0.98736, 0.99152, 0.99546, 0.99965, 1.00452, 1.01041, 1.01742, 1.02525, 1.03323,\
        1.04037, 1.04570, 1.04866, 1.04929, 1.04804, 1.04555, 1.04240, 1.03897, 1.03543, 1.03179, 1.02798,\
        1.02390, 1.01947, 1.01462, 1.00929, 1.00343, 0.99702, 0.99005, 0.98254, 0.97453, 0.96608, 0.95726,\
        0.94817, 0.93894, 0.92974, 0.92067, 0.91169, 0.90273, 0.89432, 0.88612, 0.87773, 0.86901, 0.85997,\
        0.85068, 0.84125, 0.83177, 0.82238, 0.81330, 0.80491, 0.79806, 0.79304, 0.78838, 0.78317, 0.77719,\
        0.77059, 0.76361, 0.75649, 0.74947, 0.74274, 0.73645, 0.73067, 0.72542, 0.72070, 0.71642, 0.71248,\
        0.70875, 0.70502, 0.70109, 0.69703, 0.69257, 0.68747, 0.68159, 0.67496, 0.66777, 0.66024, 0.65263,\
        0.64516, 0.63801, 0.63134, 0.62521, 0.61966, 0.61466, 0.61014, 0.60600, 0.60216, 0.59856, 0.59502],\
        [0.73370, 0.74128, 0.74862, 0.75573, 0.76261, 0.76929, 0.77576, 0.78203, 0.78807, 0.79389, 0.79987,\
        0.80634, 0.81292, 0.81940, 0.82579, 0.83214, 0.83848, 0.84489, 0.85141, 0.85815, 0.86527, 0.87295,\
        0.88142, 0.89080, 0.90073, 0.91148, 0.92263, 0.93368, 0.94434, 0.95443, 0.96374, 0.97202, 0.97911,\
        0.98491, 0.98944, 0.99286, 0.99547, 0.99773, 1.00022, 1.00359, 1.00836, 1.01481, 1.02273, 1.03132,\
        1.03916, 1.04480, 1.04746, 1.04738, 1.04542, 1.04251, 1.03939, 1.03646, 1.03378, 1.03121, 1.02855,\
        1.02563, 1.02232, 1.01851, 1.01411, 1.00904, 1.00326, 0.99673, 0.98949, 0.98158, 0.97308, 0.96412,\
        0.95481, 0.94532, 0.93580, 0.92636, 0.91704, 0.90795, 0.89923, 0.89072, 0.88197, 0.87281, 0.86329,\
        0.85353, 0.84369, 0.83391, 0.82436, 0.81526, 0.80694, 0.79999, 0.79490, 0.78994, 0.78440, 0.77817,\
        0.77141, 0.76431, 0.75711, 0.75003, 0.74325, 0.73692, 0.73114, 0.72595, 0.72134, 0.71725, 0.71359,\
        0.71019, 0.70687, 0.70344, 0.69976, 0.69561, 0.69069, 0.68471, 0.67776, 0.67012, 0.66211, 0.65402,\
        0.64611, 0.63858, 0.63160, 0.62523, 0.61949, 0.61434, 0.60966, 0.60534, 0.60133, 0.59755, 0.59384],\
        [0.73677, 0.74434, 0.75165, 0.75872, 0.76555, 0.77216, 0.77857, 0.78480, 0.79085, 0.79654, 0.80219,\
        0.80869, 0.81534, 0.82200, 0.82862, 0.83520, 0.84177, 0.84834, 0.85497, 0.86174, 0.86883, 0.87648,\
        0.88501, 0.89492, 0.90592, 0.91747, 0.92953, 0.94148, 0.95287, 0.96339, 0.97276, 0.98074, 0.98715,\
        0.99187, 0.99493, 0.99648, 0.99684, 0.99654, 0.99635, 0.99717, 0.99983, 1.00490, 1.01244, 1.02163,\
        1.03049, 1.03658, 1.03876, 1.03784, 1.03526, 1.03229, 1.02983, 1.02821, 1.02723, 1.02649, 1.02563,\
        1.02442, 1.02271, 1.02037, 1.01727, 1.01331, 1.00839, 1.00250, 0.99567, 0.98796, 0.97951, 0.97044,\
        0.96095, 0.95120, 0.94140, 0.93169, 0.92219, 0.91302, 0.90415, 0.89536, 0.88621, 0.87656, 0.86656,\
        0.85640, 0.84628, 0.83639, 0.82690, 0.81807, 0.81020, 0.80365, 0.79815, 0.79248, 0.78623, 0.77943,\
        0.77223, 0.76480, 0.75736, 0.75010, 0.74318, 0.73675, 0.73093, 0.72577, 0.72127, 0.71740, 0.71406,\
        0.71108, 0.70828, 0.70543, 0.70225, 0.69850, 0.69375, 0.68749, 0.68006, 0.67187, 0.66329, 0.65467,\
        0.64629, 0.63838, 0.63109, 0.62453, 0.61869, 0.61351, 0.60882, 0.60444, 0.60030, 0.59643, 0.59260],\
        [0.73967, 0.74723, 0.75453, 0.76157, 0.76836, 0.77491, 0.78125, 0.78746, 0.79364, 0.79981, 0.80537,\
        0.81150, 0.81806, 0.82480, 0.83161, 0.83844, 0.84526, 0.85208, 0.85888, 0.86574, 0.87282, 0.88043,\
        0.88894, 0.89851, 0.91071, 0.92374, 0.93686, 0.94963, 0.96161, 0.97240, 0.98167, 0.98915, 0.99460,\
        0.99791, 0.99905, 0.99815, 0.99552, 0.99172, 0.98768, 0.98465, 0.98387, 0.98629, 0.99243, 1.00197,\
        1.01233, 1.01899, 1.02019, 1.01831, 1.01563, 1.01337, 1.01261, 1.01350, 1.01536, 1.01739, 1.01906,\
        1.02017, 1.02059, 1.02019, 1.01881, 1.01627, 1.01248, 1.00742, 1.00112, 0.99371, 0.98533, 0.97618,\
        0.96649, 0.95647, 0.94636, 0.93637, 0.92668, 0.91741, 0.90849, 0.89959, 0.89006, 0.87990, 0.86947,\
        0.85901, 0.84873, 0.83882, 0.82949, 0.82097, 0.81347, 0.80710, 0.80139, 0.79506, 0.78804, 0.78058,\
        0.77284, 0.76498, 0.75719, 0.74965, 0.74252, 0.73594, 0.73003, 0.72486, 0.72046, 0.71681, 0.71379,\
        0.71126, 0.70901, 0.70681, 0.70435, 0.70121, 0.69636, 0.68963, 0.68162, 0.67281, 0.66364, 0.65448,\
        0.64563, 0.63733, 0.62977, 0.62306, 0.61721, 0.61213, 0.60762, 0.60338, 0.59923, 0.59526, 0.59130],\
        [0.74239, 0.74994, 0.75724, 0.76428, 0.77105, 0.77754, 0.78378, 0.78986, 0.79592, 0.80200, 0.80804,\
        0.81428, 0.82086, 0.82771, 0.83471, 0.84181, 0.84896, 0.85612, 0.86324, 0.87031, 0.87743, 0.88507,\
        0.89409, 0.90464, 0.91710, 0.93088, 0.94480, 0.95822, 0.97060, 0.98150, 0.99049, 0.99725, 1.00149,\
        1.00304, 1.00184, 0.99795, 0.99162, 0.98334, 0.97414, 0.96574, 0.95998, 0.95799, 0.96074, 0.96927,\
        0.98174, 0.98899, 0.98781, 0.98561, 0.98455, 0.98424, 0.98685, 0.99198, 0.99811, 1.00393, 1.00886,\
        1.01289, 1.01602, 1.01810, 1.01886, 1.01810, 1.01567, 1.01159, 1.00593, 0.99886, 0.99058, 0.98133,\
        0.97139, 0.96103, 0.95053, 0.94016, 0.93015, 0.92064, 0.91160, 0.90262, 0.89283, 0.88238, 0.87167,\
        0.86103, 0.85068, 0.84082, 0.83163, 0.82328, 0.81591, 0.80945, 0.80335, 0.79671, 0.78928, 0.78127,\
        0.77300, 0.76470, 0.75654, 0.74869, 0.74130, 0.73450, 0.72845, 0.72323, 0.71890, 0.71542, 0.71270,\
        0.71057, 0.70881, 0.70717, 0.70532, 0.70256, 0.69774, 0.69069, 0.68212, 0.67273, 0.66302, 0.65335,\
        0.64406, 0.63540, 0.62760, 0.62079, 0.61498, 0.61011, 0.60597, 0.60212, 0.59807, 0.59395, 0.58985],\
        [0.74491, 0.75247, 0.75978, 0.76683, 0.77361, 0.78009, 0.78624, 0.79208, 0.79792, 0.80402, 0.81032,\
        0.81682, 0.82357, 0.83058, 0.83782, 0.84523, 0.85278, 0.86040, 0.86803, 0.87556, 0.88289, 0.89024,\
        0.89963, 0.91129, 0.92461, 0.93893, 0.95340, 0.96726, 0.97988, 0.99071, 0.99925, 1.00508, 1.00785,\
        1.00732, 1.00337, 0.99600, 0.98538, 0.97181, 0.95608, 0.94055, 0.92836, 0.92017, 0.91673, 0.91972,\
        0.93338, 0.94035, 0.93464, 0.93593, 0.94071, 0.94383, 0.95254, 0.96391, 0.97588, 0.98640, 0.99515,\
        1.00271, 1.00922, 1.01437, 1.01774, 1.01906, 1.01821, 1.01522, 1.01024, 1.00351, 0.99529, 0.98590,\
        0.97565, 0.96485, 0.95384, 0.94292, 0.93237, 0.92241, 0.91306, 0.90390, 0.89420, 0.88369, 0.87287,\
        0.86221, 0.85191, 0.84213, 0.83301, 0.82467, 0.81719, 0.81050, 0.80429, 0.79757, 0.78969, 0.78121,\
        0.77253, 0.76387, 0.75539, 0.74723, 0.73956, 0.73251, 0.72626, 0.72092, 0.71660, 0.71326, 0.71078,\
        0.70895, 0.70750, 0.70617, 0.70467, 0.70244, 0.69784, 0.69028, 0.68127, 0.67149, 0.66137, 0.65131,\
        0.64162, 0.63264, 0.62462, 0.61773, 0.61198, 0.60730, 0.60357, 0.60025, 0.59632, 0.59223, 0.58813],\
        [0.74721, 0.75478, 0.76211, 0.76920, 0.77604, 0.78259, 0.78879, 0.79448, 0.79976, 0.80604, 0.81254,\
        0.81921, 0.82611, 0.83330, 0.84078, 0.84854, 0.85654, 0.86472, 0.87300, 0.88134, 0.88957, 0.89685,\
        0.90570, 0.91856, 0.93274, 0.94762, 0.96250, 0.97664, 0.98937, 1.00001, 1.00797, 1.01270, 1.01376,\
        1.01083, 1.00372, 0.99240, 0.97696, 0.95766, 0.93456, 0.90967, 0.88986, 0.87413, 0.86384, 0.85640,\
        0.85117, 0.85472, 0.85361, 0.86601, 0.88256, 0.89448, 0.91147, 0.93016, 0.94944, 0.96513, 0.97804,\
        0.98986, 1.00060, 1.00950, 1.01593, 1.01958, 1.02040, 1.01852, 1.01420, 1.00776, 0.99955, 0.98995,\
        0.97930, 0.96798, 0.95632, 0.94467, 0.93335, 0.92264, 0.91275, 0.90379, 0.89423, 0.88354, 0.87290,\
        0.86246, 0.85234, 0.84265, 0.83354, 0.82509, 0.81733, 0.81018, 0.80364, 0.79706, 0.78890, 0.78023,\
        0.77137, 0.76250, 0.75378, 0.74537, 0.73742, 0.73009, 0.72357, 0.71805, 0.71366, 0.71041, 0.70813,\
        0.70652, 0.70521, 0.70383, 0.70218, 0.69980, 0.69528, 0.68795, 0.67898, 0.66910, 0.65881, 0.64847,\
        0.63847, 0.62917, 0.62093, 0.61401, 0.60839, 0.60380, 0.60023, 0.59720, 0.59375, 0.58999, 0.58610],\
        [0.74929, 0.75686, 0.76420, 0.77133, 0.77823, 0.78492, 0.79136, 0.79747, 0.80282, 0.80854, 0.81483,\
        0.82146, 0.82842, 0.83573, 0.84342, 0.85151, 0.85997, 0.86874, 0.87772, 0.88683, 0.89615, 0.90515,\
        0.91454, 0.92702, 0.94141, 0.95662, 0.97181, 0.98618, 0.99894, 1.00934, 1.01664, 1.02016, 1.01933,\
        1.01370, 1.00303, 0.98720, 0.96618, 0.94043, 0.91026, 0.87608, 0.84700, 0.82202, 0.80361, 0.78149,\
        0.75148, 0.74040, 0.75492, 0.78403, 0.81393, 0.84101, 0.86563, 0.89143, 0.91891, 0.93983, 0.95744,\
        0.97496, 0.99107, 1.00434, 1.01410, 1.02016, 1.02261, 1.02176, 1.01799, 1.01173, 1.00344, 0.99354,\
        0.98243, 0.97050, 0.95811, 0.94561, 0.93334, 0.92164, 0.91075, 0.90069, 0.89154, 0.88186, 0.87189,\
        0.86191, 0.85206, 0.84249, 0.83333, 0.82470, 0.81664, 0.80901, 0.80152, 0.79460, 0.78693, 0.77844,\
        0.76961, 0.76069, 0.75185, 0.74326, 0.73506, 0.72743, 0.72059, 0.71479, 0.71026, 0.70707, 0.70497,\
        0.70351, 0.70228, 0.70072, 0.69859, 0.69574, 0.69104, 0.68415, 0.67555, 0.66586, 0.65558, 0.64512,\
        0.63488, 0.62527, 0.61676, 0.60984, 0.60455, 0.60025, 0.59675, 0.59383, 0.59078, 0.58745, 0.58388],\
        [0.75111, 0.75867, 0.76601, 0.77315, 0.78008, 0.78683, 0.79337, 0.79960, 0.80528, 0.81089, 0.81694,\
        0.82345, 0.83038, 0.83774, 0.84558, 0.85392, 0.86279, 0.87213, 0.88180, 0.89150, 0.90125, 0.91145,\
        0.92247, 0.93537, 0.95000, 0.96550, 0.98099, 0.99559, 1.00842, 1.01860, 1.02527, 1.02757, 1.02473,\
        1.01615, 1.00151, 0.98070, 0.95333, 0.91830, 0.87714, 0.83551, 0.80413, 0.77164, 0.74162, 0.70058,\
        0.65247, 0.63853, 0.66957, 0.70875, 0.74015, 0.77760, 0.81342, 0.84356, 0.88036, 0.90947, 0.93426,\
        0.95990, 0.98221, 1.00005, 1.01307, 1.02135, 1.02520, 1.02514, 1.02172, 1.01550, 1.00700, 0.99672,\
        0.98511, 0.97256, 0.95945, 0.94609, 0.93284, 0.92009, 0.90835, 0.89797, 0.88870, 0.87971, 0.87045,\
        0.86096, 0.85138, 0.84188, 0.83261, 0.82374, 0.81540, 0.80768, 0.80029, 0.79262, 0.78464, 0.77624,\
        0.76753, 0.75866, 0.74981, 0.74112, 0.73273, 0.72483, 0.71763, 0.71146, 0.70668, 0.70353, 0.70166,\
        0.70031, 0.69905, 0.69739, 0.69492, 0.69150, 0.68657, 0.67988, 0.67162, 0.66222, 0.65210, 0.64166,\
        0.63128, 0.62138, 0.61249, 0.60539, 0.60070, 0.59685, 0.59362, 0.59079, 0.58798, 0.58495, 0.58167],\
        [0.75268, 0.76021, 0.76752, 0.77462, 0.78152, 0.78822, 0.79469, 0.80088, 0.80675, 0.81254, 0.81858,\
        0.82500, 0.83187, 0.83922, 0.84710, 0.85557, 0.86470, 0.87450, 0.88489, 0.89543, 0.90568, 0.91653,\
        0.92869, 0.94249, 0.95772, 0.97373, 0.98964, 1.00457, 1.01756, 1.02766, 1.03384, 1.03505, 1.03027,\
        1.01859, 0.99951, 0.97317, 0.94007, 0.89794, 0.84501, 0.79259, 0.76015, 0.72215, 0.68285, 0.63152,\
        0.56802, 0.55389, 0.59065, 0.64301, 0.67749, 0.71426, 0.75910, 0.79643, 0.84306, 0.88043, 0.91547,\
        0.94835, 0.97598, 0.99782, 1.01360, 1.02359, 1.02841, 1.02878, 1.02544, 1.01905, 1.01023, 0.99953,\
        0.98741, 0.97430, 0.96055, 0.94650, 0.93244, 0.91875, 0.90612, 0.89558, 0.88667, 0.87812, 0.86929,\
        0.86009, 0.85064, 0.84110, 0.83165, 0.82244, 0.81366, 0.80559, 0.79817, 0.79032, 0.78227, 0.77397,\
        0.76541, 0.75667, 0.74789, 0.73920, 0.73073, 0.72264, 0.71513, 0.70852, 0.70336, 0.70023, 0.69861,\
        0.69736, 0.69597, 0.69406, 0.69132, 0.68752, 0.68237, 0.67575, 0.66775, 0.65865, 0.64879, 0.63853,\
        0.62821, 0.61820, 0.60897, 0.60139, 0.59690, 0.59378, 0.59093, 0.58826, 0.58559, 0.58276, 0.57969],\
        [0.75399, 0.76146, 0.76872, 0.77575, 0.78257, 0.78917, 0.79554, 0.80169, 0.80765, 0.81357, 0.81965,\
        0.82604, 0.83282, 0.84009, 0.84789, 0.85632, 0.86545, 0.87538, 0.88617, 0.89761, 0.90821, 0.91970,\
        0.93302, 0.94798, 0.96414, 0.98089, 0.99740, 1.01279, 1.02610, 1.03630, 1.04224, 1.04270, 1.03633,\
        1.02176, 0.99788, 0.96457, 0.92377, 0.87558, 0.82046, 0.76557, 0.72328, 0.67343, 0.61761, 0.54518,\
        0.48414, 0.47870, 0.50084, 0.56026, 0.62488, 0.67044, 0.71527, 0.76449, 0.81448, 0.86210, 0.90461,\
        0.94135, 0.97337, 0.99844, 1.01613, 1.02709, 1.03227, 1.03264, 1.02906, 1.02231, 1.01306, 1.00189,\
        0.98930, 0.97574, 0.96157, 0.94712, 0.93268, 0.91856, 0.90528, 0.89472, 0.88619, 0.87771, 0.86888,\
        0.85966, 0.85013, 0.84045, 0.83075, 0.82116, 0.81183, 0.80296, 0.79522, 0.78778, 0.77999, 0.77188,\
        0.76350, 0.75493, 0.74630, 0.73771, 0.72929, 0.72120, 0.71361, 0.70675, 0.70111, 0.69816, 0.69654,\
        0.69514, 0.69348, 0.69124, 0.68817, 0.68405, 0.67872, 0.67212, 0.66432, 0.65551, 0.64599, 0.63605,\
        0.62604, 0.61633, 0.60737, 0.59997, 0.59528, 0.59197, 0.58911, 0.58644, 0.58379, 0.58103, 0.57807],\
        [0.75504, 0.76245, 0.76964, 0.77659, 0.78331, 0.78979, 0.79605, 0.80215, 0.80816, 0.81414, 0.82023,\
        0.82657, 0.83326, 0.84038, 0.84801, 0.85622, 0.86509, 0.87467, 0.88499, 0.89579, 0.90729, 0.92058,\
        0.93549, 0.95181, 0.96911, 0.98675, 1.00399, 1.01997, 1.03373, 1.04422, 1.05025, 1.05045, 1.04320,\
        1.02665, 0.99875, 0.95802, 0.90627, 0.85018, 0.78380, 0.72272, 0.67276, 0.61491, 0.54353, 0.47202,\
        0.41928, 0.41518, 0.42946, 0.46570, 0.52735, 0.60453, 0.66820, 0.72717, 0.78991, 0.84465, 0.89304,\
        0.93776, 0.97486, 1.00212, 1.02059, 1.03162, 1.03652, 1.03644, 1.03235, 1.02507, 1.01531, 1.00367,\
        0.99069, 0.97681, 0.96246, 0.94797, 0.93370, 0.92002, 0.90756, 0.89703, 0.88767, 0.87858, 0.86933,\
        0.85978, 0.84999, 0.84007, 0.83014, 0.82031, 0.81072, 0.80167, 0.79353, 0.78592, 0.77819, 0.77020,\
        0.76199, 0.75361, 0.74517, 0.73679, 0.72858, 0.72070, 0.71334, 0.70680, 0.70162, 0.69822, 0.69594,\
        0.69399, 0.69184, 0.68916, 0.68569, 0.68127, 0.67578, 0.66917, 0.66153, 0.65302, 0.64386, 0.63437,\
        0.62487, 0.61577, 0.60759, 0.60086, 0.59562, 0.59157, 0.58828, 0.58538, 0.58263, 0.57983, 0.57688],\
        [0.75584, 0.76319, 0.77030, 0.77717, 0.78381, 0.79020, 0.79636, 0.80240, 0.80839, 0.81436, 0.82042,\
        0.82668, 0.83325, 0.84021, 0.84761, 0.85552, 0.86397, 0.87302, 0.88271, 0.89316, 0.90551, 0.92021,\
        0.93659, 0.95425, 0.97269, 0.99126, 1.00924, 1.02582, 1.04009, 1.05101, 1.05739, 1.05782, 1.05059,\
        1.03355, 1.00402, 0.95903, 0.89856, 0.83100, 0.74605, 0.67710, 0.62192, 0.54957, 0.47614, 0.41613,\
        0.36891, 0.35688, 0.37879, 0.41725, 0.45559, 0.52781, 0.61589, 0.68506, 0.75519, 0.82828, 0.89175,\
        0.94207, 0.98077, 1.00827, 1.02623, 1.03649, 1.04057, 1.03972, 1.03493, 1.02704, 1.01675, 1.00468,\
        0.99137, 0.97732, 0.96296, 0.94870, 0.93494, 0.92206, 0.91042, 0.89994, 0.89001, 0.88023, 0.87033,\
        0.86027, 0.85012, 0.83996, 0.82987, 0.81995, 0.81034, 0.80127, 0.79289, 0.78494, 0.77707, 0.76909,\
        0.76098, 0.75278, 0.74457, 0.73646, 0.72856, 0.72104, 0.71410, 0.70802, 0.70313, 0.69942, 0.69645,\
        0.69385, 0.69110, 0.68789, 0.68399, 0.67926, 0.67359, 0.66698, 0.65947, 0.65123, 0.64246, 0.63343,\
        0.62447, 0.61598, 0.60838, 0.60201, 0.59650, 0.59191, 0.58820, 0.58502, 0.58208, 0.57917, 0.57616],\
        [0.75642, 0.76369, 0.77073, 0.77753, 0.78411, 0.79044, 0.79654, 0.80251, 0.80843, 0.81432, 0.82029,\
        0.82646, 0.83292, 0.83972, 0.84692, 0.85454, 0.86264, 0.87128, 0.88067, 0.89138, 0.90420, 0.91945,\
        0.93688, 0.95567, 0.97508, 0.99443, 1.01304, 1.03013, 1.04484, 1.05617, 1.06298, 1.06389, 1.05724,\
        1.04088, 1.01193, 0.96599, 0.89548, 0.79788, 0.69931, 0.63225, 0.57119, 0.49296, 0.42612, 0.36681,\
        0.32035, 0.30071, 0.32348, 0.37246, 0.41667, 0.46502, 0.54855, 0.63418, 0.71529, 0.80874, 0.88862,\
        0.94805, 0.98823, 1.01488, 1.03156, 1.04060, 1.04364, 1.04191, 1.03639, 1.02789, 1.01711, 1.00468,\
        0.99113, 0.97699, 0.96272, 0.94879, 0.93559, 0.92346, 0.91249, 0.90228, 0.89230, 0.88191, 0.87135,\
        0.86079, 0.85031, 0.83995, 0.82980, 0.81992, 0.81041, 0.80134, 0.79284, 0.78462, 0.77653, 0.76849,\
        0.76046, 0.75243, 0.74447, 0.73666, 0.72912, 0.72199, 0.71546, 0.70973, 0.70491, 0.70091, 0.69765,\
        0.69449, 0.69111, 0.68735, 0.68303, 0.67800, 0.67216, 0.66552, 0.65814, 0.65013, 0.64171, 0.63310,\
        0.62461, 0.61653, 0.60917, 0.60268, 0.59712, 0.59250, 0.58860, 0.58520, 0.58207, 0.57901, 0.57589],\
        [0.75678, 0.76397, 0.77093, 0.77767, 0.78420, 0.79054, 0.79666, 0.80255, 0.80829, 0.81403, 0.81990,\
        0.82599, 0.83236, 0.83906, 0.84614, 0.85361, 0.86153, 0.86996, 0.87912, 0.88955, 0.90236, 0.91842,\
        0.93682, 0.95644, 0.97650, 0.99635, 1.01533, 1.03272, 1.04767, 1.05925, 1.06632, 1.06758, 1.06139,\
        1.04566, 1.01742, 0.97145, 0.89438, 0.77860, 0.67371, 0.59432, 0.52105, 0.45249, 0.38676, 0.32669,\
        0.28487, 0.27017, 0.28856, 0.33357, 0.38419, 0.43922, 0.51735, 0.60490, 0.70277, 0.79742, 0.88780,\
        0.95320, 0.99382, 1.01935, 1.03478, 1.04274, 1.04489, 1.04243, 1.03631, 1.02734, 1.01618, 1.00347,\
        0.98975, 0.97556, 0.96140, 0.94773, 0.93499, 0.92347, 0.91323, 0.90379, 0.89369, 0.88277, 0.87178,\
        0.86091, 0.85024, 0.83981, 0.82968, 0.81992, 0.81061, 0.80175, 0.79307, 0.78461, 0.77637, 0.76829,\
        0.76033, 0.75248, 0.74476, 0.73726, 0.73006, 0.72330, 0.71711, 0.71162, 0.70690, 0.70290, 0.69932,\
        0.69555, 0.69162, 0.68740, 0.68271, 0.67741, 0.67143, 0.66475, 0.65746, 0.64965, 0.64152, 0.63327,\
        0.62513, 0.61736, 0.61014, 0.60363, 0.59806, 0.59340, 0.58939, 0.58583, 0.58253, 0.57930, 0.57603],\
        [0.75693, 0.76404, 0.77091, 0.77756, 0.78402, 0.79032, 0.79648, 0.80229, 0.80782, 0.81343, 0.81924,\
        0.82530, 0.83165, 0.83835, 0.84542, 0.85291, 0.86087, 0.86939, 0.87863, 0.88886, 0.90123, 0.91810,\
        0.93702, 0.95692, 0.97714, 0.99709, 1.01611, 1.03349, 1.04841, 1.05993, 1.06695, 1.06816, 1.06188,\
        1.04589, 1.01684, 0.96917, 0.89177, 0.78130, 0.68602, 0.60745, 0.52323, 0.45400, 0.38032, 0.31883,\
        0.28020, 0.27048, 0.29181, 0.33454, 0.38795, 0.44203, 0.51416, 0.60090, 0.70274, 0.79419, 0.89189,\
        0.95559, 0.99467, 1.01962, 1.03457, 1.04206, 1.04376, 1.04091, 1.03445, 1.02519, 1.01381, 1.00092,\
        0.98711, 0.97290, 0.95879, 0.94527, 0.93276, 0.92156, 0.91179, 0.90284, 0.89291, 0.88212, 0.87115,\
        0.86031, 0.84968, 0.83931, 0.82928, 0.81964, 0.81049, 0.80176, 0.79310, 0.78458, 0.77634, 0.76832,\
        0.76048, 0.75281, 0.74533, 0.73811, 0.73123, 0.72478, 0.71885, 0.71353, 0.70882, 0.70465, 0.70073,\
        0.69662, 0.69238, 0.68787, 0.68291, 0.67740, 0.67128, 0.66456, 0.65733, 0.64969, 0.64179, 0.63383,\
        0.62599, 0.61846, 0.61142, 0.60503, 0.59944, 0.59467, 0.59057, 0.58687, 0.58339, 0.57999, 0.57654],\
        [0.75689, 0.76390, 0.77067, 0.77721, 0.78354, 0.78969, 0.79563, 0.80133, 0.80690, 0.81252, 0.81835,\
        0.82444, 0.83085, 0.83762, 0.84480, 0.85246, 0.86069, 0.86963, 0.87948, 0.89063, 0.90364, 0.91955,\
        0.93778, 0.95720, 0.97706, 0.99667, 1.01538, 1.03244, 1.04703, 1.05820, 1.06484, 1.06561, 1.05881,\
        1.04217, 1.01229, 0.96378, 0.89082, 0.80078, 0.71012, 0.63914, 0.57705, 0.50366, 0.41611, 0.35473,\
        0.31409, 0.29794, 0.32593, 0.37647, 0.41990, 0.47763, 0.56136, 0.64081, 0.71530, 0.80050, 0.88421,\
        0.94834, 0.98953, 1.01533, 1.03070, 1.03839, 1.04015, 1.03727, 1.03075, 1.02142, 1.00997, 0.99704,\
        0.98321, 0.96900, 0.95491, 0.94141, 0.92891, 0.91768, 0.90785, 0.89925, 0.89003, 0.87980, 0.86933,\
        0.85885, 0.84851, 0.83837, 0.82850, 0.81896, 0.80982, 0.80117, 0.79268, 0.78437, 0.77631, 0.76845,\
        0.76078, 0.75330, 0.74606, 0.73910, 0.73248, 0.72628, 0.72055, 0.71532, 0.71056, 0.70615, 0.70188,\
        0.69763, 0.69333, 0.68868, 0.68353, 0.67784, 0.67160, 0.66483, 0.65764, 0.65012, 0.64242, 0.63469,\
        0.62710, 0.61981, 0.61297, 0.60668, 0.60106, 0.59632, 0.59211, 0.58825, 0.58459, 0.58099, 0.57736],\
        [0.75668, 0.76360, 0.77026, 0.77667, 0.78283, 0.78878, 0.79453, 0.80014, 0.80567, 0.81136, 0.81728,\
        0.82346, 0.82997, 0.83686, 0.84421, 0.85213, 0.86074, 0.87020, 0.88076, 0.89273, 0.90620, 0.92136,\
        0.93852, 0.95702, 0.97611, 0.99506, 1.01317, 1.02965, 1.04368, 1.05428, 1.06035, 1.06054, 1.05324,\
        1.03643, 1.00770, 0.96477, 0.90582, 0.82606, 0.74140, 0.67223, 0.62241, 0.55653, 0.47590, 0.41588,\
        0.37301, 0.35208, 0.37886, 0.42384, 0.46481, 0.52910, 0.61216, 0.68277, 0.75026, 0.82107, 0.88961,\
        0.94367, 0.98235, 1.00825, 1.02416, 1.03233, 1.03443, 1.03177, 1.02539, 1.01615, 1.00479, 0.99194,\
        0.97818, 0.96401, 0.94994, 0.93641, 0.92388, 0.91263, 0.90239, 0.89389, 0.88544, 0.87624, 0.86655,\
        0.85668, 0.84680, 0.83703, 0.82742, 0.81801, 0.80884, 0.80001, 0.79194, 0.78406, 0.77627, 0.76862,\
        0.76114, 0.75386, 0.74683, 0.74010, 0.73370, 0.72770, 0.72212, 0.71696, 0.71217, 0.70764, 0.70324,\
        0.69896, 0.69459, 0.68979, 0.68448, 0.67863, 0.67226, 0.66543, 0.65825, 0.65082, 0.64327, 0.63574,\
        0.62838, 0.62132, 0.61468, 0.60858, 0.60312, 0.59830, 0.59392, 0.58989, 0.58604, 0.58224, 0.57842],\
        [0.75633, 0.76317, 0.76972, 0.77600, 0.78201, 0.78777, 0.79330, 0.79873, 0.80428, 0.81008, 0.81612,\
        0.82241, 0.82902, 0.83603, 0.84355, 0.85171, 0.86065, 0.87056, 0.88158, 0.89376, 0.90720, 0.92203,\
        0.93835, 0.95590, 0.97409, 0.99220, 1.00953, 1.02528, 1.03861, 1.04858, 1.05407, 1.05383, 1.04637,\
        1.03003, 1.00313, 0.96446, 0.91373, 0.85331, 0.78893, 0.71946, 0.67125, 0.61329, 0.53685, 0.47233,\
        0.43274, 0.40990, 0.42408, 0.46986, 0.53510, 0.60366, 0.65710, 0.71483, 0.78091, 0.84126, 0.89597,\
        0.94095, 0.97572, 1.00059, 1.01651, 1.02493, 1.02731, 1.02489, 1.01872, 1.00967, 0.99849, 0.98582,\
        0.97223, 0.95823, 0.94425, 0.93071, 0.91815, 0.90726, 0.89796, 0.88910, 0.88082, 0.87226, 0.86329,\
        0.85405, 0.84472, 0.83539, 0.82616, 0.81706, 0.80816, 0.79957, 0.79162, 0.78390, 0.77629, 0.76881,\
        0.76150, 0.75440, 0.74755, 0.74101, 0.73480, 0.72897, 0.72352, 0.71843, 0.71366, 0.70914, 0.70479,\
        0.70053, 0.69600, 0.69111, 0.68565, 0.67962, 0.67311, 0.66621, 0.65902, 0.65164, 0.64421, 0.63686,\
        0.62969, 0.62285, 0.61642, 0.61049, 0.60511, 0.60022, 0.59580, 0.59167, 0.58763, 0.58364, 0.57963],\
        [0.75585, 0.76263, 0.76911, 0.77530, 0.78120, 0.78682, 0.79221, 0.79747, 0.80300, 0.80890, 0.81500,\
        0.82133, 0.82798, 0.83506, 0.84269, 0.85100, 0.86014, 0.87026, 0.88152, 0.89400, 0.90716, 0.92123,\
        0.93681, 0.95354, 0.97084, 0.98807, 1.00455, 1.01952, 1.03217, 1.04156, 1.04667, 1.04633, 1.03928,\
        1.02414, 0.99963, 0.96470, 0.91974, 0.87308, 0.81898, 0.76526, 0.72517, 0.67596, 0.61195, 0.55137,\
        0.50600, 0.48096, 0.50233, 0.55813, 0.62255, 0.66892, 0.70697, 0.75613, 0.81480, 0.86138, 0.90094,\
        0.93939, 0.97110, 0.99423, 1.00927, 1.01732, 1.01960, 1.01721, 1.01116, 1.00229, 0.99134, 0.97894,\
        0.96566, 0.95198, 0.93830, 0.92496, 0.91233, 0.90129, 0.89269, 0.88460, 0.87658, 0.86840, 0.85993,\
        0.85123, 0.84241, 0.83357, 0.82479, 0.81614, 0.80773, 0.79962, 0.79162, 0.78388, 0.77634, 0.76897,\
        0.76178, 0.75482, 0.74813, 0.74174, 0.73570, 0.73001, 0.72468, 0.71968, 0.71496, 0.71048, 0.70616,\
        0.70183, 0.69748, 0.69254, 0.68686, 0.68064, 0.67400, 0.66701, 0.65980, 0.65246, 0.64513, 0.63791,\
        0.63093, 0.62427, 0.61803, 0.61227, 0.60699, 0.60218, 0.59779, 0.59348, 0.58926, 0.58509, 0.58091],\
        [0.75527, 0.76200, 0.76845, 0.77461, 0.78048, 0.78607, 0.79143, 0.79671, 0.80220, 0.80798, 0.81399,\
        0.82025, 0.82685, 0.83391, 0.84155, 0.84988, 0.85902, 0.86907, 0.88011, 0.89212, 0.90480, 0.91864,\
        0.93382, 0.94988, 0.96635, 0.98272, 0.99837, 1.01261, 1.02465, 1.03365, 1.03866, 1.03869, 1.03270,\
        1.01971, 0.99899, 0.97050, 0.93602, 0.89707, 0.85006, 0.80504, 0.76849, 0.72899, 0.68264, 0.63629,\
        0.59316, 0.55807, 0.58770, 0.64051, 0.68299, 0.71941, 0.75232, 0.79012, 0.84484, 0.88791, 0.91370,\
        0.94454, 0.97100, 0.99067, 1.00356, 1.01035, 1.01194, 1.00924, 1.00312, 0.99435, 0.98362, 0.97155,\
        0.95870, 0.94554, 0.93249, 0.91984, 0.90784, 0.89714, 0.88857, 0.88070, 0.87284, 0.86485, 0.85668,\
        0.84837, 0.83998, 0.83161, 0.82329, 0.81508, 0.80703, 0.79917, 0.79141, 0.78377, 0.77630, 0.76900,\
        0.76189, 0.75504, 0.74846, 0.74222, 0.73631, 0.73076, 0.72554, 0.72063, 0.71598, 0.71153, 0.70727,\
        0.70322, 0.69908, 0.69376, 0.68787, 0.68151, 0.67476, 0.66769, 0.66045, 0.65314, 0.64588, 0.63878,\
        0.63195, 0.62547, 0.61941, 0.61379, 0.60865, 0.60397, 0.59954, 0.59509, 0.59076, 0.58650, 0.58219],\
        [0.75456, 0.76129, 0.76774, 0.77392, 0.77983, 0.78548, 0.79092, 0.79625, 0.80165, 0.80725, 0.81306,\
        0.81912, 0.82556, 0.83251, 0.84006, 0.84831, 0.85732, 0.86712, 0.87768, 0.88887, 0.90090, 0.91490,\
        0.92973, 0.94509, 0.96073, 0.97626, 0.99115, 1.00475, 1.01634, 1.02516, 1.03040, 1.03122, 1.02686,\
        1.01671, 1.00047, 0.97853, 0.95184, 0.92050, 0.88456, 0.84843, 0.81803, 0.79300, 0.76338, 0.71517,\
        0.66706, 0.64782, 0.66877, 0.70174, 0.73285, 0.76637, 0.79814, 0.83060, 0.87545, 0.91400, 0.93702,\
        0.95705, 0.97562, 0.99012, 0.99968, 1.00437, 1.00470, 1.00132, 0.99490, 0.98612, 0.97558, 0.96386,\
        0.95151, 0.93904, 0.92691, 0.91543, 0.90472, 0.89487, 0.88597, 0.87772, 0.86967, 0.86163, 0.85357,\
        0.84550, 0.83746, 0.82950, 0.82163, 0.81383, 0.80605, 0.79839, 0.79092, 0.78346, 0.77607, 0.76880,\
        0.76175, 0.75495, 0.74848, 0.74234, 0.73657, 0.73114, 0.72605, 0.72124, 0.71665, 0.71222, 0.70785,\
        0.70353, 0.69925, 0.69420, 0.68845, 0.68209, 0.67525, 0.66812, 0.66084, 0.65354, 0.64635, 0.63936,\
        0.63267, 0.62635, 0.62044, 0.61496, 0.60991, 0.60521, 0.60071, 0.59634, 0.59207, 0.58780, 0.58343],\
        [0.75372, 0.76047, 0.76695, 0.77319, 0.77919, 0.78497, 0.79053, 0.79589, 0.80118, 0.80659, 0.81210,\
        0.81786, 0.82406, 0.83082, 0.83823, 0.84632, 0.85514, 0.86468, 0.87493, 0.88591, 0.89790, 0.91111,\
        0.92497, 0.93935, 0.95410, 0.96884, 0.98305, 0.99613, 1.00745, 1.01630, 1.02203, 1.02396, 1.02156,\
        1.01445, 1.00259, 0.98629, 0.96612, 0.94264, 0.91715, 0.89014, 0.86633, 0.85105, 0.83592, 0.80380,\
        0.75893, 0.73896, 0.74448, 0.76397, 0.78785, 0.81665, 0.84622, 0.87795, 0.91076, 0.93742, 0.95602,\
        0.97020, 0.98214, 0.99134, 0.99710, 0.99923, 0.99792, 0.99357, 0.98668, 0.97778, 0.96739, 0.95603,\
        0.94421, 0.93246, 0.92135, 0.91130, 0.90194, 0.89286, 0.88391, 0.87524, 0.86684, 0.85860, 0.85050,\
        0.84256, 0.83478, 0.82720, 0.81980, 0.81247, 0.80510, 0.79771, 0.79036, 0.78295, 0.77555, 0.76827,\
        0.76122, 0.75448, 0.74807, 0.74204, 0.73639, 0.73109, 0.72613, 0.72145, 0.71697, 0.71260, 0.70817,\
        0.70348, 0.69888, 0.69435, 0.68875, 0.68233, 0.67540, 0.66818, 0.66086, 0.65356, 0.64643, 0.63954,\
        0.63298, 0.62681, 0.62106, 0.61573, 0.61078, 0.60612, 0.60165, 0.59739, 0.59323, 0.58900, 0.58459],\
        [0.75273, 0.75949, 0.76602, 0.77234, 0.77845, 0.78437, 0.79012, 0.79565, 0.80090, 0.80593, 0.81096,\
        0.81634, 0.82226, 0.82881, 0.83605, 0.84397, 0.85259, 0.86189, 0.87192, 0.88278, 0.89460, 0.90685,\
        0.91937, 0.93269, 0.94659, 0.96061, 0.97424, 0.98694, 0.99811, 1.00717, 1.01356, 1.01678, 1.01642,\
        1.01227, 1.00437, 0.99296, 0.97852, 0.96161, 0.94291, 0.92282, 0.90603, 0.89536, 0.88446, 0.87137,\
        0.85090, 0.83006, 0.83387, 0.84809, 0.85620, 0.86976, 0.89221, 0.91770, 0.94028, 0.95846, 0.97164,\
        0.98109, 0.98806, 0.99275, 0.99494, 0.99447, 0.99140, 0.98596, 0.97851, 0.96944, 0.95920, 0.94821,\
        0.93693, 0.92582, 0.91550, 0.90702, 0.89975, 0.89094, 0.88175, 0.87273, 0.86399, 0.85552, 0.84733,\
        0.83942, 0.83183, 0.82459, 0.81766, 0.81092, 0.80409, 0.79699, 0.78963, 0.78212, 0.77462, 0.76728,\
        0.76022, 0.75350, 0.74717, 0.74124, 0.73571, 0.73055, 0.72573, 0.72120, 0.71689, 0.71271, 0.70856,\
        0.70433, 0.69992, 0.69485, 0.68886, 0.68219, 0.67508, 0.66776, 0.66038, 0.65310, 0.64602, 0.63924,\
        0.63282, 0.62681, 0.62124, 0.61609, 0.61131, 0.60682, 0.60252, 0.59840, 0.59434, 0.59012, 0.58567],\
        [0.75154, 0.75833, 0.76489, 0.77126, 0.77745, 0.78348, 0.78938, 0.79513, 0.80048, 0.80484, 0.80936,\
        0.81440, 0.82009, 0.82648, 0.83357, 0.84132, 0.84971, 0.85873, 0.86840, 0.87876, 0.88978, 0.90099,\
        0.91256, 0.92513, 0.93834, 0.95175, 0.96491, 0.97731, 0.98846, 0.99783, 1.00497, 1.00949, 1.01110,\
        1.00968, 1.00529, 0.99814, 0.98861, 0.97720, 0.96450, 0.95168, 0.94140, 0.93463, 0.93003, 0.93071,\
        0.93318, 0.93332, 0.93518, 0.93541, 0.92636, 0.92457, 0.93478, 0.95032, 0.96471, 0.97611, 0.98416,\
        0.98933, 0.99228, 0.99330, 0.99242, 0.98959, 0.98486, 0.97837, 0.97036, 0.96115, 0.95109, 0.94053,\
        0.92987, 0.91947, 0.90971, 0.90123, 0.89602, 0.88794, 0.87886, 0.86975, 0.86083, 0.85219, 0.84389,\
        0.83597, 0.82849, 0.82149, 0.81498, 0.80891, 0.80284, 0.79595, 0.78843, 0.78072, 0.77310, 0.76570,\
        0.75863, 0.75195, 0.74569, 0.73987, 0.73447, 0.72946, 0.72478, 0.72040, 0.71627, 0.71232, 0.70850,\
        0.70472, 0.70057, 0.69496, 0.68845, 0.68145, 0.67415, 0.66674, 0.65934, 0.65208, 0.64507, 0.63840,\
        0.63213, 0.62631, 0.62094, 0.61602, 0.61149, 0.60727, 0.60326, 0.59933, 0.59537, 0.59114, 0.58661],\
        [0.75014, 0.75693, 0.76351, 0.76990, 0.77610, 0.78212, 0.78795, 0.79351, 0.79842, 0.80265, 0.80713,\
        0.81200, 0.81755, 0.82386, 0.83086, 0.83847, 0.84662, 0.85527, 0.86441, 0.87401, 0.88396, 0.89412,\
        0.90505, 0.91704, 0.92963, 0.94249, 0.95523, 0.96742, 0.97858, 0.98830, 0.99618, 1.00194, 1.00536,\
        1.00637, 1.00504, 1.00159, 0.99635, 0.98982, 0.98270, 0.97616, 0.97174, 0.97044, 0.97292, 0.97988,\
        0.98869, 0.99475, 0.99610, 0.99094, 0.98035, 0.97285, 0.97306, 0.97850, 0.98503, 0.99033, 0.99362,\
        0.99490, 0.99443, 0.99243, 0.98900, 0.98419, 0.97803, 0.97064, 0.96220, 0.95292, 0.94311, 0.93308,\
        0.92315, 0.91369, 0.90519, 0.89822, 0.89121, 0.88362, 0.87506, 0.86613, 0.85721, 0.84849, 0.84009,\
        0.83212, 0.82468, 0.81782, 0.81150, 0.80578, 0.80053, 0.79393, 0.78626, 0.77847, 0.77081, 0.76340,\
        0.75636, 0.74975, 0.74359, 0.73789, 0.73263, 0.72777, 0.72324, 0.71898, 0.71495, 0.71111, 0.70741,\
        0.70372, 0.69936, 0.69367, 0.68701, 0.67986, 0.67248, 0.66503, 0.65764, 0.65044, 0.64352, 0.63698,\
        0.63088, 0.62525, 0.62012, 0.61546, 0.61124, 0.60736, 0.60373, 0.60018, 0.59622, 0.59194, 0.58736],\
        [0.74851, 0.75529, 0.76187, 0.76824, 0.77440, 0.78032, 0.78594, 0.79114, 0.79579, 0.80015, 0.80452,\
        0.80921, 0.81473, 0.82109, 0.82810, 0.83559, 0.84347, 0.85170, 0.86026, 0.86912, 0.87823, 0.88768,\
        0.89785, 0.90901, 0.92083, 0.93308, 0.94543, 0.95740, 0.96859, 0.97861, 0.98716, 0.99401, 0.99901,\
        1.00213, 1.00343, 1.00314, 1.00158, 0.99926, 0.99687, 0.99536, 0.99577, 0.99896, 1.00536, 1.01450,\
        1.02402, 1.03049, 1.03141, 1.02603, 1.01649, 1.00749, 1.00231, 1.00075, 1.00085, 1.00087, 0.99994,\
        0.99776, 0.99437, 0.98988, 0.98440, 0.97799, 0.97072, 0.96267, 0.95396, 0.94475, 0.93528, 0.92586,\
        0.91677, 0.90819, 0.90050, 0.89363, 0.88664, 0.87902, 0.87074, 0.86204, 0.85321, 0.84446, 0.83598,\
        0.82791, 0.82043, 0.81365, 0.80745, 0.80145, 0.79612, 0.78997, 0.78278, 0.77522, 0.76767, 0.76035,\
        0.75339, 0.74686, 0.74083, 0.73528, 0.73018, 0.72548, 0.72108, 0.71691, 0.71290, 0.70903, 0.70515,\
        0.70118, 0.69675, 0.69102, 0.68444, 0.67735, 0.67001, 0.66260, 0.65528, 0.64816, 0.64136, 0.63497,\
        0.62904, 0.62363, 0.61875, 0.61438, 0.61048, 0.60696, 0.60365, 0.60028, 0.59658, 0.59242, 0.58781],\
        [0.74662, 0.75339, 0.75995, 0.76629, 0.77240, 0.77822, 0.78367, 0.78869, 0.79325, 0.79755, 0.80167,\
        0.80615, 0.81188, 0.81845, 0.82550, 0.83286, 0.84045, 0.84825, 0.85625, 0.86444, 0.87287, 0.88167,\
        0.89109, 0.90128, 0.91212, 0.92377, 0.93572, 0.94745, 0.95858, 0.96881, 0.97790, 0.98565, 0.99196,\
        0.99682, 1.00030, 1.00262, 1.00411, 1.00521, 1.00652, 1.00877, 1.01264, 1.01863, 1.02674, 1.03610,\
        1.04479, 1.05037, 1.05103, 1.04646, 1.03824, 1.02919, 1.02152, 1.01582, 1.01142, 1.00735, 1.00294,\
        0.99787, 0.99205, 0.98555, 0.97846, 0.97085, 0.96280, 0.95438, 0.94563, 0.93666, 0.92763, 0.91887,\
        0.91078, 0.90312, 0.89593, 0.88911, 0.88201, 0.87441, 0.86626, 0.85772, 0.84898, 0.84023, 0.83166,\
        0.82345, 0.81583, 0.80906, 0.80339, 0.79786, 0.79170, 0.78537, 0.77844, 0.77114, 0.76377, 0.75658,\
        0.74973, 0.74332, 0.73743, 0.73204, 0.72714, 0.72262, 0.71837, 0.71425, 0.71018, 0.70621, 0.70232,\
        0.69782, 0.69306, 0.68741, 0.68097, 0.67402, 0.66679, 0.65949, 0.65227, 0.64527, 0.63860, 0.63237,\
        0.62664, 0.62145, 0.61683, 0.61275, 0.60920, 0.60606, 0.60313, 0.60005, 0.59661, 0.59257, 0.58795],\
        [0.74448, 0.75122, 0.75777, 0.76411, 0.77020, 0.77599, 0.78139, 0.78635, 0.79084, 0.79489, 0.79861,\
        0.80326, 0.80953, 0.81632, 0.82334, 0.83050, 0.83774, 0.84507, 0.85252, 0.86012, 0.86792, 0.87606,\
        0.88466, 0.89379, 0.90374, 0.91488, 0.92638, 0.93772, 0.94865, 0.95894, 0.96839, 0.97683, 0.98416,\
        0.99037, 0.99557, 0.99994, 1.00378, 1.00748, 1.01149, 1.01631, 1.02236, 1.02981, 1.03837, 1.04716,\
        1.05469, 1.05928, 1.05975, 1.05593, 1.04885, 1.04024, 1.03162, 1.02372, 1.01650, 1.00958, 1.00257,\
        0.99524, 0.98752, 0.97945, 0.97115, 0.96271, 0.95423, 0.94575, 0.93726, 0.92878, 0.92032, 0.91212,\
        0.90488, 0.89891, 0.89193, 0.88483, 0.87754, 0.86988, 0.86179, 0.85337, 0.84472, 0.83600, 0.82736,\
        0.81900, 0.81115, 0.80407, 0.79836, 0.79307, 0.78703, 0.78055, 0.77368, 0.76653, 0.75931, 0.75223,\
        0.74548, 0.73918, 0.73343, 0.72822, 0.72353, 0.71924, 0.71518, 0.71115, 0.70696, 0.70265, 0.69867,\
        0.69419, 0.68901, 0.68324, 0.67687, 0.67004, 0.66295, 0.65577, 0.64868, 0.64181, 0.63530, 0.62923,\
        0.62370, 0.61875, 0.61439, 0.61062, 0.60740, 0.60470, 0.60238, 0.59980, 0.59650, 0.59242, 0.58771],\
        [0.74207, 0.74880, 0.75535, 0.76171, 0.76785, 0.77371, 0.77922, 0.78431, 0.78894, 0.79313, 0.79720,\
        0.80220, 0.80838, 0.81503, 0.82181, 0.82860, 0.83541, 0.84224, 0.84914, 0.85617, 0.86339, 0.87090,\
        0.87881, 0.88724, 0.89650, 0.90687, 0.91762, 0.92832, 0.93885, 0.94905, 0.95868, 0.96756, 0.97559,\
        0.98278, 0.98922, 0.99509, 1.00062, 1.00612, 1.01193, 1.01835, 1.02557, 1.03355, 1.04192, 1.04986,\
        1.05628, 1.06003, 1.06032, 1.05706, 1.05089, 1.04288, 1.03409, 1.02519, 1.01643, 1.00773, 0.99895,\
        0.99000, 0.98088, 0.97167, 0.96252, 0.95360, 0.94501, 0.93679, 0.92889, 0.92125, 0.91376, 0.90619,\
        0.89938, 0.89380, 0.88754, 0.88053, 0.87314, 0.86543, 0.85740, 0.84909, 0.84057, 0.83194, 0.82331,\
        0.81486, 0.80691, 0.79979, 0.79369, 0.78811, 0.78213, 0.77565, 0.76879, 0.76167, 0.75450, 0.74746,\
        0.74076, 0.73454, 0.72890, 0.72387, 0.71940, 0.71538, 0.71159, 0.70777, 0.70358, 0.69906, 0.69457,\
        0.68986, 0.68457, 0.67871, 0.67235, 0.66561, 0.65863, 0.65158, 0.64462, 0.63789, 0.63152, 0.62562,\
        0.62030, 0.61559, 0.61151, 0.60804, 0.60514, 0.60282, 0.60132, 0.59943, 0.59606, 0.59181, 0.58698],\
        [0.73938, 0.74608, 0.75264, 0.75907, 0.76532, 0.77137, 0.77715, 0.78260, 0.78768, 0.79244, 0.79719,\
        0.80244, 0.80833, 0.81456, 0.82087, 0.82716, 0.83343, 0.83972, 0.84608, 0.85257, 0.85925, 0.86622,\
        0.87358, 0.88150, 0.89018, 0.89965, 0.90938, 0.91920, 0.92921, 0.93919, 0.94883, 0.95791, 0.96633,\
        0.97411, 0.98134, 0.98816, 0.99477, 1.00140, 1.00825, 1.01550, 1.02317, 1.03113, 1.03896, 1.04601,\
        1.05146, 1.05451, 1.05464, 1.05178, 1.04633, 1.03897, 1.03044, 1.02130, 1.01186, 1.00222, 0.99240,\
        0.98242, 0.97237, 0.96241, 0.95274, 0.94359, 0.93517, 0.92751, 0.92047, 0.91399, 0.90812, 0.90276,\
        0.89641, 0.88996, 0.88324, 0.87611, 0.86867, 0.86099, 0.85307, 0.84494, 0.83663, 0.82821, 0.81974,\
        0.81134, 0.80322, 0.79621, 0.78991, 0.78384, 0.77758, 0.77095, 0.76398, 0.75679, 0.74955, 0.74246,\
        0.73573, 0.72952, 0.72395, 0.71906, 0.71481, 0.71106, 0.70756, 0.70409, 0.70007, 0.69534, 0.69046,\
        0.68537, 0.67987, 0.67393, 0.66757, 0.66088, 0.65400, 0.64705, 0.64021, 0.63360, 0.62736, 0.62163,\
        0.61651, 0.61205, 0.60827, 0.60514, 0.60261, 0.60056, 0.59964, 0.59821, 0.59474, 0.59045, 0.58560],\
        [0.73638, 0.74305, 0.74963, 0.75612, 0.76255, 0.76887, 0.77506, 0.78105, 0.78679, 0.79228, 0.79764,\
        0.80314, 0.80882, 0.81457, 0.82029, 0.82598, 0.83167, 0.83740, 0.84323, 0.84921, 0.85540, 0.86189,\
        0.86875, 0.87613, 0.88411, 0.89264, 0.90137, 0.91022, 0.91975, 0.92948, 0.93898, 0.94800, 0.95649,\
        0.96448, 0.97207, 0.97937, 0.98653, 0.99371, 1.00101, 1.00851, 1.01615, 1.02374, 1.03088, 1.03706,\
        1.04167, 1.04415, 1.04414, 1.04159, 1.03675, 1.03006, 1.02204, 1.01313, 1.00362, 0.99368, 0.98342,\
        0.97294, 0.96238, 0.95199, 0.94205, 0.93289, 0.92482, 0.91801, 0.91196, 0.90655, 0.90207, 0.89871,\
        0.89318, 0.88593, 0.87869, 0.87139, 0.86399, 0.85644, 0.84874, 0.84089, 0.83289, 0.82481, 0.81671,\
        0.80871, 0.80084, 0.79359, 0.78686, 0.78029, 0.77360, 0.76667, 0.75948, 0.75210, 0.74468, 0.73742,\
        0.73055, 0.72428, 0.71873, 0.71393, 0.70984, 0.70636, 0.70302, 0.69963, 0.69582, 0.69110, 0.68603,\
        0.68070, 0.67504, 0.66901, 0.66264, 0.65599, 0.64917, 0.64232, 0.63557, 0.62907, 0.62296, 0.61738,\
        0.61244, 0.60824, 0.60477, 0.60196, 0.59988, 0.59821, 0.59703, 0.59532, 0.59225, 0.58821, 0.58352],\
        [0.73306, 0.73966, 0.74622, 0.75279, 0.75938, 0.76601, 0.77267, 0.77931, 0.78585, 0.79218, 0.79812,\
        0.80387, 0.80938, 0.81461, 0.81971, 0.82477, 0.82988, 0.83509, 0.84044, 0.84597, 0.85171, 0.85775,\
        0.86414, 0.87094, 0.87818, 0.88576, 0.89348, 0.90158, 0.91077, 0.92016, 0.92928, 0.93795, 0.94619,\
        0.95405, 0.96162, 0.96899, 0.97626, 0.98353, 0.99084, 0.99819, 1.00547, 1.01248, 1.01887, 1.02424,\
        1.02812, 1.03012, 1.02999, 1.02770, 1.02341, 1.01740, 1.01006, 1.00169, 0.99256, 0.98282, 0.97262,\
        0.96207, 0.95138, 0.94084, 0.93088, 0.92185, 0.91412, 0.90848, 0.90365, 0.89873, 0.89506, 0.89143,\
        0.88658, 0.88017, 0.87321, 0.86610, 0.85893, 0.85169, 0.84435, 0.83689, 0.82931, 0.82164, 0.81397,\
        0.80650, 0.79932, 0.79167, 0.78439, 0.77731, 0.77019, 0.76290, 0.75540, 0.74775, 0.74008, 0.73256,\
        0.72544, 0.71901, 0.71346, 0.70872, 0.70464, 0.70144, 0.69825, 0.69474, 0.69085, 0.68632, 0.68128,\
        0.67587, 0.67012, 0.66404, 0.65767, 0.65105, 0.64430, 0.63752, 0.63085, 0.62445, 0.61846, 0.61301,\
        0.60822, 0.60427, 0.60117, 0.59866, 0.59676, 0.59524, 0.59378, 0.59184, 0.58899, 0.58526, 0.58081],\
        [0.72938, 0.73587, 0.74237, 0.74894, 0.75566, 0.76256, 0.76966, 0.77693, 0.78430, 0.79159, 0.79852,\
        0.80437, 0.80952, 0.81419, 0.81868, 0.82318, 0.82779, 0.83257, 0.83752, 0.84267, 0.84804, 0.85368,\
        0.85963, 0.86591, 0.87253, 0.87941, 0.88655, 0.89422, 0.90269, 0.91144, 0.91986, 0.92787, 0.93557,\
        0.94302, 0.95026, 0.95735, 0.96438, 0.97140, 0.97839, 0.98531, 0.99203, 0.99834, 1.00397, 1.00857,\
        1.01181, 1.01341, 1.01319, 1.01114, 1.00736, 1.00206, 0.99550, 0.98792, 0.97949, 0.97035, 0.96062,\
        0.95043, 0.93995, 0.92951, 0.91979, 0.91121, 0.90375, 0.89883, 0.89552, 0.89111, 0.88729, 0.88344,\
        0.87877, 0.87308, 0.86675, 0.86013, 0.85342, 0.84666, 0.83983, 0.83289, 0.82582, 0.81860, 0.81128,\
        0.80395, 0.79675, 0.78950, 0.78212, 0.77472, 0.76725, 0.75963, 0.75184, 0.74391, 0.73594, 0.72810,\
        0.72063, 0.71391, 0.70835, 0.70384, 0.69981, 0.69645, 0.69334, 0.68982, 0.68586, 0.68137, 0.67638,\
        0.67097, 0.66521, 0.65912, 0.65276, 0.64619, 0.63949, 0.63278, 0.62619, 0.61987, 0.61401, 0.60873,\
        0.60409, 0.60028, 0.59753, 0.59530, 0.59348, 0.59190, 0.59024, 0.58816, 0.58539, 0.58187, 0.57766],\
        [0.72535, 0.73165, 0.73800, 0.74448, 0.75122, 0.75827, 0.76569, 0.77346, 0.78152, 0.78962, 0.79718,\
        0.80364, 0.80852, 0.81268, 0.81672, 0.82083, 0.82512, 0.82962, 0.83431, 0.83918, 0.84426, 0.84958,\
        0.85516, 0.86102, 0.86716, 0.87358, 0.88034, 0.88757, 0.89532, 0.90328, 0.91069, 0.91777, 0.92478,\
        0.93162, 0.93827, 0.94483, 0.95136, 0.95788, 0.96434, 0.97064, 0.97666, 0.98222, 0.98709, 0.99099,\
        0.99366, 0.99490, 0.99461, 0.99278, 0.98950, 0.98493, 0.97924, 0.97261, 0.96514, 0.95691, 0.94802,\
        0.93861, 0.92876, 0.91862, 0.90922, 0.90170, 0.89514, 0.89031, 0.88676, 0.88301, 0.87918, 0.87516,\
        0.87060, 0.86536, 0.85961, 0.85360, 0.84749, 0.84135, 0.83517, 0.82887, 0.82240, 0.81569, 0.80875,\
        0.80167, 0.79457, 0.78738, 0.78000, 0.77244, 0.76474, 0.75687, 0.74884, 0.74068, 0.73247, 0.72433,\
        0.71646, 0.70926, 0.70354, 0.69943, 0.69561, 0.69211, 0.68874, 0.68511, 0.68106, 0.67654, 0.67156,\
        0.66616, 0.66040, 0.65433, 0.64801, 0.64149, 0.63487, 0.62824, 0.62172, 0.61548, 0.60973, 0.60470,\
        0.60038, 0.59679, 0.59409, 0.59196, 0.59014, 0.58844, 0.58662, 0.58442, 0.58167, 0.57827, 0.57424],\
        [0.72096, 0.72699, 0.73307, 0.73935, 0.74594, 0.75297, 0.76050, 0.76853, 0.77700, 0.78568, 0.79420,\
        0.80103, 0.80541, 0.80949, 0.81342, 0.81742, 0.82165, 0.82608, 0.83067, 0.83539, 0.84028, 0.84536,\
        0.85066, 0.85620, 0.86199, 0.86804, 0.87439, 0.88110, 0.88811, 0.89511, 0.90146, 0.90769, 0.91401,\
        0.92011, 0.92599, 0.93180, 0.93766, 0.94356, 0.94934, 0.95488, 0.96013, 0.96493, 0.96908, 0.97233,\
        0.97448, 0.97540, 0.97505, 0.97343, 0.97062, 0.96677, 0.96202, 0.95647, 0.95014, 0.94305, 0.93526,\
        0.92698, 0.91844, 0.90933, 0.89965, 0.89323, 0.88745, 0.88266, 0.87868, 0.87489, 0.87100, 0.86687,\
        0.86235, 0.85739, 0.85211, 0.84667, 0.84121, 0.83578, 0.83036, 0.82483, 0.81907, 0.81296, 0.80646,\
        0.79967, 0.79272, 0.78554, 0.77811, 0.77044, 0.76258, 0.75455, 0.74638, 0.73810, 0.72979, 0.72153,\
        0.71347, 0.70587, 0.69950, 0.69563, 0.69200, 0.68838, 0.68472, 0.68085, 0.67663, 0.67201, 0.66697,\
        0.66156, 0.65581, 0.64976, 0.64349, 0.63705, 0.63053, 0.62401, 0.61760, 0.61143, 0.60571, 0.60090,\
        0.59698, 0.59364, 0.59096, 0.58878, 0.58686, 0.58501, 0.58303, 0.58072, 0.57793, 0.57459, 0.57067],\
        [0.71627, 0.72192, 0.72762, 0.73352, 0.73981, 0.74661, 0.75400, 0.76198, 0.77045, 0.77915, 0.78749,\
        0.79433, 0.79965, 0.80436, 0.80854, 0.81278, 0.81725, 0.82187, 0.82653, 0.83122, 0.83600, 0.84093,\
        0.84604, 0.85135, 0.85687, 0.86258, 0.86851, 0.87463, 0.88082, 0.88683, 0.89245, 0.89799, 0.90358,\
        0.90881, 0.91375, 0.91868, 0.92380, 0.92903, 0.93405, 0.93874, 0.94315, 0.94723, 0.95071, 0.95336,\
        0.95503, 0.95564, 0.95523, 0.95382, 0.95146, 0.94828, 0.94447, 0.94010, 0.93505, 0.92923, 0.92263,\
        0.91552, 0.90864, 0.90194, 0.89366, 0.88629, 0.88042, 0.87548, 0.87116, 0.86710, 0.86301, 0.85873,\
        0.85418, 0.84938, 0.84442, 0.83947, 0.83465, 0.82998, 0.82541, 0.82076, 0.81582, 0.81041, 0.80442,\
        0.79793, 0.79110, 0.78391, 0.77641, 0.76865, 0.76070, 0.75260, 0.74439, 0.73613, 0.72788, 0.71975,\
        0.71190, 0.70467, 0.69854, 0.69358, 0.68939, 0.68538, 0.68133, 0.67712, 0.67264, 0.66785, 0.66271,\
        0.65725, 0.65149, 0.64548, 0.63927, 0.63293, 0.62653, 0.62016, 0.61393, 0.60795, 0.60235, 0.59759,\
        0.59389, 0.59074, 0.58807, 0.58579, 0.58372, 0.58170, 0.57954, 0.57710, 0.57426, 0.57092, 0.56706],\
        [0.71133, 0.71652, 0.72169, 0.72707, 0.73288, 0.73925, 0.74627, 0.75392, 0.76207, 0.77044, 0.77850,\
        0.78567, 0.79188, 0.79733, 0.80210, 0.80693, 0.81201, 0.81703, 0.82187, 0.82660, 0.83136, 0.83623,\
        0.84124, 0.84640, 0.85171, 0.85713, 0.86264, 0.86820, 0.87369, 0.87897, 0.88398, 0.88885, 0.89357,\
        0.89793, 0.90194, 0.90592, 0.91031, 0.91496, 0.91916, 0.92282, 0.92639, 0.92983, 0.93272, 0.93482,\
        0.93603, 0.93634, 0.93585, 0.93467, 0.93272, 0.93009, 0.92719, 0.92404, 0.92031, 0.91584, 0.91047,\
        0.90422, 0.89832, 0.89293, 0.88629, 0.87955, 0.87368, 0.86859, 0.86402, 0.85969, 0.85536, 0.85089,\
        0.84623, 0.84145, 0.83668, 0.83211, 0.82786, 0.82397, 0.82031, 0.81663, 0.81262, 0.80798, 0.80258,\
        0.79638, 0.78958, 0.78234, 0.77476, 0.76694, 0.75896, 0.75088, 0.74274, 0.73462, 0.72657, 0.71873,\
        0.71123, 0.70427, 0.69804, 0.69260, 0.68769, 0.68306, 0.67851, 0.67389, 0.66910, 0.66408, 0.65880,\
        0.65326, 0.64749, 0.64150, 0.63537, 0.62914, 0.62289, 0.61672, 0.61073, 0.60506, 0.59982, 0.59518,\
        0.59136, 0.58817, 0.58543, 0.58300, 0.58075, 0.57854, 0.57620, 0.57362, 0.57068, 0.56730, 0.56345],\
        [0.70623, 0.71088, 0.71539, 0.72013, 0.72532, 0.73111, 0.73757, 0.74471, 0.75239, 0.76038, 0.76830,\
        0.77578, 0.78265, 0.78888, 0.79460, 0.80028, 0.80615, 0.81169, 0.81670, 0.82149, 0.82630, 0.83120,\
        0.83620, 0.84130, 0.84647, 0.85165, 0.85681, 0.86189, 0.86681, 0.87149, 0.87593, 0.88013, 0.88408,\
        0.88769, 0.89099, 0.89424, 0.89789, 0.90197, 0.90523, 0.90768, 0.91047, 0.91348, 0.91580, 0.91737,\
        0.91820, 0.91815, 0.91759, 0.91674, 0.91522, 0.91286, 0.91078, 0.90884, 0.90630, 0.90301, 0.89893,\
        0.89371, 0.88852, 0.88352, 0.87801, 0.87227, 0.86681, 0.86181, 0.85715, 0.85265, 0.84811, 0.84344,\
        0.83861, 0.83373, 0.82901, 0.82467, 0.82089, 0.81773, 0.81500, 0.81237, 0.80938, 0.80551, 0.80061,\
        0.79465, 0.78788, 0.78058, 0.77296, 0.76514, 0.75720, 0.74922, 0.74124, 0.73332, 0.72556, 0.71804,\
        0.71089, 0.70422, 0.69794, 0.69197, 0.68643, 0.68120, 0.67612, 0.67107, 0.66595, 0.66068, 0.65524,\
        0.64961, 0.64381, 0.63786, 0.63180, 0.62569, 0.61960, 0.61363, 0.60789, 0.60249, 0.59751, 0.59305,\
        0.58919, 0.58589, 0.58301, 0.58041, 0.57797, 0.57555, 0.57302, 0.57028, 0.56722, 0.56378, 0.55990],\
        [0.70107, 0.70512, 0.70888, 0.71293, 0.71744, 0.72252, 0.72832, 0.73483, 0.74197, 0.74959, 0.75744,\
        0.76524, 0.77274, 0.77984, 0.78660, 0.79327, 0.79987, 0.80589, 0.81091, 0.81582, 0.82079, 0.82582,\
        0.83091, 0.83604, 0.84114, 0.84616, 0.85104, 0.85573, 0.86018, 0.86436, 0.86827, 0.87190, 0.87524,\
        0.87829, 0.88111, 0.88391, 0.88693, 0.89005, 0.89242, 0.89414, 0.89630, 0.89886, 0.90050, 0.90160,\
        0.90220, 0.90178, 0.90116, 0.90064, 0.89979, 0.89752, 0.89610, 0.89500, 0.89330, 0.89083, 0.88759,\
        0.88350, 0.87906, 0.87450, 0.86970, 0.86473, 0.85981, 0.85509, 0.85052, 0.84598, 0.84133, 0.83648,\
        0.83145, 0.82637, 0.82152, 0.81725, 0.81381, 0.81129, 0.80943, 0.80786, 0.80598, 0.80297, 0.79832,\
        0.79247, 0.78569, 0.77838, 0.77079, 0.76304, 0.75523, 0.74742, 0.73967, 0.73202, 0.72456, 0.71733,\
        0.71043, 0.70392, 0.69756, 0.69125, 0.68524, 0.67952, 0.67399, 0.66853, 0.66308, 0.65758, 0.65197,\
        0.64626, 0.64043, 0.63452, 0.62855, 0.62256, 0.61662, 0.61083, 0.60529, 0.60012, 0.59535, 0.59103,\
        0.58720, 0.58381, 0.58078, 0.57800, 0.57535, 0.57272, 0.57000, 0.56709, 0.56389, 0.56035, 0.55641],\
        [0.69582, 0.69930, 0.70234, 0.70587, 0.70968, 0.71393, 0.71902, 0.72483, 0.73135, 0.73859, 0.74642,\
        0.75454, 0.76265, 0.77054, 0.77819, 0.78564, 0.79284, 0.79910, 0.80425, 0.80958, 0.81486, 0.82012,\
        0.82538, 0.83062, 0.83575, 0.84068, 0.84534, 0.84971, 0.85378, 0.85753, 0.86099, 0.86415, 0.86703,\
        0.86967, 0.87212, 0.87451, 0.87692, 0.87921, 0.88106, 0.88258, 0.88423, 0.88595, 0.88719, 0.88796,\
        0.88826, 0.88792, 0.88740, 0.88697, 0.88622, 0.88486, 0.88372, 0.88278, 0.88149, 0.87956, 0.87692,\
        0.87360, 0.86984, 0.86581, 0.86158, 0.85721, 0.85282, 0.84847, 0.84414, 0.83972, 0.83508, 0.83014,\
        0.82491, 0.81956, 0.81444, 0.81009, 0.80680, 0.80479, 0.80359, 0.80284, 0.80198, 0.80005, 0.79566,\
        0.78949, 0.78268, 0.77547, 0.76802, 0.76046, 0.75287, 0.74532, 0.73785, 0.73051, 0.72334, 0.71637,\
        0.70958, 0.70298, 0.69654, 0.69015, 0.68390, 0.67783, 0.67193, 0.66614, 0.66041, 0.65469, 0.64894,\
        0.64316, 0.63732, 0.63146, 0.62558, 0.61973, 0.61395, 0.60830, 0.60292, 0.59793, 0.59335, 0.58915,\
        0.58532, 0.58186, 0.57868, 0.57572, 0.57287, 0.57004, 0.56712, 0.56403, 0.56069, 0.55703, 0.55301],\
        [0.69063, 0.69372, 0.69649, 0.69954, 0.70252, 0.70582, 0.71033, 0.71531, 0.72103, 0.72785, 0.73570,\
        0.74416, 0.75279, 0.76130, 0.76955, 0.77746, 0.78485, 0.79136, 0.79723, 0.80303, 0.80863, 0.81412,\
        0.81963, 0.82508, 0.83031, 0.83521, 0.83972, 0.84382, 0.84756, 0.85095, 0.85404, 0.85683, 0.85937,\
        0.86168, 0.86383, 0.86587, 0.86781, 0.86958, 0.87109, 0.87239, 0.87365, 0.87483, 0.87573, 0.87626,\
        0.87640, 0.87619, 0.87580, 0.87535, 0.87472, 0.87386, 0.87298, 0.87210, 0.87095, 0.86930, 0.86706,\
        0.86425, 0.86099, 0.85743, 0.85367, 0.84981, 0.84591, 0.84201, 0.83804, 0.83389, 0.82942, 0.82453,\
        0.81920, 0.81360, 0.80809, 0.80352, 0.80044, 0.79859, 0.79771, 0.79714, 0.79649, 0.79448, 0.79061,\
        0.78499, 0.77854, 0.77165, 0.76452, 0.75728, 0.75001, 0.74278, 0.73564, 0.72864, 0.72179, 0.71507,\
        0.70843, 0.70177, 0.69523, 0.68876, 0.68233, 0.67601, 0.66983, 0.66378, 0.65783, 0.65194, 0.64608,\
        0.64025, 0.63442, 0.62862, 0.62286, 0.61717, 0.61157, 0.60608, 0.60083, 0.59600, 0.59154, 0.58740,\
        0.58356, 0.58001, 0.57670, 0.57356, 0.57052, 0.56748, 0.56436, 0.56109, 0.55760, 0.55381, 0.54969],\
        [0.68598, 0.68890, 0.69153, 0.69408, 0.69639, 0.69923, 0.70306, 0.70681, 0.71151, 0.71786, 0.72581,\
        0.73460, 0.74361, 0.75249, 0.76107, 0.76922, 0.77680, 0.78372, 0.79014, 0.79628, 0.80215, 0.80788,\
        0.81373, 0.81949, 0.82488, 0.82978, 0.83413, 0.83799, 0.84144, 0.84453, 0.84732, 0.84984, 0.85212,\
        0.85420, 0.85611, 0.85788, 0.85951, 0.86096, 0.86223, 0.86333, 0.86432, 0.86517, 0.86581, 0.86619,\
        0.86628, 0.86612, 0.86581, 0.86541, 0.86488, 0.86423, 0.86350, 0.86265, 0.86155, 0.86003, 0.85801,\
        0.85550, 0.85257, 0.84936, 0.84598, 0.84254, 0.83912, 0.83570, 0.83220, 0.82848, 0.82435, 0.81969,\
        0.81444, 0.80877, 0.80308, 0.79815, 0.79510, 0.79304, 0.79179, 0.79085, 0.78968, 0.78753, 0.78396,\
        0.77905, 0.77323, 0.76688, 0.76024, 0.75343, 0.74656, 0.73971, 0.73295, 0.72631, 0.71979, 0.71337,\
        0.70697, 0.70054, 0.69382, 0.68712, 0.68050, 0.67398, 0.66760, 0.66136, 0.65525, 0.64924, 0.64332,\
        0.63746, 0.63167, 0.62595, 0.62032, 0.61481, 0.60945, 0.60420, 0.59909, 0.59436, 0.58992, 0.58577,\
        0.58188, 0.57823, 0.57479, 0.57148, 0.56825, 0.56502, 0.56171, 0.55826, 0.55461, 0.55068, 0.54645],\
        [0.68196, 0.68484, 0.68739, 0.68970, 0.69193, 0.69446, 0.69718, 0.69990, 0.70336, 0.70931, 0.71748,\
        0.72643, 0.73553, 0.74448, 0.75312, 0.76133, 0.76904, 0.77623, 0.78298, 0.78939, 0.79549, 0.80151,\
        0.80782, 0.81391, 0.81949, 0.82436, 0.82853, 0.83213, 0.83532, 0.83817, 0.84074, 0.84306, 0.84517,\
        0.84709, 0.84884, 0.85044, 0.85188, 0.85315, 0.85425, 0.85519, 0.85600, 0.85667, 0.85716, 0.85744,\
        0.85750, 0.85738, 0.85712, 0.85676, 0.85629, 0.85572, 0.85505, 0.85421, 0.85311, 0.85164, 0.84972,\
        0.84736, 0.84461, 0.84160, 0.83848, 0.83538, 0.83239, 0.82949, 0.82655, 0.82338, 0.81976, 0.81552,\
        0.81056, 0.80497, 0.79933, 0.79434, 0.79050, 0.78774, 0.78573, 0.78408, 0.78231, 0.77990, 0.77653,\
        0.77218, 0.76703, 0.76132, 0.75524, 0.74895, 0.74254, 0.73610, 0.72972, 0.72344, 0.71727, 0.71116,\
        0.70494, 0.69851, 0.69186, 0.68508, 0.67833, 0.67168, 0.66517, 0.65881, 0.65260, 0.64652, 0.64058,\
        0.63475, 0.62902, 0.62339, 0.61789, 0.61254, 0.60743, 0.60257, 0.59769, 0.59289, 0.58841, 0.58420,\
        0.58025, 0.57650, 0.57292, 0.56946, 0.56605, 0.56264, 0.55915, 0.55552, 0.55170, 0.54764, 0.54328],\
        [0.67857, 0.68152, 0.68413, 0.68649, 0.68873, 0.69098, 0.69316, 0.69533, 0.69801, 0.70367, 0.71157,\
        0.72015, 0.72889, 0.73752, 0.74592, 0.75400, 0.76169, 0.76898, 0.77591, 0.78256, 0.78901, 0.79547,\
        0.80202, 0.80836, 0.81410, 0.81885, 0.82275, 0.82608, 0.82904, 0.83172, 0.83415, 0.83637, 0.83838,\
        0.84022, 0.84190, 0.84342, 0.84477, 0.84595, 0.84695, 0.84780, 0.84850, 0.84906, 0.84947, 0.84971,\
        0.84977, 0.84967, 0.84943, 0.84909, 0.84864, 0.84809, 0.84742, 0.84657, 0.84546, 0.84399, 0.84210,\
        0.83978, 0.83708, 0.83414, 0.83113, 0.82825, 0.82564, 0.82326, 0.82094, 0.81842, 0.81542, 0.81171,\
        0.80719, 0.80204, 0.79612, 0.79061, 0.78596, 0.78228, 0.77938, 0.77694, 0.77458, 0.77192, 0.76868,\
        0.76477, 0.76022, 0.75516, 0.74970, 0.74394, 0.73800, 0.73197, 0.72595, 0.71999, 0.71414, 0.70839,\
        0.70247, 0.69605, 0.68937, 0.68256, 0.67577, 0.66906, 0.66248, 0.65606, 0.64981, 0.64373, 0.63782,\
        0.63205, 0.62641, 0.62090, 0.61551, 0.61027, 0.60525, 0.60054, 0.59593, 0.59129, 0.58685, 0.58264,\
        0.57863, 0.57479, 0.57109, 0.56747, 0.56390, 0.56031, 0.55664, 0.55284, 0.54886, 0.54465, 0.54017],\
        [0.67574, 0.67886, 0.68167, 0.68425, 0.68668, 0.68902, 0.69131, 0.69369, 0.69680, 0.70169, 0.70836,\
        0.71585, 0.72370, 0.73165, 0.73954, 0.74726, 0.75475, 0.76198, 0.76898, 0.77580, 0.78255, 0.78931,\
        0.79608, 0.80268, 0.80861, 0.81304, 0.81654, 0.81962, 0.82244, 0.82505, 0.82745, 0.82964, 0.83164,\
        0.83348, 0.83517, 0.83669, 0.83803, 0.83918, 0.84015, 0.84095, 0.84160, 0.84212, 0.84250, 0.84272,\
        0.84280, 0.84271, 0.84248, 0.84213, 0.84167, 0.84110, 0.84041, 0.83955, 0.83844, 0.83696, 0.83506,\
        0.83271, 0.82996, 0.82695, 0.82390, 0.82108, 0.81875, 0.81689, 0.81519, 0.81331, 0.81098, 0.80781,\
        0.80346, 0.79827, 0.79236, 0.78623, 0.78083, 0.77630, 0.77256, 0.76940, 0.76654, 0.76367, 0.76056,\
        0.75705, 0.75307, 0.74864, 0.74378, 0.73856, 0.73308, 0.72742, 0.72170, 0.71600, 0.71034, 0.70479,\
        0.69936, 0.69299, 0.68631, 0.67954, 0.67277, 0.66608, 0.65950, 0.65309, 0.64686, 0.64082, 0.63498,\
        0.62932, 0.62382, 0.61846, 0.61320, 0.60804, 0.60304, 0.59835, 0.59393, 0.58954, 0.58522, 0.58103,\
        0.57698, 0.57307, 0.56925, 0.56550, 0.56177, 0.55801, 0.55418, 0.55021, 0.54608, 0.54172, 0.53710],\
        [0.67335, 0.67672, 0.67984, 0.68276, 0.68557, 0.68832, 0.69109, 0.69405, 0.69757, 0.70185, 0.70698,\
        0.71301, 0.71965, 0.72667, 0.73384, 0.74105, 0.74818, 0.75519, 0.76209, 0.76893, 0.77578, 0.78271,\
        0.78970, 0.79666, 0.80256, 0.80633, 0.80952, 0.81253, 0.81537, 0.81804, 0.82052, 0.82279, 0.82484,\
        0.82675, 0.82853, 0.83014, 0.83154, 0.83270, 0.83367, 0.83445, 0.83509, 0.83560, 0.83599, 0.83625,\
        0.83634, 0.83627, 0.83603, 0.83565, 0.83514, 0.83453, 0.83382, 0.83296, 0.83186, 0.83041, 0.82850,\
        0.82609, 0.82323, 0.82006, 0.81681, 0.81383, 0.81162, 0.81028, 0.80915, 0.80772, 0.80600, 0.80332,\
        0.79906, 0.79327, 0.78700, 0.78063, 0.77472, 0.76954, 0.76514, 0.76141, 0.75818, 0.75521, 0.75228,\
        0.74919, 0.74577, 0.74196, 0.73769, 0.73299, 0.72792, 0.72257, 0.71709, 0.71160, 0.70611, 0.70043,\
        0.69497, 0.68905, 0.68264, 0.67603, 0.66937, 0.66275, 0.65624, 0.64988, 0.64371, 0.63776, 0.63203,\
        0.62653, 0.62122, 0.61606, 0.61100, 0.60599, 0.60106, 0.59647, 0.59210, 0.58781, 0.58356, 0.57939,\
        0.57531, 0.57132, 0.56740, 0.56352, 0.55965, 0.55573, 0.55174, 0.54762, 0.54333, 0.53883, 0.53408],\
        [0.67125, 0.67493, 0.67839, 0.68173, 0.68500, 0.68829, 0.69164, 0.69510, 0.69867, 0.70226, 0.70615,\
        0.71081, 0.71623, 0.72224, 0.72862, 0.73520, 0.74185, 0.74850, 0.75514, 0.76179, 0.76851, 0.77533,\
        0.78220, 0.78884, 0.79433, 0.79815, 0.80149, 0.80469, 0.80773, 0.81061, 0.81331, 0.81574, 0.81788,\
        0.81992, 0.82190, 0.82367, 0.82515, 0.82636, 0.82734, 0.82813, 0.82877, 0.82931, 0.82974, 0.83005,\
        0.83019, 0.83013, 0.82987, 0.82943, 0.82884, 0.82816, 0.82742, 0.82658, 0.82555, 0.82416, 0.82230,\
        0.81986, 0.81685, 0.81351, 0.81001, 0.80663, 0.80425, 0.80342, 0.80270, 0.80134, 0.79975, 0.79785,\
        0.79286, 0.78690, 0.78038, 0.77382, 0.76758, 0.76196, 0.75708, 0.75295, 0.74950, 0.74656, 0.74392,\
        0.74130, 0.73849, 0.73531, 0.73163, 0.72743, 0.72273, 0.71761, 0.71227, 0.70695, 0.70166, 0.69623,\
        0.69058, 0.68472, 0.67853, 0.67211, 0.66560, 0.65911, 0.65269, 0.64643, 0.64035, 0.63451, 0.62893,\
        0.62362, 0.61856, 0.61368, 0.60890, 0.60416, 0.59943, 0.59485, 0.59045, 0.58614, 0.58190, 0.57772,\
        0.57360, 0.56954, 0.56553, 0.56153, 0.55751, 0.55345, 0.54931, 0.54504, 0.54060, 0.53596, 0.53108],\
        [0.66929, 0.67326, 0.67707, 0.68080, 0.68453, 0.68832, 0.69219, 0.69602, 0.69948, 0.70229, 0.70506,\
        0.70856, 0.71294, 0.71804, 0.72364, 0.72956, 0.73565, 0.74182, 0.74802, 0.75428, 0.76063, 0.76706,\
        0.77345, 0.77951, 0.78472, 0.78894, 0.79265, 0.79615, 0.79952, 0.80270, 0.80578, 0.80847, 0.81067,\
        0.81289, 0.81523, 0.81721, 0.81876, 0.82001, 0.82101, 0.82179, 0.82244, 0.82301, 0.82353, 0.82393,\
        0.82413, 0.82408, 0.82379, 0.82327, 0.82256, 0.82176, 0.82098, 0.82021, 0.81930, 0.81803, 0.81628,\
        0.81392, 0.81075, 0.80723, 0.80382, 0.80003, 0.79730, 0.79631, 0.79544, 0.79402, 0.79224, 0.78957,\
        0.78494, 0.77905, 0.77257, 0.76596, 0.75955, 0.75365, 0.74845, 0.74407, 0.74054, 0.73778, 0.73556,\
        0.73353, 0.73139, 0.72887, 0.72579, 0.72208, 0.71773, 0.71278, 0.70742, 0.70218, 0.69701, 0.69167,\
        0.68609, 0.68026, 0.67419, 0.66793, 0.66157, 0.65521, 0.64892, 0.64276, 0.63680, 0.63108, 0.62566,\
        0.62056, 0.61577, 0.61122, 0.60680, 0.60235, 0.59782, 0.59327, 0.58883, 0.58450, 0.58023, 0.57602,\
        0.57186, 0.56773, 0.56362, 0.55951, 0.55536, 0.55116, 0.54687, 0.54246, 0.53788, 0.53311, 0.52809],\
        [0.66731, 0.67152, 0.67561, 0.67965, 0.68372, 0.68786, 0.69206, 0.69618, 0.69964, 0.70155, 0.70319,\
        0.70583, 0.70945, 0.71380, 0.71871, 0.72400, 0.72950, 0.73510, 0.74073, 0.74642, 0.75220, 0.75805,\
        0.76387, 0.76944, 0.77450, 0.77900, 0.78311, 0.78701, 0.79076, 0.79436, 0.79790, 0.80092, 0.80309,\
        0.80566, 0.80861, 0.81067, 0.81218, 0.81350, 0.81452, 0.81527, 0.81590, 0.81651, 0.81714, 0.81769,\
        0.81797, 0.81792, 0.81758, 0.81699, 0.81613, 0.81513, 0.81425, 0.81361, 0.81290, 0.81173, 0.81012,\
        0.80810, 0.80492, 0.80067, 0.79791, 0.79395, 0.79087, 0.78895, 0.78742, 0.78567, 0.78341, 0.78023,\
        0.77574, 0.77013, 0.76386, 0.75734, 0.75091, 0.74486, 0.73944, 0.73490, 0.73141, 0.72898, 0.72733,\
        0.72603, 0.72464, 0.72280, 0.72031, 0.71706, 0.71309, 0.70841, 0.70299, 0.69755, 0.69246, 0.68711,\
        0.68152, 0.67571, 0.66971, 0.66357, 0.65735, 0.65113, 0.64497, 0.63893, 0.63307, 0.62747, 0.62219,\
        0.61729, 0.61277, 0.60858, 0.60454, 0.60038, 0.59599, 0.59154, 0.58713, 0.58279, 0.57851, 0.57427,\
        0.57006, 0.56587, 0.56167, 0.55745, 0.55318, 0.54885, 0.54443, 0.53987, 0.53516, 0.53025, 0.52511],\
        [0.66515, 0.66955, 0.67382, 0.67804, 0.68227, 0.68652, 0.69075, 0.69478, 0.69806, 0.69928, 0.70035,\
        0.70256, 0.70568, 0.70940, 0.71370, 0.71843, 0.72336, 0.72833, 0.73328, 0.73827, 0.74335, 0.74854,\
        0.75379, 0.75896, 0.76391, 0.76860, 0.77306, 0.77736, 0.78154, 0.78557, 0.78940, 0.79275, 0.79542,\
        0.79876, 0.80195, 0.80380, 0.80515, 0.80672, 0.80770, 0.80838, 0.80899, 0.80961, 0.81037, 0.81117,\
        0.81154, 0.81140, 0.81103, 0.81043, 0.80941, 0.80811, 0.80702, 0.80659, 0.80615, 0.80495, 0.80326,\
        0.80172, 0.79898, 0.79457, 0.79072, 0.78701, 0.78371, 0.78113, 0.77893, 0.77667, 0.77396, 0.77048,\
        0.76603, 0.76067, 0.75468, 0.74836, 0.74202, 0.73593, 0.73036, 0.72568, 0.72228, 0.72032, 0.71943,\
        0.71899, 0.71840, 0.71725, 0.71529, 0.71243, 0.70876, 0.70445, 0.69952, 0.69377, 0.68821, 0.68264,\
        0.67694, 0.67110, 0.66514, 0.65909, 0.65300, 0.64692, 0.64090, 0.63499, 0.62924, 0.62374, 0.61856,\
        0.61380, 0.60952, 0.60566, 0.60199, 0.59810, 0.59388, 0.58958, 0.58526, 0.58097, 0.57671, 0.57246,\
        0.56821, 0.56395, 0.55967, 0.55535, 0.55097, 0.54651, 0.54195, 0.53726, 0.53242, 0.52739, 0.52213],\
        [0.66273, 0.66722, 0.67156, 0.67582, 0.68003, 0.68416, 0.68813, 0.69169, 0.69437, 0.69586, 0.69707,\
        0.69903, 0.70166, 0.70475, 0.70858, 0.71287, 0.71725, 0.72155, 0.72573, 0.72991, 0.73422, 0.73873,\
        0.74344, 0.74826, 0.75312, 0.75794, 0.76271, 0.76739, 0.77197, 0.77638, 0.78052, 0.78426, 0.78770,\
        0.79120, 0.79429, 0.79619, 0.79770, 0.79951, 0.80032, 0.80095, 0.80157, 0.80216, 0.80302, 0.80421,\
        0.80463, 0.80424, 0.80385, 0.80338, 0.80225, 0.80069, 0.79927, 0.79900, 0.79867, 0.79738, 0.79552,\
        0.79370, 0.79099, 0.78698, 0.78288, 0.77906, 0.77563, 0.77267, 0.77000, 0.76732, 0.76429, 0.76066,\
        0.75627, 0.75114, 0.74545, 0.73943, 0.73330, 0.72730, 0.72168, 0.71682, 0.71344, 0.71208, 0.71216,\
        0.71263, 0.71283, 0.71231, 0.71079, 0.70814, 0.70456, 0.70029, 0.69536, 0.68976, 0.68396, 0.67815,\
        0.67230, 0.66641, 0.66047, 0.65452, 0.64857, 0.64264, 0.63678, 0.63101, 0.62538, 0.61997, 0.61485,\
        0.61016, 0.60603, 0.60242, 0.59902, 0.59539, 0.59147, 0.58739, 0.58323, 0.57903, 0.57481, 0.57056,\
        0.56629, 0.56198, 0.55763, 0.55321, 0.54872, 0.54414, 0.53945, 0.53463, 0.52966, 0.52451, 0.51913],\
        [0.65997, 0.66447, 0.66879, 0.67297, 0.67702, 0.68090, 0.68450, 0.68763, 0.69004, 0.69175, 0.69323,\
        0.69503, 0.69723, 0.69987, 0.70341, 0.70735, 0.71121, 0.71480, 0.71815, 0.72145, 0.72496, 0.72880,\
        0.73300, 0.73753, 0.74230, 0.74724, 0.75227, 0.75730, 0.76225, 0.76701, 0.77146, 0.77557, 0.77934,\
        0.78282, 0.78577, 0.78795, 0.78973, 0.79126, 0.79224, 0.79293, 0.79357, 0.79422, 0.79515, 0.79635,\
        0.79674, 0.79626, 0.79584, 0.79534, 0.79425, 0.79278, 0.79151, 0.79078, 0.79006, 0.78871, 0.78680,\
        0.78454, 0.78162, 0.77795, 0.77402, 0.77023, 0.76676, 0.76364, 0.76074, 0.75784, 0.75466, 0.75100,\
        0.74674, 0.74188, 0.73654, 0.73089, 0.72513, 0.71945, 0.71400, 0.70898, 0.70536, 0.70480, 0.70593,\
        0.70716, 0.70797, 0.70798, 0.70686, 0.70423, 0.70043, 0.69602, 0.69092, 0.68530, 0.67941, 0.67345,\
        0.66750, 0.66158, 0.65570, 0.64986, 0.64407, 0.63833, 0.63266, 0.62708, 0.62161, 0.61631, 0.61124,\
        0.60654, 0.60251, 0.59913, 0.59592, 0.59255, 0.58891, 0.58507, 0.58107, 0.57699, 0.57283, 0.56860,\
        0.56432, 0.55996, 0.55554, 0.55103, 0.54642, 0.54172, 0.53691, 0.53196, 0.52687, 0.52161, 0.51612],\
        [0.65684, 0.66127, 0.66549, 0.66950, 0.67333, 0.67692, 0.68018, 0.68301, 0.68530, 0.68713, 0.68877,\
        0.69053, 0.69259, 0.69511, 0.69830, 0.70183, 0.70517, 0.70807, 0.71056, 0.71296, 0.71571, 0.71894,\
        0.72269, 0.72695, 0.73164, 0.73668, 0.74195, 0.74731, 0.75260, 0.75766, 0.76238, 0.76670, 0.77059,\
        0.77403, 0.77691, 0.77920, 0.78103, 0.78248, 0.78352, 0.78430, 0.78498, 0.78568, 0.78651, 0.78733,\
        0.78764, 0.78740, 0.78697, 0.78637, 0.78541, 0.78420, 0.78301, 0.78198, 0.78085, 0.77930, 0.77724,\
        0.77470, 0.77163, 0.76809, 0.76435, 0.76071, 0.75733, 0.75424, 0.75133, 0.74841, 0.74527, 0.74173,\
        0.73768, 0.73314, 0.72819, 0.72299, 0.71774, 0.71267, 0.70789, 0.70327, 0.69948, 0.69949, 0.70109,\
        0.70253, 0.70369, 0.70393, 0.70336, 0.70085, 0.69655, 0.69168, 0.68627, 0.68048, 0.67449, 0.66845,\
        0.66246, 0.65657, 0.65078, 0.64511, 0.63952, 0.63402, 0.62860, 0.62325, 0.61801, 0.61292, 0.60804,\
        0.60337, 0.59934, 0.59618, 0.59309, 0.58985, 0.58639, 0.58272, 0.57887, 0.57489, 0.57079, 0.56659,\
        0.56230, 0.55791, 0.55341, 0.54881, 0.54410, 0.53927, 0.53433, 0.52926, 0.52405, 0.51868, 0.51309],\
        [0.65333, 0.65765, 0.66170, 0.66550, 0.66907, 0.67238, 0.67537, 0.67798, 0.68020, 0.68209, 0.68384,\
        0.68564, 0.68767, 0.69008, 0.69292, 0.69598, 0.69887, 0.70123, 0.70295, 0.70455, 0.70669, 0.70941,\
        0.71277, 0.71678, 0.72141, 0.72653, 0.73201, 0.73766, 0.74324, 0.74855, 0.75344, 0.75784, 0.76172,\
        0.76507, 0.76786, 0.77012, 0.77191, 0.77331, 0.77436, 0.77516, 0.77584, 0.77647, 0.77711, 0.77764,\
        0.77786, 0.77774, 0.77738, 0.77682, 0.77601, 0.77500, 0.77389, 0.77271, 0.77131, 0.76953, 0.76726,\
        0.76452, 0.76133, 0.75783, 0.75423, 0.75077, 0.74759, 0.74469, 0.74197, 0.73925, 0.73632, 0.73302,\
        0.72928, 0.72511, 0.72059, 0.71589, 0.71120, 0.70683, 0.70318, 0.69974, 0.69674, 0.69610, 0.69709,\
        0.69837, 0.69968, 0.69955, 0.69920, 0.69704, 0.69233, 0.68696, 0.68123, 0.67525, 0.66916, 0.66309,\
        0.65712, 0.65131, 0.64569, 0.64024, 0.63493, 0.62973, 0.62461, 0.61957, 0.61461, 0.60981, 0.60533,\
        0.60134, 0.59738, 0.59390, 0.59067, 0.58741, 0.58401, 0.58043, 0.57668, 0.57278, 0.56873, 0.56455,\
        0.56025, 0.55582, 0.55126, 0.54657, 0.54175, 0.53680, 0.53172, 0.52651, 0.52119, 0.51571, 0.51004],\
        [0.64945, 0.65361, 0.65745, 0.66103, 0.66434, 0.66740, 0.67016, 0.67262, 0.67478, 0.67671, 0.67852,\
        0.68036, 0.68235, 0.68460, 0.68709, 0.68966, 0.69205, 0.69399, 0.69534, 0.69657, 0.69831, 0.70060,\
        0.70362, 0.70737, 0.71189, 0.71707, 0.72270, 0.72861, 0.73444, 0.73991, 0.74485, 0.74920, 0.75295,\
        0.75612, 0.75874, 0.76087, 0.76256, 0.76387, 0.76488, 0.76564, 0.76625, 0.76675, 0.76719, 0.76751,\
        0.76764, 0.76756, 0.76730, 0.76686, 0.76623, 0.76541, 0.76440, 0.76317, 0.76163, 0.75968, 0.75724,\
        0.75432, 0.75101, 0.74749, 0.74398, 0.74072, 0.73782, 0.73527, 0.73292, 0.73057, 0.72801, 0.72508,\
        0.72170, 0.71792, 0.71385, 0.70968, 0.70557, 0.70165, 0.69882, 0.69628, 0.69407, 0.69316, 0.69344,\
        0.69422, 0.69482, 0.69461, 0.69364, 0.69115, 0.68680, 0.68143, 0.67559, 0.66953, 0.66340, 0.65734,\
        0.65145, 0.64580, 0.64040, 0.63524, 0.63028, 0.62545, 0.62072, 0.61603, 0.61141, 0.60691, 0.60265,\
        0.59902, 0.59551, 0.59190, 0.58851, 0.58517, 0.58176, 0.57822, 0.57453, 0.57067, 0.56666, 0.56250,\
        0.55818, 0.55371, 0.54909, 0.54432, 0.53939, 0.53430, 0.52908, 0.52373, 0.51828, 0.51272, 0.50697],\
        [0.64523, 0.64918, 0.65280, 0.65613, 0.65920, 0.66203, 0.66461, 0.66695, 0.66907, 0.67100, 0.67285,\
        0.67470, 0.67664, 0.67871, 0.68090, 0.68308, 0.68507, 0.68672, 0.68803, 0.68929, 0.69086, 0.69295,\
        0.69570, 0.69913, 0.70354, 0.70863, 0.71427, 0.72044, 0.72644, 0.73192, 0.73676, 0.74091, 0.74441,\
        0.74731, 0.74968, 0.75159, 0.75312, 0.75431, 0.75522, 0.75589, 0.75637, 0.75672, 0.75697, 0.75711,\
        0.75715, 0.75710, 0.75695, 0.75669, 0.75627, 0.75565, 0.75479, 0.75362, 0.75205, 0.75002, 0.74746,\
        0.74440, 0.74097, 0.73737, 0.73390, 0.73083, 0.72829, 0.72622, 0.72442, 0.72260, 0.72054, 0.71806,\
        0.71509, 0.71170, 0.70804, 0.70437, 0.70087, 0.69759, 0.69489, 0.69275, 0.69100, 0.68998, 0.68969,\
        0.68975, 0.68964, 0.68894, 0.68734, 0.68451, 0.68031, 0.67511, 0.66934, 0.66330, 0.65720, 0.65121,\
        0.64545, 0.64000, 0.63490, 0.63011, 0.62557, 0.62119, 0.61689, 0.61263, 0.60838, 0.60419, 0.60022,\
        0.59660, 0.59321, 0.58979, 0.58640, 0.58303, 0.57961, 0.57609, 0.57242, 0.56859, 0.56460, 0.56044,\
        0.55611, 0.55160, 0.54693, 0.54207, 0.53704, 0.53182, 0.52644, 0.52092, 0.51532, 0.50968, 0.50388],\
        [0.64067, 0.64440, 0.64776, 0.65084, 0.65368, 0.65632, 0.65875, 0.66100, 0.66308, 0.66502, 0.66689,\
        0.66873, 0.67061, 0.67254, 0.67451, 0.67643, 0.67820, 0.67975, 0.68114, 0.68255, 0.68423, 0.68636,\
        0.68909, 0.69251, 0.69679, 0.70136, 0.70705, 0.71351, 0.71945, 0.72473, 0.72928, 0.73308, 0.73619,\
        0.73870, 0.74073, 0.74237, 0.74369, 0.74473, 0.74551, 0.74607, 0.74641, 0.74658, 0.74663, 0.74660,\
        0.74656, 0.74653, 0.74652, 0.74647, 0.74631, 0.74595, 0.74530, 0.74428, 0.74281, 0.74079, 0.73820,\
        0.73506, 0.73150, 0.72778, 0.72428, 0.72139, 0.71930, 0.71785, 0.71672, 0.71558, 0.71412, 0.71214,\
        0.70960, 0.70657, 0.70321, 0.69995, 0.69690, 0.69405, 0.69153, 0.68944, 0.68774, 0.68651, 0.68570,\
        0.68506, 0.68425, 0.68292, 0.68078, 0.67762, 0.67340, 0.66830, 0.66263, 0.65666, 0.65063, 0.64474,\
        0.63914, 0.63394, 0.62918, 0.62483, 0.62079, 0.61693, 0.61312, 0.60931, 0.60548, 0.60161, 0.59786,\
        0.59433, 0.59096, 0.58762, 0.58428, 0.58092, 0.57750, 0.57399, 0.57033, 0.56652, 0.56254, 0.55838,\
        0.55403, 0.54949, 0.54476, 0.53984, 0.53471, 0.52939, 0.52386, 0.51814, 0.51232, 0.50658, 0.50079],\
        [0.63580, 0.63927, 0.64237, 0.64519, 0.64782, 0.65028, 0.65261, 0.65479, 0.65685, 0.65881, 0.66068,\
        0.66252, 0.66436, 0.66619, 0.66803, 0.66982, 0.67151, 0.67311, 0.67466, 0.67631, 0.67822, 0.68057,\
        0.68348, 0.68704, 0.69124, 0.69598, 0.70201, 0.70801, 0.71344, 0.71829, 0.72241, 0.72572, 0.72832,\
        0.73034, 0.73196, 0.73328, 0.73436, 0.73524, 0.73591, 0.73633, 0.73653, 0.73651, 0.73636, 0.73615,\
        0.73602, 0.73602, 0.73618, 0.73641, 0.73656, 0.73651, 0.73614, 0.73537, 0.73409, 0.73222, 0.72971,\
        0.72658, 0.72295, 0.71908, 0.71543, 0.71269, 0.71116, 0.71044, 0.71011, 0.70974, 0.70893, 0.70745,\
        0.70532, 0.70265, 0.69959, 0.69656, 0.69369, 0.69097, 0.68847, 0.68627, 0.68440, 0.68283, 0.68150,\
        0.68023, 0.67875, 0.67681, 0.67418, 0.67071, 0.66638, 0.66130, 0.65569, 0.64978, 0.64382, 0.63802,\
        0.63258, 0.62765, 0.62327, 0.61942, 0.61595, 0.61265, 0.60935, 0.60600, 0.60259, 0.59906, 0.59548,\
        0.59204, 0.58870, 0.58541, 0.58212, 0.57879, 0.57540, 0.57189, 0.56826, 0.56446, 0.56048, 0.55631,\
        0.55195, 0.54738, 0.54260, 0.53762, 0.53243, 0.52703, 0.52140, 0.51551, 0.50936, 0.50332, 0.49765],\
        [0.63063, 0.63381, 0.63662, 0.63919, 0.64162, 0.64395, 0.64619, 0.64835, 0.65042, 0.65240, 0.65430,\
        0.65614, 0.65796, 0.65975, 0.66153, 0.66329, 0.66502, 0.66675, 0.66852, 0.67045, 0.67267, 0.67531,\
        0.67848, 0.68227, 0.68673, 0.69194, 0.69781, 0.70312, 0.70796, 0.71241, 0.71606, 0.71878, 0.72075,\
        0.72221, 0.72337, 0.72435, 0.72521, 0.72595, 0.72652, 0.72685, 0.72691, 0.72672, 0.72634, 0.72594,\
        0.72569, 0.72573, 0.72611, 0.72668, 0.72719, 0.72749, 0.72746, 0.72701, 0.72605, 0.72446, 0.72218,\
        0.71920, 0.71563, 0.71170, 0.70785, 0.70513, 0.70440, 0.70436, 0.70480, 0.70526, 0.70512, 0.70405,\
        0.70219, 0.69987, 0.69705, 0.69405, 0.69109, 0.68826, 0.68560, 0.68317, 0.68099, 0.67902, 0.67718,\
        0.67531, 0.67324, 0.67075, 0.66768, 0.66392, 0.65944, 0.65433, 0.64873, 0.64285, 0.63692, 0.63119,\
        0.62588, 0.62118, 0.61721, 0.61390, 0.61106, 0.60837, 0.60557, 0.60257, 0.59948, 0.59626, 0.59292,\
        0.58962, 0.58637, 0.58315, 0.57991, 0.57663, 0.57326, 0.56978, 0.56616, 0.56237, 0.55840, 0.55423,\
        0.54984, 0.54524, 0.54042, 0.53540, 0.53017, 0.52474, 0.51912, 0.51324, 0.50700, 0.50042, 0.49455],\
        [0.62518, 0.62803, 0.63051, 0.63284, 0.63510, 0.63734, 0.63955, 0.64172, 0.64382, 0.64585, 0.64779,\
        0.64966, 0.65147, 0.65326, 0.65505, 0.65685, 0.65869, 0.66060, 0.66263, 0.66486, 0.66740, 0.67034,\
        0.67378, 0.67779, 0.68240, 0.68756, 0.69294, 0.69794, 0.70262, 0.70691, 0.71009, 0.71215, 0.71341,\
        0.71427, 0.71498, 0.71564, 0.71630, 0.71696, 0.71750, 0.71776, 0.71772, 0.71739, 0.71679, 0.71614,\
        0.71576, 0.71583, 0.71650, 0.71750, 0.71839, 0.71903, 0.71937, 0.71931, 0.71875, 0.71759, 0.71571,\
        0.71305, 0.70974, 0.70608, 0.70243, 0.69956, 0.69966, 0.69989, 0.70097, 0.70221, 0.70267, 0.70201,\
        0.70016, 0.69796, 0.69515, 0.69201, 0.68883, 0.68573, 0.68280, 0.68006, 0.67750, 0.67510, 0.67276,\
        0.67037, 0.66778, 0.66483, 0.66138, 0.65736, 0.65274, 0.64756, 0.64195, 0.63607, 0.63014, 0.62441,\
        0.61917, 0.61468, 0.61106, 0.60831, 0.60616, 0.60409, 0.60173, 0.59904, 0.59617, 0.59320, 0.59013,\
        0.58702, 0.58391, 0.58078, 0.57762, 0.57439, 0.57106, 0.56761, 0.56402, 0.56025, 0.55628, 0.55210,\
        0.54770, 0.54307, 0.53821, 0.53313, 0.52786, 0.52243, 0.51689, 0.51124, 0.50537, 0.49908, 0.49235],\
        [0.61946, 0.62189, 0.62403, 0.62614, 0.62829, 0.63049, 0.63271, 0.63493, 0.63711, 0.63921, 0.64121,\
        0.64311, 0.64494, 0.64675, 0.64858, 0.65047, 0.65246, 0.65458, 0.65688, 0.65942, 0.66226, 0.66548,\
        0.66914, 0.67328, 0.67789, 0.68285, 0.68791, 0.69279, 0.69749, 0.70148, 0.70426, 0.70562, 0.70616,\
        0.70651, 0.70685, 0.70725, 0.70772, 0.70836, 0.70894, 0.70914, 0.70903, 0.70870, 0.70789, 0.70699,\
        0.70655, 0.70653, 0.70760, 0.70915, 0.71028, 0.71118, 0.71192, 0.71226, 0.71215, 0.71156, 0.71024,\
        0.70806, 0.70513, 0.70201, 0.69913, 0.69706, 0.69662, 0.69718, 0.69855, 0.70017, 0.70109, 0.70113,\
        0.69952, 0.69678, 0.69354, 0.69007, 0.68658, 0.68318, 0.67992, 0.67683, 0.67389, 0.67107, 0.66829,\
        0.66544, 0.66241, 0.65907, 0.65532, 0.65110, 0.64636, 0.64114, 0.63553, 0.62966, 0.62370, 0.61793,\
        0.61269, 0.60836, 0.60502, 0.60278, 0.60125, 0.59976, 0.59768, 0.59532, 0.59272, 0.59000, 0.58717,\
        0.58426, 0.58130, 0.57829, 0.57521, 0.57205, 0.56878, 0.56537, 0.56180, 0.55805, 0.55410, 0.54992,\
        0.54551, 0.54085, 0.53594, 0.53079, 0.52544, 0.51994, 0.51439, 0.50890, 0.50340, 0.49707, 0.48991],\
        [0.61349, 0.61537, 0.61714, 0.61912, 0.62124, 0.62344, 0.62571, 0.62803, 0.63034, 0.63255, 0.63462,\
        0.63655, 0.63840, 0.64023, 0.64212, 0.64412, 0.64628, 0.64863, 0.65120, 0.65402, 0.65714, 0.66058,\
        0.66440, 0.66859, 0.67312, 0.67788, 0.68270, 0.68739, 0.69177, 0.69541, 0.69805, 0.69886, 0.69900,\
        0.69903, 0.69913, 0.69935, 0.69962, 0.70028, 0.70094, 0.70101, 0.70087, 0.70067, 0.69988, 0.69876,\
        0.69844, 0.69840, 0.69990, 0.70180, 0.70285, 0.70394, 0.70514, 0.70582, 0.70610, 0.70620, 0.70557,\
        0.70401, 0.70144, 0.69872, 0.69651, 0.69499, 0.69457, 0.69517, 0.69651, 0.69807, 0.69927, 0.69985,\
        0.69838, 0.69523, 0.69161, 0.68785, 0.68408, 0.68039, 0.67683, 0.67341, 0.67012, 0.66692, 0.66374,\
        0.66051, 0.65713, 0.65350, 0.64953, 0.64517, 0.64038, 0.63517, 0.62961, 0.62379, 0.61786, 0.61205,\
        0.60674, 0.60260, 0.59958, 0.59766, 0.59642, 0.59517, 0.59347, 0.59142, 0.58913, 0.58666, 0.58405,\
        0.58134, 0.57854, 0.57566, 0.57268, 0.56959, 0.56638, 0.56302, 0.55949, 0.55577, 0.55183, 0.54766,\
        0.54323, 0.53855, 0.53359, 0.52835, 0.52287, 0.51720, 0.51143, 0.50571, 0.50010, 0.49388, 0.48667],\
        [0.60729, 0.60837, 0.60987, 0.61190, 0.61406, 0.61629, 0.61863, 0.62110, 0.62358, 0.62592, 0.62806,\
        0.63002, 0.63186, 0.63370, 0.63563, 0.63775, 0.64010, 0.64270, 0.64553, 0.64862, 0.65196, 0.65559,\
        0.65950, 0.66368, 0.66809, 0.67263, 0.67716, 0.68151, 0.68546, 0.68870, 0.69089, 0.69182, 0.69200,\
        0.69197, 0.69196, 0.69206, 0.69230, 0.69277, 0.69325, 0.69336, 0.69325, 0.69305, 0.69245, 0.69169,\
        0.69145, 0.69184, 0.69321, 0.69487, 0.69610, 0.69740, 0.69897, 0.69984, 0.70035, 0.70135, 0.70112,\
        0.70029, 0.69827, 0.69592, 0.69407, 0.69291, 0.69259, 0.69310, 0.69419, 0.69547, 0.69649, 0.69667,\
        0.69527, 0.69243, 0.68887, 0.68503, 0.68111, 0.67722, 0.67342, 0.66972, 0.66613, 0.66261, 0.65912,\
        0.65558, 0.65194, 0.64810, 0.64400, 0.63958, 0.63481, 0.62970, 0.62427, 0.61862, 0.61285, 0.60715,\
        0.60185, 0.59781, 0.59520, 0.59336, 0.59205, 0.59080, 0.58931, 0.58752, 0.58547, 0.58323, 0.58083,\
        0.57830, 0.57565, 0.57289, 0.57001, 0.56701, 0.56386, 0.56055, 0.55706, 0.55336, 0.54944, 0.54528,\
        0.54085, 0.53615, 0.53114, 0.52582, 0.52019, 0.51428, 0.50819, 0.50204, 0.49604, 0.48976, 0.48270],\
        [0.60086, 0.60083, 0.60252, 0.60472, 0.60690, 0.60911, 0.61151, 0.61419, 0.61691, 0.61938, 0.62158,\
        0.62352, 0.62532, 0.62713, 0.62910, 0.63134, 0.63390, 0.63674, 0.63984, 0.64317, 0.64671, 0.65046,\
        0.65441, 0.65854, 0.66279, 0.66709, 0.67131, 0.67528, 0.67881, 0.68166, 0.68362, 0.68466, 0.68504,\
        0.68511, 0.68512, 0.68519, 0.68538, 0.68568, 0.68598, 0.68611, 0.68606, 0.68588, 0.68553, 0.68519,\
        0.68521, 0.68580, 0.68700, 0.68847, 0.68991, 0.69141, 0.69295, 0.69414, 0.69517, 0.69621, 0.69637,\
        0.69585, 0.69452, 0.69282, 0.69138, 0.69048, 0.69023, 0.69058, 0.69135, 0.69222, 0.69278, 0.69254,\
        0.69113, 0.68857, 0.68524, 0.68150, 0.67756, 0.67358, 0.66961, 0.66572, 0.66189, 0.65812, 0.65438,\
        0.65063, 0.64681, 0.64286, 0.63871, 0.63433, 0.62966, 0.62471, 0.61952, 0.61414, 0.60870, 0.60336,\
        0.59847, 0.59452, 0.59174, 0.58976, 0.58824, 0.58686, 0.58539, 0.58372, 0.58184, 0.57977, 0.57753,\
        0.57515, 0.57263, 0.56998, 0.56719, 0.56427, 0.56118, 0.55792, 0.55446, 0.55080, 0.54690, 0.54275,\
        0.53833, 0.53363, 0.52860, 0.52322, 0.51747, 0.51136, 0.50497, 0.49853, 0.49219, 0.48559, 0.47841],\
        [0.59295, 0.59393, 0.59554, 0.59767, 0.59982, 0.60201, 0.60439, 0.60745, 0.61041, 0.61297, 0.61518,\
        0.61707, 0.61877, 0.62049, 0.62248, 0.62486, 0.62764, 0.63076, 0.63413, 0.63767, 0.64136, 0.64519,\
        0.64913, 0.65317, 0.65725, 0.66130, 0.66520, 0.66883, 0.67200, 0.67457, 0.67640, 0.67752, 0.67809,\
        0.67832, 0.67843, 0.67853, 0.67868, 0.67890, 0.67910, 0.67922, 0.67924, 0.67917, 0.67906, 0.67904,\
        0.67932, 0.68002, 0.68114, 0.68254, 0.68405, 0.68562, 0.68717, 0.68856, 0.68980, 0.69078, 0.69121,\
        0.69100, 0.69023, 0.68916, 0.68816, 0.68751, 0.68729, 0.68748, 0.68793, 0.68837, 0.68847, 0.68790,\
        0.68642, 0.68403, 0.68091, 0.67732, 0.67345, 0.66945, 0.66541, 0.66138, 0.65738, 0.65343, 0.64952,\
        0.64562, 0.64170, 0.63772, 0.63361, 0.62934, 0.62486, 0.62016, 0.61525, 0.61025, 0.60524, 0.60027,\
        0.59574, 0.59191, 0.58894, 0.58669, 0.58490, 0.58332, 0.58176, 0.58009, 0.57828, 0.57631, 0.57418,\
        0.57190, 0.56948, 0.56692, 0.56422, 0.56136, 0.55832, 0.55511, 0.55169, 0.54804, 0.54416, 0.54003,\
        0.53562, 0.53093, 0.52591, 0.52053, 0.51474, 0.50850, 0.50189, 0.49521, 0.48847, 0.48145, 0.47398],\
        [0.58457, 0.58656, 0.58847, 0.59060, 0.59281, 0.59509, 0.59765, 0.60106, 0.60404, 0.60657, 0.60883,\
        0.61066, 0.61217, 0.61376, 0.61575, 0.61830, 0.62136, 0.62478, 0.62841, 0.63214, 0.63594, 0.63979,\
        0.64369, 0.64760, 0.65150, 0.65530, 0.65892, 0.66224, 0.66514, 0.66751, 0.66927, 0.67046, 0.67117,\
        0.67157, 0.67180, 0.67197, 0.67214, 0.67233, 0.67251, 0.67265, 0.67275, 0.67281, 0.67291, 0.67313,\
        0.67360, 0.67439, 0.67551, 0.67687, 0.67837, 0.67995, 0.68151, 0.68297, 0.68426, 0.68526, 0.68584,\
        0.68592, 0.68557, 0.68498, 0.68436, 0.68392, 0.68373, 0.68378, 0.68393, 0.68400, 0.68373, 0.68289,\
        0.68133, 0.67902, 0.67606, 0.67262, 0.66886, 0.66489, 0.66082, 0.65671, 0.65261, 0.64854, 0.64451,\
        0.64053, 0.63658, 0.63262, 0.62863, 0.62456, 0.62035, 0.61594, 0.61134, 0.60667, 0.60218, 0.59764,\
        0.59327, 0.58950, 0.58640, 0.58391, 0.58185, 0.58005, 0.57834, 0.57660, 0.57478, 0.57284, 0.57077,\
        0.56856, 0.56621, 0.56371, 0.56107, 0.55826, 0.55527, 0.55209, 0.54869, 0.54506, 0.54119, 0.53706,\
        0.53266, 0.52797, 0.52299, 0.51766, 0.51192, 0.50565, 0.49889, 0.49190, 0.48471, 0.47724, 0.46939],\
        [0.57593, 0.57863, 0.58101, 0.58337, 0.58580, 0.58836, 0.59125, 0.59450, 0.59741, 0.60001, 0.60242,\
        0.60420, 0.60547, 0.60695, 0.60894, 0.61171, 0.61513, 0.61888, 0.62273, 0.62660, 0.63045, 0.63428,\
        0.63809, 0.64186, 0.64556, 0.64914, 0.65251, 0.65559, 0.65827, 0.66050, 0.66222, 0.66346, 0.66430,\
        0.66484, 0.66520, 0.66546, 0.66568, 0.66590, 0.66611, 0.66630, 0.66649, 0.66669, 0.66696, 0.66737,\
        0.66798, 0.66885, 0.66997, 0.67131, 0.67279, 0.67433, 0.67587, 0.67733, 0.67862, 0.67965, 0.68034,\
        0.68064, 0.68061, 0.68036, 0.68004, 0.67976, 0.67959, 0.67951, 0.67943, 0.67920, 0.67865, 0.67760,\
        0.67595, 0.67368, 0.67083, 0.66752, 0.66385, 0.65994, 0.65588, 0.65174, 0.64758, 0.64343, 0.63933,\
        0.63532, 0.63138, 0.62752, 0.62370, 0.61988, 0.61602, 0.61201, 0.60771, 0.60321, 0.59892, 0.59481,\
        0.59075, 0.58707, 0.58390, 0.58122, 0.57893, 0.57691, 0.57503, 0.57318, 0.57130, 0.56934, 0.56728,\
        0.56510, 0.56279, 0.56033, 0.55772, 0.55495, 0.55199, 0.54883, 0.54544, 0.54182, 0.53793, 0.53378,\
        0.52936, 0.52467, 0.51971, 0.51442, 0.50878, 0.50266, 0.49569, 0.48831, 0.48068, 0.47278, 0.46456],\
        [0.56702, 0.57027, 0.57314, 0.57587, 0.57859, 0.58142, 0.58443, 0.58755, 0.59049, 0.59320, 0.59565,\
        0.59743, 0.59869, 0.60021, 0.60219, 0.60528, 0.60910, 0.61313, 0.61713, 0.62105, 0.62489, 0.62865,\
        0.63233, 0.63594, 0.63946, 0.64283, 0.64600, 0.64888, 0.65141, 0.65353, 0.65522, 0.65651, 0.65745,\
        0.65812, 0.65860, 0.65897, 0.65927, 0.65955, 0.65982, 0.66009, 0.66038, 0.66071, 0.66112, 0.66167,\
        0.66239, 0.66332, 0.66446, 0.66578, 0.66721, 0.66872, 0.67021, 0.67164, 0.67291, 0.67396, 0.67472,\
        0.67519, 0.67538, 0.67537, 0.67524, 0.67508, 0.67492, 0.67474, 0.67448, 0.67403, 0.67326, 0.67205,\
        0.67033, 0.66807, 0.66530, 0.66209, 0.65853, 0.65469, 0.65066, 0.64652, 0.64232, 0.63812, 0.63398,\
        0.62996, 0.62608, 0.62234, 0.61871, 0.61518, 0.61173, 0.60826, 0.60447, 0.60007, 0.59594, 0.59194,\
        0.58808, 0.58448, 0.58127, 0.57845, 0.57599, 0.57378, 0.57173, 0.56975, 0.56778, 0.56577, 0.56368,\
        0.56150, 0.55919, 0.55675, 0.55416, 0.55140, 0.54846, 0.54530, 0.54192, 0.53828, 0.53436, 0.53015,\
        0.52567, 0.52093, 0.51594, 0.51062, 0.50486, 0.49878, 0.49176, 0.48411, 0.47615, 0.46790, 0.45935],\
        [0.55783, 0.56155, 0.56488, 0.56799, 0.57103, 0.57409, 0.57720, 0.58031, 0.58328, 0.58602, 0.58844,\
        0.59039, 0.59203, 0.59378, 0.59606, 0.59934, 0.60341, 0.60756, 0.61157, 0.61547, 0.61924, 0.62288,\
        0.62641, 0.62985, 0.63319, 0.63639, 0.63939, 0.64212, 0.64453, 0.64658, 0.64826, 0.64959, 0.65062,\
        0.65140, 0.65199, 0.65247, 0.65287, 0.65324, 0.65359, 0.65395, 0.65434, 0.65478, 0.65531, 0.65597,\
        0.65678, 0.65776, 0.65891, 0.66020, 0.66160, 0.66305, 0.66449, 0.66587, 0.66711, 0.66816, 0.66898,\
        0.66956, 0.66990, 0.67004, 0.67005, 0.66996, 0.66980, 0.66954, 0.66915, 0.66853, 0.66760, 0.66627,\
        0.66450, 0.66225, 0.65954, 0.65642, 0.65295, 0.64919, 0.64522, 0.64109, 0.63687, 0.63264, 0.62847,\
        0.62446, 0.62064, 0.61704, 0.61362, 0.61034, 0.60723, 0.60432, 0.60144, 0.59742, 0.59309, 0.58903,\
        0.58521, 0.58165, 0.57841, 0.57551, 0.57291, 0.57054, 0.56834, 0.56624, 0.56416, 0.56208, 0.55994,\
        0.55771, 0.55539, 0.55294, 0.55034, 0.54758, 0.54464, 0.54148, 0.53808, 0.53441, 0.53044, 0.52614,\
        0.52154, 0.51666, 0.51158, 0.50627, 0.50035, 0.49385, 0.48681, 0.47912, 0.47098, 0.46250, 0.45368],\
        [0.54836, 0.55250, 0.55624, 0.55973, 0.56307, 0.56635, 0.56959, 0.57276, 0.57578, 0.57859, 0.58111,\
        0.58334, 0.58541, 0.58763, 0.59034, 0.59380, 0.59787, 0.60197, 0.60591, 0.60976, 0.61345, 0.61692,\
        0.62026, 0.62354, 0.62674, 0.62981, 0.63268, 0.63529, 0.63761, 0.63961, 0.64129, 0.64266, 0.64377,\
        0.64464, 0.64535, 0.64594, 0.64644, 0.64690, 0.64735, 0.64781, 0.64830, 0.64885, 0.64948, 0.65023,\
        0.65110, 0.65212, 0.65327, 0.65455, 0.65590, 0.65730, 0.65868, 0.66001, 0.66121, 0.66226, 0.66311,\
        0.66374, 0.66417, 0.66442, 0.66450, 0.66446, 0.66428, 0.66397, 0.66347, 0.66273, 0.66169, 0.66028,\
        0.65847, 0.65624, 0.65359, 0.65055, 0.64717, 0.64351, 0.63960, 0.63550, 0.63128, 0.62702, 0.62282,\
        0.61880, 0.61506, 0.61162, 0.60840, 0.60531, 0.60240, 0.59972, 0.59715, 0.59372, 0.58972, 0.58576,\
        0.58200, 0.57849, 0.57526, 0.57230, 0.56960, 0.56711, 0.56478, 0.56256, 0.56038, 0.55820, 0.55599,\
        0.55371, 0.55135, 0.54886, 0.54624, 0.54346, 0.54050, 0.53732, 0.53390, 0.53020, 0.52617, 0.52176,\
        0.51697, 0.51186, 0.50654, 0.50112, 0.49520, 0.48848, 0.48123, 0.47344, 0.46518, 0.45653, 0.44751],\
        [0.53860, 0.54311, 0.54723, 0.55107, 0.55471, 0.55822, 0.56162, 0.56491, 0.56804, 0.57098, 0.57370,\
        0.57624, 0.57874, 0.58140, 0.58445, 0.58802, 0.59198, 0.59601, 0.59996, 0.60380, 0.60745, 0.61070,\
        0.61384, 0.61700, 0.62010, 0.62308, 0.62586, 0.62839, 0.63064, 0.63261, 0.63429, 0.63570, 0.63687,\
        0.63784, 0.63865, 0.63934, 0.63995, 0.64052, 0.64106, 0.64162, 0.64221, 0.64285, 0.64357, 0.64439,\
        0.64532, 0.64636, 0.64752, 0.64877, 0.65008, 0.65143, 0.65275, 0.65402, 0.65519, 0.65622, 0.65707,\
        0.65774, 0.65822, 0.65852, 0.65865, 0.65862, 0.65843, 0.65807, 0.65750, 0.65667, 0.65555, 0.65410,\
        0.65227, 0.65006, 0.64746, 0.64452, 0.64124, 0.63767, 0.63385, 0.62983, 0.62563, 0.62134, 0.61708,\
        0.61304, 0.60937, 0.60608, 0.60310, 0.60019, 0.59741, 0.59484, 0.59222, 0.58914, 0.58561, 0.58195,\
        0.57836, 0.57494, 0.57174, 0.56876, 0.56600, 0.56342, 0.56099, 0.55865, 0.55637, 0.55410, 0.55180,\
        0.54946, 0.54703, 0.54449, 0.54182, 0.53900, 0.53600, 0.53279, 0.52935, 0.52562, 0.52153, 0.51702,\
        0.51203, 0.50662, 0.50098, 0.49526, 0.48918, 0.48242, 0.47506, 0.46716, 0.45880, 0.45000, 0.44082],\
        [0.52855, 0.53340, 0.53787, 0.54203, 0.54596, 0.54971, 0.55331, 0.55676, 0.56005, 0.56318, 0.56615,\
        0.56901, 0.57187, 0.57488, 0.57818, 0.58181, 0.58571, 0.58969, 0.59364, 0.59752, 0.60120, 0.60403,\
        0.60709, 0.61023, 0.61326, 0.61618, 0.61891, 0.62137, 0.62358, 0.62552, 0.62721, 0.62866, 0.62990,\
        0.63095, 0.63186, 0.63265, 0.63337, 0.63404, 0.63469, 0.63534, 0.63602, 0.63675, 0.63755, 0.63843,\
        0.63940, 0.64047, 0.64162, 0.64285, 0.64412, 0.64541, 0.64668, 0.64790, 0.64902, 0.65002, 0.65087,\
        0.65154, 0.65205, 0.65237, 0.65252, 0.65249, 0.65228, 0.65188, 0.65125, 0.65037, 0.64920, 0.64772,\
        0.64589, 0.64371, 0.64119, 0.63833, 0.63516, 0.63172, 0.62802, 0.62410, 0.61998, 0.61568, 0.61133,\
        0.60724, 0.60362, 0.60051, 0.59769, 0.59499, 0.59233, 0.58977, 0.58713, 0.58422, 0.58101, 0.57763,\
        0.57425, 0.57097, 0.56783, 0.56486, 0.56206, 0.55942, 0.55690, 0.55446, 0.55208, 0.54972, 0.54733,\
        0.54490, 0.54239, 0.53978, 0.53705, 0.53416, 0.53111, 0.52785, 0.52436, 0.52060, 0.51650, 0.51193,\
        0.50678, 0.50112, 0.49520, 0.48915, 0.48276, 0.47583, 0.46835, 0.46035, 0.45186, 0.44294, 0.43362],\
        [0.51820, 0.52338, 0.52816, 0.53263, 0.53684, 0.54084, 0.54466, 0.54831, 0.55180, 0.55515, 0.55838,\
        0.56153, 0.56471, 0.56799, 0.57147, 0.57517, 0.57904, 0.58298, 0.58689, 0.59070, 0.59414, 0.59699,\
        0.60020, 0.60323, 0.60618, 0.60913, 0.61181, 0.61422, 0.61639, 0.61832, 0.62002, 0.62151, 0.62282,\
        0.62395, 0.62495, 0.62585, 0.62667, 0.62744, 0.62819, 0.62893, 0.62970, 0.63051, 0.63138, 0.63231,\
        0.63332, 0.63440, 0.63555, 0.63675, 0.63799, 0.63923, 0.64045, 0.64162, 0.64270, 0.64366, 0.64448,\
        0.64515, 0.64565, 0.64598, 0.64613, 0.64609, 0.64586, 0.64543, 0.64476, 0.64385, 0.64265, 0.64115,\
        0.63934, 0.63721, 0.63475, 0.63199, 0.62895, 0.62564, 0.62209, 0.61833, 0.61437, 0.61018, 0.60575,\
        0.60156, 0.59805, 0.59504, 0.59229, 0.58967, 0.58709, 0.58454, 0.58190, 0.57909, 0.57608, 0.57293,\
        0.56973, 0.56658, 0.56352, 0.56058, 0.55776, 0.55507, 0.55248, 0.54996, 0.54748, 0.54502, 0.54254,\
        0.54001, 0.53741, 0.53471, 0.53189, 0.52892, 0.52578, 0.52245, 0.51888, 0.51507, 0.51096, 0.50643,\
        0.50115, 0.49528, 0.48911, 0.48271, 0.47598, 0.46882, 0.46117, 0.45303, 0.44442, 0.43538, 0.42592],\
        [0.50756, 0.51303, 0.51812, 0.52288, 0.52737, 0.53162, 0.53568, 0.53955, 0.54327, 0.54685, 0.55033,\
        0.55375, 0.55719, 0.56069, 0.56432, 0.56809, 0.57195, 0.57584, 0.57968, 0.58336, 0.58674, 0.58987,\
        0.59299, 0.59596, 0.59895, 0.60191, 0.60452, 0.60688, 0.60903, 0.61096, 0.61269, 0.61423, 0.61560,\
        0.61682, 0.61791, 0.61890, 0.61982, 0.62069, 0.62153, 0.62237, 0.62322, 0.62410, 0.62503, 0.62601,\
        0.62705, 0.62814, 0.62928, 0.63047, 0.63167, 0.63287, 0.63404, 0.63516, 0.63619, 0.63712, 0.63791,\
        0.63856, 0.63904, 0.63936, 0.63950, 0.63945, 0.63920, 0.63874, 0.63804, 0.63710, 0.63590, 0.63441,\
        0.63262, 0.63054, 0.62816, 0.62550, 0.62258, 0.61942, 0.61604, 0.61247, 0.60873, 0.60483, 0.60067,\
        0.59634, 0.59281, 0.58974, 0.58694, 0.58430, 0.58173, 0.57916, 0.57652, 0.57377, 0.57089, 0.56789,\
        0.56484, 0.56180, 0.55881, 0.55590, 0.55309, 0.55036, 0.54770, 0.54510, 0.54254, 0.53998, 0.53739,\
        0.53476, 0.53206, 0.52925, 0.52633, 0.52325, 0.52000, 0.51655, 0.51287, 0.50894, 0.50472, 0.50022,\
        0.49488, 0.48884, 0.48245, 0.47577, 0.46877, 0.46137, 0.45353, 0.44524, 0.43650, 0.42732, 0.41774]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=60 (m/s) and translational velocity of Vt=2.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax60vt2.5
    zgrid_H_Hmax_vmax60vt2_5=[[0.45892, 0.46607, 0.47262, 0.47848, 0.48350, 0.48756, 0.49091, 0.49425, 0.49652, 0.49755, 0.49899,\
        0.50258, 0.50724, 0.51242, 0.51785, 0.52331, 0.52867, 0.53379, 0.53855, 0.54289, 0.54675, 0.55014,\
        0.55307, 0.55562, 0.55787, 0.55994, 0.56193, 0.56394, 0.56608, 0.56844, 0.57106, 0.57402, 0.57732,\
        0.58100, 0.58505, 0.58947, 0.59421, 0.59896, 0.60308, 0.60696, 0.61081, 0.61463, 0.61842, 0.62217,\
        0.62585, 0.62945, 0.63295, 0.63634, 0.63959, 0.64270, 0.64565, 0.64843, 0.65105, 0.65350, 0.65578,\
        0.65787, 0.65976, 0.66142, 0.66282, 0.66393, 0.66472, 0.66517, 0.66527, 0.66502, 0.66445, 0.66357,\
        0.66242, 0.66104, 0.65946, 0.65770, 0.65578, 0.65373, 0.65155, 0.64926, 0.64687, 0.64437, 0.64179,\
        0.63912, 0.63636, 0.63353, 0.63063, 0.62765, 0.62461, 0.62152, 0.61837, 0.61519, 0.61198, 0.60876,\
        0.60554, 0.60235, 0.59919, 0.59612, 0.59308, 0.58991, 0.58658, 0.58308, 0.57940, 0.57554, 0.57153,\
        0.56738, 0.56312, 0.55877, 0.55436, 0.54989, 0.54539, 0.54086, 0.53632, 0.53179, 0.52728, 0.52282,\
        0.51843, 0.51414, 0.50995, 0.50588, 0.50195, 0.49718, 0.49111, 0.48457, 0.47771, 0.47059, 0.46321],\
        [0.46599, 0.47320, 0.47983, 0.48580, 0.49091, 0.49482, 0.49747, 0.50063, 0.50339, 0.50472, 0.50655,\
        0.51003, 0.51464, 0.51984, 0.52531, 0.53082, 0.53621, 0.54133, 0.54605, 0.55027, 0.55393, 0.55702,\
        0.55958, 0.56170, 0.56348, 0.56506, 0.56657, 0.56812, 0.56984, 0.57181, 0.57410, 0.57677, 0.57983,\
        0.58328, 0.58712, 0.59134, 0.59592, 0.60069, 0.60521, 0.60948, 0.61368, 0.61781, 0.62189, 0.62589,\
        0.62980, 0.63362, 0.63731, 0.64088, 0.64429, 0.64754, 0.65062, 0.65352, 0.65625, 0.65880, 0.66117,\
        0.66337, 0.66537, 0.66715, 0.66866, 0.66986, 0.67071, 0.67119, 0.67129, 0.67100, 0.67035, 0.66939,\
        0.66814, 0.66665, 0.66496, 0.66310, 0.66109, 0.65894, 0.65667, 0.65429, 0.65180, 0.64922, 0.64655,\
        0.64379, 0.64094, 0.63801, 0.63500, 0.63192, 0.62877, 0.62556, 0.62229, 0.61899, 0.61566, 0.61232,\
        0.60899, 0.60570, 0.60246, 0.59935, 0.59632, 0.59305, 0.58963, 0.58605, 0.58226, 0.57828, 0.57413,\
        0.56983, 0.56543, 0.56094, 0.55638, 0.55176, 0.54710, 0.54240, 0.53768, 0.53294, 0.52820, 0.52348,\
        0.51882, 0.51422, 0.50969, 0.50520, 0.50069, 0.49629, 0.49105, 0.48515, 0.47883, 0.47219, 0.46525],\
        [0.47248, 0.47968, 0.48633, 0.49236, 0.49760, 0.50160, 0.50411, 0.50681, 0.50961, 0.51187, 0.51441,\
        0.51794, 0.52243, 0.52754, 0.53297, 0.53849, 0.54389, 0.54900, 0.55366, 0.55775, 0.56119, 0.56395,\
        0.56610, 0.56773, 0.56899, 0.57004, 0.57103, 0.57209, 0.57335, 0.57492, 0.57688, 0.57926, 0.58209,\
        0.58532, 0.58891, 0.59283, 0.59712, 0.60188, 0.60684, 0.61165, 0.61630, 0.62081, 0.62520, 0.62948,\
        0.63364, 0.63768, 0.64158, 0.64534, 0.64892, 0.65232, 0.65553, 0.65855, 0.66137, 0.66401, 0.66648,\
        0.66879, 0.67091, 0.67281, 0.67445, 0.67575, 0.67668, 0.67719, 0.67728, 0.67695, 0.67623, 0.67516,\
        0.67380, 0.67220, 0.67041, 0.66844, 0.66633, 0.66409, 0.66173, 0.65926, 0.65669, 0.65402, 0.65125,\
        0.64840, 0.64546, 0.64243, 0.63932, 0.63613, 0.63286, 0.62952, 0.62613, 0.62268, 0.61922, 0.61574,\
        0.61230, 0.60890, 0.60561, 0.60246, 0.59930, 0.59589, 0.59243, 0.58879, 0.58490, 0.58078, 0.57648,\
        0.57204, 0.56749, 0.56287, 0.55818, 0.55342, 0.54861, 0.54375, 0.53883, 0.53387, 0.52889, 0.52389,\
        0.51890, 0.51396, 0.50908, 0.50430, 0.49964, 0.49518, 0.49044, 0.48518, 0.47947, 0.47337, 0.46692],\
        [0.47836, 0.48548, 0.49203, 0.49802, 0.50317, 0.50723, 0.51032, 0.51315, 0.51602, 0.51885, 0.52198,\
        0.52575, 0.53024, 0.53530, 0.54068, 0.54618, 0.55159, 0.55670, 0.56133, 0.56530, 0.56850, 0.57091,\
        0.57259, 0.57369, 0.57438, 0.57486, 0.57530, 0.57585, 0.57664, 0.57781, 0.57944, 0.58158, 0.58423,\
        0.58733, 0.59079, 0.59452, 0.59860, 0.60347, 0.60873, 0.61392, 0.61895, 0.62380, 0.62849, 0.63304,\
        0.63744, 0.64170, 0.64581, 0.64976, 0.65352, 0.65708, 0.66042, 0.66354, 0.66645, 0.66916, 0.67172,\
        0.67412, 0.67636, 0.67841, 0.68019, 0.68162, 0.68262, 0.68318, 0.68326, 0.68288, 0.68207, 0.68089,\
        0.67941, 0.67769, 0.67580, 0.67374, 0.67154, 0.66920, 0.66676, 0.66420, 0.66154, 0.65879, 0.65593,\
        0.65298, 0.64994, 0.64682, 0.64360, 0.64030, 0.63691, 0.63344, 0.62990, 0.62630, 0.62267, 0.61904,\
        0.61544, 0.61191, 0.60848, 0.60514, 0.60178, 0.59838, 0.59500, 0.59131, 0.58729, 0.58302, 0.57857,\
        0.57400, 0.56933, 0.56460, 0.55980, 0.55493, 0.54999, 0.54498, 0.53989, 0.53473, 0.52951, 0.52423,\
        0.51894, 0.51366, 0.50848, 0.50351, 0.49884, 0.49438, 0.48990, 0.48512, 0.47994, 0.47436, 0.46840],\
        [0.48368, 0.49068, 0.49694, 0.50283, 0.50798, 0.51226, 0.51585, 0.51910, 0.52230, 0.52559, 0.52916,\
        0.53321, 0.53781, 0.54287, 0.54825, 0.55375, 0.55918, 0.56432, 0.56895, 0.57282, 0.57580, 0.57784,\
        0.57901, 0.57953, 0.57962, 0.57951, 0.57939, 0.57942, 0.57975, 0.58053, 0.58186, 0.58380, 0.58636,\
        0.58948, 0.59302, 0.59684, 0.60102, 0.60591, 0.61123, 0.61660, 0.62186, 0.62696, 0.63189, 0.63666,\
        0.64127, 0.64574, 0.65005, 0.65419, 0.65813, 0.66185, 0.66532, 0.66853, 0.67151, 0.67427, 0.67689,\
        0.67937, 0.68174, 0.68393, 0.68589, 0.68745, 0.68853, 0.68914, 0.68923, 0.68878, 0.68787, 0.68658,\
        0.68497, 0.68314, 0.68115, 0.67901, 0.67672, 0.67430, 0.67177, 0.66913, 0.66639, 0.66355, 0.66060,\
        0.65756, 0.65443, 0.65120, 0.64788, 0.64447, 0.64095, 0.63734, 0.63364, 0.62987, 0.62605, 0.62222,\
        0.61842, 0.61469, 0.61106, 0.60754, 0.60410, 0.60077, 0.59743, 0.59356, 0.58938, 0.58496, 0.58039,\
        0.57571, 0.57096, 0.56616, 0.56129, 0.55635, 0.55132, 0.54620, 0.54098, 0.53565, 0.53022, 0.52471,\
        0.51914, 0.51357, 0.50808, 0.50287, 0.49815, 0.49386, 0.48967, 0.48527, 0.48052, 0.47538, 0.46985],\
        [0.48844, 0.49551, 0.50150, 0.50717, 0.51233, 0.51686, 0.52087, 0.52458, 0.52821, 0.53194, 0.53590,\
        0.54023, 0.54497, 0.55009, 0.55548, 0.56100, 0.56648, 0.57170, 0.57638, 0.58022, 0.58300, 0.58464,\
        0.58527, 0.58516, 0.58465, 0.58397, 0.58332, 0.58284, 0.58272, 0.58312, 0.58419, 0.58599, 0.58854,\
        0.59180, 0.59564, 0.59979, 0.60419, 0.60908, 0.61437, 0.61978, 0.62515, 0.63039, 0.63548, 0.64041,\
        0.64519, 0.64983, 0.65433, 0.65865, 0.66278, 0.66667, 0.67027, 0.67358, 0.67659, 0.67938, 0.68202,\
        0.68458, 0.68704, 0.68938, 0.69153, 0.69327, 0.69437, 0.69503, 0.69518, 0.69462, 0.69364, 0.69225,\
        0.69048, 0.68855, 0.68651, 0.68429, 0.68191, 0.67941, 0.67680, 0.67408, 0.67126, 0.66833, 0.66529,\
        0.66215, 0.65892, 0.65560, 0.65218, 0.64865, 0.64501, 0.64125, 0.63738, 0.63342, 0.62938, 0.62531,\
        0.62126, 0.61727, 0.61338, 0.60965, 0.60611, 0.60277, 0.59927, 0.59535, 0.59109, 0.58656, 0.58190,\
        0.57718, 0.57241, 0.56759, 0.56271, 0.55775, 0.55268, 0.54749, 0.54218, 0.53674, 0.53118, 0.52550,\
        0.51974, 0.51395, 0.50822, 0.50273, 0.49795, 0.49392, 0.48999, 0.48585, 0.48140, 0.47659, 0.47142],\
        [0.49236, 0.49958, 0.50566, 0.51114, 0.51631, 0.52107, 0.52547, 0.52963, 0.53371, 0.53784, 0.54216,\
        0.54675, 0.55165, 0.55683, 0.56225, 0.56778, 0.57330, 0.57861, 0.58342, 0.58730, 0.58992, 0.59119,\
        0.59123, 0.59048, 0.58941, 0.58824, 0.58712, 0.58617, 0.58560, 0.58565, 0.58650, 0.58822, 0.59082,\
        0.59426, 0.59844, 0.60296, 0.60767, 0.61271, 0.61803, 0.62346, 0.62887, 0.63417, 0.63933, 0.64435,\
        0.64924, 0.65401, 0.65866, 0.66317, 0.66748, 0.67156, 0.67532, 0.67873, 0.68176, 0.68452, 0.68716,\
        0.68977, 0.69232, 0.69476, 0.69707, 0.69908, 0.70010, 0.70079, 0.70101, 0.70035, 0.69935, 0.69791,\
        0.69591, 0.69398, 0.69193, 0.68963, 0.68715, 0.68456, 0.68188, 0.67909, 0.67618, 0.67316, 0.67002,\
        0.66678, 0.66346, 0.66005, 0.65653, 0.65290, 0.64913, 0.64522, 0.64117, 0.63698, 0.63270, 0.62836,\
        0.62400, 0.61968, 0.61547, 0.61142, 0.60761, 0.60403, 0.60049, 0.59670, 0.59236, 0.58776, 0.58310,\
        0.57841, 0.57371, 0.56895, 0.56413, 0.55919, 0.55414, 0.54895, 0.54360, 0.53811, 0.53248, 0.52672,\
        0.52088, 0.51501, 0.50922, 0.50369, 0.49890, 0.49494, 0.49105, 0.48698, 0.48267, 0.47811, 0.47322],\
        [0.49531, 0.50235, 0.50871, 0.51446, 0.51983, 0.52490, 0.52969, 0.53428, 0.53878, 0.54330, 0.54794,\
        0.55276, 0.55780, 0.56305, 0.56845, 0.57395, 0.57944, 0.58480, 0.58979, 0.59381, 0.59623, 0.59722,\
        0.59667, 0.59535, 0.59387, 0.59236, 0.59087, 0.58949, 0.58848, 0.58820, 0.58889, 0.59063, 0.59333,\
        0.59692, 0.60139, 0.60633, 0.61145, 0.61675, 0.62217, 0.62762, 0.63302, 0.63830, 0.64346, 0.64849,\
        0.65342, 0.65827, 0.66305, 0.66772, 0.67224, 0.67653, 0.68049, 0.68403, 0.68708, 0.68974, 0.69234,\
        0.69504, 0.69767, 0.70018, 0.70254, 0.70455, 0.70579, 0.70647, 0.70659, 0.70605, 0.70502, 0.70346,\
        0.70144, 0.69956, 0.69750, 0.69506, 0.69244, 0.68979, 0.68705, 0.68418, 0.68119, 0.67807, 0.67482,\
        0.67147, 0.66805, 0.66456, 0.66096, 0.65723, 0.65335, 0.64928, 0.64504, 0.64063, 0.63607, 0.63142,\
        0.62672, 0.62202, 0.61739, 0.61290, 0.60865, 0.60474, 0.60134, 0.59758, 0.59308, 0.58853, 0.58399,\
        0.57947, 0.57493, 0.57031, 0.56560, 0.56077, 0.55578, 0.55063, 0.54531, 0.53983, 0.53421, 0.52845,\
        0.52263, 0.51681, 0.51113, 0.50581, 0.50110, 0.49699, 0.49284, 0.48863, 0.48438, 0.47997, 0.47531],\
        [0.49784, 0.50470, 0.51115, 0.51720, 0.52293, 0.52839, 0.53359, 0.53860, 0.54349, 0.54835, 0.55327,\
        0.55829, 0.56345, 0.56873, 0.57409, 0.57946, 0.58479, 0.59002, 0.59503, 0.59935, 0.60141, 0.60216,\
        0.60132, 0.59977, 0.59811, 0.59643, 0.59472, 0.59294, 0.59145, 0.59085, 0.59152, 0.59345, 0.59642,\
        0.60026, 0.60502, 0.61030, 0.61576, 0.62128, 0.62680, 0.63226, 0.63761, 0.64281, 0.64788, 0.65284,\
        0.65774, 0.66261, 0.66747, 0.67230, 0.67703, 0.68156, 0.68577, 0.68952, 0.69266, 0.69516, 0.69763,\
        0.70051, 0.70326, 0.70584, 0.70818, 0.71010, 0.71143, 0.71213, 0.71224, 0.71175, 0.71071, 0.70918,\
        0.70731, 0.70534, 0.70315, 0.70057, 0.69783, 0.69520, 0.69235, 0.68939, 0.68632, 0.68309, 0.67970,\
        0.67623, 0.67272, 0.66916, 0.66550, 0.66170, 0.65771, 0.65349, 0.64905, 0.64440, 0.63956, 0.63458,\
        0.62950, 0.62439, 0.61930, 0.61430, 0.60945, 0.60486, 0.60063, 0.59714, 0.59311, 0.58891, 0.58470,\
        0.58047, 0.57617, 0.57177, 0.56723, 0.56254, 0.55766, 0.55260, 0.54736, 0.54195, 0.53638, 0.53070,\
        0.52496, 0.51925, 0.51372, 0.50853, 0.50387, 0.49954, 0.49501, 0.49066, 0.48646, 0.48220, 0.47774],\
        [0.49998, 0.50664, 0.51316, 0.51958, 0.52575, 0.53164, 0.53727, 0.54267, 0.54791, 0.55307, 0.55822,\
        0.56341, 0.56866, 0.57396, 0.57923, 0.58441, 0.58944, 0.59430, 0.59892, 0.60290, 0.60526, 0.60599,\
        0.60540, 0.60402, 0.60235, 0.60059, 0.59882, 0.59660, 0.59462, 0.59377, 0.59455, 0.59691, 0.60036,\
        0.60463, 0.60964, 0.61511, 0.62076, 0.62642, 0.63199, 0.63742, 0.64266, 0.64771, 0.65260, 0.65740,\
        0.66218, 0.66700, 0.67190, 0.67685, 0.68179, 0.68657, 0.69103, 0.69511, 0.69848, 0.70099, 0.70340,\
        0.70630, 0.70913, 0.71173, 0.71402, 0.71588, 0.71719, 0.71791, 0.71803, 0.71756, 0.71657, 0.71511,\
        0.71330, 0.71125, 0.70893, 0.70630, 0.70357, 0.70081, 0.69780, 0.69476, 0.69163, 0.68824, 0.68466,\
        0.68107, 0.67748, 0.67387, 0.67018, 0.66633, 0.66225, 0.65789, 0.65325, 0.64835, 0.64321, 0.63790,\
        0.63246, 0.62694, 0.62139, 0.61589, 0.61049, 0.60531, 0.60065, 0.59672, 0.59302, 0.58927, 0.58546,\
        0.58158, 0.57759, 0.57344, 0.56911, 0.56458, 0.55985, 0.55491, 0.54979, 0.54448, 0.53902, 0.53344,\
        0.52781, 0.52221, 0.51675, 0.51153, 0.50660, 0.50184, 0.49713, 0.49300, 0.48896, 0.48484, 0.48054],\
        [0.50047, 0.50768, 0.51485, 0.52183, 0.52850, 0.53483, 0.54085, 0.54660, 0.55213, 0.55753, 0.56286,\
        0.56817, 0.57350, 0.57881, 0.58402, 0.58902, 0.59370, 0.59809, 0.60238, 0.60605, 0.60846, 0.60945,\
        0.60924, 0.60819, 0.60667, 0.60491, 0.60294, 0.60042, 0.59835, 0.59727, 0.59828, 0.60126, 0.60523,\
        0.60991, 0.61516, 0.62077, 0.62652, 0.63223, 0.63779, 0.64312, 0.64818, 0.65300, 0.65762, 0.66216,\
        0.66672, 0.67140, 0.67627, 0.68131, 0.68647, 0.69152, 0.69616, 0.70044, 0.70411, 0.70700, 0.70969,\
        0.71251, 0.71530, 0.71788, 0.72011, 0.72190, 0.72317, 0.72387, 0.72399, 0.72355, 0.72259, 0.72119,\
        0.71941, 0.71732, 0.71494, 0.71232, 0.70952, 0.70659, 0.70348, 0.70035, 0.69717, 0.69349, 0.68968,\
        0.68598, 0.68235, 0.67873, 0.67504, 0.67117, 0.66702, 0.66252, 0.65767, 0.65251, 0.64709, 0.64146,\
        0.63567, 0.62979, 0.62385, 0.61792, 0.61210, 0.60652, 0.60143, 0.59715, 0.59350, 0.59002, 0.58656,\
        0.58303, 0.57933, 0.57543, 0.57131, 0.56695, 0.56238, 0.55759, 0.55260, 0.54743, 0.54210, 0.53665,\
        0.53113, 0.52562, 0.52018, 0.51486, 0.50967, 0.50455, 0.49992, 0.49600, 0.49198, 0.48791, 0.48375],\
        [0.50114, 0.50898, 0.51682, 0.52432, 0.53143, 0.53815, 0.54450, 0.55052, 0.55627, 0.56183, 0.56727,\
        0.57267, 0.57804, 0.58338, 0.58858, 0.59349, 0.59799, 0.60212, 0.60604, 0.60942, 0.61178, 0.61299,\
        0.61312, 0.61243, 0.61116, 0.60949, 0.60746, 0.60520, 0.60339, 0.60232, 0.60351, 0.60689, 0.61119,\
        0.61613, 0.62154, 0.62723, 0.63301, 0.63871, 0.64419, 0.64936, 0.65418, 0.65868, 0.66294, 0.66711,\
        0.67134, 0.67580, 0.68056, 0.68563, 0.69095, 0.69638, 0.70123, 0.70560, 0.70953, 0.71291, 0.71598,\
        0.71891, 0.72171, 0.72425, 0.72642, 0.72816, 0.72937, 0.73004, 0.73015, 0.72972, 0.72879, 0.72742,\
        0.72565, 0.72355, 0.72116, 0.71851, 0.71565, 0.71261, 0.70943, 0.70615, 0.70270, 0.69871, 0.69476,\
        0.69103, 0.68737, 0.68376, 0.68011, 0.67626, 0.67205, 0.66740, 0.66234, 0.65692, 0.65121, 0.64529,\
        0.63921, 0.63302, 0.62677, 0.62055, 0.61443, 0.60856, 0.60316, 0.59868, 0.59488, 0.59147, 0.58825,\
        0.58500, 0.58155, 0.57785, 0.57390, 0.56970, 0.56528, 0.56065, 0.55581, 0.55078, 0.54559, 0.54028,\
        0.53488, 0.52946, 0.52405, 0.51872, 0.51348, 0.50845, 0.50389, 0.49976, 0.49540, 0.49140, 0.48741],\
        [0.50415, 0.51179, 0.51968, 0.52740, 0.53478, 0.54176, 0.54834, 0.55455, 0.56043, 0.56607, 0.57155,\
        0.57696, 0.58235, 0.58772, 0.59297, 0.59791, 0.60233, 0.60635, 0.61001, 0.61314, 0.61546, 0.61683,\
        0.61728, 0.61695, 0.61601, 0.61462, 0.61293, 0.61120, 0.60988, 0.60951, 0.61088, 0.61404, 0.61831,\
        0.62328, 0.62870, 0.63439, 0.64015, 0.64579, 0.65116, 0.65613, 0.66066, 0.66476, 0.66857, 0.67225,\
        0.67605, 0.68019, 0.68479, 0.68983, 0.69515, 0.70074, 0.70606, 0.71076, 0.71502, 0.71883, 0.72225,\
        0.72539, 0.72826, 0.73080, 0.73295, 0.73463, 0.73580, 0.73642, 0.73651, 0.73608, 0.73515, 0.73379,\
        0.73204, 0.72994, 0.72753, 0.72486, 0.72195, 0.71883, 0.71552, 0.71202, 0.70824, 0.70415, 0.70016,\
        0.69635, 0.69260, 0.68898, 0.68543, 0.68164, 0.67737, 0.67254, 0.66723, 0.66154, 0.65557, 0.64940,\
        0.64308, 0.63666, 0.63021, 0.62380, 0.61752, 0.61153, 0.60605, 0.60127, 0.59710, 0.59371, 0.59069,\
        0.58763, 0.58434, 0.58076, 0.57692, 0.57285, 0.56856, 0.56407, 0.55939, 0.55451, 0.54948, 0.54430,\
        0.53903, 0.53371, 0.52838, 0.52309, 0.51790, 0.51291, 0.50819, 0.50360, 0.49918, 0.49544, 0.49158],\
        [0.50814, 0.51567, 0.52344, 0.53117, 0.53865, 0.54577, 0.55248, 0.55879, 0.56473, 0.57035, 0.57578,\
        0.58112, 0.58646, 0.59184, 0.59716, 0.60213, 0.60657, 0.61058, 0.61415, 0.61719, 0.61953, 0.62107,\
        0.62183, 0.62188, 0.62135, 0.62041, 0.61925, 0.61815, 0.61749, 0.61774, 0.61931, 0.62228, 0.62635,\
        0.63117, 0.63649, 0.64210, 0.64779, 0.65335, 0.65859, 0.66335, 0.66756, 0.67123, 0.67449, 0.67759,\
        0.68086, 0.68460, 0.68904, 0.69414, 0.69956, 0.70519, 0.71081, 0.71598, 0.72064, 0.72483, 0.72859,\
        0.73196, 0.73494, 0.73752, 0.73966, 0.74130, 0.74242, 0.74300, 0.74305, 0.74259, 0.74166, 0.74030,\
        0.73855, 0.73646, 0.73405, 0.73135, 0.72840, 0.72520, 0.72177, 0.71811, 0.71419, 0.71010, 0.70601,\
        0.70201, 0.69810, 0.69442, 0.69104, 0.68735, 0.68296, 0.67787, 0.67228, 0.66632, 0.66013, 0.65376,\
        0.64727, 0.64071, 0.63414, 0.62764, 0.62130, 0.61525, 0.60964, 0.60457, 0.60018, 0.59690, 0.59398,\
        0.59100, 0.58772, 0.58414, 0.58034, 0.57636, 0.57220, 0.56785, 0.56332, 0.55860, 0.55371, 0.54868,\
        0.54354, 0.53833, 0.53309, 0.52788, 0.52274, 0.51775, 0.51294, 0.50830, 0.50401, 0.50012, 0.49627],\
        [0.51270, 0.52019, 0.52787, 0.53554, 0.54302, 0.55020, 0.55699, 0.56333, 0.56924, 0.57478, 0.58006,\
        0.58526, 0.59048, 0.59577, 0.60111, 0.60615, 0.61073, 0.61483, 0.61845, 0.62152, 0.62397, 0.62573,\
        0.62680, 0.62726, 0.62720, 0.62679, 0.62622, 0.62576, 0.62571, 0.62644, 0.62820, 0.63107, 0.63491,\
        0.63950, 0.64464, 0.65012, 0.65572, 0.66121, 0.66634, 0.67092, 0.67482, 0.67805, 0.68073, 0.68317,\
        0.68581, 0.68911, 0.69338, 0.69862, 0.70435, 0.71019, 0.71599, 0.72147, 0.72650, 0.73103, 0.73507,\
        0.73865, 0.74176, 0.74439, 0.74653, 0.74813, 0.74920, 0.74973, 0.74973, 0.74923, 0.74828, 0.74691,\
        0.74517, 0.74308, 0.74067, 0.73797, 0.73498, 0.73173, 0.72822, 0.72446, 0.72048, 0.71634, 0.71217,\
        0.70802, 0.70394, 0.70021, 0.69700, 0.69335, 0.68867, 0.68325, 0.67735, 0.67116, 0.66480, 0.65832,\
        0.65174, 0.64511, 0.63850, 0.63198, 0.62564, 0.61960, 0.61398, 0.60890, 0.60457, 0.60107, 0.59806,\
        0.59503, 0.59160, 0.58790, 0.58409, 0.58019, 0.57615, 0.57194, 0.56755, 0.56299, 0.55825, 0.55337,\
        0.54835, 0.54325, 0.53811, 0.53297, 0.52789, 0.52292, 0.51811, 0.51351, 0.50921, 0.50518, 0.50119],\
        [0.51762, 0.52511, 0.53273, 0.54035, 0.54782, 0.55503, 0.56185, 0.56821, 0.57407, 0.57944, 0.58451,\
        0.58951, 0.59461, 0.59979, 0.60512, 0.61025, 0.61496, 0.61920, 0.62293, 0.62613, 0.62874, 0.63075,\
        0.63217, 0.63304, 0.63348, 0.63361, 0.63363, 0.63375, 0.63423, 0.63532, 0.63722, 0.64001, 0.64362,\
        0.64795, 0.65285, 0.65815, 0.66364, 0.66909, 0.67419, 0.67864, 0.68228, 0.68512, 0.68726, 0.68902,\
        0.69104, 0.69392, 0.69803, 0.70348, 0.70950, 0.71564, 0.72166, 0.72738, 0.73267, 0.73747, 0.74174,\
        0.74549, 0.74871, 0.75140, 0.75353, 0.75511, 0.75612, 0.75659, 0.75652, 0.75597, 0.75498, 0.75359,\
        0.75185, 0.74978, 0.74739, 0.74469, 0.74170, 0.73841, 0.73485, 0.73104, 0.72700, 0.72282, 0.71858,\
        0.71435, 0.71025, 0.70650, 0.70308, 0.69927, 0.69413, 0.68843, 0.68227, 0.67595, 0.66954, 0.66303,\
        0.65644, 0.64981, 0.64322, 0.63673, 0.63044, 0.62446, 0.61889, 0.61387, 0.60952, 0.60584, 0.60263,\
        0.59948, 0.59569, 0.59186, 0.58809, 0.58428, 0.58037, 0.57630, 0.57205, 0.56763, 0.56304, 0.55828,\
        0.55339, 0.54840, 0.54334, 0.53826, 0.53322, 0.52827, 0.52345, 0.51882, 0.51439, 0.51009, 0.50563],\
        [0.52276, 0.53027, 0.53786, 0.54545, 0.55292, 0.56016, 0.56704, 0.57345, 0.57927, 0.58446, 0.58921,\
        0.59397, 0.59902, 0.60424, 0.60952, 0.61464, 0.61942, 0.62377, 0.62763, 0.63099, 0.63381, 0.63609,\
        0.63786, 0.63915, 0.64006, 0.64072, 0.64127, 0.64189, 0.64279, 0.64416, 0.64614, 0.64881, 0.65217,\
        0.65619, 0.66079, 0.66584, 0.67121, 0.67665, 0.68179, 0.68620, 0.68963, 0.69220, 0.69401, 0.69514,\
        0.69673, 0.69954, 0.70367, 0.70922, 0.71536, 0.72165, 0.72783, 0.73373, 0.73921, 0.74419, 0.74863,\
        0.75251, 0.75580, 0.75852, 0.76065, 0.76219, 0.76314, 0.76352, 0.76337, 0.76274, 0.76170, 0.76030,\
        0.75857, 0.75652, 0.75417, 0.75149, 0.74851, 0.74522, 0.74163, 0.73778, 0.73370, 0.72945, 0.72513,\
        0.72081, 0.71660, 0.71257, 0.70859, 0.70416, 0.69912, 0.69317, 0.68683, 0.68060, 0.67430, 0.66786,\
        0.66131, 0.65473, 0.64820, 0.64178, 0.63556, 0.62965, 0.62413, 0.61913, 0.61468, 0.61077, 0.60721,\
        0.60363, 0.59972, 0.59594, 0.59229, 0.58861, 0.58481, 0.58087, 0.57675, 0.57246, 0.56800, 0.56337,\
        0.55859, 0.55368, 0.54869, 0.54365, 0.53863, 0.53366, 0.52878, 0.52403, 0.51938, 0.51473, 0.50989],\
        [0.52800, 0.53554, 0.54313, 0.55071, 0.55819, 0.56547, 0.57245, 0.57898, 0.58487, 0.58995, 0.59431,\
        0.59873, 0.60381, 0.60905, 0.61429, 0.61937, 0.62416, 0.62857, 0.63255, 0.63607, 0.63911, 0.64167,\
        0.64377, 0.64547, 0.64683, 0.64795, 0.64897, 0.65002, 0.65124, 0.65279, 0.65477, 0.65727, 0.66032,\
        0.66395, 0.66815, 0.67286, 0.67801, 0.68342, 0.68871, 0.69319, 0.69639, 0.69874, 0.70059, 0.70152,\
        0.70310, 0.70618, 0.71049, 0.71590, 0.72197, 0.72827, 0.73451, 0.74049, 0.74608, 0.75118, 0.75572,\
        0.75967, 0.76301, 0.76572, 0.76782, 0.76932, 0.77020, 0.77049, 0.77023, 0.76951, 0.76840, 0.76699,\
        0.76528, 0.76328, 0.76097, 0.75835, 0.75540, 0.75211, 0.74851, 0.74462, 0.74048, 0.73616, 0.73172,\
        0.72724, 0.72278, 0.71831, 0.71372, 0.70875, 0.70322, 0.69694, 0.69104, 0.68524, 0.67916, 0.67281,\
        0.66633, 0.65982, 0.65336, 0.64702, 0.64087, 0.63502, 0.62953, 0.62447, 0.61989, 0.61572, 0.61182,\
        0.60795, 0.60404, 0.60032, 0.59673, 0.59312, 0.58943, 0.58559, 0.58158, 0.57741, 0.57308, 0.56856,\
        0.56387, 0.55902, 0.55406, 0.54904, 0.54399, 0.53896, 0.53398, 0.52905, 0.52415, 0.51918, 0.51402],\
        [0.53327, 0.54084, 0.54843, 0.55599, 0.56347, 0.57080, 0.57789, 0.58461, 0.59071, 0.59594, 0.60011,\
        0.60425, 0.60913, 0.61425, 0.61937, 0.62437, 0.62914, 0.63358, 0.63766, 0.64132, 0.64457, 0.64739,\
        0.64981, 0.65187, 0.65363, 0.65517, 0.65659, 0.65798, 0.65944, 0.66108, 0.66299, 0.66523, 0.66789,\
        0.67103, 0.67470, 0.67890, 0.68364, 0.68882, 0.69422, 0.69899, 0.70189, 0.70415, 0.70608, 0.70776,\
        0.70996, 0.71331, 0.71777, 0.72315, 0.72913, 0.73537, 0.74160, 0.74760, 0.75324, 0.75839, 0.76297,\
        0.76694, 0.77027, 0.77295, 0.77500, 0.77644, 0.77726, 0.77744, 0.77704, 0.77619, 0.77502, 0.77359,\
        0.77193, 0.77000, 0.76778, 0.76522, 0.76232, 0.75906, 0.75545, 0.75151, 0.74730, 0.74287, 0.73828,\
        0.73359, 0.72881, 0.72391, 0.71878, 0.71325, 0.70718, 0.70095, 0.69557, 0.69022, 0.68418, 0.67785,\
        0.67144, 0.66501, 0.65864, 0.65237, 0.64628, 0.64045, 0.63495, 0.62982, 0.62509, 0.62070, 0.61655,\
        0.61255, 0.60865, 0.60492, 0.60131, 0.59776, 0.59415, 0.59036, 0.58644, 0.58240, 0.57819, 0.57377,\
        0.56914, 0.56433, 0.55938, 0.55434, 0.54924, 0.54411, 0.53899, 0.53386, 0.52870, 0.52345, 0.51801],\
        [0.53850, 0.54608, 0.55366, 0.56120, 0.56865, 0.57597, 0.58311, 0.59000, 0.59637, 0.60173, 0.60609,\
        0.61026, 0.61485, 0.61972, 0.62468, 0.62957, 0.63428, 0.63874, 0.64288, 0.64667, 0.65009, 0.65315,\
        0.65585, 0.65823, 0.66034, 0.66224, 0.66398, 0.66564, 0.66727, 0.66894, 0.67069, 0.67260, 0.67478,\
        0.67731, 0.68029, 0.68381, 0.68788, 0.69244, 0.69733, 0.70201, 0.70544, 0.70822, 0.71079, 0.71341,\
        0.71648, 0.72036, 0.72511, 0.73059, 0.73656, 0.74276, 0.74896, 0.75494, 0.76057, 0.76572, 0.77030,\
        0.77425, 0.77753, 0.78012, 0.78208, 0.78346, 0.78425, 0.78431, 0.78372, 0.78271, 0.78147, 0.78008,\
        0.77850, 0.77668, 0.77455, 0.77209, 0.76925, 0.76602, 0.76239, 0.75840, 0.75410, 0.74954, 0.74478,\
        0.73986, 0.73479, 0.72954, 0.72405, 0.71824, 0.71217, 0.70624, 0.70115, 0.69552, 0.68923, 0.68292,\
        0.67658, 0.67025, 0.66396, 0.65776, 0.65171, 0.64588, 0.64033, 0.63511, 0.63023, 0.62566, 0.62135,\
        0.61724, 0.61331, 0.60955, 0.60594, 0.60242, 0.59886, 0.59506, 0.59122, 0.58733, 0.58326, 0.57892,\
        0.57433, 0.56953, 0.56457, 0.55948, 0.55429, 0.54904, 0.54375, 0.53842, 0.53302, 0.52751, 0.52182],\
        [0.54364, 0.55123, 0.55879, 0.56628, 0.57366, 0.58088, 0.58793, 0.59480, 0.60130, 0.60677, 0.61148,\
        0.61592, 0.62048, 0.62522, 0.63004, 0.63482, 0.63948, 0.64393, 0.64813, 0.65202, 0.65559, 0.65885,\
        0.66178, 0.66443, 0.66682, 0.66901, 0.67102, 0.67289, 0.67463, 0.67626, 0.67780, 0.67931, 0.68090,\
        0.68272, 0.68492, 0.68762, 0.69086, 0.69462, 0.69876, 0.70322, 0.70741, 0.71116, 0.71474, 0.71842,\
        0.72246, 0.72706, 0.73226, 0.73798, 0.74404, 0.75025, 0.75641, 0.76237, 0.76797, 0.77307, 0.77758,\
        0.78148, 0.78469, 0.78714, 0.78892, 0.79025, 0.79109, 0.79104, 0.79018, 0.78898, 0.78772, 0.78641,\
        0.78495, 0.78327, 0.78127, 0.77891, 0.77615, 0.77295, 0.76931, 0.76525, 0.76083, 0.75614, 0.75120,\
        0.74608, 0.74077, 0.73527, 0.72956, 0.72366, 0.71769, 0.71189, 0.70631, 0.70025, 0.69416, 0.68797,\
        0.68172, 0.67549, 0.66929, 0.66314, 0.65709, 0.65122, 0.64560, 0.64027, 0.63526, 0.63055, 0.62611,\
        0.62191, 0.61792, 0.61411, 0.61046, 0.60689, 0.60328, 0.59953, 0.59583, 0.59217, 0.58824, 0.58393,\
        0.57934, 0.57453, 0.56953, 0.56437, 0.55909, 0.55370, 0.54823, 0.54270, 0.53708, 0.53134, 0.52543],\
        [0.54864, 0.55623, 0.56376, 0.57120, 0.57849, 0.58557, 0.59240, 0.59903, 0.60546, 0.61123, 0.61634,\
        0.62111, 0.62580, 0.63053, 0.63529, 0.64001, 0.64463, 0.64908, 0.65330, 0.65727, 0.66097, 0.66437,\
        0.66749, 0.67034, 0.67295, 0.67535, 0.67756, 0.67958, 0.68140, 0.68297, 0.68426, 0.68530, 0.68624,\
        0.68728, 0.68863, 0.69046, 0.69290, 0.69598, 0.69972, 0.70410, 0.70876, 0.71342, 0.71809, 0.72288,\
        0.72793, 0.73333, 0.73911, 0.74518, 0.75142, 0.75767, 0.76381, 0.76972, 0.77528, 0.78031, 0.78469,\
        0.78846, 0.79166, 0.79392, 0.79540, 0.79661, 0.79763, 0.79750, 0.79629, 0.79498, 0.79377, 0.79259,\
        0.79131, 0.78977, 0.78788, 0.78563, 0.78297, 0.77982, 0.77614, 0.77199, 0.76746, 0.76262, 0.75752,\
        0.75222, 0.74674, 0.74107, 0.73524, 0.72928, 0.72328, 0.71733, 0.71138, 0.70530, 0.69927, 0.69298,\
        0.68683, 0.68074, 0.67461, 0.66846, 0.66236, 0.65642, 0.65070, 0.64527, 0.64014, 0.63530, 0.63075,\
        0.62645, 0.62237, 0.61850, 0.61478, 0.61115, 0.60752, 0.60387, 0.60034, 0.59689, 0.59303, 0.58866,\
        0.58405, 0.57924, 0.57421, 0.56898, 0.56357, 0.55803, 0.55240, 0.54667, 0.54085, 0.53490, 0.52881],\
        [0.55348, 0.56106, 0.56855, 0.57594, 0.58317, 0.59017, 0.59683, 0.60328, 0.60961, 0.61549, 0.62087,\
        0.62591, 0.63077, 0.63557, 0.64032, 0.64502, 0.64962, 0.65406, 0.65831, 0.66234, 0.66612, 0.66963,\
        0.67287, 0.67585, 0.67860, 0.68112, 0.68345, 0.68558, 0.68744, 0.68895, 0.68998, 0.69052, 0.69076,\
        0.69099, 0.69150, 0.69251, 0.69420, 0.69674, 0.70024, 0.70471, 0.70984, 0.71535, 0.72105, 0.72692,\
        0.73297, 0.73922, 0.74563, 0.75213, 0.75859, 0.76491, 0.77101, 0.77684, 0.78235, 0.78735, 0.79150,\
        0.79488, 0.79822, 0.80032, 0.80154, 0.80259, 0.80346, 0.80330, 0.80213, 0.80084, 0.79977, 0.79870,\
        0.79757, 0.79619, 0.79435, 0.79219, 0.78968, 0.78656, 0.78281, 0.77855, 0.77390, 0.76893, 0.76370,\
        0.75827, 0.75265, 0.74688, 0.74095, 0.73493, 0.72884, 0.72274, 0.71662, 0.71044, 0.70417, 0.69791,\
        0.69198, 0.68601, 0.67989, 0.67366, 0.66746, 0.66141, 0.65558, 0.65004, 0.64479, 0.63984, 0.63518,\
        0.63078, 0.62662, 0.62267, 0.61887, 0.61520, 0.61159, 0.60806, 0.60465, 0.60130, 0.59734, 0.59287,\
        0.58835, 0.58361, 0.57855, 0.57322, 0.56768, 0.56199, 0.55619, 0.55029, 0.54429, 0.53817, 0.53192],\
        [0.55814, 0.56569, 0.57315, 0.58048, 0.58768, 0.59469, 0.60132, 0.60765, 0.61381, 0.61968, 0.62519,\
        0.63040, 0.63540, 0.64029, 0.64508, 0.64978, 0.65437, 0.65881, 0.66308, 0.66714, 0.67096, 0.67453,\
        0.67783, 0.68087, 0.68365, 0.68620, 0.68854, 0.69068, 0.69256, 0.69400, 0.69481, 0.69485, 0.69438,\
        0.69386, 0.69361, 0.69390, 0.69494, 0.69701, 0.70037, 0.70508, 0.71081, 0.71715, 0.72383, 0.73070,\
        0.73771, 0.74478, 0.75185, 0.75881, 0.76553, 0.77190, 0.77791, 0.78359, 0.78896, 0.79400, 0.79809,\
        0.80080, 0.80380, 0.80604, 0.80738, 0.80833, 0.80888, 0.80873, 0.80791, 0.80685, 0.80583, 0.80482,\
        0.80374, 0.80240, 0.80058, 0.79851, 0.79622, 0.79308, 0.78919, 0.78484, 0.78010, 0.77502, 0.76969,\
        0.76416, 0.75847, 0.75262, 0.74663, 0.74053, 0.73436, 0.72813, 0.72188, 0.71560, 0.70931, 0.70317,\
        0.69723, 0.69129, 0.68507, 0.67867, 0.67230, 0.66612, 0.66018, 0.65453, 0.64918, 0.64413, 0.63937,\
        0.63487, 0.63061, 0.62657, 0.62270, 0.61898, 0.61536, 0.61183, 0.60836, 0.60477, 0.60067, 0.59648,\
        0.59226, 0.58761, 0.58250, 0.57704, 0.57136, 0.56553, 0.55958, 0.55354, 0.54739, 0.54113, 0.53474],\
        [0.56260, 0.57012, 0.57753, 0.58479, 0.59190, 0.59885, 0.60551, 0.61182, 0.61790, 0.62374, 0.62931,\
        0.63462, 0.63973, 0.64468, 0.64951, 0.65423, 0.65881, 0.66325, 0.66752, 0.67159, 0.67542, 0.67901,\
        0.68231, 0.68532, 0.68803, 0.69047, 0.69268, 0.69469, 0.69648, 0.69781, 0.69846, 0.69803, 0.69694,\
        0.69585, 0.69506, 0.69482, 0.69539, 0.69706, 0.70028, 0.70546, 0.71187, 0.71899, 0.72654, 0.73432,\
        0.74220, 0.75007, 0.75779, 0.76524, 0.77223, 0.77865, 0.78448, 0.78989, 0.79494, 0.79969, 0.80386,\
        0.80670, 0.80922, 0.81131, 0.81275, 0.81368, 0.81415, 0.81408, 0.81353, 0.81272, 0.81182, 0.81084,\
        0.80973, 0.80836, 0.80663, 0.80464, 0.80235, 0.79908, 0.79513, 0.79077, 0.78601, 0.78083, 0.77542,\
        0.76987, 0.76415, 0.75827, 0.75223, 0.74605, 0.73979, 0.73346, 0.72711, 0.72076, 0.71449, 0.70838,\
        0.70246, 0.69648, 0.68996, 0.68330, 0.67678, 0.67048, 0.66445, 0.65871, 0.65327, 0.64813, 0.64327,\
        0.63868, 0.63432, 0.63018, 0.62622, 0.62242, 0.61873, 0.61513, 0.61154, 0.60781, 0.60386, 0.59994,\
        0.59593, 0.59123, 0.58596, 0.58036, 0.57454, 0.56859, 0.56253, 0.55637, 0.55011, 0.54374, 0.53725],\
        [0.56685, 0.57433, 0.58169, 0.58888, 0.59587, 0.60271, 0.60935, 0.61570, 0.62178, 0.62762, 0.63322,\
        0.63859, 0.64376, 0.64876, 0.65362, 0.65834, 0.66292, 0.66734, 0.67159, 0.67564, 0.67945, 0.68300,\
        0.68625, 0.68916, 0.69171, 0.69389, 0.69577, 0.69742, 0.69888, 0.69985, 0.70010, 0.69951, 0.69831,\
        0.69701, 0.69597, 0.69549, 0.69586, 0.69745, 0.70080, 0.70632, 0.71324, 0.72100, 0.72927, 0.73784,\
        0.74650, 0.75509, 0.76344, 0.77139, 0.77872, 0.78519, 0.79073, 0.79583, 0.80060, 0.80499, 0.80887,\
        0.81192, 0.81436, 0.81632, 0.81777, 0.81873, 0.81923, 0.81929, 0.81897, 0.81837, 0.81759, 0.81666,\
        0.81553, 0.81414, 0.81242, 0.81035, 0.80779, 0.80450, 0.80050, 0.79642, 0.79158, 0.78627, 0.78087,\
        0.77536, 0.76969, 0.76380, 0.75771, 0.75146, 0.74509, 0.73866, 0.73220, 0.72577, 0.71943, 0.71323,\
        0.70716, 0.70093, 0.69414, 0.68739, 0.68080, 0.67444, 0.66835, 0.66255, 0.65705, 0.65183, 0.64688,\
        0.64219, 0.63772, 0.63347, 0.62940, 0.62549, 0.62170, 0.61799, 0.61431, 0.61059, 0.60684, 0.60314,\
        0.59915, 0.59424, 0.58877, 0.58304, 0.57715, 0.57113, 0.56500, 0.55877, 0.55243, 0.54599, 0.53943],\
        [0.57087, 0.57832, 0.58564, 0.59280, 0.59972, 0.60646, 0.61302, 0.61934, 0.62542, 0.63126, 0.63688,\
        0.64228, 0.64749, 0.65252, 0.65739, 0.66210, 0.66665, 0.67104, 0.67525, 0.67924, 0.68300, 0.68649,\
        0.68964, 0.69240, 0.69470, 0.69651, 0.69787, 0.69894, 0.69972, 0.69999, 0.70006, 0.69973, 0.69875,\
        0.69754, 0.69652, 0.69605, 0.69651, 0.69834, 0.70205, 0.70775, 0.71495, 0.72316, 0.73201, 0.74123,\
        0.75057, 0.75981, 0.76873, 0.77716, 0.78492, 0.79162, 0.79675, 0.80137, 0.80600, 0.81011, 0.81371,\
        0.81670, 0.81911, 0.82103, 0.82249, 0.82351, 0.82410, 0.82432, 0.82418, 0.82377, 0.82312, 0.82225,\
        0.82113, 0.81970, 0.81792, 0.81571, 0.81297, 0.80962, 0.80577, 0.80164, 0.79656, 0.79128, 0.78603,\
        0.78069, 0.77509, 0.76921, 0.76306, 0.75672, 0.75025, 0.74369, 0.73710, 0.73052, 0.72400, 0.71756,\
        0.71111, 0.70445, 0.69768, 0.69099, 0.68440, 0.67801, 0.67190, 0.66606, 0.66051, 0.65523, 0.65019,\
        0.64540, 0.64082, 0.63644, 0.63224, 0.62819, 0.62426, 0.62042, 0.61663, 0.61286, 0.60910, 0.60529,\
        0.60117, 0.59619, 0.59073, 0.58503, 0.57916, 0.57314, 0.56699, 0.56073, 0.55436, 0.54789, 0.54129],\
        [0.57465, 0.58206, 0.58933, 0.59646, 0.60337, 0.61002, 0.61649, 0.62277, 0.62883, 0.63467, 0.64029,\
        0.64571, 0.65092, 0.65595, 0.66081, 0.66549, 0.67000, 0.67433, 0.67846, 0.68238, 0.68605, 0.68943,\
        0.69246, 0.69504, 0.69707, 0.69844, 0.69917, 0.69963, 0.69993, 0.70000, 0.70001, 0.69965, 0.69877,\
        0.69772, 0.69687, 0.69659, 0.69730, 0.69947, 0.70353, 0.70943, 0.71683, 0.72535, 0.73465, 0.74442,\
        0.75435, 0.76416, 0.77355, 0.78234, 0.79043, 0.79758, 0.80279, 0.80696, 0.81113, 0.81495, 0.81830,\
        0.82115, 0.82352, 0.82543, 0.82693, 0.82803, 0.82875, 0.82913, 0.82919, 0.82895, 0.82844, 0.82764,\
        0.82654, 0.82508, 0.82321, 0.82088, 0.81802, 0.81460, 0.81067, 0.80619, 0.80115, 0.79597, 0.79106,\
        0.78595, 0.78042, 0.77450, 0.76827, 0.76181, 0.75521, 0.74850, 0.74175, 0.73499, 0.72824, 0.72150,\
        0.71472, 0.70788, 0.70106, 0.69427, 0.68762, 0.68123, 0.67512, 0.66928, 0.66369, 0.65835, 0.65323,\
        0.64832, 0.64362, 0.63910, 0.63474, 0.63052, 0.62643, 0.62242, 0.61848, 0.61455, 0.61059, 0.60647,\
        0.60198, 0.59710, 0.59189, 0.58637, 0.58060, 0.57464, 0.56851, 0.56227, 0.55590, 0.54942, 0.54282],\
        [0.57820, 0.58555, 0.59272, 0.59977, 0.60664, 0.61329, 0.61973, 0.62597, 0.63200, 0.63783, 0.64345,\
        0.64886, 0.65406, 0.65907, 0.66389, 0.66852, 0.67296, 0.67721, 0.68124, 0.68504, 0.68858, 0.69182,\
        0.69468, 0.69706, 0.69880, 0.69980, 0.69982, 0.69987, 0.69996, 0.69998, 0.69989, 0.69941, 0.69861,\
        0.69776, 0.69715, 0.69715, 0.69815, 0.70067, 0.70501, 0.71106, 0.71860, 0.72735, 0.73701, 0.74726,\
        0.75773, 0.76807, 0.77786, 0.78680, 0.79484, 0.80196, 0.80753, 0.81191, 0.81582, 0.81937, 0.82253,\
        0.82526, 0.82758, 0.82951, 0.83108, 0.83230, 0.83318, 0.83374, 0.83399, 0.83394, 0.83357, 0.83287,\
        0.83180, 0.83032, 0.82838, 0.82595, 0.82299, 0.81949, 0.81545, 0.81089, 0.80590, 0.80092, 0.79630,\
        0.79127, 0.78568, 0.77964, 0.77329, 0.76670, 0.75995, 0.75309, 0.74617, 0.73921, 0.73222, 0.72520,\
        0.71816, 0.71110, 0.70410, 0.69720, 0.69052, 0.68417, 0.67810, 0.67226, 0.66664, 0.66123, 0.65602,\
        0.65099, 0.64615, 0.64146, 0.63692, 0.63251, 0.62822, 0.62401, 0.61987, 0.61573, 0.61154, 0.60716,\
        0.60247, 0.59763, 0.59261, 0.58725, 0.58158, 0.57568, 0.56961, 0.56340, 0.55706, 0.55061, 0.54404],\
        [0.58153, 0.58882, 0.59591, 0.60285, 0.60968, 0.61630, 0.62271, 0.62892, 0.63494, 0.64074, 0.64634,\
        0.65173, 0.65691, 0.66189, 0.66665, 0.67121, 0.67555, 0.67968, 0.68357, 0.68721, 0.69058, 0.69364,\
        0.69631, 0.69849, 0.69990, 0.70040, 0.70022, 0.70002, 0.69995, 0.69986, 0.69958, 0.69905, 0.69837,\
        0.69776, 0.69746, 0.69776, 0.69905, 0.70186, 0.70635, 0.71245, 0.72003, 0.72892, 0.73886, 0.74954,\
        0.76055, 0.77146, 0.78175, 0.79083, 0.79845, 0.80547, 0.81129, 0.81593, 0.81983, 0.82326, 0.82629,\
        0.82897, 0.83129, 0.83329, 0.83496, 0.83633, 0.83740, 0.83817, 0.83864, 0.83878, 0.83857, 0.83798,\
        0.83697, 0.83548, 0.83350, 0.83099, 0.82794, 0.82437, 0.82030, 0.81580, 0.81104, 0.80630, 0.80164,\
        0.79652, 0.79074, 0.78455, 0.77807, 0.77135, 0.76446, 0.75746, 0.75038, 0.74323, 0.73603, 0.72877,\
        0.72147, 0.71416, 0.70692, 0.69988, 0.69319, 0.68694, 0.68093, 0.67508, 0.66941, 0.66392, 0.65860,\
        0.65344, 0.64844, 0.64357, 0.63883, 0.63420, 0.62967, 0.62524, 0.62086, 0.61652, 0.61214, 0.60765,\
        0.60298, 0.59817, 0.59315, 0.58779, 0.58216, 0.57631, 0.57030, 0.56415, 0.55788, 0.55148, 0.54496],\
        [0.58461, 0.59191, 0.59895, 0.60581, 0.61253, 0.61908, 0.62545, 0.63164, 0.63763, 0.64341, 0.64899,\
        0.65435, 0.65949, 0.66441, 0.66910, 0.67357, 0.67779, 0.68177, 0.68550, 0.68895, 0.69210, 0.69491,\
        0.69732, 0.69925, 0.70043, 0.70077, 0.70063, 0.70034, 0.70007, 0.69976, 0.69926, 0.69867, 0.69811,\
        0.69778, 0.69784, 0.69851, 0.70006, 0.70299, 0.70744, 0.71340, 0.72087, 0.72979, 0.73994, 0.75102,\
        0.76259, 0.77412, 0.78508, 0.79489, 0.80250, 0.80898, 0.81459, 0.81921, 0.82311, 0.82651, 0.82954,\
        0.83224, 0.83464, 0.83675, 0.83858, 0.84015, 0.84144, 0.84245, 0.84316, 0.84353, 0.84351, 0.84305,\
        0.84211, 0.84063, 0.83861, 0.83603, 0.83291, 0.82927, 0.82518, 0.82074, 0.81609, 0.81137, 0.80653,\
        0.80128, 0.79535, 0.78914, 0.78258, 0.77573, 0.76872, 0.76160, 0.75440, 0.74711, 0.73973, 0.73228,\
        0.72477, 0.71724, 0.70975, 0.70243, 0.69582, 0.68978, 0.68373, 0.67780, 0.67204, 0.66646, 0.66102,\
        0.65572, 0.65054, 0.64546, 0.64050, 0.63562, 0.63084, 0.62614, 0.62152, 0.61695, 0.61242, 0.60789,\
        0.60334, 0.59860, 0.59344, 0.58801, 0.58238, 0.57658, 0.57064, 0.56456, 0.55837, 0.55205, 0.54560],\
        [0.58737, 0.59474, 0.60177, 0.60852, 0.61512, 0.62161, 0.62794, 0.63411, 0.64008, 0.64585, 0.65140,\
        0.65673, 0.66182, 0.66667, 0.67128, 0.67563, 0.67972, 0.68353, 0.68706, 0.69029, 0.69319, 0.69571,\
        0.69782, 0.69941, 0.70043, 0.70087, 0.70085, 0.70057, 0.70016, 0.69957, 0.69885, 0.69821, 0.69781,\
        0.69778, 0.69825, 0.69932, 0.70113, 0.70398, 0.70814, 0.71373, 0.72090, 0.72968, 0.73997, 0.75144,\
        0.76358, 0.77574, 0.78725, 0.79749, 0.80564, 0.81197, 0.81728, 0.82177, 0.82564, 0.82910, 0.83223,\
        0.83508, 0.83765, 0.83995, 0.84199, 0.84379, 0.84535, 0.84664, 0.84763, 0.84825, 0.84845, 0.84815,\
        0.84729, 0.84583, 0.84376, 0.84110, 0.83788, 0.83416, 0.83002, 0.82556, 0.82087, 0.81601, 0.81094,\
        0.80548, 0.79962, 0.79347, 0.78677, 0.77981, 0.77272, 0.76554, 0.75827, 0.75089, 0.74340, 0.73581,\
        0.72815, 0.72048, 0.71291, 0.70567, 0.69930, 0.69284, 0.68651, 0.68044, 0.67458, 0.66889, 0.66332,\
        0.65786, 0.65249, 0.64720, 0.64198, 0.63683, 0.63176, 0.62677, 0.62188, 0.61707, 0.61235, 0.60770,\
        0.60310, 0.59842, 0.59322, 0.58780, 0.58221, 0.57649, 0.57064, 0.56467, 0.55857, 0.55235, 0.54600],\
        [0.58967, 0.59708, 0.60410, 0.61079, 0.61738, 0.62386, 0.63019, 0.63635, 0.64231, 0.64807, 0.65360,\
        0.65889, 0.66392, 0.66870, 0.67321, 0.67743, 0.68137, 0.68500, 0.68832, 0.69132, 0.69395, 0.69620,\
        0.69801, 0.69935, 0.70026, 0.70077, 0.70083, 0.70051, 0.69987, 0.69905, 0.69822, 0.69762, 0.69742,\
        0.69773, 0.69860, 0.70004, 0.70200, 0.70468, 0.70834, 0.71331, 0.71989, 0.72833, 0.73864, 0.75052,\
        0.76333, 0.77621, 0.78820, 0.79858, 0.80715, 0.81381, 0.81903, 0.82342, 0.82734, 0.83099, 0.83438,\
        0.83751, 0.84036, 0.84293, 0.84525, 0.84734, 0.84921, 0.85081, 0.85211, 0.85303, 0.85347, 0.85335,\
        0.85258, 0.85113, 0.84900, 0.84622, 0.84286, 0.83900, 0.83474, 0.83017, 0.82535, 0.82032, 0.81505,\
        0.80950, 0.80371, 0.79742, 0.79054, 0.78352, 0.77645, 0.76931, 0.76204, 0.75463, 0.74707, 0.73938,\
        0.73163, 0.72389, 0.71630, 0.70909, 0.70225, 0.69552, 0.68913, 0.68300, 0.67706, 0.67126, 0.66555,\
        0.65992, 0.65435, 0.64882, 0.64333, 0.63788, 0.63250, 0.62719, 0.62198, 0.61689, 0.61194, 0.60711,\
        0.60225, 0.59742, 0.59246, 0.58716, 0.58168, 0.57607, 0.57035, 0.56450, 0.55853, 0.55243, 0.54619],\
        [0.59146, 0.59861, 0.60572, 0.61261, 0.61930, 0.62584, 0.63220, 0.63837, 0.64434, 0.65009, 0.65559,\
        0.66085, 0.66583, 0.67053, 0.67493, 0.67903, 0.68280, 0.68625, 0.68935, 0.69211, 0.69449, 0.69648,\
        0.69807, 0.69925, 0.70008, 0.70061, 0.70061, 0.70018, 0.69928, 0.69824, 0.69735, 0.69685, 0.69691,\
        0.69762, 0.69895, 0.70075, 0.70282, 0.70517, 0.70808, 0.71204, 0.71766, 0.72545, 0.73564, 0.74796,\
        0.76164, 0.77554, 0.78844, 0.79931, 0.80810, 0.81463, 0.81962, 0.82395, 0.82810, 0.83216, 0.83605,\
        0.83964, 0.84289, 0.84581, 0.84847, 0.85089, 0.85310, 0.85506, 0.85672, 0.85796, 0.85867, 0.85874,\
        0.85806, 0.85659, 0.85434, 0.85138, 0.84782, 0.84377, 0.83932, 0.83457, 0.82957, 0.82433, 0.81886,\
        0.81313, 0.80710, 0.80063, 0.79379, 0.78689, 0.77999, 0.77297, 0.76577, 0.75836, 0.75076, 0.74301,\
        0.73518, 0.72735, 0.71966, 0.71219, 0.70496, 0.69811, 0.69174, 0.68555, 0.67952, 0.67361, 0.66776,\
        0.66196, 0.65617, 0.65038, 0.64461, 0.63884, 0.63312, 0.62746, 0.62190, 0.61648, 0.61124, 0.60626,\
        0.60159, 0.59662, 0.59146, 0.58622, 0.58087, 0.57540, 0.56982, 0.56412, 0.55829, 0.55232, 0.54621],\
        [0.59313, 0.60003, 0.60717, 0.61420, 0.62101, 0.62761, 0.63401, 0.64021, 0.64619, 0.65193, 0.65742,\
        0.66264, 0.66757, 0.67219, 0.67649, 0.68046, 0.68407, 0.68733, 0.69022, 0.69273, 0.69487, 0.69664,\
        0.69805, 0.69912, 0.69993, 0.70047, 0.70027, 0.69941, 0.69828, 0.69712, 0.69625, 0.69593, 0.69633,\
        0.69754, 0.69948, 0.70178, 0.70394, 0.70576, 0.70754, 0.71000, 0.71410, 0.72079, 0.73062, 0.74339,\
        0.75814, 0.77347, 0.78780, 0.79979, 0.80838, 0.81422, 0.81870, 0.82310, 0.82783, 0.83270, 0.83738,\
        0.84164, 0.84541, 0.84876, 0.85178, 0.85457, 0.85716, 0.85951, 0.86154, 0.86314, 0.86416, 0.86443,\
        0.86382, 0.86226, 0.85981, 0.85660, 0.85276, 0.84844, 0.84375, 0.83877, 0.83356, 0.82811, 0.82243,\
        0.81647, 0.81017, 0.80351, 0.79673, 0.79012, 0.78348, 0.77664, 0.76952, 0.76213, 0.75450, 0.74668,\
        0.73875, 0.73082, 0.72298, 0.71534, 0.70799, 0.70108, 0.69446, 0.68811, 0.68200, 0.67598, 0.67000,\
        0.66401, 0.65800, 0.65196, 0.64587, 0.63977, 0.63369, 0.62765, 0.62171, 0.61591, 0.61030, 0.60494,\
        0.60000, 0.59519, 0.59017, 0.58505, 0.57985, 0.57454, 0.56912, 0.56358, 0.55790, 0.55208, 0.54610],\
        [0.59498, 0.60189, 0.60886, 0.61581, 0.62262, 0.62924, 0.63567, 0.64189, 0.64788, 0.65363, 0.65911,\
        0.66430, 0.66918, 0.67374, 0.67794, 0.68177, 0.68523, 0.68830, 0.69097, 0.69325, 0.69514, 0.69667,\
        0.69789, 0.69885, 0.69964, 0.70016, 0.69955, 0.69829, 0.69692, 0.69572, 0.69498, 0.69494, 0.69580,\
        0.69767, 0.70044, 0.70351, 0.70581, 0.70688, 0.70705, 0.70736, 0.70925, 0.71420, 0.72327, 0.73640,\
        0.75232, 0.76921, 0.78528, 0.79850, 0.80714, 0.81174, 0.81566, 0.82059, 0.82656, 0.83281, 0.83868,\
        0.84382, 0.84820, 0.85198, 0.85538, 0.85856, 0.86155, 0.86431, 0.86674, 0.86870, 0.87003, 0.87051,\
        0.86992, 0.86820, 0.86544, 0.86184, 0.85763, 0.85298, 0.84800, 0.84278, 0.83736, 0.83172, 0.82587,\
        0.81975, 0.81333, 0.80664, 0.79995, 0.79356, 0.78714, 0.78041, 0.77335, 0.76595, 0.75827, 0.75036,\
        0.74232, 0.73426, 0.72628, 0.71849, 0.71098, 0.70381, 0.69698, 0.69070, 0.68456, 0.67841, 0.67228,\
        0.66611, 0.65989, 0.65359, 0.64720, 0.64074, 0.63427, 0.62784, 0.62151, 0.61535, 0.60938, 0.60365,\
        0.59835, 0.59355, 0.58871, 0.58377, 0.57872, 0.57358, 0.56833, 0.56295, 0.55743, 0.55175, 0.54590],\
        [0.59683, 0.60371, 0.61057, 0.61741, 0.62417, 0.63078, 0.63722, 0.64344, 0.64945, 0.65520, 0.66068,\
        0.66585, 0.67070, 0.67519, 0.67931, 0.68302, 0.68633, 0.68922, 0.69168, 0.69372, 0.69534, 0.69659,\
        0.69753, 0.69823, 0.69871, 0.69879, 0.69799, 0.69664, 0.69526, 0.69417, 0.69367, 0.69402, 0.69549,\
        0.69830, 0.70236, 0.70642, 0.70890, 0.70898, 0.70698, 0.70443, 0.70343, 0.70581, 0.71354, 0.72676,\
        0.74363, 0.76168, 0.77908, 0.79384, 0.80217, 0.80584, 0.80990, 0.81641, 0.82464, 0.83296, 0.84039,\
        0.84657, 0.85157, 0.85575, 0.85950, 0.86307, 0.86651, 0.86968, 0.87247, 0.87476, 0.87640, 0.87708,\
        0.87646, 0.87442, 0.87117, 0.86706, 0.86239, 0.85735, 0.85207, 0.84660, 0.84099, 0.83521, 0.82925,\
        0.82309, 0.81672, 0.81020, 0.80371, 0.79745, 0.79106, 0.78433, 0.77725, 0.76982, 0.76204, 0.75401,\
        0.74583, 0.73763, 0.72951, 0.72159, 0.71393, 0.70659, 0.69980, 0.69354, 0.68720, 0.68089, 0.67462,\
        0.66831, 0.66190, 0.65534, 0.64864, 0.64182, 0.63494, 0.62810, 0.62138, 0.61488, 0.60866, 0.60276,\
        0.59731, 0.59227, 0.58739, 0.58252, 0.57761, 0.57263, 0.56753, 0.56231, 0.55694, 0.55139, 0.54567],\
        [0.59849, 0.60534, 0.61214, 0.61892, 0.62563, 0.63222, 0.63865, 0.64489, 0.65090, 0.65667, 0.66215,\
        0.66732, 0.67215, 0.67660, 0.68064, 0.68426, 0.68743, 0.69016, 0.69242, 0.69421, 0.69555, 0.69647,\
        0.69703, 0.69730, 0.69728, 0.69686, 0.69593, 0.69468, 0.69347, 0.69263, 0.69248, 0.69333, 0.69556,\
        0.69960, 0.70533, 0.71068, 0.71347, 0.71239, 0.70764, 0.70146, 0.69733, 0.69680, 0.70200, 0.71459,\
        0.73168, 0.74985, 0.76749, 0.78227, 0.79159, 0.79664, 0.80181, 0.81144, 0.82288, 0.83378, 0.84307,\
        0.85036, 0.85593, 0.86039, 0.86443, 0.86842, 0.87234, 0.87593, 0.87897, 0.88145, 0.88333, 0.88424,\
        0.88349, 0.88088, 0.87692, 0.87216, 0.86695, 0.86151, 0.85592, 0.85023, 0.84445, 0.83858, 0.83258,\
        0.82646, 0.82023, 0.81394, 0.80771, 0.80154, 0.79503, 0.78827, 0.78118, 0.77367, 0.76576, 0.75756,\
        0.74921, 0.74085, 0.73260, 0.72456, 0.71685, 0.70959, 0.70293, 0.69638, 0.68976, 0.68333, 0.67701,\
        0.67062, 0.66406, 0.65727, 0.65025, 0.64305, 0.63575, 0.62847, 0.62134, 0.61451, 0.60811, 0.60214,\
        0.59653, 0.59126, 0.58627, 0.58142, 0.57661, 0.57175, 0.56680, 0.56171, 0.55647, 0.55105, 0.54545],\
        [0.59990, 0.60675, 0.61356, 0.62032, 0.62700, 0.63356, 0.63998, 0.64623, 0.65226, 0.65804, 0.66354,\
        0.66873, 0.67355, 0.67798, 0.68198, 0.68552, 0.68859, 0.69117, 0.69325, 0.69482, 0.69588, 0.69644,\
        0.69657, 0.69635, 0.69583, 0.69502, 0.69397, 0.69283, 0.69186, 0.69135, 0.69160, 0.69302, 0.69610,\
        0.70157, 0.70913, 0.71592, 0.71919, 0.71691, 0.70881, 0.69829, 0.69051, 0.68738, 0.69005, 0.70015,\
        0.71538, 0.73258, 0.75076, 0.76713, 0.77826, 0.78596, 0.79453, 0.80730, 0.82192, 0.83579, 0.84726,\
        0.85574, 0.86172, 0.86628, 0.87052, 0.87498, 0.87951, 0.88353, 0.88658, 0.88894, 0.89087, 0.89198,\
        0.89097, 0.88737, 0.88247, 0.87699, 0.87125, 0.86540, 0.85953, 0.85364, 0.84774, 0.84180, 0.83581,\
        0.82976, 0.82367, 0.81756, 0.81145, 0.80524, 0.79878, 0.79214, 0.78507, 0.77743, 0.76932, 0.76090,\
        0.75236, 0.74383, 0.73544, 0.72730, 0.71950, 0.71214, 0.70522, 0.69852, 0.69193, 0.68568, 0.67948,\
        0.67311, 0.66644, 0.65942, 0.65209, 0.64449, 0.63675, 0.62899, 0.62138, 0.61413, 0.60748, 0.60153,\
        0.59575, 0.59031, 0.58528, 0.58049, 0.57577, 0.57103, 0.56620, 0.56123, 0.55610, 0.55078, 0.54527],\
        [0.60128, 0.60808, 0.61487, 0.62160, 0.62824, 0.63479, 0.64120, 0.64745, 0.65351, 0.65932, 0.66486,\
        0.67009, 0.67494, 0.67938, 0.68336, 0.68685, 0.68983, 0.69230, 0.69426, 0.69566, 0.69647, 0.69668,\
        0.69638, 0.69571, 0.69476, 0.69363, 0.69247, 0.69145, 0.69074, 0.69055, 0.69117, 0.69308, 0.69696,\
        0.70377, 0.71279, 0.72089, 0.72467, 0.72127, 0.70905, 0.69202, 0.67897, 0.67296, 0.67300, 0.67863,\
        0.69035, 0.70710, 0.72862, 0.74923, 0.76189, 0.77242, 0.78531, 0.80221, 0.82106, 0.83905, 0.85337,\
        0.86321, 0.86947, 0.87390, 0.87826, 0.88331, 0.88856, 0.89313, 0.89585, 0.89762, 0.89913, 0.90000,\
        0.89840, 0.89336, 0.88759, 0.88148, 0.87524, 0.86903, 0.86291, 0.85686, 0.85085, 0.84486, 0.83888,\
        0.83288, 0.82689, 0.82091, 0.81491, 0.80880, 0.80250, 0.79597, 0.78885, 0.78098, 0.77257, 0.76390,\
        0.75515, 0.74647, 0.73797, 0.72973, 0.72183, 0.71428, 0.70701, 0.70001, 0.69398, 0.68813, 0.68215,\
        0.67584, 0.66908, 0.66185, 0.65419, 0.64620, 0.63800, 0.62974, 0.62159, 0.61376, 0.60654, 0.60023,\
        0.59454, 0.58933, 0.58446, 0.57979, 0.57518, 0.57054, 0.56579, 0.56091, 0.55585, 0.55061, 0.54517],\
        [0.60268, 0.60938, 0.61609, 0.62277, 0.62937, 0.63589, 0.64230, 0.64856, 0.65464, 0.66051, 0.66611,\
        0.67139, 0.67631, 0.68080, 0.68480, 0.68828, 0.69121, 0.69361, 0.69549, 0.69682, 0.69746, 0.69738,\
        0.69671, 0.69568, 0.69439, 0.69301, 0.69178, 0.69091, 0.69045, 0.69050, 0.69128, 0.69329, 0.69741,\
        0.70474, 0.71463, 0.72352, 0.72700, 0.72165, 0.70477, 0.67660, 0.65615, 0.64780, 0.64431, 0.64340,\
        0.65033, 0.67065, 0.70312, 0.72674, 0.73914, 0.75295, 0.77020, 0.79278, 0.81895, 0.84338, 0.86171,\
        0.87327, 0.87993, 0.88409, 0.88855, 0.89445, 0.90025, 0.90503, 0.90744, 0.90844, 0.90872, 0.90783,\
        0.90449, 0.89876, 0.89243, 0.88571, 0.87898, 0.87244, 0.86610, 0.85990, 0.85379, 0.84774, 0.84172,\
        0.83574, 0.82979, 0.82388, 0.81798, 0.81204, 0.80599, 0.79962, 0.79230, 0.78402, 0.77529, 0.76638,\
        0.75746, 0.74866, 0.74008, 0.73181, 0.72390, 0.71635, 0.70917, 0.70252, 0.69658, 0.69092, 0.68513,\
        0.67888, 0.67203, 0.66456, 0.65656, 0.64818, 0.63955, 0.63083, 0.62217, 0.61375, 0.60581, 0.59891,\
        0.59353, 0.58865, 0.58400, 0.57945, 0.57492, 0.57033, 0.56563, 0.56079, 0.55578, 0.55058, 0.54517],\
        [0.60398, 0.61060, 0.61722, 0.62381, 0.63037, 0.63685, 0.64325, 0.64953, 0.65565, 0.66157, 0.66726,\
        0.67264, 0.67766, 0.68225, 0.68633, 0.68985, 0.69276, 0.69509, 0.69695, 0.69833, 0.69896, 0.69869,\
        0.69775, 0.69658, 0.69509, 0.69348, 0.69223, 0.69161, 0.69140, 0.69155, 0.69217, 0.69357, 0.69666,\
        0.70306, 0.71298, 0.72181, 0.72268, 0.71157, 0.68367, 0.64329, 0.61658, 0.60709, 0.59772, 0.58964,\
        0.59191, 0.61658, 0.65759, 0.68882, 0.70824, 0.72712, 0.74817, 0.77841, 0.81525, 0.84835, 0.87225,\
        0.88615, 0.89412, 0.89868, 0.90344, 0.91012, 0.91577, 0.91967, 0.92135, 0.92117, 0.91959, 0.91646,\
        0.91140, 0.90476, 0.89746, 0.88984, 0.88257, 0.87573, 0.86919, 0.86283, 0.85658, 0.85041, 0.84431,\
        0.83827, 0.83229, 0.82636, 0.82046, 0.81456, 0.80859, 0.80231, 0.79472, 0.78614, 0.77723, 0.76819,\
        0.75917, 0.75032, 0.74173, 0.73350, 0.72569, 0.71835, 0.71151, 0.70523, 0.69947, 0.69404, 0.68845,\
        0.68223, 0.67523, 0.66748, 0.65914, 0.65040, 0.64141, 0.63233, 0.62333, 0.61458, 0.60635, 0.59923,\
        0.59367, 0.58876, 0.58412, 0.57959, 0.57506, 0.57046, 0.56575, 0.56091, 0.55589, 0.55070, 0.54530],\
        [0.60516, 0.61168, 0.61820, 0.62472, 0.63121, 0.63766, 0.64404, 0.65033, 0.65650, 0.66250, 0.66829,\
        0.67380, 0.67897, 0.68371, 0.68793, 0.69157, 0.69453, 0.69681, 0.69862, 0.70014, 0.70089, 0.70066,\
        0.69972, 0.69881, 0.69726, 0.69543, 0.69422, 0.69409, 0.69414, 0.69417, 0.69435, 0.69437, 0.69525,\
        0.69905, 0.70678, 0.71509, 0.71051, 0.68907, 0.64986, 0.60137, 0.56940, 0.54991, 0.53051, 0.51993,\
        0.51994, 0.53255, 0.58131, 0.63510, 0.67423, 0.69785, 0.71908, 0.75576, 0.80683, 0.85139, 0.88385,\
        0.90181, 0.91249, 0.91951, 0.92518, 0.93072, 0.93500, 0.93725, 0.93729, 0.93531, 0.93161, 0.92629,\
        0.91940, 0.91130, 0.90253, 0.89392, 0.88622, 0.87910, 0.87231, 0.86572, 0.85926, 0.85290, 0.84663,\
        0.84045, 0.83434, 0.82828, 0.82223, 0.81614, 0.80986, 0.80313, 0.79552, 0.78711, 0.77827, 0.76925,\
        0.76023, 0.75138, 0.74283, 0.73470, 0.72707, 0.72003, 0.71361, 0.70777, 0.70240, 0.69732, 0.69200,\
        0.68577, 0.67853, 0.67046, 0.66178, 0.65273, 0.64348, 0.63420, 0.62506, 0.61627, 0.60813, 0.60107,\
        0.59498, 0.58974, 0.58493, 0.58027, 0.57563, 0.57095, 0.56617, 0.56126, 0.55620, 0.55097, 0.54556],\
        [0.60618, 0.61260, 0.61902, 0.62545, 0.63187, 0.63827, 0.64464, 0.65094, 0.65716, 0.66325, 0.66916,\
        0.67483, 0.68019, 0.68514, 0.68958, 0.69342, 0.69658, 0.69890, 0.70070, 0.70233, 0.70333, 0.70353,\
        0.70319, 0.70255, 0.70127, 0.69951, 0.69841, 0.69915, 0.69933, 0.69897, 0.69860, 0.69638, 0.69377,\
        0.69338, 0.69568, 0.70206, 0.68859, 0.65587, 0.60553, 0.55055, 0.52313, 0.49581, 0.47582, 0.46274,\
        0.44663, 0.43852, 0.48053, 0.55691, 0.62861, 0.66916, 0.68780, 0.72412, 0.78938, 0.84644, 0.88928,\
        0.91648, 0.93209, 0.94238, 0.94937, 0.95409, 0.95661, 0.95677, 0.95465, 0.95047, 0.94451, 0.93702,\
        0.92823, 0.91849, 0.90830, 0.89868, 0.89036, 0.88277, 0.87558, 0.86863, 0.86184, 0.85519, 0.84867,\
        0.84226, 0.83593, 0.82965, 0.82335, 0.81694, 0.81024, 0.80302, 0.79539, 0.78726, 0.77858, 0.76963,\
        0.76065, 0.75182, 0.74333, 0.73530, 0.72787, 0.72114, 0.71515, 0.70987, 0.70512, 0.70061, 0.69554,\
        0.68919, 0.68163, 0.67321, 0.66424, 0.65498, 0.64561, 0.63628, 0.62716, 0.61845, 0.61038, 0.60310,\
        0.59681, 0.59139, 0.58634, 0.58145, 0.57661, 0.57176, 0.56685, 0.56183, 0.55669, 0.55140, 0.54593],\
        [0.60703, 0.61334, 0.61965, 0.62598, 0.63232, 0.63867, 0.64502, 0.65134, 0.65761, 0.66378, 0.66982,\
        0.67568, 0.68126, 0.68648, 0.69118, 0.69524, 0.69873, 0.70138, 0.70341, 0.70522, 0.70663, 0.70751,\
        0.70794, 0.70796, 0.70750, 0.70675, 0.70656, 0.70746, 0.70789, 0.70717, 0.70508, 0.70040, 0.69320,\
        0.68801, 0.68263, 0.67054, 0.63815, 0.59896, 0.53410, 0.47716, 0.45676, 0.44098, 0.42443, 0.39708,\
        0.35992, 0.33930, 0.36394, 0.44868, 0.54677, 0.62854, 0.66122, 0.69782, 0.76436, 0.83043, 0.88403,\
        0.92317, 0.94817, 0.96398, 0.97327, 0.97787, 0.97887, 0.97693, 0.97249, 0.96598, 0.95776, 0.94819,\
        0.93760, 0.92639, 0.91513, 0.90453, 0.89515, 0.88683, 0.87904, 0.87155, 0.86429, 0.85726, 0.85041,\
        0.84370, 0.83710, 0.83054, 0.82395, 0.81726, 0.81032, 0.80295, 0.79525, 0.78707, 0.77839, 0.76945,\
        0.76048, 0.75167, 0.74319, 0.73523, 0.72793, 0.72144, 0.71582, 0.71104, 0.70694, 0.70309, 0.69842,\
        0.69197, 0.68404, 0.67535, 0.66623, 0.65692, 0.64757, 0.63833, 0.62936, 0.62082, 0.61287, 0.60564,\
        0.59928, 0.59359, 0.58822, 0.58301, 0.57790, 0.57283, 0.56775, 0.56259, 0.55734, 0.55196, 0.54642],\
        [0.60770, 0.61388, 0.62007, 0.62629, 0.63255, 0.63885, 0.64517, 0.65149, 0.65780, 0.66406, 0.67023,\
        0.67626, 0.68208, 0.68761, 0.69268, 0.69698, 0.70050, 0.70368, 0.70629, 0.70861, 0.71066, 0.71241,\
        0.71386, 0.71502, 0.71594, 0.71681, 0.71799, 0.71942, 0.72022, 0.71939, 0.71588, 0.70830, 0.69813,\
        0.68970, 0.67641, 0.64765, 0.59923, 0.55323, 0.49563, 0.42937, 0.39559, 0.38294, 0.35035, 0.29892,\
        0.28222, 0.28436, 0.32597, 0.39750, 0.47514, 0.56803, 0.62957, 0.67883, 0.74170, 0.80801, 0.87314,\
        0.92547, 0.96170, 0.98395, 0.99584, 1.00058, 1.00032, 0.99643, 0.98978, 0.98103, 0.97068, 0.95917,\
        0.94693, 0.93443, 0.92216, 0.91066, 0.90031, 0.89115, 0.88254, 0.87433, 0.86652, 0.85904, 0.85182,\
        0.84479, 0.83788, 0.83102, 0.82414, 0.81717, 0.81010, 0.80299, 0.79514, 0.78658, 0.77778, 0.76881,\
        0.75982, 0.75098, 0.74246, 0.73447, 0.72718, 0.72078, 0.71536, 0.71088, 0.70714, 0.70376, 0.69988,\
        0.69317, 0.68515, 0.67649, 0.66748, 0.65832, 0.64917, 0.64016, 0.63143, 0.62314, 0.61540, 0.60834,\
        0.60196, 0.59605, 0.59030, 0.58474, 0.57936, 0.57407, 0.56879, 0.56349, 0.55811, 0.55261, 0.54698],\
        [0.60818, 0.61421, 0.62027, 0.62638, 0.63254, 0.63877, 0.64506, 0.65139, 0.65773, 0.66406, 0.67033,\
        0.67650, 0.68254, 0.68837, 0.69389, 0.69885, 0.70257, 0.70595, 0.70916, 0.71221, 0.71515, 0.71798,\
        0.72069, 0.72333, 0.72594, 0.72864, 0.73148, 0.73422, 0.73617, 0.73638, 0.73381, 0.72772, 0.71903,\
        0.70874, 0.69108, 0.65550, 0.59684, 0.54042, 0.48851, 0.42994, 0.38741, 0.35466, 0.32458, 0.30093,\
        0.28038, 0.28502, 0.32645, 0.39046, 0.45580, 0.52840, 0.59788, 0.66547, 0.73058, 0.79628, 0.86960,\
        0.93314, 0.97754, 1.00359, 1.01678, 1.02126, 1.01979, 1.01415, 1.00553, 0.99477, 0.98250, 0.96927,\
        0.95556, 0.94186, 0.92865, 0.91637, 0.90533, 0.89526, 0.88564, 0.87670, 0.86835, 0.86044, 0.85287,\
        0.84552, 0.83832, 0.83116, 0.82399, 0.81672, 0.80928, 0.80165, 0.79389, 0.78554, 0.77679, 0.76782,\
        0.75878, 0.74986, 0.74122, 0.73308, 0.72565, 0.71914, 0.71371, 0.70931, 0.70559, 0.70189, 0.69782,\
        0.69199, 0.68463, 0.67645, 0.66786, 0.65908, 0.65027, 0.64160, 0.63319, 0.62518, 0.61767, 0.61074,\
        0.60437, 0.59826, 0.59223, 0.58644, 0.58084, 0.57536, 0.56992, 0.56446, 0.55894, 0.55333, 0.54761],\
        [0.60846, 0.61434, 0.62024, 0.62622, 0.63228, 0.63843, 0.64468, 0.65101, 0.65738, 0.66376, 0.67011,\
        0.67639, 0.68254, 0.68853, 0.69427, 0.69954, 0.70393, 0.70791, 0.71185, 0.71581, 0.71981, 0.72386,\
        0.72800, 0.73227, 0.73672, 0.74138, 0.74619, 0.75085, 0.75483, 0.75739, 0.75781, 0.75573, 0.75129,\
        0.74367, 0.72839, 0.69034, 0.59882, 0.53693, 0.49057, 0.43660, 0.38623, 0.34637, 0.32148, 0.30832,\
        0.29542, 0.29343, 0.33288, 0.39785, 0.45113, 0.50776, 0.57884, 0.65604, 0.72469, 0.79081, 0.88167,\
        0.95323, 0.99695, 1.02252, 1.03524, 1.03892, 1.03626, 1.02913, 1.01887, 1.00643, 0.99255, 0.97784,\
        0.96284, 0.94805, 0.93392, 0.92080, 0.90891, 0.89800, 0.88775, 0.87836, 0.86963, 0.86140, 0.85352,\
        0.84591, 0.83844, 0.83104, 0.82362, 0.81611, 0.80843, 0.80052, 0.79259, 0.78431, 0.77560, 0.76662,\
        0.75752, 0.74847, 0.73964, 0.73124, 0.72350, 0.71667, 0.71103, 0.70658, 0.70288, 0.69919, 0.69478,\
        0.68944, 0.68286, 0.67541, 0.66742, 0.65917, 0.65084, 0.64258, 0.63454, 0.62682, 0.61951, 0.61264,\
        0.60615, 0.59988, 0.59383, 0.58797, 0.58226, 0.57663, 0.57105, 0.56545, 0.55981, 0.55408, 0.54825],\
        [0.60855, 0.61424, 0.61998, 0.62580, 0.63175, 0.63783, 0.64405, 0.65037, 0.65678, 0.66321, 0.66962,\
        0.67595, 0.68215, 0.68816, 0.69391, 0.69933, 0.70442, 0.70932, 0.71421, 0.71920, 0.72434, 0.72968,\
        0.73530, 0.74123, 0.74754, 0.75422, 0.76116, 0.76811, 0.77462, 0.78014, 0.78409, 0.78592, 0.78484,\
        0.77871, 0.76162, 0.71533, 0.61625, 0.54690, 0.49998, 0.43666, 0.37772, 0.33879, 0.31770, 0.30818,\
        0.30252, 0.29793, 0.34177, 0.40209, 0.45006, 0.50012, 0.56607, 0.64721, 0.72129, 0.78600, 0.89359,\
        0.97028, 1.01317, 1.03765, 1.04956, 1.05245, 1.04885, 1.04062, 1.02913, 1.01540, 1.00025, 0.98435,\
        0.96827, 0.95253, 0.93755, 0.92361, 0.91079, 0.89910, 0.88877, 0.87926, 0.87032, 0.86185, 0.85376,\
        0.84595, 0.83830, 0.83072, 0.82312, 0.81546, 0.80775, 0.79991, 0.79167, 0.78319, 0.77442, 0.76539,\
        0.75620, 0.74700, 0.73795, 0.72923, 0.72106, 0.71371, 0.70760, 0.70307, 0.69940, 0.69574, 0.69150,\
        0.68645, 0.68048, 0.67371, 0.66637, 0.65870, 0.65089, 0.64309, 0.63544, 0.62802, 0.62091, 0.61412,\
        0.60759, 0.60128, 0.59524, 0.58935, 0.58357, 0.57784, 0.57214, 0.56642, 0.56066, 0.55483, 0.54889],\
        [0.60844, 0.61392, 0.61947, 0.62513, 0.63095, 0.63696, 0.64315, 0.64949, 0.65594, 0.66244, 0.66892,\
        0.67531, 0.68154, 0.68754, 0.69330, 0.69890, 0.70459, 0.71038, 0.71624, 0.72225, 0.72850, 0.73509,\
        0.74211, 0.74963, 0.75770, 0.76631, 0.77536, 0.78463, 0.79373, 0.80214, 0.80917, 0.81380, 0.81415,\
        0.80634, 0.78243, 0.73016, 0.64516, 0.56843, 0.51182, 0.43733, 0.37195, 0.33648, 0.31940, 0.31340,\
        0.31225, 0.32092, 0.36309, 0.41536, 0.45537, 0.49701, 0.55809, 0.63648, 0.72312, 0.80698, 0.90440,\
        0.97770, 1.02168, 1.04657, 1.05845, 1.06107, 1.05702, 1.04815, 1.03588, 1.02129, 1.00524, 0.98845,\
        0.97154, 0.95503, 0.93938, 0.92491, 0.91169, 0.89969, 0.88926, 0.87956, 0.87042, 0.86179, 0.85358,\
        0.84566, 0.83792, 0.83024, 0.82252, 0.81472, 0.80688, 0.79896, 0.79071, 0.78220, 0.77338, 0.76429,\
        0.75500, 0.74565, 0.73638, 0.72737, 0.71879, 0.71088, 0.70401, 0.69926, 0.69575, 0.69218, 0.68813,\
        0.68339, 0.67788, 0.67168, 0.66493, 0.65782, 0.65052, 0.64318, 0.63591, 0.62881, 0.62192, 0.61526,\
        0.60882, 0.60259, 0.59653, 0.59061, 0.58477, 0.57896, 0.57316, 0.56733, 0.56146, 0.55553, 0.54950],\
        [0.60814, 0.61338, 0.61871, 0.62418, 0.62988, 0.63581, 0.64199, 0.64838, 0.65492, 0.66153, 0.66812,\
        0.67461, 0.68093, 0.68700, 0.69285, 0.69866, 0.70483, 0.71132, 0.71799, 0.72488, 0.73210, 0.73978,\
        0.74802, 0.75692, 0.76654, 0.77687, 0.78782, 0.79918, 0.81058, 0.82144, 0.83091, 0.83753, 0.83880,\
        0.83032, 0.80522, 0.75562, 0.67971, 0.59712, 0.52230, 0.44616, 0.38439, 0.35261, 0.33725, 0.33496,\
        0.33825, 0.35127, 0.38607, 0.42778, 0.46896, 0.52080, 0.57581, 0.64090, 0.73149, 0.82755, 0.91428,\
        0.98095, 1.02438, 1.04967, 1.06190, 1.06470, 1.06064, 1.05160, 1.03900, 1.02396, 1.00738, 0.99001,\
        0.97250, 0.95542, 0.93928, 0.92457, 0.91161, 0.90014, 0.88928, 0.87915, 0.86983, 0.86115, 0.85295,\
        0.84506, 0.83735, 0.82968, 0.82193, 0.81400, 0.80588, 0.79783, 0.78982, 0.78141, 0.77258, 0.76341,\
        0.75402, 0.74455, 0.73514, 0.72595, 0.71715, 0.70899, 0.70186, 0.69658, 0.69278, 0.68912, 0.68512,\
        0.68055, 0.67535, 0.66957, 0.66329, 0.65667, 0.64984, 0.64292, 0.63602, 0.62922, 0.62258, 0.61611,\
        0.60981, 0.60365, 0.59763, 0.59171, 0.58583, 0.57996, 0.57406, 0.56814, 0.56218, 0.55615, 0.55004],\
        [0.60765, 0.61262, 0.61769, 0.62296, 0.62851, 0.63439, 0.64060, 0.64708, 0.65376, 0.66053, 0.66730,\
        0.67397, 0.68046, 0.68674, 0.69281, 0.69881, 0.70536, 0.71229, 0.71948, 0.72700, 0.73495, 0.74347,\
        0.75267, 0.76267, 0.77353, 0.78525, 0.79774, 0.81078, 0.82401, 0.83680, 0.84819, 0.85662, 0.85962,\
        0.85343, 0.83310, 0.79370, 0.73349, 0.64792, 0.54536, 0.47415, 0.43229, 0.39672, 0.38014, 0.37757,\
        0.37931, 0.38194, 0.40397, 0.44655, 0.49398, 0.55655, 0.60975, 0.66001, 0.75561, 0.85122, 0.92786,\
        0.98527, 1.02463, 1.04873, 1.06086, 1.06388, 1.06006, 1.05117, 1.03861, 1.02348, 1.00671, 0.98907,\
        0.97122, 0.95376, 0.93722, 0.92217, 0.90921, 0.89831, 0.88759, 0.87758, 0.86842, 0.85991, 0.85188,\
        0.84416, 0.83660, 0.82909, 0.82148, 0.81366, 0.80556, 0.79747, 0.78951, 0.78104, 0.77209, 0.76280,\
        0.75331, 0.74377, 0.73430, 0.72508, 0.71627, 0.70814, 0.70111, 0.69547, 0.69098, 0.68689, 0.68267,\
        0.67809, 0.67304, 0.66753, 0.66161, 0.65537, 0.64892, 0.64237, 0.63581, 0.62931, 0.62292, 0.61666,\
        0.61052, 0.60448, 0.59852, 0.59265, 0.58674, 0.58079, 0.57482, 0.56882, 0.56278, 0.55668, 0.55049],\
        [0.60699, 0.61164, 0.61640, 0.62144, 0.62685, 0.63269, 0.63896, 0.64559, 0.65249, 0.65951, 0.66654,\
        0.67345, 0.68020, 0.68677, 0.69325, 0.69969, 0.70624, 0.71322, 0.72064, 0.72851, 0.73691, 0.74596,\
        0.75580, 0.76654, 0.77825, 0.79094, 0.80452, 0.81877, 0.83329, 0.84747, 0.86037, 0.87059, 0.87611,\
        0.87433, 0.86225, 0.83725, 0.79668, 0.72902, 0.61300, 0.52912, 0.49811, 0.47647, 0.45218, 0.43658,\
        0.43046, 0.42761, 0.44097, 0.48730, 0.54584, 0.60256, 0.64436, 0.70408, 0.81212, 0.88615, 0.94465,\
        0.99018, 1.02348, 1.04503, 1.05638, 1.05940, 1.05587, 1.04733, 1.03506, 1.02014, 1.00348, 0.98585,\
        0.96793, 0.95031, 0.93353, 0.91806, 0.90440, 0.89353, 0.88396, 0.87482, 0.86622, 0.85811, 0.85040,\
        0.84296, 0.83567, 0.82843, 0.82114, 0.81372, 0.80613, 0.79827, 0.78987, 0.78104, 0.77185, 0.76241,\
        0.75284, 0.74326, 0.73382, 0.72468, 0.71600, 0.70799, 0.70084, 0.69505, 0.69003, 0.68540, 0.68082,\
        0.67607, 0.67102, 0.66565, 0.65995, 0.65399, 0.64785, 0.64161, 0.63534, 0.62910, 0.62294, 0.61690,\
        0.61097, 0.60513, 0.59931, 0.59342, 0.58744, 0.58143, 0.57540, 0.56934, 0.56324, 0.55708, 0.55084],\
        [0.60617, 0.61043, 0.61484, 0.61960, 0.62485, 0.63069, 0.63708, 0.64395, 0.65114, 0.65851, 0.66587,\
        0.67310, 0.68011, 0.68690, 0.69359, 0.70030, 0.70687, 0.71387, 0.72134, 0.72931, 0.73785, 0.74711,\
        0.75722, 0.76831, 0.78046, 0.79367, 0.80785, 0.82277, 0.83803, 0.85305, 0.86698, 0.87867, 0.88667,\
        0.88931, 0.88488, 0.87157, 0.84609, 0.80007, 0.72693, 0.62277, 0.56144, 0.53838, 0.51335, 0.49211,\
        0.48291, 0.48551, 0.50412, 0.54759, 0.61121, 0.65767, 0.71234, 0.79536, 0.86518, 0.91226, 0.95562,\
        0.99244, 1.02037, 1.03911, 1.04930, 1.05209, 1.04878, 1.04065, 1.02883, 1.01433, 0.99803, 0.98068,\
        0.96299, 0.94553, 0.92883, 0.91343, 0.90008, 0.88919, 0.88003, 0.87156, 0.86354, 0.85591, 0.84859,\
        0.84149, 0.83453, 0.82762, 0.82068, 0.81367, 0.80650, 0.79885, 0.79016, 0.78104, 0.77166, 0.76212,\
        0.75250, 0.74293, 0.73356, 0.72455, 0.71608, 0.70833, 0.70145, 0.69521, 0.68958, 0.68439, 0.67941,\
        0.67441, 0.66928, 0.66393, 0.65837, 0.65260, 0.64669, 0.64068, 0.63464, 0.62862, 0.62265, 0.61679,\
        0.61106, 0.60545, 0.59981, 0.59388, 0.58784, 0.58179, 0.57574, 0.56965, 0.56352, 0.55733, 0.55105],\
        [0.60521, 0.60901, 0.61298, 0.61741, 0.62251, 0.62837, 0.63497, 0.64216, 0.64976, 0.65757, 0.66536,\
        0.67296, 0.68024, 0.68714, 0.69368, 0.70011, 0.70700, 0.71412, 0.72152, 0.72934, 0.73773, 0.74684,\
        0.75685, 0.76790, 0.78006, 0.79334, 0.80764, 0.82272, 0.83818, 0.85346, 0.86780, 0.88027, 0.88983,\
        0.89537, 0.89581, 0.88986, 0.87538, 0.84917, 0.80811, 0.74464, 0.66296, 0.60132, 0.55906, 0.53283,\
        0.52505, 0.53580, 0.56359, 0.61009, 0.67862, 0.73065, 0.79170, 0.85159, 0.88953, 0.92512, 0.96094,\
        0.99157, 1.01524, 1.03139, 1.04029, 1.04267, 1.03949, 1.03176, 1.02047, 1.00653, 0.99077, 0.97394,\
        0.95674, 0.93980, 0.92366, 0.90872, 0.89589, 0.88545, 0.87645, 0.86831, 0.86070, 0.85347, 0.84652,\
        0.83977, 0.83313, 0.82652, 0.81987, 0.81309, 0.80602, 0.79833, 0.78981, 0.78072, 0.77133, 0.76177,\
        0.75216, 0.74264, 0.73335, 0.72445, 0.71613, 0.70855, 0.70168, 0.69524, 0.68920, 0.68358, 0.67824,\
        0.67301, 0.66774, 0.66237, 0.65686, 0.65121, 0.64545, 0.63961, 0.63375, 0.62790, 0.62208, 0.61633,\
        0.61067, 0.60513, 0.59964, 0.59378, 0.58782, 0.58184, 0.57582, 0.56976, 0.56363, 0.55743, 0.55113],\
        [0.60413, 0.60738, 0.61081, 0.61484, 0.61979, 0.62574, 0.63263, 0.64026, 0.64838, 0.65673, 0.66505,\
        0.67312, 0.68074, 0.68781, 0.69431, 0.70064, 0.70734, 0.71421, 0.72126, 0.72863, 0.73653, 0.74517,\
        0.75471, 0.76532, 0.77709, 0.79003, 0.80402, 0.81881, 0.83399, 0.84899, 0.86309, 0.87548, 0.88530,\
        0.89172, 0.89393, 0.89106, 0.88192, 0.86496, 0.83784, 0.79716, 0.74501, 0.68877, 0.62924, 0.58777,\
        0.57297, 0.59060, 0.62647, 0.69190, 0.74366, 0.78541, 0.83519, 0.87528, 0.90520, 0.93403, 0.96319,\
        0.98872, 1.00868, 1.02241, 1.02997, 1.03180, 1.02863, 1.02125, 1.01051, 0.99722, 0.98215, 0.96600,\
        0.94949, 0.93332, 0.91825, 0.90464, 0.89243, 0.88209, 0.87315, 0.86517, 0.85783, 0.85091, 0.84427,\
        0.83782, 0.83146, 0.82509, 0.81863, 0.81198, 0.80497, 0.79734, 0.78895, 0.78002, 0.77074, 0.76126,\
        0.75173, 0.74228, 0.73305, 0.72420, 0.71590, 0.70830, 0.70150, 0.69488, 0.68861, 0.68275, 0.67718,\
        0.67175, 0.66636, 0.66092, 0.65541, 0.64982, 0.64415, 0.63844, 0.63271, 0.62698, 0.62127, 0.61557,\
        0.60990, 0.60424, 0.59865, 0.59312, 0.58743, 0.58160, 0.57568, 0.56966, 0.56356, 0.55736, 0.55106],\
        [0.60298, 0.60553, 0.60828, 0.61186, 0.61668, 0.62282, 0.63012, 0.63830, 0.64705, 0.65603, 0.66497,\
        0.67359, 0.68167, 0.68906, 0.69572, 0.70186, 0.70806, 0.71427, 0.72060, 0.72721, 0.73433, 0.74216,\
        0.75090, 0.76073, 0.77175, 0.78398, 0.79732, 0.81148, 0.82602, 0.84033, 0.85366, 0.86522, 0.87418,\
        0.87978, 0.88134, 0.87822, 0.86980, 0.85556, 0.83529, 0.80987, 0.78149, 0.74974, 0.71404, 0.68621,\
        0.65998, 0.65043, 0.67991, 0.74249, 0.79324, 0.82540, 0.85976, 0.88901, 0.91496, 0.94008, 0.96397,\
        0.98489, 1.00139, 1.01275, 1.01888, 1.02003, 1.01674, 1.00965, 0.99945, 0.98686, 0.97256, 0.95723,\
        0.94153, 0.92612, 0.91190, 0.90000, 0.88875, 0.87864, 0.86986, 0.86206, 0.85492, 0.84824, 0.84187,\
        0.83567, 0.82954, 0.82337, 0.81703, 0.81043, 0.80346, 0.79592, 0.78772, 0.77900, 0.76989, 0.76056,\
        0.75114, 0.74178, 0.73261, 0.72377, 0.71539, 0.70753, 0.70028, 0.69390, 0.68775, 0.68184, 0.67614,\
        0.67058, 0.66507, 0.65955, 0.65401, 0.64842, 0.64281, 0.63717, 0.63154, 0.62591, 0.62029, 0.61467,\
        0.60902, 0.60335, 0.59779, 0.59239, 0.58686, 0.58118, 0.57535, 0.56939, 0.56333, 0.55715, 0.55086],\
        [0.60178, 0.60345, 0.60538, 0.60848, 0.61324, 0.61969, 0.62754, 0.63637, 0.64580, 0.65549, 0.66509,\
        0.67432, 0.68290, 0.69066, 0.69758, 0.70351, 0.70888, 0.71417, 0.71952, 0.72513, 0.73120, 0.73796,\
        0.74562, 0.75436, 0.76434, 0.77559, 0.78803, 0.80135, 0.81506, 0.82847, 0.84078, 0.85111, 0.85860,\
        0.86249, 0.86217, 0.85726, 0.84768, 0.83390, 0.81776, 0.80370, 0.79539, 0.78722, 0.77328, 0.75661,\
        0.73652, 0.72550, 0.76176, 0.81001, 0.84303, 0.86647, 0.88571, 0.90226, 0.92349, 0.94445, 0.96383,\
        0.98060, 0.99379, 1.00280, 1.00742, 1.00780, 1.00428, 0.99738, 0.98770, 0.97583, 0.96241, 0.94801,\
        0.93327, 0.91877, 0.90495, 0.89352, 0.88382, 0.87469, 0.86638, 0.85885, 0.85193, 0.84546, 0.83930,\
        0.83332, 0.82741, 0.82141, 0.81517, 0.80858, 0.80163, 0.79422, 0.78623, 0.77773, 0.76882, 0.75965,\
        0.75038, 0.74113, 0.73203, 0.72321, 0.71479, 0.70689, 0.69964, 0.69304, 0.68686, 0.68092, 0.67514,\
        0.66946, 0.66383, 0.65823, 0.65262, 0.64701, 0.64140, 0.63581, 0.63024, 0.62471, 0.61921, 0.61371,\
        0.60818, 0.60264, 0.59717, 0.59176, 0.58628, 0.58065, 0.57488, 0.56898, 0.56294, 0.55679, 0.55051],\
        [0.60052, 0.60103, 0.60221, 0.60480, 0.60964, 0.61658, 0.62507, 0.63459, 0.64472, 0.65508, 0.66534,\
        0.67519, 0.68426, 0.69225, 0.69898, 0.70461, 0.70935, 0.71369, 0.71797, 0.72243, 0.72730, 0.73280,\
        0.73915, 0.74657, 0.75526, 0.76533, 0.77673, 0.78915, 0.80205, 0.81464, 0.82600, 0.83515, 0.84119,\
        0.84337, 0.84124, 0.83481, 0.82472, 0.81270, 0.80234, 0.80123, 0.81142, 0.82109, 0.81986, 0.80791,\
        0.79779, 0.80275, 0.84098, 0.86690, 0.88566, 0.90060, 0.91116, 0.92025, 0.93348, 0.94861, 0.96323,\
        0.97603, 0.98607, 0.99277, 0.99587, 0.99539, 0.99156, 0.98480, 0.97559, 0.96447, 0.95199, 0.93864,\
        0.92500, 0.91184, 0.89971, 0.88859, 0.87913, 0.87059, 0.86274, 0.85552, 0.84882, 0.84253, 0.83655,\
        0.83076, 0.82506, 0.81928, 0.81322, 0.80665, 0.79961, 0.79241, 0.78463, 0.77628, 0.76754, 0.75854,\
        0.74943, 0.74032, 0.73133, 0.72257, 0.71416, 0.70626, 0.69907, 0.69235, 0.68608, 0.68007, 0.67419,\
        0.66839, 0.66264, 0.65692, 0.65122, 0.64554, 0.63991, 0.63433, 0.62882, 0.62338, 0.61802, 0.61269,\
        0.60734, 0.60194, 0.59656, 0.59116, 0.58567, 0.58005, 0.57430, 0.56842, 0.56242, 0.55629, 0.55003],\
        [0.59826, 0.59854, 0.59916, 0.60120, 0.60637, 0.61388, 0.62301, 0.63314, 0.64385, 0.65477, 0.66560,\
        0.67600, 0.68559, 0.69384, 0.70035, 0.70540, 0.70935, 0.71272, 0.71591, 0.71920, 0.72281, 0.72694,\
        0.73184, 0.73777, 0.74499, 0.75374, 0.76406, 0.77567, 0.78797, 0.80005, 0.81086, 0.81932, 0.82449,\
        0.82569, 0.82273, 0.81619, 0.80781, 0.80094, 0.80135, 0.81379, 0.83627, 0.85922, 0.86981, 0.87105,\
        0.87610, 0.88689, 0.89916, 0.90702, 0.91593, 0.92378, 0.92923, 0.93449, 0.94210, 0.95178, 0.96189,\
        0.97101, 0.97816, 0.98271, 0.98434, 0.98299, 0.97882, 0.97213, 0.96335, 0.95299, 0.94153, 0.92937,\
        0.91682, 0.90463, 0.89380, 0.88379, 0.87474, 0.86656, 0.85905, 0.85208, 0.84556, 0.83941, 0.83356,\
        0.82793, 0.82243, 0.81695, 0.81126, 0.80502, 0.79811, 0.79080, 0.78296, 0.77466, 0.76603, 0.75719,\
        0.74827, 0.73936, 0.73054, 0.72190, 0.71354, 0.70558, 0.69825, 0.69168, 0.68542, 0.67933, 0.67333,\
        0.66738, 0.66147, 0.65560, 0.64978, 0.64401, 0.63832, 0.63272, 0.62724, 0.62187, 0.61664, 0.61153,\
        0.60645, 0.60126, 0.59594, 0.59051, 0.58496, 0.57932, 0.57358, 0.56772, 0.56175, 0.55565, 0.54942],\
        [0.59565, 0.59633, 0.59701, 0.59913, 0.60419, 0.61212, 0.62168, 0.63218, 0.64323, 0.65449, 0.66567,\
        0.67647, 0.68651, 0.69523, 0.70163, 0.70574, 0.70874, 0.71119, 0.71339, 0.71560, 0.71799, 0.72078,\
        0.72417, 0.72849, 0.73410, 0.74141, 0.75067, 0.76170, 0.77378, 0.78585, 0.79672, 0.80523, 0.81043,\
        0.81176, 0.80932, 0.80438, 0.79987, 0.80023, 0.80915, 0.82852, 0.85673, 0.88788, 0.90502, 0.91346,\
        0.92466, 0.93309, 0.93519, 0.93482, 0.93574, 0.93770, 0.93963, 0.94223, 0.94654, 0.95249, 0.95909,\
        0.96519, 0.96989, 0.97256, 0.97287, 0.97072, 0.96619, 0.95954, 0.95114, 0.94149, 0.93113, 0.92043,\
        0.90928, 0.89792, 0.88800, 0.87885, 0.87037, 0.86255, 0.85530, 0.84853, 0.84215, 0.83609, 0.83030,\
        0.82474, 0.81939, 0.81419, 0.80904, 0.80342, 0.79667, 0.78908, 0.78105, 0.77272, 0.76418, 0.75554,\
        0.74686, 0.73821, 0.72964, 0.72121, 0.71300, 0.70519, 0.69800, 0.69128, 0.68493, 0.67872, 0.67254,\
        0.66640, 0.66030, 0.65425, 0.64827, 0.64238, 0.63661, 0.63096, 0.62546, 0.62014, 0.61500, 0.61007,\
        0.60531, 0.60045, 0.59514, 0.58963, 0.58406, 0.57841, 0.57269, 0.56687, 0.56093, 0.55487, 0.54867],\
        [0.59375, 0.59500, 0.59635, 0.59878, 0.60379, 0.61176, 0.62128, 0.63178, 0.64284, 0.65414, 0.66536,\
        0.67621, 0.68631, 0.69513, 0.70130, 0.70483, 0.70729, 0.70908, 0.71053, 0.71185, 0.71320, 0.71474,\
        0.71670, 0.71937, 0.72323, 0.72897, 0.73723, 0.74797, 0.76032, 0.77296, 0.78457, 0.79394, 0.80013,\
        0.80268, 0.80195, 0.79978, 0.79970, 0.80474, 0.81713, 0.83738, 0.86320, 0.88846, 0.90651, 0.92091,\
        0.93449, 0.94364, 0.94637, 0.94527, 0.94360, 0.94263, 0.94255, 0.94365, 0.94622, 0.95002, 0.95434,\
        0.95829, 0.96110, 0.96224, 0.96144, 0.95861, 0.95380, 0.94718, 0.93908, 0.93000, 0.92059, 0.91150,\
        0.90256, 0.89239, 0.88274, 0.87400, 0.86595, 0.85847, 0.85147, 0.84487, 0.83858, 0.83255, 0.82674,\
        0.82115, 0.81579, 0.81064, 0.80580, 0.80106, 0.79433, 0.78658, 0.77849, 0.77023, 0.76186, 0.75347,\
        0.74510, 0.73680, 0.72860, 0.72052, 0.71257, 0.70479, 0.69757, 0.69107, 0.68464, 0.67821, 0.67179,\
        0.66540, 0.65906, 0.65280, 0.64665, 0.64063, 0.63474, 0.62902, 0.62348, 0.61814, 0.61301, 0.60812,\
        0.60353, 0.59897, 0.59377, 0.58835, 0.58285, 0.57727, 0.57161, 0.56585, 0.55997, 0.55395, 0.54780],\
        [0.59266, 0.59462, 0.59681, 0.60016, 0.60529, 0.61265, 0.62171, 0.63183, 0.64259, 0.65362, 0.66458,\
        0.67511, 0.68476, 0.69288, 0.69872, 0.70268, 0.70507, 0.70652, 0.70752, 0.70825, 0.70881, 0.70935,\
        0.71007, 0.71121, 0.71324, 0.71715, 0.72440, 0.73524, 0.74835, 0.76208, 0.77498, 0.78583, 0.79373,\
        0.79824, 0.79989, 0.80042, 0.80293, 0.80977, 0.82221, 0.83987, 0.86023, 0.87907, 0.89405, 0.90942,\
        0.92532, 0.93642, 0.94134, 0.94187, 0.94054, 0.93916, 0.93867, 0.93947, 0.94155, 0.94449, 0.94764,\
        0.95026, 0.95175, 0.95173, 0.95005, 0.94669, 0.94172, 0.93522, 0.92738, 0.91867, 0.90985, 0.90173,\
        0.89429, 0.88580, 0.87712, 0.86896, 0.86137, 0.85425, 0.84753, 0.84109, 0.83487, 0.82882, 0.82292,\
        0.81720, 0.81167, 0.80629, 0.80108, 0.79620, 0.79011, 0.78281, 0.77501, 0.76701, 0.75894, 0.75088,\
        0.74290, 0.73504, 0.72732, 0.71976, 0.71232, 0.70500, 0.69798, 0.69121, 0.68448, 0.67772, 0.67097,\
        0.66428, 0.65768, 0.65121, 0.64488, 0.63871, 0.63272, 0.62692, 0.62131, 0.61593, 0.61076, 0.60579,\
        0.60094, 0.59654, 0.59176, 0.58664, 0.58133, 0.57590, 0.57034, 0.56467, 0.55886, 0.55291, 0.54681],\
        [0.59218, 0.59494, 0.59782, 0.60155, 0.60698, 0.61401, 0.62251, 0.63208, 0.64233, 0.65288, 0.66336,\
        0.67339, 0.68249, 0.69016, 0.69606, 0.70015, 0.70241, 0.70380, 0.70469, 0.70514, 0.70522, 0.70506,\
        0.70489, 0.70493, 0.70531, 0.70693, 0.71311, 0.72450, 0.73864, 0.75362, 0.76804, 0.78070, 0.79061,\
        0.79719, 0.80070, 0.80292, 0.80649, 0.81338, 0.82429, 0.83859, 0.85433, 0.86915, 0.88224, 0.89569,\
        0.91066, 0.92180, 0.92783, 0.92993, 0.92982, 0.92920, 0.92937, 0.93088, 0.93348, 0.93653, 0.93935,\
        0.94130, 0.94194, 0.94107, 0.93869, 0.93496, 0.93000, 0.92380, 0.91633, 0.90795, 0.89972, 0.89232,\
        0.88552, 0.87835, 0.87088, 0.86354, 0.85653, 0.84986, 0.84344, 0.83720, 0.83106, 0.82496, 0.81892,\
        0.81297, 0.80722, 0.80170, 0.79618, 0.79080, 0.78483, 0.77801, 0.77065, 0.76304, 0.75535, 0.74770,\
        0.74016, 0.73278, 0.72561, 0.71867, 0.71192, 0.70521, 0.69837, 0.69136, 0.68422, 0.67704, 0.66992,\
        0.66292, 0.65607, 0.64940, 0.64292, 0.63662, 0.63053, 0.62465, 0.61899, 0.61356, 0.60837, 0.60343,\
        0.59872, 0.59415, 0.58953, 0.58467, 0.57959, 0.57433, 0.56892, 0.56335, 0.55763, 0.55176, 0.54571],\
        [0.59194, 0.59555, 0.59932, 0.60334, 0.60859, 0.61528, 0.62326, 0.63225, 0.64191, 0.65189, 0.66182,\
        0.67132, 0.67996, 0.68735, 0.69318, 0.69726, 0.69976, 0.70134, 0.70243, 0.70288, 0.70273, 0.70219,\
        0.70156, 0.70119, 0.70094, 0.70031, 0.70528, 0.71709, 0.73183, 0.74773, 0.76345, 0.77781, 0.78968,\
        0.79799, 0.80248, 0.80516, 0.80880, 0.81496, 0.82405, 0.83546, 0.84795, 0.86026, 0.87208, 0.88419,\
        0.89629, 0.90522, 0.91112, 0.91411, 0.91504, 0.91537, 0.91665, 0.91952, 0.92329, 0.92705, 0.93009,\
        0.93182, 0.93191, 0.93034, 0.92737, 0.92335, 0.91856, 0.91297, 0.90618, 0.89826, 0.89070, 0.88383,\
        0.87737, 0.87092, 0.86434, 0.85781, 0.85144, 0.84526, 0.83922, 0.83323, 0.82719, 0.82106, 0.81486,\
        0.80863, 0.80254, 0.79683, 0.79114, 0.78535, 0.77923, 0.77261, 0.76560, 0.75839, 0.75111, 0.74389,\
        0.73679, 0.72987, 0.72321, 0.71688, 0.71086, 0.70491, 0.69834, 0.69102, 0.68343, 0.67586, 0.66842,\
        0.66117, 0.65413, 0.64731, 0.64072, 0.63434, 0.62819, 0.62226, 0.61657, 0.61113, 0.60595, 0.60102,\
        0.59640, 0.59189, 0.58735, 0.58265, 0.57775, 0.57266, 0.56738, 0.56193, 0.55630, 0.55050, 0.54453],\
        [0.59149, 0.59565, 0.60000, 0.60435, 0.60963, 0.61608, 0.62363, 0.63209, 0.64118, 0.65061, 0.66002,\
        0.66907, 0.67737, 0.68461, 0.69049, 0.69487, 0.69783, 0.69979, 0.70115, 0.70168, 0.70150, 0.70087,\
        0.70015, 0.69986, 0.69952, 0.69933, 0.70319, 0.71365, 0.72801, 0.74410, 0.76047, 0.77598, 0.78946,\
        0.79910, 0.80347, 0.80575, 0.80908, 0.81448, 0.82203, 0.83131, 0.84153, 0.85199, 0.86238, 0.87264,\
        0.88207, 0.88945, 0.89476, 0.89796, 0.89963, 0.90066, 0.90296, 0.90737, 0.91237, 0.91698, 0.92046,\
        0.92221, 0.92190, 0.91969, 0.91612, 0.91181, 0.90721, 0.90233, 0.89640, 0.88931, 0.88229, 0.87577,\
        0.86964, 0.86367, 0.85774, 0.85188, 0.84613, 0.84049, 0.83488, 0.82921, 0.82334, 0.81720, 0.81086,\
        0.80446, 0.79802, 0.79192, 0.78590, 0.77977, 0.77344, 0.76685, 0.76006, 0.75316, 0.74627, 0.73946,\
        0.73276, 0.72624, 0.71995, 0.71401, 0.70849, 0.70333, 0.69708, 0.68950, 0.68166, 0.67388, 0.66628,\
        0.65890, 0.65177, 0.64489, 0.63825, 0.63186, 0.62571, 0.61978, 0.61409, 0.60867, 0.60355, 0.59874,\
        0.59417, 0.58973, 0.58527, 0.58068, 0.57591, 0.57095, 0.56579, 0.56044, 0.55491, 0.54918, 0.54326],\
        [0.59062, 0.59499, 0.59940, 0.60433, 0.60987, 0.61622, 0.62344, 0.63145, 0.64005, 0.64898, 0.65796,\
        0.66667, 0.67478, 0.68202, 0.68814, 0.69303, 0.69670, 0.69931, 0.70087, 0.70146, 0.70135, 0.70085,\
        0.70037, 0.70006, 0.69965, 0.69916, 0.70311, 0.71299, 0.72656, 0.74207, 0.75819, 0.77379, 0.78776,\
        0.79828, 0.80161, 0.80375, 0.80721, 0.81218, 0.81867, 0.82644, 0.83503, 0.84397, 0.85289, 0.86147,\
        0.86917, 0.87549, 0.88027, 0.88361, 0.88595, 0.88809, 0.89132, 0.89626, 0.90176, 0.90697, 0.91083,\
        0.91268, 0.91207, 0.90921, 0.90508, 0.90057, 0.89610, 0.89154, 0.88622, 0.88012, 0.87384, 0.86780,\
        0.86205, 0.85650, 0.85109, 0.84579, 0.84061, 0.83551, 0.83041, 0.82514, 0.81954, 0.81345, 0.80692,\
        0.80032, 0.79372, 0.78713, 0.78061, 0.77407, 0.76747, 0.76080, 0.75410, 0.74745, 0.74090, 0.73446,\
        0.72813, 0.72190, 0.71581, 0.71000, 0.70444, 0.69919, 0.69344, 0.68626, 0.67862, 0.67093, 0.66337,\
        0.65602, 0.64893, 0.64210, 0.63552, 0.62920, 0.62311, 0.61726, 0.61163, 0.60625, 0.60121, 0.59654,\
        0.59208, 0.58770, 0.58330, 0.57878, 0.57411, 0.56924, 0.56418, 0.55892, 0.55346, 0.54780, 0.54193],\
        [0.58941, 0.59405, 0.59881, 0.60389, 0.60942, 0.61563, 0.62258, 0.63022, 0.63841, 0.64695, 0.65560,\
        0.66408, 0.67214, 0.67951, 0.68600, 0.69146, 0.69587, 0.69916, 0.70106, 0.70181, 0.70187, 0.70157,\
        0.70120, 0.70095, 0.70079, 0.70093, 0.70512, 0.71426, 0.72668, 0.74100, 0.75600, 0.77052, 0.78325,\
        0.79235, 0.79683, 0.79995, 0.80382, 0.80865, 0.81439, 0.82107, 0.82848, 0.83620, 0.84385, 0.85108,\
        0.85756, 0.86307, 0.86753, 0.87111, 0.87416, 0.87732, 0.88129, 0.88629, 0.89180, 0.89707, 0.90094,\
        0.90303, 0.90219, 0.89885, 0.89454, 0.89003, 0.88559, 0.88107, 0.87615, 0.87078, 0.86522, 0.85973,\
        0.85442, 0.84929, 0.84433, 0.83951, 0.83485, 0.83029, 0.82573, 0.82099, 0.81580, 0.80989, 0.80322,\
        0.79623, 0.78930, 0.78227, 0.77523, 0.76823, 0.76129, 0.75446, 0.74778, 0.74130, 0.73504, 0.72897,\
        0.72300, 0.71701, 0.71097, 0.70517, 0.69958, 0.69406, 0.68825, 0.68161, 0.67441, 0.66703, 0.65970,\
        0.65254, 0.64562, 0.63895, 0.63253, 0.62636, 0.62043, 0.61474, 0.60926, 0.60401, 0.59906, 0.59452,\
        0.59016, 0.58583, 0.58147, 0.57700, 0.57238, 0.56758, 0.56259, 0.55739, 0.55199, 0.54638, 0.54056],\
        [0.58783, 0.59272, 0.59781, 0.60284, 0.60823, 0.61432, 0.62104, 0.62837, 0.63623, 0.64446, 0.65287,\
        0.66124, 0.66933, 0.67692, 0.68381, 0.68986, 0.69489, 0.69873, 0.70110, 0.70231, 0.70269, 0.70262,\
        0.70243, 0.70236, 0.70269, 0.70419, 0.70863, 0.71672, 0.72769, 0.74042, 0.75380, 0.76668, 0.77787,\
        0.78626, 0.79174, 0.79580, 0.79995, 0.80450, 0.80956, 0.81546, 0.82205, 0.82884, 0.83544, 0.84157,\
        0.84710, 0.85197, 0.85622, 0.86002, 0.86365, 0.86748, 0.87186, 0.87682, 0.88201, 0.88679, 0.89035,\
        0.89217, 0.89133, 0.88823, 0.88413, 0.87974, 0.87533, 0.87089, 0.86627, 0.86144, 0.85649, 0.85154,\
        0.84669, 0.84196, 0.83739, 0.83299, 0.82877, 0.82472, 0.82072, 0.81659, 0.81198, 0.80643, 0.79976,\
        0.79240, 0.78486, 0.77726, 0.76968, 0.76220, 0.75488, 0.74782, 0.74109, 0.73474, 0.72874, 0.72303,\
        0.71748, 0.71183, 0.70580, 0.69969, 0.69424, 0.68857, 0.68263, 0.67621, 0.66938, 0.66237, 0.65537,\
        0.64851, 0.64188, 0.63548, 0.62932, 0.62339, 0.61769, 0.61224, 0.60704, 0.60203, 0.59727, 0.59278,\
        0.58844, 0.58413, 0.57978, 0.57533, 0.57074, 0.56598, 0.56102, 0.55587, 0.55051, 0.54494, 0.53915],\
        [0.58578, 0.59070, 0.59567, 0.60067, 0.60627, 0.61231, 0.61882, 0.62587, 0.63345, 0.64146, 0.64971,\
        0.65804, 0.66622, 0.67406, 0.68138, 0.68798, 0.69365, 0.69805, 0.70104, 0.70277, 0.70351, 0.70375,\
        0.70388, 0.70423, 0.70521, 0.70755, 0.71215, 0.71942, 0.72900, 0.74008, 0.75173, 0.76298, 0.77289,\
        0.78082, 0.78677, 0.79148, 0.79579, 0.80001, 0.80449, 0.80990, 0.81604, 0.82213, 0.82780, 0.83296,\
        0.83764, 0.84194, 0.84599, 0.84994, 0.85395, 0.85818, 0.86273, 0.86751, 0.87221, 0.87631, 0.87926,\
        0.88051, 0.87970, 0.87708, 0.87340, 0.86927, 0.86503, 0.86078, 0.85648, 0.85212, 0.84769, 0.84325,\
        0.83884, 0.83449, 0.83025, 0.82618, 0.82232, 0.81869, 0.81519, 0.81165, 0.80771, 0.80261, 0.79586,\
        0.78815, 0.78012, 0.77201, 0.76392, 0.75596, 0.74825, 0.74091, 0.73408, 0.72780, 0.72203, 0.71665,\
        0.71156, 0.70648, 0.70074, 0.69453, 0.68863, 0.68277, 0.67672, 0.67040, 0.66384, 0.65717, 0.65053,\
        0.64404, 0.63778, 0.63175, 0.62594, 0.62033, 0.61491, 0.60974, 0.60492, 0.60032, 0.59569, 0.59123,\
        0.58688, 0.58257, 0.57821, 0.57376, 0.56917, 0.56442, 0.55949, 0.55436, 0.54903, 0.54348, 0.53772],\
        [0.58331, 0.58821, 0.59317, 0.59835, 0.60392, 0.60970, 0.61593, 0.62272, 0.63007, 0.63790, 0.64607,\
        0.65440, 0.66270, 0.67080, 0.67850, 0.68564, 0.69200, 0.69732, 0.70106, 0.70296, 0.70404, 0.70470,\
        0.70529, 0.70616, 0.70775, 0.71059, 0.71520, 0.72180, 0.73018, 0.73978, 0.74987, 0.75969, 0.76856,\
        0.77604, 0.78212, 0.78715, 0.79154, 0.79560, 0.79960, 0.80502, 0.81092, 0.81629, 0.82099, 0.82515,\
        0.82898, 0.83271, 0.83653, 0.84056, 0.84481, 0.84927, 0.85383, 0.85834, 0.86248, 0.86589, 0.86814,\
        0.86890, 0.86801, 0.86567, 0.86236, 0.85855, 0.85459, 0.85062, 0.84670, 0.84280, 0.83888, 0.83493,\
        0.83094, 0.82693, 0.82295, 0.81910, 0.81550, 0.81218, 0.80901, 0.80583, 0.80238, 0.79761, 0.79086,\
        0.78309, 0.77485, 0.76641, 0.75792, 0.74954, 0.74143, 0.73378, 0.72680, 0.72057, 0.71499, 0.70985,\
        0.70504, 0.70040, 0.69506, 0.68892, 0.68280, 0.67674, 0.67061, 0.66434, 0.65798, 0.65159, 0.64531,\
        0.63923, 0.63340, 0.62782, 0.62247, 0.61727, 0.61217, 0.60719, 0.60256, 0.59841, 0.59407, 0.58972,\
        0.58542, 0.58110, 0.57673, 0.57226, 0.56766, 0.56291, 0.55798, 0.55286, 0.54754, 0.54200, 0.53625],\
        [0.58050, 0.58536, 0.59035, 0.59558, 0.60095, 0.60641, 0.61236, 0.61891, 0.62609, 0.63381, 0.64193,\
        0.65028, 0.65870, 0.66701, 0.67504, 0.68261, 0.68950, 0.69542, 0.69964, 0.70223, 0.70398, 0.70522,\
        0.70637, 0.70780, 0.70989, 0.71303, 0.71757, 0.72362, 0.73102, 0.73938, 0.74816, 0.75680, 0.76480,\
        0.77184, 0.77789, 0.78308, 0.78769, 0.79199, 0.79645, 0.80174, 0.80695, 0.81135, 0.81492, 0.81799,\
        0.82093, 0.82406, 0.82762, 0.83169, 0.83613, 0.84072, 0.84523, 0.84941, 0.85300, 0.85573, 0.85732,\
        0.85759, 0.85651, 0.85426, 0.85119, 0.84767, 0.84400, 0.84038, 0.83688, 0.83347, 0.83007, 0.82661,\
        0.82304, 0.81934, 0.81558, 0.81186, 0.80838, 0.80534, 0.80231, 0.79919, 0.79575, 0.79105, 0.78472,\
        0.77723, 0.76905, 0.76050, 0.75178, 0.74307, 0.73458, 0.72657, 0.71936, 0.71319, 0.70784, 0.70285,\
        0.69811, 0.69348, 0.68836, 0.68260, 0.67658, 0.67049, 0.66433, 0.65813, 0.65192, 0.64579, 0.63984,\
        0.63417, 0.62882, 0.62377, 0.61897, 0.61431, 0.60966, 0.60496, 0.60026, 0.59634, 0.59238, 0.58824,\
        0.58400, 0.57969, 0.57530, 0.57081, 0.56619, 0.56142, 0.55649, 0.55136, 0.54604, 0.54050, 0.53475],\
        [0.57739, 0.58214, 0.58701, 0.59201, 0.59711, 0.60241, 0.60816, 0.61453, 0.62157, 0.62922, 0.63731,\
        0.64569, 0.65420, 0.66267, 0.67094, 0.67880, 0.68601, 0.69224, 0.69712, 0.70077, 0.70318, 0.70509,\
        0.70689, 0.70888, 0.71139, 0.71476, 0.71919, 0.72478, 0.73140, 0.73877, 0.74649, 0.75417, 0.76144,\
        0.76807, 0.77399, 0.77931, 0.78419, 0.78894, 0.79387, 0.79904, 0.80357, 0.80690, 0.80927, 0.81123,\
        0.81330, 0.81579, 0.81910, 0.82328, 0.82794, 0.83263, 0.83703, 0.84085, 0.84389, 0.84596, 0.84692,\
        0.84670, 0.84534, 0.84302, 0.84006, 0.83674, 0.83333, 0.83005, 0.82703, 0.82418, 0.82131, 0.81835,\
        0.81523, 0.81182, 0.80828, 0.80472, 0.80112, 0.79840, 0.79550, 0.79236, 0.78871, 0.78402, 0.77803,\
        0.77090, 0.76296, 0.75448, 0.74569, 0.73678, 0.72797, 0.71957, 0.71200, 0.70586, 0.70086, 0.69602,\
        0.69131, 0.68658, 0.68151, 0.67599, 0.67013, 0.66410, 0.65799, 0.65185, 0.64577, 0.63985, 0.63420,\
        0.62893, 0.62408, 0.61962, 0.61547, 0.61148, 0.60749, 0.60335, 0.59913, 0.59500, 0.59097, 0.58685,\
        0.58262, 0.57831, 0.57390, 0.56938, 0.56474, 0.55995, 0.55499, 0.54985, 0.54452, 0.53898, 0.53323],\
        [0.57404, 0.57863, 0.58330, 0.58804, 0.59290, 0.59798, 0.60351, 0.60970, 0.61664, 0.62425, 0.63232,\
        0.64069, 0.64924, 0.65782, 0.66625, 0.67433, 0.68181, 0.68842, 0.69395, 0.69828, 0.70151, 0.70429,\
        0.70678, 0.70928, 0.71216, 0.71568, 0.72003, 0.72525, 0.73126, 0.73784, 0.74473, 0.75163, 0.75828,\
        0.76451, 0.77028, 0.77561, 0.78066, 0.78561, 0.79060, 0.79548, 0.79962, 0.80227, 0.80358, 0.80466,\
        0.80605, 0.80788, 0.81096, 0.81542, 0.82034, 0.82510, 0.82934, 0.83276, 0.83522, 0.83665, 0.83700,\
        0.83627, 0.83455, 0.83206, 0.82910, 0.82593, 0.82271, 0.81971, 0.81719, 0.81499, 0.81262, 0.81010,\
        0.80758, 0.80443, 0.80098, 0.79790, 0.79447, 0.79157, 0.78870, 0.78551, 0.78172, 0.77705, 0.77129,\
        0.76450, 0.75686, 0.74859, 0.73988, 0.73093, 0.72196, 0.71328, 0.70540, 0.69888, 0.69423, 0.68954,\
        0.68483, 0.68001, 0.67491, 0.66947, 0.66373, 0.65777, 0.65171, 0.64563, 0.63965, 0.63389, 0.62850,\
        0.62360, 0.61924, 0.61539, 0.61194, 0.60870, 0.60546, 0.60191, 0.59799, 0.59386, 0.58971, 0.58552,\
        0.58126, 0.57692, 0.57248, 0.56794, 0.56327, 0.55845, 0.55348, 0.54832, 0.54298, 0.53743, 0.53166],\
        [0.57052, 0.57494, 0.57940, 0.58392, 0.58856, 0.59343, 0.59870, 0.60466, 0.61150, 0.61908, 0.62707,\
        0.63537, 0.64389, 0.65252, 0.66108, 0.66935, 0.67710, 0.68410, 0.69017, 0.69521, 0.69937, 0.70296,\
        0.70600, 0.70895, 0.71213, 0.71579, 0.72007, 0.72502, 0.73055, 0.73652, 0.74274, 0.74901, 0.75513,\
        0.76098, 0.76652, 0.77177, 0.77679, 0.78166, 0.78639, 0.79078, 0.79439, 0.79661, 0.79760, 0.79828,\
        0.79941, 0.80049, 0.80357, 0.80843, 0.81346, 0.81819, 0.82220, 0.82516, 0.82698, 0.82775, 0.82752,\
        0.82632, 0.82421, 0.82143, 0.81843, 0.81543, 0.81237, 0.80946, 0.80740, 0.80603, 0.80408, 0.80171,\
        0.79983, 0.79720, 0.79390, 0.79090, 0.78788, 0.78496, 0.78202, 0.77876, 0.77494, 0.77032, 0.76478,\
        0.75830, 0.75100, 0.74303, 0.73456, 0.72578, 0.71688, 0.70811, 0.70028, 0.69391, 0.68863, 0.68376,\
        0.67890, 0.67393, 0.66875, 0.66330, 0.65759, 0.65169, 0.64567, 0.63964, 0.63371, 0.62804, 0.62283,\
        0.61824, 0.61435, 0.61109, 0.60831, 0.60580, 0.60330, 0.60035, 0.59659, 0.59252, 0.58834, 0.58412,\
        0.57983, 0.57547, 0.57102, 0.56645, 0.56176, 0.55693, 0.55193, 0.54676, 0.54140, 0.53583, 0.53005],\
        [0.56693, 0.57118, 0.57546, 0.57981, 0.58428, 0.58898, 0.59405, 0.59968, 0.60650, 0.61397, 0.62172,\
        0.62981, 0.63824, 0.64688, 0.65555, 0.66400, 0.67203, 0.67942, 0.68603, 0.69181, 0.69683, 0.70094,\
        0.70451, 0.70787, 0.71129, 0.71507, 0.71933, 0.72407, 0.72925, 0.73474, 0.74042, 0.74616, 0.75181,\
        0.75730, 0.76257, 0.76761, 0.77244, 0.77706, 0.78139, 0.78525, 0.78831, 0.79030, 0.79141, 0.79220,\
        0.79320, 0.79464, 0.79782, 0.80237, 0.80721, 0.81179, 0.81555, 0.81795, 0.81901, 0.81912, 0.81840,\
        0.81683, 0.81437, 0.81120, 0.80813, 0.80542, 0.80266, 0.79966, 0.79796, 0.79712, 0.79558, 0.79351,\
        0.79166, 0.78944, 0.78671, 0.78393, 0.78116, 0.77837, 0.77545, 0.77220, 0.76842, 0.76393, 0.75861,\
        0.75245, 0.74551, 0.73791, 0.72980, 0.72138, 0.71287, 0.70452, 0.69670, 0.69002, 0.68420, 0.67886,\
        0.67367, 0.66846, 0.66313, 0.65760, 0.65189, 0.64601, 0.64002, 0.63402, 0.62812, 0.62248, 0.61735,\
        0.61298, 0.60951, 0.60681, 0.60460, 0.60263, 0.60065, 0.59811, 0.59466, 0.59079, 0.58672, 0.58255,\
        0.57828, 0.57393, 0.56947, 0.56490, 0.56019, 0.55534, 0.55033, 0.54514, 0.53976, 0.53418, 0.52838],\
        [0.56332, 0.56743, 0.57157, 0.57578, 0.58015, 0.58477, 0.58980, 0.59549, 0.60216, 0.60906, 0.61627,\
        0.62403, 0.63234, 0.64101, 0.64978, 0.65842, 0.66671, 0.67447, 0.68157, 0.68793, 0.69352, 0.69829,\
        0.70246, 0.70607, 0.70963, 0.71354, 0.71783, 0.72244, 0.72733, 0.73244, 0.73768, 0.74297, 0.74821,\
        0.75334, 0.75830, 0.76308, 0.76763, 0.77192, 0.77585, 0.77925, 0.78194, 0.78385, 0.78513, 0.78616,\
        0.78740, 0.78931, 0.79240, 0.79656, 0.80108, 0.80554, 0.80922, 0.81092, 0.81098, 0.81050, 0.80942,\
        0.80768, 0.80520, 0.80151, 0.79840, 0.79592, 0.79343, 0.79097, 0.78925, 0.78814, 0.78684, 0.78520,\
        0.78346, 0.78150, 0.77925, 0.77684, 0.77436, 0.77177, 0.76898, 0.76583, 0.76218, 0.75788, 0.75283,\
        0.74701, 0.74045, 0.73324, 0.72553, 0.71753, 0.70950, 0.70165, 0.69396, 0.68686, 0.68051, 0.67467,\
        0.66909, 0.66359, 0.65807, 0.65245, 0.64670, 0.64085, 0.63491, 0.62897, 0.62309, 0.61745, 0.61228,\
        0.60801, 0.60494, 0.60278, 0.60100, 0.59935, 0.59757, 0.59525, 0.59222, 0.58867, 0.58481, 0.58076,\
        0.57657, 0.57226, 0.56782, 0.56325, 0.55855, 0.55369, 0.54867, 0.54347, 0.53807, 0.53247, 0.52666],\
        [0.55974, 0.56373, 0.56775, 0.57186, 0.57615, 0.58074, 0.58576, 0.59144, 0.59768, 0.60393, 0.61058,\
        0.61804, 0.62629, 0.63502, 0.64391, 0.65271, 0.66123, 0.66929, 0.67678, 0.68363, 0.68978, 0.69525,\
        0.69996, 0.70351, 0.70717, 0.71128, 0.71562, 0.72014, 0.72480, 0.72958, 0.73445, 0.73934, 0.74421,\
        0.74900, 0.75366, 0.75814, 0.76240, 0.76636, 0.76993, 0.77300, 0.77546, 0.77733, 0.77874, 0.78001,\
        0.78149, 0.78358, 0.78653, 0.79030, 0.79452, 0.79888, 0.80280, 0.80360, 0.80239, 0.80166, 0.80033,\
        0.79845, 0.79624, 0.79277, 0.78960, 0.78701, 0.78465, 0.78252, 0.78082, 0.77950, 0.77821, 0.77680,\
        0.77526, 0.77357, 0.77169, 0.76967, 0.76751, 0.76518, 0.76261, 0.75966, 0.75622, 0.75216, 0.74742,\
        0.74196, 0.73579, 0.72899, 0.72165, 0.71397, 0.70623, 0.69863, 0.69107, 0.68388, 0.67719, 0.67095,\
        0.66502, 0.65925, 0.65354, 0.64783, 0.64208, 0.63627, 0.63044, 0.62461, 0.61885, 0.61327, 0.60804,\
        0.60366, 0.60099, 0.59935, 0.59786, 0.59634, 0.59459, 0.59238, 0.58961, 0.58633, 0.58269, 0.57880,\
        0.57471, 0.57045, 0.56605, 0.56151, 0.55681, 0.55196, 0.54693, 0.54172, 0.53631, 0.53070, 0.52487],\
        [0.55623, 0.56011, 0.56403, 0.56805, 0.57226, 0.57675, 0.58163, 0.58695, 0.59260, 0.59847, 0.60460,\
        0.61196, 0.62031, 0.62911, 0.63807, 0.64699, 0.65567, 0.66394, 0.67171, 0.67890, 0.68547, 0.69140,\
        0.69650, 0.69997, 0.70418, 0.70838, 0.71276, 0.71718, 0.72162, 0.72611, 0.73065, 0.73521, 0.73974,\
        0.74422, 0.74859, 0.75280, 0.75679, 0.76047, 0.76376, 0.76658, 0.76888, 0.77071, 0.77220, 0.77359,\
        0.77516, 0.77720, 0.77989, 0.78323, 0.78704, 0.79102, 0.79431, 0.79477, 0.79342, 0.79229, 0.79092,\
        0.78907, 0.78682, 0.78399, 0.78111, 0.77854, 0.77625, 0.77423, 0.77253, 0.77110, 0.76979, 0.76848,\
        0.76712, 0.76569, 0.76414, 0.76248, 0.76066, 0.75866, 0.75636, 0.75368, 0.75051, 0.74674, 0.74233,\
        0.73724, 0.73148, 0.72509, 0.71812, 0.71065, 0.70292, 0.69545, 0.68809, 0.68089, 0.67401, 0.66750,\
        0.66129, 0.65531, 0.64947, 0.64371, 0.63799, 0.63229, 0.62662, 0.62102, 0.61554, 0.61025, 0.60528,\
        0.60087, 0.59841, 0.59689, 0.59539, 0.59378, 0.59194, 0.58972, 0.58705, 0.58394, 0.58047, 0.57671,\
        0.57272, 0.56853, 0.56418, 0.55967, 0.55499, 0.55014, 0.54511, 0.53989, 0.53448, 0.52885, 0.52300],\
        [0.55279, 0.55657, 0.56040, 0.56433, 0.56844, 0.57279, 0.57745, 0.58241, 0.58763, 0.59302, 0.59885,\
        0.60639, 0.61471, 0.62345, 0.63241, 0.64136, 0.65010, 0.65848, 0.66638, 0.67374, 0.68051, 0.68661,\
        0.69191, 0.69649, 0.70061, 0.70479, 0.70929, 0.71354, 0.71772, 0.72195, 0.72623, 0.73051, 0.73476,\
        0.73896, 0.74309, 0.74707, 0.75084, 0.75430, 0.75738, 0.76002, 0.76220, 0.76397, 0.76544, 0.76681,\
        0.76829, 0.77010, 0.77237, 0.77511, 0.77820, 0.78126, 0.78352, 0.78412, 0.78343, 0.78237, 0.78109,\
        0.77947, 0.77748, 0.77513, 0.77265, 0.77027, 0.76808, 0.76611, 0.76438, 0.76288, 0.76154, 0.76029,\
        0.75909, 0.75789, 0.75665, 0.75533, 0.75388, 0.75222, 0.75026, 0.74789, 0.74502, 0.74158, 0.73750,\
        0.73277, 0.72742, 0.72146, 0.71490, 0.70775, 0.70008, 0.69259, 0.68517, 0.67788, 0.67085, 0.66416,\
        0.65778, 0.65166, 0.64576, 0.64001, 0.63437, 0.62884, 0.62341, 0.61813, 0.61306, 0.60831, 0.60409,\
        0.60068, 0.59781, 0.59555, 0.59361, 0.59171, 0.58966, 0.58733, 0.58465, 0.58160, 0.57822, 0.57455,\
        0.57064, 0.56652, 0.56221, 0.55772, 0.55306, 0.54823, 0.54320, 0.53799, 0.53256, 0.52693, 0.52106],\
        [0.54943, 0.55312, 0.55686, 0.56071, 0.56471, 0.56893, 0.57340, 0.57813, 0.58313, 0.58849, 0.59457,\
        0.60175, 0.60967, 0.61817, 0.62701, 0.63590, 0.64460, 0.65295, 0.66085, 0.66822, 0.67501, 0.68118,\
        0.68668, 0.69158, 0.69603, 0.70073, 0.70515, 0.70907, 0.71299, 0.71704, 0.72114, 0.72519, 0.72919,\
        0.73318, 0.73713, 0.74096, 0.74458, 0.74789, 0.75083, 0.75333, 0.75541, 0.75709, 0.75847, 0.75969,\
        0.76093, 0.76233, 0.76402, 0.76602, 0.76823, 0.77035, 0.77194, 0.77262, 0.77249, 0.77190, 0.77100,\
        0.76979, 0.76822, 0.76634, 0.76426, 0.76213, 0.76007, 0.75814, 0.75639, 0.75483, 0.75346, 0.75225,\
        0.75117, 0.75019, 0.74924, 0.74827, 0.74719, 0.74591, 0.74431, 0.74229, 0.73975, 0.73662, 0.73285,\
        0.72844, 0.72344, 0.71787, 0.71176, 0.70510, 0.69782, 0.68994, 0.68223, 0.67476, 0.66760, 0.66080,\
        0.65433, 0.64818, 0.64229, 0.63662, 0.63113, 0.62582, 0.62069, 0.61578, 0.61115, 0.60693, 0.60324,\
        0.60024, 0.59734, 0.59464, 0.59225, 0.58999, 0.58768, 0.58518, 0.58241, 0.57935, 0.57599, 0.57237,\
        0.56850, 0.56442, 0.56015, 0.55569, 0.55105, 0.54622, 0.54121, 0.53599, 0.53056, 0.52491, 0.51903],\
        [0.54616, 0.54977, 0.55342, 0.55718, 0.56110, 0.56521, 0.56957, 0.57420, 0.57915, 0.58455, 0.59061,\
        0.59745, 0.60498, 0.61324, 0.62194, 0.63069, 0.63924, 0.64744, 0.65518, 0.66242, 0.66911, 0.67522,\
        0.68079, 0.68589, 0.69074, 0.69555, 0.69977, 0.70349, 0.70726, 0.71133, 0.71538, 0.71923, 0.72301,\
        0.72684, 0.73071, 0.73448, 0.73803, 0.74127, 0.74412, 0.74653, 0.74851, 0.75008, 0.75131, 0.75230,\
        0.75318, 0.75407, 0.75507, 0.75624, 0.75756, 0.75889, 0.76002, 0.76078, 0.76113, 0.76114, 0.76083,\
        0.76018, 0.75914, 0.75774, 0.75606, 0.75420, 0.75227, 0.75037, 0.74857, 0.74695, 0.74553, 0.74435,\
        0.74338, 0.74261, 0.74195, 0.74133, 0.74064, 0.73975, 0.73854, 0.73689, 0.73468, 0.73183, 0.72831,\
        0.72413, 0.71935, 0.71408, 0.70830, 0.70174, 0.69450, 0.68671, 0.67889, 0.67133, 0.66412, 0.65730,\
        0.65085, 0.64476, 0.63897, 0.63345, 0.62817, 0.62311, 0.61829, 0.61374, 0.60952, 0.60571, 0.60233,\
        0.59923, 0.59642, 0.59365, 0.59101, 0.58847, 0.58590, 0.58321, 0.58032, 0.57718, 0.57380, 0.57018,\
        0.56633, 0.56227, 0.55801, 0.55357, 0.54895, 0.54413, 0.53912, 0.53391, 0.52848, 0.52282, 0.51692],\
        [0.54298, 0.54650, 0.55008, 0.55376, 0.55760, 0.56165, 0.56595, 0.57055, 0.57551, 0.58093, 0.58690,\
        0.59344, 0.60060, 0.60875, 0.61731, 0.62581, 0.63406, 0.64196, 0.64944, 0.65642, 0.66287, 0.66881,\
        0.67428, 0.67938, 0.68421, 0.68880, 0.69296, 0.69674, 0.70059, 0.70491, 0.70896, 0.71259, 0.71613,\
        0.71992, 0.72384, 0.72766, 0.73122, 0.73444, 0.73726, 0.73963, 0.74153, 0.74298, 0.74403, 0.74476,\
        0.74523, 0.74555, 0.74581, 0.74613, 0.74659, 0.74723, 0.74803, 0.74892, 0.74977, 0.75046, 0.75086,\
        0.75086, 0.75041, 0.74950, 0.74818, 0.74656, 0.74474, 0.74285, 0.74099, 0.73927, 0.73778, 0.73660,\
        0.73573, 0.73516, 0.73480, 0.73454, 0.73424, 0.73377, 0.73297, 0.73170, 0.72982, 0.72722, 0.72387,\
        0.71977, 0.71503, 0.70979, 0.70419, 0.69786, 0.69042, 0.68267, 0.67493, 0.66742, 0.66028, 0.65356,\
        0.64724, 0.64130, 0.63571, 0.63041, 0.62538, 0.62061, 0.61610, 0.61188, 0.60798, 0.60445, 0.60134,\
        0.59841, 0.59542, 0.59255, 0.58976, 0.58701, 0.58423, 0.58136, 0.57832, 0.57509, 0.57164, 0.56798,\
        0.56412, 0.56005, 0.55580, 0.55137, 0.54675, 0.54195, 0.53694, 0.53173, 0.52630, 0.52063, 0.51472],\
        [0.53987, 0.54331, 0.54681, 0.55043, 0.55421, 0.55821, 0.56249, 0.56710, 0.57211, 0.57756, 0.58353,\
        0.59001, 0.59708, 0.60497, 0.61319, 0.62123, 0.62904, 0.63654, 0.64364, 0.65026, 0.65637, 0.66200,\
        0.66721, 0.67209, 0.67672, 0.68110, 0.68523, 0.68919, 0.69331, 0.69770, 0.70178, 0.70513, 0.70846,\
        0.71245, 0.71659, 0.72054, 0.72416, 0.72743, 0.73027, 0.73264, 0.73450, 0.73585, 0.73674, 0.73721,\
        0.73728, 0.73703, 0.73656, 0.73602, 0.73564, 0.73565, 0.73621, 0.73732, 0.73875, 0.74020, 0.74138,\
        0.74208, 0.74222, 0.74176, 0.74078, 0.73935, 0.73760, 0.73566, 0.73369, 0.73183, 0.73023, 0.72902,\
        0.72824, 0.72787, 0.72781, 0.72792, 0.72803, 0.72798, 0.72761, 0.72673, 0.72518, 0.72280, 0.71954,\
        0.71539, 0.71047, 0.70493, 0.69909, 0.69268, 0.68545, 0.67786, 0.67028, 0.66295, 0.65601, 0.64950,\
        0.64343, 0.63775, 0.63243, 0.62742, 0.62269, 0.61823, 0.61402, 0.61010, 0.60646, 0.60308, 0.59996,\
        0.59711, 0.59424, 0.59134, 0.58845, 0.58556, 0.58261, 0.57956, 0.57638, 0.57303, 0.56950, 0.56578,\
        0.56187, 0.55778, 0.55352, 0.54908, 0.54446, 0.53966, 0.53467, 0.52946, 0.52403, 0.51836, 0.51243],\
        [0.53683, 0.54018, 0.54360, 0.54715, 0.55087, 0.55484, 0.55911, 0.56374, 0.56879, 0.57432, 0.58037,\
        0.58700, 0.59423, 0.60195, 0.60942, 0.61676, 0.62407, 0.63114, 0.63782, 0.64399, 0.64965, 0.65485,\
        0.65966, 0.66420, 0.66852, 0.67270, 0.67678, 0.68085, 0.68505, 0.68934, 0.69339, 0.69674, 0.70026,\
        0.70465, 0.70907, 0.71316, 0.71685, 0.72021, 0.72316, 0.72559, 0.72743, 0.72873, 0.72951, 0.72978,\
        0.72954, 0.72879, 0.72764, 0.72629, 0.72508, 0.72445, 0.72482, 0.72629, 0.72845, 0.73074, 0.73271,\
        0.73410, 0.73477, 0.73471, 0.73398, 0.73267, 0.73093, 0.72891, 0.72677, 0.72471, 0.72294, 0.72165,\
        0.72094, 0.72079, 0.72105, 0.72153, 0.72204, 0.72242, 0.72247, 0.72201, 0.72079, 0.71862, 0.71538,\
        0.71108, 0.70588, 0.69995, 0.69363, 0.68695, 0.67977, 0.67234, 0.66499, 0.65792, 0.65128, 0.64511,\
        0.63939, 0.63408, 0.62912, 0.62446, 0.62007, 0.61592, 0.61201, 0.60836, 0.60497, 0.60180, 0.59875,\
        0.59586, 0.59299, 0.59007, 0.58711, 0.58409, 0.58099, 0.57779, 0.57447, 0.57099, 0.56735, 0.56354,\
        0.55957, 0.55543, 0.55114, 0.54669, 0.54207, 0.53728, 0.53230, 0.52710, 0.52167, 0.51599, 0.51005],\
        [0.53385, 0.53710, 0.54043, 0.54389, 0.54756, 0.55148, 0.55573, 0.56036, 0.56543, 0.57101, 0.57712,\
        0.58380, 0.59104, 0.59849, 0.60517, 0.61207, 0.61903, 0.62574, 0.63197, 0.63764, 0.64275, 0.64740,\
        0.65172, 0.65581, 0.65978, 0.66371, 0.66767, 0.67173, 0.67590, 0.68011, 0.68417, 0.68800, 0.69206,\
        0.69659, 0.70132, 0.70544, 0.70921, 0.71277, 0.71593, 0.71848, 0.72033, 0.72160, 0.72237, 0.72260,\
        0.72218, 0.72108, 0.71938, 0.71731, 0.71529, 0.71398, 0.71420, 0.71622, 0.71927, 0.72245, 0.72516,\
        0.72713, 0.72822, 0.72845, 0.72787, 0.72662, 0.72483, 0.72267, 0.72032, 0.71800, 0.71599, 0.71458,\
        0.71391, 0.71399, 0.71459, 0.71542, 0.71630, 0.71706, 0.71755, 0.71753, 0.71669, 0.71473, 0.71145,\
        0.70689, 0.70127, 0.69496, 0.68812, 0.68098, 0.67363, 0.66627, 0.65912, 0.65237, 0.64611, 0.64038,\
        0.63512, 0.63027, 0.62575, 0.62151, 0.61748, 0.61366, 0.61005, 0.60664, 0.60347, 0.60052, 0.59759,\
        0.59467, 0.59176, 0.58879, 0.58575, 0.58261, 0.57937, 0.57602, 0.57254, 0.56893, 0.56517, 0.56126,\
        0.55720, 0.55301, 0.54867, 0.54419, 0.53956, 0.53478, 0.52981, 0.52464, 0.51922, 0.51354, 0.50758],\
        [0.53091, 0.53406, 0.53727, 0.54063, 0.54420, 0.54805, 0.55225, 0.55685, 0.56190, 0.56745, 0.57351,\
        0.58004, 0.58690, 0.59369, 0.60029, 0.60711, 0.61393, 0.62036, 0.62615, 0.63123, 0.63569, 0.63971,\
        0.64344, 0.64703, 0.65059, 0.65424, 0.65804, 0.66202, 0.66616, 0.67039, 0.67463, 0.67890, 0.68337,\
        0.68812, 0.69291, 0.69719, 0.70124, 0.70511, 0.70859, 0.71131, 0.71314, 0.71442, 0.71533, 0.71569,\
        0.71528, 0.71406, 0.71203, 0.70947, 0.70684, 0.70485, 0.70491, 0.70776, 0.71178, 0.71569, 0.71896,\
        0.72132, 0.72267, 0.72303, 0.72251, 0.72122, 0.71934, 0.71702, 0.71445, 0.71182, 0.70951, 0.70795,\
        0.70727, 0.70762, 0.70856, 0.70966, 0.71081, 0.71191, 0.71279, 0.71325, 0.71288, 0.71114, 0.70776,\
        0.70287, 0.69669, 0.68975, 0.68234, 0.67471, 0.66709, 0.65970, 0.65273, 0.64632, 0.64054, 0.63534,\
        0.63065, 0.62636, 0.62237, 0.61859, 0.61497, 0.61149, 0.60815, 0.60496, 0.60194, 0.59910, 0.59633,\
        0.59348, 0.59055, 0.58752, 0.58437, 0.58111, 0.57773, 0.57422, 0.57059, 0.56683, 0.56293, 0.55891,\
        0.55475, 0.55048, 0.54608, 0.54157, 0.53692, 0.53214, 0.52720, 0.52207, 0.51669, 0.51102, 0.50504],\
        [0.52802, 0.53102, 0.53410, 0.53733, 0.54078, 0.54452, 0.54862, 0.55314, 0.55811, 0.56356, 0.56948,\
        0.57580, 0.58235, 0.58892, 0.59547, 0.60219, 0.60891, 0.61508, 0.62038, 0.62477, 0.62849, 0.63181,\
        0.63491, 0.63795, 0.64106, 0.64440, 0.64802, 0.65193, 0.65607, 0.66039, 0.66484, 0.66943, 0.67421,\
        0.67914, 0.68403, 0.68866, 0.69305, 0.69724, 0.70109, 0.70404, 0.70571, 0.70706, 0.70832, 0.70904,\
        0.70878, 0.70771, 0.70566, 0.70293, 0.70027, 0.69824, 0.69843, 0.70196, 0.70640, 0.71063, 0.71416,\
        0.71668, 0.71808, 0.71841, 0.71780, 0.71642, 0.71441, 0.71196, 0.70924, 0.70638, 0.70366, 0.70212,\
        0.70126, 0.70198, 0.70311, 0.70428, 0.70560, 0.70693, 0.70808, 0.70905, 0.70931, 0.70787, 0.70419,\
        0.69876, 0.69195, 0.68426, 0.67618, 0.66802, 0.66009, 0.65264, 0.64585, 0.63984, 0.63461, 0.63006,\
        0.62604, 0.62240, 0.61900, 0.61574, 0.61256, 0.60944, 0.60637, 0.60340, 0.60055, 0.59784, 0.59515,\
        0.59234, 0.58937, 0.58625, 0.58299, 0.57958, 0.57605, 0.57238, 0.56858, 0.56467, 0.56063, 0.55647,\
        0.55221, 0.54784, 0.54336, 0.53880, 0.53413, 0.52935, 0.52444, 0.51937, 0.51407, 0.50846, 0.50245],\
        [0.52515, 0.52800, 0.53091, 0.53397, 0.53726, 0.54084, 0.54480, 0.54918, 0.55402, 0.55933, 0.56508,\
        0.57120, 0.57758, 0.58409, 0.59070, 0.59739, 0.60398, 0.60991, 0.61463, 0.61819, 0.62108, 0.62370,\
        0.62619, 0.62866, 0.63130, 0.63433, 0.63780, 0.64165, 0.64582, 0.65024, 0.65488, 0.65973, 0.66475,\
        0.66989, 0.67499, 0.67992, 0.68464, 0.68909, 0.69310, 0.69617, 0.69798, 0.69957, 0.70127, 0.70261,\
        0.70227, 0.70169, 0.69998, 0.69740, 0.69529, 0.69421, 0.69519, 0.69853, 0.70279, 0.70702, 0.71058,\
        0.71303, 0.71427, 0.71439, 0.71359, 0.71205, 0.70992, 0.70736, 0.70463, 0.70186, 0.69902, 0.69731,\
        0.69662, 0.69713, 0.69817, 0.69926, 0.70070, 0.70214, 0.70330, 0.70479, 0.70580, 0.70489, 0.70076,\
        0.69437, 0.68671, 0.67825, 0.66950, 0.66083, 0.65259, 0.64507, 0.63849, 0.63294, 0.62838, 0.62460,\
        0.62137, 0.61847, 0.61572, 0.61301, 0.61030, 0.60756, 0.60478, 0.60203, 0.59939, 0.59681, 0.59414,\
        0.59129, 0.58823, 0.58499, 0.58158, 0.57801, 0.57431, 0.57047, 0.56651, 0.56243, 0.55824, 0.55394,\
        0.54954, 0.54506, 0.54050, 0.53586, 0.53115, 0.52636, 0.52148, 0.51647, 0.51128, 0.50589, 0.50000],\
        [0.52233, 0.52498, 0.52769, 0.53054, 0.53362, 0.53700, 0.54077, 0.54496, 0.54962, 0.55475, 0.56032,\
        0.56627, 0.57252, 0.57900, 0.58565, 0.59236, 0.59889, 0.60466, 0.60875, 0.61129, 0.61335, 0.61542,\
        0.61741, 0.61930, 0.62145, 0.62420, 0.62758, 0.63143, 0.63562, 0.64012, 0.64491, 0.64996, 0.65520,\
        0.66055, 0.66588, 0.67107, 0.67601, 0.68059, 0.68463, 0.68782, 0.69017, 0.69218, 0.69411, 0.69554,\
        0.69552, 0.69515, 0.69400, 0.69226, 0.69100, 0.69091, 0.69252, 0.69581, 0.70002, 0.70427, 0.70780,\
        0.71002, 0.71089, 0.71066, 0.70957, 0.70786, 0.70565, 0.70297, 0.70022, 0.69761, 0.69511, 0.69333,\
        0.69257, 0.69275, 0.69353, 0.69462, 0.69599, 0.69746, 0.69860, 0.70073, 0.70155, 0.70124, 0.69684,\
        0.68929, 0.68068, 0.67151, 0.66220, 0.65310, 0.64459, 0.63703, 0.63070, 0.62572, 0.62196, 0.61909,\
        0.61675, 0.61464, 0.61257, 0.61044, 0.60822, 0.60589, 0.60344, 0.60093, 0.59851, 0.59602, 0.59329,\
        0.59031, 0.58710, 0.58369, 0.58011, 0.57638, 0.57250, 0.56848, 0.56435, 0.56010, 0.55574, 0.55129,\
        0.54676, 0.54215, 0.53748, 0.53275, 0.52797, 0.52314, 0.51826, 0.51330, 0.50816, 0.50275, 0.49722],\
        [0.51954, 0.52197, 0.52444, 0.52704, 0.52986, 0.53300, 0.53652, 0.54048, 0.54491, 0.54982, 0.55518,\
        0.56095, 0.56706, 0.57344, 0.58001, 0.58662, 0.59304, 0.59872, 0.60226, 0.60374, 0.60519, 0.60707,\
        0.60878, 0.61006, 0.61167, 0.61423, 0.61765, 0.62152, 0.62570, 0.63022, 0.63511, 0.64032, 0.64576,\
        0.65131, 0.65685, 0.66223, 0.66733, 0.67200, 0.67609, 0.67947, 0.68219, 0.68447, 0.68640, 0.68773,\
        0.68820, 0.68810, 0.68753, 0.68678, 0.68650, 0.68728, 0.68945, 0.69295, 0.69726, 0.70169, 0.70528,\
        0.70712, 0.70743, 0.70678, 0.70536, 0.70347, 0.70131, 0.69861, 0.69583, 0.69329, 0.69106, 0.68941,\
        0.68857, 0.68850, 0.68903, 0.68997, 0.69118, 0.69253, 0.69396, 0.69545, 0.69563, 0.69431, 0.68980,\
        0.68229, 0.67340, 0.66390, 0.65428, 0.64491, 0.63621, 0.62863, 0.62258, 0.61828, 0.61550, 0.61368,\
        0.61230, 0.61100, 0.60959, 0.60801, 0.60629, 0.60444, 0.60241, 0.60018, 0.59798, 0.59543, 0.59252,\
        0.58932, 0.58591, 0.58231, 0.57855, 0.57463, 0.57058, 0.56639, 0.56208, 0.55766, 0.55314, 0.54853,\
        0.54385, 0.53910, 0.53430, 0.52946, 0.52458, 0.51968, 0.51477, 0.50981, 0.50476, 0.49933, 0.49347],\
        [0.51683, 0.51900, 0.52118, 0.52348, 0.52600, 0.52884, 0.53207, 0.53575, 0.53992, 0.54457, 0.54969,\
        0.55524, 0.56113, 0.56730, 0.57363, 0.57994, 0.58591, 0.59098, 0.59419, 0.59567, 0.59701, 0.59885,\
        0.60045, 0.60115, 0.60230, 0.60483, 0.60843, 0.61223, 0.61625, 0.62073, 0.62569, 0.63102, 0.63662,\
        0.64236, 0.64807, 0.65361, 0.65881, 0.66354, 0.66769, 0.67120, 0.67410, 0.67648, 0.67838, 0.67971,\
        0.68044, 0.68070, 0.68070, 0.68076, 0.68132, 0.68281, 0.68546, 0.68923, 0.69378, 0.69853, 0.70221,\
        0.70355, 0.70316, 0.70228, 0.70060, 0.69853, 0.69636, 0.69382, 0.69120, 0.68879, 0.68675, 0.68522,\
        0.68434, 0.68409, 0.68438, 0.68507, 0.68600, 0.68703, 0.68801, 0.68854, 0.68791, 0.68553, 0.68073,\
        0.67359, 0.66493, 0.65553, 0.64591, 0.63650, 0.62773, 0.62013, 0.61435, 0.61083, 0.60925, 0.60857,\
        0.60815, 0.60761, 0.60676, 0.60562, 0.60435, 0.60304, 0.60167, 0.60007, 0.59774, 0.59483, 0.59162,\
        0.58818, 0.58455, 0.58077, 0.57683, 0.57274, 0.56852, 0.56417, 0.55969, 0.55511, 0.55042, 0.54565,\
        0.54081, 0.53592, 0.53098, 0.52600, 0.52100, 0.51599, 0.51098, 0.50595, 0.50083, 0.49550, 0.48956],\
        [0.51420, 0.51608, 0.51794, 0.51990, 0.52206, 0.52455, 0.52745, 0.53081, 0.53467, 0.53904, 0.54388,\
        0.54914, 0.55476, 0.56061, 0.56658, 0.57242, 0.57783, 0.58232, 0.58546, 0.58744, 0.58905, 0.59069,\
        0.59205, 0.59294, 0.59420, 0.59674, 0.60021, 0.60372, 0.60747, 0.61185, 0.61688, 0.62227, 0.62798,\
        0.63387, 0.63974, 0.64539, 0.65066, 0.65541, 0.65957, 0.66310, 0.66603, 0.66840, 0.67024, 0.67158,\
        0.67246, 0.67302, 0.67346, 0.67409, 0.67525, 0.67723, 0.68020, 0.68410, 0.68866, 0.69330, 0.69702,\
        0.69819, 0.69754, 0.69659, 0.69501, 0.69297, 0.69078, 0.68845, 0.68609, 0.68390, 0.68204, 0.68063,\
        0.67973, 0.67935, 0.67940, 0.67976, 0.68030, 0.68082, 0.68108, 0.68070, 0.67918, 0.67604, 0.67096,\
        0.66405, 0.65577, 0.64674, 0.63745, 0.62829, 0.61963, 0.61206, 0.60651, 0.60377, 0.60367, 0.60394,\
        0.60436, 0.60446, 0.60399, 0.60313, 0.60216, 0.60126, 0.60039, 0.59934, 0.59688, 0.59372, 0.59030,\
        0.58668, 0.58291, 0.57897, 0.57489, 0.57067, 0.56631, 0.56181, 0.55718, 0.55243, 0.54758, 0.54265,\
        0.53766, 0.53262, 0.52754, 0.52242, 0.51728, 0.51212, 0.50699, 0.50187, 0.49665, 0.49120, 0.48530],\
        [0.51169, 0.51326, 0.51476, 0.51633, 0.51810, 0.52019, 0.52271, 0.52572, 0.52925, 0.53329, 0.53781,\
        0.54276, 0.54803, 0.55353, 0.55907, 0.56445, 0.56938, 0.57355, 0.57675, 0.57911, 0.58101, 0.58269,\
        0.58415, 0.58548, 0.58716, 0.58963, 0.59275, 0.59605, 0.59958, 0.60393, 0.60899, 0.61427, 0.62004,\
        0.62608, 0.63207, 0.63778, 0.64305, 0.64776, 0.65185, 0.65530, 0.65814, 0.66040, 0.66214, 0.66343,\
        0.66436, 0.66507, 0.66579, 0.66675, 0.66824, 0.67047, 0.67354, 0.67737, 0.68166, 0.68585, 0.68910,\
        0.69054, 0.69047, 0.68967, 0.68832, 0.68657, 0.68460, 0.68252, 0.68045, 0.67852, 0.67687, 0.67557,\
        0.67468, 0.67418, 0.67400, 0.67403, 0.67411, 0.67404, 0.67358, 0.67238, 0.67009, 0.66637, 0.66108,\
        0.65432, 0.64644, 0.63793, 0.62923, 0.62070, 0.61258, 0.60525, 0.60007, 0.59824, 0.59903, 0.59971,\
        0.60081, 0.60135, 0.60110, 0.60036, 0.59957, 0.59886, 0.59808, 0.59684, 0.59468, 0.59173, 0.58835,\
        0.58471, 0.58088, 0.57687, 0.57271, 0.56839, 0.56391, 0.55929, 0.55453, 0.54964, 0.54464, 0.53955,\
        0.53441, 0.52922, 0.52401, 0.51878, 0.51349, 0.50815, 0.50288, 0.49769, 0.49239, 0.48683, 0.48089],\
        [0.50935, 0.51059, 0.51170, 0.51283, 0.51416, 0.51582, 0.51793, 0.52056, 0.52373, 0.52743, 0.53161,\
        0.53621, 0.54112, 0.54623, 0.55138, 0.55636, 0.56096, 0.56495, 0.56825, 0.57091, 0.57313, 0.57507,\
        0.57685, 0.57862, 0.58067, 0.58321, 0.58619, 0.58946, 0.59309, 0.59750, 0.60215, 0.60711, 0.61302,\
        0.61920, 0.62525, 0.63094, 0.63613, 0.64071, 0.64463, 0.64791, 0.65055, 0.65262, 0.65419, 0.65534,\
        0.65619, 0.65691, 0.65770, 0.65878, 0.66037, 0.66262, 0.66560, 0.66920, 0.67312, 0.67687, 0.67983,\
        0.68154, 0.68204, 0.68169, 0.68075, 0.67939, 0.67777, 0.67603, 0.67428, 0.67264, 0.67121, 0.67005,\
        0.66918, 0.66859, 0.66821, 0.66791, 0.66755, 0.66693, 0.66582, 0.66394, 0.66103, 0.65689, 0.65144,\
        0.64481, 0.63731, 0.62937, 0.62145, 0.61394, 0.60702, 0.60062, 0.59618, 0.59457, 0.59487, 0.59582,\
        0.59698, 0.59772, 0.59771, 0.59721, 0.59659, 0.59591, 0.59504, 0.59371, 0.59168, 0.58897, 0.58577,\
        0.58223, 0.57844, 0.57444, 0.57025, 0.56588, 0.56134, 0.55663, 0.55176, 0.54675, 0.54161, 0.53637,\
        0.53106, 0.52574, 0.52043, 0.51512, 0.50977, 0.50427, 0.49879, 0.49353, 0.48812, 0.48245, 0.47643],\
        [0.50723, 0.50812, 0.50881, 0.50948, 0.51032, 0.51153, 0.51321, 0.51543, 0.51823, 0.52158, 0.52541,\
        0.52965, 0.53420, 0.53894, 0.54374, 0.54840, 0.55277, 0.55669, 0.56010, 0.56304, 0.56559, 0.56790,\
        0.57009, 0.57231, 0.57473, 0.57747, 0.58056, 0.58396, 0.58774, 0.59193, 0.59624, 0.60127, 0.60730,\
        0.61347, 0.61945, 0.62501, 0.63000, 0.63435, 0.63802, 0.64101, 0.64337, 0.64516, 0.64645, 0.64737,\
        0.64804, 0.64862, 0.64930, 0.65029, 0.65176, 0.65387, 0.65664, 0.65995, 0.66354, 0.66699, 0.66985,\
        0.67179, 0.67278, 0.67295, 0.67250, 0.67159, 0.67039, 0.66904, 0.66764, 0.66630, 0.66510, 0.66410,\
        0.66329, 0.66265, 0.66210, 0.66152, 0.66078, 0.65970, 0.65806, 0.65565, 0.65228, 0.64783, 0.64228,\
        0.63577, 0.62862, 0.62123, 0.61408, 0.60767, 0.60227, 0.59743, 0.59343, 0.59156, 0.59136, 0.59202,\
        0.59295, 0.59363, 0.59383, 0.59360, 0.59312, 0.59246, 0.59152, 0.59015, 0.58821, 0.58569, 0.58269,\
        0.57930, 0.57562, 0.57168, 0.56752, 0.56315, 0.55858, 0.55382, 0.54888, 0.54377, 0.53850, 0.53312,\
        0.52766, 0.52218, 0.51676, 0.51143, 0.50614, 0.50069, 0.49507, 0.48954, 0.48394, 0.47811, 0.47196],\
        [0.50536, 0.50591, 0.50618, 0.50636, 0.50669, 0.50742, 0.50867, 0.51049, 0.51291, 0.51590, 0.51937,\
        0.52324, 0.52744, 0.53185, 0.53633, 0.54075, 0.54497, 0.54887, 0.55242, 0.55561, 0.55851, 0.56123,\
        0.56386, 0.56652, 0.56934, 0.57240, 0.57573, 0.57937, 0.58331, 0.58755, 0.59210, 0.59726, 0.60299,\
        0.60897, 0.61475, 0.62004, 0.62472, 0.62873, 0.63204, 0.63467, 0.63666, 0.63809, 0.63903, 0.63962,\
        0.63999, 0.64030, 0.64073, 0.64145, 0.64265, 0.64447, 0.64695, 0.65000, 0.65334, 0.65664, 0.65952,\
        0.66171, 0.66311, 0.66377, 0.66381, 0.66337, 0.66260, 0.66164, 0.66060, 0.65957, 0.65862, 0.65779,\
        0.65707, 0.65642, 0.65576, 0.65498, 0.65395, 0.65250, 0.65047, 0.64769, 0.64402, 0.63938, 0.63380,\
        0.62742, 0.62056, 0.61366, 0.60717, 0.60152, 0.59718, 0.59340, 0.59034, 0.58857, 0.58805, 0.58834,\
        0.58895, 0.58949, 0.58973, 0.58964, 0.58926, 0.58861, 0.58765, 0.58627, 0.58440, 0.58204, 0.57921,\
        0.57599, 0.57245, 0.56862, 0.56453, 0.56021, 0.55566, 0.55090, 0.54591, 0.54073, 0.53536, 0.52984,\
        0.52423, 0.51858, 0.51298, 0.50760, 0.50245, 0.49699, 0.49133, 0.48561, 0.47980, 0.47379, 0.46750],\
        [0.50378, 0.50403, 0.50391, 0.50359, 0.50340, 0.50369, 0.50449, 0.50589, 0.50795, 0.51058, 0.51367,\
        0.51717, 0.52101, 0.52510, 0.52933, 0.53355, 0.53766, 0.54158, 0.54526, 0.54870, 0.55195, 0.55507,\
        0.55815, 0.56126, 0.56450, 0.56794, 0.57161, 0.57555, 0.57976, 0.58426, 0.58909, 0.59432, 0.59987,\
        0.60565, 0.61108, 0.61598, 0.62025, 0.62384, 0.62672, 0.62892, 0.63047, 0.63146, 0.63199, 0.63217,\
        0.63216, 0.63210, 0.63215, 0.63248, 0.63327, 0.63471, 0.63687, 0.63968, 0.64290, 0.64618, 0.64917,\
        0.65161, 0.65337, 0.65445, 0.65492, 0.65491, 0.65455, 0.65397, 0.65327, 0.65255, 0.65186, 0.65122,\
        0.65062, 0.65001, 0.64931, 0.64840, 0.64717, 0.64549, 0.64320, 0.64019, 0.63636, 0.63166, 0.62613,\
        0.61992, 0.61332, 0.60686, 0.60109, 0.59618, 0.59238, 0.58932, 0.58692, 0.58539, 0.58473, 0.58472,\
        0.58503, 0.58538, 0.58555, 0.58548, 0.58513, 0.58448, 0.58351, 0.58214, 0.58035, 0.57810, 0.57544,\
        0.57238, 0.56899, 0.56530, 0.56133, 0.55709, 0.55260, 0.54786, 0.54288, 0.53766, 0.53222, 0.52660,\
        0.52084, 0.51501, 0.50919, 0.50352, 0.49834, 0.49304, 0.48741, 0.48162, 0.47566, 0.46948, 0.46303],\
        [0.50248, 0.50248, 0.50208, 0.50137, 0.50062, 0.50057, 0.50091, 0.50186, 0.50357, 0.50582, 0.50850,\
        0.51158, 0.51506, 0.51886, 0.52285, 0.52691, 0.53093, 0.53487, 0.53868, 0.54236, 0.54594, 0.54945,\
        0.55295, 0.55651, 0.56018, 0.56402, 0.56808, 0.57236, 0.57691, 0.58172, 0.58683, 0.59223, 0.59786,\
        0.60330, 0.60825, 0.61268, 0.61649, 0.61961, 0.62202, 0.62374, 0.62481, 0.62532, 0.62538, 0.62511,\
        0.62466, 0.62419, 0.62381, 0.62364, 0.62392, 0.62489, 0.62672, 0.62935, 0.63255, 0.63592, 0.63908,\
        0.64176, 0.64382, 0.64523, 0.64606, 0.64641, 0.64640, 0.64616, 0.64578, 0.64535, 0.64492, 0.64449,\
        0.64405, 0.64353, 0.64285, 0.64189, 0.64056, 0.63874, 0.63632, 0.63322, 0.62936, 0.62473, 0.61938,\
        0.61344, 0.60715, 0.60096, 0.59593, 0.59168, 0.58826, 0.58555, 0.58349, 0.58210, 0.58136, 0.58112,\
        0.58118, 0.58131, 0.58136, 0.58121, 0.58083, 0.58016, 0.57917, 0.57783, 0.57609, 0.57395, 0.57143,\
        0.56853, 0.56530, 0.56175, 0.55792, 0.55380, 0.54941, 0.54475, 0.53981, 0.53460, 0.52913, 0.52344,\
        0.51758, 0.51164, 0.50567, 0.49978, 0.49438, 0.48904, 0.48344, 0.47758, 0.47149, 0.46515, 0.45853],\
        [0.50143, 0.50117, 0.50066, 0.49980, 0.49878, 0.49828, 0.49823, 0.49876, 0.50004, 0.50183, 0.50400,\
        0.50661, 0.50973, 0.51325, 0.51702, 0.52091, 0.52484, 0.52879, 0.53271, 0.53661, 0.54049, 0.54437,\
        0.54827, 0.55224, 0.55633, 0.56058, 0.56501, 0.56966, 0.57454, 0.57966, 0.58500, 0.59053, 0.59608,\
        0.60129, 0.60590, 0.60993, 0.61331, 0.61596, 0.61788, 0.61910, 0.61968, 0.61970, 0.61926, 0.61850,\
        0.61761, 0.61673, 0.61593, 0.61525, 0.61490, 0.61535, 0.61684, 0.61936, 0.62264, 0.62618, 0.62953,\
        0.63241, 0.63468, 0.63632, 0.63741, 0.63803, 0.63831, 0.63834, 0.63824, 0.63807, 0.63790, 0.63770,\
        0.63745, 0.63707, 0.63647, 0.63554, 0.63419, 0.63233, 0.62987, 0.62678, 0.62302, 0.61858, 0.61350,\
        0.60795, 0.60227, 0.59662, 0.59180, 0.58786, 0.58465, 0.58211, 0.58018, 0.57882, 0.57797, 0.57754,\
        0.57736, 0.57728, 0.57715, 0.57690, 0.57643, 0.57571, 0.57470, 0.57336, 0.57167, 0.56963, 0.56722,\
        0.56447, 0.56140, 0.55801, 0.55433, 0.55036, 0.54610, 0.54155, 0.53671, 0.53156, 0.52612, 0.52039,\
        0.51446, 0.50846, 0.50257, 0.49666, 0.49094, 0.48532, 0.47956, 0.47356, 0.46731, 0.46079, 0.45400],\
        [0.50059, 0.50003, 0.49938, 0.49850, 0.49747, 0.49671, 0.49639, 0.49660, 0.49740, 0.49866, 0.50030,\
        0.50240, 0.50517, 0.50843, 0.51195, 0.51563, 0.51944, 0.52335, 0.52737, 0.53146, 0.53560, 0.53980,\
        0.54407, 0.54843, 0.55290, 0.55752, 0.56232, 0.56730, 0.57248, 0.57785, 0.58336, 0.58892, 0.59435,\
        0.59939, 0.60380, 0.60757, 0.61058, 0.61277, 0.61420, 0.61493, 0.61505, 0.61460, 0.61366, 0.61242,\
        0.61109, 0.60986, 0.60879, 0.60769, 0.60660, 0.60651, 0.60770, 0.61011, 0.61358, 0.61731, 0.62079,\
        0.62377, 0.62614, 0.62789, 0.62912, 0.62992, 0.63039, 0.63064, 0.63076, 0.63084, 0.63091, 0.63096,\
        0.63094, 0.63074, 0.63026, 0.62941, 0.62811, 0.62627, 0.62385, 0.62083, 0.61724, 0.61311, 0.60840,\
        0.60318, 0.59795, 0.59292, 0.58837, 0.58455, 0.58142, 0.57892, 0.57699, 0.57557, 0.57459, 0.57396,\
        0.57356, 0.57326, 0.57295, 0.57254, 0.57197, 0.57118, 0.57012, 0.56878, 0.56712, 0.56514, 0.56285,\
        0.56023, 0.55731, 0.55409, 0.55058, 0.54677, 0.54267, 0.53828, 0.53357, 0.52855, 0.52320, 0.51750,\
        0.51150, 0.50536, 0.49942, 0.49357, 0.48770, 0.48181, 0.47580, 0.46958, 0.46312, 0.45640, 0.44941],\
        [0.49969, 0.49903, 0.49823, 0.49731, 0.49635, 0.49555, 0.49510, 0.49508, 0.49551, 0.49634, 0.49753,\
        0.49917, 0.50160, 0.50455, 0.50773, 0.51111, 0.51472, 0.51858, 0.52264, 0.52689, 0.53127, 0.53576,\
        0.54034, 0.54503, 0.54984, 0.55478, 0.55988, 0.56515, 0.57058, 0.57614, 0.58177, 0.58737, 0.59272,\
        0.59761, 0.60189, 0.60555, 0.60821, 0.60991, 0.61084, 0.61112, 0.61084, 0.61002, 0.60864, 0.60691,\
        0.60520, 0.60365, 0.60247, 0.60131, 0.59957, 0.59907, 0.59992, 0.60213, 0.60585, 0.60956, 0.61303,\
        0.61600, 0.61830, 0.62002, 0.62129, 0.62219, 0.62278, 0.62317, 0.62346, 0.62374, 0.62405, 0.62437,\
        0.62460, 0.62461, 0.62428, 0.62355, 0.62233, 0.62056, 0.61821, 0.61528, 0.61187, 0.60811, 0.60396,\
        0.59910, 0.59415, 0.58946, 0.58518, 0.58148, 0.57838, 0.57586, 0.57387, 0.57234, 0.57121, 0.57038,\
        0.56975, 0.56924, 0.56874, 0.56817, 0.56746, 0.56657, 0.56545, 0.56409, 0.56245, 0.56052, 0.55832,\
        0.55582, 0.55305, 0.55000, 0.54666, 0.54303, 0.53911, 0.53489, 0.53037, 0.52553, 0.52036, 0.51481,\
        0.50883, 0.50252, 0.49644, 0.49054, 0.48453, 0.47838, 0.47208, 0.46560, 0.45889, 0.45195, 0.44474],\
        [0.49757, 0.49750, 0.49690, 0.49611, 0.49530, 0.49460, 0.49414, 0.49402, 0.49425, 0.49483, 0.49577,\
        0.49717, 0.49919, 0.50163, 0.50434, 0.50734, 0.51069, 0.51444, 0.51854, 0.52292, 0.52749, 0.53221,\
        0.53705, 0.54201, 0.54708, 0.55228, 0.55763, 0.56311, 0.56871, 0.57441, 0.58014, 0.58578, 0.59116,\
        0.59609, 0.60042, 0.60386, 0.60603, 0.60717, 0.60760, 0.60749, 0.60694, 0.60592, 0.60426, 0.60202,\
        0.60004, 0.59829, 0.59691, 0.59564, 0.59416, 0.59351, 0.59415, 0.59629, 0.59956, 0.60295, 0.60629,\
        0.60914, 0.61119, 0.61273, 0.61398, 0.61492, 0.61558, 0.61606, 0.61646, 0.61690, 0.61744, 0.61805,\
        0.61855, 0.61876, 0.61859, 0.61798, 0.61687, 0.61519, 0.61292, 0.61006, 0.60674, 0.60320, 0.59964,\
        0.59525, 0.59057, 0.58612, 0.58205, 0.57846, 0.57539, 0.57284, 0.57076, 0.56910, 0.56779, 0.56676,\
        0.56593, 0.56520, 0.56450, 0.56375, 0.56290, 0.56190, 0.56070, 0.55930, 0.55765, 0.55576, 0.55363,\
        0.55125, 0.54861, 0.54572, 0.54256, 0.53912, 0.53538, 0.53135, 0.52702, 0.52241, 0.51751, 0.51226,\
        0.50652, 0.50029, 0.49400, 0.48774, 0.48140, 0.47493, 0.46833, 0.46155, 0.45458, 0.44739, 0.43996],\
        [0.49489, 0.49535, 0.49520, 0.49475, 0.49420, 0.49371, 0.49338, 0.49329, 0.49348, 0.49396, 0.49476,\
        0.49595, 0.49756, 0.49950, 0.50170, 0.50425, 0.50730, 0.51093, 0.51506, 0.51954, 0.52426, 0.52915,\
        0.53418, 0.53932, 0.54458, 0.54997, 0.55547, 0.56109, 0.56680, 0.57257, 0.57834, 0.58400, 0.58944,\
        0.59451, 0.59898, 0.60211, 0.60367, 0.60426, 0.60427, 0.60385, 0.60311, 0.60208, 0.60059, 0.59798,\
        0.59572, 0.59379, 0.59217, 0.59077, 0.58960, 0.58912, 0.58973, 0.59154, 0.59424, 0.59729, 0.60047,\
        0.60316, 0.60466, 0.60596, 0.60723, 0.60819, 0.60887, 0.60941, 0.60990, 0.61044, 0.61119, 0.61214,\
        0.61291, 0.61327, 0.61320, 0.61268, 0.61166, 0.61009, 0.60793, 0.60513, 0.60183, 0.59837, 0.59490,\
        0.59103, 0.58683, 0.58269, 0.57884, 0.57537, 0.57234, 0.56975, 0.56758, 0.56579, 0.56432, 0.56309,\
        0.56205, 0.56111, 0.56022, 0.55929, 0.55829, 0.55716, 0.55587, 0.55440, 0.55274, 0.55086, 0.54878,\
        0.54649, 0.54398, 0.54125, 0.53826, 0.53501, 0.53145, 0.52760, 0.52344, 0.51902, 0.51440, 0.50955,\
        0.50428, 0.49818, 0.49157, 0.48488, 0.47815, 0.47134, 0.46442, 0.45735, 0.45011, 0.44267, 0.43501],\
        [0.49188, 0.49281, 0.49316, 0.49315, 0.49299, 0.49280, 0.49271, 0.49278, 0.49305, 0.49355, 0.49430,\
        0.49530, 0.49657, 0.49805, 0.49977, 0.50181, 0.50452, 0.50807, 0.51223, 0.51678, 0.52158, 0.52656,\
        0.53169, 0.53694, 0.54230, 0.54777, 0.55335, 0.55902, 0.56476, 0.57052, 0.57624, 0.58184, 0.58721,\
        0.59217, 0.59644, 0.59935, 0.60062, 0.60090, 0.60067, 0.60008, 0.59920, 0.59809, 0.59654, 0.59425,\
        0.59189, 0.58982, 0.58808, 0.58667, 0.58567, 0.58534, 0.58589, 0.58738, 0.58963, 0.59231, 0.59513,\
        0.59742, 0.59856, 0.59976, 0.60111, 0.60203, 0.60273, 0.60333, 0.60394, 0.60457, 0.60545, 0.60688,\
        0.60781, 0.60813, 0.60805, 0.60760, 0.60664, 0.60515, 0.60313, 0.60042, 0.59722, 0.59384, 0.59040,\
        0.58675, 0.58292, 0.57910, 0.57546, 0.57213, 0.56914, 0.56654, 0.56429, 0.56238, 0.56074, 0.55934,\
        0.55810, 0.55696, 0.55587, 0.55477, 0.55360, 0.55234, 0.55095, 0.54940, 0.54769, 0.54582, 0.54377,\
        0.54155, 0.53915, 0.53656, 0.53375, 0.53068, 0.52730, 0.52359, 0.51955, 0.51522, 0.51072, 0.50616,\
        0.50127, 0.49528, 0.48852, 0.48157, 0.47454, 0.46744, 0.46024, 0.45291, 0.44542, 0.43774, 0.42986],\
        [0.48863, 0.49001, 0.49085, 0.49132, 0.49159, 0.49179, 0.49202, 0.49235, 0.49282, 0.49344, 0.49422,\
        0.49514, 0.49617, 0.49730, 0.49853, 0.50001, 0.50243, 0.50597, 0.51013, 0.51466, 0.51944, 0.52442,\
        0.52955, 0.53481, 0.54018, 0.54566, 0.55122, 0.55686, 0.56253, 0.56820, 0.57378, 0.57920, 0.58432,\
        0.58895, 0.59279, 0.59543, 0.59675, 0.59707, 0.59681, 0.59616, 0.59521, 0.59398, 0.59236, 0.59032,\
        0.58815, 0.58613, 0.58440, 0.58303, 0.58214, 0.58187, 0.58233, 0.58354, 0.58538, 0.58759, 0.58986,\
        0.59172, 0.59303, 0.59423, 0.59544, 0.59640, 0.59720, 0.59795, 0.59879, 0.59968, 0.60060, 0.60270,\
        0.60326, 0.60326, 0.60306, 0.60263, 0.60168, 0.60017, 0.59824, 0.59566, 0.59261, 0.58936, 0.58599,\
        0.58249, 0.57891, 0.57534, 0.57190, 0.56869, 0.56577, 0.56315, 0.56085, 0.55882, 0.55704, 0.55547,\
        0.55405, 0.55272, 0.55144, 0.55016, 0.54884, 0.54743, 0.54592, 0.54429, 0.54252, 0.54061, 0.53856,\
        0.53639, 0.53409, 0.53164, 0.52901, 0.52612, 0.52292, 0.51933, 0.51535, 0.51098, 0.50635, 0.50169,\
        0.49690, 0.49109, 0.48451, 0.47754, 0.47039, 0.46310, 0.45569, 0.44814, 0.44044, 0.43255, 0.42446],\
        [0.48516, 0.48697, 0.48829, 0.48926, 0.49000, 0.49063, 0.49125, 0.49191, 0.49266, 0.49350, 0.49442,\
        0.49538, 0.49633, 0.49727, 0.49826, 0.49939, 0.50149, 0.50486, 0.50883, 0.51318, 0.51782, 0.52268,\
        0.52771, 0.53288, 0.53817, 0.54357, 0.54904, 0.55456, 0.56009, 0.56558, 0.57094, 0.57608, 0.58087,\
        0.58511, 0.58856, 0.59099, 0.59235, 0.59281, 0.59264, 0.59203, 0.59107, 0.58980, 0.58822, 0.58636,\
        0.58440, 0.58253, 0.58090, 0.57963, 0.57881, 0.57854, 0.57890, 0.57985, 0.58131, 0.58307, 0.58489,\
        0.58650, 0.58784, 0.58905, 0.59019, 0.59121, 0.59217, 0.59314, 0.59425, 0.59555, 0.59732, 0.59880,\
        0.59880, 0.59849, 0.59813, 0.59761, 0.59662, 0.59510, 0.59315, 0.59071, 0.58786, 0.58477, 0.58153,\
        0.57819, 0.57479, 0.57142, 0.56815, 0.56506, 0.56219, 0.55957, 0.55721, 0.55509, 0.55319, 0.55146,\
        0.54987, 0.54837, 0.54691, 0.54546, 0.54398, 0.54243, 0.54079, 0.53905, 0.53720, 0.53523, 0.53317,\
        0.53101, 0.52878, 0.52646, 0.52400, 0.52131, 0.51828, 0.51482, 0.51090, 0.50646, 0.50159, 0.49674,\
        0.49175, 0.48607, 0.47969, 0.47282, 0.46565, 0.45827, 0.45072, 0.44300, 0.43512, 0.42705, 0.41878],\
        [0.48150, 0.48372, 0.48550, 0.48694, 0.48816, 0.48925, 0.49029, 0.49134, 0.49244, 0.49359, 0.49476,\
        0.49589, 0.49691, 0.49782, 0.49879, 0.50010, 0.50191, 0.50469, 0.50823, 0.51225, 0.51661, 0.52125,\
        0.52609, 0.53109, 0.53623, 0.54146, 0.54677, 0.55211, 0.55743, 0.56267, 0.56775, 0.57258, 0.57700,\
        0.58087, 0.58401, 0.58628, 0.58766, 0.58824, 0.58819, 0.58767, 0.58677, 0.58555, 0.58407, 0.58238,\
        0.58062, 0.57894, 0.57745, 0.57629, 0.57553, 0.57525, 0.57549, 0.57621, 0.57735, 0.57874, 0.58022,\
        0.58164, 0.58292, 0.58411, 0.58525, 0.58634, 0.58742, 0.58857, 0.58985, 0.59130, 0.59282, 0.59386,\
        0.59400, 0.59368, 0.59318, 0.59247, 0.59140, 0.58989, 0.58797, 0.58564, 0.58296, 0.58004, 0.57695,\
        0.57377, 0.57054, 0.56733, 0.56420, 0.56120, 0.55839, 0.55578, 0.55338, 0.55118, 0.54916, 0.54730,\
        0.54555, 0.54389, 0.54227, 0.54065, 0.53901, 0.53732, 0.53555, 0.53369, 0.53174, 0.52969, 0.52757,\
        0.52539, 0.52318, 0.52097, 0.51871, 0.51624, 0.51337, 0.51000, 0.50618, 0.50182, 0.49683, 0.49172,\
        0.48641, 0.48065, 0.47432, 0.46752, 0.46036, 0.45294, 0.44530, 0.43746, 0.42943, 0.42121, 0.41279],\
        [0.47760, 0.48023, 0.48245, 0.48435, 0.48603, 0.48758, 0.48906, 0.49053, 0.49202, 0.49354, 0.49506,\
        0.49649, 0.49772, 0.49866, 0.49957, 0.50077, 0.50251, 0.50493, 0.50802, 0.51164, 0.51567, 0.52001,\
        0.52459, 0.52936, 0.53428, 0.53930, 0.54438, 0.54948, 0.55455, 0.55950, 0.56428, 0.56876, 0.57284,\
        0.57639, 0.57927, 0.58141, 0.58278, 0.58344, 0.58352, 0.58311, 0.58231, 0.58121, 0.57987, 0.57836,\
        0.57680, 0.57530, 0.57397, 0.57291, 0.57221, 0.57191, 0.57204, 0.57258, 0.57346, 0.57456, 0.57579,\
        0.57702, 0.57822, 0.57937, 0.58050, 0.58163, 0.58277, 0.58398, 0.58527, 0.58661, 0.58783, 0.58862,\
        0.58883, 0.58857, 0.58802, 0.58721, 0.58607, 0.58456, 0.58268, 0.58045, 0.57792, 0.57516, 0.57223,\
        0.56919, 0.56612, 0.56305, 0.56003, 0.55713, 0.55436, 0.55176, 0.54933, 0.54706, 0.54495, 0.54296,\
        0.54108, 0.53926, 0.53749, 0.53572, 0.53393, 0.53209, 0.53019, 0.52821, 0.52614, 0.52400, 0.52178,\
        0.51953, 0.51730, 0.51514, 0.51307, 0.51089, 0.50818, 0.50477, 0.50096, 0.49663, 0.49166, 0.48636,\
        0.48081, 0.47489, 0.46853, 0.46174, 0.45459, 0.44713, 0.43943, 0.43150, 0.42337, 0.41502, 0.40646],\
        [0.47347, 0.47647, 0.47911, 0.48145, 0.48358, 0.48556, 0.48747, 0.48935, 0.49125, 0.49315, 0.49506,\
        0.49690, 0.49851, 0.49955, 0.50032, 0.50145, 0.50302, 0.50516, 0.50789, 0.51114, 0.51481, 0.51883,\
        0.52312, 0.52761, 0.53226, 0.53703, 0.54185, 0.54667, 0.55145, 0.55610, 0.56055, 0.56471, 0.56847,\
        0.57173, 0.57440, 0.57642, 0.57777, 0.57850, 0.57867, 0.57839, 0.57773, 0.57677, 0.57559, 0.57426,\
        0.57289, 0.57156, 0.57039, 0.56944, 0.56879, 0.56848, 0.56852, 0.56891, 0.56958, 0.57047, 0.57148,\
        0.57256, 0.57365, 0.57475, 0.57585, 0.57696, 0.57810, 0.57928, 0.58048, 0.58164, 0.58262, 0.58325,\
        0.58343, 0.58321, 0.58265, 0.58179, 0.58062, 0.57912, 0.57729, 0.57516, 0.57275, 0.57013, 0.56734,\
        0.56445, 0.56151, 0.55857, 0.55566, 0.55283, 0.55011, 0.54751, 0.54505, 0.54273, 0.54053, 0.53844,\
        0.53643, 0.53448, 0.53256, 0.53065, 0.52872, 0.52675, 0.52471, 0.52261, 0.52043, 0.51817, 0.51585,\
        0.51349, 0.51115, 0.50893, 0.50701, 0.50520, 0.50278, 0.49914, 0.49519, 0.49079, 0.48585, 0.48050,\
        0.47481, 0.46877, 0.46234, 0.45553, 0.44836, 0.44089, 0.43314, 0.42514, 0.41692, 0.40847, 0.39980],\
        [0.46907, 0.47243, 0.47546, 0.47821, 0.48075, 0.48315, 0.48546, 0.48772, 0.48998, 0.49225, 0.49451,\
        0.49671, 0.49874, 0.50017, 0.50082, 0.50180, 0.50321, 0.50514, 0.50758, 0.51051, 0.51386, 0.51757,\
        0.52155, 0.52575, 0.53012, 0.53460, 0.53914, 0.54367, 0.54815, 0.55249, 0.55662, 0.56047, 0.56394,\
        0.56695, 0.56943, 0.57134, 0.57266, 0.57343, 0.57370, 0.57354, 0.57302, 0.57222, 0.57120, 0.57006,\
        0.56887, 0.56771, 0.56667, 0.56583, 0.56523, 0.56491, 0.56490, 0.56516, 0.56567, 0.56639, 0.56723,\
        0.56817, 0.56916, 0.57017, 0.57121, 0.57227, 0.57335, 0.57444, 0.57551, 0.57649, 0.57727, 0.57776,\
        0.57789, 0.57766, 0.57709, 0.57622, 0.57505, 0.57357, 0.57179, 0.56974, 0.56744, 0.56495, 0.56229,\
        0.55953, 0.55672, 0.55388, 0.55106, 0.54830, 0.54562, 0.54303, 0.54055, 0.53818, 0.53591, 0.53372,\
        0.53160, 0.52953, 0.52748, 0.52544, 0.52337, 0.52127, 0.51912, 0.51689, 0.51460, 0.51223, 0.50980,\
        0.50733, 0.50486, 0.50248, 0.50054, 0.49888, 0.49670, 0.49312, 0.48900, 0.48448, 0.47951, 0.47413,\
        0.46837, 0.46225, 0.45576, 0.44892, 0.44173, 0.43422, 0.42643, 0.41838, 0.41009, 0.40155, 0.39278]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=60 (m/s) and translational velocity of Vt=5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax60vt5
    zgrid_H_Hmax_vmax60vt5=[[0.32692, 0.33367, 0.34018, 0.34644, 0.35247, 0.35828, 0.36387, 0.36927, 0.37449, 0.37953, 0.38442,\
        0.38916, 0.39377, 0.39827, 0.40268, 0.40700, 0.41126, 0.41547, 0.41965, 0.42381, 0.42798, 0.43217,\
        0.43639, 0.44067, 0.44502, 0.44945, 0.45397, 0.45861, 0.46338, 0.46829, 0.47335, 0.47858, 0.48400,\
        0.48959, 0.49531, 0.50076, 0.50555, 0.51060, 0.51608, 0.52198, 0.52821, 0.53464, 0.54114, 0.54757,\
        0.55381, 0.55976, 0.56536, 0.57058, 0.57544, 0.57996, 0.58415, 0.58804, 0.59150, 0.59445, 0.59668,\
        0.59888, 0.60173, 0.60474, 0.60774, 0.61064, 0.61340, 0.61598, 0.61835, 0.62050, 0.62242, 0.62411,\
        0.62555, 0.62675, 0.62769, 0.62837, 0.62880, 0.62896, 0.62887, 0.62853, 0.62794, 0.62713, 0.62610,\
        0.62485, 0.62341, 0.62176, 0.61992, 0.61786, 0.61559, 0.61309, 0.61036, 0.60738, 0.60417, 0.60074,\
        0.59712, 0.59336, 0.58951, 0.58563, 0.58180, 0.57806, 0.57437, 0.57065, 0.56689, 0.56310, 0.55926,\
        0.55533, 0.55128, 0.54711, 0.54280, 0.53838, 0.53384, 0.52919, 0.52443, 0.51957, 0.51462, 0.50958,\
        0.50448, 0.49936, 0.49440, 0.48942, 0.48405, 0.47831, 0.47205, 0.46510, 0.45748, 0.44926, 0.44048],\
        [0.33533, 0.34190, 0.34821, 0.35427, 0.36010, 0.36569, 0.37108, 0.37626, 0.38125, 0.38607, 0.39073,\
        0.39524, 0.39961, 0.40388, 0.40804, 0.41212, 0.41613, 0.42009, 0.42402, 0.42794, 0.43186, 0.43580,\
        0.43978, 0.44381, 0.44790, 0.45208, 0.45636, 0.46074, 0.46524, 0.46987, 0.47463, 0.47955, 0.48463,\
        0.48985, 0.49514, 0.50023, 0.50548, 0.51107, 0.51714, 0.52368, 0.53058, 0.53770, 0.54486, 0.55192,\
        0.55872, 0.56515, 0.57113, 0.57665, 0.58177, 0.58651, 0.59089, 0.59508, 0.59881, 0.60196, 0.60446,\
        0.60680, 0.60969, 0.61276, 0.61584, 0.61881, 0.62163, 0.62426, 0.62667, 0.62886, 0.63081, 0.63252,\
        0.63399, 0.63520, 0.63615, 0.63684, 0.63726, 0.63741, 0.63730, 0.63693, 0.63632, 0.63548, 0.63443,\
        0.63318, 0.63173, 0.63010, 0.62827, 0.62625, 0.62402, 0.62155, 0.61883, 0.61586, 0.61263, 0.60915,\
        0.60545, 0.60159, 0.59761, 0.59356, 0.58963, 0.58590, 0.58221, 0.57843, 0.57463, 0.57086, 0.56705,\
        0.56315, 0.55912, 0.55495, 0.55066, 0.54627, 0.54177, 0.53717, 0.53249, 0.52773, 0.52289, 0.51798,\
        0.51304, 0.50811, 0.50340, 0.49849, 0.49325, 0.48779, 0.48176, 0.47486, 0.46724, 0.45907, 0.45038],\
        [0.34353, 0.34991, 0.35602, 0.36188, 0.36750, 0.37289, 0.37806, 0.38302, 0.38779, 0.39238, 0.39680,\
        0.40108, 0.40521, 0.40923, 0.41315, 0.41698, 0.42074, 0.42445, 0.42813, 0.43179, 0.43545, 0.43914,\
        0.44286, 0.44663, 0.45047, 0.45439, 0.45840, 0.46251, 0.46672, 0.47104, 0.47547, 0.48001, 0.48467,\
        0.48942, 0.49424, 0.49921, 0.50475, 0.51088, 0.51762, 0.52491, 0.53259, 0.54047, 0.54837, 0.55612,\
        0.56354, 0.57048, 0.57684, 0.58262, 0.58798, 0.59297, 0.59743, 0.60177, 0.60580, 0.60919, 0.61211,\
        0.61487, 0.61782, 0.62091, 0.62401, 0.62702, 0.62988, 0.63253, 0.63496, 0.63716, 0.63912, 0.64084,\
        0.64231, 0.64353, 0.64448, 0.64517, 0.64558, 0.64571, 0.64557, 0.64516, 0.64451, 0.64364, 0.64255,\
        0.64128, 0.63983, 0.63821, 0.63642, 0.63445, 0.63227, 0.62986, 0.62719, 0.62424, 0.62100, 0.61749,\
        0.61373, 0.60976, 0.60564, 0.60143, 0.59735, 0.59365, 0.58993, 0.58597, 0.58208, 0.57833, 0.57458,\
        0.57070, 0.56665, 0.56247, 0.55818, 0.55380, 0.54933, 0.54479, 0.54019, 0.53553, 0.53081, 0.52606,\
        0.52130, 0.51658, 0.51191, 0.50707, 0.50202, 0.49686, 0.49106, 0.48403, 0.47637, 0.46831, 0.45978],\
        [0.35151, 0.35770, 0.36362, 0.36928, 0.37470, 0.37987, 0.38483, 0.38957, 0.39411, 0.39847, 0.40266,\
        0.40669, 0.41059, 0.41436, 0.41802, 0.42160, 0.42511, 0.42856, 0.43198, 0.43538, 0.43878, 0.44221,\
        0.44566, 0.44918, 0.45275, 0.45641, 0.46016, 0.46400, 0.46793, 0.47195, 0.47605, 0.48022, 0.48446,\
        0.48881, 0.49335, 0.49829, 0.50392, 0.51040, 0.51781, 0.52592, 0.53443, 0.54311, 0.55176, 0.56022,\
        0.56830, 0.57581, 0.58256, 0.58853, 0.59401, 0.59936, 0.60404, 0.60846, 0.61261, 0.61630, 0.61961,\
        0.62274, 0.62587, 0.62904, 0.63219, 0.63523, 0.63811, 0.64077, 0.64320, 0.64539, 0.64735, 0.64906,\
        0.65053, 0.65174, 0.65269, 0.65337, 0.65376, 0.65385, 0.65367, 0.65321, 0.65251, 0.65158, 0.65045,\
        0.64915, 0.64769, 0.64609, 0.64434, 0.64243, 0.64033, 0.63801, 0.63541, 0.63251, 0.62930, 0.62577,\
        0.62196, 0.61790, 0.61367, 0.60936, 0.60520, 0.60139, 0.59749, 0.59314, 0.58918, 0.58556, 0.58187,\
        0.57796, 0.57386, 0.56963, 0.56532, 0.56094, 0.55650, 0.55202, 0.54749, 0.54293, 0.53834, 0.53373,\
        0.52913, 0.52454, 0.51993, 0.51522, 0.51036, 0.50533, 0.49957, 0.49221, 0.48473, 0.47698, 0.46869],\
        [0.35929, 0.36529, 0.37102, 0.37648, 0.38169, 0.38665, 0.39139, 0.39591, 0.40022, 0.40435, 0.40830,\
        0.41210, 0.41575, 0.41927, 0.42269, 0.42601, 0.42926, 0.43245, 0.43560, 0.43874, 0.44188, 0.44504,\
        0.44824, 0.45149, 0.45481, 0.45821, 0.46170, 0.46529, 0.46897, 0.47272, 0.47654, 0.48041, 0.48432,\
        0.48832, 0.49256, 0.49743, 0.50294, 0.50981, 0.51804, 0.52699, 0.53631, 0.54574, 0.55511, 0.56426,\
        0.57301, 0.58116, 0.58841, 0.59451, 0.59994, 0.60540, 0.61046, 0.61508, 0.61937, 0.62331, 0.62695,\
        0.63039, 0.63374, 0.63705, 0.64028, 0.64338, 0.64628, 0.64895, 0.65136, 0.65353, 0.65546, 0.65716,\
        0.65862, 0.65983, 0.66077, 0.66143, 0.66179, 0.66184, 0.66160, 0.66107, 0.66029, 0.65929, 0.65810,\
        0.65675, 0.65528, 0.65370, 0.65200, 0.65018, 0.64819, 0.64598, 0.64349, 0.64067, 0.63750, 0.63397,\
        0.63011, 0.62599, 0.62167, 0.61728, 0.61298, 0.60882, 0.60448, 0.59988, 0.59602, 0.59264, 0.58894,\
        0.58489, 0.58068, 0.57638, 0.57203, 0.56765, 0.56325, 0.55882, 0.55436, 0.54989, 0.54541, 0.54093,\
        0.53645, 0.53196, 0.52742, 0.52277, 0.51791, 0.51266, 0.50650, 0.49949, 0.49252, 0.48519, 0.47717],\
        [0.36688, 0.37269, 0.37822, 0.38348, 0.38848, 0.39324, 0.39776, 0.40205, 0.40614, 0.41004, 0.41375,\
        0.41731, 0.42071, 0.42399, 0.42715, 0.43021, 0.43320, 0.43613, 0.43903, 0.44190, 0.44477, 0.44767,\
        0.45060, 0.45360, 0.45667, 0.45983, 0.46309, 0.46645, 0.46992, 0.47348, 0.47712, 0.48080, 0.48450,\
        0.48826, 0.49214, 0.49628, 0.50164, 0.50967, 0.51879, 0.52848, 0.53845, 0.54850, 0.55846, 0.56819,\
        0.57754, 0.58634, 0.59427, 0.60075, 0.60622, 0.61160, 0.61675, 0.62155, 0.62602, 0.63019, 0.63409,\
        0.63780, 0.64139, 0.64487, 0.64823, 0.65141, 0.65436, 0.65703, 0.65942, 0.66155, 0.66345, 0.66512,\
        0.66657, 0.66777, 0.66871, 0.66935, 0.66967, 0.66967, 0.66935, 0.66874, 0.66786, 0.66676, 0.66548,\
        0.66408, 0.66257, 0.66100, 0.65937, 0.65766, 0.65582, 0.65376, 0.65142, 0.64871, 0.64559, 0.64207,\
        0.63817, 0.63396, 0.62954, 0.62501, 0.62047, 0.61595, 0.61135, 0.60683, 0.60299, 0.59957, 0.59563,\
        0.59137, 0.58702, 0.58264, 0.57826, 0.57390, 0.56953, 0.56516, 0.56076, 0.55637, 0.55198, 0.54760,\
        0.54322, 0.53882, 0.53433, 0.52970, 0.52478, 0.51941, 0.51337, 0.50683, 0.50011, 0.49314, 0.48526],\
        [0.37427, 0.37989, 0.38523, 0.39030, 0.39509, 0.39964, 0.40394, 0.40802, 0.41188, 0.41554, 0.41902,\
        0.42234, 0.42550, 0.42852, 0.43143, 0.43424, 0.43697, 0.43964, 0.44227, 0.44488, 0.44749, 0.45012,\
        0.45280, 0.45554, 0.45837, 0.46130, 0.46435, 0.46753, 0.47085, 0.47429, 0.47785, 0.48150, 0.48521,\
        0.48898, 0.49291, 0.49734, 0.50314, 0.51119, 0.52058, 0.53063, 0.54100, 0.55146, 0.56183, 0.57196,\
        0.58173, 0.59095, 0.59936, 0.60631, 0.61214, 0.61762, 0.62285, 0.62781, 0.63247, 0.63686, 0.64100,\
        0.64496, 0.64877, 0.65246, 0.65599, 0.65930, 0.66232, 0.66501, 0.66736, 0.66943, 0.67127, 0.67293,\
        0.67438, 0.67557, 0.67649, 0.67711, 0.67741, 0.67733, 0.67692, 0.67619, 0.67519, 0.67397, 0.67259,\
        0.67109, 0.66954, 0.66797, 0.66641, 0.66483, 0.66318, 0.66133, 0.65917, 0.65659, 0.65355, 0.65003,\
        0.64607, 0.64175, 0.63718, 0.63246, 0.62769, 0.62292, 0.61820, 0.61370, 0.60962, 0.60577, 0.60161,\
        0.59724, 0.59282, 0.58837, 0.58397, 0.57965, 0.57534, 0.57101, 0.56667, 0.56233, 0.55802, 0.55373,\
        0.54944, 0.54511, 0.54068, 0.53606, 0.53116, 0.52585, 0.52007, 0.51390, 0.50749, 0.50062, 0.49284],\
        [0.38148, 0.38692, 0.39206, 0.39693, 0.40152, 0.40586, 0.40995, 0.41380, 0.41744, 0.42088, 0.42412,\
        0.42720, 0.43011, 0.43289, 0.43555, 0.43810, 0.44058, 0.44299, 0.44535, 0.44770, 0.45005, 0.45243,\
        0.45485, 0.45735, 0.45995, 0.46266, 0.46552, 0.46855, 0.47176, 0.47516, 0.47875, 0.48253, 0.48645,\
        0.49053, 0.49487, 0.49991, 0.50620, 0.51410, 0.52336, 0.53345, 0.54398, 0.55465, 0.56524, 0.57558,\
        0.58552, 0.59490, 0.60344, 0.61086, 0.61729, 0.62313, 0.62861, 0.63377, 0.63865, 0.64326, 0.64764,\
        0.65183, 0.65586, 0.65976, 0.66350, 0.66701, 0.67015, 0.67286, 0.67516, 0.67714, 0.67892, 0.68057,\
        0.68204, 0.68321, 0.68410, 0.68472, 0.68498, 0.68481, 0.68428, 0.68342, 0.68227, 0.68090, 0.67939,\
        0.67778, 0.67614, 0.67455, 0.67306, 0.67164, 0.67022, 0.66864, 0.66672, 0.66431, 0.66135, 0.65781,\
        0.65376, 0.64930, 0.64454, 0.63960, 0.63460, 0.62960, 0.62472, 0.62004, 0.61563, 0.61135, 0.60699,\
        0.60253, 0.59805, 0.59350, 0.58914, 0.58491, 0.58067, 0.57637, 0.57205, 0.56776, 0.56351, 0.55931,\
        0.55510, 0.55085, 0.54649, 0.54193, 0.53710, 0.53190, 0.52633, 0.52039, 0.51408, 0.50726, 0.49952],\
        [0.38851, 0.39376, 0.39872, 0.40339, 0.40778, 0.41191, 0.41579, 0.41943, 0.42284, 0.42605, 0.42906,\
        0.43190, 0.43458, 0.43711, 0.43952, 0.44182, 0.44404, 0.44619, 0.44830, 0.45039, 0.45249, 0.45461,\
        0.45678, 0.45904, 0.46141, 0.46392, 0.46661, 0.46950, 0.47263, 0.47602, 0.47972, 0.48373, 0.48805,\
        0.49262, 0.49745, 0.50294, 0.50956, 0.51750, 0.52666, 0.53670, 0.54726, 0.55802, 0.56871, 0.57910,\
        0.58906, 0.59848, 0.60717, 0.61495, 0.62187, 0.62815, 0.63395, 0.63939, 0.64452, 0.64936, 0.65397,\
        0.65837, 0.66261, 0.66673, 0.67073, 0.67449, 0.67784, 0.68059, 0.68281, 0.68467, 0.68635, 0.68802,\
        0.68959, 0.69071, 0.69150, 0.69216, 0.69240, 0.69207, 0.69140, 0.69042, 0.68907, 0.68753, 0.68588,\
        0.68413, 0.68236, 0.68071, 0.67926, 0.67803, 0.67692, 0.67568, 0.67405, 0.67184, 0.66894, 0.66536,\
        0.66117, 0.65652, 0.65153, 0.64635, 0.64110, 0.63587, 0.63076, 0.62583, 0.62108, 0.61646, 0.61186,\
        0.60725, 0.60261, 0.59800, 0.59381, 0.58974, 0.58553, 0.58119, 0.57687, 0.57261, 0.56844, 0.56433,\
        0.56023, 0.55608, 0.55180, 0.54732, 0.54257, 0.53749, 0.53204, 0.52619, 0.51987, 0.51292, 0.50509],\
        [0.39536, 0.40044, 0.40521, 0.40968, 0.41388, 0.41780, 0.42147, 0.42489, 0.42809, 0.43107, 0.43386,\
        0.43646, 0.43890, 0.44119, 0.44336, 0.44542, 0.44738, 0.44928, 0.45114, 0.45297, 0.45481, 0.45669,\
        0.45862, 0.46064, 0.46279, 0.46510, 0.46761, 0.47036, 0.47341, 0.47681, 0.48060, 0.48486, 0.48962,\
        0.49484, 0.50021, 0.50599, 0.51281, 0.52086, 0.53004, 0.54009, 0.55069, 0.56151, 0.57226, 0.58266,\
        0.59252, 0.60192, 0.61076, 0.61882, 0.62611, 0.63276, 0.63891, 0.64465, 0.65006, 0.65516, 0.65999,\
        0.66458, 0.66899, 0.67330, 0.67756, 0.68168, 0.68536, 0.68818, 0.69030, 0.69206, 0.69358, 0.69526,\
        0.69710, 0.69808, 0.69869, 0.69938, 0.69963, 0.69903, 0.69827, 0.69719, 0.69553, 0.69387, 0.69210,\
        0.69014, 0.68819, 0.68641, 0.68496, 0.68394, 0.68321, 0.68240, 0.68113, 0.67913, 0.67627, 0.67260,\
        0.66823, 0.66333, 0.65809, 0.65264, 0.64712, 0.64164, 0.63627, 0.63106, 0.62602, 0.62110, 0.61626,\
        0.61148, 0.60675, 0.60220, 0.59819, 0.59420, 0.58987, 0.58543, 0.58108, 0.57688, 0.57281, 0.56883,\
        0.56485, 0.56082, 0.55664, 0.55226, 0.54759, 0.54259, 0.53721, 0.53138, 0.52503, 0.51804, 0.51029],\
        [0.40205, 0.40695, 0.41153, 0.41581, 0.41981, 0.42353, 0.42699, 0.43021, 0.43319, 0.43595, 0.43851,\
        0.44089, 0.44310, 0.44515, 0.44708, 0.44890, 0.45062, 0.45227, 0.45388, 0.45547, 0.45706, 0.45869,\
        0.46038, 0.46217, 0.46410, 0.46620, 0.46853, 0.47114, 0.47408, 0.47742, 0.48125, 0.48563, 0.49066,\
        0.49635, 0.50228, 0.50838, 0.51553, 0.52384, 0.53319, 0.54336, 0.55406, 0.56500, 0.57587, 0.58639,\
        0.59622, 0.60552, 0.61440, 0.62260, 0.63012, 0.63707, 0.64353, 0.64958, 0.65528, 0.66066, 0.66571,\
        0.67048, 0.67501, 0.67943, 0.68387, 0.68839, 0.69267, 0.69559, 0.69758, 0.69945, 0.70087, 0.70247,\
        0.70432, 0.70538, 0.70596, 0.70642, 0.70644, 0.70581, 0.70486, 0.70355, 0.70166, 0.69999, 0.69805,\
        0.69583, 0.69367, 0.69166, 0.69012, 0.68931, 0.68907, 0.68880, 0.68792, 0.68611, 0.68325, 0.67943,\
        0.67482, 0.66964, 0.66411, 0.65839, 0.65261, 0.64687, 0.64125, 0.63577, 0.63046, 0.62529, 0.62025,\
        0.61534, 0.61061, 0.60619, 0.60219, 0.59812, 0.59350, 0.58894, 0.58463, 0.58056, 0.57665, 0.57283,\
        0.56901, 0.56511, 0.56105, 0.55676, 0.55218, 0.54724, 0.54189, 0.53608, 0.52972, 0.52274, 0.51509],\
        [0.40857, 0.41329, 0.41769, 0.42179, 0.42559, 0.42911, 0.43237, 0.43538, 0.43815, 0.44069, 0.44304,\
        0.44519, 0.44718, 0.44900, 0.45070, 0.45228, 0.45377, 0.45518, 0.45655, 0.45789, 0.45924, 0.46063,\
        0.46208, 0.46364, 0.46535, 0.46725, 0.46939, 0.47182, 0.47462, 0.47785, 0.48160, 0.48594, 0.49094,\
        0.49660, 0.50285, 0.50974, 0.51751, 0.52628, 0.53596, 0.54636, 0.55724, 0.56832, 0.57936, 0.59014,\
        0.60026, 0.60940, 0.61808, 0.62626, 0.63393, 0.64109, 0.64783, 0.65418, 0.66020, 0.66588, 0.67120,\
        0.67615, 0.68075, 0.68517, 0.68962, 0.69430, 0.69908, 0.70259, 0.70473, 0.70676, 0.70846, 0.71007,\
        0.71160, 0.71265, 0.71321, 0.71340, 0.71314, 0.71239, 0.71122, 0.70967, 0.70779, 0.70584, 0.70364,\
        0.70125, 0.69891, 0.69655, 0.69475, 0.69410, 0.69449, 0.69480, 0.69429, 0.69264, 0.68972, 0.68569,\
        0.68081, 0.67533, 0.66952, 0.66354, 0.65752, 0.65155, 0.64569, 0.63998, 0.63443, 0.62905, 0.62383,\
        0.61879, 0.61396, 0.60943, 0.60514, 0.60076, 0.59610, 0.59167, 0.58755, 0.58369, 0.58001, 0.57640,\
        0.57276, 0.56902, 0.56508, 0.56089, 0.55638, 0.55149, 0.54617, 0.54037, 0.53402, 0.52709, 0.51952],\
        [0.41492, 0.41947, 0.42369, 0.42761, 0.43122, 0.43455, 0.43761, 0.44041, 0.44297, 0.44531, 0.44744,\
        0.44938, 0.45115, 0.45275, 0.45423, 0.45558, 0.45684, 0.45802, 0.45915, 0.46026, 0.46138, 0.46253,\
        0.46376, 0.46509, 0.46658, 0.46827, 0.47020, 0.47245, 0.47507, 0.47813, 0.48172, 0.48592, 0.49078,\
        0.49639, 0.50292, 0.51045, 0.51888, 0.52819, 0.53829, 0.54901, 0.56012, 0.57136, 0.58248, 0.59326,\
        0.60348, 0.61279, 0.62148, 0.62969, 0.63746, 0.64481, 0.65180, 0.65846, 0.66481, 0.67085, 0.67651,\
        0.68170, 0.68639, 0.69074, 0.69509, 0.69979, 0.70466, 0.70876, 0.71173, 0.71411, 0.71608, 0.71775,\
        0.71910, 0.72000, 0.72041, 0.72036, 0.71984, 0.71886, 0.71748, 0.71573, 0.71371, 0.71151, 0.70912,\
        0.70658, 0.70397, 0.70136, 0.69924, 0.69844, 0.69956, 0.70023, 0.69992, 0.69843, 0.69537, 0.69120,\
        0.68602, 0.68028, 0.67424, 0.66806, 0.66185, 0.65568, 0.64962, 0.64372, 0.63798, 0.63241, 0.62702,\
        0.62181, 0.61680, 0.61198, 0.60728, 0.60256, 0.59800, 0.59381, 0.58998, 0.58642, 0.58300, 0.57962,\
        0.57618, 0.57260, 0.56878, 0.56468, 0.56023, 0.55538, 0.55009, 0.54430, 0.53798, 0.53109, 0.52360],\
        [0.42111, 0.42549, 0.42954, 0.43327, 0.43670, 0.43983, 0.44270, 0.44530, 0.44767, 0.44980, 0.45173,\
        0.45346, 0.45502, 0.45641, 0.45767, 0.45881, 0.45984, 0.46081, 0.46172, 0.46260, 0.46349, 0.46442,\
        0.46543, 0.46654, 0.46781, 0.46928, 0.47101, 0.47306, 0.47549, 0.47836, 0.48177, 0.48580, 0.49055,\
        0.49616, 0.50293, 0.51092, 0.51989, 0.52970, 0.54022, 0.55129, 0.56267, 0.57410, 0.58529, 0.59595,\
        0.60611, 0.61564, 0.62448, 0.63278, 0.64067, 0.64819, 0.65542, 0.66238, 0.66909, 0.67553, 0.68165,\
        0.68726, 0.69214, 0.69650, 0.70092, 0.70576, 0.71071, 0.71510, 0.71866, 0.72148, 0.72373, 0.72548,\
        0.72673, 0.72746, 0.72765, 0.72734, 0.72654, 0.72530, 0.72368, 0.72174, 0.71955, 0.71717, 0.71462,\
        0.71195, 0.70923, 0.70662, 0.70449, 0.70360, 0.70426, 0.70480, 0.70442, 0.70282, 0.69990, 0.69574,\
        0.69026, 0.68441, 0.67828, 0.67198, 0.66562, 0.65930, 0.65309, 0.64703, 0.64114, 0.63542, 0.62987,\
        0.62449, 0.61927, 0.61419, 0.60921, 0.60432, 0.59971, 0.59569, 0.59218, 0.58892, 0.58576, 0.58261,\
        0.57936, 0.57592, 0.57222, 0.56820, 0.56379, 0.55897, 0.55369, 0.54792, 0.54163, 0.53478, 0.52735],\
        [0.42714, 0.43135, 0.43523, 0.43878, 0.44202, 0.44498, 0.44765, 0.45006, 0.45223, 0.45417, 0.45590,\
        0.45744, 0.45879, 0.45999, 0.46104, 0.46197, 0.46280, 0.46355, 0.46425, 0.46492, 0.46560, 0.46631,\
        0.46710, 0.46801, 0.46907, 0.47034, 0.47187, 0.47371, 0.47595, 0.47864, 0.48188, 0.48577, 0.49044,\
        0.49610, 0.50305, 0.51134, 0.52071, 0.53092, 0.54181, 0.55320, 0.56487, 0.57656, 0.58796, 0.59868,\
        0.60879, 0.61825, 0.62712, 0.63550, 0.64350, 0.65120, 0.65866, 0.66592, 0.67298, 0.67984, 0.68647,\
        0.69276, 0.69810, 0.70262, 0.70736, 0.71232, 0.71730, 0.72188, 0.72579, 0.72897, 0.73146, 0.73329,\
        0.73447, 0.73501, 0.73496, 0.73435, 0.73325, 0.73174, 0.72987, 0.72773, 0.72538, 0.72286, 0.72023,\
        0.71754, 0.71487, 0.71241, 0.71047, 0.70945, 0.70926, 0.70909, 0.70821, 0.70625, 0.70311, 0.69854,\
        0.69345, 0.68784, 0.68174, 0.67536, 0.66890, 0.66248, 0.65616, 0.64999, 0.64399, 0.63815, 0.63247,\
        0.62695, 0.62158, 0.61633, 0.61120, 0.60623, 0.60159, 0.59775, 0.59444, 0.59137, 0.58841, 0.58545,\
        0.58237, 0.57906, 0.57545, 0.57148, 0.56710, 0.56229, 0.55702, 0.55126, 0.54498, 0.53817, 0.53081],\
        [0.43301, 0.43705, 0.44076, 0.44413, 0.44720, 0.44997, 0.45246, 0.45469, 0.45667, 0.45842, 0.45996,\
        0.46131, 0.46248, 0.46348, 0.46434, 0.46508, 0.46572, 0.46627, 0.46677, 0.46724, 0.46771, 0.46822,\
        0.46881, 0.46952, 0.47038, 0.47146, 0.47280, 0.47446, 0.47652, 0.47904, 0.48214, 0.48592, 0.49055,\
        0.49625, 0.50333, 0.51182, 0.52143, 0.53192, 0.54309, 0.55475, 0.56666, 0.57859, 0.59028, 0.60133,\
        0.61122, 0.62054, 0.62939, 0.63782, 0.64594, 0.65382, 0.66152, 0.66906, 0.67646, 0.68369, 0.69070,\
        0.69756, 0.70367, 0.70878, 0.71388, 0.71908, 0.72422, 0.72900, 0.73319, 0.73666, 0.73934, 0.74123,\
        0.74232, 0.74267, 0.74233, 0.74140, 0.73998, 0.73816, 0.73605, 0.73371, 0.73122, 0.72862, 0.72597,\
        0.72333, 0.72078, 0.71847, 0.71660, 0.71530, 0.71442, 0.71347, 0.71193, 0.70942, 0.70574, 0.70119,\
        0.69650, 0.69098, 0.68481, 0.67834, 0.67180, 0.66530, 0.65892, 0.65269, 0.64661, 0.64069, 0.63493,\
        0.62932, 0.62385, 0.61854, 0.61339, 0.60850, 0.60404, 0.60021, 0.59687, 0.59386, 0.59104, 0.58823,\
        0.58528, 0.58207, 0.57850, 0.57455, 0.57018, 0.56536, 0.56008, 0.55432, 0.54806, 0.54129, 0.53398],\
        [0.43871, 0.44259, 0.44613, 0.44933, 0.45222, 0.45481, 0.45712, 0.45917, 0.46097, 0.46255, 0.46391,\
        0.46508, 0.46607, 0.46690, 0.46758, 0.46814, 0.46860, 0.46897, 0.46928, 0.46956, 0.46985, 0.47017,\
        0.47056, 0.47108, 0.47177, 0.47266, 0.47383, 0.47534, 0.47724, 0.47962, 0.48260, 0.48630, 0.49091,\
        0.49666, 0.50381, 0.51237, 0.52208, 0.53272, 0.54406, 0.55589, 0.56798, 0.58004, 0.59175, 0.60278,\
        0.61292, 0.62232, 0.63121, 0.63974, 0.64800, 0.65607, 0.66401, 0.67186, 0.67961, 0.68720, 0.69448,\
        0.70149, 0.70830, 0.71441, 0.72018, 0.72582, 0.73127, 0.73634, 0.74083, 0.74456, 0.74742, 0.74934,\
        0.75032, 0.75043, 0.74977, 0.74847, 0.74669, 0.74456, 0.74218, 0.73966, 0.73706, 0.73443, 0.73180,\
        0.72924, 0.72681, 0.72460, 0.72270, 0.72112, 0.71969, 0.71809, 0.71595, 0.71300, 0.70919, 0.70477,\
        0.69990, 0.69401, 0.68763, 0.68104, 0.67443, 0.66789, 0.66148, 0.65522, 0.64911, 0.64315, 0.63733,\
        0.63167, 0.62616, 0.62084, 0.61574, 0.61095, 0.60660, 0.60276, 0.59938, 0.59638, 0.59365, 0.59097,\
        0.58812, 0.58496, 0.58141, 0.57743, 0.57303, 0.56819, 0.56289, 0.55713, 0.55088, 0.54414, 0.53688],\
        [0.44424, 0.44796, 0.45133, 0.45437, 0.45708, 0.45950, 0.46163, 0.46351, 0.46514, 0.46655, 0.46774,\
        0.46875, 0.46958, 0.47024, 0.47076, 0.47116, 0.47145, 0.47165, 0.47179, 0.47190, 0.47200, 0.47215,\
        0.47237, 0.47272, 0.47324, 0.47398, 0.47500, 0.47636, 0.47814, 0.48040, 0.48328, 0.48690, 0.49146,\
        0.49720, 0.50436, 0.51291, 0.52264, 0.53331, 0.54472, 0.55663, 0.56881, 0.58097, 0.59273, 0.60378,\
        0.61405, 0.62358, 0.63259, 0.64125, 0.64969, 0.65799, 0.66620, 0.67436, 0.68249, 0.69056, 0.69841,\
        0.70579, 0.71298, 0.71980, 0.72626, 0.73245, 0.73834, 0.74380, 0.74865, 0.75267, 0.75571, 0.75765,\
        0.75848, 0.75830, 0.75725, 0.75553, 0.75335, 0.75087, 0.74824, 0.74555, 0.74287, 0.74023, 0.73766,\
        0.73520, 0.73288, 0.73074, 0.72878, 0.72695, 0.72510, 0.72300, 0.72037, 0.71706, 0.71301, 0.70831,\
        0.70292, 0.69681, 0.69029, 0.68359, 0.67692, 0.67037, 0.66397, 0.65770, 0.65158, 0.64559, 0.63975,\
        0.63406, 0.62854, 0.62322, 0.61817, 0.61346, 0.60916, 0.60531, 0.60189, 0.59890, 0.59627, 0.59370,\
        0.59092, 0.58775, 0.58415, 0.58011, 0.57565, 0.57076, 0.56545, 0.55968, 0.55345, 0.54673, 0.53952],\
        [0.44960, 0.45316, 0.45637, 0.45923, 0.46177, 0.46402, 0.46598, 0.46769, 0.46916, 0.47041, 0.47146,\
        0.47231, 0.47299, 0.47351, 0.47388, 0.47413, 0.47427, 0.47432, 0.47430, 0.47425, 0.47420, 0.47418,\
        0.47424, 0.47443, 0.47479, 0.47540, 0.47629, 0.47755, 0.47922, 0.48140, 0.48418, 0.48771, 0.49217,\
        0.49783, 0.50493, 0.51342, 0.52308, 0.53368, 0.54503, 0.55693, 0.56914, 0.58141, 0.59338, 0.60456,\
        0.61477, 0.62437, 0.63354, 0.64241, 0.65110, 0.65967, 0.66818, 0.67668, 0.68518, 0.69368, 0.70206,\
        0.71010, 0.71780, 0.72520, 0.73227, 0.73901, 0.74540, 0.75132, 0.75659, 0.76096, 0.76420, 0.76615,\
        0.76679, 0.76624, 0.76472, 0.76252, 0.75989, 0.75706, 0.75418, 0.75134, 0.74861, 0.74599, 0.74351,\
        0.74116, 0.73894, 0.73685, 0.73485, 0.73286, 0.73072, 0.72824, 0.72523, 0.72156, 0.71715, 0.71202,\
        0.70619, 0.69979, 0.69301, 0.68616, 0.67945, 0.67291, 0.66652, 0.66027, 0.65413, 0.64813, 0.64225,\
        0.63653, 0.63099, 0.62567, 0.62063, 0.61595, 0.61167, 0.60783, 0.60443, 0.60147, 0.59887, 0.59639,\
        0.59363, 0.59039, 0.58667, 0.58254, 0.57800, 0.57307, 0.56773, 0.56196, 0.55575, 0.54907, 0.54191],\
        [0.45478, 0.45819, 0.46122, 0.46392, 0.46629, 0.46836, 0.47016, 0.47171, 0.47303, 0.47413, 0.47503,\
        0.47575, 0.47630, 0.47669, 0.47693, 0.47705, 0.47706, 0.47698, 0.47682, 0.47663, 0.47642, 0.47625,\
        0.47616, 0.47620, 0.47644, 0.47692, 0.47771, 0.47887, 0.48048, 0.48260, 0.48531, 0.48875, 0.49309,\
        0.49857, 0.50556, 0.51391, 0.52338, 0.53380, 0.54499, 0.55676, 0.56892, 0.58119, 0.59325, 0.60454,\
        0.61490, 0.62470, 0.63413, 0.64331, 0.65234, 0.66126, 0.67013, 0.67899, 0.68789, 0.69682, 0.70573,\
        0.71443, 0.72276, 0.73073, 0.73833, 0.74557, 0.75244, 0.75884, 0.76458, 0.76937, 0.77287, 0.77483,\
        0.77522, 0.77421, 0.77214, 0.76938, 0.76626, 0.76306, 0.75994, 0.75699, 0.75424, 0.75169, 0.74931,\
        0.74709, 0.74498, 0.74297, 0.74097, 0.73888, 0.73656, 0.73383, 0.73054, 0.72655, 0.72178, 0.71623,\
        0.70995, 0.70313, 0.69600, 0.68895, 0.68221, 0.67571, 0.66932, 0.66305, 0.65688, 0.65082, 0.64489,\
        0.63911, 0.63352, 0.62816, 0.62309, 0.61837, 0.61407, 0.61023, 0.60685, 0.60393, 0.60138, 0.59899,\
        0.59615, 0.59274, 0.58888, 0.58464, 0.58004, 0.57508, 0.56974, 0.56399, 0.55780, 0.55116, 0.54406],\
        [0.45978, 0.46302, 0.46589, 0.46841, 0.47061, 0.47252, 0.47416, 0.47555, 0.47673, 0.47769, 0.47847,\
        0.47907, 0.47950, 0.47978, 0.47992, 0.47993, 0.47983, 0.47963, 0.47935, 0.47903, 0.47868, 0.47837,\
        0.47814, 0.47804, 0.47814, 0.47851, 0.47921, 0.48032, 0.48188, 0.48397, 0.48666, 0.49003, 0.49426,\
        0.49962, 0.50635, 0.51440, 0.52355, 0.53366, 0.54457, 0.55613, 0.56812, 0.58029, 0.59229, 0.60373,\
        0.61445, 0.62462, 0.63446, 0.64407, 0.65355, 0.66295, 0.67227, 0.68154, 0.69089, 0.70041, 0.70990,\
        0.71919, 0.72808, 0.73652, 0.74454, 0.75218, 0.75946, 0.76631, 0.77255, 0.77783, 0.78165, 0.78362,\
        0.78369, 0.78212, 0.77940, 0.77602, 0.77238, 0.76881, 0.76550, 0.76249, 0.75977, 0.75731, 0.75506,\
        0.75297, 0.75100, 0.74909, 0.74714, 0.74505, 0.74266, 0.73981, 0.73634, 0.73211, 0.72704, 0.72112,\
        0.71441, 0.70707, 0.69946, 0.69220, 0.68552, 0.67900, 0.67254, 0.66617, 0.65991, 0.65375, 0.64772,\
        0.64184, 0.63614, 0.63068, 0.62552, 0.62071, 0.61632, 0.61239, 0.60896, 0.60602, 0.60348, 0.60109,\
        0.59813, 0.59459, 0.59063, 0.58633, 0.58172, 0.57678, 0.57146, 0.56575, 0.55960, 0.55302, 0.54597],\
        [0.46458, 0.46766, 0.47035, 0.47270, 0.47472, 0.47646, 0.47794, 0.47920, 0.48024, 0.48108, 0.48175,\
        0.48225, 0.48258, 0.48277, 0.48283, 0.48275, 0.48256, 0.48227, 0.48189, 0.48145, 0.48098, 0.48053,\
        0.48015, 0.47992, 0.47989, 0.48015, 0.48077, 0.48182, 0.48336, 0.48546, 0.48816, 0.49151, 0.49561,\
        0.50075, 0.50719, 0.51483, 0.52355, 0.53324, 0.54379, 0.55504, 0.56682, 0.57887, 0.59090, 0.60255,\
        0.61366, 0.62432, 0.63468, 0.64487, 0.65495, 0.66495, 0.67490, 0.68469, 0.69438, 0.70457, 0.71480,\
        0.72466, 0.73399, 0.74276, 0.75105, 0.75893, 0.76648, 0.77366, 0.78034, 0.78618, 0.79040, 0.79235,\
        0.79204, 0.78980, 0.78641, 0.78237, 0.77820, 0.77431, 0.77086, 0.76786, 0.76522, 0.76288, 0.76077,\
        0.75882, 0.75699, 0.75521, 0.75338, 0.75137, 0.74902, 0.74617, 0.74264, 0.73831, 0.73306, 0.72688,\
        0.71983, 0.71207, 0.70394, 0.69650, 0.68981, 0.68301, 0.67629, 0.66972, 0.66329, 0.65698, 0.65079,\
        0.64474, 0.63887, 0.63324, 0.62791, 0.62293, 0.61836, 0.61425, 0.61064, 0.60752, 0.60481, 0.60224,\
        0.59933, 0.59577, 0.59181, 0.58757, 0.58303, 0.57816, 0.57291, 0.56726, 0.56118, 0.55465, 0.54767],\
        [0.46918, 0.47208, 0.47460, 0.47676, 0.47861, 0.48018, 0.48151, 0.48262, 0.48354, 0.48428, 0.48486,\
        0.48527, 0.48553, 0.48566, 0.48565, 0.48551, 0.48526, 0.48490, 0.48443, 0.48389, 0.48331, 0.48272,\
        0.48220, 0.48182, 0.48165, 0.48179, 0.48232, 0.48331, 0.48484, 0.48696, 0.48971, 0.49312, 0.49717,\
        0.50196, 0.50800, 0.51512, 0.52331, 0.53251, 0.54263, 0.55354, 0.56509, 0.57707, 0.58922, 0.60119,\
        0.61275, 0.62396, 0.63500, 0.64591, 0.65671, 0.66746, 0.67825, 0.68900, 0.69935, 0.71003, 0.72085,\
        0.73114, 0.74074, 0.74966, 0.75801, 0.76595, 0.77358, 0.78091, 0.78782, 0.79406, 0.79882, 0.80055,\
        0.79992, 0.79706, 0.79315, 0.78842, 0.78370, 0.77958, 0.77611, 0.77319, 0.77067, 0.76845, 0.76646,\
        0.76464, 0.76295, 0.76132, 0.75965, 0.75780, 0.75561, 0.75289, 0.74945, 0.74515, 0.73986, 0.73358,\
        0.72637, 0.71846, 0.71029, 0.70264, 0.69521, 0.68772, 0.68059, 0.67374, 0.66708, 0.66055, 0.65412,\
        0.64783, 0.64172, 0.63585, 0.63027, 0.62504, 0.62021, 0.61581, 0.61188, 0.60842, 0.60538, 0.60259,\
        0.59966, 0.59621, 0.59242, 0.58837, 0.58399, 0.57926, 0.57412, 0.56855, 0.56254, 0.55608, 0.54916],\
        [0.47356, 0.47629, 0.47861, 0.48058, 0.48224, 0.48363, 0.48481, 0.48580, 0.48661, 0.48726, 0.48777,\
        0.48812, 0.48833, 0.48841, 0.48837, 0.48820, 0.48791, 0.48751, 0.48699, 0.48637, 0.48567, 0.48495,\
        0.48428, 0.48373, 0.48340, 0.48340, 0.48381, 0.48472, 0.48622, 0.48834, 0.49112, 0.49457, 0.49874,\
        0.50326, 0.50864, 0.51515, 0.52276, 0.53143, 0.54111, 0.55169, 0.56302, 0.57494, 0.58728, 0.59970,\
        0.61178, 0.62371, 0.63565, 0.64747, 0.65908, 0.67059, 0.68226, 0.69425, 0.70593, 0.71716, 0.72828,\
        0.73883, 0.74854, 0.75742, 0.76562, 0.77341, 0.78097, 0.78830, 0.79517, 0.80129, 0.80601, 0.80785,\
        0.80703, 0.80405, 0.79974, 0.79416, 0.78899, 0.78484, 0.78146, 0.77864, 0.77622, 0.77409, 0.77216,\
        0.77041, 0.76883, 0.76736, 0.76589, 0.76429, 0.76236, 0.75990, 0.75670, 0.75257, 0.74739, 0.74115,\
        0.73391, 0.72594, 0.71762, 0.70934, 0.70099, 0.69296, 0.68542, 0.67826, 0.67132, 0.66447, 0.65772,\
        0.65110, 0.64468, 0.63851, 0.63263, 0.62709, 0.62191, 0.61714, 0.61278, 0.60887, 0.60537, 0.60223,\
        0.59922, 0.59603, 0.59260, 0.58885, 0.58472, 0.58015, 0.57514, 0.56967, 0.56373, 0.55733, 0.55047],\
        [0.47772, 0.48025, 0.48237, 0.48413, 0.48559, 0.48680, 0.48783, 0.48869, 0.48942, 0.49001, 0.49046,\
        0.49078, 0.49096, 0.49102, 0.49097, 0.49080, 0.49052, 0.49011, 0.48956, 0.48887, 0.48808, 0.48722,\
        0.48638, 0.48564, 0.48513, 0.48494, 0.48519, 0.48598, 0.48739, 0.48947, 0.49221, 0.49555, 0.49935,\
        0.50374, 0.50879, 0.51475, 0.52179, 0.52996, 0.53923, 0.54953, 0.56069, 0.57256, 0.58501, 0.59797,\
        0.61068, 0.62371, 0.63699, 0.64997, 0.66242, 0.67455, 0.68686, 0.69970, 0.71271, 0.72518, 0.73695,\
        0.74778, 0.75751, 0.76620, 0.77409, 0.78155, 0.78896, 0.79628, 0.80316, 0.80911, 0.81338, 0.81514,\
        0.81426, 0.81110, 0.80612, 0.79991, 0.79460, 0.79062, 0.78723, 0.78442, 0.78203, 0.77986, 0.77788,\
        0.77611, 0.77457, 0.77324, 0.77201, 0.77072, 0.76916, 0.76708, 0.76425, 0.76044, 0.75550, 0.74938,\
        0.74217, 0.73408, 0.72544, 0.71651, 0.70752, 0.69885, 0.69080, 0.68331, 0.67601, 0.66874, 0.66155,\
        0.65453, 0.64774, 0.64123, 0.63502, 0.62913, 0.62358, 0.61839, 0.61356, 0.60912, 0.60510, 0.60158,\
        0.59851, 0.59566, 0.59264, 0.58923, 0.58535, 0.58095, 0.57605, 0.57065, 0.56477, 0.55843, 0.55162],\
        [0.48163, 0.48396, 0.48585, 0.48738, 0.48862, 0.48965, 0.49052, 0.49127, 0.49193, 0.49248, 0.49291,\
        0.49322, 0.49340, 0.49347, 0.49343, 0.49330, 0.49306, 0.49268, 0.49214, 0.49142, 0.49054, 0.48954,\
        0.48851, 0.48756, 0.48682, 0.48640, 0.48644, 0.48704, 0.48830, 0.49027, 0.49294, 0.49623, 0.49988,\
        0.50386, 0.50840, 0.51384, 0.52036, 0.52806, 0.53701, 0.54716, 0.55833, 0.57024, 0.58258, 0.59544,\
        0.60935, 0.62444, 0.63962, 0.65400, 0.66730, 0.67982, 0.69254, 0.70637, 0.72062, 0.73429, 0.74685,\
        0.75803, 0.76775, 0.77617, 0.78365, 0.79068, 0.79794, 0.80539, 0.81226, 0.81796, 0.82185, 0.82338,\
        0.82237, 0.81902, 0.81378, 0.80753, 0.80190, 0.79765, 0.79369, 0.79075, 0.78824, 0.78580, 0.78358,\
        0.78165, 0.78006, 0.77884, 0.77786, 0.77694, 0.77584, 0.77426, 0.77193, 0.76858, 0.76398, 0.75805,\
        0.75085, 0.74261, 0.73362, 0.72421, 0.71470, 0.70541, 0.69678, 0.68894, 0.68113, 0.67327, 0.66554,\
        0.65806, 0.65088, 0.64401, 0.63746, 0.63123, 0.62532, 0.61974, 0.61447, 0.60953, 0.60498, 0.60104,\
        0.59804, 0.59556, 0.59291, 0.58977, 0.58605, 0.58175, 0.57690, 0.57154, 0.56569, 0.55938, 0.55262],\
        [0.48529, 0.48739, 0.48903, 0.49030, 0.49129, 0.49212, 0.49284, 0.49350, 0.49410, 0.49465, 0.49509,\
        0.49542, 0.49563, 0.49573, 0.49574, 0.49568, 0.49553, 0.49522, 0.49473, 0.49401, 0.49306, 0.49192,\
        0.49069, 0.48950, 0.48849, 0.48780, 0.48756, 0.48791, 0.48892, 0.49068, 0.49320, 0.49639, 0.50000,\
        0.50343, 0.50745, 0.51239, 0.51842, 0.52571, 0.53441, 0.54460, 0.55617, 0.56873, 0.58155, 0.59410,\
        0.60954, 0.62729, 0.64437, 0.66010, 0.67425, 0.68711, 0.69995, 0.71484, 0.73012, 0.74474, 0.75801,\
        0.76951, 0.77918, 0.78738, 0.79453, 0.80131, 0.80848, 0.81585, 0.82256, 0.82801, 0.83161, 0.83291,\
        0.83177, 0.82835, 0.82316, 0.81706, 0.81114, 0.80583, 0.80088, 0.79795, 0.79482, 0.79181, 0.78918,\
        0.78690, 0.78516, 0.78399, 0.78327, 0.78277, 0.78219, 0.78121, 0.77949, 0.77671, 0.77257, 0.76688,\
        0.75968, 0.75124, 0.74192, 0.73212, 0.72220, 0.71253, 0.70350, 0.69512, 0.68644, 0.67783, 0.66953,\
        0.66160, 0.65404, 0.64684, 0.63998, 0.63345, 0.62723, 0.62133, 0.61573, 0.61046, 0.60556, 0.60123,\
        0.59838, 0.59621, 0.59371, 0.59064, 0.58693, 0.58260, 0.57773, 0.57235, 0.56651, 0.56021, 0.55348],\
        [0.48869, 0.49053, 0.49188, 0.49285, 0.49357, 0.49417, 0.49474, 0.49531, 0.49590, 0.49648, 0.49697,\
        0.49736, 0.49762, 0.49777, 0.49786, 0.49792, 0.49791, 0.49773, 0.49732, 0.49665, 0.49566, 0.49437,\
        0.49294, 0.49150, 0.49020, 0.48919, 0.48862, 0.48862, 0.48931, 0.49074, 0.49292, 0.49573, 0.49887,\
        0.50216, 0.50595, 0.51049, 0.51603, 0.52285, 0.53128, 0.54164, 0.55407, 0.56828, 0.58358, 0.59886,\
        0.61532, 0.63359, 0.65147, 0.66817, 0.68325, 0.69669, 0.71025, 0.72524, 0.74096, 0.75628, 0.77019,\
        0.78195, 0.79152, 0.79949, 0.80660, 0.81340, 0.82042, 0.82747, 0.83393, 0.83917, 0.84258, 0.84373,\
        0.84247, 0.83898, 0.83379, 0.82763, 0.82125, 0.81515, 0.80976, 0.80552, 0.80139, 0.79777, 0.79455,\
        0.79166, 0.78967, 0.78853, 0.78805, 0.78798, 0.78796, 0.78762, 0.78660, 0.78453, 0.78095, 0.77554,\
        0.76831, 0.75963, 0.74997, 0.73980, 0.72952, 0.71949, 0.71001, 0.70082, 0.69131, 0.68209, 0.67333,\
        0.66504, 0.65718, 0.64971, 0.64259, 0.63580, 0.62935, 0.62322, 0.61745, 0.61207, 0.60720, 0.60313,\
        0.60023, 0.59778, 0.59513, 0.59187, 0.58797, 0.58350, 0.57852, 0.57308, 0.56721, 0.56091, 0.55420],\
        [0.49179, 0.49335, 0.49436, 0.49499, 0.49541, 0.49576, 0.49616, 0.49666, 0.49727, 0.49792, 0.49852,\
        0.49898, 0.49935, 0.49959, 0.49980, 0.50002, 0.50020, 0.50018, 0.49987, 0.49933, 0.49832, 0.49690,\
        0.49529, 0.49362, 0.49202, 0.49066, 0.48972, 0.48933, 0.48961, 0.49063, 0.49238, 0.49476, 0.49756,\
        0.50071, 0.50424, 0.50837, 0.51334, 0.51951, 0.52741, 0.53778, 0.55120, 0.56765, 0.58622, 0.60544,\
        0.62373, 0.64198, 0.65970, 0.67673, 0.69300, 0.70688, 0.72088, 0.73610, 0.75213, 0.76807, 0.78273,\
        0.79471, 0.80408, 0.81189, 0.81898, 0.82585, 0.83278, 0.83963, 0.84594, 0.85111, 0.85448, 0.85556,\
        0.85416, 0.85051, 0.84514, 0.83871, 0.83186, 0.82509, 0.81883, 0.81321, 0.80807, 0.80352, 0.79924,\
        0.79572, 0.79353, 0.79233, 0.79204, 0.79237, 0.79289, 0.79315, 0.79282, 0.79158, 0.78874, 0.78364,\
        0.77631, 0.76733, 0.75733, 0.74681, 0.73616, 0.72565, 0.71543, 0.70535, 0.69541, 0.68589, 0.67688,\
        0.66836, 0.66029, 0.65261, 0.64528, 0.63830, 0.63166, 0.62539, 0.61954, 0.61419, 0.60947, 0.60558,\
        0.60252, 0.59987, 0.59692, 0.59328, 0.58905, 0.58435, 0.57922, 0.57370, 0.56778, 0.56148, 0.55479],\
        [0.49457, 0.49581, 0.49643, 0.49668, 0.49676, 0.49686, 0.49708, 0.49751, 0.49814, 0.49892, 0.49967,\
        0.50023, 0.50072, 0.50116, 0.50160, 0.50203, 0.50240, 0.50255, 0.50242, 0.50194, 0.50096, 0.49952,\
        0.49783, 0.49595, 0.49406, 0.49236, 0.49104, 0.49023, 0.49007, 0.49064, 0.49198, 0.49403, 0.49660,\
        0.49952, 0.50273, 0.50640, 0.51064, 0.51579, 0.52258, 0.53227, 0.54635, 0.56534, 0.58742, 0.61013,\
        0.63129, 0.64998, 0.66717, 0.68335, 0.69896, 0.71462, 0.72996, 0.74589, 0.76234, 0.77880, 0.79429,\
        0.80654, 0.81581, 0.82370, 0.83092, 0.83790, 0.84485, 0.85168, 0.85804, 0.86334, 0.86687, 0.86797,\
        0.86637, 0.86240, 0.85669, 0.84987, 0.84247, 0.83497, 0.82773, 0.82092, 0.81459, 0.80875, 0.80342,\
        0.79951, 0.79680, 0.79532, 0.79509, 0.79572, 0.79669, 0.79739, 0.79755, 0.79725, 0.79538, 0.79063,\
        0.78309, 0.77383, 0.76359, 0.75283, 0.74188, 0.73094, 0.72012, 0.70947, 0.69916, 0.68937, 0.68022,\
        0.67159, 0.66339, 0.65555, 0.64805, 0.64090, 0.63411, 0.62773, 0.62181, 0.61646, 0.61178, 0.60788,\
        0.60466, 0.60181, 0.59854, 0.59447, 0.58994, 0.58501, 0.57975, 0.57415, 0.56821, 0.56191, 0.55525],\
        [0.49698, 0.49786, 0.49803, 0.49787, 0.49762, 0.49745, 0.49749, 0.49783, 0.49847, 0.49936, 0.50029,\
        0.50105, 0.50176, 0.50251, 0.50328, 0.50402, 0.50464, 0.50501, 0.50503, 0.50463, 0.50372, 0.50235,\
        0.50064, 0.49860, 0.49646, 0.49447, 0.49279, 0.49161, 0.49099, 0.49110, 0.49208, 0.49390, 0.49634,\
        0.49898, 0.50189, 0.50507, 0.50838, 0.51207, 0.51687, 0.52447, 0.53789, 0.55965, 0.58770, 0.61494,\
        0.63712, 0.65573, 0.67256, 0.68855, 0.70402, 0.72046, 0.73729, 0.75414, 0.77097, 0.78755, 0.80309,\
        0.81596, 0.82604, 0.83443, 0.84195, 0.84912, 0.85619, 0.86312, 0.86963, 0.87524, 0.87917, 0.88041,\
        0.87845, 0.87401, 0.86790, 0.86064, 0.85268, 0.84444, 0.83625, 0.82835, 0.82090, 0.81401, 0.80792,\
        0.80303, 0.79933, 0.79757, 0.79716, 0.79783, 0.79907, 0.79994, 0.80021, 0.80061, 0.79992, 0.79553,\
        0.78786, 0.77862, 0.76843, 0.75767, 0.74661, 0.73544, 0.72430, 0.71328, 0.70260, 0.69266, 0.68350,\
        0.67482, 0.66652, 0.65854, 0.65089, 0.64357, 0.63663, 0.63010, 0.62407, 0.61862, 0.61382, 0.60972,\
        0.60621, 0.60297, 0.59940, 0.59516, 0.59042, 0.58537, 0.58003, 0.57441, 0.56847, 0.56220, 0.55558],\
        [0.49891, 0.49935, 0.49907, 0.49855, 0.49801, 0.49761, 0.49747, 0.49767, 0.49829, 0.49928, 0.50041,\
        0.50146, 0.50255, 0.50373, 0.50496, 0.50612, 0.50708, 0.50772, 0.50793, 0.50766, 0.50686, 0.50556,\
        0.50382, 0.50169, 0.49947, 0.49718, 0.49526, 0.49383, 0.49274, 0.49238, 0.49303, 0.49470, 0.49722,\
        0.50004, 0.50247, 0.50486, 0.50705, 0.50902, 0.51106, 0.51453, 0.52413, 0.54671, 0.58184, 0.61464,\
        0.63773, 0.65693, 0.67441, 0.69123, 0.70814, 0.72555, 0.74340, 0.76117, 0.77838, 0.79493, 0.81071,\
        0.82430, 0.83526, 0.84423, 0.85205, 0.85939, 0.86658, 0.87360, 0.88021, 0.88611, 0.89056, 0.89214,\
        0.88949, 0.88461, 0.87822, 0.87062, 0.86216, 0.85322, 0.84418, 0.83534, 0.82693, 0.81915, 0.81222,\
        0.80638, 0.80199, 0.79938, 0.79834, 0.79867, 0.79960, 0.80025, 0.80074, 0.80123, 0.80052, 0.79684,\
        0.79010, 0.78151, 0.77176, 0.76130, 0.75039, 0.73926, 0.72807, 0.71697, 0.70617, 0.69616, 0.68694,\
        0.67816, 0.66973, 0.66160, 0.65377, 0.64627, 0.63914, 0.63243, 0.62621, 0.62054, 0.61547, 0.61101,\
        0.60707, 0.60343, 0.59968, 0.59523, 0.59043, 0.58537, 0.58006, 0.57447, 0.56858, 0.56237, 0.55580],\
        [0.49996, 0.49998, 0.49950, 0.49880, 0.49807, 0.49749, 0.49717, 0.49723, 0.49773, 0.49876, 0.50015,\
        0.50160, 0.50322, 0.50501, 0.50684, 0.50851, 0.50989, 0.51085, 0.51132, 0.51122, 0.51056, 0.50936,\
        0.50766, 0.50557, 0.50325, 0.50065, 0.49889, 0.49733, 0.49572, 0.49490, 0.49532, 0.49688, 0.49929,\
        0.50211, 0.50429, 0.50588, 0.50689, 0.50725, 0.50670, 0.50494, 0.50615, 0.52472, 0.56276, 0.60133,\
        0.62942, 0.65173, 0.67165, 0.69050, 0.70943, 0.72882, 0.74826, 0.76746, 0.78577, 0.80266, 0.81896,\
        0.83305, 0.84445, 0.85365, 0.86149, 0.86882, 0.87607, 0.88312, 0.88952, 0.89526, 0.89984, 0.90151,\
        0.89817, 0.89368, 0.88727, 0.87951, 0.87063, 0.86108, 0.85130, 0.84168, 0.83249, 0.82397, 0.81632,\
        0.80978, 0.80458, 0.80089, 0.79908, 0.79878, 0.79910, 0.79949, 0.79987, 0.80027, 0.79939, 0.79616,\
        0.79046, 0.78281, 0.77379, 0.76386, 0.75335, 0.74249, 0.73151, 0.72058, 0.70999, 0.69999, 0.69055,\
        0.68160, 0.67300, 0.66469, 0.65666, 0.64896, 0.64161, 0.63466, 0.62817, 0.62219, 0.61674, 0.61182,\
        0.60736, 0.60317, 0.59903, 0.59469, 0.59001, 0.58507, 0.57987, 0.57437, 0.56857, 0.56244, 0.55594],\
        [0.49990, 0.49998, 0.49958, 0.49884, 0.49802, 0.49735, 0.49693, 0.49681, 0.49712, 0.49801, 0.49964,\
        0.50165, 0.50402, 0.50663, 0.50918, 0.51145, 0.51329, 0.51461, 0.51534, 0.51546, 0.51497, 0.51390,\
        0.51234, 0.51037, 0.50812, 0.50585, 0.50401, 0.50225, 0.50040, 0.49931, 0.49976, 0.50128, 0.50356,\
        0.50587, 0.50744, 0.50801, 0.50766, 0.50647, 0.50407, 0.49895, 0.49290, 0.50529, 0.54060, 0.57991,\
        0.61323, 0.64046, 0.66448, 0.68686, 0.70865, 0.73026, 0.75171, 0.77287, 0.79364, 0.81204, 0.82867,\
        0.84297, 0.85437, 0.86331, 0.87073, 0.87770, 0.88490, 0.89207, 0.89800, 0.90294, 0.90674, 0.90759,\
        0.90528, 0.90099, 0.89481, 0.88715, 0.87789, 0.86779, 0.85741, 0.84719, 0.83742, 0.82834, 0.82017,\
        0.81307, 0.80723, 0.80283, 0.80016, 0.79894, 0.79866, 0.79882, 0.79904, 0.79907, 0.79799, 0.79499,\
        0.78996, 0.78313, 0.77488, 0.76560, 0.75562, 0.74519, 0.73457, 0.72396, 0.71359, 0.70360, 0.69410,\
        0.68505, 0.67628, 0.66776, 0.65952, 0.65159, 0.64400, 0.63678, 0.62999, 0.62364, 0.61776, 0.61234,\
        0.60735, 0.60269, 0.59827, 0.59393, 0.58941, 0.58463, 0.57957, 0.57420, 0.56851, 0.56246, 0.55604],\
        [0.50008, 0.50019, 0.49980, 0.49900, 0.49817, 0.49755, 0.49715, 0.49696, 0.49704, 0.49761, 0.49914,\
        0.50198, 0.50548, 0.50903, 0.51234, 0.51519, 0.51748, 0.51913, 0.52012, 0.52045, 0.52015, 0.51927,\
        0.51791, 0.51617, 0.51421, 0.51224, 0.51045, 0.50877, 0.50726, 0.50653, 0.50700, 0.50837, 0.51018,\
        0.51165, 0.51212, 0.51121, 0.50893, 0.50554, 0.50080, 0.49314, 0.48775, 0.49631, 0.52287, 0.55823,\
        0.59342, 0.62516, 0.65371, 0.68016, 0.70538, 0.72981, 0.75384, 0.77689, 0.79927, 0.82075, 0.83935,\
        0.85428, 0.86549, 0.87384, 0.88040, 0.88657, 0.89342, 0.90067, 0.90635, 0.91042, 0.91293, 0.91315,\
        0.91097, 0.90680, 0.90120, 0.89327, 0.88363, 0.87310, 0.86230, 0.85167, 0.84155, 0.83215, 0.82365,\
        0.81621, 0.80996, 0.80503, 0.80151, 0.79940, 0.79865, 0.79847, 0.79841, 0.79801, 0.79665, 0.79378,\
        0.78920, 0.78300, 0.77542, 0.76678, 0.75735, 0.74743, 0.73723, 0.72699, 0.71688, 0.70705, 0.69758,\
        0.68845, 0.67948, 0.67074, 0.66228, 0.65412, 0.64629, 0.63881, 0.63170, 0.62498, 0.61866, 0.61276,\
        0.60729, 0.60232, 0.59773, 0.59332, 0.58888, 0.58424, 0.57931, 0.57405, 0.56845, 0.56249, 0.55614],\
        [0.50056, 0.50073, 0.50036, 0.49955, 0.49886, 0.49851, 0.49834, 0.49829, 0.49838, 0.49885, 0.50036,\
        0.50400, 0.50842, 0.51274, 0.51664, 0.51997, 0.52261, 0.52454, 0.52574, 0.52624, 0.52612, 0.52544,\
        0.52431, 0.52286, 0.52123, 0.51960, 0.51808, 0.51677, 0.51583, 0.51556, 0.51613, 0.51731, 0.51855,\
        0.51913, 0.51831, 0.51557, 0.51071, 0.50408, 0.49634, 0.48790, 0.48317, 0.48874, 0.50813, 0.53853,\
        0.57289, 0.60765, 0.64028, 0.67063, 0.69920, 0.72782, 0.75554, 0.78131, 0.80572, 0.83030, 0.85139,\
        0.86728, 0.87809, 0.88591, 0.89149, 0.89664, 0.90295, 0.90953, 0.91462, 0.91775, 0.91899, 0.91823,\
        0.91553, 0.91117, 0.90523, 0.89728, 0.88757, 0.87681, 0.86578, 0.85500, 0.84478, 0.83530, 0.82671,\
        0.81914, 0.81269, 0.80742, 0.80340, 0.80073, 0.79933, 0.79863, 0.79812, 0.79733, 0.79572, 0.79286,\
        0.78853, 0.78276, 0.77571, 0.76761, 0.75870, 0.74925, 0.73948, 0.72960, 0.71980, 0.71022, 0.70091,\
        0.69164, 0.68244, 0.67351, 0.66487, 0.65652, 0.64848, 0.64076, 0.63338, 0.62633, 0.61963, 0.61328,\
        0.60733, 0.60200, 0.59733, 0.59296, 0.58860, 0.58404, 0.57920, 0.57402, 0.56848, 0.56257, 0.55628],\
        [0.50146, 0.50173, 0.50149, 0.50091, 0.50059, 0.50063, 0.50084, 0.50121, 0.50175, 0.50283, 0.50509,\
        0.50885, 0.51345, 0.51808, 0.52231, 0.52591, 0.52877, 0.53086, 0.53218, 0.53280, 0.53281, 0.53230,\
        0.53140, 0.53024, 0.52894, 0.52767, 0.52655, 0.52570, 0.52528, 0.52543, 0.52615, 0.52719, 0.52800,\
        0.52784, 0.52586, 0.52135, 0.51391, 0.50386, 0.49307, 0.48321, 0.47754, 0.48051, 0.49491, 0.51971,\
        0.55088, 0.58748, 0.62467, 0.65987, 0.69304, 0.72548, 0.75697, 0.78719, 0.81492, 0.84128, 0.86477,\
        0.88222, 0.89200, 0.89973, 0.90534, 0.91002, 0.91529, 0.92023, 0.92376, 0.92538, 0.92505, 0.92287,\
        0.91905, 0.91387, 0.90746, 0.89964, 0.88964, 0.87874, 0.86777, 0.85713, 0.84707, 0.83775, 0.82930,\
        0.82182, 0.81536, 0.81000, 0.80578, 0.80272, 0.80068, 0.79935, 0.79835, 0.79718, 0.79532, 0.79237,\
        0.78814, 0.78263, 0.77594, 0.76825, 0.75977, 0.75073, 0.74133, 0.73177, 0.72222, 0.71279, 0.70351,\
        0.69418, 0.68497, 0.67598, 0.66724, 0.65876, 0.65056, 0.64266, 0.63507, 0.62778, 0.62081, 0.61414,\
        0.60782, 0.60195, 0.59736, 0.59308, 0.58872, 0.58416, 0.57933, 0.57417, 0.56865, 0.56276, 0.55649],\
        [0.50287, 0.50339, 0.50352, 0.50348, 0.50361, 0.50405, 0.50475, 0.50569, 0.50699, 0.50890, 0.51180,\
        0.51574, 0.52033, 0.52501, 0.52933, 0.53302, 0.53594, 0.53805, 0.53939, 0.54004, 0.54011, 0.53972,\
        0.53901, 0.53809, 0.53711, 0.53618, 0.53543, 0.53499, 0.53496, 0.53542, 0.53628, 0.53726, 0.53781,\
        0.53717, 0.53440, 0.52856, 0.51899, 0.50600, 0.49155, 0.47840, 0.46995, 0.46974, 0.47965, 0.49781,\
        0.52553, 0.56404, 0.60616, 0.64774, 0.68621, 0.72143, 0.75527, 0.78887, 0.82064, 0.85039, 0.87697,\
        0.89795, 0.90781, 0.91537, 0.92134, 0.92581, 0.92964, 0.93256, 0.93399, 0.93357, 0.93125, 0.92718,\
        0.92168, 0.91512, 0.90772, 0.89920, 0.88944, 0.87888, 0.86832, 0.85812, 0.84848, 0.83955, 0.83143,\
        0.82420, 0.81792, 0.81263, 0.80835, 0.80504, 0.80256, 0.80069, 0.79917, 0.79766, 0.79552, 0.79237,\
        0.78811, 0.78270, 0.77623, 0.76882, 0.76065, 0.75191, 0.74280, 0.73348, 0.72410, 0.71474, 0.70538,\
        0.69614, 0.68710, 0.67814, 0.66936, 0.66081, 0.65252, 0.64452, 0.63680, 0.62939, 0.62228, 0.61550,\
        0.60915, 0.60344, 0.59845, 0.59384, 0.58933, 0.58467, 0.57976, 0.57454, 0.56899, 0.56308, 0.55679],\
        [0.50479, 0.50569, 0.50632, 0.50687, 0.50756, 0.50853, 0.50980, 0.51142, 0.51350, 0.51621, 0.51968,\
        0.52389, 0.52855, 0.53322, 0.53751, 0.54115, 0.54398, 0.54598, 0.54721, 0.54778, 0.54783, 0.54750,\
        0.54692, 0.54621, 0.54547, 0.54481, 0.54434, 0.54417, 0.54437, 0.54497, 0.54586, 0.54676, 0.54714,\
        0.54625, 0.54308, 0.53644, 0.52521, 0.50914, 0.49011, 0.47175, 0.45888, 0.45484, 0.46014, 0.47401,\
        0.49968, 0.53751, 0.58425, 0.63293, 0.67567, 0.71255, 0.74866, 0.78512, 0.82158, 0.85595, 0.88548,\
        0.90812, 0.92253, 0.93155, 0.93784, 0.94210, 0.94473, 0.94577, 0.94502, 0.94232, 0.93771, 0.93140,\
        0.92374, 0.91526, 0.90648, 0.89740, 0.88762, 0.87764, 0.86777, 0.85822, 0.84919, 0.84080, 0.83315,\
        0.82630, 0.82031, 0.81521, 0.81097, 0.80755, 0.80482, 0.80260, 0.80068, 0.79883, 0.79629, 0.79284,\
        0.78843, 0.78300, 0.77661, 0.76935, 0.76138, 0.75284, 0.74393, 0.73479, 0.72554, 0.71629, 0.70710,\
        0.69803, 0.68894, 0.67998, 0.67121, 0.66265, 0.65434, 0.64630, 0.63855, 0.63111, 0.62399, 0.61724,\
        0.61094, 0.60516, 0.59991, 0.59511, 0.59039, 0.58554, 0.58047, 0.57514, 0.56950, 0.56353, 0.55719],\
        [0.50718, 0.50853, 0.50969, 0.51085, 0.51215, 0.51372, 0.51563, 0.51797, 0.52080, 0.52421, 0.52824,\
        0.53279, 0.53760, 0.54230, 0.54654, 0.55005, 0.55269, 0.55446, 0.55546, 0.55584, 0.55578, 0.55544,\
        0.55493, 0.55436, 0.55380, 0.55331, 0.55299, 0.55290, 0.55311, 0.55362, 0.55433, 0.55496, 0.55506,\
        0.55392, 0.55052, 0.54351, 0.53109, 0.51147, 0.48624, 0.46073, 0.44264, 0.43556, 0.43843, 0.44977,\
        0.47121, 0.50552, 0.55539, 0.61338, 0.66079, 0.69879, 0.73826, 0.78013, 0.82073, 0.85848, 0.89167,\
        0.91705, 0.93450, 0.94602, 0.95350, 0.95791, 0.95975, 0.95925, 0.95649, 0.95154, 0.94457, 0.93585,\
        0.92579, 0.91496, 0.90427, 0.89449, 0.88506, 0.87575, 0.86662, 0.85781, 0.84945, 0.84167, 0.83455,\
        0.82816, 0.82253, 0.81767, 0.81357, 0.81015, 0.80730, 0.80487, 0.80267, 0.80039, 0.79739, 0.79364,\
        0.78904, 0.78352, 0.77710, 0.76988, 0.76198, 0.75356, 0.74475, 0.73572, 0.72658, 0.71743, 0.70834,\
        0.69932, 0.69035, 0.68149, 0.67278, 0.66426, 0.65598, 0.64798, 0.64027, 0.63288, 0.62583, 0.61916,\
        0.61293, 0.60715, 0.60182, 0.59680, 0.59179, 0.58668, 0.58140, 0.57591, 0.57016, 0.56410, 0.55770],\
        [0.50992, 0.51174, 0.51345, 0.51520, 0.51712, 0.51932, 0.52189, 0.52491, 0.52843, 0.53249, 0.53706,\
        0.54201, 0.54706, 0.55187, 0.55610, 0.55945, 0.56181, 0.56322, 0.56387, 0.56396, 0.56371, 0.56330,\
        0.56283, 0.56237, 0.56192, 0.56151, 0.56118, 0.56097, 0.56093, 0.56107, 0.56128, 0.56130, 0.56072,\
        0.55891, 0.55492, 0.54736, 0.53399, 0.51152, 0.47735, 0.44234, 0.41960, 0.41142, 0.41374, 0.42223,\
        0.43865, 0.46724, 0.51392, 0.58313, 0.64107, 0.68218, 0.72485, 0.77040, 0.81398, 0.85672, 0.89522,\
        0.92362, 0.94397, 0.95832, 0.96764, 0.97274, 0.97425, 0.97260, 0.96812, 0.96112, 0.95192, 0.94092,\
        0.92859, 0.91555, 0.90275, 0.89221, 0.88293, 0.87404, 0.86548, 0.85729, 0.84955, 0.84235, 0.83577,\
        0.82984, 0.82459, 0.82001, 0.81608, 0.81272, 0.80982, 0.80722, 0.80470, 0.80193, 0.79865, 0.79471,\
        0.78990, 0.78421, 0.77768, 0.77039, 0.76247, 0.75406, 0.74528, 0.73630, 0.72722, 0.71814, 0.70910,\
        0.70011, 0.69138, 0.68266, 0.67404, 0.66561, 0.65742, 0.64951, 0.64190, 0.63461, 0.62766, 0.62110,\
        0.61494, 0.60920, 0.60383, 0.59864, 0.59333, 0.58795, 0.58246, 0.57682, 0.57093, 0.56477, 0.55828],\
        [0.51290, 0.51519, 0.51743, 0.51975, 0.52226, 0.52507, 0.52826, 0.53190, 0.53605, 0.54069, 0.54578,\
        0.55114, 0.55652, 0.56154, 0.56581, 0.56901, 0.57101, 0.57196, 0.57214, 0.57186, 0.57138, 0.57088,\
        0.57045, 0.57009, 0.56974, 0.56934, 0.56886, 0.56832, 0.56779, 0.56724, 0.56656, 0.56548, 0.56358,\
        0.56026, 0.55460, 0.54510, 0.52958, 0.50181, 0.45671, 0.41514, 0.39092, 0.38286, 0.38428, 0.39025,\
        0.40234, 0.42590, 0.47037, 0.54279, 0.60901, 0.65982, 0.71060, 0.75892, 0.80438, 0.85049, 0.89209,\
        0.92528, 0.95055, 0.96859, 0.98029, 0.98652, 0.98807, 0.98562, 0.97974, 0.97096, 0.95981, 0.94681,\
        0.93259, 0.91797, 0.90415, 0.89221, 0.88213, 0.87314, 0.86481, 0.85701, 0.84973, 0.84302, 0.83690,\
        0.83139, 0.82651, 0.82222, 0.81848, 0.81521, 0.81228, 0.80954, 0.80678, 0.80375, 0.80024, 0.79601,\
        0.79093, 0.78501, 0.77829, 0.77086, 0.76283, 0.75434, 0.74553, 0.73653, 0.72746, 0.71845, 0.70960,\
        0.70092, 0.69223, 0.68350, 0.67497, 0.66666, 0.65861, 0.65084, 0.64336, 0.63621, 0.62939, 0.62292,\
        0.61682, 0.61106, 0.60560, 0.60023, 0.59473, 0.58919, 0.58357, 0.57778, 0.57178, 0.56551, 0.55893],\
        [0.51602, 0.51875, 0.52148, 0.52432, 0.52737, 0.53073, 0.53447, 0.53866, 0.54333, 0.54847, 0.55401,\
        0.55980, 0.56556, 0.57090, 0.57531, 0.57837, 0.57995, 0.58032, 0.57994, 0.57924, 0.57851, 0.57796,\
        0.57762, 0.57744, 0.57724, 0.57683, 0.57609, 0.57506, 0.57381, 0.57233, 0.57043, 0.56775, 0.56379,\
        0.55791, 0.54917, 0.53561, 0.51340, 0.47939, 0.42856, 0.38536, 0.36283, 0.35364, 0.35245, 0.35529,\
        0.36168, 0.37923, 0.42503, 0.50008, 0.56471, 0.62722, 0.69018, 0.74574, 0.79476, 0.84290, 0.88729,\
        0.92545, 0.95568, 0.97760, 0.99189, 0.99944, 1.00124, 0.99821, 0.99119, 0.98092, 0.96811, 0.95344,\
        0.93768, 0.92171, 0.90657, 0.89349, 0.88265, 0.87326, 0.86485, 0.85718, 0.85017, 0.84379, 0.83804,\
        0.83289, 0.82832, 0.82429, 0.82073, 0.81756, 0.81463, 0.81180, 0.80888, 0.80565, 0.80192, 0.79741,\
        0.79204, 0.78585, 0.77888, 0.77124, 0.76303, 0.75440, 0.74549, 0.73642, 0.72731, 0.71828, 0.70946,\
        0.70101, 0.69240, 0.68383, 0.67549, 0.66739, 0.65952, 0.65193, 0.64463, 0.63763, 0.63094, 0.62457,\
        0.61850, 0.61270, 0.60708, 0.60152, 0.59597, 0.59038, 0.58467, 0.57878, 0.57267, 0.56630, 0.55962],\
        [0.51915, 0.52229, 0.52547, 0.52877, 0.53229, 0.53611, 0.54031, 0.54493, 0.55001, 0.55553, 0.56143,\
        0.56758, 0.57371, 0.57943, 0.58414, 0.58712, 0.58817, 0.58785, 0.58689, 0.58580, 0.58490, 0.58437,\
        0.58424, 0.58437, 0.58446, 0.58410, 0.58307, 0.58143, 0.57936, 0.57683, 0.57356, 0.56900, 0.56242,\
        0.55305, 0.54008, 0.52194, 0.49369, 0.45244, 0.40455, 0.36409, 0.33977, 0.32828, 0.32393, 0.32290,\
        0.32162, 0.32137, 0.35440, 0.43103, 0.50941, 0.58317, 0.65832, 0.72650, 0.78251, 0.83416, 0.88303,\
        0.92597, 0.96062, 0.98626, 1.00299, 1.01177, 1.01382, 1.01031, 1.00232, 0.99079, 0.97659, 0.96054,\
        0.94351, 0.92638, 0.91019, 0.89604, 0.88430, 0.87435, 0.86561, 0.85785, 0.85091, 0.84473, 0.83923,\
        0.83435, 0.83004, 0.82623, 0.82283, 0.81974, 0.81683, 0.81394, 0.81089, 0.80748, 0.80349, 0.79872,\
        0.79312, 0.78665, 0.77939, 0.77146, 0.76303, 0.75421, 0.74515, 0.73598, 0.72681, 0.71770, 0.70869,\
        0.69989, 0.69176, 0.68368, 0.67566, 0.66780, 0.66016, 0.65278, 0.64567, 0.63883, 0.63228, 0.62601,\
        0.61998, 0.61416, 0.60846, 0.60281, 0.59719, 0.59155, 0.58578, 0.57980, 0.57359, 0.56711, 0.56034],\
        [0.52223, 0.52573, 0.52929, 0.53299, 0.53689, 0.54108, 0.54562, 0.55055, 0.55589, 0.56164, 0.56774,\
        0.57407, 0.58044, 0.58651, 0.59165, 0.59469, 0.59503, 0.59404, 0.59265, 0.59130, 0.59038, 0.59006,\
        0.59026, 0.59090, 0.59148, 0.59136, 0.59005, 0.58777, 0.58494, 0.58152, 0.57704, 0.57073, 0.56155,\
        0.54830, 0.52999, 0.50668, 0.47439, 0.43221, 0.38763, 0.34908, 0.32242, 0.30914, 0.30156, 0.29543,\
        0.28704, 0.27462, 0.29677, 0.37470, 0.45691, 0.53237, 0.61707, 0.69812, 0.76117, 0.81888, 0.87482,\
        0.92521, 0.96588, 0.99520, 1.01399, 1.02366, 1.02577, 1.02175, 1.01287, 1.00026, 0.98491, 0.96774,\
        0.94967, 0.93161, 0.91449, 0.89921, 0.88689, 0.87624, 0.86697, 0.85893, 0.85193, 0.84582, 0.84048,\
        0.83579, 0.83167, 0.82802, 0.82474, 0.82172, 0.81882, 0.81589, 0.81274, 0.80918, 0.80501, 0.80002,\
        0.79416, 0.78734, 0.77972, 0.77148, 0.76276, 0.75372, 0.74450, 0.73522, 0.72600, 0.71693, 0.70804,\
        0.69942, 0.69128, 0.68335, 0.67556, 0.66794, 0.66054, 0.65338, 0.64647, 0.63982, 0.63341, 0.62725,\
        0.62129, 0.61549, 0.60978, 0.60411, 0.59845, 0.59275, 0.58690, 0.58083, 0.57451, 0.56793, 0.56106],\
        [0.52517, 0.52899, 0.53287, 0.53688, 0.54108, 0.54554, 0.55031, 0.55541, 0.56086, 0.56666, 0.57273,\
        0.57899, 0.58528, 0.59132, 0.59667, 0.59997, 0.59957, 0.59841, 0.59701, 0.59567, 0.59494, 0.59514,\
        0.59573, 0.59704, 0.59834, 0.59871, 0.59721, 0.59450, 0.59124, 0.58732, 0.58217, 0.57481, 0.56386,\
        0.54770, 0.52492, 0.49551, 0.46126, 0.42121, 0.37931, 0.34123, 0.31186, 0.29642, 0.28826, 0.27777,\
        0.26425, 0.25697, 0.28352, 0.35680, 0.43253, 0.49770, 0.58504, 0.66594, 0.73538, 0.80132, 0.86766,\
        0.92693, 0.97271, 1.00488, 1.02498, 1.03502, 1.03688, 1.03225, 1.02254, 1.00899, 0.99268, 0.97460,\
        0.95571, 0.93696, 0.91929, 0.90380, 0.89030, 0.87850, 0.86860, 0.86023, 0.85312, 0.84701, 0.84175,\
        0.83718, 0.83318, 0.82964, 0.82645, 0.82347, 0.82057, 0.81759, 0.81435, 0.81067, 0.80637, 0.80128,\
        0.79503, 0.78777, 0.77977, 0.77119, 0.76218, 0.75291, 0.74351, 0.73411, 0.72486, 0.71589, 0.70733,\
        0.69905, 0.69082, 0.68288, 0.67523, 0.66783, 0.66066, 0.65373, 0.64704, 0.64057, 0.63433, 0.62829,\
        0.62242, 0.61668, 0.61103, 0.60539, 0.59972, 0.59397, 0.58801, 0.58183, 0.57541, 0.56873, 0.56177],\
        [0.52792, 0.53200, 0.53614, 0.54039, 0.54481, 0.54944, 0.55432, 0.55947, 0.56488, 0.57055, 0.57639,\
        0.58231, 0.58812, 0.59353, 0.59800, 0.60070, 0.60134, 0.60091, 0.60008, 0.59911, 0.59876, 0.59999,\
        0.60119, 0.60303, 0.60502, 0.60579, 0.60477, 0.60237, 0.59917, 0.59531, 0.59018, 0.58285, 0.57177,\
        0.55485, 0.53017, 0.49731, 0.45967, 0.42096, 0.38121, 0.34376, 0.31440, 0.29763, 0.28928, 0.27800,\
        0.26391, 0.25778, 0.28902, 0.36361, 0.43447, 0.49381, 0.56665, 0.64284, 0.71613, 0.78836, 0.86345,\
        0.93108, 0.98116, 1.01504, 1.03558, 1.04541, 1.04676, 1.04142, 1.03093, 1.01658, 0.99949, 0.98069,\
        0.96114, 0.94178, 0.92352, 0.90706, 0.89264, 0.88040, 0.87016, 0.86158, 0.85436, 0.84824, 0.84300,\
        0.83848, 0.83455, 0.83107, 0.82791, 0.82494, 0.82202, 0.81898, 0.81564, 0.81181, 0.80731, 0.80190,\
        0.79536, 0.78779, 0.77943, 0.77052, 0.76122, 0.75172, 0.74215, 0.73263, 0.72331, 0.71432, 0.70586,\
        0.69786, 0.68981, 0.68206, 0.67463, 0.66747, 0.66055, 0.65386, 0.64738, 0.64112, 0.63504, 0.62914,\
        0.62338, 0.61772, 0.61214, 0.60658, 0.60099, 0.59515, 0.58905, 0.58276, 0.57625, 0.56949, 0.56245],\
        [0.53044, 0.53474, 0.53908, 0.54351, 0.54806, 0.55278, 0.55768, 0.56276, 0.56802, 0.57341, 0.57886,\
        0.58425, 0.58939, 0.59397, 0.59758, 0.60001, 0.60147, 0.60205, 0.60218, 0.60223, 0.60288, 0.60477,\
        0.60709, 0.60963, 0.61204, 0.61346, 0.61344, 0.61209, 0.60972, 0.60647, 0.60204, 0.59577, 0.58643,\
        0.57151, 0.54797, 0.51282, 0.47203, 0.43260, 0.39319, 0.35670, 0.32971, 0.31403, 0.30492, 0.29533,\
        0.28466, 0.27313, 0.29951, 0.38990, 0.45274, 0.49968, 0.55150, 0.62126, 0.70803, 0.78044, 0.86031,\
        0.93675, 0.99010, 1.02461, 1.04489, 1.05416, 1.05483, 1.04880, 1.03763, 1.02263, 1.00496, 0.98563,\
        0.96558, 0.94568, 0.92671, 0.90934, 0.89433, 0.88191, 0.87155, 0.86288, 0.85559, 0.84943, 0.84418,\
        0.83967, 0.83574, 0.83226, 0.82909, 0.82609, 0.82311, 0.81998, 0.81652, 0.81252, 0.80778, 0.80210,\
        0.79523, 0.78732, 0.77864, 0.76941, 0.75984, 0.75013, 0.74041, 0.73080, 0.72141, 0.71230, 0.70360,\
        0.69567, 0.68820, 0.68088, 0.67377, 0.66689, 0.66022, 0.65378, 0.64753, 0.64146, 0.63556, 0.62980,\
        0.62415, 0.61858, 0.61304, 0.60751, 0.60191, 0.59602, 0.58989, 0.58356, 0.57701, 0.57019, 0.56307],\
        [0.53271, 0.53717, 0.54166, 0.54621, 0.55085, 0.55558, 0.56043, 0.56537, 0.57039, 0.57543, 0.58041,\
        0.58522, 0.58969, 0.59360, 0.59678, 0.59923, 0.60110, 0.60260, 0.60390, 0.60531, 0.60726, 0.60999,\
        0.61322, 0.61661, 0.61974, 0.62210, 0.62334, 0.62341, 0.62245, 0.62057, 0.61770, 0.61347, 0.60711,\
        0.59675, 0.58028, 0.55102, 0.50471, 0.45987, 0.41599, 0.37806, 0.35324, 0.33827, 0.32826, 0.32060,\
        0.31531, 0.30988, 0.34556, 0.42243, 0.47104, 0.50544, 0.54571, 0.61494, 0.70888, 0.77061, 0.85542,\
        0.94270, 0.99770, 1.03212, 1.05188, 1.06051, 1.06054, 1.05393, 1.04225, 1.02682, 1.00878, 0.98914,\
        0.96880, 0.94856, 0.92918, 0.91138, 0.89598, 0.88331, 0.87285, 0.86411, 0.85676, 0.85054, 0.84524,\
        0.84068, 0.83671, 0.83318, 0.82995, 0.82687, 0.82380, 0.82056, 0.81696, 0.81278, 0.80777, 0.80170,\
        0.79454, 0.78635, 0.77735, 0.76782, 0.75800, 0.74810, 0.73829, 0.72867, 0.71932, 0.71025, 0.70163,\
        0.69381, 0.68656, 0.67955, 0.67273, 0.66612, 0.65972, 0.65351, 0.64749, 0.64163, 0.63591, 0.63029,\
        0.62475, 0.61925, 0.61373, 0.60816, 0.60246, 0.59660, 0.59054, 0.58424, 0.57768, 0.57083, 0.56365],\
        [0.53470, 0.53930, 0.54391, 0.54854, 0.55320, 0.55791, 0.56266, 0.56742, 0.57216, 0.57682, 0.58133,\
        0.58560, 0.58950, 0.59293, 0.59586, 0.59833, 0.60061, 0.60302, 0.60550, 0.60823, 0.61144, 0.61522,\
        0.61944, 0.62380, 0.62794, 0.63148, 0.63412, 0.63575, 0.63640, 0.63619, 0.63522, 0.63342, 0.63036,\
        0.62521, 0.61694, 0.59943, 0.55083, 0.49873, 0.44806, 0.40776, 0.38407, 0.36942, 0.35789, 0.34900,\
        0.34545, 0.35521, 0.39335, 0.44310, 0.48366, 0.51514, 0.55325, 0.62445, 0.71369, 0.77326, 0.85957,\
        0.94705, 1.00241, 1.03664, 1.05590, 1.06398, 1.06351, 1.05649, 1.04450, 1.02887, 1.01072, 0.99103,\
        0.97066, 0.95039, 0.93094, 0.91300, 0.89740, 0.88464, 0.87410, 0.86528, 0.85784, 0.85152, 0.84613,\
        0.84148, 0.83741, 0.83377, 0.83043, 0.82723, 0.82402, 0.82064, 0.81690, 0.81257, 0.80739, 0.80103,\
        0.79350, 0.78491, 0.77556, 0.76573, 0.75567, 0.74562, 0.73575, 0.72622, 0.71710, 0.70836, 0.69997,\
        0.69227, 0.68506, 0.67819, 0.67158, 0.66522, 0.65906, 0.65309, 0.64730, 0.64164, 0.63609, 0.63063,\
        0.62520, 0.61978, 0.61429, 0.60870, 0.60297, 0.59711, 0.59112, 0.58487, 0.57831, 0.57142, 0.56418],\
        [0.53642, 0.54113, 0.54582, 0.55050, 0.55518, 0.55984, 0.56447, 0.56904, 0.57351, 0.57781, 0.58189,\
        0.58568, 0.58914, 0.59223, 0.59501, 0.59765, 0.60042, 0.60355, 0.60705, 0.61095, 0.61534, 0.62023,\
        0.62551, 0.63095, 0.63627, 0.64114, 0.64527, 0.64846, 0.65067, 0.65200, 0.65268, 0.65278, 0.65202,\
        0.64946, 0.64189, 0.61923, 0.57884, 0.53267, 0.48391, 0.44371, 0.41935, 0.40529, 0.39313, 0.38237,\
        0.37801, 0.39067, 0.42183, 0.45851, 0.49415, 0.52744, 0.56695, 0.63501, 0.72132, 0.79378, 0.87739,\
        0.95347, 1.00585, 1.03871, 1.05708, 1.06451, 1.06360, 1.05632, 1.04423, 1.02863, 1.01062, 0.99115,\
        0.97106, 0.95109, 0.93192, 0.91422, 0.89871, 0.88591, 0.87527, 0.86633, 0.85876, 0.85232, 0.84679,\
        0.84200, 0.83778, 0.83399, 0.83049, 0.82711, 0.82373, 0.82016, 0.81624, 0.81176, 0.80649, 0.80007,\
        0.79200, 0.78294, 0.77323, 0.76311, 0.75284, 0.74264, 0.73274, 0.72333, 0.71454, 0.70637, 0.69853,\
        0.69076, 0.68352, 0.67675, 0.67034, 0.66419, 0.65828, 0.65255, 0.64697, 0.64151, 0.63615, 0.63084,\
        0.62555, 0.62022, 0.61481, 0.60925, 0.60352, 0.59769, 0.59175, 0.58551, 0.57892, 0.57197, 0.56467],\
        [0.53787, 0.54266, 0.54742, 0.55214, 0.55682, 0.56144, 0.56596, 0.57036, 0.57458, 0.57858, 0.58230,\
        0.58572, 0.58882, 0.59166, 0.59437, 0.59718, 0.60040, 0.60417, 0.60855, 0.61348, 0.61893, 0.62487,\
        0.63121, 0.63777, 0.64431, 0.65057, 0.65621, 0.66094, 0.66454, 0.66705, 0.66887, 0.67020, 0.67061,\
        0.66908, 0.66279, 0.64476, 0.61335, 0.57211, 0.52366, 0.47682, 0.45545, 0.44405, 0.43322, 0.42280,\
        0.41765, 0.42699, 0.44912, 0.47898, 0.51397, 0.55356, 0.59678, 0.66240, 0.74831, 0.83017, 0.90487,\
        0.96574, 1.01020, 1.03941, 1.05590, 1.06233, 1.06091, 1.05345, 1.04141, 1.02605, 1.00841, 0.98942,\
        0.96988, 0.95051, 0.93195, 0.91481, 0.89970, 0.88691, 0.87620, 0.86714, 0.85944, 0.85284, 0.84715,\
        0.84218, 0.83778, 0.83380, 0.83009, 0.82649, 0.82287, 0.81906, 0.81489, 0.81016, 0.80457, 0.79782,\
        0.78957, 0.78026, 0.77031, 0.75999, 0.74954, 0.73921, 0.72924, 0.71988, 0.71131, 0.70350, 0.69604,\
        0.68866, 0.68168, 0.67514, 0.66896, 0.66306, 0.65739, 0.65189, 0.64653, 0.64128, 0.63609, 0.63095,\
        0.62579, 0.62060, 0.61530, 0.60986, 0.60425, 0.59847, 0.59249, 0.58618, 0.57952, 0.57249, 0.56509],\
        [0.53904, 0.54391, 0.54873, 0.55349, 0.55818, 0.56277, 0.56722, 0.57149, 0.57553, 0.57930, 0.58275,\
        0.58587, 0.58870, 0.59134, 0.59397, 0.59687, 0.60037, 0.60482, 0.61000, 0.61579, 0.62214, 0.62901,\
        0.63630, 0.64390, 0.65163, 0.65923, 0.66638, 0.67262, 0.67749, 0.68081, 0.68329, 0.68572, 0.68706,\
        0.68684, 0.68595, 0.68010, 0.65616, 0.61884, 0.56650, 0.51633, 0.49808, 0.48672, 0.47657, 0.46813,\
        0.46490, 0.46934, 0.48308, 0.51168, 0.55293, 0.59930, 0.64493, 0.72263, 0.80761, 0.87291, 0.92999,\
        0.97853, 1.01469, 1.03897, 1.05273, 1.05771, 1.05563, 1.04798, 1.03611, 1.02114, 1.00407, 0.98577,\
        0.96703, 0.94851, 0.93082, 0.91444, 0.89981, 0.88733, 0.87668, 0.86757, 0.85974, 0.85299, 0.84712,\
        0.84196, 0.83736, 0.83316, 0.82921, 0.82535, 0.82145, 0.81733, 0.81283, 0.80780, 0.80187, 0.79467,\
        0.78630, 0.77693, 0.76689, 0.75646, 0.74589, 0.73542, 0.72533, 0.71592, 0.70743, 0.69981, 0.69276,\
        0.68599, 0.67950, 0.67334, 0.66747, 0.66185, 0.65642, 0.65116, 0.64601, 0.64095, 0.63594, 0.63095,\
        0.62595, 0.62090, 0.61575, 0.61046, 0.60501, 0.59931, 0.59325, 0.58683, 0.58006, 0.57293, 0.56544],\
        [0.53995, 0.54489, 0.54977, 0.55458, 0.55929, 0.56389, 0.56831, 0.57252, 0.57646, 0.58008, 0.58335,\
        0.58628, 0.58891, 0.59139, 0.59396, 0.59696, 0.60080, 0.60569, 0.61145, 0.61790, 0.62494, 0.63252,\
        0.64058, 0.64903, 0.65775, 0.66655, 0.67510, 0.68285, 0.68909, 0.69304, 0.69573, 0.69965, 0.70279,\
        0.70490, 0.70864, 0.71075, 0.69451, 0.66025, 0.61109, 0.56652, 0.54205, 0.52722, 0.51563, 0.50802,\
        0.50642, 0.50986, 0.52228, 0.55297, 0.60610, 0.66523, 0.72158, 0.80989, 0.86955, 0.91494, 0.95467,\
        0.99014, 1.01790, 1.03690, 1.04753, 1.05080, 1.04792, 1.04010, 1.02846, 1.01401, 0.99766, 0.98022,\
        0.96243, 0.94498, 0.92839, 0.91303, 0.89916, 0.88707, 0.87656, 0.86744, 0.85953, 0.85266, 0.84663,\
        0.84129, 0.83648, 0.83204, 0.82783, 0.82370, 0.81948, 0.81500, 0.81010, 0.80465, 0.79853, 0.79105,\
        0.78252, 0.77314, 0.76311, 0.75268, 0.74207, 0.73152, 0.72129, 0.71172, 0.70327, 0.69599, 0.68939,\
        0.68317, 0.67720, 0.67146, 0.66593, 0.66060, 0.65542, 0.65037, 0.64542, 0.64054, 0.63570, 0.63086,\
        0.62600, 0.62109, 0.61607, 0.61093, 0.60560, 0.59999, 0.59387, 0.58735, 0.58048, 0.57326, 0.56568],\
        [0.54058, 0.54559, 0.55054, 0.55542, 0.56019, 0.56483, 0.56928, 0.57350, 0.57742, 0.58101, 0.58421,\
        0.58705, 0.58957, 0.59193, 0.59441, 0.59747, 0.60156, 0.60680, 0.61294, 0.61981, 0.62730, 0.63534,\
        0.64389, 0.65290, 0.66228, 0.67193, 0.68159, 0.69072, 0.69812, 0.70321, 0.70693, 0.71218, 0.71780,\
        0.72338, 0.72927, 0.73127, 0.72109, 0.69495, 0.65257, 0.61108, 0.58467, 0.56508, 0.54872, 0.53864,\
        0.53759, 0.54576, 0.56701, 0.60734, 0.66841, 0.73357, 0.78750, 0.85267, 0.90195, 0.93976, 0.97107,\
        0.99750, 1.01827, 1.03248, 1.04009, 1.04163, 1.03794, 1.02998, 1.01865, 1.00483, 0.98931, 0.97284,\
        0.95611, 0.93979, 0.92446, 0.91046, 0.89758, 0.88594, 0.87562, 0.86658, 0.85869, 0.85176, 0.84562,\
        0.84012, 0.83511, 0.83044, 0.82597, 0.82156, 0.81704, 0.81222, 0.80690, 0.80095, 0.79439, 0.78691,\
        0.77842, 0.76911, 0.75918, 0.74885, 0.73832, 0.72780, 0.71757, 0.70788, 0.69931, 0.69250, 0.68638,\
        0.68060, 0.67505, 0.66967, 0.66445, 0.65937, 0.65442, 0.64957, 0.64480, 0.64007, 0.63538, 0.63068,\
        0.62594, 0.62114, 0.61622, 0.61116, 0.60589, 0.60028, 0.59418, 0.58765, 0.58074, 0.57345, 0.56579],\
        [0.54093, 0.54601, 0.55105, 0.55601, 0.56087, 0.56561, 0.57015, 0.57446, 0.57847, 0.58212, 0.58537,\
        0.58823, 0.59075, 0.59307, 0.59544, 0.59840, 0.60268, 0.60816, 0.61451, 0.62155, 0.62920, 0.63741,\
        0.64615, 0.65536, 0.66498, 0.67493, 0.68503, 0.69494, 0.70321, 0.70950, 0.71558, 0.72254, 0.73017,\
        0.73782, 0.74423, 0.74600, 0.73842, 0.71816, 0.68344, 0.64963, 0.62498, 0.60492, 0.58549, 0.57353,\
        0.57485, 0.58918, 0.62518, 0.67864, 0.72886, 0.78571, 0.83156, 0.87877, 0.91945, 0.95129, 0.97759,\
        0.99877, 1.01473, 1.02522, 1.03026, 1.03026, 1.02587, 1.01783, 1.00691, 0.99383, 0.97926, 0.96383,\
        0.94818, 0.93294, 0.91875, 0.90623, 0.89467, 0.88354, 0.87363, 0.86487, 0.85712, 0.85022, 0.84404,\
        0.83842, 0.83323, 0.82833, 0.82361, 0.81893, 0.81416, 0.80910, 0.80347, 0.79719, 0.79029, 0.78265,\
        0.77419, 0.76502, 0.75528, 0.74516, 0.73483, 0.72453, 0.71452, 0.70518, 0.69693, 0.69006, 0.68408,\
        0.67853, 0.67323, 0.66810, 0.66312, 0.65824, 0.65347, 0.64878, 0.64415, 0.63956, 0.63499, 0.63040,\
        0.62576, 0.62104, 0.61619, 0.61115, 0.60587, 0.60028, 0.59421, 0.58772, 0.58081, 0.57349, 0.56576],\
        [0.54100, 0.54615, 0.55127, 0.55634, 0.56133, 0.56620, 0.57091, 0.57538, 0.57957, 0.58340, 0.58683,\
        0.58983, 0.59247, 0.59488, 0.59730, 0.60013, 0.60444, 0.60990, 0.61616, 0.62310, 0.63065, 0.63876,\
        0.64738, 0.65646, 0.66592, 0.67564, 0.68541, 0.69482, 0.70367, 0.71211, 0.72052, 0.72947, 0.73878,\
        0.74762, 0.75438, 0.75633, 0.75015, 0.73324, 0.70768, 0.68235, 0.66483, 0.65123, 0.63833, 0.63021,\
        0.63302, 0.64365, 0.67582, 0.73890, 0.78498, 0.82735, 0.86129, 0.89223, 0.92506, 0.95380, 0.97692,\
        0.99472, 1.00741, 1.01513, 1.01813, 1.01686, 1.01190, 1.00392, 0.99353, 0.98131, 0.96780, 0.95351,\
        0.93898, 0.92476, 0.91140, 0.89953, 0.88938, 0.87961, 0.87052, 0.86227, 0.85481, 0.84805, 0.84189,\
        0.83620, 0.83086, 0.82575, 0.82078, 0.81583, 0.81081, 0.80562, 0.79986, 0.79329, 0.78614, 0.77838,\
        0.76999, 0.76100, 0.75153, 0.74172, 0.73174, 0.72178, 0.71211, 0.70316, 0.69532, 0.68851, 0.68252,\
        0.67703, 0.67184, 0.66684, 0.66199, 0.65725, 0.65260, 0.64803, 0.64351, 0.63901, 0.63453, 0.63002,\
        0.62546, 0.62081, 0.61601, 0.61099, 0.60566, 0.59999, 0.59405, 0.58764, 0.58073, 0.57337, 0.56559],\
        [0.54076, 0.54599, 0.55121, 0.55640, 0.56153, 0.56658, 0.57150, 0.57621, 0.58066, 0.58478, 0.58850,\
        0.59177, 0.59462, 0.59723, 0.59991, 0.60291, 0.60689, 0.61195, 0.61785, 0.62445, 0.63167, 0.63944,\
        0.64772, 0.65642, 0.66544, 0.67465, 0.68388, 0.69304, 0.70227, 0.71197, 0.72226, 0.73297, 0.74371,\
        0.75359, 0.76115, 0.76438, 0.76120, 0.75054, 0.73420, 0.71748, 0.70839, 0.70332, 0.70055, 0.69942,\
        0.70218, 0.70913, 0.73425, 0.79382, 0.82911, 0.85875, 0.88235, 0.90416, 0.92841, 0.95245, 0.97222,\
        0.98711, 0.99719, 1.00270, 1.00402, 1.00169, 0.99632, 0.98851, 0.97879, 0.96760, 0.95531, 0.94230,\
        0.92899, 0.91594, 0.90367, 0.89282, 0.88346, 0.87477, 0.86657, 0.85892, 0.85184, 0.84529, 0.83920,\
        0.83347, 0.82801, 0.82272, 0.81750, 0.81227, 0.80694, 0.80148, 0.79566, 0.78903, 0.78182, 0.77409,\
        0.76584, 0.75712, 0.74800, 0.73859, 0.72906, 0.71959, 0.71038, 0.70170, 0.69420, 0.68758, 0.68158,\
        0.67605, 0.67086, 0.66589, 0.66109, 0.65641, 0.65183, 0.64732, 0.64286, 0.63843, 0.63401, 0.62956,\
        0.62505, 0.62045, 0.61571, 0.61077, 0.60553, 0.59991, 0.59396, 0.58749, 0.58053, 0.57312, 0.56528],\
        [0.54021, 0.54550, 0.55081, 0.55613, 0.56143, 0.56669, 0.57186, 0.57687, 0.58165, 0.58614, 0.59024,\
        0.59388, 0.59701, 0.59980, 0.60259, 0.60569, 0.60944, 0.61403, 0.61942, 0.62552, 0.63225, 0.63955,\
        0.64734, 0.65553, 0.66401, 0.67266, 0.68138, 0.69020, 0.69948, 0.71013, 0.72165, 0.73370, 0.74566,\
        0.75664, 0.76550, 0.77100, 0.77213, 0.76875, 0.76220, 0.75561, 0.75236, 0.75210, 0.75240, 0.75153,\
        0.75196, 0.76648, 0.80749, 0.84901, 0.87017, 0.88489, 0.89813, 0.91421, 0.93197, 0.94998, 0.96563,\
        0.97747, 0.98509, 0.98861, 0.98842, 0.98513, 0.97943, 0.97192, 0.96305, 0.95307, 0.94219, 0.93065,\
        0.91868, 0.90693, 0.89614, 0.88635, 0.87765, 0.86966, 0.86214, 0.85503, 0.84833, 0.84201, 0.83602,\
        0.83030, 0.82475, 0.81929, 0.81385, 0.80835, 0.80273, 0.79696, 0.79097, 0.78442, 0.77733, 0.76977,\
        0.76177, 0.75338, 0.74467, 0.73576, 0.72676, 0.71786, 0.70928, 0.70127, 0.69385, 0.68715, 0.68107,\
        0.67546, 0.67020, 0.66520, 0.66038, 0.65570, 0.65114, 0.64665, 0.64222, 0.63783, 0.63343, 0.62901,\
        0.62453, 0.61997, 0.61527, 0.61042, 0.60536, 0.60003, 0.59384, 0.58721, 0.58017, 0.57271, 0.56482],\
        [0.53932, 0.54466, 0.55004, 0.55549, 0.56097, 0.56645, 0.57190, 0.57724, 0.58240, 0.58730, 0.59185,\
        0.59595, 0.59942, 0.60234, 0.60516, 0.60820, 0.61170, 0.61585, 0.62070, 0.62622, 0.63238, 0.63913,\
        0.64640, 0.65407, 0.66204, 0.67022, 0.67854, 0.68718, 0.69665, 0.70728, 0.71944, 0.73247, 0.74541,\
        0.75740, 0.76766, 0.77559, 0.78094, 0.78414, 0.78594, 0.78690, 0.78847, 0.79159, 0.79468, 0.79648,\
        0.80182, 0.82443, 0.86501, 0.88832, 0.89747, 0.90305, 0.90966, 0.91982, 0.93260, 0.94592, 0.95775,\
        0.96663, 0.97190, 0.97350, 0.97184, 0.96760, 0.96158, 0.95447, 0.94664, 0.93807, 0.92876, 0.91900,\
        0.90865, 0.89799, 0.88856, 0.87984, 0.87183, 0.86441, 0.85742, 0.85077, 0.84441, 0.83830, 0.83242,\
        0.82671, 0.82110, 0.81551, 0.80987, 0.80418, 0.79834, 0.79238, 0.78621, 0.77968, 0.77275, 0.76542,\
        0.75774, 0.74976, 0.74153, 0.73314, 0.72472, 0.71640, 0.70836, 0.70076, 0.69359, 0.68695, 0.68083,\
        0.67514, 0.66978, 0.66469, 0.65981, 0.65510, 0.65051, 0.64601, 0.64158, 0.63719, 0.63280, 0.62838,\
        0.62391, 0.61935, 0.61465, 0.60979, 0.60470, 0.59922, 0.59321, 0.58664, 0.57962, 0.57215, 0.56423],\
        [0.53808, 0.54343, 0.54888, 0.55443, 0.56008, 0.56579, 0.57153, 0.57721, 0.58276, 0.58807, 0.59305,\
        0.59759, 0.60142, 0.60449, 0.60735, 0.61028, 0.61351, 0.61722, 0.62153, 0.62647, 0.63206, 0.63826,\
        0.64503, 0.65226, 0.65985, 0.66769, 0.67576, 0.68414, 0.69313, 0.70361, 0.71646, 0.73018, 0.74367,\
        0.75628, 0.76754, 0.77717, 0.78530, 0.79252, 0.79951, 0.80452, 0.80960, 0.81643, 0.82496, 0.83686,\
        0.85845, 0.89029, 0.90577, 0.90930, 0.90931, 0.91023, 0.91347, 0.92016, 0.92954, 0.93964, 0.94859,\
        0.95500, 0.95814, 0.95793, 0.95479, 0.94953, 0.94314, 0.93650, 0.92993, 0.92305, 0.91520, 0.90718,\
        0.89936, 0.88991, 0.88119, 0.87324, 0.86589, 0.85901, 0.85249, 0.84622, 0.84015, 0.83423, 0.82844,\
        0.82275, 0.81712, 0.81143, 0.80558, 0.79971, 0.79374, 0.78762, 0.78136, 0.77487, 0.76811, 0.76107,\
        0.75377, 0.74623, 0.73852, 0.73069, 0.72283, 0.71505, 0.70746, 0.70017, 0.69330, 0.68682, 0.68070,\
        0.67494, 0.66949, 0.66430, 0.65933, 0.65454, 0.64990, 0.64537, 0.64092, 0.63651, 0.63211, 0.62769,\
        0.62319, 0.61860, 0.61387, 0.60896, 0.60379, 0.59827, 0.59232, 0.58587, 0.57890, 0.57145, 0.56352],\
        [0.53645, 0.54179, 0.54727, 0.55290, 0.55870, 0.56463, 0.57064, 0.57667, 0.58261, 0.58832, 0.59366,\
        0.59847, 0.60260, 0.60601, 0.60898, 0.61181, 0.61476, 0.61805, 0.62182, 0.62619, 0.63123, 0.63694,\
        0.64329, 0.65020, 0.65758, 0.66532, 0.67337, 0.68177, 0.69074, 0.70119, 0.71404, 0.72764, 0.74103,\
        0.75367, 0.76519, 0.77542, 0.78444, 0.79247, 0.79969, 0.80652, 0.81511, 0.82601, 0.83956, 0.85744,\
        0.88076, 0.89919, 0.90264, 0.90473, 0.90548, 0.90655, 0.90890, 0.91459, 0.92262, 0.93100, 0.93811,\
        0.94275, 0.94418, 0.94235, 0.93783, 0.93150, 0.92461, 0.91844, 0.91325, 0.90854, 0.90209, 0.89501,\
        0.88865, 0.88117, 0.87357, 0.86643, 0.85974, 0.85342, 0.84734, 0.84143, 0.83561, 0.82984, 0.82411,\
        0.81842, 0.81278, 0.80712, 0.80110, 0.79496, 0.78884, 0.78264, 0.77636, 0.76996, 0.76341, 0.75670,\
        0.74981, 0.74277, 0.73559, 0.72832, 0.72104, 0.71380, 0.70666, 0.69972, 0.69310, 0.68672, 0.68060,\
        0.67477, 0.66921, 0.66391, 0.65885, 0.65398, 0.64928, 0.64471, 0.64023, 0.63580, 0.63138, 0.62693,\
        0.62242, 0.61779, 0.61301, 0.60803, 0.60281, 0.59730, 0.59140, 0.58501, 0.57811, 0.57067, 0.56272],\
        [0.53445, 0.53972, 0.54518, 0.55086, 0.55676, 0.56287, 0.56916, 0.57554, 0.58188, 0.58802, 0.59373,\
        0.59882, 0.60327, 0.60697, 0.61004, 0.61276, 0.61541, 0.61826, 0.62151, 0.62533, 0.62986, 0.63514,\
        0.64119, 0.64793, 0.65528, 0.66314, 0.67145, 0.68025, 0.68978, 0.70056, 0.71247, 0.72516, 0.73784,\
        0.74991, 0.76103, 0.77098, 0.77969, 0.78714, 0.79349, 0.80000, 0.81078, 0.82372, 0.83884, 0.85627,\
        0.87347, 0.88403, 0.88771, 0.89057, 0.89314, 0.89555, 0.89827, 0.90450, 0.91243, 0.92011, 0.92635,\
        0.93001, 0.93026, 0.92709, 0.92149, 0.91435, 0.90681, 0.90128, 0.89715, 0.89399, 0.88940, 0.88354,\
        0.87781, 0.87174, 0.86545, 0.85927, 0.85333, 0.84758, 0.84198, 0.83642, 0.83084, 0.82519, 0.81948,\
        0.81370, 0.80796, 0.80236, 0.79638, 0.78999, 0.78367, 0.77741, 0.77116, 0.76491, 0.75863, 0.75229,\
        0.74585, 0.73932, 0.73270, 0.72600, 0.71927, 0.71258, 0.70599, 0.69943, 0.69296, 0.68660, 0.68044,\
        0.67452, 0.66887, 0.66347, 0.65830, 0.65336, 0.64860, 0.64399, 0.63948, 0.63504, 0.63061, 0.62615,\
        0.62161, 0.61696, 0.61214, 0.60712, 0.60186, 0.59638, 0.59053, 0.58419, 0.57731, 0.56987, 0.56188],\
        [0.53207, 0.53722, 0.54260, 0.54825, 0.55420, 0.56046, 0.56699, 0.57371, 0.58049, 0.58712, 0.59334,\
        0.59888, 0.60364, 0.60749, 0.61055, 0.61311, 0.61544, 0.61783, 0.62055, 0.62384, 0.62790, 0.63283,\
        0.63866, 0.64537, 0.65285, 0.66101, 0.66977, 0.67908, 0.68895, 0.69947, 0.71083, 0.72258, 0.73421,\
        0.74535, 0.75565, 0.76489, 0.77292, 0.77977, 0.78587, 0.79276, 0.80245, 0.81395, 0.82778, 0.84288,\
        0.85661, 0.86604, 0.87125, 0.87471, 0.87774, 0.88086, 0.88500, 0.89149, 0.89940, 0.90700, 0.91312,\
        0.91666, 0.91649, 0.91216, 0.90595, 0.89909, 0.89164, 0.88640, 0.88279, 0.87982, 0.87628, 0.87190,\
        0.86716, 0.86219, 0.85702, 0.85181, 0.84664, 0.84152, 0.83640, 0.83121, 0.82586, 0.82033, 0.81462,\
        0.80868, 0.80267, 0.79678, 0.79079, 0.78447, 0.77812, 0.77186, 0.76572, 0.75969, 0.75373, 0.74780,\
        0.74187, 0.73588, 0.72982, 0.72368, 0.71747, 0.71125, 0.70515, 0.69909, 0.69265, 0.68629, 0.68009,\
        0.67411, 0.66837, 0.66287, 0.65763, 0.65261, 0.64781, 0.64317, 0.63866, 0.63422, 0.62980, 0.62536,\
        0.62082, 0.61615, 0.61133, 0.60633, 0.60114, 0.59566, 0.58980, 0.58346, 0.57657, 0.56910, 0.56103],\
        [0.52932, 0.53428, 0.53950, 0.54506, 0.55099, 0.55732, 0.56404, 0.57107, 0.57828, 0.58548, 0.59238,\
        0.59860, 0.60365, 0.60748, 0.61047, 0.61286, 0.61485, 0.61675, 0.61892, 0.62168, 0.62529, 0.62993,\
        0.63565, 0.64241, 0.65013, 0.65869, 0.66797, 0.67785, 0.68811, 0.69848, 0.70913, 0.71973, 0.73017,\
        0.74023, 0.74961, 0.75805, 0.76545, 0.77191, 0.77789, 0.78436, 0.79194, 0.80091, 0.81236, 0.82522,\
        0.83740, 0.84698, 0.85329, 0.85738, 0.86063, 0.86415, 0.86894, 0.87558, 0.88348, 0.89137, 0.89792,\
        0.90197, 0.90263, 0.89746, 0.89117, 0.88472, 0.87837, 0.87332, 0.86968, 0.86675, 0.86377, 0.86039,\
        0.85667, 0.85268, 0.84849, 0.84416, 0.83974, 0.83524, 0.83064, 0.82584, 0.82073, 0.81527, 0.80962,\
        0.80363, 0.79727, 0.79108, 0.78482, 0.77847, 0.77214, 0.76597, 0.76000, 0.75424, 0.74867, 0.74322,\
        0.73783, 0.73243, 0.72695, 0.72136, 0.71562, 0.70974, 0.70372, 0.69788, 0.69186, 0.68566, 0.67949,\
        0.67346, 0.66764, 0.66207, 0.65675, 0.65169, 0.64686, 0.64222, 0.63773, 0.63333, 0.62897, 0.62457,\
        0.62008, 0.61543, 0.61060, 0.60560, 0.60055, 0.59507, 0.58922, 0.58288, 0.57594, 0.56838, 0.56020],\
        [0.52624, 0.53093, 0.53591, 0.54127, 0.54709, 0.55341, 0.56023, 0.56751, 0.57510, 0.58282, 0.59038,\
        0.59738, 0.60272, 0.60659, 0.60971, 0.61201, 0.61366, 0.61504, 0.61662, 0.61882, 0.62201, 0.62640,\
        0.63208, 0.63898, 0.64699, 0.65594, 0.66569, 0.67610, 0.68696, 0.69773, 0.70717, 0.71629, 0.72564,\
        0.73473, 0.74326, 0.75101, 0.75791, 0.76407, 0.76981, 0.77564, 0.78194, 0.78913, 0.79793, 0.80838,\
        0.81954, 0.82936, 0.83587, 0.83987, 0.84280, 0.84612, 0.85094, 0.85762, 0.86560, 0.87369, 0.88057,\
        0.88507, 0.88602, 0.88246, 0.87687, 0.87100, 0.86548, 0.86089, 0.85738, 0.85458, 0.85201, 0.84934,\
        0.84645, 0.84332, 0.83997, 0.83642, 0.83269, 0.82877, 0.82467, 0.82032, 0.81550, 0.81000, 0.80419,\
        0.79862, 0.79188, 0.78519, 0.77861, 0.77212, 0.76580, 0.75973, 0.75399, 0.74857, 0.74344, 0.73853,\
        0.73374, 0.72896, 0.72410, 0.71907, 0.71384, 0.70836, 0.70263, 0.69681, 0.69090, 0.68477, 0.67859,\
        0.67252, 0.66663, 0.66099, 0.65562, 0.65053, 0.64571, 0.64110, 0.63668, 0.63237, 0.62810, 0.62381,\
        0.61941, 0.61485, 0.61005, 0.60498, 0.59968, 0.59455, 0.58884, 0.58247, 0.57544, 0.56774, 0.55940],\
        [0.52289, 0.52722, 0.53186, 0.53692, 0.54251, 0.54870, 0.55553, 0.56295, 0.57082, 0.57893, 0.58690,\
        0.59416, 0.59988, 0.60467, 0.60832, 0.61062, 0.61191, 0.61272, 0.61367, 0.61529, 0.61805, 0.62225,\
        0.62795, 0.63505, 0.64334, 0.65260, 0.66264, 0.67326, 0.68416, 0.69458, 0.70297, 0.71184, 0.72058,\
        0.72895, 0.73682, 0.74406, 0.75061, 0.75657, 0.76216, 0.76770, 0.77353, 0.78004, 0.78760, 0.79589,\
        0.80663, 0.81532, 0.82064, 0.82353, 0.82539, 0.82782, 0.83226, 0.83893, 0.84709, 0.85541, 0.86250,\
        0.86718, 0.86856, 0.86655, 0.86237, 0.85747, 0.85274, 0.84872, 0.84557, 0.84308, 0.84093, 0.83883,\
        0.83660, 0.83420, 0.83157, 0.82869, 0.82554, 0.82210, 0.81844, 0.81458, 0.81024, 0.80495, 0.79848,\
        0.79243, 0.78580, 0.77889, 0.77205, 0.76542, 0.75910, 0.75317, 0.74769, 0.74266, 0.73804, 0.73374,\
        0.72960, 0.72550, 0.72128, 0.71683, 0.71210, 0.70707, 0.70179, 0.69604, 0.68989, 0.68365, 0.67740,\
        0.67124, 0.66528, 0.65958, 0.65418, 0.64909, 0.64430, 0.63978, 0.63547, 0.63129, 0.62719, 0.62307,\
        0.61885, 0.61445, 0.60983, 0.60497, 0.59993, 0.59459, 0.58876, 0.58228, 0.57508, 0.56718, 0.55861],\
        [0.51934, 0.52324, 0.52743, 0.53207, 0.53730, 0.54323, 0.54994, 0.55739, 0.56546, 0.57386, 0.58220,\
        0.58995, 0.59669, 0.60245, 0.60656, 0.60878, 0.60963, 0.60985, 0.61014, 0.61116, 0.61351, 0.61757,\
        0.62339, 0.63073, 0.63927, 0.64872, 0.65882, 0.66928, 0.67967, 0.68934, 0.69823, 0.70693, 0.71516,\
        0.72301, 0.73043, 0.73733, 0.74370, 0.74960, 0.75520, 0.76076, 0.76659, 0.77305, 0.78048, 0.78916,\
        0.79807, 0.80521, 0.80801, 0.80929, 0.80968, 0.81063, 0.81452, 0.82109, 0.82954, 0.83814, 0.84538,\
        0.85022, 0.85211, 0.85113, 0.84810, 0.84415, 0.84020, 0.83682, 0.83419, 0.83216, 0.83047, 0.82886,\
        0.82719, 0.82537, 0.82334, 0.82103, 0.81835, 0.81526, 0.81180, 0.80835, 0.80462, 0.79951, 0.79312,\
        0.78620, 0.77919, 0.77209, 0.76509, 0.75838, 0.75208, 0.74630, 0.74111, 0.73653, 0.73250, 0.72887,\
        0.72546, 0.72207, 0.71852, 0.71465, 0.71035, 0.70557, 0.70036, 0.69467, 0.68855, 0.68222, 0.67585,\
        0.66959, 0.66354, 0.65778, 0.65237, 0.64732, 0.64261, 0.63821, 0.63406, 0.63008, 0.62620, 0.62232,\
        0.61834, 0.61420, 0.60983, 0.60520, 0.60029, 0.59491, 0.58896, 0.58228, 0.57483, 0.56666, 0.55780],\
        [0.51572, 0.51909, 0.52273, 0.52682, 0.53154, 0.53707, 0.54354, 0.55093, 0.55911, 0.56780, 0.57661,\
        0.58512, 0.59298, 0.59972, 0.60423, 0.60634, 0.60678, 0.60646, 0.60620, 0.60665, 0.60858, 0.61260,\
        0.61866, 0.62629, 0.63504, 0.64455, 0.65454, 0.66470, 0.67470, 0.68420, 0.69320, 0.70162, 0.70948,\
        0.71701, 0.72416, 0.73090, 0.73722, 0.74317, 0.74888, 0.75460, 0.76059, 0.76715, 0.77452, 0.78266,\
        0.79080, 0.79676, 0.79729, 0.79751, 0.79717, 0.79689, 0.79990, 0.80607, 0.81472, 0.82337, 0.83056,\
        0.83543, 0.83757, 0.83713, 0.83477, 0.83144, 0.82809, 0.82535, 0.82334, 0.82187, 0.82065, 0.81947,\
        0.81823, 0.81687, 0.81533, 0.81347, 0.81116, 0.80836, 0.80485, 0.80120, 0.79806, 0.79280, 0.78639,\
        0.77937, 0.77213, 0.76488, 0.75779, 0.75103, 0.74478, 0.73918, 0.73432, 0.73023, 0.72684, 0.72397,\
        0.72135, 0.71874, 0.71587, 0.71256, 0.70866, 0.70404, 0.69878, 0.69308, 0.68688, 0.68042, 0.67390,\
        0.66751, 0.66136, 0.65556, 0.65016, 0.64517, 0.64058, 0.63635, 0.63242, 0.62871, 0.62511, 0.62153,\
        0.61786, 0.61402, 0.60993, 0.60549, 0.60059, 0.59538, 0.58934, 0.58239, 0.57462, 0.56611, 0.55692],\
        [0.51215, 0.51494, 0.51793, 0.52133, 0.52538, 0.53036, 0.53645, 0.54370, 0.55195, 0.56090, 0.57016,\
        0.57929, 0.58787, 0.59531, 0.60059, 0.60290, 0.60322, 0.60268, 0.60221, 0.60219, 0.60372, 0.60780,\
        0.61426, 0.62216, 0.63099, 0.64041, 0.65016, 0.66000, 0.66966, 0.67893, 0.68771, 0.69593, 0.70366,\
        0.71104, 0.71808, 0.72480, 0.73117, 0.73721, 0.74307, 0.74893, 0.75502, 0.76153, 0.76852, 0.77576,\
        0.78239, 0.78673, 0.78798, 0.78801, 0.78768, 0.78799, 0.79058, 0.79613, 0.80401, 0.81211, 0.81887,\
        0.82347, 0.82554, 0.82518, 0.82297, 0.81982, 0.81678, 0.81460, 0.81326, 0.81236, 0.81156, 0.81072,\
        0.80977, 0.80871, 0.80753, 0.80606, 0.80396, 0.80136, 0.79809, 0.79408, 0.79019, 0.78509, 0.77883,\
        0.77189, 0.76464, 0.75735, 0.75024, 0.74350, 0.73732, 0.73189, 0.72738, 0.72383, 0.72116, 0.71912,\
        0.71738, 0.71556, 0.71338, 0.71061, 0.70709, 0.70274, 0.69742, 0.69139, 0.68492, 0.67823, 0.67151,\
        0.66495, 0.65871, 0.65287, 0.64749, 0.64260, 0.63817, 0.63417, 0.63052, 0.62712, 0.62387, 0.62065,\
        0.61734, 0.61383, 0.61006, 0.60595, 0.60145, 0.59622, 0.58981, 0.58247, 0.57433, 0.56545, 0.55590],\
        [0.50880, 0.51098, 0.51324, 0.51582, 0.51904, 0.52328, 0.52886, 0.53590, 0.54419, 0.55336, 0.56295,\
        0.57247, 0.58141, 0.58914, 0.59487, 0.59799, 0.59895, 0.59881, 0.59864, 0.59865, 0.59981, 0.60418,\
        0.61092, 0.61881, 0.62746, 0.63660, 0.64600, 0.65545, 0.66475, 0.67374, 0.68229, 0.69035, 0.69797,\
        0.70526, 0.71228, 0.71907, 0.72552, 0.73165, 0.73760, 0.74352, 0.74957, 0.75585, 0.76230, 0.76858,\
        0.77401, 0.77773, 0.77955, 0.78025, 0.78077, 0.78208, 0.78515, 0.79040, 0.79732, 0.80446, 0.81050,\
        0.81459, 0.81631, 0.81570, 0.81325, 0.80983, 0.80669, 0.80500, 0.80428, 0.80384, 0.80332, 0.80265,\
        0.80186, 0.80094, 0.79994, 0.79872, 0.79668, 0.79406, 0.79078, 0.78681, 0.78234, 0.77707, 0.77090,\
        0.76408, 0.75692, 0.74969, 0.74262, 0.73594, 0.72986, 0.72461, 0.72042, 0.71744, 0.71558, 0.71448,\
        0.71365, 0.71265, 0.71111, 0.70878, 0.70550, 0.70120, 0.69575, 0.68941, 0.68259, 0.67559, 0.66862,\
        0.66188, 0.65552, 0.64966, 0.64433, 0.63958, 0.63537, 0.63164, 0.62833, 0.62531, 0.62246, 0.61964,\
        0.61672, 0.61357, 0.61008, 0.60618, 0.60194, 0.59665, 0.58995, 0.58232, 0.57384, 0.56459, 0.55466],\
        [0.50584, 0.50741, 0.50893, 0.51060, 0.51281, 0.51610, 0.52104, 0.52783, 0.53616, 0.54552, 0.55534,\
        0.56507, 0.57417, 0.58207, 0.58817, 0.59216, 0.59427, 0.59525, 0.59587, 0.59672, 0.59875, 0.60294,\
        0.60909, 0.61649, 0.62465, 0.63329, 0.64220, 0.65121, 0.66013, 0.66881, 0.67714, 0.68507, 0.69260,\
        0.69974, 0.70686, 0.71378, 0.72028, 0.72642, 0.73235, 0.73823, 0.74415, 0.75012, 0.75603, 0.76158,\
        0.76633, 0.76989, 0.77224, 0.77387, 0.77550, 0.77789, 0.78165, 0.78697, 0.79335, 0.79974, 0.80509,\
        0.80864, 0.80986, 0.80882, 0.80597, 0.80214, 0.79877, 0.79726, 0.79670, 0.79636, 0.79592, 0.79531,\
        0.79458, 0.79371, 0.79270, 0.79135, 0.78936, 0.78669, 0.78333, 0.77928, 0.77456, 0.76912, 0.76296,\
        0.75624, 0.74920, 0.74210, 0.73516, 0.72861, 0.72266, 0.71756, 0.71367, 0.71127, 0.71031, 0.71023,\
        0.71034, 0.71011, 0.70913, 0.70712, 0.70387, 0.69933, 0.69361, 0.68696, 0.67978, 0.67243, 0.66517,\
        0.65823, 0.65177, 0.64590, 0.64066, 0.63609, 0.63214, 0.62875, 0.62583, 0.62325, 0.62086, 0.61850,\
        0.61600, 0.61321, 0.60996, 0.60607, 0.60132, 0.59611, 0.58963, 0.58185, 0.57306, 0.56345, 0.55314],\
        [0.50341, 0.50447, 0.50528, 0.50605, 0.50711, 0.50920, 0.51335, 0.51993, 0.52834, 0.53785, 0.54779,\
        0.55763, 0.56683, 0.57493, 0.58150, 0.58637, 0.58972, 0.59203, 0.59388, 0.59585, 0.59858, 0.60274,\
        0.60834, 0.61502, 0.62248, 0.63047, 0.63881, 0.64733, 0.65585, 0.66423, 0.67237, 0.68019, 0.68769,\
        0.69493, 0.70209, 0.70900, 0.71540, 0.72140, 0.72723, 0.73300, 0.73874, 0.74442, 0.74990, 0.75498,\
        0.75940, 0.76301, 0.76588, 0.76839, 0.77108, 0.77449, 0.77901, 0.78468, 0.79102, 0.79711, 0.80208,\
        0.80521, 0.80578, 0.80420, 0.80106, 0.79710, 0.79372, 0.79165, 0.79058, 0.78992, 0.78932, 0.78865,\
        0.78787, 0.78695, 0.78580, 0.78428, 0.78219, 0.77943, 0.77598, 0.77183, 0.76697, 0.76143, 0.75527,\
        0.74863, 0.74173, 0.73481, 0.72809, 0.72177, 0.71606, 0.71116, 0.70748, 0.70563, 0.70572, 0.70670,\
        0.70765, 0.70804, 0.70747, 0.70564, 0.70231, 0.69741, 0.69118, 0.68406, 0.67643, 0.66869, 0.66113,\
        0.65398, 0.64743, 0.64158, 0.63648, 0.63213, 0.62849, 0.62550, 0.62304, 0.62095, 0.61908, 0.61722,\
        0.61518, 0.61277, 0.60982, 0.60617, 0.60174, 0.59614, 0.58923, 0.58113, 0.57200, 0.56200, 0.55130],\
        [0.50164, 0.50232, 0.50259, 0.50258, 0.50252, 0.50311, 0.50636, 0.51288, 0.52137, 0.53090, 0.54084,\
        0.55067, 0.55994, 0.56824, 0.57531, 0.58103, 0.58554, 0.58917, 0.59237, 0.59551, 0.59887, 0.60303,\
        0.60813, 0.61405, 0.62073, 0.62801, 0.63575, 0.64377, 0.65191, 0.66000, 0.66796, 0.67572, 0.68325,\
        0.69061, 0.69782, 0.70463, 0.71070, 0.71645, 0.72214, 0.72779, 0.73338, 0.73882, 0.74400, 0.74878,\
        0.75306, 0.75680, 0.76013, 0.76336, 0.76690, 0.77116, 0.77640, 0.78258, 0.78932, 0.79588, 0.80110,\
        0.80356, 0.80319, 0.80090, 0.79745, 0.79344, 0.78987, 0.78726, 0.78554, 0.78437, 0.78344, 0.78257,\
        0.78165, 0.78058, 0.77926, 0.77756, 0.77533, 0.77247, 0.76892, 0.76467, 0.75973, 0.75413, 0.74798,\
        0.74142, 0.73468, 0.72799, 0.72159, 0.71568, 0.71043, 0.70596, 0.70258, 0.70116, 0.70244, 0.70421,\
        0.70569, 0.70647, 0.70611, 0.70428, 0.70069, 0.69523, 0.68838, 0.68065, 0.67251, 0.66432, 0.65644,\
        0.64911, 0.64250, 0.63672, 0.63180, 0.62772, 0.62446, 0.62191, 0.61997, 0.61844, 0.61714, 0.61581,\
        0.61426, 0.61225, 0.60961, 0.60615, 0.60168, 0.59588, 0.58864, 0.58016, 0.57062, 0.56021, 0.54910],\
        [0.50052, 0.50102, 0.50100, 0.50045, 0.49965, 0.49899, 0.50141, 0.50767, 0.51589, 0.52517, 0.53489,\
        0.54458, 0.55382, 0.56230, 0.56980, 0.57624, 0.58174, 0.58654, 0.59097, 0.59530, 0.59964, 0.60359,\
        0.60800, 0.61316, 0.61908, 0.62571, 0.63290, 0.64047, 0.64824, 0.65608, 0.66386, 0.67151, 0.67902,\
        0.68637, 0.69353, 0.70018, 0.70581, 0.71140, 0.71703, 0.72261, 0.72808, 0.73334, 0.73831, 0.74290,\
        0.74710, 0.75095, 0.75460, 0.75832, 0.76247, 0.76733, 0.77311, 0.77976, 0.78696, 0.79401, 0.79961,\
        0.80236, 0.80113, 0.79783, 0.79402, 0.78993, 0.78620, 0.78321, 0.78099, 0.77935, 0.77806, 0.77693,\
        0.77581, 0.77457, 0.77308, 0.77122, 0.76885, 0.76588, 0.76225, 0.75794, 0.75295, 0.74734, 0.74121,\
        0.73474, 0.72817, 0.72176, 0.71577, 0.71044, 0.70595, 0.70246, 0.69993, 0.69961, 0.70102, 0.70278,\
        0.70435, 0.70524, 0.70490, 0.70283, 0.69873, 0.69260, 0.68505, 0.67666, 0.66794, 0.65930, 0.65111,\
        0.64361, 0.63701, 0.63136, 0.62668, 0.62294, 0.62009, 0.61805, 0.61668, 0.61578, 0.61508, 0.61434,\
        0.61327, 0.61166, 0.60929, 0.60594, 0.60132, 0.59533, 0.58779, 0.57888, 0.56889, 0.55803, 0.54649],\
        [0.49998, 0.50046, 0.50042, 0.49969, 0.49846, 0.49785, 0.49962, 0.50484, 0.51226, 0.52092, 0.53018,\
        0.53956, 0.54867, 0.55722, 0.56502, 0.57202, 0.57827, 0.58394, 0.58925, 0.59437, 0.59918, 0.60329,\
        0.60733, 0.61192, 0.61726, 0.62337, 0.63012, 0.63732, 0.64481, 0.65240, 0.65999, 0.66749, 0.67484,\
        0.68199, 0.68882, 0.69501, 0.70064, 0.70628, 0.71195, 0.71750, 0.72288, 0.72800, 0.73279, 0.73723,\
        0.74133, 0.74520, 0.74898, 0.75295, 0.75739, 0.76255, 0.76859, 0.77545, 0.78285, 0.79018, 0.79649,\
        0.79923, 0.79768, 0.79409, 0.78999, 0.78588, 0.78211, 0.77893, 0.77642, 0.77446, 0.77289, 0.77154,\
        0.77023, 0.76884, 0.76722, 0.76523, 0.76277, 0.75973, 0.75605, 0.75171, 0.74672, 0.74113, 0.73505,\
        0.72867, 0.72225, 0.71612, 0.71062, 0.70594, 0.70232, 0.70010, 0.69906, 0.69916, 0.70035, 0.70169,\
        0.70314, 0.70408, 0.70365, 0.70109, 0.69624, 0.68936, 0.68107, 0.67202, 0.66271, 0.65362, 0.64513,\
        0.63753, 0.63100, 0.62558, 0.62124, 0.61789, 0.61550, 0.61400, 0.61327, 0.61304, 0.61301, 0.61285,\
        0.61228, 0.61102, 0.60885, 0.60561, 0.60122, 0.59477, 0.58665, 0.57725, 0.56677, 0.55544, 0.54345],\
        [0.49982, 0.50036, 0.50039, 0.49975, 0.49859, 0.49819, 0.49985, 0.50391, 0.51025, 0.51808, 0.52672,\
        0.53566, 0.54452, 0.55302, 0.56098, 0.56832, 0.57506, 0.58127, 0.58708, 0.59253, 0.59752, 0.60185,\
        0.60581, 0.61007, 0.61506, 0.62086, 0.62734, 0.63430, 0.64155, 0.64894, 0.65632, 0.66362, 0.67075,\
        0.67762, 0.68412, 0.69014, 0.69578, 0.70146, 0.70699, 0.71252, 0.71781, 0.72277, 0.72738, 0.73164,\
        0.73559, 0.73935, 0.74307, 0.74701, 0.75142, 0.75655, 0.76251, 0.76924, 0.77641, 0.78338, 0.78903,\
        0.79173, 0.79106, 0.78825, 0.78459, 0.78078, 0.77720, 0.77408, 0.77150, 0.76941, 0.76770, 0.76622,\
        0.76481, 0.76333, 0.76163, 0.75958, 0.75707, 0.75401, 0.75033, 0.74601, 0.74107, 0.73555, 0.72956,\
        0.72328, 0.71697, 0.71108, 0.70609, 0.70207, 0.69919, 0.69769, 0.69730, 0.69774, 0.69877, 0.70004,\
        0.70145, 0.70258, 0.70227, 0.69905, 0.69322, 0.68544, 0.67639, 0.66668, 0.65683, 0.64732, 0.63858,\
        0.63095, 0.62461, 0.61955, 0.61565, 0.61275, 0.61083, 0.60991, 0.60986, 0.61037, 0.61103, 0.61147,\
        0.61136, 0.61041, 0.60834, 0.60489, 0.59992, 0.59347, 0.58509, 0.57523, 0.56424, 0.55241, 0.53996],\
        [0.49987, 0.50044, 0.50050, 0.50001, 0.49913, 0.49886, 0.50017, 0.50372, 0.50929, 0.51634, 0.52433,\
        0.53278, 0.54132, 0.54967, 0.55764, 0.56513, 0.57212, 0.57861, 0.58463, 0.59017, 0.59518, 0.59963,\
        0.60352, 0.60758, 0.61246, 0.61819, 0.62458, 0.63142, 0.63850, 0.64569, 0.65286, 0.65993, 0.66680,\
        0.67341, 0.67969, 0.68561, 0.69127, 0.69676, 0.70217, 0.70775, 0.71286, 0.71760, 0.72199, 0.72604,\
        0.72976, 0.73327, 0.73673, 0.74037, 0.74446, 0.74923, 0.75482, 0.76113, 0.76779, 0.77412, 0.77914,\
        0.78191, 0.78213, 0.78041, 0.77762, 0.77443, 0.77130, 0.76846, 0.76604, 0.76402, 0.76234, 0.76086,\
        0.75944, 0.75795, 0.75625, 0.75420, 0.75171, 0.74869, 0.74507, 0.74083, 0.73599, 0.73061, 0.72477,\
        0.71864, 0.71243, 0.70657, 0.70210, 0.69881, 0.69642, 0.69519, 0.69494, 0.69541, 0.69636, 0.69758,\
        0.69890, 0.70010, 0.70058, 0.69634, 0.68928, 0.68062, 0.67092, 0.66068, 0.65039, 0.64052, 0.63157,\
        0.62397, 0.61797, 0.61347, 0.61017, 0.60780, 0.60632, 0.60595, 0.60666, 0.60796, 0.60930, 0.61030,\
        0.61062, 0.60995, 0.60798, 0.60444, 0.59917, 0.59210, 0.58325, 0.57286, 0.56131, 0.54894, 0.53601],\
        [0.50002, 0.50058, 0.50062, 0.50026, 0.49966, 0.49942, 0.50063, 0.50387, 0.50895, 0.51542, 0.52283,\
        0.53079, 0.53895, 0.54707, 0.55495, 0.56245, 0.56952, 0.57610, 0.58215, 0.58765, 0.59255, 0.59683,\
        0.60058, 0.60464, 0.60968, 0.61554, 0.62198, 0.62876, 0.63571, 0.64271, 0.64964, 0.65645, 0.66305,\
        0.66942, 0.67550, 0.68130, 0.68690, 0.69235, 0.69787, 0.70318, 0.70788, 0.71236, 0.71655, 0.72034,\
        0.72376, 0.72691, 0.72993, 0.73305, 0.73653, 0.74068, 0.74567, 0.75142, 0.75753, 0.76333, 0.76802,\
        0.77095, 0.77191, 0.77118, 0.76934, 0.76694, 0.76442, 0.76204, 0.75996, 0.75819, 0.75668, 0.75534,\
        0.75403, 0.75263, 0.75101, 0.74905, 0.74664, 0.74371, 0.74021, 0.73610, 0.73142, 0.72622, 0.72062,\
        0.71478, 0.70887, 0.70312, 0.69853, 0.69578, 0.69364, 0.69242, 0.69205, 0.69235, 0.69311, 0.69412,\
        0.69516, 0.69591, 0.69535, 0.69105, 0.68368, 0.67467, 0.66466, 0.65414, 0.64358, 0.63345, 0.62432,\
        0.61680, 0.61129, 0.60761, 0.60512, 0.60338, 0.60232, 0.60244, 0.60402, 0.60604, 0.60795, 0.60942,\
        0.61010, 0.60968, 0.60783, 0.60424, 0.59864, 0.59088, 0.58129, 0.57020, 0.55800, 0.54503, 0.53159],\
        [0.50018, 0.50071, 0.50075, 0.50048, 0.50011, 0.49977, 0.50101, 0.50428, 0.50909, 0.51512, 0.52203,\
        0.52953, 0.53731, 0.54515, 0.55285, 0.56028, 0.56732, 0.57390, 0.57993, 0.58535, 0.59009, 0.59415,\
        0.59777, 0.60194, 0.60718, 0.61325, 0.61975, 0.62647, 0.63326, 0.64002, 0.64668, 0.65319, 0.65951,\
        0.66560, 0.67146, 0.67710, 0.68257, 0.68794, 0.69324, 0.69815, 0.70256, 0.70694, 0.71097, 0.71445,\
        0.71751, 0.72024, 0.72270, 0.72512, 0.72779, 0.73109, 0.73535, 0.74056, 0.74625, 0.75173, 0.75632,\
        0.75950, 0.76106, 0.76115, 0.76016, 0.75854, 0.75669, 0.75488, 0.75327, 0.75190, 0.75071, 0.74962,\
        0.74853, 0.74732, 0.74586, 0.74405, 0.74179, 0.73900, 0.73565, 0.73171, 0.72722, 0.72223, 0.71690,\
        0.71144, 0.70607, 0.70094, 0.69637, 0.69300, 0.69067, 0.68922, 0.68856, 0.68853, 0.68892, 0.68949,\
        0.68994, 0.68974, 0.68795, 0.68351, 0.67649, 0.66769, 0.65780, 0.64732, 0.63674, 0.62653, 0.61726,\
        0.60974, 0.60486, 0.60226, 0.60075, 0.59982, 0.59944, 0.60021, 0.60238, 0.60479, 0.60706, 0.60885,\
        0.60981, 0.60957, 0.60775, 0.60401, 0.59795, 0.58952, 0.57915, 0.56727, 0.55432, 0.54069, 0.52673],\
        [0.50021, 0.50076, 0.50084, 0.50066, 0.50041, 0.50041, 0.50190, 0.50511, 0.50966, 0.51531, 0.52178,\
        0.52884, 0.53625, 0.54379, 0.55129, 0.55859, 0.56557, 0.57213, 0.57817, 0.58359, 0.58833, 0.59242,\
        0.59614, 0.60012, 0.60552, 0.61169, 0.61814, 0.62467, 0.63120, 0.63765, 0.64398, 0.65015, 0.65613,\
        0.66191, 0.66749, 0.67288, 0.67812, 0.68322, 0.68814, 0.69274, 0.69706, 0.70138, 0.70519, 0.70819,\
        0.71095, 0.71326, 0.71513, 0.71677, 0.71848, 0.72074, 0.72423, 0.72905, 0.73452, 0.73988, 0.74453,\
        0.74798, 0.75003, 0.75075, 0.75044, 0.74950, 0.74830, 0.74711, 0.74606, 0.74519, 0.74443, 0.74370,\
        0.74291, 0.74196, 0.74073, 0.73913, 0.73706, 0.73446, 0.73128, 0.72753, 0.72322, 0.71843, 0.71327,\
        0.70805, 0.70314, 0.69832, 0.69371, 0.68999, 0.68727, 0.68545, 0.68440, 0.68393, 0.68382, 0.68380,\
        0.68350, 0.68240, 0.67982, 0.67521, 0.66851, 0.66016, 0.65069, 0.64058, 0.63031, 0.62033, 0.61114,\
        0.60352, 0.59923, 0.59766, 0.59704, 0.59702, 0.59769, 0.59935, 0.60161, 0.60407, 0.60652, 0.60853,\
        0.60968, 0.60954, 0.60768, 0.60369, 0.59713, 0.58797, 0.57677, 0.56404, 0.55028, 0.53595, 0.52144],\
        [0.49999, 0.50068, 0.50091, 0.50093, 0.50100, 0.50158, 0.50326, 0.50632, 0.51059, 0.51587, 0.52194,\
        0.52860, 0.53565, 0.54289, 0.55017, 0.55734, 0.56426, 0.57083, 0.57695, 0.58251, 0.58747, 0.59185,\
        0.59587, 0.60017, 0.60529, 0.61113, 0.61724, 0.62340, 0.62954, 0.63559, 0.64151, 0.64728, 0.65287,\
        0.65829, 0.66352, 0.66858, 0.67349, 0.67824, 0.68280, 0.68715, 0.69133, 0.69540, 0.69874, 0.70132,\
        0.70408, 0.70590, 0.70726, 0.70816, 0.70902, 0.71002, 0.71276, 0.71752, 0.72293, 0.72826, 0.73305,\
        0.73678, 0.73918, 0.74032, 0.74048, 0.74007, 0.73945, 0.73887, 0.73845, 0.73815, 0.73790, 0.73760,\
        0.73717, 0.73652, 0.73557, 0.73422, 0.73238, 0.72997, 0.72696, 0.72338, 0.71925, 0.71463, 0.70954,\
        0.70416, 0.69926, 0.69458, 0.69006, 0.68619, 0.68317, 0.68097, 0.67950, 0.67856, 0.67793, 0.67731,\
        0.67633, 0.67454, 0.67148, 0.66679, 0.66042, 0.65261, 0.64374, 0.63426, 0.62463, 0.61531, 0.60684,\
        0.59992, 0.59590, 0.59447, 0.59432, 0.59496, 0.59634, 0.59844, 0.60089, 0.60351, 0.60614, 0.60834,\
        0.60963, 0.60953, 0.60755, 0.60323, 0.59609, 0.58619, 0.57416, 0.56054, 0.54592, 0.53084, 0.51579],\
        [0.49964, 0.50051, 0.50096, 0.50129, 0.50178, 0.50280, 0.50470, 0.50767, 0.51171, 0.51666, 0.52237,\
        0.52867, 0.53538, 0.54234, 0.54940, 0.55644, 0.56332, 0.56994, 0.57620, 0.58203, 0.58737, 0.59223,\
        0.59666, 0.60106, 0.60603, 0.61136, 0.61692, 0.62256, 0.62819, 0.63375, 0.63920, 0.64451, 0.64967,\
        0.65466, 0.65949, 0.66416, 0.66867, 0.67301, 0.67718, 0.68117, 0.68498, 0.68851, 0.69149, 0.69406,\
        0.69662, 0.69787, 0.69913, 0.69926, 0.69986, 0.69960, 0.70189, 0.70684, 0.71198, 0.71721, 0.72222,\
        0.72617, 0.72877, 0.73010, 0.73051, 0.73045, 0.73030, 0.73033, 0.73057, 0.73091, 0.73121, 0.73136,\
        0.73130, 0.73099, 0.73034, 0.72926, 0.72765, 0.72542, 0.72255, 0.71908, 0.71508, 0.71062, 0.70569,\
        0.70018, 0.69482, 0.68998, 0.68546, 0.68149, 0.67824, 0.67573, 0.67388, 0.67252, 0.67141, 0.67028,\
        0.66878, 0.66655, 0.66324, 0.65861, 0.65262, 0.64541, 0.63727, 0.62859, 0.61981, 0.61146, 0.60411,\
        0.59829, 0.59451, 0.59287, 0.59272, 0.59360, 0.59526, 0.59752, 0.60003, 0.60299, 0.60582, 0.60819,\
        0.60961, 0.60951, 0.60736, 0.60259, 0.59483, 0.58422, 0.57137, 0.55688, 0.54137, 0.52550, 0.50991],\
        [0.49913, 0.50020, 0.50096, 0.50166, 0.50257, 0.50394, 0.50603, 0.50899, 0.51285, 0.51754, 0.52293,\
        0.52890, 0.53530, 0.54200, 0.54886, 0.55577, 0.56261, 0.56930, 0.57574, 0.58189, 0.58769, 0.59312,\
        0.59814, 0.60264, 0.60717, 0.61194, 0.61690, 0.62195, 0.62702, 0.63204, 0.63697, 0.64178, 0.64646,\
        0.65098, 0.65536, 0.65958, 0.66363, 0.66753, 0.67124, 0.67478, 0.67809, 0.68111, 0.68375, 0.68605,\
        0.68797, 0.68920, 0.68998, 0.69025, 0.69045, 0.69083, 0.69304, 0.69735, 0.70178, 0.70698, 0.71233,\
        0.71633, 0.71897, 0.72024, 0.72067, 0.72080, 0.72103, 0.72164, 0.72260, 0.72363, 0.72448, 0.72504,\
        0.72532, 0.72532, 0.72498, 0.72418, 0.72280, 0.72072, 0.71790, 0.71442, 0.71041, 0.70600, 0.70117,\
        0.69567, 0.68997, 0.68474, 0.68003, 0.67593, 0.67250, 0.66975, 0.66761, 0.66591, 0.66445, 0.66295,\
        0.66113, 0.65867, 0.65532, 0.65089, 0.64535, 0.63880, 0.63147, 0.62368, 0.61584, 0.60844, 0.60213,\
        0.59722, 0.59390, 0.59225, 0.59201, 0.59284, 0.59452, 0.59688, 0.59975, 0.60257, 0.60548, 0.60804,\
        0.60958, 0.60948, 0.60713, 0.60196, 0.59351, 0.58211, 0.56847, 0.55319, 0.53686, 0.52015, 0.50401],\
        [0.49858, 0.49986, 0.50094, 0.50201, 0.50328, 0.50494, 0.50720, 0.51018, 0.51391, 0.51838, 0.52350,\
        0.52917, 0.53529, 0.54174, 0.54841, 0.55519, 0.56199, 0.56873, 0.57534, 0.58176, 0.58795, 0.59384,\
        0.59928, 0.60387, 0.60811, 0.61241, 0.61683, 0.62132, 0.62583, 0.63030, 0.63469, 0.63899, 0.64316,\
        0.64720, 0.65109, 0.65482, 0.65839, 0.66179, 0.66501, 0.66803, 0.67082, 0.67334, 0.67555, 0.67742,\
        0.67891, 0.67994, 0.68057, 0.68095, 0.68139, 0.68239, 0.68469, 0.68833, 0.69265, 0.69816, 0.70325,\
        0.70710, 0.70981, 0.71076, 0.71107, 0.71132, 0.71181, 0.71302, 0.71481, 0.71653, 0.71783, 0.71869,\
        0.71921, 0.71947, 0.71943, 0.71892, 0.71776, 0.71577, 0.71291, 0.70927, 0.70503, 0.70038, 0.69532,\
        0.68975, 0.68404, 0.67865, 0.67380, 0.66957, 0.66601, 0.66311, 0.66077, 0.65887, 0.65719, 0.65550,\
        0.65354, 0.65106, 0.64784, 0.64375, 0.63874, 0.63291, 0.62644, 0.61959, 0.61271, 0.60617, 0.60050,\
        0.59654, 0.59378, 0.59225, 0.59190, 0.59256, 0.59406, 0.59622, 0.59887, 0.60185, 0.60507, 0.60789,\
        0.60956, 0.60939, 0.60676, 0.60112, 0.59193, 0.57977, 0.56544, 0.54960, 0.53276, 0.51537, 0.49862],\
        [0.49810, 0.49958, 0.50094, 0.50232, 0.50389, 0.50579, 0.50818, 0.51117, 0.51481, 0.51909, 0.52397,\
        0.52938, 0.53524, 0.54146, 0.54793, 0.55458, 0.56131, 0.56806, 0.57475, 0.58134, 0.58774, 0.59385,\
        0.59943, 0.60416, 0.60835, 0.61239, 0.61641, 0.62044, 0.62444, 0.62839, 0.63226, 0.63604, 0.63971,\
        0.64325, 0.64665, 0.64989, 0.65295, 0.65584, 0.65853, 0.66102, 0.66328, 0.66529, 0.66703, 0.66848,\
        0.66963, 0.67047, 0.67108, 0.67164, 0.67243, 0.67385, 0.67627, 0.67977, 0.68418, 0.68928, 0.69400,\
        0.69814, 0.70110, 0.70145, 0.70190, 0.70237, 0.70298, 0.70482, 0.70751, 0.70979, 0.71137, 0.71233,\
        0.71293, 0.71339, 0.71363, 0.71339, 0.71244, 0.71053, 0.70748, 0.70354, 0.69892, 0.69388, 0.68849,\
        0.68281, 0.67710, 0.67168, 0.66677, 0.66249, 0.65888, 0.65591, 0.65351, 0.65153, 0.64979, 0.64808,\
        0.64617, 0.64383, 0.64090, 0.63725, 0.63286, 0.62779, 0.62220, 0.61632, 0.61047, 0.60499, 0.60032,\
        0.59676, 0.59422, 0.59272, 0.59225, 0.59270, 0.59394, 0.59581, 0.59817, 0.60121, 0.60484, 0.60790,\
        0.60961, 0.60928, 0.60619, 0.59977, 0.58988, 0.57703, 0.56212, 0.54595, 0.52917, 0.51230, 0.49564],\
        [0.49771, 0.49938, 0.50097, 0.50261, 0.50440, 0.50648, 0.50896, 0.51195, 0.51550, 0.51961, 0.52428,\
        0.52945, 0.53506, 0.54104, 0.54731, 0.55380, 0.56043, 0.56712, 0.57382, 0.58044, 0.58689, 0.59304,\
        0.59867, 0.60353, 0.60775, 0.61169, 0.61547, 0.61914, 0.62271, 0.62618, 0.62956, 0.63286, 0.63605,\
        0.63911, 0.64202, 0.64476, 0.64733, 0.64970, 0.65186, 0.65382, 0.65557, 0.65708, 0.65837, 0.65943,\
        0.66029, 0.66098, 0.66161, 0.66234, 0.66343, 0.66513, 0.66770, 0.67117, 0.67543, 0.68014, 0.68472,\
        0.68878, 0.69158, 0.69266, 0.69343, 0.69428, 0.69540, 0.69777, 0.70091, 0.70339, 0.70507, 0.70593,\
        0.70642, 0.70703, 0.70757, 0.70751, 0.70669, 0.70493, 0.70148, 0.69716, 0.69213, 0.68668, 0.68097,\
        0.67513, 0.66937, 0.66395, 0.65905, 0.65480, 0.65122, 0.64830, 0.64594, 0.64403, 0.64238, 0.64081,\
        0.63911, 0.63707, 0.63455, 0.63143, 0.62770, 0.62341, 0.61870, 0.61377, 0.60891, 0.60446, 0.60075,\
        0.59748, 0.59501, 0.59353, 0.59297, 0.59324, 0.59427, 0.59600, 0.59841, 0.60158, 0.60517, 0.60819,\
        0.60977, 0.60917, 0.60563, 0.59841, 0.58751, 0.57376, 0.55817, 0.54167, 0.52500, 0.50867, 0.49231],\
        [0.49742, 0.49928, 0.50106, 0.50287, 0.50481, 0.50700, 0.50953, 0.51249, 0.51594, 0.51989, 0.52436,\
        0.52929, 0.53466, 0.54041, 0.54647, 0.55277, 0.55925, 0.56583, 0.57244, 0.57899, 0.58539, 0.59148,\
        0.59710, 0.60200, 0.60632, 0.61028, 0.61394, 0.61733, 0.62053, 0.62357, 0.62652, 0.62936, 0.63211,\
        0.63473, 0.63718, 0.63946, 0.64154, 0.64341, 0.64507, 0.64652, 0.64777, 0.64881, 0.64968, 0.65039,\
        0.65100, 0.65157, 0.65221, 0.65307, 0.65435, 0.65626, 0.65892, 0.66238, 0.66649, 0.67094, 0.67534,\
        0.67923, 0.68216, 0.68408, 0.68553, 0.68699, 0.68883, 0.69140, 0.69438, 0.69690, 0.69864, 0.69943,\
        0.69978, 0.70051, 0.70126, 0.70120, 0.70016, 0.69861, 0.69482, 0.69003, 0.68466, 0.67889, 0.67289,\
        0.66688, 0.66106, 0.65565, 0.65081, 0.64666, 0.64321, 0.64043, 0.63823, 0.63651, 0.63509, 0.63380,\
        0.63245, 0.63084, 0.62882, 0.62631, 0.62327, 0.61975, 0.61586, 0.61179, 0.60775, 0.60400, 0.60069,\
        0.59796, 0.59588, 0.59455, 0.59400, 0.59417, 0.59505, 0.59670, 0.59923, 0.60250, 0.60594, 0.60871,\
        0.60995, 0.60887, 0.60479, 0.59675, 0.58455, 0.56972, 0.55329, 0.53623, 0.51930, 0.50272, 0.48620],\
        [0.49718, 0.49919, 0.50112, 0.50306, 0.50510, 0.50734, 0.50988, 0.51279, 0.51612, 0.51991, 0.52416,\
        0.52887, 0.53399, 0.53950, 0.54532, 0.55141, 0.55770, 0.56412, 0.57058, 0.57700, 0.58325, 0.58922,\
        0.59472, 0.59966, 0.60416, 0.60823, 0.61182, 0.61497, 0.61782, 0.62049, 0.62304, 0.62550, 0.62786,\
        0.63008, 0.63213, 0.63398, 0.63561, 0.63702, 0.63821, 0.63919, 0.63997, 0.64058, 0.64106, 0.64146,\
        0.64185, 0.64231, 0.64293, 0.64387, 0.64527, 0.64728, 0.65002, 0.65347, 0.65749, 0.66182, 0.66611,\
        0.67001, 0.67326, 0.67583, 0.67800, 0.68010, 0.68241, 0.68506, 0.68781, 0.69020, 0.69191, 0.69287,\
        0.69345, 0.69405, 0.69456, 0.69428, 0.69299, 0.69081, 0.68711, 0.68222, 0.67663, 0.67061, 0.66441,\
        0.65824, 0.65235, 0.64696, 0.64224, 0.63826, 0.63503, 0.63248, 0.63055, 0.62911, 0.62805, 0.62717,\
        0.62628, 0.62520, 0.62376, 0.62188, 0.61954, 0.61675, 0.61363, 0.61030, 0.60696, 0.60377, 0.60091,\
        0.59861, 0.59686, 0.59575, 0.59530, 0.59545, 0.59621, 0.59773, 0.60027, 0.60361, 0.60692, 0.60934,\
        0.61003, 0.60815, 0.60295, 0.59375, 0.58063, 0.56481, 0.54743, 0.52953, 0.51212, 0.49553, 0.47903],\
        [0.49696, 0.49911, 0.50115, 0.50318, 0.50526, 0.50751, 0.51001, 0.51283, 0.51603, 0.51964, 0.52367,\
        0.52814, 0.53301, 0.53826, 0.54384, 0.54969, 0.55576, 0.56197, 0.56823, 0.57447, 0.58057, 0.58640,\
        0.59185, 0.59686, 0.60149, 0.60565, 0.60911, 0.61200, 0.61453, 0.61685, 0.61906, 0.62121, 0.62326,\
        0.62517, 0.62687, 0.62833, 0.62956, 0.63055, 0.63132, 0.63188, 0.63225, 0.63246, 0.63259, 0.63271,\
        0.63291, 0.63326, 0.63387, 0.63482, 0.63627, 0.63834, 0.64112, 0.64460, 0.64862, 0.65294, 0.65727,\
        0.66132, 0.66492, 0.66803, 0.67079, 0.67340, 0.67604, 0.67873, 0.68132, 0.68356, 0.68524, 0.68634,\
        0.68702, 0.68745, 0.68752, 0.68688, 0.68530, 0.68267, 0.67884, 0.67392, 0.66824, 0.66208, 0.65571,\
        0.64940, 0.64345, 0.63811, 0.63355, 0.62983, 0.62690, 0.62467, 0.62306, 0.62200, 0.62138, 0.62101,\
        0.62069, 0.62020, 0.61939, 0.61816, 0.61649, 0.61439, 0.61196, 0.60930, 0.60656, 0.60390, 0.60152,\
        0.59951, 0.59795, 0.59710, 0.59685, 0.59707, 0.59776, 0.59912, 0.60165, 0.60494, 0.60800, 0.60998,\
        0.61001, 0.60722, 0.60076, 0.59024, 0.57608, 0.55927, 0.54094, 0.52211, 0.50374, 0.48721, 0.47124],\
        [0.49674, 0.49903, 0.50115, 0.50319, 0.50527, 0.50749, 0.50991, 0.51262, 0.51566, 0.51907, 0.52288,\
        0.52709, 0.53170, 0.53668, 0.54199, 0.54758, 0.55340, 0.55937, 0.56542, 0.57145, 0.57737, 0.58308,\
        0.58850, 0.59360, 0.59834, 0.60251, 0.60572, 0.60833, 0.61059, 0.61260, 0.61453, 0.61645, 0.61830,\
        0.61998, 0.62139, 0.62253, 0.62340, 0.62403, 0.62445, 0.62466, 0.62468, 0.62454, 0.62435, 0.62423,\
        0.62427, 0.62455, 0.62512, 0.62606, 0.62749, 0.62956, 0.63239, 0.63594, 0.64005, 0.64448, 0.64895,\
        0.65323, 0.65717, 0.66071, 0.66393, 0.66692, 0.66977, 0.67249, 0.67498, 0.67710, 0.67873, 0.67983,\
        0.68047, 0.68068, 0.68038, 0.67935, 0.67743, 0.67450, 0.67050, 0.66553, 0.65979, 0.65354, 0.64704,\
        0.64062, 0.63459, 0.62931, 0.62498, 0.62162, 0.61909, 0.61724, 0.61599, 0.61533, 0.61522, 0.61545,\
        0.61576, 0.61590, 0.61572, 0.61513, 0.61409, 0.61263, 0.61082, 0.60874, 0.60652, 0.60430, 0.60221,\
        0.60037, 0.59908, 0.59850, 0.59857, 0.59893, 0.59961, 0.60107, 0.60346, 0.60642, 0.60907, 0.61054,\
        0.60990, 0.60622, 0.59861, 0.58671, 0.57133, 0.55349, 0.53431, 0.51499, 0.49650, 0.47952, 0.46360],\
        [0.49646, 0.49884, 0.50101, 0.50305, 0.50511, 0.50727, 0.50960, 0.51216, 0.51502, 0.51821, 0.52177,\
        0.52571, 0.53005, 0.53474, 0.53977, 0.54509, 0.55064, 0.55635, 0.56215, 0.56795, 0.57367, 0.57923,\
        0.58456, 0.58962, 0.59431, 0.59838, 0.60139, 0.60385, 0.60596, 0.60769, 0.60940, 0.61122, 0.61298,\
        0.61451, 0.61570, 0.61655, 0.61713, 0.61747, 0.61761, 0.61757, 0.61733, 0.61689, 0.61643, 0.61611,\
        0.61604, 0.61629, 0.61686, 0.61776, 0.61911, 0.62113, 0.62399, 0.62768, 0.63198, 0.63660, 0.64129,\
        0.64582, 0.65005, 0.65393, 0.65747, 0.66072, 0.66371, 0.66644, 0.66885, 0.67086, 0.67238, 0.67340,\
        0.67390, 0.67388, 0.67325, 0.67189, 0.66966, 0.66650, 0.66237, 0.65734, 0.65158, 0.64528, 0.63871,\
        0.63218, 0.62606, 0.62083, 0.61680, 0.61395, 0.61193, 0.61049, 0.60956, 0.60928, 0.60972, 0.61063,\
        0.61160, 0.61236, 0.61277, 0.61276, 0.61231, 0.61143, 0.61017, 0.60860, 0.60683, 0.60497, 0.60315,\
        0.60154, 0.60035, 0.59990, 0.60021, 0.60066, 0.60140, 0.60288, 0.60510, 0.60769, 0.60992, 0.61086,\
        0.60956, 0.60505, 0.59624, 0.58305, 0.56652, 0.54771, 0.52775, 0.50807, 0.48966, 0.47259, 0.45651],\
        [0.49608, 0.49847, 0.50066, 0.50275, 0.50479, 0.50688, 0.50909, 0.51148, 0.51412, 0.51707, 0.52036,\
        0.52402, 0.52806, 0.53246, 0.53720, 0.54223, 0.54749, 0.55293, 0.55845, 0.56400, 0.56949, 0.57484,\
        0.57998, 0.58484, 0.58928, 0.59308, 0.59610, 0.59861, 0.60068, 0.60210, 0.60371, 0.60554, 0.60732,\
        0.60879, 0.60976, 0.61037, 0.61074, 0.61086, 0.61079, 0.61065, 0.61030, 0.60960, 0.60889, 0.60848,\
        0.60835, 0.60865, 0.60929, 0.61014, 0.61134, 0.61322, 0.61613, 0.62004, 0.62461, 0.62949, 0.63440,\
        0.63916, 0.64362, 0.64774, 0.65149, 0.65488, 0.65794, 0.66065, 0.66296, 0.66484, 0.66623, 0.66710,\
        0.66742, 0.66716, 0.66626, 0.66462, 0.66216, 0.65882, 0.65460, 0.64957, 0.64384, 0.63759, 0.63104,\
        0.62445, 0.61822, 0.61299, 0.60936, 0.60719, 0.60580, 0.60476, 0.60405, 0.60405, 0.60509, 0.60670,\
        0.60828, 0.60958, 0.61051, 0.61101, 0.61108, 0.61071, 0.60994, 0.60881, 0.60743, 0.60589, 0.60433,\
        0.60292, 0.60185, 0.60137, 0.60147, 0.60189, 0.60269, 0.60409, 0.60610, 0.60839, 0.61027, 0.61077,\
        0.60878, 0.60339, 0.59349, 0.57913, 0.56164, 0.54213, 0.52156, 0.50110, 0.48308, 0.46612, 0.44992],\
        [0.49566, 0.49808, 0.50028, 0.50236, 0.50437, 0.50636, 0.50841, 0.51060, 0.51300, 0.51567, 0.51867,\
        0.52203, 0.52576, 0.52986, 0.53429, 0.53902, 0.54399, 0.54913, 0.55437, 0.55964, 0.56486, 0.56994,\
        0.57482, 0.57940, 0.58355, 0.58715, 0.59015, 0.59262, 0.59458, 0.59607, 0.59768, 0.59949, 0.60130,\
        0.60279, 0.60349, 0.60390, 0.60420, 0.60420, 0.60396, 0.60387, 0.60368, 0.60274, 0.60181, 0.60154,\
        0.60136, 0.60183, 0.60260, 0.60343, 0.60444, 0.60606, 0.60909, 0.61335, 0.61821, 0.62331, 0.62841,\
        0.63333, 0.63794, 0.64218, 0.64603, 0.64948, 0.65253, 0.65516, 0.65736, 0.65908, 0.66030, 0.66099,\
        0.66111, 0.66062, 0.65948, 0.65763, 0.65500, 0.65156, 0.64732, 0.64234, 0.63673, 0.63066, 0.62430,\
        0.61786, 0.61162, 0.60626, 0.60313, 0.60183, 0.60100, 0.60030, 0.59980, 0.59999, 0.60167, 0.60384,\
        0.60582, 0.60750, 0.60884, 0.60978, 0.61030, 0.61039, 0.61004, 0.60930, 0.60825, 0.60698, 0.60562,\
        0.60434, 0.60330, 0.60267, 0.60247, 0.60265, 0.60327, 0.60443, 0.60617, 0.60822, 0.60995, 0.61025,\
        0.60772, 0.60100, 0.59007, 0.57479, 0.55651, 0.53668, 0.51639, 0.49634, 0.47758, 0.46020, 0.44375],\
        [0.49524, 0.49770, 0.49991, 0.50195, 0.50388, 0.50574, 0.50761, 0.50956, 0.51167, 0.51404, 0.51672,\
        0.51976, 0.52317, 0.52696, 0.53108, 0.53551, 0.54018, 0.54502, 0.54996, 0.55493, 0.55984, 0.56463,\
        0.56921, 0.57349, 0.57738, 0.58078, 0.58368, 0.58609, 0.58805, 0.58972, 0.59136, 0.59308, 0.59477,\
        0.59613, 0.59677, 0.59714, 0.59743, 0.59744, 0.59720, 0.59719, 0.59716, 0.59635, 0.59559, 0.59536,\
        0.59540, 0.59596, 0.59682, 0.59773, 0.59872, 0.60010, 0.60347, 0.60799, 0.61300, 0.61823, 0.62342,\
        0.62839, 0.63304, 0.63730, 0.64115, 0.64455, 0.64752, 0.65002, 0.65206, 0.65360, 0.65462, 0.65510,\
        0.65500, 0.65431, 0.65297, 0.65095, 0.64822, 0.64474, 0.64055, 0.63569, 0.63029, 0.62453, 0.61861,\
        0.61272, 0.60705, 0.60188, 0.59920, 0.59829, 0.59754, 0.59701, 0.59691, 0.59768, 0.59963, 0.60188,\
        0.60401, 0.60593, 0.60759, 0.60891, 0.60985, 0.61033, 0.61037, 0.60996, 0.60918, 0.60812, 0.60689,\
        0.60564, 0.60452, 0.60365, 0.60310, 0.60291, 0.60313, 0.60385, 0.60516, 0.60696, 0.60872, 0.60929,\
        0.60687, 0.59963, 0.58695, 0.57003, 0.55064, 0.53050, 0.51061, 0.49113, 0.47215, 0.45442, 0.43776],\
        [0.49485, 0.49735, 0.49955, 0.50155, 0.50338, 0.50508, 0.50672, 0.50839, 0.51019, 0.51222, 0.51456,\
        0.51725, 0.52034, 0.52381, 0.52763, 0.53175, 0.53611, 0.54065, 0.54528, 0.54993, 0.55453, 0.55900,\
        0.56327, 0.56725, 0.57088, 0.57409, 0.57686, 0.57922, 0.58121, 0.58295, 0.58460, 0.58621, 0.58770,\
        0.58888, 0.58963, 0.59009, 0.59040, 0.59053, 0.59053, 0.59059, 0.59056, 0.59023, 0.58991, 0.58991,\
        0.59026, 0.59099, 0.59199, 0.59314, 0.59449, 0.59648, 0.59989, 0.60410, 0.60911, 0.61433, 0.61946,\
        0.62436, 0.62893, 0.63310, 0.63684, 0.64011, 0.64292, 0.64524, 0.64707, 0.64839, 0.64919, 0.64944,\
        0.64913, 0.64823, 0.64673, 0.64459, 0.64180, 0.63835, 0.63425, 0.62957, 0.62443, 0.61906, 0.61373,\
        0.60874, 0.60435, 0.60108, 0.59844, 0.59627, 0.59502, 0.59447, 0.59468, 0.59580, 0.59780, 0.60015,\
        0.60244, 0.60456, 0.60652, 0.60822, 0.60955, 0.61041, 0.61078, 0.61066, 0.61010, 0.60917, 0.60799,\
        0.60669, 0.60540, 0.60425, 0.60330, 0.60264, 0.60235, 0.60243, 0.60307, 0.60447, 0.60627, 0.60746,\
        0.60581, 0.59802, 0.58347, 0.56437, 0.54336, 0.52270, 0.50311, 0.48407, 0.46573, 0.44830, 0.43173],\
        [0.49447, 0.49705, 0.49927, 0.50122, 0.50293, 0.50442, 0.50579, 0.50715, 0.50859, 0.51025, 0.51222,\
        0.51456, 0.51731, 0.52046, 0.52398, 0.52780, 0.53187, 0.53609, 0.54041, 0.54474, 0.54902, 0.55316,\
        0.55712, 0.56081, 0.56418, 0.56719, 0.56983, 0.57211, 0.57408, 0.57583, 0.57744, 0.57894, 0.58029,\
        0.58139, 0.58221, 0.58279, 0.58323, 0.58354, 0.58380, 0.58405, 0.58425, 0.58438, 0.58457, 0.58499,\
        0.58572, 0.58678, 0.58813, 0.58972, 0.59165, 0.59419, 0.59749, 0.60170, 0.60657, 0.61157, 0.61649,\
        0.62119, 0.62557, 0.62954, 0.63308, 0.63615, 0.63873, 0.64081, 0.64239, 0.64346, 0.64401, 0.64402,\
        0.64348, 0.64239, 0.64074, 0.63851, 0.63571, 0.63232, 0.62835, 0.62386, 0.61897, 0.61396, 0.60915,\
        0.60497, 0.60164, 0.59884, 0.59638, 0.59405, 0.59256, 0.59201, 0.59238, 0.59366, 0.59571, 0.59819,\
        0.60071, 0.60305, 0.60539, 0.60752, 0.60924, 0.61046, 0.61113, 0.61126, 0.61086, 0.61000, 0.60880,\
        0.60739, 0.60589, 0.60441, 0.60305, 0.60189, 0.60108, 0.60049, 0.60025, 0.60099, 0.60235, 0.60382,\
        0.60380, 0.59566, 0.57864, 0.55731, 0.53437, 0.51281, 0.49417, 0.47603, 0.45863, 0.44185, 0.42563],\
        [0.49409, 0.49679, 0.49908, 0.50098, 0.50252, 0.50378, 0.50486, 0.50587, 0.50693, 0.50818, 0.50975,\
        0.51173, 0.51415, 0.51699, 0.52021, 0.52375, 0.52751, 0.53144, 0.53544, 0.53944, 0.54339, 0.54720,\
        0.55084, 0.55424, 0.55736, 0.56017, 0.56265, 0.56483, 0.56675, 0.56845, 0.57000, 0.57140, 0.57266,\
        0.57373, 0.57462, 0.57534, 0.57596, 0.57651, 0.57705, 0.57760, 0.57817, 0.57877, 0.57950, 0.58044,\
        0.58167, 0.58320, 0.58504, 0.58720, 0.58974, 0.59279, 0.59644, 0.60067, 0.60513, 0.60976, 0.61436,\
        0.61877, 0.62286, 0.62656, 0.62983, 0.63262, 0.63491, 0.63671, 0.63800, 0.63878, 0.63905, 0.63881,\
        0.63804, 0.63674, 0.63493, 0.63263, 0.62983, 0.62655, 0.62275, 0.61844, 0.61373, 0.60894, 0.60442,\
        0.60056, 0.59773, 0.59511, 0.59275, 0.59075, 0.58945, 0.58904, 0.58957, 0.59098, 0.59312, 0.59572,\
        0.59843, 0.60117, 0.60405, 0.60665, 0.60874, 0.61028, 0.61125, 0.61160, 0.61132, 0.61047, 0.60918,\
        0.60761, 0.60587, 0.60408, 0.60236, 0.60075, 0.59940, 0.59832, 0.59749, 0.59731, 0.59762, 0.59791,\
        0.59719, 0.58866, 0.57104, 0.54907, 0.52517, 0.50182, 0.48430, 0.46791, 0.45157, 0.43546, 0.41963],\
        [0.49366, 0.49648, 0.49884, 0.50071, 0.50212, 0.50318, 0.50398, 0.50463, 0.50527, 0.50608, 0.50723,\
        0.50884, 0.51094, 0.51348, 0.51642, 0.51966, 0.52314, 0.52676, 0.53044, 0.53412, 0.53772, 0.54121,\
        0.54452, 0.54763, 0.55050, 0.55309, 0.55541, 0.55747, 0.55929, 0.56091, 0.56238, 0.56371, 0.56491,\
        0.56598, 0.56694, 0.56781, 0.56864, 0.56947, 0.57033, 0.57126, 0.57226, 0.57338, 0.57467, 0.57618,\
        0.57797, 0.58007, 0.58248, 0.58524, 0.58837, 0.59192, 0.59583, 0.59997, 0.60424, 0.60859, 0.61286,\
        0.61693, 0.62069, 0.62407, 0.62700, 0.62946, 0.63142, 0.63288, 0.63384, 0.63431, 0.63429, 0.63377,\
        0.63275, 0.63124, 0.62925, 0.62684, 0.62405, 0.62090, 0.61731, 0.61318, 0.60856, 0.60383, 0.59945,\
        0.59574, 0.59278, 0.59028, 0.58812, 0.58644, 0.58543, 0.58526, 0.58599, 0.58758, 0.58989, 0.59270,\
        0.59579, 0.59905, 0.60247, 0.60546, 0.60781, 0.60965, 0.61095, 0.61153, 0.61133, 0.61042, 0.60897,\
        0.60720, 0.60525, 0.60315, 0.60103, 0.59924, 0.59739, 0.59575, 0.59437, 0.59328, 0.59227, 0.59068,\
        0.58677, 0.57717, 0.56092, 0.54046, 0.51829, 0.49665, 0.47797, 0.46134, 0.44530, 0.42953, 0.41395],\
        [0.49312, 0.49607, 0.49850, 0.50039, 0.50173, 0.50263, 0.50318, 0.50349, 0.50370, 0.50404, 0.50474,\
        0.50599, 0.50778, 0.51004, 0.51269, 0.51565, 0.51884, 0.52216, 0.52552, 0.52885, 0.53210, 0.53524,\
        0.53824, 0.54105, 0.54366, 0.54604, 0.54817, 0.55008, 0.55178, 0.55330, 0.55468, 0.55593, 0.55710,\
        0.55819, 0.55923, 0.56026, 0.56132, 0.56244, 0.56366, 0.56500, 0.56648, 0.56813, 0.57000, 0.57211,\
        0.57450, 0.57720, 0.58021, 0.58356, 0.58725, 0.59127, 0.59552, 0.59971, 0.60379, 0.60781, 0.61175,\
        0.61549, 0.61891, 0.62193, 0.62450, 0.62659, 0.62818, 0.62927, 0.62988, 0.63000, 0.62967, 0.62887,\
        0.62759, 0.62584, 0.62363, 0.62105, 0.61821, 0.61517, 0.61185, 0.60796, 0.60333, 0.59852, 0.59415,\
        0.59037, 0.58724, 0.58469, 0.58266, 0.58121, 0.58048, 0.58056, 0.58152, 0.58332, 0.58586, 0.58899,\
        0.59257, 0.59648, 0.60046, 0.60357, 0.60608, 0.60829, 0.61003, 0.61091, 0.61074, 0.60965, 0.60794,\
        0.60594, 0.60386, 0.60155, 0.59907, 0.59680, 0.59461, 0.59248, 0.59043, 0.58833, 0.58584, 0.58215,\
        0.57585, 0.56528, 0.55003, 0.53151, 0.51200, 0.49245, 0.47344, 0.45619, 0.43995, 0.42418, 0.40866],\
        [0.49245, 0.49556, 0.49809, 0.50003, 0.50136, 0.50217, 0.50253, 0.50254, 0.50235, 0.50218, 0.50241,\
        0.50333, 0.50483, 0.50678, 0.50913, 0.51181, 0.51471, 0.51772, 0.52075, 0.52372, 0.52660, 0.52938,\
        0.53204, 0.53456, 0.53691, 0.53906, 0.54100, 0.54274, 0.54430, 0.54570, 0.54698, 0.54816, 0.54929,\
        0.55040, 0.55152, 0.55272, 0.55401, 0.55544, 0.55704, 0.55881, 0.56079, 0.56298, 0.56543, 0.56814,\
        0.57113, 0.57443, 0.57803, 0.58194, 0.58612, 0.59054, 0.59509, 0.59952, 0.60330, 0.60713, 0.61085,\
        0.61429, 0.61738, 0.62004, 0.62223, 0.62393, 0.62512, 0.62581, 0.62603, 0.62579, 0.62513, 0.62405,\
        0.62252, 0.62050, 0.61802, 0.61517, 0.61215, 0.60913, 0.60605, 0.60248, 0.59783, 0.59287, 0.58832,\
        0.58436, 0.58106, 0.57842, 0.57643, 0.57514, 0.57461, 0.57492, 0.57608, 0.57809, 0.58086, 0.58430,\
        0.58826, 0.59258, 0.59682, 0.60021, 0.60315, 0.60594, 0.60839, 0.60961, 0.60935, 0.60796, 0.60588,\
        0.60352, 0.60134, 0.59900, 0.59623, 0.59355, 0.59094, 0.58831, 0.58556, 0.58246, 0.57857, 0.57318,\
        0.56532, 0.55411, 0.53941, 0.52211, 0.50378, 0.48605, 0.46837, 0.45139, 0.43512, 0.41931, 0.40375],\
        [0.49161, 0.49492, 0.49761, 0.49963, 0.50105, 0.50184, 0.50207, 0.50186, 0.50136, 0.50075, 0.50044,\
        0.50111, 0.50226, 0.50382, 0.50587, 0.50824, 0.51084, 0.51353, 0.51621, 0.51879, 0.52126, 0.52365,\
        0.52597, 0.52820, 0.53029, 0.53222, 0.53395, 0.53551, 0.53691, 0.53818, 0.53935, 0.54046, 0.54155,\
        0.54266, 0.54386, 0.54521, 0.54674, 0.54848, 0.55046, 0.55267, 0.55514, 0.55788, 0.56089, 0.56419,\
        0.56777, 0.57165, 0.57580, 0.58020, 0.58481, 0.58952, 0.59414, 0.59838, 0.60248, 0.60642, 0.61000,\
        0.61319, 0.61595, 0.61826, 0.62007, 0.62137, 0.62216, 0.62243, 0.62223, 0.62160, 0.62060, 0.61926,\
        0.61750, 0.61524, 0.61243, 0.60920, 0.60582, 0.60261, 0.59948, 0.59591, 0.59134, 0.58642, 0.58175,\
        0.57764, 0.57419, 0.57145, 0.56946, 0.56826, 0.56787, 0.56833, 0.56966, 0.57183, 0.57478, 0.57842,\
        0.58258, 0.58706, 0.59145, 0.59542, 0.59883, 0.60259, 0.60604, 0.60749, 0.60688, 0.60504, 0.60259,\
        0.59984, 0.59737, 0.59490, 0.59210, 0.58920, 0.58626, 0.58319, 0.57982, 0.57587, 0.57089, 0.56432,\
        0.55551, 0.54399, 0.52976, 0.51343, 0.49622, 0.47953, 0.46300, 0.44662, 0.43054, 0.41473, 0.39910],\
        [0.49054, 0.49411, 0.49705, 0.49931, 0.50083, 0.50162, 0.50178, 0.50142, 0.50075, 0.50000, 0.49938,\
        0.49958, 0.50019, 0.50127, 0.50302, 0.50502, 0.50728, 0.50968, 0.51198, 0.51411, 0.51612, 0.51809,\
        0.52006, 0.52201, 0.52387, 0.52556, 0.52708, 0.52844, 0.52967, 0.53081, 0.53187, 0.53290, 0.53392,\
        0.53502, 0.53628, 0.53777, 0.53953, 0.54158, 0.54393, 0.54658, 0.54953, 0.55279, 0.55634, 0.56020,\
        0.56433, 0.56874, 0.57340, 0.57826, 0.58325, 0.58825, 0.59312, 0.59772, 0.60194, 0.60571, 0.60911,\
        0.61206, 0.61450, 0.61646, 0.61791, 0.61883, 0.61921, 0.61905, 0.61840, 0.61735, 0.61599, 0.61440,\
        0.61249, 0.61005, 0.60693, 0.60330, 0.59942, 0.59590, 0.59238, 0.58847, 0.58394, 0.57911, 0.57442,\
        0.57019, 0.56661, 0.56378, 0.56177, 0.56060, 0.56030, 0.56086, 0.56229, 0.56455, 0.56760, 0.57132,\
        0.57558, 0.58017, 0.58482, 0.58935, 0.59378, 0.59846, 0.60281, 0.60401, 0.60271, 0.60046, 0.59779,\
        0.59494, 0.59219, 0.58949, 0.58665, 0.58367, 0.58054, 0.57717, 0.57336, 0.56880, 0.56311, 0.55582,\
        0.54653, 0.53502, 0.52137, 0.50607, 0.49004, 0.47399, 0.45800, 0.44202, 0.42612, 0.41033, 0.39461],\
        [0.48916, 0.49305, 0.49628, 0.49879, 0.50050, 0.50142, 0.50158, 0.50112, 0.50032, 0.49942, 0.49872,\
        0.49845, 0.49862, 0.49938, 0.50067, 0.50212, 0.50409, 0.50623, 0.50812, 0.50971, 0.51118, 0.51270,\
        0.51433, 0.51604, 0.51769, 0.51916, 0.52044, 0.52157, 0.52263, 0.52363, 0.52460, 0.52553, 0.52646,\
        0.52751, 0.52880, 0.53042, 0.53241, 0.53475, 0.53746, 0.54052, 0.54393, 0.54768, 0.55175, 0.55611,\
        0.56076, 0.56565, 0.57076, 0.57604, 0.58139, 0.58671, 0.59186, 0.59670, 0.60097, 0.60475, 0.60805,\
        0.61075, 0.61287, 0.61449, 0.61561, 0.61620, 0.61620, 0.61562, 0.61451, 0.61297, 0.61119, 0.60936,\
        0.60739, 0.60488, 0.60145, 0.59747, 0.59331, 0.58923, 0.58514, 0.58080, 0.57609, 0.57120, 0.56644,\
        0.56209, 0.55839, 0.55547, 0.55342, 0.55226, 0.55199, 0.55259, 0.55404, 0.55631, 0.55936, 0.56308,\
        0.56735, 0.57200, 0.57687, 0.58184, 0.58695, 0.59219, 0.59664, 0.59761, 0.59608, 0.59380, 0.59116,\
        0.58838, 0.58560, 0.58283, 0.57999, 0.57700, 0.57382, 0.57032, 0.56629, 0.56144, 0.55540, 0.54779,\
        0.53836, 0.52707, 0.51408, 0.49966, 0.48452, 0.46903, 0.45338, 0.43762, 0.42181, 0.40599, 0.39017],\
        [0.48737, 0.49159, 0.49518, 0.49803, 0.50003, 0.50117, 0.50135, 0.50077, 0.49984, 0.49886, 0.49801,\
        0.49749, 0.49739, 0.49778, 0.49854, 0.49958, 0.50135, 0.50329, 0.50463, 0.50555, 0.50642, 0.50748,\
        0.50879, 0.51032, 0.51181, 0.51305, 0.51405, 0.51494, 0.51581, 0.51671, 0.51760, 0.51845, 0.51925,\
        0.52018, 0.52147, 0.52322, 0.52541, 0.52803, 0.53106, 0.53450, 0.53834, 0.54254, 0.54708, 0.55191,\
        0.55700, 0.56232, 0.56783, 0.57346, 0.57914, 0.58477, 0.59018, 0.59517, 0.59951, 0.60354, 0.60673,\
        0.60910, 0.61088, 0.61219, 0.61305, 0.61336, 0.61305, 0.61209, 0.61052, 0.60846, 0.60616, 0.60401,\
        0.60194, 0.59952, 0.59577, 0.59149, 0.58701, 0.58247, 0.57786, 0.57306, 0.56806, 0.56296, 0.55802,\
        0.55349, 0.54963, 0.54661, 0.54451, 0.54334, 0.54306, 0.54364, 0.54504, 0.54724, 0.55018, 0.55381,\
        0.55800, 0.56260, 0.56747, 0.57252, 0.57763, 0.58248, 0.58615, 0.58745, 0.58667, 0.58486, 0.58259,\
        0.58009, 0.57751, 0.57487, 0.57214, 0.56926, 0.56615, 0.56270, 0.55870, 0.55387, 0.54785, 0.54027,\
        0.53091, 0.51987, 0.50757, 0.49401, 0.47944, 0.46437, 0.44896, 0.43332, 0.41752, 0.40163, 0.38567],\
        [0.48502, 0.48960, 0.49358, 0.49690, 0.49939, 0.50078, 0.50082, 0.50020, 0.49913, 0.49805, 0.49708,\
        0.49639, 0.49608, 0.49617, 0.49665, 0.49757, 0.49909, 0.50068, 0.50130, 0.50153, 0.50181, 0.50247,\
        0.50347, 0.50492, 0.50630, 0.50725, 0.50793, 0.50856, 0.50925, 0.51007, 0.51095, 0.51175, 0.51236,\
        0.51307, 0.51432, 0.51622, 0.51862, 0.52147, 0.52478, 0.52856, 0.53278, 0.53739, 0.54233, 0.54756,\
        0.55303, 0.55871, 0.56454, 0.57047, 0.57642, 0.58231, 0.58800, 0.59338, 0.59835, 0.60221, 0.60491,\
        0.60686, 0.60828, 0.60936, 0.61006, 0.61020, 0.60967, 0.60839, 0.60643, 0.60394, 0.60100, 0.59853,\
        0.59612, 0.59329, 0.58955, 0.58516, 0.58045, 0.57556, 0.57052, 0.56532, 0.55997, 0.55458, 0.54936,\
        0.54458, 0.54050, 0.53735, 0.53519, 0.53400, 0.53370, 0.53420, 0.53547, 0.53749, 0.54024, 0.54367,\
        0.54767, 0.55206, 0.55671, 0.56146, 0.56613, 0.57032, 0.57344, 0.57495, 0.57492, 0.57387, 0.57221,\
        0.57021, 0.56801, 0.56566, 0.56317, 0.56049, 0.55759, 0.55436, 0.55062, 0.54612, 0.54049, 0.53331,\
        0.52420, 0.51322, 0.50121, 0.48843, 0.47453, 0.45983, 0.44458, 0.42897, 0.41312, 0.39711, 0.38098],\
        [0.48200, 0.48687, 0.49119, 0.49491, 0.49790, 0.49945, 0.49940, 0.49895, 0.49787, 0.49673, 0.49571,\
        0.49493, 0.49446, 0.49437, 0.49467, 0.49539, 0.49646, 0.49741, 0.49766, 0.49755, 0.49751, 0.49784,\
        0.49855, 0.49987, 0.50116, 0.50169, 0.50205, 0.50246, 0.50296, 0.50375, 0.50469, 0.50554, 0.50588,\
        0.50621, 0.50745, 0.50956, 0.51213, 0.51512, 0.51863, 0.52271, 0.52727, 0.53223, 0.53751, 0.54307,\
        0.54884, 0.55479, 0.56086, 0.56699, 0.57313, 0.57918, 0.58505, 0.59061, 0.59559, 0.59943, 0.60210,\
        0.60372, 0.60484, 0.60582, 0.60653, 0.60659, 0.60589, 0.60442, 0.60209, 0.59933, 0.59620, 0.59316,\
        0.59020, 0.58690, 0.58299, 0.57851, 0.57364, 0.56850, 0.56317, 0.55764, 0.55197, 0.54625, 0.54068,\
        0.53554, 0.53117, 0.52786, 0.52567, 0.52447, 0.52411, 0.52448, 0.52553, 0.52727, 0.52972, 0.53288,\
        0.53660, 0.54065, 0.54486, 0.54909, 0.55313, 0.55668, 0.55940, 0.56105, 0.56162, 0.56132, 0.56038,\
        0.55900, 0.55730, 0.55535, 0.55317, 0.55079, 0.54817, 0.54527, 0.54198, 0.53808, 0.53322, 0.52690,\
        0.51854, 0.50782, 0.49573, 0.48339, 0.46983, 0.45528, 0.44006, 0.42440, 0.40844, 0.39229, 0.37600],\
        [0.47821, 0.48327, 0.48778, 0.49163, 0.49461, 0.49630, 0.49674, 0.49645, 0.49565, 0.49466, 0.49370,\
        0.49291, 0.49237, 0.49215, 0.49225, 0.49265, 0.49323, 0.49366, 0.49372, 0.49353, 0.49339, 0.49353,\
        0.49406, 0.49498, 0.49586, 0.49623, 0.49643, 0.49670, 0.49710, 0.49783, 0.49875, 0.49976, 0.49984,\
        0.49982, 0.50112, 0.50348, 0.50604, 0.50901, 0.51267, 0.51700, 0.52185, 0.52710, 0.53265, 0.53844,\
        0.54442, 0.55055, 0.55676, 0.56301, 0.56922, 0.57531, 0.58116, 0.58661, 0.59140, 0.59523, 0.59795,\
        0.59949, 0.60050, 0.60150, 0.60238, 0.60235, 0.60144, 0.59995, 0.59733, 0.59430, 0.59102, 0.58763,\
        0.58417, 0.58045, 0.57629, 0.57167, 0.56667, 0.56138, 0.55584, 0.55009, 0.54417, 0.53816, 0.53221,\
        0.52661, 0.52186, 0.51837, 0.51617, 0.51499, 0.51456, 0.51473, 0.51549, 0.51683, 0.51886, 0.52170,\
        0.52510, 0.52869, 0.53233, 0.53591, 0.53928, 0.54221, 0.54457, 0.54631, 0.54738, 0.54776, 0.54754,\
        0.54680, 0.54563, 0.54412, 0.54232, 0.54025, 0.53796, 0.53543, 0.53265, 0.52950, 0.52570, 0.52073,\
        0.51375, 0.50415, 0.49223, 0.47919, 0.46527, 0.45054, 0.43519, 0.41942, 0.40333, 0.38704, 0.37058],\
        [0.47360, 0.47876, 0.48333, 0.48718, 0.49014, 0.49202, 0.49286, 0.49292, 0.49246, 0.49173, 0.49094,\
        0.49022, 0.48968, 0.48936, 0.48928, 0.48940, 0.48959, 0.48969, 0.48960, 0.48937, 0.48920, 0.48922,\
        0.48953, 0.49006, 0.49057, 0.49086, 0.49103, 0.49125, 0.49162, 0.49224, 0.49303, 0.49377, 0.49406,\
        0.49440, 0.49572, 0.49790, 0.50031, 0.50316, 0.50698, 0.51151, 0.51659, 0.52204, 0.52777, 0.53370,\
        0.53979, 0.54599, 0.55225, 0.55851, 0.56469, 0.57070, 0.57641, 0.58166, 0.58624, 0.58994, 0.59263,\
        0.59437, 0.59555, 0.59655, 0.59734, 0.59720, 0.59611, 0.59440, 0.59184, 0.58872, 0.58530, 0.58169,\
        0.57790, 0.57386, 0.56947, 0.56472, 0.55962, 0.55423, 0.54859, 0.54274, 0.53670, 0.53051, 0.52426,\
        0.51815, 0.51285, 0.50917, 0.50702, 0.50588, 0.50530, 0.50519, 0.50562, 0.50649, 0.50792, 0.51047,\
        0.51356, 0.51657, 0.51951, 0.52242, 0.52514, 0.52744, 0.52940, 0.53120, 0.53271, 0.53370, 0.53411,\
        0.53396, 0.53331, 0.53223, 0.53079, 0.52903, 0.52703, 0.52484, 0.52252, 0.52006, 0.51734, 0.51391,\
        0.50863, 0.49995, 0.48806, 0.47454, 0.46019, 0.44520, 0.42970, 0.41380, 0.39761, 0.38119, 0.36461],\
        [0.46820, 0.47341, 0.47799, 0.48183, 0.48483, 0.48690, 0.48806, 0.48850, 0.48840, 0.48797, 0.48740,\
        0.48682, 0.48633, 0.48597, 0.48576, 0.48566, 0.48561, 0.48550, 0.48530, 0.48504, 0.48483, 0.48476,\
        0.48487, 0.48512, 0.48539, 0.48558, 0.48574, 0.48595, 0.48629, 0.48681, 0.48745, 0.48807, 0.48860,\
        0.48932, 0.49065, 0.49260, 0.49496, 0.49788, 0.50173, 0.50634, 0.51157, 0.51710, 0.52287, 0.52883,\
        0.53493, 0.54112, 0.54734, 0.55353, 0.55959, 0.56544, 0.57094, 0.57596, 0.58033, 0.58391, 0.58661,\
        0.58852, 0.58986, 0.59081, 0.59130, 0.59104, 0.58994, 0.58811, 0.58559, 0.58250, 0.57903, 0.57529,\
        0.57131, 0.56706, 0.56252, 0.55766, 0.55251, 0.54708, 0.54143, 0.53559, 0.52959, 0.52345, 0.51717,\
        0.51073, 0.50462, 0.50079, 0.49862, 0.49734, 0.49649, 0.49605, 0.49610, 0.49653, 0.49739, 0.49968,\
        0.50230, 0.50458, 0.50675, 0.50904, 0.51127, 0.51285, 0.51428, 0.51618, 0.51811, 0.51959, 0.52049,\
        0.52082, 0.52061, 0.51992, 0.51879, 0.51731, 0.51554, 0.51360, 0.51159, 0.50960, 0.50757, 0.50530,\
        0.50162, 0.49329, 0.48163, 0.46824, 0.45386, 0.43882, 0.42328, 0.40734, 0.39110, 0.37463, 0.35796],\
        [0.46208, 0.46730, 0.47188, 0.47575, 0.47883, 0.48108, 0.48253, 0.48331, 0.48356, 0.48345, 0.48313,\
        0.48272, 0.48232, 0.48197, 0.48170, 0.48148, 0.48128, 0.48105, 0.48078, 0.48049, 0.48025, 0.48010,\
        0.48008, 0.48015, 0.48026, 0.48038, 0.48051, 0.48072, 0.48104, 0.48149, 0.48206, 0.48269, 0.48340,\
        0.48437, 0.48578, 0.48770, 0.49013, 0.49318, 0.49696, 0.50168, 0.50681, 0.51225, 0.51795, 0.52384,\
        0.52986, 0.53596, 0.54206, 0.54810, 0.55398, 0.55962, 0.56489, 0.56968, 0.57385, 0.57730, 0.58000,\
        0.58199, 0.58338, 0.58425, 0.58456, 0.58418, 0.58305, 0.58121, 0.57872, 0.57569, 0.57223, 0.56844,\
        0.56437, 0.56001, 0.55538, 0.55048, 0.54531, 0.53991, 0.53432, 0.52858, 0.52275, 0.51686, 0.51097,\
        0.50506, 0.49871, 0.49414, 0.49136, 0.48951, 0.48823, 0.48741, 0.48704, 0.48712, 0.48780, 0.48934,\
        0.49120, 0.49289, 0.49447, 0.49620, 0.49805, 0.49894, 0.49985, 0.50188, 0.50405, 0.50579, 0.50700,\
        0.50766, 0.50778, 0.50739, 0.50655, 0.50530, 0.50373, 0.50199, 0.50018, 0.49845, 0.49664, 0.49453,\
        0.49081, 0.48334, 0.47260, 0.45987, 0.44589, 0.43110, 0.41571, 0.39988, 0.38370, 0.36724, 0.35056]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=60 (m/s) and translational velocity of Vt=7.5 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax60vt7.5
    zgrid_H_Hmax_vmax60vt7_5=[[0.32193, 0.32579, 0.32933, 0.33256, 0.33550, 0.33818, 0.34061, 0.34282, 0.34482, 0.34664, 0.34830,\
        0.34983, 0.35124, 0.35255, 0.35379, 0.35497, 0.35611, 0.35723, 0.35834, 0.35945, 0.36059, 0.36177,\
        0.36300, 0.36428, 0.36564, 0.36708, 0.36861, 0.37024, 0.37198, 0.37384, 0.37582, 0.37793, 0.38018,\
        0.38258, 0.38514, 0.38788, 0.39082, 0.39398, 0.39738, 0.40087, 0.40435, 0.40817, 0.41236, 0.41687,\
        0.42164, 0.42661, 0.43172, 0.43694, 0.44222, 0.44753, 0.45285, 0.45814, 0.46342, 0.46868, 0.47393,\
        0.47921, 0.48453, 0.48990, 0.49524, 0.50012, 0.50404, 0.50773, 0.51139, 0.51503, 0.51860, 0.52207,\
        0.52538, 0.52850, 0.53141, 0.53409, 0.53652, 0.53871, 0.54066, 0.54237, 0.54386, 0.54514, 0.54622,\
        0.54709, 0.54777, 0.54826, 0.54856, 0.54869, 0.54863, 0.54841, 0.54801, 0.54745, 0.54672, 0.54583,\
        0.54476, 0.54352, 0.54209, 0.54048, 0.53869, 0.53671, 0.53454, 0.53220, 0.52970, 0.52703, 0.52422,\
        0.52128, 0.51822, 0.51507, 0.51184, 0.50857, 0.50531, 0.50212, 0.49911, 0.49658, 0.49495, 0.49295,\
        0.48985, 0.48557, 0.48033, 0.47445, 0.46809, 0.46131, 0.45412, 0.44653, 0.43851, 0.43005, 0.42113],\
        [0.32821, 0.33189, 0.33524, 0.33827, 0.34100, 0.34347, 0.34568, 0.34766, 0.34944, 0.35104, 0.35249,\
        0.35379, 0.35499, 0.35609, 0.35713, 0.35811, 0.35905, 0.35998, 0.36091, 0.36185, 0.36283, 0.36384,\
        0.36491, 0.36605, 0.36727, 0.36857, 0.36997, 0.37148, 0.37310, 0.37484, 0.37670, 0.37870, 0.38083,\
        0.38310, 0.38551, 0.38809, 0.39085, 0.39381, 0.39700, 0.40034, 0.40395, 0.40800, 0.41244, 0.41718,\
        0.42217, 0.42733, 0.43263, 0.43800, 0.44341, 0.44881, 0.45419, 0.45950, 0.46475, 0.46993, 0.47505,\
        0.48015, 0.48528, 0.49044, 0.49559, 0.50041, 0.50481, 0.50906, 0.51325, 0.51740, 0.52144, 0.52533,\
        0.52902, 0.53247, 0.53566, 0.53856, 0.54119, 0.54354, 0.54562, 0.54744, 0.54903, 0.55040, 0.55154,\
        0.55249, 0.55322, 0.55377, 0.55412, 0.55428, 0.55427, 0.55408, 0.55372, 0.55319, 0.55251, 0.55165,\
        0.55062, 0.54942, 0.54803, 0.54645, 0.54469, 0.54273, 0.54058, 0.53825, 0.53575, 0.53309, 0.53029,\
        0.52736, 0.52431, 0.52117, 0.51797, 0.51473, 0.51150, 0.50835, 0.50538, 0.50290, 0.50163, 0.49973,\
        0.49676, 0.49226, 0.48681, 0.48087, 0.47454, 0.46783, 0.46076, 0.45328, 0.44536, 0.43699, 0.42815],\
        [0.33431, 0.33780, 0.34095, 0.34378, 0.34630, 0.34854, 0.35053, 0.35229, 0.35385, 0.35522, 0.35644,\
        0.35753, 0.35851, 0.35941, 0.36024, 0.36102, 0.36177, 0.36252, 0.36327, 0.36404, 0.36484, 0.36570,\
        0.36662, 0.36761, 0.36869, 0.36986, 0.37113, 0.37252, 0.37403, 0.37566, 0.37742, 0.37931, 0.38133,\
        0.38348, 0.38576, 0.38818, 0.39074, 0.39346, 0.39635, 0.39956, 0.40337, 0.40772, 0.41245, 0.41746,\
        0.42267, 0.42804, 0.43352, 0.43906, 0.44461, 0.45012, 0.45556, 0.46089, 0.46609, 0.47115, 0.47609,\
        0.48093, 0.48573, 0.49054, 0.49535, 0.50015, 0.50512, 0.51007, 0.51493, 0.51967, 0.52426, 0.52862,\
        0.53272, 0.53652, 0.53999, 0.54313, 0.54594, 0.54844, 0.55063, 0.55256, 0.55423, 0.55566, 0.55688,\
        0.55788, 0.55867, 0.55926, 0.55964, 0.55984, 0.55985, 0.55969, 0.55936, 0.55886, 0.55821, 0.55739,\
        0.55641, 0.55524, 0.55389, 0.55234, 0.55059, 0.54865, 0.54651, 0.54419, 0.54170, 0.53905, 0.53625,\
        0.53333, 0.53030, 0.52719, 0.52403, 0.52084, 0.51768, 0.51464, 0.51184, 0.50951, 0.50771, 0.50561,\
        0.50253, 0.49785, 0.49244, 0.48661, 0.48040, 0.47385, 0.46694, 0.45962, 0.45184, 0.44358, 0.43482],\
        [0.34021, 0.34352, 0.34647, 0.34908, 0.35139, 0.35341, 0.35518, 0.35671, 0.35804, 0.35918, 0.36018,\
        0.36105, 0.36181, 0.36250, 0.36312, 0.36371, 0.36427, 0.36483, 0.36541, 0.36601, 0.36665, 0.36735,\
        0.36812, 0.36897, 0.36991, 0.37096, 0.37212, 0.37339, 0.37479, 0.37633, 0.37800, 0.37980, 0.38173,\
        0.38379, 0.38597, 0.38826, 0.39068, 0.39323, 0.39598, 0.39909, 0.40304, 0.40763, 0.41259, 0.41782,\
        0.42325, 0.42882, 0.43448, 0.44019, 0.44589, 0.45153, 0.45705, 0.46242, 0.46759, 0.47254, 0.47728,\
        0.48183, 0.48627, 0.49072, 0.49536, 0.50036, 0.50570, 0.51123, 0.51674, 0.52212, 0.52726, 0.53211,\
        0.53661, 0.54075, 0.54450, 0.54786, 0.55084, 0.55345, 0.55574, 0.55774, 0.55948, 0.56097, 0.56224,\
        0.56329, 0.56413, 0.56475, 0.56516, 0.56538, 0.56541, 0.56526, 0.56494, 0.56447, 0.56385, 0.56306,\
        0.56211, 0.56098, 0.55966, 0.55814, 0.55641, 0.55448, 0.55234, 0.55003, 0.54753, 0.54488, 0.54209,\
        0.53919, 0.53618, 0.53310, 0.52998, 0.52685, 0.52377, 0.52081, 0.51807, 0.51561, 0.51330, 0.51067,\
        0.50718, 0.50261, 0.49741, 0.49176, 0.48571, 0.47939, 0.47272, 0.46559, 0.45797, 0.44982, 0.44115],\
        [0.34592, 0.34904, 0.35179, 0.35419, 0.35628, 0.35808, 0.35961, 0.36091, 0.36200, 0.36292, 0.36369,\
        0.36434, 0.36489, 0.36537, 0.36579, 0.36618, 0.36656, 0.36694, 0.36734, 0.36777, 0.36826, 0.36881,\
        0.36943, 0.37015, 0.37096, 0.37188, 0.37293, 0.37410, 0.37541, 0.37686, 0.37845, 0.38019, 0.38206,\
        0.38407, 0.38619, 0.38844, 0.39079, 0.39330, 0.39607, 0.39935, 0.40328, 0.40791, 0.41299, 0.41837,\
        0.42397, 0.42972, 0.43556, 0.44145, 0.44731, 0.45310, 0.45875, 0.46419, 0.46938, 0.47428, 0.47887,\
        0.48318, 0.48729, 0.49137, 0.49578, 0.50092, 0.50676, 0.51289, 0.51903, 0.52498, 0.53064, 0.53594,\
        0.54083, 0.54528, 0.54928, 0.55282, 0.55593, 0.55864, 0.56099, 0.56303, 0.56481, 0.56635, 0.56766,\
        0.56875, 0.56962, 0.57027, 0.57070, 0.57092, 0.57095, 0.57079, 0.57048, 0.57002, 0.56942, 0.56866,\
        0.56775, 0.56665, 0.56536, 0.56385, 0.56214, 0.56021, 0.55807, 0.55574, 0.55324, 0.55059, 0.54780,\
        0.54490, 0.54191, 0.53886, 0.53577, 0.53269, 0.52965, 0.52670, 0.52389, 0.52119, 0.51845, 0.51536,\
        0.51161, 0.50710, 0.50204, 0.49640, 0.49057, 0.48459, 0.47819, 0.47126, 0.46379, 0.45575, 0.44716],\
        [0.35144, 0.35436, 0.35691, 0.35910, 0.36096, 0.36253, 0.36382, 0.36489, 0.36575, 0.36644, 0.36698,\
        0.36741, 0.36775, 0.36802, 0.36824, 0.36844, 0.36864, 0.36884, 0.36907, 0.36935, 0.36968, 0.37008,\
        0.37056, 0.37114, 0.37183, 0.37264, 0.37358, 0.37465, 0.37588, 0.37726, 0.37879, 0.38049, 0.38234,\
        0.38434, 0.38648, 0.38875, 0.39114, 0.39370, 0.39651, 0.39979, 0.40386, 0.40853, 0.41367, 0.41914,\
        0.42486, 0.43076, 0.43678, 0.44286, 0.44892, 0.45489, 0.46071, 0.46630, 0.47159, 0.47652, 0.48107,\
        0.48526, 0.48916, 0.49294, 0.49701, 0.50237, 0.50872, 0.51541, 0.52206, 0.52849, 0.53458, 0.54026,\
        0.54548, 0.55020, 0.55440, 0.55809, 0.56128, 0.56403, 0.56640, 0.56845, 0.57024, 0.57181, 0.57316,\
        0.57429, 0.57519, 0.57584, 0.57627, 0.57648, 0.57649, 0.57632, 0.57598, 0.57552, 0.57493, 0.57421,\
        0.57332, 0.57225, 0.57098, 0.56949, 0.56778, 0.56584, 0.56369, 0.56134, 0.55882, 0.55615, 0.55335,\
        0.55045, 0.54747, 0.54443, 0.54137, 0.53829, 0.53525, 0.53226, 0.52932, 0.52637, 0.52329, 0.51987,\
        0.51592, 0.51137, 0.50625, 0.50063, 0.49522, 0.48963, 0.48344, 0.47666, 0.46932, 0.46138, 0.45285],\
        [0.35676, 0.35949, 0.36183, 0.36380, 0.36543, 0.36676, 0.36782, 0.36865, 0.36927, 0.36973, 0.37004,\
        0.37025, 0.37038, 0.37045, 0.37048, 0.37050, 0.37051, 0.37055, 0.37062, 0.37074, 0.37092, 0.37118,\
        0.37153, 0.37198, 0.37255, 0.37324, 0.37408, 0.37507, 0.37621, 0.37753, 0.37903, 0.38071, 0.38256,\
        0.38460, 0.38680, 0.38917, 0.39170, 0.39440, 0.39732, 0.40068, 0.40475, 0.40943, 0.41457, 0.42009,\
        0.42590, 0.43195, 0.43815, 0.44442, 0.45071, 0.45691, 0.46296, 0.46878, 0.47426, 0.47935, 0.48400,\
        0.48825, 0.49217, 0.49603, 0.49997, 0.50538, 0.51207, 0.51907, 0.52604, 0.53280, 0.53920, 0.54517,\
        0.55064, 0.55557, 0.55993, 0.56371, 0.56693, 0.56966, 0.57200, 0.57402, 0.57580, 0.57737, 0.57875,\
        0.57992, 0.58084, 0.58150, 0.58190, 0.58209, 0.58206, 0.58184, 0.58146, 0.58098, 0.58040, 0.57970,\
        0.57884, 0.57780, 0.57654, 0.57506, 0.57333, 0.57137, 0.56918, 0.56679, 0.56424, 0.56154, 0.55873,\
        0.55582, 0.55284, 0.54980, 0.54672, 0.54364, 0.54056, 0.53748, 0.53439, 0.53123, 0.52789, 0.52424,\
        0.52016, 0.51560, 0.51058, 0.50530, 0.50015, 0.49464, 0.48848, 0.48179, 0.47456, 0.46671, 0.45821],\
        [0.36189, 0.36442, 0.36655, 0.36829, 0.36969, 0.37078, 0.37159, 0.37218, 0.37256, 0.37279, 0.37288,\
        0.37288, 0.37280, 0.37267, 0.37252, 0.37235, 0.37220, 0.37207, 0.37198, 0.37195, 0.37199, 0.37211,\
        0.37233, 0.37266, 0.37311, 0.37370, 0.37444, 0.37534, 0.37642, 0.37768, 0.37915, 0.38082, 0.38271,\
        0.38481, 0.38712, 0.38964, 0.39235, 0.39527, 0.39847, 0.40190, 0.40590, 0.41050, 0.41560, 0.42115,\
        0.42705, 0.43324, 0.43963, 0.44613, 0.45267, 0.45916, 0.46550, 0.47161, 0.47741, 0.48278, 0.48766,\
        0.49214, 0.49633, 0.50039, 0.50503, 0.51047, 0.51697, 0.52396, 0.53104, 0.53795, 0.54454, 0.55070,\
        0.55635, 0.56144, 0.56590, 0.56972, 0.57291, 0.57555, 0.57780, 0.57976, 0.58151, 0.58306, 0.58446,\
        0.58568, 0.58663, 0.58725, 0.58760, 0.58775, 0.58769, 0.58737, 0.58691, 0.58639, 0.58582, 0.58515,\
        0.58431, 0.58328, 0.58203, 0.58053, 0.57878, 0.57678, 0.57454, 0.57209, 0.56948, 0.56675, 0.56391,\
        0.56098, 0.55798, 0.55493, 0.55183, 0.54870, 0.54556, 0.54239, 0.53916, 0.53582, 0.53229, 0.52849,\
        0.52434, 0.51982, 0.51497, 0.50995, 0.50482, 0.49925, 0.49315, 0.48660, 0.47951, 0.47173, 0.46324],\
        [0.36681, 0.36915, 0.37106, 0.37257, 0.37373, 0.37457, 0.37514, 0.37547, 0.37562, 0.37562, 0.37549,\
        0.37528, 0.37500, 0.37468, 0.37435, 0.37401, 0.37370, 0.37341, 0.37318, 0.37300, 0.37291, 0.37290,\
        0.37299, 0.37320, 0.37354, 0.37403, 0.37467, 0.37549, 0.37649, 0.37770, 0.37914, 0.38081, 0.38273,\
        0.38490, 0.38734, 0.39004, 0.39297, 0.39610, 0.39938, 0.40300, 0.40701, 0.41155, 0.41664, 0.42222,\
        0.42823, 0.43458, 0.44119, 0.44795, 0.45478, 0.46158, 0.46826, 0.47474, 0.48094, 0.48673, 0.49195,\
        0.49664, 0.50135, 0.50584, 0.51084, 0.51653, 0.52296, 0.52987, 0.53694, 0.54390, 0.55057, 0.55683,\
        0.56259, 0.56778, 0.57233, 0.57615, 0.57924, 0.58172, 0.58380, 0.58569, 0.58738, 0.58890, 0.59031,\
        0.59159, 0.59260, 0.59314, 0.59336, 0.59349, 0.59341, 0.59294, 0.59236, 0.59178, 0.59122, 0.59058,\
        0.58973, 0.58869, 0.58743, 0.58592, 0.58413, 0.58206, 0.57973, 0.57720, 0.57452, 0.57174, 0.56887,\
        0.56591, 0.56289, 0.55980, 0.55667, 0.55349, 0.55027, 0.54700, 0.54364, 0.54016, 0.53650, 0.53259,\
        0.52840, 0.52391, 0.51915, 0.51419, 0.50898, 0.50340, 0.49738, 0.49107, 0.48420, 0.47643, 0.46788],\
        [0.37152, 0.37366, 0.37535, 0.37662, 0.37753, 0.37812, 0.37844, 0.37853, 0.37844, 0.37821, 0.37788,\
        0.37746, 0.37699, 0.37649, 0.37599, 0.37549, 0.37502, 0.37459, 0.37422, 0.37391, 0.37368, 0.37355,\
        0.37352, 0.37362, 0.37385, 0.37423, 0.37478, 0.37551, 0.37644, 0.37759, 0.37899, 0.38065, 0.38259,\
        0.38483, 0.38738, 0.39025, 0.39340, 0.39677, 0.40028, 0.40387, 0.40788, 0.41242, 0.41753, 0.42321,\
        0.42936, 0.43592, 0.44277, 0.44983, 0.45697, 0.46412, 0.47117, 0.47805, 0.48468, 0.49100, 0.49681,\
        0.50170, 0.50666, 0.51176, 0.51717, 0.52312, 0.52961, 0.53650, 0.54355, 0.55052, 0.55721, 0.56350,\
        0.56928, 0.57453, 0.57916, 0.58299, 0.58593, 0.58816, 0.58995, 0.59182, 0.59350, 0.59492, 0.59634,\
        0.59765, 0.59877, 0.59917, 0.59921, 0.59929, 0.59926, 0.59852, 0.59787, 0.59715, 0.59664, 0.59601,\
        0.59510, 0.59403, 0.59274, 0.59118, 0.58934, 0.58718, 0.58474, 0.58209, 0.57933, 0.57650, 0.57360,\
        0.57061, 0.56755, 0.56442, 0.56123, 0.55799, 0.55469, 0.55132, 0.54786, 0.54426, 0.54049, 0.53650,\
        0.53227, 0.52778, 0.52305, 0.51808, 0.51282, 0.50722, 0.50127, 0.49542, 0.48866, 0.48072, 0.47205],\
        [0.37602, 0.37795, 0.37941, 0.38044, 0.38109, 0.38142, 0.38149, 0.38134, 0.38102, 0.38057, 0.38003,\
        0.37942, 0.37877, 0.37810, 0.37744, 0.37679, 0.37618, 0.37562, 0.37511, 0.37468, 0.37433, 0.37408,\
        0.37394, 0.37393, 0.37405, 0.37433, 0.37478, 0.37542, 0.37627, 0.37735, 0.37869, 0.38032, 0.38225,\
        0.38451, 0.38713, 0.39012, 0.39346, 0.39708, 0.40073, 0.40434, 0.40836, 0.41295, 0.41818, 0.42401,\
        0.43039, 0.43720, 0.44435, 0.45172, 0.45922, 0.46672, 0.47414, 0.48141, 0.48842, 0.49509, 0.50148,\
        0.50702, 0.51236, 0.51789, 0.52373, 0.52998, 0.53663, 0.54359, 0.55067, 0.55766, 0.56436, 0.57061,\
        0.57634, 0.58155, 0.58629, 0.59021, 0.59292, 0.59496, 0.59618, 0.59810, 0.59997, 0.60125, 0.60270,\
        0.60396, 0.60494, 0.60534, 0.60535, 0.60529, 0.60502, 0.60430, 0.60355, 0.60271, 0.60215, 0.60141,\
        0.60041, 0.59928, 0.59791, 0.59629, 0.59439, 0.59212, 0.58950, 0.58671, 0.58388, 0.58101, 0.57809,\
        0.57507, 0.57197, 0.56878, 0.56552, 0.56221, 0.55883, 0.55537, 0.55181, 0.54812, 0.54426, 0.54020,\
        0.53592, 0.53140, 0.52665, 0.52165, 0.51640, 0.51090, 0.50532, 0.49984, 0.49250, 0.48431, 0.47560],\
        [0.38028, 0.38200, 0.38323, 0.38400, 0.38439, 0.38446, 0.38427, 0.38388, 0.38334, 0.38268, 0.38194,\
        0.38115, 0.38034, 0.37952, 0.37871, 0.37793, 0.37719, 0.37650, 0.37588, 0.37534, 0.37488, 0.37452,\
        0.37427, 0.37415, 0.37417, 0.37434, 0.37469, 0.37523, 0.37599, 0.37699, 0.37826, 0.37982, 0.38170,\
        0.38394, 0.38655, 0.38956, 0.39295, 0.39664, 0.40036, 0.40417, 0.40831, 0.41307, 0.41851, 0.42460,\
        0.43127, 0.43840, 0.44588, 0.45361, 0.46147, 0.46935, 0.47714, 0.48475, 0.49214, 0.49917, 0.50573,\
        0.51200, 0.51801, 0.52407, 0.53036, 0.53695, 0.54383, 0.55093, 0.55811, 0.56519, 0.57191, 0.57810,\
        0.58369, 0.58866, 0.59334, 0.59777, 0.60002, 0.60202, 0.60309, 0.60480, 0.60675, 0.60819, 0.60950,\
        0.61060, 0.61137, 0.61171, 0.61170, 0.61147, 0.61100, 0.61027, 0.60941, 0.60852, 0.60769, 0.60677,\
        0.60566, 0.60438, 0.60290, 0.60118, 0.59920, 0.59680, 0.59394, 0.59100, 0.58814, 0.58528, 0.58235,\
        0.57930, 0.57613, 0.57287, 0.56954, 0.56615, 0.56269, 0.55915, 0.55550, 0.55172, 0.54778, 0.54365,\
        0.53931, 0.53475, 0.52995, 0.52491, 0.51962, 0.51409, 0.50831, 0.50194, 0.49483, 0.48703, 0.47851],\
        [0.38429, 0.38580, 0.38678, 0.38729, 0.38740, 0.38721, 0.38677, 0.38615, 0.38539, 0.38454, 0.38362,\
        0.38267, 0.38171, 0.38076, 0.37982, 0.37892, 0.37807, 0.37727, 0.37655, 0.37590, 0.37534, 0.37488,\
        0.37453, 0.37431, 0.37422, 0.37429, 0.37454, 0.37498, 0.37563, 0.37653, 0.37770, 0.37917, 0.38097,\
        0.38312, 0.38565, 0.38857, 0.39188, 0.39552, 0.39939, 0.40336, 0.40770, 0.41274, 0.41853, 0.42499,\
        0.43202, 0.43952, 0.44737, 0.45547, 0.46372, 0.47198, 0.48013, 0.48807, 0.49572, 0.50312, 0.51016,\
        0.51693, 0.52356, 0.53019, 0.53697, 0.54393, 0.55109, 0.55838, 0.56573, 0.57297, 0.57981, 0.58591,\
        0.59136, 0.59612, 0.59980, 0.60450, 0.60741, 0.60944, 0.61098, 0.61255, 0.61418, 0.61558, 0.61672,\
        0.61759, 0.61815, 0.61835, 0.61821, 0.61782, 0.61719, 0.61635, 0.61538, 0.61435, 0.61328, 0.61212,\
        0.61082, 0.60935, 0.60768, 0.60579, 0.60362, 0.60103, 0.59794, 0.59496, 0.59214, 0.58935, 0.58642,\
        0.58330, 0.58005, 0.57669, 0.57328, 0.56981, 0.56628, 0.56266, 0.55893, 0.55508, 0.55106, 0.54686,\
        0.54245, 0.53782, 0.53296, 0.52784, 0.52244, 0.51674, 0.51063, 0.50399, 0.49698, 0.48930, 0.48093],\
        [0.38803, 0.38932, 0.39003, 0.39026, 0.39010, 0.38964, 0.38897, 0.38813, 0.38718, 0.38615, 0.38508,\
        0.38399, 0.38290, 0.38182, 0.38078, 0.37978, 0.37883, 0.37795, 0.37713, 0.37639, 0.37574, 0.37519,\
        0.37474, 0.37442, 0.37423, 0.37420, 0.37434, 0.37467, 0.37522, 0.37601, 0.37707, 0.37842, 0.38009,\
        0.38211, 0.38450, 0.38728, 0.39043, 0.39396, 0.39782, 0.40191, 0.40661, 0.41209, 0.41834, 0.42526,\
        0.43273, 0.44061, 0.44883, 0.45730, 0.46593, 0.47460, 0.48312, 0.49135, 0.49940, 0.50706, 0.51454,\
        0.52184, 0.52904, 0.53624, 0.54351, 0.55089, 0.55835, 0.56585, 0.57336, 0.58083, 0.58801, 0.59408,\
        0.59914, 0.60410, 0.60796, 0.61188, 0.61512, 0.61748, 0.61928, 0.62083, 0.62222, 0.62340, 0.62430,\
        0.62492, 0.62523, 0.62522, 0.62490, 0.62432, 0.62352, 0.62252, 0.62139, 0.62016, 0.61885, 0.61743,\
        0.61589, 0.61417, 0.61226, 0.61012, 0.60768, 0.60486, 0.60174, 0.59875, 0.59601, 0.59329, 0.59031,\
        0.58708, 0.58369, 0.58023, 0.57673, 0.57319, 0.56960, 0.56591, 0.56212, 0.55819, 0.55410, 0.54982,\
        0.54534, 0.54063, 0.53568, 0.53045, 0.52494, 0.51908, 0.51284, 0.50620, 0.49909, 0.49132, 0.48300],\
        [0.39146, 0.39251, 0.39295, 0.39288, 0.39244, 0.39174, 0.39084, 0.38981, 0.38869, 0.38751, 0.38631,\
        0.38510, 0.38391, 0.38274, 0.38161, 0.38053, 0.37950, 0.37854, 0.37765, 0.37683, 0.37610, 0.37547,\
        0.37494, 0.37452, 0.37424, 0.37410, 0.37414, 0.37436, 0.37479, 0.37546, 0.37639, 0.37761, 0.37914,\
        0.38102, 0.38325, 0.38585, 0.38882, 0.39217, 0.39592, 0.40023, 0.40530, 0.41131, 0.41815, 0.42560,\
        0.43351, 0.44176, 0.45029, 0.45908, 0.46807, 0.47718, 0.48616, 0.49464, 0.50281, 0.51097, 0.51890,\
        0.52671, 0.53447, 0.54222, 0.55001, 0.55782, 0.56562, 0.57332, 0.58088, 0.58845, 0.59606, 0.60260,\
        0.60760, 0.61238, 0.61660, 0.62035, 0.62355, 0.62605, 0.62796, 0.62946, 0.63066, 0.63157, 0.63219,\
        0.63251, 0.63254, 0.63228, 0.63173, 0.63094, 0.62993, 0.62873, 0.62739, 0.62593, 0.62436, 0.62268,\
        0.62086, 0.61888, 0.61671, 0.61431, 0.61165, 0.60872, 0.60563, 0.60261, 0.59990, 0.59715, 0.59401,\
        0.59057, 0.58702, 0.58345, 0.57989, 0.57631, 0.57266, 0.56892, 0.56507, 0.56107, 0.55690, 0.55254,\
        0.54798, 0.54318, 0.53813, 0.53280, 0.52717, 0.52118, 0.51482, 0.50804, 0.50079, 0.49303, 0.48478],\
        [0.39452, 0.39533, 0.39545, 0.39509, 0.39439, 0.39346, 0.39237, 0.39118, 0.38993, 0.38864, 0.38733,\
        0.38604, 0.38477, 0.38353, 0.38234, 0.38120, 0.38011, 0.37909, 0.37814, 0.37726, 0.37646, 0.37575,\
        0.37514, 0.37464, 0.37426, 0.37403, 0.37396, 0.37407, 0.37438, 0.37493, 0.37574, 0.37682, 0.37822,\
        0.37995, 0.38203, 0.38449, 0.38732, 0.39055, 0.39424, 0.39862, 0.40405, 0.41075, 0.41826, 0.42625,\
        0.43456, 0.44310, 0.45183, 0.46079, 0.47002, 0.47952, 0.48914, 0.49833, 0.50661, 0.51488, 0.52320,\
        0.53153, 0.53985, 0.54816, 0.55647, 0.56475, 0.57295, 0.58092, 0.58845, 0.59577, 0.60350, 0.61048,\
        0.61620, 0.62119, 0.62559, 0.62939, 0.63257, 0.63508, 0.63698, 0.63837, 0.63935, 0.63998, 0.64028,\
        0.64029, 0.64001, 0.63947, 0.63866, 0.63762, 0.63637, 0.63494, 0.63335, 0.63164, 0.62980, 0.62784,\
        0.62574, 0.62349, 0.62106, 0.61843, 0.61559, 0.61258, 0.60949, 0.60647, 0.60362, 0.60069, 0.59734,\
        0.59367, 0.58997, 0.58636, 0.58278, 0.57918, 0.57550, 0.57171, 0.56779, 0.56372, 0.55948, 0.55504,\
        0.55038, 0.54550, 0.54035, 0.53492, 0.52917, 0.52307, 0.51658, 0.50966, 0.50229, 0.49453, 0.48636],\
        [0.39714, 0.39766, 0.39745, 0.39680, 0.39589, 0.39479, 0.39356, 0.39226, 0.39091, 0.38954, 0.38817,\
        0.38682, 0.38550, 0.38422, 0.38299, 0.38181, 0.38068, 0.37962, 0.37862, 0.37770, 0.37684, 0.37607,\
        0.37539, 0.37481, 0.37434, 0.37401, 0.37384, 0.37384, 0.37404, 0.37447, 0.37515, 0.37612, 0.37739,\
        0.37901, 0.38099, 0.38337, 0.38617, 0.38943, 0.39320, 0.39766, 0.40358, 0.41096, 0.41903, 0.42747,\
        0.43608, 0.44479, 0.45357, 0.46249, 0.47168, 0.48125, 0.49122, 0.50109, 0.50978, 0.51849, 0.52738,\
        0.53631, 0.54522, 0.55409, 0.56292, 0.57169, 0.58035, 0.58879, 0.59670, 0.60375, 0.61143, 0.61865,\
        0.62490, 0.63027, 0.63489, 0.63879, 0.64198, 0.64446, 0.64627, 0.64748, 0.64821, 0.64852, 0.64849,\
        0.64816, 0.64756, 0.64671, 0.64562, 0.64431, 0.64280, 0.64111, 0.63926, 0.63727, 0.63515, 0.63291,\
        0.63053, 0.62801, 0.62533, 0.62248, 0.61947, 0.61635, 0.61317, 0.61001, 0.60688, 0.60364, 0.60011,\
        0.59626, 0.59254, 0.58898, 0.58544, 0.58184, 0.57814, 0.57430, 0.57032, 0.56618, 0.56185, 0.55732,\
        0.55258, 0.54759, 0.54235, 0.53682, 0.53097, 0.52478, 0.51821, 0.51121, 0.50379, 0.49602, 0.48785],\
        [0.39916, 0.39932, 0.39880, 0.39797, 0.39693, 0.39573, 0.39443, 0.39306, 0.39166, 0.39025, 0.38885,\
        0.38748, 0.38614, 0.38484, 0.38359, 0.38240, 0.38125, 0.38017, 0.37914, 0.37818, 0.37728, 0.37645,\
        0.37571, 0.37505, 0.37450, 0.37408, 0.37381, 0.37370, 0.37379, 0.37411, 0.37468, 0.37554, 0.37673,\
        0.37827, 0.38022, 0.38260, 0.38550, 0.38898, 0.39318, 0.39818, 0.40459, 0.41229, 0.42071, 0.42946,\
        0.43829, 0.44704, 0.45569, 0.46434, 0.47316, 0.48238, 0.49202, 0.50178, 0.51183, 0.52168, 0.53149,\
        0.54117, 0.55068, 0.56007, 0.56941, 0.57865, 0.58771, 0.59660, 0.60505, 0.61270, 0.62022, 0.62744,\
        0.63394, 0.63960, 0.64443, 0.64845, 0.65166, 0.65407, 0.65573, 0.65671, 0.65713, 0.65710, 0.65671,\
        0.65604, 0.65511, 0.65395, 0.65257, 0.65098, 0.64918, 0.64721, 0.64508, 0.64281, 0.64040, 0.63788,\
        0.63522, 0.63244, 0.62951, 0.62645, 0.62327, 0.61998, 0.61663, 0.61325, 0.60981, 0.60621, 0.60236,\
        0.59845, 0.59487, 0.59145, 0.58797, 0.58436, 0.58062, 0.57672, 0.57267, 0.56845, 0.56403, 0.55942,\
        0.55457, 0.54949, 0.54415, 0.53852, 0.53259, 0.52633, 0.51971, 0.51271, 0.50530, 0.49751, 0.48927],\
        [0.40001, 0.39996, 0.39943, 0.39860, 0.39754, 0.39633, 0.39501, 0.39363, 0.39223, 0.39081, 0.38942,\
        0.38805, 0.38672, 0.38544, 0.38420, 0.38301, 0.38186, 0.38077, 0.37973, 0.37875, 0.37781, 0.37694,\
        0.37613, 0.37541, 0.37478, 0.37427, 0.37389, 0.37368, 0.37367, 0.37388, 0.37435, 0.37513, 0.37626,\
        0.37777, 0.37973, 0.38220, 0.38528, 0.38909, 0.39384, 0.39980, 0.40655, 0.41454, 0.42329, 0.43233,\
        0.44132, 0.45005, 0.45847, 0.46666, 0.47486, 0.48347, 0.49293, 0.50325, 0.51398, 0.52502, 0.53589,\
        0.54636, 0.55641, 0.56619, 0.57595, 0.58570, 0.59513, 0.60431, 0.61321, 0.62151, 0.62933, 0.63665,\
        0.64331, 0.64917, 0.65417, 0.65828, 0.66149, 0.66380, 0.66525, 0.66595, 0.66602, 0.66561, 0.66485,\
        0.66382, 0.66257, 0.66111, 0.65944, 0.65756, 0.65548, 0.65322, 0.65079, 0.64823, 0.64554, 0.64273,\
        0.63980, 0.63676, 0.63360, 0.63033, 0.62695, 0.62348, 0.61994, 0.61633, 0.61261, 0.60875, 0.60474,\
        0.60078, 0.59729, 0.59397, 0.59046, 0.58679, 0.58297, 0.57900, 0.57486, 0.57055, 0.56604, 0.56133,\
        0.55638, 0.55120, 0.54576, 0.54004, 0.53403, 0.52770, 0.52105, 0.51405, 0.50669, 0.49889, 0.49055],\
        [0.39998, 0.39996, 0.39958, 0.39885, 0.39785, 0.39668, 0.39539, 0.39403, 0.39266, 0.39128, 0.38993,\
        0.38860, 0.38731, 0.38605, 0.38484, 0.38368, 0.38256, 0.38148, 0.38044, 0.37943, 0.37847, 0.37756,\
        0.37670, 0.37591, 0.37520, 0.37460, 0.37412, 0.37381, 0.37369, 0.37380, 0.37419, 0.37490, 0.37599,\
        0.37751, 0.37954, 0.38214, 0.38544, 0.38956, 0.39469, 0.40101, 0.40875, 0.41740, 0.42663, 0.43604,\
        0.44526, 0.45402, 0.46221, 0.46989, 0.47733, 0.48513, 0.49407, 0.50476, 0.51687, 0.52918, 0.54109,\
        0.55227, 0.56268, 0.57256, 0.58246, 0.59281, 0.60278, 0.61229, 0.62158, 0.63037, 0.63855, 0.64607,\
        0.65287, 0.65886, 0.66398, 0.66815, 0.67133, 0.67352, 0.67474, 0.67509, 0.67476, 0.67394, 0.67279,\
        0.67143, 0.66988, 0.66814, 0.66619, 0.66403, 0.66166, 0.65910, 0.65637, 0.65352, 0.65054, 0.64745,\
        0.64426, 0.64097, 0.63759, 0.63410, 0.63052, 0.62687, 0.62313, 0.61933, 0.61543, 0.61145, 0.60744,\
        0.60357, 0.60011, 0.59662, 0.59293, 0.58912, 0.58520, 0.58113, 0.57691, 0.57250, 0.56789, 0.56307,\
        0.55802, 0.55273, 0.54719, 0.54137, 0.53526, 0.52887, 0.52216, 0.51514, 0.50778, 0.49997, 0.49154],\
        [0.40016, 0.40005, 0.39965, 0.39896, 0.39801, 0.39688, 0.39564, 0.39434, 0.39302, 0.39172, 0.39043,\
        0.38917, 0.38794, 0.38674, 0.38558, 0.38446, 0.38337, 0.38232, 0.38129, 0.38029, 0.37931, 0.37835,\
        0.37744, 0.37658, 0.37579, 0.37510, 0.37452, 0.37410, 0.37387, 0.37389, 0.37419, 0.37485, 0.37591,\
        0.37746, 0.37958, 0.38235, 0.38590, 0.39037, 0.39597, 0.40297, 0.41136, 0.42071, 0.43055, 0.44048,\
        0.45009, 0.45904, 0.46716, 0.47448, 0.48127, 0.48816, 0.49665, 0.50814, 0.52130, 0.53472, 0.54754,\
        0.55932, 0.56989, 0.57949, 0.58883, 0.59917, 0.61004, 0.62034, 0.63019, 0.63941, 0.64789, 0.65558,\
        0.66246, 0.66853, 0.67373, 0.67792, 0.68105, 0.68310, 0.68404, 0.68401, 0.68323, 0.68197, 0.68046,\
        0.67878, 0.67697, 0.67499, 0.67279, 0.67035, 0.66768, 0.66481, 0.66179, 0.65864, 0.65538, 0.65203,\
        0.64859, 0.64506, 0.64145, 0.63776, 0.63399, 0.63015, 0.62625, 0.62228, 0.61827, 0.61423, 0.61024,\
        0.60642, 0.60280, 0.59910, 0.59525, 0.59132, 0.58729, 0.58313, 0.57881, 0.57430, 0.56959, 0.56466,\
        0.55950, 0.55410, 0.54844, 0.54252, 0.53631, 0.52981, 0.52301, 0.51589, 0.50844, 0.50059, 0.49210],\
        [0.40050, 0.40030, 0.39981, 0.39909, 0.39815, 0.39705, 0.39586, 0.39462, 0.39340, 0.39219, 0.39101,\
        0.38984, 0.38869, 0.38757, 0.38648, 0.38541, 0.38437, 0.38335, 0.38235, 0.38134, 0.38034, 0.37936,\
        0.37839, 0.37746, 0.37658, 0.37579, 0.37510, 0.37456, 0.37422, 0.37412, 0.37433, 0.37493, 0.37597,\
        0.37756, 0.37978, 0.38274, 0.38657, 0.39144, 0.39754, 0.40517, 0.41425, 0.42428, 0.43483, 0.44543,\
        0.45564, 0.46504, 0.47336, 0.48064, 0.48725, 0.49380, 0.50167, 0.51384, 0.52778, 0.54193, 0.55544,\
        0.56771, 0.57839, 0.58762, 0.59651, 0.60624, 0.61764, 0.62873, 0.63912, 0.64871, 0.65739, 0.66510,\
        0.67193, 0.67799, 0.68325, 0.68742, 0.69038, 0.69234, 0.69297, 0.69255, 0.69129, 0.68962, 0.68777,\
        0.68582, 0.68381, 0.68165, 0.67921, 0.67648, 0.67351, 0.67034, 0.66701, 0.66358, 0.66005, 0.65644,\
        0.65276, 0.64901, 0.64519, 0.64130, 0.63735, 0.63334, 0.62928, 0.62518, 0.62106, 0.61697, 0.61293,\
        0.60902, 0.60518, 0.60131, 0.59734, 0.59336, 0.58926, 0.58501, 0.58058, 0.57597, 0.57115, 0.56611,\
        0.56084, 0.55532, 0.54955, 0.54350, 0.53718, 0.53056, 0.52362, 0.51635, 0.50870, 0.50065, 0.49223],\
        [0.40101, 0.40070, 0.40012, 0.39934, 0.39837, 0.39727, 0.39611, 0.39496, 0.39385, 0.39278, 0.39173,\
        0.39069, 0.38964, 0.38860, 0.38758, 0.38659, 0.38561, 0.38463, 0.38365, 0.38265, 0.38163, 0.38060,\
        0.37957, 0.37856, 0.37759, 0.37669, 0.37588, 0.37521, 0.37472, 0.37449, 0.37459, 0.37510, 0.37611,\
        0.37773, 0.38005, 0.38319, 0.38728, 0.39249, 0.39914, 0.40736, 0.41705, 0.42777, 0.43906, 0.45047,\
        0.46150, 0.47166, 0.48051, 0.48805, 0.49502, 0.50216, 0.51077, 0.52233, 0.53605, 0.55046, 0.56441,\
        0.57714, 0.58821, 0.59711, 0.60606, 0.61599, 0.62687, 0.63792, 0.64850, 0.65829, 0.66703, 0.67459,\
        0.68115, 0.68701, 0.69226, 0.69648, 0.69896, 0.70093, 0.70119, 0.70054, 0.69880, 0.69687, 0.69473,\
        0.69253, 0.69039, 0.68811, 0.68544, 0.68240, 0.67910, 0.67563, 0.67201, 0.66830, 0.66452, 0.66068,\
        0.65678, 0.65282, 0.64880, 0.64472, 0.64060, 0.63642, 0.63221, 0.62799, 0.62376, 0.61958, 0.61545,\
        0.61140, 0.60740, 0.60337, 0.59933, 0.59530, 0.59111, 0.58676, 0.58224, 0.57751, 0.57259, 0.56743,\
        0.56204, 0.55641, 0.55051, 0.54435, 0.53790, 0.53115, 0.52408, 0.51665, 0.50884, 0.50065, 0.49225],\
        [0.40174, 0.40131, 0.40062, 0.39975, 0.39872, 0.39760, 0.39648, 0.39543, 0.39448, 0.39359, 0.39270,\
        0.39180, 0.39086, 0.38991, 0.38897, 0.38805, 0.38713, 0.38621, 0.38526, 0.38426, 0.38321, 0.38213,\
        0.38102, 0.37992, 0.37884, 0.37782, 0.37687, 0.37603, 0.37538, 0.37498, 0.37492, 0.37530, 0.37624,\
        0.37784, 0.38023, 0.38353, 0.38788, 0.39342, 0.40026, 0.40911, 0.41938, 0.43070, 0.44269, 0.45495,\
        0.46699, 0.47824, 0.48803, 0.49585, 0.50317, 0.51090, 0.51995, 0.53129, 0.54473, 0.55926, 0.57361,\
        0.58637, 0.59795, 0.60730, 0.61634, 0.62628, 0.63688, 0.64767, 0.65818, 0.66804, 0.67682, 0.68402,\
        0.68998, 0.69549, 0.70040, 0.70454, 0.70697, 0.70841, 0.70863, 0.70773, 0.70596, 0.70383, 0.70139,\
        0.69898, 0.69675, 0.69441, 0.69140, 0.68801, 0.68442, 0.68065, 0.67675, 0.67279, 0.66878, 0.66472,\
        0.66063, 0.65648, 0.65227, 0.64802, 0.64372, 0.63939, 0.63503, 0.63067, 0.62633, 0.62203, 0.61779,\
        0.61362, 0.60949, 0.60539, 0.60132, 0.59713, 0.59283, 0.58839, 0.58376, 0.57894, 0.57390, 0.56864,\
        0.56314, 0.55739, 0.55137, 0.54508, 0.53850, 0.53163, 0.52445, 0.51693, 0.50905, 0.50084, 0.49235],\
        [0.40275, 0.40214, 0.40130, 0.40031, 0.39922, 0.39807, 0.39697, 0.39609, 0.39536, 0.39470, 0.39403,\
        0.39328, 0.39244, 0.39158, 0.39072, 0.38987, 0.38902, 0.38814, 0.38722, 0.38621, 0.38512, 0.38396,\
        0.38276, 0.38156, 0.38036, 0.37919, 0.37808, 0.37705, 0.37618, 0.37556, 0.37528, 0.37547, 0.37626,\
        0.37777, 0.38016, 0.38355, 0.38809, 0.39394, 0.40130, 0.41028, 0.42081, 0.43251, 0.44502, 0.45801,\
        0.47106, 0.48356, 0.49472, 0.50338, 0.51059, 0.51833, 0.52745, 0.53869, 0.55205, 0.56683, 0.58191,\
        0.59521, 0.60514, 0.61588, 0.62598, 0.63626, 0.64683, 0.65743, 0.66777, 0.67759, 0.68657, 0.69352,\
        0.69832, 0.70361, 0.70817, 0.71183, 0.71422, 0.71541, 0.71547, 0.71451, 0.71276, 0.71051, 0.70797,\
        0.70539, 0.70293, 0.70041, 0.69688, 0.69323, 0.68941, 0.68536, 0.68120, 0.67702, 0.67282, 0.66859,\
        0.66431, 0.65999, 0.65562, 0.65119, 0.64673, 0.64224, 0.63773, 0.63322, 0.62875, 0.62432, 0.61995,\
        0.61565, 0.61141, 0.60721, 0.60300, 0.59873, 0.59442, 0.58989, 0.58517, 0.58025, 0.57511, 0.56975,\
        0.56415, 0.55828, 0.55214, 0.54572, 0.53902, 0.53203, 0.52475, 0.51717, 0.50928, 0.50103, 0.49247],\
        [0.40413, 0.40331, 0.40223, 0.40105, 0.39991, 0.39868, 0.39762, 0.39703, 0.39664, 0.39624, 0.39583,\
        0.39524, 0.39447, 0.39368, 0.39290, 0.39212, 0.39132, 0.39050, 0.38960, 0.38856, 0.38739, 0.38613,\
        0.38482, 0.38348, 0.38215, 0.38084, 0.37953, 0.37827, 0.37713, 0.37621, 0.37563, 0.37554, 0.37607,\
        0.37739, 0.37965, 0.38301, 0.38762, 0.39357, 0.40102, 0.41022, 0.42080, 0.43259, 0.44530, 0.45869,\
        0.47247, 0.48607, 0.49826, 0.50785, 0.51539, 0.52294, 0.53194, 0.54315, 0.55661, 0.57166, 0.58702,\
        0.60102, 0.61251, 0.62351, 0.63455, 0.64551, 0.65632, 0.66682, 0.67689, 0.68636, 0.69521, 0.70268,\
        0.70711, 0.71163, 0.71575, 0.71896, 0.72108, 0.72207, 0.72200, 0.72098, 0.71921, 0.71691, 0.71429,\
        0.71153, 0.70870, 0.70556, 0.70188, 0.69812, 0.69411, 0.68972, 0.68534, 0.68100, 0.67665, 0.67227,\
        0.66785, 0.66337, 0.65883, 0.65425, 0.64962, 0.64496, 0.64029, 0.63563, 0.63100, 0.62643, 0.62192,\
        0.61749, 0.61314, 0.60885, 0.60459, 0.60034, 0.59590, 0.59125, 0.58643, 0.58143, 0.57622, 0.57078,\
        0.56508, 0.55911, 0.55286, 0.54631, 0.53947, 0.53235, 0.52496, 0.51732, 0.50943, 0.50122, 0.49252],\
        [0.40601, 0.40497, 0.40361, 0.40207, 0.40061, 0.39922, 0.39851, 0.39847, 0.39854, 0.39831, 0.39826,\
        0.39780, 0.39703, 0.39632, 0.39563, 0.39489, 0.39411, 0.39335, 0.39247, 0.39135, 0.39005, 0.38865,\
        0.38718, 0.38571, 0.38425, 0.38278, 0.38126, 0.37971, 0.37822, 0.37692, 0.37594, 0.37545, 0.37560,\
        0.37657, 0.37854, 0.38169, 0.38619, 0.39220, 0.39973, 0.40857, 0.41881, 0.43041, 0.44290, 0.45606,\
        0.46994, 0.48434, 0.49820, 0.50809, 0.51586, 0.52339, 0.53241, 0.54382, 0.55769, 0.57333, 0.58927,\
        0.60372, 0.61714, 0.62974, 0.64206, 0.65398, 0.66527, 0.67576, 0.68551, 0.69464, 0.70251, 0.70982,\
        0.71521, 0.71954, 0.72320, 0.72599, 0.72778, 0.72853, 0.72830, 0.72720, 0.72537, 0.72300, 0.72026,\
        0.71726, 0.71405, 0.71052, 0.70669, 0.70272, 0.69831, 0.69361, 0.68918, 0.68478, 0.68032, 0.67581,\
        0.67125, 0.66662, 0.66193, 0.65718, 0.65239, 0.64755, 0.64271, 0.63788, 0.63309, 0.62835, 0.62369,\
        0.61912, 0.61464, 0.61025, 0.60592, 0.60160, 0.59709, 0.59238, 0.58754, 0.58250, 0.57725, 0.57175,\
        0.56599, 0.55993, 0.55357, 0.54689, 0.53990, 0.53262, 0.52508, 0.51731, 0.50933, 0.50107, 0.49229],\
        [0.40855, 0.40738, 0.40579, 0.40388, 0.40177, 0.40034, 0.40037, 0.40080, 0.40121, 0.40116, 0.40134,\
        0.40098, 0.40026, 0.39966, 0.39906, 0.39829, 0.39746, 0.39676, 0.39589, 0.39457, 0.39310, 0.39155,\
        0.38987, 0.38821, 0.38665, 0.38506, 0.38330, 0.38140, 0.37948, 0.37769, 0.37620, 0.37518, 0.37479,\
        0.37523, 0.37668, 0.37935, 0.38343, 0.38908, 0.39629, 0.40445, 0.41442, 0.42576, 0.43749, 0.44952,\
        0.46244, 0.47651, 0.49056, 0.50264, 0.51089, 0.51890, 0.52843, 0.54048, 0.55515, 0.57180, 0.58944,\
        0.60560, 0.61997, 0.63449, 0.64860, 0.66185, 0.67388, 0.68449, 0.69361, 0.70259, 0.71049, 0.71728,\
        0.72280, 0.72713, 0.73048, 0.73291, 0.73437, 0.73486, 0.73444, 0.73321, 0.73128, 0.72880, 0.72589,\
        0.72266, 0.71913, 0.71529, 0.71116, 0.70671, 0.70185, 0.69729, 0.69293, 0.68846, 0.68388, 0.67924,\
        0.67454, 0.66978, 0.66494, 0.66002, 0.65504, 0.65003, 0.64500, 0.63998, 0.63501, 0.63009, 0.62527,\
        0.62054, 0.61591, 0.61138, 0.60691, 0.60243, 0.59790, 0.59331, 0.58850, 0.58347, 0.57821, 0.57269,\
        0.56689, 0.56077, 0.55431, 0.54751, 0.54036, 0.53289, 0.52513, 0.51714, 0.50894, 0.50048, 0.49172],\
        [0.41189, 0.41073, 0.40915, 0.40727, 0.40542, 0.40427, 0.40414, 0.40453, 0.40494, 0.40514, 0.40521,\
        0.40497, 0.40445, 0.40388, 0.40325, 0.40243, 0.40154, 0.40080, 0.39989, 0.39816, 0.39657, 0.39486,\
        0.39281, 0.39097, 0.38941, 0.38776, 0.38572, 0.38337, 0.38091, 0.37854, 0.37643, 0.37474, 0.37365,\
        0.37333, 0.37401, 0.37590, 0.37919, 0.38399, 0.39022, 0.39784, 0.40782, 0.41883, 0.42917, 0.43896,\
        0.44976, 0.46275, 0.47693, 0.48989, 0.50008, 0.50979, 0.52035, 0.53371, 0.54966, 0.56731, 0.58544,\
        0.60306, 0.62087, 0.63822, 0.65462, 0.66948, 0.68230, 0.69351, 0.70231, 0.71062, 0.71845, 0.72506,\
        0.73041, 0.73457, 0.73765, 0.73974, 0.74087, 0.74109, 0.74044, 0.73903, 0.73695, 0.73432, 0.73124,\
        0.72779, 0.72400, 0.71989, 0.71548, 0.71079, 0.70595, 0.70127, 0.69680, 0.69208, 0.68734, 0.68258,\
        0.67776, 0.67286, 0.66786, 0.66277, 0.65761, 0.65239, 0.64716, 0.64194, 0.63677, 0.63166, 0.62665,\
        0.62175, 0.61697, 0.61230, 0.60772, 0.60319, 0.59873, 0.59417, 0.58938, 0.58438, 0.57914, 0.57364,\
        0.56783, 0.56168, 0.55516, 0.54824, 0.54093, 0.53325, 0.52523, 0.51694, 0.50847, 0.49985, 0.49106],\
        [0.41607, 0.41510, 0.41374, 0.41219, 0.41075, 0.40981, 0.40952, 0.40967, 0.40993, 0.41011, 0.41012,\
        0.40990, 0.40948, 0.40892, 0.40824, 0.40742, 0.40649, 0.40549, 0.40415, 0.40232, 0.40057, 0.39861,\
        0.39585, 0.39391, 0.39264, 0.39101, 0.38857, 0.38562, 0.38253, 0.37950, 0.37669, 0.37421, 0.37224,\
        0.37097, 0.37060, 0.37139, 0.37356, 0.37727, 0.38267, 0.39001, 0.39959, 0.40972, 0.41821, 0.42469,\
        0.43238, 0.44452, 0.45898, 0.47306, 0.48575, 0.49717, 0.50969, 0.52514, 0.54306, 0.56217, 0.58150,\
        0.60110, 0.62181, 0.64205, 0.66077, 0.67737, 0.69089, 0.70127, 0.71125, 0.71950, 0.72688, 0.73313,\
        0.73817, 0.74201, 0.74476, 0.74650, 0.74729, 0.74721, 0.74632, 0.74469, 0.74244, 0.73963, 0.73637,\
        0.73271, 0.72871, 0.72439, 0.71980, 0.71499, 0.71011, 0.70532, 0.70050, 0.69553, 0.69071, 0.68586,\
        0.68093, 0.67589, 0.67072, 0.66544, 0.66008, 0.65465, 0.64921, 0.64377, 0.63838, 0.63307, 0.62786,\
        0.62278, 0.61784, 0.61304, 0.60839, 0.60389, 0.59948, 0.59488, 0.59016, 0.58523, 0.58007, 0.57463,\
        0.56886, 0.56272, 0.55617, 0.54917, 0.54172, 0.53384, 0.52554, 0.51692, 0.50811, 0.49931, 0.49051],\
        [0.42104, 0.42038, 0.41938, 0.41822, 0.41714, 0.41638, 0.41601, 0.41595, 0.41601, 0.41604, 0.41595,\
        0.41570, 0.41528, 0.41471, 0.41401, 0.41315, 0.41213, 0.41088, 0.40929, 0.40731, 0.40506, 0.40220,\
        0.39899, 0.39725, 0.39658, 0.39493, 0.39177, 0.38810, 0.38436, 0.38066, 0.37708, 0.37372, 0.37073,\
        0.36828, 0.36662, 0.36601, 0.36673, 0.36904, 0.37323, 0.37958, 0.38796, 0.39669, 0.40419, 0.40724,\
        0.41133, 0.42438, 0.43981, 0.45548, 0.47055, 0.48470, 0.49930, 0.51723, 0.53777, 0.55886, 0.58004,\
        0.60182, 0.62476, 0.64720, 0.66764, 0.68524, 0.70043, 0.71071, 0.72023, 0.72857, 0.73564, 0.74147,\
        0.74608, 0.74951, 0.75186, 0.75320, 0.75364, 0.75323, 0.75206, 0.75020, 0.74774, 0.74475, 0.74130,\
        0.73746, 0.73328, 0.72880, 0.72407, 0.71916, 0.71416, 0.70912, 0.70399, 0.69897, 0.69410, 0.68912,\
        0.68408, 0.67889, 0.67354, 0.66805, 0.66247, 0.65681, 0.65114, 0.64547, 0.63986, 0.63432, 0.62890,\
        0.62363, 0.61853, 0.61361, 0.60888, 0.60430, 0.59979, 0.59546, 0.59087, 0.58605, 0.58101, 0.57570,\
        0.57003, 0.56395, 0.55741, 0.55038, 0.54283, 0.53479, 0.52627, 0.51734, 0.50812, 0.49898, 0.49033],\
        [0.42668, 0.42640, 0.42581, 0.42505, 0.42429, 0.42368, 0.42328, 0.42306, 0.42292, 0.42278, 0.42256,\
        0.42222, 0.42176, 0.42117, 0.42045, 0.41957, 0.41847, 0.41710, 0.41534, 0.41315, 0.41046, 0.40724,\
        0.40404, 0.40203, 0.40125, 0.39911, 0.39500, 0.39076, 0.38651, 0.38217, 0.37780, 0.37348, 0.36933,\
        0.36552, 0.36233, 0.36003, 0.35894, 0.35937, 0.36164, 0.36591, 0.37188, 0.37854, 0.38411, 0.38679,\
        0.39175, 0.40590, 0.42103, 0.43897, 0.45723, 0.47486, 0.49302, 0.51341, 0.53568, 0.55886, 0.58192,\
        0.60569, 0.63037, 0.65429, 0.67595, 0.69374, 0.70774, 0.71997, 0.72974, 0.73790, 0.74462, 0.75000,\
        0.75411, 0.75705, 0.75893, 0.75984, 0.75989, 0.75914, 0.75767, 0.75556, 0.75288, 0.74969, 0.74607,\
        0.74206, 0.73773, 0.73311, 0.72827, 0.72325, 0.71813, 0.71293, 0.70771, 0.70261, 0.69741, 0.69237,\
        0.68725, 0.68188, 0.67632, 0.67060, 0.66477, 0.65888, 0.65297, 0.64706, 0.64121, 0.63545, 0.62981,\
        0.62434, 0.61906, 0.61401, 0.60922, 0.60473, 0.60055, 0.59609, 0.59150, 0.58684, 0.58200, 0.57688,\
        0.57138, 0.56542, 0.55895, 0.55192, 0.54434, 0.53622, 0.52758, 0.51851, 0.50911, 0.49978, 0.49110],\
        [0.43283, 0.43299, 0.43282, 0.43245, 0.43197, 0.43151, 0.43110, 0.43076, 0.43046, 0.43013, 0.42976,\
        0.42932, 0.42880, 0.42819, 0.42747, 0.42660, 0.42551, 0.42413, 0.42234, 0.42009, 0.41733, 0.41418,\
        0.41106, 0.40849, 0.40619, 0.40278, 0.39814, 0.39393, 0.38924, 0.38422, 0.37906, 0.37376, 0.36834,\
        0.36302, 0.35808, 0.35386, 0.35065, 0.34877, 0.34852, 0.35007, 0.35321, 0.35716, 0.36088, 0.36478,\
        0.37249, 0.38591, 0.40266, 0.42495, 0.44631, 0.46723, 0.48892, 0.51136, 0.53651, 0.56270, 0.58800,\
        0.61292, 0.63858, 0.66314, 0.68548, 0.70411, 0.71809, 0.72978, 0.73949, 0.74741, 0.75373, 0.75861,\
        0.76219, 0.76458, 0.76594, 0.76639, 0.76602, 0.76491, 0.76314, 0.76076, 0.75785, 0.75446, 0.75067,\
        0.74651, 0.74204, 0.73731, 0.73236, 0.72725, 0.72202, 0.71672, 0.71139, 0.70604, 0.70073, 0.69580,\
        0.69046, 0.68484, 0.67902, 0.67305, 0.66698, 0.66085, 0.65469, 0.64856, 0.64247, 0.63648, 0.63061,\
        0.62493, 0.61946, 0.61424, 0.60933, 0.60475, 0.60044, 0.59623, 0.59196, 0.58761, 0.58307, 0.57823,\
        0.57296, 0.56717, 0.56081, 0.55384, 0.54627, 0.53813, 0.52948, 0.52046, 0.51123, 0.50209, 0.49292],\
        [0.43936, 0.43996, 0.44022, 0.44021, 0.44001, 0.43968, 0.43929, 0.43886, 0.43840, 0.43790, 0.43737,\
        0.43681, 0.43623, 0.43561, 0.43493, 0.43414, 0.43316, 0.43190, 0.43025, 0.42813, 0.42550, 0.42247,\
        0.41925, 0.41598, 0.41239, 0.40798, 0.40312, 0.39835, 0.39278, 0.38701, 0.38116, 0.37488, 0.36810,\
        0.36111, 0.35430, 0.34803, 0.34256, 0.33814, 0.33506, 0.33366, 0.33406, 0.33575, 0.33792, 0.34178,\
        0.35045, 0.36583, 0.38821, 0.41312, 0.43667, 0.46058, 0.48496, 0.51098, 0.54007, 0.56959, 0.59818,\
        0.62384, 0.64882, 0.67293, 0.69510, 0.71356, 0.72813, 0.73985, 0.74938, 0.75700, 0.76290, 0.76725,\
        0.77024, 0.77204, 0.77285, 0.77281, 0.77202, 0.77054, 0.76845, 0.76580, 0.76265, 0.75907, 0.75510,\
        0.75080, 0.74621, 0.74137, 0.73633, 0.73113, 0.72583, 0.72045, 0.71505, 0.70968, 0.70447, 0.69922,\
        0.69358, 0.68769, 0.68160, 0.67539, 0.66908, 0.66272, 0.65633, 0.64996, 0.64365, 0.63743, 0.63135,\
        0.62544, 0.61977, 0.61438, 0.60930, 0.60456, 0.60017, 0.59622, 0.59237, 0.58844, 0.58430, 0.57980,\
        0.57482, 0.56925, 0.56303, 0.55613, 0.54858, 0.54043, 0.53179, 0.52280, 0.51367, 0.50450, 0.49493],\
        [0.44610, 0.44716, 0.44784, 0.44819, 0.44824, 0.44805, 0.44768, 0.44716, 0.44655, 0.44588, 0.44519,\
        0.44452, 0.44388, 0.44327, 0.44268, 0.44205, 0.44129, 0.44029, 0.43893, 0.43710, 0.43472, 0.43182,\
        0.42846, 0.42465, 0.42024, 0.41515, 0.40957, 0.40358, 0.39701, 0.39066, 0.38445, 0.37723, 0.36892,\
        0.36013, 0.35140, 0.34312, 0.33549, 0.32862, 0.32272, 0.31839, 0.31636, 0.31657, 0.31710, 0.31910,\
        0.32754, 0.34433, 0.36971, 0.39855, 0.42614, 0.45483, 0.48456, 0.51482, 0.54579, 0.57730, 0.60744,\
        0.63471, 0.65966, 0.68301, 0.70456, 0.72303, 0.73798, 0.74988, 0.75929, 0.76659, 0.77204, 0.77585,\
        0.77820, 0.77935, 0.77958, 0.77905, 0.77784, 0.77600, 0.77359, 0.77066, 0.76728, 0.76350, 0.75936,\
        0.75492, 0.75021, 0.74528, 0.74015, 0.73488, 0.72950, 0.72405, 0.71858, 0.71314, 0.70773, 0.70221,\
        0.69641, 0.69031, 0.68400, 0.67757, 0.67106, 0.66448, 0.65788, 0.65130, 0.64477, 0.63834, 0.63205,\
        0.62595, 0.62008, 0.61451, 0.60930, 0.60453, 0.60025, 0.59648, 0.59296, 0.58948, 0.58578, 0.58168,\
        0.57702, 0.57168, 0.56560, 0.55877, 0.55120, 0.54300, 0.53427, 0.52517, 0.51582, 0.50624, 0.49627],\
        [0.45289, 0.45442, 0.45552, 0.45622, 0.45652, 0.45646, 0.45610, 0.45549, 0.45472, 0.45387, 0.45302,\
        0.45223, 0.45155, 0.45099, 0.45053, 0.45014, 0.44971, 0.44911, 0.44819, 0.44679, 0.44477, 0.44208,\
        0.43868, 0.43454, 0.42961, 0.42385, 0.41732, 0.41002, 0.40226, 0.39546, 0.38955, 0.38110, 0.37097,\
        0.36028, 0.34970, 0.33963, 0.33022, 0.32144, 0.31318, 0.30612, 0.30185, 0.30105, 0.30071, 0.29937,\
        0.30586, 0.32233, 0.34992, 0.38204, 0.41565, 0.44948, 0.48308, 0.51717, 0.55165, 0.58542, 0.61728,\
        0.64590, 0.67117, 0.69374, 0.71487, 0.73327, 0.74824, 0.76005, 0.76921, 0.77613, 0.78111, 0.78435,\
        0.78602, 0.78646, 0.78608, 0.78509, 0.78350, 0.78129, 0.77854, 0.77533, 0.77172, 0.76774, 0.76345,\
        0.75887, 0.75405, 0.74902, 0.74381, 0.73846, 0.73299, 0.72746, 0.72187, 0.71627, 0.71062, 0.70488,\
        0.69895, 0.69262, 0.68617, 0.67960, 0.67290, 0.66614, 0.65935, 0.65258, 0.64586, 0.63925, 0.63278,\
        0.62650, 0.62047, 0.61474, 0.60941, 0.60463, 0.60056, 0.59693, 0.59380, 0.59082, 0.58761, 0.58393,\
        0.57959, 0.57449, 0.56854, 0.56172, 0.55408, 0.54573, 0.53679, 0.52742, 0.51771, 0.50764, 0.49728],\
        [0.45958, 0.46158, 0.46312, 0.46417, 0.46472, 0.46477, 0.46440, 0.46367, 0.46273, 0.46169, 0.46067,\
        0.45976, 0.45904, 0.45854, 0.45827, 0.45818, 0.45819, 0.45813, 0.45780, 0.45698, 0.45545, 0.45308,\
        0.44979, 0.44555, 0.44034, 0.43414, 0.42697, 0.41886, 0.41015, 0.40227, 0.39661, 0.38625, 0.37420,\
        0.36165, 0.34939, 0.33785, 0.32719, 0.31739, 0.30822, 0.29945, 0.29322, 0.28940, 0.28645, 0.28472,\
        0.28949, 0.30560, 0.33327, 0.37016, 0.40894, 0.44544, 0.48131, 0.51879, 0.55785, 0.59526, 0.62897,\
        0.65859, 0.68433, 0.70696, 0.72723, 0.74486, 0.75923, 0.77049, 0.77915, 0.78558, 0.78999, 0.79273,\
        0.79364, 0.79330, 0.79233, 0.79095, 0.78899, 0.78639, 0.78330, 0.77981, 0.77596, 0.77179, 0.76734,\
        0.76264, 0.75771, 0.75259, 0.74729, 0.74185, 0.73630, 0.73065, 0.72492, 0.71912, 0.71322, 0.70716,\
        0.70083, 0.69460, 0.68818, 0.68148, 0.67463, 0.66771, 0.66075, 0.65382, 0.64694, 0.64018, 0.63357,\
        0.62717, 0.62101, 0.61517, 0.60972, 0.60478, 0.60054, 0.59739, 0.59496, 0.59259, 0.58988, 0.58659,\
        0.58257, 0.57768, 0.57181, 0.56495, 0.55715, 0.54854, 0.53927, 0.52952, 0.51943, 0.50912, 0.49860],\
        [0.46602, 0.46848, 0.47045, 0.47187, 0.47267, 0.47283, 0.47241, 0.47155, 0.47040, 0.46914, 0.46794,\
        0.46691, 0.46616, 0.46574, 0.46568, 0.46595, 0.46647, 0.46707, 0.46749, 0.46741, 0.46652, 0.46462,\
        0.46162, 0.45751, 0.45229, 0.44601, 0.43869, 0.43043, 0.42159, 0.41309, 0.40418, 0.39248, 0.37861,\
        0.36438, 0.35067, 0.33787, 0.32624, 0.31592, 0.30690, 0.29818, 0.28888, 0.28142, 0.27592, 0.27354,\
        0.27775, 0.29209, 0.32111, 0.36111, 0.40409, 0.44308, 0.48033, 0.52038, 0.56370, 0.60596, 0.64208,\
        0.67279, 0.69862, 0.72142, 0.74099, 0.75753, 0.77088, 0.78112, 0.78895, 0.79492, 0.79847, 0.80081,\
        0.80098, 0.79993, 0.79839, 0.79673, 0.79435, 0.79127, 0.78784, 0.78407, 0.77999, 0.77565, 0.77105,\
        0.76623, 0.76119, 0.75597, 0.75059, 0.74506, 0.73940, 0.73363, 0.72776, 0.72178, 0.71567, 0.70942,\
        0.70312, 0.69686, 0.69013, 0.68324, 0.67625, 0.66918, 0.66208, 0.65501, 0.64801, 0.64114, 0.63445,\
        0.62799, 0.62180, 0.61593, 0.61045, 0.60547, 0.60124, 0.59841, 0.59678, 0.59495, 0.59261, 0.58966,\
        0.58594, 0.58124, 0.57541, 0.56842, 0.56035, 0.55136, 0.54166, 0.53141, 0.52080, 0.50997, 0.49891],\
        [0.47204, 0.47495, 0.47736, 0.47916, 0.48021, 0.48047, 0.47997, 0.47892, 0.47753, 0.47604, 0.47464,\
        0.47350, 0.47272, 0.47238, 0.47254, 0.47320, 0.47428, 0.47563, 0.47695, 0.47779, 0.47770, 0.47640,\
        0.47386, 0.47011, 0.46516, 0.45914, 0.45215, 0.44416, 0.43521, 0.42537, 0.41402, 0.40016, 0.38460,\
        0.36891, 0.35389, 0.33988, 0.32707, 0.31579, 0.30607, 0.29652, 0.28624, 0.27700, 0.26986, 0.26636,\
        0.26942, 0.28313, 0.31148, 0.35392, 0.40071, 0.44307, 0.48218, 0.52391, 0.56869, 0.61316, 0.65465,\
        0.68868, 0.71444, 0.73634, 0.75490, 0.77043, 0.78278, 0.79184, 0.79827, 0.80375, 0.80682, 0.80827,\
        0.80807, 0.80663, 0.80463, 0.80242, 0.79939, 0.79588, 0.79216, 0.78810, 0.78383, 0.77932, 0.77458,\
        0.76963, 0.76449, 0.75917, 0.75370, 0.74807, 0.74231, 0.73642, 0.73041, 0.72428, 0.71802, 0.71167,\
        0.70527, 0.69867, 0.69182, 0.68485, 0.67775, 0.67056, 0.66334, 0.65616, 0.64907, 0.64213, 0.63541,\
        0.62896, 0.62284, 0.61709, 0.61179, 0.60706, 0.60315, 0.60075, 0.59954, 0.59784, 0.59573, 0.59308,\
        0.58963, 0.58512, 0.57928, 0.57206, 0.56361, 0.55415, 0.54391, 0.53309, 0.52185, 0.51034, 0.49871],\
        [0.47747, 0.48079, 0.48362, 0.48582, 0.48716, 0.48748, 0.48687, 0.48558, 0.48392, 0.48219, 0.48062,\
        0.47937, 0.47857, 0.47832, 0.47868, 0.47970, 0.48135, 0.48349, 0.48582, 0.48775, 0.48859, 0.48799,\
        0.48606, 0.48296, 0.47846, 0.47301, 0.46679, 0.45940, 0.45047, 0.43971, 0.42660, 0.41080, 0.39338,\
        0.37609, 0.35985, 0.34453, 0.32999, 0.31672, 0.30565, 0.29563, 0.28542, 0.27613, 0.26839, 0.26384,\
        0.26528, 0.27660, 0.30323, 0.34916, 0.40033, 0.44557, 0.48690, 0.52977, 0.57511, 0.62116, 0.66561,\
        0.70335, 0.72943, 0.75009, 0.76758, 0.78247, 0.79424, 0.80248, 0.80805, 0.81228, 0.81484, 0.81567,\
        0.81503, 0.81325, 0.81077, 0.80778, 0.80419, 0.80036, 0.79625, 0.79189, 0.78749, 0.78283, 0.77793,\
        0.77285, 0.76760, 0.76219, 0.75663, 0.75090, 0.74503, 0.73903, 0.73289, 0.72662, 0.72023, 0.71371,\
        0.70703, 0.70018, 0.69335, 0.68635, 0.67916, 0.67186, 0.66452, 0.65723, 0.65007, 0.64310, 0.63641,\
        0.63005, 0.62408, 0.61859, 0.61364, 0.60940, 0.60608, 0.60392, 0.60252, 0.60093, 0.59907, 0.59670,\
        0.59352, 0.58927, 0.58335, 0.57576, 0.56682, 0.55681, 0.54600, 0.53459, 0.52276, 0.51069, 0.49867],\
        [0.48216, 0.48579, 0.48898, 0.49160, 0.49327, 0.49362, 0.49284, 0.49128, 0.48936, 0.48743, 0.48574,\
        0.48443, 0.48363, 0.48344, 0.48399, 0.48532, 0.48747, 0.49032, 0.49364, 0.49680, 0.49855, 0.49875,\
        0.49761, 0.49560, 0.49162, 0.48701, 0.48213, 0.47574, 0.46724, 0.45634, 0.44261, 0.42562, 0.40616,\
        0.38709, 0.36972, 0.35313, 0.33644, 0.31981, 0.30506, 0.29500, 0.28752, 0.27951, 0.27192, 0.26651,\
        0.26635, 0.27611, 0.30172, 0.34684, 0.40002, 0.44641, 0.49092, 0.53443, 0.57970, 0.62689, 0.67237,\
        0.71073, 0.73924, 0.76068, 0.77782, 0.79220, 0.80411, 0.81198, 0.81736, 0.82088, 0.82273, 0.82299,\
        0.82187, 0.81967, 0.81668, 0.81308, 0.80899, 0.80456, 0.79971, 0.79557, 0.79109, 0.78618, 0.78109,\
        0.77588, 0.77054, 0.76504, 0.75938, 0.75355, 0.74757, 0.74144, 0.73519, 0.72880, 0.72228, 0.71563,\
        0.70884, 0.70195, 0.69501, 0.68784, 0.68049, 0.67304, 0.66559, 0.65820, 0.65097, 0.64400, 0.63736,\
        0.63113, 0.62539, 0.62022, 0.61570, 0.61195, 0.60908, 0.60707, 0.60559, 0.60417, 0.60251, 0.60024,\
        0.59748, 0.59362, 0.58743, 0.57930, 0.56979, 0.55923, 0.54787, 0.53592, 0.52359, 0.51104, 0.49858],\
        [0.48600, 0.48978, 0.49315, 0.49607, 0.49814, 0.49844, 0.49753, 0.49575, 0.49364, 0.49165, 0.48994,\
        0.48864, 0.48787, 0.48777, 0.48845, 0.49005, 0.49260, 0.49601, 0.50005, 0.50410, 0.50685, 0.50811,\
        0.50816, 0.50708, 0.50419, 0.50094, 0.49774, 0.49261, 0.48498, 0.47481, 0.46186, 0.44474, 0.42414,\
        0.40334, 0.38450, 0.36723, 0.34864, 0.32880, 0.31005, 0.30025, 0.29502, 0.28789, 0.28063, 0.27435,\
        0.27210, 0.27928, 0.30026, 0.34204, 0.39101, 0.43901, 0.48605, 0.53217, 0.57984, 0.62935, 0.67670,\
        0.71604, 0.74587, 0.76868, 0.78638, 0.79992, 0.81181, 0.82048, 0.82601, 0.82920, 0.83050, 0.83020,\
        0.82858, 0.82590, 0.82242, 0.81832, 0.81375, 0.80887, 0.80402, 0.79958, 0.79460, 0.78930, 0.78405,\
        0.77875, 0.77333, 0.76774, 0.76197, 0.75602, 0.74991, 0.74366, 0.73728, 0.73079, 0.72417, 0.71743,\
        0.71060, 0.70373, 0.69664, 0.68919, 0.68167, 0.67409, 0.66650, 0.65901, 0.65171, 0.64473, 0.63815,\
        0.63208, 0.62660, 0.62177, 0.61766, 0.61434, 0.61182, 0.61000, 0.60861, 0.60732, 0.60582, 0.60386,\
        0.60153, 0.59787, 0.59096, 0.58230, 0.57232, 0.56127, 0.54941, 0.53700, 0.52430, 0.51150, 0.49878],\
        [0.48897, 0.49272, 0.49597, 0.49869, 0.50083, 0.50130, 0.50050, 0.49866, 0.49664, 0.49481, 0.49323,\
        0.49203, 0.49136, 0.49136, 0.49219, 0.49400, 0.49696, 0.50091, 0.50555, 0.51012, 0.51382, 0.51630,\
        0.51759, 0.51766, 0.51653, 0.51489, 0.51272, 0.50890, 0.50263, 0.49389, 0.48344, 0.46696, 0.44553,\
        0.42392, 0.40471, 0.38757, 0.36746, 0.34597, 0.32717, 0.31536, 0.30797, 0.30055, 0.29394, 0.28633,\
        0.28136, 0.28591, 0.30499, 0.33912, 0.38319, 0.42840, 0.47458, 0.52504, 0.57919, 0.63181, 0.67961,\
        0.71994, 0.75147, 0.77579, 0.79473, 0.80942, 0.82077, 0.82922, 0.83460, 0.83741, 0.83816, 0.83729,\
        0.83513, 0.83196, 0.82801, 0.82348, 0.81852, 0.81333, 0.80814, 0.80294, 0.79740, 0.79211, 0.78685,\
        0.78151, 0.77601, 0.77030, 0.76439, 0.75829, 0.75204, 0.74565, 0.73916, 0.73256, 0.72585, 0.71902,\
        0.71205, 0.70492, 0.69762, 0.69025, 0.68268, 0.67497, 0.66723, 0.65959, 0.65221, 0.64519, 0.63868,\
        0.63276, 0.62751, 0.62301, 0.61929, 0.61635, 0.61415, 0.61256, 0.61133, 0.61018, 0.60883, 0.60701,\
        0.60441, 0.59995, 0.59322, 0.58451, 0.57427, 0.56284, 0.55054, 0.53770, 0.52465, 0.51174, 0.49897],\
        [0.49116, 0.49480, 0.49779, 0.50010, 0.50184, 0.50243, 0.50172, 0.50014, 0.49860, 0.49711, 0.49575,\
        0.49472, 0.49422, 0.49440, 0.49543, 0.49747, 0.50084, 0.50544, 0.51056, 0.51565, 0.52014, 0.52370,\
        0.52621, 0.52767, 0.52817, 0.52790, 0.52673, 0.52406, 0.51936, 0.51280, 0.50490, 0.48841, 0.46687,\
        0.44626, 0.42833, 0.41098, 0.39021, 0.36771, 0.34863, 0.33508, 0.32519, 0.31662, 0.30896, 0.29883,\
        0.29394, 0.29462, 0.31055, 0.34005, 0.37643, 0.41700, 0.46381, 0.51697, 0.57565, 0.63297, 0.68301,\
        0.72554, 0.75858, 0.78381, 0.80375, 0.81926, 0.83085, 0.83885, 0.84364, 0.84576, 0.84580, 0.84426,\
        0.84151, 0.83783, 0.83344, 0.82851, 0.82320, 0.81766, 0.81198, 0.80615, 0.80041, 0.79497, 0.78965,\
        0.78425, 0.77863, 0.77275, 0.76664, 0.76035, 0.75392, 0.74738, 0.74076, 0.73407, 0.72730, 0.72041,\
        0.71335, 0.70606, 0.69869, 0.69130, 0.68358, 0.67567, 0.66771, 0.65989, 0.65237, 0.64530, 0.63881,\
        0.63301, 0.62799, 0.62379, 0.62041, 0.61783, 0.61594, 0.61460, 0.61357, 0.61256, 0.61127, 0.60933,\
        0.60628, 0.60150, 0.59483, 0.58613, 0.57570, 0.56395, 0.55124, 0.53791, 0.52434, 0.51091, 0.49783],\
        [0.49268, 0.49623, 0.49904, 0.50110, 0.50242, 0.50285, 0.50234, 0.50122, 0.50001, 0.49874, 0.49760,\
        0.49680, 0.49656, 0.49702, 0.49840, 0.50097, 0.50479, 0.50977, 0.51530, 0.52087, 0.52606, 0.53056,\
        0.53422, 0.53698, 0.53886, 0.53983, 0.53973, 0.53818, 0.53484, 0.52940, 0.52055, 0.50527, 0.48608,\
        0.46755, 0.45199, 0.43505, 0.41342, 0.39002, 0.37090, 0.35706, 0.34562, 0.33549, 0.32578, 0.31641,\
        0.31132, 0.31143, 0.32369, 0.34758, 0.37658, 0.41029, 0.45222, 0.50441, 0.56967, 0.63412, 0.69208,\
        0.73866, 0.77034, 0.79420, 0.81479, 0.83099, 0.84247, 0.84965, 0.85333, 0.85436, 0.85345, 0.85110,\
        0.84769, 0.84347, 0.83864, 0.83335, 0.82773, 0.82188, 0.81588, 0.80980, 0.80381, 0.79804, 0.79260,\
        0.78707, 0.78121, 0.77507, 0.76869, 0.76216, 0.75551, 0.74879, 0.74204, 0.73527, 0.72846, 0.72158,\
        0.71459, 0.70745, 0.70014, 0.69239, 0.68435, 0.67614, 0.66790, 0.65983, 0.65211, 0.64493, 0.63843,\
        0.63273, 0.62790, 0.62397, 0.62092, 0.61868, 0.61711, 0.61605, 0.61523, 0.61437, 0.61313, 0.61111,\
        0.60788, 0.60309, 0.59630, 0.58736, 0.57672, 0.56468, 0.55157, 0.53775, 0.52356, 0.50938, 0.49580],\
        [0.49359, 0.49709, 0.49984, 0.50158, 0.50262, 0.50295, 0.50260, 0.50181, 0.50082, 0.49981, 0.49889,\
        0.49831, 0.49842, 0.49940, 0.50121, 0.50439, 0.50872, 0.51401, 0.51985, 0.52585, 0.53164, 0.53696,\
        0.54163, 0.54555, 0.54863, 0.55075, 0.55171, 0.55122, 0.54888, 0.54406, 0.53548, 0.52205, 0.50498,\
        0.48821, 0.47779, 0.46196, 0.43763, 0.41201, 0.39371, 0.38016, 0.36772, 0.35729, 0.34810, 0.33973,\
        0.33437, 0.33527, 0.34550, 0.36405, 0.38459, 0.40724, 0.44456, 0.50212, 0.56976, 0.64066, 0.71115,\
        0.76413, 0.78796, 0.80977, 0.82998, 0.84591, 0.85631, 0.86177, 0.86367, 0.86317, 0.86103, 0.85774,\
        0.85361, 0.84883, 0.84357, 0.83795, 0.83206, 0.82598, 0.81979, 0.81357, 0.80740, 0.80143, 0.79587,\
        0.78998, 0.78371, 0.77719, 0.77048, 0.76364, 0.75673, 0.74981, 0.74291, 0.73606, 0.72924, 0.72242,\
        0.71552, 0.70846, 0.70107, 0.69317, 0.68484, 0.67628, 0.66770, 0.65932, 0.65135, 0.64401, 0.63746,\
        0.63182, 0.62716, 0.62348, 0.62075, 0.61884, 0.61761, 0.61684, 0.61627, 0.61560, 0.61447, 0.61246,\
        0.60915, 0.60408, 0.59711, 0.58827, 0.57745, 0.56511, 0.55166, 0.53747, 0.52286, 0.50817, 0.49429],\
        [0.49394, 0.49727, 0.49979, 0.50151, 0.50248, 0.50279, 0.50255, 0.50195, 0.50117, 0.50038, 0.49960,\
        0.49918, 0.49976, 0.50132, 0.50393, 0.50766, 0.51244, 0.51805, 0.52420, 0.53057, 0.53688, 0.54290,\
        0.54845, 0.55339, 0.55756, 0.56078, 0.56280, 0.56337, 0.56218, 0.55871, 0.55207, 0.54154, 0.52776,\
        0.51513, 0.50797, 0.49160, 0.46311, 0.43581, 0.41519, 0.40028, 0.38929, 0.38115, 0.37488, 0.36801,\
        0.36287, 0.36411, 0.37241, 0.38750, 0.40081, 0.40996, 0.43605, 0.49445, 0.58099, 0.65670, 0.73397,\
        0.80203, 0.81406, 0.83269, 0.85081, 0.86469, 0.87251, 0.87495, 0.87436, 0.87195, 0.86839, 0.86407,\
        0.85918, 0.85384, 0.84815, 0.84221, 0.83609, 0.82984, 0.82353, 0.81724, 0.81104, 0.80508, 0.79918,\
        0.79267, 0.78592, 0.77900, 0.77192, 0.76473, 0.75752, 0.75035, 0.74328, 0.73634, 0.72952, 0.72276,\
        0.71597, 0.70900, 0.70160, 0.69356, 0.68494, 0.67601, 0.66704, 0.65829, 0.65003, 0.64247, 0.63582,\
        0.63021, 0.62570, 0.62227, 0.61985, 0.61829, 0.61740, 0.61696, 0.61668, 0.61626, 0.61531, 0.61344,\
        0.61022, 0.60527, 0.59832, 0.58910, 0.57792, 0.56524, 0.55150, 0.53709, 0.52243, 0.50794, 0.49393],\
        [0.49389, 0.49707, 0.49949, 0.50116, 0.50210, 0.50244, 0.50230, 0.50181, 0.50114, 0.50044, 0.49975,\
        0.49951, 0.50054, 0.50298, 0.50639, 0.51073, 0.51595, 0.52188, 0.52831, 0.53501, 0.54175, 0.54835,\
        0.55465, 0.56049, 0.56569, 0.57000, 0.57314, 0.57487, 0.57512, 0.57380, 0.57045, 0.56427, 0.55556,\
        0.54714, 0.53852, 0.52006, 0.49012, 0.46015, 0.43653, 0.41957, 0.41115, 0.40609, 0.40463, 0.40041,\
        0.39728, 0.40027, 0.40568, 0.41514, 0.41896, 0.41870, 0.43533, 0.49646, 0.60296, 0.67657, 0.75241,\
        0.81776, 0.84616, 0.86215, 0.87502, 0.88600, 0.88996, 0.88823, 0.88491, 0.88037, 0.87534, 0.86998,\
        0.86432, 0.85841, 0.85230, 0.84606, 0.83972, 0.83332, 0.82692, 0.82054, 0.81423, 0.80795, 0.80150,\
        0.79469, 0.78767, 0.78040, 0.77292, 0.76535, 0.75779, 0.75034, 0.74308, 0.73604, 0.72920, 0.72252,\
        0.71583, 0.70897, 0.70173, 0.69350, 0.68455, 0.67526, 0.66588, 0.65672, 0.64809, 0.64026, 0.63347,\
        0.62786, 0.62349, 0.62033, 0.61824, 0.61704, 0.61651, 0.61642, 0.61647, 0.61634, 0.61566, 0.61401,\
        0.61098, 0.60620, 0.59919, 0.58958, 0.57799, 0.56495, 0.55094, 0.53640, 0.52172, 0.50724, 0.49310],\
        [0.49359, 0.49671, 0.49913, 0.50074, 0.50166, 0.50206, 0.50201, 0.50161, 0.50100, 0.50036, 0.49978,\
        0.49996, 0.50172, 0.50482, 0.50887, 0.51374, 0.51932, 0.52552, 0.53217, 0.53912, 0.54619, 0.55325,\
        0.56017, 0.56682, 0.57300, 0.57845, 0.58280, 0.58577, 0.58755, 0.58871, 0.58929, 0.58796, 0.58327,\
        0.57772, 0.57182, 0.55623, 0.52626, 0.49100, 0.46348, 0.44433, 0.43511, 0.43312, 0.43503, 0.43676,\
        0.43972, 0.44512, 0.44905, 0.45148, 0.44954, 0.44342, 0.45669, 0.53517, 0.63546, 0.69722, 0.77161,\
        0.83791, 0.87281, 0.89128, 0.89930, 0.90533, 0.90598, 0.90100, 0.89499, 0.88813, 0.88177, 0.87541,\
        0.86896, 0.86245, 0.85592, 0.84938, 0.84282, 0.83628, 0.82976, 0.82328, 0.81679, 0.81019, 0.80330,\
        0.79627, 0.78902, 0.78135, 0.77342, 0.76541, 0.75747, 0.74971, 0.74222, 0.73505, 0.72821, 0.72162,\
        0.71506, 0.70822, 0.70078, 0.69267, 0.68364, 0.67402, 0.66421, 0.65459, 0.64554, 0.63738, 0.63039,\
        0.62477, 0.62056, 0.61769, 0.61596, 0.61514, 0.61499, 0.61525, 0.61564, 0.61585, 0.61548, 0.61408,\
        0.61121, 0.60642, 0.59924, 0.58937, 0.57743, 0.56404, 0.54977, 0.53510, 0.52038, 0.50580, 0.49159],\
        [0.49315, 0.49625, 0.49860, 0.50017, 0.50128, 0.50184, 0.50193, 0.50165, 0.50111, 0.50048, 0.49997,\
        0.50107, 0.50357, 0.50716, 0.51163, 0.51682, 0.52265, 0.52902, 0.53581, 0.54291, 0.55019, 0.55757,\
        0.56495, 0.57225, 0.57935, 0.58598, 0.59169, 0.59605, 0.59932, 0.60286, 0.60671, 0.60916, 0.60802,\
        0.60574, 0.60515, 0.59635, 0.57167, 0.53215, 0.49691, 0.47199, 0.46185, 0.46150, 0.46647, 0.47338,\
        0.48084, 0.48873, 0.49069, 0.48937, 0.49049, 0.48595, 0.50169, 0.58576, 0.66382, 0.72315, 0.79880,\
        0.87522, 0.90447, 0.92022, 0.92528, 0.92500, 0.92119, 0.91355, 0.90414, 0.89536, 0.88787, 0.88033,\
        0.87300, 0.86589, 0.85894, 0.85208, 0.84530, 0.83859, 0.83195, 0.82534, 0.81872, 0.81199, 0.80505,\
        0.79783, 0.79002, 0.78176, 0.77332, 0.76486, 0.75652, 0.74842, 0.74067, 0.73335, 0.72649, 0.72004,\
        0.71378, 0.70724, 0.69991, 0.69169, 0.68242, 0.67238, 0.66208, 0.65196, 0.64243, 0.63388, 0.62664,\
        0.62098, 0.61696, 0.61444, 0.61313, 0.61271, 0.61291, 0.61350, 0.61425, 0.61481, 0.61478, 0.61365,\
        0.61091, 0.60604, 0.59859, 0.58846, 0.57617, 0.56240, 0.54780, 0.53298, 0.51835, 0.50391, 0.48969],\
        [0.49269, 0.49582, 0.49825, 0.50004, 0.50124, 0.50196, 0.50226, 0.50220, 0.50192, 0.50163, 0.50186,\
        0.50341, 0.50624, 0.51008, 0.51474, 0.52008, 0.52601, 0.53243, 0.53925, 0.54639, 0.55376, 0.56128,\
        0.56892, 0.57666, 0.58448, 0.59220, 0.59927, 0.60517, 0.61045, 0.61605, 0.62180, 0.62647, 0.62905,\
        0.63059, 0.63089, 0.62441, 0.60163, 0.57147, 0.53393, 0.50463, 0.49179, 0.49086, 0.49536, 0.50317,\
        0.50930, 0.51470, 0.51570, 0.51569, 0.51983, 0.52471, 0.55073, 0.61839, 0.69165, 0.76478, 0.85399,\
        0.92631, 0.94371, 0.95037, 0.94988, 0.94451, 0.93601, 0.92522, 0.91356, 0.90307, 0.89335, 0.88445,\
        0.87628, 0.86861, 0.86125, 0.85408, 0.84705, 0.84015, 0.83334, 0.82661, 0.81987, 0.81304, 0.80601,\
        0.79853, 0.79017, 0.78142, 0.77252, 0.76363, 0.75490, 0.74646, 0.73843, 0.73091, 0.72398, 0.71768,\
        0.71188, 0.70615, 0.69933, 0.69081, 0.68095, 0.67036, 0.65954, 0.64890, 0.63886, 0.62986, 0.62231,\
        0.61658, 0.61281, 0.61075, 0.60992, 0.60989, 0.61038, 0.61126, 0.61232, 0.61326, 0.61360, 0.61279,\
        0.61022, 0.60530, 0.59757, 0.58704, 0.57427, 0.55998, 0.54490, 0.52977, 0.51531, 0.50154, 0.48721],\
        [0.49229, 0.49552, 0.49810, 0.50011, 0.50152, 0.50249, 0.50310, 0.50341, 0.50359, 0.50390, 0.50478,\
        0.50666, 0.50962, 0.51352, 0.51819, 0.52353, 0.52942, 0.53579, 0.54254, 0.54962, 0.55694, 0.56445,\
        0.57212, 0.57998, 0.58813, 0.59656, 0.60473, 0.61227, 0.61960, 0.62701, 0.63427, 0.64066, 0.64557,\
        0.64874, 0.64876, 0.64206, 0.62649, 0.60262, 0.57225, 0.54443, 0.52611, 0.52013, 0.52044, 0.52285,\
        0.52475, 0.52576, 0.52674, 0.53091, 0.54051, 0.55488, 0.58051, 0.64372, 0.72994, 0.81586, 0.89689,\
        0.94768, 0.97009, 0.97402, 0.96949, 0.96029, 0.94839, 0.93505, 0.92145, 0.90870, 0.89746, 0.88742,\
        0.87866, 0.87055, 0.86280, 0.85530, 0.84798, 0.84084, 0.83384, 0.82694, 0.82007, 0.81308, 0.80575,\
        0.79779, 0.78923, 0.78019, 0.77095, 0.76173, 0.75267, 0.74391, 0.73558, 0.72781, 0.72074, 0.71445,\
        0.70892, 0.70407, 0.69832, 0.68925, 0.67891, 0.66791, 0.65666, 0.64557, 0.63505, 0.62555, 0.61759,\
        0.61173, 0.60830, 0.60687, 0.60660, 0.60689, 0.60757, 0.60861, 0.60994, 0.61127, 0.61203, 0.61159,\
        0.60925, 0.60434, 0.59629, 0.58519, 0.57178, 0.55685, 0.54115, 0.52538, 0.51020, 0.49687, 0.48315],\
        [0.49202, 0.49541, 0.49810, 0.50025, 0.50207, 0.50346, 0.50447, 0.50524, 0.50594, 0.50683, 0.50823,\
        0.51040, 0.51345, 0.51732, 0.52192, 0.52714, 0.53290, 0.53911, 0.54572, 0.55265, 0.55983, 0.56722,\
        0.57477, 0.58245, 0.59041, 0.59911, 0.60819, 0.61741, 0.62652, 0.63544, 0.64393, 0.65150, 0.65756,\
        0.66138, 0.66162, 0.65641, 0.64471, 0.62680, 0.60520, 0.57959, 0.55972, 0.54853, 0.54304, 0.53940,\
        0.53572, 0.53291, 0.53424, 0.54424, 0.56489, 0.59307, 0.62796, 0.69401, 0.78705, 0.86526, 0.92885,\
        0.96763, 0.98523, 0.98787, 0.98157, 0.97036, 0.95656, 0.94166, 0.92670, 0.91242, 0.89954, 0.88929,\
        0.88022, 0.87170, 0.86353, 0.85565, 0.84801, 0.84058, 0.83333, 0.82623, 0.81921, 0.81206, 0.80452,\
        0.79633, 0.78740, 0.77810, 0.76864, 0.75920, 0.74992, 0.74091, 0.73231, 0.72427, 0.71698, 0.71059,\
        0.70498, 0.69999, 0.69449, 0.68623, 0.67612, 0.66506, 0.65360, 0.64219, 0.63128, 0.62133, 0.61289,\
        0.60672, 0.60371, 0.60316, 0.60345, 0.60393, 0.60462, 0.60567, 0.60722, 0.60896, 0.61016, 0.61007,\
        0.60799, 0.60312, 0.59467, 0.58290, 0.56875, 0.55310, 0.53681, 0.52065, 0.50518, 0.49104, 0.47790],\
        [0.49191, 0.49559, 0.49856, 0.50098, 0.50312, 0.50489, 0.50633, 0.50757, 0.50878, 0.51018, 0.51198,\
        0.51440, 0.51752, 0.52133, 0.52580, 0.53085, 0.53641, 0.54242, 0.54881, 0.55554, 0.56255, 0.56980,\
        0.57722, 0.58471, 0.59222, 0.60040, 0.61068, 0.62128, 0.63165, 0.64163, 0.65096, 0.65922, 0.66584,\
        0.67007, 0.67088, 0.66722, 0.65850, 0.64495, 0.62710, 0.60616, 0.58824, 0.57467, 0.56556, 0.55721,\
        0.54878, 0.54343, 0.54662, 0.56481, 0.60088, 0.64058, 0.68940, 0.76635, 0.84776, 0.90606, 0.94918,\
        0.97819, 0.99152, 0.99274, 0.98581, 0.97400, 0.95965, 0.94434, 0.92912, 0.91468, 0.90164, 0.89084,\
        0.88111, 0.87200, 0.86336, 0.85507, 0.84707, 0.83931, 0.83175, 0.82439, 0.81718, 0.80992, 0.80217,\
        0.79369, 0.78462, 0.77521, 0.76568, 0.75617, 0.74681, 0.73768, 0.72890, 0.72064, 0.71312, 0.70662,\
        0.70159, 0.69673, 0.69059, 0.68269, 0.67300, 0.66211, 0.65061, 0.63904, 0.62790, 0.61769, 0.60893,\
        0.60228, 0.59965, 0.59991, 0.60062, 0.60110, 0.60179, 0.60259, 0.60437, 0.60658, 0.60813, 0.60829,\
        0.60634, 0.60150, 0.59254, 0.58007, 0.56516, 0.54880, 0.53194, 0.51546, 0.50050, 0.48633, 0.47269],\
        [0.49191, 0.49602, 0.49948, 0.50224, 0.50462, 0.50673, 0.50858, 0.51027, 0.51194, 0.51376, 0.51591,\
        0.51852, 0.52169, 0.52545, 0.52977, 0.53461, 0.53993, 0.54569, 0.55183, 0.55832, 0.56514, 0.57227,\
        0.57970, 0.58740, 0.59528, 0.60355, 0.61373, 0.62465, 0.63556, 0.64605, 0.65580, 0.66439, 0.67128,\
        0.67584, 0.67734, 0.67512, 0.66882, 0.65848, 0.64454, 0.62849, 0.61280, 0.59881, 0.58975, 0.57843,\
        0.56710, 0.56208, 0.57178, 0.60669, 0.66247, 0.70373, 0.75820, 0.82424, 0.88460, 0.92963, 0.96003,\
        0.98024, 0.98981, 0.98994, 0.98300, 0.97150, 0.95755, 0.94281, 0.92837, 0.91486, 0.90265, 0.89134,\
        0.88088, 0.87119, 0.86209, 0.85343, 0.84510, 0.83700, 0.82909, 0.82137, 0.81388, 0.80653, 0.79884,\
        0.79009, 0.78093, 0.77157, 0.76215, 0.75278, 0.74353, 0.73449, 0.72572, 0.71737, 0.70972, 0.70323,\
        0.69808, 0.69354, 0.68750, 0.67972, 0.67026, 0.65950, 0.64800, 0.63636, 0.62515, 0.61494, 0.60637,\
        0.60026, 0.59726, 0.59703, 0.59771, 0.59846, 0.59933, 0.59984, 0.60205, 0.60448, 0.60611, 0.60638,\
        0.60430, 0.59901, 0.58977, 0.57669, 0.56101, 0.54395, 0.52660, 0.50986, 0.49434, 0.48054, 0.46726],\
        [0.49189, 0.49639, 0.50029, 0.50351, 0.50631, 0.50882, 0.51109, 0.51321, 0.51529, 0.51748, 0.51990,\
        0.52268, 0.52589, 0.52957, 0.53373, 0.53836, 0.54342, 0.54890, 0.55477, 0.56101, 0.56763, 0.57462,\
        0.58206, 0.59004, 0.59862, 0.60735, 0.61703, 0.62766, 0.63854, 0.64911, 0.65895, 0.66761, 0.67462,\
        0.67948, 0.68168, 0.68083, 0.67675, 0.66954, 0.65963, 0.64797, 0.63572, 0.62432, 0.61419, 0.60182,\
        0.58932, 0.58911, 0.61103, 0.66730, 0.72659, 0.76502, 0.80945, 0.85591, 0.89756, 0.93225, 0.95753,\
        0.97374, 0.98127, 0.98097, 0.97445, 0.96372, 0.95071, 0.93709, 0.92403, 0.91205, 0.90054, 0.88947,\
        0.87886, 0.86887, 0.85951, 0.85062, 0.84207, 0.83372, 0.82546, 0.81728, 0.80929, 0.80167, 0.79380,\
        0.78530, 0.77637, 0.76728, 0.75818, 0.74916, 0.74028, 0.73159, 0.72312, 0.71496, 0.70732, 0.70093,\
        0.69632, 0.69159, 0.68561, 0.67786, 0.66840, 0.65760, 0.64601, 0.63427, 0.62304, 0.61293, 0.60457,\
        0.59869, 0.59557, 0.59468, 0.59507, 0.59604, 0.59727, 0.59882, 0.60094, 0.60268, 0.60401, 0.60452,\
        0.60283, 0.59716, 0.58701, 0.57287, 0.55624, 0.53850, 0.52074, 0.50406, 0.48888, 0.47491, 0.46183],\
        [0.49182, 0.49673, 0.50090, 0.50472, 0.50805, 0.51102, 0.51373, 0.51627, 0.51874, 0.52125, 0.52390,\
        0.52681, 0.53004, 0.53364, 0.53764, 0.54205, 0.54685, 0.55205, 0.55763, 0.56360, 0.56997, 0.57678,\
        0.58407, 0.59198, 0.60053, 0.60973, 0.61957, 0.63003, 0.64071, 0.65112, 0.66083, 0.66941, 0.67645,\
        0.68158, 0.68447, 0.68493, 0.68297, 0.67883, 0.67292, 0.66577, 0.65777, 0.64902, 0.63883, 0.62599,\
        0.61245, 0.61729, 0.65018, 0.71190, 0.76504, 0.79824, 0.83527, 0.86878, 0.89767, 0.92468, 0.94662,\
        0.96112, 0.96791, 0.96769, 0.96182, 0.95198, 0.94000, 0.92762, 0.91608, 0.90569, 0.89619, 0.88524,\
        0.87471, 0.86479, 0.85546, 0.84659, 0.83803, 0.82957, 0.82107, 0.81247, 0.80388, 0.79583, 0.78796,\
        0.77969, 0.77112, 0.76246, 0.75386, 0.74541, 0.73716, 0.72914, 0.72135, 0.71385, 0.70683, 0.70097,\
        0.69636, 0.69152, 0.68539, 0.67744, 0.66773, 0.65663, 0.64474, 0.63275, 0.62141, 0.61140, 0.60326,\
        0.59738, 0.59398, 0.59264, 0.59274, 0.59376, 0.59536, 0.59742, 0.59933, 0.60039, 0.60133, 0.60230,\
        0.60207, 0.59613, 0.58402, 0.56827, 0.55065, 0.53242, 0.51444, 0.49767, 0.48298, 0.46940, 0.45657],\
        [0.49162, 0.49714, 0.50192, 0.50602, 0.50979, 0.51324, 0.51642, 0.51937, 0.52220, 0.52500, 0.52785,\
        0.53086, 0.53409, 0.53761, 0.54145, 0.54564, 0.55018, 0.55510, 0.56040, 0.56609, 0.57220, 0.57876,\
        0.58581, 0.59347, 0.60197, 0.61134, 0.62133, 0.63170, 0.64216, 0.65233, 0.66181, 0.67023, 0.67726,\
        0.68263, 0.68615, 0.68780, 0.68779, 0.68648, 0.68437, 0.68180, 0.67853, 0.67360, 0.66646, 0.65698,\
        0.64747, 0.65085, 0.68953, 0.75632, 0.80582, 0.82933, 0.85312, 0.87515, 0.89453, 0.91457, 0.93312,\
        0.94615, 0.95237, 0.95222, 0.94693, 0.93792, 0.92680, 0.91539, 0.90561, 0.89621, 0.88742, 0.87800,\
        0.86830, 0.85889, 0.84991, 0.84134, 0.83302, 0.82469, 0.81615, 0.80739, 0.79856, 0.79007, 0.78189,\
        0.77368, 0.76543, 0.75726, 0.74929, 0.74159, 0.73421, 0.72713, 0.72037, 0.71397, 0.70808, 0.70291,\
        0.69827, 0.69330, 0.68692, 0.67853, 0.66828, 0.65662, 0.64419, 0.63172, 0.62002, 0.60990, 0.60193,\
        0.59604, 0.59229, 0.59061, 0.59046, 0.59137, 0.59292, 0.59475, 0.59632, 0.59734, 0.59804, 0.59896,\
        0.59941, 0.59291, 0.57895, 0.56201, 0.54382, 0.52568, 0.50824, 0.49200, 0.47749, 0.46416, 0.45160],\
        [0.49112, 0.49699, 0.50228, 0.50709, 0.51146, 0.51544, 0.51908, 0.52245, 0.52562, 0.52867, 0.53170,\
        0.53479, 0.53801, 0.54144, 0.54512, 0.54909, 0.55339, 0.55803, 0.56305, 0.56847, 0.57433, 0.58067,\
        0.58751, 0.59490, 0.60322, 0.61267, 0.62263, 0.63287, 0.64309, 0.65298, 0.66221, 0.67045, 0.67745,\
        0.68301, 0.68706, 0.68973, 0.69134, 0.69245, 0.69365, 0.69532, 0.69699, 0.69680, 0.69511, 0.69354,\
        0.69205, 0.69674, 0.74301, 0.80879, 0.84108, 0.85556, 0.86728, 0.87936, 0.89158, 0.90572, 0.92087,\
        0.93196, 0.93702, 0.93642, 0.93155, 0.92338, 0.91298, 0.90248, 0.89371, 0.88574, 0.87745, 0.86890,\
        0.86012, 0.85140, 0.84298, 0.83492, 0.82711, 0.81920, 0.81080, 0.80197, 0.79299, 0.78417, 0.77566,\
        0.76740, 0.75940, 0.75173, 0.74447, 0.73766, 0.73132, 0.72541, 0.71990, 0.71479, 0.71008, 0.70572,\
        0.70138, 0.69668, 0.68993, 0.68088, 0.66991, 0.65750, 0.64432, 0.63112, 0.61874, 0.60804, 0.59986,\
        0.59398, 0.59027, 0.58845, 0.58809, 0.58874, 0.58999, 0.59144, 0.59268, 0.59349, 0.59388, 0.59374,\
        0.59164, 0.58416, 0.57063, 0.55368, 0.53546, 0.51780, 0.50199, 0.48647, 0.47227, 0.45925, 0.44697],\
        [0.49046, 0.49675, 0.50260, 0.50807, 0.51305, 0.51757, 0.52169, 0.52545, 0.52894, 0.53223, 0.53540,\
        0.53855, 0.54176, 0.54509, 0.54861, 0.55238, 0.55643, 0.56081, 0.56556, 0.57072, 0.57635, 0.58252,\
        0.58931, 0.59675, 0.60491, 0.61401, 0.62372, 0.63372, 0.64368, 0.65330, 0.66228, 0.67036, 0.67732,\
        0.68305, 0.68756, 0.69103, 0.69391, 0.69692, 0.70079, 0.70572, 0.71088, 0.71508, 0.71899, 0.72438,\
        0.73326, 0.75377, 0.79568, 0.83559, 0.85620, 0.86563, 0.87228, 0.87967, 0.88857, 0.89951, 0.91160,\
        0.92034, 0.92339, 0.92146, 0.91676, 0.91030, 0.90008, 0.89114, 0.88258, 0.87473, 0.86689, 0.85890,\
        0.85080, 0.84274, 0.83488, 0.82738, 0.82032, 0.81318, 0.80501, 0.79606, 0.78688, 0.77778, 0.76904,\
        0.76075, 0.75300, 0.74586, 0.73937, 0.73356, 0.72837, 0.72375, 0.71960, 0.71582, 0.71232, 0.70893,\
        0.70534, 0.70080, 0.69360, 0.68402, 0.67232, 0.65909, 0.64507, 0.63103, 0.61780, 0.60623, 0.59742,\
        0.59176, 0.58817, 0.58627, 0.58567, 0.58597, 0.58679, 0.58773, 0.58848, 0.58874, 0.58828, 0.58654,\
        0.58226, 0.57376, 0.56064, 0.54421, 0.52624, 0.50860, 0.49363, 0.48012, 0.46707, 0.45465, 0.44274],\
        [0.48971, 0.49667, 0.50311, 0.50906, 0.51458, 0.51962, 0.52420, 0.52834, 0.53212, 0.53562, 0.53892,\
        0.54211, 0.54528, 0.54851, 0.55188, 0.55545, 0.55927, 0.56341, 0.56791, 0.57282, 0.57822, 0.58419,\
        0.59088, 0.59843, 0.60642, 0.61519, 0.62465, 0.63440, 0.64411, 0.65349, 0.66225, 0.67020, 0.67715,\
        0.68305, 0.68796, 0.69210, 0.69595, 0.70043, 0.70641, 0.71334, 0.72057, 0.72769, 0.73527, 0.74477,\
        0.75876, 0.78128, 0.81122, 0.83597, 0.85123, 0.85996, 0.86659, 0.87405, 0.88341, 0.89429, 0.90502,\
        0.91167, 0.91199, 0.90801, 0.90234, 0.89830, 0.88987, 0.88038, 0.87180, 0.86381, 0.85608, 0.84843,\
        0.84083, 0.83330, 0.82591, 0.81877, 0.81240, 0.80664, 0.79882, 0.78957, 0.78007, 0.77074, 0.76187,\
        0.75363, 0.74617, 0.73958, 0.73392, 0.72917, 0.72523, 0.72195, 0.71918, 0.71671, 0.71436, 0.71186,\
        0.70876, 0.70421, 0.69729, 0.68751, 0.67514, 0.66116, 0.64635, 0.63154, 0.61758, 0.60551, 0.59632,\
        0.59021, 0.58645, 0.58433, 0.58338, 0.58321, 0.58346, 0.58378, 0.58385, 0.58331, 0.58178, 0.57861,\
        0.57290, 0.56375, 0.55094, 0.53522, 0.51799, 0.50119, 0.48694, 0.47447, 0.46241, 0.45059, 0.43901],\
        [0.48875, 0.49624, 0.50328, 0.50991, 0.51600, 0.52156, 0.52657, 0.53107, 0.53512, 0.53880, 0.54220,\
        0.54541, 0.54853, 0.55166, 0.55488, 0.55827, 0.56188, 0.56580, 0.57006, 0.57475, 0.57992, 0.58563,\
        0.59194, 0.59901, 0.60716, 0.61606, 0.62542, 0.63500, 0.64450, 0.65368, 0.66229, 0.67014, 0.67712,\
        0.68320, 0.68850, 0.69328, 0.69803, 0.70371, 0.71073, 0.71859, 0.72682, 0.73527, 0.74431, 0.75485,\
        0.76814, 0.78482, 0.80368, 0.82022, 0.83326, 0.84327, 0.85211, 0.86214, 0.87431, 0.88776, 0.90003,\
        0.90474, 0.90188, 0.89664, 0.89045, 0.88497, 0.87770, 0.86915, 0.86075, 0.85278, 0.84514, 0.83773,\
        0.83053, 0.82350, 0.81658, 0.80958, 0.80289, 0.79857, 0.79118, 0.78192, 0.77233, 0.76293, 0.75406,\
        0.74597, 0.73885, 0.73287, 0.72807, 0.72443, 0.72178, 0.71988, 0.71845, 0.71722, 0.71590, 0.71415,\
        0.71149, 0.70732, 0.70092, 0.69074, 0.67791, 0.66342, 0.64809, 0.63269, 0.61808, 0.60539, 0.59582,\
        0.58950, 0.58543, 0.58288, 0.58139, 0.58059, 0.58015, 0.57974, 0.57900, 0.57758, 0.57504, 0.57081,\
        0.56428, 0.55492, 0.54264, 0.52791, 0.51184, 0.49638, 0.48292, 0.47068, 0.45890, 0.44731, 0.43587],\
        [0.48755, 0.49578, 0.50345, 0.51059, 0.51727, 0.52333, 0.52877, 0.53360, 0.53789, 0.54172, 0.54518,\
        0.54840, 0.55146, 0.55449, 0.55757, 0.56078, 0.56421, 0.56793, 0.57200, 0.57649, 0.58148, 0.58701,\
        0.59314, 0.59992, 0.60803, 0.61691, 0.62618, 0.63560, 0.64495, 0.65398, 0.66248, 0.67029, 0.67732,\
        0.68359, 0.68925, 0.69462, 0.70020, 0.70651, 0.71382, 0.72181, 0.73014, 0.73866, 0.74751, 0.75706,\
        0.76763, 0.77903, 0.78993, 0.79965, 0.81073, 0.82169, 0.83226, 0.84517, 0.86075, 0.87693, 0.89046,\
        0.89457, 0.89118, 0.88547, 0.87895, 0.87226, 0.86499, 0.85715, 0.84927, 0.84160, 0.83415, 0.82696,\
        0.82016, 0.81363, 0.80722, 0.80111, 0.79414, 0.78845, 0.78152, 0.77290, 0.76363, 0.75440, 0.74567,\
        0.73781, 0.73108, 0.72572, 0.72182, 0.71933, 0.71801, 0.71749, 0.71737, 0.71728, 0.71685, 0.71571,\
        0.71340, 0.70935, 0.70278, 0.69298, 0.68022, 0.66566, 0.65017, 0.63453, 0.61956, 0.60617, 0.59628,\
        0.58991, 0.58528, 0.58202, 0.57978, 0.57821, 0.57698, 0.57574, 0.57417, 0.57189, 0.56851, 0.56357,\
        0.55665, 0.54748, 0.53606, 0.52270, 0.50810, 0.49388, 0.48090, 0.46852, 0.45653, 0.44482, 0.43330],\
        [0.48598, 0.49475, 0.50304, 0.51106, 0.51834, 0.52489, 0.53072, 0.53586, 0.54036, 0.54430, 0.54781,\
        0.55100, 0.55400, 0.55692, 0.55986, 0.56293, 0.56621, 0.56977, 0.57368, 0.57802, 0.58288, 0.58835,\
        0.59458, 0.60160, 0.60930, 0.61789, 0.62698, 0.63627, 0.64550, 0.65443, 0.66288, 0.67069, 0.67780,\
        0.68423, 0.69013, 0.69585, 0.70185, 0.70846, 0.71568, 0.72332, 0.73112, 0.73891, 0.74666, 0.75441,\
        0.76214, 0.76954, 0.77623, 0.78280, 0.79104, 0.80084, 0.81059, 0.82591, 0.84427, 0.86160, 0.87442,\
        0.87967, 0.87817, 0.87310, 0.86651, 0.85939, 0.85209, 0.84476, 0.83757, 0.83048, 0.82332, 0.81631,\
        0.80989, 0.80401, 0.79782, 0.79189, 0.78545, 0.77883, 0.77156, 0.76326, 0.75435, 0.74540, 0.73691,\
        0.72932, 0.72300, 0.71825, 0.71525, 0.71395, 0.71400, 0.71486, 0.71597, 0.71690, 0.71724, 0.71659,\
        0.71450, 0.71045, 0.70391, 0.69436, 0.68200, 0.66774, 0.65245, 0.63700, 0.62224, 0.60928, 0.59934,\
        0.59157, 0.58589, 0.58169, 0.57854, 0.57608, 0.57397, 0.57189, 0.56949, 0.56644, 0.56238, 0.55699,\
        0.54997, 0.54121, 0.53081, 0.51914, 0.50669, 0.49356, 0.48017, 0.46725, 0.45485, 0.44285, 0.43113],\
        [0.48402, 0.49377, 0.50296, 0.51134, 0.51912, 0.52614, 0.53236, 0.53778, 0.54245, 0.54647, 0.54999,\
        0.55314, 0.55606, 0.55887, 0.56171, 0.56465, 0.56781, 0.57125, 0.57505, 0.57929, 0.58404, 0.58942,\
        0.59554, 0.60252, 0.61033, 0.61883, 0.62781, 0.63699, 0.64614, 0.65503, 0.66348, 0.67135, 0.67855,\
        0.68510, 0.69113, 0.69696, 0.70306, 0.70963, 0.71650, 0.72348, 0.73039, 0.73704, 0.74335, 0.74924,\
        0.75464, 0.75951, 0.76403, 0.76895, 0.77526, 0.78301, 0.79268, 0.80909, 0.82839, 0.84561, 0.85792,\
        0.86380, 0.86367, 0.85947, 0.85313, 0.84601, 0.83890, 0.83218, 0.82592, 0.81980, 0.81315, 0.80615,\
        0.80004, 0.79459, 0.78887, 0.78282, 0.77638, 0.76945, 0.76192, 0.75370, 0.74503, 0.73634, 0.72812,\
        0.72082, 0.71490, 0.71075, 0.70859, 0.70850, 0.70997, 0.71213, 0.71436, 0.71617, 0.71716, 0.71692,\
        0.71502, 0.71100, 0.70452, 0.69531, 0.68339, 0.66959, 0.65473, 0.63967, 0.62531, 0.61254, 0.60184,\
        0.59346, 0.58682, 0.58161, 0.57750, 0.57412, 0.57113, 0.56820, 0.56502, 0.56129, 0.55670, 0.55099,\
        0.54399, 0.53564, 0.52608, 0.51564, 0.50466, 0.49244, 0.47904, 0.46591, 0.45325, 0.44101, 0.42910],\
        [0.48137, 0.49193, 0.50186, 0.51106, 0.51943, 0.52696, 0.53358, 0.53926, 0.54407, 0.54813, 0.55162,\
        0.55471, 0.55754, 0.56026, 0.56300, 0.56586, 0.56894, 0.57232, 0.57606, 0.58025, 0.58495, 0.59025,\
        0.59627, 0.60324, 0.61113, 0.61964, 0.62857, 0.63770, 0.64682, 0.65571, 0.66422, 0.67219, 0.67952,\
        0.68618, 0.69227, 0.69807, 0.70403, 0.71025, 0.71654, 0.72268, 0.72852, 0.73390, 0.73873, 0.74296,\
        0.74662, 0.74983, 0.75302, 0.75688, 0.76227, 0.76989, 0.78104, 0.79699, 0.81538, 0.83124, 0.84282,\
        0.84879, 0.84919, 0.84541, 0.83925, 0.83227, 0.82554, 0.81954, 0.81445, 0.80983, 0.80441, 0.79741,\
        0.79146, 0.78595, 0.78028, 0.77418, 0.76760, 0.76048, 0.75279, 0.74458, 0.73607, 0.72763, 0.71970,\
        0.71273, 0.70726, 0.70374, 0.70230, 0.70340, 0.70623, 0.70950, 0.71260, 0.71513, 0.71668, 0.71681,\
        0.71512, 0.71122, 0.70480, 0.69582, 0.68444, 0.67112, 0.65671, 0.64205, 0.62798, 0.61523, 0.60436,\
        0.59524, 0.58761, 0.58149, 0.57649, 0.57222, 0.56837, 0.56464, 0.56075, 0.55641, 0.55138, 0.54544,\
        0.53844, 0.53036, 0.52128, 0.51136, 0.50060, 0.48904, 0.47658, 0.46384, 0.45127, 0.43899, 0.42699],\
        [0.47783, 0.48921, 0.50012, 0.50996, 0.51908, 0.52726, 0.53430, 0.54021, 0.54510, 0.54916, 0.55258,\
        0.55558, 0.55833, 0.56097, 0.56365, 0.56648, 0.56953, 0.57290, 0.57665, 0.58086, 0.58558, 0.59090,\
        0.59693, 0.60391, 0.61182, 0.62033, 0.62922, 0.63832, 0.64744, 0.65639, 0.66499, 0.67311, 0.68062,\
        0.68744, 0.69356, 0.69902, 0.70477, 0.71053, 0.71606, 0.72124, 0.72593, 0.73004, 0.73351, 0.73634,\
        0.73864, 0.74066, 0.74287, 0.74599, 0.75093, 0.75873, 0.77048, 0.78640, 0.80323, 0.81813, 0.82945,\
        0.83535, 0.83565, 0.83174, 0.82553, 0.81866, 0.81250, 0.80726, 0.80322, 0.80017, 0.79591, 0.79002,\
        0.78398, 0.77823, 0.77237, 0.76613, 0.75939, 0.75211, 0.74431, 0.73612, 0.72775, 0.71955, 0.71206,\
        0.70558, 0.70071, 0.69795, 0.69743, 0.69938, 0.70307, 0.70695, 0.71063, 0.71372, 0.71574, 0.71623,\
        0.71481, 0.71114, 0.70504, 0.69636, 0.68518, 0.67218, 0.65813, 0.64379, 0.62988, 0.61701, 0.60562,\
        0.59591, 0.58785, 0.58110, 0.57533, 0.57025, 0.56560, 0.56113, 0.55659, 0.55173, 0.54632, 0.54018,\
        0.53315, 0.52519, 0.51634, 0.50669, 0.49628, 0.48510, 0.47326, 0.46104, 0.44876, 0.43660, 0.42464],\
        [0.47328, 0.48539, 0.49695, 0.50791, 0.51808, 0.52699, 0.53444, 0.54052, 0.54541, 0.54939, 0.55272,\
        0.55562, 0.55829, 0.56089, 0.56356, 0.56639, 0.56949, 0.57292, 0.57675, 0.58106, 0.58590, 0.59136,\
        0.59752, 0.60454, 0.61238, 0.62082, 0.62967, 0.63875, 0.64788, 0.65690, 0.66564, 0.67394, 0.68166,\
        0.68869, 0.69500, 0.70067, 0.70560, 0.71055, 0.71517, 0.71932, 0.72290, 0.72582, 0.72806, 0.72969,\
        0.73088, 0.73194, 0.73330, 0.73566, 0.73992, 0.74720, 0.75842, 0.77335, 0.78986, 0.80551, 0.81759,\
        0.82365, 0.82340, 0.81895, 0.81273, 0.80594, 0.80064, 0.79652, 0.79320, 0.79057, 0.78704, 0.78224,\
        0.77678, 0.77109, 0.76512, 0.75873, 0.75183, 0.74441, 0.73656, 0.72842, 0.72020, 0.71223, 0.70513,\
        0.70018, 0.69564, 0.69348, 0.69362, 0.69604, 0.69994, 0.70404, 0.70810, 0.71171, 0.71416, 0.71503,\
        0.71394, 0.71054, 0.70466, 0.69624, 0.68527, 0.67253, 0.65876, 0.64466, 0.63088, 0.61792, 0.60610,\
        0.59601, 0.58767, 0.58039, 0.57393, 0.56810, 0.56271, 0.55756, 0.55243, 0.54712, 0.54141, 0.53511,\
        0.52804, 0.52010, 0.51133, 0.50185, 0.49170, 0.48087, 0.46947, 0.45769, 0.44574, 0.43380, 0.42196],\
        [0.46777, 0.48076, 0.49327, 0.50536, 0.51663, 0.52618, 0.53390, 0.54000, 0.54480, 0.54864, 0.55184,\
        0.55465, 0.55729, 0.55990, 0.56261, 0.56552, 0.56872, 0.57228, 0.57626, 0.58075, 0.58578, 0.59144,\
        0.59781, 0.60487, 0.61263, 0.62099, 0.62978, 0.63884, 0.64799, 0.65707, 0.66594, 0.67443, 0.68239,\
        0.68963, 0.69591, 0.70121, 0.70601, 0.71015, 0.71380, 0.71695, 0.71950, 0.72136, 0.72251, 0.72309,\
        0.72334, 0.72357, 0.72410, 0.72548, 0.72859, 0.73451, 0.74445, 0.75869, 0.77572, 0.79288, 0.80680,\
        0.81343, 0.81225, 0.80707, 0.80143, 0.79581, 0.79106, 0.78763, 0.78476, 0.78209, 0.77887, 0.77473,\
        0.76982, 0.76437, 0.75842, 0.75195, 0.74491, 0.73738, 0.72948, 0.72143, 0.71346, 0.70581, 0.69897,\
        0.69415, 0.69078, 0.68905, 0.68947, 0.69190, 0.69572, 0.69999, 0.70465, 0.70885, 0.71160, 0.71285,\
        0.71227, 0.70926, 0.70334, 0.69504, 0.68445, 0.67193, 0.65836, 0.64448, 0.63092, 0.61815, 0.60644,\
        0.59616, 0.58732, 0.57936, 0.57217, 0.56563, 0.55956, 0.55380, 0.54815, 0.54245, 0.53651, 0.53012,\
        0.52307, 0.51514, 0.50636, 0.49697, 0.48701, 0.47644, 0.46536, 0.45391, 0.44227, 0.43058, 0.41892],\
        [0.46119, 0.47536, 0.48939, 0.50278, 0.51481, 0.52467, 0.53243, 0.53839, 0.54298, 0.54664, 0.54973,\
        0.55251, 0.55517, 0.55787, 0.56070, 0.56377, 0.56714, 0.57090, 0.57510, 0.57981, 0.58508, 0.59094,\
        0.59740, 0.60456, 0.61238, 0.62069, 0.62943, 0.63845, 0.64760, 0.65673, 0.66568, 0.67431, 0.68248,\
        0.69000, 0.69660, 0.70186, 0.70577, 0.70897, 0.71173, 0.71402, 0.71570, 0.71666, 0.71686, 0.71648,\
        0.71590, 0.71545, 0.71519, 0.71544, 0.71702, 0.72084, 0.72880, 0.74249, 0.76027, 0.77899, 0.79514,\
        0.80307, 0.80127, 0.79656, 0.79181, 0.78743, 0.78345, 0.78022, 0.77744, 0.77472, 0.77159, 0.76779,\
        0.76325, 0.75805, 0.75221, 0.74572, 0.73860, 0.73094, 0.72294, 0.71495, 0.70725, 0.70000, 0.69353,\
        0.68860, 0.68537, 0.68388, 0.68431, 0.68657, 0.69025, 0.69487, 0.70025, 0.70465, 0.70742, 0.70910,\
        0.70937, 0.70727, 0.70210, 0.69365, 0.68271, 0.67015, 0.65670, 0.64303, 0.62975, 0.61738, 0.60631,\
        0.59616, 0.58647, 0.57775, 0.56988, 0.56270, 0.55603, 0.54972, 0.54361, 0.53757, 0.53146, 0.52509,\
        0.51820, 0.51038, 0.50153, 0.49218, 0.48227, 0.47182, 0.46095, 0.44977, 0.43842, 0.42698, 0.41553],\
        [0.45318, 0.46856, 0.48429, 0.49945, 0.51178, 0.52188, 0.52956, 0.53526, 0.53961, 0.54312, 0.54617,\
        0.54902, 0.55182, 0.55470, 0.55776, 0.56107, 0.56470, 0.56872, 0.57319, 0.57817, 0.58370, 0.58983,\
        0.59655, 0.60382, 0.61154, 0.61979, 0.62848, 0.63747, 0.64660, 0.65570, 0.66465, 0.67330, 0.68153,\
        0.68917, 0.69595, 0.70119, 0.70418, 0.70653, 0.70867, 0.71034, 0.71135, 0.71167, 0.71107, 0.70973,\
        0.70836, 0.70748, 0.70657, 0.70565, 0.70583, 0.70720, 0.71219, 0.72561, 0.74391, 0.76307, 0.77931,\
        0.78839, 0.78943, 0.78691, 0.78345, 0.77994, 0.77663, 0.77365, 0.77090, 0.76813, 0.76503, 0.76140,\
        0.75711, 0.75213, 0.74643, 0.73999, 0.73282, 0.72501, 0.71680, 0.70868, 0.70111, 0.69401, 0.68771,\
        0.68272, 0.67934, 0.67770, 0.67787, 0.67979, 0.68325, 0.68796, 0.69340, 0.69757, 0.70105, 0.70292,\
        0.70447, 0.70378, 0.69889, 0.69038, 0.67933, 0.66681, 0.65353, 0.64010, 0.62705, 0.61491, 0.60400,\
        0.59399, 0.58423, 0.57516, 0.56684, 0.55916, 0.55200, 0.54522, 0.53870, 0.53234, 0.52605, 0.51976,\
        0.51327, 0.50589, 0.49705, 0.48746, 0.47738, 0.46692, 0.45619, 0.44526, 0.43419, 0.42302, 0.41182],\
        [0.44328, 0.45949, 0.47633, 0.49232, 0.50631, 0.51716, 0.52464, 0.53010, 0.53428, 0.53777, 0.54096,\
        0.54403, 0.54713, 0.55034, 0.55374, 0.55739, 0.56135, 0.56568, 0.57047, 0.57575, 0.58157, 0.58796,\
        0.59494, 0.60225, 0.60991, 0.61817, 0.62687, 0.63584, 0.64489, 0.65389, 0.66269, 0.67117, 0.67920,\
        0.68660, 0.69307, 0.69784, 0.70038, 0.70250, 0.70442, 0.70580, 0.70618, 0.70627, 0.70503, 0.70275,\
        0.70051, 0.69939, 0.69791, 0.69616, 0.69526, 0.69511, 0.69807, 0.71085, 0.72865, 0.74705, 0.76259,\
        0.77260, 0.77669, 0.77686, 0.77517, 0.77275, 0.77009, 0.76742, 0.76475, 0.76196, 0.75889, 0.75536,\
        0.75125, 0.74648, 0.74097, 0.73464, 0.72748, 0.71957, 0.71108, 0.70249, 0.69480, 0.68770, 0.68135,\
        0.67617, 0.67246, 0.67040, 0.67005, 0.67140, 0.67428, 0.67840, 0.68313, 0.68756, 0.69130, 0.69417,\
        0.69639, 0.69651, 0.69261, 0.68432, 0.67369, 0.66163, 0.64877, 0.63569, 0.62288, 0.61081, 0.59987,\
        0.58999, 0.58056, 0.57152, 0.56298, 0.55496, 0.54740, 0.54024, 0.53335, 0.52666, 0.52014, 0.51375,\
        0.50761, 0.50114, 0.49220, 0.48227, 0.47200, 0.46154, 0.45097, 0.44033, 0.42961, 0.41875, 0.40782],\
        [0.43140, 0.44789, 0.46538, 0.48283, 0.49888, 0.50924, 0.51681, 0.52238, 0.52661, 0.53037, 0.53397,\
        0.53750, 0.54108, 0.54478, 0.54864, 0.55272, 0.55708, 0.56180, 0.56694, 0.57255, 0.57866, 0.58523,\
        0.59221, 0.59955, 0.60745, 0.61586, 0.62461, 0.63354, 0.64247, 0.65126, 0.65978, 0.66789, 0.67547,\
        0.68230, 0.68805, 0.69228, 0.69503, 0.69710, 0.69895, 0.70044, 0.69976, 0.70014, 0.69833, 0.69549,\
        0.69283, 0.69084, 0.68893, 0.68704, 0.68582, 0.68604, 0.69003, 0.70068, 0.71648, 0.73325, 0.74788,\
        0.75841, 0.76440, 0.76675, 0.76678, 0.76550, 0.76353, 0.76120, 0.75866, 0.75589, 0.75283, 0.74939,\
        0.74544, 0.74089, 0.73561, 0.72949, 0.72243, 0.71449, 0.70597, 0.69704, 0.68875, 0.68118, 0.67445,\
        0.66885, 0.66464, 0.66197, 0.66091, 0.66144, 0.66343, 0.66666, 0.67070, 0.67498, 0.67905, 0.68257,\
        0.68501, 0.68527, 0.68210, 0.67530, 0.66585, 0.65469, 0.64256, 0.63006, 0.61771, 0.60580, 0.59511,\
        0.58543, 0.57612, 0.56708, 0.55839, 0.55010, 0.54223, 0.53476, 0.52757, 0.52054, 0.51374, 0.50720,\
        0.50046, 0.49413, 0.48570, 0.47598, 0.46580, 0.45547, 0.44516, 0.43492, 0.42470, 0.41427, 0.40361],\
        [0.41793, 0.43417, 0.45158, 0.46904, 0.48472, 0.49655, 0.50570, 0.51182, 0.51639, 0.52090, 0.52527,\
        0.52947, 0.53370, 0.53806, 0.54253, 0.54713, 0.55194, 0.55708, 0.56263, 0.56863, 0.57506, 0.58187,\
        0.58901, 0.59650, 0.60452, 0.61304, 0.62182, 0.63066, 0.63940, 0.64787, 0.65597, 0.66356, 0.67052,\
        0.67666, 0.68175, 0.68561, 0.68836, 0.69039, 0.69200, 0.69286, 0.69254, 0.69193, 0.69012, 0.68740,\
        0.68466, 0.68229, 0.68026, 0.67869, 0.67812, 0.67953, 0.68445, 0.69408, 0.70761, 0.72243, 0.73595,\
        0.74648, 0.75343, 0.75717, 0.75848, 0.75815, 0.75679, 0.75478, 0.75235, 0.74961, 0.74656, 0.74318,\
        0.73938, 0.73505, 0.73005, 0.72423, 0.71739, 0.70934, 0.70068, 0.69165, 0.68260, 0.67429, 0.66694,\
        0.66076, 0.65593, 0.65256, 0.65067, 0.65024, 0.65119, 0.65341, 0.65671, 0.66071, 0.66497, 0.66887,\
        0.67163, 0.67228, 0.66998, 0.66455, 0.65647, 0.64646, 0.63527, 0.62354, 0.61192, 0.60125, 0.59054,\
        0.58067, 0.57124, 0.56209, 0.55320, 0.54464, 0.53648, 0.52877, 0.52143, 0.51413, 0.50681, 0.50069,\
        0.49380, 0.48630, 0.47795, 0.46859, 0.45869, 0.44861, 0.43863, 0.42891, 0.41944, 0.40982, 0.39944],\
        [0.40396, 0.41957, 0.43579, 0.45234, 0.46781, 0.48091, 0.49128, 0.49838, 0.50380, 0.50976, 0.51511,\
        0.52007, 0.52511, 0.53032, 0.53552, 0.54069, 0.54597, 0.55157, 0.55760, 0.56407, 0.57093, 0.57810,\
        0.58553, 0.59323, 0.60134, 0.60989, 0.61862, 0.62732, 0.63577, 0.64383, 0.65138, 0.65833, 0.66459,\
        0.67003, 0.67451, 0.67796, 0.68047, 0.68225, 0.68340, 0.68383, 0.68347, 0.68246, 0.68069, 0.67834,\
        0.67588, 0.67368, 0.67196, 0.67104, 0.67146, 0.67405, 0.67972, 0.68890, 0.70093, 0.71416, 0.72658,\
        0.73673, 0.74399, 0.74840, 0.75045, 0.75075, 0.74981, 0.74801, 0.74563, 0.74286, 0.73979, 0.73643,\
        0.73272, 0.72857, 0.72384, 0.71836, 0.71204, 0.70423, 0.69482, 0.68522, 0.67561, 0.66664, 0.65867,\
        0.65191, 0.64646, 0.64241, 0.63970, 0.63829, 0.63813, 0.63928, 0.64179, 0.64550, 0.64992, 0.65426,\
        0.65754, 0.65883, 0.65746, 0.65325, 0.64643, 0.63756, 0.62735, 0.61655, 0.60556, 0.59518, 0.58538,\
        0.57551, 0.56595, 0.55663, 0.54752, 0.53864, 0.53011, 0.52217, 0.51492, 0.50772, 0.50002, 0.49288,\
        0.48591, 0.47808, 0.46961, 0.46048, 0.45088, 0.44106, 0.43136, 0.42211, 0.41345, 0.40502, 0.39566],\
        [0.39207, 0.40525, 0.41939, 0.43493, 0.45021, 0.46376, 0.47478, 0.48325, 0.49043, 0.49749, 0.50372,\
        0.50946, 0.51550, 0.52183, 0.52781, 0.53350, 0.53923, 0.54532, 0.55193, 0.55899, 0.56640, 0.57404,\
        0.58184, 0.58981, 0.59802, 0.60653, 0.61516, 0.62365, 0.63172, 0.63925, 0.64614, 0.65238, 0.65789,\
        0.66261, 0.66647, 0.66943, 0.67154, 0.67291, 0.67362, 0.67369, 0.67315, 0.67206, 0.67048, 0.66856,\
        0.66658, 0.66486, 0.66373, 0.66360, 0.66502, 0.66862, 0.67495, 0.68409, 0.69541, 0.70763, 0.71913,\
        0.72877, 0.73587, 0.74040, 0.74272, 0.74330, 0.74255, 0.74081, 0.73835, 0.73544, 0.73227, 0.72886,\
        0.72514, 0.72106, 0.71649, 0.71113, 0.70496, 0.69868, 0.68831, 0.67764, 0.66742, 0.65797, 0.64955,\
        0.64233, 0.63641, 0.63182, 0.62847, 0.62620, 0.62495, 0.62499, 0.62671, 0.63015, 0.63488, 0.63984,\
        0.64382, 0.64586, 0.64539, 0.64222, 0.63649, 0.62860, 0.61919, 0.60914, 0.59990, 0.58956, 0.57956,\
        0.56978, 0.56018, 0.55074, 0.54143, 0.53222, 0.52318, 0.51473, 0.50755, 0.50117, 0.49304, 0.48504,\
        0.47728, 0.46927, 0.46084, 0.45196, 0.44267, 0.43309, 0.42351, 0.41447, 0.40633, 0.39854, 0.38980],\
        [0.38030, 0.39151, 0.40451, 0.41844, 0.43355, 0.44701, 0.45833, 0.46783, 0.47623, 0.48405, 0.49124,\
        0.49796, 0.50531, 0.51295, 0.51955, 0.52563, 0.53173, 0.53838, 0.54570, 0.55351, 0.56159, 0.56978,\
        0.57801, 0.58628, 0.59459, 0.60302, 0.61155, 0.61976, 0.62736, 0.63423, 0.64036, 0.64580, 0.65053,\
        0.65453, 0.65775, 0.66016, 0.66177, 0.66267, 0.66294, 0.66267, 0.66197, 0.66093, 0.65965, 0.65822,\
        0.65683, 0.65572, 0.65525, 0.65587, 0.65813, 0.66256, 0.66951, 0.67891, 0.69012, 0.70190, 0.71283,\
        0.72206, 0.72872, 0.73290, 0.73509, 0.73568, 0.73495, 0.73310, 0.73039, 0.72719, 0.72381, 0.72027,\
        0.71638, 0.71210, 0.70756, 0.70211, 0.69534, 0.68792, 0.67850, 0.66799, 0.65772, 0.64817, 0.63959,\
        0.63217, 0.62600, 0.62113, 0.61748, 0.61469, 0.61252, 0.61143, 0.61243, 0.61565, 0.62091, 0.62670,\
        0.63139, 0.63411, 0.63439, 0.63207, 0.62721, 0.62011, 0.61129, 0.60165, 0.59225, 0.58290, 0.57315,\
        0.56345, 0.55388, 0.54441, 0.53500, 0.52558, 0.51604, 0.50659, 0.49885, 0.49209, 0.48439, 0.47618,\
        0.46806, 0.45996, 0.45172, 0.44323, 0.43441, 0.42521, 0.41569, 0.40647, 0.39873, 0.39130, 0.38314],\
        [0.37000, 0.38060, 0.39182, 0.40535, 0.41940, 0.43170, 0.44264, 0.45253, 0.46164, 0.47018, 0.47833,\
        0.48644, 0.49527, 0.50353, 0.51064, 0.51707, 0.52342, 0.53080, 0.53910, 0.54782, 0.55662, 0.56539,\
        0.57407, 0.58266, 0.59113, 0.59952, 0.60793, 0.61577, 0.62273, 0.62882, 0.63409, 0.63865, 0.64258,\
        0.64588, 0.64847, 0.65031, 0.65139, 0.65176, 0.65157, 0.65096, 0.65012, 0.64920, 0.64828, 0.64742,\
        0.64665, 0.64616, 0.64627, 0.64746, 0.65032, 0.65535, 0.66286, 0.67271, 0.68421, 0.69600, 0.70729,\
        0.71612, 0.72197, 0.72543, 0.72721, 0.72771, 0.72693, 0.72486, 0.72170, 0.71800, 0.71432, 0.71055,\
        0.70646, 0.70145, 0.69658, 0.69072, 0.68356, 0.67544, 0.66624, 0.65635, 0.64657, 0.63736, 0.62898,\
        0.62164, 0.61551, 0.61063, 0.60711, 0.60434, 0.60180, 0.59983, 0.60049, 0.60328, 0.60939, 0.61588,\
        0.62098, 0.62409, 0.62484, 0.62309, 0.61891, 0.61243, 0.60399, 0.59467, 0.58523, 0.57574, 0.56612,\
        0.55651, 0.54700, 0.53757, 0.52817, 0.51880, 0.50925, 0.49928, 0.49034, 0.48249, 0.47451, 0.46630,\
        0.45816, 0.45018, 0.44232, 0.43443, 0.42633, 0.41787, 0.40903, 0.39973, 0.39198, 0.38455, 0.37655],\
        [0.36130, 0.37154, 0.38293, 0.39566, 0.40712, 0.41770, 0.42787, 0.43778, 0.44727, 0.45634, 0.46523,\
        0.47429, 0.48367, 0.49267, 0.50101, 0.50776, 0.51423, 0.52289, 0.53250, 0.54216, 0.55164, 0.56091,\
        0.56999, 0.57890, 0.58764, 0.59621, 0.60441, 0.61160, 0.61778, 0.62300, 0.62731, 0.63094, 0.63408,\
        0.63673, 0.63875, 0.64004, 0.64057, 0.64040, 0.63969, 0.63871, 0.63771, 0.63693, 0.63645, 0.63620,\
        0.63608, 0.63612, 0.63661, 0.63810, 0.64124, 0.64660, 0.65451, 0.66491, 0.67721, 0.69038, 0.70224,\
        0.71002, 0.71490, 0.71732, 0.71865, 0.71915, 0.71839, 0.71611, 0.71239, 0.70775, 0.70411, 0.69951,\
        0.69519, 0.68967, 0.68390, 0.67746, 0.67002, 0.66169, 0.65268, 0.64340, 0.63440, 0.62593, 0.61805,\
        0.61107, 0.60537, 0.60066, 0.59756, 0.59514, 0.59299, 0.59167, 0.59239, 0.59561, 0.60186, 0.60793,\
        0.61281, 0.61599, 0.61690, 0.61533, 0.61152, 0.60557, 0.59729, 0.58782, 0.57806, 0.56827, 0.55852,\
        0.54890, 0.53947, 0.53011, 0.52069, 0.51130, 0.50224, 0.49197, 0.48197, 0.47284, 0.46417, 0.45576,\
        0.44767, 0.43995, 0.43260, 0.42550, 0.41840, 0.41092, 0.40322, 0.39501, 0.38651, 0.37846, 0.37026],\
        [0.35371, 0.36354, 0.37415, 0.38506, 0.39526, 0.40458, 0.41410, 0.42404, 0.43364, 0.44282, 0.45196,\
        0.46138, 0.47113, 0.48089, 0.49012, 0.49724, 0.50486, 0.51566, 0.52647, 0.53680, 0.54671, 0.55631,\
        0.56566, 0.57479, 0.58370, 0.59235, 0.60029, 0.60679, 0.61233, 0.61672, 0.61999, 0.62262, 0.62503,\
        0.62716, 0.62869, 0.62946, 0.62949, 0.62876, 0.62748, 0.62605, 0.62484, 0.62416, 0.62417, 0.62462,\
        0.62513, 0.62556, 0.62620, 0.62764, 0.63071, 0.63607, 0.64414, 0.65496, 0.66794, 0.68166, 0.69371,\
        0.70289, 0.70666, 0.70797, 0.70905, 0.70991, 0.70913, 0.70674, 0.70245, 0.69719, 0.69261, 0.68759,\
        0.68228, 0.67647, 0.67016, 0.66325, 0.65558, 0.64723, 0.63845, 0.62974, 0.62170, 0.61437, 0.60731,\
        0.60077, 0.59585, 0.59188, 0.58907, 0.58715, 0.58589, 0.58570, 0.58727, 0.59106, 0.59661, 0.60195,\
        0.60657, 0.60973, 0.61062, 0.60874, 0.60447, 0.59887, 0.59030, 0.58048, 0.57033, 0.56019, 0.55022,\
        0.54051, 0.53117, 0.52202, 0.51250, 0.50265, 0.49300, 0.48293, 0.47257, 0.46271, 0.45345, 0.44479,\
        0.43674, 0.42931, 0.42252, 0.41630, 0.41032, 0.40427, 0.39671, 0.38900, 0.38075, 0.37246, 0.36408],\
        [0.34688, 0.35610, 0.36572, 0.37532, 0.38444, 0.39302, 0.40210, 0.41195, 0.42116, 0.42992, 0.43881,\
        0.44822, 0.45825, 0.46865, 0.47881, 0.48828, 0.49874, 0.51032, 0.52131, 0.53174, 0.54176, 0.55148,\
        0.56091, 0.57007, 0.57890, 0.58724, 0.59469, 0.60094, 0.60631, 0.60998, 0.61208, 0.61361, 0.61543,\
        0.61732, 0.61843, 0.61866, 0.61829, 0.61701, 0.61506, 0.61316, 0.61166, 0.61101, 0.61151, 0.61271,\
        0.61377, 0.61445, 0.61501, 0.61604, 0.61874, 0.62381, 0.63168, 0.64270, 0.65619, 0.67039, 0.68323,\
        0.69269, 0.69641, 0.69767, 0.69872, 0.70019, 0.69855, 0.69608, 0.69106, 0.68543, 0.67978, 0.67417,\
        0.66836, 0.66225, 0.65569, 0.64859, 0.64086, 0.63262, 0.62415, 0.61590, 0.60876, 0.60310, 0.59707,\
        0.59149, 0.58711, 0.58387, 0.58166, 0.58039, 0.58005, 0.58089, 0.58324, 0.58721, 0.59224, 0.59734,\
        0.60187, 0.60511, 0.60603, 0.60367, 0.59817, 0.59123, 0.58234, 0.57217, 0.56169, 0.55131, 0.54111,\
        0.53119, 0.52188, 0.51318, 0.50406, 0.49355, 0.48322, 0.47276, 0.46221, 0.45197, 0.44234, 0.43351,\
        0.42558, 0.41844, 0.41209, 0.40670, 0.40163, 0.39634, 0.38990, 0.38226, 0.37429, 0.36611, 0.35776],\
        [0.34074, 0.34936, 0.35816, 0.36691, 0.37542, 0.38382, 0.39268, 0.40166, 0.41004, 0.41783, 0.42616,\
        0.43532, 0.44556, 0.45679, 0.46853, 0.48061, 0.49330, 0.50544, 0.51627, 0.52657, 0.53656, 0.54626,\
        0.55564, 0.56466, 0.57322, 0.58118, 0.58830, 0.59447, 0.59977, 0.60258, 0.60355, 0.60398, 0.60545,\
        0.60741, 0.60810, 0.60746, 0.60708, 0.60529, 0.60254, 0.60030, 0.59856, 0.59787, 0.59864, 0.60038,\
        0.60175, 0.60250, 0.60296, 0.60332, 0.60568, 0.61023, 0.61746, 0.62863, 0.64284, 0.65764, 0.67067,\
        0.68005, 0.68505, 0.68721, 0.68834, 0.68854, 0.68665, 0.68309, 0.67787, 0.67182, 0.66566, 0.65963,\
        0.65362, 0.64744, 0.64091, 0.63390, 0.62632, 0.61839, 0.61044, 0.60285, 0.59624, 0.59175, 0.58700,\
        0.58257, 0.57902, 0.57648, 0.57496, 0.57442, 0.57490, 0.57654, 0.57947, 0.58361, 0.58857, 0.59368,\
        0.59826, 0.60177, 0.60288, 0.59970, 0.59278, 0.58375, 0.57352, 0.56270, 0.55194, 0.54153, 0.53130,\
        0.52103, 0.51140, 0.50286, 0.49418, 0.48374, 0.47289, 0.46209, 0.45134, 0.44087, 0.43100, 0.42210,\
        0.41445, 0.40777, 0.40160, 0.39686, 0.39238, 0.38742, 0.38157, 0.37470, 0.36722, 0.35935, 0.35119],\
        [0.33533, 0.34345, 0.35167, 0.35985, 0.36795, 0.37608, 0.38437, 0.39265, 0.39982, 0.40679, 0.41456,\
        0.42313, 0.43351, 0.44569, 0.45890, 0.47259, 0.48633, 0.49917, 0.51038, 0.52084, 0.53091, 0.54059,\
        0.54983, 0.55859, 0.56680, 0.57435, 0.58110, 0.58691, 0.59141, 0.59385, 0.59452, 0.59463, 0.59571,\
        0.59726, 0.59745, 0.59604, 0.59535, 0.59331, 0.59012, 0.58759, 0.58582, 0.58516, 0.58577, 0.58718,\
        0.58846, 0.58924, 0.58964, 0.59008, 0.59212, 0.59603, 0.60242, 0.61396, 0.62928, 0.64473, 0.65789,\
        0.66742, 0.67309, 0.67582, 0.67666, 0.67591, 0.67333, 0.66900, 0.66327, 0.65682, 0.65037, 0.64426,\
        0.63836, 0.63238, 0.62612, 0.61958, 0.61236, 0.60486, 0.59775, 0.59108, 0.58547, 0.58110, 0.57730,\
        0.57392, 0.57126, 0.56948, 0.56865, 0.56879, 0.56993, 0.57215, 0.57545, 0.57975, 0.58474, 0.58999,\
        0.59500, 0.59921, 0.60082, 0.59542, 0.58624, 0.57522, 0.56348, 0.55182, 0.54087, 0.53081, 0.52119,\
        0.51086, 0.50029, 0.49157, 0.48250, 0.47244, 0.46181, 0.45106, 0.44034, 0.42982, 0.41981, 0.41082,\
        0.40359, 0.39763, 0.39211, 0.38742, 0.38310, 0.37841, 0.37298, 0.36670, 0.35974, 0.35227, 0.34439],\
        [0.33068, 0.33842, 0.34621, 0.35398, 0.36171, 0.36943, 0.37709, 0.38446, 0.39117, 0.39782, 0.40494,\
        0.41206, 0.42263, 0.43585, 0.45011, 0.46463, 0.47878, 0.49189, 0.50362, 0.51456, 0.52484, 0.53450,\
        0.54353, 0.55194, 0.55973, 0.56682, 0.57307, 0.57827, 0.58211, 0.58431, 0.58514, 0.58540, 0.58588,\
        0.58631, 0.58585, 0.58443, 0.58273, 0.58034, 0.57736, 0.57475, 0.57296, 0.57222, 0.57253, 0.57347,\
        0.57447, 0.57518, 0.57566, 0.57645, 0.57845, 0.58237, 0.58921, 0.60101, 0.61708, 0.63268, 0.64575,\
        0.65511, 0.66074, 0.66341, 0.66390, 0.66251, 0.65925, 0.65425, 0.64790, 0.64095, 0.63428, 0.62837,\
        0.62294, 0.61743, 0.61148, 0.60579, 0.59937, 0.59234, 0.58601, 0.58024, 0.57534, 0.57140, 0.56820,\
        0.56562, 0.56374, 0.56267, 0.56247, 0.56317, 0.56479, 0.56735, 0.57081, 0.57507, 0.57990, 0.58501,\
        0.59001, 0.59429, 0.59517, 0.58800, 0.57710, 0.56466, 0.55173, 0.53928, 0.52823, 0.51878, 0.51039,\
        0.50233, 0.49086, 0.48052, 0.47064, 0.46054, 0.45023, 0.43989, 0.42955, 0.41923, 0.40936, 0.40050,\
        0.39352, 0.38802, 0.38306, 0.37855, 0.37425, 0.36968, 0.36454, 0.35868, 0.35215, 0.34505, 0.33747],\
        [0.32677, 0.33424, 0.34172, 0.34917, 0.35658, 0.36391, 0.37109, 0.37800, 0.38461, 0.39118, 0.39724,\
        0.40264, 0.41437, 0.42822, 0.44284, 0.45755, 0.47176, 0.48498, 0.49709, 0.50825, 0.51865, 0.52819,\
        0.53685, 0.54481, 0.55212, 0.55871, 0.56443, 0.56908, 0.57246, 0.57449, 0.57538, 0.57559, 0.57550,\
        0.57505, 0.57394, 0.57212, 0.56980, 0.56704, 0.56410, 0.56151, 0.55971, 0.55888, 0.55895, 0.55959,\
        0.56036, 0.56100, 0.56160, 0.56262, 0.56490, 0.56946, 0.57756, 0.59028, 0.60596, 0.62128, 0.63416,\
        0.64283, 0.64776, 0.65001, 0.65019, 0.64846, 0.64481, 0.63930, 0.63234, 0.62478, 0.61782, 0.61241,\
        0.60770, 0.60296, 0.59743, 0.59214, 0.58675, 0.58069, 0.57502, 0.56998, 0.56571, 0.56226, 0.55959,\
        0.55762, 0.55638, 0.55591, 0.55622, 0.55734, 0.55925, 0.56191, 0.56527, 0.56918, 0.57345, 0.57776,\
        0.58158, 0.58385, 0.58237, 0.57549, 0.56470, 0.55191, 0.53836, 0.52518, 0.51400, 0.50525, 0.49744,\
        0.48964, 0.47965, 0.46887, 0.45847, 0.44833, 0.43842, 0.42877, 0.41923, 0.40952, 0.39992, 0.39173,\
        0.38491, 0.37937, 0.37457, 0.37018, 0.36590, 0.36141, 0.35645, 0.35087, 0.34466, 0.33786, 0.33054],\
        [0.32359, 0.33087, 0.33813, 0.34536, 0.35252, 0.35958, 0.36649, 0.37322, 0.37978, 0.38621, 0.39251,\
        0.40001, 0.41038, 0.42341, 0.43753, 0.45187, 0.46585, 0.47904, 0.49118, 0.50221, 0.51276, 0.52184,\
        0.52989, 0.53728, 0.54409, 0.55020, 0.55544, 0.55964, 0.56266, 0.56449, 0.56529, 0.56530, 0.56474,\
        0.56364, 0.56191, 0.55959, 0.55678, 0.55367, 0.55058, 0.54793, 0.54609, 0.54521, 0.54519, 0.54570,\
        0.54638, 0.54697, 0.54756, 0.54862, 0.55097, 0.55583, 0.56446, 0.57737, 0.59312, 0.60939, 0.62245,\
        0.62972, 0.63349, 0.63532, 0.63549, 0.63385, 0.63021, 0.62455, 0.61721, 0.60921, 0.60176, 0.59705,\
        0.59294, 0.58874, 0.58408, 0.57929, 0.57443, 0.56937, 0.56450, 0.56013, 0.55643, 0.55348, 0.55129,\
        0.54983, 0.54910, 0.54909, 0.54979, 0.55116, 0.55317, 0.55573, 0.55873, 0.56200, 0.56530, 0.56827,\
        0.57030, 0.57037, 0.56724, 0.56026, 0.55006, 0.53775, 0.52431, 0.51075, 0.49906, 0.49088, 0.48320,\
        0.47517, 0.46597, 0.45593, 0.44578, 0.43587, 0.42647, 0.41771, 0.40922, 0.40084, 0.39174, 0.38372,\
        0.37703, 0.37146, 0.36667, 0.36231, 0.35807, 0.35364, 0.34881, 0.34342, 0.33742, 0.33084, 0.32373],\
        [0.32108, 0.32825, 0.33539, 0.34247, 0.34949, 0.35641, 0.36323, 0.36994, 0.37661, 0.38336, 0.39049,\
        0.39859, 0.40879, 0.42081, 0.43397, 0.44758, 0.46106, 0.47402, 0.48625, 0.49775, 0.50746, 0.51533,\
        0.52255, 0.52938, 0.53575, 0.54146, 0.54630, 0.55011, 0.55280, 0.55441, 0.55501, 0.55476, 0.55380,\
        0.55217, 0.54993, 0.54713, 0.54388, 0.54039, 0.53703, 0.53422, 0.53229, 0.53140, 0.53139, 0.53193,\
        0.53262, 0.53312, 0.53354, 0.53430, 0.53614, 0.54040, 0.54856, 0.56133, 0.57785, 0.59597, 0.60898,\
        0.61402, 0.61701, 0.61904, 0.61973, 0.61865, 0.61545, 0.61027, 0.60273, 0.59512, 0.58811, 0.58308,\
        0.57909, 0.57524, 0.57124, 0.56707, 0.56281, 0.55850, 0.55435, 0.55060, 0.54743, 0.54494, 0.54317,\
        0.54213, 0.54180, 0.54214, 0.54309, 0.54459, 0.54655, 0.54883, 0.55128, 0.55369, 0.55580, 0.55724,\
        0.55746, 0.55571, 0.55130, 0.54407, 0.53451, 0.52323, 0.51051, 0.49790, 0.48626, 0.47713, 0.46895,\
        0.46066, 0.45181, 0.44245, 0.43289, 0.42341, 0.41442, 0.40679, 0.39906, 0.39130, 0.38333, 0.37584,\
        0.36938, 0.36391, 0.35918, 0.35489, 0.35071, 0.34638, 0.34166, 0.33640, 0.33054, 0.32409, 0.31711],\
        [0.31918, 0.32632, 0.33340, 0.34043, 0.34740, 0.35430, 0.36113, 0.36795, 0.37485, 0.38203, 0.38982,\
        0.39865, 0.40836, 0.41942, 0.43158, 0.44430, 0.45706, 0.46951, 0.48138, 0.49234, 0.50109, 0.50806,\
        0.51473, 0.52118, 0.52726, 0.53267, 0.53717, 0.54060, 0.54296, 0.54429, 0.54466, 0.54414, 0.54282,\
        0.54078, 0.53812, 0.53497, 0.53137, 0.52744, 0.52372, 0.52069, 0.51854, 0.51770, 0.51775, 0.51835,\
        0.51912, 0.51938, 0.51940, 0.51960, 0.52031, 0.52299, 0.52968, 0.54162, 0.55797, 0.57564, 0.58855,\
        0.59440, 0.59832, 0.60136, 0.60286, 0.60285, 0.59985, 0.59601, 0.58892, 0.58175, 0.57538, 0.57031,\
        0.56626, 0.56265, 0.55913, 0.55554, 0.55186, 0.54817, 0.54460, 0.54137, 0.53865, 0.53656, 0.53515,\
        0.53446, 0.53444, 0.53503, 0.53613, 0.53764, 0.53943, 0.54133, 0.54313, 0.54457, 0.54543, 0.54541,\
        0.54413, 0.54097, 0.53546, 0.52788, 0.51883, 0.50884, 0.49762, 0.48530, 0.47412, 0.46426, 0.45532,\
        0.44665, 0.43796, 0.42919, 0.42036, 0.41182, 0.40272, 0.39615, 0.38903, 0.38168, 0.37445, 0.36766,\
        0.36165, 0.35646, 0.35194, 0.34782, 0.34380, 0.33961, 0.33501, 0.32987, 0.32410, 0.31770, 0.31078],\
        [0.31783, 0.32499, 0.33209, 0.33914, 0.34613, 0.35308, 0.36002, 0.36699, 0.37410, 0.38154, 0.38956,\
        0.39836, 0.40793, 0.41844, 0.42981, 0.44163, 0.45351, 0.46509, 0.47600, 0.48572, 0.49366, 0.50033,\
        0.50672, 0.51294, 0.51886, 0.52406, 0.52818, 0.53118, 0.53316, 0.53419, 0.53430, 0.53354, 0.53194,\
        0.52957, 0.52655, 0.52322, 0.51954, 0.51516, 0.51092, 0.50787, 0.50525, 0.50454, 0.50452, 0.50490,\
        0.50587, 0.50546, 0.50482, 0.50447, 0.50373, 0.50435, 0.50877, 0.51919, 0.53432, 0.55056, 0.56399,\
        0.57290, 0.57879, 0.58285, 0.58517, 0.58565, 0.58370, 0.57999, 0.57448, 0.56833, 0.56268, 0.55796,\
        0.55412, 0.55084, 0.54778, 0.54473, 0.54159, 0.53840, 0.53528, 0.53245, 0.53009, 0.52832, 0.52721,\
        0.52677, 0.52699, 0.52775, 0.52891, 0.53034, 0.53190, 0.53338, 0.53452, 0.53502, 0.53465, 0.53334,\
        0.53099, 0.52707, 0.52061, 0.51244, 0.50366, 0.49430, 0.48408, 0.47284, 0.46191, 0.45179, 0.44232,\
        0.43327, 0.42464, 0.41653, 0.40842, 0.40079, 0.39395, 0.38609, 0.37902, 0.37207, 0.36541, 0.35924,\
        0.35376, 0.34901, 0.34484, 0.34102, 0.33727, 0.33329, 0.32887, 0.32385, 0.31817, 0.31180, 0.30483],\
        [0.31695, 0.32417, 0.33134, 0.33846, 0.34555, 0.35261, 0.35968, 0.36681, 0.37409, 0.38163, 0.38957,\
        0.39806, 0.40749, 0.41768, 0.42837, 0.43933, 0.45025, 0.46081, 0.47064, 0.47938, 0.48680, 0.49320,\
        0.49916, 0.50503, 0.51088, 0.51589, 0.51942, 0.52184, 0.52338, 0.52406, 0.52394, 0.52300, 0.52120,\
        0.51862, 0.51524, 0.51180, 0.50865, 0.50418, 0.49895, 0.49616, 0.49343, 0.49226, 0.49184, 0.49184,\
        0.49204, 0.49094, 0.48953, 0.48819, 0.48662, 0.48618, 0.48908, 0.49754, 0.51073, 0.52582, 0.54014,\
        0.55131, 0.55896, 0.56399, 0.56695, 0.56791, 0.56678, 0.56382, 0.55948, 0.55453, 0.54977, 0.54570,\
        0.54239, 0.53963, 0.53713, 0.53465, 0.53201, 0.52923, 0.52644, 0.52387, 0.52178, 0.52027, 0.51934,\
        0.51907, 0.51946, 0.52034, 0.52146, 0.52272, 0.52400, 0.52509, 0.52566, 0.52533, 0.52385, 0.52130,\
        0.51828, 0.51485, 0.50826, 0.49868, 0.49017, 0.48074, 0.47071, 0.46017, 0.44969, 0.43964, 0.42999,\
        0.42062, 0.41185, 0.40435, 0.39750, 0.38996, 0.38318, 0.37606, 0.36910, 0.36248, 0.35628, 0.35066,\
        0.34574, 0.34151, 0.33783, 0.33445, 0.33108, 0.32741, 0.32320, 0.31832, 0.31275, 0.30659, 0.29947],\
        [0.31645, 0.32377, 0.33105, 0.33829, 0.34549, 0.35270, 0.35992, 0.36722, 0.37464, 0.38226, 0.39015,\
        0.39848, 0.40757, 0.41724, 0.42722, 0.43731, 0.44725, 0.45680, 0.46567, 0.47363, 0.48060, 0.48676,\
        0.49245, 0.49798, 0.50375, 0.50838, 0.51076, 0.51243, 0.51358, 0.51380, 0.51350, 0.51245, 0.51046,\
        0.50799, 0.50450, 0.50034, 0.49810, 0.49386, 0.48895, 0.48526, 0.48250, 0.48069, 0.47956, 0.47875,\
        0.47775, 0.47605, 0.47391, 0.47170, 0.46983, 0.46942, 0.47212, 0.47918, 0.49025, 0.50384, 0.51912,\
        0.53134, 0.53963, 0.54519, 0.54872, 0.55030, 0.54990, 0.54777, 0.54445, 0.54056, 0.53675, 0.53351,\
        0.53095, 0.52892, 0.52713, 0.52529, 0.52316, 0.52071, 0.51811, 0.51567, 0.51377, 0.51248, 0.51163,\
        0.51137, 0.51193, 0.51287, 0.51383, 0.51479, 0.51575, 0.51650, 0.51667, 0.51572, 0.51330, 0.50950,\
        0.50555, 0.50362, 0.49948, 0.48866, 0.47829, 0.46815, 0.45793, 0.44769, 0.43761, 0.42790, 0.41847,\
        0.40907, 0.39996, 0.39269, 0.38612, 0.37931, 0.37256, 0.36582, 0.35920, 0.35291, 0.34712, 0.34199,\
        0.33762, 0.33398, 0.33090, 0.32809, 0.32520, 0.32192, 0.31799, 0.31323, 0.30760, 0.30155, 0.29494],\
        [0.31623, 0.32369, 0.33109, 0.33846, 0.34582, 0.35318, 0.36057, 0.36803, 0.37560, 0.38332, 0.39124,\
        0.39947, 0.40810, 0.41710, 0.42629, 0.43548, 0.44448, 0.45308, 0.46108, 0.46835, 0.47487, 0.48078,\
        0.48633, 0.49183, 0.49754, 0.50094, 0.50170, 0.50275, 0.50381, 0.50315, 0.50285, 0.50176, 0.49948,\
        0.49705, 0.49407, 0.49003, 0.48680, 0.48292, 0.47863, 0.47484, 0.47182, 0.46949, 0.46764, 0.46592,\
        0.46397, 0.46153, 0.45876, 0.45603, 0.45404, 0.45381, 0.45660, 0.46317, 0.47341, 0.48673, 0.50181,\
        0.51327, 0.52099, 0.52674, 0.53089, 0.53324, 0.53356, 0.53215, 0.52973, 0.52679, 0.52384, 0.52150,\
        0.51987, 0.51871, 0.51775, 0.51664, 0.51505, 0.51285, 0.51038, 0.50792, 0.50611, 0.50515, 0.50426,\
        0.50371, 0.50452, 0.50545, 0.50600, 0.50652, 0.50710, 0.50749, 0.50739, 0.50600, 0.50304, 0.49847,\
        0.49384, 0.49106, 0.48645, 0.47718, 0.46643, 0.45578, 0.44543, 0.43542, 0.42574, 0.41649, 0.40778,\
        0.39873, 0.38995, 0.38230, 0.37549, 0.36887, 0.36230, 0.35580, 0.34946, 0.34346, 0.33801, 0.33330,\
        0.32946, 0.32645, 0.32407, 0.32193, 0.31962, 0.31675, 0.31313, 0.30859, 0.30287, 0.29643, 0.28965],\
        [0.31620, 0.32381, 0.33135, 0.33887, 0.34637, 0.35388, 0.36143, 0.36905, 0.37676, 0.38458, 0.39251,\
        0.40053, 0.40873, 0.41706, 0.42544, 0.43376, 0.44186, 0.44957, 0.45675, 0.46333, 0.46931, 0.47480,\
        0.47996, 0.48492, 0.48928, 0.49176, 0.49237, 0.49291, 0.49333, 0.49246, 0.49166, 0.49034, 0.48815,\
        0.48561, 0.48261, 0.47910, 0.47555, 0.47182, 0.46796, 0.46435, 0.46121, 0.45852, 0.45611, 0.45370,\
        0.45101, 0.44792, 0.44459, 0.44146, 0.43929, 0.43910, 0.44195, 0.44843, 0.45839, 0.47110, 0.48460,\
        0.49515, 0.50270, 0.50883, 0.51379, 0.51715, 0.51813, 0.51718, 0.51570, 0.51367, 0.51135, 0.50993,\
        0.50931, 0.50902, 0.50892, 0.50863, 0.50766, 0.50561, 0.50323, 0.50103, 0.49901, 0.49826, 0.49733,\
        0.49666, 0.49729, 0.49788, 0.49794, 0.49796, 0.49804, 0.49789, 0.49739, 0.49544, 0.49205, 0.48735,\
        0.48249, 0.47793, 0.47205, 0.46360, 0.45352, 0.44305, 0.43285, 0.42327, 0.41426, 0.40502, 0.39727,\
        0.38874, 0.38031, 0.37262, 0.36563, 0.35901, 0.35256, 0.34624, 0.34010, 0.33431, 0.32910, 0.32471,\
        0.32135, 0.31899, 0.31736, 0.31601, 0.31431, 0.31180, 0.30836, 0.30408, 0.29860, 0.29176, 0.28443],\
        [0.31628, 0.32402, 0.33171, 0.33937, 0.34700, 0.35464, 0.36232, 0.37006, 0.37788, 0.38578, 0.39373,\
        0.40158, 0.40923, 0.41688, 0.42450, 0.43200, 0.43925, 0.44611, 0.45249, 0.45833, 0.46365, 0.46849,\
        0.47292, 0.47687, 0.47997, 0.48174, 0.48232, 0.48239, 0.48208, 0.48118, 0.47997, 0.47837, 0.47624,\
        0.47371, 0.47080, 0.46756, 0.46415, 0.46063, 0.45710, 0.45372, 0.45061, 0.44776, 0.44503, 0.44219,\
        0.43902, 0.43544, 0.43164, 0.42812, 0.42560, 0.42513, 0.42789, 0.43428, 0.44383, 0.45540, 0.46721,\
        0.47733, 0.48536, 0.49206, 0.49776, 0.50223, 0.50390, 0.50295, 0.50278, 0.50173, 0.49985, 0.49936,\
        0.49959, 0.49995, 0.50054, 0.50093, 0.50074, 0.49877, 0.49651, 0.49461, 0.49274, 0.49157, 0.49062,\
        0.49001, 0.48994, 0.48995, 0.48966, 0.48920, 0.48863, 0.48778, 0.48638, 0.48389, 0.48015, 0.47543,\
        0.47020, 0.46456, 0.45787, 0.44960, 0.44008, 0.43002, 0.42016, 0.41102, 0.40316, 0.39490, 0.38650,\
        0.37857, 0.37068, 0.36323, 0.35630, 0.34976, 0.34344, 0.33729, 0.33133, 0.32568, 0.32058, 0.31639,\
        0.31345, 0.31169, 0.31086, 0.31037, 0.30931, 0.30698, 0.30335, 0.29889, 0.29335, 0.28654, 0.27895],\
        [0.31636, 0.32425, 0.33207, 0.33984, 0.34758, 0.35532, 0.36308, 0.37087, 0.37872, 0.38661, 0.39447,\
        0.40206, 0.40928, 0.41634, 0.42330, 0.43007, 0.43653, 0.44258, 0.44815, 0.45320, 0.45774, 0.46177,\
        0.46527, 0.46816, 0.47023, 0.47131, 0.47152, 0.47117, 0.47041, 0.46928, 0.46782, 0.46606, 0.46394,\
        0.46148, 0.45873, 0.45573, 0.45258, 0.44934, 0.44611, 0.44298, 0.44002, 0.43720, 0.43440, 0.43142,\
        0.42804, 0.42418, 0.42003, 0.41615, 0.41321, 0.41204, 0.41454, 0.42084, 0.42995, 0.44052, 0.45121,\
        0.46095, 0.46937, 0.47663, 0.48286, 0.48774, 0.49018, 0.49054, 0.49098, 0.49085, 0.49018, 0.49017,\
        0.49079, 0.49162, 0.49252, 0.49318, 0.49311, 0.49180, 0.48995, 0.48813, 0.48646, 0.48510, 0.48401,\
        0.48317, 0.48257, 0.48198, 0.48119, 0.48018, 0.47891, 0.47726, 0.47501, 0.47191, 0.46784, 0.46294,\
        0.45739, 0.45119, 0.44409, 0.43592, 0.42686, 0.41720, 0.40774, 0.39912, 0.39146, 0.38410, 0.37624,\
        0.36857, 0.36114, 0.35407, 0.34740, 0.34107, 0.33499, 0.32908, 0.32333, 0.31781, 0.31277, 0.30860,\
        0.30605, 0.30484, 0.30468, 0.30504, 0.30453, 0.30231, 0.29818, 0.29329, 0.28747, 0.28061, 0.27293],\
        [0.31637, 0.32438, 0.33231, 0.34017, 0.34799, 0.35579, 0.36357, 0.37134, 0.37911, 0.38685, 0.39452,\
        0.40185, 0.40864, 0.41528, 0.42173, 0.42787, 0.43360, 0.43886, 0.44362, 0.44784, 0.45153, 0.45466,\
        0.45720, 0.45908, 0.46020, 0.46056, 0.46025, 0.45947, 0.45836, 0.45697, 0.45536, 0.45352, 0.45143,\
        0.44908, 0.44651, 0.44376, 0.44090, 0.43796, 0.43503, 0.43219, 0.42944, 0.42677, 0.42411, 0.42129,\
        0.41797, 0.41409, 0.40977, 0.40556, 0.40262, 0.40039, 0.40274, 0.40874, 0.41746, 0.42709, 0.43687,\
        0.44621, 0.45472, 0.46231, 0.46886, 0.47402, 0.47736, 0.47919, 0.48035, 0.48102, 0.48138, 0.48193,\
        0.48280, 0.48381, 0.48478, 0.48543, 0.48542, 0.48458, 0.48318, 0.48161, 0.48006, 0.47864, 0.47738,\
        0.47624, 0.47517, 0.47402, 0.47266, 0.47102, 0.46902, 0.46658, 0.46357, 0.45986, 0.45542, 0.45028,\
        0.44451, 0.43808, 0.43086, 0.42282, 0.41440, 0.40541, 0.39616, 0.38808, 0.38058, 0.37334, 0.36602,\
        0.35880, 0.35182, 0.34519, 0.33891, 0.33294, 0.32720, 0.32166, 0.31625, 0.31098, 0.30613, 0.30194,\
        0.29969, 0.29906, 0.29938, 0.29982, 0.29924, 0.29697, 0.29262, 0.28727, 0.28111, 0.27412, 0.26636],\
        [0.31622, 0.32434, 0.33235, 0.34028, 0.34814, 0.35595, 0.36370, 0.37139, 0.37899, 0.38646, 0.39369,\
        0.40057, 0.40727, 0.41371, 0.41977, 0.42535, 0.43039, 0.43488, 0.43881, 0.44219, 0.44501, 0.44724,\
        0.44884, 0.44976, 0.44999, 0.44958, 0.44869, 0.44747, 0.44604, 0.44445, 0.44271, 0.44086, 0.43885,\
        0.43663, 0.43424, 0.43176, 0.42919, 0.42654, 0.42390, 0.42134, 0.41886, 0.41637, 0.41392, 0.41156,\
        0.40852, 0.40466, 0.40121, 0.39632, 0.39334, 0.39172, 0.39343, 0.39895, 0.40685, 0.41524, 0.42417,\
        0.43304, 0.44148, 0.44923, 0.45605, 0.46164, 0.46581, 0.46870, 0.47071, 0.47214, 0.47323, 0.47426,\
        0.47534, 0.47642, 0.47736, 0.47794, 0.47798, 0.47741, 0.47634, 0.47500, 0.47355, 0.47211, 0.47069,\
        0.46927, 0.46778, 0.46611, 0.46417, 0.46188, 0.45917, 0.45598, 0.45225, 0.44796, 0.44313, 0.43778,\
        0.43190, 0.42547, 0.41846, 0.41046, 0.40238, 0.39481, 0.38601, 0.37791, 0.37040, 0.36321, 0.35617,\
        0.34933, 0.34279, 0.33663, 0.33083, 0.32533, 0.32003, 0.31497, 0.31017, 0.30529, 0.30074, 0.29715,\
        0.29483, 0.29402, 0.29400, 0.29394, 0.29282, 0.29015, 0.28591, 0.28048, 0.27417, 0.26707, 0.25925],\
        [0.31585, 0.32405, 0.33212, 0.34008, 0.34795, 0.35573, 0.36342, 0.37100, 0.37842, 0.38560, 0.39246,\
        0.39900, 0.40557, 0.41183, 0.41750, 0.42252, 0.42686, 0.43056, 0.43367, 0.43621, 0.43818, 0.43955,\
        0.44028, 0.44032, 0.43968, 0.43849, 0.43697, 0.43533, 0.43364, 0.43188, 0.43005, 0.42825, 0.42639,\
        0.42429, 0.42203, 0.41983, 0.41759, 0.41513, 0.41273, 0.41046, 0.40825, 0.40597, 0.40349, 0.40177,\
        0.39912, 0.39529, 0.39173, 0.38785, 0.38493, 0.38394, 0.38572, 0.39055, 0.39718, 0.40460, 0.41298,\
        0.42140, 0.42968, 0.43753, 0.44465, 0.45071, 0.45555, 0.45923, 0.46197, 0.46404, 0.46566, 0.46704,\
        0.46828, 0.46938, 0.47026, 0.47078, 0.47084, 0.47039, 0.46952, 0.46835, 0.46699, 0.46552, 0.46397,\
        0.46230, 0.46044, 0.45831, 0.45582, 0.45291, 0.44952, 0.44562, 0.44121, 0.43634, 0.43116, 0.42569,\
        0.41980, 0.41338, 0.40705, 0.39959, 0.39145, 0.38395, 0.37608, 0.36827, 0.36082, 0.35367, 0.34680,\
        0.34024, 0.33407, 0.32838, 0.32315, 0.31822, 0.31338, 0.30873, 0.30477, 0.30099, 0.29644, 0.29289,\
        0.29043, 0.28903, 0.28823, 0.28732, 0.28559, 0.28262, 0.27835, 0.27295, 0.26661, 0.25947, 0.25161],\
        [0.31519, 0.32344, 0.33154, 0.33950, 0.34735, 0.35508, 0.36269, 0.37016, 0.37744, 0.38446, 0.39116,\
        0.39760, 0.40387, 0.40977, 0.41497, 0.41934, 0.42293, 0.42583, 0.42811, 0.42983, 0.43101, 0.43161,\
        0.43157, 0.43082, 0.42936, 0.42738, 0.42521, 0.42319, 0.42136, 0.41950, 0.41751, 0.41583, 0.41425,\
        0.41224, 0.40993, 0.40805, 0.40626, 0.40381, 0.40161, 0.39955, 0.39755, 0.39547, 0.39313, 0.39120,\
        0.38873, 0.38542, 0.38201, 0.37885, 0.37664, 0.37619, 0.37806, 0.38229, 0.38823, 0.39544, 0.40323,\
        0.41122, 0.41936, 0.42730, 0.43474, 0.44123, 0.44659, 0.45083, 0.45411, 0.45663, 0.45862, 0.46023,\
        0.46157, 0.46266, 0.46347, 0.46392, 0.46395, 0.46356, 0.46278, 0.46170, 0.46040, 0.45891, 0.45726,\
        0.45538, 0.45322, 0.45069, 0.44771, 0.44424, 0.44023, 0.43570, 0.43063, 0.42517, 0.41961, 0.41412,\
        0.40851, 0.40200, 0.39580, 0.38921, 0.38158, 0.37404, 0.36652, 0.35903, 0.35173, 0.34467, 0.33791,\
        0.33153, 0.32566, 0.32041, 0.31581, 0.31158, 0.30732, 0.30271, 0.29911, 0.29593, 0.29203, 0.28845,\
        0.28570, 0.28369, 0.28209, 0.28036, 0.27800, 0.27468, 0.27027, 0.26484, 0.25849, 0.25134, 0.24348],\
        [0.31418, 0.32246, 0.33055, 0.33848, 0.34626, 0.35392, 0.36145, 0.36882, 0.37600, 0.38293, 0.38959,\
        0.39593, 0.40195, 0.40750, 0.41209, 0.41572, 0.41850, 0.42056, 0.42202, 0.42297, 0.42343, 0.42338,\
        0.42273, 0.42134, 0.41915, 0.41637, 0.41353, 0.41120, 0.40938, 0.40767, 0.40527, 0.40377, 0.40256,\
        0.40067, 0.39823, 0.39652, 0.39502, 0.39278, 0.39066, 0.38871, 0.38677, 0.38474, 0.38260, 0.38044,\
        0.37796, 0.37508, 0.37214, 0.36961, 0.36807, 0.36812, 0.37015, 0.37422, 0.38001, 0.38707, 0.39474,\
        0.40300, 0.41048, 0.41859, 0.42639, 0.43321, 0.43890, 0.44348, 0.44708, 0.44987, 0.45206, 0.45378,\
        0.45515, 0.45621, 0.45694, 0.45731, 0.45730, 0.45690, 0.45614, 0.45509, 0.45381, 0.45231, 0.45058,\
        0.44857, 0.44618, 0.44333, 0.43993, 0.43595, 0.43142, 0.42635, 0.42074, 0.41462, 0.40856, 0.40313,\
        0.39771, 0.39188, 0.38541, 0.37888, 0.37188, 0.36469, 0.35747, 0.35027, 0.34315, 0.33620, 0.32951,\
        0.32326, 0.31759, 0.31264, 0.30865, 0.30525, 0.30164, 0.29770, 0.29368, 0.29027, 0.28675, 0.28334,\
        0.28037, 0.27786, 0.27556, 0.27311, 0.27017, 0.26645, 0.26184, 0.25631, 0.24993, 0.24277, 0.23490],\
        [0.31278, 0.32104, 0.32909, 0.33694, 0.34464, 0.35219, 0.35961, 0.36687, 0.37395, 0.38083, 0.38746,\
        0.39382, 0.39983, 0.40472, 0.40860, 0.41144, 0.41339, 0.41460, 0.41526, 0.41554, 0.41535, 0.41474,\
        0.41368, 0.41188, 0.40915, 0.40566, 0.40222, 0.39966, 0.39786, 0.39630, 0.39406, 0.39234, 0.39108,\
        0.38930, 0.38724, 0.38545, 0.38379, 0.38188, 0.37990, 0.37797, 0.37604, 0.37404, 0.37194, 0.36971,\
        0.36729, 0.36470, 0.36219, 0.36020, 0.35919, 0.35965, 0.36193, 0.36609, 0.37199, 0.37928, 0.38753,\
        0.39582, 0.40296, 0.41169, 0.41962, 0.42654, 0.43236, 0.43707, 0.44079, 0.44367, 0.44591, 0.44765,\
        0.44900, 0.44999, 0.45064, 0.45092, 0.45083, 0.45037, 0.44958, 0.44851, 0.44722, 0.44571, 0.44395,\
        0.44188, 0.43936, 0.43628, 0.43254, 0.42811, 0.42310, 0.41766, 0.41176, 0.40518, 0.39841, 0.39288,\
        0.38742, 0.38165, 0.37551, 0.36914, 0.36253, 0.35576, 0.34891, 0.34202, 0.33513, 0.32830, 0.32166,\
        0.31546, 0.31000, 0.30515, 0.30147, 0.29889, 0.29549, 0.29185, 0.28806, 0.28446, 0.28096, 0.27758,\
        0.27442, 0.27149, 0.26862, 0.26558, 0.26211, 0.25800, 0.25314, 0.24747, 0.24101, 0.23382, 0.22593],\
        [0.31094, 0.31916, 0.32712, 0.33485, 0.34241, 0.34982, 0.35709, 0.36421, 0.37115, 0.37789, 0.38438,\
        0.39056, 0.39614, 0.40068, 0.40411, 0.40614, 0.40741, 0.40780, 0.40769, 0.40746, 0.40666, 0.40552,\
        0.40424, 0.40218, 0.39920, 0.39538, 0.39177, 0.38899, 0.38695, 0.38516, 0.38325, 0.38149, 0.37995,\
        0.37826, 0.37644, 0.37467, 0.37295, 0.37114, 0.36927, 0.36738, 0.36547, 0.36349, 0.36140, 0.35919,\
        0.35685, 0.35447, 0.35229, 0.35067, 0.35004, 0.35082, 0.35333, 0.35771, 0.36387, 0.37152, 0.38014,\
        0.38897, 0.39788, 0.40635, 0.41408, 0.42097, 0.42678, 0.43146, 0.43511, 0.43791, 0.44008, 0.44175,\
        0.44302, 0.44394, 0.44451, 0.44470, 0.44451, 0.44395, 0.44307, 0.44194, 0.44061, 0.43910, 0.43737,\
        0.43531, 0.43277, 0.42958, 0.42560, 0.42077, 0.41525, 0.40943, 0.40357, 0.39689, 0.38994, 0.38369,\
        0.37784, 0.37199, 0.36600, 0.35987, 0.35363, 0.34728, 0.34083, 0.33431, 0.32771, 0.32108, 0.31449,\
        0.30822, 0.30298, 0.29866, 0.29483, 0.29202, 0.28893, 0.28549, 0.28190, 0.27830, 0.27475, 0.27129,\
        0.26792, 0.26463, 0.26129, 0.25775, 0.25382, 0.24934, 0.24420, 0.23836, 0.23179, 0.22453, 0.21661],\
        [0.30863, 0.31678, 0.32459, 0.33213, 0.33950, 0.34674, 0.35384, 0.36076, 0.36748, 0.37395, 0.38012,\
        0.38586, 0.39095, 0.39519, 0.39805, 0.39949, 0.40038, 0.40004, 0.39924, 0.39862, 0.39737, 0.39570,\
        0.39407, 0.39175, 0.38868, 0.38510, 0.38166, 0.37878, 0.37648, 0.37450, 0.37265, 0.37091, 0.36926,\
        0.36759, 0.36587, 0.36415, 0.36242, 0.36065, 0.35885, 0.35701, 0.35513, 0.35318, 0.35113, 0.34897,\
        0.34673, 0.34452, 0.34255, 0.34114, 0.34071, 0.34166, 0.34435, 0.34896, 0.35547, 0.36357, 0.37278,\
        0.38248, 0.39216, 0.40111, 0.40915, 0.41617, 0.42193, 0.42643, 0.42986, 0.43245, 0.43443, 0.43597,\
        0.43715, 0.43801, 0.43851, 0.43861, 0.43831, 0.43761, 0.43659, 0.43534, 0.43395, 0.43244, 0.43077,\
        0.42882, 0.42639, 0.42324, 0.41915, 0.41401, 0.40796, 0.40146, 0.39538, 0.38873, 0.38177, 0.37512,\
        0.36887, 0.36284, 0.35691, 0.35102, 0.34513, 0.33921, 0.33322, 0.32712, 0.32089, 0.31457, 0.30822,\
        0.30190, 0.29658, 0.29245, 0.28874, 0.28549, 0.28229, 0.27890, 0.27536, 0.27176, 0.26814, 0.26454,\
        0.26095, 0.25733, 0.25359, 0.24963, 0.24529, 0.24046, 0.23505, 0.22901, 0.22231, 0.21496, 0.20698],\
        [0.30587, 0.31390, 0.32144, 0.32872, 0.33587, 0.34291, 0.34980, 0.35648, 0.36290, 0.36901, 0.37472,\
        0.37992, 0.38442, 0.38800, 0.39036, 0.39157, 0.39191, 0.39130, 0.39016, 0.38891, 0.38729, 0.38536,\
        0.38326, 0.38076, 0.37780, 0.37455, 0.37139, 0.36860, 0.36623, 0.36417, 0.36231, 0.36057, 0.35891,\
        0.35725, 0.35558, 0.35389, 0.35218, 0.35045, 0.34869, 0.34690, 0.34508, 0.34320, 0.34123, 0.33917,\
        0.33704, 0.33494, 0.33308, 0.33175, 0.33134, 0.33229, 0.33504, 0.33985, 0.34675, 0.35542, 0.36534,\
        0.37586, 0.38630, 0.39599, 0.40463, 0.41196, 0.41759, 0.42178, 0.42483, 0.42708, 0.42880, 0.43018,\
        0.43129, 0.43212, 0.43260, 0.43264, 0.43222, 0.43134, 0.43011, 0.42868, 0.42717, 0.42566, 0.42408,\
        0.42231, 0.42011, 0.41716, 0.41315, 0.40785, 0.40145, 0.39453, 0.38769, 0.38074, 0.37366, 0.36676,\
        0.36023, 0.35405, 0.34817, 0.34249, 0.33696, 0.33149, 0.32599, 0.32039, 0.31458, 0.30860, 0.30266,\
        0.29674, 0.29126, 0.28673, 0.28279, 0.27920, 0.27572, 0.27221, 0.26860, 0.26491, 0.26117, 0.25740,\
        0.25356, 0.24964, 0.24555, 0.24121, 0.23651, 0.23137, 0.22569, 0.21944, 0.21259, 0.20513, 0.19708],\
        [0.30273, 0.31050, 0.31758, 0.32454, 0.33148, 0.33834, 0.34501, 0.35140, 0.35745, 0.36312, 0.36831,\
        0.37292, 0.37678, 0.37973, 0.38163, 0.38249, 0.38244, 0.38159, 0.38020, 0.37851, 0.37656, 0.37438,\
        0.37204, 0.36947, 0.36668, 0.36376, 0.36092, 0.35833, 0.35604, 0.35402, 0.35219, 0.35048, 0.34883,\
        0.34720, 0.34557, 0.34391, 0.34224, 0.34054, 0.33882, 0.33710, 0.33536, 0.33359, 0.33176, 0.32985,\
        0.32784, 0.32584, 0.32402, 0.32266, 0.32212, 0.32287, 0.32551, 0.33046, 0.33778, 0.34711, 0.35783,\
        0.36924, 0.38064, 0.39138, 0.40094, 0.40812, 0.41349, 0.41724, 0.41977, 0.42159, 0.42303, 0.42426,\
        0.42534, 0.42622, 0.42674, 0.42678, 0.42625, 0.42516, 0.42364, 0.42193, 0.42025, 0.41870, 0.41721,\
        0.41563, 0.41374, 0.41113, 0.40731, 0.40220, 0.39530, 0.38796, 0.38050, 0.37303, 0.36564, 0.35848,\
        0.35173, 0.34545, 0.33963, 0.33419, 0.32903, 0.32401, 0.31904, 0.31400, 0.30866, 0.30292, 0.29713,\
        0.29145, 0.28604, 0.28122, 0.27692, 0.27296, 0.26918, 0.26543, 0.26165, 0.25780, 0.25389, 0.24990,\
        0.24582, 0.24161, 0.23719, 0.23252, 0.22749, 0.22205, 0.21612, 0.20966, 0.20264, 0.19506, 0.18691],\
        [0.29934, 0.30614, 0.31274, 0.31950, 0.32636, 0.33309, 0.33953, 0.34558, 0.35122, 0.35640, 0.36104,\
        0.36505, 0.36830, 0.37068, 0.37211, 0.37260, 0.37223, 0.37113, 0.36950, 0.36752, 0.36529, 0.36293,\
        0.36048, 0.35796, 0.35539, 0.35281, 0.35030, 0.34798, 0.34587, 0.34397, 0.34223, 0.34060, 0.33901,\
        0.33742, 0.33583, 0.33423, 0.33260, 0.33094, 0.32927, 0.32762, 0.32601, 0.32439, 0.32275, 0.32105,\
        0.31921, 0.31729, 0.31549, 0.31409, 0.31331, 0.31366, 0.31598, 0.32098, 0.32876, 0.33881, 0.35036,\
        0.36264, 0.37492, 0.38653, 0.39645, 0.40392, 0.40943, 0.41251, 0.41437, 0.41575, 0.41694, 0.41807,\
        0.41921, 0.42026, 0.42094, 0.42104, 0.42042, 0.41910, 0.41721, 0.41509, 0.41316, 0.41156, 0.41010,\
        0.40865, 0.40699, 0.40487, 0.40094, 0.39571, 0.38884, 0.38114, 0.37326, 0.36539, 0.35765, 0.35021,\
        0.34325, 0.33690, 0.33118, 0.32601, 0.32124, 0.31666, 0.31216, 0.30779, 0.30294, 0.29748, 0.29167,\
        0.28599, 0.28057, 0.27555, 0.27094, 0.26665, 0.26255, 0.25853, 0.25451, 0.25045, 0.24632, 0.24210,\
        0.23776, 0.23326, 0.22854, 0.22355, 0.21823, 0.21250, 0.20633, 0.19966, 0.19247, 0.18475, 0.17650],\
        [0.29344, 0.30026, 0.30688, 0.31369, 0.32066, 0.32731, 0.33347, 0.33913, 0.34430, 0.34897, 0.35305,\
        0.35649, 0.35918, 0.36105, 0.36204, 0.36216, 0.36150, 0.36015, 0.35829, 0.35606, 0.35363, 0.35112,\
        0.34866, 0.34629, 0.34400, 0.34177, 0.33961, 0.33758, 0.33571, 0.33400, 0.33242, 0.33090, 0.32941,\
        0.32789, 0.32637, 0.32483, 0.32328, 0.32167, 0.32004, 0.31849, 0.31703, 0.31558, 0.31417, 0.31277,\
        0.31118, 0.30933, 0.30757, 0.30624, 0.30527, 0.30505, 0.30685, 0.31182, 0.32010, 0.33086, 0.34312,\
        0.35604, 0.36892, 0.38110, 0.39181, 0.40060, 0.40520, 0.40708, 0.40830, 0.40942, 0.41047, 0.41155,\
        0.41286, 0.41422, 0.41518, 0.41539, 0.41474, 0.41322, 0.41092, 0.40822, 0.40599, 0.40437, 0.40283,\
        0.40134, 0.39962, 0.39759, 0.39389, 0.38821, 0.38140, 0.37376, 0.36574, 0.35763, 0.34962, 0.34192,\
        0.33476, 0.32834, 0.32276, 0.31790, 0.31353, 0.30937, 0.30515, 0.30114, 0.29731, 0.29166, 0.28589,\
        0.28020, 0.27472, 0.26956, 0.26471, 0.26013, 0.25573, 0.25143, 0.24715, 0.24285, 0.23847, 0.23400,\
        0.22940, 0.22462, 0.21961, 0.21433, 0.20872, 0.20273, 0.19632, 0.18944, 0.18208, 0.17422, 0.16585],\
        [0.28700, 0.29397, 0.30048, 0.30758, 0.31477, 0.32120, 0.32694, 0.33211, 0.33678, 0.34092, 0.34448,\
        0.34738, 0.34956, 0.35096, 0.35155, 0.35135, 0.35041, 0.34883, 0.34675, 0.34432, 0.34171, 0.33910,\
        0.33667, 0.33452, 0.33258, 0.33073, 0.32892, 0.32719, 0.32559, 0.32411, 0.32273, 0.32139, 0.32002,\
        0.31858, 0.31715, 0.31572, 0.31429, 0.31280, 0.31112, 0.30968, 0.30848, 0.30714, 0.30594, 0.30496,\
        0.30372, 0.30193, 0.30025, 0.29909, 0.29823, 0.29771, 0.29899, 0.30393, 0.31249, 0.32377, 0.33639,\
        0.34953, 0.36256, 0.37492, 0.38603, 0.39481, 0.39888, 0.40039, 0.40146, 0.40267, 0.40372, 0.40473,\
        0.40627, 0.40813, 0.40941, 0.40972, 0.40909, 0.40750, 0.40492, 0.40165, 0.39900, 0.39732, 0.39560,\
        0.39388, 0.39192, 0.38933, 0.38545, 0.37999, 0.37334, 0.36587, 0.35792, 0.34975, 0.34159, 0.33366,\
        0.32628, 0.31978, 0.31438, 0.30986, 0.30583, 0.30215, 0.29810, 0.29416, 0.29012, 0.28503, 0.27947,\
        0.27387, 0.26840, 0.26314, 0.25811, 0.25330, 0.24864, 0.24407, 0.23954, 0.23498, 0.23035, 0.22562,\
        0.22075, 0.21569, 0.21041, 0.20485, 0.19897, 0.19273, 0.18609, 0.17901, 0.17147, 0.16346, 0.15497],\
        [0.28039, 0.28769, 0.29466, 0.30210, 0.30903, 0.31482, 0.31992, 0.32453, 0.32869, 0.33233, 0.33539,\
        0.33782, 0.33955, 0.34055, 0.34079, 0.34030, 0.33912, 0.33734, 0.33508, 0.33248, 0.32971, 0.32700,\
        0.32461, 0.32274, 0.32127, 0.31984, 0.31831, 0.31685, 0.31555, 0.31431, 0.31314, 0.31202, 0.31084,\
        0.30945, 0.30816, 0.30686, 0.30557, 0.30448, 0.30251, 0.30123, 0.30035, 0.29913, 0.29806, 0.29738,\
        0.29646, 0.29497, 0.29358, 0.29261, 0.29205, 0.29216, 0.29395, 0.29837, 0.30696, 0.31811, 0.33046,\
        0.34324, 0.35583, 0.36769, 0.37809, 0.38600, 0.39062, 0.39290, 0.39436, 0.39571, 0.39685, 0.39793,\
        0.39970, 0.40204, 0.40349, 0.40371, 0.40314, 0.40152, 0.39904, 0.39564, 0.39269, 0.39046, 0.38840,\
        0.38629, 0.38386, 0.38078, 0.37665, 0.37131, 0.36489, 0.35767, 0.34991, 0.34184, 0.33367, 0.32559,\
        0.31797, 0.31136, 0.30608, 0.30222, 0.29817, 0.29461, 0.29076, 0.28676, 0.28249, 0.27767, 0.27240,\
        0.26697, 0.26155, 0.25625, 0.25111, 0.24611, 0.24123, 0.23642, 0.23163, 0.22682, 0.22194, 0.21695,\
        0.21182, 0.20649, 0.20094, 0.19512, 0.18899, 0.18251, 0.17564, 0.16836, 0.16065, 0.15248, 0.14385],\
        [0.27353, 0.28134, 0.28899, 0.29661, 0.30279, 0.30790, 0.31228, 0.31636, 0.32005, 0.32324, 0.32587,\
        0.32788, 0.32923, 0.32988, 0.32984, 0.32911, 0.32775, 0.32583, 0.32345, 0.32074, 0.31786, 0.31504,\
        0.31262, 0.31111, 0.31021, 0.30922, 0.30787, 0.30658, 0.30567, 0.30459, 0.30364, 0.30275, 0.30183,\
        0.30049, 0.29934, 0.29829, 0.29706, 0.29634, 0.29457, 0.29328, 0.29239, 0.29146, 0.29061, 0.28997,\
        0.28923, 0.28828, 0.28742, 0.28693, 0.28702, 0.28809, 0.29083, 0.29599, 0.30366, 0.31394, 0.32536,\
        0.33720, 0.34887, 0.35979, 0.36929, 0.37669, 0.38169, 0.38479, 0.38686, 0.38849, 0.38992, 0.39139,\
        0.39329, 0.39545, 0.39678, 0.39695, 0.39638, 0.39474, 0.39230, 0.38923, 0.38622, 0.38355, 0.38105,\
        0.37849, 0.37559, 0.37210, 0.36776, 0.36246, 0.35625, 0.34931, 0.34183, 0.33402, 0.32604, 0.31800,\
        0.31013, 0.30354, 0.29836, 0.29474, 0.29078, 0.28696, 0.28309, 0.27901, 0.27465, 0.26991, 0.26483,\
        0.25955, 0.25420, 0.24890, 0.24367, 0.23853, 0.23346, 0.22844, 0.22342, 0.21836, 0.21323, 0.20799,\
        0.20260, 0.19702, 0.19121, 0.18513, 0.17876, 0.17204, 0.16496, 0.15749, 0.14960, 0.14128, 0.13251],\
        [0.26608, 0.27419, 0.28199, 0.28928, 0.29555, 0.30006, 0.30389, 0.30759, 0.31089, 0.31369, 0.31595,\
        0.31761, 0.31865, 0.31903, 0.31877, 0.31788, 0.31640, 0.31441, 0.31199, 0.30926, 0.30639, 0.30356,\
        0.30111, 0.29988, 0.29948, 0.29885, 0.29768, 0.29652, 0.29588, 0.29500, 0.29422, 0.29350, 0.29271,\
        0.29167, 0.29065, 0.28973, 0.28878, 0.28787, 0.28668, 0.28557, 0.28471, 0.28398, 0.28336, 0.28285,\
        0.28238, 0.28193, 0.28167, 0.28186, 0.28277, 0.28481, 0.28846, 0.29412, 0.30179, 0.31069, 0.32073,\
        0.33130, 0.34182, 0.35169, 0.36035, 0.36735, 0.37255, 0.37621, 0.37884, 0.38089, 0.38267, 0.38442,\
        0.38627, 0.38801, 0.38912, 0.38933, 0.38868, 0.38715, 0.38487, 0.38207, 0.37913, 0.37625, 0.37340,\
        0.37044, 0.36715, 0.36334, 0.35884, 0.35356, 0.34754, 0.34088, 0.33374, 0.32631, 0.31872, 0.31114,\
        0.30361, 0.29656, 0.29161, 0.28734, 0.28331, 0.27931, 0.27526, 0.27106, 0.26662, 0.26189, 0.25689,\
        0.25170, 0.24641, 0.24110, 0.23580, 0.23055, 0.22532, 0.22010, 0.21487, 0.20959, 0.20422, 0.19874,\
        0.19310, 0.18727, 0.18121, 0.17489, 0.16828, 0.16134, 0.15406, 0.14639, 0.13832, 0.12985, 0.12094]]


    #--------------------------------------------------------------------------
    #Contour (isobar) of hurricane waves based on Young (1988)
    #Contour are for hurricane moving toward North
    #Data obtained from Ian Young
    #For hurricane with maximum velocity of Vmax=60 (m/s) and translational velocity of Vt=10 (m/s)

    #z grid values in form of H/Hmax ratio
    #vmax60vt10
    zgrid_H_Hmax_vmax60vt10=[[0.29402, 0.29699, 0.29913, 0.29991, 0.29912, 0.29785, 0.29639, 0.29489, 0.29348, 0.29222, 0.29113,\
        0.29024, 0.28954, 0.28903, 0.28872, 0.28858, 0.28861, 0.28879, 0.28910, 0.28950, 0.29000, 0.29058,\
        0.29123, 0.29196, 0.29275, 0.29360, 0.29450, 0.29542, 0.29636, 0.29726, 0.29811, 0.29885, 0.29949,\
        0.30023, 0.30130, 0.30262, 0.30419, 0.30603, 0.30818, 0.31068, 0.31356, 0.31686, 0.32060, 0.32482,\
        0.32954, 0.33479, 0.34059, 0.34693, 0.35383, 0.36126, 0.36919, 0.37759, 0.38642, 0.39564, 0.40486,\
        0.41408, 0.42349, 0.43295, 0.44239, 0.45169, 0.46080, 0.46967, 0.47840, 0.48726, 0.49629, 0.50535,\
        0.51445, 0.52322, 0.53143, 0.53894, 0.54566, 0.55156, 0.55664, 0.56094, 0.56451, 0.56739, 0.56965,\
        0.57135, 0.57257, 0.57334, 0.57370, 0.57369, 0.57337, 0.57280, 0.57203, 0.57110, 0.57004, 0.56886,\
        0.56757, 0.56615, 0.56459, 0.56289, 0.56102, 0.55898, 0.55677, 0.55438, 0.55182, 0.54909, 0.54618,\
        0.54310, 0.53984, 0.53641, 0.53280, 0.52899, 0.52500, 0.52081, 0.51641, 0.51180, 0.50695, 0.50185,\
        0.49628, 0.49045, 0.48466, 0.47872, 0.47257, 0.46618, 0.45956, 0.45271, 0.44563, 0.43834, 0.43083],\
        [0.29582, 0.29829, 0.29975, 0.29999, 0.29944, 0.29826, 0.29682, 0.29535, 0.29399, 0.29280, 0.29179,\
        0.29097, 0.29032, 0.28986, 0.28959, 0.28950, 0.28957, 0.28978, 0.29011, 0.29052, 0.29100, 0.29156,\
        0.29217, 0.29286, 0.29361, 0.29443, 0.29530, 0.29621, 0.29713, 0.29802, 0.29886, 0.29956, 0.30010,\
        0.30063, 0.30146, 0.30256, 0.30393, 0.30559, 0.30757, 0.30991, 0.31265, 0.31582, 0.31947, 0.32362,\
        0.32831, 0.33356, 0.33939, 0.34582, 0.35284, 0.36044, 0.36860, 0.37724, 0.38627, 0.39556, 0.40536,\
        0.41556, 0.42582, 0.43608, 0.44625, 0.45625, 0.46599, 0.47535, 0.48429, 0.49327, 0.50306, 0.51270,\
        0.52224, 0.53137, 0.53986, 0.54752, 0.55428, 0.56010, 0.56505, 0.56919, 0.57256, 0.57519, 0.57718,\
        0.57862, 0.57961, 0.58016, 0.58028, 0.58002, 0.57946, 0.57867, 0.57770, 0.57660, 0.57540, 0.57412,\
        0.57275, 0.57127, 0.56967, 0.56791, 0.56599, 0.56390, 0.56163, 0.55919, 0.55657, 0.55379, 0.55084,\
        0.54771, 0.54442, 0.54095, 0.53729, 0.53345, 0.52942, 0.52518, 0.52073, 0.51604, 0.51109, 0.50579,\
        0.50006, 0.49460, 0.48906, 0.48326, 0.47719, 0.47086, 0.46429, 0.45748, 0.45046, 0.44322, 0.43576],\
        [0.29730, 0.29927, 0.30023, 0.30024, 0.29960, 0.29845, 0.29705, 0.29567, 0.29444, 0.29340, 0.29251,\
        0.29178, 0.29120, 0.29080, 0.29057, 0.29053, 0.29063, 0.29088, 0.29122, 0.29162, 0.29207, 0.29257,\
        0.29313, 0.29375, 0.29444, 0.29521, 0.29603, 0.29690, 0.29779, 0.29866, 0.29945, 0.30010, 0.30053,\
        0.30095, 0.30157, 0.30244, 0.30360, 0.30506, 0.30685, 0.30902, 0.31160, 0.31464, 0.31817, 0.32224,\
        0.32687, 0.33210, 0.33796, 0.34445, 0.35160, 0.35938, 0.36778, 0.37673, 0.38615, 0.39592, 0.40629,\
        0.41719, 0.42824, 0.43932, 0.45032, 0.46110, 0.47160, 0.48173, 0.49113, 0.50000, 0.51013, 0.52054,\
        0.53056, 0.54001, 0.54870, 0.55646, 0.56317, 0.56882, 0.57355, 0.57746, 0.58060, 0.58295, 0.58461,\
        0.58575, 0.58651, 0.58685, 0.58672, 0.58618, 0.58536, 0.58434, 0.58317, 0.58189, 0.58055, 0.57917,\
        0.57773, 0.57621, 0.57456, 0.57275, 0.57077, 0.56862, 0.56629, 0.56379, 0.56112, 0.55828, 0.55528,\
        0.55211, 0.54877, 0.54526, 0.54156, 0.53768, 0.53361, 0.52933, 0.52482, 0.52007, 0.51504, 0.50972,\
        0.50423, 0.49887, 0.49342, 0.48766, 0.48163, 0.47533, 0.46880, 0.46204, 0.45506, 0.44788, 0.44047],\
        [0.29879, 0.30039, 0.30107, 0.30089, 0.30007, 0.29880, 0.29743, 0.29614, 0.29509, 0.29424, 0.29351,\
        0.29286, 0.29234, 0.29197, 0.29178, 0.29176, 0.29189, 0.29216, 0.29250, 0.29287, 0.29326, 0.29367,\
        0.29412, 0.29465, 0.29525, 0.29593, 0.29669, 0.29749, 0.29832, 0.29912, 0.29983, 0.30035, 0.30071,\
        0.30106, 0.30153, 0.30222, 0.30317, 0.30443, 0.30603, 0.30801, 0.31043, 0.31332, 0.31672, 0.32069,\
        0.32525, 0.33045, 0.33631, 0.34287, 0.35012, 0.35809, 0.36676, 0.37611, 0.38611, 0.39678, 0.40767,\
        0.41902, 0.43078, 0.44271, 0.45459, 0.46623, 0.47748, 0.48841, 0.49934, 0.50853, 0.51865, 0.52916,\
        0.53941, 0.54907, 0.55790, 0.56570, 0.57228, 0.57763, 0.58200, 0.58564, 0.58858, 0.59062, 0.59186,\
        0.59267, 0.59322, 0.59340, 0.59300, 0.59213, 0.59105, 0.58981, 0.58842, 0.58696, 0.58549, 0.58402,\
        0.58253, 0.58096, 0.57927, 0.57741, 0.57538, 0.57315, 0.57075, 0.56818, 0.56545, 0.56256, 0.55950,\
        0.55629, 0.55291, 0.54936, 0.54563, 0.54171, 0.53760, 0.53327, 0.52873, 0.52396, 0.51895, 0.51374,\
        0.50844, 0.50315, 0.49769, 0.49188, 0.48586, 0.47959, 0.47308, 0.46637, 0.45945, 0.45232, 0.44497],\
        [0.30053, 0.30191, 0.30239, 0.30207, 0.30107, 0.29974, 0.29829, 0.29706, 0.29623, 0.29560, 0.29500,\
        0.29441, 0.29388, 0.29352, 0.29333, 0.29330, 0.29343, 0.29370, 0.29403, 0.29434, 0.29461, 0.29488,\
        0.29519, 0.29558, 0.29605, 0.29662, 0.29727, 0.29798, 0.29871, 0.29942, 0.30001, 0.30042, 0.30071,\
        0.30098, 0.30134, 0.30187, 0.30265, 0.30371, 0.30512, 0.30692, 0.30916, 0.31189, 0.31516, 0.31901,\
        0.32349, 0.32864, 0.33450, 0.34109, 0.34845, 0.35660, 0.36553, 0.37525, 0.38570, 0.39677, 0.40847,\
        0.42070, 0.43333, 0.44619, 0.45904, 0.47163, 0.48367, 0.49492, 0.50606, 0.51707, 0.52770, 0.53831,\
        0.54864, 0.55837, 0.56728, 0.57511, 0.58153, 0.58641, 0.59026, 0.59357, 0.59637, 0.59812, 0.59890,\
        0.59937, 0.59970, 0.59975, 0.59903, 0.59784, 0.59654, 0.59511, 0.59349, 0.59183, 0.59022, 0.58867,\
        0.58714, 0.58555, 0.58382, 0.58191, 0.57980, 0.57749, 0.57501, 0.57237, 0.56957, 0.56662, 0.56352,\
        0.56026, 0.55684, 0.55325, 0.54949, 0.54554, 0.54140, 0.53706, 0.53250, 0.52774, 0.52277, 0.51764,\
        0.51242, 0.50709, 0.50155, 0.49582, 0.48990, 0.48362, 0.47713, 0.47046, 0.46361, 0.45654, 0.44927],\
        [0.30267, 0.30395, 0.30437, 0.30401, 0.30302, 0.30161, 0.29997, 0.29884, 0.29823, 0.29777, 0.29725,\
        0.29659, 0.29600, 0.29559, 0.29536, 0.29528, 0.29534, 0.29557, 0.29588, 0.29610, 0.29617, 0.29623,\
        0.29635, 0.29656, 0.29687, 0.29728, 0.29780, 0.29838, 0.29899, 0.29957, 0.30004, 0.30036, 0.30058,\
        0.30078, 0.30104, 0.30144, 0.30204, 0.30293, 0.30414, 0.30575, 0.30781, 0.31038, 0.31350, 0.31724,\
        0.32162, 0.32672, 0.33255, 0.33918, 0.34662, 0.35492, 0.36410, 0.37417, 0.38509, 0.39671, 0.40915,\
        0.42223, 0.43579, 0.44967, 0.46359, 0.47727, 0.49041, 0.50258, 0.51418, 0.52565, 0.53679, 0.54758,\
        0.55794, 0.56766, 0.57655, 0.58441, 0.59083, 0.59503, 0.59824, 0.60115, 0.60366, 0.60518, 0.60578,\
        0.60599, 0.60600, 0.60567, 0.60475, 0.60338, 0.60189, 0.60026, 0.59837, 0.59652, 0.59478, 0.59316,\
        0.59161, 0.59000, 0.58823, 0.58624, 0.58403, 0.58163, 0.57906, 0.57634, 0.57348, 0.57047, 0.56733,\
        0.56403, 0.56058, 0.55696, 0.55318, 0.54920, 0.54504, 0.54069, 0.53613, 0.53138, 0.52644, 0.52135,\
        0.51613, 0.51078, 0.50528, 0.49975, 0.49366, 0.48734, 0.48092, 0.47434, 0.46756, 0.46057, 0.45336],\
        [0.30531, 0.30661, 0.30710, 0.30683, 0.30595, 0.30463, 0.30316, 0.30208, 0.30144, 0.30096, 0.30039,\
        0.29956, 0.29885, 0.29835, 0.29803, 0.29782, 0.29771, 0.29783, 0.29813, 0.29818, 0.29796, 0.29775,\
        0.29764, 0.29764, 0.29773, 0.29795, 0.29829, 0.29874, 0.29921, 0.29965, 0.30000, 0.30021, 0.30037,\
        0.30051, 0.30068, 0.30094, 0.30139, 0.30210, 0.30313, 0.30455, 0.30643, 0.30883, 0.31180, 0.31541,\
        0.31970, 0.32473, 0.33054, 0.33717, 0.34468, 0.35310, 0.36248, 0.37288, 0.38435, 0.39686, 0.40984,\
        0.42359, 0.43812, 0.45309, 0.46814, 0.48290, 0.49707, 0.51024, 0.52253, 0.53439, 0.54581, 0.55672,\
        0.56705, 0.57666, 0.58536, 0.59299, 0.59954, 0.60339, 0.60611, 0.60859, 0.61058, 0.61182, 0.61233,\
        0.61236, 0.61205, 0.61137, 0.61026, 0.60878, 0.60709, 0.60522, 0.60318, 0.60116, 0.59925, 0.59753,\
        0.59598, 0.59431, 0.59247, 0.59038, 0.58804, 0.58554, 0.58289, 0.58010, 0.57717, 0.57412, 0.57093,\
        0.56760, 0.56413, 0.56049, 0.55669, 0.55270, 0.54853, 0.54417, 0.53961, 0.53487, 0.52995, 0.52487,\
        0.51964, 0.51426, 0.50873, 0.50296, 0.49685, 0.49075, 0.48449, 0.47801, 0.47132, 0.46441, 0.45728],\
        [0.30849, 0.30993, 0.31058, 0.31053, 0.30988, 0.30884, 0.30766, 0.30664, 0.30586, 0.30518, 0.30442,\
        0.30352, 0.30268, 0.30200, 0.30149, 0.30107, 0.30069, 0.30058, 0.30074, 0.30052, 0.29997, 0.29947,\
        0.29912, 0.29886, 0.29867, 0.29863, 0.29879, 0.29908, 0.29940, 0.29969, 0.29990, 0.30005, 0.30017,\
        0.30024, 0.30031, 0.30044, 0.30073, 0.30126, 0.30210, 0.30334, 0.30504, 0.30727, 0.31009, 0.31357,\
        0.31776, 0.32272, 0.32850, 0.33515, 0.34271, 0.35122, 0.36074, 0.37134, 0.38309, 0.39593, 0.40978,\
        0.42460, 0.44028, 0.45644, 0.47264, 0.48844, 0.50343, 0.51747, 0.53058, 0.54295, 0.55462, 0.56557,\
        0.57578, 0.58521, 0.59362, 0.60070, 0.60661, 0.61076, 0.61351, 0.61564, 0.61722, 0.61817, 0.61852,\
        0.61836, 0.61781, 0.61688, 0.61558, 0.61397, 0.61212, 0.61009, 0.60795, 0.60581, 0.60376, 0.60192,\
        0.60024, 0.59845, 0.59654, 0.59428, 0.59178, 0.58919, 0.58647, 0.58362, 0.58065, 0.57755, 0.57434,\
        0.57099, 0.56750, 0.56386, 0.56004, 0.55605, 0.55187, 0.54751, 0.54295, 0.53822, 0.53330, 0.52822,\
        0.52296, 0.51754, 0.51191, 0.50602, 0.50001, 0.49410, 0.48795, 0.48154, 0.47491, 0.46807, 0.46102],\
        [0.31219, 0.31387, 0.31480, 0.31504, 0.31471, 0.31398, 0.31306, 0.31212, 0.31124, 0.31037, 0.30944,\
        0.30843, 0.30745, 0.30655, 0.30577, 0.30507, 0.30445, 0.30401, 0.30367, 0.30312, 0.30233, 0.30154,\
        0.30085, 0.30025, 0.29970, 0.29935, 0.29931, 0.29945, 0.29960, 0.29974, 0.29987, 0.29997, 0.30004,\
        0.30003, 0.29998, 0.29997, 0.30009, 0.30044, 0.30111, 0.30217, 0.30369, 0.30575, 0.30842, 0.31178,\
        0.31587, 0.32077, 0.32653, 0.33320, 0.34082, 0.34943, 0.35905, 0.36975, 0.38161, 0.39486, 0.40946,\
        0.42546, 0.44248, 0.45985, 0.47717, 0.49397, 0.50977, 0.52453, 0.53835, 0.55124, 0.56316, 0.57406,\
        0.58399, 0.59312, 0.60126, 0.60793, 0.61331, 0.61734, 0.62016, 0.62214, 0.62346, 0.62419, 0.62435,\
        0.62402, 0.62328, 0.62216, 0.62071, 0.61896, 0.61699, 0.61486, 0.61263, 0.61040, 0.60824, 0.60622,\
        0.60429, 0.60232, 0.60031, 0.59774, 0.59518, 0.59260, 0.58983, 0.58692, 0.58390, 0.58078, 0.57755,\
        0.57420, 0.57071, 0.56706, 0.56325, 0.55925, 0.55507, 0.55071, 0.54616, 0.54142, 0.53651, 0.53142,\
        0.52615, 0.52070, 0.51507, 0.50928, 0.50344, 0.49755, 0.49131, 0.48492, 0.47835, 0.47159, 0.46460],\
        [0.31640, 0.31839, 0.31966, 0.32025, 0.32028, 0.31987, 0.31917, 0.31832, 0.31739, 0.31638, 0.31530,\
        0.31415, 0.31298, 0.31185, 0.31079, 0.30979, 0.30885, 0.30800, 0.30716, 0.30621, 0.30511, 0.30397,\
        0.30287, 0.30184, 0.30091, 0.30020, 0.29987, 0.29987, 0.29979, 0.29980, 0.29987, 0.29996, 0.30000,\
        0.29990, 0.29972, 0.29955, 0.29951, 0.29970, 0.30019, 0.30107, 0.30242, 0.30432, 0.30684, 0.31007,\
        0.31407, 0.31893, 0.32469, 0.33141, 0.33914, 0.34790, 0.35771, 0.36858, 0.38051, 0.39365, 0.40910,\
        0.42681, 0.44522, 0.46361, 0.48170, 0.49945, 0.51589, 0.53140, 0.54589, 0.55927, 0.57144, 0.58227,\
        0.59173, 0.60033, 0.60808, 0.61449, 0.61957, 0.62343, 0.62620, 0.62809, 0.62926, 0.62981, 0.62981,\
        0.62934, 0.62845, 0.62719, 0.62561, 0.62375, 0.62169, 0.61947, 0.61716, 0.61483, 0.61254, 0.61032,\
        0.60812, 0.60588, 0.60344, 0.60064, 0.59845, 0.59577, 0.59295, 0.59000, 0.58695, 0.58381, 0.58059,\
        0.57725, 0.57377, 0.57013, 0.56632, 0.56232, 0.55814, 0.55378, 0.54923, 0.54450, 0.53959, 0.53450,\
        0.52923, 0.52379, 0.51819, 0.51246, 0.50664, 0.50063, 0.49445, 0.48813, 0.48165, 0.47495, 0.46804],\
        [0.32104, 0.32341, 0.32507, 0.32606, 0.32645, 0.32636, 0.32588, 0.32513, 0.32418, 0.32307, 0.32185,\
        0.32053, 0.31916, 0.31778, 0.31643, 0.31511, 0.31381, 0.31254, 0.31124, 0.30985, 0.30835, 0.30681,\
        0.30526, 0.30377, 0.30241, 0.30129, 0.30057, 0.30018, 0.29991, 0.29981, 0.29985, 0.29997, 0.30001,\
        0.29983, 0.29952, 0.29922, 0.29903, 0.29906, 0.29939, 0.30010, 0.30128, 0.30302, 0.30539, 0.30850,\
        0.31242, 0.31723, 0.32302, 0.32984, 0.33774, 0.34676, 0.35694, 0.36828, 0.38083, 0.39473, 0.41101,\
        0.42972, 0.44914, 0.46830, 0.48646, 0.50415, 0.52182, 0.53818, 0.55324, 0.56702, 0.57946, 0.59037,\
        0.59940, 0.60738, 0.61457, 0.62060, 0.62541, 0.62908, 0.63175, 0.63354, 0.63460, 0.63503, 0.63491,\
        0.63432, 0.63331, 0.63194, 0.63026, 0.62833, 0.62618, 0.62389, 0.62150, 0.61907, 0.61664, 0.61422,\
        0.61178, 0.60927, 0.60663, 0.60399, 0.60145, 0.59865, 0.59587, 0.59287, 0.58977, 0.58665, 0.58346,\
        0.58016, 0.57670, 0.57307, 0.56926, 0.56527, 0.56109, 0.55673, 0.55218, 0.54745, 0.54254, 0.53746,\
        0.53220, 0.52678, 0.52120, 0.51548, 0.50961, 0.50359, 0.49745, 0.49124, 0.48482, 0.47819, 0.47135],\
        [0.32603, 0.32884, 0.33094, 0.33235, 0.33312, 0.33333, 0.33307, 0.33243, 0.33148, 0.33030, 0.32894,\
        0.32745, 0.32587, 0.32425, 0.32261, 0.32096, 0.31930, 0.31761, 0.31587, 0.31404, 0.31212, 0.31012,\
        0.30810, 0.30611, 0.30425, 0.30264, 0.30138, 0.30052, 0.30000, 0.29974, 0.29976, 0.29993, 0.30000,\
        0.29978, 0.29939, 0.29899, 0.29869, 0.29859, 0.29878, 0.29933, 0.30034, 0.30191, 0.30413, 0.30710,\
        0.31094, 0.31573, 0.32156, 0.32852, 0.33667, 0.34606, 0.35676, 0.36886, 0.38253, 0.39797, 0.41518,\
        0.43425, 0.45435, 0.47444, 0.49353, 0.51114, 0.52862, 0.54518, 0.56047, 0.57440, 0.58688, 0.59792,\
        0.60679, 0.61422, 0.62078, 0.62635, 0.63085, 0.63433, 0.63686, 0.63855, 0.63951, 0.63985, 0.63964,\
        0.63896, 0.63787, 0.63643, 0.63467, 0.63267, 0.63046, 0.62810, 0.62564, 0.62311, 0.62054, 0.61795,\
        0.61531, 0.61262, 0.60986, 0.60709, 0.60430, 0.60150, 0.59867, 0.59549, 0.59238, 0.58933, 0.58622,\
        0.58295, 0.57952, 0.57590, 0.57209, 0.56810, 0.56392, 0.55956, 0.55501, 0.55029, 0.54539, 0.54031,\
        0.53507, 0.52966, 0.52409, 0.51838, 0.51252, 0.50653, 0.50046, 0.49428, 0.48787, 0.48131, 0.47453],\
        [0.33130, 0.33460, 0.33717, 0.33902, 0.34018, 0.34070, 0.34065, 0.34012, 0.33919, 0.33795, 0.33647,\
        0.33481, 0.33303, 0.33117, 0.32925, 0.32728, 0.32528, 0.32321, 0.32106, 0.31881, 0.31644, 0.31398,\
        0.31146, 0.30894, 0.30651, 0.30428, 0.30239, 0.30094, 0.29998, 0.29955, 0.29959, 0.29986, 0.30000,\
        0.29977, 0.29936, 0.29892, 0.29856, 0.29836, 0.29842, 0.29881, 0.29965, 0.30104, 0.30310, 0.30593,\
        0.30967, 0.31442, 0.32031, 0.32743, 0.33587, 0.34571, 0.35701, 0.36988, 0.38453, 0.40119, 0.41937,\
        0.43924, 0.46009, 0.48101, 0.50100, 0.51905, 0.53605, 0.55228, 0.56745, 0.58133, 0.59354, 0.60403,\
        0.61294, 0.62025, 0.62646, 0.63167, 0.63590, 0.63917, 0.64156, 0.64315, 0.64404, 0.64430, 0.64403,\
        0.64329, 0.64214, 0.64064, 0.63884, 0.63678, 0.63453, 0.63211, 0.62958, 0.62696, 0.62427, 0.62154,\
        0.61875, 0.61590, 0.61300, 0.61007, 0.60712, 0.60414, 0.60103, 0.59778, 0.59480, 0.59193, 0.58891,\
        0.58568, 0.58225, 0.57863, 0.57482, 0.57082, 0.56664, 0.56227, 0.55773, 0.55301, 0.54812, 0.54306,\
        0.53783, 0.53244, 0.52689, 0.52119, 0.51534, 0.50938, 0.50335, 0.49709, 0.49077, 0.48432, 0.47761],\
        [0.33675, 0.34057, 0.34365, 0.34597, 0.34754, 0.34838, 0.34854, 0.34811, 0.34720, 0.34590, 0.34431,\
        0.34250, 0.34054, 0.33845, 0.33628, 0.33403, 0.33171, 0.32931, 0.32679, 0.32415, 0.32137, 0.31846,\
        0.31544, 0.31236, 0.30931, 0.30640, 0.30376, 0.30159, 0.30008, 0.29931, 0.29937, 0.29984, 0.30005,\
        0.29989, 0.29953, 0.29912, 0.29874, 0.29847, 0.29840, 0.29865, 0.29930, 0.30050, 0.30235, 0.30502,\
        0.30862, 0.31332, 0.31924, 0.32652, 0.33528, 0.34560, 0.35753, 0.37110, 0.38630, 0.40359, 0.42319,\
        0.44407, 0.46563, 0.48709, 0.50736, 0.52589, 0.54303, 0.55903, 0.57389, 0.58759, 0.59982, 0.60975,\
        0.61829, 0.62548, 0.63150, 0.63649, 0.64051, 0.64362, 0.64588, 0.64738, 0.64820, 0.64841, 0.64809,\
        0.64731, 0.64612, 0.64459, 0.64276, 0.64067, 0.63838, 0.63592, 0.63333, 0.63063, 0.62785, 0.62500,\
        0.62209, 0.61911, 0.61608, 0.61300, 0.60986, 0.60664, 0.60329, 0.59997, 0.59731, 0.59462, 0.59160,\
        0.58835, 0.58490, 0.58126, 0.57744, 0.57343, 0.56924, 0.56487, 0.56033, 0.55562, 0.55074, 0.54570,\
        0.54049, 0.53511, 0.52959, 0.52390, 0.51808, 0.51210, 0.50595, 0.49969, 0.49366, 0.48727, 0.48060],\
        [0.34228, 0.34665, 0.35028, 0.35311, 0.35511, 0.35626, 0.35663, 0.35630, 0.35541, 0.35406, 0.35237,\
        0.35043, 0.34830, 0.34603, 0.34365, 0.34116, 0.33858, 0.33588, 0.33306, 0.33008, 0.32694, 0.32362,\
        0.32013, 0.31652, 0.31285, 0.30921, 0.30575, 0.30269, 0.30035, 0.29906, 0.29921, 0.29998, 0.30031,\
        0.30030, 0.30008, 0.29974, 0.29936, 0.29904, 0.29885, 0.29892, 0.29937, 0.30033, 0.30196, 0.30440,\
        0.30783, 0.31241, 0.31832, 0.32572, 0.33475, 0.34556, 0.35820, 0.37266, 0.38880, 0.40686, 0.42717,\
        0.44869, 0.47070, 0.49252, 0.51311, 0.53203, 0.54938, 0.56524, 0.57962, 0.59267, 0.60449, 0.61453,\
        0.62297, 0.63008, 0.63599, 0.64083, 0.64470, 0.64768, 0.64985, 0.65127, 0.65203, 0.65220, 0.65185,\
        0.65105, 0.64984, 0.64830, 0.64645, 0.64435, 0.64204, 0.63954, 0.63691, 0.63415, 0.63130, 0.62836,\
        0.62534, 0.62226, 0.61911, 0.61590, 0.61264, 0.60931, 0.60596, 0.60283, 0.60028, 0.59739, 0.59424,\
        0.59094, 0.58746, 0.58380, 0.57995, 0.57593, 0.57172, 0.56735, 0.56281, 0.55811, 0.55325, 0.54822,\
        0.54303, 0.53769, 0.53219, 0.52654, 0.52074, 0.51481, 0.50877, 0.50280, 0.49662, 0.49011, 0.48348],\
        [0.34776, 0.35273, 0.35695, 0.36033, 0.36277, 0.36426, 0.36482, 0.36458, 0.36369, 0.36230, 0.36053,\
        0.35850, 0.35625, 0.35384, 0.35129, 0.34862, 0.34583, 0.34291, 0.33985, 0.33662, 0.33317, 0.32951,\
        0.32563, 0.32155, 0.31733, 0.31303, 0.30878, 0.30477, 0.30131, 0.29923, 0.29935, 0.30043, 0.30103,\
        0.30127, 0.30123, 0.30098, 0.30061, 0.30021, 0.29988, 0.29974, 0.29994, 0.30063, 0.30196, 0.30413,\
        0.30732, 0.31171, 0.31752, 0.32494, 0.33417, 0.34537, 0.35870, 0.37421, 0.39174, 0.41073, 0.43121,\
        0.45304, 0.47535, 0.49726, 0.51830, 0.53773, 0.55538, 0.57119, 0.58509, 0.59735, 0.60867, 0.61871,\
        0.62716, 0.63421, 0.64004, 0.64477, 0.64853, 0.65141, 0.65349, 0.65485, 0.65556, 0.65571, 0.65534,\
        0.65453, 0.65333, 0.65178, 0.64994, 0.64783, 0.64551, 0.64300, 0.64034, 0.63754, 0.63462, 0.63161,\
        0.62852, 0.62535, 0.62211, 0.61882, 0.61549, 0.61215, 0.60888, 0.60582, 0.60296, 0.59990, 0.59677,\
        0.59345, 0.58992, 0.58622, 0.58235, 0.57830, 0.57408, 0.56971, 0.56517, 0.56047, 0.55563, 0.55062,\
        0.54546, 0.54015, 0.53469, 0.52908, 0.52333, 0.51745, 0.51147, 0.50537, 0.49905, 0.49277, 0.48625],\
        [0.35309, 0.35867, 0.36352, 0.36749, 0.37044, 0.37225, 0.37299, 0.37282, 0.37194, 0.37051, 0.36870,\
        0.36661, 0.36432, 0.36182, 0.35916, 0.35635, 0.35343, 0.35037, 0.34716, 0.34374, 0.34009, 0.33618,\
        0.33200, 0.32756, 0.32291, 0.31812, 0.31329, 0.30861, 0.30443, 0.30150, 0.30111, 0.30198, 0.30281,\
        0.30323, 0.30329, 0.30308, 0.30265, 0.30213, 0.30160, 0.30122, 0.30111, 0.30146, 0.30245, 0.30426,\
        0.30712, 0.31124, 0.31684, 0.32416, 0.33340, 0.34480, 0.35855, 0.37477, 0.39330, 0.41315, 0.43430,\
        0.45684, 0.47990, 0.50232, 0.52361, 0.54331, 0.56123, 0.57721, 0.59088, 0.60261, 0.61338, 0.62295,\
        0.63117, 0.63808, 0.64378, 0.64840, 0.65205, 0.65483, 0.65684, 0.65815, 0.65884, 0.65897, 0.65860,\
        0.65780, 0.65660, 0.65507, 0.65324, 0.65115, 0.64883, 0.64632, 0.64364, 0.64081, 0.63785, 0.63479,\
        0.63163, 0.62840, 0.62510, 0.62175, 0.61838, 0.61503, 0.61175, 0.60861, 0.60553, 0.60242, 0.59927,\
        0.59583, 0.59225, 0.58852, 0.58461, 0.58054, 0.57631, 0.57192, 0.56739, 0.56271, 0.55787, 0.55289,\
        0.54777, 0.54249, 0.53707, 0.53151, 0.52582, 0.51999, 0.51403, 0.50793, 0.50175, 0.49546, 0.48896],\
        [0.35812, 0.36432, 0.36982, 0.37446, 0.37797, 0.38012, 0.38099, 0.38087, 0.38002, 0.37857, 0.37675,\
        0.37470, 0.37244, 0.36993, 0.36720, 0.36432, 0.36133, 0.35821, 0.35494, 0.35144, 0.34768, 0.34362,\
        0.33926, 0.33460, 0.32969, 0.32460, 0.31947, 0.31453, 0.31020, 0.30716, 0.30590, 0.30591, 0.30630,\
        0.30656, 0.30653, 0.30621, 0.30565, 0.30493, 0.30415, 0.30344, 0.30298, 0.30292, 0.30348, 0.30487,\
        0.30732, 0.31105, 0.31633, 0.32339, 0.33247, 0.34380, 0.35758, 0.37393, 0.39280, 0.41358, 0.43603,\
        0.45979, 0.48400, 0.50734, 0.52888, 0.54878, 0.56684, 0.58307, 0.59724, 0.60826, 0.61823, 0.62725,\
        0.63512, 0.64179, 0.64732, 0.65179, 0.65533, 0.65802, 0.65996, 0.66123, 0.66189, 0.66202, 0.66166,\
        0.66087, 0.65971, 0.65820, 0.65640, 0.65433, 0.65203, 0.64952, 0.64683, 0.64399, 0.64100, 0.63790,\
        0.63470, 0.63141, 0.62805, 0.62465, 0.62124, 0.61784, 0.61451, 0.61125, 0.60803, 0.60481, 0.60149,\
        0.59799, 0.59442, 0.59068, 0.58674, 0.58264, 0.57839, 0.57399, 0.56946, 0.56480, 0.55999, 0.55503,\
        0.54993, 0.54470, 0.53932, 0.53381, 0.52817, 0.52240, 0.51651, 0.51049, 0.50434, 0.49804, 0.49164],\
        [0.36274, 0.36950, 0.37563, 0.38099, 0.38519, 0.38770, 0.38857, 0.38853, 0.38782, 0.38636, 0.38456,\
        0.38271, 0.38059, 0.37813, 0.37538, 0.37246, 0.36948, 0.36640, 0.36316, 0.35968, 0.35591, 0.35180,\
        0.34737, 0.34262, 0.33761, 0.33241, 0.32718, 0.32220, 0.31785, 0.31460, 0.31268, 0.31184, 0.31155,\
        0.31138, 0.31106, 0.31051, 0.30971, 0.30871, 0.30760, 0.30651, 0.30560, 0.30508, 0.30514, 0.30603,\
        0.30797, 0.31124, 0.31608, 0.32276, 0.33152, 0.34260, 0.35615, 0.37230, 0.39115, 0.41290, 0.43700,\
        0.46213, 0.48733, 0.51151, 0.53385, 0.55415, 0.57209, 0.58762, 0.60137, 0.61281, 0.62271, 0.63145,\
        0.63902, 0.64542, 0.65073, 0.65503, 0.65843, 0.66102, 0.66290, 0.66412, 0.66477, 0.66490, 0.66456,\
        0.66381, 0.66268, 0.66121, 0.65944, 0.65741, 0.65513, 0.65264, 0.64995, 0.64710, 0.64410, 0.64096,\
        0.63772, 0.63438, 0.63097, 0.62751, 0.62404, 0.62057, 0.61714, 0.61376, 0.61040, 0.60702, 0.60357,\
        0.60003, 0.59651, 0.59272, 0.58871, 0.58457, 0.58030, 0.57591, 0.57139, 0.56674, 0.56195, 0.55702,\
        0.55195, 0.54675, 0.54141, 0.53594, 0.53034, 0.52464, 0.51882, 0.51291, 0.50688, 0.50072, 0.49433],\
        [0.36687, 0.37407, 0.38070, 0.38671, 0.39179, 0.39479, 0.39540, 0.39554, 0.39532, 0.39370, 0.39202,\
        0.39069, 0.38879, 0.38642, 0.38365, 0.38071, 0.37784, 0.37490, 0.37179, 0.36841, 0.36470, 0.36063,\
        0.35623, 0.35150, 0.34651, 0.34132, 0.33610, 0.33111, 0.32669, 0.32318, 0.32071, 0.31918, 0.31824,\
        0.31757, 0.31687, 0.31599, 0.31487, 0.31352, 0.31201, 0.31047, 0.30906, 0.30800, 0.30750, 0.30781,\
        0.30919, 0.31190, 0.31623, 0.32244, 0.33082, 0.34159, 0.35495, 0.37101, 0.38999, 0.41280, 0.43858,\
        0.46499, 0.49049, 0.51560, 0.53912, 0.55997, 0.57785, 0.59236, 0.60450, 0.61671, 0.62695, 0.63558,\
        0.64288, 0.64900, 0.65405, 0.65815, 0.66140, 0.66389, 0.66569, 0.66688, 0.66752, 0.66767, 0.66736,\
        0.66664, 0.66555, 0.66413, 0.66241, 0.66042, 0.65817, 0.65570, 0.65303, 0.65018, 0.64716, 0.64399,\
        0.64071, 0.63731, 0.63384, 0.63032, 0.62677, 0.62321, 0.61966, 0.61614, 0.61264, 0.60913, 0.60561,\
        0.60212, 0.59850, 0.59454, 0.59048, 0.58631, 0.58204, 0.57766, 0.57316, 0.56853, 0.56376, 0.55886,\
        0.55382, 0.54863, 0.54332, 0.53787, 0.53231, 0.52664, 0.52088, 0.51504, 0.50915, 0.50320, 0.49666],\
        [0.37051, 0.37802, 0.38486, 0.39112, 0.39684, 0.40077, 0.40132, 0.40191, 0.40218, 0.40051, 0.39923,\
        0.39884, 0.39696, 0.39481, 0.39196, 0.38897, 0.38642, 0.38368, 0.38078, 0.37756, 0.37395, 0.36998,\
        0.36566, 0.36105, 0.35617, 0.35105, 0.34585, 0.34082, 0.33626, 0.33246, 0.32954, 0.32747, 0.32602,\
        0.32489, 0.32382, 0.32260, 0.32112, 0.31937, 0.31741, 0.31535, 0.31339, 0.31174, 0.31062, 0.31031,\
        0.31107, 0.31318, 0.31693, 0.32264, 0.33060, 0.34113, 0.35451, 0.37095, 0.39072, 0.41467, 0.44178,\
        0.46979, 0.49680, 0.52195, 0.54558, 0.56663, 0.58459, 0.59962, 0.61124, 0.62200, 0.63163, 0.63985,\
        0.64677, 0.65255, 0.65732, 0.66119, 0.66427, 0.66665, 0.66839, 0.66955, 0.67020, 0.67036, 0.67009,\
        0.66942, 0.66839, 0.66702, 0.66535, 0.66340, 0.66120, 0.65876, 0.65610, 0.65324, 0.65021, 0.64701,\
        0.64368, 0.64023, 0.63669, 0.63308, 0.62943, 0.62576, 0.62208, 0.61841, 0.61475, 0.61110, 0.60746,\
        0.60379, 0.59999, 0.59609, 0.59201, 0.58785, 0.58360, 0.57925, 0.57478, 0.57017, 0.56543, 0.56054,\
        0.55551, 0.55034, 0.54504, 0.53960, 0.53404, 0.52836, 0.52258, 0.51671, 0.51072, 0.50450, 0.49778],\
        [0.37379, 0.38154, 0.38837, 0.39438, 0.39977, 0.40434, 0.40676, 0.40810, 0.40857, 0.40797, 0.40727,\
        0.40671, 0.40523, 0.40327, 0.40014, 0.39732, 0.39544, 0.39263, 0.39012, 0.38702, 0.38350, 0.37967,\
        0.37546, 0.37103, 0.36633, 0.36131, 0.35610, 0.35097, 0.34619, 0.34208, 0.33880, 0.33635, 0.33454,\
        0.33310, 0.33174, 0.33022, 0.32840, 0.32623, 0.32377, 0.32117, 0.31861, 0.31631, 0.31455, 0.31358,\
        0.31369, 0.31518, 0.31834, 0.32351, 0.33104, 0.34133, 0.35485, 0.37205, 0.39311, 0.41804, 0.44606,\
        0.47533, 0.50372, 0.52926, 0.55271, 0.57383, 0.59172, 0.60600, 0.61785, 0.62788, 0.63667, 0.64426,\
        0.65070, 0.65609, 0.66054, 0.66417, 0.66708, 0.66935, 0.67104, 0.67219, 0.67285, 0.67305, 0.67283,\
        0.67221, 0.67123, 0.66992, 0.66831, 0.66641, 0.66424, 0.66183, 0.65919, 0.65633, 0.65328, 0.65005,\
        0.64667, 0.64315, 0.63953, 0.63582, 0.63205, 0.62824, 0.62441, 0.62058, 0.61675, 0.61292, 0.60909,\
        0.60525, 0.60139, 0.59738, 0.59331, 0.58920, 0.58501, 0.58070, 0.57626, 0.57167, 0.56694, 0.56207,\
        0.55705, 0.55188, 0.54656, 0.54110, 0.53551, 0.52979, 0.52393, 0.51794, 0.51174, 0.50525, 0.49845],\
        [0.37687, 0.38501, 0.39196, 0.39764, 0.40332, 0.40839, 0.41206, 0.41437, 0.41557, 0.41586, 0.41567,\
        0.41508, 0.41382, 0.41182, 0.40906, 0.40667, 0.40463, 0.40171, 0.39994, 0.39649, 0.39322, 0.38953,\
        0.38530, 0.38119, 0.37674, 0.37178, 0.36653, 0.36120, 0.35613, 0.35170, 0.34814, 0.34548, 0.34352,\
        0.34197, 0.34047, 0.33875, 0.33662, 0.33403, 0.33106, 0.32787, 0.32468, 0.32173, 0.31930, 0.31768,\
        0.31715, 0.31802, 0.32060, 0.32522, 0.33228, 0.34223, 0.35564, 0.37323, 0.39565, 0.42137, 0.45030,\
        0.48049, 0.50931, 0.53552, 0.55922, 0.58051, 0.59878, 0.61266, 0.62387, 0.63340, 0.64159, 0.64862,\
        0.65458, 0.65957, 0.66371, 0.66711, 0.66986, 0.67204, 0.67369, 0.67485, 0.67555, 0.67580, 0.67562,\
        0.67506, 0.67414, 0.67289, 0.67134, 0.66949, 0.66736, 0.66497, 0.66234, 0.65948, 0.65640, 0.65313,\
        0.64968, 0.64609, 0.64237, 0.63854, 0.63463, 0.63067, 0.62667, 0.62266, 0.61864, 0.61462, 0.61059,\
        0.60655, 0.60244, 0.59840, 0.59447, 0.59045, 0.58631, 0.58203, 0.57761, 0.57304, 0.56832, 0.56344,\
        0.55841, 0.55323, 0.54789, 0.54239, 0.53675, 0.53095, 0.52498, 0.51884, 0.51251, 0.50597, 0.49919],\
        [0.37980, 0.38852, 0.39638, 0.40212, 0.40802, 0.41360, 0.41803, 0.42116, 0.42312, 0.42411, 0.42436,\
        0.42398, 0.42291, 0.42116, 0.41892, 0.41664, 0.41432, 0.41181, 0.40933, 0.40618, 0.40307, 0.39922,\
        0.39476, 0.39137, 0.38710, 0.38206, 0.37681, 0.37121, 0.36573, 0.36097, 0.35725, 0.35457, 0.35271,\
        0.35127, 0.34984, 0.34804, 0.34567, 0.34268, 0.33919, 0.33540, 0.33156, 0.32795, 0.32487, 0.32262,\
        0.32150, 0.32180, 0.32384, 0.32795, 0.33453, 0.34404, 0.35699, 0.37396, 0.39575, 0.42335, 0.45422,\
        0.48565, 0.51489, 0.54108, 0.56469, 0.58559, 0.60344, 0.61760, 0.62891, 0.63827, 0.64614, 0.65276,\
        0.65833, 0.66297, 0.66682, 0.67001, 0.67264, 0.67476, 0.67641, 0.67761, 0.67836, 0.67867, 0.67856,\
        0.67805, 0.67718, 0.67599, 0.67449, 0.67269, 0.67060, 0.66823, 0.66560, 0.66272, 0.65961, 0.65628,\
        0.65276, 0.64908, 0.64524, 0.64127, 0.63721, 0.63308, 0.62889, 0.62469, 0.62047, 0.61626, 0.61205,\
        0.60785, 0.60368, 0.59962, 0.59566, 0.59166, 0.58753, 0.58326, 0.57885, 0.57428, 0.56956, 0.56467,\
        0.55962, 0.55441, 0.54903, 0.54348, 0.53776, 0.53187, 0.52579, 0.51951, 0.51303, 0.50640, 0.49971],\
        [0.38259, 0.39184, 0.39983, 0.40735, 0.41392, 0.41990, 0.42488, 0.42864, 0.43121, 0.43275, 0.43342,\
        0.43333, 0.43252, 0.43108, 0.42916, 0.42699, 0.42463, 0.42209, 0.41932, 0.41618, 0.41270, 0.40857,\
        0.40442, 0.40137, 0.39655, 0.39170, 0.38671, 0.38062, 0.37459, 0.36955, 0.36583, 0.36338, 0.36188,\
        0.36085, 0.35971, 0.35798, 0.35542, 0.35204, 0.34802, 0.34363, 0.33915, 0.33490, 0.33121, 0.32839,\
        0.32675, 0.32659, 0.32820, 0.33190, 0.33808, 0.34724, 0.35991, 0.37662, 0.39838, 0.42725, 0.45932,\
        0.49037, 0.51978, 0.54631, 0.56959, 0.58981, 0.60709, 0.62150, 0.63310, 0.64253, 0.65027, 0.65666,\
        0.66194, 0.66630, 0.66992, 0.67294, 0.67547, 0.67757, 0.67927, 0.68055, 0.68138, 0.68176, 0.68170,\
        0.68124, 0.68042, 0.67928, 0.67782, 0.67606, 0.67400, 0.67164, 0.66900, 0.66609, 0.66294, 0.65954,\
        0.65594, 0.65214, 0.64817, 0.64405, 0.63981, 0.63549, 0.63110, 0.62668, 0.62226, 0.61786, 0.61349,\
        0.60918, 0.60497, 0.60086, 0.59693, 0.59285, 0.58867, 0.58440, 0.57998, 0.57541, 0.57067, 0.56576,\
        0.56068, 0.55542, 0.54999, 0.54438, 0.53859, 0.53261, 0.52643, 0.52001, 0.51336, 0.50644, 0.49935],\
        [0.38524, 0.39566, 0.40456, 0.41280, 0.42041, 0.42705, 0.43255, 0.43679, 0.43982, 0.44178, 0.44281,\
        0.44302, 0.44249, 0.44132, 0.43963, 0.43756, 0.43520, 0.43256, 0.42961, 0.42629, 0.42255, 0.41840,\
        0.41422, 0.41004, 0.40493, 0.40093, 0.39586, 0.38885, 0.38227, 0.37720, 0.37372, 0.37172, 0.37086,\
        0.37053, 0.36995, 0.36842, 0.36570, 0.36192, 0.35737, 0.35239, 0.34730, 0.34246, 0.33823, 0.33493,\
        0.33290, 0.33242, 0.33377, 0.33722, 0.34314, 0.35211, 0.36496, 0.38260, 0.40567, 0.43469, 0.46704,\
        0.49894, 0.52677, 0.55208, 0.57462, 0.59430, 0.61113, 0.62528, 0.63694, 0.64644, 0.65416, 0.66042,\
        0.66549, 0.66963, 0.67305, 0.67594, 0.67842, 0.68057, 0.68237, 0.68377, 0.68472, 0.68518, 0.68516,\
        0.68473, 0.68393, 0.68282, 0.68141, 0.67968, 0.67763, 0.67526, 0.67259, 0.66964, 0.66642, 0.66295,\
        0.65924, 0.65531, 0.65119, 0.64690, 0.64246, 0.63793, 0.63332, 0.62867, 0.62403, 0.61943, 0.61489,\
        0.61047, 0.60620, 0.60214, 0.59806, 0.59389, 0.58971, 0.58544, 0.58102, 0.57643, 0.57166, 0.56672,\
        0.56159, 0.55628, 0.55078, 0.54511, 0.53925, 0.53319, 0.52692, 0.52043, 0.51369, 0.50671, 0.49962],\
        [0.38764, 0.39840, 0.40915, 0.41886, 0.42751, 0.43490, 0.44091, 0.44553, 0.44888, 0.45112, 0.45244,\
        0.45293, 0.45266, 0.45172, 0.45019, 0.44819, 0.44582, 0.44311, 0.44003, 0.43653, 0.43260, 0.42828,\
        0.42369, 0.41882, 0.41368, 0.40866, 0.40268, 0.39503, 0.38871, 0.38403, 0.38097, 0.37957, 0.37952,\
        0.38015, 0.38041, 0.37914, 0.37623, 0.37204, 0.36699, 0.36147, 0.35583, 0.35048, 0.34579, 0.34215,\
        0.33988, 0.33927, 0.34057, 0.34399, 0.34981, 0.35858, 0.37142, 0.39005, 0.41446, 0.44354, 0.47527,\
        0.50629, 0.53380, 0.55819, 0.57985, 0.59887, 0.61525, 0.62914, 0.64075, 0.65029, 0.65802, 0.66421,\
        0.66913, 0.67308, 0.67634, 0.67913, 0.68161, 0.68385, 0.68582, 0.68741, 0.68850, 0.68904, 0.68904,\
        0.68859, 0.68778, 0.68669, 0.68531, 0.68360, 0.68153, 0.67912, 0.67640, 0.67339, 0.67010, 0.66652,\
        0.66269, 0.65862, 0.65433, 0.64984, 0.64519, 0.64042, 0.63556, 0.63066, 0.62578, 0.62096, 0.61624,\
        0.61166, 0.60723, 0.60295, 0.59879, 0.59476, 0.59065, 0.58639, 0.58196, 0.57734, 0.57253, 0.56754,\
        0.56235, 0.55698, 0.55141, 0.54566, 0.53973, 0.53361, 0.52730, 0.52077, 0.51403, 0.50708, 0.49996],\
        [0.39064, 0.40263, 0.41445, 0.42550, 0.43525, 0.44341, 0.44987, 0.45474, 0.45824, 0.46064, 0.46216,\
        0.46289, 0.46286, 0.46209, 0.46067, 0.45872, 0.45635, 0.45361, 0.45046, 0.44684, 0.44274, 0.43816,\
        0.43314, 0.42771, 0.42188, 0.41557, 0.40829, 0.40070, 0.39508, 0.39070, 0.38792, 0.38708, 0.38778,\
        0.38937, 0.39084, 0.38969, 0.38658, 0.38202, 0.37658, 0.37063, 0.36454, 0.35876, 0.35376, 0.34991,\
        0.34756, 0.34702, 0.34855, 0.35226, 0.35819, 0.36669, 0.37894, 0.39670, 0.42285, 0.45301, 0.48381,\
        0.51348, 0.54055, 0.56439, 0.58522, 0.60349, 0.61945, 0.63313, 0.64470, 0.65430, 0.66209, 0.66825,\
        0.67305, 0.67683, 0.67993, 0.68265, 0.68517, 0.68756, 0.68976, 0.69162, 0.69287, 0.69346, 0.69346,\
        0.69293, 0.69206, 0.69096, 0.68961, 0.68789, 0.68574, 0.68325, 0.68047, 0.67739, 0.67399, 0.67030,\
        0.66633, 0.66210, 0.65762, 0.65292, 0.64803, 0.64298, 0.63785, 0.63267, 0.62752, 0.62245, 0.61751,\
        0.61275, 0.60818, 0.60378, 0.59957, 0.59565, 0.59156, 0.58728, 0.58281, 0.57815, 0.57329, 0.56824,\
        0.56298, 0.55752, 0.55187, 0.54603, 0.54002, 0.53384, 0.52750, 0.52098, 0.51426, 0.50734, 0.50019],\
        [0.39393, 0.40748, 0.42038, 0.43275, 0.44367, 0.45258, 0.45938, 0.46429, 0.46773, 0.47013, 0.47178,\
        0.47273, 0.47292, 0.47229, 0.47090, 0.46896, 0.46663, 0.46393, 0.46079, 0.45712, 0.45289, 0.44807,\
        0.44268, 0.43675, 0.43028, 0.42324, 0.41574, 0.40854, 0.40275, 0.39781, 0.39527, 0.39476, 0.39601,\
        0.39757, 0.40040, 0.39898, 0.39625, 0.39142, 0.38583, 0.37964, 0.37319, 0.36711, 0.36197, 0.35809,\
        0.35580, 0.35551, 0.35752, 0.36191, 0.36838, 0.37662, 0.38862, 0.40738, 0.43376, 0.46444, 0.49494,\
        0.52252, 0.54804, 0.57100, 0.59110, 0.60848, 0.62386, 0.63727, 0.64884, 0.65861, 0.66658, 0.67279,\
        0.67750, 0.68111, 0.68405, 0.68671, 0.68933, 0.69190, 0.69436, 0.69655, 0.69798, 0.69855, 0.69856,\
        0.69786, 0.69688, 0.69573, 0.69439, 0.69263, 0.69026, 0.68765, 0.68484, 0.68167, 0.67814, 0.67430,\
        0.67018, 0.66577, 0.66109, 0.65615, 0.65099, 0.64565, 0.64021, 0.63472, 0.62925, 0.62390, 0.61872,\
        0.61377, 0.60909, 0.60470, 0.60069, 0.59661, 0.59242, 0.58808, 0.58355, 0.57884, 0.57393, 0.56881,\
        0.56347, 0.55792, 0.55216, 0.54622, 0.54010, 0.53385, 0.52746, 0.52094, 0.51427, 0.50737, 0.50020],\
        [0.39445, 0.41031, 0.42588, 0.44039, 0.45277, 0.46243, 0.46938, 0.47404, 0.47714, 0.47938, 0.48108,\
        0.48223, 0.48267, 0.48215, 0.48068, 0.47870, 0.47647, 0.47392, 0.47090, 0.46729, 0.46302, 0.45805,\
        0.45240, 0.44613, 0.43930, 0.43204, 0.42463, 0.41763, 0.41152, 0.40665, 0.40420, 0.40367, 0.40499,\
        0.40639, 0.40788, 0.40742, 0.40468, 0.40008, 0.39459, 0.38829, 0.38145, 0.37529, 0.37033, 0.36663,\
        0.36451, 0.36453, 0.36717, 0.37255, 0.38022, 0.38819, 0.39787, 0.41896, 0.44700, 0.47720, 0.50627,\
        0.53212, 0.55568, 0.57747, 0.59722, 0.61367, 0.62823, 0.64139, 0.65314, 0.66333, 0.67171, 0.67811,\
        0.68275, 0.68619, 0.68902, 0.69157, 0.69444, 0.69721, 0.69988, 0.70227, 0.70385, 0.70446, 0.70438,\
        0.70362, 0.70250, 0.70117, 0.69971, 0.69785, 0.69495, 0.69235, 0.68957, 0.68627, 0.68256, 0.67854,\
        0.67425, 0.66968, 0.66478, 0.65958, 0.65411, 0.64845, 0.64265, 0.63681, 0.63100, 0.62531, 0.61983,\
        0.61466, 0.60986, 0.60543, 0.60121, 0.59728, 0.59315, 0.58874, 0.58416, 0.57940, 0.57445, 0.56927,\
        0.56385, 0.55818, 0.55229, 0.54619, 0.53994, 0.53356, 0.52710, 0.52057, 0.51392, 0.50707, 0.49988],\
        [0.39407, 0.41242, 0.43144, 0.44869, 0.46269, 0.47295, 0.47982, 0.48385, 0.48622, 0.48816, 0.48985,\
        0.49117, 0.49190, 0.49146, 0.48969, 0.48765, 0.48571, 0.48346, 0.48068, 0.47724, 0.47305, 0.46806,\
        0.46230, 0.45588, 0.44896, 0.44173, 0.43453, 0.42779, 0.42196, 0.41758, 0.41512, 0.41453, 0.41529,\
        0.41635, 0.41691, 0.41597, 0.41300, 0.40832, 0.40266, 0.39622, 0.38871, 0.38313, 0.37895, 0.37564,\
        0.37369, 0.37398, 0.37724, 0.38366, 0.39223, 0.40168, 0.40847, 0.43292, 0.46238, 0.49180, 0.51734,\
        0.54075, 0.56240, 0.58264, 0.60116, 0.61746, 0.63194, 0.64524, 0.65749, 0.66847, 0.67767, 0.68450,\
        0.68913, 0.69233, 0.69542, 0.69748, 0.70095, 0.70406, 0.70686, 0.70917, 0.71069, 0.71130, 0.71113,\
        0.71030, 0.70902, 0.70742, 0.70555, 0.70302, 0.69966, 0.69764, 0.69476, 0.69120, 0.68722, 0.68301,\
        0.67859, 0.67384, 0.66871, 0.66321, 0.65741, 0.65138, 0.64521, 0.63897, 0.63277, 0.62669, 0.62085,\
        0.61538, 0.61039, 0.60595, 0.60201, 0.59806, 0.59368, 0.58917, 0.58457, 0.57983, 0.57487, 0.56964,\
        0.56412, 0.55832, 0.55225, 0.54596, 0.53950, 0.53295, 0.52635, 0.51973, 0.51309, 0.50630, 0.49911],\
        [0.39541, 0.41616, 0.43811, 0.45803, 0.47358, 0.48392, 0.49050, 0.49356, 0.49474, 0.49644, 0.49801,\
        0.49930, 0.50011, 0.49979, 0.49754, 0.49561, 0.49427, 0.49241, 0.48993, 0.48682, 0.48286, 0.47798,\
        0.47226, 0.46589, 0.45907, 0.45205, 0.44515, 0.43878, 0.43339, 0.42942, 0.42714, 0.42645, 0.42681,\
        0.42737, 0.42724, 0.42559, 0.42198, 0.41650, 0.40955, 0.40127, 0.39437, 0.39112, 0.38829, 0.38535,\
        0.38349, 0.38407, 0.38789, 0.39575, 0.40574, 0.41657, 0.43047, 0.45260, 0.47935, 0.50591, 0.52819,\
        0.54849, 0.56799, 0.58670, 0.60430, 0.62041, 0.63511, 0.64880, 0.66178, 0.67395, 0.68457, 0.69220,\
        0.69703, 0.70004, 0.70346, 0.70620, 0.70942, 0.71275, 0.71548, 0.71753, 0.71879, 0.71921, 0.71886,\
        0.71786, 0.71634, 0.71440, 0.71201, 0.70910, 0.70604, 0.70347, 0.70029, 0.69638, 0.69207, 0.68771,\
        0.68322, 0.67831, 0.67290, 0.66706, 0.66089, 0.65448, 0.64791, 0.64125, 0.63461, 0.62808, 0.62180,\
        0.61590, 0.61055, 0.60590, 0.60197, 0.59805, 0.59366, 0.58926, 0.58479, 0.58015, 0.57523, 0.56997,\
        0.56434, 0.55836, 0.55207, 0.54553, 0.53880, 0.53197, 0.52513, 0.51835, 0.51163, 0.50482, 0.49762],\
        [0.39884, 0.42129, 0.44535, 0.46800, 0.48540, 0.49476, 0.50042, 0.50298, 0.50324, 0.50449, 0.50566,\
        0.50656, 0.50698, 0.50633, 0.50450, 0.50296, 0.50205, 0.50050, 0.49843, 0.49580, 0.49222, 0.48758,\
        0.48208, 0.47593, 0.46939, 0.46272, 0.45619, 0.45022, 0.44524, 0.44164, 0.43959, 0.43894, 0.43917,\
        0.43944, 0.43886, 0.43662, 0.43227, 0.42579, 0.41748, 0.40831, 0.40234, 0.40059, 0.39851, 0.39580,\
        0.39400, 0.39549, 0.40023, 0.40972, 0.42258, 0.43733, 0.45467, 0.47468, 0.49632, 0.51762, 0.53691,\
        0.55487, 0.57265, 0.59040, 0.60750, 0.62357, 0.63842, 0.65239, 0.66599, 0.67939, 0.69185, 0.70135,\
        0.70642, 0.71058, 0.71415, 0.71744, 0.72054, 0.72343, 0.72575, 0.72736, 0.72818, 0.72824, 0.72757,\
        0.72627, 0.72444, 0.72213, 0.71938, 0.71626, 0.71302, 0.70974, 0.70605, 0.70177, 0.69708, 0.69273,\
        0.68827, 0.68313, 0.67735, 0.67111, 0.66455, 0.65775, 0.65077, 0.64369, 0.63658, 0.62957, 0.62276,\
        0.61632, 0.61042, 0.60520, 0.60076, 0.59691, 0.59304, 0.58906, 0.58491, 0.58047, 0.57563, 0.57032,\
        0.56456, 0.55836, 0.55179, 0.54493, 0.53786, 0.53067, 0.52347, 0.51638, 0.50949, 0.50257, 0.49530],\
        [0.39980, 0.42446, 0.45101, 0.47637, 0.49656, 0.50559, 0.50970, 0.51152, 0.51177, 0.51206, 0.51248,\
        0.51278, 0.51271, 0.51201, 0.51079, 0.50965, 0.50873, 0.50755, 0.50593, 0.50378, 0.50073, 0.49654,\
        0.49150, 0.48576, 0.47969, 0.47352, 0.46741, 0.46179, 0.45715, 0.45387, 0.45210, 0.45168, 0.45207,\
        0.45242, 0.45172, 0.44915, 0.44429, 0.43720, 0.42860, 0.42009, 0.41405, 0.41089, 0.40860, 0.40647,\
        0.40562, 0.40856, 0.41519, 0.42622, 0.44107, 0.45871, 0.47822, 0.49655, 0.51279, 0.52921, 0.54488,\
        0.56033, 0.57669, 0.59432, 0.61166, 0.62793, 0.64287, 0.65673, 0.67042, 0.68461, 0.69867, 0.70975,\
        0.71737, 0.72284, 0.72709, 0.73057, 0.73349, 0.73585, 0.73755, 0.73851, 0.73874, 0.73827, 0.73716,\
        0.73547, 0.73325, 0.73056, 0.72747, 0.72405, 0.72041, 0.71654, 0.71232, 0.70763, 0.70276, 0.69832,\
        0.69385, 0.68821, 0.68194, 0.67529, 0.66835, 0.66118, 0.65383, 0.64634, 0.63879, 0.63128, 0.62393,\
        0.61689, 0.61036, 0.60457, 0.59970, 0.59587, 0.59241, 0.58893, 0.58517, 0.58096, 0.57618, 0.57079,\
        0.56482, 0.55834, 0.55144, 0.54420, 0.53672, 0.52911, 0.52144, 0.51387, 0.50666, 0.49988, 0.49247],\
        [0.39909, 0.42544, 0.45372, 0.48043, 0.50132, 0.51222, 0.51665, 0.51814, 0.51825, 0.51803, 0.51785,\
        0.51768, 0.51735, 0.51673, 0.51591, 0.51511, 0.51437, 0.51349, 0.51227, 0.51054, 0.50802, 0.50457,\
        0.50025, 0.49513, 0.48978, 0.48434, 0.47864, 0.47328, 0.46892, 0.46591, 0.46444, 0.46439, 0.46524,\
        0.46603, 0.46560, 0.46299, 0.45783, 0.45036, 0.44161, 0.43311, 0.42639, 0.42189, 0.41901, 0.41753,\
        0.41823, 0.42240, 0.43056, 0.44286, 0.45889, 0.47762, 0.49740, 0.51443, 0.52787, 0.54059, 0.55300,\
        0.56582, 0.58057, 0.59834, 0.61721, 0.63474, 0.64989, 0.66310, 0.67607, 0.69060, 0.70580, 0.71963,\
        0.72976, 0.73673, 0.74161, 0.74520, 0.74784, 0.74962, 0.75059, 0.75077, 0.75026, 0.74916, 0.74751,\
        0.74535, 0.74269, 0.73958, 0.73609, 0.73227, 0.72819, 0.72384, 0.71917, 0.71423, 0.70925, 0.70453,\
        0.69942, 0.69316, 0.68649, 0.67949, 0.67226, 0.66479, 0.65710, 0.64925, 0.64131, 0.63336, 0.62551,\
        0.61790, 0.61073, 0.60439, 0.59937, 0.59542, 0.59231, 0.58928, 0.58587, 0.58181, 0.57701, 0.57144,\
        0.56518, 0.55834, 0.55103, 0.54337, 0.53547, 0.52742, 0.51930, 0.51118, 0.50319, 0.49617, 0.48915],\
        [0.39917, 0.42546, 0.45385, 0.48047, 0.50134, 0.51382, 0.51953, 0.52150, 0.52180, 0.52155, 0.52123,\
        0.52094, 0.52060, 0.52018, 0.51971, 0.51926, 0.51883, 0.51830, 0.51751, 0.51626, 0.51436, 0.51168,\
        0.50821, 0.50399, 0.49971, 0.49518, 0.48974, 0.48464, 0.48059, 0.47777, 0.47653, 0.47688, 0.47834,\
        0.47988, 0.47998, 0.47750, 0.47212, 0.46416, 0.45490, 0.44588, 0.43836, 0.43294, 0.42968, 0.42871,\
        0.43054, 0.43585, 0.44507, 0.45821, 0.47469, 0.49319, 0.51160, 0.52762, 0.54070, 0.55200, 0.56242,\
        0.57304, 0.58611, 0.60469, 0.62654, 0.64605, 0.66141, 0.67357, 0.68454, 0.69976, 0.71742, 0.73324,\
        0.74498, 0.75259, 0.75752, 0.76101, 0.76329, 0.76440, 0.76455, 0.76385, 0.76248, 0.76067, 0.75845,\
        0.75577, 0.75261, 0.74902, 0.74508, 0.74083, 0.73630, 0.73149, 0.72642, 0.72114, 0.71577, 0.71031,\
        0.70439, 0.69794, 0.69088, 0.68367, 0.67625, 0.66855, 0.66061, 0.65247, 0.64422, 0.63594, 0.62772,\
        0.61967, 0.61198, 0.60489, 0.59918, 0.59584, 0.59319, 0.59048, 0.58725, 0.58320, 0.57821, 0.57231,\
        0.56563, 0.55832, 0.55055, 0.54245, 0.53412, 0.52568, 0.51722, 0.50889, 0.50072, 0.49317, 0.48586],\
        [0.39913, 0.42474, 0.45228, 0.47816, 0.49900, 0.51212, 0.51856, 0.52118, 0.52207, 0.52230, 0.52234,\
        0.52232, 0.52227, 0.52218, 0.52211, 0.52209, 0.52208, 0.52201, 0.52174, 0.52110, 0.51994, 0.51814,\
        0.51568, 0.51266, 0.50935, 0.50547, 0.50061, 0.49604, 0.49253, 0.48969, 0.48856, 0.48915, 0.49104,\
        0.49336, 0.49401, 0.49152, 0.48605, 0.47725, 0.46720, 0.45748, 0.44925, 0.44325, 0.43986, 0.43936,\
        0.44207, 0.44830, 0.45826, 0.47194, 0.48863, 0.50672, 0.52429, 0.53988, 0.55319, 0.56468, 0.57476,\
        0.58462, 0.59724, 0.61782, 0.64247, 0.66378, 0.67865, 0.69047, 0.70038, 0.71484, 0.73476, 0.75190,\
        0.76383, 0.77053, 0.77446, 0.77773, 0.77956, 0.77980, 0.77916, 0.77745, 0.77511, 0.77257, 0.76981,\
        0.76658, 0.76284, 0.75872, 0.75428, 0.74957, 0.74459, 0.73934, 0.73382, 0.72807, 0.72213, 0.71595,\
        0.70939, 0.70234, 0.69508, 0.68788, 0.68036, 0.67248, 0.66431, 0.65596, 0.64751, 0.63905, 0.63065,\
        0.62245, 0.61462, 0.60750, 0.60182, 0.59810, 0.59544, 0.59273, 0.58943, 0.58517, 0.57978, 0.57335,\
        0.56608, 0.55820, 0.54991, 0.54135, 0.53263, 0.52385, 0.51511, 0.50667, 0.49877, 0.49062, 0.48280],\
        [0.39964, 0.42383, 0.44958, 0.47394, 0.49424, 0.50737, 0.51395, 0.51729, 0.51917, 0.52037, 0.52122,\
        0.52183, 0.52229, 0.52269, 0.52312, 0.52361, 0.52414, 0.52465, 0.52503, 0.52514, 0.52485, 0.52406,\
        0.52273, 0.52088, 0.51854, 0.51554, 0.51189, 0.50828, 0.50523, 0.50237, 0.50133, 0.50182, 0.50337,\
        0.50553, 0.50588, 0.50333, 0.49776, 0.48807, 0.47748, 0.46717, 0.45843, 0.45216, 0.44891, 0.44899,\
        0.45257, 0.45969, 0.47022, 0.48432, 0.50160, 0.51984, 0.53725, 0.55278, 0.56666, 0.57949, 0.59095,\
        0.60163, 0.61638, 0.63791, 0.66407, 0.68769, 0.70065, 0.71231, 0.72443, 0.73984, 0.75862, 0.77502,\
        0.78587, 0.79000, 0.79178, 0.79490, 0.79616, 0.79535, 0.79417, 0.79128, 0.78786, 0.78466, 0.78150,\
        0.77761, 0.77321, 0.76849, 0.76354, 0.75835, 0.75292, 0.74722, 0.74121, 0.73493, 0.72838, 0.72156,\
        0.71441, 0.70697, 0.69951, 0.69233, 0.68459, 0.67648, 0.66814, 0.65966, 0.65112, 0.64261, 0.63424,\
        0.62614, 0.61852, 0.61172, 0.60617, 0.60208, 0.59900, 0.59590, 0.59231, 0.58765, 0.58159, 0.57438,\
        0.56636, 0.55782, 0.54897, 0.53997, 0.53092, 0.52186, 0.51283, 0.50393, 0.49563, 0.48766, 0.47965],\
        [0.40137, 0.42305, 0.44585, 0.46767, 0.48615, 0.49890, 0.50591, 0.51039, 0.51367, 0.51623, 0.51820,\
        0.51965, 0.52077, 0.52177, 0.52279, 0.52389, 0.52507, 0.52628, 0.52743, 0.52842, 0.52912, 0.52944,\
        0.52932, 0.52871, 0.52758, 0.52585, 0.52362, 0.52119, 0.51880, 0.51666, 0.51551, 0.51537, 0.51586,\
        0.51629, 0.51525, 0.51180, 0.50522, 0.49607, 0.48518, 0.47444, 0.46542, 0.45920, 0.45641, 0.45734,\
        0.46203, 0.47035, 0.48177, 0.49584, 0.51440, 0.53374, 0.55153, 0.56700, 0.58124, 0.59577, 0.60948,\
        0.62211, 0.63795, 0.65949, 0.68560, 0.70982, 0.72460, 0.73639, 0.74973, 0.76595, 0.78442, 0.79920,\
        0.80771, 0.81009, 0.81043, 0.81205, 0.81250, 0.81115, 0.80893, 0.80516, 0.80068, 0.79693, 0.79348,\
        0.78860, 0.78347, 0.77815, 0.77267, 0.76703, 0.76117, 0.75500, 0.74849, 0.74164, 0.73450, 0.72710,\
        0.71950, 0.71183, 0.70438, 0.69679, 0.68866, 0.68039, 0.67193, 0.66340, 0.65488, 0.64646, 0.63825,\
        0.63039, 0.62308, 0.61658, 0.61112, 0.60676, 0.60312, 0.59941, 0.59572, 0.59040, 0.58329, 0.57507,\
        0.56619, 0.55694, 0.54754, 0.53813, 0.52881, 0.51963, 0.51054, 0.50149, 0.49285, 0.48455, 0.47632],\
        [0.40308, 0.42111, 0.44081, 0.45989, 0.47623, 0.48826, 0.49610, 0.50188, 0.50659, 0.51064, 0.51385,\
        0.51611, 0.51789, 0.51954, 0.52123, 0.52302, 0.52494, 0.52695, 0.52899, 0.53097, 0.53276, 0.53428,\
        0.53543, 0.53617, 0.53644, 0.53620, 0.53547, 0.53436, 0.53298, 0.53148, 0.53014, 0.52901, 0.52785,\
        0.52611, 0.52304, 0.51801, 0.51071, 0.50120, 0.49006, 0.47905, 0.46996, 0.46412, 0.46219, 0.46434,\
        0.47042, 0.48033, 0.49404, 0.50953, 0.52862, 0.54917, 0.56781, 0.58316, 0.59718, 0.61336, 0.62932,\
        0.64398, 0.66011, 0.68082, 0.70576, 0.72946, 0.74659, 0.75972, 0.77370, 0.78980, 0.80631, 0.81982,\
        0.82692, 0.82940, 0.82970, 0.82969, 0.82889, 0.82683, 0.82357, 0.81914, 0.81413, 0.80959, 0.80490,\
        0.79915, 0.79346, 0.78752, 0.78151, 0.77546, 0.76920, 0.76257, 0.75553, 0.74809, 0.74035, 0.73239,\
        0.72433, 0.71628, 0.70832, 0.70022, 0.69235, 0.68401, 0.67550, 0.66701, 0.65859, 0.65033, 0.64234,\
        0.63475, 0.62772, 0.62141, 0.61595, 0.61130, 0.60719, 0.60336, 0.59941, 0.59263, 0.58423, 0.57498,\
        0.56525, 0.55531, 0.54538, 0.53560, 0.52608, 0.51687, 0.50793, 0.49903, 0.49001, 0.48127, 0.47276],\
        [0.39905, 0.41669, 0.43457, 0.45149, 0.46620, 0.47783, 0.48655, 0.49338, 0.49902, 0.50470, 0.50868,\
        0.51147, 0.51383, 0.51616, 0.51859, 0.52115, 0.52386, 0.52675, 0.52976, 0.53280, 0.53575, 0.53852,\
        0.54101, 0.54316, 0.54492, 0.54624, 0.54709, 0.54740, 0.54710, 0.54612, 0.54447, 0.54218, 0.53913,\
        0.53508, 0.52969, 0.52272, 0.51415, 0.50403, 0.49245, 0.48100, 0.47201, 0.46689, 0.46630, 0.47022,\
        0.47806, 0.48920, 0.50429, 0.52251, 0.54295, 0.56507, 0.58568, 0.60109, 0.61611, 0.63353, 0.65226,\
        0.66865, 0.68402, 0.70569, 0.72991, 0.75284, 0.77012, 0.78222, 0.79640, 0.81196, 0.82664, 0.83802,\
        0.84464, 0.84726, 0.84756, 0.84675, 0.84500, 0.84211, 0.83803, 0.83297, 0.82737, 0.82170, 0.81579,\
        0.80947, 0.80305, 0.79636, 0.78992, 0.78355, 0.77692, 0.76982, 0.76220, 0.75413, 0.74577, 0.73724,\
        0.72867, 0.72015, 0.71177, 0.70377, 0.69564, 0.68707, 0.67865, 0.67030, 0.66204, 0.65398, 0.64621,\
        0.63884, 0.63200, 0.62577, 0.62018, 0.61515, 0.61049, 0.60593, 0.60052, 0.59296, 0.58374, 0.57369,\
        0.56326, 0.55273, 0.54232, 0.53221, 0.52249, 0.51321, 0.50436, 0.49545, 0.48638, 0.47748, 0.46881],\
        [0.39685, 0.41221, 0.42797, 0.44312, 0.45659, 0.46790, 0.47718, 0.48499, 0.49205, 0.49842, 0.50252,\
        0.50580, 0.50874, 0.51184, 0.51510, 0.51843, 0.52195, 0.52575, 0.52978, 0.53395, 0.53810, 0.54213,\
        0.54595, 0.54950, 0.55275, 0.55564, 0.55807, 0.55987, 0.56072, 0.56024, 0.55822, 0.55470, 0.54975,\
        0.54335, 0.53545, 0.52616, 0.51578, 0.50464, 0.49224, 0.48040, 0.47179, 0.46779, 0.46906, 0.47546,\
        0.48592, 0.49909, 0.51505, 0.53404, 0.55499, 0.57788, 0.60063, 0.61773, 0.63440, 0.65379, 0.67683,\
        0.69743, 0.71399, 0.73596, 0.75916, 0.77997, 0.79706, 0.80664, 0.81995, 0.83428, 0.84684, 0.85618,\
        0.86160, 0.86374, 0.86378, 0.86253, 0.86019, 0.85662, 0.85181, 0.84601, 0.83962, 0.83297, 0.82612,\
        0.81909, 0.81195, 0.80476, 0.79794, 0.79117, 0.78422, 0.77661, 0.76831, 0.75957, 0.75057, 0.74148,\
        0.73239, 0.72339, 0.71452, 0.70576, 0.69726, 0.68941, 0.68131, 0.67314, 0.66507, 0.65718, 0.64959,\
        0.64238, 0.63561, 0.62933, 0.62348, 0.61792, 0.61240, 0.60653, 0.59982, 0.59145, 0.58170, 0.57110,\
        0.56011, 0.54908, 0.53827, 0.52786, 0.51795, 0.50850, 0.49949, 0.49072, 0.48184, 0.47303, 0.46437],\
        [0.39302, 0.40693, 0.42145, 0.43505, 0.44742, 0.45830, 0.46779, 0.47619, 0.48377, 0.49024, 0.49524,\
        0.49929, 0.50287, 0.50696, 0.51108, 0.51506, 0.51932, 0.52405, 0.52917, 0.53448, 0.53982, 0.54507,\
        0.55015, 0.55501, 0.55964, 0.56399, 0.56797, 0.57130, 0.57343, 0.57358, 0.57122, 0.56652, 0.55983,\
        0.55123, 0.54078, 0.52877, 0.51586, 0.50269, 0.48940, 0.47784, 0.47012, 0.46756, 0.47097, 0.48039,\
        0.49432, 0.51001, 0.52699, 0.54533, 0.56467, 0.58532, 0.60788, 0.62957, 0.65070, 0.67276, 0.69726,\
        0.72530, 0.74785, 0.77021, 0.79173, 0.80667, 0.82280, 0.83439, 0.84587, 0.85744, 0.86732, 0.87437,\
        0.87786, 0.87882, 0.87826, 0.87668, 0.87402, 0.86992, 0.86436, 0.85774, 0.85051, 0.84300, 0.83539,\
        0.82773, 0.82009, 0.81259, 0.80529, 0.79816, 0.79098, 0.78265, 0.77357, 0.76413, 0.75454, 0.74493,\
        0.73539, 0.72600, 0.71681, 0.70793, 0.69972, 0.69167, 0.68353, 0.67546, 0.66752, 0.65976, 0.65228,\
        0.64512, 0.63833, 0.63190, 0.62572, 0.61958, 0.61314, 0.60593, 0.59781, 0.58877, 0.57846, 0.56732,\
        0.55583, 0.54438, 0.53323, 0.52258, 0.51256, 0.50321, 0.49406, 0.48530, 0.47662, 0.46800, 0.45946],\
        [0.38985, 0.40321, 0.41524, 0.42724, 0.43851, 0.44883, 0.45824, 0.46685, 0.47468, 0.48157, 0.48741,\
        0.49241, 0.49708, 0.50222, 0.50681, 0.51114, 0.51606, 0.52180, 0.52806, 0.53452, 0.54098, 0.54734,\
        0.55353, 0.55954, 0.56535, 0.57094, 0.57626, 0.58107, 0.58473, 0.58581, 0.58314, 0.57748, 0.56957,\
        0.55929, 0.54650, 0.53161, 0.51568, 0.50001, 0.48599, 0.47518, 0.46882, 0.46782, 0.47315, 0.48539,\
        0.50297, 0.52153, 0.53977, 0.55736, 0.57455, 0.59251, 0.61502, 0.64122, 0.66700, 0.69615, 0.72481,\
        0.75549, 0.78117, 0.80213, 0.82098, 0.83516, 0.84856, 0.86060, 0.87156, 0.88030, 0.88717, 0.89204,\
        0.89279, 0.89222, 0.89076, 0.88889, 0.88612, 0.88154, 0.87512, 0.86762, 0.85960, 0.85139, 0.84316,\
        0.83500, 0.82698, 0.81917, 0.81163, 0.80442, 0.79667, 0.78735, 0.77753, 0.76752, 0.75746, 0.74745,\
        0.73759, 0.72794, 0.71861, 0.70976, 0.70149, 0.69324, 0.68512, 0.67714, 0.66928, 0.66158, 0.65411,\
        0.64692, 0.64001, 0.63335, 0.62681, 0.62016, 0.61304, 0.60503, 0.59585, 0.58553, 0.57431, 0.56252,\
        0.55055, 0.53874, 0.52732, 0.51652, 0.50640, 0.49705, 0.48821, 0.47951, 0.47098, 0.46254, 0.45417],\
        [0.38555, 0.39719, 0.40842, 0.41944, 0.42969, 0.43933, 0.44851, 0.45721, 0.46534, 0.47277, 0.47946,\
        0.48557, 0.49144, 0.49719, 0.50227, 0.50655, 0.51233, 0.51928, 0.52670, 0.53423, 0.54168, 0.54897,\
        0.55608, 0.56299, 0.56971, 0.57622, 0.58249, 0.58840, 0.59359, 0.59623, 0.59320, 0.58724, 0.57927,\
        0.56835, 0.55385, 0.53626, 0.51715, 0.49909, 0.48448, 0.47525, 0.47078, 0.47145, 0.47818, 0.49270,\
        0.51369, 0.53528, 0.55451, 0.57103, 0.58601, 0.60304, 0.62655, 0.65637, 0.68637, 0.71929, 0.75860,\
        0.79027, 0.81338, 0.83242, 0.84814, 0.86196, 0.87408, 0.88479, 0.89553, 0.90137, 0.90509, 0.90732,\
        0.90598, 0.90386, 0.90099, 0.89882, 0.89604, 0.89082, 0.88337, 0.87509, 0.86646, 0.85774, 0.84907,\
        0.84054, 0.83220, 0.82405, 0.81610, 0.80817, 0.79961, 0.78998, 0.77986, 0.76954, 0.75918, 0.74892,\
        0.73885, 0.72907, 0.71962, 0.71053, 0.70181, 0.69380, 0.68600, 0.67812, 0.67028, 0.66255, 0.65499,\
        0.64766, 0.64054, 0.63358, 0.62664, 0.61950, 0.61187, 0.60330, 0.59301, 0.58147, 0.56927, 0.55679,\
        0.54438, 0.53232, 0.52074, 0.50981, 0.50012, 0.49070, 0.48192, 0.47340, 0.46504, 0.45678, 0.44860],\
        [0.37997, 0.39105, 0.40178, 0.41164, 0.42075, 0.42971, 0.43868, 0.44750, 0.45598, 0.46400, 0.47151,\
        0.47858, 0.48530, 0.49156, 0.49684, 0.50154, 0.50883, 0.51700, 0.52542, 0.53379, 0.54200, 0.55000,\
        0.55779, 0.56537, 0.57272, 0.57980, 0.58655, 0.59284, 0.59840, 0.60208, 0.60018, 0.59564, 0.58943,\
        0.57928, 0.56422, 0.54484, 0.52244, 0.50087, 0.48749, 0.48174, 0.47954, 0.48246, 0.49117, 0.50770,\
        0.53140, 0.55354, 0.57176, 0.58590, 0.59666, 0.61009, 0.63417, 0.67098, 0.70906, 0.74443, 0.78549,\
        0.82126, 0.84403, 0.86138, 0.87501, 0.88788, 0.89950, 0.90737, 0.91549, 0.91977, 0.92094, 0.92017,\
        0.91739, 0.91357, 0.90960, 0.90599, 0.90228, 0.89610, 0.88833, 0.87974, 0.87081, 0.86181, 0.85289,\
        0.84411, 0.83548, 0.82698, 0.81848, 0.80969, 0.80033, 0.79070, 0.78058, 0.77013, 0.75962, 0.74923,\
        0.73909, 0.72928, 0.71987, 0.71088, 0.70234, 0.69437, 0.68641, 0.67841, 0.67046, 0.66259, 0.65486,\
        0.64728, 0.63985, 0.63251, 0.62511, 0.61741, 0.60905, 0.59967, 0.58860, 0.57616, 0.56319, 0.55013,\
        0.53738, 0.52527, 0.51390, 0.50295, 0.49310, 0.48414, 0.47548, 0.46709, 0.45890, 0.45083, 0.44283],\
        [0.37412, 0.38464, 0.39462, 0.40335, 0.41153, 0.41993, 0.42890, 0.43794, 0.44679, 0.45537, 0.46365,\
        0.47160, 0.47914, 0.48608, 0.49227, 0.49862, 0.50658, 0.51546, 0.52447, 0.53334, 0.54200, 0.55046,\
        0.55872, 0.56676, 0.57454, 0.58198, 0.58894, 0.59529, 0.60090, 0.60507, 0.60587, 0.60387, 0.60067,\
        0.59206, 0.57845, 0.55976, 0.53640, 0.51246, 0.50219, 0.49813, 0.49842, 0.50361, 0.51704, 0.53567,\
        0.55727, 0.57579, 0.58974, 0.59945, 0.60430, 0.60994, 0.62999, 0.67508, 0.72109, 0.75828, 0.80321,\
        0.84491, 0.87355, 0.89235, 0.90417, 0.91520, 0.92451, 0.93042, 0.93436, 0.93574, 0.93447, 0.93124,\
        0.92653, 0.92103, 0.91537, 0.90989, 0.90430, 0.89794, 0.89016, 0.88153, 0.87257, 0.86351, 0.85451,\
        0.84560, 0.83678, 0.82800, 0.81913, 0.80998, 0.80035, 0.79037, 0.77996, 0.76934, 0.75874, 0.74833,\
        0.73823, 0.72851, 0.71923, 0.71052, 0.70254, 0.69441, 0.68608, 0.67788, 0.66976, 0.66168, 0.65367,\
        0.64576, 0.63792, 0.63011, 0.62219, 0.61388, 0.60480, 0.59426, 0.58246, 0.56953, 0.55609, 0.54263,\
        0.52961, 0.51755, 0.50671, 0.49668, 0.48663, 0.47752, 0.46894, 0.46068, 0.45265, 0.44476, 0.43694],\
        [0.36795, 0.37764, 0.38662, 0.39473, 0.40216, 0.41029, 0.41958, 0.42877, 0.43787, 0.44694, 0.45596,\
        0.46482, 0.47338, 0.48150, 0.48921, 0.49701, 0.50558, 0.51469, 0.52382, 0.53280, 0.54163, 0.55031,\
        0.55886, 0.56723, 0.57536, 0.58313, 0.59041, 0.59710, 0.60332, 0.60850, 0.61152, 0.61236, 0.61088,\
        0.60537, 0.59587, 0.58114, 0.56122, 0.54151, 0.53001, 0.52563, 0.52709, 0.53503, 0.54979, 0.56847,\
        0.58619, 0.59799, 0.60456, 0.60806, 0.60774, 0.60507, 0.60976, 0.67064, 0.72620, 0.76140, 0.82345,\
        0.88031, 0.90839, 0.92843, 0.93864, 0.94552, 0.95014, 0.95226, 0.95218, 0.95002, 0.94589, 0.94018,\
        0.93339, 0.92601, 0.91846, 0.91099, 0.90392, 0.89710, 0.88921, 0.88071, 0.87188, 0.86293, 0.85396,\
        0.84503, 0.83612, 0.82715, 0.81806, 0.80880, 0.79928, 0.78874, 0.77792, 0.76712, 0.75649, 0.74617,\
        0.73623, 0.72670, 0.71754, 0.70879, 0.70055, 0.69267, 0.68461, 0.67642, 0.66815, 0.65983, 0.65148,\
        0.64313, 0.63479, 0.62644, 0.61794, 0.60892, 0.59916, 0.58791, 0.57532, 0.56196, 0.54825, 0.53460,\
        0.52135, 0.50919, 0.49872, 0.48911, 0.47974, 0.47080, 0.46233, 0.45422, 0.44634, 0.43862, 0.43098],\
        [0.36167, 0.37060, 0.37889, 0.38651, 0.39373, 0.40231, 0.41125, 0.42012, 0.42925, 0.43874, 0.44847,\
        0.45826, 0.46798, 0.47752, 0.48683, 0.49594, 0.50510, 0.51421, 0.52312, 0.53192, 0.54069, 0.54944,\
        0.55816, 0.56681, 0.57529, 0.58351, 0.59133, 0.59872, 0.60578, 0.61201, 0.61676, 0.61968, 0.62038,\
        0.61827, 0.61307, 0.60398, 0.58936, 0.57369, 0.56312, 0.55829, 0.55997, 0.56844, 0.58163, 0.59708,\
        0.60892, 0.61376, 0.61418, 0.61251, 0.60916, 0.60193, 0.59591, 0.66006, 0.72471, 0.76272, 0.85925,\
        0.93460, 0.95507, 0.96724, 0.97310, 0.97503, 0.97456, 0.97212, 0.96792, 0.96216, 0.95502, 0.94681,\
        0.93791, 0.92869, 0.91941, 0.91017, 0.90114, 0.89366, 0.88602, 0.87773, 0.86907, 0.86024, 0.85138,\
        0.84251, 0.83359, 0.82450, 0.81515, 0.80554, 0.79555, 0.78506, 0.77421, 0.76340, 0.75284, 0.74270,\
        0.73307, 0.72390, 0.71504, 0.70630, 0.69799, 0.69023, 0.68232, 0.67412, 0.66569, 0.65709, 0.64834,\
        0.63950, 0.63057, 0.62157, 0.61247, 0.60313, 0.59230, 0.58035, 0.56738, 0.55375, 0.53997, 0.52648,\
        0.51348, 0.50080, 0.49089, 0.48160, 0.47262, 0.46397, 0.45569, 0.44774, 0.44003, 0.43248, 0.42502],\
        [0.35555, 0.36389, 0.37176, 0.37926, 0.38689, 0.39514, 0.40345, 0.41172, 0.42082, 0.43073, 0.44111,\
        0.45177, 0.46260, 0.47351, 0.48439, 0.49491, 0.50459, 0.51333, 0.52177, 0.53028, 0.53891, 0.54767,\
        0.55653, 0.56544, 0.57433, 0.58309, 0.59163, 0.59986, 0.60775, 0.61496, 0.62105, 0.62563, 0.62836,\
        0.62887, 0.62677, 0.62140, 0.61212, 0.60187, 0.59404, 0.58915, 0.58981, 0.59756, 0.60689, 0.61608,\
        0.62189, 0.62231, 0.61866, 0.61299, 0.60683, 0.59752, 0.59316, 0.65222, 0.72817, 0.78928, 0.88216,\
        0.95958, 0.98964, 0.99999, 1.00216, 0.99993, 0.99509, 0.98848, 0.98049, 0.97141, 0.96145, 0.95087,\
        0.94000, 0.92916, 0.91863, 0.90847, 0.89876, 0.88999, 0.88165, 0.87316, 0.86447, 0.85570, 0.84693,\
        0.83817, 0.82933, 0.82024, 0.81062, 0.80069, 0.79041, 0.77976, 0.76893, 0.75819, 0.74778, 0.73791,\
        0.72869, 0.72016, 0.71201, 0.70374, 0.69554, 0.68761, 0.67952, 0.67114, 0.66248, 0.65356, 0.64440,\
        0.63502, 0.62548, 0.61575, 0.60586, 0.59570, 0.58460, 0.57222, 0.55896, 0.54518, 0.53141, 0.51828,\
        0.50629, 0.49498, 0.48412, 0.47459, 0.46569, 0.45721, 0.44909, 0.44129, 0.43374, 0.42637, 0.41908],\
        [0.34972, 0.35763, 0.36520, 0.37258, 0.38005, 0.38774, 0.39544, 0.40333, 0.41266, 0.42299, 0.43385,\
        0.44514, 0.45680, 0.46872, 0.48079, 0.49273, 0.50282, 0.51084, 0.51904, 0.52743, 0.53599, 0.54478,\
        0.55380, 0.56302, 0.57235, 0.58171, 0.59099, 0.60008, 0.60873, 0.61681, 0.62398, 0.62990, 0.63423,\
        0.63667, 0.63683, 0.63432, 0.62927, 0.62329, 0.61799, 0.61414, 0.61402, 0.61902, 0.62467, 0.62907,\
        0.63044, 0.62770, 0.62128, 0.61253, 0.60310, 0.59589, 0.59617, 0.65667, 0.74488, 0.83051, 0.91812,\
        0.97779, 1.00944, 1.02123, 1.02227, 1.01760, 1.00972, 0.99993, 0.98891, 0.97708, 0.96474, 0.95215,\
        0.93958, 0.92736, 0.91587, 0.90537, 0.89523, 0.88543, 0.87623, 0.86726, 0.85834, 0.84949, 0.84075,\
        0.83211, 0.82343, 0.81450, 0.80516, 0.79460, 0.78396, 0.77311, 0.76224, 0.75160, 0.74138, 0.73177,\
        0.72299, 0.71521, 0.70829, 0.70101, 0.69289, 0.68455, 0.67611, 0.66747, 0.65856, 0.64934, 0.63978,\
        0.62988, 0.61977, 0.60957, 0.59879, 0.58807, 0.57645, 0.56385, 0.55045, 0.53661, 0.52278, 0.50970,\
        0.49823, 0.48762, 0.47741, 0.46791, 0.45903, 0.45062, 0.44260, 0.43492, 0.42752, 0.42032, 0.41320],\
        [0.34425, 0.35181, 0.35911, 0.36627, 0.37344, 0.38070, 0.38806, 0.39593, 0.40538, 0.41569, 0.42663,\
        0.43822, 0.45029, 0.46265, 0.47505, 0.48686, 0.49692, 0.50587, 0.51460, 0.52314, 0.53175, 0.54063,\
        0.54987, 0.55943, 0.56923, 0.57916, 0.58908, 0.59883, 0.60831, 0.61726, 0.62533, 0.63229, 0.63789,\
        0.64187, 0.64391, 0.64383, 0.64185, 0.63880, 0.63563, 0.63330, 0.63310, 0.63524, 0.63782, 0.63911,\
        0.63792, 0.63364, 0.62646, 0.61745, 0.60940, 0.60823, 0.62420, 0.69044, 0.77900, 0.86907, 0.94286,\
        0.99236, 1.02029, 1.03191, 1.03275, 1.02704, 1.01747, 1.00563, 0.99251, 0.97871, 0.96464, 0.95055,\
        0.93665, 0.92322, 0.91077, 0.89966, 0.88930, 0.87920, 0.86948, 0.86006, 0.85082, 0.84176, 0.83295,\
        0.82441, 0.81590, 0.80697, 0.79761, 0.78730, 0.77622, 0.76516, 0.75429, 0.74379, 0.73381, 0.72442,\
        0.71589, 0.70862, 0.70271, 0.69690, 0.68879, 0.68035, 0.67176, 0.66296, 0.65389, 0.64448, 0.63465,\
        0.62431, 0.61354, 0.60302, 0.59234, 0.58060, 0.56842, 0.55559, 0.54219, 0.52845, 0.51471, 0.50159,\
        0.49040, 0.48033, 0.47066, 0.46142, 0.45262, 0.44425, 0.43627, 0.42865, 0.42138, 0.41434, 0.40739],\
        [0.33917, 0.34642, 0.35347, 0.36040, 0.36733, 0.37438, 0.38173, 0.38983, 0.39904, 0.40860, 0.41940,\
        0.43099, 0.44308, 0.45543, 0.46768, 0.47928, 0.48979, 0.49953, 0.50875, 0.51743, 0.52610, 0.53515,\
        0.54465, 0.55459, 0.56488, 0.57538, 0.58594, 0.59635, 0.60657, 0.61624, 0.62499, 0.63276, 0.63942,\
        0.64473, 0.64843, 0.65040, 0.65077, 0.65000, 0.64873, 0.64765, 0.64740, 0.64798, 0.64856, 0.64815,\
        0.64612, 0.64230, 0.63701, 0.63118, 0.62735, 0.63212, 0.66345, 0.74064, 0.82231, 0.89959, 0.95827,\
        0.99868, 1.02285, 1.03361, 1.03444, 1.02852, 1.01828, 1.00541, 0.99109, 0.97612, 0.96105, 0.94615,\
        0.93150, 0.91720, 0.90388, 0.89228, 0.88170, 0.87144, 0.86143, 0.85165, 0.84205, 0.83267, 0.82365,\
        0.81514, 0.80687, 0.79799, 0.78825, 0.77791, 0.76698, 0.75595, 0.74518, 0.73494, 0.72534, 0.71625,\
        0.70774, 0.70075, 0.69508, 0.68957, 0.68257, 0.67464, 0.66625, 0.65750, 0.64840, 0.63895, 0.62909,\
        0.61862, 0.60737, 0.59594, 0.58484, 0.57294, 0.56045, 0.54756, 0.53434, 0.52096, 0.50776, 0.49519,\
        0.48397, 0.47387, 0.46436, 0.45526, 0.44653, 0.43815, 0.43013, 0.42253, 0.41534, 0.40843, 0.40165],\
        [0.33446, 0.34146, 0.34827, 0.35500, 0.36175, 0.36867, 0.37595, 0.38383, 0.39232, 0.40168, 0.41234,\
        0.42360, 0.43542, 0.44750, 0.45950, 0.47103, 0.48187, 0.49210, 0.50163, 0.51027, 0.51906, 0.52838,\
        0.53819, 0.54852, 0.55929, 0.57037, 0.58159, 0.59278, 0.60368, 0.61371, 0.62286, 0.63130, 0.63892,\
        0.64548, 0.65071, 0.65445, 0.65677, 0.65791, 0.65827, 0.65827, 0.65824, 0.65821, 0.65789, 0.65698,\
        0.65548, 0.65380, 0.65283, 0.65354, 0.65750, 0.67202, 0.71131, 0.79960, 0.86154, 0.91593, 0.96253,\
        0.99649, 1.01773, 1.02769, 1.02865, 1.02298, 1.01275, 0.99960, 0.98483, 0.96942, 0.95409, 0.93918,\
        0.92462, 0.91023, 0.89640, 0.88438, 0.87325, 0.86259, 0.85226, 0.84217, 0.83224, 0.82251, 0.81309,\
        0.80442, 0.79630, 0.78750, 0.77781, 0.76745, 0.75664, 0.74572, 0.73511, 0.72519, 0.71622, 0.70785,\
        0.69957, 0.69262, 0.68694, 0.68132, 0.67493, 0.66763, 0.65964, 0.65106, 0.64201, 0.63261, 0.62293,\
        0.61291, 0.60164, 0.58972, 0.57753, 0.56508, 0.55240, 0.53962, 0.52678, 0.51392, 0.50139, 0.48939,\
        0.47829, 0.46811, 0.45860, 0.44955, 0.44083, 0.43240, 0.42427, 0.41658, 0.40945, 0.40263, 0.39592],\
        [0.33013, 0.33690, 0.34352, 0.35007, 0.35666, 0.36344, 0.37056, 0.37817, 0.38644, 0.39563, 0.40544,\
        0.41618, 0.42758, 0.43927, 0.45096, 0.46235, 0.47323, 0.48350, 0.49300, 0.50167, 0.51096, 0.52056,\
        0.53061, 0.54130, 0.55251, 0.56407, 0.57584, 0.58765, 0.59909, 0.60929, 0.61880, 0.62789, 0.63651,\
        0.64432, 0.65099, 0.65635, 0.66038, 0.66319, 0.66502, 0.66611, 0.66662, 0.66665, 0.66624, 0.66562,\
        0.66535, 0.66646, 0.67062, 0.67996, 0.69369, 0.71964, 0.76676, 0.82747, 0.87756, 0.91925, 0.95774,\
        0.98729, 1.00637, 1.01570, 1.01685, 1.01167, 1.00187, 0.98896, 0.97426, 0.95892, 0.94392, 0.92979,\
        0.91619, 0.90268, 0.88894, 0.87600, 0.86403, 0.85278, 0.84210, 0.83179, 0.82162, 0.81163, 0.80210,\
        0.79307, 0.78485, 0.77614, 0.76662, 0.75639, 0.74568, 0.73486, 0.72440, 0.71467, 0.70635, 0.69898,\
        0.69133, 0.68441, 0.67842, 0.67268, 0.66656, 0.65974, 0.65213, 0.64373, 0.63468, 0.62522, 0.61556,\
        0.60587, 0.59575, 0.58283, 0.56975, 0.55680, 0.54402, 0.53153, 0.51936, 0.50713, 0.49506, 0.48364,\
        0.47281, 0.46271, 0.45324, 0.44424, 0.43557, 0.42711, 0.41881, 0.41085, 0.40378, 0.39713, 0.39042],\
        [0.32617, 0.33274, 0.33918, 0.34556, 0.35200, 0.35862, 0.36555, 0.37292, 0.38086, 0.38943, 0.39868,\
        0.40898, 0.41984, 0.43102, 0.44236, 0.45353, 0.46426, 0.47440, 0.48388, 0.49303, 0.50258, 0.51189,\
        0.52211, 0.53313, 0.54466, 0.55653, 0.56857, 0.58055, 0.59208, 0.60276, 0.61277, 0.62263, 0.63239,\
        0.64150, 0.64955, 0.65638, 0.66193, 0.66628, 0.66955, 0.67183, 0.67320, 0.67376, 0.67375, 0.67376,\
        0.67477, 0.67817, 0.68579, 0.70019, 0.72221, 0.75378, 0.79929, 0.83977, 0.87869, 0.91450, 0.94760,\
        0.97354, 0.99061, 0.99920, 1.00051, 0.99592, 0.98681, 0.97447, 0.96018, 0.94521, 0.93079, 0.91817,\
        0.90622, 0.89328, 0.87979, 0.86634, 0.85362, 0.84179, 0.83082, 0.82049, 0.81036, 0.80004, 0.79073,\
        0.78184, 0.77333, 0.76460, 0.75522, 0.74515, 0.73455, 0.72381, 0.71356, 0.70413, 0.69582, 0.68894,\
        0.68199, 0.67537, 0.66931, 0.66354, 0.65763, 0.65120, 0.64395, 0.63571, 0.62651, 0.61673, 0.60688,\
        0.59660, 0.58594, 0.57375, 0.56076, 0.54775, 0.53504, 0.52288, 0.51159, 0.50084, 0.48907, 0.47784,\
        0.46729, 0.45738, 0.44806, 0.43921, 0.43072, 0.42242, 0.41412, 0.40575, 0.39823, 0.39187, 0.38525],\
        [0.32254, 0.32893, 0.33521, 0.34143, 0.34772, 0.35417, 0.36090, 0.36802, 0.37563, 0.38380, 0.39268,\
        0.40236, 0.41234, 0.42297, 0.43398, 0.44490, 0.45542, 0.46542, 0.47493, 0.48419, 0.49331, 0.50252,\
        0.51310, 0.52433, 0.53602, 0.54798, 0.56005, 0.57203, 0.58362, 0.59467, 0.60489, 0.61590, 0.62698,\
        0.63730, 0.64660, 0.65471, 0.66163, 0.66742, 0.67214, 0.67581, 0.67837, 0.67984, 0.68050, 0.68115,\
        0.68315, 0.68814, 0.69757, 0.71365, 0.73650, 0.76534, 0.79936, 0.83738, 0.87303, 0.90547, 0.93477,\
        0.95717, 0.97195, 0.97956, 0.98091, 0.97700, 0.96878, 0.95727, 0.94363, 0.92927, 0.91538, 0.90377,\
        0.89586, 0.88250, 0.86877, 0.85498, 0.84167, 0.82934, 0.81818, 0.80793, 0.79810, 0.78811, 0.77875,\
        0.77009, 0.76169, 0.75310, 0.74394, 0.73410, 0.72369, 0.71302, 0.70273, 0.69418, 0.68602, 0.67880,\
        0.67200, 0.66552, 0.65946, 0.65374, 0.64808, 0.64208, 0.63526, 0.62723, 0.61780, 0.60740, 0.59709,\
        0.58661, 0.57534, 0.56326, 0.55064, 0.53795, 0.52547, 0.51348, 0.50256, 0.49264, 0.48215, 0.47158,\
        0.46148, 0.45192, 0.44288, 0.43429, 0.42610, 0.41820, 0.41042, 0.40243, 0.39456, 0.38746, 0.38050],\
        [0.31921, 0.32545, 0.33157, 0.33764, 0.34377, 0.35006, 0.35660, 0.36348, 0.37078, 0.37859, 0.38695,\
        0.39575, 0.40496, 0.41537, 0.42615, 0.43678, 0.44701, 0.45678, 0.46616, 0.47527, 0.48430, 0.49376,\
        0.50413, 0.51527, 0.52688, 0.53874, 0.55070, 0.56262, 0.57431, 0.58555, 0.59664, 0.60884, 0.62073,\
        0.63198, 0.64227, 0.65142, 0.65949, 0.66659, 0.67281, 0.67811, 0.68226, 0.68505, 0.68659, 0.68782,\
        0.69033, 0.69634, 0.70709, 0.72278, 0.74338, 0.76806, 0.79679, 0.83071, 0.86450, 0.89531, 0.92058,\
        0.93886, 0.95119, 0.95767, 0.95906, 0.95599, 0.94893, 0.93853, 0.92574, 0.91226, 0.89970, 0.88893,\
        0.88021, 0.86887, 0.85578, 0.84196, 0.82816, 0.81537, 0.80404, 0.79378, 0.78406, 0.77469, 0.76590,\
        0.75773, 0.74982, 0.74165, 0.73285, 0.72337, 0.71334, 0.70298, 0.69303, 0.68421, 0.67618, 0.66869,\
        0.66165, 0.65500, 0.64882, 0.64313, 0.63775, 0.63224, 0.62601, 0.61845, 0.60882, 0.59804, 0.58700,\
        0.57588, 0.56426, 0.55215, 0.53984, 0.52773, 0.51584, 0.50406, 0.49353, 0.48400, 0.47444, 0.46477,\
        0.45532, 0.44624, 0.43755, 0.42925, 0.42137, 0.41393, 0.40683, 0.39955, 0.39140, 0.38352, 0.37602],\
        [0.31616, 0.32225, 0.32821, 0.33414, 0.34012, 0.34624, 0.35260, 0.35927, 0.36631, 0.37376, 0.38162,\
        0.38984, 0.39876, 0.40884, 0.41919, 0.42936, 0.43918, 0.44862, 0.45775, 0.46673, 0.47577, 0.48521,\
        0.49533, 0.50615, 0.51748, 0.52905, 0.54080, 0.55269, 0.56462, 0.57656, 0.58882, 0.60131, 0.61360,\
        0.62560, 0.63660, 0.64643, 0.65532, 0.66354, 0.67125, 0.67840, 0.68467, 0.68939, 0.69217, 0.69409,\
        0.69702, 0.70305, 0.71408, 0.72843, 0.74627, 0.76787, 0.79370, 0.82307, 0.85338, 0.88139, 0.90249,\
        0.91826, 0.92872, 0.93406, 0.93570, 0.93380, 0.92821, 0.91912, 0.90804, 0.89482, 0.88368, 0.87324,\
        0.86360, 0.85319, 0.84128, 0.82784, 0.81353, 0.80022, 0.78879, 0.77842, 0.76884, 0.76003, 0.75203,\
        0.74470, 0.73763, 0.73022, 0.72192, 0.71279, 0.70339, 0.69341, 0.68369, 0.67466, 0.66629, 0.65842,\
        0.65096, 0.64392, 0.63740, 0.63155, 0.62643, 0.62146, 0.61571, 0.60897, 0.59978, 0.58814, 0.57648,\
        0.56476, 0.55285, 0.54071, 0.52859, 0.51720, 0.50668, 0.49566, 0.48531, 0.47586, 0.46673, 0.45772,\
        0.44889, 0.44032, 0.43202, 0.42398, 0.41626, 0.40899, 0.40210, 0.39497, 0.38710, 0.37914, 0.37139],\
        [0.31333, 0.31927, 0.32509, 0.33087, 0.33670, 0.34268, 0.34888, 0.35536, 0.36219, 0.36940, 0.37702,\
        0.38513, 0.39393, 0.40339, 0.41310, 0.42267, 0.43193, 0.44093, 0.44974, 0.45852, 0.46744, 0.47674,\
        0.48659, 0.49704, 0.50794, 0.51904, 0.53049, 0.54234, 0.55452, 0.56700, 0.57975, 0.59255, 0.60549,\
        0.61823, 0.62947, 0.63949, 0.64880, 0.65780, 0.66681, 0.67590, 0.68478, 0.69247, 0.69721, 0.70031,\
        0.70405, 0.71024, 0.71930, 0.73090, 0.74530, 0.76335, 0.78564, 0.81143, 0.83813, 0.86258, 0.88179,\
        0.89631, 0.90489, 0.90922, 0.91155, 0.91115, 0.90718, 0.89946, 0.88963, 0.87821, 0.86684, 0.85626,\
        0.84624, 0.83648, 0.82601, 0.81329, 0.79886, 0.78505, 0.77306, 0.76237, 0.75280, 0.74442, 0.73724,\
        0.73094, 0.72498, 0.71873, 0.71127, 0.70199, 0.69305, 0.68358, 0.67404, 0.66488, 0.65618, 0.64788,\
        0.63993, 0.63234, 0.62527, 0.61895, 0.61393, 0.60963, 0.60375, 0.59687, 0.58810, 0.57674, 0.56488,\
        0.55305, 0.54121, 0.52928, 0.51726, 0.50591, 0.49684, 0.48702, 0.47718, 0.46793, 0.45912, 0.45058,\
        0.44228, 0.43421, 0.42632, 0.41851, 0.41076, 0.40339, 0.39648, 0.38941, 0.38188, 0.37412, 0.36639],\
        [0.31068, 0.31648, 0.32214, 0.32777, 0.33347, 0.33931, 0.34536, 0.35170, 0.35838, 0.36543, 0.37292,\
        0.38091, 0.38948, 0.39853, 0.40762, 0.41651, 0.42515, 0.43362, 0.44205, 0.45055, 0.45924, 0.46828,\
        0.47778, 0.48778, 0.49812, 0.50869, 0.51982, 0.53156, 0.54390, 0.55669, 0.56982, 0.58323, 0.59696,\
        0.60973, 0.62046, 0.63020, 0.63952, 0.64886, 0.65868, 0.66935, 0.68079, 0.69218, 0.70109, 0.70550,\
        0.70939, 0.71462, 0.72137, 0.72943, 0.73953, 0.75322, 0.77160, 0.79426, 0.81876, 0.84173, 0.85965,\
        0.87222, 0.88000, 0.88463, 0.88770, 0.88845, 0.88573, 0.87953, 0.87039, 0.85994, 0.84902, 0.83823,\
        0.82775, 0.81830, 0.81005, 0.79784, 0.78347, 0.76943, 0.75688, 0.74583, 0.73613, 0.72803, 0.72169,\
        0.71647, 0.71159, 0.70651, 0.70072, 0.69182, 0.68247, 0.67314, 0.66374, 0.65452, 0.64560, 0.63695,\
        0.62853, 0.62037, 0.61275, 0.60577, 0.60009, 0.59654, 0.59036, 0.58283, 0.57384, 0.56324, 0.55197,\
        0.54063, 0.52934, 0.51815, 0.50693, 0.49580, 0.48657, 0.47771, 0.46872, 0.45994, 0.45149, 0.44334,\
        0.43549, 0.42793, 0.42054, 0.41308, 0.40532, 0.39771, 0.39070, 0.38361, 0.37623, 0.36865, 0.36102],\
        [0.30817, 0.31381, 0.31930, 0.32479, 0.33035, 0.33607, 0.34200, 0.34821, 0.35475, 0.36166, 0.36901,\
        0.37683, 0.38512, 0.39374, 0.40235, 0.41063, 0.41858, 0.42652, 0.43456, 0.44274, 0.45112, 0.45981,\
        0.46888, 0.47834, 0.48810, 0.49823, 0.50894, 0.52043, 0.53281, 0.54561, 0.55875, 0.57232, 0.58602,\
        0.59844, 0.60877, 0.61820, 0.62726, 0.63641, 0.64627, 0.65760, 0.67053, 0.68419, 0.69650, 0.70386,\
        0.70773, 0.71299, 0.71814, 0.72276, 0.72808, 0.73696, 0.75110, 0.77111, 0.79533, 0.81873, 0.83531,\
        0.84641, 0.85431, 0.86011, 0.86420, 0.86592, 0.86428, 0.85887, 0.85044, 0.84062, 0.83026, 0.81945,\
        0.80892, 0.79823, 0.79126, 0.77986, 0.76602, 0.75234, 0.73991, 0.72893, 0.71917, 0.71114, 0.70566,\
        0.70144, 0.69730, 0.69287, 0.68739, 0.67986, 0.67102, 0.66185, 0.65258, 0.64335, 0.63433, 0.62548,\
        0.61675, 0.60812, 0.60001, 0.59288, 0.58683, 0.58179, 0.57558, 0.56783, 0.55870, 0.54854, 0.53806,\
        0.52766, 0.51722, 0.50697, 0.49699, 0.48676, 0.47737, 0.46867, 0.46019, 0.45188, 0.44380, 0.43597,\
        0.42848, 0.42138, 0.41458, 0.40769, 0.40034, 0.39255, 0.38511, 0.37779, 0.37039, 0.36290, 0.35537],\
        [0.30571, 0.31117, 0.31650, 0.32185, 0.32730, 0.33291, 0.33872, 0.34480, 0.35120, 0.35795, 0.36511,\
        0.37267, 0.38061, 0.38880, 0.39701, 0.40463, 0.41196, 0.41950, 0.42721, 0.43505, 0.44306, 0.45134,\
        0.45997, 0.46896, 0.47828, 0.48796, 0.49805, 0.50925, 0.52158, 0.53378, 0.54630, 0.55929, 0.57221,\
        0.58407, 0.59441, 0.60361, 0.61230, 0.62074, 0.62960, 0.64049, 0.65351, 0.66737, 0.68003, 0.68910,\
        0.69554, 0.70241, 0.70733, 0.70974, 0.71049, 0.71534, 0.72508, 0.74276, 0.76734, 0.79172, 0.80769,\
        0.81872, 0.82756, 0.83501, 0.84056, 0.84365, 0.84327, 0.83833, 0.82983, 0.82032, 0.81112, 0.80007,\
        0.78992, 0.77970, 0.77038, 0.75950, 0.74676, 0.73387, 0.72215, 0.71202, 0.70259, 0.69485, 0.68979,\
        0.68607, 0.68236, 0.67819, 0.67293, 0.66618, 0.65816, 0.64945, 0.64037, 0.63115, 0.62213, 0.61331,\
        0.60416, 0.59567, 0.58708, 0.57965, 0.57314, 0.56712, 0.56051, 0.55262, 0.54340, 0.53338, 0.52359,\
        0.51449, 0.50518, 0.49547, 0.48639, 0.47719, 0.46830, 0.45988, 0.45176, 0.44384, 0.43608, 0.42849,\
        0.42118, 0.41437, 0.40813, 0.40184, 0.39464, 0.38699, 0.37936, 0.37187, 0.36441, 0.35696, 0.34952],\
        [0.30321, 0.30844, 0.31365, 0.31892, 0.32428, 0.32978, 0.33548, 0.34142, 0.34765, 0.35421, 0.36111,\
        0.36835, 0.37587, 0.38353, 0.39107, 0.39816, 0.40524, 0.41256, 0.42004, 0.42751, 0.43508, 0.44293,\
        0.45114, 0.45974, 0.46872, 0.47812, 0.48807, 0.49933, 0.51056, 0.52116, 0.53260, 0.54475, 0.55692,\
        0.56825, 0.57829, 0.58719, 0.59532, 0.60284, 0.60961, 0.61949, 0.63156, 0.64429, 0.65556, 0.66451,\
        0.67260, 0.68033, 0.68600, 0.68837, 0.68804, 0.69069, 0.69641, 0.71230, 0.73629, 0.75984, 0.77781,\
        0.79068, 0.80074, 0.81014, 0.81665, 0.82139, 0.82288, 0.81826, 0.80893, 0.79907, 0.79073, 0.78068,\
        0.77019, 0.76007, 0.74997, 0.73906, 0.72696, 0.71481, 0.70397, 0.69494, 0.68669, 0.67963, 0.67457,\
        0.67074, 0.66712, 0.66300, 0.65791, 0.65159, 0.64413, 0.63587, 0.62710, 0.61777, 0.60857, 0.60004,\
        0.59088, 0.58189, 0.57344, 0.56578, 0.55895, 0.55253, 0.54580, 0.53799, 0.52870, 0.51844, 0.50891,\
        0.50122, 0.49326, 0.48450, 0.47590, 0.46743, 0.45914, 0.45115, 0.44343, 0.43590, 0.42846, 0.42103,\
        0.41371, 0.40687, 0.40085, 0.39494, 0.38808, 0.38071, 0.37320, 0.36571, 0.35826, 0.35086, 0.34351],\
        [0.30039, 0.30552, 0.31078, 0.31602, 0.32129, 0.32668, 0.33225, 0.33803, 0.34407, 0.35040, 0.35701,\
        0.36389, 0.37096, 0.37810, 0.38514, 0.39197, 0.39878, 0.40592, 0.41319, 0.42014, 0.42716, 0.43455,\
        0.44239, 0.45066, 0.45937, 0.46856, 0.47841, 0.48891, 0.49829, 0.50764, 0.51805, 0.52950, 0.54112,\
        0.55187, 0.56138, 0.56974, 0.57713, 0.58347, 0.58934, 0.59788, 0.60764, 0.61921, 0.62778, 0.63459,\
        0.64260, 0.65082, 0.65717, 0.66070, 0.66254, 0.66550, 0.67204, 0.68624, 0.70739, 0.73015, 0.74970,\
        0.76458, 0.77600, 0.78573, 0.79228, 0.79865, 0.80191, 0.79795, 0.78842, 0.77872, 0.76959, 0.75998,\
        0.74993, 0.73993, 0.72986, 0.71938, 0.70795, 0.69640, 0.68657, 0.67811, 0.67068, 0.66435, 0.65941,\
        0.65547, 0.65174, 0.64752, 0.64242, 0.63632, 0.62912, 0.62107, 0.61275, 0.60353, 0.59369, 0.58494,\
        0.57604, 0.56718, 0.55884, 0.55118, 0.54427, 0.53794, 0.53159, 0.52442, 0.51552, 0.50499, 0.49552,\
        0.48830, 0.48118, 0.47348, 0.46559, 0.45772, 0.44997, 0.44244, 0.43516, 0.42808, 0.42104, 0.41385,\
        0.40654, 0.39942, 0.39342, 0.38748, 0.38097, 0.37395, 0.36667, 0.35930, 0.35194, 0.34462, 0.33739],\
        [0.29746, 0.30274, 0.30805, 0.31319, 0.31834, 0.32359, 0.32901, 0.33462, 0.34045, 0.34652, 0.35282,\
        0.35933, 0.36599, 0.37272, 0.37944, 0.38612, 0.39287, 0.39991, 0.40679, 0.41279, 0.41919, 0.42620,\
        0.43370, 0.44166, 0.45005, 0.45884, 0.46797, 0.47708, 0.48564, 0.49413, 0.50348, 0.51446, 0.52564,\
        0.53562, 0.54430, 0.55202, 0.55881, 0.56473, 0.57053, 0.57744, 0.58559, 0.59592, 0.59977, 0.60380,\
        0.61132, 0.61989, 0.62704, 0.63207, 0.63637, 0.64168, 0.64968, 0.66317, 0.68229, 0.70414, 0.72444,\
        0.74021, 0.75179, 0.76067, 0.76780, 0.77402, 0.77710, 0.77431, 0.76682, 0.75778, 0.74854, 0.73896,\
        0.72928, 0.71972, 0.71005, 0.70026, 0.69012, 0.67954, 0.67020, 0.66202, 0.65486, 0.64882, 0.64400,\
        0.64005, 0.63622, 0.63177, 0.62639, 0.62037, 0.61327, 0.60487, 0.59670, 0.58778, 0.57810, 0.56892,\
        0.56010, 0.55157, 0.54351, 0.53599, 0.52916, 0.52321, 0.51759, 0.51168, 0.50399, 0.49408, 0.48474,\
        0.47705, 0.46997, 0.46283, 0.45553, 0.44818, 0.44087, 0.43373, 0.42688, 0.42032, 0.41383, 0.40704,\
        0.39994, 0.39296, 0.38654, 0.38033, 0.37387, 0.36705, 0.35996, 0.35275, 0.34551, 0.33831, 0.33121],\
        [0.29521, 0.30057, 0.30556, 0.31042, 0.31540, 0.32053, 0.32579, 0.33121, 0.33680, 0.34259, 0.34856,\
        0.35471, 0.36101, 0.36739, 0.37384, 0.38038, 0.38708, 0.39399, 0.40001, 0.40500, 0.41116, 0.41792,\
        0.42509, 0.43274, 0.44077, 0.44904, 0.45738, 0.46559, 0.47356, 0.48160, 0.49041, 0.50063, 0.51115,\
        0.51974, 0.52732, 0.53449, 0.54101, 0.54673, 0.55212, 0.55797, 0.56462, 0.57098, 0.57440, 0.57774,\
        0.58398, 0.59172, 0.59922, 0.60522, 0.61187, 0.61941, 0.62778, 0.64081, 0.65900, 0.68019, 0.70098,\
        0.71636, 0.72730, 0.73507, 0.74180, 0.74786, 0.75101, 0.74937, 0.74365, 0.73586, 0.72740, 0.71806,\
        0.70863, 0.69983, 0.69076, 0.68166, 0.67244, 0.66316, 0.65444, 0.64645, 0.63923, 0.63306, 0.62823,\
        0.62430, 0.62045, 0.61570, 0.60960, 0.60338, 0.59649, 0.58788, 0.57921, 0.57032, 0.56111, 0.55212,\
        0.54354, 0.53540, 0.52785, 0.52069, 0.51371, 0.50844, 0.50332, 0.49849, 0.49181, 0.48313, 0.47443,\
        0.46666, 0.45956, 0.45268, 0.44581, 0.43888, 0.43192, 0.42504, 0.41849, 0.41244, 0.40665, 0.40035,\
        0.39343, 0.38655, 0.37994, 0.37350, 0.36697, 0.36022, 0.35327, 0.34619, 0.33907, 0.33200, 0.32503],\
        [0.29295, 0.29819, 0.30285, 0.30760, 0.31251, 0.31751, 0.32261, 0.32781, 0.33315, 0.33863, 0.34427,\
        0.35007, 0.35600, 0.36205, 0.36820, 0.37445, 0.38076, 0.38690, 0.39230, 0.39735, 0.40357, 0.40982,\
        0.41660, 0.42398, 0.43167, 0.43942, 0.44710, 0.45464, 0.46213, 0.46987, 0.47832, 0.48769, 0.49707,\
        0.50369, 0.51039, 0.51730, 0.52377, 0.52920, 0.53392, 0.53868, 0.54369, 0.54816, 0.55157, 0.55527,\
        0.56069, 0.56757, 0.57488, 0.58192, 0.59042, 0.59831, 0.60589, 0.61874, 0.63605, 0.65625, 0.67576,\
        0.69128, 0.70211, 0.70873, 0.71516, 0.72204, 0.72594, 0.72494, 0.72029, 0.71351, 0.70617, 0.69824,\
        0.68863, 0.68039, 0.67205, 0.66371, 0.65536, 0.64706, 0.63906, 0.63136, 0.62385, 0.61707, 0.61212,\
        0.60805, 0.60406, 0.59903, 0.59228, 0.58532, 0.57808, 0.56977, 0.56102, 0.55214, 0.54328, 0.53475,\
        0.52668, 0.51896, 0.51204, 0.50614, 0.49880, 0.49379, 0.48926, 0.48435, 0.47844, 0.47120, 0.46358,\
        0.45630, 0.44947, 0.44291, 0.43642, 0.42989, 0.42322, 0.41647, 0.40997, 0.40421, 0.39899, 0.39311,\
        0.38658, 0.37993, 0.37336, 0.36685, 0.36028, 0.35358, 0.34671, 0.33974, 0.33273, 0.32578, 0.31895],\
        [0.29051, 0.29544, 0.30006, 0.30488, 0.30974, 0.31461, 0.31952, 0.32448, 0.32953, 0.33468, 0.33997,\
        0.34540, 0.35096, 0.35666, 0.36246, 0.36832, 0.37416, 0.37979, 0.38508, 0.39041, 0.39611, 0.40176,\
        0.40834, 0.41554, 0.42292, 0.43017, 0.43720, 0.44411, 0.45107, 0.45830, 0.46600, 0.47405, 0.48157,\
        0.48779, 0.49392, 0.50053, 0.50692, 0.51188, 0.51570, 0.51951, 0.52368, 0.52737, 0.53076, 0.53470,\
        0.53994, 0.54644, 0.55371, 0.56145, 0.56973, 0.57781, 0.58636, 0.59821, 0.61415, 0.63310, 0.65075,\
        0.66541, 0.67621, 0.68362, 0.69056, 0.69805, 0.70281, 0.70207, 0.69781, 0.69190, 0.68510, 0.67790,\
        0.66959, 0.66163, 0.65396, 0.64638, 0.63884, 0.63131, 0.62403, 0.61681, 0.60932, 0.60114, 0.59596,\
        0.59133, 0.58652, 0.58088, 0.57406, 0.56676, 0.55914, 0.55098, 0.54243, 0.53374, 0.52519, 0.51720,\
        0.50998, 0.50273, 0.49626, 0.49117, 0.48518, 0.47981, 0.47521, 0.47044, 0.46504, 0.45886, 0.45229,\
        0.44575, 0.43944, 0.43331, 0.42728, 0.42121, 0.41492, 0.40832, 0.40168, 0.39605, 0.39093, 0.38543,\
        0.37941, 0.37311, 0.36673, 0.36030, 0.35380, 0.34716, 0.34039, 0.33351, 0.32661, 0.31975, 0.31306],\
        [0.28825, 0.29315, 0.29783, 0.30253, 0.30722, 0.31190, 0.31658, 0.32126, 0.32598, 0.33077, 0.33567,\
        0.34071, 0.34590, 0.35123, 0.35666, 0.36214, 0.36759, 0.37292, 0.37813, 0.38336, 0.38871, 0.39431,\
        0.40064, 0.40762, 0.41467, 0.42132, 0.42761, 0.43383, 0.44014, 0.44665, 0.45338, 0.46014, 0.46649,\
        0.47226, 0.47794, 0.48387, 0.48954, 0.49404, 0.49724, 0.50056, 0.50473, 0.50797, 0.51146, 0.51563,\
        0.52104, 0.52760, 0.53493, 0.54288, 0.55122, 0.55982, 0.56933, 0.58100, 0.59589, 0.61301, 0.62745,\
        0.64095, 0.65191, 0.66068, 0.66858, 0.67567, 0.68044, 0.68035, 0.67638, 0.67108, 0.66476, 0.65789,\
        0.65059, 0.64346, 0.63652, 0.62968, 0.62296, 0.61583, 0.60918, 0.60230, 0.59518, 0.58705, 0.58029,\
        0.57447, 0.56859, 0.56220, 0.55522, 0.54787, 0.54021, 0.53223, 0.52401, 0.51566, 0.50746, 0.49992,\
        0.49350, 0.48715, 0.48123, 0.47606, 0.47096, 0.46603, 0.46145, 0.45686, 0.45193, 0.44655, 0.44085,\
        0.43507, 0.42936, 0.42376, 0.41824, 0.41271, 0.40699, 0.40071, 0.39435, 0.38857, 0.38327, 0.37789,\
        0.37220, 0.36625, 0.36013, 0.35389, 0.34753, 0.34102, 0.33436, 0.32761, 0.32082, 0.31408, 0.30749],\
        [0.28627, 0.29124, 0.29594, 0.30051, 0.30497, 0.30943, 0.31383, 0.31818, 0.32252, 0.32690, 0.33139,\
        0.33603, 0.34084, 0.34579, 0.35087, 0.35602, 0.36117, 0.36628, 0.37137, 0.37651, 0.38180, 0.38741,\
        0.39357, 0.40024, 0.40690, 0.41276, 0.41816, 0.42364, 0.42928, 0.43499, 0.44075, 0.44644, 0.45185,\
        0.45694, 0.46192, 0.46690, 0.47158, 0.47553, 0.47880, 0.48223, 0.48604, 0.48954, 0.49342, 0.49791,\
        0.50381, 0.51056, 0.51804, 0.52623, 0.53487, 0.54402, 0.55416, 0.56611, 0.58034, 0.59351, 0.60625,\
        0.61885, 0.62970, 0.63916, 0.64759, 0.65461, 0.65880, 0.65894, 0.65568, 0.65081, 0.64512, 0.63873,\
        0.63204, 0.62594, 0.62004, 0.61351, 0.60788, 0.60109, 0.59404, 0.58748, 0.58002, 0.57227, 0.56484,\
        0.55788, 0.55092, 0.54374, 0.53645, 0.52922, 0.52174, 0.51400, 0.50632, 0.49843, 0.49081, 0.48383,\
        0.47766, 0.47191, 0.46652, 0.46158, 0.45689, 0.45236, 0.44801, 0.44369, 0.43922, 0.43447, 0.42947,\
        0.42438, 0.41926, 0.41417, 0.40915, 0.40407, 0.39895, 0.39313, 0.38709, 0.38137, 0.37598, 0.37063,\
        0.36515, 0.35948, 0.35365, 0.34766, 0.34151, 0.33520, 0.32872, 0.32210, 0.31549, 0.30898, 0.30247],\
        [0.28454, 0.28960, 0.29429, 0.29868, 0.30298, 0.30725, 0.31134, 0.31529, 0.31918, 0.32310, 0.32714,\
        0.33135, 0.33577, 0.34038, 0.34514, 0.34999, 0.35490, 0.35983, 0.36479, 0.36984, 0.37509, 0.38065,\
        0.38662, 0.39295, 0.39915, 0.40407, 0.40859, 0.41351, 0.41859, 0.42345, 0.42831, 0.43312, 0.43765,\
        0.44189, 0.44600, 0.45005, 0.45390, 0.45738, 0.46066, 0.46416, 0.46799, 0.47205, 0.47653, 0.48173,\
        0.48798, 0.49493, 0.50277, 0.51124, 0.52018, 0.52967, 0.53989, 0.55124, 0.56351, 0.57550, 0.58752,\
        0.59870, 0.60947, 0.61893, 0.62778, 0.63516, 0.63917, 0.63912, 0.63578, 0.63131, 0.62657, 0.62073,\
        0.61424, 0.60905, 0.60572, 0.59808, 0.59289, 0.58639, 0.57923, 0.57215, 0.56469, 0.55698, 0.54931,\
        0.54168, 0.53385, 0.52590, 0.51821, 0.51128, 0.50440, 0.49666, 0.48959, 0.48231, 0.47527, 0.46873,\
        0.46279, 0.45734, 0.45231, 0.44767, 0.44329, 0.43906, 0.43496, 0.43097, 0.42697, 0.42275, 0.41831,\
        0.41382, 0.40923, 0.40456, 0.40000, 0.39528, 0.39038, 0.38511, 0.37958, 0.37412, 0.36882, 0.36359,\
        0.35830, 0.35289, 0.34735, 0.34165, 0.33579, 0.32975, 0.32350, 0.31706, 0.31058, 0.30453, 0.29871],\
        [0.28298, 0.28819, 0.29297, 0.29737, 0.30154, 0.30544, 0.30911, 0.31258, 0.31595, 0.31934, 0.32289,\
        0.32668, 0.33073, 0.33501, 0.33948, 0.34409, 0.34878, 0.35352, 0.35834, 0.36328, 0.36839, 0.37376,\
        0.37937, 0.38509, 0.39045, 0.39493, 0.39897, 0.40366, 0.40831, 0.41207, 0.41620, 0.42033, 0.42400,\
        0.42733, 0.43055, 0.43379, 0.43697, 0.44000, 0.44314, 0.44677, 0.45095, 0.45562, 0.46080, 0.46666,\
        0.47328, 0.48064, 0.48875, 0.49735, 0.50669, 0.51620, 0.52603, 0.53650, 0.54746, 0.55859, 0.56971,\
        0.58052, 0.59093, 0.59996, 0.60964, 0.61778, 0.62169, 0.62149, 0.61716, 0.61264, 0.60971, 0.60419,\
        0.59816, 0.59314, 0.59062, 0.58453, 0.57782, 0.57113, 0.56406, 0.55680, 0.54929, 0.54159, 0.53390,\
        0.52612, 0.51773, 0.50920, 0.50109, 0.49431, 0.48800, 0.48092, 0.47403, 0.46726, 0.46070, 0.45451,\
        0.44879, 0.44354, 0.43875, 0.43440, 0.43033, 0.42635, 0.42242, 0.41873, 0.41527, 0.41156, 0.40744,\
        0.40350, 0.39941, 0.39509, 0.39082, 0.38644, 0.38183, 0.37696, 0.37190, 0.36679, 0.36172, 0.35668,\
        0.35162, 0.34648, 0.34124, 0.33588, 0.33037, 0.32468, 0.31878, 0.31262, 0.30619, 0.29991, 0.29462],\
        [0.28149, 0.28694, 0.29190, 0.29637, 0.30034, 0.30391, 0.30716, 0.31006, 0.31281, 0.31560, 0.31863,\
        0.32199, 0.32570, 0.32970, 0.33393, 0.33831, 0.34279, 0.34735, 0.35200, 0.35675, 0.36164, 0.36668,\
        0.37183, 0.37692, 0.38166, 0.38589, 0.38992, 0.39437, 0.39813, 0.40081, 0.40465, 0.40829, 0.41103,\
        0.41349, 0.41585, 0.41848, 0.42125, 0.42372, 0.42661, 0.43043, 0.43509, 0.44040, 0.44624, 0.45265,\
        0.45967, 0.46732, 0.47558, 0.48442, 0.49380, 0.50302, 0.51242, 0.52213, 0.53209, 0.54248, 0.55307,\
        0.56354, 0.57368, 0.58348, 0.59380, 0.60160, 0.60537, 0.60608, 0.60123, 0.59569, 0.59383, 0.58860,\
        0.58298, 0.57819, 0.57409, 0.56874, 0.56233, 0.55559, 0.54861, 0.54143, 0.53398, 0.52625, 0.51862,\
        0.51137, 0.50293, 0.49405, 0.48611, 0.47913, 0.47270, 0.46618, 0.45967, 0.45328, 0.44709, 0.44120,\
        0.43567, 0.43054, 0.42591, 0.42187, 0.41817, 0.41445, 0.41054, 0.40702, 0.40415, 0.40111, 0.39706,\
        0.39346, 0.38975, 0.38584, 0.38183, 0.37771, 0.37340, 0.36888, 0.36420, 0.35944, 0.35466, 0.34988,\
        0.34509, 0.34024, 0.33533, 0.33031, 0.32518, 0.31992, 0.31450, 0.30884, 0.30282, 0.29667, 0.29093],\
        [0.27993, 0.28570, 0.29094, 0.29552, 0.29931, 0.30271, 0.30548, 0.30768, 0.30969, 0.31183, 0.31432,\
        0.31729, 0.32070, 0.32447, 0.32849, 0.33267, 0.33695, 0.34131, 0.34573, 0.35024, 0.35484, 0.35951,\
        0.36419, 0.36876, 0.37306, 0.37706, 0.38089, 0.38464, 0.38780, 0.39055, 0.39377, 0.39682, 0.39873,\
        0.40063, 0.40225, 0.40450, 0.40732, 0.40880, 0.41145, 0.41551, 0.42072, 0.42659, 0.43293, 0.43974,\
        0.44704, 0.45483, 0.46312, 0.47185, 0.48087, 0.48992, 0.49906, 0.50820, 0.51714, 0.52717, 0.53737,\
        0.54749, 0.55759, 0.56764, 0.57722, 0.58451, 0.58870, 0.58984, 0.58646, 0.58141, 0.57748, 0.57276,\
        0.56756, 0.56263, 0.55783, 0.55245, 0.54636, 0.53982, 0.53302, 0.52612, 0.51894, 0.51123, 0.50328,\
        0.49650, 0.48888, 0.48022, 0.47234, 0.46530, 0.45883, 0.45258, 0.44642, 0.44035, 0.43446, 0.42885,\
        0.42353, 0.41843, 0.41383, 0.41011, 0.40685, 0.40350, 0.39970, 0.39616, 0.39356, 0.39076, 0.38736,\
        0.38389, 0.38041, 0.37680, 0.37306, 0.36919, 0.36515, 0.36094, 0.35660, 0.35216, 0.34769, 0.34320,\
        0.33869, 0.33415, 0.32957, 0.32491, 0.32017, 0.31533, 0.31039, 0.30531, 0.29990, 0.29382, 0.28775],\
        [0.27810, 0.28424, 0.28991, 0.29492, 0.29902, 0.30210, 0.30402, 0.30529, 0.30649, 0.30794, 0.30994,\
        0.31258, 0.31577, 0.31937, 0.32322, 0.32721, 0.33127, 0.33539, 0.33956, 0.34379, 0.34806, 0.35235,\
        0.35660, 0.36073, 0.36465, 0.36834, 0.37182, 0.37506, 0.37794, 0.38059, 0.38322, 0.38555, 0.38730,\
        0.38884, 0.39029, 0.39240, 0.39484, 0.39581, 0.39849, 0.40243, 0.40824, 0.41435, 0.42092, 0.42798,\
        0.43536, 0.44312, 0.45125, 0.45969, 0.46832, 0.47706, 0.48587, 0.49434, 0.50267, 0.51305, 0.52246,\
        0.53206, 0.54187, 0.55156, 0.56035, 0.56716, 0.57129, 0.57233, 0.57013, 0.56602, 0.56149, 0.55666,\
        0.55166, 0.54672, 0.54170, 0.53624, 0.53032, 0.52408, 0.51742, 0.51096, 0.50416, 0.49686, 0.48892,\
        0.48172, 0.47437, 0.46662, 0.45918, 0.45236, 0.44605, 0.44006, 0.43423, 0.42847, 0.42280, 0.41746,\
        0.41248, 0.40745, 0.40265, 0.39920, 0.39628, 0.39314, 0.38977, 0.38650, 0.38372, 0.38095, 0.37793,\
        0.37474, 0.37146, 0.36808, 0.36457, 0.36093, 0.35714, 0.35322, 0.34916, 0.34503, 0.34084, 0.33663,\
        0.33242, 0.32820, 0.32395, 0.31965, 0.31524, 0.31075, 0.30618, 0.30137, 0.29624, 0.29049, 0.28448],\
        [0.27576, 0.28223, 0.28833, 0.29393, 0.29870, 0.30149, 0.30222, 0.30256, 0.30311, 0.30392, 0.30552,\
        0.30793, 0.31097, 0.31447, 0.31818, 0.32196, 0.32578, 0.32962, 0.33350, 0.33742, 0.34136, 0.34529,\
        0.34915, 0.35289, 0.35645, 0.35981, 0.36296, 0.36585, 0.36848, 0.37089, 0.37312, 0.37508, 0.37670,\
        0.37816, 0.37967, 0.38151, 0.38340, 0.38517, 0.38788, 0.39208, 0.39769, 0.40353, 0.41027, 0.41739,\
        0.42459, 0.43216, 0.44001, 0.44804, 0.45621, 0.46453, 0.47292, 0.48133, 0.49049, 0.49975, 0.50793,\
        0.51708, 0.52648, 0.53571, 0.54384, 0.55010, 0.55394, 0.55501, 0.55338, 0.54983, 0.54530, 0.54039,\
        0.53547, 0.53069, 0.52579, 0.52032, 0.51444, 0.50887, 0.50216, 0.49590, 0.48953, 0.48241, 0.47495,\
        0.46763, 0.46042, 0.45327, 0.44642, 0.44005, 0.43410, 0.42846, 0.42304, 0.41765, 0.41209, 0.40686,\
        0.40245, 0.39780, 0.39306, 0.38946, 0.38646, 0.38347, 0.38040, 0.37740, 0.37459, 0.37184, 0.36898,\
        0.36600, 0.36291, 0.35972, 0.35642, 0.35299, 0.34943, 0.34575, 0.34195, 0.33808, 0.33416, 0.33022,\
        0.32629, 0.32237, 0.31847, 0.31453, 0.31042, 0.30613, 0.30194, 0.29726, 0.29224, 0.28676, 0.28096],\
        [0.27271, 0.27935, 0.28565, 0.29143, 0.29625, 0.29888, 0.29927, 0.29927, 0.29954, 0.29989, 0.30131,\
        0.30348, 0.30643, 0.30988, 0.31343, 0.31696, 0.32047, 0.32400, 0.32758, 0.33118, 0.33479, 0.33837,\
        0.34188, 0.34527, 0.34850, 0.35156, 0.35441, 0.35704, 0.35944, 0.36164, 0.36363, 0.36541, 0.36697,\
        0.36845, 0.37000, 0.37173, 0.37365, 0.37588, 0.37892, 0.38312, 0.38835, 0.39422, 0.40119, 0.40780,\
        0.41455, 0.42195, 0.42945, 0.43693, 0.44456, 0.45242, 0.46053, 0.46891, 0.47762, 0.48601, 0.49423,\
        0.50284, 0.51171, 0.52068, 0.52805, 0.53373, 0.53727, 0.53831, 0.53689, 0.53357, 0.52905, 0.52405,\
        0.51920, 0.51470, 0.51028, 0.50499, 0.49894, 0.49387, 0.48788, 0.48153, 0.47519, 0.46836, 0.46121,\
        0.45404, 0.44705, 0.44036, 0.43412, 0.42832, 0.42287, 0.41762, 0.41268, 0.40792, 0.40256, 0.39720,\
        0.39300, 0.38868, 0.38443, 0.38074, 0.37757, 0.37460, 0.37168, 0.36883, 0.36607, 0.36334, 0.36058,\
        0.35773, 0.35479, 0.35176, 0.34863, 0.34540, 0.34204, 0.33857, 0.33501, 0.33136, 0.32768, 0.32399,\
        0.32032, 0.31668, 0.31311, 0.30961, 0.30587, 0.30158, 0.29746, 0.29309, 0.28818, 0.28288, 0.27726],\
        [0.26882, 0.27544, 0.28164, 0.28717, 0.29156, 0.29418, 0.29517, 0.29548, 0.29575, 0.29627, 0.29752,\
        0.29938, 0.30235, 0.30572, 0.30902, 0.31217, 0.31532, 0.31852, 0.32179, 0.32509, 0.32840, 0.33165,\
        0.33484, 0.33791, 0.34085, 0.34363, 0.34624, 0.34865, 0.35088, 0.35293, 0.35479, 0.35649, 0.35807,\
        0.35961, 0.36123, 0.36303, 0.36511, 0.36765, 0.37092, 0.37510, 0.38018, 0.38606, 0.39252, 0.39853,\
        0.40520, 0.41264, 0.41958, 0.42631, 0.43330, 0.44073, 0.44853, 0.45665, 0.46495, 0.47319, 0.48140,\
        0.48972, 0.49851, 0.50710, 0.51296, 0.51829, 0.52151, 0.52249, 0.52102, 0.51771, 0.51316, 0.50799,\
        0.50319, 0.49895, 0.49489, 0.49006, 0.48454, 0.47926, 0.47374, 0.46779, 0.46155, 0.45492, 0.44798,\
        0.44099, 0.43422, 0.42795, 0.42231, 0.41717, 0.41234, 0.40748, 0.40267, 0.39900, 0.39385, 0.38882,\
        0.38437, 0.38022, 0.37627, 0.37268, 0.36945, 0.36646, 0.36360, 0.36082, 0.35811, 0.35541, 0.35270,\
        0.34994, 0.34711, 0.34422, 0.34124, 0.33818, 0.33502, 0.33174, 0.32837, 0.32493, 0.32145, 0.31799,\
        0.31455, 0.31114, 0.30778, 0.30479, 0.30170, 0.29762, 0.29330, 0.28887, 0.28407, 0.27891, 0.27344],\
        [0.26414, 0.27059, 0.27656, 0.28177, 0.28591, 0.28868, 0.29024, 0.29109, 0.29175, 0.29260, 0.29395,\
        0.29597, 0.29876, 0.30195, 0.30484, 0.30749, 0.31028, 0.31315, 0.31613, 0.31917, 0.32220, 0.32516,\
        0.32805, 0.33084, 0.33352, 0.33607, 0.33847, 0.34072, 0.34282, 0.34477, 0.34659, 0.34829, 0.34992,\
        0.35155, 0.35329, 0.35523, 0.35749, 0.36023, 0.36361, 0.36776, 0.37267, 0.37825, 0.38421, 0.39032,\
        0.39721, 0.40415, 0.41020, 0.41596, 0.42236, 0.42942, 0.43692, 0.44476, 0.45284, 0.46102, 0.46927,\
        0.47769, 0.48634, 0.49337, 0.49869, 0.50407, 0.50648, 0.50776, 0.50578, 0.50252, 0.49797, 0.49289,\
        0.48811, 0.48389, 0.47976, 0.47525, 0.47035, 0.46534, 0.46016, 0.45463, 0.44867, 0.44226, 0.43550,\
        0.42864, 0.42203, 0.41606, 0.41108, 0.40657, 0.40232, 0.39819, 0.39356, 0.38964, 0.38527, 0.38071,\
        0.37639, 0.37237, 0.36861, 0.36514, 0.36194, 0.35896, 0.35612, 0.35338, 0.35069, 0.34802, 0.34534,\
        0.34263, 0.33989, 0.33710, 0.33426, 0.33137, 0.32838, 0.32528, 0.32208, 0.31880, 0.31551, 0.31225,\
        0.30904, 0.30590, 0.30261, 0.29967, 0.29722, 0.29342, 0.28917, 0.28470, 0.27996, 0.27490, 0.26955],\
        [0.25880, 0.26503, 0.27074, 0.27573, 0.27978, 0.28277, 0.28482, 0.28624, 0.28740, 0.28867, 0.29029,\
        0.29244, 0.29514, 0.29814, 0.30044, 0.30275, 0.30529, 0.30786, 0.31061, 0.31345, 0.31623, 0.31891,\
        0.32153, 0.32408, 0.32653, 0.32888, 0.33113, 0.33325, 0.33527, 0.33718, 0.33899, 0.34073, 0.34245,\
        0.34420, 0.34607, 0.34815, 0.35056, 0.35342, 0.35684, 0.36091, 0.36562, 0.37089, 0.37658, 0.38265,\
        0.38913, 0.39547, 0.40077, 0.40564, 0.41180, 0.41861, 0.42579, 0.43336, 0.44126, 0.44935, 0.45751,\
        0.46562, 0.47337, 0.47995, 0.48549, 0.48988, 0.49222, 0.49343, 0.49126, 0.48786, 0.48348, 0.47864,\
        0.47391, 0.46951, 0.46528, 0.46096, 0.45649, 0.45194, 0.44723, 0.44219, 0.43664, 0.43052, 0.42394,\
        0.41718, 0.41072, 0.40485, 0.40054, 0.39661, 0.39272, 0.38890, 0.38489, 0.38096, 0.37695, 0.37283,\
        0.36882, 0.36501, 0.36142, 0.35806, 0.35493, 0.35199, 0.34918, 0.34646, 0.34379, 0.34114, 0.33848,\
        0.33580, 0.33311, 0.33040, 0.32770, 0.32496, 0.32216, 0.31923, 0.31618, 0.31302, 0.30988, 0.30682,\
        0.30377, 0.30094, 0.29804, 0.29503, 0.29224, 0.28882, 0.28483, 0.28047, 0.27581, 0.27085, 0.26560],\
        [0.25295, 0.25896, 0.26447, 0.26933, 0.27341, 0.27664, 0.27911, 0.28105, 0.28272, 0.28438, 0.28627,\
        0.28849, 0.29101, 0.29356, 0.29580, 0.29802, 0.30037, 0.30262, 0.30524, 0.30799, 0.31052, 0.31291,\
        0.31528, 0.31763, 0.31990, 0.32209, 0.32420, 0.32624, 0.32822, 0.33012, 0.33197, 0.33379, 0.33561,\
        0.33748, 0.33947, 0.34167, 0.34418, 0.34709, 0.35050, 0.35445, 0.35893, 0.36390, 0.36928, 0.37497,\
        0.38080, 0.38637, 0.39124, 0.39607, 0.40205, 0.40848, 0.41522, 0.42245, 0.43021, 0.43814, 0.44600,\
        0.45361, 0.46070, 0.46690, 0.47208, 0.47593, 0.47819, 0.47869, 0.47698, 0.47370, 0.46950, 0.46485,\
        0.46017, 0.45569, 0.45141, 0.44724, 0.44311, 0.43903, 0.43492, 0.43052, 0.42551, 0.41976, 0.41342,\
        0.40683, 0.40050, 0.39526, 0.39089, 0.38726, 0.38367, 0.38010, 0.37647, 0.37279, 0.36907, 0.36531,\
        0.36161, 0.35804, 0.35464, 0.35142, 0.34838, 0.34550, 0.34273, 0.34004, 0.33739, 0.33475, 0.33210,\
        0.32944, 0.32676, 0.32412, 0.32153, 0.31898, 0.31637, 0.31363, 0.31071, 0.30765, 0.30456, 0.30179,\
        0.29890, 0.29613, 0.29342, 0.29054, 0.28755, 0.28420, 0.28038, 0.27617, 0.27163, 0.26677, 0.26163],\
        [0.24676, 0.25258, 0.25795, 0.26276, 0.26692, 0.27039, 0.27325, 0.27563, 0.27773, 0.27976, 0.28186,\
        0.28412, 0.28650, 0.28886, 0.29108, 0.29325, 0.29541, 0.29758, 0.30020, 0.30281, 0.30503, 0.30711,\
        0.30930, 0.31154, 0.31364, 0.31567, 0.31768, 0.31968, 0.32164, 0.32357, 0.32548, 0.32739, 0.32932,\
        0.33130, 0.33340, 0.33569, 0.33825, 0.34117, 0.34451, 0.34831, 0.35256, 0.35722, 0.36221, 0.36743,\
        0.37269, 0.37776, 0.38257, 0.38753, 0.39307, 0.39910, 0.40518, 0.41212, 0.41981, 0.42748, 0.43488,\
        0.44192, 0.44843, 0.45417, 0.45892, 0.46241, 0.46439, 0.46462, 0.46305, 0.45999, 0.45597, 0.45145,\
        0.44682, 0.44234, 0.43810, 0.43407, 0.43022, 0.42659, 0.42317, 0.41959, 0.41525, 0.40990, 0.40390,\
        0.39765, 0.39175, 0.38668, 0.38240, 0.37872, 0.37527, 0.37189, 0.36851, 0.36509, 0.36165, 0.35820,\
        0.35478, 0.35145, 0.34824, 0.34517, 0.34224, 0.33944, 0.33672, 0.33408, 0.33146, 0.32885, 0.32621,\
        0.32354, 0.32085, 0.31823, 0.31574, 0.31339, 0.31102, 0.30849, 0.30572, 0.30277, 0.29971, 0.29699,\
        0.29436, 0.29165, 0.28893, 0.28609, 0.28304, 0.27970, 0.27597, 0.27188, 0.26744, 0.26268, 0.25763],\
        [0.24037, 0.24604, 0.25133, 0.25615, 0.26042, 0.26413, 0.26731, 0.27007, 0.27254, 0.27486, 0.27716,\
        0.27948, 0.28181, 0.28409, 0.28630, 0.28845, 0.29060, 0.29284, 0.29529, 0.29764, 0.29969, 0.30146,\
        0.30364, 0.30588, 0.30774, 0.30960, 0.31157, 0.31357, 0.31554, 0.31749, 0.31947, 0.32149, 0.32354,\
        0.32563, 0.32780, 0.33013, 0.33271, 0.33559, 0.33883, 0.34246, 0.34646, 0.35081, 0.35543, 0.36022,\
        0.36506, 0.36983, 0.37456, 0.37945, 0.38470, 0.39023, 0.39592, 0.40283, 0.41031, 0.41748, 0.42424,\
        0.43066, 0.43659, 0.44185, 0.44624, 0.44947, 0.45123, 0.45134, 0.44979, 0.44685, 0.44294, 0.43850,\
        0.43393, 0.42948, 0.42535, 0.42152, 0.41788, 0.41456, 0.41184, 0.40922, 0.40566, 0.40065, 0.39499,\
        0.38920, 0.38371, 0.37883, 0.37461, 0.37090, 0.36751, 0.36426, 0.36106, 0.35788, 0.35468, 0.35149,\
        0.34833, 0.34523, 0.34221, 0.33929, 0.33648, 0.33377, 0.33113, 0.32854, 0.32598, 0.32341, 0.32080,\
        0.31812, 0.31540, 0.31272, 0.31028, 0.30817, 0.30610, 0.30376, 0.30114, 0.29832, 0.29543, 0.29268,\
        0.29004, 0.28737, 0.28464, 0.28177, 0.27869, 0.27534, 0.27166, 0.26763, 0.26328, 0.25860, 0.25363],\
        [0.23392, 0.23947, 0.24472, 0.24959, 0.25399, 0.25792, 0.26138, 0.26446, 0.26723, 0.26980, 0.27227,\
        0.27467, 0.27701, 0.27929, 0.28150, 0.28367, 0.28583, 0.28804, 0.29030, 0.29247, 0.29442, 0.29625,\
        0.29849, 0.30062, 0.30209, 0.30388, 0.30588, 0.30791, 0.30988, 0.31183, 0.31391, 0.31607, 0.31823,\
        0.32039, 0.32260, 0.32494, 0.32748, 0.33030, 0.33342, 0.33687, 0.34064, 0.34469, 0.34898, 0.35342,\
        0.35793, 0.36248, 0.36709, 0.37187, 0.37691, 0.38226, 0.38807, 0.39466, 0.40171, 0.40802, 0.41401,\
        0.41986, 0.42525, 0.43004, 0.43417, 0.43726, 0.43892, 0.43894, 0.43736, 0.43442, 0.43053, 0.42612,\
        0.42159, 0.41720, 0.41323, 0.40974, 0.40633, 0.40300, 0.40089, 0.39895, 0.39604, 0.39151, 0.38632,\
        0.38104, 0.37598, 0.37137, 0.36727, 0.36362, 0.36029, 0.35715, 0.35411, 0.35112, 0.34815, 0.34519,\
        0.34226, 0.33936, 0.33652, 0.33375, 0.33106, 0.32845, 0.32590, 0.32340, 0.32090, 0.31840, 0.31586,\
        0.31322, 0.31045, 0.30764, 0.30510, 0.30327, 0.30158, 0.29932, 0.29681, 0.29412, 0.29136, 0.28865,\
        0.28597, 0.28328, 0.28051, 0.27760, 0.27448, 0.27111, 0.26745, 0.26347, 0.25917, 0.25455, 0.24964],\
        [0.22750, 0.23298, 0.23824, 0.24318, 0.24773, 0.25186, 0.25556, 0.25889, 0.26190, 0.26468, 0.26730,\
        0.26979, 0.27218, 0.27448, 0.27672, 0.27890, 0.28106, 0.28322, 0.28536, 0.28743, 0.28940, 0.29135,\
        0.29342, 0.29528, 0.29683, 0.29862, 0.30063, 0.30272, 0.30463, 0.30653, 0.30877, 0.31109, 0.31334,\
        0.31553, 0.31773, 0.32005, 0.32254, 0.32525, 0.32824, 0.33152, 0.33507, 0.33887, 0.34287, 0.34702,\
        0.35128, 0.35563, 0.36010, 0.36476, 0.36967, 0.37492, 0.38061, 0.38678, 0.39307, 0.39866, 0.40416,\
        0.40965, 0.41445, 0.41874, 0.42278, 0.42591, 0.42755, 0.42748, 0.42579, 0.42278, 0.41884, 0.41440,\
        0.41002, 0.40571, 0.40184, 0.39879, 0.39573, 0.39274, 0.39061, 0.38867, 0.38598, 0.38217, 0.37768,\
        0.37299, 0.36840, 0.36413, 0.36024, 0.35672, 0.35349, 0.35048, 0.34760, 0.34479, 0.34202, 0.33927,\
        0.33654, 0.33382, 0.33115, 0.32852, 0.32596, 0.32346, 0.32101, 0.31859, 0.31619, 0.31378, 0.31136,\
        0.30883, 0.30612, 0.30323, 0.30039, 0.29866, 0.29717, 0.29505, 0.29265, 0.29008, 0.28743, 0.28476,\
        0.28208, 0.27935, 0.27653, 0.27357, 0.27041, 0.26702, 0.26335, 0.25938, 0.25512, 0.25055, 0.24568],\
        [0.22125, 0.22669, 0.23199, 0.23704, 0.24173, 0.24603, 0.24992, 0.25345, 0.25665, 0.25959, 0.26233,\
        0.26490, 0.26735, 0.26969, 0.27195, 0.27415, 0.27631, 0.27843, 0.28052, 0.28255, 0.28452, 0.28648,\
        0.28841, 0.29023, 0.29195, 0.29378, 0.29578, 0.29790, 0.29969, 0.30158, 0.30408, 0.30657, 0.30884,\
        0.31098, 0.31314, 0.31540, 0.31781, 0.32042, 0.32328, 0.32639, 0.32975, 0.33332, 0.33708, 0.34099,\
        0.34502, 0.34919, 0.35352, 0.35804, 0.36281, 0.36789, 0.37329, 0.37893, 0.38453, 0.38982, 0.39502,\
        0.40012, 0.40422, 0.40788, 0.41223, 0.41550, 0.41710, 0.41691, 0.41503, 0.41193, 0.40799, 0.40349,\
        0.39926, 0.39537, 0.39164, 0.38864, 0.38585, 0.38326, 0.38104, 0.37889, 0.37630, 0.37299, 0.36914,\
        0.36503, 0.36094, 0.35703, 0.35341, 0.35008, 0.34702, 0.34417, 0.34145, 0.33883, 0.33625, 0.33369,\
        0.33114, 0.32860, 0.32607, 0.32357, 0.32112, 0.31873, 0.31640, 0.31409, 0.31177, 0.30946, 0.30719,\
        0.30487, 0.30233, 0.29955, 0.29681, 0.29472, 0.29294, 0.29092, 0.28863, 0.28618, 0.28361, 0.28098,\
        0.27830, 0.27554, 0.27268, 0.26967, 0.26647, 0.26304, 0.25935, 0.25538, 0.25113, 0.24658, 0.24174],\
        [0.21527, 0.22072, 0.22610, 0.23126, 0.23608, 0.24052, 0.24455, 0.24821, 0.25154, 0.25460, 0.25743,\
        0.26007, 0.26257, 0.26495, 0.26723, 0.26944, 0.27160, 0.27371, 0.27578, 0.27779, 0.27976, 0.28170,\
        0.28360, 0.28545, 0.28727, 0.28916, 0.29116, 0.29319, 0.29512, 0.29728, 0.29986, 0.30245, 0.30456,\
        0.30662, 0.30873, 0.31092, 0.31325, 0.31577, 0.31850, 0.32147, 0.32466, 0.32804, 0.33158, 0.33527,\
        0.33910, 0.34309, 0.34725, 0.35162, 0.35622, 0.36106, 0.36612, 0.37131, 0.37646, 0.38145, 0.38630,\
        0.39082, 0.39453, 0.39813, 0.40285, 0.40584, 0.40732, 0.40702, 0.40488, 0.40169, 0.39792, 0.39365,\
        0.38953, 0.38581, 0.38237, 0.37937, 0.37669, 0.37424, 0.37198, 0.36973, 0.36720, 0.36422, 0.36085,\
        0.35725, 0.35361, 0.35008, 0.34675, 0.34366, 0.34081, 0.33814, 0.33561, 0.33318, 0.33079, 0.32842,\
        0.32605, 0.32366, 0.32127, 0.31887, 0.31652, 0.31424, 0.31203, 0.30986, 0.30760, 0.30533, 0.30320,\
        0.30109, 0.29873, 0.29606, 0.29346, 0.29118, 0.28913, 0.28704, 0.28479, 0.28240, 0.27990, 0.27730,\
        0.27462, 0.27183, 0.26893, 0.26587, 0.26262, 0.25915, 0.25544, 0.25146, 0.24720, 0.24266, 0.23783],\
        [0.20971, 0.21522, 0.22072, 0.22599, 0.23089, 0.23540, 0.23950, 0.24324, 0.24664, 0.24976, 0.25265,\
        0.25534, 0.25787, 0.26028, 0.26258, 0.26480, 0.26696, 0.26907, 0.27113, 0.27314, 0.27512, 0.27706,\
        0.27897, 0.28086, 0.28276, 0.28470, 0.28671, 0.28878, 0.29090, 0.29322, 0.29579, 0.29828, 0.30025,\
        0.30235, 0.30444, 0.30656, 0.30884, 0.31125, 0.31387, 0.31673, 0.31979, 0.32301, 0.32635, 0.32983,\
        0.33346, 0.33726, 0.34125, 0.34544, 0.34983, 0.35442, 0.35915, 0.36396, 0.36873, 0.37339, 0.37786,\
        0.38201, 0.38580, 0.38969, 0.39356, 0.39625, 0.39757, 0.39723, 0.39506, 0.39193, 0.38833, 0.38441,\
        0.38054, 0.37695, 0.37367, 0.37073, 0.36810, 0.36568, 0.36341, 0.36112, 0.35866, 0.35591, 0.35290,\
        0.34970, 0.34646, 0.34328, 0.34025, 0.33742, 0.33479, 0.33234, 0.33003, 0.32779, 0.32560, 0.32341,\
        0.32121, 0.31898, 0.31670, 0.31439, 0.31210, 0.30989, 0.30784, 0.30587, 0.30374, 0.30131, 0.29931,\
        0.29725, 0.29500, 0.29253, 0.29007, 0.28775, 0.28556, 0.28338, 0.28112, 0.27876, 0.27628, 0.27370,\
        0.27101, 0.26821, 0.26526, 0.26215, 0.25886, 0.25534, 0.25160, 0.24759, 0.24332, 0.23878, 0.23394],\
        [0.20478, 0.21043, 0.21604, 0.22134, 0.22624, 0.23074, 0.23484, 0.23858, 0.24199, 0.24513, 0.24803,\
        0.25074, 0.25328, 0.25570, 0.25801, 0.26025, 0.26241, 0.26452, 0.26658, 0.26860, 0.27059, 0.27255,\
        0.27449, 0.27642, 0.27837, 0.28036, 0.28241, 0.28453, 0.28675, 0.28910, 0.29154, 0.29387, 0.29600,\
        0.29813, 0.30020, 0.30227, 0.30456, 0.30684, 0.30935, 0.31216, 0.31513, 0.31820, 0.32135, 0.32461,\
        0.32804, 0.33165, 0.33545, 0.33945, 0.34363, 0.34796, 0.35240, 0.35686, 0.36129, 0.36561, 0.36975,\
        0.37366, 0.37736, 0.38093, 0.38413, 0.38642, 0.38747, 0.38706, 0.38523, 0.38242, 0.37909, 0.37552,\
        0.37193, 0.36853, 0.36539, 0.36254, 0.35995, 0.35756, 0.35529, 0.35301, 0.35063, 0.34806, 0.34532,\
        0.34244, 0.33952, 0.33665, 0.33390, 0.33133, 0.32894, 0.32673, 0.32465, 0.32263, 0.32063, 0.31862,\
        0.31659, 0.31451, 0.31235, 0.31011, 0.30783, 0.30565, 0.30371, 0.30196, 0.30022, 0.29778, 0.29557,\
        0.29349, 0.29129, 0.28897, 0.28662, 0.28432, 0.28208, 0.27985, 0.27758, 0.27521, 0.27275, 0.27017,\
        0.26747, 0.26464, 0.26166, 0.25851, 0.25516, 0.25160, 0.24781, 0.24378, 0.23949, 0.23492, 0.23008],\
        [0.20090, 0.20668, 0.21222, 0.21739, 0.22216, 0.22655, 0.23057, 0.23424, 0.23761, 0.24071, 0.24359,\
        0.24628, 0.24882, 0.25123, 0.25355, 0.25578, 0.25795, 0.26007, 0.26214, 0.26417, 0.26617, 0.26815,\
        0.27012, 0.27209, 0.27409, 0.27612, 0.27821, 0.28036, 0.28260, 0.28492, 0.28726, 0.28955, 0.29173,\
        0.29387, 0.29595, 0.29812, 0.30040, 0.30245, 0.30494, 0.30778, 0.31070, 0.31360, 0.31653, 0.31957,\
        0.32278, 0.32620, 0.32983, 0.33364, 0.33760, 0.34169, 0.34584, 0.34999, 0.35409, 0.35807, 0.36190,\
        0.36553, 0.36894, 0.37209, 0.37475, 0.37662, 0.37742, 0.37702, 0.37547, 0.37305, 0.37008, 0.36685,\
        0.36357, 0.36039, 0.35741, 0.35467, 0.35215, 0.34981, 0.34757, 0.34534, 0.34305, 0.34064, 0.33810,\
        0.33547, 0.33282, 0.33021, 0.32771, 0.32538, 0.32323, 0.32127, 0.31944, 0.31765, 0.31584, 0.31399,\
        0.31213, 0.31022, 0.30819, 0.30601, 0.30375, 0.30150, 0.29962, 0.29797, 0.29623, 0.29413, 0.29194,\
        0.28979, 0.28762, 0.28539, 0.28313, 0.28088, 0.27864, 0.27640, 0.27411, 0.27174, 0.26928, 0.26670,\
        0.26399, 0.26113, 0.25811, 0.25491, 0.25152, 0.24791, 0.24408, 0.24000, 0.23568, 0.23109, 0.22623],\
        [0.19863, 0.20405, 0.20922, 0.21409, 0.21862, 0.22282, 0.22668, 0.23023, 0.23350, 0.23653, 0.23935,\
        0.24199, 0.24450, 0.24689, 0.24919, 0.25142, 0.25359, 0.25571, 0.25779, 0.25984, 0.26186, 0.26386,\
        0.26586, 0.26786, 0.26989, 0.27195, 0.27406, 0.27623, 0.27846, 0.28073, 0.28302, 0.28527, 0.28747,\
        0.28963, 0.29177, 0.29395, 0.29610, 0.29821, 0.30073, 0.30368, 0.30648, 0.30918, 0.31186, 0.31464,\
        0.31765, 0.32090, 0.32435, 0.32798, 0.33174, 0.33560, 0.33947, 0.34332, 0.34709, 0.35074, 0.35424,\
        0.35753, 0.36059, 0.36332, 0.36554, 0.36705, 0.36765, 0.36727, 0.36594, 0.36386, 0.36126, 0.35838,\
        0.35539, 0.35246, 0.34966, 0.34705, 0.34464, 0.34238, 0.34021, 0.33807, 0.33587, 0.33360, 0.33123,\
        0.32880, 0.32636, 0.32397, 0.32168, 0.31955, 0.31763, 0.31593, 0.31437, 0.31282, 0.31118, 0.30945,\
        0.30774, 0.30604, 0.30416, 0.30202, 0.29983, 0.29764, 0.29570, 0.29398, 0.29221, 0.29026, 0.28819,\
        0.28609, 0.28396, 0.28179, 0.27960, 0.27740, 0.27519, 0.27296, 0.27068, 0.26831, 0.26585, 0.26327,\
        0.26055, 0.25767, 0.25461, 0.25136, 0.24791, 0.24425, 0.24036, 0.23625, 0.23189, 0.22727, 0.22238],\
        [0.19669, 0.20191, 0.20671, 0.21125, 0.21551, 0.21947, 0.22314, 0.22652, 0.22965, 0.23256, 0.23529,\
        0.23786, 0.24031, 0.24267, 0.24495, 0.24717, 0.24933, 0.25146, 0.25354, 0.25560, 0.25764, 0.25966,\
        0.26168, 0.26371, 0.26575, 0.26783, 0.26995, 0.27212, 0.27433, 0.27657, 0.27881, 0.28104, 0.28324,\
        0.28542, 0.28759, 0.28978, 0.29197, 0.29427, 0.29690, 0.29977, 0.30234, 0.30487, 0.30728, 0.30977,\
        0.31261, 0.31572, 0.31901, 0.32247, 0.32605, 0.32969, 0.33330, 0.33685, 0.34029, 0.34360, 0.34674,\
        0.34968, 0.35236, 0.35470, 0.35656, 0.35779, 0.35825, 0.35789, 0.35674, 0.35494, 0.35266, 0.35009,\
        0.34739, 0.34470, 0.34210, 0.33965, 0.33737, 0.33523, 0.33317, 0.33114, 0.32906, 0.32691, 0.32468,\
        0.32241, 0.32015, 0.31793, 0.31581, 0.31384, 0.31210, 0.31066, 0.30941, 0.30814, 0.30664, 0.30495,\
        0.30328, 0.30183, 0.30015, 0.29804, 0.29589, 0.29380, 0.29185, 0.29004, 0.28823, 0.28634, 0.28436,\
        0.28232, 0.28026, 0.27816, 0.27603, 0.27389, 0.27172, 0.26951, 0.26725, 0.26490, 0.26245, 0.25986,\
        0.25713, 0.25423, 0.25114, 0.24784, 0.24434, 0.24061, 0.23667, 0.23250, 0.22809, 0.22344, 0.21852],\
        [0.19475, 0.19979, 0.20438, 0.20869, 0.21272, 0.21645, 0.21990, 0.22309, 0.22605, 0.22881, 0.23141,\
        0.23389, 0.23627, 0.23857, 0.24082, 0.24302, 0.24518, 0.24730, 0.24939, 0.25146, 0.25351, 0.25554,\
        0.25758, 0.25962, 0.26168, 0.26376, 0.26588, 0.26803, 0.27022, 0.27242, 0.27464, 0.27685, 0.27905,\
        0.28124, 0.28343, 0.28565, 0.28793, 0.29033, 0.29292, 0.29557, 0.29812, 0.30063, 0.30270, 0.30489,\
        0.30772, 0.31070, 0.31380, 0.31710, 0.32053, 0.32396, 0.32733, 0.33057, 0.33367, 0.33662, 0.33940,\
        0.34198, 0.34431, 0.34631, 0.34788, 0.34889, 0.34925, 0.34891, 0.34790, 0.34632, 0.34431, 0.34202,\
        0.33959, 0.33712, 0.33472, 0.33244, 0.33030, 0.32831, 0.32641, 0.32453, 0.32259, 0.32054, 0.31843,\
        0.31629, 0.31418, 0.31211, 0.31014, 0.30827, 0.30663, 0.30543, 0.30451, 0.30356, 0.30217, 0.30047,\
        0.29881, 0.29740, 0.29583, 0.29390, 0.29186, 0.28985, 0.28792, 0.28607, 0.28424, 0.28238, 0.28046,\
        0.27849, 0.27649, 0.27446, 0.27240, 0.27032, 0.26820, 0.26603, 0.26380, 0.26147, 0.25904, 0.25646,\
        0.25373, 0.25081, 0.24768, 0.24434, 0.24078, 0.23698, 0.23297, 0.22874, 0.22429, 0.21960, 0.21465],\
        [0.19282, 0.19767, 0.20219, 0.20637, 0.21019, 0.21369, 0.21692, 0.21989, 0.22266, 0.22525, 0.22770,\
        0.23006, 0.23236, 0.23460, 0.23680, 0.23898, 0.24113, 0.24324, 0.24534, 0.24741, 0.24946, 0.25150,\
        0.25354, 0.25558, 0.25764, 0.25972, 0.26183, 0.26396, 0.26612, 0.26830, 0.27049, 0.27268, 0.27487,\
        0.27707, 0.27928, 0.28153, 0.28384, 0.28625, 0.28876, 0.29130, 0.29378, 0.29615, 0.29802, 0.30027,\
        0.30312, 0.30585, 0.30870, 0.31188, 0.31520, 0.31845, 0.32155, 0.32447, 0.32722, 0.32981, 0.33222,\
        0.33445, 0.33646, 0.33817, 0.33951, 0.34037, 0.34067, 0.34036, 0.33946, 0.33804, 0.33625, 0.33419,\
        0.33199, 0.32974, 0.32752, 0.32540, 0.32343, 0.32161, 0.31991, 0.31821, 0.31643, 0.31449, 0.31244,\
        0.31042, 0.30846, 0.30651, 0.30470, 0.30295, 0.30126, 0.30029, 0.29957, 0.29885, 0.29761, 0.29599,\
        0.29441, 0.29291, 0.29133, 0.28956, 0.28767, 0.28575, 0.28387, 0.28203, 0.28020, 0.27837, 0.27650,\
        0.27459, 0.27265, 0.27069, 0.26870, 0.26667, 0.26461, 0.26249, 0.26030, 0.25802, 0.25561, 0.25306,\
        0.25033, 0.24740, 0.24425, 0.24086, 0.23723, 0.23336, 0.22926, 0.22496, 0.22045, 0.21574, 0.21076],\
        [0.19105, 0.19588, 0.20032, 0.20427, 0.20788, 0.21117, 0.21416, 0.21691, 0.21946, 0.22186, 0.22415,\
        0.22637, 0.22856, 0.23074, 0.23290, 0.23505, 0.23718, 0.23929, 0.24137, 0.24344, 0.24549, 0.24753,\
        0.24956, 0.25160, 0.25365, 0.25572, 0.25781, 0.25991, 0.26204, 0.26419, 0.26635, 0.26852, 0.27070,\
        0.27290, 0.27511, 0.27737, 0.27968, 0.28206, 0.28449, 0.28692, 0.28930, 0.29154, 0.29364, 0.29601,\
        0.29866, 0.30102, 0.30367, 0.30686, 0.31009, 0.31316, 0.31596, 0.31853, 0.32092, 0.32315, 0.32522,\
        0.32711, 0.32883, 0.33032, 0.33149, 0.33226, 0.33253, 0.33225, 0.33142, 0.33013, 0.32849, 0.32663,\
        0.32464, 0.32258, 0.32052, 0.31854, 0.31671, 0.31509, 0.31362, 0.31216, 0.31056, 0.30872, 0.30668,\
        0.30474, 0.30304, 0.30110, 0.29945, 0.29791, 0.29636, 0.29535, 0.29461, 0.29384, 0.29274, 0.29133,\
        0.28985, 0.28834, 0.28678, 0.28510, 0.28332, 0.28150, 0.27969, 0.27788, 0.27608, 0.27427, 0.27245,\
        0.27060, 0.26873, 0.26683, 0.26490, 0.26294, 0.26094, 0.25888, 0.25674, 0.25451, 0.25214, 0.24962,\
        0.24692, 0.24399, 0.24083, 0.23740, 0.23371, 0.22975, 0.22554, 0.22114, 0.21657, 0.21182, 0.20683],\
        [0.18940, 0.19418, 0.19849, 0.20235, 0.20579, 0.20885, 0.21161, 0.21413, 0.21644, 0.21862, 0.22073,\
        0.22280, 0.22489, 0.22699, 0.22910, 0.23122, 0.23334, 0.23543, 0.23750, 0.23956, 0.24159, 0.24362,\
        0.24564, 0.24766, 0.24969, 0.25174, 0.25380, 0.25588, 0.25798, 0.26009, 0.26222, 0.26437, 0.26653,\
        0.26871, 0.27091, 0.27316, 0.27545, 0.27778, 0.28013, 0.28248, 0.28479, 0.28703, 0.28925, 0.29159,\
        0.29400, 0.29631, 0.29903, 0.30209, 0.30521, 0.30809, 0.31052, 0.31271, 0.31476, 0.31666, 0.31839,\
        0.31997, 0.32143, 0.32275, 0.32383, 0.32456, 0.32485, 0.32461, 0.32383, 0.32259, 0.32105, 0.31937,\
        0.31758, 0.31569, 0.31375, 0.31188, 0.31016, 0.30871, 0.30751, 0.30630, 0.30495, 0.30322, 0.30119,\
        0.29919, 0.29775, 0.29599, 0.29436, 0.29291, 0.29158, 0.29053, 0.28968, 0.28881, 0.28776, 0.28651,\
        0.28513, 0.28367, 0.28215, 0.28054, 0.27885, 0.27712, 0.27537, 0.27361, 0.27185, 0.27008, 0.26830,\
        0.26651, 0.26469, 0.26285, 0.26099, 0.25909, 0.25716, 0.25517, 0.25310, 0.25092, 0.24862, 0.24615,\
        0.24348, 0.24058, 0.23741, 0.23396, 0.23021, 0.22616, 0.22183, 0.21728, 0.21260, 0.20785, 0.20286],\
        [0.18783, 0.19263, 0.19692, 0.20069, 0.20389, 0.20672, 0.20927, 0.21154, 0.21360, 0.21553, 0.21742,\
        0.21934, 0.22131, 0.22334, 0.22542, 0.22751, 0.22960, 0.23168, 0.23373, 0.23576, 0.23777, 0.23977,\
        0.24176, 0.24376, 0.24576, 0.24777, 0.24980, 0.25185, 0.25391, 0.25599, 0.25808, 0.26020, 0.26233,\
        0.26449, 0.26667, 0.26889, 0.27114, 0.27342, 0.27572, 0.27801, 0.28028, 0.28252, 0.28477, 0.28708,\
        0.28943, 0.29186, 0.29454, 0.29745, 0.30046, 0.30317, 0.30509, 0.30692, 0.30871, 0.31032, 0.31173,\
        0.31304, 0.31428, 0.31549, 0.31654, 0.31729, 0.31763, 0.31747, 0.31672, 0.31544, 0.31392, 0.31241,\
        0.31086, 0.30913, 0.30724, 0.30550, 0.30384, 0.30248, 0.30157, 0.30051, 0.29938, 0.29780, 0.29589,\
        0.29406, 0.29247, 0.29090, 0.28938, 0.28800, 0.28677, 0.28570, 0.28476, 0.28382, 0.28278, 0.28159,\
        0.28029, 0.27890, 0.27743, 0.27588, 0.27427, 0.27261, 0.27092, 0.26921, 0.26750, 0.26578, 0.26404,\
        0.26230, 0.26054, 0.25876, 0.25695, 0.25513, 0.25326, 0.25134, 0.24934, 0.24724, 0.24501, 0.24261,\
        0.24000, 0.23714, 0.23400, 0.23054, 0.22675, 0.22262, 0.21817, 0.21343, 0.20854, 0.20375, 0.19881],\
        [0.18631, 0.19118, 0.19546, 0.19908, 0.20211, 0.20481, 0.20715, 0.20917, 0.21093, 0.21258, 0.21422,\
        0.21596, 0.21783, 0.21980, 0.22185, 0.22392, 0.22599, 0.22803, 0.23005, 0.23204, 0.23401, 0.23597,\
        0.23792, 0.23988, 0.24185, 0.24382, 0.24581, 0.24782, 0.24984, 0.25188, 0.25393, 0.25601, 0.25811,\
        0.26023, 0.26239, 0.26457, 0.26678, 0.26901, 0.27126, 0.27351, 0.27575, 0.27799, 0.28024, 0.28253,\
        0.28489, 0.28735, 0.28997, 0.29275, 0.29556, 0.29792, 0.29948, 0.30114, 0.30278, 0.30413, 0.30528,\
        0.30634, 0.30739, 0.30857, 0.30968, 0.31045, 0.31088, 0.31084, 0.31017, 0.30873, 0.30708, 0.30576,\
        0.30448, 0.30300, 0.30101, 0.29944, 0.29792, 0.29659, 0.29572, 0.29476, 0.29364, 0.29223, 0.29057,\
        0.28889, 0.28732, 0.28584, 0.28442, 0.28310, 0.28191, 0.28083, 0.27983, 0.27884, 0.27778, 0.27663,\
        0.27538, 0.27404, 0.27262, 0.27113, 0.26957, 0.26798, 0.26634, 0.26469, 0.26302, 0.26134, 0.25965,\
        0.25795, 0.25624, 0.25452, 0.25278, 0.25101, 0.24921, 0.24737, 0.24545, 0.24344, 0.24130, 0.23899,\
        0.23646, 0.23366, 0.23057, 0.22713, 0.22334, 0.21917, 0.21463, 0.20975, 0.20457, 0.19945, 0.19457],\
        [0.18479, 0.18978, 0.19412, 0.19772, 0.20063, 0.20318, 0.20533, 0.20704, 0.20844, 0.20974, 0.21111,\
        0.21267, 0.21444, 0.21639, 0.21842, 0.22046, 0.22250, 0.22450, 0.22646, 0.22839, 0.23031, 0.23222,\
        0.23412, 0.23603, 0.23795, 0.23988, 0.24182, 0.24378, 0.24575, 0.24774, 0.24975, 0.25179, 0.25385,\
        0.25593, 0.25804, 0.26019, 0.26236, 0.26455, 0.26675, 0.26896, 0.27117, 0.27339, 0.27564, 0.27791,\
        0.28025, 0.28267, 0.28519, 0.28776, 0.29021, 0.29228, 0.29391, 0.29543, 0.29689, 0.29806, 0.29904,\
        0.29997, 0.30083, 0.30213, 0.30329, 0.30405, 0.30450, 0.30463, 0.30420, 0.30259, 0.30056, 0.29949,\
        0.29834, 0.29705, 0.29527, 0.29366, 0.29226, 0.29103, 0.29003, 0.28905, 0.28794, 0.28665, 0.28519,\
        0.28367, 0.28219, 0.28078, 0.27943, 0.27817, 0.27700, 0.27591, 0.27487, 0.27384, 0.27277, 0.27163,\
        0.27040, 0.26910, 0.26772, 0.26628, 0.26478, 0.26323, 0.26165, 0.26004, 0.25841, 0.25677, 0.25513,\
        0.25347, 0.25180, 0.25013, 0.24844, 0.24673, 0.24500, 0.24323, 0.24140, 0.23949, 0.23745, 0.23524,\
        0.23281, 0.23011, 0.22709, 0.22370, 0.21994, 0.21580, 0.21124, 0.20632, 0.20141, 0.19593, 0.19053],\
        [0.18319, 0.18841, 0.19295, 0.19667, 0.19959, 0.20203, 0.20390, 0.20516, 0.20610, 0.20699, 0.20805,\
        0.20944, 0.21117, 0.21312, 0.21514, 0.21717, 0.21914, 0.22107, 0.22296, 0.22482, 0.22666, 0.22850,\
        0.23034, 0.23219, 0.23406, 0.23594, 0.23783, 0.23973, 0.24165, 0.24359, 0.24554, 0.24753, 0.24954,\
        0.25158, 0.25365, 0.25575, 0.25787, 0.26002, 0.26218, 0.26435, 0.26654, 0.26873, 0.27095, 0.27320,\
        0.27549, 0.27783, 0.28019, 0.28253, 0.28473, 0.28664, 0.28826, 0.28970, 0.29100, 0.29210, 0.29305,\
        0.29394, 0.29489, 0.29620, 0.29728, 0.29814, 0.29838, 0.29856, 0.29831, 0.29694, 0.29495, 0.29360,\
        0.29240, 0.29109, 0.28959, 0.28809, 0.28674, 0.28554, 0.28447, 0.28344, 0.28234, 0.28112, 0.27979,\
        0.27841, 0.27703, 0.27569, 0.27440, 0.27318, 0.27203, 0.27093, 0.26987, 0.26881, 0.26772, 0.26658,\
        0.26537, 0.26410, 0.26275, 0.26134, 0.25988, 0.25837, 0.25683, 0.25526, 0.25367, 0.25206, 0.25045,\
        0.24883, 0.24721, 0.24557, 0.24393, 0.24228, 0.24061, 0.23891, 0.23716, 0.23535, 0.23342, 0.23133,\
        0.22903, 0.22644, 0.22351, 0.22019, 0.21648, 0.21241, 0.20796, 0.20298, 0.19774, 0.19242, 0.18661],\
        [0.18137, 0.18691, 0.19182, 0.19597, 0.19917, 0.20148, 0.20280, 0.20345, 0.20387, 0.20432, 0.20504,\
        0.20631, 0.20806, 0.21006, 0.21208, 0.21404, 0.21593, 0.21775, 0.21954, 0.22129, 0.22304, 0.22479,\
        0.22656, 0.22835, 0.23016, 0.23198, 0.23381, 0.23566, 0.23751, 0.23939, 0.24129, 0.24322, 0.24517,\
        0.24716, 0.24918, 0.25124, 0.25332, 0.25542, 0.25754, 0.25968, 0.26183, 0.26399, 0.26617, 0.26836,\
        0.27058, 0.27282, 0.27504, 0.27719, 0.27919, 0.28099, 0.28256, 0.28394, 0.28517, 0.28624, 0.28721,\
        0.28815, 0.28916, 0.29029, 0.29129, 0.29201, 0.29235, 0.29241, 0.29203, 0.29092, 0.28938, 0.28794,\
        0.28664, 0.28533, 0.28394, 0.28257, 0.28127, 0.28008, 0.27898, 0.27791, 0.27681, 0.27564, 0.27440,\
        0.27311, 0.27182, 0.27055, 0.26932, 0.26813, 0.26700, 0.26589, 0.26482, 0.26374, 0.26263, 0.26148,\
        0.26028, 0.25902, 0.25769, 0.25631, 0.25488, 0.25340, 0.25188, 0.25034, 0.24878, 0.24721, 0.24563,\
        0.24404, 0.24244, 0.24084, 0.23924, 0.23762, 0.23600, 0.23437, 0.23271, 0.23099, 0.22918, 0.22722,\
        0.22506, 0.22261, 0.21979, 0.21653, 0.21283, 0.20878, 0.20451, 0.19977, 0.19426, 0.18848, 0.18243],\
        [0.17915, 0.18501, 0.19034, 0.19504, 0.19891, 0.20108, 0.20156, 0.20163, 0.20164, 0.20176, 0.20214,\
        0.20335, 0.20524, 0.20730, 0.20927, 0.21110, 0.21284, 0.21452, 0.21617, 0.21780, 0.21943, 0.22108,\
        0.22277, 0.22449, 0.22624, 0.22800, 0.22977, 0.23155, 0.23334, 0.23515, 0.23698, 0.23885, 0.24074,\
        0.24268, 0.24465, 0.24666, 0.24869, 0.25075, 0.25283, 0.25493, 0.25703, 0.25915, 0.26128, 0.26341,\
        0.26555, 0.26768, 0.26976, 0.27176, 0.27363, 0.27532, 0.27684, 0.27818, 0.27938, 0.28045, 0.28144,\
        0.28240, 0.28338, 0.28436, 0.28522, 0.28585, 0.28616, 0.28615, 0.28572, 0.28481, 0.28357, 0.28224,\
        0.28094, 0.27965, 0.27834, 0.27705, 0.27580, 0.27463, 0.27352, 0.27243, 0.27133, 0.27019, 0.26900,\
        0.26779, 0.26657, 0.26536, 0.26417, 0.26301, 0.26189, 0.26078, 0.25969, 0.25860, 0.25748, 0.25633,\
        0.25512, 0.25387, 0.25255, 0.25119, 0.24977, 0.24831, 0.24682, 0.24530, 0.24376, 0.24221, 0.24065,\
        0.23908, 0.23751, 0.23593, 0.23435, 0.23277, 0.23118, 0.22960, 0.22800, 0.22637, 0.22468, 0.22286,\
        0.22085, 0.21855, 0.21586, 0.21266, 0.20892, 0.20469, 0.20037, 0.19560, 0.19009, 0.18413, 0.17787],\
        [0.17634, 0.18242, 0.18800, 0.19294, 0.19695, 0.19916, 0.19949, 0.19947, 0.19940, 0.19948, 0.19960,\
        0.20092, 0.20287, 0.20491, 0.20671, 0.20830, 0.20982, 0.21133, 0.21282, 0.21430, 0.21580, 0.21735,\
        0.21895, 0.22061, 0.22229, 0.22399, 0.22570, 0.22740, 0.22912, 0.23085, 0.23261, 0.23441, 0.23624,\
        0.23812, 0.24004, 0.24200, 0.24399, 0.24601, 0.24804, 0.25009, 0.25215, 0.25422, 0.25629, 0.25835,\
        0.26040, 0.26243, 0.26439, 0.26627, 0.26803, 0.26964, 0.27110, 0.27241, 0.27359, 0.27466, 0.27565,\
        0.27660, 0.27752, 0.27838, 0.27912, 0.27965, 0.27991, 0.27985, 0.27943, 0.27866, 0.27762, 0.27644,\
        0.27522, 0.27398, 0.27274, 0.27151, 0.27031, 0.26916, 0.26804, 0.26695, 0.26585, 0.26474, 0.26360,\
        0.26244, 0.26127, 0.26010, 0.25895, 0.25781, 0.25670, 0.25559, 0.25449, 0.25339, 0.25226, 0.25109,\
        0.24989, 0.24863, 0.24732, 0.24596, 0.24456, 0.24311, 0.24163, 0.24012, 0.23859, 0.23706, 0.23551,\
        0.23396, 0.23241, 0.23084, 0.22927, 0.22770, 0.22613, 0.22457, 0.22303, 0.22147, 0.21987, 0.21818,\
        0.21632, 0.21420, 0.21167, 0.20855, 0.20477, 0.20045, 0.19571, 0.19079, 0.18527, 0.17925, 0.17287],\
        [0.17282, 0.17900, 0.18462, 0.18950, 0.19332, 0.19563, 0.19656, 0.19687, 0.19703, 0.19727, 0.19782,\
        0.19908, 0.20098, 0.20288, 0.20430, 0.20552, 0.20678, 0.20812, 0.20945, 0.21077, 0.21212, 0.21355,\
        0.21508, 0.21668, 0.21831, 0.21995, 0.22158, 0.22321, 0.22484, 0.22649, 0.22817, 0.22989, 0.23166,\
        0.23348, 0.23535, 0.23726, 0.23920, 0.24117, 0.24316, 0.24517, 0.24718, 0.24919, 0.25119, 0.25319,\
        0.25515, 0.25708, 0.25895, 0.26072, 0.26239, 0.26393, 0.26534, 0.26662, 0.26778, 0.26883, 0.26981,\
        0.27073, 0.27158, 0.27235, 0.27299, 0.27343, 0.27362, 0.27354, 0.27316, 0.27250, 0.27160, 0.27056,\
        0.26943, 0.26827, 0.26709, 0.26592, 0.26477, 0.26364, 0.26254, 0.26145, 0.26037, 0.25927, 0.25816,\
        0.25703, 0.25590, 0.25477, 0.25364, 0.25253, 0.25142, 0.25032, 0.24921, 0.24809, 0.24695, 0.24578,\
        0.24457, 0.24331, 0.24200, 0.24064, 0.23924, 0.23779, 0.23631, 0.23480, 0.23328, 0.23176, 0.23022,\
        0.22868, 0.22714, 0.22558, 0.22400, 0.22242, 0.22085, 0.21930, 0.21777, 0.21626, 0.21475, 0.21315,\
        0.21139, 0.20944, 0.20707, 0.20409, 0.20023, 0.19576, 0.19087, 0.18563, 0.17997, 0.17388, 0.16740],\
        [0.16858, 0.17475, 0.18031, 0.18508, 0.18882, 0.19137, 0.19286, 0.19370, 0.19428, 0.19492, 0.19586,\
        0.19730, 0.19921, 0.20086, 0.20172, 0.20256, 0.20359, 0.20486, 0.20604, 0.20716, 0.20835, 0.20967,\
        0.21114, 0.21270, 0.21430, 0.21587, 0.21742, 0.21895, 0.22049, 0.22205, 0.22364, 0.22528, 0.22699,\
        0.22875, 0.23057, 0.23243, 0.23433, 0.23625, 0.23819, 0.24015, 0.24210, 0.24406, 0.24600, 0.24792,\
        0.24981, 0.25165, 0.25343, 0.25512, 0.25671, 0.25819, 0.25955, 0.26079, 0.26192, 0.26296, 0.26391,\
        0.26479, 0.26558, 0.26626, 0.26681, 0.26718, 0.26733, 0.26724, 0.26689, 0.26631, 0.26553, 0.26460,\
        0.26357, 0.26249, 0.26138, 0.26027, 0.25916, 0.25807, 0.25698, 0.25591, 0.25484, 0.25376, 0.25267,\
        0.25157, 0.25047, 0.24936, 0.24826, 0.24715, 0.24605, 0.24495, 0.24384, 0.24271, 0.24156, 0.24038,\
        0.23916, 0.23789, 0.23658, 0.23522, 0.23381, 0.23236, 0.23087, 0.22935, 0.22783, 0.22630, 0.22478,\
        0.22325, 0.22171, 0.22015, 0.21855, 0.21694, 0.21535, 0.21380, 0.21225, 0.21073, 0.20929, 0.20777,\
        0.20598, 0.20416, 0.20192, 0.19887, 0.19507, 0.19047, 0.18544, 0.18003, 0.17423, 0.16804, 0.16149],\
        [0.16367, 0.16980, 0.17528, 0.17999, 0.18378, 0.18660, 0.18856, 0.18992, 0.19100, 0.19205, 0.19330,\
        0.19484, 0.19657, 0.19791, 0.19866, 0.19932, 0.20024, 0.20153, 0.20253, 0.20340, 0.20445, 0.20570,\
        0.20711, 0.20867, 0.21025, 0.21174, 0.21318, 0.21462, 0.21606, 0.21752, 0.21902, 0.22058, 0.22221,\
        0.22392, 0.22569, 0.22750, 0.22936, 0.23124, 0.23313, 0.23504, 0.23694, 0.23883, 0.24071, 0.24256,\
        0.24437, 0.24613, 0.24783, 0.24945, 0.25097, 0.25239, 0.25370, 0.25491, 0.25601, 0.25702, 0.25794,\
        0.25877, 0.25950, 0.26012, 0.26060, 0.26091, 0.26102, 0.26093, 0.26062, 0.26010, 0.25940, 0.25857,\
        0.25764, 0.25664, 0.25560, 0.25454, 0.25348, 0.25241, 0.25136, 0.25030, 0.24924, 0.24818, 0.24711,\
        0.24604, 0.24495, 0.24387, 0.24277, 0.24168, 0.24058, 0.23948, 0.23836, 0.23722, 0.23606, 0.23487,\
        0.23364, 0.23237, 0.23105, 0.22968, 0.22827, 0.22680, 0.22530, 0.22377, 0.22223, 0.22071, 0.21918,\
        0.21765, 0.21612, 0.21458, 0.21296, 0.21127, 0.20963, 0.20811, 0.20651, 0.20490, 0.20350, 0.20217,\
        0.20008, 0.19825, 0.19599, 0.19285, 0.18900, 0.18445, 0.17939, 0.17390, 0.16802, 0.16175, 0.15514],\
        [0.15816, 0.16424, 0.16968, 0.17439, 0.17829, 0.18138, 0.18374, 0.18557, 0.18709, 0.18850, 0.18996,\
        0.19151, 0.19302, 0.19424, 0.19511, 0.19586, 0.19680, 0.19790, 0.19876, 0.19947, 0.20047, 0.20163,\
        0.20299, 0.20462, 0.20619, 0.20753, 0.20885, 0.21020, 0.21154, 0.21289, 0.21428, 0.21575, 0.21733,\
        0.21899, 0.22071, 0.22249, 0.22430, 0.22613, 0.22797, 0.22983, 0.23167, 0.23351, 0.23532, 0.23710,\
        0.23885, 0.24053, 0.24216, 0.24371, 0.24517, 0.24653, 0.24780, 0.24896, 0.25003, 0.25101, 0.25189,\
        0.25267, 0.25335, 0.25391, 0.25434, 0.25460, 0.25469, 0.25460, 0.25431, 0.25385, 0.25323, 0.25247,\
        0.25162, 0.25070, 0.24972, 0.24872, 0.24770, 0.24668, 0.24565, 0.24461, 0.24357, 0.24253, 0.24147,\
        0.24041, 0.23935, 0.23827, 0.23719, 0.23610, 0.23500, 0.23390, 0.23277, 0.23163, 0.23046, 0.22926,\
        0.22802, 0.22674, 0.22541, 0.22403, 0.22261, 0.22113, 0.21960, 0.21804, 0.21649, 0.21496, 0.21344,\
        0.21189, 0.21037, 0.20887, 0.20727, 0.20545, 0.20370, 0.20228, 0.20071, 0.19888, 0.19744, 0.19603,\
        0.19409, 0.19190, 0.18937, 0.18619, 0.18231, 0.17780, 0.17275, 0.16724, 0.16132, 0.15501, 0.14835],\
        [0.15212, 0.15817, 0.16361, 0.16836, 0.17240, 0.17574, 0.17844, 0.18066, 0.18255, 0.18425, 0.18588,\
        0.18745, 0.18890, 0.19011, 0.19110, 0.19199, 0.19291, 0.19385, 0.19468, 0.19547, 0.19642, 0.19754,\
        0.19886, 0.20061, 0.20206, 0.20317, 0.20438, 0.20566, 0.20691, 0.20815, 0.20942, 0.21081, 0.21234,\
        0.21395, 0.21564, 0.21737, 0.21914, 0.22092, 0.22272, 0.22452, 0.22631, 0.22808, 0.22983, 0.23155,\
        0.23323, 0.23485, 0.23640, 0.23789, 0.23929, 0.24060, 0.24182, 0.24295, 0.24398, 0.24492, 0.24576,\
        0.24649, 0.24713, 0.24764, 0.24802, 0.24826, 0.24833, 0.24824, 0.24797, 0.24755, 0.24699, 0.24631,\
        0.24553, 0.24467, 0.24376, 0.24281, 0.24183, 0.24084, 0.23984, 0.23883, 0.23781, 0.23678, 0.23574,\
        0.23469, 0.23364, 0.23257, 0.23149, 0.23041, 0.22931, 0.22820, 0.22707, 0.22591, 0.22473, 0.22352,\
        0.22227, 0.22098, 0.21965, 0.21827, 0.21683, 0.21534, 0.21379, 0.21219, 0.21058, 0.20908, 0.20760,\
        0.20597, 0.20443, 0.20300, 0.20148, 0.19959, 0.19769, 0.19620, 0.19465, 0.19284, 0.19119, 0.18949,\
        0.18749, 0.18513, 0.18237, 0.17906, 0.17514, 0.17064, 0.16561, 0.16010, 0.15416, 0.14784, 0.14114],\
        [0.14562, 0.15166, 0.15712, 0.16195, 0.16613, 0.16969, 0.17269, 0.17522, 0.17741, 0.17937, 0.18115,\
        0.18280, 0.18427, 0.18555, 0.18665, 0.18764, 0.18859, 0.18951, 0.19037, 0.19125, 0.19222, 0.19336,\
        0.19472, 0.19630, 0.19758, 0.19860, 0.19979, 0.20099, 0.20216, 0.20331, 0.20441, 0.20575, 0.20725,\
        0.20883, 0.21048, 0.21217, 0.21388, 0.21562, 0.21736, 0.21911, 0.22084, 0.22255, 0.22424, 0.22590,\
        0.22751, 0.22907, 0.23056, 0.23199, 0.23333, 0.23460, 0.23577, 0.23685, 0.23785, 0.23874, 0.23954,\
        0.24024, 0.24083, 0.24130, 0.24165, 0.24186, 0.24192, 0.24183, 0.24159, 0.24120, 0.24069, 0.24006,\
        0.23934, 0.23854, 0.23769, 0.23679, 0.23585, 0.23490, 0.23392, 0.23293, 0.23193, 0.23092, 0.22989,\
        0.22886, 0.22781, 0.22675, 0.22568, 0.22459, 0.22349, 0.22237, 0.22124, 0.22007, 0.21888, 0.21766,\
        0.21640, 0.21510, 0.21375, 0.21236, 0.21092, 0.20943, 0.20788, 0.20624, 0.20449, 0.20304, 0.20172,\
        0.19991, 0.19834, 0.19689, 0.19534, 0.19354, 0.19166, 0.18996, 0.18827, 0.18647, 0.18463, 0.18268,\
        0.18049, 0.17795, 0.17500, 0.17156, 0.16758, 0.16306, 0.15803, 0.15252, 0.14658, 0.14024, 0.13353],\
        [0.13870, 0.14475, 0.15024, 0.15516, 0.15948, 0.16324, 0.16649, 0.16930, 0.17175, 0.17392, 0.17587,\
        0.17762, 0.17919, 0.18056, 0.18178, 0.18287, 0.18388, 0.18485, 0.18579, 0.18674, 0.18777, 0.18892,\
        0.19022, 0.19158, 0.19279, 0.19387, 0.19500, 0.19612, 0.19726, 0.19829, 0.19931, 0.20063, 0.20208,\
        0.20362, 0.20522, 0.20686, 0.20853, 0.21021, 0.21190, 0.21359, 0.21526, 0.21692, 0.21855, 0.22014,\
        0.22169, 0.22319, 0.22462, 0.22599, 0.22729, 0.22850, 0.22963, 0.23067, 0.23162, 0.23248, 0.23324,\
        0.23390, 0.23445, 0.23489, 0.23521, 0.23540, 0.23545, 0.23536, 0.23514, 0.23479, 0.23432, 0.23374,\
        0.23307, 0.23232, 0.23151, 0.23066, 0.22976, 0.22884, 0.22789, 0.22692, 0.22594, 0.22494, 0.22393,\
        0.22290, 0.22186, 0.22081, 0.21973, 0.21865, 0.21754, 0.21642, 0.21527, 0.21409, 0.21289, 0.21165,\
        0.21038, 0.20907, 0.20771, 0.20631, 0.20487, 0.20337, 0.20182, 0.20020, 0.19840, 0.19685, 0.19551,\
        0.19382, 0.19216, 0.19058, 0.18894, 0.18717, 0.18532, 0.18349, 0.18165, 0.17974, 0.17773, 0.17557,\
        0.17317, 0.17045, 0.16732, 0.16375, 0.15968, 0.15512, 0.15007, 0.14455, 0.13861, 0.13227, 0.12554]]

    #--------------------------------------------------------------------------
    #Converting contours from list to numpy array 

    zgrid_H_Hmax_vmax15vt5=np.array(zgrid_H_Hmax_vmax15vt5)
    zgrid_H_Hmax_vmax20vt0=np.array(zgrid_H_Hmax_vmax20vt0)
    zgrid_H_Hmax_vmax20vt2_5=np.array(zgrid_H_Hmax_vmax20vt2_5)
    zgrid_H_Hmax_vmax20vt5=np.array(zgrid_H_Hmax_vmax20vt5)
    zgrid_H_Hmax_vmax20vt7_5=np.array(zgrid_H_Hmax_vmax20vt7_5)
    zgrid_H_Hmax_vmax20vt10=np.array(zgrid_H_Hmax_vmax20vt10)
    zgrid_H_Hmax_vmax25vt0=np.array(zgrid_H_Hmax_vmax25vt0)
    zgrid_H_Hmax_vmax25vt2_5=np.array(zgrid_H_Hmax_vmax25vt2_5)
    zgrid_H_Hmax_vmax25vt5=np.array(zgrid_H_Hmax_vmax25vt5)
    zgrid_H_Hmax_vmax25vt7_5=np.array(zgrid_H_Hmax_vmax25vt7_5)
    zgrid_H_Hmax_vmax25vt10=np.array(zgrid_H_Hmax_vmax25vt10)
    zgrid_H_Hmax_vmax30vt0=np.array(zgrid_H_Hmax_vmax30vt0)
    zgrid_H_Hmax_vmax30vt2_5=np.array(zgrid_H_Hmax_vmax30vt2_5)
    zgrid_H_Hmax_vmax30vt5=np.array(zgrid_H_Hmax_vmax30vt5)
    zgrid_H_Hmax_vmax30vt7_5=np.array(zgrid_H_Hmax_vmax30vt7_5)
    zgrid_H_Hmax_vmax30vt10=np.array(zgrid_H_Hmax_vmax30vt10)
    zgrid_H_Hmax_vmax40vt0=np.array(zgrid_H_Hmax_vmax40vt0)
    zgrid_H_Hmax_vmax40vt2_5=np.array(zgrid_H_Hmax_vmax40vt2_5)
    zgrid_H_Hmax_vmax40vt5=np.array(zgrid_H_Hmax_vmax40vt5)
    zgrid_H_Hmax_vmax40vt7_5=np.array(zgrid_H_Hmax_vmax40vt7_5)
    zgrid_H_Hmax_vmax40vt10=np.array(zgrid_H_Hmax_vmax40vt10)
    zgrid_H_Hmax_vmax50vt0=np.array(zgrid_H_Hmax_vmax50vt0)
    zgrid_H_Hmax_vmax50vt2_5=np.array(zgrid_H_Hmax_vmax50vt2_5)
    zgrid_H_Hmax_vmax50vt5=np.array(zgrid_H_Hmax_vmax50vt5)
    zgrid_H_Hmax_vmax50vt7_5=np.array(zgrid_H_Hmax_vmax50vt7_5)
    zgrid_H_Hmax_vmax50vt10=np.array(zgrid_H_Hmax_vmax50vt10)
    zgrid_H_Hmax_vmax60vt0=np.array(zgrid_H_Hmax_vmax60vt0)
    zgrid_H_Hmax_vmax60vt2_5=np.array(zgrid_H_Hmax_vmax60vt2_5)
    zgrid_H_Hmax_vmax60vt5=np.array(zgrid_H_Hmax_vmax60vt5)
    zgrid_H_Hmax_vmax60vt7_5=np.array(zgrid_H_Hmax_vmax60vt7_5)
    zgrid_H_Hmax_vmax60vt10=np.array(zgrid_H_Hmax_vmax60vt10)

    #--------------------------------------------------------------------------
    #Pre-assigning array

    #xgrid and ygrid are 2d array
    if np.ndim(xgrid)==2: 
        Mgrid,Ngrid=np.shape(xgrid)

    #xgrid and ygrid are 1d array
    else: 
        Mgrid=(np.shape(xgrid))[0]
        Ngrid=1 #Ngrid=1 when xgrid and ygrid are 1d array

    Mdata,Ndata=np.shape(xgrid_x_RVmax)

    if len(xCenter)>1:
        Rgrid=np.zeros((Mgrid,Ngrid,len(xCenter))) #Pre-assigning array
        thetagrid=np.zeros((Mgrid,Ngrid,len(xCenter))) #Pre-assigning array
        xgrid_RVmax=np.zeros((Mgrid,Ngrid,len(xCenter))) #Pre-assigning array
        ygrid_RVmax=np.zeros((Mgrid,Ngrid,len(xCenter))) #Pre-assigning array
        Hgrid=np.zeros((Mgrid,Ngrid,len(xCenter))) #Pre-assigning array
        zgrid_H_Hmax=np.zeros((Mdata,Ndata,len(xCenter))) #Pre-assigning array
    else:
        Rgrid=np.zeros((Mgrid,Ngrid,1)) #Pre-assigning array
        thetagrid=np.zeros((Mgrid,Ngrid,1)) #Pre-assigning array
        xgrid_RVmax=np.zeros((Mgrid,Ngrid,1)) #Pre-assigning array
        ygrid_RVmax=np.zeros((Mgrid,Ngrid,1)) #Pre-assigning array
        Hgrid=np.zeros((Mgrid,Ngrid,1)) #Pre-assigning array
        zgrid_H_Hmax=np.zeros((Mdata,Ndata,1)) #Pre-assigning array


    #--------------------------------------------------------------------------
    #Finding an appropriate dataset

    #               Vt=0.0    Vt=2.5    Vt=5.0    Vt=7.5    Vt=10
    #
    # Vmax=15                             *
    # Vmax=20         *         *         *         *         *
    # Vmax=25         *         *         *         *         *
    # Vmax=30         *         *         *         *         *
    # Vmax=40         *         *         *         *         *
    # Vmax=50         *         *         *         *         *
    # Vmax=60         *         *         *         *         *

    for k in range(0,len(xCenter),1):

        #Finding dataset if Vmax and Vt both are equal to known values
        if ((Vmax[k]==15) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax15vt5

        elif ((Vmax[k]==20) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt0

        elif ((Vmax[k]==20) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt0

        elif ((Vmax[k]==20) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt2_5

        elif ((Vmax[k]==20) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt5

        elif ((Vmax[k]==20) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt7_5

        elif ((Vmax[k]==20) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt10

        elif ((Vmax[k]==25) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt0

        elif ((Vmax[k]==25) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt2_5

        elif ((Vmax[k]==25) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt5

        elif ((Vmax[k]==25) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt7_5

        elif ((Vmax[k]==25) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt10

        elif ((Vmax[k]==30) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt0

        elif ((Vmax[k]==30) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt2_5

        elif ((Vmax[k]==30) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt5

        elif ((Vmax[k]==30) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt7_5

        elif ((Vmax[k]==30) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt10

        elif ((Vmax[k]==40) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt0

        elif ((Vmax[k]==40) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt2_5

        elif ((Vmax[k]==40) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt5

        elif ((Vmax[k]==40) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt7_5

        elif ((Vmax[k]==40) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt10

        elif ((Vmax[k]==50) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt0

        elif ((Vmax[k]==50) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt2_5

        elif ((Vmax[k]==50) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt5

        elif ((Vmax[k]==50) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt7_5

        elif ((Vmax[k]==50) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt10

        elif ((Vmax[k]==60) and (Vt[k]==0)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt0

        elif ((Vmax[k]==60) and (Vt[k]==2.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt2_5

        elif ((Vmax[k]==60) and (Vt[k]==5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt5

        elif ((Vmax[k]==60) and (Vt[k]==7.5)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt7_5

        elif ((Vmax[k]==60) and (Vt[k]==10)):
            zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt10


        #Finding dataset if Vmax or Vt is not equal to known values
        #Vmax<20
        if Vmax[k]<20:
            if ((Vmax[k]!=15) and (Vt[k]!=5)):
                if ((Vt[k]>0) and (Vt[k]<=1.25)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt0
                elif ((Vt[k]>1.25) and (Vt[k]<=3.75)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt2_5
                elif ((Vt[k]>3.75) and (Vt[k]<=6.25)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt5
                elif ((Vt[k]>6.25) and (Vt[k]<=8.75)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt7_5
                elif Vt[k]>8.75:
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt10


        #Vmax=20
        if Vmax[k]==20:
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax20vt0[i,j],zgrid_H_Hmax_vmax20vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax20vt2_5[i,j],zgrid_H_Hmax_vmax20vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax20vt5[i,j],zgrid_H_Hmax_vmax20vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax20vt7_5[i,j],zgrid_H_Hmax_vmax20vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt10


        #Vmax>20 & Vmax<25
        if ((Vmax[k]>20) and (Vmax[k]<25)):
            if Vt[k]==0:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[20,25],[zgrid_H_Hmax_vmax20vt0[i,j],zgrid_H_Hmax_vmax25vt0[i,j]])

            elif ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[0,2.5,0,2.5] #Vt values
                        yvalues=[20,20,25,25] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax20vt0[i,j],zgrid_H_Hmax_vmax20vt2_5[i,j],zgrid_H_Hmax_vmax25vt0[i,j],zgrid_H_Hmax_vmax25vt2_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==2.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[20,25],[zgrid_H_Hmax_vmax20vt2_5[i,j],zgrid_H_Hmax_vmax25vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[2.5,5,2.5,5] #Vt values
                        yvalues=[20,20,25,25] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax20vt2_5[i,j],zgrid_H_Hmax_vmax20vt5[i,j],zgrid_H_Hmax_vmax25vt2_5[i,j],zgrid_H_Hmax_vmax25vt5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[20,25],[zgrid_H_Hmax_vmax20vt5[i,j],zgrid_H_Hmax_vmax25vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[5,7.5,5,7.5] #Vt values
                        yvalues=[20,20,25,25] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax20vt5[i,j],zgrid_H_Hmax_vmax20vt7_5[i,j],zgrid_H_Hmax_vmax25vt5[i,j],zgrid_H_Hmax_vmax25vt7_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==7.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[20,25],[zgrid_H_Hmax_vmax20vt7_5[i,j],zgrid_H_Hmax_vmax25vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[7.5,10,7.5,10] #Vt values
                        yvalues=[20,20,25,25] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax20vt7_5[i,j],zgrid_H_Hmax_vmax20vt10[i,j],zgrid_H_Hmax_vmax25vt7_5[i,j],zgrid_H_Hmax_vmax25vt10[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==10:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[20,25],[zgrid_H_Hmax_vmax20vt10[i,j],zgrid_H_Hmax_vmax25vt10[i,j]])

            elif Vt[k]>10:
                if ((Vmax[k]>20) and (Vmax[k]<=22.5)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax20vt10
                elif ((Vmax[k]>22.5) and (Vmax[k]<25)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt10


        #Vmax=25
        if Vmax[k]==25:
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax25vt0[i,j],zgrid_H_Hmax_vmax25vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax25vt2_5[i,j],zgrid_H_Hmax_vmax25vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax25vt5[i,j],zgrid_H_Hmax_vmax25vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax25vt7_5[i,j],zgrid_H_Hmax_vmax25vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt10


        #Vmax>25 & Vmax<30
        if ((Vmax[k]>25) and (Vmax[k]<30)):
            if Vt[k]==0:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[25,30],[zgrid_H_Hmax_vmax25vt0[i,j],zgrid_H_Hmax_vmax30vt0[i,j]])

            elif ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[0,2.5,0,2.5] #Vt values
                        yvalues=[25,25,30,30] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax25vt0[i,j],zgrid_H_Hmax_vmax25vt2_5[i,j],zgrid_H_Hmax_vmax30vt0[i,j],zgrid_H_Hmax_vmax30vt2_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==2.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[25,30],[zgrid_H_Hmax_vmax25vt2_5[i,j],zgrid_H_Hmax_vmax30vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[2.5,5,2.5,5] #Vt values
                        yvalues=[25,25,30,30] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax25vt2_5[i,j],zgrid_H_Hmax_vmax25vt5[i,j],zgrid_H_Hmax_vmax30vt2_5[i,j],zgrid_H_Hmax_vmax30vt5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[25,30],[zgrid_H_Hmax_vmax25vt5[i,j],zgrid_H_Hmax_vmax30vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[5,7.5,5,7.5] #Vt values
                        yvalues=[25,25,30,30] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax25vt5[i,j],zgrid_H_Hmax_vmax25vt7_5[i,j],zgrid_H_Hmax_vmax30vt5[i,j],zgrid_H_Hmax_vmax30vt7_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==7.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[25,30],[zgrid_H_Hmax_vmax25vt7_5[i,j],zgrid_H_Hmax_vmax30vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[7.5,10,7.5,10] #Vt values
                        yvalues=[25,25,30,30] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax25vt7_5[i,j],zgrid_H_Hmax_vmax25vt10[i,j],zgrid_H_Hmax_vmax30vt7_5[i,j],zgrid_H_Hmax_vmax30vt10[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==10:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[25,30],[zgrid_H_Hmax_vmax25vt10[i,j],zgrid_H_Hmax_vmax30vt10[i,j]])

            elif Vt[k]>10:
                if ((Vmax[k]>25) and (Vmax[k]<=27.5)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax25vt10
                elif ((Vmax[k]>27.5) and (Vmax[k]<30)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt10


        #Vmax=30
        if Vmax[k]==30: 
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax30vt0[i,j],zgrid_H_Hmax_vmax30vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax30vt2_5[i,j],zgrid_H_Hmax_vmax30vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax30vt5[i,j],zgrid_H_Hmax_vmax30vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax30vt7_5[i,j],zgrid_H_Hmax_vmax30vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt10


        #Vmax>30 & Vmax<40
        if ((Vmax[k]>30) and (Vmax[k]<40)):
            if Vt[k]==0:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[30,40],[zgrid_H_Hmax_vmax30vt0[i,j],zgrid_H_Hmax_vmax40vt0[i,j]])

            elif ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[0,2.5,0,2.5] #Vt values
                        yvalues=[30,30,40,40] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax30vt0[i,j],zgrid_H_Hmax_vmax30vt2_5[i,j],zgrid_H_Hmax_vmax40vt0[i,j],zgrid_H_Hmax_vmax40vt2_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==2.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[30,40],[zgrid_H_Hmax_vmax30vt2_5[i,j],zgrid_H_Hmax_vmax40vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[2.5,5,2.5,5] #Vt values
                        yvalues=[30,30,40,40] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax30vt2_5[i,j],zgrid_H_Hmax_vmax30vt5[i,j],zgrid_H_Hmax_vmax40vt2_5[i,j],zgrid_H_Hmax_vmax40vt5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[30,40],[zgrid_H_Hmax_vmax30vt5[i,j],zgrid_H_Hmax_vmax40vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[5,7.5,5,7.5] #Vt values
                        yvalues=[30,30,40,40] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax30vt5[i,j],zgrid_H_Hmax_vmax30vt7_5[i,j],zgrid_H_Hmax_vmax40vt5[i,j],zgrid_H_Hmax_vmax40vt7_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==7.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[30,40],[zgrid_H_Hmax_vmax30vt7_5[i,j],zgrid_H_Hmax_vmax40vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[7.5,10,7.5,10] #Vt values
                        yvalues=[30,30,40,40] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax30vt7_5[i,j],zgrid_H_Hmax_vmax30vt10[i,j],zgrid_H_Hmax_vmax40vt7_5[i,j],zgrid_H_Hmax_vmax40vt10[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==10:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[30,40],[zgrid_H_Hmax_vmax30vt10[i,j],zgrid_H_Hmax_vmax40vt10[i,j]])

            elif Vt[k]>10:
                if ((Vmax[k]>30) and (Vmax[k]<=35)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax30vt10
                elif ((Vmax[k]>35) and (Vmax[k]<40)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt10


        #Vmax=40
        if Vmax[k]==40:
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax40vt0[i,j],zgrid_H_Hmax_vmax40vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax40vt2_5[i,j],zgrid_H_Hmax_vmax40vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax40vt5[i,j],zgrid_H_Hmax_vmax40vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax40vt7_5[i,j],zgrid_H_Hmax_vmax40vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt10


        #Vmax>40 & Vmax<50
        if ((Vmax[k]>40) and (Vmax[k]<50)):
            if Vt[k]==0:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[40,50],[zgrid_H_Hmax_vmax40vt0[i,j],zgrid_H_Hmax_vmax50vt0[i,j]])

            elif ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[0,2.5,0,2.5] #Vt values
                        yvalues=[40,40,50,50] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax40vt0[i,j],zgrid_H_Hmax_vmax40vt2_5[i,j],zgrid_H_Hmax_vmax50vt0[i,j],zgrid_H_Hmax_vmax50vt2_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==2.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[40,50],[zgrid_H_Hmax_vmax40vt2_5[i,j],zgrid_H_Hmax_vmax50vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[2.5,5,2.5,5] #Vt values
                        yvalues=[40,40,50,50] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax40vt2_5[i,j],zgrid_H_Hmax_vmax40vt5[i,j],zgrid_H_Hmax_vmax50vt2_5[i,j],zgrid_H_Hmax_vmax50vt5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[40,50],[zgrid_H_Hmax_vmax40vt5[i,j],zgrid_H_Hmax_vmax50vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[5,7.5,5,7.5] #Vt values
                        yvalues=[40,40,50,50] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax40vt5[i,j],zgrid_H_Hmax_vmax40vt7_5[i,j],zgrid_H_Hmax_vmax50vt5[i,j],zgrid_H_Hmax_vmax50vt7_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==7.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[40,50],[zgrid_H_Hmax_vmax40vt7_5[i,j],zgrid_H_Hmax_vmax50vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[7.5,10,7.5,10] #Vt values
                        yvalues=[40,40,50,50] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax40vt7_5[i,j],zgrid_H_Hmax_vmax40vt10[i,j],zgrid_H_Hmax_vmax50vt7_5[i,j],zgrid_H_Hmax_vmax50vt10[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==10:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[40,50],[zgrid_H_Hmax_vmax40vt10[i,j],zgrid_H_Hmax_vmax50vt10[i,j]])

            elif Vt[k]>10:
                if ((Vmax[k]>40) and (Vmax[k]<=45)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax40vt10
                elif ((Vmax[k]>45) and (Vmax[k]<50)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt10


        #Vmax=50
        if Vmax[k]==50:
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax50vt0[i,j],zgrid_H_Hmax_vmax50vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax50vt2_5[i,j],zgrid_H_Hmax_vmax50vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax50vt5[i,j],zgrid_H_Hmax_vmax50vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax50vt7_5[i,j],zgrid_H_Hmax_vmax50vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt10


        #Vmax>50 & Vmax<60
        if ((Vmax[k]>50) and (Vmax[k]<60)):
            if Vt[k]==0:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[50,60],[zgrid_H_Hmax_vmax50vt0[i,j],zgrid_H_Hmax_vmax60vt0[i,j]])

            elif ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[0,2.5,0,2.5] #Vt values
                        yvalues=[50,50,60,60] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax50vt0[i,j],zgrid_H_Hmax_vmax50vt2_5[i,j],zgrid_H_Hmax_vmax60vt0[i,j],zgrid_H_Hmax_vmax60vt2_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==2.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[50,60],[zgrid_H_Hmax_vmax50vt2_5[i,j],zgrid_H_Hmax_vmax60vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[2.5,5,2.5,5] #Vt values
                        yvalues=[50,50,60,60] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax50vt2_5[i,j],zgrid_H_Hmax_vmax50vt5[i,j],zgrid_H_Hmax_vmax60vt2_5[i,j],zgrid_H_Hmax_vmax60vt5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[50,60],[zgrid_H_Hmax_vmax50vt5[i,j],zgrid_H_Hmax_vmax60vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[5,7.5,5,7.5] #Vt values
                        yvalues=[50,50,60,60] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax50vt5[i,j],zgrid_H_Hmax_vmax50vt7_5[i,j],zgrid_H_Hmax_vmax60vt5[i,j],zgrid_H_Hmax_vmax60vt7_5[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==7.5:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[50,60],[zgrid_H_Hmax_vmax50vt7_5[i,j],zgrid_H_Hmax_vmax60vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        xvalues=[7.5,10,7.5,10] #Vt values
                        yvalues=[50,50,60,60] #Vmax values
                        zvalues=[zgrid_H_Hmax_vmax50vt7_5[i,j],zgrid_H_Hmax_vmax50vt10[i,j],zgrid_H_Hmax_vmax60vt7_5[i,j],zgrid_H_Hmax_vmax60vt10[i,j]] #H/Hmax values
                        fun=sp.interpolate.LinearNDInterpolator((np.reshape(xvalues,np.size(xvalues)),np.reshape(yvalues,np.size(yvalues))),np.reshape(zvalues,np.size(zvalues)))
                        zgrid_H_Hmax[i,j,k]=fun(Vt[k],Vmax[k])

            elif Vt[k]==10:
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vmax[k],[50,60],[zgrid_H_Hmax_vmax50vt10[i,j],zgrid_H_Hmax_vmax60vt10[i,j]])

            elif Vt[k]>10:
                if ((Vmax[k]>50) and (Vmax[k]<=55)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax50vt10
                elif ((Vmax[k]>55) and (Vmax[k]<60)):
                    zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt10


        #Vmax=60
        if Vmax[k]==60:
            if ((Vt[k]>0) and (Vt[k]<2.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[0,2.5],[zgrid_H_Hmax_vmax60vt0[i,j],zgrid_H_Hmax_vmax60vt2_5[i,j]])

            elif ((Vt[k]>2.5) and (Vt[k]<5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[2.5,5],[zgrid_H_Hmax_vmax60vt2_5[i,j],zgrid_H_Hmax_vmax60vt5[i,j]])

            elif ((Vt[k]>5) and (Vt[k]<7.5)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[5,7.5],[zgrid_H_Hmax_vmax60vt5[i,j],zgrid_H_Hmax_vmax60vt7_5[i,j]])

            elif ((Vt[k]>7.5) and (Vt[k]<10)):
                for i in range(0,Mdata,1):
                    for j in range(0,Ndata,1):
                        zgrid_H_Hmax[i,j,k]=np.interp(Vt[k],[7.5,10],[zgrid_H_Hmax_vmax60vt7_5[i,j],zgrid_H_Hmax_vmax60vt10[i,j]])

            elif Vt[k]>10:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt10


        #Vmax>60
        if Vmax[k]>60:
            if ((Vt[k]>0) and (Vt[k]<=1.25)):
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt0
            elif ((Vt[k]>1.25) and (Vt[k]<=3.75)):
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt2_5
            elif ((Vt[k]>3.75) and (Vt[k]<=6.25)):
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt5
            elif ((Vt[k]>6.25) and (Vt[k]<=8.75)):
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt7_5
            elif Vt[k]>8.75:
                zgrid_H_Hmax[:,:,k]=zgrid_H_Hmax_vmax60vt10


    zgrid_H_Hmax[zgrid_H_Hmax<0]=0
    zgrid_H_Hmax[zgrid_H_Hmax>1]=1
    zgrid_H_Hmax[np.isnan(zgrid_H_Hmax)==True]=0

    #--------------------------------------------------------------------------
    #Calculating distance (radius) from hurricane center to each point

    for i in range(0,len(xCenter),1):
    
        #Calculating distance using cartesian formula
        if distCalcMethod=='cart':
    
            distxy=np.sqrt((xgrid-xCenter[i])**2+(ygrid-yCenter[i])**2) #Calculating distance from (x,y) to (x(1),y(1))
    
            #Calculating angle of the line between start and end points
    
            thetarad=np.arctan2(ygrid-yCenter[i],xgrid-xCenter[i]) #Angle in radian
            theta=np.rad2deg(thetarad) #Angle in degree
    
            #Add 360 to all numbers to have them all positive
            #Use mod(360) to take care of the ones larger than 360
            theta=((theta+360)%360) 
    
            #xgrid and ygrid are 2d array
            if np.ndim(xgrid)==2:
                Rgrid[:,:,i]=distxy.copy() #Distance from hurricane center to each point in (m)
                thetagrid[:,:,i]=theta.copy() #Angle from hurricane center to each point in (degree)

            #xgrid and ygrid are 1d array
            else:
                #Using np.newaxis (or None) to convert shape (Mgrid) into shape (Mgrid,1) when xgrid and ygrid are 1d array
                Rgrid[:,:,i]=distxy[:,np.newaxis].copy() #Distance from hurricane center to each point in (m)
                thetagrid[:,:,i]=theta[:,np.newaxis].copy() #Angle from hurricane center to each point in (degree)
    
        #Calculating distance using Vincenty formula
        elif distCalcMethod=='gc':
    
            #Converting to radian
            lat1rad=np.deg2rad(yCenter[i])
            lon1rad=np.deg2rad(xCenter[i])
            lat2rad=np.deg2rad(ygrid)
            lon2rad=np.deg2rad(xgrid)
    
            deltalatrad21=lat2rad-lat1rad
            deltalonrad21=lon2rad-lon1rad
    
            REarth=6371000 #Earth radius in (m), mean earth radius=6371000 m
            deltasigma=np.arctan2(np.sqrt((np.cos(lat2rad)*np.sin(deltalonrad21))**2+(np.cos(lat1rad)*np.sin(lat2rad)-np.sin(lat1rad)*np.cos(lat2rad)*np.cos(deltalonrad21))**2),np.sin(lat1rad)*np.sin(lat2rad)+np.cos(lat1rad)*np.cos(lat2rad)*np.cos(deltalonrad21)) #Central angle
            arclen=REarth*deltasigma #Total distance of the line 
            distxy=arclen.copy()
    
            #Calculating azimuth (bearing) between start and end of the line
    
            azimuthrad=np.arctan2(np.sin(deltalonrad21)*np.cos(lat2rad),np.cos(lat1rad)*np.sin(lat2rad)-np.sin(lat1rad)*np.cos(lat2rad)*np.cos(deltalonrad21)) #Azimuth (bearing) in radian
            azimuth=np.rad2deg(azimuthrad) #Azimuth (bearing) in degree

            #Add 360 to all numbers to have them all positive
            #Use mod(360) to take care of the ones larger than 360
            azimuth=((azimuth+360)%360) 
    
            #xgrid and ygrid are 2d array
            if np.ndim(xgrid)==2:
                Rgrid[:,:,i]=distxy.copy() #Distance from hurricane center to each point in (m)
                thetagrid[:,:,i]=azimuth.copy() #Angle from hurricane center to each point in (degree)

            #xgrid and ygrid are 1d array
            else:
                #Using np.newaxis (or None) to convert shape (Mgrid) into shape (Mgrid,1) when xgrid and ygrid are 1d array
                Rgrid[:,:,i]=distxy[:,np.newaxis].copy() #Distance from hurricane center to each point in (m)
                thetagrid[:,:,i]=azimuth[:,np.newaxis].copy() #Angle from hurricane center to each point in (degree)
    

    #--------------------------------------------------------------------------
    #Converting azimuth (bearing) to trigonometric direction

    #Converting azimuth (bearing) to trigonometric direction
    VtTrigdir=-VtAzmdir+90
    
    #Add 360 to all numbers to have them all positive
    #Use mod(360) to take care of the ones larger than 360
    VtTrigdir=((VtTrigdir+360)%360) 
    
    #Converting azimuth (bearing) to trigonometric direction
    if distCalcMethod=='gc':
    
        #Converting azimuth (bearing) to trigonometric direction
        thetagrid=-thetagrid+90
    
        #Add 360 to all numbers to have them all positive
        #Use mod(360) to take care of the ones larger than 360
        thetagrid=((thetagrid+360)%360) 
    

    #--------------------------------------------------------------------------
    #Calculating new grid location

    #Calculating new grid location as a function of x/RVmax based on Young (1988)
    for i in range(0,len(xCenter),1):

        #Axis rotation respect to North (90 degree)
        Beta=VtTrigdir[i]-90

        #Adjusting direction of each grid point based on moving direction of hurricane
        #SPM contours are for hurricane moving toward North
        thetagridtoVt=thetagrid[:,:,i]-Beta

        #Converting polar to Cartesian system with axis rotation equal to Beta
        xgridprime,ygridprime=pol2cart(np.deg2rad(thetagridtoVt),Rgrid[:,:,i])

        #Calculating x/RVmax and y/RVmax for each grid location
        xgrid_RVmax[:,:,i]=xgridprime/RVmax[i]
        ygrid_RVmax[:,:,i]=ygridprime/RVmax[i]

    #--------------------------------------------------------------------------
    #Calculating hurricane wave height contour

    #Calculating new grid location as a function of x/RVmax based on Young (1988)
    for i in range(0,len(xCenter),1):
        
        #Hurricane is in northern hemisphere
        if np.mean(yCenter)>=0:
            #In northern hemisphere, velocity vector rotate counter clockwise with right angle respect to radius

            #Calculating H/Hmax ratio for each grid point
            #H_Hmax_grid=sp.interpolate.griddata((xgrid_x_RVmax,ygrid_y_RVmax),zgrid_H_Hmax[:,:,i],(xgrid_RVmax[:,:,i],ygrid_RVmax[:,:,i]))
            H_Hmax_grid=sp.interpolate.griddata((np.reshape(xgrid_x_RVmax,np.size(xgrid_x_RVmax)),np.reshape(ygrid_y_RVmax,np.size(ygrid_y_RVmax))),np.reshape(zgrid_H_Hmax[:,:,i],np.size(zgrid_H_Hmax[:,:,i])),(xgrid_RVmax[:,:,i],ygrid_RVmax[:,:,i]))

        #Hurricane is in southern hemisphere
        elif np.mean(yCenter)<0:
            #In southern hemisphere, velocity vector rotate clockwise with right angle respect to radius

            #Calculating H/Hmax ratio for each grid point
            #H_Hmax_grid=sp.interpolate.griddata((-1*xgrid_x_RVmax,ygrid_y_RVmax),zgrid_H_Hmax[:,:,i],(xgrid_RVmax[:,:,i],ygrid_RVmax[:,:,i]))
            H_Hmax_grid=sp.interpolate.griddata((np.reshape(-1*xgrid_x_RVmax,np.size(xgrid_x_RVmax)),np.reshape(ygrid_y_RVmax,np.size(ygrid_y_RVmax))),np.reshape(zgrid_H_Hmax[:,:,i],np.size(zgrid_H_Hmax[:,:,i])),(xgrid_RVmax[:,:,i],ygrid_RVmax[:,:,i]))


        H_Hmax_grid[np.isnan(H_Hmax_grid)==True]=0
        
        #Calcuating wave height at each grid point
        Hgrid[:,:,i]=Hmax[i]*H_Hmax_grid

    Hgrid[Hgrid<0]=0
    Hgrid[Hgrid>1]=1
    Hgrid[np.isnan(Hgrid)==True]=0

    #--------------------------------------------------------------------------
    #Displaying results

    if dispout=='imagesc': #2 dimensional plot using imagesc or imshow
        
        plt.imshow(Hgrid[:,:,-1],extent=[np.nanmin(xgrid),np.nanmax(xgrid),np.nanmin(ygrid),np.nanmax(ygrid)],interpolation=None,cmap=plt.get_cmap(),aspect='auto',origin='lower')
        plt.plot([xCenter[-1],xCenter[-1]],[np.min(ygrid),np.max(ygrid)],color=[1,1,1])
        plt.plot([np.min(xgrid),np.max(xgrid)],[yCenter[-1],yCenter[-1]],color=[1,1,1])
    
    elif dispout=='pcolor': #2 dimensional plot using pcolor
    
        plt.pcolormesh(xgrid,ygrid,Hgrid[:,:,-1],cmap=plt.get_cmap())    
        plt.plot([xCenter[-1],xCenter[-1]],[np.min(ygrid),np.max(ygrid)],color=[1,1,1])
        plt.plot([np.min(xgrid),np.max(xgrid)],[yCenter[-1],yCenter[-1]],color=[1,1,1])
    
    elif dispout=='contour': #2 dimensional contour plot
    
        plt.contourf(xgrid,ygrid,Hgrid[:,:,-1],cmap=plt.get_cmap())
        plt.plot([xCenter[-1],xCenter[-1]],[np.min(ygrid),np.max(ygrid)],color=[1,1,1])
        plt.plot([np.min(xgrid),np.max(xgrid)],[yCenter[-1],yCenter[-1]],color=[1,1,1])
    
    
    #Setting plot properties
    if ((dispout=='imagesc') or (dispout=='pcolor') or (dispout=='contour')):
    
        #Setting axis limits
        #plt.xlim([np.min(xgrid),np.max(xgrid)])
        #plt.ylim([np.min(ygrid),np.max(ygrid)])
    
        #Plotting colorbar
        plt.colorbar()
    
        #Setting label
        plt.xlabel('x')
        plt.ylabel('y')
        plt.title('Wave Height')
        

    #--------------------------------------------------------------------------
    #Converting output array with size of (Mgrid,Ngrid,1) to array with size of (Mgrid,Ngrid)

    if len(xCenter)==1:
        Hgrid=np.reshape(Hgrid,(Mgrid,Ngrid)) #Reshaping array


    #--------------------------------------------------------------------------
    #Outputs
    return Hgrid

    #--------------------------------------------------------------------------
